#!/usr/bin/perl -w
# remediator.pl version 1.56 3/29/08
# Copyright 2007, Jeffrey J. Headd and Robert Immormino

# revision 1.02 - JJH 070726 - fixed HN2/H2 N-terminus ambiguity
# revision 1.52 - JJH 070727 - added KIN file support
# revision 1.53 - JJH 070731 - updated hash tables to correct conversions
# revision 1.54 - JJH 070731 - fixed KIN file support for new probe dot format
# revision 1.55 - RI  070803 - added support for DU bases
# revision      - JJH 070823 - added support for CNS Xplor and Coot RNA names
# revision	- JJH 070908 - added REMARK   4 comment addition
# revision      - JJH 070913 - added support for left-justified RNA/DNA old names
# revision      - JJH 070913 - added support for all left-justified residue names
# revision 1.56 - JJH 080328 - fixed REMARK   4 comment addition to work w/ PHENIX generated files

if(!$ARGV[0]){
	&help;
	exit(0);
}

#-----------------------------------------------------------------------
sub help{
        print "\n*******************************************************************
remediator.pl: version 1.56 3/28/08\nCopyright 2007, Jeffrey J. Headd and Robert Immormino 
For a log of changes, view remediator.pl in your favorite text editor

LAST CIF DICTIONARY UPDATE:  080507
 

USAGE: remediator.pl [-options] input_file

options:
  -h 		outputs this help message
  -pdb		takes a .pdb formatted file as input (default)
  -kin		takes a .kin kinemage formatted file as input (under development) 
  -oldout 	output file will use the PDBv2.3 naming conventions
  -remediated 	output file will use the remediated naming conventions (default)

remediator.pl is generally inteded to convert from PDBv2.3 to PDBv3.0. 
This changes files from the pre-wwPDB format into the wwPDB remediated format.
Output is directed to standard out.

EXAMPLES:   remediator.pl 404D.pdb > 404Dr.pdb 
            remediator.pl -kin 1BAB.kin > 1BABr.kin \n\n";
        exit(0);
}
#-----------------------------------------------------------------------

$oldout = 0;
$remediated = 0;
$dopdb = 0;
$dokin = 0;
$input_pdb_flag = 0;
$input_kin_flag = 0;
$input=NULL;

for ($i=0; $i < scalar(@ARGV); $i++) {
	if(substr($ARGV[$i],0,1) eq "-"){
		if ($ARGV[$i] eq "-h"){
			&help;
			exit(0);
		}
		elsif ($ARGV[$i] eq "-oldout") { $oldout = 1; } 
		elsif ($ARGV[$i] eq "-remediated") { $remediated = 1; }
		elsif ($ARGV[$i] eq "-pdb") { $dopdb = 1; }
		elsif ($ARGV[$i] eq "-kin") { $dokin = 1; }
		else {	print STDERR "Unrecognized flag ".$ARGV[$i]." IGNORED\n"; }
	}
	elsif ($ARGV[$i] =~ m/.pdb/){
		if($input_pdb_flag==0 && $input_kin_flag==0){
			$input=$ARGV[$i];
			$input_pdb_flag=1;
		}
		else{
			print STDERR "Too many options!!  Exiting...\n";
			&help;
			exit(0);
		}
	}

	elsif ($ARGV[$i] =~ m/.kin/){ 
		if($input_pdb_flag==0 && $input_kin_flag==0){
			$input=$ARGV[$i]; 
			$input_kin_flag=1;
		}
		else{
			print STDERR "Too many options!!  Exiting...\n";
			&help;
			exit(0);
		}
	}
	else{ 
		if($input_pdb_flag==1 || $input_kin_flag==1){
			print STDERR "Too many options!!  Exiting...\n";
			&help;
			exit(0);
		}
		elsif($dokin == 1){
			print STDERR "Unrecognized input file extension!!  Assuming .kin file...\n";
			$input=$ARGV[0];
			$input_kin_flag=1;
		} 
		else{
			print STDERR "Unrecognized input file extension!!  Assuming .pdb file...\n";
			$input=$ARGV[0];
			$input_pdb_flag=1;
		}
	}
}

if($input eq NULL){
	print STDERR "User must specify input file!!  Exiting...\n";
	&help;
	exit(0);
}

if($dopdb == 0 && $dokin == 0){ $dopdb=1; }

if($dopdb == 1 && $input_pdb_flag == 0){
	print STDERR "Wrong file type for PDB mode!!  Exiting...\n";
	&help;
	exit(0);
}
elsif($dokin == 1 && $input_kin_flag == 0){
	print STDERR "Wrong file type for KIN mode!!  Exiting...\n";
	&help;
	exit(0);
}

if (!defined $input){ 
	print "\n Sorry, input file was not found!!  Exiting...\n"; 
	&help;
	exit(0);
}

if($remediated == 0 && $oldout == 0){ $remediated = 1; }

if($remediated == 1){ 
	&build_old2new; 
	$remark4="REMARK   4 REMEDIATOR VALIDATED PDB VERSION 3.0 COMPLIANT";
}
elsif($oldout == 1){ 
	&build_new2old; 
	$remark4="REMARK   4 REMEDIATOR VALIDATED PDB VERSION 2.3 COMPLIANT";
}

open IN, "$input" || die "Could not open input file!!  Exiting...\n";
if($dopdb == 1){ &pdb_mod; }
elsif($dokin == 1){ &kin_mod; }
close IN;

#--pdb_mod sub-routine-------------------------------------------------------------------------------------
sub pdb_mod{

$previous = "NULL";
$current = "NULL";
$print_line = "";
$remark_flag="NULL";
$remark_block="NULL";

while ($line=<IN>){

	chomp($line);
	$line =~ s/\s+$//;
	
	if($remark_flag eq "NULL"){
		if(substr($line,0,6) eq "REMARK"){
			#print "found remark\n";
			if($remark_block eq "NULL"){
				$remark_block = "DONE";
			}
			if($line=~m/$remark4/){
				$remark_flag="DONE";
			}
			elsif(length($line) < 10){
				$print_line .= $line."\n";
				next;
			}
			elsif($line=~m/REMARK...\D/){
				$print_line .= $line."\n";
				next;
			}
			elsif($line=~m/REMARK   4 REMEDIATOR/){
				next;
			}
			elsif(substr($line,6,4) =~m/[0-9]/ && substr($line,6,4)>4){
				$print_line .= "$remark4\n";
				$remark_flag="DONE";
			}
			else{
				$print_line .= $line."\n";
				next;
			}
			#print substr($line,6,4)."\n";
		}
	}

	if (substr($line, 0,6) eq "ATOM  " || substr($line, 0,6) eq "HETATM" ||
            substr($line, 0,6) eq "TER   " || substr($line, 0,6) eq "ANISOU" ||
            substr($line, 0,6) eq "SIGATM" || substr($line, 0,6) eq "SIGUIJ" ||
	    substr($line, 0,6) eq "LINK  "){
		if($remark_flag eq "NULL"){
			$print_line .= "$remark4\n";
			$remark_flag="DONE";
		}	
		#--pre-screen for CNS Xplor RNA base names and Coot RNA base names--------
		if($line=~m/\b.{17}(GUA|ADE|CYT|THY|URI)/){
			$line=~s/\b(.{17})(.)../$1  $2/;
		}
		elsif($line=~m/\b.{17}(OIP| Ar| Gr| Cr| Ur)/){
			$line=~s/\b(.{17}).(.)./$1  $2/;
		}
		#-------------------------------------------------------------------------

		#REMOVED IN FAVOR OF GENERIC SUBSITUTION BELOW
                #--pre-screen for left-justified RNA/DNA base names-----------------------
		#if($line=~m/\b.{17}(G  |A  |C  |T  |U  |I  )/){
		#	$line=~s/\b(.{17})(.)../$1  $2/;
		#}
                #-------------------------------------------------------------------------
		#-pre-screen for all left-justified residue names-------------------------
		if($line=~m/\A.{17}([a-zA-Z])  /){
			$line=~s/\A(.{17})(.)\s\s/$1  $2/;
		}
		elsif($line=~m/\A.{17}([a-zA-Z][a-zA-Z]) /){
			$line=~s/\A(.{17})(..)\s/$1 $2/;
		}
		#-------------------------------------------------------------------------
		$entry=substr($line,12,8);
		$previous=$current;
		$current=substr($line,18,8);
		$clean_entry = substr($entry,0,4)." ".substr($entry,5,3);
		if($resn_exch{$clean_entry}){
			substr($line,12,4) = substr($resn_exch{$clean_entry},0,4);
			substr($line,17,3) = substr($resn_exch{$clean_entry},5,3);
		} 
	}
	
	elsif ($remark_flag ne "DONE" && $remark_block eq "DONE"){
		$print_line .= "$remark4\n";
		$remark_flag = "DONE";
	} 
	if($previous eq "NULL"){ $previous = $current; }
	
	if($current eq $previous){
		$print_line .= $line."\n";
	}
	if($current ne $previous){
		
		if($print_line =~ m/(.\S..)  .[A,C,T,G,I,U] /){
			if($print_line !~ m/O2[\',\*]   ./){
				$DNA_base = substr($previous,1,1);
				if($remediated == 1){
					$print_line =~ s/(.\S..)   $DNA_base /$1  D$DNA_base /g;
					$print_line =~ s/(TER.{15}) $DNA_base /$1D$DNA_base /g;
				}
				elsif($oldout == 1){
					$print_line =~ s/(.\S..)  D$DNA_base /$1   $DNA_base /g;
					$print_line =~ s/(TER.{15})D$DNA_base /$1 $DNA_base /g;
				}
			}
			
		}
		if($oldout == 1){
			if($print_line =~ m/ HN2 (ALA|ARG|ASN|ASP|ASX|CSE|CYS|GLN|GLU|GLX|GLY|HIS|ILE|LEU|LYS|MET|MSE|PHE|PRO|SER|THR|TRP|UNK|TYR|VAL)/g){
				$res=$1;
				if($print_line =~ m/1H   $res/ || $print_line =~ m/3H   $res/){
					$print_line =~ s/ HN2 (ALA|ARG|ASN|ASP|ASX|CSE|CYS|GLN|GLU|GLX|GLY|HIS|ILE|LEU|LYS|MET|MSE|PHE|PRO|SER|THR|TRP|UNK|TYR|VAL)/2H   $1/g;
				}
			}
		}
		print $print_line;
		$print_line=$line."\n";
	} 
}

if($print_line =~ m/(.\S..)   [A,C,T,G,I,U] /){
	if($print_line !~ m/O2[\',\*]   ./){
		$DNA_base = substr($previous,1,1);
		if($remediated == 1){
			$print_line =~ s/(.\S..)   $DNA_base /$1  D$DNA_base /g;
			$print_line =~ s/(TER.{15}) $DNA_base /$1D$DNA_base /g;
		}
		elsif($oldout == 1){
			$print_line =~ s/(.\S..)  D$DNA_base /$1   $DNA_base /g;
			$print_line =~ s/(TER.{15})D$DNA_base /$1 $DNA_base /g;
		}
	}
	if($oldout == 1){
		if($print_line =~ m/ HN2 (ALA|ARG|ASN|ASP|ASX|CSE|CYS|GLN|GLU|GLX|GLY|HIS|ILE|LEU|LYS|MET|MSE|PHE|PRO|SER|THR|TRP|UNK|TYR|VAL)/g){
			$res=$1;
			if($print_line =~ m/1H   $res/ || $print_line =~ m/3H   $res/){
				$print_line =~ s/ HN2 (ALA|ARG|ASN|ASP|ASX|CSE|CYS|GLN|GLU|GLX|GLY|HIS|ILE|LEU|LYS|MET|MSE|PHE|PRO|SER|THR|TRP|UNK|TYR|VAL)/2H   $1/g;
			}
		}
	}
}
print $print_line;

}
#----------------------------------------------------------------------------------------------------------


#--kin_mod sub-routine-------------------------------------------------------------------------------------
sub kin_mod{

%RNA_ID=();

#pre-screen for RNA vs. DNA for -remediated
if($remediated == 1){
	while($line=<IN>){
		chomp($line);
		my(@records) = $line =~ m/\{.{8,}?\}/g;
	
		for($i=0;$i<scalar(@records);$i++){
			
			$atom=substr($records[$i],1,4);
			$chain=substr($records[$i],10,1);
			$atom =~ tr/[a-z]/[A-Z]/;
			$chain=~ tr/[a-z]/[A-Z]/;
			
			if($records[$i] =~ m/O2[\',\*]   ./i){
				
				if($records[$i] =~ m/\{.\S.....\S\s.\s\d+\s\sB\d/){
					my($res,$num) = $records[$i] =~ m/\{.\S...(..\S)\s.\s(\S+)\s/;
					$res .= $num;
					$res =~ tr/[a-z]/[A-Z]/;
					$RNA_ID{$chain." ".$res}="RNA";
				}
			}
		}
	}
	close IN;
	open IN, "$input" || die "Could not open input file!!  Exiting...\n";

}

while($line=<IN>){

	chomp($line);
	my(@records) = $line =~ m/\{.{8,}?\}/g;
	
	
	for($i=0;$i<scalar(@records);$i++){
		$atom=substr($records[$i],1,4); 
		$atom =~ tr/[a-z]/[A-Z]/;
		if($records[$i] =~ m/\{.\S...(..\S)(\S+)\s(\S)\}/){
			$res = $1;
			$num = $2;
			$chain = $3;
			$res =~ tr/[a-z]/[A-Z]/;
			$chain =~ tr/[a-z]/[A-Z]/;
			if($resn_exch{$atom." ".$res}){
				$new_atom=substr($resn_exch{$atom." ".$res},0,4);
				$atom =~ s/(\*)/\\$1/g;
				$line =~ s/$atom(.)$res/$new_atom$1$res/;
				$atom=$new_atom;
			}
			if($remediated == 1){
				if($res =~ m/(  G|  C|  T|  A|  I|  U)/i){
					if(!$RNA_ID{$chain." ".$res.$num}){
						$base = substr($res,2,1);
						$line =~ s/$atom(.)$res/$atom$1 D$base/;
					}
				}
			}
			elsif($oldout == 1){
				if($res =~ m/( DG| DC| DT| DA| DI| DU)/i){
					$base = substr($res,2,1);
					$atom =~ s/(\*)/\\$1/g;
					$line =~ s/($atom)(.)$res/$1$2  $base/;
				}
			}
			
		}
		elsif($records[$i] =~ m/\{.\S...(..\S)\s(.)\s(\d+)\s\sB\d/){
			$res=$1;
			$chain=$2;
			$num=$3;
			$res =~ tr/[a-z]/[A-Z]/;
			$chain =~ tr/[a-z]/[A-Z]/;
			if($resn_exch{$atom." ".$res}){
				$new_atom=substr($resn_exch{$atom." ".$res},0,4);
				$atom =~ tr/[A-Z]/[a-z]/;
				$res =~ tr/[A-Z]/[a-z]/;
				$new_atom =~ tr/[A-Z]/[a-z]/;
				$atom =~ s/(\*)/\\$1/g;
				if(!($line =~ s/$atom(.)$res/$new_atom$1$res/)){
					print STDERR "substitution failed!!!\n";
				}
				$atom=$new_atom;
			}
			if($remediated == 1){
				if($res =~ m/(  G|  C|  T|  A|  I|  U)/i){
					$res .= $num;
					$res =~ tr/[a-z]/[A-Z]/;
					if(!$RNA_ID{$chain." ".$res}){
						$res =~ tr/[A-Z]/[a-z]/;
						$atom =~ tr/[A-Z]/[a-z]/;
						$base = substr($res,2,1);
						$res = substr($res,0,3);
						$line =~ s/$atom(.)$res/$atom$1 d$base/;
					}
				}
			}
			elsif($oldout == 1){
				if($res =~ m/( DG| DC| DT| DA| DI| DU)/i){
					$res =~ tr/[A-Z]/[a-z]/;
					$atom =~ tr/[A-Z]/[a-z]/;
					$atom =~ s/(\*)/\\$1/g;
					$base = substr($res,2,1);
					$line =~ s/($atom)(.)$res/$1$2  $base/;
				}
			}

		}
	}
	
	print $line."\n";

}


}
#----------------------------------------------------------------------------------------------------------


sub build_old2new{

%resn_exch=();

$resn_exch{" C5A  DT"}=" C7   DT"; $resn_exch{"2H2*   A"}="H2''   A"; $resn_exch{"2H2*   C"}="H2''   C";
$resn_exch{"2H2*   T"}="H2''   T"; $resn_exch{"2H2*   G"}="H2''   G"; $resn_exch{"2H2*   I"}="H2''   I";
$resn_exch{"1H2*   A"}=" H2'   A"; $resn_exch{"1H2*   C"}=" H2'   C"; $resn_exch{"1H2*   U"}=" H2'   U";
$resn_exch{"1H2*   G"}=" H2'   G"; $resn_exch{"1H2*   I"}=" H2'   I"; $resn_exch{"5HO*   A"}="HO5'   A";
$resn_exch{"5HO*   C"}="HO5'   C"; $resn_exch{"5HO*   U"}="HO5'   U"; $resn_exch{"5HO*   G"}="HO5'   G";
$resn_exch{"5HO*   I"}="HO5'   I"; $resn_exch{"5HO*  DT"}="HO5'  DT"; $resn_exch{"5HO*  DA"}="HO5'  DA";
$resn_exch{"5HO*  DC"}="HO5'  DC"; $resn_exch{"5HO*  DG"}="HO5'  DG"; $resn_exch{"5HO*  DI"}="HO5'  DI";
$resn_exch{"5HO*  DU"}="HO5'  DU"; $resn_exch{"3HO*   A"}="HO3'   A"; $resn_exch{"3HO*   C"}="HO3'   C";
$resn_exch{"3HO*   U"}="HO3'   U"; $resn_exch{"3HO*   G"}="HO3'   G"; $resn_exch{"3HO*   I"}="HO3'   I";
$resn_exch{"3HO*  DT"}="HO3'  DT"; $resn_exch{"3HO*  DA"}="HO3'  DA"; $resn_exch{"3HO*  DC"}="HO3'  DC";
$resn_exch{"3HO*  DG"}="HO3'  DG"; $resn_exch{"3HO*  DI"}="HO3'  DI"; $resn_exch{"3HO*  DU"}="HO3'  DU";
$resn_exch{"1H   GLY"}=" H1  GLY"; $resn_exch{"2H   GLY"}=" H2  GLY"; $resn_exch{"3H   GLY"}=" H3  GLY";
$resn_exch{"1H   ALA"}=" H1  ALA"; $resn_exch{"2H   ALA"}=" H2  ALA"; $resn_exch{"3H   ALA"}=" H3  ALA";
$resn_exch{"1H   VAL"}=" H1  VAL"; $resn_exch{"2H   VAL"}=" H2  VAL"; $resn_exch{"3H   VAL"}=" H3  VAL";
$resn_exch{"1H   PHE"}=" H1  PHE"; $resn_exch{"2H   PHE"}=" H2  PHE"; $resn_exch{"3H   PHE"}=" H3  PHE";
$resn_exch{"1H   PRO"}=" H1  PRO"; $resn_exch{"2H   PRO"}=" H2  PRO"; $resn_exch{"3H   PRO"}=" H3  PRO";
$resn_exch{"1H   MET"}=" H1  MET"; $resn_exch{"2H   MET"}=" H2  MET"; $resn_exch{"3H   MET"}=" H3  MET";
$resn_exch{"1H   ILE"}=" H1  ILE"; $resn_exch{"2H   ILE"}=" H2  ILE"; $resn_exch{"3H   ILE"}=" H3  ILE";
$resn_exch{"1H   LEU"}=" H1  LEU"; $resn_exch{"2H   LEU"}=" H2  LEU"; $resn_exch{"3H   LEU"}=" H3  LEU";
$resn_exch{"1H   ASP"}=" H1  ASP"; $resn_exch{"2H   ASP"}=" H2  ASP"; $resn_exch{"3H   ASP"}=" H3  ASP";
$resn_exch{"1H   GLU"}=" H1  GLU"; $resn_exch{"2H   GLU"}=" H2  GLU"; $resn_exch{"3H   GLU"}=" H3  GLU";
$resn_exch{"1H   LYS"}=" H1  LYS"; $resn_exch{"2H   LYS"}=" H2  LYS"; $resn_exch{"3H   LYS"}=" H3  LYS";
$resn_exch{"1H   ARG"}=" H1  ARG"; $resn_exch{"2H   ARG"}=" H2  ARG"; $resn_exch{"3H   ARG"}=" H3  ARG";
$resn_exch{"1H   SER"}=" H1  SER"; $resn_exch{"2H   SER"}=" H2  SER"; $resn_exch{"3H   SER"}=" H3  SER";
$resn_exch{"1H   THR"}=" H1  THR"; $resn_exch{"2H   THR"}=" H2  THR"; $resn_exch{"3H   THR"}=" H3  THR";
$resn_exch{"1H   HIS"}=" H1  HIS"; $resn_exch{"2H   HIS"}=" H2  HIS"; $resn_exch{"3H   HIS"}=" H3  HIS";
$resn_exch{"1H   CYS"}=" H1  CYS"; $resn_exch{"2H   CYS"}=" H2  CYS"; $resn_exch{"3H   CYS"}=" H3  CYS";
$resn_exch{"1H   ASN"}=" H1  ASN"; $resn_exch{"2H   ASN"}=" H2  ASN"; $resn_exch{"3H   ASN"}=" H3  ASN";
$resn_exch{"1H   GLN"}=" H1  GLN"; $resn_exch{"2H   GLN"}=" H2  GLN"; $resn_exch{"3H   GLN"}=" H3  GLN";
$resn_exch{"1H   TRP"}=" H1  TRP"; $resn_exch{"2H   TRP"}=" H2  TRP"; $resn_exch{"3H   TRP"}=" H3  TRP";
$resn_exch{"1H   ASX"}=" H1  ASX"; $resn_exch{"2H   ASX"}=" H2  ASX"; $resn_exch{"3H   ASX"}=" H3  ASX";
$resn_exch{"1H   GLX"}=" H1  GLX"; $resn_exch{"2H   GLX"}=" H2  GLX"; $resn_exch{"3H   GLX"}=" H3  GLX";
$resn_exch{"1H   ABU"}=" H1  ABU"; $resn_exch{"2H   ABU"}=" H2  ABU"; $resn_exch{"3H   ABU"}=" H3  ABU";
$resn_exch{"1H   AIB"}=" H1  AIB"; $resn_exch{"2H   AIB"}=" H2  AIB"; $resn_exch{"3H   AIB"}=" H3  AIB";
$resn_exch{"1H   ABU"}=" H1  ABU"; $resn_exch{"2H   ABU"}=" H2  ABU"; $resn_exch{"3H   ABU"}=" H3  ABU";
$resn_exch{"1H   MSE"}=" H1  MSE"; $resn_exch{"2H   MSE"}=" H2  MSE"; $resn_exch{"3H   MSE"}=" H3  MSE";
$resn_exch{"1H   PCA"}=" H1  PCA"; $resn_exch{"2H   PCA"}=" H2  PCA"; $resn_exch{"3H   PCA"}=" H3  PCA";
$resn_exch{"1HN  0AO"}=" HN1 0AO"; $resn_exch{" OD  0AZ"}=" OD1 0AZ"; $resn_exch{" O5* 101"}=" O5' 101";
$resn_exch{" C5* 101"}=" C5' 101"; $resn_exch{" C4* 101"}=" C4' 101"; $resn_exch{" O4* 101"}=" O4' 101";
$resn_exch{" C3* 101"}=" C3' 101"; $resn_exch{" O3* 101"}=" O3' 101"; $resn_exch{" C2* 101"}=" C2' 101";
$resn_exch{" C1* 101"}=" C1' 101"; $resn_exch{" O5* 102"}=" O5' 102"; $resn_exch{" C5* 102"}=" C5' 102";
$resn_exch{" C4* 102"}=" C4' 102"; $resn_exch{" O4* 102"}=" O4' 102"; $resn_exch{" C3* 102"}=" C3' 102";
$resn_exch{" O3* 102"}=" O3' 102"; $resn_exch{" C2* 102"}=" C2' 102"; $resn_exch{" O2* 102"}=" O2' 102";
$resn_exch{" C1* 102"}=" C1' 102"; $resn_exch{" C5* 103"}=" C5' 103"; $resn_exch{" C4* 103"}=" C4' 103";
$resn_exch{" O4* 103"}=" O4' 103"; $resn_exch{" C3* 103"}=" C3' 103"; $resn_exch{" O3* 103"}=" O3' 103";
$resn_exch{" C2* 103"}=" C2' 103"; $resn_exch{" C1* 103"}=" C1' 103"; $resn_exch{" O5* 10C"}=" O5' 10C";
$resn_exch{" C5* 10C"}=" C5' 10C"; $resn_exch{" C4* 10C"}=" C4' 10C"; $resn_exch{" O4* 10C"}=" O4' 10C";
$resn_exch{" C1* 10C"}=" C1' 10C"; $resn_exch{" C3* 10C"}=" C3' 10C"; $resn_exch{" C2* 10C"}=" C2' 10C";
$resn_exch{" O2* 10C"}=" O2' 10C"; $resn_exch{" O3* 10C"}=" O3' 10C"; $resn_exch{" C6* 10C"}=" C6' 10C";
$resn_exch{" O5* 112"}=" O5' 112"; $resn_exch{" C5* 112"}=" C5' 112"; $resn_exch{" C4* 112"}=" C4' 112";
$resn_exch{" O4* 112"}=" O4' 112"; $resn_exch{" C3* 112"}=" C3' 112"; $resn_exch{" O3* 112"}=" O3' 112";
$resn_exch{" C2* 112"}=" C2' 112"; $resn_exch{" O2* 112"}=" O2' 112"; $resn_exch{" C1* 112"}=" C1' 112";
$resn_exch{" O1P 125"}=" OP1 125"; $resn_exch{" O2P 125"}=" OP2 125"; $resn_exch{" O3P 125"}=" OP3 125";
$resn_exch{" O5* 125"}=" O5' 125"; $resn_exch{" C5* 125"}=" C5' 125"; $resn_exch{" C4* 125"}=" C4' 125";
$resn_exch{" O4* 125"}=" O4' 125"; $resn_exch{" C3* 125"}=" C3' 125"; $resn_exch{" O3* 125"}=" O3' 125";
$resn_exch{" C2* 125"}=" C2' 125"; $resn_exch{" O2* 125"}=" O2' 125"; $resn_exch{" C1* 125"}=" C1' 125";
$resn_exch{" CA* 125"}=" CA' 125"; $resn_exch{" CB* 125"}=" CB' 125"; $resn_exch{" FC* 125"}=" FC' 125";
$resn_exch{" O1P 126"}=" OP1 126"; $resn_exch{" O2P 126"}=" OP2 126"; $resn_exch{" O3P 126"}=" OP3 126";
$resn_exch{" O5* 126"}=" O5' 126"; $resn_exch{" C5* 126"}=" C5' 126"; $resn_exch{" C4* 126"}=" C4' 126";
$resn_exch{" O4* 126"}=" O4' 126"; $resn_exch{" C3* 126"}=" C3' 126"; $resn_exch{" O3* 126"}=" O3' 126";
$resn_exch{" C2* 126"}=" C2' 126"; $resn_exch{" O2* 126"}=" O2' 126"; $resn_exch{" C1* 126"}=" C1' 126";
$resn_exch{" CA* 126"}=" CA' 126"; $resn_exch{" CB* 126"}=" CB' 126"; $resn_exch{" OC* 126"}=" OC' 126";
$resn_exch{" CD* 126"}=" CD' 126"; $resn_exch{" CE* 126"}=" CE' 126"; $resn_exch{" OF* 126"}=" OF' 126";
$resn_exch{" CG* 126"}=" CG' 126"; $resn_exch{" CH* 126"}=" CH' 126"; $resn_exch{" OI* 126"}=" OI' 126";
$resn_exch{" CK* 126"}=" CK' 126"; $resn_exch{" O1P 127"}=" OP1 127"; $resn_exch{" O2P 127"}=" OP2 127";
$resn_exch{" O3P 127"}=" OP3 127"; $resn_exch{" O5* 127"}=" O5' 127"; $resn_exch{" C5* 127"}=" C5' 127";
$resn_exch{" C4* 127"}=" C4' 127"; $resn_exch{" O4* 127"}=" O4' 127"; $resn_exch{" C3* 127"}=" C3' 127";
$resn_exch{" O3* 127"}=" O3' 127"; $resn_exch{" C2* 127"}=" C2' 127"; $resn_exch{" O2* 127"}=" O2' 127";
$resn_exch{" C1* 127"}=" C1' 127"; $resn_exch{" CA* 127"}=" CA' 127"; $resn_exch{" CB* 127"}=" CB' 127";
$resn_exch{" OC* 127"}=" OC' 127"; $resn_exch{" ND* 127"}=" ND' 127"; $resn_exch{" O5* 128"}=" O5' 128";
$resn_exch{" C5* 128"}=" C5' 128"; $resn_exch{" C4* 128"}=" C4' 128"; $resn_exch{" O4* 128"}=" O4' 128";
$resn_exch{" C3* 128"}=" C3' 128"; $resn_exch{" O3* 128"}=" O3' 128"; $resn_exch{" C2* 128"}=" C2' 128";
$resn_exch{" O2* 128"}=" O2' 128"; $resn_exch{" C1* 128"}=" C1' 128"; $resn_exch{" O3P 12A"}=" OP3 12A";
$resn_exch{" O1P 12A"}=" OP1 12A"; $resn_exch{" O2P 12A"}=" OP2 12A"; $resn_exch{" O5* 12A"}=" O5' 12A";
$resn_exch{" C5* 12A"}=" C5' 12A"; $resn_exch{" C4* 12A"}=" C4' 12A"; $resn_exch{" O4* 12A"}=" O4' 12A";
$resn_exch{" C3* 12A"}=" C3' 12A"; $resn_exch{" O3* 12A"}=" O3' 12A"; $resn_exch{" C2* 12A"}=" C2' 12A";
$resn_exch{" O2* 12A"}=" O2' 12A"; $resn_exch{" C1* 12A"}=" C1' 12A"; $resn_exch{" C1* 130"}=" C1B 130";
$resn_exch{" C2* 130"}=" C2B 130"; $resn_exch{" C3* 130"}=" C3B 130"; $resn_exch{" C4* 130"}=" C4B 130";
$resn_exch{" C5* 130"}=" C5B 130"; $resn_exch{" C6* 130"}=" C6B 130"; $resn_exch{" C1* 132"}=" C1B 132";
$resn_exch{" C2* 132"}=" C2B 132"; $resn_exch{" C3* 132"}=" C3B 132"; $resn_exch{" C4* 132"}=" C4B 132";
$resn_exch{" C5* 132"}=" C5B 132"; $resn_exch{" C6* 132"}=" C6B 132"; $resn_exch{" C1* 134"}=" C1B 134";
$resn_exch{" C2* 134"}=" C2B 134"; $resn_exch{" C3* 134"}=" C3B 134"; $resn_exch{" C4* 134"}=" C4B 134";
$resn_exch{" C5* 134"}=" C5B 134"; $resn_exch{" C6* 134"}=" C6B 134"; $resn_exch{"O5*A 139"}=" O5D 139";
$resn_exch{"C5*A 139"}=" C5D 139"; $resn_exch{"C4*A 139"}=" C4D 139"; $resn_exch{"O4*A 139"}=" O4D 139";
$resn_exch{"C1*A 139"}=" C1D 139"; $resn_exch{"C2*A 139"}=" C2D 139"; $resn_exch{"O2*A 139"}=" O2D 139";
$resn_exch{"C3*A 139"}=" C3D 139"; $resn_exch{"O3*A 139"}=" O3D 139"; $resn_exch{"O5*U 139"}=" O5B 139";
$resn_exch{"C5*U 139"}=" C5B 139"; $resn_exch{"C4*U 139"}=" C4B 139"; $resn_exch{"O4*U 139"}=" O4B 139";
$resn_exch{"C1*U 139"}=" C1B 139"; $resn_exch{"C2*U 139"}=" C2B 139"; $resn_exch{"C3*U 139"}=" C3B 139";
$resn_exch{"O3*U 139"}=" O3B 139"; $resn_exch{" CD* 176"}=" CDB 176"; $resn_exch{" CG* 176"}=" CGB 176";
$resn_exch{" CD* 177"}=" CDB 177"; $resn_exch{" CG* 177"}=" CGB 177"; $resn_exch{" O5* 191"}=" O5' 191";
$resn_exch{" C5* 191"}=" C5' 191"; $resn_exch{" C4* 191"}=" C4' 191"; $resn_exch{" O4* 191"}=" O4' 191";
$resn_exch{" C3* 191"}=" C3' 191"; $resn_exch{" O3* 191"}=" O3' 191"; $resn_exch{" C2* 191"}=" C2' 191";
$resn_exch{" O2* 191"}=" O2' 191"; $resn_exch{" C1* 191"}=" C1' 191"; $resn_exch{" C9* 1AA"}=" C9' 1AA";
$resn_exch{" C8* 1AA"}=" C8' 1AA"; $resn_exch{" C7* 1AA"}=" C7' 1AA"; $resn_exch{" O2* 1AA"}=" O2' 1AA";
$resn_exch{" O4* 1AA"}=" O4' 1AA"; $resn_exch{" O3* 1AA"}=" O3' 1AA"; $resn_exch{" O1* 1AA"}=" O1' 1AA";
$resn_exch{" C1* 1AA"}=" C1' 1AA"; $resn_exch{" C2* 1AA"}=" C2' 1AA"; $resn_exch{" C3* 1AA"}=" C3' 1AA";
$resn_exch{" C4* 1AA"}=" C4' 1AA"; $resn_exch{" C5* 1AA"}=" C5' 1AA"; $resn_exch{" C6* 1AA"}=" C6' 1AA";
$resn_exch{" O1P 1AP"}=" OP1 1AP"; $resn_exch{" O2P 1AP"}=" OP2 1AP"; $resn_exch{" O3P 1AP"}=" OP3 1AP";
$resn_exch{" O5* 1DA"}=" O5' 1DA"; $resn_exch{" C5* 1DA"}=" C5' 1DA"; $resn_exch{" C4* 1DA"}=" C4' 1DA";
$resn_exch{" O4* 1DA"}=" O4' 1DA"; $resn_exch{" C3* 1DA"}=" C3' 1DA"; $resn_exch{" O3* 1DA"}=" O3' 1DA";
$resn_exch{" C2* 1DA"}=" C2' 1DA"; $resn_exch{" O2* 1DA"}=" O2' 1DA"; $resn_exch{" C1* 1DA"}=" C1' 1DA";
$resn_exch{" C1* 1DG"}=" C1' 1DG"; $resn_exch{" O4* 1DG"}=" O4' 1DG"; $resn_exch{" C2* 1DG"}=" C2' 1DG";
$resn_exch{" O2* 1DG"}=" O2' 1DG"; $resn_exch{" C3* 1DG"}=" C3' 1DG"; $resn_exch{" O3* 1DG"}=" O3' 1DG";
$resn_exch{" C4* 1DG"}=" C4' 1DG"; $resn_exch{" C5* 1DG"}=" C5' 1DG"; $resn_exch{" O5* 1DG"}=" O5' 1DG";
$resn_exch{"C10' 1IQ"}=" CA' 1IQ"; $resn_exch{"C11' 1IQ"}=" CB' 1IQ"; $resn_exch{"O12' 1IQ"}=" OC' 1IQ";
$resn_exch{"O13' 1IQ"}=" OD' 1IQ"; $resn_exch{" O1P 1MA"}=" OP1 1MA"; $resn_exch{" O2P 1MA"}=" OP2 1MA";
$resn_exch{" O3P 1MA"}=" OP3 1MA"; $resn_exch{" O5* 1MA"}=" O5' 1MA"; $resn_exch{" C5* 1MA"}=" C5' 1MA";
$resn_exch{" C4* 1MA"}=" C4' 1MA"; $resn_exch{" O4* 1MA"}=" O4' 1MA"; $resn_exch{" C3* 1MA"}=" C3' 1MA";
$resn_exch{" O3* 1MA"}=" O3' 1MA"; $resn_exch{" C2* 1MA"}=" C2' 1MA"; $resn_exch{" O2* 1MA"}=" O2' 1MA";
$resn_exch{" C1* 1MA"}=" C1' 1MA"; $resn_exch{" O1P 1MG"}=" OP1 1MG"; $resn_exch{" O2P 1MG"}=" OP2 1MG";
$resn_exch{" O3P 1MG"}=" OP3 1MG"; $resn_exch{" O5* 1MG"}=" O5' 1MG"; $resn_exch{" C5* 1MG"}=" C5' 1MG";
$resn_exch{" C4* 1MG"}=" C4' 1MG"; $resn_exch{" O4* 1MG"}=" O4' 1MG"; $resn_exch{" C3* 1MG"}=" C3' 1MG";
$resn_exch{" O3* 1MG"}=" O3' 1MG"; $resn_exch{" C2* 1MG"}=" C2' 1MG"; $resn_exch{" O2* 1MG"}=" O2' 1MG";
$resn_exch{" C1* 1MG"}=" C1' 1MG"; $resn_exch{"1O0' 1PR"}=" O'1 1PR"; $resn_exch{" C1\" 1PR"}=" C1B 1PR";
$resn_exch{" C2\" 1PR"}=" C2B 1PR"; $resn_exch{" C3\" 1PR"}=" C3B 1PR"; $resn_exch{" C4\" 1PR"}=" C4B 1PR";
$resn_exch{" C5\" 1PR"}=" C5B 1PR"; $resn_exch{" O2\" 1PR"}=" O2B 1PR"; $resn_exch{" O3\" 1PR"}=" O3B 1PR";
$resn_exch{" O4\" 1PR"}=" O4B 1PR"; $resn_exch{" O6\" 1PR"}=" O6B 1PR"; $resn_exch{" P7\" 1PR"}=" P7B 1PR";
$resn_exch{"1O2\" 1PR"}=" O'L 1PR"; $resn_exch{" O8\" 1PR"}=" O8B 1PR"; $resn_exch{" O9\" 1PR"}=" O9B 1PR";
$resn_exch{" O5* 1RB"}=" O5' 1RB"; $resn_exch{" C5* 1RB"}=" C5' 1RB"; $resn_exch{" C4* 1RB"}=" C4' 1RB";
$resn_exch{" O4* 1RB"}=" O4' 1RB"; $resn_exch{" C3* 1RB"}=" C3' 1RB"; $resn_exch{" O3* 1RB"}=" O3' 1RB";
$resn_exch{" C2* 1RB"}=" C2' 1RB"; $resn_exch{" O2* 1RB"}=" O2' 1RB"; $resn_exch{" C1* 1RB"}=" C1' 1RB";
$resn_exch{" O5* 1SC"}=" O5' 1SC"; $resn_exch{" C5* 1SC"}=" C5' 1SC"; $resn_exch{" C4* 1SC"}=" C4' 1SC";
$resn_exch{" O4* 1SC"}=" O4' 1SC"; $resn_exch{" C3* 1SC"}=" C3' 1SC"; $resn_exch{" O3* 1SC"}=" O3' 1SC";
$resn_exch{" C2* 1SC"}=" C2' 1SC"; $resn_exch{" O2* 1SC"}=" O2' 1SC"; $resn_exch{" C1* 1SC"}=" C1' 1SC";
$resn_exch{" N1* 1TP"}=" N1' 1TP"; $resn_exch{" C2* 1TP"}=" C2' 1TP"; $resn_exch{" N3* 1TP"}=" N3' 1TP";
$resn_exch{" C4* 1TP"}=" C4' 1TP"; $resn_exch{" N4* 1TP"}=" N4' 1TP"; $resn_exch{" C5* 1TP"}=" C5' 1TP";
$resn_exch{" C6* 1TP"}=" C6' 1TP"; $resn_exch{" C7* 1TP"}=" C7' 1TP"; $resn_exch{" N1* 223"}=" N1' 223";
$resn_exch{" C6* 223"}=" C6' 223"; $resn_exch{" C4* 223"}=" C4' 223"; $resn_exch{" C5* 223"}=" C5' 223";
$resn_exch{" O5* 223"}=" O5' 223"; $resn_exch{" C3* 223"}=" C3' 223"; $resn_exch{" O3* 223"}=" O3' 223";
$resn_exch{" C2* 223"}=" C2' 223"; $resn_exch{"C11' 232"}=" CB' 232"; $resn_exch{"C12' 232"}=" CC' 232";
$resn_exch{"C13' 232"}=" CD' 232"; $resn_exch{"C14' 232"}=" CE' 232"; $resn_exch{"C15' 232"}=" CF' 232";
$resn_exch{"C16' 232"}=" CG' 232"; $resn_exch{"C17' 232"}=" CH' 232"; $resn_exch{"C19' 232"}=" CJ' 232";
$resn_exch{"C10' 232"}=" CA' 232"; $resn_exch{"O10' 232"}=" OA' 232"; $resn_exch{"C18' 232"}=" CI' 232";
$resn_exch{"C20' 232"}=" CK' 232"; $resn_exch{"N21' 232"}=" NL' 232"; $resn_exch{"C22' 232"}=" CM' 232";
$resn_exch{"C23' 232"}=" CN' 232"; $resn_exch{"C24' 232"}=" CO' 232"; $resn_exch{"C25' 232"}=" CP' 232";
$resn_exch{"C26' 232"}=" CQ' 232"; $resn_exch{"C27' 232"}=" CR' 232"; $resn_exch{"CLAR 23D"}="CL1  23D";
$resn_exch{" O   23F"}=" OXT 23F"; $resn_exch{" OXT 23F"}=" O   23F"; $resn_exch{" O5* 23T"}=" O5' 23T";
$resn_exch{" C5* 23T"}=" C5' 23T"; $resn_exch{" C4* 23T"}=" C4' 23T"; $resn_exch{" O4* 23T"}=" O4' 23T";
$resn_exch{" C1* 23T"}=" C1' 23T"; $resn_exch{" C2* 23T"}=" C2' 23T"; $resn_exch{" C3* 23T"}=" C3' 23T";
$resn_exch{" O5* 25A"}=" O5' 25A"; $resn_exch{" C5* 25A"}=" C5' 25A"; $resn_exch{" C4* 25A"}=" C4' 25A";
$resn_exch{" O4* 25A"}=" O4' 25A"; $resn_exch{" C1* 25A"}=" C1' 25A"; $resn_exch{" C2* 25A"}=" C2' 25A";
$resn_exch{" O2* 25A"}=" O2' 25A"; $resn_exch{" C3* 25A"}=" C3' 25A"; $resn_exch{" O3* 25A"}=" O3' 25A";
$resn_exch{"O11P 25A"}=" OBP 25A"; $resn_exch{"O12P 25A"}=" OCP 25A"; $resn_exch{"O15* 25A"}=" OF' 25A";
$resn_exch{"C15* 25A"}=" CF' 25A"; $resn_exch{"C14* 25A"}=" CE' 25A"; $resn_exch{"O14* 25A"}=" OE' 25A";
$resn_exch{"C11* 25A"}=" CB' 25A"; $resn_exch{"C12* 25A"}=" CC' 25A"; $resn_exch{"O12* 25A"}=" OC' 25A";
$resn_exch{"C13* 25A"}=" CD' 25A"; $resn_exch{"O13* 25A"}=" OD' 25A"; $resn_exch{"O21P 25A"}=" OLP 25A";
$resn_exch{"O22P 25A"}=" OMP 25A"; $resn_exch{"O25* 25A"}=" OP' 25A"; $resn_exch{"C25* 25A"}=" CP' 25A";
$resn_exch{"C24* 25A"}=" CO' 25A"; $resn_exch{"O24* 25A"}=" OO' 25A"; $resn_exch{"C21* 25A"}=" CL' 25A";
$resn_exch{"C22* 25A"}=" CM' 25A"; $resn_exch{"O22* 25A"}=" OM' 25A"; $resn_exch{"C23* 25A"}=" CN' 25A";
$resn_exch{"O23* 25A"}=" ON' 25A"; $resn_exch{" C1* 26A"}=" C1' 26A"; $resn_exch{" O4* 26A"}=" O4' 26A";
$resn_exch{" C4* 26A"}=" C4' 26A"; $resn_exch{" C5* 26A"}=" C5' 26A"; $resn_exch{" O5* 26A"}=" O5' 26A";
$resn_exch{" C2* 26A"}=" C2' 26A"; $resn_exch{" O2* 26A"}=" O2' 26A"; $resn_exch{" C3* 26A"}=" C3' 26A";
$resn_exch{" O3* 26A"}=" O3' 26A"; $resn_exch{" C9* 2AA"}=" C9' 2AA"; $resn_exch{" C8* 2AA"}=" C8' 2AA";
$resn_exch{" C7* 2AA"}=" C7' 2AA"; $resn_exch{" C6* 2AA"}=" C6' 2AA"; $resn_exch{" C5* 2AA"}=" C5' 2AA";
$resn_exch{" C4* 2AA"}=" C4' 2AA"; $resn_exch{" C3* 2AA"}=" C3' 2AA"; $resn_exch{" C2* 2AA"}=" C2' 2AA";
$resn_exch{" O3* 2AA"}=" O3' 2AA"; $resn_exch{" O4* 2AA"}=" O4' 2AA"; $resn_exch{" C1* 2AA"}=" C1' 2AA";
$resn_exch{" O2* 2AA"}=" O2' 2AA"; $resn_exch{" O1* 2AA"}=" O1' 2AA"; $resn_exch{" O5* 2AD"}=" O5' 2AD";
$resn_exch{" C5* 2AD"}=" C5' 2AD"; $resn_exch{" C4* 2AD"}=" C4' 2AD"; $resn_exch{" O4* 2AD"}=" O4' 2AD";
$resn_exch{" C3* 2AD"}=" C3' 2AD"; $resn_exch{" O3* 2AD"}=" O3' 2AD"; $resn_exch{" C2* 2AD"}=" C2' 2AD";
$resn_exch{" C1* 2AD"}=" C1' 2AD"; $resn_exch{" C5* 2AM"}=" C5' 2AM"; $resn_exch{" O5* 2AM"}=" O5' 2AM";
$resn_exch{" C4* 2AM"}=" C4' 2AM"; $resn_exch{" O4* 2AM"}=" O4' 2AM"; $resn_exch{" C3* 2AM"}=" C3' 2AM";
$resn_exch{" O3* 2AM"}=" O3' 2AM"; $resn_exch{" C2* 2AM"}=" C2' 2AM"; $resn_exch{" O2* 2AM"}=" O2' 2AM";
$resn_exch{" C1* 2AM"}=" C1' 2AM"; $resn_exch{" O1P 2AR"}=" OP1 2AR"; $resn_exch{" O2P 2AR"}=" OP2 2AR";
$resn_exch{" O3P 2AR"}=" OP3 2AR"; $resn_exch{" O5* 2AR"}=" O5' 2AR"; $resn_exch{" C5* 2AR"}=" C5' 2AR";
$resn_exch{" C4* 2AR"}=" C4' 2AR"; $resn_exch{" CX* 2AR"}=" CX' 2AR"; $resn_exch{" C1* 2AR"}=" C1' 2AR";
$resn_exch{" C2* 2AR"}=" C2' 2AR"; $resn_exch{" C3* 2AR"}=" C3' 2AR"; $resn_exch{" O3* 2AR"}=" O3' 2AR";
$resn_exch{" O1P 2AT"}=" OP1 2AT"; $resn_exch{" O2P 2AT"}=" OP2 2AT"; $resn_exch{" O5* 2AT"}=" O5' 2AT";
$resn_exch{" C5* 2AT"}=" C5' 2AT"; $resn_exch{" C4* 2AT"}=" C4' 2AT"; $resn_exch{" O4* 2AT"}=" O4' 2AT";
$resn_exch{" C1* 2AT"}=" C1' 2AT"; $resn_exch{" C2* 2AT"}=" C2' 2AT"; $resn_exch{" O2* 2AT"}=" O2' 2AT";
$resn_exch{" C3* 2AT"}=" C3' 2AT"; $resn_exch{" O3* 2AT"}=" O3' 2AT"; $resn_exch{" O3P 2AT"}=" OP3 2AT";
$resn_exch{" O3P 2AU"}=" OP3 2AU"; $resn_exch{" O1P 2AU"}=" OP1 2AU"; $resn_exch{" O5* 2AU"}=" O5' 2AU";
$resn_exch{" C5* 2AU"}=" C5' 2AU"; $resn_exch{" C4* 2AU"}=" C4' 2AU"; $resn_exch{" O4* 2AU"}=" O4' 2AU";
$resn_exch{" C3* 2AU"}=" C3' 2AU"; $resn_exch{" O3* 2AU"}=" O3' 2AU"; $resn_exch{" C2* 2AU"}=" C2' 2AU";
$resn_exch{" C1* 2AU"}=" C1' 2AU"; $resn_exch{" N2* 2AU"}=" N2' 2AU"; $resn_exch{"1H1P 2AU"}="H1P1 2AU";
$resn_exch{" O5* 2BA"}=" O5' 2BA"; $resn_exch{" C5* 2BA"}=" C5' 2BA"; $resn_exch{" C4* 2BA"}=" C4' 2BA";
$resn_exch{" O4* 2BA"}=" O4' 2BA"; $resn_exch{" C3* 2BA"}=" C3' 2BA"; $resn_exch{" O3* 2BA"}=" O3' 2BA";
$resn_exch{" C2* 2BA"}=" C2' 2BA"; $resn_exch{" O2* 2BA"}=" O2' 2BA"; $resn_exch{" C1* 2BA"}=" C1' 2BA";
$resn_exch{"1P   2BA"}=" P1  2BA"; $resn_exch{"1O1P 2BA"}="O1P1 2BA"; $resn_exch{"1O2P 2BA"}="O2P1 2BA";
$resn_exch{"1O5* 2BA"}="O5'1 2BA"; $resn_exch{"1C5* 2BA"}="C5'1 2BA"; $resn_exch{"1C4* 2BA"}="C4'1 2BA";
$resn_exch{"1O4* 2BA"}="O4'1 2BA"; $resn_exch{"1C3* 2BA"}="C3'1 2BA"; $resn_exch{"1O3* 2BA"}="O3'1 2BA";
$resn_exch{"1C2* 2BA"}="C2'1 2BA"; $resn_exch{"1O2* 2BA"}="O2'1 2BA"; $resn_exch{"1C1* 2BA"}="C1'1 2BA";
$resn_exch{"1N9  2BA"}=" N91 2BA"; $resn_exch{"1C8  2BA"}=" C81 2BA"; $resn_exch{"1N7  2BA"}=" N71 2BA";
$resn_exch{"1C5  2BA"}=" C51 2BA"; $resn_exch{"1C6  2BA"}=" C61 2BA"; $resn_exch{"1N6  2BA"}=" N61 2BA";
$resn_exch{"1N1  2BA"}=" N11 2BA"; $resn_exch{"1C2  2BA"}=" C21 2BA"; $resn_exch{"1N3  2BA"}=" N31 2BA";
$resn_exch{"1C4  2BA"}=" C41 2BA"; $resn_exch{" O1P 2BD"}=" OP1 2BD"; $resn_exch{" O2P 2BD"}=" OP2 2BD";
$resn_exch{" O3P 2BD"}=" OP3 2BD"; $resn_exch{" O5* 2BD"}=" O5' 2BD"; $resn_exch{" C5* 2BD"}=" C5' 2BD";
$resn_exch{" C4* 2BD"}=" C4' 2BD"; $resn_exch{" O4* 2BD"}=" O4' 2BD"; $resn_exch{" C1* 2BD"}=" C1' 2BD";
$resn_exch{" C2* 2BD"}=" C2' 2BD"; $resn_exch{" C3* 2BD"}=" C3' 2BD"; $resn_exch{" O3* 2BD"}=" O3' 2BD";
$resn_exch{" O3P 2BT"}=" OP3 2BT"; $resn_exch{" O2P 2BT"}=" OP2 2BT"; $resn_exch{" O5* 2BT"}=" O5' 2BT";
$resn_exch{" C5* 2BT"}=" C5' 2BT"; $resn_exch{" C4* 2BT"}=" C4' 2BT"; $resn_exch{" O4* 2BT"}=" O4' 2BT";
$resn_exch{" C1* 2BT"}=" C1' 2BT"; $resn_exch{" C2* 2BT"}=" C2' 2BT"; $resn_exch{" O2* 2BT"}=" O2' 2BT";
$resn_exch{" CA* 2BT"}=" CA' 2BT"; $resn_exch{" CB* 2BT"}=" CB' 2BT"; $resn_exch{" CC* 2BT"}=" CC' 2BT";
$resn_exch{" C3* 2BT"}=" C3' 2BT"; $resn_exch{" O3* 2BT"}=" O3' 2BT"; $resn_exch{" CD* 2BT"}=" CD' 2BT";
$resn_exch{" O1P 2BT"}=" OP1 2BT"; $resn_exch{" O1P 2BU"}=" OP1 2BU"; $resn_exch{" O2P 2BU"}=" OP2 2BU";
$resn_exch{" O3P 2BU"}=" OP3 2BU"; $resn_exch{" O5* 2BU"}=" O5' 2BU"; $resn_exch{" C5* 2BU"}=" C5' 2BU";
$resn_exch{" C4* 2BU"}=" C4' 2BU"; $resn_exch{" O4* 2BU"}=" O4' 2BU"; $resn_exch{" C1* 2BU"}=" C1' 2BU";
$resn_exch{" C2* 2BU"}=" C2' 2BU"; $resn_exch{" C3* 2BU"}=" C3' 2BU"; $resn_exch{" O3* 2BU"}=" O3' 2BU";
$resn_exch{" O1P 2DA"}=" OP1 2DA"; $resn_exch{" O2P 2DA"}=" OP2 2DA"; $resn_exch{" O3P 2DA"}=" OP3 2DA";
$resn_exch{" O5* 2DA"}=" O5' 2DA"; $resn_exch{" C5* 2DA"}=" C5' 2DA"; $resn_exch{" C4* 2DA"}=" C4' 2DA";
$resn_exch{" O4* 2DA"}=" O4' 2DA"; $resn_exch{" C3* 2DA"}=" C3' 2DA"; $resn_exch{" C2* 2DA"}=" C2' 2DA";
$resn_exch{" C1* 2DA"}=" C1' 2DA"; $resn_exch{" O5* 2DF"}=" O5' 2DF"; $resn_exch{" C5* 2DF"}=" C5' 2DF";
$resn_exch{" C4* 2DF"}=" C4' 2DF"; $resn_exch{" O4* 2DF"}=" O4' 2DF"; $resn_exch{" C1* 2DF"}=" C1' 2DF";
$resn_exch{" C3* 2DF"}=" C3' 2DF"; $resn_exch{" C2* 2DF"}=" C2' 2DF"; $resn_exch{" O3* 2DF"}=" O3' 2DF";
$resn_exch{" C5* 2DI"}=" C5' 2DI"; $resn_exch{" O5* 2DI"}=" O5' 2DI"; $resn_exch{" C4* 2DI"}=" C4' 2DI";
$resn_exch{" O4* 2DI"}=" O4' 2DI"; $resn_exch{" C1* 2DI"}=" C1' 2DI"; $resn_exch{" C2* 2DI"}=" C2' 2DI";
$resn_exch{" C3* 2DI"}=" C3' 2DI"; $resn_exch{" O1P 2DM"}=" OP1 2DM"; $resn_exch{" O2P 2DM"}=" OP2 2DM";
$resn_exch{" O3* 2DM"}=" O3' 2DM"; $resn_exch{" O3P 2DM"}=" OP3 2DM"; $resn_exch{" O1P 2DT"}=" OP1 2DT";
$resn_exch{" O2P 2DT"}=" OP2 2DT"; $resn_exch{" O3P 2DT"}=" OP3 2DT"; $resn_exch{" O5* 2DT"}=" O5' 2DT";
$resn_exch{" C2* 2DT"}=" C2' 2DT"; $resn_exch{" C5* 2DT"}=" C5' 2DT"; $resn_exch{" C4* 2DT"}=" C4' 2DT";
$resn_exch{" O4* 2DT"}=" O4' 2DT"; $resn_exch{" C1* 2DT"}=" C1' 2DT"; $resn_exch{" C3* 2DT"}=" C3' 2DT";
$resn_exch{" O3P 2EG"}=" OP3 2EG"; $resn_exch{" O1P 2EG"}=" OP1 2EG"; $resn_exch{" O2P 2EG"}=" OP2 2EG";
$resn_exch{" O5* 2EG"}=" O5' 2EG"; $resn_exch{" C5* 2EG"}=" C5' 2EG"; $resn_exch{" C4* 2EG"}=" C4' 2EG";
$resn_exch{" O4* 2EG"}=" O4' 2EG"; $resn_exch{" C3* 2EG"}=" C3' 2EG"; $resn_exch{" O3* 2EG"}=" O3' 2EG";
$resn_exch{" C2* 2EG"}=" C2' 2EG"; $resn_exch{" C1* 2EG"}=" C1' 2EG"; $resn_exch{" N_A 2FH"}=" NA  2FH";
$resn_exch{" N_B 2FH"}=" NB  2FH"; $resn_exch{" N_C 2FH"}=" NC  2FH"; $resn_exch{" N_D 2FH"}=" ND  2FH";
$resn_exch{" O5* 2GP"}=" O5' 2GP"; $resn_exch{" C5* 2GP"}=" C5' 2GP"; $resn_exch{" C4* 2GP"}=" C4' 2GP";
$resn_exch{" O4* 2GP"}=" O4' 2GP"; $resn_exch{" C3* 2GP"}=" C3' 2GP"; $resn_exch{" O3* 2GP"}=" O3' 2GP";
$resn_exch{" C2* 2GP"}=" C2' 2GP"; $resn_exch{" O2* 2GP"}=" O2' 2GP"; $resn_exch{" C1* 2GP"}=" C1' 2GP";
$resn_exch{" O1P 2GT"}=" OP1 2GT"; $resn_exch{" O2P 2GT"}=" OP2 2GT"; $resn_exch{" O5* 2GT"}=" O5' 2GT";
$resn_exch{" C5* 2GT"}=" C5' 2GT"; $resn_exch{" C4* 2GT"}=" C4' 2GT"; $resn_exch{" O4* 2GT"}=" O4' 2GT";
$resn_exch{" C1* 2GT"}=" C1' 2GT"; $resn_exch{" C2* 2GT"}=" C2' 2GT"; $resn_exch{" O2* 2GT"}=" O2' 2GT";
$resn_exch{" C3* 2GT"}=" C3' 2GT"; $resn_exch{" O3* 2GT"}=" O3' 2GT"; $resn_exch{" O3P 2GT"}=" OP3 2GT";
$resn_exch{" O1P 2MA"}=" OP1 2MA"; $resn_exch{" O2P 2MA"}=" OP2 2MA"; $resn_exch{" O3P 2MA"}=" OP3 2MA";
$resn_exch{" O5* 2MA"}=" O5' 2MA"; $resn_exch{" C5* 2MA"}=" C5' 2MA"; $resn_exch{" C4* 2MA"}=" C4' 2MA";
$resn_exch{" O4* 2MA"}=" O4' 2MA"; $resn_exch{" C3* 2MA"}=" C3' 2MA"; $resn_exch{" O3* 2MA"}=" O3' 2MA";
$resn_exch{" C2* 2MA"}=" C2' 2MA"; $resn_exch{" O2* 2MA"}=" O2' 2MA"; $resn_exch{" C1* 2MA"}=" C1' 2MA";
$resn_exch{" AN1 2MC"}=" N1A 2MC"; $resn_exch{" AC2 2MC"}=" C2A 2MC"; $resn_exch{" AN3 2MC"}=" N3A 2MC";
$resn_exch{" AC4 2MC"}=" C4A 2MC"; $resn_exch{" AC5 2MC"}=" C5A 2MC"; $resn_exch{" AC6 2MC"}=" C6A 2MC";
$resn_exch{" AN6 2MC"}=" N6A 2MC"; $resn_exch{" AN7 2MC"}=" N7A 2MC"; $resn_exch{" AC8 2MC"}=" C8A 2MC";
$resn_exch{" AN9 2MC"}=" N9A 2MC"; $resn_exch{"AC1* 2MC"}=" C1B 2MC"; $resn_exch{"AC2* 2MC"}=" C2B 2MC";
$resn_exch{"AO2* 2MC"}=" O2B 2MC"; $resn_exch{"AC3* 2MC"}=" C3B 2MC"; $resn_exch{"AO3* 2MC"}=" O3B 2MC";
$resn_exch{"AP3* 2MC"}=" P3B 2MC"; $resn_exch{" AO7 2MC"}=" O7A 2MC"; $resn_exch{" AO8 2MC"}=" O8A 2MC";
$resn_exch{" AO9 2MC"}=" O9A 2MC"; $resn_exch{"AC4* 2MC"}=" C4B 2MC"; $resn_exch{"AO4* 2MC"}=" O4B 2MC";
$resn_exch{"AC5* 2MC"}=" C5B 2MC"; $resn_exch{"AO5* 2MC"}=" O5B 2MC"; $resn_exch{" AP1 2MC"}=" P1A 2MC";
$resn_exch{" AO1 2MC"}=" O1A 2MC"; $resn_exch{" AO2 2MC"}=" O2A 2MC"; $resn_exch{" AO3 2MC"}=" O3A 2MC";
$resn_exch{" AP2 2MC"}=" P2A 2MC"; $resn_exch{" AO4 2MC"}=" O4A 2MC"; $resn_exch{" AO5 2MC"}=" O5A 2MC";
$resn_exch{" AO6 2MC"}=" O6A 2MC"; $resn_exch{"PC11 2MC"}=" CBP 2MC"; $resn_exch{"PC12 2MC"}=" CCP 2MC";
$resn_exch{"PC13 2MC"}=" CDP 2MC"; $resn_exch{"PC14 2MC"}=" CEP 2MC"; $resn_exch{"PC10 2MC"}=" CAP 2MC";
$resn_exch{"PO10 2MC"}=" OAP 2MC"; $resn_exch{" PC9 2MC"}=" C9P 2MC"; $resn_exch{" PO9 2MC"}=" O9P 2MC";
$resn_exch{" PN8 2MC"}=" N8P 2MC"; $resn_exch{" PC7 2MC"}=" C7P 2MC"; $resn_exch{" PC6 2MC"}=" C6P 2MC";
$resn_exch{" PC5 2MC"}=" C5P 2MC"; $resn_exch{" PO5 2MC"}=" O5P 2MC"; $resn_exch{" PN4 2MC"}=" N4P 2MC";
$resn_exch{" PC3 2MC"}=" C3P 2MC"; $resn_exch{" PC2 2MC"}=" C2P 2MC"; $resn_exch{" PS1 2MC"}=" S1P 2MC";
$resn_exch{" O5* 2MD"}=" O5' 2MD"; $resn_exch{" C5* 2MD"}=" C5' 2MD"; $resn_exch{" C4* 2MD"}=" C4' 2MD";
$resn_exch{" O4* 2MD"}=" O4' 2MD"; $resn_exch{" C1* 2MD"}=" C1' 2MD"; $resn_exch{" C2* 2MD"}=" C2' 2MD";
$resn_exch{" O2* 2MD"}=" O2' 2MD"; $resn_exch{" C3* 2MD"}=" C3' 2MD"; $resn_exch{" O3* 2MD"}=" O3' 2MD";
$resn_exch{" CA* 2ME"}=" CA' 2ME"; $resn_exch{" CB* 2ME"}=" CB' 2ME"; $resn_exch{" OC* 2ME"}=" OC' 2ME";
$resn_exch{" CD* 2ME"}=" CD' 2ME"; $resn_exch{" O1P 2MG"}=" OP1 2MG"; $resn_exch{" O2P 2MG"}=" OP2 2MG";
$resn_exch{" O3P 2MG"}=" OP3 2MG"; $resn_exch{" O5* 2MG"}=" O5' 2MG"; $resn_exch{" C5* 2MG"}=" C5' 2MG";
$resn_exch{" C4* 2MG"}=" C4' 2MG"; $resn_exch{" O4* 2MG"}=" O4' 2MG"; $resn_exch{" C3* 2MG"}=" C3' 2MG";
$resn_exch{" O3* 2MG"}=" O3' 2MG"; $resn_exch{" C2* 2MG"}=" C2' 2MG"; $resn_exch{" O2* 2MG"}=" O2' 2MG";
$resn_exch{" C1* 2MG"}=" C1' 2MG"; $resn_exch{" C1* 2MU"}=" C1' 2MU"; $resn_exch{" C2* 2MU"}=" C2' 2MU";
$resn_exch{" O2* 2MU"}=" O2' 2MU"; $resn_exch{" C3* 2MU"}=" C3' 2MU"; $resn_exch{" C4* 2MU"}=" C4' 2MU";
$resn_exch{" O3* 2MU"}=" O3' 2MU"; $resn_exch{" O4* 2MU"}=" O4' 2MU"; $resn_exch{" C5* 2MU"}=" C5' 2MU";
$resn_exch{" O5* 2MU"}=" O5' 2MU"; $resn_exch{" C6* 2MU"}=" C6' 2MU"; $resn_exch{" O1P 2MU"}=" OP1 2MU";
$resn_exch{" O2P 2MU"}=" OP2 2MU"; $resn_exch{" O3P 2MU"}=" OP3 2MU"; $resn_exch{" C1* 2ND"}=" C1' 2ND";
$resn_exch{" O4* 2ND"}=" O4' 2ND"; $resn_exch{" C2* 2ND"}=" C2' 2ND"; $resn_exch{" C3* 2ND"}=" C3' 2ND";
$resn_exch{" O3* 2ND"}=" O3' 2ND"; $resn_exch{" C4* 2ND"}=" C4' 2ND"; $resn_exch{" C5* 2ND"}=" C5' 2ND";
$resn_exch{" O5* 2ND"}=" O5' 2ND"; $resn_exch{" O3P 2NT"}=" OP3 2NT"; $resn_exch{" O2P 2NT"}=" OP2 2NT";
$resn_exch{" O5* 2NT"}=" O5' 2NT"; $resn_exch{" C5* 2NT"}=" C5' 2NT"; $resn_exch{" C4* 2NT"}=" C4' 2NT";
$resn_exch{" O4* 2NT"}=" O4' 2NT"; $resn_exch{" C1* 2NT"}=" C1' 2NT"; $resn_exch{" C2* 2NT"}=" C2' 2NT";
$resn_exch{" O2* 2NT"}=" O2' 2NT"; $resn_exch{" C3* 2NT"}=" C3' 2NT"; $resn_exch{" O3* 2NT"}=" O3' 2NT";
$resn_exch{" O1P 2NT"}=" OP1 2NT"; $resn_exch{" O1P 2OT"}=" OP1 2OT"; $resn_exch{" O2P 2OT"}=" OP2 2OT";
$resn_exch{" O5* 2OT"}=" O5' 2OT"; $resn_exch{" C5* 2OT"}=" C5' 2OT"; $resn_exch{" C4* 2OT"}=" C4' 2OT";
$resn_exch{" O4* 2OT"}=" O4' 2OT"; $resn_exch{" C1* 2OT"}=" C1' 2OT"; $resn_exch{" C2* 2OT"}=" C2' 2OT";
$resn_exch{" O2* 2OT"}=" O2' 2OT"; $resn_exch{" C3* 2OT"}=" C3' 2OT"; $resn_exch{" O3* 2OT"}=" O3' 2OT";
$resn_exch{" O3P 2OT"}=" OP3 2OT"; $resn_exch{" O3P 2PR"}=" OP3 2PR"; $resn_exch{" O1P 2PR"}=" OP1 2PR";
$resn_exch{" O2P 2PR"}=" OP2 2PR"; $resn_exch{" O5* 2PR"}=" O5' 2PR"; $resn_exch{" C5* 2PR"}=" C5' 2PR";
$resn_exch{" C4* 2PR"}=" C4' 2PR"; $resn_exch{" O4* 2PR"}=" O4' 2PR"; $resn_exch{" C3* 2PR"}=" C3' 2PR";
$resn_exch{" O3* 2PR"}=" O3' 2PR"; $resn_exch{" C2* 2PR"}=" C2' 2PR"; $resn_exch{" C1* 2PR"}=" C1' 2PR";
$resn_exch{" O3P 2ST"}=" OP3 2ST"; $resn_exch{" O1P 2ST"}=" OP1 2ST"; $resn_exch{" O2P 2ST"}=" OP2 2ST";
$resn_exch{" O5* 2ST"}=" O5' 2ST"; $resn_exch{" C2* 2ST"}=" C2' 2ST"; $resn_exch{" C5* 2ST"}=" C5' 2ST";
$resn_exch{" C4* 2ST"}=" C4' 2ST"; $resn_exch{" O4* 2ST"}=" O4' 2ST"; $resn_exch{" C1* 2ST"}=" C1' 2ST";
$resn_exch{" C3* 2ST"}=" C3' 2ST"; $resn_exch{" CA* 2ST"}=" CA' 2ST"; $resn_exch{" O3* 2ST"}=" O3' 2ST";
$resn_exch{" N1* 2TP"}=" N1' 2TP"; $resn_exch{" C2* 2TP"}=" C2' 2TP"; $resn_exch{" N3* 2TP"}=" N3' 2TP";
$resn_exch{" C4* 2TP"}=" C4' 2TP"; $resn_exch{" N4* 2TP"}=" N4' 2TP"; $resn_exch{" C5* 2TP"}=" C5' 2TP";
$resn_exch{" C6* 2TP"}=" C6' 2TP"; $resn_exch{" C2* 2VA"}=" C2' 2VA"; $resn_exch{" C3* 2VA"}=" C3' 2VA";
$resn_exch{" O3* 2VA"}=" O3' 2VA"; $resn_exch{" C4* 2VA"}=" C4' 2VA"; $resn_exch{" C5* 2VA"}=" C5' 2VA";
$resn_exch{" O5* 2VA"}=" O5' 2VA"; $resn_exch{" O4* 2VA"}=" O4' 2VA"; $resn_exch{" C1* 2VA"}=" C1' 2VA";
$resn_exch{"C31' 312"}=" CVX 312"; $resn_exch{"C32' 312"}=" CWX 312"; $resn_exch{"O33' 312"}=" OXX 312";
$resn_exch{"O34' 312"}=" OYX 312"; $resn_exch{"C42' 312"}=" C6X 312"; $resn_exch{"C43' 312"}=" C7X 312";
$resn_exch{"O44' 312"}=" O8X 312"; $resn_exch{"O45' 312"}=" O9X 312"; $resn_exch{"C1'' 312"}=" C1B 312";
$resn_exch{"C2'' 312"}=" C2B 312"; $resn_exch{"C3'' 312"}=" C3B 312"; $resn_exch{"C4'' 312"}=" C4B 312";
$resn_exch{"C5'' 312"}=" C5B 312"; $resn_exch{"C6'' 312"}=" C6B 312"; $resn_exch{"O2'' 312"}=" O2B 312";
$resn_exch{"C31' 334"}=" CV' 334"; $resn_exch{"1H2' 34B"}="H2'1 34B"; $resn_exch{"2H2' 34B"}="H2'2 34B";
$resn_exch{"1H4' 34B"}="H4'1 34B"; $resn_exch{"2H4' 34B"}="H4'2 34B"; $resn_exch{" C1* 357"}=" C1' 357";
$resn_exch{" C7* 357"}=" C7' 357"; $resn_exch{" N1\" 357"}=" N1' 357"; $resn_exch{" C6\" 357"}=" C6' 357";
$resn_exch{" C2\" 357"}=" C2' 357"; $resn_exch{" C3\" 357"}=" C3' 357"; $resn_exch{" O4\" 357"}=" O4' 357";
$resn_exch{" C5\" 357"}=" C5' 357"; $resn_exch{" C6* 357"}=" C6B 357"; $resn_exch{" C5* 357"}=" C5B 357";
$resn_exch{" C4* 357"}=" C4' 357"; $resn_exch{" C3* 357"}=" C3B 357"; $resn_exch{" C8* 357"}=" C8' 357";
$resn_exch{" C2* 357"}=" C2B 357"; $resn_exch{" O5* 35G"}=" O5' 35G"; $resn_exch{" C5* 35G"}=" C5' 35G";
$resn_exch{" C4* 35G"}=" C4' 35G"; $resn_exch{" O4* 35G"}=" O4' 35G"; $resn_exch{" C3* 35G"}=" C3' 35G";
$resn_exch{" O3* 35G"}=" O3' 35G"; $resn_exch{" C2* 35G"}=" C2' 35G"; $resn_exch{" O2* 35G"}=" O2' 35G";
$resn_exch{" C1* 35G"}=" C1' 35G"; $resn_exch{" AP  3AA"}=" PA  3AA"; $resn_exch{" AO1 3AA"}=" O1A 3AA";
$resn_exch{" AO2 3AA"}=" O2A 3AA"; $resn_exch{"AO5* 3AA"}=" O5B 3AA"; $resn_exch{"AC5* 3AA"}=" C5B 3AA";
$resn_exch{"AC4* 3AA"}=" C4B 3AA"; $resn_exch{"AO4* 3AA"}=" O4B 3AA"; $resn_exch{"AC1* 3AA"}=" C1B 3AA";
$resn_exch{" AN9 3AA"}=" N9A 3AA"; $resn_exch{" AC4 3AA"}=" C4A 3AA"; $resn_exch{" AN3 3AA"}=" N3A 3AA";
$resn_exch{" AC2 3AA"}=" C2A 3AA"; $resn_exch{" AN1 3AA"}=" N1A 3AA"; $resn_exch{" AC6 3AA"}=" C6A 3AA";
$resn_exch{" AN6 3AA"}=" N6A 3AA"; $resn_exch{" AC5 3AA"}=" C5A 3AA"; $resn_exch{" AN7 3AA"}=" N7A 3AA";
$resn_exch{" AC8 3AA"}=" C8A 3AA"; $resn_exch{"AC2* 3AA"}=" C2B 3AA"; $resn_exch{"AO2* 3AA"}=" O2B 3AA";
$resn_exch{"AP2* 3AA"}=" P2B 3AA"; $resn_exch{"AOP1 3AA"}=" O1X 3AA"; $resn_exch{"AOP2 3AA"}=" O2X 3AA";
$resn_exch{"AOP3 3AA"}=" O3X 3AA"; $resn_exch{"AC3* 3AA"}=" C3B 3AA"; $resn_exch{"AO3* 3AA"}=" O3B 3AA";
$resn_exch{" NP  3AA"}=" PN  3AA"; $resn_exch{" NO1 3AA"}=" O1N 3AA"; $resn_exch{" NO2 3AA"}=" O2N 3AA";
$resn_exch{"NO5* 3AA"}=" O5D 3AA"; $resn_exch{"NC5* 3AA"}=" C5D 3AA"; $resn_exch{"NC4* 3AA"}=" C4D 3AA";
$resn_exch{"NO4* 3AA"}=" O4D 3AA"; $resn_exch{"NC3* 3AA"}=" C3D 3AA"; $resn_exch{"NO3* 3AA"}=" O3D 3AA";
$resn_exch{"NC2* 3AA"}=" C2D 3AA"; $resn_exch{"NO2* 3AA"}=" O2D 3AA"; $resn_exch{"NC1* 3AA"}=" C1D 3AA";
$resn_exch{" NN1 3AA"}=" N1N 3AA"; $resn_exch{" NC2 3AA"}=" C2N 3AA"; $resn_exch{" NC4 3AA"}=" C4N 3AA";
$resn_exch{" NC5 3AA"}=" C5N 3AA"; $resn_exch{" NC6 3AA"}=" C6N 3AA"; $resn_exch{" NC3 3AA"}=" C3N 3AA";
$resn_exch{" NN7 3AA"}=" N7N 3AA"; $resn_exch{" O5* 3AD"}=" O5' 3AD"; $resn_exch{" C5* 3AD"}=" C5' 3AD";
$resn_exch{" C4* 3AD"}=" C4' 3AD"; $resn_exch{" O4* 3AD"}=" O4' 3AD"; $resn_exch{" C3* 3AD"}=" C3' 3AD";
$resn_exch{" C2* 3AD"}=" C2' 3AD"; $resn_exch{" O2* 3AD"}=" O2' 3AD"; $resn_exch{" C1* 3AD"}=" C1' 3AD";
$resn_exch{" O2* 3AM"}=" O2' 3AM"; $resn_exch{" C2* 3AM"}=" C2' 3AM"; $resn_exch{" C3* 3AM"}=" C3' 3AM";
$resn_exch{" O3* 3AM"}=" O3' 3AM"; $resn_exch{" C1* 3AM"}=" C1' 3AM"; $resn_exch{" O4* 3AM"}=" O4' 3AM";
$resn_exch{" C4* 3AM"}=" C4' 3AM"; $resn_exch{" C5* 3AM"}=" C5' 3AM"; $resn_exch{" O5* 3AM"}=" O5' 3AM";
$resn_exch{" O5* 3AN"}=" O5' 3AN"; $resn_exch{" C5* 3AN"}=" C5' 3AN"; $resn_exch{" C4* 3AN"}=" C4' 3AN";
$resn_exch{" O4* 3AN"}=" O4' 3AN"; $resn_exch{" C3* 3AN"}=" C3' 3AN"; $resn_exch{" N3* 3AN"}=" N3' 3AN";
$resn_exch{" C2* 3AN"}=" C2' 3AN"; $resn_exch{" O2* 3AN"}=" O2' 3AN"; $resn_exch{" C1* 3AN"}=" C1' 3AN";
$resn_exch{" O5* 3AT"}=" O5' 3AT"; $resn_exch{" C5* 3AT"}=" C5' 3AT"; $resn_exch{" C4* 3AT"}=" C4' 3AT";
$resn_exch{" O4* 3AT"}=" O4' 3AT"; $resn_exch{" C3* 3AT"}=" C3' 3AT"; $resn_exch{" C2* 3AT"}=" C2' 3AT";
$resn_exch{" O2* 3AT"}=" O2' 3AT"; $resn_exch{" C1* 3AT"}=" C1' 3AT"; $resn_exch{" O5* 3DA"}=" O5' 3DA";
$resn_exch{" C5* 3DA"}=" C5' 3DA"; $resn_exch{" C4* 3DA"}=" C4' 3DA"; $resn_exch{" O4* 3DA"}=" O4' 3DA";
$resn_exch{" C1* 3DA"}=" C1' 3DA"; $resn_exch{" C2* 3DA"}=" C2' 3DA"; $resn_exch{" O2* 3DA"}=" O2' 3DA";
$resn_exch{" C3* 3DA"}=" C3' 3DA"; $resn_exch{" O5* 3DR"}=" O5' 3DR"; $resn_exch{" O1P 3DR"}=" OP1 3DR";
$resn_exch{" O2P 3DR"}=" OP2 3DR"; $resn_exch{" O3P 3DR"}=" OP3 3DR"; $resn_exch{" C2* 3DR"}=" C2' 3DR";
$resn_exch{" C5* 3DR"}=" C5' 3DR"; $resn_exch{" C4* 3DR"}=" C4' 3DR"; $resn_exch{" O4* 3DR"}=" O4' 3DR";
$resn_exch{" C1* 3DR"}=" C1' 3DR"; $resn_exch{" C3* 3DR"}=" C3' 3DR"; $resn_exch{" O3* 3DR"}=" O3' 3DR";
$resn_exch{" O5* 3GP"}=" O5' 3GP"; $resn_exch{" C5* 3GP"}=" C5' 3GP"; $resn_exch{" C4* 3GP"}=" C4' 3GP";
$resn_exch{" O4* 3GP"}=" O4' 3GP"; $resn_exch{" C3* 3GP"}=" C3' 3GP"; $resn_exch{" O3* 3GP"}=" O3' 3GP";
$resn_exch{" C2* 3GP"}=" C2' 3GP"; $resn_exch{" O2* 3GP"}=" O2' 3GP"; $resn_exch{" C1* 3GP"}=" C1' 3GP";
$resn_exch{" AN1 3HC"}=" N1A 3HC"; $resn_exch{" AC2 3HC"}=" C2A 3HC"; $resn_exch{" AN3 3HC"}=" N3A 3HC";
$resn_exch{" AC4 3HC"}=" C4A 3HC"; $resn_exch{" AC5 3HC"}=" C5A 3HC"; $resn_exch{" AC6 3HC"}=" C6A 3HC";
$resn_exch{" AN6 3HC"}=" N6A 3HC"; $resn_exch{" AN7 3HC"}=" N7A 3HC"; $resn_exch{" AC8 3HC"}=" C8A 3HC";
$resn_exch{" AN9 3HC"}=" N9A 3HC"; $resn_exch{"AC1* 3HC"}=" C1B 3HC"; $resn_exch{"AC2* 3HC"}=" C2B 3HC";
$resn_exch{"AO2* 3HC"}=" O2B 3HC"; $resn_exch{"AC3* 3HC"}=" C3B 3HC"; $resn_exch{"AO3* 3HC"}=" O3B 3HC";
$resn_exch{"AP3* 3HC"}=" P3B 3HC"; $resn_exch{" AO7 3HC"}=" O7A 3HC"; $resn_exch{" AO8 3HC"}=" O8A 3HC";
$resn_exch{" AO9 3HC"}=" O9A 3HC"; $resn_exch{"AC4* 3HC"}=" C4B 3HC"; $resn_exch{"AO4* 3HC"}=" O4B 3HC";
$resn_exch{"AC5* 3HC"}=" C5B 3HC"; $resn_exch{"AO5* 3HC"}=" O5B 3HC"; $resn_exch{" AP1 3HC"}=" P1A 3HC";
$resn_exch{" AO1 3HC"}=" O1A 3HC"; $resn_exch{" AO2 3HC"}=" O2A 3HC"; $resn_exch{" AO3 3HC"}=" O3A 3HC";
$resn_exch{" AP2 3HC"}=" P2A 3HC"; $resn_exch{" AO4 3HC"}=" O4A 3HC"; $resn_exch{" AO5 3HC"}=" O5A 3HC";
$resn_exch{" AO6 3HC"}=" O6A 3HC"; $resn_exch{"PC11 3HC"}=" CBP 3HC"; $resn_exch{"PC12 3HC"}=" CCP 3HC";
$resn_exch{"PC13 3HC"}=" CDP 3HC"; $resn_exch{"PC14 3HC"}=" CEP 3HC"; $resn_exch{"PC10 3HC"}=" CAP 3HC";
$resn_exch{"PO10 3HC"}=" OAP 3HC"; $resn_exch{" PC9 3HC"}=" C9P 3HC"; $resn_exch{" PO9 3HC"}=" O9P 3HC";
$resn_exch{" PN8 3HC"}=" N8P 3HC"; $resn_exch{" PC7 3HC"}=" C7P 3HC"; $resn_exch{" PC6 3HC"}=" C6P 3HC";
$resn_exch{" PC5 3HC"}=" C5P 3HC"; $resn_exch{" PO5 3HC"}=" O5P 3HC"; $resn_exch{" PN4 3HC"}=" N4P 3HC";
$resn_exch{" PC3 3HC"}=" C3P 3HC"; $resn_exch{" PC2 3HC"}=" C2P 3HC"; $resn_exch{" PS1 3HC"}=" S1P 3HC";
$resn_exch{" O1P 3ME"}=" OP1 3ME"; $resn_exch{" O2P 3ME"}=" OP2 3ME"; $resn_exch{" O5* 3PO"}=" O5' 3PO";
$resn_exch{" C1* 44D"}=" C1B 44D"; $resn_exch{" C2* 44D"}=" C2B 44D"; $resn_exch{" C3* 44D"}=" C3B 44D";
$resn_exch{" C4* 44D"}=" C4B 44D"; $resn_exch{" C5* 44D"}=" C5B 44D"; $resn_exch{" C6* 44D"}=" C6B 44D";
$resn_exch{" O3* 44D"}=" O3' 44D"; $resn_exch{" O4* 44D"}=" O4B 44D"; $resn_exch{" O5* 44D"}=" O5B 44D";
$resn_exch{" O3P 4AC"}=" OP3 4AC"; $resn_exch{" O1P 4AC"}=" OP1 4AC"; $resn_exch{" O2P 4AC"}=" OP2 4AC";
$resn_exch{" O5* 4AC"}=" O5' 4AC"; $resn_exch{" C5* 4AC"}=" C5' 4AC"; $resn_exch{" C4* 4AC"}=" C4' 4AC";
$resn_exch{" O4* 4AC"}=" O4' 4AC"; $resn_exch{" C3* 4AC"}=" C3' 4AC"; $resn_exch{" O3* 4AC"}=" O3' 4AC";
$resn_exch{" C2* 4AC"}=" C2' 4AC"; $resn_exch{" O2* 4AC"}=" O2' 4AC"; $resn_exch{" C1* 4AC"}=" C1' 4AC";
$resn_exch{" O5* 4AD"}=" O5' 4AD"; $resn_exch{" C5* 4AD"}=" C5' 4AD"; $resn_exch{" C4* 4AD"}=" C4' 4AD";
$resn_exch{" C3* 4AD"}=" C3' 4AD"; $resn_exch{" O3* 4AD"}=" O3' 4AD"; $resn_exch{" C2* 4AD"}=" C2' 4AD";
$resn_exch{" O2* 4AD"}=" O2' 4AD"; $resn_exch{" C1* 4AD"}=" C1' 4AD"; $resn_exch{" O4* 4AD"}=" O4' 4AD";
$resn_exch{" OT1 4BF"}=" OXT 4BF"; $resn_exch{" AN1 4CA"}=" N1A 4CA"; $resn_exch{" AC2 4CA"}=" C2A 4CA";
$resn_exch{" AN3 4CA"}=" N3A 4CA"; $resn_exch{" AC4 4CA"}=" C4A 4CA"; $resn_exch{" AC5 4CA"}=" C5A 4CA";
$resn_exch{" AC6 4CA"}=" C6A 4CA"; $resn_exch{" AN6 4CA"}=" N6A 4CA"; $resn_exch{" AN7 4CA"}=" N7A 4CA";
$resn_exch{" AC8 4CA"}=" C8A 4CA"; $resn_exch{" AN9 4CA"}=" N9A 4CA"; $resn_exch{"AC1* 4CA"}=" C1D 4CA";
$resn_exch{"AC2* 4CA"}=" C2D 4CA"; $resn_exch{"AO2* 4CA"}=" O2D 4CA"; $resn_exch{"AC3* 4CA"}=" C3D 4CA";
$resn_exch{"AO3* 4CA"}=" O3D 4CA"; $resn_exch{"AP3* 4CA"}=" P3D 4CA"; $resn_exch{" AO7 4CA"}=" O7A 4CA";
$resn_exch{" AO8 4CA"}=" O8A 4CA"; $resn_exch{" AO9 4CA"}=" O9A 4CA"; $resn_exch{"AC4* 4CA"}=" C4D 4CA";
$resn_exch{"AO4* 4CA"}=" O4D 4CA"; $resn_exch{"AC5* 4CA"}=" C5D 4CA"; $resn_exch{"AO5* 4CA"}=" O5D 4CA";
$resn_exch{" AP1 4CA"}=" P1A 4CA"; $resn_exch{" AO1 4CA"}=" O1A 4CA"; $resn_exch{" AO2 4CA"}=" O2A 4CA";
$resn_exch{" AO3 4CA"}=" O3A 4CA"; $resn_exch{" AP2 4CA"}=" P2A 4CA"; $resn_exch{" AO4 4CA"}=" O4A 4CA";
$resn_exch{" AO5 4CA"}=" O5A 4CA"; $resn_exch{" AO6 4CA"}=" O6A 4CA"; $resn_exch{"PC11 4CA"}=" CBP 4CA";
$resn_exch{"PC12 4CA"}=" CCP 4CA"; $resn_exch{"PC13 4CA"}=" CDP 4CA"; $resn_exch{"PC14 4CA"}=" CEP 4CA";
$resn_exch{"PC10 4CA"}=" CAP 4CA"; $resn_exch{"PO10 4CA"}=" OAP 4CA"; $resn_exch{" PC9 4CA"}=" C9P 4CA";
$resn_exch{" PO9 4CA"}=" O9P 4CA"; $resn_exch{" PN8 4CA"}=" N8P 4CA"; $resn_exch{" PC7 4CA"}=" C7P 4CA";
$resn_exch{" PC6 4CA"}=" C6P 4CA"; $resn_exch{" PC5 4CA"}=" C5P 4CA"; $resn_exch{" PO5 4CA"}=" O5P 4CA";
$resn_exch{" PN4 4CA"}=" N4P 4CA"; $resn_exch{" PC3 4CA"}=" C3P 4CA"; $resn_exch{" PC2 4CA"}=" C2P 4CA";
$resn_exch{" PS1 4CA"}=" S1P 4CA"; $resn_exch{" BC1 4CA"}=" C1B 4CA"; $resn_exch{" BC2 4CA"}=" C2B 4CA";
$resn_exch{" BC3 4CA"}=" C3B 4CA"; $resn_exch{" BC4 4CA"}=" C4B 4CA"; $resn_exch{" BC5 4CA"}=" C5B 4CA";
$resn_exch{" BO2 4CA"}=" O2B 4CA"; $resn_exch{" BC6 4CA"}=" C6B 4CA"; $resn_exch{" BC7 4CA"}=" C7B 4CA";
$resn_exch{" AN1 4CO"}=" N1A 4CO"; $resn_exch{" AC2 4CO"}=" C2A 4CO"; $resn_exch{" AN3 4CO"}=" N3A 4CO";
$resn_exch{" AC4 4CO"}=" C4A 4CO"; $resn_exch{" AC5 4CO"}=" C5A 4CO"; $resn_exch{" AC6 4CO"}=" C6A 4CO";
$resn_exch{" AN6 4CO"}=" N6A 4CO"; $resn_exch{" AN7 4CO"}=" N7A 4CO"; $resn_exch{" AC8 4CO"}=" C8A 4CO";
$resn_exch{" AN9 4CO"}=" N9A 4CO"; $resn_exch{"AC1* 4CO"}=" C1D 4CO"; $resn_exch{"AC2* 4CO"}=" C2D 4CO";
$resn_exch{"AO2* 4CO"}=" O2D 4CO"; $resn_exch{"AC3* 4CO"}=" C3D 4CO"; $resn_exch{"AO3* 4CO"}=" O3D 4CO";
$resn_exch{"AP3* 4CO"}=" P3D 4CO"; $resn_exch{" AO7 4CO"}=" O7A 4CO"; $resn_exch{" AO8 4CO"}=" O8A 4CO";
$resn_exch{" AO9 4CO"}=" O9A 4CO"; $resn_exch{"AC4* 4CO"}=" C4D 4CO"; $resn_exch{"AO4* 4CO"}=" O4D 4CO";
$resn_exch{"AC5* 4CO"}=" C5D 4CO"; $resn_exch{"AO5* 4CO"}=" O5D 4CO"; $resn_exch{" AP1 4CO"}=" P1A 4CO";
$resn_exch{" AO1 4CO"}=" O1A 4CO"; $resn_exch{" AO2 4CO"}=" O2A 4CO"; $resn_exch{" AO3 4CO"}=" O3A 4CO";
$resn_exch{" AP2 4CO"}=" P2A 4CO"; $resn_exch{" AO4 4CO"}=" O4A 4CO"; $resn_exch{" AO5 4CO"}=" O5A 4CO";
$resn_exch{" AO6 4CO"}=" O6A 4CO"; $resn_exch{"PC11 4CO"}=" CBP 4CO"; $resn_exch{"PC12 4CO"}=" CCP 4CO";
$resn_exch{"PC13 4CO"}=" CDP 4CO"; $resn_exch{"PC14 4CO"}=" CEP 4CO"; $resn_exch{"PC10 4CO"}=" CAP 4CO";
$resn_exch{"PO10 4CO"}=" OAP 4CO"; $resn_exch{" PC9 4CO"}=" C9P 4CO"; $resn_exch{" PO9 4CO"}=" O9P 4CO";
$resn_exch{" PN8 4CO"}=" N8P 4CO"; $resn_exch{" PC7 4CO"}=" C7P 4CO"; $resn_exch{" PC6 4CO"}=" C6P 4CO";
$resn_exch{" PC5 4CO"}=" C5P 4CO"; $resn_exch{" PO5 4CO"}=" O5P 4CO"; $resn_exch{" PN4 4CO"}=" N4P 4CO";
$resn_exch{" PC3 4CO"}=" C3P 4CO"; $resn_exch{" PC2 4CO"}=" C2P 4CO"; $resn_exch{" PS1 4CO"}=" S1P 4CO";
$resn_exch{" BO1 4CO"}=" O1B 4CO"; $resn_exch{" BC1 4CO"}=" C1B 4CO"; $resn_exch{" BC2 4CO"}=" C2B 4CO";
$resn_exch{" BC3 4CO"}=" C3B 4CO"; $resn_exch{" BC4 4CO"}=" C4B 4CO"; $resn_exch{" BC5 4CO"}=" C5B 4CO";
$resn_exch{" BO2 4CO"}=" O2B 4CO"; $resn_exch{" BC6 4CO"}=" C6B 4CO"; $resn_exch{" BC7 4CO"}=" C7B 4CO";
$resn_exch{" BC  4CO"}=" CB  4CO"; $resn_exch{" O1  4IN"}=" OXT 4IN"; $resn_exch{"1OP5 4IP"}=" OPF 4IP";
$resn_exch{"2OP5 4IP"}=" OPG 4IP"; $resn_exch{"3OP5 4IP"}=" OPH 4IP"; $resn_exch{" O5* 4MF"}=" O5' 4MF";
$resn_exch{" C5* 4MF"}=" C5' 4MF"; $resn_exch{" C4* 4MF"}=" C4' 4MF"; $resn_exch{" C3* 4MF"}=" C3' 4MF";
$resn_exch{" O3* 4MF"}=" O3' 4MF"; $resn_exch{" C2* 4MF"}=" C2' 4MF"; $resn_exch{" C1* 4MF"}=" C1' 4MF";
$resn_exch{" O4* 4MF"}=" O4' 4MF"; $resn_exch{" O1P 4OC"}=" OP1 4OC"; $resn_exch{" O2P 4OC"}=" OP2 4OC";
$resn_exch{" O5* 4OC"}=" O5' 4OC"; $resn_exch{" C5* 4OC"}=" C5' 4OC"; $resn_exch{" C4* 4OC"}=" C4' 4OC";
$resn_exch{" O4* 4OC"}=" O4' 4OC"; $resn_exch{" C3* 4OC"}=" C3' 4OC"; $resn_exch{" C2* 4OC"}=" C2' 4OC";
$resn_exch{" O2* 4OC"}=" O2' 4OC"; $resn_exch{" C1* 4OC"}=" C1' 4OC"; $resn_exch{" O3* 4OC"}=" O3' 4OC";
$resn_exch{" O3P 4OC"}=" OP3 4OC"; $resn_exch{" O1P 4PC"}=" OP1 4PC"; $resn_exch{" O2P 4PC"}=" OP2 4PC";
$resn_exch{" O5* 4PC"}=" O5' 4PC"; $resn_exch{" C5* 4PC"}=" C5' 4PC"; $resn_exch{" C4* 4PC"}=" C4' 4PC";
$resn_exch{" O4* 4PC"}=" O4' 4PC"; $resn_exch{" C3* 4PC"}=" C3' 4PC"; $resn_exch{" O3* 4PC"}=" O3' 4PC";
$resn_exch{" C2* 4PC"}=" C2' 4PC"; $resn_exch{" C1* 4PC"}=" C1' 4PC"; $resn_exch{" O3P 4PC"}=" OP3 4PC";
$resn_exch{" O1P 4PD"}=" OP1 4PD"; $resn_exch{" O2P 4PD"}=" OP2 4PD"; $resn_exch{" O5* 4PD"}=" O5' 4PD";
$resn_exch{" C5* 4PD"}=" C5' 4PD"; $resn_exch{" C4* 4PD"}=" C4' 4PD"; $resn_exch{" O4* 4PD"}=" O4' 4PD";
$resn_exch{" C3* 4PD"}=" C3' 4PD"; $resn_exch{" O3* 4PD"}=" O3' 4PD"; $resn_exch{" C2* 4PD"}=" C2' 4PD";
$resn_exch{" C1* 4PD"}=" C1' 4PD"; $resn_exch{" O3P 4PD"}=" OP3 4PD"; $resn_exch{" O1P 4PE"}=" OP1 4PE";
$resn_exch{" O2P 4PE"}=" OP2 4PE"; $resn_exch{" O5* 4PE"}=" O5' 4PE"; $resn_exch{" C5* 4PE"}=" C5' 4PE";
$resn_exch{" C4* 4PE"}=" C4' 4PE"; $resn_exch{" O4* 4PE"}=" O4' 4PE"; $resn_exch{" C3* 4PE"}=" C3' 4PE";
$resn_exch{" O3* 4PE"}=" O3' 4PE"; $resn_exch{" C2* 4PE"}=" C2' 4PE"; $resn_exch{" C1* 4PE"}=" C1' 4PE";
$resn_exch{" O3P 4PE"}=" OP3 4PE"; $resn_exch{" O1P 4SC"}=" OP1 4SC"; $resn_exch{" O2P 4SC"}=" OP2 4SC";
$resn_exch{" O5* 4SC"}=" O5' 4SC"; $resn_exch{" C5* 4SC"}=" C5' 4SC"; $resn_exch{" C4* 4SC"}=" C4' 4SC";
$resn_exch{" S4* 4SC"}=" S4' 4SC"; $resn_exch{" C3* 4SC"}=" C3' 4SC"; $resn_exch{" O3* 4SC"}=" O3' 4SC";
$resn_exch{" C2* 4SC"}=" C2' 4SC"; $resn_exch{" C1* 4SC"}=" C1' 4SC"; $resn_exch{" C1* 4SU"}=" C1' 4SU";
$resn_exch{" C2* 4SU"}=" C2' 4SU"; $resn_exch{" O2* 4SU"}=" O2' 4SU"; $resn_exch{" C3* 4SU"}=" C3' 4SU";
$resn_exch{" C4* 4SU"}=" C4' 4SU"; $resn_exch{" O3* 4SU"}=" O3' 4SU"; $resn_exch{" O4* 4SU"}=" O4' 4SU";
$resn_exch{" C5* 4SU"}=" C5' 4SU"; $resn_exch{" O5* 4SU"}=" O5' 4SU"; $resn_exch{" O1P 4SU"}=" OP1 4SU";
$resn_exch{" O2P 4SU"}=" OP2 4SU"; $resn_exch{" O3P 4SU"}=" OP3 4SU"; $resn_exch{" C1* 4TA"}=" C1B 4TA";
$resn_exch{" C2* 4TA"}=" C2B 4TA"; $resn_exch{" C3* 4TA"}=" C3B 4TA"; $resn_exch{" O3* 4TA"}=" O3B 4TA";
$resn_exch{" C4* 4TA"}=" C4B 4TA"; $resn_exch{" C5* 4TA"}=" C5B 4TA"; $resn_exch{" O5* 4TA"}=" O5' 4TA";
$resn_exch{" O4* 4TA"}=" O4' 4TA"; $resn_exch{" O5* 4TC"}=" O5' 4TC"; $resn_exch{" C5* 4TC"}=" C5' 4TC";
$resn_exch{" C4* 4TC"}=" C4' 4TC"; $resn_exch{" C3* 4TC"}=" C3' 4TC"; $resn_exch{" O3* 4TC"}=" O3' 4TC";
$resn_exch{" C2* 4TC"}=" C2' 4TC"; $resn_exch{" O2* 4TC"}=" O2' 4TC"; $resn_exch{" C1* 4TC"}=" C1' 4TC";
$resn_exch{" O4* 4TC"}=" O4' 4TC"; $resn_exch{" C1* 523"}=" C1' 523"; $resn_exch{" C2* 523"}=" C2' 523";
$resn_exch{" C3* 523"}=" C3' 523"; $resn_exch{" C4* 523"}=" C4' 523"; $resn_exch{" O4* 523"}=" O4' 523";
$resn_exch{" O3* 523"}=" O3' 523"; $resn_exch{" C5* 523"}=" C5' 523"; $resn_exch{" O5* 523"}=" O5' 523";
$resn_exch{" O3P 5AA"}=" OP3 5AA"; $resn_exch{" O2P 5AA"}=" OP2 5AA"; $resn_exch{" O5* 5AA"}=" O5' 5AA";
$resn_exch{" C5* 5AA"}=" C5' 5AA"; $resn_exch{" C4* 5AA"}=" C4' 5AA"; $resn_exch{" O4* 5AA"}=" O4' 5AA";
$resn_exch{" C1* 5AA"}=" C1' 5AA"; $resn_exch{" C2* 5AA"}=" C2' 5AA"; $resn_exch{" O2* 5AA"}=" O2' 5AA";
$resn_exch{" C3* 5AA"}=" C3' 5AA"; $resn_exch{" N3* 5AA"}=" N3' 5AA"; $resn_exch{" O1P 5AA"}=" OP1 5AA";
$resn_exch{" O5* 5AS"}=" O5' 5AS"; $resn_exch{" C5* 5AS"}=" C5' 5AS"; $resn_exch{" C4* 5AS"}=" C4' 5AS";
$resn_exch{" O4* 5AS"}=" O4' 5AS"; $resn_exch{" C3* 5AS"}=" C3' 5AS"; $resn_exch{" O3* 5AS"}=" O3' 5AS";
$resn_exch{" C2* 5AS"}=" C2' 5AS"; $resn_exch{" O2* 5AS"}=" O2' 5AS"; $resn_exch{" C1* 5AS"}=" C1' 5AS";
$resn_exch{" N5* 5AT"}=" N5' 5AT"; $resn_exch{" C2* 5AT"}=" C2' 5AT"; $resn_exch{" C5* 5AT"}=" C5' 5AT";
$resn_exch{" C4* 5AT"}=" C4' 5AT"; $resn_exch{" O4* 5AT"}=" O4' 5AT"; $resn_exch{" C1* 5AT"}=" C1' 5AT";
$resn_exch{" C3* 5AT"}=" C3' 5AT"; $resn_exch{" O3* 5AT"}=" O3' 5AT"; $resn_exch{" O1P 5BU"}=" OP1 5BU";
$resn_exch{" O2P 5BU"}=" OP2 5BU"; $resn_exch{" O3P 5BU"}=" OP3 5BU"; $resn_exch{" O5* 5BU"}=" O5' 5BU";
$resn_exch{" C5* 5BU"}=" C5' 5BU"; $resn_exch{" C4* 5BU"}=" C4' 5BU"; $resn_exch{" O4* 5BU"}=" O4' 5BU";
$resn_exch{" C3* 5BU"}=" C3' 5BU"; $resn_exch{" O3* 5BU"}=" O3' 5BU"; $resn_exch{" C2* 5BU"}=" C2' 5BU";
$resn_exch{" O2* 5BU"}=" O2' 5BU"; $resn_exch{" C1* 5BU"}=" C1' 5BU"; $resn_exch{" O5* 5CA"}=" O5' 5CA";
$resn_exch{" C5* 5CA"}=" C5' 5CA"; $resn_exch{" C4* 5CA"}=" C4' 5CA"; $resn_exch{" O4* 5CA"}=" O4' 5CA";
$resn_exch{" C3* 5CA"}=" C3' 5CA"; $resn_exch{" O3* 5CA"}=" O3' 5CA"; $resn_exch{" C2* 5CA"}=" C2' 5CA";
$resn_exch{" O2* 5CA"}=" O2' 5CA"; $resn_exch{" C1* 5CA"}=" C1' 5CA"; $resn_exch{" O2* 5CD"}=" O2' 5CD";
$resn_exch{" C2* 5CD"}=" C2' 5CD"; $resn_exch{" C3* 5CD"}=" C3' 5CD"; $resn_exch{" O3* 5CD"}=" O3' 5CD";
$resn_exch{" C4* 5CD"}=" C4' 5CD"; $resn_exch{" C5* 5CD"}=" C5' 5CD"; $resn_exch{" O4* 5CD"}=" O4' 5CD";
$resn_exch{" C1* 5CD"}=" C1' 5CD"; $resn_exch{"CL5* 5CG"}="CL5' 5CG"; $resn_exch{" C5* 5CG"}=" C5' 5CG";
$resn_exch{" C4* 5CG"}=" C4' 5CG"; $resn_exch{" O4* 5CG"}=" O4' 5CG"; $resn_exch{" C3* 5CG"}=" C3' 5CG";
$resn_exch{" O3* 5CG"}=" O3' 5CG"; $resn_exch{" C2* 5CG"}=" C2' 5CG"; $resn_exch{" O2* 5CG"}=" O2' 5CG";
$resn_exch{" C1* 5CG"}=" C1' 5CG"; $resn_exch{" C1* 5CM"}=" C1' 5CM"; $resn_exch{" C2* 5CM"}=" C2' 5CM";
$resn_exch{" C3* 5CM"}=" C3' 5CM"; $resn_exch{" C4* 5CM"}=" C4' 5CM"; $resn_exch{" O4* 5CM"}=" O4' 5CM";
$resn_exch{" O3* 5CM"}=" O3' 5CM"; $resn_exch{" C5* 5CM"}=" C5' 5CM"; $resn_exch{" O5* 5CM"}=" O5' 5CM";
$resn_exch{" O1P 5CM"}=" OP1 5CM"; $resn_exch{" O2P 5CM"}=" OP2 5CM"; $resn_exch{" O3P 5CM"}=" OP3 5CM";
$resn_exch{" C2* 5F1"}=" C2' 5F1"; $resn_exch{" C3* 5F1"}=" C3' 5F1"; $resn_exch{" O3* 5F1"}=" O3' 5F1";
$resn_exch{" C4* 5F1"}=" C4' 5F1"; $resn_exch{" C5* 5F1"}=" C5' 5F1"; $resn_exch{" O4* 5F1"}=" O4' 5F1";
$resn_exch{" C1* 5F1"}=" C1' 5F1"; $resn_exch{" O5* 5FA"}=" O5' 5FA"; $resn_exch{" C5* 5FA"}=" C5' 5FA";
$resn_exch{" C4* 5FA"}=" C4' 5FA"; $resn_exch{" O4* 5FA"}=" O4' 5FA"; $resn_exch{" C3* 5FA"}=" C3' 5FA";
$resn_exch{" O3* 5FA"}=" O3' 5FA"; $resn_exch{" C2* 5FA"}=" C2' 5FA"; $resn_exch{" O2* 5FA"}=" O2' 5FA";
$resn_exch{" C1* 5FA"}=" C1' 5FA"; $resn_exch{" O2P 5FC"}=" OP2 5FC"; $resn_exch{" O1P 5FC"}=" OP1 5FC";
$resn_exch{" O5* 5FC"}=" O5' 5FC"; $resn_exch{" C2* 5FC"}=" C2' 5FC"; $resn_exch{" C5* 5FC"}=" C5' 5FC";
$resn_exch{" C4* 5FC"}=" C4' 5FC"; $resn_exch{" O4* 5FC"}=" O4' 5FC"; $resn_exch{" C1* 5FC"}=" C1' 5FC";
$resn_exch{" C3* 5FC"}=" C3' 5FC"; $resn_exch{" O3* 5FC"}=" O3' 5FC"; $resn_exch{" O3P 5FC"}=" OP3 5FC";
$resn_exch{" O2* 5FD"}=" O2' 5FD"; $resn_exch{" C2* 5FD"}=" C2' 5FD"; $resn_exch{" C3* 5FD"}=" C3' 5FD";
$resn_exch{" O3* 5FD"}=" O3' 5FD"; $resn_exch{" C4* 5FD"}=" C4' 5FD"; $resn_exch{" C5* 5FD"}=" C5' 5FD";
$resn_exch{" O4* 5FD"}=" O4' 5FD"; $resn_exch{" C1* 5FD"}=" C1' 5FD"; $resn_exch{" C1* 5FU"}=" C1' 5FU";
$resn_exch{" O4* 5FU"}=" O4' 5FU"; $resn_exch{" C2* 5FU"}=" C2' 5FU"; $resn_exch{" O2* 5FU"}=" O2' 5FU";
$resn_exch{" C3* 5FU"}=" C3' 5FU"; $resn_exch{" O3* 5FU"}=" O3' 5FU"; $resn_exch{" C4* 5FU"}=" C4' 5FU";
$resn_exch{" C5* 5FU"}=" C5' 5FU"; $resn_exch{" O5* 5FU"}=" O5' 5FU"; $resn_exch{" O5* 5GP"}=" O5' 5GP";
$resn_exch{" C5* 5GP"}=" C5' 5GP"; $resn_exch{" C4* 5GP"}=" C4' 5GP"; $resn_exch{" O4* 5GP"}=" O4' 5GP";
$resn_exch{" C3* 5GP"}=" C3' 5GP"; $resn_exch{" O3* 5GP"}=" O3' 5GP"; $resn_exch{" C2* 5GP"}=" C2' 5GP";
$resn_exch{" O2* 5GP"}=" O2' 5GP"; $resn_exch{" C1* 5GP"}=" C1' 5GP"; $resn_exch{" O5* 5HT"}=" O5' 5HT";
$resn_exch{" C5* 5HT"}=" C5' 5HT"; $resn_exch{" C4* 5HT"}=" C4' 5HT"; $resn_exch{" O4* 5HT"}=" O4' 5HT";
$resn_exch{" C3* 5HT"}=" C3' 5HT"; $resn_exch{" O3* 5HT"}=" O3' 5HT"; $resn_exch{" C2* 5HT"}=" C2' 5HT";
$resn_exch{" C1* 5HT"}=" C1' 5HT"; $resn_exch{" O1P 5HU"}=" OP1 5HU"; $resn_exch{" O2P 5HU"}=" OP2 5HU";
$resn_exch{" O3P 5HU"}=" OP3 5HU"; $resn_exch{" O5* 5HU"}=" O5' 5HU"; $resn_exch{" C5* 5HU"}=" C5' 5HU";
$resn_exch{" C4* 5HU"}=" C4' 5HU"; $resn_exch{" O4* 5HU"}=" O4' 5HU"; $resn_exch{" C3* 5HU"}=" C3' 5HU";
$resn_exch{" O3* 5HU"}=" O3' 5HU"; $resn_exch{" C2* 5HU"}=" C2' 5HU"; $resn_exch{" C1* 5HU"}=" C1' 5HU";
$resn_exch{" O2* 5IC"}=" O2' 5IC"; $resn_exch{" C2* 5IC"}=" C2' 5IC"; $resn_exch{" C3* 5IC"}=" C3' 5IC";
$resn_exch{" O3* 5IC"}=" O3' 5IC"; $resn_exch{" C4* 5IC"}=" C4' 5IC"; $resn_exch{" O4* 5IC"}=" O4' 5IC";
$resn_exch{" C1* 5IC"}=" C1' 5IC"; $resn_exch{" C5* 5IC"}=" C5' 5IC"; $resn_exch{" O5* 5IC"}=" O5' 5IC";
$resn_exch{" O1P 5IC"}=" OP1 5IC"; $resn_exch{" O3P 5IC"}=" OP3 5IC"; $resn_exch{" O2P 5IC"}=" OP2 5IC";
$resn_exch{" O1P 5IT"}=" OP1 5IT"; $resn_exch{" O2P 5IT"}=" OP2 5IT"; $resn_exch{" O3P 5IT"}=" OP3 5IT";
$resn_exch{" O5* 5IT"}=" O5' 5IT"; $resn_exch{" C5* 5IT"}=" C5' 5IT"; $resn_exch{" C4* 5IT"}=" C4' 5IT";
$resn_exch{" O4* 5IT"}=" O4' 5IT"; $resn_exch{" C3* 5IT"}=" C3' 5IT"; $resn_exch{" O3* 5IT"}=" O3' 5IT";
$resn_exch{" C2* 5IT"}=" C2' 5IT"; $resn_exch{" C1* 5IT"}=" C1' 5IT"; $resn_exch{" C1* 5IU"}=" C1' 5IU";
$resn_exch{" C2* 5IU"}=" C2' 5IU"; $resn_exch{" C3* 5IU"}=" C3' 5IU"; $resn_exch{" C4* 5IU"}=" C4' 5IU";
$resn_exch{" O3* 5IU"}=" O3' 5IU"; $resn_exch{" O4* 5IU"}=" O4' 5IU"; $resn_exch{" C5* 5IU"}=" C5' 5IU";
$resn_exch{" O5* 5IU"}=" O5' 5IU"; $resn_exch{" O1P 5IU"}=" OP1 5IU"; $resn_exch{" O2P 5IU"}=" OP2 5IU";
$resn_exch{" O3P 5IU"}=" OP3 5IU"; $resn_exch{" O1P 5MC"}=" OP1 5MC"; $resn_exch{" O2P 5MC"}=" OP2 5MC";
$resn_exch{" O3P 5MC"}=" OP3 5MC"; $resn_exch{" O5* 5MC"}=" O5' 5MC"; $resn_exch{" C5* 5MC"}=" C5' 5MC";
$resn_exch{" C4* 5MC"}=" C4' 5MC"; $resn_exch{" O4* 5MC"}=" O4' 5MC"; $resn_exch{" C3* 5MC"}=" C3' 5MC";
$resn_exch{" O3* 5MC"}=" O3' 5MC"; $resn_exch{" C2* 5MC"}=" C2' 5MC"; $resn_exch{" O2* 5MC"}=" O2' 5MC";
$resn_exch{" C1* 5MC"}=" C1' 5MC"; $resn_exch{" C1* 5MU"}=" C1' 5MU"; $resn_exch{" C2* 5MU"}=" C2' 5MU";
$resn_exch{" O2* 5MU"}=" O2' 5MU"; $resn_exch{" C3* 5MU"}=" C3' 5MU"; $resn_exch{" C4* 5MU"}=" C4' 5MU";
$resn_exch{" O3* 5MU"}=" O3' 5MU"; $resn_exch{" O4* 5MU"}=" O4' 5MU"; $resn_exch{" C5* 5MU"}=" C5' 5MU";
$resn_exch{" O5* 5MU"}=" O5' 5MU"; $resn_exch{" O1P 5MU"}=" OP1 5MU"; $resn_exch{" O2P 5MU"}=" OP2 5MU";
$resn_exch{" O3P 5MU"}=" OP3 5MU"; $resn_exch{" O1P 5NC"}=" OP1 5NC"; $resn_exch{" O2P 5NC"}=" OP2 5NC";
$resn_exch{" O3P 5NC"}=" OP3 5NC"; $resn_exch{" O5* 5NC"}=" O5' 5NC"; $resn_exch{" C5* 5NC"}=" C5' 5NC";
$resn_exch{" C4* 5NC"}=" C4' 5NC"; $resn_exch{" O4* 5NC"}=" O4' 5NC"; $resn_exch{" C3* 5NC"}=" C3' 5NC";
$resn_exch{" O3* 5NC"}=" O3' 5NC"; $resn_exch{" C2* 5NC"}=" C2' 5NC"; $resn_exch{" C1* 5NC"}=" C1' 5NC";
$resn_exch{" O3P 5PC"}=" OP3 5PC"; $resn_exch{" O1P 5PC"}=" OP1 5PC"; $resn_exch{" O2P 5PC"}=" OP2 5PC";
$resn_exch{" C5* 5PC"}=" C5' 5PC"; $resn_exch{" O5* 5PC"}=" O5' 5PC"; $resn_exch{" C4* 5PC"}=" C4' 5PC";
$resn_exch{" O4* 5PC"}=" O4' 5PC"; $resn_exch{" C3* 5PC"}=" C3' 5PC"; $resn_exch{" O3* 5PC"}=" O3' 5PC";
$resn_exch{" C2* 5PC"}=" C2' 5PC"; $resn_exch{" C1* 5PC"}=" C1' 5PC"; $resn_exch{" O2P 5PY"}=" OP2 5PY";
$resn_exch{" O1P 5PY"}=" OP1 5PY"; $resn_exch{" O5* 5PY"}=" O5' 5PY"; $resn_exch{" C5* 5PY"}=" C5' 5PY";
$resn_exch{" C4* 5PY"}=" C4' 5PY"; $resn_exch{" O4* 5PY"}=" O4' 5PY"; $resn_exch{" C1* 5PY"}=" C1' 5PY";
$resn_exch{" C2* 5PY"}=" C2' 5PY"; $resn_exch{" C3* 5PY"}=" C3' 5PY"; $resn_exch{" O3* 5PY"}=" O3' 5PY";
$resn_exch{" O3P 5PY"}=" OP3 5PY"; $resn_exch{" C1* 5UD"}=" C1' 5UD"; $resn_exch{" C2* 5UD"}=" C2' 5UD";
$resn_exch{" O2* 5UD"}=" O2' 5UD"; $resn_exch{" C3* 5UD"}=" C3' 5UD"; $resn_exch{" O3* 5UD"}=" O3' 5UD";
$resn_exch{" C4* 5UD"}=" C4' 5UD"; $resn_exch{" O4* 5UD"}=" O4' 5UD"; $resn_exch{" C5* 5UD"}=" C5' 5UD";
$resn_exch{" O5* 5UD"}=" O5' 5UD"; $resn_exch{"C31' 607"}=" CVX 607"; $resn_exch{"C32' 607"}=" CWX 607";
$resn_exch{"O33' 607"}=" OXX 607"; $resn_exch{"O34' 607"}=" OYX 607"; $resn_exch{"C42' 607"}=" C6X 607";
$resn_exch{"C43' 607"}=" C7X 607"; $resn_exch{"O44' 607"}=" O8X 607"; $resn_exch{"O45' 607"}=" O9X 607";
$resn_exch{"C1'' 607"}=" C1B 607"; $resn_exch{"C2'' 607"}=" C2B 607"; $resn_exch{"C3'' 607"}=" C3B 607";
$resn_exch{"C4'' 607"}=" C4B 607"; $resn_exch{"C5'' 607"}=" C5B 607"; $resn_exch{"C6'' 607"}=" C6B 607";
$resn_exch{"N2'' 607"}=" N2B 607"; $resn_exch{" O1P 64T"}=" OP1 64T"; $resn_exch{" O2P 64T"}=" OP2 64T";
$resn_exch{" O5* 64T"}=" O5' 64T"; $resn_exch{" C5* 64T"}=" C5' 64T"; $resn_exch{" C4* 64T"}=" C4' 64T";
$resn_exch{" O4* 64T"}=" O4' 64T"; $resn_exch{" C3* 64T"}=" C3' 64T"; $resn_exch{" O3* 64T"}=" O3' 64T";
$resn_exch{" C2* 64T"}=" C2' 64T"; $resn_exch{" C1* 64T"}=" C1' 64T"; $resn_exch{" O3P 64T"}=" OP3 64T";
$resn_exch{"C31' 653"}=" CV' 653"; $resn_exch{"C32' 653"}=" CW' 653"; $resn_exch{"N33' 653"}=" NX' 653";
$resn_exch{"C1'' 653"}=" C1B 653"; $resn_exch{"C2'' 653"}=" C2B 653"; $resn_exch{"C3'' 653"}=" C3B 653";
$resn_exch{"C4'' 653"}=" C4B 653"; $resn_exch{"C5'' 653"}=" C5B 653"; $resn_exch{"C6'' 653"}=" C6B 653";
$resn_exch{"C1'' 655"}=" C1B 655"; $resn_exch{"C2'' 655"}=" C2B 655"; $resn_exch{"C3'' 655"}=" C3B 655";
$resn_exch{"C4'' 655"}=" C4B 655"; $resn_exch{"C5'' 655"}=" C5B 655"; $resn_exch{"C31' 678"}=" CV' 678";
$resn_exch{"C32' 678"}=" CW' 678"; $resn_exch{"O33' 678"}=" OX' 678"; $resn_exch{"O34' 678"}=" OY' 678";
$resn_exch{"C31' 693"}=" CVX 693"; $resn_exch{"C32' 693"}=" CWX 693"; $resn_exch{"O33' 693"}=" OXX 693";
$resn_exch{"O34' 693"}=" OYX 693"; $resn_exch{"C42' 693"}=" C6X 693"; $resn_exch{"C43' 693"}=" C7X 693";
$resn_exch{"O44' 693"}=" O8X 693"; $resn_exch{"O45' 693"}=" O9X 693"; $resn_exch{"C1'' 693"}=" C1B 693";
$resn_exch{"C2'' 693"}=" C2B 693"; $resn_exch{"C3'' 693"}=" C3B 693"; $resn_exch{"C4'' 693"}=" C4B 693";
$resn_exch{"C5'' 693"}=" C5B 693"; $resn_exch{"C6'' 693"}=" C6B 693"; $resn_exch{"F2'' 693"}=" F2B 693";
$resn_exch{"C1'' 696"}=" C1B 696"; $resn_exch{"C2'' 696"}=" C2B 696"; $resn_exch{"C3'' 696"}=" C3B 696";
$resn_exch{"C4'' 696"}=" C4B 696"; $resn_exch{"C5'' 696"}=" C5B 696"; $resn_exch{"C6'' 696"}=" C6B 696";
$resn_exch{" O2P 6CT"}=" OP2 6CT"; $resn_exch{" O1P 6CT"}=" OP1 6CT"; $resn_exch{" O5* 6CT"}=" O5' 6CT";
$resn_exch{" C5* 6CT"}=" C5' 6CT"; $resn_exch{" C4* 6CT"}=" C4' 6CT"; $resn_exch{" C3* 6CT"}=" C3' 6CT";
$resn_exch{" O3* 6CT"}=" O3' 6CT"; $resn_exch{" C2* 6CT"}=" C2' 6CT"; $resn_exch{" C1* 6CT"}=" C1' 6CT";
$resn_exch{" C*  6CT"}=" C'  6CT"; $resn_exch{" CM* 6CT"}=" CM' 6CT"; $resn_exch{"PHO1 6CT"}="HO1P 6CT";
$resn_exch{" C1* 6FA"}=" C1' 6FA"; $resn_exch{" C2* 6FA"}=" C2' 6FA"; $resn_exch{" O2* 6FA"}=" O2' 6FA";
$resn_exch{" C3* 6FA"}=" C3' 6FA"; $resn_exch{" O3* 6FA"}=" O3' 6FA"; $resn_exch{" C4* 6FA"}=" C4' 6FA";
$resn_exch{" O4* 6FA"}=" O4' 6FA"; $resn_exch{" C5* 6FA"}=" C5' 6FA"; $resn_exch{" O5* 6FA"}=" O5' 6FA";
$resn_exch{" AP  6FA"}=" PA  6FA"; $resn_exch{" AO1 6FA"}=" O1A 6FA"; $resn_exch{" AO2 6FA"}=" O2A 6FA";
$resn_exch{"AO5* 6FA"}=" O5B 6FA"; $resn_exch{"AC5* 6FA"}=" C5B 6FA"; $resn_exch{"AC4* 6FA"}=" C4B 6FA";
$resn_exch{"AO4* 6FA"}=" O4B 6FA"; $resn_exch{"AC3* 6FA"}=" C3B 6FA"; $resn_exch{"AO3* 6FA"}=" O3B 6FA";
$resn_exch{"AC2* 6FA"}=" C2B 6FA"; $resn_exch{"AO2* 6FA"}=" O2B 6FA"; $resn_exch{"AC1* 6FA"}=" C1B 6FA";
$resn_exch{" AN9 6FA"}=" N9A 6FA"; $resn_exch{" AC8 6FA"}=" C8A 6FA"; $resn_exch{" AN7 6FA"}=" N7A 6FA";
$resn_exch{" AC5 6FA"}=" C5X 6FA"; $resn_exch{" AC6 6FA"}=" C6A 6FA"; $resn_exch{" AN6 6FA"}=" N6A 6FA";
$resn_exch{" AN1 6FA"}=" N1A 6FA"; $resn_exch{" AC2 6FA"}=" C2A 6FA"; $resn_exch{" AN3 6FA"}=" N3A 6FA";
$resn_exch{" AC4 6FA"}=" C4X 6FA"; $resn_exch{" O1P 6HA"}=" OP1 6HA"; $resn_exch{" O2P 6HA"}=" OP2 6HA";
$resn_exch{" O3P 6HA"}=" OP3 6HA"; $resn_exch{" O5* 6HA"}=" O5' 6HA"; $resn_exch{" C5* 6HA"}=" C5' 6HA";
$resn_exch{" C4* 6HA"}=" C4' 6HA"; $resn_exch{" O4* 6HA"}=" O4' 6HA"; $resn_exch{" C3* 6HA"}=" C3' 6HA";
$resn_exch{" O3* 6HA"}=" O3' 6HA"; $resn_exch{" C2* 6HA"}=" C2' 6HA"; $resn_exch{" C1* 6HA"}=" C1' 6HA";
$resn_exch{" C6* 6HA"}=" C6' 6HA"; $resn_exch{" O1P 6HC"}=" OP1 6HC"; $resn_exch{" O2P 6HC"}=" OP2 6HC";
$resn_exch{" O3P 6HC"}=" OP3 6HC"; $resn_exch{" O5* 6HC"}=" O5' 6HC"; $resn_exch{" C5* 6HC"}=" C5' 6HC";
$resn_exch{" C4* 6HC"}=" C4' 6HC"; $resn_exch{" O4* 6HC"}=" O4' 6HC"; $resn_exch{" C3* 6HC"}=" C3' 6HC";
$resn_exch{" O3* 6HC"}=" O3' 6HC"; $resn_exch{" C2* 6HC"}=" C2' 6HC"; $resn_exch{" C1* 6HC"}=" C1' 6HC";
$resn_exch{" C6* 6HC"}=" C6' 6HC"; $resn_exch{" O1P 6HG"}=" OP1 6HG"; $resn_exch{" O2P 6HG"}=" OP2 6HG";
$resn_exch{" O3P 6HG"}=" OP3 6HG"; $resn_exch{" O5* 6HG"}=" O5' 6HG"; $resn_exch{" C5* 6HG"}=" C5' 6HG";
$resn_exch{" C4* 6HG"}=" C4' 6HG"; $resn_exch{" O4* 6HG"}=" O4' 6HG"; $resn_exch{" C3* 6HG"}=" C3' 6HG";
$resn_exch{" O3* 6HG"}=" O3' 6HG"; $resn_exch{" C2* 6HG"}=" C2' 6HG"; $resn_exch{" C1* 6HG"}=" C1' 6HG";
$resn_exch{" C6* 6HG"}=" C6' 6HG"; $resn_exch{" O1P 6HT"}=" OP1 6HT"; $resn_exch{" O2P 6HT"}=" OP2 6HT";
$resn_exch{" O3P 6HT"}=" OP3 6HT"; $resn_exch{" O5* 6HT"}=" O5' 6HT"; $resn_exch{" C5* 6HT"}=" C5' 6HT";
$resn_exch{" C4* 6HT"}=" C4' 6HT"; $resn_exch{" O4* 6HT"}=" O4' 6HT"; $resn_exch{" C3* 6HT"}=" C3' 6HT";
$resn_exch{" O3* 6HT"}=" O3' 6HT"; $resn_exch{" C2* 6HT"}=" C2' 6HT"; $resn_exch{" C1* 6HT"}=" C1' 6HT";
$resn_exch{" C6* 6HT"}=" C6' 6HT"; $resn_exch{" O3P 6IA"}=" OP3 6IA"; $resn_exch{" O1P 6IA"}=" OP1 6IA";
$resn_exch{" O2P 6IA"}=" OP2 6IA"; $resn_exch{" O5* 6IA"}=" O5' 6IA"; $resn_exch{" C5* 6IA"}=" C5' 6IA";
$resn_exch{" C4* 6IA"}=" C4' 6IA"; $resn_exch{" O4* 6IA"}=" O4' 6IA"; $resn_exch{" C3* 6IA"}=" C3' 6IA";
$resn_exch{" O3* 6IA"}=" O3' 6IA"; $resn_exch{" C2* 6IA"}=" C2' 6IA"; $resn_exch{" O2* 6IA"}=" O2' 6IA";
$resn_exch{" C1* 6IA"}=" C1' 6IA"; $resn_exch{" O3P 6MA"}=" OP3 6MA"; $resn_exch{" O1P 6MA"}=" OP1 6MA";
$resn_exch{" O2P 6MA"}=" OP2 6MA"; $resn_exch{" O5* 6MA"}=" O5' 6MA"; $resn_exch{" C5* 6MA"}=" C5' 6MA";
$resn_exch{" C4* 6MA"}=" C4' 6MA"; $resn_exch{" O4* 6MA"}=" O4' 6MA"; $resn_exch{" C3* 6MA"}=" C3' 6MA";
$resn_exch{" O3* 6MA"}=" O3' 6MA"; $resn_exch{" C2* 6MA"}=" C2' 6MA"; $resn_exch{" C1* 6MA"}=" C1' 6MA";
$resn_exch{" O3P 6MC"}=" OP3 6MC"; $resn_exch{" O1P 6MC"}=" OP1 6MC"; $resn_exch{" O2P 6MC"}=" OP2 6MC";
$resn_exch{" O5* 6MC"}=" O5' 6MC"; $resn_exch{" C5* 6MC"}=" C5' 6MC"; $resn_exch{" C4* 6MC"}=" C4' 6MC";
$resn_exch{" O4* 6MC"}=" O4' 6MC"; $resn_exch{" C3* 6MC"}=" C3' 6MC"; $resn_exch{" O3* 6MC"}=" O3' 6MC";
$resn_exch{" C2* 6MC"}=" C2' 6MC"; $resn_exch{" C1* 6MC"}=" C1' 6MC"; $resn_exch{" O5* 6MI"}=" O5' 6MI";
$resn_exch{" C2* 6MI"}=" C2' 6MI"; $resn_exch{" C5* 6MI"}=" C5' 6MI"; $resn_exch{" C4* 6MI"}=" C4' 6MI";
$resn_exch{" O4* 6MI"}=" O4' 6MI"; $resn_exch{" C1* 6MI"}=" C1' 6MI"; $resn_exch{" C3* 6MI"}=" C3' 6MI";
$resn_exch{" O3* 6MI"}=" O3' 6MI"; $resn_exch{" O3P 6MT"}=" OP3 6MT"; $resn_exch{" O1P 6MT"}=" OP1 6MT";
$resn_exch{" O2P 6MT"}=" OP2 6MT"; $resn_exch{" O5* 6MT"}=" O5' 6MT"; $resn_exch{" C5* 6MT"}=" C5' 6MT";
$resn_exch{" C4* 6MT"}=" C4' 6MT"; $resn_exch{" O4* 6MT"}=" O4' 6MT"; $resn_exch{" C3* 6MT"}=" C3' 6MT";
$resn_exch{" O3* 6MT"}=" O3' 6MT"; $resn_exch{" C2* 6MT"}=" C2' 6MT"; $resn_exch{" C1* 6MT"}=" C1' 6MT";
$resn_exch{" O5* 6MZ"}=" O5' 6MZ"; $resn_exch{" C5* 6MZ"}=" C5' 6MZ"; $resn_exch{" C4* 6MZ"}=" C4' 6MZ";
$resn_exch{" O4* 6MZ"}=" O4' 6MZ"; $resn_exch{" C1* 6MZ"}=" C1' 6MZ"; $resn_exch{" C2* 6MZ"}=" C2' 6MZ";
$resn_exch{" O2* 6MZ"}=" O2' 6MZ"; $resn_exch{" C3* 6MZ"}=" C3' 6MZ"; $resn_exch{" O3* 6MZ"}=" O3' 6MZ";
$resn_exch{"C1'' 6NH"}=" C1B 6NH"; $resn_exch{"C2'' 6NH"}=" C2B 6NH"; $resn_exch{"C3'' 6NH"}=" C3B 6NH";
$resn_exch{"C4'' 6NH"}=" C4B 6NH"; $resn_exch{"C5'' 6NH"}=" C5B 6NH"; $resn_exch{"C6'' 6NH"}=" C6B 6NH";
$resn_exch{" O1P 6OG"}=" OP1 6OG"; $resn_exch{" O2P 6OG"}=" OP2 6OG"; $resn_exch{" O5* 6OG"}=" O5' 6OG";
$resn_exch{" C2* 6OG"}=" C2' 6OG"; $resn_exch{" C5* 6OG"}=" C5' 6OG"; $resn_exch{" C4* 6OG"}=" C4' 6OG";
$resn_exch{" O4* 6OG"}=" O4' 6OG"; $resn_exch{" C1* 6OG"}=" C1' 6OG"; $resn_exch{" C3* 6OG"}=" C3' 6OG";
$resn_exch{" O3* 6OG"}=" O3' 6OG"; $resn_exch{" O3P 6OG"}=" OP3 6OG"; $resn_exch{" C1* 70U"}=" C1' 70U";
$resn_exch{" O2* 70U"}=" O2' 70U"; $resn_exch{" C2* 70U"}=" C2' 70U"; $resn_exch{" O3* 70U"}=" O3' 70U";
$resn_exch{" C3* 70U"}=" C3' 70U"; $resn_exch{" C4* 70U"}=" C4' 70U"; $resn_exch{" O4* 70U"}=" O4' 70U";
$resn_exch{" C5* 70U"}=" C5' 70U"; $resn_exch{" O5* 70U"}=" O5' 70U"; $resn_exch{" O1P 70U"}=" OP1 70U";
$resn_exch{" O2P 70U"}=" OP2 70U"; $resn_exch{" O3P 70U"}=" OP3 70U"; $resn_exch{"C1'' 762"}=" C1B 762";
$resn_exch{"C2'' 762"}=" C2B 762"; $resn_exch{"C3'' 762"}=" C3B 762"; $resn_exch{"C4'' 762"}=" C4B 762";
$resn_exch{"C5'' 762"}=" C5B 762"; $resn_exch{"C6'' 762"}=" C6B 762"; $resn_exch{"C1'' 780"}=" C1B 780";
$resn_exch{"C2'' 780"}=" C2B 780"; $resn_exch{"C3'' 780"}=" C3B 780"; $resn_exch{"C4'' 780"}=" C4B 780";
$resn_exch{"C5'' 780"}=" C5B 780"; $resn_exch{"C6'' 780"}=" C6B 780"; $resn_exch{"CM3' 783"}=" C3X 783";
$resn_exch{"C1'' 783"}=" C1B 783"; $resn_exch{"C2'' 783"}=" C2B 783"; $resn_exch{"C3'' 783"}=" C3B 783";
$resn_exch{"C4'' 783"}=" C4B 783"; $resn_exch{"C5'' 783"}=" C5B 783"; $resn_exch{"C6'' 783"}=" C6B 783";
$resn_exch{"CF3' 785"}=" C3X 785"; $resn_exch{"F31' 785"}=" FV' 785"; $resn_exch{"F32' 785"}=" FW' 785";
$resn_exch{"F33' 785"}=" FX' 785"; $resn_exch{" O1P 7DA"}=" OP1 7DA"; $resn_exch{" O2P 7DA"}=" OP2 7DA";
$resn_exch{" O5* 7DA"}=" O5' 7DA"; $resn_exch{" C2* 7DA"}=" C2' 7DA"; $resn_exch{" C5* 7DA"}=" C5' 7DA";
$resn_exch{" C4* 7DA"}=" C4' 7DA"; $resn_exch{" O4* 7DA"}=" O4' 7DA"; $resn_exch{" C1* 7DA"}=" C1' 7DA";
$resn_exch{" C3* 7DA"}=" C3' 7DA"; $resn_exch{" O3* 7DA"}=" O3' 7DA"; $resn_exch{" O3P 7DA"}=" OP3 7DA";
$resn_exch{" O1P 7GU"}=" OP1 7GU"; $resn_exch{" O2P 7GU"}=" OP2 7GU"; $resn_exch{" O5* 7GU"}=" O5' 7GU";
$resn_exch{" C2* 7GU"}=" C2' 7GU"; $resn_exch{" C5* 7GU"}=" C5' 7GU"; $resn_exch{" C4* 7GU"}=" C4' 7GU";
$resn_exch{" O4* 7GU"}=" O4' 7GU"; $resn_exch{" C1* 7GU"}=" C1' 7GU"; $resn_exch{" C3* 7GU"}=" C3' 7GU";
$resn_exch{" O3* 7GU"}=" O3' 7GU"; $resn_exch{" O3P 7GU"}=" OP3 7GU"; $resn_exch{" O1P 7MG"}=" OP1 7MG";
$resn_exch{" O2P 7MG"}=" OP2 7MG"; $resn_exch{" O3P 7MG"}=" OP3 7MG"; $resn_exch{" O5* 7MG"}=" O5' 7MG";
$resn_exch{" C5* 7MG"}=" C5' 7MG"; $resn_exch{" C4* 7MG"}=" C4' 7MG"; $resn_exch{" O4* 7MG"}=" O4' 7MG";
$resn_exch{" C3* 7MG"}=" C3' 7MG"; $resn_exch{" O3* 7MG"}=" O3' 7MG"; $resn_exch{" C2* 7MG"}=" C2' 7MG";
$resn_exch{" O2* 7MG"}=" O2' 7MG"; $resn_exch{" C1* 7MG"}=" C1' 7MG"; $resn_exch{"C1'' 7NH"}=" C1B 7NH";
$resn_exch{"C2'' 7NH"}=" C2B 7NH"; $resn_exch{"C3'' 7NH"}=" C3B 7NH"; $resn_exch{"C4'' 7NH"}=" C4B 7NH";
$resn_exch{"C5'' 7NH"}=" C5B 7NH"; $resn_exch{"C6'' 7NH"}=" C6B 7NH"; $resn_exch{" O5* 7RA"}=" O5' 7RA";
$resn_exch{" C5* 7RA"}=" C5' 7RA"; $resn_exch{" C4* 7RA"}=" C4' 7RA"; $resn_exch{" O4* 7RA"}=" O4' 7RA";
$resn_exch{" C3* 7RA"}=" C3' 7RA"; $resn_exch{" O3* 7RA"}=" O3' 7RA"; $resn_exch{" C2* 7RA"}=" C2' 7RA";
$resn_exch{" O2* 7RA"}=" O2' 7RA"; $resn_exch{" C1* 7RA"}=" C1' 7RA"; $resn_exch{" O5* 7RP"}=" O5' 7RP";
$resn_exch{" C5* 7RP"}=" C5' 7RP"; $resn_exch{" C4* 7RP"}=" C4' 7RP"; $resn_exch{" O4* 7RP"}=" O4' 7RP";
$resn_exch{" C3* 7RP"}=" C3' 7RP"; $resn_exch{" O3* 7RP"}=" O3' 7RP"; $resn_exch{" C2* 7RP"}=" C2' 7RP";
$resn_exch{" O2* 7RP"}=" O2' 7RP"; $resn_exch{" C1* 7RP"}=" C1' 7RP"; $resn_exch{"C14A 81A"}=" CEA 81A";
$resn_exch{"C14B 81A"}=" CEB 81A"; $resn_exch{"C31' 847"}=" CVX 847"; $resn_exch{"C32' 847"}=" CWX 847";
$resn_exch{"O33' 847"}=" OXX 847"; $resn_exch{"O34' 847"}=" OYX 847"; $resn_exch{"C42' 847"}=" C6X 847";
$resn_exch{"C43' 847"}=" C7X 847"; $resn_exch{"O44' 847"}=" O8X 847"; $resn_exch{"O45' 847"}=" O9X 847";
$resn_exch{" O5* 8BR"}=" O5' 8BR"; $resn_exch{" C5* 8BR"}=" C5' 8BR"; $resn_exch{" C4* 8BR"}=" C4' 8BR";
$resn_exch{" O4* 8BR"}=" O4' 8BR"; $resn_exch{" C3* 8BR"}=" C3' 8BR"; $resn_exch{" O3* 8BR"}=" O3' 8BR";
$resn_exch{" C2* 8BR"}=" C2' 8BR"; $resn_exch{" O2* 8BR"}=" O2' 8BR"; $resn_exch{" C1* 8BR"}=" C1' 8BR";
$resn_exch{" C5* 8DA"}=" C5' 8DA"; $resn_exch{" C4* 8DA"}=" C4' 8DA"; $resn_exch{" C3* 8DA"}=" C3' 8DA";
$resn_exch{" O3* 8DA"}=" O3' 8DA"; $resn_exch{" O4* 8DA"}=" O4' 8DA"; $resn_exch{" C1* 8DA"}=" C1' 8DA";
$resn_exch{" C2* 8DA"}=" C2' 8DA"; $resn_exch{" O5* 8DG"}=" O5' 8DG"; $resn_exch{" C5* 8DG"}=" C5' 8DG";
$resn_exch{" C4* 8DG"}=" C4' 8DG"; $resn_exch{" O4* 8DG"}=" O4' 8DG"; $resn_exch{" C3* 8DG"}=" C3' 8DG";
$resn_exch{" O3* 8DG"}=" O3' 8DG"; $resn_exch{" C2* 8DG"}=" C2' 8DG"; $resn_exch{" C1* 8DG"}=" C1' 8DG";
$resn_exch{" O3P 8FG"}=" OP3 8FG"; $resn_exch{" O1P 8FG"}=" OP1 8FG"; $resn_exch{" O2P 8FG"}=" OP2 8FG";
$resn_exch{" O5* 8FG"}=" O5' 8FG"; $resn_exch{" C2* 8FG"}=" C2' 8FG"; $resn_exch{" C5* 8FG"}=" C5' 8FG";
$resn_exch{" C4* 8FG"}=" C4' 8FG"; $resn_exch{" O4* 8FG"}=" O4' 8FG"; $resn_exch{" C1* 8FG"}=" C1' 8FG";
$resn_exch{" C3* 8FG"}=" C3' 8FG"; $resn_exch{" O3* 8FG"}=" O3' 8FG"; $resn_exch{" O5* 8GT"}=" O5' 8GT";
$resn_exch{" C5* 8GT"}=" C5' 8GT"; $resn_exch{" C4* 8GT"}=" C4' 8GT"; $resn_exch{" O4* 8GT"}=" O4' 8GT";
$resn_exch{" C3* 8GT"}=" C3' 8GT"; $resn_exch{" O3* 8GT"}=" O3' 8GT"; $resn_exch{" C2* 8GT"}=" C2' 8GT";
$resn_exch{" O2* 8GT"}=" O2' 8GT"; $resn_exch{" C1* 8GT"}=" C1' 8GT"; $resn_exch{" C1* 8HG"}=" C1' 8HG";
$resn_exch{" O4* 8HG"}=" O4' 8HG"; $resn_exch{" C2* 8HG"}=" C2' 8HG"; $resn_exch{" C3* 8HG"}=" C3' 8HG";
$resn_exch{" O3* 8HG"}=" O3' 8HG"; $resn_exch{" C4* 8HG"}=" C4' 8HG"; $resn_exch{" C5* 8HG"}=" C5' 8HG";
$resn_exch{" O5* 8HG"}=" O5' 8HG"; $resn_exch{" O5* 8ID"}=" O5B 8ID"; $resn_exch{" C5* 8ID"}=" C5B 8ID";
$resn_exch{" C4* 8ID"}=" C4B 8ID"; $resn_exch{" O4* 8ID"}=" O4B 8ID"; $resn_exch{" C1* 8ID"}=" C1B 8ID";
$resn_exch{" C2* 8ID"}=" C2B 8ID"; $resn_exch{" O2* 8ID"}=" O2B 8ID"; $resn_exch{" C3* 8ID"}=" C3B 8ID";
$resn_exch{" O3* 8ID"}=" O3B 8ID"; $resn_exch{" O3P 8MG"}=" OP3 8MG"; $resn_exch{" O2P 8MG"}=" OP2 8MG";
$resn_exch{" O5* 8MG"}=" O5' 8MG"; $resn_exch{" C2* 8MG"}=" C2' 8MG"; $resn_exch{" C5* 8MG"}=" C5' 8MG";
$resn_exch{" C4* 8MG"}=" C4' 8MG"; $resn_exch{" O4* 8MG"}=" O4' 8MG"; $resn_exch{" C1* 8MG"}=" C1' 8MG";
$resn_exch{" C3* 8MG"}=" C3' 8MG"; $resn_exch{" O3* 8MG"}=" O3' 8MG"; $resn_exch{" O1P 8MG"}=" OP1 8MG";
$resn_exch{" O3P 8OG"}=" OP3 8OG"; $resn_exch{" O1P 8OG"}=" OP1 8OG"; $resn_exch{" O2P 8OG"}=" OP2 8OG";
$resn_exch{" O5* 8OG"}=" O5' 8OG"; $resn_exch{" C5* 8OG"}=" C5' 8OG"; $resn_exch{" C4* 8OG"}=" C4' 8OG";
$resn_exch{" O4* 8OG"}=" O4' 8OG"; $resn_exch{" C3* 8OG"}=" C3' 8OG"; $resn_exch{" O3* 8OG"}=" O3' 8OG";
$resn_exch{" C2* 8OG"}=" C2' 8OG"; $resn_exch{" C1* 8OG"}=" C1' 8OG"; $resn_exch{"C31' 907"}=" CV' 907";
$resn_exch{" C5* 93A"}=" C5' 93A"; $resn_exch{" O5* 93A"}=" O5' 93A"; $resn_exch{"C1'' 991"}=" C1B 991";
$resn_exch{"C2'' 991"}=" C2B 991"; $resn_exch{"C3'' 991"}=" C3B 991"; $resn_exch{"C4'' 991"}=" C4B 991";
$resn_exch{"C5'' 991"}=" C5B 991"; $resn_exch{" O3P   A"}=" OP3   A"; $resn_exch{" O1P   A"}=" OP1   A";
$resn_exch{" O2P   A"}=" OP2   A"; $resn_exch{" O5*   A"}=" O5'   A"; $resn_exch{" C5*   A"}=" C5'   A";
$resn_exch{" C4*   A"}=" C4'   A"; $resn_exch{" O4*   A"}=" O4'   A"; $resn_exch{" C3*   A"}=" C3'   A";
$resn_exch{" O3*   A"}=" O3'   A"; $resn_exch{" C2*   A"}=" C2'   A"; $resn_exch{" O2*   A"}=" O2'   A";
$resn_exch{" C1*   A"}=" C1'   A"; $resn_exch{" O5* A12"}=" O5' A12"; $resn_exch{" C5* A12"}=" C5' A12";
$resn_exch{" C4* A12"}=" C4' A12"; $resn_exch{" O4* A12"}=" O4' A12"; $resn_exch{" C3* A12"}=" C3' A12";
$resn_exch{" O3* A12"}=" O3' A12"; $resn_exch{" C2* A12"}=" C2' A12"; $resn_exch{" O2* A12"}=" O2' A12";
$resn_exch{" C1* A12"}=" C1' A12"; $resn_exch{" O5* A1P"}=" O5' A1P"; $resn_exch{" C2* A1P"}=" C2' A1P";
$resn_exch{" C5* A1P"}=" C5' A1P"; $resn_exch{" C4* A1P"}=" C4' A1P"; $resn_exch{" O4* A1P"}=" O4' A1P";
$resn_exch{" C1* A1P"}=" C1' A1P"; $resn_exch{" C3* A1P"}=" C3' A1P"; $resn_exch{" O3* A1P"}=" O3' A1P";
$resn_exch{" C1* A1R"}=" C1' A1R"; $resn_exch{" C2* A1R"}=" C2' A1R"; $resn_exch{" O2* A1R"}=" O2' A1R";
$resn_exch{" C3* A1R"}=" C3' A1R"; $resn_exch{" O3* A1R"}=" O3' A1R"; $resn_exch{" O4* A1R"}=" O4' A1R";
$resn_exch{" C4* A1R"}=" C4' A1R"; $resn_exch{" C5* A1R"}=" C5' A1R"; $resn_exch{" O5* A1R"}=" O5' A1R";
$resn_exch{"P2*A A22"}=" P2D A22"; $resn_exch{"O1PA A22"}=" O11 A22"; $resn_exch{"O2PA A22"}=" O12 A22";
$resn_exch{"O3PA A22"}=" O13 A22"; $resn_exch{"O5PA A22"}=" O14 A22"; $resn_exch{"O6PA A22"}=" O15 A22";
$resn_exch{"O5*A A22"}=" O5D A22"; $resn_exch{"C5*A A22"}=" C5D A22"; $resn_exch{"C4*A A22"}=" C4D A22";
$resn_exch{"O4*A A22"}=" O4D A22"; $resn_exch{"C3*A A22"}=" C3D A22"; $resn_exch{"O3*A A22"}=" O3D A22";
$resn_exch{"C2*A A22"}=" C2D A22"; $resn_exch{"O2*A A22"}=" O2D A22"; $resn_exch{"C1*A A22"}=" C1D A22";
$resn_exch{"O1PB A22"}=" O16 A22"; $resn_exch{"O2PB A22"}=" O17 A22"; $resn_exch{"O3PB A22"}=" O18 A22";
$resn_exch{"O5*B A22"}=" O5E A22"; $resn_exch{"C5*B A22"}=" C5E A22"; $resn_exch{"C4*B A22"}=" C4E A22";
$resn_exch{"O4*B A22"}=" O4E A22"; $resn_exch{"C3*B A22"}=" C3E A22"; $resn_exch{"O3*B A22"}=" O3E A22";
$resn_exch{"C2*B A22"}=" C2E A22"; $resn_exch{"O2*B A22"}=" O2E A22"; $resn_exch{"C1*B A22"}=" C1E A22";
$resn_exch{" O1P A23"}=" OP1 A23"; $resn_exch{" O2P A23"}=" OP2 A23"; $resn_exch{" O3P A23"}=" OP3 A23";
$resn_exch{" O5* A23"}=" O5' A23"; $resn_exch{" C5* A23"}=" C5' A23"; $resn_exch{" C4* A23"}=" C4' A23";
$resn_exch{" O4* A23"}=" O4' A23"; $resn_exch{" C3* A23"}=" C3' A23"; $resn_exch{" O3* A23"}=" O3' A23";
$resn_exch{" C2* A23"}=" C2' A23"; $resn_exch{" O2* A23"}=" O2' A23"; $resn_exch{" C1* A23"}=" C1' A23";
$resn_exch{" O1* A24"}=" O1B A24"; $resn_exch{" C2* A24"}=" C2B A24"; $resn_exch{" C3* A24"}=" C3B A24";
$resn_exch{" N4* A24"}=" N4' A24"; $resn_exch{" C5* A24"}=" C5B A24"; $resn_exch{" C6* A24"}=" C6B A24";
$resn_exch{" C7* A24"}=" C7B A24"; $resn_exch{" C8* A24"}=" C8' A24"; $resn_exch{"O5*A A2D"}=" O5D A2D";
$resn_exch{"C5*A A2D"}=" C5D A2D"; $resn_exch{"C4*A A2D"}=" C4D A2D"; $resn_exch{"O4*A A2D"}=" O4D A2D";
$resn_exch{"C3*A A2D"}=" C3D A2D"; $resn_exch{"O3*A A2D"}=" O3D A2D"; $resn_exch{"C2*A A2D"}=" C2D A2D";
$resn_exch{"O2*A A2D"}=" O2D A2D"; $resn_exch{"C1*A A2D"}=" C1D A2D"; $resn_exch{"O5*B A2D"}=" O5E A2D";
$resn_exch{"C5*B A2D"}=" C5E A2D"; $resn_exch{"C4*B A2D"}=" C4E A2D"; $resn_exch{"O4*B A2D"}=" O4E A2D";
$resn_exch{"C3*B A2D"}=" C3E A2D"; $resn_exch{"O3*B A2D"}=" O3E A2D"; $resn_exch{"C2*B A2D"}=" C2E A2D";
$resn_exch{"O2*B A2D"}=" O2E A2D"; $resn_exch{"C1*B A2D"}=" C1E A2D"; $resn_exch{" O2* A2L"}=" O2' A2L";
$resn_exch{" C2* A2L"}=" C2' A2L"; $resn_exch{" C1* A2L"}=" C1' A2L"; $resn_exch{" O4* A2L"}=" O4' A2L";
$resn_exch{" C4* A2L"}=" C4' A2L"; $resn_exch{" C5* A2L"}=" C5' A2L"; $resn_exch{" O5* A2L"}=" O5' A2L";
$resn_exch{" O1P A2L"}=" OP1 A2L"; $resn_exch{" O3P A2L"}=" OP3 A2L"; $resn_exch{" O2P A2L"}=" OP2 A2L";
$resn_exch{" C3* A2L"}=" C3' A2L"; $resn_exch{" O3* A2L"}=" O3' A2L"; $resn_exch{" CA* A2L"}=" CA' A2L";
$resn_exch{" CB* A2L"}=" CB' A2L"; $resn_exch{" OC* A2L"}=" OC' A2L"; $resn_exch{" CD* A2L"}=" CD' A2L";
$resn_exch{" O3P A2M"}=" OP3 A2M"; $resn_exch{" O1P A2M"}=" OP1 A2M"; $resn_exch{" O2P A2M"}=" OP2 A2M";
$resn_exch{" O5* A2M"}=" O5' A2M"; $resn_exch{" C5* A2M"}=" C5' A2M"; $resn_exch{" C4* A2M"}=" C4' A2M";
$resn_exch{" O4* A2M"}=" O4' A2M"; $resn_exch{" C3* A2M"}=" C3' A2M"; $resn_exch{" O3* A2M"}=" O3' A2M";
$resn_exch{" C2* A2M"}=" C2' A2M"; $resn_exch{" O2* A2M"}=" O2' A2M"; $resn_exch{" C1* A2M"}=" C1' A2M";
$resn_exch{" CM* A2M"}=" CM' A2M"; $resn_exch{" O5* A2P"}=" O5' A2P"; $resn_exch{" C5* A2P"}=" C5' A2P";
$resn_exch{" C4* A2P"}=" C4' A2P"; $resn_exch{" O4* A2P"}=" O4' A2P"; $resn_exch{" C3* A2P"}=" C3' A2P";
$resn_exch{" O3* A2P"}=" O3' A2P"; $resn_exch{" C2* A2P"}=" C2' A2P"; $resn_exch{" O2* A2P"}=" O2' A2P";
$resn_exch{" C1* A2P"}=" C1' A2P"; $resn_exch{" P2* A2R"}=" P2' A2R"; $resn_exch{" O2* A2R"}=" O2' A2R";
$resn_exch{" C2* A2R"}=" C2' A2R"; $resn_exch{" C1* A2R"}=" C1' A2R"; $resn_exch{" O4* A2R"}=" O4' A2R";
$resn_exch{" C3* A2R"}=" C3' A2R"; $resn_exch{" O3* A2R"}=" O3' A2R"; $resn_exch{" C4* A2R"}=" C4' A2R";
$resn_exch{" C5* A2R"}=" C5' A2R"; $resn_exch{" O5* A2R"}=" O5' A2R"; $resn_exch{"O15* A2R"}="O15' A2R";
$resn_exch{"C15* A2R"}="C15' A2R"; $resn_exch{"C14* A2R"}="C14' A2R"; $resn_exch{"C13* A2R"}="C13' A2R";
$resn_exch{"O13* A2R"}="O13' A2R"; $resn_exch{"C12* A2R"}="C12' A2R"; $resn_exch{"O12* A2R"}="O12' A2R";
$resn_exch{"C11* A2R"}="C11' A2R"; $resn_exch{"O14* A2R"}="O14' A2R"; $resn_exch{"O11* A2R"}="O11' A2R";
$resn_exch{" O1* A32"}=" O1B A32"; $resn_exch{" C2* A32"}=" C2B A32"; $resn_exch{" C3* A32"}=" C3B A32";
$resn_exch{" N4* A32"}=" N4' A32"; $resn_exch{" C5* A32"}=" C5B A32"; $resn_exch{" C6* A32"}=" C6B A32";
$resn_exch{" C7* A32"}=" C7B A32"; $resn_exch{" C8* A32"}=" C8' A32"; $resn_exch{" C9* A32"}=" C9' A32";
$resn_exch{" O5* A34"}=" O5' A34"; $resn_exch{" C2* A34"}=" C2' A34"; $resn_exch{" C5* A34"}=" C5' A34";
$resn_exch{" C4* A34"}=" C4' A34"; $resn_exch{" O4* A34"}=" O4' A34"; $resn_exch{" C1* A34"}=" C1' A34";
$resn_exch{" C3* A34"}=" C3' A34"; $resn_exch{" O3* A34"}=" O3' A34"; $resn_exch{" O5* A35"}=" O5' A35";
$resn_exch{" C5* A35"}=" C5' A35"; $resn_exch{" C4* A35"}=" C4' A35"; $resn_exch{" O4* A35"}=" O4' A35";
$resn_exch{" C1* A35"}=" C1' A35"; $resn_exch{" C2* A35"}=" C2' A35"; $resn_exch{" C3* A35"}=" C3' A35";
$resn_exch{" O3* A35"}=" O3' A35"; $resn_exch{" O5* A38"}=" O5' A38"; $resn_exch{" C5* A38"}=" C5' A38";
$resn_exch{" C4* A38"}=" C4' A38"; $resn_exch{" O4* A38"}=" O4' A38"; $resn_exch{" C3* A38"}=" C3' A38";
$resn_exch{" O3* A38"}=" O3' A38"; $resn_exch{" C2* A38"}=" C2' A38"; $resn_exch{" C1* A38"}=" C1' A38";
$resn_exch{" O5* A39"}=" O5' A39"; $resn_exch{" C5* A39"}=" C5' A39"; $resn_exch{" C4* A39"}=" C4' A39";
$resn_exch{" O4* A39"}=" O4' A39"; $resn_exch{" C3* A39"}=" C3' A39"; $resn_exch{" O3* A39"}=" O3' A39";
$resn_exch{" C2* A39"}=" C2' A39"; $resn_exch{" O2* A39"}=" O2' A39"; $resn_exch{" C1* A39"}=" C1' A39";
$resn_exch{" O1P A3A"}=" OP1 A3A"; $resn_exch{" O2P A3A"}=" OP2 A3A"; $resn_exch{" O6* A3A"}=" O6' A3A";
$resn_exch{" C5* A3A"}=" C5' A3A"; $resn_exch{" C4* A3A"}=" C4' A3A"; $resn_exch{" O5* A3A"}=" O5' A3A";
$resn_exch{" C3* A3A"}=" C3' A3A"; $resn_exch{" O3* A3A"}=" O3' A3A"; $resn_exch{" C2* A3A"}=" C2' A3A";
$resn_exch{" C1* A3A"}=" C1' A3A"; $resn_exch{" AP  A3D"}=" PA  A3D"; $resn_exch{" AO1 A3D"}=" O1A A3D";
$resn_exch{" AO2 A3D"}=" O2A A3D"; $resn_exch{"AO5* A3D"}=" O5B A3D"; $resn_exch{"AC5* A3D"}=" C5B A3D";
$resn_exch{"AC4* A3D"}=" C4B A3D"; $resn_exch{"AO4* A3D"}=" O4B A3D"; $resn_exch{"AC3* A3D"}=" C3B A3D";
$resn_exch{"AO3* A3D"}=" O3B A3D"; $resn_exch{"AC2* A3D"}=" C2B A3D"; $resn_exch{"AO2* A3D"}=" O2B A3D";
$resn_exch{"AC1* A3D"}=" C1B A3D"; $resn_exch{" AN9 A3D"}=" N9A A3D"; $resn_exch{" AC8 A3D"}=" C8A A3D";
$resn_exch{" AN7 A3D"}=" N7A A3D"; $resn_exch{" AC5 A3D"}=" C5A A3D"; $resn_exch{" AC6 A3D"}=" C6A A3D";
$resn_exch{" AN6 A3D"}=" N6A A3D"; $resn_exch{" AN1 A3D"}=" N1A A3D"; $resn_exch{" AC2 A3D"}=" C2A A3D";
$resn_exch{" AN3 A3D"}=" N3A A3D"; $resn_exch{" AC4 A3D"}=" C4A A3D"; $resn_exch{" NP  A3D"}=" PN  A3D";
$resn_exch{" NO1 A3D"}=" O1N A3D"; $resn_exch{" NO2 A3D"}=" O2N A3D"; $resn_exch{"NO5* A3D"}=" O5D A3D";
$resn_exch{"NC5* A3D"}=" C5D A3D"; $resn_exch{"NC4* A3D"}=" C4D A3D"; $resn_exch{"NO4* A3D"}=" O4D A3D";
$resn_exch{"NC3* A3D"}=" C3D A3D"; $resn_exch{"NO3* A3D"}=" O3D A3D"; $resn_exch{"NC2* A3D"}=" C2D A3D";
$resn_exch{"NO2* A3D"}=" O2D A3D"; $resn_exch{"NC1* A3D"}=" C1D A3D"; $resn_exch{" NN1 A3D"}=" N1N A3D";
$resn_exch{" NC2 A3D"}=" C2N A3D"; $resn_exch{" NC3 A3D"}=" C3N A3D"; $resn_exch{" NC7 A3D"}=" C7N A3D";
$resn_exch{" NO7 A3D"}=" O7N A3D"; $resn_exch{" NC8 A3D"}=" C8N A3D"; $resn_exch{" NC4 A3D"}=" C4N A3D";
$resn_exch{" NC5 A3D"}=" C5N A3D"; $resn_exch{" NC6 A3D"}=" C6N A3D"; $resn_exch{" O5* A3P"}=" O5' A3P";
$resn_exch{" C5* A3P"}=" C5' A3P"; $resn_exch{" C4* A3P"}=" C4' A3P"; $resn_exch{" O4* A3P"}=" O4' A3P";
$resn_exch{" C3* A3P"}=" C3' A3P"; $resn_exch{" O3* A3P"}=" O3' A3P"; $resn_exch{" C2* A3P"}=" C2' A3P";
$resn_exch{" O2* A3P"}=" O2' A3P"; $resn_exch{" C1* A3P"}=" C1' A3P"; $resn_exch{" O5* A40"}=" O5' A40";
$resn_exch{" C5* A40"}=" C5' A40"; $resn_exch{" C4* A40"}=" C4' A40"; $resn_exch{" O4* A40"}=" O4' A40";
$resn_exch{" C1* A40"}=" C1' A40"; $resn_exch{" C2* A40"}=" C2' A40"; $resn_exch{" C3* A40"}=" C3' A40";
$resn_exch{" O3* A40"}=" O3' A40"; $resn_exch{" O5* A43"}=" O5' A43"; $resn_exch{" C5* A43"}=" C5' A43";
$resn_exch{" C4* A43"}=" C4' A43"; $resn_exch{" O4* A43"}=" O4' A43"; $resn_exch{" C3* A43"}=" C3' A43";
$resn_exch{" C2* A43"}=" C2' A43"; $resn_exch{" C1* A43"}=" C1' A43"; $resn_exch{" O5* A44"}=" O5' A44";
$resn_exch{" C5* A44"}=" C5' A44"; $resn_exch{" C4* A44"}=" C4' A44"; $resn_exch{" O4* A44"}=" O4' A44";
$resn_exch{" C3* A44"}=" C3' A44"; $resn_exch{" O3* A44"}=" O3' A44"; $resn_exch{" C2* A44"}=" C2' A44";
$resn_exch{" O2* A44"}=" O2' A44"; $resn_exch{" C1* A44"}=" C1' A44"; $resn_exch{" CA* A44"}=" CA' A44";
$resn_exch{" OC* A44"}=" OC' A44"; $resn_exch{" O5* A47"}=" O5' A47"; $resn_exch{" C5* A47"}=" C5' A47";
$resn_exch{" C4* A47"}=" C4' A47"; $resn_exch{" O4* A47"}=" O4' A47"; $resn_exch{" C3* A47"}=" C3' A47";
$resn_exch{" O3* A47"}=" O3' A47"; $resn_exch{" C2* A47"}=" C2' A47"; $resn_exch{" C1* A47"}=" C1' A47";
$resn_exch{" O5* A4P"}=" O5' A4P"; $resn_exch{" C5* A4P"}=" C5' A4P"; $resn_exch{" C4* A4P"}=" C4' A4P";
$resn_exch{" O4* A4P"}=" O4' A4P"; $resn_exch{" C3* A4P"}=" C3' A4P"; $resn_exch{" O3* A4P"}=" O3' A4P";
$resn_exch{" C2* A4P"}=" C2' A4P"; $resn_exch{" O2* A4P"}=" O2' A4P"; $resn_exch{" O5* A5A"}=" O5' A5A";
$resn_exch{" C5* A5A"}=" C5' A5A"; $resn_exch{" C4* A5A"}=" C4' A5A"; $resn_exch{" O4* A5A"}=" O4' A5A";
$resn_exch{" C3* A5A"}=" C3' A5A"; $resn_exch{" O3* A5A"}=" O3' A5A"; $resn_exch{" C2* A5A"}=" C2' A5A";
$resn_exch{" O2* A5A"}=" O2' A5A"; $resn_exch{" C1* A5A"}=" C1' A5A"; $resn_exch{" O1P A5M"}=" OP1 A5M";
$resn_exch{" O2P A5M"}=" OP2 A5M"; $resn_exch{" O5* A5M"}=" O5' A5M"; $resn_exch{" C5* A5M"}=" C5' A5M";
$resn_exch{" C4* A5M"}=" C4' A5M"; $resn_exch{" O4* A5M"}=" O4' A5M"; $resn_exch{" C1* A5M"}=" C1' A5M";
$resn_exch{" C2* A5M"}=" C2' A5M"; $resn_exch{" N2* A5M"}=" N2' A5M"; $resn_exch{" C3* A5M"}=" C3' A5M";
$resn_exch{" O3* A5M"}=" O3' A5M"; $resn_exch{" O3P A5M"}=" OP3 A5M"; $resn_exch{" C4* AAD"}=" C4' AAD";
$resn_exch{" O4* AAD"}=" O4' AAD"; $resn_exch{" C3* AAD"}=" C3' AAD"; $resn_exch{" O3* AAD"}=" O3' AAD";
$resn_exch{" C2* AAD"}=" C2' AAD"; $resn_exch{" O2* AAD"}=" O2' AAD"; $resn_exch{" C1* AAD"}=" C1' AAD";
$resn_exch{" C5* AAD"}=" C5' AAD"; $resn_exch{" O6* AAM"}=" O6' AAM"; $resn_exch{" C5* AAM"}=" C5' AAM";
$resn_exch{" C4* AAM"}=" C4' AAM"; $resn_exch{" O5* AAM"}=" O5' AAM"; $resn_exch{" C3* AAM"}=" C3' AAM";
$resn_exch{" O3* AAM"}=" O3' AAM"; $resn_exch{" C2* AAM"}=" C2' AAM"; $resn_exch{" O2* AAM"}=" O2' AAM";
$resn_exch{" C1* AAM"}=" C1' AAM"; $resn_exch{" C5* AAT"}=" C5' AAT"; $resn_exch{" C4* AAT"}=" C4' AAT";
$resn_exch{" O4* AAT"}=" O4' AAT"; $resn_exch{" C3* AAT"}=" C3' AAT"; $resn_exch{" O3* AAT"}=" O3' AAT";
$resn_exch{" C2* AAT"}=" C2' AAT"; $resn_exch{" O2* AAT"}=" O2' AAT"; $resn_exch{" C1* AAT"}=" C1' AAT";
$resn_exch{"O1P' ABF"}=" O1X ABF"; $resn_exch{"O2P' ABF"}=" O2X ABF"; $resn_exch{"O3P' ABF"}=" O3X ABF";
$resn_exch{" O5* ABM"}=" O5' ABM"; $resn_exch{" C5* ABM"}=" C5' ABM"; $resn_exch{" C4* ABM"}=" C4' ABM";
$resn_exch{" O4* ABM"}=" O4' ABM"; $resn_exch{" C3* ABM"}=" C3' ABM"; $resn_exch{" O3* ABM"}=" O3' ABM";
$resn_exch{" C2* ABM"}=" C2' ABM"; $resn_exch{" O2* ABM"}=" O2' ABM"; $resn_exch{" C1* ABM"}=" C1' ABM";
$resn_exch{" O5* ABP"}=" O5' ABP"; $resn_exch{" C5* ABP"}=" C5' ABP"; $resn_exch{" C4* ABP"}=" C4' ABP";
$resn_exch{" O4* ABP"}=" O4' ABP"; $resn_exch{" C3* ABP"}=" C3' ABP"; $resn_exch{" O3* ABP"}=" O3' ABP";
$resn_exch{" C2* ABP"}=" C2' ABP"; $resn_exch{" O2* ABP"}=" O2' ABP"; $resn_exch{" C1* ABP"}=" C1' ABP";
$resn_exch{" O1P ABR"}=" OP1 ABR"; $resn_exch{" O2P ABR"}=" OP2 ABR"; $resn_exch{" O5* ABR"}=" O5' ABR";
$resn_exch{" C5* ABR"}=" C5' ABR"; $resn_exch{" C4* ABR"}=" C4' ABR"; $resn_exch{" O4* ABR"}=" O4' ABR";
$resn_exch{" C1* ABR"}=" C1' ABR"; $resn_exch{" C2* ABR"}=" C2' ABR"; $resn_exch{" C3* ABR"}=" C3' ABR";
$resn_exch{" O3* ABR"}=" O3' ABR"; $resn_exch{" O3P ABR"}=" OP3 ABR"; $resn_exch{" O1P ABS"}=" OP1 ABS";
$resn_exch{" O2P ABS"}=" OP2 ABS"; $resn_exch{" O5* ABS"}=" O5' ABS"; $resn_exch{" C5* ABS"}=" C5' ABS";
$resn_exch{" C4* ABS"}=" C4' ABS"; $resn_exch{" O4* ABS"}=" O4' ABS"; $resn_exch{" C1* ABS"}=" C1' ABS";
$resn_exch{" C2* ABS"}=" C2' ABS"; $resn_exch{" C3* ABS"}=" C3' ABS"; $resn_exch{" O3* ABS"}=" O3' ABS";
$resn_exch{" O3P ABS"}=" OP3 ABS"; $resn_exch{" O5* ABT"}=" O5' ABT"; $resn_exch{" C5* ABT"}=" C5' ABT";
$resn_exch{" C4* ABT"}=" C4' ABT"; $resn_exch{" O4* ABT"}=" O4' ABT"; $resn_exch{" C1* ABT"}=" C1' ABT";
$resn_exch{" C2* ABT"}=" C2' ABT"; $resn_exch{" C3* ABT"}=" C3' ABT"; $resn_exch{" N3* ABT"}=" N3' ABT";
$resn_exch{" N4* ABT"}=" N4' ABT"; $resn_exch{" N5* ABT"}=" N5' ABT"; $resn_exch{" C3* AC2"}=" C3' AC2";
$resn_exch{" O3* AC2"}=" O3' AC2"; $resn_exch{" C2* AC2"}=" C2' AC2"; $resn_exch{" O1* AC2"}=" O1' AC2";
$resn_exch{" C1* AC2"}=" C1' AC2"; $resn_exch{" O5* ACK"}=" O5' ACK"; $resn_exch{" C5* ACK"}=" C5' ACK";
$resn_exch{" C4* ACK"}=" C4' ACK"; $resn_exch{" O4* ACK"}=" O4' ACK"; $resn_exch{" C3* ACK"}=" C3' ACK";
$resn_exch{" O3* ACK"}=" O3' ACK"; $resn_exch{" C1* ACK"}=" C1' ACK"; $resn_exch{" C2* ACK"}=" C2' ACK";
$resn_exch{" AN1 ACO"}=" N1A ACO"; $resn_exch{" AC2 ACO"}=" C2A ACO"; $resn_exch{" AN3 ACO"}=" N3A ACO";
$resn_exch{" AC4 ACO"}=" C4A ACO"; $resn_exch{" AC5 ACO"}=" C5A ACO"; $resn_exch{" AC6 ACO"}=" C6A ACO";
$resn_exch{" AN6 ACO"}=" N6A ACO"; $resn_exch{" AN7 ACO"}=" N7A ACO"; $resn_exch{" AC8 ACO"}=" C8A ACO";
$resn_exch{" AN9 ACO"}=" N9A ACO"; $resn_exch{"AC1* ACO"}=" C1B ACO"; $resn_exch{"AC2* ACO"}=" C2B ACO";
$resn_exch{"AO2* ACO"}=" O2B ACO"; $resn_exch{"AC3* ACO"}=" C3B ACO"; $resn_exch{"AO3* ACO"}=" O3B ACO";
$resn_exch{"AP3* ACO"}=" P3B ACO"; $resn_exch{" AO7 ACO"}=" O7A ACO"; $resn_exch{" AO8 ACO"}=" O8A ACO";
$resn_exch{" AO9 ACO"}=" O9A ACO"; $resn_exch{"AC4* ACO"}=" C4B ACO"; $resn_exch{"AO4* ACO"}=" O4B ACO";
$resn_exch{"AC5* ACO"}=" C5B ACO"; $resn_exch{"AO5* ACO"}=" O5B ACO"; $resn_exch{" AP1 ACO"}=" P1A ACO";
$resn_exch{" AO1 ACO"}=" O1A ACO"; $resn_exch{" AO2 ACO"}=" O2A ACO"; $resn_exch{" AO3 ACO"}=" O3A ACO";
$resn_exch{" AP2 ACO"}=" P2A ACO"; $resn_exch{" AO4 ACO"}=" O4A ACO"; $resn_exch{" AO5 ACO"}=" O5A ACO";
$resn_exch{" AO6 ACO"}=" O6A ACO"; $resn_exch{"PC11 ACO"}=" CBP ACO"; $resn_exch{"PC12 ACO"}=" CCP ACO";
$resn_exch{"PC13 ACO"}=" CDP ACO"; $resn_exch{"PC14 ACO"}=" CEP ACO"; $resn_exch{"PC10 ACO"}=" CAP ACO";
$resn_exch{"PO10 ACO"}=" OAP ACO"; $resn_exch{" PC9 ACO"}=" C9P ACO"; $resn_exch{" PO9 ACO"}=" O9P ACO";
$resn_exch{" PN8 ACO"}=" N8P ACO"; $resn_exch{" PC7 ACO"}=" C7P ACO"; $resn_exch{" PC6 ACO"}=" C6P ACO";
$resn_exch{" PC5 ACO"}=" C5P ACO"; $resn_exch{" PO5 ACO"}=" O5P ACO"; $resn_exch{" PN4 ACO"}=" N4P ACO";
$resn_exch{" PC3 ACO"}=" C3P ACO"; $resn_exch{" PC2 ACO"}=" C2P ACO"; $resn_exch{" PS1 ACO"}=" S1P ACO";
$resn_exch{" O5* ACP"}=" O5' ACP"; $resn_exch{" C5* ACP"}=" C5' ACP"; $resn_exch{" C4* ACP"}=" C4' ACP";
$resn_exch{" O4* ACP"}=" O4' ACP"; $resn_exch{" C3* ACP"}=" C3' ACP"; $resn_exch{" O3* ACP"}=" O3' ACP";
$resn_exch{" C2* ACP"}=" C2' ACP"; $resn_exch{" O2* ACP"}=" O2' ACP"; $resn_exch{" C1* ACP"}=" C1' ACP";
$resn_exch{" O5* ACQ"}=" O5' ACQ"; $resn_exch{" C5* ACQ"}=" C5' ACQ"; $resn_exch{" C4* ACQ"}=" C4' ACQ";
$resn_exch{" O4* ACQ"}=" O4' ACQ"; $resn_exch{" C3* ACQ"}=" C3' ACQ"; $resn_exch{" O3* ACQ"}=" O3' ACQ";
$resn_exch{" C2* ACQ"}=" C2' ACQ"; $resn_exch{" O2* ACQ"}=" O2' ACQ"; $resn_exch{" C1* ACQ"}=" C1' ACQ";
$resn_exch{" O1P AD2"}=" OP1 AD2"; $resn_exch{" O2P AD2"}=" OP2 AD2"; $resn_exch{" O3P AD2"}=" OP3 AD2";
$resn_exch{" O5* AD3"}=" O5' AD3"; $resn_exch{" C5* AD3"}=" C5' AD3"; $resn_exch{" C4* AD3"}=" C4' AD3";
$resn_exch{" O4* AD3"}=" O4' AD3"; $resn_exch{" C3* AD3"}=" C3' AD3"; $resn_exch{" O3* AD3"}=" O3' AD3";
$resn_exch{" C2* AD3"}=" C2' AD3"; $resn_exch{" O2* AD3"}=" O2' AD3"; $resn_exch{" C1* AD3"}=" C1' AD3";
$resn_exch{" O5* ADI"}=" O5' ADI"; $resn_exch{" C5* ADI"}=" C5' ADI"; $resn_exch{" C4* ADI"}=" C4' ADI";
$resn_exch{" O4* ADI"}=" O4' ADI"; $resn_exch{" C3* ADI"}=" C3' ADI"; $resn_exch{" C2* ADI"}=" C2' ADI";
$resn_exch{" C1* ADI"}=" C1' ADI"; $resn_exch{" O5* ADN"}=" O5' ADN"; $resn_exch{" C5* ADN"}=" C5' ADN";
$resn_exch{" C4* ADN"}=" C4' ADN"; $resn_exch{" O4* ADN"}=" O4' ADN"; $resn_exch{" C3* ADN"}=" C3' ADN";
$resn_exch{" O3* ADN"}=" O3' ADN"; $resn_exch{" C2* ADN"}=" C2' ADN"; $resn_exch{" O2* ADN"}=" O2' ADN";
$resn_exch{" C1* ADN"}=" C1' ADN"; $resn_exch{" O5* ADP"}=" O5' ADP"; $resn_exch{" C5* ADP"}=" C5' ADP";
$resn_exch{" C4* ADP"}=" C4' ADP"; $resn_exch{" O4* ADP"}=" O4' ADP"; $resn_exch{" C3* ADP"}=" C3' ADP";
$resn_exch{" O3* ADP"}=" O3' ADP"; $resn_exch{" C2* ADP"}=" C2' ADP"; $resn_exch{" O2* ADP"}=" O2' ADP";
$resn_exch{" C1* ADP"}=" C1' ADP"; $resn_exch{" O5* ADQ"}=" O5D ADQ"; $resn_exch{" C5* ADQ"}=" C5D ADQ";
$resn_exch{" C4* ADQ"}=" C4D ADQ"; $resn_exch{" O4* ADQ"}=" O4D ADQ"; $resn_exch{" C3* ADQ"}=" C3D ADQ";
$resn_exch{" O3* ADQ"}=" O3D ADQ"; $resn_exch{" C2* ADQ"}=" C2D ADQ"; $resn_exch{" O2* ADQ"}=" O2D ADQ";
$resn_exch{" C1* ADQ"}=" C1D ADQ"; $resn_exch{" O5* ADS"}=" O5' ADS"; $resn_exch{" C5* ADS"}=" C5' ADS";
$resn_exch{" C4* ADS"}=" C4' ADS"; $resn_exch{" O4* ADS"}=" O4' ADS"; $resn_exch{" C3* ADS"}=" C3' ADS";
$resn_exch{" O3* ADS"}=" O3' ADS"; $resn_exch{" C2* ADS"}=" C2' ADS"; $resn_exch{" O2* ADS"}=" O2' ADS";
$resn_exch{" C1* ADS"}=" C1' ADS"; $resn_exch{" C1* ADT"}=" C1' ADT"; $resn_exch{" O4* ADT"}=" O4' ADT";
$resn_exch{" C4* ADT"}=" C4' ADT"; $resn_exch{" C5* ADT"}=" C5' ADT"; $resn_exch{" O5* ADT"}=" O5' ADT";
$resn_exch{" C3* ADT"}=" C3' ADT"; $resn_exch{" N3* ADT"}=" N3' ADT"; $resn_exch{" C8* ADT"}=" C8' ADT";
$resn_exch{" O8* ADT"}=" O8' ADT"; $resn_exch{" C9* ADT"}=" C9' ADT"; $resn_exch{" C2* ADT"}=" C2' ADT";
$resn_exch{" C1* ADU"}=" C1' ADU"; $resn_exch{" O4* ADU"}=" O4' ADU"; $resn_exch{" C4* ADU"}=" C4' ADU";
$resn_exch{" C5* ADU"}=" C5' ADU"; $resn_exch{" O5* ADU"}=" O5' ADU"; $resn_exch{" C3* ADU"}=" C3' ADU";
$resn_exch{" N3* ADU"}=" N3' ADU"; $resn_exch{" C8* ADU"}=" C8' ADU"; $resn_exch{" O8* ADU"}=" O8' ADU";
$resn_exch{" C9* ADU"}=" C9' ADU"; $resn_exch{" C2* ADU"}=" C2' ADU"; $resn_exch{" O2* ADU"}=" O2' ADU";
$resn_exch{" O5* ADV"}=" O5' ADV"; $resn_exch{" C5* ADV"}=" C5' ADV"; $resn_exch{" C4* ADV"}=" C4' ADV";
$resn_exch{" O4* ADV"}=" O4' ADV"; $resn_exch{" C3* ADV"}=" C3' ADV"; $resn_exch{" O3* ADV"}=" O3' ADV";
$resn_exch{" C2* ADV"}=" C2' ADV"; $resn_exch{" O2* ADV"}=" O2' ADV"; $resn_exch{" C1* ADV"}=" C1' ADV";
$resn_exch{" O5* ADW"}=" O5' ADW"; $resn_exch{" C5* ADW"}=" C5' ADW"; $resn_exch{" C4* ADW"}=" C4' ADW";
$resn_exch{" O4* ADW"}=" O4' ADW"; $resn_exch{" C3* ADW"}=" C3' ADW"; $resn_exch{" O3* ADW"}=" O3' ADW";
$resn_exch{" C2* ADW"}=" C2' ADW"; $resn_exch{" O2* ADW"}=" O2' ADW"; $resn_exch{" C1* ADW"}=" C1' ADW";
$resn_exch{" O5* ADX"}=" O5' ADX"; $resn_exch{" C5* ADX"}=" C5' ADX"; $resn_exch{" C4* ADX"}=" C4' ADX";
$resn_exch{" O4* ADX"}=" O4' ADX"; $resn_exch{" C3* ADX"}=" C3' ADX"; $resn_exch{" O3* ADX"}=" O3' ADX";
$resn_exch{" C2* ADX"}=" C2' ADX"; $resn_exch{" O2* ADX"}=" O2' ADX"; $resn_exch{" C1* ADX"}=" C1' ADX";
$resn_exch{" O5* ADY"}=" O5' ADY"; $resn_exch{" C5* ADY"}=" C5' ADY"; $resn_exch{" C4* ADY"}=" C4' ADY";
$resn_exch{" O4* ADY"}=" O4' ADY"; $resn_exch{" C3* ADY"}=" C3' ADY"; $resn_exch{" O3* ADY"}=" O3' ADY";
$resn_exch{" C2* ADY"}=" C2' ADY"; $resn_exch{" O2* ADY"}=" O2' ADY"; $resn_exch{" C1* ADY"}=" C1' ADY";
$resn_exch{" C1* ADZ"}=" C1' ADZ"; $resn_exch{" O1P AET"}=" OP1 AET"; $resn_exch{" O2P AET"}=" OP2 AET";
$resn_exch{" O3P AET"}=" OP3 AET"; $resn_exch{" O5* AET"}=" O5' AET"; $resn_exch{" C5* AET"}=" C5' AET";
$resn_exch{" C4* AET"}=" C4' AET"; $resn_exch{" O4* AET"}=" O4' AET"; $resn_exch{" C3* AET"}=" C3' AET";
$resn_exch{" O3* AET"}=" O3' AET"; $resn_exch{" C2* AET"}=" C2' AET"; $resn_exch{" O2* AET"}=" O2' AET";
$resn_exch{" C1* AET"}=" C1' AET"; $resn_exch{"O13A AET"}=" ODA AET"; $resn_exch{"O13B AET"}=" ODB AET";
$resn_exch{" O3P AFG"}=" OP3 AFG"; $resn_exch{" O1P AFG"}=" OP1 AFG"; $resn_exch{" O2P AFG"}=" OP2 AFG";
$resn_exch{" O5* AFG"}=" O5' AFG"; $resn_exch{" C2* AFG"}=" C2' AFG"; $resn_exch{" C5* AFG"}=" C5' AFG";
$resn_exch{" C4* AFG"}=" C4' AFG"; $resn_exch{" O4* AFG"}=" O4' AFG"; $resn_exch{" C1* AFG"}=" C1' AFG";
$resn_exch{" C3* AFG"}=" C3' AFG"; $resn_exch{" O3* AFG"}=" O3' AFG"; $resn_exch{" C1* AFH"}=" C1D AFH";
$resn_exch{" C2* AFH"}=" C2D AFH"; $resn_exch{" O2* AFH"}=" O2' AFH"; $resn_exch{" C3* AFH"}=" C3D AFH";
$resn_exch{" C4* AFH"}=" C4D AFH"; $resn_exch{" O4* AFH"}=" O4D AFH"; $resn_exch{" O3* AFH"}=" O3D AFH";
$resn_exch{" C5* AFH"}=" C5D AFH"; $resn_exch{" O5* AFH"}=" O5D AFH"; $resn_exch{"1CD1 AGF"}=" CDB AGF";
$resn_exch{"2CD1 AGF"}=" CDL AGF"; $resn_exch{"1CE1 AGF"}=" CEB AGF"; $resn_exch{"2CE1 AGF"}=" CEL AGF";
$resn_exch{"1CD4 AGF"}=" CDE AGF"; $resn_exch{"2CD4 AGF"}=" CDO AGF"; $resn_exch{"1CE4 AGF"}=" CEE AGF";
$resn_exch{"2CE4 AGF"}=" CEO AGF"; $resn_exch{" O5* AGN"}=" O5' AGN"; $resn_exch{" C5* AGN"}=" C5B AGN";
$resn_exch{" C4* AGN"}=" C4B AGN"; $resn_exch{" O4* AGN"}=" O4' AGN"; $resn_exch{" C3* AGN"}=" C3B AGN";
$resn_exch{" O3* AGN"}=" O3' AGN"; $resn_exch{" C2* AGN"}=" C2B AGN"; $resn_exch{" C1* AGN"}=" C1B AGN";
$resn_exch{" O5* AGS"}=" O5' AGS"; $resn_exch{" C5* AGS"}=" C5' AGS"; $resn_exch{" C4* AGS"}=" C4' AGS";
$resn_exch{" O4* AGS"}=" O4' AGS"; $resn_exch{" C3* AGS"}=" C3' AGS"; $resn_exch{" O3* AGS"}=" O3' AGS";
$resn_exch{" C2* AGS"}=" C2' AGS"; $resn_exch{" O2* AGS"}=" O2' AGS"; $resn_exch{" C1* AGS"}=" C1' AGS";
$resn_exch{" O5* AHR"}=" O5' AHR"; $resn_exch{" C5* AHR"}=" C5' AHR"; $resn_exch{" C4* AHR"}=" C4' AHR";
$resn_exch{" O4* AHR"}=" O4' AHR"; $resn_exch{" C3* AHR"}=" C3' AHR"; $resn_exch{" O3* AHR"}=" O3' AHR";
$resn_exch{" C2* AHR"}=" C2' AHR"; $resn_exch{" O2* AHR"}=" O2' AHR"; $resn_exch{" C1* AHR"}=" C1' AHR";
$resn_exch{" O1* AHR"}=" O1' AHR"; $resn_exch{" C4* AHU"}=" C4' AHU"; $resn_exch{" C7* AHU"}=" C7' AHU";
$resn_exch{" O7* AHU"}=" O7' AHU"; $resn_exch{" O5* AHU"}=" O5' AHU"; $resn_exch{" C6* AHU"}=" C6' AHU";
$resn_exch{" C1* AHU"}=" C1' AHU"; $resn_exch{" C2* AHU"}=" C2' AHU"; $resn_exch{" C3* AHU"}=" C3' AHU";
$resn_exch{" O3* AHU"}=" O3' AHU"; $resn_exch{" O5* AHX"}=" O5' AHX"; $resn_exch{" C5* AHX"}=" C5' AHX";
$resn_exch{" C4* AHX"}=" C4' AHX"; $resn_exch{" O4* AHX"}=" O4' AHX"; $resn_exch{" C3* AHX"}=" C3' AHX";
$resn_exch{" O3* AHX"}=" O3' AHX"; $resn_exch{" C2* AHX"}=" C2' AHX"; $resn_exch{" O2* AHX"}=" O2' AHX";
$resn_exch{" C1* AHX"}=" C1' AHX"; $resn_exch{" O2* AHZ"}=" O2' AHZ"; $resn_exch{" C2* AHZ"}=" C2' AHZ";
$resn_exch{" C3* AHZ"}=" C3' AHZ"; $resn_exch{" O3* AHZ"}=" O3' AHZ"; $resn_exch{" C1* AHZ"}=" C1' AHZ";
$resn_exch{" O4* AHZ"}=" O4' AHZ"; $resn_exch{" C4* AHZ"}=" C4' AHZ"; $resn_exch{" C5* AHZ"}=" C5' AHZ";
$resn_exch{" O5* AHZ"}=" O5' AHZ"; $resn_exch{" C1* AI1"}=" C1B AI1"; $resn_exch{" C2* AI1"}=" C2B AI1";
$resn_exch{" C3* AI1"}=" C3B AI1"; $resn_exch{" C4* AI1"}=" C4B AI1"; $resn_exch{" C5* AI1"}=" C5B AI1";
$resn_exch{" C6* AI1"}=" C6B AI1"; $resn_exch{" C7* AI1"}=" C7B AI1"; $resn_exch{" O1  AIB"}=" O   AIB";
$resn_exch{" O2  AIB"}=" OXT AIB"; $resn_exch{" O2* AIS"}=" O2' AIS"; $resn_exch{" C2* AIS"}=" C2' AIS";
$resn_exch{" C3* AIS"}=" C3' AIS"; $resn_exch{" O3* AIS"}=" O3' AIS"; $resn_exch{" C4* AIS"}=" C4' AIS";
$resn_exch{" C5* AIS"}=" C5' AIS"; $resn_exch{" O5* AIS"}=" O5' AIS"; $resn_exch{" O4* AIS"}=" O4' AIS";
$resn_exch{" C1* AIS"}=" C1' AIS"; $resn_exch{"8CL2 AIZ"}="CL28 AIZ"; $resn_exch{" O4* AKA"}=" O4' AKA";
$resn_exch{" C4* AKA"}=" C4' AKA"; $resn_exch{" C5* AKA"}=" C5' AKA"; $resn_exch{" C6* AKA"}=" C6' AKA";
$resn_exch{" O5* AKA"}=" O5' AKA"; $resn_exch{" C3* AKA"}=" C3' AKA"; $resn_exch{" N3* AKA"}=" N3' AKA";
$resn_exch{" C8* AKA"}=" C8' AKA"; $resn_exch{" C7* AKA"}=" C7' AKA"; $resn_exch{" C2* AKA"}=" C2' AKA";
$resn_exch{" C1* AKA"}=" C1' AKA"; $resn_exch{" C1* AKT"}=" C1' AKT"; $resn_exch{" C2* AKT"}=" C2' AKT";
$resn_exch{" C3* AKT"}=" C3' AKT"; $resn_exch{" N3* AKT"}=" N3' AKT"; $resn_exch{" C8* AKT"}=" C8' AKT";
$resn_exch{" C7* AKT"}=" C7' AKT"; $resn_exch{" C4* AKT"}=" C4' AKT"; $resn_exch{" O4* AKT"}=" O4' AKT";
$resn_exch{" C5* AKT"}=" C5' AKT"; $resn_exch{" O5* AKT"}=" O5' AKT"; $resn_exch{" C6* AKT"}=" C6' AKT";
$resn_exch{" C1* ALL"}=" C1' ALL"; $resn_exch{" C2* ALL"}=" C2' ALL"; $resn_exch{" C3* ALL"}=" C3' ALL";
$resn_exch{" C4* ALL"}=" C4' ALL"; $resn_exch{" C5* ALL"}=" C5' ALL"; $resn_exch{" C6* ALL"}=" C6' ALL";
$resn_exch{" O1* ALL"}=" O1' ALL"; $resn_exch{" O2* ALL"}=" O2' ALL"; $resn_exch{" O3* ALL"}=" O3' ALL";
$resn_exch{" O4* ALL"}=" O4' ALL"; $resn_exch{" O5* ALL"}=" O5' ALL"; $resn_exch{" O6* ALL"}=" O6' ALL";
$resn_exch{"3C11 ALW"}=" CX5 ALW"; $resn_exch{" O5* AMD"}=" O5' AMD"; $resn_exch{" C5* AMD"}=" C5' AMD";
$resn_exch{" C4* AMD"}=" C4' AMD"; $resn_exch{" O4* AMD"}=" O4' AMD"; $resn_exch{" C1* AMD"}=" C1' AMD";
$resn_exch{" C2* AMD"}=" C2' AMD"; $resn_exch{" C3* AMD"}=" C3' AMD"; $resn_exch{" O3* AMD"}=" O3' AMD";
$resn_exch{" O5* AMO"}=" O5' AMO"; $resn_exch{" C5* AMO"}=" C5' AMO"; $resn_exch{" C4* AMO"}=" C4' AMO";
$resn_exch{" O4* AMO"}=" O4' AMO"; $resn_exch{" C1* AMO"}=" C1' AMO"; $resn_exch{" C2* AMO"}=" C2' AMO";
$resn_exch{" C3* AMO"}=" C3' AMO"; $resn_exch{" O3* AMO"}=" O3' AMO"; $resn_exch{" O2* AMO"}=" O2' AMO";
$resn_exch{" O5* AMP"}=" O5' AMP"; $resn_exch{" C5* AMP"}=" C5' AMP"; $resn_exch{" C4* AMP"}=" C4' AMP";
$resn_exch{" O4* AMP"}=" O4' AMP"; $resn_exch{" C3* AMP"}=" C3' AMP"; $resn_exch{" O3* AMP"}=" O3' AMP";
$resn_exch{" C2* AMP"}=" C2' AMP"; $resn_exch{" O2* AMP"}=" O2' AMP"; $resn_exch{" C1* AMP"}=" C1' AMP";
$resn_exch{" O5* AMW"}=" O5' AMW"; $resn_exch{" C5* AMW"}=" C5' AMW"; $resn_exch{" C4* AMW"}=" C4' AMW";
$resn_exch{" O4* AMW"}=" O4' AMW"; $resn_exch{" C3* AMW"}=" C3' AMW"; $resn_exch{" O3* AMW"}=" O3' AMW";
$resn_exch{" C2* AMW"}=" C2' AMW"; $resn_exch{" O2* AMW"}=" O2' AMW"; $resn_exch{" C1* AMW"}=" C1' AMW";
$resn_exch{" AN1 AMX"}=" N1A AMX"; $resn_exch{" AC2 AMX"}=" C2A AMX"; $resn_exch{" AN3 AMX"}=" N3A AMX";
$resn_exch{" AC4 AMX"}=" C4A AMX"; $resn_exch{" AC5 AMX"}=" C5A AMX"; $resn_exch{" AC6 AMX"}=" C6A AMX";
$resn_exch{" AN6 AMX"}=" N6A AMX"; $resn_exch{" AN7 AMX"}=" N7A AMX"; $resn_exch{" AC8 AMX"}=" C8A AMX";
$resn_exch{" AN9 AMX"}=" N9A AMX"; $resn_exch{"AC1* AMX"}=" C1B AMX"; $resn_exch{"AC2* AMX"}=" C2B AMX";
$resn_exch{"AO2* AMX"}=" O2B AMX"; $resn_exch{"AC3* AMX"}=" C3B AMX"; $resn_exch{"AO3* AMX"}=" O3B AMX";
$resn_exch{"AP3* AMX"}=" P3B AMX"; $resn_exch{" AO7 AMX"}=" O7A AMX"; $resn_exch{" AO8 AMX"}=" O8A AMX";
$resn_exch{" AO9 AMX"}=" O9A AMX"; $resn_exch{"AC4* AMX"}=" C4B AMX"; $resn_exch{"AO4* AMX"}=" O4B AMX";
$resn_exch{"AC5* AMX"}=" C5B AMX"; $resn_exch{"AO5* AMX"}=" O5B AMX"; $resn_exch{" AP1 AMX"}=" P1A AMX";
$resn_exch{" AO1 AMX"}=" O1A AMX"; $resn_exch{" AO2 AMX"}=" O2A AMX"; $resn_exch{" AO3 AMX"}=" O3A AMX";
$resn_exch{" AP2 AMX"}=" P2A AMX"; $resn_exch{" AO4 AMX"}=" O4A AMX"; $resn_exch{" AO5 AMX"}=" O5A AMX";
$resn_exch{" AO6 AMX"}=" O6A AMX"; $resn_exch{"PC11 AMX"}=" CBP AMX"; $resn_exch{"PC12 AMX"}=" CCP AMX";
$resn_exch{"PC13 AMX"}=" CDP AMX"; $resn_exch{"PC14 AMX"}=" CEP AMX"; $resn_exch{"PC10 AMX"}=" CAP AMX";
$resn_exch{"PO10 AMX"}=" OAP AMX"; $resn_exch{" PC9 AMX"}=" C9P AMX"; $resn_exch{" PO9 AMX"}=" O9P AMX";
$resn_exch{" PN8 AMX"}=" N8P AMX"; $resn_exch{" PC7 AMX"}=" C7P AMX"; $resn_exch{" PC6 AMX"}=" C6P AMX";
$resn_exch{" PC5 AMX"}=" C5P AMX"; $resn_exch{" PO5 AMX"}=" O5P AMX"; $resn_exch{" PN4 AMX"}=" N4P AMX";
$resn_exch{" PC3 AMX"}=" C3P AMX"; $resn_exch{" PC2 AMX"}=" C2P AMX"; $resn_exch{" O5* AN2"}=" O5' AN2";
$resn_exch{" C5* AN2"}=" C5' AN2"; $resn_exch{" C4* AN2"}=" C4' AN2"; $resn_exch{" O4* AN2"}=" O4' AN2";
$resn_exch{" C3* AN2"}=" C3' AN2"; $resn_exch{" O3* AN2"}=" O3' AN2"; $resn_exch{" C2* AN2"}=" C2' AN2";
$resn_exch{" O2* AN2"}=" O2' AN2"; $resn_exch{" C1* AN2"}=" C1' AN2"; $resn_exch{" O5* ANP"}=" O5' ANP";
$resn_exch{" C5* ANP"}=" C5' ANP"; $resn_exch{" C4* ANP"}=" C4' ANP"; $resn_exch{" O4* ANP"}=" O4' ANP";
$resn_exch{" C3* ANP"}=" C3' ANP"; $resn_exch{" O3* ANP"}=" O3' ANP"; $resn_exch{" C2* ANP"}=" C2' ANP";
$resn_exch{" O2* ANP"}=" O2' ANP"; $resn_exch{" C1* ANP"}=" C1' ANP"; $resn_exch{" O3* ANU"}=" O3' ANU";
$resn_exch{" C3* ANU"}=" C3' ANU"; $resn_exch{" C2* ANU"}=" C2' ANU"; $resn_exch{" C1* ANU"}=" C1' ANU";
$resn_exch{" O4* ANU"}=" O4' ANU"; $resn_exch{" C4* ANU"}=" C4' ANU"; $resn_exch{" C5* ANU"}=" C5' ANU";
$resn_exch{" O5* ANU"}=" O5' ANU"; $resn_exch{" O5* ANZ"}=" O5' ANZ"; $resn_exch{" C5* ANZ"}=" C5' ANZ";
$resn_exch{" C4* ANZ"}=" C4' ANZ"; $resn_exch{" O4* ANZ"}=" O4' ANZ"; $resn_exch{" C3* ANZ"}=" C3' ANZ";
$resn_exch{" O3* ANZ"}=" O3' ANZ"; $resn_exch{" C2* ANZ"}=" C2' ANZ"; $resn_exch{" O2* ANZ"}=" O2' ANZ";
$resn_exch{" C1* ANZ"}=" C1' ANZ"; $resn_exch{" NO7 AP0"}=" O7N AP0"; $resn_exch{" NC7 AP0"}=" C7N AP0";
$resn_exch{" NC8 AP0"}=" C8N AP0"; $resn_exch{" NC3 AP0"}=" C3N AP0"; $resn_exch{" NC4 AP0"}=" C4N AP0";
$resn_exch{" NC5 AP0"}=" C5N AP0"; $resn_exch{" NC6 AP0"}=" C6N AP0"; $resn_exch{" NC2 AP0"}=" C2N AP0";
$resn_exch{" NN1 AP0"}=" N1N AP0"; $resn_exch{"NC1* AP0"}=" C1D AP0"; $resn_exch{"NC2* AP0"}=" C2D AP0";
$resn_exch{"NO2* AP0"}=" O2D AP0"; $resn_exch{"NC3* AP0"}=" C3D AP0"; $resn_exch{"NO3* AP0"}=" O3D AP0";
$resn_exch{"NO4* AP0"}=" O4D AP0"; $resn_exch{"NC4* AP0"}=" C4D AP0"; $resn_exch{"NC5* AP0"}=" C5D AP0";
$resn_exch{"NO5* AP0"}=" O5D AP0"; $resn_exch{" NP  AP0"}=" PN  AP0"; $resn_exch{" NO1 AP0"}=" O1N AP0";
$resn_exch{" NO2 AP0"}=" O2N AP0"; $resn_exch{" AP  AP0"}=" PA  AP0"; $resn_exch{" AO1 AP0"}=" O1A AP0";
$resn_exch{" AO2 AP0"}=" O2A AP0"; $resn_exch{"AO5* AP0"}=" O5B AP0"; $resn_exch{"AC5* AP0"}=" C5B AP0";
$resn_exch{"AC4* AP0"}=" C4B AP0"; $resn_exch{"AO4* AP0"}=" O4B AP0"; $resn_exch{"AC1* AP0"}=" C1B AP0";
$resn_exch{"AC2* AP0"}=" C2B AP0"; $resn_exch{"AO2* AP0"}=" O2B AP0"; $resn_exch{"AC3* AP0"}=" C3B AP0";
$resn_exch{"AO3* AP0"}=" O3B AP0"; $resn_exch{" AN9 AP0"}=" N9A AP0"; $resn_exch{" AC8 AP0"}=" C8A AP0";
$resn_exch{" AN7 AP0"}=" N7A AP0"; $resn_exch{" AC5 AP0"}=" C5A AP0"; $resn_exch{" AC6 AP0"}=" C6A AP0";
$resn_exch{" AN6 AP0"}=" N6A AP0"; $resn_exch{" AC4 AP0"}=" C4A AP0"; $resn_exch{" AN3 AP0"}=" N3A AP0";
$resn_exch{" AC2 AP0"}=" C2A AP0"; $resn_exch{" AN1 AP0"}=" N1A AP0"; $resn_exch{" O5* AP2"}=" O5' AP2";
$resn_exch{" C5* AP2"}=" C5' AP2"; $resn_exch{" C4* AP2"}=" C4' AP2"; $resn_exch{" O4* AP2"}=" O4' AP2";
$resn_exch{" C3* AP2"}=" C3' AP2"; $resn_exch{" O3* AP2"}=" O3' AP2"; $resn_exch{" C2* AP2"}=" C2' AP2";
$resn_exch{" O2* AP2"}=" O2' AP2"; $resn_exch{" C1* AP2"}=" C1' AP2"; $resn_exch{"AO5* AP5"}=" O5F AP5";
$resn_exch{"AC5* AP5"}=" C5F AP5"; $resn_exch{"AC4* AP5"}=" C4F AP5"; $resn_exch{"AO4* AP5"}=" O4F AP5";
$resn_exch{"AC3* AP5"}=" C3F AP5"; $resn_exch{"AO3* AP5"}=" O3F AP5"; $resn_exch{"AC2* AP5"}=" C2F AP5";
$resn_exch{"AO2* AP5"}=" O2F AP5"; $resn_exch{"AC1* AP5"}=" C1F AP5"; $resn_exch{" AN9 AP5"}=" N9A AP5";
$resn_exch{" AC8 AP5"}=" C8A AP5"; $resn_exch{" AN7 AP5"}=" N7A AP5"; $resn_exch{" AC5 AP5"}=" C5A AP5";
$resn_exch{" AC6 AP5"}=" C6A AP5"; $resn_exch{" AN6 AP5"}=" N6A AP5"; $resn_exch{" AN1 AP5"}=" N1A AP5";
$resn_exch{" AC2 AP5"}=" C2A AP5"; $resn_exch{" AN3 AP5"}=" N3A AP5"; $resn_exch{" AC4 AP5"}=" C4A AP5";
$resn_exch{"BO5* AP5"}=" O5J AP5"; $resn_exch{"BC5* AP5"}=" C5J AP5"; $resn_exch{"BC4* AP5"}=" C4J AP5";
$resn_exch{"BO4* AP5"}=" O4J AP5"; $resn_exch{"BC3* AP5"}=" C3J AP5"; $resn_exch{"BO3* AP5"}=" O3J AP5";
$resn_exch{"BC2* AP5"}=" C2J AP5"; $resn_exch{"BO2* AP5"}=" O2J AP5"; $resn_exch{"BC1* AP5"}=" C1J AP5";
$resn_exch{" BN9 AP5"}=" N9B AP5"; $resn_exch{" BC8 AP5"}=" C8B AP5"; $resn_exch{" BN7 AP5"}=" N7B AP5";
$resn_exch{" BC5 AP5"}=" C5B AP5"; $resn_exch{" BC6 AP5"}=" C6B AP5"; $resn_exch{" BN6 AP5"}=" N6B AP5";
$resn_exch{" BN1 AP5"}=" N1B AP5"; $resn_exch{" BC2 AP5"}=" C2B AP5"; $resn_exch{" BN3 AP5"}=" N3B AP5";
$resn_exch{" BC4 AP5"}=" C4B AP5"; $resn_exch{" C1* AP7"}=" C1' AP7"; $resn_exch{" C2* AP7"}=" C2' AP7";
$resn_exch{" O2* AP7"}=" O2' AP7"; $resn_exch{" C3* AP7"}=" C3' AP7"; $resn_exch{" O3* AP7"}=" O3' AP7";
$resn_exch{" C4* AP7"}=" C4' AP7"; $resn_exch{" O4* AP7"}=" O4' AP7"; $resn_exch{" C5* AP7"}=" C5' AP7";
$resn_exch{" O5* AP7"}=" O5' AP7"; $resn_exch{" O1P AP7"}=" OP1 AP7"; $resn_exch{" O2P AP7"}=" OP2 AP7";
$resn_exch{" O3P AP7"}=" OP3 AP7"; $resn_exch{" O5* APC"}=" O5' APC"; $resn_exch{" C5* APC"}=" C5' APC";
$resn_exch{" C4* APC"}=" C4' APC"; $resn_exch{" O4* APC"}=" O4' APC"; $resn_exch{" C3* APC"}=" C3' APC";
$resn_exch{" O3* APC"}=" O3' APC"; $resn_exch{" C2* APC"}=" C2' APC"; $resn_exch{" O2* APC"}=" O2' APC";
$resn_exch{" C1* APC"}=" C1' APC"; $resn_exch{" O5* APK"}=" O5' APK"; $resn_exch{" C5* APK"}=" C5' APK";
$resn_exch{" C4* APK"}=" C4' APK"; $resn_exch{" O4* APK"}=" O4' APK"; $resn_exch{" C3* APK"}=" C3' APK";
$resn_exch{" O3* APK"}=" O3' APK"; $resn_exch{" C2* APK"}=" C2' APK"; $resn_exch{" O2* APK"}=" O2' APK";
$resn_exch{" C1* APK"}=" C1' APK"; $resn_exch{" C1* APR"}=" C1' APR"; $resn_exch{" C2* APR"}=" C2' APR";
$resn_exch{" O2* APR"}=" O2' APR"; $resn_exch{" C3* APR"}=" C3' APR"; $resn_exch{" O3* APR"}=" O3' APR";
$resn_exch{" O4* APR"}=" O4' APR"; $resn_exch{" C4* APR"}=" C4' APR"; $resn_exch{" C5* APR"}=" C5' APR";
$resn_exch{" O5* APR"}=" O5' APR"; $resn_exch{"RO5* APR"}=" O5D APR"; $resn_exch{"RC5* APR"}=" C5D APR";
$resn_exch{"RO4* APR"}=" O4D APR"; $resn_exch{"RO1* APR"}=" O1D APR"; $resn_exch{"RC1* APR"}=" C1D APR";
$resn_exch{"RO2* APR"}=" O2D APR"; $resn_exch{"RC2* APR"}=" C2D APR"; $resn_exch{"RO3* APR"}=" O3D APR";
$resn_exch{"RC3* APR"}=" C3D APR"; $resn_exch{"RC4* APR"}=" C4D APR"; $resn_exch{" AP  APU"}=" PA  APU";
$resn_exch{" AO1 APU"}=" O1A APU"; $resn_exch{" AO2 APU"}=" O2A APU"; $resn_exch{"AO5* APU"}=" O5B APU";
$resn_exch{"AC5* APU"}=" C5B APU"; $resn_exch{"AC4* APU"}=" C4B APU"; $resn_exch{"AO4* APU"}=" O4B APU";
$resn_exch{"AC3* APU"}=" C3B APU"; $resn_exch{"AO3* APU"}=" O3B APU"; $resn_exch{"AC2* APU"}=" C2B APU";
$resn_exch{"AO2* APU"}=" O2B APU"; $resn_exch{"AC1* APU"}=" C1B APU"; $resn_exch{" AN9 APU"}=" N9A APU";
$resn_exch{" AC8 APU"}=" C8A APU"; $resn_exch{" AN7 APU"}=" N7A APU"; $resn_exch{" AC5 APU"}=" C5A APU";
$resn_exch{" AC6 APU"}=" C6A APU"; $resn_exch{" AN6 APU"}=" N6A APU"; $resn_exch{" AN1 APU"}=" N1A APU";
$resn_exch{" AC2 APU"}=" C2A APU"; $resn_exch{" AN3 APU"}=" N3A APU"; $resn_exch{" AC4 APU"}=" C4A APU";
$resn_exch{" UN1 APU"}=" N1U APU"; $resn_exch{" UC2 APU"}=" C2U APU"; $resn_exch{" UO2 APU"}=" O2U APU";
$resn_exch{" UN3 APU"}=" N3U APU"; $resn_exch{" UC4 APU"}=" C4U APU"; $resn_exch{" UO4 APU"}=" O4U APU";
$resn_exch{" UC5 APU"}=" C5U APU"; $resn_exch{" UC6 APU"}=" C6U APU"; $resn_exch{"UC1* APU"}=" C1D APU";
$resn_exch{"UC2* APU"}=" C2D APU"; $resn_exch{"UO2* APU"}=" O2D APU"; $resn_exch{"UC3* APU"}=" C3D APU";
$resn_exch{"UO3* APU"}=" O3D APU"; $resn_exch{"UC4* APU"}=" C4D APU"; $resn_exch{"UO4* APU"}=" O4D APU";
$resn_exch{"UO5* APU"}=" O5D APU"; $resn_exch{"UC5* APU"}=" C5D APU"; $resn_exch{" UP  APU"}=" PU  APU";
$resn_exch{"UO1P APU"}=" O1X APU"; $resn_exch{"UO2P APU"}=" O2X APU"; $resn_exch{"UO3P APU"}=" O3X APU";
$resn_exch{" O5* APW"}=" O5' APW"; $resn_exch{" C5* APW"}=" C5' APW"; $resn_exch{" C4* APW"}=" C4' APW";
$resn_exch{" C3* APW"}=" C3' APW"; $resn_exch{" O3* APW"}=" O3' APW"; $resn_exch{" C2* APW"}=" C2' APW";
$resn_exch{" O2* APW"}=" O2' APW"; $resn_exch{" C1* APW"}=" C1' APW"; $resn_exch{" O4* APW"}=" O4' APW";
$resn_exch{" C1* AQP"}=" C1' AQP"; $resn_exch{" O4* AQP"}=" O4' AQP"; $resn_exch{" C4* AQP"}=" C4' AQP";
$resn_exch{" C5* AQP"}=" C5' AQP"; $resn_exch{" O5* AQP"}=" O5' AQP"; $resn_exch{" C3* AQP"}=" C3' AQP";
$resn_exch{" O3* AQP"}=" O3' AQP"; $resn_exch{" C2* AQP"}=" C2' AQP"; $resn_exch{" O2* AQP"}=" O2' AQP";
$resn_exch{" O   AR4"}=" OXT AR4"; $resn_exch{" O5* AR4"}=" O5' AR4"; $resn_exch{" C5* AR4"}=" C5' AR4";
$resn_exch{" C4* AR4"}=" C4' AR4"; $resn_exch{" O4* AR4"}=" O4' AR4"; $resn_exch{" C1* AR4"}=" C1' AR4";
$resn_exch{" C2* AR4"}=" C2' AR4"; $resn_exch{" F2* AR4"}=" F2' AR4"; $resn_exch{" C3* AR4"}=" C3' AR4";
$resn_exch{" O3* AR4"}=" O3' AR4"; $resn_exch{" O1  AR4"}=" O   AR4"; $resn_exch{" O2* AR4"}=" O2' AR4";
$resn_exch{" O1P  AS"}=" OP1  AS"; $resn_exch{" O3P  AS"}=" OP3  AS"; $resn_exch{" O5*  AS"}=" O5'  AS";
$resn_exch{" C5*  AS"}=" C5'  AS"; $resn_exch{" C4*  AS"}=" C4'  AS"; $resn_exch{" O4*  AS"}=" O4'  AS";
$resn_exch{" C3*  AS"}=" C3'  AS"; $resn_exch{" O3*  AS"}=" O3'  AS"; $resn_exch{" C2*  AS"}=" C2'  AS";
$resn_exch{" C1*  AS"}=" C1'  AS"; $resn_exch{" O1P ASU"}=" OP1 ASU"; $resn_exch{" O2P ASU"}=" OP2 ASU";
$resn_exch{" O3P ASU"}=" OP3 ASU"; $resn_exch{" O5* ASU"}=" O5' ASU"; $resn_exch{" S4* ASU"}=" S4' ASU";
$resn_exch{" O1* ASU"}=" O1' ASU"; $resn_exch{" C2* ASU"}=" C2' ASU"; $resn_exch{" C5* ASU"}=" C5' ASU";
$resn_exch{" C4* ASU"}=" C4' ASU"; $resn_exch{" C1* ASU"}=" C1' ASU"; $resn_exch{" C3* ASU"}=" C3' ASU";
$resn_exch{" O3* ASU"}=" O3' ASU"; $resn_exch{" AD1 ASX"}=" XD1 ASX"; $resn_exch{" AD2 ASX"}=" XD2 ASX";
$resn_exch{" O5* AT4"}=" O5' AT4"; $resn_exch{" C5* AT4"}=" C5' AT4"; $resn_exch{" C4* AT4"}=" C4' AT4";
$resn_exch{" O4* AT4"}=" O4' AT4"; $resn_exch{" C3* AT4"}=" C3' AT4"; $resn_exch{" O3* AT4"}=" O3' AT4";
$resn_exch{" C2* AT4"}=" C2' AT4"; $resn_exch{" C1* AT4"}=" C1' AT4"; $resn_exch{" O2* AT4"}=" O2' AT4";
$resn_exch{" N\"  ATA"}=" ND  ATA"; $resn_exch{" CA\" ATA"}=" CAD ATA"; $resn_exch{" C\"  ATA"}=" CD  ATA";
$resn_exch{" O\"  ATA"}=" OD  ATA"; $resn_exch{" OT\" ATA"}=" OT' ATA"; $resn_exch{" O1P ATD"}=" OP1 ATD";
$resn_exch{" O2P ATD"}=" OP2 ATD"; $resn_exch{" O3P ATD"}=" OP3 ATD"; $resn_exch{" O5* ATD"}=" O5' ATD";
$resn_exch{" C5* ATD"}=" C5' ATD"; $resn_exch{" C4* ATD"}=" C4' ATD"; $resn_exch{" O4* ATD"}=" O4' ATD";
$resn_exch{" C3* ATD"}=" C3' ATD"; $resn_exch{" O3* ATD"}=" O3' ATD"; $resn_exch{" C2* ATD"}=" C2' ATD";
$resn_exch{" C1* ATD"}=" C1' ATD"; $resn_exch{" O5* ATF"}=" O5' ATF"; $resn_exch{" C5* ATF"}=" C5' ATF";
$resn_exch{" C4* ATF"}=" C4' ATF"; $resn_exch{" O4* ATF"}=" O4' ATF"; $resn_exch{" C3* ATF"}=" C3' ATF";
$resn_exch{" O3* ATF"}=" O3' ATF"; $resn_exch{" C2* ATF"}=" C2' ATF"; $resn_exch{" O2* ATF"}=" O2' ATF";
$resn_exch{" C1* ATF"}=" C1' ATF"; $resn_exch{" O5* ATG"}=" O5' ATG"; $resn_exch{" C5* ATG"}=" C5' ATG";
$resn_exch{" C4* ATG"}=" C4' ATG"; $resn_exch{" O4* ATG"}=" O4' ATG"; $resn_exch{" C3* ATG"}=" C3' ATG";
$resn_exch{" O3* ATG"}=" O3' ATG"; $resn_exch{" C2* ATG"}=" C2' ATG"; $resn_exch{" O2* ATG"}=" O2' ATG";
$resn_exch{" C1* ATG"}=" C1' ATG"; $resn_exch{" O1P ATL"}=" OP1 ATL"; $resn_exch{" O2P ATL"}=" OP2 ATL";
$resn_exch{" O5* ATL"}=" O5' ATL"; $resn_exch{" C5* ATL"}=" C5' ATL"; $resn_exch{" C4* ATL"}=" C4' ATL";
$resn_exch{" O4* ATL"}=" O4' ATL"; $resn_exch{" C3* ATL"}=" C3' ATL"; $resn_exch{" O3* ATL"}=" O3' ATL";
$resn_exch{" C2* ATL"}=" C2' ATL"; $resn_exch{" O2* ATL"}=" O2' ATL"; $resn_exch{" C1* ATL"}=" C1' ATL";
$resn_exch{" C6* ATL"}=" C6' ATL"; $resn_exch{" O1P ATM"}=" OP1 ATM"; $resn_exch{" O2P ATM"}=" OP2 ATM";
$resn_exch{" O3P ATM"}=" OP3 ATM"; $resn_exch{" O5* ATM"}=" O5' ATM"; $resn_exch{" C5* ATM"}=" C5' ATM";
$resn_exch{" C4* ATM"}=" C4' ATM"; $resn_exch{" O4* ATM"}=" O4' ATM"; $resn_exch{" C3* ATM"}=" C3' ATM";
$resn_exch{" N3* ATM"}=" N3' ATM"; $resn_exch{" N4* ATM"}=" N4' ATM"; $resn_exch{" N5* ATM"}=" N5' ATM";
$resn_exch{" C2* ATM"}=" C2' ATM"; $resn_exch{" C1* ATM"}=" C1' ATM"; $resn_exch{" O5* ATP"}=" O5' ATP";
$resn_exch{" C5* ATP"}=" C5' ATP"; $resn_exch{" C4* ATP"}=" C4' ATP"; $resn_exch{" O4* ATP"}=" O4' ATP";
$resn_exch{" C3* ATP"}=" C3' ATP"; $resn_exch{" O3* ATP"}=" O3' ATP"; $resn_exch{" C2* ATP"}=" C2' ATP";
$resn_exch{" O2* ATP"}=" O2' ATP"; $resn_exch{" C1* ATP"}=" C1' ATP"; $resn_exch{" O5* ATR"}=" O5' ATR";
$resn_exch{" C5* ATR"}=" C5' ATR"; $resn_exch{" C4* ATR"}=" C4' ATR"; $resn_exch{" O4* ATR"}=" O4' ATR";
$resn_exch{" C3* ATR"}=" C3' ATR"; $resn_exch{" O3* ATR"}=" O3' ATR"; $resn_exch{" C2* ATR"}=" C2' ATR";
$resn_exch{" O2* ATR"}=" O2' ATR"; $resn_exch{" P2* ATR"}=" P2' ATR"; $resn_exch{" C1* ATR"}=" C1' ATR";
$resn_exch{" O5* ATY"}=" O5' ATY"; $resn_exch{" C5* ATY"}=" C5' ATY"; $resn_exch{" C4* ATY"}=" C4' ATY";
$resn_exch{" O4* ATY"}=" O4' ATY"; $resn_exch{" C3* ATY"}=" C3' ATY"; $resn_exch{" O3* ATY"}=" O3' ATY";
$resn_exch{" C2* ATY"}=" C2' ATY"; $resn_exch{" C1* ATY"}=" C1' ATY"; $resn_exch{" O5* AV2"}=" O5' AV2";
$resn_exch{" C5* AV2"}=" C5' AV2"; $resn_exch{" C4* AV2"}=" C4' AV2"; $resn_exch{" O4* AV2"}=" O4' AV2";
$resn_exch{" C3* AV2"}=" C3' AV2"; $resn_exch{" O3* AV2"}=" O3' AV2"; $resn_exch{" C2* AV2"}=" C2' AV2";
$resn_exch{" O2* AV2"}=" O2' AV2"; $resn_exch{" C1* AV2"}=" C1' AV2"; $resn_exch{" O3P AVC"}=" OP3 AVC";
$resn_exch{" O1P AVC"}=" OP1 AVC"; $resn_exch{" O2P AVC"}=" OP2 AVC"; $resn_exch{" O5* AVC"}=" O5' AVC";
$resn_exch{" C5* AVC"}=" C5' AVC"; $resn_exch{" C4* AVC"}=" C4' AVC"; $resn_exch{" O4* AVC"}=" O4' AVC";
$resn_exch{" C3* AVC"}=" C3' AVC"; $resn_exch{" O3* AVC"}=" O3' AVC"; $resn_exch{" C2* AVC"}=" C2' AVC";
$resn_exch{" O2* AVC"}=" O2' AVC"; $resn_exch{" C1* AVC"}=" C1' AVC"; $resn_exch{" O5* AZD"}=" O5' AZD";
$resn_exch{" C5* AZD"}=" C5' AZD"; $resn_exch{" C4* AZD"}=" C4' AZD"; $resn_exch{" O4* AZD"}=" O4' AZD";
$resn_exch{" C3* AZD"}=" C3' AZD"; $resn_exch{" N3* AZD"}=" N3' AZD"; $resn_exch{" N4* AZD"}=" N4' AZD";
$resn_exch{" N5* AZD"}=" N5' AZD"; $resn_exch{" C2* AZD"}=" C2' AZD"; $resn_exch{" C1* AZD"}=" C1' AZD";
$resn_exch{" O5* AZT"}=" O5' AZT"; $resn_exch{" C5* AZT"}=" C5' AZT"; $resn_exch{" C4* AZT"}=" C4' AZT";
$resn_exch{" O4* AZT"}=" O4' AZT"; $resn_exch{" C3* AZT"}=" C3' AZT"; $resn_exch{" N3* AZT"}=" N3' AZT";
$resn_exch{" N4* AZT"}=" N4' AZT"; $resn_exch{" N5* AZT"}=" N5' AZT"; $resn_exch{" C2* AZT"}=" C2' AZT";
$resn_exch{" C1* AZT"}=" C1' AZT"; $resn_exch{" O3P B1P"}=" OP3 B1P"; $resn_exch{" O1P B1P"}=" OP1 B1P";
$resn_exch{" O2P B1P"}=" OP2 B1P"; $resn_exch{" O5* B1P"}=" O5' B1P"; $resn_exch{" C5* B1P"}=" C5' B1P";
$resn_exch{" C4* B1P"}=" C4' B1P"; $resn_exch{" O4* B1P"}=" O4' B1P"; $resn_exch{" C1* B1P"}=" C1' B1P";
$resn_exch{" O1* B1P"}=" O1' B1P"; $resn_exch{" C2* B1P"}=" C2' B1P"; $resn_exch{" C3* B1P"}=" C3' B1P";
$resn_exch{" O3* B1P"}=" O3' B1P"; $resn_exch{"AO5* B4P"}=" O5E B4P"; $resn_exch{"AC5* B4P"}=" C5E B4P";
$resn_exch{"AC4* B4P"}=" C4E B4P"; $resn_exch{"AO4* B4P"}=" O4E B4P"; $resn_exch{"AC3* B4P"}=" C3E B4P";
$resn_exch{"AO3* B4P"}=" O3E B4P"; $resn_exch{"AC2* B4P"}=" C2E B4P"; $resn_exch{"AO2* B4P"}=" O2E B4P";
$resn_exch{"AC1* B4P"}=" C1E B4P"; $resn_exch{" AN9 B4P"}=" N9A B4P"; $resn_exch{" AC8 B4P"}=" C8A B4P";
$resn_exch{" AN7 B4P"}=" N7A B4P"; $resn_exch{" AC5 B4P"}=" C5A B4P"; $resn_exch{" AC6 B4P"}=" C6A B4P";
$resn_exch{" AN6 B4P"}=" N6A B4P"; $resn_exch{" AN1 B4P"}=" N1A B4P"; $resn_exch{" AC2 B4P"}=" C2A B4P";
$resn_exch{" AN3 B4P"}=" N3A B4P"; $resn_exch{" AC4 B4P"}=" C4A B4P"; $resn_exch{"BO5* B4P"}=" O5F B4P";
$resn_exch{"BC5* B4P"}=" C5F B4P"; $resn_exch{"BC4* B4P"}=" C4F B4P"; $resn_exch{"BO4* B4P"}=" O4F B4P";
$resn_exch{"BC3* B4P"}=" C3F B4P"; $resn_exch{"BO3* B4P"}=" O3F B4P"; $resn_exch{"BC2* B4P"}=" C2F B4P";
$resn_exch{"BO2* B4P"}=" O2F B4P"; $resn_exch{"BC1* B4P"}=" C1F B4P"; $resn_exch{" BN9 B4P"}=" N9B B4P";
$resn_exch{" BC8 B4P"}=" C8B B4P"; $resn_exch{" BN7 B4P"}=" N7B B4P"; $resn_exch{" BC5 B4P"}=" C5B B4P";
$resn_exch{" BC6 B4P"}=" C6B B4P"; $resn_exch{" BN6 B4P"}=" N6B B4P"; $resn_exch{" BN1 B4P"}=" N1B B4P";
$resn_exch{" BC2 B4P"}=" C2B B4P"; $resn_exch{" BN3 B4P"}=" N3B B4P"; $resn_exch{" BC4 B4P"}=" C4B B4P";
$resn_exch{" AC1 B9A"}=" C1A B9A"; $resn_exch{" AC2 B9A"}=" C2A B9A"; $resn_exch{" AC3 B9A"}=" C3A B9A";
$resn_exch{" AC4 B9A"}=" C4A B9A"; $resn_exch{" AC5 B9A"}=" C5A B9A"; $resn_exch{" AC6 B9A"}=" C6A B9A";
$resn_exch{" AC7 B9A"}=" C7A B9A"; $resn_exch{" AC8 B9A"}=" C8A B9A"; $resn_exch{" AC9 B9A"}=" C9A B9A";
$resn_exch{" AN9 B9A"}=" N9A B9A"; $resn_exch{"AN10 B9A"}=" NAA B9A"; $resn_exch{"AC11 B9A"}=" CBA B9A";
$resn_exch{"AC12 B9A"}=" CCA B9A"; $resn_exch{"AC13 B9A"}=" CDA B9A"; $resn_exch{"AC14 B9A"}=" CEA B9A";
$resn_exch{" BC1 B9A"}=" C1B B9A"; $resn_exch{" BC2 B9A"}=" C2B B9A"; $resn_exch{" BC3 B9A"}=" C3B B9A";
$resn_exch{" BC4 B9A"}=" C4B B9A"; $resn_exch{" BC5 B9A"}=" C5B B9A"; $resn_exch{" BC6 B9A"}=" C6B B9A";
$resn_exch{" BC7 B9A"}=" C7B B9A"; $resn_exch{" BC8 B9A"}=" C8B B9A"; $resn_exch{" BC9 B9A"}=" C9B B9A";
$resn_exch{"BN10 B9A"}=" NAB B9A"; $resn_exch{"BC11 B9A"}=" CBB B9A"; $resn_exch{"BC12 B9A"}=" CCB B9A";
$resn_exch{"BC13 B9A"}=" CDB B9A"; $resn_exch{"BC14 B9A"}=" CEB B9A"; $resn_exch{" BN9 B9A"}=" N9B B9A";
$resn_exch{"2O1A BA1"}=" OAC BA1"; $resn_exch{" C1\" BA1"}=" C1B BA1"; $resn_exch{"2O1B BA1"}=" OBC BA1";
$resn_exch{" C2\" BA1"}=" C2B BA1"; $resn_exch{" C3\" BA1"}=" C3B BA1"; $resn_exch{" C4\" BA1"}=" C4B BA1";
$resn_exch{" O4\" BA1"}=" O4' BA1"; $resn_exch{" C5\" BA1"}=" C5B BA1"; $resn_exch{" C6\" BA1"}=" C6B BA1";
$resn_exch{" O6\" BA1"}=" O6' BA1"; $resn_exch{" C7\" BA1"}=" C7B BA1"; $resn_exch{" C8\" BA1"}=" C8' BA1";
$resn_exch{" O8\" BA1"}=" O8' BA1"; $resn_exch{" C9\" BA1"}=" C9' BA1"; $resn_exch{"2C10 BA1"}=" CXU BA1";
$resn_exch{"2O10 BA1"}=" OXU BA1"; $resn_exch{"2C11 BA1"}=" CX4 BA1"; $resn_exch{"2C12 BA1"}=" CXE BA1";
$resn_exch{"2C13 BA1"}=" CXO BA1"; $resn_exch{"2C14 BA1"}=" CXY BA1"; $resn_exch{"2C15 BA1"}=" CX8 BA1";
$resn_exch{"2O15 BA1"}=" OX8 BA1"; $resn_exch{"2O16 BA1"}=" OXI BA1"; $resn_exch{" C1* BA2"}=" C1' BA2";
$resn_exch{" C2* BA2"}=" C2' BA2"; $resn_exch{" O2* BA2"}=" O2' BA2"; $resn_exch{" C3* BA2"}=" C3' BA2";
$resn_exch{" O3* BA2"}=" O3' BA2"; $resn_exch{" C4* BA2"}=" C4' BA2"; $resn_exch{" O4* BA2"}=" O4' BA2";
$resn_exch{" C5* BA2"}=" C5' BA2"; $resn_exch{"C10A  BC"}=" CAA  BC"; $resn_exch{"C14A  BC"}=" CEA  BC";
$resn_exch{"C14B  BC"}=" CEB  BC"; $resn_exch{"C14C  BC"}=" CEC  BC"; $resn_exch{" AN1 BCA"}=" N1A BCA";
$resn_exch{" AC2 BCA"}=" C2A BCA"; $resn_exch{" AN3 BCA"}=" N3A BCA"; $resn_exch{" AC4 BCA"}=" C4A BCA";
$resn_exch{" AC5 BCA"}=" C5A BCA"; $resn_exch{" AC6 BCA"}=" C6A BCA"; $resn_exch{" AN6 BCA"}=" N6A BCA";
$resn_exch{" AN7 BCA"}=" N7A BCA"; $resn_exch{" AC8 BCA"}=" C8A BCA"; $resn_exch{" AN9 BCA"}=" N9A BCA";
$resn_exch{"AC1* BCA"}=" C1D BCA"; $resn_exch{"AC2* BCA"}=" C2D BCA"; $resn_exch{"AO2* BCA"}=" O2D BCA";
$resn_exch{"AC3* BCA"}=" C3D BCA"; $resn_exch{"AO3* BCA"}=" O3D BCA"; $resn_exch{"AP3* BCA"}=" P3D BCA";
$resn_exch{" AO7 BCA"}=" O7A BCA"; $resn_exch{" AO8 BCA"}=" O8A BCA"; $resn_exch{" AO9 BCA"}=" O9A BCA";
$resn_exch{"AC4* BCA"}=" C4D BCA"; $resn_exch{"AO4* BCA"}=" O4D BCA"; $resn_exch{"AC5* BCA"}=" C5D BCA";
$resn_exch{"AO5* BCA"}=" O5D BCA"; $resn_exch{" AP1 BCA"}=" P1A BCA"; $resn_exch{" AO1 BCA"}=" O1A BCA";
$resn_exch{" AO2 BCA"}=" O2A BCA"; $resn_exch{" AO3 BCA"}=" O3A BCA"; $resn_exch{" AP2 BCA"}=" P2A BCA";
$resn_exch{" AO4 BCA"}=" O4A BCA"; $resn_exch{" AO5 BCA"}=" O5A BCA"; $resn_exch{" AO6 BCA"}=" O6A BCA";
$resn_exch{"PC11 BCA"}=" CBP BCA"; $resn_exch{"PC12 BCA"}=" CCP BCA"; $resn_exch{"PC13 BCA"}=" CDP BCA";
$resn_exch{"PC14 BCA"}=" CEP BCA"; $resn_exch{"PC10 BCA"}=" CAP BCA"; $resn_exch{"PO10 BCA"}=" OAP BCA";
$resn_exch{" PC9 BCA"}=" C9P BCA"; $resn_exch{" PO9 BCA"}=" O9P BCA"; $resn_exch{" PN8 BCA"}=" N8P BCA";
$resn_exch{" PC7 BCA"}=" C7P BCA"; $resn_exch{" PC6 BCA"}=" C6P BCA"; $resn_exch{" PC5 BCA"}=" C5P BCA";
$resn_exch{" PO5 BCA"}=" O5P BCA"; $resn_exch{" PN4 BCA"}=" N4P BCA"; $resn_exch{" PC3 BCA"}=" C3P BCA";
$resn_exch{" PC2 BCA"}=" C2P BCA"; $resn_exch{" PS1 BCA"}=" S1P BCA"; $resn_exch{" BC1 BCA"}=" C1B BCA";
$resn_exch{" BO1 BCA"}=" O1B BCA"; $resn_exch{" BC2 BCA"}=" C2B BCA"; $resn_exch{" BC3 BCA"}=" C3B BCA";
$resn_exch{" BC4 BCA"}=" C4B BCA"; $resn_exch{" BC5 BCA"}=" C5B BCA"; $resn_exch{" BO2 BCA"}=" O2B BCA";
$resn_exch{" BC6 BCA"}=" C6B BCA"; $resn_exch{" BC7 BCA"}=" C7B BCA"; $resn_exch{" N_A BCB"}=" NA  BCB";
$resn_exch{" N_B BCB"}=" NB  BCB"; $resn_exch{" N_C BCB"}=" NC  BCB"; $resn_exch{" N_D BCB"}=" ND  BCB";
$resn_exch{" N_A BCL"}=" NA  BCL"; $resn_exch{" N_B BCL"}=" NB  BCL"; $resn_exch{" N_C BCL"}=" NC  BCL";
$resn_exch{" N_D BCL"}=" ND  BCL"; $resn_exch{" C1* BDA"}=" C1' BDA"; $resn_exch{" C2* BDA"}=" C2' BDA";
$resn_exch{" C3* BDA"}=" C3' BDA"; $resn_exch{" N3* BDA"}=" N3' BDA"; $resn_exch{" C4* BDA"}=" C4' BDA";
$resn_exch{" O4* BDA"}=" O4' BDA"; $resn_exch{" C5* BDA"}=" C5' BDA"; $resn_exch{" O5* BDA"}=" O5' BDA";
$resn_exch{" C6* BDA"}=" C6' BDA"; $resn_exch{" C7* BDA"}=" C7' BDA"; $resn_exch{" C8* BDA"}=" C8' BDA";
$resn_exch{"'C1  BDA"}=" C1B BDA"; $resn_exch{"'C2  BDA"}=" C2B BDA"; $resn_exch{"'C3  BDA"}=" C3B BDA";
$resn_exch{"'C4  BDA"}=" C4B BDA"; $resn_exch{"'O4  BDA"}=" O4B BDA"; $resn_exch{"'C5  BDA"}=" C5B BDA";
$resn_exch{"'C6  BDA"}=" C6B BDA"; $resn_exch{"'O6  BDA"}=" O6' BDA"; $resn_exch{"'C7  BDA"}=" C7B BDA";
$resn_exch{"'C8  BDA"}=" C8B BDA"; $resn_exch{"'O8  BDA"}=" O8' BDA"; $resn_exch{"'C9  BDA"}=" C9' BDA";
$resn_exch{"'C10 BDA"}=" CA' BDA"; $resn_exch{"'O10 BDA"}=" OA' BDA"; $resn_exch{"'C11 BDA"}=" CB' BDA";
$resn_exch{"'C12 BDA"}=" CC' BDA"; $resn_exch{"'O12 BDA"}=" OC' BDA"; $resn_exch{"'C13 BDA"}=" CD' BDA";
$resn_exch{"'O13 BDA"}=" OD' BDA"; $resn_exch{"'C14 BDA"}=" CE' BDA"; $resn_exch{"'C15 BDA"}=" CF' BDA";
$resn_exch{"'C16 BDA"}=" CG' BDA"; $resn_exch{"'C17 BDA"}=" CH' BDA"; $resn_exch{"'O17 BDA"}=" OH' BDA";
$resn_exch{"'C18 BDA"}=" CI' BDA"; $resn_exch{"'C19 BDA"}=" CJ' BDA"; $resn_exch{"'O19 BDA"}=" OJ' BDA";
$resn_exch{"'C20 BDA"}=" CK' BDA"; $resn_exch{"'C21 BDA"}=" CL' BDA"; $resn_exch{"'C1* BDA"}=" C1D BDA";
$resn_exch{"'C2* BDA"}=" C2D BDA"; $resn_exch{"'C3* BDA"}=" C3D BDA"; $resn_exch{"'N3* BDA"}=" N3D BDA";
$resn_exch{"'C4* BDA"}=" C4D BDA"; $resn_exch{"'O4* BDA"}=" O4D BDA"; $resn_exch{"'C5* BDA"}=" C5D BDA";
$resn_exch{"'O5* BDA"}=" O5D BDA"; $resn_exch{"'C6* BDA"}=" C6D BDA"; $resn_exch{"'C7* BDA"}=" C7D BDA";
$resn_exch{"'C8* BDA"}=" C8D BDA"; $resn_exch{"'C9A BDA"}=" C1A BDA"; $resn_exch{"'C9B BDA"}=" C1C BDA";
$resn_exch{"'CE4 BDC"}=" CF4 BDC"; $resn_exch{"'CE3 BDC"}=" CF3 BDC"; $resn_exch{"'CE2 BDC"}=" CF2 BDC";
$resn_exch{"'CE1 BDC"}=" CF1 BDC"; $resn_exch{"'N9  BDC"}=" N9' BDC"; $resn_exch{"'C5  BDC"}=" C5' BDC";
$resn_exch{"'C6  BDC"}=" C6' BDC"; $resn_exch{"'C7  BDC"}=" C7' BDC"; $resn_exch{"'C8  BDC"}=" C8' BDC";
$resn_exch{"'C9  BDC"}=" C9' BDC"; $resn_exch{"'C1  BDC"}=" C1' BDC"; $resn_exch{"'C2  BDC"}=" C2' BDC";
$resn_exch{"'C3  BDC"}=" C3' BDC"; $resn_exch{"'C4  BDC"}=" C4' BDC"; $resn_exch{"'C14 BDC"}=" CE' BDC";
$resn_exch{"'C13 BDC"}=" CD' BDC"; $resn_exch{"'C11 BDC"}=" CB' BDC"; $resn_exch{"'C12 BDC"}=" CC' BDC";
$resn_exch{"'N10 BDC"}=" NA' BDC"; $resn_exch{"'CD1 BDC"}=" CB1 BDC"; $resn_exch{"'OD1 BDC"}=" O1X BDC";
$resn_exch{"'ND1 BDC"}=" NB1 BDC"; $resn_exch{"'CD2 BDC"}=" CB2 BDC"; $resn_exch{"'CD3 BDC"}=" CB3 BDC";
$resn_exch{"'ND2 BDC"}=" NB2 BDC"; $resn_exch{"'CD7 BDC"}=" CB7 BDC"; $resn_exch{"'CD8 BDC"}=" CB8 BDC";
$resn_exch{"1O21 BDM"}=" OXV BDM"; $resn_exch{"2O21 BDM"}=" OXW BDM"; $resn_exch{"1O22 BDM"}=" OX5 BDM";
$resn_exch{"2O22 BDM"}=" OX6 BDM"; $resn_exch{" O1P BGM"}=" OP1 BGM"; $resn_exch{" O2P BGM"}=" OP2 BGM";
$resn_exch{" O5* BGM"}=" O5' BGM"; $resn_exch{" C5* BGM"}=" C5' BGM"; $resn_exch{" C4* BGM"}=" C4' BGM";
$resn_exch{" O4* BGM"}=" O4' BGM"; $resn_exch{" C1* BGM"}=" C1' BGM"; $resn_exch{" C2* BGM"}=" C2' BGM";
$resn_exch{" C3* BGM"}=" C3' BGM"; $resn_exch{" O3* BGM"}=" O3' BGM"; $resn_exch{" O3P BGM"}=" OP3 BGM";
$resn_exch{" C2* BGS"}=" C2' BGS"; $resn_exch{" O2* BGS"}=" O2' BGS"; $resn_exch{" O1* BGS"}=" O1' BGS";
$resn_exch{" O5* BIS"}=" O5' BIS"; $resn_exch{" C5* BIS"}=" C5' BIS"; $resn_exch{" C4* BIS"}=" C4' BIS";
$resn_exch{" O4* BIS"}=" O4' BIS"; $resn_exch{" C3* BIS"}=" C3' BIS"; $resn_exch{" O3* BIS"}=" O3' BIS";
$resn_exch{" C2* BIS"}=" C2' BIS"; $resn_exch{" O2* BIS"}=" O2' BIS"; $resn_exch{" C1* BIS"}=" C1' BIS";
$resn_exch{" *N1 BIZ"}=" N1' BIZ"; $resn_exch{" *C2 BIZ"}=" C2' BIZ"; $resn_exch{" *C3 BIZ"}=" C3' BIZ";
$resn_exch{" *C4 BIZ"}=" C4' BIZ"; $resn_exch{" *C5 BIZ"}=" C5' BIZ"; $resn_exch{" *C6 BIZ"}=" C6' BIZ";
$resn_exch{" *C7 BIZ"}=" C7' BIZ"; $resn_exch{" *N8 BIZ"}=" N8' BIZ"; $resn_exch{" *C9 BIZ"}=" C9' BIZ";
$resn_exch{"*C10 BIZ"}=" CA' BIZ"; $resn_exch{"*C11 BIZ"}=" CB' BIZ"; $resn_exch{"*O12 BIZ"}=" OC' BIZ";
$resn_exch{"*N13 BIZ"}=" ND' BIZ"; $resn_exch{"*C14 BIZ"}=" CE' BIZ"; $resn_exch{"*C15 BIZ"}=" CF' BIZ";
$resn_exch{"*C16 BIZ"}=" CG' BIZ"; $resn_exch{"*C17 BIZ"}=" CH' BIZ"; $resn_exch{"*C18 BIZ"}=" CI' BIZ";
$resn_exch{"*C19 BIZ"}=" CJ' BIZ"; $resn_exch{"*C20 BIZ"}=" CK' BIZ"; $resn_exch{"*O21 BIZ"}=" OL' BIZ";
$resn_exch{"*C22 BIZ"}=" CM' BIZ"; $resn_exch{"*C23 BIZ"}=" CN' BIZ"; $resn_exch{"*C24 BIZ"}=" CO' BIZ";
$resn_exch{"*C25 BIZ"}=" CP' BIZ"; $resn_exch{"*C26 BIZ"}=" CQ' BIZ"; $resn_exch{"*N27 BIZ"}=" NR' BIZ";
$resn_exch{" N_A BLA"}=" NA  BLA"; $resn_exch{" N_B BLA"}=" NB  BLA"; $resn_exch{" O_B BLA"}=" OB  BLA";
$resn_exch{" N_C BLA"}=" NC  BLA"; $resn_exch{" O_C BLA"}=" OC  BLA"; $resn_exch{" N_D BLA"}=" ND  BLA";
$resn_exch{" C6* BLI"}=" C6B BLI"; $resn_exch{" C1* BLI"}=" C1B BLI"; $resn_exch{" C2* BLI"}=" C2B BLI";
$resn_exch{" C3* BLI"}=" C3B BLI"; $resn_exch{" C4* BLI"}=" C4B BLI"; $resn_exch{" C5* BLI"}=" C5B BLI";
$resn_exch{" C1* BLS"}=" C1' BLS"; $resn_exch{" C2* BLS"}=" C2' BLS"; $resn_exch{" C3* BLS"}=" C3' BLS";
$resn_exch{" C4* BLS"}=" C4' BLS"; $resn_exch{" C5* BLS"}=" C5' BLS"; $resn_exch{" O5* BLS"}=" O5' BLS";
$resn_exch{" C6* BLS"}=" C6' BLS"; $resn_exch{" N_A BLV"}=" NA  BLV"; $resn_exch{" O_A BLV"}=" OA  BLV";
$resn_exch{" N_B BLV"}=" NB  BLV"; $resn_exch{" N_C BLV"}=" NC  BLV"; $resn_exch{" N_D BLV"}=" ND  BLV";
$resn_exch{" O_D BLV"}=" OD  BLV"; $resn_exch{" O1P BMP"}=" OP1 BMP"; $resn_exch{" O2P BMP"}=" OP2 BMP";
$resn_exch{" O3P BMP"}=" OP3 BMP"; $resn_exch{" O5* BMP"}=" O5' BMP"; $resn_exch{" C5* BMP"}=" C5' BMP";
$resn_exch{" C4* BMP"}=" C4' BMP"; $resn_exch{" O4* BMP"}=" O4' BMP"; $resn_exch{" C3* BMP"}=" C3' BMP";
$resn_exch{" O3* BMP"}=" O3' BMP"; $resn_exch{" C2* BMP"}=" C2' BMP"; $resn_exch{" O2* BMP"}=" O2' BMP";
$resn_exch{" C1* BMP"}=" C1' BMP"; $resn_exch{" O5* BMQ"}=" O5' BMQ"; $resn_exch{" C5* BMQ"}=" C5' BMQ";
$resn_exch{" C4* BMQ"}=" C4' BMQ"; $resn_exch{" O4* BMQ"}=" O4' BMQ"; $resn_exch{" C3* BMQ"}=" C3' BMQ";
$resn_exch{" O3* BMQ"}=" O3' BMQ"; $resn_exch{" C2* BMQ"}=" C2' BMQ"; $resn_exch{" O2* BMQ"}=" O2' BMQ";
$resn_exch{" C1* BMQ"}=" C1' BMQ"; $resn_exch{" O1\" BMY"}=" O1' BMY"; $resn_exch{" C2\" BMY"}=" C2B BMY";
$resn_exch{" C3\" BMY"}=" C3B BMY"; $resn_exch{" C4\" BMY"}=" C4B BMY"; $resn_exch{" C5\" BMY"}=" C5B BMY";
$resn_exch{" C6\" BMY"}=" C6' BMY"; $resn_exch{" C7\" BMY"}=" C7B BMY"; $resn_exch{" O8\" BMY"}=" O8' BMY";
$resn_exch{" O9\" BMY"}=" O9B BMY"; $resn_exch{"AC4A BNA"}=" C11 BNA"; $resn_exch{"AC8A BNA"}=" C12 BNA";
$resn_exch{"AC4B BNA"}=" C13 BNA"; $resn_exch{"AC8B BNA"}=" C14 BNA"; $resn_exch{" C1* BNR"}=" C1' BNR";
$resn_exch{" C2* BNR"}=" C2' BNR"; $resn_exch{" C3* BNR"}=" C3' BNR"; $resn_exch{" O3* BNR"}=" O3' BNR";
$resn_exch{" C4* BNR"}=" C4' BNR"; $resn_exch{" N4* BNR"}=" N4' BNR"; $resn_exch{" C5* BNR"}=" C5' BNR";
$resn_exch{" O5* BNR"}=" O5' BNR"; $resn_exch{" C6* BNR"}=" C6' BNR"; $resn_exch{" C7* BNR"}=" C7' BNR";
$resn_exch{" C8* BNR"}=" C8' BNR"; $resn_exch{"'C1  BNR"}=" C1B BNR"; $resn_exch{"'C2  BNR"}=" C2B BNR";
$resn_exch{"'C3  BNR"}=" C3B BNR"; $resn_exch{"'C4  BNR"}=" C4B BNR"; $resn_exch{"'O4  BNR"}=" O4' BNR";
$resn_exch{"'C5  BNR"}=" C5B BNR"; $resn_exch{"'C6  BNR"}=" C6B BNR"; $resn_exch{"'O6  BNR"}=" O6' BNR";
$resn_exch{"'C7  BNR"}=" C7B BNR"; $resn_exch{"'C8  BNR"}=" C8B BNR"; $resn_exch{"'O8  BNR"}=" O8' BNR";
$resn_exch{"'C9  BNR"}=" C9' BNR"; $resn_exch{"'C10 BNR"}=" CA' BNR"; $resn_exch{"'O10 BNR"}=" OA' BNR";
$resn_exch{"'C11 BNR"}=" CB' BNR"; $resn_exch{"'C12 BNR"}=" CC' BNR"; $resn_exch{"'O12 BNR"}=" OC' BNR";
$resn_exch{"'C13 BNR"}=" CD' BNR"; $resn_exch{"'O13 BNR"}=" OD' BNR"; $resn_exch{"'C14 BNR"}=" CE' BNR";
$resn_exch{"'C15 BNR"}=" CF' BNR"; $resn_exch{"'C16 BNR"}=" CG' BNR"; $resn_exch{"'C17 BNR"}=" CH' BNR";
$resn_exch{"'O17 BNR"}=" OH' BNR"; $resn_exch{"'C18 BNR"}=" CI' BNR"; $resn_exch{"'C19 BNR"}=" CJ' BNR";
$resn_exch{"'O19 BNR"}=" OJ' BNR"; $resn_exch{"'C20 BNR"}=" CK' BNR"; $resn_exch{"'C21 BNR"}=" CL' BNR";
$resn_exch{"'C1* BNR"}=" C1D BNR"; $resn_exch{"'C2* BNR"}=" C2D BNR"; $resn_exch{"'C3* BNR"}=" C3D BNR";
$resn_exch{"'O3* BNR"}=" O3D BNR"; $resn_exch{"'C4* BNR"}=" C4D BNR"; $resn_exch{"'N4* BNR"}=" N4D BNR";
$resn_exch{"'C5* BNR"}=" C5D BNR"; $resn_exch{"'O5* BNR"}=" O5D BNR"; $resn_exch{"'C6* BNR"}=" C6D BNR";
$resn_exch{"'C7* BNR"}=" C7D BNR"; $resn_exch{"'C8* BNR"}=" C8D BNR"; $resn_exch{"'C9A BNR"}=" C1A BNR";
$resn_exch{"'C9B BNR"}=" C1C BNR"; $resn_exch{" O1P BOE"}=" OP1 BOE"; $resn_exch{" O2P BOE"}=" OP2 BOE";
$resn_exch{" O5* BOE"}=" O5' BOE"; $resn_exch{" C5* BOE"}=" C5' BOE"; $resn_exch{" C4* BOE"}=" C4' BOE";
$resn_exch{" O4* BOE"}=" O4' BOE"; $resn_exch{" C1* BOE"}=" C1' BOE"; $resn_exch{" C2* BOE"}=" C2' BOE";
$resn_exch{" O2* BOE"}=" O2' BOE"; $resn_exch{" C3* BOE"}=" C3' BOE"; $resn_exch{" O3* BOE"}=" O3' BOE";
$resn_exch{" O3P BOE"}=" OP3 BOE"; $resn_exch{" N_A BPB"}=" NA  BPB"; $resn_exch{" N_B BPB"}=" NB  BPB";
$resn_exch{" N_C BPB"}=" NC  BPB"; $resn_exch{" N_D BPB"}=" ND  BPB"; $resn_exch{" C3* BPG"}=" C3' BPG";
$resn_exch{" O3* BPG"}=" O3' BPG"; $resn_exch{" C2* BPG"}=" C2' BPG"; $resn_exch{" C1* BPG"}=" C1' BPG";
$resn_exch{" N_A BPH"}=" NA  BPH"; $resn_exch{" N_B BPH"}=" NB  BPH"; $resn_exch{" N_C BPH"}=" NC  BPH";
$resn_exch{" N_D BPH"}=" ND  BPH"; $resn_exch{"CX10 BPS"}=" CXA BPS"; $resn_exch{"CX11 BPS"}=" CXB BPS";
$resn_exch{"CX12 BPS"}=" CXC BPS"; $resn_exch{"CK10 BPY"}=" CKA BPY"; $resn_exch{"CK11 BPY"}=" CKB BPY";
$resn_exch{"CK12 BPY"}=" CKC BPY"; $resn_exch{" C1* BRU"}=" C1' BRU"; $resn_exch{" C2* BRU"}=" C2' BRU";
$resn_exch{" C3* BRU"}=" C3' BRU"; $resn_exch{" C4* BRU"}=" C4' BRU"; $resn_exch{" O3* BRU"}=" O3' BRU";
$resn_exch{" O4* BRU"}=" O4' BRU"; $resn_exch{" C5* BRU"}=" C5' BRU"; $resn_exch{" O5* BRU"}=" O5' BRU";
$resn_exch{" O1P BRU"}=" OP1 BRU"; $resn_exch{" O2P BRU"}=" OP2 BRU"; $resn_exch{" O3P BRU"}=" OP3 BRU";
$resn_exch{" C1* BSP"}=" C1B BSP"; $resn_exch{" C2* BSP"}=" C2B BSP"; $resn_exch{" C3* BSP"}=" C3B BSP";
$resn_exch{" C4* BSP"}=" C4B BSP"; $resn_exch{" C5* BSP"}=" C5B BSP"; $resn_exch{" C6* BSP"}=" C6B BSP";
$resn_exch{" O4* BSP"}=" O4B BSP"; $resn_exch{" S1* BSP"}=" S1B BSP"; $resn_exch{"C11B BT5"}=" CBB BT5";
$resn_exch{"O11B BT5"}=" OBB BT5"; $resn_exch{"O12B BT5"}=" OCB BT5"; $resn_exch{"C10B BT5"}=" CAB BT5";
$resn_exch{" O1P BT5"}=" OP1 BT5"; $resn_exch{" O2P BT5"}=" OP2 BT5"; $resn_exch{" O5* BT5"}=" O5' BT5";
$resn_exch{" C5* BT5"}=" C5' BT5"; $resn_exch{" C4* BT5"}=" C4' BT5"; $resn_exch{" O4* BT5"}=" O4' BT5";
$resn_exch{" C3* BT5"}=" C3' BT5"; $resn_exch{" O3* BT5"}=" O3' BT5"; $resn_exch{" C2* BT5"}=" C2' BT5";
$resn_exch{" O2* BT5"}=" O2' BT5"; $resn_exch{" C1* BT5"}=" C1' BT5"; $resn_exch{" O5* BTD"}=" O5' BTD";
$resn_exch{" C5* BTD"}=" C5' BTD"; $resn_exch{" C4* BTD"}=" C4' BTD"; $resn_exch{" O4* BTD"}=" O4' BTD";
$resn_exch{" C1* BTD"}=" C1' BTD"; $resn_exch{" C2* BTD"}=" C2' BTD"; $resn_exch{" C3* BTD"}=" C3' BTD";
$resn_exch{" O3* BTD"}=" O3' BTD"; $resn_exch{" C*  BTP"}=" C'  BTP"; $resn_exch{" S*  BTP"}=" S'  BTP";
$resn_exch{" O5* BTX"}=" O5' BTX"; $resn_exch{" C5* BTX"}=" C5' BTX"; $resn_exch{" C4* BTX"}=" C4' BTX";
$resn_exch{" O4* BTX"}=" O4' BTX"; $resn_exch{" C3* BTX"}=" C3' BTX"; $resn_exch{" O3* BTX"}=" O3' BTX";
$resn_exch{" C2* BTX"}=" C2' BTX"; $resn_exch{" O2* BTX"}=" O2' BTX"; $resn_exch{" C1* BTX"}=" C1' BTX";
$resn_exch{"C11B BTX"}=" CBB BTX"; $resn_exch{"O11B BTX"}=" OBB BTX"; $resn_exch{"C10B BTX"}=" CAB BTX";
$resn_exch{" O5* BVD"}=" O5' BVD"; $resn_exch{" C5* BVD"}=" C5' BVD"; $resn_exch{" C4* BVD"}=" C4' BVD";
$resn_exch{" O4* BVD"}=" O4' BVD"; $resn_exch{" C1* BVD"}=" C1' BVD"; $resn_exch{" C2* BVD"}=" C2' BVD";
$resn_exch{" C3* BVD"}=" C3' BVD"; $resn_exch{" O3* BVD"}=" O3' BVD"; $resn_exch{" O3* BVP"}=" O3' BVP";
$resn_exch{" C3* BVP"}=" C3' BVP"; $resn_exch{" C2* BVP"}=" C2' BVP"; $resn_exch{" C1* BVP"}=" C1' BVP";
$resn_exch{" O4* BVP"}=" O4' BVP"; $resn_exch{" C4* BVP"}=" C4' BVP"; $resn_exch{" C5* BVP"}=" C5' BVP";
$resn_exch{" O5* BVP"}=" O5' BVP"; $resn_exch{" O5* BZG"}=" O5' BZG"; $resn_exch{" CT* BZG"}=" CT' BZG";
$resn_exch{" OS* BZG"}=" OS' BZG"; $resn_exch{" CP* BZG"}=" CP' BZG"; $resn_exch{" C5* BZG"}=" C5' BZG";
$resn_exch{" C4* BZG"}=" C4' BZG"; $resn_exch{" C3* BZG"}=" C3' BZG"; $resn_exch{" O3* BZG"}=" O3' BZG";
$resn_exch{" O3P   C"}=" OP3   C"; $resn_exch{" O1P   C"}=" OP1   C"; $resn_exch{" O2P   C"}=" OP2   C";
$resn_exch{" O5*   C"}=" O5'   C"; $resn_exch{" C5*   C"}=" C5'   C"; $resn_exch{" C4*   C"}=" C4'   C";
$resn_exch{" O4*   C"}=" O4'   C"; $resn_exch{" C3*   C"}=" C3'   C"; $resn_exch{" O3*   C"}=" O3'   C";
$resn_exch{" C2*   C"}=" C2'   C"; $resn_exch{" O2*   C"}=" O2'   C"; $resn_exch{" C1*   C"}=" C1'   C";
$resn_exch{"C31' C1B"}=" CVX C1B"; $resn_exch{"C32' C1B"}=" CWX C1B"; $resn_exch{"O33' C1B"}=" OXX C1B";
$resn_exch{"O34' C1B"}=" OYX C1B"; $resn_exch{"C42' C1B"}=" C6X C1B"; $resn_exch{"C43' C1B"}=" C7X C1B";
$resn_exch{"O44' C1B"}=" O8X C1B"; $resn_exch{"O45' C1B"}=" O9X C1B"; $resn_exch{"C1'' C1B"}=" C1B C1B";
$resn_exch{"C2'' C1B"}=" C2B C1B"; $resn_exch{"C3'' C1B"}=" C3B C1B"; $resn_exch{"C4'' C1B"}=" C4B C1B";
$resn_exch{"C5'' C1B"}=" C5B C1B"; $resn_exch{"C6'' C1B"}=" C6B C1B"; $resn_exch{"O2'' C1B"}=" O2B C1B";
$resn_exch{" O3P C25"}=" OP3 C25"; $resn_exch{" O1P C25"}=" OP1 C25"; $resn_exch{" O2P C25"}=" OP2 C25";
$resn_exch{" O5* C25"}=" O5' C25"; $resn_exch{" C5* C25"}=" C5' C25"; $resn_exch{" C4* C25"}=" C4' C25";
$resn_exch{" O4* C25"}=" O4' C25"; $resn_exch{" C3* C25"}=" C3' C25"; $resn_exch{" O3* C25"}=" O3' C25";
$resn_exch{" C2* C25"}=" C2' C25"; $resn_exch{" O2* C25"}=" O2' C25"; $resn_exch{" C1* C25"}=" C1' C25";
$resn_exch{" C1* C2G"}=" C1' C2G"; $resn_exch{" C2* C2G"}=" C2' C2G"; $resn_exch{" O2* C2G"}=" O2' C2G";
$resn_exch{" C3* C2G"}=" C3' C2G"; $resn_exch{" C4* C2G"}=" C4' C2G"; $resn_exch{" O4* C2G"}=" O4' C2G";
$resn_exch{" O3* C2G"}=" O3' C2G"; $resn_exch{" C5* C2G"}=" C5' C2G"; $resn_exch{" O5* C2G"}=" O5' C2G";
$resn_exch{" O3P C2L"}=" OP3 C2L"; $resn_exch{" O1P C2L"}=" OP1 C2L"; $resn_exch{" O2P C2L"}=" OP2 C2L";
$resn_exch{" O5* C2L"}=" O5' C2L"; $resn_exch{" C5* C2L"}=" C5' C2L"; $resn_exch{" C4* C2L"}=" C4' C2L";
$resn_exch{" O4* C2L"}=" O4' C2L"; $resn_exch{" C3* C2L"}=" C3' C2L"; $resn_exch{" O3* C2L"}=" O3' C2L";
$resn_exch{" C2* C2L"}=" C2' C2L"; $resn_exch{" O2* C2L"}=" O2' C2L"; $resn_exch{" C1* C2L"}=" C1' C2L";
$resn_exch{" CA* C2L"}=" CA' C2L"; $resn_exch{" CB* C2L"}=" CB' C2L"; $resn_exch{" OC* C2L"}=" OC' C2L";
$resn_exch{" CD* C2L"}=" CD' C2L"; $resn_exch{" C1* C2P"}=" C1' C2P"; $resn_exch{" C2* C2P"}=" C2' C2P";
$resn_exch{" O2* C2P"}=" O2' C2P"; $resn_exch{" C3* C2P"}=" C3' C2P"; $resn_exch{" C4* C2P"}=" C4' C2P";
$resn_exch{" O4* C2P"}=" O4' C2P"; $resn_exch{" O3* C2P"}=" O3' C2P"; $resn_exch{" C5* C2P"}=" C5' C2P";
$resn_exch{" O5* C2P"}=" O5' C2P"; $resn_exch{" O5* C2R"}=" O5' C2R"; $resn_exch{" C2* C2R"}=" C2' C2R";
$resn_exch{" O2* C2R"}=" O2' C2R"; $resn_exch{" C3* C2R"}=" C3' C2R"; $resn_exch{" O3* C2R"}=" O3' C2R";
$resn_exch{" C5* C2R"}=" C5' C2R"; $resn_exch{" C4* C2R"}=" C4' C2R"; $resn_exch{" O4* C2R"}=" O4' C2R";
$resn_exch{" C1* C2R"}=" C1' C2R"; $resn_exch{" O3P C2S"}=" OP3 C2S"; $resn_exch{" O5* C2S"}=" O5' C2S";
$resn_exch{" C5* C2S"}=" C5' C2S"; $resn_exch{" C4* C2S"}=" C4' C2S"; $resn_exch{" O4* C2S"}=" O4' C2S";
$resn_exch{" C3* C2S"}=" C3' C2S"; $resn_exch{" O3* C2S"}=" O3' C2S"; $resn_exch{" C2* C2S"}=" C2' C2S";
$resn_exch{" C1* C2S"}=" C1' C2S"; $resn_exch{" O5* C31"}=" O5' C31"; $resn_exch{" C5* C31"}=" C5' C31";
$resn_exch{" C4* C31"}=" C4' C31"; $resn_exch{" O4* C31"}=" O4' C31"; $resn_exch{" C3* C31"}=" C3' C31";
$resn_exch{" O3* C31"}=" O3' C31"; $resn_exch{" C2* C31"}=" C2' C31"; $resn_exch{" O2* C31"}=" O2' C31";
$resn_exch{" C1* C31"}=" C1' C31"; $resn_exch{" CA* C31"}=" CA' C31"; $resn_exch{" CB* C31"}=" CB' C31";
$resn_exch{" ND* C31"}=" ND' C31"; $resn_exch{" O5* C32"}=" O5' C32"; $resn_exch{" C5* C32"}=" C5' C32";
$resn_exch{" C4* C32"}=" C4' C32"; $resn_exch{" O4* C32"}=" O4' C32"; $resn_exch{" C3* C32"}=" C3' C32";
$resn_exch{" O3* C32"}=" O3' C32"; $resn_exch{" C2* C32"}=" C2' C32"; $resn_exch{" C1* C32"}=" C1' C32";
$resn_exch{" O1P C34"}=" OP1 C34"; $resn_exch{" O2P C34"}=" OP2 C34"; $resn_exch{" O3P C34"}=" OP3 C34";
$resn_exch{" O5* C34"}=" O5' C34"; $resn_exch{" C5* C34"}=" C5' C34"; $resn_exch{" C4* C34"}=" C4' C34";
$resn_exch{" O4* C34"}=" O4' C34"; $resn_exch{" C1* C34"}=" C1' C34"; $resn_exch{" C2* C34"}=" C2' C34";
$resn_exch{" C3* C34"}=" C3' C34"; $resn_exch{" O3* C34"}=" O3' C34"; $resn_exch{" O5* C36"}=" O5' C36";
$resn_exch{" C5* C36"}=" C5' C36"; $resn_exch{" C4* C36"}=" C4' C36"; $resn_exch{" O4* C36"}=" O4' C36";
$resn_exch{" C3* C36"}=" C3' C36"; $resn_exch{" O3* C36"}=" O3' C36"; $resn_exch{" C2* C36"}=" C2' C36";
$resn_exch{" C1* C36"}=" C1' C36"; $resn_exch{" O5* C37"}=" O5' C37"; $resn_exch{" C5* C37"}=" C5' C37";
$resn_exch{" C4* C37"}=" C4' C37"; $resn_exch{" O4* C37"}=" O4' C37"; $resn_exch{" C3* C37"}=" C3' C37";
$resn_exch{" O3* C37"}=" O3' C37"; $resn_exch{" C2* C37"}=" C2' C37"; $resn_exch{" C1* C37"}=" C1' C37";
$resn_exch{" O5* C38"}=" O5' C38"; $resn_exch{" C5* C38"}=" C5' C38"; $resn_exch{" C4* C38"}=" C4' C38";
$resn_exch{" O4* C38"}=" O4' C38"; $resn_exch{" C3* C38"}=" C3' C38"; $resn_exch{" O3* C38"}=" O3' C38";
$resn_exch{" C2* C38"}=" C2' C38"; $resn_exch{" C1* C38"}=" C1' C38"; $resn_exch{" O5* C3P"}=" O5' C3P";
$resn_exch{" C5* C3P"}=" C5' C3P"; $resn_exch{" C4* C3P"}=" C4' C3P"; $resn_exch{" O4* C3P"}=" O4' C3P";
$resn_exch{" C3* C3P"}=" C3' C3P"; $resn_exch{" O3* C3P"}=" O3' C3P"; $resn_exch{" C2* C3P"}=" C2' C3P";
$resn_exch{" O2* C3P"}=" O2' C3P"; $resn_exch{" C1* C3P"}=" C1' C3P"; $resn_exch{" C1* C42"}=" C1' C42";
$resn_exch{" C2* C42"}=" C2' C42"; $resn_exch{" C3* C42"}=" C3' C42"; $resn_exch{" C4* C42"}=" C4' C42";
$resn_exch{" O4* C42"}=" O4' C42"; $resn_exch{" C5* C42"}=" C5' C42"; $resn_exch{" O5* C42"}=" O5' C42";
$resn_exch{" O5* C43"}=" O5' C43"; $resn_exch{" C5* C43"}=" C5' C43"; $resn_exch{" C4* C43"}=" C4' C43";
$resn_exch{" O4* C43"}=" O4' C43"; $resn_exch{" C3* C43"}=" C3' C43"; $resn_exch{" O3* C43"}=" O3' C43";
$resn_exch{" C2* C43"}=" C2' C43"; $resn_exch{" O2* C43"}=" O2' C43"; $resn_exch{" C1* C43"}=" C1' C43";
$resn_exch{" CA* C43"}=" CA' C43"; $resn_exch{" CB* C43"}=" CB' C43"; $resn_exch{" CD* C43"}=" CD' C43";
$resn_exch{" OC* C43"}=" OC' C43"; $resn_exch{" C1* C45"}=" C1' C45"; $resn_exch{" C2* C45"}=" C2' C45";
$resn_exch{" C3* C45"}=" C3' C45"; $resn_exch{" C4* C45"}=" C4' C45"; $resn_exch{" O4* C45"}=" O4' C45";
$resn_exch{" O3* C45"}=" O3' C45"; $resn_exch{" C5* C45"}=" C5' C45"; $resn_exch{" O5* C45"}=" O5' C45";
$resn_exch{" O5* C46"}=" O5' C46"; $resn_exch{" C5* C46"}=" C5' C46"; $resn_exch{" C4* C46"}=" C4' C46";
$resn_exch{" O4* C46"}=" O4' C46"; $resn_exch{" C3* C46"}=" C3' C46"; $resn_exch{" O3* C46"}=" O3' C46";
$resn_exch{" C2* C46"}=" C2' C46"; $resn_exch{" C1* C46"}=" C1' C46"; $resn_exch{" O5* C49"}=" O5' C49";
$resn_exch{" C5* C49"}=" C5' C49"; $resn_exch{" C4* C49"}=" C4' C49"; $resn_exch{" O4* C49"}=" O4' C49";
$resn_exch{" C3* C49"}=" C3' C49"; $resn_exch{" O3* C49"}=" O3' C49"; $resn_exch{" C2* C49"}=" C2' C49";
$resn_exch{" C1* C49"}=" C1' C49"; $resn_exch{" C1* C5G"}=" C1D C5G"; $resn_exch{" C2* C5G"}=" C2D C5G";
$resn_exch{" O2* C5G"}=" O2D C5G"; $resn_exch{" C3* C5G"}=" C3D C5G"; $resn_exch{" C4* C5G"}=" C4D C5G";
$resn_exch{" O4* C5G"}=" O4D C5G"; $resn_exch{" O3* C5G"}=" O3D C5G"; $resn_exch{" C5* C5G"}=" C5D C5G";
$resn_exch{" O5* C5G"}=" O5D C5G"; $resn_exch{" O5* C5P"}=" O5' C5P"; $resn_exch{" C5* C5P"}=" C5' C5P";
$resn_exch{" C4* C5P"}=" C4' C5P"; $resn_exch{" O4* C5P"}=" O4' C5P"; $resn_exch{" C3* C5P"}=" C3' C5P";
$resn_exch{" O3* C5P"}=" O3' C5P"; $resn_exch{" C2* C5P"}=" C2' C5P"; $resn_exch{" O2* C5P"}=" O2' C5P";
$resn_exch{" C1* C5P"}=" C1' C5P"; $resn_exch{"0CT1 CA3"}=" C11 CA3"; $resn_exch{"0CT1 CA5"}=" C11 CA5";
$resn_exch{" AN1 CAA"}=" N1A CAA"; $resn_exch{" AC2 CAA"}=" C2A CAA"; $resn_exch{" AN3 CAA"}=" N3A CAA";
$resn_exch{" AC4 CAA"}=" C4A CAA"; $resn_exch{" AC5 CAA"}=" C5A CAA"; $resn_exch{" AC6 CAA"}=" C6A CAA";
$resn_exch{" AN6 CAA"}=" N6A CAA"; $resn_exch{" AN7 CAA"}=" N7A CAA"; $resn_exch{" AC8 CAA"}=" C8A CAA";
$resn_exch{" AN9 CAA"}=" N9A CAA"; $resn_exch{"AC1* CAA"}=" C1B CAA"; $resn_exch{"AC2* CAA"}=" C2B CAA";
$resn_exch{"AO2* CAA"}=" O2B CAA"; $resn_exch{"AC3* CAA"}=" C3B CAA"; $resn_exch{"AO3* CAA"}=" O3B CAA";
$resn_exch{"AP3* CAA"}=" P3B CAA"; $resn_exch{" AO7 CAA"}=" O7A CAA"; $resn_exch{" AO8 CAA"}=" O8A CAA";
$resn_exch{" AO9 CAA"}=" O9A CAA"; $resn_exch{"AC4* CAA"}=" C4B CAA"; $resn_exch{"AO4* CAA"}=" O4B CAA";
$resn_exch{"AC5* CAA"}=" C5B CAA"; $resn_exch{"AO5* CAA"}=" O5B CAA"; $resn_exch{" AP1 CAA"}=" P1A CAA";
$resn_exch{" AO1 CAA"}=" O1A CAA"; $resn_exch{" AO2 CAA"}=" O2A CAA"; $resn_exch{" AO3 CAA"}=" O3A CAA";
$resn_exch{" AP2 CAA"}=" P2A CAA"; $resn_exch{" AO4 CAA"}=" O4A CAA"; $resn_exch{" AO5 CAA"}=" O5A CAA";
$resn_exch{" AO6 CAA"}=" O6A CAA"; $resn_exch{"PC11 CAA"}=" CBP CAA"; $resn_exch{"PC12 CAA"}=" CCP CAA";
$resn_exch{"PC13 CAA"}=" CDP CAA"; $resn_exch{"PC14 CAA"}=" CEP CAA"; $resn_exch{"PC10 CAA"}=" CAP CAA";
$resn_exch{"PO10 CAA"}=" OAP CAA"; $resn_exch{" PC9 CAA"}=" C9P CAA"; $resn_exch{" PO9 CAA"}=" O9P CAA";
$resn_exch{" PN8 CAA"}=" N8P CAA"; $resn_exch{" PC7 CAA"}=" C7P CAA"; $resn_exch{" PC6 CAA"}=" C6P CAA";
$resn_exch{" PC5 CAA"}=" C5P CAA"; $resn_exch{" PO5 CAA"}=" O5P CAA"; $resn_exch{" PN4 CAA"}=" N4P CAA";
$resn_exch{" PC3 CAA"}=" C3P CAA"; $resn_exch{" PC2 CAA"}=" C2P CAA"; $resn_exch{" PS1 CAA"}=" S1P CAA";
$resn_exch{" O5* CAG"}=" O5' CAG"; $resn_exch{" C5* CAG"}=" C5B CAG"; $resn_exch{" C4* CAG"}=" C4B CAG";
$resn_exch{" O4* CAG"}=" O4' CAG"; $resn_exch{" C3* CAG"}=" C3B CAG"; $resn_exch{" O3* CAG"}=" O3' CAG";
$resn_exch{" C2* CAG"}=" C2B CAG"; $resn_exch{" O2* CAG"}=" O2' CAG"; $resn_exch{" C1* CAG"}=" C1B CAG";
$resn_exch{"1O2' CAG"}=" O'L CAG"; $resn_exch{"2O2' CAG"}=" O'M CAG"; $resn_exch{" AN1 CAO"}=" N1A CAO";
$resn_exch{" AC2 CAO"}=" C2A CAO"; $resn_exch{" AN3 CAO"}=" N3A CAO"; $resn_exch{" AC4 CAO"}=" C4A CAO";
$resn_exch{" AC5 CAO"}=" C5A CAO"; $resn_exch{" AC6 CAO"}=" C6A CAO"; $resn_exch{" AN6 CAO"}=" N6A CAO";
$resn_exch{" AN7 CAO"}=" N7A CAO"; $resn_exch{" AC8 CAO"}=" C8A CAO"; $resn_exch{" AN9 CAO"}=" N9A CAO";
$resn_exch{"AC1* CAO"}=" C1B CAO"; $resn_exch{"AC2* CAO"}=" C2B CAO"; $resn_exch{"AO2* CAO"}=" O2B CAO";
$resn_exch{"AC3* CAO"}=" C3B CAO"; $resn_exch{"AO3* CAO"}=" O3B CAO"; $resn_exch{"AP3* CAO"}=" P3B CAO";
$resn_exch{" AO7 CAO"}=" O7A CAO"; $resn_exch{" AO8 CAO"}=" O8A CAO"; $resn_exch{" AO9 CAO"}=" O9A CAO";
$resn_exch{"AC4* CAO"}=" C4B CAO"; $resn_exch{"AO4* CAO"}=" O4B CAO"; $resn_exch{"AC5* CAO"}=" C5B CAO";
$resn_exch{"AO5* CAO"}=" O5B CAO"; $resn_exch{" AP1 CAO"}=" P1A CAO"; $resn_exch{" AO1 CAO"}=" O1A CAO";
$resn_exch{" AO2 CAO"}=" O2A CAO"; $resn_exch{" AO3 CAO"}=" O3A CAO"; $resn_exch{" AP2 CAO"}=" P2A CAO";
$resn_exch{" AO4 CAO"}=" O4A CAO"; $resn_exch{" AO5 CAO"}=" O5A CAO"; $resn_exch{" AO6 CAO"}=" O6A CAO";
$resn_exch{"PC11 CAO"}=" CBP CAO"; $resn_exch{"PC12 CAO"}=" CCP CAO"; $resn_exch{"PC13 CAO"}=" CDP CAO";
$resn_exch{"PC14 CAO"}=" CEP CAO"; $resn_exch{"PC10 CAO"}=" CAP CAO"; $resn_exch{"PO10 CAO"}=" OAP CAO";
$resn_exch{" PC9 CAO"}=" C9P CAO"; $resn_exch{" PO9 CAO"}=" O9P CAO"; $resn_exch{" PN8 CAO"}=" N8P CAO";
$resn_exch{" PC7 CAO"}=" C7P CAO"; $resn_exch{" PC6 CAO"}=" C6P CAO"; $resn_exch{" PC5 CAO"}=" C5P CAO";
$resn_exch{" PO5 CAO"}=" O5P CAO"; $resn_exch{" PN4 CAO"}=" N4P CAO"; $resn_exch{" PC3 CAO"}=" C3P CAO";
$resn_exch{" PC2 CAO"}=" C2P CAO"; $resn_exch{" PS1 CAO"}=" S1P CAO"; $resn_exch{" PO1 CAO"}=" O1P CAO";
$resn_exch{" O1P CAR"}=" OP1 CAR"; $resn_exch{" O2P CAR"}=" OP2 CAR"; $resn_exch{" O3P CAR"}=" OP3 CAR";
$resn_exch{" O5* CAR"}=" O5' CAR"; $resn_exch{" C5* CAR"}=" C5' CAR"; $resn_exch{" C4* CAR"}=" C4' CAR";
$resn_exch{" O4* CAR"}=" O4' CAR"; $resn_exch{" C3* CAR"}=" C3' CAR"; $resn_exch{" O3* CAR"}=" O3' CAR";
$resn_exch{" C2* CAR"}=" C2' CAR"; $resn_exch{" O2* CAR"}=" O2' CAR"; $resn_exch{" C1* CAR"}=" C1' CAR";
$resn_exch{" CA* CAV"}=" CA' CAV"; $resn_exch{" CB* CAV"}=" CB' CAV"; $resn_exch{" O3* CB2"}=" O3' CB2";
$resn_exch{" C3* CB2"}=" C3' CB2"; $resn_exch{" C2* CB2"}=" C2' CB2"; $resn_exch{" O2* CB2"}=" O2' CB2";
$resn_exch{" C1* CB2"}=" C1' CB2"; $resn_exch{" O4* CB2"}=" O4' CB2"; $resn_exch{" C4* CB2"}=" C4' CB2";
$resn_exch{" C5* CB2"}=" C5' CB2"; $resn_exch{" O5* CB2"}=" O5' CB2"; $resn_exch{" O1P CB2"}=" OP1 CB2";
$resn_exch{" O2P CB2"}=" OP2 CB2"; $resn_exch{" O3P CB2"}=" OP3 CB2"; $resn_exch{" O1P CBR"}=" OP1 CBR";
$resn_exch{" O2P CBR"}=" OP2 CBR"; $resn_exch{" O5* CBR"}=" O5' CBR"; $resn_exch{" C2* CBR"}=" C2' CBR";
$resn_exch{" C5* CBR"}=" C5' CBR"; $resn_exch{" C4* CBR"}=" C4' CBR"; $resn_exch{" O4* CBR"}=" O4' CBR";
$resn_exch{" C1* CBR"}=" C1' CBR"; $resn_exch{" C3* CBR"}=" C3' CBR"; $resn_exch{" O3* CBR"}=" O3' CBR";
$resn_exch{" O3P CBR"}=" OP3 CBR"; $resn_exch{" AC1 CBS"}=" C1A CBS"; $resn_exch{" AC2 CBS"}=" C2A CBS";
$resn_exch{" AC3 CBS"}=" C3A CBS"; $resn_exch{" AC4 CBS"}=" C4A CBS"; $resn_exch{" AC5 CBS"}=" C5A CBS";
$resn_exch{" AC6 CBS"}=" C6A CBS"; $resn_exch{" AC7 CBS"}=" C7A CBS"; $resn_exch{" AC8 CBS"}=" C8A CBS";
$resn_exch{" AN2 CBS"}=" N2A CBS"; $resn_exch{" AO1 CBS"}=" O1A CBS"; $resn_exch{" AO3 CBS"}=" O3A CBS";
$resn_exch{" AO4 CBS"}=" O4A CBS"; $resn_exch{" AO5 CBS"}=" O5A CBS"; $resn_exch{" AO6 CBS"}=" O6A CBS";
$resn_exch{" AO7 CBS"}=" O7A CBS"; $resn_exch{" BC1 CBS"}=" C1B CBS"; $resn_exch{" BC2 CBS"}=" C2B CBS";
$resn_exch{" BC3 CBS"}=" C3B CBS"; $resn_exch{" BC4 CBS"}=" C4B CBS"; $resn_exch{" BC5 CBS"}=" C5B CBS";
$resn_exch{" BC6 CBS"}=" C6B CBS"; $resn_exch{" BC7 CBS"}=" C7B CBS"; $resn_exch{" BC8 CBS"}=" C8B CBS";
$resn_exch{" BN2 CBS"}=" N2B CBS"; $resn_exch{" BO1 CBS"}=" O1B CBS"; $resn_exch{" BO3 CBS"}=" O3B CBS";
$resn_exch{" BO5 CBS"}=" O5B CBS"; $resn_exch{" BO6 CBS"}=" O6B CBS"; $resn_exch{" BO7 CBS"}=" O7B CBS";
$resn_exch{" O5* CBV"}=" O5' CBV"; $resn_exch{" C5* CBV"}=" C5' CBV"; $resn_exch{" C4* CBV"}=" C4' CBV";
$resn_exch{" O4* CBV"}=" O4' CBV"; $resn_exch{" C3* CBV"}=" C3' CBV"; $resn_exch{" O3* CBV"}=" O3' CBV";
$resn_exch{" C2* CBV"}=" C2' CBV"; $resn_exch{" O2* CBV"}=" O2' CBV"; $resn_exch{" C1* CBV"}=" C1' CBV";
$resn_exch{" O1P CCC"}=" OP1 CCC"; $resn_exch{" O2P CCC"}=" OP2 CCC"; $resn_exch{" O3P CCC"}=" OP3 CCC";
$resn_exch{" O5* CCC"}=" O5' CCC"; $resn_exch{" C5* CCC"}=" C5' CCC"; $resn_exch{" C4* CCC"}=" C4' CCC";
$resn_exch{" O4* CCC"}=" O4' CCC"; $resn_exch{" C3* CCC"}=" C3' CCC"; $resn_exch{" O3* CCC"}=" O3' CCC";
$resn_exch{" C2* CCC"}=" C2' CCC"; $resn_exch{" O2* CCC"}=" O2' CCC"; $resn_exch{" C1* CCC"}=" C1' CCC";
$resn_exch{" N_A CCH"}=" NA  CCH"; $resn_exch{" N_B CCH"}=" NB  CCH"; $resn_exch{" N_C CCH"}=" NC  CCH";
$resn_exch{" N_D CCH"}=" ND  CCH"; $resn_exch{" PS1 CCQ"}=" S1P CCQ"; $resn_exch{" PC2 CCQ"}=" C2P CCQ";
$resn_exch{" PC3 CCQ"}=" C3P CCQ"; $resn_exch{" PN4 CCQ"}=" N4P CCQ"; $resn_exch{" PC5 CCQ"}=" C5P CCQ";
$resn_exch{" PO5 CCQ"}=" O5P CCQ"; $resn_exch{" PC6 CCQ"}=" C6P CCQ"; $resn_exch{" PC7 CCQ"}=" C7P CCQ";
$resn_exch{" PN8 CCQ"}=" N8P CCQ"; $resn_exch{" PC9 CCQ"}=" C9P CCQ"; $resn_exch{" PO9 CCQ"}=" O9P CCQ";
$resn_exch{"PC10 CCQ"}=" CAP CCQ"; $resn_exch{"PO10 CCQ"}=" OAP CCQ"; $resn_exch{"PC11 CCQ"}=" CBP CCQ";
$resn_exch{"PC13 CCQ"}=" CDP CCQ"; $resn_exch{"PC14 CCQ"}=" CEP CCQ"; $resn_exch{"PC12 CCQ"}=" CCP CCQ";
$resn_exch{" AO6 CCQ"}=" O6A CCQ"; $resn_exch{" AP2 CCQ"}=" P2A CCQ"; $resn_exch{" AO4 CCQ"}=" O4A CCQ";
$resn_exch{" AO5 CCQ"}=" O5A CCQ"; $resn_exch{" AO3 CCQ"}=" O3A CCQ"; $resn_exch{" AP1 CCQ"}=" P1A CCQ";
$resn_exch{" AO1 CCQ"}=" O1A CCQ"; $resn_exch{" AO2 CCQ"}=" O2A CCQ"; $resn_exch{"AO5* CCQ"}=" O5D CCQ";
$resn_exch{"AC5* CCQ"}=" C5D CCQ"; $resn_exch{"AC4* CCQ"}=" C4D CCQ"; $resn_exch{"AC3* CCQ"}=" C3D CCQ";
$resn_exch{"AO3* CCQ"}=" O3D CCQ"; $resn_exch{"AP3* CCQ"}=" P3D CCQ"; $resn_exch{" AO7 CCQ"}=" O7A CCQ";
$resn_exch{" AO8 CCQ"}=" O8A CCQ"; $resn_exch{" AO9 CCQ"}=" O9A CCQ"; $resn_exch{"AC2* CCQ"}=" C2D CCQ";
$resn_exch{"AO2* CCQ"}=" O2D CCQ"; $resn_exch{"AC1* CCQ"}=" C1D CCQ"; $resn_exch{"AO4* CCQ"}=" O4D CCQ";
$resn_exch{" AN9 CCQ"}=" N9A CCQ"; $resn_exch{" AC4 CCQ"}=" C4A CCQ"; $resn_exch{" AC5 CCQ"}=" C5X CCQ";
$resn_exch{" AN7 CCQ"}=" N7A CCQ"; $resn_exch{" AC8 CCQ"}=" C8A CCQ"; $resn_exch{" AC6 CCQ"}=" C6A CCQ";
$resn_exch{" AN6 CCQ"}=" N6A CCQ"; $resn_exch{" AN1 CCQ"}=" N1A CCQ"; $resn_exch{" AC2 CCQ"}=" C2A CCQ";
$resn_exch{" AN3 CCQ"}=" N3A CCQ"; $resn_exch{" C1* CDC"}=" C1' CDC"; $resn_exch{" C2* CDC"}=" C2' CDC";
$resn_exch{" O2* CDC"}=" O2' CDC"; $resn_exch{" C3* CDC"}=" C3' CDC"; $resn_exch{" C4* CDC"}=" C4' CDC";
$resn_exch{" O4* CDC"}=" O4' CDC"; $resn_exch{" O3* CDC"}=" O3' CDC"; $resn_exch{" C5* CDC"}=" C5' CDC";
$resn_exch{" O5* CDC"}=" O5' CDC"; $resn_exch{" O5* CDF"}=" O5' CDF"; $resn_exch{" C5* CDF"}=" C5' CDF";
$resn_exch{" C4* CDF"}=" C4' CDF"; $resn_exch{" O4* CDF"}=" O4' CDF"; $resn_exch{" C3* CDF"}=" C3' CDF";
$resn_exch{" O3* CDF"}=" O3' CDF"; $resn_exch{" C2* CDF"}=" C2' CDF"; $resn_exch{" O2* CDF"}=" O2' CDF";
$resn_exch{" C1* CDF"}=" C1' CDF"; $resn_exch{" C1* CDM"}=" C1' CDM"; $resn_exch{" C2* CDM"}=" C2' CDM";
$resn_exch{" O2* CDM"}=" O2' CDM"; $resn_exch{" C3* CDM"}=" C3' CDM"; $resn_exch{" C4* CDM"}=" C4' CDM";
$resn_exch{" O4* CDM"}=" O4' CDM"; $resn_exch{" O3* CDM"}=" O3' CDM"; $resn_exch{" C5* CDM"}=" C5' CDM";
$resn_exch{" O5* CDM"}=" O5' CDM"; $resn_exch{" C1* CDP"}=" C1' CDP"; $resn_exch{" C2* CDP"}=" C2' CDP";
$resn_exch{" O2* CDP"}=" O2' CDP"; $resn_exch{" C3* CDP"}=" C3' CDP"; $resn_exch{" C4* CDP"}=" C4' CDP";
$resn_exch{" O4* CDP"}=" O4' CDP"; $resn_exch{" O3* CDP"}=" O3' CDP"; $resn_exch{" C5* CDP"}=" C5' CDP";
$resn_exch{" O5* CDP"}=" O5' CDP"; $resn_exch{" O5* CDY"}=" O5' CDY"; $resn_exch{" C5* CDY"}=" C5' CDY";
$resn_exch{" C4* CDY"}=" C4' CDY"; $resn_exch{" O4* CDY"}=" O4' CDY"; $resn_exch{" C3* CDY"}=" C3' CDY";
$resn_exch{" C2* CDY"}=" C2' CDY"; $resn_exch{" C1* CDY"}=" C1' CDY"; $resn_exch{" O5* CFL"}=" O5' CFL";
$resn_exch{" C5* CFL"}=" C5' CFL"; $resn_exch{" C4* CFL"}=" C4' CFL"; $resn_exch{" O4* CFL"}=" O4' CFL";
$resn_exch{" C3* CFL"}=" C3' CFL"; $resn_exch{" O3* CFL"}=" O3' CFL"; $resn_exch{" C2* CFL"}=" C2' CFL";
$resn_exch{" C1* CFL"}=" C1' CFL"; $resn_exch{" CA* CG2"}=" CA' CG2"; $resn_exch{" CB* CG2"}=" CB' CG2";
$resn_exch{" OB* CG2"}=" OB' CG2"; $resn_exch{" CC* CG2"}=" CC' CG2"; $resn_exch{" OC* CG2"}=" OC' CG2";
$resn_exch{" CD* CG2"}=" CD' CG2"; $resn_exch{" OD* CG2"}=" OD' CG2"; $resn_exch{" CE* CG2"}=" CE' CG2";
$resn_exch{" OE* CG2"}=" OE' CG2"; $resn_exch{" O5* CG2"}=" O5' CG2"; $resn_exch{" C5* CG2"}=" C5' CG2";
$resn_exch{" C4* CG2"}=" C4' CG2"; $resn_exch{" O4* CG2"}=" O4' CG2"; $resn_exch{" C3* CG2"}=" C3' CG2";
$resn_exch{" O3* CG2"}=" O3' CG2"; $resn_exch{" C2* CG2"}=" C2' CG2"; $resn_exch{" O2* CG2"}=" O2' CG2";
$resn_exch{" C1* CG2"}=" C1' CG2"; $resn_exch{"CO5* CGP"}=" O5B CGP"; $resn_exch{"CC5* CGP"}=" C5X CGP";
$resn_exch{"CC4* CGP"}=" C4X CGP"; $resn_exch{"CO4* CGP"}=" O4B CGP"; $resn_exch{"CC3* CGP"}=" C3X CGP";
$resn_exch{"CO3* CGP"}=" O3B CGP"; $resn_exch{"CC2* CGP"}=" C2X CGP"; $resn_exch{"CC1* CGP"}=" C1X CGP";
$resn_exch{" CN1 CGP"}=" N1C CGP"; $resn_exch{" CN3 CGP"}=" N3C CGP"; $resn_exch{" CO2 CGP"}=" O2C CGP";
$resn_exch{" CN4 CGP"}=" N4C CGP"; $resn_exch{"GO5* CGP"}=" O5D CGP"; $resn_exch{"GC5* CGP"}=" C5D CGP";
$resn_exch{"GC4* CGP"}=" C4D CGP"; $resn_exch{"GO4* CGP"}=" O4D CGP"; $resn_exch{"GC3* CGP"}=" C3D CGP";
$resn_exch{"GO3* CGP"}=" O3D CGP"; $resn_exch{"GC2* CGP"}=" C2D CGP"; $resn_exch{"GC1* CGP"}=" C1D CGP";
$resn_exch{" GN9 CGP"}=" N9G CGP"; $resn_exch{" GC8 CGP"}=" C8G CGP"; $resn_exch{" GN7 CGP"}=" N7G CGP";
$resn_exch{" GC5 CGP"}=" C5G CGP"; $resn_exch{" GC6 CGP"}=" C6G CGP"; $resn_exch{" GO6 CGP"}=" O6G CGP";
$resn_exch{" GN1 CGP"}=" N1G CGP"; $resn_exch{" GC2 CGP"}=" C2G CGP"; $resn_exch{" GN2 CGP"}=" N2G CGP";
$resn_exch{" GN3 CGP"}=" N3G CGP"; $resn_exch{" GC4 CGP"}=" C4G CGP"; $resn_exch{"RO5* CGR"}="O5'R CGR";
$resn_exch{"RC5* CGR"}="C5'R CGR"; $resn_exch{"RC4* CGR"}="C4'R CGR"; $resn_exch{"RO4* CGR"}="O4'R CGR";
$resn_exch{"RC1* CGR"}="C1'R CGR"; $resn_exch{"RC2* CGR"}="C2'R CGR"; $resn_exch{"RO2* CGR"}="O2'R CGR";
$resn_exch{"RC3* CGR"}="C3'R CGR"; $resn_exch{"RO3* CGR"}="O3'R CGR"; $resn_exch{" C1* CGR"}=" C1' CGR";
$resn_exch{" O4* CGR"}=" O4' CGR"; $resn_exch{" C2* CGR"}=" C2' CGR"; $resn_exch{" O2* CGR"}=" O2' CGR";
$resn_exch{" C3* CGR"}=" C3' CGR"; $resn_exch{" O3* CGR"}=" O3' CGR"; $resn_exch{" C4* CGR"}=" C4' CGR";
$resn_exch{" C5* CGR"}=" C5' CGR"; $resn_exch{" O5* CGR"}=" O5' CGR"; $resn_exch{"OE1  CGU"}="OE11 CGU";
$resn_exch{"OE2  CGU"}="OE12 CGU"; $resn_exch{"OE3  CGU"}="OE21 CGU"; $resn_exch{"OE4  CGU"}="OE22 CGU";
$resn_exch{" O3P  CH"}=" OP3  CH"; $resn_exch{" O1P  CH"}=" OP1  CH"; $resn_exch{" O2P  CH"}=" OP2  CH";
$resn_exch{" O5*  CH"}=" O5'  CH"; $resn_exch{" C5*  CH"}=" C5'  CH"; $resn_exch{" C4*  CH"}=" C4'  CH";
$resn_exch{" O4*  CH"}=" O4'  CH"; $resn_exch{" C3*  CH"}=" C3'  CH"; $resn_exch{" O3*  CH"}=" O3'  CH";
$resn_exch{" C2*  CH"}=" C2'  CH"; $resn_exch{" O2*  CH"}=" O2'  CH"; $resn_exch{" C1*  CH"}=" C1'  CH";
$resn_exch{" C1* CH1"}=" C1' CH1"; $resn_exch{" C2* CH1"}=" C2' CH1"; $resn_exch{" O2* CH1"}=" O2' CH1";
$resn_exch{" C3* CH1"}=" C3' CH1"; $resn_exch{" C4* CH1"}=" C4' CH1"; $resn_exch{" O4* CH1"}=" O4' CH1";
$resn_exch{" C5* CH1"}=" C5' CH1"; $resn_exch{" O5* CH1"}=" O5' CH1"; $resn_exch{" N_A CHL"}=" NA  CHL";
$resn_exch{" N_B CHL"}=" NB  CHL"; $resn_exch{" N_C CHL"}=" NC  CHL"; $resn_exch{" N_D CHL"}=" ND  CHL";
$resn_exch{" AN1 CIC"}=" N1A CIC"; $resn_exch{" AC2 CIC"}=" C2A CIC"; $resn_exch{" AN3 CIC"}=" N3A CIC";
$resn_exch{" AC4 CIC"}=" C4A CIC"; $resn_exch{" AC5 CIC"}=" C5A CIC"; $resn_exch{" AC6 CIC"}=" C6A CIC";
$resn_exch{" AN6 CIC"}=" N6A CIC"; $resn_exch{" AN7 CIC"}=" N7A CIC"; $resn_exch{" AC8 CIC"}=" C8A CIC";
$resn_exch{" AN9 CIC"}=" N9A CIC"; $resn_exch{"AC1* CIC"}=" C1B CIC"; $resn_exch{"AC2* CIC"}=" C2B CIC";
$resn_exch{"AO2* CIC"}=" O2B CIC"; $resn_exch{"AC3* CIC"}=" C3B CIC"; $resn_exch{"AO3* CIC"}=" O3B CIC";
$resn_exch{"AP3* CIC"}=" P3B CIC"; $resn_exch{" AO7 CIC"}=" O7A CIC"; $resn_exch{" AO8 CIC"}=" O8A CIC";
$resn_exch{" AO9 CIC"}=" O9A CIC"; $resn_exch{"AC4* CIC"}=" C4B CIC"; $resn_exch{"AO4* CIC"}=" O4B CIC";
$resn_exch{"AC5* CIC"}=" C5B CIC"; $resn_exch{"AO5* CIC"}=" O5B CIC"; $resn_exch{" AP1 CIC"}=" P1A CIC";
$resn_exch{" AO1 CIC"}=" O1A CIC"; $resn_exch{" AO2 CIC"}=" O2A CIC"; $resn_exch{" AO3 CIC"}=" O3A CIC";
$resn_exch{" AP2 CIC"}=" P2A CIC"; $resn_exch{" AO4 CIC"}=" O4A CIC"; $resn_exch{" AO5 CIC"}=" O5A CIC";
$resn_exch{" AO6 CIC"}=" O6A CIC"; $resn_exch{"PC11 CIC"}=" CBP CIC"; $resn_exch{"PC12 CIC"}=" CCP CIC";
$resn_exch{"PC13 CIC"}=" CDP CIC"; $resn_exch{"PC14 CIC"}=" CEP CIC"; $resn_exch{"PC10 CIC"}=" CAP CIC";
$resn_exch{"PO10 CIC"}=" OAP CIC"; $resn_exch{" PC9 CIC"}=" C9P CIC"; $resn_exch{" PO9 CIC"}=" O9P CIC";
$resn_exch{" PN8 CIC"}=" N8P CIC"; $resn_exch{" PC7 CIC"}=" C7P CIC"; $resn_exch{" PC6 CIC"}=" C6P CIC";
$resn_exch{" PC5 CIC"}=" C5P CIC"; $resn_exch{" PO5 CIC"}=" O5P CIC"; $resn_exch{" PN4 CIC"}=" N4P CIC";
$resn_exch{" PC3 CIC"}=" C3P CIC"; $resn_exch{" PC2 CIC"}=" C2P CIC"; $resn_exch{" PS1 CIC"}=" S1P CIC";
$resn_exch{" PC1 CIC"}=" C1P CIC"; $resn_exch{"O11A CIE"}=" OBA CIE"; $resn_exch{"O11B CIE"}=" OBB CIE";
$resn_exch{" N_A CL1"}=" NA  CL1"; $resn_exch{" N_B CL1"}=" NB  CL1"; $resn_exch{" N_C CL1"}=" NC  CL1";
$resn_exch{" N_D CL1"}=" ND  CL1"; $resn_exch{" N_A CL2"}=" NA  CL2"; $resn_exch{" N_B CL2"}=" NB  CL2";
$resn_exch{" N_C CL2"}=" NC  CL2"; $resn_exch{" N_D CL2"}=" ND  CL2"; $resn_exch{" N_A CLA"}=" NA  CLA";
$resn_exch{" N_B CLA"}=" NB  CLA"; $resn_exch{" N_C CLA"}=" NC  CLA"; $resn_exch{" N_D CLA"}=" ND  CLA";
$resn_exch{"CL1A CLM"}="CL1  CLM"; $resn_exch{"CL1B CLM"}="CL2  CLM"; $resn_exch{" N_A CLN"}=" NA  CLN";
$resn_exch{" N_B CLN"}=" NB  CLN"; $resn_exch{" N_C CLN"}=" NC  CLN"; $resn_exch{" N_D CLN"}=" ND  CLN";
$resn_exch{" O5* CM0"}=" O5' CM0"; $resn_exch{" C3* CM0"}=" C3' CM0"; $resn_exch{" C5* CM0"}=" C5' CM0";
$resn_exch{" C4* CM0"}=" C4' CM0"; $resn_exch{" O4* CM0"}=" O4' CM0"; $resn_exch{" C2* CM0"}=" C2' CM0";
$resn_exch{" C1* CM0"}=" C1' CM0"; $resn_exch{" O2* CM0"}=" O2' CM0"; $resn_exch{" O3* CM0"}=" O3' CM0";
$resn_exch{" AN1 CMC"}=" N1A CMC"; $resn_exch{" AC2 CMC"}=" C2A CMC"; $resn_exch{" AN3 CMC"}=" N3A CMC";
$resn_exch{" AC4 CMC"}=" C4A CMC"; $resn_exch{" AC5 CMC"}=" C5A CMC"; $resn_exch{" AC6 CMC"}=" C6A CMC";
$resn_exch{" AN6 CMC"}=" N6A CMC"; $resn_exch{" AN7 CMC"}=" N7A CMC"; $resn_exch{" AC8 CMC"}=" C8A CMC";
$resn_exch{" AN9 CMC"}=" N9A CMC"; $resn_exch{"AC1* CMC"}=" C1B CMC"; $resn_exch{"AC2* CMC"}=" C2B CMC";
$resn_exch{"AO2* CMC"}=" O2B CMC"; $resn_exch{"AC3* CMC"}=" C3B CMC"; $resn_exch{"AO3* CMC"}=" O3B CMC";
$resn_exch{"AP3* CMC"}=" P3B CMC"; $resn_exch{" AO7 CMC"}=" O7A CMC"; $resn_exch{" AO8 CMC"}=" O8A CMC";
$resn_exch{" AO9 CMC"}=" O9A CMC"; $resn_exch{"AC4* CMC"}=" C4B CMC"; $resn_exch{"AO4* CMC"}=" O4B CMC";
$resn_exch{"AC5* CMC"}=" C5B CMC"; $resn_exch{"AO5* CMC"}=" O5B CMC"; $resn_exch{" AP1 CMC"}=" P1A CMC";
$resn_exch{" AO1 CMC"}=" O1A CMC"; $resn_exch{" AO2 CMC"}=" O2A CMC"; $resn_exch{" AO3 CMC"}=" O3A CMC";
$resn_exch{" AP2 CMC"}=" P2A CMC"; $resn_exch{" AO4 CMC"}=" O4A CMC"; $resn_exch{" AO5 CMC"}=" O5A CMC";
$resn_exch{" AO6 CMC"}=" O6A CMC"; $resn_exch{"PC11 CMC"}=" CBP CMC"; $resn_exch{"PC12 CMC"}=" CCP CMC";
$resn_exch{"PC13 CMC"}=" CDP CMC"; $resn_exch{"PC14 CMC"}=" CEP CMC"; $resn_exch{"PC10 CMC"}=" CAP CMC";
$resn_exch{"PO10 CMC"}=" OAP CMC"; $resn_exch{" PC9 CMC"}=" C9P CMC"; $resn_exch{" PO9 CMC"}=" O9P CMC";
$resn_exch{" PN8 CMC"}=" N8P CMC"; $resn_exch{" PC7 CMC"}=" C7P CMC"; $resn_exch{" PC6 CMC"}=" C6P CMC";
$resn_exch{" PC5 CMC"}=" C5P CMC"; $resn_exch{" PO5 CMC"}=" O5P CMC"; $resn_exch{" PN4 CMC"}=" N4P CMC";
$resn_exch{" PC3 CMC"}=" C3P CMC"; $resn_exch{" PC2 CMC"}=" C2P CMC"; $resn_exch{" PS1 CMC"}=" S1P CMC";
$resn_exch{" C1* CMD"}=" C1' CMD"; $resn_exch{" C2* CMD"}=" C2' CMD"; $resn_exch{" C3* CMD"}=" C3' CMD";
$resn_exch{" C4* CMD"}=" C4' CMD"; $resn_exch{" O4* CMD"}=" O4' CMD"; $resn_exch{" C5* CMD"}=" C5' CMD";
$resn_exch{" O5* CMD"}=" O5' CMD"; $resn_exch{" C6* CMD"}=" C6' CMD"; $resn_exch{" N3* CMD"}=" N3' CMD";
$resn_exch{" C1* CMK"}=" C1' CMK"; $resn_exch{" C2* CMK"}=" C2' CMK"; $resn_exch{" O2* CMK"}=" O2' CMK";
$resn_exch{" C3* CMK"}=" C3' CMK"; $resn_exch{" C4* CMK"}=" C4' CMK"; $resn_exch{" O4* CMK"}=" O4' CMK";
$resn_exch{" O3* CMK"}=" O3' CMK"; $resn_exch{" C5* CMK"}=" C5' CMK"; $resn_exch{" O5* CMK"}=" O5' CMK";
$resn_exch{" O5* CMP"}=" O5' CMP"; $resn_exch{" C5* CMP"}=" C5' CMP"; $resn_exch{" C4* CMP"}=" C4' CMP";
$resn_exch{" O4* CMP"}=" O4' CMP"; $resn_exch{" C3* CMP"}=" C3' CMP"; $resn_exch{" O3* CMP"}=" O3' CMP";
$resn_exch{" C2* CMP"}=" C2' CMP"; $resn_exch{" O2* CMP"}=" O2' CMP"; $resn_exch{" C1* CMP"}=" C1' CMP";
$resn_exch{" O1P CMR"}=" OP1 CMR"; $resn_exch{" O5* CMR"}=" O5' CMR"; $resn_exch{" C5* CMR"}=" C5' CMR";
$resn_exch{" C4* CMR"}=" C4' CMR"; $resn_exch{" O4* CMR"}=" O4' CMR"; $resn_exch{" C1* CMR"}=" C1' CMR";
$resn_exch{" C3* CMR"}=" C3' CMR"; $resn_exch{" C2* CMR"}=" C2' CMR"; $resn_exch{" O3* CMR"}=" O3' CMR";
$resn_exch{" O3P CMR"}=" OP3 CMR"; $resn_exch{" AN1 CMX"}=" N1A CMX"; $resn_exch{" AC2 CMX"}=" C2A CMX";
$resn_exch{" AN3 CMX"}=" N3A CMX"; $resn_exch{" AC4 CMX"}=" C4A CMX"; $resn_exch{" AC5 CMX"}=" C5A CMX";
$resn_exch{" AC6 CMX"}=" C6A CMX"; $resn_exch{" AN6 CMX"}=" N6A CMX"; $resn_exch{" AN7 CMX"}=" N7A CMX";
$resn_exch{" AC8 CMX"}=" C8A CMX"; $resn_exch{" AN9 CMX"}=" N9A CMX"; $resn_exch{"AC1* CMX"}=" C1B CMX";
$resn_exch{"AC2* CMX"}=" C2B CMX"; $resn_exch{"AO2* CMX"}=" O2B CMX"; $resn_exch{"AC3* CMX"}=" C3B CMX";
$resn_exch{"AO3* CMX"}=" O3B CMX"; $resn_exch{"AP3* CMX"}=" P3B CMX"; $resn_exch{" AO7 CMX"}=" O7A CMX";
$resn_exch{" AO8 CMX"}=" O8A CMX"; $resn_exch{" AO9 CMX"}=" O9A CMX"; $resn_exch{"AC4* CMX"}=" C4B CMX";
$resn_exch{"AO4* CMX"}=" O4B CMX"; $resn_exch{"AC5* CMX"}=" C5B CMX"; $resn_exch{"AO5* CMX"}=" O5B CMX";
$resn_exch{" AP1 CMX"}=" P1A CMX"; $resn_exch{" AO1 CMX"}=" O1A CMX"; $resn_exch{" AO2 CMX"}=" O2A CMX";
$resn_exch{" AO3 CMX"}=" O3A CMX"; $resn_exch{" AP2 CMX"}=" P2A CMX"; $resn_exch{" AO4 CMX"}=" O4A CMX";
$resn_exch{" AO5 CMX"}=" O5A CMX"; $resn_exch{" AO6 CMX"}=" O6A CMX"; $resn_exch{"PC11 CMX"}=" CBP CMX";
$resn_exch{"PC12 CMX"}=" CCP CMX"; $resn_exch{"PC13 CMX"}=" CDP CMX"; $resn_exch{"PC14 CMX"}=" CEP CMX";
$resn_exch{"PC10 CMX"}=" CAP CMX"; $resn_exch{"PO10 CMX"}=" OAP CMX"; $resn_exch{" PC9 CMX"}=" C9P CMX";
$resn_exch{" PO9 CMX"}=" O9P CMX"; $resn_exch{" PN8 CMX"}=" N8P CMX"; $resn_exch{" PC7 CMX"}=" C7P CMX";
$resn_exch{" PC6 CMX"}=" C6P CMX"; $resn_exch{" PC5 CMX"}=" C5P CMX"; $resn_exch{" PO5 CMX"}=" O5P CMX";
$resn_exch{" PN4 CMX"}=" N4P CMX"; $resn_exch{" PC3 CMX"}=" C3P CMX"; $resn_exch{" PC2 CMX"}=" C2P CMX";
$resn_exch{" AP  CNA"}=" PA  CNA"; $resn_exch{" AO1 CNA"}=" O1A CNA"; $resn_exch{" AO2 CNA"}=" O2A CNA";
$resn_exch{"AO5* CNA"}=" O5B CNA"; $resn_exch{"AC5* CNA"}=" C5B CNA"; $resn_exch{"AC4* CNA"}=" C4B CNA";
$resn_exch{"AO4* CNA"}=" O4B CNA"; $resn_exch{"AC3* CNA"}=" C3B CNA"; $resn_exch{"AO3* CNA"}=" O3B CNA";
$resn_exch{"AC2* CNA"}=" C2B CNA"; $resn_exch{"AO2* CNA"}=" O2B CNA"; $resn_exch{"AC1* CNA"}=" C1B CNA";
$resn_exch{" AN9 CNA"}=" N9A CNA"; $resn_exch{" AC8 CNA"}=" C8A CNA"; $resn_exch{" AN7 CNA"}=" N7A CNA";
$resn_exch{" AC5 CNA"}=" C5A CNA"; $resn_exch{" AC6 CNA"}=" C6A CNA"; $resn_exch{" AN6 CNA"}=" N6A CNA";
$resn_exch{" AN1 CNA"}=" N1A CNA"; $resn_exch{" AC2 CNA"}=" C2A CNA"; $resn_exch{" AN3 CNA"}=" N3A CNA";
$resn_exch{" AC4 CNA"}=" C4A CNA"; $resn_exch{" NP  CNA"}=" PN  CNA"; $resn_exch{" NO1 CNA"}=" O1N CNA";
$resn_exch{" NO2 CNA"}=" O2N CNA"; $resn_exch{"NO5* CNA"}=" O5D CNA"; $resn_exch{"NC5* CNA"}=" C5D CNA";
$resn_exch{"NC4* CNA"}=" C4D CNA"; $resn_exch{" C4* CNA"}=" C4' CNA"; $resn_exch{"NC3* CNA"}=" C3D CNA";
$resn_exch{"NO3* CNA"}=" O3D CNA"; $resn_exch{"NC2* CNA"}=" C2D CNA"; $resn_exch{"NO2* CNA"}=" O2D CNA";
$resn_exch{"NC1* CNA"}=" C1D CNA"; $resn_exch{" NN1 CNA"}=" N1N CNA"; $resn_exch{" NC2 CNA"}=" C2N CNA";
$resn_exch{" NC3 CNA"}=" C3N CNA"; $resn_exch{" NC7 CNA"}=" C7N CNA"; $resn_exch{" NO7 CNA"}=" O7N CNA";
$resn_exch{" NN7 CNA"}=" N7N CNA"; $resn_exch{" NC4 CNA"}=" C4N CNA"; $resn_exch{" NC5 CNA"}=" C5N CNA";
$resn_exch{" NC6 CNA"}=" C6N CNA"; $resn_exch{" AP  CND"}=" PA  CND"; $resn_exch{" AO1 CND"}=" O1A CND";
$resn_exch{" AO2 CND"}=" O2A CND"; $resn_exch{"AO5* CND"}=" O5B CND"; $resn_exch{"AC5* CND"}=" C5B CND";
$resn_exch{"AC4* CND"}=" C4B CND"; $resn_exch{"AO4* CND"}=" O4B CND"; $resn_exch{"AC3* CND"}=" C3B CND";
$resn_exch{"AO3* CND"}=" O3B CND"; $resn_exch{"AC2* CND"}=" C2B CND"; $resn_exch{"AO2* CND"}=" O2B CND";
$resn_exch{"AC1* CND"}=" C1B CND"; $resn_exch{" AN9 CND"}=" N9A CND"; $resn_exch{" AC8 CND"}=" C8A CND";
$resn_exch{" AN7 CND"}=" N7A CND"; $resn_exch{" AC5 CND"}=" C5A CND"; $resn_exch{" AC6 CND"}=" C6A CND";
$resn_exch{" AN6 CND"}=" N6A CND"; $resn_exch{" AN1 CND"}=" N1A CND"; $resn_exch{" AC2 CND"}=" C2A CND";
$resn_exch{" AN3 CND"}=" N3A CND"; $resn_exch{" AC4 CND"}=" C4A CND"; $resn_exch{" NP  CND"}=" PN  CND";
$resn_exch{" NO1 CND"}=" O1N CND"; $resn_exch{" NO2 CND"}=" O2N CND"; $resn_exch{"NO5* CND"}=" O5D CND";
$resn_exch{"NC5* CND"}=" C5D CND"; $resn_exch{"NC4* CND"}=" C4D CND"; $resn_exch{"NO4* CND"}=" O4D CND";
$resn_exch{"NC3* CND"}=" C3D CND"; $resn_exch{"NO3* CND"}=" O3D CND"; $resn_exch{"NC2* CND"}=" C2D CND";
$resn_exch{"NO2* CND"}=" O2D CND"; $resn_exch{"NC1* CND"}=" C1D CND"; $resn_exch{" NC1 CND"}=" C1N CND";
$resn_exch{" NC2 CND"}=" C2N CND"; $resn_exch{" NC3 CND"}=" C3N CND"; $resn_exch{" NC7 CND"}=" C7N CND";
$resn_exch{" NO7 CND"}=" O7N CND"; $resn_exch{" NN7 CND"}=" N7N CND"; $resn_exch{" NC4 CND"}=" C4N CND";
$resn_exch{" NN5 CND"}=" N5N CND"; $resn_exch{" NC6 CND"}=" C6N CND"; $resn_exch{" N2* CO4"}=" N2' CO4";
$resn_exch{" C2* CO4"}=" C2D CO4"; $resn_exch{" N3* CO4"}=" N3' CO4"; $resn_exch{" C4* CO4"}=" C4D CO4";
$resn_exch{" N4* CO4"}=" N4' CO4"; $resn_exch{" C5* CO4"}=" C5D CO4"; $resn_exch{" C6* CO4"}=" C6D CO4";
$resn_exch{" C7* CO4"}=" C7' CO4"; $resn_exch{" N8* CO4"}=" N8' CO4"; $resn_exch{" N1* CO4"}=" N1' CO4";
$resn_exch{" C8* CO4"}=" C8' CO4"; $resn_exch{" N9* CO4"}=" N9' CO4"; $resn_exch{" C9* CO4"}=" C9' CO4";
$resn_exch{" AN1 CO6"}=" N1A CO6"; $resn_exch{" AC2 CO6"}=" C2A CO6"; $resn_exch{" AN3 CO6"}=" N3A CO6";
$resn_exch{" AC4 CO6"}=" C4A CO6"; $resn_exch{" AC5 CO6"}=" C5A CO6"; $resn_exch{" AC6 CO6"}=" C6A CO6";
$resn_exch{" AN6 CO6"}=" N6A CO6"; $resn_exch{" AN7 CO6"}=" N7A CO6"; $resn_exch{" AC8 CO6"}=" C8A CO6";
$resn_exch{" AN9 CO6"}=" N9A CO6"; $resn_exch{"AC1* CO6"}=" C1B CO6"; $resn_exch{"AC2* CO6"}=" C2B CO6";
$resn_exch{"AO2* CO6"}=" O2B CO6"; $resn_exch{"AC3* CO6"}=" C3B CO6"; $resn_exch{"AO3* CO6"}=" O3B CO6";
$resn_exch{"AP3* CO6"}=" P3B CO6"; $resn_exch{" AO7 CO6"}=" O7A CO6"; $resn_exch{" AO8 CO6"}=" O8A CO6";
$resn_exch{" AO9 CO6"}=" O9A CO6"; $resn_exch{"AC4* CO6"}=" C4B CO6"; $resn_exch{"AO4* CO6"}=" O4B CO6";
$resn_exch{"AC5* CO6"}=" C5B CO6"; $resn_exch{"AO5* CO6"}=" O5B CO6"; $resn_exch{" AP1 CO6"}=" P1A CO6";
$resn_exch{" AO1 CO6"}=" O1A CO6"; $resn_exch{" AO2 CO6"}=" O2A CO6"; $resn_exch{" AO3 CO6"}=" O3A CO6";
$resn_exch{" AP2 CO6"}=" P2A CO6"; $resn_exch{" AO4 CO6"}=" O4A CO6"; $resn_exch{" AO5 CO6"}=" O5A CO6";
$resn_exch{" AO6 CO6"}=" O6A CO6"; $resn_exch{"PC11 CO6"}=" CBP CO6"; $resn_exch{"PC12 CO6"}=" CCP CO6";
$resn_exch{"PC13 CO6"}=" CDP CO6"; $resn_exch{"PC14 CO6"}=" CEP CO6"; $resn_exch{"PC10 CO6"}=" CAP CO6";
$resn_exch{"PO10 CO6"}=" OAP CO6"; $resn_exch{" PC9 CO6"}=" C9P CO6"; $resn_exch{" PO9 CO6"}=" O9P CO6";
$resn_exch{" PN8 CO6"}=" N8P CO6"; $resn_exch{" PC7 CO6"}=" C7P CO6"; $resn_exch{" PC6 CO6"}=" C6P CO6";
$resn_exch{" PC5 CO6"}=" C5P CO6"; $resn_exch{" PO5 CO6"}=" O5P CO6"; $resn_exch{" PN4 CO6"}=" N4P CO6";
$resn_exch{" PC3 CO6"}=" C3P CO6"; $resn_exch{" PC2 CO6"}=" C2P CO6"; $resn_exch{" PS1 CO6"}=" S1P CO6";
$resn_exch{" AN1 CO8"}=" N1A CO8"; $resn_exch{" AC2 CO8"}=" C2A CO8"; $resn_exch{" AN3 CO8"}=" N3A CO8";
$resn_exch{" AC4 CO8"}=" C4A CO8"; $resn_exch{" AC5 CO8"}=" C5A CO8"; $resn_exch{" AC6 CO8"}=" C6A CO8";
$resn_exch{" AN6 CO8"}=" N6A CO8"; $resn_exch{" AN7 CO8"}=" N7A CO8"; $resn_exch{" AC8 CO8"}=" C8A CO8";
$resn_exch{" AN9 CO8"}=" N9A CO8"; $resn_exch{"AC1* CO8"}=" C1B CO8"; $resn_exch{"AC2* CO8"}=" C2B CO8";
$resn_exch{"AO2* CO8"}=" O2B CO8"; $resn_exch{"AC3* CO8"}=" C3B CO8"; $resn_exch{"AO3* CO8"}=" O3B CO8";
$resn_exch{"AP3* CO8"}=" P3B CO8"; $resn_exch{" AO7 CO8"}=" O7A CO8"; $resn_exch{" AO8 CO8"}=" O8A CO8";
$resn_exch{" AO9 CO8"}=" O9A CO8"; $resn_exch{"AC4* CO8"}=" C4B CO8"; $resn_exch{"AO4* CO8"}=" O4B CO8";
$resn_exch{"AC5* CO8"}=" C5B CO8"; $resn_exch{"AO5* CO8"}=" O5B CO8"; $resn_exch{" AP1 CO8"}=" P1A CO8";
$resn_exch{" AO1 CO8"}=" O1A CO8"; $resn_exch{" AO2 CO8"}=" O2A CO8"; $resn_exch{" AO3 CO8"}=" O3A CO8";
$resn_exch{" AP2 CO8"}=" P2A CO8"; $resn_exch{" AO4 CO8"}=" O4A CO8"; $resn_exch{" AO5 CO8"}=" O5A CO8";
$resn_exch{" AO6 CO8"}=" O6A CO8"; $resn_exch{"PC11 CO8"}=" CBP CO8"; $resn_exch{"PC12 CO8"}=" CCP CO8";
$resn_exch{"PC13 CO8"}=" CDP CO8"; $resn_exch{"PC14 CO8"}=" CEP CO8"; $resn_exch{"PC10 CO8"}=" CAP CO8";
$resn_exch{"PO10 CO8"}=" OAP CO8"; $resn_exch{" PC9 CO8"}=" C9P CO8"; $resn_exch{" PO9 CO8"}=" O9P CO8";
$resn_exch{" PN8 CO8"}=" N8P CO8"; $resn_exch{" PC7 CO8"}=" C7P CO8"; $resn_exch{" PC6 CO8"}=" C6P CO8";
$resn_exch{" PC5 CO8"}=" C5P CO8"; $resn_exch{" PO5 CO8"}=" O5P CO8"; $resn_exch{" PN4 CO8"}=" N4P CO8";
$resn_exch{" PC3 CO8"}=" C3P CO8"; $resn_exch{" PC2 CO8"}=" C2P CO8"; $resn_exch{" PS1 CO8"}=" S1P CO8";
$resn_exch{" C1* CO8"}=" C1' CO8"; $resn_exch{" O1* CO8"}=" O1' CO8"; $resn_exch{" C2* CO8"}=" C2' CO8";
$resn_exch{" C3* CO8"}=" C3' CO8"; $resn_exch{" C4* CO8"}=" C4' CO8"; $resn_exch{" C5* CO8"}=" C5' CO8";
$resn_exch{" C6* CO8"}=" C6' CO8"; $resn_exch{" C7* CO8"}=" C7' CO8"; $resn_exch{" C8* CO8"}=" C8' CO8";
$resn_exch{" AN1 COA"}=" N1A COA"; $resn_exch{" AC2 COA"}=" C2A COA"; $resn_exch{" AN3 COA"}=" N3A COA";
$resn_exch{" AC4 COA"}=" C4A COA"; $resn_exch{" AC5 COA"}=" C5A COA"; $resn_exch{" AC6 COA"}=" C6A COA";
$resn_exch{" AN6 COA"}=" N6A COA"; $resn_exch{" AN7 COA"}=" N7A COA"; $resn_exch{" AC8 COA"}=" C8A COA";
$resn_exch{" AN9 COA"}=" N9A COA"; $resn_exch{"AC1* COA"}=" C1B COA"; $resn_exch{"AC2* COA"}=" C2B COA";
$resn_exch{"AO2* COA"}=" O2B COA"; $resn_exch{"AC3* COA"}=" C3B COA"; $resn_exch{"AO3* COA"}=" O3B COA";
$resn_exch{"AP3* COA"}=" P3B COA"; $resn_exch{" AO7 COA"}=" O7A COA"; $resn_exch{" AO8 COA"}=" O8A COA";
$resn_exch{" AO9 COA"}=" O9A COA"; $resn_exch{"AC4* COA"}=" C4B COA"; $resn_exch{"AO4* COA"}=" O4B COA";
$resn_exch{"AC5* COA"}=" C5B COA"; $resn_exch{"AO5* COA"}=" O5B COA"; $resn_exch{" AP1 COA"}=" P1A COA";
$resn_exch{" AO1 COA"}=" O1A COA"; $resn_exch{" AO2 COA"}=" O2A COA"; $resn_exch{" AO3 COA"}=" O3A COA";
$resn_exch{" AP2 COA"}=" P2A COA"; $resn_exch{" AO4 COA"}=" O4A COA"; $resn_exch{" AO5 COA"}=" O5A COA";
$resn_exch{" AO6 COA"}=" O6A COA"; $resn_exch{"PC11 COA"}=" CBP COA"; $resn_exch{"PC12 COA"}=" CCP COA";
$resn_exch{"PC13 COA"}=" CDP COA"; $resn_exch{"PC14 COA"}=" CEP COA"; $resn_exch{"PC10 COA"}=" CAP COA";
$resn_exch{"PO10 COA"}=" OAP COA"; $resn_exch{" PC9 COA"}=" C9P COA"; $resn_exch{" PO9 COA"}=" O9P COA";
$resn_exch{" PN8 COA"}=" N8P COA"; $resn_exch{" PC7 COA"}=" C7P COA"; $resn_exch{" PC6 COA"}=" C6P COA";
$resn_exch{" PC5 COA"}=" C5P COA"; $resn_exch{" PO5 COA"}=" O5P COA"; $resn_exch{" PN4 COA"}=" N4P COA";
$resn_exch{" PC3 COA"}=" C3P COA"; $resn_exch{" PC2 COA"}=" C2P COA"; $resn_exch{" PS1 COA"}=" S1P COA";
$resn_exch{" AN1 COF"}=" N1A COF"; $resn_exch{" AC2 COF"}=" C2A COF"; $resn_exch{" AN3 COF"}=" N3A COF";
$resn_exch{" AC4 COF"}=" C4A COF"; $resn_exch{" AC5 COF"}=" C5A COF"; $resn_exch{" AC6 COF"}=" C6A COF";
$resn_exch{" AN6 COF"}=" N6A COF"; $resn_exch{" AN7 COF"}=" N7A COF"; $resn_exch{" AC8 COF"}=" C8A COF";
$resn_exch{" AN9 COF"}=" N9A COF"; $resn_exch{"AC1* COF"}=" C1B COF"; $resn_exch{"AC2* COF"}=" C2B COF";
$resn_exch{"AO2* COF"}=" O2B COF"; $resn_exch{"AC3* COF"}=" C3B COF"; $resn_exch{"AO3* COF"}=" O3B COF";
$resn_exch{"AP3* COF"}=" P3B COF"; $resn_exch{" AO7 COF"}=" O7A COF"; $resn_exch{" AO8 COF"}=" O8A COF";
$resn_exch{" AO9 COF"}=" O9A COF"; $resn_exch{"AC4* COF"}=" C4B COF"; $resn_exch{"AO4* COF"}=" O4B COF";
$resn_exch{"AC5* COF"}=" C5B COF"; $resn_exch{"AO5* COF"}=" O5B COF"; $resn_exch{" AP1 COF"}=" P1A COF";
$resn_exch{" AO1 COF"}=" O1A COF"; $resn_exch{" AO2 COF"}=" O2A COF"; $resn_exch{" AO3 COF"}=" O3A COF";
$resn_exch{" AP2 COF"}=" P2A COF"; $resn_exch{" AO4 COF"}=" O4A COF"; $resn_exch{" AO5 COF"}=" O5A COF";
$resn_exch{" AO6 COF"}=" O6A COF"; $resn_exch{"PC11 COF"}=" CBP COF"; $resn_exch{"PC12 COF"}=" CCP COF";
$resn_exch{"PC13 COF"}=" CDP COF"; $resn_exch{"PC14 COF"}=" CEP COF"; $resn_exch{"PC10 COF"}=" CAP COF";
$resn_exch{"PO10 COF"}=" OAP COF"; $resn_exch{" PC9 COF"}=" C9P COF"; $resn_exch{" PO9 COF"}=" O9P COF";
$resn_exch{" PN8 COF"}=" N8P COF"; $resn_exch{" PC7 COF"}=" C7P COF"; $resn_exch{" PC6 COF"}=" C6P COF";
$resn_exch{" PC5 COF"}=" C5P COF"; $resn_exch{" PO5 COF"}=" O5P COF"; $resn_exch{" PN4 COF"}=" N4P COF";
$resn_exch{" PC3 COF"}=" C3P COF"; $resn_exch{" PC2 COF"}=" C2P COF"; $resn_exch{" PS1 COF"}=" S1P COF";
$resn_exch{" IC1 COF"}=" C1I COF"; $resn_exch{" IC2 COF"}=" C2I COF"; $resn_exch{" IO1 COF"}=" O1I COF";
$resn_exch{" IC3 COF"}=" C3I COF"; $resn_exch{" IF1 COF"}=" F1I COF"; $resn_exch{" IF2 COF"}=" F2I COF";
$resn_exch{" IF3 COF"}=" F3I COF"; $resn_exch{" C2* COG"}=" C2B COG"; $resn_exch{" C4* COG"}=" C4B COG";
$resn_exch{" C5* COG"}=" C5B COG"; $resn_exch{" C6* COG"}=" C6B COG"; $resn_exch{" C7* COG"}=" C7B COG";
$resn_exch{" N1* COG"}=" N1' COG"; $resn_exch{" N2* COG"}=" N2' COG"; $resn_exch{" N3* COG"}=" N3' COG";
$resn_exch{" N4* COG"}=" N4' COG"; $resn_exch{" C8* COG"}=" C8' COG"; $resn_exch{" N6* COG"}=" N6' COG";
$resn_exch{" N_A COH"}=" NA  COH"; $resn_exch{" N_B COH"}=" NB  COH"; $resn_exch{" N_C COH"}=" NC  COH";
$resn_exch{" N_D COH"}=" ND  COH"; $resn_exch{" AN6 COK"}=" N6A COK"; $resn_exch{" AC6 COK"}=" C6A COK";
$resn_exch{" AN1 COK"}=" N1A COK"; $resn_exch{" AC2 COK"}=" C2A COK"; $resn_exch{" AN3 COK"}=" N3A COK";
$resn_exch{" AC4 COK"}=" C4A COK"; $resn_exch{" AC5 COK"}=" C5A COK"; $resn_exch{" AN7 COK"}=" N7A COK";
$resn_exch{" AC8 COK"}=" C8A COK"; $resn_exch{" AN9 COK"}=" N9A COK"; $resn_exch{"AC1* COK"}=" C1B COK";
$resn_exch{"AO4* COK"}=" O4B COK"; $resn_exch{"AC2* COK"}=" C2B COK"; $resn_exch{"AO2* COK"}=" O2B COK";
$resn_exch{"AC3* COK"}=" C3B COK"; $resn_exch{"AO3* COK"}=" O3B COK"; $resn_exch{"AP3* COK"}=" P3B COK";
$resn_exch{" AO8 COK"}=" O8A COK"; $resn_exch{" AO9 COK"}=" O9A COK"; $resn_exch{" AO7 COK"}=" O7A COK";
$resn_exch{"AC4* COK"}=" C4B COK"; $resn_exch{"AC5* COK"}=" C5B COK"; $resn_exch{"AO5* COK"}=" O5B COK";
$resn_exch{" AP1 COK"}=" P1A COK"; $resn_exch{" AO1 COK"}=" O1A COK"; $resn_exch{" AO2 COK"}=" O2A COK";
$resn_exch{" AO3 COK"}=" O3A COK"; $resn_exch{" AP2 COK"}=" P2A COK"; $resn_exch{" AO4 COK"}=" O4A COK";
$resn_exch{" AO5 COK"}=" O5A COK"; $resn_exch{" AO6 COK"}=" O6A COK"; $resn_exch{"PC12 COK"}=" CCP COK";
$resn_exch{"PC11 COK"}=" CBP COK"; $resn_exch{"PC13 COK"}=" CDP COK"; $resn_exch{"PC14 COK"}=" CEP COK";
$resn_exch{"PC10 COK"}=" CAP COK"; $resn_exch{"PO10 COK"}=" OAP COK"; $resn_exch{" PC9 COK"}=" C9P COK";
$resn_exch{" PO9 COK"}=" O9P COK"; $resn_exch{" PN8 COK"}=" N8P COK"; $resn_exch{" PC7 COK"}=" C7P COK";
$resn_exch{" PC6 COK"}=" C6P COK"; $resn_exch{" PC5 COK"}=" C5P COK"; $resn_exch{" PO5 COK"}=" O5P COK";
$resn_exch{" PN4 COK"}=" N4P COK"; $resn_exch{" PC3 COK"}=" C3P COK"; $resn_exch{" PC2 COK"}=" C2P COK";
$resn_exch{" PS1 COK"}=" S1P COK"; $resn_exch{" C2* COQ"}=" C2B COQ"; $resn_exch{" C4* COQ"}=" C4B COQ";
$resn_exch{" C5* COQ"}=" C5B COQ"; $resn_exch{" C6* COQ"}=" C6B COQ"; $resn_exch{" C7* COQ"}=" C7' COQ";
$resn_exch{" C9* COQ"}=" C9' COQ"; $resn_exch{" CN* COQ"}=" CN' COQ"; $resn_exch{" N1* COQ"}=" N1' COQ";
$resn_exch{" N2* COQ"}=" N2' COQ"; $resn_exch{" N3* COQ"}=" N3' COQ"; $resn_exch{" N4* COQ"}=" N4' COQ";
$resn_exch{" N8* COQ"}=" N8' COQ"; $resn_exch{" N9* COQ"}=" N9' COQ"; $resn_exch{" AN1 COS"}=" N1A COS";
$resn_exch{" AC2 COS"}=" C2A COS"; $resn_exch{" AN3 COS"}=" N3A COS"; $resn_exch{" AC4 COS"}=" C4A COS";
$resn_exch{" AC5 COS"}=" C5A COS"; $resn_exch{" AC6 COS"}=" C6A COS"; $resn_exch{" AN6 COS"}=" N6A COS";
$resn_exch{" AN7 COS"}=" N7A COS"; $resn_exch{" AC8 COS"}=" C8A COS"; $resn_exch{" AN9 COS"}=" N9A COS";
$resn_exch{"AC1* COS"}=" C1B COS"; $resn_exch{"AC2* COS"}=" C2B COS"; $resn_exch{"AO2* COS"}=" O2B COS";
$resn_exch{"AC3* COS"}=" C3B COS"; $resn_exch{"AO3* COS"}=" O3B COS"; $resn_exch{"AP3* COS"}=" P3B COS";
$resn_exch{" AO7 COS"}=" O7A COS"; $resn_exch{" AO8 COS"}=" O8A COS"; $resn_exch{" AO9 COS"}=" O9A COS";
$resn_exch{"AC4* COS"}=" C4B COS"; $resn_exch{"AO4* COS"}=" O4B COS"; $resn_exch{"AC5* COS"}=" C5B COS";
$resn_exch{"AO5* COS"}=" O5B COS"; $resn_exch{" AP1 COS"}=" P1A COS"; $resn_exch{" AO1 COS"}=" O1A COS";
$resn_exch{" AO2 COS"}=" O2A COS"; $resn_exch{" AO3 COS"}=" O3A COS"; $resn_exch{" AP2 COS"}=" P2A COS";
$resn_exch{" AO4 COS"}=" O4A COS"; $resn_exch{" AO5 COS"}=" O5A COS"; $resn_exch{" AO6 COS"}=" O6A COS";
$resn_exch{"PC11 COS"}=" CBP COS"; $resn_exch{"PC12 COS"}=" CCP COS"; $resn_exch{"PC13 COS"}=" CDP COS";
$resn_exch{"PC14 COS"}=" CEP COS"; $resn_exch{"PC10 COS"}=" CAP COS"; $resn_exch{"PO10 COS"}=" OAP COS";
$resn_exch{" PC9 COS"}=" C9P COS"; $resn_exch{" PO9 COS"}=" O9P COS"; $resn_exch{" PN8 COS"}=" N8P COS";
$resn_exch{" PC7 COS"}=" C7P COS"; $resn_exch{" PC6 COS"}=" C6P COS"; $resn_exch{" PC5 COS"}=" C5P COS";
$resn_exch{" PO5 COS"}=" O5P COS"; $resn_exch{" PN4 COS"}=" N4P COS"; $resn_exch{" PC3 COS"}=" C3P COS";
$resn_exch{" PC2 COS"}=" C2P COS"; $resn_exch{" PS1 COS"}=" S1P COS"; $resn_exch{" PS' COS"}=" S'P COS";
$resn_exch{"CD2T COT"}=" C11 COT"; $resn_exch{"CE2T COT"}=" C12 COT"; $resn_exch{"CE3T COT"}=" C13 COT";
$resn_exch{"CD1T COT"}=" C14 COT"; $resn_exch{"NE1T COT"}=" N11 COT"; $resn_exch{"CZ2T COT"}=" C15 COT";
$resn_exch{"CZ3T COT"}=" C16 COT"; $resn_exch{"CH2T COT"}=" C17 COT"; $resn_exch{"OPB3 COT"}=" O11 COT";
$resn_exch{"OPB1 COT"}=" O12 COT"; $resn_exch{"OPB2 COT"}=" O13 COT"; $resn_exch{"OPA3 COT"}=" O14 COT";
$resn_exch{"O3*A COT"}=" O3B COT"; $resn_exch{"OPA2 COT"}=" O15 COT"; $resn_exch{"OPA1 COT"}=" O16 COT";
$resn_exch{"O5*A COT"}=" O5B COT"; $resn_exch{"C5*A COT"}=" C5B COT"; $resn_exch{"C4*A COT"}=" C4B COT";
$resn_exch{"C3*A COT"}=" C3B COT"; $resn_exch{"O2*A COT"}=" O2B COT"; $resn_exch{"O4*A COT"}=" O4B COT";
$resn_exch{"OPC1 COT"}=" O17 COT"; $resn_exch{"OPC2 COT"}=" O18 COT"; $resn_exch{"OPC3 COT"}=" O19 COT";
$resn_exch{"C2*A COT"}=" C2B COT"; $resn_exch{"C1*A COT"}=" C1B COT"; $resn_exch{"O10P COT"}=" OAP COT";
$resn_exch{"C11P COT"}=" CBP COT"; $resn_exch{"O12P COT"}=" OCP COT"; $resn_exch{"C13P COT"}=" CDP COT";
$resn_exch{"C16P COT"}=" CGP COT"; $resn_exch{"C14P COT"}=" CEP COT"; $resn_exch{"C15P COT"}=" CFP COT";
$resn_exch{" AN1 COZ"}=" N1A COZ"; $resn_exch{" AC2 COZ"}=" C2A COZ"; $resn_exch{" AN3 COZ"}=" N3A COZ";
$resn_exch{" AC4 COZ"}=" C4A COZ"; $resn_exch{" AC5 COZ"}=" C5A COZ"; $resn_exch{" AC6 COZ"}=" C6A COZ";
$resn_exch{" AN6 COZ"}=" N6A COZ"; $resn_exch{" AN7 COZ"}=" N7A COZ"; $resn_exch{" AC8 COZ"}=" C8A COZ";
$resn_exch{" AN9 COZ"}=" N9A COZ"; $resn_exch{"AC1* COZ"}=" C1B COZ"; $resn_exch{"AC2* COZ"}=" C2B COZ";
$resn_exch{"AO2* COZ"}=" O2B COZ"; $resn_exch{"AC3* COZ"}=" C3B COZ"; $resn_exch{"AO3* COZ"}=" O3B COZ";
$resn_exch{"AP3* COZ"}=" P3B COZ"; $resn_exch{" AO7 COZ"}=" O7A COZ"; $resn_exch{" AO8 COZ"}=" O8A COZ";
$resn_exch{" AO9 COZ"}=" O9A COZ"; $resn_exch{"AC4* COZ"}=" C4B COZ"; $resn_exch{"AO4* COZ"}=" O4B COZ";
$resn_exch{"AC5* COZ"}=" C5B COZ"; $resn_exch{"AO5* COZ"}=" O5B COZ"; $resn_exch{" AP1 COZ"}=" P1A COZ";
$resn_exch{" AO1 COZ"}=" O1A COZ"; $resn_exch{" AO2 COZ"}=" O2A COZ"; $resn_exch{" AO3 COZ"}=" O3A COZ";
$resn_exch{" AP2 COZ"}=" P2A COZ"; $resn_exch{" AO4 COZ"}=" O4A COZ"; $resn_exch{" AO5 COZ"}=" O5A COZ";
$resn_exch{" AO6 COZ"}=" O6A COZ"; $resn_exch{"PC11 COZ"}=" CBP COZ"; $resn_exch{"PC12 COZ"}=" CCP COZ";
$resn_exch{"PC13 COZ"}=" CDP COZ"; $resn_exch{"PC14 COZ"}=" CEP COZ"; $resn_exch{"PC10 COZ"}=" CAP COZ";
$resn_exch{"PO10 COZ"}=" OAP COZ"; $resn_exch{" PC9 COZ"}=" C9P COZ"; $resn_exch{" PO9 COZ"}=" O9P COZ";
$resn_exch{" PN8 COZ"}=" N8P COZ"; $resn_exch{" PC7 COZ"}=" C7P COZ"; $resn_exch{" PC6 COZ"}=" C6P COZ";
$resn_exch{" PC5 COZ"}=" C5P COZ"; $resn_exch{" PO5 COZ"}=" O5P COZ"; $resn_exch{" PN4 COZ"}=" N4P COZ";
$resn_exch{" PC3 COZ"}=" C3P COZ"; $resn_exch{" PC2 COZ"}=" C2P COZ"; $resn_exch{" PS1 COZ"}=" S1P COZ";
$resn_exch{"CO5' CPA"}=" O5D CPA"; $resn_exch{"CC5' CPA"}=" C5X CPA"; $resn_exch{"CC4' CPA"}=" C4X CPA";
$resn_exch{"CO4' CPA"}=" O4D CPA"; $resn_exch{"CC3' CPA"}=" C3X CPA"; $resn_exch{"CO3' CPA"}=" O3D CPA";
$resn_exch{"CC2' CPA"}=" C2X CPA"; $resn_exch{"CC1' CPA"}=" C1X CPA"; $resn_exch{" CN1 CPA"}=" N1C CPA";
$resn_exch{" CN3 CPA"}=" N3C CPA"; $resn_exch{" CO2 CPA"}=" O2C CPA"; $resn_exch{" CN4 CPA"}=" N4C CPA";
$resn_exch{"AO5' CPA"}=" O5B CPA"; $resn_exch{"AC5' CPA"}=" C5B CPA"; $resn_exch{"AC4' CPA"}=" C4B CPA";
$resn_exch{"AO4' CPA"}=" O4B CPA"; $resn_exch{"AC3' CPA"}=" C3B CPA"; $resn_exch{"AO3' CPA"}=" O3B CPA";
$resn_exch{"AC2' CPA"}=" C2B CPA"; $resn_exch{"AC1' CPA"}=" C1B CPA"; $resn_exch{" AN9 CPA"}=" N9A CPA";
$resn_exch{" AC8 CPA"}=" C8A CPA"; $resn_exch{" AN7 CPA"}=" N7A CPA"; $resn_exch{" AC5 CPA"}=" C5A CPA";
$resn_exch{" AC6 CPA"}=" C6A CPA"; $resn_exch{" AN6 CPA"}=" N6A CPA"; $resn_exch{" AN1 CPA"}=" N1A CPA";
$resn_exch{" AC2 CPA"}=" C2A CPA"; $resn_exch{" AN3 CPA"}=" N3A CPA"; $resn_exch{" AC4 CPA"}=" C4A CPA";
$resn_exch{" O5* CPG"}=" O5' CPG"; $resn_exch{" C5* CPG"}=" C5' CPG"; $resn_exch{" C4* CPG"}=" C4' CPG";
$resn_exch{" O4* CPG"}=" O4' CPG"; $resn_exch{" C3* CPG"}=" C3' CPG"; $resn_exch{" O3* CPG"}=" O3' CPG";
$resn_exch{" C2* CPG"}=" C2' CPG"; $resn_exch{" O2* CPG"}=" O2' CPG"; $resn_exch{" C1* CPG"}=" C1' CPG";
$resn_exch{" C1* CPH"}=" C1' CPH"; $resn_exch{" O1* CPH"}=" O1' CPH"; $resn_exch{" C2* CPH"}=" C2' CPH";
$resn_exch{" O2* CPH"}=" O2' CPH"; $resn_exch{" C3* CPH"}=" C3' CPH"; $resn_exch{" O3* CPH"}=" O3' CPH";
$resn_exch{" C4* CPH"}=" C4' CPH"; $resn_exch{" O4* CPH"}=" O4' CPH"; $resn_exch{" C5* CPH"}=" C5' CPH";
$resn_exch{"C19* CPO"}=" CJ' CPO"; $resn_exch{" O5* CPR"}=" O5' CPR"; $resn_exch{" C5* CPR"}=" C5' CPR";
$resn_exch{" C4* CPR"}=" C4' CPR"; $resn_exch{" O4* CPR"}=" O4' CPR"; $resn_exch{" C3* CPR"}=" C3' CPR";
$resn_exch{" O3* CPR"}=" O3' CPR"; $resn_exch{" C2* CPR"}=" C2' CPR"; $resn_exch{" O2* CPR"}=" O2' CPR";
$resn_exch{" C1* CPR"}=" C1' CPR"; $resn_exch{" CA* CPV"}=" CA' CPV"; $resn_exch{" CB* CPV"}=" CB' CPV";
$resn_exch{"C1'' CR3"}=" C1B CR3"; $resn_exch{"C2'' CR3"}=" C2B CR3"; $resn_exch{"C3'' CR3"}=" C3B CR3";
$resn_exch{"C4'' CR3"}=" C4B CR3"; $resn_exch{"C5'' CR3"}=" C5B CR3"; $resn_exch{"C1'' CR9"}=" C1B CR9";
$resn_exch{"C2'' CR9"}=" C2B CR9"; $resn_exch{"C3'' CR9"}=" C3B CR9"; $resn_exch{"C4'' CR9"}=" C4B CR9";
$resn_exch{"C5'' CR9"}=" C5B CR9"; $resn_exch{"C6'' CR9"}=" C6B CR9"; $resn_exch{" C1* CRH"}=" C1' CRH";
$resn_exch{" C2* CRH"}=" C2' CRH"; $resn_exch{" C3* CRH"}=" C3' CRH"; $resn_exch{" C4* CRH"}=" C4' CRH";
$resn_exch{" O1* CRH"}=" O1' CRH"; $resn_exch{" O2* CRH"}=" O2' CRH"; $resn_exch{" O3* CRH"}=" O3' CRH";
$resn_exch{" O4* CRH"}=" O4' CRH"; $resn_exch{" O   CS3"}=" OXT CS3"; $resn_exch{" OXT CS3"}=" O   CS3";
$resn_exch{" O   CS4"}=" OXT CS4"; $resn_exch{" OXT CS4"}=" O   CS4"; $resn_exch{"PO10 CS8"}=" OAP CS8";
$resn_exch{"PC10 CS8"}=" CAP CS8"; $resn_exch{" PC9 CS8"}=" C9P CS8"; $resn_exch{" PO9 CS8"}=" O9P CS8";
$resn_exch{" PN8 CS8"}=" N8P CS8"; $resn_exch{" PC7 CS8"}=" C7P CS8"; $resn_exch{" PC6 CS8"}=" C6P CS8";
$resn_exch{" PC5 CS8"}=" C5P CS8"; $resn_exch{" PO5 CS8"}=" O5P CS8"; $resn_exch{" PN4 CS8"}=" N4P CS8";
$resn_exch{" PC3 CS8"}=" C3P CS8"; $resn_exch{" PC2 CS8"}=" C2P CS8"; $resn_exch{" PS1 CS8"}=" S1P CS8";
$resn_exch{" C1* CS8"}=" C1' CS8"; $resn_exch{" O1* CS8"}=" O1' CS8"; $resn_exch{" C2* CS8"}=" C2' CS8";
$resn_exch{" S3* CS8"}=" S3' CS8"; $resn_exch{" C4* CS8"}=" C4' CS8"; $resn_exch{" C5* CS8"}=" C5' CS8";
$resn_exch{" C6* CS8"}=" C6' CS8"; $resn_exch{" C7* CS8"}=" C7' CS8"; $resn_exch{" C8* CS8"}=" C8' CS8";
$resn_exch{"PC11 CS8"}=" CBP CS8"; $resn_exch{"PC13 CS8"}=" CDP CS8"; $resn_exch{"PC14 CS8"}=" CEP CS8";
$resn_exch{"PC12 CS8"}=" CCP CS8"; $resn_exch{" AO6 CS8"}=" O6A CS8"; $resn_exch{" AP2 CS8"}=" P2A CS8";
$resn_exch{" AO4 CS8"}=" O4A CS8"; $resn_exch{" AO5 CS8"}=" O5A CS8"; $resn_exch{" AO3 CS8"}=" O3A CS8";
$resn_exch{" AP1 CS8"}=" P1A CS8"; $resn_exch{" AO1 CS8"}=" O1A CS8"; $resn_exch{" AO2 CS8"}=" O2A CS8";
$resn_exch{"AO5* CS8"}=" O5B CS8"; $resn_exch{"AC5* CS8"}=" C5B CS8"; $resn_exch{"AC4* CS8"}=" C4B CS8";
$resn_exch{"AO4* CS8"}=" O4B CS8"; $resn_exch{"AC1* CS8"}=" C1B CS8"; $resn_exch{" AN9 CS8"}=" N9A CS8";
$resn_exch{" AC8 CS8"}=" C8A CS8"; $resn_exch{" AN7 CS8"}=" N7A CS8"; $resn_exch{" AC5 CS8"}=" C5A CS8";
$resn_exch{" AC6 CS8"}=" C6A CS8"; $resn_exch{" AN1 CS8"}=" N1A CS8"; $resn_exch{" AC2 CS8"}=" C2A CS8";
$resn_exch{" AN3 CS8"}=" N3A CS8"; $resn_exch{" AC4 CS8"}=" C4A CS8"; $resn_exch{" AN6 CS8"}=" N6A CS8";
$resn_exch{"AC2* CS8"}=" C2B CS8"; $resn_exch{"AO2* CS8"}=" O2B CS8"; $resn_exch{"AC3* CS8"}=" C3B CS8";
$resn_exch{"AO3* CS8"}=" O3B CS8"; $resn_exch{"AP3* CS8"}=" P3B CS8"; $resn_exch{" AO7 CS8"}=" O7A CS8";
$resn_exch{" AO8 CS8"}=" O8A CS8"; $resn_exch{" AO9 CS8"}=" O9A CS8"; $resn_exch{" C3* CSF"}=" C3' CSF";
$resn_exch{" C2* CSF"}=" C2' CSF"; $resn_exch{" C1* CSF"}=" C1' CSF"; $resn_exch{" O4* CSF"}=" O4' CSF";
$resn_exch{" C4* CSF"}=" C4' CSF"; $resn_exch{" O5* CSF"}=" O5' CSF"; $resn_exch{" C5* CSF"}=" C5' CSF";
$resn_exch{" O3* CSF"}=" O3' CSF"; $resn_exch{" O2* CSF"}=" O2' CSF"; $resn_exch{" O5* CSL"}=" O5' CSL";
$resn_exch{" C5* CSL"}=" C5' CSL"; $resn_exch{" C4* CSL"}=" C4' CSL"; $resn_exch{" O4* CSL"}=" O4' CSL";
$resn_exch{" C1* CSL"}=" C1' CSL"; $resn_exch{" C2* CSL"}=" C2' CSL"; $resn_exch{"SE2* CSL"}="SE2' CSL";
$resn_exch{" CA* CSL"}=" CA' CSL"; $resn_exch{" C3* CSL"}=" C3' CSL"; $resn_exch{" O3* CSL"}=" O3' CSL";
$resn_exch{" C4\" CTC"}=" C4D CTC"; $resn_exch{" C1* CTD"}=" C1' CTD"; $resn_exch{" C2* CTD"}=" C2' CTD";
$resn_exch{" O2* CTD"}=" O2' CTD"; $resn_exch{" C3* CTD"}=" C3' CTD"; $resn_exch{" C4* CTD"}=" C4' CTD";
$resn_exch{" O4* CTD"}=" O4' CTD"; $resn_exch{" O3* CTD"}=" O3' CTD"; $resn_exch{" C5* CTD"}=" C5' CTD";
$resn_exch{" O5* CTD"}=" O5' CTD"; $resn_exch{" O1P CTG"}=" OP1 CTG"; $resn_exch{" O2P CTG"}=" OP2 CTG";
$resn_exch{" O5* CTG"}=" O5' CTG"; $resn_exch{" C5* CTG"}=" C5' CTG"; $resn_exch{" C4* CTG"}=" C4' CTG";
$resn_exch{" O4* CTG"}=" O4' CTG"; $resn_exch{" C1* CTG"}=" C1' CTG"; $resn_exch{" C2* CTG"}=" C2' CTG";
$resn_exch{" C3* CTG"}=" C3' CTG"; $resn_exch{" O3* CTG"}=" O3' CTG"; $resn_exch{" O3P CTG"}=" OP3 CTG";
$resn_exch{" O5* CTN"}=" O5' CTN"; $resn_exch{" C5* CTN"}=" C5' CTN"; $resn_exch{" C4* CTN"}=" C4' CTN";
$resn_exch{" O4* CTN"}=" O4' CTN"; $resn_exch{" C1* CTN"}=" C1' CTN"; $resn_exch{" C2* CTN"}=" C2' CTN";
$resn_exch{" O2* CTN"}=" O2' CTN"; $resn_exch{" C3* CTN"}=" C3' CTN"; $resn_exch{" O3* CTN"}=" O3' CTN";
$resn_exch{" C1* CTP"}=" C1' CTP"; $resn_exch{" C2* CTP"}=" C2' CTP"; $resn_exch{" O2* CTP"}=" O2' CTP";
$resn_exch{" C3* CTP"}=" C3' CTP"; $resn_exch{" C4* CTP"}=" C4' CTP"; $resn_exch{" O4* CTP"}=" O4' CTP";
$resn_exch{" O3* CTP"}=" O3' CTP"; $resn_exch{" C5* CTP"}=" C5' CTP"; $resn_exch{" O5* CTP"}=" O5' CTP";
$resn_exch{" O1  CXM"}=" ON1 CXM"; $resn_exch{" O2  CXM"}=" ON2 CXM"; $resn_exch{"RO5* CXR"}=" O5D CXR";
$resn_exch{"RC5* CXR"}=" C5D CXR"; $resn_exch{"RC4* CXR"}=" C4D CXR"; $resn_exch{"RC3* CXR"}=" C3D CXR";
$resn_exch{"RO3* CXR"}=" O3D CXR"; $resn_exch{"RC2* CXR"}=" C2D CXR"; $resn_exch{"RO2* CXR"}=" O2D CXR";
$resn_exch{"RO4* CXR"}=" O4D CXR"; $resn_exch{"RC1* CXR"}=" C1D CXR"; $resn_exch{" C1* CXR"}=" C1' CXR";
$resn_exch{" O4* CXR"}=" O4' CXR"; $resn_exch{" C2* CXR"}=" C2' CXR"; $resn_exch{" O2* CXR"}=" O2' CXR";
$resn_exch{" C3* CXR"}=" C3' CXR"; $resn_exch{" O3* CXR"}=" O3' CXR"; $resn_exch{" C4* CXR"}=" C4' CXR";
$resn_exch{" C5* CXR"}=" C5' CXR"; $resn_exch{" O5* CXR"}=" O5' CXR"; $resn_exch{" O5* CXY"}=" O5D CXY";
$resn_exch{" C5* CXY"}=" C5D CXY"; $resn_exch{" C4* CXY"}=" C4D CXY"; $resn_exch{" O4* CXY"}=" O4D CXY";
$resn_exch{" C3* CXY"}=" C3D CXY"; $resn_exch{" O3* CXY"}=" O3D CXY"; $resn_exch{" C2* CXY"}=" C2D CXY";
$resn_exch{" O2* CXY"}=" O2D CXY"; $resn_exch{" C1* CXY"}=" C1D CXY"; $resn_exch{" N_A CYB"}=" NA  CYB";
$resn_exch{" N_B CYB"}=" NB  CYB"; $resn_exch{" O_B CYB"}=" OB  CYB"; $resn_exch{" N_C CYB"}=" NC  CYB";
$resn_exch{" O_C CYB"}=" OC  CYB"; $resn_exch{" N_D CYB"}=" ND  CYB"; $resn_exch{" N_A CYC"}=" NA  CYC";
$resn_exch{" N_B CYC"}=" NB  CYC"; $resn_exch{" O_B CYC"}=" OB  CYC"; $resn_exch{" N_C CYC"}=" NC  CYC";
$resn_exch{" O_C CYC"}=" OC  CYC"; $resn_exch{" N_D CYC"}=" ND  CYC"; $resn_exch{"4CL1 D05"}="CL14 D05";
$resn_exch{"6CL1 D05"}="CL16 D05"; $resn_exch{"2CL1 D05"}="CL12 D05"; $resn_exch{"'C10 D18"}=" CAX D18";
$resn_exch{"'C10 D19"}=" CAX D19"; $resn_exch{" O3P D1P"}=" OP3 D1P"; $resn_exch{" O1P D1P"}=" OP1 D1P";
$resn_exch{" O2P D1P"}=" OP2 D1P"; $resn_exch{" O5* D1P"}=" O5' D1P"; $resn_exch{" C5* D1P"}=" C5' D1P";
$resn_exch{" C4* D1P"}=" C4' D1P"; $resn_exch{" O4* D1P"}=" O4' D1P"; $resn_exch{" C3* D1P"}=" C3' D1P";
$resn_exch{" O3* D1P"}=" O3' D1P"; $resn_exch{" C2* D1P"}=" C2' D1P"; $resn_exch{" C1* D1P"}=" C1' D1P";
$resn_exch{" O1* D1P"}=" O1' D1P"; $resn_exch{"C10' D24"}=" CAX D24"; $resn_exch{"C11' D24"}=" CBX D24";
$resn_exch{"C12' D24"}=" CCX D24"; $resn_exch{" O1P  D3"}=" OP1  D3"; $resn_exch{" O2P  D3"}=" OP2  D3";
$resn_exch{" O3P  D3"}=" OP3  D3"; $resn_exch{" O5*  D3"}=" O5'  D3"; $resn_exch{" C5*  D3"}=" C5'  D3";
$resn_exch{" C4*  D3"}=" C4'  D3"; $resn_exch{" O4*  D3"}=" O4'  D3"; $resn_exch{" C3*  D3"}=" C3'  D3";
$resn_exch{" O3*  D3"}=" O3'  D3"; $resn_exch{" C2*  D3"}=" C2'  D3"; $resn_exch{" C1*  D3"}=" C1'  D3";
$resn_exch{"C10' D34"}=" CAX D34"; $resn_exch{"C11' D34"}=" CBX D34"; $resn_exch{"C10' D35"}=" CAX D35";
$resn_exch{"C11' D35"}=" CBX D35"; $resn_exch{"C12' D35"}=" CCX D35"; $resn_exch{"C13' D35"}=" CDX D35";
$resn_exch{" O5* D3T"}=" O5' D3T"; $resn_exch{" C5* D3T"}=" C5' D3T"; $resn_exch{" C4* D3T"}=" C4' D3T";
$resn_exch{" O4* D3T"}=" O4' D3T"; $resn_exch{" C1* D3T"}=" C1' D3T"; $resn_exch{" C2* D3T"}=" C2' D3T";
$resn_exch{" C3* D3T"}=" C3' D3T"; $resn_exch{" O2P D4M"}=" OP2 D4M"; $resn_exch{" O1P D4M"}=" OP1 D4M";
$resn_exch{" O3P D4M"}=" OP3 D4M"; $resn_exch{" O5* D4M"}=" O5' D4M"; $resn_exch{" C5* D4M"}=" C5' D4M";
$resn_exch{" C4* D4M"}=" C4' D4M"; $resn_exch{" O4* D4M"}=" O4' D4M"; $resn_exch{" C3* D4M"}=" C3' D4M";
$resn_exch{" C2* D4M"}=" C2' D4M"; $resn_exch{" C1* D4M"}=" C1' D4M"; $resn_exch{" O5* D5M"}=" O5' D5M";
$resn_exch{" C5* D5M"}=" C5' D5M"; $resn_exch{" C4* D5M"}=" C4' D5M"; $resn_exch{" O4* D5M"}=" O4' D5M";
$resn_exch{" C3* D5M"}=" C3' D5M"; $resn_exch{" O3* D5M"}=" O3' D5M"; $resn_exch{" C2* D5M"}=" C2' D5M";
$resn_exch{" C1* D5M"}=" C1' D5M"; $resn_exch{" O3P  DA"}=" OP3  DA"; $resn_exch{" O1P  DA"}=" OP1  DA";
$resn_exch{" O2P  DA"}=" OP2  DA"; $resn_exch{" O5*  DA"}=" O5'  DA"; $resn_exch{" C5*  DA"}=" C5'  DA";
$resn_exch{" C4*  DA"}=" C4'  DA"; $resn_exch{" O4*  DA"}=" O4'  DA"; $resn_exch{" C3*  DA"}=" C3'  DA";
$resn_exch{" O3*  DA"}=" O3'  DA"; $resn_exch{" C2*  DA"}=" C2'  DA"; $resn_exch{" C1*  DA"}=" C1'  DA";
$resn_exch{" O5* DAD"}=" O5' DAD"; $resn_exch{" C5* DAD"}=" C5' DAD"; $resn_exch{" C4* DAD"}=" C4' DAD";
$resn_exch{" O4* DAD"}=" O4' DAD"; $resn_exch{" C3* DAD"}=" C3' DAD"; $resn_exch{" C2* DAD"}=" C2' DAD";
$resn_exch{" C1* DAD"}=" C1' DAD"; $resn_exch{" C1* DAP"}=" C1' DAP"; $resn_exch{" C2* DAP"}=" C2' DAP";
$resn_exch{" C3* DAP"}=" C3' DAP"; $resn_exch{" C4* DAP"}=" C4' DAP"; $resn_exch{" C5* DAP"}=" C5' DAP";
$resn_exch{" C6* DAP"}=" C6' DAP"; $resn_exch{" O5* DAT"}=" O5' DAT"; $resn_exch{" C5* DAT"}=" C5' DAT";
$resn_exch{" C4* DAT"}=" C4' DAT"; $resn_exch{" O4* DAT"}=" O4' DAT"; $resn_exch{" C3* DAT"}=" C3' DAT";
$resn_exch{" O3* DAT"}=" O3' DAT"; $resn_exch{" C2* DAT"}=" C2' DAT"; $resn_exch{" C1* DAT"}=" C1' DAT";
$resn_exch{" O5* DBG"}=" O5' DBG"; $resn_exch{" C5* DBG"}=" C5' DBG"; $resn_exch{" C4* DBG"}=" C4' DBG";
$resn_exch{" O4* DBG"}=" O4' DBG"; $resn_exch{" C2* DBG"}=" C2' DBG"; $resn_exch{" O2* DBG"}=" O2' DBG";
$resn_exch{" C3* DBG"}=" C3' DBG"; $resn_exch{" O3* DBG"}=" O3' DBG"; $resn_exch{" C1* DBG"}=" C1' DBG";
$resn_exch{" O1\" DBL"}=" O1' DBL"; $resn_exch{" C2\" DBL"}=" C2B DBL"; $resn_exch{" O3P  DC"}=" OP3  DC";
$resn_exch{" O1P  DC"}=" OP1  DC"; $resn_exch{" O2P  DC"}=" OP2  DC"; $resn_exch{" O5*  DC"}=" O5'  DC";
$resn_exch{" C5*  DC"}=" C5'  DC"; $resn_exch{" C4*  DC"}=" C4'  DC"; $resn_exch{" O4*  DC"}=" O4'  DC";
$resn_exch{" C3*  DC"}=" C3'  DC"; $resn_exch{" O3*  DC"}=" O3'  DC"; $resn_exch{" C2*  DC"}=" C2'  DC";
$resn_exch{" C1*  DC"}=" C1'  DC"; $resn_exch{" AN1 DCA"}=" N1A DCA"; $resn_exch{" AC2 DCA"}=" C2A DCA";
$resn_exch{" AN3 DCA"}=" N3A DCA"; $resn_exch{" AC4 DCA"}=" C4A DCA"; $resn_exch{" AC5 DCA"}=" C5A DCA";
$resn_exch{" AC6 DCA"}=" C6A DCA"; $resn_exch{" AN6 DCA"}=" N6A DCA"; $resn_exch{" AN7 DCA"}=" N7A DCA";
$resn_exch{" AC8 DCA"}=" C8A DCA"; $resn_exch{" AN9 DCA"}=" N9A DCA"; $resn_exch{"AC1* DCA"}=" C1B DCA";
$resn_exch{"AC2* DCA"}=" C2B DCA"; $resn_exch{"AO2* DCA"}=" O2B DCA"; $resn_exch{"AC3* DCA"}=" C3B DCA";
$resn_exch{"AO3* DCA"}=" O3B DCA"; $resn_exch{"AP3* DCA"}=" P3B DCA"; $resn_exch{" AO7 DCA"}=" O7A DCA";
$resn_exch{" AO8 DCA"}=" O8A DCA"; $resn_exch{" AO9 DCA"}=" O9A DCA"; $resn_exch{"AC4* DCA"}=" C4B DCA";
$resn_exch{"AO4* DCA"}=" O4B DCA"; $resn_exch{"AC5* DCA"}=" C5B DCA"; $resn_exch{"AO5* DCA"}=" O5B DCA";
$resn_exch{" AP1 DCA"}=" P1A DCA"; $resn_exch{" AO1 DCA"}=" O1A DCA"; $resn_exch{" AO2 DCA"}=" O2A DCA";
$resn_exch{" AO3 DCA"}=" O3A DCA"; $resn_exch{" AP2 DCA"}=" P2A DCA"; $resn_exch{" AO4 DCA"}=" O4A DCA";
$resn_exch{" AO5 DCA"}=" O5A DCA"; $resn_exch{" AO6 DCA"}=" O6A DCA"; $resn_exch{"PC11 DCA"}=" CBP DCA";
$resn_exch{"PC12 DCA"}=" CCP DCA"; $resn_exch{"PC13 DCA"}=" CDP DCA"; $resn_exch{"PC14 DCA"}=" CEP DCA";
$resn_exch{"PC10 DCA"}=" CAP DCA"; $resn_exch{"PO10 DCA"}=" OAP DCA"; $resn_exch{" PC9 DCA"}=" C9P DCA";
$resn_exch{" PO9 DCA"}=" O9P DCA"; $resn_exch{" PN8 DCA"}=" N8P DCA"; $resn_exch{" PC7 DCA"}=" C7P DCA";
$resn_exch{" PC6 DCA"}=" C6P DCA"; $resn_exch{" PC5 DCA"}=" C5P DCA"; $resn_exch{" PO5 DCA"}=" O5P DCA";
$resn_exch{" PN4 DCA"}=" N4P DCA"; $resn_exch{" PC3 DCA"}=" C3P DCA"; $resn_exch{" PC2 DCA"}=" C2P DCA";
$resn_exch{" O1P DCG"}=" OP1 DCG"; $resn_exch{" O2P DCG"}=" OP2 DCG"; $resn_exch{" O3P DCG"}=" OP3 DCG";
$resn_exch{" O5* DCG"}=" O5' DCG"; $resn_exch{" C5* DCG"}=" C5' DCG"; $resn_exch{" C4* DCG"}=" C4' DCG";
$resn_exch{" O4* DCG"}=" O4' DCG"; $resn_exch{" C3* DCG"}=" C3' DCG"; $resn_exch{" O3* DCG"}=" O3' DCG";
$resn_exch{" C2* DCG"}=" C2' DCG"; $resn_exch{" C1* DCG"}=" C1' DCG"; $resn_exch{" C1* DCM"}=" C1' DCM";
$resn_exch{" C2* DCM"}=" C2' DCM"; $resn_exch{" C3* DCM"}=" C3' DCM"; $resn_exch{" C4* DCM"}=" C4' DCM";
$resn_exch{" O4* DCM"}=" O4' DCM"; $resn_exch{" O3* DCM"}=" O3' DCM"; $resn_exch{" C5* DCM"}=" C5' DCM";
$resn_exch{" O5* DCM"}=" O5' DCM"; $resn_exch{" C1* DCP"}=" C1' DCP"; $resn_exch{" C2* DCP"}=" C2' DCP";
$resn_exch{" C3* DCP"}=" C3' DCP"; $resn_exch{" C4* DCP"}=" C4' DCP"; $resn_exch{" O4* DCP"}=" O4' DCP";
$resn_exch{" O3* DCP"}=" O3' DCP"; $resn_exch{" C5* DCP"}=" C5' DCP"; $resn_exch{" O5* DCP"}=" O5' DCP";
$resn_exch{" C1* DCT"}=" C1' DCT"; $resn_exch{" C2* DCT"}=" C2' DCT"; $resn_exch{" C3* DCT"}=" C3' DCT";
$resn_exch{" C4* DCT"}=" C4' DCT"; $resn_exch{" O4* DCT"}=" O4' DCT"; $resn_exch{" C5* DCT"}=" C5' DCT";
$resn_exch{" O5* DCT"}=" O5' DCT"; $resn_exch{" C1* DCZ"}=" C1' DCZ"; $resn_exch{" C2* DCZ"}=" C2' DCZ";
$resn_exch{" C3* DCZ"}=" C3' DCZ"; $resn_exch{" C4* DCZ"}=" C4' DCZ"; $resn_exch{" O4* DCZ"}=" O4' DCZ";
$resn_exch{" O3* DCZ"}=" O3' DCZ"; $resn_exch{" C5* DCZ"}=" C5' DCZ"; $resn_exch{" O5* DCZ"}=" O5' DCZ";
$resn_exch{" O3P DDG"}=" OP3 DDG"; $resn_exch{" O1P DDG"}=" OP1 DDG"; $resn_exch{" O2P DDG"}=" OP2 DDG";
$resn_exch{" O5* DDG"}=" O5' DDG"; $resn_exch{" C5* DDG"}=" C5' DDG"; $resn_exch{" C4* DDG"}=" C4' DDG";
$resn_exch{" O4* DDG"}=" O4' DDG"; $resn_exch{" C3* DDG"}=" C3' DDG"; $resn_exch{" C2* DDG"}=" C2' DDG";
$resn_exch{" C1* DDG"}=" C1' DDG"; $resn_exch{" N_A DDH"}=" NA  DDH"; $resn_exch{" N_B DDH"}=" NB  DDH";
$resn_exch{" N_C DDH"}=" NC  DDH"; $resn_exch{" N_D DDH"}=" ND  DDH"; $resn_exch{" O3P DDN"}=" OP3 DDN";
$resn_exch{" O1P DDN"}=" OP1 DDN"; $resn_exch{" O2P DDN"}=" OP2 DDN"; $resn_exch{" O5* DDN"}=" O5' DDN";
$resn_exch{" C5* DDN"}=" C5' DDN"; $resn_exch{" C4* DDN"}=" C4' DDN"; $resn_exch{" O4* DDN"}=" O4' DDN";
$resn_exch{" C3* DDN"}=" C3' DDN"; $resn_exch{" O3* DDN"}=" O3' DDN"; $resn_exch{" C2* DDN"}=" C2' DDN";
$resn_exch{" C1* DDN"}=" C1' DDN"; $resn_exch{" S*  DDT"}=" S'  DDT"; $resn_exch{" O1* DDT"}=" O1' DDT";
$resn_exch{" O2* DDT"}=" O2' DDT"; $resn_exch{" C1* DDT"}=" C1' DDT"; $resn_exch{" C2* DDT"}=" C2' DDT";
$resn_exch{" C3* DDT"}=" C3' DDT"; $resn_exch{" C4* DDT"}=" C4' DDT"; $resn_exch{"C4A* DDT"}=" C4X DDT";
$resn_exch{" C5* DDT"}=" C5' DDT"; $resn_exch{" C6* DDT"}=" C6' DDT"; $resn_exch{" C7* DDT"}=" C7' DDT";
$resn_exch{" C8* DDT"}=" C8' DDT"; $resn_exch{"C8A* DDT"}=" C8X DDT"; $resn_exch{" C9* DDT"}=" C9' DDT";
$resn_exch{"C10* DDT"}=" CA' DDT"; $resn_exch{" N1* DDT"}=" N1' DDT"; $resn_exch{" C1* DDU"}=" C1' DDU";
$resn_exch{" C2* DDU"}=" C2' DDU"; $resn_exch{" C3* DDU"}=" C3' DDU"; $resn_exch{" C4* DDU"}=" C4' DDU";
$resn_exch{" O3* DDU"}=" O3' DDU"; $resn_exch{" O4* DDU"}=" O4' DDU"; $resn_exch{" C5* DDU"}=" C5' DDU";
$resn_exch{" O1P DDX"}=" OP1 DDX"; $resn_exch{" O2P DDX"}=" OP2 DDX"; $resn_exch{" O3P DDX"}=" OP3 DDX";
$resn_exch{" O5* DDX"}=" O5' DDX"; $resn_exch{" C5* DDX"}=" C5' DDX"; $resn_exch{" C4* DDX"}=" C4' DDX";
$resn_exch{" O4* DDX"}=" O4' DDX"; $resn_exch{" C1* DDX"}=" C1' DDX"; $resn_exch{" C3* DDX"}=" C3' DDX";
$resn_exch{" C2* DDX"}=" C2' DDX"; $resn_exch{" O1* DDX"}=" O1' DDX"; $resn_exch{" O5* DDY"}=" O5' DDY";
$resn_exch{" C5* DDY"}=" C5' DDY"; $resn_exch{" C4* DDY"}=" C4' DDY"; $resn_exch{" O4* DDY"}=" O4' DDY";
$resn_exch{" C3* DDY"}=" C3' DDY"; $resn_exch{" C2* DDY"}=" C2' DDY"; $resn_exch{" C1* DDY"}=" C1' DDY";
$resn_exch{" C1* DEA"}=" C1' DEA"; $resn_exch{" C2* DEA"}=" C2' DEA"; $resn_exch{" O2* DEA"}=" O2' DEA";
$resn_exch{" C3* DEA"}=" C3' DEA"; $resn_exch{" O3* DEA"}=" O3' DEA"; $resn_exch{" C4* DEA"}=" C4' DEA";
$resn_exch{" N_A DEU"}=" NA  DEU"; $resn_exch{" N_B DEU"}=" NB  DEU"; $resn_exch{" N_C DEU"}=" NC  DEU";
$resn_exch{" N_D DEU"}=" ND  DEU"; $resn_exch{" O3P DFC"}=" OP3 DFC"; $resn_exch{" O1P DFC"}=" OP1 DFC";
$resn_exch{" O2P DFC"}=" OP2 DFC"; $resn_exch{" O5* DFC"}=" O5' DFC"; $resn_exch{" C5* DFC"}=" C5' DFC";
$resn_exch{" C4* DFC"}=" C4' DFC"; $resn_exch{" O4* DFC"}=" O4' DFC"; $resn_exch{" C3* DFC"}=" C3' DFC";
$resn_exch{" O3* DFC"}=" O3' DFC"; $resn_exch{" C2* DFC"}=" C2' DFC"; $resn_exch{" C1* DFC"}=" C1' DFC";
$resn_exch{" O3P DFG"}=" OP3 DFG"; $resn_exch{" O1P DFG"}=" OP1 DFG"; $resn_exch{" O2P DFG"}=" OP2 DFG";
$resn_exch{" O5* DFG"}=" O5' DFG"; $resn_exch{" C5* DFG"}=" C5' DFG"; $resn_exch{" C4* DFG"}=" C4' DFG";
$resn_exch{" O4* DFG"}=" O4' DFG"; $resn_exch{" C3* DFG"}=" C3' DFG"; $resn_exch{" O3* DFG"}=" O3' DFG";
$resn_exch{" C2* DFG"}=" C2' DFG"; $resn_exch{" C1* DFG"}=" C1' DFG"; $resn_exch{" O1P DFT"}=" OP1 DFT";
$resn_exch{" O2P DFT"}=" OP2 DFT"; $resn_exch{" O3P DFT"}=" OP3 DFT"; $resn_exch{" O5* DFT"}=" O5' DFT";
$resn_exch{" C5* DFT"}=" C5' DFT"; $resn_exch{" C4* DFT"}=" C4' DFT"; $resn_exch{" O4* DFT"}=" O4' DFT";
$resn_exch{" C3* DFT"}=" C3' DFT"; $resn_exch{" O3* DFT"}=" O3' DFT"; $resn_exch{" C2* DFT"}=" C2' DFT";
$resn_exch{" C1* DFT"}=" C1' DFT"; $resn_exch{" O3P  DG"}=" OP3  DG"; $resn_exch{" O1P  DG"}=" OP1  DG";
$resn_exch{" O2P  DG"}=" OP2  DG"; $resn_exch{" O5*  DG"}=" O5'  DG"; $resn_exch{" C5*  DG"}=" C5'  DG";
$resn_exch{" C4*  DG"}=" C4'  DG"; $resn_exch{" O4*  DG"}=" O4'  DG"; $resn_exch{" C3*  DG"}=" C3'  DG";
$resn_exch{" O3*  DG"}=" O3'  DG"; $resn_exch{" C2*  DG"}=" C2'  DG"; $resn_exch{" C1*  DG"}=" C1'  DG";
$resn_exch{" C1* DG1"}=" C1' DG1"; $resn_exch{" O4* DG1"}=" O4' DG1"; $resn_exch{" C2* DG1"}=" C2' DG1";
$resn_exch{" O2* DG1"}=" O2' DG1"; $resn_exch{" C3* DG1"}=" C3' DG1"; $resn_exch{" O3* DG1"}=" O3' DG1";
$resn_exch{" C4* DG1"}=" C4' DG1"; $resn_exch{" C5* DG1"}=" C5' DG1"; $resn_exch{" O5* DG1"}=" O5' DG1";
$resn_exch{" O5* DG3"}=" O5' DG3"; $resn_exch{" C5* DG3"}=" C5' DG3"; $resn_exch{" C4* DG3"}=" C4' DG3";
$resn_exch{" O4* DG3"}=" O4' DG3"; $resn_exch{" C3* DG3"}=" C3' DG3"; $resn_exch{" C2* DG3"}=" C2' DG3";
$resn_exch{" C1* DG3"}=" C1' DG3"; $resn_exch{"AC10 DGA"}=" CAA DGA"; $resn_exch{"AC11 DGA"}=" CBA DGA";
$resn_exch{"AC12 DGA"}=" CCA DGA"; $resn_exch{"AC13 DGA"}=" CDA DGA"; $resn_exch{"AC14 DGA"}=" CEA DGA";
$resn_exch{"AC15 DGA"}=" CFA DGA"; $resn_exch{"AC16 DGA"}=" CGA DGA"; $resn_exch{"AC17 DGA"}=" CHA DGA";
$resn_exch{"AC18 DGA"}=" CIA DGA"; $resn_exch{"BC10 DGA"}=" CAB DGA"; $resn_exch{"BC11 DGA"}=" CBB DGA";
$resn_exch{"BC12 DGA"}=" CCB DGA"; $resn_exch{"BC13 DGA"}=" CDB DGA"; $resn_exch{"BC14 DGA"}=" CEB DGA";
$resn_exch{"BC15 DGA"}=" CFB DGA"; $resn_exch{"BC16 DGA"}=" CGB DGA"; $resn_exch{"BC17 DGA"}=" CHB DGA";
$resn_exch{"BC18 DGA"}=" CIB DGA"; $resn_exch{" AC1 DGD"}=" C1A DGD"; $resn_exch{" AC2 DGD"}=" C2A DGD";
$resn_exch{" AC3 DGD"}=" C3A DGD"; $resn_exch{" AC4 DGD"}=" C4A DGD"; $resn_exch{" AC5 DGD"}=" C5A DGD";
$resn_exch{" AC6 DGD"}=" C6A DGD"; $resn_exch{" AC7 DGD"}=" C7A DGD"; $resn_exch{" AC8 DGD"}=" C8A DGD";
$resn_exch{" AC9 DGD"}=" C9A DGD"; $resn_exch{"AC10 DGD"}=" CAA DGD"; $resn_exch{"AC11 DGD"}=" CBA DGD";
$resn_exch{"AC12 DGD"}=" CCA DGD"; $resn_exch{"AC13 DGD"}=" CDA DGD"; $resn_exch{"AC14 DGD"}=" CEA DGD";
$resn_exch{"AC15 DGD"}=" CFA DGD"; $resn_exch{"AC16 DGD"}=" CGA DGD"; $resn_exch{"AC17 DGD"}=" CHA DGD";
$resn_exch{"AC18 DGD"}=" CIA DGD"; $resn_exch{" AO1 DGD"}=" O1A DGD"; $resn_exch{" BC1 DGD"}=" C1B DGD";
$resn_exch{" BC2 DGD"}=" C2B DGD"; $resn_exch{" BC3 DGD"}=" C3B DGD"; $resn_exch{" BC4 DGD"}=" C4B DGD";
$resn_exch{" BC5 DGD"}=" C5B DGD"; $resn_exch{" BC6 DGD"}=" C6B DGD"; $resn_exch{" BC7 DGD"}=" C7B DGD";
$resn_exch{" BC8 DGD"}=" C8B DGD"; $resn_exch{" BC9 DGD"}=" C9B DGD"; $resn_exch{"BC10 DGD"}=" CAB DGD";
$resn_exch{"BC11 DGD"}=" CBB DGD"; $resn_exch{"BC12 DGD"}=" CCB DGD"; $resn_exch{"BC13 DGD"}=" CDB DGD";
$resn_exch{"BC14 DGD"}=" CEB DGD"; $resn_exch{"BC15 DGD"}=" CFB DGD"; $resn_exch{"BC16 DGD"}=" CGB DGD";
$resn_exch{"BC17 DGD"}=" CHB DGD"; $resn_exch{"BC18 DGD"}=" CIB DGD"; $resn_exch{" BO1 DGD"}=" O1B DGD";
$resn_exch{" GO1 DGD"}=" O1G DGD"; $resn_exch{" GC1 DGD"}=" C1G DGD"; $resn_exch{" GC2 DGD"}=" C2G DGD";
$resn_exch{" GO2 DGD"}=" O2G DGD"; $resn_exch{" GC3 DGD"}=" C3G DGD"; $resn_exch{" GO3 DGD"}=" O3G DGD";
$resn_exch{" DC1 DGD"}=" C1D DGD"; $resn_exch{" DC2 DGD"}=" C2D DGD"; $resn_exch{" DO2 DGD"}=" O2D DGD";
$resn_exch{" DC3 DGD"}=" C3D DGD"; $resn_exch{" DO3 DGD"}=" O3D DGD"; $resn_exch{" DC4 DGD"}=" C4D DGD";
$resn_exch{" DO4 DGD"}=" O4D DGD"; $resn_exch{" DC5 DGD"}=" C5D DGD"; $resn_exch{" DO5 DGD"}=" O5D DGD";
$resn_exch{" DC6 DGD"}=" C6D DGD"; $resn_exch{" DO6 DGD"}=" O6D DGD"; $resn_exch{" EC1 DGD"}=" C1E DGD";
$resn_exch{" EC2 DGD"}=" C2E DGD"; $resn_exch{" EO2 DGD"}=" O2E DGD"; $resn_exch{" EC3 DGD"}=" C3E DGD";
$resn_exch{" EO3 DGD"}=" O3E DGD"; $resn_exch{" EC4 DGD"}=" C4E DGD"; $resn_exch{" EO4 DGD"}=" O4E DGD";
$resn_exch{" EC5 DGD"}=" C5E DGD"; $resn_exch{" EO6 DGD"}=" O6E DGD"; $resn_exch{" EC6 DGD"}=" C6E DGD";
$resn_exch{" EO5 DGD"}=" O5E DGD"; $resn_exch{" O5* DGI"}=" O5' DGI"; $resn_exch{" C5* DGI"}=" C5' DGI";
$resn_exch{" C4* DGI"}=" C4' DGI"; $resn_exch{" O4* DGI"}=" O4' DGI"; $resn_exch{" C3* DGI"}=" C3' DGI";
$resn_exch{" O3* DGI"}=" O3' DGI"; $resn_exch{" C2* DGI"}=" C2' DGI"; $resn_exch{" C1* DGI"}=" C1' DGI";
$resn_exch{" O1P DGP"}=" OP1 DGP"; $resn_exch{" O2P DGP"}=" OP2 DGP"; $resn_exch{" O3P DGP"}=" OP3 DGP";
$resn_exch{" O5* DGP"}=" O5' DGP"; $resn_exch{" C5* DGP"}=" C5' DGP"; $resn_exch{" C4* DGP"}=" C4' DGP";
$resn_exch{" O4* DGP"}=" O4' DGP"; $resn_exch{" C3* DGP"}=" C3' DGP"; $resn_exch{" O3* DGP"}=" O3' DGP";
$resn_exch{" C2* DGP"}=" C2' DGP"; $resn_exch{" C1* DGP"}=" C1' DGP"; $resn_exch{" O5* DGT"}=" O5' DGT";
$resn_exch{" C5* DGT"}=" C5' DGT"; $resn_exch{" C4* DGT"}=" C4' DGT"; $resn_exch{" O4* DGT"}=" O4' DGT";
$resn_exch{" C3* DGT"}=" C3' DGT"; $resn_exch{" O3* DGT"}=" O3' DGT"; $resn_exch{" C2* DGT"}=" C2' DGT";
$resn_exch{" C1* DGT"}=" C1' DGT"; $resn_exch{" C5* DH3"}=" C5' DH3"; $resn_exch{" C4* DH3"}=" C4B DH3";
$resn_exch{" C2* DH9"}=" C2' DH9"; $resn_exch{" C1* DH9"}=" C1' DH9"; $resn_exch{" O1* DH9"}=" O1' DH9";
$resn_exch{" O2* DH9"}=" O2' DH9"; $resn_exch{" C3* DH9"}=" C3' DH9"; $resn_exch{" O3* DH9"}=" O3' DH9";
$resn_exch{" C4* DH9"}=" C4' DH9"; $resn_exch{" C5* DH9"}=" C5' DH9"; $resn_exch{" O4* DH9"}=" O4' DH9";
$resn_exch{" O5* DH9"}=" O5' DH9"; $resn_exch{" N1* DH9"}=" N1' DH9"; $resn_exch{" O6* DH9"}=" O6' DH9";
$resn_exch{" C6* DH9"}=" C6' DH9"; $resn_exch{" C7* DH9"}=" C7' DH9"; $resn_exch{" C8* DH9"}=" C8' DH9";
$resn_exch{" C9* DH9"}=" C9' DH9"; $resn_exch{" N_A DHE"}=" NA  DHE"; $resn_exch{" N_B DHE"}=" NB  DHE";
$resn_exch{" N_C DHE"}=" NC  DHE"; $resn_exch{" N_D DHE"}=" ND  DHE"; $resn_exch{" O1P DHU"}=" OP1 DHU";
$resn_exch{" O2P DHU"}=" OP2 DHU"; $resn_exch{" O3P DHU"}=" OP3 DHU"; $resn_exch{" O5* DHU"}=" O5' DHU";
$resn_exch{" C5* DHU"}=" C5' DHU"; $resn_exch{" C4* DHU"}=" C4' DHU"; $resn_exch{" O4* DHU"}=" O4' DHU";
$resn_exch{" C3* DHU"}=" C3' DHU"; $resn_exch{" O3* DHU"}=" O3' DHU"; $resn_exch{" C2* DHU"}=" C2' DHU";
$resn_exch{" O2* DHU"}=" O2' DHU"; $resn_exch{" C1* DHU"}=" C1' DHU"; $resn_exch{" O3P  DI"}=" OP3  DI";
$resn_exch{" O1P  DI"}=" OP1  DI"; $resn_exch{" O2P  DI"}=" OP2  DI"; $resn_exch{" O5*  DI"}=" O5'  DI";
$resn_exch{" C5*  DI"}=" C5'  DI"; $resn_exch{" C4*  DI"}=" C4'  DI"; $resn_exch{" O4*  DI"}=" O4'  DI";
$resn_exch{" C3*  DI"}=" C3'  DI"; $resn_exch{" O3*  DI"}=" O3'  DI"; $resn_exch{" C2*  DI"}=" C2'  DI";
$resn_exch{" C1*  DI"}=" C1'  DI"; $resn_exch{" O5* DIH"}=" O5' DIH"; $resn_exch{" C5* DIH"}=" C5' DIH";
$resn_exch{" C4* DIH"}=" C4' DIH"; $resn_exch{" C6* DIH"}=" C6' DIH"; $resn_exch{" C3* DIH"}=" C3' DIH";
$resn_exch{" O3* DIH"}=" O3' DIH"; $resn_exch{" C2* DIH"}=" C2' DIH"; $resn_exch{" N1* DIH"}=" N1' DIH";
$resn_exch{" XC1 DIT"}=" C1X DIT"; $resn_exch{" XN2 DIT"}=" N2X DIT"; $resn_exch{" XC3 DIT"}=" C3X DIT";
$resn_exch{" XC4 DIT"}=" C4X DIT"; $resn_exch{" XC5 DIT"}=" C5X DIT"; $resn_exch{" XC6 DIT"}=" C6X DIT";
$resn_exch{" XN7 DIT"}=" N7X DIT"; $resn_exch{" XC8 DIT"}=" C8X DIT"; $resn_exch{" XC9 DIT"}=" C9X DIT";
$resn_exch{"XC10 DIT"}=" CAX DIT"; $resn_exch{"XO10 DIT"}=" OAX DIT"; $resn_exch{"XC11 DIT"}=" CBX DIT";
$resn_exch{"XC12 DIT"}=" CCX DIT"; $resn_exch{"XC13 DIT"}=" CDX DIT"; $resn_exch{"XC14 DIT"}=" CEX DIT";
$resn_exch{"XC15 DIT"}=" CFX DIT"; $resn_exch{"XC16 DIT"}=" CGX DIT"; $resn_exch{"XC17 DIT"}=" CHX DIT";
$resn_exch{"XC18 DIT"}=" CIX DIT"; $resn_exch{"XC19 DIT"}=" CJX DIT"; $resn_exch{"XC20 DIT"}=" CKX DIT";
$resn_exch{"XN21 DIT"}=" NLX DIT"; $resn_exch{"XC22 DIT"}=" CMX DIT"; $resn_exch{"XC23 DIT"}=" CNX DIT";
$resn_exch{"XC24 DIT"}=" COX DIT"; $resn_exch{"XC25 DIT"}=" CPX DIT"; $resn_exch{"XC26 DIT"}=" CQX DIT";
$resn_exch{" C1* DM1"}=" C1' DM1"; $resn_exch{" C2* DM1"}=" C2' DM1"; $resn_exch{" C3* DM1"}=" C3' DM1";
$resn_exch{" N3* DM1"}=" N3' DM1"; $resn_exch{" C4* DM1"}=" C4' DM1"; $resn_exch{" O4* DM1"}=" O4' DM1";
$resn_exch{" C5* DM1"}=" C5' DM1"; $resn_exch{" O5* DM1"}=" O5' DM1"; $resn_exch{" C6* DM1"}=" C6' DM1";
$resn_exch{" C1* DM2"}=" C1' DM2"; $resn_exch{" C2* DM2"}=" C2' DM2"; $resn_exch{" C3* DM2"}=" C3' DM2";
$resn_exch{" N3* DM2"}=" N3' DM2"; $resn_exch{" C4* DM2"}=" C4' DM2"; $resn_exch{" O4* DM2"}=" O4' DM2";
$resn_exch{" C5* DM2"}=" C5' DM2"; $resn_exch{" O5* DM2"}=" O5' DM2"; $resn_exch{" C6* DM2"}=" C6' DM2";
$resn_exch{" C1* DM3"}=" C1' DM3"; $resn_exch{" C2* DM3"}=" C2' DM3"; $resn_exch{" C3* DM3"}=" C3' DM3";
$resn_exch{" N3* DM3"}=" N3' DM3"; $resn_exch{" C4* DM3"}=" C4' DM3"; $resn_exch{" O4* DM3"}=" O4' DM3";
$resn_exch{" C5* DM3"}=" C5' DM3"; $resn_exch{" O5* DM3"}=" O5' DM3"; $resn_exch{" C6* DM3"}=" C6' DM3";
$resn_exch{" C1* DM4"}=" C1' DM4"; $resn_exch{" C2* DM4"}=" C2' DM4"; $resn_exch{" C3* DM4"}=" C3' DM4";
$resn_exch{" N3* DM4"}=" N3' DM4"; $resn_exch{" C4* DM4"}=" C4' DM4"; $resn_exch{" O4* DM4"}=" O4' DM4";
$resn_exch{" C5* DM4"}=" C5' DM4"; $resn_exch{" O5* DM4"}=" O5' DM4"; $resn_exch{" C6* DM4"}=" C6' DM4";
$resn_exch{" C1* DM5"}=" C1' DM5"; $resn_exch{" C2* DM5"}=" C2' DM5"; $resn_exch{" C3* DM5"}=" C3' DM5";
$resn_exch{" N3* DM5"}=" N3' DM5"; $resn_exch{" C4* DM5"}=" C4' DM5"; $resn_exch{" O4* DM5"}=" O4' DM5";
$resn_exch{" C5* DM5"}=" C5' DM5"; $resn_exch{" O5* DM5"}=" O5' DM5"; $resn_exch{" C6* DM5"}=" C6' DM5";
$resn_exch{" C1* DM6"}=" C1' DM6"; $resn_exch{" C2* DM6"}=" C2' DM6"; $resn_exch{" C3* DM6"}=" C3' DM6";
$resn_exch{" C4* DM6"}=" C4' DM6"; $resn_exch{" C5* DM6"}=" C5' DM6"; $resn_exch{" C6* DM6"}=" C6' DM6";
$resn_exch{" O4* DM6"}=" O4' DM6"; $resn_exch{" O5* DM6"}=" O5' DM6"; $resn_exch{" N3* DM6"}=" N3' DM6";
$resn_exch{" C1* DM7"}=" C1' DM7"; $resn_exch{" C2* DM7"}=" C2' DM7"; $resn_exch{" C3* DM7"}=" C3' DM7";
$resn_exch{" C4* DM7"}=" C4' DM7"; $resn_exch{" C5* DM7"}=" C5' DM7"; $resn_exch{" C6* DM7"}=" C6' DM7";
$resn_exch{" O5* DM7"}=" O5' DM7"; $resn_exch{" N3* DM7"}=" N3' DM7"; $resn_exch{" I4* DM7"}=" I4' DM7";
$resn_exch{" C1* DM8"}=" C1' DM8"; $resn_exch{" C2* DM8"}=" C2' DM8"; $resn_exch{" C3* DM8"}=" C3' DM8";
$resn_exch{" C4* DM8"}=" C4' DM8"; $resn_exch{" C5* DM8"}=" C5' DM8"; $resn_exch{" C6* DM8"}=" C6' DM8";
$resn_exch{" O5* DM8"}=" O5' DM8"; $resn_exch{" O4* DM8"}=" O4' DM8"; $resn_exch{" N3* DM8"}=" N3' DM8";
$resn_exch{" C1* DM9"}=" C1' DM9"; $resn_exch{" C2* DM9"}=" C2' DM9"; $resn_exch{" C3* DM9"}=" C3' DM9";
$resn_exch{" C4* DM9"}=" C4' DM9"; $resn_exch{" O4* DM9"}=" O4' DM9"; $resn_exch{" C5* DM9"}=" C5' DM9";
$resn_exch{" O5* DM9"}=" O5' DM9"; $resn_exch{" C6* DM9"}=" C6' DM9"; $resn_exch{" N3* DM9"}=" N3' DM9";
$resn_exch{" O1  DMK"}=" OXT DMK"; $resn_exch{" C1* DMM"}=" C1' DMM"; $resn_exch{" C2* DMM"}=" C2' DMM";
$resn_exch{" C3* DMM"}=" C3' DMM"; $resn_exch{" C4* DMM"}=" C4' DMM"; $resn_exch{" O4* DMM"}=" O4' DMM";
$resn_exch{" C5* DMM"}=" C5' DMM"; $resn_exch{" O5* DMM"}=" O5' DMM"; $resn_exch{" C6* DMM"}=" C6' DMM";
$resn_exch{" N3* DMM"}=" N3' DMM"; $resn_exch{"O1PN DND"}=" O11 DND"; $resn_exch{"O2PN DND"}=" O12 DND";
$resn_exch{"O5'N DND"}=" O5D DND"; $resn_exch{"C5'N DND"}=" C5D DND"; $resn_exch{"C4'N DND"}=" C4D DND";
$resn_exch{"O4'N DND"}=" O4D DND"; $resn_exch{"C3'N DND"}=" C3D DND"; $resn_exch{"O3'N DND"}=" O3D DND";
$resn_exch{"C2'N DND"}=" C2D DND"; $resn_exch{"O2'N DND"}=" O2D DND"; $resn_exch{"C1'N DND"}=" C1D DND";
$resn_exch{"O1PA DND"}=" O13 DND"; $resn_exch{"O2PA DND"}=" O14 DND"; $resn_exch{"O5'A DND"}=" O5B DND";
$resn_exch{"C5'A DND"}=" C5B DND"; $resn_exch{"C4'A DND"}=" C4B DND"; $resn_exch{"O4'A DND"}=" O4B DND";
$resn_exch{"C3'A DND"}=" C3B DND"; $resn_exch{"O3'A DND"}=" O3B DND"; $resn_exch{"C2'A DND"}=" C2B DND";
$resn_exch{"O2'A DND"}=" O2B DND"; $resn_exch{"C1'A DND"}=" C1B DND"; $resn_exch{" O1  DNE"}=" OXT DNE";
$resn_exch{" O1  DNM"}=" OXT DNM"; $resn_exch{" O1P DNR"}=" OP1 DNR"; $resn_exch{" O2P DNR"}=" OP2 DNR";
$resn_exch{" O5* DNR"}=" O5' DNR"; $resn_exch{" C5* DNR"}=" C5' DNR"; $resn_exch{" C4* DNR"}=" C4' DNR";
$resn_exch{" O4* DNR"}=" O4' DNR"; $resn_exch{" C1* DNR"}=" C1' DNR"; $resn_exch{" C2* DNR"}=" C2' DNR";
$resn_exch{" C3* DNR"}=" C3' DNR"; $resn_exch{" O3* DNR"}=" O3' DNR"; $resn_exch{" O3P DNR"}=" OP3 DNR";
$resn_exch{" O   DNS"}=" OXT DNS"; $resn_exch{" OXT DNS"}=" O   DNS"; $resn_exch{" C1* DOC"}=" C1' DOC";
$resn_exch{" C2* DOC"}=" C2' DOC"; $resn_exch{" C3* DOC"}=" C3' DOC"; $resn_exch{" C4* DOC"}=" C4' DOC";
$resn_exch{" O4* DOC"}=" O4' DOC"; $resn_exch{" C5* DOC"}=" C5' DOC"; $resn_exch{" O5* DOC"}=" O5' DOC";
$resn_exch{" O1P DOC"}=" OP1 DOC"; $resn_exch{" O2P DOC"}=" OP2 DOC"; $resn_exch{" O3P DOC"}=" OP3 DOC";
$resn_exch{" O5* DOI"}=" O5' DOI"; $resn_exch{" C5* DOI"}=" C5' DOI"; $resn_exch{" C4* DOI"}=" C4' DOI";
$resn_exch{" O4* DOI"}=" O4' DOI"; $resn_exch{" C1* DOI"}=" C1' DOI"; $resn_exch{" C2* DOI"}=" C2' DOI";
$resn_exch{" C3* DOI"}=" C3' DOI"; $resn_exch{" O3* DOI"}=" O3' DOI"; $resn_exch{" O5* DOT"}=" O5' DOT";
$resn_exch{" C5* DOT"}=" C5B DOT"; $resn_exch{" C4* DOT"}=" C4B DOT"; $resn_exch{" O4* DOT"}=" O4' DOT";
$resn_exch{" C3* DOT"}=" C3B DOT"; $resn_exch{" O3* DOT"}=" O3' DOT"; $resn_exch{" C2* DOT"}=" C2B DOT";
$resn_exch{" C1* DOT"}=" C1B DOT"; $resn_exch{" C1* DPB"}=" C1' DPB"; $resn_exch{" O4* DPB"}=" O4' DPB";
$resn_exch{" C4* DPB"}=" C4' DPB"; $resn_exch{" C3* DPB"}=" C3' DPB"; $resn_exch{" C2* DPB"}=" C2' DPB";
$resn_exch{" C5* DPB"}=" C5' DPB"; $resn_exch{" O5* DPB"}=" O5' DPB"; $resn_exch{" C6* DPB"}=" C6' DPB";
$resn_exch{" O3* DPB"}=" O3' DPB"; $resn_exch{" O2P DPB"}=" OP2 DPB"; $resn_exch{" O1P DPB"}=" OP1 DPB";
$resn_exch{" O3P DPB"}=" OP3 DPB"; $resn_exch{" C1\" DPI"}=" C1D DPI"; $resn_exch{" C2\" DPI"}=" C2D DPI";
$resn_exch{" C3\" DPI"}=" C3D DPI"; $resn_exch{" C4\" DPI"}=" C4D DPI"; $resn_exch{" C5\" DPI"}=" C5D DPI";
$resn_exch{" C6\" DPI"}=" C6' DPI"; $resn_exch{" O7\" DPI"}=" O7' DPI"; $resn_exch{" O8\" DPI"}=" O8' DPI";
$resn_exch{" O9\" DPI"}=" O9D DPI"; $resn_exch{" O5* DPY"}=" O5' DPY"; $resn_exch{" C2* DPY"}=" C2' DPY";
$resn_exch{" C5* DPY"}=" C5' DPY"; $resn_exch{" C4* DPY"}=" C4' DPY"; $resn_exch{" O4* DPY"}=" O4' DPY";
$resn_exch{" C1* DPY"}=" C1' DPY"; $resn_exch{" C3* DPY"}=" C3' DPY"; $resn_exch{" O3* DPY"}=" O3' DPY";
$resn_exch{" O1* DR2"}=" O1' DR2"; $resn_exch{" C1* DR2"}=" C1' DR2"; $resn_exch{" O5* DR2"}=" O5' DR2";
$resn_exch{" C5* DR2"}=" C5' DR2"; $resn_exch{" C6* DR2"}=" C6' DR2"; $resn_exch{" O6* DR2"}=" O6' DR2";
$resn_exch{" C4* DR2"}=" C4' DR2"; $resn_exch{" O4* DR2"}=" O4' DR2"; $resn_exch{" C3* DR2"}=" C3' DR2";
$resn_exch{" O3* DR2"}=" O3' DR2"; $resn_exch{" C2* DR2"}=" C2' DR2"; $resn_exch{" O2* DR2"}=" O2' DR2";
$resn_exch{" O2* DR3"}=" O2' DR3"; $resn_exch{" C2* DR3"}=" C2' DR3"; $resn_exch{" C3* DR3"}=" C3' DR3";
$resn_exch{" O3* DR3"}=" O3' DR3"; $resn_exch{" C4* DR3"}=" C4' DR3"; $resn_exch{" O4* DR3"}=" O4' DR3";
$resn_exch{" C5* DR3"}=" C5' DR3"; $resn_exch{" C6* DR3"}=" C6' DR3"; $resn_exch{" O6* DR3"}=" O6' DR3";
$resn_exch{" O5* DR3"}=" O5' DR3"; $resn_exch{" C1* DR3"}=" C1' DR3"; $resn_exch{" O1* DR3"}=" O1' DR3";
$resn_exch{" C1* DRA"}=" C1' DRA"; $resn_exch{" C2* DRA"}=" C2' DRA"; $resn_exch{" C3* DRA"}=" C3' DRA";
$resn_exch{" N3* DRA"}=" N3' DRA"; $resn_exch{" C7* DRA"}=" C7' DRA"; $resn_exch{" C8* DRA"}=" C8' DRA";
$resn_exch{" O5* DRA"}=" O5' DRA"; $resn_exch{" C5* DRA"}=" C5' DRA"; $resn_exch{" C6* DRA"}=" C6' DRA";
$resn_exch{" C4* DRA"}=" C4' DRA"; $resn_exch{" O4* DRA"}=" O4' DRA"; $resn_exch{" O2P DRM"}=" OP2 DRM";
$resn_exch{" O1P DRM"}=" OP1 DRM"; $resn_exch{" O3P DRM"}=" OP3 DRM"; $resn_exch{" C6* DRM"}=" C6' DRM";
$resn_exch{" O5* DRM"}=" O5' DRM"; $resn_exch{" C5* DRM"}=" C5' DRM"; $resn_exch{" C4* DRM"}=" C4' DRM";
$resn_exch{" C3* DRM"}=" C3' DRM"; $resn_exch{" C2* DRM"}=" C2' DRM"; $resn_exch{" C1* DRM"}=" C1' DRM";
$resn_exch{" O5* DRP"}=" O5' DRP"; $resn_exch{" C2* DRP"}=" C2' DRP"; $resn_exch{" C5* DRP"}=" C5' DRP";
$resn_exch{" C4* DRP"}=" C4' DRP"; $resn_exch{" O4* DRP"}=" O4' DRP"; $resn_exch{" C1* DRP"}=" C1' DRP";
$resn_exch{" C3* DRP"}=" C3' DRP"; $resn_exch{" O3* DRP"}=" O3' DRP"; $resn_exch{" O3P DRT"}=" OP3 DRT";
$resn_exch{" O1P DRT"}=" OP1 DRT"; $resn_exch{" O2P DRT"}=" OP2 DRT"; $resn_exch{" O5* DRT"}=" O5' DRT";
$resn_exch{" C5* DRT"}=" C5' DRT"; $resn_exch{" C4* DRT"}=" C4' DRT"; $resn_exch{" O4* DRT"}=" O4' DRT";
$resn_exch{" C3* DRT"}=" C3' DRT"; $resn_exch{" O3* DRT"}=" O3' DRT"; $resn_exch{" C2* DRT"}=" C2' DRT";
$resn_exch{" C1* DRT"}=" C1' DRT"; $resn_exch{" O1P DRZ"}=" OP1 DRZ"; $resn_exch{" O2P DRZ"}=" OP2 DRZ";
$resn_exch{" O3P DRZ"}=" OP3 DRZ"; $resn_exch{" O5* DRZ"}=" O5' DRZ"; $resn_exch{" C5* DRZ"}=" C5' DRZ";
$resn_exch{" C4* DRZ"}=" C4' DRZ"; $resn_exch{" O4* DRZ"}=" O4' DRZ"; $resn_exch{" C3* DRZ"}=" C3' DRZ";
$resn_exch{" C2* DRZ"}=" C2' DRZ"; $resn_exch{" C1* DRZ"}=" C1' DRZ"; $resn_exch{" O3* DRZ"}=" O3' DRZ";
$resn_exch{" O6* DSU"}=" O6' DSU"; $resn_exch{" C6* DSU"}=" C6' DSU"; $resn_exch{" C5* DSU"}=" C5' DSU";
$resn_exch{" O2* DSU"}=" O2' DSU"; $resn_exch{" C4* DSU"}=" C4' DSU"; $resn_exch{" O4* DSU"}=" O4' DSU";
$resn_exch{" C3* DSU"}=" C3' DSU"; $resn_exch{" O3* DSU"}=" O3' DSU"; $resn_exch{" C2* DSU"}=" C2' DSU";
$resn_exch{" C1* DSU"}=" C1' DSU"; $resn_exch{" O1* DSU"}=" O1' DSU"; $resn_exch{" O3P  DT"}=" OP3  DT";
$resn_exch{" O1P  DT"}=" OP1  DT"; $resn_exch{" O2P  DT"}=" OP2  DT"; $resn_exch{" O5*  DT"}=" O5'  DT";
$resn_exch{" C5*  DT"}=" C5'  DT"; $resn_exch{" C4*  DT"}=" C4'  DT"; $resn_exch{" O4*  DT"}=" O4'  DT";
$resn_exch{" C3*  DT"}=" C3'  DT"; $resn_exch{" O3*  DT"}=" O3'  DT"; $resn_exch{" C2*  DT"}=" C2'  DT";
$resn_exch{" C1*  DT"}=" C1'  DT"; $resn_exch{" C5M  DT"}=" C7   DT"; $resn_exch{" C1* DTA"}=" C1' DTA";
$resn_exch{" O4* DTA"}=" O4' DTA"; $resn_exch{" C2* DTA"}=" C2' DTA"; $resn_exch{" O2* DTA"}=" O2' DTA";
$resn_exch{" C3* DTA"}=" C3' DTA"; $resn_exch{" O3* DTA"}=" O3' DTA"; $resn_exch{" C4* DTA"}=" C4' DTA";
$resn_exch{" C5* DTA"}=" C5' DTA"; $resn_exch{" S5* DTA"}=" S5' DTA"; $resn_exch{" C2* DTM"}=" C2B DTM";
$resn_exch{" C4* DTM"}=" C4B DTM"; $resn_exch{" C5* DTM"}=" C5B DTM"; $resn_exch{" C6* DTM"}=" C6B DTM";
$resn_exch{" C7* DTM"}=" C7' DTM"; $resn_exch{" C9* DTM"}=" C9' DTM"; $resn_exch{" CN* DTM"}=" CN' DTM";
$resn_exch{" N1* DTM"}=" N1' DTM"; $resn_exch{" N2* DTM"}=" N2' DTM"; $resn_exch{" N3* DTM"}=" N3' DTM";
$resn_exch{" N4* DTM"}=" N4' DTM"; $resn_exch{" N8* DTM"}=" N8' DTM"; $resn_exch{" N9* DTM"}=" N9' DTM";
$resn_exch{" O5* DTP"}=" O5' DTP"; $resn_exch{" C5* DTP"}=" C5' DTP"; $resn_exch{" C4* DTP"}=" C4' DTP";
$resn_exch{" O4* DTP"}=" O4' DTP"; $resn_exch{" C3* DTP"}=" C3' DTP"; $resn_exch{" O3* DTP"}=" O3' DTP";
$resn_exch{" C2* DTP"}=" C2' DTP"; $resn_exch{" C1* DTP"}=" C1' DTP"; $resn_exch{" O3P  DU"}=" OP3  DU";
$resn_exch{" O1P  DU"}=" OP1  DU"; $resn_exch{" O2P  DU"}=" OP2  DU"; $resn_exch{" O5*  DU"}=" O5'  DU";
$resn_exch{" C5*  DU"}=" C5'  DU"; $resn_exch{" C4*  DU"}=" C4'  DU"; $resn_exch{" O4*  DU"}=" O4'  DU";
$resn_exch{" C3*  DU"}=" C3'  DU"; $resn_exch{" O3*  DU"}=" O3'  DU"; $resn_exch{" C2*  DU"}=" C2'  DU";
$resn_exch{" C1*  DU"}=" C1'  DU"; $resn_exch{" C1* DUD"}=" C1' DUD"; $resn_exch{" C2* DUD"}=" C2' DUD";
$resn_exch{" C3* DUD"}=" C3' DUD"; $resn_exch{" C4* DUD"}=" C4' DUD"; $resn_exch{" O4* DUD"}=" O4' DUD";
$resn_exch{" O3* DUD"}=" O3' DUD"; $resn_exch{" C5* DUD"}=" C5' DUD"; $resn_exch{" O5* DUD"}=" O5' DUD";
$resn_exch{" C1* DUN"}=" C1' DUN"; $resn_exch{" C2* DUN"}=" C2' DUN"; $resn_exch{" C3* DUN"}=" C3' DUN";
$resn_exch{" C4* DUN"}=" C4' DUN"; $resn_exch{" O4* DUN"}=" O4' DUN"; $resn_exch{" O3* DUN"}=" O3' DUN";
$resn_exch{" C5* DUN"}=" C5' DUN"; $resn_exch{" O5* DUN"}=" O5' DUN"; $resn_exch{" C1* DUP"}=" C1' DUP";
$resn_exch{" C2* DUP"}=" C2' DUP"; $resn_exch{" C3* DUP"}=" C3' DUP"; $resn_exch{" O3* DUP"}=" O3' DUP";
$resn_exch{" O4* DUP"}=" O4' DUP"; $resn_exch{" C4* DUP"}=" C4' DUP"; $resn_exch{" C5* DUP"}=" C5' DUP";
$resn_exch{" O5* DUP"}=" O5' DUP"; $resn_exch{" C1* DUR"}=" C1' DUR"; $resn_exch{" C2* DUR"}=" C2' DUR";
$resn_exch{" C3* DUR"}=" C3' DUR"; $resn_exch{" C4* DUR"}=" C4' DUR"; $resn_exch{" O3* DUR"}=" O3' DUR";
$resn_exch{" O4* DUR"}=" O4' DUR"; $resn_exch{" C5* DUR"}=" C5' DUR"; $resn_exch{" O5* DUR"}=" O5' DUR";
$resn_exch{" C1* DUT"}=" C1' DUT"; $resn_exch{" C2* DUT"}=" C2' DUT"; $resn_exch{" C3* DUT"}=" C3' DUT";
$resn_exch{" C4* DUT"}=" C4' DUT"; $resn_exch{" O4* DUT"}=" O4' DUT"; $resn_exch{" O3* DUT"}=" O3' DUT";
$resn_exch{" C5* DUT"}=" C5' DUT"; $resn_exch{" O5* DUT"}=" O5' DUT"; $resn_exch{" O1P DXD"}=" OP1 DXD";
$resn_exch{" O2P DXD"}=" OP2 DXD"; $resn_exch{" O5* DXD"}=" O5' DXD"; $resn_exch{" C5* DXD"}=" C5' DXD";
$resn_exch{" C4* DXD"}=" C4' DXD"; $resn_exch{" C1* DXD"}=" C1' DXD"; $resn_exch{" C6* DXD"}=" C6' DXD";
$resn_exch{" O6* DXD"}=" O6' DXD"; $resn_exch{" C2* DXD"}=" C2' DXD"; $resn_exch{" C3* DXD"}=" C3' DXD";
$resn_exch{" O3* DXD"}=" O3' DXD"; $resn_exch{" O3P DXD"}=" OP3 DXD"; $resn_exch{" O1P DXN"}=" OP1 DXN";
$resn_exch{" O2P DXN"}=" OP2 DXN"; $resn_exch{" O5* DXN"}=" O5' DXN"; $resn_exch{" C5* DXN"}=" C5' DXN";
$resn_exch{" C4* DXN"}=" C4' DXN"; $resn_exch{" C1* DXN"}=" C1' DXN"; $resn_exch{" C6* DXN"}=" C6' DXN";
$resn_exch{" O6* DXN"}=" O6' DXN"; $resn_exch{" C2* DXN"}=" C2' DXN"; $resn_exch{" C3* DXN"}=" C3' DXN";
$resn_exch{" O3* DXN"}=" O3' DXN"; $resn_exch{" O3P DXN"}=" OP3 DXN"; $resn_exch{" O1P   E"}=" OP1   E";
$resn_exch{" O2P   E"}=" OP2   E"; $resn_exch{" O3P   E"}=" OP3   E"; $resn_exch{" O5*   E"}=" O5'   E";
$resn_exch{" C5*   E"}=" C5'   E"; $resn_exch{" C4*   E"}=" C4'   E"; $resn_exch{" O4*   E"}=" O4'   E";
$resn_exch{" C3*   E"}=" C3'   E"; $resn_exch{" O3*   E"}=" O3'   E"; $resn_exch{" C2*   E"}=" C2'   E";
$resn_exch{" C1*   E"}=" C1'   E"; $resn_exch{" O1P E1X"}=" OP1 E1X"; $resn_exch{" O2P E1X"}=" OP2 E1X";
$resn_exch{" O5* E1X"}=" O5' E1X"; $resn_exch{" C2* E1X"}=" C2' E1X"; $resn_exch{" C5* E1X"}=" C5' E1X";
$resn_exch{" C4* E1X"}=" C4' E1X"; $resn_exch{" O4* E1X"}=" O4' E1X"; $resn_exch{" C1* E1X"}=" C1' E1X";
$resn_exch{" C3* E1X"}=" C3' E1X"; $resn_exch{" O3* E1X"}=" O3' E1X"; $resn_exch{" O3P E1X"}=" OP3 E1X";
$resn_exch{" O1P EDA"}=" OP1 EDA"; $resn_exch{" O2P EDA"}=" OP2 EDA"; $resn_exch{" O3P EDA"}=" OP3 EDA";
$resn_exch{" O5* EDA"}=" O5' EDA"; $resn_exch{" C2* EDA"}=" C2' EDA"; $resn_exch{" C5* EDA"}=" C5' EDA";
$resn_exch{" C4* EDA"}=" C4' EDA"; $resn_exch{" O4* EDA"}=" O4' EDA"; $resn_exch{" C1* EDA"}=" C1' EDA";
$resn_exch{" C3* EDA"}=" C3' EDA"; $resn_exch{" O3* EDA"}=" O3' EDA"; $resn_exch{" O1P EDC"}=" OP1 EDC";
$resn_exch{" O2P EDC"}=" OP2 EDC"; $resn_exch{" O3P EDC"}=" OP3 EDC"; $resn_exch{" O5* EDC"}=" O5' EDC";
$resn_exch{" C5* EDC"}=" C5' EDC"; $resn_exch{" C4* EDC"}=" C4' EDC"; $resn_exch{" O4* EDC"}=" O4' EDC";
$resn_exch{" C1* EDC"}=" C1' EDC"; $resn_exch{" C2* EDC"}=" C2' EDC"; $resn_exch{" C3* EDC"}=" C3' EDC";
$resn_exch{" O3* EDC"}=" O3' EDC"; $resn_exch{" UN1 EEB"}=" N1U EEB"; $resn_exch{" UC2 EEB"}=" C2U EEB";
$resn_exch{" UN3 EEB"}=" N3U EEB"; $resn_exch{" UC4 EEB"}=" C4U EEB"; $resn_exch{" UC5 EEB"}=" C5U EEB";
$resn_exch{" UC6 EEB"}=" C6U EEB"; $resn_exch{" UO2 EEB"}=" O2U EEB"; $resn_exch{" UO4 EEB"}=" O4U EEB";
$resn_exch{"UC1* EEB"}=" C1D EEB"; $resn_exch{"UC2* EEB"}=" C2D EEB"; $resn_exch{"UO2* EEB"}=" O2D EEB";
$resn_exch{"UC3* EEB"}=" C3D EEB"; $resn_exch{"UC4* EEB"}=" C4D EEB"; $resn_exch{"UO4* EEB"}=" O4D EEB";
$resn_exch{"UO3* EEB"}=" O3D EEB"; $resn_exch{"UC5* EEB"}=" C5D EEB"; $resn_exch{"UO5* EEB"}=" O5D EEB";
$resn_exch{" EC1 EEB"}=" C1E EEB"; $resn_exch{" EC2 EEB"}=" C2E EEB"; $resn_exch{" EC3 EEB"}=" C3E EEB";
$resn_exch{" EC4 EEB"}=" C4E EEB"; $resn_exch{" EO1 EEB"}=" O1E EEB"; $resn_exch{" EO2 EEB"}=" O2E EEB";
$resn_exch{" O3P EIT"}=" OP3 EIT"; $resn_exch{" O2P EIT"}=" OP2 EIT"; $resn_exch{" O5* EIT"}=" O5' EIT";
$resn_exch{" C5* EIT"}=" C5' EIT"; $resn_exch{" C4* EIT"}=" C4' EIT"; $resn_exch{" O4* EIT"}=" O4' EIT";
$resn_exch{" C1* EIT"}=" C1' EIT"; $resn_exch{" C2* EIT"}=" C2' EIT"; $resn_exch{" O2* EIT"}=" O2' EIT";
$resn_exch{" C3* EIT"}=" C3' EIT"; $resn_exch{" O3* EIT"}=" O3' EIT"; $resn_exch{" O1P EIT"}=" OP1 EIT";
$resn_exch{" C5* EMA"}=" C5' EMA"; $resn_exch{" O5* EMA"}=" O5' EMA"; $resn_exch{" C4* EMA"}=" C4' EMA";
$resn_exch{" C1* EMA"}=" C1' EMA"; $resn_exch{" C1* ENA"}=" C1' ENA"; $resn_exch{" O2* ENA"}=" O2' ENA";
$resn_exch{" C2* ENA"}=" C2' ENA"; $resn_exch{" C4* ENA"}=" C4' ENA"; $resn_exch{" C3* ENA"}=" C3' ENA";
$resn_exch{" O4* ENA"}=" O4' ENA"; $resn_exch{" C5* ENA"}=" C5' ENA"; $resn_exch{" O3* ENA"}=" O3' ENA";
$resn_exch{" O5* ENA"}=" O5' ENA"; $resn_exch{" C6* ENA"}=" C6' ENA"; $resn_exch{" C7* ENA"}=" C7' ENA";
$resn_exch{" O6* ENA"}=" O6' ENA"; $resn_exch{" C8* ENA"}=" C8' ENA"; $resn_exch{" C9* ENA"}=" C9' ENA";
$resn_exch{" C0* ENA"}=" C0' ENA"; $resn_exch{" O7* ENA"}=" O7' ENA"; $resn_exch{" O8* ENA"}=" O8' ENA";
$resn_exch{" O5* ENP"}=" O5' ENP"; $resn_exch{" C5* ENP"}=" C5' ENP"; $resn_exch{" C4* ENP"}=" C4' ENP";
$resn_exch{" O4* ENP"}=" O4' ENP"; $resn_exch{" C3* ENP"}=" C3' ENP"; $resn_exch{" O3* ENP"}=" O3' ENP";
$resn_exch{" C2* ENP"}=" C2' ENP"; $resn_exch{" O2* ENP"}=" O2' ENP"; $resn_exch{" C1* ENP"}=" C1' ENP";
$resn_exch{" O9* ENP"}=" O9' ENP"; $resn_exch{" C0* ENP"}=" C0' ENP"; $resn_exch{" C9* ENP"}=" C9' ENP";
$resn_exch{" O8* ENP"}=" O8' ENP"; $resn_exch{" C8* ENP"}=" C8' ENP"; $resn_exch{" O7* ENP"}=" O7' ENP";
$resn_exch{" C7* ENP"}=" C7' ENP"; $resn_exch{" O6* ENP"}=" O6' ENP"; $resn_exch{" C6* ENP"}=" C6' ENP";
$resn_exch{" O2* EOM"}=" O2' EOM"; $resn_exch{" CA* EOM"}=" CA' EOM"; $resn_exch{" OB* EOM"}=" OB' EOM";
$resn_exch{" CC* EOM"}=" CC' EOM"; $resn_exch{" CD* EOM"}=" CD' EOM"; $resn_exch{" UN1 EPU"}=" N1U EPU";
$resn_exch{" UC2 EPU"}=" C2U EPU"; $resn_exch{" UN3 EPU"}=" N3U EPU"; $resn_exch{" UC4 EPU"}=" C4U EPU";
$resn_exch{" UC5 EPU"}=" C5U EPU"; $resn_exch{" UC6 EPU"}=" C6U EPU"; $resn_exch{" UO2 EPU"}=" O2U EPU";
$resn_exch{" UO4 EPU"}=" O4U EPU"; $resn_exch{"UC1* EPU"}=" C1D EPU"; $resn_exch{"UC2* EPU"}=" C2D EPU";
$resn_exch{"UO2* EPU"}=" O2D EPU"; $resn_exch{"UC3* EPU"}=" C3D EPU"; $resn_exch{"UC4* EPU"}=" C4D EPU";
$resn_exch{"UO4* EPU"}=" O4D EPU"; $resn_exch{"UO3* EPU"}=" O3D EPU"; $resn_exch{"UC5* EPU"}=" C5D EPU";
$resn_exch{"UO5* EPU"}=" O5D EPU"; $resn_exch{" EC1 EPU"}=" C1E EPU"; $resn_exch{" EO1 EPU"}=" O1E EPU";
$resn_exch{" EO2 EPU"}=" O2E EPU"; $resn_exch{" EC2 EPU"}=" C2E EPU"; $resn_exch{" EC3 EPU"}=" C3E EPU";
$resn_exch{" C1* ERT"}=" C1' ERT"; $resn_exch{" C2* ERT"}=" C2' ERT"; $resn_exch{" C3* ERT"}=" C3' ERT";
$resn_exch{" N3* ERT"}=" N3' ERT"; $resn_exch{" C7* ERT"}=" C7' ERT"; $resn_exch{" C8* ERT"}=" C8' ERT";
$resn_exch{" C4* ERT"}=" C4' ERT"; $resn_exch{" O4* ERT"}=" O4' ERT"; $resn_exch{" C5* ERT"}=" C5' ERT";
$resn_exch{" O5* ERT"}=" O5' ERT"; $resn_exch{" C6* ERT"}=" C6' ERT"; $resn_exch{" O5* F2N"}=" O5' F2N";
$resn_exch{" C5* F2N"}=" C5' F2N"; $resn_exch{" C4* F2N"}=" C4' F2N"; $resn_exch{" O4* F2N"}=" O4' F2N";
$resn_exch{" C3* F2N"}=" C3' F2N"; $resn_exch{" O3* F2N"}=" O3' F2N"; $resn_exch{" C2* F2N"}=" C2' F2N";
$resn_exch{" O2* F2N"}=" O2' F2N"; $resn_exch{" C1* F2N"}=" C1' F2N"; $resn_exch{" C1* F42"}=" C1' F42";
$resn_exch{" C2* F42"}=" C2' F42"; $resn_exch{" O2* F42"}=" O2' F42"; $resn_exch{" C3* F42"}=" C3' F42";
$resn_exch{" O3* F42"}=" O3' F42"; $resn_exch{" C4* F42"}=" C4' F42"; $resn_exch{" O4* F42"}=" O4' F42";
$resn_exch{" C5* F42"}=" C5' F42"; $resn_exch{" O5* F42"}=" O5' F42"; $resn_exch{" N_A F43"}=" NA  F43";
$resn_exch{" N_B F43"}=" NB  F43"; $resn_exch{" N_C F43"}=" NC  F43"; $resn_exch{" N_D F43"}=" ND  F43";
$resn_exch{" O1P FA2"}=" OP1 FA2"; $resn_exch{" O2P FA2"}=" OP2 FA2"; $resn_exch{" O3* FA2"}=" O3' FA2";
$resn_exch{" C2* FA2"}=" C2' FA2"; $resn_exch{" C4* FA2"}=" C4' FA2"; $resn_exch{" O4* FA2"}=" O4' FA2";
$resn_exch{" C1* FA2"}=" C1' FA2"; $resn_exch{" C3* FA2"}=" C3' FA2"; $resn_exch{" O2* FA2"}=" O2' FA2";
$resn_exch{" O3P FA2"}=" OP3 FA2"; $resn_exch{" O1P FA5"}=" OP1 FA5"; $resn_exch{" O2P FA5"}=" OP2 FA5";
$resn_exch{" O3P FA5"}=" OP3 FA5"; $resn_exch{" O5* FA5"}=" O5' FA5"; $resn_exch{" C5* FA5"}=" C5' FA5";
$resn_exch{" C4* FA5"}=" C4' FA5"; $resn_exch{" O4* FA5"}=" O4' FA5"; $resn_exch{" C3* FA5"}=" C3' FA5";
$resn_exch{" O3* FA5"}=" O3' FA5"; $resn_exch{" C2* FA5"}=" C2' FA5"; $resn_exch{" O2* FA5"}=" O2' FA5";
$resn_exch{" C1* FA5"}=" C1' FA5"; $resn_exch{" BC3 FA9"}=" C3B FA9"; $resn_exch{"BC1* FA9"}=" CB1 FA9";
$resn_exch{"BC2* FA9"}=" CB2 FA9"; $resn_exch{"BC3* FA9"}=" CB3 FA9"; $resn_exch{"BC4* FA9"}=" CB4 FA9";
$resn_exch{"BC5* FA9"}=" CB5 FA9"; $resn_exch{"BC6* FA9"}=" CB6 FA9"; $resn_exch{" BC2 FA9"}=" C2B FA9";
$resn_exch{" BC1 FA9"}=" C1B FA9"; $resn_exch{" BO1 FA9"}=" O1B FA9"; $resn_exch{" C1* FA9"}=" C1' FA9";
$resn_exch{" C2* FA9"}=" C2' FA9"; $resn_exch{" O2* FA9"}=" O2' FA9"; $resn_exch{" C3* FA9"}=" C3' FA9";
$resn_exch{" O3* FA9"}=" O3' FA9"; $resn_exch{" C4* FA9"}=" C4' FA9"; $resn_exch{" O4* FA9"}=" O4' FA9";
$resn_exch{" C5* FA9"}=" C5' FA9"; $resn_exch{" O5* FA9"}=" O5' FA9"; $resn_exch{" AP  FA9"}=" PA  FA9";
$resn_exch{" AO1 FA9"}=" O1A FA9"; $resn_exch{" AO2 FA9"}=" O2A FA9"; $resn_exch{"AO5* FA9"}=" OA5 FA9";
$resn_exch{"AC5* FA9"}=" CA5 FA9"; $resn_exch{"AC4* FA9"}=" CA4 FA9"; $resn_exch{"AO4* FA9"}=" OA4 FA9";
$resn_exch{"AC3* FA9"}=" CA3 FA9"; $resn_exch{"AO3* FA9"}=" OA3 FA9"; $resn_exch{"AC2* FA9"}=" CA2 FA9";
$resn_exch{"AO2* FA9"}=" OA2 FA9"; $resn_exch{"AC1* FA9"}=" CA1 FA9"; $resn_exch{" AN9 FA9"}=" N9A FA9";
$resn_exch{" AC8 FA9"}=" C8A FA9"; $resn_exch{" AN7 FA9"}=" N7A FA9"; $resn_exch{" AC5 FA9"}=" C5C FA9";
$resn_exch{" AC4 FA9"}=" C4C FA9"; $resn_exch{" AN3 FA9"}=" N3A FA9"; $resn_exch{" AC2 FA9"}=" C2A FA9";
$resn_exch{" AN1 FA9"}=" N1A FA9"; $resn_exch{" AC6 FA9"}=" C6A FA9"; $resn_exch{" AN6 FA9"}=" N6A FA9";
$resn_exch{" AP  FAA"}=" PA  FAA"; $resn_exch{" AO1 FAA"}=" O1A FAA"; $resn_exch{" AO2 FAA"}=" O2A FAA";
$resn_exch{"AO5* FAA"}=" O5B FAA"; $resn_exch{"AC5* FAA"}=" C5B FAA"; $resn_exch{"AC4* FAA"}=" C4B FAA";
$resn_exch{"AO4* FAA"}=" O4B FAA"; $resn_exch{"AC3* FAA"}=" C3B FAA"; $resn_exch{"AO3* FAA"}=" O3B FAA";
$resn_exch{"AC2* FAA"}=" C2B FAA"; $resn_exch{"AO2* FAA"}=" O2B FAA"; $resn_exch{"AC1* FAA"}=" C1B FAA";
$resn_exch{" AN9 FAA"}=" N9A FAA"; $resn_exch{" AC8 FAA"}=" C8A FAA"; $resn_exch{" AN7 FAA"}=" N7A FAA";
$resn_exch{" AC5 FAA"}=" C5A FAA"; $resn_exch{" AC6 FAA"}=" C6A FAA"; $resn_exch{" AN6 FAA"}=" N6A FAA";
$resn_exch{" AN1 FAA"}=" N1A FAA"; $resn_exch{" AC2 FAA"}=" C2A FAA"; $resn_exch{" AN3 FAA"}=" N3A FAA";
$resn_exch{" AC4 FAA"}=" C4A FAA"; $resn_exch{" C4A FAA"}=" C4X FAA"; $resn_exch{" C5A FAA"}=" C5X FAA";
$resn_exch{" C1* FAA"}=" C1' FAA"; $resn_exch{" C2* FAA"}=" C2' FAA"; $resn_exch{" O2* FAA"}=" O2' FAA";
$resn_exch{" C3* FAA"}=" C3' FAA"; $resn_exch{" O3* FAA"}=" O3' FAA"; $resn_exch{" C4* FAA"}=" C4' FAA";
$resn_exch{" O4* FAA"}=" O4' FAA"; $resn_exch{" C5* FAA"}=" C5' FAA"; $resn_exch{" O5* FAA"}=" O5' FAA";
$resn_exch{" AP  FAB"}=" PA  FAB"; $resn_exch{" AO1 FAB"}=" O1A FAB"; $resn_exch{" AO2 FAB"}=" O2A FAB";
$resn_exch{"AO5* FAB"}=" O5B FAB"; $resn_exch{"AC5* FAB"}=" C5B FAB"; $resn_exch{"AC4* FAB"}=" C4B FAB";
$resn_exch{"AO4* FAB"}=" O4B FAB"; $resn_exch{"AC3* FAB"}=" C3B FAB"; $resn_exch{"AO3* FAB"}=" O3B FAB";
$resn_exch{"AC2* FAB"}=" C2B FAB"; $resn_exch{"AO2* FAB"}=" O2B FAB"; $resn_exch{"AC1* FAB"}=" C1B FAB";
$resn_exch{" AN9 FAB"}=" N9A FAB"; $resn_exch{" AC8 FAB"}=" C8A FAB"; $resn_exch{" AN7 FAB"}=" N7A FAB";
$resn_exch{" AC5 FAB"}=" C5A FAB"; $resn_exch{" AC6 FAB"}=" C6A FAB"; $resn_exch{" AN6 FAB"}=" N6A FAB";
$resn_exch{" AN1 FAB"}=" N1A FAB"; $resn_exch{" AC2 FAB"}=" C2A FAB"; $resn_exch{" AN3 FAB"}=" N3A FAB";
$resn_exch{" AC4 FAB"}=" C4A FAB"; $resn_exch{" C4A FAB"}=" C4X FAB"; $resn_exch{" C5A FAB"}=" C5X FAB";
$resn_exch{" C1* FAB"}=" C1' FAB"; $resn_exch{" C2* FAB"}=" C2D FAB"; $resn_exch{" O2* FAB"}=" O2' FAB";
$resn_exch{" C3* FAB"}=" C3D FAB"; $resn_exch{" O3* FAB"}=" O3D FAB"; $resn_exch{" C4* FAB"}=" C4D FAB";
$resn_exch{" O4* FAB"}=" O4' FAB"; $resn_exch{" C5* FAB"}=" C5D FAB"; $resn_exch{" O5* FAB"}=" O5' FAB";
$resn_exch{" AP  FAD"}=" PA  FAD"; $resn_exch{" AO1 FAD"}=" O1A FAD"; $resn_exch{" AO2 FAD"}=" O2A FAD";
$resn_exch{"AO5* FAD"}=" O5B FAD"; $resn_exch{"AC5* FAD"}=" C5B FAD"; $resn_exch{"AC4* FAD"}=" C4B FAD";
$resn_exch{"AO4* FAD"}=" O4B FAD"; $resn_exch{"AC3* FAD"}=" C3B FAD"; $resn_exch{"AO3* FAD"}=" O3B FAD";
$resn_exch{"AC2* FAD"}=" C2B FAD"; $resn_exch{"AO2* FAD"}=" O2B FAD"; $resn_exch{"AC1* FAD"}=" C1B FAD";
$resn_exch{" AN9 FAD"}=" N9A FAD"; $resn_exch{" AC8 FAD"}=" C8A FAD"; $resn_exch{" AN7 FAD"}=" N7A FAD";
$resn_exch{" AC5 FAD"}=" C5A FAD"; $resn_exch{" AC6 FAD"}=" C6A FAD"; $resn_exch{" AN6 FAD"}=" N6A FAD";
$resn_exch{" AN1 FAD"}=" N1A FAD"; $resn_exch{" AC2 FAD"}=" C2A FAD"; $resn_exch{" AN3 FAD"}=" N3A FAD";
$resn_exch{" AC4 FAD"}=" C4A FAD"; $resn_exch{" C4A FAD"}=" C4X FAD"; $resn_exch{" C5A FAD"}=" C5X FAD";
$resn_exch{" C1* FAD"}=" C1' FAD"; $resn_exch{" C2* FAD"}=" C2' FAD"; $resn_exch{" O2* FAD"}=" O2' FAD";
$resn_exch{" C3* FAD"}=" C3' FAD"; $resn_exch{" O3* FAD"}=" O3' FAD"; $resn_exch{" C4* FAD"}=" C4' FAD";
$resn_exch{" O4* FAD"}=" O4' FAD"; $resn_exch{" C5* FAD"}=" C5' FAD"; $resn_exch{" O5* FAD"}=" O5' FAD";
$resn_exch{" AP  FAE"}=" PA  FAE"; $resn_exch{" AO1 FAE"}=" O1A FAE"; $resn_exch{" AO2 FAE"}=" O2A FAE";
$resn_exch{"AO5* FAE"}=" O5B FAE"; $resn_exch{"AC5* FAE"}=" C5B FAE"; $resn_exch{"AC4* FAE"}=" C4B FAE";
$resn_exch{"AO4* FAE"}=" O4B FAE"; $resn_exch{"AC3* FAE"}=" C3B FAE"; $resn_exch{"AO3* FAE"}=" O3B FAE";
$resn_exch{"AC2* FAE"}=" C2B FAE"; $resn_exch{"AO2* FAE"}=" O2B FAE"; $resn_exch{"AC1* FAE"}=" C1B FAE";
$resn_exch{" AN9 FAE"}=" N9A FAE"; $resn_exch{" AC8 FAE"}=" C8A FAE"; $resn_exch{" AN7 FAE"}=" N7A FAE";
$resn_exch{" AC5 FAE"}=" C5A FAE"; $resn_exch{" AC6 FAE"}=" C6A FAE"; $resn_exch{" AN6 FAE"}=" N6A FAE";
$resn_exch{" AN1 FAE"}=" N1A FAE"; $resn_exch{" AC2 FAE"}=" C2A FAE"; $resn_exch{" AN3 FAE"}=" N3A FAE";
$resn_exch{" AC4 FAE"}=" C4A FAE"; $resn_exch{" C4A FAE"}=" C4X FAE"; $resn_exch{" C5A FAE"}=" C5X FAE";
$resn_exch{" C1* FAE"}=" C1' FAE"; $resn_exch{" C2* FAE"}=" C2' FAE"; $resn_exch{" O2* FAE"}=" O2' FAE";
$resn_exch{" C3* FAE"}=" C3' FAE"; $resn_exch{" O3* FAE"}=" O3' FAE"; $resn_exch{" C4* FAE"}=" C4' FAE";
$resn_exch{" O4* FAE"}=" O4' FAE"; $resn_exch{" C5* FAE"}=" C5' FAE"; $resn_exch{" O5* FAE"}=" O5' FAE";
$resn_exch{" O5* FAG"}=" O5' FAG"; $resn_exch{" C5* FAG"}=" C5' FAG"; $resn_exch{" C4* FAG"}=" C4' FAG";
$resn_exch{" O4* FAG"}=" O4' FAG"; $resn_exch{" C1* FAG"}=" C1' FAG"; $resn_exch{" C2* FAG"}=" C2' FAG";
$resn_exch{" C3* FAG"}=" C3' FAG"; $resn_exch{" O3* FAG"}=" O3' FAG"; $resn_exch{"AC10 FAG"}=" CAA FAG";
$resn_exch{"AC11 FAG"}=" CBA FAG"; $resn_exch{"1H3  FAG"}=" H31 FAG"; $resn_exch{" AN1 FAM"}=" N1A FAM";
$resn_exch{" AC2 FAM"}=" C2A FAM"; $resn_exch{" AN3 FAM"}=" N3A FAM"; $resn_exch{" AC4 FAM"}=" C4A FAM";
$resn_exch{" AC5 FAM"}=" C5A FAM"; $resn_exch{" AC6 FAM"}=" C6A FAM"; $resn_exch{" AN6 FAM"}=" N6A FAM";
$resn_exch{" AN7 FAM"}=" N7A FAM"; $resn_exch{" AC8 FAM"}=" C8A FAM"; $resn_exch{" AN9 FAM"}=" N9A FAM";
$resn_exch{"AC1* FAM"}=" C1B FAM"; $resn_exch{"AC2* FAM"}=" C2B FAM"; $resn_exch{"AO2* FAM"}=" O2B FAM";
$resn_exch{"AC3* FAM"}=" C3B FAM"; $resn_exch{"AO3* FAM"}=" O3B FAM"; $resn_exch{"AP3* FAM"}=" P3B FAM";
$resn_exch{" AO7 FAM"}=" O7A FAM"; $resn_exch{" AO8 FAM"}=" O8A FAM"; $resn_exch{" AO9 FAM"}=" O9A FAM";
$resn_exch{"AC4* FAM"}=" C4B FAM"; $resn_exch{"AO4* FAM"}=" O4B FAM"; $resn_exch{"AC5* FAM"}=" C5B FAM";
$resn_exch{"AO5* FAM"}=" O5B FAM"; $resn_exch{" AP1 FAM"}=" P1A FAM"; $resn_exch{" AO1 FAM"}=" O1A FAM";
$resn_exch{" AO2 FAM"}=" O2A FAM"; $resn_exch{" AO3 FAM"}=" O3A FAM"; $resn_exch{" AP2 FAM"}=" P2A FAM";
$resn_exch{" AO4 FAM"}=" O4A FAM"; $resn_exch{" AO5 FAM"}=" O5A FAM"; $resn_exch{" AO6 FAM"}=" O6A FAM";
$resn_exch{"PC11 FAM"}=" CBP FAM"; $resn_exch{"PC12 FAM"}=" CCP FAM"; $resn_exch{"PC13 FAM"}=" CDP FAM";
$resn_exch{"PC14 FAM"}=" CEP FAM"; $resn_exch{"PC10 FAM"}=" CAP FAM"; $resn_exch{"PO10 FAM"}=" OAP FAM";
$resn_exch{" PC9 FAM"}=" C9P FAM"; $resn_exch{" PO9 FAM"}=" O9P FAM"; $resn_exch{" PN8 FAM"}=" N8P FAM";
$resn_exch{" PC7 FAM"}=" C7P FAM"; $resn_exch{" PC6 FAM"}=" C6P FAM"; $resn_exch{" PC5 FAM"}=" C5P FAM";
$resn_exch{" PO5 FAM"}=" O5P FAM"; $resn_exch{" PN4 FAM"}=" N4P FAM"; $resn_exch{" PC3 FAM"}=" C3P FAM";
$resn_exch{" PC2 FAM"}=" C2P FAM"; $resn_exch{" AP  FAS"}=" PA  FAS"; $resn_exch{" AO1 FAS"}=" O1A FAS";
$resn_exch{" AO2 FAS"}=" O2A FAS"; $resn_exch{"AO5* FAS"}=" O5B FAS"; $resn_exch{"AC5* FAS"}=" C5B FAS";
$resn_exch{"AC4* FAS"}=" C4B FAS"; $resn_exch{"AO4* FAS"}=" O4B FAS"; $resn_exch{"AC3* FAS"}=" C3B FAS";
$resn_exch{"AO3* FAS"}=" O3B FAS"; $resn_exch{"AC2* FAS"}=" C2B FAS"; $resn_exch{"AO2* FAS"}=" O2B FAS";
$resn_exch{"AC1* FAS"}=" C1B FAS"; $resn_exch{" AN9 FAS"}=" N9A FAS"; $resn_exch{" AC8 FAS"}=" C8A FAS";
$resn_exch{" AN7 FAS"}=" N7A FAS"; $resn_exch{" AC5 FAS"}=" C5A FAS"; $resn_exch{" AC6 FAS"}=" C6A FAS";
$resn_exch{" AN6 FAS"}=" N6A FAS"; $resn_exch{" AN1 FAS"}=" N1A FAS"; $resn_exch{" AC2 FAS"}=" C2A FAS";
$resn_exch{" AN3 FAS"}=" N3A FAS"; $resn_exch{" AC4 FAS"}=" C4A FAS"; $resn_exch{" C4A FAS"}=" C4X FAS";
$resn_exch{" C5A FAS"}=" C5X FAS"; $resn_exch{" C1* FAS"}=" C1' FAS"; $resn_exch{" C2* FAS"}=" C2' FAS";
$resn_exch{" O2* FAS"}=" O2' FAS"; $resn_exch{" C3* FAS"}=" C3' FAS"; $resn_exch{" O3* FAS"}=" O3' FAS";
$resn_exch{" C4* FAS"}=" C4' FAS"; $resn_exch{" O4* FAS"}=" O4' FAS"; $resn_exch{" C5* FAS"}=" C5' FAS";
$resn_exch{" O5* FAS"}=" O5' FAS"; $resn_exch{"6OS  FBQ"}=" OS6 FBQ"; $resn_exch{" AP  FCG"}=" PA  FCG";
$resn_exch{" AO1 FCG"}=" O1A FCG"; $resn_exch{" AO2 FCG"}=" O2A FCG"; $resn_exch{"AO5* FCG"}=" O5B FCG";
$resn_exch{"AC5* FCG"}=" C5B FCG"; $resn_exch{"AC4* FCG"}=" C4B FCG"; $resn_exch{"AO4* FCG"}=" O4B FCG";
$resn_exch{"AC1* FCG"}=" C1B FCG"; $resn_exch{" AN9 FCG"}=" N9A FCG"; $resn_exch{" AC4 FCG"}=" C4A FCG";
$resn_exch{" AN3 FCG"}=" N3A FCG"; $resn_exch{" AC2 FCG"}=" C2A FCG"; $resn_exch{" AN1 FCG"}=" N1A FCG";
$resn_exch{" AC6 FCG"}=" C6A FCG"; $resn_exch{" AN6 FCG"}=" N6A FCG"; $resn_exch{" AC5 FCG"}=" C5A FCG";
$resn_exch{" AN7 FCG"}=" N7A FCG"; $resn_exch{" AC8 FCG"}=" C8A FCG"; $resn_exch{"AC2* FCG"}=" C2B FCG";
$resn_exch{"AO2* FCG"}=" O2B FCG"; $resn_exch{"AC3* FCG"}=" C3B FCG"; $resn_exch{"AO3* FCG"}=" O3B FCG";
$resn_exch{" O5* FCG"}=" O5' FCG"; $resn_exch{" C5* FCG"}=" C5' FCG"; $resn_exch{" C4* FCG"}=" C4' FCG";
$resn_exch{" O4* FCG"}=" O4' FCG"; $resn_exch{" C3* FCG"}=" C3' FCG"; $resn_exch{" O3* FCG"}=" O3' FCG";
$resn_exch{" C2* FCG"}=" C2' FCG"; $resn_exch{" O2* FCG"}=" O2' FCG"; $resn_exch{" C1* FCG"}=" C1' FCG";
$resn_exch{" C4A FCG"}=" C4X FCG"; $resn_exch{" C5A FCG"}=" C5X FCG"; $resn_exch{" AN1 FCX"}=" N1A FCX";
$resn_exch{" AC2 FCX"}=" C2A FCX"; $resn_exch{" AN3 FCX"}=" N3A FCX"; $resn_exch{" AC4 FCX"}=" C4A FCX";
$resn_exch{" AC5 FCX"}=" C5A FCX"; $resn_exch{" AC6 FCX"}=" C6A FCX"; $resn_exch{" AN6 FCX"}=" N6A FCX";
$resn_exch{" AN7 FCX"}=" N7A FCX"; $resn_exch{" AC8 FCX"}=" C8A FCX"; $resn_exch{" AN9 FCX"}=" N9A FCX";
$resn_exch{"AC1* FCX"}=" C1B FCX"; $resn_exch{"AC2* FCX"}=" C2B FCX"; $resn_exch{"AO2* FCX"}=" O2B FCX";
$resn_exch{"AC3* FCX"}=" C3B FCX"; $resn_exch{"AO3* FCX"}=" O3B FCX"; $resn_exch{"AP3* FCX"}=" P3B FCX";
$resn_exch{" AO7 FCX"}=" O7A FCX"; $resn_exch{" AO8 FCX"}=" O8A FCX"; $resn_exch{" AO9 FCX"}=" O9A FCX";
$resn_exch{"AC4* FCX"}=" C4B FCX"; $resn_exch{"AO4* FCX"}=" O4B FCX"; $resn_exch{"AC5* FCX"}=" C5B FCX";
$resn_exch{"AO5* FCX"}=" O5B FCX"; $resn_exch{" AP1 FCX"}=" P1A FCX"; $resn_exch{" AO1 FCX"}=" O1A FCX";
$resn_exch{" AO2 FCX"}=" O2A FCX"; $resn_exch{" AO3 FCX"}=" O3A FCX"; $resn_exch{" AP2 FCX"}=" P2A FCX";
$resn_exch{" AO4 FCX"}=" O4A FCX"; $resn_exch{" AO5 FCX"}=" O5A FCX"; $resn_exch{" AO6 FCX"}=" O6A FCX";
$resn_exch{"PC11 FCX"}=" CBP FCX"; $resn_exch{"PC12 FCX"}=" CCP FCX"; $resn_exch{"PC13 FCX"}=" CDP FCX";
$resn_exch{"PC14 FCX"}=" CEP FCX"; $resn_exch{"PC10 FCX"}=" CAP FCX"; $resn_exch{"PO10 FCX"}=" OAP FCX";
$resn_exch{" PC9 FCX"}=" C9P FCX"; $resn_exch{" PO9 FCX"}=" O9P FCX"; $resn_exch{" PN8 FCX"}=" N8P FCX";
$resn_exch{" PC7 FCX"}=" C7P FCX"; $resn_exch{" PC6 FCX"}=" C6P FCX"; $resn_exch{" PC5 FCX"}=" C5P FCX";
$resn_exch{" PO5 FCX"}=" O5P FCX"; $resn_exch{" PN4 FCX"}=" N4P FCX"; $resn_exch{" PC3 FCX"}=" C3P FCX";
$resn_exch{" PC2 FCX"}=" C2P FCX"; $resn_exch{" AP  FDA"}=" PA  FDA"; $resn_exch{" AO1 FDA"}=" O1A FDA";
$resn_exch{" AO2 FDA"}=" O2A FDA"; $resn_exch{"AO5* FDA"}=" O5B FDA"; $resn_exch{"AC5* FDA"}=" C5B FDA";
$resn_exch{"AC4* FDA"}=" C4B FDA"; $resn_exch{"AO4* FDA"}=" O4B FDA"; $resn_exch{"AC3* FDA"}=" C3B FDA";
$resn_exch{"AO3* FDA"}=" O3B FDA"; $resn_exch{"AC2* FDA"}=" C2B FDA"; $resn_exch{"AO2* FDA"}=" O2B FDA";
$resn_exch{"AC1* FDA"}=" C1B FDA"; $resn_exch{" AN9 FDA"}=" N9A FDA"; $resn_exch{" AC8 FDA"}=" C8A FDA";
$resn_exch{" AN7 FDA"}=" N7A FDA"; $resn_exch{" AC5 FDA"}=" C5A FDA"; $resn_exch{" AC6 FDA"}=" C6A FDA";
$resn_exch{" AN6 FDA"}=" N6A FDA"; $resn_exch{" AN1 FDA"}=" N1A FDA"; $resn_exch{" AC2 FDA"}=" C2A FDA";
$resn_exch{" AN3 FDA"}=" N3A FDA"; $resn_exch{" AC4 FDA"}=" C4A FDA"; $resn_exch{" C4A FDA"}=" C4X FDA";
$resn_exch{" C5A FDA"}=" C5X FDA"; $resn_exch{" C1* FDA"}=" C1' FDA"; $resn_exch{" C2* FDA"}=" C2' FDA";
$resn_exch{" O2* FDA"}=" O2' FDA"; $resn_exch{" C3* FDA"}=" C3' FDA"; $resn_exch{" O3* FDA"}=" O3' FDA";
$resn_exch{" C4* FDA"}=" C4' FDA"; $resn_exch{" O4* FDA"}=" O4' FDA"; $resn_exch{" C5* FDA"}=" C5' FDA";
$resn_exch{" O5* FDA"}=" O5' FDA"; $resn_exch{" N_D FDD"}=" ND  FDD"; $resn_exch{" N_A FDD"}=" NA  FDD";
$resn_exch{" N_C FDD"}=" NC  FDD"; $resn_exch{" N_B FDD"}=" NB  FDD"; $resn_exch{" N_A FDE"}=" NA  FDE";
$resn_exch{" N_B FDE"}=" NB  FDE"; $resn_exch{" N_C FDE"}=" NC  FDE"; $resn_exch{" N_D FDE"}=" ND  FDE";
$resn_exch{" O4* FDM"}=" O4' FDM"; $resn_exch{" C1* FDM"}=" C1' FDM"; $resn_exch{" C2* FDM"}=" C2' FDM";
$resn_exch{" C3* FDM"}=" C3' FDM"; $resn_exch{" C4* FDM"}=" C4' FDM"; $resn_exch{" O5* FDM"}=" O5' FDM";
$resn_exch{" C5* FDM"}=" C5' FDM"; $resn_exch{" C1* FFD"}=" C1' FFD"; $resn_exch{" O4* FFD"}=" O4' FFD";
$resn_exch{" C2* FFD"}=" C2' FFD"; $resn_exch{" C3* FFD"}=" C3' FFD"; $resn_exch{" O3* FFD"}=" O3' FFD";
$resn_exch{" C4* FFD"}=" C4' FFD"; $resn_exch{" C5* FFD"}=" C5' FFD"; $resn_exch{" O5* FFD"}=" O5' FFD";
$resn_exch{" C1* FHU"}=" C1' FHU"; $resn_exch{" C2* FHU"}=" C2' FHU"; $resn_exch{" O2* FHU"}=" O2' FHU";
$resn_exch{" C3* FHU"}=" C3' FHU"; $resn_exch{" C4* FHU"}=" C4' FHU"; $resn_exch{" O3* FHU"}=" O3' FHU";
$resn_exch{" O4* FHU"}=" O4' FHU"; $resn_exch{" C5* FHU"}=" C5' FHU"; $resn_exch{" O5* FHU"}=" O5' FHU";
$resn_exch{" O1P FHU"}=" OP1 FHU"; $resn_exch{" O2P FHU"}=" OP2 FHU"; $resn_exch{" O3P FHU"}=" OP3 FHU";
$resn_exch{" C3* FL8"}=" C3B FL8"; $resn_exch{" C1* FL9"}=" C1' FL9"; $resn_exch{" C2* FL9"}=" C2' FL9";
$resn_exch{" C3* FL9"}=" C3' FL9"; $resn_exch{" C4* FL9"}=" C4' FL9"; $resn_exch{" C5* FL9"}=" C5' FL9";
$resn_exch{" C6* FL9"}=" C6' FL9"; $resn_exch{" O2* FL9"}=" O2' FL9"; $resn_exch{" O4* FL9"}=" O4' FL9";
$resn_exch{" XC1 FLD"}=" C1X FLD"; $resn_exch{" XN2 FLD"}=" N2X FLD"; $resn_exch{" XC3 FLD"}=" C3X FLD";
$resn_exch{" XC4 FLD"}=" C4X FLD"; $resn_exch{" XC5 FLD"}=" C5X FLD"; $resn_exch{" XC6 FLD"}=" C6X FLD";
$resn_exch{" XN7 FLD"}=" N7X FLD"; $resn_exch{" XC8 FLD"}=" C8X FLD"; $resn_exch{" XC9 FLD"}=" C9X FLD";
$resn_exch{"XC10 FLD"}=" CAX FLD"; $resn_exch{"XO10 FLD"}=" OAX FLD"; $resn_exch{"XC11 FLD"}=" CBX FLD";
$resn_exch{"XC12 FLD"}=" CCX FLD"; $resn_exch{"XC13 FLD"}=" CDX FLD"; $resn_exch{"XC14 FLD"}=" CEX FLD";
$resn_exch{"XC15 FLD"}=" CFX FLD"; $resn_exch{"XC16 FLD"}=" CGX FLD"; $resn_exch{"XC17 FLD"}=" CHX FLD";
$resn_exch{"XC18 FLD"}=" CIX FLD"; $resn_exch{"XC19 FLD"}=" CJX FLD"; $resn_exch{"XC20 FLD"}=" CKX FLD";
$resn_exch{"XN21 FLD"}=" NLX FLD"; $resn_exch{"XC22 FLD"}=" CMX FLD"; $resn_exch{"XC23 FLD"}=" CNX FLD";
$resn_exch{"XC24 FLD"}=" COX FLD"; $resn_exch{" C1* FLN"}=" C1' FLN"; $resn_exch{" C2* FLN"}=" C2' FLN";
$resn_exch{" C3* FLN"}=" C3' FLN"; $resn_exch{" C4* FLN"}=" C4' FLN"; $resn_exch{" C5* FLN"}=" C5' FLN";
$resn_exch{" C6* FLN"}=" C6' FLN"; $resn_exch{" C1* FLQ"}=" C1' FLQ"; $resn_exch{" C2* FLQ"}=" C2' FLQ";
$resn_exch{" C3* FLQ"}=" C3' FLQ"; $resn_exch{" C4* FLQ"}=" C4' FLQ"; $resn_exch{" C5* FLQ"}=" C5' FLQ";
$resn_exch{" C6* FLQ"}=" C6' FLQ"; $resn_exch{" O8* FLQ"}=" O8' FLQ"; $resn_exch{" C9* FLQ"}=" C9' FLQ";
$resn_exch{" O9* FLQ"}=" O9' FLQ"; $resn_exch{" C1* FMA"}=" C1B FMA"; $resn_exch{" C2* FMA"}=" C2B FMA";
$resn_exch{" O2* FMA"}=" O2B FMA"; $resn_exch{" C3* FMA"}=" C3B FMA"; $resn_exch{" O3* FMA"}=" O3' FMA";
$resn_exch{" C4* FMA"}=" C4B FMA"; $resn_exch{" O4* FMA"}=" O4' FMA"; $resn_exch{" C5* FMA"}=" C5B FMA";
$resn_exch{" O5* FMA"}=" O5' FMA"; $resn_exch{" N_A FMI"}=" NA  FMI"; $resn_exch{" N_B FMI"}=" NB  FMI";
$resn_exch{" N_C FMI"}=" NC  FMI"; $resn_exch{" N_D FMI"}=" ND  FMI"; $resn_exch{" C1* FMN"}=" C1' FMN";
$resn_exch{" C2* FMN"}=" C2' FMN"; $resn_exch{" O2* FMN"}=" O2' FMN"; $resn_exch{" C3* FMN"}=" C3' FMN";
$resn_exch{" O3* FMN"}=" O3' FMN"; $resn_exch{" C4* FMN"}=" C4' FMN"; $resn_exch{" O4* FMN"}=" O4' FMN";
$resn_exch{" C5* FMN"}=" C5' FMN"; $resn_exch{" O5* FMN"}=" O5' FMN"; $resn_exch{" O5* FMP"}=" O5' FMP";
$resn_exch{" C5* FMP"}=" C5' FMP"; $resn_exch{" C4* FMP"}=" C4' FMP"; $resn_exch{" O4* FMP"}=" O4' FMP";
$resn_exch{" C3* FMP"}=" C3' FMP"; $resn_exch{" O3* FMP"}=" O3' FMP"; $resn_exch{" C2* FMP"}=" C2' FMP";
$resn_exch{" O2* FMP"}=" O2' FMP"; $resn_exch{" C1* FMP"}=" C1' FMP"; $resn_exch{" C1* FMU"}=" C1' FMU";
$resn_exch{" C2* FMU"}=" C2' FMU"; $resn_exch{" O2* FMU"}=" O2' FMU"; $resn_exch{" C3* FMU"}=" C3' FMU";
$resn_exch{" C4* FMU"}=" C4' FMU"; $resn_exch{" O3* FMU"}=" O3' FMU"; $resn_exch{" O4* FMU"}=" O4' FMU";
$resn_exch{" C5* FMU"}=" C5' FMU"; $resn_exch{" O5* FMU"}=" O5' FMU"; $resn_exch{"C10A FNR"}=" CAA FNR";
$resn_exch{" C1* FNR"}=" C1' FNR"; $resn_exch{" C2* FNR"}=" C2' FNR"; $resn_exch{" O2* FNR"}=" O2' FNR";
$resn_exch{" C3* FNR"}=" C3' FNR"; $resn_exch{" O3* FNR"}=" O3' FNR"; $resn_exch{" C4* FNR"}=" C4' FNR";
$resn_exch{" O4* FNR"}=" O4' FNR"; $resn_exch{" C5* FNR"}=" C5' FNR"; $resn_exch{" O5* FNR"}=" O5' FNR";
$resn_exch{" C1* FNS"}=" C1' FNS"; $resn_exch{" C2* FNS"}=" C2' FNS"; $resn_exch{" O2* FNS"}=" O2' FNS";
$resn_exch{" C3* FNS"}=" C3' FNS"; $resn_exch{" O3* FNS"}=" O3' FNS"; $resn_exch{" C4* FNS"}=" C4' FNS";
$resn_exch{" O4* FNS"}=" O4' FNS"; $resn_exch{" C5* FNS"}=" C5' FNS"; $resn_exch{" O5* FNS"}=" O5' FNS";
$resn_exch{" O5* FOX"}=" O5' FOX"; $resn_exch{" C5* FOX"}=" C5' FOX"; $resn_exch{" C4* FOX"}=" C4' FOX";
$resn_exch{" C6* FOX"}=" C6' FOX"; $resn_exch{" C3* FOX"}=" C3' FOX"; $resn_exch{" O3* FOX"}=" O3' FOX";
$resn_exch{" C2* FOX"}=" C2' FOX"; $resn_exch{" C1* FOX"}=" C1' FOX"; $resn_exch{" AN1 FRE"}=" N1A FRE";
$resn_exch{" AC2 FRE"}=" C2A FRE"; $resn_exch{" AN3 FRE"}=" N3A FRE"; $resn_exch{" AC4 FRE"}=" C4A FRE";
$resn_exch{" AC5 FRE"}=" C5A FRE"; $resn_exch{" AC6 FRE"}=" C6A FRE"; $resn_exch{" AN6 FRE"}=" N6A FRE";
$resn_exch{" AN7 FRE"}=" N7A FRE"; $resn_exch{" AC8 FRE"}=" C8A FRE"; $resn_exch{" AN9 FRE"}=" N9A FRE";
$resn_exch{"AC1* FRE"}=" C1B FRE"; $resn_exch{"AC2* FRE"}=" C2B FRE"; $resn_exch{"AO2* FRE"}=" O2B FRE";
$resn_exch{"AC3* FRE"}=" C3B FRE"; $resn_exch{"AO3* FRE"}=" O3B FRE"; $resn_exch{"AP3* FRE"}=" P3B FRE";
$resn_exch{" AO7 FRE"}=" O7A FRE"; $resn_exch{" AO8 FRE"}=" O8A FRE"; $resn_exch{" AO9 FRE"}=" O9A FRE";
$resn_exch{"AC4* FRE"}=" C4B FRE"; $resn_exch{"AO4* FRE"}=" O4B FRE"; $resn_exch{"AC5* FRE"}=" C5B FRE";
$resn_exch{"AO5* FRE"}=" O5B FRE"; $resn_exch{" AP1 FRE"}=" P1A FRE"; $resn_exch{" AO1 FRE"}=" O1A FRE";
$resn_exch{" AO2 FRE"}=" O2A FRE"; $resn_exch{" AO3 FRE"}=" O3A FRE"; $resn_exch{" AP2 FRE"}=" P2A FRE";
$resn_exch{" AO4 FRE"}=" O4A FRE"; $resn_exch{" AO5 FRE"}=" O5A FRE"; $resn_exch{" AO6 FRE"}=" O6A FRE";
$resn_exch{"PC11 FRE"}=" CBP FRE"; $resn_exch{"PC12 FRE"}=" CCP FRE"; $resn_exch{"PC13 FRE"}=" CDP FRE";
$resn_exch{"PC14 FRE"}=" CEP FRE"; $resn_exch{"PC10 FRE"}=" CAP FRE"; $resn_exch{"PO10 FRE"}=" OAP FRE";
$resn_exch{" PC9 FRE"}=" C9P FRE"; $resn_exch{" PO9 FRE"}=" O9P FRE"; $resn_exch{" PN8 FRE"}=" N8P FRE";
$resn_exch{" PC7 FRE"}=" C7P FRE"; $resn_exch{" PC6 FRE"}=" C6P FRE"; $resn_exch{" PC5 FRE"}=" C5P FRE";
$resn_exch{" PO5 FRE"}=" O5P FRE"; $resn_exch{" PN4 FRE"}=" N4P FRE"; $resn_exch{" PC3 FRE"}=" C3P FRE";
$resn_exch{" PC2 FRE"}=" C2P FRE"; $resn_exch{" PS1 FRE"}=" S1P FRE"; $resn_exch{"1OCT FRN"}=" O13 FRN";
$resn_exch{" AC2 FSM"}=" C2A FSM"; $resn_exch{" AC3 FSM"}=" C3A FSM"; $resn_exch{" AC4 FSM"}=" C4A FSM";
$resn_exch{" AC5 FSM"}=" C5A FSM"; $resn_exch{" AC6 FSM"}=" C6A FSM"; $resn_exch{" AC7 FSM"}=" C7A FSM";
$resn_exch{" AC8 FSM"}=" C8A FSM"; $resn_exch{" AN2 FSM"}=" N2A FSM"; $resn_exch{" AO1 FSM"}=" O1A FSM";
$resn_exch{" AO3 FSM"}=" O3A FSM"; $resn_exch{" AO4 FSM"}=" O4A FSM"; $resn_exch{" AO5 FSM"}=" O5A FSM";
$resn_exch{" AO6 FSM"}=" O6A FSM"; $resn_exch{" AO7 FSM"}=" O7A FSM"; $resn_exch{" BC1 FSM"}=" C1B FSM";
$resn_exch{" BC2 FSM"}=" C2B FSM"; $resn_exch{" BC3 FSM"}=" C3B FSM"; $resn_exch{" BC4 FSM"}=" C4B FSM";
$resn_exch{" BC5 FSM"}=" C5B FSM"; $resn_exch{" BC6 FSM"}=" C6B FSM"; $resn_exch{" BC7 FSM"}=" C7B FSM";
$resn_exch{" BC8 FSM"}=" C8B FSM"; $resn_exch{" BN2 FSM"}=" N2B FSM"; $resn_exch{" BO1 FSM"}=" O1B FSM";
$resn_exch{" BO3 FSM"}=" O3B FSM"; $resn_exch{" BO5 FSM"}=" O5B FSM"; $resn_exch{" BO6 FSM"}=" O6B FSM";
$resn_exch{" BO7 FSM"}=" O7B FSM"; $resn_exch{" O5* FUP"}=" O5' FUP"; $resn_exch{" C5* FUP"}=" C5' FUP";
$resn_exch{" C4* FUP"}=" C4' FUP"; $resn_exch{" O4* FUP"}=" O4' FUP"; $resn_exch{" C3* FUP"}=" C3' FUP";
$resn_exch{" F3* FUP"}=" F3' FUP"; $resn_exch{" C2* FUP"}=" C2' FUP"; $resn_exch{" C1* FUP"}=" C1' FUP";
$resn_exch{"1C   FVF"}=" C1  FVF"; $resn_exch{"1O   FVF"}=" O1  FVF"; $resn_exch{"1CB  FVF"}=" CB1 FVF";
$resn_exch{"1CG  FVF"}=" CG1 FVF"; $resn_exch{"1CD1 FVF"}=" CDB FVF"; $resn_exch{"1CD2 FVF"}=" CDC FVF";
$resn_exch{"1CE1 FVF"}=" CEB FVF"; $resn_exch{"1CE2 FVF"}=" CEC FVF"; $resn_exch{"1CZ  FVF"}=" CZ1 FVF";
$resn_exch{"2N   FVF"}=" N2  FVF"; $resn_exch{"2C   FVF"}=" C2  FVF"; $resn_exch{"2O   FVF"}=" O2  FVF";
$resn_exch{"2CA  FVF"}=" CA2 FVF"; $resn_exch{"2CB  FVF"}=" CB2 FVF"; $resn_exch{"2CG  FVF"}=" CG2 FVF";
$resn_exch{"2CD1 FVF"}=" CDL FVF"; $resn_exch{"2CD2 FVF"}=" CDM FVF"; $resn_exch{"2CE1 FVF"}=" CEL FVF";
$resn_exch{"2CE2 FVF"}=" CEM FVF"; $resn_exch{"2CZ  FVF"}=" CZ2 FVF"; $resn_exch{"3N   FVF"}=" N3  FVF";
$resn_exch{"3CA  FVF"}=" CA3 FVF"; $resn_exch{"3CB  FVF"}=" CB3 FVF"; $resn_exch{"3CG1 FVF"}=" CGD FVF";
$resn_exch{"3CG2 FVF"}=" CGN FVF"; $resn_exch{"3P   FVF"}=" P3  FVF"; $resn_exch{"1OP3 FVF"}=" OPV FVF";
$resn_exch{"2OP3 FVF"}=" OPW FVF"; $resn_exch{"3OP3 FVF"}=" OPX FVF"; $resn_exch{"4CA  FVF"}=" CA4 FVF";
$resn_exch{"4C   FVF"}=" C4  FVF"; $resn_exch{"4O   FVF"}=" O4  FVF"; $resn_exch{"4OXT FVF"}=" O11 FVF";
$resn_exch{"4CB  FVF"}=" CB4 FVF"; $resn_exch{"4CG  FVF"}=" CG4 FVF"; $resn_exch{"4CD1 FVF"}=" CD5 FVF";
$resn_exch{"4CD2 FVF"}=" CD6 FVF"; $resn_exch{"4CE1 FVF"}=" CE5 FVF"; $resn_exch{"4CE2 FVF"}=" CE6 FVF";
$resn_exch{"4CZ  FVF"}=" CZ4 FVF"; $resn_exch{" O5* FX3"}=" O5' FX3"; $resn_exch{" C5* FX3"}=" C5' FX3";
$resn_exch{" C4* FX3"}=" C4' FX3"; $resn_exch{" O4* FX3"}=" O4' FX3"; $resn_exch{" C3* FX3"}=" C3' FX3";
$resn_exch{" O3* FX3"}=" O3' FX3"; $resn_exch{" C2* FX3"}=" C2' FX3"; $resn_exch{" O2* FX3"}=" O2' FX3";
$resn_exch{" C1* FX3"}=" C1' FX3"; $resn_exch{" O5* FYA"}=" O5' FYA"; $resn_exch{" C5* FYA"}=" C5' FYA";
$resn_exch{" C4* FYA"}=" C4' FYA"; $resn_exch{" O4* FYA"}=" O4' FYA"; $resn_exch{" C3* FYA"}=" C3' FYA";
$resn_exch{" O3* FYA"}=" O3' FYA"; $resn_exch{" C2* FYA"}=" C2' FYA"; $resn_exch{" O2* FYA"}=" O2' FYA";
$resn_exch{" C1* FYA"}=" C1' FYA"; $resn_exch{" O3P   G"}=" OP3   G"; $resn_exch{" O1P   G"}=" OP1   G";
$resn_exch{" O2P   G"}=" OP2   G"; $resn_exch{" O5*   G"}=" O5'   G"; $resn_exch{" C5*   G"}=" C5'   G";
$resn_exch{" C4*   G"}=" C4'   G"; $resn_exch{" O4*   G"}=" O4'   G"; $resn_exch{" C3*   G"}=" C3'   G";
$resn_exch{" O3*   G"}=" O3'   G"; $resn_exch{" C2*   G"}=" C2'   G"; $resn_exch{" O2*   G"}=" O2'   G";
$resn_exch{" C1*   G"}=" C1'   G"; $resn_exch{" P*  G16"}=" P'  G16"; $resn_exch{"O1P* G16"}=" O1X G16";
$resn_exch{"O2P* G16"}=" O2X G16"; $resn_exch{"O3P* G16"}=" O3X G16"; $resn_exch{" C1* G1R"}=" C1' G1R";
$resn_exch{" O4* G1R"}=" O4' G1R"; $resn_exch{" C2* G1R"}=" C2' G1R"; $resn_exch{" O2* G1R"}=" O2' G1R";
$resn_exch{" C3* G1R"}=" C3' G1R"; $resn_exch{" O3* G1R"}=" O3' G1R"; $resn_exch{" C4* G1R"}=" C4' G1R";
$resn_exch{" C5* G1R"}=" C5' G1R"; $resn_exch{" O5* G1R"}=" O5' G1R"; $resn_exch{" O3P G25"}=" OP3 G25";
$resn_exch{" O1P G25"}=" OP1 G25"; $resn_exch{" O2P G25"}=" OP2 G25"; $resn_exch{" O5* G25"}=" O5' G25";
$resn_exch{" C5* G25"}=" C5' G25"; $resn_exch{" C4* G25"}=" C4' G25"; $resn_exch{" O4* G25"}=" O4' G25";
$resn_exch{" C3* G25"}=" C3' G25"; $resn_exch{" O3* G25"}=" O3' G25"; $resn_exch{" C2* G25"}=" C2' G25";
$resn_exch{" O2* G25"}=" O2' G25"; $resn_exch{" C1* G25"}=" C1' G25"; $resn_exch{" O3P G2L"}=" OP3 G2L";
$resn_exch{" O1P G2L"}=" OP1 G2L"; $resn_exch{" O2P G2L"}=" OP2 G2L"; $resn_exch{" O5* G2L"}=" O5' G2L";
$resn_exch{" C5* G2L"}=" C5' G2L"; $resn_exch{" C4* G2L"}=" C4' G2L"; $resn_exch{" O4* G2L"}=" O4' G2L";
$resn_exch{" C3* G2L"}=" C3' G2L"; $resn_exch{" O3* G2L"}=" O3' G2L"; $resn_exch{" C2* G2L"}=" C2' G2L";
$resn_exch{" O2* G2L"}=" O2' G2L"; $resn_exch{" C1* G2L"}=" C1' G2L"; $resn_exch{" CA* G2L"}=" CA' G2L";
$resn_exch{" CB* G2L"}=" CB' G2L"; $resn_exch{" OC* G2L"}=" OC' G2L"; $resn_exch{" CD* G2L"}=" CD' G2L";
$resn_exch{" O5* G2P"}=" O5' G2P"; $resn_exch{" C5* G2P"}=" C5' G2P"; $resn_exch{" C4* G2P"}=" C4' G2P";
$resn_exch{" O4* G2P"}=" O4' G2P"; $resn_exch{" C3* G2P"}=" C3' G2P"; $resn_exch{" O3* G2P"}=" O3' G2P";
$resn_exch{" C2* G2P"}=" C2' G2P"; $resn_exch{" O2* G2P"}=" O2' G2P"; $resn_exch{" C1* G2P"}=" C1' G2P";
$resn_exch{" C1* G2R"}=" C1' G2R"; $resn_exch{" O4* G2R"}=" O4' G2R"; $resn_exch{" C2* G2R"}=" C2' G2R";
$resn_exch{" O2* G2R"}=" O2' G2R"; $resn_exch{" C3* G2R"}=" C3' G2R"; $resn_exch{" O3* G2R"}=" O3' G2R";
$resn_exch{" C4* G2R"}=" C4' G2R"; $resn_exch{" C5* G2R"}=" C5' G2R"; $resn_exch{" O5* G2R"}=" O5' G2R";
$resn_exch{" O3P G2S"}=" OP3 G2S"; $resn_exch{" O5* G2S"}=" O5' G2S"; $resn_exch{" C5* G2S"}=" C5' G2S";
$resn_exch{" C4* G2S"}=" C4' G2S"; $resn_exch{" O4* G2S"}=" O4' G2S"; $resn_exch{" C3* G2S"}=" C3' G2S";
$resn_exch{" O3* G2S"}=" O3' G2S"; $resn_exch{" C2* G2S"}=" C2' G2S"; $resn_exch{" C1* G2S"}=" C1' G2S";
$resn_exch{" O5* G31"}=" O5' G31"; $resn_exch{" C5* G31"}=" C5' G31"; $resn_exch{" C4* G31"}=" C4' G31";
$resn_exch{" O4* G31"}=" O4' G31"; $resn_exch{" C3* G31"}=" C3' G31"; $resn_exch{" C2* G31"}=" C2' G31";
$resn_exch{" C1* G31"}=" C1' G31"; $resn_exch{" O5* G32"}=" O5' G32"; $resn_exch{" C5* G32"}=" C5' G32";
$resn_exch{" C4* G32"}=" C4' G32"; $resn_exch{" O4* G32"}=" O4' G32"; $resn_exch{" C3* G32"}=" C3' G32";
$resn_exch{" O3* G32"}=" O3' G32"; $resn_exch{" C2* G32"}=" C2' G32"; $resn_exch{" C1* G32"}=" C1' G32";
$resn_exch{" O5* G33"}=" O5' G33"; $resn_exch{" C5* G33"}=" C5' G33"; $resn_exch{" C4* G33"}=" C4' G33";
$resn_exch{" O4* G33"}=" O4' G33"; $resn_exch{" C3* G33"}=" C3' G33"; $resn_exch{" O3* G33"}=" O3' G33";
$resn_exch{" C2* G33"}=" C2' G33"; $resn_exch{" C1* G33"}=" C1' G33"; $resn_exch{" O5* G36"}=" O5' G36";
$resn_exch{" C5* G36"}=" C5' G36"; $resn_exch{" C4* G36"}=" C4' G36"; $resn_exch{" O4* G36"}=" O4' G36";
$resn_exch{" C3* G36"}=" C3' G36"; $resn_exch{" O3* G36"}=" O3' G36"; $resn_exch{" C2* G36"}=" C2' G36";
$resn_exch{" C1* G36"}=" C1' G36"; $resn_exch{" O5* G38"}=" O5' G38"; $resn_exch{" C5* G38"}=" C5' G38";
$resn_exch{" C4* G38"}=" C4' G38"; $resn_exch{" O4* G38"}=" O4' G38"; $resn_exch{" C3* G38"}=" C3' G38";
$resn_exch{" C2* G38"}=" C2' G38"; $resn_exch{" C1* G38"}=" C1' G38"; $resn_exch{" O5* G3D"}=" O5' G3D";
$resn_exch{" C5* G3D"}=" C5' G3D"; $resn_exch{" C4* G3D"}=" C4' G3D"; $resn_exch{" O4* G3D"}=" O4' G3D";
$resn_exch{" C3* G3D"}=" C3' G3D"; $resn_exch{" O3* G3D"}=" O3' G3D"; $resn_exch{" C2* G3D"}=" C2' G3D";
$resn_exch{" O2* G3D"}=" O2' G3D"; $resn_exch{" C1* G3D"}=" C1' G3D"; $resn_exch{" O5* G42"}=" O5' G42";
$resn_exch{" C5* G42"}=" C5' G42"; $resn_exch{" C4* G42"}=" C4' G42"; $resn_exch{" O4* G42"}=" O4' G42";
$resn_exch{" C3* G42"}=" C3' G42"; $resn_exch{" O3* G42"}=" O3' G42"; $resn_exch{" C2* G42"}=" C2' G42";
$resn_exch{" C1* G42"}=" C1' G42"; $resn_exch{" O5* G46"}=" O5' G46"; $resn_exch{" C5* G46"}=" C5' G46";
$resn_exch{" C4* G46"}=" C4' G46"; $resn_exch{" O4* G46"}=" O4' G46"; $resn_exch{" C3* G46"}=" C3' G46";
$resn_exch{" O3* G46"}=" O3' G46"; $resn_exch{" C2* G46"}=" C2' G46"; $resn_exch{" C1* G46"}=" C1' G46";
$resn_exch{" O2* G46"}=" O2' G46"; $resn_exch{" O5* G47"}=" O5' G47"; $resn_exch{" C5* G47"}=" C5' G47";
$resn_exch{" C4* G47"}=" C4' G47"; $resn_exch{" O4* G47"}=" O4' G47"; $resn_exch{" C3* G47"}=" C3' G47";
$resn_exch{" O3* G47"}=" O3' G47"; $resn_exch{" C2* G47"}=" C2' G47"; $resn_exch{" C1* G47"}=" C1' G47";
$resn_exch{" O5* G48"}=" O5' G48"; $resn_exch{" C5* G48"}=" C5' G48"; $resn_exch{" C4* G48"}=" C4' G48";
$resn_exch{" O4* G48"}=" O4' G48"; $resn_exch{" C3* G48"}=" C3' G48"; $resn_exch{" O3* G48"}=" O3' G48";
$resn_exch{" C2* G48"}=" C2' G48"; $resn_exch{" O2* G48"}=" O2' G48"; $resn_exch{" C1* G48"}=" C1' G48";
$resn_exch{" CA* G48"}=" CA' G48"; $resn_exch{" CB* G48"}=" CB' G48"; $resn_exch{" OC* G48"}=" OC' G48";
$resn_exch{" CD* G48"}=" CD' G48"; $resn_exch{" O5* G49"}=" O5' G49"; $resn_exch{" C5* G49"}=" C5' G49";
$resn_exch{" C4* G49"}=" C4' G49"; $resn_exch{" O4* G49"}=" O4' G49"; $resn_exch{" C3* G49"}=" C3' G49";
$resn_exch{" O3* G49"}=" O3' G49"; $resn_exch{" C2* G49"}=" C2' G49"; $resn_exch{" C1* G49"}=" C1' G49";
$resn_exch{" O5* G4M"}=" O5' G4M"; $resn_exch{" C5* G4M"}=" C5' G4M"; $resn_exch{" C4* G4M"}=" C4' G4M";
$resn_exch{" O4* G4M"}=" O4' G4M"; $resn_exch{" C3* G4M"}=" C3' G4M"; $resn_exch{" O3* G4M"}=" O3' G4M";
$resn_exch{" C2* G4M"}=" C2' G4M"; $resn_exch{" O2* G4M"}=" O2' G4M"; $resn_exch{" C1* G4M"}=" C1' G4M";
$resn_exch{" O5* G4P"}=" O5' G4P"; $resn_exch{" C5* G4P"}=" C5' G4P"; $resn_exch{" C4* G4P"}=" C4' G4P";
$resn_exch{" O4* G4P"}=" O4' G4P"; $resn_exch{" C3* G4P"}=" C3' G4P"; $resn_exch{" O3* G4P"}=" O3' G4P";
$resn_exch{" C2* G4P"}=" C2' G4P"; $resn_exch{" O2* G4P"}=" O2' G4P"; $resn_exch{" C1* G4P"}=" C1' G4P";
$resn_exch{" O5* G5P"}=" O5' G5P"; $resn_exch{" C5* G5P"}=" C5' G5P"; $resn_exch{" C4* G5P"}=" C4' G5P";
$resn_exch{" O4* G5P"}=" O4' G5P"; $resn_exch{" C3* G5P"}=" C3' G5P"; $resn_exch{" O3* G5P"}=" O3' G5P";
$resn_exch{" C2* G5P"}=" C2' G5P"; $resn_exch{" O2* G5P"}=" O2' G5P"; $resn_exch{" C1* G5P"}=" C1' G5P";
$resn_exch{"AO5* G5P"}=" O5F G5P"; $resn_exch{"AC5* G5P"}=" C5F G5P"; $resn_exch{"AC4* G5P"}=" C4F G5P";
$resn_exch{"AO4* G5P"}=" O4F G5P"; $resn_exch{"AC1* G5P"}=" C1F G5P"; $resn_exch{"AC2* G5P"}=" C2F G5P";
$resn_exch{"AO2* G5P"}=" O2F G5P"; $resn_exch{"AC3* G5P"}=" C3F G5P"; $resn_exch{"AO3* G5P"}=" O3F G5P";
$resn_exch{" AN9 G5P"}=" N9A G5P"; $resn_exch{" AC4 G5P"}=" C4A G5P"; $resn_exch{" AN3 G5P"}=" N3A G5P";
$resn_exch{" AC2 G5P"}=" C2A G5P"; $resn_exch{" AN1 G5P"}=" N1A G5P"; $resn_exch{" AC6 G5P"}=" C6A G5P";
$resn_exch{" AN6 G5P"}=" N6A G5P"; $resn_exch{" AC5 G5P"}=" C5A G5P"; $resn_exch{" AN7 G5P"}=" N7A G5P";
$resn_exch{" AC8 G5P"}=" C8A G5P"; $resn_exch{" O3P G7M"}=" OP3 G7M"; $resn_exch{" O1P G7M"}=" OP1 G7M";
$resn_exch{" O2P G7M"}=" OP2 G7M"; $resn_exch{" O5* G7M"}=" O5' G7M"; $resn_exch{" C5* G7M"}=" C5' G7M";
$resn_exch{" C4* G7M"}=" C4' G7M"; $resn_exch{" O4* G7M"}=" O4' G7M"; $resn_exch{" C3* G7M"}=" C3' G7M";
$resn_exch{" O3* G7M"}=" O3' G7M"; $resn_exch{" C2* G7M"}=" C2' G7M"; $resn_exch{" O2* G7M"}=" O2' G7M";
$resn_exch{" C1* G7M"}=" C1' G7M"; $resn_exch{" C4* GA2"}=" C4' GA2"; $resn_exch{" O4* GA2"}=" O4' GA2";
$resn_exch{" C3* GA2"}=" C3' GA2"; $resn_exch{" O3* GA2"}=" O3' GA2"; $resn_exch{" C2* GA2"}=" C2' GA2";
$resn_exch{" O1* GA2"}=" O1' GA2"; $resn_exch{" C1* GA2"}=" C1' GA2"; $resn_exch{" O3P GAO"}=" OP3 GAO";
$resn_exch{" O1P GAO"}=" OP1 GAO"; $resn_exch{" O2P GAO"}=" OP2 GAO"; $resn_exch{" O5* GAO"}=" O5' GAO";
$resn_exch{" C5* GAO"}=" C5' GAO"; $resn_exch{" C4* GAO"}=" C4' GAO"; $resn_exch{" O4* GAO"}=" O4' GAO";
$resn_exch{" C3* GAO"}=" C3' GAO"; $resn_exch{" O3* GAO"}=" O3' GAO"; $resn_exch{" C2* GAO"}=" C2' GAO";
$resn_exch{" O2* GAO"}=" O2' GAO"; $resn_exch{" C1* GAO"}=" C1' GAO"; $resn_exch{" C5* GAP"}=" C5' GAP";
$resn_exch{" O5* GAP"}=" O5' GAP"; $resn_exch{" C4* GAP"}=" C4' GAP"; $resn_exch{" O4* GAP"}=" O4' GAP";
$resn_exch{" C3* GAP"}=" C3' GAP"; $resn_exch{" O3* GAP"}=" O3' GAP"; $resn_exch{" C2* GAP"}=" C2' GAP";
$resn_exch{" O2* GAP"}=" O2' GAP"; $resn_exch{" C1* GAP"}=" C1' GAP"; $resn_exch{" C1* GAV"}=" C1' GAV";
$resn_exch{" C2* GAV"}=" C2' GAV"; $resn_exch{" O2* GAV"}=" O2' GAV"; $resn_exch{" C3* GAV"}=" C3' GAV";
$resn_exch{" O3* GAV"}=" O3' GAV"; $resn_exch{" O4* GAV"}=" O4' GAV"; $resn_exch{" C4* GAV"}=" C4' GAV";
$resn_exch{" C5* GAV"}=" C5' GAV"; $resn_exch{" O5* GAV"}=" O5' GAV"; $resn_exch{"N10S GCG"}=" N11 GCG";
$resn_exch{" O5* GCK"}=" O5' GCK"; $resn_exch{" C2* GCK"}=" C2' GCK"; $resn_exch{" C5* GCK"}=" C5' GCK";
$resn_exch{" C4* GCK"}=" C4' GCK"; $resn_exch{" O4* GCK"}=" O4' GCK"; $resn_exch{" C1* GCK"}=" C1' GCK";
$resn_exch{" C3* GCK"}=" C3' GCK"; $resn_exch{" O3* GCK"}=" O3' GCK"; $resn_exch{" O5* GCP"}=" O5' GCP";
$resn_exch{" C5* GCP"}=" C5' GCP"; $resn_exch{" C4* GCP"}=" C4' GCP"; $resn_exch{" O4* GCP"}=" O4' GCP";
$resn_exch{" C3* GCP"}=" C3' GCP"; $resn_exch{" O3* GCP"}=" O3' GCP"; $resn_exch{" C2* GCP"}=" C2' GCP";
$resn_exch{" O2* GCP"}=" O2' GCP"; $resn_exch{" C1* GCP"}=" C1' GCP"; $resn_exch{" O5* GCQ"}=" O5' GCQ";
$resn_exch{" C5* GCQ"}=" C5' GCQ"; $resn_exch{" C4* GCQ"}=" C4' GCQ"; $resn_exch{" C3* GCQ"}=" C3' GCQ";
$resn_exch{" O3* GCQ"}=" O3' GCQ"; $resn_exch{" C2* GCQ"}=" C2' GCQ"; $resn_exch{" O4* GCQ"}=" O4' GCQ";
$resn_exch{" C1* GCQ"}=" C1' GCQ"; $resn_exch{" O5* GDC"}=" O5' GDC"; $resn_exch{" C5* GDC"}=" C5' GDC";
$resn_exch{" C4* GDC"}=" C4' GDC"; $resn_exch{" O4* GDC"}=" O4' GDC"; $resn_exch{" C3* GDC"}=" C3' GDC";
$resn_exch{" O3* GDC"}=" O3' GDC"; $resn_exch{" C2* GDC"}=" C2' GDC"; $resn_exch{" O2* GDC"}=" O2' GDC";
$resn_exch{" C1* GDC"}=" C1' GDC"; $resn_exch{"AO1P GDC"}=" O1X GDC"; $resn_exch{"AO2P GDC"}=" O2X GDC";
$resn_exch{" C1* GDD"}=" C1' GDD"; $resn_exch{" C2* GDD"}=" C2' GDD"; $resn_exch{" O2* GDD"}=" O2' GDD";
$resn_exch{" C3* GDD"}=" C3' GDD"; $resn_exch{" O3* GDD"}=" O3' GDD"; $resn_exch{" C4* GDD"}=" C4' GDD";
$resn_exch{" O4* GDD"}=" O4' GDD"; $resn_exch{" C5* GDD"}=" C5' GDD"; $resn_exch{" O5* GDD"}=" O5' GDD";
$resn_exch{" O5* GDP"}=" O5' GDP"; $resn_exch{" C5* GDP"}=" C5' GDP"; $resn_exch{" C4* GDP"}=" C4' GDP";
$resn_exch{" O4* GDP"}=" O4' GDP"; $resn_exch{" C3* GDP"}=" C3' GDP"; $resn_exch{" O3* GDP"}=" O3' GDP";
$resn_exch{" C2* GDP"}=" C2' GDP"; $resn_exch{" O2* GDP"}=" O2' GDP"; $resn_exch{" C1* GDP"}=" C1' GDP";
$resn_exch{" O5* GDR"}=" O5D GDR"; $resn_exch{" C5* GDR"}=" C5D GDR"; $resn_exch{" C4* GDR"}=" C4D GDR";
$resn_exch{" O4* GDR"}=" O4D GDR"; $resn_exch{" C1* GDR"}=" C1D GDR"; $resn_exch{" C2* GDR"}=" C2D GDR";
$resn_exch{" O2* GDR"}=" O2D GDR"; $resn_exch{" C3* GDR"}=" C3D GDR"; $resn_exch{" O3* GDR"}=" O3D GDR";
$resn_exch{" C1* GDU"}=" C1D GDU"; $resn_exch{" C2* GDU"}=" C2D GDU"; $resn_exch{" O2* GDU"}=" O2D GDU";
$resn_exch{" C3* GDU"}=" C3D GDU"; $resn_exch{" C4* GDU"}=" C4D GDU"; $resn_exch{" O4* GDU"}=" O4D GDU";
$resn_exch{" O3* GDU"}=" O3D GDU"; $resn_exch{" C5* GDU"}=" C5D GDU"; $resn_exch{" O5* GDU"}=" O5D GDU";
$resn_exch{" C1* GDX"}=" C1D GDX"; $resn_exch{" C2* GDX"}=" C2D GDX"; $resn_exch{" O2* GDX"}=" O2D GDX";
$resn_exch{" C3* GDX"}=" C3D GDX"; $resn_exch{" O3* GDX"}=" O3D GDX"; $resn_exch{" C4* GDX"}=" C4D GDX";
$resn_exch{" O4* GDX"}=" O4D GDX"; $resn_exch{" C5* GDX"}=" C5D GDX"; $resn_exch{" O5* GDX"}=" O5D GDX";
$resn_exch{" O5* GFL"}=" O5' GFL"; $resn_exch{" C5* GFL"}=" C5' GFL"; $resn_exch{" C4* GFL"}=" C4' GFL";
$resn_exch{" O4* GFL"}=" O4' GFL"; $resn_exch{" C3* GFL"}=" C3' GFL"; $resn_exch{" O3* GFL"}=" O3' GFL";
$resn_exch{" C2* GFL"}=" C2' GFL"; $resn_exch{" C1* GFL"}=" C1' GFL"; $resn_exch{" O2* GGZ"}=" O2' GGZ";
$resn_exch{" O3* GGZ"}=" O3' GGZ"; $resn_exch{" C3* GGZ"}=" C3' GGZ"; $resn_exch{" C4* GGZ"}=" C4' GGZ";
$resn_exch{" C5* GGZ"}=" C5' GGZ"; $resn_exch{" O5* GGZ"}=" O5' GGZ"; $resn_exch{" C2* GGZ"}=" C2' GGZ";
$resn_exch{" C1* GGZ"}=" C1' GGZ"; $resn_exch{" O4* GGZ"}=" O4' GGZ"; $resn_exch{" O5* GH3"}=" O5' GH3";
$resn_exch{" C5* GH3"}=" C5' GH3"; $resn_exch{" C4* GH3"}=" C4' GH3"; $resn_exch{" O4* GH3"}=" O4' GH3";
$resn_exch{" C3* GH3"}=" C3' GH3"; $resn_exch{" C2* GH3"}=" C2' GH3"; $resn_exch{" O2* GH3"}=" O2' GH3";
$resn_exch{" C1* GH3"}=" C1' GH3"; $resn_exch{" C1* GKD"}=" C1' GKD"; $resn_exch{" C2* GKD"}=" C2' GKD";
$resn_exch{" O2* GKD"}=" O2' GKD"; $resn_exch{" C3* GKD"}=" C3' GKD"; $resn_exch{" O3* GKD"}=" O3' GKD";
$resn_exch{" C4* GKD"}=" C4' GKD"; $resn_exch{" O4* GKD"}=" O4' GKD"; $resn_exch{" C5* GKD"}=" C5' GKD";
$resn_exch{" O5* GKD"}=" O5' GKD"; $resn_exch{" C1* GKE"}=" C1' GKE"; $resn_exch{" C2* GKE"}=" C2' GKE";
$resn_exch{" O2* GKE"}=" O2' GKE"; $resn_exch{" C3* GKE"}=" C3' GKE"; $resn_exch{" O3* GKE"}=" O3' GKE";
$resn_exch{" C4* GKE"}=" C4' GKE"; $resn_exch{" O4* GKE"}=" O4' GKE"; $resn_exch{" C5* GKE"}=" C5' GKE";
$resn_exch{" O5* GKE"}=" O5' GKE"; $resn_exch{"CD31 GLI"}=" CDV GLI"; $resn_exch{"CD32 GLI"}=" CDW GLI";
$resn_exch{"CE31 GLI"}=" CEV GLI"; $resn_exch{"CE32 GLI"}=" CEW GLI"; $resn_exch{"CG51 GLI"}=" CGF GLI";
$resn_exch{"CG52 GLI"}=" CGG GLI"; $resn_exch{"CG61 GLI"}=" CGP GLI"; $resn_exch{"CG62 GLI"}=" CGQ GLI";
$resn_exch{" AE1 GLX"}=" XE1 GLX"; $resn_exch{" AE2 GLX"}=" XE2 GLX"; $resn_exch{"CR10 GM2"}=" CRA GM2";
$resn_exch{"CR11 GM2"}=" CRB GM2"; $resn_exch{"CR12 GM2"}=" CRC GM2"; $resn_exch{"CR13 GM2"}=" CRD GM2";
$resn_exch{"CR14 GM2"}=" CRE GM2"; $resn_exch{"CR15 GM2"}=" CRF GM2"; $resn_exch{"CR16 GM2"}=" CRG GM2";
$resn_exch{"CR17 GM2"}=" CRH GM2"; $resn_exch{"CL10 GM2"}=" CLA GM2"; $resn_exch{"CL11 GM2"}=" CLB GM2";
$resn_exch{"CL12 GM2"}=" CLC GM2"; $resn_exch{"CL13 GM2"}=" CLD GM2"; $resn_exch{"CL14 GM2"}=" CLE GM2";
$resn_exch{"CL15 GM2"}=" CLF GM2"; $resn_exch{"CL16 GM2"}=" CLG GM2"; $resn_exch{"CL17 GM2"}=" CLH GM2";
$resn_exch{"CL18 GM2"}=" CLI GM2"; $resn_exch{" C1* GMC"}=" C1' GMC"; $resn_exch{" O4* GMC"}=" O4' GMC";
$resn_exch{" C4* GMC"}=" C4' GMC"; $resn_exch{" C5* GMC"}=" C5' GMC"; $resn_exch{" O5* GMC"}=" O5' GMC";
$resn_exch{" C2* GMC"}=" C2' GMC"; $resn_exch{" O2* GMC"}=" O2' GMC"; $resn_exch{" C3* GMC"}=" C3' GMC";
$resn_exch{" N3* GMC"}=" N3' GMC"; $resn_exch{" O5* GMP"}=" O5' GMP"; $resn_exch{" C5* GMP"}=" C5' GMP";
$resn_exch{" C4* GMP"}=" C4' GMP"; $resn_exch{" O4* GMP"}=" O4' GMP"; $resn_exch{" C3* GMP"}=" C3' GMP";
$resn_exch{" O3* GMP"}=" O3' GMP"; $resn_exch{" C2* GMP"}=" C2' GMP"; $resn_exch{" O2* GMP"}=" O2' GMP";
$resn_exch{" C1* GMP"}=" C1' GMP"; $resn_exch{" O1P GMS"}=" OP1 GMS"; $resn_exch{" O5* GMS"}=" O5' GMS";
$resn_exch{" C2* GMS"}=" C2' GMS"; $resn_exch{" C5* GMS"}=" C5' GMS"; $resn_exch{" C4* GMS"}=" C4' GMS";
$resn_exch{" O4* GMS"}=" O4' GMS"; $resn_exch{" C1* GMS"}=" C1' GMS"; $resn_exch{" C3* GMS"}=" C3' GMS";
$resn_exch{" O3* GMS"}=" O3' GMS"; $resn_exch{" O2P GMS"}=" OP2 GMS"; $resn_exch{" O3P GMU"}=" OP3 GMU";
$resn_exch{" O2P GMU"}=" OP2 GMU"; $resn_exch{" O5* GMU"}=" O5' GMU"; $resn_exch{" C5* GMU"}=" C5' GMU";
$resn_exch{" C4* GMU"}=" C4' GMU"; $resn_exch{" O4* GMU"}=" O4' GMU"; $resn_exch{" C3* GMU"}=" C3' GMU";
$resn_exch{" O3* GMU"}=" O3' GMU"; $resn_exch{" C2* GMU"}=" C2' GMU"; $resn_exch{" C1* GMU"}=" C1' GMU";
$resn_exch{" O2* GMU"}=" O2' GMU"; $resn_exch{" CA* GMU"}=" CA' GMU"; $resn_exch{" CB* GMU"}=" CB' GMU";
$resn_exch{" NC* GMU"}=" NC' GMU"; $resn_exch{" CD* GMU"}=" CD' GMU"; $resn_exch{" NE* GMU"}=" NE' GMU";
$resn_exch{" NF* GMU"}=" NF' GMU"; $resn_exch{" O1P GMU"}=" OP1 GMU"; $resn_exch{" O1P GN7"}=" OP1 GN7";
$resn_exch{" O2P GN7"}=" OP2 GN7"; $resn_exch{" O3P GN7"}=" OP3 GN7"; $resn_exch{" O5* GN7"}=" O5' GN7";
$resn_exch{" C5* GN7"}=" C5' GN7"; $resn_exch{" C4* GN7"}=" C4' GN7"; $resn_exch{" O4* GN7"}=" O4' GN7";
$resn_exch{" C3* GN7"}=" C3' GN7"; $resn_exch{" O3* GN7"}=" O3' GN7"; $resn_exch{" C2* GN7"}=" C2' GN7";
$resn_exch{" C1* GN7"}=" C1' GN7"; $resn_exch{" O1P GNE"}=" OP1 GNE"; $resn_exch{" O2P GNE"}=" OP2 GNE";
$resn_exch{" O3P GNE"}=" OP3 GNE"; $resn_exch{" O5* GNE"}=" O5' GNE"; $resn_exch{" C5* GNE"}=" C5' GNE";
$resn_exch{" C4* GNE"}=" C4' GNE"; $resn_exch{" O4* GNE"}=" O4' GNE"; $resn_exch{" C3* GNE"}=" C3' GNE";
$resn_exch{" O3* GNE"}=" O3' GNE"; $resn_exch{" C2* GNE"}=" C2' GNE"; $resn_exch{" C1* GNE"}=" C1' GNE";
$resn_exch{" O5* GNG"}=" O5' GNG"; $resn_exch{" C5* GNG"}=" C5' GNG"; $resn_exch{" C4* GNG"}=" C4' GNG";
$resn_exch{" O4* GNG"}=" O4' GNG"; $resn_exch{" C1* GNG"}=" C1' GNG"; $resn_exch{" C2* GNG"}=" C2' GNG";
$resn_exch{" C3* GNG"}=" C3' GNG"; $resn_exch{" O3* GNG"}=" O3' GNG"; $resn_exch{" O5* GNH"}=" O5' GNH";
$resn_exch{" C5* GNH"}=" C5' GNH"; $resn_exch{" C4* GNH"}=" C4' GNH"; $resn_exch{" O4* GNH"}=" O4' GNH";
$resn_exch{" C3* GNH"}=" C3' GNH"; $resn_exch{" O3* GNH"}=" O3' GNH"; $resn_exch{" C2* GNH"}=" C2' GNH";
$resn_exch{" O2* GNH"}=" O2' GNH"; $resn_exch{" C1* GNH"}=" C1' GNH"; $resn_exch{" O5* GNP"}=" O5' GNP";
$resn_exch{" C5* GNP"}=" C5' GNP"; $resn_exch{" C4* GNP"}=" C4' GNP"; $resn_exch{" O4* GNP"}=" O4' GNP";
$resn_exch{" C3* GNP"}=" C3' GNP"; $resn_exch{" O3* GNP"}=" O3' GNP"; $resn_exch{" C2* GNP"}=" C2' GNP";
$resn_exch{" O2* GNP"}=" O2' GNP"; $resn_exch{" C1* GNP"}=" C1' GNP"; $resn_exch{" O1P GOM"}=" OP1 GOM";
$resn_exch{" O2P GOM"}=" OP2 GOM"; $resn_exch{" O3P GOM"}=" OP3 GOM"; $resn_exch{" O5* GOM"}=" O5' GOM";
$resn_exch{" C5* GOM"}=" C5' GOM"; $resn_exch{" C4* GOM"}=" C4' GOM"; $resn_exch{" O4* GOM"}=" O4' GOM";
$resn_exch{" C1* GOM"}=" C1' GOM"; $resn_exch{" C2* GOM"}=" C2' GOM"; $resn_exch{" O2* GOM"}=" O2' GOM";
$resn_exch{" C3* GOM"}=" C3' GOM"; $resn_exch{" O3* GOM"}=" O3' GOM"; $resn_exch{" C1* GP2"}=" C1' GP2";
$resn_exch{" C2* GP2"}=" C2' GP2"; $resn_exch{" O2* GP2"}=" O2' GP2"; $resn_exch{" C3* GP2"}=" C3' GP2";
$resn_exch{" O3* GP2"}=" O3' GP2"; $resn_exch{" O4* GP2"}=" O4' GP2"; $resn_exch{" C4* GP2"}=" C4' GP2";
$resn_exch{" C5* GP2"}=" C5' GP2"; $resn_exch{" O5* GP2"}=" O5' GP2"; $resn_exch{" AN9 GP3"}=" N9A GP3";
$resn_exch{" AC8 GP3"}=" C8A GP3"; $resn_exch{" AN7 GP3"}=" N7A GP3"; $resn_exch{" AC5 GP3"}=" C5A GP3";
$resn_exch{" AC6 GP3"}=" C6A GP3"; $resn_exch{" AO6 GP3"}=" O6A GP3"; $resn_exch{" AN1 GP3"}=" N1A GP3";
$resn_exch{" AC2 GP3"}=" C2A GP3"; $resn_exch{" AN2 GP3"}=" N2A GP3"; $resn_exch{" AN3 GP3"}=" N3A GP3";
$resn_exch{" AC4 GP3"}=" C4A GP3"; $resn_exch{"AO5* GP3"}=" O5D GP3"; $resn_exch{"AC5* GP3"}=" C5D GP3";
$resn_exch{"AC4* GP3"}=" C4D GP3"; $resn_exch{"AO4* GP3"}=" O4D GP3"; $resn_exch{"AC3* GP3"}=" C3D GP3";
$resn_exch{"AO3* GP3"}=" O3D GP3"; $resn_exch{"AC2* GP3"}=" C2D GP3"; $resn_exch{"AO2* GP3"}=" O2D GP3";
$resn_exch{"AC1* GP3"}=" C1D GP3"; $resn_exch{"BO5* GP3"}=" O5E GP3"; $resn_exch{"BC5* GP3"}=" C5E GP3";
$resn_exch{"BC4* GP3"}=" C4E GP3"; $resn_exch{"BO4* GP3"}=" O4E GP3"; $resn_exch{"BC3* GP3"}=" C3E GP3";
$resn_exch{"BO3* GP3"}=" O3E GP3"; $resn_exch{"BC2* GP3"}=" C2E GP3"; $resn_exch{"BO2* GP3"}=" O2E GP3";
$resn_exch{"BC1* GP3"}=" C1E GP3"; $resn_exch{" BN9 GP3"}=" N9B GP3"; $resn_exch{" BC8 GP3"}=" C8B GP3";
$resn_exch{" BN7 GP3"}=" N7B GP3"; $resn_exch{" BC5 GP3"}=" C5B GP3"; $resn_exch{" BC6 GP3"}=" C6B GP3";
$resn_exch{" BO6 GP3"}=" O6B GP3"; $resn_exch{" BN1 GP3"}=" N1B GP3"; $resn_exch{" BC2 GP3"}=" C2B GP3";
$resn_exch{" BN2 GP3"}=" N2B GP3"; $resn_exch{" BN3 GP3"}=" N3B GP3"; $resn_exch{" BC4 GP3"}=" C4B GP3";
$resn_exch{" C1* GP5"}=" C1' GP5"; $resn_exch{" O4* GP5"}=" O4' GP5"; $resn_exch{" C2* GP5"}=" C2' GP5";
$resn_exch{" O2* GP5"}=" O2' GP5"; $resn_exch{" C3* GP5"}=" C3' GP5"; $resn_exch{" O3* GP5"}=" O3' GP5";
$resn_exch{" C4* GP5"}=" C4' GP5"; $resn_exch{" C5* GP5"}=" C5' GP5"; $resn_exch{" O5* GP5"}=" O5' GP5";
$resn_exch{" GN1 GPC"}=" N1G GPC"; $resn_exch{" GC2 GPC"}=" C2G GPC"; $resn_exch{" GN2 GPC"}=" N2G GPC";
$resn_exch{" GN3 GPC"}=" N3G GPC"; $resn_exch{" GC4 GPC"}=" C4G GPC"; $resn_exch{" GC5 GPC"}=" C5G GPC";
$resn_exch{" GC6 GPC"}=" C6G GPC"; $resn_exch{" GO6 GPC"}=" O6G GPC"; $resn_exch{" GN7 GPC"}=" N7G GPC";
$resn_exch{" GC8 GPC"}=" C8G GPC"; $resn_exch{" GN9 GPC"}=" N9G GPC"; $resn_exch{"GC1* GPC"}=" C1D GPC";
$resn_exch{"GC2* GPC"}=" C2D GPC"; $resn_exch{"GF2* GPC"}=" F2D GPC"; $resn_exch{"GC3* GPC"}=" C3D GPC";
$resn_exch{"GO3* GPC"}=" O3D GPC"; $resn_exch{"GC4* GPC"}=" C4D GPC"; $resn_exch{"GO4* GPC"}=" O4D GPC";
$resn_exch{"GC5* GPC"}=" C5D GPC"; $resn_exch{"GO5* GPC"}=" O5D GPC"; $resn_exch{" CP  GPC"}=" PC  GPC";
$resn_exch{" CO1 GPC"}=" O1C GPC"; $resn_exch{" CO2 GPC"}=" O2C GPC"; $resn_exch{"CO5* GPC"}=" O5B GPC";
$resn_exch{"CC5* GPC"}=" C5X GPC"; $resn_exch{"CC4* GPC"}=" C4X GPC"; $resn_exch{"CO4* GPC"}=" O4B GPC";
$resn_exch{"CC3* GPC"}=" C3X GPC"; $resn_exch{"CO3* GPC"}=" O3B GPC"; $resn_exch{"CC2* GPC"}=" C2X GPC";
$resn_exch{"CO2* GPC"}=" O2B GPC"; $resn_exch{"CC1* GPC"}=" C1X GPC"; $resn_exch{" CN1 GPC"}=" N1C GPC";
$resn_exch{"CO2' GPC"}=" O2X GPC"; $resn_exch{" CN3 GPC"}=" N3C GPC"; $resn_exch{" CN4 GPC"}=" N4C GPC";
$resn_exch{" AN9 GPG"}=" N9A GPG"; $resn_exch{" AC8 GPG"}=" C8A GPG"; $resn_exch{" AN7 GPG"}=" N7A GPG";
$resn_exch{" AC5 GPG"}=" C5A GPG"; $resn_exch{" AC6 GPG"}=" C6A GPG"; $resn_exch{" AO6 GPG"}=" O6A GPG";
$resn_exch{" AN1 GPG"}=" N1A GPG"; $resn_exch{" AC2 GPG"}=" C2A GPG"; $resn_exch{" AN2 GPG"}=" N2A GPG";
$resn_exch{" AN3 GPG"}=" N3A GPG"; $resn_exch{" AC4 GPG"}=" C4A GPG"; $resn_exch{"AO5* GPG"}=" O5D GPG";
$resn_exch{"AC5* GPG"}=" C5D GPG"; $resn_exch{"AC4* GPG"}=" C4D GPG"; $resn_exch{"AO4* GPG"}=" O4D GPG";
$resn_exch{"AC3* GPG"}=" C3D GPG"; $resn_exch{"AO3* GPG"}=" O3D GPG"; $resn_exch{"AC2* GPG"}=" C2D GPG";
$resn_exch{"AO2* GPG"}=" O2D GPG"; $resn_exch{"AC1* GPG"}=" C1D GPG"; $resn_exch{"BO5* GPG"}=" O5E GPG";
$resn_exch{"BC5* GPG"}=" C5E GPG"; $resn_exch{"BC4* GPG"}=" C4E GPG"; $resn_exch{"BO4* GPG"}=" O4E GPG";
$resn_exch{"BC3* GPG"}=" C3E GPG"; $resn_exch{"BO3* GPG"}=" O3E GPG"; $resn_exch{"BC2* GPG"}=" C2E GPG";
$resn_exch{"BO2* GPG"}=" O2E GPG"; $resn_exch{"BC1* GPG"}=" C1E GPG"; $resn_exch{" BN9 GPG"}=" N9B GPG";
$resn_exch{" BC8 GPG"}=" C8B GPG"; $resn_exch{" BN7 GPG"}=" N7B GPG"; $resn_exch{" BC5 GPG"}=" C5B GPG";
$resn_exch{" BC6 GPG"}=" C6B GPG"; $resn_exch{" BO6 GPG"}=" O6B GPG"; $resn_exch{" BN1 GPG"}=" N1B GPG";
$resn_exch{" BC2 GPG"}=" C2B GPG"; $resn_exch{" BN2 GPG"}=" N2B GPG"; $resn_exch{" BN3 GPG"}=" N3B GPG";
$resn_exch{" BC4 GPG"}=" C4B GPG"; $resn_exch{" O5* GPL"}=" O5' GPL"; $resn_exch{" C5* GPL"}=" C5' GPL";
$resn_exch{" C4* GPL"}=" C4' GPL"; $resn_exch{" O4* GPL"}=" O4' GPL"; $resn_exch{" C3* GPL"}=" C3' GPL";
$resn_exch{" O3* GPL"}=" O3' GPL"; $resn_exch{" C2* GPL"}=" C2' GPL"; $resn_exch{" O2* GPL"}=" O2' GPL";
$resn_exch{" C1* GPL"}=" C1' GPL"; $resn_exch{" O1P  GS"}=" OP1  GS"; $resn_exch{" O3P  GS"}=" OP3  GS";
$resn_exch{" O5*  GS"}=" O5'  GS"; $resn_exch{" C5*  GS"}=" C5'  GS"; $resn_exch{" C4*  GS"}=" C4'  GS";
$resn_exch{" O4*  GS"}=" O4'  GS"; $resn_exch{" C3*  GS"}=" C3'  GS"; $resn_exch{" O3*  GS"}=" O3'  GS";
$resn_exch{" C2*  GS"}=" C2'  GS"; $resn_exch{" C1*  GS"}=" C1'  GS"; $resn_exch{" O5* GSP"}=" O5' GSP";
$resn_exch{" C5* GSP"}=" C5' GSP"; $resn_exch{" C4* GSP"}=" C4' GSP"; $resn_exch{" O4* GSP"}=" O4' GSP";
$resn_exch{" C3* GSP"}=" C3' GSP"; $resn_exch{" O3* GSP"}=" O3' GSP"; $resn_exch{" C2* GSP"}=" C2' GSP";
$resn_exch{" O2* GSP"}=" O2' GSP"; $resn_exch{" C1* GSP"}=" C1' GSP"; $resn_exch{" O1P GSR"}=" OP1 GSR";
$resn_exch{" O2P GSR"}=" OP2 GSR"; $resn_exch{" O3P GSR"}=" OP3 GSR"; $resn_exch{" O5* GSR"}=" O5' GSR";
$resn_exch{" C5* GSR"}=" C5' GSR"; $resn_exch{" C4* GSR"}=" C4' GSR"; $resn_exch{" O4* GSR"}=" O4' GSR";
$resn_exch{" C3* GSR"}=" C3' GSR"; $resn_exch{" O3* GSR"}=" O3' GSR"; $resn_exch{" C2* GSR"}=" C2' GSR";
$resn_exch{" C1* GSR"}=" C1' GSR"; $resn_exch{" O1P GSS"}=" OP1 GSS"; $resn_exch{" O2P GSS"}=" OP2 GSS";
$resn_exch{" O3P GSS"}=" OP3 GSS"; $resn_exch{" O5* GSS"}=" O5' GSS"; $resn_exch{" C5* GSS"}=" C5' GSS";
$resn_exch{" C4* GSS"}=" C4' GSS"; $resn_exch{" O4* GSS"}=" O4' GSS"; $resn_exch{" C3* GSS"}=" C3' GSS";
$resn_exch{" O3* GSS"}=" O3' GSS"; $resn_exch{" C2* GSS"}=" C2' GSS"; $resn_exch{" C1* GSS"}=" C1' GSS";
$resn_exch{" O5* GSU"}=" O5' GSU"; $resn_exch{" C5* GSU"}=" C5' GSU"; $resn_exch{" C4* GSU"}=" C4' GSU";
$resn_exch{" O4* GSU"}=" O4' GSU"; $resn_exch{" C1* GSU"}=" C1' GSU"; $resn_exch{" C2* GSU"}=" C2' GSU";
$resn_exch{" O2* GSU"}=" O2' GSU"; $resn_exch{" C3* GSU"}=" C3' GSU"; $resn_exch{" O3* GSU"}=" O3' GSU";
$resn_exch{"C7MA GTG"}=" C7X GTG"; $resn_exch{"O5*A GTG"}=" O5D GTG"; $resn_exch{"C5*A GTG"}=" C5D GTG";
$resn_exch{"C4*A GTG"}=" C4D GTG"; $resn_exch{"O4*A GTG"}=" O4D GTG"; $resn_exch{"C3*A GTG"}=" C3D GTG";
$resn_exch{"O3*A GTG"}=" O3D GTG"; $resn_exch{"C2*A GTG"}=" C2D GTG"; $resn_exch{"O2*A GTG"}=" O2D GTG";
$resn_exch{"C1*A GTG"}=" C1D GTG"; $resn_exch{"O5*B GTG"}=" O5E GTG"; $resn_exch{"C5*B GTG"}=" C5E GTG";
$resn_exch{"C4*B GTG"}=" C4E GTG"; $resn_exch{"O4*B GTG"}=" O4E GTG"; $resn_exch{"C3*B GTG"}=" C3E GTG";
$resn_exch{"O3*B GTG"}=" O3E GTG"; $resn_exch{"C2*B GTG"}=" C2E GTG"; $resn_exch{"O2*B GTG"}=" O2E GTG";
$resn_exch{"C1*B GTG"}=" C1E GTG"; $resn_exch{" O5* GTN"}=" O5' GTN"; $resn_exch{" C5* GTN"}=" C5' GTN";
$resn_exch{" C4* GTN"}=" C4' GTN"; $resn_exch{" O4* GTN"}=" O4' GTN"; $resn_exch{" C3* GTN"}=" C3' GTN";
$resn_exch{" O3* GTN"}=" O3' GTN"; $resn_exch{" C2* GTN"}=" C2' GTN"; $resn_exch{" O2* GTN"}=" O2' GTN";
$resn_exch{" C1* GTN"}=" C1' GTN"; $resn_exch{" O5* GTO"}=" O5' GTO"; $resn_exch{" C5* GTO"}=" C5' GTO";
$resn_exch{" C4* GTO"}=" C4' GTO"; $resn_exch{" O4* GTO"}=" O4' GTO"; $resn_exch{" C3* GTO"}=" C3' GTO";
$resn_exch{" O3* GTO"}=" O3' GTO"; $resn_exch{" C2* GTO"}=" C2' GTO"; $resn_exch{" O2* GTO"}=" O2' GTO";
$resn_exch{" C1* GTO"}=" C1' GTO"; $resn_exch{" O5* GTP"}=" O5' GTP"; $resn_exch{" C5* GTP"}=" C5' GTP";
$resn_exch{" C4* GTP"}=" C4' GTP"; $resn_exch{" O4* GTP"}=" O4' GTP"; $resn_exch{" C3* GTP"}=" C3' GTP";
$resn_exch{" O3* GTP"}=" O3' GTP"; $resn_exch{" C2* GTP"}=" C2' GTP"; $resn_exch{" O2* GTP"}=" O2' GTP";
$resn_exch{" C1* GTP"}=" C1' GTP"; $resn_exch{" C1* GUD"}=" C1D GUD"; $resn_exch{" C2* GUD"}=" C2D GUD";
$resn_exch{" O2* GUD"}=" O2D GUD"; $resn_exch{" C3* GUD"}=" C3D GUD"; $resn_exch{" C4* GUD"}=" C4D GUD";
$resn_exch{" O4* GUD"}=" O4D GUD"; $resn_exch{" O3* GUD"}=" O3D GUD"; $resn_exch{" C5* GUD"}=" C5D GUD";
$resn_exch{" O5* GUD"}=" O5D GUD"; $resn_exch{" N_A H02"}=" NA  H02"; $resn_exch{" N_B H02"}=" NB  H02";
$resn_exch{" N_C H02"}=" NC  H02"; $resn_exch{" N_D H02"}=" ND  H02"; $resn_exch{" O1P H2U"}=" OP1 H2U";
$resn_exch{" O2P H2U"}=" OP2 H2U"; $resn_exch{" O3P H2U"}=" OP3 H2U"; $resn_exch{" O5* H2U"}=" O5' H2U";
$resn_exch{" C5* H2U"}=" C5' H2U"; $resn_exch{" C4* H2U"}=" C4' H2U"; $resn_exch{" O4* H2U"}=" O4' H2U";
$resn_exch{" C3* H2U"}=" C3' H2U"; $resn_exch{" O3* H2U"}=" O3' H2U"; $resn_exch{" C1* H2U"}=" C1' H2U";
$resn_exch{" C2* H2U"}=" C2' H2U"; $resn_exch{" O2* H2U"}=" O2' H2U"; $resn_exch{" O5* HAM"}=" O5' HAM";
$resn_exch{" C5* HAM"}=" C5' HAM"; $resn_exch{" C4* HAM"}=" C4' HAM"; $resn_exch{" O4* HAM"}=" O4' HAM";
$resn_exch{" C3* HAM"}=" C3' HAM"; $resn_exch{" O3* HAM"}=" O3' HAM"; $resn_exch{" C2* HAM"}=" C2' HAM";
$resn_exch{" O2* HAM"}=" O2' HAM"; $resn_exch{" C1* HAM"}=" C1' HAM"; $resn_exch{" N_A HAS"}=" NA  HAS";
$resn_exch{" N_B HAS"}=" NB  HAS"; $resn_exch{" N_C HAS"}=" NC  HAS"; $resn_exch{" N_D HAS"}=" ND  HAS";
$resn_exch{" AN1 HAX"}=" N1A HAX"; $resn_exch{" AC2 HAX"}=" C2A HAX"; $resn_exch{" AN3 HAX"}=" N3A HAX";
$resn_exch{" AC4 HAX"}=" C4A HAX"; $resn_exch{" AC5 HAX"}=" C5A HAX"; $resn_exch{" AC6 HAX"}=" C6A HAX";
$resn_exch{" AN6 HAX"}=" N6A HAX"; $resn_exch{" AN7 HAX"}=" N7A HAX"; $resn_exch{" AC8 HAX"}=" C8A HAX";
$resn_exch{" AN9 HAX"}=" N9A HAX"; $resn_exch{"AC1* HAX"}=" C1B HAX"; $resn_exch{"AC2* HAX"}=" C2B HAX";
$resn_exch{"AO2* HAX"}=" O2B HAX"; $resn_exch{"AC3* HAX"}=" C3B HAX"; $resn_exch{"AO3* HAX"}=" O3B HAX";
$resn_exch{"AP3* HAX"}=" P3B HAX"; $resn_exch{" AO7 HAX"}=" O7A HAX"; $resn_exch{" AO8 HAX"}=" O8A HAX";
$resn_exch{" AO9 HAX"}=" O9A HAX"; $resn_exch{"AC4* HAX"}=" C4B HAX"; $resn_exch{"AO4* HAX"}=" O4B HAX";
$resn_exch{"AC5* HAX"}=" C5B HAX"; $resn_exch{"AO5* HAX"}=" O5B HAX"; $resn_exch{" AP1 HAX"}=" P1A HAX";
$resn_exch{" AO1 HAX"}=" O1A HAX"; $resn_exch{" AO2 HAX"}=" O2A HAX"; $resn_exch{" AO3 HAX"}=" O3A HAX";
$resn_exch{" AP2 HAX"}=" P2A HAX"; $resn_exch{" AO4 HAX"}=" O4A HAX"; $resn_exch{" AO5 HAX"}=" O5A HAX";
$resn_exch{" AO6 HAX"}=" O6A HAX"; $resn_exch{"PC11 HAX"}=" CBP HAX"; $resn_exch{"PC12 HAX"}=" CCP HAX";
$resn_exch{"PC13 HAX"}=" CDP HAX"; $resn_exch{"PC14 HAX"}=" CEP HAX"; $resn_exch{"PC10 HAX"}=" CAP HAX";
$resn_exch{"PO10 HAX"}=" OAP HAX"; $resn_exch{" PC9 HAX"}=" C9P HAX"; $resn_exch{" PO9 HAX"}=" O9P HAX";
$resn_exch{" PN8 HAX"}=" N8P HAX"; $resn_exch{" PC7 HAX"}=" C7P HAX"; $resn_exch{" PC6 HAX"}=" C6P HAX";
$resn_exch{" PC5 HAX"}=" C5P HAX"; $resn_exch{" PO5 HAX"}=" O5P HAX"; $resn_exch{" PN4 HAX"}=" N4P HAX";
$resn_exch{" PC3 HAX"}=" C3P HAX"; $resn_exch{" PC2 HAX"}=" C2P HAX"; $resn_exch{" N_A HCO"}=" NA  HCO";
$resn_exch{" N_B HCO"}=" NB  HCO"; $resn_exch{" N_C HCO"}=" NC  HCO"; $resn_exch{" N_D HCO"}=" ND  HCO";
$resn_exch{" OT1 HCS"}=" OXT HCS"; $resn_exch{" OXT HCS"}=" O   HCS"; $resn_exch{" O5* HCX"}=" O5' HCX";
$resn_exch{" C2* HCX"}=" C2' HCX"; $resn_exch{" C5* HCX"}=" C5' HCX"; $resn_exch{" C4* HCX"}=" C4' HCX";
$resn_exch{" C1* HCX"}=" C1' HCX"; $resn_exch{" C3* HCX"}=" C3' HCX"; $resn_exch{" O3* HCX"}=" O3' HCX";
$resn_exch{"PO10 HDC"}=" OAP HDC"; $resn_exch{"PC10 HDC"}=" CAP HDC"; $resn_exch{" PC9 HDC"}=" C9P HDC";
$resn_exch{" PO9 HDC"}=" O9P HDC"; $resn_exch{" PN8 HDC"}=" N8P HDC"; $resn_exch{" PC7 HDC"}=" C7P HDC";
$resn_exch{" PC6 HDC"}=" C6P HDC"; $resn_exch{" PC5 HDC"}=" C5P HDC"; $resn_exch{" PO5 HDC"}=" O5P HDC";
$resn_exch{" PN4 HDC"}=" N4P HDC"; $resn_exch{" PC3 HDC"}=" C3P HDC"; $resn_exch{" PC2 HDC"}=" C2P HDC";
$resn_exch{" PS1 HDC"}=" S1P HDC"; $resn_exch{" C1* HDC"}=" C1' HDC"; $resn_exch{" O1* HDC"}=" O1' HDC";
$resn_exch{" C2* HDC"}=" C2' HDC"; $resn_exch{" C3* HDC"}=" C3' HDC"; $resn_exch{" O2* HDC"}=" O2' HDC";
$resn_exch{" C4* HDC"}=" C4' HDC"; $resn_exch{" C5* HDC"}=" C5' HDC"; $resn_exch{" C6* HDC"}=" C6' HDC";
$resn_exch{" C7* HDC"}=" C7' HDC"; $resn_exch{" C8* HDC"}=" C8' HDC"; $resn_exch{" C9* HDC"}=" C9' HDC";
$resn_exch{"PC11 HDC"}=" CBP HDC"; $resn_exch{"PC13 HDC"}=" CDP HDC"; $resn_exch{"PC14 HDC"}=" CEP HDC";
$resn_exch{"PC12 HDC"}=" CCP HDC"; $resn_exch{" AO6 HDC"}=" O6A HDC"; $resn_exch{" AP2 HDC"}=" P2A HDC";
$resn_exch{" AO4 HDC"}=" O4A HDC"; $resn_exch{" AO5 HDC"}=" O5A HDC"; $resn_exch{" AO3 HDC"}=" O3A HDC";
$resn_exch{" AP1 HDC"}=" P1A HDC"; $resn_exch{" AO1 HDC"}=" O1A HDC"; $resn_exch{" AO2 HDC"}=" O2A HDC";
$resn_exch{"AO5* HDC"}=" O5B HDC"; $resn_exch{"AC5* HDC"}=" C5B HDC"; $resn_exch{"AC4* HDC"}=" C4B HDC";
$resn_exch{"AO4* HDC"}=" O4B HDC"; $resn_exch{"AC1* HDC"}=" C1B HDC"; $resn_exch{" AN9 HDC"}=" N9A HDC";
$resn_exch{" AC8 HDC"}=" C8A HDC"; $resn_exch{" AN7 HDC"}=" N7A HDC"; $resn_exch{" AC5 HDC"}=" C5A HDC";
$resn_exch{" AC4 HDC"}=" C4A HDC"; $resn_exch{" AN3 HDC"}=" N3A HDC"; $resn_exch{" AC2 HDC"}=" C2A HDC";
$resn_exch{" AN1 HDC"}=" N1A HDC"; $resn_exch{" AC6 HDC"}=" C6A HDC"; $resn_exch{" AN6 HDC"}=" N6A HDC";
$resn_exch{"AC2* HDC"}=" C2B HDC"; $resn_exch{"AO2* HDC"}=" O2B HDC"; $resn_exch{"AC3* HDC"}=" C3B HDC";
$resn_exch{"AO3* HDC"}=" O3B HDC"; $resn_exch{"AP3* HDC"}=" P3B HDC"; $resn_exch{" AO9 HDC"}=" O9A HDC";
$resn_exch{" AO8 HDC"}=" O8A HDC"; $resn_exch{" AO7 HDC"}=" O7A HDC"; $resn_exch{" N_A HDD"}=" NA  HDD";
$resn_exch{" N_B HDD"}=" NB  HDD"; $resn_exch{" N_C HDD"}=" NC  HDD"; $resn_exch{" N_D HDD"}=" ND  HDD";
$resn_exch{" C1* HDF"}=" C1' HDF"; $resn_exch{" C2* HDF"}=" C2' HDF"; $resn_exch{" O2* HDF"}=" O2' HDF";
$resn_exch{" C3* HDF"}=" C3' HDF"; $resn_exch{" O3* HDF"}=" O3' HDF"; $resn_exch{" C4* HDF"}=" C4' HDF";
$resn_exch{" O4* HDF"}=" O4' HDF"; $resn_exch{" C5* HDF"}=" C5' HDF"; $resn_exch{" O5* HDF"}=" O5' HDF";
$resn_exch{" N_A HDM"}=" NA  HDM"; $resn_exch{" N_B HDM"}=" NB  HDM"; $resn_exch{" N_C HDM"}=" NC  HDM";
$resn_exch{" N_D HDM"}=" ND  HDM"; $resn_exch{" O1P HDP"}=" OP1 HDP"; $resn_exch{" O2P HDP"}=" OP2 HDP";
$resn_exch{" O5* HDP"}=" O5' HDP"; $resn_exch{" C5* HDP"}=" C5B HDP"; $resn_exch{" O3* HDP"}=" O3' HDP";
$resn_exch{" O3P HDP"}=" OP3 HDP"; $resn_exch{" N_A HE6"}=" NA  HE6"; $resn_exch{" N_B HE6"}=" NB  HE6";
$resn_exch{" N_C HE6"}=" NC  HE6"; $resn_exch{" N_D HE6"}=" ND  HE6"; $resn_exch{" N_A HEA"}=" NA  HEA";
$resn_exch{" N_B HEA"}=" NB  HEA"; $resn_exch{" N_C HEA"}=" NC  HEA"; $resn_exch{" N_D HEA"}=" ND  HEA";
$resn_exch{" N_A HEB"}=" NA  HEB"; $resn_exch{" N_B HEB"}=" NB  HEB"; $resn_exch{" N_C HEB"}=" NC  HEB";
$resn_exch{" N_D HEB"}=" ND  HEB"; $resn_exch{" N_A HEC"}=" NA  HEC"; $resn_exch{" N_B HEC"}=" NB  HEC";
$resn_exch{" N_C HEC"}=" NC  HEC"; $resn_exch{" N_D HEC"}=" ND  HEC"; $resn_exch{" N_A HEG"}=" NA  HEG";
$resn_exch{" N_B HEG"}=" NB  HEG"; $resn_exch{" N_C HEG"}=" NC  HEG"; $resn_exch{" N_D HEG"}=" ND  HEG";
$resn_exch{"C12B HEH"}=" CCB HEH"; $resn_exch{"O11D HEH"}=" O11 HEH"; $resn_exch{"C11A HEH"}=" CBA HEH";
$resn_exch{"C12A HEH"}=" CCA HEH"; $resn_exch{"O12C HEH"}=" OCC HEH"; $resn_exch{"1CEM HEH"}=" C1X HEH";
$resn_exch{"2CEG HEH"}=" C2X HEH"; $resn_exch{"3CEG HEH"}=" C3X HEH"; $resn_exch{"CFM7 HEH"}=" C1A HEH";
$resn_exch{"5CFK HEH"}=" C5X HEH"; $resn_exch{"6CKF HEH"}=" C1B HEH"; $resn_exch{"CFM4 HEH"}=" C1C HEH";
$resn_exch{" N_A HEM"}=" NA  HEM"; $resn_exch{" N_B HEM"}=" NB  HEM"; $resn_exch{" N_C HEM"}=" NC  HEM";
$resn_exch{" N_D HEM"}=" ND  HEM"; $resn_exch{" N_A HEO"}=" NA  HEO"; $resn_exch{" N_B HEO"}=" NB  HEO";
$resn_exch{" N_C HEO"}=" NC  HEO"; $resn_exch{" N_D HEO"}=" ND  HEO"; $resn_exch{" N_A HES"}=" NA  HES";
$resn_exch{" N_B HES"}=" NB  HES"; $resn_exch{" N_C HES"}=" NC  HES"; $resn_exch{" N_D HES"}=" ND  HES";
$resn_exch{" O2P HEU"}=" OP2 HEU"; $resn_exch{" O1P HEU"}=" OP1 HEU"; $resn_exch{" O5* HEU"}=" O5' HEU";
$resn_exch{" C5* HEU"}=" C5' HEU"; $resn_exch{" C4* HEU"}=" C4' HEU"; $resn_exch{" O4* HEU"}=" O4' HEU";
$resn_exch{" C1* HEU"}=" C1' HEU"; $resn_exch{" C3* HEU"}=" C3' HEU"; $resn_exch{" C2* HEU"}=" C2' HEU";
$resn_exch{" O3* HEU"}=" O3' HEU"; $resn_exch{" O3P HEU"}=" OP3 HEU"; $resn_exch{" N_A HEV"}=" NA  HEV";
$resn_exch{" N_B HEV"}=" NB  HEV"; $resn_exch{" N_C HEV"}=" NC  HEV"; $resn_exch{" N_D HEV"}=" ND  HEV";
$resn_exch{" N_A HFM"}=" NA  HFM"; $resn_exch{" N_B HFM"}=" NB  HFM"; $resn_exch{" N_C HFM"}=" NC  HFM";
$resn_exch{" N_D HFM"}=" ND  HFM"; $resn_exch{" N_A HIF"}=" NA  HIF"; $resn_exch{" N_B HIF"}=" NB  HIF";
$resn_exch{" N_C HIF"}=" NC  HIF"; $resn_exch{" N_D HIF"}=" ND  HIF"; $resn_exch{" N_C HME"}=" NC  HME";
$resn_exch{" N_B HME"}=" NB  HME"; $resn_exch{" N_A HME"}=" NA  HME"; $resn_exch{" N_D HME"}=" ND  HME";
$resn_exch{"O5*A HMG"}=" O5B HMG"; $resn_exch{"C5*A HMG"}=" C5B HMG"; $resn_exch{"C4*A HMG"}=" C4B HMG";
$resn_exch{"O4*A HMG"}=" O4B HMG"; $resn_exch{"C3*A HMG"}=" C3B HMG"; $resn_exch{"O3*A HMG"}=" O3B HMG";
$resn_exch{"C2*A HMG"}=" C2B HMG"; $resn_exch{"O2*A HMG"}=" O2B HMG"; $resn_exch{"C1*A HMG"}=" C1B HMG";
$resn_exch{"P3*A HMG"}=" P3B HMG"; $resn_exch{"C11P HMG"}=" CBP HMG"; $resn_exch{"C12P HMG"}=" CCP HMG";
$resn_exch{"C13P HMG"}=" CDP HMG"; $resn_exch{"C14P HMG"}=" CEP HMG"; $resn_exch{"C10P HMG"}=" CAP HMG";
$resn_exch{"O10P HMG"}=" OAP HMG"; $resn_exch{" N_A HNI"}=" NA  HNI"; $resn_exch{" N_B HNI"}=" NB  HNI";
$resn_exch{" N_C HNI"}=" NC  HNI"; $resn_exch{" N_D HNI"}=" ND  HNI"; $resn_exch{" O1P HOB"}=" OP1 HOB";
$resn_exch{" O2P HOB"}=" OP2 HOB"; $resn_exch{" O5* HOB"}=" O5' HOB"; $resn_exch{" C5* HOB"}=" C5' HOB";
$resn_exch{" C4* HOB"}=" C4' HOB"; $resn_exch{" C3* HOB"}=" C3' HOB"; $resn_exch{" C2* HOB"}=" C2' HOB";
$resn_exch{" C1* HOB"}=" C1' HOB"; $resn_exch{" O4* HOB"}=" O4' HOB"; $resn_exch{" O3* HOB"}=" O3' HOB";
$resn_exch{" O3P HOB"}=" OP3 HOB"; $resn_exch{" O1P HOL"}=" OP1 HOL"; $resn_exch{" O2P HOL"}=" OP2 HOL";
$resn_exch{" O5* HOL"}=" O5' HOL"; $resn_exch{" C5* HOL"}=" C5' HOL"; $resn_exch{" C4* HOL"}=" C4' HOL";
$resn_exch{" C3* HOL"}=" C3' HOL"; $resn_exch{" C2* HOL"}=" C2' HOL"; $resn_exch{" C1* HOL"}=" C1' HOL";
$resn_exch{" O4* HOL"}=" O4' HOL"; $resn_exch{" O3* HOL"}=" O3' HOL"; $resn_exch{" O3P HOL"}=" OP3 HOL";
$resn_exch{" O5* HPD"}=" O5' HPD"; $resn_exch{" C5* HPD"}=" C5' HPD"; $resn_exch{" O4* HPD"}=" O4' HPD";
$resn_exch{" O3* HPD"}=" O3' HPD"; $resn_exch{" C1* HPD"}=" C1' HPD"; $resn_exch{" O1* HPD"}=" O1' HPD";
$resn_exch{" C3* HPD"}=" C3' HPD"; $resn_exch{" C4* HPD"}=" C4' HPD"; $resn_exch{" C2* HPD"}=" C2' HPD";
$resn_exch{" C1* HPR"}=" C1' HPR"; $resn_exch{" C2* HPR"}=" C2' HPR"; $resn_exch{" O2* HPR"}=" O2' HPR";
$resn_exch{" C3* HPR"}=" C3' HPR"; $resn_exch{" O3* HPR"}=" O3' HPR"; $resn_exch{" C4* HPR"}=" C4' HPR";
$resn_exch{" O4* HPR"}=" O4' HPR"; $resn_exch{" C5* HPR"}=" C5' HPR"; $resn_exch{" O5* HPR"}=" O5' HPR";
$resn_exch{" O5* HSS"}=" O5' HSS"; $resn_exch{" C5* HSS"}=" C5' HSS"; $resn_exch{" C4* HSS"}=" C4' HSS";
$resn_exch{" O4* HSS"}=" O4' HSS"; $resn_exch{" C3* HSS"}=" C3' HSS"; $resn_exch{" O3* HSS"}=" O3' HSS";
$resn_exch{" C2* HSS"}=" C2' HSS"; $resn_exch{" O2* HSS"}=" O2' HSS"; $resn_exch{" C1* HSS"}=" C1' HSS";
$resn_exch{" N1* HTL"}=" N1' HTL"; $resn_exch{" C2* HTL"}=" C2' HTL"; $resn_exch{" N3* HTL"}=" N3' HTL";
$resn_exch{" C4* HTL"}=" C4' HTL"; $resn_exch{" N4* HTL"}=" N4' HTL"; $resn_exch{" C5* HTL"}=" C5' HTL";
$resn_exch{" C6* HTL"}=" C6' HTL"; $resn_exch{" C1* HY1"}=" C1' HY1"; $resn_exch{" C2* HY1"}=" C2' HY1";
$resn_exch{" C3* HY1"}=" C3' HY1"; $resn_exch{" C4* HY1"}=" C4' HY1"; $resn_exch{" C5* HY1"}=" C5' HY1";
$resn_exch{" C6* HY1"}=" C6' HY1"; $resn_exch{" OD  HYP"}=" OD1 HYP"; $resn_exch{" O3P   I"}=" OP3   I";
$resn_exch{" O1P   I"}=" OP1   I"; $resn_exch{" O2P   I"}=" OP2   I"; $resn_exch{" O5*   I"}=" O5'   I";
$resn_exch{" C5*   I"}=" C5'   I"; $resn_exch{" C4*   I"}=" C4'   I"; $resn_exch{" O4*   I"}=" O4'   I";
$resn_exch{" C3*   I"}=" C3'   I"; $resn_exch{" O3*   I"}=" O3'   I"; $resn_exch{" C2*   I"}=" C2'   I";
$resn_exch{" O2*   I"}=" O2'   I"; $resn_exch{" C1*   I"}=" C1'   I"; $resn_exch{" C3* I3A"}=" C3' I3A";
$resn_exch{" O1P I5C"}=" OP1 I5C"; $resn_exch{" O2P I5C"}=" OP2 I5C"; $resn_exch{" O5* I5C"}=" O5' I5C";
$resn_exch{" C2* I5C"}=" C2' I5C"; $resn_exch{" C5* I5C"}=" C5' I5C"; $resn_exch{" C4* I5C"}=" C4' I5C";
$resn_exch{" O4* I5C"}=" O4' I5C"; $resn_exch{" C1* I5C"}=" C1' I5C"; $resn_exch{" C3* I5C"}=" C3' I5C";
$resn_exch{" O3* I5C"}=" O3' I5C"; $resn_exch{" O3P I5C"}=" OP3 I5C"; $resn_exch{" O5* IB2"}=" O5D IB2";
$resn_exch{" C5* IB2"}=" C5D IB2"; $resn_exch{" C4* IB2"}=" C4D IB2"; $resn_exch{" O4* IB2"}=" O4D IB2";
$resn_exch{" C3* IB2"}=" C3D IB2"; $resn_exch{" O3* IB2"}=" O3D IB2"; $resn_exch{" C2* IB2"}=" C2D IB2";
$resn_exch{" O2* IB2"}=" O2D IB2"; $resn_exch{" C1* IB2"}=" C1D IB2"; $resn_exch{" AN9 IB2"}=" N9A IB2";
$resn_exch{" AC8 IB2"}=" C8A IB2"; $resn_exch{" AN7 IB2"}=" N7A IB2"; $resn_exch{" AC5 IB2"}=" C5A IB2";
$resn_exch{" AC6 IB2"}=" C6A IB2"; $resn_exch{" AN6 IB2"}=" N6A IB2"; $resn_exch{" AN1 IB2"}=" N1A IB2";
$resn_exch{" AC2 IB2"}=" C2A IB2"; $resn_exch{" AN3 IB2"}=" N3A IB2"; $resn_exch{" AC4 IB2"}=" C4A IB2";
$resn_exch{" O1P  IC"}=" OP1  IC"; $resn_exch{" O2P  IC"}=" OP2  IC"; $resn_exch{" O3P  IC"}=" OP3  IC";
$resn_exch{" O5*  IC"}=" O5'  IC"; $resn_exch{" O3*  IC"}=" O3'  IC"; $resn_exch{" C1*  IC"}=" C1'  IC";
$resn_exch{" C2*  IC"}=" C2'  IC"; $resn_exch{" C3*  IC"}=" C3'  IC"; $resn_exch{" C4*  IC"}=" C4'  IC";
$resn_exch{" C5*  IC"}=" C5'  IC"; $resn_exch{" O4*  IC"}=" O4'  IC"; $resn_exch{" O2*  IC"}=" O2'  IC";
$resn_exch{" O5* ID2"}=" O5' ID2"; $resn_exch{" C5* ID2"}=" C5' ID2"; $resn_exch{" C4* ID2"}=" C4' ID2";
$resn_exch{" O4* ID2"}=" O4' ID2"; $resn_exch{" C1* ID2"}=" C1' ID2"; $resn_exch{" C2* ID2"}=" C2' ID2";
$resn_exch{" C3* ID2"}=" C3' ID2"; $resn_exch{" O3* ID2"}=" O3' ID2"; $resn_exch{" O5* IDP"}=" O5' IDP";
$resn_exch{" C5* IDP"}=" C5' IDP"; $resn_exch{" C4* IDP"}=" C4' IDP"; $resn_exch{" O4* IDP"}=" O4' IDP";
$resn_exch{" C3* IDP"}=" C3' IDP"; $resn_exch{" O3* IDP"}=" O3' IDP"; $resn_exch{" C2* IDP"}=" C2' IDP";
$resn_exch{" O2* IDP"}=" O2' IDP"; $resn_exch{" C1* IDP"}=" C1' IDP"; $resn_exch{" O1P  IG"}=" OP1  IG";
$resn_exch{" O2P  IG"}=" OP2  IG"; $resn_exch{" O3P  IG"}=" OP3  IG"; $resn_exch{" O5*  IG"}=" O5'  IG";
$resn_exch{" O3*  IG"}=" O3'  IG"; $resn_exch{" C1*  IG"}=" C1'  IG"; $resn_exch{" C2*  IG"}=" C2'  IG";
$resn_exch{" C3*  IG"}=" C3'  IG"; $resn_exch{" C4*  IG"}=" C4'  IG"; $resn_exch{" C5*  IG"}=" C5'  IG";
$resn_exch{" O4*  IG"}=" O4'  IG"; $resn_exch{" O2*  IG"}=" O2'  IG"; $resn_exch{" O1P IGU"}=" OP1 IGU";
$resn_exch{" O2P IGU"}=" OP2 IGU"; $resn_exch{" O3P IGU"}=" OP3 IGU"; $resn_exch{" O5* IGU"}=" O5' IGU";
$resn_exch{" C5* IGU"}=" C5' IGU"; $resn_exch{" C4* IGU"}=" C4' IGU"; $resn_exch{" O4* IGU"}=" O4' IGU";
$resn_exch{" C3* IGU"}=" C3' IGU"; $resn_exch{" O3* IGU"}=" O3' IGU"; $resn_exch{" C2* IGU"}=" C2' IGU";
$resn_exch{" C1* IGU"}=" C1' IGU"; $resn_exch{"2O1  IHS"}=" O12 IHS"; $resn_exch{"2O3  IHS"}=" O32 IHS";
$resn_exch{"2O2  IHS"}=" O22 IHS"; $resn_exch{"2O4  IHS"}=" O42 IHS"; $resn_exch{"3O1  IHS"}=" O13 IHS";
$resn_exch{"3O3  IHS"}=" O33 IHS"; $resn_exch{"3O2  IHS"}=" O23 IHS"; $resn_exch{"3O4  IHS"}=" O43 IHS";
$resn_exch{"4O1  IHS"}=" O14 IHS"; $resn_exch{"4O3  IHS"}=" O34 IHS"; $resn_exch{"4O2  IHS"}=" O24 IHS";
$resn_exch{"4O4  IHS"}=" O44 IHS"; $resn_exch{"5O1  IHS"}=" O15 IHS"; $resn_exch{"5O3  IHS"}=" O35 IHS";
$resn_exch{"5O2  IHS"}=" O25 IHS"; $resn_exch{"5O4  IHS"}=" O45 IHS"; $resn_exch{"6O1  IHS"}=" O16 IHS";
$resn_exch{"6O3  IHS"}=" O36 IHS"; $resn_exch{"6O2  IHS"}=" O26 IHS"; $resn_exch{"6O4  IHS"}=" O46 IHS";
$resn_exch{" O1P IMC"}=" OP1 IMC"; $resn_exch{" O2P IMC"}=" OP2 IMC"; $resn_exch{" O3P IMC"}=" OP3 IMC";
$resn_exch{" O5* IMC"}=" O5' IMC"; $resn_exch{" C5* IMC"}=" C5' IMC"; $resn_exch{" C4* IMC"}=" C4' IMC";
$resn_exch{" O4* IMC"}=" O4' IMC"; $resn_exch{" C1* IMC"}=" C1' IMC"; $resn_exch{" C2* IMC"}=" C2' IMC";
$resn_exch{" C3* IMC"}=" C3' IMC"; $resn_exch{" O3* IMC"}=" O3' IMC"; $resn_exch{"1CG  IME"}=" CG1 IME";
$resn_exch{"1CD2 IME"}=" CDC IME"; $resn_exch{"1ND1 IME"}=" NDB IME"; $resn_exch{"1CE1 IME"}=" CEB IME";
$resn_exch{"1NE2 IME"}=" NEC IME"; $resn_exch{"2CG  IME"}=" CG2 IME"; $resn_exch{"2CD2 IME"}=" CDM IME";
$resn_exch{"2ND1 IME"}=" NDL IME"; $resn_exch{"2CE1 IME"}=" CEL IME"; $resn_exch{"2NE2 IME"}=" NEM IME";
$resn_exch{"3CG  IME"}=" CG3 IME"; $resn_exch{"3CD2 IME"}=" CDW IME"; $resn_exch{"3ND1 IME"}=" NDV IME";
$resn_exch{"3CE1 IME"}=" CEV IME"; $resn_exch{"3NE2 IME"}=" NEW IME"; $resn_exch{"4CG  IME"}=" CG4 IME";
$resn_exch{"4CD2 IME"}=" CD6 IME"; $resn_exch{"4ND1 IME"}=" ND5 IME"; $resn_exch{"4CE1 IME"}=" CE5 IME";
$resn_exch{"4NE2 IME"}=" NE6 IME"; $resn_exch{"1CG  IMF"}=" CG1 IMF"; $resn_exch{"1CD2 IMF"}=" CDC IMF";
$resn_exch{"1ND1 IMF"}=" NDB IMF"; $resn_exch{"1CE1 IMF"}=" CEB IMF"; $resn_exch{"1NE2 IMF"}=" NEC IMF";
$resn_exch{"2CG  IMF"}=" CG2 IMF"; $resn_exch{"2CD2 IMF"}=" CDM IMF"; $resn_exch{"2ND1 IMF"}=" NDL IMF";
$resn_exch{"2CE1 IMF"}=" CEL IMF"; $resn_exch{"2NE2 IMF"}=" NEM IMF"; $resn_exch{"3CG  IMF"}=" CG3 IMF";
$resn_exch{"3CD2 IMF"}=" CDW IMF"; $resn_exch{"3ND1 IMF"}=" NDV IMF"; $resn_exch{"3CE1 IMF"}=" CEV IMF";
$resn_exch{"3NE2 IMF"}=" NEW IMF"; $resn_exch{"4CG  IMF"}=" CG4 IMF"; $resn_exch{"4CD2 IMF"}=" CD6 IMF";
$resn_exch{"4ND1 IMF"}=" ND5 IMF"; $resn_exch{"4CE1 IMF"}=" CE5 IMF"; $resn_exch{"4NE2 IMF"}=" NE6 IMF";
$resn_exch{" O5* IMG"}=" O5' IMG"; $resn_exch{" C5* IMG"}=" C5' IMG"; $resn_exch{" C4* IMG"}=" C4' IMG";
$resn_exch{" N4* IMG"}=" N4' IMG"; $resn_exch{" C3* IMG"}=" C3' IMG"; $resn_exch{" O3* IMG"}=" O3' IMG";
$resn_exch{" C2* IMG"}=" C2' IMG"; $resn_exch{" O2* IMG"}=" O2' IMG"; $resn_exch{" C1* IMG"}=" C1' IMG";
$resn_exch{" O5* IMH"}=" O5' IMH"; $resn_exch{" C5* IMH"}=" C5' IMH"; $resn_exch{" C4* IMH"}=" C4' IMH";
$resn_exch{" N4* IMH"}=" N4' IMH"; $resn_exch{" C3* IMH"}=" C3' IMH"; $resn_exch{" O3* IMH"}=" O3' IMH";
$resn_exch{" C2* IMH"}=" C2' IMH"; $resn_exch{" O2* IMH"}=" O2' IMH"; $resn_exch{" C1* IMH"}=" C1' IMH";
$resn_exch{" O5* IMP"}=" O5' IMP"; $resn_exch{" C5* IMP"}=" C5' IMP"; $resn_exch{" C4* IMP"}=" C4' IMP";
$resn_exch{" O4* IMP"}=" O4' IMP"; $resn_exch{" C3* IMP"}=" C3' IMP"; $resn_exch{" O3* IMP"}=" O3' IMP";
$resn_exch{" C2* IMP"}=" C2' IMP"; $resn_exch{" O2* IMP"}=" O2' IMP"; $resn_exch{" C1* IMP"}=" C1' IMP";
$resn_exch{" O5* IMR"}=" O5' IMR"; $resn_exch{" C5* IMR"}=" C5' IMR"; $resn_exch{" C4* IMR"}=" C4' IMR";
$resn_exch{" N4* IMR"}=" N4' IMR"; $resn_exch{" C3* IMR"}=" C3' IMR"; $resn_exch{" O3* IMR"}=" O3' IMR";
$resn_exch{" C2* IMR"}=" C2' IMR"; $resn_exch{" O2* IMR"}=" O2' IMR"; $resn_exch{" C1* IMR"}=" C1' IMR";
$resn_exch{" O5* IMU"}=" O5' IMU"; $resn_exch{" C5* IMU"}=" C5' IMU"; $resn_exch{" C4* IMU"}=" C4' IMU";
$resn_exch{" N4* IMU"}=" N4' IMU"; $resn_exch{" C3* IMU"}=" C3' IMU"; $resn_exch{" O3* IMU"}=" O3' IMU";
$resn_exch{" C2* IMU"}=" C2' IMU"; $resn_exch{" O2* IMU"}=" O2' IMU"; $resn_exch{" C1* IMU"}=" C1' IMU";
$resn_exch{"'O10 IN3"}=" OA' IN3"; $resn_exch{"'C11 IN3"}=" CB' IN3"; $resn_exch{"'C12 IN3"}=" CC' IN3";
$resn_exch{"'C14 IN3"}=" CE' IN3"; $resn_exch{"'C15 IN3"}=" CF' IN3"; $resn_exch{"'C16 IN3"}=" CG' IN3";
$resn_exch{"'C17 IN3"}=" CH' IN3"; $resn_exch{"'O18 IN3"}=" OI' IN3"; $resn_exch{"'N19 IN3"}=" NJ' IN3";
$resn_exch{"'N20 IN3"}=" NK' IN3"; $resn_exch{"'N24 IN3"}=" NO' IN3"; $resn_exch{"'O10 IN6"}=" OA' IN6";
$resn_exch{"'C11 IN6"}=" CB' IN6"; $resn_exch{"'C12 IN6"}=" CC' IN6"; $resn_exch{"'C14 IN6"}=" CE' IN6";
$resn_exch{"'C15 IN6"}=" CF' IN6"; $resn_exch{"'C16 IN6"}=" CG' IN6"; $resn_exch{"'C17 IN6"}=" CH' IN6";
$resn_exch{"'O18 IN6"}=" OI' IN6"; $resn_exch{"'N19 IN6"}=" NJ' IN6"; $resn_exch{"'N20 IN6"}=" NK' IN6";
$resn_exch{"'N24 IN6"}=" NO' IN6"; $resn_exch{" C1\" INB"}=" C1B INB"; $resn_exch{" C2\" INB"}=" C2B INB";
$resn_exch{" CM\" INB"}=" CMB INB"; $resn_exch{" O5* IOT"}=" O5' IOT"; $resn_exch{" C5* IOT"}=" C5' IOT";
$resn_exch{" C4* IOT"}=" C4' IOT"; $resn_exch{" O4* IOT"}=" O4' IOT"; $resn_exch{" C3* IOT"}=" C3' IOT";
$resn_exch{" O3* IOT"}=" O3' IOT"; $resn_exch{" C2* IOT"}=" C2' IOT"; $resn_exch{" O2* IOT"}=" O2' IOT";
$resn_exch{" C1* IOT"}=" C1' IOT"; $resn_exch{" O   IOY"}=" OXT IOY"; $resn_exch{" OXT IOY"}=" O   IOY";
$resn_exch{" C1* IQB"}=" C1' IQB"; $resn_exch{" C2* IQB"}=" C2B IQB"; $resn_exch{" C3* IQB"}=" C3B IQB";
$resn_exch{" C4* IQB"}=" C4' IQB"; $resn_exch{"BR4* IQB"}="BR4' IQB"; $resn_exch{" C5* IQB"}=" C5B IQB";
$resn_exch{" C6* IQB"}=" C6B IQB"; $resn_exch{" C1* IRF"}=" C1' IRF"; $resn_exch{" C2* IRF"}=" C2' IRF";
$resn_exch{" O2* IRF"}=" O2' IRF"; $resn_exch{" C3* IRF"}=" C3' IRF"; $resn_exch{" O3* IRF"}=" O3' IRF";
$resn_exch{" C4* IRF"}=" C4' IRF"; $resn_exch{" O4* IRF"}=" O4' IRF"; $resn_exch{" C5* IRF"}=" C5' IRF";
$resn_exch{" O5* IRF"}=" O5' IRF"; $resn_exch{" O5* IRP"}=" O5' IRP"; $resn_exch{" C5* IRP"}=" C5' IRP";
$resn_exch{" C4* IRP"}=" C4' IRP"; $resn_exch{" N4* IRP"}=" N4' IRP"; $resn_exch{" C3* IRP"}=" C3' IRP";
$resn_exch{" O3* IRP"}=" O3' IRP"; $resn_exch{" C2* IRP"}=" C2' IRP"; $resn_exch{" O2* IRP"}=" O2' IRP";
$resn_exch{" C1* IRP"}=" C1' IRP"; $resn_exch{" O5* ITT"}=" O5' ITT"; $resn_exch{" C5* ITT"}=" C5' ITT";
$resn_exch{" C4* ITT"}=" C4' ITT"; $resn_exch{" O4* ITT"}=" O4' ITT"; $resn_exch{" C3* ITT"}=" C3' ITT";
$resn_exch{" O3* ITT"}=" O3' ITT"; $resn_exch{" C2* ITT"}=" C2' ITT"; $resn_exch{" O2* ITT"}=" O2' ITT";
$resn_exch{" C1* ITT"}=" C1' ITT"; $resn_exch{" O1P  IU"}=" OP1  IU"; $resn_exch{" O2P  IU"}=" OP2  IU";
$resn_exch{" O3P  IU"}=" OP3  IU"; $resn_exch{" O5*  IU"}=" O5'  IU"; $resn_exch{" C5*  IU"}=" C5'  IU";
$resn_exch{" C4*  IU"}=" C4'  IU"; $resn_exch{" O4*  IU"}=" O4'  IU"; $resn_exch{" C3*  IU"}=" C3'  IU";
$resn_exch{" O3*  IU"}=" O3'  IU"; $resn_exch{" C2*  IU"}=" C2'  IU"; $resn_exch{" O2*  IU"}=" O2'  IU";
$resn_exch{" C1*  IU"}=" C1'  IU"; $resn_exch{" C1* JW5"}=" C1' JW5"; $resn_exch{" O4* JW5"}=" O4' JW5";
$resn_exch{" C2* JW5"}=" C2' JW5"; $resn_exch{" O2* JW5"}=" O2' JW5"; $resn_exch{" C3* JW5"}=" C3' JW5";
$resn_exch{" O3* JW5"}=" O3' JW5"; $resn_exch{" C4* JW5"}=" C4' JW5"; $resn_exch{" C5* JW5"}=" C5' JW5";
$resn_exch{" O5* JW5"}=" O5' JW5"; $resn_exch{" C1* K05"}=" C1' K05"; $resn_exch{" C2* K05"}=" C2' K05";
$resn_exch{" C3* K05"}=" C3' K05"; $resn_exch{" C4* K05"}=" C4' K05"; $resn_exch{" C5* K05"}=" C5' K05";
$resn_exch{" C6* K05"}=" C6' K05"; $resn_exch{" O3P KAG"}=" OP3 KAG"; $resn_exch{" O1P KAG"}=" OP1 KAG";
$resn_exch{" O2P KAG"}=" OP2 KAG"; $resn_exch{" O5* KAG"}=" O5' KAG"; $resn_exch{" C5* KAG"}=" C5' KAG";
$resn_exch{" C4* KAG"}=" C4' KAG"; $resn_exch{" O4* KAG"}=" O4' KAG"; $resn_exch{" C3* KAG"}=" C3' KAG";
$resn_exch{" O3* KAG"}=" O3' KAG"; $resn_exch{" C2* KAG"}=" C2' KAG"; $resn_exch{" C1* KAG"}=" C1' KAG";
$resn_exch{" OX1 KCX"}=" OQ1 KCX"; $resn_exch{" OX2 KCX"}=" OQ2 KCX"; $resn_exch{" O1  KGC"}=" OXT KGC";
$resn_exch{" OXT KGC"}=" O   KGC"; $resn_exch{" O1* KHO"}=" O1' KHO"; $resn_exch{" C1* KHO"}=" C1' KHO";
$resn_exch{" C2* KHO"}=" C2' KHO"; $resn_exch{" C3* KHO"}=" C3' KHO"; $resn_exch{" O3* KHO"}=" O3' KHO";
$resn_exch{" O2* KHO"}=" O2' KHO"; $resn_exch{" O4* KHO"}=" O4' KHO"; $resn_exch{" C4* KHO"}=" C4' KHO";
$resn_exch{" C5* KHO"}=" C5' KHO"; $resn_exch{" O5* KHO"}=" O5' KHO"; $resn_exch{" O5* KHP"}=" O5' KHP";
$resn_exch{" C5* KHP"}=" C5B KHP"; $resn_exch{" C4* KHP"}=" C4B KHP"; $resn_exch{" O4* KHP"}=" O4' KHP";
$resn_exch{" C1* KHP"}=" C1B KHP"; $resn_exch{" C2* KHP"}=" C2B KHP"; $resn_exch{" O2* KHP"}=" O2B KHP";
$resn_exch{" C3* KHP"}=" C3B KHP"; $resn_exch{" O3* KHP"}=" O3B KHP"; $resn_exch{" O1* KHP"}=" O1' KHP";
$resn_exch{"2N   KT3"}=" N2A KT3"; $resn_exch{"1CD  KT3"}=" CD1 KT3"; $resn_exch{"1OE1 KT3"}=" OEB KT3";
$resn_exch{"1CG  KT3"}=" CG1 KT3"; $resn_exch{"1CB  KT3"}=" CB1 KT3"; $resn_exch{"1CA  KT3"}=" CA1 KT3";
$resn_exch{"1C   KT3"}=" C1A KT3"; $resn_exch{"1OXT KT3"}=" O11 KT3"; $resn_exch{"1O   KT3"}=" O1A KT3";
$resn_exch{"1N   KT3"}=" N1A KT3"; $resn_exch{"3OE2 KT3"}=" OEN KT3"; $resn_exch{"3OE1 KT3"}=" OED KT3";
$resn_exch{"3CD  KT3"}=" CD3 KT3"; $resn_exch{"3CG  KT3"}=" CG3 KT3"; $resn_exch{"3CB  KT3"}=" CB3 KT3";
$resn_exch{"3CA  KT3"}=" CA3 KT3"; $resn_exch{"3C   KT3"}=" C3A KT3"; $resn_exch{"3O   KT3"}=" O3  KT3";
$resn_exch{"3OXT KT3"}=" O12 KT3"; $resn_exch{"3N   KT3"}=" N3A KT3"; $resn_exch{"2OE1 KT3"}=" OEC KT3";
$resn_exch{"2CD  KT3"}=" CD2 KT3"; $resn_exch{"2CG  KT3"}=" CG2 KT3"; $resn_exch{"2CB  KT3"}=" CB2 KT3";
$resn_exch{"2CA  KT3"}=" CA2 KT3"; $resn_exch{"2C   KT3"}=" C2A KT3"; $resn_exch{"2O   KT3"}=" O2  KT3";
$resn_exch{"2OXT KT3"}=" O13 KT3"; $resn_exch{"2N   KT5"}=" N2A KT5"; $resn_exch{"1CD  KT5"}=" CD1 KT5";
$resn_exch{"1OE1 KT5"}=" OEB KT5"; $resn_exch{"1CG  KT5"}=" CG1 KT5"; $resn_exch{"1CB  KT5"}=" CB1 KT5";
$resn_exch{"1CA  KT5"}=" CA1 KT5"; $resn_exch{"1C   KT5"}=" C1A KT5"; $resn_exch{"1OXT KT5"}=" O11 KT5";
$resn_exch{"1O   KT5"}=" O1A KT5"; $resn_exch{"1N   KT5"}=" N1A KT5"; $resn_exch{"5OE1 KT5"}=" OEF KT5";
$resn_exch{"5OE2 KT5"}=" OEP KT5"; $resn_exch{"5CD  KT5"}=" CD5 KT5"; $resn_exch{"5CG  KT5"}=" CG5 KT5";
$resn_exch{"5CB  KT5"}=" CB5 KT5"; $resn_exch{"5CA  KT5"}=" CA5 KT5"; $resn_exch{"5C   KT5"}=" C5A KT5";
$resn_exch{"5O   KT5"}=" O5  KT5"; $resn_exch{"5OXT KT5"}=" O12 KT5"; $resn_exch{"5N   KT5"}=" N5  KT5";
$resn_exch{"4OE1 KT5"}=" OEE KT5"; $resn_exch{"4CD  KT5"}=" CD4 KT5"; $resn_exch{"4CG  KT5"}=" CG4 KT5";
$resn_exch{"4CB  KT5"}=" CB4 KT5"; $resn_exch{"4CA  KT5"}=" CA4 KT5"; $resn_exch{"4C   KT5"}=" C4A KT5";
$resn_exch{"4O   KT5"}=" O4  KT5"; $resn_exch{"4OXT KT5"}=" O13 KT5"; $resn_exch{"4N   KT5"}=" N4  KT5";
$resn_exch{"3OE1 KT5"}=" OED KT5"; $resn_exch{"3CD  KT5"}=" CD3 KT5"; $resn_exch{"3CG  KT5"}=" CG3 KT5";
$resn_exch{"3CB  KT5"}=" CB3 KT5"; $resn_exch{"3CA  KT5"}=" CA3 KT5"; $resn_exch{"3C   KT5"}=" C3A KT5";
$resn_exch{"3O   KT5"}=" O3  KT5"; $resn_exch{"3OXT KT5"}=" O14 KT5"; $resn_exch{"3N   KT5"}=" N3A KT5";
$resn_exch{"2OE1 KT5"}=" OEC KT5"; $resn_exch{"2CD  KT5"}=" CD2 KT5"; $resn_exch{"2CG  KT5"}=" CG2 KT5";
$resn_exch{"2CB  KT5"}=" CB2 KT5"; $resn_exch{"2CA  KT5"}=" CA2 KT5"; $resn_exch{"2C   KT5"}=" C2A KT5";
$resn_exch{"2O   KT5"}=" O2  KT5"; $resn_exch{"2OXT KT5"}=" O15 KT5"; $resn_exch{"1O4' LAM"}=" O'5 LAM";
$resn_exch{"2O4' LAM"}=" O'6 LAM"; $resn_exch{" O5* LAQ"}=" O5' LAQ"; $resn_exch{" C5* LAQ"}=" C5' LAQ";
$resn_exch{" C4* LAQ"}=" C4' LAQ"; $resn_exch{" C3* LAQ"}=" C3' LAQ"; $resn_exch{" O3* LAQ"}=" O3' LAQ";
$resn_exch{" C2* LAQ"}=" C2' LAQ"; $resn_exch{" O2* LAQ"}=" O2' LAQ"; $resn_exch{" C1* LAQ"}=" C1' LAQ";
$resn_exch{" O4* LAQ"}=" O4' LAQ"; $resn_exch{" O3P  LC"}=" OP3  LC"; $resn_exch{" O1P  LC"}=" OP1  LC";
$resn_exch{" O2P  LC"}=" OP2  LC"; $resn_exch{" O5*  LC"}=" O5'  LC"; $resn_exch{" C5*  LC"}=" C5'  LC";
$resn_exch{" C4*  LC"}=" C4'  LC"; $resn_exch{" O4*  LC"}=" O4'  LC"; $resn_exch{" C3*  LC"}=" C3'  LC";
$resn_exch{" O3*  LC"}=" O3'  LC"; $resn_exch{" C2*  LC"}=" C2'  LC"; $resn_exch{" O2*  LC"}=" O2'  LC";
$resn_exch{" C1*  LC"}=" C1'  LC"; $resn_exch{" O5* LCA"}=" O5' LCA"; $resn_exch{" C5* LCA"}=" C5' LCA";
$resn_exch{" C3* LCA"}=" C3' LCA"; $resn_exch{" C6* LCA"}=" C6' LCA"; $resn_exch{" C2* LCA"}=" C2' LCA";
$resn_exch{" C4* LCA"}=" C4' LCA"; $resn_exch{" C1* LCA"}=" C1' LCA"; $resn_exch{" O4* LCA"}=" O4' LCA";
$resn_exch{" O3* LCA"}=" O3' LCA"; $resn_exch{" O2* LCA"}=" O2' LCA"; $resn_exch{" O5* LCC"}=" O5' LCC";
$resn_exch{" C5* LCC"}=" C5' LCC"; $resn_exch{" C4* LCC"}=" C4' LCC"; $resn_exch{" O4* LCC"}=" O4' LCC";
$resn_exch{" C1* LCC"}=" C1' LCC"; $resn_exch{" C3* LCC"}=" C3' LCC"; $resn_exch{" C2* LCC"}=" C2' LCC";
$resn_exch{" O2* LCC"}=" O2' LCC"; $resn_exch{" O3* LCC"}=" O3' LCC"; $resn_exch{" C6* LCC"}=" C6' LCC";
$resn_exch{" O1P LCG"}=" OP1 LCG"; $resn_exch{" O5* LCG"}=" O5' LCG"; $resn_exch{" C5* LCG"}=" C5' LCG";
$resn_exch{" C3* LCG"}=" C3' LCG"; $resn_exch{" C6* LCG"}=" C6' LCG"; $resn_exch{" C2* LCG"}=" C2' LCG";
$resn_exch{" C4* LCG"}=" C4' LCG"; $resn_exch{" C1* LCG"}=" C1' LCG"; $resn_exch{" O4* LCG"}=" O4' LCG";
$resn_exch{" O2P LCG"}=" OP2 LCG"; $resn_exch{" O2* LCG"}=" O2' LCG"; $resn_exch{" O3* LCG"}=" O3' LCG";
$resn_exch{" O3P LCG"}=" OP3 LCG"; $resn_exch{" O5* LCH"}=" O5' LCH"; $resn_exch{" C5* LCH"}=" C5' LCH";
$resn_exch{" C4* LCH"}=" C4' LCH"; $resn_exch{" O4* LCH"}=" O4' LCH"; $resn_exch{" C1* LCH"}=" C1' LCH";
$resn_exch{" C3* LCH"}=" C3' LCH"; $resn_exch{" C2* LCH"}=" C2' LCH"; $resn_exch{" O2* LCH"}=" O2' LCH";
$resn_exch{" O3* LCH"}=" O3' LCH"; $resn_exch{" C6* LCH"}=" C6' LCH"; $resn_exch{" C1* LFN"}=" C1' LFN";
$resn_exch{" O3P  LG"}=" OP3  LG"; $resn_exch{" O1P  LG"}=" OP1  LG"; $resn_exch{" O2P  LG"}=" OP2  LG";
$resn_exch{" O5*  LG"}=" O5'  LG"; $resn_exch{" C5*  LG"}=" C5'  LG"; $resn_exch{" C4*  LG"}=" C4'  LG";
$resn_exch{" O4*  LG"}=" O4'  LG"; $resn_exch{" C3*  LG"}=" C3'  LG"; $resn_exch{" O3*  LG"}=" O3'  LG";
$resn_exch{" C2*  LG"}=" C2'  LG"; $resn_exch{" O2*  LG"}=" O2'  LG"; $resn_exch{" C1*  LG"}=" C1'  LG";
$resn_exch{" O1P LGP"}=" OP1 LGP"; $resn_exch{" O2P LGP"}=" OP2 LGP"; $resn_exch{" O3P LGP"}=" OP3 LGP";
$resn_exch{" O5* LGP"}=" O5' LGP"; $resn_exch{" C5* LGP"}=" C5' LGP"; $resn_exch{" C4* LGP"}=" C4' LGP";
$resn_exch{" O4* LGP"}=" O4' LGP"; $resn_exch{" C3* LGP"}=" C3' LGP"; $resn_exch{" O3* LGP"}=" O3' LGP";
$resn_exch{" C1* LGP"}=" C1' LGP"; $resn_exch{" O3P LHU"}=" OP3 LHU"; $resn_exch{" O1P LHU"}=" OP1 LHU";
$resn_exch{" O2P LHU"}=" OP2 LHU"; $resn_exch{" O5* LHU"}=" O5' LHU"; $resn_exch{" C5* LHU"}=" C5' LHU";
$resn_exch{" C4* LHU"}=" C4' LHU"; $resn_exch{" O4* LHU"}=" O4' LHU"; $resn_exch{" C3* LHU"}=" C3' LHU";
$resn_exch{" O3* LHU"}=" O3' LHU"; $resn_exch{" C2* LHU"}=" C2' LHU"; $resn_exch{" O2* LHU"}=" O2' LHU";
$resn_exch{" C1* LHU"}=" C1' LHU"; $resn_exch{"C4A' LIH"}=" C4X LIH"; $resn_exch{"C8A' LIH"}=" C8X LIH";
$resn_exch{"C2A' LII"}=" C2X LII"; $resn_exch{"C5A' LII"}=" C5X LII"; $resn_exch{" C1* LKC"}=" C1' LKC";
$resn_exch{" C2* LKC"}=" C2' LKC"; $resn_exch{" C3* LKC"}=" C3' LKC"; $resn_exch{" C4* LKC"}=" C4' LKC";
$resn_exch{" O4* LKC"}=" O4' LKC"; $resn_exch{" O3* LKC"}=" O3' LKC"; $resn_exch{" C5* LKC"}=" C5' LKC";
$resn_exch{" O5* LKC"}=" O5' LKC"; $resn_exch{" O2* LKC"}=" O2' LKC"; $resn_exch{" C6* LKC"}=" C6' LKC";
$resn_exch{" C2A LLP"}=" C2' LLP"; $resn_exch{" C4A LLP"}=" C4' LLP"; $resn_exch{" C5A LLP"}=" C5' LLP";
$resn_exch{" O4P LLP"}=" OP4 LLP"; $resn_exch{" O1P LLP"}=" OP1 LLP"; $resn_exch{" O2P LLP"}=" OP2 LLP";
$resn_exch{" O3P LLP"}=" OP3 LLP"; $resn_exch{" O5* LMS"}=" O5' LMS"; $resn_exch{" C5* LMS"}=" C5' LMS";
$resn_exch{" C4* LMS"}=" C4' LMS"; $resn_exch{" O4* LMS"}=" O4' LMS"; $resn_exch{" C3* LMS"}=" C3' LMS";
$resn_exch{" C1* LMT"}=" C1B LMT"; $resn_exch{" C2* LMT"}=" C2B LMT"; $resn_exch{" C3* LMT"}=" C3B LMT";
$resn_exch{" C4* LMT"}=" C4B LMT"; $resn_exch{" C5* LMT"}=" C5B LMT"; $resn_exch{" C6* LMT"}=" C6B LMT";
$resn_exch{" O1* LMT"}=" O1B LMT"; $resn_exch{" O2* LMT"}=" O2B LMT"; $resn_exch{" O3* LMT"}=" O3B LMT";
$resn_exch{" O4* LMT"}=" O4' LMT"; $resn_exch{" O5* LMT"}=" O5B LMT"; $resn_exch{" O6* LMT"}=" O6B LMT";
$resn_exch{" C1* LMU"}=" C1B LMU"; $resn_exch{" C2* LMU"}=" C2B LMU"; $resn_exch{" C3* LMU"}=" C3B LMU";
$resn_exch{" C4* LMU"}=" C4B LMU"; $resn_exch{" C5* LMU"}=" C5B LMU"; $resn_exch{" C6* LMU"}=" C6B LMU";
$resn_exch{" O1* LMU"}=" O1B LMU"; $resn_exch{" O2* LMU"}=" O2B LMU"; $resn_exch{" O3* LMU"}=" O3B LMU";
$resn_exch{" O4* LMU"}=" O4' LMU"; $resn_exch{" O5* LMU"}=" O5B LMU"; $resn_exch{" O6* LMU"}=" O6B LMU";
$resn_exch{" NN7 LNC"}=" N7N LNC"; $resn_exch{" NC7 LNC"}=" C7N LNC"; $resn_exch{" NO7 LNC"}=" O7N LNC";
$resn_exch{" NC3 LNC"}=" C3N LNC"; $resn_exch{" NC4 LNC"}=" C4N LNC"; $resn_exch{" NC5 LNC"}=" C5N LNC";
$resn_exch{" C_S LNC"}=" CS  LNC"; $resn_exch{" NC6 LNC"}=" C6N LNC"; $resn_exch{" NN1 LNC"}=" N1N LNC";
$resn_exch{" NC2 LNC"}=" C2N LNC"; $resn_exch{"NC1* LNC"}=" C1D LNC"; $resn_exch{"NO4* LNC"}=" O4D LNC";
$resn_exch{"NC4* LNC"}=" C4D LNC"; $resn_exch{"NC5* LNC"}=" C5D LNC"; $resn_exch{"NO5* LNC"}=" O5D LNC";
$resn_exch{" NP  LNC"}=" PN  LNC"; $resn_exch{" NO1 LNC"}=" O1N LNC"; $resn_exch{" NO2 LNC"}=" O2N LNC";
$resn_exch{" AP  LNC"}=" PA  LNC"; $resn_exch{" AO1 LNC"}=" O1A LNC"; $resn_exch{" AO2 LNC"}=" O2A LNC";
$resn_exch{"AO5* LNC"}=" O5B LNC"; $resn_exch{"AC5* LNC"}=" C5B LNC"; $resn_exch{"AC4* LNC"}=" C4B LNC";
$resn_exch{"AO4* LNC"}=" O4B LNC"; $resn_exch{"AC1* LNC"}=" C1B LNC"; $resn_exch{"AC2* LNC"}=" C2B LNC";
$resn_exch{"AO2* LNC"}=" O2B LNC"; $resn_exch{"AC3* LNC"}=" C3B LNC"; $resn_exch{"AO3* LNC"}=" O3B LNC";
$resn_exch{" AN9 LNC"}=" N9A LNC"; $resn_exch{" AC8 LNC"}=" C8A LNC"; $resn_exch{" AN7 LNC"}=" N7A LNC";
$resn_exch{" AC5 LNC"}=" C5A LNC"; $resn_exch{" AC6 LNC"}=" C6A LNC"; $resn_exch{" AN6 LNC"}=" N6A LNC";
$resn_exch{" AN1 LNC"}=" N1A LNC"; $resn_exch{" AC2 LNC"}=" C2A LNC"; $resn_exch{" AN3 LNC"}=" N3A LNC";
$resn_exch{" AC4 LNC"}=" C4A LNC"; $resn_exch{"NC3* LNC"}=" C3D LNC"; $resn_exch{"NO3* LNC"}=" O3D LNC";
$resn_exch{"NC2* LNC"}=" C2D LNC"; $resn_exch{"NO2* LNC"}=" O2D LNC"; $resn_exch{" C1* LVS"}=" C1' LVS";
$resn_exch{" O1* LVS"}=" O1' LVS"; $resn_exch{" O2* LVS"}=" O2' LVS"; $resn_exch{" C2* LVS"}=" C2' LVS";
$resn_exch{" O1P M1G"}=" OP1 M1G"; $resn_exch{" O2P M1G"}=" OP2 M1G"; $resn_exch{" O3P M1G"}=" OP3 M1G";
$resn_exch{" O5* M1G"}=" O5' M1G"; $resn_exch{" C5* M1G"}=" C5' M1G"; $resn_exch{" C4* M1G"}=" C4' M1G";
$resn_exch{" O4* M1G"}=" O4' M1G"; $resn_exch{" C3* M1G"}=" C3' M1G"; $resn_exch{" O3* M1G"}=" O3' M1G";
$resn_exch{" C2* M1G"}=" C2' M1G"; $resn_exch{" C1* M1G"}=" C1' M1G"; $resn_exch{" O1P M2G"}=" OP1 M2G";
$resn_exch{" O2P M2G"}=" OP2 M2G"; $resn_exch{" O3P M2G"}=" OP3 M2G"; $resn_exch{" O5* M2G"}=" O5' M2G";
$resn_exch{" C5* M2G"}=" C5' M2G"; $resn_exch{" C4* M2G"}=" C4' M2G"; $resn_exch{" O4* M2G"}=" O4' M2G";
$resn_exch{" C3* M2G"}=" C3' M2G"; $resn_exch{" O3* M2G"}=" O3' M2G"; $resn_exch{" C2* M2G"}=" C2' M2G";
$resn_exch{" O2* M2G"}=" O2' M2G"; $resn_exch{" C1* M2G"}=" C1' M2G"; $resn_exch{" N2* M5M"}=" N2' M5M";
$resn_exch{" C6* M5M"}=" C6' M5M"; $resn_exch{" C7* M5M"}=" C7' M5M"; $resn_exch{" O8* M5M"}=" O8' M5M";
$resn_exch{" O1P M5M"}=" OP1 M5M"; $resn_exch{" O2P M5M"}=" OP2 M5M"; $resn_exch{" O5* M5M"}=" O5' M5M";
$resn_exch{" C5* M5M"}=" C5' M5M"; $resn_exch{" C4* M5M"}=" C4' M5M"; $resn_exch{" O4* M5M"}=" O4' M5M";
$resn_exch{" C1* M5M"}=" C1' M5M"; $resn_exch{" C2* M5M"}=" C2' M5M"; $resn_exch{" C3* M5M"}=" C3' M5M";
$resn_exch{" O3* M5M"}=" O3' M5M"; $resn_exch{" O3P M5M"}=" OP3 M5M"; $resn_exch{" O5* M7G"}=" O5' M7G";
$resn_exch{" C5* M7G"}=" C5' M7G"; $resn_exch{" C4* M7G"}=" C4' M7G"; $resn_exch{" O4* M7G"}=" O4' M7G";
$resn_exch{" C3* M7G"}=" C3' M7G"; $resn_exch{" O3* M7G"}=" O3' M7G"; $resn_exch{" C2* M7G"}=" C2' M7G";
$resn_exch{" O2* M7G"}=" O2' M7G"; $resn_exch{" C1* M7G"}=" C1' M7G"; $resn_exch{" O1P MA6"}=" OP1 MA6";
$resn_exch{" O2P MA6"}=" OP2 MA6"; $resn_exch{" O3P MA6"}=" OP3 MA6"; $resn_exch{" O5* MA6"}=" O5' MA6";
$resn_exch{" C5* MA6"}=" C5' MA6"; $resn_exch{" C4* MA6"}=" C4' MA6"; $resn_exch{" O4* MA6"}=" O4' MA6";
$resn_exch{" C1* MA6"}=" C1' MA6"; $resn_exch{" C2* MA6"}=" C2' MA6"; $resn_exch{" O2* MA6"}=" O2' MA6";
$resn_exch{" C3* MA6"}=" C3' MA6"; $resn_exch{" O3* MA6"}=" O3' MA6"; $resn_exch{" O1P MA7"}=" OP1 MA7";
$resn_exch{" O2P MA7"}=" OP2 MA7"; $resn_exch{" O5* MA7"}=" O5' MA7"; $resn_exch{" C2* MA7"}=" C2' MA7";
$resn_exch{" C5* MA7"}=" C5' MA7"; $resn_exch{" C4* MA7"}=" C4' MA7"; $resn_exch{" O4* MA7"}=" O4' MA7";
$resn_exch{" C1* MA7"}=" C1' MA7"; $resn_exch{" C3* MA7"}=" C3' MA7"; $resn_exch{" O3* MA7"}=" O3' MA7";
$resn_exch{" O3P MA7"}=" OP3 MA7"; $resn_exch{" O1P MAD"}=" OP1 MAD"; $resn_exch{" O2P MAD"}=" OP2 MAD";
$resn_exch{" O5* MAD"}=" O5' MAD"; $resn_exch{" C5* MAD"}=" C5' MAD"; $resn_exch{" C4* MAD"}=" C4' MAD";
$resn_exch{" O4* MAD"}=" O4' MAD"; $resn_exch{" C3* MAD"}=" C3' MAD"; $resn_exch{" O3* MAD"}=" O3' MAD";
$resn_exch{" C2* MAD"}=" C2' MAD"; $resn_exch{" O2* MAD"}=" O2' MAD"; $resn_exch{" C1* MAD"}=" C1' MAD";
$resn_exch{" C5* MAO"}=" C5' MAO"; $resn_exch{" C4* MAO"}=" C4' MAO"; $resn_exch{" O4* MAO"}=" O4' MAO";
$resn_exch{" C1* MAO"}=" C1' MAO"; $resn_exch{" C2* MAO"}=" C2' MAO"; $resn_exch{" O2* MAO"}=" O2' MAO";
$resn_exch{" C3* MAO"}=" C3' MAO"; $resn_exch{" O3* MAO"}=" O3' MAO"; $resn_exch{" O5* MAP"}=" O5' MAP";
$resn_exch{" C5* MAP"}=" C5' MAP"; $resn_exch{" C4* MAP"}=" C4' MAP"; $resn_exch{" O4* MAP"}=" O4' MAP";
$resn_exch{" C3* MAP"}=" C3' MAP"; $resn_exch{" O3* MAP"}=" O3' MAP"; $resn_exch{" C2* MAP"}=" C2' MAP";
$resn_exch{" O2* MAP"}=" O2' MAP"; $resn_exch{" C1* MAP"}=" C1' MAP"; $resn_exch{" C1* MAR"}=" C1' MAR";
$resn_exch{" C2* MAR"}=" C2' MAR"; $resn_exch{" C3* MAR"}=" C3' MAR"; $resn_exch{" N3* MAR"}=" N3' MAR";
$resn_exch{" C4* MAR"}=" C4' MAR"; $resn_exch{" O4* MAR"}=" O4' MAR"; $resn_exch{" C5* MAR"}=" C5' MAR";
$resn_exch{" O5* MAR"}=" O5' MAR"; $resn_exch{" C6* MAR"}=" C6' MAR"; $resn_exch{" C4* MBP"}=" C4' MBP";
$resn_exch{" O4* MBP"}=" O4' MBP"; $resn_exch{" C3* MBP"}=" C3' MBP"; $resn_exch{" C2* MBP"}=" C2' MBP";
$resn_exch{" O   MBQ"}=" OXT MBQ"; $resn_exch{" O1  MBQ"}=" O   MBQ"; $resn_exch{" O1P MBZ"}=" OP1 MBZ";
$resn_exch{" O2P MBZ"}=" OP2 MBZ"; $resn_exch{" O3P MBZ"}=" OP3 MBZ"; $resn_exch{" O5* MBZ"}=" O5' MBZ";
$resn_exch{" O3* MBZ"}=" O3' MBZ"; $resn_exch{" C1* MBZ"}=" C1' MBZ"; $resn_exch{" C2* MBZ"}=" C2' MBZ";
$resn_exch{" C3* MBZ"}=" C3' MBZ"; $resn_exch{" C4* MBZ"}=" C4' MBZ"; $resn_exch{" C5* MBZ"}=" C5' MBZ";
$resn_exch{" O4* MBZ"}=" O4' MBZ"; $resn_exch{" O1  MC1"}=" OXT MC1"; $resn_exch{" OT  MCG"}=" OXT MCG";
$resn_exch{" C1* MCN"}=" C1' MCN"; $resn_exch{" C2* MCN"}=" C2D MCN"; $resn_exch{" O2* MCN"}=" O2' MCN";
$resn_exch{" C3* MCN"}=" C3' MCN"; $resn_exch{" C4* MCN"}=" C4D MCN"; $resn_exch{" O4* MCN"}=" O4D MCN";
$resn_exch{" O3* MCN"}=" O3' MCN"; $resn_exch{" C5* MCN"}=" C5' MCN"; $resn_exch{" O5* MCN"}=" O5' MCN";
$resn_exch{" C1* MCY"}=" C1' MCY"; $resn_exch{" C2* MCY"}=" C2' MCY"; $resn_exch{" C3* MCY"}=" C3' MCY";
$resn_exch{" C4* MCY"}=" C4' MCY"; $resn_exch{" O4* MCY"}=" O4' MCY"; $resn_exch{" O3* MCY"}=" O3' MCY";
$resn_exch{" C5* MCY"}=" C5' MCY"; $resn_exch{" O5* MCY"}=" O5' MCY"; $resn_exch{" O5* MD1"}=" O5' MD1";
$resn_exch{" C5* MD1"}=" C5' MD1"; $resn_exch{" C4* MD1"}=" C4' MD1"; $resn_exch{" O4* MD1"}=" O4' MD1";
$resn_exch{" C1* MD1"}=" C1' MD1"; $resn_exch{" C2* MD1"}=" C2' MD1"; $resn_exch{" O2* MD1"}=" O2' MD1";
$resn_exch{" C3* MD1"}=" C3' MD1"; $resn_exch{" O3* MD1"}=" O3' MD1"; $resn_exch{" AP1 MDE"}=" P1A MDE";
$resn_exch{" AO1 MDE"}=" O1A MDE"; $resn_exch{" AO2 MDE"}=" O2A MDE"; $resn_exch{"AO5* MDE"}=" O5B MDE";
$resn_exch{"AC5* MDE"}=" C5B MDE"; $resn_exch{"AC4* MDE"}=" C4B MDE"; $resn_exch{"AO4* MDE"}=" O4B MDE";
$resn_exch{"AC1* MDE"}=" C1B MDE"; $resn_exch{" AN9 MDE"}=" N9A MDE"; $resn_exch{" AC4 MDE"}=" C4A MDE";
$resn_exch{" AN3 MDE"}=" N3A MDE"; $resn_exch{" AC2 MDE"}=" C2A MDE"; $resn_exch{" AN1 MDE"}=" N1A MDE";
$resn_exch{" AC6 MDE"}=" C6A MDE"; $resn_exch{" AN6 MDE"}=" N6A MDE"; $resn_exch{" AC5 MDE"}=" C5A MDE";
$resn_exch{" AN7 MDE"}=" N7A MDE"; $resn_exch{" AC8 MDE"}=" C8A MDE"; $resn_exch{"AC2* MDE"}=" C2B MDE";
$resn_exch{"AO2* MDE"}=" O2B MDE"; $resn_exch{"AC3* MDE"}=" C3B MDE"; $resn_exch{"AO3* MDE"}=" O3B MDE";
$resn_exch{"AP3* MDE"}=" P3B MDE"; $resn_exch{"AO7* MDE"}=" O7B MDE"; $resn_exch{"AO8* MDE"}=" O8B MDE";
$resn_exch{"AO9* MDE"}=" O9B MDE"; $resn_exch{" AO3 MDE"}=" O3A MDE"; $resn_exch{" AP2 MDE"}=" P2A MDE";
$resn_exch{" AO4 MDE"}=" O4A MDE"; $resn_exch{" AO5 MDE"}=" O5A MDE"; $resn_exch{" AO6 MDE"}=" O6A MDE";
$resn_exch{"PC12 MDE"}=" CCP MDE"; $resn_exch{"PC11 MDE"}=" CBP MDE"; $resn_exch{"PC13 MDE"}=" CDP MDE";
$resn_exch{"PC14 MDE"}=" CEP MDE"; $resn_exch{"PC10 MDE"}=" CAP MDE"; $resn_exch{"PO10 MDE"}=" OAP MDE";
$resn_exch{" PC9 MDE"}=" C9P MDE"; $resn_exch{" PO9 MDE"}=" O9P MDE"; $resn_exch{" PN8 MDE"}=" N8P MDE";
$resn_exch{" PC7 MDE"}=" C7P MDE"; $resn_exch{" PC6 MDE"}=" C6P MDE"; $resn_exch{" PC5 MDE"}=" C5P MDE";
$resn_exch{" PO5 MDE"}=" O5P MDE"; $resn_exch{" PN4 MDE"}=" N4P MDE"; $resn_exch{" PC3 MDE"}=" C3P MDE";
$resn_exch{" PC2 MDE"}=" C2P MDE"; $resn_exch{" PS1 MDE"}=" S1P MDE"; $resn_exch{" C1* MDE"}=" C1' MDE";
$resn_exch{" O1* MDE"}=" O1' MDE"; $resn_exch{" C2* MDE"}=" C2' MDE"; $resn_exch{" C3* MDE"}=" C3' MDE";
$resn_exch{" C4* MDE"}=" C4' MDE"; $resn_exch{" C5* MDE"}=" C5' MDE"; $resn_exch{" C6* MDE"}=" C6' MDE";
$resn_exch{" C7* MDE"}=" C7' MDE"; $resn_exch{" C8* MDE"}=" C8' MDE"; $resn_exch{" C9* MDE"}=" C9' MDE";
$resn_exch{" O5* MDG"}=" O5' MDG"; $resn_exch{" C5* MDG"}=" C5' MDG"; $resn_exch{" C4* MDG"}=" C4' MDG";
$resn_exch{" O4* MDG"}=" O4' MDG"; $resn_exch{" C3* MDG"}=" C3' MDG"; $resn_exch{" O3* MDG"}=" O3' MDG";
$resn_exch{" C2* MDG"}=" C2' MDG"; $resn_exch{" O2* MDG"}=" O2' MDG"; $resn_exch{" C1* MDG"}=" C1' MDG";
$resn_exch{" O1P MEP"}=" OP1 MEP"; $resn_exch{" O2P MEP"}=" OP2 MEP"; $resn_exch{" O3P MEP"}=" OP3 MEP";
$resn_exch{" O5* MEP"}=" O5' MEP"; $resn_exch{" C5* MEP"}=" C5' MEP"; $resn_exch{" C4* MEP"}=" C4' MEP";
$resn_exch{" O4* MEP"}=" O4' MEP"; $resn_exch{" C3* MEP"}=" C3' MEP"; $resn_exch{" C2* MEP"}=" C2' MEP";
$resn_exch{" O2* MEP"}=" O2' MEP"; $resn_exch{" C1* MEP"}=" C1' MEP"; $resn_exch{" CA* MEP"}=" CA' MEP";
$resn_exch{" O3P MG1"}=" OP3 MG1"; $resn_exch{" O1P MG1"}=" OP1 MG1"; $resn_exch{" O2P MG1"}=" OP2 MG1";
$resn_exch{" O5* MG1"}=" O5' MG1"; $resn_exch{" C5* MG1"}=" C5' MG1"; $resn_exch{" C4* MG1"}=" C4' MG1";
$resn_exch{" O4* MG1"}=" O4' MG1"; $resn_exch{" C3* MG1"}=" C3' MG1"; $resn_exch{" O3* MG1"}=" O3' MG1";
$resn_exch{" C2* MG1"}=" C2' MG1"; $resn_exch{" C1* MG1"}=" C1' MG1"; $resn_exch{"C10A MGE"}=" CAA MGE";
$resn_exch{"C11A MGE"}=" CBA MGE"; $resn_exch{"C12A MGE"}=" CCA MGE"; $resn_exch{"C13A MGE"}=" CDA MGE";
$resn_exch{"C10B MGE"}=" CAB MGE"; $resn_exch{"C11B MGE"}=" CBB MGE"; $resn_exch{"C12B MGE"}=" CCB MGE";
$resn_exch{"C13B MGE"}=" CDB MGE"; $resn_exch{"C14B MGE"}=" CEB MGE"; $resn_exch{"C15B MGE"}=" CFB MGE";
$resn_exch{"C16B MGE"}=" CGB MGE"; $resn_exch{" O5* MGO"}=" O5' MGO"; $resn_exch{" C5* MGO"}=" C5' MGO";
$resn_exch{" C4* MGO"}=" C4' MGO"; $resn_exch{" O4* MGO"}=" O4' MGO"; $resn_exch{" C3* MGO"}=" C3' MGO";
$resn_exch{" O3* MGO"}=" O3' MGO"; $resn_exch{" C2* MGO"}=" C2' MGO"; $resn_exch{" O2* MGO"}=" O2' MGO";
$resn_exch{" C1* MGO"}=" C1' MGO"; $resn_exch{" O5* MGT"}=" O5' MGT"; $resn_exch{" C5* MGT"}=" C5' MGT";
$resn_exch{" C4* MGT"}=" C4' MGT"; $resn_exch{" O4* MGT"}=" O4' MGT"; $resn_exch{" C3* MGT"}=" C3' MGT";
$resn_exch{" O3* MGT"}=" O3' MGT"; $resn_exch{" C2* MGT"}=" C2' MGT"; $resn_exch{" O2* MGT"}=" O2' MGT";
$resn_exch{" C1* MGT"}=" C1' MGT"; $resn_exch{" N_A MHM"}=" NA  MHM"; $resn_exch{" N_B MHM"}=" NB  MHM";
$resn_exch{" N_C MHM"}=" NC  MHM"; $resn_exch{" N_D MHM"}=" ND  MHM"; $resn_exch{" C5* MHZ"}=" C5' MHZ";
$resn_exch{" C4* MHZ"}=" C4' MHZ"; $resn_exch{" O4* MHZ"}=" O4' MHZ"; $resn_exch{" C1* MHZ"}=" C1' MHZ";
$resn_exch{" C2* MHZ"}=" C2' MHZ"; $resn_exch{" O2* MHZ"}=" O2' MHZ"; $resn_exch{" C3* MHZ"}=" C3' MHZ";
$resn_exch{" O3* MHZ"}=" O3' MHZ"; $resn_exch{" O3P MIA"}=" OP3 MIA"; $resn_exch{" O1P MIA"}=" OP1 MIA";
$resn_exch{" O2P MIA"}=" OP2 MIA"; $resn_exch{" O5* MIA"}=" O5' MIA"; $resn_exch{" C5* MIA"}=" C5' MIA";
$resn_exch{" C4* MIA"}=" C4' MIA"; $resn_exch{" O4* MIA"}=" O4' MIA"; $resn_exch{" C3* MIA"}=" C3' MIA";
$resn_exch{" O3* MIA"}=" O3' MIA"; $resn_exch{" C2* MIA"}=" C2' MIA"; $resn_exch{" O2* MIA"}=" O2' MIA";
$resn_exch{" C1* MIA"}=" C1' MIA"; $resn_exch{"CO3* MMT"}=" C3X MMT"; $resn_exch{" O5* MMT"}=" O5' MMT";
$resn_exch{" C5* MMT"}=" C5' MMT"; $resn_exch{" C4* MMT"}=" C4' MMT"; $resn_exch{" O4* MMT"}=" O4' MMT";
$resn_exch{" C1* MMT"}=" C1' MMT"; $resn_exch{" C2* MMT"}=" C2' MMT"; $resn_exch{" C3* MMT"}=" C3' MMT";
$resn_exch{" O3* MMT"}=" O3' MMT"; $resn_exch{" C1* MNG"}=" C1' MNG"; $resn_exch{" C2* MNG"}=" C2' MNG";
$resn_exch{" C3* MNG"}=" C3' MNG"; $resn_exch{" C4* MNG"}=" C4' MNG"; $resn_exch{" C5* MNG"}=" C5' MNG";
$resn_exch{" O1* MNG"}=" O1' MNG"; $resn_exch{" O2* MNG"}=" O2' MNG"; $resn_exch{" O4* MNG"}=" O4' MNG";
$resn_exch{" N_A MNH"}=" NA  MNH"; $resn_exch{" N_B MNH"}=" NB  MNH"; $resn_exch{" N_C MNH"}=" NC  MNH";
$resn_exch{" N_D MNH"}=" ND  MNH"; $resn_exch{" N_A MNR"}=" NA  MNR"; $resn_exch{" N_B MNR"}=" NB  MNR";
$resn_exch{" N_C MNR"}=" NC  MNR"; $resn_exch{" N_D MNR"}=" ND  MNR"; $resn_exch{" O5* MNT"}=" O5' MNT";
$resn_exch{" C5* MNT"}=" C5B MNT"; $resn_exch{" C4* MNT"}=" C4B MNT"; $resn_exch{" O4* MNT"}=" O4' MNT";
$resn_exch{" C3* MNT"}=" C3B MNT"; $resn_exch{" O3* MNT"}=" O3' MNT"; $resn_exch{" C2* MNT"}=" C2B MNT";
$resn_exch{" C1* MNT"}=" C1B MNT"; $resn_exch{" O2P MNU"}=" OP2 MNU"; $resn_exch{" O1P MNU"}=" OP1 MNU";
$resn_exch{" O5* MNU"}=" O5' MNU"; $resn_exch{" C5* MNU"}=" C5' MNU"; $resn_exch{" C4* MNU"}=" C4' MNU";
$resn_exch{" O4* MNU"}=" O4' MNU"; $resn_exch{" C1* MNU"}=" C1' MNU"; $resn_exch{" C2* MNU"}=" C2' MNU";
$resn_exch{" O2* MNU"}=" O2' MNU"; $resn_exch{" C3* MNU"}=" C3' MNU"; $resn_exch{" O3* MNU"}=" O3' MNU";
$resn_exch{" O3P MNU"}=" OP3 MNU"; $resn_exch{"3CH3 MOB"}=" CHX MOB"; $resn_exch{"5CH3 MOB"}=" CHZ MOB";
$resn_exch{" O5* MOD"}=" O5' MOD"; $resn_exch{" C5* MOD"}=" C5' MOD"; $resn_exch{" C4* MOD"}=" C4' MOD";
$resn_exch{" O4* MOD"}=" O4' MOD"; $resn_exch{" C3* MOD"}=" C3' MOD"; $resn_exch{" O3* MOD"}=" O3' MOD";
$resn_exch{" C2* MOD"}=" C2' MOD"; $resn_exch{" O2* MOD"}=" O2' MOD"; $resn_exch{" C1* MOD"}=" C1' MOD";
$resn_exch{" O2* MOE"}=" O2' MOE"; $resn_exch{" CA* MOE"}=" CA' MOE"; $resn_exch{" CB* MOE"}=" CB' MOE";
$resn_exch{" OC* MOE"}=" OC' MOE"; $resn_exch{" CD* MOE"}=" CD' MOE"; $resn_exch{"3CL3 MON"}="CL33 MON";
$resn_exch{"1H1  MPA"}=" H11 MPA"; $resn_exch{"1H1  MPC"}=" H11 MPC"; $resn_exch{"1C12 MPG"}=" CXD MPG";
$resn_exch{"1C11 MPG"}=" CX3 MPG"; $resn_exch{"1CD1 MQQ"}=" CDB MQQ"; $resn_exch{"1CD2 MQQ"}=" CDC MQQ";
$resn_exch{" C1* MRC"}=" C1' MRC"; $resn_exch{" C2* MRC"}=" C2' MRC"; $resn_exch{" C3* MRC"}=" C3' MRC";
$resn_exch{" C4* MRC"}=" C4' MRC"; $resn_exch{" C5* MRC"}=" C5' MRC"; $resn_exch{" C6* MRC"}=" C6' MRC";
$resn_exch{" C7* MRC"}=" C7' MRC"; $resn_exch{" C8* MRC"}=" C8' MRC"; $resn_exch{" C9* MRC"}=" C9' MRC";
$resn_exch{" O1P MRG"}=" OP1 MRG"; $resn_exch{" O2P MRG"}=" OP2 MRG"; $resn_exch{" O5* MRG"}=" O5' MRG";
$resn_exch{" C2* MRG"}=" C2' MRG"; $resn_exch{" C5* MRG"}=" C5' MRG"; $resn_exch{" C4* MRG"}=" C4' MRG";
$resn_exch{" O4* MRG"}=" O4' MRG"; $resn_exch{" C1* MRG"}=" C1' MRG"; $resn_exch{" C3* MRG"}=" C3' MRG";
$resn_exch{" O3* MRG"}=" O3' MRG"; $resn_exch{" O3P MRG"}=" OP3 MRG"; $resn_exch{" AO7 MRR"}=" O7A MRR";
$resn_exch{"AP3* MRR"}=" P3B MRR"; $resn_exch{" AO8 MRR"}=" O8A MRR"; $resn_exch{" AO9 MRR"}=" O9A MRR";
$resn_exch{"AO3* MRR"}=" O3B MRR"; $resn_exch{"AC3* MRR"}=" C3B MRR"; $resn_exch{"AC2* MRR"}=" C2B MRR";
$resn_exch{"AO2* MRR"}=" O2B MRR"; $resn_exch{"AC1* MRR"}=" C1B MRR"; $resn_exch{" AN9 MRR"}=" N9A MRR";
$resn_exch{" AC4 MRR"}=" C4A MRR"; $resn_exch{" AN3 MRR"}=" N3A MRR"; $resn_exch{" AC2 MRR"}=" C2A MRR";
$resn_exch{" AN1 MRR"}=" N1A MRR"; $resn_exch{" AC6 MRR"}=" C6A MRR"; $resn_exch{" AN6 MRR"}=" N6A MRR";
$resn_exch{" AC5 MRR"}=" C5A MRR"; $resn_exch{" AN7 MRR"}=" N7A MRR"; $resn_exch{" AC8 MRR"}=" C8A MRR";
$resn_exch{"AO4* MRR"}=" O4B MRR"; $resn_exch{"AC4* MRR"}=" C4B MRR"; $resn_exch{"AC5* MRR"}=" C5B MRR";
$resn_exch{"AO5* MRR"}=" O5B MRR"; $resn_exch{" AP1 MRR"}=" P1A MRR"; $resn_exch{" AO1 MRR"}=" O1A MRR";
$resn_exch{" AO2 MRR"}=" O2A MRR"; $resn_exch{" AO3 MRR"}=" O3A MRR"; $resn_exch{" AP2 MRR"}=" P2A MRR";
$resn_exch{" AO4 MRR"}=" O4A MRR"; $resn_exch{" AO5 MRR"}=" O5A MRR"; $resn_exch{" AO6 MRR"}=" O6A MRR";
$resn_exch{"PC12 MRR"}=" CCP MRR"; $resn_exch{"PC11 MRR"}=" CBP MRR"; $resn_exch{"PC13 MRR"}=" CDP MRR";
$resn_exch{"PC14 MRR"}=" CEP MRR"; $resn_exch{"PC10 MRR"}=" CAP MRR"; $resn_exch{"PO10 MRR"}=" OAP MRR";
$resn_exch{" PC9 MRR"}=" C9P MRR"; $resn_exch{" PO9 MRR"}=" O9P MRR"; $resn_exch{" PN8 MRR"}=" N8P MRR";
$resn_exch{" PC7 MRR"}=" C7P MRR"; $resn_exch{" PC6 MRR"}=" C6P MRR"; $resn_exch{" PC5 MRR"}=" C5P MRR";
$resn_exch{" PO5 MRR"}=" O5P MRR"; $resn_exch{" PN4 MRR"}=" N4P MRR"; $resn_exch{" PC3 MRR"}=" C3P MRR";
$resn_exch{" PC2 MRR"}=" C2P MRR"; $resn_exch{" PS1 MRR"}=" S1P MRR"; $resn_exch{" AO7 MRS"}=" O7A MRS";
$resn_exch{"AP3* MRS"}=" P3B MRS"; $resn_exch{" AO8 MRS"}=" O8A MRS"; $resn_exch{" AO9 MRS"}=" O9A MRS";
$resn_exch{"AO3* MRS"}=" O3B MRS"; $resn_exch{"AC3* MRS"}=" C3B MRS"; $resn_exch{"AC2* MRS"}=" C2B MRS";
$resn_exch{"AO2* MRS"}=" O2B MRS"; $resn_exch{"AC1* MRS"}=" C1B MRS"; $resn_exch{" AN9 MRS"}=" N9A MRS";
$resn_exch{" AC4 MRS"}=" C4A MRS"; $resn_exch{" AN3 MRS"}=" N3A MRS"; $resn_exch{" AC2 MRS"}=" C2A MRS";
$resn_exch{" AN1 MRS"}=" N1A MRS"; $resn_exch{" AC6 MRS"}=" C6A MRS"; $resn_exch{" AN6 MRS"}=" N6A MRS";
$resn_exch{" AC5 MRS"}=" C5A MRS"; $resn_exch{" AN7 MRS"}=" N7A MRS"; $resn_exch{" AC8 MRS"}=" C8A MRS";
$resn_exch{"AO4* MRS"}=" O4B MRS"; $resn_exch{"AC4* MRS"}=" C4B MRS"; $resn_exch{"AC5* MRS"}=" C5B MRS";
$resn_exch{"AO5* MRS"}=" O5B MRS"; $resn_exch{" AP1 MRS"}=" P1A MRS"; $resn_exch{" AO1 MRS"}=" O1A MRS";
$resn_exch{" AO2 MRS"}=" O2A MRS"; $resn_exch{" AO3 MRS"}=" O3A MRS"; $resn_exch{" AP2 MRS"}=" P2A MRS";
$resn_exch{" AO4 MRS"}=" O4A MRS"; $resn_exch{" AO5 MRS"}=" O5A MRS"; $resn_exch{" AO6 MRS"}=" O6A MRS";
$resn_exch{"PC12 MRS"}=" CCP MRS"; $resn_exch{"PC11 MRS"}=" CBP MRS"; $resn_exch{"PC13 MRS"}=" CDP MRS";
$resn_exch{"PC14 MRS"}=" CEP MRS"; $resn_exch{"PC10 MRS"}=" CAP MRS"; $resn_exch{"PO10 MRS"}=" OAP MRS";
$resn_exch{" PC9 MRS"}=" C9P MRS"; $resn_exch{" PO9 MRS"}=" O9P MRS"; $resn_exch{" PN8 MRS"}=" N8P MRS";
$resn_exch{" PC7 MRS"}=" C7P MRS"; $resn_exch{" PC6 MRS"}=" C6P MRS"; $resn_exch{" PC5 MRS"}=" C5P MRS";
$resn_exch{" PO5 MRS"}=" O5P MRS"; $resn_exch{" PN4 MRS"}=" N4P MRS"; $resn_exch{" PC3 MRS"}=" C3P MRS";
$resn_exch{" PC2 MRS"}=" C2P MRS"; $resn_exch{" PS1 MRS"}=" S1P MRS"; $resn_exch{" O5* MSG"}=" O5' MSG";
$resn_exch{" C5* MSG"}=" C5' MSG"; $resn_exch{" C4* MSG"}=" C4' MSG"; $resn_exch{" O4* MSG"}=" O4' MSG";
$resn_exch{" C1* MSG"}=" C1' MSG"; $resn_exch{" C2* MSG"}=" C2' MSG"; $resn_exch{" O2* MSG"}=" O2' MSG";
$resn_exch{" C3* MSG"}=" C3' MSG"; $resn_exch{" O3* MSG"}=" O3' MSG"; $resn_exch{" O5* MSP"}=" O5' MSP";
$resn_exch{" C5* MSP"}=" C5' MSP"; $resn_exch{" C4* MSP"}=" C4' MSP"; $resn_exch{" O4* MSP"}=" O4' MSP";
$resn_exch{" C3* MSP"}=" C3' MSP"; $resn_exch{" O3* MSP"}=" O3' MSP"; $resn_exch{" C2* MSP"}=" C2' MSP";
$resn_exch{" O2* MSP"}=" O2' MSP"; $resn_exch{" C1* MSP"}=" C1' MSP"; $resn_exch{" S5* MTA"}=" S5' MTA";
$resn_exch{" C5* MTA"}=" C5' MTA"; $resn_exch{" C4* MTA"}=" C4' MTA"; $resn_exch{" O4* MTA"}=" O4' MTA";
$resn_exch{" C2* MTA"}=" C2' MTA"; $resn_exch{" O2* MTA"}=" O2' MTA"; $resn_exch{" C3* MTA"}=" C3' MTA";
$resn_exch{" O3* MTA"}=" O3' MTA"; $resn_exch{" C1* MTA"}=" C1' MTA"; $resn_exch{"1CH3 MTB"}=" CHV MTB";
$resn_exch{"2CH3 MTB"}=" CHW MTB"; $resn_exch{"3CH3 MTB"}=" CHX MTB"; $resn_exch{" S5* MTH"}=" S5' MTH";
$resn_exch{" C5* MTH"}=" C5' MTH"; $resn_exch{" C4* MTH"}=" C4' MTH"; $resn_exch{" O4* MTH"}=" O4' MTH";
$resn_exch{" C2* MTH"}=" C2' MTH"; $resn_exch{" O2* MTH"}=" O2' MTH"; $resn_exch{" C3* MTH"}=" C3' MTH";
$resn_exch{" O3* MTH"}=" O3' MTH"; $resn_exch{" C1* MTH"}=" C1' MTH"; $resn_exch{" S5* MTI"}=" S5' MTI";
$resn_exch{" C5* MTI"}=" C5' MTI"; $resn_exch{" C4* MTI"}=" C4' MTI"; $resn_exch{" N4* MTI"}=" N4' MTI";
$resn_exch{" C2* MTI"}=" C2' MTI"; $resn_exch{" O2* MTI"}=" O2' MTI"; $resn_exch{" C3* MTI"}=" C3' MTI";
$resn_exch{" O3* MTI"}=" O3' MTI"; $resn_exch{" C1* MTI"}=" C1' MTI"; $resn_exch{" S5* MTM"}=" S5' MTM";
$resn_exch{" C5* MTM"}=" C5' MTM"; $resn_exch{" C4* MTM"}=" C4' MTM"; $resn_exch{" N4* MTM"}=" N4' MTM";
$resn_exch{" C2* MTM"}=" C2' MTM"; $resn_exch{" O2* MTM"}=" O2' MTM"; $resn_exch{" C3* MTM"}=" C3' MTM";
$resn_exch{" O3* MTM"}=" O3' MTM"; $resn_exch{" C1* MTM"}=" C1' MTM"; $resn_exch{" O1P MTR"}=" OP1 MTR";
$resn_exch{" O2P MTR"}=" OP2 MTR"; $resn_exch{" O5* MTR"}=" O5' MTR"; $resn_exch{" C2* MTR"}=" C2' MTR";
$resn_exch{" C5* MTR"}=" C5' MTR"; $resn_exch{" C4* MTR"}=" C4' MTR"; $resn_exch{" O4* MTR"}=" O4' MTR";
$resn_exch{" C1* MTR"}=" C1' MTR"; $resn_exch{" C3* MTR"}=" C3' MTR"; $resn_exch{" O3* MTR"}=" O3' MTR";
$resn_exch{" O3P MTR"}=" OP3 MTR"; $resn_exch{" O1P MTU"}=" OP1 MTU"; $resn_exch{" O2P MTU"}=" OP2 MTU";
$resn_exch{" O5* MTU"}=" O5' MTU"; $resn_exch{" C5* MTU"}=" C5' MTU"; $resn_exch{" C4* MTU"}=" C4' MTU";
$resn_exch{" O4* MTU"}=" O4' MTU"; $resn_exch{" C1* MTU"}=" C1' MTU"; $resn_exch{" C2* MTU"}=" C2' MTU";
$resn_exch{" O2* MTU"}=" O2' MTU"; $resn_exch{" C3* MTU"}=" C3' MTU"; $resn_exch{" O3* MTU"}=" O3' MTU";
$resn_exch{" O5* MZP"}=" O5' MZP"; $resn_exch{" C5* MZP"}=" C5' MZP"; $resn_exch{" C4* MZP"}=" C4' MZP";
$resn_exch{" O4* MZP"}=" O4' MZP"; $resn_exch{" C3* MZP"}=" C3' MZP"; $resn_exch{" O3* MZP"}=" O3' MZP";
$resn_exch{" C2* MZP"}=" C2' MZP"; $resn_exch{" O2* MZP"}=" O2' MZP"; $resn_exch{" C1* MZP"}=" C1' MZP";
$resn_exch{" O3P   N"}=" OP3   N"; $resn_exch{" O1P   N"}=" OP1   N"; $resn_exch{" O2P   N"}=" OP2   N";
$resn_exch{" O5*   N"}=" O5'   N"; $resn_exch{" C5*   N"}=" C5'   N"; $resn_exch{" C4*   N"}=" C4'   N";
$resn_exch{" O4*   N"}=" O4'   N"; $resn_exch{" C3*   N"}=" C3'   N"; $resn_exch{" O3*   N"}=" O3'   N";
$resn_exch{" C2*   N"}=" C2'   N"; $resn_exch{" O2*   N"}=" O2'   N"; $resn_exch{" C1*   N"}=" C1'   N";
$resn_exch{" C1* N1C"}=" C1' N1C"; $resn_exch{" O4* N1C"}=" O4' N1C"; $resn_exch{" C2* N1C"}=" C2' N1C";
$resn_exch{" O2* N1C"}=" O2' N1C"; $resn_exch{" C3* N1C"}=" C3' N1C"; $resn_exch{" O3* N1C"}=" O3' N1C";
$resn_exch{" C4* N1C"}=" C4' N1C"; $resn_exch{" C5* N1C"}=" C5' N1C"; $resn_exch{" O5* N1C"}=" O5' N1C";
$resn_exch{"C31' N1H"}=" CV' N1H"; $resn_exch{"C32' N1H"}=" CW' N1H"; $resn_exch{"O33' N1H"}=" OX' N1H";
$resn_exch{"O34' N1H"}=" OY' N1H"; $resn_exch{"C1'' N1H"}=" C1B N1H"; $resn_exch{"C2'' N1H"}=" C2B N1H";
$resn_exch{"C3'' N1H"}=" C3B N1H"; $resn_exch{"C4'' N1H"}=" C4B N1H"; $resn_exch{"C5'' N1H"}=" C5B N1H";
$resn_exch{"N5'' N1H"}=" N5B N1H"; $resn_exch{"C6'' N1H"}=" C6B N1H"; $resn_exch{" N5* N5A"}=" N5' N5A";
$resn_exch{" C5* N5A"}=" C5' N5A"; $resn_exch{" O5* N5A"}=" O5' N5A"; $resn_exch{" C4* N5A"}=" C4' N5A";
$resn_exch{" O4* N5A"}=" O4' N5A"; $resn_exch{" C3* N5A"}=" C3' N5A"; $resn_exch{" O3* N5A"}=" O3' N5A";
$resn_exch{" C2* N5A"}=" C2' N5A"; $resn_exch{" O2* N5A"}=" O2' N5A"; $resn_exch{" C1* N5A"}=" C1' N5A";
$resn_exch{" C1* N5I"}=" C1' N5I"; $resn_exch{" O4* N5I"}=" O4' N5I"; $resn_exch{" C4* N5I"}=" C4' N5I";
$resn_exch{" C3* N5I"}=" C3' N5I"; $resn_exch{" O3* N5I"}=" O3' N5I"; $resn_exch{" C2* N5I"}=" C2' N5I";
$resn_exch{" C5* N5I"}=" C5' N5I"; $resn_exch{" O5* N5I"}=" O5' N5I"; $resn_exch{" N5* N5O"}=" N5' N5O";
$resn_exch{" C5* N5O"}=" C5' N5O"; $resn_exch{" O5* N5O"}=" O5' N5O"; $resn_exch{" C4* N5O"}=" C4' N5O";
$resn_exch{" O4* N5O"}=" O4' N5O"; $resn_exch{" C3* N5O"}=" C3' N5O"; $resn_exch{" O3* N5O"}=" O3' N5O";
$resn_exch{" C2* N5O"}=" C2' N5O"; $resn_exch{" O2* N5O"}=" O2' N5O"; $resn_exch{" C1* N5O"}=" C1' N5O";
$resn_exch{" C1* N5P"}=" C1' N5P"; $resn_exch{" O4* N5P"}=" O4' N5P"; $resn_exch{" C4* N5P"}=" C4' N5P";
$resn_exch{" C3* N5P"}=" C3' N5P"; $resn_exch{" O3* N5P"}=" O3' N5P"; $resn_exch{" C2* N5P"}=" C2' N5P";
$resn_exch{" C5* N5P"}=" C5' N5P"; $resn_exch{" O5* N5P"}=" O5' N5P"; $resn_exch{" O1P N6G"}=" OP1 N6G";
$resn_exch{" O2P N6G"}=" OP2 N6G"; $resn_exch{" O5* N6G"}=" O5' N6G"; $resn_exch{" C5* N6G"}=" C5' N6G";
$resn_exch{" C4* N6G"}=" C4' N6G"; $resn_exch{" O4* N6G"}=" O4' N6G"; $resn_exch{" C1* N6G"}=" C1' N6G";
$resn_exch{" C2* N6G"}=" C2' N6G"; $resn_exch{" O2* N6G"}=" O2' N6G"; $resn_exch{" C3* N6G"}=" C3' N6G";
$resn_exch{" O3* N6G"}=" O3' N6G"; $resn_exch{" O3P N6G"}=" OP3 N6G"; $resn_exch{" AP  NA7"}=" PA  NA7";
$resn_exch{" AO1 NA7"}=" O1A NA7"; $resn_exch{" AO2 NA7"}=" O2A NA7"; $resn_exch{"AO5* NA7"}=" O5B NA7";
$resn_exch{"AC5* NA7"}=" C5B NA7"; $resn_exch{"AC4* NA7"}=" C4B NA7"; $resn_exch{"AO4* NA7"}=" O4B NA7";
$resn_exch{"AC3* NA7"}=" C3B NA7"; $resn_exch{"AO3* NA7"}=" O3B NA7"; $resn_exch{"AC2* NA7"}=" C2B NA7";
$resn_exch{"AO2* NA7"}=" O2B NA7"; $resn_exch{"AC1* NA7"}=" C1B NA7"; $resn_exch{" AN9 NA7"}=" N9A NA7";
$resn_exch{" AC8 NA7"}=" C8A NA7"; $resn_exch{" AN7 NA7"}=" N7A NA7"; $resn_exch{" AC5 NA7"}=" C5A NA7";
$resn_exch{" AC6 NA7"}=" C6A NA7"; $resn_exch{" AN6 NA7"}=" N6A NA7"; $resn_exch{" AN1 NA7"}=" N1A NA7";
$resn_exch{" AC2 NA7"}=" C2A NA7"; $resn_exch{" AN3 NA7"}=" N3A NA7"; $resn_exch{" AC4 NA7"}=" C4A NA7";
$resn_exch{" NP  NA7"}=" PN  NA7"; $resn_exch{" NO1 NA7"}=" O1N NA7"; $resn_exch{" NO2 NA7"}=" O2N NA7";
$resn_exch{"NO5* NA7"}=" O5D NA7"; $resn_exch{"NC5* NA7"}=" C5D NA7"; $resn_exch{"NC4* NA7"}=" C4D NA7";
$resn_exch{"NO4* NA7"}=" O4D NA7"; $resn_exch{"NC3* NA7"}=" C3D NA7"; $resn_exch{"NO3* NA7"}=" O3D NA7";
$resn_exch{"NC2* NA7"}=" C2D NA7"; $resn_exch{"NO2* NA7"}=" O2D NA7"; $resn_exch{"NC1* NA7"}=" C1D NA7";
$resn_exch{"AP2* NA7"}=" P2B NA7"; $resn_exch{"AOP1 NA7"}=" O1X NA7"; $resn_exch{"AOP2 NA7"}=" O2X NA7";
$resn_exch{"AOP3 NA7"}=" O3X NA7"; $resn_exch{" AP  NAC"}=" PA  NAC"; $resn_exch{" AO1 NAC"}=" O1A NAC";
$resn_exch{" AO2 NAC"}=" O2A NAC"; $resn_exch{"AO5* NAC"}=" O5B NAC"; $resn_exch{"AC5* NAC"}=" C5B NAC";
$resn_exch{"AC4* NAC"}=" C4B NAC"; $resn_exch{"AO4* NAC"}=" O4B NAC"; $resn_exch{"AC3* NAC"}=" C3B NAC";
$resn_exch{"AO3* NAC"}=" O3B NAC"; $resn_exch{"AC2* NAC"}=" C2B NAC"; $resn_exch{"AO2* NAC"}=" O2B NAC";
$resn_exch{"AC1* NAC"}=" C1B NAC"; $resn_exch{" AN9 NAC"}=" N9A NAC"; $resn_exch{" AC8 NAC"}=" C8A NAC";
$resn_exch{" AN7 NAC"}=" N7A NAC"; $resn_exch{" AC5 NAC"}=" C5A NAC"; $resn_exch{" AC6 NAC"}=" C6A NAC";
$resn_exch{" AN6 NAC"}=" N6A NAC"; $resn_exch{" AN1 NAC"}=" N1A NAC"; $resn_exch{" AC2 NAC"}=" C2A NAC";
$resn_exch{" AN3 NAC"}=" N3A NAC"; $resn_exch{" AC4 NAC"}=" C4A NAC"; $resn_exch{" NP  NAC"}=" PN  NAC";
$resn_exch{" NO1 NAC"}=" O1N NAC"; $resn_exch{" NO2 NAC"}=" O2N NAC"; $resn_exch{"NO5* NAC"}=" O5D NAC";
$resn_exch{"NC5* NAC"}=" C5D NAC"; $resn_exch{"NC4* NAC"}=" C4D NAC"; $resn_exch{"NO4* NAC"}=" O4D NAC";
$resn_exch{"NC3* NAC"}=" C3D NAC"; $resn_exch{"NO3* NAC"}=" O3D NAC"; $resn_exch{"NC2* NAC"}=" C2D NAC";
$resn_exch{"NO2* NAC"}=" O2D NAC"; $resn_exch{"NC1* NAC"}=" C1D NAC"; $resn_exch{" NN1 NAC"}=" N1N NAC";
$resn_exch{" NC2 NAC"}=" C2N NAC"; $resn_exch{" NC3 NAC"}=" C3N NAC"; $resn_exch{" NC7 NAC"}=" C7N NAC";
$resn_exch{" NO7 NAC"}=" O7N NAC"; $resn_exch{" NC8 NAC"}=" C8N NAC"; $resn_exch{" NC4 NAC"}=" C4N NAC";
$resn_exch{" NC5 NAC"}=" C5N NAC"; $resn_exch{" NC6 NAC"}=" C6N NAC"; $resn_exch{" AP  NAD"}=" PA  NAD";
$resn_exch{" AO1 NAD"}=" O1A NAD"; $resn_exch{" AO2 NAD"}=" O2A NAD"; $resn_exch{"AO5* NAD"}=" O5B NAD";
$resn_exch{"AC5* NAD"}=" C5B NAD"; $resn_exch{"AC4* NAD"}=" C4B NAD"; $resn_exch{"AO4* NAD"}=" O4B NAD";
$resn_exch{"AC3* NAD"}=" C3B NAD"; $resn_exch{"AO3* NAD"}=" O3B NAD"; $resn_exch{"AC2* NAD"}=" C2B NAD";
$resn_exch{"AO2* NAD"}=" O2B NAD"; $resn_exch{"AC1* NAD"}=" C1B NAD"; $resn_exch{" AN9 NAD"}=" N9A NAD";
$resn_exch{" AC8 NAD"}=" C8A NAD"; $resn_exch{" AN7 NAD"}=" N7A NAD"; $resn_exch{" AC5 NAD"}=" C5A NAD";
$resn_exch{" AC6 NAD"}=" C6A NAD"; $resn_exch{" AN6 NAD"}=" N6A NAD"; $resn_exch{" AN1 NAD"}=" N1A NAD";
$resn_exch{" AC2 NAD"}=" C2A NAD"; $resn_exch{" AN3 NAD"}=" N3A NAD"; $resn_exch{" AC4 NAD"}=" C4A NAD";
$resn_exch{" NP  NAD"}=" PN  NAD"; $resn_exch{" NO1 NAD"}=" O1N NAD"; $resn_exch{" NO2 NAD"}=" O2N NAD";
$resn_exch{"NO5* NAD"}=" O5D NAD"; $resn_exch{"NC5* NAD"}=" C5D NAD"; $resn_exch{"NC4* NAD"}=" C4D NAD";
$resn_exch{"NO4* NAD"}=" O4D NAD"; $resn_exch{"NC3* NAD"}=" C3D NAD"; $resn_exch{"NO3* NAD"}=" O3D NAD";
$resn_exch{"NC2* NAD"}=" C2D NAD"; $resn_exch{"NO2* NAD"}=" O2D NAD"; $resn_exch{"NC1* NAD"}=" C1D NAD";
$resn_exch{" NN1 NAD"}=" N1N NAD"; $resn_exch{" NC2 NAD"}=" C2N NAD"; $resn_exch{" NC3 NAD"}=" C3N NAD";
$resn_exch{" NC7 NAD"}=" C7N NAD"; $resn_exch{" NO7 NAD"}=" O7N NAD"; $resn_exch{" NN7 NAD"}=" N7N NAD";
$resn_exch{" NC4 NAD"}=" C4N NAD"; $resn_exch{" NC5 NAD"}=" C5N NAD"; $resn_exch{" NC6 NAD"}=" C6N NAD";
$resn_exch{" O5* NAE"}=" O5B NAE"; $resn_exch{" C5* NAE"}=" C5B NAE"; $resn_exch{" C4* NAE"}=" C4B NAE";
$resn_exch{" O4* NAE"}=" O4B NAE"; $resn_exch{" C3* NAE"}=" C3B NAE"; $resn_exch{" O3* NAE"}=" O3B NAE";
$resn_exch{" C2* NAE"}=" C2B NAE"; $resn_exch{" O2* NAE"}=" O2B NAE"; $resn_exch{" C1* NAE"}=" C1B NAE";
$resn_exch{" AP  NAH"}=" PA  NAH"; $resn_exch{" AO1 NAH"}=" O1A NAH"; $resn_exch{" AO2 NAH"}=" O2A NAH";
$resn_exch{"AO5* NAH"}=" O5B NAH"; $resn_exch{"AC5* NAH"}=" C5B NAH"; $resn_exch{"AC4* NAH"}=" C4B NAH";
$resn_exch{"AO4* NAH"}=" O4B NAH"; $resn_exch{"AC3* NAH"}=" C3B NAH"; $resn_exch{"AO3* NAH"}=" O3B NAH";
$resn_exch{"AC2* NAH"}=" C2B NAH"; $resn_exch{"AO2* NAH"}=" O2B NAH"; $resn_exch{"AC1* NAH"}=" C1B NAH";
$resn_exch{" AN9 NAH"}=" N9A NAH"; $resn_exch{" AC8 NAH"}=" C8A NAH"; $resn_exch{" AN7 NAH"}=" N7A NAH";
$resn_exch{" AC5 NAH"}=" C5A NAH"; $resn_exch{" AC6 NAH"}=" C6A NAH"; $resn_exch{" AN6 NAH"}=" N6A NAH";
$resn_exch{" AN1 NAH"}=" N1A NAH"; $resn_exch{" AC2 NAH"}=" C2A NAH"; $resn_exch{" AN3 NAH"}=" N3A NAH";
$resn_exch{" AC4 NAH"}=" C4A NAH"; $resn_exch{" NP  NAH"}=" PN  NAH"; $resn_exch{" NO1 NAH"}=" O1N NAH";
$resn_exch{" NO2 NAH"}=" O2N NAH"; $resn_exch{"NO5* NAH"}=" O5D NAH"; $resn_exch{"NC5* NAH"}=" C5D NAH";
$resn_exch{"NC4* NAH"}=" C4D NAH"; $resn_exch{"NO4* NAH"}=" O4D NAH"; $resn_exch{"NC3* NAH"}=" C3D NAH";
$resn_exch{"NO3* NAH"}=" O3D NAH"; $resn_exch{"NC2* NAH"}=" C2D NAH"; $resn_exch{"NO2* NAH"}=" O2D NAH";
$resn_exch{"NC1* NAH"}=" C1D NAH"; $resn_exch{" NN1 NAH"}=" N1N NAH"; $resn_exch{" NC2 NAH"}=" C2N NAH";
$resn_exch{" NC3 NAH"}=" C3N NAH"; $resn_exch{" NC7 NAH"}=" C7N NAH"; $resn_exch{" NO7 NAH"}=" O7N NAH";
$resn_exch{" NN7 NAH"}=" N7N NAH"; $resn_exch{" NC4 NAH"}=" C4N NAH"; $resn_exch{" NC5 NAH"}=" C5N NAH";
$resn_exch{" NC6 NAH"}=" C6N NAH"; $resn_exch{" AP  NAI"}=" PA  NAI"; $resn_exch{" AO1 NAI"}=" O1A NAI";
$resn_exch{" AO2 NAI"}=" O2A NAI"; $resn_exch{"AO5* NAI"}=" O5B NAI"; $resn_exch{"AC5* NAI"}=" C5B NAI";
$resn_exch{"AC4* NAI"}=" C4B NAI"; $resn_exch{"AO4* NAI"}=" O4B NAI"; $resn_exch{"AC3* NAI"}=" C3B NAI";
$resn_exch{"AO3* NAI"}=" O3B NAI"; $resn_exch{"AC2* NAI"}=" C2B NAI"; $resn_exch{"AO2* NAI"}=" O2B NAI";
$resn_exch{"AC1* NAI"}=" C1B NAI"; $resn_exch{" AN9 NAI"}=" N9A NAI"; $resn_exch{" AC8 NAI"}=" C8A NAI";
$resn_exch{" AN7 NAI"}=" N7A NAI"; $resn_exch{" AC5 NAI"}=" C5A NAI"; $resn_exch{" AC6 NAI"}=" C6A NAI";
$resn_exch{" AN6 NAI"}=" N6A NAI"; $resn_exch{" AN1 NAI"}=" N1A NAI"; $resn_exch{" AC2 NAI"}=" C2A NAI";
$resn_exch{" AN3 NAI"}=" N3A NAI"; $resn_exch{" AC4 NAI"}=" C4A NAI"; $resn_exch{" NP  NAI"}=" PN  NAI";
$resn_exch{" NO1 NAI"}=" O1N NAI"; $resn_exch{" NO2 NAI"}=" O2N NAI"; $resn_exch{"NO5* NAI"}=" O5D NAI";
$resn_exch{"NC5* NAI"}=" C5D NAI"; $resn_exch{"NC4* NAI"}=" C4D NAI"; $resn_exch{"NO4* NAI"}=" O4D NAI";
$resn_exch{"NC3* NAI"}=" C3D NAI"; $resn_exch{"NO3* NAI"}=" O3D NAI"; $resn_exch{"NC2* NAI"}=" C2D NAI";
$resn_exch{"NO2* NAI"}=" O2D NAI"; $resn_exch{"NC1* NAI"}=" C1D NAI"; $resn_exch{" NN1 NAI"}=" N1N NAI";
$resn_exch{" NC2 NAI"}=" C2N NAI"; $resn_exch{" NC3 NAI"}=" C3N NAI"; $resn_exch{" NC7 NAI"}=" C7N NAI";
$resn_exch{" NO7 NAI"}=" O7N NAI"; $resn_exch{" NN7 NAI"}=" N7N NAI"; $resn_exch{" NC4 NAI"}=" C4N NAI";
$resn_exch{" NC5 NAI"}=" C5N NAI"; $resn_exch{" NC6 NAI"}=" C6N NAI"; $resn_exch{" AP  NAJ"}=" PA  NAJ";
$resn_exch{" AO1 NAJ"}=" O1A NAJ"; $resn_exch{" AO2 NAJ"}=" O2A NAJ"; $resn_exch{"AO5* NAJ"}=" O5B NAJ";
$resn_exch{"AC5* NAJ"}=" C5B NAJ"; $resn_exch{"AC4* NAJ"}=" C4B NAJ"; $resn_exch{"AO4* NAJ"}=" O4B NAJ";
$resn_exch{"AC3* NAJ"}=" C3B NAJ"; $resn_exch{"AO3* NAJ"}=" O3B NAJ"; $resn_exch{"AC2* NAJ"}=" C2B NAJ";
$resn_exch{"AO2* NAJ"}=" O2B NAJ"; $resn_exch{"AC1* NAJ"}=" C1B NAJ"; $resn_exch{" AN9 NAJ"}=" N9A NAJ";
$resn_exch{" AC8 NAJ"}=" C8A NAJ"; $resn_exch{" AN7 NAJ"}=" N7A NAJ"; $resn_exch{" AC5 NAJ"}=" C5A NAJ";
$resn_exch{" AC6 NAJ"}=" C6A NAJ"; $resn_exch{" AN6 NAJ"}=" N6A NAJ"; $resn_exch{" AN1 NAJ"}=" N1A NAJ";
$resn_exch{" AC2 NAJ"}=" C2A NAJ"; $resn_exch{" AN3 NAJ"}=" N3A NAJ"; $resn_exch{" AC4 NAJ"}=" C4A NAJ";
$resn_exch{" NP  NAJ"}=" PN  NAJ"; $resn_exch{" NO1 NAJ"}=" O1N NAJ"; $resn_exch{" NO2 NAJ"}=" O2N NAJ";
$resn_exch{"NO5* NAJ"}=" O5D NAJ"; $resn_exch{"NC5* NAJ"}=" C5D NAJ"; $resn_exch{"NC4* NAJ"}=" C4D NAJ";
$resn_exch{"NO4* NAJ"}=" O4D NAJ"; $resn_exch{"NC3* NAJ"}=" C3D NAJ"; $resn_exch{"NO3* NAJ"}=" O3D NAJ";
$resn_exch{"NC2* NAJ"}=" C2D NAJ"; $resn_exch{"NO2* NAJ"}=" O2D NAJ"; $resn_exch{"NC1* NAJ"}=" C1D NAJ";
$resn_exch{" NN1 NAJ"}=" N1N NAJ"; $resn_exch{" NC2 NAJ"}=" C2N NAJ"; $resn_exch{" NC3 NAJ"}=" C3N NAJ";
$resn_exch{" NC7 NAJ"}=" C7N NAJ"; $resn_exch{" NO7 NAJ"}=" O7N NAJ"; $resn_exch{" NN7 NAJ"}=" N7N NAJ";
$resn_exch{" NC4 NAJ"}=" C4N NAJ"; $resn_exch{" NC5 NAJ"}=" C5N NAJ"; $resn_exch{" NC6 NAJ"}=" C6N NAJ";
$resn_exch{" AP  NAP"}=" PA  NAP"; $resn_exch{" AO1 NAP"}=" O1A NAP"; $resn_exch{" AO2 NAP"}=" O2A NAP";
$resn_exch{"AO5* NAP"}=" O5B NAP"; $resn_exch{"AC5* NAP"}=" C5B NAP"; $resn_exch{"AC4* NAP"}=" C4B NAP";
$resn_exch{"AO4* NAP"}=" O4B NAP"; $resn_exch{"AC3* NAP"}=" C3B NAP"; $resn_exch{"AO3* NAP"}=" O3B NAP";
$resn_exch{"AC2* NAP"}=" C2B NAP"; $resn_exch{"AO2* NAP"}=" O2B NAP"; $resn_exch{"AC1* NAP"}=" C1B NAP";
$resn_exch{" AN9 NAP"}=" N9A NAP"; $resn_exch{" AC8 NAP"}=" C8A NAP"; $resn_exch{" AN7 NAP"}=" N7A NAP";
$resn_exch{" AC5 NAP"}=" C5A NAP"; $resn_exch{" AC6 NAP"}=" C6A NAP"; $resn_exch{" AN6 NAP"}=" N6A NAP";
$resn_exch{" AN1 NAP"}=" N1A NAP"; $resn_exch{" AC2 NAP"}=" C2A NAP"; $resn_exch{" AN3 NAP"}=" N3A NAP";
$resn_exch{" AC4 NAP"}=" C4A NAP"; $resn_exch{" NP  NAP"}=" PN  NAP"; $resn_exch{" NO1 NAP"}=" O1N NAP";
$resn_exch{" NO2 NAP"}=" O2N NAP"; $resn_exch{"NO5* NAP"}=" O5D NAP"; $resn_exch{"NC5* NAP"}=" C5D NAP";
$resn_exch{"NC4* NAP"}=" C4D NAP"; $resn_exch{"NO4* NAP"}=" O4D NAP"; $resn_exch{"NC3* NAP"}=" C3D NAP";
$resn_exch{"NO3* NAP"}=" O3D NAP"; $resn_exch{"NC2* NAP"}=" C2D NAP"; $resn_exch{"NO2* NAP"}=" O2D NAP";
$resn_exch{"NC1* NAP"}=" C1D NAP"; $resn_exch{" NN1 NAP"}=" N1N NAP"; $resn_exch{" NC2 NAP"}=" C2N NAP";
$resn_exch{" NC3 NAP"}=" C3N NAP"; $resn_exch{" NC7 NAP"}=" C7N NAP"; $resn_exch{" NO7 NAP"}=" O7N NAP";
$resn_exch{" NN7 NAP"}=" N7N NAP"; $resn_exch{" NC4 NAP"}=" C4N NAP"; $resn_exch{" NC5 NAP"}=" C5N NAP";
$resn_exch{" NC6 NAP"}=" C6N NAP"; $resn_exch{"AP2* NAP"}=" P2B NAP"; $resn_exch{"AOP1 NAP"}=" O1X NAP";
$resn_exch{"AOP2 NAP"}=" O2X NAP"; $resn_exch{"AOP3 NAP"}=" O3X NAP"; $resn_exch{" O5* NAQ"}=" O5B NAQ";
$resn_exch{" C5* NAQ"}=" C5B NAQ"; $resn_exch{" C4* NAQ"}=" C4B NAQ"; $resn_exch{" O4* NAQ"}=" O4B NAQ";
$resn_exch{" C3* NAQ"}=" C3B NAQ"; $resn_exch{" O3* NAQ"}=" O3B NAQ"; $resn_exch{" C2* NAQ"}=" C2B NAQ";
$resn_exch{" O2* NAQ"}=" O2B NAQ"; $resn_exch{" C1* NAQ"}=" C1B NAQ"; $resn_exch{" AP  NAX"}=" PA  NAX";
$resn_exch{" AO1 NAX"}=" O1A NAX"; $resn_exch{" AO2 NAX"}=" O2A NAX"; $resn_exch{"AO5* NAX"}=" O5B NAX";
$resn_exch{"AC5* NAX"}=" C5B NAX"; $resn_exch{"AC4* NAX"}=" C4B NAX"; $resn_exch{"AO4* NAX"}=" O4B NAX";
$resn_exch{"AC3* NAX"}=" C3B NAX"; $resn_exch{"AO3* NAX"}=" O3B NAX"; $resn_exch{"AC2* NAX"}=" C2B NAX";
$resn_exch{"AO2* NAX"}=" O2B NAX"; $resn_exch{"AC1* NAX"}=" C1B NAX"; $resn_exch{" AN9 NAX"}=" N9A NAX";
$resn_exch{" AC8 NAX"}=" C8A NAX"; $resn_exch{" AN7 NAX"}=" N7A NAX"; $resn_exch{" AC5 NAX"}=" C5A NAX";
$resn_exch{" AC6 NAX"}=" C6A NAX"; $resn_exch{" AN6 NAX"}=" N6A NAX"; $resn_exch{" AN1 NAX"}=" N1A NAX";
$resn_exch{" AC2 NAX"}=" C2A NAX"; $resn_exch{" AN3 NAX"}=" N3A NAX"; $resn_exch{" AC4 NAX"}=" C4A NAX";
$resn_exch{" NP  NAX"}=" PN  NAX"; $resn_exch{" NO1 NAX"}=" O1N NAX"; $resn_exch{" NO2 NAX"}=" O2N NAX";
$resn_exch{"NO5* NAX"}=" O5D NAX"; $resn_exch{"NC5* NAX"}=" C5D NAX"; $resn_exch{"NC4* NAX"}=" C4D NAX";
$resn_exch{"NO4* NAX"}=" O4D NAX"; $resn_exch{"NC3* NAX"}=" C3D NAX"; $resn_exch{"NO3* NAX"}=" O3D NAX";
$resn_exch{"NC2* NAX"}=" C2D NAX"; $resn_exch{"NO2* NAX"}=" O2D NAX"; $resn_exch{"NC1* NAX"}=" C1D NAX";
$resn_exch{" NN1 NAX"}=" N1N NAX"; $resn_exch{" NC2 NAX"}=" C2N NAX"; $resn_exch{" NC3 NAX"}=" C3N NAX";
$resn_exch{" NC7 NAX"}=" C7N NAX"; $resn_exch{" NO7 NAX"}=" O7N NAX"; $resn_exch{" NN7 NAX"}=" N7N NAX";
$resn_exch{" NC4 NAX"}=" C4N NAX"; $resn_exch{" NC5 NAX"}=" C5N NAX"; $resn_exch{" NC6 NAX"}=" C6N NAX";
$resn_exch{" NO6 NAX"}=" O6N NAX"; $resn_exch{" C1* NBC"}=" C1' NBC"; $resn_exch{" O1* NBC"}=" O1' NBC";
$resn_exch{" C2* NBC"}=" C2' NBC"; $resn_exch{" C3* NBC"}=" C3' NBC"; $resn_exch{" C4* NBC"}=" C4' NBC";
$resn_exch{" N5* NBC"}=" N5' NBC"; $resn_exch{" O2* NBC"}=" O2' NBC"; $resn_exch{" O3* NBC"}=" O3' NBC";
$resn_exch{" AP  NBD"}=" PA  NBD"; $resn_exch{"AO1P NBD"}=" O11 NBD"; $resn_exch{"AO2P NBD"}=" O12 NBD";
$resn_exch{"AO5* NBD"}=" O5B NBD"; $resn_exch{"AC5* NBD"}=" C5B NBD"; $resn_exch{"AC4* NBD"}=" C4B NBD";
$resn_exch{"AO4* NBD"}=" O4B NBD"; $resn_exch{"AC3* NBD"}=" C3B NBD"; $resn_exch{"AO3* NBD"}=" O3B NBD";
$resn_exch{"AC2* NBD"}=" C2B NBD"; $resn_exch{"AO2* NBD"}=" O2B NBD"; $resn_exch{"AC1* NBD"}=" C1B NBD";
$resn_exch{" AN9 NBD"}=" N9A NBD"; $resn_exch{" AC8 NBD"}=" C8A NBD"; $resn_exch{" AN7 NBD"}=" N7A NBD";
$resn_exch{" AC5 NBD"}=" C5A NBD"; $resn_exch{" AC6 NBD"}=" C6A NBD"; $resn_exch{" AN6 NBD"}=" N6A NBD";
$resn_exch{" AN1 NBD"}=" N1A NBD"; $resn_exch{" AC2 NBD"}=" C2A NBD"; $resn_exch{" AN3 NBD"}=" N3A NBD";
$resn_exch{" AC4 NBD"}=" C4A NBD"; $resn_exch{" NP  NBD"}=" PN  NBD"; $resn_exch{"NO1P NBD"}=" O13 NBD";
$resn_exch{"NO2P NBD"}=" O14 NBD"; $resn_exch{"NO5* NBD"}=" O5D NBD"; $resn_exch{"NC5* NBD"}=" C5D NBD";
$resn_exch{"NC4* NBD"}=" C4D NBD"; $resn_exch{"NO4* NBD"}=" O4D NBD"; $resn_exch{"NC3* NBD"}=" C3D NBD";
$resn_exch{"NO3* NBD"}=" O3D NBD"; $resn_exch{"NC2* NBD"}=" C2D NBD"; $resn_exch{"NO2* NBD"}=" O2D NBD";
$resn_exch{"NC1* NBD"}=" C1D NBD"; $resn_exch{" NN1 NBD"}=" N1N NBD"; $resn_exch{" NC2 NBD"}=" C2N NBD";
$resn_exch{" NC3 NBD"}=" C3N NBD"; $resn_exch{" NC7 NBD"}=" C7N NBD"; $resn_exch{" NO7 NBD"}=" O7N NBD";
$resn_exch{" NN7 NBD"}=" N7N NBD"; $resn_exch{" NC4 NBD"}=" C4N NBD"; $resn_exch{" NC5 NBD"}=" C5N NBD";
$resn_exch{" NC6 NBD"}=" C6N NBD"; $resn_exch{" AP  NBP"}=" PA  NBP"; $resn_exch{" AO1 NBP"}=" O1A NBP";
$resn_exch{" AO2 NBP"}=" O2A NBP"; $resn_exch{"AO5* NBP"}=" O5B NBP"; $resn_exch{"AC5* NBP"}=" C5B NBP";
$resn_exch{"AC4* NBP"}=" C4B NBP"; $resn_exch{"AO4* NBP"}=" O4B NBP"; $resn_exch{"AC3* NBP"}=" C3B NBP";
$resn_exch{"AO3* NBP"}=" O3B NBP"; $resn_exch{"AC2* NBP"}=" C2B NBP"; $resn_exch{"AO2* NBP"}=" O2B NBP";
$resn_exch{"AC1* NBP"}=" C1B NBP"; $resn_exch{" AN9 NBP"}=" N9A NBP"; $resn_exch{" AC8 NBP"}=" C8A NBP";
$resn_exch{" AN7 NBP"}=" N7A NBP"; $resn_exch{" AC5 NBP"}=" C5A NBP"; $resn_exch{" AC6 NBP"}=" C6A NBP";
$resn_exch{" AN6 NBP"}=" N6A NBP"; $resn_exch{" AN1 NBP"}=" N1A NBP"; $resn_exch{" AC2 NBP"}=" C2A NBP";
$resn_exch{" AN3 NBP"}=" N3A NBP"; $resn_exch{" AC4 NBP"}=" C4A NBP"; $resn_exch{" NP  NBP"}=" PN  NBP";
$resn_exch{" NO1 NBP"}=" O1N NBP"; $resn_exch{" NO2 NBP"}=" O2N NBP"; $resn_exch{"NO5* NBP"}=" O5D NBP";
$resn_exch{"NC5* NBP"}=" C5D NBP"; $resn_exch{"NC4* NBP"}=" C4D NBP"; $resn_exch{"NO4* NBP"}=" O4D NBP";
$resn_exch{"NC3* NBP"}=" C3D NBP"; $resn_exch{"NO3* NBP"}=" O3D NBP"; $resn_exch{"NC2* NBP"}=" C2D NBP";
$resn_exch{"NO2* NBP"}=" O2D NBP"; $resn_exch{"NC1* NBP"}=" C1D NBP"; $resn_exch{" NN1 NBP"}=" N1N NBP";
$resn_exch{" NC2 NBP"}=" C2N NBP"; $resn_exch{" NC3 NBP"}=" C3N NBP"; $resn_exch{" NC7 NBP"}=" C7N NBP";
$resn_exch{" NO7 NBP"}=" O7N NBP"; $resn_exch{" NN7 NBP"}=" N7N NBP"; $resn_exch{" NC4 NBP"}=" C4N NBP";
$resn_exch{" NC5 NBP"}=" C5N NBP"; $resn_exch{" NC6 NBP"}=" C6N NBP"; $resn_exch{"AP2* NBP"}=" P2B NBP";
$resn_exch{"AOP1 NBP"}=" O1X NBP"; $resn_exch{"AOP2 NBP"}=" O2X NBP"; $resn_exch{"AOP3 NBP"}=" O3X NBP";
$resn_exch{" O   NBQ"}=" OXT NBQ"; $resn_exch{" O1  NBQ"}=" O   NBQ"; $resn_exch{" O5* NBS"}=" O5' NBS";
$resn_exch{" C5* NBS"}=" C5' NBS"; $resn_exch{" C4* NBS"}=" C4' NBS"; $resn_exch{" O4* NBS"}=" O4' NBS";
$resn_exch{" C1* NBS"}=" C1' NBS"; $resn_exch{" C2* NBS"}=" C2' NBS"; $resn_exch{" O2* NBS"}=" O2' NBS";
$resn_exch{" C3* NBS"}=" C3' NBS"; $resn_exch{" O3* NBS"}=" O3' NBS"; $resn_exch{" O1  NC1"}=" OXT NC1";
$resn_exch{"O100 NCG"}=" OXS NCG"; $resn_exch{"C102 NCG"}=" CXU NCG"; $resn_exch{"O105 NCG"}=" OXX NCG";
$resn_exch{"N108 NCG"}=" NX0 NCG"; $resn_exch{"C110 NCG"}=" CX2 NCG"; $resn_exch{"O118 NCG"}=" OXA NCG";
$resn_exch{" O5* NCN"}=" O5' NCN"; $resn_exch{" C5* NCN"}=" C5' NCN"; $resn_exch{" C4* NCN"}=" C4' NCN";
$resn_exch{" O4* NCN"}=" O4' NCN"; $resn_exch{" C3* NCN"}=" C3' NCN"; $resn_exch{" O3* NCN"}=" O3' NCN";
$resn_exch{" C2* NCN"}=" C2' NCN"; $resn_exch{" O2* NCN"}=" O2' NCN"; $resn_exch{" C1* NCN"}=" C1' NCN";
$resn_exch{" O1* NCS"}=" O1' NCS"; $resn_exch{" C1* NCS"}=" C1' NCS"; $resn_exch{" C2* NCS"}=" C2' NCS";
$resn_exch{" N2* NCS"}=" N2' NCS"; $resn_exch{" C3* NCS"}=" C3' NCS"; $resn_exch{" O3* NCS"}=" O3' NCS";
$resn_exch{" C4* NCS"}=" C4' NCS"; $resn_exch{" O4* NCS"}=" O4' NCS"; $resn_exch{" C5* NCS"}=" C5' NCS";
$resn_exch{" O5* NCS"}=" O5' NCS"; $resn_exch{" O1P NCX"}=" OP1 NCX"; $resn_exch{" O2P NCX"}=" OP2 NCX";
$resn_exch{" O3P NCX"}=" OP3 NCX"; $resn_exch{" O5* NCX"}=" O5' NCX"; $resn_exch{" C5* NCX"}=" C5' NCX";
$resn_exch{" C4* NCX"}=" C4' NCX"; $resn_exch{" O4* NCX"}=" O4' NCX"; $resn_exch{" C1* NCX"}=" C1' NCX";
$resn_exch{" C3* NCX"}=" C3' NCX"; $resn_exch{" C2* NCX"}=" C2' NCX"; $resn_exch{" O3* NCX"}=" O3' NCX";
$resn_exch{" AP  NDA"}=" PA  NDA"; $resn_exch{" AO1 NDA"}=" O1A NDA"; $resn_exch{" AO2 NDA"}=" O2A NDA";
$resn_exch{"AO5* NDA"}=" O5B NDA"; $resn_exch{"AC5* NDA"}=" C5B NDA"; $resn_exch{"AC4* NDA"}=" C4B NDA";
$resn_exch{"AO4* NDA"}=" O4B NDA"; $resn_exch{"AC3* NDA"}=" C3B NDA"; $resn_exch{"AO3* NDA"}=" O3B NDA";
$resn_exch{"AC2* NDA"}=" C2B NDA"; $resn_exch{"AO2* NDA"}=" O2B NDA"; $resn_exch{"AC1* NDA"}=" C1B NDA";
$resn_exch{" AN9 NDA"}=" N9A NDA"; $resn_exch{" AC8 NDA"}=" C8A NDA"; $resn_exch{" AN7 NDA"}=" N7A NDA";
$resn_exch{" AC5 NDA"}=" C5A NDA"; $resn_exch{" AC6 NDA"}=" C6A NDA"; $resn_exch{" AN6 NDA"}=" N6A NDA";
$resn_exch{" AN1 NDA"}=" N1A NDA"; $resn_exch{" AC2 NDA"}=" C2A NDA"; $resn_exch{" AN3 NDA"}=" N3A NDA";
$resn_exch{" AC4 NDA"}=" C4A NDA"; $resn_exch{" NP  NDA"}=" PN  NDA"; $resn_exch{" NO1 NDA"}=" O1N NDA";
$resn_exch{" NO2 NDA"}=" O2N NDA"; $resn_exch{"NO5* NDA"}=" O5D NDA"; $resn_exch{"NC5* NDA"}=" C5D NDA";
$resn_exch{"NC4* NDA"}=" C4D NDA"; $resn_exch{"NO4* NDA"}=" O4D NDA"; $resn_exch{"NC3* NDA"}=" C3D NDA";
$resn_exch{"NO3* NDA"}=" O3D NDA"; $resn_exch{"NC2* NDA"}=" C2D NDA"; $resn_exch{"NO2* NDA"}=" O2D NDA";
$resn_exch{"NC1* NDA"}=" C1D NDA"; $resn_exch{" NN1 NDA"}=" N1N NDA"; $resn_exch{" NC2 NDA"}=" C2N NDA";
$resn_exch{" NC3 NDA"}=" C3N NDA"; $resn_exch{" NC7 NDA"}=" C7N NDA"; $resn_exch{" NN7 NDA"}=" N7N NDA";
$resn_exch{" NC4 NDA"}=" C4N NDA"; $resn_exch{" NC5 NDA"}=" C5N NDA"; $resn_exch{" NC6 NDA"}=" C6N NDA";
$resn_exch{" O5* NDC"}=" O5B NDC"; $resn_exch{" C5* NDC"}=" C5B NDC"; $resn_exch{" C4* NDC"}=" C4B NDC";
$resn_exch{" O4* NDC"}=" O4B NDC"; $resn_exch{" C3* NDC"}=" C3B NDC"; $resn_exch{" O3* NDC"}=" O3B NDC";
$resn_exch{" C2* NDC"}=" C2B NDC"; $resn_exch{" O2* NDC"}=" O2B NDC"; $resn_exch{" C1* NDC"}=" C1B NDC";
$resn_exch{" NO7 NDE"}=" O7N NDE"; $resn_exch{" NC7 NDE"}=" C7N NDE"; $resn_exch{" NN7 NDE"}=" N7N NDE";
$resn_exch{" NC3 NDE"}=" C3N NDE"; $resn_exch{" NC4 NDE"}=" C4N NDE"; $resn_exch{" NC5 NDE"}=" C5N NDE";
$resn_exch{" NC6 NDE"}=" C6N NDE"; $resn_exch{" NC2 NDE"}=" C2N NDE"; $resn_exch{" NN1 NDE"}=" N1N NDE";
$resn_exch{"NC1* NDE"}=" C1D NDE"; $resn_exch{"NC2* NDE"}=" C2D NDE"; $resn_exch{"NO2* NDE"}=" O2D NDE";
$resn_exch{"NC3* NDE"}=" C3D NDE"; $resn_exch{"NO3* NDE"}=" O3D NDE"; $resn_exch{"NO4* NDE"}=" O4D NDE";
$resn_exch{"NC4* NDE"}=" C4D NDE"; $resn_exch{"NC5* NDE"}=" C5D NDE"; $resn_exch{"NO5* NDE"}=" O5D NDE";
$resn_exch{" NP  NDE"}=" PN  NDE"; $resn_exch{" NO1 NDE"}=" O1N NDE"; $resn_exch{" NO2 NDE"}=" O2N NDE";
$resn_exch{" AP  NDE"}=" PA  NDE"; $resn_exch{" AO1 NDE"}=" O1A NDE"; $resn_exch{" AO2 NDE"}=" O2A NDE";
$resn_exch{"AO5* NDE"}=" O5B NDE"; $resn_exch{"AC5* NDE"}=" C5B NDE"; $resn_exch{"AC4* NDE"}=" C4B NDE";
$resn_exch{"AO4* NDE"}=" O4B NDE"; $resn_exch{"AC1* NDE"}=" C1B NDE"; $resn_exch{"AC2* NDE"}=" C2B NDE";
$resn_exch{"AO2* NDE"}=" O2B NDE"; $resn_exch{"AC3* NDE"}=" C3B NDE"; $resn_exch{"AO3* NDE"}=" O3B NDE";
$resn_exch{" AN9 NDE"}=" N9A NDE"; $resn_exch{" AC8 NDE"}=" C8A NDE"; $resn_exch{" AN7 NDE"}=" N7A NDE";
$resn_exch{" AC5 NDE"}=" C5A NDE"; $resn_exch{" AC6 NDE"}=" C6A NDE"; $resn_exch{" AN6 NDE"}=" N6A NDE";
$resn_exch{" AC4 NDE"}=" C4A NDE"; $resn_exch{" AN3 NDE"}=" N3A NDE"; $resn_exch{" AC2 NDE"}=" C2A NDE";
$resn_exch{" AN1 NDE"}=" N1A NDE"; $resn_exch{" AP  NDO"}=" PA  NDO"; $resn_exch{" AO1 NDO"}=" O1A NDO";
$resn_exch{" AO2 NDO"}=" O2A NDO"; $resn_exch{"AO5* NDO"}=" O5B NDO"; $resn_exch{"AC5* NDO"}=" C5B NDO";
$resn_exch{"AC4* NDO"}=" C4B NDO"; $resn_exch{"AO4* NDO"}=" O4B NDO"; $resn_exch{"AC3* NDO"}=" C3B NDO";
$resn_exch{"AO3* NDO"}=" O3B NDO"; $resn_exch{"AC2* NDO"}=" C2B NDO"; $resn_exch{"AO2* NDO"}=" O2B NDO";
$resn_exch{"AC1* NDO"}=" C1B NDO"; $resn_exch{" AN9 NDO"}=" N9A NDO"; $resn_exch{" AC8 NDO"}=" C8A NDO";
$resn_exch{" AN7 NDO"}=" N7A NDO"; $resn_exch{" AC5 NDO"}=" C5A NDO"; $resn_exch{" AC6 NDO"}=" C6A NDO";
$resn_exch{" AO6 NDO"}=" O6A NDO"; $resn_exch{" AN1 NDO"}=" N1A NDO"; $resn_exch{" AC2 NDO"}=" C2A NDO";
$resn_exch{" AN3 NDO"}=" N3A NDO"; $resn_exch{" AC4 NDO"}=" C4A NDO"; $resn_exch{" NP  NDO"}=" PN  NDO";
$resn_exch{" NO1 NDO"}=" O1N NDO"; $resn_exch{" NO2 NDO"}=" O2N NDO"; $resn_exch{"NO5* NDO"}=" O5D NDO";
$resn_exch{"NC5* NDO"}=" C5D NDO"; $resn_exch{"NC4* NDO"}=" C4D NDO"; $resn_exch{"NO4* NDO"}=" O4D NDO";
$resn_exch{"NC3* NDO"}=" C3D NDO"; $resn_exch{"NO3* NDO"}=" O3D NDO"; $resn_exch{"NC2* NDO"}=" C2D NDO";
$resn_exch{"NO2* NDO"}=" O2D NDO"; $resn_exch{"NC1* NDO"}=" C1D NDO"; $resn_exch{" NN1 NDO"}=" N1N NDO";
$resn_exch{" NC2 NDO"}=" C2N NDO"; $resn_exch{" NC3 NDO"}=" C3N NDO"; $resn_exch{" NC7 NDO"}=" C7N NDO";
$resn_exch{" NO7 NDO"}=" O7N NDO"; $resn_exch{" NN7 NDO"}=" N7N NDO"; $resn_exch{" NC4 NDO"}=" C4N NDO";
$resn_exch{" NC5 NDO"}=" C5N NDO"; $resn_exch{" NC6 NDO"}=" C6N NDO"; $resn_exch{"AP2* NDO"}=" P2B NDO";
$resn_exch{"AOP1 NDO"}=" O1X NDO"; $resn_exch{"AOP2 NDO"}=" O2X NDO"; $resn_exch{"AOP3 NDO"}=" O3X NDO";
$resn_exch{" AP  NDP"}=" PA  NDP"; $resn_exch{" AO1 NDP"}=" O1A NDP"; $resn_exch{" AO2 NDP"}=" O2A NDP";
$resn_exch{"AO5* NDP"}=" O5B NDP"; $resn_exch{"AC5* NDP"}=" C5B NDP"; $resn_exch{"AC4* NDP"}=" C4B NDP";
$resn_exch{"AO4* NDP"}=" O4B NDP"; $resn_exch{"AC3* NDP"}=" C3B NDP"; $resn_exch{"AO3* NDP"}=" O3B NDP";
$resn_exch{"AC2* NDP"}=" C2B NDP"; $resn_exch{"AO2* NDP"}=" O2B NDP"; $resn_exch{"AC1* NDP"}=" C1B NDP";
$resn_exch{" AN9 NDP"}=" N9A NDP"; $resn_exch{" AC8 NDP"}=" C8A NDP"; $resn_exch{" AN7 NDP"}=" N7A NDP";
$resn_exch{" AC5 NDP"}=" C5A NDP"; $resn_exch{" AC6 NDP"}=" C6A NDP"; $resn_exch{" AN6 NDP"}=" N6A NDP";
$resn_exch{" AN1 NDP"}=" N1A NDP"; $resn_exch{" AC2 NDP"}=" C2A NDP"; $resn_exch{" AN3 NDP"}=" N3A NDP";
$resn_exch{" AC4 NDP"}=" C4A NDP"; $resn_exch{" NP  NDP"}=" PN  NDP"; $resn_exch{" NO1 NDP"}=" O1N NDP";
$resn_exch{" NO2 NDP"}=" O2N NDP"; $resn_exch{"NO5* NDP"}=" O5D NDP"; $resn_exch{"NC5* NDP"}=" C5D NDP";
$resn_exch{"NC4* NDP"}=" C4D NDP"; $resn_exch{"NO4* NDP"}=" O4D NDP"; $resn_exch{"NC3* NDP"}=" C3D NDP";
$resn_exch{"NO3* NDP"}=" O3D NDP"; $resn_exch{"NC2* NDP"}=" C2D NDP"; $resn_exch{"NO2* NDP"}=" O2D NDP";
$resn_exch{"NC1* NDP"}=" C1D NDP"; $resn_exch{" NN1 NDP"}=" N1N NDP"; $resn_exch{" NC2 NDP"}=" C2N NDP";
$resn_exch{" NC3 NDP"}=" C3N NDP"; $resn_exch{" NC7 NDP"}=" C7N NDP"; $resn_exch{" NO7 NDP"}=" O7N NDP";
$resn_exch{" NN7 NDP"}=" N7N NDP"; $resn_exch{" NC4 NDP"}=" C4N NDP"; $resn_exch{" NC5 NDP"}=" C5N NDP";
$resn_exch{" NC6 NDP"}=" C6N NDP"; $resn_exch{"AP2* NDP"}=" P2B NDP"; $resn_exch{"AOP1 NDP"}=" O1X NDP";
$resn_exch{"AOP2 NDP"}=" O2X NDP"; $resn_exch{"AOP3 NDP"}=" O3X NDP"; $resn_exch{" C5* NEA"}=" C5' NEA";
$resn_exch{" C4* NEA"}=" C4' NEA"; $resn_exch{" O4* NEA"}=" O4' NEA"; $resn_exch{" C3* NEA"}=" C3' NEA";
$resn_exch{" O3* NEA"}=" O3' NEA"; $resn_exch{" C2* NEA"}=" C2' NEA"; $resn_exch{" O2* NEA"}=" O2' NEA";
$resn_exch{" C1* NEA"}=" C1' NEA"; $resn_exch{" C5* NEC"}=" C5' NEC"; $resn_exch{" O5* NEC"}=" O5' NEC";
$resn_exch{" N5* NEC"}=" N5' NEC"; $resn_exch{" C4* NEC"}=" C4' NEC"; $resn_exch{" O4* NEC"}=" O4' NEC";
$resn_exch{" C3* NEC"}=" C3' NEC"; $resn_exch{" O3* NEC"}=" O3' NEC"; $resn_exch{" C2* NEC"}=" C2' NEC";
$resn_exch{" O2* NEC"}=" O2' NEC"; $resn_exch{" C1* NEC"}=" C1' NEC"; $resn_exch{" N5* NEI"}=" N5' NEI";
$resn_exch{" C5* NEI"}=" C5' NEI"; $resn_exch{" O5* NEI"}=" O5' NEI"; $resn_exch{" C4* NEI"}=" C4' NEI";
$resn_exch{" O4* NEI"}=" O4' NEI"; $resn_exch{" C3* NEI"}=" C3' NEI"; $resn_exch{" O3* NEI"}=" O3' NEI";
$resn_exch{" C2* NEI"}=" C2' NEI"; $resn_exch{" O2* NEI"}=" O2' NEI"; $resn_exch{" C1* NEI"}=" C1' NEI";
$resn_exch{"1O1   NF"}=" O11  NF"; $resn_exch{"1C1   NF"}=" C11  NF"; $resn_exch{"1N2   NF"}=" N21  NF";
$resn_exch{"2C1   NF"}=" C12  NF"; $resn_exch{"1C3   NF"}=" C31  NF"; $resn_exch{"1O3   NF"}=" O31  NF";
$resn_exch{"1C11  NF"}=" CX3  NF"; $resn_exch{"1C12  NF"}=" CXD  NF"; $resn_exch{"1C10  NF"}=" CXT  NF";
$resn_exch{"1C9   NF"}=" C91  NF"; $resn_exch{"1C8   NF"}=" C81  NF"; $resn_exch{"1C7   NF"}=" C71  NF";
$resn_exch{"1C13  NF"}=" CXN  NF"; $resn_exch{"1C6   NF"}=" C61  NF"; $resn_exch{"1C5   NF"}=" C51  NF";
$resn_exch{"1C4   NF"}=" C41  NF"; $resn_exch{"2C2   NF"}=" C22  NF"; $resn_exch{"2N3   NF"}=" N32  NF";
$resn_exch{"2C4   NF"}=" C42  NF"; $resn_exch{"2C5   NF"}=" C52  NF"; $resn_exch{"2C6   NF"}=" C62  NF";
$resn_exch{"2N7   NF"}=" N72  NF"; $resn_exch{"2C8   NF"}=" C82  NF"; $resn_exch{"3O1   NF"}=" O13  NF";
$resn_exch{"3C1   NF"}=" C13  NF"; $resn_exch{"3N2   NF"}=" N23  NF"; $resn_exch{"2C9   NF"}=" C92  NF";
$resn_exch{"3C3   NF"}=" C33  NF"; $resn_exch{"3O3   NF"}=" O33  NF"; $resn_exch{"3C11  NF"}=" CX5  NF";
$resn_exch{"3C12  NF"}=" CXF  NF"; $resn_exch{"3C10  NF"}=" CXV  NF"; $resn_exch{"3C9   NF"}=" C93  NF";
$resn_exch{"3C8   NF"}=" C83  NF"; $resn_exch{"3C7   NF"}=" C73  NF"; $resn_exch{"3C13  NF"}=" CXP  NF";
$resn_exch{"3C6   NF"}=" C63  NF"; $resn_exch{"3C5   NF"}=" C53  NF"; $resn_exch{"3C4   NF"}=" C43  NF";
$resn_exch{" C1* NF2"}=" C1' NF2"; $resn_exch{" O4* NF2"}=" O4' NF2"; $resn_exch{" C2* NF2"}=" C2' NF2";
$resn_exch{" O2* NF2"}=" O2' NF2"; $resn_exch{" C3* NF2"}=" C3' NF2"; $resn_exch{" O3* NF2"}=" O3' NF2";
$resn_exch{" C4* NF2"}=" C4' NF2"; $resn_exch{" C5* NF2"}=" C5' NF2"; $resn_exch{" O5* NF2"}=" O5' NF2";
$resn_exch{" O1P NF2"}=" OP1 NF2"; $resn_exch{" O2P NF2"}=" OP2 NF2"; $resn_exch{" O3P NF2"}=" OP3 NF2";
$resn_exch{" C1* NGD"}=" C1' NGD"; $resn_exch{" O4* NGD"}=" O4' NGD"; $resn_exch{" C2* NGD"}=" C2' NGD";
$resn_exch{" O2* NGD"}=" O2' NGD"; $resn_exch{" C3* NGD"}=" C3' NGD"; $resn_exch{" O3* NGD"}=" O3' NGD";
$resn_exch{" C4* NGD"}=" C4' NGD"; $resn_exch{" C5* NGD"}=" C5' NGD"; $resn_exch{" O5* NGD"}=" O5' NGD";
$resn_exch{"RO5* NGD"}="O5'R NGD"; $resn_exch{"RC5* NGD"}="C5'R NGD"; $resn_exch{"RC4* NGD"}="C4'R NGD";
$resn_exch{"RC3* NGD"}="C3'R NGD"; $resn_exch{"RO3* NGD"}="O3'R NGD"; $resn_exch{"RC2* NGD"}="C2'R NGD";
$resn_exch{"RO2* NGD"}="O2'R NGD"; $resn_exch{"RO4* NGD"}="O4'R NGD"; $resn_exch{"RC1* NGD"}="C1'R NGD";
$resn_exch{" C1* NGM"}=" C1' NGM"; $resn_exch{" C2* NGM"}=" C2' NGM"; $resn_exch{" C3* NGM"}=" C3' NGM";
$resn_exch{" C4* NGM"}=" C4' NGM"; $resn_exch{" C5* NGM"}=" C5' NGM"; $resn_exch{" O1* NGM"}=" O1' NGM";
$resn_exch{" O2* NGM"}=" O2' NGM"; $resn_exch{" O3* NGM"}=" O3' NGM"; $resn_exch{" O4* NGM"}=" O4' NGM";
$resn_exch{" C1* NGP"}=" C1' NGP"; $resn_exch{" C2* NGP"}=" C2' NGP"; $resn_exch{" C3* NGP"}=" C3' NGP";
$resn_exch{" C4* NGP"}=" C4' NGP"; $resn_exch{" C5* NGP"}=" C5' NGP"; $resn_exch{" O1* NGP"}=" O1' NGP";
$resn_exch{" O2* NGP"}=" O2' NGP"; $resn_exch{" O3* NGP"}=" O3' NGP"; $resn_exch{" O4* NGP"}=" O4' NGP";
$resn_exch{" C1* NGU"}=" C1' NGU"; $resn_exch{" C2* NGU"}=" C2' NGU"; $resn_exch{" C3* NGU"}=" C3' NGU";
$resn_exch{" C4* NGU"}=" C4' NGU"; $resn_exch{" C5* NGU"}=" C5' NGU"; $resn_exch{" O1* NGU"}=" O1' NGU";
$resn_exch{" O2* NGU"}=" O2' NGU"; $resn_exch{" O3* NGU"}=" O3' NGU"; $resn_exch{" O4* NGU"}=" O4' NGU";
$resn_exch{" AP  NHD"}=" PA  NHD"; $resn_exch{" AO1 NHD"}=" O1A NHD"; $resn_exch{" AO2 NHD"}=" O2A NHD";
$resn_exch{"AO5* NHD"}=" O5B NHD"; $resn_exch{"AC5* NHD"}=" C5B NHD"; $resn_exch{"AC4* NHD"}=" C4B NHD";
$resn_exch{"AO4* NHD"}=" O4B NHD"; $resn_exch{"AC3* NHD"}=" C3B NHD"; $resn_exch{"AO3* NHD"}=" O3B NHD";
$resn_exch{"AC2* NHD"}=" C2B NHD"; $resn_exch{"AO2* NHD"}=" O2B NHD"; $resn_exch{"AC1* NHD"}=" C1B NHD";
$resn_exch{" AN9 NHD"}=" N9A NHD"; $resn_exch{" AC8 NHD"}=" C8A NHD"; $resn_exch{" AN7 NHD"}=" N7A NHD";
$resn_exch{" AC5 NHD"}=" C5A NHD"; $resn_exch{" AC6 NHD"}=" C6A NHD"; $resn_exch{" AO6 NHD"}=" O6A NHD";
$resn_exch{" AN1 NHD"}=" N1A NHD"; $resn_exch{" AC2 NHD"}=" C2A NHD"; $resn_exch{" AN3 NHD"}=" N3A NHD";
$resn_exch{" AC4 NHD"}=" C4A NHD"; $resn_exch{" NP  NHD"}=" PN  NHD"; $resn_exch{" NO1 NHD"}=" O1N NHD";
$resn_exch{" NO2 NHD"}=" O2N NHD"; $resn_exch{"NO5* NHD"}=" O5D NHD"; $resn_exch{"NC5* NHD"}=" C5D NHD";
$resn_exch{"NC4* NHD"}=" C4D NHD"; $resn_exch{"NO4* NHD"}=" O4D NHD"; $resn_exch{"NC3* NHD"}=" C3D NHD";
$resn_exch{"NO3* NHD"}=" O3D NHD"; $resn_exch{"NC2* NHD"}=" C2D NHD"; $resn_exch{"NO2* NHD"}=" O2D NHD";
$resn_exch{"NC1* NHD"}=" C1D NHD"; $resn_exch{" NN1 NHD"}=" N1N NHD"; $resn_exch{" NC2 NHD"}=" C2N NHD";
$resn_exch{" NC3 NHD"}=" C3N NHD"; $resn_exch{" NC7 NHD"}=" C7N NHD"; $resn_exch{" NO7 NHD"}=" O7N NHD";
$resn_exch{" NN7 NHD"}=" N7N NHD"; $resn_exch{" NC4 NHD"}=" C4N NHD"; $resn_exch{" NC5 NHD"}=" C5N NHD";
$resn_exch{" NC6 NHD"}=" C6N NHD"; $resn_exch{" O5* NHO"}=" O5B NHO"; $resn_exch{" C5* NHO"}=" C5B NHO";
$resn_exch{" C4* NHO"}=" C4B NHO"; $resn_exch{" O4* NHO"}=" O4B NHO"; $resn_exch{" C3* NHO"}=" C3B NHO";
$resn_exch{" O3* NHO"}=" O3B NHO"; $resn_exch{" C2* NHO"}=" C2B NHO"; $resn_exch{" O2* NHO"}=" O2B NHO";
$resn_exch{" C1* NHO"}=" C1B NHO"; $resn_exch{" C1* NIR"}=" C1' NIR"; $resn_exch{" C2* NIR"}=" C2' NIR";
$resn_exch{" O2* NIR"}=" O2' NIR"; $resn_exch{" C3* NIR"}=" C3' NIR"; $resn_exch{" O3* NIR"}=" O3' NIR";
$resn_exch{" C4* NIR"}=" C4' NIR"; $resn_exch{" O4* NIR"}=" O4' NIR"; $resn_exch{" C5* NIR"}=" C5' NIR";
$resn_exch{" O   NLQ"}=" OXT NLQ"; $resn_exch{" OXT NLQ"}=" O   NLQ"; $resn_exch{" C5* NMD"}=" C5' NMD";
$resn_exch{" C4* NMD"}=" C4' NMD"; $resn_exch{" O4* NMD"}=" O4' NMD"; $resn_exch{" C3* NMD"}=" C3' NMD";
$resn_exch{" O3* NMD"}=" O3' NMD"; $resn_exch{" C2* NMD"}=" C2' NMD"; $resn_exch{" N2* NMD"}=" N2' NMD";
$resn_exch{" C1* NMD"}=" C1' NMD"; $resn_exch{" O5* NMD"}=" O5' NMD"; $resn_exch{" O2P NMS"}=" OP2 NMS";
$resn_exch{" O1P NMS"}=" OP1 NMS"; $resn_exch{" O5* NMS"}=" O5' NMS"; $resn_exch{" C2* NMS"}=" C2' NMS";
$resn_exch{" C5* NMS"}=" C5' NMS"; $resn_exch{" C4* NMS"}=" C4' NMS"; $resn_exch{" O4* NMS"}=" O4' NMS";
$resn_exch{" C1* NMS"}=" C1' NMS"; $resn_exch{" C3* NMS"}=" C3' NMS"; $resn_exch{" O3* NMS"}=" O3' NMS";
$resn_exch{" O3P NMS"}=" OP3 NMS"; $resn_exch{" O3P NMT"}=" OP3 NMT"; $resn_exch{" O2P NMT"}=" OP2 NMT";
$resn_exch{" O5* NMT"}=" O5' NMT"; $resn_exch{" C2* NMT"}=" C2' NMT"; $resn_exch{" C5* NMT"}=" C5' NMT";
$resn_exch{" C4* NMT"}=" C4' NMT"; $resn_exch{" O4* NMT"}=" O4' NMT"; $resn_exch{" C1* NMT"}=" C1' NMT";
$resn_exch{" C3* NMT"}=" C3' NMT"; $resn_exch{" O3* NMT"}=" O3' NMT"; $resn_exch{" O1P NMT"}=" OP1 NMT";
$resn_exch{" AN1 NMX"}=" N1A NMX"; $resn_exch{" AC2 NMX"}=" C2A NMX"; $resn_exch{" AN3 NMX"}=" N3A NMX";
$resn_exch{" AC4 NMX"}=" C4A NMX"; $resn_exch{" AC5 NMX"}=" C5A NMX"; $resn_exch{" AC6 NMX"}=" C6A NMX";
$resn_exch{" AN6 NMX"}=" N6A NMX"; $resn_exch{" AN7 NMX"}=" N7A NMX"; $resn_exch{" AC8 NMX"}=" C8A NMX";
$resn_exch{" AN9 NMX"}=" N9A NMX"; $resn_exch{"AC1* NMX"}=" C1B NMX"; $resn_exch{"AC2* NMX"}=" C2B NMX";
$resn_exch{"AO2* NMX"}=" O2B NMX"; $resn_exch{"AC3* NMX"}=" C3B NMX"; $resn_exch{"AO3* NMX"}=" O3B NMX";
$resn_exch{"AP3* NMX"}=" P3B NMX"; $resn_exch{" AO7 NMX"}=" O7A NMX"; $resn_exch{" AO8 NMX"}=" O8A NMX";
$resn_exch{" AO9 NMX"}=" O9A NMX"; $resn_exch{"AC4* NMX"}=" C4B NMX"; $resn_exch{"AO4* NMX"}=" O4B NMX";
$resn_exch{"AC5* NMX"}=" C5B NMX"; $resn_exch{"AO5* NMX"}=" O5B NMX"; $resn_exch{" AP1 NMX"}=" P1A NMX";
$resn_exch{" AO1 NMX"}=" O1A NMX"; $resn_exch{" AO2 NMX"}=" O2A NMX"; $resn_exch{" AO3 NMX"}=" O3A NMX";
$resn_exch{" AP2 NMX"}=" P2A NMX"; $resn_exch{" AO4 NMX"}=" O4A NMX"; $resn_exch{" AO5 NMX"}=" O5A NMX";
$resn_exch{" AO6 NMX"}=" O6A NMX"; $resn_exch{"PC11 NMX"}=" CBP NMX"; $resn_exch{"PC12 NMX"}=" CCP NMX";
$resn_exch{"PC13 NMX"}=" CDP NMX"; $resn_exch{"PC14 NMX"}=" CEP NMX"; $resn_exch{"PC10 NMX"}=" CAP NMX";
$resn_exch{"PO10 NMX"}=" OAP NMX"; $resn_exch{" PC9 NMX"}=" C9P NMX"; $resn_exch{" PO9 NMX"}=" O9P NMX";
$resn_exch{" PN8 NMX"}=" N8P NMX"; $resn_exch{" PC7 NMX"}=" C7P NMX"; $resn_exch{" PC6 NMX"}=" C6P NMX";
$resn_exch{" PC5 NMX"}=" C5P NMX"; $resn_exch{" PO5 NMX"}=" O5P NMX"; $resn_exch{" PN4 NMX"}=" N4P NMX";
$resn_exch{" PC3 NMX"}=" C3P NMX"; $resn_exch{" PC2 NMX"}=" C2P NMX"; $resn_exch{" IC1 NMX"}=" C1I NMX";
$resn_exch{" IN1 NMX"}=" N1I NMX"; $resn_exch{" IO1 NMX"}=" O1I NMX"; $resn_exch{" IO2 NMX"}=" O2I NMX";
$resn_exch{" C6* NOC"}=" C6' NOC"; $resn_exch{" O6* NOC"}=" O6' NOC"; $resn_exch{" C5* NOC"}=" C5' NOC";
$resn_exch{" C4* NOC"}=" C4' NOC"; $resn_exch{" C3* NOC"}=" C3' NOC"; $resn_exch{" O3* NOC"}=" O3' NOC";
$resn_exch{" C2* NOC"}=" C2' NOC"; $resn_exch{" O2* NOC"}=" O2' NOC"; $resn_exch{" C1* NOC"}=" C1' NOC";
$resn_exch{" C1* NOD"}=" C1' NOD"; $resn_exch{" C2* NOD"}=" C2' NOD"; $resn_exch{" C3* NOD"}=" C3' NOD";
$resn_exch{" C4* NOD"}=" C4' NOD"; $resn_exch{" O4* NOD"}=" O4' NOD"; $resn_exch{" C5* NOD"}=" C5' NOD";
$resn_exch{" O5* NOD"}=" O5' NOD"; $resn_exch{" C6* NOD"}=" C6' NOD"; $resn_exch{" N3* NOD"}=" N3' NOD";
$resn_exch{" C5* NOS"}=" C5' NOS"; $resn_exch{" O5* NOS"}=" O5' NOS"; $resn_exch{" C4* NOS"}=" C4' NOS";
$resn_exch{" O4* NOS"}=" O4' NOS"; $resn_exch{" C1* NOS"}=" C1' NOS"; $resn_exch{" C2* NOS"}=" C2' NOS";
$resn_exch{" O2* NOS"}=" O2' NOS"; $resn_exch{" C3* NOS"}=" C3' NOS"; $resn_exch{" O3* NOS"}=" O3' NOS";
$resn_exch{" O5* NP3"}=" O5' NP3"; $resn_exch{" C5* NP3"}=" C5' NP3"; $resn_exch{" C4* NP3"}=" C4' NP3";
$resn_exch{" O4* NP3"}=" O4' NP3"; $resn_exch{" C1* NP3"}=" C1' NP3"; $resn_exch{" C2* NP3"}=" C2' NP3";
$resn_exch{" C3* NP3"}=" C3' NP3"; $resn_exch{" O3* NP3"}=" O3' NP3"; $resn_exch{" O5* NRI"}=" O5' NRI";
$resn_exch{" C2* NRI"}=" C2' NRI"; $resn_exch{" C5* NRI"}=" C5' NRI"; $resn_exch{" C4* NRI"}=" C4' NRI";
$resn_exch{" C3* NRI"}=" C3' NRI"; $resn_exch{" C6* NRI"}=" C6' NRI"; $resn_exch{" N1* NRI"}=" N1' NRI";
$resn_exch{" O3* NRI"}=" O3' NRI"; $resn_exch{" C4* NRT"}=" C4' NRT"; $resn_exch{" O4* NRT"}=" O4' NRT";
$resn_exch{" C3* NRT"}=" C3' NRT"; $resn_exch{" C2* NRT"}=" C2' NRT"; $resn_exch{" O5* NSS"}=" O5' NSS";
$resn_exch{" C5* NSS"}=" C5' NSS"; $resn_exch{" C4* NSS"}=" C4' NSS"; $resn_exch{" C3* NSS"}=" C3' NSS";
$resn_exch{" O3* NSS"}=" O3' NSS"; $resn_exch{" C2* NSS"}=" C2' NSS"; $resn_exch{" O2* NSS"}=" O2' NSS";
$resn_exch{" C1* NSS"}=" C1' NSS"; $resn_exch{" O4* NSS"}=" O4' NSS"; $resn_exch{" C2* NTU"}=" C2' NTU";
$resn_exch{" S1* NTU"}=" S1' NTU"; $resn_exch{" C1* NTU"}=" C1' NTU"; $resn_exch{" C4* NTU"}=" C4' NTU";
$resn_exch{" S2* NTU"}=" S2' NTU"; $resn_exch{" C3* NTU"}=" C3' NTU"; $resn_exch{" C1* NUP"}=" C1' NUP";
$resn_exch{" O4* NUP"}=" O4' NUP"; $resn_exch{" C2* NUP"}=" C2' NUP"; $resn_exch{" O2* NUP"}=" O2' NUP";
$resn_exch{" C3* NUP"}=" C3' NUP"; $resn_exch{" O3* NUP"}=" O3' NUP"; $resn_exch{" C4* NUP"}=" C4' NUP";
$resn_exch{" C5* NUP"}=" C5' NUP"; $resn_exch{" O5* NUP"}=" O5' NUP"; $resn_exch{" OXT NYC"}=" OXY NYC";
$resn_exch{" O4* NYM"}=" O4' NYM"; $resn_exch{" N3* NYM"}=" N3' NYM"; $resn_exch{" C1* NYM"}=" C1' NYM";
$resn_exch{" C2* NYM"}=" C2' NYM"; $resn_exch{" C3* NYM"}=" C3' NYM"; $resn_exch{" C4* NYM"}=" C4' NYM";
$resn_exch{" O5* NYM"}=" O5' NYM"; $resn_exch{" C5* NYM"}=" C5' NYM"; $resn_exch{"AOP1 NZQ"}=" O1X NZQ";
$resn_exch{"AP2* NZQ"}=" P2B NZQ"; $resn_exch{"AOP2 NZQ"}=" O2X NZQ"; $resn_exch{"AOP3 NZQ"}=" O3X NZQ";
$resn_exch{"AO2* NZQ"}=" O2B NZQ"; $resn_exch{"AC2* NZQ"}=" C2B NZQ"; $resn_exch{"AC3* NZQ"}=" C3B NZQ";
$resn_exch{"AO3* NZQ"}=" O3B NZQ"; $resn_exch{"AC1* NZQ"}=" C1B NZQ"; $resn_exch{" AN9 NZQ"}=" N9A NZQ";
$resn_exch{" AC8 NZQ"}=" C8A NZQ"; $resn_exch{" AN7 NZQ"}=" N7A NZQ"; $resn_exch{" AC5 NZQ"}=" C5A NZQ";
$resn_exch{" AC4 NZQ"}=" C4A NZQ"; $resn_exch{" AN3 NZQ"}=" N3A NZQ"; $resn_exch{" AC2 NZQ"}=" C2A NZQ";
$resn_exch{" AN1 NZQ"}=" N1A NZQ"; $resn_exch{" AC6 NZQ"}=" C6A NZQ"; $resn_exch{" AN6 NZQ"}=" N6A NZQ";
$resn_exch{"AO4* NZQ"}=" O4B NZQ"; $resn_exch{"AC4* NZQ"}=" C4B NZQ"; $resn_exch{"AC5* NZQ"}=" C5B NZQ";
$resn_exch{"AO5* NZQ"}=" O5B NZQ"; $resn_exch{" AP  NZQ"}=" PA  NZQ"; $resn_exch{" AO1 NZQ"}=" O1A NZQ";
$resn_exch{" AO2 NZQ"}=" O2A NZQ"; $resn_exch{" NP  NZQ"}=" PN  NZQ"; $resn_exch{" NO1 NZQ"}=" O1N NZQ";
$resn_exch{" NO2 NZQ"}=" O2N NZQ"; $resn_exch{"NO5* NZQ"}=" O5D NZQ"; $resn_exch{"NC5* NZQ"}=" C5D NZQ";
$resn_exch{"NC4* NZQ"}=" C4D NZQ"; $resn_exch{"NO4* NZQ"}=" O4D NZQ"; $resn_exch{"NC1* NZQ"}=" C1D NZQ";
$resn_exch{"NC2* NZQ"}=" C2D NZQ"; $resn_exch{"NO2* NZQ"}=" O2D NZQ"; $resn_exch{"NC3* NZQ"}=" C3D NZQ";
$resn_exch{"NO3* NZQ"}=" O3D NZQ"; $resn_exch{" NN1 NZQ"}=" N1N NZQ"; $resn_exch{" NC2 NZQ"}=" C2N NZQ";
$resn_exch{" NC3 NZQ"}=" C3N NZQ"; $resn_exch{" NC7 NZQ"}=" C7N NZQ"; $resn_exch{" NN7 NZQ"}=" N7N NZQ";
$resn_exch{" NO7 NZQ"}=" O7N NZQ"; $resn_exch{" NC4 NZQ"}=" C4N NZQ"; $resn_exch{" NC5 NZQ"}=" C5N NZQ";
$resn_exch{" NO5 NZQ"}=" O5N NZQ"; $resn_exch{" NC6 NZQ"}=" C6N NZQ"; $resn_exch{" NO6 NZQ"}=" O6N NZQ";
$resn_exch{" O5* O2C"}=" O5' O2C"; $resn_exch{" C5* O2C"}=" C5' O2C"; $resn_exch{" C4* O2C"}=" C4' O2C";
$resn_exch{" O4* O2C"}=" O4' O2C"; $resn_exch{" C1* O2C"}=" C1' O2C"; $resn_exch{" C2* O2C"}=" C2' O2C";
$resn_exch{" O2* O2C"}=" O2' O2C"; $resn_exch{" C3* O2C"}=" C3' O2C"; $resn_exch{" O5* OAD"}=" O5' OAD";
$resn_exch{" C5* OAD"}=" C5' OAD"; $resn_exch{" C4* OAD"}=" C4' OAD"; $resn_exch{" O4* OAD"}=" O4' OAD";
$resn_exch{" C3* OAD"}=" C3' OAD"; $resn_exch{" O3* OAD"}=" O3' OAD"; $resn_exch{" C2* OAD"}=" C2' OAD";
$resn_exch{" O2* OAD"}=" O2' OAD"; $resn_exch{" C1* OAD"}=" C1' OAD"; $resn_exch{"RO2* OAD"}=" O2D OAD";
$resn_exch{"RO5* OAD"}=" O5D OAD"; $resn_exch{"RC5* OAD"}=" C5D OAD"; $resn_exch{"RC4* OAD"}=" C4D OAD";
$resn_exch{"RO4* OAD"}=" O4D OAD"; $resn_exch{"RC3* OAD"}=" C3D OAD"; $resn_exch{"RO3* OAD"}=" O3D OAD";
$resn_exch{"RC2* OAD"}=" C2D OAD"; $resn_exch{"RC1* OAD"}=" C1D OAD"; $resn_exch{"RO1* OAD"}=" O1D OAD";
$resn_exch{"RC6* OAD"}=" C6D OAD"; $resn_exch{"RO6* OAD"}=" O6D OAD"; $resn_exch{"RC7* OAD"}=" C7D OAD";
$resn_exch{"CX10 OBP"}=" CXA OBP"; $resn_exch{"CX11 OBP"}=" CXB OBP"; $resn_exch{"CX12 OBP"}=" CXC OBP";
$resn_exch{"AOP1 ODP"}=" O1X ODP"; $resn_exch{"AP2* ODP"}=" P2B ODP"; $resn_exch{"AOP2 ODP"}=" O2X ODP";
$resn_exch{"AOP3 ODP"}=" O3X ODP"; $resn_exch{"AO2* ODP"}=" O2B ODP"; $resn_exch{"AC2* ODP"}=" C2B ODP";
$resn_exch{"AC3* ODP"}=" C3B ODP"; $resn_exch{"AO3* ODP"}=" O3B ODP"; $resn_exch{"AC1* ODP"}=" C1B ODP";
$resn_exch{" AN9 ODP"}=" N9A ODP"; $resn_exch{" AC8 ODP"}=" C8A ODP"; $resn_exch{" AN7 ODP"}=" N7A ODP";
$resn_exch{" AC5 ODP"}=" C5A ODP"; $resn_exch{" AC4 ODP"}=" C4A ODP"; $resn_exch{" AN3 ODP"}=" N3A ODP";
$resn_exch{" AC2 ODP"}=" C2A ODP"; $resn_exch{" AN1 ODP"}=" N1A ODP"; $resn_exch{" AC6 ODP"}=" C6A ODP";
$resn_exch{" AN6 ODP"}=" N6A ODP"; $resn_exch{"AO4* ODP"}=" O4B ODP"; $resn_exch{"AC4* ODP"}=" C4B ODP";
$resn_exch{"AC5* ODP"}=" C5B ODP"; $resn_exch{"AO5* ODP"}=" O5B ODP"; $resn_exch{" AP  ODP"}=" PA  ODP";
$resn_exch{" AO1 ODP"}=" O1A ODP"; $resn_exch{" AO2 ODP"}=" O2A ODP"; $resn_exch{" NP  ODP"}=" PN  ODP";
$resn_exch{" NO1 ODP"}=" O1N ODP"; $resn_exch{" NO2 ODP"}=" O2N ODP"; $resn_exch{"NO5* ODP"}=" O5D ODP";
$resn_exch{"NC5* ODP"}=" C5D ODP"; $resn_exch{"NC4* ODP"}=" C4D ODP"; $resn_exch{"NO4* ODP"}=" O4D ODP";
$resn_exch{"NC1* ODP"}=" C1D ODP"; $resn_exch{"NC2* ODP"}=" C2D ODP"; $resn_exch{"NO2* ODP"}=" O2D ODP";
$resn_exch{"NC3* ODP"}=" C3D ODP"; $resn_exch{"NO3* ODP"}=" O3D ODP"; $resn_exch{" NN1 ODP"}=" N1N ODP";
$resn_exch{" NC2 ODP"}=" C2N ODP"; $resn_exch{" NC3 ODP"}=" C3N ODP"; $resn_exch{" NC7 ODP"}=" C7N ODP";
$resn_exch{" NN7 ODP"}=" N7N ODP"; $resn_exch{" NO7 ODP"}=" O7N ODP"; $resn_exch{" NC4 ODP"}=" C4N ODP";
$resn_exch{" NC5 ODP"}=" C5N ODP"; $resn_exch{" NC6 ODP"}=" C6N ODP"; $resn_exch{" O1P OIP"}=" OP1 OIP";
$resn_exch{" O2P OIP"}=" OP2 OIP"; $resn_exch{" O3P OIP"}=" OP3 OIP"; $resn_exch{" O5* OIP"}=" O5' OIP";
$resn_exch{" C5* OIP"}=" C5' OIP"; $resn_exch{" C4* OIP"}=" C4' OIP"; $resn_exch{" O4* OIP"}=" O4' OIP";
$resn_exch{" C3* OIP"}=" C3' OIP"; $resn_exch{" O3* OIP"}=" O3' OIP"; $resn_exch{" C2* OIP"}=" C2' OIP";
$resn_exch{" C1* OIP"}=" C1' OIP"; $resn_exch{" O6* OLU"}=" O6' OLU"; $resn_exch{" C4* OLU"}=" C4' OLU";
$resn_exch{" N3* OLU"}=" N3' OLU"; $resn_exch{" C2* OLU"}=" C2' OLU"; $resn_exch{" S1* OLU"}=" S1' OLU";
$resn_exch{" C5* OLU"}=" C5' OLU"; $resn_exch{" C1* OMC"}=" C1' OMC"; $resn_exch{" C2* OMC"}=" C2' OMC";
$resn_exch{" O2* OMC"}=" O2' OMC"; $resn_exch{" C3* OMC"}=" C3' OMC"; $resn_exch{" C4* OMC"}=" C4' OMC";
$resn_exch{" O4* OMC"}=" O4' OMC"; $resn_exch{" O3* OMC"}=" O3' OMC"; $resn_exch{" C5* OMC"}=" C5' OMC";
$resn_exch{" O5* OMC"}=" O5' OMC"; $resn_exch{" O1P OMC"}=" OP1 OMC"; $resn_exch{" O2P OMC"}=" OP2 OMC";
$resn_exch{" O3P OMC"}=" OP3 OMC"; $resn_exch{" O1P OMG"}=" OP1 OMG"; $resn_exch{" O2P OMG"}=" OP2 OMG";
$resn_exch{" O3P OMG"}=" OP3 OMG"; $resn_exch{" O5* OMG"}=" O5' OMG"; $resn_exch{" C5* OMG"}=" C5' OMG";
$resn_exch{" C4* OMG"}=" C4' OMG"; $resn_exch{" O4* OMG"}=" O4' OMG"; $resn_exch{" C3* OMG"}=" C3' OMG";
$resn_exch{" O3* OMG"}=" O3' OMG"; $resn_exch{" C2* OMG"}=" C2' OMG"; $resn_exch{" O2* OMG"}=" O2' OMG";
$resn_exch{" C1* OMG"}=" C1' OMG"; $resn_exch{" O5* OMP"}=" O5' OMP"; $resn_exch{" C5* OMP"}=" C5' OMP";
$resn_exch{" C4* OMP"}=" C4' OMP"; $resn_exch{" O4* OMP"}=" O4' OMP"; $resn_exch{" C3* OMP"}=" C3' OMP";
$resn_exch{" O3* OMP"}=" O3' OMP"; $resn_exch{" C2* OMP"}=" C2' OMP"; $resn_exch{" O2* OMP"}=" O2' OMP";
$resn_exch{" C1* OMP"}=" C1' OMP"; $resn_exch{" C1* OMU"}=" C1' OMU"; $resn_exch{" C2* OMU"}=" C2' OMU";
$resn_exch{" O2* OMU"}=" O2' OMU"; $resn_exch{" C3* OMU"}=" C3' OMU"; $resn_exch{" C4* OMU"}=" C4' OMU";
$resn_exch{" O3* OMU"}=" O3' OMU"; $resn_exch{" O4* OMU"}=" O4' OMU"; $resn_exch{" C5* OMU"}=" C5' OMU";
$resn_exch{" O5* OMU"}=" O5' OMU"; $resn_exch{" O1P OMU"}=" OP1 OMU"; $resn_exch{" O2P OMU"}=" OP2 OMU";
$resn_exch{" O3P OMU"}=" OP3 OMU"; $resn_exch{" O3* ONA"}=" O3' ONA"; $resn_exch{" C3* ONA"}=" C3' ONA";
$resn_exch{" C2* ONA"}=" C2' ONA"; $resn_exch{" O2* ONA"}=" O2' ONA"; $resn_exch{" C4* ONA"}=" C4' ONA";
$resn_exch{" C5* ONA"}=" C5' ONA"; $resn_exch{" O5* ONA"}=" O5' ONA"; $resn_exch{" O4* ONA"}=" O4' ONA";
$resn_exch{" C1* ONA"}=" C1' ONA"; $resn_exch{" O3P ONE"}=" OP3 ONE"; $resn_exch{" O1P ONE"}=" OP1 ONE";
$resn_exch{" O2P ONE"}=" OP2 ONE"; $resn_exch{" O5* ONE"}=" O5' ONE"; $resn_exch{" C5* ONE"}=" C5' ONE";
$resn_exch{" C4* ONE"}=" C4' ONE"; $resn_exch{" O4* ONE"}=" O4' ONE"; $resn_exch{" C3* ONE"}=" C3' ONE";
$resn_exch{" C2* ONE"}=" C2' ONE"; $resn_exch{" O2* ONE"}=" O2' ONE"; $resn_exch{" O3* ONE"}=" O3' ONE";
$resn_exch{" C1* ONE"}=" C1' ONE"; $resn_exch{" O3* ONM"}=" O3' ONM"; $resn_exch{" C3* ONM"}=" C3' ONM";
$resn_exch{" C2* ONM"}=" C2' ONM"; $resn_exch{" O2* ONM"}=" O2' ONM"; $resn_exch{" C4* ONM"}=" C4' ONM";
$resn_exch{" C5* ONM"}=" C5' ONM"; $resn_exch{" O5* ONM"}=" O5' ONM"; $resn_exch{" O4* ONM"}=" O4' ONM";
$resn_exch{" C1* ONM"}=" C1' ONM"; $resn_exch{" C1* OR5"}=" C1' OR5"; $resn_exch{" O1* OR5"}=" O1' OR5";
$resn_exch{" C2* OR5"}=" C2' OR5"; $resn_exch{" O2* OR5"}=" O2' OR5"; $resn_exch{" C3* OR5"}=" C3' OR5";
$resn_exch{" O3* OR5"}=" O3' OR5"; $resn_exch{" C4* OR5"}=" C4' OR5"; $resn_exch{" O4* OR5"}=" O4' OR5";
$resn_exch{" C5* OR5"}=" C5' OR5"; $resn_exch{" O5* OR5"}=" O5' OR5"; $resn_exch{" C1* OTG"}=" C1' OTG";
$resn_exch{" C2* OTG"}=" C2' OTG"; $resn_exch{" C3* OTG"}=" C3' OTG"; $resn_exch{" C4* OTG"}=" C4' OTG";
$resn_exch{" C5* OTG"}=" C5' OTG"; $resn_exch{" C6* OTG"}=" C6' OTG"; $resn_exch{" C7* OTG"}=" C7' OTG";
$resn_exch{" C8* OTG"}=" C8' OTG"; $resn_exch{" O8* OTG"}=" O8' OTG"; $resn_exch{" P3* OXT"}=" P3' OXT";
$resn_exch{" O3* OXT"}=" O3' OXT"; $resn_exch{" C3* OXT"}=" C3' OXT"; $resn_exch{" C2* OXT"}=" C2' OXT";
$resn_exch{" O2* OXT"}=" O2' OXT"; $resn_exch{" C1* OXT"}=" C1' OXT"; $resn_exch{" O4* OXT"}=" O4' OXT";
$resn_exch{" C4* OXT"}=" C4' OXT"; $resn_exch{" C5* OXT"}=" C5' OXT"; $resn_exch{" O5* OXT"}=" O5' OXT";
$resn_exch{" O4  OXX"}=" OXT OXX"; $resn_exch{" O1P   P"}=" OP1   P"; $resn_exch{" O2P   P"}=" OP2   P";
$resn_exch{" O3P   P"}=" OP3   P"; $resn_exch{" O1P P1P"}=" OP1 P1P"; $resn_exch{" O2P P1P"}=" OP2 P1P";
$resn_exch{" O5* P1P"}=" O5' P1P"; $resn_exch{" C5* P1P"}=" C5' P1P"; $resn_exch{" C4* P1P"}=" C4' P1P";
$resn_exch{" C3* P1P"}=" C3' P1P"; $resn_exch{" O3* P1P"}=" O3' P1P"; $resn_exch{" O6* P2P"}=" O6' P2P";
$resn_exch{" C5* P2P"}=" C5' P2P"; $resn_exch{" C4* P2P"}=" C4' P2P"; $resn_exch{" O5* P2P"}=" O5' P2P";
$resn_exch{" C3* P2P"}=" C3' P2P"; $resn_exch{" O3* P2P"}=" O3' P2P"; $resn_exch{" C2* P2P"}=" C2' P2P";
$resn_exch{" O2* P2P"}=" O2' P2P"; $resn_exch{" C1* P2P"}=" C1' P2P"; $resn_exch{" O1P P2T"}=" OP1 P2T";
$resn_exch{" O2P P2T"}=" OP2 P2T"; $resn_exch{" O5* P2T"}=" O5' P2T"; $resn_exch{" C5* P2T"}=" C5' P2T";
$resn_exch{" C4* P2T"}=" C4' P2T"; $resn_exch{" O4* P2T"}=" O4' P2T"; $resn_exch{" C1* P2T"}=" C1' P2T";
$resn_exch{" C2* P2T"}=" C2' P2T"; $resn_exch{" O2* P2T"}=" O2' P2T"; $resn_exch{" C3* P2T"}=" C3' P2T";
$resn_exch{" O3* P2T"}=" O3' P2T"; $resn_exch{" O3P P2T"}=" OP3 P2T"; $resn_exch{" C1* P2U"}=" C1' P2U";
$resn_exch{" C2* P2U"}=" C2' P2U"; $resn_exch{" C3* P2U"}=" C3' P2U"; $resn_exch{" C4* P2U"}=" C4' P2U";
$resn_exch{" O3* P2U"}=" O3' P2U"; $resn_exch{" O4* P2U"}=" O4' P2U"; $resn_exch{" C5* P2U"}=" C5' P2U";
$resn_exch{" O5* P2U"}=" O5' P2U"; $resn_exch{" O1P P2U"}=" OP1 P2U"; $resn_exch{" O2P P2U"}=" OP2 P2U";
$resn_exch{" O3P P2U"}=" OP3 P2U"; $resn_exch{" O5* P5A"}=" O5' P5A"; $resn_exch{" C5* P5A"}=" C5' P5A";
$resn_exch{" C4* P5A"}=" C4' P5A"; $resn_exch{" O4* P5A"}=" O4' P5A"; $resn_exch{" C1* P5A"}=" C1' P5A";
$resn_exch{" C2* P5A"}=" C2' P5A"; $resn_exch{" O2* P5A"}=" O2' P5A"; $resn_exch{" C3* P5A"}=" C3' P5A";
$resn_exch{" O3* P5A"}=" O3' P5A"; $resn_exch{" O1P P5P"}=" OP1 P5P"; $resn_exch{" O2P P5P"}=" OP2 P5P";
$resn_exch{" O3P P5P"}=" OP3 P5P"; $resn_exch{" C5* PA7"}=" C5' PA7"; $resn_exch{" O5* PA7"}=" O5' PA7";
$resn_exch{" N5* PA7"}=" N5' PA7"; $resn_exch{" C4* PA7"}=" C4' PA7"; $resn_exch{" O4* PA7"}=" O4' PA7";
$resn_exch{" C3* PA7"}=" C3' PA7"; $resn_exch{" O3* PA7"}=" O3' PA7"; $resn_exch{" C2* PA7"}=" C2' PA7";
$resn_exch{" O2* PA7"}=" O2' PA7"; $resn_exch{" C1* PA7"}=" C1' PA7"; $resn_exch{" AP  PAD"}=" PA  PAD";
$resn_exch{" AO1 PAD"}=" O1A PAD"; $resn_exch{" AO2 PAD"}=" O2A PAD"; $resn_exch{"AO5* PAD"}=" O5B PAD";
$resn_exch{"AC5* PAD"}=" C5B PAD"; $resn_exch{"AC4* PAD"}=" C4B PAD"; $resn_exch{"AO4* PAD"}=" O4B PAD";
$resn_exch{"AC3* PAD"}=" C3B PAD"; $resn_exch{"AO3* PAD"}=" O3B PAD"; $resn_exch{"AC2* PAD"}=" C2B PAD";
$resn_exch{"AO2* PAD"}=" O2B PAD"; $resn_exch{"AC1* PAD"}=" C1B PAD"; $resn_exch{" AN9 PAD"}=" N9A PAD";
$resn_exch{" AC8 PAD"}=" C8A PAD"; $resn_exch{" AN7 PAD"}=" N7A PAD"; $resn_exch{" AC5 PAD"}=" C5A PAD";
$resn_exch{" AC6 PAD"}=" C6A PAD"; $resn_exch{" AN6 PAD"}=" N6A PAD"; $resn_exch{" AN1 PAD"}=" N1A PAD";
$resn_exch{" AC2 PAD"}=" C2A PAD"; $resn_exch{" AN3 PAD"}=" N3A PAD"; $resn_exch{" AC4 PAD"}=" C4A PAD";
$resn_exch{" NP  PAD"}=" PN  PAD"; $resn_exch{" NO1 PAD"}=" O1N PAD"; $resn_exch{" NO2 PAD"}=" O2N PAD";
$resn_exch{"NO5* PAD"}=" O5D PAD"; $resn_exch{"NC5* PAD"}=" C5D PAD"; $resn_exch{"NC4* PAD"}=" C4D PAD";
$resn_exch{"NO4* PAD"}=" O4D PAD"; $resn_exch{"NC3* PAD"}=" C3D PAD"; $resn_exch{"NO3* PAD"}=" O3D PAD";
$resn_exch{"NC2* PAD"}=" C2D PAD"; $resn_exch{"NO2* PAD"}=" O2D PAD"; $resn_exch{"NC1* PAD"}=" C1D PAD";
$resn_exch{" NC1 PAD"}=" C1N PAD"; $resn_exch{" NN2 PAD"}=" N2N PAD"; $resn_exch{" NC3 PAD"}=" C3N PAD";
$resn_exch{" NC7 PAD"}=" C7N PAD"; $resn_exch{" NO7 PAD"}=" O7N PAD"; $resn_exch{" NN7 PAD"}=" N7N PAD";
$resn_exch{" NC4 PAD"}=" C4N PAD"; $resn_exch{" NC5 PAD"}=" C5N PAD"; $resn_exch{" NC6 PAD"}=" C6N PAD";
$resn_exch{" O5* PAJ"}=" O5' PAJ"; $resn_exch{" C5* PAJ"}=" C5' PAJ"; $resn_exch{" C4* PAJ"}=" C4' PAJ";
$resn_exch{" O4* PAJ"}=" O4' PAJ"; $resn_exch{" C3* PAJ"}=" C3' PAJ"; $resn_exch{" O3* PAJ"}=" O3' PAJ";
$resn_exch{" C2* PAJ"}=" C2' PAJ"; $resn_exch{" O2* PAJ"}=" O2' PAJ"; $resn_exch{" C1* PAJ"}=" C1' PAJ";
$resn_exch{" O5* PAP"}=" O5' PAP"; $resn_exch{" C5* PAP"}=" C5' PAP"; $resn_exch{" C4* PAP"}=" C4' PAP";
$resn_exch{" O4* PAP"}=" O4' PAP"; $resn_exch{" C3* PAP"}=" C3' PAP"; $resn_exch{" O3* PAP"}=" O3' PAP";
$resn_exch{" C2* PAP"}=" C2' PAP"; $resn_exch{" O2* PAP"}=" O2' PAP"; $resn_exch{" C1* PAP"}=" C1' PAP";
$resn_exch{" O5* PAU"}=" O5' PAU"; $resn_exch{" O6* PAU"}=" O6' PAU"; $resn_exch{" O5* PAX"}=" O5' PAX";
$resn_exch{" C5* PAX"}=" C5' PAX"; $resn_exch{" C4* PAX"}=" C4' PAX"; $resn_exch{" O4* PAX"}=" O4' PAX";
$resn_exch{" C1* PAX"}=" C1' PAX"; $resn_exch{" C2* PAX"}=" C2' PAX"; $resn_exch{" O2* PAX"}=" O2' PAX";
$resn_exch{" C3* PAX"}=" C3' PAX"; $resn_exch{" O3* PAX"}=" O3' PAX"; $resn_exch{" N1* PBO"}=" N1' PBO";
$resn_exch{" O5* PBT"}=" O5' PBT"; $resn_exch{" C5* PBT"}=" C5' PBT"; $resn_exch{" C4* PBT"}=" C4' PBT";
$resn_exch{" O4* PBT"}=" O4' PBT"; $resn_exch{" C3* PBT"}=" C3' PBT"; $resn_exch{" O3* PBT"}=" O3' PBT";
$resn_exch{" C2* PBT"}=" C2' PBT"; $resn_exch{" C1* PBT"}=" C1' PBT"; $resn_exch{"1C22 PC3"}=" CX5 PC3";
$resn_exch{" C1* PCD"}=" C1' PCD"; $resn_exch{" C2* PCD"}=" C2D PCD"; $resn_exch{" O2* PCD"}=" O2' PCD";
$resn_exch{" C3* PCD"}=" C3' PCD"; $resn_exch{" C4* PCD"}=" C4D PCD"; $resn_exch{" O4* PCD"}=" O4D PCD";
$resn_exch{" O3* PCD"}=" O3' PCD"; $resn_exch{" C5* PCD"}=" C5' PCD"; $resn_exch{" O5* PCD"}=" O5' PCD";
$resn_exch{" O5* PCG"}=" O5' PCG"; $resn_exch{" C5* PCG"}=" C5' PCG"; $resn_exch{" C4* PCG"}=" C4' PCG";
$resn_exch{" O4* PCG"}=" O4' PCG"; $resn_exch{" C3* PCG"}=" C3' PCG"; $resn_exch{" O3* PCG"}=" O3' PCG";
$resn_exch{" C2* PCG"}=" C2' PCG"; $resn_exch{" O2* PCG"}=" O2' PCG"; $resn_exch{" C1* PCG"}=" C1' PCG";
$resn_exch{" N_A PCU"}=" NA  PCU"; $resn_exch{" N_B PCU"}=" NB  PCU"; $resn_exch{" N_C PCU"}=" NC  PCU";
$resn_exch{" N_D PCU"}=" ND  PCU"; $resn_exch{" O\"  PD7"}=" O'' PD7"; $resn_exch{" C\"  PD7"}=" C'' PD7";
$resn_exch{"C1\"  PD7"}="C1'' PD7"; $resn_exch{"C2\"  PD7"}="C2'' PD7"; $resn_exch{"C3\"  PD7"}="C3'' PD7";
$resn_exch{"C4\"  PD7"}="C4'' PD7"; $resn_exch{"C5\"  PD7"}="C5'' PD7"; $resn_exch{"C6\"  PD7"}="C6'' PD7";
$resn_exch{" O3P PDU"}=" OP3 PDU"; $resn_exch{" O1P PDU"}=" OP1 PDU"; $resn_exch{" O2P PDU"}=" OP2 PDU";
$resn_exch{" O5* PDU"}=" O5' PDU"; $resn_exch{" C5* PDU"}=" C5' PDU"; $resn_exch{" C4* PDU"}=" C4' PDU";
$resn_exch{" O4* PDU"}=" O4' PDU"; $resn_exch{" C3* PDU"}=" C3' PDU"; $resn_exch{" O3* PDU"}=" O3' PDU";
$resn_exch{" C2* PDU"}=" C2' PDU"; $resn_exch{" C1* PDU"}=" C1' PDU"; $resn_exch{"O1S* PDX"}=" O1X PDX";
$resn_exch{"O2S* PDX"}=" O2X PDX"; $resn_exch{"O3S* PDX"}=" O3X PDX"; $resn_exch{" S*  PDX"}=" S'  PDX";
$resn_exch{" C4* PE2"}=" C4' PE2"; $resn_exch{" O4* PE2"}=" O4' PE2"; $resn_exch{" C3* PE2"}=" C3' PE2";
$resn_exch{" O3* PE2"}=" O3' PE2"; $resn_exch{" C2* PE2"}=" C2' PE2"; $resn_exch{" C1* PE2"}=" C1' PE2";
$resn_exch{"1OP  PE6"}=" OP1 PE6"; $resn_exch{"2OP  PE6"}=" OP2 PE6"; $resn_exch{"3OP  PE6"}=" OP3 PE6";
$resn_exch{" O5* PED"}=" O5' PED"; $resn_exch{" C2* PED"}=" C2' PED"; $resn_exch{" C5* PED"}=" C5' PED";
$resn_exch{" C4* PED"}=" C4' PED"; $resn_exch{" O4* PED"}=" O4' PED"; $resn_exch{" C3* PED"}=" C3' PED";
$resn_exch{" C1* PED"}=" C1' PED"; $resn_exch{" O3* PED"}=" O3' PED"; $resn_exch{" C1* PEL"}=" C1' PEL";
$resn_exch{" C2* PEL"}=" C2' PEL"; $resn_exch{" C3* PEL"}=" C3' PEL"; $resn_exch{" C4* PEL"}=" C4' PEL";
$resn_exch{" C5* PEL"}=" C5' PEL"; $resn_exch{" C6* PEL"}=" C6' PEL"; $resn_exch{" O1  PG1"}=" OXT PG1";
$resn_exch{" O1P PG7"}=" OP1 PG7"; $resn_exch{" O2P PG7"}=" OP2 PG7"; $resn_exch{" O5* PG7"}=" O5' PG7";
$resn_exch{" O3* PG7"}=" O3' PG7"; $resn_exch{" C1* PG7"}=" C1' PG7"; $resn_exch{" C2* PG7"}=" C2' PG7";
$resn_exch{" C3* PG7"}=" C3' PG7"; $resn_exch{" C4* PG7"}=" C4' PG7"; $resn_exch{" C5* PG7"}=" C5' PG7";
$resn_exch{" O4* PG7"}=" O4' PG7"; $resn_exch{" O3P PG7"}=" OP3 PG7"; $resn_exch{" O1P PGN"}=" OP1 PGN";
$resn_exch{" O2P PGN"}=" OP2 PGN"; $resn_exch{" O3P PGN"}=" OP3 PGN"; $resn_exch{" O5* PGN"}=" O5' PGN";
$resn_exch{" C5* PGN"}=" C5' PGN"; $resn_exch{" C4* PGN"}=" C4' PGN"; $resn_exch{" O4* PGN"}=" O4' PGN";
$resn_exch{" C3* PGN"}=" C3' PGN"; $resn_exch{" O3* PGN"}=" O3' PGN"; $resn_exch{" C2* PGN"}=" C2' PGN";
$resn_exch{" C1* PGN"}=" C1' PGN"; $resn_exch{" O1P PGP"}=" OP1 PGP"; $resn_exch{" O2P PGP"}=" OP2 PGP";
$resn_exch{" O3P PGP"}=" OP3 PGP"; $resn_exch{" O5* PGP"}=" O5' PGP"; $resn_exch{" C5* PGP"}=" C5' PGP";
$resn_exch{" C4* PGP"}=" C4' PGP"; $resn_exch{" O4* PGP"}=" O4' PGP"; $resn_exch{" C3* PGP"}=" C3' PGP";
$resn_exch{" O3* PGP"}=" O3' PGP"; $resn_exch{" C2* PGP"}=" C2' PGP"; $resn_exch{" O2* PGP"}=" O2' PGP";
$resn_exch{" C1* PGP"}=" C1' PGP"; $resn_exch{" O5* PGS"}=" O5' PGS"; $resn_exch{" C5* PGS"}=" C5' PGS";
$resn_exch{" C4* PGS"}=" C4' PGS"; $resn_exch{" O4* PGS"}=" O4' PGS"; $resn_exch{" C3* PGS"}=" C3' PGS";
$resn_exch{" O3* PGS"}=" O3' PGS"; $resn_exch{" C2* PGS"}=" C2' PGS"; $resn_exch{" O2* PGS"}=" O2' PGS";
$resn_exch{" C1* PGS"}=" C1' PGS"; $resn_exch{" C1  PGW"}=" CAD PGW"; $resn_exch{" O1  PGW"}=" OAE PGW";
$resn_exch{" O2  PGW"}=" OAF PGW"; $resn_exch{" P1  PGW"}=" P   PGW"; $resn_exch{" C2  PGW"}=" C01 PGW";
$resn_exch{" C3  PGW"}=" C1  PGW"; $resn_exch{" O3  PGW"}=" O01 PGW"; $resn_exch{" C4  PGW"}=" C02 PGW";
$resn_exch{" C5  PGW"}=" C2  PGW"; $resn_exch{" O4  PGW"}=" O02 PGW"; $resn_exch{" C6  PGW"}=" C03 PGW";
$resn_exch{" C7  PGW"}=" C3  PGW"; $resn_exch{" O5  PGW"}=" O03 PGW"; $resn_exch{" C8  PGW"}=" C04 PGW";
$resn_exch{" C9  PGW"}=" C4  PGW"; $resn_exch{" O6  PGW"}=" O04 PGW"; $resn_exch{" C10 PGW"}=" C05 PGW";
$resn_exch{" C11 PGW"}=" C5  PGW"; $resn_exch{" C12 PGW"}=" C06 PGW"; $resn_exch{" C13 PGW"}=" C6  PGW";
$resn_exch{" C14 PGW"}=" C07 PGW"; $resn_exch{" C15 PGW"}=" C7  PGW"; $resn_exch{" C16 PGW"}=" C08 PGW";
$resn_exch{" C17 PGW"}=" C8  PGW"; $resn_exch{" C18 PGW"}=" C09 PGW"; $resn_exch{" C19 PGW"}=" C9  PGW";
$resn_exch{" C20 PGW"}=" C10 PGW"; $resn_exch{" C21 PGW"}=" C11 PGW"; $resn_exch{" O7  PGW"}=" O11 PGW";
$resn_exch{" C22 PGW"}=" C12 PGW"; $resn_exch{" O8  PGW"}=" O12 PGW"; $resn_exch{" C23 PGW"}=" C13 PGW";
$resn_exch{" O9  PGW"}=" O13 PGW"; $resn_exch{" C24 PGW"}=" C14 PGW"; $resn_exch{" O10 PGW"}=" O14 PGW";
$resn_exch{" C25 PGW"}=" C15 PGW"; $resn_exch{" C26 PGW"}=" C16 PGW"; $resn_exch{" C27 PGW"}=" C17 PGW";
$resn_exch{" C28 PGW"}=" C18 PGW"; $resn_exch{" C29 PGW"}=" C19 PGW"; $resn_exch{" C30 PGW"}=" C20 PGW";
$resn_exch{" C31 PGW"}=" C21 PGW"; $resn_exch{" C32 PGW"}=" C22 PGW"; $resn_exch{" C33 PGW"}=" C23 PGW";
$resn_exch{" C34 PGW"}=" C24 PGW"; $resn_exch{" C35 PGW"}=" C25 PGW"; $resn_exch{" C36 PGW"}=" C26 PGW";
$resn_exch{" C37 PGW"}=" C27 PGW"; $resn_exch{" C38 PGW"}=" C28 PGW"; $resn_exch{" C39 PGW"}=" C29 PGW";
$resn_exch{" C40 PGW"}=" C30 PGW"; $resn_exch{" N_A PHO"}=" NA  PHO"; $resn_exch{" N_B PHO"}=" NB  PHO";
$resn_exch{" N_C PHO"}=" NC  PHO"; $resn_exch{" N_D PHO"}=" ND  PHO"; $resn_exch{"1CD1 PHP"}=" CDB PHP";
$resn_exch{"2CD1 PHP"}=" CDL PHP"; $resn_exch{"1CE1 PHP"}=" CEB PHP"; $resn_exch{"2CE1 PHP"}=" CEL PHP";
$resn_exch{"1CG2 PHP"}=" CGL PHP"; $resn_exch{"2CG2 PHP"}=" CGM PHP"; $resn_exch{"1CD3 PHP"}=" CDD PHP";
$resn_exch{"2CD3 PHP"}=" CDN PHP"; $resn_exch{"1CE3 PHP"}=" CED PHP"; $resn_exch{"2CE3 PHP"}=" CEN PHP";
$resn_exch{"1CD4 PHP"}=" CDE PHP"; $resn_exch{"2CD4 PHP"}=" CDO PHP"; $resn_exch{"1CE4 PHP"}=" CEE PHP";
$resn_exch{"2CE4 PHP"}=" CEO PHP"; $resn_exch{"1CG5 PHP"}=" CGF PHP"; $resn_exch{"2CG5 PHP"}=" CGG PHP";
$resn_exch{"1CD6 PHP"}=" CDG PHP"; $resn_exch{"1CE6 PHP"}=" CEG PHP"; $resn_exch{"2CE6 PHP"}=" CEQ PHP";
$resn_exch{"2CD6 PHP"}=" CDQ PHP"; $resn_exch{"CR10 PII"}=" CRA PII"; $resn_exch{"CR11 PII"}=" CRB PII";
$resn_exch{"CR12 PII"}=" CRC PII"; $resn_exch{"CR13 PII"}=" CRD PII"; $resn_exch{"CR14 PII"}=" CRE PII";
$resn_exch{"CR15 PII"}=" CRF PII"; $resn_exch{"CR16 PII"}=" CRG PII"; $resn_exch{"CL10 PII"}=" CLA PII";
$resn_exch{"CL11 PII"}=" CLB PII"; $resn_exch{"CL12 PII"}=" CLC PII"; $resn_exch{"CL13 PII"}=" CLD PII";
$resn_exch{"CL14 PII"}=" CLE PII"; $resn_exch{"CL15 PII"}=" CLF PII"; $resn_exch{"'C10 PLC"}=" CA' PLC";
$resn_exch{"'C11 PLC"}=" CB' PLC"; $resn_exch{" C\"  PLC"}=" CB  PLC"; $resn_exch{" C1\" PLC"}=" C1B PLC";
$resn_exch{" C2\" PLC"}=" C2B PLC"; $resn_exch{" C3\" PLC"}=" C3B PLC"; $resn_exch{" C4\" PLC"}=" C4B PLC";
$resn_exch{" C5\" PLC"}=" C5B PLC"; $resn_exch{" C6\" PLC"}=" C6B PLC"; $resn_exch{" C7\" PLC"}=" C7B PLC";
$resn_exch{" C8\" PLC"}=" C8B PLC"; $resn_exch{" C9\" PLC"}=" C9B PLC"; $resn_exch{"\"C10 PLC"}=" CAA PLC";
$resn_exch{"\"C11 PLC"}=" CBA PLC"; $resn_exch{" O\"  PLC"}=" OB  PLC"; $resn_exch{"'C10 PLD"}=" CA' PLD";
$resn_exch{"'C11 PLD"}=" CB' PLD"; $resn_exch{" C\"  PLD"}=" CB  PLD"; $resn_exch{" C1\" PLD"}=" C1B PLD";
$resn_exch{" C2\" PLD"}=" C2B PLD"; $resn_exch{" C3\" PLD"}=" C3B PLD"; $resn_exch{" C4\" PLD"}=" C4B PLD";
$resn_exch{" C5\" PLD"}=" C5B PLD"; $resn_exch{" C6\" PLD"}=" C6B PLD"; $resn_exch{" C7\" PLD"}=" C7B PLD";
$resn_exch{" C8\" PLD"}=" C8B PLD"; $resn_exch{" C9\" PLD"}=" C9B PLD"; $resn_exch{"\"C10 PLD"}=" CAA PLD";
$resn_exch{"\"C11 PLD"}=" CBA PLD"; $resn_exch{" O\"  PLD"}=" OB  PLD"; $resn_exch{"'C12 PLD"}=" CC' PLD";
$resn_exch{"'C13 PLD"}=" CD' PLD"; $resn_exch{"'C14 PLD"}=" CE' PLD"; $resn_exch{"'C15 PLD"}=" CF' PLD";
$resn_exch{"'C16 PLD"}=" CG' PLD"; $resn_exch{"'C17 PLD"}=" CH' PLD"; $resn_exch{"'C18 PLD"}=" CI' PLD";
$resn_exch{"'C19 PLD"}=" CJ' PLD"; $resn_exch{"'C20 PLD"}=" CK' PLD"; $resn_exch{"\"C12 PLD"}=" CCA PLD";
$resn_exch{"\"C13 PLD"}=" CDA PLD"; $resn_exch{"\"C14 PLD"}=" CEA PLD"; $resn_exch{"\"C15 PLD"}=" CFA PLD";
$resn_exch{"\"C16 PLD"}=" CGA PLD"; $resn_exch{"\"C17 PLD"}=" CHA PLD"; $resn_exch{"\"C18 PLD"}=" CIA PLD";
$resn_exch{"\"C19 PLD"}=" CJA PLD"; $resn_exch{"\"C20 PLD"}=" CKA PLD"; $resn_exch{" O   PM3"}=" OXT PM3";
$resn_exch{" OXT PM3"}=" O   PM3"; $resn_exch{" C2* PMD"}=" C2B PMD"; $resn_exch{" C4* PMD"}=" C4B PMD";
$resn_exch{" C6* PMD"}=" C6B PMD"; $resn_exch{" C7* PMD"}=" C7B PMD"; $resn_exch{" C9* PMD"}=" C9B PMD";
$resn_exch{"'C10 PMD"}=" CA' PMD"; $resn_exch{"'C11 PMD"}=" CB' PMD"; $resn_exch{"'C12 PMD"}=" CC' PMD";
$resn_exch{" N1* PMD"}=" N1' PMD"; $resn_exch{" N2* PMD"}=" N2' PMD"; $resn_exch{" N3* PMD"}=" N3' PMD";
$resn_exch{" N4* PMD"}=" N4' PMD"; $resn_exch{" N5* PMD"}=" N5' PMD"; $resn_exch{" N8* PMD"}=" N8' PMD";
$resn_exch{" O5* PMO"}=" O5' PMO"; $resn_exch{" C5* PMO"}=" C5' PMO"; $resn_exch{" C4* PMO"}=" C4' PMO";
$resn_exch{" O4* PMO"}=" O4' PMO"; $resn_exch{" C3* PMO"}=" C3' PMO"; $resn_exch{" O3* PMO"}=" O3' PMO";
$resn_exch{" C2* PMO"}=" C2' PMO"; $resn_exch{" O2* PMO"}=" O2' PMO"; $resn_exch{" C1* PMO"}=" C1' PMO";
$resn_exch{"1OP' PMT"}=" O11 PMT"; $resn_exch{"3OP' PMT"}=" O12 PMT"; $resn_exch{"2OP' PMT"}=" O13 PMT";
$resn_exch{" O2P PMT"}=" OP2 PMT"; $resn_exch{" O1P PMT"}=" OP1 PMT"; $resn_exch{" O3P PMT"}=" OP3 PMT";
$resn_exch{" O5* PMT"}=" O5B PMT"; $resn_exch{" C5* PMT"}=" C5B PMT"; $resn_exch{" C4* PMT"}=" C4B PMT";
$resn_exch{" O4* PMT"}=" O4B PMT"; $resn_exch{" C3* PMT"}=" C3B PMT"; $resn_exch{" O3* PMT"}=" O3' PMT";
$resn_exch{" C2* PMT"}=" C2B PMT"; $resn_exch{" O2* PMT"}=" O2' PMT"; $resn_exch{" C1* PMT"}=" C1B PMT";
$resn_exch{" N_A PNI"}=" NA  PNI"; $resn_exch{" N_B PNI"}=" NB  PNI"; $resn_exch{" N_C PNI"}=" NC  PNI";
$resn_exch{" N_D PNI"}=" ND  PNI"; $resn_exch{"1C11 POH"}=" CX3 POH"; $resn_exch{"1C7  POH"}=" C71 POH";
$resn_exch{"1C8  POH"}=" C81 POH"; $resn_exch{"1C9  POH"}=" C91 POH"; $resn_exch{"1C13 POH"}=" CXN POH";
$resn_exch{"1C12 POH"}=" CXD POH"; $resn_exch{"1N10 POH"}=" NXT POH"; $resn_exch{" N_A POH"}=" NA  POH";
$resn_exch{" N_B POH"}=" NB  POH"; $resn_exch{" N_C POH"}=" NC  POH"; $resn_exch{" N_D POH"}=" ND  POH";
$resn_exch{"2C11 POH"}=" CX4 POH"; $resn_exch{"2C7  POH"}=" C72 POH"; $resn_exch{"2C8  POH"}=" C82 POH";
$resn_exch{"2C9  POH"}=" C92 POH"; $resn_exch{"2C13 POH"}=" CXO POH"; $resn_exch{"2C12 POH"}=" CXE POH";
$resn_exch{"2N10 POH"}=" NXU POH"; $resn_exch{"3C11 POH"}=" CX5 POH"; $resn_exch{"3C7  POH"}=" C73 POH";
$resn_exch{"3C8  POH"}=" C83 POH"; $resn_exch{"3C9  POH"}=" C93 POH"; $resn_exch{"3C13 POH"}=" CXP POH";
$resn_exch{"3C12 POH"}=" CXF POH"; $resn_exch{"3N10 POH"}=" NXV POH"; $resn_exch{"4C11 POH"}=" CX6 POH";
$resn_exch{"4C7  POH"}=" C74 POH"; $resn_exch{"4C8  POH"}=" C84 POH"; $resn_exch{"4C9  POH"}=" C94 POH";
$resn_exch{"4C13 POH"}=" CXQ POH"; $resn_exch{"4C12 POH"}=" CXG POH"; $resn_exch{"4N10 POH"}=" NXW POH";
$resn_exch{" N_A POR"}=" NA  POR"; $resn_exch{" N_B POR"}=" NB  POR"; $resn_exch{" N_C POR"}=" NC  POR";
$resn_exch{" N_D POR"}=" ND  POR"; $resn_exch{" N_A PP9"}=" NA  PP9"; $resn_exch{" N_B PP9"}=" NB  PP9";
$resn_exch{" N_C PP9"}=" NC  PP9"; $resn_exch{" N_D PP9"}=" ND  PP9"; $resn_exch{" O5* PPS"}=" O5' PPS";
$resn_exch{" C5* PPS"}=" C5' PPS"; $resn_exch{" C4* PPS"}=" C4' PPS"; $resn_exch{" O4* PPS"}=" O4' PPS";
$resn_exch{" C1* PPS"}=" C1' PPS"; $resn_exch{" C2* PPS"}=" C2' PPS"; $resn_exch{" O2* PPS"}=" O2' PPS";
$resn_exch{" C3* PPS"}=" C3' PPS"; $resn_exch{" O3* PPS"}=" O3' PPS"; $resn_exch{" O1P PPU"}=" OP1 PPU";
$resn_exch{" O2P PPU"}=" OP2 PPU"; $resn_exch{" O3P PPU"}=" OP3 PPU"; $resn_exch{" O5* PPU"}=" O5' PPU";
$resn_exch{" C5* PPU"}=" C5' PPU"; $resn_exch{" C4* PPU"}=" C4' PPU"; $resn_exch{" O4* PPU"}=" O4' PPU";
$resn_exch{" C3* PPU"}=" C3' PPU"; $resn_exch{" N3* PPU"}=" N3' PPU"; $resn_exch{" C2* PPU"}=" C2' PPU";
$resn_exch{" O2* PPU"}=" O2' PPU"; $resn_exch{" C1* PPU"}=" C1' PPU"; $resn_exch{" O1P PPW"}=" OP1 PPW";
$resn_exch{" O2P PPW"}=" OP2 PPW"; $resn_exch{" O5* PPW"}=" O5' PPW"; $resn_exch{" C2* PPW"}=" C2' PPW";
$resn_exch{" C5* PPW"}=" C5' PPW"; $resn_exch{" C4* PPW"}=" C4' PPW"; $resn_exch{" O4* PPW"}=" O4' PPW";
$resn_exch{" C1* PPW"}=" C1' PPW"; $resn_exch{" C3* PPW"}=" C3' PPW"; $resn_exch{" O3* PPW"}=" O3' PPW";
$resn_exch{" O3P PPW"}=" OP3 PPW"; $resn_exch{" O3P PQ1"}=" OP3 PQ1"; $resn_exch{" O2P PQ1"}=" OP2 PQ1";
$resn_exch{" O5* PQ1"}=" O5' PQ1"; $resn_exch{" C5* PQ1"}=" C5' PQ1"; $resn_exch{" C4* PQ1"}=" C4' PQ1";
$resn_exch{" O4* PQ1"}=" O4' PQ1"; $resn_exch{" C1* PQ1"}=" C1' PQ1"; $resn_exch{" C2* PQ1"}=" C2' PQ1";
$resn_exch{" O2* PQ1"}=" O2' PQ1"; $resn_exch{" C3* PQ1"}=" C3' PQ1"; $resn_exch{" O3* PQ1"}=" O3' PQ1";
$resn_exch{" O1P PQ1"}=" OP1 PQ1"; $resn_exch{" OR  PR0"}=" OXT PR0"; $resn_exch{" O1P PR5"}=" OP1 PR5";
$resn_exch{" O2P PR5"}=" OP2 PR5"; $resn_exch{" O5* PR5"}=" O5' PR5"; $resn_exch{" C5* PR5"}=" C5' PR5";
$resn_exch{" C4* PR5"}=" C4' PR5"; $resn_exch{" O4* PR5"}=" O4' PR5"; $resn_exch{" C1* PR5"}=" C1' PR5";
$resn_exch{" C3* PR5"}=" C3' PR5"; $resn_exch{" O3* PR5"}=" O3' PR5"; $resn_exch{" C2* PR5"}=" C2' PR5";
$resn_exch{" O2* PR5"}=" O2' PR5"; $resn_exch{" O3P PR5"}=" OP3 PR5"; $resn_exch{" N1* PRD"}=" N1' PRD";
$resn_exch{" N8* PRD"}=" N8' PRD"; $resn_exch{" C7* PRD"}=" C7B PRD"; $resn_exch{" C6* PRD"}=" C6B PRD";
$resn_exch{" N6* PRD"}=" N6' PRD"; $resn_exch{" C5* PRD"}=" C5B PRD"; $resn_exch{" C4* PRD"}=" C4B PRD";
$resn_exch{" N4* PRD"}=" N4' PRD"; $resn_exch{" N3* PRD"}=" N3' PRD"; $resn_exch{" C2* PRD"}=" C2B PRD";
$resn_exch{" N2* PRD"}=" N2' PRD"; $resn_exch{"1O7  PRE"}=" O71 PRE"; $resn_exch{"2O7  PRE"}=" O72 PRE";
$resn_exch{"1O2' PRE"}=" O'L PRE"; $resn_exch{"2O2' PRE"}=" O'M PRE"; $resn_exch{" C1* PRH"}=" C1' PRH";
$resn_exch{" C2* PRH"}=" C2' PRH"; $resn_exch{" O2* PRH"}=" O2' PRH"; $resn_exch{" C3* PRH"}=" C3' PRH";
$resn_exch{" O3* PRH"}=" O3' PRH"; $resn_exch{" C4* PRH"}=" C4' PRH"; $resn_exch{" O4* PRH"}=" O4' PRH";
$resn_exch{" C5* PRH"}=" C5' PRH"; $resn_exch{" O5* PRH"}=" O5' PRH"; $resn_exch{" C1* PRN"}=" C1' PRN";
$resn_exch{" C2* PRN"}=" C2' PRN"; $resn_exch{" C3* PRN"}=" C3' PRN"; $resn_exch{" O3* PRN"}=" O3' PRN";
$resn_exch{" C4* PRN"}=" C4' PRN"; $resn_exch{" O4* PRN"}=" O4' PRN"; $resn_exch{" C5* PRN"}=" C5' PRN";
$resn_exch{" O5* PRN"}=" O5' PRN"; $resn_exch{" O1P PRN"}=" OP1 PRN"; $resn_exch{" O2P PRN"}=" OP2 PRN";
$resn_exch{" O3P PRN"}=" OP3 PRN"; $resn_exch{" O5* PRT"}=" O5' PRT"; $resn_exch{" C5* PRT"}=" C5' PRT";
$resn_exch{" C4* PRT"}=" C4' PRT"; $resn_exch{" O4* PRT"}=" O4' PRT"; $resn_exch{" C3* PRT"}=" C3' PRT";
$resn_exch{" O3* PRT"}=" O3' PRT"; $resn_exch{" C2* PRT"}=" C2' PRT"; $resn_exch{" O2* PRT"}=" O2' PRT";
$resn_exch{" C1* PRT"}=" C1' PRT"; $resn_exch{" O2* PRX"}=" O2' PRX"; $resn_exch{" C2* PRX"}=" C2' PRX";
$resn_exch{" C1* PRX"}=" C1' PRX"; $resn_exch{" O4* PRX"}=" O4' PRX"; $resn_exch{" C4* PRX"}=" C4' PRX";
$resn_exch{" C5* PRX"}=" C5' PRX"; $resn_exch{" O5* PRX"}=" O5' PRX"; $resn_exch{" C3* PRX"}=" C3' PRX";
$resn_exch{" O3* PRX"}=" O3' PRX"; $resn_exch{" O1P PST"}=" OP1 PST"; $resn_exch{" O2P PST"}=" OP2 PST";
$resn_exch{" O5* PST"}=" O5' PST"; $resn_exch{" C5* PST"}=" C5' PST"; $resn_exch{" C4* PST"}=" C4' PST";
$resn_exch{" O4* PST"}=" O4' PST"; $resn_exch{" C3* PST"}=" C3' PST"; $resn_exch{" O3* PST"}=" O3' PST";
$resn_exch{" C2* PST"}=" C2' PST"; $resn_exch{" C1* PST"}=" C1' PST"; $resn_exch{" C1* PSU"}=" C1' PSU";
$resn_exch{" C2* PSU"}=" C2' PSU"; $resn_exch{" O2* PSU"}=" O2' PSU"; $resn_exch{" C3* PSU"}=" C3' PSU";
$resn_exch{" C4* PSU"}=" C4' PSU"; $resn_exch{" O3* PSU"}=" O3' PSU"; $resn_exch{" O4* PSU"}=" O4' PSU";
$resn_exch{" C5* PSU"}=" C5' PSU"; $resn_exch{" O5* PSU"}=" O5' PSU"; $resn_exch{" O1P PSU"}=" OP1 PSU";
$resn_exch{" O2P PSU"}=" OP2 PSU"; $resn_exch{" O3P PSU"}=" OP3 PSU"; $resn_exch{" C1* PTP"}=" C1' PTP";
$resn_exch{" C2* PTP"}=" C2' PTP"; $resn_exch{" C3* PTP"}=" C3' PTP"; $resn_exch{" O3* PTP"}=" O3' PTP";
$resn_exch{" P3* PTP"}=" P3' PTP"; $resn_exch{" C4* PTP"}=" C4' PTP"; $resn_exch{" O4* PTP"}=" O4' PTP";
$resn_exch{" C5* PTP"}=" C5' PTP"; $resn_exch{" O5* PTP"}=" O5' PTP"; $resn_exch{" P5* PTP"}=" P5' PTP";
$resn_exch{" O1P  PU"}=" OP1  PU"; $resn_exch{" O2P  PU"}=" OP2  PU"; $resn_exch{" O3P  PU"}=" OP3  PU";
$resn_exch{" O5*  PU"}=" O5'  PU"; $resn_exch{" C5*  PU"}=" C5'  PU"; $resn_exch{" C4*  PU"}=" C4'  PU";
$resn_exch{" O4*  PU"}=" O4'  PU"; $resn_exch{" C3*  PU"}=" C3'  PU"; $resn_exch{" N3*  PU"}=" N3'  PU";
$resn_exch{" C2*  PU"}=" C2'  PU"; $resn_exch{" O2*  PU"}=" O2'  PU"; $resn_exch{" C1*  PU"}=" C1'  PU";
$resn_exch{" O5* PUA"}=" O5' PUA"; $resn_exch{" C5* PUA"}=" C5' PUA"; $resn_exch{" C4* PUA"}=" C4' PUA";
$resn_exch{" O4* PUA"}=" O4' PUA"; $resn_exch{" C1* PUA"}=" C1' PUA"; $resn_exch{" C2* PUA"}=" C2' PUA";
$resn_exch{" O2* PUA"}=" O2' PUA"; $resn_exch{" C3* PUA"}=" C3' PUA"; $resn_exch{" O3* PUA"}=" O3' PUA";
$resn_exch{" C1* PUR"}=" C1' PUR"; $resn_exch{" C2* PUR"}=" C2' PUR"; $resn_exch{" O2* PUR"}=" O2' PUR";
$resn_exch{" C3* PUR"}=" C3' PUR"; $resn_exch{" O3* PUR"}=" O3' PUR"; $resn_exch{" C4* PUR"}=" C4' PUR";
$resn_exch{" O4* PUR"}=" O4' PUR"; $resn_exch{" C5* PUR"}=" C5' PUR"; $resn_exch{" O5* PUR"}=" O5' PUR";
$resn_exch{" O5* PUY"}=" O5' PUY"; $resn_exch{" C5* PUY"}=" C5' PUY"; $resn_exch{" C4* PUY"}=" C4' PUY";
$resn_exch{" O4* PUY"}=" O4' PUY"; $resn_exch{" C1* PUY"}=" C1' PUY"; $resn_exch{" C2* PUY"}=" C2' PUY";
$resn_exch{" O2* PUY"}=" O2' PUY"; $resn_exch{" C3* PUY"}=" C3' PUY"; $resn_exch{" N3* PUY"}=" N3' PUY";
$resn_exch{" C*  PX1"}=" C'  PX1"; $resn_exch{" O*  PX1"}=" O'  PX1"; $resn_exch{" C*  PXA"}=" C'  PXA";
$resn_exch{" O*  PXA"}=" O'  PXA"; $resn_exch{" C*  PXF"}=" C'  PXF"; $resn_exch{" O*  PXF"}=" O'  PXF";
$resn_exch{" C*  PXZ"}=" C'  PXZ"; $resn_exch{" O*  PXZ"}=" O'  PXZ"; $resn_exch{" O3P PYO"}=" OP3 PYO";
$resn_exch{" O1P PYO"}=" OP1 PYO"; $resn_exch{" O2P PYO"}=" OP2 PYO"; $resn_exch{" O5* PYO"}=" O5' PYO";
$resn_exch{" C5* PYO"}=" C5' PYO"; $resn_exch{" C4* PYO"}=" C4' PYO"; $resn_exch{" O4* PYO"}=" O4' PYO";
$resn_exch{" C3* PYO"}=" C3' PYO"; $resn_exch{" C1* PYO"}=" C1' PYO"; $resn_exch{" O3* PYO"}=" O3' PYO";
$resn_exch{" C2* PYO"}=" C2' PYO"; $resn_exch{" O2* PYO"}=" O2' PYO"; $resn_exch{" O5* PYP"}=" O5' PYP";
$resn_exch{" C5* PYP"}=" C5' PYP"; $resn_exch{" C4* PYP"}=" C4' PYP"; $resn_exch{" O4* PYP"}=" O4' PYP";
$resn_exch{" C1* PYP"}=" C1' PYP"; $resn_exch{" C2* PYP"}=" C2' PYP"; $resn_exch{" C3* PYP"}=" C3' PYP";
$resn_exch{" O3* PYP"}=" O3' PYP"; $resn_exch{" O5* PYY"}=" O5' PYY"; $resn_exch{" C5* PYY"}=" C5' PYY";
$resn_exch{" C4* PYY"}=" C4' PYY"; $resn_exch{" O4* PYY"}=" O4' PYY"; $resn_exch{" C1* PYY"}=" C1' PYY";
$resn_exch{" C2* PYY"}=" C2' PYY"; $resn_exch{" O2* PYY"}=" O2' PYY"; $resn_exch{" C3* PYY"}=" C3' PYY";
$resn_exch{" O3* PYY"}=" O3' PYY"; $resn_exch{" O1P QUO"}=" OP1 QUO"; $resn_exch{" O2P QUO"}=" OP2 QUO";
$resn_exch{" O3P QUO"}=" OP3 QUO"; $resn_exch{" O5* QUO"}=" O5' QUO"; $resn_exch{" C5* QUO"}=" C5' QUO";
$resn_exch{" C4* QUO"}=" C4' QUO"; $resn_exch{" O4* QUO"}=" O4' QUO"; $resn_exch{" C1* QUO"}=" C1' QUO";
$resn_exch{" C2* QUO"}=" C2' QUO"; $resn_exch{" O2* QUO"}=" O2' QUO"; $resn_exch{" C3* QUO"}=" C3' QUO";
$resn_exch{" O3* QUO"}=" O3' QUO"; $resn_exch{" O1P   R"}=" OP1   R"; $resn_exch{" O2P   R"}=" OP2   R";
$resn_exch{" O3P   R"}=" OP3   R"; $resn_exch{" O5*   R"}=" O5'   R"; $resn_exch{" C5*   R"}=" C5'   R";
$resn_exch{" C4*   R"}=" C4'   R"; $resn_exch{" O4*   R"}=" O4'   R"; $resn_exch{" C3*   R"}=" C3'   R";
$resn_exch{" O3*   R"}=" O3'   R"; $resn_exch{" C2*   R"}=" C2'   R"; $resn_exch{" C1*   R"}=" C1'   R";
$resn_exch{" C1* R22"}=" C1' R22"; $resn_exch{" C2* R22"}=" C2' R22"; $resn_exch{" C3* R22"}=" C3' R22";
$resn_exch{" C4* R22"}=" C4' R22"; $resn_exch{" C5* R22"}=" C5' R22"; $resn_exch{" C6* R22"}=" C6' R22";
$resn_exch{" O7* R22"}=" O7' R22"; $resn_exch{" C8* R22"}=" C8' R22"; $resn_exch{" C9* R22"}=" C9' R22";
$resn_exch{" O   R7A"}=" OXT R7A"; $resn_exch{" OXT R7A"}=" O   R7A"; $resn_exch{" C1* RBF"}=" C1' RBF";
$resn_exch{" C2* RBF"}=" C2' RBF"; $resn_exch{" O2* RBF"}=" O2' RBF"; $resn_exch{" C3* RBF"}=" C3' RBF";
$resn_exch{" O3* RBF"}=" O3' RBF"; $resn_exch{" C4* RBF"}=" C4' RBF"; $resn_exch{" O4* RBF"}=" O4' RBF";
$resn_exch{" C5* RBF"}=" C5' RBF"; $resn_exch{" O5* RBF"}=" O5' RBF"; $resn_exch{" O5* RBY"}=" O5' RBY";
$resn_exch{" C5* RBY"}=" C5' RBY"; $resn_exch{" C4* RBY"}=" C4' RBY"; $resn_exch{" O4* RBY"}=" O4' RBY";
$resn_exch{" C3* RBY"}=" C3' RBY"; $resn_exch{" O3* RBY"}=" O3' RBY"; $resn_exch{" C2* RBY"}=" C2' RBY";
$resn_exch{" O2* RBY"}=" O2' RBY"; $resn_exch{" C1* RBY"}=" C1' RBY"; $resn_exch{" O5* RBZ"}=" O5' RBZ";
$resn_exch{" C5* RBZ"}=" C5' RBZ"; $resn_exch{" C4* RBZ"}=" C4' RBZ"; $resn_exch{" O4* RBZ"}=" O4' RBZ";
$resn_exch{" C3* RBZ"}=" C3' RBZ"; $resn_exch{" O3* RBZ"}=" O3' RBZ"; $resn_exch{" C2* RBZ"}=" C2' RBZ";
$resn_exch{" O2* RBZ"}=" O2' RBZ"; $resn_exch{" C1* RBZ"}=" C1' RBZ"; $resn_exch{" O1* RDP"}=" O1' RDP";
$resn_exch{" C1* RDP"}=" C1' RDP"; $resn_exch{" C2* RDP"}=" C2' RDP"; $resn_exch{" O2* RDP"}=" O2' RDP";
$resn_exch{" C3* RDP"}=" C3' RDP"; $resn_exch{" O3* RDP"}=" O3' RDP"; $resn_exch{" C4* RDP"}=" C4' RDP";
$resn_exch{" O4* RDP"}=" O4' RDP"; $resn_exch{" C5* RDP"}=" C5' RDP"; $resn_exch{" O5* RDP"}=" O5' RDP";
$resn_exch{" AO7 RFC"}=" O7A RFC"; $resn_exch{"AP3* RFC"}=" P3B RFC"; $resn_exch{" AO9 RFC"}=" O9A RFC";
$resn_exch{" AO8 RFC"}=" O8A RFC"; $resn_exch{"AO3* RFC"}=" O3B RFC"; $resn_exch{"AC3* RFC"}=" C3B RFC";
$resn_exch{"AC2* RFC"}=" C2B RFC"; $resn_exch{"AO2* RFC"}=" O2B RFC"; $resn_exch{"AC1* RFC"}=" C1B RFC";
$resn_exch{" AN9 RFC"}=" N9A RFC"; $resn_exch{" AC4 RFC"}=" C4A RFC"; $resn_exch{" AC5 RFC"}=" C5A RFC";
$resn_exch{" AC6 RFC"}=" C6A RFC"; $resn_exch{" AN6 RFC"}=" N6A RFC"; $resn_exch{" AN1 RFC"}=" N1A RFC";
$resn_exch{" AC2 RFC"}=" C2A RFC"; $resn_exch{" AN3 RFC"}=" N3A RFC"; $resn_exch{" AN7 RFC"}=" N7A RFC";
$resn_exch{" AC8 RFC"}=" C8A RFC"; $resn_exch{"AO4* RFC"}=" O4B RFC"; $resn_exch{"AC4* RFC"}=" C4B RFC";
$resn_exch{"AC5* RFC"}=" C5B RFC"; $resn_exch{"AO5* RFC"}=" O5B RFC"; $resn_exch{" AP1 RFC"}=" P1A RFC";
$resn_exch{" AO1 RFC"}=" O1A RFC"; $resn_exch{" AO2 RFC"}=" O2A RFC"; $resn_exch{" AO3 RFC"}=" O3A RFC";
$resn_exch{" AP2 RFC"}=" P2A RFC"; $resn_exch{" AO4 RFC"}=" O4A RFC"; $resn_exch{" AO5 RFC"}=" O5A RFC";
$resn_exch{" AO6 RFC"}=" O6A RFC"; $resn_exch{"PC12 RFC"}=" CCP RFC"; $resn_exch{"PC11 RFC"}=" CBP RFC";
$resn_exch{"PC13 RFC"}=" CDP RFC"; $resn_exch{"PC14 RFC"}=" CEP RFC"; $resn_exch{"PC10 RFC"}=" CAP RFC";
$resn_exch{"PO10 RFC"}=" OAP RFC"; $resn_exch{" PC9 RFC"}=" C9P RFC"; $resn_exch{" PO9 RFC"}=" O9P RFC";
$resn_exch{" PN8 RFC"}=" N8P RFC"; $resn_exch{" PC7 RFC"}=" C7P RFC"; $resn_exch{" PC6 RFC"}=" C6P RFC";
$resn_exch{" PC5 RFC"}=" C5P RFC"; $resn_exch{" PO5 RFC"}=" O5P RFC"; $resn_exch{" PN4 RFC"}=" N4P RFC";
$resn_exch{" PC3 RFC"}=" C3P RFC"; $resn_exch{" PC2 RFC"}=" C2P RFC"; $resn_exch{" PS1 RFC"}=" S1P RFC";
$resn_exch{" SG* RGS"}=" SG' RGS"; $resn_exch{" CB* RGS"}=" CB' RGS"; $resn_exch{" CA* RGS"}=" CA' RGS";
$resn_exch{" N*  RGS"}=" N'  RGS"; $resn_exch{" C5* RGS"}=" C5' RGS"; $resn_exch{" O5* RGS"}=" O5' RGS";
$resn_exch{" C6* RGS"}=" C6' RGS"; $resn_exch{" C7* RGS"}=" C7' RGS"; $resn_exch{"*O7A RGS"}=" O11 RGS";
$resn_exch{"*O7B RGS"}=" O12 RGS"; $resn_exch{" C*  RGS"}=" C'  RGS"; $resn_exch{" O*  RGS"}=" O'  RGS";
$resn_exch{" N4* RGS"}=" N4' RGS"; $resn_exch{" C4* RGS"}=" C4' RGS"; $resn_exch{" C3* RGS"}=" C3' RGS";
$resn_exch{" C2* RGS"}=" C2' RGS"; $resn_exch{" N2* RGS"}=" N2' RGS"; $resn_exch{" C1* RGS"}=" C1' RGS";
$resn_exch{"*O1A RGS"}=" O13 RGS"; $resn_exch{"*O1B RGS"}=" O14 RGS"; $resn_exch{" O1P RIA"}=" OP1 RIA";
$resn_exch{" O2P RIA"}=" OP2 RIA"; $resn_exch{" O3P RIA"}=" OP3 RIA"; $resn_exch{" O5* RIA"}=" O5A RIA";
$resn_exch{" C5* RIA"}=" C5A RIA"; $resn_exch{" C4* RIA"}=" C4A RIA"; $resn_exch{" O4* RIA"}=" O4' RIA";
$resn_exch{" C3* RIA"}=" C3A RIA"; $resn_exch{" O3* RIA"}=" O3A RIA"; $resn_exch{" C2* RIA"}=" C2A RIA";
$resn_exch{" O2* RIA"}=" O2A RIA"; $resn_exch{" C1* RIA"}=" C1A RIA"; $resn_exch{"O1P' RIA"}=" O1X RIA";
$resn_exch{"O2P' RIA"}=" O2X RIA"; $resn_exch{"O3P' RIA"}=" O3X RIA"; $resn_exch{" O5* RIB"}=" O5' RIB";
$resn_exch{" C5* RIB"}=" C5' RIB"; $resn_exch{" C4* RIB"}=" C4' RIB"; $resn_exch{" O4* RIB"}=" O4' RIB";
$resn_exch{" C3* RIB"}=" C3' RIB"; $resn_exch{" O3* RIB"}=" O3' RIB"; $resn_exch{" C2* RIB"}=" C2' RIB";
$resn_exch{" O2* RIB"}=" O2' RIB"; $resn_exch{" C1* RIB"}=" C1' RIB"; $resn_exch{" O1* RIB"}=" O1' RIB";
$resn_exch{" O5* RMB"}=" O5' RMB"; $resn_exch{" C5* RMB"}=" C5' RMB"; $resn_exch{" C4* RMB"}=" C4' RMB";
$resn_exch{" O4* RMB"}=" O4' RMB"; $resn_exch{" C3* RMB"}=" C3' RMB"; $resn_exch{" O3* RMB"}=" O3' RMB";
$resn_exch{" C2* RMB"}=" C2' RMB"; $resn_exch{" O2* RMB"}=" O2' RMB"; $resn_exch{" C1* RMB"}=" C1' RMB";
$resn_exch{" O1P RMP"}=" OP1 RMP"; $resn_exch{" O5* RMP"}=" O5' RMP"; $resn_exch{" C5* RMP"}=" C5' RMP";
$resn_exch{" C4* RMP"}=" C4' RMP"; $resn_exch{" O4* RMP"}=" O4' RMP"; $resn_exch{" C1* RMP"}=" C1' RMP";
$resn_exch{" C3* RMP"}=" C3' RMP"; $resn_exch{" C2* RMP"}=" C2' RMP"; $resn_exch{" O3* RMP"}=" O3' RMP";
$resn_exch{" O3P RMP"}=" OP3 RMP"; $resn_exch{"O1P' ROB"}=" O1X ROB"; $resn_exch{"O2P' ROB"}=" O2X ROB";
$resn_exch{"O3P' ROB"}=" O3X ROB"; $resn_exch{"C6'B ROM"}=" C6D ROM"; $resn_exch{"C6'A ROM"}=" C6B ROM";
$resn_exch{"C4'B ROM"}=" C4D ROM"; $resn_exch{"C4'A ROM"}=" C4B ROM"; $resn_exch{"C'10 ROM"}=" C15 ROM";
$resn_exch{"C'12 ROM"}=" C1A ROM"; $resn_exch{"C'11 ROM"}=" C1B ROM"; $resn_exch{"C'7A ROM"}=" C7X ROM";
$resn_exch{"O'12 ROM"}=" O11 ROM"; $resn_exch{" O5* RP1"}=" O5' RP1"; $resn_exch{" C5* RP1"}=" C5' RP1";
$resn_exch{" C4* RP1"}=" C4' RP1"; $resn_exch{" O4* RP1"}=" O4' RP1"; $resn_exch{" C3* RP1"}=" C3' RP1";
$resn_exch{" O3* RP1"}=" O3' RP1"; $resn_exch{" C2* RP1"}=" C2' RP1"; $resn_exch{" O2* RP1"}=" O2' RP1";
$resn_exch{" C1* RP1"}=" C1' RP1"; $resn_exch{"O1P' RP5"}=" O1X RP5"; $resn_exch{"O2P' RP5"}=" O2X RP5";
$resn_exch{"O3P' RP5"}=" O3X RP5"; $resn_exch{"1OM6 RPA"}=" O11 RPA"; $resn_exch{" O5* RPD"}=" O5' RPD";
$resn_exch{" C5* RPD"}=" C5' RPD"; $resn_exch{" C4* RPD"}=" C4' RPD"; $resn_exch{" O4* RPD"}=" O4' RPD";
$resn_exch{" C3* RPD"}=" C3' RPD"; $resn_exch{" O3* RPD"}=" O3' RPD"; $resn_exch{" C2* RPD"}=" C2' RPD";
$resn_exch{" O2* RPD"}=" O2' RPD"; $resn_exch{" C1* RPD"}=" C1' RPD"; $resn_exch{"O91A RPD"}=" OJA RPD";
$resn_exch{"O92A RPD"}=" OKA RPD"; $resn_exch{"N10A RPD"}=" NAA RPD"; $resn_exch{"O10A RPD"}=" OAA RPD";
$resn_exch{"C11A RPD"}=" CBA RPD"; $resn_exch{"O11A RPD"}=" OBA RPD"; $resn_exch{" O5* RPL"}=" O5' RPL";
$resn_exch{" C5* RPL"}=" C5' RPL"; $resn_exch{" C4* RPL"}=" C4' RPL"; $resn_exch{" O4* RPL"}=" O4' RPL";
$resn_exch{" C3* RPL"}=" C3' RPL"; $resn_exch{" O3* RPL"}=" O3' RPL"; $resn_exch{" C2* RPL"}=" C2' RPL";
$resn_exch{" O2* RPL"}=" O2' RPL"; $resn_exch{" C1* RPL"}=" C1' RPL"; $resn_exch{"O91A RPL"}=" OJA RPL";
$resn_exch{"O92A RPL"}=" OKA RPL"; $resn_exch{"N10A RPL"}=" NAA RPL"; $resn_exch{"O10A RPL"}=" OAA RPL";
$resn_exch{"C11A RPL"}=" CBA RPL"; $resn_exch{"O11A RPL"}=" OBA RPL"; $resn_exch{" C1* RPP"}=" C1' RPP";
$resn_exch{" C2* RPP"}=" C2' RPP"; $resn_exch{" C3* RPP"}=" C3' RPP"; $resn_exch{" C4* RPP"}=" C4' RPP";
$resn_exch{" O4* RPP"}=" O4' RPP"; $resn_exch{" O2* RPP"}=" O2' RPP"; $resn_exch{" O3* RPP"}=" O3' RPP";
$resn_exch{" C5* RPP"}=" C5' RPP"; $resn_exch{" O5* RPP"}=" O5' RPP"; $resn_exch{" O3P  RT"}=" OP3  RT";
$resn_exch{" O1P  RT"}=" OP1  RT"; $resn_exch{" O2P  RT"}=" OP2  RT"; $resn_exch{" O5*  RT"}=" O5'  RT";
$resn_exch{" C5*  RT"}=" C5'  RT"; $resn_exch{" C4*  RT"}=" C4'  RT"; $resn_exch{" O4*  RT"}=" O4'  RT";
$resn_exch{" C3*  RT"}=" C3'  RT"; $resn_exch{" O3*  RT"}=" O3'  RT"; $resn_exch{" C2*  RT"}=" C2'  RT";
$resn_exch{" O2*  RT"}=" O2'  RT"; $resn_exch{" C1*  RT"}=" C1'  RT"; $resn_exch{" O5* RTP"}=" O5' RTP";
$resn_exch{" C5* RTP"}=" C5' RTP"; $resn_exch{" C4* RTP"}=" C4' RTP"; $resn_exch{" O4* RTP"}=" O4' RTP";
$resn_exch{" C1* RTP"}=" C1' RTP"; $resn_exch{" C2* RTP"}=" C2' RTP"; $resn_exch{" O2* RTP"}=" O2' RTP";
$resn_exch{" C3* RTP"}=" C3' RTP"; $resn_exch{" O3* RTP"}=" O3' RTP"; $resn_exch{" C1\" RUN"}=" C1' RUN";
$resn_exch{" O1\" RUN"}=" O1' RUN"; $resn_exch{" C2\" RUN"}=" C2D RUN"; $resn_exch{" C3\" RUN"}=" C3' RUN";
$resn_exch{" C4\" RUN"}=" C4D RUN"; $resn_exch{" O5* RVP"}=" O5' RVP"; $resn_exch{" C5* RVP"}=" C5' RVP";
$resn_exch{" C4* RVP"}=" C4' RVP"; $resn_exch{" O4* RVP"}=" O4' RVP"; $resn_exch{" C3* RVP"}=" C3' RVP";
$resn_exch{" O3* RVP"}=" O3' RVP"; $resn_exch{" C2* RVP"}=" C2' RVP"; $resn_exch{" O2* RVP"}=" O2' RVP";
$resn_exch{" C1* RVP"}=" C1' RVP"; $resn_exch{" O5* S02"}=" O5' S02"; $resn_exch{" C5* S02"}=" C5' S02";
$resn_exch{" CA* S02"}=" CA' S02"; $resn_exch{" OB* S02"}=" OB' S02"; $resn_exch{" OC* S02"}=" OC' S02";
$resn_exch{" CD* S02"}=" CD' S02"; $resn_exch{" C3* S02"}=" C3' S02"; $resn_exch{" O3* S02"}=" O3' S02";
$resn_exch{" C5* S2M"}=" C5' S2M"; $resn_exch{" O5* S2M"}=" O5' S2M"; $resn_exch{" O2P S2M"}=" OP2 S2M";
$resn_exch{" O1P S2M"}=" OP1 S2M"; $resn_exch{" C4* S2M"}=" C4' S2M"; $resn_exch{" O4* S2M"}=" O4' S2M";
$resn_exch{" C1* S2M"}=" C1' S2M"; $resn_exch{" C2* S2M"}=" C2' S2M"; $resn_exch{" C3* S2M"}=" C3' S2M";
$resn_exch{" O3* S2M"}=" O3' S2M"; $resn_exch{" O2* S2M"}=" O2' S2M"; $resn_exch{" CA* S2M"}=" CA' S2M";
$resn_exch{" CB* S2M"}=" CB' S2M"; $resn_exch{" OC* S2M"}=" OC' S2M"; $resn_exch{" CD* S2M"}=" CD' S2M";
$resn_exch{" O3P S2M"}=" OP3 S2M"; $resn_exch{" C1* S4C"}=" C1' S4C"; $resn_exch{" C2* S4C"}=" C2' S4C";
$resn_exch{" O2* S4C"}=" O2' S4C"; $resn_exch{" C3* S4C"}=" C3' S4C"; $resn_exch{" O3* S4C"}=" O3' S4C";
$resn_exch{" C4* S4C"}=" C4' S4C"; $resn_exch{" S4* S4C"}=" S4' S4C"; $resn_exch{" O1P S4C"}=" OP1 S4C";
$resn_exch{" C5* S4C"}=" C5' S4C"; $resn_exch{" O5* S4C"}=" O5' S4C"; $resn_exch{" O2P S4C"}=" OP2 S4C";
$resn_exch{" O3P S4C"}=" OP3 S4C"; $resn_exch{" C5* S4M"}=" C5' S4M"; $resn_exch{" C4* S4M"}=" C4' S4M";
$resn_exch{" O4* S4M"}=" O4' S4M"; $resn_exch{" C1* S4M"}=" C1' S4M"; $resn_exch{" C2* S4M"}=" C2' S4M";
$resn_exch{" O2* S4M"}=" O2' S4M"; $resn_exch{" C3* S4M"}=" C3' S4M"; $resn_exch{" O3* S4M"}=" O3' S4M";
$resn_exch{" O1P S4U"}=" OP1 S4U"; $resn_exch{" O2P S4U"}=" OP2 S4U"; $resn_exch{" O3P S4U"}=" OP3 S4U";
$resn_exch{" O1P S6G"}=" OP1 S6G"; $resn_exch{" O2P S6G"}=" OP2 S6G"; $resn_exch{" O3P S6G"}=" OP3 S6G";
$resn_exch{" O5* S6G"}=" O5' S6G"; $resn_exch{" C5* S6G"}=" C5' S6G"; $resn_exch{" C4* S6G"}=" C4' S6G";
$resn_exch{" O4* S6G"}=" O4' S6G"; $resn_exch{" C3* S6G"}=" C3' S6G"; $resn_exch{" O3* S6G"}=" O3' S6G";
$resn_exch{" C2* S6G"}=" C2' S6G"; $resn_exch{" C1* S6G"}=" C1' S6G"; $resn_exch{" O5* S9L"}=" O5' S9L";
$resn_exch{" O3* S9L"}=" O3' S9L"; $resn_exch{" C5* SA8"}=" C5' SA8"; $resn_exch{" C4* SA8"}=" C4' SA8";
$resn_exch{" O4* SA8"}=" O4' SA8"; $resn_exch{" C3* SA8"}=" C3' SA8"; $resn_exch{" O3* SA8"}=" O3' SA8";
$resn_exch{" C2* SA8"}=" C2' SA8"; $resn_exch{" O2* SA8"}=" O2' SA8"; $resn_exch{" C1* SA8"}=" C1' SA8";
$resn_exch{" AP  SAD"}=" PA  SAD"; $resn_exch{" AO1 SAD"}=" O1A SAD"; $resn_exch{" AO2 SAD"}=" O2A SAD";
$resn_exch{"AO5* SAD"}=" O5B SAD"; $resn_exch{"AC5* SAD"}=" C5B SAD"; $resn_exch{"AC4* SAD"}=" C4B SAD";
$resn_exch{"AO4* SAD"}=" O4B SAD"; $resn_exch{"AC3* SAD"}=" C3B SAD"; $resn_exch{"AO3* SAD"}=" O3B SAD";
$resn_exch{"AC2* SAD"}=" C2B SAD"; $resn_exch{"AO2* SAD"}=" O2B SAD"; $resn_exch{"AC1* SAD"}=" C1B SAD";
$resn_exch{" AN9 SAD"}=" N9A SAD"; $resn_exch{" AC8 SAD"}=" C8A SAD"; $resn_exch{" AN7 SAD"}=" N7A SAD";
$resn_exch{" AC5 SAD"}=" C5A SAD"; $resn_exch{" AC6 SAD"}=" C6A SAD"; $resn_exch{" AN6 SAD"}=" N6A SAD";
$resn_exch{" AN1 SAD"}=" N1A SAD"; $resn_exch{" AC2 SAD"}=" C2A SAD"; $resn_exch{" AN3 SAD"}=" N3A SAD";
$resn_exch{" AC4 SAD"}=" C4A SAD"; $resn_exch{" NP  SAD"}=" PN  SAD"; $resn_exch{" NO1 SAD"}=" O1N SAD";
$resn_exch{" NO2 SAD"}=" O2N SAD"; $resn_exch{"NO5* SAD"}=" O5D SAD"; $resn_exch{"NC5* SAD"}=" C5D SAD";
$resn_exch{"NC4* SAD"}=" C4D SAD"; $resn_exch{"NO4* SAD"}=" O4D SAD"; $resn_exch{"NC3* SAD"}=" C3D SAD";
$resn_exch{"NO3* SAD"}=" O3D SAD"; $resn_exch{"NC2* SAD"}=" C2D SAD"; $resn_exch{"NO2* SAD"}=" O2D SAD";
$resn_exch{"NC1* SAD"}=" C1D SAD"; $resn_exch{" NC2 SAD"}=" C2N SAD"; $resn_exch{" NN3 SAD"}=" N3N SAD";
$resn_exch{" NC4 SAD"}=" C4N SAD"; $resn_exch{" NC5 SAD"}=" C5N SAD"; $resn_exch{"NSE1 SAD"}="SE1N SAD";
$resn_exch{" NC6 SAD"}=" C6N SAD"; $resn_exch{" NO6 SAD"}=" O6N SAD"; $resn_exch{" NN6 SAD"}=" N6N SAD";
$resn_exch{" AP  SAE"}=" PA  SAE"; $resn_exch{" AO1 SAE"}=" O1A SAE"; $resn_exch{" AO2 SAE"}=" O2A SAE";
$resn_exch{"AO5* SAE"}=" O5B SAE"; $resn_exch{"AC5* SAE"}=" C5B SAE"; $resn_exch{"AC4* SAE"}=" C4B SAE";
$resn_exch{"AO4* SAE"}=" O4B SAE"; $resn_exch{"AC3* SAE"}=" C3B SAE"; $resn_exch{"AO3* SAE"}=" O3B SAE";
$resn_exch{"AC2* SAE"}=" C2B SAE"; $resn_exch{"AO2* SAE"}=" O2B SAE"; $resn_exch{"AC1* SAE"}=" C1B SAE";
$resn_exch{" AN9 SAE"}=" N9A SAE"; $resn_exch{" AC8 SAE"}=" C8A SAE"; $resn_exch{" AN7 SAE"}=" N7A SAE";
$resn_exch{" AC5 SAE"}=" C5A SAE"; $resn_exch{" AC6 SAE"}=" C6A SAE"; $resn_exch{" AN6 SAE"}=" N6A SAE";
$resn_exch{" AN1 SAE"}=" N1A SAE"; $resn_exch{" AC2 SAE"}=" C2A SAE"; $resn_exch{" AN3 SAE"}=" N3A SAE";
$resn_exch{" AC4 SAE"}=" C4A SAE"; $resn_exch{" NP  SAE"}=" PN  SAE"; $resn_exch{" NO1 SAE"}=" O1N SAE";
$resn_exch{" NO2 SAE"}=" O2N SAE"; $resn_exch{"NO5* SAE"}=" O5D SAE"; $resn_exch{"NC5* SAE"}=" C5D SAE";
$resn_exch{"NC4* SAE"}=" C4D SAE"; $resn_exch{"NO4* SAE"}=" O4D SAE"; $resn_exch{"NC3* SAE"}=" C3D SAE";
$resn_exch{"NO3* SAE"}=" O3D SAE"; $resn_exch{"NC2* SAE"}=" C2D SAE"; $resn_exch{"NO2* SAE"}=" O2D SAE";
$resn_exch{"NC1* SAE"}=" C1D SAE"; $resn_exch{" NC2 SAE"}=" C2N SAE"; $resn_exch{" NN3 SAE"}=" N3N SAE";
$resn_exch{" NC4 SAE"}=" C4N SAE"; $resn_exch{" NC5 SAE"}=" C5N SAE"; $resn_exch{"NSE1 SAE"}="SE1N SAE";
$resn_exch{" NC6 SAE"}=" C6N SAE"; $resn_exch{" NO6 SAE"}=" O6N SAE"; $resn_exch{" NN6 SAE"}=" N6N SAE";
$resn_exch{" C5* SAH"}=" C5' SAH"; $resn_exch{" C4* SAH"}=" C4' SAH"; $resn_exch{" O4* SAH"}=" O4' SAH";
$resn_exch{" C3* SAH"}=" C3' SAH"; $resn_exch{" O3* SAH"}=" O3' SAH"; $resn_exch{" C2* SAH"}=" C2' SAH";
$resn_exch{" O2* SAH"}=" O2' SAH"; $resn_exch{" C1* SAH"}=" C1' SAH"; $resn_exch{" C5* SAI"}=" C5' SAI";
$resn_exch{" C4* SAI"}=" C4' SAI"; $resn_exch{" O4* SAI"}=" O4' SAI"; $resn_exch{" C3* SAI"}=" C3' SAI";
$resn_exch{" O3* SAI"}=" O3' SAI"; $resn_exch{" C2* SAI"}=" C2' SAI"; $resn_exch{" O2* SAI"}=" O2' SAI";
$resn_exch{" C1* SAI"}=" C1' SAI"; $resn_exch{" C5* SAM"}=" C5' SAM"; $resn_exch{" C4* SAM"}=" C4' SAM";
$resn_exch{" O4* SAM"}=" O4' SAM"; $resn_exch{" C3* SAM"}=" C3' SAM"; $resn_exch{" O3* SAM"}=" O3' SAM";
$resn_exch{" C2* SAM"}=" C2' SAM"; $resn_exch{" O2* SAM"}=" O2' SAM"; $resn_exch{" C1* SAM"}=" C1' SAM";
$resn_exch{" O5* SAP"}=" O5' SAP"; $resn_exch{" C5* SAP"}=" C5' SAP"; $resn_exch{" C4* SAP"}=" C4' SAP";
$resn_exch{" O4* SAP"}=" O4' SAP"; $resn_exch{" C3* SAP"}=" C3' SAP"; $resn_exch{" O3* SAP"}=" O3' SAP";
$resn_exch{" C2* SAP"}=" C2' SAP"; $resn_exch{" O2* SAP"}=" O2' SAP"; $resn_exch{" C1* SAP"}=" C1' SAP";
$resn_exch{" C1*  SC"}=" C1'  SC"; $resn_exch{" C2*  SC"}=" C2'  SC"; $resn_exch{" C3*  SC"}=" C3'  SC";
$resn_exch{" C4*  SC"}=" C4'  SC"; $resn_exch{" O4*  SC"}=" O4'  SC"; $resn_exch{" O3*  SC"}=" O3'  SC";
$resn_exch{" C5*  SC"}=" C5'  SC"; $resn_exch{" O5*  SC"}=" O5'  SC"; $resn_exch{" O1P  SC"}=" OP1  SC";
$resn_exch{" O3P  SC"}=" OP3  SC"; $resn_exch{" C1* SCO"}=" C1' SCO"; $resn_exch{" C2* SCO"}=" C2' SCO";
$resn_exch{" C3* SCO"}=" C3' SCO"; $resn_exch{" C4* SCO"}=" C4' SCO"; $resn_exch{" C5* SCO"}=" C5' SCO";
$resn_exch{" O1* SCO"}=" O1' SCO"; $resn_exch{" O2* SCO"}=" O2' SCO"; $resn_exch{" O3* SCO"}=" O3' SCO";
$resn_exch{" O4* SCO"}=" O4' SCO"; $resn_exch{" O5* SCO"}=" O5' SCO"; $resn_exch{" O6* SCO"}=" O6' SCO";
$resn_exch{" AO7 SFC"}=" O7A SFC"; $resn_exch{"AP3* SFC"}=" P3B SFC"; $resn_exch{" AO9 SFC"}=" O9A SFC";
$resn_exch{" AO8 SFC"}=" O8A SFC"; $resn_exch{"AO3* SFC"}=" O3B SFC"; $resn_exch{"AC3* SFC"}=" C3B SFC";
$resn_exch{"AC2* SFC"}=" C2B SFC"; $resn_exch{"AO2* SFC"}=" O2B SFC"; $resn_exch{"AC1* SFC"}=" C1B SFC";
$resn_exch{" AN9 SFC"}=" N9A SFC"; $resn_exch{" AC4 SFC"}=" C4A SFC"; $resn_exch{" AC5 SFC"}=" C5A SFC";
$resn_exch{" AC6 SFC"}=" C6A SFC"; $resn_exch{" AN6 SFC"}=" N6A SFC"; $resn_exch{" AN1 SFC"}=" N1A SFC";
$resn_exch{" AC2 SFC"}=" C2A SFC"; $resn_exch{" AN3 SFC"}=" N3A SFC"; $resn_exch{" AN7 SFC"}=" N7A SFC";
$resn_exch{" AC8 SFC"}=" C8A SFC"; $resn_exch{"AO4* SFC"}=" O4B SFC"; $resn_exch{"AC4* SFC"}=" C4B SFC";
$resn_exch{"AC5* SFC"}=" C5B SFC"; $resn_exch{"AO5* SFC"}=" O5B SFC"; $resn_exch{" AP1 SFC"}=" P1A SFC";
$resn_exch{" AO1 SFC"}=" O1A SFC"; $resn_exch{" AO2 SFC"}=" O2A SFC"; $resn_exch{" AO3 SFC"}=" O3A SFC";
$resn_exch{" AP2 SFC"}=" P2A SFC"; $resn_exch{" AO4 SFC"}=" O4A SFC"; $resn_exch{" AO5 SFC"}=" O5A SFC";
$resn_exch{" AO6 SFC"}=" O6A SFC"; $resn_exch{"PC12 SFC"}=" CCP SFC"; $resn_exch{"PC11 SFC"}=" CBP SFC";
$resn_exch{"PC13 SFC"}=" CDP SFC"; $resn_exch{"PC14 SFC"}=" CEP SFC"; $resn_exch{"PC10 SFC"}=" CAP SFC";
$resn_exch{"PO10 SFC"}=" OAP SFC"; $resn_exch{" PC9 SFC"}=" C9P SFC"; $resn_exch{" PO9 SFC"}=" O9P SFC";
$resn_exch{" PN8 SFC"}=" N8P SFC"; $resn_exch{" PC7 SFC"}=" C7P SFC"; $resn_exch{" PC6 SFC"}=" C6P SFC";
$resn_exch{" PC5 SFC"}=" C5P SFC"; $resn_exch{" PO5 SFC"}=" O5P SFC"; $resn_exch{" PN4 SFC"}=" N4P SFC";
$resn_exch{" PC3 SFC"}=" C3P SFC"; $resn_exch{" PC2 SFC"}=" C2P SFC"; $resn_exch{" PS1 SFC"}=" S1P SFC";
$resn_exch{" C5* SFG"}=" C5' SFG"; $resn_exch{" C4* SFG"}=" C4' SFG"; $resn_exch{" O4* SFG"}=" O4' SFG";
$resn_exch{" C3* SFG"}=" C3' SFG"; $resn_exch{" O3* SFG"}=" O3' SFG"; $resn_exch{" C2* SFG"}=" C2' SFG";
$resn_exch{" O2* SFG"}=" O2' SFG"; $resn_exch{" C1* SFG"}=" C1' SFG"; $resn_exch{"2C4  SFT"}=" C42 SFT";
$resn_exch{"2C3  SFT"}=" C32 SFT"; $resn_exch{"2C2  SFT"}=" C22 SFT"; $resn_exch{"2C1  SFT"}=" CCA SFT";
$resn_exch{"2C0  SFT"}=" C02 SFT"; $resn_exch{"1C9  SFT"}=" C91 SFT"; $resn_exch{"1C8  SFT"}=" C81 SFT";
$resn_exch{"1C7  SFT"}=" C71 SFT"; $resn_exch{"1C6  SFT"}=" C61 SFT"; $resn_exch{"1C5  SFT"}=" C51 SFT";
$resn_exch{"1C4  SFT"}=" C41 SFT"; $resn_exch{"1C3  SFT"}=" C31 SFT"; $resn_exch{"1C2  SFT"}=" C21 SFT";
$resn_exch{"1C1  SFT"}=" CBA SFT"; $resn_exch{"1C0  SFT"}=" C01 SFT"; $resn_exch{" C1* SFT"}=" C1B SFT";
$resn_exch{" C2* SFT"}=" C2B SFT"; $resn_exch{" O2* SFT"}=" O2B SFT"; $resn_exch{" O5* SFT"}=" O5' SFT";
$resn_exch{" C5* SFT"}=" C5B SFT"; $resn_exch{" C6* SFT"}=" C6B SFT"; $resn_exch{" O6* SFT"}=" O6' SFT";
$resn_exch{" C4* SFT"}=" C4B SFT"; $resn_exch{" O4* SFT"}=" O4' SFT"; $resn_exch{" C3* SFT"}=" C3B SFT";
$resn_exch{" O3* SFT"}=" O3' SFT"; $resn_exch{" O5* SGP"}=" O5' SGP"; $resn_exch{" C5* SGP"}=" C5' SGP";
$resn_exch{" C4* SGP"}=" C4' SGP"; $resn_exch{" O4* SGP"}=" O4' SGP"; $resn_exch{" C3* SGP"}=" C3' SGP";
$resn_exch{" O3* SGP"}=" O3' SGP"; $resn_exch{" C2* SGP"}=" C2' SGP"; $resn_exch{" O2* SGP"}=" O2' SGP";
$resn_exch{" C1* SGP"}=" C1' SGP"; $resn_exch{" O1  SHC"}=" O   SHC"; $resn_exch{" C1* SHP"}=" C1B SHP";
$resn_exch{" C2* SHP"}=" C2B SHP"; $resn_exch{" C3* SHP"}=" C3B SHP"; $resn_exch{" C4* SHP"}=" C4B SHP";
$resn_exch{" O4* SHP"}=" O4B SHP"; $resn_exch{" C5* SHP"}=" C5B SHP"; $resn_exch{" C6* SHP"}=" C6B SHP";
$resn_exch{" O   SIB"}=" OXT SIB"; $resn_exch{" OXT SIB"}=" O   SIB"; $resn_exch{" C5* SIB"}=" C5' SIB";
$resn_exch{" C4* SIB"}=" C4' SIB"; $resn_exch{" O4* SIB"}=" O4' SIB"; $resn_exch{" C3* SIB"}=" C3' SIB";
$resn_exch{" O3* SIB"}=" O3' SIB"; $resn_exch{" C2* SIB"}=" C2' SIB"; $resn_exch{" O2* SIB"}=" O2' SIB";
$resn_exch{" C1* SIB"}=" C1' SIB"; $resn_exch{" C5* SMM"}=" C5' SMM"; $resn_exch{" C4* SMM"}=" C4' SMM";
$resn_exch{" O4* SMM"}=" O4' SMM"; $resn_exch{" C1* SMM"}=" C1' SMM"; $resn_exch{" C2* SMM"}=" C2' SMM";
$resn_exch{" O2* SMM"}=" O2' SMM"; $resn_exch{" C3* SMM"}=" C3' SMM"; $resn_exch{" O3* SMM"}=" O3' SMM";
$resn_exch{" O1P SMP"}=" OP1 SMP"; $resn_exch{" O5* SMP"}=" O5' SMP"; $resn_exch{" C5* SMP"}=" C5' SMP";
$resn_exch{" C4* SMP"}=" C4' SMP"; $resn_exch{" O4* SMP"}=" O4' SMP"; $resn_exch{" C1* SMP"}=" C1' SMP";
$resn_exch{" C3* SMP"}=" C3' SMP"; $resn_exch{" C2* SMP"}=" C2' SMP"; $resn_exch{" O3* SMP"}=" O3' SMP";
$resn_exch{" O3P SMP"}=" OP3 SMP"; $resn_exch{" O1P SMT"}=" OP1 SMT"; $resn_exch{" O2P SMT"}=" OP2 SMT";
$resn_exch{" O3P SMT"}=" OP3 SMT"; $resn_exch{" AP  SND"}=" PA  SND"; $resn_exch{" AO1 SND"}=" O1A SND";
$resn_exch{" AO2 SND"}=" O2A SND"; $resn_exch{"AO5* SND"}=" O5B SND"; $resn_exch{"AC5* SND"}=" C5B SND";
$resn_exch{"AC4* SND"}=" C4B SND"; $resn_exch{"AO4* SND"}=" O4B SND"; $resn_exch{"AC3* SND"}=" C3B SND";
$resn_exch{"AO3* SND"}=" O3B SND"; $resn_exch{"AC2* SND"}=" C2B SND"; $resn_exch{"AO2* SND"}=" O2B SND";
$resn_exch{"AC1* SND"}=" C1B SND"; $resn_exch{" AN9 SND"}=" N9A SND"; $resn_exch{" AC8 SND"}=" C8A SND";
$resn_exch{" AN7 SND"}=" N7A SND"; $resn_exch{" AC5 SND"}=" C5A SND"; $resn_exch{" AC6 SND"}=" C6A SND";
$resn_exch{" AN6 SND"}=" N6A SND"; $resn_exch{" AN1 SND"}=" N1A SND"; $resn_exch{" AC2 SND"}=" C2A SND";
$resn_exch{" AN3 SND"}=" N3A SND"; $resn_exch{" AC4 SND"}=" C4A SND"; $resn_exch{" NP  SND"}=" PN  SND";
$resn_exch{" NO1 SND"}=" O1N SND"; $resn_exch{" NO2 SND"}=" O2N SND"; $resn_exch{"NO5* SND"}=" O5D SND";
$resn_exch{"NC5* SND"}=" C5D SND"; $resn_exch{"NC4* SND"}=" C4D SND"; $resn_exch{"NO4* SND"}=" O4D SND";
$resn_exch{"NC3* SND"}=" C3D SND"; $resn_exch{"NO3* SND"}=" O3D SND"; $resn_exch{"NC2* SND"}=" C2D SND";
$resn_exch{"NO2* SND"}=" O2D SND"; $resn_exch{"NC1* SND"}=" C1D SND"; $resn_exch{" NN1 SND"}=" N1N SND";
$resn_exch{" NC2 SND"}=" C2N SND"; $resn_exch{" NC3 SND"}=" C3N SND"; $resn_exch{" NC7 SND"}=" C7N SND";
$resn_exch{" NS7 SND"}=" S7N SND"; $resn_exch{" NN7 SND"}=" N7N SND"; $resn_exch{" NC4 SND"}=" C4N SND";
$resn_exch{" NC5 SND"}=" C5N SND"; $resn_exch{" NC6 SND"}=" C6N SND"; $resn_exch{" O5* SNI"}=" O5' SNI";
$resn_exch{" C5* SNI"}=" C5' SNI"; $resn_exch{" C4* SNI"}=" C4' SNI"; $resn_exch{" O4* SNI"}=" O4' SNI";
$resn_exch{" C1* SNI"}=" C1' SNI"; $resn_exch{" C2* SNI"}=" C2' SNI"; $resn_exch{" O2* SNI"}=" O2' SNI";
$resn_exch{" C3* SNI"}=" C3' SNI"; $resn_exch{" O3* SNI"}=" O3' SNI"; $resn_exch{" C1* SOP"}=" C1' SOP";
$resn_exch{" O4* SOP"}=" O4' SOP"; $resn_exch{" C2* SOP"}=" C2' SOP"; $resn_exch{" O2* SOP"}=" O2' SOP";
$resn_exch{" C3* SOP"}=" C3' SOP"; $resn_exch{" O3* SOP"}=" O3' SOP"; $resn_exch{" C4* SOP"}=" C4' SOP";
$resn_exch{" C5* SOP"}=" C5' SOP"; $resn_exch{" O5* SOP"}=" O5' SOP"; $resn_exch{" O5* SP1"}=" O5' SP1";
$resn_exch{" C5* SP1"}=" C5' SP1"; $resn_exch{" C4* SP1"}=" C4' SP1"; $resn_exch{" O4* SP1"}=" O4' SP1";
$resn_exch{" C3* SP1"}=" C3' SP1"; $resn_exch{" O3* SP1"}=" O3' SP1"; $resn_exch{" C2* SP1"}=" C2' SP1";
$resn_exch{" O2* SP1"}=" O2' SP1"; $resn_exch{" C1* SP1"}=" C1' SP1"; $resn_exch{" AN1 SPF"}=" N1A SPF";
$resn_exch{" AC2 SPF"}=" C2A SPF"; $resn_exch{" AN3 SPF"}=" N3A SPF"; $resn_exch{" AC4 SPF"}=" C4A SPF";
$resn_exch{" AC5 SPF"}=" C5A SPF"; $resn_exch{" AC6 SPF"}=" C6A SPF"; $resn_exch{" AN6 SPF"}=" N6A SPF";
$resn_exch{" AN7 SPF"}=" N7A SPF"; $resn_exch{" AC8 SPF"}=" C8A SPF"; $resn_exch{" AN9 SPF"}=" N9A SPF";
$resn_exch{"AC1* SPF"}=" C1B SPF"; $resn_exch{"AC2* SPF"}=" C2B SPF"; $resn_exch{"AO2* SPF"}=" O2B SPF";
$resn_exch{"AC3* SPF"}=" C3B SPF"; $resn_exch{"AO3* SPF"}=" O3B SPF"; $resn_exch{"AP3* SPF"}=" P3B SPF";
$resn_exch{" AO7 SPF"}=" O7A SPF"; $resn_exch{" AO8 SPF"}=" O8A SPF"; $resn_exch{" AO9 SPF"}=" O9A SPF";
$resn_exch{"AC4* SPF"}=" C4B SPF"; $resn_exch{"AO4* SPF"}=" O4B SPF"; $resn_exch{"AC5* SPF"}=" C5B SPF";
$resn_exch{"AO5* SPF"}=" O5B SPF"; $resn_exch{" AP1 SPF"}=" P1A SPF"; $resn_exch{" AO1 SPF"}=" O1A SPF";
$resn_exch{" AO2 SPF"}=" O2A SPF"; $resn_exch{" AO3 SPF"}=" O3A SPF"; $resn_exch{" AP2 SPF"}=" P2A SPF";
$resn_exch{" AO4 SPF"}=" O4A SPF"; $resn_exch{" AO5 SPF"}=" O5A SPF"; $resn_exch{" AO6 SPF"}=" O6A SPF";
$resn_exch{"PC11 SPF"}=" CBP SPF"; $resn_exch{"PC12 SPF"}=" CCP SPF"; $resn_exch{"PC13 SPF"}=" CDP SPF";
$resn_exch{"PC14 SPF"}=" CEP SPF"; $resn_exch{"PC10 SPF"}=" CAP SPF"; $resn_exch{"PO10 SPF"}=" OAP SPF";
$resn_exch{" PC9 SPF"}=" C9P SPF"; $resn_exch{" PO9 SPF"}=" O9P SPF"; $resn_exch{" PN8 SPF"}=" N8P SPF";
$resn_exch{" PC7 SPF"}=" C7P SPF"; $resn_exch{" PC6 SPF"}=" C6P SPF"; $resn_exch{" PC5 SPF"}=" C5P SPF";
$resn_exch{" PO5 SPF"}=" O5P SPF"; $resn_exch{" PN4 SPF"}=" N4P SPF"; $resn_exch{" PC3 SPF"}=" C3P SPF";
$resn_exch{" PC2 SPF"}=" C2P SPF"; $resn_exch{" PS1 SPF"}=" S1P SPF"; $resn_exch{" O5* SPG"}=" O5' SPG";
$resn_exch{" C5* SPG"}=" C5' SPG"; $resn_exch{" C4* SPG"}=" C4' SPG"; $resn_exch{" O4* SPG"}=" O4' SPG";
$resn_exch{" C3* SPG"}=" C3' SPG"; $resn_exch{" O3* SPG"}=" O3' SPG"; $resn_exch{" C2* SPG"}=" C2' SPG";
$resn_exch{" O2* SPG"}=" O2' SPG"; $resn_exch{" C1* SPG"}=" C1' SPG"; $resn_exch{" S5* SPT"}=" S5' SPT";
$resn_exch{" C2* SPT"}=" C2' SPT"; $resn_exch{" C5* SPT"}=" C5' SPT"; $resn_exch{" C4* SPT"}=" C4' SPT";
$resn_exch{" O4* SPT"}=" O4' SPT"; $resn_exch{" C1* SPT"}=" C1' SPT"; $resn_exch{" C3* SPT"}=" C3' SPT";
$resn_exch{" O3* SPT"}=" O3' SPT"; $resn_exch{" O3P SPT"}=" OP3 SPT"; $resn_exch{" O1P SPT"}=" OP1 SPT";
$resn_exch{" O2P SPT"}=" OP2 SPT"; $resn_exch{" O1P SRA"}=" OP1 SRA"; $resn_exch{" O3P SRA"}=" OP3 SRA";
$resn_exch{" O5* SRA"}=" O5' SRA"; $resn_exch{" C5* SRA"}=" C5' SRA"; $resn_exch{" C4* SRA"}=" C4' SRA";
$resn_exch{" O4* SRA"}=" O4' SRA"; $resn_exch{" C3* SRA"}=" C3' SRA"; $resn_exch{" O3* SRA"}=" O3' SRA";
$resn_exch{" C2* SRA"}=" C2' SRA"; $resn_exch{" C1* SRA"}=" C1' SRA"; $resn_exch{" O2* SRA"}=" O2' SRA";
$resn_exch{" N_A SRM"}=" NA  SRM"; $resn_exch{" N_B SRM"}=" NB  SRM"; $resn_exch{" N_C SRM"}=" NC  SRM";
$resn_exch{" N_D SRM"}=" ND  SRM"; $resn_exch{" O5* SRP"}=" O5' SRP"; $resn_exch{" C5* SRP"}=" C5' SRP";
$resn_exch{" C4* SRP"}=" C4' SRP"; $resn_exch{" O4* SRP"}=" O4' SRP"; $resn_exch{" C3* SRP"}=" C3' SRP";
$resn_exch{" O3* SRP"}=" O3' SRP"; $resn_exch{" C2* SRP"}=" C2' SRP"; $resn_exch{" O2* SRP"}=" O2' SRP";
$resn_exch{" C1* SRP"}=" C1' SRP"; $resn_exch{" O5* SSA"}=" O5' SSA"; $resn_exch{" C5* SSA"}=" C5' SSA";
$resn_exch{" C4* SSA"}=" C4' SSA"; $resn_exch{" O4* SSA"}=" O4' SSA"; $resn_exch{" C3* SSA"}=" C3' SSA";
$resn_exch{" O3* SSA"}=" O3' SSA"; $resn_exch{" C2* SSA"}=" C2' SSA"; $resn_exch{" O2* SSA"}=" O2' SSA";
$resn_exch{" C1* SSA"}=" C1' SSA"; $resn_exch{" O2* SSU"}=" O2' SSU"; $resn_exch{" C2* SSU"}=" C2' SSU";
$resn_exch{" C1* SSU"}=" C1' SSU"; $resn_exch{" O4* SSU"}=" O4' SSU"; $resn_exch{" C4* SSU"}=" C4' SSU";
$resn_exch{" C5* SSU"}=" C5' SSU"; $resn_exch{" O5* SSU"}=" O5' SSU"; $resn_exch{" O2P SSU"}=" OP2 SSU";
$resn_exch{" O3P SSU"}=" OP3 SSU"; $resn_exch{" C3* SSU"}=" C3' SSU"; $resn_exch{" O3* SSU"}=" O3' SSU";
$resn_exch{" O5* STP"}=" O5' STP"; $resn_exch{" C5* STP"}=" C5' STP"; $resn_exch{" C4* STP"}=" C4' STP";
$resn_exch{" O4* STP"}=" O4' STP"; $resn_exch{" C3* STP"}=" C3' STP"; $resn_exch{" O3* STP"}=" O3' STP";
$resn_exch{" C2* STP"}=" C2' STP"; $resn_exch{" C1* STP"}=" C1' STP"; $resn_exch{" N1* SU2"}=" N1' SU2";
$resn_exch{" C2* SU2"}=" C2B SU2"; $resn_exch{" C3* SU2"}=" C3B SU2"; $resn_exch{" N4* SU2"}=" N4' SU2";
$resn_exch{" C5* SU2"}=" C5B SU2"; $resn_exch{" C6* SU2"}=" C6B SU2"; $resn_exch{" C1* SUD"}=" C1' SUD";
$resn_exch{" C2* SUD"}=" C2' SUD"; $resn_exch{" O2* SUD"}=" O2' SUD"; $resn_exch{" C3* SUD"}=" C3' SUD";
$resn_exch{" C4* SUD"}=" C4' SUD"; $resn_exch{" O4* SUD"}=" O4' SUD"; $resn_exch{" O3* SUD"}=" O3' SUD";
$resn_exch{" C5* SUD"}=" C5' SUD"; $resn_exch{" O5* SUD"}=" O5' SUD"; $resn_exch{" O1P SUR"}=" OP1 SUR";
$resn_exch{" O2P SUR"}=" OP2 SUR"; $resn_exch{" O3P SUR"}=" OP3 SUR"; $resn_exch{" O3P   T"}=" OP3   T";
$resn_exch{" O1P   T"}=" OP1   T"; $resn_exch{" O2P   T"}=" OP2   T"; $resn_exch{" O5*   T"}=" O5'   T";
$resn_exch{" C5*   T"}=" C5'   T"; $resn_exch{" C4*   T"}=" C4'   T"; $resn_exch{" O4*   T"}=" O4'   T";
$resn_exch{" C3*   T"}=" C3'   T"; $resn_exch{" O3*   T"}=" O3'   T"; $resn_exch{" C2*   T"}=" C2'   T";
$resn_exch{" C1*   T"}=" C1'   T"; $resn_exch{" C5M   T"}=" C7    T"; $resn_exch{" O1P T23"}=" OP1 T23";
$resn_exch{" O2P T23"}=" OP2 T23"; $resn_exch{" O3P T23"}=" OP3 T23"; $resn_exch{" O5* T23"}=" O5' T23";
$resn_exch{" C2* T23"}=" C2' T23"; $resn_exch{" O2* T23"}=" O2' T23"; $resn_exch{" C5* T23"}=" C5' T23";
$resn_exch{" C4* T23"}=" C4' T23"; $resn_exch{" O4* T23"}=" O4' T23"; $resn_exch{" C1* T23"}=" C1' T23";
$resn_exch{" C3* T23"}=" C3' T23"; $resn_exch{" O3P T2S"}=" OP3 T2S"; $resn_exch{" O5* T2S"}=" O5' T2S";
$resn_exch{" C5* T2S"}=" C5' T2S"; $resn_exch{" C4* T2S"}=" C4' T2S"; $resn_exch{" O4* T2S"}=" O4' T2S";
$resn_exch{" C3* T2S"}=" C3' T2S"; $resn_exch{" O3* T2S"}=" O3' T2S"; $resn_exch{" C2* T2S"}=" C2' T2S";
$resn_exch{" C1* T2S"}=" C1' T2S"; $resn_exch{" O1P T2T"}=" OP1 T2T"; $resn_exch{" O2P T2T"}=" OP2 T2T";
$resn_exch{" O5* T2T"}=" O5' T2T"; $resn_exch{" C5* T2T"}=" C5' T2T"; $resn_exch{" C4* T2T"}=" C4' T2T";
$resn_exch{" O4* T2T"}=" O4' T2T"; $resn_exch{" C1* T2T"}=" C1' T2T"; $resn_exch{" C2* T2T"}=" C2' T2T";
$resn_exch{" O2* T2T"}=" O2' T2T"; $resn_exch{" C3* T2T"}=" C3' T2T"; $resn_exch{" O3* T2T"}=" O3' T2T";
$resn_exch{" O3P T2T"}=" OP3 T2T"; $resn_exch{" O5* T31"}=" O5' T31"; $resn_exch{" C5* T31"}=" C5' T31";
$resn_exch{" C4* T31"}=" C4' T31"; $resn_exch{" O4* T31"}=" O4' T31"; $resn_exch{" C3* T31"}=" C3' T31";
$resn_exch{" O3* T31"}=" O3' T31"; $resn_exch{" C2* T31"}=" C2' T31"; $resn_exch{" O2* T31"}=" O2' T31";
$resn_exch{" C1* T31"}=" C1' T31"; $resn_exch{" O5* T32"}=" O5' T32"; $resn_exch{" C5* T32"}=" C5' T32";
$resn_exch{" C4* T32"}=" C4' T32"; $resn_exch{" C3* T32"}=" C3' T32"; $resn_exch{" O3* T32"}=" O3' T32";
$resn_exch{" C2* T32"}=" C2' T32"; $resn_exch{" C1* T32"}=" C1' T32"; $resn_exch{" C*  T32"}=" C'  T32";
$resn_exch{" CM* T32"}=" CM' T32"; $resn_exch{" O5* T36"}=" O5' T36"; $resn_exch{" C5* T36"}=" C5' T36";
$resn_exch{" C4* T36"}=" C4' T36"; $resn_exch{" O4* T36"}=" O4' T36"; $resn_exch{" C3* T36"}=" C3' T36";
$resn_exch{" O3* T36"}=" O3' T36"; $resn_exch{" C2* T36"}=" C2' T36"; $resn_exch{" C1* T36"}=" C1' T36";
$resn_exch{" O5* T37"}=" O5' T37"; $resn_exch{" C5* T37"}=" C5' T37"; $resn_exch{" C4* T37"}=" C4' T37";
$resn_exch{" O4* T37"}=" O4' T37"; $resn_exch{" C3* T37"}=" C3' T37"; $resn_exch{" N3* T37"}=" N3' T37";
$resn_exch{" C2* T37"}=" C2' T37"; $resn_exch{" C1* T37"}=" C1' T37"; $resn_exch{" O5* T38"}=" O5' T38";
$resn_exch{" C5* T38"}=" C5' T38"; $resn_exch{" C4* T38"}=" C4' T38"; $resn_exch{" O4* T38"}=" O4' T38";
$resn_exch{" C3* T38"}=" C3' T38"; $resn_exch{" O3* T38"}=" O3' T38"; $resn_exch{" C2* T38"}=" C2' T38";
$resn_exch{" O2* T38"}=" O2' T38"; $resn_exch{" C1* T38"}=" C1' T38"; $resn_exch{" CA* T38"}=" CA' T38";
$resn_exch{" CB* T38"}=" CB' T38"; $resn_exch{" OC* T38"}=" OC' T38"; $resn_exch{" CD* T38"}=" CD' T38";
$resn_exch{" O5* T39"}=" O5' T39"; $resn_exch{" C5* T39"}=" C5' T39"; $resn_exch{" C4* T39"}=" C4' T39";
$resn_exch{" O4* T39"}=" O4' T39"; $resn_exch{" C3* T39"}=" C3' T39"; $resn_exch{" O3* T39"}=" O3' T39";
$resn_exch{" C2* T39"}=" C2' T39"; $resn_exch{" O2* T39"}=" O2' T39"; $resn_exch{" C1* T39"}=" C1' T39";
$resn_exch{" CA* T39"}=" CA' T39"; $resn_exch{" CD* T39"}=" CD' T39"; $resn_exch{" CB* T39"}=" CB' T39";
$resn_exch{" OC* T39"}=" OC' T39"; $resn_exch{" O1P T3P"}=" OP1 T3P"; $resn_exch{" O2P T3P"}=" OP2 T3P";
$resn_exch{" O3P T3P"}=" OP3 T3P"; $resn_exch{" O5* T3P"}=" O5' T3P"; $resn_exch{" C5* T3P"}=" C5' T3P";
$resn_exch{" C4* T3P"}=" C4' T3P"; $resn_exch{" O4* T3P"}=" O4' T3P"; $resn_exch{" C3* T3P"}=" C3' T3P";
$resn_exch{" O3* T3P"}=" O3' T3P"; $resn_exch{" C2* T3P"}=" C2' T3P"; $resn_exch{" C1* T3P"}=" C1' T3P";
$resn_exch{" C1* T41"}=" C1' T41"; $resn_exch{" C2* T41"}=" C2' T41"; $resn_exch{" O2* T41"}=" O2' T41";
$resn_exch{" C3* T41"}=" C3' T41"; $resn_exch{" C4* T41"}=" C4' T41"; $resn_exch{" O3* T41"}=" O3' T41";
$resn_exch{" O4* T41"}=" O4' T41"; $resn_exch{" C5* T41"}=" C5' T41"; $resn_exch{" O5* T41"}=" O5' T41";
$resn_exch{" C6* T41"}=" C6' T41"; $resn_exch{" O5* T48"}=" O5' T48"; $resn_exch{" C5* T48"}=" C5' T48";
$resn_exch{" C4* T48"}=" C4' T48"; $resn_exch{" C3* T48"}=" C3' T48"; $resn_exch{" O3* T48"}=" O3' T48";
$resn_exch{" C2* T48"}=" C2' T48"; $resn_exch{" C1* T48"}=" C1' T48"; $resn_exch{" O5* T49"}=" O5' T49";
$resn_exch{" C5* T49"}=" C5' T49"; $resn_exch{" C4* T49"}=" C4' T49"; $resn_exch{" C3* T49"}=" C3' T49";
$resn_exch{" O3* T49"}=" O3' T49"; $resn_exch{" C2* T49"}=" C2' T49"; $resn_exch{" C1* T49"}=" C1' T49";
$resn_exch{" O3P T4S"}=" OP3 T4S"; $resn_exch{" O1P T4S"}=" OP1 T4S"; $resn_exch{" O2P T4S"}=" OP2 T4S";
$resn_exch{" O5* T4S"}=" O5' T4S"; $resn_exch{" C2* T4S"}=" C2' T4S"; $resn_exch{" C5* T4S"}=" C5' T4S";
$resn_exch{" C4* T4S"}=" C4' T4S"; $resn_exch{" O4* T4S"}=" O4' T4S"; $resn_exch{" C1* T4S"}=" C1' T4S";
$resn_exch{" C3* T4S"}=" C3' T4S"; $resn_exch{" O3* T4S"}=" O3' T4S"; $resn_exch{" BN1 T5A"}=" N1B T5A";
$resn_exch{" BC6 T5A"}=" C6B T5A"; $resn_exch{" BC2 T5A"}=" C2B T5A"; $resn_exch{" BO2 T5A"}=" O2B T5A";
$resn_exch{" BN3 T5A"}=" N3B T5A"; $resn_exch{" BC4 T5A"}=" C4B T5A"; $resn_exch{" BO4 T5A"}=" O4B T5A";
$resn_exch{" BC5 T5A"}=" C5B T5A"; $resn_exch{" BC7 T5A"}=" C7B T5A"; $resn_exch{"BC5* T5A"}=" C5E T5A";
$resn_exch{"BC4* T5A"}=" C4E T5A"; $resn_exch{"BO4* T5A"}=" O4E T5A"; $resn_exch{"BC1* T5A"}=" C1E T5A";
$resn_exch{"BC2* T5A"}=" C2E T5A"; $resn_exch{"BC3* T5A"}=" C3E T5A"; $resn_exch{"BO3* T5A"}=" O3E T5A";
$resn_exch{"BO5* T5A"}=" O5E T5A"; $resn_exch{" O2B T5A"}=" O2X T5A"; $resn_exch{"AO5* T5A"}=" O5F T5A";
$resn_exch{"AC5* T5A"}=" C5F T5A"; $resn_exch{"AC4* T5A"}=" C4F T5A"; $resn_exch{"AO4* T5A"}=" O4F T5A";
$resn_exch{"AC1* T5A"}=" C1F T5A"; $resn_exch{"AC2* T5A"}=" C2F T5A"; $resn_exch{"AO2* T5A"}=" O2F T5A";
$resn_exch{"AC3* T5A"}=" C3F T5A"; $resn_exch{"AO3* T5A"}=" O3F T5A"; $resn_exch{" AN9 T5A"}=" N9A T5A";
$resn_exch{" AC4 T5A"}=" C4A T5A"; $resn_exch{" AN3 T5A"}=" N3A T5A"; $resn_exch{" AC2 T5A"}=" C2A T5A";
$resn_exch{" AN1 T5A"}=" N1A T5A"; $resn_exch{" AC6 T5A"}=" C6A T5A"; $resn_exch{" AN6 T5A"}=" N6A T5A";
$resn_exch{" AC5 T5A"}=" C5A T5A"; $resn_exch{" AN7 T5A"}=" N7A T5A"; $resn_exch{" AC8 T5A"}=" C8A T5A";
$resn_exch{" O1P T6A"}=" OP1 T6A"; $resn_exch{" O2P T6A"}=" OP2 T6A"; $resn_exch{" O3P T6A"}=" OP3 T6A";
$resn_exch{" O5* T6A"}=" O5' T6A"; $resn_exch{" C5* T6A"}=" C5' T6A"; $resn_exch{" C4* T6A"}=" C4' T6A";
$resn_exch{" O4* T6A"}=" O4' T6A"; $resn_exch{" C3* T6A"}=" C3' T6A"; $resn_exch{" O3* T6A"}=" O3' T6A";
$resn_exch{" C2* T6A"}=" C2' T6A"; $resn_exch{" O2* T6A"}=" O2' T6A"; $resn_exch{" C1* T6A"}=" C1' T6A";
$resn_exch{"O13A T6A"}=" ODA T6A"; $resn_exch{"O13B T6A"}=" ODB T6A"; $resn_exch{" O3P TA3"}=" OP3 TA3";
$resn_exch{" O1P TA3"}=" OP1 TA3"; $resn_exch{" O2P TA3"}=" OP2 TA3"; $resn_exch{" O5* TA3"}=" O5' TA3";
$resn_exch{" C5* TA3"}=" C5' TA3"; $resn_exch{" C4* TA3"}=" C4' TA3"; $resn_exch{" O4* TA3"}=" O4' TA3";
$resn_exch{" C3* TA3"}=" C3' TA3"; $resn_exch{" O3* TA3"}=" O3' TA3"; $resn_exch{" C2* TA3"}=" C2' TA3";
$resn_exch{" C1* TA3"}=" C1' TA3"; $resn_exch{" AP  TAD"}=" PA  TAD"; $resn_exch{" AO1 TAD"}=" O1A TAD";
$resn_exch{" AO2 TAD"}=" O2A TAD"; $resn_exch{"AO5* TAD"}=" O5B TAD"; $resn_exch{"AC5* TAD"}=" C5B TAD";
$resn_exch{"AC4* TAD"}=" C4B TAD"; $resn_exch{"AO4* TAD"}=" O4B TAD"; $resn_exch{"AC3* TAD"}=" C3B TAD";
$resn_exch{"AO3* TAD"}=" O3B TAD"; $resn_exch{"AC2* TAD"}=" C2B TAD"; $resn_exch{"AO2* TAD"}=" O2B TAD";
$resn_exch{"AC1* TAD"}=" C1B TAD"; $resn_exch{" AN9 TAD"}=" N9A TAD"; $resn_exch{" AC8 TAD"}=" C8A TAD";
$resn_exch{" AN7 TAD"}=" N7A TAD"; $resn_exch{" AC5 TAD"}=" C5A TAD"; $resn_exch{" AC6 TAD"}=" C6A TAD";
$resn_exch{" AN6 TAD"}=" N6A TAD"; $resn_exch{" AN1 TAD"}=" N1A TAD"; $resn_exch{" AC2 TAD"}=" C2A TAD";
$resn_exch{" AN3 TAD"}=" N3A TAD"; $resn_exch{" AC4 TAD"}=" C4A TAD"; $resn_exch{" NP  TAD"}=" PN  TAD";
$resn_exch{" NO1 TAD"}=" O1N TAD"; $resn_exch{" NO2 TAD"}=" O2N TAD"; $resn_exch{"NO5* TAD"}=" O5D TAD";
$resn_exch{"NC5* TAD"}=" C5D TAD"; $resn_exch{"NC4* TAD"}=" C4D TAD"; $resn_exch{"NO4* TAD"}=" O4D TAD";
$resn_exch{"NC3* TAD"}=" C3D TAD"; $resn_exch{"NO3* TAD"}=" O3D TAD"; $resn_exch{"NC2* TAD"}=" C2D TAD";
$resn_exch{"NO2* TAD"}=" O2D TAD"; $resn_exch{"NC1* TAD"}=" C1D TAD"; $resn_exch{" NC2 TAD"}=" C2N TAD";
$resn_exch{" NN3 TAD"}=" N3N TAD"; $resn_exch{" NC4 TAD"}=" C4N TAD"; $resn_exch{" NC5 TAD"}=" C5N TAD";
$resn_exch{" NS1 TAD"}=" S1N TAD"; $resn_exch{" NC6 TAD"}=" C6N TAD"; $resn_exch{" NO6 TAD"}=" O6N TAD";
$resn_exch{" NN6 TAD"}=" N6N TAD"; $resn_exch{" O1P TAF"}=" OP1 TAF"; $resn_exch{" O2P TAF"}=" OP2 TAF";
$resn_exch{" O3P TAF"}=" OP3 TAF"; $resn_exch{" O5* TAF"}=" O5' TAF"; $resn_exch{" F2* TAF"}=" F2' TAF";
$resn_exch{" C2* TAF"}=" C2' TAF"; $resn_exch{" C5* TAF"}=" C5' TAF"; $resn_exch{" C4* TAF"}=" C4' TAF";
$resn_exch{" O4* TAF"}=" O4' TAF"; $resn_exch{" C1* TAF"}=" C1' TAF"; $resn_exch{" C3* TAF"}=" C3' TAF";
$resn_exch{" O3* TAF"}=" O3' TAF"; $resn_exch{" AP  TAP"}=" PA  TAP"; $resn_exch{" AO1 TAP"}=" O1A TAP";
$resn_exch{" AO2 TAP"}=" O2A TAP"; $resn_exch{"AO5* TAP"}=" O5B TAP"; $resn_exch{"AC5* TAP"}=" C5B TAP";
$resn_exch{"AC4* TAP"}=" C4B TAP"; $resn_exch{"AO4* TAP"}=" O4B TAP"; $resn_exch{"AC3* TAP"}=" C3B TAP";
$resn_exch{"AO3* TAP"}=" O3B TAP"; $resn_exch{"AC2* TAP"}=" C2B TAP"; $resn_exch{"AO2* TAP"}=" O2B TAP";
$resn_exch{"AC1* TAP"}=" C1B TAP"; $resn_exch{" AN9 TAP"}=" N9A TAP"; $resn_exch{" AC8 TAP"}=" C8A TAP";
$resn_exch{" AN7 TAP"}=" N7A TAP"; $resn_exch{" AC5 TAP"}=" C5A TAP"; $resn_exch{" AC6 TAP"}=" C6A TAP";
$resn_exch{" AN6 TAP"}=" N6A TAP"; $resn_exch{" AN1 TAP"}=" N1A TAP"; $resn_exch{" AC2 TAP"}=" C2A TAP";
$resn_exch{" AN3 TAP"}=" N3A TAP"; $resn_exch{" AC4 TAP"}=" C4A TAP"; $resn_exch{" NP  TAP"}=" PN  TAP";
$resn_exch{" NO1 TAP"}=" O1N TAP"; $resn_exch{" NO2 TAP"}=" O2N TAP"; $resn_exch{"NO5* TAP"}=" O5D TAP";
$resn_exch{"NC5* TAP"}=" C5D TAP"; $resn_exch{"NC4* TAP"}=" C4D TAP"; $resn_exch{"NO4* TAP"}=" O4D TAP";
$resn_exch{"NC3* TAP"}=" C3D TAP"; $resn_exch{"NO3* TAP"}=" O3D TAP"; $resn_exch{"NC2* TAP"}=" C2D TAP";
$resn_exch{"NO2* TAP"}=" O2D TAP"; $resn_exch{"NC1* TAP"}=" C1D TAP"; $resn_exch{" NN1 TAP"}=" N1N TAP";
$resn_exch{" NC2 TAP"}=" C2N TAP"; $resn_exch{" NC3 TAP"}=" C3N TAP"; $resn_exch{" NC7 TAP"}=" C7N TAP";
$resn_exch{" NS7 TAP"}=" S7N TAP"; $resn_exch{" NN7 TAP"}=" N7N TAP"; $resn_exch{" NC4 TAP"}=" C4N TAP";
$resn_exch{" NC5 TAP"}=" C5N TAP"; $resn_exch{" NC6 TAP"}=" C6N TAP"; $resn_exch{"AP2* TAP"}=" P2B TAP";
$resn_exch{"AOP1 TAP"}=" O1X TAP"; $resn_exch{"AOP2 TAP"}=" O2X TAP"; $resn_exch{"AOP3 TAP"}=" O3X TAP";
$resn_exch{" O5* TAT"}=" O5' TAT"; $resn_exch{" C5* TAT"}=" C5' TAT"; $resn_exch{" C4* TAT"}=" C4' TAT";
$resn_exch{" O4* TAT"}=" O4' TAT"; $resn_exch{" C3* TAT"}=" C3' TAT"; $resn_exch{" O3* TAT"}=" O3' TAT";
$resn_exch{" C2* TAT"}=" C2' TAT"; $resn_exch{" O2* TAT"}=" O2' TAT"; $resn_exch{" C1* TAT"}=" C1' TAT";
$resn_exch{" O5* TBD"}=" O5' TBD"; $resn_exch{" C5* TBD"}=" C5' TBD"; $resn_exch{" C4* TBD"}=" C4' TBD";
$resn_exch{" O4* TBD"}=" O4' TBD"; $resn_exch{" C1* TBD"}=" C1' TBD"; $resn_exch{" C2* TBD"}=" C2' TBD";
$resn_exch{" C3* TBD"}=" C3' TBD"; $resn_exch{" O3* TBD"}=" O3' TBD"; $resn_exch{"1H5  TBS"}=" H51 TBS";
$resn_exch{" O5* TC1"}=" O5' TC1"; $resn_exch{" C5* TC1"}=" C5' TC1"; $resn_exch{" C4* TC1"}=" C4' TC1";
$resn_exch{" C3* TC1"}=" C3' TC1"; $resn_exch{" C2* TC1"}=" C2' TC1"; $resn_exch{" C1* TC1"}=" C1' TC1";
$resn_exch{" O4* TC1"}=" O4' TC1"; $resn_exch{" O3* TC1"}=" O3' TC1"; $resn_exch{" O3P TC1"}=" OP3 TC1";
$resn_exch{" O5* TCP"}=" O5' TCP"; $resn_exch{" C5* TCP"}=" C5' TCP"; $resn_exch{" C4* TCP"}=" C4' TCP";
$resn_exch{" O4* TCP"}=" O4' TCP"; $resn_exch{" C3* TCP"}=" C3' TCP"; $resn_exch{" O3* TCP"}=" O3' TCP";
$resn_exch{" C2* TCP"}=" C2' TCP"; $resn_exch{" C1* TCP"}=" C1' TCP"; $resn_exch{" C5* TDI"}=" C5' TDI";
$resn_exch{" C4* TDI"}=" C4' TDI"; $resn_exch{" C1* TDI"}=" C1' TDI"; $resn_exch{" N1* TDI"}=" N1' TDI";
$resn_exch{" C2* TDI"}=" C2' TDI"; $resn_exch{" C3* TDI"}=" C3' TDI"; $resn_exch{" O3* TDI"}=" O3' TDI";
$resn_exch{" C7* TDK"}=" C7' TDK"; $resn_exch{" C5* TDK"}=" C5' TDK"; $resn_exch{" C6* TDK"}=" C6' TDK";
$resn_exch{" N1* TDK"}=" N1' TDK"; $resn_exch{" C2* TDK"}=" C2' TDK"; $resn_exch{" N3* TDK"}=" N3' TDK";
$resn_exch{" C4* TDK"}=" C4' TDK"; $resn_exch{" N4* TDK"}=" N4' TDK"; $resn_exch{" C7* TDL"}=" C7' TDL";
$resn_exch{" C5* TDL"}=" C5' TDL"; $resn_exch{" C6* TDL"}=" C6' TDL"; $resn_exch{" N1* TDL"}=" N1' TDL";
$resn_exch{" C2* TDL"}=" C2' TDL"; $resn_exch{" N3* TDL"}=" N3' TDL"; $resn_exch{" C4* TDL"}=" C4' TDL";
$resn_exch{" N4* TDL"}=" N4' TDL"; $resn_exch{" C7* TDM"}=" C7' TDM"; $resn_exch{" C5* TDM"}=" C5' TDM";
$resn_exch{" C4* TDM"}=" C4' TDM"; $resn_exch{" N4* TDM"}=" N4' TDM"; $resn_exch{" N3* TDM"}=" N3' TDM";
$resn_exch{" C2* TDM"}=" C2' TDM"; $resn_exch{" N1* TDM"}=" N1' TDM"; $resn_exch{" C6* TDM"}=" C6' TDM";
$resn_exch{" N1* TDP"}=" N1' TDP"; $resn_exch{" C2* TDP"}=" C2' TDP"; $resn_exch{" N3* TDP"}=" N3' TDP";
$resn_exch{" C4* TDP"}=" C4' TDP"; $resn_exch{" N4* TDP"}=" N4' TDP"; $resn_exch{" C5* TDP"}=" C5' TDP";
$resn_exch{" C6* TDP"}=" C6' TDP"; $resn_exch{" C1* TDX"}=" C1B TDX"; $resn_exch{" C2* TDX"}=" C2B TDX";
$resn_exch{" C3* TDX"}=" C3B TDX"; $resn_exch{" O3* TDX"}=" O3B TDX"; $resn_exch{" O4* TDX"}=" O4B TDX";
$resn_exch{" C4* TDX"}=" C4B TDX"; $resn_exch{" C5* TDX"}=" C5B TDX"; $resn_exch{" O5* TDX"}=" O5B TDX";
$resn_exch{" O1P TFE"}=" OP1 TFE"; $resn_exch{" O2P TFE"}=" OP2 TFE"; $resn_exch{" O5* TFE"}=" O5' TFE";
$resn_exch{" C5* TFE"}=" C5' TFE"; $resn_exch{" C4* TFE"}=" C4' TFE"; $resn_exch{" O4* TFE"}=" O4' TFE";
$resn_exch{" C1* TFE"}=" C1' TFE"; $resn_exch{" C2* TFE"}=" C2' TFE"; $resn_exch{" O2* TFE"}=" O2' TFE";
$resn_exch{" C3* TFE"}=" C3' TFE"; $resn_exch{" O3* TFE"}=" O3' TFE"; $resn_exch{" O3P TFE"}=" OP3 TFE";
$resn_exch{" O2P TFO"}=" OP2 TFO"; $resn_exch{" O1P TFO"}=" OP1 TFO"; $resn_exch{" O3P TFO"}=" OP3 TFO";
$resn_exch{" O1P TFT"}=" OP1 TFT"; $resn_exch{" O2P TFT"}=" OP2 TFT"; $resn_exch{" O3P TFT"}=" OP3 TFT";
$resn_exch{"OE11 TGF"}=" OEB TGF"; $resn_exch{"OE12 TGF"}=" OEC TGF"; $resn_exch{"OE13 TGF"}=" OED TGF";
$resn_exch{"OE23 TGF"}=" OEN TGF"; $resn_exch{" S5* TGP"}=" S5' TGP"; $resn_exch{" C2* TGP"}=" C2' TGP";
$resn_exch{" C5* TGP"}=" C5' TGP"; $resn_exch{" C4* TGP"}=" C4' TGP"; $resn_exch{" O4* TGP"}=" O4' TGP";
$resn_exch{" C1* TGP"}=" C1' TGP"; $resn_exch{" C3* TGP"}=" C3' TGP"; $resn_exch{" O3* TGP"}=" O3' TGP";
$resn_exch{" O3P TGP"}=" OP3 TGP"; $resn_exch{" O1P TGP"}=" OP1 TGP"; $resn_exch{" O2P TGP"}=" OP2 TGP";
$resn_exch{" O5* THM"}=" O5' THM"; $resn_exch{" C5* THM"}=" C5' THM"; $resn_exch{" C4* THM"}=" C4' THM";
$resn_exch{" O4* THM"}=" O4' THM"; $resn_exch{" C3* THM"}=" C3' THM"; $resn_exch{" O3* THM"}=" O3' THM";
$resn_exch{" C2* THM"}=" C2' THM"; $resn_exch{" C1* THM"}=" C1' THM"; $resn_exch{" O5* THP"}=" O5' THP";
$resn_exch{" C5* THP"}=" C5' THP"; $resn_exch{" C4* THP"}=" C4' THP"; $resn_exch{" O4* THP"}=" O4' THP";
$resn_exch{" C3* THP"}=" C3' THP"; $resn_exch{" O3* THP"}=" O3' THP"; $resn_exch{" C2* THP"}=" C2' THP";
$resn_exch{" C1* THP"}=" C1' THP"; $resn_exch{" O5* THS"}=" O5' THS"; $resn_exch{" C5* THS"}=" C5' THS";
$resn_exch{" C4* THS"}=" C4' THS"; $resn_exch{" O4* THS"}=" O4' THS"; $resn_exch{" C3* THS"}=" C3' THS";
$resn_exch{" O3* THS"}=" O3' THS"; $resn_exch{" C2* THS"}=" C2' THS"; $resn_exch{" C1* THS"}=" C1' THS";
$resn_exch{" C1* THU"}=" C1' THU"; $resn_exch{" C2* THU"}=" C2' THU"; $resn_exch{" C3* THU"}=" C3' THU";
$resn_exch{" O3* THU"}=" O3' THU"; $resn_exch{" C4* THU"}=" C4' THU"; $resn_exch{" O4* THU"}=" O4' THU";
$resn_exch{" C5* THU"}=" C5' THU"; $resn_exch{" O5* THU"}=" O5' THU"; $resn_exch{" C5* THV"}=" C5' THV";
$resn_exch{" C6* THV"}=" C6' THV"; $resn_exch{" N1* THV"}=" N1' THV"; $resn_exch{" C2* THV"}=" C2' THV";
$resn_exch{" N3* THV"}=" N3' THV"; $resn_exch{" C4* THV"}=" C4' THV"; $resn_exch{" N4* THV"}=" N4' THV";
$resn_exch{" C5* THW"}=" C5' THW"; $resn_exch{" C6* THW"}=" C6' THW"; $resn_exch{" N1* THW"}=" N1' THW";
$resn_exch{" C2* THW"}=" C2' THW"; $resn_exch{" N3* THW"}=" N3' THW"; $resn_exch{" C4* THW"}=" C4' THW";
$resn_exch{" N4* THW"}=" N4' THW"; $resn_exch{" C1* THX"}=" C1' THX"; $resn_exch{" C2* THX"}=" C2' THX";
$resn_exch{" C3* THX"}=" C3' THX"; $resn_exch{" C4* THX"}=" C4' THX"; $resn_exch{" C5* THX"}=" C5' THX";
$resn_exch{" O3* THX"}=" O3' THX"; $resn_exch{" O4* THX"}=" O4' THX"; $resn_exch{" O5* THX"}=" O5' THX";
$resn_exch{" C5* THY"}=" C5' THY"; $resn_exch{" C6* THY"}=" C6' THY"; $resn_exch{" N1* THY"}=" N1' THY";
$resn_exch{" C2* THY"}=" C2' THY"; $resn_exch{" N3* THY"}=" N3' THY"; $resn_exch{" C4* THY"}=" C4' THY";
$resn_exch{" N4* THY"}=" N4' THY"; $resn_exch{" C1* TIA"}=" C1' TIA"; $resn_exch{" C2* TIA"}=" C2' TIA";
$resn_exch{" O2* TIA"}=" O2' TIA"; $resn_exch{" C3* TIA"}=" C3' TIA"; $resn_exch{" O3* TIA"}=" O3' TIA";
$resn_exch{" C4* TIA"}=" C4' TIA"; $resn_exch{" O4* TIA"}=" O4' TIA"; $resn_exch{" C5* TIA"}=" C5' TIA";
$resn_exch{" O2* TIZ"}=" O2' TIZ"; $resn_exch{" C2* TIZ"}=" C2' TIZ"; $resn_exch{" C3* TIZ"}=" C3' TIZ";
$resn_exch{" O3* TIZ"}=" O3' TIZ"; $resn_exch{" C4* TIZ"}=" C4' TIZ"; $resn_exch{" C5* TIZ"}=" C5' TIZ";
$resn_exch{" O5* TIZ"}=" O5' TIZ"; $resn_exch{" O4* TIZ"}=" O4' TIZ"; $resn_exch{" C1* TIZ"}=" C1' TIZ";
$resn_exch{" O5* TLB"}=" O5' TLB"; $resn_exch{" C5* TLB"}=" C5' TLB"; $resn_exch{" C4* TLB"}=" C4' TLB";
$resn_exch{" O4* TLB"}=" O4' TLB"; $resn_exch{" C3* TLB"}=" C3' TLB"; $resn_exch{" O3* TLB"}=" O3' TLB";
$resn_exch{" C2* TLB"}=" C2' TLB"; $resn_exch{" O2* TLB"}=" O2' TLB"; $resn_exch{" C1* TLB"}=" C1' TLB";
$resn_exch{" C9* TLB"}=" C9' TLB"; $resn_exch{" O1P TLC"}=" OP1 TLC"; $resn_exch{" O2P TLC"}=" OP2 TLC";
$resn_exch{" O3P TLC"}=" OP3 TLC"; $resn_exch{" O5* TLC"}=" O5' TLC"; $resn_exch{" C5* TLC"}=" C5' TLC";
$resn_exch{" C4* TLC"}=" C4' TLC"; $resn_exch{" O4* TLC"}=" O4' TLC"; $resn_exch{" C1* TLC"}=" C1' TLC";
$resn_exch{" C2* TLC"}=" C2' TLC"; $resn_exch{" C3* TLC"}=" C3' TLC"; $resn_exch{" O3* TLC"}=" O3' TLC";
$resn_exch{" O1P TLN"}=" OP1 TLN"; $resn_exch{" O2P TLN"}=" OP2 TLN"; $resn_exch{" O3P TLN"}=" OP3 TLN";
$resn_exch{" O5* TLN"}=" O5' TLN"; $resn_exch{" C5* TLN"}=" C5' TLN"; $resn_exch{" C4* TLN"}=" C4' TLN";
$resn_exch{" O4* TLN"}=" O4' TLN"; $resn_exch{" C1* TLN"}=" C1' TLN"; $resn_exch{" C3* TLN"}=" C3' TLN";
$resn_exch{" C2* TLN"}=" C2' TLN"; $resn_exch{" O2* TLN"}=" O2' TLN"; $resn_exch{" O3* TLN"}=" O3' TLN";
$resn_exch{" C6* TLN"}=" C6' TLN"; $resn_exch{" O5* TMP"}=" O5' TMP"; $resn_exch{" C5* TMP"}=" C5' TMP";
$resn_exch{" C4* TMP"}=" C4' TMP"; $resn_exch{" O4* TMP"}=" O4' TMP"; $resn_exch{" C3* TMP"}=" C3' TMP";
$resn_exch{" O3* TMP"}=" O3' TMP"; $resn_exch{" C2* TMP"}=" C2' TMP"; $resn_exch{" C1* TMP"}=" C1' TMP";
$resn_exch{" C5* TND"}=" C5' TND"; $resn_exch{" C4* TND"}=" C4' TND"; $resn_exch{" O4* TND"}=" O4' TND";
$resn_exch{" C3* TND"}=" C3' TND"; $resn_exch{" O3* TND"}=" O3' TND"; $resn_exch{" C2* TND"}=" C2' TND";
$resn_exch{" N2* TND"}=" N2' TND"; $resn_exch{" C1* TND"}=" C1' TND"; $resn_exch{" O5* TND"}=" O5' TND";
$resn_exch{" OB  TNR"}=" OXT TNR"; $resn_exch{" OA  TNR"}=" O   TNR"; $resn_exch{" O2* TOE"}=" O2' TOE";
$resn_exch{" CA* TOE"}=" CA' TOE"; $resn_exch{" CB* TOE"}=" CB' TOE"; $resn_exch{" OC* TOE"}=" OC' TOE";
$resn_exch{" CD* TOE"}=" CD' TOE"; $resn_exch{" CE* TOE"}=" CE' TOE"; $resn_exch{" OF* TOE"}=" OF' TOE";
$resn_exch{" CG* TOE"}=" CG' TOE"; $resn_exch{" CH* TOE"}=" CH' TOE"; $resn_exch{" OI* TOE"}=" OI' TOE";
$resn_exch{" CK* TOE"}=" CK' TOE"; $resn_exch{" O1P TPC"}=" OP1 TPC"; $resn_exch{" O2P TPC"}=" OP2 TPC";
$resn_exch{" O3P TPC"}=" OP3 TPC"; $resn_exch{" O5* TPE"}=" O5' TPE"; $resn_exch{" C5* TPE"}=" C5' TPE";
$resn_exch{" C4* TPE"}=" C4' TPE"; $resn_exch{" O4* TPE"}=" O4' TPE"; $resn_exch{" C3* TPE"}=" C3' TPE";
$resn_exch{" O3* TPE"}=" O3' TPE"; $resn_exch{" C2* TPE"}=" C2' TPE"; $resn_exch{" C1* TPE"}=" C1' TPE";
$resn_exch{" O5* TPG"}=" O5' TPG"; $resn_exch{" C5* TPG"}=" C5' TPG"; $resn_exch{" C4* TPG"}=" C4' TPG";
$resn_exch{" O4* TPG"}=" O4' TPG"; $resn_exch{" C3* TPG"}=" C3' TPG"; $resn_exch{" O3* TPG"}=" O3' TPG";
$resn_exch{" C2* TPG"}=" C2' TPG"; $resn_exch{" O2* TPG"}=" O2' TPG"; $resn_exch{" C1* TPG"}=" C1' TPG";
$resn_exch{" N1* TPW"}=" N1' TPW"; $resn_exch{" C2* TPW"}=" C2' TPW"; $resn_exch{" N3* TPW"}=" N3' TPW";
$resn_exch{" C4* TPW"}=" C4' TPW"; $resn_exch{" N4* TPW"}=" N4' TPW"; $resn_exch{" C5* TPW"}=" C5' TPW";
$resn_exch{" C6* TPW"}=" C6' TPW"; $resn_exch{" C2* TQD"}=" C2D TQD"; $resn_exch{" C4* TQD"}=" C4D TQD";
$resn_exch{" C5* TQD"}=" C5D TQD"; $resn_exch{" C6* TQD"}=" C6D TQD"; $resn_exch{" C7* TQD"}=" C7' TQD";
$resn_exch{" C8* TQD"}=" C8' TQD"; $resn_exch{" N1* TQD"}=" N1' TQD"; $resn_exch{" N2* TQD"}=" N2' TQD";
$resn_exch{" N3* TQD"}=" N3' TQD"; $resn_exch{" N4* TQD"}=" N4' TQD"; $resn_exch{" N9* TQD"}=" N9' TQD";
$resn_exch{" C9* TQD"}=" C9' TQD"; $resn_exch{" C4* TQT"}=" C4B TQT"; $resn_exch{" C8* TQT"}=" C8B TQT";
$resn_exch{" C*0 TQT"}=" C'0 TQT"; $resn_exch{" C*1 TQT"}=" C'1 TQT"; $resn_exch{" C*2 TQT"}=" C'2 TQT";
$resn_exch{" C*3 TQT"}=" C'3 TQT"; $resn_exch{" C*4 TQT"}=" C'4 TQT"; $resn_exch{" C*5 TQT"}=" C'5 TQT";
$resn_exch{" C*6 TQT"}=" C'6 TQT"; $resn_exch{" C*7 TQT"}=" C'7 TQT"; $resn_exch{" N*0 TQT"}=" N'0 TQT";
$resn_exch{" O1P  TS"}=" OP1  TS"; $resn_exch{" O3P  TS"}=" OP3  TS"; $resn_exch{" O5*  TS"}=" O5'  TS";
$resn_exch{" C5*  TS"}=" C5'  TS"; $resn_exch{" C4*  TS"}=" C4'  TS"; $resn_exch{" O4*  TS"}=" O4'  TS";
$resn_exch{" C3*  TS"}=" C3'  TS"; $resn_exch{" O3*  TS"}=" O3'  TS"; $resn_exch{" C2*  TS"}=" C2'  TS";
$resn_exch{" C1*  TS"}=" C1'  TS"; $resn_exch{" O5* TSB"}=" O5' TSB"; $resn_exch{" C5* TSB"}=" C5' TSB";
$resn_exch{" C4* TSB"}=" C4' TSB"; $resn_exch{" O4* TSB"}=" O4' TSB"; $resn_exch{" C3* TSB"}=" C3' TSB";
$resn_exch{" O3* TSB"}=" O3' TSB"; $resn_exch{" C2* TSB"}=" C2' TSB"; $resn_exch{" O2* TSB"}=" O2' TSB";
$resn_exch{" C1* TSB"}=" C1' TSB"; $resn_exch{" O1P TSP"}=" OP1 TSP"; $resn_exch{" O2P TSP"}=" OP2 TSP";
$resn_exch{" O3P TSP"}=" OP3 TSP"; $resn_exch{" O5* TSP"}=" O5' TSP"; $resn_exch{" C5* TSP"}=" C5' TSP";
$resn_exch{" C4* TSP"}=" C4' TSP"; $resn_exch{" O4* TSP"}=" O4' TSP"; $resn_exch{" C3* TSP"}=" C3' TSP";
$resn_exch{" S3* TSP"}=" S3' TSP"; $resn_exch{" C2* TSP"}=" C2' TSP"; $resn_exch{" C1* TSP"}=" C1' TSP";
$resn_exch{" O5*  TT"}=" O5'  TT"; $resn_exch{" C5*  TT"}=" C5'  TT"; $resn_exch{" C4*  TT"}=" C4'  TT";
$resn_exch{" O4*  TT"}=" O4'  TT"; $resn_exch{" C3*  TT"}=" C3'  TT"; $resn_exch{" O3*  TT"}=" O3'  TT";
$resn_exch{" C2*  TT"}=" C2'  TT"; $resn_exch{" C1*  TT"}=" C1'  TT"; $resn_exch{" O1P TTD"}=" OP1 TTD";
$resn_exch{" O2P TTD"}=" OP2 TTD"; $resn_exch{" O3P TTD"}=" OP3 TTD"; $resn_exch{" O5* TTD"}=" O5' TTD";
$resn_exch{" C5* TTD"}=" C5' TTD"; $resn_exch{" O4* TTD"}=" O4' TTD"; $resn_exch{" C2* TTD"}=" C2' TTD";
$resn_exch{" C1* TTD"}=" C1' TTD"; $resn_exch{" C4* TTD"}=" C4' TTD"; $resn_exch{" C3* TTD"}=" C3' TTD";
$resn_exch{" O3* TTD"}=" O3' TTD"; $resn_exch{" C1* TTE"}=" C1' TTE"; $resn_exch{" C2* TTE"}=" C2' TTE";
$resn_exch{" C3* TTE"}=" C3' TTE"; $resn_exch{" O3* TTE"}=" O3' TTE"; $resn_exch{" C4* TTE"}=" C4' TTE";
$resn_exch{" O4* TTE"}=" O4' TTE"; $resn_exch{" C5* TTE"}=" C5' TTE"; $resn_exch{" O5* TTE"}=" O5' TTE";
$resn_exch{" O1P TTM"}=" OP1 TTM"; $resn_exch{" O2P TTM"}=" OP2 TTM"; $resn_exch{" O5* TTM"}=" O5' TTM";
$resn_exch{" C5* TTM"}=" C5' TTM"; $resn_exch{" C4* TTM"}=" C4' TTM"; $resn_exch{" O4* TTM"}=" O4' TTM";
$resn_exch{" C1* TTM"}=" C1' TTM"; $resn_exch{" C2* TTM"}=" C2' TTM"; $resn_exch{" C3* TTM"}=" C3' TTM";
$resn_exch{" O3* TTM"}=" O3' TTM"; $resn_exch{" O3P TTM"}=" OP3 TTM"; $resn_exch{" O5* TTP"}=" O5' TTP";
$resn_exch{" C5* TTP"}=" C5' TTP"; $resn_exch{" C4* TTP"}=" C4' TTP"; $resn_exch{" O4* TTP"}=" O4' TTP";
$resn_exch{" C3* TTP"}=" C3' TTP"; $resn_exch{" O3* TTP"}=" O3' TTP"; $resn_exch{" C2* TTP"}=" C2' TTP";
$resn_exch{" C1* TTP"}=" C1' TTP"; $resn_exch{" C3* TTS"}=" C3' TTS"; $resn_exch{" C2* TTS"}=" C2' TTS";
$resn_exch{" C1* TTS"}=" C1' TTS"; $resn_exch{" C6* TTS"}=" C6' TTS"; $resn_exch{" C5* TTS"}=" C5' TTS";
$resn_exch{" C4* TTS"}=" C4' TTS"; $resn_exch{" C1  TUO"}=" CAE TUO"; $resn_exch{" C2  TUO"}=" CAA TUO";
$resn_exch{" C3  TUO"}=" CAB TUO"; $resn_exch{" C4  TUO"}=" CAC TUO"; $resn_exch{" C5  TUO"}=" CAG TUO";
$resn_exch{" C6  TUO"}=" CAF TUO"; $resn_exch{" C7  TUO"}=" CAN TUO"; $resn_exch{" C8  TUO"}=" CAM TUO";
$resn_exch{" C9  TUO"}=" CAL TUO"; $resn_exch{" C10 TUO"}=" CAK TUO"; $resn_exch{" S1  TUO"}=" SAJ TUO";
$resn_exch{" O1  TUO"}=" OAS TUO"; $resn_exch{" O2  TUO"}=" OAD TUO"; $resn_exch{" N1  TUO"}=" NAI TUO";
$resn_exch{" C11 TUO"}=" CAT TUO"; $resn_exch{" C12 TUO"}=" CAU TUO"; $resn_exch{" C13 TUO"}=" CAV TUO";
$resn_exch{" N2  TUO"}=" NAW TUO"; $resn_exch{" C14 TUO"}=" CAO TUO"; $resn_exch{" C15 TUO"}=" CAP TUO";
$resn_exch{" O3  TUO"}=" OAH TUO"; $resn_exch{" N3  TUO"}=" NAQ TUO"; $resn_exch{" N4  TUO"}=" NAR TUO";
$resn_exch{"C1*  TXD"}="C1'A TXD"; $resn_exch{"O4*  TXD"}="O4'A TXD"; $resn_exch{"C2*  TXD"}="C2'A TXD";
$resn_exch{"O2*  TXD"}="O2'A TXD"; $resn_exch{"C3*  TXD"}="C3'A TXD"; $resn_exch{"O3*  TXD"}="O3'A TXD";
$resn_exch{"C4*  TXD"}="C4'A TXD"; $resn_exch{"C5*  TXD"}="C5'A TXD"; $resn_exch{"O5*  TXD"}="O5'A TXD";
$resn_exch{"O5*  TXP"}="O5'A TXP"; $resn_exch{"C5*  TXP"}="C5'A TXP"; $resn_exch{"C4*  TXP"}="C4'A TXP";
$resn_exch{"C3*  TXP"}="C3'A TXP"; $resn_exch{"O3*  TXP"}="O3'A TXP"; $resn_exch{"C2*  TXP"}="C2'A TXP";
$resn_exch{"O2*  TXP"}="O2'A TXP"; $resn_exch{"C1*  TXP"}="C1'A TXP"; $resn_exch{"O4*  TXP"}="O4'A TXP";
$resn_exch{" O5* TYA"}=" O5' TYA"; $resn_exch{" C5* TYA"}=" C5' TYA"; $resn_exch{" C4* TYA"}=" C4' TYA";
$resn_exch{" O4* TYA"}=" O4' TYA"; $resn_exch{" C3* TYA"}=" C3' TYA"; $resn_exch{" O3* TYA"}=" O3' TYA";
$resn_exch{" C2* TYA"}=" C2' TYA"; $resn_exch{" O2* TYA"}=" O2' TYA"; $resn_exch{" C1* TYA"}=" C1' TYA";
$resn_exch{" O5* TYD"}=" O5' TYD"; $resn_exch{" C5* TYD"}=" C5' TYD"; $resn_exch{" C4* TYD"}=" C4' TYD";
$resn_exch{" O4* TYD"}=" O4' TYD"; $resn_exch{" C3* TYD"}=" C3' TYD"; $resn_exch{" O3* TYD"}=" O3' TYD";
$resn_exch{" C2* TYD"}=" C2' TYD"; $resn_exch{" C1* TYD"}=" C1' TYD"; $resn_exch{" O5* TYU"}=" O5' TYU";
$resn_exch{" C5* TYU"}=" C5' TYU"; $resn_exch{" C4* TYU"}=" C4' TYU"; $resn_exch{" O4* TYU"}=" O4' TYU";
$resn_exch{" C1* TYU"}=" C1' TYU"; $resn_exch{" C2* TYU"}=" C2' TYU"; $resn_exch{" O2* TYU"}=" O2' TYU";
$resn_exch{" C3* TYU"}=" C3' TYU"; $resn_exch{" O3* TYU"}=" O3' TYU"; $resn_exch{" N1* TZD"}=" N1' TZD";
$resn_exch{" C2* TZD"}=" C2' TZD"; $resn_exch{" N3* TZD"}=" N3' TZD"; $resn_exch{" C4* TZD"}=" C4' TZD";
$resn_exch{" N4* TZD"}=" N4' TZD"; $resn_exch{" C5* TZD"}=" C5' TZD"; $resn_exch{" C6* TZD"}=" C6' TZD";
$resn_exch{" C1* TZE"}=" C1' TZE"; $resn_exch{" C2* TZE"}=" C2' TZE"; $resn_exch{" O3P   U"}=" OP3   U";
$resn_exch{" O1P   U"}=" OP1   U"; $resn_exch{" O2P   U"}=" OP2   U"; $resn_exch{" O5*   U"}=" O5'   U";
$resn_exch{" C5*   U"}=" C5'   U"; $resn_exch{" C4*   U"}=" C4'   U"; $resn_exch{" O4*   U"}=" O4'   U";
$resn_exch{" C3*   U"}=" C3'   U"; $resn_exch{" O3*   U"}=" O3'   U"; $resn_exch{" C2*   U"}=" C2'   U";
$resn_exch{" O2*   U"}=" O2'   U"; $resn_exch{" C1*   U"}=" C1'   U"; $resn_exch{" O3P U25"}=" OP3 U25";
$resn_exch{" O1P U25"}=" OP1 U25"; $resn_exch{" O2P U25"}=" OP2 U25"; $resn_exch{" O5* U25"}=" O5' U25";
$resn_exch{" C5* U25"}=" C5' U25"; $resn_exch{" C4* U25"}=" C4' U25"; $resn_exch{" O4* U25"}=" O4' U25";
$resn_exch{" C3* U25"}=" C3' U25"; $resn_exch{" O3* U25"}=" O3' U25"; $resn_exch{" C2* U25"}=" C2' U25";
$resn_exch{" O2* U25"}=" O2' U25"; $resn_exch{" C1* U25"}=" C1' U25"; $resn_exch{"C1*G U2G"}=" C1B U2G";
$resn_exch{"C2*G U2G"}=" C2B U2G"; $resn_exch{"O2*G U2G"}=" O2B U2G"; $resn_exch{"C3*G U2G"}=" C3B U2G";
$resn_exch{"O3*G U2G"}=" O3B U2G"; $resn_exch{"C4*G U2G"}=" C4B U2G"; $resn_exch{"O4*G U2G"}=" O4B U2G";
$resn_exch{"C5*G U2G"}=" C5B U2G"; $resn_exch{"O5*G U2G"}=" O5B U2G"; $resn_exch{"O5*U U2G"}=" O5D U2G";
$resn_exch{"C5*U U2G"}=" C5D U2G"; $resn_exch{"C4*U U2G"}=" C4D U2G"; $resn_exch{"O4*U U2G"}=" O4D U2G";
$resn_exch{"C3*U U2G"}=" C3D U2G"; $resn_exch{"O3*U U2G"}=" O3D U2G"; $resn_exch{"C2*U U2G"}=" C2D U2G";
$resn_exch{"O2*U U2G"}=" O2D U2G"; $resn_exch{"C1*U U2G"}=" C1D U2G"; $resn_exch{" O3P U2L"}=" OP3 U2L";
$resn_exch{" O1P U2L"}=" OP1 U2L"; $resn_exch{" O2P U2L"}=" OP2 U2L"; $resn_exch{" O5* U2L"}=" O5' U2L";
$resn_exch{" C5* U2L"}=" C5' U2L"; $resn_exch{" C4* U2L"}=" C4' U2L"; $resn_exch{" O4* U2L"}=" O4' U2L";
$resn_exch{" C3* U2L"}=" C3' U2L"; $resn_exch{" O3* U2L"}=" O3' U2L"; $resn_exch{" C2* U2L"}=" C2' U2L";
$resn_exch{" O2* U2L"}=" O2' U2L"; $resn_exch{" C1* U2L"}=" C1' U2L"; $resn_exch{" CA* U2L"}=" CA' U2L";
$resn_exch{" CB* U2L"}=" CB' U2L"; $resn_exch{" OC* U2L"}=" OC' U2L"; $resn_exch{" CD* U2L"}=" CD' U2L";
$resn_exch{" O3P U2N"}=" OP3 U2N"; $resn_exch{" O1P U2N"}=" OP1 U2N"; $resn_exch{" O2P U2N"}=" OP2 U2N";
$resn_exch{" O5* U2N"}=" O5' U2N"; $resn_exch{" C5* U2N"}=" C5' U2N"; $resn_exch{" C4* U2N"}=" C4' U2N";
$resn_exch{" O4* U2N"}=" O4' U2N"; $resn_exch{" C1* U2N"}=" C1' U2N"; $resn_exch{" C2* U2N"}=" C2' U2N";
$resn_exch{" N2* U2N"}=" N2' U2N"; $resn_exch{" C3* U2N"}=" C3' U2N"; $resn_exch{" O3* U2N"}=" O3' U2N";
$resn_exch{" O5* U31"}=" O5' U31"; $resn_exch{" C5* U31"}=" C5' U31"; $resn_exch{" C4* U31"}=" C4' U31";
$resn_exch{" O4* U31"}=" O4' U31"; $resn_exch{" C3* U31"}=" C3' U31"; $resn_exch{" O3* U31"}=" O3' U31";
$resn_exch{" C2* U31"}=" C2' U31"; $resn_exch{" O2* U31"}=" O2' U31"; $resn_exch{" C1* U31"}=" C1' U31";
$resn_exch{" CA* U31"}=" CA' U31"; $resn_exch{" CB* U31"}=" CB' U31"; $resn_exch{" ND* U31"}=" ND' U31";
$resn_exch{" C1* U33"}=" C1' U33"; $resn_exch{" C2* U33"}=" C2' U33"; $resn_exch{" C3* U33"}=" C3' U33";
$resn_exch{" C4* U33"}=" C4' U33"; $resn_exch{" O3* U33"}=" O3' U33"; $resn_exch{" O4* U33"}=" O4' U33";
$resn_exch{" C5* U33"}=" C5' U33"; $resn_exch{" O5* U33"}=" O5' U33"; $resn_exch{" C5* U34"}=" C5' U34";
$resn_exch{" C4* U34"}=" C4' U34"; $resn_exch{" O4* U34"}=" O4' U34"; $resn_exch{" C3* U34"}=" C3' U34";
$resn_exch{" O3* U34"}=" O3' U34"; $resn_exch{" C2* U34"}=" C2' U34"; $resn_exch{" O2* U34"}=" O2' U34";
$resn_exch{" C1* U34"}=" C1' U34"; $resn_exch{" O5* U36"}=" O5' U36"; $resn_exch{" C5* U36"}=" C5' U36";
$resn_exch{" C4* U36"}=" C4' U36"; $resn_exch{" O4* U36"}=" O4' U36"; $resn_exch{" C3* U36"}=" C3' U36";
$resn_exch{" O3* U36"}=" O3' U36"; $resn_exch{" C2* U36"}=" C2' U36"; $resn_exch{" O2* U36"}=" O2' U36";
$resn_exch{" C1* U36"}=" C1' U36"; $resn_exch{" CA* U36"}=" CA' U36"; $resn_exch{" CB* U36"}=" CB' U36";
$resn_exch{" CD* U36"}=" CD' U36"; $resn_exch{" OC* U36"}=" OC' U36"; $resn_exch{" O5* U37"}=" O5' U37";
$resn_exch{" C5* U37"}=" C5' U37"; $resn_exch{" C4* U37"}=" C4' U37"; $resn_exch{" O4* U37"}=" O4' U37";
$resn_exch{" C3* U37"}=" C3' U37"; $resn_exch{" O3* U37"}=" O3' U37"; $resn_exch{" C2* U37"}=" C2' U37";
$resn_exch{" O2* U37"}=" O2' U37"; $resn_exch{" C1* U37"}=" C1' U37"; $resn_exch{" C1* U3H"}=" C1' U3H";
$resn_exch{" C2* U3H"}=" C2' U3H"; $resn_exch{" O2* U3H"}=" O2' U3H"; $resn_exch{" C3* U3H"}=" C3' U3H";
$resn_exch{" C4* U3H"}=" C4' U3H"; $resn_exch{" O4* U3H"}=" O4' U3H"; $resn_exch{" C5* U3H"}=" C5' U3H";
$resn_exch{" O5* U3H"}=" O5' U3H"; $resn_exch{" O5* U3P"}=" O5' U3P"; $resn_exch{" C5* U3P"}=" C5' U3P";
$resn_exch{" C4* U3P"}=" C4' U3P"; $resn_exch{" O4* U3P"}=" O4' U3P"; $resn_exch{" C3* U3P"}=" C3' U3P";
$resn_exch{" O3* U3P"}=" O3' U3P"; $resn_exch{" C2* U3P"}=" C2' U3P"; $resn_exch{" O2* U3P"}=" O2' U3P";
$resn_exch{" C1* U3P"}=" C1' U3P"; $resn_exch{" C1* U5P"}=" C1' U5P"; $resn_exch{" C2* U5P"}=" C2' U5P";
$resn_exch{" O2* U5P"}=" O2' U5P"; $resn_exch{" C3* U5P"}=" C3' U5P"; $resn_exch{" C4* U5P"}=" C4' U5P";
$resn_exch{" O3* U5P"}=" O3' U5P"; $resn_exch{" O4* U5P"}=" O4' U5P"; $resn_exch{" C5* U5P"}=" C5' U5P";
$resn_exch{" O5* U5P"}=" O5' U5P"; $resn_exch{" C1* U8U"}=" C1' U8U"; $resn_exch{" O2* U8U"}=" O2' U8U";
$resn_exch{" C2* U8U"}=" C2' U8U"; $resn_exch{" O3* U8U"}=" O3' U8U"; $resn_exch{" C3* U8U"}=" C3' U8U";
$resn_exch{" C4* U8U"}=" C4' U8U"; $resn_exch{" O4* U8U"}=" O4' U8U"; $resn_exch{" C5* U8U"}=" C5' U8U";
$resn_exch{" O5* U8U"}=" O5' U8U"; $resn_exch{" O1P U8U"}=" OP1 U8U"; $resn_exch{" O2P U8U"}=" OP2 U8U";
$resn_exch{" O3P U8U"}=" OP3 U8U"; $resn_exch{" O5* UA2"}=" O5' UA2"; $resn_exch{" C5* UA2"}=" C5' UA2";
$resn_exch{" C4* UA2"}=" C4' UA2"; $resn_exch{" N4* UA2"}=" N4' UA2"; $resn_exch{" C3* UA2"}=" C3' UA2";
$resn_exch{" O3* UA2"}=" O3' UA2"; $resn_exch{" C2* UA2"}=" C2' UA2"; $resn_exch{" O2* UA2"}=" O2' UA2";
$resn_exch{" C1* UA2"}=" C1' UA2"; $resn_exch{" O5* UA3"}=" O5' UA3"; $resn_exch{" C5* UA3"}=" C5' UA3";
$resn_exch{" C4* UA3"}=" C4' UA3"; $resn_exch{" O4* UA3"}=" O4' UA3"; $resn_exch{" C3* UA3"}=" C3' UA3";
$resn_exch{" O3* UA3"}=" O3' UA3"; $resn_exch{" C2* UA3"}=" C2' UA3"; $resn_exch{" O2* UA3"}=" O2' UA3";
$resn_exch{" C1* UA3"}=" C1' UA3"; $resn_exch{" C1* UAG"}=" C1B UAG"; $resn_exch{" C2* UAG"}=" C2B UAG";
$resn_exch{" O2* UAG"}=" O2' UAG"; $resn_exch{" C3* UAG"}=" C3B UAG"; $resn_exch{" O3* UAG"}=" O3B UAG";
$resn_exch{" C4* UAG"}=" C4B UAG"; $resn_exch{" O4* UAG"}=" O4B UAG"; $resn_exch{" C5* UAG"}=" C5B UAG";
$resn_exch{" O5* UAG"}=" O5B UAG"; $resn_exch{" O3P UAR"}=" OP3 UAR"; $resn_exch{" O1P UAR"}=" OP1 UAR";
$resn_exch{" O2P UAR"}=" OP2 UAR"; $resn_exch{" O5* UAR"}=" O5' UAR"; $resn_exch{" C5* UAR"}=" C5' UAR";
$resn_exch{" C4* UAR"}=" C4' UAR"; $resn_exch{" O4* UAR"}=" O4' UAR"; $resn_exch{" C3* UAR"}=" C3' UAR";
$resn_exch{" O3* UAR"}=" O3' UAR"; $resn_exch{" C2* UAR"}=" C2' UAR"; $resn_exch{" O2* UAR"}=" O2' UAR";
$resn_exch{" C1* UAR"}=" C1' UAR"; $resn_exch{" O5* UBB"}=" O5' UBB"; $resn_exch{" C5* UBB"}=" C5' UBB";
$resn_exch{" C4* UBB"}=" C4' UBB"; $resn_exch{" O4* UBB"}=" O4' UBB"; $resn_exch{" C1* UBB"}=" C1' UBB";
$resn_exch{" C2* UBB"}=" C2' UBB"; $resn_exch{" C3* UBB"}=" C3' UBB"; $resn_exch{" C1* UC5"}=" C1' UC5";
$resn_exch{" C2* UC5"}=" C2' UC5"; $resn_exch{" C3* UC5"}=" C3' UC5"; $resn_exch{" O3* UC5"}=" O3' UC5";
$resn_exch{" O4* UC5"}=" O4' UC5"; $resn_exch{" C4* UC5"}=" C4' UC5"; $resn_exch{" C5* UC5"}=" C5' UC5";
$resn_exch{" O5* UC5"}=" O5' UC5"; $resn_exch{" O5* UCL"}=" O5' UCL"; $resn_exch{" C5* UCL"}=" C5' UCL";
$resn_exch{" C4* UCL"}=" C4' UCL"; $resn_exch{" O4* UCL"}=" O4' UCL"; $resn_exch{" C3* UCL"}=" C3' UCL";
$resn_exch{" O3* UCL"}=" O3' UCL"; $resn_exch{" C2* UCL"}=" C2' UCL"; $resn_exch{" C1* UCL"}=" C1' UCL";
$resn_exch{" C1* UCP"}=" C1' UCP"; $resn_exch{" C2* UCP"}=" C2' UCP"; $resn_exch{" C3* UCP"}=" C3' UCP";
$resn_exch{" O3* UCP"}=" O3' UCP"; $resn_exch{" C4* UCP"}=" C4' UCP"; $resn_exch{" O4* UCP"}=" O4' UCP";
$resn_exch{" C5* UCP"}=" C5' UCP"; $resn_exch{" O5* UCP"}=" O5' UCP"; $resn_exch{" C1* UD1"}=" C1B UD1";
$resn_exch{" C2* UD1"}=" C2B UD1"; $resn_exch{" O2* UD1"}=" O2' UD1"; $resn_exch{" C3* UD1"}=" C3B UD1";
$resn_exch{" C4* UD1"}=" C4B UD1"; $resn_exch{" O4* UD1"}=" O4B UD1"; $resn_exch{" O3* UD1"}=" O3B UD1";
$resn_exch{" C5* UD1"}=" C5B UD1"; $resn_exch{" O5* UD1"}=" O5B UD1"; $resn_exch{" C1* UD2"}=" C1B UD2";
$resn_exch{" C2* UD2"}=" C2B UD2"; $resn_exch{" O2* UD2"}=" O2' UD2"; $resn_exch{" C3* UD2"}=" C3B UD2";
$resn_exch{" C4* UD2"}=" C4B UD2"; $resn_exch{" O4* UD2"}=" O4B UD2"; $resn_exch{" O3* UD2"}=" O3B UD2";
$resn_exch{" C5* UD2"}=" C5B UD2"; $resn_exch{" O5* UD2"}=" O5B UD2"; $resn_exch{" O1P UD5"}=" OP1 UD5";
$resn_exch{" O2P UD5"}=" OP2 UD5"; $resn_exch{" O5* UD5"}=" O5' UD5"; $resn_exch{" C5* UD5"}=" C5' UD5";
$resn_exch{" C4* UD5"}=" C4' UD5"; $resn_exch{" O4* UD5"}=" O4' UD5"; $resn_exch{" C3* UD5"}=" C3' UD5";
$resn_exch{" O3* UD5"}=" O3' UD5"; $resn_exch{" C2* UD5"}=" C2' UD5"; $resn_exch{" O2* UD5"}=" O2' UD5";
$resn_exch{" C1* UD5"}=" C1' UD5"; $resn_exch{" C1* UDH"}=" C1B UDH"; $resn_exch{" C2* UDH"}=" C2B UDH";
$resn_exch{" O2* UDH"}=" O2' UDH"; $resn_exch{" C3* UDH"}=" C3B UDH"; $resn_exch{" C4* UDH"}=" C4B UDH";
$resn_exch{" O4* UDH"}=" O4' UDH"; $resn_exch{" O3* UDH"}=" O3' UDH"; $resn_exch{" C5* UDH"}=" C5B UDH";
$resn_exch{" O5* UDH"}=" O5' UDH"; $resn_exch{" C1* UDM"}=" C1D UDM"; $resn_exch{" C2* UDM"}=" C2D UDM";
$resn_exch{" O2* UDM"}=" O2' UDM"; $resn_exch{" C3* UDM"}=" C3D UDM"; $resn_exch{" C4* UDM"}=" C4D UDM";
$resn_exch{" O4* UDM"}=" O4D UDM"; $resn_exch{" O3* UDM"}=" O3D UDM"; $resn_exch{" C5* UDM"}=" C5D UDM";
$resn_exch{" O5* UDM"}=" O5D UDM"; $resn_exch{" C1* UDP"}=" C1' UDP"; $resn_exch{" C2* UDP"}=" C2' UDP";
$resn_exch{" O2* UDP"}=" O2' UDP"; $resn_exch{" C3* UDP"}=" C3' UDP"; $resn_exch{" C4* UDP"}=" C4' UDP";
$resn_exch{" O4* UDP"}=" O4' UDP"; $resn_exch{" O3* UDP"}=" O3' UDP"; $resn_exch{" C5* UDP"}=" C5' UDP";
$resn_exch{" O5* UDP"}=" O5' UDP"; $resn_exch{" O5* UDX"}=" O5D UDX"; $resn_exch{" C5* UDX"}=" C5D UDX";
$resn_exch{" C4* UDX"}=" C4D UDX"; $resn_exch{" O4* UDX"}=" O4D UDX"; $resn_exch{" C3* UDX"}=" C3D UDX";
$resn_exch{" O3* UDX"}=" O3D UDX"; $resn_exch{" C2* UDX"}=" C2D UDX"; $resn_exch{" O2* UDX"}=" O2D UDX";
$resn_exch{" C1* UDX"}=" C1D UDX"; $resn_exch{" C1* UFG"}=" C1D UFG"; $resn_exch{" C2* UFG"}=" C2D UFG";
$resn_exch{" O2* UFG"}=" O2D UFG"; $resn_exch{" C3* UFG"}=" C3D UFG"; $resn_exch{" C4* UFG"}=" C4D UFG";
$resn_exch{" O4* UFG"}=" O4' UFG"; $resn_exch{" O3* UFG"}=" O3D UFG"; $resn_exch{" C5* UFG"}=" C5D UFG";
$resn_exch{" O5* UFG"}=" O5D UFG"; $resn_exch{" C1* UFM"}=" C1D UFM"; $resn_exch{" C2* UFM"}=" C2D UFM";
$resn_exch{" O2* UFM"}=" O2D UFM"; $resn_exch{" C3* UFM"}=" C3D UFM"; $resn_exch{" C4* UFM"}=" C4D UFM";
$resn_exch{" O4* UFM"}=" O4D UFM"; $resn_exch{" O3* UFM"}=" O3D UFM"; $resn_exch{" C5* UFM"}=" C5D UFM";
$resn_exch{" O5* UFM"}=" O5D UFM"; $resn_exch{" C1* UFP"}=" C1' UFP"; $resn_exch{" C2* UFP"}=" C2' UFP";
$resn_exch{" C3* UFP"}=" C3' UFP"; $resn_exch{" C4* UFP"}=" C4' UFP"; $resn_exch{" O3* UFP"}=" O3' UFP";
$resn_exch{" O4* UFP"}=" O4' UFP"; $resn_exch{" C5* UFP"}=" C5' UFP"; $resn_exch{" O5* UFP"}=" O5' UFP";
$resn_exch{" O5* UFR"}=" O5' UFR"; $resn_exch{" C5* UFR"}=" C5' UFR"; $resn_exch{" C4* UFR"}=" C4' UFR";
$resn_exch{" O4* UFR"}=" O4' UFR"; $resn_exch{" C3* UFR"}=" C3' UFR"; $resn_exch{" O3* UFR"}=" O3' UFR";
$resn_exch{" C2* UFR"}=" C2' UFR"; $resn_exch{" C1* UFR"}=" C1' UFR"; $resn_exch{"O6'1 UGA"}=" O'P UGA";
$resn_exch{"O6'2 UGA"}=" O'Q UGA"; $resn_exch{" O5* UGA"}=" O5D UGA"; $resn_exch{" C5* UGA"}=" C5D UGA";
$resn_exch{" C4* UGA"}=" C4D UGA"; $resn_exch{" O4* UGA"}=" O4D UGA"; $resn_exch{" C3* UGA"}=" C3D UGA";
$resn_exch{" O3* UGA"}=" O3D UGA"; $resn_exch{" C2* UGA"}=" C2D UGA"; $resn_exch{" O2* UGA"}=" O2D UGA";
$resn_exch{" C1* UGA"}=" C1D UGA"; $resn_exch{" C1* UM3"}=" C1' UM3"; $resn_exch{" C2* UM3"}=" C2' UM3";
$resn_exch{" C3* UM3"}=" C3' UM3"; $resn_exch{" C4* UM3"}=" C4' UM3"; $resn_exch{" O3* UM3"}=" O3' UM3";
$resn_exch{" O4* UM3"}=" O4' UM3"; $resn_exch{" C5* UM3"}=" C5' UM3"; $resn_exch{" O5* UM3"}=" O5' UM3";
$resn_exch{" C1* UMA"}=" C1B UMA"; $resn_exch{" C2* UMA"}=" C2B UMA"; $resn_exch{" O2* UMA"}=" O2' UMA";
$resn_exch{" C3* UMA"}=" C3B UMA"; $resn_exch{" C4* UMA"}=" C4B UMA"; $resn_exch{" O4* UMA"}=" O4B UMA";
$resn_exch{" O3* UMA"}=" O3B UMA"; $resn_exch{" C5* UMA"}=" C5B UMA"; $resn_exch{" O5* UMA"}=" O5B UMA";
$resn_exch{" O5* UMF"}=" O5' UMF"; $resn_exch{" C5* UMF"}=" C5' UMF"; $resn_exch{" C4* UMF"}=" C4' UMF";
$resn_exch{" O4* UMF"}=" O4' UMF"; $resn_exch{" C3* UMF"}=" C3' UMF"; $resn_exch{" O3* UMF"}=" O3' UMF";
$resn_exch{" C2* UMF"}=" C2' UMF"; $resn_exch{" F2* UMF"}=" F2' UMF"; $resn_exch{" C1* UMF"}=" C1' UMF";
$resn_exch{" C1* UMP"}=" C1' UMP"; $resn_exch{" C2* UMP"}=" C2' UMP"; $resn_exch{" C3* UMP"}=" C3' UMP";
$resn_exch{" C4* UMP"}=" C4' UMP"; $resn_exch{" O3* UMP"}=" O3' UMP"; $resn_exch{" O4* UMP"}=" O4' UMP";
$resn_exch{" C5* UMP"}=" C5' UMP"; $resn_exch{" O5* UMP"}=" O5' UMP"; $resn_exch{" O1P UMP"}=" OP1 UMP";
$resn_exch{" O2P UMP"}=" OP2 UMP"; $resn_exch{" O3P UMP"}=" OP3 UMP"; $resn_exch{" O3P UMS"}=" OP3 UMS";
$resn_exch{" O1P UMS"}=" OP1 UMS"; $resn_exch{" O2P UMS"}=" OP2 UMS"; $resn_exch{" C4* UN4"}=" C4' UN4";
$resn_exch{" C3* UN4"}=" C3' UN4"; $resn_exch{" C2* UN4"}=" C2' UN4"; $resn_exch{" C1* UP1"}=" C1' UP1";
$resn_exch{" C2* UP1"}=" C2' UP1"; $resn_exch{" O2* UP1"}=" O2' UP1"; $resn_exch{" C3* UP1"}=" C3' UP1";
$resn_exch{" C4* UP1"}=" C4' UP1"; $resn_exch{" O4* UP1"}=" O4' UP1"; $resn_exch{" O3* UP1"}=" O3' UP1";
$resn_exch{" C5* UP1"}=" C5' UP1"; $resn_exch{" O5* UP1"}=" O5' UP1"; $resn_exch{" AN9 UP5"}=" N9A UP5";
$resn_exch{" AC8 UP5"}=" C8A UP5"; $resn_exch{" AN7 UP5"}=" N7A UP5"; $resn_exch{" AC5 UP5"}=" C5A UP5";
$resn_exch{" AC6 UP5"}=" C6A UP5"; $resn_exch{" AN6 UP5"}=" N6A UP5"; $resn_exch{" AN1 UP5"}=" N1A UP5";
$resn_exch{" AC2 UP5"}=" C2A UP5"; $resn_exch{" AN3 UP5"}=" N3A UP5"; $resn_exch{" AC4 UP5"}=" C4A UP5";
$resn_exch{"AO5* UP5"}=" O5F UP5"; $resn_exch{"AC5* UP5"}=" C5F UP5"; $resn_exch{"AC4* UP5"}=" C4F UP5";
$resn_exch{"AO4* UP5"}=" O4F UP5"; $resn_exch{"AC3* UP5"}=" C3F UP5"; $resn_exch{"AO3* UP5"}=" O3F UP5";
$resn_exch{"AC2* UP5"}=" C2F UP5"; $resn_exch{"AO2* UP5"}=" O2F UP5"; $resn_exch{"AC1* UP5"}=" C1F UP5";
$resn_exch{"UO5* UP5"}=" O5J UP5"; $resn_exch{"UC5* UP5"}=" C5J UP5"; $resn_exch{"UC4* UP5"}=" C4J UP5";
$resn_exch{"UO4* UP5"}=" O4J UP5"; $resn_exch{"UC3* UP5"}=" C3J UP5"; $resn_exch{"UO3* UP5"}=" O3J UP5";
$resn_exch{"UC2* UP5"}=" C2J UP5"; $resn_exch{"UO2* UP5"}=" O2J UP5"; $resn_exch{"UC1* UP5"}=" C1J UP5";
$resn_exch{" UN1 UP5"}=" N1U UP5"; $resn_exch{" UC2 UP5"}=" C2U UP5"; $resn_exch{"UO2' UP5"}=" O2X UP5";
$resn_exch{" UN3 UP5"}=" N3U UP5"; $resn_exch{" UC4 UP5"}=" C4U UP5"; $resn_exch{" UO4 UP5"}=" O4U UP5";
$resn_exch{" UC5 UP5"}=" C5U UP5"; $resn_exch{" UC6 UP5"}=" C6U UP5"; $resn_exch{" C1* UP6"}=" C1' UP6";
$resn_exch{" C2* UP6"}=" C2' UP6"; $resn_exch{" C3* UP6"}=" C3' UP6"; $resn_exch{" C4* UP6"}=" C4' UP6";
$resn_exch{" O2* UP6"}=" O2' UP6"; $resn_exch{" O3* UP6"}=" O3' UP6"; $resn_exch{" O4* UP6"}=" O4' UP6";
$resn_exch{" C5* UP6"}=" C5' UP6"; $resn_exch{" O5* UP6"}=" O5' UP6"; $resn_exch{"C5*U UPA"}=" C5D UPA";
$resn_exch{"O5*U UPA"}=" O5D UPA"; $resn_exch{"C4*U UPA"}=" C4D UPA"; $resn_exch{"O4*U UPA"}=" O4D UPA";
$resn_exch{"C3*U UPA"}=" C3D UPA"; $resn_exch{"O3*U UPA"}=" O3D UPA"; $resn_exch{"C2*U UPA"}=" C2D UPA";
$resn_exch{"O2*U UPA"}=" O2D UPA"; $resn_exch{"C1*U UPA"}=" C1D UPA"; $resn_exch{"O5*A UPA"}=" O5B UPA";
$resn_exch{"C5*A UPA"}=" C5B UPA"; $resn_exch{"C4*A UPA"}=" C4B UPA"; $resn_exch{"O4*A UPA"}=" O4B UPA";
$resn_exch{"C3*A UPA"}=" C3B UPA"; $resn_exch{"O3*A UPA"}=" O3B UPA"; $resn_exch{"C2*A UPA"}=" C2B UPA";
$resn_exch{"O2*A UPA"}=" O2B UPA"; $resn_exch{"C1*A UPA"}=" C1B UPA"; $resn_exch{" C1* UPF"}=" C1D UPF";
$resn_exch{" C2* UPF"}=" C2D UPF"; $resn_exch{" O2* UPF"}=" O2' UPF"; $resn_exch{" C3* UPF"}=" C3D UPF";
$resn_exch{" C4* UPF"}=" C4D UPF"; $resn_exch{" O4* UPF"}=" O4D UPF"; $resn_exch{" O3* UPF"}=" O3D UPF";
$resn_exch{" C5* UPF"}=" C5D UPF"; $resn_exch{" O5* UPF"}=" O5D UPF"; $resn_exch{" C1* UPG"}=" C1C UPG";
$resn_exch{" C2* UPG"}=" C2C UPG"; $resn_exch{" O2* UPG"}=" O2C UPG"; $resn_exch{" C3* UPG"}=" C3C UPG";
$resn_exch{" C4* UPG"}=" C4C UPG"; $resn_exch{" O4* UPG"}=" O4C UPG"; $resn_exch{" O3* UPG"}=" O3C UPG";
$resn_exch{" C5* UPG"}=" C5C UPG"; $resn_exch{" O5* UPG"}=" O5C UPG"; $resn_exch{" C1* UPM"}=" C1' UPM";
$resn_exch{" C2* UPM"}=" C2' UPM"; $resn_exch{" O2* UPM"}=" O2' UPM"; $resn_exch{" C3* UPM"}=" C3' UPM";
$resn_exch{" O3* UPM"}=" O3' UPM"; $resn_exch{" C4* UPM"}=" C4' UPM"; $resn_exch{" O4* UPM"}=" O4' UPM";
$resn_exch{" C5* UPM"}=" C5' UPM"; $resn_exch{" O5* UPM"}=" O5' UPM"; $resn_exch{" C1* UPP"}=" C1B UPP";
$resn_exch{" C2* UPP"}=" C2B UPP"; $resn_exch{" O2* UPP"}=" O2' UPP"; $resn_exch{" C3* UPP"}=" C3B UPP";
$resn_exch{" C4* UPP"}=" C4B UPP"; $resn_exch{" O4* UPP"}=" O4' UPP"; $resn_exch{" O3* UPP"}=" O3' UPP";
$resn_exch{" C5* UPP"}=" C5B UPP"; $resn_exch{" O5* UPP"}=" O5' UPP"; $resn_exch{" C5* UPR"}=" C5' UPR";
$resn_exch{" O5* UPR"}=" O5' UPR"; $resn_exch{" C4* UPR"}=" C4' UPR"; $resn_exch{" O4* UPR"}=" O4' UPR";
$resn_exch{" C3* UPR"}=" C3' UPR"; $resn_exch{" O3* UPR"}=" O3' UPR"; $resn_exch{" C2* UPR"}=" C2' UPR";
$resn_exch{" C1* UPR"}=" C1' UPR"; $resn_exch{"C1*A UPU"}="C1'A UPU"; $resn_exch{"O4*A UPU"}="O4'A UPU";
$resn_exch{"C2*A UPU"}="C2'A UPU"; $resn_exch{"O2*A UPU"}="O2'A UPU"; $resn_exch{"C3*A UPU"}="C3'A UPU";
$resn_exch{"O3*A UPU"}="O3'A UPU"; $resn_exch{"C4*A UPU"}="C4'A UPU"; $resn_exch{"C5*A UPU"}="C5'A UPU";
$resn_exch{"O5*A UPU"}="O5'A UPU"; $resn_exch{"O3*B UPU"}="O3'B UPU"; $resn_exch{"C3*B UPU"}="C3'B UPU";
$resn_exch{"C4*B UPU"}="C4'B UPU"; $resn_exch{"C5*B UPU"}="C5'B UPU"; $resn_exch{"O5*B UPU"}="O5'B UPU";
$resn_exch{"O4*B UPU"}="O4'B UPU"; $resn_exch{"C2*B UPU"}="C2'B UPU"; $resn_exch{"O2*B UPU"}="O2'B UPU";
$resn_exch{"C1*B UPU"}="C1'B UPU"; $resn_exch{" O1P UR3"}=" OP1 UR3"; $resn_exch{" O2P UR3"}=" OP2 UR3";
$resn_exch{" O5* UR3"}=" O5' UR3"; $resn_exch{" C5* UR3"}=" C5' UR3"; $resn_exch{" C4* UR3"}=" C4' UR3";
$resn_exch{" O4* UR3"}=" O4' UR3"; $resn_exch{" C1* UR3"}=" C1' UR3"; $resn_exch{" C2* UR3"}=" C2' UR3";
$resn_exch{" O2* UR3"}=" O2' UR3"; $resn_exch{" C3* UR3"}=" C3' UR3"; $resn_exch{" O3* UR3"}=" O3' UR3";
$resn_exch{" O3P UR3"}=" OP3 UR3"; $resn_exch{" O5* URD"}=" O5' URD"; $resn_exch{" C5* URD"}=" C5' URD";
$resn_exch{" C4* URD"}=" C4' URD"; $resn_exch{" O4* URD"}=" O4' URD"; $resn_exch{" C1* URD"}=" C1' URD";
$resn_exch{" C2* URD"}=" C2' URD"; $resn_exch{" O2* URD"}=" O2' URD"; $resn_exch{" C3* URD"}=" C3' URD";
$resn_exch{" O3* URD"}=" O3' URD"; $resn_exch{" C1* URI"}=" C1' URI"; $resn_exch{" C2* URI"}=" C2' URI";
$resn_exch{" C3* URI"}=" C3' URI"; $resn_exch{" C4* URI"}=" C4' URI"; $resn_exch{" O2* URI"}=" O2' URI";
$resn_exch{" O3* URI"}=" O3' URI"; $resn_exch{" O4* URI"}=" O4' URI"; $resn_exch{" C5* URI"}=" C5' URI";
$resn_exch{" O5* URI"}=" O5' URI"; $resn_exch{" O5* US1"}=" O5' US1"; $resn_exch{" C5* US1"}=" C5' US1";
$resn_exch{" C4* US1"}=" C4' US1"; $resn_exch{" O4* US1"}=" O4' US1"; $resn_exch{" C3* US1"}=" C3' US1";
$resn_exch{" C2* US1"}=" C2' US1"; $resn_exch{" C1* US1"}=" C1' US1"; $resn_exch{" C1* USQ"}=" C1D USQ";
$resn_exch{" C2* USQ"}=" C2D USQ"; $resn_exch{" O2* USQ"}=" O2D USQ"; $resn_exch{" C3* USQ"}=" C3D USQ";
$resn_exch{" C4* USQ"}=" C4D USQ"; $resn_exch{" O4* USQ"}=" O4D USQ"; $resn_exch{" O3* USQ"}=" O3D USQ";
$resn_exch{" C5* USQ"}=" C5D USQ"; $resn_exch{" O5* USQ"}=" O5D USQ"; $resn_exch{" O5* UTP"}=" O5' UTP";
$resn_exch{" C5* UTP"}=" C5' UTP"; $resn_exch{" C4* UTP"}=" C4' UTP"; $resn_exch{" O4* UTP"}=" O4' UTP";
$resn_exch{" C1* UTP"}=" C1' UTP"; $resn_exch{" C2* UTP"}=" C2' UTP"; $resn_exch{" O2* UTP"}=" O2' UTP";
$resn_exch{" C3* UTP"}=" C3' UTP"; $resn_exch{" O3* UTP"}=" O3' UTP"; $resn_exch{" C1* UVC"}=" C1' UVC";
$resn_exch{" C2* UVC"}=" C2' UVC"; $resn_exch{" O2* UVC"}=" O2' UVC"; $resn_exch{" C3* UVC"}=" C3' UVC";
$resn_exch{" C4* UVC"}=" C4' UVC"; $resn_exch{" O3* UVC"}=" O3' UVC"; $resn_exch{" O4* UVC"}=" O4' UVC";
$resn_exch{" C5* UVC"}=" C5' UVC"; $resn_exch{" O5* UVC"}=" O5' UVC"; $resn_exch{" N5* VAA"}=" N5' VAA";
$resn_exch{" C5* VAA"}=" C5' VAA"; $resn_exch{" C4* VAA"}=" C4' VAA"; $resn_exch{" O4* VAA"}=" O4' VAA";
$resn_exch{" C1* VAA"}=" C1' VAA"; $resn_exch{" C2* VAA"}=" C2' VAA"; $resn_exch{" O2* VAA"}=" O2' VAA";
$resn_exch{" C3* VAA"}=" C3' VAA"; $resn_exch{" O3* VAA"}=" O3' VAA"; $resn_exch{" C1\" VAN"}=" C1B VAN";
$resn_exch{" C2\" VAN"}=" C2B VAN"; $resn_exch{" C3\" VAN"}=" C3B VAN"; $resn_exch{" N3\" VAN"}=" N3' VAN";
$resn_exch{" C4\" VAN"}=" C4B VAN"; $resn_exch{" O4\" VAN"}=" O4B VAN"; $resn_exch{" C5\" VAN"}=" C5B VAN";
$resn_exch{" O5\" VAN"}=" O5B VAN"; $resn_exch{" C1\" VAX"}=" C1B VAX"; $resn_exch{" C2\" VAX"}=" C2B VAX";
$resn_exch{" C3\" VAX"}=" C3B VAX"; $resn_exch{" N3\" VAX"}=" N3' VAX"; $resn_exch{" C4\" VAX"}=" C4B VAX";
$resn_exch{" O4\" VAX"}=" O4B VAX"; $resn_exch{" C5\" VAX"}=" C5B VAX"; $resn_exch{" O5\" VAX"}=" O5B VAX";
$resn_exch{" N1* VDM"}=" N1' VDM"; $resn_exch{" C1* VDM"}=" C1' VDM"; $resn_exch{" C2* VDM"}=" C2' VDM";
$resn_exch{" O2* VDM"}=" O2' VDM"; $resn_exch{" C6* VDM"}=" C6' VDM"; $resn_exch{" C5* VDM"}=" C5' VDM";
$resn_exch{" C7* VDM"}=" C7' VDM"; $resn_exch{" O7* VDM"}=" O7' VDM"; $resn_exch{" C4* VDM"}=" C4' VDM";
$resn_exch{" O4* VDM"}=" O4' VDM"; $resn_exch{" C3* VDM"}=" C3' VDM"; $resn_exch{" O3* VDM"}=" O3' VDM";
$resn_exch{" O5* VMS"}=" O5' VMS"; $resn_exch{" C5* VMS"}=" C5' VMS"; $resn_exch{" C4* VMS"}=" C4' VMS";
$resn_exch{" O4* VMS"}=" O4' VMS"; $resn_exch{" C3* VMS"}=" C3' VMS"; $resn_exch{" O3* VMS"}=" O3' VMS";
$resn_exch{" C2* VMS"}=" C2' VMS"; $resn_exch{" O2* VMS"}=" O2' VMS"; $resn_exch{" C1* VMS"}=" C1' VMS";
$resn_exch{"1H    VX"}=" H1   VX"; $resn_exch{" C5* WWF"}=" C5' WWF"; $resn_exch{" C6* WWF"}=" C6' WWF";
$resn_exch{" N1* WWF"}=" N1' WWF"; $resn_exch{" C2* WWF"}=" C2' WWF"; $resn_exch{" N3* WWF"}=" N3' WWF";
$resn_exch{" C4* WWF"}=" C4' WWF"; $resn_exch{" N4* WWF"}=" N4' WWF"; $resn_exch{" O1P   X"}=" OP1   X";
$resn_exch{" O2P   X"}=" OP2   X"; $resn_exch{" O3P   X"}=" OP3   X"; $resn_exch{" O5*   X"}=" O5'   X";
$resn_exch{" C5*   X"}=" C5'   X"; $resn_exch{" C4*   X"}=" C4'   X"; $resn_exch{" O4*   X"}=" O4'   X";
$resn_exch{" C3*   X"}=" C3'   X"; $resn_exch{" O3*   X"}=" O3'   X"; $resn_exch{" C2*   X"}=" C2'   X";
$resn_exch{" C1*   X"}=" C1'   X"; $resn_exch{" O3P X4A"}=" OP3 X4A"; $resn_exch{" O1P X4A"}=" OP1 X4A";
$resn_exch{" O2P X4A"}=" OP2 X4A"; $resn_exch{" O5* X4A"}=" O5' X4A"; $resn_exch{" C5* X4A"}=" C5' X4A";
$resn_exch{" C4* X4A"}=" C4' X4A"; $resn_exch{" O4* X4A"}=" O4' X4A"; $resn_exch{" C1* X4A"}=" C1' X4A";
$resn_exch{" C3* X4A"}=" C3' X4A"; $resn_exch{" C2* X4A"}=" C2' X4A"; $resn_exch{" O3* X4A"}=" O3' X4A";
$resn_exch{" O3P XAD"}=" OP3 XAD"; $resn_exch{" O1P XAD"}=" OP1 XAD"; $resn_exch{" O2P XAD"}=" OP2 XAD";
$resn_exch{" O1P XAE"}=" OP1 XAE"; $resn_exch{" O2P XAE"}=" OP2 XAE"; $resn_exch{" O3P XAE"}=" OP3 XAE";
$resn_exch{" O1P XAL"}=" OP1 XAL"; $resn_exch{" O2P XAL"}=" OP2 XAL"; $resn_exch{" O5* XAL"}=" O5' XAL";
$resn_exch{" C5* XAL"}=" C5' XAL"; $resn_exch{" C4* XAL"}=" C4' XAL"; $resn_exch{" C6* XAL"}=" C6' XAL";
$resn_exch{" C7* XAL"}=" C7' XAL"; $resn_exch{" C3* XAL"}=" C3' XAL"; $resn_exch{" O3* XAL"}=" O3' XAL";
$resn_exch{" C2* XAL"}=" C2' XAL"; $resn_exch{" C1* XAL"}=" C1' XAL"; $resn_exch{" O3P XAL"}=" OP3 XAL";
$resn_exch{" O5* XAR"}=" O5' XAR"; $resn_exch{" C5* XAR"}=" C5' XAR"; $resn_exch{" C4* XAR"}=" C4' XAR";
$resn_exch{" C3* XAR"}=" C3' XAR"; $resn_exch{" O3* XAR"}=" O3' XAR"; $resn_exch{" C2* XAR"}=" C2' XAR";
$resn_exch{" C1* XAR"}=" C1' XAR"; $resn_exch{" C7* XAR"}=" C7' XAR"; $resn_exch{" C6* XAR"}=" C6' XAR";
$resn_exch{" O1P XCL"}=" OP1 XCL"; $resn_exch{" O2P XCL"}=" OP2 XCL"; $resn_exch{" O5* XCL"}=" O5' XCL";
$resn_exch{" C5* XCL"}=" C5' XCL"; $resn_exch{" C4* XCL"}=" C4' XCL"; $resn_exch{" C6* XCL"}=" C6' XCL";
$resn_exch{" C7* XCL"}=" C7' XCL"; $resn_exch{" C3* XCL"}=" C3' XCL"; $resn_exch{" O3* XCL"}=" O3' XCL";
$resn_exch{" C2* XCL"}=" C2' XCL"; $resn_exch{" C1* XCL"}=" C1' XCL"; $resn_exch{" O3P XCL"}=" OP3 XCL";
$resn_exch{" O2P XCS"}=" OP2 XCS"; $resn_exch{" O1P XCS"}=" OP1 XCS"; $resn_exch{" O3P XCS"}=" OP3 XCS";
$resn_exch{" O3P XCT"}=" OP3 XCT"; $resn_exch{" O1P XCT"}=" OP1 XCT"; $resn_exch{" O2P XCT"}=" OP2 XCT";
$resn_exch{" O3P XCY"}=" OP3 XCY"; $resn_exch{" O2P XCY"}=" OP2 XCY"; $resn_exch{" O5* XCY"}=" O5' XCY";
$resn_exch{" C2* XCY"}=" C2' XCY"; $resn_exch{" C5* XCY"}=" C5' XCY"; $resn_exch{" C4* XCY"}=" C4' XCY";
$resn_exch{" O4* XCY"}=" O4' XCY"; $resn_exch{" C1* XCY"}=" C1' XCY"; $resn_exch{" C3* XCY"}=" C3' XCY";
$resn_exch{" O3* XCY"}=" O3' XCY"; $resn_exch{" O1P XCY"}=" OP1 XCY"; $resn_exch{" O1P XGA"}=" OP1 XGA";
$resn_exch{" O2P XGA"}=" OP2 XGA"; $resn_exch{" O3P XGA"}=" OP3 XGA"; $resn_exch{" O1P XGL"}=" OP1 XGL";
$resn_exch{" O2P XGL"}=" OP2 XGL"; $resn_exch{" O5* XGL"}=" O5' XGL"; $resn_exch{" C5* XGL"}=" C5' XGL";
$resn_exch{" C4* XGL"}=" C4' XGL"; $resn_exch{" C6* XGL"}=" C6' XGL"; $resn_exch{" C7* XGL"}=" C7' XGL";
$resn_exch{" C3* XGL"}=" C3' XGL"; $resn_exch{" O3* XGL"}=" O3' XGL"; $resn_exch{" C2* XGL"}=" C2' XGL";
$resn_exch{" C1* XGL"}=" C1' XGL"; $resn_exch{" O3P XGL"}=" OP3 XGL"; $resn_exch{" O3P XGU"}=" OP3 XGU";
$resn_exch{" O1P XGU"}=" OP1 XGU"; $resn_exch{" O2P XGU"}=" OP2 XGU"; $resn_exch{"C12' XR2"}=" CCX XR2";
$resn_exch{"C13' XR2"}=" CDX XR2"; $resn_exch{"C14' XR2"}=" CEX XR2"; $resn_exch{"N10' XR2"}=" NAX XR2";
$resn_exch{"C11' XR2"}=" CBX XR2"; $resn_exch{"C9A' XR2"}=" C9X XR2"; $resn_exch{" O3P XTH"}=" OP3 XTH";
$resn_exch{" O1P XTH"}=" OP1 XTH"; $resn_exch{" O2P XTH"}=" OP2 XTH"; $resn_exch{" O1P XTL"}=" OP1 XTL";
$resn_exch{" O2P XTL"}=" OP2 XTL"; $resn_exch{" O5* XTL"}=" O5' XTL"; $resn_exch{" C5* XTL"}=" C5' XTL";
$resn_exch{" C4* XTL"}=" C4' XTL"; $resn_exch{" C6* XTL"}=" C6' XTL"; $resn_exch{" C7* XTL"}=" C7' XTL";
$resn_exch{" C3* XTL"}=" C3' XTL"; $resn_exch{" O3* XTL"}=" O3' XTL"; $resn_exch{" C2* XTL"}=" C2' XTL";
$resn_exch{" C1* XTL"}=" C1' XTL"; $resn_exch{" O3P XTL"}=" OP3 XTL"; $resn_exch{" C1* XTS"}=" C1' XTS";
$resn_exch{" O4* XTS"}=" O4' XTS"; $resn_exch{" C4* XTS"}=" C4' XTS"; $resn_exch{" C5* XTS"}=" C5' XTS";
$resn_exch{" O5* XTS"}=" O5' XTS"; $resn_exch{" C3* XTS"}=" C3' XTS"; $resn_exch{" O3* XTS"}=" O3' XTS";
$resn_exch{" C2* XTS"}=" C2' XTS"; $resn_exch{" O2* XTS"}=" O2' XTS"; $resn_exch{" O1P XTY"}=" OP1 XTY";
$resn_exch{" O2P XTY"}=" OP2 XTY"; $resn_exch{" O3P XTY"}=" OP3 XTY"; $resn_exch{" C1* XUG"}=" C1' XUG";
$resn_exch{" O4* XUG"}=" O4' XUG"; $resn_exch{" C2* XUG"}=" C2' XUG"; $resn_exch{"SE2* XUG"}="SE2' XUG";
$resn_exch{" CA* XUG"}=" CA' XUG"; $resn_exch{" C3* XUG"}=" C3' XUG"; $resn_exch{" O3* XUG"}=" O3' XUG";
$resn_exch{" C4* XUG"}=" C4' XUG"; $resn_exch{" C5* XUG"}=" C5' XUG"; $resn_exch{" O5* XUG"}=" O5' XUG";
$resn_exch{" O1P XUG"}=" OP1 XUG"; $resn_exch{" O2P XUG"}=" OP2 XUG"; $resn_exch{" O3P XUG"}=" OP3 XUG";
$resn_exch{" O1P   Y"}=" OP1   Y"; $resn_exch{" O2P   Y"}=" OP2   Y"; $resn_exch{" O3P   Y"}=" OP3   Y";
$resn_exch{" O5*   Y"}=" O5'   Y"; $resn_exch{" C5*   Y"}=" C5'   Y"; $resn_exch{" C4*   Y"}=" C4'   Y";
$resn_exch{" O4*   Y"}=" O4'   Y"; $resn_exch{" C3*   Y"}=" C3'   Y"; $resn_exch{" O3*   Y"}=" O3'   Y";
$resn_exch{" C2*   Y"}=" C2'   Y"; $resn_exch{" C1*   Y"}=" C1'   Y"; $resn_exch{" AN1 YE1"}=" N1A YE1";
$resn_exch{" AC2 YE1"}=" C2A YE1"; $resn_exch{" AN3 YE1"}=" N3A YE1"; $resn_exch{" AC4 YE1"}=" C4A YE1";
$resn_exch{" AC5 YE1"}=" C5A YE1"; $resn_exch{" AC6 YE1"}=" C6A YE1"; $resn_exch{" AN6 YE1"}=" N6A YE1";
$resn_exch{" AN7 YE1"}=" N7A YE1"; $resn_exch{" AC8 YE1"}=" C8A YE1"; $resn_exch{" AN9 YE1"}=" N9A YE1";
$resn_exch{" C1* YE1"}=" C1' YE1"; $resn_exch{" C2* YE1"}=" C2' YE1"; $resn_exch{" O2* YE1"}=" O2' YE1";
$resn_exch{" C3* YE1"}=" C3' YE1"; $resn_exch{" O3* YE1"}=" O3' YE1"; $resn_exch{" P3* YE1"}=" P3' YE1";
$resn_exch{" AO7 YE1"}=" O7A YE1"; $resn_exch{" AO8 YE1"}=" O8A YE1"; $resn_exch{" AO9 YE1"}=" O9A YE1";
$resn_exch{" C4* YE1"}=" C4' YE1"; $resn_exch{" O4* YE1"}=" O4' YE1"; $resn_exch{" C5* YE1"}=" C5' YE1";
$resn_exch{" O5* YE1"}=" O5' YE1"; $resn_exch{" AP1 YE1"}=" P1A YE1"; $resn_exch{" AO1 YE1"}=" O1A YE1";
$resn_exch{" AO2 YE1"}=" O2A YE1"; $resn_exch{" AO3 YE1"}=" O3A YE1"; $resn_exch{" AP2 YE1"}=" P2A YE1";
$resn_exch{" AO4 YE1"}=" O4A YE1"; $resn_exch{" AO5 YE1"}=" O5A YE1"; $resn_exch{" AO6 YE1"}=" O6A YE1";
$resn_exch{" PC9 YE1"}=" C9P YE1"; $resn_exch{" PO9 YE1"}=" O9P YE1"; $resn_exch{" PN8 YE1"}=" N8P YE1";
$resn_exch{" PC7 YE1"}=" C7P YE1"; $resn_exch{" PC6 YE1"}=" C6P YE1"; $resn_exch{" PC5 YE1"}=" C5P YE1";
$resn_exch{" PO5 YE1"}=" O5P YE1"; $resn_exch{" PN4 YE1"}=" N4P YE1"; $resn_exch{" PC3 YE1"}=" C3P YE1";
$resn_exch{" PC2 YE1"}=" C2P YE1"; $resn_exch{" C1*  YG"}=" C1'  YG"; $resn_exch{" C2*  YG"}=" C2'  YG";
$resn_exch{" O2*  YG"}=" O2'  YG"; $resn_exch{" C3*  YG"}=" C3'  YG"; $resn_exch{" O3*  YG"}=" O3'  YG";
$resn_exch{" C4*  YG"}=" C4'  YG"; $resn_exch{" O4*  YG"}=" O4'  YG"; $resn_exch{" C5*  YG"}=" C5'  YG";
$resn_exch{" O5*  YG"}=" O5'  YG"; $resn_exch{" O1P  YG"}=" OP1  YG"; $resn_exch{" O2P  YG"}=" OP2  YG";
$resn_exch{" O3P  YG"}=" OP3  YG"; $resn_exch{" O5* YLY"}=" O5' YLY"; $resn_exch{" C5* YLY"}=" C5' YLY";
$resn_exch{" C4* YLY"}=" C4' YLY"; $resn_exch{" O4* YLY"}=" O4' YLY"; $resn_exch{" C3* YLY"}=" C3' YLY";
$resn_exch{" O3* YLY"}=" O3' YLY"; $resn_exch{" C2* YLY"}=" C2' YLY"; $resn_exch{" O2* YLY"}=" O2' YLY";
$resn_exch{" C1* YLY"}=" C1' YLY"; $resn_exch{" O5* YMP"}=" O5' YMP"; $resn_exch{" C5* YMP"}=" C5' YMP";
$resn_exch{" C4* YMP"}=" C4' YMP"; $resn_exch{" O4* YMP"}=" O4' YMP"; $resn_exch{" C1* YMP"}=" C1' YMP";
$resn_exch{" C2* YMP"}=" C2' YMP"; $resn_exch{" C3* YMP"}=" C3' YMP"; $resn_exch{" O3* YMP"}=" O3' YMP";
$resn_exch{" O2* YMP"}=" O2' YMP"; $resn_exch{" O3P YRR"}=" OP3 YRR"; $resn_exch{" O1P YRR"}=" OP1 YRR";
$resn_exch{" O2P YRR"}=" OP2 YRR"; $resn_exch{" O5* YRR"}=" O5' YRR"; $resn_exch{" C5* YRR"}=" C5' YRR";
$resn_exch{" C4* YRR"}=" C4' YRR"; $resn_exch{" N4* YRR"}=" N4' YRR"; $resn_exch{" C3* YRR"}=" C3' YRR";
$resn_exch{" O3* YRR"}=" O3' YRR"; $resn_exch{" C2* YRR"}=" C2' YRR"; $resn_exch{" C1* YRR"}=" C1' YRR";
$resn_exch{" O5* YSA"}=" O5' YSA"; $resn_exch{" C5* YSA"}=" C5' YSA"; $resn_exch{" C4* YSA"}=" C4' YSA";
$resn_exch{" O4* YSA"}=" O4' YSA"; $resn_exch{" C3* YSA"}=" C3' YSA"; $resn_exch{" O3* YSA"}=" O3' YSA";
$resn_exch{" C2* YSA"}=" C2' YSA"; $resn_exch{" O2* YSA"}=" O2' YSA"; $resn_exch{" C1* YSA"}=" C1' YSA";
$resn_exch{" O1P YYG"}=" OP1 YYG"; $resn_exch{" O2P YYG"}=" OP2 YYG"; $resn_exch{" O3P YYG"}=" OP3 YYG";
$resn_exch{" O5* YYG"}=" O5' YYG"; $resn_exch{" C5* YYG"}=" C5' YYG"; $resn_exch{" C4* YYG"}=" C4' YYG";
$resn_exch{" O4* YYG"}=" O4' YYG"; $resn_exch{" C3* YYG"}=" C3' YYG"; $resn_exch{" O3* YYG"}=" O3' YYG";
$resn_exch{" C2* YYG"}=" C2' YYG"; $resn_exch{" O2* YYG"}=" O2' YYG"; $resn_exch{" C1* YYG"}=" C1' YYG";
$resn_exch{" O5* YYY"}=" O5' YYY"; $resn_exch{" C5* YYY"}=" C5' YYY"; $resn_exch{" C4* YYY"}=" C4' YYY";
$resn_exch{" O4* YYY"}=" O4' YYY"; $resn_exch{" C3* YYY"}=" C3' YYY"; $resn_exch{" O3* YYY"}=" O3' YYY";
$resn_exch{" C2* YYY"}=" C2' YYY"; $resn_exch{" C1* YYY"}=" C1' YYY"; $resn_exch{" O1P   Z"}=" OP1   Z";
$resn_exch{" O2P   Z"}=" OP2   Z"; $resn_exch{" O5*   Z"}=" O5'   Z"; $resn_exch{" C5*   Z"}=" C5'   Z";
$resn_exch{" C4*   Z"}=" C4'   Z"; $resn_exch{" O4*   Z"}=" O4'   Z"; $resn_exch{" C3*   Z"}=" C3'   Z";
$resn_exch{" O3*   Z"}=" O3'   Z"; $resn_exch{" C2*   Z"}=" C2'   Z"; $resn_exch{" C1*   Z"}=" C1'   Z";
$resn_exch{" O3P   Z"}=" OP3   Z"; $resn_exch{" O5* Z5A"}=" O5B Z5A"; $resn_exch{" C5* Z5A"}=" C5B Z5A";
$resn_exch{" C4* Z5A"}=" C4B Z5A"; $resn_exch{" O4* Z5A"}=" O4B Z5A"; $resn_exch{" C3* Z5A"}=" C3B Z5A";
$resn_exch{" O3* Z5A"}=" O3' Z5A"; $resn_exch{" C2* Z5A"}=" C2B Z5A"; $resn_exch{" O2* Z5A"}=" O2' Z5A";
$resn_exch{" C1* Z5A"}=" C1B Z5A"; $resn_exch{"*CP1 ZAF"}=" C1X ZAF"; $resn_exch{"*CP2 ZAF"}=" C2X ZAF";
$resn_exch{"*CP3 ZAF"}=" C3X ZAF"; $resn_exch{"*CP4 ZAF"}=" C4X ZAF"; $resn_exch{"*CP5 ZAF"}=" C5X ZAF";
$resn_exch{"*CP6 ZAF"}=" C6X ZAF"; $resn_exch{" O1P ZDU"}=" OP1 ZDU"; $resn_exch{" O2P ZDU"}=" OP2 ZDU";
$resn_exch{" O5* ZDU"}=" O5' ZDU"; $resn_exch{" C2* ZDU"}=" C2' ZDU"; $resn_exch{" C5* ZDU"}=" C5' ZDU";
$resn_exch{" C4* ZDU"}=" C4' ZDU"; $resn_exch{" O4* ZDU"}=" O4' ZDU"; $resn_exch{" C1* ZDU"}=" C1' ZDU";
$resn_exch{" C3* ZDU"}=" C3' ZDU"; $resn_exch{" O3* ZDU"}=" O3' ZDU"; $resn_exch{" O3P ZDU"}=" OP3 ZDU";
$resn_exch{"CZ10 ZED"}=" CZA ZED"; $resn_exch{"CZ11 ZED"}=" CZB ZED"; $resn_exch{"CZ12 ZED"}=" CZC ZED";
$resn_exch{"CZ13 ZED"}=" CZD ZED"; $resn_exch{"CZ14 ZED"}=" CZE ZED"; $resn_exch{"CZ15 ZED"}=" CZF ZED";
$resn_exch{" N_A ZEM"}=" NA  ZEM"; $resn_exch{" N_B ZEM"}=" NB  ZEM"; $resn_exch{" N_C ZEM"}=" NC  ZEM";
$resn_exch{" N_D ZEM"}=" ND  ZEM"; $resn_exch{" AP  ZID"}=" PA  ZID"; $resn_exch{" AO1 ZID"}=" O1A ZID";
$resn_exch{" AO2 ZID"}=" O2A ZID"; $resn_exch{"AO5* ZID"}=" O5B ZID"; $resn_exch{"AC5* ZID"}=" C5B ZID";
$resn_exch{"AC4* ZID"}=" C4B ZID"; $resn_exch{"AO4* ZID"}=" O4B ZID"; $resn_exch{"AC3* ZID"}=" C3B ZID";
$resn_exch{"AO3* ZID"}=" O3B ZID"; $resn_exch{"AC2* ZID"}=" C2B ZID"; $resn_exch{"AO2* ZID"}=" O2B ZID";
$resn_exch{"AC1* ZID"}=" C1B ZID"; $resn_exch{" AN9 ZID"}=" N9A ZID"; $resn_exch{" AC8 ZID"}=" C8A ZID";
$resn_exch{" AN7 ZID"}=" N7A ZID"; $resn_exch{" AC5 ZID"}=" C5A ZID"; $resn_exch{" AC6 ZID"}=" C6A ZID";
$resn_exch{" AN6 ZID"}=" N6A ZID"; $resn_exch{" AN1 ZID"}=" N1A ZID"; $resn_exch{" AC2 ZID"}=" C2A ZID";
$resn_exch{" AN3 ZID"}=" N3A ZID"; $resn_exch{" AC4 ZID"}=" C4A ZID"; $resn_exch{" NP  ZID"}=" PN  ZID";
$resn_exch{" NO1 ZID"}=" O1N ZID"; $resn_exch{" NO2 ZID"}=" O2N ZID"; $resn_exch{"NO5* ZID"}=" O5D ZID";
$resn_exch{"NC5* ZID"}=" C5D ZID"; $resn_exch{"NC4* ZID"}=" C4D ZID"; $resn_exch{"NO4* ZID"}=" O4D ZID";
$resn_exch{"NC3* ZID"}=" C3D ZID"; $resn_exch{"NO3* ZID"}=" O3D ZID"; $resn_exch{"NC2* ZID"}=" C2D ZID";
$resn_exch{"NO2* ZID"}=" O2D ZID"; $resn_exch{"NC1* ZID"}=" C1D ZID"; $resn_exch{" NN1 ZID"}=" N1N ZID";
$resn_exch{" NC2 ZID"}=" C2N ZID"; $resn_exch{" NC3 ZID"}=" C3N ZID"; $resn_exch{" NC7 ZID"}=" C7N ZID";
$resn_exch{" NO7 ZID"}=" O7N ZID"; $resn_exch{" NN7 ZID"}=" N7N ZID"; $resn_exch{" NC4 ZID"}=" C4N ZID";
$resn_exch{" NC5 ZID"}=" C5N ZID"; $resn_exch{" NC6 ZID"}=" C6N ZID"; $resn_exch{" ZC7 ZID"}=" C7Z ZID";
$resn_exch{" ZO7 ZID"}=" O7Z ZID"; $resn_exch{" ZN1 ZID"}=" N1Z ZID"; $resn_exch{" ZC2 ZID"}=" C2Z ZID";
$resn_exch{" ZC3 ZID"}=" C3Z ZID"; $resn_exch{" ZC4 ZID"}=" C4Z ZID"; $resn_exch{" ZC5 ZID"}=" C5Z ZID";
$resn_exch{" ZC6 ZID"}=" C6Z ZID"; $resn_exch{" N_A ZNH"}=" NA  ZNH"; $resn_exch{" N_B ZNH"}=" NB  ZNH";
$resn_exch{" N_C ZNH"}=" NC  ZNH"; $resn_exch{" N_D ZNH"}=" ND  ZNH"; $resn_exch{"1H02 001"}="H021 001";
$resn_exch{"1H06 001"}="H061 001"; $resn_exch{"1H07 001"}="H071 001"; $resn_exch{"2H07 001"}="H072 001";
$resn_exch{"3H07 001"}="H073 001"; $resn_exch{"1H08 001"}="H081 001"; $resn_exch{"2H08 001"}="H082 001";
$resn_exch{"3H08 001"}="H083 001"; $resn_exch{"1H09 001"}="H091 001"; $resn_exch{"2H09 001"}="H092 001";
$resn_exch{"3H09 001"}="H093 001"; $resn_exch{"1H12 001"}="H121 001"; $resn_exch{"2H12 001"}="H122 001";
$resn_exch{"1H13 001"}="H131 001"; $resn_exch{"2H13 001"}="H132 001"; $resn_exch{"1H14 001"}="H141 001";
$resn_exch{"2H14 001"}="H142 001"; $resn_exch{"1H15 001"}="H151 001"; $resn_exch{"2H15 001"}="H152 001";
$resn_exch{"1H16 001"}="H161 001"; $resn_exch{"1H18 001"}="H181 001"; $resn_exch{"1H19 001"}="H191 001";
$resn_exch{"2H19 001"}="H192 001"; $resn_exch{"1H20 001"}="H201 001"; $resn_exch{"2H20 001"}="H202 001";
$resn_exch{"1H21 001"}="H211 001"; $resn_exch{"2H21 001"}="H212 001"; $resn_exch{"1H23 001"}="H231 001";
$resn_exch{"1H24 001"}="H241 001"; $resn_exch{"1H25 001"}="H251 001"; $resn_exch{"1H26 001"}="H261 001";
$resn_exch{"1H27 001"}="H271 001"; $resn_exch{"2H27 001"}="H272 001"; $resn_exch{"1H28 001"}="H281 001";
$resn_exch{"2H28 001"}="H282 001"; $resn_exch{"1H29 001"}="H291 001"; $resn_exch{"2H29 001"}="H292 001";
$resn_exch{"1H31 001"}="H311 001"; $resn_exch{"1H32 001"}="H321 001"; $resn_exch{"1H33 001"}="H331 001";
$resn_exch{"1H34 001"}="H341 001"; $resn_exch{"1H35 001"}="H351 001"; $resn_exch{"1H3  002"}=" H31 002";
$resn_exch{"2H3  002"}=" H32 002"; $resn_exch{"1H4  002"}=" H41 002"; $resn_exch{"2H4  002"}=" H42 002";
$resn_exch{"3H4  002"}=" H43 002"; $resn_exch{"1H8  002"}=" H81 002"; $resn_exch{"2H8  002"}=" H82 002";
$resn_exch{"1H10 002"}="H101 002"; $resn_exch{"2H10 002"}="H102 002"; $resn_exch{"1H12 002"}="H121 002";
$resn_exch{"2H12 002"}="H122 002"; $resn_exch{"3H12 002"}="H123 002"; $resn_exch{"1H20 002"}="H201 002";
$resn_exch{"2H20 002"}="H202 002"; $resn_exch{"3H20 002"}="H203 002"; $resn_exch{"1H21 002"}="H211 002";
$resn_exch{"2H21 002"}="H212 002"; $resn_exch{"3H21 002"}="H213 002"; $resn_exch{"1H23 002"}="H231 002";
$resn_exch{"2H23 002"}="H232 002"; $resn_exch{"1H7C 003"}="H7C1 003"; $resn_exch{"2H7C 003"}="H7C2 003";
$resn_exch{"1H10 003"}="H101 003"; $resn_exch{"2H10 003"}="H102 003"; $resn_exch{"3H10 003"}="H103 003";
$resn_exch{"1H9C 003"}="H9C1 003"; $resn_exch{"2H9C 003"}="H9C2 003"; $resn_exch{"3H9C 003"}="H9C3 003";
$resn_exch{"1H23 003"}="H231 003"; $resn_exch{"2H23 003"}="H232 003"; $resn_exch{"1H12 003"}="H121 003";
$resn_exch{"2H12 003"}="H122 003"; $resn_exch{"3H12 003"}="H123 003"; $resn_exch{"1H1C 007"}="H1C1 007";
$resn_exch{"2H1C 007"}="H1C2 007"; $resn_exch{"1H2C 007"}="H2C1 007"; $resn_exch{"2H2C 007"}="H2C2 007";
$resn_exch{"1H4C 007"}="H4C1 007"; $resn_exch{"2H4C 007"}="H4C2 007"; $resn_exch{"1H5C 007"}="H5C1 007";
$resn_exch{"2H5C 007"}="H5C2 007"; $resn_exch{"1H12 007"}="H121 007"; $resn_exch{"2H12 007"}="H122 007";
$resn_exch{"1H13 007"}="H131 007"; $resn_exch{"2H13 007"}="H132 007"; $resn_exch{"1H8C 008"}="H8C1 008";
$resn_exch{"2H8C 008"}="H8C2 008"; $resn_exch{"1H10 008"}="H101 008"; $resn_exch{"2H10 008"}="H102 008";
$resn_exch{"1H11 008"}="H111 008"; $resn_exch{"2H11 008"}="H112 008"; $resn_exch{"1H19 008"}="H191 008";
$resn_exch{"2H19 008"}="H192 008"; $resn_exch{"1H16 008"}="H161 008"; $resn_exch{"2H16 008"}="H162 008";
$resn_exch{"1H26 008"}="H261 008"; $resn_exch{"2H26 008"}="H262 008"; $resn_exch{"1H1  017"}=" H11 017";
$resn_exch{"2H1  017"}=" H12 017"; $resn_exch{"1H12 017"}="H121 017"; $resn_exch{"2H12 017"}="H122 017";
$resn_exch{"1H14 017"}="H141 017"; $resn_exch{"2H14 017"}="H142 017"; $resn_exch{"3H14 017"}="H143 017";
$resn_exch{"1H15 017"}="H151 017"; $resn_exch{"2H15 017"}="H152 017"; $resn_exch{"3H15 017"}="H153 017";
$resn_exch{"1H16 017"}="H161 017"; $resn_exch{"2H16 017"}="H162 017"; $resn_exch{"1H25 017"}="H251 017";
$resn_exch{"2H25 017"}="H252 017"; $resn_exch{"1H29 017"}="H291 017"; $resn_exch{"2H29 017"}="H292 017";
$resn_exch{"1H30 017"}="H301 017"; $resn_exch{"2H30 017"}="H302 017"; $resn_exch{"1H32 017"}="H321 017";
$resn_exch{"2H32 017"}="H322 017"; $resn_exch{"1H21 024"}="H211 024"; $resn_exch{"2H21 024"}="H212 024";
$resn_exch{"1H30 024"}="H301 024"; $resn_exch{"2H30 024"}="H302 024"; $resn_exch{"1H31 024"}="H311 024";
$resn_exch{"2H31 024"}="H312 024"; $resn_exch{"1H32 024"}="H321 024"; $resn_exch{"2H32 024"}="H322 024";
$resn_exch{"1H33 024"}="H331 024"; $resn_exch{"2H33 024"}="H332 024"; $resn_exch{"1H34 024"}="H341 024";
$resn_exch{"2H34 024"}="H342 024"; $resn_exch{"1H8  033"}=" H81 033"; $resn_exch{"2H8  033"}=" H82 033";
$resn_exch{"3H8  033"}=" H83 033"; $resn_exch{"1H9  033"}=" H91 033"; $resn_exch{"2H9  033"}=" H92 033";
$resn_exch{"3H9  033"}=" H93 033"; $resn_exch{"1HAU 039"}="HAU1 039"; $resn_exch{"2HAU 039"}="HAU2 039";
$resn_exch{"3HAU 039"}="HAU3 039"; $resn_exch{"1HAS 039"}="HAS1 039"; $resn_exch{"2HAS 039"}="HAS2 039";
$resn_exch{"1H22 03R"}="H221 03R"; $resn_exch{"2H22 03R"}="H222 03R"; $resn_exch{"1H23 03R"}="H231 03R";
$resn_exch{"2H23 03R"}="H232 03R"; $resn_exch{"3H23 03R"}="H233 03R"; $resn_exch{"1H29 03R"}="H291 03R";
$resn_exch{"2H29 03R"}="H292 03R"; $resn_exch{"1H34 03R"}="H341 03R"; $resn_exch{"2H34 03R"}="H342 03R";
$resn_exch{"1H25 041"}="H251 041"; $resn_exch{"2H25 041"}="H252 041"; $resn_exch{"1H10 042"}="H101 042";
$resn_exch{"2H10 042"}="H102 042"; $resn_exch{"1H11 042"}="H111 042"; $resn_exch{"2H11 042"}="H112 042";
$resn_exch{"3H11 042"}="H113 042"; $resn_exch{"1H13 042"}="H131 042"; $resn_exch{"2H13 042"}="H132 042";
$resn_exch{"3H13 042"}="H133 042"; $resn_exch{"1H16 042"}="H161 042"; $resn_exch{"2H16 042"}="H162 042";
$resn_exch{"1H15 042"}="H151 042"; $resn_exch{"2H15 042"}="H152 042"; $resn_exch{"3H15 042"}="H153 042";
$resn_exch{"1H17 042"}="H171 042"; $resn_exch{"2H17 042"}="H172 042"; $resn_exch{"3H17 042"}="H173 042";
$resn_exch{"1H7  047"}=" H71 047"; $resn_exch{"2H7  047"}=" H72 047"; $resn_exch{"1H12 047"}="H121 047";
$resn_exch{"2H12 047"}="H122 047"; $resn_exch{"1H17 047"}="H171 047"; $resn_exch{"2H17 047"}="H172 047";
$resn_exch{"1H20 047"}="H201 047"; $resn_exch{"2H20 047"}="H202 047"; $resn_exch{"1H23 047"}="H231 047";
$resn_exch{"2H23 047"}="H232 047"; $resn_exch{"1H26 047"}="H261 047"; $resn_exch{"2H26 047"}="H262 047";
$resn_exch{"1H29 047"}="H291 047"; $resn_exch{"2H29 047"}="H292 047"; $resn_exch{"1H50 047"}="H501 047";
$resn_exch{"2H50 047"}="H502 047"; $resn_exch{"3H50 047"}="H503 047"; $resn_exch{"1H34 059"}="H341 059";
$resn_exch{"2H34 059"}="H342 059"; $resn_exch{"1H33 059"}="H331 059"; $resn_exch{"2H33 059"}="H332 059";
$resn_exch{"1H30 059"}="H301 059"; $resn_exch{"2H30 059"}="H302 059"; $resn_exch{"3H30 059"}="H303 059";
$resn_exch{"1H29 059"}="H291 059"; $resn_exch{"2H29 059"}="H292 059"; $resn_exch{"3H29 059"}="H293 059";
$resn_exch{"1H15 059"}="H151 059"; $resn_exch{"2H15 059"}="H152 059"; $resn_exch{"1H9  059"}=" H91 059";
$resn_exch{"2H9  059"}=" H92 059"; $resn_exch{"1H10 059"}="H101 059"; $resn_exch{"2H10 059"}="H102 059";
$resn_exch{"1H18 061"}="H181 061"; $resn_exch{"2H18 061"}="H182 061"; $resn_exch{"1H21 061"}="H211 061";
$resn_exch{"2H21 061"}="H212 061"; $resn_exch{"1H22 061"}="H221 061"; $resn_exch{"2H22 061"}="H222 061";
$resn_exch{"1H23 061"}="H231 061"; $resn_exch{"2H23 061"}="H232 061"; $resn_exch{"1H24 061"}="H241 061";
$resn_exch{"2H24 061"}="H242 061"; $resn_exch{"3H24 061"}="H243 061"; $resn_exch{"1H25 062"}="H251 062";
$resn_exch{"2H25 062"}="H252 062"; $resn_exch{"1H26 062"}="H261 062"; $resn_exch{"2H26 062"}="H262 062";
$resn_exch{"1H27 062"}="H271 062"; $resn_exch{"2H27 062"}="H272 062"; $resn_exch{"1H37 062"}="H371 062";
$resn_exch{"2H37 062"}="H372 062"; $resn_exch{"1H53 062"}="H531 062"; $resn_exch{"2H53 062"}="H532 062";
$resn_exch{"1H54 062"}="H541 062"; $resn_exch{"2H54 062"}="H542 062"; $resn_exch{"1H14 065"}="H141 065";
$resn_exch{"2H14 065"}="H142 065"; $resn_exch{"3H14 065"}="H143 065"; $resn_exch{"1H15 065"}="H151 065";
$resn_exch{"2H15 065"}="H152 065"; $resn_exch{"3H15 065"}="H153 065"; $resn_exch{"1H12 065"}="H121 065";
$resn_exch{"2H12 065"}="H122 065"; $resn_exch{"1H16 065"}="H161 065"; $resn_exch{"2H16 065"}="H162 065";
$resn_exch{"1H32 065"}="H321 065"; $resn_exch{"2H32 065"}="H322 065"; $resn_exch{"1H25 065"}="H251 065";
$resn_exch{"2H25 065"}="H252 065"; $resn_exch{"1H29 065"}="H291 065"; $resn_exch{"2H29 065"}="H292 065";
$resn_exch{"1H30 065"}="H301 065"; $resn_exch{"2H30 065"}="H302 065"; $resn_exch{"1H40 065"}="H401 065";
$resn_exch{"2H40 065"}="H402 065"; $resn_exch{"1H1C 072"}="H1C1 072"; $resn_exch{"2H1C 072"}="H1C2 072";
$resn_exch{"1H2B 072"}="H2B1 072"; $resn_exch{"2H2B 072"}="H2B2 072"; $resn_exch{"1H2C 072"}="H2C1 072";
$resn_exch{"2H2C 072"}="H2C2 072"; $resn_exch{"1H2D 072"}="H2D1 072"; $resn_exch{"2H2D 072"}="H2D2 072";
$resn_exch{"1H2E 072"}="H2E1 072"; $resn_exch{"2H2E 072"}="H2E2 072"; $resn_exch{"1H2F 072"}="H2F1 072";
$resn_exch{"2H2F 072"}="H2F2 072"; $resn_exch{"1H2G 072"}="H2G1 072"; $resn_exch{"2H2G 072"}="H2G2 072";
$resn_exch{"1H2H 072"}="H2H1 072"; $resn_exch{"2H2H 072"}="H2H2 072"; $resn_exch{"3H2H 072"}="H2H3 072";
$resn_exch{"1H3B 072"}="H3B1 072"; $resn_exch{"2H3B 072"}="H3B2 072"; $resn_exch{"1H3C 072"}="H3C1 072";
$resn_exch{"2H3C 072"}="H3C2 072"; $resn_exch{"1H3D 072"}="H3D1 072"; $resn_exch{"2H3D 072"}="H3D2 072";
$resn_exch{"1H3E 072"}="H3E1 072"; $resn_exch{"2H3E 072"}="H3E2 072"; $resn_exch{"3HOM 072"}="HOM3 072";
$resn_exch{"1H4B 072"}="H4B1 072"; $resn_exch{"2H4B 072"}="H4B2 072"; $resn_exch{"1H5B 072"}="H5B1 072";
$resn_exch{"2H5B 072"}="H5B2 072"; $resn_exch{"1H67 073"}="H671 073"; $resn_exch{"2H67 073"}="H672 073";
$resn_exch{"3H67 073"}="H673 073"; $resn_exch{"1H13 073"}="H131 073"; $resn_exch{"2H13 073"}="H132 073";
$resn_exch{"1H37 073"}="H371 073"; $resn_exch{"2H37 073"}="H372 073"; $resn_exch{"1H1  074"}=" H11 074";
$resn_exch{"2H1  074"}=" H12 074"; $resn_exch{"3H1  074"}=" H13 074"; $resn_exch{"1H2  074"}=" H21 074";
$resn_exch{"2H2  074"}=" H22 074"; $resn_exch{"1H3  074"}=" H31 074"; $resn_exch{"2H3  074"}=" H32 074";
$resn_exch{"1HN1 074"}="HN11 074"; $resn_exch{"1H5  074"}=" H51 074"; $resn_exch{"1H6  074"}=" H61 074";
$resn_exch{"2H6  074"}=" H62 074"; $resn_exch{"1HO2 074"}="HO21 074"; $resn_exch{"1HN2 074"}="HN21 074";
$resn_exch{"1H8  074"}=" H81 074"; $resn_exch{"1H9  074"}=" H91 074"; $resn_exch{"1H10 074"}="H101 074";
$resn_exch{"2H10 074"}="H102 074"; $resn_exch{"3H10 074"}="H103 074"; $resn_exch{"1H11 074"}="H111 074";
$resn_exch{"2H11 074"}="H112 074"; $resn_exch{"1H12 074"}="H121 074"; $resn_exch{"2H12 074"}="H122 074";
$resn_exch{"3H12 074"}="H123 074"; $resn_exch{"1H14 074"}="H141 074"; $resn_exch{"2H14 074"}="H142 074";
$resn_exch{"1H15 074"}="H151 074"; $resn_exch{"2H15 074"}="H152 074"; $resn_exch{"1H16 074"}="H161 074";
$resn_exch{"2H16 074"}="H162 074"; $resn_exch{"1H17 074"}="H171 074"; $resn_exch{"1H39 075"}="H391 075";
$resn_exch{"2H39 075"}="H392 075"; $resn_exch{"1H29 075"}="H291 075"; $resn_exch{"2H29 075"}="H292 075";
$resn_exch{"3H29 075"}="H293 075"; $resn_exch{"1H30 075"}="H301 075"; $resn_exch{"2H30 075"}="H302 075";
$resn_exch{"3H30 075"}="H303 075"; $resn_exch{"1H19 075"}="H191 075"; $resn_exch{"2H19 075"}="H192 075";
$resn_exch{"1H15 075"}="H151 075"; $resn_exch{"2H15 075"}="H152 075"; $resn_exch{"1H11 075"}="H111 075";
$resn_exch{"2H11 075"}="H112 075"; $resn_exch{"1H28 075"}="H281 075"; $resn_exch{"2H28 075"}="H282 075";
$resn_exch{"3H28 075"}="H283 075"; $resn_exch{"1H9  075"}=" H91 075"; $resn_exch{"2H9  075"}=" H92 075";
$resn_exch{"1H50 084"}="H501 084"; $resn_exch{"2H50 084"}="H502 084"; $resn_exch{"3H50 084"}="H503 084";
$resn_exch{"1H13 084"}="H131 084"; $resn_exch{"2H13 084"}="H132 084"; $resn_exch{"1H14 084"}="H141 084";
$resn_exch{"2H14 084"}="H142 084"; $resn_exch{"1H16 084"}="H161 084"; $resn_exch{"2H16 084"}="H162 084";
$resn_exch{"1H17 084"}="H171 084"; $resn_exch{"2H17 084"}="H172 084"; $resn_exch{"1H6  084"}=" H61 084";
$resn_exch{"2H6  084"}=" H62 084"; $resn_exch{"3H6  084"}=" H63 084"; $resn_exch{"1HAZ 090"}="HAZ1 090";
$resn_exch{"2HAZ 090"}="HAZ2 090"; $resn_exch{"3HAZ 090"}="HAZ3 090"; $resn_exch{"1HAY 090"}="HAY1 090";
$resn_exch{"2HAY 090"}="HAY2 090"; $resn_exch{"3HAY 090"}="HAY3 090"; $resn_exch{"1HAV 093"}="HAV1 093";
$resn_exch{"2HAV 093"}="HAV2 093"; $resn_exch{"1HAW 093"}="HAW1 093"; $resn_exch{"2HAW 093"}="HAW2 093";
$resn_exch{"1HAE 093"}="HAE1 093"; $resn_exch{"2HAE 093"}="HAE2 093"; $resn_exch{"3HAE 093"}="HAE3 093";
$resn_exch{"1HAT 093"}="HAT1 093"; $resn_exch{"2HAT 093"}="HAT2 093"; $resn_exch{"3HAT 093"}="HAT3 093";
$resn_exch{"1H10 094"}="H101 094"; $resn_exch{"2H10 094"}="H102 094"; $resn_exch{"1H20 094"}="H201 094";
$resn_exch{"2H20 094"}="H202 094"; $resn_exch{"1H21 094"}="H211 094"; $resn_exch{"2H21 094"}="H212 094";
$resn_exch{"1H22 094"}="H221 094"; $resn_exch{"2H22 094"}="H222 094"; $resn_exch{"1H2  097"}=" H21 097";
$resn_exch{"2H2  097"}=" H22 097"; $resn_exch{"3H2  097"}=" H23 097"; $resn_exch{"1H8  097"}=" H81 097";
$resn_exch{"2H8  097"}=" H82 097"; $resn_exch{"3H8  097"}=" H83 097"; $resn_exch{"1H9  097"}=" H91 097";
$resn_exch{"2H9  097"}=" H92 097"; $resn_exch{"3H9  097"}=" H93 097"; $resn_exch{"1H11 097"}="H111 097";
$resn_exch{"2H11 097"}="H112 097"; $resn_exch{"3H11 097"}="H113 097"; $resn_exch{"1H15 097"}="H151 097";
$resn_exch{"2H15 097"}="H152 097"; $resn_exch{"1H3  097"}=" H31 097"; $resn_exch{"2H3  097"}=" H32 097";
$resn_exch{"3H3  097"}=" H33 097"; $resn_exch{"1H7  097"}=" H71 097"; $resn_exch{"2H7  097"}=" H72 097";
$resn_exch{"3H7  097"}=" H73 097"; $resn_exch{"1HN  0A0"}=" HN1 0A0"; $resn_exch{"2HN  0A0"}=" HN2 0A0";
$resn_exch{"1HBC 0A0"}="HBC1 0A0"; $resn_exch{"2HBC 0A0"}="HBC2 0A0"; $resn_exch{"1HMC 0A0"}="HMC1 0A0";
$resn_exch{"2HMC 0A0"}="HMC2 0A0"; $resn_exch{"3HMC 0A0"}="HMC3 0A0"; $resn_exch{"1HN  0A1"}=" HN1 0A1";
$resn_exch{"2HN  0A1"}=" HN2 0A1"; $resn_exch{"1HBC 0A1"}="HBC1 0A1"; $resn_exch{"2HBC 0A1"}="HBC2 0A1";
$resn_exch{"1HMC 0A1"}="HMC1 0A1"; $resn_exch{"2HMC 0A1"}="HMC2 0A1"; $resn_exch{"3HMC 0A1"}="HMC3 0A1";
$resn_exch{"1H01 0A2"}="H011 0A2"; $resn_exch{"2H01 0A2"}="H012 0A2"; $resn_exch{"1H02 0A2"}="H021 0A2";
$resn_exch{"2H02 0A2"}="H022 0A2"; $resn_exch{"3H02 0A2"}="H023 0A2"; $resn_exch{"1H03 0A2"}="H031 0A2";
$resn_exch{"2H03 0A2"}="H032 0A2"; $resn_exch{"1H05 0A2"}="H051 0A2"; $resn_exch{"2H05 0A2"}="H052 0A2";
$resn_exch{"1H06 0A2"}="H061 0A2"; $resn_exch{"2H06 0A2"}="H062 0A2"; $resn_exch{"1H07 0A2"}="H071 0A2";
$resn_exch{"2H07 0A2"}="H072 0A2"; $resn_exch{"1H08 0A2"}="H081 0A2"; $resn_exch{"2H08 0A2"}="H082 0A2";
$resn_exch{"1H09 0A2"}="H091 0A2"; $resn_exch{"2H09 0A2"}="H092 0A2"; $resn_exch{"1H18 0A2"}="H181 0A2";
$resn_exch{"2H18 0A2"}="H182 0A2"; $resn_exch{"1H19 0A2"}="H191 0A2"; $resn_exch{"2H19 0A2"}="H192 0A2";
$resn_exch{"3H19 0A2"}="H193 0A2"; $resn_exch{"1H2C 0A3"}="H2C1 0A3"; $resn_exch{"2H2C 0A3"}="H2C2 0A3";
$resn_exch{"1H3' 0A3"}="H3'1 0A3"; $resn_exch{"2H3' 0A3"}="H3'2 0A3"; $resn_exch{"3H3' 0A3"}="H3'3 0A3";
$resn_exch{"1H18 0A3"}="H181 0A3"; $resn_exch{"2H18 0A3"}="H182 0A3"; $resn_exch{"3H18 0A3"}="H183 0A3";
$resn_exch{"1HA  0A3"}=" HA1 0A3"; $resn_exch{"2HB  0A3"}=" HB2 0A3"; $resn_exch{"1H2C 0A4"}="H2C1 0A4";
$resn_exch{"2H2C 0A4"}="H2C2 0A4"; $resn_exch{"1H3' 0A4"}="H3'1 0A4"; $resn_exch{"2H3' 0A4"}="H3'2 0A4";
$resn_exch{"1H13 0A4"}="H131 0A4"; $resn_exch{"2H13 0A4"}="H132 0A4"; $resn_exch{"1H15 0A4"}="H151 0A4";
$resn_exch{"1H2C 0A5"}="H2C1 0A5"; $resn_exch{"2H2C 0A5"}="H2C2 0A5"; $resn_exch{"1H3C 0A5"}="H3C1 0A5";
$resn_exch{"2H3C 0A5"}="H3C2 0A5"; $resn_exch{"3H3C 0A5"}="H3C3 0A5"; $resn_exch{"1HBC 0A5"}="HBC1 0A5";
$resn_exch{"2HBC 0A5"}="HBC2 0A5"; $resn_exch{"1HD2 0A5"}="HD21 0A5"; $resn_exch{"2HD2 0A5"}="HD22 0A5";
$resn_exch{"1H05 0A6"}="H051 0A6"; $resn_exch{"2H05 0A6"}="H052 0A6"; $resn_exch{"1H08 0A6"}="H081 0A6";
$resn_exch{"2H08 0A6"}="H082 0A6"; $resn_exch{"1H09 0A6"}="H091 0A6"; $resn_exch{"2H09 0A6"}="H092 0A6";
$resn_exch{"3H09 0A6"}="H093 0A6"; $resn_exch{"1H10 0A7"}="H101 0A7"; $resn_exch{"2H10 0A7"}="H102 0A7";
$resn_exch{"1HN  0A8"}=" HN1 0A8"; $resn_exch{"2HN  0A8"}=" HN2 0A8"; $resn_exch{"1HBC 0A8"}="HBC1 0A8";
$resn_exch{"2HBC 0A8"}="HBC2 0A8"; $resn_exch{"1H2C 0A8"}="H2C1 0A8"; $resn_exch{"2H2C 0A8"}="H2C2 0A8";
$resn_exch{"1H3C 0A8"}="H3C1 0A8"; $resn_exch{"2H3C 0A8"}="H3C2 0A8"; $resn_exch{"1HN  0A9"}=" HN1 0A9";
$resn_exch{"2HN  0A9"}=" HN2 0A9"; $resn_exch{"1HBC 0A9"}="HBC1 0A9"; $resn_exch{"2HBC 0A9"}="HBC2 0A9";
$resn_exch{"1HMC 0A9"}="HMC1 0A9"; $resn_exch{"2HMC 0A9"}="HMC2 0A9"; $resn_exch{"3HMC 0A9"}="HMC3 0A9";
$resn_exch{"1HN  0AA"}=" HN1 0AA"; $resn_exch{"2HA  0AA"}=" HA2 0AA"; $resn_exch{"1HG1 0AA"}="HG11 0AA";
$resn_exch{"2HG1 0AA"}="HG12 0AA"; $resn_exch{"3HG1 0AA"}="HG13 0AA"; $resn_exch{"1HG2 0AA"}="HG21 0AA";
$resn_exch{"2HG2 0AA"}="HG22 0AA"; $resn_exch{"3HG2 0AA"}="HG23 0AA"; $resn_exch{"1H1C 0AA"}="H1C1 0AA";
$resn_exch{"2H1C 0AA"}="H1C2 0AA"; $resn_exch{"3H1C 0AA"}="H1C3 0AA"; $resn_exch{"1HN  0AB"}=" HN1 0AB";
$resn_exch{"2HA  0AB"}=" HA2 0AB"; $resn_exch{"1HG1 0AB"}="HG11 0AB"; $resn_exch{"2HG1 0AB"}="HG12 0AB";
$resn_exch{"3HG1 0AB"}="HG13 0AB"; $resn_exch{"1HN  0AC"}=" HN1 0AC"; $resn_exch{"2HN  0AC"}=" HN2 0AC";
$resn_exch{"1HBC 0AC"}="HBC1 0AC"; $resn_exch{"2HBC 0AC"}="HBC2 0AC"; $resn_exch{"1H1C 0AC"}="H1C1 0AC";
$resn_exch{"2H1C 0AC"}="H1C2 0AC"; $resn_exch{"1HA1 0AC"}="HA11 0AC"; $resn_exch{"2HA1 0AC"}="HA12 0AC";
$resn_exch{"1H5' 0AD"}="H5'1 0AD"; $resn_exch{"2H5' 0AD"}="H5'2 0AD"; $resn_exch{"1H2' 0AD"}="H2'1 0AD";
$resn_exch{"2H2' 0AD"}="H2'2 0AD"; $resn_exch{"1H4A 0AD"}="H4A1 0AD"; $resn_exch{"2H4A 0AD"}="H4A2 0AD";
$resn_exch{"1H5A 0AD"}="H5A1 0AD"; $resn_exch{"2H5A 0AD"}="H5A2 0AD"; $resn_exch{"1HN  0AF"}=" HN1 0AF";
$resn_exch{"2HN  0AF"}=" HN2 0AF"; $resn_exch{"1HBC 0AF"}="HBC1 0AF"; $resn_exch{"2HBC 0AF"}="HBC2 0AF";
$resn_exch{"1H3  0AG"}=" H31 0AG"; $resn_exch{"2H3  0AG"}=" H32 0AG"; $resn_exch{"3H3  0AG"}=" H33 0AG";
$resn_exch{"1H4  0AG"}=" H41 0AG"; $resn_exch{"2H4  0AG"}=" H42 0AG"; $resn_exch{"1HBC 0AG"}="HBC1 0AG";
$resn_exch{"2HBC 0AG"}="HBC2 0AG"; $resn_exch{"1HD1 0AG"}="HD11 0AG"; $resn_exch{"2HD1 0AG"}="HD12 0AG";
$resn_exch{"3HD1 0AG"}="HD13 0AG"; $resn_exch{"1HD2 0AG"}="HD21 0AG"; $resn_exch{"2HD2 0AG"}="HD22 0AG";
$resn_exch{"3HD2 0AG"}="HD23 0AG"; $resn_exch{"1HN  0AH"}=" HN1 0AH"; $resn_exch{"2HN  0AH"}=" HN2 0AH";
$resn_exch{"1HBC 0AH"}="HBC1 0AH"; $resn_exch{"2HBC 0AH"}="HBC2 0AH"; $resn_exch{"1H2C 0AH"}="H2C1 0AH";
$resn_exch{"2H2C 0AH"}="H2C2 0AH"; $resn_exch{"1H8C 0AI"}="H8C1 0AI"; $resn_exch{"2H8C 0AI"}="H8C2 0AI";
$resn_exch{"3H8C 0AI"}="H8C3 0AI"; $resn_exch{"1HMC 0AI"}="HMC1 0AI"; $resn_exch{"2HMC 0AI"}="HMC2 0AI";
$resn_exch{"3HMC 0AI"}="HMC3 0AI"; $resn_exch{"1HAC 0AJ"}="HAC1 0AJ"; $resn_exch{"2HAC 0AJ"}="HAC2 0AJ";
$resn_exch{"3HAC 0AJ"}="HAC3 0AJ"; $resn_exch{"1HDC 0AJ"}="HDC1 0AJ"; $resn_exch{"2HDC 0AJ"}="HDC2 0AJ";
$resn_exch{"3HDC 0AJ"}="HDC3 0AJ"; $resn_exch{"1H1C 0AJ"}="H1C1 0AJ"; $resn_exch{"2H1C 0AJ"}="H1C2 0AJ";
$resn_exch{"1H2C 0AJ"}="H2C1 0AJ"; $resn_exch{"2H2C 0AJ"}="H2C2 0AJ"; $resn_exch{"3H2C 0AJ"}="H2C3 0AJ";
$resn_exch{"1HN  0AK"}=" HN1 0AK"; $resn_exch{"2HN  0AK"}=" HN2 0AK"; $resn_exch{"1HBC 0AK"}="HBC1 0AK";
$resn_exch{"2HBC 0AK"}="HBC2 0AK"; $resn_exch{"1H1C 0AK"}="H1C1 0AK"; $resn_exch{"2H1C 0AK"}="H1C2 0AK";
$resn_exch{"1H2C 0AK"}="H2C1 0AK"; $resn_exch{"2H2C 0AK"}="H2C2 0AK"; $resn_exch{"1H7C 0AL"}="H7C1 0AL";
$resn_exch{"2H7C 0AL"}="H7C2 0AL"; $resn_exch{"2HN  0AO"}=" HN2 0AO"; $resn_exch{"1H5' 0AP"}="H5'1 0AP";
$resn_exch{"2H5' 0AP"}="H5'2 0AP"; $resn_exch{"1H2' 0AP"}="H2'1 0AP"; $resn_exch{"2H2' 0AP"}="H2'2 0AP";
$resn_exch{"1H4N 0AP"}="H4N1 0AP"; $resn_exch{"2H4N 0AP"}="H4N2 0AP"; $resn_exch{"1H6C 0AT"}="H6C1 0AT";
$resn_exch{"2H6C 0AT"}="H6C2 0AT"; $resn_exch{"1H8C 0AT"}="H8C1 0AT"; $resn_exch{"2H8C 0AT"}="H8C2 0AT";
$resn_exch{"3H8C 0AT"}="H8C3 0AT"; $resn_exch{"1H5' 0AU"}="H5'1 0AU"; $resn_exch{"2H5' 0AU"}="H5'2 0AU";
$resn_exch{"1H5' 0AV"}="H5'1 0AV"; $resn_exch{"2HA  0AV"}=" HA2 0AV"; $resn_exch{"1HMC 0AV"}="HMC1 0AV";
$resn_exch{"2HMC 0AV"}="HMC2 0AV"; $resn_exch{"3HMC 0AV"}="HMC3 0AV"; $resn_exch{"1H6N 0AV"}="H6N1 0AV";
$resn_exch{"2H6N 0AV"}="H6N2 0AV"; $resn_exch{" HB1 0AZ"}=" HB2 0AZ"; $resn_exch{" HB2 0AZ"}=" HB3 0AZ";
$resn_exch{"HD1  0AZ"}="HD22 0AZ"; $resn_exch{"HD2  0AZ"}="HD23 0AZ"; $resn_exch{" HOD 0AZ"}=" HD1 0AZ";
$resn_exch{"1HN  0CS"}=" H   0CS"; $resn_exch{"2HN  0CS"}=" H2  0CS"; $resn_exch{"1HB  0CS"}=" HB2 0CS";
$resn_exch{"2HB  0CS"}=" HB3 0CS"; $resn_exch{"HOD3 0CS"}=" HD3 0CS"; $resn_exch{"2HOP 101"}="HOP2 101";
$resn_exch{"3HOP 101"}="HOP3 101"; $resn_exch{"HO5* 101"}="HO5' 101"; $resn_exch{"1H5* 101"}="H5'1 101";
$resn_exch{"2H5* 101"}="H5'2 101"; $resn_exch{" H4* 101"}=" H4' 101"; $resn_exch{" H3* 101"}=" H3' 101";
$resn_exch{"1H2* 101"}="H2'1 101"; $resn_exch{"2H2* 101"}="H2'2 101"; $resn_exch{" H1* 101"}=" H1' 101";
$resn_exch{"1HN6 101"}="HN61 101"; $resn_exch{"2HN6 101"}="HN62 101"; $resn_exch{"2HOG 102"}="HOG2 102";
$resn_exch{"3HOG 102"}="HOG3 102"; $resn_exch{"2HOB 102"}="HOB2 102"; $resn_exch{"2HOA 102"}="HOA2 102";
$resn_exch{"1H5* 102"}="H5'1 102"; $resn_exch{"2H5* 102"}="H5'2 102"; $resn_exch{" H4* 102"}=" H4' 102";
$resn_exch{" H3* 102"}=" H3' 102"; $resn_exch{"*HO3 102"}="HO3' 102"; $resn_exch{" H2* 102"}=" H2' 102";
$resn_exch{"*HO2 102"}="HO2' 102"; $resn_exch{" H1* 102"}=" H1' 102"; $resn_exch{"1HN2 102"}="HN21 102";
$resn_exch{"2HN2 102"}="HN22 102"; $resn_exch{"2HOP 103"}="HOP2 103"; $resn_exch{"3HOP 103"}="HOP3 103";
$resn_exch{"1H5* 103"}="H5'1 103"; $resn_exch{"2H5* 103"}="H5'2 103"; $resn_exch{"3H5* 103"}="H5'3 103";
$resn_exch{" H4* 103"}=" H4' 103"; $resn_exch{" H3* 103"}=" H3' 103"; $resn_exch{"1H2* 103"}="H2'1 103";
$resn_exch{"2H2* 103"}="H2'2 103"; $resn_exch{" H1* 103"}=" H1' 103"; $resn_exch{"1HN6 103"}="HN61 103";
$resn_exch{"2HN6 103"}="HN62 103"; $resn_exch{"1H1  104"}=" H11 104"; $resn_exch{"2H1  104"}=" H12 104";
$resn_exch{"1H2  104"}=" H21 104"; $resn_exch{"2H2  104"}=" H22 104"; $resn_exch{"1H3  104"}=" H31 104";
$resn_exch{"2H3  104"}=" H32 104"; $resn_exch{"1H5  104"}=" H51 104"; $resn_exch{"2H5  104"}=" H52 104";
$resn_exch{"1H6  104"}=" H61 104"; $resn_exch{"2H6  104"}=" H62 104"; $resn_exch{"1H8  104"}=" H81 104";
$resn_exch{"2H8  104"}=" H82 104"; $resn_exch{"1H9  104"}=" H91 104"; $resn_exch{"2H9  104"}=" H92 104";
$resn_exch{"1H10 104"}="H101 104"; $resn_exch{"2H10 104"}="H102 104"; $resn_exch{"1H3  105"}=" H31 105";
$resn_exch{"2H3  105"}=" H32 105"; $resn_exch{"1H9  105"}=" H91 105"; $resn_exch{"2H9  105"}=" H92 105";
$resn_exch{"3H9  105"}=" H93 105"; $resn_exch{"1H30 106"}="H301 106"; $resn_exch{"2H30 106"}="H302 106";
$resn_exch{"1H14 107"}="H141 107"; $resn_exch{"2H14 107"}="H142 107"; $resn_exch{"1H7  108"}=" H71 108";
$resn_exch{"2H7  108"}=" H72 108"; $resn_exch{"1HN1 108"}="HN11 108"; $resn_exch{"2HN1 108"}="HN12 108";
$resn_exch{"1HB  108"}=" HB1 108"; $resn_exch{"2HB  108"}=" HB2 108"; $resn_exch{"3HB  108"}=" HB3 108";
$resn_exch{"1H7  109"}=" H71 109"; $resn_exch{"2H7  109"}=" H72 109"; $resn_exch{"1HN1 109"}="HN11 109";
$resn_exch{"2HN1 109"}="HN12 109"; $resn_exch{"1HB  109"}=" HB1 109"; $resn_exch{"2HB  109"}=" HB2 109";
$resn_exch{"3HB  109"}=" HB3 109"; $resn_exch{"1H1  10A"}=" H11 10A"; $resn_exch{"2H1  10A"}=" H12 10A";
$resn_exch{"3H1  10A"}=" H13 10A"; $resn_exch{"1H2  10A"}=" H21 10A"; $resn_exch{"2H2  10A"}=" H22 10A";
$resn_exch{"1H3  10A"}=" H31 10A"; $resn_exch{"2H3  10A"}=" H32 10A"; $resn_exch{"1H4  10A"}=" H41 10A";
$resn_exch{"2H4  10A"}=" H42 10A"; $resn_exch{"1H5  10A"}=" H51 10A"; $resn_exch{"2H5  10A"}=" H52 10A";
$resn_exch{"1H6  10A"}=" H61 10A"; $resn_exch{"2H6  10A"}=" H62 10A"; $resn_exch{"1H7  10A"}=" H71 10A";
$resn_exch{"2H7  10A"}=" H72 10A"; $resn_exch{"1H8  10A"}=" H81 10A"; $resn_exch{"2H8  10A"}=" H82 10A";
$resn_exch{"1H9  10A"}=" H91 10A"; $resn_exch{"2H9  10A"}=" H92 10A"; $resn_exch{"1H10 10A"}="H101 10A";
$resn_exch{"2H10 10A"}="H102 10A"; $resn_exch{"1H11 10A"}="H111 10A"; $resn_exch{"2H11 10A"}="H112 10A";
$resn_exch{"3H11 10A"}="H113 10A"; $resn_exch{"1H12 10A"}="H121 10A"; $resn_exch{"2H12 10A"}="H122 10A";
$resn_exch{"3H12 10A"}="H123 10A"; $resn_exch{"1H13 10A"}="H131 10A"; $resn_exch{"2H13 10A"}="H132 10A";
$resn_exch{"1H14 10A"}="H141 10A"; $resn_exch{"2H14 10A"}="H142 10A"; $resn_exch{"1H15 10A"}="H151 10A";
$resn_exch{"2H15 10A"}="H152 10A"; $resn_exch{"1H16 10A"}="H161 10A"; $resn_exch{"2H16 10A"}="H162 10A";
$resn_exch{"1H17 10A"}="H171 10A"; $resn_exch{"2H17 10A"}="H172 10A"; $resn_exch{"1H18 10A"}="H181 10A";
$resn_exch{"2H18 10A"}="H182 10A"; $resn_exch{"1H19 10A"}="H191 10A"; $resn_exch{"2H19 10A"}="H192 10A";
$resn_exch{"1H20 10A"}="H201 10A"; $resn_exch{"2H20 10A"}="H202 10A"; $resn_exch{"1H21 10A"}="H211 10A";
$resn_exch{"2H21 10A"}="H212 10A"; $resn_exch{"1H22 10A"}="H221 10A"; $resn_exch{"2H22 10A"}="H222 10A";
$resn_exch{"3H22 10A"}="H223 10A"; $resn_exch{"1HOP 10C"}="HOP1 10C"; $resn_exch{"2HOP 10C"}="HOP2 10C";
$resn_exch{"1H5* 10C"}="H5'1 10C"; $resn_exch{"2H5* 10C"}="H5'2 10C"; $resn_exch{" H1* 10C"}=" H1' 10C";
$resn_exch{" H3* 10C"}=" H3' 10C"; $resn_exch{"1H6* 10C"}="H6'1 10C"; $resn_exch{"2H6* 10C"}="H6'2 10C";
$resn_exch{"1H4  10C"}=" H41 10C"; $resn_exch{"2H4  10C"}=" H42 10C"; $resn_exch{"1H2* 10C"}="H2'1 10C";
$resn_exch{"1H10 110"}="H101 110"; $resn_exch{"2H10 110"}="H102 110"; $resn_exch{"1H11 110"}="H111 110";
$resn_exch{"2H11 110"}="H112 110"; $resn_exch{"1H21 110"}="H211 110"; $resn_exch{"2H21 110"}="H212 110";
$resn_exch{"1H22 110"}="H221 110"; $resn_exch{"2H22 110"}="H222 110"; $resn_exch{"1H19 110"}="H191 110";
$resn_exch{"2H19 110"}="H192 110"; $resn_exch{"1H12 110"}="H121 110"; $resn_exch{"2H12 110"}="H122 110";
$resn_exch{"1H13 110"}="H131 110"; $resn_exch{"2H13 110"}="H132 110"; $resn_exch{"3H13 110"}="H133 110";
$resn_exch{"1H4  110"}=" H41 110"; $resn_exch{"2H4  110"}=" H42 110"; $resn_exch{"1H5  110"}=" H51 110";
$resn_exch{"2H5  110"}=" H52 110"; $resn_exch{"1HD2 110"}="HD21 110"; $resn_exch{"2HD2 110"}="HD22 110";
$resn_exch{"1H7  110"}=" H71 110"; $resn_exch{"2H7  110"}=" H72 110"; $resn_exch{"1H8  110"}=" H81 110";
$resn_exch{"2H8  110"}=" H82 110"; $resn_exch{"1H13 111"}="H131 111"; $resn_exch{"2H13 111"}="H132 111";
$resn_exch{"3H13 111"}="H133 111"; $resn_exch{"1H11 111"}="H111 111"; $resn_exch{"2H11 111"}="H112 111";
$resn_exch{"1H9  111"}=" H91 111"; $resn_exch{"2H9  111"}=" H92 111"; $resn_exch{"1H7  111"}=" H71 111";
$resn_exch{"2H7  111"}=" H72 111"; $resn_exch{"1H10 111"}="H101 111"; $resn_exch{"2H10 111"}="H102 111";
$resn_exch{"1H12 111"}="H121 111"; $resn_exch{"2H12 111"}="H122 111"; $resn_exch{"1H14 111"}="H141 111";
$resn_exch{"2H14 111"}="H142 111"; $resn_exch{"3H14 111"}="H143 111"; $resn_exch{"1HNS 112"}="HNS1 112";
$resn_exch{"2HNS 112"}="HNS2 112"; $resn_exch{"1H1S 112"}="H1S1 112"; $resn_exch{"2H1S 112"}="H1S2 112";
$resn_exch{"2HOG 112"}="HOG2 112"; $resn_exch{"2HOB 112"}="HOB2 112"; $resn_exch{"2HOA 112"}="HOA2 112";
$resn_exch{"1H5* 112"}="H5'1 112"; $resn_exch{"2H5* 112"}="H5'2 112"; $resn_exch{" H4* 112"}=" H4' 112";
$resn_exch{" H3* 112"}=" H3' 112"; $resn_exch{"*HO3 112"}="HO3' 112"; $resn_exch{" H2* 112"}=" H2' 112";
$resn_exch{"*HO2 112"}="HO2' 112"; $resn_exch{" H1* 112"}=" H1' 112"; $resn_exch{"1HN6 112"}="HN61 112";
$resn_exch{"2HN6 112"}="HN62 112"; $resn_exch{"1H12 113"}="H121 113"; $resn_exch{"2H12 113"}="H122 113";
$resn_exch{"1H22 113"}="H221 113"; $resn_exch{"2H22 113"}="H222 113"; $resn_exch{"3H22 113"}="H223 113";
$resn_exch{"1H26 113"}="H261 113"; $resn_exch{"2H26 113"}="H262 113"; $resn_exch{"3H26 113"}="H263 113";
$resn_exch{"1H2  114"}=" H21 114"; $resn_exch{"2H2  114"}=" H22 114"; $resn_exch{"1H4  114"}=" H41 114";
$resn_exch{"2H4  114"}=" H42 114"; $resn_exch{"1H6  114"}=" H61 114"; $resn_exch{"2H6  114"}=" H62 114";
$resn_exch{"1H7  114"}=" H71 114"; $resn_exch{"2H7  114"}=" H72 114"; $resn_exch{"1H16 114"}="H161 114";
$resn_exch{"2H16 114"}="H162 114"; $resn_exch{"1H15 114"}="H151 114"; $resn_exch{"2H15 114"}="H152 114";
$resn_exch{"1H9A 114"}="H9A1 114"; $resn_exch{"2H9A 114"}="H9A2 114"; $resn_exch{"3H9A 114"}="H9A3 114";
$resn_exch{"1H20 114"}="H201 114"; $resn_exch{"2H20 114"}="H202 114"; $resn_exch{"1H21 114"}="H211 114";
$resn_exch{"2H21 114"}="H212 114"; $resn_exch{"3H21 114"}="H213 114"; $resn_exch{"1H22 114"}="H221 114";
$resn_exch{"2H22 114"}="H222 114"; $resn_exch{"3H22 114"}="H223 114"; $resn_exch{"1H2  115"}=" H21 115";
$resn_exch{"2H2  115"}=" H22 115"; $resn_exch{"1H4  115"}=" H41 115"; $resn_exch{"2H4  115"}=" H42 115";
$resn_exch{"1H6  115"}=" H61 115"; $resn_exch{"2H6  115"}=" H62 115"; $resn_exch{"1H7  115"}=" H71 115";
$resn_exch{"2H7  115"}=" H72 115"; $resn_exch{"1H92 115"}="H921 115"; $resn_exch{"2H92 115"}="H922 115";
$resn_exch{"3H92 115"}="H923 115"; $resn_exch{"1H93 115"}="H931 115"; $resn_exch{"2H93 115"}="H932 115";
$resn_exch{"3H93 115"}="H933 115"; $resn_exch{"1H14 116"}="H141 116"; $resn_exch{"2H14 116"}="H142 116";
$resn_exch{"3H14 116"}="H143 116"; $resn_exch{"1H13 116"}="H131 116"; $resn_exch{"2H13 116"}="H132 116";
$resn_exch{"1H17 116"}="H171 116"; $resn_exch{"2H17 116"}="H172 116"; $resn_exch{"3H17 116"}="H173 116";
$resn_exch{"1H16 116"}="H161 116"; $resn_exch{"2H16 116"}="H162 116"; $resn_exch{"3H16 116"}="H163 116";
$resn_exch{"1H2  116"}=" H21 116"; $resn_exch{"2H2  116"}=" H22 116"; $resn_exch{"1H4  116"}=" H41 116";
$resn_exch{"2H4  116"}=" H42 116"; $resn_exch{"1H6  116"}=" H61 116"; $resn_exch{"2H6  116"}=" H62 116";
$resn_exch{"1H7  116"}=" H71 116"; $resn_exch{"2H7  116"}=" H72 116"; $resn_exch{"1H92 116"}="H921 116";
$resn_exch{"2H92 116"}="H922 116"; $resn_exch{"3H92 116"}="H923 116"; $resn_exch{"1H93 116"}="H931 116";
$resn_exch{"2H93 116"}="H932 116"; $resn_exch{"3H93 116"}="H933 116"; $resn_exch{"1H2  117"}="H21A 117";
$resn_exch{"2H2  117"}="H22A 117"; $resn_exch{"1H4  117"}=" H41 117"; $resn_exch{"2H4  117"}=" H42 117";
$resn_exch{"1H6  117"}=" H61 117"; $resn_exch{"2H6  117"}=" H62 117"; $resn_exch{"1H7  117"}=" H71 117";
$resn_exch{"2H7  117"}=" H72 117"; $resn_exch{"1H93 117"}="H931 117"; $resn_exch{"2H93 117"}="H932 117";
$resn_exch{"3H93 117"}="H933 117"; $resn_exch{"1H92 117"}="H921 117"; $resn_exch{"2H92 117"}="H922 117";
$resn_exch{"3H92 117"}="H923 117"; $resn_exch{"1H35 11B"}="H351 11B"; $resn_exch{"2H35 11B"}="H352 11B";
$resn_exch{"1H26 11B"}="H261 11B"; $resn_exch{"2H26 11B"}="H262 11B"; $resn_exch{"1H27 11B"}="H271 11B";
$resn_exch{"2H27 11B"}="H272 11B"; $resn_exch{"1H28 11B"}="H281 11B"; $resn_exch{"2H28 11B"}="H282 11B";
$resn_exch{"1H29 11B"}="H291 11B"; $resn_exch{"2H29 11B"}="H292 11B"; $resn_exch{"1H30 11B"}="H301 11B";
$resn_exch{"2H30 11B"}="H302 11B"; $resn_exch{"3H30 11B"}="H303 11B"; $resn_exch{"1H11 11B"}="H111 11B";
$resn_exch{"2H11 11B"}="H112 11B"; $resn_exch{"1H2  11O"}=" H21 11O"; $resn_exch{"2H2  11O"}=" H22 11O";
$resn_exch{"1H3  11O"}=" H31 11O"; $resn_exch{"2H3  11O"}=" H32 11O"; $resn_exch{"1H4  11O"}=" H41 11O";
$resn_exch{"2H4  11O"}=" H42 11O"; $resn_exch{"1H5  11O"}=" H51 11O"; $resn_exch{"2H5  11O"}=" H52 11O";
$resn_exch{"1H6  11O"}=" H61 11O"; $resn_exch{"2H6  11O"}=" H62 11O"; $resn_exch{"1H7  11O"}=" H71 11O";
$resn_exch{"2H7  11O"}=" H72 11O"; $resn_exch{"1H8  11O"}=" H81 11O"; $resn_exch{"2H8  11O"}=" H82 11O";
$resn_exch{"1H9  11O"}=" H91 11O"; $resn_exch{"1H10 11O"}="H101 11O"; $resn_exch{"1H11 11O"}="H111 11O";
$resn_exch{"1H12 11O"}="H121 11O"; $resn_exch{"1H13 11O"}="H131 11O"; $resn_exch{"1H14 11O"}="H141 11O";
$resn_exch{"2H14 11O"}="H142 11O"; $resn_exch{"1H15 11O"}="H151 11O"; $resn_exch{"2H15 11O"}="H152 11O";
$resn_exch{"1H16 11O"}="H161 11O"; $resn_exch{"2H16 11O"}="H162 11O"; $resn_exch{"1H17 11O"}="H171 11O";
$resn_exch{"2H17 11O"}="H172 11O"; $resn_exch{"1H18 11O"}="H181 11O"; $resn_exch{"2H18 11O"}="H182 11O";
$resn_exch{"3H18 11O"}="H183 11O"; $resn_exch{"1H2O 11O"}="H2O1 11O"; $resn_exch{"1H21 11O"}="H211 11O";
$resn_exch{"1H16 11P"}="H161 11P"; $resn_exch{"2H16 11P"}="H162 11P"; $resn_exch{"1H17 11P"}="H171 11P";
$resn_exch{"2H17 11P"}="H172 11P"; $resn_exch{"3HM2 11R"}="HM23 11R"; $resn_exch{"2HM2 11R"}="HM22 11R";
$resn_exch{"1HM2 11R"}="HM21 11R"; $resn_exch{"3HM1 11R"}="HM13 11R"; $resn_exch{"2HM1 11R"}="HM12 11R";
$resn_exch{"1HM1 11R"}="HM11 11R"; $resn_exch{"1HX6 11R"}="HX61 11R"; $resn_exch{"2HX6 11R"}="HX62 11R";
$resn_exch{"1HX5 11R"}="HX51 11R"; $resn_exch{"2HX5 11R"}="HX52 11R"; $resn_exch{"1HX4 11R"}="HX41 11R";
$resn_exch{"2HX4 11R"}="HX42 11R"; $resn_exch{"1HX3 11R"}="HX31 11R"; $resn_exch{"2HX3 11R"}="HX32 11R";
$resn_exch{"1HX2 11R"}="HX21 11R"; $resn_exch{"2HX2 11R"}="HX22 11R"; $resn_exch{"1HL6 11R"}="HL61 11R";
$resn_exch{"2HL6 11R"}="HL62 11R"; $resn_exch{"1HL5 11R"}="HL51 11R"; $resn_exch{"2HL5 11R"}="HL52 11R";
$resn_exch{"1HL4 11R"}="HL41 11R"; $resn_exch{"2HL4 11R"}="HL42 11R"; $resn_exch{"1HL3 11R"}="HL31 11R";
$resn_exch{"2HL3 11R"}="HL32 11R"; $resn_exch{"1HL2 11R"}="HL21 11R"; $resn_exch{"2HL2 11R"}="HL22 11R";
$resn_exch{"1HL1 11R"}="HL11 11R"; $resn_exch{"2HL1 11R"}="HL12 11R"; $resn_exch{"1HAN 11R"}="HAN1 11R";
$resn_exch{"1HCK 11R"}="HCK1 11R"; $resn_exch{"1HCJ 11R"}="HCJ1 11R"; $resn_exch{"1HCE 11R"}="HCE1 11R";
$resn_exch{"1HCD 11R"}="HCD1 11R"; $resn_exch{"1HCC 11R"}="HCC1 11R"; $resn_exch{"1HCB 11R"}="HCB1 11R";
$resn_exch{"1HBK 11R"}="HBK1 11R"; $resn_exch{"1HBJ 11R"}="HBJ1 11R"; $resn_exch{"1HBI 11R"}="HBI1 11R";
$resn_exch{"1HBH 11R"}="HBH1 11R"; $resn_exch{"1HBE 11R"}="HBE1 11R"; $resn_exch{"1HBD 11R"}="HBD1 11R";
$resn_exch{"1HBC 11R"}="HBC1 11R"; $resn_exch{"1HBB 11R"}="HBB1 11R"; $resn_exch{"1HH1 120"}="HH11 120";
$resn_exch{"2HH1 120"}="HH12 120"; $resn_exch{"1HH2 120"}="HH21 120"; $resn_exch{"2HH2 120"}="HH22 120";
$resn_exch{"1HH1 121"}="HH11 121"; $resn_exch{"2HH1 121"}="HH12 121"; $resn_exch{"1HH2 121"}="HH21 121";
$resn_exch{"2HH2 121"}="HH22 121"; $resn_exch{"1HH1 122"}="HH11 122"; $resn_exch{"2HH1 122"}="HH12 122";
$resn_exch{"1HH2 122"}="HH21 122"; $resn_exch{"2HH2 122"}="HH22 122"; $resn_exch{"1HH1 123"}="HH11 123";
$resn_exch{"2HH1 123"}="HH12 123"; $resn_exch{"1HH2 123"}="HH21 123"; $resn_exch{"2HH2 123"}="HH22 123";
$resn_exch{"1H3' 123"}="H3'1 123"; $resn_exch{"2H3' 123"}="H3'2 123"; $resn_exch{"3H3' 123"}="H3'3 123";
$resn_exch{"1HH1 124"}="HH11 124"; $resn_exch{"2HH1 124"}="HH12 124"; $resn_exch{"1HH2 124"}="HH21 124";
$resn_exch{"2HH2 124"}="HH22 124"; $resn_exch{"2HOP 125"}="HOP2 125"; $resn_exch{"3HOP 125"}="HOP3 125";
$resn_exch{"1H5* 125"}=" H5' 125"; $resn_exch{"2H5* 125"}="H5'' 125"; $resn_exch{" H4* 125"}=" H4' 125";
$resn_exch{" H3* 125"}=" H3' 125"; $resn_exch{"H3T  125"}="HO3' 125"; $resn_exch{" H2* 125"}=" H2' 125";
$resn_exch{" H1* 125"}=" H1' 125"; $resn_exch{"1H5M 125"}=" H71 125"; $resn_exch{"2H5M 125"}=" H72 125";
$resn_exch{"3H5M 125"}=" H73 125"; $resn_exch{"1HA* 125"}="HA'1 125"; $resn_exch{"2HA* 125"}="HA'2 125";
$resn_exch{"1HB* 125"}="HB'1 125"; $resn_exch{"2HB* 125"}="HB'2 125"; $resn_exch{"2HOP 126"}="HOP2 126";
$resn_exch{"3HOP 126"}="HOP3 126"; $resn_exch{"1H5* 126"}=" H5' 126"; $resn_exch{"2H5* 126"}="H5'' 126";
$resn_exch{" H4* 126"}=" H4' 126"; $resn_exch{" H3* 126"}=" H3' 126"; $resn_exch{"H3T  126"}="HO3' 126";
$resn_exch{" H2* 126"}=" H2' 126"; $resn_exch{" H1* 126"}=" H1' 126"; $resn_exch{"1H5M 126"}=" H71 126";
$resn_exch{"2H5M 126"}=" H72 126"; $resn_exch{"3H5M 126"}=" H73 126"; $resn_exch{"1HA* 126"}="HA'1 126";
$resn_exch{"2HA* 126"}="HA'2 126"; $resn_exch{"1HB* 126"}="HB'1 126"; $resn_exch{"2HB* 126"}="HB'2 126";
$resn_exch{"1HD* 126"}="HD'1 126"; $resn_exch{"2HD* 126"}="HD'2 126"; $resn_exch{"1HE* 126"}="HE'1 126";
$resn_exch{"2HE* 126"}="HE'2 126"; $resn_exch{"1HG* 126"}="HG'1 126"; $resn_exch{"2HG* 126"}="HG'2 126";
$resn_exch{"1HH* 126"}="HH'1 126"; $resn_exch{"2HH* 126"}="HH'2 126"; $resn_exch{"1HK* 126"}="HK'1 126";
$resn_exch{"2HK* 126"}="HK'2 126"; $resn_exch{"3HK* 126"}="HK'3 126"; $resn_exch{"2HOP 127"}="HOP2 127";
$resn_exch{"3HOP 127"}="HOP3 127"; $resn_exch{"1H5* 127"}=" H5' 127"; $resn_exch{"2H5* 127"}="H5'' 127";
$resn_exch{" H4* 127"}=" H4' 127"; $resn_exch{" H3* 127"}=" H3' 127"; $resn_exch{"H3T  127"}="HO3' 127";
$resn_exch{" H2* 127"}=" H2' 127"; $resn_exch{" H1* 127"}=" H1' 127"; $resn_exch{"1H5M 127"}=" H71 127";
$resn_exch{"2H5M 127"}=" H72 127"; $resn_exch{"3H5M 127"}=" H73 127"; $resn_exch{"1HA* 127"}="HA'1 127";
$resn_exch{"2HA* 127"}="HA'2 127"; $resn_exch{"1HB* 127"}="HB'1 127"; $resn_exch{"2HB* 127"}="HB'2 127";
$resn_exch{"1HD* 127"}="HD'1 127"; $resn_exch{"2HD* 127"}="HD'2 127"; $resn_exch{"2HOG 128"}="HOG2 128";
$resn_exch{"3HOG 128"}="HOG3 128"; $resn_exch{"2HOB 128"}="HOB2 128"; $resn_exch{"2HOA 128"}="HOA2 128";
$resn_exch{"1H5* 128"}=" H5' 128"; $resn_exch{"2H5* 128"}="H5'' 128"; $resn_exch{" H4* 128"}=" H4' 128";
$resn_exch{" H3* 128"}=" H3' 128"; $resn_exch{" H2* 128"}=" H2' 128"; $resn_exch{" H1* 128"}=" H1' 128";
$resn_exch{"1HN6 128"}="HN61 128"; $resn_exch{"2HN6 128"}="HN62 128"; $resn_exch{"2HOP 129"}="HOP2 129";
$resn_exch{"3HOP 129"}="HOP3 129"; $resn_exch{"1H4  129"}=" H41 129"; $resn_exch{"2H4  129"}=" H42 129";
$resn_exch{"1H3  129"}=" H31 129"; $resn_exch{"2H3  129"}=" H32 129"; $resn_exch{"3HOP 12A"}="HOP3 12A";
$resn_exch{"2HOP 12A"}="HOP2 12A"; $resn_exch{"1H5* 12A"}=" H5' 12A"; $resn_exch{"2H5* 12A"}="H5'' 12A";
$resn_exch{" H4* 12A"}=" H4' 12A"; $resn_exch{" H3* 12A"}=" H3' 12A"; $resn_exch{"H3T  12A"}="HO3' 12A";
$resn_exch{" H2* 12A"}=" H2' 12A"; $resn_exch{"2HO* 12A"}="HO2' 12A"; $resn_exch{" H1* 12A"}=" H1' 12A";
$resn_exch{"1H2M 12A"}="H2M1 12A"; $resn_exch{"2H2M 12A"}="H2M2 12A"; $resn_exch{"3H2M 12A"}="H2M3 12A";
$resn_exch{"1HG2 12A"}="HG21 12A"; $resn_exch{"2HG2 12A"}="HG22 12A"; $resn_exch{"3HG2 12A"}="HG23 12A";
$resn_exch{"1H18 12C"}="H181 12C"; $resn_exch{"2H18 12C"}="H182 12C"; $resn_exch{"1H19 12C"}="H191 12C";
$resn_exch{"2H19 12C"}="H192 12C"; $resn_exch{"1H20 12C"}="H201 12C"; $resn_exch{"2H20 12C"}="H202 12C";
$resn_exch{"1H21 12C"}="H211 12C"; $resn_exch{"2H21 12C"}="H212 12C"; $resn_exch{"1H22 12C"}="H221 12C";
$resn_exch{"2H22 12C"}="H222 12C"; $resn_exch{"1H1  12H"}=" H11 12H"; $resn_exch{"2H1  12H"}=" H12 12H";
$resn_exch{"1H2  12H"}=" H21 12H"; $resn_exch{"2H2  12H"}=" H22 12H"; $resn_exch{"1H3  12H"}=" H31 12H";
$resn_exch{"2H3  12H"}=" H32 12H"; $resn_exch{"1H4  12H"}=" H41 12H"; $resn_exch{"2H4  12H"}=" H42 12H";
$resn_exch{"1H5  12H"}=" H51 12H"; $resn_exch{"2H5  12H"}=" H52 12H"; $resn_exch{"1H6  12H"}=" H61 12H";
$resn_exch{"2H6  12H"}=" H62 12H"; $resn_exch{"1H7  12H"}=" H71 12H"; $resn_exch{"2H7  12H"}=" H72 12H";
$resn_exch{"1H8  12H"}=" H81 12H"; $resn_exch{"2H8  12H"}=" H82 12H"; $resn_exch{"1H9  12H"}=" H91 12H";
$resn_exch{"2H9  12H"}=" H92 12H"; $resn_exch{"1H10 12H"}="H101 12H"; $resn_exch{"2H10 12H"}="H102 12H";
$resn_exch{"1H11 12H"}="H111 12H"; $resn_exch{"2H11 12H"}="H112 12H"; $resn_exch{"1HAH 12M"}="HAH1 12M";
$resn_exch{"2HAH 12M"}="HAH2 12M"; $resn_exch{"3HAH 12M"}="HAH3 12M"; $resn_exch{"1HAD 12M"}="HAD1 12M";
$resn_exch{"2HAD 12M"}="HAD2 12M"; $resn_exch{"1HAF 12M"}="HAF1 12M"; $resn_exch{"2HAF 12M"}="HAF2 12M";
$resn_exch{"1H36 12P"}="H361 12P"; $resn_exch{"2H36 12P"}="H362 12P"; $resn_exch{"1H35 12P"}="H351 12P";
$resn_exch{"2H35 12P"}="H352 12P"; $resn_exch{"1H33 12P"}="H331 12P"; $resn_exch{"2H33 12P"}="H332 12P";
$resn_exch{"1H32 12P"}="H321 12P"; $resn_exch{"2H32 12P"}="H322 12P"; $resn_exch{"1H30 12P"}="H301 12P";
$resn_exch{"2H30 12P"}="H302 12P"; $resn_exch{"1H29 12P"}="H291 12P"; $resn_exch{"2H29 12P"}="H292 12P";
$resn_exch{"1H27 12P"}="H271 12P"; $resn_exch{"2H27 12P"}="H272 12P"; $resn_exch{"1H26 12P"}="H261 12P";
$resn_exch{"2H26 12P"}="H262 12P"; $resn_exch{"1H24 12P"}="H241 12P"; $resn_exch{"2H24 12P"}="H242 12P";
$resn_exch{"1H23 12P"}="H231 12P"; $resn_exch{"2H23 12P"}="H232 12P"; $resn_exch{"1H21 12P"}="H211 12P";
$resn_exch{"2H21 12P"}="H212 12P"; $resn_exch{"1H20 12P"}="H201 12P"; $resn_exch{"2H20 12P"}="H202 12P";
$resn_exch{"1H18 12P"}="H181 12P"; $resn_exch{"2H18 12P"}="H182 12P"; $resn_exch{"1H17 12P"}="H171 12P";
$resn_exch{"2H17 12P"}="H172 12P"; $resn_exch{"1H15 12P"}="H151 12P"; $resn_exch{"2H15 12P"}="H152 12P";
$resn_exch{"1H14 12P"}="H141 12P"; $resn_exch{"2H14 12P"}="H142 12P"; $resn_exch{"1H12 12P"}="H121 12P";
$resn_exch{"2H12 12P"}="H122 12P"; $resn_exch{"1H11 12P"}="H111 12P"; $resn_exch{"2H11 12P"}="H112 12P";
$resn_exch{"1H9  12P"}=" H91 12P"; $resn_exch{"2H9  12P"}=" H92 12P"; $resn_exch{"1H8  12P"}=" H81 12P";
$resn_exch{"2H8  12P"}=" H82 12P"; $resn_exch{"1H6  12P"}=" H61 12P"; $resn_exch{"2H6  12P"}=" H62 12P";
$resn_exch{"1H5  12P"}=" H51 12P"; $resn_exch{"2H5  12P"}=" H52 12P"; $resn_exch{"1H3  12P"}=" H31 12P";
$resn_exch{"2H3  12P"}=" H32 12P"; $resn_exch{"1H2  12P"}=" H21 12P"; $resn_exch{"2H2  12P"}=" H22 12P";
$resn_exch{"1HAH 12Q"}="HAH1 12Q"; $resn_exch{"2HAH 12Q"}="HAH2 12Q"; $resn_exch{"3HAH 12Q"}="HAH3 12Q";
$resn_exch{"1HH1 130"}="HH11 130"; $resn_exch{"2HH1 130"}="HH12 130"; $resn_exch{"1HH2 130"}="HH21 130";
$resn_exch{"2HC' 130"}="HC'2 130"; $resn_exch{"3HC' 130"}="HC'3 130"; $resn_exch{"4HC' 130"}="HC'4 130";
$resn_exch{" H2* 130"}=" H2' 130"; $resn_exch{" H3* 130"}=" H3' 130"; $resn_exch{" H4* 130"}=" H4' 130";
$resn_exch{" H5* 130"}=" H5' 130"; $resn_exch{" H6* 130"}=" H6' 130"; $resn_exch{"1HH1 132"}="HH11 132";
$resn_exch{"2HH1 132"}="HH12 132"; $resn_exch{"1HH2 132"}="HH21 132"; $resn_exch{"2HH2 132"}="HH22 132";
$resn_exch{"2HC' 132"}="HC'2 132"; $resn_exch{"3HC' 132"}="HC'3 132"; $resn_exch{"4HC' 132"}="HC'4 132";
$resn_exch{" H2* 132"}=" H2' 132"; $resn_exch{" H3* 132"}=" H3' 132"; $resn_exch{" H4* 132"}=" H4' 132";
$resn_exch{" H5* 132"}=" H5' 132"; $resn_exch{" H6* 132"}=" H6' 132"; $resn_exch{"1HH1 133"}="HH11 133";
$resn_exch{"2HH1 133"}="HH12 133"; $resn_exch{"1HH2 133"}="HH21 133"; $resn_exch{"2HC' 133"}="HC'2 133";
$resn_exch{"3HC' 133"}="HC'3 133"; $resn_exch{"4HC' 133"}="HC'4 133"; $resn_exch{"6HO' 133"}="HO'6 133";
$resn_exch{"1H10 133"}="H101 133"; $resn_exch{"2H10 133"}="H102 133"; $resn_exch{"1H30 133"}="H301 133";
$resn_exch{"2H30 133"}="H302 133"; $resn_exch{"3H30 133"}="H303 133"; $resn_exch{"1H40 133"}="H401 133";
$resn_exch{"2H40 133"}="H402 133"; $resn_exch{"3H40 133"}="H403 133"; $resn_exch{"1HH1 134"}="HH11 134";
$resn_exch{"2HH1 134"}="HH12 134"; $resn_exch{"1HH2 134"}="HH21 134"; $resn_exch{"2HC' 134"}="HC'2 134";
$resn_exch{"3HC' 134"}="HC'3 134"; $resn_exch{"4HC' 134"}="HC'4 134"; $resn_exch{"6HO' 134"}="HO'6 134";
$resn_exch{" H1* 134"}=" H1' 134"; $resn_exch{"1H2* 134"}="H2'1 134"; $resn_exch{"2H2* 134"}="H2'2 134";
$resn_exch{"1H3* 134"}="H3'1 134"; $resn_exch{"2H3* 134"}="H3'2 134"; $resn_exch{"1H4* 134"}="H4'1 134";
$resn_exch{"2H4* 134"}="H4'2 134"; $resn_exch{"1H5* 134"}="H5'1 134"; $resn_exch{"2H5* 134"}="H5'2 134";
$resn_exch{" H6* 134"}=" H6' 134"; $resn_exch{"1HM  134"}=" HM1 134"; $resn_exch{"2HM  134"}=" HM2 134";
$resn_exch{"3HM  134"}=" HM3 134"; $resn_exch{"1HH1 135"}="HH11 135"; $resn_exch{"2HH1 135"}="HH12 135";
$resn_exch{"1HH2 135"}="HH21 135"; $resn_exch{"1HH1 136"}="HH11 136"; $resn_exch{"2HH1 136"}="HH12 136";
$resn_exch{"1HH2 136"}="HH21 136"; $resn_exch{"1H21 136"}="H211 136"; $resn_exch{"2H21 136"}="H212 136";
$resn_exch{"3H21 136"}="H213 136"; $resn_exch{"2HOP 137"}="HOP2 137"; $resn_exch{"3HOP 137"}="HOP3 137";
$resn_exch{"1H5' 137"}="H5'1 137"; $resn_exch{"2H5' 137"}="H5'2 137"; $resn_exch{"4HO' 137"}="HO'4 137";
$resn_exch{"3HO' 137"}="HO'3 137"; $resn_exch{"2HO' 137"}="HO'2 137"; $resn_exch{"1H1' 137"}="H1'1 137";
$resn_exch{"2H1' 137"}="H1'2 137"; $resn_exch{"1HA2 138"}="HA21 138"; $resn_exch{"2HA2 138"}="HA22 138";
$resn_exch{"1H9  138"}=" H91 138"; $resn_exch{"2H9  138"}=" H92 138"; $resn_exch{"1HA1 138"}="HA11 138";
$resn_exch{"2HA1 138"}="HA12 138"; $resn_exch{"1HB  138"}=" HB1 138"; $resn_exch{"2HB  138"}=" HB2 138";
$resn_exch{"1HG  138"}=" HG1 138"; $resn_exch{"2HG  138"}=" HG2 138"; $resn_exch{"1H27 138"}="H271 138";
$resn_exch{"2H27 138"}="H272 138"; $resn_exch{"1H19 138"}="H191 138"; $resn_exch{"2H19 138"}="H192 138";
$resn_exch{"2HOA 139"}="HOA2 139"; $resn_exch{"2HOB 139"}="HOB2 139"; $resn_exch{"1H6A 139"}="H6A1 139";
$resn_exch{"2H6A 139"}="H6A2 139"; $resn_exch{"1HA5 139"}="HA51 139"; $resn_exch{"2HA5 139"}="HA52 139";
$resn_exch{"H4*A 139"}=" H4D 139"; $resn_exch{"H1*A 139"}=" H1D 139"; $resn_exch{"H2*A 139"}=" H2D 139";
$resn_exch{"2H*A 139"}=" HD2 139"; $resn_exch{"H3*A 139"}=" H3D 139"; $resn_exch{"3H*A 139"}=" HD3 139";
$resn_exch{"5H*U 139"}=" HB5 139"; $resn_exch{"1HU5 139"}="HU51 139"; $resn_exch{"2HU5 139"}="HU52 139";
$resn_exch{"H4*U 139"}=" H4B 139"; $resn_exch{"H1*U 139"}=" H1B 139"; $resn_exch{"1HU2 139"}="HU21 139";
$resn_exch{"2HU2 139"}="HU22 139"; $resn_exch{"H3*U 139"}=" H3B 139"; $resn_exch{"1H8  13A"}=" H81 13A";
$resn_exch{"2H8  13A"}=" H82 13A"; $resn_exch{"1HAC 13D"}="HAC1 13D"; $resn_exch{"2HAC 13D"}="HAC2 13D";
$resn_exch{"1HBC 13D"}="HBC1 13D"; $resn_exch{"2HBC 13D"}="HBC2 13D"; $resn_exch{"1HCC 13D"}="HCC1 13D";
$resn_exch{"2HCC 13D"}="HCC2 13D"; $resn_exch{"1HDN 13D"}="HDN1 13D"; $resn_exch{"2HDN 13D"}="HDN2 13D";
$resn_exch{"1HAA 13D"}="HAA1 13D"; $resn_exch{"2HAA 13D"}="HAA2 13D"; $resn_exch{"2HOP 13P"}="HOP2 13P";
$resn_exch{"3HOP 13P"}="HOP3 13P"; $resn_exch{"1H1  13P"}=" H11 13P"; $resn_exch{"2H1  13P"}=" H12 13P";
$resn_exch{"1H3  13P"}=" H31 13P"; $resn_exch{"2H3  13P"}=" H32 13P"; $resn_exch{"1H2  13R"}=" H21 13R";
$resn_exch{"2H2  13R"}=" H22 13R"; $resn_exch{"1H3  13R"}=" H31 13R"; $resn_exch{"2H3  13R"}=" H32 13R";
$resn_exch{"1H4  13R"}=" H41 13R"; $resn_exch{"2H4  13R"}=" H42 13R"; $resn_exch{"1H5  13R"}=" H51 13R";
$resn_exch{"2H5  13R"}=" H52 13R"; $resn_exch{"1H6  13R"}=" H61 13R"; $resn_exch{"2H6  13R"}=" H62 13R";
$resn_exch{"1H7  13R"}=" H71 13R"; $resn_exch{"2H7  13R"}=" H72 13R"; $resn_exch{"1H8  13R"}=" H81 13R";
$resn_exch{"2H8  13R"}=" H82 13R"; $resn_exch{"1H9  13R"}=" H91 13R"; $resn_exch{"1H10 13R"}="H101 13R";
$resn_exch{"1H11 13R"}="H111 13R"; $resn_exch{"1H12 13R"}="H121 13R"; $resn_exch{"1H13 13R"}="H131 13R";
$resn_exch{"1H14 13R"}="H141 13R"; $resn_exch{"2H14 13R"}="H142 13R"; $resn_exch{"1H15 13R"}="H151 13R";
$resn_exch{"2H15 13R"}="H152 13R"; $resn_exch{"1H16 13R"}="H161 13R"; $resn_exch{"2H16 13R"}="H162 13R";
$resn_exch{"1H17 13R"}="H171 13R"; $resn_exch{"2H17 13R"}="H172 13R"; $resn_exch{"1H18 13R"}="H181 13R";
$resn_exch{"2H18 13R"}="H182 13R"; $resn_exch{"3H18 13R"}="H183 13R"; $resn_exch{"1H20 13R"}="H201 13R";
$resn_exch{"1H22 13R"}="H221 13R"; $resn_exch{"1H2  13S"}=" H21 13S"; $resn_exch{"2H2  13S"}=" H22 13S";
$resn_exch{"1H3  13S"}=" H31 13S"; $resn_exch{"2H3  13S"}=" H32 13S"; $resn_exch{"1H4  13S"}=" H41 13S";
$resn_exch{"2H4  13S"}=" H42 13S"; $resn_exch{"1H5  13S"}=" H51 13S"; $resn_exch{"2H5  13S"}=" H52 13S";
$resn_exch{"1H6  13S"}=" H61 13S"; $resn_exch{"2H6  13S"}=" H62 13S"; $resn_exch{"1H7  13S"}=" H71 13S";
$resn_exch{"2H7  13S"}=" H72 13S"; $resn_exch{"1H8  13S"}=" H81 13S"; $resn_exch{"2H8  13S"}=" H82 13S";
$resn_exch{"1H9  13S"}=" H91 13S"; $resn_exch{"1H10 13S"}="H101 13S"; $resn_exch{"1H11 13S"}="H111 13S";
$resn_exch{"1H12 13S"}="H121 13S"; $resn_exch{"1H13 13S"}="H131 13S"; $resn_exch{"1H14 13S"}="H141 13S";
$resn_exch{"2H14 13S"}="H142 13S"; $resn_exch{"1H15 13S"}="H151 13S"; $resn_exch{"2H15 13S"}="H152 13S";
$resn_exch{"1H16 13S"}="H161 13S"; $resn_exch{"2H16 13S"}="H162 13S"; $resn_exch{"1H17 13S"}="H171 13S";
$resn_exch{"2H17 13S"}="H172 13S"; $resn_exch{"1H18 13S"}="H181 13S"; $resn_exch{"2H18 13S"}="H182 13S";
$resn_exch{"3H18 13S"}="H183 13S"; $resn_exch{"1H20 13S"}="H201 13S"; $resn_exch{"1H22 13S"}="H221 13S";
$resn_exch{"1H12 13T"}="H121 13T"; $resn_exch{"2H12 13T"}="H122 13T"; $resn_exch{"1H13 13T"}="H131 13T";
$resn_exch{"2H13 13T"}="H132 13T"; $resn_exch{"1H23 13T"}="H231 13T"; $resn_exch{"2H23 13T"}="H232 13T";
$resn_exch{"3H23 13T"}="H233 13T"; $resn_exch{"1H24 13T"}="H241 13T"; $resn_exch{"2H24 13T"}="H242 13T";
$resn_exch{"3H24 13T"}="H243 13T"; $resn_exch{"1H25 13T"}="H251 13T"; $resn_exch{"2H25 13T"}="H252 13T";
$resn_exch{"3H25 13T"}="H253 13T"; $resn_exch{"1H26 13T"}="H261 13T"; $resn_exch{"2H26 13T"}="H262 13T";
$resn_exch{"3H26 13T"}="H263 13T"; $resn_exch{"1H27 13T"}="H271 13T"; $resn_exch{"2H27 13T"}="H272 13T";
$resn_exch{"3H27 13T"}="H273 13T"; $resn_exch{"1H28 13T"}="H281 13T"; $resn_exch{"2H28 13T"}="H282 13T";
$resn_exch{"3H28 13T"}="H283 13T"; $resn_exch{"1H29 13T"}="H291 13T"; $resn_exch{"2H29 13T"}="H292 13T";
$resn_exch{"1H30 13T"}="H301 13T"; $resn_exch{"2H30 13T"}="H302 13T"; $resn_exch{"3H30 13T"}="H303 13T";
$resn_exch{"1H31 13T"}="H311 13T"; $resn_exch{"2H31 13T"}="H312 13T"; $resn_exch{"3H31 13T"}="H313 13T";
$resn_exch{"1H32 13T"}="H321 13T"; $resn_exch{"2H32 13T"}="H322 13T"; $resn_exch{"3H32 13T"}="H323 13T";
$resn_exch{"1H2  140"}=" H21 140"; $resn_exch{"2H2  140"}=" H22 140"; $resn_exch{"1H4  140"}=" H41 140";
$resn_exch{"2H4  140"}=" H42 140"; $resn_exch{"1H5  140"}=" H51 140"; $resn_exch{"2H5  140"}=" H52 140";
$resn_exch{"1H6  140"}=" H61 140"; $resn_exch{"2H6  140"}=" H62 140"; $resn_exch{"1H7  140"}=" H71 140";
$resn_exch{"2H7  140"}=" H72 140"; $resn_exch{"1H8  140"}=" H81 140"; $resn_exch{"2H8  140"}=" H82 140";
$resn_exch{"1H9  140"}=" H91 140"; $resn_exch{"2H9  140"}=" H92 140"; $resn_exch{"1H10 140"}="H101 140";
$resn_exch{"2H10 140"}="H102 140"; $resn_exch{"1H11 140"}="H111 140"; $resn_exch{"2H11 140"}="H112 140";
$resn_exch{"1H12 140"}="H121 140"; $resn_exch{"2H12 140"}="H122 140"; $resn_exch{"1H13 140"}="H131 140";
$resn_exch{"2H13 140"}="H132 140"; $resn_exch{"1H14 140"}="H141 140"; $resn_exch{"2H14 140"}="H142 140";
$resn_exch{"1H15 140"}="H151 140"; $resn_exch{"2H15 140"}="H152 140"; $resn_exch{"1H16 140"}="H161 140";
$resn_exch{"2H16 140"}="H162 140"; $resn_exch{"1H17 140"}="H171 140"; $resn_exch{"2H17 140"}="H172 140";
$resn_exch{"1H18 140"}="H181 140"; $resn_exch{"2H18 140"}="H182 140"; $resn_exch{"3H18 140"}="H183 140";
$resn_exch{"1HN  142"}=" HN1 142"; $resn_exch{"2HN  142"}=" HN2 142"; $resn_exch{"1HB  142"}=" HB1 142";
$resn_exch{"2HB  142"}=" HB2 142"; $resn_exch{"1HB1 142"}="HB11 142"; $resn_exch{"2HB1 142"}="HB12 142";
$resn_exch{"3HB1 142"}="HB13 142"; $resn_exch{" HN2 143"}=" H2  143"; $resn_exch{"1HB  143"}=" HB2 143";
$resn_exch{"2HB  143"}=" HB3 143"; $resn_exch{"1HI  143"}=" HI1 143"; $resn_exch{"2HI  143"}=" HI2 143";
$resn_exch{"1H1  144"}=" H11 144"; $resn_exch{"2H1  144"}=" H12 144"; $resn_exch{"3H1  144"}=" H13 144";
$resn_exch{"1H2  144"}=" H21 144"; $resn_exch{"2H2  144"}=" H22 144"; $resn_exch{"1H3  144"}=" H31 144";
$resn_exch{"2H3  144"}=" H32 144"; $resn_exch{"1H4  144"}=" H41 144"; $resn_exch{"2H4  144"}=" H42 144";
$resn_exch{"1H6  145"}=" H61 145"; $resn_exch{"2H6  145"}=" H62 145"; $resn_exch{"1H2  146"}="H21A 146";
$resn_exch{"2H2  146"}=" H22 146"; $resn_exch{"1H7  146"}="H71A 146"; $resn_exch{"2H7  146"}=" H72 146";
$resn_exch{"1H31 146"}="H311 146"; $resn_exch{"2H31 146"}="H312 146"; $resn_exch{"1H61 146"}="H611 146";
$resn_exch{"2H61 146"}="H612 146"; $resn_exch{"7HN7 146"}="HN77 146"; $resn_exch{"1H6  147"}=" H61 147";
$resn_exch{"2H6  147"}=" H62 147"; $resn_exch{"1H2  148"}=" H21 148"; $resn_exch{"2H2  148"}=" H22 148";
$resn_exch{"1H3  148"}=" H31 148"; $resn_exch{"2H3  148"}=" H32 148"; $resn_exch{"1H4  148"}=" H41 148";
$resn_exch{"2H4  148"}=" H42 148"; $resn_exch{"1H5  148"}=" H51 148"; $resn_exch{"2H5  148"}=" H52 148";
$resn_exch{"1H6  148"}=" H61 148"; $resn_exch{"2H6  148"}=" H62 148"; $resn_exch{"1H7  148"}=" H71 148";
$resn_exch{"2H7  148"}=" H72 148"; $resn_exch{"1H8  148"}=" H81 148"; $resn_exch{"2H8  148"}=" H82 148";
$resn_exch{"1H6  149"}=" H61 149"; $resn_exch{"2H6  149"}=" H62 149"; $resn_exch{"1HN3 14A"}="HN31 14A";
$resn_exch{"2HN3 14A"}="HN32 14A"; $resn_exch{"1H6  14A"}=" H61 14A"; $resn_exch{"2H6  14A"}=" H62 14A";
$resn_exch{"1H7  14A"}=" H71 14A"; $resn_exch{"2H7  14A"}=" H72 14A"; $resn_exch{"1H8  14A"}=" H81 14A";
$resn_exch{"2H8  14A"}=" H82 14A"; $resn_exch{"1H20 14A"}="H201 14A"; $resn_exch{"2H20 14A"}="H202 14A";
$resn_exch{"1H22 14A"}="H221 14A"; $resn_exch{"2H22 14A"}="H222 14A"; $resn_exch{"1H24 14A"}="H241 14A";
$resn_exch{"2H24 14A"}="H242 14A"; $resn_exch{"1H25 14A"}="H251 14A"; $resn_exch{"2H25 14A"}="H252 14A";
$resn_exch{"1H26 14A"}="H261 14A"; $resn_exch{"2H26 14A"}="H262 14A"; $resn_exch{"1H27 14A"}="H271 14A";
$resn_exch{"2H27 14A"}="H272 14A"; $resn_exch{"1H7  14W"}=" H71 14W"; $resn_exch{"2H7  14W"}=" H72 14W";
$resn_exch{"1H10 14W"}="H101 14W"; $resn_exch{"2H10 14W"}="H102 14W"; $resn_exch{"3H10 14W"}="H103 14W";
$resn_exch{"1H12 14W"}="H121 14W"; $resn_exch{"2H12 14W"}="H122 14W"; $resn_exch{"1H13 14W"}="H131 14W";
$resn_exch{"2H13 14W"}="H132 14W"; $resn_exch{"1HN1 150"}="HN11 150"; $resn_exch{"2HN1 150"}="HN12 150";
$resn_exch{"1HN2 150"}="HN21 150"; $resn_exch{"2HN2 150"}="HN22 150"; $resn_exch{"1H7  150"}=" H71 150";
$resn_exch{"2H7  150"}=" H72 150"; $resn_exch{"3H7  150"}=" H73 150"; $resn_exch{"1H8  150"}=" H81 150";
$resn_exch{"2H8  150"}=" H82 150"; $resn_exch{"3H8  150"}=" H83 150"; $resn_exch{"1H26 151"}="H261 151";
$resn_exch{"2H26 151"}="H262 151"; $resn_exch{"1H28 151"}="H281 151"; $resn_exch{"2H28 151"}="H282 151";
$resn_exch{"1H3C 151"}="H3C1 151"; $resn_exch{"2H3C 151"}="H3C2 151"; $resn_exch{"3H3C 151"}="H3C3 151";
$resn_exch{"1H1C 151"}="H1C1 151"; $resn_exch{"2H1C 151"}="H1C2 151"; $resn_exch{"3H1C 151"}="H1C3 151";
$resn_exch{"1H17 151"}="H171 151"; $resn_exch{"2H17 151"}="H172 151"; $resn_exch{"3H17 151"}="H173 151";
$resn_exch{"1H9C 151"}="H9C1 151"; $resn_exch{"2H9C 151"}="H9C2 151"; $resn_exch{"1H11 151"}="H111 151";
$resn_exch{"2H11 151"}="H112 151"; $resn_exch{"1H13 151"}="H131 151"; $resn_exch{"2H13 151"}="H132 151";
$resn_exch{"1H14 151"}="H141 151"; $resn_exch{"2H14 151"}="H142 151"; $resn_exch{"1H12 151"}="H121 151";
$resn_exch{"2H12 151"}="H122 151"; $resn_exch{"1H2  152"}=" H21 152"; $resn_exch{"2H2  152"}=" H22 152";
$resn_exch{"1H4  152"}=" H41 152"; $resn_exch{"2H4  152"}=" H42 152"; $resn_exch{"1H5A 152"}="H5A1 152";
$resn_exch{"2H5A 152"}="H5A2 152"; $resn_exch{"3H5A 152"}="H5A3 152"; $resn_exch{"1H5B 152"}="H5B1 152";
$resn_exch{"2H5B 152"}="H5B2 152"; $resn_exch{"3H5B 152"}="H5B3 152"; $resn_exch{"1H5C 152"}="H5C1 152";
$resn_exch{"2H5C 152"}="H5C2 152"; $resn_exch{"3H5C 152"}="H5C3 152"; $resn_exch{"1H16 153"}="H161 153";
$resn_exch{"2H16 153"}="H162 153"; $resn_exch{"1H23 153"}="H231 153"; $resn_exch{"2H23 153"}="H232 153";
$resn_exch{"1H16 154"}="H161 154"; $resn_exch{"2H16 154"}="H162 154"; $resn_exch{"1H23 154"}="H231 154";
$resn_exch{"2H23 154"}="H232 154"; $resn_exch{"1H19 155"}="H191 155"; $resn_exch{"2H19 155"}="H192 155";
$resn_exch{"1H32 155"}="H321 155"; $resn_exch{"2H32 155"}="H322 155"; $resn_exch{"1H33 155"}="H331 155";
$resn_exch{"2H33 155"}="H332 155"; $resn_exch{"1H41 155"}="H411 155"; $resn_exch{"2H41 155"}="H412 155";
$resn_exch{"3H41 155"}="H413 155"; $resn_exch{"1H45 155"}="H451 155"; $resn_exch{"2H45 155"}="H452 155";
$resn_exch{"3H45 155"}="H453 155"; $resn_exch{"1H15 156"}="H151 156"; $resn_exch{"2H15 156"}="H152 156";
$resn_exch{"1H16 156"}="H161 156"; $resn_exch{"2H16 156"}="H162 156"; $resn_exch{"1H21 156"}="H211 156";
$resn_exch{"2H21 156"}="H212 156"; $resn_exch{"3H21 156"}="H213 156"; $resn_exch{"1H22 156"}="H221 156";
$resn_exch{"2H22 156"}="H222 156"; $resn_exch{"3H22 156"}="H223 156"; $resn_exch{"1H23 156"}="H231 156";
$resn_exch{"2H23 156"}="H232 156"; $resn_exch{"3H23 156"}="H233 156"; $resn_exch{"1H24 156"}="H241 156";
$resn_exch{"2H24 156"}="H242 156"; $resn_exch{"3H24 156"}="H243 156"; $resn_exch{"1H1  157"}=" H11 157";
$resn_exch{"2H1  157"}=" H12 157"; $resn_exch{"1H2  157"}=" H21 157"; $resn_exch{"2H2  157"}=" H22 157";
$resn_exch{"1H3  157"}=" H31 157"; $resn_exch{"2H3  157"}=" H32 157"; $resn_exch{"1H4  157"}=" H41 157";
$resn_exch{"2H4  157"}=" H42 157"; $resn_exch{"1HN6 157"}="HN61 157"; $resn_exch{"2HN6 157"}="HN62 157";
$resn_exch{"1H11 157"}="H111 157"; $resn_exch{"2H11 157"}="H112 157"; $resn_exch{"1H12 157"}="H121 157";
$resn_exch{"2H12 157"}="H122 157"; $resn_exch{"1H14 157"}="H141 157"; $resn_exch{"2H14 157"}="H142 157";
$resn_exch{"1H6  158"}=" H61 158"; $resn_exch{"2H6  158"}=" H62 158"; $resn_exch{"1H1  158"}=" H11 158";
$resn_exch{"2H1  158"}="H12A 158"; $resn_exch{"3H1  158"}=" H13 158"; $resn_exch{"1H9  158"}=" H91 158";
$resn_exch{"2H9  158"}=" H92 158"; $resn_exch{"1H5  159"}=" H51 159"; $resn_exch{"2H5  159"}=" H52 159";
$resn_exch{"1H4  159"}=" H41 159"; $resn_exch{"2H4  159"}=" H42 159"; $resn_exch{"1H6C 15A"}="H6C1 15A";
$resn_exch{"2H6C 15A"}="H6C2 15A"; $resn_exch{"1H7C 15A"}="H7C1 15A"; $resn_exch{"2H7C 15A"}="H7C2 15A";
$resn_exch{"1H8C 15A"}="H8C1 15A"; $resn_exch{"2H8C 15A"}="H8C2 15A"; $resn_exch{"1H2N 15A"}="H2N1 15A";
$resn_exch{"2H2N 15A"}="H2N2 15A"; $resn_exch{"1H6  15B"}=" H61 15B"; $resn_exch{"2H6  15B"}=" H62 15B";
$resn_exch{"1H19 15B"}="H191 15B"; $resn_exch{"2H19 15B"}="H192 15B"; $resn_exch{"1H20 15B"}="H201 15B";
$resn_exch{"2H20 15B"}="H202 15B"; $resn_exch{"1H21 15B"}="H211 15B"; $resn_exch{"2H21 15B"}="H212 15B";
$resn_exch{"1H23 15B"}="H231 15B"; $resn_exch{"2H23 15B"}="H232 15B"; $resn_exch{"1H24 15B"}="H241 15B";
$resn_exch{"2H24 15B"}="H242 15B"; $resn_exch{"1H26 15B"}="H261 15B"; $resn_exch{"2H26 15B"}="H262 15B";
$resn_exch{"1H27 15B"}="H271 15B"; $resn_exch{"2H27 15B"}="H272 15B"; $resn_exch{"1H28 15B"}="H281 15B";
$resn_exch{"2H28 15B"}="H282 15B"; $resn_exch{"1H35 15B"}="H351 15B"; $resn_exch{"2H35 15B"}="H352 15B";
$resn_exch{"1H36 15B"}="H361 15B"; $resn_exch{"2H36 15B"}="H362 15B"; $resn_exch{"1H37 15B"}="H371 15B";
$resn_exch{"2H37 15B"}="H372 15B"; $resn_exch{"1H2  15M"}="H21A 15M"; $resn_exch{"2H2  15M"}="H22A 15M";
$resn_exch{"1H3  15M"}=" H31 15M"; $resn_exch{"2H3  15M"}=" H32 15M"; $resn_exch{"1H4  15M"}=" H41 15M";
$resn_exch{"2H4  15M"}=" H42 15M"; $resn_exch{"1H7  15M"}=" H71 15M"; $resn_exch{"2H7  15M"}=" H72 15M";
$resn_exch{"1H10 15M"}="H101 15M"; $resn_exch{"2H10 15M"}="H102 15M"; $resn_exch{"1H16 15M"}="H161 15M";
$resn_exch{"2H16 15M"}="H162 15M"; $resn_exch{"1H17 15M"}="H171 15M"; $resn_exch{"2H17 15M"}="H172 15M";
$resn_exch{"1H24 15M"}="H241 15M"; $resn_exch{"2H24 15M"}="H242 15M"; $resn_exch{"3H24 15M"}="H243 15M";
$resn_exch{"1H25 15M"}="H251 15M"; $resn_exch{"2H25 15M"}="H252 15M"; $resn_exch{"1H10 15P"}="H101 15P";
$resn_exch{"2H10 15P"}="H102 15P"; $resn_exch{"1H11 15P"}="H111 15P"; $resn_exch{"2H11 15P"}="H112 15P";
$resn_exch{"1H12 15P"}="H121 15P"; $resn_exch{"2H12 15P"}="H122 15P"; $resn_exch{"1H13 15P"}="H131 15P";
$resn_exch{"2H13 15P"}="H132 15P"; $resn_exch{"1H14 15P"}="H141 15P"; $resn_exch{"2H14 15P"}="H142 15P";
$resn_exch{"1H15 15P"}="H151 15P"; $resn_exch{"2H15 15P"}="H152 15P"; $resn_exch{"1H16 15P"}="H161 15P";
$resn_exch{"2H16 15P"}="H162 15P"; $resn_exch{"1H17 15P"}="H171 15P"; $resn_exch{"2H17 15P"}="H172 15P";
$resn_exch{"1H18 15P"}="H181 15P"; $resn_exch{"2H18 15P"}="H182 15P"; $resn_exch{"1H19 15P"}="H191 15P";
$resn_exch{"2H19 15P"}="H192 15P"; $resn_exch{"1H20 15P"}="H201 15P"; $resn_exch{"2H20 15P"}="H202 15P";
$resn_exch{"1H21 15P"}="H211 15P"; $resn_exch{"2H21 15P"}="H212 15P"; $resn_exch{"1H22 15P"}="H221 15P";
$resn_exch{"2H22 15P"}="H222 15P"; $resn_exch{"1H23 15P"}="H231 15P"; $resn_exch{"2H23 15P"}="H232 15P";
$resn_exch{"1H24 15P"}="H241 15P"; $resn_exch{"2H24 15P"}="H242 15P"; $resn_exch{"1H25 15P"}="H251 15P";
$resn_exch{"2H25 15P"}="H252 15P"; $resn_exch{"1H26 15P"}="H261 15P"; $resn_exch{"2H26 15P"}="H262 15P";
$resn_exch{"1H27 15P"}="H271 15P"; $resn_exch{"2H27 15P"}="H272 15P"; $resn_exch{"1H28 15P"}="H281 15P";
$resn_exch{"2H28 15P"}="H282 15P"; $resn_exch{"1H29 15P"}="H291 15P"; $resn_exch{"2H29 15P"}="H292 15P";
$resn_exch{"1H30 15P"}="H301 15P"; $resn_exch{"2H30 15P"}="H302 15P"; $resn_exch{"1H31 15P"}="H311 15P";
$resn_exch{"2H31 15P"}="H312 15P"; $resn_exch{"1H32 15P"}="H321 15P"; $resn_exch{"2H32 15P"}="H322 15P";
$resn_exch{"1H33 15P"}="H331 15P"; $resn_exch{"2H33 15P"}="H332 15P"; $resn_exch{"1H34 15P"}="H341 15P";
$resn_exch{"2H34 15P"}="H342 15P"; $resn_exch{"1H35 15P"}="H351 15P"; $resn_exch{"2H35 15P"}="H352 15P";
$resn_exch{"1H36 15P"}="H361 15P"; $resn_exch{"2H36 15P"}="H362 15P"; $resn_exch{"1H37 15P"}="H371 15P";
$resn_exch{"2H37 15P"}="H372 15P"; $resn_exch{"1H38 15P"}="H381 15P"; $resn_exch{"2H38 15P"}="H382 15P";
$resn_exch{"1H39 15P"}="H391 15P"; $resn_exch{"2H39 15P"}="H392 15P"; $resn_exch{"1H40 15P"}="H401 15P";
$resn_exch{"2H40 15P"}="H402 15P"; $resn_exch{"1H41 15P"}="H411 15P"; $resn_exch{"2H41 15P"}="H412 15P";
$resn_exch{"1H42 15P"}="H421 15P"; $resn_exch{"2H42 15P"}="H422 15P"; $resn_exch{"1H43 15P"}="H431 15P";
$resn_exch{"2H43 15P"}="H432 15P"; $resn_exch{"1H44 15P"}="H441 15P"; $resn_exch{"2H44 15P"}="H442 15P";
$resn_exch{"1H45 15P"}="H451 15P"; $resn_exch{"2H45 15P"}="H452 15P"; $resn_exch{"1H46 15P"}="H461 15P";
$resn_exch{"2H46 15P"}="H462 15P"; $resn_exch{"1H47 15P"}="H471 15P"; $resn_exch{"2H47 15P"}="H472 15P";
$resn_exch{"1H48 15P"}="H481 15P"; $resn_exch{"2H48 15P"}="H482 15P"; $resn_exch{"1H49 15P"}="H491 15P";
$resn_exch{"2H49 15P"}="H492 15P"; $resn_exch{"1H50 15P"}="H501 15P"; $resn_exch{"2H50 15P"}="H502 15P";
$resn_exch{"1H51 15P"}="H511 15P"; $resn_exch{"2H51 15P"}="H512 15P"; $resn_exch{"1H52 15P"}="H521 15P";
$resn_exch{"2H52 15P"}="H522 15P"; $resn_exch{"1H53 15P"}="H531 15P"; $resn_exch{"2H53 15P"}="H532 15P";
$resn_exch{"1H54 15P"}="H541 15P"; $resn_exch{"2H54 15P"}="H542 15P"; $resn_exch{"1H55 15P"}="H551 15P";
$resn_exch{"2H55 15P"}="H552 15P"; $resn_exch{"1H56 15P"}="H561 15P"; $resn_exch{"2H56 15P"}="H562 15P";
$resn_exch{"1H57 15P"}="H571 15P"; $resn_exch{"2H57 15P"}="H572 15P"; $resn_exch{"1H58 15P"}="H581 15P";
$resn_exch{"2H58 15P"}="H582 15P"; $resn_exch{"1H59 15P"}="H591 15P"; $resn_exch{"2H59 15P"}="H592 15P";
$resn_exch{"1H60 15P"}="H601 15P"; $resn_exch{"2H60 15P"}="H602 15P"; $resn_exch{"1H61 15P"}="H611 15P";
$resn_exch{"2H61 15P"}="H612 15P"; $resn_exch{"1H62 15P"}="H621 15P"; $resn_exch{"2H62 15P"}="H622 15P";
$resn_exch{"1H63 15P"}="H631 15P"; $resn_exch{"2H63 15P"}="H632 15P"; $resn_exch{"1H64 15P"}="H641 15P";
$resn_exch{"2H64 15P"}="H642 15P"; $resn_exch{"1H65 15P"}="H651 15P"; $resn_exch{"2H65 15P"}="H652 15P";
$resn_exch{"1H66 15P"}="H661 15P"; $resn_exch{"2H66 15P"}="H662 15P"; $resn_exch{"1H67 15P"}="H671 15P";
$resn_exch{"2H67 15P"}="H672 15P"; $resn_exch{"1H68 15P"}="H681 15P"; $resn_exch{"2H68 15P"}="H682 15P";
$resn_exch{"1HCM 15P"}="HCM1 15P"; $resn_exch{"2HCM 15P"}="HCM2 15P"; $resn_exch{"3HCM 15P"}="HCM3 15P";
$resn_exch{"1H25 160"}="H251 160"; $resn_exch{"2H25 160"}="H252 160"; $resn_exch{"3H25 160"}="H253 160";
$resn_exch{"1H16 160"}="H161 160"; $resn_exch{"2H16 160"}="H162 160"; $resn_exch{"1H15 160"}="H151 160";
$resn_exch{"2H15 160"}="H152 160"; $resn_exch{"1H12 160"}="H121 160"; $resn_exch{"2H12 160"}="H122 160";
$resn_exch{"1H1  160"}="H11A 160"; $resn_exch{"2H1  160"}=" H12 160"; $resn_exch{"3H1  160"}=" H13 160";
$resn_exch{"1H29 160"}="H291 160"; $resn_exch{"2H29 160"}="H292 160"; $resn_exch{"1H1  161"}="H11A 161";
$resn_exch{"2H1  161"}=" H12 161"; $resn_exch{"3H1  161"}="H13A 161"; $resn_exch{"1H25 161"}="H251 161";
$resn_exch{"2H25 161"}="H252 161"; $resn_exch{"1H12 161"}="H121 161"; $resn_exch{"2H12 161"}="H122 161";
$resn_exch{"1H1  162"}="H11A 162"; $resn_exch{"2H1  162"}=" H12 162"; $resn_exch{"1H2  162"}=" H21 162";
$resn_exch{"2H2  162"}=" H22 162"; $resn_exch{"3H2  162"}=" H23 162"; $resn_exch{"1H6  162"}=" H61 162";
$resn_exch{"2H6  162"}=" H62 162"; $resn_exch{"1H14 162"}="H141 162"; $resn_exch{"2H14 162"}="H142 162";
$resn_exch{"1H16 162"}="H161 162"; $resn_exch{"2H16 162"}="H162 162"; $resn_exch{"1H17 162"}="H171 162";
$resn_exch{"2H17 162"}="H172 162"; $resn_exch{"1H18 162"}="H181 162"; $resn_exch{"2H18 162"}="H182 162";
$resn_exch{"1H19 162"}="H191 162"; $resn_exch{"2H19 162"}="H192 162"; $resn_exch{"1H20 162"}="H201 162";
$resn_exch{"2H20 162"}="H202 162"; $resn_exch{"1H21 162"}="H211 162"; $resn_exch{"2H21 162"}="H212 162";
$resn_exch{"1H1  163"}="H11A 163"; $resn_exch{"2H1  163"}=" H12 163"; $resn_exch{"1H2  163"}=" H21 163";
$resn_exch{"2H2  163"}=" H22 163"; $resn_exch{"3H2  163"}=" H23 163"; $resn_exch{"1H6  163"}=" H61 163";
$resn_exch{"2H6  163"}=" H62 163"; $resn_exch{"1H14 163"}="H141 163"; $resn_exch{"2H14 163"}="H142 163";
$resn_exch{"1H16 163"}="H161 163"; $resn_exch{"2H16 163"}="H162 163"; $resn_exch{"1H17 163"}="H171 163";
$resn_exch{"2H17 163"}="H172 163"; $resn_exch{"1H18 163"}="H181 163"; $resn_exch{"2H18 163"}="H182 163";
$resn_exch{"1H19 163"}="H191 163"; $resn_exch{"2H19 163"}="H192 163"; $resn_exch{"1H20 163"}="H201 163";
$resn_exch{"2H20 163"}="H202 163"; $resn_exch{"1H21 163"}="H211 163"; $resn_exch{"2H21 163"}="H212 163";
$resn_exch{"1H9  164"}=" H91 164"; $resn_exch{"2H9  164"}=" H92 164"; $resn_exch{"1H10 164"}="H101 164";
$resn_exch{"2H10 164"}="H102 164"; $resn_exch{"1H22 165"}="H221 165"; $resn_exch{"2H22 165"}="H222 165";
$resn_exch{"1H21 165"}="H211 165"; $resn_exch{"2H21 165"}="H212 165"; $resn_exch{"1H20 165"}="H201 165";
$resn_exch{"2H20 165"}="H202 165"; $resn_exch{"1H35 165"}="H351 165"; $resn_exch{"2H35 165"}="H352 165";
$resn_exch{"1H7  165"}=" H71 165"; $resn_exch{"2H7  165"}=" H72 165"; $resn_exch{"1H11 165"}="H111 165";
$resn_exch{"2H11 165"}="H112 165"; $resn_exch{"1H13 165"}="H131 165"; $resn_exch{"2H13 165"}="H132 165";
$resn_exch{"1H14 165"}="H141 165"; $resn_exch{"2H14 165"}="H142 165"; $resn_exch{"1H15 165"}="H151 165";
$resn_exch{"2H15 165"}="H152 165"; $resn_exch{"1H16 165"}="H161 165"; $resn_exch{"2H16 165"}="H162 165";
$resn_exch{"1H17 165"}="H171 165"; $resn_exch{"2H17 165"}="H172 165"; $resn_exch{"2H1  166"}=" H12 166";
$resn_exch{"1H1  166"}=" H11 166"; $resn_exch{"2H2  166"}=" H22 166"; $resn_exch{"1H2  166"}=" H21 166";
$resn_exch{"2H3  166"}=" H32 166"; $resn_exch{"1H3  166"}=" H31 166"; $resn_exch{"2H4  166"}=" H42 166";
$resn_exch{"1H4  166"}=" H41 166"; $resn_exch{"2H6  166"}=" H62 166"; $resn_exch{"1H6  166"}=" H61 166";
$resn_exch{"2H14 166"}="H142 166"; $resn_exch{"1H14 166"}="H141 166"; $resn_exch{"2H1  167"}=" H12 167";
$resn_exch{"1H1  167"}=" H11 167"; $resn_exch{"2H2  167"}=" H22 167"; $resn_exch{"1H2  167"}=" H21 167";
$resn_exch{"2H3  167"}=" H32 167"; $resn_exch{"1H3  167"}=" H31 167"; $resn_exch{"2H4  167"}=" H42 167";
$resn_exch{"1H4  167"}=" H41 167"; $resn_exch{"2H6  167"}=" H62 167"; $resn_exch{"1H6  167"}=" H61 167";
$resn_exch{"2H14 167"}="H142 167"; $resn_exch{"1H14 167"}="H141 167"; $resn_exch{"3H53 167"}="H533 167";
$resn_exch{"2H53 167"}="H532 167"; $resn_exch{"1H53 167"}="H531 167"; $resn_exch{"1H2  168"}=" H21 168";
$resn_exch{"2H2  168"}=" H22 168"; $resn_exch{"3H2  168"}=" H23 168"; $resn_exch{"1HP2 168"}="HP21 168";
$resn_exch{"2HP2 168"}="HP22 168"; $resn_exch{"1HP3 168"}="HP31 168"; $resn_exch{"2HP3 168"}="HP32 168";
$resn_exch{"1HP6 168"}="HP61 168"; $resn_exch{"2HP6 168"}="HP62 168"; $resn_exch{"1HP7 168"}="HP71 168";
$resn_exch{"2HP7 168"}="HP72 168"; $resn_exch{"0HPA 168"}="HPA0 168"; $resn_exch{"1HPD 168"}="HPD1 168";
$resn_exch{"2HPD 168"}="HPD2 168"; $resn_exch{"3HPD 168"}="HPD3 168"; $resn_exch{"1HPE 168"}="HPE1 168";
$resn_exch{"2HPE 168"}="HPE2 168"; $resn_exch{"3HPE 168"}="HPE3 168"; $resn_exch{"1HPC 168"}="HPC1 168";
$resn_exch{"2HPC 168"}="HPC2 168"; $resn_exch{"1HT3 168"}="HT31 168"; $resn_exch{"2HT3 168"}="HT32 168";
$resn_exch{"3HT3 168"}="HT33 168"; $resn_exch{"1HT4 168"}="HT41 168"; $resn_exch{"2HT4 168"}="HT42 168";
$resn_exch{"3HT4 168"}="HT43 168"; $resn_exch{"1HT5 168"}="HT51 168"; $resn_exch{"2HT5 168"}="HT52 168";
$resn_exch{"3HT5 168"}="HT53 168"; $resn_exch{"1H1  169"}="H11A 169"; $resn_exch{"2H1  169"}="H12A 169";
$resn_exch{"3H1  169"}=" H13 169"; $resn_exch{"1H3  169"}=" H31 169"; $resn_exch{"2H3  169"}=" H32 169";
$resn_exch{"3H3  169"}=" H33 169"; $resn_exch{"1H4  169"}=" H41 169"; $resn_exch{"2H4  169"}=" H42 169";
$resn_exch{"3H4  169"}=" H43 169"; $resn_exch{"1H5  169"}=" H51 169"; $resn_exch{"2H5  169"}=" H52 169";
$resn_exch{"1H6  169"}=" H61 169"; $resn_exch{"2H6  169"}=" H62 169"; $resn_exch{"1H7  169"}=" H71 169";
$resn_exch{"2H7  169"}=" H72 169"; $resn_exch{"1H8  169"}=" H81 169"; $resn_exch{"2H8  169"}=" H82 169";
$resn_exch{"1H13 169"}="H131 169"; $resn_exch{"2H13 169"}="H132 169"; $resn_exch{"1H14 169"}="H141 169";
$resn_exch{"2H14 169"}="H142 169"; $resn_exch{"1H15 169"}="H151 169"; $resn_exch{"2H15 169"}="H152 169";
$resn_exch{"1HN5 169"}="HN51 169"; $resn_exch{"5HN5 169"}="HN55 169"; $resn_exch{"1H1  16A"}=" H11 16A";
$resn_exch{"2H1  16A"}=" H12 16A"; $resn_exch{"3H1  16A"}=" H13 16A"; $resn_exch{"1H2  16A"}=" H21 16A";
$resn_exch{"2H2  16A"}=" H22 16A"; $resn_exch{"1H3  16A"}=" H31 16A"; $resn_exch{"2H3  16A"}=" H32 16A";
$resn_exch{"1H4  16A"}=" H41 16A"; $resn_exch{"2H4  16A"}=" H42 16A"; $resn_exch{"1H5  16A"}=" H51 16A";
$resn_exch{"2H5  16A"}=" H52 16A"; $resn_exch{"1H6  16A"}=" H61 16A"; $resn_exch{"2H6  16A"}=" H62 16A";
$resn_exch{"1H7  16A"}=" H71 16A"; $resn_exch{"2H7  16A"}=" H72 16A"; $resn_exch{"1H8  16A"}=" H81 16A";
$resn_exch{"2H8  16A"}=" H82 16A"; $resn_exch{"1H9  16A"}=" H91 16A"; $resn_exch{"2H9  16A"}=" H92 16A";
$resn_exch{"1H10 16A"}="H101 16A"; $resn_exch{"2H10 16A"}="H102 16A"; $resn_exch{"1H11 16A"}="H111 16A";
$resn_exch{"2H11 16A"}="H112 16A"; $resn_exch{"1H12 16A"}="H121 16A"; $resn_exch{"2H12 16A"}="H122 16A";
$resn_exch{"1H13 16A"}="H131 16A"; $resn_exch{"2H13 16A"}="H132 16A"; $resn_exch{"1H14 16A"}="H141 16A";
$resn_exch{"2H14 16A"}="H142 16A"; $resn_exch{"1H15 16A"}="H151 16A"; $resn_exch{"2H15 16A"}="H152 16A";
$resn_exch{"1H16 16A"}="H161 16A"; $resn_exch{"2H16 16A"}="H162 16A"; $resn_exch{"1H17 16A"}="H171 16A";
$resn_exch{"2H17 16A"}="H172 16A"; $resn_exch{"3H17 16A"}="H173 16A"; $resn_exch{"1H18 16A"}="H181 16A";
$resn_exch{"2H18 16A"}="H182 16A"; $resn_exch{"3H18 16A"}="H183 16A"; $resn_exch{"1H19 16A"}="H191 16A";
$resn_exch{"2H19 16A"}="H192 16A"; $resn_exch{"3H19 16A"}="H193 16A"; $resn_exch{"1H1  16C"}=" H11 16C";
$resn_exch{"2H1  16C"}=" H12 16C"; $resn_exch{"1H6  16C"}=" H61 16C"; $resn_exch{"2H6  16C"}=" H62 16C";
$resn_exch{"1H7  16C"}=" H71 16C"; $resn_exch{"2H7  16C"}=" H72 16C"; $resn_exch{"1H8  16C"}=" H81 16C";
$resn_exch{"2H8  16C"}=" H82 16C"; $resn_exch{"1H9  16C"}=" H91 16C"; $resn_exch{"2H9  16C"}=" H92 16C";
$resn_exch{"1H10 16C"}="H101 16C"; $resn_exch{"2H10 16C"}="H102 16C"; $resn_exch{"1H11 16C"}="H111 16C";
$resn_exch{"2H11 16C"}="H112 16C"; $resn_exch{"1H12 16C"}="H121 16C"; $resn_exch{"2H12 16C"}="H122 16C";
$resn_exch{"1H13 16C"}="H131 16C"; $resn_exch{"2H13 16C"}="H132 16C"; $resn_exch{"1H14 16C"}="H141 16C";
$resn_exch{"2H14 16C"}="H142 16C"; $resn_exch{"1H15 16C"}="H151 16C"; $resn_exch{"2H15 16C"}="H152 16C";
$resn_exch{"1H16 16C"}="H161 16C"; $resn_exch{"2H16 16C"}="H162 16C"; $resn_exch{"1H17 16C"}="H171 16C";
$resn_exch{"2H17 16C"}="H172 16C"; $resn_exch{"1H18 16C"}="H181 16C"; $resn_exch{"2H18 16C"}="H182 16C";
$resn_exch{"3H18 16C"}="H183 16C"; $resn_exch{"1H20 16C"}="H201 16C"; $resn_exch{"2H20 16C"}="H202 16C";
$resn_exch{"1H21 16C"}="H211 16C"; $resn_exch{"2H21 16C"}="H212 16C"; $resn_exch{"1H22 16C"}="H221 16C";
$resn_exch{"2H22 16C"}="H222 16C"; $resn_exch{"1H23 16C"}="H231 16C"; $resn_exch{"2H23 16C"}="H232 16C";
$resn_exch{"1H24 16C"}="H241 16C"; $resn_exch{"2H24 16C"}="H242 16C"; $resn_exch{"1H25 16C"}="H251 16C";
$resn_exch{"2H25 16C"}="H252 16C"; $resn_exch{"1H26 16C"}="H261 16C"; $resn_exch{"2H26 16C"}="H262 16C";
$resn_exch{"1H27 16C"}="H271 16C"; $resn_exch{"2H27 16C"}="H272 16C"; $resn_exch{"1H28 16C"}="H281 16C";
$resn_exch{"2H28 16C"}="H282 16C"; $resn_exch{"1H29 16C"}="H291 16C"; $resn_exch{"2H29 16C"}="H292 16C";
$resn_exch{"1H30 16C"}="H301 16C"; $resn_exch{"2H30 16C"}="H302 16C"; $resn_exch{"1H31 16C"}="H311 16C";
$resn_exch{"2H31 16C"}="H312 16C"; $resn_exch{"1H32 16C"}="H321 16C"; $resn_exch{"2H32 16C"}="H322 16C";
$resn_exch{"1H33 16C"}="H331 16C"; $resn_exch{"2H33 16C"}="H332 16C"; $resn_exch{"1H34 16C"}="H341 16C";
$resn_exch{"2H34 16C"}="H342 16C"; $resn_exch{"3H34 16C"}="H343 16C"; $resn_exch{"1HN1 16D"}="HN11 16D";
$resn_exch{"2HN1 16D"}="HN12 16D"; $resn_exch{"1H1  16D"}=" H11 16D"; $resn_exch{"2H1  16D"}=" H12 16D";
$resn_exch{"1H2  16D"}=" H21 16D"; $resn_exch{"2H2  16D"}=" H22 16D"; $resn_exch{"1H3  16D"}=" H31 16D";
$resn_exch{"2H3  16D"}=" H32 16D"; $resn_exch{"1H4  16D"}=" H41 16D"; $resn_exch{"2H4  16D"}=" H42 16D";
$resn_exch{"1H5  16D"}=" H51 16D"; $resn_exch{"2H5  16D"}=" H52 16D"; $resn_exch{"1H6  16D"}=" H61 16D";
$resn_exch{"2H6  16D"}=" H62 16D"; $resn_exch{"1HN2 16D"}="HN21 16D"; $resn_exch{"2HN2 16D"}="HN22 16D";
$resn_exch{"1H6  16G"}=" H61 16G"; $resn_exch{"2H6  16G"}=" H62 16G"; $resn_exch{"1H8  16G"}=" H81 16G";
$resn_exch{"2H8  16G"}=" H82 16G"; $resn_exch{"3H8  16G"}=" H83 16G"; $resn_exch{"2HOP 16G"}="HOP2 16G";
$resn_exch{"3HOP 16G"}="HOP3 16G"; $resn_exch{"1H1  16P"}=" H11 16P"; $resn_exch{"2H1  16P"}=" H12 16P";
$resn_exch{"3H1  16P"}=" H13 16P"; $resn_exch{"1H2  16P"}=" H21 16P"; $resn_exch{"2H2  16P"}=" H22 16P";
$resn_exch{"1H3  16P"}=" H31 16P"; $resn_exch{"2H3  16P"}=" H32 16P"; $resn_exch{"1H4  16P"}=" H41 16P";
$resn_exch{"2H4  16P"}=" H42 16P"; $resn_exch{"1H5  16P"}=" H51 16P"; $resn_exch{"2H5  16P"}=" H52 16P";
$resn_exch{"1H6  16P"}=" H61 16P"; $resn_exch{"2H6  16P"}=" H62 16P"; $resn_exch{"1H7  16P"}=" H71 16P";
$resn_exch{"2H7  16P"}=" H72 16P"; $resn_exch{"1H8  16P"}=" H81 16P"; $resn_exch{"2H8  16P"}=" H82 16P";
$resn_exch{"1H9  16P"}=" H91 16P"; $resn_exch{"2H9  16P"}=" H92 16P"; $resn_exch{"1H10 16P"}="H101 16P";
$resn_exch{"2H10 16P"}="H102 16P"; $resn_exch{"1H11 16P"}="H111 16P"; $resn_exch{"2H11 16P"}="H112 16P";
$resn_exch{"1H12 16P"}="H121 16P"; $resn_exch{"2H12 16P"}="H122 16P"; $resn_exch{"1H13 16P"}="H131 16P";
$resn_exch{"2H13 16P"}="H132 16P"; $resn_exch{"1H14 16P"}="H141 16P"; $resn_exch{"2H14 16P"}="H142 16P";
$resn_exch{"3H14 16P"}="H143 16P"; $resn_exch{"1H1  170"}=" H11 170"; $resn_exch{"2H1  170"}=" H12 170";
$resn_exch{"1H33 170"}="H331 170"; $resn_exch{"2H33 170"}="H332 170"; $resn_exch{"1H39 170"}="H391 170";
$resn_exch{"2H39 170"}="H392 170"; $resn_exch{"1H1  171"}=" H11 171"; $resn_exch{"2H1  171"}=" H12 171";
$resn_exch{"1H2  171"}=" H21 171"; $resn_exch{"2H2  171"}=" H22 171"; $resn_exch{"1H14 172"}="H141 172";
$resn_exch{"2H14 172"}="H142 172"; $resn_exch{" HN2 175"}=" H2  175"; $resn_exch{" H2  175"}=" HC2 175";
$resn_exch{"1HB  175"}=" HB1 175"; $resn_exch{"2HB  175"}=" HB2 175"; $resn_exch{"3HB  175"}=" HB3 175";
$resn_exch{"2HN  175"}="HN2A 175"; $resn_exch{"1H4  175"}=" H41 175"; $resn_exch{"2H4  175"}=" H42 175";
$resn_exch{"1HA  175"}=" HA1 175"; $resn_exch{"2HA  175"}=" HA2 175"; $resn_exch{"1HN5 175"}="HN51 175";
$resn_exch{"2HN5 175"}="HN52 175"; $resn_exch{"1HN0 176"}="HN01 176"; $resn_exch{"2HN0 176"}="HN02 176";
$resn_exch{"1HD  176"}="HD1A 176"; $resn_exch{"2HD  176"}="HD2A 176"; $resn_exch{"1HB  176"}=" HB1 176";
$resn_exch{"2HB  176"}=" HB2 176"; $resn_exch{"1HG  176"}=" HG1 176"; $resn_exch{"2HG  176"}=" HG2 176";
$resn_exch{"1HA1 176"}="HA11 176"; $resn_exch{"2HA1 176"}="HA12 176"; $resn_exch{" HG* 176"}=" HG' 176";
$resn_exch{"1HG1 177"}="HG11 177"; $resn_exch{"2HG1 177"}="HG12 177"; $resn_exch{"1HG2 177"}="HG21 177";
$resn_exch{"2HG2 177"}="HG22 177"; $resn_exch{"1HD1 177"}="HD11 177"; $resn_exch{"2HD1 177"}="HD12 177";
$resn_exch{"1HD2 177"}="HD21 177"; $resn_exch{"2HD2 177"}="HD22 177"; $resn_exch{"1HE0 177"}="HE01 177";
$resn_exch{"2HE0 177"}="HE02 177"; $resn_exch{"1H0  177"}=" H01 177"; $resn_exch{"2H0  177"}=" H02 177";
$resn_exch{"1HD  177"}=" HD1 177"; $resn_exch{"2HD  177"}=" HD2 177"; $resn_exch{"1HB  177"}=" HB1 177";
$resn_exch{"2HB  177"}=" HB2 177"; $resn_exch{"1HG  177"}=" HG1 177"; $resn_exch{"2HG  177"}=" HG2 177";
$resn_exch{"1HA' 177"}="HA'1 177"; $resn_exch{"2HA' 177"}="HA'2 177"; $resn_exch{" HG* 177"}=" HG' 177";
$resn_exch{"1HF  177"}=" HF1 177"; $resn_exch{"2HF  177"}=" HF2 177"; $resn_exch{"1HM  177"}=" HM1 177";
$resn_exch{"2HM  177"}=" HM2 177"; $resn_exch{"1HN  177"}=" HN1 177"; $resn_exch{"2HN  177"}="HN2A 177";
$resn_exch{"3HN  177"}=" HN3 177"; $resn_exch{"1H2B 178"}="H2B1 178"; $resn_exch{"2H2B 178"}="H2B2 178";
$resn_exch{"3H2B 178"}="H2B3 178"; $resn_exch{"1H2G 178"}="H2G1 178"; $resn_exch{"2H2G 178"}="H2G2 178";
$resn_exch{"3H2G 178"}="H2G3 178"; $resn_exch{"1H2Y 178"}="H2Y1 178"; $resn_exch{"2H2Y 178"}="H2Y2 178";
$resn_exch{"3H2Y 178"}="H2Y3 178"; $resn_exch{"1H2Z 178"}="H2Z1 178"; $resn_exch{"2H2Z 178"}="H2Z2 178";
$resn_exch{"3H2Z 178"}="H2Z3 178"; $resn_exch{"1H30 17A"}="H301 17A"; $resn_exch{"2H30 17A"}="H302 17A";
$resn_exch{"3H30 17A"}="H303 17A"; $resn_exch{"1H29 17A"}="H291 17A"; $resn_exch{"2H29 17A"}="H292 17A";
$resn_exch{"3H29 17A"}="H293 17A"; $resn_exch{"1H15 17A"}="H151 17A"; $resn_exch{"2H15 17A"}="H152 17A";
$resn_exch{"1H9  17A"}=" H91 17A"; $resn_exch{"2H9  17A"}=" H92 17A"; $resn_exch{"1H10 17A"}="H101 17A";
$resn_exch{"2H10 17A"}="H102 17A"; $resn_exch{"1H8C 17B"}="H8C1 17B"; $resn_exch{"2H8C 17B"}="H8C2 17B";
$resn_exch{"1H7C 17B"}="H7C1 17B"; $resn_exch{"2H7C 17B"}="H7C2 17B"; $resn_exch{"1H9C 17B"}="H9C1 17B";
$resn_exch{"2H9C 17B"}="H9C2 17B"; $resn_exch{"1H6C 17B"}="H6C1 17B"; $resn_exch{"2H6C 17B"}="H6C2 17B";
$resn_exch{"1H1N 17B"}="H1N1 17B"; $resn_exch{"2H1N 17B"}="H1N2 17B"; $resn_exch{"1H10 17B"}="H101 17B";
$resn_exch{"2H10 17B"}="H102 17B"; $resn_exch{"3H1N 17B"}="H1N3 17B"; $resn_exch{"1H2  17H"}=" H21 17H";
$resn_exch{"2H2  17H"}=" H22 17H"; $resn_exch{"1H7  17H"}=" H71 17H"; $resn_exch{"2H7  17H"}=" H72 17H";
$resn_exch{"1H6  17H"}=" H61 17H"; $resn_exch{"2H6  17H"}=" H62 17H"; $resn_exch{"1H1  17H"}="H11A 17H";
$resn_exch{"2H1  17H"}="H12A 17H"; $resn_exch{"1H15 17H"}="H151 17H"; $resn_exch{"2H15 17H"}="H152 17H";
$resn_exch{"1H16 17H"}="H161 17H"; $resn_exch{"2H16 17H"}="H162 17H"; $resn_exch{"1H18 17H"}="H181 17H";
$resn_exch{"2H18 17H"}="H182 17H"; $resn_exch{"1H22 17H"}="H221 17H"; $resn_exch{"2H22 17H"}="H222 17H";
$resn_exch{"3H22 17H"}="H223 17H"; $resn_exch{"1H21 17H"}="H211 17H"; $resn_exch{"2H21 17H"}="H212 17H";
$resn_exch{"3H21 17H"}="H213 17H"; $resn_exch{"1H20 17H"}="H201 17H"; $resn_exch{"2H20 17H"}="H202 17H";
$resn_exch{"1HAR 17M"}="HAR1 17M"; $resn_exch{"2HAR 17M"}="HAR2 17M"; $resn_exch{"3HAR 17M"}="HAR3 17M";
$resn_exch{"1HAP 17M"}="HAP1 17M"; $resn_exch{"2HAP 17M"}="HAP2 17M"; $resn_exch{"1HAJ 17M"}="HAJ1 17M";
$resn_exch{"2HAJ 17M"}="HAJ2 17M"; $resn_exch{"1HAQ 17M"}="HAQ1 17M"; $resn_exch{"2HAQ 17M"}="HAQ2 17M";
$resn_exch{"1HAC 17M"}="HAC1 17M"; $resn_exch{"2HAC 17M"}="HAC2 17M"; $resn_exch{"1HAS 17M"}="HAS1 17M";
$resn_exch{"2HAS 17M"}="HAS2 17M"; $resn_exch{"3HAS 17M"}="HAS3 17M"; $resn_exch{"1H3  180"}=" H31 180";
$resn_exch{"2H3  180"}=" H32 180"; $resn_exch{"1H10 180"}="H101 180"; $resn_exch{"2H10 180"}="H102 180";
$resn_exch{"1H11 180"}="H111 180"; $resn_exch{"2H11 180"}="H112 180"; $resn_exch{"1H13 180"}="H131 180";
$resn_exch{"2H13 180"}="H132 180"; $resn_exch{"1H14 180"}="H141 180"; $resn_exch{"2H14 180"}="H142 180";
$resn_exch{"1H15 180"}="H151 180"; $resn_exch{"2H15 180"}="H152 180"; $resn_exch{"1H16 180"}="H161 180";
$resn_exch{"2H16 180"}="H162 180"; $resn_exch{"1HAJ 181"}="HAJ1 181"; $resn_exch{"2HAJ 181"}="HAJ2 181";
$resn_exch{"1HAK 181"}="HAK1 181"; $resn_exch{"2HAK 181"}="HAK2 181"; $resn_exch{"1HAL 181"}="HAL1 181";
$resn_exch{"2HAL 181"}="HAL2 181"; $resn_exch{"1HAJ 182"}="HAJ1 182"; $resn_exch{"2HAJ 182"}="HAJ2 182";
$resn_exch{"1HAK 182"}="HAK1 182"; $resn_exch{"2HAK 182"}="HAK2 182"; $resn_exch{"1HAL 182"}="HAL1 182";
$resn_exch{"2HAL 182"}="HAL2 182"; $resn_exch{"1HAN 183"}="HAN1 183"; $resn_exch{"2HAN 183"}="HAN2 183";
$resn_exch{"1HAO 183"}="HAO1 183"; $resn_exch{"2HAO 183"}="HAO2 183"; $resn_exch{"1HAR 183"}="HAR1 183";
$resn_exch{"2HAR 183"}="HAR2 183"; $resn_exch{"1HAQ 183"}="HAQ1 183"; $resn_exch{"2HAQ 183"}="HAQ2 183";
$resn_exch{"1HAP 183"}="HAP1 183"; $resn_exch{"2HAP 183"}="HAP2 183"; $resn_exch{"1H17 184"}="H171 184";
$resn_exch{"2H17 184"}="H172 184"; $resn_exch{"1H18 184"}="H181 184"; $resn_exch{"2H18 184"}="H182 184";
$resn_exch{"1H23 184"}="H231 184"; $resn_exch{"2H23 184"}="H232 184"; $resn_exch{"3H23 184"}="H233 184";
$resn_exch{"1H24 184"}="H241 184"; $resn_exch{"2H24 184"}="H242 184"; $resn_exch{"3H24 184"}="H243 184";
$resn_exch{"1H25 184"}="H251 184"; $resn_exch{"2H25 184"}="H252 184"; $resn_exch{"3H25 184"}="H253 184";
$resn_exch{"1H26 184"}="H261 184"; $resn_exch{"2H26 184"}="H262 184"; $resn_exch{"3H26 184"}="H263 184";
$resn_exch{"1H2' 185"}="H2'1 185"; $resn_exch{"2H2' 185"}="H2'2 185"; $resn_exch{"3H2' 185"}="H2'3 185";
$resn_exch{"1H6' 185"}="H6'1 185"; $resn_exch{"2H6' 185"}="H6'2 185"; $resn_exch{"3H6' 185"}="H6'3 185";
$resn_exch{"1H18 185"}="H181 185"; $resn_exch{"2H18 185"}="H182 185"; $resn_exch{"1H27 186"}="H271 186";
$resn_exch{"2H27 186"}="H272 186"; $resn_exch{"1H16 186"}="H161 186"; $resn_exch{"2H16 186"}="H162 186";
$resn_exch{"1H14 186"}="H141 186"; $resn_exch{"2H14 186"}="H142 186"; $resn_exch{"3H14 186"}="H143 186";
$resn_exch{"1HN4 187"}="HN41 187"; $resn_exch{"2HN4 187"}="HN42 187"; $resn_exch{"1H13 188"}="H131 188";
$resn_exch{"2H13 188"}="H132 188"; $resn_exch{"1H16 188"}="H161 188"; $resn_exch{"2H16 188"}="H162 188";
$resn_exch{"1H18 188"}="H181 188"; $resn_exch{"2H18 188"}="H182 188"; $resn_exch{"3H18 188"}="H183 188";
$resn_exch{"1H20 189"}="H201 189"; $resn_exch{"2H20 189"}="H202 189"; $resn_exch{"1H26 189"}="H261 189";
$resn_exch{"2H26 189"}="H262 189"; $resn_exch{"1H36 189"}="H361 189"; $resn_exch{"2H36 189"}="H362 189";
$resn_exch{"1H45 189"}="H451 189"; $resn_exch{"2H45 189"}="H452 189"; $resn_exch{"3H45 189"}="H453 189";
$resn_exch{"1H44 189"}="H441 189"; $resn_exch{"2H44 189"}="H442 189"; $resn_exch{"3H44 189"}="H443 189";
$resn_exch{"1H24 189"}="H241 189"; $resn_exch{"2H24 189"}="H242 189"; $resn_exch{"1H22 189"}="H221 189";
$resn_exch{"2H22 189"}="H222 189"; $resn_exch{"1H16 189"}="H161 189"; $resn_exch{"2H16 189"}="H162 189";
$resn_exch{"3H16 189"}="H163 189"; $resn_exch{"1H19 189"}="H191 189"; $resn_exch{"2H19 189"}="H192 189";
$resn_exch{"1H18 189"}="H181 189"; $resn_exch{"2H18 189"}="H182 189"; $resn_exch{"3H18 189"}="H183 189";
$resn_exch{"1H28 189"}="H281 189"; $resn_exch{"2H28 189"}="H282 189"; $resn_exch{"1H36 18C"}="H361 18C";
$resn_exch{"2H36 18C"}="H362 18C"; $resn_exch{"3H36 18C"}="H363 18C"; $resn_exch{"1H35 18C"}="H351 18C";
$resn_exch{"2H35 18C"}="H352 18C"; $resn_exch{"1H34 18C"}="H341 18C"; $resn_exch{"2H34 18C"}="H342 18C";
$resn_exch{"1H33 18C"}="H331 18C"; $resn_exch{"2H33 18C"}="H332 18C"; $resn_exch{"1H32 18C"}="H321 18C";
$resn_exch{"2H32 18C"}="H322 18C"; $resn_exch{"1H31 18C"}="H311 18C"; $resn_exch{"2H31 18C"}="H312 18C";
$resn_exch{"1H30 18C"}="H301 18C"; $resn_exch{"2H30 18C"}="H302 18C"; $resn_exch{"1H29 18C"}="H291 18C";
$resn_exch{"2H29 18C"}="H292 18C"; $resn_exch{"1H28 18C"}="H281 18C"; $resn_exch{"2H28 18C"}="H282 18C";
$resn_exch{"1H27 18C"}="H271 18C"; $resn_exch{"2H27 18C"}="H272 18C"; $resn_exch{"1H26 18C"}="H261 18C";
$resn_exch{"2H26 18C"}="H262 18C"; $resn_exch{"1H25 18C"}="H251 18C"; $resn_exch{"2H25 18C"}="H252 18C";
$resn_exch{"1H24 18C"}="H241 18C"; $resn_exch{"2H24 18C"}="H242 18C"; $resn_exch{"1H23 18C"}="H231 18C";
$resn_exch{"2H23 18C"}="H232 18C"; $resn_exch{"1H22 18C"}="H221 18C"; $resn_exch{"2H22 18C"}="H222 18C";
$resn_exch{"1H21 18C"}="H211 18C"; $resn_exch{"2H21 18C"}="H212 18C"; $resn_exch{"1H20 18C"}="H201 18C";
$resn_exch{"2H20 18C"}="H202 18C"; $resn_exch{"1H1  18C"}=" H11 18C"; $resn_exch{"2H1  18C"}=" H12 18C";
$resn_exch{"1H6  18C"}=" H61 18C"; $resn_exch{"2H6  18C"}=" H62 18C"; $resn_exch{"1H7  18C"}=" H71 18C";
$resn_exch{"2H7  18C"}=" H72 18C"; $resn_exch{"1H8  18C"}=" H81 18C"; $resn_exch{"2H8  18C"}=" H82 18C";
$resn_exch{"1H9  18C"}=" H91 18C"; $resn_exch{"2H9  18C"}=" H92 18C"; $resn_exch{"1H10 18C"}="H101 18C";
$resn_exch{"2H10 18C"}="H102 18C"; $resn_exch{"1H11 18C"}="H111 18C"; $resn_exch{"2H11 18C"}="H112 18C";
$resn_exch{"1H12 18C"}="H121 18C"; $resn_exch{"2H12 18C"}="H122 18C"; $resn_exch{"1H13 18C"}="H131 18C";
$resn_exch{"2H13 18C"}="H132 18C"; $resn_exch{"1H14 18C"}="H141 18C"; $resn_exch{"2H14 18C"}="H142 18C";
$resn_exch{"1H15 18C"}="H151 18C"; $resn_exch{"2H15 18C"}="H152 18C"; $resn_exch{"1H16 18C"}="H161 18C";
$resn_exch{"2H16 18C"}="H162 18C"; $resn_exch{"1H17 18C"}="H171 18C"; $resn_exch{"2H17 18C"}="H172 18C";
$resn_exch{"1H18 18C"}="H181 18C"; $resn_exch{"2H18 18C"}="H182 18C"; $resn_exch{"3H18 18C"}="H183 18C";
$resn_exch{"1H8  190"}=" H81 190"; $resn_exch{"2H8  190"}=" H82 190"; $resn_exch{"3H8  190"}=" H83 190";
$resn_exch{"1H7  190"}=" H71 190"; $resn_exch{"2H7  190"}=" H72 190"; $resn_exch{"3H7  190"}=" H73 190";
$resn_exch{"1H10 190"}="H101 190"; $resn_exch{"2H10 190"}="H102 190"; $resn_exch{"1H16 190"}="H161 190";
$resn_exch{"2H16 190"}="H162 190"; $resn_exch{"1H24 190"}="H241 190"; $resn_exch{"2H24 190"}="H242 190";
$resn_exch{"1H26 190"}="H261 190"; $resn_exch{"2H26 190"}="H262 190"; $resn_exch{"1H28 190"}="H281 190";
$resn_exch{"2H28 190"}="H282 190"; $resn_exch{"3H28 190"}="H283 190"; $resn_exch{"1H29 190"}="H291 190";
$resn_exch{"2H29 190"}="H292 190"; $resn_exch{"3H29 190"}="H293 190"; $resn_exch{"1H3  191"}=" H31 191";
$resn_exch{"2H3  191"}="H32A 191"; $resn_exch{"3H3  191"}="H33A 191"; $resn_exch{"1H2  191"}=" H21 191";
$resn_exch{"2H2  191"}="H22A 191"; $resn_exch{"1H5  191"}=" H51 191"; $resn_exch{"2H5  191"}=" H52 191";
$resn_exch{"1H6  191"}=" H61 191"; $resn_exch{"2H6  191"}=" H62 191"; $resn_exch{"1H9  191"}=" H91 191";
$resn_exch{"2H9  191"}=" H92 191"; $resn_exch{"1H10 191"}="H101 191"; $resn_exch{"2H10 191"}="H102 191";
$resn_exch{"1H17 191"}="H171 191"; $resn_exch{"2H17 191"}="H172 191"; $resn_exch{"3H17 191"}="H173 191";
$resn_exch{"1H18 191"}="H181 191"; $resn_exch{"2H18 191"}="H182 191"; $resn_exch{"3H18 191"}="H183 191";
$resn_exch{"1H19 191"}="H191 191"; $resn_exch{"2H19 191"}="H192 191"; $resn_exch{"1H5* 191"}="H5'1 191";
$resn_exch{"2H5* 191"}="H5'2 191"; $resn_exch{" H4* 191"}=" H4' 191"; $resn_exch{" H3* 191"}=" H3' 191";
$resn_exch{" H2* 191"}=" H2' 191"; $resn_exch{"2HO* 191"}="HO'2 191"; $resn_exch{" H1* 191"}=" H1' 191";
$resn_exch{"1H41 191"}="H411 191"; $resn_exch{"2H41 191"}="H412 191"; $resn_exch{"1HB1 192"}="HB11 192";
$resn_exch{"2HB1 192"}="HB12 192"; $resn_exch{"1HG  192"}=" HG1 192"; $resn_exch{"2HG  192"}=" HG2 192";
$resn_exch{"1HB2 192"}="HB21 192"; $resn_exch{"2HB2 192"}="HB22 192"; $resn_exch{"2HB  193"}=" HB2 193";
$resn_exch{"1HB  193"}=" HB1 193"; $resn_exch{"2HG  193"}=" HG2 193"; $resn_exch{"1HG  193"}=" HG1 193";
$resn_exch{"2HZ  193"}=" HZ2 193"; $resn_exch{"1HZ  193"}=" HZ1 193"; $resn_exch{"2HT  193"}=" HT2 193";
$resn_exch{"1HT  193"}=" HT1 193"; $resn_exch{"1H7  197"}="H71A 197"; $resn_exch{"2H7  197"}=" H72 197";
$resn_exch{"1H13 197"}="H131 197"; $resn_exch{"2H13 197"}="H132 197"; $resn_exch{"3H13 197"}="H133 197";
$resn_exch{"1H17 197"}="H171 197"; $resn_exch{"2H17 197"}="H172 197"; $resn_exch{"1H18 197"}="H181 197";
$resn_exch{"2H18 197"}="H182 197"; $resn_exch{"1H19 197"}="H191 197"; $resn_exch{"2H19 197"}="H192 197";
$resn_exch{"1H20 197"}="H201 197"; $resn_exch{"2H20 197"}="H202 197"; $resn_exch{"1H23 197"}="H231 197";
$resn_exch{"2H23 197"}="H232 197"; $resn_exch{"1H13 198"}="H131 198"; $resn_exch{"2H13 198"}="H132 198";
$resn_exch{"1H12 198"}="H121 198"; $resn_exch{"2H12 198"}="H122 198"; $resn_exch{"3H12 198"}="H123 198";
$resn_exch{"1H25 199"}="H251 199"; $resn_exch{"2H25 199"}="H252 199"; $resn_exch{"1H26 199"}="H261 199";
$resn_exch{"2H26 199"}="H262 199"; $resn_exch{"1H28 199"}="H281 199"; $resn_exch{"2H28 199"}="H282 199";
$resn_exch{"1H29 199"}="H291 199"; $resn_exch{"2H29 199"}="H292 199"; $resn_exch{"1H31 199"}="H311 199";
$resn_exch{"2H31 199"}="H312 199"; $resn_exch{"3H31 199"}="H313 199"; $resn_exch{"1H1  19A"}=" H11 19A";
$resn_exch{"2H1  19A"}=" H12 19A"; $resn_exch{"3H1  19A"}=" H13 19A"; $resn_exch{"1H3  19A"}=" H31 19A";
$resn_exch{"2H3  19A"}=" H32 19A"; $resn_exch{"3H3  19A"}=" H33 19A"; $resn_exch{"1HN3 1AA"}="HN31 1AA";
$resn_exch{"2HN3 1AA"}="HN32 1AA"; $resn_exch{" H8* 1AA"}=" H8' 1AA"; $resn_exch{" H7* 1AA"}=" H7' 1AA";
$resn_exch{" H4* 1AA"}=" H4' 1AA"; $resn_exch{"1H6  1AA"}=" H61 1AA"; $resn_exch{"2H6  1AA"}=" H62 1AA";
$resn_exch{"1H7  1AA"}=" H71 1AA"; $resn_exch{"2H7  1AA"}=" H72 1AA"; $resn_exch{"1H8  1AA"}=" H81 1AA";
$resn_exch{"2H8  1AA"}=" H82 1AA"; $resn_exch{" H2* 1AA"}=" H2' 1AA"; $resn_exch{" H3* 1AA"}=" H3' 1AA";
$resn_exch{" H5* 1AA"}=" H5' 1AA"; $resn_exch{"1HN1 1AA"}="HN11 1AA"; $resn_exch{"2HN1 1AA"}="HN12 1AA";
$resn_exch{"1H2  1AB"}=" H21 1AB"; $resn_exch{"2H2  1AB"}=" H22 1AB"; $resn_exch{"1H6  1AB"}=" H61 1AB";
$resn_exch{"2H6  1AB"}=" H62 1AB"; $resn_exch{"1H3  1AC"}=" H31 1AC"; $resn_exch{"2H3  1AC"}=" H32 1AC";
$resn_exch{"1H4  1AC"}=" H41 1AC"; $resn_exch{"2H4  1AC"}=" H42 1AC"; $resn_exch{"1H1  1AC"}=" H11 1AC";
$resn_exch{"2H1  1AC"}=" H12 1AC"; $resn_exch{"1H5  1AD"}=" H51 1AD"; $resn_exch{"2H5  1AD"}=" H52 1AD";
$resn_exch{"1H7  1AD"}=" H71 1AD"; $resn_exch{"2H7  1AD"}=" H72 1AD"; $resn_exch{"1H8  1AD"}=" H81 1AD";
$resn_exch{"2H8  1AD"}=" H82 1AD"; $resn_exch{"1H12 1AD"}="H121 1AD"; $resn_exch{"2H12 1AD"}="H122 1AD";
$resn_exch{"1H14 1AD"}="H141 1AD"; $resn_exch{"2H14 1AD"}="H142 1AD"; $resn_exch{"1H15 1AD"}="H151 1AD";
$resn_exch{"2H15 1AD"}="H152 1AD"; $resn_exch{"3H15 1AD"}="H153 1AD"; $resn_exch{"1H16 1AD"}="H161 1AD";
$resn_exch{"2H16 1AD"}="H162 1AD"; $resn_exch{"1H17 1AD"}="H171 1AD"; $resn_exch{"2H17 1AD"}="H172 1AD";
$resn_exch{"1H18 1AD"}="H181 1AD"; $resn_exch{"2H18 1AD"}="H182 1AD"; $resn_exch{"1H19 1AD"}="H191 1AD";
$resn_exch{"2H19 1AD"}="H192 1AD"; $resn_exch{"1H1  1AE"}=" H11 1AE"; $resn_exch{"2H1  1AE"}=" H12 1AE";
$resn_exch{"3H1  1AE"}=" H13 1AE"; $resn_exch{"1H2  1AE"}=" H21 1AE"; $resn_exch{"2H2  1AE"}=" H22 1AE";
$resn_exch{"1H3  1AE"}=" H31 1AE"; $resn_exch{"2H3  1AE"}=" H32 1AE"; $resn_exch{"2H6  1AE"}=" H62 1AE";
$resn_exch{"1H4  1AF"}=" H41 1AF"; $resn_exch{"2H4  1AF"}=" H42 1AF"; $resn_exch{"3H4  1AF"}=" H43 1AF";
$resn_exch{"1H5  1AF"}=" H51 1AF"; $resn_exch{"2H5  1AF"}=" H52 1AF"; $resn_exch{"3H5  1AF"}=" H53 1AF";
$resn_exch{"1H6  1AF"}=" H61 1AF"; $resn_exch{"2H6  1AF"}=" H62 1AF"; $resn_exch{"3H6  1AF"}=" H63 1AF";
$resn_exch{"1H32 1AH"}="H321 1AH"; $resn_exch{"2H32 1AH"}="H322 1AH"; $resn_exch{"1H7C 1AH"}="H7C1 1AH";
$resn_exch{"2H7C 1AH"}="H7C2 1AH"; $resn_exch{"3H7C 1AH"}="H7C3 1AH"; $resn_exch{"1H4C 1AH"}="H4C1 1AH";
$resn_exch{"2H4C 1AH"}="H4C2 1AH"; $resn_exch{"1H22 1AH"}="H221 1AH"; $resn_exch{"2H22 1AH"}="H222 1AH";
$resn_exch{"1H58 1AH"}="H581 1AH"; $resn_exch{"2H58 1AH"}="H582 1AH"; $resn_exch{"1H43 1AH"}="H431 1AH";
$resn_exch{"2H43 1AH"}="H432 1AH"; $resn_exch{"3H43 1AH"}="H433 1AH"; $resn_exch{"1H44 1AH"}="H441 1AH";
$resn_exch{"2H44 1AH"}="H442 1AH"; $resn_exch{"3H44 1AH"}="H443 1AH"; $resn_exch{"1H86 1AH"}="H861 1AH";
$resn_exch{"2H86 1AH"}="H862 1AH"; $resn_exch{"3H86 1AH"}="H863 1AH"; $resn_exch{"1HN3 1AL"}="HN31 1AL";
$resn_exch{"2HN3 1AL"}="HN32 1AL"; $resn_exch{"1H11 1AL"}="H111 1AL"; $resn_exch{"2H11 1AL"}="H112 1AL";
$resn_exch{"1HN  1AN"}=" HN1 1AN"; $resn_exch{"2HN  1AN"}=" HN2 1AN"; $resn_exch{"1H8  1AP"}=" H81 1AP";
$resn_exch{"1HN2 1AP"}="HN21 1AP"; $resn_exch{"2HN2 1AP"}="HN22 1AP"; $resn_exch{"1HN6 1AP"}="HN61 1AP";
$resn_exch{"2HN6 1AP"}="HN62 1AP"; $resn_exch{"2H5  1AP"}=" H52 1AP"; $resn_exch{"1H5  1AP"}=" H51 1AP";
$resn_exch{"1H1  1AP"}=" H11 1AP"; $resn_exch{"1H2  1AP"}=" H21 1AP"; $resn_exch{"2H2  1AP"}=" H22 1AP";
$resn_exch{"1H3  1AP"}=" H31 1AP"; $resn_exch{"1HO3 1AP"}="HO31 1AP"; $resn_exch{"1H2  1AR"}=" H21 1AR";
$resn_exch{"2H2  1AR"}=" H22 1AR"; $resn_exch{"1H6  1AR"}=" H61 1AR"; $resn_exch{"2H6  1AR"}=" H62 1AR";
$resn_exch{"3H6  1AR"}=" H63 1AR"; $resn_exch{"1H8  1BA"}=" H81 1BA"; $resn_exch{"2H8  1BA"}=" H82 1BA";
$resn_exch{"1H1  1BA"}="H11A 1BA"; $resn_exch{"2H1  1BA"}="H12A 1BA"; $resn_exch{"3H1  1BA"}=" H13 1BA";
$resn_exch{"1H3  1BA"}=" H31 1BA"; $resn_exch{"2H3  1BA"}=" H32 1BA"; $resn_exch{"3H3  1BA"}=" H33 1BA";
$resn_exch{"1H1  1BH"}=" H11 1BH"; $resn_exch{"2H1  1BH"}=" H12 1BH"; $resn_exch{"1H41 1BH"}="H411 1BH";
$resn_exch{"2H41 1BH"}="H412 1BH"; $resn_exch{"3H41 1BH"}="H413 1BH"; $resn_exch{"1HD  1BH"}=" HD1 1BH";
$resn_exch{"2HD  1BH"}=" HD2 1BH"; $resn_exch{"1H61 1BH"}="H611 1BH"; $resn_exch{"2H61 1BH"}="H612 1BH";
$resn_exch{"1HG  1BH"}=" HG1 1BH"; $resn_exch{"2HG  1BH"}=" HG2 1BH"; $resn_exch{"1H81 1BH"}="H811 1BH";
$resn_exch{"2H81 1BH"}="H812 1BH"; $resn_exch{"1HC  1BH"}=" HC1 1BH"; $resn_exch{"2HC  1BH"}=" HC2 1BH";
$resn_exch{"3HC  1BH"}=" HC3 1BH"; $resn_exch{"1H19 1BM"}="H191 1BM"; $resn_exch{"2H19 1BM"}="H192 1BM";
$resn_exch{"3H19 1BM"}="H193 1BM"; $resn_exch{"1H27 1BM"}="H271 1BM"; $resn_exch{"2H27 1BM"}="H272 1BM";
$resn_exch{"1H30 1BM"}="H301 1BM"; $resn_exch{"2H30 1BM"}="H302 1BM"; $resn_exch{"1H36 1BM"}="H361 1BM";
$resn_exch{"2H36 1BM"}="H362 1BM"; $resn_exch{"1H42 1BM"}="H421 1BM"; $resn_exch{"2H42 1BM"}="H422 1BM";
$resn_exch{"3H42 1BM"}="H423 1BM"; $resn_exch{"1H35 1BM"}="H351 1BM"; $resn_exch{"2H35 1BM"}="H352 1BM";
$resn_exch{"1H38 1BM"}="H381 1BM"; $resn_exch{"2H38 1BM"}="H382 1BM"; $resn_exch{"3H38 1BM"}="H383 1BM";
$resn_exch{"1H4  1BN"}=" H41 1BN"; $resn_exch{"2H4  1BN"}=" H42 1BN"; $resn_exch{"1H1  1BO"}=" H11 1BO";
$resn_exch{"2H1  1BO"}=" H12 1BO"; $resn_exch{"3H1  1BO"}=" H13 1BO"; $resn_exch{"1H2  1BO"}=" H21 1BO";
$resn_exch{"2H2  1BO"}=" H22 1BO"; $resn_exch{"1H3  1BO"}=" H31 1BO"; $resn_exch{"2H3  1BO"}=" H32 1BO";
$resn_exch{"1H4  1BO"}=" H41 1BO"; $resn_exch{"2H4  1BO"}=" H42 1BO"; $resn_exch{"1H1  1BP"}=" H11 1BP";
$resn_exch{"2H1  1BP"}=" H12 1BP"; $resn_exch{"1H2  1BP"}=" H21 1BP"; $resn_exch{"1H3  1BP"}=" H31 1BP";
$resn_exch{"2H3  1BP"}=" H32 1BP"; $resn_exch{"3H3  1BP"}=" H33 1BP"; $resn_exch{"1HO2 1BP"}="HO21 1BP";
$resn_exch{"1H2  1BY"}=" H21 1BY"; $resn_exch{"2H2  1BY"}=" H22 1BY"; $resn_exch{"1H12 1C2"}="H121 1C2";
$resn_exch{"2H12 1C2"}="H122 1C2"; $resn_exch{"1H11 1C2"}="H111 1C2"; $resn_exch{"2H11 1C2"}="H112 1C2";
$resn_exch{"3H11 1C2"}="H113 1C2"; $resn_exch{"1H10 1C2"}="H101 1C2"; $resn_exch{"2H10 1C2"}="H102 1C2";
$resn_exch{"3H10 1C2"}="H103 1C2"; $resn_exch{"1H9  1C2"}=" H91 1C2"; $resn_exch{"2H9  1C2"}=" H92 1C2";
$resn_exch{"3H9  1C2"}=" H93 1C2"; $resn_exch{"1H6  1C2"}=" H61 1C2"; $resn_exch{"2H6  1C2"}=" H62 1C2";
$resn_exch{"3H6  1C2"}=" H63 1C2"; $resn_exch{"1H3  1C2"}=" H31 1C2"; $resn_exch{"2H3  1C2"}=" H32 1C2";
$resn_exch{"1H1  1C5"}=" H11 1C5"; $resn_exch{"2H1  1C5"}="H12A 1C5"; $resn_exch{"1H2  1C5"}="H21A 1C5";
$resn_exch{"2H2  1C5"}=" H22 1C5"; $resn_exch{"1H3  1C5"}=" H31 1C5"; $resn_exch{"2H3  1C5"}=" H32 1C5";
$resn_exch{"1H5  1C5"}=" H51 1C5"; $resn_exch{"2H5  1C5"}=" H52 1C5"; $resn_exch{"1H6  1C5"}=" H61 1C5";
$resn_exch{"2H6  1C5"}=" H62 1C5"; $resn_exch{"1H7  1C5"}=" H71 1C5"; $resn_exch{"2H7  1C5"}=" H72 1C5";
$resn_exch{"1H8  1C5"}=" H81 1C5"; $resn_exch{"2H8  1C5"}=" H82 1C5"; $resn_exch{"1H13 1C5"}="H131 1C5";
$resn_exch{"2H13 1C5"}="H132 1C5"; $resn_exch{"1H14 1C5"}="H141 1C5"; $resn_exch{"2H14 1C5"}="H142 1C5";
$resn_exch{"1H18 1C5"}="H181 1C5"; $resn_exch{"2H18 1C5"}="H182 1C5"; $resn_exch{"1H26 1C5"}="H261 1C5";
$resn_exch{"2H26 1C5"}="H262 1C5"; $resn_exch{"3H26 1C5"}="H263 1C5"; $resn_exch{"1HN1 1C5"}="HN11 1C5";
$resn_exch{"2HN1 1C5"}="HN12 1C5"; $resn_exch{"2HOP 1C5"}="HOP2 1C5"; $resn_exch{"3HOP 1C5"}="HOP3 1C5";
$resn_exch{"1H1  1CA"}=" H11 1CA"; $resn_exch{"2H1  1CA"}=" H12 1CA"; $resn_exch{"1H2  1CA"}=" H21 1CA";
$resn_exch{"2H2  1CA"}=" H22 1CA"; $resn_exch{"1H6  1CA"}=" H61 1CA"; $resn_exch{"2H6  1CA"}=" H62 1CA";
$resn_exch{"1H7  1CA"}=" H71 1CA"; $resn_exch{"2H7  1CA"}=" H72 1CA"; $resn_exch{"1H11 1CA"}="H111 1CA";
$resn_exch{"2H11 1CA"}="H112 1CA"; $resn_exch{"1H12 1CA"}="H121 1CA"; $resn_exch{"2H12 1CA"}="H122 1CA";
$resn_exch{"1H15 1CA"}="H151 1CA"; $resn_exch{"2H15 1CA"}="H152 1CA"; $resn_exch{"1H16 1CA"}="H161 1CA";
$resn_exch{"2H16 1CA"}="H162 1CA"; $resn_exch{"1H18 1CA"}="H181 1CA"; $resn_exch{"2H18 1CA"}="H182 1CA";
$resn_exch{"3H18 1CA"}="H183 1CA"; $resn_exch{"1H19 1CA"}="H191 1CA"; $resn_exch{"2H19 1CA"}="H192 1CA";
$resn_exch{"3H19 1CA"}="H193 1CA"; $resn_exch{"1H21 1CA"}="H211 1CA"; $resn_exch{"2H21 1CA"}="H212 1CA";
$resn_exch{"1H2  1CB"}=" H21 1CB"; $resn_exch{"2H2  1CB"}=" H22 1CB"; $resn_exch{"1H4  1CB"}=" H41 1CB";
$resn_exch{"2H4  1CB"}=" H42 1CB"; $resn_exch{"1H5  1CB"}=" H51 1CB"; $resn_exch{"2H5  1CB"}=" H52 1CB";
$resn_exch{"1H3  1CB"}=" H31 1CB"; $resn_exch{"2H3  1CB"}=" H32 1CB"; $resn_exch{"1HN2 1CB"}="HN21 1CB";
$resn_exch{"2HN2 1CB"}="HN22 1CB"; $resn_exch{"1H1  1CD"}="H11A 1CD"; $resn_exch{"2H1  1CD"}=" H12 1CD";
$resn_exch{"3H1  1CD"}="H13A 1CD"; $resn_exch{"1H3  1CD"}=" H31 1CD"; $resn_exch{"2H3  1CD"}=" H32 1CD";
$resn_exch{"1H5  1CD"}=" H51 1CD"; $resn_exch{"2H5  1CD"}=" H52 1CD"; $resn_exch{"1H28 1CD"}="H281 1CD";
$resn_exch{"2H28 1CD"}="H282 1CD"; $resn_exch{"1H30 1CD"}="H301 1CD"; $resn_exch{"2H30 1CD"}="H302 1CD";
$resn_exch{"1H36 1CM"}="H361 1CM"; $resn_exch{"2H36 1CM"}="H362 1CM"; $resn_exch{"1H35 1CM"}="H351 1CM";
$resn_exch{"2H35 1CM"}="H352 1CM"; $resn_exch{"3H35 1CM"}="H353 1CM"; $resn_exch{"1HN1 1CN"}="HN11 1CN";
$resn_exch{"2HN1 1CN"}="HN12 1CN"; $resn_exch{"1HN5 1CN"}="HN51 1CN"; $resn_exch{"2HN5 1CN"}="HN52 1CN";
$resn_exch{"1H42 1CP"}="H421 1CP"; $resn_exch{"2H42 1CP"}="H422 1CP"; $resn_exch{"1H41 1CP"}="H411 1CP";
$resn_exch{"2H41 1CP"}="H412 1CP"; $resn_exch{"1H7A 1CP"}="H7A1 1CP"; $resn_exch{"2H7A 1CP"}="H7A2 1CP";
$resn_exch{"3H7A 1CP"}="H7A3 1CP"; $resn_exch{"1H10 1CP"}="H101 1CP"; $resn_exch{"2H10 1CP"}="H102 1CP";
$resn_exch{"1H5  1CP"}=" H51 1CP"; $resn_exch{"2H5  1CP"}=" H52 1CP"; $resn_exch{"1H31 1CP"}="H311 1CP";
$resn_exch{"2H31 1CP"}="H312 1CP"; $resn_exch{"1H32 1CP"}="H321 1CP"; $resn_exch{"2H32 1CP"}="H322 1CP";
$resn_exch{"1H2A 1CP"}="H2A1 1CP"; $resn_exch{"2H2A 1CP"}="H2A2 1CP"; $resn_exch{"3H2A 1CP"}="H2A3 1CP";
$resn_exch{"1H20 1CP"}="H201 1CP"; $resn_exch{"2H20 1CP"}="H202 1CP"; $resn_exch{"1H61 1CP"}="H611 1CP";
$resn_exch{"2H61 1CP"}="H612 1CP"; $resn_exch{"1H62 1CP"}="H621 1CP"; $resn_exch{"2H62 1CP"}="H622 1CP";
$resn_exch{"1H86 1CP"}="H861 1CP"; $resn_exch{"2H86 1CP"}="H862 1CP"; $resn_exch{"3H86 1CP"}="H863 1CP";
$resn_exch{"1H15 1CP"}="H151 1CP"; $resn_exch{"2H15 1CP"}="H152 1CP"; $resn_exch{"1H81 1CP"}="H811 1CP";
$resn_exch{"2H81 1CP"}="H812 1CP"; $resn_exch{"3H81 1CP"}="H813 1CP"; $resn_exch{"1H91 1CP"}="H911 1CP";
$resn_exch{"2H91 1CP"}="H912 1CP"; $resn_exch{"1H92 1CP"}="H921 1CP"; $resn_exch{"2H92 1CP"}="H922 1CP";
$resn_exch{"1H5' 1CS"}="H5'1 1CS"; $resn_exch{"2H5' 1CS"}="H5'2 1CS"; $resn_exch{"3H5' 1CS"}="H5'3 1CS";
$resn_exch{"1H7' 1CS"}="H7'1 1CS"; $resn_exch{"2H7' 1CS"}="H7'2 1CS"; $resn_exch{"3H7' 1CS"}="H7'3 1CS";
$resn_exch{"1HO1 1CU"}="HO11 1CU"; $resn_exch{"2HO1 1CU"}="HO12 1CU"; $resn_exch{"*HO5 1DA"}="HO5' 1DA";
$resn_exch{"1H5* 1DA"}="H5'1 1DA"; $resn_exch{"2H5* 1DA"}="H5'2 1DA"; $resn_exch{" H4* 1DA"}=" H4' 1DA";
$resn_exch{" H3* 1DA"}=" H3' 1DA"; $resn_exch{"*HO3 1DA"}="HO3' 1DA"; $resn_exch{" H2* 1DA"}=" H2' 1DA";
$resn_exch{"*HO2 1DA"}="HO2' 1DA"; $resn_exch{" H1* 1DA"}=" H1' 1DA"; $resn_exch{"1HN6 1DA"}="HN61 1DA";
$resn_exch{"2HN6 1DA"}="HN62 1DA"; $resn_exch{"1H1C 1DB"}="H1C1 1DB"; $resn_exch{"2H1C 1DB"}="H1C2 1DB";
$resn_exch{"1H2C 1DB"}="H2C1 1DB"; $resn_exch{"2H2C 1DB"}="H2C2 1DB"; $resn_exch{"1H3C 1DB"}="H3C1 1DB";
$resn_exch{"2H3C 1DB"}="H3C2 1DB"; $resn_exch{"1H4C 1DB"}="H4C1 1DB"; $resn_exch{"2H4C 1DB"}="H4C2 1DB";
$resn_exch{"1H5C 1DB"}="H5C1 1DB"; $resn_exch{"2H5C 1DB"}="H5C2 1DB"; $resn_exch{"1H6C 1DB"}="H6C1 1DB";
$resn_exch{"2H6C 1DB"}="H6C2 1DB"; $resn_exch{"1H7C 1DB"}="H7C1 1DB"; $resn_exch{"2H7C 1DB"}="H7C2 1DB";
$resn_exch{"1H8C 1DB"}="H8C1 1DB"; $resn_exch{"2H8C 1DB"}="H8C2 1DB"; $resn_exch{"1H9C 1DB"}="H9C1 1DB";
$resn_exch{"2H9C 1DB"}="H9C2 1DB"; $resn_exch{"1H10 1DB"}="H101 1DB"; $resn_exch{"2H10 1DB"}="H102 1DB";
$resn_exch{"3H10 1DB"}="H103 1DB"; $resn_exch{"1HAA 1DG"}="HAA1 1DG"; $resn_exch{"2HAA 1DG"}="HAA2 1DG";
$resn_exch{" H1* 1DG"}=" H1' 1DG"; $resn_exch{" H2* 1DG"}=" H2' 1DG"; $resn_exch{" H3* 1DG"}=" H3' 1DG";
$resn_exch{" H4* 1DG"}=" H4' 1DG"; $resn_exch{"1H5* 1DG"}="H5'1 1DG"; $resn_exch{"2H5* 1DG"}="H5'2 1DG";
$resn_exch{"1HAY 1DG"}="HAY1 1DG"; $resn_exch{"2HAY 1DG"}="HAY2 1DG"; $resn_exch{"1H6N 1DG"}="H6N1 1DG";
$resn_exch{"2H6N 1DG"}="H6N2 1DG"; $resn_exch{"1H21 1DM"}="H211 1DM"; $resn_exch{"2H21 1DM"}="H212 1DM";
$resn_exch{"1H22 1DM"}="H221 1DM"; $resn_exch{"2H22 1DM"}="H222 1DM"; $resn_exch{"1H23 1DM"}="H231 1DM";
$resn_exch{"2H23 1DM"}="H232 1DM"; $resn_exch{"1H24 1DM"}="H241 1DM"; $resn_exch{"2H24 1DM"}="H242 1DM";
$resn_exch{"1H20 1DM"}="H201 1DM"; $resn_exch{"2H20 1DM"}="H202 1DM"; $resn_exch{"1H30 1DM"}="H301 1DM";
$resn_exch{"2H30 1DM"}="H302 1DM"; $resn_exch{"1H25 1DM"}="H251 1DM"; $resn_exch{"2H25 1DM"}="H252 1DM";
$resn_exch{"1H26 1DM"}="H261 1DM"; $resn_exch{"2H26 1DM"}="H262 1DM"; $resn_exch{"1H27 1DM"}="H271 1DM";
$resn_exch{"2H27 1DM"}="H272 1DM"; $resn_exch{"1H28 1DM"}="H281 1DM"; $resn_exch{"2H28 1DM"}="H282 1DM";
$resn_exch{"1H6  1DM"}=" H61 1DM"; $resn_exch{"2H6  1DM"}=" H62 1DM"; $resn_exch{"1H1C 1DO"}="H1C1 1DO";
$resn_exch{"2H1C 1DO"}="H1C2 1DO"; $resn_exch{"1H2C 1DO"}="H2C1 1DO"; $resn_exch{"2H2C 1DO"}="H2C2 1DO";
$resn_exch{"1H3C 1DO"}="H3C1 1DO"; $resn_exch{"2H3C 1DO"}="H3C2 1DO"; $resn_exch{"1H4C 1DO"}="H4C1 1DO";
$resn_exch{"2H4C 1DO"}="H4C2 1DO"; $resn_exch{"1H5C 1DO"}="H5C1 1DO"; $resn_exch{"2H5C 1DO"}="H5C2 1DO";
$resn_exch{"1H6C 1DO"}="H6C1 1DO"; $resn_exch{"2H6C 1DO"}="H6C2 1DO"; $resn_exch{"1H7C 1DO"}="H7C1 1DO";
$resn_exch{"2H7C 1DO"}="H7C2 1DO"; $resn_exch{"1H8C 1DO"}="H8C1 1DO"; $resn_exch{"2H8C 1DO"}="H8C2 1DO";
$resn_exch{"1H9C 1DO"}="H9C1 1DO"; $resn_exch{"2H9C 1DO"}="H9C2 1DO"; $resn_exch{"1H10 1DO"}="H101 1DO";
$resn_exch{"2H10 1DO"}="H102 1DO"; $resn_exch{"1H11 1DO"}="H111 1DO"; $resn_exch{"2H11 1DO"}="H112 1DO";
$resn_exch{"1H12 1DO"}="H121 1DO"; $resn_exch{"2H12 1DO"}="H122 1DO"; $resn_exch{"3H12 1DO"}="H123 1DO";
$resn_exch{"1H12 1EM"}="H121 1EM"; $resn_exch{"2H12 1EM"}="H122 1EM"; $resn_exch{"1H13 1EM"}="H131 1EM";
$resn_exch{"2H13 1EM"}="H132 1EM"; $resn_exch{"1H14 1EM"}="H141 1EM"; $resn_exch{"2H14 1EM"}="H142 1EM";
$resn_exch{"1H15 1EM"}="H151 1EM"; $resn_exch{"2H15 1EM"}="H152 1EM"; $resn_exch{"1H16 1EM"}="H161 1EM";
$resn_exch{"2H16 1EM"}="H162 1EM"; $resn_exch{"1H17 1EM"}="H171 1EM"; $resn_exch{"2H17 1EM"}="H172 1EM";
$resn_exch{"1H18 1EM"}="H181 1EM"; $resn_exch{"2H18 1EM"}="H182 1EM"; $resn_exch{"1H19 1EM"}="H191 1EM";
$resn_exch{"2H19 1EM"}="H192 1EM"; $resn_exch{"3H19 1EM"}="H193 1EM"; $resn_exch{"1H22 1EM"}="H221 1EM";
$resn_exch{"2H22 1EM"}="H222 1EM"; $resn_exch{"1H23 1EM"}="H231 1EM"; $resn_exch{"2H23 1EM"}="H232 1EM";
$resn_exch{"1H24 1EM"}="H241 1EM"; $resn_exch{"2H24 1EM"}="H242 1EM"; $resn_exch{"1H25 1EM"}="H251 1EM";
$resn_exch{"2H25 1EM"}="H252 1EM"; $resn_exch{"1H26 1EM"}="H261 1EM"; $resn_exch{"2H26 1EM"}="H262 1EM";
$resn_exch{"1H27 1EM"}="H271 1EM"; $resn_exch{"2H27 1EM"}="H272 1EM"; $resn_exch{"1H28 1EM"}="H281 1EM";
$resn_exch{"2H28 1EM"}="H282 1EM"; $resn_exch{"1H29 1EM"}="H291 1EM"; $resn_exch{"2H29 1EM"}="H292 1EM";
$resn_exch{"1H30 1EM"}="H301 1EM"; $resn_exch{"2H30 1EM"}="H302 1EM"; $resn_exch{"1H31 1EM"}="H311 1EM";
$resn_exch{"2H31 1EM"}="H312 1EM"; $resn_exch{"1H32 1EM"}="H321 1EM"; $resn_exch{"2H32 1EM"}="H322 1EM";
$resn_exch{"1H33 1EM"}="H331 1EM"; $resn_exch{"2H33 1EM"}="H332 1EM"; $resn_exch{"1H34 1EM"}="H341 1EM";
$resn_exch{"2H34 1EM"}="H342 1EM"; $resn_exch{"3H34 1EM"}="H343 1EM"; $resn_exch{"1H41 1EM"}="H411 1EM";
$resn_exch{"2H41 1EM"}="H412 1EM"; $resn_exch{"1H43 1EM"}="H431 1EM"; $resn_exch{"2H43 1EM"}="H432 1EM";
$resn_exch{"1HAA 1FH"}="HAA1 1FH"; $resn_exch{"2HAA 1FH"}="HAA2 1FH"; $resn_exch{"1HMA 1FH"}="HMA1 1FH";
$resn_exch{"2HMA 1FH"}="HMA2 1FH"; $resn_exch{"3HMA 1FH"}="HMA3 1FH"; $resn_exch{"1HBA 1FH"}="HBA1 1FH";
$resn_exch{"2HBA 1FH"}="HBA2 1FH"; $resn_exch{"1HMB 1FH"}="HMB1 1FH"; $resn_exch{"2HMB 1FH"}="HMB2 1FH";
$resn_exch{"3HMB 1FH"}="HMB3 1FH"; $resn_exch{"1HBB 1FH"}="HBB1 1FH"; $resn_exch{"2HBB 1FH"}="HBB2 1FH";
$resn_exch{"1HMC 1FH"}="HMC1 1FH"; $resn_exch{"2HMC 1FH"}="HMC2 1FH"; $resn_exch{"3HMC 1FH"}="HMC3 1FH";
$resn_exch{"1HBC 1FH"}="HBC1 1FH"; $resn_exch{"2HBC 1FH"}="HBC2 1FH"; $resn_exch{"1HMD 1FH"}="HMD1 1FH";
$resn_exch{"2HMD 1FH"}="HMD2 1FH"; $resn_exch{"3HMD 1FH"}="HMD3 1FH"; $resn_exch{"1HAD 1FH"}="HAD1 1FH";
$resn_exch{"2HAD 1FH"}="HAD2 1FH"; $resn_exch{"1HBD 1FH"}="HBD1 1FH"; $resn_exch{"2HBD 1FH"}="HBD2 1FH";
$resn_exch{"1H2  1GL"}=" H21 1GL"; $resn_exch{"2H2  1GL"}=" H22 1GL"; $resn_exch{"1H6  1GL"}=" H61 1GL";
$resn_exch{"2H6  1GL"}=" H62 1GL"; $resn_exch{"3H6  1GL"}=" H63 1GL"; $resn_exch{"1HN2 1GN"}="HN21 1GN";
$resn_exch{"2HN2 1GN"}="HN22 1GN"; $resn_exch{"1H6  1GN"}=" H61 1GN"; $resn_exch{"2H6  1GN"}=" H62 1GN";
$resn_exch{"1H3  1GP"}=" H31 1GP"; $resn_exch{"2H3  1GP"}=" H32 1GP"; $resn_exch{"1H1  1GP"}=" H11 1GP";
$resn_exch{"2H1  1GP"}=" H12 1GP"; $resn_exch{"1HAA 1HP"}="HAA1 1HP"; $resn_exch{"2HAA 1HP"}="HAA2 1HP";
$resn_exch{"3HAA 1HP"}="HAA3 1HP"; $resn_exch{"1HAB 1HP"}="HAB1 1HP"; $resn_exch{"2HAB 1HP"}="HAB2 1HP";
$resn_exch{"1HN3 1IG"}="HN31 1IG"; $resn_exch{"2HN3 1IG"}="HN32 1IG"; $resn_exch{"1HN4 1IG"}="HN41 1IG";
$resn_exch{"2HN4 1IG"}="HN42 1IG"; $resn_exch{"1H5  1IG"}=" H51 1IG"; $resn_exch{"2H5  1IG"}=" H52 1IG";
$resn_exch{"1H6  1IG"}=" H61 1IG"; $resn_exch{"2H6  1IG"}=" H62 1IG"; $resn_exch{"3H6  1IG"}=" H63 1IG";
$resn_exch{"1H16 1IG"}="H161 1IG"; $resn_exch{"2H16 1IG"}="H162 1IG"; $resn_exch{"1H17 1IG"}="H171 1IG";
$resn_exch{"2H17 1IG"}="H172 1IG"; $resn_exch{"1H19 1IG"}="H191 1IG"; $resn_exch{"2H19 1IG"}="H192 1IG";
$resn_exch{"3H19 1IG"}="H193 1IG"; $resn_exch{"1H21 1IG"}="H211 1IG"; $resn_exch{"2H21 1IG"}="H212 1IG";
$resn_exch{"3H21 1IG"}="H213 1IG"; $resn_exch{"1H20 1IG"}="H201 1IG"; $resn_exch{"2H20 1IG"}="H202 1IG";
$resn_exch{"3H20 1IG"}="H203 1IG"; $resn_exch{"1H1  1IN"}="H11A 1IN"; $resn_exch{"2H1  1IN"}=" H12 1IN";
$resn_exch{"1H7  1IN"}=" H71 1IN"; $resn_exch{"2H7  1IN"}=" H72 1IN"; $resn_exch{"3H7  1IN"}=" H73 1IN";
$resn_exch{"1H8  1IN"}=" H81 1IN"; $resn_exch{"2H8  1IN"}=" H82 1IN"; $resn_exch{"1H9  1IN"}=" H91 1IN";
$resn_exch{"2H9  1IN"}=" H92 1IN"; $resn_exch{"1H10 1IN"}="H101 1IN"; $resn_exch{"2H10 1IN"}="H102 1IN";
$resn_exch{"1H12 1IN"}="H121 1IN"; $resn_exch{"2H12 1IN"}="H122 1IN"; $resn_exch{"1H14 1IN"}="H141 1IN";
$resn_exch{"2H14 1IN"}="H142 1IN"; $resn_exch{"1H24 1IN"}="H241 1IN"; $resn_exch{"2H24 1IN"}="H242 1IN";
$resn_exch{"1H27 1IN"}="H271 1IN"; $resn_exch{"2H27 1IN"}="H272 1IN"; $resn_exch{"1H5  1IN"}=" H51 1IN";
$resn_exch{"2H5  1IN"}=" H52 1IN"; $resn_exch{"3H5  1IN"}=" H53 1IN"; $resn_exch{"1H6  1IN"}=" H61 1IN";
$resn_exch{"2H6  1IN"}=" H62 1IN"; $resn_exch{"3H6  1IN"}=" H63 1IN"; $resn_exch{"1H25 1IN"}="H251 1IN";
$resn_exch{"2H25 1IN"}="H252 1IN"; $resn_exch{"1H29 1IN"}="H291 1IN"; $resn_exch{"2H29 1IN"}="H292 1IN";
$resn_exch{"3H29 1IN"}="H293 1IN"; $resn_exch{"1H26 1IN"}="H261 1IN"; $resn_exch{"2H26 1IN"}="H262 1IN";
$resn_exch{"1HAJ 1IP"}="HAJ1 1IP"; $resn_exch{"2HAJ 1IP"}="HAJ2 1IP"; $resn_exch{"1HB  1IP"}=" HB1 1IP";
$resn_exch{"2HB  1IP"}=" HB2 1IP"; $resn_exch{"1HD2 1IP"}="HD21 1IP"; $resn_exch{"2HD2 1IP"}="HD22 1IP";
$resn_exch{"1H7  1IQ"}=" H71 1IQ"; $resn_exch{"2H7  1IQ"}=" H72 1IQ"; $resn_exch{"3H7  1IQ"}=" H73 1IQ";
$resn_exch{"1H10 1IQ"}="H101 1IQ"; $resn_exch{"2H10 1IQ"}="H102 1IQ"; $resn_exch{"3H10 1IQ"}="H103 1IQ";
$resn_exch{"1H9  1IQ"}=" H91 1IQ"; $resn_exch{"2H9  1IQ"}=" H92 1IQ"; $resn_exch{"3H9  1IQ"}=" H93 1IQ";
$resn_exch{"1H4  1LG"}=" H41 1LG"; $resn_exch{"2H4  1LG"}=" H42 1LG"; $resn_exch{"1H8  1LG"}=" H81 1LG";
$resn_exch{"2H8  1LG"}=" H82 1LG"; $resn_exch{"1H28 1LG"}="H281 1LG"; $resn_exch{"2H28 1LG"}="H282 1LG";
$resn_exch{"1H9  1LG"}=" H91 1LG"; $resn_exch{"2H9  1LG"}=" H92 1LG"; $resn_exch{"1H12 1LG"}="H121 1LG";
$resn_exch{"2H12 1LG"}="H122 1LG"; $resn_exch{"1H27 1LG"}="H271 1LG"; $resn_exch{"2H27 1LG"}="H272 1LG";
$resn_exch{"1H29 1LG"}="H291 1LG"; $resn_exch{"2H29 1LG"}="H292 1LG"; $resn_exch{"1H30 1LG"}="H301 1LG";
$resn_exch{"2H30 1LG"}="H302 1LG"; $resn_exch{"1H1C 1LP"}="H1C1 1LP"; $resn_exch{"1H3C 1LP"}="H3C1 1LP";
$resn_exch{"2H3C 1LP"}="H3C2 1LP"; $resn_exch{"3H3C 1LP"}="H3C3 1LP"; $resn_exch{"1HB  1LU"}=" HB1 1LU";
$resn_exch{"2HB  1LU"}=" HB2 1LU"; $resn_exch{"1HD1 1LU"}="HD11 1LU"; $resn_exch{"2HD1 1LU"}="HD12 1LU";
$resn_exch{"3HD1 1LU"}="HD13 1LU"; $resn_exch{"1HD2 1LU"}="HD21 1LU"; $resn_exch{"2HD2 1LU"}="HD22 1LU";
$resn_exch{"3HD2 1LU"}="HD23 1LU"; $resn_exch{"2HOP 1MA"}="HOP2 1MA"; $resn_exch{"3HOP 1MA"}="HOP3 1MA";
$resn_exch{"1H5* 1MA"}=" H5' 1MA"; $resn_exch{"2H5* 1MA"}="H5'' 1MA"; $resn_exch{" H4* 1MA"}=" H4' 1MA";
$resn_exch{" H3* 1MA"}=" H3' 1MA"; $resn_exch{"*HO3 1MA"}="HO3' 1MA"; $resn_exch{" H2* 1MA"}=" H2' 1MA";
$resn_exch{"*HO2 1MA"}="HO2' 1MA"; $resn_exch{" H1* 1MA"}=" H1' 1MA"; $resn_exch{"1HN6 1MA"}="HN61 1MA";
$resn_exch{"2HN6 1MA"}="HN62 1MA"; $resn_exch{"1HM1 1MA"}="HM11 1MA"; $resn_exch{"2HM1 1MA"}="HM12 1MA";
$resn_exch{"3HM1 1MA"}="HM13 1MA"; $resn_exch{"1H1  1MC"}=" H11 1MC"; $resn_exch{"2H1  1MC"}=" H12 1MC";
$resn_exch{"3H1  1MC"}=" H13 1MC"; $resn_exch{"1HN4 1MC"}="HN41 1MC"; $resn_exch{"2HN4 1MC"}="HN42 1MC";
$resn_exch{"2HOP 1MG"}="HOP2 1MG"; $resn_exch{"3HOP 1MG"}="HOP3 1MG"; $resn_exch{"1H5* 1MG"}=" H5' 1MG";
$resn_exch{"2H5* 1MG"}="H5'' 1MG"; $resn_exch{" H4* 1MG"}=" H4' 1MG"; $resn_exch{" H3* 1MG"}=" H3' 1MG";
$resn_exch{"*HO3 1MG"}="HO3' 1MG"; $resn_exch{" H2* 1MG"}=" H2' 1MG"; $resn_exch{"*HO2 1MG"}="HO2' 1MG";
$resn_exch{" H1* 1MG"}=" H1' 1MG"; $resn_exch{"1HM1 1MG"}="HM11 1MG"; $resn_exch{"2HM1 1MG"}="HM12 1MG";
$resn_exch{"3HM1 1MG"}="HM13 1MG"; $resn_exch{"1HN2 1MG"}="HN21 1MG"; $resn_exch{"2HN2 1MG"}="HN22 1MG";
$resn_exch{"1H13 1MM"}="H131 1MM"; $resn_exch{"2H13 1MM"}="H132 1MM"; $resn_exch{"3H13 1MM"}="H133 1MM";
$resn_exch{"1H5' 1MM"}="H5'1 1MM"; $resn_exch{"2H5' 1MM"}="H5'2 1MM"; $resn_exch{"3H5' 1MM"}="H5'3 1MM";
$resn_exch{"1H7' 1MM"}="H7'1 1MM"; $resn_exch{"2H7' 1MM"}="H7'2 1MM"; $resn_exch{"3H7' 1MM"}="H7'3 1MM";
$resn_exch{"1HAA 1MR"}="HAA1 1MR"; $resn_exch{"2HAA 1MR"}="HAA2 1MR"; $resn_exch{"3HAA 1MR"}="HAA3 1MR";
$resn_exch{"1HM1 1MZ"}="HM11 1MZ"; $resn_exch{"2HM1 1MZ"}="HM12 1MZ"; $resn_exch{"3HM1 1MZ"}="HM13 1MZ";
$resn_exch{"1H10 1N1"}="H101 1N1"; $resn_exch{"2H10 1N1"}="H102 1N1"; $resn_exch{"3H10 1N1"}="H103 1N1";
$resn_exch{"1H15 1N1"}="H151 1N1"; $resn_exch{"2H15 1N1"}="H152 1N1"; $resn_exch{"3H15 1N1"}="H153 1N1";
$resn_exch{"1H16 1N1"}="H161 1N1"; $resn_exch{"2H16 1N1"}="H162 1N1"; $resn_exch{"1H19 1N1"}="H191 1N1";
$resn_exch{"2H19 1N1"}="H192 1N1"; $resn_exch{"1H20 1N1"}="H201 1N1"; $resn_exch{"2H20 1N1"}="H202 1N1";
$resn_exch{"1H21 1N1"}="H211 1N1"; $resn_exch{"2H21 1N1"}="H212 1N1"; $resn_exch{"1H17 1N1"}="H171 1N1";
$resn_exch{"2H17 1N1"}="H172 1N1"; $resn_exch{"1H18 1N1"}="H181 1N1"; $resn_exch{"2H18 1N1"}="H182 1N1";
$resn_exch{"1H25 1N8"}="H251 1N8"; $resn_exch{"2H25 1N8"}="H252 1N8"; $resn_exch{"3H25 1N8"}="H253 1N8";
$resn_exch{"1H27 1N8"}="H271 1N8"; $resn_exch{"2H27 1N8"}="H272 1N8"; $resn_exch{"1H28 1N8"}="H281 1N8";
$resn_exch{"2H28 1N8"}="H282 1N8"; $resn_exch{"1H29 1N8"}="H291 1N8"; $resn_exch{"2H29 1N8"}="H292 1N8";
$resn_exch{"1H30 1N8"}="H301 1N8"; $resn_exch{"2H30 1N8"}="H302 1N8"; $resn_exch{"3H30 1N8"}="H303 1N8";
$resn_exch{"1H5  1N8"}=" H51 1N8"; $resn_exch{"2H5  1N8"}=" H52 1N8"; $resn_exch{"3H5  1N8"}=" H53 1N8";
$resn_exch{"1H1  1N8"}=" H11 1N8"; $resn_exch{"2H1  1N8"}="H12A 1N8"; $resn_exch{"3H1  1N8"}=" H13 1N8";
$resn_exch{"1H31 1N9"}="H311 1N9"; $resn_exch{"2H31 1N9"}="H312 1N9"; $resn_exch{"3H31 1N9"}="H313 1N9";
$resn_exch{"1H32 1N9"}="H321 1N9"; $resn_exch{"2H32 1N9"}="H322 1N9"; $resn_exch{"3H32 1N9"}="H323 1N9";
$resn_exch{"1H33 1N9"}="H331 1N9"; $resn_exch{"2H33 1N9"}="H332 1N9"; $resn_exch{"3H33 1N9"}="H333 1N9";
$resn_exch{"1H36 1N9"}="H361 1N9"; $resn_exch{"2H36 1N9"}="H362 1N9"; $resn_exch{"3H36 1N9"}="H363 1N9";
$resn_exch{"1H37 1N9"}="H371 1N9"; $resn_exch{"2H37 1N9"}="H372 1N9"; $resn_exch{"3H37 1N9"}="H373 1N9";
$resn_exch{"1H7  1N9"}=" H71 1N9"; $resn_exch{"2H7  1N9"}=" H72 1N9"; $resn_exch{"1H8  1N9"}=" H81 1N9";
$resn_exch{"2H8  1N9"}=" H82 1N9"; $resn_exch{"1H35 1N9"}="H351 1N9"; $resn_exch{"2H35 1N9"}="H352 1N9";
$resn_exch{"1H34 1N9"}="H341 1N9"; $resn_exch{"2H34 1N9"}="H342 1N9"; $resn_exch{"3H34 1N9"}="H343 1N9";
$resn_exch{"1H1  1N9"}=" H11 1N9"; $resn_exch{"2H1  1N9"}=" H12 1N9"; $resn_exch{"1H3  1N9"}=" H31 1N9";
$resn_exch{"2H3  1N9"}=" H32 1N9"; $resn_exch{"1H4  1N9"}=" H41 1N9"; $resn_exch{"2H4  1N9"}=" H42 1N9";
$resn_exch{"1H6  1N9"}=" H61 1N9"; $resn_exch{"2H6  1N9"}=" H62 1N9"; $resn_exch{"1H2  1N9"}=" H21 1N9";
$resn_exch{"2H2  1N9"}=" H22 1N9"; $resn_exch{"3H2  1N9"}=" H23 1N9"; $resn_exch{"1H6  1NA"}=" H61 1NA";
$resn_exch{"2H6  1NA"}=" H62 1NA"; $resn_exch{"1H8  1NA"}=" H81 1NA"; $resn_exch{"2H8  1NA"}=" H82 1NA";
$resn_exch{"3H8  1NA"}=" H83 1NA"; $resn_exch{"1HM  1NA"}=" HM1 1NA"; $resn_exch{"2HM  1NA"}=" HM2 1NA";
$resn_exch{"3HM  1NA"}=" HM3 1NA"; $resn_exch{"1H9  1NB"}=" H91 1NB"; $resn_exch{"2H9  1NB"}=" H92 1NB";
$resn_exch{"1H26 1NB"}="H261 1NB"; $resn_exch{"2H26 1NB"}="H262 1NB"; $resn_exch{"1H27 1NB"}="H271 1NB";
$resn_exch{"2H27 1NB"}="H272 1NB"; $resn_exch{"1H28 1NB"}="H281 1NB"; $resn_exch{"2H28 1NB"}="H282 1NB";
$resn_exch{"1H19 1NB"}="H191 1NB"; $resn_exch{"2H19 1NB"}="H192 1NB"; $resn_exch{"1H4  1NH"}=" H41 1NH";
$resn_exch{"2H4  1NH"}=" H42 1NH"; $resn_exch{"3H4  1NH"}=" H43 1NH"; $resn_exch{"1H3  1NH"}=" H31 1NH";
$resn_exch{"2H3  1NH"}=" H32 1NH"; $resn_exch{"1H5  1NH"}=" H51 1NH"; $resn_exch{"2H5  1NH"}=" H52 1NH";
$resn_exch{"1H8  1NH"}=" H81 1NH"; $resn_exch{"2H8  1NH"}=" H82 1NH"; $resn_exch{"3H8  1NH"}=" H83 1NH";
$resn_exch{"1H9  1NH"}=" H91 1NH"; $resn_exch{"2H9  1NH"}=" H92 1NH"; $resn_exch{"1H11 1NH"}="H111 1NH";
$resn_exch{"2H11 1NH"}="H112 1NH"; $resn_exch{"1H10 1NH"}="H101 1NH"; $resn_exch{"2H10 1NH"}="H102 1NH";
$resn_exch{"1H1  1NI"}=" H11 1NI"; $resn_exch{"2H1  1NI"}=" H12 1NI"; $resn_exch{"3H1  1NI"}="H13A 1NI";
$resn_exch{"1H24 1NI"}="H241 1NI"; $resn_exch{"2H24 1NI"}="H242 1NI"; $resn_exch{"3H24 1NI"}="H243 1NI";
$resn_exch{"1H30 1NI"}="H301 1NI"; $resn_exch{"2H30 1NI"}="H302 1NI"; $resn_exch{"3H30 1NI"}="H303 1NI";
$resn_exch{"1H31 1NI"}="H311 1NI"; $resn_exch{"2H31 1NI"}="H312 1NI"; $resn_exch{"3H31 1NI"}="H313 1NI";
$resn_exch{"1H33 1NI"}="H331 1NI"; $resn_exch{"2H33 1NI"}="H332 1NI"; $resn_exch{"1H6  1NI"}=" H61 1NI";
$resn_exch{"2H6  1NI"}=" H62 1NI"; $resn_exch{"1H25 1NI"}="H251 1NI"; $resn_exch{"2H25 1NI"}="H252 1NI";
$resn_exch{"3H25 1NI"}="H253 1NI"; $resn_exch{"1H28 1NI"}="H281 1NI"; $resn_exch{"2H28 1NI"}="H282 1NI";
$resn_exch{"1H39 1NI"}="H391 1NI"; $resn_exch{"2H39 1NI"}="H392 1NI"; $resn_exch{"1H40 1NI"}="H401 1NI";
$resn_exch{"2H40 1NI"}="H402 1NI"; $resn_exch{"3H40 1NI"}="H403 1NI"; $resn_exch{"1H45 1NI"}="H451 1NI";
$resn_exch{"2H45 1NI"}="H452 1NI"; $resn_exch{"1HN  1NI"}=" HN1 1NI"; $resn_exch{"2HN  1NI"}=" HN2 1NI";
$resn_exch{"1HI3 1NT"}="HI31 1NT"; $resn_exch{"2HI3 1NT"}="HI32 1NT"; $resn_exch{"1HA5 1NT"}="HA51 1NT";
$resn_exch{"2HA5 1NT"}="HA52 1NT"; $resn_exch{"1H2C 1P3"}="H2C1 1P3"; $resn_exch{"2H2C 1P3"}="H2C2 1P3";
$resn_exch{"3H2C 1P3"}="H2C3 1P3"; $resn_exch{"1H20 1P3"}="H201 1P3"; $resn_exch{"2H20 1P3"}="H202 1P3";
$resn_exch{"1H4C 1P3"}="H4C1 1P3"; $resn_exch{"2H4C 1P3"}="H4C2 1P3"; $resn_exch{"1H5C 1P3"}="H5C1 1P3";
$resn_exch{"2H5C 1P3"}="H5C2 1P3"; $resn_exch{"1H6C 1P3"}="H6C1 1P3"; $resn_exch{"2H6C 1P3"}="H6C2 1P3";
$resn_exch{"1H7C 1P3"}="H7C1 1P3"; $resn_exch{"2H7C 1P3"}="H7C2 1P3"; $resn_exch{"3H7C 1P3"}="H7C3 1P3";
$resn_exch{"1HN  1PA"}=" H   1PA"; $resn_exch{"2HN  1PA"}=" H2  1PA"; $resn_exch{"1HB  1PA"}=" HB2 1PA";
$resn_exch{"2HB  1PA"}=" HB3 1PA"; $resn_exch{"1HH  1PA"}=" HH2 1PA"; $resn_exch{"2HH  1PA"}=" HH3 1PA";
$resn_exch{"1HCC 1PB"}="HCC1 1PB"; $resn_exch{"2HCC 1PB"}="HCC2 1PB"; $resn_exch{"1HDC 1PB"}="HDC1 1PB";
$resn_exch{"2HDC 1PB"}="HDC2 1PB"; $resn_exch{"1HP  1PC"}=" HP1 1PC"; $resn_exch{"2HP  1PC"}=" HP2 1PC";
$resn_exch{"1HH  1PC"}=" HH1 1PC"; $resn_exch{"2HH  1PC"}=" HH2 1PC"; $resn_exch{"1HI  1PC"}=" HI1 1PC";
$resn_exch{"2HI  1PC"}=" HI2 1PC"; $resn_exch{"1HJ  1PC"}=" HJ1 1PC"; $resn_exch{"2HJ  1PC"}=" HJ2 1PC";
$resn_exch{"1HK  1PC"}=" HK1 1PC"; $resn_exch{"2HK  1PC"}=" HK2 1PC"; $resn_exch{"1HL  1PC"}=" HL1 1PC";
$resn_exch{"2HL  1PC"}=" HL2 1PC"; $resn_exch{"1HN  1PC"}=" HN1 1PC"; $resn_exch{"2HN  1PC"}=" HN2 1PC";
$resn_exch{"1HO  1PC"}=" HO1 1PC"; $resn_exch{"2HO  1PC"}=" HO2 1PC"; $resn_exch{"1HQ  1PC"}=" HQ1 1PC";
$resn_exch{"2HQ  1PC"}=" HQ2 1PC"; $resn_exch{"1HR  1PC"}=" HR1 1PC"; $resn_exch{"2HR  1PC"}=" HR2 1PC";
$resn_exch{"1H12 1PE"}="H121 1PE"; $resn_exch{"2H12 1PE"}="H122 1PE"; $resn_exch{"1H22 1PE"}="H221 1PE";
$resn_exch{"2H22 1PE"}="H222 1PE"; $resn_exch{"1H13 1PE"}="H131 1PE"; $resn_exch{"2H13 1PE"}="H132 1PE";
$resn_exch{"1H23 1PE"}="H231 1PE"; $resn_exch{"2H23 1PE"}="H232 1PE"; $resn_exch{"1H14 1PE"}="H141 1PE";
$resn_exch{"2H14 1PE"}="H142 1PE"; $resn_exch{"1H24 1PE"}="H241 1PE"; $resn_exch{"2H24 1PE"}="H242 1PE";
$resn_exch{"1H15 1PE"}="H151 1PE"; $resn_exch{"2H15 1PE"}="H152 1PE"; $resn_exch{"1H25 1PE"}="H251 1PE";
$resn_exch{"2H25 1PE"}="H252 1PE"; $resn_exch{"1H16 1PE"}="H161 1PE"; $resn_exch{"2H16 1PE"}="H162 1PE";
$resn_exch{"1H26 1PE"}="H261 1PE"; $resn_exch{"2H26 1PE"}="H262 1PE"; $resn_exch{"1H2  1PG"}=" H21 1PG";
$resn_exch{"2H2  1PG"}=" H22 1PG"; $resn_exch{"1H1  1PG"}=" H11 1PG"; $resn_exch{"2H1  1PG"}=" H12 1PG";
$resn_exch{"3H1  1PG"}=" H13 1PG"; $resn_exch{"1H3  1PG"}=" H31 1PG"; $resn_exch{"2H3  1PG"}=" H32 1PG";
$resn_exch{"1H4  1PG"}=" H41 1PG"; $resn_exch{"2H4  1PG"}=" H42 1PG"; $resn_exch{"1H5  1PG"}=" H51 1PG";
$resn_exch{"2H5  1PG"}=" H52 1PG"; $resn_exch{"1H6  1PG"}=" H61 1PG"; $resn_exch{"2H6  1PG"}=" H62 1PG";
$resn_exch{"1H7  1PG"}=" H71 1PG"; $resn_exch{"2H7  1PG"}=" H72 1PG"; $resn_exch{"1H8  1PG"}=" H81 1PG";
$resn_exch{"2H8  1PG"}=" H82 1PG"; $resn_exch{"1H9  1PG"}=" H91 1PG"; $resn_exch{"2H9  1PG"}=" H92 1PG";
$resn_exch{"1H10 1PG"}="H101 1PG"; $resn_exch{"2H10 1PG"}="H102 1PG"; $resn_exch{"1H11 1PG"}="H111 1PG";
$resn_exch{"2H11 1PG"}="H112 1PG"; $resn_exch{"1H3  1PI"}=" H31 1PI"; $resn_exch{"2H3  1PI"}=" H32 1PI";
$resn_exch{"1HN  1PI"}="HN1A 1PI"; $resn_exch{"2HN  1PI"}=" HN2 1PI"; $resn_exch{"1H17 1PI"}="H171 1PI";
$resn_exch{"2H17 1PI"}="H172 1PI"; $resn_exch{"1HD  1PI"}=" HD1 1PI"; $resn_exch{"2HD  1PI"}=" HD2 1PI";
$resn_exch{"1HG  1PI"}=" HG1 1PI"; $resn_exch{"2HG  1PI"}=" HG2 1PI"; $resn_exch{"1H1  1PI"}=" H11 1PI";
$resn_exch{"2H1  1PI"}=" H12 1PI"; $resn_exch{"1HN2 1PI"}="HN21 1PI"; $resn_exch{"2HN2 1PI"}="HN22 1PI";
$resn_exch{"1H10 1PL"}="H101 1PL"; $resn_exch{"2H10 1PL"}="H102 1PL"; $resn_exch{"3H10 1PL"}="H103 1PL";
$resn_exch{"1H11 1PL"}="H111 1PL"; $resn_exch{"2H11 1PL"}="H112 1PL"; $resn_exch{"1H12 1PL"}="H121 1PL";
$resn_exch{"2H12 1PL"}="H122 1PL"; $resn_exch{"1H3  1PL"}=" H31 1PL"; $resn_exch{"2H3  1PL"}=" H32 1PL";
$resn_exch{"1H14 1PL"}="H141 1PL"; $resn_exch{"2H14 1PL"}="H142 1PL"; $resn_exch{"3H14 1PL"}="H143 1PL";
$resn_exch{"1H15 1PL"}="H151 1PL"; $resn_exch{"2H15 1PL"}="H152 1PL"; $resn_exch{"3H15 1PL"}="H153 1PL";
$resn_exch{"1H1  1PL"}=" H11 1PL"; $resn_exch{"2H1  1PL"}=" H12 1PL"; $resn_exch{"1H30 1PM"}="H301 1PM";
$resn_exch{"2H30 1PM"}="H302 1PM"; $resn_exch{"3H30 1PM"}="H303 1PM"; $resn_exch{"1H31 1PM"}="H311 1PM";
$resn_exch{"2H31 1PM"}="H312 1PM"; $resn_exch{"1H36 1PM"}="H361 1PM"; $resn_exch{"2H36 1PM"}="H362 1PM";
$resn_exch{"1H48 1PM"}="H481 1PM"; $resn_exch{"2H48 1PM"}="H482 1PM"; $resn_exch{"1H12 1PN"}="H121 1PN";
$resn_exch{"2H12 1PN"}="H122 1PN"; $resn_exch{"1H13 1PN"}="H131 1PN"; $resn_exch{"2H13 1PN"}="H132 1PN";
$resn_exch{"1H14 1PN"}="H141 1PN"; $resn_exch{"2H14 1PN"}="H142 1PN"; $resn_exch{"3H14 1PN"}="H143 1PN";
$resn_exch{"1H16 1PN"}="H161 1PN"; $resn_exch{"2H16 1PN"}="H162 1PN"; $resn_exch{"3H16 1PN"}="H163 1PN";
$resn_exch{"1H20 1PN"}="H201 1PN"; $resn_exch{"2H20 1PN"}="H202 1PN"; $resn_exch{"3H20 1PN"}="H203 1PN";
$resn_exch{"1H15 1PP"}="H151 1PP"; $resn_exch{"2H15 1PP"}="H152 1PP"; $resn_exch{"3H15 1PP"}="H153 1PP";
$resn_exch{"1H14 1PP"}="H141 1PP"; $resn_exch{"2H14 1PP"}="H142 1PP"; $resn_exch{"3H14 1PP"}="H143 1PP";
$resn_exch{"1H13 1PP"}="H131 1PP"; $resn_exch{"2H13 1PP"}="H132 1PP"; $resn_exch{"3H13 1PP"}="H133 1PP";
$resn_exch{"1H5' 1PR"}="H5'1 1PR"; $resn_exch{"2H5' 1PR"}="H5'2 1PR"; $resn_exch{"'HO3 1PR"}="HO3' 1PR";
$resn_exch{"'HO2 1PR"}="HO2' 1PR"; $resn_exch{"1HN9 1PR"}="HN91 1PR"; $resn_exch{"2HN9 1PR"}="HN92 1PR";
$resn_exch{"1H7  1PR"}=" H71 1PR"; $resn_exch{"2H7  1PR"}=" H72 1PR"; $resn_exch{"1H\"  1PR"}=" H'1 1PR";
$resn_exch{"2H\"  1PR"}=" H'2 1PR"; $resn_exch{" H2\" 1PR"}=" H2B 1PR"; $resn_exch{" H3\" 1PR"}=" H3B 1PR";
$resn_exch{" H4\" 1PR"}=" H4B 1PR"; $resn_exch{"1H5\" 1PR"}="H5A1 1PR"; $resn_exch{"2H5\" 1PR"}="H5A2 1PR";
$resn_exch{" H8\" 1PR"}=" H8B 1PR"; $resn_exch{" H9\" 1PR"}=" H9B 1PR"; $resn_exch{"1H6  1PS"}=" H61 1PS";
$resn_exch{"2H6  1PS"}=" H62 1PS"; $resn_exch{"1H7  1PS"}=" H71 1PS"; $resn_exch{"2H7  1PS"}=" H72 1PS";
$resn_exch{"1H8  1PS"}=" H81 1PS"; $resn_exch{"2H8  1PS"}=" H82 1PS"; $resn_exch{"1H1  1PT"}=" H11 1PT";
$resn_exch{"2H1  1PT"}=" H12 1PT"; $resn_exch{"1H2  1PT"}=" H21 1PT"; $resn_exch{"2H2  1PT"}=" H22 1PT";
$resn_exch{"1H3  1PT"}=" H31 1PT"; $resn_exch{"2H3  1PT"}=" H32 1PT"; $resn_exch{"1H4  1PT"}=" H41 1PT";
$resn_exch{"2H4  1PT"}=" H42 1PT"; $resn_exch{"1H5  1PT"}=" H51 1PT"; $resn_exch{"2H5  1PT"}=" H52 1PT";
$resn_exch{"1H6  1PT"}=" H61 1PT"; $resn_exch{"2H6  1PT"}=" H62 1PT"; $resn_exch{"1H6  1PU"}=" H61 1PU";
$resn_exch{"2H6  1PU"}=" H62 1PU"; $resn_exch{"1H11 1PU"}="H111 1PU"; $resn_exch{"2H11 1PU"}="H112 1PU";
$resn_exch{"1H12 1PU"}="H121 1PU"; $resn_exch{"2H12 1PU"}="H122 1PU"; $resn_exch{"1HB  1PY"}=" HB1 1PY";
$resn_exch{"2HB  1PY"}=" HB2 1PY"; $resn_exch{"2HOP 1RB"}="HOP2 1RB"; $resn_exch{"3HOP 1RB"}="HOP3 1RB";
$resn_exch{"1H5* 1RB"}="H5'1 1RB"; $resn_exch{"2H5* 1RB"}="H5'2 1RB"; $resn_exch{" H4* 1RB"}=" H4' 1RB";
$resn_exch{" H3* 1RB"}=" H3' 1RB"; $resn_exch{"3HO2 1RB"}="HO23 1RB"; $resn_exch{" H2* 1RB"}=" H2' 1RB";
$resn_exch{"2HO2 1RB"}="HO22 1RB"; $resn_exch{" H1* 1RB"}=" H1' 1RB"; $resn_exch{"1HN1 1SA"}="HN11 1SA";
$resn_exch{"2HN1 1SA"}="HN12 1SA"; $resn_exch{"1HN4 1SA"}="HN41 1SA"; $resn_exch{"2HN4 1SA"}="HN42 1SA";
$resn_exch{"1H5* 1SC"}="H5'1 1SC"; $resn_exch{"2H5* 1SC"}="H5'2 1SC"; $resn_exch{" H4* 1SC"}=" H4' 1SC";
$resn_exch{" H3* 1SC"}=" H3' 1SC"; $resn_exch{"HO3* 1SC"}="HO3' 1SC"; $resn_exch{" H2* 1SC"}=" H2' 1SC";
$resn_exch{"HO2* 1SC"}="HO2' 1SC"; $resn_exch{" H1* 1SC"}=" H1' 1SC"; $resn_exch{"1HN4 1SC"}="HN41 1SC";
$resn_exch{"2HN4 1SC"}="HN42 1SC"; $resn_exch{"1H13 1SM"}="H131 1SM"; $resn_exch{"2H13 1SM"}="H132 1SM";
$resn_exch{"3H13 1SM"}="H133 1SM"; $resn_exch{"1H8' 1SM"}="H8'1 1SM"; $resn_exch{"2H8' 1SM"}="H8'2 1SM";
$resn_exch{"3H8' 1SM"}="H8'3 1SM"; $resn_exch{"1H7' 1SM"}="H7'1 1SM"; $resn_exch{"2H7' 1SM"}="H7'2 1SM";
$resn_exch{"3H7' 1SM"}="H7'3 1SM"; $resn_exch{"1HN1 1SQ"}="HN11 1SQ"; $resn_exch{"2HN1 1SQ"}="HN12 1SQ";
$resn_exch{"1H27 1ST"}="H271 1ST"; $resn_exch{"2H27 1ST"}="H272 1ST"; $resn_exch{"1H32 1ST"}="H321 1ST";
$resn_exch{"2H32 1ST"}="H322 1ST"; $resn_exch{"3H32 1ST"}="H323 1ST"; $resn_exch{"1H34 1ST"}="H341 1ST";
$resn_exch{"2H34 1ST"}="H342 1ST"; $resn_exch{"3H34 1ST"}="H343 1ST"; $resn_exch{"1H35 1ST"}="H351 1ST";
$resn_exch{"2H35 1ST"}="H352 1ST"; $resn_exch{"3H35 1ST"}="H353 1ST"; $resn_exch{"1H13 1TB"}="H131 1TB";
$resn_exch{"2H13 1TB"}="H132 1TB"; $resn_exch{"3H13 1TB"}="H133 1TB"; $resn_exch{"1H10 1TB"}="H101 1TB";
$resn_exch{"2H10 1TB"}="H102 1TB"; $resn_exch{"3H10 1TB"}="H103 1TB"; $resn_exch{"1H5' 1TB"}="H5'1 1TB";
$resn_exch{"2H5' 1TB"}="H5'2 1TB"; $resn_exch{"3H5' 1TB"}="H5'3 1TB"; $resn_exch{"1H7' 1TB"}="H7'1 1TB";
$resn_exch{"2H7' 1TB"}="H7'2 1TB"; $resn_exch{"3H7' 1TB"}="H7'3 1TB"; $resn_exch{"1H2A 1TP"}="H2A1 1TP";
$resn_exch{"2H2A 1TP"}="H2A2 1TP"; $resn_exch{"3H2A 1TP"}="H2A3 1TP"; $resn_exch{"1H4* 1TP"}="H4'1 1TP";
$resn_exch{"2H4* 1TP"}="H4'2 1TP"; $resn_exch{" H6* 1TP"}=" H6' 1TP"; $resn_exch{"1H7* 1TP"}="H7'1 1TP";
$resn_exch{"2H7* 1TP"}="H7'2 1TP"; $resn_exch{"1H4A 1TP"}="H4A1 1TP"; $resn_exch{"2H4A 1TP"}="H4A2 1TP";
$resn_exch{"3H4A 1TP"}="H4A3 1TP"; $resn_exch{"1H5A 1TP"}="H5A1 1TP"; $resn_exch{"2H5A 1TP"}="H5A2 1TP";
$resn_exch{"1H5B 1TP"}="H5B1 1TP"; $resn_exch{"2H5B 1TP"}="H5B2 1TP"; $resn_exch{"3HO1 1TP"}="HO13 1TP";
$resn_exch{"1H6B 1TP"}="H6B1 1TP"; $resn_exch{"3HO2 1TP"}="HO23 1TP"; $resn_exch{"1H35 1TP"}="H351 1TP";
$resn_exch{"2H35 1TP"}="H352 1TP"; $resn_exch{"3H35 1TP"}="H353 1TP"; $resn_exch{"1HN  1TQ"}=" H   1TQ";
$resn_exch{"2HN  1TQ"}=" H2  1TQ"; $resn_exch{"1HB  1TQ"}=" HB2 1TQ"; $resn_exch{"2HB  1TQ"}=" HB3 1TQ";
$resn_exch{"HNE1 1TQ"}=" HE1 1TQ"; $resn_exch{"HOXT 1TQ"}=" HXT 1TQ"; $resn_exch{"1HN  1TY"}=" H   1TY";
$resn_exch{"2HN  1TY"}=" H2  1TY"; $resn_exch{"1HB  1TY"}=" HB2 1TY"; $resn_exch{"2HB  1TY"}=" HB3 1TY";
$resn_exch{" HOH 1TY"}=" HH  1TY"; $resn_exch{"1H2  1TY"}=" H21 1TY"; $resn_exch{"2H2  1TY"}=" H22 1TY";
$resn_exch{"1H1  1TY"}=" H11 1TY"; $resn_exch{"2H1  1TY"}=" H12 1TY"; $resn_exch{"1H3  1UN"}="H31A 1UN";
$resn_exch{"2H3  1UN"}="H32A 1UN"; $resn_exch{"1H4  1UN"}=" H41 1UN"; $resn_exch{"2H4  1UN"}=" H42 1UN";
$resn_exch{"1H5  1UN"}=" H51 1UN"; $resn_exch{"2H5  1UN"}=" H52 1UN"; $resn_exch{"1H6  1UN"}=" H61 1UN";
$resn_exch{"2H6  1UN"}=" H62 1UN"; $resn_exch{"1H9  1UN"}=" H91 1UN"; $resn_exch{"2H9  1UN"}=" H92 1UN";
$resn_exch{"1H10 1UN"}="H101 1UN"; $resn_exch{"2H10 1UN"}="H102 1UN"; $resn_exch{"1H14 1UN"}="H141 1UN";
$resn_exch{"2H14 1UN"}="H142 1UN"; $resn_exch{"3H14 1UN"}="H143 1UN"; $resn_exch{"1H15 1UN"}="H151 1UN";
$resn_exch{"2H15 1UN"}="H152 1UN"; $resn_exch{"3H15 1UN"}="H153 1UN"; $resn_exch{"1H16 1UN"}="H161 1UN";
$resn_exch{"2H16 1UN"}="H162 1UN"; $resn_exch{"3H16 1UN"}="H163 1UN"; $resn_exch{"1H18 1UN"}="H181 1UN";
$resn_exch{"2H18 1UN"}="H182 1UN"; $resn_exch{"1H23 1UN"}="H231 1UN"; $resn_exch{"2H23 1UN"}="H232 1UN";
$resn_exch{"1H39 1UN"}="H391 1UN"; $resn_exch{"2H39 1UN"}="H392 1UN"; $resn_exch{"3H39 1UN"}="H393 1UN";
$resn_exch{"1HN  200"}=" H2  200"; $resn_exch{"1HB  200"}=" HB2 200"; $resn_exch{"2HB  200"}=" HB3 200";
$resn_exch{"1HD1 200"}=" HD1 200"; $resn_exch{"1HD2 200"}=" HD2 200"; $resn_exch{"1HE1 200"}=" HE1 200";
$resn_exch{"1HE2 200"}=" HE2 200"; $resn_exch{"1HN1 201"}="HN11 201"; $resn_exch{"2HN1 201"}="HN12 201";
$resn_exch{"1H1  204"}=" H11 204"; $resn_exch{"2H1  204"}=" H12 204"; $resn_exch{"3H1  204"}="H13A 204";
$resn_exch{"1H2  204"}=" H21 204"; $resn_exch{"2H2  204"}=" H22 204"; $resn_exch{"1H10 204"}="H101 204";
$resn_exch{"2H10 204"}="H102 204"; $resn_exch{"1H9C 207"}="H9C1 207"; $resn_exch{"2H9C 207"}="H9C2 207";
$resn_exch{"1H2N 207"}="H2N1 207"; $resn_exch{"2H2N 207"}="H2N2 207"; $resn_exch{"1H11 207"}="H111 207";
$resn_exch{"2H11 207"}="H112 207"; $resn_exch{"1H12 207"}="H121 207"; $resn_exch{"2H12 207"}="H122 207";
$resn_exch{"1H13 207"}="H131 207"; $resn_exch{"2H13 207"}="H132 207"; $resn_exch{"1H14 207"}="H141 207";
$resn_exch{"2H14 207"}="H142 207"; $resn_exch{"1H15 207"}="H151 207"; $resn_exch{"2H15 207"}="H152 207";
$resn_exch{"1H30 208"}="H301 208"; $resn_exch{"2H30 208"}="H302 208"; $resn_exch{"3H30 208"}="H303 208";
$resn_exch{"1H28 208"}="H281 208"; $resn_exch{"2H28 208"}="H282 208"; $resn_exch{"1H19 208"}="H191 208";
$resn_exch{"2H19 208"}="H192 208"; $resn_exch{"1H9  208"}=" H91 208"; $resn_exch{"2H9  208"}=" H92 208";
$resn_exch{"1H26 208"}="H261 208"; $resn_exch{"2H26 208"}="H262 208"; $resn_exch{"3H26 208"}="H263 208";
$resn_exch{"1H10 209"}="H101 209"; $resn_exch{"2H10 209"}="H102 209"; $resn_exch{"1H11 209"}="H111 209";
$resn_exch{"2H11 209"}="H112 209"; $resn_exch{"3H11 209"}="H113 209"; $resn_exch{"1H15 209"}="H151 209";
$resn_exch{"2H15 209"}="H152 209"; $resn_exch{"3H15 209"}="H153 209"; $resn_exch{"1H19 20E"}="H191 20E";
$resn_exch{"2H19 20E"}="H192 20E"; $resn_exch{"3H19 20E"}="H193 20E"; $resn_exch{"1H1  20E"}=" H11 20E";
$resn_exch{"2H1  20E"}=" H12 20E"; $resn_exch{"1H4  20E"}=" H41 20E"; $resn_exch{"2H4  20E"}=" H42 20E";
$resn_exch{"1H11 20E"}="H111 20E"; $resn_exch{"2H11 20E"}="H112 20E"; $resn_exch{"1H12 20E"}="H121 20E";
$resn_exch{"2H12 20E"}="H122 20E"; $resn_exch{"1H18 20E"}="H181 20E"; $resn_exch{"2H18 20E"}="H182 20E";
$resn_exch{"3H18 20E"}="H183 20E"; $resn_exch{"1H15 20E"}="H151 20E"; $resn_exch{"2H15 20E"}="H152 20E";
$resn_exch{"1H16 20E"}="H161 20E"; $resn_exch{"2H16 20E"}="H162 20E"; $resn_exch{"1H21 20E"}="H211 20E";
$resn_exch{"2H21 20E"}="H212 20E"; $resn_exch{"3H21 20E"}="H213 20E"; $resn_exch{"1H23 20E"}="H231 20E";
$resn_exch{"2H23 20E"}="H232 20E"; $resn_exch{"1H24 20E"}="H241 20E"; $resn_exch{"2H24 20E"}="H242 20E";
$resn_exch{"1H26 20E"}="H261 20E"; $resn_exch{"2H26 20E"}="H262 20E"; $resn_exch{"3H26 20E"}="H263 20E";
$resn_exch{"1H27 20E"}="H271 20E"; $resn_exch{"2H27 20E"}="H272 20E"; $resn_exch{"3H27 20E"}="H273 20E";
$resn_exch{"1H7  20S"}=" H71 20S"; $resn_exch{"2H7  20S"}=" H72 20S"; $resn_exch{"1H8  20S"}=" H81 20S";
$resn_exch{"2H8  20S"}=" H82 20S"; $resn_exch{"1H9  20S"}=" H91 20S"; $resn_exch{"2H9  20S"}=" H92 20S";
$resn_exch{"1H10 20S"}="H101 20S"; $resn_exch{"2H10 20S"}="H102 20S"; $resn_exch{"1H17 20S"}="H171 20S";
$resn_exch{"2H17 20S"}="H172 20S"; $resn_exch{"1H18 20S"}="H181 20S"; $resn_exch{"2H18 20S"}="H182 20S";
$resn_exch{"1H32 20S"}="H321 20S"; $resn_exch{"2H32 20S"}="H322 20S"; $resn_exch{"1H31 20S"}="H311 20S";
$resn_exch{"2H31 20S"}="H312 20S"; $resn_exch{"3H31 20S"}="H313 20S"; $resn_exch{"1H16 210"}="H161 210";
$resn_exch{"2H16 210"}="H162 210"; $resn_exch{"1H19 210"}="H191 210"; $resn_exch{"2H19 210"}="H192 210";
$resn_exch{"1H22 210"}="H221 210"; $resn_exch{"2H22 210"}="H222 210"; $resn_exch{"1H1  211"}=" H11 211";
$resn_exch{"2H1  211"}=" H12 211"; $resn_exch{"1H2  211"}=" H21 211"; $resn_exch{"2H2  211"}=" H22 211";
$resn_exch{"1H3  211"}=" H31 211"; $resn_exch{"2H3  211"}=" H32 211"; $resn_exch{"1H4  211"}=" H41 211";
$resn_exch{"2H4  211"}=" H42 211"; $resn_exch{"1H5  211"}=" H51 211"; $resn_exch{"2H5  211"}=" H52 211";
$resn_exch{"1H6  211"}=" H61 211"; $resn_exch{"2H6  211"}=" H62 211"; $resn_exch{"1H32 213"}="H321 213";
$resn_exch{"2H32 213"}="H322 213"; $resn_exch{"1H35 213"}="H351 213"; $resn_exch{"2H35 213"}="H352 213";
$resn_exch{"1H32 214"}="H321 214"; $resn_exch{"2H32 214"}="H322 214"; $resn_exch{"1H35 214"}="H351 214";
$resn_exch{"2H35 214"}="H352 214"; $resn_exch{"1H71 214"}="H711 214"; $resn_exch{"2H71 214"}="H712 214";
$resn_exch{"3H71 214"}="H713 214"; $resn_exch{"1H23 215"}="H231 215"; $resn_exch{"2H23 215"}="H232 215";
$resn_exch{"1H31 215"}="H311 215"; $resn_exch{"2H31 215"}="H312 215"; $resn_exch{"1H30 215"}="H301 215";
$resn_exch{"2H30 215"}="H302 215"; $resn_exch{"1H24 215"}="H241 215"; $resn_exch{"2H24 215"}="H242 215";
$resn_exch{"1H27 215"}="H271 215"; $resn_exch{"2H27 215"}="H272 215"; $resn_exch{"3H27 215"}="H273 215";
$resn_exch{"1H26 215"}="H261 215"; $resn_exch{"2H26 215"}="H262 215"; $resn_exch{"3H26 215"}="H263 215";
$resn_exch{"1H21 216"}="H211 216"; $resn_exch{"2H21 216"}="H212 216"; $resn_exch{"1H23 216"}="H231 216";
$resn_exch{"2H23 216"}="H232 216"; $resn_exch{"1H31 216"}="H311 216"; $resn_exch{"2H31 216"}="H312 216";
$resn_exch{"1H61 216"}="H611 216"; $resn_exch{"2H61 216"}="H612 216"; $resn_exch{"1H71 216"}="H711 216";
$resn_exch{"2H71 216"}="H712 216"; $resn_exch{"1H73 216"}="H731 216"; $resn_exch{"2H73 216"}="H732 216";
$resn_exch{"1H1  217"}=" H11 217"; $resn_exch{"2H1  217"}=" H12 217"; $resn_exch{"1H3  217"}=" H31 217";
$resn_exch{"2H3  217"}=" H32 217"; $resn_exch{"1H4  217"}=" H41 217"; $resn_exch{"2H4  217"}=" H42 217";
$resn_exch{"1H6  217"}=" H61 217"; $resn_exch{"2H6  217"}=" H62 217"; $resn_exch{"1HM2 218"}="HM21 218";
$resn_exch{"2HM2 218"}="HM22 218"; $resn_exch{"3HM2 218"}="HM23 218"; $resn_exch{"1H7A 218"}="H7A1 218";
$resn_exch{"2H7A 218"}="H7A2 218"; $resn_exch{"1HM6 218"}="HM61 218"; $resn_exch{"2HM6 218"}="HM62 218";
$resn_exch{"1HM7 218"}="HM71 218"; $resn_exch{"2HM7 218"}="HM72 218"; $resn_exch{"1HM4 218"}="HM41 218";
$resn_exch{"2HM4 218"}="HM42 218"; $resn_exch{"3HM4 218"}="HM43 218"; $resn_exch{"1H4A 218"}="H4A1 218";
$resn_exch{"2H4A 218"}="H4A2 218"; $resn_exch{"1H12 219"}="H121 219"; $resn_exch{"2H12 219"}="H122 219";
$resn_exch{"1H24 219"}="H241 219"; $resn_exch{"2H24 219"}="H242 219"; $resn_exch{"1H25 219"}="H251 219";
$resn_exch{"2H25 219"}="H252 219"; $resn_exch{"1H26 219"}="H261 219"; $resn_exch{"2H26 219"}="H262 219";
$resn_exch{"1H27 219"}="H271 219"; $resn_exch{"2H27 219"}="H272 219"; $resn_exch{"1H28 219"}="H281 219";
$resn_exch{"2H28 219"}="H282 219"; $resn_exch{"1H2  220"}=" H21 220"; $resn_exch{"2H2  220"}=" H22 220";
$resn_exch{"1H3  220"}=" H31 220"; $resn_exch{"2H3  220"}=" H32 220"; $resn_exch{"1H4  220"}=" H41 220";
$resn_exch{"2H4  220"}=" H42 220"; $resn_exch{"1H7  220"}=" H71 220"; $resn_exch{"2H7  220"}=" H72 220";
$resn_exch{"1H8  220"}=" H81 220"; $resn_exch{"2H8  220"}=" H82 220"; $resn_exch{"1H11 220"}="H111 220";
$resn_exch{"2H11 220"}="H112 220"; $resn_exch{"1H12 220"}="H121 220"; $resn_exch{"2H12 220"}="H122 220";
$resn_exch{"1H13 221"}="H131 221"; $resn_exch{"2H13 221"}="H132 221"; $resn_exch{"1H22 222"}="H221 222";
$resn_exch{"2H22 222"}="H222 222"; $resn_exch{"1H24 222"}="H241 222"; $resn_exch{"2H24 222"}="H242 222";
$resn_exch{"1H27 222"}="H271 222"; $resn_exch{"2H27 222"}="H272 222"; $resn_exch{"1HN6 223"}="HN61 223";
$resn_exch{"2HN6 223"}="HN62 223"; $resn_exch{"1H10 223"}="H101 223"; $resn_exch{"2H10 223"}="H102 223";
$resn_exch{"1H6* 223"}="H6'1 223"; $resn_exch{"2H6* 223"}="H6'2 223"; $resn_exch{" H4* 223"}=" H4' 223";
$resn_exch{"1H5* 223"}="H5'1 223"; $resn_exch{"2H5* 223"}="H5'2 223"; $resn_exch{"HO5* 223"}="HO5' 223";
$resn_exch{" H3* 223"}=" H3' 223"; $resn_exch{"HO3* 223"}="HO3' 223"; $resn_exch{"1H2* 223"}="H2'1 223";
$resn_exch{"2H2* 223"}="H2'2 223"; $resn_exch{"1H4  224"}=" H41 224"; $resn_exch{"2H4  224"}=" H42 224";
$resn_exch{"1H6  224"}=" H61 224"; $resn_exch{"2H6  224"}=" H62 224"; $resn_exch{"1H7  224"}=" H71 224";
$resn_exch{"2H7  224"}=" H72 224"; $resn_exch{"1H8  224"}=" H81 224"; $resn_exch{"2H8  224"}=" H82 224";
$resn_exch{"1H9  224"}=" H91 224"; $resn_exch{"2H9  224"}=" H92 224"; $resn_exch{"1H15 225"}="H151 225";
$resn_exch{"2H15 225"}="H152 225"; $resn_exch{"3H15 225"}="H153 225"; $resn_exch{"1H14 225"}="H141 225";
$resn_exch{"2H14 225"}="H142 225"; $resn_exch{"1H12 225"}="H121 225"; $resn_exch{"2H12 225"}="H122 225";
$resn_exch{"3H12 225"}="H123 225"; $resn_exch{"1H18 225"}="H181 225"; $resn_exch{"2H18 225"}="H182 225";
$resn_exch{"3H18 225"}="H183 225"; $resn_exch{"1H17 225"}="H171 225"; $resn_exch{"2H17 225"}="H172 225";
$resn_exch{"3H17 225"}="H173 225"; $resn_exch{"1H34 227"}="H341 227"; $resn_exch{"2H34 227"}="H342 227";
$resn_exch{"3H34 227"}="H343 227"; $resn_exch{"1H32 227"}="H321 227"; $resn_exch{"2H32 227"}="H322 227";
$resn_exch{"1H21 227"}="H211 227"; $resn_exch{"2H21 227"}="H212 227"; $resn_exch{"3H21 227"}="H213 227";
$resn_exch{"1H18 227"}="H181 227"; $resn_exch{"2H18 227"}="H182 227"; $resn_exch{"3H18 227"}="H183 227";
$resn_exch{"1H16 227"}="H161 227"; $resn_exch{"2H16 227"}="H162 227"; $resn_exch{"1H30 227"}="H301 227";
$resn_exch{"2H30 227"}="H302 227"; $resn_exch{"1H31 227"}="H311 227"; $resn_exch{"2H31 227"}="H312 227";
$resn_exch{"1H29 227"}="H291 227"; $resn_exch{"2H29 227"}="H292 227"; $resn_exch{"1H10 227"}="H101 227";
$resn_exch{"2H10 227"}="H102 227"; $resn_exch{"1H11 227"}="H111 227"; $resn_exch{"2H11 227"}="H112 227";
$resn_exch{"1H30 228"}="H301 228"; $resn_exch{"2H30 228"}="H302 228"; $resn_exch{"1H22 228"}="H221 228";
$resn_exch{"2H22 228"}="H222 228"; $resn_exch{"1H21 228"}="H211 228"; $resn_exch{"2H21 228"}="H212 228";
$resn_exch{"1H16 228"}="H161 228"; $resn_exch{"2H16 228"}="H162 228"; $resn_exch{"1H15 228"}="H151 228";
$resn_exch{"2H15 228"}="H152 228"; $resn_exch{"1H14 228"}="H141 228"; $resn_exch{"2H14 228"}="H142 228";
$resn_exch{"1H12 228"}="H121 228"; $resn_exch{"2H12 228"}="H122 228"; $resn_exch{"3H12 228"}="H123 228";
$resn_exch{"1H50 22B"}="H501 22B"; $resn_exch{"2H50 22B"}="H502 22B"; $resn_exch{"3H50 22B"}="H503 22B";
$resn_exch{"1H20 22B"}="H201 22B"; $resn_exch{"2H20 22B"}="H202 22B"; $resn_exch{"3H20 22B"}="H203 22B";
$resn_exch{"1H19 22B"}="H191 22B"; $resn_exch{"2H19 22B"}="H192 22B"; $resn_exch{"3H19 22B"}="H193 22B";
$resn_exch{"1H18 22B"}="H181 22B"; $resn_exch{"2H18 22B"}="H182 22B"; $resn_exch{"3H18 22B"}="H183 22B";
$resn_exch{"1H21 22B"}="H211 22B"; $resn_exch{"2H21 22B"}="H212 22B"; $resn_exch{"1H22 22B"}="H221 22B";
$resn_exch{"2H22 22B"}="H222 22B"; $resn_exch{"1H24 22B"}="H241 22B"; $resn_exch{"2H24 22B"}="H242 22B";
$resn_exch{"3H24 22B"}="H243 22B"; $resn_exch{"1H25 22B"}="H251 22B"; $resn_exch{"2H25 22B"}="H252 22B";
$resn_exch{"3H25 22B"}="H253 22B"; $resn_exch{"1H16 22B"}="H161 22B"; $resn_exch{"2H16 22B"}="H162 22B";
$resn_exch{"3H16 22B"}="H163 22B"; $resn_exch{"1H17 22B"}="H171 22B"; $resn_exch{"2H17 22B"}="H172 22B";
$resn_exch{"3H17 22B"}="H173 22B"; $resn_exch{"1H26 22B"}="H261 22B"; $resn_exch{"2H26 22B"}="H262 22B";
$resn_exch{"3H26 22B"}="H263 22B"; $resn_exch{"1H30 22B"}="H301 22B"; $resn_exch{"2H30 22B"}="H302 22B";
$resn_exch{"3H30 22B"}="H303 22B"; $resn_exch{"1H34 22B"}="H341 22B"; $resn_exch{"2H34 22B"}="H342 22B";
$resn_exch{"1H35 22B"}="H351 22B"; $resn_exch{"2H35 22B"}="H352 22B"; $resn_exch{"1H37 22B"}="H371 22B";
$resn_exch{"2H37 22B"}="H372 22B"; $resn_exch{"3H37 22B"}="H373 22B"; $resn_exch{"1H46 22B"}="H461 22B";
$resn_exch{"2H46 22B"}="H462 22B"; $resn_exch{"3H46 22B"}="H463 22B"; $resn_exch{"1H48 22B"}="H481 22B";
$resn_exch{"2H48 22B"}="H482 22B"; $resn_exch{"3H48 22B"}="H483 22B"; $resn_exch{"1H49 22B"}="H491 22B";
$resn_exch{"2H49 22B"}="H492 22B"; $resn_exch{"3H49 22B"}="H493 22B"; $resn_exch{"1HN5 22M"}="HN51 22M";
$resn_exch{"2HN5 22M"}="HN52 22M"; $resn_exch{"1H18 22M"}="H181 22M"; $resn_exch{"2H18 22M"}="H182 22M";
$resn_exch{"3H18 22M"}="H183 22M"; $resn_exch{"1H20 230"}="H201 230"; $resn_exch{"2H20 230"}="H202 230";
$resn_exch{"1H22 230"}="H221 230"; $resn_exch{"2H22 230"}="H222 230"; $resn_exch{"1H24 230"}="H241 230";
$resn_exch{"2H24 230"}="H242 230"; $resn_exch{"3H24 230"}="H243 230"; $resn_exch{"1H19 232"}="H191 232";
$resn_exch{"2H19 232"}="H192 232"; $resn_exch{"1H18 232"}="H181 232"; $resn_exch{"2H18 232"}="H182 232";
$resn_exch{"3H18 232"}="H183 232"; $resn_exch{"1H20 232"}="H201 232"; $resn_exch{"2H20 232"}="H202 232";
$resn_exch{"1H22 232"}="H221 232"; $resn_exch{"2H22 232"}="H222 232"; $resn_exch{"1H23 232"}="H231 232";
$resn_exch{"2H23 232"}="H232 232"; $resn_exch{"1H25 232"}="H251 232"; $resn_exch{"2H25 232"}="H252 232";
$resn_exch{"1H26 232"}="H261 232"; $resn_exch{"2H26 232"}="H262 232"; $resn_exch{"1H27 232"}="H271 232";
$resn_exch{"2H27 232"}="H272 232"; $resn_exch{"1H28 232"}="H281 232"; $resn_exch{"2H28 232"}="H282 232";
$resn_exch{"3H19 232"}="H193 232"; $resn_exch{"4H19 232"}="H194 232"; $resn_exch{"4H18 232"}="H184 232";
$resn_exch{"5H18 232"}="H185 232"; $resn_exch{"6H18 232"}="H186 232"; $resn_exch{"3H20 232"}="H203 232";
$resn_exch{"4H20 232"}="H204 232"; $resn_exch{"3H22 232"}="H223 232"; $resn_exch{"4H22 232"}="H224 232";
$resn_exch{"3H23 232"}="H233 232"; $resn_exch{"4H23 232"}="H234 232"; $resn_exch{"3H25 232"}="H253 232";
$resn_exch{"4H25 232"}="H254 232"; $resn_exch{"3H26 232"}="H263 232"; $resn_exch{"4H26 232"}="H264 232";
$resn_exch{"3H27 232"}="H273 232"; $resn_exch{"4H27 232"}="H274 232"; $resn_exch{"1H39 234"}="H391 234";
$resn_exch{"2H39 234"}="H392 234"; $resn_exch{"3H39 234"}="H393 234"; $resn_exch{"1H40 234"}="H401 234";
$resn_exch{"2H40 234"}="H402 234"; $resn_exch{"1H30 235"}="H301 235"; $resn_exch{"2H30 235"}="H302 235";
$resn_exch{"3H30 235"}="H303 235"; $resn_exch{"1H29 235"}="H291 235"; $resn_exch{"2H29 235"}="H292 235";
$resn_exch{"3H29 235"}="H293 235"; $resn_exch{"1H15 235"}="H151 235"; $resn_exch{"2H15 235"}="H152 235";
$resn_exch{"1H9  235"}=" H91 235"; $resn_exch{"2H9  235"}=" H92 235"; $resn_exch{"1H10 235"}="H101 235";
$resn_exch{"2H10 235"}="H102 235"; $resn_exch{"1H20 237"}="H201 237"; $resn_exch{"2H20 237"}="H202 237";
$resn_exch{"3H20 237"}="H203 237"; $resn_exch{"1H9  237"}=" H91 237"; $resn_exch{"2H9  237"}=" H92 237";
$resn_exch{"1H11 237"}="H111 237"; $resn_exch{"2H11 237"}="H112 237"; $resn_exch{"1H19 237"}="H191 237";
$resn_exch{"2H19 237"}="H192 237"; $resn_exch{"3H19 237"}="H193 237"; $resn_exch{"1H19 239"}="H191 239";
$resn_exch{"2H19 239"}="H192 239"; $resn_exch{"1H37 239"}="H371 239"; $resn_exch{"2H37 239"}="H372 239";
$resn_exch{"1H38 239"}="H381 239"; $resn_exch{"2H38 239"}="H382 239"; $resn_exch{"1HAZ 23D"}="HAZ1 23D";
$resn_exch{"2HAZ 23D"}="HAZ2 23D"; $resn_exch{"1HBA 23D"}="HBA1 23D"; $resn_exch{"2HBA 23D"}="HBA2 23D";
$resn_exch{"3HBA 23D"}="HBA3 23D"; $resn_exch{"1HAF 23D"}="HAF1 23D"; $resn_exch{"2HAF 23D"}="HAF2 23D";
$resn_exch{"1HAE 23D"}="HAE1 23D"; $resn_exch{"2HAE 23D"}="HAE2 23D"; $resn_exch{"1HAD 23D"}="HAD1 23D";
$resn_exch{"2HAD 23D"}="HAD2 23D"; $resn_exch{"1HAC 23D"}="HAC1 23D"; $resn_exch{"2HAC 23D"}="HAC2 23D";
$resn_exch{"1HAB 23D"}="HAB1 23D"; $resn_exch{"2HAB 23D"}="HAB2 23D"; $resn_exch{" HN  23F"}=" H   23F";
$resn_exch{" HN2 23F"}=" H2  23F"; $resn_exch{"2HB  23F"}=" HB  23F"; $resn_exch{"1H20 23I"}="H201 23I";
$resn_exch{"2H20 23I"}="H202 23I"; $resn_exch{"3H20 23I"}="H203 23I"; $resn_exch{"1H33 23I"}="H331 23I";
$resn_exch{"2H33 23I"}="H332 23I"; $resn_exch{"1H35 23I"}="H351 23I"; $resn_exch{"2H35 23I"}="H352 23I";
$resn_exch{"3H35 23I"}="H353 23I"; $resn_exch{"1H18 23I"}="H181 23I"; $resn_exch{"2H18 23I"}="H182 23I";
$resn_exch{"3H18 23I"}="H183 23I"; $resn_exch{"1H29 23I"}="H291 23I"; $resn_exch{"2H29 23I"}="H292 23I";
$resn_exch{"3H29 23I"}="H293 23I"; $resn_exch{"1H36 23I"}="H361 23I"; $resn_exch{"2H36 23I"}="H362 23I";
$resn_exch{"3H36 23I"}="H363 23I"; $resn_exch{"1H37 23I"}="H371 23I"; $resn_exch{"2H37 23I"}="H372 23I";
$resn_exch{"1H39 23I"}="H391 23I"; $resn_exch{"2H39 23I"}="H392 23I"; $resn_exch{"3H39 23I"}="H393 23I";
$resn_exch{"1H43 23I"}="H431 23I"; $resn_exch{"2H43 23I"}="H432 23I"; $resn_exch{"3H43 23I"}="H433 23I";
$resn_exch{"1H44 23I"}="H441 23I"; $resn_exch{"2H44 23I"}="H442 23I"; $resn_exch{"3H44 23I"}="H443 23I";
$resn_exch{"1H53 23I"}="H531 23I"; $resn_exch{"2H53 23I"}="H532 23I"; $resn_exch{"3H53 23I"}="H533 23I";
$resn_exch{"1H52 23I"}="H521 23I"; $resn_exch{"2H52 23I"}="H522 23I"; $resn_exch{"3H52 23I"}="H523 23I";
$resn_exch{"1H12 23M"}="H121 23M"; $resn_exch{"2H12 23M"}="H122 23M"; $resn_exch{"3H12 23M"}="H123 23M";
$resn_exch{"1H19 23M"}="H191 23M"; $resn_exch{"2H19 23M"}="H192 23M"; $resn_exch{"1HAJ 23N"}="HAJ1 23N";
$resn_exch{"2HAJ 23N"}="HAJ2 23N"; $resn_exch{"1HAK 23N"}="HAK1 23N"; $resn_exch{"2HAK 23N"}="HAK2 23N";
$resn_exch{"1HB  23S"}=" HB1 23S"; $resn_exch{"2HB  23S"}=" HB2 23S"; $resn_exch{"1H5* 23T"}="H5'1 23T";
$resn_exch{"2H5* 23T"}="H5'2 23T"; $resn_exch{" H4* 23T"}=" H4' 23T"; $resn_exch{" H1* 23T"}=" H1' 23T";
$resn_exch{"1H5M 23T"}="H5M1 23T"; $resn_exch{"2H5M 23T"}="H5M2 23T"; $resn_exch{"3H5M 23T"}="H5M3 23T";
$resn_exch{"1H2* 23T"}="H2'1 23T"; $resn_exch{"2H2* 23T"}="H2'2 23T"; $resn_exch{"1H3* 23T"}="H3'1 23T";
$resn_exch{"2H3* 23T"}="H3'2 23T"; $resn_exch{"1HAC 240"}="HAC1 240"; $resn_exch{"2HAC 240"}="HAC2 240";
$resn_exch{"3HAC 240"}="HAC3 240"; $resn_exch{"1HAU 240"}="HAU1 240"; $resn_exch{"2HAU 240"}="HAU2 240";
$resn_exch{"1HAB 240"}="HAB1 240"; $resn_exch{"2HAB 240"}="HAB2 240"; $resn_exch{"3HAB 240"}="HAB3 240";
$resn_exch{"1HAA 240"}="HAA1 240"; $resn_exch{"2HAA 240"}="HAA2 240"; $resn_exch{"3HAA 240"}="HAA3 240";
$resn_exch{"1HAA 241"}="HAA1 241"; $resn_exch{"2HAA 241"}="HAA2 241"; $resn_exch{"3HAA 241"}="HAA3 241";
$resn_exch{"1HAB 241"}="HAB1 241"; $resn_exch{"2HAB 241"}="HAB2 241"; $resn_exch{"3HAB 241"}="HAB3 241";
$resn_exch{"1HAU 241"}="HAU1 241"; $resn_exch{"2HAU 241"}="HAU2 241"; $resn_exch{"1HAC 241"}="HAC1 241";
$resn_exch{"2HAC 241"}="HAC2 241"; $resn_exch{"3HAC 241"}="HAC3 241"; $resn_exch{"1H15 242"}="H151 242";
$resn_exch{"2H15 242"}="H152 242"; $resn_exch{"1H25 242"}="H251 242"; $resn_exch{"2H25 242"}="H252 242";
$resn_exch{"3H25 242"}="H253 242"; $resn_exch{"1H13 244"}="H131 244"; $resn_exch{"2H13 244"}="H132 244";
$resn_exch{"1H0A 24A"}="H0A1 24A"; $resn_exch{"2H0A 24A"}="H0A2 24A"; $resn_exch{"3H0A 24A"}="H0A3 24A";
$resn_exch{"1H1A 24A"}="H1A1 24A"; $resn_exch{"2H1A 24A"}="H1A2 24A"; $resn_exch{"1HAB 24A"}="HAB1 24A";
$resn_exch{"2HAB 24A"}="HAB2 24A"; $resn_exch{"1HS1 24A"}="HS11 24A"; $resn_exch{"2HS1 24A"}="HS12 24A";
$resn_exch{"1HS2 24A"}="HS21 24A"; $resn_exch{"2HS2 24A"}="HS22 24A"; $resn_exch{"1HS3 24A"}="HS31 24A";
$resn_exch{"2HS3 24A"}="HS32 24A"; $resn_exch{"1HS5 24A"}="HS51 24A"; $resn_exch{"2HS5 24A"}="HS52 24A";
$resn_exch{"1HS6 24A"}="HS61 24A"; $resn_exch{"2HS6 24A"}="HS62 24A"; $resn_exch{"1H7  24B"}=" H71 24B";
$resn_exch{"2H7  24B"}=" H72 24B"; $resn_exch{"1H13 24I"}="H131 24I"; $resn_exch{"2H13 24I"}="H132 24I";
$resn_exch{"1H12 24I"}="H121 24I"; $resn_exch{"2H12 24I"}="H122 24I"; $resn_exch{"1H7C 24I"}="H7C1 24I";
$resn_exch{"2H7C 24I"}="H7C2 24I"; $resn_exch{"1H6C 24I"}="H6C1 24I"; $resn_exch{"2H6C 24I"}="H6C2 24I";
$resn_exch{"1HN  24T"}=" HN1 24T"; $resn_exch{"2HN  24T"}=" HN2 24T"; $resn_exch{"1HM  24T"}=" HM1 24T";
$resn_exch{"2HM  24T"}=" HM2 24T"; $resn_exch{"3HM  24T"}=" HM3 24T"; $resn_exch{"1HN4 24X"}="HN41 24X";
$resn_exch{"2HN4 24X"}="HN42 24X"; $resn_exch{"1H7  24X"}=" H71 24X"; $resn_exch{"2H7  24X"}=" H72 24X";
$resn_exch{"1H16 24X"}="H161 24X"; $resn_exch{"2H16 24X"}="H162 24X"; $resn_exch{"1H11 24X"}="H111 24X";
$resn_exch{"2H11 24X"}="H112 24X"; $resn_exch{"1H12 24X"}="H121 24X"; $resn_exch{"2H12 24X"}="H122 24X";
$resn_exch{"3H12 24X"}="H123 24X"; $resn_exch{"1H13 24X"}="H131 24X"; $resn_exch{"2H13 24X"}="H132 24X";
$resn_exch{"3H13 24X"}="H133 24X"; $resn_exch{"1H17 254"}="H171 254"; $resn_exch{"2H17 254"}="H172 254";
$resn_exch{"1H18 254"}="H181 254"; $resn_exch{"2H18 254"}="H182 254"; $resn_exch{"1H23 254"}="H231 254";
$resn_exch{"2H23 254"}="H232 254"; $resn_exch{"3H23 254"}="H233 254"; $resn_exch{"1H24 254"}="H241 254";
$resn_exch{"2H24 254"}="H242 254"; $resn_exch{"3H24 254"}="H243 254"; $resn_exch{"1H25 254"}="H251 254";
$resn_exch{"2H25 254"}="H252 254"; $resn_exch{"3H25 254"}="H253 254"; $resn_exch{"1H26 254"}="H261 254";
$resn_exch{"2H26 254"}="H262 254"; $resn_exch{"3H26 254"}="H263 254"; $resn_exch{"1H01 255"}="H011 255";
$resn_exch{"2H01 255"}="H012 255"; $resn_exch{"3H01 255"}="H013 255"; $resn_exch{"1H02 255"}="H021 255";
$resn_exch{"2H02 255"}="H022 255"; $resn_exch{"3H02 255"}="H023 255"; $resn_exch{"1H03 255"}="H031 255";
$resn_exch{"2H03 255"}="H032 255"; $resn_exch{"3H03 255"}="H033 255"; $resn_exch{"1H5* 25A"}="H5'1 25A";
$resn_exch{"2H5* 25A"}="H5'2 25A"; $resn_exch{" H4* 25A"}=" H4' 25A"; $resn_exch{" H1* 25A"}=" H1' 25A";
$resn_exch{" H2* 25A"}=" H2' 25A"; $resn_exch{" H3* 25A"}=" H3' 25A"; $resn_exch{"1HN6 25A"}="HN61 25A";
$resn_exch{"2HN6 25A"}="HN62 25A"; $resn_exch{"1H15 25A"}="H151 25A"; $resn_exch{"2H15 25A"}="H152 25A";
$resn_exch{"H14* 25A"}="H14' 25A"; $resn_exch{"H11* 25A"}="H11' 25A"; $resn_exch{"1H16 25A"}="H161 25A";
$resn_exch{"2H16 25A"}="H162 25A"; $resn_exch{"H12* 25A"}="H12' 25A"; $resn_exch{"H13* 25A"}="H13' 25A";
$resn_exch{"1H25 25A"}="H251 25A"; $resn_exch{"2H25 25A"}="H252 25A"; $resn_exch{"H24* 25A"}="H24' 25A";
$resn_exch{"H21* 25A"}="H21' 25A"; $resn_exch{"1H26 25A"}="H261 25A"; $resn_exch{"2H26 25A"}="H262 25A";
$resn_exch{"H22* 25A"}="H22' 25A"; $resn_exch{"H23* 25A"}="H23' 25A"; $resn_exch{"1HN  25T"}=" HN1 25T";
$resn_exch{"2HN  25T"}=" HN2 25T"; $resn_exch{"1HM  25T"}=" HM1 25T"; $resn_exch{"2HM  25T"}=" HM2 25T";
$resn_exch{"3HM  25T"}=" HM3 25T"; $resn_exch{"1H10 26A"}="H101 26A"; $resn_exch{"2H10 26A"}="H102 26A";
$resn_exch{"3H10 26A"}="H103 26A"; $resn_exch{"1H9  26A"}=" H91 26A"; $resn_exch{"2H9  26A"}=" H92 26A";
$resn_exch{"3H9  26A"}=" H93 26A"; $resn_exch{" H1* 26A"}=" H1' 26A"; $resn_exch{" H4* 26A"}=" H4' 26A";
$resn_exch{"1H5* 26A"}="H5'1 26A"; $resn_exch{"2H5* 26A"}="H5'2 26A"; $resn_exch{" H5* 26A"}=" H5' 26A";
$resn_exch{" H2* 26A"}=" H2' 26A"; $resn_exch{"2HO* 26A"}="HO'2 26A"; $resn_exch{" H3* 26A"}=" H3' 26A";
$resn_exch{"3HO* 26A"}="HO'3 26A"; $resn_exch{"1HN1 26D"}="HN11 26D"; $resn_exch{"2HN1 26D"}="HN12 26D";
$resn_exch{"1HN2 26D"}="HN21 26D"; $resn_exch{"2HN2 26D"}="HN22 26D"; $resn_exch{"1HN2 26P"}="HN21 26P";
$resn_exch{"2HN2 26P"}="HN22 26P"; $resn_exch{"1H3  26P"}=" H31 26P"; $resn_exch{"2H3  26P"}=" H32 26P";
$resn_exch{"1H4  26P"}=" H41 26P"; $resn_exch{"2H4  26P"}=" H42 26P"; $resn_exch{"1H5  26P"}=" H51 26P";
$resn_exch{"2H5  26P"}=" H52 26P"; $resn_exch{"1H2  270"}=" H21 270"; $resn_exch{"2H2  270"}=" H22 270";
$resn_exch{"1H21 274"}="H211 274"; $resn_exch{"2H21 274"}="H212 274"; $resn_exch{"3H21 274"}="H213 274";
$resn_exch{"1H23 274"}="H231 274"; $resn_exch{"2H23 274"}="H232 274"; $resn_exch{"3H23 274"}="H233 274";
$resn_exch{"1H7C 274"}="H7C1 274"; $resn_exch{"2H7C 274"}="H7C2 274"; $resn_exch{"3H7C 274"}="H7C3 274";
$resn_exch{"1H24 274"}="H241 274"; $resn_exch{"2H24 274"}="H242 274"; $resn_exch{"1H27 274"}="H271 274";
$resn_exch{"2H27 274"}="H272 274"; $resn_exch{"1H28 274"}="H281 274"; $resn_exch{"2H28 274"}="H282 274";
$resn_exch{"3H28 274"}="H283 274"; $resn_exch{"1H26 274"}="H261 274"; $resn_exch{"2H26 274"}="H262 274";
$resn_exch{"1H25 274"}="H251 274"; $resn_exch{"2H25 274"}="H252 274"; $resn_exch{"1H13 276"}="H131 276";
$resn_exch{"2H13 276"}="H132 276"; $resn_exch{"3H13 276"}="H133 276"; $resn_exch{"1H1  277"}=" H11 277";
$resn_exch{"2H1  277"}=" H12 277"; $resn_exch{"1H2  277"}=" H21 277"; $resn_exch{"2H2  277"}=" H22 277";
$resn_exch{"1H4  277"}=" H41 277"; $resn_exch{"2H4  277"}=" H42 277"; $resn_exch{"1H5  277"}=" H51 277";
$resn_exch{"2H5  277"}=" H52 277"; $resn_exch{"1H23 277"}="H231 277"; $resn_exch{"2H23 277"}="H232 277";
$resn_exch{"3H23 277"}="H233 277"; $resn_exch{"1H27 277"}="H271 277"; $resn_exch{"2H27 277"}="H272 277";
$resn_exch{"3H27 277"}="H273 277"; $resn_exch{"1H33 277"}="H331 277"; $resn_exch{"2H33 277"}="H332 277";
$resn_exch{"1H35 277"}="H351 277"; $resn_exch{"2H35 277"}="H352 277"; $resn_exch{"3H35 277"}="H353 277";
$resn_exch{"1H42 277"}="H421 277"; $resn_exch{"2H42 277"}="H422 277"; $resn_exch{"1H44 277"}="H441 277";
$resn_exch{"2H44 277"}="H442 277"; $resn_exch{"1H45 277"}="H451 277"; $resn_exch{"2H45 277"}="H452 277";
$resn_exch{"1H30 281"}="H301 281"; $resn_exch{"2H30 281"}="H302 281"; $resn_exch{"3H30 281"}="H303 281";
$resn_exch{"1H29 281"}="H291 281"; $resn_exch{"2H29 281"}="H292 281"; $resn_exch{"3H29 281"}="H293 281";
$resn_exch{"1H15 281"}="H151 281"; $resn_exch{"2H15 281"}="H152 281"; $resn_exch{"1H9  281"}=" H91 281";
$resn_exch{"2H9  281"}=" H92 281"; $resn_exch{"1H10 281"}="H101 281"; $resn_exch{"2H10 281"}="H102 281";
$resn_exch{"1H5  283"}=" H51 283"; $resn_exch{"2H5  283"}=" H52 283"; $resn_exch{"1H6  283"}=" H61 283";
$resn_exch{"2H6  283"}=" H62 283"; $resn_exch{"3H6  283"}=" H63 283"; $resn_exch{"1H10 283"}="H101 283";
$resn_exch{"2H10 283"}="H102 283"; $resn_exch{"1H11 283"}="H111 283"; $resn_exch{"2H11 283"}="H112 283";
$resn_exch{"1H13 283"}="H131 283"; $resn_exch{"2H13 283"}="H132 283"; $resn_exch{"1H14 288"}="H141 288";
$resn_exch{"2H14 288"}="H142 288"; $resn_exch{"1H19 288"}="H191 288"; $resn_exch{"2H19 288"}="H192 288";
$resn_exch{"1H20 288"}="H201 288"; $resn_exch{"2H20 288"}="H202 288"; $resn_exch{"1H21 288"}="H211 288";
$resn_exch{"2H21 288"}="H212 288"; $resn_exch{"1H22 288"}="H221 288"; $resn_exch{"2H22 288"}="H222 288";
$resn_exch{"1H23 288"}="H231 288"; $resn_exch{"2H23 288"}="H232 288"; $resn_exch{"1H24 288"}="H241 288";
$resn_exch{"2H24 288"}="H242 288"; $resn_exch{"1H1C 292"}="H1C1 292"; $resn_exch{"2H1C 292"}="H1C2 292";
$resn_exch{"1H2C 292"}="H2C1 292"; $resn_exch{"2H2C 292"}="H2C2 292"; $resn_exch{"1H11 292"}="H111 292";
$resn_exch{"2H11 292"}="H112 292"; $resn_exch{"1H7  299"}=" H71 299"; $resn_exch{"2H7  299"}=" H72 299";
$resn_exch{"3H7  299"}=" H73 299"; $resn_exch{"1H10 2A6"}="H101 2A6"; $resn_exch{"2H10 2A6"}="H102 2A6";
$resn_exch{"1H12 2A6"}="H121 2A6"; $resn_exch{"2H12 2A6"}="H122 2A6"; $resn_exch{"1H13 2A6"}="H131 2A6";
$resn_exch{"2H13 2A6"}="H132 2A6"; $resn_exch{"1H14 2A6"}="H141 2A6"; $resn_exch{"2H14 2A6"}="H142 2A6";
$resn_exch{"1H15 2A6"}="H151 2A6"; $resn_exch{"2H15 2A6"}="H152 2A6"; $resn_exch{"1H16 2A6"}="H161 2A6";
$resn_exch{"2H16 2A6"}="H162 2A6"; $resn_exch{"1H13 2AA"}="H131 2AA"; $resn_exch{"2H13 2AA"}="H132 2AA";
$resn_exch{"3H13 2AA"}="H133 2AA"; $resn_exch{"1H12 2AA"}="H121 2AA"; $resn_exch{"2H12 2AA"}="H122 2AA";
$resn_exch{"3H12 2AA"}="H123 2AA"; $resn_exch{" H9* 2AA"}=" H9' 2AA"; $resn_exch{" H8* 2AA"}=" H8' 2AA";
$resn_exch{" H5* 2AA"}=" H5' 2AA"; $resn_exch{" H4* 2AA"}=" H4' 2AA"; $resn_exch{" H3* 2AA"}=" H3' 2AA";
$resn_exch{"1H1* 2AA"}="H1'1 2AA"; $resn_exch{"2H1* 2AA"}="H1'2 2AA"; $resn_exch{"1H9  2AA"}=" H91 2AA";
$resn_exch{"2H9  2AA"}=" H92 2AA"; $resn_exch{"1H14 2AA"}="H141 2AA"; $resn_exch{"2H14 2AA"}="H142 2AA";
$resn_exch{"1HN2 2AA"}="HN21 2AA"; $resn_exch{"2HN2 2AA"}="HN22 2AA"; $resn_exch{"1H1  2AB"}=" H11 2AB";
$resn_exch{"2H1  2AB"}="H12A 2AB"; $resn_exch{"1H2  2AB"}=" H21 2AB"; $resn_exch{"2H2  2AB"}=" H22 2AB";
$resn_exch{"1HN9 2AB"}="HN91 2AB"; $resn_exch{"2HN9 2AB"}="HN92 2AB"; $resn_exch{"1HN  2AC"}=" HN1 2AC";
$resn_exch{"2HN  2AC"}=" HN2 2AC"; $resn_exch{"1H8  2AC"}=" H81 2AC"; $resn_exch{"2H8  2AC"}=" H82 2AC";
$resn_exch{"3H8  2AC"}=" H83 2AC"; $resn_exch{" H5* 2AD"}=" H5' 2AD"; $resn_exch{"1H5* 2AD"}="H5'1 2AD";
$resn_exch{"2H5* 2AD"}="H5'2 2AD"; $resn_exch{" H4* 2AD"}=" H4' 2AD"; $resn_exch{" H3* 2AD"}=" H3' 2AD";
$resn_exch{" H2* 2AD"}=" H2' 2AD"; $resn_exch{" H1* 2AD"}=" H1' 2AD"; $resn_exch{"1H6N 2AD"}="H6N1 2AD";
$resn_exch{"2H6N 2AD"}="H6N2 2AD"; $resn_exch{"1HN  2AD"}=" HN1 2AD"; $resn_exch{"2HN  2AD"}=" HN2 2AD";
$resn_exch{"1HN  2AF"}=" HN1 2AF"; $resn_exch{"2HN  2AF"}=" HN2 2AF"; $resn_exch{"1H12 2AH"}="H121 2AH";
$resn_exch{"2H12 2AH"}="H122 2AH"; $resn_exch{"1H24 2AH"}="H241 2AH"; $resn_exch{"2H24 2AH"}="H242 2AH";
$resn_exch{"1H41 2AH"}="H411 2AH"; $resn_exch{"2H41 2AH"}="H412 2AH"; $resn_exch{"1H44 2AH"}="H441 2AH";
$resn_exch{"2H44 2AH"}="H442 2AH"; $resn_exch{"1H49 2AH"}="H491 2AH"; $resn_exch{"2H49 2AH"}="H492 2AH";
$resn_exch{"3H49 2AH"}="H493 2AH"; $resn_exch{"1H50 2AH"}="H501 2AH"; $resn_exch{"2H50 2AH"}="H502 2AH";
$resn_exch{"3H50 2AH"}="H503 2AH"; $resn_exch{"1H91 2AH"}="H911 2AH"; $resn_exch{"2H91 2AH"}="H912 2AH";
$resn_exch{"3H91 2AH"}="H913 2AH"; $resn_exch{"1H92 2AH"}="H921 2AH"; $resn_exch{"2H92 2AH"}="H922 2AH";
$resn_exch{"3H92 2AH"}="H923 2AH"; $resn_exch{"1HN1 2AL"}="HN11 2AL"; $resn_exch{"2HN1 2AL"}="HN12 2AL";
$resn_exch{"2HOP 2AM"}="HOP2 2AM"; $resn_exch{"3HOP 2AM"}="HOP3 2AM"; $resn_exch{"1H5* 2AM"}="H5'1 2AM";
$resn_exch{"2H5* 2AM"}="H5'2 2AM"; $resn_exch{"*HO5 2AM"}="HO5' 2AM"; $resn_exch{" H4* 2AM"}=" H4' 2AM";
$resn_exch{" H3* 2AM"}=" H3' 2AM"; $resn_exch{"*HO3 2AM"}="HO3' 2AM"; $resn_exch{" H2* 2AM"}=" H2' 2AM";
$resn_exch{" H1* 2AM"}=" H1' 2AM"; $resn_exch{"1HN6 2AM"}="HN61 2AM"; $resn_exch{"2HN6 2AM"}="HN62 2AM";
$resn_exch{" HN1 2AO"}=" H   2AO"; $resn_exch{" HN2 2AO"}=" H2  2AO"; $resn_exch{"HCB1 2AO"}=" HB2 2AO";
$resn_exch{"HCB2 2AO"}=" HB3 2AO"; $resn_exch{"HCG1 2AO"}=" HG2 2AO"; $resn_exch{"HCG2 2AO"}=" HG3 2AO";
$resn_exch{"HCD1 2AO"}=" HD2 2AO"; $resn_exch{"HCD2 2AO"}=" HD3 2AO"; $resn_exch{"HCE1 2AO"}=" HE1 2AO";
$resn_exch{"HCE2 2AO"}=" HE2 2AO"; $resn_exch{"HCE3 2AO"}=" HE3 2AO"; $resn_exch{"1HN  2AP"}="HN1A 2AP";
$resn_exch{"2HN  2AP"}=" HN2 2AP"; $resn_exch{"1HN1 2AQ"}="HN11 2AQ"; $resn_exch{"2HN1 2AQ"}="HN12 2AQ";
$resn_exch{"2HOP 2AR"}="HOP2 2AR"; $resn_exch{"3HOP 2AR"}="HOP3 2AR"; $resn_exch{"1H5* 2AR"}=" H5' 2AR";
$resn_exch{"2H5* 2AR"}="H5'' 2AR"; $resn_exch{" H4* 2AR"}=" H4' 2AR"; $resn_exch{"1HX* 2AR"}="HX'1 2AR";
$resn_exch{"2HX* 2AR"}="HX'2 2AR"; $resn_exch{" H1* 2AR"}=" H1' 2AR"; $resn_exch{"1H6  2AR"}=" H61 2AR";
$resn_exch{"2H6  2AR"}=" H62 2AR"; $resn_exch{"2H2* 2AR"}="H2'' 2AR"; $resn_exch{"1H2* 2AR"}=" H2' 2AR";
$resn_exch{" H3* 2AR"}=" H3' 2AR"; $resn_exch{"H3T  2AR"}="HO3' 2AR"; $resn_exch{"1HBB 2AS"}="HBB1 2AS";
$resn_exch{"2HBB 2AS"}="HBB2 2AS"; $resn_exch{"3HBB 2AS"}="HBB3 2AS"; $resn_exch{"2HOP 2AT"}="HOP2 2AT";
$resn_exch{"1H5* 2AT"}=" H5' 2AT"; $resn_exch{"2H5* 2AT"}="H5'' 2AT"; $resn_exch{" H4* 2AT"}=" H4' 2AT";
$resn_exch{" H1* 2AT"}=" H1' 2AT"; $resn_exch{"1H5M 2AT"}=" H71 2AT"; $resn_exch{"2H5M 2AT"}=" H72 2AT";
$resn_exch{"3H5M 2AT"}=" H73 2AT"; $resn_exch{" H2* 2AT"}=" H2' 2AT"; $resn_exch{"1HB' 2AT"}="HB'1 2AT";
$resn_exch{"2HB' 2AT"}="HB'2 2AT"; $resn_exch{"1HD' 2AT"}="HD'1 2AT"; $resn_exch{"2HD' 2AT"}="HD'2 2AT";
$resn_exch{" H3* 2AT"}=" H3' 2AT"; $resn_exch{"H3T  2AT"}="HO3' 2AT"; $resn_exch{"3HOP 2AT"}="HOP3 2AT";
$resn_exch{"1H5* 2AU"}=" H5' 2AU"; $resn_exch{"2H5* 2AU"}="H5'' 2AU"; $resn_exch{" H4* 2AU"}=" H4' 2AU";
$resn_exch{" H3* 2AU"}=" H3' 2AU"; $resn_exch{"H3T  2AU"}="HO3' 2AU"; $resn_exch{"1H2* 2AU"}=" H2' 2AU";
$resn_exch{" H1* 2AU"}=" H1' 2AU"; $resn_exch{"1HA  2AU"}=" HA1 2AU"; $resn_exch{"2H2* 2AU"}="H2'' 2AU";
$resn_exch{" O2P 2AU"}=" OP2 2AU"; $resn_exch{"2H1P 2AU"}="H2P1 2AU"; $resn_exch{"1H1  2AZ"}=" H11 2AZ";
$resn_exch{"2H1  2AZ"}=" H12 2AZ"; $resn_exch{"1H6  2AZ"}="H61A 2AZ"; $resn_exch{"2H6  2AZ"}=" H62 2AZ";
$resn_exch{"1H23 2AZ"}="H231 2AZ"; $resn_exch{"2H23 2AZ"}="H232 2AZ"; $resn_exch{"1H39 2AZ"}="H391 2AZ";
$resn_exch{"2H39 2AZ"}="H392 2AZ"; $resn_exch{"1H42 2AZ"}="H421 2AZ"; $resn_exch{"2H42 2AZ"}="H422 2AZ";
$resn_exch{"1H45 2AZ"}="H451 2AZ"; $resn_exch{"2H45 2AZ"}="H452 2AZ"; $resn_exch{"1H48 2AZ"}="H481 2AZ";
$resn_exch{"2H48 2AZ"}="H482 2AZ"; $resn_exch{"1H63 2AZ"}="H631 2AZ"; $resn_exch{"2H63 2AZ"}="H632 2AZ";
$resn_exch{"3H63 2AZ"}="H633 2AZ"; $resn_exch{" H5* 2BA"}=" H5' 2BA"; $resn_exch{"H5*A 2BA"}="H5'A 2BA";
$resn_exch{" H4* 2BA"}=" H4' 2BA"; $resn_exch{" H3* 2BA"}=" H3' 2BA"; $resn_exch{" H2* 2BA"}=" H2' 2BA";
$resn_exch{"HO2* 2BA"}="HO2' 2BA"; $resn_exch{" H1* 2BA"}=" H1' 2BA"; $resn_exch{"HC5* 2BA"}="HC5' 2BA";
$resn_exch{"HC4* 2BA"}="HC4' 2BA"; $resn_exch{"HC3* 2BA"}="HC3' 2BA"; $resn_exch{"HC2* 2BA"}="HC2' 2BA";
$resn_exch{"HC1* 2BA"}="HC1' 2BA"; $resn_exch{"2HOP 2BD"}="HOP2 2BD"; $resn_exch{"3HOP 2BD"}="HOP3 2BD";
$resn_exch{"1H5* 2BD"}=" H5' 2BD"; $resn_exch{"2H5* 2BD"}="H5'' 2BD"; $resn_exch{" H4* 2BD"}=" H4' 2BD";
$resn_exch{" H1* 2BD"}=" H1' 2BD"; $resn_exch{"2HAL 2BD"}="HAL2 2BD"; $resn_exch{"1HAM 2BD"}="HAM1 2BD";
$resn_exch{"2HAM 2BD"}="HAM2 2BD"; $resn_exch{"1HAO 2BD"}="HAO1 2BD"; $resn_exch{"2HAO 2BD"}="HAO2 2BD";
$resn_exch{"1H2* 2BD"}=" H2' 2BD"; $resn_exch{"2H2* 2BD"}="H2'' 2BD"; $resn_exch{" H3* 2BD"}=" H3' 2BD";
$resn_exch{"H3T  2BD"}="HO3' 2BD"; $resn_exch{"2HOP 2BF"}="HOP2 2BF"; $resn_exch{"3HOP 2BF"}="HOP3 2BF";
$resn_exch{"1H19 2BF"}="H191 2BF"; $resn_exch{"2H19 2BF"}="H192 2BF"; $resn_exch{"2HP1 2BH"}="HP12 2BH";
$resn_exch{"1HP1 2BH"}="HP11 2BH"; $resn_exch{"1H06 2BH"}="H061 2BH"; $resn_exch{"2H06 2BH"}="H062 2BH";
$resn_exch{"1H07 2BH"}="H071 2BH"; $resn_exch{"2H07 2BH"}="H072 2BH"; $resn_exch{"1HD  2BL"}=" HD1 2BL";
$resn_exch{"2HD  2BL"}=" HD2 2BL"; $resn_exch{"1HE  2BL"}=" HE1 2BL"; $resn_exch{"2HE  2BL"}=" HE2 2BL";
$resn_exch{"3HE  2BL"}=" HE3 2BL"; $resn_exch{"1HJ  2BL"}=" HJ1 2BL"; $resn_exch{"2HJ  2BL"}=" HJ2 2BL";
$resn_exch{"3HJ  2BL"}=" HJ3 2BL"; $resn_exch{"1HC4 2BN"}="HC41 2BN"; $resn_exch{"2HC4 2BN"}="HC42 2BN";
$resn_exch{"1HC6 2BN"}="HC61 2BN"; $resn_exch{"2HC6 2BN"}="HC62 2BN"; $resn_exch{"1HC7 2BN"}="HC71 2BN";
$resn_exch{"2HC7 2BN"}="HC72 2BN"; $resn_exch{"1HC9 2BN"}="HC91 2BN"; $resn_exch{"2HC9 2BN"}="HC92 2BN";
$resn_exch{"3HC9 2BN"}="HC93 2BN"; $resn_exch{"1H10 2BN"}="H101 2BN"; $resn_exch{"2H10 2BN"}="H102 2BN";
$resn_exch{"3H10 2BN"}="H103 2BN"; $resn_exch{"1H11 2BN"}="H111 2BN"; $resn_exch{"2H11 2BN"}="H112 2BN";
$resn_exch{"3H11 2BN"}="H113 2BN"; $resn_exch{"3HOP 2BT"}="HOP3 2BT"; $resn_exch{"2HOP 2BT"}="HOP2 2BT";
$resn_exch{"1H5* 2BT"}=" H5' 2BT"; $resn_exch{"2H5* 2BT"}="H5'' 2BT"; $resn_exch{" H4* 2BT"}=" H4' 2BT";
$resn_exch{" H1* 2BT"}=" H1' 2BT"; $resn_exch{"1H5M 2BT"}=" H71 2BT"; $resn_exch{"2H5M 2BT"}=" H72 2BT";
$resn_exch{"3H5M 2BT"}=" H73 2BT"; $resn_exch{" H2* 2BT"}=" H2' 2BT"; $resn_exch{"1HA* 2BT"}="HA'1 2BT";
$resn_exch{"2HA* 2BT"}="HA'2 2BT"; $resn_exch{"1HB* 2BT"}="HB'1 2BT"; $resn_exch{"2HB* 2BT"}="HB'2 2BT";
$resn_exch{"1HC* 2BT"}="HC'1 2BT"; $resn_exch{"2HC* 2BT"}="HC'2 2BT"; $resn_exch{" H3* 2BT"}=" H3' 2BT";
$resn_exch{"H3T  2BT"}="HO3' 2BT"; $resn_exch{"1HD* 2BT"}="HD'1 2BT"; $resn_exch{"2HD* 2BT"}="HD'2 2BT";
$resn_exch{"3HD* 2BT"}="HD'3 2BT"; $resn_exch{"2HOP 2BU"}="HOP2 2BU"; $resn_exch{"3HOP 2BU"}="HOP3 2BU";
$resn_exch{" H5* 2BU"}=" H5' 2BU"; $resn_exch{"H5** 2BU"}="H5'' 2BU"; $resn_exch{" H4* 2BU"}=" H4' 2BU";
$resn_exch{" H1* 2BU"}=" H1' 2BU"; $resn_exch{" H2* 2BU"}=" H2' 2BU"; $resn_exch{"H2** 2BU"}="H2'' 2BU";
$resn_exch{" H3* 2BU"}=" H3' 2BU"; $resn_exch{"H3T  2BU"}="HO3' 2BU"; $resn_exch{"1HC1 2C5"}="HC11 2C5";
$resn_exch{"2HC1 2C5"}="HC12 2C5"; $resn_exch{"1H15 2C5"}="H151 2C5"; $resn_exch{"2H15 2C5"}="H152 2C5";
$resn_exch{"3H15 2C5"}="H153 2C5"; $resn_exch{"1H10 2CA"}="H101 2CA"; $resn_exch{"2H10 2CA"}="H102 2CA";
$resn_exch{"1H12 2CA"}="H121 2CA"; $resn_exch{"2H12 2CA"}="H122 2CA"; $resn_exch{"3H12 2CA"}="H123 2CA";
$resn_exch{"1H13 2CA"}="H131 2CA"; $resn_exch{"2H13 2CA"}="H132 2CA"; $resn_exch{"3H13 2CA"}="H133 2CA";
$resn_exch{"1H02 2CA"}="H021 2CA"; $resn_exch{"2H02 2CA"}="H022 2CA"; $resn_exch{"1H03 2CA"}="H031 2CA";
$resn_exch{"2H03 2CA"}="H032 2CA"; $resn_exch{"1H04 2CA"}="H041 2CA"; $resn_exch{"2H04 2CA"}="H042 2CA";
$resn_exch{"1H06 2CA"}="H061 2CA"; $resn_exch{"2H06 2CA"}="H062 2CA"; $resn_exch{"1H9C 2CE"}="H9C1 2CE";
$resn_exch{"2H9C 2CE"}="H9C2 2CE"; $resn_exch{"1H14 2CE"}="H141 2CE"; $resn_exch{"2H14 2CE"}="H142 2CE";
$resn_exch{"1H24 2CE"}="H241 2CE"; $resn_exch{"2H24 2CE"}="H242 2CE"; $resn_exch{"1H28 2CE"}="H281 2CE";
$resn_exch{"2H28 2CE"}="H282 2CE"; $resn_exch{"1H29 2CE"}="H291 2CE"; $resn_exch{"2H29 2CE"}="H292 2CE";
$resn_exch{"1H31 2CE"}="H311 2CE"; $resn_exch{"2H31 2CE"}="H312 2CE"; $resn_exch{"1H32 2CE"}="H321 2CE";
$resn_exch{"2H32 2CE"}="H322 2CE"; $resn_exch{"1H34 2CE"}="H341 2CE"; $resn_exch{"2H34 2CE"}="H342 2CE";
$resn_exch{"1H37 2CE"}="H371 2CE"; $resn_exch{"2H37 2CE"}="H372 2CE"; $resn_exch{"1HN  2CG"}=" HN1 2CG";
$resn_exch{"2HN  2CG"}=" HN2 2CG"; $resn_exch{"1H7  2CL"}=" H71 2CL"; $resn_exch{"2H7  2CL"}=" H72 2CL";
$resn_exch{"1HN4 2CM"}="HN41 2CM"; $resn_exch{"2HN4 2CM"}="HN42 2CM"; $resn_exch{"1HC6 2CM"}="HC61 2CM";
$resn_exch{"2HC6 2CM"}="HC62 2CM"; $resn_exch{"3HC6 2CM"}="HC63 2CM"; $resn_exch{"1H6  2CP"}=" H61 2CP";
$resn_exch{"2H6  2CP"}=" H62 2CP"; $resn_exch{"2HO' 2CP"}="HO'2 2CP"; $resn_exch{"1H5' 2CP"}="H5'1 2CP";
$resn_exch{"2H5' 2CP"}="H5'2 2CP"; $resn_exch{"1HB  2CP"}=" HB1 2CP"; $resn_exch{"2HB  2CP"}=" HB2 2CP";
$resn_exch{"1HP9 2CP"}="HP91 2CP"; $resn_exch{"2HP9 2CP"}="HP92 2CP"; $resn_exch{"3HP9 2CP"}="HP93 2CP";
$resn_exch{"1HP8 2CP"}="HP81 2CP"; $resn_exch{"2HP8 2CP"}="HP82 2CP"; $resn_exch{"3HP8 2CP"}="HP83 2CP";
$resn_exch{"3HOP 2CP"}="HOP3 2CP"; $resn_exch{"1HP5 2CP"}="HP51 2CP"; $resn_exch{"2HP5 2CP"}="HP52 2CP";
$resn_exch{"1HP4 2CP"}="HP41 2CP"; $resn_exch{"2HP4 2CP"}="HP42 2CP"; $resn_exch{"1HP2 2CP"}="HP21 2CP";
$resn_exch{"2HP2 2CP"}="HP22 2CP"; $resn_exch{"1HP1 2CP"}="HP11 2CP"; $resn_exch{"2HP1 2CP"}="HP12 2CP";
$resn_exch{"1HS1 2CP"}="HS11 2CP"; $resn_exch{"2HS1 2CP"}="HS12 2CP"; $resn_exch{"1HS3 2CP"}="HS31 2CP";
$resn_exch{"2HS3 2CP"}="HS32 2CP"; $resn_exch{"3HS3 2CP"}="HS33 2CP"; $resn_exch{"5HOS 2CP"}="HOS5 2CP";
$resn_exch{"1H1  2CS"}=" H11 2CS"; $resn_exch{"2H1  2CS"}="H12A 2CS"; $resn_exch{"3H1  2CS"}="H13A 2CS";
$resn_exch{"1H7  2CS"}=" H71 2CS"; $resn_exch{"2H7  2CS"}=" H72 2CS"; $resn_exch{"1H10 2CS"}="H101 2CS";
$resn_exch{"2H10 2CS"}="H102 2CS"; $resn_exch{"1H25 2CS"}="H251 2CS"; $resn_exch{"2H25 2CS"}="H252 2CS";
$resn_exch{"1H41 2CS"}="H411 2CS"; $resn_exch{"2H41 2CS"}="H412 2CS"; $resn_exch{"3H41 2CS"}="H413 2CS";
$resn_exch{"1H39 2CS"}="H391 2CS"; $resn_exch{"2H39 2CS"}="H392 2CS"; $resn_exch{"3H39 2CS"}="H393 2CS";
$resn_exch{"1H40 2CS"}="H401 2CS"; $resn_exch{"2H40 2CS"}="H402 2CS"; $resn_exch{"3H40 2CS"}="H403 2CS";
$resn_exch{"1H6  2CS"}=" H61 2CS"; $resn_exch{"2H6  2CS"}=" H62 2CS"; $resn_exch{"3H6  2CS"}=" H63 2CS";
$resn_exch{"1H18 2D7"}="H181 2D7"; $resn_exch{"2H18 2D7"}="H182 2D7"; $resn_exch{"3H18 2D7"}="H183 2D7";
$resn_exch{"1H31 2D9"}="H311 2D9"; $resn_exch{"2H31 2D9"}="H312 2D9"; $resn_exch{"3H31 2D9"}="H313 2D9";
$resn_exch{"1H18 2D9"}="H181 2D9"; $resn_exch{"2H18 2D9"}="H182 2D9"; $resn_exch{"1H16 2D9"}="H161 2D9";
$resn_exch{"2H16 2D9"}="H162 2D9"; $resn_exch{"1H17 2D9"}="H171 2D9"; $resn_exch{"2H17 2D9"}="H172 2D9";
$resn_exch{"1H14 2D9"}="H141 2D9"; $resn_exch{"2H14 2D9"}="H142 2D9"; $resn_exch{"1H13 2D9"}="H131 2D9";
$resn_exch{"2H13 2D9"}="H132 2D9"; $resn_exch{"2HOP 2DA"}="HOP2 2DA"; $resn_exch{"3HOP 2DA"}="HOP3 2DA";
$resn_exch{"1H5* 2DA"}=" H5' 2DA"; $resn_exch{"2H5* 2DA"}="H5'' 2DA"; $resn_exch{" H4* 2DA"}=" H4' 2DA";
$resn_exch{"1H3* 2DA"}="H3'1 2DA"; $resn_exch{"2H3* 2DA"}="H3'2 2DA"; $resn_exch{"1H2* 2DA"}=" H2' 2DA";
$resn_exch{"2H2* 2DA"}="H2'' 2DA"; $resn_exch{" H1* 2DA"}=" H1' 2DA"; $resn_exch{"1HN6 2DA"}="HN61 2DA";
$resn_exch{"2HN6 2DA"}="HN62 2DA"; $resn_exch{"1H1  2DB"}=" H11 2DB"; $resn_exch{"2H1  2DB"}=" H12 2DB";
$resn_exch{"1H3  2DB"}=" H31 2DB"; $resn_exch{"2H3  2DB"}=" H32 2DB"; $resn_exch{"1H17 2DD"}="H171 2DD";
$resn_exch{"2H17 2DD"}="H172 2DD"; $resn_exch{"1H4N 2DD"}="H4N1 2DD"; $resn_exch{"2H4N 2DD"}="H4N2 2DD";
$resn_exch{"2HOP 2DF"}="HOP2 2DF"; $resn_exch{"3HOP 2DF"}="HOP3 2DF"; $resn_exch{"1H5* 2DF"}="H5'1 2DF";
$resn_exch{"2H5* 2DF"}="H5'2 2DF"; $resn_exch{" H4* 2DF"}=" H4' 2DF"; $resn_exch{" H1* 2DF"}=" H1' 2DF";
$resn_exch{" H3* 2DF"}=" H3' 2DF"; $resn_exch{"1H2* 2DF"}="H2'1 2DF"; $resn_exch{"2H2* 2DF"}="H2'2 2DF";
$resn_exch{"1H2  2DG"}=" H21 2DG"; $resn_exch{"2H2  2DG"}=" H22 2DG"; $resn_exch{"1H6  2DG"}=" H61 2DG";
$resn_exch{"2H6  2DG"}=" H62 2DG"; $resn_exch{"1H5* 2DI"}="H5'1 2DI"; $resn_exch{"2H5* 2DI"}="H5'2 2DI";
$resn_exch{" H5* 2DI"}=" H5' 2DI"; $resn_exch{" H4* 2DI"}=" H4' 2DI"; $resn_exch{" H1* 2DI"}=" H1' 2DI";
$resn_exch{"1H2* 2DI"}="H2'1 2DI"; $resn_exch{"2H2* 2DI"}="H2'2 2DI"; $resn_exch{"1H3* 2DI"}="H3'1 2DI";
$resn_exch{"2H3* 2DI"}="H3'2 2DI"; $resn_exch{"2HOP 2DM"}="HOP2 2DM"; $resn_exch{"H3T  2DM"}="HO3' 2DM";
$resn_exch{"3HOP 2DM"}="HOP3 2DM"; $resn_exch{"1HN  2DO"}=" H   2DO"; $resn_exch{"2HN  2DO"}=" H2  2DO";
$resn_exch{" H   2DO"}=" HC  2DO"; $resn_exch{"1HCB 2DO"}=" HB2 2DO"; $resn_exch{"2HCB 2DO"}=" HB3 2DO";
$resn_exch{"1HCG 2DO"}=" HG2 2DO"; $resn_exch{"2HCG 2DO"}=" HG3 2DO"; $resn_exch{"1HCD 2DO"}=" HD2 2DO";
$resn_exch{"2HCD 2DO"}=" HD3 2DO"; $resn_exch{"1HCE 2DO"}=" HE1 2DO"; $resn_exch{"2HCE 2DO"}=" HE2 2DO";
$resn_exch{"3HCE 2DO"}=" HE3 2DO"; $resn_exch{"1H1  2DP"}=" H11 2DP"; $resn_exch{"2H1  2DP"}=" H12 2DP";
$resn_exch{"1H3  2DP"}=" H31 2DP"; $resn_exch{"2H3  2DP"}=" H32 2DP"; $resn_exch{"1H4  2DP"}=" H41 2DP";
$resn_exch{"2H4  2DP"}=" H42 2DP"; $resn_exch{"1H6  2DP"}=" H61 2DP"; $resn_exch{"2H6  2DP"}=" H62 2DP";
$resn_exch{"1H11 2DP"}="H111 2DP"; $resn_exch{"2H11 2DP"}="H112 2DP"; $resn_exch{"1H12 2DP"}="H121 2DP";
$resn_exch{"2H12 2DP"}="H122 2DP"; $resn_exch{"1H14 2DP"}="H141 2DP"; $resn_exch{"2H14 2DP"}="H142 2DP";
$resn_exch{"3H14 2DP"}="H143 2DP"; $resn_exch{"1H15 2DP"}="H151 2DP"; $resn_exch{"2H15 2DP"}="H152 2DP";
$resn_exch{"1H16 2DP"}="H161 2DP"; $resn_exch{"2H16 2DP"}="H162 2DP"; $resn_exch{"1H17 2DP"}="H171 2DP";
$resn_exch{"2H17 2DP"}="H172 2DP"; $resn_exch{"1H19 2DP"}="H191 2DP"; $resn_exch{"2H19 2DP"}="H192 2DP";
$resn_exch{"3H19 2DP"}="H193 2DP"; $resn_exch{"1H20 2DP"}="H201 2DP"; $resn_exch{"2H20 2DP"}="H202 2DP";
$resn_exch{"1H21 2DP"}="H211 2DP"; $resn_exch{"2H21 2DP"}="H212 2DP"; $resn_exch{"1H22 2DP"}="H221 2DP";
$resn_exch{"2H22 2DP"}="H222 2DP"; $resn_exch{"1H24 2DP"}="H241 2DP"; $resn_exch{"2H24 2DP"}="H242 2DP";
$resn_exch{"3H24 2DP"}="H243 2DP"; $resn_exch{"1H25 2DP"}="H251 2DP"; $resn_exch{"2H25 2DP"}="H252 2DP";
$resn_exch{"1H26 2DP"}="H261 2DP"; $resn_exch{"2H26 2DP"}="H262 2DP"; $resn_exch{"1H27 2DP"}="H271 2DP";
$resn_exch{"2H27 2DP"}="H272 2DP"; $resn_exch{"1H29 2DP"}="H291 2DP"; $resn_exch{"2H29 2DP"}="H292 2DP";
$resn_exch{"3H29 2DP"}="H293 2DP"; $resn_exch{"1H30 2DP"}="H301 2DP"; $resn_exch{"2H30 2DP"}="H302 2DP";
$resn_exch{"3H30 2DP"}="H303 2DP"; $resn_exch{"1H41 2DP"}="H411 2DP"; $resn_exch{"2H41 2DP"}="H412 2DP";
$resn_exch{"1H42 2DP"}="H421 2DP"; $resn_exch{"2H42 2DP"}="H422 2DP"; $resn_exch{"1H44 2DP"}="H441 2DP";
$resn_exch{"2H44 2DP"}="H442 2DP"; $resn_exch{"3H44 2DP"}="H443 2DP"; $resn_exch{"1H45 2DP"}="H451 2DP";
$resn_exch{"2H45 2DP"}="H452 2DP"; $resn_exch{"1H46 2DP"}="H461 2DP"; $resn_exch{"2H46 2DP"}="H462 2DP";
$resn_exch{"1H47 2DP"}="H471 2DP"; $resn_exch{"2H47 2DP"}="H472 2DP"; $resn_exch{"1H49 2DP"}="H491 2DP";
$resn_exch{"2H49 2DP"}="H492 2DP"; $resn_exch{"3H49 2DP"}="H493 2DP"; $resn_exch{"1H50 2DP"}="H501 2DP";
$resn_exch{"2H50 2DP"}="H502 2DP"; $resn_exch{"1H51 2DP"}="H511 2DP"; $resn_exch{"2H51 2DP"}="H512 2DP";
$resn_exch{"1H52 2DP"}="H521 2DP"; $resn_exch{"2H52 2DP"}="H522 2DP"; $resn_exch{"1H54 2DP"}="H541 2DP";
$resn_exch{"2H54 2DP"}="H542 2DP"; $resn_exch{"3H54 2DP"}="H543 2DP"; $resn_exch{"1H55 2DP"}="H551 2DP";
$resn_exch{"2H55 2DP"}="H552 2DP"; $resn_exch{"1H56 2DP"}="H561 2DP"; $resn_exch{"2H56 2DP"}="H562 2DP";
$resn_exch{"1H57 2DP"}="H571 2DP"; $resn_exch{"2H57 2DP"}="H572 2DP"; $resn_exch{"1H59 2DP"}="H591 2DP";
$resn_exch{"2H59 2DP"}="H592 2DP"; $resn_exch{"3H59 2DP"}="H593 2DP"; $resn_exch{"1H60 2DP"}="H601 2DP";
$resn_exch{"2H60 2DP"}="H602 2DP"; $resn_exch{"3H60 2DP"}="H603 2DP"; $resn_exch{"1HM  2DP"}=" HM1 2DP";
$resn_exch{"2HM  2DP"}=" HM2 2DP"; $resn_exch{"3HM  2DP"}=" HM3 2DP"; $resn_exch{"2HOP 2DT"}="HOP2 2DT";
$resn_exch{"3HOP 2DT"}="HOP3 2DT"; $resn_exch{"1H5M 2DT"}=" H71 2DT"; $resn_exch{"2H5M 2DT"}=" H72 2DT";
$resn_exch{"3H5M 2DT"}=" H73 2DT"; $resn_exch{"1H2* 2DT"}=" H2' 2DT"; $resn_exch{"2H2* 2DT"}="H2'' 2DT";
$resn_exch{"1H5* 2DT"}=" H5' 2DT"; $resn_exch{"2H5* 2DT"}="H5'' 2DT"; $resn_exch{" H4* 2DT"}=" H4' 2DT";
$resn_exch{" H1* 2DT"}=" H1' 2DT"; $resn_exch{"1H3* 2DT"}="H3'1 2DT"; $resn_exch{"2H3* 2DT"}="H3'2 2DT";
$resn_exch{"1H19 2E1"}="H191 2E1"; $resn_exch{"2H19 2E1"}="H192 2E1"; $resn_exch{"1H20 2E1"}="H201 2E1";
$resn_exch{"2H20 2E1"}="H202 2E1"; $resn_exch{"1H17 2E1"}="H171 2E1"; $resn_exch{"2H17 2E1"}="H172 2E1";
$resn_exch{"1H16 2E1"}="H161 2E1"; $resn_exch{"2H16 2E1"}="H162 2E1"; $resn_exch{"1H26 2E1"}="H261 2E1";
$resn_exch{"2H26 2E1"}="H262 2E1"; $resn_exch{"1H27 2E1"}="H271 2E1"; $resn_exch{"2H27 2E1"}="H272 2E1";
$resn_exch{"3H27 2E1"}="H273 2E1"; $resn_exch{"1H24 2EA"}="H241 2EA"; $resn_exch{"2H24 2EA"}="H242 2EA";
$resn_exch{"1H25 2EA"}="H251 2EA"; $resn_exch{"2H25 2EA"}="H252 2EA"; $resn_exch{"1HN4 2EA"}="HN41 2EA";
$resn_exch{"2HN4 2EA"}="HN42 2EA"; $resn_exch{"1H5* 2EG"}=" H5' 2EG"; $resn_exch{"2H5* 2EG"}="H5'' 2EG";
$resn_exch{" H4* 2EG"}=" H4' 2EG"; $resn_exch{" H3* 2EG"}=" H3' 2EG"; $resn_exch{"1H2* 2EG"}=" H2' 2EG";
$resn_exch{"2H2* 2EG"}="H2'' 2EG"; $resn_exch{" H1* 2EG"}=" H1' 2EG"; $resn_exch{"1H2  2EG"}=" H21 2EG";
$resn_exch{"1H21 2EG"}="H211 2EG"; $resn_exch{"2H21 2EG"}="H212 2EG"; $resn_exch{"1H22 2EG"}="H221 2EG";
$resn_exch{"2H22 2EG"}="H222 2EG"; $resn_exch{"3H22 2EG"}="H223 2EG"; $resn_exch{"1H3  2EP"}=" H31 2EP";
$resn_exch{"2H3  2EP"}=" H32 2EP"; $resn_exch{"1H4  2EP"}=" H41 2EP"; $resn_exch{"2H4  2EP"}=" H42 2EP";
$resn_exch{"1H5  2EP"}=" H51 2EP"; $resn_exch{"2H5  2EP"}=" H52 2EP"; $resn_exch{"1H6  2EP"}=" H61 2EP";
$resn_exch{"2H6  2EP"}=" H62 2EP"; $resn_exch{"1H1' 2EP"}="H1'1 2EP"; $resn_exch{"2H1' 2EP"}="H1'2 2EP";
$resn_exch{"1H2' 2EP"}="H2'1 2EP"; $resn_exch{"2H2' 2EP"}="H2'2 2EP"; $resn_exch{"3H2' 2EP"}="H2'3 2EP";
$resn_exch{"1H20 2EQ"}="H201 2EQ"; $resn_exch{"2H20 2EQ"}="H202 2EQ"; $resn_exch{"1H21 2EQ"}="H211 2EQ";
$resn_exch{"2H21 2EQ"}="H212 2EQ"; $resn_exch{"1H22 2EQ"}="H221 2EQ"; $resn_exch{"2H22 2EQ"}="H222 2EQ";
$resn_exch{"1H23 2EQ"}="H231 2EQ"; $resn_exch{"2H23 2EQ"}="H232 2EQ"; $resn_exch{"1H19 2EQ"}="H191 2EQ";
$resn_exch{"2H19 2EQ"}="H192 2EQ"; $resn_exch{"1H11 2EQ"}="H111 2EQ"; $resn_exch{"2H11 2EQ"}="H112 2EQ";
$resn_exch{"1H12 2EQ"}="H121 2EQ"; $resn_exch{"2H12 2EQ"}="H122 2EQ"; $resn_exch{"3H12 2EQ"}="H123 2EQ";
$resn_exch{"1H6  2EZ"}=" H61 2EZ"; $resn_exch{"2H6  2EZ"}=" H62 2EZ"; $resn_exch{"1H7  2EZ"}=" H71 2EZ";
$resn_exch{"2H7  2EZ"}=" H72 2EZ"; $resn_exch{"3H7  2EZ"}=" H73 2EZ"; $resn_exch{"1H5' 2FA"}="H5'1 2FA";
$resn_exch{"2H5' 2FA"}="H5'2 2FA"; $resn_exch{"1HN6 2FA"}="HN61 2FA"; $resn_exch{"2HN6 2FA"}="HN62 2FA";
$resn_exch{"1H5' 2FD"}="H5'1 2FD"; $resn_exch{"2H5' 2FD"}="H5'2 2FD"; $resn_exch{"1HN6 2FD"}="HN61 2FD";
$resn_exch{"2HN6 2FD"}="HN62 2FD"; $resn_exch{"1H2' 2FD"}="H2'1 2FD"; $resn_exch{"2H2' 2FD"}="H2'2 2FD";
$resn_exch{"1H6  2FG"}=" H61 2FG"; $resn_exch{"2H6  2FG"}=" H62 2FG"; $resn_exch{"1HAA 2FH"}="HAA1 2FH";
$resn_exch{"2HAA 2FH"}="HAA2 2FH"; $resn_exch{"1HBA 2FH"}="HBA1 2FH"; $resn_exch{"2HBA 2FH"}="HBA2 2FH";
$resn_exch{"1HMA 2FH"}="HMA1 2FH"; $resn_exch{"2HMA 2FH"}="HMA2 2FH"; $resn_exch{"3HMA 2FH"}="HMA3 2FH";
$resn_exch{"1HMB 2FH"}="HMB1 2FH"; $resn_exch{"2HMB 2FH"}="HMB2 2FH"; $resn_exch{"3HMB 2FH"}="HMB3 2FH";
$resn_exch{"1HBB 2FH"}="HBB1 2FH"; $resn_exch{"2HBB 2FH"}="HBB2 2FH"; $resn_exch{"1HMC 2FH"}="HMC1 2FH";
$resn_exch{"2HMC 2FH"}="HMC2 2FH"; $resn_exch{"3HMC 2FH"}="HMC3 2FH"; $resn_exch{"1HBC 2FH"}="HBC1 2FH";
$resn_exch{"2HBC 2FH"}="HBC2 2FH"; $resn_exch{"1HMD 2FH"}="HMD1 2FH"; $resn_exch{"2HMD 2FH"}="HMD2 2FH";
$resn_exch{"3HMD 2FH"}="HMD3 2FH"; $resn_exch{"1HAD 2FH"}="HAD1 2FH"; $resn_exch{"2HAD 2FH"}="HAD2 2FH";
$resn_exch{"1HBD 2FH"}="HBD1 2FH"; $resn_exch{"2HBD 2FH"}="HBD2 2FH"; $resn_exch{"1H6  2FL"}=" H61 2FL";
$resn_exch{"2H6  2FL"}=" H62 2FL"; $resn_exch{"1H6' 2FL"}="H6'1 2FL"; $resn_exch{"2H6' 2FL"}="H6'2 2FL";
$resn_exch{"'HO1 2FL"}="HO1' 2FL"; $resn_exch{"'HO2 2FL"}="HO2' 2FL"; $resn_exch{"'HO3 2FL"}="HO3' 2FL";
$resn_exch{"'HO6 2FL"}="HO6' 2FL"; $resn_exch{"1HN  2FM"}=" HN1 2FM"; $resn_exch{"2HN  2FM"}=" HN2 2FM";
$resn_exch{"1HB  2FM"}=" HB1 2FM"; $resn_exch{"2HB  2FM"}=" HB2 2FM"; $resn_exch{"1HG  2FM"}=" HG1 2FM";
$resn_exch{"2HG  2FM"}=" HG2 2FM"; $resn_exch{"1H1  2FP"}=" H11 2FP"; $resn_exch{"2H1  2FP"}=" H12 2FP";
$resn_exch{"1H6  2FP"}=" H61 2FP"; $resn_exch{"2H6  2FP"}="H62A 2FP"; $resn_exch{"1H22 2GG"}="H221 2GG";
$resn_exch{"2H22 2GG"}="H222 2GG"; $resn_exch{"3H22 2GG"}="H223 2GG"; $resn_exch{"1H26 2GG"}="H261 2GG";
$resn_exch{"2H26 2GG"}="H262 2GG"; $resn_exch{"1H27 2GG"}="H271 2GG"; $resn_exch{"2H27 2GG"}="H272 2GG";
$resn_exch{"3H27 2GG"}="H273 2GG"; $resn_exch{"1H25 2GJ"}="H251 2GJ"; $resn_exch{"2H25 2GJ"}="H252 2GJ";
$resn_exch{"1H26 2GJ"}="H261 2GJ"; $resn_exch{"2H26 2GJ"}="H262 2GJ"; $resn_exch{"1H28 2GJ"}="H281 2GJ";
$resn_exch{"2H28 2GJ"}="H282 2GJ"; $resn_exch{"1H29 2GJ"}="H291 2GJ"; $resn_exch{"2H29 2GJ"}="H292 2GJ";
$resn_exch{"1H23 2GJ"}="H231 2GJ"; $resn_exch{"2H23 2GJ"}="H232 2GJ"; $resn_exch{"1H11 2GJ"}="H111 2GJ";
$resn_exch{"2H11 2GJ"}="H112 2GJ"; $resn_exch{"1H12 2GJ"}="H121 2GJ"; $resn_exch{"2H12 2GJ"}="H122 2GJ";
$resn_exch{"3H12 2GJ"}="H123 2GJ"; $resn_exch{"1H32 2GJ"}="H321 2GJ"; $resn_exch{"2H32 2GJ"}="H322 2GJ";
$resn_exch{"3H32 2GJ"}="H323 2GJ"; $resn_exch{"1H31 2GJ"}="H311 2GJ"; $resn_exch{"2H31 2GJ"}="H312 2GJ";
$resn_exch{"3H31 2GJ"}="H313 2GJ"; $resn_exch{"1H2  2GL"}=" H21 2GL"; $resn_exch{"2H2  2GL"}=" H22 2GL";
$resn_exch{"1H6  2GL"}=" H61 2GL"; $resn_exch{"2H6  2GL"}=" H62 2GL"; $resn_exch{"3H6  2GL"}=" H63 2GL";
$resn_exch{"2HOP 2GP"}="HOP2 2GP"; $resn_exch{"3HOP 2GP"}="HOP3 2GP"; $resn_exch{"*HO5 2GP"}="HO5' 2GP";
$resn_exch{"1H5* 2GP"}="H5'1 2GP"; $resn_exch{"2H5* 2GP"}="H5'2 2GP"; $resn_exch{" H4* 2GP"}=" H4' 2GP";
$resn_exch{" H3* 2GP"}=" H3' 2GP"; $resn_exch{"*HO3 2GP"}="HO3' 2GP"; $resn_exch{" H2* 2GP"}=" H2' 2GP";
$resn_exch{" H1* 2GP"}=" H1' 2GP"; $resn_exch{"1HN2 2GP"}="HN21 2GP"; $resn_exch{"2HN2 2GP"}="HN22 2GP";
$resn_exch{"1H6  2GS"}=" H61 2GS"; $resn_exch{"2H6  2GS"}=" H62 2GS"; $resn_exch{"1H7  2GS"}=" H71 2GS";
$resn_exch{"2H7  2GS"}=" H72 2GS"; $resn_exch{"3H7  2GS"}=" H73 2GS"; $resn_exch{"2HOP 2GT"}="HOP2 2GT";
$resn_exch{"1H5* 2GT"}=" H5' 2GT"; $resn_exch{"2H5* 2GT"}="H5'' 2GT"; $resn_exch{" H4* 2GT"}=" H4' 2GT";
$resn_exch{" H1* 2GT"}=" H1' 2GT"; $resn_exch{"1H5M 2GT"}=" H71 2GT"; $resn_exch{"2H5M 2GT"}=" H72 2GT";
$resn_exch{"3H5M 2GT"}=" H73 2GT"; $resn_exch{" H2* 2GT"}=" H2' 2GT"; $resn_exch{"1HB' 2GT"}="HB'1 2GT";
$resn_exch{"2HB' 2GT"}="HB'2 2GT"; $resn_exch{" H3* 2GT"}=" H3' 2GT"; $resn_exch{"H3T  2GT"}="HO3' 2GT";
$resn_exch{"3HOP 2GT"}="HOP3 2GT"; $resn_exch{"1H1C 2HA"}="H1C1 2HA"; $resn_exch{"2H1C 2HA"}="H1C2 2HA";
$resn_exch{"1H3C 2HA"}="H3C1 2HA"; $resn_exch{"2H3C 2HA"}="H3C2 2HA"; $resn_exch{"1H6  2HR"}=" H61 2HR";
$resn_exch{"2H6  2HR"}=" H62 2HR"; $resn_exch{"3H6  2HR"}=" H63 2HR"; $resn_exch{"1H5  2HR"}=" H51 2HR";
$resn_exch{"2H5  2HR"}=" H52 2HR"; $resn_exch{"1H4  2HR"}=" H41 2HR"; $resn_exch{"2H4  2HR"}=" H42 2HR";
$resn_exch{"1H3  2HR"}=" H31 2HR"; $resn_exch{"2H3  2HR"}=" H32 2HR"; $resn_exch{"1H2  2HR"}=" H21 2HR";
$resn_exch{"2H2  2HR"}=" H22 2HR"; $resn_exch{"1H1  2HR"}=" H11 2HR"; $resn_exch{"2H1  2HR"}="H12A 2HR";
$resn_exch{"1H7  2HR"}=" H71 2HR"; $resn_exch{"2H7  2HR"}=" H72 2HR"; $resn_exch{"1H9  2HR"}=" H91 2HR";
$resn_exch{"2H9  2HR"}=" H92 2HR"; $resn_exch{"3H9  2HR"}=" H93 2HR"; $resn_exch{"1H10 2HR"}="H101 2HR";
$resn_exch{"2H10 2HR"}="H102 2HR"; $resn_exch{"1H6  2HS"}=" H61 2HS"; $resn_exch{"2H6  2HS"}=" H62 2HS";
$resn_exch{"3H6  2HS"}=" H63 2HS"; $resn_exch{"1H5  2HS"}=" H51 2HS"; $resn_exch{"2H5  2HS"}=" H52 2HS";
$resn_exch{"1H4  2HS"}=" H41 2HS"; $resn_exch{"2H4  2HS"}=" H42 2HS"; $resn_exch{"1H3  2HS"}=" H31 2HS";
$resn_exch{"2H3  2HS"}=" H32 2HS"; $resn_exch{"1H2  2HS"}=" H21 2HS"; $resn_exch{"2H2  2HS"}=" H22 2HS";
$resn_exch{"1H1  2HS"}=" H11 2HS"; $resn_exch{"2H1  2HS"}="H12A 2HS"; $resn_exch{"1H7  2HS"}=" H71 2HS";
$resn_exch{"2H7  2HS"}=" H72 2HS"; $resn_exch{"1H9  2HS"}=" H91 2HS"; $resn_exch{"2H9  2HS"}=" H92 2HS";
$resn_exch{"3H9  2HS"}=" H93 2HS"; $resn_exch{"1H10 2HS"}="H101 2HS"; $resn_exch{"2H10 2HS"}="H102 2HS";
$resn_exch{"1H   2IB"}=" H1  2IB"; $resn_exch{"2H   2IB"}=" H2  2IB"; $resn_exch{"3H   2IB"}=" H3A 2IB";
$resn_exch{"1H15 2IC"}="H151 2IC"; $resn_exch{"2H15 2IC"}="H152 2IC"; $resn_exch{"1H19 2IC"}="H191 2IC";
$resn_exch{"2H19 2IC"}="H192 2IC"; $resn_exch{"1H36 2IC"}="H361 2IC"; $resn_exch{"2H36 2IC"}="H362 2IC";
$resn_exch{"1H52 2IC"}="H521 2IC"; $resn_exch{"2H52 2IC"}="H522 2IC"; $resn_exch{"3H52 2IC"}="H523 2IC";
$resn_exch{"1HN3 2IG"}="HN31 2IG"; $resn_exch{"2HN3 2IG"}="HN32 2IG"; $resn_exch{"1HN4 2IG"}="HN41 2IG";
$resn_exch{"2HN4 2IG"}="HN42 2IG"; $resn_exch{"1H5  2IG"}=" H51 2IG"; $resn_exch{"2H5  2IG"}=" H52 2IG";
$resn_exch{"1H6  2IG"}=" H61 2IG"; $resn_exch{"2H6  2IG"}=" H62 2IG"; $resn_exch{"3H6  2IG"}=" H63 2IG";
$resn_exch{"1H13 2IG"}="H131 2IG"; $resn_exch{"2H13 2IG"}="H132 2IG"; $resn_exch{"1H14 2IG"}="H141 2IG";
$resn_exch{"2H14 2IG"}="H142 2IG"; $resn_exch{"1H15 2IG"}="H151 2IG"; $resn_exch{"2H15 2IG"}="H152 2IG";
$resn_exch{"1H16 2IG"}="H161 2IG"; $resn_exch{"2H16 2IG"}="H162 2IG"; $resn_exch{"3H16 2IG"}="H163 2IG";
$resn_exch{"1H6C 2IH"}="H6C1 2IH"; $resn_exch{"2H6C 2IH"}="H6C2 2IH"; $resn_exch{"1H7C 2IH"}="H7C1 2IH";
$resn_exch{"2H7C 2IH"}="H7C2 2IH"; $resn_exch{"1H8C 2IH"}="H8C1 2IH"; $resn_exch{"2H8C 2IH"}="H8C2 2IH";
$resn_exch{"1H9C 2IH"}="H9C1 2IH"; $resn_exch{"2H9C 2IH"}="H9C2 2IH"; $resn_exch{"1H10 2IH"}="H101 2IH";
$resn_exch{"2H10 2IH"}="H102 2IH"; $resn_exch{"1H11 2IH"}="H111 2IH"; $resn_exch{"2H11 2IH"}="H112 2IH";
$resn_exch{"1H4  2IM"}=" H41 2IM"; $resn_exch{"2H4  2IM"}=" H42 2IM"; $resn_exch{"1H5  2IM"}=" H51 2IM";
$resn_exch{"2H5  2IM"}=" H52 2IM"; $resn_exch{"1H3  2IM"}=" H31 2IM"; $resn_exch{"2H3  2IM"}=" H32 2IM";
$resn_exch{"1H2  2IM"}=" H21 2IM"; $resn_exch{"2H2  2IM"}=" H22 2IM"; $resn_exch{"1HN4 2IN"}="HN41 2IN";
$resn_exch{"2HN4 2IN"}="HN42 2IN"; $resn_exch{"1H11 2IN"}="H111 2IN"; $resn_exch{"2H11 2IN"}="H112 2IN";
$resn_exch{"1H8  2IN"}=" H81 2IN"; $resn_exch{"2H8  2IN"}=" H82 2IN"; $resn_exch{"1H6  2IN"}=" H61 2IN";
$resn_exch{"2H6  2IN"}=" H62 2IN"; $resn_exch{"1H1  2IN"}=" H11 2IN"; $resn_exch{"2H1  2IN"}="H12A 2IN";
$resn_exch{"2HO4 2IP"}="HO42 2IP"; $resn_exch{"3HO4 2IP"}="HO43 2IP"; $resn_exch{"2HO1 2IP"}="HO12 2IP";
$resn_exch{"3HO1 2IP"}="HO13 2IP"; $resn_exch{"1H4  2KT"}=" H41 2KT"; $resn_exch{"2H4  2KT"}=" H42 2KT";
$resn_exch{"3H4  2KT"}=" H43 2KT"; $resn_exch{"1H3  2KT"}=" H31 2KT"; $resn_exch{"2H3  2KT"}=" H32 2KT";
$resn_exch{"1H4  2LG"}=" H41 2LG"; $resn_exch{"2H4  2LG"}=" H42 2LG"; $resn_exch{"1H6  2LG"}=" H61 2LG";
$resn_exch{"2H6  2LG"}=" H62 2LG"; $resn_exch{"1H26 2LG"}="H261 2LG"; $resn_exch{"2H26 2LG"}="H262 2LG";
$resn_exch{"1H28 2LG"}="H281 2LG"; $resn_exch{"2H28 2LG"}="H282 2LG"; $resn_exch{"3H28 2LG"}="H283 2LG";
$resn_exch{"1H9  2LG"}=" H91 2LG"; $resn_exch{"2H9  2LG"}=" H92 2LG"; $resn_exch{"1H25 2LG"}="H251 2LG";
$resn_exch{"2H25 2LG"}="H252 2LG"; $resn_exch{"1H27 2LG"}="H271 2LG"; $resn_exch{"2H27 2LG"}="H272 2LG";
$resn_exch{"3H27 2LG"}="H273 2LG"; $resn_exch{"1HC4 2LP"}="HC41 2LP"; $resn_exch{"2HC4 2LP"}="HC42 2LP";
$resn_exch{"1H14 2LP"}="H141 2LP"; $resn_exch{"2H14 2LP"}="H142 2LP"; $resn_exch{" HN2 2LU"}=" H2  2LU";
$resn_exch{"1HB  2LU"}=" HB2 2LU"; $resn_exch{"2HB  2LU"}=" HB3 2LU"; $resn_exch{"1HD1 2LU"}="HD11 2LU";
$resn_exch{"2HD1 2LU"}="HD12 2LU"; $resn_exch{"3HD1 2LU"}="HD13 2LU"; $resn_exch{"1HD2 2LU"}="HD21 2LU";
$resn_exch{"2HD2 2LU"}="HD22 2LU"; $resn_exch{"3HD2 2LU"}="HD23 2LU"; $resn_exch{"2HOP 2MA"}="HOP2 2MA";
$resn_exch{"3HOP 2MA"}="HOP3 2MA"; $resn_exch{"1H5* 2MA"}=" H5' 2MA"; $resn_exch{"2H5* 2MA"}="H5'' 2MA";
$resn_exch{" H4* 2MA"}=" H4' 2MA"; $resn_exch{" H3* 2MA"}=" H3' 2MA"; $resn_exch{"*HO3 2MA"}="HO3' 2MA";
$resn_exch{" H2* 2MA"}=" H2' 2MA"; $resn_exch{"*HO2 2MA"}="HO2' 2MA"; $resn_exch{" H1* 2MA"}=" H1' 2MA";
$resn_exch{"1HM2 2MA"}="HM21 2MA"; $resn_exch{"2HM2 2MA"}="HM22 2MA"; $resn_exch{"3HM2 2MA"}="HM23 2MA";
$resn_exch{"1H4C 2MB"}="H4C1 2MB"; $resn_exch{"2H4C 2MB"}="H4C2 2MB"; $resn_exch{"3H4C 2MB"}="H4C3 2MB";
$resn_exch{"1H1C 2MB"}="H1C1 2MB"; $resn_exch{"2H1C 2MB"}="H1C2 2MB"; $resn_exch{"3H1C 2MB"}="H1C3 2MB";
$resn_exch{"1H2' 2MB"}="H2'1 2MB"; $resn_exch{"2H2' 2MB"}="H2'2 2MB"; $resn_exch{"3H2' 2MB"}="H2'3 2MB";
$resn_exch{"1HN6 2MC"}="HN61 2MC"; $resn_exch{"2HN6 2MC"}="HN62 2MC"; $resn_exch{" H1* 2MC"}=" H1' 2MC";
$resn_exch{" H2* 2MC"}=" H2' 2MC"; $resn_exch{" H3* 2MC"}=" H3' 2MC"; $resn_exch{" H4* 2MC"}=" H4' 2MC";
$resn_exch{"1H5* 2MC"}="H5'1 2MC"; $resn_exch{"2H5* 2MC"}="H5'2 2MC"; $resn_exch{"1H12 2MC"}="H121 2MC";
$resn_exch{"2H12 2MC"}="H122 2MC"; $resn_exch{"1H13 2MC"}="H131 2MC"; $resn_exch{"2H13 2MC"}="H132 2MC";
$resn_exch{"3H13 2MC"}="H133 2MC"; $resn_exch{"1H14 2MC"}="H141 2MC"; $resn_exch{"2H14 2MC"}="H142 2MC";
$resn_exch{"3H14 2MC"}="H143 2MC"; $resn_exch{"1H7  2MC"}=" H71 2MC"; $resn_exch{"2H7  2MC"}=" H72 2MC";
$resn_exch{"1H6  2MC"}=" H61 2MC"; $resn_exch{"2H6  2MC"}=" H62 2MC"; $resn_exch{"1H3  2MC"}=" H31 2MC";
$resn_exch{"2H3  2MC"}=" H32 2MC"; $resn_exch{"1H2  2MC"}=" H21 2MC"; $resn_exch{"2H2  2MC"}=" H22 2MC";
$resn_exch{"1H1  2MC"}=" H11 2MC"; $resn_exch{"2H1  2MC"}=" H12 2MC"; $resn_exch{"1H2M 2MC"}="H2M1 2MC";
$resn_exch{"2H2M 2MC"}="H2M2 2MC"; $resn_exch{"3H2M 2MC"}="H2M3 2MC"; $resn_exch{"BHO2 2MD"}="HO2B 2MD";
$resn_exch{"1H10 2MD"}="H101 2MD"; $resn_exch{"2H10 2MD"}="H102 2MD"; $resn_exch{"1HNG 2MD"}="HNG1 2MD";
$resn_exch{"2HNG 2MD"}="HNG2 2MD"; $resn_exch{"AHO2 2MD"}="HO2A 2MD"; $resn_exch{"1H5* 2MD"}="H5'1 2MD";
$resn_exch{"2H5* 2MD"}="H5'2 2MD"; $resn_exch{" H4* 2MD"}=" H4' 2MD"; $resn_exch{" H1* 2MD"}=" H1' 2MD";
$resn_exch{"1HN2 2MD"}="HN21 2MD"; $resn_exch{"2HN2 2MD"}="HN22 2MD"; $resn_exch{" H2* 2MD"}=" H2' 2MD";
$resn_exch{"*HO2 2MD"}="HO2' 2MD"; $resn_exch{" H3* 2MD"}=" H3' 2MD"; $resn_exch{"*HO3 2MD"}="HO3' 2MD";
$resn_exch{"1HA* 2ME"}="HA'1 2ME"; $resn_exch{"2HA* 2ME"}="HA'2 2ME"; $resn_exch{"3HA* 2ME"}="HA'3 2ME";
$resn_exch{"1HB* 2ME"}="HB'1 2ME"; $resn_exch{"2HB* 2ME"}="HB'2 2ME"; $resn_exch{"1HD* 2ME"}="HD'1 2ME";
$resn_exch{"2HD* 2ME"}="HD'2 2ME"; $resn_exch{"3HD* 2ME"}="HD'3 2ME"; $resn_exch{"2HOP 2MG"}="HOP2 2MG";
$resn_exch{"3HOP 2MG"}="HOP3 2MG"; $resn_exch{"1H5* 2MG"}=" H5' 2MG"; $resn_exch{"2H5* 2MG"}="H5'' 2MG";
$resn_exch{" H4* 2MG"}=" H4' 2MG"; $resn_exch{" H3* 2MG"}=" H3' 2MG"; $resn_exch{"*HO3 2MG"}="HO3' 2MG";
$resn_exch{" H2* 2MG"}=" H2' 2MG"; $resn_exch{"*HO2 2MG"}="HO2' 2MG"; $resn_exch{" H1* 2MG"}=" H1' 2MG";
$resn_exch{"1HM2 2MG"}="HM21 2MG"; $resn_exch{"2HM2 2MG"}="HM22 2MG"; $resn_exch{"3HM2 2MG"}="HM23 2MG";
$resn_exch{"1HN1 2MH"}="HN11 2MH"; $resn_exch{"2HN1 2MH"}="HN12 2MH"; $resn_exch{"1H6  2MH"}=" H61 2MH";
$resn_exch{"2H6  2MH"}=" H62 2MH"; $resn_exch{"1H4  2MH"}=" H41 2MH"; $resn_exch{"2H4  2MH"}=" H42 2MH";
$resn_exch{"1H3  2MH"}=" H31 2MH"; $resn_exch{"2H3  2MH"}=" H32 2MH"; $resn_exch{"1H1  2MH"}=" H11 2MH";
$resn_exch{"2H1  2MH"}=" H12 2MH"; $resn_exch{"3H1  2MH"}=" H13 2MH"; $resn_exch{"1H7  2MH"}=" H71 2MH";
$resn_exch{"2H7  2MH"}=" H72 2MH"; $resn_exch{"1HAJ 2MI"}="HAJ1 2MI"; $resn_exch{"2HAJ 2MI"}="HAJ2 2MI";
$resn_exch{"3HAJ 2MI"}="HAJ3 2MI"; $resn_exch{" HN2 2ML"}=" H2  2ML"; $resn_exch{"1HB1 2ML"}="HB11 2ML";
$resn_exch{"2HB1 2ML"}="HB12 2ML"; $resn_exch{"1HB2 2ML"}="HB21 2ML"; $resn_exch{"2HB2 2ML"}="HB22 2ML";
$resn_exch{"3HB2 2ML"}="HB23 2ML"; $resn_exch{"1HD1 2ML"}="HD11 2ML"; $resn_exch{"2HD1 2ML"}="HD12 2ML";
$resn_exch{"3HD1 2ML"}="HD13 2ML"; $resn_exch{"1HD2 2ML"}="HD21 2ML"; $resn_exch{"2HD2 2ML"}="HD22 2ML";
$resn_exch{"3HD2 2ML"}="HD23 2ML"; $resn_exch{"1HA5 2MN"}="HA51 2MN"; $resn_exch{"2HA5 2MN"}="HA52 2MN";
$resn_exch{"3HA5 2MN"}="HA53 2MN"; $resn_exch{"1H7  2MP"}=" H71 2MP"; $resn_exch{"2H7  2MP"}=" H72 2MP";
$resn_exch{"3H7  2MP"}=" H73 2MP"; $resn_exch{"1H8  2MP"}=" H81 2MP"; $resn_exch{"2H8  2MP"}=" H82 2MP";
$resn_exch{"3H8  2MP"}=" H83 2MP"; $resn_exch{" HN2 2MR"}=" H2  2MR"; $resn_exch{"1HB  2MR"}=" HB2 2MR";
$resn_exch{"2HB  2MR"}=" HB3 2MR"; $resn_exch{"1HG  2MR"}=" HG2 2MR"; $resn_exch{"2HG  2MR"}=" HG3 2MR";
$resn_exch{"1HD  2MR"}=" HD2 2MR"; $resn_exch{"2HD  2MR"}=" HD3 2MR"; $resn_exch{"1HQ1 2MR"}="HQ11 2MR";
$resn_exch{"2HQ1 2MR"}="HQ12 2MR"; $resn_exch{"3HQ1 2MR"}="HQ13 2MR"; $resn_exch{"1HH2 2MR"}=" HH2 2MR";
$resn_exch{"1HQ2 2MR"}="HQ21 2MR"; $resn_exch{"2HQ2 2MR"}="HQ22 2MR"; $resn_exch{"3HQ2 2MR"}="HQ23 2MR";
$resn_exch{" HN  2MT"}=" H   2MT"; $resn_exch{"2HB  2MT"}=" HB2 2MT"; $resn_exch{"1HB  2MT"}=" HB3 2MT";
$resn_exch{"1HD2 2MT"}="HD21 2MT"; $resn_exch{"2HD2 2MT"}="HD22 2MT"; $resn_exch{"3HD2 2MT"}="HD23 2MT";
$resn_exch{"1HD3 2MT"}="HD31 2MT"; $resn_exch{"2HD3 2MT"}="HD32 2MT"; $resn_exch{"3HD3 2MT"}="HD33 2MT";
$resn_exch{"1HM5 2MU"}="HM51 2MU"; $resn_exch{"2HM5 2MU"}="HM52 2MU"; $resn_exch{"3HM5 2MU"}="HM53 2MU";
$resn_exch{" H1* 2MU"}=" H1' 2MU"; $resn_exch{" H2* 2MU"}=" H2' 2MU"; $resn_exch{" H3* 2MU"}=" H3' 2MU";
$resn_exch{" H4* 2MU"}=" H4' 2MU"; $resn_exch{"*HO3 2MU"}="HO3' 2MU"; $resn_exch{"1H5* 2MU"}=" H5' 2MU";
$resn_exch{"2H5* 2MU"}="H5'' 2MU"; $resn_exch{"1H6* 2MU"}="H6'1 2MU"; $resn_exch{"2H6* 2MU"}="H6'2 2MU";
$resn_exch{"3H6* 2MU"}="H6'3 2MU"; $resn_exch{"2HOP 2MU"}="HOP2 2MU"; $resn_exch{"3HOP 2MU"}="HOP3 2MU";
$resn_exch{"1H6  2MZ"}=" H61 2MZ"; $resn_exch{"2H6  2MZ"}=" H62 2MZ"; $resn_exch{"3H6  2MZ"}=" H63 2MZ";
$resn_exch{" H1* 2ND"}=" H1' 2ND"; $resn_exch{"1H2* 2ND"}="H2'1 2ND"; $resn_exch{"2H2* 2ND"}="H2'2 2ND";
$resn_exch{" H3* 2ND"}=" H3' 2ND"; $resn_exch{" H4* 2ND"}=" H4' 2ND"; $resn_exch{"1H5* 2ND"}="H5'1 2ND";
$resn_exch{"2H5* 2ND"}="H5'2 2ND"; $resn_exch{" H5* 2ND"}=" H5' 2ND"; $resn_exch{"1HD1 2NH"}="HD11 2NH";
$resn_exch{"2HD1 2NH"}="HD12 2NH"; $resn_exch{"3HD1 2NH"}="HD13 2NH"; $resn_exch{"1HG4 2NH"}="HG41 2NH";
$resn_exch{"2HG4 2NH"}="HG42 2NH"; $resn_exch{"1HG3 2NH"}="HG31 2NH"; $resn_exch{"2HG3 2NH"}="HG32 2NH";
$resn_exch{"3HG3 2NH"}="HG33 2NH"; $resn_exch{"1HB2 2NH"}="HB21 2NH"; $resn_exch{"2HB2 2NH"}="HB22 2NH";
$resn_exch{"1H   2NH"}=" H1  2NH"; $resn_exch{"2H   2NH"}=" H2A 2NH"; $resn_exch{"1H4  2NH"}=" H41 2NH";
$resn_exch{"2H4  2NH"}=" H42 2NH"; $resn_exch{"3H4  2NH"}=" H43 2NH"; $resn_exch{"1H3  2NH"}=" H31 2NH";
$resn_exch{"2H3  2NH"}=" H32 2NH"; $resn_exch{"1H5  2NH"}=" H51 2NH"; $resn_exch{"2H5  2NH"}=" H52 2NH";
$resn_exch{"1H8  2NH"}=" H81 2NH"; $resn_exch{"2H8  2NH"}=" H82 2NH"; $resn_exch{"3H8  2NH"}=" H83 2NH";
$resn_exch{"1H9  2NH"}=" H91 2NH"; $resn_exch{"2H9  2NH"}=" H92 2NH"; $resn_exch{"1H10 2NH"}="H101 2NH";
$resn_exch{"2H10 2NH"}="H102 2NH"; $resn_exch{"1H13 2NH"}="H131 2NH"; $resn_exch{"2H13 2NH"}="H132 2NH";
$resn_exch{"3H13 2NH"}="H133 2NH"; $resn_exch{"1H14 2NH"}="H141 2NH"; $resn_exch{"2H14 2NH"}="H142 2NH";
$resn_exch{"3H14 2NH"}="H143 2NH"; $resn_exch{"1HA  2NH"}=" HA1 2NH"; $resn_exch{"2HA  2NH"}="HA2A 2NH";
$resn_exch{"3HA  2NH"}="HA3A 2NH"; $resn_exch{"1HG1 2NH"}="HG11 2NH"; $resn_exch{"2HG1 2NH"}="HG12 2NH";
$resn_exch{"3HG1 2NH"}="HG13 2NH"; $resn_exch{"1HG2 2NH"}="HG21 2NH"; $resn_exch{"2HG2 2NH"}="HG22 2NH";
$resn_exch{"3HG2 2NH"}="HG23 2NH"; $resn_exch{"1HB5 2NH"}="HB51 2NH"; $resn_exch{"2HB5 2NH"}="HB52 2NH";
$resn_exch{"1HG  2NH"}=" HG1 2NH"; $resn_exch{"2HG  2NH"}=" HG2 2NH"; $resn_exch{"1HE  2NH"}=" HE1 2NH";
$resn_exch{"2HE  2NH"}=" HE2 2NH"; $resn_exch{"3HE  2NH"}=" HE3 2NH"; $resn_exch{"1H14 2NI"}="H141 2NI";
$resn_exch{"2H14 2NI"}="H142 2NI"; $resn_exch{"1H15 2NI"}="H151 2NI"; $resn_exch{"2H15 2NI"}="H152 2NI";
$resn_exch{"1H16 2NI"}="H161 2NI"; $resn_exch{"2H16 2NI"}="H162 2NI"; $resn_exch{"3H16 2NI"}="H163 2NI";
$resn_exch{"1H2  2NP"}=" H21 2NP"; $resn_exch{"1HN2 2NP"}="HN21 2NP"; $resn_exch{"2HN2 2NP"}="HN22 2NP";
$resn_exch{"1H3  2NP"}=" H31 2NP"; $resn_exch{"2H3  2NP"}=" H32 2NP"; $resn_exch{"1H4  2NP"}=" H41 2NP";
$resn_exch{"2H4  2NP"}=" H42 2NP"; $resn_exch{"1H5  2NP"}=" H51 2NP"; $resn_exch{"2H5  2NP"}=" H52 2NP";
$resn_exch{"1H8  2NP"}=" H81 2NP"; $resn_exch{"2H8  2NP"}=" H82 2NP"; $resn_exch{"3HOP 2NT"}="HOP3 2NT";
$resn_exch{"2HOP 2NT"}="HOP2 2NT"; $resn_exch{"1H5* 2NT"}=" H5' 2NT"; $resn_exch{"2H5* 2NT"}="H5'' 2NT";
$resn_exch{" H4* 2NT"}=" H4' 2NT"; $resn_exch{" H1* 2NT"}=" H1' 2NT"; $resn_exch{"1H5M 2NT"}=" H71 2NT";
$resn_exch{"2H5M 2NT"}=" H72 2NT"; $resn_exch{"3H5M 2NT"}=" H73 2NT"; $resn_exch{" H2* 2NT"}=" H2' 2NT";
$resn_exch{"1HB' 2NT"}="HB'1 2NT"; $resn_exch{"2HB' 2NT"}="HB'2 2NT"; $resn_exch{"1HC' 2NT"}="HC'1 2NT";
$resn_exch{"2HC' 2NT"}="HC'2 2NT"; $resn_exch{"1HF' 2NT"}="HF'1 2NT"; $resn_exch{"2HF' 2NT"}="HF'2 2NT";
$resn_exch{" H3* 2NT"}=" H3' 2NT"; $resn_exch{"H3T  2NT"}="HO3' 2NT"; $resn_exch{"1H3  2O7"}=" H31 2O7";
$resn_exch{"2H3  2O7"}=" H32 2O7"; $resn_exch{"1H9  2O7"}=" H91 2O7"; $resn_exch{"2H9  2O7"}=" H92 2O7";
$resn_exch{"1H6  2O7"}=" H61 2O7"; $resn_exch{"2H6  2O7"}=" H62 2O7"; $resn_exch{"1H29 2O7"}="H291 2O7";
$resn_exch{"2H29 2O7"}="H292 2O7"; $resn_exch{"1H2  2OB"}=" H21 2OB"; $resn_exch{"2H2  2OB"}=" H22 2OB";
$resn_exch{"1H4  2OB"}=" H41 2OB"; $resn_exch{"2H4  2OB"}=" H42 2OB"; $resn_exch{"1H11 2OB"}="H111 2OB";
$resn_exch{"2H11 2OB"}="H112 2OB"; $resn_exch{"1H12 2OB"}="H121 2OB"; $resn_exch{"2H12 2OB"}="H122 2OB";
$resn_exch{"1H18 2OB"}="H181 2OB"; $resn_exch{"2H18 2OB"}="H182 2OB"; $resn_exch{"3H18 2OB"}="H183 2OB";
$resn_exch{"1H22 2OB"}="H221 2OB"; $resn_exch{"2H22 2OB"}="H222 2OB"; $resn_exch{"1H23 2OB"}="H231 2OB";
$resn_exch{"2H23 2OB"}="H232 2OB"; $resn_exch{"1H24 2OB"}="H241 2OB"; $resn_exch{"2H24 2OB"}="H242 2OB";
$resn_exch{"1H27 2OB"}="H271 2OB"; $resn_exch{"2H27 2OB"}="H272 2OB"; $resn_exch{"3H27 2OB"}="H273 2OB";
$resn_exch{"1H26 2OB"}="H261 2OB"; $resn_exch{"2H26 2OB"}="H262 2OB"; $resn_exch{"3H26 2OB"}="H263 2OB";
$resn_exch{"1H21 2OB"}="H211 2OB"; $resn_exch{"2H21 2OB"}="H212 2OB"; $resn_exch{"3H21 2OB"}="H213 2OB";
$resn_exch{"1H16 2OB"}="H161 2OB"; $resn_exch{"2H16 2OB"}="H162 2OB"; $resn_exch{"1H15 2OB"}="H151 2OB";
$resn_exch{"2H15 2OB"}="H152 2OB"; $resn_exch{"1H7  2OB"}=" H71 2OB"; $resn_exch{"2H7  2OB"}=" H72 2OB";
$resn_exch{"1H19 2OB"}="H191 2OB"; $resn_exch{"2H19 2OB"}="H192 2OB"; $resn_exch{"3H19 2OB"}="H193 2OB";
$resn_exch{"1H1  2OB"}=" H11 2OB"; $resn_exch{"2H1  2OB"}=" H12 2OB"; $resn_exch{"1H47 2OB"}="H471 2OB";
$resn_exch{"2H47 2OB"}="H472 2OB"; $resn_exch{"1H46 2OB"}="H461 2OB"; $resn_exch{"2H46 2OB"}="H462 2OB";
$resn_exch{"1H45 2OB"}="H451 2OB"; $resn_exch{"2H45 2OB"}="H452 2OB"; $resn_exch{"1H44 2OB"}="H441 2OB";
$resn_exch{"2H44 2OB"}="H442 2OB"; $resn_exch{"1H43 2OB"}="H431 2OB"; $resn_exch{"2H43 2OB"}="H432 2OB";
$resn_exch{"1H42 2OB"}="H421 2OB"; $resn_exch{"2H42 2OB"}="H422 2OB"; $resn_exch{"1H41 2OB"}="H411 2OB";
$resn_exch{"2H41 2OB"}="H412 2OB"; $resn_exch{"1H38 2OB"}="H381 2OB"; $resn_exch{"2H38 2OB"}="H382 2OB";
$resn_exch{"1H37 2OB"}="H371 2OB"; $resn_exch{"2H37 2OB"}="H372 2OB"; $resn_exch{"1H36 2OB"}="H361 2OB";
$resn_exch{"2H36 2OB"}="H362 2OB"; $resn_exch{"1H35 2OB"}="H351 2OB"; $resn_exch{"2H35 2OB"}="H352 2OB";
$resn_exch{"1H34 2OB"}="H341 2OB"; $resn_exch{"2H34 2OB"}="H342 2OB"; $resn_exch{"1H33 2OB"}="H331 2OB";
$resn_exch{"2H33 2OB"}="H332 2OB"; $resn_exch{"1H32 2OB"}="H321 2OB"; $resn_exch{"2H32 2OB"}="H322 2OB";
$resn_exch{"1H31 2OB"}="H311 2OB"; $resn_exch{"2H31 2OB"}="H312 2OB"; $resn_exch{"3H31 2OB"}="H313 2OB";
$resn_exch{"1HO1 2OF"}="HO11 2OF"; $resn_exch{"2HO1 2OF"}="HO12 2OF"; $resn_exch{"1HO2 2OF"}="HO21 2OF";
$resn_exch{"2HO2 2OF"}="HO22 2OF"; $resn_exch{"1H3  2OG"}=" H31 2OG"; $resn_exch{"2H3  2OG"}=" H32 2OG";
$resn_exch{"1H4  2OG"}=" H41 2OG"; $resn_exch{"2H4  2OG"}=" H42 2OG"; $resn_exch{"1H3  2OH"}=" H31 2OH";
$resn_exch{"2H3  2OH"}=" H32 2OH"; $resn_exch{"3H3  2OH"}=" H33 2OH"; $resn_exch{"1H5  2OH"}=" H51 2OH";
$resn_exch{"2H5  2OH"}=" H52 2OH"; $resn_exch{"3H5  2OH"}=" H53 2OH"; $resn_exch{"1HB  2OP"}=" HB1 2OP";
$resn_exch{"2HB  2OP"}=" HB2 2OP"; $resn_exch{"3HB  2OP"}=" HB3 2OP"; $resn_exch{"1H7  2OS"}=" H71 2OS";
$resn_exch{"2H7  2OS"}=" H72 2OS"; $resn_exch{"1H8  2OS"}=" H81 2OS"; $resn_exch{"2H8  2OS"}=" H82 2OS";
$resn_exch{"1H9  2OS"}=" H91 2OS"; $resn_exch{"2H9  2OS"}=" H92 2OS"; $resn_exch{"1H10 2OS"}="H101 2OS";
$resn_exch{"2H10 2OS"}="H102 2OS"; $resn_exch{"1H11 2OS"}="H111 2OS"; $resn_exch{"2H11 2OS"}="H112 2OS";
$resn_exch{"1H12 2OS"}="H121 2OS"; $resn_exch{"2H12 2OS"}="H122 2OS"; $resn_exch{"1H17 2OS"}="H171 2OS";
$resn_exch{"2H17 2OS"}="H172 2OS"; $resn_exch{"1H18 2OS"}="H181 2OS"; $resn_exch{"2H18 2OS"}="H182 2OS";
$resn_exch{"1H32 2OS"}="H321 2OS"; $resn_exch{"2H32 2OS"}="H322 2OS"; $resn_exch{"1H31 2OS"}="H311 2OS";
$resn_exch{"2H31 2OS"}="H312 2OS"; $resn_exch{"3H31 2OS"}="H313 2OS"; $resn_exch{"2HOP 2OT"}="HOP2 2OT";
$resn_exch{"1H5* 2OT"}=" H5' 2OT"; $resn_exch{"2H5* 2OT"}="H5'' 2OT"; $resn_exch{" H4* 2OT"}=" H4' 2OT";
$resn_exch{" H1* 2OT"}=" H1' 2OT"; $resn_exch{"1H5M 2OT"}=" H71 2OT"; $resn_exch{"2H5M 2OT"}=" H72 2OT";
$resn_exch{"3H5M 2OT"}=" H73 2OT"; $resn_exch{" H2* 2OT"}=" H2' 2OT"; $resn_exch{"1HB' 2OT"}="HB'1 2OT";
$resn_exch{"2HB' 2OT"}="HB'2 2OT"; $resn_exch{"1HC' 2OT"}="HC'1 2OT"; $resn_exch{"2HC' 2OT"}="HC'2 2OT";
$resn_exch{"1HF' 2OT"}="HF'1 2OT"; $resn_exch{"2HF' 2OT"}="HF'2 2OT"; $resn_exch{"3HF' 2OT"}="HF'3 2OT";
$resn_exch{"1HG' 2OT"}="HG'1 2OT"; $resn_exch{"2HG' 2OT"}="HG'2 2OT"; $resn_exch{"3HG' 2OT"}="HG'3 2OT";
$resn_exch{" H3* 2OT"}=" H3' 2OT"; $resn_exch{"H3T  2OT"}="HO3' 2OT"; $resn_exch{"3HOP 2OT"}="HOP3 2OT";
$resn_exch{"1HN4 2PA"}="HN41 2PA"; $resn_exch{"2HN4 2PA"}="HN42 2PA"; $resn_exch{"1HN5 2PA"}="HN51 2PA";
$resn_exch{"2HN5 2PA"}="HN52 2PA"; $resn_exch{"1HO3 2PA"}="HO31 2PA"; $resn_exch{"1H3  2PC"}=" H31 2PC";
$resn_exch{"2H3  2PC"}=" H32 2PC"; $resn_exch{"1H4  2PC"}=" H41 2PC"; $resn_exch{"2H4  2PC"}=" H42 2PC";
$resn_exch{"1H5  2PC"}=" H51 2PC"; $resn_exch{"2H5  2PC"}=" H52 2PC"; $resn_exch{"1HO  2PE"}=" HO1 2PE";
$resn_exch{"1H2  2PE"}=" H21 2PE"; $resn_exch{"2H2  2PE"}=" H22 2PE"; $resn_exch{"1H3  2PE"}=" H31 2PE";
$resn_exch{"2H3  2PE"}=" H32 2PE"; $resn_exch{"1H5  2PE"}=" H51 2PE"; $resn_exch{"2H5  2PE"}=" H52 2PE";
$resn_exch{"1H6  2PE"}=" H61 2PE"; $resn_exch{"2H6  2PE"}=" H62 2PE"; $resn_exch{"1H8  2PE"}=" H81 2PE";
$resn_exch{"2H8  2PE"}=" H82 2PE"; $resn_exch{"1H9  2PE"}=" H91 2PE"; $resn_exch{"2H9  2PE"}=" H92 2PE";
$resn_exch{"1H11 2PE"}="H111 2PE"; $resn_exch{"2H11 2PE"}="H112 2PE"; $resn_exch{"1H12 2PE"}="H121 2PE";
$resn_exch{"2H12 2PE"}="H122 2PE"; $resn_exch{"1H14 2PE"}="H141 2PE"; $resn_exch{"2H14 2PE"}="H142 2PE";
$resn_exch{"1H15 2PE"}="H151 2PE"; $resn_exch{"2H15 2PE"}="H152 2PE"; $resn_exch{"1H17 2PE"}="H171 2PE";
$resn_exch{"2H17 2PE"}="H172 2PE"; $resn_exch{"1H18 2PE"}="H181 2PE"; $resn_exch{"2H18 2PE"}="H182 2PE";
$resn_exch{"1H20 2PE"}="H201 2PE"; $resn_exch{"2H20 2PE"}="H202 2PE"; $resn_exch{"1H21 2PE"}="H211 2PE";
$resn_exch{"2H21 2PE"}="H212 2PE"; $resn_exch{"1H23 2PE"}="H231 2PE"; $resn_exch{"2H23 2PE"}="H232 2PE";
$resn_exch{"1H24 2PE"}="H241 2PE"; $resn_exch{"2H24 2PE"}="H242 2PE"; $resn_exch{"1H26 2PE"}="H261 2PE";
$resn_exch{"2H26 2PE"}="H262 2PE"; $resn_exch{"1H27 2PE"}="H271 2PE"; $resn_exch{"2H27 2PE"}="H272 2PE";
$resn_exch{"2HO  2PE"}=" HO2 2PE"; $resn_exch{"1H3  2PG"}=" H31 2PG"; $resn_exch{"2H3  2PG"}=" H32 2PG";
$resn_exch{"3HOP 2PG"}="HOP3 2PG"; $resn_exch{"4HOP 2PG"}="HOP4 2PG"; $resn_exch{"1H3  2PH"}=" H31 2PH";
$resn_exch{"2H3  2PH"}=" H32 2PH"; $resn_exch{"1HN6 2PH"}="HN61 2PH"; $resn_exch{"2HN6 2PH"}="HN62 2PH";
$resn_exch{"1H11 2PH"}="H111 2PH"; $resn_exch{"2H11 2PH"}="H112 2PH"; $resn_exch{"2HOP 2PH"}="HOP2 2PH";
$resn_exch{"5HOP 2PH"}="HOP5 2PH"; $resn_exch{"6HOP 2PH"}="HOP6 2PH"; $resn_exch{"1HB  2PI"}=" HB1 2PI";
$resn_exch{"2HB  2PI"}=" HB2 2PI"; $resn_exch{"1HG  2PI"}=" HG1 2PI"; $resn_exch{"2HG  2PI"}=" HG2 2PI";
$resn_exch{"1HD  2PI"}=" HD1 2PI"; $resn_exch{"2HD  2PI"}=" HD2 2PI"; $resn_exch{"3HD  2PI"}=" HD3 2PI";
$resn_exch{"1H2  2PL"}=" H21 2PL"; $resn_exch{"2H2  2PL"}=" H22 2PL"; $resn_exch{"3HOP 2PL"}="HOP3 2PL";
$resn_exch{"4HOP 2PL"}="HOP4 2PL"; $resn_exch{"1H14 2PM"}="H141 2PM"; $resn_exch{"2H14 2PM"}="H142 2PM";
$resn_exch{"1H15 2PM"}="H151 2PM"; $resn_exch{"2H15 2PM"}="H152 2PM"; $resn_exch{"1H16 2PM"}="H161 2PM";
$resn_exch{"2H16 2PM"}="H162 2PM"; $resn_exch{"3H16 2PM"}="H163 2PM"; $resn_exch{"1H17 2PM"}="H171 2PM";
$resn_exch{"2H17 2PM"}="H172 2PM"; $resn_exch{"3H17 2PM"}="H173 2PM"; $resn_exch{"1H3  2PP"}=" H31 2PP";
$resn_exch{"2H3  2PP"}=" H32 2PP"; $resn_exch{"1H4  2PP"}=" H41 2PP"; $resn_exch{"2H4  2PP"}=" H42 2PP";
$resn_exch{"1H5  2PP"}=" H51 2PP"; $resn_exch{"2H5  2PP"}=" H52 2PP"; $resn_exch{"3H5  2PP"}=" H53 2PP";
$resn_exch{"1H1' 2PP"}="H1'1 2PP"; $resn_exch{"2H1' 2PP"}="H1'2 2PP"; $resn_exch{"1H2' 2PP"}="H2'1 2PP";
$resn_exch{"2H2' 2PP"}="H2'2 2PP"; $resn_exch{"1H3' 2PP"}="H3'1 2PP"; $resn_exch{"2H3' 2PP"}="H3'2 2PP";
$resn_exch{"3H3' 2PP"}="H3'3 2PP"; $resn_exch{"3HOP 2PR"}="HOP3 2PR"; $resn_exch{"2HOP 2PR"}="HOP2 2PR";
$resn_exch{"1H5* 2PR"}=" H5' 2PR"; $resn_exch{"2H5* 2PR"}="H5'' 2PR"; $resn_exch{" H4* 2PR"}=" H4' 2PR";
$resn_exch{" H3* 2PR"}=" H3' 2PR"; $resn_exch{"H3T  2PR"}="HO3' 2PR"; $resn_exch{"1H2* 2PR"}=" H2' 2PR";
$resn_exch{"2H2* 2PR"}="H2'' 2PR"; $resn_exch{" H1* 2PR"}=" H1' 2PR"; $resn_exch{"1HN2 2PR"}="HN21 2PR";
$resn_exch{"2HN2 2PR"}="HN22 2PR"; $resn_exch{" NH7 2PT"}=" H7N 2PT"; $resn_exch{"1H23 2PT"}="H231 2PT";
$resn_exch{"2H23 2PT"}="H232 2PT"; $resn_exch{"3H23 2PT"}="H233 2PT"; $resn_exch{"1H24 2PT"}="H241 2PT";
$resn_exch{"2H24 2PT"}="H242 2PT"; $resn_exch{"3H24 2PT"}="H243 2PT"; $resn_exch{"1HN5 2PT"}="HN51 2PT";
$resn_exch{"2HN5 2PT"}="HN52 2PT"; $resn_exch{"1H6  2PU"}=" H61 2PU"; $resn_exch{"2H6  2PU"}=" H62 2PU";
$resn_exch{"1H11 2PU"}="H111 2PU"; $resn_exch{"2H11 2PU"}="H112 2PU"; $resn_exch{"1H12 2PU"}="H121 2PU";
$resn_exch{"2H12 2PU"}="H122 2PU"; $resn_exch{"1H24 2PU"}="H241 2PU"; $resn_exch{"2H24 2PU"}="H242 2PU";
$resn_exch{"1H26 2PU"}="H261 2PU"; $resn_exch{"2H26 2PU"}="H262 2PU"; $resn_exch{"1H27 2PU"}="H271 2PU";
$resn_exch{"2H27 2PU"}="H272 2PU"; $resn_exch{"1H24 2PY"}="H241 2PY"; $resn_exch{"2H24 2PY"}="H242 2PY";
$resn_exch{"1H25 2PY"}="H251 2PY"; $resn_exch{"2H25 2PY"}="H252 2PY"; $resn_exch{"1HN4 2PY"}="HN41 2PY";
$resn_exch{"2HN4 2PY"}="HN42 2PY"; $resn_exch{"1HAA 2RD"}="HAA1 2RD"; $resn_exch{"2HAA 2RD"}="HAA2 2RD";
$resn_exch{"3HAA 2RD"}="HAA3 2RD"; $resn_exch{"1HAD 2RD"}="HAD1 2RD"; $resn_exch{"2HAD 2RD"}="HAD2 2RD";
$resn_exch{"1HAE 2RD"}="HAE1 2RD"; $resn_exch{"2HAE 2RD"}="HAE2 2RD"; $resn_exch{"1HAF 2RD"}="HAF1 2RD";
$resn_exch{"2HAF 2RD"}="HAF2 2RD"; $resn_exch{"1HAG 2RD"}="HAG1 2RD"; $resn_exch{"2HAG 2RD"}="HAG2 2RD";
$resn_exch{"1HAH 2RD"}="HAH1 2RD"; $resn_exch{"2HAH 2RD"}="HAH2 2RD"; $resn_exch{"1HAI 2RD"}="HAI1 2RD";
$resn_exch{"2HAI 2RD"}="HAI2 2RD"; $resn_exch{"1HAJ 2RD"}="HAJ1 2RD"; $resn_exch{"2HAJ 2RD"}="HAJ2 2RD";
$resn_exch{"1HAK 2RD"}="HAK1 2RD"; $resn_exch{"2HAK 2RD"}="HAK2 2RD"; $resn_exch{"1HAL 2RD"}="HAL1 2RD";
$resn_exch{"2HAL 2RD"}="HAL2 2RD"; $resn_exch{"1HAM 2RD"}="HAM1 2RD"; $resn_exch{"2HAM 2RD"}="HAM2 2RD";
$resn_exch{"1HAN 2RD"}="HAN1 2RD"; $resn_exch{"2HAN 2RD"}="HAN2 2RD"; $resn_exch{"1HC5 2SA"}="HC51 2SA";
$resn_exch{"2HC5 2SA"}="HC52 2SA"; $resn_exch{"1H63 2SA"}="H631 2SA"; $resn_exch{"2H63 2SA"}="H632 2SA";
$resn_exch{"1H12 2SC"}="H121 2SC"; $resn_exch{"2H12 2SC"}="H122 2SC"; $resn_exch{"1H24 2SC"}="H241 2SC";
$resn_exch{"2H24 2SC"}="H242 2SC"; $resn_exch{"1H1O 2SI"}="H1O1 2SI"; $resn_exch{"1H3O 2SI"}="H3O1 2SI";
$resn_exch{"1HO6 2SI"}="HO61 2SI"; $resn_exch{"3HOS 2SI"}="HOS3 2SI"; $resn_exch{"1H4O 2SI"}="H4O1 2SI";
$resn_exch{"1H7  2SK"}=" H71 2SK"; $resn_exch{"2H7  2SK"}=" H72 2SK"; $resn_exch{"3H7  2SK"}=" H73 2SK";
$resn_exch{"1H8  2SK"}=" H81 2SK"; $resn_exch{"2H8  2SK"}=" H82 2SK"; $resn_exch{"1H1  2SP"}=" H11 2SP";
$resn_exch{"2H1  2SP"}=" H12 2SP"; $resn_exch{"1H32 2SP"}="H321 2SP"; $resn_exch{"2H32 2SP"}="H322 2SP";
$resn_exch{"1H33 2SP"}="H331 2SP"; $resn_exch{"2H33 2SP"}="H332 2SP"; $resn_exch{"1H34 2SP"}="H341 2SP";
$resn_exch{"2H34 2SP"}="H342 2SP"; $resn_exch{"1H35 2SP"}="H351 2SP"; $resn_exch{"2H35 2SP"}="H352 2SP";
$resn_exch{"1H36 2SP"}="H361 2SP"; $resn_exch{"2H36 2SP"}="H362 2SP"; $resn_exch{"1H37 2SP"}="H371 2SP";
$resn_exch{"2H37 2SP"}="H372 2SP"; $resn_exch{"1H38 2SP"}="H381 2SP"; $resn_exch{"2H38 2SP"}="H382 2SP";
$resn_exch{"1H39 2SP"}="H391 2SP"; $resn_exch{"2H39 2SP"}="H392 2SP"; $resn_exch{"1H40 2SP"}="H401 2SP";
$resn_exch{"2H40 2SP"}="H402 2SP"; $resn_exch{"1H41 2SP"}="H411 2SP"; $resn_exch{"2H41 2SP"}="H412 2SP";
$resn_exch{"1H42 2SP"}="H421 2SP"; $resn_exch{"2H42 2SP"}="H422 2SP"; $resn_exch{"1H43 2SP"}="H431 2SP";
$resn_exch{"2H43 2SP"}="H432 2SP"; $resn_exch{"1H44 2SP"}="H441 2SP"; $resn_exch{"2H44 2SP"}="H442 2SP";
$resn_exch{"1H45 2SP"}="H451 2SP"; $resn_exch{"2H45 2SP"}="H452 2SP"; $resn_exch{"1H46 2SP"}="H461 2SP";
$resn_exch{"2H46 2SP"}="H462 2SP"; $resn_exch{"1H47 2SP"}="H471 2SP"; $resn_exch{"2H47 2SP"}="H472 2SP";
$resn_exch{"1H48 2SP"}="H481 2SP"; $resn_exch{"2H48 2SP"}="H482 2SP"; $resn_exch{"3H48 2SP"}="H483 2SP";
$resn_exch{"1H3  2SP"}=" H31 2SP"; $resn_exch{"2H3  2SP"}=" H32 2SP"; $resn_exch{"1H12 2SP"}="H121 2SP";
$resn_exch{"2H12 2SP"}="H122 2SP"; $resn_exch{"1H13 2SP"}="H131 2SP"; $resn_exch{"2H13 2SP"}="H132 2SP";
$resn_exch{"1H14 2SP"}="H141 2SP"; $resn_exch{"2H14 2SP"}="H142 2SP"; $resn_exch{"1H15 2SP"}="H151 2SP";
$resn_exch{"2H15 2SP"}="H152 2SP"; $resn_exch{"1H16 2SP"}="H161 2SP"; $resn_exch{"2H16 2SP"}="H162 2SP";
$resn_exch{"1H17 2SP"}="H171 2SP"; $resn_exch{"2H17 2SP"}="H172 2SP"; $resn_exch{"1H18 2SP"}="H181 2SP";
$resn_exch{"2H18 2SP"}="H182 2SP"; $resn_exch{"1H19 2SP"}="H191 2SP"; $resn_exch{"2H19 2SP"}="H192 2SP";
$resn_exch{"1H20 2SP"}="H201 2SP"; $resn_exch{"2H20 2SP"}="H202 2SP"; $resn_exch{"1H21 2SP"}="H211 2SP";
$resn_exch{"2H21 2SP"}="H212 2SP"; $resn_exch{"1H22 2SP"}="H221 2SP"; $resn_exch{"2H22 2SP"}="H222 2SP";
$resn_exch{"1H23 2SP"}="H231 2SP"; $resn_exch{"2H23 2SP"}="H232 2SP"; $resn_exch{"1H24 2SP"}="H241 2SP";
$resn_exch{"2H24 2SP"}="H242 2SP"; $resn_exch{"1H25 2SP"}="H251 2SP"; $resn_exch{"2H25 2SP"}="H252 2SP";
$resn_exch{"1H26 2SP"}="H261 2SP"; $resn_exch{"2H26 2SP"}="H262 2SP"; $resn_exch{"1H27 2SP"}="H271 2SP";
$resn_exch{"2H27 2SP"}="H272 2SP"; $resn_exch{"1H28 2SP"}="H281 2SP"; $resn_exch{"2H28 2SP"}="H282 2SP";
$resn_exch{"3H28 2SP"}="H283 2SP"; $resn_exch{"3HOP 2ST"}="HOP3 2ST"; $resn_exch{"2HOP 2ST"}="HOP2 2ST";
$resn_exch{"1H5A 2ST"}="H5A1 2ST"; $resn_exch{"2H5A 2ST"}="H5A2 2ST"; $resn_exch{"3H5A 2ST"}="H5A3 2ST";
$resn_exch{" H2* 2ST"}=" H2' 2ST"; $resn_exch{"1H5* 2ST"}=" H5' 2ST"; $resn_exch{"2H5* 2ST"}="H5'' 2ST";
$resn_exch{" H4* 2ST"}=" H4' 2ST"; $resn_exch{" H1* 2ST"}=" H1' 2ST"; $resn_exch{" H3* 2ST"}=" H3' 2ST";
$resn_exch{"1HA* 2ST"}="HA'1 2ST"; $resn_exch{"2HA* 2ST"}="HA'2 2ST"; $resn_exch{"3HA* 2ST"}="HA'3 2ST";
$resn_exch{"H3T  2ST"}="HO3' 2ST"; $resn_exch{"1H2  2TB"}=" H21 2TB"; $resn_exch{"2H2  2TB"}=" H22 2TB";
$resn_exch{"1HN1 2TB"}="HN11 2TB"; $resn_exch{"2HN1 2TB"}="HN12 2TB"; $resn_exch{"3HN1 2TB"}="HN13 2TB";
$resn_exch{"1HN3 2TB"}="HN31 2TB"; $resn_exch{"2HN3 2TB"}="HN32 2TB"; $resn_exch{"3HN3 2TB"}="HN33 2TB";
$resn_exch{"1H8  2TH"}=" H81 2TH"; $resn_exch{"2H8  2TH"}=" H82 2TH"; $resn_exch{"1HN2 2TN"}="HN21 2TN";
$resn_exch{"2HN2 2TN"}="HN22 2TN"; $resn_exch{"1H10 2TN"}="H101 2TN"; $resn_exch{"2H10 2TN"}="H102 2TN";
$resn_exch{"1H1  2TN"}=" H11 2TN"; $resn_exch{"2H1  2TN"}="H12A 2TN"; $resn_exch{"1H3  2TN"}=" H31 2TN";
$resn_exch{"2H3  2TN"}=" H32 2TN"; $resn_exch{"1H14 2TN"}="H141 2TN"; $resn_exch{"2H14 2TN"}="H142 2TN";
$resn_exch{"3H14 2TN"}="H143 2TN"; $resn_exch{"1H13 2TN"}="H131 2TN"; $resn_exch{"2H13 2TN"}="H132 2TN";
$resn_exch{"3H13 2TN"}="H133 2TN"; $resn_exch{"1H2A 2TP"}="H2A1 2TP"; $resn_exch{"2H2A 2TP"}="H2A2 2TP";
$resn_exch{"3H2A 2TP"}="H2A3 2TP"; $resn_exch{" H4* 2TP"}=" H4' 2TP"; $resn_exch{" H6* 2TP"}=" H6' 2TP";
$resn_exch{"1H35 2TP"}="H351 2TP"; $resn_exch{"2H35 2TP"}="H352 2TP"; $resn_exch{"1H4A 2TP"}="H4A1 2TP";
$resn_exch{"2H4A 2TP"}="H4A2 2TP"; $resn_exch{"3H4A 2TP"}="H4A3 2TP"; $resn_exch{"1H5A 2TP"}="H5A1 2TP";
$resn_exch{"2H5A 2TP"}="H5A2 2TP"; $resn_exch{"1H5B 2TP"}="H5B1 2TP"; $resn_exch{"2H5B 2TP"}="H5B2 2TP";
$resn_exch{"1HN  2TY"}=" H   2TY"; $resn_exch{"2HN  2TY"}=" H2  2TY"; $resn_exch{"1HB  2TY"}=" HB2 2TY";
$resn_exch{"2HB  2TY"}=" HB3 2TY"; $resn_exch{" HOH 2TY"}=" HH  2TY"; $resn_exch{"1H2  2TY"}=" H21 2TY";
$resn_exch{"2H2  2TY"}=" H22 2TY"; $resn_exch{"1H19 2UN"}="H191 2UN"; $resn_exch{"2H19 2UN"}="H192 2UN";
$resn_exch{"1H17 2UN"}="H171 2UN"; $resn_exch{"2H17 2UN"}="H172 2UN"; $resn_exch{"3H17 2UN"}="H173 2UN";
$resn_exch{"1H18 2UN"}="H181 2UN"; $resn_exch{"2H18 2UN"}="H182 2UN"; $resn_exch{"3H18 2UN"}="H183 2UN";
$resn_exch{"1H10 2UN"}="H101 2UN"; $resn_exch{"2H10 2UN"}="H102 2UN"; $resn_exch{"1HG2 2VA"}="HG21 2VA";
$resn_exch{"2HG2 2VA"}="HG22 2VA"; $resn_exch{"3HG2 2VA"}="HG23 2VA"; $resn_exch{"1HG1 2VA"}="HG11 2VA";
$resn_exch{"2HG1 2VA"}="HG12 2VA"; $resn_exch{"3HG1 2VA"}="HG13 2VA"; $resn_exch{"1HN  2VA"}=" H   2VA";
$resn_exch{"2HN  2VA"}=" H2  2VA"; $resn_exch{" H2* 2VA"}=" H2' 2VA"; $resn_exch{" H3* 2VA"}=" H3' 2VA";
$resn_exch{" H4* 2VA"}=" H4' 2VA"; $resn_exch{"1H5* 2VA"}="H5'1 2VA"; $resn_exch{"2H5* 2VA"}="H5'2 2VA";
$resn_exch{" H5* 2VA"}=" H5' 2VA"; $resn_exch{" H1* 2VA"}=" H1' 2VA"; $resn_exch{" H2  2VA"}=" HC2 2VA";
$resn_exch{"1HN6 2VA"}="HN61 2VA"; $resn_exch{"2HN6 2VA"}="HN62 2VA"; $resn_exch{"1H13 300"}="H131 300";
$resn_exch{"2H13 300"}="H132 300"; $resn_exch{"3H13 300"}="H133 300"; $resn_exch{"1H13 302"}="H131 302";
$resn_exch{"2H13 302"}="H132 302"; $resn_exch{"1H12 302"}="H121 302"; $resn_exch{"2H12 302"}="H122 302";
$resn_exch{"1H19 303"}="H191 303"; $resn_exch{"2H19 303"}="H192 303"; $resn_exch{"1H32 303"}="H321 303";
$resn_exch{"2H32 303"}="H322 303"; $resn_exch{"1H33 303"}="H331 303"; $resn_exch{"2H33 303"}="H332 303";
$resn_exch{"1H42 303"}="H421 303"; $resn_exch{"2H42 303"}="H422 303"; $resn_exch{"3H42 303"}="H423 303";
$resn_exch{"1H46 303"}="H461 303"; $resn_exch{"2H46 303"}="H462 303"; $resn_exch{"3H46 303"}="H463 303";
$resn_exch{"1H13 306"}="H131 306"; $resn_exch{"2H13 306"}="H132 306"; $resn_exch{"1H25 306"}="H251 306";
$resn_exch{"2H25 306"}="H252 306"; $resn_exch{"1H26 306"}="H261 306"; $resn_exch{"2H26 306"}="H262 306";
$resn_exch{"1H27 306"}="H271 306"; $resn_exch{"2H27 306"}="H272 306"; $resn_exch{"1H28 306"}="H281 306";
$resn_exch{"2H28 306"}="H282 306"; $resn_exch{"1H29 306"}="H291 306"; $resn_exch{"2H29 306"}="H292 306";
$resn_exch{"1H14 307"}="H141 307"; $resn_exch{"2H14 307"}="H142 307"; $resn_exch{"1H22 307"}="H221 307";
$resn_exch{"2H22 307"}="H222 307"; $resn_exch{"3H22 307"}="H223 307"; $resn_exch{"1H23 307"}="H231 307";
$resn_exch{"2H23 307"}="H232 307"; $resn_exch{"1H24 307"}="H241 307"; $resn_exch{"2H24 307"}="H242 307";
$resn_exch{"1H28 307"}="H281 307"; $resn_exch{"2H28 307"}="H282 307"; $resn_exch{"3H28 307"}="H283 307";
$resn_exch{"1H26 307"}="H261 307"; $resn_exch{"2H26 307"}="H262 307"; $resn_exch{"1H39 307"}="H391 307";
$resn_exch{"2H39 307"}="H392 307"; $resn_exch{"3H39 307"}="H393 307"; $resn_exch{"1H37 307"}="H371 307";
$resn_exch{"2H37 307"}="H372 307"; $resn_exch{"3H37 307"}="H373 307"; $resn_exch{"H3'' 312"}=" H3B 312";
$resn_exch{"H4'' 312"}=" H4B 312"; $resn_exch{"H5'' 312"}=" H5B 312"; $resn_exch{"H6'' 312"}=" H6B 312";
$resn_exch{"1H17 326"}="H171 326"; $resn_exch{"2H17 326"}="H172 326"; $resn_exch{"3H17 326"}="H173 326";
$resn_exch{"1H21 326"}="H211 326"; $resn_exch{"2H21 326"}="H212 326"; $resn_exch{"3H21 326"}="H213 326";
$resn_exch{"1H7  32P"}=" H71 32P"; $resn_exch{"2H7  32P"}=" H72 32P"; $resn_exch{"1H15 32P"}="H151 32P";
$resn_exch{"2H15 32P"}="H152 32P"; $resn_exch{"1H18 32P"}="H181 32P"; $resn_exch{"2H18 32P"}="H182 32P";
$resn_exch{"3H18 32P"}="H183 32P"; $resn_exch{"1H19 32P"}="H191 32P"; $resn_exch{"2H19 32P"}="H192 32P";
$resn_exch{"1H20 32P"}="H201 32P"; $resn_exch{"2H20 32P"}="H202 32P"; $resn_exch{"1H21 32P"}="H211 32P";
$resn_exch{"2H21 32P"}="H212 32P"; $resn_exch{"1H22 32P"}="H221 32P"; $resn_exch{"2H22 32P"}="H222 32P";
$resn_exch{"1HN3 32P"}="HN31 32P"; $resn_exch{"2HN3 32P"}="HN32 32P"; $resn_exch{"1H23 32P"}="H231 32P";
$resn_exch{"2H23 32P"}="H232 32P"; $resn_exch{"1HB  32S"}=" HB1 32S"; $resn_exch{"2HB  32S"}=" HB2 32S";
$resn_exch{"1HB  32T"}=" HB1 32T"; $resn_exch{"2HB  32T"}=" HB2 32T"; $resn_exch{"1H21 330"}="H211 330";
$resn_exch{"2H21 330"}="H212 330"; $resn_exch{"1H22 330"}="H221 330"; $resn_exch{"2H22 330"}="H222 330";
$resn_exch{"1H23 330"}="H231 330"; $resn_exch{"2H23 330"}="H232 330"; $resn_exch{"1H24 330"}="H241 330";
$resn_exch{"2H24 330"}="H242 330"; $resn_exch{"3H24 330"}="H243 330"; $resn_exch{"1H38 333"}="H381 333";
$resn_exch{"2H38 333"}="H382 333"; $resn_exch{"1H32 335"}="H321 335"; $resn_exch{"2H32 335"}="H322 335";
$resn_exch{"1H12 335"}="H121 335"; $resn_exch{"2H12 335"}="H122 335"; $resn_exch{"1H1  336"}=" H11 336";
$resn_exch{"2H1  336"}=" H12 336"; $resn_exch{"1H6  336"}=" H61 336"; $resn_exch{"2H6  336"}=" H62 336";
$resn_exch{"1H9  336"}=" H91 336"; $resn_exch{"2H9  336"}=" H92 336"; $resn_exch{"1H13 336"}="H131 336";
$resn_exch{"2H13 336"}="H132 336"; $resn_exch{"1H22 336"}="H221 336"; $resn_exch{"2H22 336"}="H222 336";
$resn_exch{"1H25 336"}="H251 336"; $resn_exch{"2H25 336"}="H252 336"; $resn_exch{"1H43 336"}="H431 336";
$resn_exch{"2H43 336"}="H432 336"; $resn_exch{"1H46 336"}="H461 336"; $resn_exch{"2H46 336"}="H462 336";
$resn_exch{"1H51 336"}="H511 336"; $resn_exch{"2H51 336"}="H512 336"; $resn_exch{"1H56 336"}="H561 336";
$resn_exch{"2H56 336"}="H562 336"; $resn_exch{"1H58 336"}="H581 336"; $resn_exch{"2H58 336"}="H582 336";
$resn_exch{"1H61 336"}="H611 336"; $resn_exch{"2H61 336"}="H612 336"; $resn_exch{"1H2  339"}="H21A 339";
$resn_exch{"2H2  339"}=" H22 339"; $resn_exch{"1H3  339"}=" H31 339"; $resn_exch{"2H3  339"}=" H32 339";
$resn_exch{"1H4  339"}=" H41 339"; $resn_exch{"2H4  339"}=" H42 339"; $resn_exch{"1HN7 339"}="HN71 339";
$resn_exch{"2HN7 339"}="HN72 339"; $resn_exch{"1HN8 339"}="HN81 339"; $resn_exch{"2HN8 339"}="HN82 339";
$resn_exch{"1H18 339"}="H181 339"; $resn_exch{"2H18 339"}="H182 339"; $resn_exch{"3H18 339"}="H183 339";
$resn_exch{"1H19 339"}="H191 339"; $resn_exch{"2H19 339"}="H192 339"; $resn_exch{"3H19 339"}="H193 339";
$resn_exch{"1H24 339"}="H241 339"; $resn_exch{"2H24 339"}="H242 339"; $resn_exch{"1H25 339"}="H251 339";
$resn_exch{"2H25 339"}="H252 339"; $resn_exch{"1H26 339"}="H261 339"; $resn_exch{"2H26 339"}="H262 339";
$resn_exch{"1H29 339"}="H291 339"; $resn_exch{"2H29 339"}="H292 339"; $resn_exch{"1H42 339"}="H421 339";
$resn_exch{"2H42 339"}="H422 339"; $resn_exch{"3H42 339"}="H423 339"; $resn_exch{"1H23 33A"}="H231 33A";
$resn_exch{"2H23 33A"}="H232 33A"; $resn_exch{"1H2A 33P"}="H2A1 33P"; $resn_exch{"2H2A 33P"}="H2A2 33P";
$resn_exch{"3H2A 33P"}="H2A3 33P"; $resn_exch{"1H4A 33P"}="H4A1 33P"; $resn_exch{"2H4A 33P"}="H4A2 33P";
$resn_exch{"1H5A 33P"}="H5A1 33P"; $resn_exch{"2H5A 33P"}="H5A2 33P"; $resn_exch{"2HOA 33P"}="HOA2 33P";
$resn_exch{"3HOA 33P"}="HOA3 33P"; $resn_exch{"2HOB 33P"}="HOB2 33P"; $resn_exch{"3HOB 33P"}="HOB3 33P";
$resn_exch{"1H8  33P"}=" H81 33P"; $resn_exch{"2H8  33P"}=" H82 33P"; $resn_exch{"1H10 33P"}="H101 33P";
$resn_exch{"2H10 33P"}="H102 33P"; $resn_exch{"3H10 33P"}="H103 33P"; $resn_exch{"1H47 33Z"}="H471 33Z";
$resn_exch{"2H47 33Z"}="H472 33Z"; $resn_exch{"1H35 33Z"}="H351 33Z"; $resn_exch{"2H35 33Z"}="H352 33Z";
$resn_exch{"1H30 33Z"}="H301 33Z"; $resn_exch{"2H30 33Z"}="H302 33Z"; $resn_exch{"3H30 33Z"}="H303 33Z";
$resn_exch{"1H29 33Z"}="H291 33Z"; $resn_exch{"2H29 33Z"}="H292 33Z"; $resn_exch{"3H29 33Z"}="H293 33Z";
$resn_exch{"1H23 33Z"}="H231 33Z"; $resn_exch{"2H23 33Z"}="H232 33Z"; $resn_exch{"1H11 33Z"}="H111 33Z";
$resn_exch{"2H11 33Z"}="H112 33Z"; $resn_exch{"1H1  340"}=" H11 340"; $resn_exch{"1H12 340"}="H121 340";
$resn_exch{"2H12 340"}="H122 340"; $resn_exch{"3H12 340"}="H123 340"; $resn_exch{"1H19 340"}="H191 340";
$resn_exch{"2H19 340"}="H192 340"; $resn_exch{"1H25 340"}="H251 340"; $resn_exch{"1H26 340"}="H261 340";
$resn_exch{"1H27 340"}="H271 340"; $resn_exch{"1H28 340"}="H281 340"; $resn_exch{"1H29 340"}="H291 340";
$resn_exch{"1H37 340"}="H371 340"; $resn_exch{"2H37 340"}="H372 340"; $resn_exch{"3H37 340"}="H373 340";
$resn_exch{"1H45 340"}="H451 340"; $resn_exch{"2H45 340"}="H452 340"; $resn_exch{"3H45 340"}="H453 340";
$resn_exch{"1H38 342"}="H381 342"; $resn_exch{"2H38 342"}="H382 342"; $resn_exch{"1H27 342"}="H271 342";
$resn_exch{"2H27 342"}="H272 342"; $resn_exch{"1H22 342"}="H221 342"; $resn_exch{"2H22 342"}="H222 342";
$resn_exch{"1H24 342"}="H241 342"; $resn_exch{"2H24 342"}="H242 342"; $resn_exch{"1H25 342"}="H251 342";
$resn_exch{"2H25 342"}="H252 342"; $resn_exch{"1H26 342"}="H261 342"; $resn_exch{"2H26 342"}="H262 342";
$resn_exch{"1H47 343"}="H471 343"; $resn_exch{"2H47 343"}="H472 343"; $resn_exch{"1H35 343"}="H351 343";
$resn_exch{"2H35 343"}="H352 343"; $resn_exch{"1H30 343"}="H301 343"; $resn_exch{"2H30 343"}="H302 343";
$resn_exch{"3H30 343"}="H303 343"; $resn_exch{"1H29 343"}="H291 343"; $resn_exch{"2H29 343"}="H292 343";
$resn_exch{"3H29 343"}="H293 343"; $resn_exch{"1H11 343"}="H111 343"; $resn_exch{"2H11 343"}="H112 343";
$resn_exch{"1HN2 344"}="HN21 344"; $resn_exch{"2HN2 344"}="HN22 344"; $resn_exch{"1H26 345"}="H261 345";
$resn_exch{"2H26 345"}="H262 345"; $resn_exch{"1H27 345"}="H271 345"; $resn_exch{"2H27 345"}="H272 345";
$resn_exch{"1H29 345"}="H291 345"; $resn_exch{"2H29 345"}="H292 345"; $resn_exch{"1H30 345"}="H301 345";
$resn_exch{"2H30 345"}="H302 345"; $resn_exch{"1H39 345"}="H391 345"; $resn_exch{"2H39 345"}="H392 345";
$resn_exch{"2H27 346"}="H272 346"; $resn_exch{"1H27 346"}="H271 346"; $resn_exch{"2H29 346"}="H292 346";
$resn_exch{"1H29 346"}="H291 346"; $resn_exch{"2H31 346"}="H312 346"; $resn_exch{"1H31 346"}="H311 346";
$resn_exch{"1H24 346"}="H241 346"; $resn_exch{"2H24 346"}="H242 346"; $resn_exch{"3H30 346"}="H303 346";
$resn_exch{"2H30 346"}="H302 346"; $resn_exch{"1H30 346"}="H301 346"; $resn_exch{"1H1  347"}=" H11 347";
$resn_exch{"2H1  347"}=" H12 347"; $resn_exch{"3H1  347"}=" H13 347"; $resn_exch{"1H4  347"}=" H41 347";
$resn_exch{"2H4  347"}=" H42 347"; $resn_exch{"1H11 347"}="H111 347"; $resn_exch{"2H11 347"}="H112 347";
$resn_exch{"1H13 347"}="H131 347"; $resn_exch{"2H13 347"}="H132 347"; $resn_exch{"1H14 347"}="H141 347";
$resn_exch{"2H14 347"}="H142 347"; $resn_exch{"1H15 347"}="H151 347"; $resn_exch{"2H15 347"}="H152 347";
$resn_exch{"1H16 347"}="H161 347"; $resn_exch{"2H16 347"}="H162 347"; $resn_exch{"1H19 347"}="H191 347";
$resn_exch{"2H19 347"}="H192 347"; $resn_exch{"3H19 347"}="H193 347"; $resn_exch{"1H20 347"}="H201 347";
$resn_exch{"2H20 347"}="H202 347"; $resn_exch{"3H20 347"}="H203 347"; $resn_exch{"1H21 347"}="H211 347";
$resn_exch{"2H21 347"}="H212 347"; $resn_exch{"3H21 347"}="H213 347"; $resn_exch{"1H1N 348"}="H1N1 348";
$resn_exch{"2H1N 348"}="H1N2 348"; $resn_exch{"1H12 348"}="H121 348"; $resn_exch{"2H12 348"}="H122 348";
$resn_exch{"1H13 348"}="H131 348"; $resn_exch{"2H13 348"}="H132 348"; $resn_exch{"1H14 348"}="H141 348";
$resn_exch{"2H14 348"}="H142 348"; $resn_exch{"1H23 348"}="H231 348"; $resn_exch{"2H23 348"}="H232 348";
$resn_exch{"1H28 348"}="H281 348"; $resn_exch{"2H28 348"}="H282 348"; $resn_exch{"3H28 348"}="H283 348";
$resn_exch{"1H41 348"}="H411 348"; $resn_exch{"2H41 348"}="H412 348"; $resn_exch{"3H41 348"}="H413 348";
$resn_exch{"1H42 348"}="H421 348"; $resn_exch{"2H42 348"}="H422 348"; $resn_exch{"3H42 348"}="H423 348";
$resn_exch{"1HN1 34A"}="HN11 34A"; $resn_exch{"2HN1 34A"}="HN12 34A"; $resn_exch{"1H7  34A"}=" H71 34A";
$resn_exch{"2H7  34A"}=" H72 34A"; $resn_exch{"3H7  34A"}=" H73 34A"; $resn_exch{"1H8  34A"}=" H81 34A";
$resn_exch{"2H8  34A"}=" H82 34A"; $resn_exch{"3H8  34A"}=" H83 34A"; $resn_exch{"1H5' 34B"}="H5'1 34B";
$resn_exch{"2H5' 34B"}="H5'2 34B"; $resn_exch{"1H6' 34B"}="H6'1 34B"; $resn_exch{"2H6' 34B"}="H6'2 34B";
$resn_exch{"3H6' 34B"}="H6'3 34B"; $resn_exch{"1HC2 34C"}="HC21 34C"; $resn_exch{"1HC4 34C"}="HC41 34C";
$resn_exch{"1HC5 34C"}="HC51 34C"; $resn_exch{"1HC7 34C"}="HC71 34C"; $resn_exch{"2HC7 34C"}="HC72 34C";
$resn_exch{"1HC9 34C"}="HC91 34C"; $resn_exch{"2HC9 34C"}="HC92 34C"; $resn_exch{"1H10 34C"}="H101 34C";
$resn_exch{"2H10 34C"}="H102 34C"; $resn_exch{"1H11 34C"}="H111 34C"; $resn_exch{"2H11 34C"}="H112 34C";
$resn_exch{"3H11 34C"}="H113 34C"; $resn_exch{"1H7  34P"}=" H71 34P"; $resn_exch{"2H7  34P"}=" H72 34P";
$resn_exch{"1H12 34P"}="H121 34P"; $resn_exch{"2H12 34P"}="H122 34P"; $resn_exch{"1H13 34P"}="H131 34P";
$resn_exch{"2H13 34P"}="H132 34P"; $resn_exch{"1H14 34P"}="H141 34P"; $resn_exch{"2H14 34P"}="H142 34P";
$resn_exch{"1H20 34P"}="H201 34P"; $resn_exch{"2H20 34P"}="H202 34P"; $resn_exch{"1H21 34P"}="H211 34P";
$resn_exch{"2H21 34P"}="H212 34P"; $resn_exch{"1H23 34P"}="H231 34P"; $resn_exch{"2H23 34P"}="H232 34P";
$resn_exch{"1H24 34P"}="H241 34P"; $resn_exch{"2H24 34P"}="H242 34P"; $resn_exch{"1H25 34P"}="H251 34P";
$resn_exch{"2H25 34P"}="H252 34P"; $resn_exch{"1H26 34P"}="H261 34P"; $resn_exch{"2H26 34P"}="H262 34P";
$resn_exch{"1H27 34P"}="H271 34P"; $resn_exch{"2H27 34P"}="H272 34P"; $resn_exch{"1H29 34P"}="H291 34P";
$resn_exch{"2H29 34P"}="H292 34P"; $resn_exch{"1HN2 354"}="HN21 354"; $resn_exch{"2HN2 354"}="HN22 354";
$resn_exch{"1HB  354"}=" HB1 354"; $resn_exch{"2HB  354"}=" HB2 354"; $resn_exch{"1HG  354"}=" HG1 354";
$resn_exch{"2HG  354"}=" HG2 354"; $resn_exch{"1H10 357"}="H101 357"; $resn_exch{"2H10 357"}="H102 357";
$resn_exch{"1H11 357"}="H111 357"; $resn_exch{"2H11 357"}="H112 357"; $resn_exch{"3H11 357"}="H113 357";
$resn_exch{"1H7* 357"}="H7'1 357"; $resn_exch{"2H7* 357"}="H7'2 357"; $resn_exch{"1H7  357"}=" H71 357";
$resn_exch{"2H7  357"}=" H72 357"; $resn_exch{"3H7  357"}=" H73 357"; $resn_exch{"1H6\" 357"}="H6'1 357";
$resn_exch{"2H6\" 357"}="H6'2 357"; $resn_exch{"3H6\" 357"}="H6'3 357"; $resn_exch{"1H2\" 357"}="H2'1 357";
$resn_exch{"2H2\" 357"}="H2'2 357"; $resn_exch{"1H3\" 357"}="H3'1 357"; $resn_exch{"2H3\" 357"}="H3'2 357";
$resn_exch{"1H5\" 357"}="H5'1 357"; $resn_exch{"2H5\" 357"}="H5'2 357"; $resn_exch{"3H5\" 357"}="H5'3 357";
$resn_exch{" H6* 357"}=" H6' 357"; $resn_exch{" H5* 357"}=" H5' 357"; $resn_exch{" H4* 357"}=" H4' 357";
$resn_exch{"1H8* 357"}="H8'1 357"; $resn_exch{"2H8* 357"}="H8'2 357"; $resn_exch{"3H8* 357"}="H8'3 357";
$resn_exch{" H2* 357"}=" H2' 357"; $resn_exch{"1H8  358"}=" H81 358"; $resn_exch{"2H8  358"}=" H82 358";
$resn_exch{"1H9  358"}=" H91 358"; $resn_exch{"2H9  358"}=" H92 358"; $resn_exch{"1H23 358"}="H231 358";
$resn_exch{"2H23 358"}="H232 358"; $resn_exch{"1H24 358"}="H241 358"; $resn_exch{"2H24 358"}="H242 358";
$resn_exch{"1H25 358"}="H251 358"; $resn_exch{"2H25 358"}="H252 358"; $resn_exch{"1H26 358"}="H261 358";
$resn_exch{"2H26 358"}="H262 358"; $resn_exch{"1H4  35A"}=" H41 35A"; $resn_exch{"2H4  35A"}=" H42 35A";
$resn_exch{"1H6  35A"}=" H61 35A"; $resn_exch{"2H6  35A"}=" H62 35A"; $resn_exch{"1H11 35A"}="H111 35A";
$resn_exch{"2H11 35A"}="H112 35A"; $resn_exch{"1H13 35A"}="H131 35A"; $resn_exch{"2H13 35A"}="H132 35A";
$resn_exch{"1H14 35A"}="H141 35A"; $resn_exch{"2H14 35A"}="H142 35A"; $resn_exch{"1H16 35A"}="H161 35A";
$resn_exch{"2H16 35A"}="H162 35A"; $resn_exch{"1H20 35A"}="H201 35A"; $resn_exch{"2H20 35A"}="H202 35A";
$resn_exch{"3H20 35A"}="H203 35A"; $resn_exch{"1H23 35A"}="H231 35A"; $resn_exch{"2H23 35A"}="H232 35A";
$resn_exch{"3H23 35A"}="H233 35A"; $resn_exch{"1H24 35A"}="H241 35A"; $resn_exch{"2H24 35A"}="H242 35A";
$resn_exch{"3H24 35A"}="H243 35A"; $resn_exch{"1H27 35A"}="H271 35A"; $resn_exch{"2H27 35A"}="H272 35A";
$resn_exch{"3H27 35A"}="H273 35A"; $resn_exch{"1H28 35A"}="H281 35A"; $resn_exch{"2H28 35A"}="H282 35A";
$resn_exch{"2HOP 35G"}="HOP2 35G"; $resn_exch{"1H5* 35G"}="H5'1 35G"; $resn_exch{"2H5* 35G"}="H5'2 35G";
$resn_exch{" H4* 35G"}=" H4' 35G"; $resn_exch{" H3* 35G"}=" H3' 35G"; $resn_exch{" H2* 35G"}=" H2' 35G";
$resn_exch{"*HO2 35G"}="HO2' 35G"; $resn_exch{" H1* 35G"}=" H1' 35G"; $resn_exch{"1HN2 35G"}="HN21 35G";
$resn_exch{"2HN2 35G"}="HN22 35G"; $resn_exch{"1HAU 364"}="HAU1 364"; $resn_exch{"2HAU 364"}="HAU2 364";
$resn_exch{"1H19 367"}="H191 367"; $resn_exch{"2H19 367"}="H192 367"; $resn_exch{"1H44 367"}="H441 367";
$resn_exch{"2H44 367"}="H442 367"; $resn_exch{"1H45 367"}="H451 367"; $resn_exch{"2H45 367"}="H452 367";
$resn_exch{"1H46 367"}="H461 367"; $resn_exch{"2H46 367"}="H462 367"; $resn_exch{"1H09 368"}="H091 368";
$resn_exch{"2H09 368"}="H092 368"; $resn_exch{"1H13 368"}="H131 368"; $resn_exch{"2H13 368"}="H132 368";
$resn_exch{"1HP6 368"}="HP61 368"; $resn_exch{"2HP6 368"}="HP62 368"; $resn_exch{"1H27 368"}="H271 368";
$resn_exch{"2H27 368"}="H272 368"; $resn_exch{"1H16 371"}="H161 371"; $resn_exch{"2H16 371"}="H162 371";
$resn_exch{"3H16 371"}="H163 371"; $resn_exch{"1H17 371"}="H171 371"; $resn_exch{"2H17 371"}="H172 371";
$resn_exch{"1H18 371"}="H181 371"; $resn_exch{"2H18 371"}="H182 371"; $resn_exch{"1H19 371"}="H191 371";
$resn_exch{"2H19 371"}="H192 371"; $resn_exch{"1H31 373"}="H311 373"; $resn_exch{"2H31 373"}="H312 373";
$resn_exch{"1H35 373"}="H351 373"; $resn_exch{"2H35 373"}="H352 373"; $resn_exch{"1H36 373"}="H361 373";
$resn_exch{"2H36 373"}="H362 373"; $resn_exch{"1H38 373"}="H381 373"; $resn_exch{"2H38 373"}="H382 373";
$resn_exch{"1H39 373"}="H391 373"; $resn_exch{"2H39 373"}="H392 373"; $resn_exch{"1H48 373"}="H481 373";
$resn_exch{"2H48 373"}="H482 373"; $resn_exch{"1H50 373"}="H501 373"; $resn_exch{"2H50 373"}="H502 373";
$resn_exch{"1H7  37A"}=" H71 37A"; $resn_exch{"2H7  37A"}=" H72 37A"; $resn_exch{"1H3  37A"}=" H31 37A";
$resn_exch{"2H3  37A"}=" H32 37A"; $resn_exch{"1H12 37A"}="H121 37A"; $resn_exch{"2H12 37A"}="H122 37A";
$resn_exch{"1H13 37T"}="H131 37T"; $resn_exch{"2H13 37T"}="H132 37T"; $resn_exch{"3H13 37T"}="H133 37T";
$resn_exch{"1H12 37T"}="H121 37T"; $resn_exch{"2H12 37T"}="H122 37T"; $resn_exch{"3H12 37T"}="H123 37T";
$resn_exch{"1H1N 380"}="H1N1 380"; $resn_exch{"2H1N 380"}="H1N2 380"; $resn_exch{"1H25 380"}="H251 380";
$resn_exch{"2H25 380"}="H252 380"; $resn_exch{"1H24 380"}="H241 380"; $resn_exch{"2H24 380"}="H242 380";
$resn_exch{"3H24 380"}="H243 380"; $resn_exch{"1H32 380"}="H321 380"; $resn_exch{"2H32 380"}="H322 380";
$resn_exch{"3H32 380"}="H323 380"; $resn_exch{"1H1  382"}=" H11 382"; $resn_exch{"2H1  382"}=" H12 382";
$resn_exch{"1H2  382"}=" H21 382"; $resn_exch{"2H2  382"}=" H22 382"; $resn_exch{"1H5  382"}=" H51 382";
$resn_exch{"2H5  382"}="H52A 382"; $resn_exch{"1H6  382"}=" H61 382"; $resn_exch{"2H6  382"}=" H62 382";
$resn_exch{"1H19 382"}="H191 382"; $resn_exch{"2H19 382"}="H192 382"; $resn_exch{"1H34 382"}="H341 382";
$resn_exch{"2H34 382"}="H342 382"; $resn_exch{"1H14 383"}="H141 383"; $resn_exch{"2H14 383"}="H142 383";
$resn_exch{"1H16 383"}="H161 383"; $resn_exch{"2H16 383"}="H162 383"; $resn_exch{"1H30 383"}="H301 383";
$resn_exch{"2H30 383"}="H302 383"; $resn_exch{"1H4  385"}=" H41 385"; $resn_exch{"2H4  385"}=" H42 385";
$resn_exch{"1H7  385"}=" H71 385"; $resn_exch{"2H7  385"}=" H72 385"; $resn_exch{"1H9  385"}=" H91 385";
$resn_exch{"2H9  385"}=" H92 385"; $resn_exch{"1H14 385"}="H141 385"; $resn_exch{"2H14 385"}="H142 385";
$resn_exch{"1H17 385"}="H171 385"; $resn_exch{"2H17 385"}="H172 385"; $resn_exch{"1H25 385"}="H251 385";
$resn_exch{"2H25 385"}="H252 385"; $resn_exch{"1H34 385"}="H341 385"; $resn_exch{"2H34 385"}="H342 385";
$resn_exch{"3H34 385"}="H343 385"; $resn_exch{"1H35 385"}="H351 385"; $resn_exch{"2H35 385"}="H352 385";
$resn_exch{"3H35 385"}="H353 385"; $resn_exch{"1H36 385"}="H361 385"; $resn_exch{"2H36 385"}="H362 385";
$resn_exch{"1H45 385"}="H451 385"; $resn_exch{"2H45 385"}="H452 385"; $resn_exch{"1H48 385"}="H481 385";
$resn_exch{"2H48 385"}="H482 385"; $resn_exch{"3H48 385"}="H483 385"; $resn_exch{"1H1  388"}=" H11 388";
$resn_exch{"2H1  388"}=" H12 388"; $resn_exch{"1H21 388"}="H211 388"; $resn_exch{"2H21 388"}="H212 388";
$resn_exch{"1H38 391"}="H381 391"; $resn_exch{"2H38 391"}="H382 391"; $resn_exch{"1H27 391"}="H271 391";
$resn_exch{"2H27 391"}="H272 391"; $resn_exch{"3H27 391"}="H273 391"; $resn_exch{"1H7  393"}=" H71 393";
$resn_exch{"2H7  393"}=" H72 393"; $resn_exch{"1H18 393"}="H181 393"; $resn_exch{"2H18 393"}="H182 393";
$resn_exch{"1H14 394"}="H141 394"; $resn_exch{"2H14 394"}="H142 394"; $resn_exch{"1H15 394"}="H151 394";
$resn_exch{"2H15 394"}="H152 394"; $resn_exch{"1H18 394"}="H181 394"; $resn_exch{"2H18 394"}="H182 394";
$resn_exch{"3H18 394"}="H183 394"; $resn_exch{"1H19 394"}="H191 394"; $resn_exch{"2H19 394"}="H192 394";
$resn_exch{"3H19 394"}="H193 394"; $resn_exch{"1H20 394"}="H201 394"; $resn_exch{"2H20 394"}="H202 394";
$resn_exch{"3H20 394"}="H203 394"; $resn_exch{"1H21 394"}="H211 394"; $resn_exch{"2H21 394"}="H212 394";
$resn_exch{"3H21 394"}="H213 394"; $resn_exch{"2HO2 394"}="HO22 394"; $resn_exch{"7HO2 394"}="HO27 394";
$resn_exch{"1H14 395"}="H141 395"; $resn_exch{"2H14 395"}="H142 395"; $resn_exch{"1H15 395"}="H151 395";
$resn_exch{"2H15 395"}="H152 395"; $resn_exch{"1H18 395"}="H181 395"; $resn_exch{"2H18 395"}="H182 395";
$resn_exch{"3H18 395"}="H183 395"; $resn_exch{"1H19 395"}="H191 395"; $resn_exch{"2H19 395"}="H192 395";
$resn_exch{"3H19 395"}="H193 395"; $resn_exch{"1H20 395"}="H201 395"; $resn_exch{"2H20 395"}="H202 395";
$resn_exch{"3H20 395"}="H203 395"; $resn_exch{"1H21 395"}="H211 395"; $resn_exch{"2H21 395"}="H212 395";
$resn_exch{"3H21 395"}="H213 395"; $resn_exch{"2HO2 395"}="HO22 395"; $resn_exch{"7HO2 395"}="HO27 395";
$resn_exch{"1HN6 39A"}="HN61 39A"; $resn_exch{"2HN6 39A"}="HN62 39A"; $resn_exch{"1H9M 39A"}="H9M1 39A";
$resn_exch{"2H9M 39A"}="H9M2 39A"; $resn_exch{"3H9M 39A"}="H9M3 39A"; $resn_exch{"1H3M 39A"}="H3M1 39A";
$resn_exch{"2H3M 39A"}="H3M2 39A"; $resn_exch{"3H3M 39A"}="H3M3 39A"; $resn_exch{"1H2' 39B"}="H2'1 39B";
$resn_exch{"2H2' 39B"}="H2'2 39B"; $resn_exch{"1H4' 39B"}="H4'1 39B"; $resn_exch{"2H4' 39B"}="H4'2 39B";
$resn_exch{"1H5' 39B"}="H5'1 39B"; $resn_exch{"2H5' 39B"}="H5'2 39B"; $resn_exch{"1H6' 39B"}="H6'1 39B";
$resn_exch{"2H6' 39B"}="H6'2 39B"; $resn_exch{"3H6' 39B"}="H6'3 39B"; $resn_exch{"1H2' 39E"}="H2'1 39E";
$resn_exch{"2HA  39E"}=" HA2 39E"; $resn_exch{"1HA  39E"}=" HA1 39E"; $resn_exch{"2HB  39E"}=" HB2 39E";
$resn_exch{"1HB  39E"}=" HB1 39E"; $resn_exch{"2HC  39E"}=" HC2 39E"; $resn_exch{"1HC  39E"}=" HC1 39E";
$resn_exch{"2HD  39E"}=" HD2 39E"; $resn_exch{"3HE  39E"}=" HE3 39E"; $resn_exch{"1H20 39Z"}="H201 39Z";
$resn_exch{"2H20 39Z"}="H202 39Z"; $resn_exch{"3H20 39Z"}="H203 39Z"; $resn_exch{"1H19 3A3"}="H191 3A3";
$resn_exch{"2H19 3A3"}="H192 3A3"; $resn_exch{"1H20 3A3"}="H201 3A3"; $resn_exch{"2H20 3A3"}="H202 3A3";
$resn_exch{"1H5* 3AA"}="H5'1 3AA"; $resn_exch{"2H5* 3AA"}="H5'2 3AA"; $resn_exch{"4H*  3AA"}=" H'4 3AA";
$resn_exch{"1H*  3AA"}=" H'1 3AA"; $resn_exch{"1HN6 3AA"}="HN61 3AA"; $resn_exch{"2HN6 3AA"}="HN62 3AA";
$resn_exch{"2H*  3AA"}=" H'2 3AA"; $resn_exch{"3H*  3AA"}=" H'3 3AA"; $resn_exch{"HA3* 3AA"}="HA3' 3AA";
$resn_exch{"H15* 3AA"}="H15' 3AA"; $resn_exch{"H25* 3AA"}="H25' 3AA"; $resn_exch{"HC4* 3AA"}="HC4' 3AA";
$resn_exch{" H3* 3AA"}=" H3' 3AA"; $resn_exch{"HO3* 3AA"}="HO3' 3AA"; $resn_exch{" H2* 3AA"}=" H2' 3AA";
$resn_exch{"HO2* 3AA"}="HO2' 3AA"; $resn_exch{" H1* 3AA"}=" H1' 3AA"; $resn_exch{"1HN7 3AA"}="HN71 3AA";
$resn_exch{"2HN7 3AA"}="HN72 3AA"; $resn_exch{"1H51 3AC"}="H511 3AC"; $resn_exch{"2H51 3AC"}="H512 3AC";
$resn_exch{"3H51 3AC"}="H513 3AC"; $resn_exch{"1H61 3AC"}="H611 3AC"; $resn_exch{"2H61 3AC"}="H612 3AC";
$resn_exch{"1H62 3AC"}="H621 3AC"; $resn_exch{"2H62 3AC"}="H622 3AC"; $resn_exch{"3H62 3AC"}="H623 3AC";
$resn_exch{"*HO5 3AD"}="HO5' 3AD"; $resn_exch{"1H5* 3AD"}="H5'1 3AD"; $resn_exch{"2H5* 3AD"}="H5'2 3AD";
$resn_exch{" H4* 3AD"}=" H4' 3AD"; $resn_exch{"1H3* 3AD"}="H3'1 3AD"; $resn_exch{"2H3* 3AD"}="H3'2 3AD";
$resn_exch{" H2* 3AD"}=" H2' 3AD"; $resn_exch{"*HO2 3AD"}="HO2' 3AD"; $resn_exch{" H1* 3AD"}=" H1' 3AD";
$resn_exch{"1HN6 3AD"}="HN61 3AD"; $resn_exch{"2HN6 3AD"}="HN62 3AD"; $resn_exch{"1H4  3AF"}=" H41 3AF";
$resn_exch{"3H4  3AF"}=" H43 3AF"; $resn_exch{"2H4  3AF"}=" H42 3AF"; $resn_exch{"1H5  3AF"}=" H51 3AF";
$resn_exch{"2H5  3AF"}=" H52 3AF"; $resn_exch{"3H5  3AF"}=" H53 3AF"; $resn_exch{"1H6  3AF"}=" H61 3AF";
$resn_exch{"3H6  3AF"}=" H63 3AF"; $resn_exch{"2H6  3AF"}=" H62 3AF"; $resn_exch{"1H7  3AF"}=" H71 3AF";
$resn_exch{"1HC1 3AG"}="HC11 3AG"; $resn_exch{"2HC1 3AG"}="HC12 3AG"; $resn_exch{"3HC1 3AG"}="HC13 3AG";
$resn_exch{"1HC3 3AG"}="HC31 3AG"; $resn_exch{"2HC3 3AG"}="HC32 3AG"; $resn_exch{"3HC3 3AG"}="HC33 3AG";
$resn_exch{"1HC5 3AG"}="HC51 3AG"; $resn_exch{"2HC5 3AG"}="HC52 3AG"; $resn_exch{"1HC6 3AG"}="HC61 3AG";
$resn_exch{"2HC6 3AG"}="HC62 3AG"; $resn_exch{"1HC7 3AG"}="HC71 3AG"; $resn_exch{"2HC7 3AG"}="HC72 3AG";
$resn_exch{"1HC8 3AG"}="HC81 3AG"; $resn_exch{"2HC8 3AG"}="HC82 3AG"; $resn_exch{"1HC9 3AG"}="HC91 3AG";
$resn_exch{"2HC9 3AG"}="HC92 3AG"; $resn_exch{"3HC9 3AG"}="HC93 3AG"; $resn_exch{" HN2 3AH"}=" H2  3AH";
$resn_exch{"1HB  3AH"}=" HB2 3AH"; $resn_exch{"2HB  3AH"}=" HB3 3AH"; $resn_exch{"1HN3 3AH"}="HN31 3AH";
$resn_exch{"2HN3 3AH"}="HN32 3AH"; $resn_exch{"1HN1 3AL"}="HN11 3AL"; $resn_exch{"2HN1 3AL"}="HN12 3AL";
$resn_exch{"HO2* 3AM"}="HO2' 3AM"; $resn_exch{" H2* 3AM"}=" H2' 3AM"; $resn_exch{" H3* 3AM"}=" H3' 3AM";
$resn_exch{" H1* 3AM"}=" H1' 3AM"; $resn_exch{" H4* 3AM"}=" H4' 3AM"; $resn_exch{" H5* 3AM"}=" H5' 3AM";
$resn_exch{"H5*A 3AM"}="H5'A 3AM"; $resn_exch{"HO5* 3AM"}="HO5' 3AM"; $resn_exch{"2HOB 3AN"}="HOB2 3AN";
$resn_exch{"3HOB 3AN"}="HOB3 3AN"; $resn_exch{"1HOA 3AN"}="HOA1 3AN"; $resn_exch{"1H5* 3AN"}="H5'1 3AN";
$resn_exch{"2H5* 3AN"}="H5'2 3AN"; $resn_exch{"4HC* 3AN"}="HC'4 3AN"; $resn_exch{"3HC* 3AN"}="HC'3 3AN";
$resn_exch{"1H3* 3AN"}="H3'1 3AN"; $resn_exch{"2H3* 3AN"}="H3'2 3AN"; $resn_exch{"2HC* 3AN"}="HC'2 3AN";
$resn_exch{"2HO* 3AN"}="HO'2 3AN"; $resn_exch{"1HC* 3AN"}="HC'1 3AN"; $resn_exch{"8HC  3AN"}=" HC8 3AN";
$resn_exch{"1HN6 3AN"}="HN61 3AN"; $resn_exch{"2HN6 3AN"}="HN62 3AN"; $resn_exch{"2HC  3AN"}=" HC2 3AN";
$resn_exch{"1HN3 3AP"}="HN31 3AP"; $resn_exch{"2HN3 3AP"}="HN32 3AP"; $resn_exch{" HN2 3AR"}=" H2  3AR";
$resn_exch{"1HB  3AR"}=" HB2 3AR"; $resn_exch{"2HB  3AR"}=" HB3 3AR"; $resn_exch{"1HG  3AR"}=" HG2 3AR";
$resn_exch{"2HG  3AR"}=" HG3 3AR"; $resn_exch{"1HD  3AR"}=" HD2 3AR"; $resn_exch{"2HD  3AR"}=" HD3 3AR";
$resn_exch{"1HH1 3AR"}=" HH1 3AR"; $resn_exch{"1HH2 3AR"}="HH21 3AR"; $resn_exch{"2HH2 3AR"}="HH22 3AR";
$resn_exch{"1H1  3AR"}=" H11 3AR"; $resn_exch{"2H1  3AR"}=" H12 3AR"; $resn_exch{"1H2  3AR"}=" H21 3AR";
$resn_exch{"2H2  3AR"}=" H22 3AR"; $resn_exch{"1H3  3AR"}=" H31 3AR"; $resn_exch{"2H3  3AR"}=" H32 3AR";
$resn_exch{"3H3  3AR"}=" H33 3AR"; $resn_exch{"2HOG 3AT"}="HOG2 3AT"; $resn_exch{"3HOG 3AT"}="HOG3 3AT";
$resn_exch{"2HOB 3AT"}="HOB2 3AT"; $resn_exch{"2HOA 3AT"}="HOA2 3AT"; $resn_exch{"1H5* 3AT"}="H5'1 3AT";
$resn_exch{"2H5* 3AT"}="H5'2 3AT"; $resn_exch{" H4* 3AT"}=" H4' 3AT"; $resn_exch{"1H3* 3AT"}="H3'1 3AT";
$resn_exch{"2H3* 3AT"}="H3'2 3AT"; $resn_exch{"1H2* 3AT"}="H2'1 3AT"; $resn_exch{"HO2* 3AT"}="HO2' 3AT";
$resn_exch{" H1* 3AT"}=" H1' 3AT"; $resn_exch{"1HN6 3AT"}="HN61 3AT"; $resn_exch{"2HN6 3AT"}="HN62 3AT";
$resn_exch{"1HAA 3AY"}="HAA1 3AY"; $resn_exch{"2HAA 3AY"}="HAA2 3AY"; $resn_exch{"1HAH 3AY"}="HAH1 3AY";
$resn_exch{"2HAH 3AY"}="HAH2 3AY"; $resn_exch{"1HAI 3AY"}="HAI1 3AY"; $resn_exch{"2HAI 3AY"}="HAI2 3AY";
$resn_exch{"1H6C 3B3"}="H6C1 3B3"; $resn_exch{"2H6C 3B3"}="H6C2 3B3"; $resn_exch{"1HAC 3B3"}="HAC1 3B3";
$resn_exch{"2HAC 3B3"}="HAC2 3B3"; $resn_exch{"1HN  3B3"}=" HN1 3B3"; $resn_exch{"2HN  3B3"}=" HN2 3B3";
$resn_exch{"1H1  3BB"}=" H11 3BB"; $resn_exch{"2H1  3BB"}=" H12 3BB"; $resn_exch{"1H3  3BB"}=" H31 3BB";
$resn_exch{"2H3  3BB"}=" H32 3BB"; $resn_exch{"1H4  3BB"}=" H41 3BB"; $resn_exch{"2H4  3BB"}=" H42 3BB";
$resn_exch{"1H5  3BC"}=" H51 3BC"; $resn_exch{"2H5  3BC"}=" H52 3BC"; $resn_exch{"3H5  3BC"}=" H53 3BC";
$resn_exch{"1H6  3BC"}=" H61 3BC"; $resn_exch{"2H6  3BC"}=" H62 3BC"; $resn_exch{"3H6  3BC"}=" H63 3BC";
$resn_exch{"1H7  3BC"}=" H71 3BC"; $resn_exch{"2H7  3BC"}=" H72 3BC"; $resn_exch{"3H7  3BC"}=" H73 3BC";
$resn_exch{"1H11 3BC"}="H111 3BC"; $resn_exch{"2H11 3BC"}="H112 3BC"; $resn_exch{"3H11 3BC"}="H113 3BC";
$resn_exch{"1H12 3BC"}="H121 3BC"; $resn_exch{"2H12 3BC"}="H122 3BC"; $resn_exch{"3H12 3BC"}="H123 3BC";
$resn_exch{"1H13 3BC"}="H131 3BC"; $resn_exch{"2H13 3BC"}="H132 3BC"; $resn_exch{"3H13 3BC"}="H133 3BC";
$resn_exch{"1H17 3BC"}="H171 3BC"; $resn_exch{"2H17 3BC"}="H172 3BC"; $resn_exch{"1H22 3BC"}="H221 3BC";
$resn_exch{"2H22 3BC"}="H222 3BC"; $resn_exch{"3H22 3BC"}="H223 3BC"; $resn_exch{"1H23 3BC"}="H231 3BC";
$resn_exch{"2H23 3BC"}="H232 3BC"; $resn_exch{"3H23 3BC"}="H233 3BC"; $resn_exch{"1H28 3BC"}="H281 3BC";
$resn_exch{"2H28 3BC"}="H282 3BC"; $resn_exch{"1H30 3BC"}="H301 3BC"; $resn_exch{"2H30 3BC"}="H302 3BC";
$resn_exch{"1H31 3BC"}="H311 3BC"; $resn_exch{"2H31 3BC"}="H312 3BC"; $resn_exch{"1H36 3BC"}="H361 3BC";
$resn_exch{"2H36 3BC"}="H362 3BC"; $resn_exch{"1H51 3BC"}="H511 3BC"; $resn_exch{"2H51 3BC"}="H512 3BC";
$resn_exch{"3H51 3BC"}="H513 3BC"; $resn_exch{"1H52 3BC"}="H521 3BC"; $resn_exch{"2H52 3BC"}="H522 3BC";
$resn_exch{"3H52 3BC"}="H523 3BC"; $resn_exch{"1H1  3BN"}=" H11 3BN"; $resn_exch{"2H1  3BN"}="H12A 3BN";
$resn_exch{"1H3  3BN"}=" H31 3BN"; $resn_exch{"2H3  3BN"}=" H32 3BN"; $resn_exch{"1H4  3BN"}=" H41 3BN";
$resn_exch{"2H4  3BN"}=" H42 3BN"; $resn_exch{"1H8  3BN"}=" H81 3BN"; $resn_exch{"2H8  3BN"}=" H82 3BN";
$resn_exch{"1H16 3BN"}="H161 3BN"; $resn_exch{"2H16 3BN"}="H162 3BN"; $resn_exch{"3H16 3BN"}="H163 3BN";
$resn_exch{"1H23 3BN"}="H231 3BN"; $resn_exch{"2H23 3BN"}="H232 3BN"; $resn_exch{"3H23 3BN"}="H233 3BN";
$resn_exch{"1H19 3BP"}="H191 3BP"; $resn_exch{"2H19 3BP"}="H192 3BP"; $resn_exch{"1H20 3BP"}="H201 3BP";
$resn_exch{"2H20 3BP"}="H202 3BP"; $resn_exch{"1H32 3BP"}="H321 3BP"; $resn_exch{"2H32 3BP"}="H322 3BP";
$resn_exch{"1HN3 3BP"}="HN31 3BP"; $resn_exch{"2HN3 3BP"}="HN32 3BP"; $resn_exch{"1H1  3BP"}="H11A 3BP";
$resn_exch{"2H1  3BP"}="H12A 3BP"; $resn_exch{"1H9  3BP"}=" H91 3BP"; $resn_exch{"2H9  3BP"}=" H92 3BP";
$resn_exch{"1H8  3BP"}=" H81 3BP"; $resn_exch{"2H8  3BP"}=" H82 3BP"; $resn_exch{"3H8  3BP"}=" H83 3BP";
$resn_exch{"1HN2 3BP"}="HN21 3BP"; $resn_exch{"2HN2 3BP"}="HN22 3BP"; $resn_exch{"1H1  3BR"}=" H11 3BR";
$resn_exch{"2H1  3BR"}=" H12 3BR"; $resn_exch{"1H2  3BR"}=" H21 3BR"; $resn_exch{"2H2  3BR"}=" H22 3BR";
$resn_exch{"1H3  3BR"}=" H31 3BR"; $resn_exch{"2H3  3BR"}=" H32 3BR"; $resn_exch{"3H3  3BR"}=" H33 3BR";
$resn_exch{"1HCA 3BT"}="HCA1 3BT"; $resn_exch{"2HCA 3BT"}="HCA2 3BT"; $resn_exch{"1HCB 3BT"}="HCB1 3BT";
$resn_exch{"2HCB 3BT"}="HCB2 3BT"; $resn_exch{"1HN1 3BT"}="HN11 3BT"; $resn_exch{"2HN1 3BT"}="HN12 3BT";
$resn_exch{"1HN2 3BT"}="HN21 3BT"; $resn_exch{"1H2  3BT"}=" H21 3BT"; $resn_exch{"1H6  3BT"}=" H61 3BT";
$resn_exch{"1H5  3BT"}=" H51 3BT"; $resn_exch{"1H4  3BT"}=" H41 3BT"; $resn_exch{"1HCC 3BT"}="HCC1 3BT";
$resn_exch{"2HCC 3BT"}="HCC2 3BT"; $resn_exch{"1HCD 3BT"}="HCD1 3BT"; $resn_exch{"2HCD 3BT"}="HCD2 3BT";
$resn_exch{"1H1' 3BT"}="H1'1 3BT"; $resn_exch{"2H1' 3BT"}="H1'2 3BT"; $resn_exch{"'HN2 3BT"}="HN2' 3BT";
$resn_exch{"1H16 3C3"}="H161 3C3"; $resn_exch{"2H16 3C3"}="H162 3C3"; $resn_exch{"1H18 3C3"}="H181 3C3";
$resn_exch{"2H18 3C3"}="H182 3C3"; $resn_exch{"1H19 3C3"}="H191 3C3"; $resn_exch{"2H19 3C3"}="H192 3C3";
$resn_exch{"1H20 3C3"}="H201 3C3"; $resn_exch{"2H20 3C3"}="H202 3C3"; $resn_exch{"3H20 3C3"}="H203 3C3";
$resn_exch{"1H17 3C3"}="H171 3C3"; $resn_exch{"2H17 3C3"}="H172 3C3"; $resn_exch{"1HG  3C4"}=" HG1 3C4";
$resn_exch{"2HG  3C4"}=" HG2 3C4"; $resn_exch{"1H19 3CB"}="H191 3CB"; $resn_exch{"2H19 3CB"}="H192 3CB";
$resn_exch{"1H20 3CB"}="H201 3CB"; $resn_exch{"2H20 3CB"}="H202 3CB"; $resn_exch{"1H32 3CB"}="H321 3CB";
$resn_exch{"2H32 3CB"}="H322 3CB"; $resn_exch{"1HN5 3CB"}="HN51 3CB"; $resn_exch{"2HN5 3CB"}="HN52 3CB";
$resn_exch{"1H4  3CB"}=" H41 3CB"; $resn_exch{"2H4  3CB"}=" H42 3CB"; $resn_exch{"1H3  3CB"}="H31A 3CB";
$resn_exch{"2H3  3CB"}=" H32 3CB"; $resn_exch{"3H3  3CB"}=" H33 3CB"; $resn_exch{"1H9  3CB"}=" H91 3CB";
$resn_exch{"2H9  3CB"}=" H92 3CB"; $resn_exch{"1H6  3CB"}=" H61 3CB"; $resn_exch{"2H6  3CB"}=" H62 3CB";
$resn_exch{"3H6  3CB"}=" H63 3CB"; $resn_exch{"1HN2 3CB"}="HN21 3CB"; $resn_exch{"2HN2 3CB"}="HN22 3CB";
$resn_exch{"1H7  3CC"}=" H71 3CC"; $resn_exch{"2H7  3CC"}=" H72 3CC"; $resn_exch{"1H9  3CC"}=" H91 3CC";
$resn_exch{"2H9  3CC"}=" H92 3CC"; $resn_exch{"1HN1 3CC"}="HN11 3CC"; $resn_exch{"2HN1 3CC"}="HN12 3CC";
$resn_exch{"1H12 3CC"}="H121 3CC"; $resn_exch{"2H12 3CC"}="H122 3CC"; $resn_exch{"1H13 3CC"}="H131 3CC";
$resn_exch{"2H13 3CC"}="H132 3CC"; $resn_exch{"1H14 3CC"}="H141 3CC"; $resn_exch{"2H14 3CC"}="H142 3CC";
$resn_exch{"3H14 3CC"}="H143 3CC"; $resn_exch{"1H15 3CC"}="H151 3CC"; $resn_exch{"2H15 3CC"}="H152 3CC";
$resn_exch{"1H16 3CC"}="H161 3CC"; $resn_exch{"2H16 3CC"}="H162 3CC"; $resn_exch{"1H17 3CC"}="H171 3CC";
$resn_exch{"2H17 3CC"}="H172 3CC"; $resn_exch{"3H17 3CC"}="H173 3CC"; $resn_exch{"1H2  3CL"}=" H21 3CL";
$resn_exch{"2H2  3CL"}=" H22 3CL"; $resn_exch{"1H3  3CL"}=" H31 3CL"; $resn_exch{"2H3  3CL"}=" H32 3CL";
$resn_exch{"1H4  3CL"}=" H41 3CL"; $resn_exch{"2H4  3CL"}=" H42 3CL"; $resn_exch{"1H6  3CM"}=" H61 3CM";
$resn_exch{"2H6  3CM"}="H62A 3CM"; $resn_exch{"1H11 3CM"}="H111 3CM"; $resn_exch{"2H11 3CM"}="H112 3CM";
$resn_exch{"1H21 3CM"}="H211 3CM"; $resn_exch{"2H21 3CM"}="H212 3CM"; $resn_exch{"1H31 3CM"}="H311 3CM";
$resn_exch{"2H31 3CM"}="H312 3CM"; $resn_exch{"1H42 3CM"}="H421 3CM"; $resn_exch{"2H42 3CM"}="H422 3CM";
$resn_exch{"1H60 3CM"}="H601 3CM"; $resn_exch{"2H60 3CM"}="H602 3CM"; $resn_exch{"1HA  3CN"}=" HA1 3CN";
$resn_exch{"2HA  3CN"}=" HA2 3CN"; $resn_exch{"3HA  3CN"}=" HA3 3CN"; $resn_exch{"1HB  3CN"}=" HB1 3CN";
$resn_exch{"2HB  3CN"}=" HB2 3CN"; $resn_exch{"1HC  3CN"}=" HC1 3CN"; $resn_exch{"2HC  3CN"}=" HC2 3CN";
$resn_exch{"1HND 3CN"}="HND1 3CN"; $resn_exch{"2HND 3CN"}="HND2 3CN"; $resn_exch{"1H6  3CP"}=" H61 3CP";
$resn_exch{"2H6  3CP"}=" H62 3CP"; $resn_exch{"2HO' 3CP"}="HO'2 3CP"; $resn_exch{"1H5' 3CP"}="H5'1 3CP";
$resn_exch{"2H5' 3CP"}="H5'2 3CP"; $resn_exch{"1HB  3CP"}=" HB1 3CP"; $resn_exch{"2HB  3CP"}=" HB2 3CP";
$resn_exch{"1HP9 3CP"}="HP91 3CP"; $resn_exch{"2HP9 3CP"}="HP92 3CP"; $resn_exch{"3HP9 3CP"}="HP93 3CP";
$resn_exch{"1HP8 3CP"}="HP81 3CP"; $resn_exch{"2HP8 3CP"}="HP82 3CP"; $resn_exch{"3HP8 3CP"}="HP83 3CP";
$resn_exch{"3HOP 3CP"}="HOP3 3CP"; $resn_exch{"1HP5 3CP"}="HP51 3CP"; $resn_exch{"2HP5 3CP"}="HP52 3CP";
$resn_exch{"1HP4 3CP"}="HP41 3CP"; $resn_exch{"2HP4 3CP"}="HP42 3CP"; $resn_exch{"1HP2 3CP"}="HP21 3CP";
$resn_exch{"2HP2 3CP"}="HP22 3CP"; $resn_exch{"1HP1 3CP"}="HP11 3CP"; $resn_exch{"2HP1 3CP"}="HP12 3CP";
$resn_exch{"1HS1 3CP"}="HS11 3CP"; $resn_exch{"2HS1 3CP"}="HS12 3CP"; $resn_exch{"1HS2 3CP"}="HS21 3CP";
$resn_exch{"2HS2 3CP"}="HS22 3CP"; $resn_exch{"1HS3 3CP"}="HS31 3CP"; $resn_exch{"2HS3 3CP"}="HS32 3CP";
$resn_exch{"5HOS 3CP"}="HOS5 3CP"; $resn_exch{"1H10 3CS"}="H101 3CS"; $resn_exch{"2H10 3CS"}="H102 3CS";
$resn_exch{"1H30 3CS"}="H301 3CS"; $resn_exch{"2H30 3CS"}="H302 3CS"; $resn_exch{"1H31 3CS"}="H311 3CS";
$resn_exch{"2H31 3CS"}="H312 3CS"; $resn_exch{"3H31 3CS"}="H313 3CS"; $resn_exch{"1H32 3CS"}="H321 3CS";
$resn_exch{"2H32 3CS"}="H322 3CS"; $resn_exch{"3H32 3CS"}="H323 3CS"; $resn_exch{"1H34 3CS"}="H341 3CS";
$resn_exch{"2H34 3CS"}="H342 3CS"; $resn_exch{"3H34 3CS"}="H343 3CS"; $resn_exch{"1H18 3CS"}="H181 3CS";
$resn_exch{"2H18 3CS"}="H182 3CS"; $resn_exch{"1H37 3CS"}="H371 3CS"; $resn_exch{"2H37 3CS"}="H372 3CS";
$resn_exch{"1H38 3CS"}="H381 3CS"; $resn_exch{"2H38 3CS"}="H382 3CS"; $resn_exch{"3H38 3CS"}="H383 3CS";
$resn_exch{"1H39 3CS"}="H391 3CS"; $resn_exch{"2H39 3CS"}="H392 3CS"; $resn_exch{"3H39 3CS"}="H393 3CS";
$resn_exch{"1H7  3CY"}=" H71 3CY"; $resn_exch{"2H7  3CY"}=" H72 3CY"; $resn_exch{"1H8  3CY"}=" H81 3CY";
$resn_exch{"2H8  3CY"}=" H82 3CY"; $resn_exch{"1H11 3CY"}="H111 3CY"; $resn_exch{"2H11 3CY"}="H112 3CY";
$resn_exch{"1H15 3CY"}="H151 3CY"; $resn_exch{"2H15 3CY"}="H152 3CY"; $resn_exch{"1H22 3CY"}="H221 3CY";
$resn_exch{"2H22 3CY"}="H222 3CY"; $resn_exch{"1H23 3CY"}="H231 3CY"; $resn_exch{"2H23 3CY"}="H232 3CY";
$resn_exch{"1H31 3CY"}="H311 3CY"; $resn_exch{"2H31 3CY"}="H312 3CY"; $resn_exch{"3H31 3CY"}="H313 3CY";
$resn_exch{"1H28 3CY"}="H281 3CY"; $resn_exch{"2H28 3CY"}="H282 3CY"; $resn_exch{"1H13 3D3"}="H131 3D3";
$resn_exch{"2H13 3D3"}="H132 3D3"; $resn_exch{"1H14 3D3"}="H141 3D3"; $resn_exch{"2H14 3D3"}="H142 3D3";
$resn_exch{"1H15 3D3"}="H151 3D3"; $resn_exch{"2H15 3D3"}="H152 3D3"; $resn_exch{"2HOP 3DA"}="HOP2 3DA";
$resn_exch{"1H5* 3DA"}="H5'1 3DA"; $resn_exch{"2H5* 3DA"}="H5'2 3DA"; $resn_exch{" H4* 3DA"}=" H4' 3DA";
$resn_exch{" H1* 3DA"}=" H1' 3DA"; $resn_exch{"1HN6 3DA"}="HN61 3DA"; $resn_exch{"2HN6 3DA"}="HN62 3DA";
$resn_exch{"1H2* 3DA"}="H2'1 3DA"; $resn_exch{"2HO* 3DA"}="HO'2 3DA"; $resn_exch{"1H3* 3DA"}="H3'1 3DA";
$resn_exch{"2H3* 3DA"}="H3'2 3DA"; $resn_exch{"3HOP 3DA"}="HOP3 3DA"; $resn_exch{"1HN6 3DD"}="HN61 3DD";
$resn_exch{"2HN6 3DD"}="HN62 3DD"; $resn_exch{"1H1' 3DD"}="H1'1 3DD"; $resn_exch{"2H1' 3DD"}="H1'2 3DD";
$resn_exch{"1H16 3DE"}="H161 3DE"; $resn_exch{"2H16 3DE"}="H162 3DE"; $resn_exch{"1H17 3DE"}="H171 3DE";
$resn_exch{"2H17 3DE"}="H172 3DE"; $resn_exch{"3H17 3DE"}="H173 3DE"; $resn_exch{"1H12 3DE"}="H121 3DE";
$resn_exch{"2H12 3DE"}="H122 3DE"; $resn_exch{"3H12 3DE"}="H123 3DE"; $resn_exch{"1H9  3DE"}=" H91 3DE";
$resn_exch{"2H9  3DE"}=" H92 3DE"; $resn_exch{"3H9  3DE"}=" H93 3DE"; $resn_exch{"1H3' 3DG"}="H3'1 3DG";
$resn_exch{"2H3' 3DG"}="H3'2 3DG"; $resn_exch{"1H5' 3DG"}="H5'1 3DG"; $resn_exch{"2H5' 3DG"}="H5'2 3DG";
$resn_exch{"1HN2 3DG"}="HN21 3DG"; $resn_exch{"2HN2 3DG"}="HN22 3DG"; $resn_exch{"1HAA 3DP"}="HAA1 3DP";
$resn_exch{"2HAA 3DP"}="HAA2 3DP"; $resn_exch{"3HAA 3DP"}="HAA3 3DP"; $resn_exch{"1HAG 3DP"}="HAG1 3DP";
$resn_exch{"2HAG 3DP"}="HAG2 3DP"; $resn_exch{"1HAH 3DP"}="HAH1 3DP"; $resn_exch{"2HAH 3DP"}="HAH2 3DP";
$resn_exch{"1HAI 3DP"}="HAI1 3DP"; $resn_exch{"2HAI 3DP"}="HAI2 3DP"; $resn_exch{"1HAJ 3DP"}="HAJ1 3DP";
$resn_exch{"2HAJ 3DP"}="HAJ2 3DP"; $resn_exch{"1HAK 3DP"}="HAK1 3DP"; $resn_exch{"2HAK 3DP"}="HAK2 3DP";
$resn_exch{"1HAL 3DP"}="HAL1 3DP"; $resn_exch{"2HAL 3DP"}="HAL2 3DP"; $resn_exch{"1HAN 3DP"}="HAN1 3DP";
$resn_exch{"2HAN 3DP"}="HAN2 3DP"; $resn_exch{"1HAB 3DP"}="HAB1 3DP"; $resn_exch{"2HAB 3DP"}="HAB2 3DP";
$resn_exch{"3HAB 3DP"}="HAB3 3DP"; $resn_exch{"1HAC 3DP"}="HAC1 3DP"; $resn_exch{"2HAC 3DP"}="HAC2 3DP";
$resn_exch{"3HAC 3DP"}="HAC3 3DP"; $resn_exch{"1HAO 3DP"}="HAO1 3DP"; $resn_exch{"2HAO 3DP"}="HAO2 3DP";
$resn_exch{"1HAM 3DP"}="HAM1 3DP"; $resn_exch{"2HAM 3DP"}="HAM2 3DP"; $resn_exch{"1HAP 3DP"}="HAP1 3DP";
$resn_exch{"2HAP 3DP"}="HAP2 3DP"; $resn_exch{"2HOP 3DR"}="HOP2 3DR"; $resn_exch{"3HOP 3DR"}="HOP3 3DR";
$resn_exch{"1H2* 3DR"}=" H2' 3DR"; $resn_exch{"2H2* 3DR"}="H2'' 3DR"; $resn_exch{"1H5* 3DR"}=" H5' 3DR";
$resn_exch{"2H5* 3DR"}="H5'' 3DR"; $resn_exch{"1H4* 3DR"}="H4'1 3DR"; $resn_exch{"1H1* 3DR"}="H1'1 3DR";
$resn_exch{"2H1* 3DR"}="H1'2 3DR"; $resn_exch{" H3* 3DR"}=" H3' 3DR"; $resn_exch{"HO3* 3DR"}="HO3' 3DR";
$resn_exch{"1H13 3EA"}="H131 3EA"; $resn_exch{"2H13 3EA"}="H132 3EA"; $resn_exch{"3H13 3EA"}="H133 3EA";
$resn_exch{"1H12 3EA"}="H121 3EA"; $resn_exch{"2H12 3EA"}="H122 3EA"; $resn_exch{"1H11 3EA"}="H111 3EA";
$resn_exch{"2H11 3EA"}="H112 3EA"; $resn_exch{"1H15 3EA"}="H151 3EA"; $resn_exch{"2H15 3EA"}="H152 3EA";
$resn_exch{"1H16 3EA"}="H161 3EA"; $resn_exch{"2H16 3EA"}="H162 3EA"; $resn_exch{"1H17 3EA"}="H171 3EA";
$resn_exch{"2H17 3EA"}="H172 3EA"; $resn_exch{"1H28 3EA"}="H281 3EA"; $resn_exch{"2H28 3EA"}="H282 3EA";
$resn_exch{"2H1  3EP"}=" H12 3EP"; $resn_exch{"1H1  3EP"}=" H11 3EP"; $resn_exch{"3H2  3EP"}=" H23 3EP";
$resn_exch{"2H2  3EP"}=" H22 3EP"; $resn_exch{"1H2  3EP"}=" H21 3EP"; $resn_exch{"2H3  3EP"}=" H32 3EP";
$resn_exch{"1H3  3EP"}=" H31 3EP"; $resn_exch{"3H4  3EP"}=" H43 3EP"; $resn_exch{"2H4  3EP"}=" H42 3EP";
$resn_exch{"1H4  3EP"}=" H41 3EP"; $resn_exch{"2H5  3EP"}=" H52 3EP"; $resn_exch{"1H5  3EP"}=" H51 3EP";
$resn_exch{"3H6  3EP"}=" H63 3EP"; $resn_exch{"2H6  3EP"}=" H62 3EP"; $resn_exch{"1H6  3EP"}=" H61 3EP";
$resn_exch{"1H1  3FC"}=" H11 3FC"; $resn_exch{"2H1  3FC"}=" H12 3FC"; $resn_exch{"3H1  3FC"}=" H13 3FC";
$resn_exch{"1H2  3FC"}="H21A 3FC"; $resn_exch{"2H2  3FC"}="H22A 3FC"; $resn_exch{"1H3  3FC"}=" H31 3FC";
$resn_exch{"2H3  3FC"}=" H32 3FC"; $resn_exch{"1H4  3FC"}=" H41 3FC"; $resn_exch{"2H4  3FC"}=" H42 3FC";
$resn_exch{"1H11 3FC"}="H111 3FC"; $resn_exch{"2H11 3FC"}="H112 3FC"; $resn_exch{"1H28 3FC"}="H281 3FC";
$resn_exch{"2H28 3FC"}="H282 3FC"; $resn_exch{"3H28 3FC"}="H283 3FC"; $resn_exch{"1H29 3FC"}="H291 3FC";
$resn_exch{"2H29 3FC"}="H292 3FC"; $resn_exch{"3H29 3FC"}="H293 3FC"; $resn_exch{"1H30 3FC"}="H301 3FC";
$resn_exch{"2H30 3FC"}="H302 3FC"; $resn_exch{"3H30 3FC"}="H303 3FC"; $resn_exch{"1H37 3FC"}="H371 3FC";
$resn_exch{"2H37 3FC"}="H372 3FC"; $resn_exch{"1H38 3FC"}="H381 3FC"; $resn_exch{"2H38 3FC"}="H382 3FC";
$resn_exch{"1HNC 3FM"}="HNC1 3FM"; $resn_exch{"2HNC 3FM"}="HNC2 3FM"; $resn_exch{"1H6  3FM"}=" H61 3FM";
$resn_exch{"2H6  3FM"}=" H62 3FM"; $resn_exch{"1H16 3FP"}="H161 3FP"; $resn_exch{"2H16 3FP"}="H162 3FP";
$resn_exch{"1H18 3FP"}="H181 3FP"; $resn_exch{"2H18 3FP"}="H182 3FP"; $resn_exch{"1H19 3FP"}="H191 3FP";
$resn_exch{"2H19 3FP"}="H192 3FP"; $resn_exch{"3H19 3FP"}="H193 3FP"; $resn_exch{"1H20 3FP"}="H201 3FP";
$resn_exch{"2H20 3FP"}="H202 3FP"; $resn_exch{"3H20 3FP"}="H203 3FP"; $resn_exch{"1H16 3FQ"}="H161 3FQ";
$resn_exch{"2H16 3FQ"}="H162 3FQ"; $resn_exch{"1H18 3FQ"}="H181 3FQ"; $resn_exch{"2H18 3FQ"}="H182 3FQ";
$resn_exch{"1H19 3FQ"}="H191 3FQ"; $resn_exch{"2H19 3FQ"}="H192 3FQ"; $resn_exch{"3H19 3FQ"}="H193 3FQ";
$resn_exch{"1H20 3FQ"}="H201 3FQ"; $resn_exch{"2H20 3FQ"}="H202 3FQ"; $resn_exch{"3H20 3FQ"}="H203 3FQ";
$resn_exch{"1H24 3FT"}="H241 3FT"; $resn_exch{"2H24 3FT"}="H242 3FT"; $resn_exch{"1H25 3FT"}="H251 3FT";
$resn_exch{"2H25 3FT"}="H252 3FT"; $resn_exch{"1H26 3FT"}="H261 3FT"; $resn_exch{"2H26 3FT"}="H262 3FT";
$resn_exch{"1H2  3GA"}=" H21 3GA"; $resn_exch{"2H2  3GA"}=" H22 3GA"; $resn_exch{"1H3  3GA"}=" H31 3GA";
$resn_exch{"2H3  3GA"}=" H32 3GA"; $resn_exch{"1HN  3GA"}="HN1A 3GA"; $resn_exch{"2HN  3GA"}=" HN2 3GA";
$resn_exch{"1HB  3GA"}=" HB1 3GA"; $resn_exch{"2HB  3GA"}=" HB2 3GA"; $resn_exch{"1H1  3GA"}=" H11 3GA";
$resn_exch{"2H1  3GA"}=" H12 3GA"; $resn_exch{"1HD  3GA"}=" HD1 3GA"; $resn_exch{"2HD  3GA"}=" HD2 3GA";
$resn_exch{"1HN2 3GA"}="HN21 3GA"; $resn_exch{"2HN2 3GA"}="HN22 3GA"; $resn_exch{"1H1  3GC"}=" H11 3GC";
$resn_exch{"2H2  3GC"}=" H22 3GC"; $resn_exch{"1HB1 3GC"}="HB11 3GC"; $resn_exch{"2HB1 3GC"}="HB12 3GC";
$resn_exch{"1HG1 3GC"}="HG11 3GC"; $resn_exch{"2HG1 3GC"}="HG12 3GC"; $resn_exch{"1HB2 3GC"}="HB21 3GC";
$resn_exch{"2HB2 3GC"}="HB22 3GC"; $resn_exch{"2HOP 3GP"}="HOP2 3GP"; $resn_exch{"3HOP 3GP"}="HOP3 3GP";
$resn_exch{"*HO5 3GP"}="HO5' 3GP"; $resn_exch{"1H5* 3GP"}="H5'1 3GP"; $resn_exch{"2H5* 3GP"}="H5'2 3GP";
$resn_exch{" H4* 3GP"}=" H4' 3GP"; $resn_exch{" H3* 3GP"}=" H3' 3GP"; $resn_exch{" H2* 3GP"}=" H2' 3GP";
$resn_exch{"*HO2 3GP"}="HO2' 3GP"; $resn_exch{" H1* 3GP"}=" H1' 3GP"; $resn_exch{"1HN2 3GP"}="HN21 3GP";
$resn_exch{"2HN2 3GP"}="HN22 3GP"; $resn_exch{"1H1C 3GR"}="H1C1 3GR"; $resn_exch{"2H1C 3GR"}="H1C2 3GR";
$resn_exch{"1H10 3HA"}="H101 3HA"; $resn_exch{"2H10 3HA"}="H102 3HA"; $resn_exch{"'HO2 3HB"}="HO2' 3HB";
$resn_exch{" AH2 3HC"}=" H2A 3HC"; $resn_exch{"AH61 3HC"}="H61A 3HC"; $resn_exch{"AH62 3HC"}="H62A 3HC";
$resn_exch{" AH8 3HC"}=" H8A 3HC"; $resn_exch{"AH1* 3HC"}=" H1B 3HC"; $resn_exch{"AH2* 3HC"}=" H2B 3HC";
$resn_exch{"AHO2 3HC"}="HO2A 3HC"; $resn_exch{"AH3* 3HC"}=" H3B 3HC"; $resn_exch{"8HOA 3HC"}="HOA8 3HC";
$resn_exch{"9HOA 3HC"}="HOA9 3HC"; $resn_exch{"AH4* 3HC"}=" H4B 3HC"; $resn_exch{"AH51 3HC"}="H51A 3HC";
$resn_exch{"AH52 3HC"}="H52A 3HC"; $resn_exch{"2HOA 3HC"}="HOA2 3HC"; $resn_exch{"5HOA 3HC"}="HOA5 3HC";
$resn_exch{"1H12 3HC"}="H121 3HC"; $resn_exch{"2H12 3HC"}="H122 3HC"; $resn_exch{"1H13 3HC"}="H131 3HC";
$resn_exch{"2H13 3HC"}="H132 3HC"; $resn_exch{"3H13 3HC"}="H133 3HC"; $resn_exch{"1H14 3HC"}="H141 3HC";
$resn_exch{"2H14 3HC"}="H142 3HC"; $resn_exch{"3H14 3HC"}="H143 3HC"; $resn_exch{"1H7  3HC"}=" H71 3HC";
$resn_exch{"2H7  3HC"}=" H72 3HC"; $resn_exch{"1H6  3HC"}=" H61 3HC"; $resn_exch{"2H6  3HC"}=" H62 3HC";
$resn_exch{"1H3  3HC"}=" H31 3HC"; $resn_exch{"2H3  3HC"}=" H32 3HC"; $resn_exch{"1H2  3HC"}=" H21 3HC";
$resn_exch{"2H2  3HC"}=" H22 3HC"; $resn_exch{"1HC2 3HC"}="HC21 3HC"; $resn_exch{"2HC2 3HC"}="HC22 3HC";
$resn_exch{"1HC3 3HC"}="HC31 3HC"; $resn_exch{"1HO3 3HC"}="HO31 3HC"; $resn_exch{"1HC4 3HC"}="HC41 3HC";
$resn_exch{"2HC4 3HC"}="HC42 3HC"; $resn_exch{"3HC4 3HC"}="HC43 3HC"; $resn_exch{"1H   3HD"}=" H1  3HD";
$resn_exch{"2H   3HD"}=" H2  3HD"; $resn_exch{"2HO  3HD"}=" HO2 3HD"; $resn_exch{"3H   3HD"}=" H3  3HD";
$resn_exch{"1H3  3HD"}=" H31 3HD"; $resn_exch{"2H3  3HD"}=" H32 3HD"; $resn_exch{"3H3  3HD"}=" H33 3HD";
$resn_exch{"4H   3HD"}=" H4  3HD"; $resn_exch{"4HO  3HD"}=" HO4 3HD"; $resn_exch{"5H   3HD"}=" H5  3HD";
$resn_exch{"1H6  3HD"}=" H61 3HD"; $resn_exch{"2H6  3HD"}=" H62 3HD"; $resn_exch{"6HO  3HD"}=" HO6 3HD";
$resn_exch{"1H2  3HG"}=" H21 3HG"; $resn_exch{"2H2  3HG"}=" H22 3HG"; $resn_exch{"1H4  3HG"}=" H41 3HG";
$resn_exch{"2H4  3HG"}=" H42 3HG"; $resn_exch{"1HA  3HL"}=" HA1 3HL"; $resn_exch{"2HA  3HL"}=" HA2 3HL";
$resn_exch{"1HG2 3HL"}="HG21 3HL"; $resn_exch{"2HG2 3HL"}="HG22 3HL"; $resn_exch{"3HG2 3HL"}="HG23 3HL";
$resn_exch{"1H7  3HP"}=" H71 3HP"; $resn_exch{"2H7  3HP"}=" H72 3HP"; $resn_exch{"1HN  3HX"}=" HN1 3HX";
$resn_exch{"2HN  3HX"}=" HN2 3HX"; $resn_exch{"1H   3HX"}=" H1  3HX"; $resn_exch{"2H   3HX"}=" H2  3HX";
$resn_exch{"1HB  3HX"}=" HB1 3HX"; $resn_exch{"2HB  3HX"}=" HB2 3HX"; $resn_exch{"1HG  3HX"}=" HG1 3HX";
$resn_exch{"2HG  3HX"}=" HG2 3HX"; $resn_exch{"1HD  3HX"}=" HD1 3HX"; $resn_exch{"2HD  3HX"}=" HD2 3HX";
$resn_exch{"1HNB 3HX"}="HNB1 3HX"; $resn_exch{"2HNB 3HX"}="HNB2 3HX"; $resn_exch{"1H1' 3HX"}="H1'1 3HX";
$resn_exch{"2H1' 3HX"}="H1'2 3HX"; $resn_exch{"1H2' 3HX"}="H2'1 3HX"; $resn_exch{"2H2' 3HX"}="H2'2 3HX";
$resn_exch{"1H10 3IB"}="H101 3IB"; $resn_exch{"2H10 3IB"}="H102 3IB"; $resn_exch{"1H11 3IB"}="H111 3IB";
$resn_exch{"2H11 3IB"}="H112 3IB"; $resn_exch{"1H12 3IB"}="H121 3IB"; $resn_exch{"2H12 3IB"}="H122 3IB";
$resn_exch{"1HN  3ID"}=" HN1 3ID"; $resn_exch{"1OH  3ID"}=" H1O 3ID"; $resn_exch{"2OH  3ID"}=" H2O 3ID";
$resn_exch{"1HN3 3IG"}="HN31 3IG"; $resn_exch{"2HN3 3IG"}="HN32 3IG"; $resn_exch{"1HN4 3IG"}="HN41 3IG";
$resn_exch{"2HN4 3IG"}="HN42 3IG"; $resn_exch{"1H5  3IG"}=" H51 3IG"; $resn_exch{"2H5  3IG"}=" H52 3IG";
$resn_exch{"1H6  3IG"}=" H61 3IG"; $resn_exch{"2H6  3IG"}=" H62 3IG"; $resn_exch{"3H6  3IG"}=" H63 3IG";
$resn_exch{"1H16 3IG"}="H161 3IG"; $resn_exch{"2H16 3IG"}="H162 3IG"; $resn_exch{"1H17 3IG"}="H171 3IG";
$resn_exch{"2H17 3IG"}="H172 3IG"; $resn_exch{"1H19 3IG"}="H191 3IG"; $resn_exch{"2H19 3IG"}="H192 3IG";
$resn_exch{"3H19 3IG"}="H193 3IG"; $resn_exch{"1H21 3IG"}="H211 3IG"; $resn_exch{"2H21 3IG"}="H212 3IG";
$resn_exch{"3H21 3IG"}="H213 3IG"; $resn_exch{"1H20 3IG"}="H201 3IG"; $resn_exch{"2H20 3IG"}="H202 3IG";
$resn_exch{"3H20 3IG"}="H203 3IG"; $resn_exch{"1H6  3IH"}=" H61 3IH"; $resn_exch{"2H6  3IH"}=" H62 3IH";
$resn_exch{"3H6  3IH"}=" H63 3IH"; $resn_exch{"1H10 3IH"}="H101 3IH"; $resn_exch{"2H10 3IH"}="H102 3IH";
$resn_exch{"3H10 3IH"}="H103 3IH"; $resn_exch{"1H11 3IH"}="H111 3IH"; $resn_exch{"2H11 3IH"}="H112 3IH";
$resn_exch{"3H11 3IH"}="H113 3IH"; $resn_exch{"1H14 3IH"}="H141 3IH"; $resn_exch{"2H14 3IH"}="H142 3IH";
$resn_exch{"1H16 3IH"}="H161 3IH"; $resn_exch{"2H16 3IH"}="H162 3IH"; $resn_exch{"3H16 3IH"}="H163 3IH";
$resn_exch{"1H17 3IH"}="H171 3IH"; $resn_exch{"2H17 3IH"}="H172 3IH"; $resn_exch{"3H17 3IH"}="H173 3IH";
$resn_exch{"1H23 3IH"}="H231 3IH"; $resn_exch{"2H23 3IH"}="H232 3IH"; $resn_exch{"1H25 3IH"}="H251 3IH";
$resn_exch{"2H25 3IH"}="H252 3IH"; $resn_exch{"1H27 3IH"}="H271 3IH"; $resn_exch{"2H27 3IH"}="H272 3IH";
$resn_exch{"1H28 3IH"}="H281 3IH"; $resn_exch{"2H28 3IH"}="H282 3IH"; $resn_exch{"1H35 3IH"}="H351 3IH";
$resn_exch{"2H35 3IH"}="H352 3IH"; $resn_exch{"3H35 3IH"}="H353 3IH"; $resn_exch{"1H15 3IL"}="H151 3IL";
$resn_exch{"2H15 3IL"}="H152 3IL"; $resn_exch{"1H1  3IN"}="H11A 3IN"; $resn_exch{"2H1  3IN"}=" H12 3IN";
$resn_exch{"1H7  3IN"}=" H71 3IN"; $resn_exch{"2H7  3IN"}=" H72 3IN"; $resn_exch{"3H7  3IN"}=" H73 3IN";
$resn_exch{"1H8  3IN"}=" H81 3IN"; $resn_exch{"2H8  3IN"}=" H82 3IN"; $resn_exch{"1H9  3IN"}=" H91 3IN";
$resn_exch{"2H9  3IN"}=" H92 3IN"; $resn_exch{"1H10 3IN"}="H101 3IN"; $resn_exch{"2H10 3IN"}="H102 3IN";
$resn_exch{"1H12 3IN"}="H121 3IN"; $resn_exch{"2H12 3IN"}="H122 3IN"; $resn_exch{"1H14 3IN"}="H141 3IN";
$resn_exch{"2H14 3IN"}="H142 3IN"; $resn_exch{"1H24 3IN"}="H241 3IN"; $resn_exch{"2H24 3IN"}="H242 3IN";
$resn_exch{"1H27 3IN"}="H271 3IN"; $resn_exch{"2H27 3IN"}="H272 3IN"; $resn_exch{"1H35 3IN"}="H351 3IN";
$resn_exch{"2H35 3IN"}="H352 3IN"; $resn_exch{"1H5  3IN"}=" H51 3IN"; $resn_exch{"2H5  3IN"}=" H52 3IN";
$resn_exch{"3H5  3IN"}=" H53 3IN"; $resn_exch{"1H6  3IN"}=" H61 3IN"; $resn_exch{"2H6  3IN"}=" H62 3IN";
$resn_exch{"3H6  3IN"}=" H63 3IN"; $resn_exch{"1H25 3IN"}="H251 3IN"; $resn_exch{"2H25 3IN"}="H252 3IN";
$resn_exch{"1H29 3IN"}="H291 3IN"; $resn_exch{"2H29 3IN"}="H292 3IN"; $resn_exch{"3H29 3IN"}="H293 3IN";
$resn_exch{"1H26 3IN"}="H261 3IN"; $resn_exch{"2H26 3IN"}="H262 3IN"; $resn_exch{"1H37 3IN"}="H371 3IN";
$resn_exch{"2H37 3IN"}="H372 3IN"; $resn_exch{"1H28 3IN"}="H281 3IN"; $resn_exch{"2H28 3IN"}="H282 3IN";
$resn_exch{"3H28 3IN"}="H283 3IN"; $resn_exch{"1HAI 3IO"}="HAI1 3IO"; $resn_exch{"2HAI 3IO"}="HAI2 3IO";
$resn_exch{"1H1N 3IP"}="H1N1 3IP"; $resn_exch{"2H1N 3IP"}="H1N2 3IP"; $resn_exch{"1H9C 3IP"}="H9C1 3IP";
$resn_exch{"2H9C 3IP"}="H9C2 3IP"; $resn_exch{"1H1  3LG"}="H11A 3LG"; $resn_exch{"2H1  3LG"}="H12A 3LG";
$resn_exch{"3H1  3LG"}="H13A 3LG"; $resn_exch{"1H4  3LG"}=" H41 3LG"; $resn_exch{"2H4  3LG"}=" H42 3LG";
$resn_exch{"1H21 3LG"}="H211 3LG"; $resn_exch{"2H21 3LG"}="H212 3LG"; $resn_exch{"1H22 3LG"}="H221 3LG";
$resn_exch{"2H22 3LG"}="H222 3LG"; $resn_exch{"1H24 3LG"}="H241 3LG"; $resn_exch{"2H24 3LG"}="H242 3LG";
$resn_exch{"1H31 3LG"}="H311 3LG"; $resn_exch{"2H31 3LG"}="H312 3LG"; $resn_exch{"1H32 3LG"}="H321 3LG";
$resn_exch{"2H32 3LG"}="H322 3LG"; $resn_exch{"1H33 3LG"}="H331 3LG"; $resn_exch{"2H33 3LG"}="H332 3LG";
$resn_exch{"1H17 3LG"}="H171 3LG"; $resn_exch{"2H17 3LG"}="H172 3LG"; $resn_exch{"1HAA 3LP"}="HAA1 3LP";
$resn_exch{"2HAA 3LP"}="HAA2 3LP"; $resn_exch{"3HAA 3LP"}="HAA3 3LP"; $resn_exch{"1HAK 3LP"}="HAK1 3LP";
$resn_exch{"2HAK 3LP"}="HAK2 3LP"; $resn_exch{"1HAG 3LP"}="HAG1 3LP"; $resn_exch{"2HAG 3LP"}="HAG2 3LP";
$resn_exch{"1HAH 3LP"}="HAH1 3LP"; $resn_exch{"2HAH 3LP"}="HAH2 3LP"; $resn_exch{"1HAL 3LP"}="HAL1 3LP";
$resn_exch{"2HAL 3LP"}="HAL2 3LP"; $resn_exch{"1HAP 3LP"}="HAP1 3LP"; $resn_exch{"2HAP 3LP"}="HAP2 3LP";
$resn_exch{"1HAO 3LP"}="HAO1 3LP"; $resn_exch{"2HAO 3LP"}="HAO2 3LP"; $resn_exch{"1HAM 3LP"}="HAM1 3LP";
$resn_exch{"2HAM 3LP"}="HAM2 3LP"; $resn_exch{"1HAI 3LP"}="HAI1 3LP"; $resn_exch{"2HAI 3LP"}="HAI2 3LP";
$resn_exch{"1HAF 3LP"}="HAF1 3LP"; $resn_exch{"2HAF 3LP"}="HAF2 3LP"; $resn_exch{"1HAJ 3LP"}="HAJ1 3LP";
$resn_exch{"2HAJ 3LP"}="HAJ2 3LP"; $resn_exch{"1HAN 3LP"}="HAN1 3LP"; $resn_exch{"2HAN 3LP"}="HAN2 3LP";
$resn_exch{"1H3  3MA"}=" H31 3MA"; $resn_exch{"2H3  3MA"}=" H32 3MA"; $resn_exch{"3H3  3MA"}=" H33 3MA";
$resn_exch{"1HN6 3MA"}="HN61 3MA"; $resn_exch{"2HN6 3MA"}="HN62 3MA"; $resn_exch{"1H7  3MB"}=" H71 3MB";
$resn_exch{"2H7  3MB"}=" H72 3MB"; $resn_exch{"3H7  3MB"}=" H73 3MB"; $resn_exch{"1HN  3MB"}=" HN1 3MB";
$resn_exch{"2HN  3MB"}=" HN2 3MB"; $resn_exch{"1H3  3MC"}=" H31 3MC"; $resn_exch{"2H3  3MC"}=" H32 3MC";
$resn_exch{"3H3  3MC"}=" H33 3MC"; $resn_exch{"1HN4 3MC"}="HN41 3MC"; $resn_exch{"2HN4 3MC"}="HN42 3MC";
$resn_exch{" HN  3MD"}=" H2  3MD"; $resn_exch{"1HB1 3MD"}="HB11 3MD"; $resn_exch{"2HB1 3MD"}="HB12 3MD";
$resn_exch{"3HB1 3MD"}="HB13 3MD"; $resn_exch{"1H5' 3ME"}="H5'1 3ME"; $resn_exch{"2H5' 3ME"}="H5'2 3ME";
$resn_exch{"1H5A 3ME"}="H5A1 3ME"; $resn_exch{"2H5A 3ME"}="H5A2 3ME"; $resn_exch{"3H5A 3ME"}="H5A3 3ME";
$resn_exch{"1HB' 3ME"}="HB'1 3ME"; $resn_exch{"2HB' 3ME"}="HB'2 3ME"; $resn_exch{"1HC' 3ME"}="HC'1 3ME";
$resn_exch{"2HC' 3ME"}="HC'2 3ME"; $resn_exch{"1HE' 3ME"}="HE'1 3ME"; $resn_exch{"2HE' 3ME"}="HE'2 3ME";
$resn_exch{"1HF' 3ME"}="HF'1 3ME"; $resn_exch{"2HF' 3ME"}="HF'2 3ME"; $resn_exch{"1HI' 3ME"}="HI'1 3ME";
$resn_exch{"2HI' 3ME"}="HI'2 3ME"; $resn_exch{"3HI' 3ME"}="HI'3 3ME"; $resn_exch{"1HJ' 3ME"}="HJ'1 3ME";
$resn_exch{"2HJ' 3ME"}="HJ'2 3ME"; $resn_exch{"3HJ' 3ME"}="HJ'3 3ME"; $resn_exch{"1H1  3MF"}=" H11 3MF";
$resn_exch{"2H1  3MF"}=" H12 3MF"; $resn_exch{"1H6  3MF"}=" H61 3MF"; $resn_exch{"2H6  3MF"}=" H62 3MF";
$resn_exch{"1H7  3MF"}=" H71 3MF"; $resn_exch{"2H7  3MF"}=" H72 3MF"; $resn_exch{"3H7  3MF"}=" H73 3MF";
$resn_exch{"1H2  3MN"}="H21A 3MN"; $resn_exch{"2H2  3MN"}=" H22 3MN"; $resn_exch{"1H1  3MN"}=" H11 3MN";
$resn_exch{"2H1  3MN"}="H12A 3MN"; $resn_exch{"1H16 3MN"}="H161 3MN"; $resn_exch{"2H16 3MN"}="H162 3MN";
$resn_exch{"1H11 3MN"}="H111 3MN"; $resn_exch{"2H11 3MN"}="H112 3MN"; $resn_exch{"1H28 3MN"}="H281 3MN";
$resn_exch{"2H28 3MN"}="H282 3MN"; $resn_exch{"3H28 3MN"}="H283 3MN"; $resn_exch{"1H4  3MO"}=" H41 3MO";
$resn_exch{"2H4  3MO"}=" H42 3MO"; $resn_exch{"1H3  3MO"}=" H31 3MO"; $resn_exch{"2H3  3MO"}=" H32 3MO";
$resn_exch{"1H7  3MO"}=" H71 3MO"; $resn_exch{"2H7  3MO"}=" H72 3MO"; $resn_exch{"3H7  3MO"}=" H73 3MO";
$resn_exch{"1HB  3MP"}=" HB1 3MP"; $resn_exch{"2HB  3MP"}=" HB2 3MP"; $resn_exch{"3HB  3MP"}=" HB3 3MP";
$resn_exch{"1H1  3MR"}=" H11 3MR"; $resn_exch{"2H1  3MR"}=" H12 3MR"; $resn_exch{"3H1  3MR"}=" H13 3MR";
$resn_exch{"1H5  3MR"}=" H51 3MR"; $resn_exch{"2H5  3MR"}=" H52 3MR"; $resn_exch{"1H6  3MR"}=" H61 3MR";
$resn_exch{"2H6  3MR"}=" H62 3MR"; $resn_exch{"1H10 3MR"}="H101 3MR"; $resn_exch{"2H10 3MR"}="H102 3MR";
$resn_exch{"1H11 3MR"}="H111 3MR"; $resn_exch{"2H11 3MR"}="H112 3MR"; $resn_exch{"1H12 3MR"}="H121 3MR";
$resn_exch{"2H12 3MR"}="H122 3MR"; $resn_exch{"1H13 3MR"}="H131 3MR"; $resn_exch{"2H13 3MR"}="H132 3MR";
$resn_exch{"1H14 3MR"}="H141 3MR"; $resn_exch{"2H14 3MR"}="H142 3MR"; $resn_exch{"1H16 3MR"}="H161 3MR";
$resn_exch{"2H16 3MR"}="H162 3MR"; $resn_exch{"1H32 3MR"}="H321 3MR"; $resn_exch{"2H32 3MR"}="H322 3MR";
$resn_exch{"1H34 3MR"}="H341 3MR"; $resn_exch{"2H34 3MR"}="H342 3MR"; $resn_exch{"1H35 3MR"}="H351 3MR";
$resn_exch{"2H35 3MR"}="H352 3MR"; $resn_exch{"1H36 3MR"}="H361 3MR"; $resn_exch{"2H36 3MR"}="H362 3MR";
$resn_exch{"1H37 3MR"}="H371 3MR"; $resn_exch{"2H37 3MR"}="H372 3MR"; $resn_exch{"1H38 3MR"}="H381 3MR";
$resn_exch{"2H38 3MR"}="H382 3MR"; $resn_exch{"1H1  3MT"}="H11A 3MT"; $resn_exch{"2H1  3MT"}=" H12 3MT";
$resn_exch{"3H1  3MT"}=" H13 3MT"; $resn_exch{"1H17 3NA"}="H171 3NA"; $resn_exch{"2H17 3NA"}="H172 3NA";
$resn_exch{"1H20 3NA"}="H201 3NA"; $resn_exch{"2H20 3NA"}="H202 3NA"; $resn_exch{"1H22 3NH"}="H221 3NH";
$resn_exch{"2H22 3NH"}="H222 3NH"; $resn_exch{"1H30 3NH"}="H301 3NH"; $resn_exch{"2H30 3NH"}="H302 3NH";
$resn_exch{"1H10 3NH"}="H101 3NH"; $resn_exch{"2H10 3NH"}="H102 3NH"; $resn_exch{"1H08 3NH"}="H081 3NH";
$resn_exch{"2H08 3NH"}="H082 3NH"; $resn_exch{"1H45 3NH"}="H451 3NH"; $resn_exch{"2H45 3NH"}="H452 3NH";
$resn_exch{"1H47 3NH"}="H471 3NH"; $resn_exch{"2H47 3NH"}="H472 3NH"; $resn_exch{"1H48 3NH"}="H481 3NH";
$resn_exch{"2H48 3NH"}="H482 3NH"; $resn_exch{"1H36 3NH"}="H361 3NH"; $resn_exch{"2H36 3NH"}="H362 3NH";
$resn_exch{"1HN1 3NH"}="HN11 3NH"; $resn_exch{"2HN1 3NH"}="HN12 3NH"; $resn_exch{"1H2  3NP"}=" H21 3NP";
$resn_exch{"2H2  3NP"}=" H22 3NP"; $resn_exch{"1H3  3NP"}=" H31 3NP"; $resn_exch{"2H3  3NP"}=" H32 3NP";
$resn_exch{"1H7C 3NT"}="H7C1 3NT"; $resn_exch{"2H7C 3NT"}="H7C2 3NT"; $resn_exch{"3H7C 3NT"}="H7C3 3NT";
$resn_exch{"1H17 3OD"}="H171 3OD"; $resn_exch{"2H17 3OD"}="H172 3OD"; $resn_exch{"3H17 3OD"}="H173 3OD";
$resn_exch{"1H11 3OD"}="H111 3OD"; $resn_exch{"2H11 3OD"}="H112 3OD"; $resn_exch{"1H1  3OD"}=" H11 3OD";
$resn_exch{"2H1  3OD"}="H12A 3OD"; $resn_exch{"1HN3 3OD"}="HN31 3OD"; $resn_exch{"2HN3 3OD"}="HN32 3OD";
$resn_exch{"1HO1 3OF"}="HO11 3OF"; $resn_exch{"2HO1 3OF"}="HO12 3OF"; $resn_exch{"1HO2 3OF"}="HO21 3OF";
$resn_exch{"2HO2 3OF"}="HO22 3OF"; $resn_exch{"1H2  3OH"}=" H21 3OH"; $resn_exch{"2H2  3OH"}=" H22 3OH";
$resn_exch{"1H3  3OH"}=" H31 3OH"; $resn_exch{"2H3  3OH"}=" H32 3OH"; $resn_exch{"1H1  3OL"}=" H11 3OL";
$resn_exch{"2H1  3OL"}=" H12 3OL"; $resn_exch{"1H2  3OL"}=" H21 3OL"; $resn_exch{"1H3  3OL"}=" H31 3OL";
$resn_exch{"1H4  3OL"}=" H41 3OL"; $resn_exch{"2H4  3OL"}=" H42 3OL"; $resn_exch{"1H5  3OL"}=" H51 3OL";
$resn_exch{"2H5  3OL"}=" H52 3OL"; $resn_exch{"1H6  3OL"}=" H61 3OL"; $resn_exch{"2H6  3OL"}=" H62 3OL";
$resn_exch{"1H7  3OL"}=" H71 3OL"; $resn_exch{"2H7  3OL"}=" H72 3OL"; $resn_exch{"1H8  3OL"}=" H81 3OL";
$resn_exch{"2H8  3OL"}=" H82 3OL"; $resn_exch{"3H8  3OL"}=" H83 3OL"; $resn_exch{"1H4C 3ON"}="H4C1 3ON";
$resn_exch{"2H4C 3ON"}="H4C2 3ON"; $resn_exch{"1H11 3ON"}="H111 3ON"; $resn_exch{"2H11 3ON"}="H112 3ON";
$resn_exch{"3H11 3ON"}="H113 3ON"; $resn_exch{"1H6C 3ON"}="H6C1 3ON"; $resn_exch{"2H6C 3ON"}="H6C2 3ON";
$resn_exch{"1H9C 3ON"}="H9C1 3ON"; $resn_exch{"2H9C 3ON"}="H9C2 3ON"; $resn_exch{"3H9C 3ON"}="H9C3 3ON";
$resn_exch{"1H8C 3ON"}="H8C1 3ON"; $resn_exch{"2H8C 3ON"}="H8C2 3ON"; $resn_exch{"3H8C 3ON"}="H8C3 3ON";
$resn_exch{"1H14 3ON"}="H141 3ON"; $resn_exch{"2H14 3ON"}="H142 3ON"; $resn_exch{"3H14 3ON"}="H143 3ON";
$resn_exch{"1H19 3ON"}="H191 3ON"; $resn_exch{"2H19 3ON"}="H192 3ON"; $resn_exch{"3H19 3ON"}="H193 3ON";
$resn_exch{"1H26 3ON"}="H261 3ON"; $resn_exch{"2H26 3ON"}="H262 3ON"; $resn_exch{"3H26 3ON"}="H263 3ON";
$resn_exch{"1H30 3ON"}="H301 3ON"; $resn_exch{"2H30 3ON"}="H302 3ON"; $resn_exch{"3H30 3ON"}="H303 3ON";
$resn_exch{"1H31 3ON"}="H311 3ON"; $resn_exch{"2H31 3ON"}="H312 3ON"; $resn_exch{"1H4C 3P4"}="H4C1 3P4";
$resn_exch{"2H4C 3P4"}="H4C2 3P4"; $resn_exch{"1H8C 3P4"}="H8C1 3P4"; $resn_exch{"2H8C 3P4"}="H8C2 3P4";
$resn_exch{"3H8C 3P4"}="H8C3 3P4"; $resn_exch{"1H10 3P4"}="H101 3P4"; $resn_exch{"2H10 3P4"}="H102 3P4";
$resn_exch{"1H11 3P4"}="H111 3P4"; $resn_exch{"2H11 3P4"}="H112 3P4"; $resn_exch{"1H15 3P4"}="H151 3P4";
$resn_exch{"2H15 3P4"}="H152 3P4"; $resn_exch{"1H17 3P4"}="H171 3P4"; $resn_exch{"2H17 3P4"}="H172 3P4";
$resn_exch{"1H18 3P4"}="H181 3P4"; $resn_exch{"2H18 3P4"}="H182 3P4"; $resn_exch{"3H18 3P4"}="H183 3P4";
$resn_exch{"1H19 3P4"}="H191 3P4"; $resn_exch{"2H19 3P4"}="H192 3P4"; $resn_exch{"1H22 3P4"}="H221 3P4";
$resn_exch{"2H22 3P4"}="H222 3P4"; $resn_exch{"1H23 3P4"}="H231 3P4"; $resn_exch{"2H23 3P4"}="H232 3P4";
$resn_exch{"1H15 3PC"}="H151 3PC"; $resn_exch{"2H15 3PC"}="H152 3PC"; $resn_exch{"3H15 3PC"}="H153 3PC";
$resn_exch{"1H14 3PC"}="H141 3PC"; $resn_exch{"2H14 3PC"}="H142 3PC"; $resn_exch{"1H13 3PC"}="H131 3PC";
$resn_exch{"2H13 3PC"}="H132 3PC"; $resn_exch{"1H12 3PC"}="H121 3PC"; $resn_exch{"2H12 3PC"}="H122 3PC";
$resn_exch{"1H11 3PC"}="H111 3PC"; $resn_exch{"2H11 3PC"}="H112 3PC"; $resn_exch{"1H9  3PC"}=" H91 3PC";
$resn_exch{"2H9  3PC"}=" H92 3PC"; $resn_exch{"1H17 3PC"}="H171 3PC"; $resn_exch{"2H17 3PC"}="H172 3PC";
$resn_exch{"1H18 3PC"}="H181 3PC"; $resn_exch{"2H18 3PC"}="H182 3PC"; $resn_exch{"1H19 3PC"}="H191 3PC";
$resn_exch{"2H19 3PC"}="H192 3PC"; $resn_exch{"1H20 3PC"}="H201 3PC"; $resn_exch{"2H20 3PC"}="H202 3PC";
$resn_exch{"1H21 3PC"}="H211 3PC"; $resn_exch{"2H21 3PC"}="H212 3PC"; $resn_exch{"3H21 3PC"}="H213 3PC";
$resn_exch{"1H7  3PC"}=" H71 3PC"; $resn_exch{"2H7  3PC"}=" H72 3PC"; $resn_exch{"1H6  3PC"}=" H61 3PC";
$resn_exch{"2H6  3PC"}=" H62 3PC"; $resn_exch{"1H5  3PC"}=" H51 3PC"; $resn_exch{"2H5  3PC"}=" H52 3PC";
$resn_exch{"1H4  3PC"}=" H41 3PC"; $resn_exch{"2H4  3PC"}=" H42 3PC"; $resn_exch{"1H1  3PC"}=" H11 3PC";
$resn_exch{"2H1  3PC"}=" H12 3PC"; $resn_exch{"3H1  3PC"}=" H13 3PC"; $resn_exch{"1H2  3PC"}=" H21 3PC";
$resn_exch{"2H2  3PC"}=" H22 3PC"; $resn_exch{"3H2  3PC"}=" H23 3PC"; $resn_exch{"1H3  3PC"}=" H31 3PC";
$resn_exch{"2H3  3PC"}=" H32 3PC"; $resn_exch{"3H3  3PC"}=" H33 3PC"; $resn_exch{"1H5' 3PD"}="H5'1 3PD";
$resn_exch{"2H5' 3PD"}="H5'2 3PD"; $resn_exch{"1H2' 3PD"}="H2'1 3PD"; $resn_exch{"2H2' 3PD"}="H2'2 3PD";
$resn_exch{"1HN2 3PD"}="HN21 3PD"; $resn_exch{"2HN2 3PD"}="HN22 3PD"; $resn_exch{"1HN  3PE"}=" HN1 3PE";
$resn_exch{"2HN  3PE"}=" HN2 3PE"; $resn_exch{"3HN  3PE"}=" HN3 3PE"; $resn_exch{"1H11 3PE"}="H111 3PE";
$resn_exch{"2H11 3PE"}="H112 3PE"; $resn_exch{"1H12 3PE"}="H121 3PE"; $resn_exch{"2H12 3PE"}="H122 3PE";
$resn_exch{"1H1  3PE"}=" H11 3PE"; $resn_exch{"2H1  3PE"}=" H12 3PE"; $resn_exch{"1H3  3PE"}=" H31 3PE";
$resn_exch{"2H3  3PE"}=" H32 3PE"; $resn_exch{"1H32 3PE"}="H321 3PE"; $resn_exch{"2H32 3PE"}="H322 3PE";
$resn_exch{"1H33 3PE"}="H331 3PE"; $resn_exch{"2H33 3PE"}="H332 3PE"; $resn_exch{"1H34 3PE"}="H341 3PE";
$resn_exch{"2H34 3PE"}="H342 3PE"; $resn_exch{"1H35 3PE"}="H351 3PE"; $resn_exch{"2H35 3PE"}="H352 3PE";
$resn_exch{"1H36 3PE"}="H361 3PE"; $resn_exch{"2H36 3PE"}="H362 3PE"; $resn_exch{"1H37 3PE"}="H371 3PE";
$resn_exch{"2H37 3PE"}="H372 3PE"; $resn_exch{"1H38 3PE"}="H381 3PE"; $resn_exch{"2H38 3PE"}="H382 3PE";
$resn_exch{"1H39 3PE"}="H391 3PE"; $resn_exch{"2H39 3PE"}="H392 3PE"; $resn_exch{"1H3A 3PE"}="H3A1 3PE";
$resn_exch{"2H3A 3PE"}="H3A2 3PE"; $resn_exch{"1H3B 3PE"}="H3B1 3PE"; $resn_exch{"2H3B 3PE"}="H3B2 3PE";
$resn_exch{"1H3C 3PE"}="H3C1 3PE"; $resn_exch{"2H3C 3PE"}="H3C2 3PE"; $resn_exch{"1H3D 3PE"}="H3D1 3PE";
$resn_exch{"2H3D 3PE"}="H3D2 3PE"; $resn_exch{"1H3E 3PE"}="H3E1 3PE"; $resn_exch{"2H3E 3PE"}="H3E2 3PE";
$resn_exch{"1H3F 3PE"}="H3F1 3PE"; $resn_exch{"2H3F 3PE"}="H3F2 3PE"; $resn_exch{"1H3G 3PE"}="H3G1 3PE";
$resn_exch{"2H3G 3PE"}="H3G2 3PE"; $resn_exch{"1H3H 3PE"}="H3H1 3PE"; $resn_exch{"2H3H 3PE"}="H3H2 3PE";
$resn_exch{"1H3I 3PE"}="H3I1 3PE"; $resn_exch{"2H3I 3PE"}="H3I2 3PE"; $resn_exch{"3H3I 3PE"}="H3I3 3PE";
$resn_exch{"1H22 3PE"}="H221 3PE"; $resn_exch{"2H22 3PE"}="H222 3PE"; $resn_exch{"1H23 3PE"}="H231 3PE";
$resn_exch{"2H23 3PE"}="H232 3PE"; $resn_exch{"1H24 3PE"}="H241 3PE"; $resn_exch{"2H24 3PE"}="H242 3PE";
$resn_exch{"1H25 3PE"}="H251 3PE"; $resn_exch{"2H25 3PE"}="H252 3PE"; $resn_exch{"1H26 3PE"}="H261 3PE";
$resn_exch{"2H26 3PE"}="H262 3PE"; $resn_exch{"1H27 3PE"}="H271 3PE"; $resn_exch{"2H27 3PE"}="H272 3PE";
$resn_exch{"1H28 3PE"}="H281 3PE"; $resn_exch{"2H28 3PE"}="H282 3PE"; $resn_exch{"1H29 3PE"}="H291 3PE";
$resn_exch{"2H29 3PE"}="H292 3PE"; $resn_exch{"1H2A 3PE"}="H2A1 3PE"; $resn_exch{"2H2A 3PE"}="H2A2 3PE";
$resn_exch{"1H2B 3PE"}="H2B1 3PE"; $resn_exch{"2H2B 3PE"}="H2B2 3PE"; $resn_exch{"1H2C 3PE"}="H2C1 3PE";
$resn_exch{"2H2C 3PE"}="H2C2 3PE"; $resn_exch{"1H2D 3PE"}="H2D1 3PE"; $resn_exch{"2H2D 3PE"}="H2D2 3PE";
$resn_exch{"1H2E 3PE"}="H2E1 3PE"; $resn_exch{"2H2E 3PE"}="H2E2 3PE"; $resn_exch{"1H2F 3PE"}="H2F1 3PE";
$resn_exch{"2H2F 3PE"}="H2F2 3PE"; $resn_exch{"1H2G 3PE"}="H2G1 3PE"; $resn_exch{"2H2G 3PE"}="H2G2 3PE";
$resn_exch{"1H2H 3PE"}="H2H1 3PE"; $resn_exch{"2H2H 3PE"}="H2H2 3PE"; $resn_exch{"1H2I 3PE"}="H2I1 3PE";
$resn_exch{"2H2I 3PE"}="H2I2 3PE"; $resn_exch{"3H2I 3PE"}="H2I3 3PE"; $resn_exch{"1H3  3PG"}=" H31 3PG";
$resn_exch{"2H3  3PG"}=" H32 3PG"; $resn_exch{"3HOP 3PG"}="HOP3 3PG"; $resn_exch{"4HOP 3PG"}="HOP4 3PG";
$resn_exch{"1H7  3PI"}=" H71 3PI"; $resn_exch{"2H7  3PI"}=" H72 3PI"; $resn_exch{"1H9  3PI"}=" H91 3PI";
$resn_exch{"2H9  3PI"}=" H92 3PI"; $resn_exch{"1H11 3PI"}="H111 3PI"; $resn_exch{"2H11 3PI"}="H112 3PI";
$resn_exch{"1H12 3PI"}="H121 3PI"; $resn_exch{"2H12 3PI"}="H122 3PI"; $resn_exch{"1H13 3PI"}="H131 3PI";
$resn_exch{"2H13 3PI"}="H132 3PI"; $resn_exch{"3H13 3PI"}="H133 3PI"; $resn_exch{"1H15 3PI"}="H151 3PI";
$resn_exch{"2H15 3PI"}="H152 3PI"; $resn_exch{"1H16 3PI"}="H161 3PI"; $resn_exch{"2H16 3PI"}="H162 3PI";
$resn_exch{"1H17 3PI"}="H171 3PI"; $resn_exch{"2H17 3PI"}="H172 3PI"; $resn_exch{"3H17 3PI"}="H173 3PI";
$resn_exch{"1HA  3PL"}=" HA1 3PL"; $resn_exch{"2HA  3PL"}=" HA2 3PL"; $resn_exch{"1HB  3PL"}=" HB1 3PL";
$resn_exch{"2HB  3PL"}=" HB2 3PL"; $resn_exch{"2HOG 3PO"}="HOG2 3PO"; $resn_exch{"3HOG 3PO"}="HOG3 3PO";
$resn_exch{"2HOB 3PO"}="HOB2 3PO"; $resn_exch{"2HOA 3PO"}="HOA2 3PO"; $resn_exch{"HO5* 3PO"}="HO5' 3PO";
$resn_exch{"1HCB 3PP"}="HCB1 3PP"; $resn_exch{"2HCB 3PP"}="HCB2 3PP"; $resn_exch{"1HCG 3PP"}="HCG1 3PP";
$resn_exch{"2HCG 3PP"}="HCG2 3PP"; $resn_exch{"1H3  3PY"}=" H31 3PY"; $resn_exch{"2H3  3PY"}=" H32 3PY";
$resn_exch{"1H2  3QC"}="H21A 3QC"; $resn_exch{"2H2  3QC"}="H22A 3QC"; $resn_exch{"1H33 3QC"}="H331 3QC";
$resn_exch{"2H33 3QC"}="H332 3QC"; $resn_exch{"3H33 3QC"}="H333 3QC"; $resn_exch{"1H4  3QC"}=" H41 3QC";
$resn_exch{"2H4  3QC"}=" H42 3QC"; $resn_exch{"1H29 3QC"}="H291 3QC"; $resn_exch{"2H29 3QC"}="H292 3QC";
$resn_exch{"3H29 3QC"}="H293 3QC"; $resn_exch{"1H40 3QC"}="H401 3QC"; $resn_exch{"2H40 3QC"}="H402 3QC";
$resn_exch{"3H40 3QC"}="H403 3QC"; $resn_exch{"1H44 3QC"}="H441 3QC"; $resn_exch{"2H44 3QC"}="H442 3QC";
$resn_exch{"3H44 3QC"}="H443 3QC"; $resn_exch{"1H6X 3SA"}="H6X1 3SA"; $resn_exch{"2H6X 3SA"}="H6X2 3SA";
$resn_exch{"1H6Y 3SA"}="H6Y1 3SA"; $resn_exch{"2H6Y 3SA"}="H6Y2 3SA"; $resn_exch{"3H6Y 3SA"}="H6Y3 3SA";
$resn_exch{"1H7Z 3SA"}="H7Z1 3SA"; $resn_exch{"2H7Z 3SA"}="H7Z2 3SA"; $resn_exch{"1H12 3SC"}="H121 3SC";
$resn_exch{"2H12 3SC"}="H122 3SC"; $resn_exch{"1H3  3SL"}=" H31 3SL"; $resn_exch{"2H3  3SL"}=" H32 3SL";
$resn_exch{"1HN8 3SP"}="HN81 3SP"; $resn_exch{"2HN8 3SP"}="HN82 3SP"; $resn_exch{"1H9  3SP"}=" H91 3SP";
$resn_exch{"2H9  3SP"}=" H92 3SP"; $resn_exch{"1H14 3SP"}="H141 3SP"; $resn_exch{"2H14 3SP"}="H142 3SP";
$resn_exch{"1H15 3SP"}="H151 3SP"; $resn_exch{"2H15 3SP"}="H152 3SP"; $resn_exch{"1H16 3SP"}="H161 3SP";
$resn_exch{"2H16 3SP"}="H162 3SP"; $resn_exch{"1H22 3SP"}="H221 3SP"; $resn_exch{"2H22 3SP"}="H222 3SP";
$resn_exch{"1H24 3SP"}="H241 3SP"; $resn_exch{"2H24 3SP"}="H242 3SP"; $resn_exch{"1H26 3SP"}="H261 3SP";
$resn_exch{"2H26 3SP"}="H262 3SP"; $resn_exch{"1H27 3SP"}="H271 3SP"; $resn_exch{"2H27 3SP"}="H272 3SP";
$resn_exch{"1H28 3SP"}="H281 3SP"; $resn_exch{"2H28 3SP"}="H282 3SP"; $resn_exch{"1H29 3SP"}="H291 3SP";
$resn_exch{"2H29 3SP"}="H292 3SP"; $resn_exch{"1H30 3SP"}="H301 3SP"; $resn_exch{"2H30 3SP"}="H302 3SP";
$resn_exch{"1HN3 3TC"}="HN31 3TC"; $resn_exch{"2HN3 3TC"}="HN32 3TC"; $resn_exch{"1H6  3TC"}=" H61 3TC";
$resn_exch{"2H6  3TC"}=" H62 3TC"; $resn_exch{"1H8  3TC"}=" H81 3TC"; $resn_exch{"2H8  3TC"}=" H82 3TC";
$resn_exch{"1H15 3TH"}="H151 3TH"; $resn_exch{"2H15 3TH"}="H152 3TH"; $resn_exch{"1H3  3TL"}=" H31 3TL";
$resn_exch{"2H3  3TL"}=" H32 3TL"; $resn_exch{"1H20 3TL"}="H201 3TL"; $resn_exch{"2H20 3TL"}="H202 3TL";
$resn_exch{"3H20 3TL"}="H203 3TL"; $resn_exch{"1HA5 3TL"}="HA51 3TL"; $resn_exch{"2HA5 3TL"}="HA52 3TL";
$resn_exch{"1H70 3TL"}="H701 3TL"; $resn_exch{"2H70 3TL"}="H702 3TL"; $resn_exch{"3H70 3TL"}="H703 3TL";
$resn_exch{"1HG6 3TL"}="HG61 3TL"; $resn_exch{"2HG6 3TL"}="HG62 3TL"; $resn_exch{"3HG6 3TL"}="HG63 3TL";
$resn_exch{"1HG5 3TL"}="HG51 3TL"; $resn_exch{"2HG5 3TL"}="HG52 3TL"; $resn_exch{"3HG5 3TL"}="HG53 3TL";
$resn_exch{"1H53 3TL"}="H531 3TL"; $resn_exch{"2H53 3TL"}="H532 3TL"; $resn_exch{"1HG2 3TL"}="HG21 3TL";
$resn_exch{"2HG2 3TL"}="HG22 3TL"; $resn_exch{"3HG2 3TL"}="HG23 3TL"; $resn_exch{"1HG1 3TL"}="HG11 3TL";
$resn_exch{"2HG1 3TL"}="HG12 3TL"; $resn_exch{"3HG1 3TL"}="HG13 3TL"; $resn_exch{"1HA  3TL"}=" HA1 3TL";
$resn_exch{"2HA  3TL"}=" HA2 3TL"; $resn_exch{"1HAA 3TN"}="HAA1 3TN"; $resn_exch{"2HAA 3TN"}="HAA2 3TN";
$resn_exch{"3HAA 3TN"}="HAA3 3TN"; $resn_exch{"1HAB 3TN"}="HAB1 3TN"; $resn_exch{"2HAB 3TN"}="HAB2 3TN";
$resn_exch{"3HAB 3TN"}="HAB3 3TN"; $resn_exch{"1HAC 3TN"}="HAC1 3TN"; $resn_exch{"2HAC 3TN"}="HAC2 3TN";
$resn_exch{"3HAC 3TN"}="HAC3 3TN"; $resn_exch{"1HAP 3TN"}="HAP1 3TN"; $resn_exch{"2HAP 3TN"}="HAP2 3TN";
$resn_exch{"1HAQ 3TN"}="HAQ1 3TN"; $resn_exch{"2HAQ 3TN"}="HAQ2 3TN"; $resn_exch{"1HAD 3TN"}="HAD1 3TN";
$resn_exch{"2HAD 3TN"}="HAD2 3TN"; $resn_exch{"3HAD 3TN"}="HAD3 3TN"; $resn_exch{"1HAE 3TN"}="HAE1 3TN";
$resn_exch{"2HAE 3TN"}="HAE2 3TN"; $resn_exch{"3HAE 3TN"}="HAE3 3TN"; $resn_exch{"1HN6 3TP"}="HN61 3TP";
$resn_exch{"2HN6 3TP"}="HN62 3TP"; $resn_exch{"1H37 3TP"}="H371 3TP"; $resn_exch{"2H37 3TP"}="H372 3TP";
$resn_exch{"3H37 3TP"}="H373 3TP"; $resn_exch{"1H44 3TP"}="H441 3TP"; $resn_exch{"2H44 3TP"}="H442 3TP";
$resn_exch{"3H44 3TP"}="H443 3TP"; $resn_exch{"1H3A 3TR"}="H3A1 3TR"; $resn_exch{"2H3A 3TR"}="H3A2 3TR";
$resn_exch{"1HT3 3TY"}="HT31 3TY"; $resn_exch{"2HT3 3TY"}="HT32 3TY"; $resn_exch{"1H51 3UN"}="H511 3UN";
$resn_exch{"2H51 3UN"}="H512 3UN"; $resn_exch{"1H54 3UN"}="H541 3UN"; $resn_exch{"2H54 3UN"}="H542 3UN";
$resn_exch{"3H54 3UN"}="H543 3UN"; $resn_exch{"1H15 3UN"}="H151 3UN"; $resn_exch{"2H15 3UN"}="H152 3UN";
$resn_exch{"1H44 3UN"}="H441 3UN"; $resn_exch{"2H44 3UN"}="H442 3UN"; $resn_exch{"1H23 3UN"}="H231 3UN";
$resn_exch{"2H23 3UN"}="H232 3UN"; $resn_exch{"3H23 3UN"}="H233 3UN"; $resn_exch{"1H22 3UN"}="H221 3UN";
$resn_exch{"2H22 3UN"}="H222 3UN"; $resn_exch{"3H22 3UN"}="H223 3UN"; $resn_exch{"1H21 3UN"}="H211 3UN";
$resn_exch{"2H21 3UN"}="H212 3UN"; $resn_exch{"3H21 3UN"}="H213 3UN"; $resn_exch{"1H50 3UN"}="H501 3UN";
$resn_exch{"2H50 3UN"}="H502 3UN"; $resn_exch{"1H60 3UN"}="H601 3UN"; $resn_exch{"2H60 3UN"}="H602 3UN";
$resn_exch{"3H60 3UN"}="H603 3UN"; $resn_exch{"1H35 3UN"}="H351 3UN"; $resn_exch{"2H35 3UN"}="H352 3UN";
$resn_exch{"1HAF 3YP"}="HAF1 3YP"; $resn_exch{"2HAF 3YP"}="HAF2 3YP"; $resn_exch{"1HAC 3YP"}="HAC1 3YP";
$resn_exch{"2HAC 3YP"}="HAC2 3YP"; $resn_exch{"1HAD 3YP"}="HAD1 3YP"; $resn_exch{"2HAD 3YP"}="HAD2 3YP";
$resn_exch{"1HAG 3YP"}="HAG1 3YP"; $resn_exch{"2HAG 3YP"}="HAG2 3YP"; $resn_exch{"1HAH 3YP"}="HAH1 3YP";
$resn_exch{"2HAH 3YP"}="HAH2 3YP"; $resn_exch{"1HAE 3YP"}="HAE1 3YP"; $resn_exch{"2HAE 3YP"}="HAE2 3YP";
$resn_exch{"1H39 406"}="H391 406"; $resn_exch{"2H39 406"}="H392 406"; $resn_exch{"3H39 406"}="H393 406";
$resn_exch{"1H11 406"}="H111 406"; $resn_exch{"2H11 406"}="H112 406"; $resn_exch{"1H13 406"}="H131 406";
$resn_exch{"2H13 406"}="H132 406"; $resn_exch{"1H14 406"}="H141 406"; $resn_exch{"2H14 406"}="H142 406";
$resn_exch{"1H16 406"}="H161 406"; $resn_exch{"2H16 406"}="H162 406"; $resn_exch{"1H18 406"}="H181 406";
$resn_exch{"2H18 406"}="H182 406"; $resn_exch{"3H18 406"}="H183 406"; $resn_exch{"1H19 406"}="H191 406";
$resn_exch{"2H19 406"}="H192 406"; $resn_exch{"3H19 406"}="H193 406"; $resn_exch{"1H07 412"}="H071 412";
$resn_exch{"2H07 412"}="H072 412"; $resn_exch{"1H08 412"}="H081 412"; $resn_exch{"2H08 412"}="H082 412";
$resn_exch{"1H12 412"}="H121 412"; $resn_exch{"2H12 412"}="H122 412"; $resn_exch{"1H14 412"}="H141 412";
$resn_exch{"2H14 412"}="H142 412"; $resn_exch{"1H19 412"}="H191 412"; $resn_exch{"2H19 412"}="H192 412";
$resn_exch{"1H1N 413"}="H1N1 413"; $resn_exch{"2H1N 413"}="H1N2 413"; $resn_exch{"1H15 413"}="H151 413";
$resn_exch{"2H15 413"}="H152 413"; $resn_exch{"3H15 413"}="H153 413"; $resn_exch{"1H16 413"}="H161 413";
$resn_exch{"2H16 413"}="H162 413"; $resn_exch{"1H33 414"}="H331 414"; $resn_exch{"2H33 414"}="H332 414";
$resn_exch{"1H51 414"}="H511 414"; $resn_exch{"2H51 414"}="H512 414"; $resn_exch{"1H52 414"}="H521 414";
$resn_exch{"2H52 414"}="H522 414"; $resn_exch{"1H53 414"}="H531 414"; $resn_exch{"2H53 414"}="H532 414";
$resn_exch{"1H54 414"}="H541 414"; $resn_exch{"2H54 414"}="H542 414"; $resn_exch{"1H55 414"}="H551 414";
$resn_exch{"2H55 414"}="H552 414"; $resn_exch{"1H11 414"}="H111 414"; $resn_exch{"2H11 414"}="H112 414";
$resn_exch{"1H10 417"}="H101 417"; $resn_exch{"2H10 417"}="H102 417"; $resn_exch{"1H11 417"}="H111 417";
$resn_exch{"2H11 417"}="H112 417"; $resn_exch{"1H13 417"}="H131 417"; $resn_exch{"2H13 417"}="H132 417";
$resn_exch{"1H22 417"}="H221 417"; $resn_exch{"2H22 417"}="H222 417"; $resn_exch{"1H23 417"}="H231 417";
$resn_exch{"2H23 417"}="H232 417"; $resn_exch{"1H26 417"}="H261 417"; $resn_exch{"2H26 417"}="H262 417";
$resn_exch{"1H38 417"}="H381 417"; $resn_exch{"2H38 417"}="H382 417"; $resn_exch{"1H7  418"}=" H71 418";
$resn_exch{"2H7  418"}=" H72 418"; $resn_exch{"1H40 418"}="H401 418"; $resn_exch{"2H40 418"}="H402 418";
$resn_exch{"1H42 418"}="H421 418"; $resn_exch{"2H42 418"}="H422 418"; $resn_exch{"1H12 418"}="H121 418";
$resn_exch{"2H12 418"}="H122 418"; $resn_exch{"1H13 418"}="H131 418"; $resn_exch{"2H13 418"}="H132 418";
$resn_exch{"1H15 418"}="H151 418"; $resn_exch{"2H15 418"}="H152 418"; $resn_exch{"1H14 418"}="H141 418";
$resn_exch{"2H14 418"}="H142 418"; $resn_exch{"1H39 418"}="H391 418"; $resn_exch{"2H39 418"}="H392 418";
$resn_exch{"3H39 418"}="H393 418"; $resn_exch{"1H12 421"}="H121 421"; $resn_exch{"2H12 421"}="H122 421";
$resn_exch{"1H23 421"}="H231 421"; $resn_exch{"2H23 421"}="H232 421"; $resn_exch{"3H23 421"}="H233 421";
$resn_exch{"1H28 421"}="H281 421"; $resn_exch{"2H28 421"}="H282 421"; $resn_exch{"3H28 421"}="H283 421";
$resn_exch{"1H7  422"}=" H71 422"; $resn_exch{"2H7  422"}=" H72 422"; $resn_exch{"1H8  422"}=" H81 422";
$resn_exch{"2H8  422"}=" H82 422"; $resn_exch{"1H9  422"}=" H91 422"; $resn_exch{"2H9  422"}=" H92 422";
$resn_exch{"1H10 422"}="H101 422"; $resn_exch{"2H10 422"}="H102 422"; $resn_exch{"1H13 422"}="H131 422";
$resn_exch{"2H13 422"}="H132 422"; $resn_exch{"1H18 422"}="H181 422"; $resn_exch{"2H18 422"}="H182 422";
$resn_exch{"1HN3 422"}="HN31 422"; $resn_exch{"2HN3 422"}="HN32 422"; $resn_exch{"1H19 426"}="H191 426";
$resn_exch{"2H19 426"}="H192 426"; $resn_exch{"1H38 426"}="H381 426"; $resn_exch{"2H38 426"}="H382 426";
$resn_exch{"1H37 426"}="H371 426"; $resn_exch{"2H37 426"}="H372 426"; $resn_exch{"1H36 426"}="H361 426";
$resn_exch{"2H36 426"}="H362 426"; $resn_exch{"1H35 426"}="H351 426"; $resn_exch{"2H35 426"}="H352 426";
$resn_exch{"1H12 427"}="H121 427"; $resn_exch{"2H12 427"}="H122 427"; $resn_exch{"1H14 427"}="H141 427";
$resn_exch{"2H14 427"}="H142 427"; $resn_exch{"1H17 427"}="H171 427"; $resn_exch{"2H17 427"}="H172 427";
$resn_exch{"1H12 429"}="H121 429"; $resn_exch{"2H12 429"}="H122 429"; $resn_exch{"1H13 429"}="H131 429";
$resn_exch{"2H13 429"}="H132 429"; $resn_exch{"1H14 429"}="H141 429"; $resn_exch{"2H14 429"}="H142 429";
$resn_exch{"1H15 429"}="H151 429"; $resn_exch{"2H15 429"}="H152 429"; $resn_exch{"1H23 429"}="H231 429";
$resn_exch{"2H23 429"}="H232 429"; $resn_exch{"3H23 429"}="H233 429"; $resn_exch{"1H8  429"}=" H81 429";
$resn_exch{"2H8  429"}=" H82 429"; $resn_exch{"1H7  429"}=" H71 429"; $resn_exch{"2H7  429"}=" H72 429";
$resn_exch{"1H39 429"}="H391 429"; $resn_exch{"2H39 429"}="H392 429"; $resn_exch{"3H39 429"}="H393 429";
$resn_exch{"1H1N 42B"}="H1N1 42B"; $resn_exch{"2H1N 42B"}="H1N2 42B"; $resn_exch{"1H5C 42B"}="H5C1 42B";
$resn_exch{"2H5C 42B"}="H5C2 42B"; $resn_exch{"1H6C 42B"}="H6C1 42B"; $resn_exch{"2H6C 42B"}="H6C2 42B";
$resn_exch{"1HOB 433"}="HOB1 433"; $resn_exch{"1H1X 433"}="H1X1 433"; $resn_exch{"2H1X 433"}="H1X2 433";
$resn_exch{"3H1X 433"}="H1X3 433"; $resn_exch{"1H1Y 433"}="H1Y1 433"; $resn_exch{"2H1Y 433"}="H1Y2 433";
$resn_exch{"3H1Y 433"}="H1Y3 433"; $resn_exch{"1H1G 433"}="H1G1 433"; $resn_exch{"1H1I 433"}="H1I1 433";
$resn_exch{"1H1J 433"}="H1J1 433"; $resn_exch{"1H1H 433"}="H1H1 433"; $resn_exch{"1H1L 433"}="H1L1 433";
$resn_exch{"2H1L 433"}="H1L2 433"; $resn_exch{"1H1M 433"}="H1M1 433"; $resn_exch{"2H1M 433"}="H1M2 433";
$resn_exch{"1H1N 433"}="H1N1 433"; $resn_exch{"2H1N 433"}="H1N2 433"; $resn_exch{"1H2C 433"}="H2C1 433";
$resn_exch{"2H2C 433"}="H2C2 433"; $resn_exch{"1HN3 433"}="HN31 433"; $resn_exch{"1H3C 433"}="H3C1 433";
$resn_exch{"1H3E 433"}="H3E1 433"; $resn_exch{"1H3G 433"}="H3G1 433"; $resn_exch{"1H2D 433"}="H2D1 433";
$resn_exch{"2H2D 433"}="H2D2 433"; $resn_exch{"1H2H 433"}="H2H1 433"; $resn_exch{"1H2J 433"}="H2J1 433";
$resn_exch{"1H2I 433"}="H2I1 433"; $resn_exch{"1H3  434"}=" H31 434"; $resn_exch{"2H3  434"}=" H32 434";
$resn_exch{"3H3  434"}="H33A 434"; $resn_exch{"1H9  43A"}=" H91 43A"; $resn_exch{"2H9  43A"}=" H92 43A";
$resn_exch{"3H9  43A"}=" H93 43A"; $resn_exch{"1H7  43M"}=" H71 43M"; $resn_exch{"2H7  43M"}=" H72 43M";
$resn_exch{"3H7  43M"}=" H73 43M"; $resn_exch{"1HAS 43P"}="HAS1 43P"; $resn_exch{"2HAS 43P"}="HAS2 43P";
$resn_exch{"1HAT 43P"}="HAT1 43P"; $resn_exch{"2HAT 43P"}="HAT2 43P"; $resn_exch{"3HAT 43P"}="HAT3 43P";
$resn_exch{"1H7  442"}=" H71 442"; $resn_exch{"2H7  442"}=" H72 442"; $resn_exch{"1H29 442"}="H291 442";
$resn_exch{"2H29 442"}="H292 442"; $resn_exch{"1H30 442"}="H301 442"; $resn_exch{"2H30 442"}="H302 442";
$resn_exch{"1H16 444"}="H161 444"; $resn_exch{"2H16 444"}="H162 444"; $resn_exch{"1H11 448"}="H111 448";
$resn_exch{"2H11 448"}="H112 448"; $resn_exch{"1H20 448"}="H201 448"; $resn_exch{"2H20 448"}="H202 448";
$resn_exch{"1H15 448"}="H151 448"; $resn_exch{"2H15 448"}="H152 448"; $resn_exch{"1H25 448"}="H251 448";
$resn_exch{"2H25 448"}="H252 448"; $resn_exch{"1H26 448"}="H261 448"; $resn_exch{"2H26 448"}="H262 448";
$resn_exch{"1H34 448"}="H341 448"; $resn_exch{"2H34 448"}="H342 448"; $resn_exch{"3H34 448"}="H343 448";
$resn_exch{"1H16 44B"}="H161 44B"; $resn_exch{"2H16 44B"}="H162 44B"; $resn_exch{"1H17 44C"}="H171 44C";
$resn_exch{"2H17 44C"}="H172 44C"; $resn_exch{"1H18 44C"}="H181 44C"; $resn_exch{"2H18 44C"}="H182 44C";
$resn_exch{"1H19 44C"}="H191 44C"; $resn_exch{"2H19 44C"}="H192 44C"; $resn_exch{"1H20 44C"}="H201 44C";
$resn_exch{"2H20 44C"}="H202 44C"; $resn_exch{"1HC8 44D"}="HC81 44D"; $resn_exch{"2HC8 44D"}="HC82 44D";
$resn_exch{"1H10 44D"}="H101 44D"; $resn_exch{"2H10 44D"}="H102 44D"; $resn_exch{"1H14 44D"}="H141 44D";
$resn_exch{"2H14 44D"}="H142 44D"; $resn_exch{" H1* 44D"}=" H1B 44D"; $resn_exch{"1H2* 44D"}="H2'1 44D";
$resn_exch{"2H2* 44D"}="H2'2 44D"; $resn_exch{" H3* 44D"}=" H3B 44D"; $resn_exch{" H4* 44D"}=" H4B 44D";
$resn_exch{" H5* 44D"}=" H5B 44D"; $resn_exch{"1H6* 44D"}="H6'1 44D"; $resn_exch{"2H6* 44D"}="H6'2 44D";
$resn_exch{"3H6* 44D"}="H6'3 44D"; $resn_exch{"OH3* 44D"}="H3'O 44D"; $resn_exch{"1H2' 44D"}=" H2B 44D";
$resn_exch{"2H2' 44D"}=" H2D 44D"; $resn_exch{"1H6' 44D"}=" H6B 44D"; $resn_exch{"2H6' 44D"}=" H6D 44D";
$resn_exch{"3H6' 44D"}=" H6E 44D"; $resn_exch{"OH4' 44D"}="H4'O 44D"; $resn_exch{"1H3' 44D"}="H3'1 44D";
$resn_exch{"2H3' 44D"}="H3'2 44D"; $resn_exch{"1H2  450"}="H21A 450"; $resn_exch{"2H2  450"}="H22A 450";
$resn_exch{"1H7  450"}="H71A 450"; $resn_exch{"2H7  450"}=" H72 450"; $resn_exch{"1HN2 450"}="HN21 450";
$resn_exch{"2HN2 450"}="HN22 450"; $resn_exch{"1H31 450"}="H311 450"; $resn_exch{"2H31 450"}="H312 450";
$resn_exch{"1H61 450"}="H611 450"; $resn_exch{"2H61 450"}="H612 450"; $resn_exch{"1HN7 450"}="HN71 450";
$resn_exch{"2HN7 450"}="HN72 450"; $resn_exch{"1H1  452"}=" H11 452"; $resn_exch{"2H1  452"}="H12A 452";
$resn_exch{"3H1  452"}="H13A 452"; $resn_exch{"1H9  452"}=" H91 452"; $resn_exch{"2H9  452"}=" H92 452";
$resn_exch{"1HAK 458"}="HAK1 458"; $resn_exch{"2HAK 458"}="HAK2 458"; $resn_exch{"1HAR 458"}="HAR1 458";
$resn_exch{"2HAR 458"}="HAR2 458"; $resn_exch{"3HAR 458"}="HAR3 458"; $resn_exch{"1HAP 458"}="HAP1 458";
$resn_exch{"2HAP 458"}="HAP2 458"; $resn_exch{"1HAT 458"}="HAT1 458"; $resn_exch{"2HAT 458"}="HAT2 458";
$resn_exch{"1HAM 458"}="HAM1 458"; $resn_exch{"2HAM 458"}="HAM2 458"; $resn_exch{"1HAT 459"}="HAT1 459";
$resn_exch{"2HAT 459"}="HAT2 459"; $resn_exch{"3HAT 459"}="HAT3 459"; $resn_exch{"1HAQ 459"}="HAQ1 459";
$resn_exch{"2HAQ 459"}="HAQ2 459"; $resn_exch{"3HAQ 459"}="HAQ3 459"; $resn_exch{"1HAP 459"}="HAP1 459";
$resn_exch{"2HAP 459"}="HAP2 459"; $resn_exch{"1HAO 459"}="HAO1 459"; $resn_exch{"2HAO 459"}="HAO2 459";
$resn_exch{"1HAR 459"}="HAR1 459"; $resn_exch{"2HAR 459"}="HAR2 459"; $resn_exch{"1HAM 459"}="HAM1 459";
$resn_exch{"2HAM 459"}="HAM2 459"; $resn_exch{"1H1  460"}=" H11 460"; $resn_exch{"2H1  460"}=" H12 460";
$resn_exch{"3H1  460"}=" H13 460"; $resn_exch{"1H12 460"}="H121 460"; $resn_exch{"2H12 460"}="H122 460";
$resn_exch{"1H38 464"}="H381 464"; $resn_exch{"2H38 464"}="H382 464"; $resn_exch{"1H39 464"}="H391 464";
$resn_exch{"2H39 464"}="H392 464"; $resn_exch{"1H40 464"}="H401 464"; $resn_exch{"2H40 464"}="H402 464";
$resn_exch{"1H41 464"}="H411 464"; $resn_exch{"2H41 464"}="H412 464"; $resn_exch{"1H42 464"}="H421 464";
$resn_exch{"2H42 464"}="H422 464"; $resn_exch{"1H36 464"}="H361 464"; $resn_exch{"2H36 464"}="H362 464";
$resn_exch{"1H35 464"}="H351 464"; $resn_exch{"2H35 464"}="H352 464"; $resn_exch{"1H18 464"}="H181 464";
$resn_exch{"2H18 464"}="H182 464"; $resn_exch{"1H14 464"}="H141 464"; $resn_exch{"2H14 464"}="H142 464";
$resn_exch{"1H10 468"}="H101 468"; $resn_exch{"2H10 468"}="H102 468"; $resn_exch{"1H8  468"}=" H81 468";
$resn_exch{"2H8  468"}=" H82 468"; $resn_exch{"1H4  468"}=" H41 468"; $resn_exch{"2H4  468"}=" H42 468";
$resn_exch{"1H23 468"}="H231 468"; $resn_exch{"2H23 468"}="H232 468"; $resn_exch{"3H23 468"}="H233 468";
$resn_exch{"1HP6 46D"}="HP61 46D"; $resn_exch{"2HP6 46D"}="HP62 46D"; $resn_exch{"1HP8 46D"}="HP81 46D";
$resn_exch{"2HP8 46D"}="HP82 46D"; $resn_exch{"3HP8 46D"}="HP83 46D"; $resn_exch{"1H8  46M"}=" H81 46M";
$resn_exch{"2H8  46M"}=" H82 46M"; $resn_exch{"3H8  46M"}=" H83 46M"; $resn_exch{"1H6  46M"}=" H61 46M";
$resn_exch{"2H6  46M"}=" H62 46M"; $resn_exch{"1HB  471"}=" HB1 471"; $resn_exch{"2HB  471"}=" HB2 471";
$resn_exch{"1H1B 471"}="H1B1 471"; $resn_exch{"2H1B 471"}="H1B2 471"; $resn_exch{"3HIB 471"}="HIB3 471";
$resn_exch{"1H3E 471"}="H3E1 471"; $resn_exch{"2H3E 471"}="H3E2 471"; $resn_exch{"3H3E 471"}="H3E3 471";
$resn_exch{"1H3A 471"}="H3A1 471"; $resn_exch{"2H3A 471"}="H3A2 471"; $resn_exch{"1H3B 471"}="H3B1 471";
$resn_exch{"2H3B 471"}="H3B2 471"; $resn_exch{"1H4A 471"}="H4A1 471"; $resn_exch{"2H4A 471"}="H4A2 471";
$resn_exch{"1H4D 471"}="H4D1 471"; $resn_exch{"2H4D 471"}="H4D2 471"; $resn_exch{"1H4E 471"}="H4E1 471";
$resn_exch{"2H4E 471"}="H4E2 471"; $resn_exch{"3H4E 471"}="H4E3 471"; $resn_exch{"1HN6 474"}="HN61 474";
$resn_exch{"2HN6 474"}="HN62 474"; $resn_exch{"1H21 474"}="H211 474"; $resn_exch{"2H21 474"}="H212 474";
$resn_exch{"1H20 474"}="H201 474"; $resn_exch{"2H20 474"}="H202 474"; $resn_exch{"1H18 474"}="H181 474";
$resn_exch{"2H18 474"}="H182 474"; $resn_exch{"1H10 474"}="H101 474"; $resn_exch{"2H10 474"}="H102 474";
$resn_exch{"3H10 474"}="H103 474"; $resn_exch{"1H14 474"}="H141 474"; $resn_exch{"2H14 474"}="H142 474";
$resn_exch{"3H14 474"}="H143 474"; $resn_exch{"1H31 474"}="H311 474"; $resn_exch{"2H31 474"}="H312 474";
$resn_exch{"1H30 474"}="H301 474"; $resn_exch{"2H30 474"}="H302 474"; $resn_exch{"1H34 474"}="H341 474";
$resn_exch{"2H34 474"}="H342 474"; $resn_exch{"1H33 474"}="H331 474"; $resn_exch{"2H33 474"}="H332 474";
$resn_exch{"1H1  478"}="H11A 478"; $resn_exch{"2H1  478"}="H12A 478"; $resn_exch{"1H2  478"}="H21A 478";
$resn_exch{"2H2  478"}="H22A 478"; $resn_exch{"1H7  478"}=" H71 478"; $resn_exch{"2H7  478"}=" H72 478";
$resn_exch{"1H14 478"}="H141 478"; $resn_exch{"2H14 478"}="H142 478"; $resn_exch{"1H15 478"}="H151 478";
$resn_exch{"2H15 478"}="H152 478"; $resn_exch{"1H23 478"}="H231 478"; $resn_exch{"2H23 478"}="H232 478";
$resn_exch{"3H23 478"}="H233 478"; $resn_exch{"1H24 478"}="H241 478"; $resn_exch{"2H24 478"}="H242 478";
$resn_exch{"3H24 478"}="H243 478"; $resn_exch{"1H25 478"}="H251 478"; $resn_exch{"2H25 478"}="H252 478";
$resn_exch{"1HN3 478"}="HN31 478"; $resn_exch{"2HN3 478"}="HN32 478"; $resn_exch{"1H3  47D"}=" H31 47D";
$resn_exch{"2H3  47D"}=" H32 47D"; $resn_exch{"1H5  47D"}=" H51 47D"; $resn_exch{"2H5  47D"}=" H52 47D";
$resn_exch{"1H8  485"}=" H81 485"; $resn_exch{"2H8  485"}=" H82 485"; $resn_exch{"1H10 485"}="H101 485";
$resn_exch{"2H10 485"}="H102 485"; $resn_exch{"1H27 485"}="H271 485"; $resn_exch{"2H27 485"}="H272 485";
$resn_exch{"3H27 485"}="H273 485"; $resn_exch{"1HC1 486"}="HC1A 486"; $resn_exch{"2HC1 486"}="HC12 486";
$resn_exch{"1HC7 486"}="HC71 486"; $resn_exch{"2HC7 486"}="HC72 486"; $resn_exch{"1HC6 486"}="HC61 486";
$resn_exch{"2HC6 486"}="HC62 486"; $resn_exch{"1H29 486"}="H291 486"; $resn_exch{"2H29 486"}="H292 486";
$resn_exch{"3H29 486"}="H293 486"; $resn_exch{"1H28 486"}="H281 486"; $resn_exch{"2H28 486"}="H282 486";
$resn_exch{"3H28 486"}="H283 486"; $resn_exch{"1H13 486"}="H131 486"; $resn_exch{"2H13 486"}="H132 486";
$resn_exch{"1H19 486"}="H191 486"; $resn_exch{"2H19 486"}="H192 486"; $resn_exch{"3H19 486"}="H193 486";
$resn_exch{"1H32 486"}="H321 486"; $resn_exch{"2H32 486"}="H322 486"; $resn_exch{"3H32 486"}="H323 486";
$resn_exch{"1H16 486"}="H161 486"; $resn_exch{"2H16 486"}="H162 486"; $resn_exch{"1H15 486"}="H151 486";
$resn_exch{"2H15 486"}="H152 486"; $resn_exch{"1HC8 486"}="HC81 486"; $resn_exch{"2HC8 486"}="HC82 486";
$resn_exch{"1H7  493"}=" H71 493"; $resn_exch{"2H7  493"}="H72A 493"; $resn_exch{"1H9  493"}=" H91 493";
$resn_exch{"2H9  493"}=" H92 493"; $resn_exch{"1H15 493"}="H151 493"; $resn_exch{"2H15 493"}="H152 493";
$resn_exch{"3H15 493"}="H153 493"; $resn_exch{"1H19 493"}="H191 493"; $resn_exch{"2H19 493"}="H192 493";
$resn_exch{"1H20 493"}="H201 493"; $resn_exch{"2H20 493"}="H202 493"; $resn_exch{"1H21 493"}="H211 493";
$resn_exch{"2H21 493"}="H212 493"; $resn_exch{"1H22 493"}="H221 493"; $resn_exch{"2H22 493"}="H222 493";
$resn_exch{"1H25 493"}="H251 493"; $resn_exch{"2H25 493"}="H252 493"; $resn_exch{"1H47 497"}="H471 497";
$resn_exch{"2H47 497"}="H472 497"; $resn_exch{"3H47 497"}="H473 497"; $resn_exch{"1H45 497"}="H451 497";
$resn_exch{"2H45 497"}="H452 497"; $resn_exch{"1H38 497"}="H381 497"; $resn_exch{"2H38 497"}="H382 497";
$resn_exch{"1H36 497"}="H361 497"; $resn_exch{"2H36 497"}="H362 497"; $resn_exch{"1H18 497"}="H181 497";
$resn_exch{"2H18 497"}="H182 497"; $resn_exch{"1H9  49A"}=" H91 49A"; $resn_exch{"2H9  49A"}=" H92 49A";
$resn_exch{"1H11 49A"}="H111 49A"; $resn_exch{"2H11 49A"}="H112 49A"; $resn_exch{"3H11 49A"}="H113 49A";
$resn_exch{"1HOB 49A"}="HOB1 49A"; $resn_exch{"1HN4 49A"}="HN41 49A"; $resn_exch{"2HN4 49A"}="HN42 49A";
$resn_exch{"1HN9 49A"}="HN91 49A"; $resn_exch{"2HN9 49A"}="HN92 49A"; $resn_exch{"1HN1 4A3"}="HN11 4A3";
$resn_exch{"2HN1 4A3"}="HN12 4A3"; $resn_exch{"1HAA 4AA"}="HAA1 4AA"; $resn_exch{"2HAA 4AA"}="HAA2 4AA";
$resn_exch{"2H2  4AB"}=" H22 4AB"; $resn_exch{"1H2  4AB"}=" H21 4AB"; $resn_exch{"1HH4 4AB"}="HH41 4AB";
$resn_exch{"2HH4 4AB"}="HH42 4AB"; $resn_exch{"1H7  4AB"}=" H71 4AB"; $resn_exch{"2H7  4AB"}=" H72 4AB";
$resn_exch{"1H9O 4AB"}="H9O1 4AB"; $resn_exch{"3H11 4AB"}="H113 4AB"; $resn_exch{"2H11 4AB"}="H112 4AB";
$resn_exch{"1H11 4AB"}="H111 4AB"; $resn_exch{"1H1O 4AB"}="H1O1 4AB"; $resn_exch{"3HOP 4AC"}="HOP3 4AC";
$resn_exch{"2HOP 4AC"}="HOP2 4AC"; $resn_exch{"1H5* 4AC"}=" H5' 4AC"; $resn_exch{"2H5* 4AC"}="H5'' 4AC";
$resn_exch{" H4* 4AC"}=" H4' 4AC"; $resn_exch{" H3* 4AC"}=" H3' 4AC"; $resn_exch{"H3T  4AC"}="HO3' 4AC";
$resn_exch{" H2* 4AC"}=" H2' 4AC"; $resn_exch{"2HO* 4AC"}="HO2' 4AC"; $resn_exch{" H1* 4AC"}=" H1' 4AC";
$resn_exch{"1HM7 4AC"}="HM71 4AC"; $resn_exch{"2HM7 4AC"}="HM72 4AC"; $resn_exch{"3HM7 4AC"}="HM73 4AC";
$resn_exch{"1HOP 4AD"}="HOP1 4AD"; $resn_exch{"1HN  4AD"}=" HN1 4AD"; $resn_exch{"2HN  4AD"}=" HN2 4AD";
$resn_exch{"3HN  4AD"}=" HN3 4AD"; $resn_exch{"1HB  4AD"}=" HB1 4AD"; $resn_exch{"2HB  4AD"}=" HB2 4AD";
$resn_exch{"1HD2 4AD"}="HD21 4AD"; $resn_exch{"2HD2 4AD"}="HD22 4AD"; $resn_exch{"1H5* 4AD"}="H5'1 4AD";
$resn_exch{"2H5* 4AD"}="H5'2 4AD"; $resn_exch{" H4* 4AD"}=" H4' 4AD"; $resn_exch{" H3* 4AD"}=" H3' 4AD";
$resn_exch{" H2* 4AD"}=" H2' 4AD"; $resn_exch{" H1* 4AD"}=" H1' 4AD"; $resn_exch{"1HN6 4AD"}="HN61 4AD";
$resn_exch{"2HN6 4AD"}="HN62 4AD"; $resn_exch{"1H11 4AF"}="H111 4AF"; $resn_exch{"2H11 4AF"}="H112 4AF";
$resn_exch{"3H11 4AF"}="H113 4AF"; $resn_exch{"1H3  4AF"}=" H31 4AF"; $resn_exch{"2H3  4AF"}=" H32 4AF";
$resn_exch{"1HN  4AF"}=" HN1 4AF"; $resn_exch{"2HN  4AF"}=" HN2 4AF"; $resn_exch{"1H12 4AH"}="H121 4AH";
$resn_exch{"2H12 4AH"}="H122 4AH"; $resn_exch{"1H24 4AH"}="H241 4AH"; $resn_exch{"2H24 4AH"}="H242 4AH";
$resn_exch{"1H41 4AH"}="H411 4AH"; $resn_exch{"2H41 4AH"}="H412 4AH"; $resn_exch{"1H44 4AH"}="H441 4AH";
$resn_exch{"2H44 4AH"}="H442 4AH"; $resn_exch{"1H49 4AH"}="H491 4AH"; $resn_exch{"2H49 4AH"}="H492 4AH";
$resn_exch{"3H49 4AH"}="H493 4AH"; $resn_exch{"1H50 4AH"}="H501 4AH"; $resn_exch{"2H50 4AH"}="H502 4AH";
$resn_exch{"3H50 4AH"}="H503 4AH"; $resn_exch{"1H91 4AH"}="H911 4AH"; $resn_exch{"2H91 4AH"}="H912 4AH";
$resn_exch{"3H91 4AH"}="H913 4AH"; $resn_exch{"1H92 4AH"}="H921 4AH"; $resn_exch{"2H92 4AH"}="H922 4AH";
$resn_exch{"3H92 4AH"}="H923 4AH"; $resn_exch{"1HOB 4AM"}="HOB1 4AM"; $resn_exch{"1HN4 4AM"}="HN41 4AM";
$resn_exch{"2HN4 4AM"}="HN42 4AM"; $resn_exch{" NH5 4AM"}=" H5N 4AM"; $resn_exch{"1H11 4AM"}="H111 4AM";
$resn_exch{"2H11 4AM"}="H112 4AM"; $resn_exch{"3H11 4AM"}="H113 4AM"; $resn_exch{" OH7 4AM"}=" H7O 4AM";
$resn_exch{" OH8 4AM"}=" H8O 4AM"; $resn_exch{"1H9  4AM"}=" H91 4AM"; $resn_exch{"2H9  4AM"}=" H92 4AM";
$resn_exch{"1HN4 4AN"}="HN41 4AN"; $resn_exch{"2HN4 4AN"}="HN42 4AN"; $resn_exch{"1HN4 4AP"}="HN41 4AP";
$resn_exch{"2HN4 4AP"}="HN42 4AP"; $resn_exch{"1HA  4AT"}=" HA1 4AT"; $resn_exch{"2HA  4AT"}=" HA2 4AT";
$resn_exch{"1HB  4AT"}=" HB1 4AT"; $resn_exch{"2HB  4AT"}=" HB2 4AT"; $resn_exch{"1HG  4AT"}=" HG1 4AT";
$resn_exch{"2HG  4AT"}=" HG2 4AT"; $resn_exch{"1H2A 4AT"}="H2A1 4AT"; $resn_exch{"2H2A 4AT"}="H2A2 4AT";
$resn_exch{"3H2A 4AT"}="H2A3 4AT"; $resn_exch{"1HB  4AX"}=" HB1 4AX"; $resn_exch{"2HB  4AX"}=" HB2 4AX";
$resn_exch{"1H21 4B3"}="H211 4B3"; $resn_exch{"2H21 4B3"}="H212 4B3"; $resn_exch{"1H30 4B3"}="H301 4B3";
$resn_exch{"2H30 4B3"}="H302 4B3"; $resn_exch{"1H32 4B3"}="H321 4B3"; $resn_exch{"2H32 4B3"}="H322 4B3";
$resn_exch{"1H34 4B3"}="H341 4B3"; $resn_exch{"2H34 4B3"}="H342 4B3"; $resn_exch{"1H42 4B3"}="H421 4B3";
$resn_exch{"2H42 4B3"}="H422 4B3"; $resn_exch{"3H42 4B3"}="H423 4B3"; $resn_exch{"1H46 4B3"}="H461 4B3";
$resn_exch{"2H46 4B3"}="H462 4B3"; $resn_exch{"3H46 4B3"}="H463 4B3"; $resn_exch{"1H50 4B3"}="H501 4B3";
$resn_exch{"2H50 4B3"}="H502 4B3"; $resn_exch{"3H50 4B3"}="H503 4B3"; $resn_exch{"1H54 4B3"}="H541 4B3";
$resn_exch{"2H54 4B3"}="H542 4B3"; $resn_exch{"3H54 4B3"}="H543 4B3"; $resn_exch{"1H19 4BC"}="H191 4BC";
$resn_exch{"2H19 4BC"}="H192 4BC"; $resn_exch{"1H26 4BC"}="H261 4BC"; $resn_exch{"2H26 4BC"}="H262 4BC";
$resn_exch{"1H27 4BC"}="H271 4BC"; $resn_exch{"2H27 4BC"}="H272 4BC"; $resn_exch{"3H27 4BC"}="H273 4BC";
$resn_exch{"1HCB 4BF"}=" HB3 4BF"; $resn_exch{"2HCB 4BF"}=" HB2 4BF"; $resn_exch{"1HN  4BF"}=" H   4BF";
$resn_exch{"2HN  4BF"}=" H2  4BF"; $resn_exch{"1H11 4BH"}="H111 4BH"; $resn_exch{"2H11 4BH"}="H112 4BH";
$resn_exch{"1H12 4BH"}="H121 4BH"; $resn_exch{"2H12 4BH"}="H122 4BH"; $resn_exch{"1H15 4BH"}="H151 4BH";
$resn_exch{"2H15 4BH"}="H152 4BH"; $resn_exch{"1H14 4BH"}="H141 4BH"; $resn_exch{"2H14 4BH"}="H142 4BH";
$resn_exch{"1HZ2 4BR"}="HZ21 4BR"; $resn_exch{"2HZ2 4BR"}="HZ22 4BR"; $resn_exch{"1HA2 4BR"}="HA21 4BR";
$resn_exch{"2HA2 4BR"}="HA22 4BR"; $resn_exch{"1HH2 4BR"}="HH21 4BR"; $resn_exch{"2HH2 4BR"}="HH22 4BR";
$resn_exch{"3HH2 4BR"}="HH23 4BR"; $resn_exch{"1HCA 4BT"}="HCA1 4BT"; $resn_exch{"2HCA 4BT"}="HCA2 4BT";
$resn_exch{"1HCB 4BT"}="HCB1 4BT"; $resn_exch{"2HCB 4BT"}="HCB2 4BT"; $resn_exch{"1H5  4BT"}=" H51 4BT";
$resn_exch{"1H6  4BT"}=" H61 4BT"; $resn_exch{"1H2  4BT"}=" H21 4BT"; $resn_exch{"1H3  4BT"}=" H31 4BT";
$resn_exch{"1HCD 4BT"}="HCD1 4BT"; $resn_exch{"2HCD 4BT"}="HCD2 4BT"; $resn_exch{"1HCC 4BT"}="HCC1 4BT";
$resn_exch{"2HCC 4BT"}="HCC2 4BT"; $resn_exch{"1H1' 4BT"}="H1'1 4BT"; $resn_exch{"2H1' 4BT"}="H1'2 4BT";
$resn_exch{"1HN1 4BT"}="HN11 4BT"; $resn_exch{"2HN1 4BT"}="HN12 4BT"; $resn_exch{"1HN2 4BT"}="HN21 4BT";
$resn_exch{"2HB  4BZ"}=" HB2 4BZ"; $resn_exch{"3HB  4BZ"}=" HB3 4BZ"; $resn_exch{"1HQ1 4BZ"}="HQ11 4BZ";
$resn_exch{"2HQ1 4BZ"}="HQ12 4BZ"; $resn_exch{"1HQ2 4BZ"}="HQ21 4BZ"; $resn_exch{" AH2 4CA"}=" H2A 4CA";
$resn_exch{"AH61 4CA"}="H61A 4CA"; $resn_exch{"AH62 4CA"}="H62A 4CA"; $resn_exch{" AH8 4CA"}=" H8A 4CA";
$resn_exch{"AH1* 4CA"}=" H1D 4CA"; $resn_exch{"AH2* 4CA"}=" H2D 4CA"; $resn_exch{"AHO2 4CA"}="HO2A 4CA";
$resn_exch{"AH3* 4CA"}=" H3D 4CA"; $resn_exch{"8HOA 4CA"}="HOA8 4CA"; $resn_exch{"9HOA 4CA"}="HOA9 4CA";
$resn_exch{"AH4* 4CA"}=" H4D 4CA"; $resn_exch{"AH51 4CA"}="H51A 4CA"; $resn_exch{"AH52 4CA"}="H52A 4CA";
$resn_exch{"2HOA 4CA"}="HOA2 4CA"; $resn_exch{"5HOA 4CA"}="HOA5 4CA"; $resn_exch{"1H12 4CA"}="H121 4CA";
$resn_exch{"2H12 4CA"}="H122 4CA"; $resn_exch{"1H13 4CA"}="H131 4CA"; $resn_exch{"2H13 4CA"}="H132 4CA";
$resn_exch{"3H13 4CA"}="H133 4CA"; $resn_exch{"1H14 4CA"}="H141 4CA"; $resn_exch{"2H14 4CA"}="H142 4CA";
$resn_exch{"3H14 4CA"}="H143 4CA"; $resn_exch{"1H7  4CA"}=" H71 4CA"; $resn_exch{"2H7  4CA"}=" H72 4CA";
$resn_exch{"1H6  4CA"}=" H61 4CA"; $resn_exch{"2H6  4CA"}=" H62 4CA"; $resn_exch{"1H3  4CA"}=" H31 4CA";
$resn_exch{"2H3  4CA"}=" H32 4CA"; $resn_exch{"1H2  4CA"}=" H21 4CA"; $resn_exch{"2H2  4CA"}=" H22 4CA";
$resn_exch{"1H1  4CA"}=" H11 4CA"; $resn_exch{"2H1  4CA"}=" H12 4CA"; $resn_exch{" BH3 4CA"}=" H3B 4CA";
$resn_exch{" BH4 4CA"}=" H4B 4CA"; $resn_exch{"BHO2 4CA"}="HO2B 4CA"; $resn_exch{" BH6 4CA"}=" H6B 4CA";
$resn_exch{" BH7 4CA"}=" H7B 4CA"; $resn_exch{"1HO1 4CB"}="HO11 4CB"; $resn_exch{"1HO2 4CB"}="HO21 4CB";
$resn_exch{"1HC2 4CB"}="HC21 4CB"; $resn_exch{"1HC3 4CB"}="HC31 4CB"; $resn_exch{"1HOX 4CB"}="HOX1 4CB";
$resn_exch{"1HC5 4CB"}="HC51 4CB"; $resn_exch{"1HC6 4CB"}="HC61 4CB"; $resn_exch{"1HN1 4CM"}="HN11 4CM";
$resn_exch{"2HN1 4CM"}="HN12 4CM"; $resn_exch{"1H8  4CM"}=" H81 4CM"; $resn_exch{"2H8  4CM"}=" H82 4CM";
$resn_exch{"3H8  4CM"}=" H83 4CM"; $resn_exch{" AH2 4CO"}=" H2A 4CO"; $resn_exch{"AH61 4CO"}="H61A 4CO";
$resn_exch{"AH62 4CO"}="H62A 4CO"; $resn_exch{" AH8 4CO"}=" H8A 4CO"; $resn_exch{"AH1* 4CO"}=" H1D 4CO";
$resn_exch{"AH2* 4CO"}=" H2D 4CO"; $resn_exch{"AHO2 4CO"}="HO2A 4CO"; $resn_exch{"AH3* 4CO"}=" H3D 4CO";
$resn_exch{"8HOA 4CO"}="HOA8 4CO"; $resn_exch{"9HOA 4CO"}="HOA9 4CO"; $resn_exch{"AH4* 4CO"}=" H4D 4CO";
$resn_exch{"AH51 4CO"}="H51A 4CO"; $resn_exch{"AH52 4CO"}="H52A 4CO"; $resn_exch{"2HOA 4CO"}="HOA2 4CO";
$resn_exch{"5HOA 4CO"}="HOA5 4CO"; $resn_exch{"1H12 4CO"}="H121 4CO"; $resn_exch{"2H12 4CO"}="H122 4CO";
$resn_exch{"1H13 4CO"}="H131 4CO"; $resn_exch{"2H13 4CO"}="H132 4CO"; $resn_exch{"3H13 4CO"}="H133 4CO";
$resn_exch{"1H14 4CO"}="H141 4CO"; $resn_exch{"2H14 4CO"}="H142 4CO"; $resn_exch{"3H14 4CO"}="H143 4CO";
$resn_exch{"1H7  4CO"}=" H71 4CO"; $resn_exch{"2H7  4CO"}=" H72 4CO"; $resn_exch{"1H6  4CO"}=" H61 4CO";
$resn_exch{"2H6  4CO"}=" H62 4CO"; $resn_exch{"1H3  4CO"}=" H31 4CO"; $resn_exch{"2H3  4CO"}=" H32 4CO";
$resn_exch{"1H2  4CO"}=" H21 4CO"; $resn_exch{"2H2  4CO"}=" H22 4CO"; $resn_exch{" BH3 4CO"}=" H3B 4CO";
$resn_exch{" BH4 4CO"}=" H4B 4CO"; $resn_exch{" BH2 4CO"}=" H2B 4CO"; $resn_exch{" BH6 4CO"}=" H6B 4CO";
$resn_exch{" BH7 4CO"}=" H7B 4CO"; $resn_exch{"1BH  4CO"}=" H1B 4CO"; $resn_exch{"2BH  4CO"}="H2BX 4CO";
$resn_exch{"1H9C 4CP"}="H9C1 4CP"; $resn_exch{"2H9C 4CP"}="H9C2 4CP"; $resn_exch{"1H14 4CP"}="H141 4CP";
$resn_exch{"2H14 4CP"}="H142 4CP"; $resn_exch{"1H24 4CP"}="H241 4CP"; $resn_exch{"2H24 4CP"}="H242 4CP";
$resn_exch{"1H27 4CP"}="H271 4CP"; $resn_exch{"2H27 4CP"}="H272 4CP"; $resn_exch{"1H4B 4CR"}="H4B1 4CR";
$resn_exch{"2H4B 4CR"}="H4B2 4CR"; $resn_exch{"1H4C 4CR"}="H4C1 4CR"; $resn_exch{"2H4C 4CR"}="H4C2 4CR";
$resn_exch{"3H4C 4CR"}="H4C3 4CR"; $resn_exch{"1HN4 4CR"}="HN41 4CR"; $resn_exch{"2HN4 4CR"}="HN42 4CR";
$resn_exch{"3HN4 4CR"}="HN43 4CR"; $resn_exch{"1H2C 4CR"}="H2C1 4CR"; $resn_exch{"2H2C 4CR"}="H2C2 4CR";
$resn_exch{"1H1C 4CR"}="H1C1 4CR"; $resn_exch{"2H1C 4CR"}="H1C2 4CR"; $resn_exch{"1H10 4CR"}="H101 4CR";
$resn_exch{"2H10 4CR"}="H102 4CR"; $resn_exch{"3H10 4CR"}="H103 4CR"; $resn_exch{"1H13 4CR"}="H131 4CR";
$resn_exch{"2H13 4CR"}="H132 4CR"; $resn_exch{"1HB  4CS"}=" HB1 4CS"; $resn_exch{"2HB  4CS"}=" HB2 4CS";
$resn_exch{"1HAD 4CS"}="HAD1 4CS"; $resn_exch{"2HAD 4CS"}="HAD2 4CS"; $resn_exch{"1HAA 4CS"}="HAA1 4CS";
$resn_exch{"2HAA 4CS"}="HAA2 4CS"; $resn_exch{"3HAA 4CS"}="HAA3 4CS"; $resn_exch{"1H5  4DE"}=" H51 4DE";
$resn_exch{"2H5  4DE"}=" H52 4DE"; $resn_exch{"1H6  4DE"}=" H61 4DE"; $resn_exch{"2H6  4DE"}=" H62 4DE";
$resn_exch{"3H6  4DE"}=" H63 4DE"; $resn_exch{"1H8  4DE"}=" H81 4DE"; $resn_exch{"2H8  4DE"}=" H82 4DE";
$resn_exch{"3H8  4DE"}=" H83 4DE"; $resn_exch{"1H13 4DE"}="H131 4DE"; $resn_exch{"2H13 4DE"}="H132 4DE";
$resn_exch{"3H13 4DE"}="H133 4DE"; $resn_exch{"1H18 4DE"}="H181 4DE"; $resn_exch{"2H18 4DE"}="H182 4DE";
$resn_exch{"3H18 4DE"}="H183 4DE"; $resn_exch{"1H2  4DI"}=" H21 4DI"; $resn_exch{"2H2  4DI"}=" H22 4DI";
$resn_exch{"1H3  4DI"}=" H31 4DI"; $resn_exch{"2H3  4DI"}=" H32 4DI"; $resn_exch{"1H4  4DI"}=" H41 4DI";
$resn_exch{"2H4  4DI"}=" H42 4DI"; $resn_exch{"1H5  4DI"}=" H51 4DI"; $resn_exch{"2H5  4DI"}=" H52 4DI";
$resn_exch{"1H6  4DI"}=" H61 4DI"; $resn_exch{"2H6  4DI"}=" H62 4DI"; $resn_exch{"3H6  4DI"}=" H63 4DI";
$resn_exch{"1H7  4DI"}=" H71 4DI"; $resn_exch{"2H7  4DI"}=" H72 4DI"; $resn_exch{"3H7  4DI"}=" H73 4DI";
$resn_exch{"1HN2 4DI"}="HN21 4DI"; $resn_exch{"2HN2 4DI"}="HN22 4DI"; $resn_exch{"1HAD 4DP"}="HAD1 4DP";
$resn_exch{"2HAD 4DP"}="HAD2 4DP"; $resn_exch{"3HAD 4DP"}="HAD3 4DP"; $resn_exch{"1HAC 4DP"}="HAC1 4DP";
$resn_exch{"2HAC 4DP"}="HAC2 4DP"; $resn_exch{"3HAC 4DP"}="HAC3 4DP"; $resn_exch{"1HB  4DP"}=" HB3 4DP";
$resn_exch{"2HB  4DP"}=" HB2 4DP"; $resn_exch{"1HN  4DP"}=" H   4DP"; $resn_exch{"2HN  4DP"}=" H2  4DP";
$resn_exch{"HOXT 4DP"}=" HXT 4DP"; $resn_exch{"1H24 4EA"}="H241 4EA"; $resn_exch{"2H24 4EA"}="H242 4EA";
$resn_exch{"1H25 4EA"}="H251 4EA"; $resn_exch{"2H25 4EA"}="H252 4EA"; $resn_exch{"1HN4 4EA"}="HN41 4EA";
$resn_exch{"2HN4 4EA"}="HN42 4EA"; $resn_exch{"0H1  4EB"}=" H10 4EB"; $resn_exch{"1H1  4EB"}=" H11 4EB";
$resn_exch{"2H1  4EB"}=" H12 4EB"; $resn_exch{"3H1  4EB"}=" H13 4EB"; $resn_exch{"1H11 4F3"}="H111 4F3";
$resn_exch{"2H11 4F3"}="H112 4F3"; $resn_exch{"1H18 4F3"}="H181 4F3"; $resn_exch{"2H18 4F3"}="H182 4F3";
$resn_exch{"2HO4 4F3"}="HO42 4F3"; $resn_exch{"1H25 4F3"}="H251 4F3"; $resn_exch{"2H25 4F3"}="H252 4F3";
$resn_exch{"3H25 4F3"}="H253 4F3"; $resn_exch{"1HC4 4FA"}="HC41 4FA"; $resn_exch{"2HC4 4FA"}="HC42 4FA";
$resn_exch{"1H13 4FA"}="H131 4FA"; $resn_exch{"2H13 4FA"}="H132 4FA"; $resn_exch{"1HBX 4FB"}="HBX1 4FB";
$resn_exch{"2HBX 4FB"}="HBX2 4FB"; $resn_exch{"1HDX 4FB"}="HDX1 4FB"; $resn_exch{"2HDX 4FB"}="HDX2 4FB";
$resn_exch{"1H6  4FP"}=" H61 4FP"; $resn_exch{"2H6  4FP"}=" H62 4FP"; $resn_exch{"1H8  4FP"}=" H81 4FP";
$resn_exch{"2H8  4FP"}=" H82 4FP"; $resn_exch{"1H9  4FP"}=" H91 4FP"; $resn_exch{"2H9  4FP"}=" H92 4FP";
$resn_exch{"1H11 4FP"}="H111 4FP"; $resn_exch{"2H11 4FP"}="H112 4FP"; $resn_exch{"1H12 4FP"}="H121 4FP";
$resn_exch{"2H12 4FP"}="H122 4FP"; $resn_exch{"1HN  4FW"}=" H   4FW"; $resn_exch{"2HN  4FW"}=" H2  4FW";
$resn_exch{"1HB  4FW"}=" HB2 4FW"; $resn_exch{"2HB  4FW"}=" HB3 4FW"; $resn_exch{" HNE 4FW"}=" HE1 4FW";
$resn_exch{"1H6  4GP"}=" H61 4GP"; $resn_exch{"2H6  4GP"}=" H62 4GP"; $resn_exch{"2HB  4HA"}=" HB2 4HA";
$resn_exch{"3HB  4HA"}=" HB3 4HA"; $resn_exch{"2HG  4HA"}=" HG2 4HA"; $resn_exch{"3HG  4HA"}=" HG3 4HA";
$resn_exch{"2HD  4HA"}=" HD2 4HA"; $resn_exch{"3HD  4HA"}=" HD3 4HA"; $resn_exch{"2HE  4HA"}=" HE2 4HA";
$resn_exch{"3HE  4HA"}=" HE3 4HA"; $resn_exch{"1HZ  4HA"}=" HZ1 4HA"; $resn_exch{"2HZ  4HA"}=" HZ2 4HA";
$resn_exch{"3HZ  4HA"}=" HZ3 4HA"; $resn_exch{"1H5  4HF"}=" H51 4HF"; $resn_exch{"2H5  4HF"}=" H52 4HF";
$resn_exch{"1HN2 4HF"}="HN21 4HF"; $resn_exch{"2HN2 4HF"}="HN22 4HF"; $resn_exch{"1H7  4HF"}=" H71 4HF";
$resn_exch{"2H7  4HF"}=" H72 4HF"; $resn_exch{"1H9  4HF"}=" H91 4HF"; $resn_exch{"2H9  4HF"}=" H92 4HF";
$resn_exch{"1H10 4HF"}="H101 4HF"; $resn_exch{"2H10 4HF"}="H102 4HF"; $resn_exch{"1HB  4HF"}=" HB1 4HF";
$resn_exch{"2HB  4HF"}=" HB2 4HF"; $resn_exch{"1HG  4HF"}=" HG1 4HF"; $resn_exch{"2HG  4HF"}=" HG2 4HF";
$resn_exch{"1H11 4HG"}="H111 4HG"; $resn_exch{"2H11 4HG"}="H112 4HG"; $resn_exch{"1H1  4HL"}=" H11 4HL";
$resn_exch{"2H1  4HL"}=" H12 4HL"; $resn_exch{"1HX2 4HL"}="HX21 4HL"; $resn_exch{"2HX2 4HL"}="HX22 4HL";
$resn_exch{"1H7  4HP"}=" H71 4HP"; $resn_exch{"2H7  4HP"}=" H72 4HP"; $resn_exch{"1HN  4HT"}=" H   4HT";
$resn_exch{"2HN  4HT"}=" H2  4HT"; $resn_exch{"1HB  4HT"}=" HB2 4HT"; $resn_exch{"2HB  4HT"}=" HB3 4HT";
$resn_exch{"1HN  4HX"}=" HN1 4HX"; $resn_exch{"2HN  4HX"}=" HN2 4HX"; $resn_exch{"1HB  4HX"}=" HB1 4HX";
$resn_exch{"2HB  4HX"}=" HB2 4HX"; $resn_exch{"1HG  4HX"}=" HG1 4HX"; $resn_exch{"2HG  4HX"}=" HG2 4HX";
$resn_exch{"1HD  4HX"}=" HD1 4HX"; $resn_exch{"2HD  4HX"}=" HD2 4HX"; $resn_exch{"1H1' 4HX"}="H1'1 4HX";
$resn_exch{"2H1' 4HX"}="H1'2 4HX"; $resn_exch{"1HB' 4HX"}="HB'1 4HX"; $resn_exch{"2HB' 4HX"}="HB'2 4HX";
$resn_exch{"1HD' 4HX"}="HD'1 4HX"; $resn_exch{"2HD' 4HX"}="HD'2 4HX"; $resn_exch{"1HC3 4HY"}="HC31 4HY";
$resn_exch{"1HC4 4HY"}="HC41 4HY"; $resn_exch{"1H10 4HY"}="H101 4HY"; $resn_exch{"1H11 4HY"}="H111 4HY";
$resn_exch{"1H12 4HY"}="H121 4HY"; $resn_exch{"1H13 4HY"}="H131 4HY"; $resn_exch{"2H13 4HY"}="H132 4HY";
$resn_exch{"1HO1 4HY"}="HO11 4HY"; $resn_exch{"1HO4 4HY"}="HO41 4HY"; $resn_exch{"1HN3 4IG"}="HN31 4IG";
$resn_exch{"2HN3 4IG"}="HN32 4IG"; $resn_exch{"1HN4 4IG"}="HN41 4IG"; $resn_exch{"2HN4 4IG"}="HN42 4IG";
$resn_exch{"1H5  4IG"}=" H51 4IG"; $resn_exch{"2H5  4IG"}=" H52 4IG"; $resn_exch{"1H6  4IG"}=" H61 4IG";
$resn_exch{"2H6  4IG"}=" H62 4IG"; $resn_exch{"3H6  4IG"}=" H63 4IG"; $resn_exch{"1H16 4IG"}="H161 4IG";
$resn_exch{"2H16 4IG"}="H162 4IG"; $resn_exch{"1H17 4IG"}="H171 4IG"; $resn_exch{"2H17 4IG"}="H172 4IG";
$resn_exch{"1H18 4IG"}="H181 4IG"; $resn_exch{"2H18 4IG"}="H182 4IG"; $resn_exch{"1H19 4IG"}="H191 4IG";
$resn_exch{"2H19 4IG"}="H192 4IG"; $resn_exch{"3H19 4IG"}="H193 4IG"; $resn_exch{"1HN  4IN"}=" H   4IN";
$resn_exch{"2HN  4IN"}=" H2  4IN"; $resn_exch{"1HB  4IN"}=" HB2 4IN"; $resn_exch{"2HB  4IN"}=" HB3 4IN";
$resn_exch{"1H16 4IN"}="H161 4IN"; $resn_exch{"2H16 4IN"}="H162 4IN"; $resn_exch{"PHO2 4IP"}="HO2P 4IP";
$resn_exch{"PHO3 4IP"}="HO3P 4IP"; $resn_exch{"PHO5 4IP"}="HO5P 4IP"; $resn_exch{"PHO6 4IP"}="HO6P 4IP";
$resn_exch{"PHO7 4IP"}="HO7P 4IP"; $resn_exch{"PHO8 4IP"}="HO8P 4IP"; $resn_exch{"1HOP 4IP"}="HOP1 4IP";
$resn_exch{"3HOP 4IP"}="HOP3 4IP"; $resn_exch{"1H1  4LG"}=" H11 4LG"; $resn_exch{"2H1  4LG"}=" H12 4LG";
$resn_exch{"3H1  4LG"}="H13A 4LG"; $resn_exch{"1H4  4LG"}=" H41 4LG"; $resn_exch{"2H4  4LG"}=" H42 4LG";
$resn_exch{"1H17 4LG"}="H171 4LG"; $resn_exch{"2H17 4LG"}="H172 4LG"; $resn_exch{"1H19 4LG"}="H191 4LG";
$resn_exch{"2H19 4LG"}="H192 4LG"; $resn_exch{"3H19 4LG"}="H193 4LG"; $resn_exch{"1H21 4LG"}="H211 4LG";
$resn_exch{"2H21 4LG"}="H212 4LG"; $resn_exch{"1H22 4LG"}="H221 4LG"; $resn_exch{"2H22 4LG"}="H222 4LG";
$resn_exch{"1H24 4LG"}="H241 4LG"; $resn_exch{"2H24 4LG"}="H242 4LG"; $resn_exch{"1H31 4LG"}="H311 4LG";
$resn_exch{"2H31 4LG"}="H312 4LG"; $resn_exch{"1H32 4LG"}="H321 4LG"; $resn_exch{"2H32 4LG"}="H322 4LG";
$resn_exch{"1H33 4LG"}="H331 4LG"; $resn_exch{"2H33 4LG"}="H332 4LG"; $resn_exch{"1H8  4MA"}=" H81 4MA";
$resn_exch{"2H8  4MA"}=" H82 4MA"; $resn_exch{"3H8  4MA"}=" H83 4MA"; $resn_exch{"1H12 4MB"}="H121 4MB";
$resn_exch{"2H12 4MB"}="H122 4MB"; $resn_exch{"3H12 4MB"}="H123 4MB"; $resn_exch{"1H33 4MC"}="H331 4MC";
$resn_exch{"2H33 4MC"}="H332 4MC"; $resn_exch{"3H33 4MC"}="H333 4MC"; $resn_exch{"1H49 4MC"}="H491 4MC";
$resn_exch{"2H49 4MC"}="H492 4MC"; $resn_exch{"1H48 4MC"}="H481 4MC"; $resn_exch{"2H48 4MC"}="H482 4MC";
$resn_exch{"1H39 4MC"}="H391 4MC"; $resn_exch{"2H39 4MC"}="H392 4MC"; $resn_exch{"1H40 4MC"}="H401 4MC";
$resn_exch{"2H40 4MC"}="H402 4MC"; $resn_exch{"1H10 4MC"}="H101 4MC"; $resn_exch{"2H10 4MC"}="H102 4MC";
$resn_exch{"1H11 4MC"}="H111 4MC"; $resn_exch{"2H11 4MC"}="H112 4MC"; $resn_exch{"1H01 4MC"}="H011 4MC";
$resn_exch{"2H01 4MC"}="H012 4MC"; $resn_exch{"1H02 4MC"}="H021 4MC"; $resn_exch{"2H02 4MC"}="H022 4MC";
$resn_exch{"1H19 4MC"}="H191 4MC"; $resn_exch{"2H19 4MC"}="H192 4MC"; $resn_exch{"1H20 4MC"}="H201 4MC";
$resn_exch{"2H20 4MC"}="H202 4MC"; $resn_exch{"3HOP 4MF"}="HOP3 4MF"; $resn_exch{"2HOP 4MF"}="HOP2 4MF";
$resn_exch{"1H5* 4MF"}="H5'1 4MF"; $resn_exch{"2H5* 4MF"}="H5'2 4MF"; $resn_exch{" H4* 4MF"}=" H4' 4MF";
$resn_exch{" H3* 4MF"}=" H3' 4MF"; $resn_exch{"1H2* 4MF"}="H2'1 4MF"; $resn_exch{"2H2* 4MF"}="H2'2 4MF";
$resn_exch{" H1* 4MF"}=" H1' 4MF"; $resn_exch{"1H4M 4MF"}="H4M1 4MF"; $resn_exch{"2H4M 4MF"}="H4M2 4MF";
$resn_exch{"3H4M 4MF"}="H4M3 4MF"; $resn_exch{"1H2  4MP"}=" H21 4MP"; $resn_exch{"2H2  4MP"}=" H22 4MP";
$resn_exch{"1H3  4MP"}=" H31 4MP"; $resn_exch{"2H3  4MP"}=" H32 4MP"; $resn_exch{"1H4  4MP"}=" H41 4MP";
$resn_exch{"2H4  4MP"}=" H42 4MP"; $resn_exch{"1H12 4MP"}="H121 4MP"; $resn_exch{"2H12 4MP"}="H122 4MP";
$resn_exch{"3H12 4MP"}="H123 4MP"; $resn_exch{"1HAR 4MR"}="HAR1 4MR"; $resn_exch{"2HAR 4MR"}="HAR2 4MR";
$resn_exch{"1HAQ 4MR"}="HAQ1 4MR"; $resn_exch{"2HAQ 4MR"}="HAQ2 4MR"; $resn_exch{"1HBC 4MR"}="HBC1 4MR";
$resn_exch{"2HBC 4MR"}="HBC2 4MR"; $resn_exch{"1HBB 4MR"}="HBB1 4MR"; $resn_exch{"2HBB 4MR"}="HBB2 4MR";
$resn_exch{"1HA  4MV"}=" HA1 4MV"; $resn_exch{"2HA  4MV"}=" HA2 4MV"; $resn_exch{"1HB  4MV"}=" HB1 4MV";
$resn_exch{"2HB  4MV"}=" HB2 4MV"; $resn_exch{"1HD1 4MV"}="HD11 4MV"; $resn_exch{"2HD1 4MV"}="HD12 4MV";
$resn_exch{"3HD1 4MV"}="HD13 4MV"; $resn_exch{"1HD2 4MV"}="HD21 4MV"; $resn_exch{"2HD2 4MV"}="HD22 4MV";
$resn_exch{"3HD2 4MV"}="HD23 4MV"; $resn_exch{"1H4  4MZ"}=" H41 4MZ"; $resn_exch{"2H4  4MZ"}=" H42 4MZ";
$resn_exch{"3H4  4MZ"}=" H43 4MZ"; $resn_exch{"1HC4 4ND"}="HC41 4ND"; $resn_exch{"1HC5 4ND"}="HC51 4ND";
$resn_exch{"1HC6 4ND"}="HC61 4ND"; $resn_exch{"1HC7 4ND"}="HC71 4ND"; $resn_exch{"1HC8 4ND"}="HC81 4ND";
$resn_exch{"1H10 4ND"}="H101 4ND"; $resn_exch{"1H11 4ND"}="H111 4ND"; $resn_exch{"1H12 4ND"}="H121 4ND";
$resn_exch{"1H13 4ND"}="H131 4ND"; $resn_exch{"1H14 4ND"}="H141 4ND"; $resn_exch{"1HN4 4ND"}="HN41 4ND";
$resn_exch{"1HN3 4ND"}="HN31 4ND"; $resn_exch{"1HN2 4ND"}="HN21 4ND"; $resn_exch{"2HN2 4ND"}="HN22 4ND";
$resn_exch{"1HN1 4ND"}="HN11 4ND"; $resn_exch{"1H1  4NH"}="H11A 4NH"; $resn_exch{"2H1  4NH"}=" H12 4NH";
$resn_exch{"3H1  4NH"}=" H13 4NH"; $resn_exch{"1H4  4NH"}=" H41 4NH"; $resn_exch{"2H4  4NH"}=" H42 4NH";
$resn_exch{"1H5  4NH"}=" H51 4NH"; $resn_exch{"2H5  4NH"}=" H52 4NH"; $resn_exch{"1H12 4NH"}="H121 4NH";
$resn_exch{"2H12 4NH"}="H122 4NH"; $resn_exch{"1H15 4NH"}="H151 4NH"; $resn_exch{"2H15 4NH"}="H152 4NH";
$resn_exch{"1HN2 4NH"}="HN21 4NH"; $resn_exch{"2HN2 4NH"}="HN22 4NH"; $resn_exch{"1H17 4NH"}="H171 4NH";
$resn_exch{"2H17 4NH"}="H172 4NH"; $resn_exch{"3H17 4NH"}="H173 4NH"; $resn_exch{"1H2  4NI"}=" H21 4NI";
$resn_exch{"2H2  4NI"}=" H22 4NI"; $resn_exch{"1H3  4NI"}=" H31 4NI"; $resn_exch{"2H3  4NI"}=" H32 4NI";
$resn_exch{"1H4  4NI"}=" H41 4NI"; $resn_exch{"2H4  4NI"}=" H42 4NI"; $resn_exch{"1H2  4NP"}=" H21 4NP";
$resn_exch{"1H3  4NP"}=" H31 4NP"; $resn_exch{"1H5  4NP"}=" H51 4NP"; $resn_exch{"1H6  4NP"}=" H61 4NP";
$resn_exch{"1HM4 4OC"}="HM41 4OC"; $resn_exch{"2HM4 4OC"}="HM42 4OC"; $resn_exch{"3HM4 4OC"}="HM43 4OC";
$resn_exch{" H1* 4OC"}=" H1' 4OC"; $resn_exch{" H4* 4OC"}=" H4' 4OC"; $resn_exch{"1H5* 4OC"}=" H5' 4OC";
$resn_exch{"2H5* 4OC"}="H5'' 4OC"; $resn_exch{"3HOP 4OC"}="HOP3 4OC"; $resn_exch{" H3* 4OC"}=" H3' 4OC";
$resn_exch{"H3T  4OC"}="HO3' 4OC"; $resn_exch{" H2* 4OC"}=" H2' 4OC"; $resn_exch{"1HM2 4OC"}="HM21 4OC";
$resn_exch{"2HM2 4OC"}="HM22 4OC"; $resn_exch{"3HM2 4OC"}="HM23 4OC"; $resn_exch{"2HOP 4OC"}="HOP2 4OC";
$resn_exch{"1HC7 4OX"}="HC71 4OX"; $resn_exch{"2HC7 4OX"}="HC72 4OX"; $resn_exch{"1HC8 4OX"}="HC81 4OX";
$resn_exch{"2HC8 4OX"}="HC82 4OX"; $resn_exch{"1HC9 4OX"}="HC91 4OX"; $resn_exch{"2HC9 4OX"}="HC92 4OX";
$resn_exch{"1HC2 4OX"}="HC21 4OX"; $resn_exch{"2HC2 4OX"}="HC22 4OX"; $resn_exch{"1HC3 4OX"}="HC31 4OX";
$resn_exch{"2HC3 4OX"}="HC32 4OX"; $resn_exch{"1HC5 4OX"}="HC51 4OX"; $resn_exch{"2HC5 4OX"}="HC52 4OX";
$resn_exch{"1HC6 4OX"}="HC61 4OX"; $resn_exch{"2HC6 4OX"}="HC62 4OX"; $resn_exch{"1HAC 4PA"}="HAC1 4PA";
$resn_exch{"2HAC 4PA"}="HAC2 4PA"; $resn_exch{"3HAC 4PA"}="HAC3 4PA"; $resn_exch{"1HAF 4PA"}="HAF1 4PA";
$resn_exch{"2HAF 4PA"}="HAF2 4PA"; $resn_exch{"1HAE 4PA"}="HAE1 4PA"; $resn_exch{"2HAE 4PA"}="HAE2 4PA";
$resn_exch{"1HAA 4PA"}="HAA1 4PA"; $resn_exch{"2HAA 4PA"}="HAA2 4PA"; $resn_exch{"3HAA 4PA"}="HAA3 4PA";
$resn_exch{"1H2  4PB"}=" H21 4PB"; $resn_exch{"2H2  4PB"}=" H22 4PB"; $resn_exch{"1H3  4PB"}=" H31 4PB";
$resn_exch{"2H3  4PB"}=" H32 4PB"; $resn_exch{"1H4  4PB"}=" H41 4PB"; $resn_exch{"2H4  4PB"}=" H42 4PB";
$resn_exch{"2HOP 4PC"}="HOP2 4PC"; $resn_exch{"1H5* 4PC"}=" H5' 4PC"; $resn_exch{"2H5* 4PC"}="H5'' 4PC";
$resn_exch{" H4* 4PC"}=" H4' 4PC"; $resn_exch{" H3* 4PC"}=" H3' 4PC"; $resn_exch{"H3T  4PC"}="HO3' 4PC";
$resn_exch{"1H2* 4PC"}=" H2' 4PC"; $resn_exch{"2H2* 4PC"}="H2'' 4PC"; $resn_exch{" H1* 4PC"}=" H1' 4PC";
$resn_exch{"1H16 4PC"}="H161 4PC"; $resn_exch{"2H16 4PC"}="H162 4PC"; $resn_exch{"3H16 4PC"}="H163 4PC";
$resn_exch{"3HOP 4PC"}="HOP3 4PC"; $resn_exch{"2HOP 4PD"}="HOP2 4PD"; $resn_exch{"1H5* 4PD"}=" H5' 4PD";
$resn_exch{"2H5* 4PD"}="H5'' 4PD"; $resn_exch{" H4* 4PD"}=" H4' 4PD"; $resn_exch{" H3* 4PD"}=" H3' 4PD";
$resn_exch{"H3T  4PD"}="HO3' 4PD"; $resn_exch{"1H2* 4PD"}=" H2' 4PD"; $resn_exch{"2H2* 4PD"}="H2'' 4PD";
$resn_exch{" H1* 4PD"}=" H1' 4PD"; $resn_exch{"1H16 4PD"}="H161 4PD"; $resn_exch{"2H16 4PD"}="H162 4PD";
$resn_exch{"3H16 4PD"}="H163 4PD"; $resn_exch{"3HOP 4PD"}="HOP3 4PD"; $resn_exch{"2HOP 4PE"}="HOP2 4PE";
$resn_exch{"1H5* 4PE"}=" H5' 4PE"; $resn_exch{"2H5* 4PE"}="H5'' 4PE"; $resn_exch{" H4* 4PE"}=" H4' 4PE";
$resn_exch{" H3* 4PE"}=" H3' 4PE"; $resn_exch{"H3T  4PE"}="HO3' 4PE"; $resn_exch{"1H2* 4PE"}=" H2' 4PE";
$resn_exch{"2H2* 4PE"}="H2'' 4PE"; $resn_exch{" H1* 4PE"}=" H1' 4PE"; $resn_exch{"1H16 4PE"}="H161 4PE";
$resn_exch{"2H16 4PE"}="H162 4PE"; $resn_exch{"3H16 4PE"}="H163 4PE"; $resn_exch{"3HOP 4PE"}="HOP3 4PE";
$resn_exch{"1H8  4PG"}=" H81 4PG"; $resn_exch{"2H8  4PG"}=" H82 4PG"; $resn_exch{"1HN2 4PG"}="HN21 4PG";
$resn_exch{"2HN2 4PG"}="HN22 4PG"; $resn_exch{"1H7  4PI"}=" H71 4PI"; $resn_exch{"2H7  4PI"}=" H72 4PI";
$resn_exch{"1H8  4PI"}=" H81 4PI"; $resn_exch{"2H8  4PI"}=" H82 4PI"; $resn_exch{"1H10 4PI"}="H101 4PI";
$resn_exch{"2H10 4PI"}="H102 4PI"; $resn_exch{"1H11 4PI"}="H111 4PI"; $resn_exch{"2H11 4PI"}="H112 4PI";
$resn_exch{"1H27 4PI"}="H271 4PI"; $resn_exch{"2H27 4PI"}="H272 4PI"; $resn_exch{"3H27 4PI"}="H273 4PI";
$resn_exch{"1H37 4PI"}="H371 4PI"; $resn_exch{"2H37 4PI"}="H372 4PI"; $resn_exch{"1H8  4PN"}=" H81 4PN";
$resn_exch{"2H8  4PN"}=" H82 4PN"; $resn_exch{"1H9  4PN"}=" H91 4PN"; $resn_exch{"2H9  4PN"}=" H92 4PN";
$resn_exch{"1H10 4PN"}="H101 4PN"; $resn_exch{"2H10 4PN"}="H102 4PN"; $resn_exch{"1H11 4PN"}="H111 4PN";
$resn_exch{"2H11 4PN"}="H112 4PN"; $resn_exch{"1H12 4PN"}="H121 4PN"; $resn_exch{"2H12 4PN"}="H122 4PN";
$resn_exch{"1H5  4PN"}=" H51 4PN"; $resn_exch{"2H5  4PN"}=" H52 4PN"; $resn_exch{"1H6  4PN"}=" H61 4PN";
$resn_exch{"2H6  4PN"}=" H62 4PN"; $resn_exch{"1H2  4PN"}=" H21 4PN"; $resn_exch{"2H2  4PN"}=" H22 4PN";
$resn_exch{"1H3  4PN"}=" H31 4PN"; $resn_exch{"2H3  4PN"}=" H32 4PN"; $resn_exch{"1H2  4PP"}=" H21 4PP";
$resn_exch{"2H2  4PP"}=" H22 4PP"; $resn_exch{"1H3  4PP"}=" H31 4PP"; $resn_exch{"2H3  4PP"}=" H32 4PP";
$resn_exch{"1H4  4PP"}=" H41 4PP"; $resn_exch{"2H4  4PP"}=" H42 4PP"; $resn_exch{"1HN2 4PP"}="HN21 4PP";
$resn_exch{"2HN2 4PP"}="HN22 4PP"; $resn_exch{"1H1  4PR"}=" H11 4PR"; $resn_exch{"2H1  4PR"}=" H12 4PR";
$resn_exch{"3H1  4PR"}=" H13 4PR"; $resn_exch{"1H2  4PR"}="H21A 4PR"; $resn_exch{"2H2  4PR"}=" H22 4PR";
$resn_exch{"1H3  4PR"}=" H31 4PR"; $resn_exch{"2H3  4PR"}=" H32 4PR"; $resn_exch{"1H4  4PR"}=" H41 4PR";
$resn_exch{"2H4  4PR"}=" H42 4PR"; $resn_exch{"1H8  4PR"}=" H81 4PR"; $resn_exch{"2H8  4PR"}=" H82 4PR";
$resn_exch{"1H9  4PR"}=" H91 4PR"; $resn_exch{"2H9  4PR"}=" H92 4PR"; $resn_exch{"1H11 4PR"}="H111 4PR";
$resn_exch{"2H11 4PR"}="H112 4PR"; $resn_exch{"3H11 4PR"}="H113 4PR"; $resn_exch{"1H12 4PR"}="H121 4PR";
$resn_exch{"2H12 4PR"}="H122 4PR"; $resn_exch{"3H12 4PR"}="H123 4PR"; $resn_exch{"1H16 4PR"}="H161 4PR";
$resn_exch{"2H16 4PR"}="H162 4PR"; $resn_exch{"3H16 4PR"}="H163 4PR"; $resn_exch{"1H18 4PR"}="H181 4PR";
$resn_exch{"2H18 4PR"}="H182 4PR"; $resn_exch{"3H18 4PR"}="H183 4PR"; $resn_exch{"1H22 4PR"}="H221 4PR";
$resn_exch{"2H22 4PR"}="H222 4PR"; $resn_exch{"3H22 4PR"}="H223 4PR"; $resn_exch{"1H0E 4PT"}="H0E1 4PT";
$resn_exch{"2H0E 4PT"}="H0E2 4PT"; $resn_exch{"1H0B 4PT"}="H0B1 4PT"; $resn_exch{"2H0B 4PT"}="H0B2 4PT";
$resn_exch{"1H0N 4PT"}="H0N1 4PT"; $resn_exch{"2H0N 4PT"}="H0N2 4PT"; $resn_exch{"1H0P 4PT"}="H0P1 4PT";
$resn_exch{"2H0P 4PT"}="H0P2 4PT"; $resn_exch{"1H0Q 4PT"}="H0Q1 4PT"; $resn_exch{"2H0Q 4PT"}="H0Q2 4PT";
$resn_exch{"3H0Q 4PT"}="H0Q3 4PT"; $resn_exch{"1H0I 4PT"}="H0I1 4PT"; $resn_exch{"2H0I 4PT"}="H0I2 4PT";
$resn_exch{"1H0J 4PT"}="H0J1 4PT"; $resn_exch{"2H0J 4PT"}="H0J2 4PT"; $resn_exch{"1H0K 4PT"}="H0K1 4PT";
$resn_exch{"2H0K 4PT"}="H0K2 4PT"; $resn_exch{"3H0K 4PT"}="H0K3 4PT"; $resn_exch{"1H6  4PY"}=" H61 4PY";
$resn_exch{"2H6  4PY"}=" H62 4PY"; $resn_exch{"1H11 4PY"}="H111 4PY"; $resn_exch{"2H11 4PY"}="H112 4PY";
$resn_exch{"1H12 4PY"}="H121 4PY"; $resn_exch{"2H12 4PY"}="H122 4PY"; $resn_exch{"1H13 4PY"}="H131 4PY";
$resn_exch{"2H13 4PY"}="H132 4PY"; $resn_exch{"1H25 4PY"}="H251 4PY"; $resn_exch{"2H25 4PY"}="H252 4PY";
$resn_exch{"1H7  4QB"}=" H71 4QB"; $resn_exch{"2H7  4QB"}=" H72 4QB"; $resn_exch{"1H9  4QB"}=" H91 4QB";
$resn_exch{"2H9  4QB"}=" H92 4QB"; $resn_exch{"1H31 4QB"}="H311 4QB"; $resn_exch{"2H31 4QB"}="H312 4QB";
$resn_exch{"3H31 4QB"}="H313 4QB"; $resn_exch{"1H11 4QB"}="H111 4QB"; $resn_exch{"2H11 4QB"}="H112 4QB";
$resn_exch{"1H12 4QB"}="H121 4QB"; $resn_exch{"2H12 4QB"}="H122 4QB"; $resn_exch{"1H13 4QB"}="H131 4QB";
$resn_exch{"2H13 4QB"}="H132 4QB"; $resn_exch{"1H14 4QB"}="H141 4QB"; $resn_exch{"2H14 4QB"}="H142 4QB";
$resn_exch{"1H11 4QC"}="H111 4QC"; $resn_exch{"2H11 4QC"}="H112 4QC"; $resn_exch{"1H12 4QC"}="H121 4QC";
$resn_exch{"2H12 4QC"}="H122 4QC"; $resn_exch{"1H20 4QC"}="H201 4QC"; $resn_exch{"2H20 4QC"}="H202 4QC";
$resn_exch{"3H20 4QC"}="H203 4QC"; $resn_exch{"1H24 4QC"}="H241 4QC"; $resn_exch{"2H24 4QC"}="H242 4QC";
$resn_exch{"1H23 4QC"}="H231 4QC"; $resn_exch{"2H23 4QC"}="H232 4QC"; $resn_exch{"3H23 4QC"}="H233 4QC";
$resn_exch{"1HN1 4QC"}="HN11 4QC"; $resn_exch{"2HN1 4QC"}="HN12 4QC"; $resn_exch{"1H18 4RR"}="H181 4RR";
$resn_exch{"2H18 4RR"}="H182 4RR"; $resn_exch{"3H18 4RR"}="H183 4RR"; $resn_exch{"1H15 4RR"}="H151 4RR";
$resn_exch{"2H15 4RR"}="H152 4RR"; $resn_exch{"1H14 4RR"}="H141 4RR"; $resn_exch{"2H14 4RR"}="H142 4RR";
$resn_exch{"1H13 4RR"}="H131 4RR"; $resn_exch{"2H13 4RR"}="H132 4RR"; $resn_exch{"1H12 4RR"}="H121 4RR";
$resn_exch{"2H12 4RR"}="H122 4RR"; $resn_exch{"1H5  4RR"}=" H51 4RR"; $resn_exch{"2H5  4RR"}=" H52 4RR";
$resn_exch{"1H1  4RR"}="H11A 4RR"; $resn_exch{"2H1  4RR"}=" H12 4RR"; $resn_exch{"1H5* 4SC"}=" H5' 4SC";
$resn_exch{"2H5* 4SC"}="H5'' 4SC"; $resn_exch{" H4* 4SC"}=" H4' 4SC"; $resn_exch{" H3* 4SC"}=" H3' 4SC";
$resn_exch{"*HO3 4SC"}="HO3' 4SC"; $resn_exch{"1H2* 4SC"}=" H2' 4SC"; $resn_exch{"2H2* 4SC"}="H2'' 4SC";
$resn_exch{" H1* 4SC"}=" H1' 4SC"; $resn_exch{"1HN4 4SC"}="HN41 4SC"; $resn_exch{"2HN4 4SC"}="HN42 4SC";
$resn_exch{"1H6  4SC"}=" H61 4SC"; $resn_exch{"2H6  4SC"}=" H62 4SC"; $resn_exch{"1H5M 4SC"}=" H71 4SC";
$resn_exch{"2H5M 4SC"}=" H72 4SC"; $resn_exch{"3H5M 4SC"}=" H73 4SC"; $resn_exch{"1H10 4SP"}="H101 4SP";
$resn_exch{"2H10 4SP"}="H102 4SP"; $resn_exch{"1H12 4SP"}="H121 4SP"; $resn_exch{"2H12 4SP"}="H122 4SP";
$resn_exch{"1H13 4SP"}="H131 4SP"; $resn_exch{"2H13 4SP"}="H132 4SP"; $resn_exch{"1H14 4SP"}="H141 4SP";
$resn_exch{"2H14 4SP"}="H142 4SP"; $resn_exch{"1H15 4SP"}="H151 4SP"; $resn_exch{"2H15 4SP"}="H152 4SP";
$resn_exch{"1H16 4SP"}="H161 4SP"; $resn_exch{"2H16 4SP"}="H162 4SP"; $resn_exch{"1H26 4SP"}="H261 4SP";
$resn_exch{"2H26 4SP"}="H262 4SP"; $resn_exch{"1H16 4SR"}="H161 4SR"; $resn_exch{"2H16 4SR"}="H162 4SR";
$resn_exch{"1H12 4SR"}="H121 4SR"; $resn_exch{"2H12 4SR"}="H122 4SR"; $resn_exch{"1H20 4SR"}="H201 4SR";
$resn_exch{"2H20 4SR"}="H202 4SR"; $resn_exch{"3H20 4SR"}="H203 4SR"; $resn_exch{"1H5  4SR"}=" H51 4SR";
$resn_exch{"2H5  4SR"}=" H52 4SR"; $resn_exch{"1H6  4SR"}=" H61 4SR"; $resn_exch{"2H6  4SR"}=" H62 4SR";
$resn_exch{"1H7  4SR"}=" H71 4SR"; $resn_exch{"2H7  4SR"}=" H72 4SR"; $resn_exch{"1H8  4SR"}=" H81 4SR";
$resn_exch{"2H8  4SR"}=" H82 4SR"; $resn_exch{"1HAB 4ST"}="HAB1 4ST"; $resn_exch{"2HAB 4ST"}="HAB2 4ST";
$resn_exch{"3HAB 4ST"}="HAB3 4ST"; $resn_exch{"1HAA 4ST"}="HAA1 4ST"; $resn_exch{"2HAA 4ST"}="HAA2 4ST";
$resn_exch{"3HAA 4ST"}="HAA3 4ST"; $resn_exch{"1HAN 4ST"}="HAN1 4ST"; $resn_exch{"2HAN 4ST"}="HAN2 4ST";
$resn_exch{"1HAC 4ST"}="HAC1 4ST"; $resn_exch{"2HAC 4ST"}="HAC2 4ST"; $resn_exch{"3HAC 4ST"}="HAC3 4ST";
$resn_exch{"1HAL 4ST"}="HAL1 4ST"; $resn_exch{"2HAL 4ST"}="HAL2 4ST"; $resn_exch{"1HAJ 4ST"}="HAJ1 4ST";
$resn_exch{"2HAJ 4ST"}="HAJ2 4ST"; $resn_exch{"1HAI 4ST"}="HAI1 4ST"; $resn_exch{"2HAI 4ST"}="HAI2 4ST";
$resn_exch{"1HAK 4ST"}="HAK1 4ST"; $resn_exch{"2HAK 4ST"}="HAK2 4ST"; $resn_exch{"1HAM 4ST"}="HAM1 4ST";
$resn_exch{"2HAM 4ST"}="HAM2 4ST"; $resn_exch{" H1* 4SU"}=" H1' 4SU"; $resn_exch{" H2* 4SU"}=" H2' 4SU";
$resn_exch{"*HO2 4SU"}="HO2' 4SU"; $resn_exch{" H3* 4SU"}=" H3' 4SU"; $resn_exch{" H4* 4SU"}=" H4' 4SU";
$resn_exch{"*HO3 4SU"}="HO3' 4SU"; $resn_exch{"1H5* 4SU"}=" H5' 4SU"; $resn_exch{"2H5* 4SU"}="H5'' 4SU";
$resn_exch{"2HOP 4SU"}="HOP2 4SU"; $resn_exch{"3HOP 4SU"}="HOP3 4SU"; $resn_exch{"1H5M 4TA"}="H5M1 4TA";
$resn_exch{"2H5M 4TA"}="H5M2 4TA"; $resn_exch{"3H5M 4TA"}="H5M3 4TA"; $resn_exch{" H1* 4TA"}=" H1B 4TA";
$resn_exch{"1H2* 4TA"}="H2'1 4TA"; $resn_exch{"2H2* 4TA"}="H2'2 4TA"; $resn_exch{" H3* 4TA"}=" H3B 4TA";
$resn_exch{" H4* 4TA"}=" H4B 4TA"; $resn_exch{"1H5* 4TA"}="H5'1 4TA"; $resn_exch{"2H5* 4TA"}="H5'2 4TA";
$resn_exch{"1H5' 4TA"}=" H5B 4TA"; $resn_exch{"2H5' 4TA"}=" H5D 4TA"; $resn_exch{"1H6A 4TA"}="H6A1 4TA";
$resn_exch{"2H6A 4TA"}="H6A2 4TA"; $resn_exch{"1H6  4TB"}=" H61 4TB"; $resn_exch{"2H6  4TB"}=" H62 4TB";
$resn_exch{"1H7  4TB"}=" H71 4TB"; $resn_exch{"2H7  4TB"}=" H72 4TB"; $resn_exch{"1H8  4TB"}=" H81 4TB";
$resn_exch{"2H8  4TB"}=" H82 4TB"; $resn_exch{"1H3B 4TC"}="H3B1 4TC"; $resn_exch{"2H3B 4TC"}="H3B2 4TC";
$resn_exch{"1H5U 4TC"}="H5U1 4TC"; $resn_exch{"2H5U 4TC"}="H5U2 4TC"; $resn_exch{"1H5* 4TC"}="H5'1 4TC";
$resn_exch{"2H5* 4TC"}="H5'2 4TC"; $resn_exch{" H4* 4TC"}=" H4' 4TC"; $resn_exch{" H3* 4TC"}=" H3' 4TC";
$resn_exch{" H2* 4TC"}=" H2' 4TC"; $resn_exch{" H1* 4TC"}=" H1' 4TC"; $resn_exch{"1H6N 4TC"}="H6N1 4TC";
$resn_exch{"2H6N 4TC"}="H6N2 4TC"; $resn_exch{"1HAP 4TN"}="HAP1 4TN"; $resn_exch{"2HAP 4TN"}="HAP2 4TN";
$resn_exch{"1HAA 4TN"}="HAA1 4TN"; $resn_exch{"2HAA 4TN"}="HAA2 4TN"; $resn_exch{"3HAA 4TN"}="HAA3 4TN";
$resn_exch{"1HAB 4TN"}="HAB1 4TN"; $resn_exch{"2HAB 4TN"}="HAB2 4TN"; $resn_exch{"3HAB 4TN"}="HAB3 4TN";
$resn_exch{"1HAC 4TN"}="HAC1 4TN"; $resn_exch{"2HAC 4TN"}="HAC2 4TN"; $resn_exch{"3HAC 4TN"}="HAC3 4TN";
$resn_exch{"1HAQ 4TN"}="HAQ1 4TN"; $resn_exch{"2HAQ 4TN"}="HAQ2 4TN"; $resn_exch{"1HAR 4TN"}="HAR1 4TN";
$resn_exch{"2HAR 4TN"}="HAR2 4TN"; $resn_exch{"1HAD 4TN"}="HAD1 4TN"; $resn_exch{"2HAD 4TN"}="HAD2 4TN";
$resn_exch{"3HAD 4TN"}="HAD3 4TN"; $resn_exch{"1HAE 4TN"}="HAE1 4TN"; $resn_exch{"2HAE 4TN"}="HAE2 4TN";
$resn_exch{"3HAE 4TN"}="HAE3 4TN"; $resn_exch{"2HN  4TP"}=" HN2 4TP"; $resn_exch{"1HG2 4TP"}="HG21 4TP";
$resn_exch{"2HG2 4TP"}="HG22 4TP"; $resn_exch{"2HOP 4TP"}="HOP2 4TP"; $resn_exch{"3HOP 4TP"}="HOP3 4TP";
$resn_exch{"1HN1 4TR"}="HN11 4TR"; $resn_exch{"2HN1 4TR"}="HN12 4TR"; $resn_exch{"1H7  4TR"}=" H71 4TR";
$resn_exch{"2H7  4TR"}=" H72 4TR"; $resn_exch{"1HN1 4TZ"}="HN11 4TZ"; $resn_exch{"2HN1 4TZ"}="HN12 4TZ";
$resn_exch{"1H10 4TZ"}="H101 4TZ"; $resn_exch{"2H10 4TZ"}="H102 4TZ"; $resn_exch{"1H49 4UN"}="H491 4UN";
$resn_exch{"2H49 4UN"}="H492 4UN"; $resn_exch{"1H28 4UN"}="H281 4UN"; $resn_exch{"2H28 4UN"}="H282 4UN";
$resn_exch{"3H28 4UN"}="H283 4UN"; $resn_exch{"1H24 4UN"}="H241 4UN"; $resn_exch{"2H24 4UN"}="H242 4UN";
$resn_exch{"3H24 4UN"}="H243 4UN"; $resn_exch{"1H19 4UN"}="H191 4UN"; $resn_exch{"2H19 4UN"}="H192 4UN";
$resn_exch{"1H22 501"}="H221 501"; $resn_exch{"2H22 501"}="H222 501"; $resn_exch{"1H23 501"}="H231 501";
$resn_exch{"2H23 501"}="H232 501"; $resn_exch{"1H26 501"}="H261 501"; $resn_exch{"2H26 501"}="H262 501";
$resn_exch{"1H29 501"}="H291 501"; $resn_exch{"2H29 501"}="H292 501"; $resn_exch{"3H29 501"}="H293 501";
$resn_exch{"1H37 501"}="H371 501"; $resn_exch{"2H37 501"}="H372 501"; $resn_exch{"3H37 501"}="H373 501";
$resn_exch{"1H42 501"}="H421 501"; $resn_exch{"2H42 501"}="H422 501"; $resn_exch{"3H42 501"}="H423 501";
$resn_exch{"1H1C 505"}="H1C1 505"; $resn_exch{"2H1C 505"}="H1C2 505"; $resn_exch{"3H1C 505"}="H1C3 505";
$resn_exch{"1H3N 505"}="H3N1 505"; $resn_exch{"2H3N 505"}="H3N2 505"; $resn_exch{"1H4C 505"}="H4C1 505";
$resn_exch{"2H4C 505"}="H4C2 505"; $resn_exch{"1H8C 505"}="H8C1 505"; $resn_exch{"2H8C 505"}="H8C2 505";
$resn_exch{"3H8C 505"}="H8C3 505"; $resn_exch{"1H13 505"}="H131 505"; $resn_exch{"2H13 505"}="H132 505";
$resn_exch{"3H13 505"}="H133 505"; $resn_exch{"1H12 509"}="H121 509"; $resn_exch{"2H12 509"}="H122 509";
$resn_exch{"1H9  512"}=" H91 512"; $resn_exch{"2H9  512"}=" H92 512"; $resn_exch{"1H18 514"}="H181 514";
$resn_exch{"2H18 514"}="H182 514"; $resn_exch{"1H20 514"}="H201 514"; $resn_exch{"2H20 514"}="H202 514";
$resn_exch{"3H20 514"}="H203 514"; $resn_exch{"1H3  515"}=" H31 515"; $resn_exch{"1H4  515"}=" H41 515";
$resn_exch{"1H9  515"}=" H91 515"; $resn_exch{"1H12 515"}="H121 515"; $resn_exch{"2H12 515"}="H122 515";
$resn_exch{"1H13 515"}="H131 515"; $resn_exch{"2H13 515"}="H132 515"; $resn_exch{"1H14 515"}="H141 515";
$resn_exch{"2H14 515"}="H142 515"; $resn_exch{"1H15 515"}="H151 515"; $resn_exch{"2H15 515"}="H152 515";
$resn_exch{"1H23 515"}="H231 515"; $resn_exch{"2H23 515"}="H232 515"; $resn_exch{"3H23 515"}="H233 515";
$resn_exch{"1H7  515"}=" H71 515"; $resn_exch{"2H7  515"}=" H72 515"; $resn_exch{"1H27 515"}="H271 515";
$resn_exch{"1H28 515"}="H281 515"; $resn_exch{"1H29 515"}="H291 515"; $resn_exch{"1H30 515"}="H301 515";
$resn_exch{"1H33 515"}="H331 515"; $resn_exch{"1H36 515"}="H361 515"; $resn_exch{"1H16 515"}="H161 515";
$resn_exch{"1H17 515"}="H171 515"; $resn_exch{"1H18 515"}="H181 515"; $resn_exch{"1H19 515"}="H191 515";
$resn_exch{"1H20 515"}="H201 515"; $resn_exch{"2H20 515"}="H202 515"; $resn_exch{"1H2  515"}=" H21 515";
$resn_exch{"1H41 515"}="H411 515"; $resn_exch{"1H42 515"}="H421 515"; $resn_exch{"1H43 515"}="H431 515";
$resn_exch{"1H44 515"}="H441 515"; $resn_exch{"1H20 517"}="H201 517"; $resn_exch{"2H20 517"}="H202 517";
$resn_exch{"3H20 517"}="H203 517"; $resn_exch{"1H16 517"}="H161 517"; $resn_exch{"2H16 517"}="H162 517";
$resn_exch{"1H11 521"}="H111 521"; $resn_exch{"2H11 521"}="H112 521"; $resn_exch{"1H5A 523"}="H5A1 523";
$resn_exch{"2H5A 523"}="H5A2 523"; $resn_exch{"3H5A 523"}="H5A3 523"; $resn_exch{"1HN4 523"}="HN41 523";
$resn_exch{"2HN4 523"}="HN42 523"; $resn_exch{" H1* 523"}=" H1' 523"; $resn_exch{"1H2* 523"}="H2'1 523";
$resn_exch{"2H2* 523"}="H2'2 523"; $resn_exch{" H3* 523"}=" H3' 523"; $resn_exch{" H4* 523"}=" H4' 523";
$resn_exch{"HO3* 523"}="HO3' 523"; $resn_exch{"1H5* 523"}="H5'1 523"; $resn_exch{"2H5* 523"}="H5'2 523";
$resn_exch{"1H19 524"}="H191 524"; $resn_exch{"2H19 524"}="H192 524"; $resn_exch{"1H34 524"}="H341 524";
$resn_exch{"2H34 524"}="H342 524"; $resn_exch{"1H33 524"}="H331 524"; $resn_exch{"2H33 524"}="H332 524";
$resn_exch{"1H31 524"}="H311 524"; $resn_exch{"2H31 524"}="H312 524"; $resn_exch{"1H23 524"}="H231 524";
$resn_exch{"2H23 524"}="H232 524"; $resn_exch{"3H23 524"}="H233 524"; $resn_exch{"1H27 524"}="H271 524";
$resn_exch{"2H27 524"}="H272 524"; $resn_exch{"3H27 524"}="H273 524"; $resn_exch{"1H45 524"}="H451 524";
$resn_exch{"2H45 524"}="H452 524"; $resn_exch{"1H44 524"}="H441 524"; $resn_exch{"2H44 524"}="H442 524";
$resn_exch{"1H48 524"}="H481 524"; $resn_exch{"2H48 524"}="H482 524"; $resn_exch{"1H47 524"}="H471 524";
$resn_exch{"2H47 524"}="H472 524"; $resn_exch{"1H21 527"}="H211 527"; $resn_exch{"2H21 527"}="H212 527";
$resn_exch{"1H31 527"}="H311 527"; $resn_exch{"2H31 527"}="H312 527"; $resn_exch{"1H32 527"}="H321 527";
$resn_exch{"2H32 527"}="H322 527"; $resn_exch{"1H34 527"}="H341 527"; $resn_exch{"2H34 527"}="H342 527";
$resn_exch{"1H35 527"}="H351 527"; $resn_exch{"2H35 527"}="H352 527"; $resn_exch{"1H49 527"}="H491 527";
$resn_exch{"2H49 527"}="H492 527"; $resn_exch{"1H35 528"}="H351 528"; $resn_exch{"2H35 528"}="H352 528";
$resn_exch{"1H34 528"}="H341 528"; $resn_exch{"2H34 528"}="H342 528"; $resn_exch{"1H33 528"}="H331 528";
$resn_exch{"2H33 528"}="H332 528"; $resn_exch{"1H30 528"}="H301 528"; $resn_exch{"2H30 528"}="H302 528";
$resn_exch{"3H30 528"}="H303 528"; $resn_exch{"1H29 528"}="H291 528"; $resn_exch{"2H29 528"}="H292 528";
$resn_exch{"3H29 528"}="H293 528"; $resn_exch{"1H15 528"}="H151 528"; $resn_exch{"2H15 528"}="H152 528";
$resn_exch{"1H9  528"}=" H91 528"; $resn_exch{"2H9  528"}=" H92 528"; $resn_exch{"1H10 528"}="H101 528";
$resn_exch{"2H10 528"}="H102 528"; $resn_exch{"1H12 529"}="H121 529"; $resn_exch{"2H12 529"}="H122 529";
$resn_exch{"1H13 529"}="H131 529"; $resn_exch{"2H13 529"}="H132 529"; $resn_exch{"1H25 529"}="H251 529";
$resn_exch{"2H25 529"}="H252 529"; $resn_exch{"3H25 529"}="H253 529"; $resn_exch{"1HN  52A"}=" HN1 52A";
$resn_exch{"2HN  52A"}=" HN2 52A"; $resn_exch{"1HB1 52A"}="HB11 52A"; $resn_exch{"2HB1 52A"}="HB12 52A";
$resn_exch{"1HB2 52A"}="HB21 52A"; $resn_exch{"2HB2 52A"}="HB22 52A"; $resn_exch{"1H4  541"}=" H41 541";
$resn_exch{"2H4  541"}=" H42 541"; $resn_exch{"3H4  541"}=" H43 541"; $resn_exch{"1H8  541"}=" H81 541";
$resn_exch{"2H8  541"}=" H82 541"; $resn_exch{"1H15 541"}="H151 541"; $resn_exch{"2H15 541"}="H152 541";
$resn_exch{"1H17 541"}="H171 541"; $resn_exch{"2H17 541"}="H172 541"; $resn_exch{"3H17 541"}="H173 541";
$resn_exch{"1H19 541"}="H191 541"; $resn_exch{"2H19 541"}="H192 541"; $resn_exch{"1H25 541"}="H251 541";
$resn_exch{"2H25 541"}="H252 541"; $resn_exch{"3H25 541"}="H253 541"; $resn_exch{"1HO1 543"}="HO11 543";
$resn_exch{"2HO1 543"}="HO12 543"; $resn_exch{"1HO2 543"}="HO21 543"; $resn_exch{"2HO2 543"}="HO22 543";
$resn_exch{"1HO3 543"}="HO31 543"; $resn_exch{"2HO3 543"}="HO32 543"; $resn_exch{"1HO4 543"}="HO41 543";
$resn_exch{"2HO4 543"}="HO42 543"; $resn_exch{"1HO5 543"}="HO51 543"; $resn_exch{"2HO5 543"}="HO52 543";
$resn_exch{"1HO6 543"}="HO61 543"; $resn_exch{"2HO6 543"}="HO62 543"; $resn_exch{"1H1  543"}=" H11 543";
$resn_exch{"2H1  543"}=" H12 543"; $resn_exch{"1H2  543"}=" H21 543"; $resn_exch{"2H2  543"}=" H22 543";
$resn_exch{"3H2  543"}=" H23 543"; $resn_exch{"1HA  544"}=" HA1 544"; $resn_exch{"1HB  544"}=" HB1 544";
$resn_exch{"2HB  544"}=" HB2 544"; $resn_exch{"1HD1 544"}="HD11 544"; $resn_exch{"1HD2 544"}="HD21 544";
$resn_exch{"1HE1 544"}="HE11 544"; $resn_exch{"1HE2 544"}="HE21 544"; $resn_exch{"1H1F 544"}="H1F1 544";
$resn_exch{"1H1B 544"}="H1B1 544"; $resn_exch{"2H1B 544"}="H1B2 544"; $resn_exch{"3H1B 544"}="H1B3 544";
$resn_exch{"1H1I 544"}="H1I1 544"; $resn_exch{"1H1J 544"}="H1J1 544"; $resn_exch{"1H1K 544"}="H1K1 544";
$resn_exch{"1H1L 544"}="H1L1 544"; $resn_exch{"1H1M 544"}="H1M1 544"; $resn_exch{"1H3E 544"}="H3E1 544";
$resn_exch{"2H3E 544"}="H3E2 544"; $resn_exch{"3H3E 544"}="H3E3 544"; $resn_exch{"1H3N 544"}="H3N1 544";
$resn_exch{"1H3M 544"}="H3M1 544"; $resn_exch{"1H3L 544"}="H3L1 544"; $resn_exch{"1H3K 544"}="H3K1 544";
$resn_exch{"1H3J 544"}="H3J1 544"; $resn_exch{"1H3A 544"}="H3A1 544"; $resn_exch{"2H3A 544"}="H3A2 544";
$resn_exch{"1H3B 544"}="H3B1 544"; $resn_exch{"2H3B 544"}="H3B2 544"; $resn_exch{"1HO1 544"}="HO11 544";
$resn_exch{"1H14 545"}="H141 545"; $resn_exch{"2H14 545"}="H142 545"; $resn_exch{"1H24 545"}="H241 545";
$resn_exch{"2H24 545"}="H242 545"; $resn_exch{"1H25 545"}="H251 545"; $resn_exch{"2H25 545"}="H252 545";
$resn_exch{"1H26 545"}="H261 545"; $resn_exch{"2H26 545"}="H262 545"; $resn_exch{"1H27 545"}="H271 545";
$resn_exch{"2H27 545"}="H272 545"; $resn_exch{"3H27 545"}="H273 545"; $resn_exch{"1H16 545"}="H161 545";
$resn_exch{"2H16 545"}="H162 545"; $resn_exch{"1H4  545"}=" H41 545"; $resn_exch{"2H4  545"}=" H42 545";
$resn_exch{"1H30 545"}="H301 545"; $resn_exch{"2H30 545"}="H302 545"; $resn_exch{"1H35 547"}="H351 547";
$resn_exch{"2H35 547"}="H352 547"; $resn_exch{"1H36 547"}="H361 547"; $resn_exch{"2H36 547"}="H362 547";
$resn_exch{"1H42 547"}="H421 547"; $resn_exch{"2H42 547"}="H422 547"; $resn_exch{"1H43 547"}="H431 547";
$resn_exch{"2H43 547"}="H432 547"; $resn_exch{"1H45 547"}="H451 547"; $resn_exch{"2H45 547"}="H452 547";
$resn_exch{"1H46 547"}="H461 547"; $resn_exch{"2H46 547"}="H462 547"; $resn_exch{"1H10 561"}="H101 561";
$resn_exch{"2H10 561"}="H102 561"; $resn_exch{"1H31 561"}="H311 561"; $resn_exch{"2H31 561"}="H312 561";
$resn_exch{"1H32 561"}="H321 561"; $resn_exch{"2H32 561"}="H322 561"; $resn_exch{"1H33 561"}="H331 561";
$resn_exch{"2H33 561"}="H332 561"; $resn_exch{"1H17 564"}="H171 564"; $resn_exch{"2H17 564"}="H172 564";
$resn_exch{"1H18 564"}="H181 564"; $resn_exch{"2H18 564"}="H182 564"; $resn_exch{"1H23 564"}="H231 564";
$resn_exch{"2H23 564"}="H232 564"; $resn_exch{"3H23 564"}="H233 564"; $resn_exch{"1H24 564"}="H241 564";
$resn_exch{"2H24 564"}="H242 564"; $resn_exch{"3H24 564"}="H243 564"; $resn_exch{"1H25 564"}="H251 564";
$resn_exch{"2H25 564"}="H252 564"; $resn_exch{"3H25 564"}="H253 564"; $resn_exch{"1H26 564"}="H261 564";
$resn_exch{"2H26 564"}="H262 564"; $resn_exch{"3H26 564"}="H263 564"; $resn_exch{"1H11 565"}="H111 565";
$resn_exch{"2H11 565"}="H112 565"; $resn_exch{"1H15 565"}="H151 565"; $resn_exch{"2H15 565"}="H152 565";
$resn_exch{"1H20 565"}="H201 565"; $resn_exch{"2H20 565"}="H202 565"; $resn_exch{"1H25 565"}="H251 565";
$resn_exch{"2H25 565"}="H252 565"; $resn_exch{"1H26 565"}="H261 565"; $resn_exch{"2H26 565"}="H262 565";
$resn_exch{"1H27 565"}="H271 565"; $resn_exch{"2H27 565"}="H272 565"; $resn_exch{"1H37 565"}="H371 565";
$resn_exch{"2H37 565"}="H372 565"; $resn_exch{"3H37 565"}="H373 565"; $resn_exch{"1H1  566"}=" H11 566";
$resn_exch{"2H1  566"}=" H12 566"; $resn_exch{"1H2  566"}=" H21 566"; $resn_exch{"2H2  566"}=" H22 566";
$resn_exch{"1H4  566"}=" H41 566"; $resn_exch{"2H4  566"}=" H42 566"; $resn_exch{"1H5  566"}=" H51 566";
$resn_exch{"2H5  566"}=" H52 566"; $resn_exch{"1H6  566"}=" H61 566"; $resn_exch{"2H6  566"}=" H62 566";
$resn_exch{"1H16 566"}="H161 566"; $resn_exch{"2H16 566"}="H162 566"; $resn_exch{"1H18 566"}="H181 566";
$resn_exch{"2H18 566"}="H182 566"; $resn_exch{"1H2C 568"}="H2C1 568"; $resn_exch{"2H2C 568"}="H2C2 568";
$resn_exch{"1H4C 568"}="H4C1 568"; $resn_exch{"2H4C 568"}="H4C2 568"; $resn_exch{"1H6C 568"}="H6C1 568";
$resn_exch{"2H6C 568"}="H6C2 568"; $resn_exch{"1H13 568"}="H131 568"; $resn_exch{"2H13 568"}="H132 568";
$resn_exch{"1H29 568"}="H291 568"; $resn_exch{"2H29 568"}="H292 568"; $resn_exch{"1H39 568"}="H391 568";
$resn_exch{"2H39 568"}="H392 568"; $resn_exch{"1H3E 570"}="H3E1 570"; $resn_exch{"2H3E 570"}="H3E2 570";
$resn_exch{"3H3E 570"}="H3E3 570"; $resn_exch{"1H3A 570"}="H3A1 570"; $resn_exch{"2H3A 570"}="H3A2 570";
$resn_exch{"1H3B 570"}="H3B1 570"; $resn_exch{"2H3B 570"}="H3B2 570"; $resn_exch{"1H14 572"}="H141 572";
$resn_exch{"2H14 572"}="H142 572"; $resn_exch{"1H19 572"}="H191 572"; $resn_exch{"2H19 572"}="H192 572";
$resn_exch{"1H15 572"}="H151 572"; $resn_exch{"2H15 572"}="H152 572"; $resn_exch{"1H13 572"}="H131 572";
$resn_exch{"2H13 572"}="H132 572"; $resn_exch{"3H13 572"}="H133 572"; $resn_exch{"1H11 572"}="H111 572";
$resn_exch{"2H11 572"}="H112 572"; $resn_exch{"3H11 572"}="H113 572"; $resn_exch{"1H16 572"}="H161 572";
$resn_exch{"2H16 572"}="H162 572"; $resn_exch{"1H18 572"}="H181 572"; $resn_exch{"2H18 572"}="H182 572";
$resn_exch{"1H21 577"}="H211 577"; $resn_exch{"2H21 577"}="H212 577"; $resn_exch{"3H21 577"}="H213 577";
$resn_exch{"1H23 578"}="H231 578"; $resn_exch{"2H23 578"}="H232 578"; $resn_exch{"1H18 578"}="H181 578";
$resn_exch{"2H18 578"}="H182 578"; $resn_exch{"3H18 578"}="H183 578"; $resn_exch{"1H17 578"}="H171 578";
$resn_exch{"2H17 578"}="H172 578"; $resn_exch{"3H17 578"}="H173 578"; $resn_exch{"1H9  578"}=" H91 578";
$resn_exch{"2H9  578"}=" H92 578"; $resn_exch{"1H10 578"}="H101 578"; $resn_exch{"2H10 578"}="H102 578";
$resn_exch{"1H6  580"}=" H61 580"; $resn_exch{"2H6  580"}=" H62 580"; $resn_exch{"1H7  580"}=" H71 580";
$resn_exch{"2H7  580"}=" H72 580"; $resn_exch{"1H8  580"}=" H81 580"; $resn_exch{"2H8  580"}=" H82 580";
$resn_exch{"1H35 580"}="H351 580"; $resn_exch{"2H35 580"}="H352 580"; $resn_exch{"3H35 580"}="H353 580";
$resn_exch{"1H18 582"}="H181 582"; $resn_exch{"2H18 582"}="H182 582"; $resn_exch{"1H9  582"}=" H91 582";
$resn_exch{"2H9  582"}=" H92 582"; $resn_exch{"3H9  582"}=" H93 582"; $resn_exch{"1H7  582"}=" H71 582";
$resn_exch{"2H7  582"}=" H72 582"; $resn_exch{"3H7  582"}=" H73 582"; $resn_exch{"1H3  587"}="H31A 587";
$resn_exch{"2H3  587"}="H32A 587"; $resn_exch{"1H4  587"}=" H41 587"; $resn_exch{"2H4  587"}=" H42 587";
$resn_exch{"1H5  587"}=" H51 587"; $resn_exch{"2H5  587"}=" H52 587"; $resn_exch{"1H6  587"}=" H61 587";
$resn_exch{"2H6  587"}="H62A 587"; $resn_exch{"1H12 587"}="H121 587"; $resn_exch{"2H12 587"}="H122 587";
$resn_exch{"1H16 587"}="H161 587"; $resn_exch{"2H16 587"}="H162 587"; $resn_exch{"1H23 587"}="H231 587";
$resn_exch{"2H23 587"}="H232 587"; $resn_exch{"1H30 587"}="H301 587"; $resn_exch{"2H30 587"}="H302 587";
$resn_exch{"1H33 587"}="H331 587"; $resn_exch{"2H33 587"}="H332 587"; $resn_exch{"1H34 587"}="H341 587";
$resn_exch{"2H34 587"}="H342 587"; $resn_exch{"1H35 587"}="H351 587"; $resn_exch{"2H35 587"}="H352 587";
$resn_exch{"3H35 587"}="H353 587"; $resn_exch{"1H36 587"}="H361 587"; $resn_exch{"2H36 587"}="H362 587";
$resn_exch{"3H36 587"}="H363 587"; $resn_exch{"1H37 587"}="H371 587"; $resn_exch{"2H37 587"}="H372 587";
$resn_exch{"3H37 587"}="H373 587"; $resn_exch{"1H38 587"}="H381 587"; $resn_exch{"2H38 587"}="H382 587";
$resn_exch{"1H39 587"}="H391 587"; $resn_exch{"2H39 587"}="H392 587"; $resn_exch{"3H39 587"}="H393 587";
$resn_exch{"1H41 587"}="H411 587"; $resn_exch{"2H41 587"}="H412 587"; $resn_exch{"3H41 587"}="H413 587";
$resn_exch{"1H42 587"}="H421 587"; $resn_exch{"2H42 587"}="H422 587"; $resn_exch{"3H42 587"}="H423 587";
$resn_exch{"1H43 587"}="H431 587"; $resn_exch{"2H43 587"}="H432 587"; $resn_exch{"3H43 587"}="H433 587";
$resn_exch{"1H44 587"}="H441 587"; $resn_exch{"2H44 587"}="H442 587"; $resn_exch{"3H44 587"}="H443 587";
$resn_exch{"1H45 587"}="H451 587"; $resn_exch{"2H45 587"}="H452 587"; $resn_exch{"3H45 587"}="H453 587";
$resn_exch{"1H67 587"}="H671 587"; $resn_exch{"2H67 587"}="H672 587"; $resn_exch{"3H67 587"}="H673 587";
$resn_exch{"0HO1 587"}="HO10 587"; $resn_exch{"3HO1 587"}="HO13 587"; $resn_exch{"1HN3 588"}="HN31 588";
$resn_exch{"2HN3 588"}="HN32 588"; $resn_exch{"1H7  588"}=" H71 588"; $resn_exch{"2H7  588"}=" H72 588";
$resn_exch{"1H20 588"}="H201 588"; $resn_exch{"2H20 588"}="H202 588"; $resn_exch{"3H20 588"}="H203 588";
$resn_exch{"1H12 588"}="H121 588"; $resn_exch{"2H12 588"}="H122 588"; $resn_exch{"1H13 588"}="H131 588";
$resn_exch{"2H13 588"}="H132 588"; $resn_exch{"1H15 588"}="H151 588"; $resn_exch{"2H15 588"}="H152 588";
$resn_exch{"1H14 588"}="H141 588"; $resn_exch{"2H14 588"}="H142 588"; $resn_exch{"1H16 588"}="H161 588";
$resn_exch{"2H16 588"}="H162 588"; $resn_exch{"3H16 588"}="H163 588"; $resn_exch{"1H10 598"}="H101 598";
$resn_exch{"2H10 598"}="H102 598"; $resn_exch{"1H30 598"}="H301 598"; $resn_exch{"2H30 598"}="H302 598";
$resn_exch{"1H31 598"}="H311 598"; $resn_exch{"2H31 598"}="H312 598"; $resn_exch{"1H33 598"}="H331 598";
$resn_exch{"2H33 598"}="H332 598"; $resn_exch{"1H34 598"}="H341 598"; $resn_exch{"2H34 598"}="H342 598";
$resn_exch{"1H45 598"}="H451 598"; $resn_exch{"2H45 598"}="H452 598"; $resn_exch{"1H12 59A"}="H121 59A";
$resn_exch{"2H12 59A"}="H122 59A"; $resn_exch{"1H11 59A"}="H111 59A"; $resn_exch{"2H11 59A"}="H112 59A";
$resn_exch{"3H11 59A"}="H113 59A"; $resn_exch{"1H13 59A"}="H131 59A"; $resn_exch{"2H13 59A"}="H132 59A";
$resn_exch{"3H13 59A"}="H133 59A"; $resn_exch{"1H8  59A"}=" H81 59A"; $resn_exch{"2H8  59A"}=" H82 59A";
$resn_exch{"1H9  59A"}=" H91 59A"; $resn_exch{"2H9  59A"}=" H92 59A"; $resn_exch{"3H9  59A"}=" H93 59A";
$resn_exch{"3HOP 5AA"}="HOP3 5AA"; $resn_exch{"2HOP 5AA"}="HOP2 5AA"; $resn_exch{"1H5* 5AA"}=" H5' 5AA";
$resn_exch{"2H5* 5AA"}="H5'' 5AA"; $resn_exch{" H4* 5AA"}=" H4' 5AA"; $resn_exch{" H1* 5AA"}=" H1' 5AA";
$resn_exch{"1H9  5AA"}=" H91 5AA"; $resn_exch{"2H9  5AA"}=" H92 5AA"; $resn_exch{"3H9  5AA"}=" H93 5AA";
$resn_exch{"1H10 5AA"}="H101 5AA"; $resn_exch{"2H10 5AA"}="H102 5AA"; $resn_exch{"3H10 5AA"}="H103 5AA";
$resn_exch{" H2* 5AA"}=" H2' 5AA"; $resn_exch{" H3* 5AA"}=" H3' 5AA"; $resn_exch{"1H3* 5AA"}="H3'1 5AA";
$resn_exch{"2H3* 5AA"}="H3'2 5AA"; $resn_exch{"1HN6 5AD"}="HN61 5AD"; $resn_exch{"2HN6 5AD"}="HN62 5AD";
$resn_exch{"1H5' 5AD"}="H5'1 5AD"; $resn_exch{"2H5' 5AD"}="H5'2 5AD"; $resn_exch{"3H5' 5AD"}="H5'3 5AD";
$resn_exch{"'HO2 5AD"}="HO2' 5AD"; $resn_exch{"'HO3 5AD"}="HO3' 5AD"; $resn_exch{"1H1  5AF"}=" H11 5AF";
$resn_exch{"3H1  5AF"}=" H13 5AF"; $resn_exch{"2H1  5AF"}=" H12 5AF"; $resn_exch{"1H2  5AF"}=" H21 5AF";
$resn_exch{"2H2  5AF"}=" H22 5AF"; $resn_exch{"1H3  5AF"}=" H31 5AF"; $resn_exch{"2H4  5AF"}=" H42 5AF";
$resn_exch{"2H5  5AF"}=" H52 5AF"; $resn_exch{"3H5  5AF"}=" H53 5AF"; $resn_exch{"1H7  5AF"}=" H71 5AF";
$resn_exch{"2H6  5AF"}=" H62 5AF"; $resn_exch{"3H6  5AF"}=" H63 5AF"; $resn_exch{"1H1  5AM"}="H11A 5AM";
$resn_exch{"2H1  5AM"}="H12A 5AM"; $resn_exch{"1H15 5AM"}="H151 5AM"; $resn_exch{"2H15 5AM"}="H152 5AM";
$resn_exch{"1H16 5AM"}="H161 5AM"; $resn_exch{"2H16 5AM"}="H162 5AM"; $resn_exch{"1H27 5AM"}="H271 5AM";
$resn_exch{"2H27 5AM"}="H272 5AM"; $resn_exch{"1H30 5AM"}="H301 5AM"; $resn_exch{"2H30 5AM"}="H302 5AM";
$resn_exch{"1H31 5AM"}="H311 5AM"; $resn_exch{"2H31 5AM"}="H312 5AM"; $resn_exch{"1H33 5AM"}="H331 5AM";
$resn_exch{"2H33 5AM"}="H332 5AM"; $resn_exch{"1H35 5AM"}="H351 5AM"; $resn_exch{"2H35 5AM"}="H352 5AM";
$resn_exch{"1H17 5AM"}="H171 5AM"; $resn_exch{"2H17 5AM"}="H172 5AM"; $resn_exch{"1H18 5AM"}="H181 5AM";
$resn_exch{"2H18 5AM"}="H182 5AM"; $resn_exch{"1H19 5AM"}="H191 5AM"; $resn_exch{"2H19 5AM"}="H192 5AM";
$resn_exch{"1H25 5AM"}="H251 5AM"; $resn_exch{"2H25 5AM"}="H252 5AM"; $resn_exch{"1H28 5AM"}="H281 5AM";
$resn_exch{"2H28 5AM"}="H282 5AM"; $resn_exch{"1H36 5AM"}="H361 5AM"; $resn_exch{"2H36 5AM"}="H362 5AM";
$resn_exch{"1HN  5AN"}=" HN1 5AN"; $resn_exch{"2HN  5AN"}=" HN2 5AN"; $resn_exch{"1H20 5AP"}="H201 5AP";
$resn_exch{"2H20 5AP"}="H202 5AP"; $resn_exch{"1H21 5AP"}="H211 5AP"; $resn_exch{"2H21 5AP"}="H212 5AP";
$resn_exch{"1H23 5AP"}="H231 5AP"; $resn_exch{"2H23 5AP"}="H232 5AP"; $resn_exch{"1H28 5AP"}="H281 5AP";
$resn_exch{"2H28 5AP"}="H282 5AP"; $resn_exch{"3H28 5AP"}="H283 5AP"; $resn_exch{"1H29 5AP"}="H291 5AP";
$resn_exch{"2H29 5AP"}="H292 5AP"; $resn_exch{"3H29 5AP"}="H293 5AP"; $resn_exch{"1H5* 5AS"}="H5'1 5AS";
$resn_exch{"2H5* 5AS"}="H5'2 5AS"; $resn_exch{" H4* 5AS"}=" H4' 5AS"; $resn_exch{" H3* 5AS"}=" H3' 5AS";
$resn_exch{"3HO* 5AS"}="HO'3 5AS"; $resn_exch{" H2* 5AS"}=" H2' 5AS"; $resn_exch{"2HO* 5AS"}="HO'2 5AS";
$resn_exch{" H1* 5AS"}=" H1' 5AS"; $resn_exch{"1H6  5AS"}=" H61 5AS"; $resn_exch{"2H6  5AS"}=" H62 5AS";
$resn_exch{"1H10 5AS"}="H101 5AS"; $resn_exch{"2H10 5AS"}="H102 5AS"; $resn_exch{"1H11 5AS"}="H111 5AS";
$resn_exch{"2H11 5AS"}="H112 5AS"; $resn_exch{"3H11 5AS"}="H113 5AS"; $resn_exch{"1HN5 5AT"}="HN51 5AT";
$resn_exch{"2HN5 5AT"}="HN52 5AT"; $resn_exch{"1H2* 5AT"}=" H2' 5AT"; $resn_exch{"2H2* 5AT"}="H2'' 5AT";
$resn_exch{"1H5* 5AT"}=" H5' 5AT"; $resn_exch{"2H5* 5AT"}="H5'' 5AT"; $resn_exch{" H4* 5AT"}=" H4' 5AT";
$resn_exch{" H1* 5AT"}=" H1' 5AT"; $resn_exch{" H3* 5AT"}=" H3' 5AT"; $resn_exch{"H3T  5AT"}="HO3' 5AT";
$resn_exch{"1H6  5AX"}=" H61 5AX"; $resn_exch{"2H6  5AX"}=" H62 5AX"; $resn_exch{"1H8  5AX"}=" H81 5AX";
$resn_exch{"2H8  5AX"}=" H82 5AX"; $resn_exch{"3H8  5AX"}=" H83 5AX"; $resn_exch{"1H1  5AX"}=" H11 5AX";
$resn_exch{"2H1  5AX"}=" H12 5AX"; $resn_exch{"1H22 5BN"}="H221 5BN"; $resn_exch{"2H22 5BN"}="H222 5BN";
$resn_exch{"1H21 5BN"}="H211 5BN"; $resn_exch{"2H21 5BN"}="H212 5BN"; $resn_exch{"1H20 5BN"}="H201 5BN";
$resn_exch{"2H20 5BN"}="H202 5BN"; $resn_exch{"1H1  5BR"}=" H11 5BR"; $resn_exch{"2H1  5BR"}=" H12 5BR";
$resn_exch{"1H2  5BR"}=" H21 5BR"; $resn_exch{"2H2  5BR"}=" H22 5BR"; $resn_exch{"1H3  5BR"}=" H31 5BR";
$resn_exch{"2H3  5BR"}=" H32 5BR"; $resn_exch{"1H4  5BR"}=" H41 5BR"; $resn_exch{"2H4  5BR"}=" H42 5BR";
$resn_exch{"1H5  5BR"}=" H51 5BR"; $resn_exch{"2H5  5BR"}=" H52 5BR"; $resn_exch{"3H5  5BR"}=" H53 5BR";
$resn_exch{"2HOP 5BU"}="HOP2 5BU"; $resn_exch{"3HOP 5BU"}="HOP3 5BU"; $resn_exch{"1H5* 5BU"}=" H5' 5BU";
$resn_exch{"2H5* 5BU"}="H5'' 5BU"; $resn_exch{" H4* 5BU"}=" H4' 5BU"; $resn_exch{" H3* 5BU"}=" H3' 5BU";
$resn_exch{"H3T  5BU"}="HO3' 5BU"; $resn_exch{" H2* 5BU"}=" H2' 5BU"; $resn_exch{"2HO* 5BU"}="HO2' 5BU";
$resn_exch{" H1* 5BU"}=" H1' 5BU"; $resn_exch{"1HB  5CA"}=" HB1 5CA"; $resn_exch{"2HB  5CA"}=" HB2 5CA";
$resn_exch{"1H5* 5CA"}="H5'1 5CA"; $resn_exch{"2H5* 5CA"}="H5'2 5CA"; $resn_exch{" H4* 5CA"}=" H4' 5CA";
$resn_exch{" H3* 5CA"}=" H3' 5CA"; $resn_exch{" H2* 5CA"}=" H2' 5CA"; $resn_exch{"2HO* 5CA"}="HO'2 5CA";
$resn_exch{" H1* 5CA"}=" H1' 5CA"; $resn_exch{"1H6  5CA"}=" H61 5CA"; $resn_exch{"2H6  5CA"}=" H62 5CA";
$resn_exch{"1H13 5CB"}="H131 5CB"; $resn_exch{"2H13 5CB"}="H132 5CB"; $resn_exch{"1H16 5CB"}="H161 5CB";
$resn_exch{"2H16 5CB"}="H162 5CB"; $resn_exch{"17H1 5CB"}="H117 5CB"; $resn_exch{"1H19 5CB"}="H191 5CB";
$resn_exch{"20H2 5CB"}="H220 5CB"; $resn_exch{"1H24 5CB"}="H241 5CB"; $resn_exch{"2H24 5CB"}="H242 5CB";
$resn_exch{"1H25 5CB"}="H251 5CB"; $resn_exch{"2H25 5CB"}="H252 5CB"; $resn_exch{"1H6N 5CD"}="H6N1 5CD";
$resn_exch{"2H6N 5CD"}="H6N2 5CD"; $resn_exch{" H2* 5CD"}=" H2' 5CD"; $resn_exch{" H3* 5CD"}=" H3' 5CD";
$resn_exch{" H4* 5CD"}=" H4' 5CD"; $resn_exch{"1H5* 5CD"}="H5'1 5CD"; $resn_exch{"2H5* 5CD"}="H5'2 5CD";
$resn_exch{" H1* 5CD"}=" H1' 5CD"; $resn_exch{"1H5* 5CG"}=" H5' 5CG"; $resn_exch{"2H5* 5CG"}="H5'' 5CG";
$resn_exch{" H4* 5CG"}=" H4' 5CG"; $resn_exch{" H3* 5CG"}=" H3' 5CG"; $resn_exch{"*HO3 5CG"}="HO3' 5CG";
$resn_exch{" H2* 5CG"}=" H2' 5CG"; $resn_exch{"*HO2 5CG"}="HO2' 5CG"; $resn_exch{" H1* 5CG"}=" H1' 5CG";
$resn_exch{"1HN2 5CG"}="HN21 5CG"; $resn_exch{"2HN2 5CG"}="HN22 5CG"; $resn_exch{"1H5A 5CM"}="H5A1 5CM";
$resn_exch{"2H5A 5CM"}="H5A2 5CM"; $resn_exch{"3H5A 5CM"}="H5A3 5CM"; $resn_exch{"1HN4 5CM"}="HN41 5CM";
$resn_exch{"2HN4 5CM"}="HN42 5CM"; $resn_exch{" H1* 5CM"}=" H1' 5CM"; $resn_exch{"1H2* 5CM"}=" H2' 5CM";
$resn_exch{"2H2* 5CM"}="H2'' 5CM"; $resn_exch{" H3* 5CM"}=" H3' 5CM"; $resn_exch{" H4* 5CM"}=" H4' 5CM";
$resn_exch{"*HO3 5CM"}="HO3' 5CM"; $resn_exch{"1H5* 5CM"}=" H5' 5CM"; $resn_exch{"2H5* 5CM"}="H5'' 5CM";
$resn_exch{"2HOP 5CM"}="HOP2 5CM"; $resn_exch{"3HOP 5CM"}="HOP3 5CM"; $resn_exch{"1H7  5CN"}=" H71 5CN";
$resn_exch{"2H7  5CN"}=" H72 5CN"; $resn_exch{"1H8  5CN"}=" H81 5CN"; $resn_exch{"2H8  5CN"}=" H82 5CN";
$resn_exch{"1H9  5CN"}=" H91 5CN"; $resn_exch{"2H9  5CN"}=" H92 5CN"; $resn_exch{"1H10 5CN"}="H101 5CN";
$resn_exch{"2H10 5CN"}="H102 5CN"; $resn_exch{"1H12 5CN"}="H121 5CN"; $resn_exch{"2H12 5CN"}="H122 5CN";
$resn_exch{"3H12 5CN"}="H123 5CN"; $resn_exch{"1H13 5CN"}="H131 5CN"; $resn_exch{"2H13 5CN"}="H132 5CN";
$resn_exch{" HN2 5CS"}=" H2  5CS"; $resn_exch{"1HB  5CS"}=" HB2 5CS"; $resn_exch{"2HB  5CS"}=" HB3 5CS";
$resn_exch{"1HN  5CS"}=" HN1 5CS"; $resn_exch{"2HN  5CS"}="HN2A 5CS"; $resn_exch{"1HE  5CS"}=" HE3 5CS";
$resn_exch{"2HE  5CS"}=" HE2 5CS"; $resn_exch{"1H17 5DE"}="H171 5DE"; $resn_exch{"2H17 5DE"}="H172 5DE";
$resn_exch{"1H18 5DE"}="H181 5DE"; $resn_exch{"2H18 5DE"}="H182 5DE"; $resn_exch{"3H18 5DE"}="H183 5DE";
$resn_exch{"1H9  5DE"}=" H91 5DE"; $resn_exch{"2H9  5DE"}=" H92 5DE"; $resn_exch{"3H9  5DE"}=" H93 5DE";
$resn_exch{"1H6  5DE"}=" H61 5DE"; $resn_exch{"2H6  5DE"}=" H62 5DE"; $resn_exch{"3H6  5DE"}=" H63 5DE";
$resn_exch{"1H14 5DE"}="H141 5DE"; $resn_exch{"2H14 5DE"}="H142 5DE"; $resn_exch{"1H1  5DS"}=" H11 5DS";
$resn_exch{"2H1  5DS"}=" H12 5DS"; $resn_exch{"3H1  5DS"}=" H13 5DS"; $resn_exch{"1H2  5DS"}=" H21 5DS";
$resn_exch{"2H2  5DS"}=" H22 5DS"; $resn_exch{"3H2  5DS"}=" H23 5DS"; $resn_exch{"1HN4 5DS"}="HN41 5DS";
$resn_exch{"2HN4 5DS"}="HN42 5DS"; $resn_exch{"1HN5 5DS"}="HN51 5DS"; $resn_exch{"2HN5 5DS"}="HN52 5DS";
$resn_exch{"1H22 5EA"}="H221 5EA"; $resn_exch{"2H22 5EA"}="H222 5EA"; $resn_exch{"1H24 5EA"}="H241 5EA";
$resn_exch{"2H24 5EA"}="H242 5EA"; $resn_exch{"1H27 5EA"}="H271 5EA"; $resn_exch{"2H27 5EA"}="H272 5EA";
$resn_exch{"1H29 5EA"}="H291 5EA"; $resn_exch{"2H29 5EA"}="H292 5EA"; $resn_exch{"1H30 5EA"}="H301 5EA";
$resn_exch{"2H30 5EA"}="H302 5EA"; $resn_exch{"1H28 5EA"}="H281 5EA"; $resn_exch{"2H28 5EA"}="H282 5EA";
$resn_exch{"1H1  5EE"}=" H11 5EE"; $resn_exch{"2H1  5EE"}=" H12 5EE"; $resn_exch{"1H2* 5F1"}="H2'1 5F1";
$resn_exch{"2H2* 5F1"}="H2'2 5F1"; $resn_exch{" H3* 5F1"}=" H3' 5F1"; $resn_exch{" H4* 5F1"}=" H4' 5F1";
$resn_exch{"1H5* 5F1"}="H5'1 5F1"; $resn_exch{"2H5* 5F1"}="H5'2 5F1"; $resn_exch{" H1* 5F1"}=" H1' 5F1";
$resn_exch{"1H6N 5F1"}="H6N1 5F1"; $resn_exch{"2H6N 5F1"}="H6N2 5F1"; $resn_exch{"1H6  5FA"}=" H61 5FA";
$resn_exch{"2H6  5FA"}=" H62 5FA"; $resn_exch{"1H5* 5FA"}="H5'1 5FA"; $resn_exch{"2H5* 5FA"}="H5'2 5FA";
$resn_exch{" H4* 5FA"}=" H4' 5FA"; $resn_exch{" H3* 5FA"}=" H3' 5FA"; $resn_exch{" H2* 5FA"}=" H2' 5FA";
$resn_exch{"2HO* 5FA"}="HO'2 5FA"; $resn_exch{" H1* 5FA"}=" H1' 5FA"; $resn_exch{"2HOP 5FC"}="HOP2 5FC";
$resn_exch{"1HN4 5FC"}="HN41 5FC"; $resn_exch{"2HN4 5FC"}="HN42 5FC"; $resn_exch{"1H2* 5FC"}=" H2' 5FC";
$resn_exch{"2H2* 5FC"}="H2'' 5FC"; $resn_exch{"1H5* 5FC"}=" H5' 5FC"; $resn_exch{"2H5* 5FC"}="H5'' 5FC";
$resn_exch{" H4* 5FC"}=" H4' 5FC"; $resn_exch{" H1* 5FC"}=" H1' 5FC"; $resn_exch{" H3* 5FC"}=" H3' 5FC";
$resn_exch{"H3T  5FC"}="HO3' 5FC"; $resn_exch{"3HOP 5FC"}="HOP3 5FC"; $resn_exch{" H2* 5FD"}=" H2' 5FD";
$resn_exch{"*H3* 5FD"}="H3'' 5FD"; $resn_exch{" H4* 5FD"}=" H4' 5FD"; $resn_exch{"1H5* 5FD"}="H5'1 5FD";
$resn_exch{"2H5* 5FD"}="H5'2 5FD"; $resn_exch{" H1* 5FD"}=" H1' 5FD"; $resn_exch{"5H5  5FD"}=" H55 5FD";
$resn_exch{"4H4  5FD"}=" H44 5FD"; $resn_exch{"1H2  5FE"}=" H21 5FE"; $resn_exch{"2H2  5FE"}=" H22 5FE";
$resn_exch{"1HA  5FE"}=" HA1 5FE"; $resn_exch{"2HA  5FE"}=" HA2 5FE"; $resn_exch{"3HA  5FE"}=" HA3 5FE";
$resn_exch{"1H29 5FE"}="H291 5FE"; $resn_exch{"2H29 5FE"}="H292 5FE"; $resn_exch{"3H29 5FE"}="H293 5FE";
$resn_exch{"1H10 5FE"}="H101 5FE"; $resn_exch{"2H10 5FE"}="H102 5FE"; $resn_exch{"1H35 5FE"}="H351 5FE";
$resn_exch{"2H35 5FE"}="H352 5FE"; $resn_exch{"3H35 5FE"}="H353 5FE"; $resn_exch{"1H34 5FE"}="H341 5FE";
$resn_exch{"2H34 5FE"}="H342 5FE"; $resn_exch{"3H34 5FE"}="H343 5FE"; $resn_exch{"1H38 5FE"}="H381 5FE";
$resn_exch{"2H38 5FE"}="H382 5FE"; $resn_exch{"3H38 5FE"}="H383 5FE"; $resn_exch{"1H37 5FE"}="H371 5FE";
$resn_exch{"2H37 5FE"}="H372 5FE"; $resn_exch{"3H37 5FE"}="H373 5FE"; $resn_exch{"1H22 5FE"}="H221 5FE";
$resn_exch{"2H22 5FE"}="H222 5FE"; $resn_exch{"1H39 5FE"}="H391 5FE"; $resn_exch{"2H39 5FE"}="H392 5FE";
$resn_exch{"3H39 5FE"}="H393 5FE"; $resn_exch{"1H24 5FE"}="H241 5FE"; $resn_exch{"2H24 5FE"}="H242 5FE";
$resn_exch{"3H24 5FE"}="H243 5FE"; $resn_exch{"1H2  5FP"}=" H21 5FP"; $resn_exch{"2H2  5FP"}=" H22 5FP";
$resn_exch{"1H29 5FP"}="H291 5FP"; $resn_exch{"2H29 5FP"}="H292 5FP"; $resn_exch{"3H29 5FP"}="H293 5FP";
$resn_exch{"1H10 5FP"}="H101 5FP"; $resn_exch{"2H10 5FP"}="H102 5FP"; $resn_exch{"1H35 5FP"}="H351 5FP";
$resn_exch{"2H35 5FP"}="H352 5FP"; $resn_exch{"3H35 5FP"}="H353 5FP"; $resn_exch{"1H34 5FP"}="H341 5FP";
$resn_exch{"2H34 5FP"}="H342 5FP"; $resn_exch{"3H34 5FP"}="H343 5FP"; $resn_exch{"1H38 5FP"}="H381 5FP";
$resn_exch{"2H38 5FP"}="H382 5FP"; $resn_exch{"3H38 5FP"}="H383 5FP"; $resn_exch{"1H37 5FP"}="H371 5FP";
$resn_exch{"2H37 5FP"}="H372 5FP"; $resn_exch{"3H37 5FP"}="H373 5FP"; $resn_exch{"1H22 5FP"}="H221 5FP";
$resn_exch{"2H22 5FP"}="H222 5FP"; $resn_exch{"1H39 5FP"}="H391 5FP"; $resn_exch{"2H39 5FP"}="H392 5FP";
$resn_exch{"3H39 5FP"}="H393 5FP"; $resn_exch{"1H24 5FP"}="H241 5FP"; $resn_exch{"2H24 5FP"}="H242 5FP";
$resn_exch{"3H24 5FP"}="H243 5FP"; $resn_exch{"2HO* 5FU"}="HO'2 5FU"; $resn_exch{" H2* 5FU"}=" H2' 5FU";
$resn_exch{" H1* 5FU"}=" H1' 5FU"; $resn_exch{" H4* 5FU"}=" H4' 5FU"; $resn_exch{"1H5* 5FU"}="H5'1 5FU";
$resn_exch{"2H5* 5FU"}="H5'2 5FU"; $resn_exch{"2HOP 5FU"}="HOP2 5FU"; $resn_exch{"3HOP 5FU"}="HOP3 5FU";
$resn_exch{" H3* 5FU"}=" H3' 5FU"; $resn_exch{"2HOP 5GP"}="HOP2 5GP"; $resn_exch{"3HOP 5GP"}="HOP3 5GP";
$resn_exch{"1H5* 5GP"}="H5'1 5GP"; $resn_exch{"2H5* 5GP"}="H5'2 5GP"; $resn_exch{" H4* 5GP"}=" H4' 5GP";
$resn_exch{" H3* 5GP"}=" H3' 5GP"; $resn_exch{"*HO3 5GP"}="HO3' 5GP"; $resn_exch{" H2* 5GP"}=" H2' 5GP";
$resn_exch{"*HO2 5GP"}="HO2' 5GP"; $resn_exch{" H1* 5GP"}=" H1' 5GP"; $resn_exch{"1HN2 5GP"}="HN21 5GP";
$resn_exch{"2HN2 5GP"}="HN22 5GP"; $resn_exch{"1H1   5H"}=" H11  5H"; $resn_exch{"2H1   5H"}=" H12  5H";
$resn_exch{"1H1  5HA"}=" H11 5HA"; $resn_exch{"2H1  5HA"}="H12A 5HA"; $resn_exch{"1H3  5HA"}="H31A 5HA";
$resn_exch{"2H3  5HA"}="H32A 5HA"; $resn_exch{"1H4  5HA"}=" H41 5HA"; $resn_exch{"2H4  5HA"}=" H42 5HA";
$resn_exch{"1H8  5HA"}=" H81 5HA"; $resn_exch{"2H8  5HA"}=" H82 5HA"; $resn_exch{"1H16 5HA"}="H161 5HA";
$resn_exch{"2H16 5HA"}="H162 5HA"; $resn_exch{"3H16 5HA"}="H163 5HA"; $resn_exch{"1H23 5HA"}="H231 5HA";
$resn_exch{"2H23 5HA"}="H232 5HA"; $resn_exch{"3H23 5HA"}="H233 5HA"; $resn_exch{"1H18 5HA"}="H181 5HA";
$resn_exch{"2H18 5HA"}="H182 5HA"; $resn_exch{"3H18 5HA"}="H183 5HA"; $resn_exch{"1H28 5HG"}="H281 5HG";
$resn_exch{"2H28 5HG"}="H282 5HG"; $resn_exch{"1H24 5HG"}="H241 5HG"; $resn_exch{"2H24 5HG"}="H242 5HG";
$resn_exch{"1H22 5HG"}="H221 5HG"; $resn_exch{"2H22 5HG"}="H222 5HG"; $resn_exch{"1H20 5HG"}="H201 5HG";
$resn_exch{"2H20 5HG"}="H202 5HG"; $resn_exch{"1HB  5HP"}=" HB1 5HP"; $resn_exch{"2HB  5HP"}=" HB2 5HP";
$resn_exch{"1HG  5HP"}=" HG1 5HP"; $resn_exch{"2HG  5HP"}=" HG2 5HP"; $resn_exch{"H5T  5HT"}="HO5' 5HT";
$resn_exch{"1H5* 5HT"}=" H5' 5HT"; $resn_exch{"2H5* 5HT"}="H5'' 5HT"; $resn_exch{" H4* 5HT"}=" H4' 5HT";
$resn_exch{" H3* 5HT"}=" H3' 5HT"; $resn_exch{"H3T  5HT"}="HO3' 5HT"; $resn_exch{"1H2* 5HT"}=" H2' 5HT";
$resn_exch{"2H2* 5HT"}="H2'' 5HT"; $resn_exch{" H1* 5HT"}=" H1' 5HT"; $resn_exch{"1H5M 5HT"}=" H71 5HT";
$resn_exch{"2H5M 5HT"}=" H72 5HT"; $resn_exch{"3H5M 5HT"}=" H73 5HT"; $resn_exch{"1H6  5HT"}=" H61 5HT";
$resn_exch{"2H6  5HT"}=" H62 5HT"; $resn_exch{"1H5* 5HU"}=" H5' 5HU"; $resn_exch{"2H5* 5HU"}="H5'' 5HU";
$resn_exch{"HC4* 5HU"}="HC4' 5HU"; $resn_exch{"HC3* 5HU"}="HC3' 5HU"; $resn_exch{"HO3* 5HU"}="HO3' 5HU";
$resn_exch{"1H2* 5HU"}=" H2' 5HU"; $resn_exch{"2H2* 5HU"}="H2'' 5HU"; $resn_exch{"HC1* 5HU"}="HC1' 5HU";
$resn_exch{"1H5A 5HU"}="H5A1 5HU"; $resn_exch{"2H5A 5HU"}="H5A2 5HU"; $resn_exch{"1HN4 5I5"}="HN41 5I5";
$resn_exch{"2HN4 5I5"}="HN42 5I5"; $resn_exch{"1H11 5I5"}="H111 5I5"; $resn_exch{"2H11 5I5"}="H112 5I5";
$resn_exch{"3H11 5I5"}="H113 5I5"; $resn_exch{"2HO* 5IC"}="HO2' 5IC"; $resn_exch{" H2* 5IC"}=" H2' 5IC";
$resn_exch{" H3* 5IC"}=" H3' 5IC"; $resn_exch{"H3T  5IC"}="HO3' 5IC"; $resn_exch{" H4* 5IC"}=" H4' 5IC";
$resn_exch{" H1* 5IC"}=" H1' 5IC"; $resn_exch{"1H4  5IC"}=" H41 5IC"; $resn_exch{"2H4  5IC"}=" H42 5IC";
$resn_exch{"1H5* 5IC"}=" H5' 5IC"; $resn_exch{"2H5* 5IC"}="H5'' 5IC"; $resn_exch{"3HOP 5IC"}="HOP3 5IC";
$resn_exch{"2HOP 5IC"}="HOP2 5IC"; $resn_exch{"1H5' 5ID"}="H5'1 5ID"; $resn_exch{"2H5' 5ID"}="H5'2 5ID";
$resn_exch{"1H6N 5ID"}="H6N1 5ID"; $resn_exch{"2H6N 5ID"}="H6N2 5ID"; $resn_exch{"1HN3 5IG"}="HN31 5IG";
$resn_exch{"2HN3 5IG"}="HN32 5IG"; $resn_exch{"1HN4 5IG"}="HN41 5IG"; $resn_exch{"2HN4 5IG"}="HN42 5IG";
$resn_exch{"1H6  5IG"}=" H61 5IG"; $resn_exch{"2H6  5IG"}=" H62 5IG"; $resn_exch{"1H16 5IG"}="H161 5IG";
$resn_exch{"2H16 5IG"}="H162 5IG"; $resn_exch{"1H17 5IG"}="H171 5IG"; $resn_exch{"2H17 5IG"}="H172 5IG";
$resn_exch{"1H18 5IG"}="H181 5IG"; $resn_exch{"2H18 5IG"}="H182 5IG"; $resn_exch{"1H19 5IG"}="H191 5IG";
$resn_exch{"2H19 5IG"}="H192 5IG"; $resn_exch{"3H19 5IG"}="H193 5IG"; $resn_exch{"1H21 5IG"}="H211 5IG";
$resn_exch{"2H21 5IG"}="H212 5IG"; $resn_exch{"3H21 5IG"}="H213 5IG"; $resn_exch{"1H20 5IG"}="H201 5IG";
$resn_exch{"2H20 5IG"}="H202 5IG"; $resn_exch{"3H20 5IG"}="H203 5IG"; $resn_exch{"1H5  5IG"}=" H51 5IG";
$resn_exch{"2H5  5IG"}=" H52 5IG"; $resn_exch{"3H5  5IG"}=" H53 5IG"; $resn_exch{"1H1  5IN"}=" H11 5IN";
$resn_exch{"2H1  5IN"}="H12A 5IN"; $resn_exch{"1H3  5IN"}="H31A 5IN"; $resn_exch{"2H3  5IN"}=" H32 5IN";
$resn_exch{"1H10 5IN"}="H101 5IN"; $resn_exch{"2H10 5IN"}="H102 5IN"; $resn_exch{"1H19 5IN"}="H191 5IN";
$resn_exch{"2H19 5IN"}="H192 5IN"; $resn_exch{"1H25 5IN"}="H251 5IN"; $resn_exch{"2H25 5IN"}="H252 5IN";
$resn_exch{"1H34 5IN"}="H341 5IN"; $resn_exch{"2H34 5IN"}="H342 5IN"; $resn_exch{"1HAJ 5IQ"}="HAJ1 5IQ";
$resn_exch{"2HAJ 5IQ"}="HAJ2 5IQ"; $resn_exch{"2HOP 5IT"}="HOP2 5IT"; $resn_exch{"3HOP 5IT"}="HOP3 5IT";
$resn_exch{"1H5* 5IT"}=" H5' 5IT"; $resn_exch{"2H5* 5IT"}="H5'' 5IT"; $resn_exch{" H4* 5IT"}=" H4' 5IT";
$resn_exch{" H3* 5IT"}=" H3' 5IT"; $resn_exch{"*HO3 5IT"}="HO3' 5IT"; $resn_exch{"1H2* 5IT"}=" H2' 5IT";
$resn_exch{"2H2* 5IT"}="H2'' 5IT"; $resn_exch{" H1* 5IT"}=" H1' 5IT"; $resn_exch{" H1* 5IU"}=" H1' 5IU";
$resn_exch{"1H2* 5IU"}=" H2' 5IU"; $resn_exch{"2H2* 5IU"}="H2'' 5IU"; $resn_exch{" H3* 5IU"}=" H3' 5IU";
$resn_exch{" H4* 5IU"}=" H4' 5IU"; $resn_exch{"*HO3 5IU"}="HO3' 5IU"; $resn_exch{"1H5* 5IU"}=" H5' 5IU";
$resn_exch{"2H5* 5IU"}="H5'' 5IU"; $resn_exch{"2HOP 5IU"}="HOP2 5IU"; $resn_exch{"3HOP 5IU"}="HOP3 5IU";
$resn_exch{"1HN3 5MB"}="HN31 5MB"; $resn_exch{"1H2  5MB"}=" H21 5MB"; $resn_exch{"1H4  5MB"}=" H41 5MB";
$resn_exch{"1H6  5MB"}=" H61 5MB"; $resn_exch{"1H7  5MB"}=" H71 5MB"; $resn_exch{"1H8  5MB"}=" H81 5MB";
$resn_exch{"2H8  5MB"}=" H82 5MB"; $resn_exch{"3H8  5MB"}=" H83 5MB"; $resn_exch{"2HOP 5MC"}="HOP2 5MC";
$resn_exch{"3HOP 5MC"}="HOP3 5MC"; $resn_exch{"1H5* 5MC"}=" H5' 5MC"; $resn_exch{"2H5* 5MC"}="H5'' 5MC";
$resn_exch{" H4* 5MC"}=" H4' 5MC"; $resn_exch{" H3* 5MC"}=" H3' 5MC"; $resn_exch{"*HO3 5MC"}="HO3' 5MC";
$resn_exch{" H2* 5MC"}=" H2' 5MC"; $resn_exch{"*HO2 5MC"}="HO2' 5MC"; $resn_exch{" H1* 5MC"}=" H1' 5MC";
$resn_exch{"1HN4 5MC"}="HN41 5MC"; $resn_exch{"2HN4 5MC"}="HN42 5MC"; $resn_exch{"1HM5 5MC"}="HM51 5MC";
$resn_exch{"2HM5 5MC"}="HM52 5MC"; $resn_exch{"3HM5 5MC"}="HM53 5MC"; $resn_exch{" H5* 5MD"}=" H5' 5MD";
$resn_exch{"1H5' 5MD"}="H5'1 5MD"; $resn_exch{"2H5' 5MD"}="H5'2 5MD"; $resn_exch{"1H5  5MD"}=" H51 5MD";
$resn_exch{"2H5  5MD"}=" H52 5MD"; $resn_exch{"3H5  5MD"}=" H53 5MD"; $resn_exch{"H3*  5MD"}="H3'A 5MD";
$resn_exch{"1H2' 5MD"}="H2'1 5MD"; $resn_exch{"2H2' 5MD"}="H2'2 5MD"; $resn_exch{"1HC4 5MP"}="HC41 5MP";
$resn_exch{"2HC4 5MP"}="HC42 5MP"; $resn_exch{"3HC4 5MP"}="HC43 5MP"; $resn_exch{"1HB  5MR"}=" HB1 5MR";
$resn_exch{"2HB  5MR"}=" HB2 5MR"; $resn_exch{"1HAA 5MR"}="HAA1 5MR"; $resn_exch{"2HAA 5MR"}="HAA2 5MR";
$resn_exch{"1HAX 5MR"}="HAX1 5MR"; $resn_exch{"2HAX 5MR"}="HAX2 5MR"; $resn_exch{"1HAV 5MR"}="HAV1 5MR";
$resn_exch{"2HAV 5MR"}="HAV2 5MR"; $resn_exch{"1H5M 5MU"}=" H71 5MU"; $resn_exch{"2H5M 5MU"}=" H72 5MU";
$resn_exch{"3H5M 5MU"}=" H73 5MU"; $resn_exch{" H1* 5MU"}=" H1' 5MU"; $resn_exch{" H2* 5MU"}=" H2' 5MU";
$resn_exch{"*HO2 5MU"}="HO2' 5MU"; $resn_exch{" H3* 5MU"}=" H3' 5MU"; $resn_exch{" H4* 5MU"}=" H4' 5MU";
$resn_exch{"*HO3 5MU"}="HO3' 5MU"; $resn_exch{"1H5* 5MU"}=" H5' 5MU"; $resn_exch{"2H5* 5MU"}="H5'' 5MU";
$resn_exch{"2HOP 5MU"}="HOP2 5MU"; $resn_exch{"3HOP 5MU"}="HOP3 5MU"; $resn_exch{"2HOP 5NC"}="HOP2 5NC";
$resn_exch{"3HOP 5NC"}="HOP3 5NC"; $resn_exch{"1H5* 5NC"}=" H5' 5NC"; $resn_exch{"2H5* 5NC"}="H5'' 5NC";
$resn_exch{" H4* 5NC"}=" H4' 5NC"; $resn_exch{" H3* 5NC"}=" H3' 5NC"; $resn_exch{"3HO* 5NC"}="HO'3 5NC";
$resn_exch{"1H2* 5NC"}=" H2' 5NC"; $resn_exch{"2H2* 5NC"}="H2'' 5NC"; $resn_exch{"1H1* 5NC"}="H1'1 5NC";
$resn_exch{"1HN4 5NC"}="HN41 5NC"; $resn_exch{"2HN4 5NC"}="HN42 5NC"; $resn_exch{"1H5  5NH"}=" H51 5NH";
$resn_exch{"2H5  5NH"}=" H52 5NH"; $resn_exch{"3H5  5NH"}=" H53 5NH"; $resn_exch{"1H6  5NH"}=" H61 5NH";
$resn_exch{"2H6  5NH"}=" H62 5NH"; $resn_exch{"3H6  5NH"}=" H63 5NH"; $resn_exch{"1H7  5NH"}=" H71 5NH";
$resn_exch{"2H7  5NH"}=" H72 5NH"; $resn_exch{"3H7  5NH"}=" H73 5NH"; $resn_exch{"1H11 5NH"}="H111 5NH";
$resn_exch{"2H11 5NH"}="H112 5NH"; $resn_exch{"1H12 5NH"}="H121 5NH"; $resn_exch{"2H12 5NH"}="H122 5NH";
$resn_exch{"1H13 5NH"}="H131 5NH"; $resn_exch{"2H13 5NH"}="H132 5NH"; $resn_exch{"1H14 5NH"}="H141 5NH";
$resn_exch{"2H14 5NH"}="H142 5NH"; $resn_exch{"1H15 5NH"}="H151 5NH"; $resn_exch{"2H15 5NH"}="H152 5NH";
$resn_exch{"1H23 5NH"}="H231 5NH"; $resn_exch{"2H23 5NH"}="H232 5NH"; $resn_exch{"1H24 5NH"}="H241 5NH";
$resn_exch{"2H24 5NH"}="H242 5NH"; $resn_exch{"1H26 5NH"}="H261 5NH"; $resn_exch{"2H26 5NH"}="H262 5NH";
$resn_exch{"1H29 5NH"}="H291 5NH"; $resn_exch{"2H29 5NH"}="H292 5NH"; $resn_exch{"1H30 5NH"}="H301 5NH";
$resn_exch{"2H30 5NH"}="H302 5NH"; $resn_exch{"1H31 5NH"}="H311 5NH"; $resn_exch{"2H31 5NH"}="H312 5NH";
$resn_exch{"1H32 5NH"}="H321 5NH"; $resn_exch{"2H32 5NH"}="H322 5NH"; $resn_exch{"3H32 5NH"}="H323 5NH";
$resn_exch{"1H37 5NH"}="H371 5NH"; $resn_exch{"2H37 5NH"}="H372 5NH"; $resn_exch{"1H52 5NH"}="H521 5NH";
$resn_exch{"2H52 5NH"}="H522 5NH"; $resn_exch{"3H52 5NH"}="H523 5NH"; $resn_exch{"1H53 5NH"}="H531 5NH";
$resn_exch{"2H53 5NH"}="H532 5NH"; $resn_exch{"3H53 5NH"}="H533 5NH"; $resn_exch{"1H6  5NI"}=" H61 5NI";
$resn_exch{"1H7  5NI"}=" H71 5NI"; $resn_exch{"1H4  5NI"}=" H41 5NI"; $resn_exch{"1HN1 5NI"}="HN11 5NI";
$resn_exch{"1H3  5NI"}=" H31 5NI"; $resn_exch{"1NH3 5OB"}="H31N 5OB"; $resn_exch{"1H2  5OB"}=" H21 5OB";
$resn_exch{"1H4  5OB"}=" H41 5OB"; $resn_exch{"1H6  5OB"}=" H61 5OB"; $resn_exch{"1H7  5OB"}=" H71 5OB";
$resn_exch{"1H11 5OB"}="H111 5OB"; $resn_exch{"2H11 5OB"}="H112 5OB"; $resn_exch{"3H11 5OB"}="H113 5OB";
$resn_exch{"1H1  5OP"}=" H11 5OP"; $resn_exch{"2H1  5OP"}=" H12 5OP"; $resn_exch{"3H1  5OP"}="H13A 5OP";
$resn_exch{"1H2  5OP"}=" H21 5OP"; $resn_exch{"2H2  5OP"}=" H22 5OP"; $resn_exch{"1H3  5OP"}=" H31 5OP";
$resn_exch{"2H3  5OP"}=" H32 5OP"; $resn_exch{"1H4  5OP"}=" H41 5OP"; $resn_exch{"2H4  5OP"}=" H42 5OP";
$resn_exch{"1H5  5OP"}=" H51 5OP"; $resn_exch{"2H5  5OP"}=" H52 5OP"; $resn_exch{"1H11 5OP"}="H111 5OP";
$resn_exch{"2H11 5OP"}="H112 5OP"; $resn_exch{"1H14 5OP"}="H141 5OP"; $resn_exch{"2H14 5OP"}="H142 5OP";
$resn_exch{"1H17 5OP"}="H171 5OP"; $resn_exch{"2H17 5OP"}="H172 5OP"; $resn_exch{"1H18 5OP"}="H181 5OP";
$resn_exch{"2H18 5OP"}="H182 5OP"; $resn_exch{"1H19 5OP"}="H191 5OP"; $resn_exch{"2H19 5OP"}="H192 5OP";
$resn_exch{"1H4A 5PA"}="H4A1 5PA"; $resn_exch{"2H4A 5PA"}="H4A2 5PA"; $resn_exch{"2HOP 5PA"}="HOP2 5PA";
$resn_exch{"3HOP 5PA"}="HOP3 5PA"; $resn_exch{"1H5A 5PA"}="H5A1 5PA"; $resn_exch{"2H5A 5PA"}="H5A2 5PA";
$resn_exch{"1H2A 5PA"}="H2A1 5PA"; $resn_exch{"2H2A 5PA"}="H2A2 5PA"; $resn_exch{"3H2A 5PA"}="H2A3 5PA";
$resn_exch{"1H9  5PA"}=" H91 5PA"; $resn_exch{"2H9  5PA"}=" H92 5PA"; $resn_exch{"1H10 5PA"}="H101 5PA";
$resn_exch{"2H10 5PA"}="H102 5PA"; $resn_exch{"3HOP 5PC"}="HOP3 5PC"; $resn_exch{"2HOP 5PC"}="HOP2 5PC";
$resn_exch{"1H5* 5PC"}=" H5' 5PC"; $resn_exch{"2H5* 5PC"}="H5'' 5PC"; $resn_exch{" H4* 5PC"}=" H4' 5PC";
$resn_exch{" H3* 5PC"}=" H3' 5PC"; $resn_exch{"H3T  5PC"}="HO3' 5PC"; $resn_exch{"1H2* 5PC"}=" H2' 5PC";
$resn_exch{"2H2* 5PC"}="H2'' 5PC"; $resn_exch{" H1* 5PC"}=" H1' 5PC"; $resn_exch{"1H25 5PH"}="H251 5PH";
$resn_exch{"2H25 5PH"}="H252 5PH"; $resn_exch{"1H36 5PH"}="H361 5PH"; $resn_exch{"2H36 5PH"}="H362 5PH";
$resn_exch{"3H36 5PH"}="H363 5PH"; $resn_exch{"1H4  5PH"}=" H41 5PH"; $resn_exch{"2H4  5PH"}=" H42 5PH";
$resn_exch{"1H1  5PH"}=" H11 5PH"; $resn_exch{"2H1  5PH"}=" H12 5PH"; $resn_exch{"1H12 5PH"}="H121 5PH";
$resn_exch{"2H12 5PH"}="H122 5PH"; $resn_exch{"1H8  5PH"}=" H81 5PH"; $resn_exch{"2H8  5PH"}=" H82 5PH";
$resn_exch{"1H18 5PI"}="H181 5PI"; $resn_exch{"2H18 5PI"}="H182 5PI"; $resn_exch{"1H19 5PI"}="H191 5PI";
$resn_exch{"2H19 5PI"}="H192 5PI"; $resn_exch{"1H21 5PI"}="H211 5PI"; $resn_exch{"2H21 5PI"}="H212 5PI";
$resn_exch{"1HN1 5PI"}="HN11 5PI"; $resn_exch{"2HN1 5PI"}="HN12 5PI"; $resn_exch{"1HN3 5PI"}="HN31 5PI";
$resn_exch{"2HN3 5PI"}="HN32 5PI"; $resn_exch{"1H1  5PI"}=" H11 5PI"; $resn_exch{"2H1  5PI"}="H12A 5PI";
$resn_exch{"1H9  5PI"}=" H91 5PI"; $resn_exch{"2H9  5PI"}=" H92 5PI"; $resn_exch{"1H8  5PI"}=" H81 5PI";
$resn_exch{"2H8  5PI"}=" H82 5PI"; $resn_exch{"3H8  5PI"}=" H83 5PI"; $resn_exch{"1H7  5PI"}=" H71 5PI";
$resn_exch{"2H7  5PI"}=" H72 5PI"; $resn_exch{"1H10 5PI"}="H101 5PI"; $resn_exch{"2H10 5PI"}="H102 5PI";
$resn_exch{"1H11 5PI"}="H111 5PI"; $resn_exch{"2H11 5PI"}="H112 5PI"; $resn_exch{"3H11 5PI"}="H113 5PI";
$resn_exch{"1H14 5PP"}="H141 5PP"; $resn_exch{"2H14 5PP"}="H142 5PP"; $resn_exch{"1H15 5PP"}="H151 5PP";
$resn_exch{"2H15 5PP"}="H152 5PP"; $resn_exch{"1H16 5PP"}="H161 5PP"; $resn_exch{"2H16 5PP"}="H162 5PP";
$resn_exch{"1H17 5PP"}="H171 5PP"; $resn_exch{"2H17 5PP"}="H172 5PP"; $resn_exch{"1H18 5PP"}="H181 5PP";
$resn_exch{"2H18 5PP"}="H182 5PP"; $resn_exch{"3H18 5PP"}="H183 5PP"; $resn_exch{"1H7  5PV"}=" H71 5PV";
$resn_exch{"2H7  5PV"}=" H72 5PV"; $resn_exch{"1H8  5PV"}=" H81 5PV"; $resn_exch{"2H8  5PV"}=" H82 5PV";
$resn_exch{"1H9  5PV"}=" H91 5PV"; $resn_exch{"2H9  5PV"}=" H92 5PV"; $resn_exch{"1H10 5PV"}="H101 5PV";
$resn_exch{"2H10 5PV"}="H102 5PV"; $resn_exch{"1H5M 5PY"}=" H71 5PY"; $resn_exch{"2H5M 5PY"}=" H72 5PY";
$resn_exch{"3H5M 5PY"}=" H73 5PY"; $resn_exch{" H1* 5PY"}=" H1' 5PY"; $resn_exch{"1H2* 5PY"}=" H2' 5PY";
$resn_exch{"2H2* 5PY"}="H2'' 5PY"; $resn_exch{" H3* 5PY"}=" H3' 5PY"; $resn_exch{"H3T  5PY"}="HO3' 5PY";
$resn_exch{" H4* 5PY"}=" H4' 5PY"; $resn_exch{"1H5* 5PY"}=" H5' 5PY"; $resn_exch{"2H5* 5PY"}="H5'' 5PY";
$resn_exch{"2HOP 5PY"}="HOP2 5PY"; $resn_exch{"3HOP 5PY"}="HOP3 5PY"; $resn_exch{"1HN1 5QC"}="HN11 5QC";
$resn_exch{"2HN1 5QC"}="HN12 5QC"; $resn_exch{"1H11 5QC"}="H111 5QC"; $resn_exch{"2H11 5QC"}="H112 5QC";
$resn_exch{"1H20 5QC"}="H201 5QC"; $resn_exch{"2H20 5QC"}="H202 5QC"; $resn_exch{"1H23 5QC"}="H231 5QC";
$resn_exch{"2H23 5QC"}="H232 5QC"; $resn_exch{"1H28 5QC"}="H281 5QC"; $resn_exch{"2H28 5QC"}="H282 5QC";
$resn_exch{"1H29 5QC"}="H291 5QC"; $resn_exch{"2H29 5QC"}="H292 5QC"; $resn_exch{"1H31 5QC"}="H311 5QC";
$resn_exch{"2H31 5QC"}="H312 5QC"; $resn_exch{"1H18 5RM"}="H181 5RM"; $resn_exch{"2H18 5RM"}="H182 5RM";
$resn_exch{"3H18 5RM"}="H183 5RM"; $resn_exch{"1H11 5RM"}="H111 5RM"; $resn_exch{"2H11 5RM"}="H112 5RM";
$resn_exch{"3H11 5RM"}="H113 5RM"; $resn_exch{"1H14 5RM"}="H141 5RM"; $resn_exch{"2H14 5RM"}="H142 5RM";
$resn_exch{"1H15 5RM"}="H151 5RM"; $resn_exch{"2H15 5RM"}="H152 5RM"; $resn_exch{"1H16 5RM"}="H161 5RM";
$resn_exch{"2H16 5RM"}="H162 5RM"; $resn_exch{"3H16 5RM"}="H163 5RM"; $resn_exch{"1H2  5RP"}=" H21 5RP";
$resn_exch{"2H2  5RP"}=" H22 5RP"; $resn_exch{"1H7  5RP"}=" H71 5RP"; $resn_exch{"2H7  5RP"}=" H72 5RP";
$resn_exch{"1HR6 5SA"}="HR61 5SA"; $resn_exch{"2HR6 5SA"}="HR62 5SA"; $resn_exch{"1HV6 5SA"}="HV61 5SA";
$resn_exch{"2HV6 5SA"}="HV62 5SA"; $resn_exch{"3HV6 5SA"}="HV63 5SA"; $resn_exch{"1HQ6 5SA"}="HQ61 5SA";
$resn_exch{"2HQ6 5SA"}="HQ62 5SA"; $resn_exch{"1HW6 5SA"}="HW61 5SA"; $resn_exch{"2HW6 5SA"}="HW62 5SA";
$resn_exch{"1HY7 5SA"}="HY71 5SA"; $resn_exch{"2HY7 5SA"}="HY72 5SA"; $resn_exch{"1H12 5SC"}="H121 5SC";
$resn_exch{"2H12 5SC"}="H122 5SC"; $resn_exch{"1H25 5SC"}="H251 5SC"; $resn_exch{"2H25 5SC"}="H252 5SC";
$resn_exch{"3H25 5SC"}="H253 5SC"; $resn_exch{"1H1C 5SD"}="H1C1 5SD"; $resn_exch{"2H1C 5SD"}="H1C2 5SD";
$resn_exch{"1H2C 5SD"}="H2C1 5SD"; $resn_exch{"2H2C 5SD"}="H2C2 5SD"; $resn_exch{"1H4C 5SD"}="H4C1 5SD";
$resn_exch{"2H4C 5SD"}="H4C2 5SD"; $resn_exch{"1H6C 5SD"}="H6C1 5SD"; $resn_exch{"2H6C 5SD"}="H6C2 5SD";
$resn_exch{"1H7C 5SD"}="H7C1 5SD"; $resn_exch{"2H7C 5SD"}="H7C2 5SD"; $resn_exch{"1H11 5SD"}="H111 5SD";
$resn_exch{"2H11 5SD"}="H112 5SD"; $resn_exch{"1H12 5SD"}="H121 5SD"; $resn_exch{"2H12 5SD"}="H122 5SD";
$resn_exch{"1H15 5SD"}="H151 5SD"; $resn_exch{"2H15 5SD"}="H152 5SD"; $resn_exch{"1H16 5SD"}="H161 5SD";
$resn_exch{"2H16 5SD"}="H162 5SD"; $resn_exch{"1H18 5SD"}="H181 5SD"; $resn_exch{"2H18 5SD"}="H182 5SD";
$resn_exch{"3H18 5SD"}="H183 5SD"; $resn_exch{"1H19 5SD"}="H191 5SD"; $resn_exch{"2H19 5SD"}="H192 5SD";
$resn_exch{"3H19 5SD"}="H193 5SD"; $resn_exch{"1HAQ 5TN"}="HAQ1 5TN"; $resn_exch{"2HAQ 5TN"}="HAQ2 5TN";
$resn_exch{"1HAP 5TN"}="HAP1 5TN"; $resn_exch{"2HAP 5TN"}="HAP2 5TN"; $resn_exch{"1HAA 5TN"}="HAA1 5TN";
$resn_exch{"2HAA 5TN"}="HAA2 5TN"; $resn_exch{"3HAA 5TN"}="HAA3 5TN"; $resn_exch{"1HAB 5TN"}="HAB1 5TN";
$resn_exch{"2HAB 5TN"}="HAB2 5TN"; $resn_exch{"3HAB 5TN"}="HAB3 5TN"; $resn_exch{"1HAC 5TN"}="HAC1 5TN";
$resn_exch{"2HAC 5TN"}="HAC2 5TN"; $resn_exch{"3HAC 5TN"}="HAC3 5TN"; $resn_exch{"1HAR 5TN"}="HAR1 5TN";
$resn_exch{"2HAR 5TN"}="HAR2 5TN"; $resn_exch{"1HAS 5TN"}="HAS1 5TN"; $resn_exch{"2HAS 5TN"}="HAS2 5TN";
$resn_exch{"1HAD 5TN"}="HAD1 5TN"; $resn_exch{"2HAD 5TN"}="HAD2 5TN"; $resn_exch{"3HAD 5TN"}="HAD3 5TN";
$resn_exch{"1HAE 5TN"}="HAE1 5TN"; $resn_exch{"2HAE 5TN"}="HAE2 5TN"; $resn_exch{"3HAE 5TN"}="HAE3 5TN";
$resn_exch{" H1* 5UD"}=" H1' 5UD"; $resn_exch{" H2* 5UD"}=" H2' 5UD"; $resn_exch{"*HO2 5UD"}="HO2' 5UD";
$resn_exch{" H3* 5UD"}=" H3' 5UD"; $resn_exch{"*HO3 5UD"}="HO3' 5UD"; $resn_exch{" H4* 5UD"}=" H4' 5UD";
$resn_exch{"1H5* 5UD"}="H5'1 5UD"; $resn_exch{"2H5* 5UD"}="H5'2 5UD"; $resn_exch{"*HO5 5UD"}="HO5' 5UD";
$resn_exch{"1H1   5X"}="H11A  5X"; $resn_exch{"2H1   5X"}=" H12  5X"; $resn_exch{"1H4   5X"}=" H41  5X";
$resn_exch{"2H4   5X"}=" H42  5X"; $resn_exch{"3H4   5X"}=" H43  5X"; $resn_exch{"3HOP 5YL"}="HOP3 5YL";
$resn_exch{"2HOP 5YL"}="HOP2 5YL"; $resn_exch{"1H17 5YL"}="H171 5YL"; $resn_exch{"2H17 5YL"}="H172 5YL";
$resn_exch{"1H16 5YL"}="H161 5YL"; $resn_exch{"2H16 5YL"}="H162 5YL"; $resn_exch{"1H15 5YL"}="H151 5YL";
$resn_exch{"2H15 5YL"}="H152 5YL"; $resn_exch{"1H14 5YL"}="H141 5YL"; $resn_exch{"2H14 5YL"}="H142 5YL";
$resn_exch{"1H13 5YL"}="H131 5YL"; $resn_exch{"2H13 5YL"}="H132 5YL"; $resn_exch{"1H8  5YL"}=" H81 5YL";
$resn_exch{"2H8  5YL"}=" H82 5YL"; $resn_exch{"1H9  5YL"}=" H91 5YL"; $resn_exch{"1H10 5YL"}="H101 5YL";
$resn_exch{"OH10 5YL"}="H10O 5YL"; $resn_exch{"1H11 5YL"}="H111 5YL"; $resn_exch{"OH11 5YL"}="H11O 5YL";
$resn_exch{"1H12 5YL"}="H121 5YL"; $resn_exch{"2H12 5YL"}="H122 5YL"; $resn_exch{"OH12 5YL"}="H12O 5YL";
$resn_exch{"1H17 5Z5"}="H171 5Z5"; $resn_exch{"2H17 5Z5"}="H172 5Z5"; $resn_exch{"3H17 5Z5"}="H173 5Z5";
$resn_exch{"1H14 5Z5"}="H141 5Z5"; $resn_exch{"2H14 5Z5"}="H142 5Z5"; $resn_exch{"3H14 5Z5"}="H143 5Z5";
$resn_exch{"1H15 5Z5"}="H151 5Z5"; $resn_exch{"2H15 5Z5"}="H152 5Z5"; $resn_exch{"3H15 5Z5"}="H153 5Z5";
$resn_exch{"1H25 5ZA"}="H251 5ZA"; $resn_exch{"2H25 5ZA"}="H252 5ZA"; $resn_exch{"3H25 5ZA"}="H253 5ZA";
$resn_exch{"1H26 5ZA"}="H261 5ZA"; $resn_exch{"2H26 5ZA"}="H262 5ZA"; $resn_exch{"1H10 5ZA"}="H101 5ZA";
$resn_exch{"2H10 5ZA"}="H102 5ZA"; $resn_exch{"1H18 5ZA"}="H181 5ZA"; $resn_exch{"2H18 5ZA"}="H182 5ZA";
$resn_exch{"1H19 5ZA"}="H191 5ZA"; $resn_exch{"2H19 5ZA"}="H192 5ZA"; $resn_exch{"1H32 600"}="H321 600";
$resn_exch{"2H32 600"}="H322 600"; $resn_exch{"1H35 600"}="H351 600"; $resn_exch{"2H35 600"}="H352 600";
$resn_exch{"1H76 600"}="H761 600"; $resn_exch{"2H76 600"}="H762 600"; $resn_exch{"3H76 600"}="H763 600";
$resn_exch{"1H72 600"}="H721 600"; $resn_exch{"2H72 600"}="H722 600"; $resn_exch{"1H74 600"}="H741 600";
$resn_exch{"2H74 600"}="H742 600"; $resn_exch{"3H74 600"}="H743 600"; $resn_exch{"1H75 600"}="H751 600";
$resn_exch{"2H75 600"}="H752 600"; $resn_exch{"3H75 600"}="H753 600"; $resn_exch{"1H27 606"}="H271 606";
$resn_exch{"2H27 606"}="H272 606"; $resn_exch{"1H26 606"}="H261 606"; $resn_exch{"2H26 606"}="H262 606";
$resn_exch{"1H25 606"}="H251 606"; $resn_exch{"2H25 606"}="H252 606"; $resn_exch{"1H23 606"}="H231 606";
$resn_exch{"2H23 606"}="H232 606"; $resn_exch{"1H12 606"}="H121 606"; $resn_exch{"2H12 606"}="H122 606";
$resn_exch{"H3'' 607"}=" H3B 607"; $resn_exch{"H4'' 607"}=" H4B 607"; $resn_exch{"H5'' 607"}=" H5B 607";
$resn_exch{"H6'' 607"}=" H6B 607"; $resn_exch{"1H7  608"}=" H71 608"; $resn_exch{"2H7  608"}=" H72 608";
$resn_exch{"1H2  612"}=" H21 612"; $resn_exch{"1H3  612"}=" H31 612"; $resn_exch{"1H4  612"}=" H41 612";
$resn_exch{"1H5  612"}=" H51 612"; $resn_exch{"1H6  612"}=" H61 612"; $resn_exch{"1HF  612"}=" HF1 612";
$resn_exch{"1HH  612"}=" HH1 612"; $resn_exch{"2HH  612"}=" HH2 612"; $resn_exch{"3HH  612"}=" HH3 612";
$resn_exch{"1HI  612"}=" HI1 612"; $resn_exch{"2HI  612"}=" HI2 612"; $resn_exch{"3HI  612"}=" HI3 612";
$resn_exch{"1HN  612"}=" HN1 612"; $resn_exch{"1HA  612"}=" HA1 612"; $resn_exch{"2HA  612"}=" HA2 612";
$resn_exch{"1HC  612"}=" HC1 612"; $resn_exch{"2HC  612"}=" HC2 612"; $resn_exch{"1HD  612"}=" HD1 612";
$resn_exch{"2HD  612"}=" HD2 612"; $resn_exch{"3HD  612"}=" HD3 612"; $resn_exch{"CH30 616"}="H30C 616";
$resn_exch{"CH27 616"}="H27C 616"; $resn_exch{"1H17 616"}="H171 616"; $resn_exch{"2H17 616"}="H172 616";
$resn_exch{"3H17 616"}="H173 616"; $resn_exch{"1HC1 616"}="HC11 616"; $resn_exch{"2HC1 616"}="HC12 616";
$resn_exch{"3HC1 616"}="HC13 616"; $resn_exch{"CH29 616"}="H29C 616"; $resn_exch{"1HC3 616"}="HC31 616";
$resn_exch{"2HC3 616"}="HC32 616"; $resn_exch{"3HC3 616"}="HC33 616"; $resn_exch{"1H28 616"}="H281 616";
$resn_exch{"2H28 616"}="H282 616"; $resn_exch{"1H26 616"}="H261 616"; $resn_exch{"2H26 616"}="H262 616";
$resn_exch{"1HC6 616"}="HC61 616"; $resn_exch{"2HC6 616"}="HC62 616"; $resn_exch{"1H50 616"}="H501 616";
$resn_exch{"2H50 616"}="H502 616"; $resn_exch{"1H45 616"}="H451 616"; $resn_exch{"2H45 616"}="H452 616";
$resn_exch{"1H46 616"}="H461 616"; $resn_exch{"2H46 616"}="H462 616"; $resn_exch{"1H47 616"}="H471 616";
$resn_exch{"2H47 616"}="H472 616"; $resn_exch{"1H48 616"}="H481 616"; $resn_exch{"2H48 616"}="H482 616";
$resn_exch{"1H9  617"}=" H91 617"; $resn_exch{"2H9  617"}=" H92 617"; $resn_exch{"1H1  618"}="H11A 618";
$resn_exch{"2H1  618"}=" H12 618"; $resn_exch{"3H1  618"}="H13A 618"; $resn_exch{"1H4  618"}=" H41 618";
$resn_exch{"2H4  618"}=" H42 618"; $resn_exch{"3H4  618"}=" H43 618"; $resn_exch{"1H12 618"}="H121 618";
$resn_exch{"2H12 618"}="H122 618"; $resn_exch{"1H19 618"}="H191 618"; $resn_exch{"2H19 618"}="H192 618";
$resn_exch{"3H19 618"}="H193 618"; $resn_exch{"1H20 618"}="H201 618"; $resn_exch{"2H20 618"}="H202 618";
$resn_exch{"3H20 618"}="H203 618"; $resn_exch{"1H29 618"}="H291 618"; $resn_exch{"2H29 618"}="H292 618";
$resn_exch{"3H29 618"}="H293 618"; $resn_exch{"1H5  623"}=" H51 623"; $resn_exch{"2H5  623"}=" H52 623";
$resn_exch{"3H5  623"}=" H53 623"; $resn_exch{"1H22 623"}="H221 623"; $resn_exch{"2H22 623"}="H222 623";
$resn_exch{"1H12 623"}="H121 623"; $resn_exch{"2H12 623"}="H122 623"; $resn_exch{"1H06 624"}="H061 624";
$resn_exch{"2H06 624"}="H062 624"; $resn_exch{"1H07 624"}="H071 624"; $resn_exch{"2H07 624"}="H072 624";
$resn_exch{"1H08 624"}="H081 624"; $resn_exch{"2H08 624"}="H082 624"; $resn_exch{"1H11 624"}="H111 624";
$resn_exch{"2H11 624"}="H112 624"; $resn_exch{"1H20 624"}="H201 624"; $resn_exch{"2H20 624"}="H202 624";
$resn_exch{"1H31 624"}="H311 624"; $resn_exch{"2H31 624"}="H312 624"; $resn_exch{"3H31 624"}="H313 624";
$resn_exch{"1H32 624"}="H321 624"; $resn_exch{"2H32 624"}="H322 624"; $resn_exch{"3H32 624"}="H323 624";
$resn_exch{"1H32 626"}="H321 626"; $resn_exch{"2H32 626"}="H322 626"; $resn_exch{"1H20 626"}="H201 626";
$resn_exch{"2H20 626"}="H202 626"; $resn_exch{"1H24 626"}="H241 626"; $resn_exch{"2H24 626"}="H242 626";
$resn_exch{"1H21 626"}="H211 626"; $resn_exch{"2H21 626"}="H212 626"; $resn_exch{"1H23 626"}="H231 626";
$resn_exch{"2H23 626"}="H232 626"; $resn_exch{"1H25 626"}="H251 626"; $resn_exch{"2H25 626"}="H252 626";
$resn_exch{"3H25 626"}="H253 626"; $resn_exch{"1H18 627"}="H181 627"; $resn_exch{"2H18 627"}="H182 627";
$resn_exch{"1H22 627"}="H221 627"; $resn_exch{"2H22 627"}="H222 627"; $resn_exch{"1H19 627"}="H191 627";
$resn_exch{"2H19 627"}="H192 627"; $resn_exch{"1H21 627"}="H211 627"; $resn_exch{"2H21 627"}="H212 627";
$resn_exch{"1H23 627"}="H231 627"; $resn_exch{"2H23 627"}="H232 627"; $resn_exch{"3H23 627"}="H233 627";
$resn_exch{"1H36 627"}="H361 627"; $resn_exch{"2H36 627"}="H362 627"; $resn_exch{"3H36 627"}="H363 627";
$resn_exch{"1HN1 628"}="HN11 628"; $resn_exch{"2HN1 628"}="HN12 628"; $resn_exch{"1H14 629"}="H141 629";
$resn_exch{"2H14 629"}="H142 629"; $resn_exch{"1H16 629"}="H161 629"; $resn_exch{"2H16 629"}="H162 629";
$resn_exch{"1H4  629"}=" H41 629"; $resn_exch{"2H4  629"}=" H42 629"; $resn_exch{"1H30 629"}="H301 629";
$resn_exch{"2H30 629"}="H302 629"; $resn_exch{"1H2  62A"}="H21A 62A"; $resn_exch{"2H2  62A"}="H22A 62A";
$resn_exch{"1H3  62A"}=" H31 62A"; $resn_exch{"2H3  62A"}=" H32 62A"; $resn_exch{"1H4  62A"}=" H41 62A";
$resn_exch{"2H4  62A"}=" H42 62A"; $resn_exch{"1HN7 62A"}="HN71 62A"; $resn_exch{"2HN7 62A"}="HN72 62A";
$resn_exch{"1H11 62A"}="H111 62A"; $resn_exch{"2H11 62A"}="H112 62A"; $resn_exch{"1H18 62A"}="H181 62A";
$resn_exch{"2H18 62A"}="H182 62A"; $resn_exch{"1H25 62A"}="H251 62A"; $resn_exch{"2H25 62A"}="H252 62A";
$resn_exch{"3H25 62A"}="H253 62A"; $resn_exch{"1H20 62P"}="H201 62P"; $resn_exch{"2H20 62P"}="H202 62P";
$resn_exch{"3H20 62P"}="H203 62P"; $resn_exch{"1H6  632"}=" H61 632"; $resn_exch{"2H6  632"}=" H62 632";
$resn_exch{"1H7  632"}=" H71 632"; $resn_exch{"2H7  632"}=" H72 632"; $resn_exch{"1H8  632"}=" H81 632";
$resn_exch{"2H8  632"}=" H82 632"; $resn_exch{"1H11 632"}="H111 632"; $resn_exch{"2H11 632"}="H112 632";
$resn_exch{"1H15 632"}="H151 632"; $resn_exch{"2H15 632"}="H152 632"; $resn_exch{"1H22 632"}="H221 632";
$resn_exch{"2H22 632"}="H222 632"; $resn_exch{"1H32 632"}="H321 632"; $resn_exch{"2H32 632"}="H322 632";
$resn_exch{"3H32 632"}="H323 632"; $resn_exch{"1H2  638"}="H21A 638"; $resn_exch{"2H2  638"}=" H22 638";
$resn_exch{"1H7  638"}="H71A 638"; $resn_exch{"2H7  638"}=" H72 638"; $resn_exch{"1H31 638"}="H311 638";
$resn_exch{"2H31 638"}="H312 638"; $resn_exch{"1H61 638"}="H611 638"; $resn_exch{"2H61 638"}="H612 638";
$resn_exch{"1H10 641"}="H101 641"; $resn_exch{"2H10 641"}="H102 641"; $resn_exch{"1H8  641"}=" H81 641";
$resn_exch{"2H8  641"}=" H82 641"; $resn_exch{"1H4  641"}=" H41 641"; $resn_exch{"2H4  641"}=" H42 641";
$resn_exch{"1H5  641"}=" H51 641"; $resn_exch{"2H5  641"}=" H52 641"; $resn_exch{"1H6  641"}=" H61 641";
$resn_exch{"2H6  641"}=" H62 641"; $resn_exch{"1H10 645"}="H101 645"; $resn_exch{"2H10 645"}="H102 645";
$resn_exch{"3H10 645"}="H103 645"; $resn_exch{"1H13 645"}="H131 645"; $resn_exch{"2H13 645"}="H132 645";
$resn_exch{"3H13 645"}="H133 645"; $resn_exch{"1H14 645"}="H141 645"; $resn_exch{"2H14 645"}="H142 645";
$resn_exch{"3H14 645"}="H143 645"; $resn_exch{"1H15 645"}="H151 645"; $resn_exch{"2H15 645"}="H152 645";
$resn_exch{"3H15 645"}="H153 645"; $resn_exch{"2HOP 64T"}="HOP2 64T"; $resn_exch{"1H5* 64T"}=" H5' 64T";
$resn_exch{"2H5* 64T"}="H5'' 64T"; $resn_exch{" H4* 64T"}=" H4' 64T"; $resn_exch{" H3* 64T"}=" H3' 64T";
$resn_exch{"H3T  64T"}="HO3' 64T"; $resn_exch{"1H2* 64T"}=" H2' 64T"; $resn_exch{"2H2* 64T"}="H2'' 64T";
$resn_exch{" H1* 64T"}=" H1' 64T"; $resn_exch{"1H5M 64T"}=" H71 64T"; $resn_exch{"2H5M 64T"}=" H72 64T";
$resn_exch{"3H5M 64T"}=" H73 64T"; $resn_exch{"1H6  64T"}=" H61 64T"; $resn_exch{"2H6  64T"}=" H62 64T";
$resn_exch{"3HOP 64T"}="HOP3 64T"; $resn_exch{"H2'' 653"}=" H2B 653"; $resn_exch{"H3'' 653"}=" H3B 653";
$resn_exch{"H4'' 653"}=" H4B 653"; $resn_exch{"H5'' 653"}=" H5B 653"; $resn_exch{"H6'' 653"}=" H6B 653";
$resn_exch{"CH3T 654"}="H3TC 654"; $resn_exch{"CH4T 654"}="H4TC 654"; $resn_exch{"CH5T 654"}="H5TC 654";
$resn_exch{"CH3I 654"}="H3IC 654"; $resn_exch{"CH5I 654"}="H5IC 654"; $resn_exch{"1HC1 654"}="HC11 654";
$resn_exch{"2HC1 654"}="HC12 654"; $resn_exch{"1HC8 654"}="HC81 654"; $resn_exch{"2HC8 654"}="HC82 654";
$resn_exch{"3HC8 654"}="HC83 654"; $resn_exch{"1H2' 65B"}="H2'1 65B"; $resn_exch{"2H2' 65B"}="H2'2 65B";
$resn_exch{"3H2' 65B"}="H2'3 65B"; $resn_exch{"1H6' 65B"}="H6'1 65B"; $resn_exch{"2H6' 65B"}="H6'2 65B";
$resn_exch{"3H6' 65B"}="H6'3 65B"; $resn_exch{"1H18 65B"}="H181 65B"; $resn_exch{"2H18 65B"}="H182 65B";
$resn_exch{"1H34 663"}="H341 663"; $resn_exch{"2H34 663"}="H342 663"; $resn_exch{"3H34 663"}="H343 663";
$resn_exch{"1H28 663"}="H281 663"; $resn_exch{"2H28 663"}="H282 663"; $resn_exch{"1H18 663"}="H181 663";
$resn_exch{"2H18 663"}="H182 663"; $resn_exch{"3H18 663"}="H183 663"; $resn_exch{"1H4  665"}=" H41 665";
$resn_exch{"2H4  665"}=" H42 665"; $resn_exch{"1H8  665"}=" H81 665"; $resn_exch{"2H8  665"}=" H82 665";
$resn_exch{"1H10 665"}="H101 665"; $resn_exch{"2H10 665"}="H102 665"; $resn_exch{"1H11 666"}="H111 666";
$resn_exch{"2H11 666"}="H112 666"; $resn_exch{"1H16 666"}="H161 666"; $resn_exch{"2H16 666"}="H162 666";
$resn_exch{"1H17 666"}="H171 666"; $resn_exch{"2H17 666"}="H172 666"; $resn_exch{"1H18 666"}="H181 666";
$resn_exch{"2H18 666"}="H182 666"; $resn_exch{"1H41 666"}="H411 666"; $resn_exch{"2H41 666"}="H412 666";
$resn_exch{"1H47 666"}="H471 666"; $resn_exch{"2H47 666"}="H472 666"; $resn_exch{"3H47 666"}="H473 666";
$resn_exch{"1HN1 667"}="HN11 667"; $resn_exch{"2HN1 667"}="HN12 667"; $resn_exch{"1H10 667"}="H101 667";
$resn_exch{"2H10 667"}="H102 667"; $resn_exch{"1H11 667"}="H111 667"; $resn_exch{"2H11 667"}="H112 667";
$resn_exch{"1H12 667"}="H121 667"; $resn_exch{"2H12 667"}="H122 667"; $resn_exch{"1H13 667"}="H131 667";
$resn_exch{"2H13 667"}="H132 667"; $resn_exch{"1H14 667"}="H141 667"; $resn_exch{"2H14 667"}="H142 667";
$resn_exch{"1H04 669"}="H041 669"; $resn_exch{"2H04 669"}="H042 669"; $resn_exch{"1H05 669"}="H051 669";
$resn_exch{"2H05 669"}="H052 669"; $resn_exch{"1H06 669"}="H061 669"; $resn_exch{"2H06 669"}="H062 669";
$resn_exch{"1H07 669"}="H071 669"; $resn_exch{"2H07 669"}="H072 669"; $resn_exch{"1H08 669"}="H081 669";
$resn_exch{"2H08 669"}="H082 669"; $resn_exch{"1H16 669"}="H161 669"; $resn_exch{"2H16 669"}="H162 669";
$resn_exch{"1H4  672"}="H41A 672"; $resn_exch{"2H4  672"}=" H42 672"; $resn_exch{"3H4  672"}=" H43 672";
$resn_exch{"1H15 672"}="H151 672"; $resn_exch{"2H15 672"}="H152 672"; $resn_exch{"3H15 672"}="H153 672";
$resn_exch{"1H14 672"}="H141 672"; $resn_exch{"2H14 672"}="H142 672"; $resn_exch{"3H14 672"}="H143 672";
$resn_exch{"1H23 672"}="H231 672"; $resn_exch{"2H23 672"}="H232 672"; $resn_exch{"1H32 672"}="H321 672";
$resn_exch{"2H32 672"}="H322 672"; $resn_exch{"1H38 672"}="H381 672"; $resn_exch{"2H38 672"}="H382 672";
$resn_exch{"1H37 672"}="H371 672"; $resn_exch{"2H37 672"}="H372 672"; $resn_exch{"1H46 672"}="H461 672";
$resn_exch{"2H46 672"}="H462 672"; $resn_exch{"1H45 672"}="H451 672"; $resn_exch{"2H45 672"}="H452 672";
$resn_exch{"1H49 672"}="H491 672"; $resn_exch{"2H49 672"}="H492 672"; $resn_exch{"1H43 672"}="H431 672";
$resn_exch{"2H43 672"}="H432 672"; $resn_exch{"1H42 672"}="H421 672"; $resn_exch{"2H42 672"}="H422 672";
$resn_exch{"1H35 672"}="H351 672"; $resn_exch{"2H35 672"}="H352 672"; $resn_exch{"1H34 672"}="H341 672";
$resn_exch{"2H34 672"}="H342 672"; $resn_exch{"1H12 672"}="H121 672"; $resn_exch{"2H12 672"}="H122 672";
$resn_exch{"3H12 672"}="H123 672"; $resn_exch{"1H11 672"}="H111 672"; $resn_exch{"2H11 672"}="H112 672";
$resn_exch{"3H11 672"}="H113 672"; $resn_exch{"1H1  672"}=" H11 672"; $resn_exch{"2H1  672"}=" H12 672";
$resn_exch{"3H1  672"}="H13A 672"; $resn_exch{"1H19 675"}="H191 675"; $resn_exch{"2H19 675"}="H192 675";
$resn_exch{"1H5  680"}=" H51 680"; $resn_exch{"2H5  680"}=" H52 680"; $resn_exch{"3H5  680"}=" H53 680";
$resn_exch{"1HN5 680"}="HN51 680"; $resn_exch{"2HN5 680"}="HN52 680"; $resn_exch{"1H1  681"}=" H11 681";
$resn_exch{"1H5  681"}=" H51 681"; $resn_exch{"2H5  681"}=" H52 681"; $resn_exch{"3H5  681"}=" H53 681";
$resn_exch{"1H6  681"}=" H61 681"; $resn_exch{"2H6  681"}=" H62 681"; $resn_exch{"3H6  681"}=" H63 681";
$resn_exch{"1H7  681"}=" H71 681"; $resn_exch{"2H7  681"}=" H72 681"; $resn_exch{"3H7  681"}=" H73 681";
$resn_exch{"1HVA 681"}="HVA1 681"; $resn_exch{"1HVB 681"}="HVB1 681"; $resn_exch{"1HV1 681"}="HV11 681";
$resn_exch{"2HV1 681"}="HV12 681"; $resn_exch{"3HV1 681"}="HV13 681"; $resn_exch{"1HV2 681"}="HV21 681";
$resn_exch{"2HV2 681"}="HV22 681"; $resn_exch{"3HV2 681"}="HV23 681"; $resn_exch{"1HNV 681"}="HNV1 681";
$resn_exch{"1HAG 681"}="HAG1 681"; $resn_exch{"2HAG 681"}="HAG2 681"; $resn_exch{"1H2X 681"}="H2X1 681";
$resn_exch{"1H3X 681"}="H3X1 681"; $resn_exch{"2H3X 681"}="H3X2 681"; $resn_exch{"1H6X 681"}="H6X1 681";
$resn_exch{"1H7X 681"}="H7X1 681"; $resn_exch{"1H8X 681"}="H8X1 681"; $resn_exch{"1H9X 681"}="H9X1 681";
$resn_exch{"1H0X 681"}="H0X1 681"; $resn_exch{"1H6  682"}=" H61 682"; $resn_exch{"2H6  682"}=" H62 682";
$resn_exch{"1H7  682"}=" H71 682"; $resn_exch{"2H7  682"}=" H72 682"; $resn_exch{"1H8  682"}=" H81 682";
$resn_exch{"2H8  682"}=" H82 682"; $resn_exch{"1H12 682"}="H121 682"; $resn_exch{"2H12 682"}="H122 682";
$resn_exch{"1H21 682"}="H211 682"; $resn_exch{"2H21 682"}="H212 682"; $resn_exch{"3H21 682"}="H213 682";
$resn_exch{"1H22 682"}="H221 682"; $resn_exch{"2H22 682"}="H222 682"; $resn_exch{"3H22 682"}="H223 682";
$resn_exch{"1H24 682"}="H241 682"; $resn_exch{"2H24 682"}="H242 682"; $resn_exch{"1H11 687"}="H111 687";
$resn_exch{"2H11 687"}="H112 687"; $resn_exch{"1H17 687"}="H171 687"; $resn_exch{"2H17 687"}="H172 687";
$resn_exch{"3H17 687"}="H173 687"; $resn_exch{"1H21 687"}="H211 687"; $resn_exch{"2H21 687"}="H212 687";
$resn_exch{"1H22 687"}="H221 687"; $resn_exch{"2H22 687"}="H222 687"; $resn_exch{"1H23 687"}="H231 687";
$resn_exch{"2H23 687"}="H232 687"; $resn_exch{"1H24 687"}="H241 687"; $resn_exch{"2H24 687"}="H242 687";
$resn_exch{"1H27 687"}="H271 687"; $resn_exch{"2H27 687"}="H272 687"; $resn_exch{"1H12 688"}="H121 688";
$resn_exch{"2H12 688"}="H122 688"; $resn_exch{"1HAN 689"}="HAN1 689"; $resn_exch{"2HAN 689"}="HAN2 689";
$resn_exch{"3HAN 689"}="HAN3 689"; $resn_exch{"1HAR 689"}="HAR1 689"; $resn_exch{"2HAR 689"}="HAR2 689";
$resn_exch{"3HAR 689"}="HAR3 689"; $resn_exch{"1HAS 689"}="HAS1 689"; $resn_exch{"2HAS 689"}="HAS2 689";
$resn_exch{"3HAS 689"}="HAS3 689"; $resn_exch{"1HAT 689"}="HAT1 689"; $resn_exch{"2HAT 689"}="HAT2 689";
$resn_exch{"1HAM 689"}="HAM1 689"; $resn_exch{"2HAM 689"}="HAM2 689"; $resn_exch{"H3'' 693"}=" H3B 693";
$resn_exch{"H4'' 693"}=" H4B 693"; $resn_exch{"H5'' 693"}=" H5B 693"; $resn_exch{"H6'' 693"}=" H6B 693";
$resn_exch{"1H12 694"}="H121 694"; $resn_exch{"2H12 694"}="H122 694"; $resn_exch{"H2'' 696"}=" H2B 696";
$resn_exch{"H3'' 696"}=" H3B 696"; $resn_exch{"H4'' 696"}=" H4B 696"; $resn_exch{"H5'' 696"}=" H5B 696";
$resn_exch{"H6'' 696"}=" H6B 696"; $resn_exch{"1H16 698"}="H161 698"; $resn_exch{"2H16 698"}="H162 698";
$resn_exch{"1H33 698"}="H331 698"; $resn_exch{"2H33 698"}="H332 698"; $resn_exch{"1H40 698"}="H401 698";
$resn_exch{"2H40 698"}="H402 698"; $resn_exch{"3H40 698"}="H403 698"; $resn_exch{"1H44 698"}="H441 698";
$resn_exch{"2H44 698"}="H442 698"; $resn_exch{"3H44 698"}="H443 698"; $resn_exch{"1H9C 698"}="H9C1 698";
$resn_exch{"2H9C 698"}="H9C2 698"; $resn_exch{"3H9C 698"}="H9C3 698"; $resn_exch{"1H8C 698"}="H8C1 698";
$resn_exch{"2H8C 698"}="H8C2 698"; $resn_exch{"3H8C 698"}="H8C3 698"; $resn_exch{"1H7C 698"}="H7C1 698";
$resn_exch{"2H7C 698"}="H7C2 698"; $resn_exch{"3H7C 698"}="H7C3 698"; $resn_exch{"1H33 699"}="H331 699";
$resn_exch{"2H33 699"}="H332 699"; $resn_exch{"1H16 699"}="H161 699"; $resn_exch{"2H16 699"}="H162 699";
$resn_exch{"1H44 699"}="H441 699"; $resn_exch{"2H44 699"}="H442 699"; $resn_exch{"3H44 699"}="H443 699";
$resn_exch{"1H40 699"}="H401 699"; $resn_exch{"2H40 699"}="H402 699"; $resn_exch{"3H40 699"}="H403 699";
$resn_exch{"'HO2 6AB"}="HO2' 6AB"; $resn_exch{"1HN6 6AB"}="HN61 6AB"; $resn_exch{"2HN6 6AB"}="HN62 6AB";
$resn_exch{"1HN2 6AP"}="HN21 6AP"; $resn_exch{"2HN2 6AP"}="HN22 6AP"; $resn_exch{"1HN6 6AP"}="HN61 6AP";
$resn_exch{"2HN6 6AP"}="HN62 6AP"; $resn_exch{"1H19 6C3"}="H191 6C3"; $resn_exch{"2H19 6C3"}="H192 6C3";
$resn_exch{"3H19 6C3"}="H193 6C3"; $resn_exch{"1H22 6CA"}="H221 6CA"; $resn_exch{"2H22 6CA"}="H222 6CA";
$resn_exch{"1H21 6CA"}="H211 6CA"; $resn_exch{"2H21 6CA"}="H212 6CA"; $resn_exch{"1H20 6CA"}="H201 6CA";
$resn_exch{"2H20 6CA"}="H202 6CA"; $resn_exch{"1H19 6CA"}="H191 6CA"; $resn_exch{"2H19 6CA"}="H192 6CA";
$resn_exch{"1H18 6CA"}="H181 6CA"; $resn_exch{"2H18 6CA"}="H182 6CA"; $resn_exch{"1H17 6CA"}="H171 6CA";
$resn_exch{"2H17 6CA"}="H172 6CA"; $resn_exch{" HN2 6CL"}=" H2  6CL"; $resn_exch{" HB1 6CL"}=" HB2 6CL";
$resn_exch{" HB2 6CL"}=" HB3 6CL"; $resn_exch{" HG1 6CL"}=" HG2 6CL"; $resn_exch{" HG2 6CL"}=" HG3 6CL";
$resn_exch{" HD1 6CL"}=" HD2 6CL"; $resn_exch{" HD2 6CL"}=" HD3 6CL"; $resn_exch{"1H1  6CM"}=" H11 6CM";
$resn_exch{"2H1  6CM"}=" H12 6CM"; $resn_exch{"1H6  6CM"}=" H61 6CM"; $resn_exch{"2H6  6CM"}=" H62 6CM";
$resn_exch{"1H7  6CM"}=" H71 6CM"; $resn_exch{"2H7  6CM"}=" H72 6CM"; $resn_exch{"1H8  6CM"}=" H81 6CM";
$resn_exch{"2H8  6CM"}=" H82 6CM"; $resn_exch{"1H9  6CM"}=" H91 6CM"; $resn_exch{"2H9  6CM"}=" H92 6CM";
$resn_exch{"1H10 6CM"}="H101 6CM"; $resn_exch{"2H10 6CM"}="H102 6CM"; $resn_exch{"1H11 6CM"}="H111 6CM";
$resn_exch{"2H11 6CM"}="H112 6CM"; $resn_exch{"1H12 6CM"}="H121 6CM"; $resn_exch{"2H12 6CM"}="H122 6CM";
$resn_exch{"1H13 6CM"}="H131 6CM"; $resn_exch{"2H13 6CM"}="H132 6CM"; $resn_exch{"1H14 6CM"}="H141 6CM";
$resn_exch{"2H14 6CM"}="H142 6CM"; $resn_exch{"1H15 6CM"}="H151 6CM"; $resn_exch{"2H15 6CM"}="H152 6CM";
$resn_exch{"1H16 6CM"}="H161 6CM"; $resn_exch{"2H16 6CM"}="H162 6CM"; $resn_exch{"1H17 6CM"}="H171 6CM";
$resn_exch{"2H17 6CM"}="H172 6CM"; $resn_exch{"1H18 6CM"}="H181 6CM"; $resn_exch{"2H18 6CM"}="H182 6CM";
$resn_exch{"3H18 6CM"}="H183 6CM"; $resn_exch{"1H20 6CM"}="H201 6CM"; $resn_exch{"2H20 6CM"}="H202 6CM";
$resn_exch{"1H21 6CM"}="H211 6CM"; $resn_exch{"2H21 6CM"}="H212 6CM"; $resn_exch{"1H22 6CM"}="H221 6CM";
$resn_exch{"2H22 6CM"}="H222 6CM"; $resn_exch{"1H23 6CM"}="H231 6CM"; $resn_exch{"2H23 6CM"}="H232 6CM";
$resn_exch{"1H24 6CM"}="H241 6CM"; $resn_exch{"2H24 6CM"}="H242 6CM"; $resn_exch{"3H24 6CM"}="H243 6CM";
$resn_exch{"1H10 6CP"}="H101 6CP"; $resn_exch{"2H10 6CP"}="H102 6CP"; $resn_exch{"1H12 6CP"}="H121 6CP";
$resn_exch{"2H12 6CP"}="H122 6CP"; $resn_exch{"1H13 6CP"}="H131 6CP"; $resn_exch{"2H13 6CP"}="H132 6CP";
$resn_exch{"1H14 6CP"}="H141 6CP"; $resn_exch{"2H14 6CP"}="H142 6CP"; $resn_exch{"1H15 6CP"}="H151 6CP";
$resn_exch{"2H15 6CP"}="H152 6CP"; $resn_exch{"1H16 6CP"}="H161 6CP"; $resn_exch{"2H16 6CP"}="H162 6CP";
$resn_exch{"1HAA 6CS"}="HAA1 6CS"; $resn_exch{"2HAA 6CS"}="HAA2 6CS"; $resn_exch{"3HAA 6CS"}="HAA3 6CS";
$resn_exch{"1HG2 6CS"}="HG21 6CS"; $resn_exch{"2HG2 6CS"}="HG22 6CS"; $resn_exch{"1H5* 6CT"}=" H5' 6CT";
$resn_exch{"2H5* 6CT"}="H5'' 6CT"; $resn_exch{" H4* 6CT"}=" H4' 6CT"; $resn_exch{" H3* 6CT"}=" H3' 6CT";
$resn_exch{"*HO3 6CT"}="HO3' 6CT"; $resn_exch{"1H2* 6CT"}=" H2' 6CT"; $resn_exch{"2H2* 6CT"}="H2'' 6CT";
$resn_exch{" H1* 6CT"}=" H1' 6CT"; $resn_exch{"1H5M 6CT"}=" H71 6CT"; $resn_exch{"2H5M 6CT"}=" H72 6CT";
$resn_exch{"3H5M 6CT"}=" H73 6CT"; $resn_exch{" H*  6CT"}=" H'  6CT"; $resn_exch{"1HM* 6CT"}="HM'1 6CT";
$resn_exch{"2HM* 6CT"}="HM'2 6CT"; $resn_exch{"3HM* 6CT"}="HM'3 6CT"; $resn_exch{" O3P 6CT"}=" OP3 6CT";
$resn_exch{"3HOP 6CT"}="HOP3 6CT"; $resn_exch{"1HN  6CW"}=" H   6CW"; $resn_exch{"2HN  6CW"}=" H2  6CW";
$resn_exch{"1HB  6CW"}=" HB2 6CW"; $resn_exch{"2HB  6CW"}=" HB3 6CW"; $resn_exch{"1H16 6DE"}="H161 6DE";
$resn_exch{"2H16 6DE"}="H162 6DE"; $resn_exch{"1H17 6DE"}="H171 6DE"; $resn_exch{"2H17 6DE"}="H172 6DE";
$resn_exch{"3H17 6DE"}="H173 6DE"; $resn_exch{"1H19 6DE"}="H191 6DE"; $resn_exch{"2H19 6DE"}="H192 6DE";
$resn_exch{"3H19 6DE"}="H193 6DE"; $resn_exch{"1H11 6DE"}="H111 6DE"; $resn_exch{"2H11 6DE"}="H112 6DE";
$resn_exch{"3H11 6DE"}="H113 6DE"; $resn_exch{"1H26 6EA"}="H261 6EA"; $resn_exch{"2H26 6EA"}="H262 6EA";
$resn_exch{"1H30 6EA"}="H301 6EA"; $resn_exch{"2H30 6EA"}="H302 6EA"; $resn_exch{"1H31 6EA"}="H311 6EA";
$resn_exch{"2H31 6EA"}="H312 6EA"; $resn_exch{"1HM7 6FA"}="HM71 6FA"; $resn_exch{"2HM7 6FA"}="HM72 6FA";
$resn_exch{"3HM7 6FA"}="HM73 6FA"; $resn_exch{"1HM8 6FA"}="HM81 6FA"; $resn_exch{"2HM8 6FA"}="HM82 6FA";
$resn_exch{"3HM8 6FA"}="HM83 6FA"; $resn_exch{"1H1* 6FA"}="H1'1 6FA"; $resn_exch{"2H1* 6FA"}="H1'2 6FA";
$resn_exch{" H2* 6FA"}=" H2' 6FA"; $resn_exch{"*HO2 6FA"}="HO2' 6FA"; $resn_exch{" H3* 6FA"}=" H3' 6FA";
$resn_exch{"*HO3 6FA"}="HO3' 6FA"; $resn_exch{" H4* 6FA"}=" H4' 6FA"; $resn_exch{"*HO4 6FA"}="HO4' 6FA";
$resn_exch{"1H5* 6FA"}="H5'1 6FA"; $resn_exch{"2H5* 6FA"}="H5'2 6FA"; $resn_exch{"2HOP 6FA"}="HOP2 6FA";
$resn_exch{"2HOA 6FA"}="HOA2 6FA"; $resn_exch{"AH51 6FA"}="H51A 6FA"; $resn_exch{"AH52 6FA"}="H52A 6FA";
$resn_exch{"AH4* 6FA"}=" H4B 6FA"; $resn_exch{"AH3* 6FA"}=" H3B 6FA"; $resn_exch{"AHO3 6FA"}="HO3A 6FA";
$resn_exch{"AH2* 6FA"}=" H2B 6FA"; $resn_exch{"AHO2 6FA"}="HO2A 6FA"; $resn_exch{"AH1* 6FA"}=" H1B 6FA";
$resn_exch{" AH8 6FA"}=" H8A 6FA"; $resn_exch{"AH61 6FA"}="H61A 6FA"; $resn_exch{"AH62 6FA"}="H62A 6FA";
$resn_exch{" AH2 6FA"}=" H2A 6FA"; $resn_exch{"1H6  6GP"}=" H61 6GP"; $resn_exch{"2H6  6GP"}=" H62 6GP";
$resn_exch{"1H9  6GP"}=" H91 6GP"; $resn_exch{"2H9  6GP"}=" H92 6GP"; $resn_exch{"3H9  6GP"}=" H93 6GP";
$resn_exch{"2HOP 6HA"}="HOP2 6HA"; $resn_exch{"3HOP 6HA"}="HOP3 6HA"; $resn_exch{"1H5* 6HA"}=" H5' 6HA";
$resn_exch{"2H5* 6HA"}="H5'' 6HA"; $resn_exch{" H4* 6HA"}=" H4' 6HA"; $resn_exch{" H3* 6HA"}=" H3' 6HA";
$resn_exch{"HO3* 6HA"}="HO3' 6HA"; $resn_exch{"1H2* 6HA"}=" H2' 6HA"; $resn_exch{"2H2* 6HA"}="H2'' 6HA";
$resn_exch{" H1* 6HA"}=" H1' 6HA"; $resn_exch{"1H6* 6HA"}="H6'1 6HA"; $resn_exch{"2H6* 6HA"}="H6'2 6HA";
$resn_exch{"1H6  6HA"}=" H61 6HA"; $resn_exch{"2H6  6HA"}=" H62 6HA"; $resn_exch{"2HOP 6HC"}="HOP2 6HC";
$resn_exch{"3HOP 6HC"}="HOP3 6HC"; $resn_exch{"1H5* 6HC"}=" H5' 6HC"; $resn_exch{"2H5* 6HC"}="H5'' 6HC";
$resn_exch{" H4* 6HC"}=" H4' 6HC"; $resn_exch{" H3* 6HC"}=" H3' 6HC"; $resn_exch{"HO3* 6HC"}="HO3' 6HC";
$resn_exch{"1H2* 6HC"}=" H2' 6HC"; $resn_exch{"2H2* 6HC"}="H2'' 6HC"; $resn_exch{" H1* 6HC"}=" H1' 6HC";
$resn_exch{"1H6* 6HC"}="H6'1 6HC"; $resn_exch{"2H6* 6HC"}="H6'2 6HC"; $resn_exch{"1H4  6HC"}=" H41 6HC";
$resn_exch{"2H4  6HC"}=" H42 6HC"; $resn_exch{"1HMA 6HE"}="HMA1 6HE"; $resn_exch{"2HMA 6HE"}="HMA2 6HE";
$resn_exch{"3HMA 6HE"}="HMA3 6HE"; $resn_exch{"1HAA 6HE"}="HAA1 6HE"; $resn_exch{"2HAA 6HE"}="HAA2 6HE";
$resn_exch{"1HBA 6HE"}="HBA1 6HE"; $resn_exch{"2HBA 6HE"}="HBA2 6HE"; $resn_exch{"1HMB 6HE"}="HMB1 6HE";
$resn_exch{"2HMB 6HE"}="HMB2 6HE"; $resn_exch{"3HMB 6HE"}="HMB3 6HE"; $resn_exch{"1HBB 6HE"}="HBB1 6HE";
$resn_exch{"2HBB 6HE"}="HBB2 6HE"; $resn_exch{"1HMC 6HE"}="HMC1 6HE"; $resn_exch{"2HMC 6HE"}="HMC2 6HE";
$resn_exch{"3HMC 6HE"}="HMC3 6HE"; $resn_exch{"1HBC 6HE"}="HBC1 6HE"; $resn_exch{"2HBC 6HE"}="HBC2 6HE";
$resn_exch{"1HMD 6HE"}="HMD1 6HE"; $resn_exch{"2HMD 6HE"}="HMD2 6HE"; $resn_exch{"3HMD 6HE"}="HMD3 6HE";
$resn_exch{"1HAD 6HE"}="HAD1 6HE"; $resn_exch{"2HAD 6HE"}="HAD2 6HE"; $resn_exch{"3HAD 6HE"}="HAD3 6HE";
$resn_exch{"2HOP 6HG"}="HOP2 6HG"; $resn_exch{"3HOP 6HG"}="HOP3 6HG"; $resn_exch{"1H5* 6HG"}=" H5' 6HG";
$resn_exch{"2H5* 6HG"}="H5'' 6HG"; $resn_exch{" H4* 6HG"}=" H4' 6HG"; $resn_exch{" H3* 6HG"}=" H3' 6HG";
$resn_exch{"HO3* 6HG"}="HO3' 6HG"; $resn_exch{"1H2* 6HG"}=" H2' 6HG"; $resn_exch{"2H2* 6HG"}="H2'' 6HG";
$resn_exch{" H1* 6HG"}=" H1' 6HG"; $resn_exch{"1H6* 6HG"}="H6'1 6HG"; $resn_exch{"2H6* 6HG"}="H6'2 6HG";
$resn_exch{"1H2  6HG"}=" H21 6HG"; $resn_exch{"2H2  6HG"}=" H22 6HG"; $resn_exch{"2HOP 6HT"}="HOP2 6HT";
$resn_exch{"3HOP 6HT"}="HOP3 6HT"; $resn_exch{"1H5* 6HT"}=" H5' 6HT"; $resn_exch{"2H5* 6HT"}="H5'' 6HT";
$resn_exch{" H4* 6HT"}=" H4' 6HT"; $resn_exch{" H3* 6HT"}=" H3' 6HT"; $resn_exch{"HO3* 6HT"}="HO3' 6HT";
$resn_exch{"1H2* 6HT"}=" H2' 6HT"; $resn_exch{"2H2* 6HT"}="H2'' 6HT"; $resn_exch{" H1* 6HT"}=" H1' 6HT";
$resn_exch{"1H6* 6HT"}="H6'1 6HT"; $resn_exch{"2H6* 6HT"}="H6'2 6HT"; $resn_exch{"1H5M 6HT"}=" H71 6HT";
$resn_exch{"2H5M 6HT"}=" H72 6HT"; $resn_exch{"3H5M 6HT"}=" H73 6HT"; $resn_exch{"3HOP 6IA"}="HOP3 6IA";
$resn_exch{"2HOP 6IA"}="HOP2 6IA"; $resn_exch{"1H5* 6IA"}=" H5' 6IA"; $resn_exch{"2H5* 6IA"}="H5'' 6IA";
$resn_exch{" H4* 6IA"}=" H4' 6IA"; $resn_exch{" H3* 6IA"}=" H3' 6IA"; $resn_exch{"H3T  6IA"}="HO3' 6IA";
$resn_exch{" H2* 6IA"}=" H2' 6IA"; $resn_exch{"2HO* 6IA"}="HO2' 6IA"; $resn_exch{" H1* 6IA"}=" H1' 6IA";
$resn_exch{"1H12 6IA"}="H121 6IA"; $resn_exch{"2H12 6IA"}="H122 6IA"; $resn_exch{"1H13 6IA"}="H131 6IA";
$resn_exch{"2H13 6IA"}="H132 6IA"; $resn_exch{"1H15 6IA"}="H151 6IA"; $resn_exch{"2H15 6IA"}="H152 6IA";
$resn_exch{"3H15 6IA"}="H153 6IA"; $resn_exch{"1H16 6IA"}="H161 6IA"; $resn_exch{"2H16 6IA"}="H162 6IA";
$resn_exch{"3H16 6IA"}="H163 6IA"; $resn_exch{"1HN4 6IG"}="HN41 6IG"; $resn_exch{"2HN4 6IG"}="HN42 6IG";
$resn_exch{"1H5  6IG"}=" H51 6IG"; $resn_exch{"2H5  6IG"}=" H52 6IG"; $resn_exch{"1H6  6IG"}=" H61 6IG";
$resn_exch{"2H6  6IG"}=" H62 6IG"; $resn_exch{"3H6  6IG"}=" H63 6IG"; $resn_exch{"1H13 6IG"}="H131 6IG";
$resn_exch{"2H13 6IG"}="H132 6IG"; $resn_exch{"1H14 6IG"}="H141 6IG"; $resn_exch{"2H14 6IG"}="H142 6IG";
$resn_exch{"1H15 6IG"}="H151 6IG"; $resn_exch{"2H15 6IG"}="H152 6IG"; $resn_exch{"1H16 6IG"}="H161 6IG";
$resn_exch{"2H16 6IG"}="H162 6IG"; $resn_exch{"1H17 6IG"}="H171 6IG"; $resn_exch{"2H17 6IG"}="H172 6IG";
$resn_exch{"1H18 6IG"}="H181 6IG"; $resn_exch{"2H18 6IG"}="H182 6IG"; $resn_exch{"1H19 6IG"}="H191 6IG";
$resn_exch{"2H19 6IG"}="H192 6IG"; $resn_exch{"3H19 6IG"}="H193 6IG"; $resn_exch{"1H20 6IG"}="H201 6IG";
$resn_exch{"2H20 6IG"}="H202 6IG"; $resn_exch{"1H21 6IG"}="H211 6IG"; $resn_exch{"2H21 6IG"}="H212 6IG";
$resn_exch{"1H13 6IN"}="H131 6IN"; $resn_exch{"2H13 6IN"}="H132 6IN"; $resn_exch{"1H27 6IN"}="H271 6IN";
$resn_exch{"2H27 6IN"}="H272 6IN"; $resn_exch{"3H27 6IN"}="H273 6IN"; $resn_exch{"1H32 6IN"}="H321 6IN";
$resn_exch{"2H32 6IN"}="H322 6IN"; $resn_exch{"1H37 6IN"}="H371 6IN"; $resn_exch{"2H37 6IN"}="H372 6IN";
$resn_exch{"1H42 6IN"}="H421 6IN"; $resn_exch{"2H42 6IN"}="H422 6IN"; $resn_exch{"1H43 6IN"}="H431 6IN";
$resn_exch{"2H43 6IN"}="H432 6IN"; $resn_exch{"1H46 6IN"}="H461 6IN"; $resn_exch{"2H46 6IN"}="H462 6IN";
$resn_exch{"1HN1 6IP"}="HN11 6IP"; $resn_exch{"2HN1 6IP"}="HN12 6IP"; $resn_exch{"1H7  6IP"}=" H71 6IP";
$resn_exch{"2H7  6IP"}=" H72 6IP"; $resn_exch{"1H8  6IP"}=" H81 6IP"; $resn_exch{"2H8  6IP"}=" H82 6IP";
$resn_exch{"3HOP 6MA"}="HOP3 6MA"; $resn_exch{"2HOP 6MA"}="HOP2 6MA"; $resn_exch{"1H5* 6MA"}=" H5' 6MA";
$resn_exch{"2H5* 6MA"}="H5'' 6MA"; $resn_exch{" H4* 6MA"}=" H4' 6MA"; $resn_exch{" H3* 6MA"}=" H3' 6MA";
$resn_exch{"H3T  6MA"}="HO3' 6MA"; $resn_exch{"1H2* 6MA"}=" H2' 6MA"; $resn_exch{"2H2* 6MA"}="H2'' 6MA";
$resn_exch{" H1* 6MA"}=" H1' 6MA"; $resn_exch{"1H6  6MA"}=" H61 6MA"; $resn_exch{"1H1  6MA"}=" H11 6MA";
$resn_exch{"2H1  6MA"}=" H12 6MA"; $resn_exch{"3H1  6MA"}=" H13 6MA"; $resn_exch{"3HOP 6MC"}="HOP3 6MC";
$resn_exch{"2HOP 6MC"}="HOP2 6MC"; $resn_exch{"1H5* 6MC"}=" H5' 6MC"; $resn_exch{"2H5* 6MC"}="H5'' 6MC";
$resn_exch{" H4* 6MC"}=" H4' 6MC"; $resn_exch{" H3* 6MC"}=" H3' 6MC"; $resn_exch{"H3T  6MC"}="HO3' 6MC";
$resn_exch{"1H2* 6MC"}=" H2' 6MC"; $resn_exch{"2H2* 6MC"}="H2'' 6MC"; $resn_exch{" H1* 6MC"}=" H1' 6MC";
$resn_exch{"1H6  6MC"}=" H61 6MC"; $resn_exch{"1H1  6MC"}=" H11 6MC"; $resn_exch{"2H1  6MC"}=" H12 6MC";
$resn_exch{"3H1  6MC"}=" H13 6MC"; $resn_exch{"2HOP 6MI"}="HOP2 6MI"; $resn_exch{"3HOP 6MI"}="HOP3 6MI";
$resn_exch{"1H7M 6MI"}="H7M1 6MI"; $resn_exch{"2H7M 6MI"}="H7M2 6MI"; $resn_exch{"1H3I 6MI"}="H3I1 6MI";
$resn_exch{"2H3I 6MI"}="H3I2 6MI"; $resn_exch{"3H3I 6MI"}="H3I3 6MI"; $resn_exch{"1H2* 6MI"}="H2'1 6MI";
$resn_exch{"2H2* 6MI"}="H2'2 6MI"; $resn_exch{"1H5* 6MI"}="H5'1 6MI"; $resn_exch{"2H5* 6MI"}="H5'2 6MI";
$resn_exch{" H4* 6MI"}=" H4' 6MI"; $resn_exch{" H1* 6MI"}=" H1' 6MI"; $resn_exch{" H3* 6MI"}=" H3' 6MI";
$resn_exch{"1H7  6MP"}=" H71 6MP"; $resn_exch{"2H7  6MP"}=" H72 6MP"; $resn_exch{"3H7  6MP"}=" H73 6MP";
$resn_exch{"1HB  6MR"}=" HB1 6MR"; $resn_exch{"2HB  6MR"}=" HB2 6MR"; $resn_exch{"3HOP 6MT"}="HOP3 6MT";
$resn_exch{"2HOP 6MT"}="HOP2 6MT"; $resn_exch{"1H5* 6MT"}=" H5' 6MT"; $resn_exch{"2H5* 6MT"}="H5'' 6MT";
$resn_exch{" H4* 6MT"}=" H4' 6MT"; $resn_exch{" H3* 6MT"}=" H3' 6MT"; $resn_exch{"H3T  6MT"}="HO3' 6MT";
$resn_exch{"1H2* 6MT"}=" H2' 6MT"; $resn_exch{"2H2* 6MT"}="H2'' 6MT"; $resn_exch{" H1* 6MT"}=" H1' 6MT";
$resn_exch{"1H6  6MT"}=" H61 6MT"; $resn_exch{"1H1  6MT"}=" H11 6MT"; $resn_exch{"2H1  6MT"}=" H12 6MT";
$resn_exch{"3H1  6MT"}=" H13 6MT"; $resn_exch{" H1* 6MZ"}=" H1' 6MZ"; $resn_exch{"1H9C 6MZ"}="H9C1 6MZ";
$resn_exch{"2H9C 6MZ"}="H9C2 6MZ"; $resn_exch{"1H5* 6MZ"}="H5'1 6MZ"; $resn_exch{"2H5* 6MZ"}="H5'2 6MZ";
$resn_exch{" H4* 6MZ"}=" H4' 6MZ"; $resn_exch{" H3* 6MZ"}=" H3' 6MZ"; $resn_exch{" H2* 6MZ"}=" H2' 6MZ";
$resn_exch{"1HAC 6NA"}="HAC1 6NA"; $resn_exch{"2HAC 6NA"}="HAC2 6NA"; $resn_exch{"1HBC 6NA"}="HBC1 6NA";
$resn_exch{"2HBC 6NA"}="HBC2 6NA"; $resn_exch{"1HGC 6NA"}="HGC1 6NA"; $resn_exch{"2HGC 6NA"}="HGC2 6NA";
$resn_exch{"1HDC 6NA"}="HDC1 6NA"; $resn_exch{"2HDC 6NA"}="HDC2 6NA"; $resn_exch{"1H6C 6NA"}="H6C1 6NA";
$resn_exch{"2H6C 6NA"}="H6C2 6NA"; $resn_exch{"3H6C 6NA"}="H6C3 6NA"; $resn_exch{"1HN1 6NH"}="HN11 6NH";
$resn_exch{"2HN1 6NH"}="HN12 6NH"; $resn_exch{"H2'' 6NH"}=" H2B 6NH"; $resn_exch{"H3'' 6NH"}=" H3B 6NH";
$resn_exch{"H4'' 6NH"}=" H4B 6NH"; $resn_exch{"H5'' 6NH"}=" H5B 6NH"; $resn_exch{"H6'' 6NH"}=" H6B 6NH";
$resn_exch{"1H7  6NI"}=" H71 6NI"; $resn_exch{"1H4  6NI"}=" H41 6NI"; $resn_exch{"1H5  6NI"}=" H51 6NI";
$resn_exch{"1HN1 6NI"}="HN11 6NI"; $resn_exch{"1H3  6NI"}=" H31 6NI"; $resn_exch{"1H10 6NP"}="H101 6NP";
$resn_exch{"2H10 6NP"}="H102 6NP"; $resn_exch{"1H28 6NP"}="H281 6NP"; $resn_exch{"2H28 6NP"}="H282 6NP";
$resn_exch{"1H29 6NP"}="H291 6NP"; $resn_exch{"2H29 6NP"}="H292 6NP"; $resn_exch{"1H30 6NP"}="H301 6NP";
$resn_exch{"2H30 6NP"}="H302 6NP"; $resn_exch{"2HOP 6OG"}="HOP2 6OG"; $resn_exch{"1HN2 6OG"}="HN21 6OG";
$resn_exch{"2HN2 6OG"}="HN22 6OG"; $resn_exch{"1H2* 6OG"}=" H2' 6OG"; $resn_exch{"2H2* 6OG"}="H2'' 6OG";
$resn_exch{"1H5* 6OG"}=" H5' 6OG"; $resn_exch{"2H5* 6OG"}="H5'' 6OG"; $resn_exch{" H4* 6OG"}=" H4' 6OG";
$resn_exch{" H1* 6OG"}=" H1' 6OG"; $resn_exch{" H3* 6OG"}=" H3' 6OG"; $resn_exch{"H3T  6OG"}="HO3' 6OG";
$resn_exch{"1H   6OG"}=" H1  6OG"; $resn_exch{"2H   6OG"}=" H2  6OG"; $resn_exch{"3H   6OG"}=" H3  6OG";
$resn_exch{"3HOP 6OG"}="HOP3 6OG"; $resn_exch{"1H6  6PG"}=" H61 6PG"; $resn_exch{"2H6  6PG"}=" H62 6PG";
$resn_exch{"2HOP 6PG"}="HOP2 6PG"; $resn_exch{"3HOP 6PG"}="HOP3 6PG"; $resn_exch{"1H26 6PL"}="H261 6PL";
$resn_exch{"2H26 6PL"}="H262 6PL"; $resn_exch{"3H26 6PL"}="H263 6PL"; $resn_exch{"1H25 6PL"}="H251 6PL";
$resn_exch{"2H25 6PL"}="H252 6PL"; $resn_exch{"1H24 6PL"}="H241 6PL"; $resn_exch{"2H24 6PL"}="H242 6PL";
$resn_exch{"1H23 6PL"}="H231 6PL"; $resn_exch{"2H23 6PL"}="H232 6PL"; $resn_exch{"1H22 6PL"}="H221 6PL";
$resn_exch{"2H22 6PL"}="H222 6PL"; $resn_exch{"1H21 6PL"}="H211 6PL"; $resn_exch{"2H21 6PL"}="H212 6PL";
$resn_exch{"1H20 6PL"}="H201 6PL"; $resn_exch{"2H20 6PL"}="H202 6PL"; $resn_exch{"1H19 6PL"}="H191 6PL";
$resn_exch{"2H19 6PL"}="H192 6PL"; $resn_exch{"1H18 6PL"}="H181 6PL"; $resn_exch{"2H18 6PL"}="H182 6PL";
$resn_exch{"1H17 6PL"}="H171 6PL"; $resn_exch{"2H17 6PL"}="H172 6PL"; $resn_exch{"1H16 6PL"}="H161 6PL";
$resn_exch{"2H16 6PL"}="H162 6PL"; $resn_exch{"1H15 6PL"}="H151 6PL"; $resn_exch{"2H15 6PL"}="H152 6PL";
$resn_exch{"1H14 6PL"}="H141 6PL"; $resn_exch{"2H14 6PL"}="H142 6PL"; $resn_exch{"1H13 6PL"}="H131 6PL";
$resn_exch{"2H13 6PL"}="H132 6PL"; $resn_exch{"1H12 6PL"}="H121 6PL"; $resn_exch{"2H12 6PL"}="H122 6PL";
$resn_exch{"1H3  6PL"}=" H31 6PL"; $resn_exch{"2H3  6PL"}=" H32 6PL"; $resn_exch{"1H1  6PL"}=" H11 6PL";
$resn_exch{"2H1  6PL"}=" H12 6PL"; $resn_exch{"1H4  6PL"}=" H41 6PL"; $resn_exch{"2H4  6PL"}=" H42 6PL";
$resn_exch{"1H5  6PL"}=" H51 6PL"; $resn_exch{"2H5  6PL"}=" H52 6PL"; $resn_exch{"1H7  6PL"}=" H71 6PL";
$resn_exch{"2H7  6PL"}=" H72 6PL"; $resn_exch{"3H7  6PL"}=" H73 6PL"; $resn_exch{"1H8  6PL"}=" H81 6PL";
$resn_exch{"2H8  6PL"}=" H82 6PL"; $resn_exch{"3H8  6PL"}=" H83 6PL"; $resn_exch{"1H6  6PL"}=" H61 6PL";
$resn_exch{"2H6  6PL"}=" H62 6PL"; $resn_exch{"3H6  6PL"}=" H63 6PL"; $resn_exch{"1H32 6PL"}="H321 6PL";
$resn_exch{"2H32 6PL"}="H322 6PL"; $resn_exch{"1H33 6PL"}="H331 6PL"; $resn_exch{"2H33 6PL"}="H332 6PL";
$resn_exch{"1H34 6PL"}="H341 6PL"; $resn_exch{"2H34 6PL"}="H342 6PL"; $resn_exch{"1H35 6PL"}="H351 6PL";
$resn_exch{"2H35 6PL"}="H352 6PL"; $resn_exch{"1H36 6PL"}="H361 6PL"; $resn_exch{"2H36 6PL"}="H362 6PL";
$resn_exch{"1H37 6PL"}="H371 6PL"; $resn_exch{"2H37 6PL"}="H372 6PL"; $resn_exch{"1H38 6PL"}="H381 6PL";
$resn_exch{"2H38 6PL"}="H382 6PL"; $resn_exch{"1H39 6PL"}="H391 6PL"; $resn_exch{"2H39 6PL"}="H392 6PL";
$resn_exch{"1H40 6PL"}="H401 6PL"; $resn_exch{"2H40 6PL"}="H402 6PL"; $resn_exch{"1H41 6PL"}="H411 6PL";
$resn_exch{"2H41 6PL"}="H412 6PL"; $resn_exch{"1H42 6PL"}="H421 6PL"; $resn_exch{"2H42 6PL"}="H422 6PL";
$resn_exch{"1H43 6PL"}="H431 6PL"; $resn_exch{"2H43 6PL"}="H432 6PL"; $resn_exch{"1H44 6PL"}="H441 6PL";
$resn_exch{"2H44 6PL"}="H442 6PL"; $resn_exch{"1H45 6PL"}="H451 6PL"; $resn_exch{"2H45 6PL"}="H452 6PL";
$resn_exch{"1H46 6PL"}="H461 6PL"; $resn_exch{"2H46 6PL"}="H462 6PL"; $resn_exch{"1H47 6PL"}="H471 6PL";
$resn_exch{"2H47 6PL"}="H472 6PL"; $resn_exch{"1H48 6PL"}="H481 6PL"; $resn_exch{"2H48 6PL"}="H482 6PL";
$resn_exch{"3H48 6PL"}="H483 6PL"; $resn_exch{"1HAJ 6PR"}="HAJ1 6PR"; $resn_exch{"2HAJ 6PR"}="HAJ2 6PR";
$resn_exch{"1HB  6PR"}=" HB1 6PR"; $resn_exch{"2HB  6PR"}=" HB2 6PR"; $resn_exch{"1HAA 6PR"}="HAA1 6PR";
$resn_exch{"2HAA 6PR"}="HAA2 6PR"; $resn_exch{"1HR6 6SA"}="HR61 6SA"; $resn_exch{"2HR6 6SA"}="HR62 6SA";
$resn_exch{"1HY6 6SA"}="HY61 6SA"; $resn_exch{"2HY6 6SA"}="HY62 6SA"; $resn_exch{"3HY6 6SA"}="HY63 6SA";
$resn_exch{"1HQ7 6SA"}="HQ71 6SA"; $resn_exch{"2HQ7 6SA"}="HQ72 6SA"; $resn_exch{"1HW6 6SA"}="HW61 6SA";
$resn_exch{"2HW6 6SA"}="HW62 6SA"; $resn_exch{"1HP6 6SA"}="HP61 6SA"; $resn_exch{"2HP6 6SA"}="HP62 6SA";
$resn_exch{"1HV6 6SA"}="HV61 6SA"; $resn_exch{"2HV6 6SA"}="HV62 6SA"; $resn_exch{"1H12 6SC"}="H121 6SC";
$resn_exch{"2H12 6SC"}="H122 6SC"; $resn_exch{"1HBP 6UL"}="HBP1 6UL"; $resn_exch{"2HBP 6UL"}="HBP2 6UL";
$resn_exch{"3HBP 6UL"}="HBP3 6UL"; $resn_exch{"1HBN 6UL"}="HBN1 6UL"; $resn_exch{"2HBN 6UL"}="HBN2 6UL";
$resn_exch{"1HAX 6UL"}="HAX1 6UL"; $resn_exch{"2HAX 6UL"}="HAX2 6UL"; $resn_exch{"1HAY 6UL"}="HAY1 6UL";
$resn_exch{"2HAY 6UL"}="HAY2 6UL"; $resn_exch{"1HAZ 6UL"}="HAZ1 6UL"; $resn_exch{"2HAZ 6UL"}="HAZ2 6UL";
$resn_exch{"1HBA 6UL"}="HBA1 6UL"; $resn_exch{"2HBA 6UL"}="HBA2 6UL"; $resn_exch{"1HBB 6UL"}="HBB1 6UL";
$resn_exch{"2HBB 6UL"}="HBB2 6UL"; $resn_exch{"1HBC 6UL"}="HBC1 6UL"; $resn_exch{"2HBC 6UL"}="HBC2 6UL";
$resn_exch{"1HBD 6UL"}="HBD1 6UL"; $resn_exch{"2HBD 6UL"}="HBD2 6UL"; $resn_exch{"1HBE 6UL"}="HBE1 6UL";
$resn_exch{"2HBE 6UL"}="HBE2 6UL"; $resn_exch{"1HBF 6UL"}="HBF1 6UL"; $resn_exch{"2HBF 6UL"}="HBF2 6UL";
$resn_exch{"1HBG 6UL"}="HBG1 6UL"; $resn_exch{"2HBG 6UL"}="HBG2 6UL"; $resn_exch{"1HBH 6UL"}="HBH1 6UL";
$resn_exch{"2HBH 6UL"}="HBH2 6UL"; $resn_exch{"1HBI 6UL"}="HBI1 6UL"; $resn_exch{"2HBI 6UL"}="HBI2 6UL";
$resn_exch{"1HBJ 6UL"}="HBJ1 6UL"; $resn_exch{"2HBJ 6UL"}="HBJ2 6UL"; $resn_exch{"1HBK 6UL"}="HBK1 6UL";
$resn_exch{"2HBK 6UL"}="HBK2 6UL"; $resn_exch{"1HBL 6UL"}="HBL1 6UL"; $resn_exch{"2HBL 6UL"}="HBL2 6UL";
$resn_exch{"1HBM 6UL"}="HBM1 6UL"; $resn_exch{"2HBM 6UL"}="HBM2 6UL"; $resn_exch{"1HAW 6UL"}="HAW1 6UL";
$resn_exch{"2HAW 6UL"}="HAW2 6UL"; $resn_exch{"1HAV 6UL"}="HAV1 6UL"; $resn_exch{"2HAV 6UL"}="HAV2 6UL";
$resn_exch{"1HAU 6UL"}="HAU1 6UL"; $resn_exch{"2HAU 6UL"}="HAU2 6UL"; $resn_exch{"1HAT 6UL"}="HAT1 6UL";
$resn_exch{"2HAT 6UL"}="HAT2 6UL"; $resn_exch{"1HAS 6UL"}="HAS1 6UL"; $resn_exch{"2HAS 6UL"}="HAS2 6UL";
$resn_exch{"1HAR 6UL"}="HAR1 6UL"; $resn_exch{"2HAR 6UL"}="HAR2 6UL"; $resn_exch{"1HAO 6UL"}="HAO1 6UL";
$resn_exch{"2HAO 6UL"}="HAO2 6UL"; $resn_exch{"1HAN 6UL"}="HAN1 6UL"; $resn_exch{"2HAN 6UL"}="HAN2 6UL";
$resn_exch{"1HAM 6UL"}="HAM1 6UL"; $resn_exch{"2HAM 6UL"}="HAM2 6UL"; $resn_exch{"1HAL 6UL"}="HAL1 6UL";
$resn_exch{"2HAL 6UL"}="HAL2 6UL"; $resn_exch{"1HAK 6UL"}="HAK1 6UL"; $resn_exch{"2HAK 6UL"}="HAK2 6UL";
$resn_exch{"1HAJ 6UL"}="HAJ1 6UL"; $resn_exch{"2HAJ 6UL"}="HAJ2 6UL"; $resn_exch{"1HAI 6UL"}="HAI1 6UL";
$resn_exch{"2HAI 6UL"}="HAI2 6UL"; $resn_exch{"1HAH 6UL"}="HAH1 6UL"; $resn_exch{"2HAH 6UL"}="HAH2 6UL";
$resn_exch{"1HAG 6UL"}="HAG1 6UL"; $resn_exch{"2HAG 6UL"}="HAG2 6UL"; $resn_exch{"1HAF 6UL"}="HAF1 6UL";
$resn_exch{"2HAF 6UL"}="HAF2 6UL"; $resn_exch{"1HAE 6UL"}="HAE1 6UL"; $resn_exch{"2HAE 6UL"}="HAE2 6UL";
$resn_exch{"1HAD 6UL"}="HAD1 6UL"; $resn_exch{"2HAD 6UL"}="HAD2 6UL"; $resn_exch{"1HAC 6UL"}="HAC1 6UL";
$resn_exch{"2HAC 6UL"}="HAC2 6UL"; $resn_exch{"1HAB 6UL"}="HAB1 6UL"; $resn_exch{"2HAB 6UL"}="HAB2 6UL";
$resn_exch{"1HAA 6UL"}="HAA1 6UL"; $resn_exch{"2HAA 6UL"}="HAA2 6UL"; $resn_exch{"3HAA 6UL"}="HAA3 6UL";
$resn_exch{"1H10 700"}="H101 700"; $resn_exch{"2H10 700"}="H102 700"; $resn_exch{"1H12 700"}="H121 700";
$resn_exch{"1H13 700"}="H131 700"; $resn_exch{"1H14 700"}="H141 700"; $resn_exch{"1H15 700"}="H151 700";
$resn_exch{"1H16 700"}="H161 700"; $resn_exch{"1H17 700"}="H171 700"; $resn_exch{"1H19 700"}="H191 700";
$resn_exch{"2H19 700"}="H192 700"; $resn_exch{"1H20 700"}="H201 700"; $resn_exch{"1H22 700"}="H221 700";
$resn_exch{"2H22 700"}="H222 700"; $resn_exch{"1H8C 701"}="H8C1 701"; $resn_exch{"2H8C 701"}="H8C2 701";
$resn_exch{"3H8C 701"}="H8C3 701"; $resn_exch{"1H13 701"}="H131 701"; $resn_exch{"2H13 701"}="H132 701";
$resn_exch{"1H14 701"}="H141 701"; $resn_exch{"2H14 701"}="H142 701"; $resn_exch{"1H17 701"}="H171 701";
$resn_exch{"2H17 701"}="H172 701"; $resn_exch{"3H17 701"}="H173 701"; $resn_exch{"1H19 701"}="H191 701";
$resn_exch{"2H19 701"}="H192 701"; $resn_exch{"1H23 701"}="H231 701"; $resn_exch{"2H23 701"}="H232 701";
$resn_exch{"1H20 701"}="H201 701"; $resn_exch{"2H20 701"}="H202 701"; $resn_exch{"1H22 701"}="H221 701";
$resn_exch{"2H22 701"}="H222 701"; $resn_exch{"1H23 703"}="H231 703"; $resn_exch{"2H23 703"}="H232 703";
$resn_exch{"1H24 703"}="H241 703"; $resn_exch{"2H24 703"}="H242 703"; $resn_exch{"1H25 703"}="H251 703";
$resn_exch{"2H25 703"}="H252 703"; $resn_exch{"1H27 703"}="H271 703"; $resn_exch{"2H27 703"}="H272 703";
$resn_exch{"1H28 703"}="H281 703"; $resn_exch{"2H28 703"}="H282 703"; $resn_exch{"1H30 703"}="H301 703";
$resn_exch{"2H30 703"}="H302 703"; $resn_exch{"1H31 703"}="H311 703"; $resn_exch{"2H31 703"}="H312 703";
$resn_exch{"1H12 709"}="H121 709"; $resn_exch{"2H12 709"}="H122 709"; $resn_exch{"1H23 709"}="H231 709";
$resn_exch{"2H23 709"}="H232 709"; $resn_exch{"3H23 709"}="H233 709"; $resn_exch{"1H5M 70U"}=" H71 70U";
$resn_exch{"2H5M 70U"}=" H72 70U"; $resn_exch{"1H9  70U"}=" H91 70U"; $resn_exch{"2H9  70U"}=" H92 70U";
$resn_exch{"3H9  70U"}=" H93 70U"; $resn_exch{" H1* 70U"}=" H1' 70U"; $resn_exch{"2HO* 70U"}="HO2' 70U";
$resn_exch{" H2* 70U"}=" H2' 70U"; $resn_exch{"H3T  70U"}="HO3' 70U"; $resn_exch{" H3* 70U"}=" H3' 70U";
$resn_exch{" H4* 70U"}=" H4' 70U"; $resn_exch{"1H5* 70U"}=" H5' 70U"; $resn_exch{"2H5* 70U"}="H5'' 70U";
$resn_exch{"2HOP 70U"}="HOP2 70U"; $resn_exch{"3HOP 70U"}="HOP3 70U"; $resn_exch{"1H17 710"}="H171 710";
$resn_exch{"2H17 710"}="H172 710"; $resn_exch{"1H19 710"}="H191 710"; $resn_exch{"2H19 710"}="H192 710";
$resn_exch{"1H20 710"}="H201 710"; $resn_exch{"2H20 710"}="H202 710"; $resn_exch{"1H21 710"}="H211 710";
$resn_exch{"2H21 710"}="H212 710"; $resn_exch{"1H22 710"}="H221 710"; $resn_exch{"2H22 710"}="H222 710";
$resn_exch{"1H23 710"}="H231 710"; $resn_exch{"2H23 710"}="H232 710"; $resn_exch{"1H29 710"}="H291 710";
$resn_exch{"2H29 710"}="H292 710"; $resn_exch{"1HN2 711"}="HN21 711"; $resn_exch{"2HN2 711"}="HN2A 711";
$resn_exch{"1H12 711"}="H121 711"; $resn_exch{"2H12 711"}="H122 711"; $resn_exch{"1H22 711"}="H221 711";
$resn_exch{"2H22 711"}="H222 711"; $resn_exch{"3H22 711"}="H223 711"; $resn_exch{"1H32 711"}="H321 711";
$resn_exch{"2H32 711"}="H322 711"; $resn_exch{"1H33 711"}="H331 711"; $resn_exch{"2H33 711"}="H332 711";
$resn_exch{"1H34 711"}="H341 711"; $resn_exch{"2H34 711"}="H342 711"; $resn_exch{"1H35 711"}="H351 711";
$resn_exch{"2H35 711"}="H352 711"; $resn_exch{"1H37 711"}="H371 711"; $resn_exch{"2H37 711"}="H372 711";
$resn_exch{"3H37 711"}="H373 711"; $resn_exch{"1HN  712"}=" HN1 712"; $resn_exch{"2HN  712"}=" HN2 712";
$resn_exch{"1H3  712"}="H31A 712"; $resn_exch{"2H3  712"}=" H32 712"; $resn_exch{"3H3  712"}=" H33 712";
$resn_exch{"1H4  712"}=" H41 712"; $resn_exch{"2H4  712"}=" H42 712"; $resn_exch{"1H21 712"}="H211 712";
$resn_exch{"2H21 712"}="H212 712"; $resn_exch{"3H21 712"}="H213 712"; $resn_exch{"1H24 712"}="H241 712";
$resn_exch{"2H24 712"}="H242 712"; $resn_exch{"3H24 712"}="H243 712"; $resn_exch{"1H32 712"}="H321 712";
$resn_exch{"2H32 712"}="H322 712"; $resn_exch{"3H32 712"}="H323 712"; $resn_exch{"1H33 712"}="H331 712";
$resn_exch{"2H33 712"}="H332 712"; $resn_exch{"1H11 715"}="H111 715"; $resn_exch{"2H11 715"}="H112 715";
$resn_exch{"1H15 715"}="H151 715"; $resn_exch{"2H15 715"}="H152 715"; $resn_exch{"1H20 715"}="H201 715";
$resn_exch{"2H20 715"}="H202 715"; $resn_exch{"1H25 715"}="H251 715"; $resn_exch{"2H25 715"}="H252 715";
$resn_exch{"1H26 715"}="H261 715"; $resn_exch{"2H26 715"}="H262 715"; $resn_exch{"1H29 715"}="H291 715";
$resn_exch{"2H29 715"}="H292 715"; $resn_exch{"1H3  716"}=" H31 716"; $resn_exch{"2H3  716"}=" H32 716";
$resn_exch{"1H4  716"}=" H41 716"; $resn_exch{"2H4  716"}=" H42 716"; $resn_exch{"1H5  716"}=" H51 716";
$resn_exch{"2H5  716"}=" H52 716"; $resn_exch{"1HN8 716"}="HN81 716"; $resn_exch{"2HN8 716"}="HN82 716";
$resn_exch{"1H19 716"}="H191 716"; $resn_exch{"2H19 716"}="H192 716"; $resn_exch{"3H19 716"}="H193 716";
$resn_exch{"1H20 716"}="H201 716"; $resn_exch{"2H20 716"}="H202 716"; $resn_exch{"3H20 716"}="H203 716";
$resn_exch{"1HN  720"}=" HN1 720"; $resn_exch{"2HN  720"}=" HN2 720"; $resn_exch{"1HAC 720"}="HAC1 720";
$resn_exch{"2HAC 720"}="HAC2 720"; $resn_exch{"1HHC 720"}="HHC1 720"; $resn_exch{"2HHC 720"}="HHC2 720";
$resn_exch{"1HE2 720"}="HE21 720"; $resn_exch{"2HE2 720"}="HE22 720"; $resn_exch{"1HZC 720"}="HZC1 720";
$resn_exch{"2HZC 720"}="HZC2 720"; $resn_exch{"1HE1 720"}="HE11 720"; $resn_exch{"2HE1 720"}="HE12 720";
$resn_exch{"3HE1 720"}="HE13 720"; $resn_exch{"1HMC 720"}="HMC1 720"; $resn_exch{"2HMC 720"}="HMC2 720";
$resn_exch{"1HB2 720"}="HB21 720"; $resn_exch{"2HB2 720"}="HB22 720"; $resn_exch{"1H31 730"}="H311 730";
$resn_exch{"2H31 730"}="H312 730"; $resn_exch{"1H30 730"}="H301 730"; $resn_exch{"2H30 730"}="H302 730";
$resn_exch{"1H32 730"}="H321 730"; $resn_exch{"2H32 730"}="H322 730"; $resn_exch{"1H33 730"}="H331 730";
$resn_exch{"2H33 730"}="H332 730"; $resn_exch{"1H36 730"}="H361 730"; $resn_exch{"2H36 730"}="H362 730";
$resn_exch{"1H28 730"}="H281 730"; $resn_exch{"2H28 730"}="H282 730"; $resn_exch{"1H27 730"}="H271 730";
$resn_exch{"2H27 730"}="H272 730"; $resn_exch{"1H14 730"}="H141 730"; $resn_exch{"2H14 730"}="H142 730";
$resn_exch{"1H22 730"}="H221 730"; $resn_exch{"2H22 730"}="H222 730"; $resn_exch{"1H25 736"}="H251 736";
$resn_exch{"2H25 736"}="H252 736"; $resn_exch{"1H26 736"}="H261 736"; $resn_exch{"2H26 736"}="H262 736";
$resn_exch{"1H28 736"}="H281 736"; $resn_exch{"2H28 736"}="H282 736"; $resn_exch{"1H29 736"}="H291 736";
$resn_exch{"2H29 736"}="H292 736"; $resn_exch{"1H2  736"}="H21A 736"; $resn_exch{"2H2  736"}=" H22 736";
$resn_exch{"1H7  736"}=" H71 736"; $resn_exch{"2H7  736"}=" H72 736"; $resn_exch{"1H8  736"}=" H81 736";
$resn_exch{"2H8  736"}=" H82 736"; $resn_exch{"1H14 736"}="H141 736"; $resn_exch{"2H14 736"}="H142 736";
$resn_exch{"1H15 736"}="H151 736"; $resn_exch{"2H15 736"}="H152 736"; $resn_exch{"3H15 736"}="H153 736";
$resn_exch{"1H06 737"}="H061 737"; $resn_exch{"2H06 737"}="H062 737"; $resn_exch{"1H07 737"}="H071 737";
$resn_exch{"2H07 737"}="H072 737"; $resn_exch{"1H08 737"}="H081 737"; $resn_exch{"2H08 737"}="H082 737";
$resn_exch{"1H11 737"}="H111 737"; $resn_exch{"2H11 737"}="H112 737"; $resn_exch{"1H15 737"}="H151 737";
$resn_exch{"2H15 737"}="H152 737"; $resn_exch{"1H29 737"}="H291 737"; $resn_exch{"2H29 737"}="H292 737";
$resn_exch{"3H29 737"}="H293 737"; $resn_exch{"1H35 737"}="H351 737"; $resn_exch{"2H35 737"}="H352 737";
$resn_exch{"1H8  738"}=" H81 738"; $resn_exch{"2H8  738"}=" H82 738"; $resn_exch{"1H12 738"}="H121 738";
$resn_exch{"2H12 738"}="H122 738"; $resn_exch{"1H9  738"}=" H91 738"; $resn_exch{"2H9  738"}=" H92 738";
$resn_exch{"1H4  738"}=" H41 738"; $resn_exch{"2H4  738"}=" H42 738"; $resn_exch{"1HBC 739"}="HBC1 739";
$resn_exch{"2HBC 739"}="HBC2 739"; $resn_exch{"1HCC 739"}="HCC1 739"; $resn_exch{"2HCC 739"}="HCC2 739";
$resn_exch{"1HNC 739"}="HNC1 739"; $resn_exch{"2HNC 739"}="HNC2 739"; $resn_exch{"1HCI 739"}="HCI1 739";
$resn_exch{"2HCI 739"}="HCI2 739"; $resn_exch{"3HCI 739"}="HCI3 739"; $resn_exch{"1HDI 739"}="HDI1 739";
$resn_exch{"2HDI 739"}="HDI2 739"; $resn_exch{"1HEI 739"}="HEI1 739"; $resn_exch{"2HEI 739"}="HEI2 739";
$resn_exch{"3HEI 739"}="HEI3 739"; $resn_exch{"1HI  739"}=" HI1 739"; $resn_exch{"2HI  739"}=" HI2 739";
$resn_exch{"1HBF 739"}="HBF1 739"; $resn_exch{"2HBF 739"}="HBF2 739"; $resn_exch{"1HBM 739"}="HBM1 739";
$resn_exch{"2HBM 739"}="HBM2 739"; $resn_exch{"1HCM 739"}="HCM1 739"; $resn_exch{"2HCM 739"}="HCM2 739";
$resn_exch{"1HEM 739"}="HEM1 739"; $resn_exch{"2HEM 739"}="HEM2 739"; $resn_exch{"3HEM 739"}="HEM3 739";
$resn_exch{"1H1  73Q"}=" H11 73Q"; $resn_exch{"2H1  73Q"}=" H12 73Q"; $resn_exch{"3H1  73Q"}=" H13 73Q";
$resn_exch{"1H3  73Q"}=" H31 73Q"; $resn_exch{"2H3  73Q"}=" H32 73Q"; $resn_exch{"3H3  73Q"}=" H33 73Q";
$resn_exch{"1H19 73V"}="H191 73V"; $resn_exch{"2H19 73V"}="H192 73V"; $resn_exch{"3H19 73V"}="H193 73V";
$resn_exch{"1H18 73V"}="H181 73V"; $resn_exch{"2H18 73V"}="H182 73V"; $resn_exch{"1H17 73V"}="H171 73V";
$resn_exch{"2H17 73V"}="H172 73V"; $resn_exch{"1H9  73V"}=" H91 73V"; $resn_exch{"2H9  73V"}=" H92 73V";
$resn_exch{"1H22 73V"}="H221 73V"; $resn_exch{"2H22 73V"}="H222 73V"; $resn_exch{"1H21 73V"}="H211 73V";
$resn_exch{"2H21 73V"}="H212 73V"; $resn_exch{"3H21 73V"}="H213 73V"; $resn_exch{"1H23 73V"}="H231 73V";
$resn_exch{"2H23 73V"}="H232 73V"; $resn_exch{"3H23 73V"}="H233 73V"; $resn_exch{"1H26 73V"}="H261 73V";
$resn_exch{"2H26 73V"}="H262 73V"; $resn_exch{"1H25 73V"}="H251 73V"; $resn_exch{"2H25 73V"}="H252 73V";
$resn_exch{"1H24 73V"}="H241 73V"; $resn_exch{"2H24 73V"}="H242 73V"; $resn_exch{"1H10 744"}="H101 744";
$resn_exch{"2H10 744"}="H102 744"; $resn_exch{"1H8  744"}=" H81 744"; $resn_exch{"2H8  744"}=" H82 744";
$resn_exch{"1H4  744"}=" H41 744"; $resn_exch{"2H4  744"}=" H42 744"; $resn_exch{"1H23 744"}="H231 744";
$resn_exch{"2H23 744"}="H232 744"; $resn_exch{"3H23 744"}="H233 744"; $resn_exch{"1H19 745"}="H191 745";
$resn_exch{"2H19 745"}="H192 745"; $resn_exch{"1H7  745"}=" H71 745"; $resn_exch{"2H7  745"}=" H72 745";
$resn_exch{"1H16 74M"}="H161 74M"; $resn_exch{"2H16 74M"}="H162 74M"; $resn_exch{"1H15 74M"}="H151 74M";
$resn_exch{"2H15 74M"}="H152 74M"; $resn_exch{"3H15 74M"}="H153 74M"; $resn_exch{"1H17 74M"}="H171 74M";
$resn_exch{"2H17 74M"}="H172 74M"; $resn_exch{"3H17 74M"}="H173 74M"; $resn_exch{"1H20 74M"}="H201 74M";
$resn_exch{"2H20 74M"}="H202 74M"; $resn_exch{"1H19 74M"}="H191 74M"; $resn_exch{"2H19 74M"}="H192 74M";
$resn_exch{"1H18 74M"}="H181 74M"; $resn_exch{"2H18 74M"}="H182 74M"; $resn_exch{"1H9  74M"}=" H91 74M";
$resn_exch{"2H9  74M"}=" H92 74M"; $resn_exch{"3H9  74M"}=" H93 74M"; $resn_exch{"1H11 74M"}="H111 74M";
$resn_exch{"2H11 74M"}="H112 74M"; $resn_exch{"1H12 74M"}="H121 74M"; $resn_exch{"2H12 74M"}="H122 74M";
$resn_exch{"1H13 74M"}="H131 74M"; $resn_exch{"2H13 74M"}="H132 74M"; $resn_exch{"3H13 74M"}="H133 74M";
$resn_exch{"1HC1 750"}="HC11 750"; $resn_exch{"2HC1 750"}="HC12 750"; $resn_exch{"1HC3 750"}="HC31 750";
$resn_exch{"2HC3 750"}="HC32 750"; $resn_exch{"1HC4 750"}="HC41 750"; $resn_exch{"2HC4 750"}="HC42 750";
$resn_exch{"1HC6 750"}="HC61 750"; $resn_exch{"2HC6 750"}="HC62 750"; $resn_exch{"1HC7 750"}="HC71 750";
$resn_exch{"2HC7 750"}="HC72 750"; $resn_exch{"1HC8 750"}="HC81 750"; $resn_exch{"2HC8 750"}="HC82 750";
$resn_exch{"1H23 750"}="H231 750"; $resn_exch{"2H23 750"}="H232 750"; $resn_exch{"1H25 750"}="H251 750";
$resn_exch{"2H25 750"}="H252 750"; $resn_exch{"3H25 750"}="H253 750"; $resn_exch{"1H26 750"}="H261 750";
$resn_exch{"2H26 750"}="H262 750"; $resn_exch{"3H26 750"}="H263 750"; $resn_exch{"1H33 750"}="H331 750";
$resn_exch{"2H33 750"}="H332 750"; $resn_exch{"1H35 750"}="H351 750"; $resn_exch{"2H35 750"}="H352 750";
$resn_exch{"1H36 750"}="H361 750"; $resn_exch{"2H36 750"}="H362 750"; $resn_exch{"1H37 750"}="H371 750";
$resn_exch{"2H37 750"}="H372 750"; $resn_exch{"1H40 750"}="H401 750"; $resn_exch{"2H40 750"}="H402 750";
$resn_exch{"1H10 75V"}="H101 75V"; $resn_exch{"2H10 75V"}="H102 75V"; $resn_exch{"1H11 75V"}="H111 75V";
$resn_exch{"2H11 75V"}="H112 75V"; $resn_exch{"3H11 75V"}="H113 75V"; $resn_exch{"1H14 75V"}="H141 75V";
$resn_exch{"2H14 75V"}="H142 75V"; $resn_exch{"1H13 75V"}="H131 75V"; $resn_exch{"2H13 75V"}="H132 75V";
$resn_exch{"3H13 75V"}="H133 75V"; $resn_exch{"1H15 75V"}="H151 75V"; $resn_exch{"2H15 75V"}="H152 75V";
$resn_exch{"3H15 75V"}="H153 75V"; $resn_exch{"1H16 75V"}="H161 75V"; $resn_exch{"2H16 75V"}="H162 75V";
$resn_exch{"3H16 75V"}="H163 75V"; $resn_exch{"2H1  760"}=" H12 760"; $resn_exch{"1H1  760"}=" H11 760";
$resn_exch{"2H2  760"}="H22A 760"; $resn_exch{"1H2  760"}="H21A 760"; $resn_exch{"2H3  760"}=" H32 760";
$resn_exch{"1H3  760"}=" H31 760"; $resn_exch{"2H4  760"}=" H42 760"; $resn_exch{"1H4  760"}=" H41 760";
$resn_exch{"1H17 760"}="H171 760"; $resn_exch{"2H17 760"}="H172 760"; $resn_exch{"1H1  761"}=" H11 761";
$resn_exch{"1H2  761"}=" H21 761"; $resn_exch{"1H5  761"}=" H51 761"; $resn_exch{"1H6  761"}=" H61 761";
$resn_exch{"1H11 761"}="H111 761"; $resn_exch{"1H14 761"}="H141 761"; $resn_exch{"H2'' 762"}=" H2B 762";
$resn_exch{"H3'' 762"}=" H3B 762"; $resn_exch{"H4'' 762"}=" H4B 762"; $resn_exch{"H5'' 762"}=" H5B 762";
$resn_exch{"H6'' 762"}=" H6B 762"; $resn_exch{"1H33 76A"}="H331 76A"; $resn_exch{"2H33 76A"}="H332 76A";
$resn_exch{"1H34 76A"}="H341 76A"; $resn_exch{"2H34 76A"}="H342 76A"; $resn_exch{"1H30 76A"}="H301 76A";
$resn_exch{"2H30 76A"}="H302 76A"; $resn_exch{"1H31 76A"}="H311 76A"; $resn_exch{"2H31 76A"}="H312 76A";
$resn_exch{"1H21 76A"}="H211 76A"; $resn_exch{"2H21 76A"}="H212 76A"; $resn_exch{"1H10 76V"}="H101 76V";
$resn_exch{"2H10 76V"}="H102 76V"; $resn_exch{"1H11 76V"}="H111 76V"; $resn_exch{"2H11 76V"}="H112 76V";
$resn_exch{"3H11 76V"}="H113 76V"; $resn_exch{"1H14 76V"}="H141 76V"; $resn_exch{"2H14 76V"}="H142 76V";
$resn_exch{"1H13 76V"}="H131 76V"; $resn_exch{"2H13 76V"}="H132 76V"; $resn_exch{"3H13 76V"}="H133 76V";
$resn_exch{"1H15 76V"}="H151 76V"; $resn_exch{"2H15 76V"}="H152 76V"; $resn_exch{"3H15 76V"}="H153 76V";
$resn_exch{"1H18 76V"}="H181 76V"; $resn_exch{"2H18 76V"}="H182 76V"; $resn_exch{"1H17 76V"}="H171 76V";
$resn_exch{"2H17 76V"}="H172 76V"; $resn_exch{"3H17 76V"}="H173 76V"; $resn_exch{"1H19 76V"}="H191 76V";
$resn_exch{"2H19 76V"}="H192 76V"; $resn_exch{"3H19 76V"}="H193 76V"; $resn_exch{"1H2N 771"}="H2N1 771";
$resn_exch{"2H2N 771"}="H2N2 771"; $resn_exch{"1H8C 771"}="H8C1 771"; $resn_exch{"2H8C 771"}="H8C2 771";
$resn_exch{"1H15 771"}="H151 771"; $resn_exch{"2H15 771"}="H152 771"; $resn_exch{"1H22 771"}="H221 771";
$resn_exch{"2H22 771"}="H222 771"; $resn_exch{"3H22 771"}="H223 771"; $resn_exch{"1H23 771"}="H231 771";
$resn_exch{"2H23 771"}="H232 771"; $resn_exch{"3H23 771"}="H233 771"; $resn_exch{"1H13 773"}="H131 773";
$resn_exch{"2H13 773"}="H132 773"; $resn_exch{"3H13 773"}="H133 773"; $resn_exch{"1H11 773"}="H111 773";
$resn_exch{"2H11 773"}="H112 773"; $resn_exch{"1H14 773"}="H141 773"; $resn_exch{"2H14 773"}="H142 773";
$resn_exch{"3H14 773"}="H143 773"; $resn_exch{"1H15 773"}="H151 773"; $resn_exch{"2H15 773"}="H152 773";
$resn_exch{"3H15 773"}="H153 773"; $resn_exch{"1H42 773"}="H421 773"; $resn_exch{"2H42 773"}="H422 773";
$resn_exch{"1H43 773"}="H431 773"; $resn_exch{"2H43 773"}="H432 773"; $resn_exch{"3H43 773"}="H433 773";
$resn_exch{"1H20 773"}="H201 773"; $resn_exch{"2H20 773"}="H202 773"; $resn_exch{"3H20 773"}="H203 773";
$resn_exch{"1H18 773"}="H181 773"; $resn_exch{"2H18 773"}="H182 773"; $resn_exch{"1H28 773"}="H281 773";
$resn_exch{"2H28 773"}="H282 773"; $resn_exch{"1H17 773"}="H171 773"; $resn_exch{"2H17 773"}="H172 773";
$resn_exch{"3H17 773"}="H173 773"; $resn_exch{"1H41 773"}="H411 773"; $resn_exch{"2H41 773"}="H412 773";
$resn_exch{"3H41 773"}="H413 773"; $resn_exch{"1H27 773"}="H271 773"; $resn_exch{"2H27 773"}="H272 773";
$resn_exch{"3H27 773"}="H273 773"; $resn_exch{"1H3  773"}=" H31 773"; $resn_exch{"2H3  773"}="H32A 773";
$resn_exch{"3H3  773"}=" H33 773"; $resn_exch{"1H6  773"}=" H61 773"; $resn_exch{"2H6  773"}=" H62 773";
$resn_exch{"3H6  773"}=" H63 773"; $resn_exch{"1H2  778"}="H21A 778"; $resn_exch{"2H2  778"}=" H22 778";
$resn_exch{"1H3  778"}=" H31 778"; $resn_exch{"2H3  778"}="H32A 778"; $resn_exch{"1H4  778"}=" H41 778";
$resn_exch{"2H4  778"}=" H42 778"; $resn_exch{"1H5  778"}=" H51 778"; $resn_exch{"2H5  778"}=" H52 778";
$resn_exch{"1H16 778"}="H161 778"; $resn_exch{"2H16 778"}="H162 778"; $resn_exch{"1H9  77A"}=" H91 77A";
$resn_exch{"2H9  77A"}=" H92 77A"; $resn_exch{"3H9  77A"}=" H93 77A"; $resn_exch{"1H22 77A"}="H221 77A";
$resn_exch{"2H22 77A"}="H222 77A"; $resn_exch{"1H24 77A"}="H241 77A"; $resn_exch{"2H24 77A"}="H242 77A";
$resn_exch{"3H24 77A"}="H243 77A"; $resn_exch{"1H25 77A"}="H251 77A"; $resn_exch{"2H25 77A"}="H252 77A";
$resn_exch{"3H25 77A"}="H253 77A"; $resn_exch{"1H26 77A"}="H261 77A"; $resn_exch{"2H26 77A"}="H262 77A";
$resn_exch{"3H26 77A"}="H263 77A"; $resn_exch{"1H17 77B"}="H171 77B"; $resn_exch{"2H17 77B"}="H172 77B";
$resn_exch{"1H26 77B"}="H261 77B"; $resn_exch{"2H26 77B"}="H262 77B"; $resn_exch{"1H25 77B"}="H251 77B";
$resn_exch{"2H25 77B"}="H252 77B"; $resn_exch{"3H25 77B"}="H253 77B"; $resn_exch{"1H27 77B"}="H271 77B";
$resn_exch{"2H27 77B"}="H272 77B"; $resn_exch{"3H27 77B"}="H273 77B"; $resn_exch{"1H9  77B"}=" H91 77B";
$resn_exch{"2H9  77B"}=" H92 77B"; $resn_exch{"1H30 77B"}="H301 77B"; $resn_exch{"2H30 77B"}="H302 77B";
$resn_exch{"1H29 77B"}="H291 77B"; $resn_exch{"2H29 77B"}="H292 77B"; $resn_exch{"1H28 77B"}="H281 77B";
$resn_exch{"2H28 77B"}="H282 77B"; $resn_exch{"H2'' 780"}=" H2B 780"; $resn_exch{"H3'' 780"}=" H3B 780";
$resn_exch{"H4'' 780"}=" H4B 780"; $resn_exch{"H5'' 780"}=" H5B 780"; $resn_exch{"H6'' 780"}=" H6B 780";
$resn_exch{"H2'' 783"}=" H2B 783"; $resn_exch{"H3'' 783"}=" H3B 783"; $resn_exch{"H4'' 783"}=" H4B 783";
$resn_exch{"H5'' 783"}=" H5B 783"; $resn_exch{"H6'' 783"}=" H6B 783"; $resn_exch{"1H19 784"}="H191 784";
$resn_exch{"2H19 784"}="H192 784"; $resn_exch{"1H21 784"}="H211 784"; $resn_exch{"2H21 784"}="H212 784";
$resn_exch{"1H23 784"}="H231 784"; $resn_exch{"2H23 784"}="H232 784"; $resn_exch{"1H32 784"}="H321 784";
$resn_exch{"2H32 784"}="H322 784"; $resn_exch{"3H32 784"}="H323 784"; $resn_exch{"1H31 784"}="H311 784";
$resn_exch{"2H31 784"}="H312 784"; $resn_exch{"3H31 784"}="H313 784"; $resn_exch{"1H30 784"}="H301 784";
$resn_exch{"2H30 784"}="H302 784"; $resn_exch{"3H30 784"}="H303 784"; $resn_exch{"1H14 784"}="H141 784";
$resn_exch{"2H14 784"}="H142 784"; $resn_exch{"1H15 784"}="H151 784"; $resn_exch{"2H15 784"}="H152 784";
$resn_exch{"1H8C 784"}="H8C1 784"; $resn_exch{"2H8C 784"}="H8C2 784"; $resn_exch{"1H24 784"}="H241 784";
$resn_exch{"2H24 784"}="H242 784"; $resn_exch{"1H10 78A"}="H101 78A"; $resn_exch{"2H10 78A"}="H102 78A";
$resn_exch{"1H19 78A"}="H191 78A"; $resn_exch{"2H19 78A"}="H192 78A"; $resn_exch{"1H18 78A"}="H181 78A";
$resn_exch{"2H18 78A"}="H182 78A"; $resn_exch{"3H18 78A"}="H183 78A"; $resn_exch{"1H20 78A"}="H201 78A";
$resn_exch{"2H20 78A"}="H202 78A"; $resn_exch{"3H20 78A"}="H203 78A"; $resn_exch{"1H23 78A"}="H231 78A";
$resn_exch{"2H23 78A"}="H232 78A"; $resn_exch{"1H22 78A"}="H221 78A"; $resn_exch{"2H22 78A"}="H222 78A";
$resn_exch{"1H21 78A"}="H211 78A"; $resn_exch{"2H21 78A"}="H212 78A"; $resn_exch{"1H1  790"}=" H11 790";
$resn_exch{"2H1  790"}=" H12 790"; $resn_exch{"1H2  790"}="H21A 790"; $resn_exch{"2H2  790"}=" H22 790";
$resn_exch{"1H6  790"}=" H61 790"; $resn_exch{"2H6  790"}=" H62 790"; $resn_exch{"1H10 796"}="H101 796";
$resn_exch{"2H10 796"}="H102 796"; $resn_exch{"3H10 796"}="H103 796"; $resn_exch{"1H27 796"}="H271 796";
$resn_exch{"2H27 796"}="H272 796"; $resn_exch{"1H43 796"}="H431 796"; $resn_exch{"2H43 796"}="H432 796";
$resn_exch{"1H28 796"}="H281 796"; $resn_exch{"2H28 796"}="H282 796"; $resn_exch{"1H5  7A8"}=" H51 7A8";
$resn_exch{"2H5  7A8"}=" H52 7A8"; $resn_exch{"1H8  7A8"}=" H81 7A8"; $resn_exch{"2H8  7A8"}=" H82 7A8";
$resn_exch{"1H9  7A8"}=" H91 7A8"; $resn_exch{"2H9  7A8"}=" H92 7A8"; $resn_exch{"1H13 7A8"}="H131 7A8";
$resn_exch{"2H13 7A8"}="H132 7A8"; $resn_exch{"3H13 7A8"}="H133 7A8"; $resn_exch{"1H14 7A8"}="H141 7A8";
$resn_exch{"2H14 7A8"}="H142 7A8"; $resn_exch{"3H14 7A8"}="H143 7A8"; $resn_exch{"1H15 7A8"}="H151 7A8";
$resn_exch{"2H15 7A8"}="H152 7A8"; $resn_exch{"3H15 7A8"}="H153 7A8"; $resn_exch{"1HN1 7AD"}="HN11 7AD";
$resn_exch{"2HN1 7AD"}="HN12 7AD"; $resn_exch{"1H15 7AD"}="H151 7AD"; $resn_exch{"2H15 7AD"}="H152 7AD";
$resn_exch{"1H16 7AD"}="H161 7AD"; $resn_exch{"2H16 7AD"}="H162 7AD"; $resn_exch{"1H17 7AD"}="H171 7AD";
$resn_exch{"2H17 7AD"}="H172 7AD"; $resn_exch{"1H18 7AD"}="H181 7AD"; $resn_exch{"2H18 7AD"}="H182 7AD";
$resn_exch{"3H18 7AD"}="H183 7AD"; $resn_exch{"1H19 7AD"}="H191 7AD"; $resn_exch{"2H19 7AD"}="H192 7AD";
$resn_exch{"3H19 7AD"}="H193 7AD"; $resn_exch{"7H7  7AP"}=" H77 7AP"; $resn_exch{"'H1' 7AP"}="H1'' 7AP";
$resn_exch{"'H2' 7AP"}="H2'' 7AP"; $resn_exch{"1H3' 7AP"}="H3'1 7AP"; $resn_exch{"2H3' 7AP"}="H3'2 7AP";
$resn_exch{"3H3' 7AP"}="H3'3 7AP"; $resn_exch{"1H1  7CA"}=" H11 7CA"; $resn_exch{"2H1  7CA"}=" H12 7CA";
$resn_exch{"3H1  7CA"}=" H13 7CA"; $resn_exch{"1H17 7CA"}="H171 7CA"; $resn_exch{"2H17 7CA"}="H172 7CA";
$resn_exch{"1H18 7CA"}="H181 7CA"; $resn_exch{"2H18 7CA"}="H182 7CA"; $resn_exch{"1H20 7CA"}="H201 7CA";
$resn_exch{"2H20 7CA"}="H202 7CA"; $resn_exch{"3H20 7CA"}="H203 7CA"; $resn_exch{"1H21 7CA"}="H211 7CA";
$resn_exch{"2H21 7CA"}="H212 7CA"; $resn_exch{"3H21 7CA"}="H213 7CA"; $resn_exch{"1H27 7CA"}="H271 7CA";
$resn_exch{"2H27 7CA"}="H272 7CA"; $resn_exch{"3H27 7CA"}="H273 7CA"; $resn_exch{"1H8  7CS"}=" H81 7CS";
$resn_exch{"2H8  7CS"}=" H82 7CS"; $resn_exch{"3H8  7CS"}=" H83 7CS"; $resn_exch{"1H7  7CS"}=" H71 7CS";
$resn_exch{"2H7  7CS"}=" H72 7CS"; $resn_exch{"1H6  7CS"}=" H61 7CS"; $resn_exch{"2H6  7CS"}=" H62 7CS";
$resn_exch{"1H4  7CS"}=" H41 7CS"; $resn_exch{"2H4  7CS"}=" H42 7CS"; $resn_exch{"1H2  7CS"}=" H21 7CS";
$resn_exch{"2H2  7CS"}=" H22 7CS"; $resn_exch{"2HOP 7DA"}="HOP2 7DA"; $resn_exch{"1HN6 7DA"}="HN61 7DA";
$resn_exch{"2HN6 7DA"}="HN62 7DA"; $resn_exch{"1H2* 7DA"}=" H2' 7DA"; $resn_exch{"2H2* 7DA"}="H2'' 7DA";
$resn_exch{"1H5* 7DA"}=" H5' 7DA"; $resn_exch{"2H5* 7DA"}="H5'' 7DA"; $resn_exch{" H4* 7DA"}=" H4' 7DA";
$resn_exch{" H1* 7DA"}=" H1' 7DA"; $resn_exch{"H3T  7DA"}="HO3' 7DA"; $resn_exch{"3HOP 7DA"}="HOP3 7DA";
$resn_exch{"1H6  7DE"}=" H61 7DE"; $resn_exch{"2H6  7DE"}=" H62 7DE"; $resn_exch{"3H6  7DE"}=" H63 7DE";
$resn_exch{"1H9  7DE"}=" H91 7DE"; $resn_exch{"2H9  7DE"}=" H92 7DE"; $resn_exch{"3H9  7DE"}=" H93 7DE";
$resn_exch{"1H16 7DE"}="H161 7DE"; $resn_exch{"2H16 7DE"}="H162 7DE"; $resn_exch{"1H17 7DE"}="H171 7DE";
$resn_exch{"2H17 7DE"}="H172 7DE"; $resn_exch{"3H17 7DE"}="H173 7DE"; $resn_exch{"1H7  7DG"}=" H71 7DG";
$resn_exch{"2H7  7DG"}=" H72 7DG"; $resn_exch{"1HN2 7DG"}="HN21 7DG"; $resn_exch{"2HN2 7DG"}="HN22 7DG";
$resn_exch{"1H6  7GP"}=" H61 7GP"; $resn_exch{"2H6  7GP"}=" H62 7GP"; $resn_exch{"1H9  7GP"}=" H91 7GP";
$resn_exch{"2H9  7GP"}=" H92 7GP"; $resn_exch{"1H10 7GP"}="H101 7GP"; $resn_exch{"2H10 7GP"}="H102 7GP";
$resn_exch{"3H10 7GP"}="H103 7GP"; $resn_exch{"2HOP 7GU"}="HOP2 7GU"; $resn_exch{"1HN2 7GU"}="HN21 7GU";
$resn_exch{"2HN2 7GU"}="HN22 7GU"; $resn_exch{"1H2* 7GU"}=" H2' 7GU"; $resn_exch{"2H2* 7GU"}="H2'' 7GU";
$resn_exch{"1H5* 7GU"}=" H5' 7GU"; $resn_exch{"2H5* 7GU"}="H5'' 7GU"; $resn_exch{" H4* 7GU"}=" H4' 7GU";
$resn_exch{" H1* 7GU"}=" H1' 7GU"; $resn_exch{" H3* 7GU"}=" H3' 7GU"; $resn_exch{"H3T  7GU"}="HO3' 7GU";
$resn_exch{"3HOP 7GU"}="HOP3 7GU"; $resn_exch{"1HMA 7HE"}="HMA1 7HE"; $resn_exch{"2HMA 7HE"}="HMA2 7HE";
$resn_exch{"3HMA 7HE"}="HMA3 7HE"; $resn_exch{"1HAA 7HE"}="HAA1 7HE"; $resn_exch{"2HAA 7HE"}="HAA2 7HE";
$resn_exch{"3HAA 7HE"}="HAA3 7HE"; $resn_exch{"1HMB 7HE"}="HMB1 7HE"; $resn_exch{"2HMB 7HE"}="HMB2 7HE";
$resn_exch{"3HMB 7HE"}="HMB3 7HE"; $resn_exch{"1HBB 7HE"}="HBB1 7HE"; $resn_exch{"2HBB 7HE"}="HBB2 7HE";
$resn_exch{"1HMC 7HE"}="HMC1 7HE"; $resn_exch{"2HMC 7HE"}="HMC2 7HE"; $resn_exch{"3HMC 7HE"}="HMC3 7HE";
$resn_exch{"1HBC 7HE"}="HBC1 7HE"; $resn_exch{"2HBC 7HE"}="HBC2 7HE"; $resn_exch{"1HMD 7HE"}="HMD1 7HE";
$resn_exch{"2HMD 7HE"}="HMD2 7HE"; $resn_exch{"3HMD 7HE"}="HMD3 7HE"; $resn_exch{"1HAD 7HE"}="HAD1 7HE";
$resn_exch{"2HAD 7HE"}="HAD2 7HE"; $resn_exch{"1HBD 7HE"}="HBD1 7HE"; $resn_exch{"2HBD 7HE"}="HBD2 7HE";
$resn_exch{"1H3  7I2"}=" H31 7I2"; $resn_exch{"1H4  7I2"}=" H41 7I2"; $resn_exch{"1H5  7I2"}=" H51 7I2";
$resn_exch{"1H6  7I2"}=" H61 7I2"; $resn_exch{"1H15 7I2"}="H151 7I2"; $resn_exch{"2H15 7I2"}="H152 7I2";
$resn_exch{"1HN3 7IG"}="HN31 7IG"; $resn_exch{"2HN3 7IG"}="HN32 7IG"; $resn_exch{"1HN4 7IG"}="HN41 7IG";
$resn_exch{"2HN4 7IG"}="HN42 7IG"; $resn_exch{"1H5  7IG"}=" H51 7IG"; $resn_exch{"2H5  7IG"}=" H52 7IG";
$resn_exch{"1H6  7IG"}=" H61 7IG"; $resn_exch{"2H6  7IG"}=" H62 7IG"; $resn_exch{"3H6  7IG"}=" H63 7IG";
$resn_exch{"1H19 7IG"}="H191 7IG"; $resn_exch{"2H19 7IG"}="H192 7IG"; $resn_exch{"1H3  7IN"}="H31A 7IN";
$resn_exch{"2H3  7IN"}=" H32 7IN"; $resn_exch{"1H10 7IN"}="H101 7IN"; $resn_exch{"2H10 7IN"}="H102 7IN";
$resn_exch{"1H19 7IN"}="H191 7IN"; $resn_exch{"2H19 7IN"}="H192 7IN"; $resn_exch{"1H25 7IN"}="H251 7IN";
$resn_exch{"2H25 7IN"}="H252 7IN"; $resn_exch{"1H34 7IN"}="H341 7IN"; $resn_exch{"2H34 7IN"}="H342 7IN";
$resn_exch{"1H1  7IP"}="H11A 7IP"; $resn_exch{"2H1  7IP"}="H12A 7IP"; $resn_exch{"3H1  7IP"}=" H13 7IP";
$resn_exch{"1H14 7IP"}="H141 7IP"; $resn_exch{"2H14 7IP"}="H142 7IP"; $resn_exch{"1H15 7IP"}="H151 7IP";
$resn_exch{"2H15 7IP"}="H152 7IP"; $resn_exch{"1H21 7IP"}="H211 7IP"; $resn_exch{"2H21 7IP"}="H212 7IP";
$resn_exch{"2HOP 7MG"}="HOP2 7MG"; $resn_exch{"3HOP 7MG"}="HOP3 7MG"; $resn_exch{"1H5* 7MG"}=" H5' 7MG";
$resn_exch{"2H5* 7MG"}="H5'' 7MG"; $resn_exch{" H4* 7MG"}=" H4' 7MG"; $resn_exch{" H3* 7MG"}=" H3' 7MG";
$resn_exch{"*HO3 7MG"}="HO3' 7MG"; $resn_exch{" H2* 7MG"}=" H2' 7MG"; $resn_exch{"*HO2 7MG"}="HO2' 7MG";
$resn_exch{" H1* 7MG"}=" H1' 7MG"; $resn_exch{"1H8  7MG"}=" H81 7MG"; $resn_exch{"2H8  7MG"}=" H82 7MG";
$resn_exch{"1HN2 7MG"}="HN21 7MG"; $resn_exch{"2HN2 7MG"}="HN22 7MG"; $resn_exch{"1HM7 7MG"}="HM71 7MG";
$resn_exch{"2HM7 7MG"}="HM72 7MG"; $resn_exch{"3HM7 7MG"}="HM73 7MG"; $resn_exch{"1HAT 7MP"}="HAT1 7MP";
$resn_exch{"2HAT 7MP"}="HAT2 7MP"; $resn_exch{"3HAT 7MP"}="HAT3 7MP"; $resn_exch{"1HAS 7MP"}="HAS1 7MP";
$resn_exch{"2HAS 7MP"}="HAS2 7MP"; $resn_exch{"1HAR 7MP"}="HAR1 7MP"; $resn_exch{"2HAR 7MP"}="HAR2 7MP";
$resn_exch{"3HAR 7MP"}="HAR3 7MP"; $resn_exch{"1H3M 7MQ"}="H3M1 7MQ"; $resn_exch{"2H3M 7MQ"}="H3M2 7MQ";
$resn_exch{"3H3M 7MQ"}="H3M3 7MQ"; $resn_exch{"1H11 7MQ"}="H111 7MQ"; $resn_exch{"2H11 7MQ"}="H112 7MQ";
$resn_exch{"1H14 7MQ"}="H141 7MQ"; $resn_exch{"2H14 7MQ"}="H142 7MQ"; $resn_exch{"3H14 7MQ"}="H143 7MQ";
$resn_exch{"1H15 7MQ"}="H151 7MQ"; $resn_exch{"2H15 7MQ"}="H152 7MQ"; $resn_exch{"1H16 7MQ"}="H161 7MQ";
$resn_exch{"2H16 7MQ"}="H162 7MQ"; $resn_exch{"1H19 7MQ"}="H191 7MQ"; $resn_exch{"2H19 7MQ"}="H192 7MQ";
$resn_exch{"3H19 7MQ"}="H193 7MQ"; $resn_exch{"1H20 7MQ"}="H201 7MQ"; $resn_exch{"2H20 7MQ"}="H202 7MQ";
$resn_exch{"1H21 7MQ"}="H211 7MQ"; $resn_exch{"2H21 7MQ"}="H212 7MQ"; $resn_exch{"1H24 7MQ"}="H241 7MQ";
$resn_exch{"2H24 7MQ"}="H242 7MQ"; $resn_exch{"3H24 7MQ"}="H243 7MQ"; $resn_exch{"1H25 7MQ"}="H251 7MQ";
$resn_exch{"2H25 7MQ"}="H252 7MQ"; $resn_exch{"1H26 7MQ"}="H261 7MQ"; $resn_exch{"2H26 7MQ"}="H262 7MQ";
$resn_exch{"1H29 7MQ"}="H291 7MQ"; $resn_exch{"2H29 7MQ"}="H292 7MQ"; $resn_exch{"3H29 7MQ"}="H293 7MQ";
$resn_exch{"1H30 7MQ"}="H301 7MQ"; $resn_exch{"2H30 7MQ"}="H302 7MQ"; $resn_exch{"1H31 7MQ"}="H311 7MQ";
$resn_exch{"2H31 7MQ"}="H312 7MQ"; $resn_exch{"1H34 7MQ"}="H341 7MQ"; $resn_exch{"2H34 7MQ"}="H342 7MQ";
$resn_exch{"3H34 7MQ"}="H343 7MQ"; $resn_exch{"1H35 7MQ"}="H351 7MQ"; $resn_exch{"2H35 7MQ"}="H352 7MQ";
$resn_exch{"1H36 7MQ"}="H361 7MQ"; $resn_exch{"2H36 7MQ"}="H362 7MQ"; $resn_exch{"1H39 7MQ"}="H391 7MQ";
$resn_exch{"2H39 7MQ"}="H392 7MQ"; $resn_exch{"3H39 7MQ"}="H393 7MQ"; $resn_exch{"1H40 7MQ"}="H401 7MQ";
$resn_exch{"2H40 7MQ"}="H402 7MQ"; $resn_exch{"1H41 7MQ"}="H411 7MQ"; $resn_exch{"2H41 7MQ"}="H412 7MQ";
$resn_exch{"1H44 7MQ"}="H441 7MQ"; $resn_exch{"2H44 7MQ"}="H442 7MQ"; $resn_exch{"3H44 7MQ"}="H443 7MQ";
$resn_exch{"1H45 7MQ"}="H451 7MQ"; $resn_exch{"2H45 7MQ"}="H452 7MQ"; $resn_exch{"3H45 7MQ"}="H453 7MQ";
$resn_exch{"1HAN 7MR"}="HAN1 7MR"; $resn_exch{"2HAN 7MR"}="HAN2 7MR"; $resn_exch{"1HAZ 7MR"}="HAZ1 7MR";
$resn_exch{"2HAZ 7MR"}="HAZ2 7MR"; $resn_exch{"H2'' 7NH"}=" H2B 7NH"; $resn_exch{"H3'' 7NH"}=" H3B 7NH";
$resn_exch{"H4'' 7NH"}=" H4B 7NH"; $resn_exch{"H6'' 7NH"}=" H6B 7NH"; $resn_exch{"1H51 7NH"}="H511 7NH";
$resn_exch{"2H51 7NH"}="H512 7NH"; $resn_exch{"1H54 7NH"}="H541 7NH"; $resn_exch{"2H54 7NH"}="H542 7NH";
$resn_exch{"1H1  7NI"}=" H11 7NI"; $resn_exch{"1H3  7NI"}=" H31 7NI"; $resn_exch{"1H4  7NI"}=" H41 7NI";
$resn_exch{"1H5  7NI"}=" H51 7NI"; $resn_exch{"1H6  7NI"}=" H61 7NI"; $resn_exch{"1H1  7PA"}=" H11 7PA";
$resn_exch{"2H1  7PA"}=" H12 7PA"; $resn_exch{"1H2  7PA"}=" H21 7PA"; $resn_exch{"2H2  7PA"}=" H22 7PA";
$resn_exch{"1H3  7PA"}=" H31 7PA"; $resn_exch{"2H3  7PA"}=" H32 7PA"; $resn_exch{"1H29 7PC"}="H291 7PC";
$resn_exch{"2H29 7PC"}="H292 7PC"; $resn_exch{"1H30 7PC"}="H301 7PC"; $resn_exch{"2H30 7PC"}="H302 7PC";
$resn_exch{"1H21 7PE"}="H211 7PE"; $resn_exch{"2H21 7PE"}="H212 7PE"; $resn_exch{"3H21 7PE"}="H213 7PE";
$resn_exch{"1H20 7PE"}="H201 7PE"; $resn_exch{"2H20 7PE"}="H202 7PE"; $resn_exch{"1H18 7PE"}="H181 7PE";
$resn_exch{"2H18 7PE"}="H182 7PE"; $resn_exch{"1H17 7PE"}="H171 7PE"; $resn_exch{"2H17 7PE"}="H172 7PE";
$resn_exch{"1H15 7PE"}="H151 7PE"; $resn_exch{"2H15 7PE"}="H152 7PE"; $resn_exch{"1H14 7PE"}="H141 7PE";
$resn_exch{"2H14 7PE"}="H142 7PE"; $resn_exch{"1H12 7PE"}="H121 7PE"; $resn_exch{"2H12 7PE"}="H122 7PE";
$resn_exch{"1H11 7PE"}="H111 7PE"; $resn_exch{"2H11 7PE"}="H112 7PE"; $resn_exch{"1H9  7PE"}=" H91 7PE";
$resn_exch{"2H9  7PE"}=" H92 7PE"; $resn_exch{"1H8  7PE"}=" H81 7PE"; $resn_exch{"2H8  7PE"}=" H82 7PE";
$resn_exch{"1H6  7PE"}=" H61 7PE"; $resn_exch{"2H6  7PE"}=" H62 7PE"; $resn_exch{"1H5  7PE"}=" H51 7PE";
$resn_exch{"2H5  7PE"}=" H52 7PE"; $resn_exch{"1H3  7PE"}=" H31 7PE"; $resn_exch{"2H3  7PE"}=" H32 7PE";
$resn_exch{"1H2  7PE"}=" H21 7PE"; $resn_exch{"2H2  7PE"}=" H22 7PE"; $resn_exch{"1H1  7PG"}=" H11 7PG";
$resn_exch{"2H1  7PG"}=" H12 7PG"; $resn_exch{"1H2  7PG"}=" H21 7PG"; $resn_exch{"2H2  7PG"}=" H22 7PG";
$resn_exch{"1H3  7PG"}=" H31 7PG"; $resn_exch{"2H3  7PG"}=" H32 7PG"; $resn_exch{"1H4  7PG"}=" H41 7PG";
$resn_exch{"2H4  7PG"}=" H42 7PG"; $resn_exch{"1H5  7PG"}=" H51 7PG"; $resn_exch{"2H5  7PG"}=" H52 7PG";
$resn_exch{"1H6  7PG"}=" H61 7PG"; $resn_exch{"2H6  7PG"}=" H62 7PG"; $resn_exch{"1H7  7PG"}=" H71 7PG";
$resn_exch{"2H7  7PG"}=" H72 7PG"; $resn_exch{"1H8  7PG"}=" H81 7PG"; $resn_exch{"2H8  7PG"}=" H82 7PG";
$resn_exch{"1H9  7PG"}=" H91 7PG"; $resn_exch{"2H9  7PG"}=" H92 7PG"; $resn_exch{"1H10 7PG"}="H101 7PG";
$resn_exch{"2H10 7PG"}="H102 7PG"; $resn_exch{"1H11 7PG"}="H111 7PG"; $resn_exch{"2H11 7PG"}="H112 7PG";
$resn_exch{"1H12 7PG"}="H121 7PG"; $resn_exch{"2H12 7PG"}="H122 7PG"; $resn_exch{"1H13 7PG"}="H131 7PG";
$resn_exch{"2H13 7PG"}="H132 7PG"; $resn_exch{"1H14 7PG"}="H141 7PG"; $resn_exch{"2H14 7PG"}="H142 7PG";
$resn_exch{"1H15 7PG"}="H151 7PG"; $resn_exch{"2H15 7PG"}="H152 7PG"; $resn_exch{"1H16 7PG"}="H161 7PG";
$resn_exch{"2H16 7PG"}="H162 7PG"; $resn_exch{"1H17 7PG"}="H171 7PG"; $resn_exch{"2H17 7PG"}="H172 7PG";
$resn_exch{"3H17 7PG"}="H173 7PG"; $resn_exch{"1HAX 7PY"}="HAX1 7PY"; $resn_exch{"2HAX 7PY"}="HAX2 7PY";
$resn_exch{"3HAX 7PY"}="HAX3 7PY"; $resn_exch{"1HAZ 7PY"}="HAZ1 7PY"; $resn_exch{"2HAZ 7PY"}="HAZ2 7PY";
$resn_exch{"3HAZ 7PY"}="HAZ3 7PY"; $resn_exch{"1HBB 7PY"}="HBB1 7PY"; $resn_exch{"2HBB 7PY"}="HBB2 7PY";
$resn_exch{"3HBB 7PY"}="HBB3 7PY"; $resn_exch{"1H10 7RA"}="H101 7RA"; $resn_exch{"2H10 7RA"}="H102 7RA";
$resn_exch{"2HOP 7RA"}="HOP2 7RA"; $resn_exch{"3HOP 7RA"}="HOP3 7RA"; $resn_exch{"1H5* 7RA"}="H5'1 7RA";
$resn_exch{"2H5* 7RA"}="H5'2 7RA"; $resn_exch{" H4* 7RA"}=" H4' 7RA"; $resn_exch{" H3* 7RA"}=" H3' 7RA";
$resn_exch{"3HO* 7RA"}="HO'3 7RA"; $resn_exch{" H2* 7RA"}=" H2' 7RA"; $resn_exch{"2HO* 7RA"}="HO'2 7RA";
$resn_exch{" H1* 7RA"}=" H1' 7RA"; $resn_exch{"2HOP 7RP"}="HOP2 7RP"; $resn_exch{"3HOP 7RP"}="HOP3 7RP";
$resn_exch{"1H5* 7RP"}="H5'1 7RP"; $resn_exch{"2H5* 7RP"}="H5'2 7RP"; $resn_exch{" H4* 7RP"}=" H4' 7RP";
$resn_exch{" H3* 7RP"}=" H3' 7RP"; $resn_exch{"3HO* 7RP"}="HO'3 7RP"; $resn_exch{" H2* 7RP"}=" H2' 7RP";
$resn_exch{"2HO* 7RP"}="HO'2 7RP"; $resn_exch{" H1* 7RP"}=" H1' 7RP"; $resn_exch{"1HAY 7XY"}="HAY1 7XY";
$resn_exch{"2HAY 7XY"}="HAY2 7XY"; $resn_exch{"1HAZ 7XY"}="HAZ1 7XY"; $resn_exch{"2HAZ 7XY"}="HAZ2 7XY";
$resn_exch{"1HAW 7XY"}="HAW1 7XY"; $resn_exch{"2HAW 7XY"}="HAW2 7XY"; $resn_exch{"1HAX 7XY"}="HAX1 7XY";
$resn_exch{"2HAX 7XY"}="HAX2 7XY"; $resn_exch{"1H2  803"}=" H21 803"; $resn_exch{"2H2  803"}=" H22 803";
$resn_exch{"1H21 803"}="H211 803"; $resn_exch{"2H21 803"}="H212 803"; $resn_exch{"3H21 803"}="H213 803";
$resn_exch{"1H20 803"}="H201 803"; $resn_exch{"2H20 803"}="H202 803"; $resn_exch{"3H20 803"}="H203 803";
$resn_exch{"1H10 803"}="H101 803"; $resn_exch{"2H10 803"}="H102 803"; $resn_exch{"1H11 803"}="H111 803";
$resn_exch{"2H11 803"}="H112 803"; $resn_exch{"1H13 803"}="H131 803"; $resn_exch{"2H13 803"}="H132 803";
$resn_exch{"1H15 803"}="H151 803"; $resn_exch{"2H15 803"}="H152 803"; $resn_exch{"1H19 803"}="H191 803";
$resn_exch{"2H19 803"}="H192 803"; $resn_exch{"3H19 803"}="H193 803"; $resn_exch{"1H17 803"}="H171 803";
$resn_exch{"2H17 803"}="H172 803"; $resn_exch{"1H18 803"}="H181 803"; $resn_exch{"2H18 803"}="H182 803";
$resn_exch{"3H18 803"}="H183 803"; $resn_exch{"1HN2 806"}="HN21 806"; $resn_exch{"2HN2 806"}="HN22 806";
$resn_exch{"1H12 806"}="H121 806"; $resn_exch{"2H12 806"}="H122 806"; $resn_exch{"1H42 806"}="H421 806";
$resn_exch{"2H42 806"}="H422 806"; $resn_exch{"1H43 806"}="H431 806"; $resn_exch{"2H43 806"}="H432 806";
$resn_exch{"1H44 806"}="H441 806"; $resn_exch{"2H44 806"}="H442 806"; $resn_exch{"1H45 806"}="H451 806";
$resn_exch{"2H45 806"}="H452 806"; $resn_exch{"1H47 806"}="H471 806"; $resn_exch{"2H47 806"}="H472 806";
$resn_exch{"3H47 806"}="H473 806"; $resn_exch{"1H1  815"}=" H11 815"; $resn_exch{"1H2  815"}=" H21 815";
$resn_exch{"1H5  815"}=" H51 815"; $resn_exch{"1H10 815"}="H101 815"; $resn_exch{"1H11 815"}="H111 815";
$resn_exch{"1H17 815"}="H171 815"; $resn_exch{"1H18 815"}="H181 815"; $resn_exch{"1H19 815"}="H191 815";
$resn_exch{"1H25 815"}="H251 815"; $resn_exch{"1H30 815"}="H301 815"; $resn_exch{"1H31 815"}="H311 815";
$resn_exch{"2H31 815"}="H312 815"; $resn_exch{"1H32 815"}="H321 815"; $resn_exch{"2H32 815"}="H322 815";
$resn_exch{"1H37 815"}="H371 815"; $resn_exch{"1H42 815"}="H421 815"; $resn_exch{"2H42 815"}="H422 815";
$resn_exch{"1H43 815"}="H431 815"; $resn_exch{"2H43 815"}="H432 815"; $resn_exch{"1HN2 817"}="HN21 817";
$resn_exch{"2HN2 817"}="HN22 817"; $resn_exch{"1H3  817"}=" H31 817"; $resn_exch{"2H3  817"}=" H32 817";
$resn_exch{"1H3  818"}="H31A 818"; $resn_exch{"2H3  818"}="H32A 818"; $resn_exch{"1H4  818"}=" H41 818";
$resn_exch{"2H4  818"}=" H42 818"; $resn_exch{"1H5  818"}=" H51 818"; $resn_exch{"2H5  818"}=" H52 818";
$resn_exch{"1H6  818"}=" H61 818"; $resn_exch{"2H6  818"}=" H62 818"; $resn_exch{"1H12 818"}="H121 818";
$resn_exch{"2H12 818"}="H122 818"; $resn_exch{"1H16 818"}="H161 818"; $resn_exch{"2H16 818"}="H162 818";
$resn_exch{"1H23 818"}="H231 818"; $resn_exch{"2H23 818"}="H232 818"; $resn_exch{"1H30 818"}="H301 818";
$resn_exch{"2H30 818"}="H302 818"; $resn_exch{"1H33 818"}="H331 818"; $resn_exch{"2H33 818"}="H332 818";
$resn_exch{"1H34 818"}="H341 818"; $resn_exch{"2H34 818"}="H342 818"; $resn_exch{"1H35 818"}="H351 818";
$resn_exch{"2H35 818"}="H352 818"; $resn_exch{"3H35 818"}="H353 818"; $resn_exch{"1H36 818"}="H361 818";
$resn_exch{"2H36 818"}="H362 818"; $resn_exch{"3H36 818"}="H363 818"; $resn_exch{"1H37 818"}="H371 818";
$resn_exch{"2H37 818"}="H372 818"; $resn_exch{"3H37 818"}="H373 818"; $resn_exch{"1H38 818"}="H381 818";
$resn_exch{"2H38 818"}="H382 818"; $resn_exch{"1H39 818"}="H391 818"; $resn_exch{"2H39 818"}="H392 818";
$resn_exch{"3H39 818"}="H393 818"; $resn_exch{"1H41 818"}="H411 818"; $resn_exch{"2H41 818"}="H412 818";
$resn_exch{"3H41 818"}="H413 818"; $resn_exch{"1H42 818"}="H421 818"; $resn_exch{"2H42 818"}="H422 818";
$resn_exch{"3H42 818"}="H423 818"; $resn_exch{"1H43 818"}="H431 818"; $resn_exch{"2H43 818"}="H432 818";
$resn_exch{"3H43 818"}="H433 818"; $resn_exch{"1H44 818"}="H441 818"; $resn_exch{"2H44 818"}="H442 818";
$resn_exch{"3H44 818"}="H443 818"; $resn_exch{"1H45 818"}="H451 818"; $resn_exch{"2H45 818"}="H452 818";
$resn_exch{"3H45 818"}="H453 818"; $resn_exch{"0HO1 818"}="HO10 818"; $resn_exch{"2HO1 818"}="HO12 818";
$resn_exch{"3HO1 818"}="HO13 818"; $resn_exch{"1H16 81A"}="H161 81A"; $resn_exch{"2H16 81A"}="H162 81A";
$resn_exch{"3H16 81A"}="H163 81A"; $resn_exch{"1H2A 81A"}="H21A 81A"; $resn_exch{"2H2A 81A"}="H22A 81A";
$resn_exch{"3H2A 81A"}="H23A 81A"; $resn_exch{"1H10 81A"}="H101 81A"; $resn_exch{"2H10 81A"}="H102 81A";
$resn_exch{"1H14 81A"}="H141 81A"; $resn_exch{"2H14 81A"}="H142 81A"; $resn_exch{"3H14 81A"}="H143 81A";
$resn_exch{"4H14 81A"}="H144 81A"; $resn_exch{"1H15 81A"}="H151 81A"; $resn_exch{"2H15 81A"}="H152 81A";
$resn_exch{"1H5  821"}=" H51 821"; $resn_exch{"2H5  821"}=" H52 821"; $resn_exch{"3H5  821"}=" H53 821";
$resn_exch{"1H15 821"}="H151 821"; $resn_exch{"2H15 821"}="H152 821"; $resn_exch{"1H16 821"}="H161 821";
$resn_exch{"2H16 821"}="H162 821"; $resn_exch{"1H19 821"}="H191 821"; $resn_exch{"2H19 821"}="H192 821";
$resn_exch{"1H22 821"}="H221 821"; $resn_exch{"2H22 821"}="H222 821"; $resn_exch{"1H30 821"}="H301 821";
$resn_exch{"2H30 821"}="H302 821"; $resn_exch{"1H65 821"}="H651 821"; $resn_exch{"2H65 821"}="H652 821";
$resn_exch{"1H18 822"}="H181 822"; $resn_exch{"2H18 822"}="H182 822"; $resn_exch{"1H28 822"}="H281 822";
$resn_exch{"2H28 822"}="H282 822"; $resn_exch{"3H28 822"}="H283 822"; $resn_exch{"1H38 822"}="H381 822";
$resn_exch{"2H38 822"}="H382 822"; $resn_exch{"1H39 822"}="H391 822"; $resn_exch{"2H39 822"}="H392 822";
$resn_exch{"1H42 822"}="H421 822"; $resn_exch{"2H42 822"}="H422 822"; $resn_exch{"1H46 822"}="H461 822";
$resn_exch{"2H46 822"}="H462 822"; $resn_exch{"1H49 822"}="H491 822"; $resn_exch{"2H49 822"}="H492 822";
$resn_exch{"1H11 826"}="H111 826"; $resn_exch{"2H11 826"}="H112 826"; $resn_exch{"1H12 826"}="H121 826";
$resn_exch{"2H12 826"}="H122 826"; $resn_exch{"1H15 826"}="H151 826"; $resn_exch{"2H15 826"}="H152 826";
$resn_exch{"1H23 826"}="H231 826"; $resn_exch{"2H23 826"}="H232 826"; $resn_exch{"1H2  82A"}="H21A 82A";
$resn_exch{"2H2  82A"}="H22A 82A"; $resn_exch{"1H21 839"}="H211 839"; $resn_exch{"2H21 839"}="H212 839";
$resn_exch{"1H22 839"}="H221 839"; $resn_exch{"2H22 839"}="H222 839"; $resn_exch{"1H11 843"}="H111 843";
$resn_exch{"2H11 843"}="H112 843"; $resn_exch{"1H17 843"}="H171 843"; $resn_exch{"2H17 843"}="H172 843";
$resn_exch{"3H17 843"}="H173 843"; $resn_exch{"1H21 843"}="H211 843"; $resn_exch{"2H21 843"}="H212 843";
$resn_exch{"1H22 843"}="H221 843"; $resn_exch{"2H22 843"}="H222 843"; $resn_exch{"1H23 843"}="H231 843";
$resn_exch{"2H23 843"}="H232 843"; $resn_exch{"1H24 843"}="H241 843"; $resn_exch{"2H24 843"}="H242 843";
$resn_exch{"1H27 843"}="H271 843"; $resn_exch{"2H27 843"}="H272 843"; $resn_exch{"1H2  846"}="H21A 846";
$resn_exch{"2H2  846"}=" H22 846"; $resn_exch{"1H7  846"}=" H71 846"; $resn_exch{"2H7  846"}=" H72 846";
$resn_exch{"1H31 846"}="H311 846"; $resn_exch{"2H31 846"}="H312 846"; $resn_exch{"1H61 846"}="H611 846";
$resn_exch{"2H61 846"}="H612 846"; $resn_exch{"1H71 846"}="H711 846"; $resn_exch{"2H71 846"}="H712 846";
$resn_exch{"1H72 846"}="H721 846"; $resn_exch{"2H72 846"}="H722 846"; $resn_exch{"1H7  852"}=" H71 852";
$resn_exch{"2H7  852"}=" H72 852"; $resn_exch{"1H15 852"}="H151 852"; $resn_exch{"2H15 852"}="H152 852";
$resn_exch{"3H15 852"}="H153 852"; $resn_exch{"1H20 852"}="H201 852"; $resn_exch{"2H20 852"}="H202 852";
$resn_exch{"1H21 852"}="H211 852"; $resn_exch{"2H21 852"}="H212 852"; $resn_exch{"1H22 852"}="H221 852";
$resn_exch{"2H22 852"}="H222 852"; $resn_exch{"1H24 852"}="H241 852"; $resn_exch{"2H24 852"}="H242 852";
$resn_exch{"1H25 852"}="H251 852"; $resn_exch{"2H25 852"}="H252 852"; $resn_exch{"1H10 853"}="H101 853";
$resn_exch{"2H10 853"}="H102 853"; $resn_exch{"1H20 853"}="H201 853"; $resn_exch{"2H20 853"}="H202 853";
$resn_exch{"3H20 853"}="H203 853"; $resn_exch{"1H28 853"}="H281 853"; $resn_exch{"2H28 853"}="H282 853";
$resn_exch{"1H29 853"}="H291 853"; $resn_exch{"2H29 853"}="H292 853"; $resn_exch{"1H32 853"}="H321 853";
$resn_exch{"2H32 853"}="H322 853"; $resn_exch{"1H36 853"}="H361 853"; $resn_exch{"2H36 853"}="H362 853";
$resn_exch{"1H39 853"}="H391 853"; $resn_exch{"2H39 853"}="H392 853"; $resn_exch{"1H71 853"}="H711 853";
$resn_exch{"2H71 853"}="H712 853"; $resn_exch{"1H22 857"}="H221 857"; $resn_exch{"2H22 857"}="H222 857";
$resn_exch{"3H22 857"}="H223 857"; $resn_exch{"1H23 857"}="H231 857"; $resn_exch{"2H23 857"}="H232 857";
$resn_exch{"1H2  857"}="H21A 857"; $resn_exch{"2H2  857"}=" H22 857"; $resn_exch{"1H3  857"}=" H31 857";
$resn_exch{"2H3  857"}=" H32 857"; $resn_exch{"1H4  857"}=" H41 857"; $resn_exch{"2H4  857"}=" H42 857";
$resn_exch{"1H26 857"}="H261 857"; $resn_exch{"2H26 857"}="H262 857"; $resn_exch{"3H26 857"}="H263 857";
$resn_exch{"1H3  858"}="H31A 858"; $resn_exch{"2H3  858"}="H32A 858"; $resn_exch{"1H4  858"}=" H41 858";
$resn_exch{"2H4  858"}=" H42 858"; $resn_exch{"1H5  858"}=" H51 858"; $resn_exch{"2H5  858"}=" H52 858";
$resn_exch{"1H6  858"}=" H61 858"; $resn_exch{"2H6  858"}="H62A 858"; $resn_exch{"1H12 858"}="H121 858";
$resn_exch{"2H12 858"}="H122 858"; $resn_exch{"1H16 858"}="H161 858"; $resn_exch{"2H16 858"}="H162 858";
$resn_exch{"2H18 858"}="H182 858"; $resn_exch{"1H23 858"}="H231 858"; $resn_exch{"2H23 858"}="H232 858";
$resn_exch{"1H30 858"}="H301 858"; $resn_exch{"2H30 858"}="H302 858"; $resn_exch{"1H33 858"}="H331 858";
$resn_exch{"2H33 858"}="H332 858"; $resn_exch{"1H34 858"}="H341 858"; $resn_exch{"2H34 858"}="H342 858";
$resn_exch{"1H35 858"}="H351 858"; $resn_exch{"2H35 858"}="H352 858"; $resn_exch{"3H35 858"}="H353 858";
$resn_exch{"1H36 858"}="H361 858"; $resn_exch{"2H36 858"}="H362 858"; $resn_exch{"3H36 858"}="H363 858";
$resn_exch{"1H37 858"}="H371 858"; $resn_exch{"2H37 858"}="H372 858"; $resn_exch{"3H37 858"}="H373 858";
$resn_exch{"1H38 858"}="H381 858"; $resn_exch{"2H38 858"}="H382 858"; $resn_exch{"1H39 858"}="H391 858";
$resn_exch{"2H39 858"}="H392 858"; $resn_exch{"3H39 858"}="H393 858"; $resn_exch{"1H41 858"}="H411 858";
$resn_exch{"2H41 858"}="H412 858"; $resn_exch{"3H41 858"}="H413 858"; $resn_exch{"1H42 858"}="H421 858";
$resn_exch{"2H42 858"}="H422 858"; $resn_exch{"3H42 858"}="H423 858"; $resn_exch{"1H43 858"}="H431 858";
$resn_exch{"2H43 858"}="H432 858"; $resn_exch{"3H43 858"}="H433 858"; $resn_exch{"1H44 858"}="H441 858";
$resn_exch{"2H44 858"}="H442 858"; $resn_exch{"3H44 858"}="H443 858"; $resn_exch{"1H45 858"}="H451 858";
$resn_exch{"2H45 858"}="H452 858"; $resn_exch{"3H45 858"}="H453 858"; $resn_exch{"1H67 858"}="H671 858";
$resn_exch{"2H67 858"}="H672 858"; $resn_exch{"1H68 858"}="H681 858"; $resn_exch{"2H68 858"}="H682 858";
$resn_exch{"3H68 858"}="H683 858"; $resn_exch{"0HO1 858"}="HO10 858"; $resn_exch{"1H23 859"}="H231 859";
$resn_exch{"2H23 859"}="H232 859"; $resn_exch{"1H22 85A"}="H221 85A"; $resn_exch{"2H22 85A"}="H222 85A";
$resn_exch{"1H26 85A"}="H261 85A"; $resn_exch{"2H26 85A"}="H262 85A"; $resn_exch{"1H27 85A"}="H271 85A";
$resn_exch{"2H27 85A"}="H272 85A"; $resn_exch{"1H23 85A"}="H231 85A"; $resn_exch{"2H23 85A"}="H232 85A";
$resn_exch{"1H24 85A"}="H241 85A"; $resn_exch{"2H24 85A"}="H242 85A"; $resn_exch{"1H28 85A"}="H281 85A";
$resn_exch{"2H28 85A"}="H282 85A"; $resn_exch{"1H9  864"}=" H91 864"; $resn_exch{"2H9  864"}=" H92 864";
$resn_exch{"1H10 864"}="H101 864"; $resn_exch{"2H10 864"}="H102 864"; $resn_exch{"1H15 864"}="H151 864";
$resn_exch{"2H15 864"}="H152 864"; $resn_exch{"1H27 864"}="H271 864"; $resn_exch{"2H27 864"}="H272 864";
$resn_exch{"3H27 864"}="H273 864"; $resn_exch{"1H28 864"}="H281 864"; $resn_exch{"2H28 864"}="H282 864";
$resn_exch{"3H28 864"}="H283 864"; $resn_exch{"1H33 864"}="H331 864"; $resn_exch{"2H33 864"}="H332 864";
$resn_exch{"1H31 869"}="H311 869"; $resn_exch{"2H31 869"}="H312 869"; $resn_exch{"3H31 869"}="H313 869";
$resn_exch{"1H7  869"}=" H71 869"; $resn_exch{"2H7  869"}=" H72 869"; $resn_exch{"3H7  869"}=" H73 869";
$resn_exch{"1H14 869"}="H141 869"; $resn_exch{"2H14 869"}="H142 869"; $resn_exch{"3H14 869"}="H143 869";
$resn_exch{"1H21 869"}="H211 869"; $resn_exch{"2H21 869"}="H212 869"; $resn_exch{"3H21 869"}="H213 869";
$resn_exch{"1H22 869"}="H221 869"; $resn_exch{"2H22 869"}="H222 869"; $resn_exch{"3H22 869"}="H223 869";
$resn_exch{"1H23 869"}="H231 869"; $resn_exch{"2H23 869"}="H232 869"; $resn_exch{"3H23 869"}="H233 869";
$resn_exch{"1H33 86A"}="H331 86A"; $resn_exch{"2H33 86A"}="H332 86A"; $resn_exch{"1H24 86A"}="H241 86A";
$resn_exch{"2H24 86A"}="H242 86A"; $resn_exch{"1H28 86A"}="H281 86A"; $resn_exch{"2H28 86A"}="H282 86A";
$resn_exch{"3H28 86A"}="H283 86A"; $resn_exch{"1H27 86A"}="H271 86A"; $resn_exch{"2H27 86A"}="H272 86A";
$resn_exch{"3H27 86A"}="H273 86A"; $resn_exch{"1H11 86A"}="H111 86A"; $resn_exch{"2H11 86A"}="H112 86A";
$resn_exch{"1H13 870"}="H131 870"; $resn_exch{"2H13 870"}="H132 870"; $resn_exch{"3H13 870"}="H133 870";
$resn_exch{"1HN2 870"}="HN21 870"; $resn_exch{"2HN2 870"}="HN22 870"; $resn_exch{"1H11 872"}="H111 872";
$resn_exch{"2H11 872"}="H112 872"; $resn_exch{"1H15 872"}="H151 872"; $resn_exch{"2H15 872"}="H152 872";
$resn_exch{"1H20 872"}="H201 872"; $resn_exch{"2H20 872"}="H202 872"; $resn_exch{"1H25 872"}="H251 872";
$resn_exch{"2H25 872"}="H252 872"; $resn_exch{"1H26 872"}="H261 872"; $resn_exch{"2H26 872"}="H262 872";
$resn_exch{"1H27 872"}="H271 872"; $resn_exch{"2H27 872"}="H272 872"; $resn_exch{"1H37 872"}="H371 872";
$resn_exch{"2H37 872"}="H372 872"; $resn_exch{"1H7  876"}=" H71 876"; $resn_exch{"2H7  876"}="H72A 876";
$resn_exch{"1H13 876"}="H131 876"; $resn_exch{"2H13 876"}="H132 876"; $resn_exch{"3H13 876"}="H133 876";
$resn_exch{"1H17 876"}="H171 876"; $resn_exch{"2H17 876"}="H172 876"; $resn_exch{"1H18 876"}="H181 876";
$resn_exch{"2H18 876"}="H182 876"; $resn_exch{"1H19 876"}="H191 876"; $resn_exch{"2H19 876"}="H192 876";
$resn_exch{"1H20 876"}="H201 876"; $resn_exch{"2H20 876"}="H202 876"; $resn_exch{"1H23 876"}="H231 876";
$resn_exch{"2H23 876"}="H232 876"; $resn_exch{"1H9  877"}=" H91 877"; $resn_exch{"2H9  877"}=" H92 877";
$resn_exch{"1H23 877"}="H231 877"; $resn_exch{"2H23 877"}="H232 877"; $resn_exch{"3H23 877"}="H233 877";
$resn_exch{"1H28 877"}="H281 877"; $resn_exch{"2H28 877"}="H282 877"; $resn_exch{"3H28 877"}="H283 877";
$resn_exch{"1H33 877"}="H331 877"; $resn_exch{"2H33 877"}="H332 877"; $resn_exch{"1H37 877"}="H371 877";
$resn_exch{"2H37 877"}="H372 877"; $resn_exch{"1H38 877"}="H381 877"; $resn_exch{"2H38 877"}="H382 877";
$resn_exch{"3H38 877"}="H383 877"; $resn_exch{"2H15 87Y"}="H152 87Y"; $resn_exch{"1H15 87Y"}="H151 87Y";
$resn_exch{"2H14 87Y"}="H142 87Y"; $resn_exch{"1H14 87Y"}="H141 87Y"; $resn_exch{"3H13 87Y"}="H133 87Y";
$resn_exch{"2H13 87Y"}="H132 87Y"; $resn_exch{"1H13 87Y"}="H131 87Y"; $resn_exch{"2H5  87Y"}=" H52 87Y";
$resn_exch{"1H5  87Y"}=" H51 87Y"; $resn_exch{"2H17 87Y"}="H172 87Y"; $resn_exch{"1H17 87Y"}="H171 87Y";
$resn_exch{"1H06 880"}="H061 880"; $resn_exch{"2H06 880"}="H062 880"; $resn_exch{"1H07 880"}="H071 880";
$resn_exch{"2H07 880"}="H072 880"; $resn_exch{"1H10 880"}="H101 880"; $resn_exch{"2H10 880"}="H102 880";
$resn_exch{"3H10 880"}="H103 880"; $resn_exch{"1H17 880"}="H171 880"; $resn_exch{"2H17 880"}="H172 880";
$resn_exch{"1H18 880"}="H181 880"; $resn_exch{"2H18 880"}="H182 880"; $resn_exch{"1H20 880"}="H201 880";
$resn_exch{"2H20 880"}="H202 880"; $resn_exch{"1H21 880"}="H211 880"; $resn_exch{"2H21 880"}="H212 880";
$resn_exch{"1H56 880"}="H561 880"; $resn_exch{"2H56 880"}="H562 880"; $resn_exch{"1H57 880"}="H571 880";
$resn_exch{"2H57 880"}="H572 880"; $resn_exch{"1H58 880"}="H581 880"; $resn_exch{"2H58 880"}="H582 880";
$resn_exch{"1H59 880"}="H591 880"; $resn_exch{"2H59 880"}="H592 880"; $resn_exch{"1H60 880"}="H601 880";
$resn_exch{"2H60 880"}="H602 880"; $resn_exch{"1H7  882"}=" H71 882"; $resn_exch{"2H7  882"}=" H72 882";
$resn_exch{"1H8  882"}=" H81 882"; $resn_exch{"2H8  882"}=" H82 882"; $resn_exch{"1H10 882"}="H101 882";
$resn_exch{"2H10 882"}="H102 882"; $resn_exch{"1H13 882"}="H131 882"; $resn_exch{"2H13 882"}="H132 882";
$resn_exch{"3H13 882"}="H133 882"; $resn_exch{"1H35 882"}="H351 882"; $resn_exch{"2H35 882"}="H352 882";
$resn_exch{"1H14 882"}="H141 882"; $resn_exch{"2H14 882"}="H142 882"; $resn_exch{"3H14 882"}="H143 882";
$resn_exch{"1H1  885"}="H11A 885"; $resn_exch{"2H1  885"}=" H12 885"; $resn_exch{"3H1  885"}=" H13 885";
$resn_exch{"1HN1 885"}="HN11 885"; $resn_exch{"2HN1 885"}="HN12 885"; $resn_exch{"1H8  885"}=" H81 885";
$resn_exch{"2H8  885"}=" H82 885"; $resn_exch{"1H9  885"}=" H91 885"; $resn_exch{"2H9  885"}=" H92 885";
$resn_exch{"1H10 885"}="H101 885"; $resn_exch{"2H10 885"}="H102 885"; $resn_exch{"1H31 888"}="H311 888";
$resn_exch{"2H31 888"}="H312 888"; $resn_exch{"1H32 888"}="H321 888"; $resn_exch{"2H32 888"}="H322 888";
$resn_exch{"1H38 888"}="H381 888"; $resn_exch{"2H38 888"}="H382 888"; $resn_exch{"3H38 888"}="H383 888";
$resn_exch{"1H7  888"}=" H71 888"; $resn_exch{"2H7  888"}=" H72 888"; $resn_exch{"3H7  888"}=" H73 888";
$resn_exch{"1H17 892"}="H171 892"; $resn_exch{"2H17 892"}="H172 892"; $resn_exch{"1H18 892"}="H181 892";
$resn_exch{"2H18 892"}="H182 892"; $resn_exch{"3H18 892"}="H183 892"; $resn_exch{"1H02 892"}="H021 892";
$resn_exch{"2H02 892"}="H022 892"; $resn_exch{"1H9  893"}=" H91 893"; $resn_exch{"2H9  893"}=" H92 893";
$resn_exch{"1H23 893"}="H231 893"; $resn_exch{"2H23 893"}="H232 893"; $resn_exch{"3H23 893"}="H233 893";
$resn_exch{"1H28 893"}="H281 893"; $resn_exch{"2H28 893"}="H282 893"; $resn_exch{"3H28 893"}="H283 893";
$resn_exch{"1H37 893"}="H371 893"; $resn_exch{"2H37 893"}="H372 893"; $resn_exch{"1H7  893"}=" H71 893";
$resn_exch{"2H7  893"}=" H72 893"; $resn_exch{"3H7  893"}=" H73 893"; $resn_exch{"1H9C 894"}="H9C1 894";
$resn_exch{"2H9C 894"}="H9C2 894"; $resn_exch{"1H6C 894"}="H6C1 894"; $resn_exch{"2H6C 894"}="H6C2 894";
$resn_exch{"1H13 894"}="H131 894"; $resn_exch{"2H13 894"}="H132 894"; $resn_exch{"1H14 894"}="H141 894";
$resn_exch{"2H14 894"}="H142 894"; $resn_exch{"1H17 894"}="H171 894"; $resn_exch{"2H17 894"}="H172 894";
$resn_exch{"3H17 894"}="H173 894"; $resn_exch{"1H19 894"}="H191 894"; $resn_exch{"2H19 894"}="H192 894";
$resn_exch{"1H23 894"}="H231 894"; $resn_exch{"2H23 894"}="H232 894"; $resn_exch{"1H20 894"}="H201 894";
$resn_exch{"2H20 894"}="H202 894"; $resn_exch{"1H22 894"}="H221 894"; $resn_exch{"2H22 894"}="H222 894";
$resn_exch{"1H13 895"}="H131 895"; $resn_exch{"2H13 895"}="H132 895"; $resn_exch{"1H14 895"}="H141 895";
$resn_exch{"2H14 895"}="H142 895"; $resn_exch{"1H17 895"}="H171 895"; $resn_exch{"2H17 895"}="H172 895";
$resn_exch{"3H17 895"}="H173 895"; $resn_exch{"1H19 895"}="H191 895"; $resn_exch{"2H19 895"}="H192 895";
$resn_exch{"1H23 895"}="H231 895"; $resn_exch{"2H23 895"}="H232 895"; $resn_exch{"1H20 895"}="H201 895";
$resn_exch{"2H20 895"}="H202 895"; $resn_exch{"1H22 895"}="H221 895"; $resn_exch{"2H22 895"}="H222 895";
$resn_exch{"1H6C 896"}="H6C1 896"; $resn_exch{"2H6C 896"}="H6C2 896"; $resn_exch{"1H5C 896"}="H5C1 896";
$resn_exch{"2H5C 896"}="H5C2 896"; $resn_exch{"3H5C 896"}="H5C3 896"; $resn_exch{"1H24 896"}="H241 896";
$resn_exch{"2H24 896"}="H242 896"; $resn_exch{"3H24 896"}="H243 896"; $resn_exch{"1H23 896"}="H231 896";
$resn_exch{"2H23 896"}="H232 896"; $resn_exch{"3H23 896"}="H233 896"; $resn_exch{"1H20 896"}="H201 896";
$resn_exch{"2H20 896"}="H202 896"; $resn_exch{"3H20 896"}="H203 896"; $resn_exch{"1H1C 896"}="H1C1 896";
$resn_exch{"2H1C 896"}="H1C2 896"; $resn_exch{"1H8C 896"}="H8C1 896"; $resn_exch{"2H8C 896"}="H8C2 896";
$resn_exch{"3H8C 896"}="H8C3 896"; $resn_exch{"1H8C 897"}="H8C1 897"; $resn_exch{"2H8C 897"}="H8C2 897";
$resn_exch{"3H8C 897"}="H8C3 897"; $resn_exch{"1H1C 897"}="H1C1 897"; $resn_exch{"2H1C 897"}="H1C2 897";
$resn_exch{"1H20 897"}="H201 897"; $resn_exch{"2H20 897"}="H202 897"; $resn_exch{"3H20 897"}="H203 897";
$resn_exch{"1H6C 897"}="H6C1 897"; $resn_exch{"2H6C 897"}="H6C2 897"; $resn_exch{"1H23 897"}="H231 897";
$resn_exch{"2H23 897"}="H232 897"; $resn_exch{"3H23 897"}="H233 897"; $resn_exch{"1H24 897"}="H241 897";
$resn_exch{"2H24 897"}="H242 897"; $resn_exch{"3H24 897"}="H243 897"; $resn_exch{"1H5C 897"}="H5C1 897";
$resn_exch{"2H5C 897"}="H5C2 897"; $resn_exch{"1H25 897"}="H251 897"; $resn_exch{"2H25 897"}="H252 897";
$resn_exch{"3H25 897"}="H253 897"; $resn_exch{"1H26 897"}="H261 897"; $resn_exch{"2H26 897"}="H262 897";
$resn_exch{"3H26 897"}="H263 897"; $resn_exch{"1H2C 897"}="H2C1 897"; $resn_exch{"2H2C 897"}="H2C2 897";
$resn_exch{"3H2C 897"}="H2C3 897"; $resn_exch{"1H31 89I"}="H311 89I"; $resn_exch{"2H31 89I"}="H312 89I";
$resn_exch{"1H30 89I"}="H301 89I"; $resn_exch{"2H30 89I"}="H302 89I"; $resn_exch{"1H28 89I"}="H281 89I";
$resn_exch{"2H28 89I"}="H282 89I"; $resn_exch{"1H27 89I"}="H271 89I"; $resn_exch{"2H27 89I"}="H272 89I";
$resn_exch{"1H44 89I"}="H441 89I"; $resn_exch{"2H44 89I"}="H442 89I"; $resn_exch{"3H44 89I"}="H443 89I";
$resn_exch{"1H48 89I"}="H481 89I"; $resn_exch{"2H48 89I"}="H482 89I"; $resn_exch{"3H48 89I"}="H483 89I";
$resn_exch{"1H52 89I"}="H521 89I"; $resn_exch{"2H52 89I"}="H522 89I"; $resn_exch{"1HD2 8AD"}="HD21 8AD";
$resn_exch{"2HD2 8AD"}="HD22 8AD"; $resn_exch{"1HD3 8AD"}="HD31 8AD"; $resn_exch{"2HD3 8AD"}="HD32 8AD";
$resn_exch{"1HD4 8AD"}="HD41 8AD"; $resn_exch{"2HD4 8AD"}="HD42 8AD"; $resn_exch{"1HD5 8AD"}="HD51 8AD";
$resn_exch{"2HD5 8AD"}="HD52 8AD"; $resn_exch{"1HD7 8AD"}="HD71 8AD"; $resn_exch{"2HD7 8AD"}="HD72 8AD";
$resn_exch{"3HD7 8AD"}="HD73 8AD"; $resn_exch{"1HD8 8AD"}="HD81 8AD"; $resn_exch{"2HD8 8AD"}="HD82 8AD";
$resn_exch{"3HD8 8AD"}="HD83 8AD"; $resn_exch{"1HN1 8AP"}="HN11 8AP"; $resn_exch{"2HN1 8AP"}="HN12 8AP";
$resn_exch{"1H9  8AP"}=" H91 8AP"; $resn_exch{"2H9  8AP"}=" H92 8AP"; $resn_exch{"2HOP 8BR"}="HOP2 8BR";
$resn_exch{"3HOP 8BR"}="HOP3 8BR"; $resn_exch{"1H5* 8BR"}="H5'1 8BR"; $resn_exch{"2H5* 8BR"}="H5'2 8BR";
$resn_exch{" H4* 8BR"}=" H4' 8BR"; $resn_exch{" H3* 8BR"}=" H3' 8BR"; $resn_exch{"*HO3 8BR"}="HO3' 8BR";
$resn_exch{" H2* 8BR"}=" H2' 8BR"; $resn_exch{"*HO2 8BR"}="HO2' 8BR"; $resn_exch{" H1* 8BR"}=" H1' 8BR";
$resn_exch{"1HN6 8BR"}="HN61 8BR"; $resn_exch{"2HN6 8BR"}="HN62 8BR"; $resn_exch{"1H2' 8CS"}="H2'1 8CS";
$resn_exch{"2H2' 8CS"}="H2'2 8CS"; $resn_exch{"1H10 8CS"}="H101 8CS"; $resn_exch{"2H10 8CS"}="H102 8CS";
$resn_exch{"1H5* 8DA"}="H5'1 8DA"; $resn_exch{"2H5* 8DA"}="H5'2 8DA"; $resn_exch{" H4* 8DA"}=" H4' 8DA";
$resn_exch{" H3* 8DA"}=" H3' 8DA"; $resn_exch{" H1* 8DA"}=" H1' 8DA"; $resn_exch{"1H2* 8DA"}="H2'1 8DA";
$resn_exch{"2H2* 8DA"}="H2'2 8DA"; $resn_exch{"1HN6 8DA"}="HN61 8DA"; $resn_exch{"2HN6 8DA"}="HN62 8DA";
$resn_exch{"1H2  8DA"}=" H21 8DA"; $resn_exch{"2H2  8DA"}=" H22 8DA"; $resn_exch{"2HOG 8DG"}="HOG2 8DG";
$resn_exch{"2HOB 8DG"}="HOB2 8DG"; $resn_exch{"2HOA 8DG"}="HOA2 8DG"; $resn_exch{"1H5* 8DG"}="H5'1 8DG";
$resn_exch{"2H5* 8DG"}="H5'2 8DG"; $resn_exch{" H4* 8DG"}=" H4' 8DG"; $resn_exch{" H3* 8DG"}=" H3' 8DG";
$resn_exch{"1H2* 8DG"}="H2'1 8DG"; $resn_exch{"2H2* 8DG"}="H2'2 8DG"; $resn_exch{" H1* 8DG"}=" H1' 8DG";
$resn_exch{"1HN2 8DG"}="HN21 8DG"; $resn_exch{"2HN2 8DG"}="HN22 8DG"; $resn_exch{"3HOP 8FG"}="HOP3 8FG";
$resn_exch{"2HOP 8FG"}="HOP2 8FG"; $resn_exch{"1HN2 8FG"}="HN21 8FG"; $resn_exch{"2HN2 8FG"}="HN22 8FG";
$resn_exch{"1H37 8FG"}="H371 8FG"; $resn_exch{"2H37 8FG"}="H372 8FG"; $resn_exch{"1H45 8FG"}="H451 8FG";
$resn_exch{"2H45 8FG"}="H452 8FG"; $resn_exch{"3H45 8FG"}="H453 8FG"; $resn_exch{"1H2* 8FG"}=" H2' 8FG";
$resn_exch{"2H2* 8FG"}="H2'' 8FG"; $resn_exch{"1H5* 8FG"}=" H5' 8FG"; $resn_exch{"2H5* 8FG"}="H5'' 8FG";
$resn_exch{" H4* 8FG"}=" H4' 8FG"; $resn_exch{" H1* 8FG"}=" H1' 8FG"; $resn_exch{" H3* 8FG"}=" H3' 8FG";
$resn_exch{"H3T  8FG"}="HO3' 8FG"; $resn_exch{"1H6  8GP"}=" H61 8GP"; $resn_exch{"2H6  8GP"}=" H62 8GP";
$resn_exch{"1H10 8GP"}="H101 8GP"; $resn_exch{"2H10 8GP"}="H102 8GP"; $resn_exch{"1H11 8GP"}="H111 8GP";
$resn_exch{"2H11 8GP"}="H112 8GP"; $resn_exch{"2HOG 8GT"}="HOG2 8GT"; $resn_exch{"2HOB 8GT"}="HOB2 8GT";
$resn_exch{"2HOA 8GT"}="HOA2 8GT"; $resn_exch{"1H5* 8GT"}="H5'1 8GT"; $resn_exch{"2H5* 8GT"}="H5'2 8GT";
$resn_exch{" H4* 8GT"}=" H4' 8GT"; $resn_exch{" H3* 8GT"}=" H3' 8GT"; $resn_exch{" H2* 8GT"}=" H2' 8GT";
$resn_exch{" H1* 8GT"}=" H1' 8GT"; $resn_exch{"1HN2 8GT"}="HN21 8GT"; $resn_exch{"2HN2 8GT"}="HN22 8GT";
$resn_exch{"1HN2 8HG"}="HN21 8HG"; $resn_exch{"2HN2 8HG"}="HN22 8HG"; $resn_exch{" H1* 8HG"}=" H1' 8HG";
$resn_exch{"1H2* 8HG"}="H2'1 8HG"; $resn_exch{"2H2* 8HG"}="H2'2 8HG"; $resn_exch{" H3* 8HG"}=" H3' 8HG";
$resn_exch{" H4* 8HG"}=" H4' 8HG"; $resn_exch{"1H5* 8HG"}="H5'1 8HG"; $resn_exch{"2H5* 8HG"}="H5'2 8HG";
$resn_exch{" H5* 8HG"}=" H5' 8HG"; $resn_exch{"1H7N 8ID"}="H7N1 8ID"; $resn_exch{"2H7N 8ID"}="H7N2 8ID";
$resn_exch{"1H5' 8ID"}="H5'1 8ID"; $resn_exch{"2H5' 8ID"}="H5'2 8ID"; $resn_exch{"1H5* 8ID"}="H5A1 8ID";
$resn_exch{"2H5* 8ID"}="H5A2 8ID"; $resn_exch{" H4* 8ID"}=" H4B 8ID"; $resn_exch{" H1* 8ID"}=" H1B 8ID";
$resn_exch{" H2* 8ID"}=" H2B 8ID"; $resn_exch{"HO2* 8ID"}="HO2A 8ID"; $resn_exch{" H3* 8ID"}=" H3B 8ID";
$resn_exch{"HO3* 8ID"}="HO3A 8ID"; $resn_exch{"1H6A 8ID"}="H6A1 8ID"; $resn_exch{"2H6A 8ID"}="H6A2 8ID";
$resn_exch{"1HN2 8IG"}="HN21 8IG"; $resn_exch{"2HN2 8IG"}="HN22 8IG"; $resn_exch{"1HN1 8IG"}="HN11 8IG";
$resn_exch{"1H10 8IN"}="H101 8IN"; $resn_exch{"2H10 8IN"}="H102 8IN"; $resn_exch{"1H13 8IN"}="H131 8IN";
$resn_exch{"2H13 8IN"}="H132 8IN"; $resn_exch{"1H14 8IN"}="H141 8IN"; $resn_exch{"2H14 8IN"}="H142 8IN";
$resn_exch{"3H14 8IN"}="H143 8IN"; $resn_exch{"1H15 8IN"}="H151 8IN"; $resn_exch{"2H15 8IN"}="H152 8IN";
$resn_exch{"1H23 8IN"}="H231 8IN"; $resn_exch{"2H23 8IN"}="H232 8IN"; $resn_exch{"1H24 8IN"}="H241 8IN";
$resn_exch{"2H24 8IN"}="H242 8IN"; $resn_exch{"1H25 8IN"}="H251 8IN"; $resn_exch{"2H25 8IN"}="H252 8IN";
$resn_exch{"1HN1 8IP"}="HN11 8IP"; $resn_exch{"2HN1 8IP"}="HN12 8IP"; $resn_exch{"1H9  8IP"}=" H91 8IP";
$resn_exch{"2H9  8IP"}=" H92 8IP"; $resn_exch{"1H2  8MG"}=" H21 8MG"; $resn_exch{"2H2  8MG"}=" H22 8MG";
$resn_exch{"1H2* 8MG"}=" H2' 8MG"; $resn_exch{"2H2* 8MG"}="H2'' 8MG"; $resn_exch{"1H5* 8MG"}=" H5' 8MG";
$resn_exch{"2H5* 8MG"}="H5'' 8MG"; $resn_exch{" H4* 8MG"}=" H4' 8MG"; $resn_exch{" H1* 8MG"}=" H1' 8MG";
$resn_exch{" H3* 8MG"}=" H3' 8MG"; $resn_exch{"H3T  8MG"}="HO3' 8MG"; $resn_exch{"1HC  8MG"}=" HC1 8MG";
$resn_exch{"2HC  8MG"}=" HC2 8MG"; $resn_exch{"3HC  8MG"}=" HC3 8MG"; $resn_exch{"1H4  8MO"}=" H41 8MO";
$resn_exch{"2H4  8MO"}=" H42 8MO"; $resn_exch{"3H4  8MO"}=" H43 8MO"; $resn_exch{"1HAI 8MR"}="HAI1 8MR";
$resn_exch{"2HAI 8MR"}="HAI2 8MR"; $resn_exch{"3HAI 8MR"}="HAI3 8MR"; $resn_exch{"1HAL 8MR"}="HAL1 8MR";
$resn_exch{"2HAL 8MR"}="HAL2 8MR"; $resn_exch{"1H19 8NH"}="H191 8NH"; $resn_exch{"2H19 8NH"}="H192 8NH";
$resn_exch{"1H22 8NH"}="H221 8NH"; $resn_exch{"2H22 8NH"}="H222 8NH"; $resn_exch{"1H30 8NH"}="H301 8NH";
$resn_exch{"2H30 8NH"}="H302 8NH"; $resn_exch{"3H30 8NH"}="H303 8NH"; $resn_exch{"3HOP 8OG"}="HOP3 8OG";
$resn_exch{"2HOP 8OG"}="HOP2 8OG"; $resn_exch{"1H5* 8OG"}=" H5' 8OG"; $resn_exch{"2H5* 8OG"}="H5'' 8OG";
$resn_exch{" H4* 8OG"}=" H4' 8OG"; $resn_exch{" H3* 8OG"}=" H3' 8OG"; $resn_exch{"H3T  8OG"}="HO3' 8OG";
$resn_exch{"1H2* 8OG"}=" H2' 8OG"; $resn_exch{"2H2* 8OG"}="H2'' 8OG"; $resn_exch{" H1* 8OG"}=" H1' 8OG";
$resn_exch{"1H2  8OG"}=" H21 8OG"; $resn_exch{"2H2  8OG"}=" H22 8OG"; $resn_exch{"1H7  8PC"}=" H71 8PC";
$resn_exch{"2H7  8PC"}=" H72 8PC"; $resn_exch{"1HAA 8PG"}="HAA1 8PG"; $resn_exch{"2HAA 8PG"}="HAA2 8PG";
$resn_exch{"3HAA 8PG"}="HAA3 8PG"; $resn_exch{"1HAJ 8PG"}="HAJ1 8PG"; $resn_exch{"2HAJ 8PG"}="HAJ2 8PG";
$resn_exch{"1HAK 8PG"}="HAK1 8PG"; $resn_exch{"2HAK 8PG"}="HAK2 8PG"; $resn_exch{"1HAO 8PG"}="HAO1 8PG";
$resn_exch{"2HAO 8PG"}="HAO2 8PG"; $resn_exch{"1HAR 8PG"}="HAR1 8PG"; $resn_exch{"2HAR 8PG"}="HAR2 8PG";
$resn_exch{"1HAS 8PG"}="HAS1 8PG"; $resn_exch{"2HAS 8PG"}="HAS2 8PG"; $resn_exch{"1HAP 8PG"}="HAP1 8PG";
$resn_exch{"2HAP 8PG"}="HAP2 8PG"; $resn_exch{"1HAM 8PG"}="HAM1 8PG"; $resn_exch{"2HAM 8PG"}="HAM2 8PG";
$resn_exch{"1HAL 8PG"}="HAL1 8PG"; $resn_exch{"2HAL 8PG"}="HAL2 8PG"; $resn_exch{"1HAN 8PG"}="HAN1 8PG";
$resn_exch{"2HAN 8PG"}="HAN2 8PG"; $resn_exch{"1HAQ 8PG"}="HAQ1 8PG"; $resn_exch{"2HAQ 8PG"}="HAQ2 8PG";
$resn_exch{"1H7  8PP"}=" H71 8PP"; $resn_exch{"2H7  8PP"}=" H72 8PP"; $resn_exch{"1H10 8PP"}="H101 8PP";
$resn_exch{"2H10 8PP"}="H102 8PP"; $resn_exch{"3H10 8PP"}="H103 8PP"; $resn_exch{"1H11 8PP"}="H111 8PP";
$resn_exch{"2H11 8PP"}="H112 8PP"; $resn_exch{"1H12 8PP"}="H121 8PP"; $resn_exch{"2H12 8PP"}="H122 8PP";
$resn_exch{"1H15 8PP"}="H151 8PP"; $resn_exch{"2H15 8PP"}="H152 8PP"; $resn_exch{"3H15 8PP"}="H153 8PP";
$resn_exch{"1H16 8PP"}="H161 8PP"; $resn_exch{"2H16 8PP"}="H162 8PP"; $resn_exch{"1H17 8PP"}="H171 8PP";
$resn_exch{"2H17 8PP"}="H172 8PP"; $resn_exch{"1H20 8PP"}="H201 8PP"; $resn_exch{"2H20 8PP"}="H202 8PP";
$resn_exch{"3H20 8PP"}="H203 8PP"; $resn_exch{"1H21 8PP"}="H211 8PP"; $resn_exch{"2H21 8PP"}="H212 8PP";
$resn_exch{"1H22 8PP"}="H221 8PP"; $resn_exch{"2H22 8PP"}="H222 8PP"; $resn_exch{"1H25 8PP"}="H251 8PP";
$resn_exch{"2H25 8PP"}="H252 8PP"; $resn_exch{"3H25 8PP"}="H253 8PP"; $resn_exch{"1H26 8PP"}="H261 8PP";
$resn_exch{"2H26 8PP"}="H262 8PP"; $resn_exch{"1H27 8PP"}="H271 8PP"; $resn_exch{"2H27 8PP"}="H272 8PP";
$resn_exch{"1H30 8PP"}="H301 8PP"; $resn_exch{"2H30 8PP"}="H302 8PP"; $resn_exch{"3H30 8PP"}="H303 8PP";
$resn_exch{"1H31 8PP"}="H311 8PP"; $resn_exch{"2H31 8PP"}="H312 8PP"; $resn_exch{"1H32 8PP"}="H321 8PP";
$resn_exch{"2H32 8PP"}="H322 8PP"; $resn_exch{"1H35 8PP"}="H351 8PP"; $resn_exch{"2H35 8PP"}="H352 8PP";
$resn_exch{"3H35 8PP"}="H353 8PP"; $resn_exch{"1H36 8PP"}="H361 8PP"; $resn_exch{"2H36 8PP"}="H362 8PP";
$resn_exch{"1H37 8PP"}="H371 8PP"; $resn_exch{"2H37 8PP"}="H372 8PP"; $resn_exch{"1H40 8PP"}="H401 8PP";
$resn_exch{"2H40 8PP"}="H402 8PP"; $resn_exch{"1H41 8PP"}="H411 8PP"; $resn_exch{"2H41 8PP"}="H412 8PP";
$resn_exch{"3H41 8PP"}="H413 8PP"; $resn_exch{"1H42 8PP"}="H421 8PP"; $resn_exch{"2H42 8PP"}="H422 8PP";
$resn_exch{"1H45 8PP"}="H451 8PP"; $resn_exch{"2H45 8PP"}="H452 8PP"; $resn_exch{"3H45 8PP"}="H453 8PP";
$resn_exch{"1H46 8PP"}="H461 8PP"; $resn_exch{"2H46 8PP"}="H462 8PP"; $resn_exch{"3H46 8PP"}="H463 8PP";
$resn_exch{"1H14 8PS"}="H141 8PS"; $resn_exch{"2H14 8PS"}="H142 8PS"; $resn_exch{"1H15 8PS"}="H151 8PS";
$resn_exch{"2H15 8PS"}="H152 8PS"; $resn_exch{"1H16 8PS"}="H161 8PS"; $resn_exch{"2H16 8PS"}="H162 8PS";
$resn_exch{"1H17 8PS"}="H171 8PS"; $resn_exch{"2H17 8PS"}="H172 8PS"; $resn_exch{"1H18 8PS"}="H181 8PS";
$resn_exch{"2H18 8PS"}="H182 8PS"; $resn_exch{"1H19 8PS"}="H191 8PS"; $resn_exch{"2H19 8PS"}="H192 8PS";
$resn_exch{"1H20 8PS"}="H201 8PS"; $resn_exch{"2H20 8PS"}="H202 8PS"; $resn_exch{"1H21 8PS"}="H211 8PS";
$resn_exch{"2H21 8PS"}="H212 8PS"; $resn_exch{"3H21 8PS"}="H213 8PS"; $resn_exch{"1H32 901"}="H321 901";
$resn_exch{"2H32 901"}="H322 901"; $resn_exch{"1H33 901"}="H331 901"; $resn_exch{"2H33 901"}="H332 901";
$resn_exch{"3H33 901"}="H333 901"; $resn_exch{"1H7  901"}=" H71 901"; $resn_exch{"2H7  901"}=" H72 901";
$resn_exch{"1H23 901"}="H231 901"; $resn_exch{"2H23 901"}="H232 901"; $resn_exch{"3H23 901"}="H233 901";
$resn_exch{"1H12 901"}="H121 901"; $resn_exch{"2H12 901"}="H122 901"; $resn_exch{"1H13 901"}="H131 901";
$resn_exch{"2H13 901"}="H132 901"; $resn_exch{"1H15 901"}="H151 901"; $resn_exch{"2H15 901"}="H152 901";
$resn_exch{"1H14 901"}="H141 901"; $resn_exch{"2H14 901"}="H142 901"; $resn_exch{"1H18 901"}="H181 901";
$resn_exch{"2H18 901"}="H182 901"; $resn_exch{"1H34 901"}="H341 901"; $resn_exch{"2H34 901"}="H342 901";
$resn_exch{"1H35 901"}="H351 901"; $resn_exch{"2H35 901"}="H352 901"; $resn_exch{"3H35 901"}="H353 901";
$resn_exch{"1H21 902"}="H211 902"; $resn_exch{"2H21 902"}="H212 902"; $resn_exch{"1H30 902"}="H301 902";
$resn_exch{"2H30 902"}="H302 902"; $resn_exch{"1H31 902"}="H311 902"; $resn_exch{"2H31 902"}="H312 902";
$resn_exch{"1H33 902"}="H331 902"; $resn_exch{"2H33 902"}="H332 902"; $resn_exch{"1H34 902"}="H341 902";
$resn_exch{"2H34 902"}="H342 902"; $resn_exch{"1H47 902"}="H471 902"; $resn_exch{"2H47 902"}="H472 902";
$resn_exch{"1H7  903"}=" H71 903"; $resn_exch{"2H7  903"}=" H72 903"; $resn_exch{"1H15 903"}="H151 903";
$resn_exch{"2H15 903"}="H152 903"; $resn_exch{"3H15 903"}="H153 903"; $resn_exch{"1H20 903"}="H201 903";
$resn_exch{"2H20 903"}="H202 903"; $resn_exch{"1H21 903"}="H211 903"; $resn_exch{"2H21 903"}="H212 903";
$resn_exch{"1H22 903"}="H221 903"; $resn_exch{"2H22 903"}="H222 903"; $resn_exch{"1H24 903"}="H241 903";
$resn_exch{"2H24 903"}="H242 903"; $resn_exch{"1H25 903"}="H251 903"; $resn_exch{"2H25 903"}="H252 903";
$resn_exch{"1H81 903"}="H811 903"; $resn_exch{"2H81 903"}="H812 903"; $resn_exch{"3H81 903"}="H813 903";
$resn_exch{"1HN3 905"}="HN31 905"; $resn_exch{"2HN3 905"}="HN32 905"; $resn_exch{"1H15 905"}="H151 905";
$resn_exch{"2H15 905"}="H152 905"; $resn_exch{"1H16 905"}="H161 905"; $resn_exch{"2H16 905"}="H162 905";
$resn_exch{"3H16 905"}="H163 905"; $resn_exch{"1H19 905"}="H191 905"; $resn_exch{"2H19 905"}="H192 905";
$resn_exch{"1H20 905"}="H201 905"; $resn_exch{"2H20 905"}="H202 905"; $resn_exch{"3H20 905"}="H203 905";
$resn_exch{"1HN5 905"}="HN51 905"; $resn_exch{"2HN5 905"}="HN52 905"; $resn_exch{"1H21 910"}="H211 910";
$resn_exch{"2H21 910"}="H212 910"; $resn_exch{"1H30 910"}="H301 910"; $resn_exch{"2H30 910"}="H302 910";
$resn_exch{"1H11 915"}="H111 915"; $resn_exch{"2H11 915"}="H112 915"; $resn_exch{"3H11 915"}="H113 915";
$resn_exch{"1H29 915"}="H291 915"; $resn_exch{"2H29 915"}="H292 915"; $resn_exch{"1H30 915"}="H301 915";
$resn_exch{"2H30 915"}="H302 915"; $resn_exch{"1H32 915"}="H321 915"; $resn_exch{"2H32 915"}="H322 915";
$resn_exch{"1H33 915"}="H331 915"; $resn_exch{"2H33 915"}="H332 915"; $resn_exch{"1H33 922"}="H331 922";
$resn_exch{"2H33 922"}="H332 922"; $resn_exch{"1H28 922"}="H281 922"; $resn_exch{"2H28 922"}="H282 922";
$resn_exch{"3H28 922"}="H283 922"; $resn_exch{"1H27 922"}="H271 922"; $resn_exch{"2H27 922"}="H272 922";
$resn_exch{"3H27 922"}="H273 922"; $resn_exch{"1H11 922"}="H111 922"; $resn_exch{"2H11 922"}="H112 922";
$resn_exch{"1H1  936"}=" H11 936"; $resn_exch{"1H12 936"}="H121 936"; $resn_exch{"2H12 936"}="H122 936";
$resn_exch{"3H12 936"}="H123 936"; $resn_exch{"1H19 936"}="H191 936"; $resn_exch{"2H19 936"}="H192 936";
$resn_exch{"1H25 936"}="H251 936"; $resn_exch{"1H26 936"}="H261 936"; $resn_exch{"1H28 936"}="H281 936";
$resn_exch{"1H29 936"}="H291 936"; $resn_exch{"1H37 936"}="H371 936"; $resn_exch{"2H37 936"}="H372 936";
$resn_exch{"3H37 936"}="H373 936"; $resn_exch{"1H45 936"}="H451 936"; $resn_exch{"2H45 936"}="H452 936";
$resn_exch{"3H45 936"}="H453 936"; $resn_exch{"1H5* 93A"}="H5'1 93A"; $resn_exch{"2H5* 93A"}="H5'2 93A";
$resn_exch{"1H12 941"}="H121 941"; $resn_exch{"2H12 941"}="H122 941"; $resn_exch{"1H13 941"}="H131 941";
$resn_exch{"2H13 941"}="H132 941"; $resn_exch{"1H14 941"}="H141 941"; $resn_exch{"2H14 941"}="H142 941";
$resn_exch{"1H15 941"}="H151 941"; $resn_exch{"2H15 941"}="H152 941"; $resn_exch{"1H7  941"}=" H71 941";
$resn_exch{"2H7  941"}=" H72 941"; $resn_exch{"1H39 941"}="H391 941"; $resn_exch{"2H39 941"}="H392 941";
$resn_exch{"3H39 941"}="H393 941"; $resn_exch{"1H42 941"}="H421 941"; $resn_exch{"2H42 941"}="H422 941";
$resn_exch{"3H42 941"}="H423 941"; $resn_exch{"1H43 941"}="H431 941"; $resn_exch{"2H43 941"}="H432 941";
$resn_exch{"3H43 941"}="H433 941"; $resn_exch{"1H44 941"}="H441 941"; $resn_exch{"2H44 941"}="H442 941";
$resn_exch{"3H44 941"}="H443 941"; $resn_exch{"1H14 95A"}="H141 95A"; $resn_exch{"2H14 95A"}="H142 95A";
$resn_exch{"1H48 95A"}="H481 95A"; $resn_exch{"2H48 95A"}="H482 95A"; $resn_exch{"3H48 95A"}="H483 95A";
$resn_exch{"1H47 95A"}="H471 95A"; $resn_exch{"2H47 95A"}="H472 95A"; $resn_exch{"1H49 95A"}="H491 95A";
$resn_exch{"2H49 95A"}="H492 95A"; $resn_exch{"3H49 95A"}="H493 95A"; $resn_exch{"1H38 95A"}="H381 95A";
$resn_exch{"2H38 95A"}="H382 95A"; $resn_exch{"1H39 95A"}="H391 95A"; $resn_exch{"2H39 95A"}="H392 95A";
$resn_exch{"1H40 95A"}="H401 95A"; $resn_exch{"2H40 95A"}="H402 95A"; $resn_exch{"3H40 95A"}="H403 95A";
$resn_exch{"1H20 95A"}="H201 95A"; $resn_exch{"2H20 95A"}="H202 95A"; $resn_exch{"1H23 95A"}="H231 95A";
$resn_exch{"2H23 95A"}="H232 95A"; $resn_exch{"1H14 961"}="H141 961"; $resn_exch{"2H14 961"}="H142 961";
$resn_exch{"1H15 961"}="H151 961"; $resn_exch{"2H15 961"}="H152 961"; $resn_exch{"1H18 961"}="H181 961";
$resn_exch{"2H18 961"}="H182 961"; $resn_exch{"3H18 961"}="H183 961"; $resn_exch{"1H19 961"}="H191 961";
$resn_exch{"2H19 961"}="H192 961"; $resn_exch{"3H19 961"}="H193 961"; $resn_exch{"1H20 961"}="H201 961";
$resn_exch{"2H20 961"}="H202 961"; $resn_exch{"3H20 961"}="H203 961"; $resn_exch{"1H21 961"}="H211 961";
$resn_exch{"2H21 961"}="H212 961"; $resn_exch{"3H21 961"}="H213 961"; $resn_exch{"2HO2 961"}="HO22 961";
$resn_exch{"7HO2 961"}="HO27 961"; $resn_exch{"1H1  964"}="H11A 964"; $resn_exch{"2H1  964"}=" H12 964";
$resn_exch{"3H1  964"}="H13A 964"; $resn_exch{"1H18 964"}="H181 964"; $resn_exch{"2H18 964"}="H182 964";
$resn_exch{"1H19 964"}="H191 964"; $resn_exch{"2H19 964"}="H192 964"; $resn_exch{"1H16 964"}="H161 964";
$resn_exch{"2H16 964"}="H162 964"; $resn_exch{"1H15 964"}="H151 964"; $resn_exch{"2H15 964"}="H152 964";
$resn_exch{"1H10 964"}="H101 964"; $resn_exch{"2H10 964"}="H102 964"; $resn_exch{"1H9  964"}=" H91 964";
$resn_exch{"2H9  964"}=" H92 964"; $resn_exch{"1H5  964"}=" H51 964"; $resn_exch{"2H5  964"}=" H52 964";
$resn_exch{"1H6  964"}=" H61 964"; $resn_exch{"2H6  964"}=" H62 964"; $resn_exch{"1H34 965"}="H341 965";
$resn_exch{"2H34 965"}="H342 965"; $resn_exch{"1H26 965"}="H261 965"; $resn_exch{"2H26 965"}="H262 965";
$resn_exch{"1H25 965"}="H251 965"; $resn_exch{"2H25 965"}="H252 965"; $resn_exch{"1H17 965"}="H171 965";
$resn_exch{"2H17 965"}="H172 965"; $resn_exch{"1H16 965"}="H161 965"; $resn_exch{"2H16 965"}="H162 965";
$resn_exch{"1H08 965"}="H081 965"; $resn_exch{"2H08 965"}="H082 965"; $resn_exch{"1H11 965"}="H111 965";
$resn_exch{"2H11 965"}="H112 965"; $resn_exch{"1H12 974"}="H121 974"; $resn_exch{"2H12 974"}="H122 974";
$resn_exch{"1H32 974"}="H321 974"; $resn_exch{"2H32 974"}="H322 974"; $resn_exch{"1H34 974"}="H341 974";
$resn_exch{"2H34 974"}="H342 974"; $resn_exch{"1H35 974"}="H351 974"; $resn_exch{"2H35 974"}="H352 974";
$resn_exch{"1H36 974"}="H361 974"; $resn_exch{"2H36 974"}="H362 974"; $resn_exch{"1H30 974"}="H301 974";
$resn_exch{"2H30 974"}="H302 974"; $resn_exch{"3H30 974"}="H303 974"; $resn_exch{"1HN1 977"}="HN11 977";
$resn_exch{"2HN1 977"}="HN12 977"; $resn_exch{"1H12 979"}="H121 979"; $resn_exch{"2H12 979"}="H122 979";
$resn_exch{"1H13 979"}="H131 979"; $resn_exch{"2H13 979"}="H132 979"; $resn_exch{"1H2  979"}=" H21 979";
$resn_exch{"2H2  979"}=" H22 979"; $resn_exch{"1H11 979"}="H111 979"; $resn_exch{"2H11 979"}="H112 979";
$resn_exch{"1H16 982"}="H161 982"; $resn_exch{"2H16 982"}="H162 982"; $resn_exch{"1H47 983"}="H471 983";
$resn_exch{"2H47 983"}="H472 983"; $resn_exch{"1H35 983"}="H351 983"; $resn_exch{"2H35 983"}="H352 983";
$resn_exch{"1H33 983"}="H331 983"; $resn_exch{"2H33 983"}="H332 983"; $resn_exch{"1H30 983"}="H301 983";
$resn_exch{"2H30 983"}="H302 983"; $resn_exch{"3H30 983"}="H303 983"; $resn_exch{"1H29 983"}="H291 983";
$resn_exch{"2H29 983"}="H292 983"; $resn_exch{"3H29 983"}="H293 983"; $resn_exch{"1H15 983"}="H151 983";
$resn_exch{"2H15 983"}="H152 983"; $resn_exch{"1H9  983"}=" H91 983"; $resn_exch{"2H9  983"}=" H92 983";
$resn_exch{"1H10 983"}="H101 983"; $resn_exch{"2H10 983"}="H102 983"; $resn_exch{"1HC6 984"}="HC61 984";
$resn_exch{"2HC6 984"}="HC62 984"; $resn_exch{"1HC7 984"}="HC71 984"; $resn_exch{"2HC7 984"}="HC72 984";
$resn_exch{"1H10 984"}="H101 984"; $resn_exch{"2H10 984"}="H102 984"; $resn_exch{"3H10 984"}="H103 984";
$resn_exch{"1H17 984"}="H171 984"; $resn_exch{"2H17 984"}="H172 984"; $resn_exch{"1H18 984"}="H181 984";
$resn_exch{"2H18 984"}="H182 984"; $resn_exch{"1H20 984"}="H201 984"; $resn_exch{"2H20 984"}="H202 984";
$resn_exch{"1H21 984"}="H211 984"; $resn_exch{"2H21 984"}="H212 984"; $resn_exch{"1H31 984"}="H311 984";
$resn_exch{"2H31 984"}="H312 984"; $resn_exch{"3H31 984"}="H313 984"; $resn_exch{"1H56 984"}="H561 984";
$resn_exch{"2H56 984"}="H562 984"; $resn_exch{"1H57 984"}="H571 984"; $resn_exch{"2H57 984"}="H572 984";
$resn_exch{"1H36 989"}="H361 989"; $resn_exch{"2H36 989"}="H362 989"; $resn_exch{"1H62 989"}="H621 989";
$resn_exch{"2H62 989"}="H622 989"; $resn_exch{"3H62 989"}="H623 989"; $resn_exch{"1H45 989"}="H451 989";
$resn_exch{"2H45 989"}="H452 989"; $resn_exch{"1H46 989"}="H461 989"; $resn_exch{"2H46 989"}="H462 989";
$resn_exch{"1H49 989"}="H491 989"; $resn_exch{"2H49 989"}="H492 989"; $resn_exch{"1H52 989"}="H521 989";
$resn_exch{"2H52 989"}="H522 989"; $resn_exch{"1H55 989"}="H551 989"; $resn_exch{"2H55 989"}="H552 989";
$resn_exch{"3H55 989"}="H553 989"; $resn_exch{"1H35 993"}="H351 993"; $resn_exch{"2H35 993"}="H352 993";
$resn_exch{"3H35 993"}="H353 993"; $resn_exch{"1H28 993"}="H281 993"; $resn_exch{"2H28 993"}="H282 993";
$resn_exch{"1H18 993"}="H181 993"; $resn_exch{"2H18 993"}="H182 993"; $resn_exch{"3H18 993"}="H183 993";
$resn_exch{"1H18 994"}="H181 994"; $resn_exch{"2H18 994"}="H182 994"; $resn_exch{"3H18 994"}="H183 994";
$resn_exch{"1H23 994"}="H231 994"; $resn_exch{"2H23 994"}="H232 994"; $resn_exch{"3H23 994"}="H233 994";
$resn_exch{"1H28 994"}="H281 994"; $resn_exch{"2H28 994"}="H282 994"; $resn_exch{"3H28 994"}="H283 994";
$resn_exch{"1H6  995"}=" H61 995"; $resn_exch{"2H6  995"}=" H62 995"; $resn_exch{"1H7  995"}=" H71 995";
$resn_exch{"2H7  995"}=" H72 995"; $resn_exch{"1H11 995"}="H111 995"; $resn_exch{"2H11 995"}="H112 995";
$resn_exch{"3H11 995"}="H113 995"; $resn_exch{"1H12 995"}="H121 995"; $resn_exch{"2H12 995"}="H122 995";
$resn_exch{"3H12 995"}="H123 995"; $resn_exch{"1H14 995"}="H141 995"; $resn_exch{"2H14 995"}="H142 995";
$resn_exch{"1H15 995"}="H151 995"; $resn_exch{"2H15 995"}="H152 995"; $resn_exch{"1H17 995"}="H171 995";
$resn_exch{"2H17 995"}="H172 995"; $resn_exch{"3H17 995"}="H173 995"; $resn_exch{"1H18 995"}="H181 995";
$resn_exch{"2H18 995"}="H182 995"; $resn_exch{"3H18 995"}="H183 995"; $resn_exch{"1H19 995"}="H191 995";
$resn_exch{"2H19 995"}="H192 995"; $resn_exch{"1HN5 995"}="HN51 995"; $resn_exch{"2HN5 995"}="HN52 995";
$resn_exch{"1HN6 995"}="HN61 995"; $resn_exch{"2HN6 995"}="HN62 995"; $resn_exch{"1HD2 9AD"}="HD21 9AD";
$resn_exch{"2HD2 9AD"}="HD22 9AD"; $resn_exch{"1HD3 9AD"}="HD31 9AD"; $resn_exch{"2HD3 9AD"}="HD32 9AD";
$resn_exch{"1HD7 9AD"}="HD71 9AD"; $resn_exch{"2HD7 9AD"}="HD72 9AD"; $resn_exch{"3HD7 9AD"}="HD73 9AD";
$resn_exch{"1HD8 9AD"}="HD81 9AD"; $resn_exch{"2HD8 9AD"}="HD82 9AD"; $resn_exch{"3HD8 9AD"}="HD83 9AD";
$resn_exch{"1H9  9AM"}=" H91 9AM"; $resn_exch{"2H9  9AM"}=" H92 9AM"; $resn_exch{"1H11 9AM"}="H111 9AM";
$resn_exch{"2H11 9AM"}="H112 9AM"; $resn_exch{"3H11 9AM"}="H113 9AM"; $resn_exch{"1HOB 9AM"}="HOB1 9AM";
$resn_exch{"1HN9 9AM"}="HN91 9AM"; $resn_exch{"2HN9 9AM"}="HN92 9AM"; $resn_exch{"1HN1 9AP"}="HN11 9AP";
$resn_exch{"2HN1 9AP"}="HN12 9AP"; $resn_exch{"1H2  9AR"}=" H21 9AR"; $resn_exch{"1H9  9AR"}=" H91 9AR";
$resn_exch{"1H11 9AR"}="H111 9AR"; $resn_exch{"1H12 9AR"}="H121 9AR"; $resn_exch{"1H15 9AR"}="H151 9AR";
$resn_exch{"2H15 9AR"}="H152 9AR"; $resn_exch{"1H17 9AR"}="H171 9AR"; $resn_exch{"2H17 9AR"}="H172 9AR";
$resn_exch{"3H17 9AR"}="H173 9AR"; $resn_exch{"1H5  9AR"}=" H51 9AR"; $resn_exch{"1H6  9AR"}=" H61 9AR";
$resn_exch{"1H6  9CS"}=" H61 9CS"; $resn_exch{"2H6  9CS"}=" H62 9CS"; $resn_exch{"1H12 9CS"}="H121 9CS";
$resn_exch{"2H12 9CS"}="H122 9CS"; $resn_exch{"1H18 9CS"}="H181 9CS"; $resn_exch{"2H18 9CS"}="H182 9CS";
$resn_exch{"1HN1 9CS"}="HN11 9CS"; $resn_exch{"2HN1 9CS"}="HN12 9CS"; $resn_exch{"1HN2 9CS"}="HN21 9CS";
$resn_exch{"2HN2 9CS"}="HN22 9CS"; $resn_exch{"1HN3 9CS"}="HN31 9CS"; $resn_exch{"2HN3 9CS"}="HN32 9CS";
$resn_exch{"1HN4 9CS"}="HN41 9CS"; $resn_exch{"2HN4 9CS"}="HN42 9CS"; $resn_exch{"1HN6 9CS"}="HN61 9CS";
$resn_exch{"2HN6 9CS"}="HN62 9CS"; $resn_exch{"1HN6 9DA"}="HN61 9DA"; $resn_exch{"2HN6 9DA"}="HN62 9DA";
$resn_exch{"1HN2 9DG"}="HN21 9DG"; $resn_exch{"2HN2 9DG"}="HN22 9DG"; $resn_exch{"1H5' 9DI"}="H5'1 9DI";
$resn_exch{"2H5' 9DI"}="H5'2 9DI"; $resn_exch{"'HO5 9DI"}="HO5' 9DI"; $resn_exch{"'HO2 9DI"}="HO2' 9DI";
$resn_exch{"'HO3 9DI"}="HO3' 9DI"; $resn_exch{"1H9  9IN"}=" H91 9IN"; $resn_exch{"2H9  9IN"}=" H92 9IN";
$resn_exch{"3H9  9IN"}=" H93 9IN"; $resn_exch{"1H10 9IN"}="H101 9IN"; $resn_exch{"2H10 9IN"}="H102 9IN";
$resn_exch{"3H10 9IN"}="H103 9IN"; $resn_exch{"1H13 9IN"}="H131 9IN"; $resn_exch{"2H13 9IN"}="H132 9IN";
$resn_exch{"3H13 9IN"}="H133 9IN"; $resn_exch{"1H14 9IN"}="H141 9IN"; $resn_exch{"2H14 9IN"}="H142 9IN";
$resn_exch{"3H14 9IN"}="H143 9IN"; $resn_exch{"1H17 9IN"}="H171 9IN"; $resn_exch{"2H17 9IN"}="H172 9IN";
$resn_exch{"1H19 9IN"}="H191 9IN"; $resn_exch{"2H19 9IN"}="H192 9IN"; $resn_exch{"3H19 9IN"}="H193 9IN";
$resn_exch{"1H20 9IN"}="H201 9IN"; $resn_exch{"2H20 9IN"}="H202 9IN"; $resn_exch{"3H20 9IN"}="H203 9IN";
$resn_exch{"1H28 9IN"}="H281 9IN"; $resn_exch{"2H28 9IN"}="H282 9IN"; $resn_exch{"1H30 9IN"}="H301 9IN";
$resn_exch{"2H30 9IN"}="H302 9IN"; $resn_exch{"1H31 9IN"}="H311 9IN"; $resn_exch{"2H31 9IN"}="H312 9IN";
$resn_exch{"1H26 9IN"}="H261 9IN"; $resn_exch{"2H26 9IN"}="H262 9IN"; $resn_exch{"1H27 9IN"}="H271 9IN";
$resn_exch{"2H27 9IN"}="H272 9IN"; $resn_exch{"3H27 9IN"}="H273 9IN"; $resn_exch{"1H1  9IP"}="H11A 9IP";
$resn_exch{"2H1  9IP"}="H12A 9IP"; $resn_exch{"3H1  9IP"}=" H13 9IP"; $resn_exch{"1H14 9IP"}="H141 9IP";
$resn_exch{"2H14 9IP"}="H142 9IP"; $resn_exch{"1H22 9IP"}="H221 9IP"; $resn_exch{"2H22 9IP"}="H222 9IP";
$resn_exch{"1H1  9MG"}=" H11 9MG"; $resn_exch{"2H1  9MG"}=" H12 9MG"; $resn_exch{"3H1  9MG"}=" H13 9MG";
$resn_exch{"1H10 9MG"}="H101 9MG"; $resn_exch{"2H10 9MG"}="H102 9MG"; $resn_exch{"1H6B 9MR"}="H6B1 9MR";
$resn_exch{"2H6B 9MR"}="H6B2 9MR"; $resn_exch{"1H5M 9MR"}="H5M1 9MR"; $resn_exch{"2H5M 9MR"}="H5M2 9MR";
$resn_exch{"1H1A 9MR"}="H1A1 9MR"; $resn_exch{"2H1A 9MR"}="H1A2 9MR"; $resn_exch{"1H6A 9MR"}="H6A1 9MR";
$resn_exch{"2H6A 9MR"}="H6A2 9MR"; $resn_exch{"1H2  9OH"}=" H21 9OH"; $resn_exch{"2H2  9OH"}=" H22 9OH";
$resn_exch{"1H3  9OH"}=" H31 9OH"; $resn_exch{"2H3  9OH"}=" H32 9OH"; $resn_exch{"1H4  9OH"}=" H41 9OH";
$resn_exch{"2H4  9OH"}=" H42 9OH"; $resn_exch{"1H5  9OH"}=" H51 9OH"; $resn_exch{"2H5  9OH"}=" H52 9OH";
$resn_exch{"1H6  9OH"}=" H61 9OH"; $resn_exch{"2H6  9OH"}=" H62 9OH"; $resn_exch{"1H7  9OH"}=" H71 9OH";
$resn_exch{"2H7  9OH"}=" H72 9OH"; $resn_exch{"1H8  9OH"}=" H81 9OH"; $resn_exch{"2H8  9OH"}=" H82 9OH";
$resn_exch{"1H9  9OH"}=" H91 9OH"; $resn_exch{"1H10 9OH"}="H101 9OH"; $resn_exch{"1H11 9OH"}="H111 9OH";
$resn_exch{"1H12 9OH"}="H121 9OH"; $resn_exch{"1H13 9OH"}="H131 9OH"; $resn_exch{"1H14 9OH"}="H141 9OH";
$resn_exch{"2H14 9OH"}="H142 9OH"; $resn_exch{"1H15 9OH"}="H151 9OH"; $resn_exch{"2H15 9OH"}="H152 9OH";
$resn_exch{"1H16 9OH"}="H161 9OH"; $resn_exch{"2H16 9OH"}="H162 9OH"; $resn_exch{"1H17 9OH"}="H171 9OH";
$resn_exch{"2H17 9OH"}="H172 9OH"; $resn_exch{"1H18 9OH"}="H181 9OH"; $resn_exch{"2H18 9OH"}="H182 9OH";
$resn_exch{"3H18 9OH"}="H183 9OH"; $resn_exch{"1H20 9OH"}="H201 9OH"; $resn_exch{"1H21 9OH"}="H211 9OH";
$resn_exch{"1H8  9PP"}=" H81 9PP"; $resn_exch{"1HN6 9PP"}="HN61 9PP"; $resn_exch{"2HN6 9PP"}="HN62 9PP";
$resn_exch{"1HN2 9PP"}="HN21 9PP"; $resn_exch{"2HN2 9PP"}="HN22 9PP"; $resn_exch{"1H10 9PP"}="H101 9PP";
$resn_exch{"2H10 9PP"}="H102 9PP"; $resn_exch{"1H11 9PP"}="H111 9PP"; $resn_exch{"1H12 9PP"}="H121 9PP";
$resn_exch{"2H12 9PP"}="H122 9PP"; $resn_exch{"3H12 9PP"}="H123 9PP"; $resn_exch{"1H14 9PP"}="H141 9PP";
$resn_exch{"2H14 9PP"}="H142 9PP"; $resn_exch{"3HOP   A"}="HOP3   A"; $resn_exch{"2HOP   A"}="HOP2   A";
$resn_exch{"1H5*   A"}=" H5'   A"; $resn_exch{"2H5*   A"}="H5''   A"; $resn_exch{" H4*   A"}=" H4'   A";
$resn_exch{" H3*   A"}=" H3'   A"; $resn_exch{"H3T    A"}="HO3'   A"; $resn_exch{" H2*   A"}=" H2'   A";
$resn_exch{"2HO*   A"}="HO2'   A"; $resn_exch{" H1*   A"}=" H1'   A"; $resn_exch{"1H6    A"}=" H61   A";
$resn_exch{"2H6    A"}=" H62   A"; $resn_exch{"1HN  A0A"}=" HN1 A0A"; $resn_exch{"2HN  A0A"}=" HN2 A0A";
$resn_exch{"1HBC A0A"}="HBC1 A0A"; $resn_exch{"2HBC A0A"}="HBC2 A0A"; $resn_exch{"2HOB A12"}="HOB2 A12";
$resn_exch{"2HOA A12"}="HOA2 A12"; $resn_exch{"1H3A A12"}="H3A1 A12"; $resn_exch{"2H3A A12"}="H3A2 A12";
$resn_exch{"1H5* A12"}="H5'1 A12"; $resn_exch{"2H5* A12"}="H5'2 A12"; $resn_exch{" H4* A12"}=" H4' A12";
$resn_exch{" H3* A12"}=" H3' A12"; $resn_exch{"*HO3 A12"}="HO3' A12"; $resn_exch{" H2* A12"}=" H2' A12";
$resn_exch{"*HO2 A12"}="HO2' A12"; $resn_exch{" H1* A12"}=" H1' A12"; $resn_exch{"1HN6 A12"}="HN61 A12";
$resn_exch{"2HN6 A12"}="HN62 A12"; $resn_exch{"1H3C A14"}="H3C1 A14"; $resn_exch{"2H3C A14"}="H3C2 A14";
$resn_exch{"1H14 A14"}="H141 A14"; $resn_exch{"2H14 A14"}="H142 A14"; $resn_exch{"1H4C A14"}="H4C1 A14";
$resn_exch{"2H4C A14"}="H4C2 A14"; $resn_exch{"1H7C A14"}="H7C1 A14"; $resn_exch{"2H7C A14"}="H7C2 A14";
$resn_exch{"1H33 A14"}="H331 A14"; $resn_exch{"2H33 A14"}="H332 A14"; $resn_exch{"3H33 A14"}="H333 A14";
$resn_exch{"1H37 A14"}="H371 A14"; $resn_exch{"2H37 A14"}="H372 A14"; $resn_exch{"3H37 A14"}="H373 A14";
$resn_exch{"1HAL A18"}="HAL1 A18"; $resn_exch{"2HAL A18"}="HAL2 A18"; $resn_exch{"1H32 A1A"}="H321 A1A";
$resn_exch{"2H32 A1A"}="H322 A1A"; $resn_exch{"1H7C A1A"}="H7C1 A1A"; $resn_exch{"2H7C A1A"}="H7C2 A1A";
$resn_exch{"3H7C A1A"}="H7C3 A1A"; $resn_exch{"1H4C A1A"}="H4C1 A1A"; $resn_exch{"2H4C A1A"}="H4C2 A1A";
$resn_exch{"1H22 A1A"}="H221 A1A"; $resn_exch{"2H22 A1A"}="H222 A1A"; $resn_exch{"1H58 A1A"}="H581 A1A";
$resn_exch{"2H58 A1A"}="H582 A1A"; $resn_exch{"1H43 A1A"}="H431 A1A"; $resn_exch{"2H43 A1A"}="H432 A1A";
$resn_exch{"3H43 A1A"}="H433 A1A"; $resn_exch{"1H44 A1A"}="H441 A1A"; $resn_exch{"2H44 A1A"}="H442 A1A";
$resn_exch{"3H44 A1A"}="H443 A1A"; $resn_exch{"1H1  A1E"}=" H11 A1E"; $resn_exch{"2H1  A1E"}="H12A A1E";
$resn_exch{"1H2  A1E"}=" H21 A1E"; $resn_exch{"2H2  A1E"}=" H22 A1E"; $resn_exch{"1H6  A1E"}=" H61 A1E";
$resn_exch{"2H6  A1E"}=" H62 A1E"; $resn_exch{"1H13 A1E"}="H131 A1E"; $resn_exch{"2H13 A1E"}="H132 A1E";
$resn_exch{"1H14 A1E"}="H141 A1E"; $resn_exch{"2H14 A1E"}="H142 A1E"; $resn_exch{"1H15 A1E"}="H151 A1E";
$resn_exch{"2H15 A1E"}="H152 A1E"; $resn_exch{"1H16 A1E"}="H161 A1E"; $resn_exch{"2H16 A1E"}="H162 A1E";
$resn_exch{"1H17 A1E"}="H171 A1E"; $resn_exch{"2H17 A1E"}="H172 A1E"; $resn_exch{"1H18 A1E"}="H181 A1E";
$resn_exch{"2H18 A1E"}="H182 A1E"; $resn_exch{"1H19 A1E"}="H191 A1E"; $resn_exch{"2H19 A1E"}="H192 A1E";
$resn_exch{"1H20 A1E"}="H201 A1E"; $resn_exch{"2H20 A1E"}="H202 A1E"; $resn_exch{"1H21 A1E"}="H211 A1E";
$resn_exch{"2H21 A1E"}="H212 A1E"; $resn_exch{"1H22 A1E"}="H221 A1E"; $resn_exch{"2H22 A1E"}="H222 A1E";
$resn_exch{"1H34 A1E"}="H341 A1E"; $resn_exch{"2H34 A1E"}="H342 A1E"; $resn_exch{"1H35 A1E"}="H351 A1E";
$resn_exch{"2H35 A1E"}="H352 A1E"; $resn_exch{"1H36 A1E"}="H361 A1E"; $resn_exch{"2H36 A1E"}="H362 A1E";
$resn_exch{"1H37 A1E"}="H371 A1E"; $resn_exch{"2H37 A1E"}="H372 A1E"; $resn_exch{"1H2N A1P"}="H2N1 A1P";
$resn_exch{"2H2N A1P"}="H2N2 A1P"; $resn_exch{"1H2* A1P"}="H2'1 A1P"; $resn_exch{"2H2* A1P"}="H2'2 A1P";
$resn_exch{"1H5* A1P"}="H5'1 A1P"; $resn_exch{"2H5* A1P"}="H5'2 A1P"; $resn_exch{" H4* A1P"}=" H4' A1P";
$resn_exch{" H1* A1P"}=" H1' A1P"; $resn_exch{" H3* A1P"}=" H3' A1P"; $resn_exch{"1HN6 A1R"}="HN61 A1R";
$resn_exch{"2HN6 A1R"}="HN62 A1R"; $resn_exch{" H1* A1R"}=" H1' A1R"; $resn_exch{" H2* A1R"}=" H2' A1R";
$resn_exch{"HO2* A1R"}="HO2' A1R"; $resn_exch{" H3* A1R"}=" H3' A1R"; $resn_exch{"HO3* A1R"}="HO3' A1R";
$resn_exch{" H4* A1R"}=" H4' A1R"; $resn_exch{"1H5* A1R"}="H5'1 A1R"; $resn_exch{"2H5* A1R"}="H5'2 A1R";
$resn_exch{"1H5N A1R"}="H5N1 A1R"; $resn_exch{"2H5N A1R"}="H5N2 A1R"; $resn_exch{"1H1N A1R"}="H1N1 A1R";
$resn_exch{"2H1N A1R"}="H1N2 A1R"; $resn_exch{"1H38 A1T"}="H381 A1T"; $resn_exch{"2H38 A1T"}="H382 A1T";
$resn_exch{"1H40 A1T"}="H401 A1T"; $resn_exch{"2H40 A1T"}="H402 A1T"; $resn_exch{"1H43 A1T"}="H431 A1T";
$resn_exch{"2H43 A1T"}="H432 A1T"; $resn_exch{"1H46 A1T"}="H461 A1T"; $resn_exch{"2H46 A1T"}="H462 A1T";
$resn_exch{"1H48 A1T"}="H481 A1T"; $resn_exch{"2H48 A1T"}="H482 A1T"; $resn_exch{"3H48 A1T"}="H483 A1T";
$resn_exch{"1H28 A1T"}="H281 A1T"; $resn_exch{"2H28 A1T"}="H282 A1T"; $resn_exch{"1H32 A1T"}="H321 A1T";
$resn_exch{"2H32 A1T"}="H322 A1T"; $resn_exch{"1H37 A1T"}="H371 A1T"; $resn_exch{"2H37 A1T"}="H372 A1T";
$resn_exch{"1H31 A1T"}="H311 A1T"; $resn_exch{"2H31 A1T"}="H312 A1T"; $resn_exch{"1H27 A1T"}="H271 A1T";
$resn_exch{"2H27 A1T"}="H272 A1T"; $resn_exch{"1H21 A1T"}="H211 A1T"; $resn_exch{"2H21 A1T"}="H212 A1T";
$resn_exch{"1H6  A1T"}=" H61 A1T"; $resn_exch{"2H6  A1T"}=" H62 A1T"; $resn_exch{"1H10 A1T"}="H101 A1T";
$resn_exch{"2H10 A1T"}="H102 A1T"; $resn_exch{"1H13 A1T"}="H131 A1T"; $resn_exch{"2H13 A1T"}="H132 A1T";
$resn_exch{"1H9  A1T"}=" H91 A1T"; $resn_exch{"2H9  A1T"}=" H92 A1T"; $resn_exch{"1H5  A1T"}=" H51 A1T";
$resn_exch{"2H5  A1T"}=" H52 A1T"; $resn_exch{"1H*A A22"}=" HD1 A22"; $resn_exch{"2H*A A22"}=" HD2 A22";
$resn_exch{"H4*A A22"}=" H4D A22"; $resn_exch{"H2*A A22"}=" H2D A22"; $resn_exch{"H1*A A22"}=" H1D A22";
$resn_exch{"1H6A A22"}="H6A1 A22"; $resn_exch{"2H6A A22"}="H6A2 A22"; $resn_exch{"1H*B A22"}=" HE1 A22";
$resn_exch{"2H*B A22"}=" HE2 A22"; $resn_exch{"H4*B A22"}=" H4E A22"; $resn_exch{"H1*B A22"}=" H1E A22";
$resn_exch{"1H6B A22"}="H6B1 A22"; $resn_exch{"2H6B A22"}="H6B2 A22"; $resn_exch{"2HOC A23"}="HOC2 A23";
$resn_exch{"2HOP A23"}="HOP2 A23"; $resn_exch{"3HOP A23"}="HOP3 A23"; $resn_exch{"1H5* A23"}=" H5' A23";
$resn_exch{"2H5* A23"}="H5'' A23"; $resn_exch{" H4* A23"}=" H4' A23"; $resn_exch{" H3* A23"}=" H3' A23";
$resn_exch{" H2* A23"}=" H2' A23"; $resn_exch{" H1* A23"}=" H1' A23"; $resn_exch{"1HN6 A23"}="HN61 A23";
$resn_exch{"2HN6 A23"}="HN62 A23"; $resn_exch{"1H1  A24"}=" H11 A24"; $resn_exch{"1H2  A24"}=" H21 A24";
$resn_exch{"1HO2 A24"}="HO21 A24"; $resn_exch{"1H3  A24"}=" H31 A24"; $resn_exch{"1HO3 A24"}="HO31 A24";
$resn_exch{"1H4  A24"}=" H41 A24"; $resn_exch{"1HO4 A24"}="HO41 A24"; $resn_exch{"1H5  A24"}=" H51 A24";
$resn_exch{"1H6  A24"}=" H61 A24"; $resn_exch{"2H6  A24"}=" H62 A24"; $resn_exch{"1HO  A24"}=" HO1 A24";
$resn_exch{"1H2' A24"}="H2'1 A24"; $resn_exch{"1H4' A24"}="H4'1 A24"; $resn_exch{"1H6' A24"}="H6'1 A24";
$resn_exch{"1HN2 A24"}="HN21 A24"; $resn_exch{"1H2* A24"}="H2A1 A24"; $resn_exch{"2H2* A24"}="H2'2 A24";
$resn_exch{"1H3* A24"}="H3'1 A24"; $resn_exch{"2H3* A24"}="H3'2 A24"; $resn_exch{"1H5* A24"}="H5'1 A24";
$resn_exch{"2H5* A24"}="H5'2 A24"; $resn_exch{"1H6* A24"}="H6A1 A24"; $resn_exch{"2H6* A24"}="H6'2 A24";
$resn_exch{"1H7* A24"}="H7'1 A24"; $resn_exch{"2H7* A24"}="H7'2 A24"; $resn_exch{"1H8* A24"}="H8'1 A24";
$resn_exch{"2H8* A24"}="H8'2 A24"; $resn_exch{"1H21 A25"}="H211 A25"; $resn_exch{"2H21 A25"}="H212 A25";
$resn_exch{"1H22 A25"}="H221 A25"; $resn_exch{"2H22 A25"}="H222 A25"; $resn_exch{"1H24 A25"}="H241 A25";
$resn_exch{"2H24 A25"}="H242 A25"; $resn_exch{"1H23 A25"}="H231 A25"; $resn_exch{"2H23 A25"}="H232 A25";
$resn_exch{"1H9  A25"}=" H91 A25"; $resn_exch{"2H9  A25"}=" H92 A25"; $resn_exch{"1H1  A26"}=" H11 A26";
$resn_exch{"1H2  A26"}=" H21 A26"; $resn_exch{"1H4  A26"}=" H41 A26"; $resn_exch{"1H5  A26"}=" H51 A26";
$resn_exch{"1HN1 A26"}="HN11 A26"; $resn_exch{"1H21 A26"}="H211 A26"; $resn_exch{"2H21 A26"}="H212 A26";
$resn_exch{"3H21 A26"}="H213 A26"; $resn_exch{"1H*A A2D"}=" HD1 A2D"; $resn_exch{"2H*A A2D"}=" HD2 A2D";
$resn_exch{"H4*A A2D"}=" H4D A2D"; $resn_exch{"H1*A A2D"}=" H1D A2D"; $resn_exch{"1H6A A2D"}="H6A1 A2D";
$resn_exch{"2H6A A2D"}="H6A2 A2D"; $resn_exch{"1H*B A2D"}=" HE1 A2D"; $resn_exch{"2H*B A2D"}=" HE2 A2D";
$resn_exch{"H4*B A2D"}=" H4E A2D"; $resn_exch{"H1*B A2D"}=" H1E A2D"; $resn_exch{"1H6B A2D"}="H6B1 A2D";
$resn_exch{"2H6B A2D"}="H6B2 A2D"; $resn_exch{"1H6  A2E"}=" H61 A2E"; $resn_exch{"2H6  A2E"}=" H62 A2E";
$resn_exch{"1H2  A2E"}=" H21 A2E"; $resn_exch{"2H2  A2E"}=" H22 A2E"; $resn_exch{"1H1  A2E"}=" H11 A2E";
$resn_exch{"2H1  A2E"}="H12A A2E"; $resn_exch{"1H13 A2E"}="H131 A2E"; $resn_exch{"2H13 A2E"}="H132 A2E";
$resn_exch{"1H14 A2E"}="H141 A2E"; $resn_exch{"2H14 A2E"}="H142 A2E"; $resn_exch{"1H15 A2E"}="H151 A2E";
$resn_exch{"2H15 A2E"}="H152 A2E"; $resn_exch{"1H16 A2E"}="H161 A2E"; $resn_exch{"2H16 A2E"}="H162 A2E";
$resn_exch{"1H17 A2E"}="H171 A2E"; $resn_exch{"2H17 A2E"}="H172 A2E"; $resn_exch{"1H18 A2E"}="H181 A2E";
$resn_exch{"2H18 A2E"}="H182 A2E"; $resn_exch{"1H19 A2E"}="H191 A2E"; $resn_exch{"2H19 A2E"}="H192 A2E";
$resn_exch{"1H20 A2E"}="H201 A2E"; $resn_exch{"2H20 A2E"}="H202 A2E"; $resn_exch{"1H21 A2E"}="H211 A2E";
$resn_exch{"2H21 A2E"}="H212 A2E"; $resn_exch{"1H22 A2E"}="H221 A2E"; $resn_exch{"2H22 A2E"}="H222 A2E";
$resn_exch{"1H30 A2E"}="H301 A2E"; $resn_exch{"2H30 A2E"}="H302 A2E"; $resn_exch{"1H33 A2E"}="H331 A2E";
$resn_exch{"2H33 A2E"}="H332 A2E"; $resn_exch{"1H32 A2E"}="H321 A2E"; $resn_exch{"2H32 A2E"}="H322 A2E";
$resn_exch{"1H31 A2E"}="H311 A2E"; $resn_exch{"2H31 A2E"}="H312 A2E"; $resn_exch{"1H6  A2G"}=" H61 A2G";
$resn_exch{"2H6  A2G"}=" H62 A2G"; $resn_exch{"1H8  A2G"}=" H81 A2G"; $resn_exch{"2H8  A2G"}=" H82 A2G";
$resn_exch{"3H8  A2G"}=" H83 A2G"; $resn_exch{"2HO* A2L"}="HO2' A2L"; $resn_exch{" H2* A2L"}=" H2' A2L";
$resn_exch{" H1* A2L"}=" H1' A2L"; $resn_exch{" H4* A2L"}=" H4' A2L"; $resn_exch{"1H5* A2L"}=" H5' A2L";
$resn_exch{"2H5* A2L"}="H5'' A2L"; $resn_exch{"3HOP A2L"}="HOP3 A2L"; $resn_exch{"2HOP A2L"}="HOP2 A2L";
$resn_exch{" H3* A2L"}=" H3' A2L"; $resn_exch{"1HA* A2L"}="HA'1 A2L"; $resn_exch{"2HA* A2L"}="HA'2 A2L";
$resn_exch{"1HB* A2L"}="HB'1 A2L"; $resn_exch{"2HB* A2L"}="HB'2 A2L"; $resn_exch{"1HD* A2L"}="HD'1 A2L";
$resn_exch{"2HD* A2L"}="HD'2 A2L"; $resn_exch{"3HD* A2L"}="HD'3 A2L"; $resn_exch{"1H6  A2L"}=" H61 A2L";
$resn_exch{"2H6  A2L"}=" H62 A2L"; $resn_exch{"3HOP A2M"}="HOP3 A2M"; $resn_exch{"2HOP A2M"}="HOP2 A2M";
$resn_exch{"1H5* A2M"}=" H5' A2M"; $resn_exch{"2H5* A2M"}="H5'' A2M"; $resn_exch{" H4* A2M"}=" H4' A2M";
$resn_exch{" H3* A2M"}=" H3' A2M"; $resn_exch{"H3T  A2M"}="HO3' A2M"; $resn_exch{" H2* A2M"}=" H2' A2M";
$resn_exch{" H1* A2M"}=" H1' A2M"; $resn_exch{"1HM* A2M"}="HM'1 A2M"; $resn_exch{"2HM* A2M"}="HM'2 A2M";
$resn_exch{"3HM* A2M"}="HM'3 A2M"; $resn_exch{"1H6  A2M"}=" H61 A2M"; $resn_exch{"2H6  A2M"}=" H62 A2M";
$resn_exch{"2HOP A2P"}="HOP2 A2P"; $resn_exch{"3HOP A2P"}="HOP3 A2P"; $resn_exch{"5HOP A2P"}="HOP5 A2P";
$resn_exch{"6HOP A2P"}="HOP6 A2P"; $resn_exch{"1H5* A2P"}="H5'1 A2P"; $resn_exch{"2H5* A2P"}="H5'2 A2P";
$resn_exch{" H4* A2P"}=" H4' A2P"; $resn_exch{" H3* A2P"}=" H3' A2P"; $resn_exch{"*HO3 A2P"}="HO3' A2P";
$resn_exch{" H2* A2P"}=" H2' A2P"; $resn_exch{" H1* A2P"}=" H1' A2P"; $resn_exch{"1HN6 A2P"}="HN61 A2P";
$resn_exch{"2HN6 A2P"}="HN62 A2P"; $resn_exch{" H2* A2R"}=" H2' A2R"; $resn_exch{" H1* A2R"}=" H1' A2R";
$resn_exch{"1HN6 A2R"}="HN61 A2R"; $resn_exch{"2HN6 A2R"}="HN62 A2R"; $resn_exch{" H3* A2R"}=" H3' A2R";
$resn_exch{"HO3* A2R"}="HO3' A2R"; $resn_exch{" H4* A2R"}=" H4' A2R"; $resn_exch{"1H5* A2R"}="H5'1 A2R";
$resn_exch{"2H5* A2R"}="H5'2 A2R"; $resn_exch{"3H5* A2R"}="H5'3 A2R"; $resn_exch{"4H5* A2R"}="H5'4 A2R";
$resn_exch{"H14* A2R"}="H14' A2R"; $resn_exch{"H13* A2R"}="H13' A2R"; $resn_exch{"H16* A2R"}="H16' A2R";
$resn_exch{"H12* A2R"}="H12' A2R"; $resn_exch{"H15* A2R"}="H15' A2R"; $resn_exch{"H11* A2R"}="H11' A2R";
$resn_exch{"H17* A2R"}="H17' A2R"; $resn_exch{"1H35 A2T"}="H351 A2T"; $resn_exch{"2H35 A2T"}="H352 A2T";
$resn_exch{"1H37 A2T"}="H371 A2T"; $resn_exch{"2H37 A2T"}="H372 A2T"; $resn_exch{"1H39 A2T"}="H391 A2T";
$resn_exch{"2H39 A2T"}="H392 A2T"; $resn_exch{"1H42 A2T"}="H421 A2T"; $resn_exch{"2H42 A2T"}="H422 A2T";
$resn_exch{"1H43 A2T"}="H431 A2T"; $resn_exch{"2H43 A2T"}="H432 A2T"; $resn_exch{"3H43 A2T"}="H433 A2T";
$resn_exch{"1H25 A2T"}="H251 A2T"; $resn_exch{"2H25 A2T"}="H252 A2T"; $resn_exch{"1H29 A2T"}="H291 A2T";
$resn_exch{"2H29 A2T"}="H292 A2T"; $resn_exch{"1H34 A2T"}="H341 A2T"; $resn_exch{"2H34 A2T"}="H342 A2T";
$resn_exch{"1H36 A2T"}="H361 A2T"; $resn_exch{"2H36 A2T"}="H362 A2T"; $resn_exch{"1H41 A2T"}="H411 A2T";
$resn_exch{"2H41 A2T"}="H412 A2T"; $resn_exch{"3H41 A2T"}="H413 A2T"; $resn_exch{"1H40 A2T"}="H401 A2T";
$resn_exch{"2H40 A2T"}="H402 A2T"; $resn_exch{"3H40 A2T"}="H403 A2T"; $resn_exch{"1H28 A2T"}="H281 A2T";
$resn_exch{"2H28 A2T"}="H282 A2T"; $resn_exch{"1H24 A2T"}="H241 A2T"; $resn_exch{"2H24 A2T"}="H242 A2T";
$resn_exch{"1H18 A2T"}="H181 A2T"; $resn_exch{"2H18 A2T"}="H182 A2T"; $resn_exch{"1H45 A2T"}="H451 A2T";
$resn_exch{"2H45 A2T"}="H452 A2T"; $resn_exch{"3H45 A2T"}="H453 A2T"; $resn_exch{"1H1  A32"}=" H11 A32";
$resn_exch{"1H2  A32"}=" H21 A32"; $resn_exch{"1HO2 A32"}="HO21 A32"; $resn_exch{"1H3  A32"}=" H31 A32";
$resn_exch{"1HO3 A32"}="HO31 A32"; $resn_exch{"1H4  A32"}=" H41 A32"; $resn_exch{"1HO4 A32"}="HO41 A32";
$resn_exch{"1H5  A32"}=" H51 A32"; $resn_exch{"1H6  A32"}=" H61 A32"; $resn_exch{"2H6  A32"}=" H62 A32";
$resn_exch{"1HO  A32"}=" HO1 A32"; $resn_exch{"1H2' A32"}="H2'1 A32"; $resn_exch{"1H4' A32"}="H4'1 A32";
$resn_exch{"1H6' A32"}="H6'1 A32"; $resn_exch{"1HN2 A32"}="HN21 A32"; $resn_exch{"1H2* A32"}="H2A1 A32";
$resn_exch{"2H2* A32"}="H2'2 A32"; $resn_exch{"1H3* A32"}="H3'1 A32"; $resn_exch{"2H3* A32"}="H3'2 A32";
$resn_exch{"1H5* A32"}="H5'1 A32"; $resn_exch{"2H5* A32"}="H5'2 A32"; $resn_exch{"1H6* A32"}="H6A1 A32";
$resn_exch{"2H6* A32"}="H6'2 A32"; $resn_exch{"1H7* A32"}="H7'1 A32"; $resn_exch{"2H7* A32"}="H7'2 A32";
$resn_exch{"1H8* A32"}="H8'1 A32"; $resn_exch{"2H8* A32"}="H8'2 A32"; $resn_exch{"1H9* A32"}="H9'1 A32";
$resn_exch{"2H9* A32"}="H9'2 A32"; $resn_exch{"1H11 A34"}="H111 A34"; $resn_exch{"2H11 A34"}="H112 A34";
$resn_exch{"3H11 A34"}="H113 A34"; $resn_exch{"1H2* A34"}="H2'1 A34"; $resn_exch{"2H2* A34"}="H2'2 A34";
$resn_exch{"1H5* A34"}="H5'1 A34"; $resn_exch{"2H5* A34"}="H5'2 A34"; $resn_exch{" H4* A34"}=" H4' A34";
$resn_exch{" H1* A34"}=" H1' A34"; $resn_exch{" H3* A34"}=" H3' A34"; $resn_exch{"1H6N A35"}="H6N1 A35";
$resn_exch{"2H6N A35"}="H6N2 A35"; $resn_exch{"1H5* A35"}="H5'1 A35"; $resn_exch{"2H5* A35"}="H5'2 A35";
$resn_exch{" H4* A35"}=" H4' A35"; $resn_exch{" H1* A35"}=" H1' A35"; $resn_exch{"1H2* A35"}="H2'1 A35";
$resn_exch{"2H2* A35"}="H2'2 A35"; $resn_exch{" H3* A35"}=" H3' A35"; $resn_exch{"1HH2 A35"}="HH21 A35";
$resn_exch{"2HH2 A35"}="HH22 A35"; $resn_exch{"3HH2 A35"}="HH23 A35"; $resn_exch{"1H5* A38"}="H5'1 A38";
$resn_exch{"2H5* A38"}="H5'2 A38"; $resn_exch{" H4* A38"}=" H4' A38"; $resn_exch{" H3* A38"}=" H3' A38";
$resn_exch{"1H2* A38"}="H2'1 A38"; $resn_exch{"2H2* A38"}="H2'2 A38"; $resn_exch{" H1* A38"}=" H1' A38";
$resn_exch{"1H6N A38"}="H6N1 A38"; $resn_exch{"2H6N A38"}="H6N2 A38"; $resn_exch{"1H5* A39"}="H5'1 A39";
$resn_exch{"2H5* A39"}="H5'2 A39"; $resn_exch{" H4* A39"}=" H4' A39"; $resn_exch{" H3* A39"}=" H3' A39";
$resn_exch{" H2* A39"}=" H2' A39"; $resn_exch{" H1* A39"}=" H1' A39"; $resn_exch{"1H6N A39"}="H6N1 A39";
$resn_exch{"2H6N A39"}="H6N2 A39"; $resn_exch{"1HH3 A39"}="HH31 A39"; $resn_exch{"2HH3 A39"}="HH32 A39";
$resn_exch{"3HH3 A39"}="HH33 A39"; $resn_exch{"1HN6 A3A"}="HN61 A3A"; $resn_exch{"2HN6 A3A"}="HN62 A3A";
$resn_exch{"1H5* A3A"}=" H5' A3A"; $resn_exch{"2H5* A3A"}="H5'' A3A"; $resn_exch{" H4* A3A"}=" H4' A3A";
$resn_exch{" H3* A3A"}=" H3' A3A"; $resn_exch{"1H2* A3A"}=" H2' A3A"; $resn_exch{"2H2* A3A"}="H2'' A3A";
$resn_exch{" H1* A3A"}=" H1' A3A"; $resn_exch{"1HAA A3B"}="HAA1 A3B"; $resn_exch{"2HAA A3B"}="HAA2 A3B";
$resn_exch{"1HAB A3B"}="HAB1 A3B"; $resn_exch{"2HAB A3B"}="HAB2 A3B"; $resn_exch{"3HAB A3B"}="HAB3 A3B";
$resn_exch{"2HOA A3D"}="HOA2 A3D"; $resn_exch{"AH51 A3D"}="H51A A3D"; $resn_exch{"AH52 A3D"}="H52A A3D";
$resn_exch{"AH4* A3D"}=" H4B A3D"; $resn_exch{"AH3* A3D"}=" H3B A3D"; $resn_exch{"AHO3 A3D"}="HO3A A3D";
$resn_exch{"AH2* A3D"}=" H2B A3D"; $resn_exch{"AHO2 A3D"}="HO2A A3D"; $resn_exch{"AH1* A3D"}=" H1B A3D";
$resn_exch{" AH8 A3D"}=" H8A A3D"; $resn_exch{"AH61 A3D"}="H61A A3D"; $resn_exch{"AH62 A3D"}="H62A A3D";
$resn_exch{" AH2 A3D"}=" H2A A3D"; $resn_exch{"NH51 A3D"}="H51N A3D"; $resn_exch{"NH52 A3D"}="H52N A3D";
$resn_exch{"NH4* A3D"}=" H4D A3D"; $resn_exch{"NH3* A3D"}=" H3D A3D"; $resn_exch{"NHO3 A3D"}="HO3N A3D";
$resn_exch{"NH2* A3D"}=" H2D A3D"; $resn_exch{"NHO2 A3D"}="HO2N A3D"; $resn_exch{"NH1* A3D"}=" H1D A3D";
$resn_exch{" NH2 A3D"}=" H2N A3D"; $resn_exch{"NH81 A3D"}="H81N A3D"; $resn_exch{"NH82 A3D"}="H82N A3D";
$resn_exch{"NH83 A3D"}="H83N A3D"; $resn_exch{" NH4 A3D"}=" H4N A3D"; $resn_exch{" NH5 A3D"}=" H5N A3D";
$resn_exch{" NH6 A3D"}=" H6N A3D"; $resn_exch{"1HC1 A3M"}="HC11 A3M"; $resn_exch{"2HC1 A3M"}="HC12 A3M";
$resn_exch{"1HC2 A3M"}="HC21 A3M"; $resn_exch{"2HC2 A3M"}="HC22 A3M"; $resn_exch{"1HC4 A3M"}="HC41 A3M";
$resn_exch{"2HC4 A3M"}="HC42 A3M"; $resn_exch{"1HC5 A3M"}="HC51 A3M"; $resn_exch{"2HC5 A3M"}="HC52 A3M";
$resn_exch{"1H17 A3M"}="H171 A3M"; $resn_exch{"2H17 A3M"}="H172 A3M"; $resn_exch{"3H17 A3M"}="H173 A3M";
$resn_exch{"1H19 A3M"}="H191 A3M"; $resn_exch{"2H19 A3M"}="H192 A3M"; $resn_exch{"1H26 A3M"}="H261 A3M";
$resn_exch{"2H26 A3M"}="H262 A3M"; $resn_exch{"3H26 A3M"}="H263 A3M"; $resn_exch{"2HOP A3P"}="HOP2 A3P";
$resn_exch{"3HOP A3P"}="HOP3 A3P"; $resn_exch{"5HOP A3P"}="HOP5 A3P"; $resn_exch{"6HOP A3P"}="HOP6 A3P";
$resn_exch{"1H5* A3P"}="H5'1 A3P"; $resn_exch{"2H5* A3P"}="H5'2 A3P"; $resn_exch{" H4* A3P"}=" H4' A3P";
$resn_exch{" H3* A3P"}=" H3' A3P"; $resn_exch{" H2* A3P"}=" H2' A3P"; $resn_exch{"*HO2 A3P"}="HO2' A3P";
$resn_exch{" H1* A3P"}=" H1' A3P"; $resn_exch{"1HN6 A3P"}="HN61 A3P"; $resn_exch{"2HN6 A3P"}="HN62 A3P";
$resn_exch{"1H5' A3S"}="H5'1 A3S"; $resn_exch{"2H5' A3S"}="H5'2 A3S"; $resn_exch{"1HN6 A3S"}="HN61 A3S";
$resn_exch{"2HN6 A3S"}="HN62 A3S"; $resn_exch{"1HN  A3S"}=" HN1 A3S"; $resn_exch{"2HN  A3S"}=" HN2 A3S";
$resn_exch{"1HB  A3S"}=" HB1 A3S"; $resn_exch{"2HB  A3S"}=" HB2 A3S"; $resn_exch{"1H6N A40"}="H6N1 A40";
$resn_exch{"2H6N A40"}="H6N2 A40"; $resn_exch{"1H5* A40"}="H5'1 A40"; $resn_exch{"2H5* A40"}="H5'2 A40";
$resn_exch{" H4* A40"}=" H4' A40"; $resn_exch{" H1* A40"}=" H1' A40"; $resn_exch{"1H2* A40"}="H2'1 A40";
$resn_exch{"2H2* A40"}="H2'2 A40"; $resn_exch{" H3* A40"}=" H3' A40"; $resn_exch{"1HH2 A40"}="HH21 A40";
$resn_exch{"2HH2 A40"}="HH22 A40"; $resn_exch{"3HH2 A40"}="HH23 A40"; $resn_exch{"1H20 A41"}="H201 A41";
$resn_exch{"2H20 A41"}="H202 A41"; $resn_exch{"3H20 A41"}="H203 A41"; $resn_exch{"1H20 A42"}="H201 A42";
$resn_exch{"2H20 A42"}="H202 A42"; $resn_exch{"3H20 A42"}="H203 A42"; $resn_exch{"1H22 A42"}="H221 A42";
$resn_exch{"2H22 A42"}="H222 A42"; $resn_exch{"3H22 A42"}="H223 A42"; $resn_exch{"1H5* A43"}="H5'1 A43";
$resn_exch{"2H5* A43"}="H5'2 A43"; $resn_exch{" H4* A43"}=" H4' A43"; $resn_exch{" H3* A43"}=" H3' A43";
$resn_exch{"1H2* A43"}="H2'1 A43"; $resn_exch{"2H2* A43"}="H2'2 A43"; $resn_exch{" H1* A43"}=" H1' A43";
$resn_exch{"1H6N A43"}="H6N1 A43"; $resn_exch{"2H6N A43"}="H6N2 A43"; $resn_exch{"1HN  A43"}=" HN1 A43";
$resn_exch{"2HN  A43"}=" HN2 A43"; $resn_exch{"1H5* A44"}="H5'1 A44"; $resn_exch{"2H5* A44"}="H5'2 A44";
$resn_exch{" H4* A44"}=" H4' A44"; $resn_exch{" H3* A44"}=" H3' A44"; $resn_exch{" H2* A44"}=" H2' A44";
$resn_exch{" H1* A44"}=" H1' A44"; $resn_exch{"1H6N A44"}="H6N1 A44"; $resn_exch{"2H6N A44"}="H6N2 A44";
$resn_exch{"1HA* A44"}="HA'1 A44"; $resn_exch{"2HA* A44"}="HA'2 A44"; $resn_exch{"1H13 A44"}="H131 A44";
$resn_exch{"2H13 A44"}="H132 A44"; $resn_exch{"3H13 A44"}="H133 A44"; $resn_exch{"1H15 A44"}="H151 A44";
$resn_exch{"2H15 A44"}="H152 A44"; $resn_exch{"1HN1 A45"}="HN11 A45"; $resn_exch{"2HN1 A45"}="HN12 A45";
$resn_exch{"1H5* A47"}="H5'1 A47"; $resn_exch{"2H5* A47"}="H5'2 A47"; $resn_exch{" H4* A47"}=" H4' A47";
$resn_exch{" H3* A47"}=" H3' A47"; $resn_exch{"1H2* A47"}="H2'1 A47"; $resn_exch{"2H2* A47"}="H2'2 A47";
$resn_exch{" H1* A47"}=" H1' A47"; $resn_exch{"1H1C A47"}="H1C1 A47"; $resn_exch{"2H1C A47"}="H1C2 A47";
$resn_exch{"3H1C A47"}="H1C3 A47"; $resn_exch{"1HAD A48"}="HAD1 A48"; $resn_exch{"2HAD A48"}="HAD2 A48";
$resn_exch{"3HAD A48"}="HAD3 A48"; $resn_exch{"1HAC A48"}="HAC1 A48"; $resn_exch{"2HAC A48"}="HAC2 A48";
$resn_exch{"3HAC A48"}="HAC3 A48"; $resn_exch{"1HAJ A48"}="HAJ1 A48"; $resn_exch{"2HAJ A48"}="HAJ2 A48";
$resn_exch{"3HAJ A48"}="HAJ3 A48"; $resn_exch{"1HAW A48"}="HAW1 A48"; $resn_exch{"2HAW A48"}="HAW2 A48";
$resn_exch{"1HAG A48"}="HAG1 A48"; $resn_exch{"2HAG A48"}="HAG2 A48"; $resn_exch{"3HAG A48"}="HAG3 A48";
$resn_exch{"1HAS A48"}="HAS1 A48"; $resn_exch{"2HAS A48"}="HAS2 A48"; $resn_exch{"3HAS A48"}="HAS3 A48";
$resn_exch{"1HBB A48"}="HBB1 A48"; $resn_exch{"2HBB A48"}="HBB2 A48"; $resn_exch{"3HBB A48"}="HBB3 A48";
$resn_exch{"1HD2 A4C"}="HD21 A4C"; $resn_exch{"2HD2 A4C"}="HD22 A4C"; $resn_exch{"1HD3 A4C"}="HD31 A4C";
$resn_exch{"2HD3 A4C"}="HD32 A4C"; $resn_exch{"1HD8 A4C"}="HD81 A4C"; $resn_exch{"2HD8 A4C"}="HD82 A4C";
$resn_exch{"3HD8 A4C"}="HD83 A4C"; $resn_exch{"1HD7 A4C"}="HD71 A4C"; $resn_exch{"2HD7 A4C"}="HD72 A4C";
$resn_exch{"3HD7 A4C"}="HD73 A4C"; $resn_exch{"1HX1 A4C"}="HX11 A4C"; $resn_exch{"2HX1 A4C"}="HX12 A4C";
$resn_exch{"1HX2 A4C"}="HX21 A4C"; $resn_exch{"2HX2 A4C"}="HX22 A4C"; $resn_exch{"1HX3 A4C"}="HX31 A4C";
$resn_exch{"2HX3 A4C"}="HX32 A4C"; $resn_exch{"1HD5 A4C"}="HD51 A4C"; $resn_exch{"2HD5 A4C"}="HD52 A4C";
$resn_exch{"1HD6 A4C"}="HD61 A4C"; $resn_exch{"2HD6 A4C"}="HD62 A4C"; $resn_exch{"1HD0 A4C"}="HD01 A4C";
$resn_exch{"2HD0 A4C"}="HD02 A4C"; $resn_exch{"3HD0 A4C"}="HD03 A4C"; $resn_exch{"1HD9 A4C"}="HD91 A4C";
$resn_exch{"2HD9 A4C"}="HD92 A4C"; $resn_exch{"3HD9 A4C"}="HD93 A4C"; $resn_exch{"1HX4 A4C"}="HX41 A4C";
$resn_exch{"2HX4 A4C"}="HX42 A4C"; $resn_exch{"1HX5 A4C"}="HX51 A4C"; $resn_exch{"2HX5 A4C"}="HX52 A4C";
$resn_exch{"1HX6 A4C"}="HX61 A4C"; $resn_exch{"2HX6 A4C"}="HX62 A4C"; $resn_exch{"1H28 A4L"}="H281 A4L";
$resn_exch{"2H28 A4L"}="H282 A4L"; $resn_exch{"3H28 A4L"}="H283 A4L"; $resn_exch{"1H22 A4L"}="H221 A4L";
$resn_exch{"2H22 A4L"}="H222 A4L"; $resn_exch{"1H21 A4L"}="H211 A4L"; $resn_exch{"2H21 A4L"}="H212 A4L";
$resn_exch{"1H20 A4L"}="H201 A4L"; $resn_exch{"2H20 A4L"}="H202 A4L"; $resn_exch{"1H19 A4L"}="H191 A4L";
$resn_exch{"2H19 A4L"}="H192 A4L"; $resn_exch{"1H17 A4L"}="H171 A4L"; $resn_exch{"2H17 A4L"}="H172 A4L";
$resn_exch{"3H17 A4L"}="H173 A4L"; $resn_exch{"1HN1 A4L"}="HN11 A4L"; $resn_exch{"2HN1 A4L"}="HN12 A4L";
$resn_exch{"1H7  A4L"}=" H71 A4L"; $resn_exch{"2H7  A4L"}=" H72 A4L"; $resn_exch{"1H15 A4L"}="H151 A4L";
$resn_exch{"2H15 A4L"}="H152 A4L"; $resn_exch{"1H16 A4L"}="H161 A4L"; $resn_exch{"2H16 A4L"}="H162 A4L";
$resn_exch{"1H18 A4L"}="H181 A4L"; $resn_exch{"2H18 A4L"}="H182 A4L"; $resn_exch{"1H34 A4L"}="H341 A4L";
$resn_exch{"2H34 A4L"}="H342 A4L"; $resn_exch{"1H35 A4L"}="H351 A4L"; $resn_exch{"2H35 A4L"}="H352 A4L";
$resn_exch{"1H36 A4L"}="H361 A4L"; $resn_exch{"2H36 A4L"}="H362 A4L"; $resn_exch{"1H37 A4L"}="H371 A4L";
$resn_exch{"2H37 A4L"}="H372 A4L"; $resn_exch{"1HN6 A4L"}="HN61 A4L"; $resn_exch{"2HN6 A4L"}="HN62 A4L";
$resn_exch{"1H38 A4L"}="H381 A4L"; $resn_exch{"2H38 A4L"}="H382 A4L"; $resn_exch{"1H39 A4L"}="H391 A4L";
$resn_exch{"2H39 A4L"}="H392 A4L"; $resn_exch{"1H40 A4L"}="H401 A4L"; $resn_exch{"2H40 A4L"}="H402 A4L";
$resn_exch{"1H41 A4L"}="H411 A4L"; $resn_exch{"2H41 A4L"}="H412 A4L"; $resn_exch{"1HN8 A4L"}="HN81 A4L";
$resn_exch{"2HN8 A4L"}="HN82 A4L"; $resn_exch{"1H42 A4L"}="H421 A4L"; $resn_exch{"2H42 A4L"}="H422 A4L";
$resn_exch{"1H43 A4L"}="H431 A4L"; $resn_exch{"2H43 A4L"}="H432 A4L"; $resn_exch{"1H44 A4L"}="H441 A4L";
$resn_exch{"2H44 A4L"}="H442 A4L"; $resn_exch{"1H45 A4L"}="H451 A4L"; $resn_exch{"2H45 A4L"}="H452 A4L";
$resn_exch{"1HN9 A4L"}="HN91 A4L"; $resn_exch{"2HN9 A4L"}="HN92 A4L"; $resn_exch{"1H6R A4P"}="H6R1 A4P";
$resn_exch{"2H6R A4P"}="H6R2 A4P"; $resn_exch{"1H5* A4P"}="H5'1 A4P"; $resn_exch{"2H5* A4P"}="H5'2 A4P";
$resn_exch{" H4* A4P"}=" H4' A4P"; $resn_exch{" H3* A4P"}=" H3' A4P"; $resn_exch{"3HO* A4P"}="HO'3 A4P";
$resn_exch{" H2* A4P"}=" H2' A4P"; $resn_exch{"2HO* A4P"}="HO'2 A4P"; $resn_exch{"1HOA A4P"}="HOA1 A4P";
$resn_exch{"1HOB A4P"}="HOB1 A4P"; $resn_exch{"1HOG A4P"}="HOG1 A4P"; $resn_exch{"1HOD A4P"}="HOD1 A4P";
$resn_exch{"1HN2 A4P"}="HN21 A4P"; $resn_exch{"2HN2 A4P"}="HN22 A4P"; $resn_exch{"1H6A A4P"}="H6A1 A4P";
$resn_exch{"2H6A A4P"}="H6A2 A4P"; $resn_exch{"1H7  A4P"}=" H71 A4P"; $resn_exch{"2H7  A4P"}=" H72 A4P";
$resn_exch{"1H24 A53"}="H241 A53"; $resn_exch{"2H24 A53"}="H242 A53"; $resn_exch{"1H26 A53"}="H261 A53";
$resn_exch{"2H26 A53"}="H262 A53"; $resn_exch{"1H27 A53"}="H271 A53"; $resn_exch{"2H27 A53"}="H272 A53";
$resn_exch{"1H1  A58"}="H11A A58"; $resn_exch{"2H1  A58"}=" H12 A58"; $resn_exch{"3H1  A58"}=" H13 A58";
$resn_exch{"1H6  A58"}=" H61 A58"; $resn_exch{"2H6  A58"}=" H62 A58"; $resn_exch{"1H7  A58"}=" H71 A58";
$resn_exch{"2H7  A58"}=" H72 A58"; $resn_exch{"1H3  A58"}=" H31 A58"; $resn_exch{"2H3  A58"}=" H32 A58";
$resn_exch{"1H4  A58"}=" H41 A58"; $resn_exch{"2H4  A58"}=" H42 A58"; $resn_exch{"1H9  A58"}=" H91 A58";
$resn_exch{"2H9  A58"}=" H92 A58"; $resn_exch{"1H16 A58"}="H161 A58"; $resn_exch{"2H16 A58"}="H162 A58";
$resn_exch{"1HB  A5A"}=" HB1 A5A"; $resn_exch{"2HB  A5A"}=" HB2 A5A"; $resn_exch{"3HB  A5A"}=" HB3 A5A";
$resn_exch{"1H5* A5A"}="H5'1 A5A"; $resn_exch{"2H5* A5A"}="H5'2 A5A"; $resn_exch{" H4* A5A"}=" H4' A5A";
$resn_exch{" H3* A5A"}=" H3' A5A"; $resn_exch{" H2* A5A"}=" H2' A5A"; $resn_exch{"2HO* A5A"}="HO'2 A5A";
$resn_exch{" H1* A5A"}=" H1' A5A"; $resn_exch{"1H6  A5A"}=" H61 A5A"; $resn_exch{"2H6  A5A"}=" H62 A5A";
$resn_exch{"2HOP A5M"}="HOP2 A5M"; $resn_exch{"1H5* A5M"}=" H5' A5M"; $resn_exch{"2H5* A5M"}="H5'' A5M";
$resn_exch{" H4* A5M"}=" H4' A5M"; $resn_exch{" H1* A5M"}=" H1' A5M"; $resn_exch{"1H4  A5M"}=" H41 A5M";
$resn_exch{"2H4  A5M"}=" H42 A5M"; $resn_exch{" H2* A5M"}=" H2' A5M"; $resn_exch{"1NH* A5M"}="H'1N A5M";
$resn_exch{"2NH* A5M"}="H'2N A5M"; $resn_exch{" H3* A5M"}=" H3' A5M"; $resn_exch{"H3T  A5M"}="HO3' A5M";
$resn_exch{"3HOP A5M"}="HOP3 A5M"; $resn_exch{"2HOP A5P"}="HOP2 A5P"; $resn_exch{"3HOP A5P"}="HOP3 A5P";
$resn_exch{"1H8' A66"}="H8'1 A66"; $resn_exch{"2H8' A66"}="H8'2 A66"; $resn_exch{"1H5' A66"}="H5'1 A66";
$resn_exch{"1H3' A66"}="H3'1 A66"; $resn_exch{"2H3' A66"}="H3'2 A66"; $resn_exch{"1H2' A66"}="H2'1 A66";
$resn_exch{"2H2' A66"}="H2'2 A66"; $resn_exch{"1H1' A66"}="H1'1 A66"; $resn_exch{"2H1' A66"}="H1'2 A66";
$resn_exch{"1HN6 A66"}="HN61 A66"; $resn_exch{"2HN6 A66"}="HN62 A66"; $resn_exch{"1HB  A66"}=" HB1 A66";
$resn_exch{"2HB  A66"}=" HB2 A66"; $resn_exch{"1HG  A66"}=" HG1 A66"; $resn_exch{"2HG  A66"}=" HG2 A66";
$resn_exch{"1HD  A66"}=" HD1 A66"; $resn_exch{"2HD  A66"}=" HD2 A66"; $resn_exch{"1HE  A66"}=" HE1 A66";
$resn_exch{"2HE  A66"}=" HE2 A66"; $resn_exch{"1HZ  A66"}=" HZ1 A66"; $resn_exch{"2HZ  A66"}=" HZ2 A66";
$resn_exch{"1H1  A70"}=" H11 A70"; $resn_exch{"2H1  A70"}=" H12 A70"; $resn_exch{"3H1  A70"}="H13A A70";
$resn_exch{"1H2  A70"}=" H21 A70"; $resn_exch{"2H2  A70"}=" H22 A70"; $resn_exch{"1H3  A70"}=" H31 A70";
$resn_exch{"2H3  A70"}="H32A A70"; $resn_exch{"1H7  A70"}=" H71 A70"; $resn_exch{"2H7  A70"}=" H72 A70";
$resn_exch{"1H9  A70"}=" H91 A70"; $resn_exch{"2H9  A70"}=" H92 A70"; $resn_exch{"1H10 A70"}="H101 A70";
$resn_exch{"2H10 A70"}="H102 A70"; $resn_exch{"1H11 A70"}="H111 A70"; $resn_exch{"2H11 A70"}="H112 A70";
$resn_exch{"1H12 A70"}="H121 A70"; $resn_exch{"2H12 A70"}="H122 A70"; $resn_exch{"3H12 A70"}="H123 A70";
$resn_exch{"1H14 A70"}="H141 A70"; $resn_exch{"2H14 A70"}="H142 A70"; $resn_exch{"3H14 A70"}="H143 A70";
$resn_exch{"1H15 A70"}="H151 A70"; $resn_exch{"2H15 A70"}="H152 A70"; $resn_exch{"1H16 A70"}="H161 A70";
$resn_exch{"2H16 A70"}="H162 A70"; $resn_exch{"3H16 A70"}="H163 A70"; $resn_exch{"1H19 A70"}="H191 A70";
$resn_exch{"2H19 A70"}="H192 A70"; $resn_exch{"1H21 A70"}="H211 A70"; $resn_exch{"2H21 A70"}="H212 A70";
$resn_exch{"1H22 A70"}="H221 A70"; $resn_exch{"2H22 A70"}="H222 A70"; $resn_exch{"1H31 A70"}="H311 A70";
$resn_exch{"2H31 A70"}="H312 A70"; $resn_exch{"1H34 A70"}="H341 A70"; $resn_exch{"2H34 A70"}="H342 A70";
$resn_exch{"1H18 A70"}="H181 A70"; $resn_exch{"2H18 A70"}="H182 A70"; $resn_exch{"1H23 A70"}="H231 A70";
$resn_exch{"2H23 A70"}="H232 A70"; $resn_exch{"1H25 A70"}="H251 A70"; $resn_exch{"2H25 A70"}="H252 A70";
$resn_exch{"3H25 A70"}="H253 A70"; $resn_exch{"1H26 A70"}="H261 A70"; $resn_exch{"2H26 A70"}="H262 A70";
$resn_exch{"1H28 A70"}="H281 A70"; $resn_exch{"2H28 A70"}="H282 A70"; $resn_exch{"1H29 A70"}="H291 A70";
$resn_exch{"2H29 A70"}="H292 A70"; $resn_exch{"1H5  A70"}=" H51 A70"; $resn_exch{"2H5  A70"}=" H52 A70";
$resn_exch{"1H6  A70"}=" H61 A70"; $resn_exch{"2H6  A70"}=" H62 A70"; $resn_exch{"1H4  A70"}="H41A A70";
$resn_exch{"2H4  A70"}=" H42 A70"; $resn_exch{"1HN1 A71"}="HN11 A71"; $resn_exch{"2HN1 A71"}="HN12 A71";
$resn_exch{"1H2  A71"}=" H21 A71"; $resn_exch{"2H2  A71"}=" H22 A71"; $resn_exch{"1H3  A71"}=" H31 A71";
$resn_exch{"2H3  A71"}=" H32 A71"; $resn_exch{"1H4  A71"}=" H41 A71"; $resn_exch{"2H4  A71"}=" H42 A71";
$resn_exch{"1H5  A71"}=" H51 A71"; $resn_exch{"2H5  A71"}=" H52 A71"; $resn_exch{"1H6  A71"}=" H61 A71";
$resn_exch{"2H6  A71"}=" H62 A71"; $resn_exch{"1H7  A71"}=" H71 A71"; $resn_exch{"2H7  A71"}=" H72 A71";
$resn_exch{"1HN8 A71"}="HN81 A71"; $resn_exch{"2HN8 A71"}="HN82 A71"; $resn_exch{"1HN9 A71"}="HN91 A71";
$resn_exch{"2HN9 A71"}="HN92 A71"; $resn_exch{"3HN9 A71"}="HN93 A71"; $resn_exch{"1H10 A71"}="H101 A71";
$resn_exch{"2H10 A71"}="H102 A71"; $resn_exch{"3H10 A71"}="H103 A71"; $resn_exch{"1H11 A71"}="H111 A71";
$resn_exch{"2H11 A71"}="H112 A71"; $resn_exch{"3H11 A71"}="H113 A71"; $resn_exch{"1H12 A72"}="H121 A72";
$resn_exch{"2H12 A72"}="H122 A72"; $resn_exch{"3H12 A72"}="H123 A72"; $resn_exch{"1H13 A72"}="H131 A72";
$resn_exch{"2H13 A72"}="H132 A72"; $resn_exch{"1H14 A72"}="H141 A72"; $resn_exch{"2H14 A72"}="H142 A72";
$resn_exch{"1H15 A72"}="H151 A72"; $resn_exch{"2H15 A72"}="H152 A72"; $resn_exch{"1H16 A72"}="H161 A72";
$resn_exch{"2H16 A72"}="H162 A72"; $resn_exch{"1H17 A72"}="H171 A72"; $resn_exch{"2H17 A72"}="H172 A72";
$resn_exch{"1H18 A72"}="H181 A72"; $resn_exch{"2H18 A72"}="H182 A72"; $resn_exch{"1H19 A72"}="H191 A72";
$resn_exch{"2H19 A72"}="H192 A72"; $resn_exch{"3H19 A72"}="H193 A72"; $resn_exch{"1H20 A72"}="H201 A72";
$resn_exch{"2H20 A72"}="H202 A72"; $resn_exch{"3H20 A72"}="H203 A72"; $resn_exch{"1H21 A72"}="H211 A72";
$resn_exch{"2H21 A72"}="H212 A72"; $resn_exch{"1H22 A72"}="H221 A72"; $resn_exch{"2H22 A72"}="H222 A72";
$resn_exch{"1H23 A72"}="H231 A72"; $resn_exch{"2H23 A72"}="H232 A72"; $resn_exch{"1H24 A72"}="H241 A72";
$resn_exch{"2H24 A72"}="H242 A72"; $resn_exch{"1H25 A72"}="H251 A72"; $resn_exch{"2H25 A72"}="H252 A72";
$resn_exch{"1H26 A72"}="H261 A72"; $resn_exch{"2H26 A72"}="H262 A72"; $resn_exch{"1H27 A72"}="H271 A72";
$resn_exch{"2H27 A72"}="H272 A72"; $resn_exch{"3H27 A72"}="H273 A72"; $resn_exch{"1H13 A74"}="H131 A74";
$resn_exch{"2H13 A74"}="H132 A74"; $resn_exch{"3H13 A74"}="H133 A74"; $resn_exch{"1H19 A74"}="H191 A74";
$resn_exch{"2H19 A74"}="H192 A74"; $resn_exch{"3H19 A74"}="H193 A74"; $resn_exch{"1H1  A75"}=" H11 A75";
$resn_exch{"2H1  A75"}="H12A A75"; $resn_exch{"1H2  A75"}="H21A A75"; $resn_exch{"2H2  A75"}="H22A A75";
$resn_exch{"1H5  A75"}=" H51 A75"; $resn_exch{"2H5  A75"}=" H52 A75"; $resn_exch{"1H6  A75"}=" H61 A75";
$resn_exch{"2H6  A75"}=" H62 A75"; $resn_exch{"1H20 A80"}="H201 A80"; $resn_exch{"2H20 A80"}="H202 A80";
$resn_exch{"3H20 A80"}="H203 A80"; $resn_exch{"1H21 A80"}="H211 A80"; $resn_exch{"2H21 A80"}="H212 A80";
$resn_exch{"3H21 A80"}="H213 A80"; $resn_exch{"1H22 A80"}="H221 A80"; $resn_exch{"2H22 A80"}="H222 A80";
$resn_exch{"3H22 A80"}="H223 A80"; $resn_exch{"1H23 A80"}="H231 A80"; $resn_exch{"2H23 A80"}="H232 A80";
$resn_exch{"3H23 A80"}="H233 A80"; $resn_exch{"1H1  A83"}=" H11 A83"; $resn_exch{"2H1  A83"}="H12A A83";
$resn_exch{"1H6  A83"}=" H61 A83"; $resn_exch{"2H6  A83"}=" H62 A83"; $resn_exch{"1H7  A83"}=" H71 A83";
$resn_exch{"2H7  A83"}=" H72 A83"; $resn_exch{"1H9  A83"}=" H91 A83"; $resn_exch{"2H9  A83"}=" H92 A83";
$resn_exch{"3H9  A83"}=" H93 A83"; $resn_exch{"1H10 A83"}="H101 A83"; $resn_exch{"2H10 A83"}="H102 A83";
$resn_exch{"3H10 A83"}="H103 A83"; $resn_exch{"1H17 A83"}="H171 A83"; $resn_exch{"2H17 A83"}="H172 A83";
$resn_exch{"3H17 A83"}="H173 A83"; $resn_exch{"1H18 A83"}="H181 A83"; $resn_exch{"2H18 A83"}="H182 A83";
$resn_exch{"1H31 A83"}="H311 A83"; $resn_exch{"2H31 A83"}="H312 A83"; $resn_exch{"1H49 A83"}="H491 A83";
$resn_exch{"2H49 A83"}="H492 A83"; $resn_exch{"3H49 A83"}="H493 A83"; $resn_exch{"1H50 A83"}="H501 A83";
$resn_exch{"2H50 A83"}="H502 A83"; $resn_exch{"3H50 A83"}="H503 A83"; $resn_exch{"1H51 A83"}="H511 A83";
$resn_exch{"2H51 A83"}="H512 A83"; $resn_exch{"3H51 A83"}="H513 A83"; $resn_exch{"1H1  A84"}=" H11 A84";
$resn_exch{"2H1  A84"}=" H12 A84"; $resn_exch{"1H2  A84"}=" H21 A84"; $resn_exch{"2H2  A84"}="H22A A84";
$resn_exch{"1H5  A84"}=" H51 A84"; $resn_exch{"2H5  A84"}=" H52 A84"; $resn_exch{"1H6  A84"}=" H61 A84";
$resn_exch{"2H6  A84"}=" H62 A84"; $resn_exch{"1H27 A84"}="H271 A84"; $resn_exch{"2H27 A84"}="H272 A84";
$resn_exch{"1H29 A84"}="H291 A84"; $resn_exch{"2H29 A84"}="H292 A84"; $resn_exch{"3H29 A84"}="H293 A84";
$resn_exch{"1H30 A84"}="H301 A84"; $resn_exch{"2H30 A84"}="H302 A84"; $resn_exch{"3H30 A84"}="H303 A84";
$resn_exch{"1H7  A88"}=" H71 A88"; $resn_exch{"2H7  A88"}=" H72 A88"; $resn_exch{"1H8  A88"}=" H81 A88";
$resn_exch{"2H8  A88"}=" H82 A88"; $resn_exch{"1H15 A88"}="H151 A88"; $resn_exch{"2H15 A88"}="H152 A88";
$resn_exch{"1H22 A88"}="H221 A88"; $resn_exch{"2H22 A88"}="H222 A88"; $resn_exch{"1H31 A88"}="H311 A88";
$resn_exch{"2H31 A88"}="H312 A88"; $resn_exch{"3H31 A88"}="H313 A88"; $resn_exch{"1H32 A88"}="H321 A88";
$resn_exch{"2H32 A88"}="H322 A88"; $resn_exch{"1H41 A88"}="H411 A88"; $resn_exch{"2H41 A88"}="H412 A88";
$resn_exch{"3H41 A88"}="H413 A88"; $resn_exch{"1H10 A8B"}="H101 A8B"; $resn_exch{"2H10 A8B"}="H102 A8B";
$resn_exch{"1H11 A8B"}="H111 A8B"; $resn_exch{"2H11 A8B"}="H112 A8B"; $resn_exch{"1H12 A8B"}="H121 A8B";
$resn_exch{"2H12 A8B"}="H122 A8B"; $resn_exch{"1H13 A8B"}="H131 A8B"; $resn_exch{"2H13 A8B"}="H132 A8B";
$resn_exch{"1H16 A8B"}="H161 A8B"; $resn_exch{"2H16 A8B"}="H162 A8B"; $resn_exch{"1H17 A8B"}="H171 A8B";
$resn_exch{"2H17 A8B"}="H172 A8B"; $resn_exch{"1H18 A8B"}="H181 A8B"; $resn_exch{"2H18 A8B"}="H182 A8B";
$resn_exch{"1H19 A8B"}="H191 A8B"; $resn_exch{"2H19 A8B"}="H192 A8B"; $resn_exch{"1H20 A8B"}="H201 A8B";
$resn_exch{"2H20 A8B"}="H202 A8B"; $resn_exch{"1H21 A8B"}="H211 A8B"; $resn_exch{"2H21 A8B"}="H212 A8B";
$resn_exch{"1H22 A8B"}="H221 A8B"; $resn_exch{"2H22 A8B"}="H222 A8B"; $resn_exch{"1H23 A8B"}="H231 A8B";
$resn_exch{"2H23 A8B"}="H232 A8B"; $resn_exch{"2H27 A8B"}="H272 A8B"; $resn_exch{"2H28 A8B"}="H282 A8B";
$resn_exch{"2H29 A8B"}="H292 A8B"; $resn_exch{"2H30 A8B"}="H302 A8B"; $resn_exch{"2H33 A8B"}="H332 A8B";
$resn_exch{"2H34 A8B"}="H342 A8B"; $resn_exch{"1H10 A8N"}="H101 A8N"; $resn_exch{"1H13 A8N"}="H131 A8N";
$resn_exch{"2H13 A8N"}="H132 A8N"; $resn_exch{"1H14 A8N"}="H141 A8N"; $resn_exch{"2H14 A8N"}="H142 A8N";
$resn_exch{"1H15 A8N"}="H151 A8N"; $resn_exch{"2H15 A8N"}="H152 A8N"; $resn_exch{"1H16 A8N"}="H161 A8N";
$resn_exch{"2H16 A8N"}="H162 A8N"; $resn_exch{"1H17 A8N"}="H171 A8N"; $resn_exch{"2H17 A8N"}="H172 A8N";
$resn_exch{"1H18 A8N"}="H181 A8N"; $resn_exch{"2H18 A8N"}="H182 A8N"; $resn_exch{"1H19 A8N"}="H191 A8N";
$resn_exch{"2H19 A8N"}="H192 A8N"; $resn_exch{"1H20 A8N"}="H201 A8N"; $resn_exch{"2H20 A8N"}="H202 A8N";
$resn_exch{"1H21 A8N"}="H211 A8N"; $resn_exch{"2H21 A8N"}="H212 A8N"; $resn_exch{"1H22 A8N"}="H221 A8N";
$resn_exch{"2H22 A8N"}="H222 A8N"; $resn_exch{"1H23 A8N"}="H231 A8N"; $resn_exch{"2H23 A8N"}="H232 A8N";
$resn_exch{"1H24 A8N"}="H241 A8N"; $resn_exch{"2H24 A8N"}="H242 A8N"; $resn_exch{"1H25 A8N"}="H251 A8N";
$resn_exch{"2H25 A8N"}="H252 A8N"; $resn_exch{"3H25 A8N"}="H253 A8N"; $resn_exch{"1H26 A9A"}="H261 A9A";
$resn_exch{"2H26 A9A"}="H262 A9A"; $resn_exch{"1H29 A9A"}="H291 A9A"; $resn_exch{"2H29 A9A"}="H292 A9A";
$resn_exch{"3H29 A9A"}="H293 A9A"; $resn_exch{"1H30 A9A"}="H301 A9A"; $resn_exch{"2H30 A9A"}="H302 A9A";
$resn_exch{"3H30 A9A"}="H303 A9A"; $resn_exch{"1H36 A9A"}="H361 A9A"; $resn_exch{"2H36 A9A"}="H362 A9A";
$resn_exch{"1H44 A9A"}="H441 A9A"; $resn_exch{"2H44 A9A"}="H442 A9A"; $resn_exch{"3H44 A9A"}="H443 A9A";
$resn_exch{"1H45 A9A"}="H451 A9A"; $resn_exch{"2H45 A9A"}="H452 A9A"; $resn_exch{"1H47 A9A"}="H471 A9A";
$resn_exch{"2H47 A9A"}="H472 A9A"; $resn_exch{"3H47 A9A"}="H473 A9A"; $resn_exch{"1H48 A9A"}="H481 A9A";
$resn_exch{"2H48 A9A"}="H482 A9A"; $resn_exch{"3H48 A9A"}="H483 A9A"; $resn_exch{"1H49 A9A"}="H491 A9A";
$resn_exch{"2H49 A9A"}="H492 A9A"; $resn_exch{"1H50 A9A"}="H501 A9A"; $resn_exch{"2H50 A9A"}="H502 A9A";
$resn_exch{"1H53 A9A"}="H531 A9A"; $resn_exch{"2H53 A9A"}="H532 A9A"; $resn_exch{"1H54 A9A"}="H541 A9A";
$resn_exch{"2H54 A9A"}="H542 A9A"; $resn_exch{"1H55 A9A"}="H551 A9A"; $resn_exch{"2H55 A9A"}="H552 A9A";
$resn_exch{"1H58 A9A"}="H581 A9A"; $resn_exch{"2H58 A9A"}="H582 A9A"; $resn_exch{"1H60 A9A"}="H601 A9A";
$resn_exch{"2H60 A9A"}="H602 A9A"; $resn_exch{"3H60 A9A"}="H603 A9A"; $resn_exch{"1H62 A9A"}="H621 A9A";
$resn_exch{"2H62 A9A"}="H622 A9A"; $resn_exch{"1H64 A9A"}="H641 A9A"; $resn_exch{"2H64 A9A"}="H642 A9A";
$resn_exch{"3H64 A9A"}="H643 A9A"; $resn_exch{"1H65 A9A"}="H651 A9A"; $resn_exch{"2H65 A9A"}="H652 A9A";
$resn_exch{"3H65 A9A"}="H653 A9A"; $resn_exch{"1H3N AA1"}="H3N1 AA1"; $resn_exch{"2H3N AA1"}="H3N2 AA1";
$resn_exch{"1H7C AA1"}="H7C1 AA1"; $resn_exch{"2H7C AA1"}="H7C2 AA1"; $resn_exch{"1H11 AA1"}="H111 AA1";
$resn_exch{"2H11 AA1"}="H112 AA1"; $resn_exch{"1H12 AA1"}="H121 AA1"; $resn_exch{"2H12 AA1"}="H122 AA1";
$resn_exch{"1H18 AA1"}="H181 AA1"; $resn_exch{"2H18 AA1"}="H182 AA1"; $resn_exch{"1H19 AA1"}="H191 AA1";
$resn_exch{"2H19 AA1"}="H192 AA1"; $resn_exch{"3H19 AA1"}="H193 AA1"; $resn_exch{"1H22 AA2"}="H221 AA2";
$resn_exch{"2H22 AA2"}="H222 AA2"; $resn_exch{"3H22 AA2"}="H223 AA2"; $resn_exch{"1H23 AA2"}="H231 AA2";
$resn_exch{"2H23 AA2"}="H232 AA2"; $resn_exch{"3H23 AA2"}="H233 AA2"; $resn_exch{"1H24 AA2"}="H241 AA2";
$resn_exch{"2H24 AA2"}="H242 AA2"; $resn_exch{"1H25 AA2"}="H251 AA2"; $resn_exch{"2H25 AA2"}="H252 AA2";
$resn_exch{"1H26 AA2"}="H261 AA2"; $resn_exch{"2H26 AA2"}="H262 AA2"; $resn_exch{"1H28 AA2"}="H281 AA2";
$resn_exch{"2H28 AA2"}="H282 AA2"; $resn_exch{"1H29 AA2"}="H291 AA2"; $resn_exch{"2H29 AA2"}="H292 AA2";
$resn_exch{"1H31 AA2"}="H311 AA2"; $resn_exch{"2H31 AA2"}="H312 AA2"; $resn_exch{"1H32 AA2"}="H321 AA2";
$resn_exch{"2H32 AA2"}="H322 AA2"; $resn_exch{" HN  AA4"}=" H2  AA4"; $resn_exch{" HB1 AA4"}=" HB2 AA4";
$resn_exch{" HB2 AA4"}=" HB3 AA4"; $resn_exch{" HG1 AA4"}=" HG2 AA4"; $resn_exch{" HG2 AA4"}=" HG3 AA4";
$resn_exch{" HD1 AA4"}=" HD2 AA4"; $resn_exch{" HD2 AA4"}=" HD3 AA4"; $resn_exch{" HOE AA4"}=" HE  AA4";
$resn_exch{"1HB  AA5"}=" HB1 AA5"; $resn_exch{"2HB  AA5"}=" HB2 AA5"; $resn_exch{"1HG  AA5"}=" HG1 AA5";
$resn_exch{"2HG  AA5"}=" HG2 AA5"; $resn_exch{"1HE  AA5"}=" HE1 AA5"; $resn_exch{"2HE  AA5"}=" HE2 AA5";
$resn_exch{"3HE  AA5"}=" HE3 AA5"; $resn_exch{"1H2A AA5"}="H2A1 AA5"; $resn_exch{"2H2A AA5"}="H2A2 AA5";
$resn_exch{"3H2A AA5"}="H2A3 AA5"; $resn_exch{"1H5A AA5"}="H5A1 AA5"; $resn_exch{"2H5A AA5"}="H5A2 AA5";
$resn_exch{"1H3C AA7"}="H3C1 AA7"; $resn_exch{"2H3C AA7"}="H3C2 AA7"; $resn_exch{"1H4C AA7"}="H4C1 AA7";
$resn_exch{"2H4C AA7"}="H4C2 AA7"; $resn_exch{"1H5C AA7"}="H5C1 AA7"; $resn_exch{"2H5C AA7"}="H5C2 AA7";
$resn_exch{"1H6C AA7"}="H6C1 AA7"; $resn_exch{"2H6C AA7"}="H6C2 AA7"; $resn_exch{"1H19 AA7"}="H191 AA7";
$resn_exch{"2H19 AA7"}="H192 AA7"; $resn_exch{"1H20 AA7"}="H201 AA7"; $resn_exch{"2H20 AA7"}="H202 AA7";
$resn_exch{"1H21 AA7"}="H211 AA7"; $resn_exch{"2H21 AA7"}="H212 AA7"; $resn_exch{"1H22 AA7"}="H221 AA7";
$resn_exch{"2H22 AA7"}="H222 AA7"; $resn_exch{"1H23 AA7"}="H231 AA7"; $resn_exch{"2H23 AA7"}="H232 AA7";
$resn_exch{"1H24 AA7"}="H241 AA7"; $resn_exch{"2H24 AA7"}="H242 AA7"; $resn_exch{"1H25 AA7"}="H251 AA7";
$resn_exch{"2H25 AA7"}="H252 AA7"; $resn_exch{"1H28 AA7"}="H281 AA7"; $resn_exch{"2H28 AA7"}="H282 AA7";
$resn_exch{"1H29 AA7"}="H291 AA7"; $resn_exch{"2H29 AA7"}="H292 AA7"; $resn_exch{"1H30 AA7"}="H301 AA7";
$resn_exch{"2H30 AA7"}="H302 AA7"; $resn_exch{"1H31 AA7"}="H311 AA7"; $resn_exch{"2H31 AA7"}="H312 AA7";
$resn_exch{"1H1B AAA"}="H1B1 AAA"; $resn_exch{"2H1B AAA"}="H1B2 AAA"; $resn_exch{"3H1B AAA"}="H1B3 AAA";
$resn_exch{"1H4A AAA"}="H4A1 AAA"; $resn_exch{"2H4A AAA"}="H4A2 AAA"; $resn_exch{"3H4A AAA"}="H4A3 AAA";
$resn_exch{"1HN8 AAA"}="HN81 AAA"; $resn_exch{"2HN8 AAA"}="HN82 AAA"; $resn_exch{"1H5' AAB"}="H5'1 AAB";
$resn_exch{"2H5' AAB"}="H5'2 AAB"; $resn_exch{"'HO1 AAB"}="HO1' AAB"; $resn_exch{"1H2' AAB"}="H2'1 AAB";
$resn_exch{"2H2' AAB"}="H2'2 AAB"; $resn_exch{"'HO3 AAB"}="HO3' AAB"; $resn_exch{"1HC2 AAC"}="HC21 AAC";
$resn_exch{"2HC2 AAC"}="HC22 AAC"; $resn_exch{"1HC4 AAC"}="HC41 AAC"; $resn_exch{"2HC4 AAC"}="HC42 AAC";
$resn_exch{"3HC4 AAC"}="HC43 AAC"; $resn_exch{"1HN6 AAD"}="HN61 AAD"; $resn_exch{"2HN6 AAD"}="HN62 AAD";
$resn_exch{"1HN2 AAD"}="HN21 AAD"; $resn_exch{"2HN2 AAD"}="HN22 AAD"; $resn_exch{"1HCB AAD"}="HCB1 AAD";
$resn_exch{"2HCB AAD"}="HCB2 AAD"; $resn_exch{"1HCG AAD"}="HCG1 AAD"; $resn_exch{"2HCG AAD"}="HCG2 AAD";
$resn_exch{"1HCE AAD"}="HCE1 AAD"; $resn_exch{"2HCE AAD"}="HCE2 AAD"; $resn_exch{"3HCE AAD"}="HCE3 AAD";
$resn_exch{"1HC5 AAD"}="HC51 AAD"; $resn_exch{"2HC5 AAD"}="HC52 AAD"; $resn_exch{"1H4  AAE"}=" H41 AAE";
$resn_exch{"2H4  AAE"}=" H42 AAE"; $resn_exch{"1H9  AAE"}=" H91 AAE"; $resn_exch{"2H9  AAE"}=" H92 AAE";
$resn_exch{"3H9  AAE"}=" H93 AAE"; $resn_exch{"1H4  AAF"}=" H41 AAF"; $resn_exch{"2H4  AAF"}=" H42 AAF";
$resn_exch{"1H5  AAF"}=" H51 AAF"; $resn_exch{"2H5  AAF"}=" H52 AAF"; $resn_exch{"1H9  AAF"}=" H91 AAF";
$resn_exch{"2H9  AAF"}=" H92 AAF"; $resn_exch{"1H11 AAF"}="H111 AAF"; $resn_exch{"2H11 AAF"}="H112 AAF";
$resn_exch{"1H12 AAF"}="H121 AAF"; $resn_exch{"2H12 AAF"}="H122 AAF"; $resn_exch{"1H13 AAF"}="H131 AAF";
$resn_exch{"2H13 AAF"}="H132 AAF"; $resn_exch{"1H14 AAF"}="H141 AAF"; $resn_exch{"2H14 AAF"}="H142 AAF";
$resn_exch{"1H18 AAF"}="H181 AAF"; $resn_exch{"2H18 AAF"}="H182 AAF"; $resn_exch{"1H29 AAF"}="H291 AAF";
$resn_exch{"2H29 AAF"}="H292 AAF"; $resn_exch{"3H29 AAF"}="H293 AAF"; $resn_exch{"1HC2 AAG"}="HC21 AAG";
$resn_exch{"2HC2 AAG"}="HC22 AAG"; $resn_exch{"1HC4 AAG"}="HC41 AAG"; $resn_exch{"2HC4 AAG"}="HC42 AAG";
$resn_exch{"1HC5 AAG"}="HC51 AAG"; $resn_exch{"2HC5 AAG"}="HC52 AAG"; $resn_exch{"1HN3 AAG"}="HN31 AAG";
$resn_exch{"2HN3 AAG"}="HN32 AAG"; $resn_exch{"1HC9 AAG"}="HC91 AAG"; $resn_exch{"2HC9 AAG"}="HC92 AAG";
$resn_exch{"3HC9 AAG"}="HC93 AAG"; $resn_exch{"1H10 AAH"}="H101 AAH"; $resn_exch{"2H10 AAH"}="H102 AAH";
$resn_exch{"1H12 AAH"}="H121 AAH"; $resn_exch{"2H12 AAH"}="H122 AAH"; $resn_exch{"1H17 AAH"}="H171 AAH";
$resn_exch{"2H17 AAH"}="H172 AAH"; $resn_exch{"1H18 AAH"}="H181 AAH"; $resn_exch{"2H18 AAH"}="H182 AAH";
$resn_exch{"1H19 AAH"}="H191 AAH"; $resn_exch{"2H19 AAH"}="H192 AAH"; $resn_exch{"1H20 AAH"}="H201 AAH";
$resn_exch{"2H20 AAH"}="H202 AAH"; $resn_exch{"1H6  AAL"}=" H61 AAL"; $resn_exch{"2H6  AAL"}=" H62 AAL";
$resn_exch{"1HN6 AAM"}="HN61 AAM"; $resn_exch{"2HN6 AAM"}="HN62 AAM"; $resn_exch{"2HOP AAM"}="HOP2 AAM";
$resn_exch{"3HOP AAM"}="HOP3 AAM"; $resn_exch{"1H5* AAM"}="H5'1 AAM"; $resn_exch{"2H5* AAM"}="H5'2 AAM";
$resn_exch{" H4* AAM"}=" H4' AAM"; $resn_exch{" H3* AAM"}=" H3' AAM"; $resn_exch{"*HO3 AAM"}="HO3' AAM";
$resn_exch{" H2* AAM"}=" H2' AAM"; $resn_exch{"*HO2 AAM"}="HO2' AAM"; $resn_exch{" H1* AAM"}=" H1' AAM";
$resn_exch{"1H2  AAN"}=" H21 AAN"; $resn_exch{"2H2  AAN"}=" H22 AAN"; $resn_exch{"1H6A AAO"}="H6A1 AAO";
$resn_exch{"2H6A AAO"}="H6A2 AAO"; $resn_exch{"1H6B AAO"}="H6B1 AAO"; $resn_exch{"2H6B AAO"}="H6B2 AAO";
$resn_exch{"1H6C AAO"}="H6C1 AAO"; $resn_exch{"2H6C AAO"}="H6C2 AAO"; $resn_exch{"3H6C AAO"}="H6C3 AAO";
$resn_exch{"1H7H AAO"}="H7H1 AAO"; $resn_exch{"2H7H AAO"}="H7H2 AAO"; $resn_exch{"1H6G AAO"}="H6G1 AAO";
$resn_exch{"2H6G AAO"}="H6G2 AAO"; $resn_exch{"1H6T AAO"}="H6T1 AAO"; $resn_exch{"2H6T AAO"}="H6T2 AAO";
$resn_exch{"1HN  AAP"}=" HN1 AAP"; $resn_exch{"2HN  AAP"}=" HN2 AAP"; $resn_exch{"1HM' AAP"}="HM'1 AAP";
$resn_exch{"2HM' AAP"}="HM'2 AAP"; $resn_exch{"3HM' AAP"}="HM'3 AAP"; $resn_exch{"1HM5 AAP"}="HM51 AAP";
$resn_exch{"2HM5 AAP"}="HM52 AAP"; $resn_exch{"3HM5 AAP"}="HM53 AAP"; $resn_exch{"1HN  AAR"}=" H   AAR";
$resn_exch{"2HN  AAR"}=" H2  AAR"; $resn_exch{"1HB  AAR"}=" HB2 AAR"; $resn_exch{"2HB  AAR"}=" HB3 AAR";
$resn_exch{"1HG  AAR"}=" HG2 AAR"; $resn_exch{"2HG  AAR"}=" HG3 AAR"; $resn_exch{"1HD  AAR"}=" HD2 AAR";
$resn_exch{"2HD  AAR"}=" HD3 AAR"; $resn_exch{"1HH1 AAR"}="HH11 AAR"; $resn_exch{"2HH1 AAR"}="HH12 AAR";
$resn_exch{"1HH2 AAR"}="HH21 AAR"; $resn_exch{"2HH2 AAR"}="HH22 AAR"; $resn_exch{"1HNT AAR"}="HNT1 AAR";
$resn_exch{"2HNT AAR"}="HNT2 AAR"; $resn_exch{"1H8  AAS"}=" H81 AAS"; $resn_exch{"2H8  AAS"}=" H82 AAS";
$resn_exch{"3H8  AAS"}=" H83 AAS"; $resn_exch{"1HN1 AAS"}="HN11 AAS"; $resn_exch{"2HN1 AAS"}="HN12 AAS";
$resn_exch{"1HN2 AAS"}="HN21 AAS"; $resn_exch{"2HN2 AAS"}="HN22 AAS"; $resn_exch{"1HCA AAT"}="HCA1 AAT";
$resn_exch{"2HCA AAT"}="HCA2 AAT"; $resn_exch{"1H5* AAT"}="H5'1 AAT"; $resn_exch{"2H5* AAT"}="H5'2 AAT";
$resn_exch{"CH4* AAT"}="H4'C AAT"; $resn_exch{"CH3* AAT"}="H3'C AAT"; $resn_exch{"OH3* AAT"}="H3'O AAT";
$resn_exch{"CH2* AAT"}="H2'C AAT"; $resn_exch{"OH2* AAT"}="H2'O AAT"; $resn_exch{"CH1* AAT"}="H1'C AAT";
$resn_exch{"1HN6 AAT"}="HN61 AAT"; $resn_exch{"2HN6 AAT"}="HN62 AAT"; $resn_exch{"1H11 AAT"}="H111 AAT";
$resn_exch{"2H11 AAT"}="H112 AAT"; $resn_exch{"1H12 AAT"}="H121 AAT"; $resn_exch{"2H12 AAT"}="H122 AAT";
$resn_exch{"1H13 AAT"}="H131 AAT"; $resn_exch{"2H13 AAT"}="H132 AAT"; $resn_exch{"1H14 AAT"}="H141 AAT";
$resn_exch{"2H14 AAT"}="H142 AAT"; $resn_exch{"1H15 AAT"}="H151 AAT"; $resn_exch{"2H15 AAT"}="H152 AAT";
$resn_exch{"1H16 AAT"}="H161 AAT"; $resn_exch{"2H16 AAT"}="H162 AAT"; $resn_exch{"1H1  AAU"}="H11A AAU";
$resn_exch{"2H1  AAU"}="H12A AAU"; $resn_exch{"3H1  AAU"}="H13A AAU"; $resn_exch{"1H6  AAU"}=" H61 AAU";
$resn_exch{"2H6  AAU"}=" H62 AAU"; $resn_exch{"1H9  AAU"}=" H91 AAU"; $resn_exch{"2H9  AAU"}=" H92 AAU";
$resn_exch{"1H18 AAU"}="H181 AAU"; $resn_exch{"2H18 AAU"}="H182 AAU"; $resn_exch{"1H21 AAU"}="H211 AAU";
$resn_exch{"2H21 AAU"}="H212 AAU"; $resn_exch{"1H29 AAU"}="H291 AAU"; $resn_exch{"2H29 AAU"}="H292 AAU";
$resn_exch{"3H29 AAU"}="H293 AAU"; $resn_exch{"1H35 AAU"}="H351 AAU"; $resn_exch{"2H35 AAU"}="H352 AAU";
$resn_exch{"1H36 AAU"}="H361 AAU"; $resn_exch{"2H36 AAU"}="H362 AAU"; $resn_exch{"1H31 AAX"}="H311 AAX";
$resn_exch{"2H31 AAX"}="H312 AAX"; $resn_exch{"1H34 AAX"}="H341 AAX"; $resn_exch{"2H34 AAX"}="H342 AAX";
$resn_exch{"1H33 AAX"}="H331 AAX"; $resn_exch{"2H33 AAX"}="H332 AAX"; $resn_exch{"1H1  AAX"}=" H11 AAX";
$resn_exch{"2H1  AAX"}="H12A AAX"; $resn_exch{"3H1  AAX"}=" H13 AAX"; $resn_exch{"1H62 AAY"}="H621 AAY";
$resn_exch{"2H62 AAY"}="H622 AAY"; $resn_exch{"3H62 AAY"}="H623 AAY"; $resn_exch{"1H55 AAY"}="H551 AAY";
$resn_exch{"2H55 AAY"}="H552 AAY"; $resn_exch{"1H58 AAY"}="H581 AAY"; $resn_exch{"2H58 AAY"}="H582 AAY";
$resn_exch{"3H58 AAY"}="H583 AAY"; $resn_exch{"1H28 AAY"}="H281 AAY"; $resn_exch{"2H28 AAY"}="H282 AAY";
$resn_exch{"1H46 AAY"}="H461 AAY"; $resn_exch{"2H46 AAY"}="H462 AAY"; $resn_exch{"3H46 AAY"}="H463 AAY";
$resn_exch{"1H42 AAY"}="H421 AAY"; $resn_exch{"2H42 AAY"}="H422 AAY"; $resn_exch{"3H42 AAY"}="H423 AAY";
$resn_exch{"1H19 AAY"}="H191 AAY"; $resn_exch{"2H19 AAY"}="H192 AAY"; $resn_exch{"1H16 AAY"}="H161 AAY";
$resn_exch{"2H16 AAY"}="H162 AAY"; $resn_exch{"1H9  AAY"}=" H91 AAY"; $resn_exch{"2H9  AAY"}="H92A AAY";
$resn_exch{"1H4  AAY"}=" H41 AAY"; $resn_exch{"2H4  AAY"}=" H42 AAY"; $resn_exch{"1H66 AAY"}="H661 AAY";
$resn_exch{"2H66 AAY"}="H662 AAY"; $resn_exch{"1H85 AAY"}="H851 AAY"; $resn_exch{"2H85 AAY"}="H852 AAY";
$resn_exch{"1H81 AAY"}="H811 AAY"; $resn_exch{"2H81 AAY"}="H812 AAY"; $resn_exch{"3H81 AAY"}="H813 AAY";
$resn_exch{"1H2  AAZ"}="H21A AAZ"; $resn_exch{"2H2  AAZ"}="H22A AAZ"; $resn_exch{"1H1  AAZ"}=" H11 AAZ";
$resn_exch{"2H1  AAZ"}=" H12 AAZ"; $resn_exch{"3H1  AAZ"}="H13A AAZ"; $resn_exch{"1H11 AAZ"}="H111 AAZ";
$resn_exch{"2H11 AAZ"}="H112 AAZ"; $resn_exch{"3H11 AAZ"}="H113 AAZ"; $resn_exch{"1H18 AB0"}="H181 AB0";
$resn_exch{"2H18 AB0"}="H182 AB0"; $resn_exch{"1H02 AB0"}="H021 AB0"; $resn_exch{"2H02 AB0"}="H022 AB0";
$resn_exch{"1H06 AB0"}="H061 AB0"; $resn_exch{"2H06 AB0"}="H062 AB0"; $resn_exch{"1H15 AB0"}="H151 AB0";
$resn_exch{"2H15 AB0"}="H152 AB0"; $resn_exch{"1H20 AB0"}="H201 AB0"; $resn_exch{"2H20 AB0"}="H202 AB0";
$resn_exch{"1H22 AB0"}="H221 AB0"; $resn_exch{"2H22 AB0"}="H222 AB0"; $resn_exch{"3H22 AB0"}="H223 AB0";
$resn_exch{"1H1  AB2"}=" H11 AB2"; $resn_exch{"2H1  AB2"}=" H12 AB2"; $resn_exch{"3H1  AB2"}="H13A AB2";
$resn_exch{"1H8  AB2"}=" H81 AB2"; $resn_exch{"2H8  AB2"}=" H82 AB2"; $resn_exch{"1H10 AB2"}="H101 AB2";
$resn_exch{"2H10 AB2"}="H102 AB2"; $resn_exch{"3H10 AB2"}="H103 AB2"; $resn_exch{"1H11 AB2"}="H111 AB2";
$resn_exch{"2H11 AB2"}="H112 AB2"; $resn_exch{"3H11 AB2"}="H113 AB2"; $resn_exch{"1H12 AB2"}="H121 AB2";
$resn_exch{"2H12 AB2"}="H122 AB2"; $resn_exch{"1H15 AB2"}="H151 AB2"; $resn_exch{"2H15 AB2"}="H152 AB2";
$resn_exch{"1H24 AB2"}="H241 AB2"; $resn_exch{"2H24 AB2"}="H242 AB2"; $resn_exch{"1H28 AB2"}="H281 AB2";
$resn_exch{"2H28 AB2"}="H282 AB2"; $resn_exch{"1H20 AB3"}="H201 AB3"; $resn_exch{"2H20 AB3"}="H202 AB3";
$resn_exch{"3H20 AB3"}="H203 AB3"; $resn_exch{"1H19 AB3"}="H191 AB3"; $resn_exch{"2H19 AB3"}="H192 AB3";
$resn_exch{"3H19 AB3"}="H193 AB3"; $resn_exch{"1H21 AB3"}="H211 AB3"; $resn_exch{"2H21 AB3"}="H212 AB3";
$resn_exch{"3H21 AB3"}="H213 AB3"; $resn_exch{"1HN1 AB3"}="HN11 AB3"; $resn_exch{"2HN1 AB3"}="HN12 AB3";
$resn_exch{"1H22 AB6"}="H221 AB6"; $resn_exch{"2H22 AB6"}="H222 AB6"; $resn_exch{"1H25 AB6"}="H251 AB6";
$resn_exch{"2H25 AB6"}="H252 AB6"; $resn_exch{"1H26 AB6"}="H261 AB6"; $resn_exch{"2H26 AB6"}="H262 AB6";
$resn_exch{"1H29 AB6"}="H291 AB6"; $resn_exch{"2H29 AB6"}="H292 AB6"; $resn_exch{"1H30 AB6"}="H301 AB6";
$resn_exch{"2H30 AB6"}="H302 AB6"; $resn_exch{"1H33 AB6"}="H331 AB6"; $resn_exch{"2H33 AB6"}="H332 AB6";
$resn_exch{"1H34 AB6"}="H341 AB6"; $resn_exch{"2H34 AB6"}="H342 AB6"; $resn_exch{"1H21 AB6"}="H211 AB6";
$resn_exch{"2H21 AB6"}="H212 AB6"; $resn_exch{"1H61 AB6"}="H611 AB6"; $resn_exch{"2H61 AB6"}="H612 AB6";
$resn_exch{"1H32 AB6"}="H321 AB6"; $resn_exch{"2H32 AB6"}="H322 AB6"; $resn_exch{"1H1  AB6"}="H11A AB6";
$resn_exch{"2H1  AB6"}="H12A AB6"; $resn_exch{"1H35 AB6"}="H351 AB6"; $resn_exch{"2H35 AB6"}="H352 AB6";
$resn_exch{"1H27 AB6"}="H271 AB6"; $resn_exch{"2H27 AB6"}="H272 AB6"; $resn_exch{"1HN  AB7"}=" HN1 AB7";
$resn_exch{"2HN  AB7"}=" HN2 AB7"; $resn_exch{"1HBC AB7"}="HBC1 AB7"; $resn_exch{"2HBC AB7"}="HBC2 AB7";
$resn_exch{"1HGC AB7"}="HGC1 AB7"; $resn_exch{"2HGC AB7"}="HGC2 AB7"; $resn_exch{"3HGC AB7"}="HGC3 AB7";
$resn_exch{"1H88 AB8"}="H881 AB8"; $resn_exch{"2H88 AB8"}="H882 AB8"; $resn_exch{"3H88 AB8"}="H883 AB8";
$resn_exch{"1H73 AB8"}="H731 AB8"; $resn_exch{"2H73 AB8"}="H732 AB8"; $resn_exch{"1H62 AB8"}="H621 AB8";
$resn_exch{"2H62 AB8"}="H622 AB8"; $resn_exch{"1H69 AB8"}="H691 AB8"; $resn_exch{"2H69 AB8"}="H692 AB8";
$resn_exch{"3H69 AB8"}="H693 AB8"; $resn_exch{"1H4  AB8"}=" H41 AB8"; $resn_exch{"2H4  AB8"}=" H42 AB8";
$resn_exch{"1H12 AB8"}="H121 AB8"; $resn_exch{"2H12 AB8"}="H122 AB8"; $resn_exch{"1H15 AB8"}="H151 AB8";
$resn_exch{"2H15 AB8"}="H152 AB8"; $resn_exch{"1H38 AB8"}="H381 AB8"; $resn_exch{"2H38 AB8"}="H382 AB8";
$resn_exch{"3H38 AB8"}="H383 AB8"; $resn_exch{"1H42 AB8"}="H421 AB8"; $resn_exch{"2H42 AB8"}="H422 AB8";
$resn_exch{"3H42 AB8"}="H423 AB8"; $resn_exch{"1H24 AB8"}="H241 AB8"; $resn_exch{"2H24 AB8"}="H242 AB8";
$resn_exch{"1H58 AB8"}="H581 AB8"; $resn_exch{"2H58 AB8"}="H582 AB8"; $resn_exch{"3H58 AB8"}="H583 AB8";
$resn_exch{"1H51 AB8"}="H511 AB8"; $resn_exch{"2H51 AB8"}="H512 AB8"; $resn_exch{"1H54 AB8"}="H541 AB8";
$resn_exch{"2H54 AB8"}="H542 AB8"; $resn_exch{"3H54 AB8"}="H543 AB8"; $resn_exch{"1H22 AB9"}="H221 AB9";
$resn_exch{"2H22 AB9"}="H222 AB9"; $resn_exch{"1H25 AB9"}="H251 AB9"; $resn_exch{"2H25 AB9"}="H252 AB9";
$resn_exch{"1H26 AB9"}="H261 AB9"; $resn_exch{"2H26 AB9"}="H262 AB9"; $resn_exch{"1H29 AB9"}="H291 AB9";
$resn_exch{"2H29 AB9"}="H292 AB9"; $resn_exch{"1H30 AB9"}="H301 AB9"; $resn_exch{"2H30 AB9"}="H302 AB9";
$resn_exch{"1H33 AB9"}="H331 AB9"; $resn_exch{"2H33 AB9"}="H332 AB9"; $resn_exch{"1H35 AB9"}="H351 AB9";
$resn_exch{"2H35 AB9"}="H352 AB9"; $resn_exch{"1H21 AB9"}="H211 AB9"; $resn_exch{"2H21 AB9"}="H212 AB9";
$resn_exch{"1H61 AB9"}="H611 AB9"; $resn_exch{"2H61 AB9"}="H612 AB9"; $resn_exch{"1H1  AB9"}="H11A AB9";
$resn_exch{"2H1  AB9"}="H12A AB9"; $resn_exch{"1H32 AB9"}="H321 AB9"; $resn_exch{"2H32 AB9"}="H322 AB9";
$resn_exch{"1H34 AB9"}="H341 AB9"; $resn_exch{"2H34 AB9"}="H342 AB9"; $resn_exch{"1H27 AB9"}="H271 AB9";
$resn_exch{"2H27 AB9"}="H272 AB9"; $resn_exch{"1HN  ABA"}=" H   ABA"; $resn_exch{"2HN  ABA"}=" HN2 ABA";
$resn_exch{"1HB  ABA"}=" HB2 ABA"; $resn_exch{"2HB  ABA"}=" HB3 ABA"; $resn_exch{"1HG  ABA"}=" HG1 ABA";
$resn_exch{"2HG  ABA"}=" HG2 ABA"; $resn_exch{"3HG  ABA"}=" HG3 ABA"; $resn_exch{"1H54 ABB"}="H541 ABB";
$resn_exch{"2H54 ABB"}="H542 ABB"; $resn_exch{"1H52 ABB"}="H521 ABB"; $resn_exch{"2H52 ABB"}="H522 ABB";
$resn_exch{"1HCB ABB"}="HCB1 ABB"; $resn_exch{"2HCB ABB"}="HCB2 ABB"; $resn_exch{"1HCG ABB"}="HCG1 ABB";
$resn_exch{"2HCG ABB"}="HCG2 ABB"; $resn_exch{"1HCD ABB"}="HCD1 ABB"; $resn_exch{"2HCD ABB"}="HCD2 ABB";
$resn_exch{"1HH2 ABB"}="HH21 ABB"; $resn_exch{"2HH2 ABB"}="HH22 ABB"; $resn_exch{"1HC2 ABB"}="HC21 ABB";
$resn_exch{"2HC2 ABB"}="HC22 ABB"; $resn_exch{"3HC2 ABB"}="HC23 ABB"; $resn_exch{"1H6G ABC"}="H6G1 ABC";
$resn_exch{"2H6G ABC"}="H6G2 ABC"; $resn_exch{"3H6G ABC"}="H6G3 ABC"; $resn_exch{"GHO2 ABC"}="HO2G ABC";
$resn_exch{"GHO3 ABC"}="HO3G ABC"; $resn_exch{"GHO4 ABC"}="HO4G ABC"; $resn_exch{"1H6H ABC"}="H6H1 ABC";
$resn_exch{"2H6H ABC"}="H6H2 ABC"; $resn_exch{"1H6I ABC"}="H6I1 ABC"; $resn_exch{"2H6I ABC"}="H6I2 ABC";
$resn_exch{"IHO2 ABC"}="HO2I ABC"; $resn_exch{"IHO3 ABC"}="HO3I ABC"; $resn_exch{"IHO6 ABC"}="HO6I ABC";
$resn_exch{"1H6J ABC"}="H6J1 ABC"; $resn_exch{"2H6J ABC"}="H6J2 ABC"; $resn_exch{"3H6J ABC"}="H6J3 ABC";
$resn_exch{"JHO2 ABC"}="HO2J ABC"; $resn_exch{"JHO3 ABC"}="HO3J ABC"; $resn_exch{"1H6K ABC"}="H6K1 ABC";
$resn_exch{"2H6K ABC"}="H6K2 ABC"; $resn_exch{"KHO2 ABC"}="HO2K ABC"; $resn_exch{"KHO3 ABC"}="HO3K ABC";
$resn_exch{"KHO6 ABC"}="HO6K ABC"; $resn_exch{"1H6L ABC"}="H6L1 ABC"; $resn_exch{"2H6L ABC"}="H6L2 ABC";
$resn_exch{"3H6L ABC"}="H6L3 ABC"; $resn_exch{"LHO2 ABC"}="HO2L ABC"; $resn_exch{"LHO3 ABC"}="HO3L ABC";
$resn_exch{"LHO1 ABC"}="HO1L ABC"; $resn_exch{"1H6L ABD"}="H6L1 ABD"; $resn_exch{"2H6L ABD"}="H6L2 ABD";
$resn_exch{"1H6K ABD"}="H6K1 ABD"; $resn_exch{"2H6K ABD"}="H6K2 ABD"; $resn_exch{"1H6J ABD"}="H6J1 ABD";
$resn_exch{"2H6J ABD"}="H6J2 ABD"; $resn_exch{"3H6J ABD"}="H6J3 ABD"; $resn_exch{"1H6I ABD"}="H6I1 ABD";
$resn_exch{"2H6I ABD"}="H6I2 ABD"; $resn_exch{"1H6H ABD"}="H6H1 ABD"; $resn_exch{"2H6H ABD"}="H6H2 ABD";
$resn_exch{"4HOG ABD"}="HOG4 ABD"; $resn_exch{"1H6G ABD"}="H6G1 ABD"; $resn_exch{"2H6G ABD"}="H6G2 ABD";
$resn_exch{"1H3  ABE"}=" H31 ABE"; $resn_exch{"2H3  ABE"}=" H32 ABE"; $resn_exch{"1H6  ABE"}=" H61 ABE";
$resn_exch{"2H6  ABE"}=" H62 ABE"; $resn_exch{"3H6  ABE"}=" H63 ABE"; $resn_exch{"1H5' ABF"}="H5'1 ABF";
$resn_exch{"2H5' ABF"}="H5'2 ABF"; $resn_exch{"2H3B ABG"}="H3B2 ABG"; $resn_exch{"1H3B ABG"}="H3B1 ABG";
$resn_exch{"2H5' ABG"}="H5'2 ABG"; $resn_exch{"1H5' ABG"}="H5'1 ABG"; $resn_exch{"2H6  ABG"}=" H62 ABG";
$resn_exch{"1H6  ABG"}=" H61 ABG"; $resn_exch{"1H2O ABG"}="H2O1 ABG"; $resn_exch{"1H3O ABG"}="H3O1 ABG";
$resn_exch{"1HN  ABH"}=" HN1 ABH"; $resn_exch{"2HN  ABH"}=" HN2 ABH"; $resn_exch{"1HOT ABH"}="HOT1 ABH";
$resn_exch{"1HCB ABH"}="HCB1 ABH"; $resn_exch{"2HCB ABH"}="HCB2 ABH"; $resn_exch{"1HCG ABH"}="HCG1 ABH";
$resn_exch{"2HCG ABH"}="HCG2 ABH"; $resn_exch{"1HCD ABH"}="HCD1 ABH"; $resn_exch{"2HCD ABH"}="HCD2 ABH";
$resn_exch{"1HCE ABH"}="HCE1 ABH"; $resn_exch{"2HCE ABH"}="HCE2 ABH"; $resn_exch{"1HH1 ABI"}="HH11 ABI";
$resn_exch{"2HH1 ABI"}="HH12 ABI"; $resn_exch{"1HH2 ABI"}="HH21 ABI"; $resn_exch{"2HH2 ABI"}="HH22 ABI";
$resn_exch{"1HB  ABK"}=" HB1 ABK"; $resn_exch{"2HB  ABK"}=" HB2 ABK"; $resn_exch{"1H6  ABL"}=" H61 ABL";
$resn_exch{"2H6  ABL"}=" H62 ABL"; $resn_exch{"OH2A ABL"}="H2AO ABL"; $resn_exch{"OH3A ABL"}="H3AO ABL";
$resn_exch{"OH4A ABL"}="H4AO ABL"; $resn_exch{"1H6A ABL"}="H6A1 ABL"; $resn_exch{"2H6A ABL"}="H6A2 ABL";
$resn_exch{"OH6A ABL"}="H6AO ABL"; $resn_exch{"3HOP ABM"}="HOP3 ABM"; $resn_exch{"1H5* ABM"}="H5'1 ABM";
$resn_exch{"2H5* ABM"}="H5'2 ABM"; $resn_exch{" H4* ABM"}=" H4' ABM"; $resn_exch{" H3* ABM"}=" H3' ABM";
$resn_exch{"*HO3 ABM"}="HO3' ABM"; $resn_exch{" H2* ABM"}=" H2' ABM"; $resn_exch{"*HO2 ABM"}="HO2' ABM";
$resn_exch{" H1* ABM"}=" H1' ABM"; $resn_exch{"1HN6 ABM"}="HN61 ABM"; $resn_exch{"2HN6 ABM"}="HN62 ABM";
$resn_exch{"1HN  ABN"}=" HN1 ABN"; $resn_exch{"2HN  ABN"}=" HN2 ABN"; $resn_exch{"1H   ABN"}=" H1  ABN";
$resn_exch{"2H   ABN"}=" H2A ABN"; $resn_exch{"1H18 ABO"}="H181 ABO"; $resn_exch{"2H18 ABO"}="H182 ABO";
$resn_exch{"3H18 ABO"}="H183 ABO"; $resn_exch{"2HOB ABP"}="HOB2 ABP"; $resn_exch{"3HOB ABP"}="HOB3 ABP";
$resn_exch{"2HOA ABP"}="HOA2 ABP"; $resn_exch{"1H5* ABP"}="H5'1 ABP"; $resn_exch{"2H5* ABP"}="H5'2 ABP";
$resn_exch{" H4* ABP"}=" H4' ABP"; $resn_exch{" H3* ABP"}=" H3' ABP"; $resn_exch{"*HO3 ABP"}="HO3' ABP";
$resn_exch{" H2* ABP"}=" H2' ABP"; $resn_exch{"*HO2 ABP"}="HO2' ABP"; $resn_exch{" H1* ABP"}=" H1' ABP";
$resn_exch{"1HN6 ABP"}="HN61 ABP"; $resn_exch{"2HN6 ABP"}="HN62 ABP"; $resn_exch{"2HOP ABR"}="HOP2 ABR";
$resn_exch{"1H5* ABR"}=" H5' ABR"; $resn_exch{"2H5* ABR"}="H5'' ABR"; $resn_exch{" H4* ABR"}=" H4' ABR";
$resn_exch{" H1* ABR"}=" H1' ABR"; $resn_exch{"1H2* ABR"}=" H2' ABR"; $resn_exch{"2H2* ABR"}="H2'' ABR";
$resn_exch{" H3* ABR"}=" H3' ABR"; $resn_exch{"H3T  ABR"}="HO3' ABR"; $resn_exch{"3HOP ABR"}="HOP3 ABR";
$resn_exch{"2HOP ABS"}="HOP2 ABS"; $resn_exch{"1H5* ABS"}=" H5' ABS"; $resn_exch{"2H5* ABS"}="H5'' ABS";
$resn_exch{" H4* ABS"}=" H4' ABS"; $resn_exch{" H1* ABS"}=" H1' ABS"; $resn_exch{"1H2* ABS"}=" H2' ABS";
$resn_exch{"2H2* ABS"}="H2'' ABS"; $resn_exch{" H3* ABS"}=" H3' ABS"; $resn_exch{"H3T  ABS"}="HO3' ABS";
$resn_exch{"3HOP ABS"}="HOP3 ABS"; $resn_exch{"2HOG ABT"}="HOG2 ABT"; $resn_exch{"3HOG ABT"}="HOG3 ABT";
$resn_exch{"2HOB ABT"}="HOB2 ABT"; $resn_exch{"2HOA ABT"}="HOA2 ABT"; $resn_exch{"1H5* ABT"}="H5'1 ABT";
$resn_exch{"2H5* ABT"}="H5'2 ABT"; $resn_exch{" H4* ABT"}=" H4' ABT"; $resn_exch{" H1* ABT"}=" H1' ABT";
$resn_exch{"1H5  ABT"}=" H51 ABT"; $resn_exch{"2H5  ABT"}=" H52 ABT"; $resn_exch{"3H5  ABT"}=" H53 ABT";
$resn_exch{"1H2* ABT"}="H2'1 ABT"; $resn_exch{"2H2* ABT"}="H2'2 ABT"; $resn_exch{" H3* ABT"}=" H3' ABT";
$resn_exch{"1HA  ABU"}=" HA1 ABU"; $resn_exch{"2HA  ABU"}=" HA2 ABU"; $resn_exch{"1HB  ABU"}=" HB1 ABU";
$resn_exch{"2HB  ABU"}=" HB2 ABU"; $resn_exch{"1HG  ABU"}=" HG1 ABU"; $resn_exch{"2HG  ABU"}=" HG2 ABU";
$resn_exch{"1H4  ABW"}=" H41 ABW"; $resn_exch{"2H4  ABW"}=" H42 ABW"; $resn_exch{"1H7  ABW"}=" H71 ABW";
$resn_exch{"2H7  ABW"}=" H72 ABW"; $resn_exch{"3H7  ABW"}=" H73 ABW"; $resn_exch{"1H2  ABW"}=" H21 ABW";
$resn_exch{"2H2  ABW"}=" H22 ABW"; $resn_exch{"1H10 ABW"}="H101 ABW"; $resn_exch{"2H10 ABW"}="H102 ABW";
$resn_exch{"3H10 ABW"}="H103 ABW"; $resn_exch{"1H15 ABW"}="H151 ABW"; $resn_exch{"2H15 ABW"}="H152 ABW";
$resn_exch{"3H15 ABW"}="H153 ABW"; $resn_exch{"1H16 ABW"}="H161 ABW"; $resn_exch{"2H16 ABW"}="H162 ABW";
$resn_exch{"3H16 ABW"}="H163 ABW"; $resn_exch{"1H4  ABX"}=" H41 ABX"; $resn_exch{"2H4  ABX"}=" H42 ABX";
$resn_exch{"1H7  ABX"}=" H71 ABX"; $resn_exch{"2H7  ABX"}=" H72 ABX"; $resn_exch{"3H7  ABX"}=" H73 ABX";
$resn_exch{"1H9  ABX"}=" H91 ABX"; $resn_exch{"2H9  ABX"}=" H92 ABX"; $resn_exch{"1H11 ABX"}="H111 ABX";
$resn_exch{"2H11 ABX"}="H112 ABX"; $resn_exch{"3H11 ABX"}="H113 ABX"; $resn_exch{"1H12 ABX"}="H121 ABX";
$resn_exch{"2H12 ABX"}="H122 ABX"; $resn_exch{"3H12 ABX"}="H123 ABX"; $resn_exch{"1H14 ABX"}="H141 ABX";
$resn_exch{"2H14 ABX"}="H142 ABX"; $resn_exch{"3H14 ABX"}="H143 ABX"; $resn_exch{"1H7' ABY"}="H7'1 ABY";
$resn_exch{"2H7' ABY"}="H7'2 ABY"; $resn_exch{"3H7' ABY"}="H7'3 ABY"; $resn_exch{"1H'  ABY"}=" H'1 ABY";
$resn_exch{"2H'  ABY"}=" H'2 ABY"; $resn_exch{"1HB2 ABY"}="HB21 ABY"; $resn_exch{"2HB2 ABY"}="HB22 ABY";
$resn_exch{"1HG1 ABY"}="HG11 ABY"; $resn_exch{"2HG1 ABY"}="HG12 ABY"; $resn_exch{"1HB1 ABY"}="HB11 ABY";
$resn_exch{"2HB1 ABY"}="HB12 ABY"; $resn_exch{"1HA1 ABY"}="HA11 ABY"; $resn_exch{"2HA1 ABY"}="HA12 ABY";
$resn_exch{"1HN1 ABY"}="HN11 ABY"; $resn_exch{"2HN1 ABY"}="HN12 ABY"; $resn_exch{"1HA3 ABY"}="HA31 ABY";
$resn_exch{"2HA3 ABY"}="HA32 ABY"; $resn_exch{"1H7  ABZ"}=" H71 ABZ"; $resn_exch{"2H7  ABZ"}=" H72 ABZ";
$resn_exch{"1HN4 ABZ"}="HN41 ABZ"; $resn_exch{"2HN4 ABZ"}="HN42 ABZ"; $resn_exch{"1H8  AC0"}=" H81 AC0";
$resn_exch{"2H8  AC0"}=" H82 AC0"; $resn_exch{"3H8  AC0"}=" H83 AC0"; $resn_exch{"3HCA AC1"}="HCA3 AC1";
$resn_exch{"3HOA AC1"}="HOA3 AC1"; $resn_exch{"4HCA AC1"}="HCA4 AC1"; $resn_exch{"5HCA AC1"}="HCA5 AC1";
$resn_exch{"1H6A AC1"}="H6A1 AC1"; $resn_exch{"2H6A AC1"}="H6A2 AC1"; $resn_exch{"3H6A AC1"}="H6A3 AC1";
$resn_exch{"1HCB AC1"}="HCB1 AC1"; $resn_exch{"2HCB AC1"}="HCB2 AC1"; $resn_exch{"3HCB AC1"}="HCB3 AC1";
$resn_exch{"3HOB AC1"}="HOB3 AC1"; $resn_exch{"1HC6 AC1"}="HC61 AC1"; $resn_exch{"2HC6 AC1"}="HC62 AC1";
$resn_exch{"1H3* AC2"}="H3'1 AC2"; $resn_exch{"2H3* AC2"}="H3'2 AC2"; $resn_exch{"3HO* AC2"}="HO'3 AC2";
$resn_exch{"1H2* AC2"}="H2'1 AC2"; $resn_exch{"2H2* AC2"}="H2'2 AC2"; $resn_exch{"1H1* AC2"}="H1'1 AC2";
$resn_exch{"2H1* AC2"}="H1'2 AC2"; $resn_exch{"1HN2 AC2"}="HN21 AC2"; $resn_exch{"2HN2 AC2"}="HN22 AC2";
$resn_exch{"1HB1 AC5"}="HB11 AC5"; $resn_exch{"2HB1 AC5"}="HB12 AC5"; $resn_exch{"1HB2 AC5"}="HB21 AC5";
$resn_exch{"2HB2 AC5"}="HB22 AC5"; $resn_exch{"1HG1 AC5"}="HG11 AC5"; $resn_exch{"2HG1 AC5"}="HG12 AC5";
$resn_exch{"1HG2 AC5"}="HG21 AC5"; $resn_exch{"2HG2 AC5"}="HG22 AC5"; $resn_exch{"1H8  AC6"}=" H81 AC6";
$resn_exch{"2H8  AC6"}=" H82 AC6"; $resn_exch{"3H8  AC6"}=" H83 AC6"; $resn_exch{"1H4  AC9"}=" H41 AC9";
$resn_exch{"2H4  AC9"}=" H42 AC9"; $resn_exch{"3H4  AC9"}=" H43 AC9"; $resn_exch{"1H5  AC9"}=" H51 AC9";
$resn_exch{"2H5  AC9"}=" H52 AC9"; $resn_exch{"3H5  AC9"}=" H53 AC9"; $resn_exch{"1H9  AC9"}=" H91 AC9";
$resn_exch{"2H9  AC9"}=" H92 AC9"; $resn_exch{"3H9  AC9"}=" H93 AC9"; $resn_exch{"1H10 AC9"}="H101 AC9";
$resn_exch{"2H10 AC9"}="H102 AC9"; $resn_exch{"3H10 AC9"}="H103 AC9"; $resn_exch{"1H14 AC9"}="H141 AC9";
$resn_exch{"2H14 AC9"}="H142 AC9"; $resn_exch{"3H14 AC9"}="H143 AC9"; $resn_exch{"1H15 AC9"}="H151 AC9";
$resn_exch{"2H15 AC9"}="H152 AC9"; $resn_exch{"3H15 AC9"}="H153 AC9"; $resn_exch{"1H2  ACA"}=" H21 ACA";
$resn_exch{"2H2  ACA"}=" H22 ACA"; $resn_exch{"1H3  ACA"}=" H31 ACA"; $resn_exch{"2H3  ACA"}=" H32 ACA";
$resn_exch{"1H4  ACA"}=" H41 ACA"; $resn_exch{"2H4  ACA"}=" H42 ACA"; $resn_exch{"1H5  ACA"}=" H51 ACA";
$resn_exch{"2H5  ACA"}=" H52 ACA"; $resn_exch{"1H6  ACA"}=" H61 ACA"; $resn_exch{"2H6  ACA"}=" H62 ACA";
$resn_exch{"1HN6 ACA"}="HN61 ACA"; $resn_exch{"2HN6 ACA"}="HN62 ACA"; $resn_exch{"1HN2 ACB"}="HN21 ACB";
$resn_exch{"2HN2 ACB"}="HN22 ACB"; $resn_exch{"1H4  ACB"}=" H41 ACB"; $resn_exch{"2H4  ACB"}=" H42 ACB";
$resn_exch{"3H4  ACB"}=" H43 ACB"; $resn_exch{"2H5  ACC"}=" H52 ACC"; $resn_exch{"1H5  ACC"}=" H51 ACC";
$resn_exch{"2H9  ACC"}=" H92 ACC"; $resn_exch{"1H9  ACC"}=" H91 ACC"; $resn_exch{"2H10 ACC"}="H102 ACC";
$resn_exch{"1H10 ACC"}="H101 ACC"; $resn_exch{"2H13 ACC"}="H132 ACC"; $resn_exch{"1H13 ACC"}="H131 ACC";
$resn_exch{"3H19 ACC"}="H193 ACC"; $resn_exch{"2H19 ACC"}="H192 ACC"; $resn_exch{"1H19 ACC"}="H191 ACC";
$resn_exch{"2H35 ACC"}="H352 ACC"; $resn_exch{"1H35 ACC"}="H351 ACC"; $resn_exch{"3H42 ACC"}="H423 ACC";
$resn_exch{"2H42 ACC"}="H422 ACC"; $resn_exch{"1H42 ACC"}="H421 ACC"; $resn_exch{"1H2  ACD"}=" H21 ACD";
$resn_exch{"2H2  ACD"}=" H22 ACD"; $resn_exch{"1H3  ACD"}=" H31 ACD"; $resn_exch{"2H3  ACD"}=" H32 ACD";
$resn_exch{"1H4  ACD"}=" H41 ACD"; $resn_exch{"2H4  ACD"}=" H42 ACD"; $resn_exch{"1H7  ACD"}=" H71 ACD";
$resn_exch{"2H7  ACD"}=" H72 ACD"; $resn_exch{"1H10 ACD"}="H101 ACD"; $resn_exch{"2H10 ACD"}="H102 ACD";
$resn_exch{"1H13 ACD"}="H131 ACD"; $resn_exch{"2H13 ACD"}="H132 ACD"; $resn_exch{"1H16 ACD"}="H161 ACD";
$resn_exch{"2H16 ACD"}="H162 ACD"; $resn_exch{"1H17 ACD"}="H171 ACD"; $resn_exch{"2H17 ACD"}="H172 ACD";
$resn_exch{"1H18 ACD"}="H181 ACD"; $resn_exch{"2H18 ACD"}="H182 ACD"; $resn_exch{"1H19 ACD"}="H191 ACD";
$resn_exch{"2H19 ACD"}="H192 ACD"; $resn_exch{"1H20 ACD"}="H201 ACD"; $resn_exch{"2H20 ACD"}="H202 ACD";
$resn_exch{"3H20 ACD"}="H203 ACD"; $resn_exch{"1H   ACE"}=" H1  ACE"; $resn_exch{"2H   ACE"}=" H2  ACE";
$resn_exch{"3H   ACE"}=" H3  ACE"; $resn_exch{"1H4  ACF"}=" H41 ACF"; $resn_exch{"2H4  ACF"}=" H42 ACF";
$resn_exch{"1H5  ACF"}=" H51 ACF"; $resn_exch{"2H5  ACF"}=" H52 ACF"; $resn_exch{"1H9  ACF"}=" H91 ACF";
$resn_exch{"2H9  ACF"}=" H92 ACF"; $resn_exch{"1H18 ACF"}="H181 ACF"; $resn_exch{"2H18 ACF"}="H182 ACF";
$resn_exch{"1H19 ACF"}="H191 ACF"; $resn_exch{"2H19 ACF"}="H192 ACF"; $resn_exch{"1H20 ACF"}="H201 ACF";
$resn_exch{"2H20 ACF"}="H202 ACF"; $resn_exch{"1H21 ACF"}="H211 ACF"; $resn_exch{"2H21 ACF"}="H212 ACF";
$resn_exch{"1H22 ACF"}="H221 ACF"; $resn_exch{"2H22 ACF"}="H222 ACF"; $resn_exch{"1H6A ACG"}="H6A1 ACG";
$resn_exch{"2H6A ACG"}="H6A2 ACG"; $resn_exch{"2HOA ACG"}="HOA2 ACG"; $resn_exch{"3HOA ACG"}="HOA3 ACG";
$resn_exch{"6HOA ACG"}="HOA6 ACG"; $resn_exch{"1H6B ACG"}="H6B1 ACG"; $resn_exch{"2H6B ACG"}="H6B2 ACG";
$resn_exch{"3H6B ACG"}="H6B3 ACG"; $resn_exch{"2HOB ACG"}="HOB2 ACG"; $resn_exch{"3HOB ACG"}="HOB3 ACG";
$resn_exch{"1H6C ACG"}="H6C1 ACG"; $resn_exch{"2H6C ACG"}="H6C2 ACG"; $resn_exch{"2HOC ACG"}="HOC2 ACG";
$resn_exch{"3HOC ACG"}="HOC3 ACG"; $resn_exch{"6HOC ACG"}="HOC6 ACG"; $resn_exch{"1H6D ACG"}="H6D1 ACG";
$resn_exch{"2H6D ACG"}="H6D2 ACG"; $resn_exch{"1HOD ACG"}="HOD1 ACG"; $resn_exch{"2HOD ACG"}="HOD2 ACG";
$resn_exch{"3HOD ACG"}="HOD3 ACG"; $resn_exch{"6HOD ACG"}="HOD6 ACG"; $resn_exch{"1H6E ACG"}="H6E1 ACG";
$resn_exch{"2H6E ACG"}="H6E2 ACG"; $resn_exch{"3H6E ACG"}="H6E3 ACG"; $resn_exch{"2HOE ACG"}="HOE2 ACG";
$resn_exch{"3HOE ACG"}="HOE3 ACG"; $resn_exch{"4HOE ACG"}="HOE4 ACG"; $resn_exch{"1H2  ACH"}=" H21 ACH";
$resn_exch{"2H2  ACH"}=" H22 ACH"; $resn_exch{"1H3  ACH"}=" H31 ACH"; $resn_exch{"2H3  ACH"}=" H32 ACH";
$resn_exch{"1H6  ACH"}=" H61 ACH"; $resn_exch{"2H6  ACH"}=" H62 ACH"; $resn_exch{"3H6  ACH"}=" H63 ACH";
$resn_exch{"1H8  ACH"}=" H81 ACH"; $resn_exch{"2H8  ACH"}=" H82 ACH"; $resn_exch{"3H8  ACH"}=" H83 ACH";
$resn_exch{"1H9  ACH"}=" H91 ACH"; $resn_exch{"2H9  ACH"}=" H92 ACH"; $resn_exch{"3H9  ACH"}=" H93 ACH";
$resn_exch{"1H10 ACH"}="H101 ACH"; $resn_exch{"2H10 ACH"}="H102 ACH"; $resn_exch{"3H10 ACH"}="H103 ACH";
$resn_exch{"1H6  ACI"}=" H61 ACI"; $resn_exch{"2H6  ACI"}=" H62 ACI"; $resn_exch{"1HN1 ACI"}="HN11 ACI";
$resn_exch{"2HN1 ACI"}="HN12 ACI"; $resn_exch{" H5* ACK"}=" H5' ACK"; $resn_exch{" H4* ACK"}=" H4' ACK";
$resn_exch{"1H3* ACK"}="H3'1 ACK"; $resn_exch{"2H3* ACK"}="H3'2 ACK"; $resn_exch{" H3* ACK"}=" H3' ACK";
$resn_exch{" H1* ACK"}=" H1' ACK"; $resn_exch{"1H6N ACK"}="H6N1 ACK"; $resn_exch{"2H6N ACK"}="H6N2 ACK";
$resn_exch{" H2* ACK"}=" H2' ACK"; $resn_exch{"1HN  ACL"}=" H   ACL"; $resn_exch{"2HN  ACL"}=" H2  ACL";
$resn_exch{"1HB  ACL"}=" HB2 ACL"; $resn_exch{"2HB  ACL"}=" HB3 ACL"; $resn_exch{"1HG  ACL"}=" HG2 ACL";
$resn_exch{"2HG  ACL"}=" HG3 ACL"; $resn_exch{"1HD  ACL"}=" HD2 ACL"; $resn_exch{"2HD  ACL"}=" HD3 ACL";
$resn_exch{" HNE ACL"}=" HE  ACL"; $resn_exch{" HN1 ACL"}=" HH1 ACL"; $resn_exch{"1HN2 ACL"}="HH21 ACL";
$resn_exch{"2HN2 ACL"}="HH22 ACL"; $resn_exch{"1HM  ACL"}=" HM1 ACL"; $resn_exch{"2HM  ACL"}=" HM2 ACL";
$resn_exch{"1HN  ACM"}=" HN1 ACM"; $resn_exch{"2HN  ACM"}=" HN2 ACM"; $resn_exch{"1H2  ACM"}=" H21 ACM";
$resn_exch{"2H2  ACM"}=" H22 ACM"; $resn_exch{"3H2  ACM"}=" H23 ACM"; $resn_exch{"1H1  ACN"}=" H11 ACN";
$resn_exch{"2H1  ACN"}=" H12 ACN"; $resn_exch{"3H1  ACN"}=" H13 ACN"; $resn_exch{"1H2  ACN"}=" H21 ACN";
$resn_exch{"2H2  ACN"}=" H22 ACN"; $resn_exch{"3H2  ACN"}=" H23 ACN"; $resn_exch{" AH2 ACO"}=" H2A ACO";
$resn_exch{"AH61 ACO"}="H61A ACO"; $resn_exch{"AH62 ACO"}="H62A ACO"; $resn_exch{" AH8 ACO"}=" H8A ACO";
$resn_exch{"AH1* ACO"}=" H1B ACO"; $resn_exch{"AH2* ACO"}=" H2B ACO"; $resn_exch{"AHO2 ACO"}="HO2A ACO";
$resn_exch{"AH3* ACO"}=" H3B ACO"; $resn_exch{"8HOA ACO"}="HOA8 ACO"; $resn_exch{"9HOA ACO"}="HOA9 ACO";
$resn_exch{"AH4* ACO"}=" H4B ACO"; $resn_exch{"AH51 ACO"}="H51A ACO"; $resn_exch{"AH52 ACO"}="H52A ACO";
$resn_exch{"2HOA ACO"}="HOA2 ACO"; $resn_exch{"5HOA ACO"}="HOA5 ACO"; $resn_exch{"1H12 ACO"}="H121 ACO";
$resn_exch{"2H12 ACO"}="H122 ACO"; $resn_exch{"1H13 ACO"}="H131 ACO"; $resn_exch{"2H13 ACO"}="H132 ACO";
$resn_exch{"3H13 ACO"}="H133 ACO"; $resn_exch{"1H14 ACO"}="H141 ACO"; $resn_exch{"2H14 ACO"}="H142 ACO";
$resn_exch{"3H14 ACO"}="H143 ACO"; $resn_exch{"1H7  ACO"}=" H71 ACO"; $resn_exch{"2H7  ACO"}=" H72 ACO";
$resn_exch{"1H6  ACO"}=" H61 ACO"; $resn_exch{"2H6  ACO"}=" H62 ACO"; $resn_exch{"1H3  ACO"}=" H31 ACO";
$resn_exch{"2H3  ACO"}=" H32 ACO"; $resn_exch{"1H2  ACO"}=" H21 ACO"; $resn_exch{"2H2  ACO"}=" H22 ACO";
$resn_exch{"1HH3 ACO"}="HH31 ACO"; $resn_exch{"2HH3 ACO"}="HH32 ACO"; $resn_exch{"3HH3 ACO"}="HH33 ACO";
$resn_exch{"2HOG ACP"}="HOG2 ACP"; $resn_exch{"3HOG ACP"}="HOG3 ACP"; $resn_exch{"2HOB ACP"}="HOB2 ACP";
$resn_exch{"1H3B ACP"}="H3B1 ACP"; $resn_exch{"2H3B ACP"}="H3B2 ACP"; $resn_exch{"2HOA ACP"}="HOA2 ACP";
$resn_exch{"1H5* ACP"}="H5'1 ACP"; $resn_exch{"2H5* ACP"}="H5'2 ACP"; $resn_exch{" H4* ACP"}=" H4' ACP";
$resn_exch{" H3* ACP"}=" H3' ACP"; $resn_exch{"*HO3 ACP"}="HO3' ACP"; $resn_exch{" H2* ACP"}=" H2' ACP";
$resn_exch{"*HO2 ACP"}="HO2' ACP"; $resn_exch{" H1* ACP"}=" H1' ACP"; $resn_exch{"1HN6 ACP"}="HN61 ACP";
$resn_exch{"2HN6 ACP"}="HN62 ACP"; $resn_exch{"2HOG ACQ"}="HOG2 ACQ"; $resn_exch{"2HOB ACQ"}="HOB2 ACQ";
$resn_exch{"1H3B ACQ"}="H3B1 ACQ"; $resn_exch{"2H3B ACQ"}="H3B2 ACQ"; $resn_exch{"2HOA ACQ"}="HOA2 ACQ";
$resn_exch{"1H5* ACQ"}="H5'1 ACQ"; $resn_exch{"2H5* ACQ"}="H5'2 ACQ"; $resn_exch{" H4* ACQ"}=" H4' ACQ";
$resn_exch{" H3* ACQ"}=" H3' ACQ"; $resn_exch{"*HO3 ACQ"}="HO3' ACQ"; $resn_exch{" H2* ACQ"}=" H2' ACQ";
$resn_exch{"*HO2 ACQ"}="HO2' ACQ"; $resn_exch{" H1* ACQ"}=" H1' ACQ"; $resn_exch{"1HN6 ACQ"}="HN61 ACQ";
$resn_exch{"2HN6 ACQ"}="HN62 ACQ"; $resn_exch{"2HOD ACQ"}="HOD2 ACQ"; $resn_exch{"3HOD ACQ"}="HOD3 ACQ";
$resn_exch{"1H6A ACR"}="H6A1 ACR"; $resn_exch{"2H6A ACR"}="H6A2 ACR"; $resn_exch{"2HOA ACR"}="HOA2 ACR";
$resn_exch{"3HOA ACR"}="HOA3 ACR"; $resn_exch{"4HOA ACR"}="HOA4 ACR"; $resn_exch{"6HOA ACR"}="HOA6 ACR";
$resn_exch{"1H6B ACR"}="H6B1 ACR"; $resn_exch{"2H6B ACR"}="H6B2 ACR"; $resn_exch{"3H6B ACR"}="H6B3 ACR";
$resn_exch{"2HOB ACR"}="HOB2 ACR"; $resn_exch{"3HOB ACR"}="HOB3 ACR"; $resn_exch{"1H6C ACR"}="H6C1 ACR";
$resn_exch{"2H6C ACR"}="H6C2 ACR"; $resn_exch{"2HOC ACR"}="HOC2 ACR"; $resn_exch{"3HOC ACR"}="HOC3 ACR";
$resn_exch{"6HOC ACR"}="HOC6 ACR"; $resn_exch{"1H6D ACR"}="H6D1 ACR"; $resn_exch{"2H6D ACR"}="H6D2 ACR";
$resn_exch{"1HOD ACR"}="HOD1 ACR"; $resn_exch{"2HOD ACR"}="HOD2 ACR"; $resn_exch{"3HOD ACR"}="HOD3 ACR";
$resn_exch{"6HOD ACR"}="HOD6 ACR"; $resn_exch{"2H1  ACS"}=" H12 ACS"; $resn_exch{"1H1  ACS"}=" H11 ACS";
$resn_exch{"2H6  ACS"}=" H62 ACS"; $resn_exch{"1H6  ACS"}=" H61 ACS"; $resn_exch{"2H7  ACS"}=" H72 ACS";
$resn_exch{"1H7  ACS"}=" H71 ACS"; $resn_exch{"2H10 ACS"}="H102 ACS"; $resn_exch{"1H10 ACS"}="H101 ACS";
$resn_exch{"2H24 ACS"}="H242 ACS"; $resn_exch{"1H24 ACS"}="H241 ACS"; $resn_exch{"3H29 ACS"}="H293 ACS";
$resn_exch{"2H29 ACS"}="H292 ACS"; $resn_exch{"1H29 ACS"}="H291 ACS"; $resn_exch{"1H   ACT"}=" H1  ACT";
$resn_exch{"2H   ACT"}=" H2  ACT"; $resn_exch{"3H   ACT"}=" H3  ACT"; $resn_exch{"1H3  ACV"}=" H31 ACV";
$resn_exch{"2H3  ACV"}="H32A ACV"; $resn_exch{"1H4  ACV"}=" H41 ACV"; $resn_exch{"2H4  ACV"}=" H42 ACV";
$resn_exch{"1H7  ACV"}=" H71 ACV"; $resn_exch{"2H7  ACV"}=" H72 ACV"; $resn_exch{"1HNE ACV"}="HNE1 ACV";
$resn_exch{"2HNE ACV"}="HNE2 ACV"; $resn_exch{"1H16 ACV"}="H161 ACV"; $resn_exch{"2H16 ACV"}="H162 ACV";
$resn_exch{"1H33 ACV"}="H331 ACV"; $resn_exch{"2H33 ACV"}="H332 ACV"; $resn_exch{"3H33 ACV"}="H333 ACV";
$resn_exch{"1H37 ACV"}="H371 ACV"; $resn_exch{"2H37 ACV"}="H372 ACV"; $resn_exch{"3H37 ACV"}="H373 ACV";
$resn_exch{"1H3C ACW"}="H3C1 ACW"; $resn_exch{"2H3C ACW"}="H3C2 ACW"; $resn_exch{"1H4C ACW"}="H4C1 ACW";
$resn_exch{"2H4C ACW"}="H4C2 ACW"; $resn_exch{"1H7C ACW"}="H7C1 ACW"; $resn_exch{"2H7C ACW"}="H7C2 ACW";
$resn_exch{"1H14 ACW"}="H141 ACW"; $resn_exch{"2H14 ACW"}="H142 ACW"; $resn_exch{"1H16 ACW"}="H161 ACW";
$resn_exch{"2H16 ACW"}="H162 ACW"; $resn_exch{"1H33 ACW"}="H331 ACW"; $resn_exch{"2H33 ACW"}="H332 ACW";
$resn_exch{"1H37 ACW"}="H371 ACW"; $resn_exch{"2H37 ACW"}="H372 ACW"; $resn_exch{"3H37 ACW"}="H373 ACW";
$resn_exch{"1H40 ACW"}="H401 ACW"; $resn_exch{"2H40 ACW"}="H402 ACW"; $resn_exch{"1H6A ACX"}="H6A1 ACX";
$resn_exch{"2H6A ACX"}="H6A2 ACX"; $resn_exch{"AHO2 ACX"}="HO2A ACX"; $resn_exch{"AHO3 ACX"}="HO3A ACX";
$resn_exch{"AHO6 ACX"}="HO6A ACX"; $resn_exch{"1H6B ACX"}="H6B1 ACX"; $resn_exch{"2H6B ACX"}="H6B2 ACX";
$resn_exch{"BHO2 ACX"}="HO2B ACX"; $resn_exch{"BHO3 ACX"}="HO3B ACX"; $resn_exch{"BHO6 ACX"}="HO6B ACX";
$resn_exch{"1H6C ACX"}="H6C1 ACX"; $resn_exch{"2H6C ACX"}="H6C2 ACX"; $resn_exch{"CHO2 ACX"}="HO2C ACX";
$resn_exch{"CHO3 ACX"}="HO3C ACX"; $resn_exch{"CHO6 ACX"}="HO6C ACX"; $resn_exch{"1H6D ACX"}="H6D1 ACX";
$resn_exch{"2H6D ACX"}="H6D2 ACX"; $resn_exch{"DHO2 ACX"}="HO2D ACX"; $resn_exch{"DHO3 ACX"}="HO3D ACX";
$resn_exch{"DHO6 ACX"}="HO6D ACX"; $resn_exch{"1H6E ACX"}="H6E1 ACX"; $resn_exch{"2H6E ACX"}="H6E2 ACX";
$resn_exch{"EHO2 ACX"}="HO2E ACX"; $resn_exch{"EHO3 ACX"}="HO3E ACX"; $resn_exch{"EHO6 ACX"}="HO6E ACX";
$resn_exch{"1H6F ACX"}="H6F1 ACX"; $resn_exch{"2H6F ACX"}="H6F2 ACX"; $resn_exch{"FHO2 ACX"}="HO2F ACX";
$resn_exch{"FHO3 ACX"}="HO3F ACX"; $resn_exch{"FHO6 ACX"}="HO6F ACX"; $resn_exch{"1H   ACY"}=" H1  ACY";
$resn_exch{"2H   ACY"}=" H2  ACY"; $resn_exch{"3H   ACY"}=" H3  ACY"; $resn_exch{"1HNH ACZ"}="HNH1 ACZ";
$resn_exch{"2HNH ACZ"}="HNH2 ACZ"; $resn_exch{"2HE2 ACZ"}="HE22 ACZ"; $resn_exch{"2HD2 ACZ"}="HD22 ACZ";
$resn_exch{"1HD1 ACZ"}="HD11 ACZ"; $resn_exch{"1HE1 ACZ"}="HE11 ACZ"; $resn_exch{"1HCX ACZ"}="HCX1 ACZ";
$resn_exch{"2HCX ACZ"}="HCX2 ACZ"; $resn_exch{"1HCB ACZ"}="HCB1 ACZ"; $resn_exch{"2HCB ACZ"}="HCB2 ACZ";
$resn_exch{"1HN  ACZ"}=" HN1 ACZ"; $resn_exch{"2HN  ACZ"}=" HN2 ACZ"; $resn_exch{"2HOP AD2"}="HOP2 AD2";
$resn_exch{"5HOP AD2"}="HOP5 AD2"; $resn_exch{"6HOP AD2"}="HOP6 AD2"; $resn_exch{"1H6  AD2"}=" H61 AD2";
$resn_exch{"2H6  AD2"}=" H62 AD2"; $resn_exch{"1H2' AD2"}="H2'1 AD2"; $resn_exch{"2H2' AD2"}="H2'2 AD2";
$resn_exch{"1H5' AD2"}="H5'1 AD2"; $resn_exch{"2H5' AD2"}="H5'2 AD2"; $resn_exch{"3HOP AD2"}="HOP3 AD2";
$resn_exch{"*HO5 AD3"}="HO5' AD3"; $resn_exch{"1H5* AD3"}="H5'1 AD3"; $resn_exch{"2H5* AD3"}="H5'2 AD3";
$resn_exch{" H4* AD3"}=" H4' AD3"; $resn_exch{" H3* AD3"}=" H3' AD3"; $resn_exch{"*HO3 AD3"}="HO3' AD3";
$resn_exch{" H2* AD3"}=" H2' AD3"; $resn_exch{"*HO2 AD3"}="HO2' AD3"; $resn_exch{" H1* AD3"}=" H1' AD3";
$resn_exch{"1HN6 AD3"}="HN61 AD3"; $resn_exch{"2HN6 AD3"}="HN62 AD3"; $resn_exch{"1H34 AD4"}="H341 AD4";
$resn_exch{"2H34 AD4"}="H342 AD4"; $resn_exch{"3H34 AD4"}="H343 AD4"; $resn_exch{"1H9C AD4"}="H9C1 AD4";
$resn_exch{"2H9C AD4"}="H9C2 AD4"; $resn_exch{"1H14 AD4"}="H141 AD4"; $resn_exch{"2H14 AD4"}="H142 AD4";
$resn_exch{"1H15 AD4"}="H151 AD4"; $resn_exch{"2H15 AD4"}="H152 AD4"; $resn_exch{"1H16 AD4"}="H161 AD4";
$resn_exch{"2H16 AD4"}="H162 AD4"; $resn_exch{"1H17 AD4"}="H171 AD4"; $resn_exch{"2H17 AD4"}="H172 AD4";
$resn_exch{"1H18 AD4"}="H181 AD4"; $resn_exch{"2H18 AD4"}="H182 AD4"; $resn_exch{"1H19 AD4"}="H191 AD4";
$resn_exch{"2H19 AD4"}="H192 AD4"; $resn_exch{"1H20 AD4"}="H201 AD4"; $resn_exch{"2H20 AD4"}="H202 AD4";
$resn_exch{"3H20 AD4"}="H203 AD4"; $resn_exch{"1H23 AD4"}="H231 AD4"; $resn_exch{"2H23 AD4"}="H232 AD4";
$resn_exch{"3H23 AD4"}="H233 AD4"; $resn_exch{"1H25 AD4"}="H251 AD4"; $resn_exch{"2H25 AD4"}="H252 AD4";
$resn_exch{"3H25 AD4"}="H253 AD4"; $resn_exch{"1H26 AD4"}="H261 AD4"; $resn_exch{"2H26 AD4"}="H262 AD4";
$resn_exch{"3H26 AD4"}="H263 AD4"; $resn_exch{"1H31 AD4"}="H311 AD4"; $resn_exch{"2H31 AD4"}="H312 AD4";
$resn_exch{"3H31 AD4"}="H313 AD4"; $resn_exch{"1H33 AD4"}="H331 AD4"; $resn_exch{"2H33 AD4"}="H332 AD4";
$resn_exch{"3H33 AD4"}="H333 AD4"; $resn_exch{"1H28 AD4"}="H281 AD4"; $resn_exch{"2H28 AD4"}="H282 AD4";
$resn_exch{"1H29 AD4"}="H291 AD4"; $resn_exch{"2H29 AD4"}="H292 AD4"; $resn_exch{"1H30 AD4"}="H301 AD4";
$resn_exch{"2H30 AD4"}="H302 AD4"; $resn_exch{"1H57 AD4"}="H571 AD4"; $resn_exch{"2H57 AD4"}="H572 AD4";
$resn_exch{"3H57 AD4"}="H573 AD4"; $resn_exch{"1H55 AD4"}="H551 AD4"; $resn_exch{"2H55 AD4"}="H552 AD4";
$resn_exch{"3H55 AD4"}="H553 AD4"; $resn_exch{"1H54 AD4"}="H541 AD4"; $resn_exch{"2H54 AD4"}="H542 AD4";
$resn_exch{"3H54 AD4"}="H543 AD4"; $resn_exch{"1H52 AD4"}="H521 AD4"; $resn_exch{"2H52 AD4"}="H522 AD4";
$resn_exch{"1H51 AD4"}="H511 AD4"; $resn_exch{"2H51 AD4"}="H512 AD4"; $resn_exch{"1H50 AD4"}="H501 AD4";
$resn_exch{"2H50 AD4"}="H502 AD4"; $resn_exch{"1H49 AD4"}="H491 AD4"; $resn_exch{"2H49 AD4"}="H492 AD4";
$resn_exch{"1H48 AD4"}="H481 AD4"; $resn_exch{"2H48 AD4"}="H482 AD4"; $resn_exch{"1H47 AD4"}="H471 AD4";
$resn_exch{"2H47 AD4"}="H472 AD4"; $resn_exch{"1H46 AD4"}="H461 AD4"; $resn_exch{"2H46 AD4"}="H462 AD4";
$resn_exch{"1H45 AD4"}="H451 AD4"; $resn_exch{"2H45 AD4"}="H452 AD4"; $resn_exch{"1HAA AD6"}="HAA1 AD6";
$resn_exch{"2HAA AD6"}="HAA2 AD6"; $resn_exch{"3HAA AD6"}="HAA3 AD6"; $resn_exch{"1HN4 ADB"}="HN41 ADB";
$resn_exch{"2HN4 ADB"}="HN42 ADB"; $resn_exch{"1HN6 ADC"}="HN61 ADC"; $resn_exch{"2HN6 ADC"}="HN62 ADC";
$resn_exch{"1HM2 ADD"}="HM21 ADD"; $resn_exch{"2HM2 ADD"}="HM22 ADD"; $resn_exch{"3HM2 ADD"}="HM23 ADD";
$resn_exch{"1HN3 ADD"}="HN31 ADD"; $resn_exch{"2HN3 ADD"}="HN32 ADD"; $resn_exch{"1H4  ADD"}=" H41 ADD";
$resn_exch{"2H4  ADD"}=" H42 ADD"; $resn_exch{"1H5  ADD"}=" H51 ADD"; $resn_exch{"2H5  ADD"}=" H52 ADD";
$resn_exch{"1HM6 ADD"}="HM61 ADD"; $resn_exch{"2HM6 ADD"}="HM62 ADD"; $resn_exch{"3HM6 ADD"}="HM63 ADD";
$resn_exch{"1H7  ADD"}=" H71 ADD"; $resn_exch{"2H7  ADD"}=" H72 ADD"; $resn_exch{"1HM8 ADD"}="HM81 ADD";
$resn_exch{"2HM8 ADD"}="HM82 ADD"; $resn_exch{"3HM8 ADD"}="HM83 ADD"; $resn_exch{"1HM9 ADD"}="HM91 ADD";
$resn_exch{"2HM9 ADD"}="HM92 ADD"; $resn_exch{"3HM9 ADD"}="HM93 ADD"; $resn_exch{"1H'  ADD"}=" H'1 ADD";
$resn_exch{"2H'  ADD"}=" H'2 ADD"; $resn_exch{"1HN6 ADE"}="HN61 ADE"; $resn_exch{"2HN6 ADE"}="HN62 ADE";
$resn_exch{"1H4  ADF"}=" H41 ADF"; $resn_exch{"2H4  ADF"}=" H42 ADF"; $resn_exch{"1H5  ADF"}=" H51 ADF";
$resn_exch{"2H5  ADF"}=" H52 ADF"; $resn_exch{"1H9  ADF"}=" H91 ADF"; $resn_exch{"2H9  ADF"}=" H92 ADF";
$resn_exch{"1H13 ADF"}="H131 ADF"; $resn_exch{"2H13 ADF"}="H132 ADF"; $resn_exch{"1H14 ADF"}="H141 ADF";
$resn_exch{"2H14 ADF"}="H142 ADF"; $resn_exch{"1H15 ADF"}="H151 ADF"; $resn_exch{"2H15 ADF"}="H152 ADF";
$resn_exch{"1H21 ADF"}="H211 ADF"; $resn_exch{"2H21 ADF"}="H212 ADF"; $resn_exch{"1H22 ADF"}="H221 ADF";
$resn_exch{"2H22 ADF"}="H222 ADF"; $resn_exch{"1H25 ADF"}="H251 ADF"; $resn_exch{"2H25 ADF"}="H252 ADF";
$resn_exch{"3H25 ADF"}="H253 ADF"; $resn_exch{"1H26 ADF"}="H261 ADF"; $resn_exch{"2H26 ADF"}="H262 ADF";
$resn_exch{"3H26 ADF"}="H263 ADF"; $resn_exch{"1H27 ADF"}="H271 ADF"; $resn_exch{"2H27 ADF"}="H272 ADF";
$resn_exch{"3H27 ADF"}="H273 ADF"; $resn_exch{"1H3  ADG"}=" H31 ADG"; $resn_exch{"2H3  ADG"}=" H32 ADG";
$resn_exch{"3H3  ADG"}=" H33 ADG"; $resn_exch{"1H6  ADG"}=" H61 ADG"; $resn_exch{"2H6  ADG"}=" H62 ADG";
$resn_exch{"1H4  ADH"}=" H41 ADH"; $resn_exch{"2H4  ADH"}=" H42 ADH"; $resn_exch{"1H6  ADH"}=" H61 ADH";
$resn_exch{"2H6  ADH"}=" H62 ADH"; $resn_exch{"1H7  ADH"}=" H71 ADH"; $resn_exch{"2HN1 ADH"}="HN12 ADH";
$resn_exch{"2HN2 ADH"}="HN22 ADH"; $resn_exch{"2HOB ADI"}="HOB2 ADI"; $resn_exch{"3HOB ADI"}="HOB3 ADI";
$resn_exch{"2HOA ADI"}="HOA2 ADI"; $resn_exch{"1H5* ADI"}="H5'1 ADI"; $resn_exch{"2H5* ADI"}="H5'2 ADI";
$resn_exch{" H4* ADI"}=" H4' ADI"; $resn_exch{"1H3* ADI"}="H3'1 ADI"; $resn_exch{"2H3* ADI"}="H3'2 ADI";
$resn_exch{"1H2* ADI"}="H2'1 ADI"; $resn_exch{"2H2* ADI"}="H2'2 ADI"; $resn_exch{" H1* ADI"}=" H1' ADI";
$resn_exch{"1HN6 ADI"}="HN61 ADI"; $resn_exch{"2HN6 ADI"}="HN62 ADI"; $resn_exch{"1H5' ADJ"}="H5'1 ADJ";
$resn_exch{"2H5' ADJ"}="H5'2 ADJ"; $resn_exch{"1HN6 ADJ"}="HN61 ADJ"; $resn_exch{"2HN6 ADJ"}="HN62 ADJ";
$resn_exch{"2HO' ADJ"}="HO'2 ADJ"; $resn_exch{"3HO' ADJ"}="HO'3 ADJ"; $resn_exch{"7HON ADJ"}="HON7 ADJ";
$resn_exch{"1H5R ADJ"}="H5R1 ADJ"; $resn_exch{"2H5R ADJ"}="H5R2 ADJ"; $resn_exch{"2HOR ADJ"}="HOR2 ADJ";
$resn_exch{"3HOR ADJ"}="HOR3 ADJ"; $resn_exch{"1H5S ADJ"}="H5S1 ADJ"; $resn_exch{"2H5S ADJ"}="H5S2 ADJ";
$resn_exch{"1HX6 ADJ"}="HX61 ADJ"; $resn_exch{"2HX6 ADJ"}="HX62 ADJ"; $resn_exch{"2HOS ADJ"}="HOS2 ADJ";
$resn_exch{"3HOS ADJ"}="HOS3 ADJ"; $resn_exch{"1HN6 ADK"}="HN61 ADK"; $resn_exch{"2HN6 ADK"}="HN62 ADK";
$resn_exch{"1H3A ADK"}="H3A1 ADK"; $resn_exch{"2H3A ADK"}="H3A2 ADK"; $resn_exch{"3H3A ADK"}="H3A3 ADK";
$resn_exch{"1H10 ADL"}="H101 ADL"; $resn_exch{"2H10 ADL"}="H102 ADL"; $resn_exch{"1H15 ADL"}="H151 ADL";
$resn_exch{"2H15 ADL"}="H152 ADL"; $resn_exch{"1H2  ADM"}=" H21 ADM"; $resn_exch{"2H2  ADM"}=" H22 ADM";
$resn_exch{"1H4  ADM"}=" H41 ADM"; $resn_exch{"2H4  ADM"}=" H42 ADM"; $resn_exch{"1H6  ADM"}=" H61 ADM";
$resn_exch{"2H6  ADM"}=" H62 ADM"; $resn_exch{"1H8  ADM"}=" H81 ADM"; $resn_exch{"2H8  ADM"}=" H82 ADM";
$resn_exch{"1H9  ADM"}=" H91 ADM"; $resn_exch{"2H9  ADM"}=" H92 ADM"; $resn_exch{"1H10 ADM"}="H101 ADM";
$resn_exch{"2H10 ADM"}="H102 ADM"; $resn_exch{"*HO5 ADN"}="HO5' ADN"; $resn_exch{"1H5* ADN"}="H5'1 ADN";
$resn_exch{"2H5* ADN"}="H5'2 ADN"; $resn_exch{" H4* ADN"}=" H4' ADN"; $resn_exch{" H3* ADN"}=" H3' ADN";
$resn_exch{"*HO3 ADN"}="HO3' ADN"; $resn_exch{" H2* ADN"}=" H2' ADN"; $resn_exch{"*HO2 ADN"}="HO2' ADN";
$resn_exch{" H1* ADN"}=" H1' ADN"; $resn_exch{"1HN6 ADN"}="HN61 ADN"; $resn_exch{"2HN6 ADN"}="HN62 ADN";
$resn_exch{"1H4  ADO"}=" H41 ADO"; $resn_exch{"2H4  ADO"}=" H42 ADO"; $resn_exch{"1H6  ADO"}=" H61 ADO";
$resn_exch{"2H6  ADO"}=" H62 ADO"; $resn_exch{"1H8  ADO"}=" H81 ADO"; $resn_exch{"2H8  ADO"}=" H82 ADO";
$resn_exch{"1H9  ADO"}=" H91 ADO"; $resn_exch{"2H9  ADO"}=" H92 ADO"; $resn_exch{"1H10 ADO"}="H101 ADO";
$resn_exch{"2H10 ADO"}="H102 ADO"; $resn_exch{"2HOB ADP"}="HOB2 ADP"; $resn_exch{"3HOB ADP"}="HOB3 ADP";
$resn_exch{"2HOA ADP"}="HOA2 ADP"; $resn_exch{"1H5* ADP"}="H5'1 ADP"; $resn_exch{"2H5* ADP"}="H5'2 ADP";
$resn_exch{" H4* ADP"}=" H4' ADP"; $resn_exch{" H3* ADP"}=" H3' ADP"; $resn_exch{"*HO3 ADP"}="HO3' ADP";
$resn_exch{" H2* ADP"}=" H2' ADP"; $resn_exch{"*HO2 ADP"}="HO2' ADP"; $resn_exch{" H1* ADP"}=" H1' ADP";
$resn_exch{"1HN6 ADP"}="HN61 ADP"; $resn_exch{"2HN6 ADP"}="HN62 ADP"; $resn_exch{"2HOB ADQ"}="HOB2 ADQ";
$resn_exch{"2HOA ADQ"}="HOA2 ADQ"; $resn_exch{"1H5* ADQ"}="H5'1 ADQ"; $resn_exch{"2H5* ADQ"}="H5'2 ADQ";
$resn_exch{" H4* ADQ"}=" H4D ADQ"; $resn_exch{" H3* ADQ"}=" H3D ADQ"; $resn_exch{"*HO3 ADQ"}="HO3' ADQ";
$resn_exch{" H2* ADQ"}=" H2D ADQ"; $resn_exch{"*HO2 ADQ"}="HO2' ADQ"; $resn_exch{" H1* ADQ"}=" H1D ADQ";
$resn_exch{"1HN6 ADQ"}="HN61 ADQ"; $resn_exch{"2HN6 ADQ"}="HN62 ADQ"; $resn_exch{"1H6' ADQ"}="H6'1 ADQ";
$resn_exch{"2H6' ADQ"}="H6'2 ADQ"; $resn_exch{"'HO2 ADQ"}="HO2A ADQ"; $resn_exch{"'HO3 ADQ"}="HO3A ADQ";
$resn_exch{"'HO4 ADQ"}="HO4' ADQ"; $resn_exch{"'HO6 ADQ"}="HO6' ADQ"; $resn_exch{"1H1  ADR"}=" H11 ADR";
$resn_exch{"2H1  ADR"}=" H12 ADR"; $resn_exch{"1H2  ADR"}=" H21 ADR"; $resn_exch{"2H2  ADR"}=" H22 ADR";
$resn_exch{"1H3  ADR"}=" H31 ADR"; $resn_exch{"2H3  ADR"}=" H32 ADR"; $resn_exch{"1H5  ADR"}=" H51 ADR";
$resn_exch{"2H5  ADR"}=" H52 ADR"; $resn_exch{"1H6  ADR"}=" H61 ADR"; $resn_exch{"2H6  ADR"}=" H62 ADR";
$resn_exch{"2HSP ADS"}="HSP2 ADS"; $resn_exch{"3HSP ADS"}="HSP3 ADS"; $resn_exch{"1H5* ADS"}="H5'1 ADS";
$resn_exch{"2H5* ADS"}="H5'2 ADS"; $resn_exch{" H4* ADS"}=" H4' ADS"; $resn_exch{" H3* ADS"}=" H3' ADS";
$resn_exch{"*HO3 ADS"}="HO3' ADS"; $resn_exch{" H2* ADS"}=" H2' ADS"; $resn_exch{"*HO2 ADS"}="HO2' ADS";
$resn_exch{" H1* ADS"}=" H1' ADS"; $resn_exch{"1HN6 ADS"}="HN61 ADS"; $resn_exch{"2HN6 ADS"}="HN62 ADS";
$resn_exch{"1HM5 ADT"}="HM51 ADT"; $resn_exch{"2HM5 ADT"}="HM52 ADT"; $resn_exch{"3HM5 ADT"}="HM53 ADT";
$resn_exch{" H1* ADT"}=" H1' ADT"; $resn_exch{" H4* ADT"}=" H4' ADT"; $resn_exch{"1H5* ADT"}="H5'1 ADT";
$resn_exch{"2H5* ADT"}="H5'2 ADT"; $resn_exch{"*HO5 ADT"}="HO5' ADT"; $resn_exch{" H3* ADT"}=" H3' ADT";
$resn_exch{"*HN3 ADT"}="HN3' ADT"; $resn_exch{"1H9* ADT"}="H9'1 ADT"; $resn_exch{"2H9* ADT"}="H9'2 ADT";
$resn_exch{"3H9* ADT"}="H9'3 ADT"; $resn_exch{"1H2* ADT"}="H2'1 ADT"; $resn_exch{"2H2* ADT"}="H2'2 ADT";
$resn_exch{" H1* ADU"}=" H1' ADU"; $resn_exch{" H4* ADU"}=" H4' ADU"; $resn_exch{"1H5* ADU"}="H5'1 ADU";
$resn_exch{"2H5* ADU"}="H5'2 ADU"; $resn_exch{"*HO5 ADU"}="HO5' ADU"; $resn_exch{" H3* ADU"}=" H3' ADU";
$resn_exch{"*HN3 ADU"}="HN3' ADU"; $resn_exch{"1H9* ADU"}="H9'1 ADU"; $resn_exch{"2H9* ADU"}="H9'2 ADU";
$resn_exch{"3H9* ADU"}="H9'3 ADU"; $resn_exch{" H2* ADU"}=" H2' ADU"; $resn_exch{"*HO2 ADU"}="HO2' ADU";
$resn_exch{"1H5* ADV"}="H5'1 ADV"; $resn_exch{"2H5* ADV"}="H5'2 ADV"; $resn_exch{"4H*  ADV"}=" H'4 ADV";
$resn_exch{"3H*  ADV"}=" H'3 ADV"; $resn_exch{"3HO* ADV"}="HO'3 ADV"; $resn_exch{"2H*  ADV"}=" H'2 ADV";
$resn_exch{"2HO* ADV"}="HO'2 ADV"; $resn_exch{"1H*  ADV"}=" H'1 ADV"; $resn_exch{"1H6  ADV"}=" H61 ADV";
$resn_exch{"2H6  ADV"}=" H62 ADV"; $resn_exch{"1HX  ADV"}=" HX1 ADV"; $resn_exch{"2HX  ADV"}=" HX2 ADV";
$resn_exch{"1H5R ADV"}="H5R1 ADV"; $resn_exch{"2H5R ADV"}="H5R2 ADV"; $resn_exch{"4HR  ADV"}=" HR4 ADV";
$resn_exch{"3HR  ADV"}=" HR3 ADV"; $resn_exch{"3HOR ADV"}="HOR3 ADV"; $resn_exch{"2HR  ADV"}=" HR2 ADV";
$resn_exch{"2HOR ADV"}="HOR2 ADV"; $resn_exch{"1HR  ADV"}=" HR1 ADV"; $resn_exch{"1HOR ADV"}="HOR1 ADV";
$resn_exch{"2HOA ADV"}="HOA2 ADV"; $resn_exch{"2HOB ADV"}="HOB2 ADV"; $resn_exch{"2HOB ADW"}="HOB2 ADW";
$resn_exch{"3HOB ADW"}="HOB3 ADW"; $resn_exch{"2HOA ADW"}="HOA2 ADW"; $resn_exch{"1H5* ADW"}="H5'1 ADW";
$resn_exch{"2H5* ADW"}="H5'2 ADW"; $resn_exch{" H4* ADW"}=" H4' ADW"; $resn_exch{" H3* ADW"}=" H3' ADW";
$resn_exch{"*HO3 ADW"}="HO3' ADW"; $resn_exch{" H2* ADW"}=" H2' ADW"; $resn_exch{"*HO2 ADW"}="HO2' ADW";
$resn_exch{" H1* ADW"}=" H1' ADW"; $resn_exch{"1HN6 ADW"}="HN61 ADW"; $resn_exch{"2HN6 ADW"}="HN62 ADW";
$resn_exch{"3HOB ADX"}="HOB3 ADX"; $resn_exch{"2HOA ADX"}="HOA2 ADX"; $resn_exch{"1H5* ADX"}=" H5' ADX";
$resn_exch{"2H5* ADX"}="H5'' ADX"; $resn_exch{" H4* ADX"}=" H4' ADX"; $resn_exch{" H3* ADX"}=" H3' ADX";
$resn_exch{"*HO3 ADX"}="HO3' ADX"; $resn_exch{" H2* ADX"}=" H2' ADX"; $resn_exch{"*HO2 ADX"}="HO2' ADX";
$resn_exch{" H1* ADX"}=" H1' ADX"; $resn_exch{"1HN6 ADX"}="HN61 ADX"; $resn_exch{"2HN6 ADX"}="HN62 ADX";
$resn_exch{"*HO5 ADY"}="HO5' ADY"; $resn_exch{"1H5* ADY"}="H5'1 ADY"; $resn_exch{"2H5* ADY"}="H5'2 ADY";
$resn_exch{" H4* ADY"}=" H4' ADY"; $resn_exch{" H2* ADY"}=" H2' ADY"; $resn_exch{"*HO2 ADY"}="HO2' ADY";
$resn_exch{" H1* ADY"}=" H1' ADY"; $resn_exch{"1HN6 ADY"}="HN61 ADY"; $resn_exch{"2HN6 ADY"}="HN62 ADY";
$resn_exch{"1H6N ADZ"}="H6N1 ADZ"; $resn_exch{"2H6N ADZ"}="H6N2 ADZ"; $resn_exch{"1H1* ADZ"}="H1'1 ADZ";
$resn_exch{"2H1* ADZ"}="H1'2 ADZ"; $resn_exch{"3H1* ADZ"}="H1'3 ADZ"; $resn_exch{"1H2  AE1"}=" H21 AE1";
$resn_exch{"2H2  AE1"}=" H22 AE1"; $resn_exch{"1HN  AE1"}=" HN1 AE1"; $resn_exch{"2HN  AE1"}=" HN2 AE1";
$resn_exch{"1H8  AE1"}=" H81 AE1"; $resn_exch{"2H8  AE1"}=" H82 AE1"; $resn_exch{"1H5  AE1"}=" H51 AE1";
$resn_exch{"2H5  AE1"}=" H52 AE1"; $resn_exch{"1H17 AE1"}="H171 AE1"; $resn_exch{"2H17 AE1"}="H172 AE1";
$resn_exch{"3H17 AE1"}="H173 AE1"; $resn_exch{"3H18 AE1"}="H183 AE1"; $resn_exch{"1H18 AE1"}="H181 AE1";
$resn_exch{"2H18 AE1"}="H182 AE1"; $resn_exch{"1H1  AE2"}=" H11 AE2"; $resn_exch{"2H1  AE2"}=" H12 AE2";
$resn_exch{"1H2  AE2"}=" H21 AE2"; $resn_exch{"2H2  AE2"}=" H22 AE2"; $resn_exch{"1H4  AE2"}=" H41 AE2";
$resn_exch{"2H4  AE2"}=" H42 AE2"; $resn_exch{"1H6  AE2"}=" H61 AE2"; $resn_exch{"2H6  AE2"}=" H62 AE2";
$resn_exch{"1H7  AE2"}=" H71 AE2"; $resn_exch{"2H7  AE2"}=" H72 AE2"; $resn_exch{"1H11 AE2"}="H111 AE2";
$resn_exch{"2H11 AE2"}="H112 AE2"; $resn_exch{"1H12 AE2"}="H121 AE2"; $resn_exch{"2H12 AE2"}="H122 AE2";
$resn_exch{"1H15 AE2"}="H151 AE2"; $resn_exch{"2H15 AE2"}="H152 AE2"; $resn_exch{"1H16 AE2"}="H161 AE2";
$resn_exch{"2H16 AE2"}="H162 AE2"; $resn_exch{"1H18 AE2"}="H181 AE2"; $resn_exch{"2H18 AE2"}="H182 AE2";
$resn_exch{"3H18 AE2"}="H183 AE2"; $resn_exch{"1H19 AE2"}="H191 AE2"; $resn_exch{"2H19 AE2"}="H192 AE2";
$resn_exch{"3H19 AE2"}="H193 AE2"; $resn_exch{"1H1C AE3"}="H1C1 AE3"; $resn_exch{"2H1C AE3"}="H1C2 AE3";
$resn_exch{"3H1C AE3"}="H1C3 AE3"; $resn_exch{"1H2C AE3"}="H2C1 AE3"; $resn_exch{"2H2C AE3"}="H2C2 AE3";
$resn_exch{"1H3C AE3"}="H3C1 AE3"; $resn_exch{"2H3C AE3"}="H3C2 AE3"; $resn_exch{"1H4C AE3"}="H4C1 AE3";
$resn_exch{"2H4C AE3"}="H4C2 AE3"; $resn_exch{"1H5C AE3"}="H5C1 AE3"; $resn_exch{"2H5C AE3"}="H5C2 AE3";
$resn_exch{"1H6C AE3"}="H6C1 AE3"; $resn_exch{"2H6C AE3"}="H6C2 AE3"; $resn_exch{"1H2C AE4"}="H2C1 AE4";
$resn_exch{"2H2C AE4"}="H2C2 AE4"; $resn_exch{"3H2C AE4"}="H2C3 AE4"; $resn_exch{"1H3C AE4"}="H3C1 AE4";
$resn_exch{"2H3C AE4"}="H3C2 AE4"; $resn_exch{"1H5C AE4"}="H5C1 AE4"; $resn_exch{"2H5C AE4"}="H5C2 AE4";
$resn_exch{"1H6C AE4"}="H6C1 AE4"; $resn_exch{"2H6C AE4"}="H6C2 AE4"; $resn_exch{"1H8C AE4"}="H8C1 AE4";
$resn_exch{"2H8C AE4"}="H8C2 AE4"; $resn_exch{"1H9C AE4"}="H9C1 AE4"; $resn_exch{"2H9C AE4"}="H9C2 AE4";
$resn_exch{"1H11 AE4"}="H111 AE4"; $resn_exch{"2H11 AE4"}="H112 AE4"; $resn_exch{"1H12 AE4"}="H121 AE4";
$resn_exch{"2H12 AE4"}="H122 AE4"; $resn_exch{"1H14 AE4"}="H141 AE4"; $resn_exch{"2H14 AE4"}="H142 AE4";
$resn_exch{"1H15 AE4"}="H151 AE4"; $resn_exch{"2H15 AE4"}="H152 AE4"; $resn_exch{"1H17 AE4"}="H171 AE4";
$resn_exch{"2H17 AE4"}="H172 AE4"; $resn_exch{"1H18 AE4"}="H181 AE4"; $resn_exch{"2H18 AE4"}="H182 AE4";
$resn_exch{"1HN1 AEA"}="HN11 AEA"; $resn_exch{"2HN1 AEA"}="HN12 AEA"; $resn_exch{"1H1  AEA"}=" H11 AEA";
$resn_exch{"1H3  AEA"}=" H31 AEA"; $resn_exch{"2H3  AEA"}=" H32 AEA"; $resn_exch{"1HN2 AEA"}="HN21 AEA";
$resn_exch{"2HN2 AEA"}="HN22 AEA"; $resn_exch{"1H4  AEA"}=" H41 AEA"; $resn_exch{"2H4  AEA"}=" H42 AEA";
$resn_exch{"1HO2 AEA"}="HO21 AEA"; $resn_exch{"1HAQ AEE"}="HAQ1 AEE"; $resn_exch{"2HAQ AEE"}="HAQ2 AEE";
$resn_exch{"1HAO AEE"}="HAO1 AEE"; $resn_exch{"2HAO AEE"}="HAO2 AEE"; $resn_exch{"1HAN AEE"}="HAN1 AEE";
$resn_exch{"2HAN AEE"}="HAN2 AEE"; $resn_exch{"1HAA AEE"}="HAA1 AEE"; $resn_exch{"2HAA AEE"}="HAA2 AEE";
$resn_exch{"3HAA AEE"}="HAA3 AEE"; $resn_exch{"1HAP AEE"}="HAP1 AEE"; $resn_exch{"2HAP AEE"}="HAP2 AEE";
$resn_exch{"1HAR AEE"}="HAR1 AEE"; $resn_exch{"2HAR AEE"}="HAR2 AEE"; $resn_exch{"1HAS AEE"}="HAS1 AEE";
$resn_exch{"2HAS AEE"}="HAS2 AEE"; $resn_exch{"1HAB AEE"}="HAB1 AEE"; $resn_exch{"2HAB AEE"}="HAB2 AEE";
$resn_exch{"3HAB AEE"}="HAB3 AEE"; $resn_exch{"1HN  AEI"}=" H   AEI"; $resn_exch{"2HN  AEI"}=" H2  AEI";
$resn_exch{"1HG  AEI"}="HG21 AEI"; $resn_exch{"2HG  AEI"}="HG22 AEI"; $resn_exch{"3HG  AEI"}="HG23 AEI";
$resn_exch{" HOE AEI"}=" HE1 AEI"; $resn_exch{"1HE2 AEI"}="HE22 AEI"; $resn_exch{"2HE2 AEI"}="HE23 AEI";
$resn_exch{"1HN1 AEI"}="HH11 AEI"; $resn_exch{"2HN1 AEI"}="HH12 AEI"; $resn_exch{"1H27 AEJ"}="H271 AEJ";
$resn_exch{"2H27 AEJ"}="H272 AEJ"; $resn_exch{"1H30 AEJ"}="H301 AEJ"; $resn_exch{"2H30 AEJ"}="H302 AEJ";
$resn_exch{"1H8  AEJ"}=" H81 AEJ"; $resn_exch{"2H8  AEJ"}=" H82 AEJ"; $resn_exch{"1H11 AEJ"}="H111 AEJ";
$resn_exch{"2H11 AEJ"}="H112 AEJ"; $resn_exch{"1H4  AEJ"}=" H41 AEJ"; $resn_exch{"2H4  AEJ"}="H42A AEJ";
$resn_exch{"1H55 AEJ"}="H551 AEJ"; $resn_exch{"2H55 AEJ"}="H552 AEJ"; $resn_exch{"1H58 AEJ"}="H581 AEJ";
$resn_exch{"2H58 AEJ"}="H582 AEJ"; $resn_exch{"1H61 AEJ"}="H611 AEJ"; $resn_exch{"2H61 AEJ"}="H612 AEJ";
$resn_exch{"1H64 AEJ"}="H641 AEJ"; $resn_exch{"2H64 AEJ"}="H642 AEJ"; $resn_exch{"1H2C AEM"}="H2C1 AEM";
$resn_exch{"2H2C AEM"}="H2C2 AEM"; $resn_exch{"1H2N AEM"}="H2N1 AEM"; $resn_exch{"2H2N AEM"}="H2N2 AEM";
$resn_exch{"1H1' AEN"}="H1'1 AEN"; $resn_exch{"2H1' AEN"}="H1'2 AEN"; $resn_exch{"3H1' AEN"}="H1'3 AEN";
$resn_exch{"3HN' AEN"}="HN'3 AEN"; $resn_exch{"1H4' AEN"}="H4'1 AEN"; $resn_exch{"2H4' AEN"}="H4'2 AEN";
$resn_exch{"1H5' AEN"}="H5'1 AEN"; $resn_exch{"2H5' AEN"}="H5'2 AEN"; $resn_exch{"6HN' AEN"}="HN'6 AEN";
$resn_exch{"3HOS AEN"}="HOS3 AEN"; $resn_exch{"1HN  AEP"}=" HN1 AEP"; $resn_exch{"2HN  AEP"}=" HN2 AEP";
$resn_exch{"1H2  AEP"}=" H21 AEP"; $resn_exch{"2H2  AEP"}=" H22 AEP"; $resn_exch{"1H4  AEP"}=" H41 AEP";
$resn_exch{"2H4  AEP"}=" H42 AEP"; $resn_exch{"1H5  AEP"}=" H51 AEP"; $resn_exch{"2H5  AEP"}=" H52 AEP";
$resn_exch{"3H5  AEP"}=" H53 AEP"; $resn_exch{"2HOP AEP"}="HOP2 AEP"; $resn_exch{"1H7  AES"}=" H71 AES";
$resn_exch{"2H7  AES"}=" H72 AES"; $resn_exch{"1H8  AES"}=" H81 AES"; $resn_exch{"2H8  AES"}=" H82 AES";
$resn_exch{"1HN8 AES"}="HN81 AES"; $resn_exch{"2HN8 AES"}="HN82 AES"; $resn_exch{"2HOP AET"}="HOP2 AET";
$resn_exch{"3HOP AET"}="HOP3 AET"; $resn_exch{"1H5* AET"}=" H5' AET"; $resn_exch{"2H5* AET"}="H5'' AET";
$resn_exch{" H4* AET"}=" H4' AET"; $resn_exch{" H3* AET"}=" H3' AET"; $resn_exch{"*HO3 AET"}="HO3' AET";
$resn_exch{" H2* AET"}=" H2' AET"; $resn_exch{"*HO2 AET"}="HO2' AET"; $resn_exch{" H1* AET"}=" H1' AET";
$resn_exch{"1HM6 AET"}="HM61 AET"; $resn_exch{"2HM6 AET"}="HM62 AET"; $resn_exch{"3HM6 AET"}="HM63 AET";
$resn_exch{"4HO  AET"}=" HO4 AET"; $resn_exch{"1H15 AET"}="H151 AET"; $resn_exch{"2H15 AET"}="H152 AET";
$resn_exch{"3H15 AET"}="H153 AET"; $resn_exch{"1HN   AF"}=" HN1  AF"; $resn_exch{"2HN   AF"}=" HN2  AF";
$resn_exch{"1H9   AF"}=" H91  AF"; $resn_exch{"2H9   AF"}=" H92  AF"; $resn_exch{"1H6  AF1"}=" H61 AF1";
$resn_exch{"2H6  AF1"}=" H62 AF1"; $resn_exch{"3H6  AF1"}=" H63 AF1"; $resn_exch{"1H7H AF1"}="H7H1 AF1";
$resn_exch{"2H7H AF1"}="H7H2 AF1"; $resn_exch{"'HO2 AF1"}="HO2' AF1"; $resn_exch{"'HO3 AF1"}="HO3' AF1";
$resn_exch{"'HO4 AF1"}="HO4' AF1"; $resn_exch{"2H3  AF4"}=" H32 AF4"; $resn_exch{"1H3  AF4"}=" H31 AF4";
$resn_exch{"3H3  AF4"}=" H33 AF4"; $resn_exch{"2H4  AF4"}=" H42 AF4"; $resn_exch{"3H4  AF4"}=" H43 AF4";
$resn_exch{"1H4  AF4"}=" H41 AF4"; $resn_exch{"1H5  AF4"}=" H51 AF4"; $resn_exch{"2H5  AF4"}=" H52 AF4";
$resn_exch{"3H5  AF4"}=" H53 AF4"; $resn_exch{" HN  AFA"}=" H   AFA"; $resn_exch{"1HB  AFA"}=" HB2 AFA";
$resn_exch{"2HB  AFA"}=" HB3 AFA"; $resn_exch{"1HND AFA"}="HD21 AFA"; $resn_exch{"2HND AFA"}="HD22 AFA";
$resn_exch{"1H6  AFA"}=" H61 AFA"; $resn_exch{"2H6  AFA"}=" H62 AFA"; $resn_exch{"1H8  AFA"}=" H81 AFA";
$resn_exch{"2H8  AFA"}=" H82 AFA"; $resn_exch{"3H8  AFA"}=" H83 AFA"; $resn_exch{"1H9  AFA"}=" H91 AFA";
$resn_exch{"2H9  AFA"}=" H92 AFA"; $resn_exch{"3H9  AFA"}=" H93 AFA"; $resn_exch{"1H8  AFB"}=" H81 AFB";
$resn_exch{"2H8  AFB"}=" H82 AFB"; $resn_exch{"1H6  AFB"}=" H61 AFB"; $resn_exch{"2H6  AFB"}=" H62 AFB";
$resn_exch{"1H10 AFB"}="H101 AFB"; $resn_exch{"1H11 AFB"}="H111 AFB"; $resn_exch{"1H12 AFB"}="H121 AFB";
$resn_exch{"2H12 AFB"}="H122 AFB"; $resn_exch{"1H13 AFB"}="H131 AFB"; $resn_exch{"2H13 AFB"}="H132 AFB";
$resn_exch{"1H14 AFB"}="H141 AFB"; $resn_exch{"2H14 AFB"}="H142 AFB"; $resn_exch{"1H16 AFB"}="H161 AFB";
$resn_exch{"2H16 AFB"}="H162 AFB"; $resn_exch{"3H16 AFB"}="H163 AFB"; $resn_exch{"1H2  AFB"}=" H21 AFB";
$resn_exch{"1H3  AFB"}=" H31 AFB"; $resn_exch{"1HG  AFC"}=" HG1 AFC"; $resn_exch{"2HG  AFC"}=" HG2 AFC";
$resn_exch{"2HB  AFC"}=" HB2 AFC"; $resn_exch{"3HB  AFC"}=" HB3 AFC"; $resn_exch{"1HD  AFC"}=" HD1 AFC";
$resn_exch{"2HD  AFC"}=" HD2 AFC"; $resn_exch{"1HE  AFC"}=" HE1 AFC"; $resn_exch{"2HE  AFC"}=" HE2 AFC";
$resn_exch{"1HF  AFC"}=" HF1 AFC"; $resn_exch{"2HF  AFC"}=" HF2 AFC"; $resn_exch{"1HI  AFC"}=" HI1 AFC";
$resn_exch{"2HI  AFC"}=" HI2 AFC"; $resn_exch{"1HJ  AFC"}=" HJ1 AFC"; $resn_exch{"2HJ  AFC"}=" HJ2 AFC";
$resn_exch{"1HK  AFC"}=" HK1 AFC"; $resn_exch{"2HK  AFC"}=" HK2 AFC"; $resn_exch{"1HL  AFC"}=" HL1 AFC";
$resn_exch{"2HL  AFC"}=" HL2 AFC"; $resn_exch{"1HN2 AFC"}="HN21 AFC"; $resn_exch{"2HN2 AFC"}="HN22 AFC";
$resn_exch{"3HN2 AFC"}="HN23 AFC"; $resn_exch{"1HN1 AFC"}="HN11 AFC"; $resn_exch{"2HN1 AFC"}="HN12 AFC";
$resn_exch{"1HO1 AFC"}="HO11 AFC"; $resn_exch{"2HO1 AFC"}="HO12 AFC"; $resn_exch{"3HO1 AFC"}="HO13 AFC";
$resn_exch{"3HOP AFG"}="HOP3 AFG"; $resn_exch{"2HOP AFG"}="HOP2 AFG"; $resn_exch{"1HN2 AFG"}="HN21 AFG";
$resn_exch{"2HN2 AFG"}="HN22 AFG"; $resn_exch{"1H37 AFG"}="H371 AFG"; $resn_exch{"2H37 AFG"}="H372 AFG";
$resn_exch{"1H2* AFG"}=" H2' AFG"; $resn_exch{"2H2* AFG"}="H2'' AFG"; $resn_exch{"1H5* AFG"}=" H5' AFG";
$resn_exch{"2H5* AFG"}="H5'' AFG"; $resn_exch{" H4* AFG"}=" H4' AFG"; $resn_exch{" H1* AFG"}=" H1' AFG";
$resn_exch{" H3* AFG"}=" H3' AFG"; $resn_exch{"H3T  AFG"}="HO3' AFG"; $resn_exch{"1HN6 AFH"}="HN61 AFH";
$resn_exch{"2HN6 AFH"}="HN62 AFH"; $resn_exch{" H1* AFH"}=" H1D AFH"; $resn_exch{" H2* AFH"}=" H2D AFH";
$resn_exch{"HO2* AFH"}="HO2' AFH"; $resn_exch{" H3* AFH"}=" H3D AFH"; $resn_exch{" H4* AFH"}=" H4D AFH";
$resn_exch{"HO3* AFH"}="HO3A AFH"; $resn_exch{"1H5* AFH"}="H5'1 AFH"; $resn_exch{"2H5* AFH"}="H5'2 AFH";
$resn_exch{"1H7' AFH"}="H7'1 AFH"; $resn_exch{"2H7' AFH"}="H7'2 AFH"; $resn_exch{"1H11 AFI"}="H111 AFI";
$resn_exch{"2H11 AFI"}="H112 AFI"; $resn_exch{"1H12 AFI"}="H121 AFI"; $resn_exch{"2H12 AFI"}="H122 AFI";
$resn_exch{"1H14 AFI"}="H141 AFI"; $resn_exch{"2H14 AFI"}="H142 AFI"; $resn_exch{"1H15 AFI"}="H151 AFI";
$resn_exch{"2H15 AFI"}="H152 AFI"; $resn_exch{"1HA  AFL"}=" HA1 AFL"; $resn_exch{"2HB  AFL"}=" HB2 AFL";
$resn_exch{"3HC  AFL"}=" HC3 AFL"; $resn_exch{"2H8A AFN"}="H8A2 AFN"; $resn_exch{"1HM  AFN"}=" HM1 AFN";
$resn_exch{"2HM  AFN"}=" HM2 AFN"; $resn_exch{"3HM  AFN"}=" HM3 AFN"; $resn_exch{"1H3  AFN"}=" H31 AFN";
$resn_exch{"2H3  AFN"}=" H32 AFN"; $resn_exch{"1H2A AFN"}="H2A1 AFN"; $resn_exch{"2H2A AFN"}="H2A2 AFN";
$resn_exch{"2HOP AFP"}="HOP2 AFP"; $resn_exch{"3HOP AFP"}="HOP3 AFP"; $resn_exch{"1H1  AFP"}=" H11 AFP";
$resn_exch{"2H1  AFP"}=" H12 AFP"; $resn_exch{"1H6  AFP"}=" H61 AFP"; $resn_exch{"2H6  AFP"}=" H62 AFP";
$resn_exch{"5HOP AFP"}="HOP5 AFP"; $resn_exch{"6HOP AFP"}="HOP6 AFP"; $resn_exch{"1H3  AFZ"}=" H31 AFZ";
$resn_exch{"3H3  AFZ"}=" H33 AFZ"; $resn_exch{"2H3  AFZ"}=" H32 AFZ"; $resn_exch{"1H4  AFZ"}=" H41 AFZ";
$resn_exch{"1HN  AG2"}=" HN1 AG2"; $resn_exch{"2HN  AG2"}=" HN2 AG2"; $resn_exch{"1HA  AG2"}=" HA1 AG2";
$resn_exch{"2HA  AG2"}=" HA2 AG2"; $resn_exch{"1HB  AG2"}=" HB1 AG2"; $resn_exch{"2HB  AG2"}=" HB2 AG2";
$resn_exch{"1HG  AG2"}=" HG1 AG2"; $resn_exch{"2HG  AG2"}=" HG2 AG2"; $resn_exch{"1HD  AG2"}=" HD1 AG2";
$resn_exch{"2HD  AG2"}=" HD2 AG2"; $resn_exch{"1HE  AG2"}=" HE1 AG2"; $resn_exch{"1HH1 AG2"}="HH11 AG2";
$resn_exch{"1HH2 AG2"}="HH21 AG2"; $resn_exch{"2HH2 AG2"}="HH22 AG2"; $resn_exch{"1HA  AG3"}=" HA1 AG3";
$resn_exch{"2HA  AG3"}=" HA2 AG3"; $resn_exch{"1HB  AG3"}=" HB1 AG3"; $resn_exch{"2HB  AG3"}=" HB2 AG3";
$resn_exch{"1HG  AG3"}=" HG1 AG3"; $resn_exch{"2HG  AG3"}=" HG2 AG3"; $resn_exch{"1HD  AG3"}=" HD1 AG3";
$resn_exch{"2HD  AG3"}=" HD2 AG3"; $resn_exch{"1HH1 AG3"}="HH11 AG3"; $resn_exch{"2HH1 AG3"}="HH12 AG3";
$resn_exch{"1H7  AG3"}=" H71 AG3"; $resn_exch{"2H7  AG3"}=" H72 AG3"; $resn_exch{"1H8  AG3"}=" H81 AG3";
$resn_exch{"2H8  AG3"}=" H82 AG3"; $resn_exch{"1H9  AG3"}=" H91 AG3"; $resn_exch{"2H9  AG3"}=" H92 AG3";
$resn_exch{"1H10 AG3"}="H101 AG3"; $resn_exch{"2H10 AG3"}="H102 AG3"; $resn_exch{"1H19 AG7"}="H191 AG7";
$resn_exch{"2H19 AG7"}="H192 AG7"; $resn_exch{"1H1  AGA"}=" H11 AGA"; $resn_exch{"2H1  AGA"}=" H12 AGA";
$resn_exch{"1H3  AGA"}=" H31 AGA"; $resn_exch{"2H3  AGA"}=" H32 AGA"; $resn_exch{"1H4  AGA"}=" H41 AGA";
$resn_exch{"2H4  AGA"}=" H42 AGA"; $resn_exch{"1H6  AGA"}=" H61 AGA"; $resn_exch{"2H6  AGA"}=" H62 AGA";
$resn_exch{"1H8  AGA"}=" H81 AGA"; $resn_exch{"2H8  AGA"}=" H82 AGA"; $resn_exch{"1H9  AGA"}=" H91 AGA";
$resn_exch{"2H9  AGA"}=" H92 AGA"; $resn_exch{"1H10 AGA"}="H101 AGA"; $resn_exch{"2H10 AGA"}="H102 AGA";
$resn_exch{"1H11 AGA"}="H111 AGA"; $resn_exch{"2H11 AGA"}="H112 AGA"; $resn_exch{"3H11 AGA"}="H113 AGA";
$resn_exch{"1H13 AGA"}="H131 AGA"; $resn_exch{"2H13 AGA"}="H132 AGA"; $resn_exch{"1H14 AGA"}="H141 AGA";
$resn_exch{"2H14 AGA"}="H142 AGA"; $resn_exch{"1H15 AGA"}="H151 AGA"; $resn_exch{"2H15 AGA"}="H152 AGA";
$resn_exch{"1H16 AGA"}="H161 AGA"; $resn_exch{"2H16 AGA"}="H162 AGA"; $resn_exch{"1H17 AGA"}="H171 AGA";
$resn_exch{"2H17 AGA"}="H172 AGA"; $resn_exch{"1H18 AGA"}="H181 AGA"; $resn_exch{"2H18 AGA"}="H182 AGA";
$resn_exch{"1H19 AGA"}="H191 AGA"; $resn_exch{"2H19 AGA"}="H192 AGA"; $resn_exch{"3H19 AGA"}="H193 AGA";
$resn_exch{"1H2  AGB"}=" H21 AGB"; $resn_exch{"2H2  AGB"}=" H22 AGB"; $resn_exch{"1H3  AGB"}=" H31 AGB";
$resn_exch{"2H3  AGB"}=" H32 AGB"; $resn_exch{"1H4  AGB"}=" H41 AGB"; $resn_exch{"2H4  AGB"}=" H42 AGB";
$resn_exch{"1H5  AGB"}=" H51 AGB"; $resn_exch{"2H5  AGB"}=" H52 AGB"; $resn_exch{"1H8  AGB"}=" H81 AGB";
$resn_exch{"2H8  AGB"}=" H82 AGB"; $resn_exch{"1H9  AGB"}=" H91 AGB"; $resn_exch{"2H9  AGB"}=" H92 AGB";
$resn_exch{"1H11 AGB"}="H111 AGB"; $resn_exch{"2H11 AGB"}="H112 AGB"; $resn_exch{"1HN5 AGB"}="HN51 AGB";
$resn_exch{"2HN5 AGB"}="HN52 AGB"; $resn_exch{"1H6  AGC"}=" H61 AGC"; $resn_exch{"2H6  AGC"}=" H62 AGC";
$resn_exch{"1HB1 AGF"}="HB11 AGF"; $resn_exch{"2HB1 AGF"}="HB12 AGF"; $resn_exch{"1HD1 AGF"}="HD11 AGF";
$resn_exch{"2HD1 AGF"}="HD12 AGF"; $resn_exch{"1HE1 AGF"}="HE11 AGF"; $resn_exch{"2HE1 AGF"}="HE12 AGF";
$resn_exch{"1HB2 AGF"}="HB21 AGF"; $resn_exch{"2HB2 AGF"}="HB22 AGF"; $resn_exch{"3HB2 AGF"}="HB23 AGF";
$resn_exch{"1HA3 AGF"}="HA31 AGF"; $resn_exch{"2HA3 AGF"}="HA32 AGF"; $resn_exch{"2HOP AGF"}="HOP2 AGF";
$resn_exch{"1HB4 AGF"}="HB41 AGF"; $resn_exch{"2HB4 AGF"}="HB42 AGF"; $resn_exch{"1HD4 AGF"}="HD41 AGF";
$resn_exch{"2HD4 AGF"}="HD42 AGF"; $resn_exch{"1HE4 AGF"}="HE41 AGF"; $resn_exch{"2HE4 AGF"}="HE42 AGF";
$resn_exch{"1H1  AGG"}=" H11 AGG"; $resn_exch{"2H1  AGG"}=" H12 AGG"; $resn_exch{"3H1  AGG"}=" H13 AGG";
$resn_exch{"1H2  AGG"}=" H21 AGG"; $resn_exch{"2H2  AGG"}=" H22 AGG"; $resn_exch{"1H3  AGG"}=" H31 AGG";
$resn_exch{"2H3  AGG"}=" H32 AGG"; $resn_exch{"1H4  AGG"}=" H41 AGG"; $resn_exch{"2H4  AGG"}=" H42 AGG";
$resn_exch{"1HB  AGG"}=" HB1 AGG"; $resn_exch{"2HB  AGG"}=" HB2 AGG"; $resn_exch{"1H11 AGG"}="H111 AGG";
$resn_exch{"2H11 AGG"}="H112 AGG"; $resn_exch{"1H12 AGG"}="H121 AGG"; $resn_exch{"2H12 AGG"}="H122 AGG";
$resn_exch{"1H13 AGG"}="H131 AGG"; $resn_exch{"2H13 AGG"}="H132 AGG"; $resn_exch{"1H14 AGG"}="H141 AGG";
$resn_exch{"2H14 AGG"}="H142 AGG"; $resn_exch{"1H16 AGG"}="H161 AGG"; $resn_exch{"2H16 AGG"}="H162 AGG";
$resn_exch{"1H17 AGG"}="H171 AGG"; $resn_exch{"2H17 AGG"}="H172 AGG"; $resn_exch{"1H18 AGG"}="H181 AGG";
$resn_exch{"2H18 AGG"}="H182 AGG"; $resn_exch{"1H19 AGG"}="H191 AGG"; $resn_exch{"2H19 AGG"}="H192 AGG";
$resn_exch{"1H1  AGH"}=" H11 AGH"; $resn_exch{"2H1  AGH"}=" H12 AGH"; $resn_exch{"1H5  AGH"}=" H51 AGH";
$resn_exch{"2H5  AGH"}=" H52 AGH"; $resn_exch{"1H6  AGH"}=" H61 AGH"; $resn_exch{"2H6  AGH"}=" H62 AGH";
$resn_exch{"1H18 AGH"}="H181 AGH"; $resn_exch{"2H18 AGH"}="H182 AGH"; $resn_exch{"3H18 AGH"}="H183 AGH";
$resn_exch{"1H17 AGH"}="H171 AGH"; $resn_exch{"2H17 AGH"}="H172 AGH"; $resn_exch{"1H16 AGH"}="H161 AGH";
$resn_exch{"2H16 AGH"}="H162 AGH"; $resn_exch{"1H15 AGH"}="H151 AGH"; $resn_exch{"2H15 AGH"}="H152 AGH";
$resn_exch{"1H14 AGH"}="H141 AGH"; $resn_exch{"2H14 AGH"}="H142 AGH"; $resn_exch{"1H13 AGH"}="H131 AGH";
$resn_exch{"2H13 AGH"}="H132 AGH"; $resn_exch{"1H12 AGH"}="H121 AGH"; $resn_exch{"2H12 AGH"}="H122 AGH";
$resn_exch{"1H11 AGH"}="H111 AGH"; $resn_exch{"2H11 AGH"}="H112 AGH"; $resn_exch{"1H10 AGH"}="H101 AGH";
$resn_exch{"2H10 AGH"}="H102 AGH"; $resn_exch{"1H9  AGH"}=" H91 AGH"; $resn_exch{"2H9  AGH"}=" H92 AGH";
$resn_exch{"1H8  AGH"}=" H81 AGH"; $resn_exch{"2H8  AGH"}=" H82 AGH"; $resn_exch{"1H7  AGH"}=" H71 AGH";
$resn_exch{"2H7  AGH"}=" H72 AGH"; $resn_exch{"1H6A AGH"}="H6A1 AGH"; $resn_exch{"2H6A AGH"}="H6A2 AGH";
$resn_exch{"1HAB AGH"}="HAB1 AGH"; $resn_exch{"2HAB AGH"}="HAB2 AGH"; $resn_exch{"1HAC AGH"}="HAC1 AGH";
$resn_exch{"2HAC AGH"}="HAC2 AGH"; $resn_exch{"1HAD AGH"}="HAD1 AGH"; $resn_exch{"2HAD AGH"}="HAD2 AGH";
$resn_exch{"1HAE AGH"}="HAE1 AGH"; $resn_exch{"2HAE AGH"}="HAE2 AGH"; $resn_exch{"1HAF AGH"}="HAF1 AGH";
$resn_exch{"2HAF AGH"}="HAF2 AGH"; $resn_exch{"1HAG AGH"}="HAG1 AGH"; $resn_exch{"2HAG AGH"}="HAG2 AGH";
$resn_exch{"1HAH AGH"}="HAH1 AGH"; $resn_exch{"2HAH AGH"}="HAH2 AGH"; $resn_exch{"1HAI AGH"}="HAI1 AGH";
$resn_exch{"2HAI AGH"}="HAI2 AGH"; $resn_exch{"1HAJ AGH"}="HAJ1 AGH"; $resn_exch{"2HAJ AGH"}="HAJ2 AGH";
$resn_exch{"1HAK AGH"}="HAK1 AGH"; $resn_exch{"2HAK AGH"}="HAK2 AGH"; $resn_exch{"1HAL AGH"}="HAL1 AGH";
$resn_exch{"2HAL AGH"}="HAL2 AGH"; $resn_exch{"1HAM AGH"}="HAM1 AGH"; $resn_exch{"2HAM AGH"}="HAM2 AGH";
$resn_exch{"1HAN AGH"}="HAN1 AGH"; $resn_exch{"2HAN AGH"}="HAN2 AGH"; $resn_exch{"1HAO AGH"}="HAO1 AGH";
$resn_exch{"2HAO AGH"}="HAO2 AGH"; $resn_exch{"1HAP AGH"}="HAP1 AGH"; $resn_exch{"2HAP AGH"}="HAP2 AGH";
$resn_exch{"1HAQ AGH"}="HAQ1 AGH"; $resn_exch{"2HAQ AGH"}="HAQ2 AGH"; $resn_exch{"1HAR AGH"}="HAR1 AGH";
$resn_exch{"2HAR AGH"}="HAR2 AGH"; $resn_exch{"1HAS AGH"}="HAS1 AGH"; $resn_exch{"2HAS AGH"}="HAS2 AGH";
$resn_exch{"1HAT AGH"}="HAT1 AGH"; $resn_exch{"2HAT AGH"}="HAT2 AGH"; $resn_exch{"1HAU AGH"}="HAU1 AGH";
$resn_exch{"2HAU AGH"}="HAU2 AGH"; $resn_exch{"1HAV AGH"}="HAV1 AGH"; $resn_exch{"2HAV AGH"}="HAV2 AGH";
$resn_exch{"1HAW AGH"}="HAW1 AGH"; $resn_exch{"2HAW AGH"}="HAW2 AGH"; $resn_exch{"1HAX AGH"}="HAX1 AGH";
$resn_exch{"2HAX AGH"}="HAX2 AGH"; $resn_exch{"1HAY AGH"}="HAY1 AGH"; $resn_exch{"2HAY AGH"}="HAY2 AGH";
$resn_exch{"1HAZ AGH"}="HAZ1 AGH"; $resn_exch{"2HAZ AGH"}="HAZ2 AGH"; $resn_exch{"3HAZ AGH"}="HAZ3 AGH";
$resn_exch{"1H6  AGL"}=" H61 AGL"; $resn_exch{"2H6  AGL"}=" H62 AGL"; $resn_exch{"3H6  AGL"}=" H63 AGL";
$resn_exch{"1HN4 AGL"}="HN41 AGL"; $resn_exch{"2HN4 AGL"}="HN42 AGL"; $resn_exch{" HN1 AGM"}=" H   AGM";
$resn_exch{" HN2 AGM"}=" H2  AGM"; $resn_exch{" HB1 AGM"}=" HB2 AGM"; $resn_exch{" HB2 AGM"}=" HB3 AGM";
$resn_exch{" HG1 AGM"}=" HG2 AGM"; $resn_exch{" HG2 AGM"}=" HG3 AGM"; $resn_exch{"2HOG AGN"}="HOG2 AGN";
$resn_exch{"3HOG AGN"}="HOG3 AGN"; $resn_exch{"3HNB AGN"}="HNB3 AGN"; $resn_exch{"2HOB AGN"}="HOB2 AGN";
$resn_exch{"2HOA AGN"}="HOA2 AGN"; $resn_exch{"2H5* AGN"}="H5'2 AGN"; $resn_exch{"1H5* AGN"}="H5'1 AGN";
$resn_exch{" H4* AGN"}=" H4B AGN"; $resn_exch{" H3* AGN"}=" H3B AGN"; $resn_exch{"1H2* AGN"}="H2'1 AGN";
$resn_exch{"2H2* AGN"}="H2'2 AGN"; $resn_exch{" H1* AGN"}=" H1' AGN"; $resn_exch{"'HN2 AGN"}="HN2' AGN";
$resn_exch{"1HM' AGN"}="HM'1 AGN"; $resn_exch{"2HM' AGN"}="HM'2 AGN"; $resn_exch{"3HM' AGN"}="HM'3 AGN";
$resn_exch{"1HN2 AGN"}="HN21 AGN"; $resn_exch{"2HN2 AGN"}="HN22 AGN"; $resn_exch{"1H1  AGP"}=" H11 AGP";
$resn_exch{"2H1  AGP"}=" H12 AGP"; $resn_exch{"1HN2 AGP"}="HN21 AGP"; $resn_exch{"2HN2 AGP"}="HN22 AGP";
$resn_exch{"1H6  AGP"}=" H61 AGP"; $resn_exch{"2H6  AGP"}=" H62 AGP"; $resn_exch{"2HOP AGP"}="HOP2 AGP";
$resn_exch{"3HOP AGP"}="HOP3 AGP"; $resn_exch{"2HOG AGS"}="HOG2 AGS"; $resn_exch{"2HOB AGS"}="HOB2 AGS";
$resn_exch{"2HOA AGS"}="HOA2 AGS"; $resn_exch{"1H5* AGS"}="H5'1 AGS"; $resn_exch{"2H5* AGS"}="H5'2 AGS";
$resn_exch{" H4* AGS"}=" H4' AGS"; $resn_exch{" H3* AGS"}=" H3' AGS"; $resn_exch{"*HO3 AGS"}="HO3' AGS";
$resn_exch{" H2* AGS"}=" H2' AGS"; $resn_exch{"*HO2 AGS"}="HO2' AGS"; $resn_exch{" H1* AGS"}=" H1' AGS";
$resn_exch{"1HN6 AGS"}="HN61 AGS"; $resn_exch{"2HN6 AGS"}="HN62 AGS"; $resn_exch{"1HN  AGT"}=" HN1 AGT";
$resn_exch{"2HN  AGT"}=" HN2 AGT"; $resn_exch{"1HH2 AGT"}="HH21 AGT"; $resn_exch{"2HH2 AGT"}="HH22 AGT";
$resn_exch{"1HDC AGT"}="HDC1 AGT"; $resn_exch{"2HDC AGT"}="HDC2 AGT"; $resn_exch{"1HGC AGT"}="HGC1 AGT";
$resn_exch{"2HGC AGT"}="HGC2 AGT"; $resn_exch{"1HKC AGT"}="HKC1 AGT"; $resn_exch{"2HKC AGT"}="HKC2 AGT";
$resn_exch{"1HJC AGT"}="HJC1 AGT"; $resn_exch{"2HJC AGT"}="HJC2 AGT"; $resn_exch{"1HBN AGT"}="HBN1 AGT";
$resn_exch{"2HBN AGT"}="HBN2 AGT"; $resn_exch{"2HB  AGT"}=" HB2 AGT"; $resn_exch{"1HN2 AGU"}="HN21 AGU";
$resn_exch{"2HN2 AGU"}="HN22 AGU"; $resn_exch{"1HN4 AGU"}="HN41 AGU"; $resn_exch{"2HN4 AGU"}="HN42 AGU";
$resn_exch{"1H8  AH0"}=" H81 AH0"; $resn_exch{"2H8  AH0"}=" H82 AH0"; $resn_exch{"3H8  AH0"}=" H83 AH0";
$resn_exch{"1H6  AH0"}=" H61 AH0"; $resn_exch{"2H6  AH0"}=" H62 AH0"; $resn_exch{"1HB  AH0"}=" HB1 AH0";
$resn_exch{"2HB  AH0"}=" HB2 AH0"; $resn_exch{"3HB  AH0"}=" HB3 AH0"; $resn_exch{"1H08 AH1"}="H081 AH1";
$resn_exch{"2H08 AH1"}="H082 AH1"; $resn_exch{"1H16 AH1"}="H161 AH1"; $resn_exch{"2H16 AH1"}="H162 AH1";
$resn_exch{"1H23 AH1"}="H231 AH1"; $resn_exch{"2H23 AH1"}="H232 AH1"; $resn_exch{"1H33 AH1"}="H331 AH1";
$resn_exch{"2H33 AH1"}="H332 AH1"; $resn_exch{"1H2  AHA"}=" H21 AHA"; $resn_exch{"2H2  AHA"}=" H22 AHA";
$resn_exch{"1H3  AHA"}=" H31 AHA"; $resn_exch{"2H3  AHA"}=" H32 AHA"; $resn_exch{"1H4  AHA"}=" H41 AHA";
$resn_exch{"2H4  AHA"}=" H42 AHA"; $resn_exch{"1H5  AHA"}=" H51 AHA"; $resn_exch{"2H5  AHA"}=" H52 AHA";
$resn_exch{"1H6  AHA"}=" H61 AHA"; $resn_exch{"2H6  AHA"}=" H62 AHA"; $resn_exch{"1HN  AHA"}=" HN1 AHA";
$resn_exch{"2HN  AHA"}=" HN2 AHA"; $resn_exch{" HN1 AHB"}=" H   AHB"; $resn_exch{" HN2 AHB"}=" H2  AHB";
$resn_exch{"HND1 AHB"}="HD21 AHB"; $resn_exch{"HND2 AHB"}="HD22 AHB"; $resn_exch{"1HA  AHC"}=" HA1 AHC";
$resn_exch{"2HA  AHC"}=" HA2 AHC"; $resn_exch{"1HB  AHC"}=" HB1 AHC"; $resn_exch{"2HB  AHC"}=" HB2 AHC";
$resn_exch{"1HNH AHC"}="HNH1 AHC"; $resn_exch{"2HNH AHC"}="HNH2 AHC"; $resn_exch{"1H2  AHD"}=" H21 AHD";
$resn_exch{"2H2  AHD"}=" H22 AHD"; $resn_exch{"1H3  AHD"}=" H31 AHD"; $resn_exch{"2H3  AHD"}=" H32 AHD";
$resn_exch{"1H7  AHD"}=" H71 AHD"; $resn_exch{"2H7  AHD"}=" H72 AHD"; $resn_exch{"1HN4 AHD"}="HN41 AHD";
$resn_exch{"2HN4 AHD"}="HN42 AHD"; $resn_exch{"1HN1 AHE"}="HN11 AHE"; $resn_exch{"2HN1 AHE"}="HN12 AHE";
$resn_exch{"1HB1 AHE"}="HB11 AHE"; $resn_exch{"2HB1 AHE"}="HB12 AHE"; $resn_exch{"1HG1 AHE"}="HG11 AHE";
$resn_exch{"2HG1 AHE"}="HG12 AHE"; $resn_exch{"1HB2 AHE"}="HB21 AHE"; $resn_exch{"2HB2 AHE"}="HB22 AHE";
$resn_exch{"1HD2 AHE"}="HD21 AHE"; $resn_exch{"2HD2 AHE"}="HD22 AHE"; $resn_exch{"1HA3 AHE"}="HA31 AHE";
$resn_exch{"2HA3 AHE"}="HA32 AHE"; $resn_exch{"2H08 AHF"}="H082 AHF"; $resn_exch{"1H08 AHF"}="H081 AHF";
$resn_exch{"1H23 AHF"}="H231 AHF"; $resn_exch{"2H23 AHF"}="H232 AHF"; $resn_exch{"1H32 AHF"}="H321 AHF";
$resn_exch{"2H32 AHF"}="H322 AHF"; $resn_exch{"1H41 AHF"}="H411 AHF"; $resn_exch{"2H41 AHF"}="H412 AHF";
$resn_exch{"1H45 AHF"}="H451 AHF"; $resn_exch{"2H45 AHF"}="H452 AHF"; $resn_exch{"3H45 AHF"}="H453 AHF";
$resn_exch{"1H46 AHF"}="H461 AHF"; $resn_exch{"2H46 AHF"}="H462 AHF"; $resn_exch{"2HOP AHG"}="HOP2 AHG";
$resn_exch{"3HOP AHG"}="HOP3 AHG"; $resn_exch{"1H1  AHG"}=" H11 AHG"; $resn_exch{"2H1  AHG"}=" H12 AHG";
$resn_exch{"1H6  AHG"}=" H61 AHG"; $resn_exch{"2H6  AHG"}=" H62 AHG"; $resn_exch{"5HOP AHG"}="HOP5 AHG";
$resn_exch{"6HOP AHG"}="HOP6 AHG"; $resn_exch{"1HN  AHH"}=" HN1 AHH"; $resn_exch{"2HN  AHH"}=" HN2 AHH";
$resn_exch{"1HCA AHH"}="HCA1 AHH"; $resn_exch{"1HCB AHH"}="HCB1 AHH"; $resn_exch{"2HCB AHH"}="HCB2 AHH";
$resn_exch{"1HCG AHH"}="HCG1 AHH"; $resn_exch{"2HCG AHH"}="HCG2 AHH"; $resn_exch{"1HO1 AHH"}="HO11 AHH";
$resn_exch{"1HCD AHH"}="HCD1 AHH"; $resn_exch{"2HCD AHH"}="HCD2 AHH"; $resn_exch{"1HCE AHH"}="HCE1 AHH";
$resn_exch{"2HCE AHH"}="HCE2 AHH"; $resn_exch{"3HCE AHH"}="HCE3 AHH"; $resn_exch{"1HC1 AHH"}="HC11 AHH";
$resn_exch{"1HO3 AHH"}="HO31 AHH"; $resn_exch{"1H10 AHI"}="H101 AHI"; $resn_exch{"2H10 AHI"}="H102 AHI";
$resn_exch{"1H07 AHI"}="H071 AHI"; $resn_exch{"2H07 AHI"}="H072 AHI"; $resn_exch{"1H06 AHI"}="H061 AHI";
$resn_exch{"2H06 AHI"}="H062 AHI"; $resn_exch{"1H03 AHI"}="H031 AHI"; $resn_exch{"2H03 AHI"}="H032 AHI";
$resn_exch{"1H02 AHI"}="H021 AHI"; $resn_exch{"2H02 AHI"}="H022 AHI"; $resn_exch{"3H02 AHI"}="H023 AHI";
$resn_exch{"1HN1 AHL"}="HN11 AHL"; $resn_exch{"2HN1 AHL"}="HN12 AHL"; $resn_exch{"1HB  AHL"}=" HB1 AHL";
$resn_exch{"2HB  AHL"}=" HB2 AHL"; $resn_exch{"1HG  AHL"}=" HG1 AHL"; $resn_exch{"2HG  AHL"}=" HG2 AHL";
$resn_exch{"1HD  AHL"}=" HD1 AHL"; $resn_exch{"2HD  AHL"}=" HD2 AHL"; $resn_exch{"1HH1 AHL"}="HH11 AHL";
$resn_exch{"2HH1 AHL"}="HH12 AHL"; $resn_exch{"2HOP AHM"}="HOP2 AHM"; $resn_exch{"3HOP AHM"}="HOP3 AHM";
$resn_exch{"1H1  AHM"}=" H11 AHM"; $resn_exch{"2H1  AHM"}=" H12 AHM"; $resn_exch{"1H6  AHM"}=" H61 AHM";
$resn_exch{"2H6  AHM"}=" H62 AHM"; $resn_exch{"5HOP AHM"}="HOP5 AHM"; $resn_exch{"6HOP AHM"}="HOP6 AHM";
$resn_exch{"1HT  AHN"}=" HT1 AHN"; $resn_exch{"2HT  AHN"}=" HT2 AHN"; $resn_exch{"3HT  AHN"}=" HT3 AHN";
$resn_exch{"1HA  AHN"}=" HA1 AHN"; $resn_exch{"2HA  AHN"}=" HA2 AHN"; $resn_exch{"1HB  AHN"}=" HB1 AHN";
$resn_exch{"2HB  AHN"}=" HB2 AHN"; $resn_exch{"1HC2 AHO"}="HC21 AHO"; $resn_exch{"1HC3 AHO"}="HC31 AHO";
$resn_exch{"2HC3 AHO"}="HC32 AHO"; $resn_exch{"1HC4 AHO"}="HC41 AHO"; $resn_exch{"2HC4 AHO"}="HC42 AHO";
$resn_exch{"1HC5 AHO"}="HC51 AHO"; $resn_exch{"2HC5 AHO"}="HC52 AHO"; $resn_exch{"1HC7 AHO"}="HC71 AHO";
$resn_exch{"2HC7 AHO"}="HC72 AHO"; $resn_exch{"3HC7 AHO"}="HC73 AHO"; $resn_exch{"1HN1 AHO"}="HN11 AHO";
$resn_exch{"2HN1 AHO"}="HN12 AHO"; $resn_exch{"1HO2 AHO"}="HO21 AHO"; $resn_exch{"1HN  AHP"}=" H   AHP";
$resn_exch{"2HN  AHP"}=" H2  AHP"; $resn_exch{"1HB  AHP"}=" HB2 AHP"; $resn_exch{"2HB  AHP"}=" HB3 AHP";
$resn_exch{"1HG  AHP"}=" HG2 AHP"; $resn_exch{"2HG  AHP"}=" HG3 AHP"; $resn_exch{"1HD  AHP"}=" HD2 AHP";
$resn_exch{"2HD  AHP"}=" HD3 AHP"; $resn_exch{"1HE  AHP"}=" HE2 AHP"; $resn_exch{"2HE  AHP"}=" HE3 AHP";
$resn_exch{"1HZ  AHP"}=" HZ1 AHP"; $resn_exch{"2HZ  AHP"}=" HZ2 AHP"; $resn_exch{"3HZ  AHP"}=" HZ3 AHP";
$resn_exch{" H5* AHR"}=" H5' AHR"; $resn_exch{"1H5* AHR"}="H5'1 AHR"; $resn_exch{"2H5* AHR"}="H5'2 AHR";
$resn_exch{" H4* AHR"}=" H4' AHR"; $resn_exch{" H3* AHR"}=" H3' AHR"; $resn_exch{" H2* AHR"}=" H2' AHR";
$resn_exch{" H1* AHR"}=" H1' AHR"; $resn_exch{"1HN  AHS"}=" H   AHS"; $resn_exch{"2HN  AHS"}=" H2  AHS";
$resn_exch{"1HB  AHS"}=" HB2 AHS"; $resn_exch{"2HB  AHS"}=" HB3 AHS"; $resn_exch{"1HD1 AHS"}="HD12 AHS";
$resn_exch{"2HD1 AHS"}="HD13 AHS"; $resn_exch{"1HD2 AHS"}="HD23 AHS"; $resn_exch{"2HD2 AHS"}="HD22 AHS";
$resn_exch{"1HE1 AHS"}="HE12 AHS"; $resn_exch{"2HE1 AHS"}="HE13 AHS"; $resn_exch{"1HE2 AHS"}="HE23 AHS";
$resn_exch{"2HE2 AHS"}="HE22 AHS"; $resn_exch{"1HZ  AHS"}=" HZ2 AHS"; $resn_exch{"2HZ  AHS"}=" HZ3 AHS";
$resn_exch{"1HM  AHS"}=" HM1 AHS"; $resn_exch{"2HM  AHS"}=" HM2 AHS"; $resn_exch{"1HB2 AHS"}="HB21 AHS";
$resn_exch{"2HB2 AHS"}="HB22 AHS"; $resn_exch{"1HD3 AHS"}="HD31 AHS"; $resn_exch{"2HD3 AHS"}="HD32 AHS";
$resn_exch{"3HD3 AHS"}="HD33 AHS"; $resn_exch{"1HD4 AHS"}="HD41 AHS"; $resn_exch{"2HD4 AHS"}="HD42 AHS";
$resn_exch{"3HD4 AHS"}="HD43 AHS"; $resn_exch{"1HN  AHT"}=" H   AHT"; $resn_exch{"2HN  AHT"}=" H2  AHT";
$resn_exch{"1HC  AHT"}=" HC1 AHT"; $resn_exch{"2HC  AHT"}=" HC2 AHT"; $resn_exch{"1HB  AHT"}=" HB2 AHT";
$resn_exch{"2HB  AHT"}=" HB3 AHT"; $resn_exch{" HO  AHT"}=" HH  AHT"; $resn_exch{"1H1  AHT"}=" H11 AHT";
$resn_exch{"2H1  AHT"}=" H12 AHT"; $resn_exch{"H2   AHT"}="HC22 AHT"; $resn_exch{"1HN2 AHT"}="HN21 AHT";
$resn_exch{"2HN2 AHT"}="HN22 AHT"; $resn_exch{" H4* AHU"}=" H4' AHU"; $resn_exch{"1H7* AHU"}="H7'1 AHU";
$resn_exch{"2H7* AHU"}="H7'2 AHU"; $resn_exch{"7HO* AHU"}="HO'7 AHU"; $resn_exch{"1H6* AHU"}="H6'1 AHU";
$resn_exch{"2H6* AHU"}="H6'2 AHU"; $resn_exch{" H1* AHU"}=" H1' AHU"; $resn_exch{"1H2* AHU"}="H2'1 AHU";
$resn_exch{"2H2* AHU"}="H2'2 AHU"; $resn_exch{" H3* AHU"}=" H3' AHU"; $resn_exch{"3HO* AHU"}="HO'3 AHU";
$resn_exch{"1HN1 AHX"}="HN11 AHX"; $resn_exch{"2HN1 AHX"}="HN12 AHX"; $resn_exch{"1HB  AHX"}=" HB1 AHX";
$resn_exch{"2HB  AHX"}=" HB2 AHX"; $resn_exch{"2HOA AHX"}="HOA2 AHX"; $resn_exch{"1H5* AHX"}="H5'1 AHX";
$resn_exch{"2H5* AHX"}="H5'2 AHX"; $resn_exch{" H4* AHX"}=" H4' AHX"; $resn_exch{" H3* AHX"}=" H3' AHX";
$resn_exch{"*HO3 AHX"}="HO3' AHX"; $resn_exch{" H2* AHX"}=" H2' AHX"; $resn_exch{"*HO2 AHX"}="HO2' AHX";
$resn_exch{" H1* AHX"}=" H1' AHX"; $resn_exch{"1HN6 AHX"}="HN61 AHX"; $resn_exch{"2HN6 AHX"}="HN62 AHX";
$resn_exch{"1H10 AHY"}="H101 AHY"; $resn_exch{"2H10 AHY"}="H102 AHY"; $resn_exch{"3H10 AHY"}="H103 AHY";
$resn_exch{"1H9C AHY"}="H9C1 AHY"; $resn_exch{"2H9C AHY"}="H9C2 AHY"; $resn_exch{"1H8C AHY"}="H8C1 AHY";
$resn_exch{"2H8C AHY"}="H8C2 AHY"; $resn_exch{"1H7C AHY"}="H7C1 AHY"; $resn_exch{"2H7C AHY"}="H7C2 AHY";
$resn_exch{"1H6C AHY"}="H6C1 AHY"; $resn_exch{"2H6C AHY"}="H6C2 AHY"; $resn_exch{"1H5C AHY"}="H5C1 AHY";
$resn_exch{"2H5C AHY"}="H5C2 AHY"; $resn_exch{"1H4C AHY"}="H4C1 AHY"; $resn_exch{"2H4C AHY"}="H4C2 AHY";
$resn_exch{"1H3N AHY"}="H3N1 AHY"; $resn_exch{"2H3N AHY"}="H3N2 AHY"; $resn_exch{" H2* AHZ"}=" H2' AHZ";
$resn_exch{" H3* AHZ"}=" H3' AHZ"; $resn_exch{" H1* AHZ"}=" H1' AHZ"; $resn_exch{"1HN6 AHZ"}="HN61 AHZ";
$resn_exch{"2HN6 AHZ"}="HN62 AHZ"; $resn_exch{" H4* AHZ"}=" H4' AHZ"; $resn_exch{"1H5* AHZ"}="H5'1 AHZ";
$resn_exch{"2H5* AHZ"}="H5'2 AHZ"; $resn_exch{"1H7T AHZ"}="H7T1 AHZ"; $resn_exch{"2H7T AHZ"}="H7T2 AHZ";
$resn_exch{"1H6T AHZ"}="H6T1 AHZ"; $resn_exch{"2H6T AHZ"}="H6T2 AHZ"; $resn_exch{"1H1M AHZ"}="H1M1 AHZ";
$resn_exch{"2H1M AHZ"}="H1M2 AHZ"; $resn_exch{"3H1M AHZ"}="H1M3 AHZ"; $resn_exch{"1H11  AI"}="H111  AI";
$resn_exch{"2H11  AI"}="H112  AI"; $resn_exch{"3H11  AI"}="H113  AI"; $resn_exch{"1H8   AI"}=" H81  AI";
$resn_exch{"2H8   AI"}=" H82  AI"; $resn_exch{"3H8   AI"}=" H83  AI"; $resn_exch{"1H12  AI"}="H121  AI";
$resn_exch{"2H12  AI"}="H122  AI"; $resn_exch{"3H12  AI"}="H123  AI"; $resn_exch{"1H2   AI"}="H21A  AI";
$resn_exch{"2H2   AI"}="H22A  AI"; $resn_exch{"1H15  AI"}="H151  AI"; $resn_exch{"2H15  AI"}="H152  AI";
$resn_exch{"1H25  AI"}="H251  AI"; $resn_exch{"2H25  AI"}="H252  AI"; $resn_exch{"1H27  AI"}="H271  AI";
$resn_exch{"2H27  AI"}="H272  AI"; $resn_exch{"1H28  AI"}="H281  AI"; $resn_exch{"2H28  AI"}="H282  AI";
$resn_exch{"1H29  AI"}="H291  AI"; $resn_exch{"2H29  AI"}="H292  AI"; $resn_exch{"1H30  AI"}="H301  AI";
$resn_exch{"2H30  AI"}="H302  AI"; $resn_exch{"1H31  AI"}="H311  AI"; $resn_exch{"2H31  AI"}="H312  AI";
$resn_exch{"1H35  AI"}="H351  AI"; $resn_exch{"2H35  AI"}="H352  AI"; $resn_exch{"3H35  AI"}="H353  AI";
$resn_exch{"1H36  AI"}="H361  AI"; $resn_exch{"2H36  AI"}="H362  AI"; $resn_exch{"3H36  AI"}="H363  AI";
$resn_exch{"1H38  AI"}="H381  AI"; $resn_exch{"2H38  AI"}="H382  AI"; $resn_exch{"3H38  AI"}="H383  AI";
$resn_exch{"1H39  AI"}="H391  AI"; $resn_exch{"2H39  AI"}="H392  AI"; $resn_exch{"1HC6 AI1"}="HC61 AI1";
$resn_exch{"2HC6 AI1"}="HC62 AI1"; $resn_exch{"1HN' AI1"}="HN'1 AI1"; $resn_exch{"1H1* AI1"}="H1'1 AI1";
$resn_exch{"2H1* AI1"}="H1'2 AI1"; $resn_exch{" H3* AI1"}=" H3B AI1"; $resn_exch{" H4* AI1"}=" H4B AI1";
$resn_exch{" H5* AI1"}=" H5' AI1"; $resn_exch{" H6* AI1"}=" H6B AI1"; $resn_exch{" H7* AI1"}=" H7' AI1";
$resn_exch{"1H11 AI2"}="H111 AI2"; $resn_exch{"2H11 AI2"}="H112 AI2"; $resn_exch{"3H11 AI2"}="H113 AI2";
$resn_exch{"1H6  AI2"}=" H61 AI2"; $resn_exch{"2H6  AI2"}=" H62 AI2"; $resn_exch{"1H22 AI3"}="H221 AI3";
$resn_exch{"2H22 AI3"}="H222 AI3"; $resn_exch{"1H39 AI3"}="H391 AI3"; $resn_exch{"2H39 AI3"}="H392 AI3";
$resn_exch{"1H47 AI3"}="H471 AI3"; $resn_exch{"2H47 AI3"}="H472 AI3"; $resn_exch{"3H47 AI3"}="H473 AI3";
$resn_exch{"1H51 AI3"}="H511 AI3"; $resn_exch{"2H51 AI3"}="H512 AI3"; $resn_exch{"3H51 AI3"}="H513 AI3";
$resn_exch{"1H4  AI7"}=" H41 AI7"; $resn_exch{"2H4  AI7"}=" H42 AI7"; $resn_exch{"1H6  AI7"}=" H61 AI7";
$resn_exch{"2H6  AI7"}=" H62 AI7"; $resn_exch{"1H8  AI7"}=" H81 AI7"; $resn_exch{"2H8  AI7"}=" H82 AI7";
$resn_exch{"1H9  AI7"}=" H91 AI7"; $resn_exch{"2H9  AI7"}=" H92 AI7"; $resn_exch{"3H9  AI7"}=" H93 AI7";
$resn_exch{"1H10 AI7"}="H101 AI7"; $resn_exch{"2H10 AI7"}="H102 AI7"; $resn_exch{"1H12 AI7"}="H121 AI7";
$resn_exch{"2H12 AI7"}="H122 AI7"; $resn_exch{"1H13 AI7"}="H131 AI7"; $resn_exch{"2H13 AI7"}="H132 AI7";
$resn_exch{"1H5  AIA"}=" H51 AIA"; $resn_exch{"2H5  AIA"}=" H52 AIA"; $resn_exch{"1H7  AIA"}=" H71 AIA";
$resn_exch{"2H7  AIA"}=" H72 AIA"; $resn_exch{"1H8  AIA"}=" H81 AIA"; $resn_exch{"2H8  AIA"}=" H82 AIA";
$resn_exch{"1H12 AIA"}="H121 AIA"; $resn_exch{"2H12 AIA"}="H122 AIA"; $resn_exch{"1H14 AIA"}="H141 AIA";
$resn_exch{"2H14 AIA"}="H142 AIA"; $resn_exch{"1H16 AIA"}="H161 AIA"; $resn_exch{"2H16 AIA"}="H162 AIA";
$resn_exch{"1H17 AIA"}="H171 AIA"; $resn_exch{"2H17 AIA"}="H172 AIA"; $resn_exch{"1H18 AIA"}="H181 AIA";
$resn_exch{"2H18 AIA"}="H182 AIA"; $resn_exch{"1H19 AIA"}="H191 AIA"; $resn_exch{"2H19 AIA"}="H192 AIA";
$resn_exch{"1HN  AIB"}=" H   AIB"; $resn_exch{"2HN  AIB"}=" H2  AIB"; $resn_exch{"1HB1 AIB"}="HB11 AIB";
$resn_exch{"2HB1 AIB"}="HB12 AIB"; $resn_exch{"3HB1 AIB"}="HB13 AIB"; $resn_exch{"1HB2 AIB"}="HB21 AIB";
$resn_exch{"2HB2 AIB"}="HB22 AIB"; $resn_exch{"3HB2 AIB"}="HB23 AIB"; $resn_exch{"8HC  AIC"}=" HC8 AIC";
$resn_exch{"1HN  AIC"}=" HN1 AIC"; $resn_exch{"4HC  AIC"}=" HC4 AIC"; $resn_exch{"1HC  AIC"}=" HC1 AIC";
$resn_exch{"2HC  AIC"}=" HC2 AIC"; $resn_exch{"3HC  AIC"}=" HC3 AIC"; $resn_exch{"7HC  AIC"}=" HC7 AIC";
$resn_exch{"9HC  AIC"}=" HC9 AIC"; $resn_exch{"1HC0 AIC"}="HC01 AIC"; $resn_exch{"1HC1 AIC"}="HC11 AIC";
$resn_exch{"1HC2 AIC"}="HC21 AIC"; $resn_exch{"1HC3 AIC"}="HC31 AIC"; $resn_exch{"1HC4 AIC"}="HC41 AIC";
$resn_exch{"1H16 AIC"}="H161 AIC"; $resn_exch{"2H16 AIC"}="H162 AIC"; $resn_exch{"3H16 AIC"}="H163 AIC";
$resn_exch{"1HN2 AIC"}="HN21 AIC"; $resn_exch{"2HN2 AIC"}="HN22 AIC"; $resn_exch{"2HO  AIC"}=" HO2 AIC";
$resn_exch{"1H6  AIG"}=" H61 AIG"; $resn_exch{"2H6  AIG"}=" H62 AIG"; $resn_exch{"1HN3 AIG"}="HN31 AIG";
$resn_exch{"2HN3 AIG"}="HN32 AIG"; $resn_exch{"1H1' AIG"}="H1'1 AIG"; $resn_exch{"2H1' AIG"}="H1'2 AIG";
$resn_exch{"1H2' AIG"}="H2'1 AIG"; $resn_exch{"2H2' AIG"}="H2'2 AIG"; $resn_exch{"1H3' AIG"}="H3'1 AIG";
$resn_exch{"2H3' AIG"}="H3'2 AIG"; $resn_exch{"1H4' AIG"}="H4'1 AIG"; $resn_exch{"2H4' AIG"}="H4'2 AIG";
$resn_exch{"1H5' AIG"}="H5'1 AIG"; $resn_exch{"2H5' AIG"}="H5'2 AIG"; $resn_exch{"1H6' AIG"}="H6'1 AIG";
$resn_exch{"2H6' AIG"}="H6'2 AIG"; $resn_exch{"3H6' AIG"}="H6'3 AIG"; $resn_exch{"1H26 AIH"}="H261 AIH";
$resn_exch{"2H26 AIH"}="H262 AIH"; $resn_exch{"1H27 AIH"}="H271 AIH"; $resn_exch{"2H27 AIH"}="H272 AIH";
$resn_exch{"1H29 AIH"}="H291 AIH"; $resn_exch{"2H29 AIH"}="H292 AIH"; $resn_exch{"1H31 AIH"}="H311 AIH";
$resn_exch{"2H31 AIH"}="H312 AIH"; $resn_exch{"3H31 AIH"}="H313 AIH"; $resn_exch{"1H33 AIH"}="H331 AIH";
$resn_exch{"2H33 AIH"}="H332 AIH"; $resn_exch{"3H33 AIH"}="H333 AIH"; $resn_exch{"1H34 AIH"}="H341 AIH";
$resn_exch{"2H34 AIH"}="H342 AIH"; $resn_exch{"1H26 AIJ"}="H261 AIJ"; $resn_exch{"2H26 AIJ"}="H262 AIJ";
$resn_exch{"1H28 AIJ"}="H281 AIJ"; $resn_exch{"2H28 AIJ"}="H282 AIJ"; $resn_exch{"3H28 AIJ"}="H283 AIJ";
$resn_exch{"1H30 AIJ"}="H301 AIJ"; $resn_exch{"2H30 AIJ"}="H302 AIJ"; $resn_exch{"1H31 AIJ"}="H311 AIJ";
$resn_exch{"2H31 AIJ"}="H312 AIJ"; $resn_exch{"1H32 AIJ"}="H321 AIJ"; $resn_exch{"2H32 AIJ"}="H322 AIJ";
$resn_exch{"1H33 AIJ"}="H331 AIJ"; $resn_exch{"2H33 AIJ"}="H332 AIJ"; $resn_exch{"1H5  AIK"}=" H51 AIK";
$resn_exch{"2H5  AIK"}=" H52 AIK"; $resn_exch{"3H5  AIK"}=" H53 AIK"; $resn_exch{"1H12 AIK"}="H121 AIK";
$resn_exch{"2H12 AIK"}="H122 AIK"; $resn_exch{"3H12 AIK"}="H123 AIK"; $resn_exch{"1H27 AIK"}="H271 AIK";
$resn_exch{"2H27 AIK"}="H272 AIK"; $resn_exch{"1H28 AIK"}="H281 AIK"; $resn_exch{"2H28 AIK"}="H282 AIK";
$resn_exch{"1H29 AIK"}="H291 AIK"; $resn_exch{"2H29 AIK"}="H292 AIK"; $resn_exch{"1H32 AIK"}="H321 AIK";
$resn_exch{"2H32 AIK"}="H322 AIK"; $resn_exch{"3H32 AIK"}="H323 AIK"; $resn_exch{"1H31 AIK"}="H311 AIK";
$resn_exch{"2H31 AIK"}="H312 AIK"; $resn_exch{"3H31 AIK"}="H313 AIK"; $resn_exch{"1H23 AIK"}="H231 AIK";
$resn_exch{"2H23 AIK"}="H232 AIK"; $resn_exch{"3H23 AIK"}="H233 AIK"; $resn_exch{"1H24 AIK"}="H241 AIK";
$resn_exch{"2H24 AIK"}="H242 AIK"; $resn_exch{"3H24 AIK"}="H243 AIK"; $resn_exch{"1H1  AIL"}=" H11 AIL";
$resn_exch{"2H1  AIL"}=" H12 AIL"; $resn_exch{"1H4  AIL"}=" H41 AIL"; $resn_exch{"2H4  AIL"}=" H42 AIL";
$resn_exch{"1H6  AIL"}=" H61 AIL"; $resn_exch{"2H6  AIL"}=" H62 AIL"; $resn_exch{"1H10 AIL"}="H101 AIL";
$resn_exch{"2H10 AIL"}="H102 AIL"; $resn_exch{"1H11 AIL"}="H111 AIL"; $resn_exch{"2H11 AIL"}="H112 AIL";
$resn_exch{"1H13 AIL"}="H131 AIL"; $resn_exch{"2H13 AIL"}="H132 AIL"; $resn_exch{"1HC1 AIM"}="HC11 AIM";
$resn_exch{"2HC1 AIM"}="HC12 AIM"; $resn_exch{"1HC2 AIM"}="HC21 AIM"; $resn_exch{"1HC3 AIM"}="HC31 AIM";
$resn_exch{"2HC3 AIM"}="HC32 AIM"; $resn_exch{"1HC4 AIM"}="HC41 AIM"; $resn_exch{"2HC4 AIM"}="HC42 AIM";
$resn_exch{"1HC6 AIM"}="HC61 AIM"; $resn_exch{"2HC6 AIM"}="HC62 AIM"; $resn_exch{"1HN1 AIM"}="HN11 AIM";
$resn_exch{"2HN1 AIM"}="HN12 AIM"; $resn_exch{"1H26 AIM"}="H261 AIM"; $resn_exch{"1HN2 AIM"}="HN21 AIM";
$resn_exch{"2HN2 AIM"}="HN22 AIM"; $resn_exch{"1H9  AIN"}=" H91 AIN"; $resn_exch{"2H9  AIN"}=" H92 AIN";
$resn_exch{"3H9  AIN"}=" H93 AIN"; $resn_exch{"1H7  AIO"}=" H71 AIO"; $resn_exch{"2H7  AIO"}=" H72 AIO";
$resn_exch{"1H8  AIO"}=" H81 AIO"; $resn_exch{"2H8  AIO"}=" H82 AIO"; $resn_exch{"3H8  AIO"}=" H83 AIO";
$resn_exch{"1H6  AIO"}=" H61 AIO"; $resn_exch{"2H6  AIO"}=" H62 AIO"; $resn_exch{"3H6  AIO"}=" H63 AIO";
$resn_exch{"1H2  AIO"}=" H21 AIO"; $resn_exch{"2H2  AIO"}=" H22 AIO"; $resn_exch{"3H2  AIO"}=" H23 AIO";
$resn_exch{"1HN1 AIO"}="HN11 AIO"; $resn_exch{"2HN1 AIO"}="HN12 AIO"; $resn_exch{"1H13 AIO"}="H131 AIO";
$resn_exch{"2H13 AIO"}="H132 AIO"; $resn_exch{"1H12 AIO"}="H121 AIO"; $resn_exch{"2H12 AIO"}="H122 AIO";
$resn_exch{"1H11 AIO"}="H111 AIO"; $resn_exch{"2H11 AIO"}="H112 AIO"; $resn_exch{"1H5  AIP"}=" H51 AIP";
$resn_exch{"2H5  AIP"}=" H52 AIP"; $resn_exch{"1H6  AIP"}=" H61 AIP"; $resn_exch{"2H6  AIP"}=" H62 AIP";
$resn_exch{"1H7  AIP"}=" H71 AIP"; $resn_exch{"2H7  AIP"}=" H72 AIP"; $resn_exch{"1H13 AIP"}="H131 AIP";
$resn_exch{"2H13 AIP"}="H132 AIP"; $resn_exch{"3H13 AIP"}="H133 AIP"; $resn_exch{"1H14 AIP"}="H141 AIP";
$resn_exch{"2H14 AIP"}="H142 AIP"; $resn_exch{"3H14 AIP"}="H143 AIP"; $resn_exch{"1H18 AIP"}="H181 AIP";
$resn_exch{"2H18 AIP"}="H182 AIP"; $resn_exch{"1H19 AIP"}="H191 AIP"; $resn_exch{"2H19 AIP"}="H192 AIP";
$resn_exch{"1H20 AIP"}="H201 AIP"; $resn_exch{"2H20 AIP"}="H202 AIP"; $resn_exch{"1H25 AIP"}="H251 AIP";
$resn_exch{"2H25 AIP"}="H252 AIP"; $resn_exch{"1HN4 AIP"}="HN41 AIP"; $resn_exch{"2HN4 AIP"}="HN42 AIP";
$resn_exch{"1HN9 AIP"}="HN91 AIP"; $resn_exch{"2HN9 AIP"}="HN92 AIP"; $resn_exch{"1H1  AIQ"}=" H11 AIQ";
$resn_exch{"1H9  AIQ"}=" H91 AIQ"; $resn_exch{"2H9  AIQ"}=" H92 AIQ"; $resn_exch{"1HN2 AIQ"}="HN21 AIQ";
$resn_exch{"2HN2 AIQ"}="HN22 AIQ"; $resn_exch{"1HN1 AIQ"}="HN11 AIQ"; $resn_exch{"1H5  AIQ"}=" H51 AIQ";
$resn_exch{"1HN4 AIQ"}="HN41 AIQ"; $resn_exch{"2HN4 AIQ"}="HN42 AIQ"; $resn_exch{"1HN6 AIQ"}="HN61 AIQ";
$resn_exch{"1H11 AIQ"}="H111 AIQ"; $resn_exch{"1H12 AIQ"}="H121 AIQ"; $resn_exch{"1HN6 AIR"}="HN61 AIR";
$resn_exch{"2HN6 AIR"}="HN62 AIR"; $resn_exch{"2HO' AIR"}="HO'2 AIR"; $resn_exch{"3HO' AIR"}="HO'3 AIR";
$resn_exch{"1HC5 AIR"}="HC51 AIR"; $resn_exch{"2HC5 AIR"}="HC52 AIR"; $resn_exch{" H2* AIS"}=" H2' AIS";
$resn_exch{" H3* AIS"}=" H3' AIS"; $resn_exch{" H4* AIS"}=" H4' AIS"; $resn_exch{"1H5* AIS"}="H5'1 AIS";
$resn_exch{"2H5* AIS"}="H5'2 AIS"; $resn_exch{" H5* AIS"}=" H5' AIS"; $resn_exch{" H1* AIS"}=" H1' AIS";
$resn_exch{"1HN6 AIS"}="HN61 AIS"; $resn_exch{"2HN6 AIS"}="HN62 AIS"; $resn_exch{"1H26 AIT"}="H261 AIT";
$resn_exch{"2H26 AIT"}="H262 AIT"; $resn_exch{"1H28 AIT"}="H281 AIT"; $resn_exch{"2H28 AIT"}="H282 AIT";
$resn_exch{"3H28 AIT"}="H283 AIT"; $resn_exch{"1H30 AIT"}="H301 AIT"; $resn_exch{"2H30 AIT"}="H302 AIT";
$resn_exch{"1H31 AIT"}="H311 AIT"; $resn_exch{"2H31 AIT"}="H312 AIT"; $resn_exch{"1H32 AIT"}="H321 AIT";
$resn_exch{"2H32 AIT"}="H322 AIT"; $resn_exch{"1H33 AIT"}="H331 AIT"; $resn_exch{"2H33 AIT"}="H332 AIT";
$resn_exch{"1H26 AIU"}="H261 AIU"; $resn_exch{"2H26 AIU"}="H262 AIU"; $resn_exch{"1H27 AIU"}="H271 AIU";
$resn_exch{"2H27 AIU"}="H272 AIU"; $resn_exch{"1H29 AIU"}="H291 AIU"; $resn_exch{"2H29 AIU"}="H292 AIU";
$resn_exch{"1H31 AIU"}="H311 AIU"; $resn_exch{"2H31 AIU"}="H312 AIU"; $resn_exch{"3H31 AIU"}="H313 AIU";
$resn_exch{"1H33 AIU"}="H331 AIU"; $resn_exch{"2H33 AIU"}="H332 AIU"; $resn_exch{"3H33 AIU"}="H333 AIU";
$resn_exch{"1H34 AIU"}="H341 AIU"; $resn_exch{"2H34 AIU"}="H342 AIU"; $resn_exch{"1H22 AIZ"}="H221 AIZ";
$resn_exch{"2H22 AIZ"}="H222 AIZ"; $resn_exch{"1H26 AIZ"}="H261 AIZ"; $resn_exch{"2H26 AIZ"}="H262 AIZ";
$resn_exch{"1H30 AIZ"}="H301 AIZ"; $resn_exch{"2H30 AIZ"}="H302 AIZ"; $resn_exch{"1H33 AIZ"}="H331 AIZ";
$resn_exch{"2H33 AIZ"}="H332 AIZ"; $resn_exch{"3H33 AIZ"}="H333 AIZ"; $resn_exch{"1H34 AIZ"}="H341 AIZ";
$resn_exch{"2H34 AIZ"}="H342 AIZ"; $resn_exch{"3H34 AIZ"}="H343 AIZ"; $resn_exch{"1H8B AJ3"}="H8B1 AJ3";
$resn_exch{"2H8B AJ3"}="H8B2 AJ3"; $resn_exch{"1H10 AJ3"}="H101 AJ3"; $resn_exch{"2H10 AJ3"}="H102 AJ3";
$resn_exch{"1H12 AJ3"}="H121 AJ3"; $resn_exch{"2H12 AJ3"}="H122 AJ3"; $resn_exch{"1H9  AJA"}=" H91 AJA";
$resn_exch{"2H9  AJA"}=" H92 AJA"; $resn_exch{"1H25 AJA"}="H251 AJA"; $resn_exch{"2H25 AJA"}="H252 AJA";
$resn_exch{"3H25 AJA"}="H253 AJA"; $resn_exch{"1H20 AJA"}="H201 AJA"; $resn_exch{"2H20 AJA"}="H202 AJA";
$resn_exch{"1H21 AJA"}="H211 AJA"; $resn_exch{"2H21 AJA"}="H212 AJA"; $resn_exch{"1H22 AJA"}="H221 AJA";
$resn_exch{"2H22 AJA"}="H222 AJA"; $resn_exch{"1H23 AJA"}="H231 AJA"; $resn_exch{"2H23 AJA"}="H232 AJA";
$resn_exch{"1H28 AJA"}="H281 AJA"; $resn_exch{"2H28 AJA"}="H282 AJA"; $resn_exch{"1H29 AJA"}="H291 AJA";
$resn_exch{"2H29 AJA"}="H292 AJA"; $resn_exch{"3H29 AJA"}="H293 AJA"; $resn_exch{"1H24 AJA"}="H241 AJA";
$resn_exch{"2H24 AJA"}="H242 AJA"; $resn_exch{"3H24 AJA"}="H243 AJA"; $resn_exch{"1H18 AJA"}="H181 AJA";
$resn_exch{"2H18 AJA"}="H182 AJA"; $resn_exch{"3H18 AJA"}="H183 AJA"; $resn_exch{"1H17 AJA"}="H171 AJA";
$resn_exch{"2H17 AJA"}="H172 AJA"; $resn_exch{"3H17 AJA"}="H173 AJA"; $resn_exch{"1H12 AJA"}="H121 AJA";
$resn_exch{"2H12 AJA"}="H122 AJA"; $resn_exch{"1H4  AJM"}=" H41 AJM"; $resn_exch{"2H4  AJM"}=" H42 AJM";
$resn_exch{"1H16 AJM"}="H161 AJM"; $resn_exch{"2H16 AJM"}="H162 AJM"; $resn_exch{"1H12 AJM"}="H121 AJM";
$resn_exch{"2H12 AJM"}="H122 AJM"; $resn_exch{"1H18 AJM"}="H181 AJM"; $resn_exch{"2H18 AJM"}="H182 AJM";
$resn_exch{"3H18 AJM"}="H183 AJM"; $resn_exch{"1H40 AKA"}="H401 AKA"; $resn_exch{"2H40 AKA"}="H402 AKA";
$resn_exch{"1H38 AKA"}="H381 AKA"; $resn_exch{"2H38 AKA"}="H382 AKA"; $resn_exch{"1H41 AKA"}="H411 AKA";
$resn_exch{"2H41 AKA"}="H412 AKA"; $resn_exch{"3H41 AKA"}="H413 AKA"; $resn_exch{"1H36 AKA"}="H361 AKA";
$resn_exch{"2H36 AKA"}="H362 AKA"; $resn_exch{"3H36 AKA"}="H363 AKA"; $resn_exch{"1H32 AKA"}="H321 AKA";
$resn_exch{"2H32 AKA"}="H322 AKA"; $resn_exch{" H4* AKA"}=" H4' AKA"; $resn_exch{" H5* AKA"}=" H5' AKA";
$resn_exch{"1H6* AKA"}="H6'1 AKA"; $resn_exch{"2H6* AKA"}="H6'2 AKA"; $resn_exch{"3H6* AKA"}="H6'3 AKA";
$resn_exch{" H3* AKA"}=" H3' AKA"; $resn_exch{"1H8* AKA"}="H8'1 AKA"; $resn_exch{"2H8* AKA"}="H8'2 AKA";
$resn_exch{"3H8* AKA"}="H8'3 AKA"; $resn_exch{"1H7* AKA"}="H7'1 AKA"; $resn_exch{"2H7* AKA"}="H7'2 AKA";
$resn_exch{"3H7* AKA"}="H7'3 AKA"; $resn_exch{"1H2* AKA"}="H2'1 AKA"; $resn_exch{"2H2* AKA"}="H2'2 AKA";
$resn_exch{" H1* AKA"}=" H1' AKA"; $resn_exch{"1H8  AKA"}=" H81 AKA"; $resn_exch{"2H8  AKA"}=" H82 AKA";
$resn_exch{"1H13 AKA"}="H131 AKA"; $resn_exch{"2H13 AKA"}="H132 AKA"; $resn_exch{"1H14 AKA"}="H141 AKA";
$resn_exch{"2H14 AKA"}="H142 AKA"; $resn_exch{"3H14 AKA"}="H143 AKA"; $resn_exch{"1H10 AKA"}="H101 AKA";
$resn_exch{"2H10 AKA"}="H102 AKA"; $resn_exch{"1H2  AKB"}=" H21 AKB"; $resn_exch{"1H4  AKB"}=" H41 AKB";
$resn_exch{"2H4  AKB"}=" H42 AKB"; $resn_exch{"3H4  AKB"}=" H43 AKB"; $resn_exch{"1HN4 AKB"}="HN41 AKB";
$resn_exch{"2HN4 AKB"}="HN42 AKB"; $resn_exch{"1H08 AKC"}="H081 AKC"; $resn_exch{"2H08 AKC"}="H082 AKC";
$resn_exch{"1H10 AKC"}="H101 AKC"; $resn_exch{"2H10 AKC"}="H102 AKC"; $resn_exch{"1H36 AKC"}="H361 AKC";
$resn_exch{"2H36 AKC"}="H362 AKC"; $resn_exch{"1H44 AKC"}="H441 AKC"; $resn_exch{"2H44 AKC"}="H442 AKC";
$resn_exch{"1H45 AKC"}="H451 AKC"; $resn_exch{"2H45 AKC"}="H452 AKC"; $resn_exch{"1H47 AKC"}="H471 AKC";
$resn_exch{"2H47 AKC"}="H472 AKC"; $resn_exch{"1H48 AKC"}="H481 AKC"; $resn_exch{"2H48 AKC"}="H482 AKC";
$resn_exch{"3H48 AKC"}="H483 AKC"; $resn_exch{"1H3  AKG"}=" H31 AKG"; $resn_exch{"2H3  AKG"}=" H32 AKG";
$resn_exch{"1H4  AKG"}=" H41 AKG"; $resn_exch{"2H4  AKG"}=" H42 AKG"; $resn_exch{"1HN  AKL"}=" H   AKL";
$resn_exch{"2HN  AKL"}=" H2  AKL"; $resn_exch{"1HB  AKL"}=" HB2 AKL"; $resn_exch{"2HB  AKL"}=" HB3 AKL";
$resn_exch{"1HM  AKL"}=" HM1 AKL"; $resn_exch{"2HM  AKL"}=" HM2 AKL"; $resn_exch{"1H9  AKN"}=" H91 AKN";
$resn_exch{"2H9  AKN"}=" H92 AKN"; $resn_exch{"1H10 AKN"}="H101 AKN"; $resn_exch{"2H10 AKN"}="H102 AKN";
$resn_exch{"1H13 AKN"}="H131 AKN"; $resn_exch{"2H13 AKN"}="H132 AKN"; $resn_exch{"1H15 AKN"}="H151 AKN";
$resn_exch{"2H15 AKN"}="H152 AKN"; $resn_exch{"1H25 AKN"}="H251 AKN"; $resn_exch{"2H25 AKN"}="H252 AKN";
$resn_exch{"1H30 AKN"}="H301 AKN"; $resn_exch{"2H30 AKN"}="H302 AKN"; $resn_exch{"1H34 AKN"}="H341 AKN";
$resn_exch{"2H34 AKN"}="H342 AKN"; $resn_exch{"1H38 AKN"}="H381 AKN"; $resn_exch{"2H38 AKN"}="H382 AKN";
$resn_exch{"1H39 AKN"}="H391 AKN"; $resn_exch{"2H39 AKN"}="H392 AKN"; $resn_exch{"1H3  AKP"}="H31A AKP";
$resn_exch{"2H3  AKP"}=" H32 AKP"; $resn_exch{"1H5  AKP"}=" H51 AKP"; $resn_exch{"2H5  AKP"}=" H52 AKP";
$resn_exch{"1H8  AKP"}=" H81 AKP"; $resn_exch{"2H8  AKP"}=" H82 AKP"; $resn_exch{"1H18 AKP"}="H181 AKP";
$resn_exch{"2H18 AKP"}="H182 AKP"; $resn_exch{"1H23 AKP"}="H231 AKP"; $resn_exch{"2H23 AKP"}="H232 AKP";
$resn_exch{"3H23 AKP"}="H233 AKP"; $resn_exch{"1H24 AKP"}="H241 AKP"; $resn_exch{"2H24 AKP"}="H242 AKP";
$resn_exch{"1H27 AKP"}="H271 AKP"; $resn_exch{"2H27 AKP"}="H272 AKP"; $resn_exch{"1H28 AKP"}="H281 AKP";
$resn_exch{"2H28 AKP"}="H282 AKP"; $resn_exch{"3H28 AKP"}="H283 AKP"; $resn_exch{"1H29 AKP"}="H291 AKP";
$resn_exch{"2H29 AKP"}="H292 AKP"; $resn_exch{"3H29 AKP"}="H293 AKP"; $resn_exch{"1H30 AKP"}="H301 AKP";
$resn_exch{"2H30 AKP"}="H302 AKP"; $resn_exch{"3H30 AKP"}="H303 AKP"; $resn_exch{"1H32 AKP"}="H321 AKP";
$resn_exch{"2H32 AKP"}="H322 AKP"; $resn_exch{"1H33 AKP"}="H331 AKP"; $resn_exch{"2H33 AKP"}="H332 AKP";
$resn_exch{"1H34 AKP"}="H341 AKP"; $resn_exch{"2H34 AKP"}="H342 AKP"; $resn_exch{"1H35 AKP"}="H351 AKP";
$resn_exch{"2H35 AKP"}="H352 AKP"; $resn_exch{"1H36 AKP"}="H361 AKP"; $resn_exch{"2H36 AKP"}="H362 AKP";
$resn_exch{"1HA  AKR"}=" HA1 AKR"; $resn_exch{"2HB  AKR"}=" HB2 AKR"; $resn_exch{"3HB  AKR"}=" HB3 AKR";
$resn_exch{"1H10 AKT"}="H101 AKT"; $resn_exch{"2H10 AKT"}="H102 AKT"; $resn_exch{"1H8  AKT"}=" H81 AKT";
$resn_exch{"2H8  AKT"}=" H82 AKT"; $resn_exch{"1H13 AKT"}="H131 AKT"; $resn_exch{"2H13 AKT"}="H132 AKT";
$resn_exch{"1H14 AKT"}="H141 AKT"; $resn_exch{"2H14 AKT"}="H142 AKT"; $resn_exch{"3H14 AKT"}="H143 AKT";
$resn_exch{" H1* AKT"}=" H1' AKT"; $resn_exch{"1H2* AKT"}="H2'1 AKT"; $resn_exch{"2H2* AKT"}="H2'2 AKT";
$resn_exch{" H3* AKT"}=" H3' AKT"; $resn_exch{"1H8* AKT"}="H8'1 AKT"; $resn_exch{"2H8* AKT"}="H8'2 AKT";
$resn_exch{"3H8* AKT"}="H8'3 AKT"; $resn_exch{"1H7* AKT"}="H7'1 AKT"; $resn_exch{"2H7* AKT"}="H7'2 AKT";
$resn_exch{"3H7* AKT"}="H7'3 AKT"; $resn_exch{" H4* AKT"}=" H4' AKT"; $resn_exch{" H5* AKT"}=" H5' AKT";
$resn_exch{"1H6* AKT"}="H6'1 AKT"; $resn_exch{"2H6* AKT"}="H6'2 AKT"; $resn_exch{"3H6* AKT"}="H6'3 AKT";
$resn_exch{"1H10 AKV"}="H101 AKV"; $resn_exch{"2H10 AKV"}="H102 AKV"; $resn_exch{"1H15 AKV"}="H151 AKV";
$resn_exch{"2H15 AKV"}="H152 AKV"; $resn_exch{"3H15 AKV"}="H153 AKV"; $resn_exch{"1H8  AKV"}=" H81 AKV";
$resn_exch{"2H8  AKV"}=" H82 AKV"; $resn_exch{"1H13 AKV"}="H131 AKV"; $resn_exch{"2H13 AKV"}="H132 AKV";
$resn_exch{"1H22 AKV"}="H221 AKV"; $resn_exch{"2H22 AKV"}="H222 AKV"; $resn_exch{"3H22 AKV"}="H223 AKV";
$resn_exch{"1H14 AKY"}="H141 AKY"; $resn_exch{"2H14 AKY"}="H142 AKY"; $resn_exch{"3H14 AKY"}="H143 AKY";
$resn_exch{"1H13 AKY"}="H131 AKY"; $resn_exch{"2H13 AKY"}="H132 AKY"; $resn_exch{"1H8  AKY"}=" H81 AKY";
$resn_exch{"2H8  AKY"}=" H82 AKY"; $resn_exch{"1H46 AKY"}="H461 AKY"; $resn_exch{"2H46 AKY"}="H462 AKY";
$resn_exch{"1H32 AKY"}="H321 AKY"; $resn_exch{"2H32 AKY"}="H322 AKY"; $resn_exch{"1H38 AKY"}="H381 AKY";
$resn_exch{"2H38 AKY"}="H382 AKY"; $resn_exch{"1H40 AKY"}="H401 AKY"; $resn_exch{"2H40 AKY"}="H402 AKY";
$resn_exch{"1H41 AKY"}="H411 AKY"; $resn_exch{"2H41 AKY"}="H412 AKY"; $resn_exch{"3H41 AKY"}="H413 AKY";
$resn_exch{"1H36 AKY"}="H361 AKY"; $resn_exch{"2H36 AKY"}="H362 AKY"; $resn_exch{"3H36 AKY"}="H363 AKY";
$resn_exch{"1H50 AKY"}="H501 AKY"; $resn_exch{"2H50 AKY"}="H502 AKY"; $resn_exch{"3H50 AKY"}="H503 AKY";
$resn_exch{"1H52 AKY"}="H521 AKY"; $resn_exch{"2H52 AKY"}="H522 AKY"; $resn_exch{"3H52 AKY"}="H523 AKY";
$resn_exch{"1H51 AKY"}="H511 AKY"; $resn_exch{"2H51 AKY"}="H512 AKY"; $resn_exch{"3H51 AKY"}="H513 AKY";
$resn_exch{"1H44 AKY"}="H441 AKY"; $resn_exch{"2H44 AKY"}="H442 AKY"; $resn_exch{"3H44 AKY"}="H443 AKY";
$resn_exch{"1HN  AL0"}=" HN1 AL0"; $resn_exch{"2HN  AL0"}=" HN2 AL0"; $resn_exch{"1HB  AL0"}=" HB1 AL0";
$resn_exch{"2HB  AL0"}=" HB2 AL0"; $resn_exch{"1H9  AL1"}=" H91 AL1"; $resn_exch{"2H9  AL1"}=" H92 AL1";
$resn_exch{"1H10 AL1"}="H101 AL1"; $resn_exch{"2H10 AL1"}="H102 AL1"; $resn_exch{"1H24 AL1"}="H241 AL1";
$resn_exch{"2H24 AL1"}="H242 AL1"; $resn_exch{"3H24 AL1"}="H243 AL1"; $resn_exch{"1HNL AL1"}="HNL1 AL1";
$resn_exch{"2HNL AL1"}="HNL2 AL1"; $resn_exch{"1H9  AL2"}=" H91 AL2"; $resn_exch{"2H9  AL2"}=" H92 AL2";
$resn_exch{"1HNL AL2"}="HNL1 AL2"; $resn_exch{"2HNL AL2"}="HNL2 AL2"; $resn_exch{"1H17 AL2"}="H171 AL2";
$resn_exch{"2H17 AL2"}="H172 AL2"; $resn_exch{"3H17 AL2"}="H173 AL2"; $resn_exch{"1H9  AL3"}=" H91 AL3";
$resn_exch{"2H9  AL3"}=" H92 AL3"; $resn_exch{"1H11 AL3"}="H111 AL3"; $resn_exch{"2H11 AL3"}="H112 AL3";
$resn_exch{"1HNL AL3"}="HNL1 AL3"; $resn_exch{"2HNL AL3"}="HNL2 AL3"; $resn_exch{"1H9  AL4"}=" H91 AL4";
$resn_exch{"2H9  AL4"}=" H92 AL4"; $resn_exch{"1H11 AL4"}="H111 AL4"; $resn_exch{"2H11 AL4"}="H112 AL4";
$resn_exch{"1H12 AL4"}="H121 AL4"; $resn_exch{"2H12 AL4"}="H122 AL4"; $resn_exch{"1H14 AL4"}="H141 AL4";
$resn_exch{"2H14 AL4"}="H142 AL4"; $resn_exch{"3H14 AL4"}="H143 AL4"; $resn_exch{"1H16 AL4"}="H161 AL4";
$resn_exch{"2H16 AL4"}="H162 AL4"; $resn_exch{"1HNL AL4"}="HNL1 AL4"; $resn_exch{"2HNL AL4"}="HNL2 AL4";
$resn_exch{"1H17 AL4"}="H171 AL4"; $resn_exch{"2H17 AL4"}="H172 AL4"; $resn_exch{"3H17 AL4"}="H173 AL4";
$resn_exch{"1H9  AL5"}=" H91 AL5"; $resn_exch{"2H9  AL5"}=" H92 AL5"; $resn_exch{"1H16 AL5"}="H161 AL5";
$resn_exch{"2H16 AL5"}="H162 AL5"; $resn_exch{"3H16 AL5"}="H163 AL5"; $resn_exch{"1HNL AL5"}="HNL1 AL5";
$resn_exch{"2HNL AL5"}="HNL2 AL5"; $resn_exch{"1HNL AL6"}="HNL1 AL6"; $resn_exch{"2HNL AL6"}="HNL2 AL6";
$resn_exch{"1H17 AL6"}="H171 AL6"; $resn_exch{"2H17 AL6"}="H172 AL6"; $resn_exch{"3H17 AL6"}="H173 AL6";
$resn_exch{"1H9  AL7"}=" H91 AL7"; $resn_exch{"2H9  AL7"}=" H92 AL7"; $resn_exch{"1H16 AL7"}="H161 AL7";
$resn_exch{"2H16 AL7"}="H162 AL7"; $resn_exch{"3H16 AL7"}="H163 AL7"; $resn_exch{"1H24 AL7"}="H241 AL7";
$resn_exch{"2H24 AL7"}="H242 AL7"; $resn_exch{"3H24 AL7"}="H243 AL7"; $resn_exch{"1HNL AL7"}="HNL1 AL7";
$resn_exch{"2HNL AL7"}="HNL2 AL7"; $resn_exch{"1H9  AL8"}=" H91 AL8"; $resn_exch{"2H9  AL8"}=" H92 AL8";
$resn_exch{"1H16 AL8"}="H161 AL8"; $resn_exch{"2H16 AL8"}="H162 AL8"; $resn_exch{"3H16 AL8"}="H163 AL8";
$resn_exch{"1H24 AL8"}="H241 AL8"; $resn_exch{"2H24 AL8"}="H242 AL8"; $resn_exch{"3H24 AL8"}="H243 AL8";
$resn_exch{"1HNL AL8"}="HNL1 AL8"; $resn_exch{"2HNL AL8"}="HNL2 AL8"; $resn_exch{"1H9  AL9"}=" H91 AL9";
$resn_exch{"2H9  AL9"}=" H92 AL9"; $resn_exch{"1HNL AL9"}="HNL1 AL9"; $resn_exch{"2HNL AL9"}="HNL2 AL9";
$resn_exch{"1H17 AL9"}="H171 AL9"; $resn_exch{"2H17 AL9"}="H172 AL9"; $resn_exch{"3H17 AL9"}="H173 AL9";
$resn_exch{" HN2 ALA"}=" H2  ALA"; $resn_exch{"1HB  ALA"}=" HB1 ALA"; $resn_exch{"2HB  ALA"}=" HB2 ALA";
$resn_exch{"3HB  ALA"}=" HB3 ALA"; $resn_exch{"2H9  ALB"}=" H92 ALB"; $resn_exch{"1H9  ALB"}=" H91 ALB";
$resn_exch{"2H10 ALB"}="H102 ALB"; $resn_exch{"1H10 ALB"}="H101 ALB"; $resn_exch{"2H11 ALB"}="H112 ALB";
$resn_exch{"1H11 ALB"}="H111 ALB"; $resn_exch{"2H12 ALB"}="H122 ALB"; $resn_exch{"1H12 ALB"}="H121 ALB";
$resn_exch{"2H13 ALB"}="H132 ALB"; $resn_exch{"1H13 ALB"}="H131 ALB"; $resn_exch{"2H14 ALB"}="H142 ALB";
$resn_exch{"1H14 ALB"}="H141 ALB"; $resn_exch{"2H15 ALB"}="H152 ALB"; $resn_exch{"1H15 ALB"}="H151 ALB";
$resn_exch{"2H16 ALB"}="H162 ALB"; $resn_exch{"1H16 ALB"}="H161 ALB"; $resn_exch{"2H17 ALB"}="H172 ALB";
$resn_exch{"1H17 ALB"}="H171 ALB"; $resn_exch{"3H18 ALB"}="H183 ALB"; $resn_exch{"2H18 ALB"}="H182 ALB";
$resn_exch{"1H18 ALB"}="H181 ALB"; $resn_exch{"3H19 ALB"}="H193 ALB"; $resn_exch{"2H19 ALB"}="H192 ALB";
$resn_exch{"1H19 ALB"}="H191 ALB"; $resn_exch{"3H20 ALB"}="H203 ALB"; $resn_exch{"2H20 ALB"}="H202 ALB";
$resn_exch{"1H20 ALB"}="H201 ALB"; $resn_exch{"2H23 ALB"}="H232 ALB"; $resn_exch{"1H23 ALB"}="H231 ALB";
$resn_exch{"3H36 ALB"}="H363 ALB"; $resn_exch{"2H36 ALB"}="H362 ALB"; $resn_exch{"1H36 ALB"}="H361 ALB";
$resn_exch{"1H7N ALB"}="H7N1 ALB"; $resn_exch{"3H1N ALB"}="H1N3 ALB"; $resn_exch{"2H1N ALB"}="H1N2 ALB";
$resn_exch{"4H1N ALB"}="H1N4 ALB"; $resn_exch{"5H1N ALB"}="H1N5 ALB"; $resn_exch{" HN2 ALC"}=" H2  ALC";
$resn_exch{"1HB  ALC"}=" HB2 ALC"; $resn_exch{"2HB  ALC"}=" HB3 ALC"; $resn_exch{"1HD2 ALC"}="HD23 ALC";
$resn_exch{"2HD2 ALC"}="HD22 ALC"; $resn_exch{"1HE2 ALC"}="HE23 ALC"; $resn_exch{"2HE2 ALC"}="HE22 ALC";
$resn_exch{"1HZ  ALC"}=" HZ3 ALC"; $resn_exch{"2HZ  ALC"}=" HZ2 ALC"; $resn_exch{"1HE1 ALC"}="HE13 ALC";
$resn_exch{"2HE1 ALC"}="HE12 ALC"; $resn_exch{"1HD1 ALC"}="HD12 ALC"; $resn_exch{"2HD1 ALC"}="HD13 ALC";
$resn_exch{"1H7  ALD"}=" H71 ALD"; $resn_exch{"2H7  ALD"}=" H72 ALD"; $resn_exch{"1H18 ALD"}="H181 ALD";
$resn_exch{"2H18 ALD"}="H182 ALD"; $resn_exch{"1H20 ALD"}="H201 ALD"; $resn_exch{"2H20 ALD"}="H202 ALD";
$resn_exch{"3H20 ALD"}="H203 ALD"; $resn_exch{"1H21 ALD"}="H211 ALD"; $resn_exch{"2H21 ALD"}="H212 ALD";
$resn_exch{"3H21 ALD"}="H213 ALD"; $resn_exch{"1H22 ALD"}="H221 ALD"; $resn_exch{"2H22 ALD"}="H222 ALD";
$resn_exch{"1H24 ALD"}="H241 ALD"; $resn_exch{"2H24 ALD"}="H242 ALD"; $resn_exch{"1H26 ALD"}="H261 ALD";
$resn_exch{"2H26 ALD"}="H262 ALD"; $resn_exch{"3H26 ALD"}="H263 ALD"; $resn_exch{"1H27 ALD"}="H271 ALD";
$resn_exch{"2H27 ALD"}="H272 ALD"; $resn_exch{"3H27 ALD"}="H273 ALD"; $resn_exch{"1H30 ALD"}="H301 ALD";
$resn_exch{"2H30 ALD"}="H302 ALD"; $resn_exch{"1H32 ALD"}="H321 ALD"; $resn_exch{"2H32 ALD"}="H322 ALD";
$resn_exch{"3H32 ALD"}="H323 ALD"; $resn_exch{"1H33 ALD"}="H331 ALD"; $resn_exch{"2H33 ALD"}="H332 ALD";
$resn_exch{"3H33 ALD"}="H333 ALD"; $resn_exch{"1H8  ALE"}=" H81 ALE"; $resn_exch{"2H8  ALE"}=" H82 ALE";
$resn_exch{"1H9  ALE"}=" H91 ALE"; $resn_exch{"2H9  ALE"}=" H92 ALE"; $resn_exch{"3H9  ALE"}=" H93 ALE";
$resn_exch{"1HA  ALG"}=" HA1 ALG"; $resn_exch{"2HA  ALG"}=" HA2 ALG"; $resn_exch{"1HB  ALG"}=" HB1 ALG";
$resn_exch{"2HB  ALG"}=" HB2 ALG"; $resn_exch{"1HD  ALG"}=" HD1 ALG"; $resn_exch{"2HD  ALG"}=" HD2 ALG";
$resn_exch{"1HH2 ALG"}="HH21 ALG"; $resn_exch{"2HH2 ALG"}="HH22 ALG"; $resn_exch{"1HAG ALH"}="HAG1 ALH";
$resn_exch{"2HAG ALH"}="HAG2 ALH"; $resn_exch{"1HAD ALH"}="HAD1 ALH"; $resn_exch{"2HAD ALH"}="HAD2 ALH";
$resn_exch{"1HAC ALH"}="HAC1 ALH"; $resn_exch{"2HAC ALH"}="HAC2 ALH"; $resn_exch{"1HAB ALH"}="HAB1 ALH";
$resn_exch{"2HAB ALH"}="HAB2 ALH"; $resn_exch{"3HAB ALH"}="HAB3 ALH"; $resn_exch{"1H6C ALI"}="H6C1 ALI";
$resn_exch{"2H6C ALI"}="H6C2 ALI"; $resn_exch{"1H8C ALI"}="H8C1 ALI"; $resn_exch{"2H8C ALI"}="H8C2 ALI";
$resn_exch{"1HDP ALJ"}="HDP1 ALJ"; $resn_exch{"2HDP ALJ"}="HDP2 ALJ"; $resn_exch{"1HGP ALJ"}="HGP1 ALJ";
$resn_exch{"2HGP ALJ"}="HGP2 ALJ"; $resn_exch{"1HBP ALJ"}="HBP1 ALJ"; $resn_exch{"2HBP ALJ"}="HBP2 ALJ";
$resn_exch{"1HBC ALJ"}="HBC1 ALJ"; $resn_exch{"2HBC ALJ"}="HBC2 ALJ"; $resn_exch{"1HGC ALJ"}="HGC1 ALJ";
$resn_exch{"2HGC ALJ"}="HGC2 ALJ"; $resn_exch{"1HDC ALJ"}="HDC1 ALJ"; $resn_exch{"2HDC ALJ"}="HDC2 ALJ";
$resn_exch{"1HH1 ALJ"}="HH11 ALJ"; $resn_exch{"2HH1 ALJ"}="HH12 ALJ"; $resn_exch{"1HH2 ALJ"}="HH21 ALJ";
$resn_exch{"2HH2 ALJ"}="HH22 ALJ"; $resn_exch{" H1* ALL"}=" H1' ALL"; $resn_exch{" H2* ALL"}=" H2' ALL";
$resn_exch{" H3* ALL"}=" H3' ALL"; $resn_exch{" H4* ALL"}=" H4' ALL"; $resn_exch{" H5* ALL"}=" H5' ALL";
$resn_exch{"1H6* ALL"}="H6'1 ALL"; $resn_exch{"2H6* ALL"}="H6'2 ALL"; $resn_exch{"1HO* ALL"}="HO'1 ALL";
$resn_exch{"2HO* ALL"}="HO'2 ALL"; $resn_exch{"3HO* ALL"}="HO'3 ALL"; $resn_exch{"4HO* ALL"}="HO'4 ALL";
$resn_exch{"6HO* ALL"}="HO'6 ALL"; $resn_exch{"1HN  ALM"}=" H   ALM"; $resn_exch{"2HN  ALM"}=" H2  ALM";
$resn_exch{"1HB  ALM"}=" HB1 ALM"; $resn_exch{"2HB  ALM"}=" HB2 ALM"; $resn_exch{"3HB  ALM"}=" HB3 ALM";
$resn_exch{"1HM  ALM"}=" HM1 ALM"; $resn_exch{"2HM  ALM"}=" HM2 ALM"; $resn_exch{"3HM  ALM"}=" HM3 ALM";
$resn_exch{"1HN  ALN"}=" H   ALN"; $resn_exch{"2HN  ALN"}=" H2  ALN"; $resn_exch{"1HB  ALN"}=" HB2 ALN";
$resn_exch{"2HB  ALN"}=" HB3 ALN"; $resn_exch{"1HN  ALO"}=" H   ALO"; $resn_exch{"2HN  ALO"}=" H2  ALO";
$resn_exch{"1HG2 ALO"}="HG21 ALO"; $resn_exch{"2HG2 ALO"}="HG22 ALO"; $resn_exch{"3HG2 ALO"}="HG23 ALO";
$resn_exch{" HOG ALO"}=" HG1 ALO"; $resn_exch{"1H21 ALP"}="H211 ALP"; $resn_exch{"2H21 ALP"}="H212 ALP";
$resn_exch{"1H51 ALP"}="H511 ALP"; $resn_exch{"2H51 ALP"}="H512 ALP"; $resn_exch{"3H51 ALP"}="H513 ALP";
$resn_exch{"1H52 ALP"}="H521 ALP"; $resn_exch{"2H52 ALP"}="H522 ALP"; $resn_exch{"3H52 ALP"}="H523 ALP";
$resn_exch{"1HM  ALQ"}=" HM1 ALQ"; $resn_exch{"2HM  ALQ"}=" HM2 ALQ"; $resn_exch{"3HM  ALQ"}=" HM3 ALQ";
$resn_exch{"1HB  ALQ"}=" HB1 ALQ"; $resn_exch{"2HB  ALQ"}=" HB2 ALQ"; $resn_exch{"3HB  ALQ"}=" HB3 ALQ";
$resn_exch{"1H7  ALR"}=" H71 ALR"; $resn_exch{"2H7  ALR"}=" H72 ALR"; $resn_exch{"1HN2 ALS"}="HN21 ALS";
$resn_exch{"2HN2 ALS"}="HN22 ALS"; $resn_exch{"4HOS ALS"}="HOS4 ALS"; $resn_exch{"1HN  ALT"}=" H   ALT";
$resn_exch{"2HN  ALT"}=" H2  ALT"; $resn_exch{"1HB  ALT"}=" HB1 ALT"; $resn_exch{"2HB  ALT"}=" HB2 ALT";
$resn_exch{"3HB  ALT"}=" HB3 ALT"; $resn_exch{"1H11 ALW"}="H111 ALW"; $resn_exch{"2H11 ALW"}="H112 ALW";
$resn_exch{"1H3' ALW"}="H3'1 ALW"; $resn_exch{"2H3' ALW"}="H3'2 ALW"; $resn_exch{"1HC  ALW"}=" HC1 ALW";
$resn_exch{"2HC  ALW"}=" HC2 ALW"; $resn_exch{"3HC  ALW"}=" HC3 ALW"; $resn_exch{"1H6C ALW"}="H6C1 ALW";
$resn_exch{"2H6C ALW"}="H6C2 ALW"; $resn_exch{"3H6C ALW"}="H6C3 ALW"; $resn_exch{"1HH3 ALY"}="HH31 ALY";
$resn_exch{"2HH3 ALY"}="HH32 ALY"; $resn_exch{"3HH3 ALY"}="HH33 ALY"; $resn_exch{" HNZ ALY"}=" HZ  ALY";
$resn_exch{"1HCE ALY"}=" HE3 ALY"; $resn_exch{"2HCE ALY"}=" HE2 ALY"; $resn_exch{"1HCD ALY"}=" HD3 ALY";
$resn_exch{"2HCD ALY"}=" HD2 ALY"; $resn_exch{"1HCG ALY"}=" HG3 ALY"; $resn_exch{"2HCG ALY"}=" HG2 ALY";
$resn_exch{"1HCB ALY"}=" HB3 ALY"; $resn_exch{"2HCB ALY"}=" HB2 ALY"; $resn_exch{"1HN  ALY"}=" H   ALY";
$resn_exch{"2HN  ALY"}=" H2  ALY"; $resn_exch{"1HN1 ALZ"}="HN11 ALZ"; $resn_exch{"2HN1 ALZ"}="HN12 ALZ";
$resn_exch{"1H2  ALZ"}="H21A ALZ"; $resn_exch{"2H2  ALZ"}="H22A ALZ"; $resn_exch{"1H3  ALZ"}=" H31 ALZ";
$resn_exch{"2H3  ALZ"}=" H32 ALZ"; $resn_exch{"1H4  ALZ"}=" H41 ALZ"; $resn_exch{"2H4  ALZ"}=" H42 ALZ";
$resn_exch{"1H5  ALZ"}=" H51 ALZ"; $resn_exch{"2H5  ALZ"}=" H52 ALZ"; $resn_exch{"1H10 ALZ"}="H101 ALZ";
$resn_exch{"2H10 ALZ"}="H102 ALZ"; $resn_exch{"1H11 ALZ"}="H111 ALZ"; $resn_exch{"2H11 ALZ"}="H112 ALZ";
$resn_exch{"1H12 ALZ"}="H121 ALZ"; $resn_exch{"2H12 ALZ"}="H122 ALZ"; $resn_exch{"1H16 ALZ"}="H161 ALZ";
$resn_exch{"2H16 ALZ"}="H162 ALZ"; $resn_exch{"1HA3 AM2"}="HA31 AM2"; $resn_exch{"2HA3 AM2"}="HA32 AM2";
$resn_exch{"1HA9 AM2"}="HA91 AM2"; $resn_exch{"2HA9 AM2"}="HA92 AM2"; $resn_exch{"3HA9 AM2"}="HA93 AM2";
$resn_exch{"1HA2 AM2"}="HA21 AM2"; $resn_exch{"2HA2 AM2"}="HA22 AM2"; $resn_exch{"1HB6 AM2"}="HB61 AM2";
$resn_exch{"2HB6 AM2"}="HB62 AM2"; $resn_exch{"1HB4 AM2"}="HB41 AM2"; $resn_exch{"2HB4 AM2"}="HB42 AM2";
$resn_exch{"1HC5 AM2"}="HC51 AM2"; $resn_exch{"2HC5 AM2"}="HC52 AM2"; $resn_exch{"1HC4 AM2"}="HC41 AM2";
$resn_exch{"2HC4 AM2"}="HC42 AM2"; $resn_exch{"1HC6 AM2"}="HC61 AM2"; $resn_exch{"2HC6 AM2"}="HC62 AM2";
$resn_exch{"1H12 AM4"}="H121 AM4"; $resn_exch{"2H12 AM4"}="H122 AM4"; $resn_exch{"1H10 AM4"}="H101 AM4";
$resn_exch{"2H10 AM4"}="H102 AM4"; $resn_exch{"3H10 AM4"}="H103 AM4"; $resn_exch{"1H4  AM7"}=" H41 AM7";
$resn_exch{"2H4  AM7"}=" H42 AM7"; $resn_exch{"1H1  AM7"}=" H11 AM7"; $resn_exch{"2H1  AM7"}=" H12 AM7";
$resn_exch{"1H2  AM7"}=" H21 AM7"; $resn_exch{"2H2  AM7"}="H22A AM7"; $resn_exch{"1H3  AM7"}=" H31 AM7";
$resn_exch{"2H3  AM7"}=" H32 AM7"; $resn_exch{"1H33 AM7"}="H331 AM7"; $resn_exch{"2H33 AM7"}="H332 AM7";
$resn_exch{"1H32 AM7"}="H321 AM7"; $resn_exch{"2H32 AM7"}="H322 AM7"; $resn_exch{"1H31 AM7"}="H311 AM7";
$resn_exch{"2H31 AM7"}="H312 AM7"; $resn_exch{"1H35 AM7"}="H351 AM7"; $resn_exch{"2H35 AM7"}="H352 AM7";
$resn_exch{"3H35 AM7"}="H353 AM7"; $resn_exch{"1H34 AM7"}="H341 AM7"; $resn_exch{"2H34 AM7"}="H342 AM7";
$resn_exch{"3H34 AM7"}="H343 AM7"; $resn_exch{"1H30 AM7"}="H301 AM7"; $resn_exch{"2H30 AM7"}="H302 AM7";
$resn_exch{"1HAA AM8"}="HAA1 AM8"; $resn_exch{"2HAA AM8"}="HAA2 AM8"; $resn_exch{"3HAA AM8"}="HAA3 AM8";
$resn_exch{"1HAE AM8"}="HAE1 AM8"; $resn_exch{"2HAE AM8"}="HAE2 AM8"; $resn_exch{"3HAE AM8"}="HAE3 AM8";
$resn_exch{"1HAG AM8"}="HAG1 AM8"; $resn_exch{"2HAG AM8"}="HAG2 AM8"; $resn_exch{"3HAG AM8"}="HAG3 AM8";
$resn_exch{"1HAJ AM8"}="HAJ1 AM8"; $resn_exch{"2HAJ AM8"}="HAJ2 AM8"; $resn_exch{"1HBE AM8"}="HBE1 AM8";
$resn_exch{"2HBE AM8"}="HBE2 AM8"; $resn_exch{"3HBE AM8"}="HBE3 AM8"; $resn_exch{"1H2A AMA"}="H2A1 AMA";
$resn_exch{"2H2A AMA"}="H2A2 AMA"; $resn_exch{"3H2A AMA"}="H2A3 AMA"; $resn_exch{"1H4A AMA"}="H4A1 AMA";
$resn_exch{"2H4A AMA"}="H4A2 AMA"; $resn_exch{"1H5A AMA"}="H5A1 AMA"; $resn_exch{"2H5A AMA"}="H5A2 AMA";
$resn_exch{"2HOP AMA"}="HOP2 AMA"; $resn_exch{"3HOP AMA"}="HOP3 AMA"; $resn_exch{"1HB2 AMA"}="HB21 AMA";
$resn_exch{"2HB2 AMA"}="HB22 AMA"; $resn_exch{"3HB2 AMA"}="HB23 AMA"; $resn_exch{"1HB1 AMA"}="HB11 AMA";
$resn_exch{"2HB1 AMA"}="HB12 AMA"; $resn_exch{"3HE  AMB"}=" HE3 AMB"; $resn_exch{"2HE  AMB"}=" HE2 AMB";
$resn_exch{"1HE  AMB"}=" HE1 AMB"; $resn_exch{"2H   AMB"}=" H2  AMB"; $resn_exch{"1H   AMB"}=" H1  AMB";
$resn_exch{"1H2  AMC"}=" H21 AMC"; $resn_exch{"2H2  AMC"}=" H22 AMC"; $resn_exch{"1H3  AMC"}=" H31 AMC";
$resn_exch{"2H3  AMC"}=" H32 AMC"; $resn_exch{"1H4  AMC"}=" H41 AMC"; $resn_exch{"2H4  AMC"}=" H42 AMC";
$resn_exch{"1H5  AMC"}=" H51 AMC"; $resn_exch{"2H5  AMC"}=" H52 AMC"; $resn_exch{"1H6  AMC"}=" H61 AMC";
$resn_exch{"2H6  AMC"}=" H62 AMC"; $resn_exch{"1HM  AMC"}=" HM1 AMC"; $resn_exch{"2HM  AMC"}=" HM2 AMC";
$resn_exch{"1HN  AMC"}=" HN1 AMC"; $resn_exch{"2HN  AMC"}=" HN2 AMC"; $resn_exch{"3HN  AMC"}=" HN3 AMC";
$resn_exch{"2HOP AMD"}="HOP2 AMD"; $resn_exch{"1H5* AMD"}="H5'1 AMD"; $resn_exch{"2H5* AMD"}="H5'2 AMD";
$resn_exch{" H4* AMD"}=" H4' AMD"; $resn_exch{" H1* AMD"}=" H1' AMD"; $resn_exch{"1H6  AMD"}=" H61 AMD";
$resn_exch{"2H6  AMD"}=" H62 AMD"; $resn_exch{"1H2* AMD"}="H2'1 AMD"; $resn_exch{"2H2* AMD"}="H2'2 AMD";
$resn_exch{" H3* AMD"}=" H3' AMD"; $resn_exch{"3HO* AMD"}="HO'3 AMD"; $resn_exch{"1HB  AMD"}=" HB1 AMD";
$resn_exch{"2HB  AMD"}=" HB2 AMD"; $resn_exch{"3HT2 AME"}="HT23 AME"; $resn_exch{"2HT2 AME"}="HT22 AME";
$resn_exch{"1HT2 AME"}="HT21 AME"; $resn_exch{"2HB  AME"}=" HB2 AME"; $resn_exch{"1HB  AME"}=" HB1 AME";
$resn_exch{"2HG  AME"}=" HG2 AME"; $resn_exch{"1HG  AME"}=" HG1 AME"; $resn_exch{"3HE  AME"}=" HE3 AME";
$resn_exch{"2HE  AME"}=" HE2 AME"; $resn_exch{"1HE  AME"}=" HE1 AME"; $resn_exch{"1HN  AME"}=" HN1 AME";
$resn_exch{"1H6  AMG"}=" H61 AMG"; $resn_exch{"2H6  AMG"}=" H62 AMG"; $resn_exch{"1H7  AMG"}=" H71 AMG";
$resn_exch{"2H7  AMG"}=" H72 AMG"; $resn_exch{"3H7  AMG"}=" H73 AMG"; $resn_exch{"1H2  AMH"}=" H21 AMH";
$resn_exch{"2H2  AMH"}=" H22 AMH"; $resn_exch{"1H3  AMH"}=" H31 AMH"; $resn_exch{"2H3  AMH"}=" H32 AMH";
$resn_exch{"1H5  AMH"}=" H51 AMH"; $resn_exch{"2H5  AMH"}=" H52 AMH"; $resn_exch{"1H6  AMH"}=" H61 AMH";
$resn_exch{"2H6  AMH"}=" H62 AMH"; $resn_exch{"1H7  AMH"}=" H71 AMH"; $resn_exch{"2H7  AMH"}=" H72 AMH";
$resn_exch{"1HN  AMH"}=" HN1 AMH"; $resn_exch{"2HN  AMH"}=" HN2 AMH"; $resn_exch{"1H6  AMI"}=" H61 AMI";
$resn_exch{"2H6  AMI"}=" H62 AMI"; $resn_exch{"1H8  AMI"}=" H81 AMI"; $resn_exch{"2H8  AMI"}=" H82 AMI";
$resn_exch{"3H8  AMI"}=" H83 AMI"; $resn_exch{"1H9  AMI"}=" H91 AMI"; $resn_exch{"2H9  AMI"}=" H92 AMI";
$resn_exch{"3H9  AMI"}=" H93 AMI"; $resn_exch{"1H13 AMK"}="H131 AMK"; $resn_exch{"2H13 AMK"}="H132 AMK";
$resn_exch{"1H16 AMK"}="H161 AMK"; $resn_exch{"2H16 AMK"}="H162 AMK"; $resn_exch{"1H19 AMK"}="H191 AMK";
$resn_exch{"2H19 AMK"}="H192 AMK"; $resn_exch{"1H28 AMK"}="H281 AMK"; $resn_exch{"2H28 AMK"}="H282 AMK";
$resn_exch{"3H28 AMK"}="H283 AMK"; $resn_exch{"1H36 AMK"}="H361 AMK"; $resn_exch{"2H36 AMK"}="H362 AMK";
$resn_exch{"1H41 AMK"}="H411 AMK"; $resn_exch{"2H41 AMK"}="H412 AMK"; $resn_exch{"3H41 AMK"}="H413 AMK";
$resn_exch{"1H45 AMK"}="H451 AMK"; $resn_exch{"2H45 AMK"}="H452 AMK"; $resn_exch{"3H45 AMK"}="H453 AMK";
$resn_exch{"1H65 AMK"}="H651 AMK"; $resn_exch{"2H65 AMK"}="H652 AMK"; $resn_exch{"1H68 AMK"}="H681 AMK";
$resn_exch{"2H68 AMK"}="H682 AMK"; $resn_exch{"1H71 AMK"}="H711 AMK"; $resn_exch{"2H71 AMK"}="H712 AMK";
$resn_exch{"1H80 AMK"}="H801 AMK"; $resn_exch{"2H80 AMK"}="H802 AMK"; $resn_exch{"3H80 AMK"}="H803 AMK";
$resn_exch{"1H84 AMK"}="H841 AMK"; $resn_exch{"2H84 AMK"}="H842 AMK"; $resn_exch{"3H84 AMK"}="H843 AMK";
$resn_exch{"1H88 AMK"}="H881 AMK"; $resn_exch{"2H88 AMK"}="H882 AMK"; $resn_exch{"1H93 AMK"}="H931 AMK";
$resn_exch{"2H93 AMK"}="H932 AMK"; $resn_exch{"1HC1 AML"}="HC11 AML"; $resn_exch{"2HC1 AML"}="HC12 AML";
$resn_exch{"3HC1 AML"}="HC13 AML"; $resn_exch{"1HC2 AML"}="HC21 AML"; $resn_exch{"2HC2 AML"}="HC22 AML";
$resn_exch{"1HC3 AML"}="HC31 AML"; $resn_exch{"2HC3 AML"}="HC32 AML"; $resn_exch{"1HC4 AML"}="HC41 AML";
$resn_exch{"2HC4 AML"}="HC42 AML"; $resn_exch{"1HC5 AML"}="HC51 AML"; $resn_exch{"2HC5 AML"}="HC52 AML";
$resn_exch{"1HN1 AML"}="HN11 AML"; $resn_exch{"2HN1 AML"}="HN12 AML"; $resn_exch{"1HN  AMM"}=" HN1 AMM";
$resn_exch{"2HN  AMM"}=" HN2 AMM"; $resn_exch{"1HA  AMM"}=" HA1 AMM"; $resn_exch{"2HA  AMM"}=" HA2 AMM";
$resn_exch{"1HH2 AMM"}="HH21 AMM"; $resn_exch{"2HH2 AMM"}="HH22 AMM"; $resn_exch{"1HB  AMM"}=" HB1 AMM";
$resn_exch{"2HB  AMM"}=" HB2 AMM"; $resn_exch{"1HD1 AMM"}="HD11 AMM"; $resn_exch{"2HD1 AMM"}="HD12 AMM";
$resn_exch{"3HD1 AMM"}="HD13 AMM"; $resn_exch{"1HD2 AMM"}="HD21 AMM"; $resn_exch{"2HD2 AMM"}="HD22 AMM";
$resn_exch{"3HD2 AMM"}="HD23 AMM"; $resn_exch{"1H3  AMN"}=" H31 AMN"; $resn_exch{"2H3  AMN"}=" H32 AMN";
$resn_exch{"1H9  AMN"}=" H91 AMN"; $resn_exch{"2H9  AMN"}=" H92 AMN"; $resn_exch{"1H11 AMN"}="H111 AMN";
$resn_exch{"2H11 AMN"}="H112 AMN"; $resn_exch{"3H11 AMN"}="H113 AMN"; $resn_exch{"1H12 AMN"}="H121 AMN";
$resn_exch{"2H12 AMN"}="H122 AMN"; $resn_exch{"3H12 AMN"}="H123 AMN"; $resn_exch{"1HN9 AMN"}="HN91 AMN";
$resn_exch{"2HN9 AMN"}="HN92 AMN"; $resn_exch{"1HOB AMN"}="HOB1 AMN"; $resn_exch{"2HOP AMO"}="HOP2 AMO";
$resn_exch{"1H5* AMO"}="H5'1 AMO"; $resn_exch{"2H5* AMO"}="H5'2 AMO"; $resn_exch{" H4* AMO"}=" H4' AMO";
$resn_exch{" H1* AMO"}=" H1' AMO"; $resn_exch{"1H6  AMO"}=" H61 AMO"; $resn_exch{"2H6  AMO"}=" H62 AMO";
$resn_exch{"2H2* AMO"}="H2'2 AMO"; $resn_exch{" H3* AMO"}=" H3' AMO"; $resn_exch{"3HO* AMO"}="HO'3 AMO";
$resn_exch{"1HB  AMO"}=" HB1 AMO"; $resn_exch{"2HB  AMO"}=" HB2 AMO"; $resn_exch{"1H2* AMO"}="H2'1 AMO";
$resn_exch{"2HOP AMP"}="HOP2 AMP"; $resn_exch{"3HOP AMP"}="HOP3 AMP"; $resn_exch{"1H5* AMP"}="H5'1 AMP";
$resn_exch{"2H5* AMP"}="H5'2 AMP"; $resn_exch{" H4* AMP"}=" H4' AMP"; $resn_exch{" H3* AMP"}=" H3' AMP";
$resn_exch{"*HO3 AMP"}="HO3' AMP"; $resn_exch{" H2* AMP"}=" H2' AMP"; $resn_exch{"*HO2 AMP"}="HO2' AMP";
$resn_exch{" H1* AMP"}=" H1' AMP"; $resn_exch{"1HN6 AMP"}="HN61 AMP"; $resn_exch{"2HN6 AMP"}="HN62 AMP";
$resn_exch{"1HB  AMQ"}=" HB1 AMQ"; $resn_exch{"2HB  AMQ"}=" HB2 AMQ"; $resn_exch{"1HE2 AMQ"}="HE21 AMQ";
$resn_exch{"2HE2 AMQ"}="HE22 AMQ"; $resn_exch{"3HE2 AMQ"}="HE23 AMQ"; $resn_exch{"1HN7 AMR"}="HN71 AMR";
$resn_exch{"2HN7 AMR"}="HN72 AMR"; $resn_exch{"1HN4 AMR"}="HN41 AMR"; $resn_exch{"2HN4 AMR"}="HN42 AMR";
$resn_exch{"1HN2 AMR"}="HN21 AMR"; $resn_exch{"2HN2 AMR"}="HN22 AMR"; $resn_exch{"1HN1 AMS"}="HN11 AMS";
$resn_exch{"2HN1 AMS"}="HN12 AMS"; $resn_exch{"1HN2 AMS"}="HN21 AMS"; $resn_exch{"2HN2 AMS"}="HN22 AMS";
$resn_exch{"1HN  AMT"}=" HN1 AMT"; $resn_exch{"2HN  AMT"}=" HN2 AMT"; $resn_exch{"1H6  AMU"}=" H61 AMU";
$resn_exch{"2H6  AMU"}=" H62 AMU"; $resn_exch{"1H8  AMU"}=" H81 AMU"; $resn_exch{"2H8  AMU"}=" H82 AMU";
$resn_exch{"3H8  AMU"}=" H83 AMU"; $resn_exch{"1H11 AMU"}="H111 AMU"; $resn_exch{"2H11 AMU"}="H112 AMU";
$resn_exch{"3H11 AMU"}="H113 AMU"; $resn_exch{"1H6C AMV"}="H6C1 AMV"; $resn_exch{"2H6C AMV"}="H6C2 AMV";
$resn_exch{"1H8C AMV"}="H8C1 AMV"; $resn_exch{"2H8C AMV"}="H8C2 AMV"; $resn_exch{"3H8C AMV"}="H8C3 AMV";
$resn_exch{"1H11 AMV"}="H111 AMV"; $resn_exch{"2H11 AMV"}="H112 AMV"; $resn_exch{"3H11 AMV"}="H113 AMV";
$resn_exch{"1H12 AMV"}="H121 AMV"; $resn_exch{"2H12 AMV"}="H122 AMV"; $resn_exch{"3H12 AMV"}="H123 AMV";
$resn_exch{"2HOW AMW"}="HOW2 AMW"; $resn_exch{"3HOW AMW"}="HOW3 AMW"; $resn_exch{"1H5* AMW"}="H5'1 AMW";
$resn_exch{"2H5* AMW"}="H5'2 AMW"; $resn_exch{" H4* AMW"}=" H4' AMW"; $resn_exch{" H3* AMW"}=" H3' AMW";
$resn_exch{"*HO3 AMW"}="HO3' AMW"; $resn_exch{" H2* AMW"}=" H2' AMW"; $resn_exch{"*HO2 AMW"}="HO2' AMW";
$resn_exch{" H1* AMW"}=" H1' AMW"; $resn_exch{"1HN6 AMW"}="HN61 AMW"; $resn_exch{"2HN6 AMW"}="HN62 AMW";
$resn_exch{" AH2 AMX"}=" H2A AMX"; $resn_exch{"AH61 AMX"}="H61A AMX"; $resn_exch{"AH62 AMX"}="H62A AMX";
$resn_exch{" AH8 AMX"}=" H8A AMX"; $resn_exch{"AH1* AMX"}=" H1B AMX"; $resn_exch{"AH2* AMX"}=" H2B AMX";
$resn_exch{"AHO2 AMX"}="HO2A AMX"; $resn_exch{"AH3* AMX"}=" H3B AMX"; $resn_exch{"8HOA AMX"}="HOA8 AMX";
$resn_exch{"9HOA AMX"}="HOA9 AMX"; $resn_exch{"AH4* AMX"}=" H4B AMX"; $resn_exch{"AH51 AMX"}="H51A AMX";
$resn_exch{"AH52 AMX"}="H52A AMX"; $resn_exch{"2HOA AMX"}="HOA2 AMX"; $resn_exch{"5HOA AMX"}="HOA5 AMX";
$resn_exch{"1H12 AMX"}="H121 AMX"; $resn_exch{"2H12 AMX"}="H122 AMX"; $resn_exch{"1H13 AMX"}="H131 AMX";
$resn_exch{"2H13 AMX"}="H132 AMX"; $resn_exch{"3H13 AMX"}="H133 AMX"; $resn_exch{"1H14 AMX"}="H141 AMX";
$resn_exch{"2H14 AMX"}="H142 AMX"; $resn_exch{"3H14 AMX"}="H143 AMX"; $resn_exch{"1H7  AMX"}=" H71 AMX";
$resn_exch{"2H7  AMX"}=" H72 AMX"; $resn_exch{"1H6  AMX"}=" H61 AMX"; $resn_exch{"2H6  AMX"}=" H62 AMX";
$resn_exch{"1H3  AMX"}=" H31 AMX"; $resn_exch{"2H3  AMX"}=" H32 AMX"; $resn_exch{"1H2  AMX"}=" H21 AMX";
$resn_exch{"2H2  AMX"}=" H22 AMX"; $resn_exch{"1H1  AMX"}=" H11 AMX"; $resn_exch{"2H1  AMX"}=" H12 AMX";
$resn_exch{"1HN2 AMX"}="HN21 AMX"; $resn_exch{"2HN2 AMX"}="HN22 AMX"; $resn_exch{"1H15 AMY"}="H151 AMY";
$resn_exch{"2H15 AMY"}="H152 AMY"; $resn_exch{"1H16 AMY"}="H161 AMY"; $resn_exch{"2H16 AMY"}="H162 AMY";
$resn_exch{"1H17 AMY"}="H171 AMY"; $resn_exch{"2H17 AMY"}="H172 AMY"; $resn_exch{"1H18 AMY"}="H181 AMY";
$resn_exch{"2H18 AMY"}="H182 AMY"; $resn_exch{"1H19 AMY"}="H191 AMY"; $resn_exch{"2H19 AMY"}="H192 AMY";
$resn_exch{"3H19 AMY"}="H193 AMY"; $resn_exch{"1H22 AMY"}="H221 AMY"; $resn_exch{"2H22 AMY"}="H222 AMY";
$resn_exch{"1H24 AMY"}="H241 AMY"; $resn_exch{"2H24 AMY"}="H242 AMY"; $resn_exch{"3H24 AMY"}="H243 AMY";
$resn_exch{"1H25 AMY"}="H251 AMY"; $resn_exch{"2H25 AMY"}="H252 AMY"; $resn_exch{"3H25 AMY"}="H253 AMY";
$resn_exch{"1H26 AMY"}="H261 AMY"; $resn_exch{"2H26 AMY"}="H262 AMY"; $resn_exch{"3H26 AMY"}="H263 AMY";
$resn_exch{"1H27 AMY"}="H271 AMY"; $resn_exch{"2H27 AMY"}="H272 AMY"; $resn_exch{"3H27 AMY"}="H273 AMY";
$resn_exch{"1HB  AN0"}=" HB1 AN0"; $resn_exch{"2HB  AN0"}=" HB2 AN0"; $resn_exch{"1HG  AN0"}=" HG1 AN0";
$resn_exch{"2HG  AN0"}=" HG2 AN0"; $resn_exch{"1HD  AN0"}=" HD1 AN0"; $resn_exch{"2HD  AN0"}=" HD2 AN0";
$resn_exch{"3HD  AN0"}=" HD3 AN0"; $resn_exch{"1H2  AN0"}=" H21 AN0"; $resn_exch{"2H2  AN0"}=" H22 AN0";
$resn_exch{"3H2  AN0"}=" H23 AN0"; $resn_exch{"1H3B AN2"}="H3B1 AN2"; $resn_exch{"2H3B AN2"}="H3B2 AN2";
$resn_exch{"BHO2 AN2"}="HO2B AN2"; $resn_exch{"AHO2 AN2"}="HO2A AN2"; $resn_exch{"1H5* AN2"}="H5'1 AN2";
$resn_exch{"2H5* AN2"}="H5'2 AN2"; $resn_exch{" H4* AN2"}=" H4' AN2"; $resn_exch{" H3* AN2"}=" H3' AN2";
$resn_exch{" H2* AN2"}=" H2' AN2"; $resn_exch{" H1* AN2"}=" H1' AN2"; $resn_exch{"1H6N AN2"}="H6N1 AN2";
$resn_exch{"2H6N AN2"}="H6N2 AN2"; $resn_exch{"1H20 AN9"}="H201 AN9"; $resn_exch{"2H20 AN9"}="H202 AN9";
$resn_exch{"1H15 AN9"}="H151 AN9"; $resn_exch{"2H15 AN9"}="H152 AN9"; $resn_exch{"1H16 AN9"}="H161 AN9";
$resn_exch{"2H16 AN9"}="H162 AN9"; $resn_exch{"1H17 AN9"}="H171 AN9"; $resn_exch{"2H17 AN9"}="H172 AN9";
$resn_exch{"1H19 AN9"}="H191 AN9"; $resn_exch{"2H19 AN9"}="H192 AN9"; $resn_exch{"3H19 AN9"}="H193 AN9";
$resn_exch{"1H21 AN9"}="H211 AN9"; $resn_exch{"2H21 AN9"}="H212 AN9"; $resn_exch{"1H22 AN9"}="H221 AN9";
$resn_exch{"2H22 AN9"}="H222 AN9"; $resn_exch{"3H22 AN9"}="H223 AN9"; $resn_exch{"1H23 AN9"}="H231 AN9";
$resn_exch{"2H23 AN9"}="H232 AN9"; $resn_exch{"1H24 AN9"}="H241 AN9"; $resn_exch{"2H24 AN9"}="H242 AN9";
$resn_exch{"1H25 AN9"}="H251 AN9"; $resn_exch{"2H25 AN9"}="H252 AN9"; $resn_exch{"3H25 AN9"}="H253 AN9";
$resn_exch{"1H26 AN9"}="H261 AN9"; $resn_exch{"2H26 AN9"}="H262 AN9"; $resn_exch{"1H27 AN9"}="H271 AN9";
$resn_exch{"2H27 AN9"}="H272 AN9"; $resn_exch{"3H27 AN9"}="H273 AN9"; $resn_exch{"1H3  ANA"}=" H31 ANA";
$resn_exch{"2H3  ANA"}=" H32 ANA"; $resn_exch{"1H9  ANA"}=" H91 ANA"; $resn_exch{"2H9  ANA"}=" H92 ANA";
$resn_exch{"1H11 ANA"}="H111 ANA"; $resn_exch{"2H11 ANA"}="H112 ANA"; $resn_exch{"3H11 ANA"}="H113 ANA";
$resn_exch{"1H12 ANA"}="H121 ANA"; $resn_exch{"2H12 ANA"}="H122 ANA"; $resn_exch{"3H12 ANA"}="H123 ANA";
$resn_exch{"1HM4 ANA"}="HM41 ANA"; $resn_exch{"2HM4 ANA"}="HM42 ANA"; $resn_exch{"3HM4 ANA"}="HM43 ANA";
$resn_exch{"1HOB ANA"}="HOB1 ANA"; $resn_exch{"1HC  ANB"}=" HC1 ANB"; $resn_exch{"1H11 ANB"}="H111 ANB";
$resn_exch{"2H11 ANB"}="H112 ANB"; $resn_exch{"1H12 ANB"}="H121 ANB"; $resn_exch{"2H12 ANB"}="H122 ANB";
$resn_exch{"1HC4 ANB"}="HC41 ANB"; $resn_exch{"1H15 ANB"}="H151 ANB"; $resn_exch{"2H15 ANB"}="H152 ANB";
$resn_exch{"1H16 ANB"}="H161 ANB"; $resn_exch{"2H16 ANB"}="H162 ANB"; $resn_exch{"1H18 ANB"}="H181 ANB";
$resn_exch{"2H18 ANB"}="H182 ANB"; $resn_exch{"3H18 ANB"}="H183 ANB"; $resn_exch{"1H19 ANB"}="H191 ANB";
$resn_exch{"2H19 ANB"}="H192 ANB"; $resn_exch{"3H19 ANB"}="H193 ANB"; $resn_exch{"2HC  ANB"}=" HC2 ANB";
$resn_exch{"4HC  ANB"}=" HC4 ANB"; $resn_exch{"1HC6 ANB"}="HC61 ANB"; $resn_exch{"2HC6 ANB"}="HC62 ANB";
$resn_exch{"1HC7 ANB"}="HC71 ANB"; $resn_exch{"2HC7 ANB"}="HC72 ANB"; $resn_exch{"8HC  ANB"}=" HC8 ANB";
$resn_exch{"9HC  ANB"}=" HC9 ANB"; $resn_exch{"CH10 ANC"}="H10C ANC"; $resn_exch{"CH13 ANC"}="H13C ANC";
$resn_exch{"CH14 ANC"}="H14C ANC"; $resn_exch{"1H1  AND"}=" H11 AND"; $resn_exch{"2H1  AND"}=" H12 AND";
$resn_exch{"1H2  AND"}=" H21 AND"; $resn_exch{"2H2  AND"}=" H22 AND"; $resn_exch{"1H4  AND"}=" H41 AND";
$resn_exch{"2H4  AND"}=" H42 AND"; $resn_exch{"1H7  AND"}=" H71 AND"; $resn_exch{"2H7  AND"}=" H72 AND";
$resn_exch{"1H11 AND"}="H111 AND"; $resn_exch{"2H11 AND"}="H112 AND"; $resn_exch{"1H12 AND"}="H121 AND";
$resn_exch{"2H12 AND"}="H122 AND"; $resn_exch{"1H15 AND"}="H151 AND"; $resn_exch{"2H15 AND"}="H152 AND";
$resn_exch{"1H16 AND"}="H161 AND"; $resn_exch{"2H16 AND"}="H162 AND"; $resn_exch{"1H18 AND"}="H181 AND";
$resn_exch{"2H18 AND"}="H182 AND"; $resn_exch{"3H18 AND"}="H183 AND"; $resn_exch{"1H19 AND"}="H191 AND";
$resn_exch{"2H19 AND"}="H192 AND"; $resn_exch{"3H19 AND"}="H193 AND"; $resn_exch{"1HN6 ANE"}="HN61 ANE";
$resn_exch{"2HN6 ANE"}="HN62 ANE"; $resn_exch{"1H2  ANG"}=" H21 ANG"; $resn_exch{"2H2  ANG"}=" H22 ANG";
$resn_exch{"1H8  ANG"}=" H81 ANG"; $resn_exch{"2H8  ANG"}=" H82 ANG"; $resn_exch{"1H30 ANH"}="H301 ANH";
$resn_exch{"2H30 ANH"}="H302 ANH"; $resn_exch{"1H16 ANH"}="H161 ANH"; $resn_exch{"2H16 ANH"}="H162 ANH";
$resn_exch{"3H16 ANH"}="H163 ANH"; $resn_exch{"1H28 ANH"}="H281 ANH"; $resn_exch{"2H28 ANH"}="H282 ANH";
$resn_exch{"1H10 ANH"}="H101 ANH"; $resn_exch{"2H10 ANH"}="H102 ANH"; $resn_exch{"1H1  ANH"}=" H11 ANH";
$resn_exch{"2H1  ANH"}="H12A ANH"; $resn_exch{"3H1  ANH"}="H13A ANH"; $resn_exch{"1HN  ANI"}=" HN1 ANI";
$resn_exch{"2HN  ANI"}=" HN2 ANI"; $resn_exch{"1H11 ANJ"}="H111 ANJ"; $resn_exch{"2H11 ANJ"}="H112 ANJ";
$resn_exch{"3H11 ANJ"}="H113 ANJ"; $resn_exch{"1H16 ANJ"}="H161 ANJ"; $resn_exch{"2H16 ANJ"}="H162 ANJ";
$resn_exch{"3H16 ANJ"}="H163 ANJ"; $resn_exch{"1H18 ANJ"}="H181 ANJ"; $resn_exch{"2H18 ANJ"}="H182 ANJ";
$resn_exch{"1H19 ANJ"}="H191 ANJ"; $resn_exch{"2H19 ANJ"}="H192 ANJ"; $resn_exch{"1H20 ANJ"}="H201 ANJ";
$resn_exch{"2H20 ANJ"}="H202 ANJ"; $resn_exch{"1H21 ANJ"}="H211 ANJ"; $resn_exch{"2H21 ANJ"}="H212 ANJ";
$resn_exch{"1H22 ANJ"}="H221 ANJ"; $resn_exch{"2H22 ANJ"}="H222 ANJ"; $resn_exch{"1H23 ANJ"}="H231 ANJ";
$resn_exch{"2H23 ANJ"}="H232 ANJ"; $resn_exch{"3H23 ANJ"}="H233 ANJ"; $resn_exch{"1H26 ANJ"}="H261 ANJ";
$resn_exch{"2H26 ANJ"}="H262 ANJ"; $resn_exch{"3H26 ANJ"}="H263 ANJ"; $resn_exch{"1H27 ANJ"}="H271 ANJ";
$resn_exch{"2H27 ANJ"}="H272 ANJ"; $resn_exch{"1H28 ANJ"}="H281 ANJ"; $resn_exch{"2H28 ANJ"}="H282 ANJ";
$resn_exch{"3H28 ANJ"}="H283 ANJ"; $resn_exch{"1HN  ANL"}=" HN1 ANL"; $resn_exch{"2HN  ANL"}=" HN2 ANL";
$resn_exch{"1H4  ANM"}=" H41 ANM"; $resn_exch{"2H4  ANM"}=" H42 ANM"; $resn_exch{"1H6  ANM"}=" H61 ANM";
$resn_exch{"2H6  ANM"}=" H62 ANM"; $resn_exch{"3H6  ANM"}=" H63 ANM"; $resn_exch{"1H14 ANM"}="H141 ANM";
$resn_exch{"2H14 ANM"}="H142 ANM"; $resn_exch{"3H14 ANM"}="H143 ANM"; $resn_exch{"1H15 ANM"}="H151 ANM";
$resn_exch{"2H15 ANM"}="H152 ANM"; $resn_exch{"1H8  ANN"}=" H81 ANN"; $resn_exch{"2H8  ANN"}=" H82 ANN";
$resn_exch{"3H8  ANN"}=" H83 ANN"; $resn_exch{"1H1  ANO"}=" H11 ANO"; $resn_exch{"2H1  ANO"}=" H12 ANO";
$resn_exch{"1H2  ANO"}=" H21 ANO"; $resn_exch{"2H2  ANO"}=" H22 ANO"; $resn_exch{"1H4  ANO"}=" H41 ANO";
$resn_exch{"2H4  ANO"}=" H42 ANO"; $resn_exch{"1H6  ANO"}=" H61 ANO"; $resn_exch{"2H6  ANO"}=" H62 ANO";
$resn_exch{"1H7  ANO"}=" H71 ANO"; $resn_exch{"2H7  ANO"}=" H72 ANO"; $resn_exch{"1H11 ANO"}="H111 ANO";
$resn_exch{"2H11 ANO"}="H112 ANO"; $resn_exch{"1H12 ANO"}="H121 ANO"; $resn_exch{"2H12 ANO"}="H122 ANO";
$resn_exch{"1H15 ANO"}="H151 ANO"; $resn_exch{"2H15 ANO"}="H152 ANO"; $resn_exch{"1H16 ANO"}="H161 ANO";
$resn_exch{"2H16 ANO"}="H162 ANO"; $resn_exch{"1H18 ANO"}="H181 ANO"; $resn_exch{"2H18 ANO"}="H182 ANO";
$resn_exch{"3H18 ANO"}="H183 ANO"; $resn_exch{"1H19 ANO"}="H191 ANO"; $resn_exch{"2H19 ANO"}="H192 ANO";
$resn_exch{"3H19 ANO"}="H193 ANO"; $resn_exch{"2HOG ANP"}="HOG2 ANP"; $resn_exch{"3HOG ANP"}="HOG3 ANP";
$resn_exch{"2HOB ANP"}="HOB2 ANP"; $resn_exch{"1HNB ANP"}="HNB1 ANP"; $resn_exch{"2HOA ANP"}="HOA2 ANP";
$resn_exch{"1H5* ANP"}="H5'1 ANP"; $resn_exch{"2H5* ANP"}="H5'2 ANP"; $resn_exch{" H4* ANP"}=" H4' ANP";
$resn_exch{" H3* ANP"}=" H3' ANP"; $resn_exch{"*HO3 ANP"}="HO3' ANP"; $resn_exch{" H2* ANP"}=" H2' ANP";
$resn_exch{"*HO2 ANP"}="HO2' ANP"; $resn_exch{" H1* ANP"}=" H1' ANP"; $resn_exch{"1HN6 ANP"}="HN61 ANP";
$resn_exch{"2HN6 ANP"}="HN62 ANP"; $resn_exch{"1H15 ANR"}="H151 ANR"; $resn_exch{"2H15 ANR"}="H152 ANR";
$resn_exch{"1H20 ANR"}="H201 ANR"; $resn_exch{"2H20 ANR"}="H202 ANR"; $resn_exch{"3H20 ANR"}="H203 ANR";
$resn_exch{"1H19 ANR"}="H191 ANR"; $resn_exch{"2H19 ANR"}="H192 ANR"; $resn_exch{"3H19 ANR"}="H193 ANR";
$resn_exch{"1H17 ANR"}="H171 ANR"; $resn_exch{"2H17 ANR"}="H172 ANR"; $resn_exch{"3H17 ANR"}="H173 ANR";
$resn_exch{"1H18 ANR"}="H181 ANR"; $resn_exch{"2H18 ANR"}="H182 ANR"; $resn_exch{"3H18 ANR"}="H183 ANR";
$resn_exch{"1H2  ANR"}=" H21 ANR"; $resn_exch{"2H2  ANR"}=" H22 ANR"; $resn_exch{"1H3  ANR"}=" H31 ANR";
$resn_exch{"2H3  ANR"}=" H32 ANR"; $resn_exch{"1H16 ANR"}="H161 ANR"; $resn_exch{"2H16 ANR"}="H162 ANR";
$resn_exch{"3H16 ANR"}="H163 ANR"; $resn_exch{"1HM1 ANS"}="HM11 ANS"; $resn_exch{"2HM1 ANS"}="HM12 ANS";
$resn_exch{"3HM1 ANS"}="HM13 ANS"; $resn_exch{"1HM2 ANS"}="HM21 ANS"; $resn_exch{"2HM2 ANS"}="HM22 ANS";
$resn_exch{"3HM2 ANS"}="HM23 ANS"; $resn_exch{"3HOS ANS"}="HOS3 ANS"; $resn_exch{"1H1  ANT"}="H11A ANT";
$resn_exch{"2H1  ANT"}="H12A ANT"; $resn_exch{"1HM8 ANT"}="HM81 ANT"; $resn_exch{"2HM8 ANT"}="HM82 ANT";
$resn_exch{"3HM8 ANT"}="HM83 ANT"; $resn_exch{"1HN5 ANT"}="HN51 ANT"; $resn_exch{"2HN5 ANT"}="HN52 ANT";
$resn_exch{" H3* ANU"}=" H3' ANU"; $resn_exch{" H2* ANU"}=" H2' ANU"; $resn_exch{" H1* ANU"}=" H1' ANU";
$resn_exch{" H4* ANU"}=" H4' ANU"; $resn_exch{"1H5* ANU"}="H5'1 ANU"; $resn_exch{"2H5* ANU"}="H5'2 ANU";
$resn_exch{" H5* ANU"}=" H5' ANU"; $resn_exch{"1H15 ANY"}="H151 ANY"; $resn_exch{"2H15 ANY"}="H152 ANY";
$resn_exch{"1H16 ANY"}="H161 ANY"; $resn_exch{"2H16 ANY"}="H162 ANY"; $resn_exch{"1H26 ANY"}="H261 ANY";
$resn_exch{"2H26 ANY"}="H262 ANY"; $resn_exch{"3H26 ANY"}="H263 ANY"; $resn_exch{"1H27 ANY"}="H271 ANY";
$resn_exch{"2H27 ANY"}="H272 ANY"; $resn_exch{"3H27 ANY"}="H273 ANY"; $resn_exch{"1H23 ANY"}="H231 ANY";
$resn_exch{"2H23 ANY"}="H232 ANY"; $resn_exch{"1H17 ANY"}="H171 ANY"; $resn_exch{"2H17 ANY"}="H172 ANY";
$resn_exch{"1H24 ANY"}="H241 ANY"; $resn_exch{"2H24 ANY"}="H242 ANY"; $resn_exch{"3H24 ANY"}="H243 ANY";
$resn_exch{"1H25 ANY"}="H251 ANY"; $resn_exch{"2H25 ANY"}="H252 ANY"; $resn_exch{"3H25 ANY"}="H253 ANY";
$resn_exch{"1H18 ANY"}="H181 ANY"; $resn_exch{"2H18 ANY"}="H182 ANY"; $resn_exch{"1H19 ANY"}="H191 ANY";
$resn_exch{"2H19 ANY"}="H192 ANY"; $resn_exch{"1H28 ANY"}="H281 ANY"; $resn_exch{"2H28 ANY"}="H282 ANY";
$resn_exch{"1H29 ANY"}="H291 ANY"; $resn_exch{"2H29 ANY"}="H292 ANY"; $resn_exch{"3H29 ANY"}="H293 ANY";
$resn_exch{"1HBC ANZ"}="HBC1 ANZ"; $resn_exch{"2HBC ANZ"}="HBC2 ANZ"; $resn_exch{" H1* ANZ"}=" H1' ANZ";
$resn_exch{"1H6N ANZ"}="H6N1 ANZ"; $resn_exch{"2H6N ANZ"}="H6N2 ANZ"; $resn_exch{"1H5* ANZ"}="H5'1 ANZ";
$resn_exch{"2H5* ANZ"}="H5'2 ANZ"; $resn_exch{" H4* ANZ"}=" H4' ANZ"; $resn_exch{" H3* ANZ"}=" H3' ANZ";
$resn_exch{" H2* ANZ"}=" H2' ANZ"; $resn_exch{"1H1  AO1"}="H11A AO1"; $resn_exch{"2H1  AO1"}=" H12 AO1";
$resn_exch{"3H1  AO1"}=" H13 AO1"; $resn_exch{"1H2  AO1"}="H21A AO1"; $resn_exch{"2H2  AO1"}="H22A AO1";
$resn_exch{"1H4  AO1"}=" H41 AO1"; $resn_exch{"2H4  AO1"}=" H42 AO1"; $resn_exch{"1H5  AO1"}=" H51 AO1";
$resn_exch{"2H5  AO1"}=" H52 AO1"; $resn_exch{"1H14 AO1"}="H141 AO1"; $resn_exch{"2H14 AO1"}="H142 AO1";
$resn_exch{"3H14 AO1"}="H143 AO1"; $resn_exch{"1HN7 AO1"}="HN71 AO1"; $resn_exch{"2HN7 AO1"}="HN72 AO1";
$resn_exch{"1H1  AO2"}="H11A AO2"; $resn_exch{"2H1  AO2"}=" H12 AO2"; $resn_exch{"1H2  AO2"}="H21A AO2";
$resn_exch{"2H2  AO2"}=" H22 AO2"; $resn_exch{"1H3  AO2"}=" H31 AO2"; $resn_exch{"2H3  AO2"}=" H32 AO2";
$resn_exch{"1H4  AO2"}=" H41 AO2"; $resn_exch{"2H4  AO2"}=" H42 AO2"; $resn_exch{"1H5  AO2"}=" H51 AO2";
$resn_exch{"2H5  AO2"}=" H52 AO2"; $resn_exch{"1H7  AO2"}=" H71 AO2"; $resn_exch{"2H7  AO2"}=" H72 AO2";
$resn_exch{"1HN9 AO2"}="HN91 AO2"; $resn_exch{"2HN9 AO2"}="HN92 AO2"; $resn_exch{"1H22 AO2"}="H221 AO2";
$resn_exch{"2H22 AO2"}="H222 AO2"; $resn_exch{"3H22 AO2"}="H223 AO2"; $resn_exch{"1H6  AO3"}=" H61 AO3";
$resn_exch{"2H6  AO3"}=" H62 AO3"; $resn_exch{"1H8  AO3"}=" H81 AO3"; $resn_exch{"2H8  AO3"}=" H82 AO3";
$resn_exch{"3H8  AO3"}=" H83 AO3"; $resn_exch{"1H16 AO3"}="H161 AO3"; $resn_exch{"2H16 AO3"}="H162 AO3";
$resn_exch{"1H18 AO3"}="H181 AO3"; $resn_exch{"2H18 AO3"}="H182 AO3"; $resn_exch{"3H18 AO3"}="H183 AO3";
$resn_exch{"1H26 AO3"}="H261 AO3"; $resn_exch{"2H26 AO3"}="H262 AO3"; $resn_exch{"1H28 AO3"}="H281 AO3";
$resn_exch{"2H28 AO3"}="H282 AO3"; $resn_exch{"3H28 AO3"}="H283 AO3"; $resn_exch{"1H29 AO3"}="H291 AO3";
$resn_exch{"2H29 AO3"}="H292 AO3"; $resn_exch{"3H29 AO3"}="H293 AO3"; $resn_exch{"1H2  AO5"}=" H21 AO5";
$resn_exch{"2H2  AO5"}="H22A AO5"; $resn_exch{"3H2  AO5"}=" H23 AO5"; $resn_exch{"1H3  AO5"}=" H31 AO5";
$resn_exch{"2H3  AO5"}=" H32 AO5"; $resn_exch{"3H3  AO5"}=" H33 AO5"; $resn_exch{"1H5  AO5"}=" H51 AO5";
$resn_exch{"2H5  AO5"}=" H52 AO5"; $resn_exch{"1H6  AO5"}=" H61 AO5"; $resn_exch{"2H6  AO5"}=" H62 AO5";
$resn_exch{"1HN8 AO5"}="HN81 AO5"; $resn_exch{"2HN8 AO5"}="HN82 AO5"; $resn_exch{"1H1C AOA"}="H1C1 AOA";
$resn_exch{"2H1C AOA"}="H1C2 AOA"; $resn_exch{"1H1N AOA"}="H1N1 AOA"; $resn_exch{"2H1N AOA"}="H1N2 AOA";
$resn_exch{"1HC  AOE"}=" HC1 AOE"; $resn_exch{"2HC  AOE"}=" HC2 AOE"; $resn_exch{"3HO  AOE"}=" HO3 AOE";
$resn_exch{"4HC  AOE"}=" HC4 AOE"; $resn_exch{"1HC6 AOE"}="HC61 AOE"; $resn_exch{"2HC6 AOE"}="HC62 AOE";
$resn_exch{"7HC  AOE"}=" HC7 AOE"; $resn_exch{"8HC  AOE"}=" HC8 AOE"; $resn_exch{"9HC  AOE"}=" HC9 AOE";
$resn_exch{"1H11 AOE"}="H111 AOE"; $resn_exch{"2H11 AOE"}="H112 AOE"; $resn_exch{"1H12 AOE"}="H121 AOE";
$resn_exch{"2H12 AOE"}="H122 AOE"; $resn_exch{"1HC4 AOE"}="HC41 AOE"; $resn_exch{"1H15 AOE"}="H151 AOE";
$resn_exch{"2H15 AOE"}="H152 AOE"; $resn_exch{"1H16 AOE"}="H161 AOE"; $resn_exch{"2H16 AOE"}="H162 AOE";
$resn_exch{"1HC7 AOE"}="HC71 AOE"; $resn_exch{"1H18 AOE"}="H181 AOE"; $resn_exch{"2H18 AOE"}="H182 AOE";
$resn_exch{"3H18 AOE"}="H183 AOE"; $resn_exch{"1HO7 AOE"}="HO71 AOE"; $resn_exch{"1H19 AOE"}="H191 AOE";
$resn_exch{"2H19 AOE"}="H192 AOE"; $resn_exch{"1H20 AOE"}="H201 AOE"; $resn_exch{"2H20 AOE"}="H202 AOE";
$resn_exch{"1H21 AOE"}="H211 AOE"; $resn_exch{"2H21 AOE"}="H212 AOE"; $resn_exch{"1H22 AOE"}="H221 AOE";
$resn_exch{"2H22 AOE"}="H222 AOE"; $resn_exch{"1H23 AOE"}="H231 AOE"; $resn_exch{"2H23 AOE"}="H232 AOE";
$resn_exch{"1H24 AOE"}="H241 AOE"; $resn_exch{"2H24 AOE"}="H242 AOE"; $resn_exch{"1H25 AOE"}="H251 AOE";
$resn_exch{"2H25 AOE"}="H252 AOE"; $resn_exch{"1H26 AOE"}="H261 AOE"; $resn_exch{"2H26 AOE"}="H262 AOE";
$resn_exch{"1H27 AOE"}="H271 AOE"; $resn_exch{"2H27 AOE"}="H272 AOE"; $resn_exch{"1H28 AOE"}="H281 AOE";
$resn_exch{"2H28 AOE"}="H282 AOE"; $resn_exch{"1H30 AOE"}="H301 AOE"; $resn_exch{"2H30 AOE"}="H302 AOE";
$resn_exch{"3H30 AOE"}="H303 AOE"; $resn_exch{"1H31 AOE"}="H311 AOE"; $resn_exch{"2H31 AOE"}="H312 AOE";
$resn_exch{"1H32 AOE"}="H321 AOE"; $resn_exch{"2H32 AOE"}="H322 AOE"; $resn_exch{"1H33 AOE"}="H331 AOE";
$resn_exch{"2H33 AOE"}="H332 AOE"; $resn_exch{"1H34 AOE"}="H341 AOE"; $resn_exch{"2H34 AOE"}="H342 AOE";
$resn_exch{"3H34 AOE"}="H343 AOE"; $resn_exch{"1H6  AOG"}=" H61 AOG"; $resn_exch{"2H6  AOG"}=" H62 AOG";
$resn_exch{"1HN3 AOG"}="HN31 AOG"; $resn_exch{"2HN3 AOG"}="HN32 AOG"; $resn_exch{"1H1' AOG"}="H1'1 AOG";
$resn_exch{"2H1' AOG"}="H1'2 AOG"; $resn_exch{"1H2' AOG"}="H2'1 AOG"; $resn_exch{"2H2' AOG"}="H2'2 AOG";
$resn_exch{"1H3' AOG"}="H3'1 AOG"; $resn_exch{"2H3' AOG"}="H3'2 AOG"; $resn_exch{"1H4' AOG"}="H4'1 AOG";
$resn_exch{"2H4' AOG"}="H4'2 AOG"; $resn_exch{"1H5' AOG"}="H5'1 AOG"; $resn_exch{"2H5' AOG"}="H5'2 AOG";
$resn_exch{"1H6' AOG"}="H6'1 AOG"; $resn_exch{"2H6' AOG"}="H6'2 AOG"; $resn_exch{"1H7' AOG"}="H7'1 AOG";
$resn_exch{"2H7' AOG"}="H7'2 AOG"; $resn_exch{"1H8' AOG"}="H8'1 AOG"; $resn_exch{"2H8' AOG"}="H8'2 AOG";
$resn_exch{"3H8' AOG"}="H8'3 AOG"; $resn_exch{"1HN5 AOL"}="HN51 AOL"; $resn_exch{"2HN5 AOL"}="HN52 AOL";
$resn_exch{"1HC1 AOM"}="HC11 AOM"; $resn_exch{"2HC1 AOM"}="HC12 AOM"; $resn_exch{"1H11 AOM"}="H111 AOM";
$resn_exch{"2H11 AOM"}="H112 AOM"; $resn_exch{"1H12 AOM"}="H121 AOM"; $resn_exch{"2H12 AOM"}="H122 AOM";
$resn_exch{"CH14 AOM"}="H14C AOM"; $resn_exch{"1H15 AOM"}="H151 AOM"; $resn_exch{"2H15 AOM"}="H152 AOM";
$resn_exch{"1H16 AOM"}="H161 AOM"; $resn_exch{"2H16 AOM"}="H162 AOM"; $resn_exch{"CH17 AOM"}="H17C AOM";
$resn_exch{"1H18 AOM"}="H181 AOM"; $resn_exch{"2H18 AOM"}="H182 AOM"; $resn_exch{"3H18 AOM"}="H183 AOM";
$resn_exch{"1H19 AOM"}="H191 AOM"; $resn_exch{"2H19 AOM"}="H192 AOM"; $resn_exch{"3H19 AOM"}="H193 AOM";
$resn_exch{"1HC2 AOM"}="HC21 AOM"; $resn_exch{"2HC2 AOM"}="HC22 AOM"; $resn_exch{"1HC4 AOM"}="HC41 AOM";
$resn_exch{"2HC4 AOM"}="HC42 AOM"; $resn_exch{"1HC6 AOM"}="HC61 AOM"; $resn_exch{"2HC6 AOM"}="HC62 AOM";
$resn_exch{"1HC7 AOM"}="HC71 AOM"; $resn_exch{"2HC7 AOM"}="HC72 AOM"; $resn_exch{"1HC1 AON"}="HC11 AON";
$resn_exch{"2HC1 AON"}="HC12 AON"; $resn_exch{"1H11 AON"}="H111 AON"; $resn_exch{"2H11 AON"}="H112 AON";
$resn_exch{"1H12 AON"}="H121 AON"; $resn_exch{"2H12 AON"}="H122 AON"; $resn_exch{"CH14 AON"}="H14C AON";
$resn_exch{"1H15 AON"}="H151 AON"; $resn_exch{"2H15 AON"}="H152 AON"; $resn_exch{"1H16 AON"}="H161 AON";
$resn_exch{"2H16 AON"}="H162 AON"; $resn_exch{"CH17 AON"}="H17C AON"; $resn_exch{"1H18 AON"}="H181 AON";
$resn_exch{"2H18 AON"}="H182 AON"; $resn_exch{"3H18 AON"}="H183 AON"; $resn_exch{"1H19 AON"}="H191 AON";
$resn_exch{"2H19 AON"}="H192 AON"; $resn_exch{"3H19 AON"}="H193 AON"; $resn_exch{"1HC2 AON"}="HC21 AON";
$resn_exch{"2HC2 AON"}="HC22 AON"; $resn_exch{"1HC4 AON"}="HC41 AON"; $resn_exch{"2HC4 AON"}="HC42 AON";
$resn_exch{"1HC6 AON"}="HC61 AON"; $resn_exch{"2HC6 AON"}="HC62 AON"; $resn_exch{"1HC7 AON"}="HC71 AON";
$resn_exch{"2HC7 AON"}="HC72 AON"; $resn_exch{"1H2  AOP"}=" H21 AOP"; $resn_exch{"2H2  AOP"}=" H22 AOP";
$resn_exch{"1H5  AOP"}=" H51 AOP"; $resn_exch{"2H5  AOP"}=" H52 AOP"; $resn_exch{"1H6  AOP"}=" H61 AOP";
$resn_exch{"2H6  AOP"}=" H62 AOP"; $resn_exch{"1H7  AOP"}=" H71 AOP"; $resn_exch{"2H7  AOP"}=" H72 AOP";
$resn_exch{"1H8  AOP"}=" H81 AOP"; $resn_exch{"2H8  AOP"}=" H82 AOP"; $resn_exch{"1H9  AOP"}=" H91 AOP";
$resn_exch{"2H9  AOP"}=" H92 AOP"; $resn_exch{"3H9  AOP"}=" H93 AOP"; $resn_exch{"1HB  AOR"}=" HB1 AOR";
$resn_exch{"2HB  AOR"}=" HB2 AOR"; $resn_exch{"1HG  AOR"}=" HG1 AOR"; $resn_exch{"2HG  AOR"}=" HG2 AOR";
$resn_exch{"1HD  AOR"}=" HD1 AOR"; $resn_exch{"2HD  AOR"}=" HD2 AOR"; $resn_exch{"1HNE AOR"}="HNE1 AOR";
$resn_exch{"2HNE AOR"}="HNE2 AOR"; $resn_exch{"1H2  AOR"}=" H21 AOR"; $resn_exch{"2H2  AOR"}=" H22 AOR";
$resn_exch{"3H2  AOR"}=" H23 AOR"; $resn_exch{"1H6  AOS"}=" H61 AOS"; $resn_exch{"2H6  AOS"}=" H62 AOS";
$resn_exch{"1HN   AP"}=" HN1  AP"; $resn_exch{"2HN   AP"}=" HN2  AP"; $resn_exch{"1H8  AP0"}=" H81 AP0";
$resn_exch{"2H8  AP0"}=" H82 AP0"; $resn_exch{"3H8  AP0"}=" H83 AP0"; $resn_exch{"1H4  AP0"}=" H41 AP0";
$resn_exch{"2H4  AP0"}=" H42 AP0"; $resn_exch{" H1* AP0"}=" H1' AP0"; $resn_exch{" H4* AP0"}=" H4' AP0";
$resn_exch{"1H5* AP0"}="H5'1 AP0"; $resn_exch{"2H5* AP0"}="H5'2 AP0"; $resn_exch{"1H1  AP0"}=" H11 AP0";
$resn_exch{"2H1  AP0"}="H12X AP0"; $resn_exch{" H2* AP0"}=" H2' AP0"; $resn_exch{" H3* AP0"}=" H3' AP0";
$resn_exch{"1HN6 AP0"}="HN61 AP0"; $resn_exch{"2HN6 AP0"}="HN62 AP0"; $resn_exch{"H11  AP0"}="H111 AP0";
$resn_exch{"1H3  AP1"}=" H31 AP1"; $resn_exch{"2H3  AP1"}=" H32 AP1"; $resn_exch{"1H4  AP1"}="H41A AP1";
$resn_exch{"2H4  AP1"}=" H42 AP1"; $resn_exch{"1H5  AP1"}=" H51 AP1"; $resn_exch{"2H5  AP1"}=" H52 AP1";
$resn_exch{"1H6  AP1"}=" H61 AP1"; $resn_exch{"2H6  AP1"}=" H62 AP1"; $resn_exch{"1H16 AP1"}="H161 AP1";
$resn_exch{"2H16 AP1"}="H162 AP1"; $resn_exch{"3H16 AP1"}="H163 AP1"; $resn_exch{"1H18 AP1"}="H181 AP1";
$resn_exch{"2H18 AP1"}="H182 AP1"; $resn_exch{"3H18 AP1"}="H183 AP1"; $resn_exch{"1H19 AP1"}="H191 AP1";
$resn_exch{"2H19 AP1"}="H192 AP1"; $resn_exch{"3H19 AP1"}="H193 AP1"; $resn_exch{"1H20 AP1"}="H201 AP1";
$resn_exch{"2H20 AP1"}="H202 AP1"; $resn_exch{"3H20 AP1"}="H203 AP1"; $resn_exch{"1H21 AP1"}="H211 AP1";
$resn_exch{"2H21 AP1"}="H212 AP1"; $resn_exch{"3H21 AP1"}="H213 AP1"; $resn_exch{"1H27 AP1"}="H271 AP1";
$resn_exch{"2H27 AP1"}="H272 AP1"; $resn_exch{"1H28 AP1"}="H281 AP1"; $resn_exch{"2H28 AP1"}="H282 AP1";
$resn_exch{"1H37 AP1"}="H371 AP1"; $resn_exch{"2H37 AP1"}="H372 AP1"; $resn_exch{"1H42 AP1"}="H421 AP1";
$resn_exch{"2H42 AP1"}="H422 AP1"; $resn_exch{"3H42 AP1"}="H423 AP1"; $resn_exch{"1H46 AP1"}="H461 AP1";
$resn_exch{"2H46 AP1"}="H462 AP1"; $resn_exch{"2HOB AP2"}="HOB2 AP2"; $resn_exch{"3HOB AP2"}="HOB3 AP2";
$resn_exch{"1H3A AP2"}="H3A1 AP2"; $resn_exch{"2H3A AP2"}="H3A2 AP2"; $resn_exch{"2HOA AP2"}="HOA2 AP2";
$resn_exch{"1H5* AP2"}="H5'1 AP2"; $resn_exch{"2H5* AP2"}="H5'2 AP2"; $resn_exch{" H4* AP2"}=" H4' AP2";
$resn_exch{" H3* AP2"}=" H3' AP2"; $resn_exch{"*HO3 AP2"}="HO3' AP2"; $resn_exch{" H2* AP2"}=" H2' AP2";
$resn_exch{"*HO2 AP2"}="HO2' AP2"; $resn_exch{" H1* AP2"}=" H1' AP2"; $resn_exch{"1HN6 AP2"}="HN61 AP2";
$resn_exch{"2HN6 AP2"}="HN62 AP2"; $resn_exch{"1H51 AP3"}="H511 AP3"; $resn_exch{"2H51 AP3"}="H512 AP3";
$resn_exch{"3H51 AP3"}="H513 AP3"; $resn_exch{"1H52 AP3"}="H521 AP3"; $resn_exch{"2H52 AP3"}="H522 AP3";
$resn_exch{"3H52 AP3"}="H523 AP3"; $resn_exch{"1H9  AP3"}=" H91 AP3"; $resn_exch{"2H9  AP3"}=" H92 AP3";
$resn_exch{"3H9  AP3"}=" H93 AP3"; $resn_exch{"1H10 AP3"}="H101 AP3"; $resn_exch{"2H10 AP3"}="H102 AP3";
$resn_exch{"3H10 AP3"}="H103 AP3"; $resn_exch{"1HN2 AP4"}="HN21 AP4"; $resn_exch{"2HN2 AP4"}="HN22 AP4";
$resn_exch{"1H7  AP4"}=" H71 AP4"; $resn_exch{"2H7  AP4"}=" H72 AP4"; $resn_exch{"1H11 AP4"}="H111 AP4";
$resn_exch{"2H11 AP4"}="H112 AP4"; $resn_exch{"2HOA AP5"}="HOA2 AP5"; $resn_exch{"2HOB AP5"}="HOB2 AP5";
$resn_exch{"2HOG AP5"}="HOG2 AP5"; $resn_exch{"2HOD AP5"}="HOD2 AP5"; $resn_exch{"2HOE AP5"}="HOE2 AP5";
$resn_exch{"AH51 AP5"}="H51A AP5"; $resn_exch{"AH52 AP5"}="H52A AP5"; $resn_exch{"AH4* AP5"}=" H4F AP5";
$resn_exch{"AH3* AP5"}=" H3F AP5"; $resn_exch{"AHO3 AP5"}="HO3A AP5"; $resn_exch{"AH2* AP5"}=" H2F AP5";
$resn_exch{"AHO2 AP5"}="HO2A AP5"; $resn_exch{"AH1* AP5"}=" H1F AP5"; $resn_exch{" AH8 AP5"}=" H8A AP5";
$resn_exch{"AH61 AP5"}="H61A AP5"; $resn_exch{"AH62 AP5"}="H62A AP5"; $resn_exch{" AH2 AP5"}=" H2A AP5";
$resn_exch{"BH51 AP5"}="H51B AP5"; $resn_exch{"BH52 AP5"}="H52B AP5"; $resn_exch{"BH4* AP5"}=" H4J AP5";
$resn_exch{"BH3* AP5"}=" H3J AP5"; $resn_exch{"BHO3 AP5"}="HO3B AP5"; $resn_exch{"BH2* AP5"}=" H2J AP5";
$resn_exch{"BHO2 AP5"}="HO2B AP5"; $resn_exch{"BH1* AP5"}=" H1J AP5"; $resn_exch{" BH8 AP5"}=" H8B AP5";
$resn_exch{"BH61 AP5"}="H61B AP5"; $resn_exch{"BH62 AP5"}="H62B AP5"; $resn_exch{" BH2 AP5"}=" H2B AP5";
$resn_exch{"1HN2 AP6"}="HN21 AP6"; $resn_exch{"2HN2 AP6"}="HN22 AP6"; $resn_exch{"1HN4 AP6"}="HN41 AP6";
$resn_exch{"2HN4 AP6"}="HN42 AP6"; $resn_exch{"1H7  AP6"}=" H71 AP6"; $resn_exch{"2H7  AP6"}=" H72 AP6";
$resn_exch{" H1* AP7"}=" H1' AP7"; $resn_exch{"1H6  AP7"}=" H61 AP7"; $resn_exch{"2H6  AP7"}=" H62 AP7";
$resn_exch{" H2* AP7"}=" H2' AP7"; $resn_exch{"2HO* AP7"}="HO2' AP7"; $resn_exch{" H3* AP7"}=" H3' AP7";
$resn_exch{"H3T  AP7"}="HO3' AP7"; $resn_exch{" H4* AP7"}=" H4' AP7"; $resn_exch{"2H5* AP7"}="H5'' AP7";
$resn_exch{"1H5* AP7"}=" H5' AP7"; $resn_exch{"2HOP AP7"}="HOP2 AP7"; $resn_exch{"3HOP AP7"}="HOP3 AP7";
$resn_exch{"1H56 AP8"}="H561 AP8"; $resn_exch{"2H56 AP8"}="H562 AP8"; $resn_exch{"3H57 AP8"}="H573 AP8";
$resn_exch{"2H57 AP8"}="H572 AP8"; $resn_exch{"1H57 AP8"}="H571 AP8"; $resn_exch{"3H59 AP8"}="H593 AP8";
$resn_exch{"2H59 AP8"}="H592 AP8"; $resn_exch{"1H59 AP8"}="H591 AP8"; $resn_exch{"3H58 AP8"}="H583 AP8";
$resn_exch{"2H58 AP8"}="H582 AP8"; $resn_exch{"1H58 AP8"}="H581 AP8"; $resn_exch{"3H35 AP8"}="H353 AP8";
$resn_exch{"2H35 AP8"}="H352 AP8"; $resn_exch{"1H35 AP8"}="H351 AP8"; $resn_exch{"3H36 AP8"}="H363 AP8";
$resn_exch{"2H36 AP8"}="H362 AP8"; $resn_exch{"1H36 AP8"}="H361 AP8"; $resn_exch{"1H11 AP8"}="H111 AP8";
$resn_exch{"2H11 AP8"}="H112 AP8"; $resn_exch{"1H12 AP8"}="H121 AP8"; $resn_exch{"2H12 AP8"}="H122 AP8";
$resn_exch{"3H53 AP8"}="H533 AP8"; $resn_exch{"2H53 AP8"}="H532 AP8"; $resn_exch{"1H53 AP8"}="H531 AP8";
$resn_exch{"3H37 AP8"}="H373 AP8"; $resn_exch{"2H37 AP8"}="H372 AP8"; $resn_exch{"1H37 AP8"}="H371 AP8";
$resn_exch{"1H16 AP8"}="H161 AP8"; $resn_exch{"2H16 AP8"}="H162 AP8"; $resn_exch{"3H38 AP8"}="H383 AP8";
$resn_exch{"2H38 AP8"}="H382 AP8"; $resn_exch{"1H38 AP8"}="H381 AP8"; $resn_exch{"1H18 AP8"}="H181 AP8";
$resn_exch{"2H18 AP8"}="H182 AP8"; $resn_exch{"3H52 AP8"}="H523 AP8"; $resn_exch{"2H52 AP8"}="H522 AP8";
$resn_exch{"1H52 AP8"}="H521 AP8"; $resn_exch{"1H22 AP8"}="H221 AP8"; $resn_exch{"2H22 AP8"}="H222 AP8";
$resn_exch{"3H39 AP8"}="H393 AP8"; $resn_exch{"2H39 AP8"}="H392 AP8"; $resn_exch{"1H39 AP8"}="H391 AP8";
$resn_exch{"3H40 AP8"}="H403 AP8"; $resn_exch{"2H40 AP8"}="H402 AP8"; $resn_exch{"1H40 AP8"}="H401 AP8";
$resn_exch{"1H27 AP8"}="H271 AP8"; $resn_exch{"2H27 AP8"}="H272 AP8"; $resn_exch{"1H28 AP8"}="H281 AP8";
$resn_exch{"2H28 AP8"}="H282 AP8"; $resn_exch{"3H49 AP8"}="H493 AP8"; $resn_exch{"2H49 AP8"}="H492 AP8";
$resn_exch{"1H49 AP8"}="H491 AP8"; $resn_exch{"3H51 AP8"}="H513 AP8"; $resn_exch{"2H51 AP8"}="H512 AP8";
$resn_exch{"1H51 AP8"}="H511 AP8"; $resn_exch{"3H50 AP8"}="H503 AP8"; $resn_exch{"2H50 AP8"}="H502 AP8";
$resn_exch{"1H50 AP8"}="H501 AP8"; $resn_exch{"3H41 AP8"}="H413 AP8"; $resn_exch{"2H41 AP8"}="H412 AP8";
$resn_exch{"1H41 AP8"}="H411 AP8"; $resn_exch{"2H46 AP8"}="H462 AP8"; $resn_exch{"1H46 AP8"}="H461 AP8";
$resn_exch{"3H42 AP8"}="H423 AP8"; $resn_exch{"2H42 AP8"}="H422 AP8"; $resn_exch{"1H42 AP8"}="H421 AP8";
$resn_exch{"3H44 AP8"}="H443 AP8"; $resn_exch{"2H44 AP8"}="H442 AP8"; $resn_exch{"1H44 AP8"}="H441 AP8";
$resn_exch{"1H15 AP9"}="H151 AP9"; $resn_exch{"2H15 AP9"}="H152 AP9"; $resn_exch{"1H14 AP9"}="H141 AP9";
$resn_exch{"2H14 AP9"}="H142 AP9"; $resn_exch{"3H14 AP9"}="H143 AP9"; $resn_exch{"1H16 AP9"}="H161 AP9";
$resn_exch{"2H16 AP9"}="H162 AP9"; $resn_exch{"3H16 AP9"}="H163 AP9"; $resn_exch{"1H10 AP9"}="H101 AP9";
$resn_exch{"2H10 AP9"}="H102 AP9"; $resn_exch{"3H10 AP9"}="H103 AP9"; $resn_exch{"1H11 AP9"}="H111 AP9";
$resn_exch{"2H11 AP9"}="H112 AP9"; $resn_exch{"3H11 AP9"}="H113 AP9"; $resn_exch{"1H1  AP9"}=" H11 AP9";
$resn_exch{"2H1  AP9"}="H12A AP9"; $resn_exch{"1H3  APA"}=" H31 APA"; $resn_exch{"2H3  APA"}=" H32 APA";
$resn_exch{"1HN1 APA"}="HN11 APA"; $resn_exch{"2HN1 APA"}="HN12 APA"; $resn_exch{"1HN1 APB"}="HN11 APB";
$resn_exch{"2HN1 APB"}="HN12 APB"; $resn_exch{"2HOG APC"}="HOG2 APC"; $resn_exch{"3HOG APC"}="HOG3 APC";
$resn_exch{"2HOB APC"}="HOB2 APC"; $resn_exch{"2HOA APC"}="HOA2 APC"; $resn_exch{"1H3A APC"}="H3A1 APC";
$resn_exch{"2H3A APC"}="H3A2 APC"; $resn_exch{"1H5* APC"}="H5'1 APC"; $resn_exch{"2H5* APC"}="H5'2 APC";
$resn_exch{" H4* APC"}=" H4' APC"; $resn_exch{" H3* APC"}=" H3' APC"; $resn_exch{"*HO3 APC"}="HO3' APC";
$resn_exch{" H2* APC"}=" H2' APC"; $resn_exch{"*HO2 APC"}="HO2' APC"; $resn_exch{" H1* APC"}=" H1' APC";
$resn_exch{"1HN6 APC"}="HN61 APC"; $resn_exch{"2HN6 APC"}="HN62 APC"; $resn_exch{"1HN  APD"}=" HN1 APD";
$resn_exch{"2HN  APD"}=" HN2 APD"; $resn_exch{"1HBC APD"}="HBC1 APD"; $resn_exch{"2HBC APD"}="HBC2 APD";
$resn_exch{"1H5C APD"}="H5C1 APD"; $resn_exch{"2H5C APD"}="H5C2 APD"; $resn_exch{"3H5C APD"}="H5C3 APD";
$resn_exch{"1HN  APE"}=" H   APE"; $resn_exch{"2HN  APE"}=" H2  APE"; $resn_exch{"1HB  APE"}=" HB2 APE";
$resn_exch{"2HB  APE"}=" HB3 APE"; $resn_exch{"1HC2 APF"}="HC21 APF"; $resn_exch{"2HC2 APF"}="HC22 APF";
$resn_exch{"3HC2 APF"}="HC23 APF"; $resn_exch{"1H4  APF"}=" H41 APF"; $resn_exch{"2H4  APF"}=" H42 APF";
$resn_exch{"1H14 APG"}="H141 APG"; $resn_exch{"2H14 APG"}="H142 APG"; $resn_exch{"3H14 APG"}="H143 APG";
$resn_exch{"1HN  APH"}=" H   APH"; $resn_exch{"2HN  APH"}=" H2  APH"; $resn_exch{"1HB  APH"}=" HB2 APH";
$resn_exch{"2HB  APH"}=" HB3 APH"; $resn_exch{"1HG2 APH"}="HG21 APH"; $resn_exch{"2HG2 APH"}="HG22 APH";
$resn_exch{"1H3  API"}=" H31 API"; $resn_exch{"2H3  API"}=" H32 API"; $resn_exch{"1H4  API"}=" H41 API";
$resn_exch{"2H4  API"}=" H42 API"; $resn_exch{"1H5  API"}=" H51 API"; $resn_exch{"2H5  API"}=" H52 API";
$resn_exch{"1HN2 API"}="HN21 API"; $resn_exch{"2HN2 API"}="HN22 API"; $resn_exch{"1HN6 API"}="HN61 API";
$resn_exch{"2HN6 API"}="HN62 API"; $resn_exch{" H   APK"}=" HC  APK"; $resn_exch{"1HN  APK"}=" H   APK";
$resn_exch{"2HN  APK"}=" H2  APK"; $resn_exch{"1HB  APK"}=" HB2 APK"; $resn_exch{"2HB  APK"}=" HB3 APK";
$resn_exch{"1HG  APK"}=" HG2 APK"; $resn_exch{"2HG  APK"}=" HG3 APK"; $resn_exch{"1HD  APK"}=" HD2 APK";
$resn_exch{"2HD  APK"}=" HD3 APK"; $resn_exch{"1HE  APK"}=" HE2 APK"; $resn_exch{"2HE  APK"}=" HE3 APK";
$resn_exch{" HNZ APK"}=" HZ  APK"; $resn_exch{"1H5* APK"}="H5'1 APK"; $resn_exch{"2H5* APK"}="H5'2 APK";
$resn_exch{" H4* APK"}=" H4' APK"; $resn_exch{" H3* APK"}=" H3' APK"; $resn_exch{"HO3* APK"}="HO3' APK";
$resn_exch{" H2* APK"}=" H2' APK"; $resn_exch{"HO2* APK"}="HO2' APK"; $resn_exch{" H1* APK"}=" H1' APK";
$resn_exch{" H2  APK"}=" HC2 APK"; $resn_exch{"1HN6 APK"}="HN61 APK"; $resn_exch{"2HN6 APK"}="HN62 APK";
$resn_exch{"1HC2 APL"}="HC21 APL"; $resn_exch{"2HC2 APL"}="HC22 APL"; $resn_exch{"3HC2 APL"}="HC23 APL";
$resn_exch{"1H4  APL"}=" H41 APL"; $resn_exch{"2H4  APL"}=" H42 APL"; $resn_exch{"1HN  APM"}=" H   APM";
$resn_exch{"2HN  APM"}=" H2  APM"; $resn_exch{"1HB  APM"}=" HB2 APM"; $resn_exch{"2HB  APM"}=" HB3 APM";
$resn_exch{"1HG2 APM"}="HG21 APM"; $resn_exch{"2HG2 APM"}="HG22 APM"; $resn_exch{"1H8' APN"}="H8'1 APN";
$resn_exch{"2H8' APN"}="H8'2 APN"; $resn_exch{"1H5' APN"}="H5'1 APN"; $resn_exch{"2H5' APN"}="H5'2 APN";
$resn_exch{"1H3' APN"}="H3'1 APN"; $resn_exch{"2H3' APN"}="H3'2 APN"; $resn_exch{"1H2' APN"}="H2'1 APN";
$resn_exch{"2H2' APN"}="H2'2 APN"; $resn_exch{"1H1' APN"}="H1'1 APN"; $resn_exch{"2H1' APN"}="H1'2 APN";
$resn_exch{"3H1' APN"}="H1'3 APN"; $resn_exch{"1HN6 APN"}="HN61 APN"; $resn_exch{"2HN6 APN"}="HN62 APN";
$resn_exch{"1HN2 APO"}="HN21 APO"; $resn_exch{"2HN2 APO"}="HN22 APO"; $resn_exch{"1HC6 APO"}="HC61 APO";
$resn_exch{"2HC6 APO"}="HC62 APO"; $resn_exch{"1HM  APP"}=" HM1 APP"; $resn_exch{"2HM  APP"}=" HM2 APP";
$resn_exch{"3HM  APP"}=" HM3 APP"; $resn_exch{"1H3  APP"}=" H31 APP"; $resn_exch{"2H3  APP"}=" H32 APP";
$resn_exch{"1H4  APP"}=" H41 APP"; $resn_exch{"2H4  APP"}=" H42 APP"; $resn_exch{"1H5  APP"}=" H51 APP";
$resn_exch{"2H5  APP"}=" H52 APP"; $resn_exch{"1H6  APP"}=" H61 APP"; $resn_exch{"2H6  APP"}=" H62 APP";
$resn_exch{"1H10 APQ"}="H101 APQ"; $resn_exch{"2H10 APQ"}="H102 APQ"; $resn_exch{"1H11 APQ"}="H111 APQ";
$resn_exch{"2H11 APQ"}="H112 APQ"; $resn_exch{"1H12 APQ"}="H121 APQ"; $resn_exch{"2H12 APQ"}="H122 APQ";
$resn_exch{"3H12 APQ"}="H123 APQ"; $resn_exch{"1H9  APQ"}=" H91 APQ"; $resn_exch{"2H9  APQ"}=" H92 APQ";
$resn_exch{"1HN1 APQ"}="HN11 APQ"; $resn_exch{"1H5  APQ"}=" H51 APQ"; $resn_exch{"1HN4 APQ"}="HN41 APQ";
$resn_exch{"2HN4 APQ"}="HN42 APQ"; $resn_exch{"1H1  APQ"}=" H11 APQ"; $resn_exch{"1HN2 APQ"}="HN21 APQ";
$resn_exch{"2HN2 APQ"}="HN22 APQ"; $resn_exch{"1H6  APR"}=" H61 APR"; $resn_exch{"2H6  APR"}=" H62 APR";
$resn_exch{"1H*  APR"}=" H'1 APR"; $resn_exch{"2H*  APR"}=" H'2 APR"; $resn_exch{"2HO* APR"}="HO'2 APR";
$resn_exch{"3H*  APR"}=" H'3 APR"; $resn_exch{"3HO* APR"}="HO'3 APR"; $resn_exch{"4H*  APR"}=" H'4 APR";
$resn_exch{"1H5* APR"}="H5'1 APR"; $resn_exch{"2H5* APR"}="H5'2 APR"; $resn_exch{"2HOA APR"}="HOA2 APR";
$resn_exch{"2HOB APR"}="HOB2 APR"; $resn_exch{"1H5R APR"}="H5R1 APR"; $resn_exch{"2H5R APR"}="H5R2 APR";
$resn_exch{"1HOR APR"}="HOR1 APR"; $resn_exch{"1HR* APR"}="HR'1 APR"; $resn_exch{"2HOR APR"}="HOR2 APR";
$resn_exch{"2HR* APR"}="HR'2 APR"; $resn_exch{"3HOR APR"}="HOR3 APR"; $resn_exch{"3HR* APR"}="HR'3 APR";
$resn_exch{"4HR* APR"}="HR'4 APR"; $resn_exch{"2H7  APS"}=" H72 APS"; $resn_exch{"1H7  APS"}=" H71 APS";
$resn_exch{"3H9  APS"}=" H93 APS"; $resn_exch{"2H9  APS"}=" H92 APS"; $resn_exch{"1H9  APS"}=" H91 APS";
$resn_exch{"1H1C APS"}="H1C1 APS"; $resn_exch{"2H5  APS"}=" H52 APS"; $resn_exch{"1H5  APS"}=" H51 APS";
$resn_exch{"2H8  APT"}=" H82 APT"; $resn_exch{"1H8  APT"}=" H81 APT"; $resn_exch{"1H2  APT"}=" H21 APT";
$resn_exch{"2H2  APT"}=" H22 APT"; $resn_exch{"2HOA APU"}="HOA2 APU"; $resn_exch{"AHO5 APU"}="HO5A APU";
$resn_exch{"AH51 APU"}="H51A APU"; $resn_exch{"AH52 APU"}="H52A APU"; $resn_exch{"AH4* APU"}=" H4B APU";
$resn_exch{"AH3* APU"}=" H3B APU"; $resn_exch{"AH2* APU"}=" H2B APU"; $resn_exch{"AHO2 APU"}="HO2A APU";
$resn_exch{"AH1* APU"}=" H1B APU"; $resn_exch{" AH8 APU"}=" H8A APU"; $resn_exch{"AH61 APU"}="H61A APU";
$resn_exch{"AH62 APU"}="H62A APU"; $resn_exch{" AH2 APU"}=" H2A APU"; $resn_exch{" UH3 APU"}=" H3U APU";
$resn_exch{" UH5 APU"}=" H5U APU"; $resn_exch{" UH6 APU"}=" H6U APU"; $resn_exch{"UH1* APU"}=" H1D APU";
$resn_exch{"UH2* APU"}=" H2D APU"; $resn_exch{"*HO2 APU"}="HO2' APU"; $resn_exch{"UH3* APU"}=" H3D APU";
$resn_exch{"UH4* APU"}=" H4D APU"; $resn_exch{"UH51 APU"}="H51U APU"; $resn_exch{"UH52 APU"}="H52U APU";
$resn_exch{"UHO2 APU"}="HO2U APU"; $resn_exch{"UHO3 APU"}="HO3U APU"; $resn_exch{"1H3C APV"}="H3C1 APV";
$resn_exch{"2H3C APV"}="H3C2 APV"; $resn_exch{"1H4C APV"}="H4C1 APV"; $resn_exch{"2H4C APV"}="H4C2 APV";
$resn_exch{"1H7C APV"}="H7C1 APV"; $resn_exch{"2H7C APV"}="H7C2 APV"; $resn_exch{"1H14 APV"}="H141 APV";
$resn_exch{"2H14 APV"}="H142 APV"; $resn_exch{"1H33 APV"}="H331 APV"; $resn_exch{"2H33 APV"}="H332 APV";
$resn_exch{"1H3B APW"}="H3B1 APW"; $resn_exch{"2H3B APW"}="H3B2 APW"; $resn_exch{"1H5* APW"}="H5'1 APW";
$resn_exch{"2H5* APW"}="H5'2 APW"; $resn_exch{" H4* APW"}=" H4' APW"; $resn_exch{" H3* APW"}=" H3' APW";
$resn_exch{" H2* APW"}=" H2' APW"; $resn_exch{" H1* APW"}=" H1' APW"; $resn_exch{"1HN6 APW"}="HN61 APW";
$resn_exch{"2HN6 APW"}="HN62 APW"; $resn_exch{"7HOM APX"}="HOM7 APX"; $resn_exch{"2H5M APX"}="H5M2 APX";
$resn_exch{"1H5M APX"}="H5M1 APX"; $resn_exch{"1H3O APX"}="H3O1 APX"; $resn_exch{"1H2O APX"}="H2O1 APX";
$resn_exch{"7HOB APX"}="HOB7 APX"; $resn_exch{"2H5B APX"}="H5B2 APX"; $resn_exch{"1H5B APX"}="H5B1 APX";
$resn_exch{"2H3O APX"}="H3O2 APX"; $resn_exch{"2H6A APX"}="H6A2 APX"; $resn_exch{"1H6A APX"}="H6A1 APX";
$resn_exch{"2HOP APX"}="HOP2 APX"; $resn_exch{"3HOP APX"}="HOP3 APX"; $resn_exch{"1HN  APY"}=" HN1 APY";
$resn_exch{"2HN  APY"}=" HN2 APY"; $resn_exch{"1HM  APY"}=" HM1 APY"; $resn_exch{"2HM  APY"}=" HM2 APY";
$resn_exch{"1H13 APZ"}="H131 APZ"; $resn_exch{"2H13 APZ"}="H132 APZ"; $resn_exch{"1H7  AQO"}=" H71 AQO";
$resn_exch{"2H7  AQO"}=" H72 AQO"; $resn_exch{"1HN6 AQP"}="HN61 AQP"; $resn_exch{"2HN6 AQP"}="HN62 AQP";
$resn_exch{" H1* AQP"}=" H1' AQP"; $resn_exch{" H4* AQP"}=" H4' AQP"; $resn_exch{"1H5* AQP"}="H5'1 AQP";
$resn_exch{"2H5* AQP"}="H5'2 AQP"; $resn_exch{" H3* AQP"}=" H3' AQP"; $resn_exch{" H2* AQP"}=" H2' AQP";
$resn_exch{"1HA1 AQS"}="HA11 AQS"; $resn_exch{"2HA1 AQS"}="HA12 AQS"; $resn_exch{"1HA2 AQS"}="HA21 AQS";
$resn_exch{"2HA2 AQS"}="HA22 AQS"; $resn_exch{"1HA3 AQS"}="HA31 AQS"; $resn_exch{"2HA3 AQS"}="HA32 AQS";
$resn_exch{"1HNA AQS"}="HNA1 AQS"; $resn_exch{"2HNA AQS"}="HNA2 AQS"; $resn_exch{"3HNA AQS"}="HNA3 AQS";
$resn_exch{"1HB1 AQS"}="HB11 AQS"; $resn_exch{"2HB1 AQS"}="HB12 AQS"; $resn_exch{"1HB2 AQS"}="HB21 AQS";
$resn_exch{"2HB2 AQS"}="HB22 AQS"; $resn_exch{"1HB3 AQS"}="HB31 AQS"; $resn_exch{"2HB3 AQS"}="HB32 AQS";
$resn_exch{"1HNB AQS"}="HNB1 AQS"; $resn_exch{"2HNB AQS"}="HNB2 AQS"; $resn_exch{"3HNB AQS"}="HNB3 AQS";
$resn_exch{"1H1  AQZ"}=" H11 AQZ"; $resn_exch{"2H1  AQZ"}="H12A AQZ"; $resn_exch{"3H1  AQZ"}=" H13 AQZ";
$resn_exch{"1H24 AQZ"}="H241 AQZ"; $resn_exch{"2H24 AQZ"}="H242 AQZ"; $resn_exch{"1H25 AQZ"}="H251 AQZ";
$resn_exch{"2H25 AQZ"}="H252 AQZ"; $resn_exch{"1H27 AQZ"}="H271 AQZ"; $resn_exch{"2H27 AQZ"}="H272 AQZ";
$resn_exch{"1H28 AQZ"}="H281 AQZ"; $resn_exch{"2H28 AQZ"}="H282 AQZ"; $resn_exch{"1H32 AQZ"}="H321 AQZ";
$resn_exch{"2H32 AQZ"}="H322 AQZ"; $resn_exch{"3H32 AQZ"}="H323 AQZ"; $resn_exch{"1H34 AQZ"}="H341 AQZ";
$resn_exch{"1H37 AQZ"}="H371 AQZ"; $resn_exch{"2H37 AQZ"}="H372 AQZ"; $resn_exch{"1H38 AQZ"}="H381 AQZ";
$resn_exch{"2H38 AQZ"}="H382 AQZ"; $resn_exch{"1H40 AQZ"}="H401 AQZ"; $resn_exch{"2H40 AQZ"}="H402 AQZ";
$resn_exch{"1H41 AQZ"}="H411 AQZ"; $resn_exch{"2H41 AQZ"}="H412 AQZ"; $resn_exch{"1H42 AQZ"}="H421 AQZ";
$resn_exch{"2H42 AQZ"}="H422 AQZ"; $resn_exch{"1H43 AQZ"}="H431 AQZ"; $resn_exch{"2H43 AQZ"}="H432 AQZ";
$resn_exch{"1HN1 AR1"}="HN11 AR1"; $resn_exch{"2HN1 AR1"}="HN12 AR1"; $resn_exch{"3HN1 AR1"}="HN13 AR1";
$resn_exch{"1HN2 AR1"}="HN21 AR1"; $resn_exch{"2HN2 AR1"}="HN22 AR1"; $resn_exch{"3HN2 AR1"}="HN23 AR1";
$resn_exch{"1HN3 AR1"}="HN31 AR1"; $resn_exch{"2HN3 AR1"}="HN32 AR1"; $resn_exch{"3HN3 AR1"}="HN33 AR1";
$resn_exch{"1H1T AR1"}="H1T1 AR1"; $resn_exch{"2H1T AR1"}="H1T2 AR1"; $resn_exch{"1H2T AR1"}="H2T1 AR1";
$resn_exch{"2H2T AR1"}="H2T2 AR1"; $resn_exch{"1HNT AR1"}="HNT1 AR1"; $resn_exch{"2HNT AR1"}="HNT2 AR1";
$resn_exch{"3HNT AR1"}="HNT3 AR1"; $resn_exch{"1HMT AR1"}="HMT1 AR1"; $resn_exch{"2HMT AR1"}="HMT2 AR1";
$resn_exch{"3HMT AR1"}="HMT3 AR1"; $resn_exch{"1HB  AR2"}=" HB1 AR2"; $resn_exch{"2HB  AR2"}=" HB2 AR2";
$resn_exch{"1HG  AR2"}=" HG1 AR2"; $resn_exch{"2HG  AR2"}=" HG2 AR2"; $resn_exch{"1HD  AR2"}=" HD1 AR2";
$resn_exch{"2HD  AR2"}=" HD2 AR2"; $resn_exch{"1HH1 AR2"}="HH11 AR2"; $resn_exch{"1HH2 AR2"}="HH21 AR2";
$resn_exch{"2HH2 AR2"}="HH22 AR2"; $resn_exch{"1HO2 AR2"}="HO21 AR2"; $resn_exch{"1H5' AR3"}="H5'1 AR3";
$resn_exch{"2H5' AR3"}="H5'2 AR3"; $resn_exch{"1HN4 AR3"}="HN41 AR3"; $resn_exch{"2HN4 AR3"}="HN42 AR3";
$resn_exch{"1HN  AR4"}=" H   AR4"; $resn_exch{"2HN  AR4"}=" H2  AR4"; $resn_exch{"1HB  AR4"}=" HB2 AR4";
$resn_exch{"2HB  AR4"}=" HB3 AR4"; $resn_exch{"1HG  AR4"}=" HG2 AR4"; $resn_exch{"2HG  AR4"}=" HG3 AR4";
$resn_exch{" H5* AR4"}=" H5' AR4"; $resn_exch{"1H5* AR4"}="H5'1 AR4"; $resn_exch{"2H5* AR4"}="H5'2 AR4";
$resn_exch{" H4* AR4"}=" H4' AR4"; $resn_exch{" H1* AR4"}=" H1' AR4"; $resn_exch{" H3* AR4"}=" H3' AR4";
$resn_exch{"1H3* AR4"}="H3'1 AR4"; $resn_exch{"HO2* AR4"}="HO2' AR4"; $resn_exch{"1H5  ARA"}=" H51 ARA";
$resn_exch{"2H5  ARA"}=" H52 ARA"; $resn_exch{"1H5  ARB"}=" H51 ARB"; $resn_exch{"2H5  ARB"}=" H52 ARB";
$resn_exch{"1H1  ARC"}=" H11 ARC"; $resn_exch{"2H1  ARC"}=" H12 ARC"; $resn_exch{"1H2  ARC"}=" H21 ARC";
$resn_exch{"2H2  ARC"}=" H22 ARC"; $resn_exch{"1H4  ARC"}=" H41 ARC"; $resn_exch{"2H4  ARC"}=" H42 ARC";
$resn_exch{"3H4  ARC"}=" H43 ARC"; $resn_exch{"1H5  ARC"}=" H51 ARC"; $resn_exch{"2H5  ARC"}=" H52 ARC";
$resn_exch{"1H6  ARC"}=" H61 ARC"; $resn_exch{"2H6  ARC"}=" H62 ARC"; $resn_exch{"1H7  ARC"}=" H71 ARC";
$resn_exch{"2H7  ARC"}=" H72 ARC"; $resn_exch{"1H9  ARC"}=" H91 ARC"; $resn_exch{"2H9  ARC"}=" H92 ARC";
$resn_exch{"3H9  ARC"}=" H93 ARC"; $resn_exch{"1H10 ARC"}="H101 ARC"; $resn_exch{"2H10 ARC"}="H102 ARC";
$resn_exch{"1H11 ARC"}="H111 ARC"; $resn_exch{"2H11 ARC"}="H112 ARC"; $resn_exch{"1H12 ARC"}="H121 ARC";
$resn_exch{"2H12 ARC"}="H122 ARC"; $resn_exch{"1H14 ARC"}="H141 ARC"; $resn_exch{"2H14 ARC"}="H142 ARC";
$resn_exch{"3H14 ARC"}="H143 ARC"; $resn_exch{"1H15 ARC"}="H151 ARC"; $resn_exch{"2H15 ARC"}="H152 ARC";
$resn_exch{"1H16 ARC"}="H161 ARC"; $resn_exch{"2H16 ARC"}="H162 ARC"; $resn_exch{"1H17 ARC"}="H171 ARC";
$resn_exch{"2H17 ARC"}="H172 ARC"; $resn_exch{"1H19 ARC"}="H191 ARC"; $resn_exch{"2H19 ARC"}="H192 ARC";
$resn_exch{"3H19 ARC"}="H193 ARC"; $resn_exch{"1H20 ARC"}="H201 ARC"; $resn_exch{"2H20 ARC"}="H202 ARC";
$resn_exch{"3H20 ARC"}="H203 ARC"; $resn_exch{"1H3  ARD"}=" H31 ARD"; $resn_exch{"2H3  ARD"}=" H32 ARD";
$resn_exch{"1H4  ARD"}=" H41 ARD"; $resn_exch{"2H4  ARD"}=" H42 ARD"; $resn_exch{"1H5  ARD"}=" H51 ARD";
$resn_exch{"2H5  ARD"}=" H52 ARD"; $resn_exch{"1H6  ARD"}=" H61 ARD"; $resn_exch{"2H6  ARD"}=" H62 ARD";
$resn_exch{"1H11 ARD"}="H111 ARD"; $resn_exch{"2H11 ARD"}="H112 ARD"; $resn_exch{"1H12 ARD"}="H121 ARD";
$resn_exch{"2H12 ARD"}="H122 ARD"; $resn_exch{"1H42 ARD"}="H421 ARD"; $resn_exch{"2H42 ARD"}="H422 ARD";
$resn_exch{"3H42 ARD"}="H423 ARD"; $resn_exch{"1H14 ARD"}="H141 ARD"; $resn_exch{"2H14 ARD"}="H142 ARD";
$resn_exch{"1H43 ARD"}="H431 ARD"; $resn_exch{"2H43 ARD"}="H432 ARD"; $resn_exch{"3H43 ARD"}="H433 ARD";
$resn_exch{"1H44 ARD"}="H441 ARD"; $resn_exch{"2H44 ARD"}="H442 ARD"; $resn_exch{"3H44 ARD"}="H443 ARD";
$resn_exch{"1H23 ARD"}="H231 ARD"; $resn_exch{"2H23 ARD"}="H232 ARD"; $resn_exch{"1H45 ARD"}="H451 ARD";
$resn_exch{"2H45 ARD"}="H452 ARD"; $resn_exch{"3H45 ARD"}="H453 ARD"; $resn_exch{"1H50 ARD"}="H501 ARD";
$resn_exch{"2H50 ARD"}="H502 ARD"; $resn_exch{"3H50 ARD"}="H503 ARD"; $resn_exch{"1H46 ARD"}="H461 ARD";
$resn_exch{"2H46 ARD"}="H462 ARD"; $resn_exch{"3H46 ARD"}="H463 ARD"; $resn_exch{"1H47 ARD"}="H471 ARD";
$resn_exch{"2H47 ARD"}="H472 ARD"; $resn_exch{"3H47 ARD"}="H473 ARD"; $resn_exch{"1H32 ARD"}="H321 ARD";
$resn_exch{"2H32 ARD"}="H322 ARD"; $resn_exch{"1H48 ARD"}="H481 ARD"; $resn_exch{"2H48 ARD"}="H482 ARD";
$resn_exch{"3H48 ARD"}="H483 ARD"; $resn_exch{"1H35 ARD"}="H351 ARD"; $resn_exch{"2H35 ARD"}="H352 ARD";
$resn_exch{"1H37 ARD"}="H371 ARD"; $resn_exch{"2H37 ARD"}="H372 ARD"; $resn_exch{"1H51 ARD"}="H511 ARD";
$resn_exch{"2H51 ARD"}="H512 ARD"; $resn_exch{"3H51 ARD"}="H513 ARD"; $resn_exch{"1H40 ARD"}="H401 ARD";
$resn_exch{"2H40 ARD"}="H402 ARD"; $resn_exch{"1H41 ARD"}="H411 ARD"; $resn_exch{"2H41 ARD"}="H412 ARD";
$resn_exch{"1H55 ARD"}="H551 ARD"; $resn_exch{"2H55 ARD"}="H552 ARD"; $resn_exch{"3H55 ARD"}="H553 ARD";
$resn_exch{"1H6A ARE"}="H6A1 ARE"; $resn_exch{"2H6A ARE"}="H6A2 ARE"; $resn_exch{"1H6B ARE"}="H6B1 ARE";
$resn_exch{"2H6B ARE"}="H6B2 ARE"; $resn_exch{"1H6C ARE"}="H6C1 ARE"; $resn_exch{"2H6C ARE"}="H6C2 ARE";
$resn_exch{"3H6C ARE"}="H6C3 ARE"; $resn_exch{"1H7H ARE"}="H7H1 ARE"; $resn_exch{"2H7H ARE"}="H7H2 ARE";
$resn_exch{"1H6G ARE"}="H6G1 ARE"; $resn_exch{"2H6G ARE"}="H6G2 ARE"; $resn_exch{"1HN  ARF"}=" HN1 ARF";
$resn_exch{"2HN  ARF"}=" HN2 ARF"; $resn_exch{" HN2 ARG"}=" H2  ARG"; $resn_exch{"1HB  ARG"}=" HB2 ARG";
$resn_exch{"2HB  ARG"}=" HB3 ARG"; $resn_exch{"1HG  ARG"}=" HG2 ARG"; $resn_exch{"2HG  ARG"}=" HG3 ARG";
$resn_exch{"1HD  ARG"}=" HD2 ARG"; $resn_exch{"2HD  ARG"}=" HD3 ARG"; $resn_exch{"1HH1 ARG"}="HH11 ARG";
$resn_exch{"2HH1 ARG"}="HH12 ARG"; $resn_exch{"1HH2 ARG"}="HH21 ARG"; $resn_exch{"2HH2 ARG"}="HH22 ARG";
$resn_exch{"1HC  ARH"}=" HC1 ARH"; $resn_exch{"1H13 ARH"}="H131 ARH"; $resn_exch{"2H13 ARH"}="H132 ARH";
$resn_exch{"1HC4 ARH"}="HC41 ARH"; $resn_exch{"1H17 ARH"}="H171 ARH"; $resn_exch{"2H17 ARH"}="H172 ARH";
$resn_exch{"3H17 ARH"}="H173 ARH"; $resn_exch{"1H25 ARH"}="H251 ARH"; $resn_exch{"2H25 ARH"}="H252 ARH";
$resn_exch{"3H25 ARH"}="H253 ARH"; $resn_exch{"1H33 ARH"}="H331 ARH"; $resn_exch{"2H33 ARH"}="H332 ARH";
$resn_exch{"3HO4 ARH"}="HO43 ARH"; $resn_exch{"3HC8 ARH"}="HC83 ARH"; $resn_exch{"3HC9 ARH"}="HC93 ARH";
$resn_exch{"4HC0 ARH"}="HC04 ARH"; $resn_exch{"4HC1 ARH"}="HC14 ARH"; $resn_exch{"4HC2 ARH"}="HC24 ARH";
$resn_exch{"1H2  ARI"}=" H21 ARI"; $resn_exch{"2H2  ARI"}=" H22 ARI"; $resn_exch{"1H3  ARI"}=" H31 ARI";
$resn_exch{"2H3  ARI"}=" H32 ARI"; $resn_exch{"1H4  ARI"}=" H41 ARI"; $resn_exch{"2H4  ARI"}=" H42 ARI";
$resn_exch{"3H4  ARI"}=" H43 ARI"; $resn_exch{"1H6  ARI"}=" H61 ARI"; $resn_exch{"2H6  ARI"}=" H62 ARI";
$resn_exch{"3H6  ARI"}=" H63 ARI"; $resn_exch{"1H16 ARL"}="H161 ARL"; $resn_exch{"2H16 ARL"}="H162 ARL";
$resn_exch{"3H16 ARL"}="H163 ARL"; $resn_exch{"1H7  ARL"}=" H71 ARL"; $resn_exch{"2H7  ARL"}=" H72 ARL";
$resn_exch{"3H7  ARL"}=" H73 ARL"; $resn_exch{"1H11 ARL"}="H111 ARL"; $resn_exch{"2H11 ARL"}="H112 ARL";
$resn_exch{"3H11 ARL"}="H113 ARL"; $resn_exch{"1H15 ARL"}="H151 ARL"; $resn_exch{"2H15 ARL"}="H152 ARL";
$resn_exch{"3H15 ARL"}="H153 ARL"; $resn_exch{"1H19 ARL"}="H191 ARL"; $resn_exch{"2H19 ARL"}="H192 ARL";
$resn_exch{"3H19 ARL"}="H193 ARL"; $resn_exch{"1H24 ARL"}="H241 ARL"; $resn_exch{"2H24 ARL"}="H242 ARL";
$resn_exch{"3H24 ARL"}="H243 ARL"; $resn_exch{"1H28 ARL"}="H281 ARL"; $resn_exch{"2H28 ARL"}="H282 ARL";
$resn_exch{"3H28 ARL"}="H283 ARL"; $resn_exch{"1H32 ARL"}="H321 ARL"; $resn_exch{"2H32 ARL"}="H322 ARL";
$resn_exch{"3H32 ARL"}="H323 ARL"; $resn_exch{"1HN  ARM"}=" H   ARM"; $resn_exch{"2HN  ARM"}=" H2  ARM";
$resn_exch{"1HB  ARM"}=" HB2 ARM"; $resn_exch{"2HB  ARM"}=" HB3 ARM"; $resn_exch{"1HG  ARM"}=" HG2 ARM";
$resn_exch{"2HG  ARM"}=" HG3 ARM"; $resn_exch{"1HD  ARM"}=" HD2 ARM"; $resn_exch{"2HD  ARM"}=" HD3 ARM";
$resn_exch{" HNE ARM"}=" HE  ARM"; $resn_exch{" HN1 ARM"}=" HH1 ARM"; $resn_exch{"1HN2 ARM"}="HH21 ARM";
$resn_exch{"2HN2 ARM"}="HH22 ARM"; $resn_exch{"1HM  ARM"}=" HM1 ARM"; $resn_exch{"2HM  ARM"}=" HM2 ARM";
$resn_exch{"3HM  ARM"}=" HM3 ARM"; $resn_exch{"1HN  ARN"}=" HN1 ARN"; $resn_exch{"1H1  ARN"}=" H11 ARN";
$resn_exch{"1H2  ARN"}=" H21 ARN"; $resn_exch{"2H2  ARN"}=" H22 ARN"; $resn_exch{"1H3  ARN"}=" H31 ARN";
$resn_exch{"2H3  ARN"}=" H32 ARN"; $resn_exch{"1HCA ARN"}="HCA1 ARN"; $resn_exch{"2HCA ARN"}="HCA2 ARN";
$resn_exch{"1HC  ARN"}=" HC1 ARN"; $resn_exch{" HN2 ARO"}=" H2  ARO"; $resn_exch{"1HB  ARO"}=" HB2 ARO";
$resn_exch{"2HB  ARO"}=" HB3 ARO"; $resn_exch{"1HG  ARO"}=" HG  ARO"; $resn_exch{" HOH ARO"}=" HH  ARO";
$resn_exch{"1HD  ARO"}=" HD2 ARO"; $resn_exch{"2HD  ARO"}=" HD3 ARO"; $resn_exch{"1HH1 ARO"}="HH11 ARO";
$resn_exch{"2HH1 ARO"}="HH12 ARO"; $resn_exch{"1HH2 ARO"}="HH21 ARO"; $resn_exch{"2HH2 ARO"}="HH22 ARO";
$resn_exch{"2H7  ARP"}=" H72 ARP"; $resn_exch{"1H7  ARP"}=" H71 ARP"; $resn_exch{"3H9  ARP"}=" H93 ARP";
$resn_exch{"2H9  ARP"}=" H92 ARP"; $resn_exch{"1H9  ARP"}=" H91 ARP"; $resn_exch{"1H1C ARP"}="H1C1 ARP";
$resn_exch{"2H5  ARP"}=" H52 ARP"; $resn_exch{"1H5  ARP"}=" H51 ARP"; $resn_exch{"1H2  ARQ"}="H21A ARQ";
$resn_exch{"2H2  ARQ"}=" H22 ARQ"; $resn_exch{"3H2  ARQ"}=" H23 ARQ"; $resn_exch{"1H3  ARQ"}=" H31 ARQ";
$resn_exch{"2H3  ARQ"}=" H32 ARQ"; $resn_exch{"3H3  ARQ"}=" H33 ARQ"; $resn_exch{"1H4  ARQ"}=" H41 ARQ";
$resn_exch{"2H4  ARQ"}=" H42 ARQ"; $resn_exch{"3H4  ARQ"}=" H43 ARQ"; $resn_exch{"1H6  ARQ"}=" H61 ARQ";
$resn_exch{"2H6  ARQ"}=" H62 ARQ"; $resn_exch{"1H14 ARQ"}="H141 ARQ"; $resn_exch{"2H14 ARQ"}="H142 ARQ";
$resn_exch{"1H22 ARQ"}="H221 ARQ"; $resn_exch{"2H22 ARQ"}="H222 ARQ"; $resn_exch{"3H22 ARQ"}="H223 ARQ";
$resn_exch{"1H15 ARQ"}="H151 ARQ"; $resn_exch{"2H15 ARQ"}="H152 ARQ"; $resn_exch{"1H12 ARR"}="H121 ARR";
$resn_exch{"2H12 ARR"}="H122 ARR"; $resn_exch{"1H13 ARR"}="H131 ARR"; $resn_exch{"2H13 ARR"}="H132 ARR";
$resn_exch{"1H14 ARR"}="H141 ARR"; $resn_exch{"2H14 ARR"}="H142 ARR"; $resn_exch{" HN2 ARV"}=" H2  ARV";
$resn_exch{"1HB  ARV"}=" HB2 ARV"; $resn_exch{"2HB  ARV"}=" HB3 ARV"; $resn_exch{"1HG  ARV"}=" HG2 ARV";
$resn_exch{"2HG  ARV"}=" HG3 ARV"; $resn_exch{"1HD  ARV"}=" HD2 ARV"; $resn_exch{"2HD  ARV"}=" HD3 ARV";
$resn_exch{"2HH1 ARV"}="HH11 ARV"; $resn_exch{"HNH1 ARV"}="HH12 ARV"; $resn_exch{"1HH2 ARV"}=" HH2 ARV";
$resn_exch{"1HC1 ARV"}="HC11 ARV"; $resn_exch{"2HC1 ARV"}="HC12 ARV"; $resn_exch{"1HC3 ARV"}="HC31 ARV";
$resn_exch{"2HC3 ARV"}="HC32 ARV"; $resn_exch{"1H5C ARW"}="H5C1 ARW"; $resn_exch{"2H5C ARW"}="H5C2 ARW";
$resn_exch{"1HMC ARW"}="HMC1 ARW"; $resn_exch{"2HMC ARW"}="HMC2 ARW"; $resn_exch{"3HMC ARW"}="HMC3 ARW";
$resn_exch{"1H50 ARY"}="H501 ARY"; $resn_exch{"2H50 ARY"}="H502 ARY"; $resn_exch{"1H51 ARY"}="H511 ARY";
$resn_exch{"2H51 ARY"}="H512 ARY"; $resn_exch{"1H52 ARY"}="H521 ARY"; $resn_exch{"2H52 ARY"}="H522 ARY";
$resn_exch{"1H53 ARY"}="H531 ARY"; $resn_exch{"2H53 ARY"}="H532 ARY"; $resn_exch{"1H54 ARY"}="H541 ARY";
$resn_exch{"2H54 ARY"}="H542 ARY"; $resn_exch{"1H55 ARY"}="H551 ARY"; $resn_exch{"2H55 ARY"}="H552 ARY";
$resn_exch{"1H56 ARY"}="H561 ARY"; $resn_exch{"2H56 ARY"}="H562 ARY"; $resn_exch{"1H58 ARY"}="H581 ARY";
$resn_exch{"2H58 ARY"}="H582 ARY"; $resn_exch{"3H58 ARY"}="H583 ARY"; $resn_exch{"1H59 ARY"}="H591 ARY";
$resn_exch{"2H59 ARY"}="H592 ARY"; $resn_exch{"3H59 ARY"}="H593 ARY"; $resn_exch{"1H48 ARY"}="H481 ARY";
$resn_exch{"2H48 ARY"}="H482 ARY"; $resn_exch{"1H49 ARY"}="H491 ARY"; $resn_exch{"2H49 ARY"}="H492 ARY";
$resn_exch{"3H49 ARY"}="H493 ARY"; $resn_exch{"1H3  ARY"}=" H31 ARY"; $resn_exch{"2H3  ARY"}=" H32 ARY";
$resn_exch{"1H9  ARY"}=" H91 ARY"; $resn_exch{"2H9  ARY"}=" H92 ARY"; $resn_exch{"3H9  ARY"}=" H93 ARY";
$resn_exch{"1H13 ARY"}="H131 ARY"; $resn_exch{"2H13 ARY"}="H132 ARY"; $resn_exch{"1H17 ARY"}="H171 ARY";
$resn_exch{"2H17 ARY"}="H172 ARY"; $resn_exch{"3H17 ARY"}="H173 ARY"; $resn_exch{"1H30 ARY"}="H301 ARY";
$resn_exch{"2H30 ARY"}="H302 ARY"; $resn_exch{"3H30 ARY"}="H303 ARY"; $resn_exch{"1H35 ARY"}="H351 ARY";
$resn_exch{"2H35 ARY"}="H352 ARY"; $resn_exch{"1HOP  AS"}="HOP1  AS"; $resn_exch{"3HOP  AS"}="HOP3  AS";
$resn_exch{"1H5*  AS"}=" H5'  AS"; $resn_exch{"2H5*  AS"}="H5''  AS"; $resn_exch{" H4*  AS"}=" H4'  AS";
$resn_exch{" H3*  AS"}=" H3'  AS"; $resn_exch{"*HO3  AS"}="HO3'  AS"; $resn_exch{"1H2*  AS"}=" H2'  AS";
$resn_exch{"2H2*  AS"}="H2''  AS"; $resn_exch{" H1*  AS"}=" H1'  AS"; $resn_exch{"1HN6  AS"}="HN61  AS";
$resn_exch{"2HN6  AS"}="HN62  AS"; $resn_exch{"1H1  AS0"}=" H11 AS0"; $resn_exch{"2H1  AS0"}="H12A AS0";
$resn_exch{"3H1  AS0"}=" H13 AS0"; $resn_exch{"1H3  AS0"}="H31A AS0"; $resn_exch{"2H3  AS0"}=" H32 AS0";
$resn_exch{"1H5  AS0"}=" H51 AS0"; $resn_exch{"2H5  AS0"}=" H52 AS0"; $resn_exch{"1H6  AS0"}=" H61 AS0";
$resn_exch{"2H6  AS0"}=" H62 AS0"; $resn_exch{"1H9  AS0"}=" H91 AS0"; $resn_exch{"2H9  AS0"}=" H92 AS0";
$resn_exch{"1H10 AS0"}="H101 AS0"; $resn_exch{"2H10 AS0"}="H102 AS0"; $resn_exch{"1H15 AS0"}="H151 AS0";
$resn_exch{"2H15 AS0"}="H152 AS0"; $resn_exch{"1H16 AS0"}="H161 AS0"; $resn_exch{"2H16 AS0"}="H162 AS0";
$resn_exch{"1H20 AS0"}="H201 AS0"; $resn_exch{"2H20 AS0"}="H202 AS0"; $resn_exch{"1H22 AS0"}="H221 AS0";
$resn_exch{"2H22 AS0"}="H222 AS0"; $resn_exch{"3H22 AS0"}="H223 AS0"; $resn_exch{"1H33 AS0"}="H331 AS0";
$resn_exch{"2H33 AS0"}="H332 AS0"; $resn_exch{"3H33 AS0"}="H333 AS0"; $resn_exch{"1HN4 AS1"}="HN41 AS1";
$resn_exch{"2HN4 AS1"}="HN42 AS1"; $resn_exch{"1H3  AS1"}=" H31 AS1"; $resn_exch{"2H3  AS1"}=" H32 AS1";
$resn_exch{"1H2  AS1"}=" H21 AS1"; $resn_exch{"2H2  AS1"}=" H22 AS1"; $resn_exch{"1H1  AS1"}=" H11 AS1";
$resn_exch{"2H1  AS1"}=" H12 AS1"; $resn_exch{"1HB  AS1"}=" HB1 AS1"; $resn_exch{"2HB  AS1"}=" HB2 AS1";
$resn_exch{"1HB  AS2"}=" HB1 AS2"; $resn_exch{"2HB  AS2"}=" HB2 AS2"; $resn_exch{"1HN  AS2"}=" HN1 AS2";
$resn_exch{"2HN  AS2"}=" HN2 AS2"; $resn_exch{"3HN  AS2"}=" HN3 AS2"; $resn_exch{"1H5  AS4"}=" H51 AS4";
$resn_exch{"2H5  AS4"}=" H52 AS4"; $resn_exch{"1H15 AS4"}="H151 AS4"; $resn_exch{"2H15 AS4"}="H152 AS4";
$resn_exch{"3H15 AS4"}="H153 AS4"; $resn_exch{"1H16 AS4"}="H161 AS4"; $resn_exch{"2H16 AS4"}="H162 AS4";
$resn_exch{"1H19 AS4"}="H191 AS4"; $resn_exch{"2H19 AS4"}="H192 AS4"; $resn_exch{"1H20 AS4"}="H201 AS4";
$resn_exch{"2H20 AS4"}="H202 AS4"; $resn_exch{"1H11 AS4"}="H111 AS4"; $resn_exch{"2H11 AS4"}="H112 AS4";
$resn_exch{"1H12 AS4"}="H121 AS4"; $resn_exch{"2H12 AS4"}="H122 AS4"; $resn_exch{"1H13 AS4"}="H131 AS4";
$resn_exch{"2H13 AS4"}="H132 AS4"; $resn_exch{"1H14 AS4"}="H141 AS4"; $resn_exch{"2H14 AS4"}="H142 AS4";
$resn_exch{"1HAA AS9"}="HAA1 AS9"; $resn_exch{"2HAA AS9"}="HAA2 AS9"; $resn_exch{"3HAA AS9"}="HAA3 AS9";
$resn_exch{" HN  AS9"}=" H   AS9"; $resn_exch{"HOXT AS9"}=" HXT AS9"; $resn_exch{"1HB  AS9"}=" HB2 AS9";
$resn_exch{"2HB  AS9"}=" HB3 AS9"; $resn_exch{"HOD1 AS9"}=" HD1 AS9"; $resn_exch{" HN2 ASA"}=" H2  ASA";
$resn_exch{"1HB  ASA"}=" HB2 ASA"; $resn_exch{"2HB  ASA"}=" HB3 ASA"; $resn_exch{" HN2 ASB"}=" H2  ASB";
$resn_exch{"1HB  ASB"}=" HB2 ASB"; $resn_exch{"2HB  ASB"}=" HB3 ASB"; $resn_exch{"1HC2 ASB"}="HC21 ASB";
$resn_exch{"2HC2 ASB"}="HC22 ASB"; $resn_exch{"1H6  ASC"}=" H61 ASC"; $resn_exch{"2H6  ASC"}=" H62 ASC";
$resn_exch{"1H1  ASD"}=" H11 ASD"; $resn_exch{"2H1  ASD"}=" H12 ASD"; $resn_exch{"1H11 ASD"}="H111 ASD";
$resn_exch{"2H11 ASD"}="H112 ASD"; $resn_exch{"1H12 ASD"}="H121 ASD"; $resn_exch{"2H12 ASD"}="H122 ASD";
$resn_exch{"1H15 ASD"}="H151 ASD"; $resn_exch{"2H15 ASD"}="H152 ASD"; $resn_exch{"1H16 ASD"}="H161 ASD";
$resn_exch{"2H16 ASD"}="H162 ASD"; $resn_exch{"1H18 ASD"}="H181 ASD"; $resn_exch{"2H18 ASD"}="H182 ASD";
$resn_exch{"3H18 ASD"}="H183 ASD"; $resn_exch{"1H19 ASD"}="H191 ASD"; $resn_exch{"2H19 ASD"}="H192 ASD";
$resn_exch{"3H19 ASD"}="H193 ASD"; $resn_exch{"1H2  ASD"}=" H21 ASD"; $resn_exch{"2H2  ASD"}=" H22 ASD";
$resn_exch{"1H6  ASD"}=" H61 ASD"; $resn_exch{"2H6  ASD"}=" H62 ASD"; $resn_exch{"1H7  ASD"}=" H71 ASD";
$resn_exch{"2H7  ASD"}=" H72 ASD"; $resn_exch{"1H11 ASE"}="H111 ASE"; $resn_exch{"2H11 ASE"}="H112 ASE";
$resn_exch{"1H12 ASE"}="H121 ASE"; $resn_exch{"2H12 ASE"}="H122 ASE"; $resn_exch{"1H15 ASE"}="H151 ASE";
$resn_exch{"2H15 ASE"}="H152 ASE"; $resn_exch{"3H15 ASE"}="H153 ASE"; $resn_exch{"1H2C ASF"}="H2C1 ASF";
$resn_exch{"2H2C ASF"}="H2C2 ASF"; $resn_exch{"3H2C ASF"}="H2C3 ASF"; $resn_exch{"1H3C ASF"}="H3C1 ASF";
$resn_exch{"2H3C ASF"}="H3C2 ASF"; $resn_exch{"1H4C ASF"}="H4C1 ASF"; $resn_exch{"2H4C ASF"}="H4C2 ASF";
$resn_exch{"1H5C ASF"}="H5C1 ASF"; $resn_exch{"2H5C ASF"}="H5C2 ASF"; $resn_exch{"1H7C ASF"}="H7C1 ASF";
$resn_exch{"2H7C ASF"}="H7C2 ASF"; $resn_exch{"1H8C ASF"}="H8C1 ASF"; $resn_exch{"2H8C ASF"}="H8C2 ASF";
$resn_exch{"1HO3 ASF"}="HO31 ASF"; $resn_exch{" HN2 ASI"}=" H2  ASI"; $resn_exch{"1H2  ASI"}=" H21 ASI";
$resn_exch{"2H2  ASI"}=" H22 ASI"; $resn_exch{"1HN  ASK"}=" H   ASK"; $resn_exch{"2HN  ASK"}=" H2  ASK";
$resn_exch{"1HB  ASK"}=" HB2 ASK"; $resn_exch{"2HB  ASK"}=" HB3 ASK"; $resn_exch{" HOD ASK"}=" HD2 ASK";
$resn_exch{"1HM  ASK"}=" HM1 ASK"; $resn_exch{"2HM  ASK"}=" HM2 ASK"; $resn_exch{"3HM  ASK"}=" HM3 ASK";
$resn_exch{" HN2 ASL"}=" H2  ASL"; $resn_exch{"1HB  ASL"}=" HB1 ASL"; $resn_exch{"2HB  ASL"}=" HB2 ASL";
$resn_exch{"1HC3 ASL"}="HC31 ASL"; $resn_exch{"2HC3 ASL"}="HC32 ASL"; $resn_exch{"3HC3 ASL"}="HC33 ASL";
$resn_exch{"1HN  ASM"}=" H   ASM"; $resn_exch{"2HN  ASM"}=" H2  ASM"; $resn_exch{"1HB  ASM"}=" HB2 ASM";
$resn_exch{"2HB  ASM"}=" HB3 ASM"; $resn_exch{" HN2 ASN"}=" H2  ASN"; $resn_exch{"1HB  ASN"}=" HB2 ASN";
$resn_exch{"2HB  ASN"}=" HB3 ASN"; $resn_exch{"1HD2 ASN"}="HD21 ASN"; $resn_exch{"2HD2 ASN"}="HD22 ASN";
$resn_exch{"1H1  ASO"}=" H11 ASO"; $resn_exch{"2H1  ASO"}=" H12 ASO"; $resn_exch{"1H6  ASO"}=" H61 ASO";
$resn_exch{"2H6  ASO"}=" H62 ASO"; $resn_exch{" HN2 ASP"}=" H2  ASP"; $resn_exch{"1HB  ASP"}=" HB2 ASP";
$resn_exch{"2HB  ASP"}=" HB3 ASP"; $resn_exch{" HN2 ASQ"}=" H2  ASQ"; $resn_exch{"1HB  ASQ"}=" HB2 ASQ";
$resn_exch{"2HB  ASQ"}=" HB3 ASQ"; $resn_exch{"2HOP ASQ"}="HOP2 ASQ"; $resn_exch{"3HOP ASQ"}="HOP3 ASQ";
$resn_exch{"1H7  ASR"}=" H71 ASR"; $resn_exch{"2H7  ASR"}=" H72 ASR"; $resn_exch{"2HOP ASU"}="HOP2 ASU";
$resn_exch{"3HOP ASU"}="HOP3 ASU"; $resn_exch{"1HO* ASU"}="HO'1 ASU"; $resn_exch{"1H2* ASU"}=" H2' ASU";
$resn_exch{"2H2* ASU"}="H2'' ASU"; $resn_exch{"1H5* ASU"}=" H5' ASU"; $resn_exch{"2H5* ASU"}="H5'' ASU";
$resn_exch{" H4* ASU"}=" H4' ASU"; $resn_exch{" H1* ASU"}=" H1' ASU"; $resn_exch{" H3* ASU"}=" H3' ASU";
$resn_exch{"3HO* ASU"}="HO'3 ASU"; $resn_exch{"1H3C ASV"}="H3C1 ASV"; $resn_exch{"2H3C ASV"}="H3C2 ASV";
$resn_exch{"1H4C ASV"}="H4C1 ASV"; $resn_exch{"2H4C ASV"}="H4C2 ASV"; $resn_exch{"1H7C ASV"}="H7C1 ASV";
$resn_exch{"2H7C ASV"}="H7C2 ASV"; $resn_exch{"1H14 ASV"}="H141 ASV"; $resn_exch{"2H14 ASV"}="H142 ASV";
$resn_exch{"1H16 ASV"}="H161 ASV"; $resn_exch{"2H16 ASV"}="H162 ASV"; $resn_exch{"1H32 ASV"}="H321 ASV";
$resn_exch{"1H33 ASV"}="H331 ASV"; $resn_exch{"2H33 ASV"}="H332 ASV"; $resn_exch{"1HB  ASX"}=" HB1 ASX";
$resn_exch{"2HB  ASX"}=" HB2 ASX"; $resn_exch{"1H2C AT3"}="H2C1 AT3"; $resn_exch{"2H2C AT3"}="H2C2 AT3";
$resn_exch{"1H3C AT3"}="H3C1 AT3"; $resn_exch{"2H3C AT3"}="H3C2 AT3"; $resn_exch{"1H6C AT3"}="H6C1 AT3";
$resn_exch{"2H6C AT3"}="H6C2 AT3"; $resn_exch{"3H6C AT3"}="H6C3 AT3"; $resn_exch{"1H8C AT3"}="H8C1 AT3";
$resn_exch{"2H8C AT3"}="H8C2 AT3"; $resn_exch{"3H8C AT3"}="H8C3 AT3"; $resn_exch{"1H9C AT3"}="H9C1 AT3";
$resn_exch{"2H9C AT3"}="H9C2 AT3"; $resn_exch{"3H9C AT3"}="H9C3 AT3"; $resn_exch{"3H10 AT3"}="H103 AT3";
$resn_exch{"1H10 AT3"}="H101 AT3"; $resn_exch{"2H10 AT3"}="H102 AT3"; $resn_exch{"1H5* AT4"}="H5'1 AT4";
$resn_exch{"2H5* AT4"}="H5'2 AT4"; $resn_exch{" H4* AT4"}=" H4' AT4"; $resn_exch{" H3* AT4"}=" H3' AT4";
$resn_exch{"HO3* AT4"}="HO3' AT4"; $resn_exch{" H2* AT4"}=" H2' AT4"; $resn_exch{" H1* AT4"}=" H1' AT4";
$resn_exch{"1HN6 AT4"}="HN61 AT4"; $resn_exch{"2HN6 AT4"}="HN62 AT4"; $resn_exch{"HO2* AT4"}="HO2' AT4";
$resn_exch{"1H11 AT5"}="H111 AT5"; $resn_exch{"2H11 AT5"}="H112 AT5"; $resn_exch{"1H17 AT5"}="H171 AT5";
$resn_exch{"2H17 AT5"}="H172 AT5"; $resn_exch{"3H17 AT5"}="H173 AT5"; $resn_exch{"1H14 AT5"}="H141 AT5";
$resn_exch{"2H14 AT5"}="H142 AT5"; $resn_exch{"1H10 AT5"}="H101 AT5"; $resn_exch{"2H10 AT5"}="H102 AT5";
$resn_exch{"3H10 AT5"}="H103 AT5"; $resn_exch{"1HM2 AT5"}="HM21 AT5"; $resn_exch{"2HM2 AT5"}="HM22 AT5";
$resn_exch{"3HM2 AT5"}="HM23 AT5"; $resn_exch{"1HM3 AT5"}="HM31 AT5"; $resn_exch{"2HM3 AT5"}="HM32 AT5";
$resn_exch{"3HM3 AT5"}="HM33 AT5"; $resn_exch{"1HN' ATA"}="HN'1 ATA"; $resn_exch{"2HN' ATA"}="HN'2 ATA";
$resn_exch{"1HB' ATA"}="HB'1 ATA"; $resn_exch{"2HB' ATA"}="HB'2 ATA"; $resn_exch{"1HG' ATA"}="HG'1 ATA";
$resn_exch{"2HG' ATA"}="HG'2 ATA"; $resn_exch{"2HO' ATA"}="HO'2 ATA"; $resn_exch{"1HB  ATA"}=" HB1 ATA";
$resn_exch{"2HB  ATA"}=" HB2 ATA"; $resn_exch{"1H12 ATA"}="H121 ATA"; $resn_exch{"2H12 ATA"}="H122 ATA";
$resn_exch{"1H13 ATA"}="H131 ATA"; $resn_exch{"2H13 ATA"}="H132 ATA"; $resn_exch{"3H13 ATA"}="H133 ATA";
$resn_exch{"1H9  ATA"}=" H91 ATA"; $resn_exch{"2H9  ATA"}=" H92 ATA"; $resn_exch{"3H9  ATA"}=" H93 ATA";
$resn_exch{"1H10 ATA"}="H101 ATA"; $resn_exch{"2H10 ATA"}="H102 ATA"; $resn_exch{"3H10 ATA"}="H103 ATA";
$resn_exch{" HN\" ATA"}=" HN' ATA"; $resn_exch{"1HA\" ATA"}="HA'1 ATA"; $resn_exch{"2HA\" ATA"}="HA'2 ATA";
$resn_exch{" HO\" ATA"}=" HO' ATA"; $resn_exch{"1HN2 ATC"}="HN21 ATC"; $resn_exch{"2HN2 ATC"}="HN22 ATC";
$resn_exch{"1H42 ATC"}="H421 ATC"; $resn_exch{"2H42 ATC"}="H422 ATC"; $resn_exch{"3H42 ATC"}="H423 ATC";
$resn_exch{"1H43 ATC"}="H431 ATC"; $resn_exch{"2H43 ATC"}="H432 ATC"; $resn_exch{"3H43 ATC"}="H433 ATC";
$resn_exch{"1H5  ATC"}="H51A ATC"; $resn_exch{"2H5  ATC"}=" H52 ATC"; $resn_exch{"1H6  ATC"}=" H61 ATC";
$resn_exch{"2H6  ATC"}=" H62 ATC"; $resn_exch{"1HOC ATC"}="HOC1 ATC"; $resn_exch{"1H92 ATC"}="H921 ATC";
$resn_exch{"2H92 ATC"}="H922 ATC"; $resn_exch{"1H93 ATC"}="H931 ATC"; $resn_exch{"2H93 ATC"}="H932 ATC";
$resn_exch{"3H93 ATC"}="H933 ATC"; $resn_exch{"1H94 ATC"}="H941 ATC"; $resn_exch{"2H94 ATC"}="H942 ATC";
$resn_exch{"3H94 ATC"}="H943 ATC"; $resn_exch{"2HOP ATD"}="HOP2 ATD"; $resn_exch{"3HOP ATD"}="HOP3 ATD";
$resn_exch{"5HO* ATD"}="HO'5 ATD"; $resn_exch{"1H5* ATD"}=" H5' ATD"; $resn_exch{"2H5* ATD"}="H5'' ATD";
$resn_exch{" H4* ATD"}=" H4' ATD"; $resn_exch{" H3* ATD"}=" H3' ATD"; $resn_exch{"1H2* ATD"}=" H2' ATD";
$resn_exch{"2H2* ATD"}="H2'' ATD"; $resn_exch{" H1* ATD"}=" H1' ATD"; $resn_exch{"1H5M ATD"}=" H71 ATD";
$resn_exch{"2H5M ATD"}=" H72 ATD"; $resn_exch{"3H5M ATD"}=" H73 ATD"; $resn_exch{"1H2  ATE"}=" H21 ATE";
$resn_exch{"2H2  ATE"}=" H22 ATE"; $resn_exch{"1H1  ATE"}=" H11 ATE"; $resn_exch{"2H1  ATE"}=" H12 ATE";
$resn_exch{"1H4  ATE"}=" H41 ATE"; $resn_exch{"2H4  ATE"}=" H42 ATE"; $resn_exch{"1H6  ATE"}=" H61 ATE";
$resn_exch{"2H6  ATE"}=" H62 ATE"; $resn_exch{"1H19 ATE"}="H191 ATE"; $resn_exch{"2H19 ATE"}="H192 ATE";
$resn_exch{"3H19 ATE"}="H193 ATE"; $resn_exch{"1H11 ATE"}="H111 ATE"; $resn_exch{"2H11 ATE"}="H112 ATE";
$resn_exch{"1H7  ATE"}=" H71 ATE"; $resn_exch{"2H7  ATE"}=" H72 ATE"; $resn_exch{"1H15 ATE"}="H151 ATE";
$resn_exch{"2H15 ATE"}="H152 ATE"; $resn_exch{"1H12 ATE"}="H121 ATE"; $resn_exch{"2H12 ATE"}="H122 ATE";
$resn_exch{"1H18 ATE"}="H181 ATE"; $resn_exch{"2H18 ATE"}="H182 ATE"; $resn_exch{"3H18 ATE"}="H183 ATE";
$resn_exch{"2HOG ATF"}="HOG2 ATF"; $resn_exch{"3HOG ATF"}="HOG3 ATF"; $resn_exch{"2HOB ATF"}="HOB2 ATF";
$resn_exch{"2HOA ATF"}="HOA2 ATF"; $resn_exch{"1H5* ATF"}="H5'1 ATF"; $resn_exch{"2H5* ATF"}="H5'2 ATF";
$resn_exch{" H4* ATF"}=" H4' ATF"; $resn_exch{" H3* ATF"}=" H3' ATF"; $resn_exch{"*HO3 ATF"}="HO3' ATF";
$resn_exch{" H2* ATF"}=" H2' ATF"; $resn_exch{"*HO2 ATF"}="HO2' ATF"; $resn_exch{" H1* ATF"}=" H1' ATF";
$resn_exch{"1HN6 ATF"}="HN61 ATF"; $resn_exch{"2HN6 ATF"}="HN62 ATF"; $resn_exch{"2HOG ATG"}="HOG2 ATG";
$resn_exch{"3HSG ATG"}="HSG3 ATG"; $resn_exch{"2HOB ATG"}="HOB2 ATG"; $resn_exch{"2HOA ATG"}="HOA2 ATG";
$resn_exch{"1H5* ATG"}="H5'1 ATG"; $resn_exch{"2H5* ATG"}="H5'2 ATG"; $resn_exch{" H4* ATG"}=" H4' ATG";
$resn_exch{" H3* ATG"}=" H3' ATG"; $resn_exch{"*HO3 ATG"}="HO3' ATG"; $resn_exch{" H2* ATG"}=" H2' ATG";
$resn_exch{"*HO2 ATG"}="HO2' ATG"; $resn_exch{" H1* ATG"}=" H1' ATG"; $resn_exch{"1HN6 ATG"}="HN61 ATG";
$resn_exch{"2HN6 ATG"}="HN62 ATG"; $resn_exch{"1HN1 ATI"}="HN11 ATI"; $resn_exch{"2HN1 ATI"}="HN12 ATI";
$resn_exch{"1H5  ATI"}=" H51 ATI"; $resn_exch{"2H5  ATI"}=" H52 ATI"; $resn_exch{"1H7  ATI"}=" H71 ATI";
$resn_exch{"2H7  ATI"}=" H72 ATI"; $resn_exch{"3H7  ATI"}=" H73 ATI"; $resn_exch{"1H8  ATI"}=" H81 ATI";
$resn_exch{"2H8  ATI"}=" H82 ATI"; $resn_exch{"3H8  ATI"}=" H83 ATI"; $resn_exch{"1H16 ATI"}="H161 ATI";
$resn_exch{"2H16 ATI"}="H162 ATI"; $resn_exch{"3H16 ATI"}="H163 ATI"; $resn_exch{"1H17 ATI"}="H171 ATI";
$resn_exch{"2H17 ATI"}="H172 ATI"; $resn_exch{"3H17 ATI"}="H173 ATI"; $resn_exch{"1H23 ATI"}="H231 ATI";
$resn_exch{"2H23 ATI"}="H232 ATI"; $resn_exch{"3H23 ATI"}="H233 ATI"; $resn_exch{"1H24 ATI"}="H241 ATI";
$resn_exch{"2H24 ATI"}="H242 ATI"; $resn_exch{"3H24 ATI"}="H243 ATI"; $resn_exch{"1H29 ATI"}="H291 ATI";
$resn_exch{"2H29 ATI"}="H292 ATI"; $resn_exch{"1H1A ATJ"}="H1A1 ATJ"; $resn_exch{"1H2B ATJ"}="H2B1 ATJ";
$resn_exch{"2H2B ATJ"}="H2B2 ATJ"; $resn_exch{"1H1B ATJ"}="H1B1 ATJ"; $resn_exch{"2H1B ATJ"}="H1B2 ATJ";
$resn_exch{"3H1B ATJ"}="H1B3 ATJ"; $resn_exch{"1H5* ATL"}=" H5' ATL"; $resn_exch{"2H5* ATL"}="H5'' ATL";
$resn_exch{" H3* ATL"}=" H3' ATL"; $resn_exch{"1H2* ATL"}=" H2' ATL"; $resn_exch{" H1* ATL"}=" H1' ATL";
$resn_exch{"1H9* ATL"}="H9'1 ATL"; $resn_exch{"2H9* ATL"}="H9'2 ATL"; $resn_exch{"1H5M ATL"}=" H71 ATL";
$resn_exch{"2H5M ATL"}=" H72 ATL"; $resn_exch{"3H5M ATL"}=" H73 ATL"; $resn_exch{"2HOP ATM"}="HOP2 ATM";
$resn_exch{"3HOP ATM"}="HOP3 ATM"; $resn_exch{"1H5* ATM"}=" H5' ATM"; $resn_exch{"2H5* ATM"}="H5'' ATM";
$resn_exch{" H4* ATM"}=" H4' ATM"; $resn_exch{" H3* ATM"}=" H3' ATM"; $resn_exch{"1H2* ATM"}=" H2' ATM";
$resn_exch{"2H2* ATM"}="H2'' ATM"; $resn_exch{" H1* ATM"}=" H1' ATM"; $resn_exch{"1H5  ATM"}=" H51 ATM";
$resn_exch{"2H5  ATM"}=" H52 ATM"; $resn_exch{"3H5  ATM"}=" H53 ATM"; $resn_exch{"1HB  ATN"}=" HB1 ATN";
$resn_exch{"2HB  ATN"}=" HB2 ATN"; $resn_exch{"1HN  ATN"}=" HN1 ATN"; $resn_exch{"2HN  ATN"}=" HN2 ATN";
$resn_exch{"1HB1 ATN"}="HB11 ATN"; $resn_exch{"2HB1 ATN"}="HB12 ATN"; $resn_exch{"1HG1 ATN"}="HG11 ATN";
$resn_exch{"2HG1 ATN"}="HG12 ATN"; $resn_exch{"1HD  ATN"}="HD1A ATN"; $resn_exch{"2HD  ATN"}="HD2A ATN";
$resn_exch{"1HB2 ATN"}="HB21 ATN"; $resn_exch{"2HB2 ATN"}="HB22 ATN"; $resn_exch{"1HG2 ATN"}="HG21 ATN";
$resn_exch{"2HG2 ATN"}="HG22 ATN"; $resn_exch{"1HD3 ATN"}="HD31 ATN"; $resn_exch{"2HD3 ATN"}="HD32 ATN";
$resn_exch{"1HB3 ATN"}="HB31 ATN"; $resn_exch{"2HB3 ATN"}="HB32 ATN"; $resn_exch{"3HB3 ATN"}="HB33 ATN";
$resn_exch{"1HN4 ATN"}="HN41 ATN"; $resn_exch{"2HN4 ATN"}="HN42 ATN"; $resn_exch{"3H1  ATO"}=" H13 ATO";
$resn_exch{"2H1  ATO"}=" H12 ATO"; $resn_exch{"1H1  ATO"}=" H11 ATO"; $resn_exch{"2H3  ATO"}=" H32 ATO";
$resn_exch{"1H3  ATO"}=" H31 ATO"; $resn_exch{"2HOG ATP"}="HOG2 ATP"; $resn_exch{"3HOG ATP"}="HOG3 ATP";
$resn_exch{"2HOB ATP"}="HOB2 ATP"; $resn_exch{"2HOA ATP"}="HOA2 ATP"; $resn_exch{"1H5* ATP"}="H5'1 ATP";
$resn_exch{"2H5* ATP"}="H5'2 ATP"; $resn_exch{" H4* ATP"}=" H4' ATP"; $resn_exch{" H3* ATP"}=" H3' ATP";
$resn_exch{"*HO3 ATP"}="HO3' ATP"; $resn_exch{" H2* ATP"}=" H2' ATP"; $resn_exch{"*HO2 ATP"}="HO2' ATP";
$resn_exch{" H1* ATP"}=" H1' ATP"; $resn_exch{"1HN6 ATP"}="HN61 ATP"; $resn_exch{"2HN6 ATP"}="HN62 ATP";
$resn_exch{"1H4  ATQ"}=" H41 ATQ"; $resn_exch{"2H4  ATQ"}=" H42 ATQ"; $resn_exch{"1H5  ATQ"}=" H51 ATQ";
$resn_exch{"2H5  ATQ"}=" H52 ATQ"; $resn_exch{"1HN2 ATQ"}="HN21 ATQ"; $resn_exch{"2HN2 ATQ"}="HN22 ATQ";
$resn_exch{"2HOB ATR"}="HOB2 ATR"; $resn_exch{"3HOB ATR"}="HOB3 ATR"; $resn_exch{"2HOA ATR"}="HOA2 ATR";
$resn_exch{"1H5* ATR"}="H5'1 ATR"; $resn_exch{"2H5* ATR"}="H5'2 ATR"; $resn_exch{" H4* ATR"}=" H4' ATR";
$resn_exch{" H3* ATR"}=" H3' ATR"; $resn_exch{"*HO3 ATR"}="HO3' ATR"; $resn_exch{" H2* ATR"}=" H2' ATR";
$resn_exch{"2HOP ATR"}="HOP2 ATR"; $resn_exch{"3HOP ATR"}="HOP3 ATR"; $resn_exch{" H1* ATR"}=" H1' ATR";
$resn_exch{"1HN6 ATR"}="HN61 ATR"; $resn_exch{"2HN6 ATR"}="HN62 ATR"; $resn_exch{"1H5' ATS"}="H5'1 ATS";
$resn_exch{"2H5' ATS"}="H5'2 ATS"; $resn_exch{"'HO2 ATS"}="HO2' ATS"; $resn_exch{"'HO3 ATS"}="HO3' ATS";
$resn_exch{"1HN6 ATS"}="HN61 ATS"; $resn_exch{"2HN6 ATS"}="HN62 ATS"; $resn_exch{"1H3B ATS"}="H3B1 ATS";
$resn_exch{"2H3B ATS"}="H3B2 ATS"; $resn_exch{"1H3  ATT"}=" H31 ATT"; $resn_exch{"2H3  ATT"}=" H32 ATT";
$resn_exch{"3H3  ATT"}=" H33 ATT"; $resn_exch{"1H4  ATT"}=" H41 ATT"; $resn_exch{"2H4  ATT"}=" H42 ATT";
$resn_exch{"1HC3 ATU"}="HC31 ATU"; $resn_exch{"2HC3 ATU"}="HC32 ATU"; $resn_exch{"1HN  ATX"}=" HN1 ATX";
$resn_exch{"2HN  ATX"}=" HN2 ATX"; $resn_exch{"1HB  ATX"}=" HB1 ATX"; $resn_exch{"2HB  ATX"}=" HB2 ATX";
$resn_exch{"3HB  ATX"}=" HB3 ATX"; $resn_exch{"1H7  ATX"}=" H71 ATX"; $resn_exch{"2H7  ATX"}=" H72 ATX";
$resn_exch{"3H7  ATX"}=" H73 ATX"; $resn_exch{"1H8  ATX"}=" H81 ATX"; $resn_exch{"2H8  ATX"}=" H82 ATX";
$resn_exch{"3H8  ATX"}=" H83 ATX"; $resn_exch{"1H9  ATX"}=" H91 ATX"; $resn_exch{"2H9  ATX"}=" H92 ATX";
$resn_exch{"3H9  ATX"}=" H93 ATX"; $resn_exch{"2HPA ATY"}="HPA2 ATY"; $resn_exch{"2HPB ATY"}="HPB2 ATY";
$resn_exch{"3HPB ATY"}="HPB3 ATY"; $resn_exch{"2H5* ATY"}="H5'2 ATY"; $resn_exch{"1H5* ATY"}="H5'1 ATY";
$resn_exch{" H4* ATY"}=" H4' ATY"; $resn_exch{" H3* ATY"}=" H3' ATY"; $resn_exch{"3HHR ATY"}="HHR3 ATY";
$resn_exch{"2HHR ATY"}="HHR2 ATY"; $resn_exch{"1HHR ATY"}="HHR1 ATY"; $resn_exch{"2H2* ATY"}="H2'2 ATY";
$resn_exch{"1H2* ATY"}="H2'1 ATY"; $resn_exch{" H1* ATY"}=" H1' ATY"; $resn_exch{"3H5M ATY"}="H5M3 ATY";
$resn_exch{"2H5M ATY"}="H5M2 ATY"; $resn_exch{"1H5M ATY"}="H5M1 ATY"; $resn_exch{"1H9  ATZ"}=" H91 ATZ";
$resn_exch{"2H9  ATZ"}=" H92 ATZ"; $resn_exch{"3H9  ATZ"}=" H93 ATZ"; $resn_exch{"1H10 ATZ"}="H101 ATZ";
$resn_exch{"2H10 ATZ"}="H102 ATZ"; $resn_exch{"3H10 ATZ"}="H103 ATZ"; $resn_exch{"1H12 ATZ"}="H121 ATZ";
$resn_exch{"2H12 ATZ"}="H122 ATZ"; $resn_exch{"1H13 ATZ"}="H131 ATZ"; $resn_exch{"2H13 ATZ"}="H132 ATZ";
$resn_exch{"3H13 ATZ"}="H133 ATZ"; $resn_exch{"1H14 AU4"}="H141 AU4"; $resn_exch{"2H14 AU4"}="H142 AU4";
$resn_exch{"3H14 AU4"}="H143 AU4"; $resn_exch{"1H15 AU4"}="H151 AU4"; $resn_exch{"2H15 AU4"}="H152 AU4";
$resn_exch{"3H15 AU4"}="H153 AU4"; $resn_exch{"1HN8 AU4"}="HN81 AU4"; $resn_exch{"2HN8 AU4"}="HN82 AU4";
$resn_exch{"1H17 AU4"}="H171 AU4"; $resn_exch{"2H17 AU4"}="H172 AU4"; $resn_exch{"3H17 AU4"}="H173 AU4";
$resn_exch{"1H16 AU4"}="H161 AU4"; $resn_exch{"2H16 AU4"}="H162 AU4"; $resn_exch{"3H16 AU4"}="H163 AU4";
$resn_exch{"1H2  AUA"}=" H21 AUA"; $resn_exch{"2H2  AUA"}=" H22 AUA"; $resn_exch{"1H5  AUA"}=" H51 AUA";
$resn_exch{"2H5  AUA"}=" H52 AUA"; $resn_exch{"1H20 AUA"}="H201 AUA"; $resn_exch{"2H20 AUA"}="H202 AUA";
$resn_exch{"3H20 AUA"}="H203 AUA"; $resn_exch{"1H24 AUA"}="H241 AUA"; $resn_exch{"2H24 AUA"}="H242 AUA";
$resn_exch{"1H29 AUA"}="H291 AUA"; $resn_exch{"2H29 AUA"}="H292 AUA"; $resn_exch{"3H29 AUA"}="H293 AUA";
$resn_exch{"1H33 AUA"}="H331 AUA"; $resn_exch{"2H33 AUA"}="H332 AUA"; $resn_exch{"3H33 AUA"}="H333 AUA";
$resn_exch{"1H51 AUA"}="H511 AUA"; $resn_exch{"2H51 AUA"}="H512 AUA"; $resn_exch{"1H55 AUA"}="H551 AUA";
$resn_exch{"2H55 AUA"}="H552 AUA"; $resn_exch{"1H67 AUA"}="H671 AUA"; $resn_exch{"2H67 AUA"}="H672 AUA";
$resn_exch{"3H67 AUA"}="H673 AUA"; $resn_exch{"1H71 AUA"}="H711 AUA"; $resn_exch{"2H71 AUA"}="H712 AUA";
$resn_exch{"3H71 AUA"}="H713 AUA"; $resn_exch{"1H79 AUA"}="H791 AUA"; $resn_exch{"2H79 AUA"}="H792 AUA";
$resn_exch{"1H82 AUA"}="H821 AUA"; $resn_exch{"2H82 AUA"}="H822 AUA"; $resn_exch{"1H85 AUA"}="H851 AUA";
$resn_exch{"2H85 AUA"}="H852 AUA"; $resn_exch{"1H88 AUA"}="H881 AUA"; $resn_exch{"2H88 AUA"}="H882 AUA";
$resn_exch{"3H88 AUA"}="H883 AUA"; $resn_exch{"1H92 AUA"}="H921 AUA"; $resn_exch{"2H92 AUA"}="H922 AUA";
$resn_exch{"1H97 AUA"}="H971 AUA"; $resn_exch{"2H97 AUA"}="H972 AUA"; $resn_exch{"3H97 AUA"}="H973 AUA";
$resn_exch{"1H98 AUA"}="H981 AUA"; $resn_exch{"2H98 AUA"}="H982 AUA"; $resn_exch{"3H98 AUA"}="H983 AUA";
$resn_exch{"1H99 AUA"}="H991 AUA"; $resn_exch{"2H99 AUA"}="H992 AUA"; $resn_exch{"3H99 AUA"}="H993 AUA";
$resn_exch{"1H3  AUP"}=" H31 AUP"; $resn_exch{"2H3  AUP"}=" H32 AUP"; $resn_exch{"1H4  AUP"}=" H41 AUP";
$resn_exch{"2H4  AUP"}=" H42 AUP"; $resn_exch{"1H5  AUP"}=" H51 AUP"; $resn_exch{"2H5  AUP"}=" H52 AUP";
$resn_exch{"1H6  AUP"}=" H61 AUP"; $resn_exch{"2H6  AUP"}=" H62 AUP"; $resn_exch{"1H1  AUR"}="H11A AUR";
$resn_exch{"2H1  AUR"}="H12A AUR"; $resn_exch{"3H1  AUR"}="H13A AUR"; $resn_exch{"1H2  AUR"}=" H21 AUR";
$resn_exch{"2H2  AUR"}=" H22 AUR"; $resn_exch{"1H20 AUR"}="H201 AUR"; $resn_exch{"2H20 AUR"}="H202 AUR";
$resn_exch{"3H20 AUR"}="H203 AUR"; $resn_exch{"1H21 AUR"}="H211 AUR"; $resn_exch{"2H21 AUR"}="H212 AUR";
$resn_exch{"3H21 AUR"}="H213 AUR"; $resn_exch{"1H24 AUR"}="H241 AUR"; $resn_exch{"2H24 AUR"}="H242 AUR";
$resn_exch{"3H24 AUR"}="H243 AUR"; $resn_exch{"1H22 AUR"}="H221 AUR"; $resn_exch{"2H22 AUR"}="H222 AUR";
$resn_exch{"3H22 AUR"}="H223 AUR"; $resn_exch{"1H23 AUR"}="H231 AUR"; $resn_exch{"2H23 AUR"}="H232 AUR";
$resn_exch{"3H23 AUR"}="H233 AUR"; $resn_exch{"2HOB AV2"}="HOB2 AV2"; $resn_exch{"3HOB AV2"}="HOB3 AV2";
$resn_exch{"2HOA AV2"}="HOA2 AV2"; $resn_exch{"1H5* AV2"}="H5'1 AV2"; $resn_exch{"2H5* AV2"}="H5'2 AV2";
$resn_exch{" H4* AV2"}=" H4' AV2"; $resn_exch{" H3* AV2"}=" H3' AV2"; $resn_exch{" H2* AV2"}=" H2' AV2";
$resn_exch{" H1* AV2"}=" H1' AV2"; $resn_exch{"1HN6 AV2"}="HN61 AV2"; $resn_exch{"2HN6 AV2"}="HN62 AV2";
$resn_exch{"3HOP AVC"}="HOP3 AVC"; $resn_exch{"2HOP AVC"}="HOP2 AVC"; $resn_exch{"1H5* AVC"}=" H5' AVC";
$resn_exch{"2H5* AVC"}="H5'' AVC"; $resn_exch{" H4* AVC"}=" H4' AVC"; $resn_exch{" H3* AVC"}=" H3' AVC";
$resn_exch{" H2* AVC"}=" H2' AVC"; $resn_exch{" H1* AVC"}=" H1' AVC"; $resn_exch{"1H6  AVC"}=" H61 AVC";
$resn_exch{"2H6  AVC"}=" H62 AVC"; $resn_exch{"1HN1 AVG"}="HN11 AVG"; $resn_exch{"2HN1 AVG"}="HN12 AVG";
$resn_exch{"1HC3 AVG"}="HC31 AVG"; $resn_exch{"2HC3 AVG"}="HC32 AVG"; $resn_exch{"1HC4 AVG"}="HC41 AVG";
$resn_exch{"2HC4 AVG"}="HC42 AVG"; $resn_exch{"1HC5 AVG"}="HC51 AVG"; $resn_exch{"2HC5 AVG"}="HC52 AVG";
$resn_exch{"1HC6 AVG"}="HC61 AVG"; $resn_exch{"2HC6 AVG"}="HC62 AVG"; $resn_exch{"1HN2 AVG"}="HN21 AVG";
$resn_exch{"2HN2 AVG"}="HN22 AVG"; $resn_exch{"1H2A AVP"}="H2A1 AVP"; $resn_exch{"1H'5 AVP"}="H'51 AVP";
$resn_exch{"2H'5 AVP"}="H'52 AVP"; $resn_exch{"1H'4 AVP"}="H'41 AVP"; $resn_exch{"2H'4 AVP"}="H'42 AVP";
$resn_exch{"1H'1 AVP"}="H'11 AVP"; $resn_exch{"2H'1 AVP"}="H'12 AVP"; $resn_exch{"1HN2 AVP"}="HN21 AVP";
$resn_exch{"2HN2 AVP"}="HN22 AVP"; $resn_exch{"1HN1 AVP"}="HN11 AVP"; $resn_exch{"1HC8 AVP"}="HC81 AVP";
$resn_exch{"1H2B AVP"}="H2B1 AVP"; $resn_exch{"1H1G AVP"}="H1G1 AVP"; $resn_exch{"1H2G AVP"}="H2G1 AVP";
$resn_exch{"1H3  AXD"}=" H31 AXD"; $resn_exch{"2H3  AXD"}=" H32 AXD"; $resn_exch{"1H4  AXD"}=" H41 AXD";
$resn_exch{"2H4  AXD"}=" H42 AXD"; $resn_exch{"1H6  AXD"}=" H61 AXD"; $resn_exch{"2H6  AXD"}=" H62 AXD";
$resn_exch{"1H7  AXD"}=" H71 AXD"; $resn_exch{"2H7  AXD"}=" H72 AXD"; $resn_exch{"1H11 AXD"}="H111 AXD";
$resn_exch{"2H11 AXD"}="H112 AXD"; $resn_exch{"1H12 AXD"}="H121 AXD"; $resn_exch{"2H12 AXD"}="H122 AXD";
$resn_exch{"1H13 AXD"}="H131 AXD"; $resn_exch{"2H13 AXD"}="H132 AXD"; $resn_exch{"1H14 AXD"}="H141 AXD";
$resn_exch{"2H14 AXD"}="H142 AXD"; $resn_exch{"1H17 AXD"}="H171 AXD"; $resn_exch{"2H17 AXD"}="H172 AXD";
$resn_exch{"1H26 AXD"}="H261 AXD"; $resn_exch{"2H26 AXD"}="H262 AXD"; $resn_exch{"3H26 AXD"}="H263 AXD";
$resn_exch{"1H8  AXF"}=" H81 AXF"; $resn_exch{"2H8  AXF"}=" H82 AXF"; $resn_exch{"1H11 AXF"}="H111 AXF";
$resn_exch{"2H11 AXF"}="H112 AXF"; $resn_exch{"1H14 AXF"}="H141 AXF"; $resn_exch{"2H14 AXF"}="H142 AXF";
$resn_exch{"1H27 AXF"}="H271 AXF"; $resn_exch{"2H27 AXF"}="H272 AXF"; $resn_exch{"1H31 AXF"}="H311 AXF";
$resn_exch{"2H31 AXF"}="H312 AXF"; $resn_exch{"1H38 AXF"}="H381 AXF"; $resn_exch{"2H38 AXF"}="H382 AXF";
$resn_exch{"1H45 AXF"}="H451 AXF"; $resn_exch{"2H45 AXF"}="H452 AXF"; $resn_exch{"1H50 AXF"}="H501 AXF";
$resn_exch{"2H50 AXF"}="H502 AXF"; $resn_exch{"3H50 AXF"}="H503 AXF"; $resn_exch{"1H58 AXF"}="H581 AXF";
$resn_exch{"2H58 AXF"}="H582 AXF"; $resn_exch{"1H61 AXF"}="H611 AXF"; $resn_exch{"2H61 AXF"}="H612 AXF";
$resn_exch{"1H64 AXF"}="H641 AXF"; $resn_exch{"2H64 AXF"}="H642 AXF"; $resn_exch{"1H67 AXF"}="H671 AXF";
$resn_exch{"2H67 AXF"}="H672 AXF"; $resn_exch{"3H67 AXF"}="H673 AXF"; $resn_exch{"1H72 AXF"}="H721 AXF";
$resn_exch{"2H72 AXF"}="H722 AXF"; $resn_exch{"3H72 AXF"}="H723 AXF"; $resn_exch{"1H51 AXL"}="H511 AXL";
$resn_exch{"2H51 AXL"}="H512 AXL"; $resn_exch{"3H51 AXL"}="H513 AXL"; $resn_exch{"1H52 AXL"}="H521 AXL";
$resn_exch{"2H52 AXL"}="H522 AXL"; $resn_exch{"3H52 AXL"}="H523 AXL"; $resn_exch{"OH62 AXL"}="H62O AXL";
$resn_exch{"1H11 AXL"}="H111 AXL"; $resn_exch{"2H11 AXL"}="H112 AXL"; $resn_exch{"OH18 AXL"}="H18O AXL";
$resn_exch{"1H3  AXP"}=" H31 AXP"; $resn_exch{"2H3  AXP"}=" H32 AXP"; $resn_exch{"1H9  AXP"}=" H91 AXP";
$resn_exch{"2H9  AXP"}=" H92 AXP"; $resn_exch{"1H11 AXP"}="H111 AXP"; $resn_exch{"2H11 AXP"}="H112 AXP";
$resn_exch{"3H11 AXP"}="H113 AXP"; $resn_exch{"2HOP AXP"}="HOP2 AXP"; $resn_exch{"3HOP AXP"}="HOP3 AXP";
$resn_exch{"1H2  AXQ"}=" H21 AXQ"; $resn_exch{"2H2  AXQ"}=" H22 AXQ"; $resn_exch{"1H17 AXQ"}="H171 AXQ";
$resn_exch{"2H17 AXQ"}="H172 AXQ"; $resn_exch{"1H21 AXQ"}="H211 AXQ"; $resn_exch{"2H21 AXQ"}="H212 AXQ";
$resn_exch{"1H28 AXQ"}="H281 AXQ"; $resn_exch{"2H28 AXQ"}="H282 AXQ"; $resn_exch{"1H37 AXQ"}="H371 AXQ";
$resn_exch{"2H37 AXQ"}="H372 AXQ"; $resn_exch{"3H37 AXQ"}="H373 AXQ"; $resn_exch{"1H41 AXQ"}="H411 AXQ";
$resn_exch{"2H41 AXQ"}="H412 AXQ"; $resn_exch{"3H41 AXQ"}="H413 AXQ"; $resn_exch{"1H45 AXQ"}="H451 AXQ";
$resn_exch{"2H45 AXQ"}="H452 AXQ"; $resn_exch{"3H45 AXQ"}="H453 AXQ"; $resn_exch{"1H50 AXQ"}="H501 AXQ";
$resn_exch{"2H50 AXQ"}="H502 AXQ"; $resn_exch{"3H50 AXQ"}="H503 AXQ"; $resn_exch{"1H59 AXQ"}="H591 AXQ";
$resn_exch{"2H59 AXQ"}="H592 AXQ"; $resn_exch{"1H64 AXQ"}="H641 AXQ"; $resn_exch{"2H64 AXQ"}="H642 AXQ";
$resn_exch{"3H64 AXQ"}="H643 AXQ"; $resn_exch{"1H76 AXQ"}="H761 AXQ"; $resn_exch{"2H76 AXQ"}="H762 AXQ";
$resn_exch{"3H76 AXQ"}="H763 AXQ"; $resn_exch{"1H80 AXQ"}="H801 AXQ"; $resn_exch{"2H80 AXQ"}="H802 AXQ";
$resn_exch{"3H80 AXQ"}="H803 AXQ"; $resn_exch{"1H88 AXQ"}="H881 AXQ"; $resn_exch{"2H88 AXQ"}="H882 AXQ";
$resn_exch{"1H91 AXQ"}="H911 AXQ"; $resn_exch{"2H91 AXQ"}="H912 AXQ"; $resn_exch{"1H94 AXQ"}="H941 AXQ";
$resn_exch{"2H94 AXQ"}="H942 AXQ"; $resn_exch{"1H97 AXQ"}="H971 AXQ"; $resn_exch{"2H97 AXQ"}="H972 AXQ";
$resn_exch{"3H97 AXQ"}="H973 AXQ"; $resn_exch{"1HC2 AXT"}="HC21 AXT"; $resn_exch{"2HC2 AXT"}="HC22 AXT";
$resn_exch{"1H16 AXT"}="H161 AXT"; $resn_exch{"2H16 AXT"}="H162 AXT"; $resn_exch{"3H16 AXT"}="H163 AXT";
$resn_exch{"1H17 AXT"}="H171 AXT"; $resn_exch{"2H17 AXT"}="H172 AXT"; $resn_exch{"3H17 AXT"}="H173 AXT";
$resn_exch{"1H18 AXT"}="H181 AXT"; $resn_exch{"2H18 AXT"}="H182 AXT"; $resn_exch{"3H18 AXT"}="H183 AXT";
$resn_exch{"1H19 AXT"}="H191 AXT"; $resn_exch{"2H19 AXT"}="H192 AXT"; $resn_exch{"3H19 AXT"}="H193 AXT";
$resn_exch{"1H20 AXT"}="H201 AXT"; $resn_exch{"2H20 AXT"}="H202 AXT"; $resn_exch{"3H20 AXT"}="H203 AXT";
$resn_exch{"1H22 AXT"}="H221 AXT"; $resn_exch{"2H22 AXT"}="H222 AXT"; $resn_exch{"1H36 AXT"}="H361 AXT";
$resn_exch{"2H36 AXT"}="H362 AXT"; $resn_exch{"3H36 AXT"}="H363 AXT"; $resn_exch{"1H37 AXT"}="H371 AXT";
$resn_exch{"2H37 AXT"}="H372 AXT"; $resn_exch{"3H37 AXT"}="H373 AXT"; $resn_exch{"1H38 AXT"}="H381 AXT";
$resn_exch{"2H38 AXT"}="H382 AXT"; $resn_exch{"3H38 AXT"}="H383 AXT"; $resn_exch{"1H39 AXT"}="H391 AXT";
$resn_exch{"2H39 AXT"}="H392 AXT"; $resn_exch{"3H39 AXT"}="H393 AXT"; $resn_exch{"1H40 AXT"}="H401 AXT";
$resn_exch{"2H40 AXT"}="H402 AXT"; $resn_exch{"3H40 AXT"}="H403 AXT"; $resn_exch{"1H3  AXX"}=" H31 AXX";
$resn_exch{"2H3  AXX"}=" H32 AXX"; $resn_exch{"1H1  AXX"}=" H11 AXX"; $resn_exch{"2H1  AXX"}=" H12 AXX";
$resn_exch{"1H18 AXX"}="H181 AXX"; $resn_exch{"2H18 AXX"}="H182 AXX"; $resn_exch{"1H17 AXX"}="H171 AXX";
$resn_exch{"2H17 AXX"}="H172 AXX"; $resn_exch{"1H16 AXX"}="H161 AXX"; $resn_exch{"2H16 AXX"}="H162 AXX";
$resn_exch{"1H15 AXX"}="H151 AXX"; $resn_exch{"2H15 AXX"}="H152 AXX"; $resn_exch{"1H14 AXX"}="H141 AXX";
$resn_exch{"2H14 AXX"}="H142 AXX"; $resn_exch{"1H11 AY1"}="H111 AY1"; $resn_exch{"2H11 AY1"}="H112 AY1";
$resn_exch{"3H11 AY1"}="H113 AY1"; $resn_exch{"1H16 AY1"}="H161 AY1"; $resn_exch{"2H16 AY1"}="H162 AY1";
$resn_exch{"3H16 AY1"}="H163 AY1"; $resn_exch{"1H18 AY1"}="H181 AY1"; $resn_exch{"2H18 AY1"}="H182 AY1";
$resn_exch{"1H19 AY1"}="H191 AY1"; $resn_exch{"2H19 AY1"}="H192 AY1"; $resn_exch{"1H20 AY1"}="H201 AY1";
$resn_exch{"2H20 AY1"}="H202 AY1"; $resn_exch{"1H21 AY1"}="H211 AY1"; $resn_exch{"2H21 AY1"}="H212 AY1";
$resn_exch{"1H22 AY1"}="H221 AY1"; $resn_exch{"2H22 AY1"}="H222 AY1"; $resn_exch{"1H23 AY1"}="H231 AY1";
$resn_exch{"2H23 AY1"}="H232 AY1"; $resn_exch{"3H23 AY1"}="H233 AY1"; $resn_exch{"1H26 AY1"}="H261 AY1";
$resn_exch{"2H26 AY1"}="H262 AY1"; $resn_exch{"3H26 AY1"}="H263 AY1"; $resn_exch{"1H27 AY1"}="H271 AY1";
$resn_exch{"2H27 AY1"}="H272 AY1"; $resn_exch{"3H27 AY1"}="H273 AY1"; $resn_exch{" HN  AYA"}=" H   AYA";
$resn_exch{"1HB  AYA"}=" HB1 AYA"; $resn_exch{"2HB  AYA"}=" HB2 AYA"; $resn_exch{"3HB  AYA"}=" HB3 AYA";
$resn_exch{"1HM  AYA"}=" HM1 AYA"; $resn_exch{"2HM  AYA"}=" HM2 AYA"; $resn_exch{"3HM  AYA"}=" HM3 AYA";
$resn_exch{"1HM2 AYD"}="HM21 AYD"; $resn_exch{"2HM2 AYD"}="HM22 AYD"; $resn_exch{"3HM2 AYD"}="HM23 AYD";
$resn_exch{"1H4' AYD"}="H4'1 AYD"; $resn_exch{"2H4' AYD"}="H4'2 AYD"; $resn_exch{"1H7' AYD"}="H7'1 AYD";
$resn_exch{"2H7' AYD"}="H7'2 AYD"; $resn_exch{"1HM4 AYD"}="HM41 AYD"; $resn_exch{"2HM4 AYD"}="HM42 AYD";
$resn_exch{"3HM4 AYD"}="HM43 AYD"; $resn_exch{"1H6  AYD"}=" H61 AYD"; $resn_exch{"2H6  AYD"}=" H62 AYD";
$resn_exch{"1H7  AYD"}=" H71 AYD"; $resn_exch{"2H7  AYD"}=" H72 AYD"; $resn_exch{"1HA3 AYG"}="HA31 AYG";
$resn_exch{"2HA3 AYG"}="HA32 AYG"; $resn_exch{"1HB1 AYG"}="HB11 AYG"; $resn_exch{"2HB1 AYG"}="HB12 AYG";
$resn_exch{"3HB1 AYG"}="HB13 AYG"; $resn_exch{"1HN1 AYG"}="HN11 AYG"; $resn_exch{" HN2 AYG"}=" H2  AYG";
$resn_exch{"1H01 AYM"}="H011 AYM"; $resn_exch{"2H01 AYM"}="H012 AYM"; $resn_exch{"1H13 AYM"}="H131 AYM";
$resn_exch{"2H13 AYM"}="H132 AYM"; $resn_exch{"3H13 AYM"}="H133 AYM"; $resn_exch{"1H14 AYM"}="H141 AYM";
$resn_exch{"2H14 AYM"}="H142 AYM"; $resn_exch{"1H20 AYM"}="H201 AYM"; $resn_exch{"2H20 AYM"}="H202 AYM";
$resn_exch{"3H20 AYM"}="H203 AYM"; $resn_exch{"1H2  AZ1"}=" H21 AZ1"; $resn_exch{"2H2  AZ1"}=" H22 AZ1";
$resn_exch{"1H3  AZ1"}=" H31 AZ1"; $resn_exch{"2H3  AZ1"}=" H32 AZ1"; $resn_exch{"1H4  AZ1"}=" H41 AZ1";
$resn_exch{"2H4  AZ1"}=" H42 AZ1"; $resn_exch{"1H5  AZ1"}=" H51 AZ1"; $resn_exch{"2H5  AZ1"}=" H52 AZ1";
$resn_exch{"1H6  AZ1"}=" H61 AZ1"; $resn_exch{"2H6  AZ1"}=" H62 AZ1"; $resn_exch{"1H7  AZ1"}=" H71 AZ1";
$resn_exch{"2H7  AZ1"}=" H72 AZ1"; $resn_exch{"1H8  AZ1"}=" H81 AZ1"; $resn_exch{"2H8  AZ1"}=" H82 AZ1";
$resn_exch{"1H2  AZ2"}=" H21 AZ2"; $resn_exch{"2H2  AZ2"}=" H22 AZ2"; $resn_exch{"1H5  AZ2"}=" H51 AZ2";
$resn_exch{"1H8  AZ2"}=" H81 AZ2"; $resn_exch{"2H8  AZ2"}=" H82 AZ2"; $resn_exch{"1H11 AZ2"}="H111 AZ2";
$resn_exch{"2H11 AZ2"}="H112 AZ2"; $resn_exch{"3H11 AZ2"}="H113 AZ2"; $resn_exch{"1H16 AZ2"}="H161 AZ2";
$resn_exch{"1H19 AZ2"}="H191 AZ2"; $resn_exch{"1H21 AZ2"}="H211 AZ2"; $resn_exch{"1H24 AZ2"}="H241 AZ2";
$resn_exch{"1H26 AZ2"}="H261 AZ2"; $resn_exch{"1H29 AZ2"}="H291 AZ2"; $resn_exch{"2H29 AZ2"}="H292 AZ2";
$resn_exch{"1H32 AZ2"}="H321 AZ2"; $resn_exch{"2H32 AZ2"}="H322 AZ2"; $resn_exch{"1H36 AZ2"}="H361 AZ2";
$resn_exch{"1H38 AZ2"}="H381 AZ2"; $resn_exch{"1H41 AZ2"}="H411 AZ2"; $resn_exch{"1H43 AZ2"}="H431 AZ2";
$resn_exch{"1H47 AZ2"}="H471 AZ2"; $resn_exch{"2H47 AZ2"}="H472 AZ2"; $resn_exch{"3H47 AZ2"}="H473 AZ2";
$resn_exch{"1H6  AZC"}=" H61 AZC"; $resn_exch{"2H6  AZC"}=" H62 AZC"; $resn_exch{"1H9  AZC"}=" H91 AZC";
$resn_exch{"2H9  AZC"}=" H92 AZC"; $resn_exch{"3H9  AZC"}=" H93 AZC"; $resn_exch{"2HOA AZD"}="HOA2 AZD";
$resn_exch{"2HOB AZD"}="HOB2 AZD"; $resn_exch{"3HOB AZD"}="HOB3 AZD"; $resn_exch{"1H5* AZD"}="H5'1 AZD";
$resn_exch{"2H5* AZD"}="H5'2 AZD"; $resn_exch{" H4* AZD"}=" H4' AZD"; $resn_exch{" H3* AZD"}=" H3' AZD";
$resn_exch{"1H2* AZD"}="H2'1 AZD"; $resn_exch{"2H2* AZD"}="H2'2 AZD"; $resn_exch{" H1* AZD"}=" H1' AZD";
$resn_exch{"1H5  AZD"}=" H51 AZD"; $resn_exch{"2H5  AZD"}=" H52 AZD"; $resn_exch{"3H5  AZD"}=" H53 AZD";
$resn_exch{"1H2  AZE"}=" H21 AZE"; $resn_exch{"2H2  AZE"}=" H22 AZE"; $resn_exch{"1H3  AZE"}=" H31 AZE";
$resn_exch{"2H3  AZE"}=" H32 AZE"; $resn_exch{"1H4  AZE"}=" H41 AZE"; $resn_exch{"2H4  AZE"}=" H42 AZE";
$resn_exch{"1H15 AZE"}="H151 AZE"; $resn_exch{"2H15 AZE"}="H152 AZE"; $resn_exch{"3H15 AZE"}="H153 AZE";
$resn_exch{"1H16 AZE"}="H161 AZE"; $resn_exch{"2H16 AZE"}="H162 AZE"; $resn_exch{"3H16 AZE"}="H163 AZE";
$resn_exch{"1H17 AZE"}="H171 AZE"; $resn_exch{"2H17 AZE"}="H172 AZE"; $resn_exch{"3H17 AZE"}="H173 AZE";
$resn_exch{"1H18 AZE"}="H181 AZE"; $resn_exch{"2H18 AZE"}="H182 AZE"; $resn_exch{"3H18 AZE"}="H183 AZE";
$resn_exch{"1H19 AZE"}="H191 AZE"; $resn_exch{"2H19 AZE"}="H192 AZE"; $resn_exch{"3H19 AZE"}="H193 AZE";
$resn_exch{"1H20 AZE"}="H201 AZE"; $resn_exch{"2H20 AZE"}="H202 AZE"; $resn_exch{"3H20 AZE"}="H203 AZE";
$resn_exch{"1H2C AZF"}="H2C1 AZF"; $resn_exch{"2H2C AZF"}="H2C2 AZF"; $resn_exch{"1H6C AZF"}="H6C1 AZF";
$resn_exch{"2H6C AZF"}="H6C2 AZF"; $resn_exch{"1HN2 AZG"}="HN21 AZG"; $resn_exch{"2HN2 AZG"}="HN22 AZG";
$resn_exch{"1HN  AZK"}=" H   AZK"; $resn_exch{"2HN  AZK"}=" H2  AZK"; $resn_exch{"1HB  AZK"}=" HB2 AZK";
$resn_exch{"2HB  AZK"}=" HB3 AZK"; $resn_exch{"1HG  AZK"}=" HG2 AZK"; $resn_exch{"2HG  AZK"}=" HG3 AZK";
$resn_exch{"1HD  AZK"}=" HD2 AZK"; $resn_exch{"2HD  AZK"}=" HD3 AZK"; $resn_exch{"1HE  AZK"}=" HE2 AZK";
$resn_exch{"2HE  AZK"}=" HE3 AZK"; $resn_exch{"1H   AZK"}=" HC1 AZK"; $resn_exch{"2H   AZK"}=" HC2 AZK";
$resn_exch{"1HN1 AZL"}="HN11 AZL"; $resn_exch{"2HN1 AZL"}="HN12 AZL"; $resn_exch{"1H2  AZL"}="H21A AZL";
$resn_exch{"2H2  AZL"}="H22A AZL"; $resn_exch{"1H3  AZL"}=" H31 AZL"; $resn_exch{"2H3  AZL"}=" H32 AZL";
$resn_exch{"1H4  AZL"}=" H41 AZL"; $resn_exch{"2H4  AZL"}=" H42 AZL"; $resn_exch{"1H5  AZL"}=" H51 AZL";
$resn_exch{"2H5  AZL"}=" H52 AZL"; $resn_exch{"1H10 AZL"}="H101 AZL"; $resn_exch{"2H10 AZL"}="H102 AZL";
$resn_exch{"1H11 AZL"}="H111 AZL"; $resn_exch{"2H11 AZL"}="H112 AZL"; $resn_exch{"1H12 AZL"}="H121 AZL";
$resn_exch{"2H12 AZL"}="H122 AZL"; $resn_exch{"1H16 AZL"}="H161 AZL"; $resn_exch{"2H16 AZL"}="H162 AZL";
$resn_exch{"1H24 AZL"}="H241 AZL"; $resn_exch{"2H24 AZL"}="H242 AZL"; $resn_exch{"1H25 AZL"}="H251 AZL";
$resn_exch{"2H25 AZL"}="H252 AZL"; $resn_exch{"3H25 AZL"}="H253 AZL"; $resn_exch{"1H4  AZM"}=" H41 AZM";
$resn_exch{"2H4  AZM"}=" H42 AZM"; $resn_exch{"3H4  AZM"}=" H43 AZM"; $resn_exch{"1HN1 AZM"}="HN11 AZM";
$resn_exch{"2HN1 AZM"}="HN12 AZM"; $resn_exch{"1H20 AZO"}="H201 AZO"; $resn_exch{"2H20 AZO"}="H202 AZO";
$resn_exch{"3H20 AZO"}="H203 AZO"; $resn_exch{"1H22 AZO"}="H221 AZO"; $resn_exch{"2H22 AZO"}="H222 AZO";
$resn_exch{"3H22 AZO"}="H223 AZO"; $resn_exch{"1HAG AZP"}="HAG1 AZP"; $resn_exch{"2HAG AZP"}="HAG2 AZP";
$resn_exch{"1HB  AZP"}=" HB1 AZP"; $resn_exch{"2HB  AZP"}=" HB2 AZP"; $resn_exch{"1HD1 AZP"}="HD11 AZP";
$resn_exch{"2HD1 AZP"}="HD12 AZP"; $resn_exch{"3HD1 AZP"}="HD13 AZP"; $resn_exch{"1HD2 AZP"}="HD21 AZP";
$resn_exch{"2HD2 AZP"}="HD22 AZP"; $resn_exch{"3HD2 AZP"}="HD23 AZP"; $resn_exch{"1HAV AZP"}="HAV1 AZP";
$resn_exch{"2HAV AZP"}="HAV2 AZP"; $resn_exch{"1HBA AZP"}="HBA1 AZP"; $resn_exch{"2HBA AZP"}="HBA2 AZP";
$resn_exch{"1HBB AZP"}="HBB1 AZP"; $resn_exch{"2HBB AZP"}="HBB2 AZP"; $resn_exch{"1HBE AZP"}="HBE1 AZP";
$resn_exch{"2HBE AZP"}="HBE2 AZP"; $resn_exch{"1HBM AZP"}="HBM1 AZP"; $resn_exch{"2HBM AZP"}="HBM2 AZP";
$resn_exch{"1HBS AZP"}="HBS1 AZP"; $resn_exch{"2HBS AZP"}="HBS2 AZP"; $resn_exch{"1HBT AZP"}="HBT1 AZP";
$resn_exch{"2HBT AZP"}="HBT2 AZP"; $resn_exch{"3HBT AZP"}="HBT3 AZP"; $resn_exch{"1H11 AZQ"}="H111 AZQ";
$resn_exch{"2H11 AZQ"}="H112 AZQ"; $resn_exch{"1H13 AZQ"}="H131 AZQ"; $resn_exch{"2H13 AZQ"}="H132 AZQ";
$resn_exch{"1H14 AZQ"}="H141 AZQ"; $resn_exch{"2H14 AZQ"}="H142 AZQ"; $resn_exch{"3H14 AZQ"}="H143 AZQ";
$resn_exch{"1H15 AZQ"}="H151 AZQ"; $resn_exch{"2H15 AZQ"}="H152 AZQ"; $resn_exch{"3H15 AZQ"}="H153 AZQ";
$resn_exch{"1H16 AZQ"}="H161 AZQ"; $resn_exch{"2H16 AZQ"}="H162 AZQ"; $resn_exch{"3H16 AZQ"}="H163 AZQ";
$resn_exch{"1H7  AZR"}=" H71 AZR"; $resn_exch{"2H7  AZR"}=" H72 AZR"; $resn_exch{"3H7  AZR"}=" H73 AZR";
$resn_exch{"1H16 AZR"}="H161 AZR"; $resn_exch{"2H16 AZR"}="H162 AZR"; $resn_exch{"1H27 AZR"}="H271 AZR";
$resn_exch{"2H27 AZR"}="H272 AZR"; $resn_exch{"3H27 AZR"}="H273 AZR"; $resn_exch{"1H28 AZR"}="H281 AZR";
$resn_exch{"2H28 AZR"}="H282 AZR"; $resn_exch{"3H28 AZR"}="H283 AZR"; $resn_exch{"2HOA AZT"}="HOA2 AZT";
$resn_exch{"2HOB AZT"}="HOB2 AZT"; $resn_exch{"2HOG AZT"}="HOG2 AZT"; $resn_exch{"3HOG AZT"}="HOG3 AZT";
$resn_exch{"1H5* AZT"}="H5'1 AZT"; $resn_exch{"2H5* AZT"}="H5'2 AZT"; $resn_exch{" H4* AZT"}=" H4' AZT";
$resn_exch{" H3* AZT"}=" H3' AZT"; $resn_exch{"1H2* AZT"}="H2'1 AZT"; $resn_exch{"2H2* AZT"}="H2'2 AZT";
$resn_exch{" H1* AZT"}=" H1' AZT"; $resn_exch{"1H5  AZT"}=" H51 AZT"; $resn_exch{"2H5  AZT"}=" H52 AZT";
$resn_exch{"3H5  AZT"}=" H53 AZT"; $resn_exch{"1H26 AZX"}="H261 AZX"; $resn_exch{"2H26 AZX"}="H262 AZX";
$resn_exch{"3H26 AZX"}="H263 AZX"; $resn_exch{"1H11 AZX"}="H111 AZX"; $resn_exch{"2H11 AZX"}="H112 AZX";
$resn_exch{"3H11 AZX"}="H113 AZX"; $resn_exch{"1H10 AZX"}="H101 AZX"; $resn_exch{"2H10 AZX"}="H102 AZX";
$resn_exch{"3H10 AZX"}="H103 AZX"; $resn_exch{"1HN  AZY"}=" HN1 AZY"; $resn_exch{"2HN  AZY"}=" HN2 AZY";
$resn_exch{"1HB  AZY"}=" HB1 AZY"; $resn_exch{"2HB  AZY"}=" HB2 AZY"; $resn_exch{"1HAU B08"}="HAU1 B08";
$resn_exch{"2HAU B08"}="HAU2 B08"; $resn_exch{"1H8  B11"}=" H81 B11"; $resn_exch{"2H8  B11"}=" H82 B11";
$resn_exch{"1H11 B11"}="H111 B11"; $resn_exch{"2H11 B11"}="H112 B11"; $resn_exch{"1H12 B11"}="H121 B11";
$resn_exch{"2H12 B11"}="H122 B11"; $resn_exch{"1H14 B11"}="H141 B11"; $resn_exch{"2H14 B11"}="H142 B11";
$resn_exch{"1H15 B11"}="H151 B11"; $resn_exch{"2H15 B11"}="H152 B11"; $resn_exch{"1H20 B12"}="H201 B12";
$resn_exch{"2H20 B12"}="H202 B12"; $resn_exch{"3H20 B12"}="H203 B12"; $resn_exch{"1H25 B12"}="H251 B12";
$resn_exch{"2H25 B12"}="H252 B12"; $resn_exch{"3H25 B12"}="H253 B12"; $resn_exch{"1H26 B12"}="H261 B12";
$resn_exch{"2H26 B12"}="H262 B12"; $resn_exch{"1H29 B12"}="H291 B12"; $resn_exch{"2H29 B12"}="H292 B12";
$resn_exch{"1H30 B12"}="H301 B12"; $resn_exch{"2H30 B12"}="H302 B12"; $resn_exch{"1H31 B12"}="H311 B12";
$resn_exch{"2H31 B12"}="H312 B12"; $resn_exch{"1H33 B12"}="H331 B12"; $resn_exch{"2H33 B12"}="H332 B12";
$resn_exch{"1H35 B12"}="H351 B12"; $resn_exch{"2H35 B12"}="H352 B12"; $resn_exch{"3H35 B12"}="H353 B12";
$resn_exch{"1H36 B12"}="H361 B12"; $resn_exch{"2H36 B12"}="H362 B12"; $resn_exch{"3H36 B12"}="H363 B12";
$resn_exch{"1H37 B12"}="H371 B12"; $resn_exch{"2H37 B12"}="H372 B12"; $resn_exch{"1H40 B12"}="H401 B12";
$resn_exch{"2H40 B12"}="H402 B12"; $resn_exch{"1H41 B12"}="H411 B12"; $resn_exch{"2H41 B12"}="H412 B12";
$resn_exch{"1H42 B12"}="H421 B12"; $resn_exch{"2H42 B12"}="H422 B12"; $resn_exch{"1H45 B12"}="H451 B12";
$resn_exch{"2H45 B12"}="H452 B12"; $resn_exch{"1H46 B12"}="H461 B12"; $resn_exch{"2H46 B12"}="H462 B12";
$resn_exch{"3H46 B12"}="H463 B12"; $resn_exch{"1H47 B12"}="H471 B12"; $resn_exch{"2H47 B12"}="H472 B12";
$resn_exch{"3H47 B12"}="H473 B12"; $resn_exch{"1H48 B12"}="H481 B12"; $resn_exch{"2H48 B12"}="H482 B12";
$resn_exch{"1H49 B12"}="H491 B12"; $resn_exch{"2H49 B12"}="H492 B12"; $resn_exch{"1H52 B12"}="H521 B12";
$resn_exch{"2H52 B12"}="H522 B12"; $resn_exch{"1H53 B12"}="H531 B12"; $resn_exch{"2H53 B12"}="H532 B12";
$resn_exch{"3H53 B12"}="H533 B12"; $resn_exch{"1H54 B12"}="H541 B12"; $resn_exch{"2H54 B12"}="H542 B12";
$resn_exch{"3H54 B12"}="H543 B12"; $resn_exch{"1H55 B12"}="H551 B12"; $resn_exch{"2H55 B12"}="H552 B12";
$resn_exch{"1H56 B12"}="H561 B12"; $resn_exch{"2H56 B12"}="H562 B12"; $resn_exch{"1H60 B12"}="H601 B12";
$resn_exch{"2H60 B12"}="H602 B12"; $resn_exch{"1H62 B12"}="H621 B12"; $resn_exch{"2H62 B12"}="H622 B12";
$resn_exch{"1H1P B12"}="H1P1 B12"; $resn_exch{"2H1P B12"}="H1P2 B12"; $resn_exch{"1H3P B12"}="H3P1 B12";
$resn_exch{"2H3P B12"}="H3P2 B12"; $resn_exch{"3H3P B12"}="H3P3 B12"; $resn_exch{"5HOP B12"}="HOP5 B12";
$resn_exch{"7HOR B12"}="HOR7 B12"; $resn_exch{"1H5R B12"}="H5R1 B12"; $resn_exch{"2H5R B12"}="H5R2 B12";
$resn_exch{"8HOR B12"}="HOR8 B12"; $resn_exch{"1HM5 B12"}="HM51 B12"; $resn_exch{"2HM5 B12"}="HM52 B12";
$resn_exch{"3HM5 B12"}="HM53 B12"; $resn_exch{"1HM6 B12"}="HM61 B12"; $resn_exch{"2HM6 B12"}="HM62 B12";
$resn_exch{"3HM6 B12"}="HM63 B12"; $resn_exch{"1H3P B13"}="H3P1 B13"; $resn_exch{"2H3P B13"}="H3P2 B13";
$resn_exch{"3H3P B13"}="H3P3 B13"; $resn_exch{"1H1P B13"}="H1P1 B13"; $resn_exch{"2H1P B13"}="H1P2 B13";
$resn_exch{"1H56 B13"}="H561 B13"; $resn_exch{"2H56 B13"}="H562 B13"; $resn_exch{"1H55 B13"}="H551 B13";
$resn_exch{"2H55 B13"}="H552 B13"; $resn_exch{"1H54 B13"}="H541 B13"; $resn_exch{"2H54 B13"}="H542 B13";
$resn_exch{"3H54 B13"}="H543 B13"; $resn_exch{"1H60 B13"}="H601 B13"; $resn_exch{"2H60 B13"}="H602 B13";
$resn_exch{"1H62 B13"}="H621 B13"; $resn_exch{"2H62 B13"}="H622 B13"; $resn_exch{"1H35 B13"}="H351 B13";
$resn_exch{"2H35 B13"}="H352 B13"; $resn_exch{"3H35 B13"}="H353 B13"; $resn_exch{"1H30 B13"}="H301 B13";
$resn_exch{"2H30 B13"}="H302 B13"; $resn_exch{"1H31 B13"}="H311 B13"; $resn_exch{"2H31 B13"}="H312 B13";
$resn_exch{"1H33 B13"}="H331 B13"; $resn_exch{"2H33 B13"}="H332 B13"; $resn_exch{"1H26 B13"}="H261 B13";
$resn_exch{"2H26 B13"}="H262 B13"; $resn_exch{"1H29 B13"}="H291 B13"; $resn_exch{"2H29 B13"}="H292 B13";
$resn_exch{"1H25 B13"}="H251 B13"; $resn_exch{"2H25 B13"}="H252 B13"; $resn_exch{"3H25 B13"}="H253 B13";
$resn_exch{"1H20 B13"}="H201 B13"; $resn_exch{"2H20 B13"}="H202 B13"; $resn_exch{"3H20 B13"}="H203 B13";
$resn_exch{"1H53 B13"}="H531 B13"; $resn_exch{"2H53 B13"}="H532 B13"; $resn_exch{"3H53 B13"}="H533 B13";
$resn_exch{"1H48 B13"}="H481 B13"; $resn_exch{"2H48 B13"}="H482 B13"; $resn_exch{"1H49 B13"}="H491 B13";
$resn_exch{"2H49 B13"}="H492 B13"; $resn_exch{"1H52 B13"}="H521 B13"; $resn_exch{"2H52 B13"}="H522 B13";
$resn_exch{"1H46 B13"}="H461 B13"; $resn_exch{"2H46 B13"}="H462 B13"; $resn_exch{"3H46 B13"}="H463 B13";
$resn_exch{"1H47 B13"}="H471 B13"; $resn_exch{"2H47 B13"}="H472 B13"; $resn_exch{"3H47 B13"}="H473 B13";
$resn_exch{"1H41 B13"}="H411 B13"; $resn_exch{"2H41 B13"}="H412 B13"; $resn_exch{"1H42 B13"}="H421 B13";
$resn_exch{"2H42 B13"}="H422 B13"; $resn_exch{"1H45 B13"}="H451 B13"; $resn_exch{"2H45 B13"}="H452 B13";
$resn_exch{"1H36 B13"}="H361 B13"; $resn_exch{"2H36 B13"}="H362 B13"; $resn_exch{"3H36 B13"}="H363 B13";
$resn_exch{"1H37 B13"}="H371 B13"; $resn_exch{"2H37 B13"}="H372 B13"; $resn_exch{"1H40 B13"}="H401 B13";
$resn_exch{"2H40 B13"}="H402 B13"; $resn_exch{"1H7  B15"}=" H71 B15"; $resn_exch{"2H7  B15"}=" H72 B15";
$resn_exch{"1H8  B15"}=" H81 B15"; $resn_exch{"2H8  B15"}=" H82 B15"; $resn_exch{"1H17 B15"}="H171 B15";
$resn_exch{"2H17 B15"}="H172 B15"; $resn_exch{"1H19 B15"}="H191 B15"; $resn_exch{"2H19 B15"}="H192 B15";
$resn_exch{"1H21 B15"}="H211 B15"; $resn_exch{"2H21 B15"}="H212 B15"; $resn_exch{"1HN1 B15"}="HN11 B15";
$resn_exch{"2HN1 B15"}="HN12 B15"; $resn_exch{"1HN  B17"}=" HN1 B17"; $resn_exch{"2HN  B17"}=" HN2 B17";
$resn_exch{"1H9  B17"}=" H91 B17"; $resn_exch{"2H9  B17"}=" H92 B17"; $resn_exch{"1H10 B17"}="H101 B17";
$resn_exch{"2H10 B17"}="H102 B17"; $resn_exch{"1H12 B17"}="H121 B17"; $resn_exch{"2H12 B17"}="H122 B17";
$resn_exch{"1H13 B17"}="H131 B17"; $resn_exch{"2H13 B17"}="H132 B17"; $resn_exch{"2H15 B17"}="H152 B17";
$resn_exch{"3H15 B17"}="H153 B17"; $resn_exch{"1H8  B17"}=" H81 B17"; $resn_exch{"2H8  B17"}=" H82 B17";
$resn_exch{"1H11 B17"}="H111 B17"; $resn_exch{"2H11 B17"}="H112 B17"; $resn_exch{"1H16 B17"}="H161 B17";
$resn_exch{"2H16 B17"}="H162 B17"; $resn_exch{"1H11 B18"}="H111 B18"; $resn_exch{"2H11 B18"}="H112 B18";
$resn_exch{"1H26 B18"}="H261 B18"; $resn_exch{"2H26 B18"}="H262 B18"; $resn_exch{"1H27 B18"}="H271 B18";
$resn_exch{"2H27 B18"}="H272 B18"; $resn_exch{"1H29 B18"}="H291 B18"; $resn_exch{"2H29 B18"}="H292 B18";
$resn_exch{"3H29 B18"}="H293 B18"; $resn_exch{"1H30 B18"}="H301 B18"; $resn_exch{"2H30 B18"}="H302 B18";
$resn_exch{"3H30 B18"}="H303 B18"; $resn_exch{"1H31 B18"}="H311 B18"; $resn_exch{"2H31 B18"}="H312 B18";
$resn_exch{"3H31 B18"}="H313 B18"; $resn_exch{"1H11 B19"}="H111 B19"; $resn_exch{"2H11 B19"}="H112 B19";
$resn_exch{"3H11 B19"}="H113 B19"; $resn_exch{"1H12 B19"}="H121 B19"; $resn_exch{"2H12 B19"}="H122 B19";
$resn_exch{"3H12 B19"}="H123 B19"; $resn_exch{"1H8  B19"}=" H81 B19"; $resn_exch{"2H8  B19"}=" H82 B19";
$resn_exch{"3H8  B19"}=" H83 B19"; $resn_exch{"1H9  B19"}=" H91 B19"; $resn_exch{"2H9  B19"}=" H92 B19";
$resn_exch{"3H9  B19"}=" H93 B19"; $resn_exch{"1H6  B19"}=" H61 B19"; $resn_exch{"2H6  B19"}=" H62 B19";
$resn_exch{"1H1  B19"}=" H11 B19"; $resn_exch{"2H1  B19"}=" H12 B19"; $resn_exch{"1HN1 B19"}="HN11 B19";
$resn_exch{"2HN1 B19"}="HN12 B19"; $resn_exch{"1HN  B1F"}=" HN1 B1F"; $resn_exch{"2HN  B1F"}=" HN2 B1F";
$resn_exch{"1HB  B1F"}=" HB1 B1F"; $resn_exch{"2HB  B1F"}=" HB2 B1F"; $resn_exch{"1H10 B1L"}="H101 B1L";
$resn_exch{"2H10 B1L"}="H102 B1L"; $resn_exch{"1H11 B1L"}="H111 B1L"; $resn_exch{"2H11 B1L"}="H112 B1L";
$resn_exch{"1H12 B1L"}="H121 B1L"; $resn_exch{"2H12 B1L"}="H122 B1L"; $resn_exch{"1H14 B1L"}="H141 B1L";
$resn_exch{"2H14 B1L"}="H142 B1L"; $resn_exch{"1H20 B1M"}="H201 B1M"; $resn_exch{"2H20 B1M"}="H202 B1M";
$resn_exch{"3H20 B1M"}="H203 B1M"; $resn_exch{"1H25 B1M"}="H251 B1M"; $resn_exch{"2H25 B1M"}="H252 B1M";
$resn_exch{"3H25 B1M"}="H253 B1M"; $resn_exch{"1H26 B1M"}="H261 B1M"; $resn_exch{"2H26 B1M"}="H262 B1M";
$resn_exch{"1H29 B1M"}="H291 B1M"; $resn_exch{"2H29 B1M"}="H292 B1M"; $resn_exch{"1H30 B1M"}="H301 B1M";
$resn_exch{"2H30 B1M"}="H302 B1M"; $resn_exch{"1H31 B1M"}="H311 B1M"; $resn_exch{"2H31 B1M"}="H312 B1M";
$resn_exch{"1H33 B1M"}="H331 B1M"; $resn_exch{"2H33 B1M"}="H332 B1M"; $resn_exch{"1H35 B1M"}="H351 B1M";
$resn_exch{"2H35 B1M"}="H352 B1M"; $resn_exch{"3H35 B1M"}="H353 B1M"; $resn_exch{"1H36 B1M"}="H361 B1M";
$resn_exch{"2H36 B1M"}="H362 B1M"; $resn_exch{"3H36 B1M"}="H363 B1M"; $resn_exch{"1H37 B1M"}="H371 B1M";
$resn_exch{"2H37 B1M"}="H372 B1M"; $resn_exch{"1H40 B1M"}="H401 B1M"; $resn_exch{"2H40 B1M"}="H402 B1M";
$resn_exch{"1H41 B1M"}="H411 B1M"; $resn_exch{"2H41 B1M"}="H412 B1M"; $resn_exch{"1H42 B1M"}="H421 B1M";
$resn_exch{"2H42 B1M"}="H422 B1M"; $resn_exch{"1H45 B1M"}="H451 B1M"; $resn_exch{"2H45 B1M"}="H452 B1M";
$resn_exch{"1H46 B1M"}="H461 B1M"; $resn_exch{"2H46 B1M"}="H462 B1M"; $resn_exch{"3H46 B1M"}="H463 B1M";
$resn_exch{"1H47 B1M"}="H471 B1M"; $resn_exch{"2H47 B1M"}="H472 B1M"; $resn_exch{"3H47 B1M"}="H473 B1M";
$resn_exch{"1H48 B1M"}="H481 B1M"; $resn_exch{"2H48 B1M"}="H482 B1M"; $resn_exch{"1H49 B1M"}="H491 B1M";
$resn_exch{"2H49 B1M"}="H492 B1M"; $resn_exch{"1H52 B1M"}="H521 B1M"; $resn_exch{"2H52 B1M"}="H522 B1M";
$resn_exch{"1H53 B1M"}="H531 B1M"; $resn_exch{"2H53 B1M"}="H532 B1M"; $resn_exch{"3H53 B1M"}="H533 B1M";
$resn_exch{"1H54 B1M"}="H541 B1M"; $resn_exch{"2H54 B1M"}="H542 B1M"; $resn_exch{"3H54 B1M"}="H543 B1M";
$resn_exch{"1H55 B1M"}="H551 B1M"; $resn_exch{"2H55 B1M"}="H552 B1M"; $resn_exch{"1H56 B1M"}="H561 B1M";
$resn_exch{"2H56 B1M"}="H562 B1M"; $resn_exch{"1H60 B1M"}="H601 B1M"; $resn_exch{"2H60 B1M"}="H602 B1M";
$resn_exch{"1H62 B1M"}="H621 B1M"; $resn_exch{"2H62 B1M"}="H622 B1M"; $resn_exch{"1H1P B1M"}="H1P1 B1M";
$resn_exch{"2H1P B1M"}="H1P2 B1M"; $resn_exch{"1H3P B1M"}="H3P1 B1M"; $resn_exch{"2H3P B1M"}="H3P2 B1M";
$resn_exch{"3H3P B1M"}="H3P3 B1M"; $resn_exch{"5HOP B1M"}="HOP5 B1M"; $resn_exch{"7HOR B1M"}="HOR7 B1M";
$resn_exch{"1H5R B1M"}="H5R1 B1M"; $resn_exch{"2H5R B1M"}="H5R2 B1M"; $resn_exch{"8HOR B1M"}="HOR8 B1M";
$resn_exch{"1HO5 B1M"}="HO51 B1M"; $resn_exch{"2HO5 B1M"}="HO52 B1M"; $resn_exch{"3HO5 B1M"}="HO53 B1M";
$resn_exch{"2HOP B1P"}="HOP2 B1P"; $resn_exch{"3HOP B1P"}="HOP3 B1P"; $resn_exch{"1H5* B1P"}=" H5' B1P";
$resn_exch{"2H5* B1P"}="H5'' B1P"; $resn_exch{" H4* B1P"}=" H4' B1P"; $resn_exch{" H1* B1P"}=" H1' B1P";
$resn_exch{"HO1* B1P"}="HO1' B1P"; $resn_exch{"1H2* B1P"}=" H2' B1P"; $resn_exch{"2H2* B1P"}="H2'' B1P";
$resn_exch{" H3* B1P"}=" H3' B1P"; $resn_exch{"H3T  B1P"}="HO3' B1P"; $resn_exch{"1H15 B1V"}="H151 B1V";
$resn_exch{"2H15 B1V"}="H152 B1V"; $resn_exch{"1H18 B1V"}="H181 B1V"; $resn_exch{"2H18 B1V"}="H182 B1V";
$resn_exch{"1H19 B1V"}="H191 B1V"; $resn_exch{"2H19 B1V"}="H192 B1V"; $resn_exch{"1H20 B1V"}="H201 B1V";
$resn_exch{"2H20 B1V"}="H202 B1V"; $resn_exch{"1H21 B1V"}="H211 B1V"; $resn_exch{"2H21 B1V"}="H212 B1V";
$resn_exch{"1H22 B1V"}="H221 B1V"; $resn_exch{"2H22 B1V"}="H222 B1V"; $resn_exch{"1HAL B21"}="HAL1 B21";
$resn_exch{"2HAL B21"}="HAL2 B21"; $resn_exch{"3HAL B21"}="HAL3 B21"; $resn_exch{"2HN1 B22"}="HN12 B22";
$resn_exch{"1HN1 B22"}="HN11 B22"; $resn_exch{"1H7  B22"}=" H71 B22"; $resn_exch{"2H7  B22"}=" H72 B22";
$resn_exch{"1H8  B22"}=" H81 B22"; $resn_exch{"2H8  B22"}=" H82 B22"; $resn_exch{"1H19 B22"}="H191 B22";
$resn_exch{"2H19 B22"}="H192 B22"; $resn_exch{"2H21 B22"}="H212 B22"; $resn_exch{"1H21 B22"}="H211 B22";
$resn_exch{"1H23 B22"}="H231 B22"; $resn_exch{"2H23 B22"}="H232 B22"; $resn_exch{"1HAU B28"}="HAU1 B28";
$resn_exch{"2HAU B28"}="HAU2 B28"; $resn_exch{"1HAS B29"}="HAS1 B29"; $resn_exch{"2HAS B29"}="HAS2 B29";
$resn_exch{"1HN  B2A"}=" H   B2A"; $resn_exch{"2HN  B2A"}=" HN2 B2A"; $resn_exch{"1HB  B2A"}=" HB1 B2A";
$resn_exch{"2HB  B2A"}=" HB2 B2A"; $resn_exch{"3HB  B2A"}=" HB3 B2A"; $resn_exch{"1HN  B2F"}=" HN1 B2F";
$resn_exch{"2HN  B2F"}=" HN2 B2F"; $resn_exch{"1HB  B2F"}=" HB1 B2F"; $resn_exch{"2HB  B2F"}=" HB2 B2F";
$resn_exch{"1H6A B2G"}="H6A1 B2G"; $resn_exch{"2H6A B2G"}="H6A2 B2G"; $resn_exch{"1H6B B2G"}="H6B1 B2G";
$resn_exch{"2H6B B2G"}="H6B2 B2G"; $resn_exch{"1HN  B2I"}=" HN1 B2I"; $resn_exch{"2HN  B2I"}=" HN2 B2I";
$resn_exch{"1HG1 B2I"}="HG11 B2I"; $resn_exch{"2HG1 B2I"}="HG12 B2I"; $resn_exch{"1HG2 B2I"}="HG21 B2I";
$resn_exch{"2HG2 B2I"}="HG22 B2I"; $resn_exch{"3HG2 B2I"}="HG23 B2I"; $resn_exch{"1HD1 B2I"}="HD11 B2I";
$resn_exch{"2HD1 B2I"}="HD12 B2I"; $resn_exch{"3HD1 B2I"}="HD13 B2I"; $resn_exch{"1HN  B2V"}=" HN1 B2V";
$resn_exch{"2HN  B2V"}=" HN2 B2V"; $resn_exch{"1HG1 B2V"}="HG11 B2V"; $resn_exch{"2HG1 B2V"}="HG12 B2V";
$resn_exch{"3HG1 B2V"}="HG13 B2V"; $resn_exch{"1HG2 B2V"}="HG21 B2V"; $resn_exch{"2HG2 B2V"}="HG22 B2V";
$resn_exch{"3HG2 B2V"}="HG23 B2V"; $resn_exch{"1HN1 B30"}="HN11 B30"; $resn_exch{"2HN1 B30"}="HN12 B30";
$resn_exch{"1H9  B30"}=" H91 B30"; $resn_exch{"2H9  B30"}=" H92 B30"; $resn_exch{"1H10 B30"}="H101 B30";
$resn_exch{"2H10 B30"}="H102 B30"; $resn_exch{"1H12 B30"}="H121 B30"; $resn_exch{"2H12 B30"}="H122 B30";
$resn_exch{"1H14 B30"}="H141 B30"; $resn_exch{"2H14 B30"}="H142 B30"; $resn_exch{"1H10 B33"}="H101 B33";
$resn_exch{"2H10 B33"}="H102 B33"; $resn_exch{"1H12 B33"}="H121 B33"; $resn_exch{"2H12 B33"}="H122 B33";
$resn_exch{"1H13 B33"}="H131 B33"; $resn_exch{"2H13 B33"}="H132 B33"; $resn_exch{"1H1  B33"}=" H11 B33";
$resn_exch{"2H1  B33"}=" H12 B33"; $resn_exch{"3H1  B33"}=" H13 B33"; $resn_exch{"1H2  B33"}=" H21 B33";
$resn_exch{"2H2  B33"}=" H22 B33"; $resn_exch{"1H4  B33"}=" H41 B33"; $resn_exch{"2H4  B33"}=" H42 B33";
$resn_exch{"1H5  B33"}=" H51 B33"; $resn_exch{"2H5  B33"}=" H52 B33"; $resn_exch{"1H6  B33"}=" H61 B33";
$resn_exch{"2H6  B33"}=" H62 B33"; $resn_exch{"1H8  B33"}=" H81 B33"; $resn_exch{"2H8  B33"}=" H82 B33";
$resn_exch{"1H9  B33"}=" H91 B33"; $resn_exch{"2H9  B33"}=" H92 B33"; $resn_exch{"1H14 B33"}="H141 B33";
$resn_exch{"2H14 B33"}="H142 B33"; $resn_exch{"3H14 B33"}="H143 B33"; $resn_exch{"1HG  B3A"}=" HG1 B3A";
$resn_exch{"2HG  B3A"}=" HG2 B3A"; $resn_exch{"3HG  B3A"}=" HG3 B3A"; $resn_exch{"1HN  B3A"}=" H   B3A";
$resn_exch{"2HN  B3A"}=" H2  B3A"; $resn_exch{"1HA  B3A"}=" HA1 B3A"; $resn_exch{"2HA  B3A"}=" HA2 B3A";
$resn_exch{"HOE1 B3D"}=" HE1 B3D"; $resn_exch{"1HG  B3D"}=" HG3 B3D"; $resn_exch{"2HG  B3D"}=" HG2 B3D";
$resn_exch{"1HN  B3D"}=" H   B3D"; $resn_exch{"2HN  B3D"}=" H2  B3D"; $resn_exch{"1HA  B3D"}=" HA1 B3D";
$resn_exch{"2HA  B3D"}=" HA2 B3D"; $resn_exch{"1HN  B3E"}=" H   B3E"; $resn_exch{"2HN  B3E"}=" H2  B3E";
$resn_exch{"1HG  B3E"}=" HG2 B3E"; $resn_exch{"2HG  B3E"}=" HG3 B3E"; $resn_exch{"1HD  B3E"}=" HD2 B3E";
$resn_exch{"2HD  B3E"}=" HD3 B3E"; $resn_exch{"1HA  B3E"}=" HA1 B3E"; $resn_exch{"2HA  B3E"}=" HA2 B3E";
$resn_exch{"1H12 B3H"}="H121 B3H"; $resn_exch{"2H12 B3H"}="H122 B3H"; $resn_exch{"1H13 B3H"}="H131 B3H";
$resn_exch{"2H13 B3H"}="H132 B3H"; $resn_exch{"1H14 B3H"}="H141 B3H"; $resn_exch{"2H14 B3H"}="H142 B3H";
$resn_exch{"1H15 B3H"}="H151 B3H"; $resn_exch{"2H15 B3H"}="H152 B3H"; $resn_exch{"1H16 B3H"}="H161 B3H";
$resn_exch{"2H16 B3H"}="H162 B3H"; $resn_exch{"1H17 B3H"}="H171 B3H"; $resn_exch{"2H17 B3H"}="H172 B3H";
$resn_exch{"1H18 B3H"}="H181 B3H"; $resn_exch{"2H18 B3H"}="H182 B3H"; $resn_exch{"1H19 B3H"}="H191 B3H";
$resn_exch{"2H19 B3H"}="H192 B3H"; $resn_exch{"3H19 B3H"}="H193 B3H"; $resn_exch{"1H22 B3H"}="H221 B3H";
$resn_exch{"2H22 B3H"}="H222 B3H"; $resn_exch{"1H23 B3H"}="H231 B3H"; $resn_exch{"2H23 B3H"}="H232 B3H";
$resn_exch{"1H24 B3H"}="H241 B3H"; $resn_exch{"2H24 B3H"}="H242 B3H"; $resn_exch{"3H24 B3H"}="H243 B3H";
$resn_exch{"1H41 B3H"}="H411 B3H"; $resn_exch{"2H41 B3H"}="H412 B3H"; $resn_exch{"1H43 B3H"}="H431 B3H";
$resn_exch{"2H43 B3H"}="H432 B3H"; $resn_exch{"1HN  B3K"}=" H   B3K"; $resn_exch{"2HN  B3K"}=" H2  B3K";
$resn_exch{"1HG  B3K"}=" HG2 B3K"; $resn_exch{"2HG  B3K"}=" HG3 B3K"; $resn_exch{"1HD  B3K"}=" HD2 B3K";
$resn_exch{"2HD  B3K"}=" HD3 B3K"; $resn_exch{"1HE  B3K"}=" HE2 B3K"; $resn_exch{"2HE  B3K"}=" HE3 B3K";
$resn_exch{"1HF  B3K"}=" HF1 B3K"; $resn_exch{"2HF  B3K"}=" HF2 B3K"; $resn_exch{"1HNZ B3K"}="HNZ1 B3K";
$resn_exch{"2HNZ B3K"}="HNZ2 B3K"; $resn_exch{"1HA  B3K"}=" HA1 B3K"; $resn_exch{"2HA  B3K"}=" HA2 B3K";
$resn_exch{"1H22 B3N"}="H221 B3N"; $resn_exch{"2H22 B3N"}="H222 B3N"; $resn_exch{"3H22 B3N"}="H223 B3N";
$resn_exch{"1H21 B3N"}="H211 B3N"; $resn_exch{"2H21 B3N"}="H212 B3N"; $resn_exch{"3H21 B3N"}="H213 B3N";
$resn_exch{"1H10 B3N"}="H101 B3N"; $resn_exch{"2H10 B3N"}="H102 B3N"; $resn_exch{"1H9  B3N"}=" H91 B3N";
$resn_exch{"2H9  B3N"}=" H92 B3N"; $resn_exch{"1H8  B3N"}=" H81 B3N"; $resn_exch{"2H8  B3N"}=" H82 B3N";
$resn_exch{"1H7  B3N"}=" H71 B3N"; $resn_exch{"2H7  B3N"}=" H72 B3N"; $resn_exch{"1H6  B3N"}=" H61 B3N";
$resn_exch{"2H6  B3N"}=" H62 B3N"; $resn_exch{"1H5  B3N"}=" H51 B3N"; $resn_exch{"2H5  B3N"}=" H52 B3N";
$resn_exch{"1H1  B3P"}=" H11 B3P"; $resn_exch{"2H1  B3P"}=" H12 B3P"; $resn_exch{"1H2  B3P"}=" H21 B3P";
$resn_exch{"2H2  B3P"}=" H22 B3P"; $resn_exch{"1H3  B3P"}=" H31 B3P"; $resn_exch{"2H3  B3P"}=" H32 B3P";
$resn_exch{"1H5  B3P"}=" H51 B3P"; $resn_exch{"2H5  B3P"}=" H52 B3P"; $resn_exch{"1H6  B3P"}=" H61 B3P";
$resn_exch{"2H6  B3P"}=" H62 B3P"; $resn_exch{"1H7  B3P"}=" H71 B3P"; $resn_exch{"2H7  B3P"}=" H72 B3P";
$resn_exch{"1H9  B3P"}=" H91 B3P"; $resn_exch{"2H9  B3P"}=" H92 B3P"; $resn_exch{"1H10 B3P"}="H101 B3P";
$resn_exch{"2H10 B3P"}="H102 B3P"; $resn_exch{"1H11 B3P"}="H111 B3P"; $resn_exch{"2H11 B3P"}="H112 B3P";
$resn_exch{"1HN  B3S"}=" H   B3S"; $resn_exch{"2HN  B3S"}=" H2  B3S"; $resn_exch{" HOD B3S"}=" HD  B3S";
$resn_exch{"1HG  B3S"}=" HG3 B3S"; $resn_exch{"2HG  B3S"}=" HG2 B3S"; $resn_exch{"1HA  B3S"}=" HA1 B3S";
$resn_exch{"2HA  B3S"}=" HA2 B3S"; $resn_exch{"1HE2 B3X"}="HE21 B3X"; $resn_exch{"2HE2 B3X"}="HE22 B3X";
$resn_exch{"1HG  B3X"}=" HG3 B3X"; $resn_exch{"2HG  B3X"}=" HG2 B3X"; $resn_exch{"1HN  B3X"}=" H   B3X";
$resn_exch{"2HN  B3X"}=" H2  B3X"; $resn_exch{"1HA  B3X"}=" HA1 B3X"; $resn_exch{"2HA  B3X"}=" HA2 B3X";
$resn_exch{"1HA  B3Y"}=" HA1 B3Y"; $resn_exch{"2HA  B3Y"}=" HA2 B3Y"; $resn_exch{"1HN  B3Y"}=" H   B3Y";
$resn_exch{"2HN  B3Y"}=" H2  B3Y"; $resn_exch{"1HG  B3Y"}=" HG2 B3Y"; $resn_exch{"2HG  B3Y"}=" HG3 B3Y";
$resn_exch{"1HJ6 B4G"}="HJ61 B4G"; $resn_exch{"2HJ6 B4G"}="HJ62 B4G"; $resn_exch{"1HQ6 B4G"}="HQ61 B4G";
$resn_exch{"2HQ6 B4G"}="HQ62 B4G"; $resn_exch{"1H6A B4G"}="H6A1 B4G"; $resn_exch{"2HOA B4P"}="HOA2 B4P";
$resn_exch{"2HOB B4P"}="HOB2 B4P"; $resn_exch{"2HOG B4P"}="HOG2 B4P"; $resn_exch{"2HOD B4P"}="HOD2 B4P";
$resn_exch{"AH51 B4P"}="H51A B4P"; $resn_exch{"AH52 B4P"}="H52A B4P"; $resn_exch{"AH4* B4P"}=" H4E B4P";
$resn_exch{"AH3* B4P"}=" H3E B4P"; $resn_exch{"AHO3 B4P"}="HO3A B4P"; $resn_exch{"AH2* B4P"}=" H2E B4P";
$resn_exch{"AHO2 B4P"}="HO2A B4P"; $resn_exch{"AH1* B4P"}=" H1E B4P"; $resn_exch{" AH8 B4P"}=" H8A B4P";
$resn_exch{"AH61 B4P"}="H61A B4P"; $resn_exch{"AH62 B4P"}="H62A B4P"; $resn_exch{" AH2 B4P"}=" H2A B4P";
$resn_exch{"BH51 B4P"}="H51B B4P"; $resn_exch{"BH52 B4P"}="H52B B4P"; $resn_exch{"BH4* B4P"}=" H4F B4P";
$resn_exch{"BH3* B4P"}=" H3F B4P"; $resn_exch{"BHO3 B4P"}="HO3B B4P"; $resn_exch{"BH2* B4P"}=" H2F B4P";
$resn_exch{"BHO2 B4P"}="HO2B B4P"; $resn_exch{"BH1* B4P"}=" H1F B4P"; $resn_exch{" BH8 B4P"}=" H8B B4P";
$resn_exch{"BH61 B4P"}="H61B B4P"; $resn_exch{"BH62 B4P"}="H62B B4P"; $resn_exch{" BH2 B4P"}=" H2B B4P";
$resn_exch{"1HAY B75"}="HAY1 B75"; $resn_exch{"2HAY B75"}="HAY2 B75"; $resn_exch{"1HAU B76"}="HAU1 B76";
$resn_exch{"2HAU B76"}="HAU2 B76"; $resn_exch{"1H5  B7G"}=" H51 B7G"; $resn_exch{"1H1  B7G"}=" H11 B7G";
$resn_exch{"1H2  B7G"}=" H21 B7G"; $resn_exch{"1H3  B7G"}=" H31 B7G"; $resn_exch{"1H4  B7G"}=" H41 B7G";
$resn_exch{"1H6  B7G"}=" H61 B7G"; $resn_exch{"2H6  B7G"}=" H62 B7G"; $resn_exch{"1H7  B7G"}=" H71 B7G";
$resn_exch{"2H7  B7G"}=" H72 B7G"; $resn_exch{"1H8  B7G"}=" H81 B7G"; $resn_exch{"2H8  B7G"}=" H82 B7G";
$resn_exch{"1H9  B7G"}=" H91 B7G"; $resn_exch{"2H9  B7G"}=" H92 B7G"; $resn_exch{"1H10 B7G"}="H101 B7G";
$resn_exch{"2H10 B7G"}="H102 B7G"; $resn_exch{"1H11 B7G"}="H111 B7G"; $resn_exch{"2H11 B7G"}="H112 B7G";
$resn_exch{"1H12 B7G"}="H121 B7G"; $resn_exch{"2H12 B7G"}="H122 B7G"; $resn_exch{"1H13 B7G"}="H131 B7G";
$resn_exch{"2H13 B7G"}="H132 B7G"; $resn_exch{"3H13 B7G"}="H133 B7G"; $resn_exch{"1H10 B8L"}="H101 B8L";
$resn_exch{"2H10 B8L"}="H102 B8L"; $resn_exch{"1H11 B8L"}="H111 B8L"; $resn_exch{"2H11 B8L"}="H112 B8L";
$resn_exch{"1H12 B8L"}="H121 B8L"; $resn_exch{"2H12 B8L"}="H122 B8L"; $resn_exch{"1H14 B8L"}="H141 B8L";
$resn_exch{"2H14 B8L"}="H142 B8L"; $resn_exch{"1H71 B8L"}="H711 B8L"; $resn_exch{"2H71 B8L"}="H712 B8L";
$resn_exch{"1H72 B8L"}="H721 B8L"; $resn_exch{"2H72 B8L"}="H722 B8L"; $resn_exch{"3H72 B8L"}="H723 B8L";
$resn_exch{"1H81 B8L"}="H811 B8L"; $resn_exch{"2H81 B8L"}="H812 B8L"; $resn_exch{"3H81 B8L"}="H813 B8L";
$resn_exch{"1H30 B8L"}="H301 B8L"; $resn_exch{"2H30 B8L"}="H302 B8L"; $resn_exch{"3H30 B8L"}="H303 B8L";
$resn_exch{"1H17 B96"}="H171 B96"; $resn_exch{"2H17 B96"}="H172 B96"; $resn_exch{"3H17 B96"}="H173 B96";
$resn_exch{"1H18 B96"}="H181 B96"; $resn_exch{"2H18 B96"}="H182 B96"; $resn_exch{"3H18 B96"}="H183 B96";
$resn_exch{"1H19 B96"}="H191 B96"; $resn_exch{"2H19 B96"}="H192 B96"; $resn_exch{"3H19 B96"}="H193 B96";
$resn_exch{"1H25 B96"}="H251 B96"; $resn_exch{"2H25 B96"}="H252 B96"; $resn_exch{"3H25 B96"}="H253 B96";
$resn_exch{"1H42 B96"}="H421 B96"; $resn_exch{"2H42 B96"}="H422 B96"; $resn_exch{"1H43 B96"}="H431 B96";
$resn_exch{"2H43 B96"}="H432 B96"; $resn_exch{"1H45 B96"}="H451 B96"; $resn_exch{"2H45 B96"}="H452 B96";
$resn_exch{"1H46 B96"}="H461 B96"; $resn_exch{"2H46 B96"}="H462 B96"; $resn_exch{"1H48 B96"}="H481 B96";
$resn_exch{"2H48 B96"}="H482 B96"; $resn_exch{"1H49 B96"}="H491 B96"; $resn_exch{"2H49 B96"}="H492 B96";
$resn_exch{" AH1 B9A"}=" H1A B9A"; $resn_exch{" AH2 B9A"}=" H2A B9A"; $resn_exch{" AH3 B9A"}=" H3A B9A";
$resn_exch{" AH5 B9A"}=" H5A B9A"; $resn_exch{" AH6 B9A"}=" H6A B9A"; $resn_exch{" AH7 B9A"}=" H7A B9A";
$resn_exch{" AH8 B9A"}=" H8A B9A"; $resn_exch{" AH9 B9A"}=" H9A B9A"; $resn_exch{"1H15 B9A"}="H151 B9A";
$resn_exch{"2H15 B9A"}="H152 B9A"; $resn_exch{"1H16 B9A"}="H161 B9A"; $resn_exch{"2H16 B9A"}="H162 B9A";
$resn_exch{"1H17 B9A"}="H171 B9A"; $resn_exch{"2H17 B9A"}="H172 B9A"; $resn_exch{"1H19 B9A"}="H191 B9A";
$resn_exch{"2H19 B9A"}="H192 B9A"; $resn_exch{"1H20 B9A"}="H201 B9A"; $resn_exch{"2H20 B9A"}="H202 B9A";
$resn_exch{"1H21 B9A"}="H211 B9A"; $resn_exch{"2H21 B9A"}="H212 B9A"; $resn_exch{"1H22 B9A"}="H221 B9A";
$resn_exch{"2H22 B9A"}="H222 B9A"; $resn_exch{"1H24 B9A"}="H241 B9A"; $resn_exch{"2H24 B9A"}="H242 B9A";
$resn_exch{"1H25 B9A"}="H251 B9A"; $resn_exch{"2H25 B9A"}="H252 B9A"; $resn_exch{"1H26 B9A"}="H261 B9A";
$resn_exch{"2H26 B9A"}="H262 B9A"; $resn_exch{"1H28 B9A"}="H281 B9A"; $resn_exch{"2H28 B9A"}="H282 B9A";
$resn_exch{"1H30 B9A"}="H301 B9A"; $resn_exch{"2H30 B9A"}="H302 B9A"; $resn_exch{"1H31 B9A"}="H311 B9A";
$resn_exch{"2H31 B9A"}="H312 B9A"; $resn_exch{"1H35 B9A"}="H351 B9A"; $resn_exch{"2H35 B9A"}="H352 B9A";
$resn_exch{"1H36 B9A"}="H361 B9A"; $resn_exch{"2H36 B9A"}="H362 B9A"; $resn_exch{"1H40 B9A"}="H401 B9A";
$resn_exch{"2H40 B9A"}="H402 B9A"; $resn_exch{"1H41 B9A"}="H411 B9A"; $resn_exch{"2H41 B9A"}="H412 B9A";
$resn_exch{"1H43 B9A"}="H431 B9A"; $resn_exch{"2H43 B9A"}="H432 B9A"; $resn_exch{" BH1 B9A"}=" H1B B9A";
$resn_exch{" BH2 B9A"}=" H2B B9A"; $resn_exch{" BH3 B9A"}=" H3B B9A"; $resn_exch{" BH4 B9A"}=" H4B B9A";
$resn_exch{" BH6 B9A"}=" H6B B9A"; $resn_exch{" BH7 B9A"}=" H7B B9A"; $resn_exch{" BH8 B9A"}=" H8B B9A";
$resn_exch{" BH9 B9A"}=" H9B B9A"; $resn_exch{"'HO5 BA1"}="HO5' BA1"; $resn_exch{"'HN1 BA1"}="HN1' BA1";
$resn_exch{"1H2  BA1"}=" H21 BA1"; $resn_exch{"2H2  BA1"}=" H22 BA1"; $resn_exch{"1H5  BA1"}=" H51 BA1";
$resn_exch{"2H5  BA1"}=" H52 BA1"; $resn_exch{"1H6  BA1"}=" H61 BA1"; $resn_exch{"2H6  BA1"}=" H62 BA1";
$resn_exch{"1H7  BA1"}=" H71 BA1"; $resn_exch{"2H7  BA1"}=" H72 BA1"; $resn_exch{" H3\" BA1"}=" H3B BA1";
$resn_exch{"\"HO4 BA1"}="HO4' BA1"; $resn_exch{"\"HO6 BA1"}="HO6' BA1"; $resn_exch{" H7\" BA1"}=" H7B BA1";
$resn_exch{"AH2O BA1"}="H2OA BA1"; $resn_exch{"2H11 BA1"}="H112 BA1"; $resn_exch{"2H12 BA1"}="H122 BA1";
$resn_exch{"2H13 BA1"}="H132 BA1"; $resn_exch{"FH2O BA1"}="H2OF BA1"; $resn_exch{"1H27 BA2"}="H271 BA2";
$resn_exch{"2H27 BA2"}="H272 BA2"; $resn_exch{"1H24 BA2"}="H241 BA2"; $resn_exch{"2H24 BA2"}="H242 BA2";
$resn_exch{"1H23 BA2"}="H231 BA2"; $resn_exch{"2H23 BA2"}="H232 BA2"; $resn_exch{"1H22 BA2"}="H221 BA2";
$resn_exch{"2H22 BA2"}="H222 BA2"; $resn_exch{"1H21 BA2"}="H211 BA2"; $resn_exch{"2H21 BA2"}="H212 BA2";
$resn_exch{"1H17 BA2"}="H171 BA2"; $resn_exch{"2H17 BA2"}="H172 BA2"; $resn_exch{"1H16 BA2"}="H161 BA2";
$resn_exch{"2H16 BA2"}="H162 BA2"; $resn_exch{"1H15 BA2"}="H151 BA2"; $resn_exch{"2H15 BA2"}="H152 BA2";
$resn_exch{"1H14 BA2"}="H141 BA2"; $resn_exch{"2H14 BA2"}="H142 BA2"; $resn_exch{" H1* BA2"}=" H1' BA2";
$resn_exch{"1H6N BA2"}="H6N1 BA2"; $resn_exch{"2H6N BA2"}="H6N2 BA2"; $resn_exch{" H2* BA2"}=" H2' BA2";
$resn_exch{" H3* BA2"}=" H3' BA2"; $resn_exch{" H4* BA2"}=" H4' BA2"; $resn_exch{"1H5* BA2"}="H5'1 BA2";
$resn_exch{"2H5* BA2"}="H5'2 BA2"; $resn_exch{"1H11 BA2"}="H111 BA2"; $resn_exch{"2H11 BA2"}="H112 BA2";
$resn_exch{"1H12 BA2"}="H121 BA2"; $resn_exch{"2H12 BA2"}="H122 BA2"; $resn_exch{"3H12 BA2"}="H123 BA2";
$resn_exch{"1H6A BA3"}="H6A1 BA3"; $resn_exch{"2H6A BA3"}="H6A2 BA3"; $resn_exch{"1H5B BA3"}="H5B1 BA3";
$resn_exch{"2H5B BA3"}="H5B2 BA3"; $resn_exch{"1H5D BA3"}="H5D1 BA3"; $resn_exch{"2H5D BA3"}="H5D2 BA3";
$resn_exch{"1H6G BA3"}="H6G1 BA3"; $resn_exch{"2H6G BA3"}="H6G2 BA3"; $resn_exch{"1HN3 BAA"}="HN31 BAA";
$resn_exch{"2HN3 BAA"}="HN32 BAA"; $resn_exch{"1HB1 BAA"}="HB11 BAA"; $resn_exch{"2HB1 BAA"}="HB12 BAA";
$resn_exch{"3HB1 BAA"}="HB13 BAA"; $resn_exch{"1HB2 BAA"}="HB21 BAA"; $resn_exch{"2HB2 BAA"}="HB22 BAA";
$resn_exch{"3HB2 BAA"}="HB23 BAA"; $resn_exch{"1H41 BAA"}="H411 BAA"; $resn_exch{"2H41 BAA"}="H412 BAA";
$resn_exch{"3H41 BAA"}="H413 BAA"; $resn_exch{"1H42 BAA"}="H421 BAA"; $resn_exch{"2H42 BAA"}="H422 BAA";
$resn_exch{"3H42 BAA"}="H423 BAA"; $resn_exch{"1H43 BAA"}="H431 BAA"; $resn_exch{"2H43 BAA"}="H432 BAA";
$resn_exch{"3H43 BAA"}="H433 BAA"; $resn_exch{"1H9  BAB"}=" H91 BAB"; $resn_exch{"2H9  BAB"}=" H92 BAB";
$resn_exch{"1HN2 BAB"}="HN21 BAB"; $resn_exch{"2HN2 BAB"}="HN22 BAB"; $resn_exch{"1H2' BAB"}="H2'1 BAB";
$resn_exch{"2H2' BAB"}="H2'2 BAB"; $resn_exch{"'HN4 BAB"}="HN4' BAB"; $resn_exch{"1H1' BAB"}=" HN1 BAB";
$resn_exch{"2H1' BAB"}=" HN2 BAB"; $resn_exch{"'HN3 BAB"}=" HN3 BAB"; $resn_exch{"1HN1 BAB"}=" HN5 BAB";
$resn_exch{"2HN1 BAB"}=" HN6 BAB"; $resn_exch{"1H3  BAC"}=" H31 BAC"; $resn_exch{"2H3  BAC"}=" H32 BAC";
$resn_exch{"1HB1 BAF"}="HB11 BAF"; $resn_exch{"2HB1 BAF"}="HB12 BAF"; $resn_exch{"3HB1 BAF"}="HB13 BAF";
$resn_exch{"1HB2 BAF"}="HB21 BAF"; $resn_exch{"2HB2 BAF"}="HB22 BAF"; $resn_exch{"3HB2 BAF"}="HB23 BAF";
$resn_exch{"1H41 BAF"}="H411 BAF"; $resn_exch{"2H41 BAF"}="H412 BAF"; $resn_exch{"3H41 BAF"}="H413 BAF";
$resn_exch{"1H42 BAF"}="H421 BAF"; $resn_exch{"2H42 BAF"}="H422 BAF"; $resn_exch{"3H42 BAF"}="H423 BAF";
$resn_exch{"1H43 BAF"}="H431 BAF"; $resn_exch{"2H43 BAF"}="H432 BAF"; $resn_exch{"3H43 BAF"}="H433 BAF";
$resn_exch{"1H18 BAG"}="H181 BAG"; $resn_exch{"2H18 BAG"}="H182 BAG"; $resn_exch{"1H3  BAG"}=" H31 BAG";
$resn_exch{"2H3  BAG"}=" H32 BAG"; $resn_exch{"1H4  BAG"}=" H41 BAG"; $resn_exch{"2H4  BAG"}=" H42 BAG";
$resn_exch{"1HD  BAG"}=" HD1 BAG"; $resn_exch{"2HD  BAG"}=" HD2 BAG"; $resn_exch{"1HH2 BAG"}="HH21 BAG";
$resn_exch{"2HH2 BAG"}="HH22 BAG"; $resn_exch{"1HN1 BAH"}="HN11 BAH"; $resn_exch{"2HN1 BAH"}="HN12 BAH";
$resn_exch{"1HN2 BAH"}="HN21 BAH"; $resn_exch{"2HN2 BAH"}="HN22 BAH"; $resn_exch{"'HC2 BAH"}="HC2' BAH";
$resn_exch{"'HC3 BAH"}="HC3' BAH"; $resn_exch{"'HC6 BAH"}="HC6' BAH"; $resn_exch{"1H1' BAH"}="H1'1 BAH";
$resn_exch{"2H1' BAH"}="H1'2 BAH"; $resn_exch{"1H2' BAH"}="H2'1 BAH"; $resn_exch{"2H2' BAH"}="H2'2 BAH";
$resn_exch{"'HN4 BAH"}="HN4' BAH"; $resn_exch{"1H9  BAI"}=" H91 BAI"; $resn_exch{"2H9  BAI"}=" H92 BAI";
$resn_exch{"1HH1 BAI"}="HH11 BAI"; $resn_exch{"2HH1 BAI"}="HH12 BAI"; $resn_exch{"1HH2 BAI"}="HH21 BAI";
$resn_exch{"'HN3 BAI"}="HN3' BAI"; $resn_exch{"1HH1 BAK"}="HH11 BAK"; $resn_exch{"2HH1 BAK"}="HH12 BAK";
$resn_exch{"1HH2 BAK"}="HH21 BAK"; $resn_exch{"2HH2 BAK"}="HH22 BAK"; $resn_exch{"1H1' BAK"}="H1'1 BAK";
$resn_exch{"2H1' BAK"}="H1'2 BAK"; $resn_exch{"1H2' BAK"}="H2'1 BAK"; $resn_exch{"2H2' BAK"}="H2'2 BAK";
$resn_exch{"'HN4 BAK"}="HN4' BAK"; $resn_exch{"1HN  BAL"}=" H   BAL"; $resn_exch{"2HN  BAL"}=" H2  BAL";
$resn_exch{"1HB  BAL"}=" HB3 BAL"; $resn_exch{"2HB  BAL"}=" HB2 BAL"; $resn_exch{"1HA  BAL"}=" HA1 BAL";
$resn_exch{"2HA  BAL"}=" HA2 BAL"; $resn_exch{"1HC2 BAM"}="HC21 BAM"; $resn_exch{"1HC3 BAM"}="HC31 BAM";
$resn_exch{"1HC4 BAM"}="HC41 BAM"; $resn_exch{"1HC5 BAM"}="HC51 BAM"; $resn_exch{"1HC6 BAM"}="HC61 BAM";
$resn_exch{"1HN1 BAM"}="HN11 BAM"; $resn_exch{"2HN1 BAM"}="HN12 BAM"; $resn_exch{"1HN2 BAM"}="HN21 BAM";
$resn_exch{"2HN2 BAM"}="HN22 BAM"; $resn_exch{"6H   BAN"}=" H6  BAN"; $resn_exch{"7H   BAN"}=" H7  BAN";
$resn_exch{"3H2  BAN"}=" H23 BAN"; $resn_exch{"4H2  BAN"}=" H24 BAN"; $resn_exch{"5H2  BAN"}=" H25 BAN";
$resn_exch{"9H2  BAN"}=" H29 BAN"; $resn_exch{"9H3  BAN"}=" H39 BAN"; $resn_exch{"1HN2 BAO"}="HN21 BAO";
$resn_exch{"2HN2 BAO"}="HN22 BAO"; $resn_exch{"1H2' BAO"}="H2'1 BAO"; $resn_exch{"2H2' BAO"}="H2'2 BAO";
$resn_exch{"'HN4 BAO"}="HN4' BAO"; $resn_exch{"1H4' BAP"}="H4'1 BAP"; $resn_exch{"2H4' BAP"}="H4'2 BAP";
$resn_exch{"1H3  BAR"}=" H31 BAR"; $resn_exch{"2H3  BAR"}=" H32 BAR"; $resn_exch{"1H6  BAR"}=" H61 BAR";
$resn_exch{"2H6  BAR"}=" H62 BAR"; $resn_exch{"1HC1 BAS"}="HC11 BAS"; $resn_exch{"1H31 BAS"}="H311 BAS";
$resn_exch{"2H31 BAS"}="H312 BAS"; $resn_exch{"1H32 BAS"}="H321 BAS"; $resn_exch{"2H32 BAS"}="H322 BAS";
$resn_exch{"3H32 BAS"}="H323 BAS"; $resn_exch{"1H3  BAT"}=" H31 BAT"; $resn_exch{"2H3  BAT"}=" H32 BAT";
$resn_exch{"1H9  BAT"}=" H91 BAT"; $resn_exch{"2H9  BAT"}=" H92 BAT"; $resn_exch{"1H11 BAT"}="H111 BAT";
$resn_exch{"2H11 BAT"}="H112 BAT"; $resn_exch{"3H11 BAT"}="H113 BAT"; $resn_exch{"1H12 BAT"}="H121 BAT";
$resn_exch{"2H12 BAT"}="H122 BAT"; $resn_exch{"3H12 BAT"}="H123 BAT"; $resn_exch{"1H16 BAT"}="H161 BAT";
$resn_exch{"2H16 BAT"}="H162 BAT"; $resn_exch{"3H16 BAT"}="H163 BAT"; $resn_exch{"1H17 BAT"}="H171 BAT";
$resn_exch{"2H17 BAT"}="H172 BAT"; $resn_exch{"1HAJ BAU"}="HAJ1 BAU"; $resn_exch{"2HAJ BAU"}="HAJ2 BAU";
$resn_exch{"1HAK BAU"}="HAK1 BAU"; $resn_exch{"2HAK BAU"}="HAK2 BAU"; $resn_exch{"1HAM BAU"}="HAM1 BAU";
$resn_exch{"2HAM BAU"}="HAM2 BAU"; $resn_exch{"1HAL BAU"}="HAL1 BAU"; $resn_exch{"2HAL BAU"}="HAL2 BAU";
$resn_exch{"1H31 BAX"}="H311 BAX"; $resn_exch{"2H31 BAX"}="H312 BAX"; $resn_exch{"3H31 BAX"}="H313 BAX";
$resn_exch{"1H1  BAY"}=" H11 BAY"; $resn_exch{"2H1  BAY"}="H12A BAY"; $resn_exch{"3H1  BAY"}="H13A BAY";
$resn_exch{"1H2  BAY"}=" H21 BAY"; $resn_exch{"2H2  BAY"}=" H22 BAY"; $resn_exch{"3H2  BAY"}=" H23 BAY";
$resn_exch{"1H3  BAY"}="H31A BAY"; $resn_exch{"2H3  BAY"}=" H32 BAY"; $resn_exch{"3H3  BAY"}=" H33 BAY";
$resn_exch{"1H8  BAY"}=" H81 BAY"; $resn_exch{"2H8  BAY"}=" H82 BAY"; $resn_exch{"1H10 BAY"}="H101 BAY";
$resn_exch{"2H10 BAY"}="H102 BAY"; $resn_exch{"1H26 BAY"}="H261 BAY"; $resn_exch{"2H26 BAY"}="H262 BAY";
$resn_exch{"3H26 BAY"}="H263 BAY"; $resn_exch{"1H27 BAY"}="H271 BAY"; $resn_exch{"2H27 BAY"}="H272 BAY";
$resn_exch{"3H27 BAY"}="H273 BAY"; $resn_exch{"1H32 BAY"}="H321 BAY"; $resn_exch{"2H32 BAY"}="H322 BAY";
$resn_exch{"1H44 BAY"}="H441 BAY"; $resn_exch{"2H44 BAY"}="H442 BAY"; $resn_exch{"1H56 BAY"}="H561 BAY";
$resn_exch{"2H56 BAY"}="H562 BAY"; $resn_exch{"3H56 BAY"}="H563 BAY"; $resn_exch{"1H57 BAY"}="H571 BAY";
$resn_exch{"2H57 BAY"}="H572 BAY"; $resn_exch{"3H57 BAY"}="H573 BAY"; $resn_exch{"1H62 BAY"}="H621 BAY";
$resn_exch{"2H62 BAY"}="H622 BAY"; $resn_exch{"1H73 BAY"}="H731 BAY"; $resn_exch{"2H73 BAY"}="H732 BAY";
$resn_exch{"1H78 BAY"}="H781 BAY"; $resn_exch{"2H78 BAY"}="H782 BAY"; $resn_exch{"3H78 BAY"}="H783 BAY";
$resn_exch{"1H79 BAY"}="H791 BAY"; $resn_exch{"2H79 BAY"}="H792 BAY"; $resn_exch{"3H79 BAY"}="H793 BAY";
$resn_exch{"1H80 BAY"}="H801 BAY"; $resn_exch{"2H80 BAY"}="H802 BAY"; $resn_exch{"3H80 BAY"}="H803 BAY";
$resn_exch{"1HN2 BAZ"}="HN21 BAZ"; $resn_exch{"2HN2 BAZ"}="HN22 BAZ"; $resn_exch{"1H2' BAZ"}="H2'1 BAZ";
$resn_exch{"2H2' BAZ"}="H2'2 BAZ"; $resn_exch{"'HN4 BAZ"}="HN4' BAZ"; $resn_exch{"1H9  BAZ"}=" H91 BAZ";
$resn_exch{"2H9  BAZ"}=" H92 BAZ"; $resn_exch{"1H2  BB1"}=" H21 BB1"; $resn_exch{"2H2  BB1"}=" H22 BB1";
$resn_exch{"1H7  BB1"}=" H71 BB1"; $resn_exch{"2H7  BB1"}=" H72 BB1"; $resn_exch{"1H8  BB1"}=" H81 BB1";
$resn_exch{"2H8  BB1"}=" H82 BB1"; $resn_exch{"1H9  BB1"}=" H91 BB1"; $resn_exch{"2H9  BB1"}=" H92 BB1";
$resn_exch{"1H10 BB1"}="H101 BB1"; $resn_exch{"2H10 BB1"}="H102 BB1"; $resn_exch{"3H10 BB1"}="H103 BB1";
$resn_exch{"1H16 BB1"}="H161 BB1"; $resn_exch{"2H16 BB1"}="H162 BB1"; $resn_exch{"3H16 BB1"}="H163 BB1";
$resn_exch{"1H15 BB1"}="H151 BB1"; $resn_exch{"2H15 BB1"}="H152 BB1"; $resn_exch{"3H15 BB1"}="H153 BB1";
$resn_exch{"1H12 BB1"}="H121 BB1"; $resn_exch{"2H12 BB1"}="H122 BB1"; $resn_exch{"3H12 BB1"}="H123 BB1";
$resn_exch{"1H13 BB1"}="H131 BB1"; $resn_exch{"2H13 BB1"}="H132 BB1"; $resn_exch{"3H13 BB1"}="H133 BB1";
$resn_exch{"1H14 BB1"}="H141 BB1"; $resn_exch{"2H14 BB1"}="H142 BB1"; $resn_exch{"3H14 BB1"}="H143 BB1";
$resn_exch{"1H5  BB2"}=" H51 BB2"; $resn_exch{"2H5  BB2"}=" H52 BB2"; $resn_exch{"1H7  BB2"}=" H71 BB2";
$resn_exch{"2H7  BB2"}=" H72 BB2"; $resn_exch{"1H8  BB2"}=" H81 BB2"; $resn_exch{"2H8  BB2"}=" H82 BB2";
$resn_exch{"1H9  BB2"}=" H91 BB2"; $resn_exch{"2H9  BB2"}=" H92 BB2"; $resn_exch{"1H10 BB2"}="H101 BB2";
$resn_exch{"2H10 BB2"}="H102 BB2"; $resn_exch{"1H11 BB2"}="H111 BB2"; $resn_exch{"2H11 BB2"}="H112 BB2";
$resn_exch{"3H11 BB2"}="H113 BB2"; $resn_exch{"1H18 BB2"}="H181 BB2"; $resn_exch{"2H18 BB2"}="H182 BB2";
$resn_exch{"3H18 BB2"}="H183 BB2"; $resn_exch{"1H17 BB2"}="H171 BB2"; $resn_exch{"2H17 BB2"}="H172 BB2";
$resn_exch{"3H17 BB2"}="H173 BB2"; $resn_exch{"1H23 BB2"}="H231 BB2"; $resn_exch{"2H23 BB2"}="H232 BB2";
$resn_exch{"1H24 BB2"}="H241 BB2"; $resn_exch{"2H24 BB2"}="H242 BB2"; $resn_exch{"1H25 BB2"}="H251 BB2";
$resn_exch{"2H25 BB2"}="H252 BB2"; $resn_exch{"1H26 BB2"}="H261 BB2"; $resn_exch{"2H26 BB2"}="H262 BB2";
$resn_exch{"1H17 BB3"}="H171 BB3"; $resn_exch{"2H17 BB3"}="H172 BB3"; $resn_exch{"1H18 BB3"}="H181 BB3";
$resn_exch{"2H18 BB3"}="H182 BB3"; $resn_exch{"3H18 BB3"}="H183 BB3"; $resn_exch{"1H01 BB3"}="H011 BB3";
$resn_exch{"2H01 BB3"}="H012 BB3"; $resn_exch{"3H01 BB3"}="H013 BB3"; $resn_exch{"1H02 BB3"}="H021 BB3";
$resn_exch{"2H02 BB3"}="H022 BB3"; $resn_exch{"3H02 BB3"}="H023 BB3"; $resn_exch{"1H8  BBA"}=" H81 BBA";
$resn_exch{"2H8  BBA"}=" H82 BBA"; $resn_exch{"1HG1 BBA"}="HG11 BBA"; $resn_exch{"2HG1 BBA"}="HG12 BBA";
$resn_exch{"1H18 BBA"}="H181 BBA"; $resn_exch{"2H18 BBA"}="H182 BBA"; $resn_exch{"1HG3 BBA"}="HG31 BBA";
$resn_exch{"2HG3 BBA"}="HG32 BBA"; $resn_exch{"1H2  BBA"}="H21A BBA"; $resn_exch{"2H2  BBA"}=" H22 BBA";
$resn_exch{"1H3  BBA"}=" H31 BBA"; $resn_exch{"2H3  BBA"}=" H32 BBA"; $resn_exch{"1H   BBA"}=" H1A BBA";
$resn_exch{"2H   BBA"}=" H2  BBA"; $resn_exch{"1H4  BBA"}=" H41 BBA"; $resn_exch{"2H4  BBA"}=" H42 BBA";
$resn_exch{"1HAN BBB"}="HAN1 BBB"; $resn_exch{"2HAN BBB"}="HAN2 BBB"; $resn_exch{"1HAO BBB"}="HAO1 BBB";
$resn_exch{"2HAO BBB"}="HAO2 BBB"; $resn_exch{"1HAR BBB"}="HAR1 BBB"; $resn_exch{"2HAR BBB"}="HAR2 BBB";
$resn_exch{"1HAQ BBB"}="HAQ1 BBB"; $resn_exch{"2HAQ BBB"}="HAQ2 BBB"; $resn_exch{"1HAP BBB"}="HAP1 BBB";
$resn_exch{"2HAP BBB"}="HAP2 BBB"; $resn_exch{"1HB  BBC"}=" HB1 BBC"; $resn_exch{"2HB  BBC"}=" HB2 BBC";
$resn_exch{"1H05 BBC"}="H051 BBC"; $resn_exch{"2H05 BBC"}="H052 BBC"; $resn_exch{"1H   BBC"}=" H1  BBC";
$resn_exch{"2H   BBC"}=" H2  BBC"; $resn_exch{"1H08 BBC"}="H081 BBC"; $resn_exch{"2H08 BBC"}="H082 BBC";
$resn_exch{"1H09 BBC"}="H091 BBC"; $resn_exch{"2H09 BBC"}="H092 BBC"; $resn_exch{"1H10 BBC"}="H101 BBC";
$resn_exch{"2H10 BBC"}="H102 BBC"; $resn_exch{"1H11 BBC"}="H111 BBC"; $resn_exch{"2H11 BBC"}="H112 BBC";
$resn_exch{"1HC2 BBH"}="HC21 BBH"; $resn_exch{"1HC3 BBH"}="HC31 BBH"; $resn_exch{"1HC5 BBH"}="HC51 BBH";
$resn_exch{"1HC6 BBH"}="HC61 BBH"; $resn_exch{"1HC8 BBH"}="HC81 BBH"; $resn_exch{"2HC8 BBH"}="HC82 BBH";
$resn_exch{"3HC8 BBH"}="HC83 BBH"; $resn_exch{"1H13 BBH"}="H131 BBH"; $resn_exch{"1H14 BBH"}="H141 BBH";
$resn_exch{"2H14 BBH"}="H142 BBH"; $resn_exch{"1H17 BBH"}="H171 BBH"; $resn_exch{"2H17 BBH"}="H172 BBH";
$resn_exch{"1HC7 BBH"}="HC71 BBH"; $resn_exch{"2HC7 BBH"}="HC72 BBH"; $resn_exch{"1HC9 BBH"}="HC91 BBH";
$resn_exch{"1H10 BBH"}="H101 BBH"; $resn_exch{"1H12 BBH"}="H121 BBH"; $resn_exch{"1H16 BBH"}="H161 BBH";
$resn_exch{"1H18 BBH"}="H181 BBH"; $resn_exch{"1HBZ BBL"}="HBZ1 BBL"; $resn_exch{"2HBZ BBL"}="HBZ2 BBL";
$resn_exch{"3HBZ BBL"}="HBZ3 BBL"; $resn_exch{"1HB  BBL"}=" HB1 BBL"; $resn_exch{"2HB  BBL"}=" HB2 BBL";
$resn_exch{"1H15 BBM"}="H151 BBM"; $resn_exch{"2H15 BBM"}="H152 BBM"; $resn_exch{"1H17 BBM"}="H171 BBM";
$resn_exch{"2H17 BBM"}="H172 BBM"; $resn_exch{"1H1C BBR"}="H1C1 BBR"; $resn_exch{"2H1C BBR"}="H1C2 BBR";
$resn_exch{"1H3C BBR"}="H3C1 BBR"; $resn_exch{"2H3C BBR"}="H3C2 BBR"; $resn_exch{"1H1  BBS"}=" H11 BBS";
$resn_exch{"2H1  BBS"}=" H12 BBS"; $resn_exch{"3H1  BBS"}=" H13 BBS"; $resn_exch{"1H3  BBS"}=" H31 BBS";
$resn_exch{"2H3  BBS"}=" H32 BBS"; $resn_exch{"3H3  BBS"}=" H33 BBS"; $resn_exch{"1H4  BBS"}=" H41 BBS";
$resn_exch{"2H4  BBS"}=" H42 BBS"; $resn_exch{"3H4  BBS"}=" H43 BBS"; $resn_exch{"1HP2 BBT"}="HP21 BBT";
$resn_exch{"2HP2 BBT"}="HP22 BBT"; $resn_exch{"1HP3 BBT"}="HP31 BBT"; $resn_exch{"2HP3 BBT"}="HP32 BBT";
$resn_exch{"1HP5 BBT"}="HP51 BBT"; $resn_exch{"2HP5 BBT"}="HP52 BBT"; $resn_exch{"1HP6 BBT"}="HP61 BBT";
$resn_exch{"2HP6 BBT"}="HP62 BBT"; $resn_exch{"1HP7 BBT"}="HP71 BBT"; $resn_exch{"2HP7 BBT"}="HP72 BBT";
$resn_exch{"1HP8 BBT"}="HP81 BBT"; $resn_exch{"2HP8 BBT"}="HP82 BBT"; $resn_exch{"1H1  BBU"}=" H11 BBU";
$resn_exch{"2H1  BBU"}=" H12 BBU"; $resn_exch{"1H2  BBU"}=" H21 BBU"; $resn_exch{"2H2  BBU"}=" H22 BBU";
$resn_exch{"1H3  BBU"}=" H31 BBU"; $resn_exch{"2H3  BBU"}=" H32 BBU"; $resn_exch{"1H4  BBU"}=" H41 BBU";
$resn_exch{"2H4  BBU"}=" H42 BBU"; $resn_exch{"3H4  BBU"}=" H43 BBU"; $resn_exch{"1H1  BBX"}=" H11 BBX";
$resn_exch{"2H1  BBX"}=" H12 BBX"; $resn_exch{"1H2  BBX"}=" H21 BBX"; $resn_exch{"2H2  BBX"}=" H22 BBX";
$resn_exch{"3H2  BBX"}=" H23 BBX"; $resn_exch{"1H20 BBZ"}="H201 BBZ"; $resn_exch{"2H20 BBZ"}="H202 BBZ";
$resn_exch{"1H21 BBZ"}="H211 BBZ"; $resn_exch{"2H21 BBZ"}="H212 BBZ"; $resn_exch{"1H22 BBZ"}="H221 BBZ";
$resn_exch{"2H22 BBZ"}="H222 BBZ"; $resn_exch{"1H23 BBZ"}="H231 BBZ"; $resn_exch{"2H23 BBZ"}="H232 BBZ";
$resn_exch{"1H24 BBZ"}="H241 BBZ"; $resn_exch{"2H24 BBZ"}="H242 BBZ"; $resn_exch{"3H24 BBZ"}="H243 BBZ";
$resn_exch{"1HN1 BBZ"}="HN11 BBZ"; $resn_exch{"2HN1 BBZ"}="HN12 BBZ"; $resn_exch{"3HN1 BBZ"}="HN13 BBZ";
$resn_exch{"1HN2 BBZ"}="HN21 BBZ"; $resn_exch{"2HN2 BBZ"}="HN22 BBZ"; $resn_exch{"3HN2 BBZ"}="HN23 BBZ";
$resn_exch{"1H14  BC"}="H141  BC"; $resn_exch{"2H14  BC"}="H142  BC"; $resn_exch{"1H4C BC2"}="H4C1 BC2";
$resn_exch{"1H6C BC2"}="H6C1 BC2"; $resn_exch{"1H36 BC2"}="H361 BC2"; $resn_exch{"2H36 BC2"}="H362 BC2";
$resn_exch{"3H36 BC2"}="H363 BC2"; $resn_exch{"1H35 BC2"}="H351 BC2"; $resn_exch{"2H35 BC2"}="H352 BC2";
$resn_exch{"3H35 BC2"}="H353 BC2"; $resn_exch{"1H39 BC2"}="H391 BC2"; $resn_exch{"2H39 BC2"}="H392 BC2";
$resn_exch{"3H39 BC2"}="H393 BC2"; $resn_exch{"1H25 BC2"}="H251 BC2"; $resn_exch{"2H25 BC2"}="H252 BC2";
$resn_exch{"3H25 BC2"}="H253 BC2"; $resn_exch{"1H9C BC2"}="H9C1 BC2"; $resn_exch{"2H9C BC2"}="H9C2 BC2";
$resn_exch{"1H27 BC2"}="H271 BC2"; $resn_exch{"2H27 BC2"}="H272 BC2"; $resn_exch{"3H27 BC2"}="H273 BC2";
$resn_exch{"1H40 BC2"}="H401 BC2"; $resn_exch{"2H40 BC2"}="H402 BC2"; $resn_exch{"3H40 BC2"}="H403 BC2";
$resn_exch{"1H29 BC2"}="H291 BC2"; $resn_exch{"2H29 BC2"}="H292 BC2"; $resn_exch{"3H29 BC2"}="H293 BC2";
$resn_exch{"1H30 BC2"}="H301 BC2"; $resn_exch{"2H30 BC2"}="H302 BC2"; $resn_exch{"3H30 BC2"}="H303 BC2";
$resn_exch{"1H34 BC2"}="H341 BC2"; $resn_exch{"2H34 BC2"}="H342 BC2"; $resn_exch{" AH2 BCA"}=" H2A BCA";
$resn_exch{"AH61 BCA"}="H61A BCA"; $resn_exch{"AH62 BCA"}="H62A BCA"; $resn_exch{" AH8 BCA"}=" H8A BCA";
$resn_exch{"AH1* BCA"}=" H1D BCA"; $resn_exch{"AH2* BCA"}=" H2D BCA"; $resn_exch{"AHO2 BCA"}="HO2A BCA";
$resn_exch{"AH3* BCA"}=" H3D BCA"; $resn_exch{"8HOA BCA"}="HOA8 BCA"; $resn_exch{"9HOA BCA"}="HOA9 BCA";
$resn_exch{"AH4* BCA"}=" H4D BCA"; $resn_exch{"AH51 BCA"}="H51A BCA"; $resn_exch{"AH52 BCA"}="H52A BCA";
$resn_exch{"2HOA BCA"}="HOA2 BCA"; $resn_exch{"5HOA BCA"}="HOA5 BCA"; $resn_exch{"1H12 BCA"}="H121 BCA";
$resn_exch{"2H12 BCA"}="H122 BCA"; $resn_exch{"1H13 BCA"}="H131 BCA"; $resn_exch{"2H13 BCA"}="H132 BCA";
$resn_exch{"3H13 BCA"}="H133 BCA"; $resn_exch{"1H14 BCA"}="H141 BCA"; $resn_exch{"2H14 BCA"}="H142 BCA";
$resn_exch{"3H14 BCA"}="H143 BCA"; $resn_exch{"1H7  BCA"}=" H71 BCA"; $resn_exch{"2H7  BCA"}=" H72 BCA";
$resn_exch{"1H6  BCA"}=" H61 BCA"; $resn_exch{"2H6  BCA"}=" H62 BCA"; $resn_exch{"1H3  BCA"}=" H31 BCA";
$resn_exch{"2H3  BCA"}=" H32 BCA"; $resn_exch{"1H2  BCA"}=" H21 BCA"; $resn_exch{"2H2  BCA"}=" H22 BCA";
$resn_exch{" BH3 BCA"}=" H3B BCA"; $resn_exch{" BH4 BCA"}=" H4B BCA"; $resn_exch{"BHO2 BCA"}="HO2B BCA";
$resn_exch{" BH6 BCA"}=" H6B BCA"; $resn_exch{" BH7 BCA"}=" H7B BCA"; $resn_exch{"1HMA BCB"}="HMA1 BCB";
$resn_exch{"2HMA BCB"}="HMA2 BCB"; $resn_exch{"3HMA BCB"}="HMA3 BCB"; $resn_exch{"1HAA BCB"}="HAA1 BCB";
$resn_exch{"2HAA BCB"}="HAA2 BCB"; $resn_exch{"1HBA BCB"}="HBA1 BCB"; $resn_exch{"2HBA BCB"}="HBA2 BCB";
$resn_exch{"1HMB BCB"}="HMB1 BCB"; $resn_exch{"2HMB BCB"}="HMB2 BCB"; $resn_exch{"3HMB BCB"}="HMB3 BCB";
$resn_exch{"1HBB BCB"}="HBB1 BCB"; $resn_exch{"2HBB BCB"}="HBB2 BCB"; $resn_exch{"3HBB BCB"}="HBB3 BCB";
$resn_exch{"1HMC BCB"}="HMC1 BCB"; $resn_exch{"2HMC BCB"}="HMC2 BCB"; $resn_exch{"3HMC BCB"}="HMC3 BCB";
$resn_exch{"1HAC BCB"}="HAC1 BCB"; $resn_exch{"1HBC BCB"}="HBC1 BCB"; $resn_exch{"2HBC BCB"}="HBC2 BCB";
$resn_exch{"3HBC BCB"}="HBC3 BCB"; $resn_exch{"1HMD BCB"}="HMD1 BCB"; $resn_exch{"2HMD BCB"}="HMD2 BCB";
$resn_exch{"3HMD BCB"}="HMD3 BCB"; $resn_exch{"1HED BCB"}="HED1 BCB"; $resn_exch{"2HED BCB"}="HED2 BCB";
$resn_exch{"3HED BCB"}="HED3 BCB"; $resn_exch{"1H1  BCB"}=" H11 BCB"; $resn_exch{"2H1  BCB"}=" H12 BCB";
$resn_exch{"1H4  BCB"}=" H41 BCB"; $resn_exch{"2H4  BCB"}=" H42 BCB"; $resn_exch{"3H4  BCB"}=" H43 BCB";
$resn_exch{"1H5  BCB"}=" H51 BCB"; $resn_exch{"2H5  BCB"}=" H52 BCB"; $resn_exch{"1H6  BCB"}=" H61 BCB";
$resn_exch{"2H6  BCB"}=" H62 BCB"; $resn_exch{"1H7  BCB"}=" H71 BCB"; $resn_exch{"2H7  BCB"}=" H72 BCB";
$resn_exch{"1H9  BCB"}=" H91 BCB"; $resn_exch{"2H9  BCB"}=" H92 BCB"; $resn_exch{"3H9  BCB"}=" H93 BCB";
$resn_exch{"1H10 BCB"}="H101 BCB"; $resn_exch{"2H10 BCB"}="H102 BCB"; $resn_exch{"1H11 BCB"}="H111 BCB";
$resn_exch{"2H11 BCB"}="H112 BCB"; $resn_exch{"1H12 BCB"}="H121 BCB"; $resn_exch{"2H12 BCB"}="H122 BCB";
$resn_exch{"1H14 BCB"}="H141 BCB"; $resn_exch{"2H14 BCB"}="H142 BCB"; $resn_exch{"3H14 BCB"}="H143 BCB";
$resn_exch{"1H15 BCB"}="H151 BCB"; $resn_exch{"2H15 BCB"}="H152 BCB"; $resn_exch{"1H16 BCB"}="H161 BCB";
$resn_exch{"2H16 BCB"}="H162 BCB"; $resn_exch{"1H17 BCB"}="H171 BCB"; $resn_exch{"2H17 BCB"}="H172 BCB";
$resn_exch{"1H19 BCB"}="H191 BCB"; $resn_exch{"2H19 BCB"}="H192 BCB"; $resn_exch{"3H19 BCB"}="H193 BCB";
$resn_exch{"1H20 BCB"}="H201 BCB"; $resn_exch{"2H20 BCB"}="H202 BCB"; $resn_exch{"3H20 BCB"}="H203 BCB";
$resn_exch{"1H3  BCC"}=" H31 BCC"; $resn_exch{"2H3  BCC"}=" H32 BCC"; $resn_exch{"1H4  BCC"}=" H41 BCC";
$resn_exch{"2H4  BCC"}=" H42 BCC"; $resn_exch{"1H6  BCC"}=" H61 BCC"; $resn_exch{"2H6  BCC"}=" H62 BCC";
$resn_exch{"1H7  BCC"}=" H71 BCC"; $resn_exch{"2H7  BCC"}=" H72 BCC"; $resn_exch{"1HN2 BCC"}="HN21 BCC";
$resn_exch{"2HN2 BCC"}="HN22 BCC"; $resn_exch{"1H10 BCC"}="H101 BCC"; $resn_exch{"2H10 BCC"}="H102 BCC";
$resn_exch{"1HO2 BCD"}="HO21 BCD"; $resn_exch{"1HO3 BCD"}="HO31 BCD"; $resn_exch{"1H61 BCD"}="H611 BCD";
$resn_exch{"2H61 BCD"}="H612 BCD"; $resn_exch{"1HO6 BCD"}="HO61 BCD"; $resn_exch{"2HO2 BCD"}="HO22 BCD";
$resn_exch{"2HO3 BCD"}="HO32 BCD"; $resn_exch{"1H62 BCD"}="H621 BCD"; $resn_exch{"2H62 BCD"}="H622 BCD";
$resn_exch{"2HO6 BCD"}="HO62 BCD"; $resn_exch{"3HO2 BCD"}="HO23 BCD"; $resn_exch{"3HO3 BCD"}="HO33 BCD";
$resn_exch{"1H63 BCD"}="H631 BCD"; $resn_exch{"2H63 BCD"}="H632 BCD"; $resn_exch{"3HO6 BCD"}="HO63 BCD";
$resn_exch{"4HO2 BCD"}="HO24 BCD"; $resn_exch{"4HO3 BCD"}="HO34 BCD"; $resn_exch{"1H64 BCD"}="H641 BCD";
$resn_exch{"2H64 BCD"}="H642 BCD"; $resn_exch{"4HO6 BCD"}="HO64 BCD"; $resn_exch{"5HO2 BCD"}="HO25 BCD";
$resn_exch{"5HO3 BCD"}="HO35 BCD"; $resn_exch{"1H65 BCD"}="H651 BCD"; $resn_exch{"2H65 BCD"}="H652 BCD";
$resn_exch{"5HO6 BCD"}="HO65 BCD"; $resn_exch{"6HO2 BCD"}="HO26 BCD"; $resn_exch{"6HO3 BCD"}="HO36 BCD";
$resn_exch{"1H66 BCD"}="H661 BCD"; $resn_exch{"2H66 BCD"}="H662 BCD"; $resn_exch{"6HO6 BCD"}="HO66 BCD";
$resn_exch{"7HO2 BCD"}="HO27 BCD"; $resn_exch{"7HO3 BCD"}="HO37 BCD"; $resn_exch{"1H67 BCD"}="H671 BCD";
$resn_exch{"2H67 BCD"}="H672 BCD"; $resn_exch{"7HO6 BCD"}="HO67 BCD"; $resn_exch{"1H4  BCG"}=" H41 BCG";
$resn_exch{"2H4  BCG"}=" H42 BCG"; $resn_exch{"1H6  BCG"}=" H61 BCG"; $resn_exch{"2H6  BCG"}=" H62 BCG";
$resn_exch{"1H7  BCG"}=" H71 BCG"; $resn_exch{"2H7  BCG"}=" H72 BCG"; $resn_exch{"1H16 BCG"}="H161 BCG";
$resn_exch{"2H16 BCG"}="H162 BCG"; $resn_exch{"3H16 BCG"}="H163 BCG"; $resn_exch{"1H21 BCH"}="H211 BCH";
$resn_exch{"2H21 BCH"}="H212 BCH"; $resn_exch{"3H21 BCH"}="H213 BCH"; $resn_exch{"1H20 BCH"}="H201 BCH";
$resn_exch{"2H20 BCH"}="H202 BCH"; $resn_exch{"1H19 BCH"}="H191 BCH"; $resn_exch{"2H19 BCH"}="H192 BCH";
$resn_exch{"1H16 BCH"}="H161 BCH"; $resn_exch{"2H16 BCH"}="H162 BCH"; $resn_exch{"1H15 BCH"}="H151 BCH";
$resn_exch{"2H15 BCH"}="H152 BCH"; $resn_exch{"1H13 BCH"}="H131 BCH"; $resn_exch{"2H13 BCH"}="H132 BCH";
$resn_exch{"3H13 BCH"}="H133 BCH"; $resn_exch{"1H22 BCH"}="H221 BCH"; $resn_exch{"2H22 BCH"}="H222 BCH";
$resn_exch{"3H22 BCH"}="H223 BCH"; $resn_exch{"1H11 BCH"}="H111 BCH"; $resn_exch{"2H11 BCH"}="H112 BCH";
$resn_exch{"3H11 BCH"}="H113 BCH"; $resn_exch{"1HMA BCL"}="HMA1 BCL"; $resn_exch{"2HMA BCL"}="HMA2 BCL";
$resn_exch{"3HMA BCL"}="HMA3 BCL"; $resn_exch{"1HAA BCL"}="HAA1 BCL"; $resn_exch{"2HAA BCL"}="HAA2 BCL";
$resn_exch{"1HBA BCL"}="HBA1 BCL"; $resn_exch{"2HBA BCL"}="HBA2 BCL"; $resn_exch{"1HMB BCL"}="HMB1 BCL";
$resn_exch{"2HMB BCL"}="HMB2 BCL"; $resn_exch{"3HMB BCL"}="HMB3 BCL"; $resn_exch{"1HBB BCL"}="HBB1 BCL";
$resn_exch{"2HBB BCL"}="HBB2 BCL"; $resn_exch{"3HBB BCL"}="HBB3 BCL"; $resn_exch{"1HMC BCL"}="HMC1 BCL";
$resn_exch{"2HMC BCL"}="HMC2 BCL"; $resn_exch{"3HMC BCL"}="HMC3 BCL"; $resn_exch{"1HAC BCL"}="HAC1 BCL";
$resn_exch{"2HAC BCL"}="HAC2 BCL"; $resn_exch{"1HBC BCL"}="HBC1 BCL"; $resn_exch{"2HBC BCL"}="HBC2 BCL";
$resn_exch{"3HBC BCL"}="HBC3 BCL"; $resn_exch{"1HMD BCL"}="HMD1 BCL"; $resn_exch{"2HMD BCL"}="HMD2 BCL";
$resn_exch{"3HMD BCL"}="HMD3 BCL"; $resn_exch{"1HED BCL"}="HED1 BCL"; $resn_exch{"2HED BCL"}="HED2 BCL";
$resn_exch{"3HED BCL"}="HED3 BCL"; $resn_exch{"1H1  BCL"}=" H11 BCL"; $resn_exch{"2H1  BCL"}=" H12 BCL";
$resn_exch{"1H4  BCL"}=" H41 BCL"; $resn_exch{"2H4  BCL"}=" H42 BCL"; $resn_exch{"3H4  BCL"}=" H43 BCL";
$resn_exch{"1H5  BCL"}=" H51 BCL"; $resn_exch{"2H5  BCL"}=" H52 BCL"; $resn_exch{"1H6  BCL"}=" H61 BCL";
$resn_exch{"2H6  BCL"}=" H62 BCL"; $resn_exch{"1H7  BCL"}=" H71 BCL"; $resn_exch{"2H7  BCL"}=" H72 BCL";
$resn_exch{"1H9  BCL"}=" H91 BCL"; $resn_exch{"2H9  BCL"}=" H92 BCL"; $resn_exch{"3H9  BCL"}=" H93 BCL";
$resn_exch{"1H10 BCL"}="H101 BCL"; $resn_exch{"2H10 BCL"}="H102 BCL"; $resn_exch{"1H11 BCL"}="H111 BCL";
$resn_exch{"2H11 BCL"}="H112 BCL"; $resn_exch{"1H12 BCL"}="H121 BCL"; $resn_exch{"2H12 BCL"}="H122 BCL";
$resn_exch{"1H14 BCL"}="H141 BCL"; $resn_exch{"2H14 BCL"}="H142 BCL"; $resn_exch{"3H14 BCL"}="H143 BCL";
$resn_exch{"1H15 BCL"}="H151 BCL"; $resn_exch{"2H15 BCL"}="H152 BCL"; $resn_exch{"1H16 BCL"}="H161 BCL";
$resn_exch{"2H16 BCL"}="H162 BCL"; $resn_exch{"1H17 BCL"}="H171 BCL"; $resn_exch{"2H17 BCL"}="H172 BCL";
$resn_exch{"1H19 BCL"}="H191 BCL"; $resn_exch{"2H19 BCL"}="H192 BCL"; $resn_exch{"3H19 BCL"}="H193 BCL";
$resn_exch{"1H20 BCL"}="H201 BCL"; $resn_exch{"2H20 BCL"}="H202 BCL"; $resn_exch{"3H20 BCL"}="H203 BCL";
$resn_exch{"1H5A BCM"}="H5A1 BCM"; $resn_exch{"2H5A BCM"}="H5A2 BCM"; $resn_exch{"1H4  BCM"}=" H41 BCM";
$resn_exch{"2H4  BCM"}=" H42 BCM"; $resn_exch{"1H3  BCM"}=" H31 BCM"; $resn_exch{"2H3  BCM"}=" H32 BCM";
$resn_exch{"1HB  BCM"}=" HB1 BCM"; $resn_exch{"2HB  BCM"}=" HB2 BCM"; $resn_exch{"3HB  BCM"}=" HB3 BCM";
$resn_exch{"1HA  BCM"}=" HA1 BCM"; $resn_exch{"2HA  BCM"}=" HA2 BCM"; $resn_exch{"1H1  BCN"}=" H11 BCN";
$resn_exch{"2H1  BCN"}=" H12 BCN"; $resn_exch{"1H3  BCN"}=" H31 BCN"; $resn_exch{"2H3  BCN"}=" H32 BCN";
$resn_exch{"1H4  BCN"}=" H41 BCN"; $resn_exch{"2H4  BCN"}=" H42 BCN"; $resn_exch{"1H5  BCN"}=" H51 BCN";
$resn_exch{"2H5  BCN"}=" H52 BCN"; $resn_exch{"1H6  BCN"}=" H61 BCN"; $resn_exch{"2H6  BCN"}=" H62 BCN";
$resn_exch{"1H2  BCQ"}=" H21 BCQ"; $resn_exch{"2H2  BCQ"}=" H22 BCQ"; $resn_exch{"1H3  BCQ"}=" H31 BCQ";
$resn_exch{"2H3  BCQ"}=" H32 BCQ"; $resn_exch{"1H4  BCQ"}=" H41 BCQ"; $resn_exch{"2H4  BCQ"}=" H42 BCQ";
$resn_exch{"1H5  BCQ"}=" H51 BCQ"; $resn_exch{"2H5  BCQ"}=" H52 BCQ"; $resn_exch{"1H03 BCQ"}="H031 BCQ";
$resn_exch{"2H03 BCQ"}="H032 BCQ"; $resn_exch{"1H09 BCQ"}="H091 BCQ"; $resn_exch{"2H09 BCQ"}="H092 BCQ";
$resn_exch{"1H18 BCQ"}="H181 BCQ"; $resn_exch{"2H18 BCQ"}="H182 BCQ"; $resn_exch{"3H18 BCQ"}="H183 BCQ";
$resn_exch{"1H17 BCQ"}="H171 BCQ"; $resn_exch{"2H17 BCQ"}="H172 BCQ"; $resn_exch{"3H17 BCQ"}="H173 BCQ";
$resn_exch{"1HB2 BCQ"}="HB21 BCQ"; $resn_exch{"2HB2 BCQ"}="HB22 BCQ"; $resn_exch{"1HB8 BCQ"}="HB81 BCQ";
$resn_exch{"2HB8 BCQ"}="HB82 BCQ"; $resn_exch{"1H7A BCQ"}="H7A1 BCQ"; $resn_exch{"2H7A BCQ"}="H7A2 BCQ";
$resn_exch{"1HT  BCQ"}=" HT1 BCQ"; $resn_exch{"2HT  BCQ"}=" HT2 BCQ"; $resn_exch{"1HC2 BCR"}="HC21 BCR";
$resn_exch{"2HC2 BCR"}="HC22 BCR"; $resn_exch{"1HC3 BCR"}="HC31 BCR"; $resn_exch{"2HC3 BCR"}="HC32 BCR";
$resn_exch{"1HC4 BCR"}="HC41 BCR"; $resn_exch{"2HC4 BCR"}="HC42 BCR"; $resn_exch{"CH10 BCR"}="H10C BCR";
$resn_exch{"CH11 BCR"}="H11C BCR"; $resn_exch{"1H33 BCR"}="H331 BCR"; $resn_exch{"2H33 BCR"}="H332 BCR";
$resn_exch{"3H33 BCR"}="H333 BCR"; $resn_exch{"1H31 BCR"}="H311 BCR"; $resn_exch{"2H31 BCR"}="H312 BCR";
$resn_exch{"3H31 BCR"}="H313 BCR"; $resn_exch{"1H32 BCR"}="H321 BCR"; $resn_exch{"2H32 BCR"}="H322 BCR";
$resn_exch{"3H32 BCR"}="H323 BCR"; $resn_exch{"1H34 BCR"}="H341 BCR"; $resn_exch{"2H34 BCR"}="H342 BCR";
$resn_exch{"3H34 BCR"}="H343 BCR"; $resn_exch{"CH12 BCR"}="H12C BCR"; $resn_exch{"CH14 BCR"}="H14C BCR";
$resn_exch{"CH15 BCR"}="H15C BCR"; $resn_exch{"CH16 BCR"}="H16C BCR"; $resn_exch{"CH17 BCR"}="H17C BCR";
$resn_exch{"CH19 BCR"}="H19C BCR"; $resn_exch{"CH20 BCR"}="H20C BCR"; $resn_exch{"CH21 BCR"}="H21C BCR";
$resn_exch{"CH23 BCR"}="H23C BCR"; $resn_exch{"CH24 BCR"}="H24C BCR"; $resn_exch{"1H27 BCR"}="H271 BCR";
$resn_exch{"2H27 BCR"}="H272 BCR"; $resn_exch{"1H28 BCR"}="H281 BCR"; $resn_exch{"2H28 BCR"}="H282 BCR";
$resn_exch{"1H29 BCR"}="H291 BCR"; $resn_exch{"2H29 BCR"}="H292 BCR"; $resn_exch{"1H35 BCR"}="H351 BCR";
$resn_exch{"2H35 BCR"}="H352 BCR"; $resn_exch{"3H35 BCR"}="H353 BCR"; $resn_exch{"1H36 BCR"}="H361 BCR";
$resn_exch{"2H36 BCR"}="H362 BCR"; $resn_exch{"3H36 BCR"}="H363 BCR"; $resn_exch{"1H37 BCR"}="H371 BCR";
$resn_exch{"2H37 BCR"}="H372 BCR"; $resn_exch{"3H37 BCR"}="H373 BCR"; $resn_exch{"1H38 BCR"}="H381 BCR";
$resn_exch{"2H38 BCR"}="H382 BCR"; $resn_exch{"3H38 BCR"}="H383 BCR"; $resn_exch{"1H39 BCR"}="H391 BCR";
$resn_exch{"2H39 BCR"}="H392 BCR"; $resn_exch{"3H39 BCR"}="H393 BCR"; $resn_exch{"1H40 BCR"}="H401 BCR";
$resn_exch{"2H40 BCR"}="H402 BCR"; $resn_exch{"3H40 BCR"}="H403 BCR"; $resn_exch{"1HN  BCS"}=" H   BCS";
$resn_exch{"2HN  BCS"}=" H2  BCS"; $resn_exch{"1HB  BCS"}=" HB2 BCS"; $resn_exch{"2HB  BCS"}=" HB3 BCS";
$resn_exch{"1HD  BCS"}=" HD2 BCS"; $resn_exch{"2HD  BCS"}=" HD3 BCS"; $resn_exch{"1H14 BCV"}="H141 BCV";
$resn_exch{"2H14 BCV"}="H142 BCV"; $resn_exch{"1H3C BCV"}="H3C1 BCV"; $resn_exch{"2H3C BCV"}="H3C2 BCV";
$resn_exch{"1H4C BCV"}="H4C1 BCV"; $resn_exch{"2H4C BCV"}="H4C2 BCV"; $resn_exch{"1H7C BCV"}="H7C1 BCV";
$resn_exch{"2H7C BCV"}="H7C2 BCV"; $resn_exch{"1H16 BCV"}="H161 BCV"; $resn_exch{"2H16 BCV"}="H162 BCV";
$resn_exch{"1H37 BCV"}="H371 BCV"; $resn_exch{"2H37 BCV"}="H372 BCV"; $resn_exch{"1H33 BCV"}="H331 BCV";
$resn_exch{"2H33 BCV"}="H332 BCV"; $resn_exch{" HN2 BCX"}=" H2  BCX"; $resn_exch{"1HC  BCX"}=" HC1 BCX";
$resn_exch{"2HC  BCX"}=" HC2 BCX"; $resn_exch{"1HB  BCX"}=" HB2 BCX"; $resn_exch{"2HB  BCX"}=" HB3 BCX";
$resn_exch{"1HN  BCY"}=" HN1 BCY"; $resn_exch{"2HN  BCY"}=" HN2 BCY"; $resn_exch{"1HB  BCY"}=" HB1 BCY";
$resn_exch{"2HB  BCY"}=" HB2 BCY"; $resn_exch{"3HB  BCY"}=" HB3 BCY"; $resn_exch{"1H3  BCY"}=" H31 BCY";
$resn_exch{"2H3  BCY"}=" H32 BCY"; $resn_exch{"1H4  BCY"}=" H41 BCY"; $resn_exch{"2H4  BCY"}=" H42 BCY";
$resn_exch{"1H6  BCY"}=" H61 BCY"; $resn_exch{"2H6  BCY"}=" H62 BCY"; $resn_exch{"1H7  BCY"}=" H71 BCY";
$resn_exch{"2H7  BCY"}=" H72 BCY"; $resn_exch{"1H2  BCZ"}=" H21 BCZ"; $resn_exch{"1H25 BCZ"}="H251 BCZ";
$resn_exch{"1H30 BCZ"}="H301 BCZ"; $resn_exch{"1H27 BCZ"}="H271 BCZ"; $resn_exch{"2H27 BCZ"}="H272 BCZ";
$resn_exch{"1H3  BCZ"}=" H31 BCZ"; $resn_exch{"1H10 BCZ"}="H101 BCZ"; $resn_exch{"1H24 BCZ"}="H241 BCZ";
$resn_exch{"1H37 BCZ"}="H371 BCZ"; $resn_exch{"2H37 BCZ"}="H372 BCZ"; $resn_exch{"1H38 BCZ"}="H381 BCZ";
$resn_exch{"2H38 BCZ"}="H382 BCZ"; $resn_exch{"3H38 BCZ"}="H383 BCZ"; $resn_exch{"1H36 BCZ"}="H361 BCZ";
$resn_exch{"2H36 BCZ"}="H362 BCZ"; $resn_exch{"1H39 BCZ"}="H391 BCZ"; $resn_exch{"2H39 BCZ"}="H392 BCZ";
$resn_exch{"3H39 BCZ"}="H393 BCZ"; $resn_exch{"1H11 BCZ"}="H111 BCZ"; $resn_exch{"1H15 BCZ"}="H151 BCZ";
$resn_exch{"2H15 BCZ"}="H152 BCZ"; $resn_exch{"3H15 BCZ"}="H153 BCZ"; $resn_exch{"1H4  BCZ"}=" H41 BCZ";
$resn_exch{"1HO9 BCZ"}="HO91 BCZ"; $resn_exch{"1H5  BCZ"}=" H51 BCZ"; $resn_exch{"1HO7 BCZ"}="HO71 BCZ";
$resn_exch{"1H1  BCZ"}=" H11 BCZ"; $resn_exch{"2H1  BCZ"}=" H12 BCZ"; $resn_exch{"1H2C BD1"}="H2C1 BD1";
$resn_exch{"2H2C BD1"}="H2C2 BD1"; $resn_exch{"1H5C BD1"}="H5C1 BD1"; $resn_exch{"2H5C BD1"}="H5C2 BD1";
$resn_exch{"1H10 BD2"}="H101 BD2"; $resn_exch{"2H10 BD2"}="H102 BD2"; $resn_exch{"1H11 BD2"}="H111 BD2";
$resn_exch{"2H11 BD2"}="H112 BD2"; $resn_exch{"1H12 BD2"}="H121 BD2"; $resn_exch{"2H12 BD2"}="H122 BD2";
$resn_exch{"1H14 BD2"}="H141 BD2"; $resn_exch{"2H14 BD2"}="H142 BD2"; $resn_exch{"1H15 BD4"}="H151 BD4";
$resn_exch{"2H15 BD4"}="H152 BD4"; $resn_exch{"3H15 BD4"}="H153 BD4"; $resn_exch{"1H14 BD4"}="H141 BD4";
$resn_exch{"2H14 BD4"}="H142 BD4"; $resn_exch{"3H14 BD4"}="H143 BD4"; $resn_exch{"1H16 BD4"}="H161 BD4";
$resn_exch{"2H16 BD4"}="H162 BD4"; $resn_exch{"1H17 BD4"}="H171 BD4"; $resn_exch{"2H17 BD4"}="H172 BD4";
$resn_exch{"1H18 BD4"}="H181 BD4"; $resn_exch{"2H18 BD4"}="H182 BD4"; $resn_exch{"1H19 BD4"}="H191 BD4";
$resn_exch{"2H19 BD4"}="H192 BD4"; $resn_exch{"1H11 BDA"}="H111 BDA"; $resn_exch{"2H11 BDA"}="H112 BDA";
$resn_exch{"1H14 BDA"}="H141 BDA"; $resn_exch{"2H14 BDA"}="H142 BDA"; $resn_exch{"3H14 BDA"}="H143 BDA";
$resn_exch{"1H15 BDA"}="H151 BDA"; $resn_exch{"2H15 BDA"}="H152 BDA"; $resn_exch{"1H21 BDA"}="H211 BDA";
$resn_exch{"2H21 BDA"}="H212 BDA"; $resn_exch{"3H21 BDA"}="H213 BDA"; $resn_exch{" H1* BDA"}=" H1B BDA";
$resn_exch{"1H2* BDA"}="H2'1 BDA"; $resn_exch{"2H2* BDA"}="H2'2 BDA"; $resn_exch{" H3* BDA"}=" H3B BDA";
$resn_exch{"1HN* BDA"}="HN'1 BDA"; $resn_exch{"2HN* BDA"}="HN'2 BDA"; $resn_exch{" H4* BDA"}=" H4' BDA";
$resn_exch{"*HO4 BDA"}="HO4' BDA"; $resn_exch{" H5* BDA"}=" H5' BDA"; $resn_exch{"1H6* BDA"}="H6'1 BDA";
$resn_exch{"2H6* BDA"}="H6'2 BDA"; $resn_exch{"3H6* BDA"}="H6'3 BDA"; $resn_exch{"1H7* BDA"}="H7'1 BDA";
$resn_exch{"2H7* BDA"}="H7'2 BDA"; $resn_exch{"'HO8 BDA"}="HO8' BDA"; $resn_exch{"'H10 BDA"}="H10' BDA";
$resn_exch{"1HL' BDA"}="HL'1 BDA"; $resn_exch{"2HL' BDA"}="HL'2 BDA"; $resn_exch{"'H12 BDA"}="H12' BDA";
$resn_exch{"1HK' BDA"}="HK'1 BDA"; $resn_exch{"2HK' BDA"}="HK'2 BDA"; $resn_exch{"3HK' BDA"}="HK'3 BDA";
$resn_exch{"1HJ' BDA"}="HJ'1 BDA"; $resn_exch{"2HJ' BDA"}="HJ'2 BDA"; $resn_exch{"'H17 BDA"}="H17' BDA";
$resn_exch{"1HI' BDA"}="HI'1 BDA"; $resn_exch{"2HI' BDA"}="HI'2 BDA"; $resn_exch{"3HI' BDA"}="HI'3 BDA";
$resn_exch{"'H1* BDA"}=" H1D BDA"; $resn_exch{"1H2' BDA"}=" H2B BDA"; $resn_exch{"2H2' BDA"}=" H2D BDA";
$resn_exch{"'H3* BDA"}=" H3D BDA"; $resn_exch{"1HN' BDA"}=" HNB BDA"; $resn_exch{"2HN' BDA"}=" HND BDA";
$resn_exch{"'H4* BDA"}=" H4D BDA"; $resn_exch{"'HO4 BDA"}="HO4A BDA"; $resn_exch{"'H5* BDA"}=" H5D BDA";
$resn_exch{"1H6' BDA"}=" H6B BDA"; $resn_exch{"2H6' BDA"}=" H6D BDA"; $resn_exch{"3H6' BDA"}=" H6E BDA";
$resn_exch{"1H7' BDA"}=" H7B BDA"; $resn_exch{"2H7' BDA"}=" H7D BDA"; $resn_exch{"'H9A BDA"}="H9A' BDA";
$resn_exch{"'H9B BDA"}="H9B' BDA"; $resn_exch{"1HO1 BDB"}="HO11 BDB"; $resn_exch{"1HO2 BDB"}="HO21 BDB";
$resn_exch{"1HC2 BDB"}="HC21 BDB"; $resn_exch{"1HC3 BDB"}="HC31 BDB"; $resn_exch{"1HC5 BDB"}="HC51 BDB";
$resn_exch{"1HC6 BDB"}="HC61 BDB"; $resn_exch{"1HB2 BDB"}="HB21 BDB"; $resn_exch{"1HB3 BDB"}="HB31 BDB";
$resn_exch{"1HB5 BDB"}="HB51 BDB"; $resn_exch{"1HB6 BDB"}="HB61 BDB"; $resn_exch{"1HH1 BDB"}="HH11 BDB";
$resn_exch{"1HH2 BDB"}="HH21 BDB"; $resn_exch{"1HE1 BDC"}="HE11 BDC"; $resn_exch{"2HE1 BDC"}="HE12 BDC";
$resn_exch{"1HE3 BDC"}="HE31 BDC"; $resn_exch{"2HE3 BDC"}="HE32 BDC"; $resn_exch{"1HE2 BDC"}="HE21 BDC";
$resn_exch{"2HE2 BDC"}="HE22 BDC"; $resn_exch{"1HE4 BDC"}="HE41 BDC"; $resn_exch{"2HE4 BDC"}="HE42 BDC";
$resn_exch{"1HD2 BDC"}="HD21 BDC"; $resn_exch{"2HD2 BDC"}="HD22 BDC"; $resn_exch{"1HD3 BDC"}="HD31 BDC";
$resn_exch{"2HD3 BDC"}="HD32 BDC"; $resn_exch{"1HD7 BDC"}="HD71 BDC"; $resn_exch{"2HD7 BDC"}="HD72 BDC";
$resn_exch{"3HD7 BDC"}="HD73 BDC"; $resn_exch{"1HD8 BDC"}="HD81 BDC"; $resn_exch{"2HD8 BDC"}="HD82 BDC";
$resn_exch{"3HD8 BDC"}="HD83 BDC"; $resn_exch{"1H4' BDC"}="H4'1 BDC"; $resn_exch{"2H4' BDC"}="H4'2 BDC";
$resn_exch{"1H3' BDC"}="H3'1 BDC"; $resn_exch{"2H3' BDC"}="H3'2 BDC"; $resn_exch{"1H2' BDC"}="H2'1 BDC";
$resn_exch{"2H2' BDC"}="H2'2 BDC"; $resn_exch{"1H1' BDC"}="H1'1 BDC"; $resn_exch{"2H1' BDC"}="H1'2 BDC";
$resn_exch{"'H9  BDC"}=" H9' BDC"; $resn_exch{"'H5  BDC"}=" H5' BDC"; $resn_exch{"'H6  BDC"}=" H6' BDC";
$resn_exch{"'H7  BDC"}=" H7' BDC"; $resn_exch{"'H8  BDC"}=" H8' BDC"; $resn_exch{"'H1  BDC"}=" H1' BDC";
$resn_exch{"'H2  BDC"}=" H2' BDC"; $resn_exch{"'H3  BDC"}=" H3' BDC"; $resn_exch{"'HD1 BDC"}="HD1' BDC";
$resn_exch{"1H'2 BDC"}="H'21 BDC"; $resn_exch{"2H'2 BDC"}="H'22 BDC"; $resn_exch{"1H'3 BDC"}="H'31 BDC";
$resn_exch{"2H'3 BDC"}="H'32 BDC"; $resn_exch{"1H'7 BDC"}="H'71 BDC"; $resn_exch{"2H'7 BDC"}="H'72 BDC";
$resn_exch{"3H'7 BDC"}="H'73 BDC"; $resn_exch{"1H'8 BDC"}="H'81 BDC"; $resn_exch{"2H'8 BDC"}="H'82 BDC";
$resn_exch{"3H'8 BDC"}="H'83 BDC"; $resn_exch{"1H1  BDD"}=" H11 BDD"; $resn_exch{"2H1  BDD"}=" H12 BDD";
$resn_exch{"1H2  BDD"}=" H21 BDD"; $resn_exch{"2H2  BDD"}=" H22 BDD"; $resn_exch{"1H3  BDD"}=" H31 BDD";
$resn_exch{"2H3  BDD"}=" H32 BDD"; $resn_exch{"1H4  BDD"}=" H41 BDD"; $resn_exch{"2H4  BDD"}=" H42 BDD";
$resn_exch{"1H5  BDD"}=" H51 BDD"; $resn_exch{"2H5  BDD"}=" H52 BDD"; $resn_exch{"1H6  BDD"}=" H61 BDD";
$resn_exch{"2H6  BDD"}=" H62 BDD"; $resn_exch{"1H7  BDD"}=" H71 BDD"; $resn_exch{"2H7  BDD"}=" H72 BDD";
$resn_exch{"1H8  BDD"}=" H81 BDD"; $resn_exch{"2H8  BDD"}=" H82 BDD"; $resn_exch{"1H9  BDD"}=" H91 BDD";
$resn_exch{"2H9  BDD"}=" H92 BDD"; $resn_exch{"1H10 BDD"}="H101 BDD"; $resn_exch{"2H10 BDD"}="H102 BDD";
$resn_exch{"1H11 BDD"}="H111 BDD"; $resn_exch{"2H11 BDD"}="H112 BDD"; $resn_exch{"1H12 BDD"}="H121 BDD";
$resn_exch{"2H12 BDD"}="H122 BDD"; $resn_exch{"1H1  BDF"}=" H11 BDF"; $resn_exch{"2H1  BDF"}=" H12 BDF";
$resn_exch{"1H6  BDF"}=" H61 BDF"; $resn_exch{"2H6  BDF"}=" H62 BDF"; $resn_exch{"1HN2 BDG"}="HN21 BDG";
$resn_exch{"2HN2 BDG"}="HN22 BDG"; $resn_exch{"1H6  BDG"}=" H61 BDG"; $resn_exch{"2H6  BDG"}=" H62 BDG";
$resn_exch{"1HN6 BDG"}="HN61 BDG"; $resn_exch{"2HN6 BDG"}="HN62 BDG"; $resn_exch{"1HN1 BDH"}="HN11 BDH";
$resn_exch{"2HN1 BDH"}="HN12 BDH"; $resn_exch{"1H2  BDH"}=" H21 BDH"; $resn_exch{"2H2  BDH"}=" H22 BDH";
$resn_exch{"1HB  BDH"}=" HB1 BDH"; $resn_exch{"2HB  BDH"}=" HB2 BDH"; $resn_exch{"1HC6 BDI"}="HC61 BDI";
$resn_exch{"2HC6 BDI"}="HC62 BDI"; $resn_exch{"1HC7 BDI"}="HC71 BDI"; $resn_exch{"2HC7 BDI"}="HC72 BDI";
$resn_exch{"1HC8 BDI"}="HC81 BDI"; $resn_exch{"2HC8 BDI"}="HC82 BDI"; $resn_exch{"1HC9 BDI"}="HC91 BDI";
$resn_exch{"2HC9 BDI"}="HC92 BDI"; $resn_exch{"3HC9 BDI"}="HC93 BDI"; $resn_exch{"1H5  BDK"}=" H51 BDK";
$resn_exch{"2H5  BDK"}=" H52 BDK"; $resn_exch{"3H5  BDK"}=" H53 BDK"; $resn_exch{"1H6  BDK"}=" H61 BDK";
$resn_exch{"2H6  BDK"}=" H62 BDK"; $resn_exch{"3H6  BDK"}=" H63 BDK"; $resn_exch{"1H8  BDK"}=" H81 BDK";
$resn_exch{"2H8  BDK"}=" H82 BDK"; $resn_exch{"1H18 BDK"}="H181 BDK"; $resn_exch{"2H18 BDK"}="H182 BDK";
$resn_exch{"1H19 BDK"}="H191 BDK"; $resn_exch{"2H19 BDK"}="H192 BDK"; $resn_exch{"1H20 BDK"}="H201 BDK";
$resn_exch{"2H20 BDK"}="H202 BDK"; $resn_exch{"1H21 BDK"}="H211 BDK"; $resn_exch{"2H21 BDK"}="H212 BDK";
$resn_exch{"1H22 BDK"}="H221 BDK"; $resn_exch{"2H22 BDK"}="H222 BDK"; $resn_exch{"1H23 BDK"}="H231 BDK";
$resn_exch{"2H23 BDK"}="H232 BDK"; $resn_exch{"1HN4 BDK"}="HN41 BDK"; $resn_exch{"2HN4 BDK"}="HN42 BDK";
$resn_exch{"1HN2 BDM"}="HN21 BDM"; $resn_exch{"2HN2 BDM"}="HN22 BDM"; $resn_exch{"1HN4 BDM"}="HN41 BDM";
$resn_exch{"2HN4 BDM"}="HN42 BDM"; $resn_exch{"1H7  BDM"}=" H71 BDM"; $resn_exch{"2H7  BDM"}=" H72 BDM";
$resn_exch{"1H14 BDM"}="H141 BDM"; $resn_exch{"2H14 BDM"}="H142 BDM"; $resn_exch{"3H14 BDM"}="H143 BDM";
$resn_exch{"1H15 BDM"}="H151 BDM"; $resn_exch{"2H15 BDM"}="H152 BDM"; $resn_exch{"1H16 BDM"}="H161 BDM";
$resn_exch{"2H16 BDM"}="H162 BDM"; $resn_exch{"1H17 BDM"}="H171 BDM"; $resn_exch{"2H17 BDM"}="H172 BDM";
$resn_exch{"1H19 BDM"}="H191 BDM"; $resn_exch{"2H19 BDM"}="H192 BDM"; $resn_exch{"1H20 BDM"}="H201 BDM";
$resn_exch{"2H20 BDM"}="H202 BDM"; $resn_exch{"1HN2 BDN"}="HN21 BDN"; $resn_exch{"2HN2 BDN"}="HN22 BDN";
$resn_exch{"1H5  BDR"}=" H51 BDR"; $resn_exch{"2H5  BDR"}=" H52 BDR"; $resn_exch{"1HC1 BDS"}="HC11 BDS";
$resn_exch{"2HXT BDS"}="HXT2 BDS"; $resn_exch{"1HC5 BDS"}="HC51 BDS"; $resn_exch{"2HC5 BDS"}="HC52 BDS";
$resn_exch{"1HC7 BDS"}="HC71 BDS"; $resn_exch{"1HC8 BDS"}="HC81 BDS"; $resn_exch{"1H11 BDS"}="H111 BDS";
$resn_exch{"2H11 BDS"}="H112 BDS"; $resn_exch{"1H14 BDS"}="H141 BDS"; $resn_exch{"1H15 BDS"}="H151 BDS";
$resn_exch{"1HXT BDS"}="HXT1 BDS"; $resn_exch{"1H19 BDS"}="H191 BDS"; $resn_exch{"2H19 BDS"}="H192 BDS";
$resn_exch{"1H21 BDS"}="H211 BDS"; $resn_exch{"1H22 BDS"}="H221 BDS"; $resn_exch{"1H25 BDS"}="H251 BDS";
$resn_exch{"2H25 BDS"}="H252 BDS"; $resn_exch{"1H28 BDS"}="H281 BDS"; $resn_exch{"1HO2 BDU"}="HO21 BDU";
$resn_exch{"2HO2 BDU"}="HO22 BDU"; $resn_exch{"3HO2 BDU"}="HO23 BDU"; $resn_exch{"1HX3 BDU"}="HX31 BDU";
$resn_exch{"2HX3 BDU"}="HX32 BDU"; $resn_exch{"1H11 BDU"}="H111 BDU"; $resn_exch{"2H11 BDU"}="H112 BDU";
$resn_exch{"3H11 BDU"}="H113 BDU"; $resn_exch{"1HX9 BDU"}="HX91 BDU"; $resn_exch{"2HX9 BDU"}="HX92 BDU";
$resn_exch{"2HOP BE1"}="HOP2 BE1"; $resn_exch{"3HOP BE1"}="HOP3 BE1"; $resn_exch{"1HN2 BE2"}="HN21 BE2";
$resn_exch{"2HN2 BE2"}="HN22 BE2"; $resn_exch{"1H13 BE3"}="H131 BE3"; $resn_exch{"2H13 BE3"}="H132 BE3";
$resn_exch{"1H23 BE3"}="H231 BE3"; $resn_exch{"2H23 BE3"}="H232 BE3"; $resn_exch{"1H38 BE3"}="H381 BE3";
$resn_exch{"2H38 BE3"}="H382 BE3"; $resn_exch{"1H48 BE3"}="H481 BE3"; $resn_exch{"2H48 BE3"}="H482 BE3";
$resn_exch{"1H13 BE4"}="H131 BE4"; $resn_exch{"2H13 BE4"}="H132 BE4"; $resn_exch{"1H23 BE4"}="H231 BE4";
$resn_exch{"2H23 BE4"}="H232 BE4"; $resn_exch{"1H38 BE4"}="H381 BE4"; $resn_exch{"2H38 BE4"}="H382 BE4";
$resn_exch{"1H48 BE4"}="H481 BE4"; $resn_exch{"2H48 BE4"}="H482 BE4"; $resn_exch{"1H13 BE5"}="H131 BE5";
$resn_exch{"2H13 BE5"}="H132 BE5"; $resn_exch{"1H23 BE5"}="H231 BE5"; $resn_exch{"2H23 BE5"}="H232 BE5";
$resn_exch{"1H38 BE5"}="H381 BE5"; $resn_exch{"2H38 BE5"}="H382 BE5"; $resn_exch{"1H48 BE5"}="H481 BE5";
$resn_exch{"2H48 BE5"}="H482 BE5"; $resn_exch{"1H13 BE6"}="H131 BE6"; $resn_exch{"2H13 BE6"}="H132 BE6";
$resn_exch{"1H23 BE6"}="H231 BE6"; $resn_exch{"2H23 BE6"}="H232 BE6"; $resn_exch{"1H38 BE6"}="H381 BE6";
$resn_exch{"2H38 BE6"}="H382 BE6"; $resn_exch{"1H48 BE6"}="H481 BE6"; $resn_exch{"2H48 BE6"}="H482 BE6";
$resn_exch{"1H2  BEA"}=" H21 BEA"; $resn_exch{"2H2  BEA"}=" H22 BEA"; $resn_exch{"3H2  BEA"}=" H23 BEA";
$resn_exch{"1H32 BEB"}="H321 BEB"; $resn_exch{"2H32 BEB"}="H322 BEB"; $resn_exch{"1H22 BEB"}="H221 BEB";
$resn_exch{"2H22 BEB"}="H222 BEB"; $resn_exch{"1H58 BEB"}="H581 BEB"; $resn_exch{"2H58 BEB"}="H582 BEB";
$resn_exch{"1H48 BEB"}="H481 BEB"; $resn_exch{"2H48 BEB"}="H482 BEB"; $resn_exch{"1H32 BEC"}="H321 BEC";
$resn_exch{"2H32 BEC"}="H322 BEC"; $resn_exch{"1H22 BEC"}="H221 BEC"; $resn_exch{"2H22 BEC"}="H222 BEC";
$resn_exch{"1H58 BEC"}="H581 BEC"; $resn_exch{"2H58 BEC"}="H582 BEC"; $resn_exch{"1H43 BEC"}="H431 BEC";
$resn_exch{"2H43 BEC"}="H432 BEC"; $resn_exch{"3H43 BEC"}="H433 BEC"; $resn_exch{"1H44 BEC"}="H441 BEC";
$resn_exch{"2H44 BEC"}="H442 BEC"; $resn_exch{"3H44 BEC"}="H443 BEC"; $resn_exch{"1H48 BEC"}="H481 BEC";
$resn_exch{"2H48 BEC"}="H482 BEC"; $resn_exch{"3H48 BEC"}="H483 BEC"; $resn_exch{"1H13 BED"}="H131 BED";
$resn_exch{"2H13 BED"}="H132 BED"; $resn_exch{"1H23 BED"}="H231 BED"; $resn_exch{"2H23 BED"}="H232 BED";
$resn_exch{"1H38 BED"}="H381 BED"; $resn_exch{"2H38 BED"}="H382 BED"; $resn_exch{"1H48 BED"}="H481 BED";
$resn_exch{"2H48 BED"}="H482 BED"; $resn_exch{"1H13 BEE"}="H131 BEE"; $resn_exch{"2H13 BEE"}="H132 BEE";
$resn_exch{"1H24 BEE"}="H241 BEE"; $resn_exch{"2H24 BEE"}="H242 BEE"; $resn_exch{"3H24 BEE"}="H243 BEE";
$resn_exch{"1H25 BEE"}="H251 BEE"; $resn_exch{"2H25 BEE"}="H252 BEE"; $resn_exch{"3H25 BEE"}="H253 BEE";
$resn_exch{"1H29 BEE"}="H291 BEE"; $resn_exch{"2H29 BEE"}="H292 BEE"; $resn_exch{"3H29 BEE"}="H293 BEE";
$resn_exch{"1H31 BEE"}="H311 BEE"; $resn_exch{"2H31 BEE"}="H312 BEE"; $resn_exch{"1H39 BEE"}="H391 BEE";
$resn_exch{"2H39 BEE"}="H392 BEE"; $resn_exch{"3H39 BEE"}="H393 BEE"; $resn_exch{"1H40 BEE"}="H401 BEE";
$resn_exch{"2H40 BEE"}="H402 BEE"; $resn_exch{"3H40 BEE"}="H403 BEE"; $resn_exch{"1H44 BEE"}="H441 BEE";
$resn_exch{"2H44 BEE"}="H442 BEE"; $resn_exch{"3H44 BEE"}="H443 BEE"; $resn_exch{"1H02 BEG"}="H021 BEG";
$resn_exch{"1H03 BEG"}="H031 BEG"; $resn_exch{"1H04 BEG"}="H041 BEG"; $resn_exch{"1H05 BEG"}="H051 BEG";
$resn_exch{"1H06 BEG"}="H061 BEG"; $resn_exch{"1H08 BEG"}="H081 BEG"; $resn_exch{"1H09 BEG"}="H091 BEG";
$resn_exch{"1H10 BEG"}="H101 BEG"; $resn_exch{"1H11 BEG"}="H111 BEG"; $resn_exch{"1H12 BEG"}="H121 BEG";
$resn_exch{"1H13 BEG"}="H131 BEG"; $resn_exch{"2H13 BEG"}="H132 BEG"; $resn_exch{"1H23 BEG"}="H231 BEG";
$resn_exch{"2H23 BEG"}="H232 BEG"; $resn_exch{"1H15 BEG"}="H151 BEG"; $resn_exch{"1H16 BEG"}="H161 BEG";
$resn_exch{"2H16 BEG"}="H162 BEG"; $resn_exch{"1H17 BEG"}="H171 BEG"; $resn_exch{"1H24 BEG"}="H241 BEG";
$resn_exch{"1H18 BEG"}="H181 BEG"; $resn_exch{"1H21 BEG"}="H211 BEG"; $resn_exch{"1H27 BEG"}="H271 BEG";
$resn_exch{"1H28 BEG"}="H281 BEG"; $resn_exch{"1H31 BEG"}="H311 BEG"; $resn_exch{"1H32 BEG"}="H321 BEG";
$resn_exch{"1H33 BEG"}="H331 BEG"; $resn_exch{"1H34 BEG"}="H341 BEG"; $resn_exch{"1H35 BEG"}="H351 BEG";
$resn_exch{"1H36 BEG"}="H361 BEG"; $resn_exch{"1H37 BEG"}="H371 BEG"; $resn_exch{"2H37 BEG"}="H372 BEG";
$resn_exch{"1H38 BEG"}="H381 BEG"; $resn_exch{"1H41 BEG"}="H411 BEG"; $resn_exch{"1H42 BEG"}="H421 BEG";
$resn_exch{"1H43 BEG"}="H431 BEG"; $resn_exch{"1H44 BEG"}="H441 BEG"; $resn_exch{"1H45 BEG"}="H451 BEG";
$resn_exch{"1H46 BEG"}="H461 BEG"; $resn_exch{"1H47 BEG"}="H471 BEG"; $resn_exch{"2H47 BEG"}="H472 BEG";
$resn_exch{"1H01 BEH"}="H011 BEH"; $resn_exch{"1H02 BEH"}="H021 BEH"; $resn_exch{"1H03 BEH"}="H031 BEH";
$resn_exch{"1H05 BEH"}="H051 BEH"; $resn_exch{"2H05 BEH"}="H052 BEH"; $resn_exch{"1H07 BEH"}="H071 BEH";
$resn_exch{"1H08 BEH"}="H081 BEH"; $resn_exch{"1H09 BEH"}="H091 BEH"; $resn_exch{"1H10 BEH"}="H101 BEH";
$resn_exch{"1H11 BEH"}="H111 BEH"; $resn_exch{"1H14 BEH"}="H141 BEH"; $resn_exch{"1H15 BEH"}="H151 BEH";
$resn_exch{"1H17 BEH"}="H171 BEH"; $resn_exch{"1H18 BEH"}="H181 BEH"; $resn_exch{"1H19 BEH"}="H191 BEH";
$resn_exch{"1H20 BEH"}="H201 BEH"; $resn_exch{"1H22 BEH"}="H221 BEH"; $resn_exch{"2H22 BEH"}="H222 BEH";
$resn_exch{"1H23 BEH"}="H231 BEH"; $resn_exch{"1H24 BEH"}="H241 BEH"; $resn_exch{"1H31 BEH"}="H311 BEH";
$resn_exch{"1H32 BEH"}="H321 BEH"; $resn_exch{"1H33 BEH"}="H331 BEH"; $resn_exch{"1H35 BEH"}="H351 BEH";
$resn_exch{"2H35 BEH"}="H352 BEH"; $resn_exch{"1H37 BEH"}="H371 BEH"; $resn_exch{"1H38 BEH"}="H381 BEH";
$resn_exch{"1H39 BEH"}="H391 BEH"; $resn_exch{"1H40 BEH"}="H401 BEH"; $resn_exch{"1H41 BEH"}="H411 BEH";
$resn_exch{"1H44 BEH"}="H441 BEH"; $resn_exch{"1H45 BEH"}="H451 BEH"; $resn_exch{"2H45 BEH"}="H452 BEH";
$resn_exch{"1H47 BEH"}="H471 BEH"; $resn_exch{"1H48 BEH"}="H481 BEH"; $resn_exch{"1H49 BEH"}="H491 BEH";
$resn_exch{"1H50 BEH"}="H501 BEH"; $resn_exch{"1H51 BEH"}="H511 BEH"; $resn_exch{"1H30 BEI"}="H301 BEI";
$resn_exch{"2H30 BEI"}="H302 BEI"; $resn_exch{"1H21 BEI"}="H211 BEI"; $resn_exch{"2H21 BEI"}="H212 BEI";
$resn_exch{"3H21 BEI"}="H213 BEI"; $resn_exch{"1H22 BEI"}="H221 BEI"; $resn_exch{"2H22 BEI"}="H222 BEI";
$resn_exch{"1H46 BEI"}="H461 BEI"; $resn_exch{"2H46 BEI"}="H462 BEI"; $resn_exch{"3H46 BEI"}="H463 BEI";
$resn_exch{"1H26 BEI"}="H261 BEI"; $resn_exch{"2H26 BEI"}="H262 BEI"; $resn_exch{"3H26 BEI"}="H263 BEI";
$resn_exch{"1H33 BEI"}="H331 BEI"; $resn_exch{"2H33 BEI"}="H332 BEI"; $resn_exch{"1H11 BEI"}="H111 BEI";
$resn_exch{"2H11 BEI"}="H112 BEI"; $resn_exch{"3H11 BEI"}="H113 BEI"; $resn_exch{"1H12 BEI"}="H121 BEI";
$resn_exch{"2H12 BEI"}="H122 BEI"; $resn_exch{"1H47 BEI"}="H471 BEI"; $resn_exch{"2H47 BEI"}="H472 BEI";
$resn_exch{"3H47 BEI"}="H473 BEI"; $resn_exch{"1H16 BEI"}="H161 BEI"; $resn_exch{"2H16 BEI"}="H162 BEI";
$resn_exch{"3H16 BEI"}="H163 BEI"; $resn_exch{"1H13 BEJ"}="H131 BEJ"; $resn_exch{"2H13 BEJ"}="H132 BEJ";
$resn_exch{"1H24 BEJ"}="H241 BEJ"; $resn_exch{"2H24 BEJ"}="H242 BEJ"; $resn_exch{"3H24 BEJ"}="H243 BEJ";
$resn_exch{"1H25 BEJ"}="H251 BEJ"; $resn_exch{"2H25 BEJ"}="H252 BEJ"; $resn_exch{"3H25 BEJ"}="H253 BEJ";
$resn_exch{"1H29 BEJ"}="H291 BEJ"; $resn_exch{"2H29 BEJ"}="H292 BEJ"; $resn_exch{"3H29 BEJ"}="H293 BEJ";
$resn_exch{"1H31 BEJ"}="H311 BEJ"; $resn_exch{"2H31 BEJ"}="H312 BEJ"; $resn_exch{"1H39 BEJ"}="H391 BEJ";
$resn_exch{"2H39 BEJ"}="H392 BEJ"; $resn_exch{"3H39 BEJ"}="H393 BEJ"; $resn_exch{"1H40 BEJ"}="H401 BEJ";
$resn_exch{"2H40 BEJ"}="H402 BEJ"; $resn_exch{"3H40 BEJ"}="H403 BEJ"; $resn_exch{"1H44 BEJ"}="H441 BEJ";
$resn_exch{"2H44 BEJ"}="H442 BEJ"; $resn_exch{"3H44 BEJ"}="H443 BEJ"; $resn_exch{"1HN  BEK"}=" HN1 BEK";
$resn_exch{"2HN  BEK"}=" HN2 BEK"; $resn_exch{"1H8  BEK"}=" H81 BEK"; $resn_exch{"2H8  BEK"}=" H82 BEK";
$resn_exch{"3H8  BEK"}=" H83 BEK"; $resn_exch{"1H7  BEL"}=" H71 BEL"; $resn_exch{"2H7  BEL"}=" H72 BEL";
$resn_exch{"1H8  BEL"}=" H81 BEL"; $resn_exch{"2H8  BEL"}=" H82 BEL"; $resn_exch{"1H9  BEL"}=" H91 BEL";
$resn_exch{"2H9  BEL"}=" H92 BEL"; $resn_exch{"1H10 BEL"}="H101 BEL"; $resn_exch{"2H10 BEL"}="H102 BEL";
$resn_exch{"1H12 BEL"}="H121 BEL"; $resn_exch{"2H12 BEL"}="H122 BEL"; $resn_exch{"1HN2 BEN"}="HN21 BEN";
$resn_exch{"2HN2 BEN"}="HN22 BEN"; $resn_exch{"1H4  BEO"}=" H41 BEO"; $resn_exch{"2H4  BEO"}=" H42 BEO";
$resn_exch{"3H4  BEO"}=" H43 BEO"; $resn_exch{"1H1  BEP"}=" H11 BEP"; $resn_exch{"2H1  BEP"}=" H12 BEP";
$resn_exch{"1H2  BEP"}=" H21 BEP"; $resn_exch{"1H3  BEP"}=" H31 BEP"; $resn_exch{"2H3  BEP"}=" H32 BEP";
$resn_exch{"1H4  BEP"}=" H41 BEP"; $resn_exch{"2H4  BEP"}=" H42 BEP"; $resn_exch{"1H5  BEP"}=" H51 BEP";
$resn_exch{"1H6  BEP"}=" H61 BEP"; $resn_exch{"2H6  BEP"}=" H62 BEP"; $resn_exch{"3H6  BEP"}=" H63 BEP";
$resn_exch{"1H7  BEP"}=" H71 BEP"; $resn_exch{"2H7  BEP"}=" H72 BEP"; $resn_exch{"3H7  BEP"}=" H73 BEP";
$resn_exch{"1H8  BEP"}=" H81 BEP"; $resn_exch{"2H8  BEP"}=" H82 BEP"; $resn_exch{"1H9  BEP"}=" H91 BEP";
$resn_exch{"2H9  BEP"}=" H92 BEP"; $resn_exch{"1H10 BEP"}="H101 BEP"; $resn_exch{"2H10 BEP"}="H102 BEP";
$resn_exch{"1H11 BEP"}="H111 BEP"; $resn_exch{"2H11 BEP"}="H112 BEP"; $resn_exch{"1H12 BEP"}="H121 BEP";
$resn_exch{"2H12 BEP"}="H122 BEP"; $resn_exch{"1H14 BEP"}="H141 BEP"; $resn_exch{"1H15 BEP"}="H151 BEP";
$resn_exch{"1H16 BEP"}="H161 BEP"; $resn_exch{"1H17 BEP"}="H171 BEP"; $resn_exch{"1H18 BEP"}="H181 BEP";
$resn_exch{"1H20 BEP"}="H201 BEP"; $resn_exch{"1H21 BEP"}="H211 BEP"; $resn_exch{"1H22 BEP"}="H221 BEP";
$resn_exch{"1H23 BEP"}="H231 BEP"; $resn_exch{"1H24 BEP"}="H241 BEP"; $resn_exch{"1HB  BEQ"}=" HB1 BEQ";
$resn_exch{"2HB  BEQ"}=" HB2 BEQ"; $resn_exch{"3HB  BEQ"}=" HB3 BEQ"; $resn_exch{"1HG  BEQ"}=" HG1 BEQ";
$resn_exch{"2HG  BEQ"}=" HG2 BEQ"; $resn_exch{"3HG  BEQ"}=" HG3 BEQ"; $resn_exch{"1HD  BEQ"}=" HD1 BEQ";
$resn_exch{"2HD  BEQ"}=" HD2 BEQ"; $resn_exch{"1H1  BEQ"}=" H11 BEQ"; $resn_exch{"2H1  BEQ"}=" H12 BEQ";
$resn_exch{"1H2  BEQ"}=" H21 BEQ"; $resn_exch{"2H2  BEQ"}=" H22 BEQ"; $resn_exch{"1H3  BEQ"}=" H31 BEQ";
$resn_exch{"2H3  BEQ"}=" H32 BEQ"; $resn_exch{"1H5  BEQ"}=" H51 BEQ"; $resn_exch{"2H5  BEQ"}=" H52 BEQ";
$resn_exch{"1H6  BEQ"}=" H61 BEQ"; $resn_exch{"2H6  BEQ"}=" H62 BEQ"; $resn_exch{"1H7  BEQ"}=" H71 BEQ";
$resn_exch{"2H7  BEQ"}=" H72 BEQ"; $resn_exch{"1H8  BEQ"}=" H81 BEQ"; $resn_exch{"2H8  BEQ"}=" H82 BEQ";
$resn_exch{"1H9  BEQ"}=" H91 BEQ"; $resn_exch{"2H9  BEQ"}=" H92 BEQ"; $resn_exch{"1H10 BEQ"}="H101 BEQ";
$resn_exch{"2H10 BEQ"}="H102 BEQ"; $resn_exch{"1H11 BEQ"}="H111 BEQ"; $resn_exch{"2H11 BEQ"}="H112 BEQ";
$resn_exch{"1H12 BEQ"}="H121 BEQ"; $resn_exch{"2H12 BEQ"}="H122 BEQ"; $resn_exch{"1H13 BEQ"}="H131 BEQ";
$resn_exch{"2H13 BEQ"}="H132 BEQ"; $resn_exch{"1H14 BEQ"}="H141 BEQ"; $resn_exch{"2H14 BEQ"}="H142 BEQ";
$resn_exch{"1H15 BEQ"}="H151 BEQ"; $resn_exch{"2H15 BEQ"}="H152 BEQ"; $resn_exch{"3H15 BEQ"}="H153 BEQ";
$resn_exch{"1H3  BER"}=" H31 BER"; $resn_exch{"1H5  BER"}=" H51 BER"; $resn_exch{"1H6  BER"}=" H61 BER";
$resn_exch{"1H7  BER"}=" H71 BER"; $resn_exch{"2H7  BER"}=" H72 BER"; $resn_exch{"1H9  BER"}=" H91 BER";
$resn_exch{"1H10 BER"}="H101 BER"; $resn_exch{"2H10 BER"}="H102 BER"; $resn_exch{"1H13 BER"}="H131 BER";
$resn_exch{"1H16 BER"}="H161 BER"; $resn_exch{"1H17 BER"}="H171 BER"; $resn_exch{"2H17 BER"}="H172 BER";
$resn_exch{"1H19 BER"}="H191 BER"; $resn_exch{"2H19 BER"}="H192 BER"; $resn_exch{"3H19 BER"}="H193 BER";
$resn_exch{"1H20 BER"}="H201 BER"; $resn_exch{"2H20 BER"}="H202 BER"; $resn_exch{"3H20 BER"}="H203 BER";
$resn_exch{"1HN2 BES"}="HN21 BES"; $resn_exch{"2HN2 BES"}="HN22 BES"; $resn_exch{"1H6  BES"}=" H61 BES";
$resn_exch{"2H6  BES"}=" H62 BES"; $resn_exch{"1H13 BES"}="H131 BES"; $resn_exch{"2H13 BES"}="H132 BES";
$resn_exch{"1H15 BES"}="H151 BES"; $resn_exch{"2H15 BES"}="H152 BES"; $resn_exch{"3H15 BES"}="H153 BES";
$resn_exch{"1H16 BES"}="H161 BES"; $resn_exch{"2H16 BES"}="H162 BES"; $resn_exch{"3H16 BES"}="H163 BES";
$resn_exch{"1HA  BET"}=" HA1 BET"; $resn_exch{"2HA  BET"}=" HA2 BET"; $resn_exch{"1H1  BET"}=" H11 BET";
$resn_exch{"2H1  BET"}=" H12 BET"; $resn_exch{"3H1  BET"}=" H13 BET"; $resn_exch{"1H2  BET"}=" H21 BET";
$resn_exch{"2H2  BET"}=" H22 BET"; $resn_exch{"3H2  BET"}=" H23 BET"; $resn_exch{"1H3  BET"}=" H31 BET";
$resn_exch{"2H3  BET"}=" H32 BET"; $resn_exch{"3H3  BET"}=" H33 BET"; $resn_exch{"1H'C BEU"}="H'C1 BEU";
$resn_exch{"2H'C BEU"}="H'C2 BEU"; $resn_exch{"1H3N BEU"}="H3N1 BEU"; $resn_exch{"2H3N BEU"}="H3N2 BEU";
$resn_exch{"1HM4 BEU"}="HM41 BEU"; $resn_exch{"2HM4 BEU"}="HM42 BEU"; $resn_exch{"3HM4 BEU"}="HM43 BEU";
$resn_exch{"1HD2 BFA"}="HD21 BFA"; $resn_exch{"2HD2 BFA"}="HD22 BFA"; $resn_exch{"1HD3 BFA"}="HD31 BFA";
$resn_exch{"2HD3 BFA"}="HD32 BFA"; $resn_exch{"1HD7 BFA"}="HD71 BFA"; $resn_exch{"2HD7 BFA"}="HD72 BFA";
$resn_exch{"3HD7 BFA"}="HD73 BFA"; $resn_exch{"1HD8 BFA"}="HD81 BFA"; $resn_exch{"2HD8 BFA"}="HD82 BFA";
$resn_exch{"3HD8 BFA"}="HD83 BFA"; $resn_exch{"1HAA BFB"}="HAA1 BFB"; $resn_exch{"2HAA BFB"}="HAA2 BFB";
$resn_exch{"1HAO BFB"}="HAO1 BFB"; $resn_exch{"2HAO BFB"}="HAO2 BFB"; $resn_exch{"1HAM BFB"}="HAM1 BFB";
$resn_exch{"2HAM BFB"}="HAM2 BFB"; $resn_exch{"1HAN BFB"}="HAN1 BFB"; $resn_exch{"2HAN BFB"}="HAN2 BFB";
$resn_exch{"1HAP BFB"}="HAP1 BFB"; $resn_exch{"2HAP BFB"}="HAP2 BFB"; $resn_exch{"3HAP BFB"}="HAP3 BFB";
$resn_exch{"1HA  BFC"}=" HA1 BFC"; $resn_exch{"2HA  BFC"}=" HA2 BFC"; $resn_exch{"1HC  BFC"}=" HC1 BFC";
$resn_exch{"2HC  BFC"}=" HC2 BFC"; $resn_exch{"1HD  BFC"}=" HD1 BFC"; $resn_exch{"2HD  BFC"}=" HD2 BFC";
$resn_exch{"1HE  BFC"}=" HE1 BFC"; $resn_exch{"2HE  BFC"}=" HE2 BFC"; $resn_exch{"1HF  BFC"}=" HF1 BFC";
$resn_exch{"2HF  BFC"}=" HF2 BFC"; $resn_exch{"1HG  BFC"}=" HG1 BFC"; $resn_exch{"2HG  BFC"}=" HG2 BFC";
$resn_exch{"1HH  BFC"}=" HH1 BFC"; $resn_exch{"2HH  BFC"}=" HH2 BFC"; $resn_exch{"1HI  BFC"}=" HI1 BFC";
$resn_exch{"2HI  BFC"}=" HI2 BFC"; $resn_exch{"1HJ  BFC"}=" HJ1 BFC"; $resn_exch{"2HJ  BFC"}=" HJ2 BFC";
$resn_exch{"1HK  BFC"}=" HK1 BFC"; $resn_exch{"2HK  BFC"}=" HK2 BFC"; $resn_exch{"1HL  BFC"}=" HL1 BFC";
$resn_exch{"2HL  BFC"}=" HL2 BFC"; $resn_exch{"1HM  BFC"}=" HM1 BFC"; $resn_exch{"2HM  BFC"}=" HM2 BFC";
$resn_exch{"3HM  BFC"}=" HM3 BFC"; $resn_exch{" HN2 BFD"}=" H2  BFD"; $resn_exch{"1HB  BFD"}=" HB2 BFD";
$resn_exch{"2HB  BFD"}=" HB3 BFD"; $resn_exch{"1H18 BFI"}="H181 BFI"; $resn_exch{"2H18 BFI"}="H182 BFI";
$resn_exch{"1H23 BFI"}="H231 BFI"; $resn_exch{"2H23 BFI"}="H232 BFI"; $resn_exch{"1H7  BFL"}=" H71 BFL";
$resn_exch{"2H7  BFL"}=" H72 BFL"; $resn_exch{"3H7  BFL"}=" H73 BFL"; $resn_exch{"2HOP BFP"}="HOP2 BFP";
$resn_exch{"3HOP BFP"}="HOP3 BFP"; $resn_exch{"1H1  BFP"}=" H11 BFP"; $resn_exch{"2H1  BFP"}=" H12 BFP";
$resn_exch{"1H6  BFP"}=" H61 BFP"; $resn_exch{"2H6  BFP"}=" H62 BFP"; $resn_exch{"5HOP BFP"}="HOP5 BFP";
$resn_exch{"6HOP BFP"}="HOP6 BFP"; $resn_exch{"1H16 BFQ"}="H161 BFQ"; $resn_exch{"2H16 BFQ"}="H162 BFQ";
$resn_exch{"1H19 BFQ"}="H191 BFQ"; $resn_exch{"2H19 BFQ"}="H192 BFQ"; $resn_exch{"1H23 BFQ"}="H231 BFQ";
$resn_exch{"2H23 BFQ"}="H232 BFQ"; $resn_exch{"1H26 BFQ"}="H261 BFQ"; $resn_exch{"2H26 BFQ"}="H262 BFQ";
$resn_exch{"1H29 BFQ"}="H291 BFQ"; $resn_exch{"2H29 BFQ"}="H292 BFQ"; $resn_exch{"1H32 BFQ"}="H321 BFQ";
$resn_exch{"2H32 BFQ"}="H322 BFQ"; $resn_exch{"1H35 BFQ"}="H351 BFQ"; $resn_exch{"2H35 BFQ"}="H352 BFQ";
$resn_exch{"3H35 BFQ"}="H353 BFQ"; $resn_exch{"1H39 BFQ"}="H391 BFQ"; $resn_exch{"2H39 BFQ"}="H392 BFQ";
$resn_exch{"3H39 BFQ"}="H393 BFQ"; $resn_exch{"1HE2 BFS"}="HE21 BFS"; $resn_exch{"2HE2 BFS"}="HE22 BFS";
$resn_exch{"3HE2 BFS"}="HE23 BFS"; $resn_exch{"1HM2 BFT"}="HM21 BFT"; $resn_exch{"2HM2 BFT"}="HM22 BFT";
$resn_exch{"3HM2 BFT"}="HM23 BFT"; $resn_exch{"1HM4 BFT"}="HM41 BFT"; $resn_exch{"2HM4 BFT"}="HM42 BFT";
$resn_exch{"3HM4 BFT"}="HM43 BFT"; $resn_exch{"1H6  BFT"}=" H61 BFT"; $resn_exch{"2H6  BFT"}=" H62 BFT";
$resn_exch{"1H7  BFT"}=" H71 BFT"; $resn_exch{"2H7  BFT"}=" H72 BFT"; $resn_exch{"1H4A BFT"}="H4A1 BFT";
$resn_exch{"2H4A BFT"}="H4A2 BFT"; $resn_exch{"1H7A BFT"}="H7A1 BFT"; $resn_exch{"2H7A BFT"}="H7A2 BFT";
$resn_exch{"1H13 BFU"}="H131 BFU"; $resn_exch{"2H13 BFU"}="H132 BFU"; $resn_exch{"1H25 BFU"}="H251 BFU";
$resn_exch{"2H25 BFU"}="H252 BFU"; $resn_exch{"1H26 BFU"}="H261 BFU"; $resn_exch{"2H26 BFU"}="H262 BFU";
$resn_exch{"3H26 BFU"}="H263 BFU"; $resn_exch{"1H4  BG3"}=" H41 BG3"; $resn_exch{"2H4  BG3"}=" H42 BG3";
$resn_exch{"1H6  BG3"}=" H61 BG3"; $resn_exch{"2H6  BG3"}=" H62 BG3"; $resn_exch{"3H6  BG3"}=" H63 BG3";
$resn_exch{"1H4  BG4"}=" H41 BG4"; $resn_exch{"2H4  BG4"}=" H42 BG4"; $resn_exch{"1H6  BG4"}=" H61 BG4";
$resn_exch{"2H6  BG4"}=" H62 BG4"; $resn_exch{"3H6  BG4"}=" H63 BG4"; $resn_exch{"1H7  BG4"}=" H71 BG4";
$resn_exch{"2H7  BG4"}=" H72 BG4"; $resn_exch{"3H7  BG4"}=" H73 BG4"; $resn_exch{"1H4  BG5"}=" H41 BG5";
$resn_exch{"2H4  BG5"}=" H42 BG5"; $resn_exch{"1H6  BG5"}=" H61 BG5"; $resn_exch{"2H6  BG5"}=" H62 BG5";
$resn_exch{"3H6  BG5"}=" H63 BG5"; $resn_exch{"1HC6 BG6"}="HC61 BG6"; $resn_exch{"2HC6 BG6"}="HC62 BG6";
$resn_exch{"1H1O BG6"}="H1O1 BG6"; $resn_exch{"2H2O BG6"}="H2O2 BG6"; $resn_exch{"1H6C BGC"}="H6C1 BGC";
$resn_exch{"2H6C BGC"}="H6C2 BGC"; $resn_exch{"1H3  BGD"}=" H31 BGD"; $resn_exch{"2H3  BGD"}=" H32 BGD";
$resn_exch{"1H4  BGD"}="H41A BGD"; $resn_exch{"2H4  BGD"}="H42A BGD"; $resn_exch{"1H22 BGD"}="H221 BGD";
$resn_exch{"2H22 BGD"}="H222 BGD"; $resn_exch{"1H38 BGD"}="H381 BGD"; $resn_exch{"2H38 BGD"}="H382 BGD";
$resn_exch{"1H54 BGD"}="H541 BGD"; $resn_exch{"2H54 BGD"}="H542 BGD"; $resn_exch{"1HN1 BGF"}="HN11 BGF";
$resn_exch{"2HN1 BGF"}="HN12 BGF"; $resn_exch{"1HN2 BGF"}="HN21 BGF"; $resn_exch{"2HN2 BGF"}="HN22 BGF";
$resn_exch{"1H1' BGF"}="H1'1 BGF"; $resn_exch{"2H1' BGF"}="H1'2 BGF"; $resn_exch{"1H2' BGF"}="H2'1 BGF";
$resn_exch{"2H2' BGF"}="H2'2 BGF"; $resn_exch{"1H7  BGG"}=" H71 BGG"; $resn_exch{"2H7  BGG"}=" H72 BGG";
$resn_exch{"1H1' BGL"}="H1'1 BGL"; $resn_exch{"2H1' BGL"}="H1'2 BGL"; $resn_exch{"1H2' BGL"}="H2'1 BGL";
$resn_exch{"2H2' BGL"}="H2'2 BGL"; $resn_exch{"1H3' BGL"}="H3'1 BGL"; $resn_exch{"2H3' BGL"}="H3'2 BGL";
$resn_exch{"1H4' BGL"}="H4'1 BGL"; $resn_exch{"2H4' BGL"}="H4'2 BGL"; $resn_exch{"1H5' BGL"}="H5'1 BGL";
$resn_exch{"2H5' BGL"}="H5'2 BGL"; $resn_exch{"1H6' BGL"}="H6'1 BGL"; $resn_exch{"2H6' BGL"}="H6'2 BGL";
$resn_exch{"1H7' BGL"}="H7'1 BGL"; $resn_exch{"2H7' BGL"}="H7'2 BGL"; $resn_exch{"1H8' BGL"}="H8'1 BGL";
$resn_exch{"2H8' BGL"}="H8'2 BGL"; $resn_exch{"3H8' BGL"}="H8'3 BGL"; $resn_exch{"2HOP BGM"}="HOP2 BGM";
$resn_exch{"1H5* BGM"}=" H5' BGM"; $resn_exch{"2H5* BGM"}="H5'' BGM"; $resn_exch{" H4* BGM"}=" H4' BGM";
$resn_exch{" H1* BGM"}=" H1' BGM"; $resn_exch{"1H2  BGM"}=" H21 BGM"; $resn_exch{"2H2  BGM"}=" H22 BGM";
$resn_exch{"1H2* BGM"}=" H2' BGM"; $resn_exch{"2H2* BGM"}="H2'' BGM"; $resn_exch{" H3* BGM"}=" H3' BGM";
$resn_exch{"H3T  BGM"}="HO3' BGM"; $resn_exch{"3HOP BGM"}="HOP3 BGM"; $resn_exch{"1H6  BGN"}=" H61 BGN";
$resn_exch{"2H6  BGN"}=" H62 BGN"; $resn_exch{"1H8  BGN"}=" H81 BGN"; $resn_exch{"2H8  BGN"}=" H82 BGN";
$resn_exch{"1H9  BGN"}=" H91 BGN"; $resn_exch{"2H9  BGN"}=" H92 BGN"; $resn_exch{"1H10 BGN"}="H101 BGN";
$resn_exch{"2H10 BGN"}="H102 BGN"; $resn_exch{"3H10 BGN"}="H103 BGN"; $resn_exch{"1H6  BGP"}=" H61 BGP";
$resn_exch{"2H6  BGP"}=" H62 BGP"; $resn_exch{"2HOP BGP"}="HOP2 BGP"; $resn_exch{"3HOP BGP"}="HOP3 BGP";
$resn_exch{"1H2* BGS"}="H2'1 BGS"; $resn_exch{"2H2* BGS"}="H2'2 BGS"; $resn_exch{"1H6  BGS"}=" H61 BGS";
$resn_exch{"2H6  BGS"}=" H62 BGS"; $resn_exch{"1HS  BGS"}=" HS1 BGS"; $resn_exch{"2HS  BGS"}=" HS2 BGS";
$resn_exch{"3HS  BGS"}=" HS3 BGS"; $resn_exch{"1HB  BGT"}=" HB1 BGT"; $resn_exch{"2HB  BGT"}=" HB2 BGT";
$resn_exch{"1HG  BGT"}=" HG1 BGT"; $resn_exch{"2HG  BGT"}=" HG2 BGT"; $resn_exch{"1HE2 BGT"}="HE21 BGT";
$resn_exch{"2HE2 BGT"}="HE22 BGT"; $resn_exch{"1H1  BGT"}=" H11 BGT"; $resn_exch{"2H1  BGT"}=" H12 BGT";
$resn_exch{"3H1  BGT"}=" H13 BGT"; $resn_exch{"1H2  BGT"}=" H21 BGT"; $resn_exch{"2H2  BGT"}=" H22 BGT";
$resn_exch{"3H2  BGT"}=" H23 BGT"; $resn_exch{"1H3  BGT"}=" H31 BGT"; $resn_exch{"2H3  BGT"}=" H32 BGT";
$resn_exch{"3H3  BGT"}=" H33 BGT"; $resn_exch{"1HN  BGT"}=" HN1 BGT"; $resn_exch{"2HN  BGT"}=" HN2 BGT";
$resn_exch{"1HC6 BGX"}="HC61 BGX"; $resn_exch{"2HC6 BGX"}="HC62 BGX"; $resn_exch{"3HC6 BGX"}="HC63 BGX";
$resn_exch{"1H1  BH0"}=" H11 BH0"; $resn_exch{"1H2  BH0"}=" H21 BH0"; $resn_exch{"2H2  BH0"}=" H22 BH0";
$resn_exch{"4H4  BH0"}=" H44 BH0"; $resn_exch{"5H5  BH0"}=" H55 BH0"; $resn_exch{"6H6  BH0"}=" H66 BH0";
$resn_exch{"7H7  BH0"}=" H77 BH0"; $resn_exch{"8H8  BH0"}=" H88 BH0"; $resn_exch{"9H9  BH0"}=" H99 BH0";
$resn_exch{"1H10 BH0"}="H101 BH0"; $resn_exch{"2H10 BH0"}="H102 BH0"; $resn_exch{"1H14 BH0"}="H141 BH0";
$resn_exch{"2H14 BH0"}="H142 BH0"; $resn_exch{"1H15 BH0"}="H151 BH0"; $resn_exch{"2H15 BH0"}="H152 BH0";
$resn_exch{"17H1 BH0"}="H117 BH0"; $resn_exch{"18H1 BH0"}="H118 BH0"; $resn_exch{"20H2 BH0"}="H220 BH0";
$resn_exch{"21H2 BH0"}="H221 BH0"; $resn_exch{"23H2 BH0"}="H223 BH0"; $resn_exch{"24H2 BH0"}="H224 BH0";
$resn_exch{"26H2 BH0"}="H226 BH0"; $resn_exch{"27H2 BH0"}="H227 BH0"; $resn_exch{"31H3 BH0"}="H331 BH0";
$resn_exch{"32H3 BH0"}="H332 BH0"; $resn_exch{"34H3 BH0"}="H334 BH0"; $resn_exch{"13H1 BH0"}="H113 BH0";
$resn_exch{"1HED BH1"}="HED1 BH1"; $resn_exch{"2HED BH1"}="HED2 BH1"; $resn_exch{"3HED BH1"}="HED3 BH1";
$resn_exch{"1HMD BH1"}="HMD1 BH1"; $resn_exch{"2HMD BH1"}="HMD2 BH1"; $resn_exch{"3HMD BH1"}="HMD3 BH1";
$resn_exch{"1HAC BH1"}="HAC1 BH1"; $resn_exch{"2HAC BH1"}="HAC2 BH1"; $resn_exch{"1HBC BH1"}="HBC1 BH1";
$resn_exch{"2HBC BH1"}="HBC2 BH1"; $resn_exch{"3HBC BH1"}="HBC3 BH1"; $resn_exch{"1HMC BH1"}="HMC1 BH1";
$resn_exch{"2HMC BH1"}="HMC2 BH1"; $resn_exch{"3HMC BH1"}="HMC3 BH1"; $resn_exch{"1HBB BH1"}="HBB1 BH1";
$resn_exch{"2HBB BH1"}="HBB2 BH1"; $resn_exch{"3HBB BH1"}="HBB3 BH1"; $resn_exch{"1HMB BH1"}="HMB1 BH1";
$resn_exch{"2HMB BH1"}="HMB2 BH1"; $resn_exch{"3HMB BH1"}="HMB3 BH1"; $resn_exch{"1HMA BH1"}="HMA1 BH1";
$resn_exch{"2HMA BH1"}="HMA2 BH1"; $resn_exch{"3HMA BH1"}="HMA3 BH1"; $resn_exch{"1HAA BH1"}="HAA1 BH1";
$resn_exch{"2HAA BH1"}="HAA2 BH1"; $resn_exch{"1HBA BH1"}="HBA1 BH1"; $resn_exch{"2HBA BH1"}="HBA2 BH1";
$resn_exch{"1H1C BH1"}="H1C1 BH1"; $resn_exch{"2H1C BH1"}="H1C2 BH1"; $resn_exch{"1H4C BH1"}="H4C1 BH1";
$resn_exch{"2H4C BH1"}="H4C2 BH1"; $resn_exch{"3H4C BH1"}="H4C3 BH1"; $resn_exch{"1H5C BH1"}="H5C1 BH1";
$resn_exch{"2H5C BH1"}="H5C2 BH1"; $resn_exch{"1H6C BH1"}="H6C1 BH1"; $resn_exch{"2H6C BH1"}="H6C2 BH1";
$resn_exch{"1H7C BH1"}="H7C1 BH1"; $resn_exch{"2H7C BH1"}="H7C2 BH1"; $resn_exch{"1H9C BH1"}="H9C1 BH1";
$resn_exch{"2H9C BH1"}="H9C2 BH1"; $resn_exch{"3H9C BH1"}="H9C3 BH1"; $resn_exch{"1H10 BH1"}="H101 BH1";
$resn_exch{"2H10 BH1"}="H102 BH1"; $resn_exch{"1H11 BH1"}="H111 BH1"; $resn_exch{"2H11 BH1"}="H112 BH1";
$resn_exch{"1H12 BH1"}="H121 BH1"; $resn_exch{"2H12 BH1"}="H122 BH1"; $resn_exch{"1H14 BH1"}="H141 BH1";
$resn_exch{"2H14 BH1"}="H142 BH1"; $resn_exch{"3H14 BH1"}="H143 BH1"; $resn_exch{"1H15 BH1"}="H151 BH1";
$resn_exch{"2H15 BH1"}="H152 BH1"; $resn_exch{"1H16 BH1"}="H161 BH1"; $resn_exch{"2H16 BH1"}="H162 BH1";
$resn_exch{"1H17 BH1"}="H171 BH1"; $resn_exch{"2H17 BH1"}="H172 BH1"; $resn_exch{"1H19 BH1"}="H191 BH1";
$resn_exch{"2H19 BH1"}="H192 BH1"; $resn_exch{"3H19 BH1"}="H193 BH1"; $resn_exch{"1H20 BH1"}="H201 BH1";
$resn_exch{"2H20 BH1"}="H202 BH1"; $resn_exch{"3H20 BH1"}="H203 BH1"; $resn_exch{"1H7  BH4"}=" H71 BH4";
$resn_exch{"2H7  BH4"}=" H72 BH4"; $resn_exch{"1H17 BH4"}="H171 BH4"; $resn_exch{"2H17 BH4"}="H172 BH4";
$resn_exch{"3H17 BH4"}="H173 BH4"; $resn_exch{"1H11 BH4"}="H111 BH4"; $resn_exch{"2H11 BH4"}="H112 BH4";
$resn_exch{"1H2  BH7"}="H21A BH7"; $resn_exch{"1H3  BH7"}=" H31 BH7"; $resn_exch{"1H6  BH7"}=" H61 BH7";
$resn_exch{"2H6  BH7"}=" H62 BH7"; $resn_exch{"1H8  BH7"}=" H81 BH7"; $resn_exch{"'HO2 BHA"}="HO2' BHA";
$resn_exch{"1HN4 BHA"}="HN41 BHA"; $resn_exch{"2HN4 BHA"}="HN42 BHA"; $resn_exch{"1HN4 BHB"}="HN41 BHB";
$resn_exch{"2HN4 BHB"}="HN42 BHB"; $resn_exch{"1HN2 BHB"}="HN21 BHB"; $resn_exch{"2HN2 BHB"}="HN22 BHB";
$resn_exch{"1H9  BHB"}=" H91 BHB"; $resn_exch{"2H9  BHB"}=" H92 BHB"; $resn_exch{"1H16 BHB"}="H161 BHB";
$resn_exch{"2H16 BHB"}="H162 BHB"; $resn_exch{"1H17 BHB"}="H171 BHB"; $resn_exch{"2H17 BHB"}="H172 BHB";
$resn_exch{"1H20 BHB"}="H201 BHB"; $resn_exch{"2H20 BHB"}="H202 BHB"; $resn_exch{"3H20 BHB"}="H203 BHB";
$resn_exch{"1H19 BHB"}="H191 BHB"; $resn_exch{"2H19 BHB"}="H192 BHB"; $resn_exch{"3H19 BHB"}="H193 BHB";
$resn_exch{"1HN  BHD"}=" H   BHD"; $resn_exch{"2HN  BHD"}=" H2  BHD"; $resn_exch{" HOD BHD"}=" HD2 BHD";
$resn_exch{"1H1' BHG"}="H1'1 BHG"; $resn_exch{"2H1' BHG"}="H1'2 BHG"; $resn_exch{"1H2' BHG"}="H2'1 BHG";
$resn_exch{"2H2' BHG"}="H2'2 BHG"; $resn_exch{"1H3' BHG"}="H3'1 BHG"; $resn_exch{"2H3' BHG"}="H3'2 BHG";
$resn_exch{"1H4' BHG"}="H4'1 BHG"; $resn_exch{"2H4' BHG"}="H4'2 BHG"; $resn_exch{"1H5' BHG"}="H5'1 BHG";
$resn_exch{"2H5' BHG"}="H5'2 BHG"; $resn_exch{"1H6' BHG"}="H6'1 BHG"; $resn_exch{"2H6' BHG"}="H6'2 BHG";
$resn_exch{"3H6' BHG"}="H6'3 BHG"; $resn_exch{"1H4  BHH"}=" H41 BHH"; $resn_exch{"2H4  BHH"}=" H42 BHH";
$resn_exch{"3H4  BHH"}=" H43 BHH"; $resn_exch{"1H3  BHH"}=" H31 BHH"; $resn_exch{"2H3  BHH"}=" H32 BHH";
$resn_exch{"1H2  BHH"}=" H21 BHH"; $resn_exch{"2H2  BHH"}=" H22 BHH"; $resn_exch{"1H1  BHH"}=" H11 BHH";
$resn_exch{"2H1  BHH"}=" H12 BHH"; $resn_exch{"1HE  BHH"}=" HE1 BHH"; $resn_exch{"1HH1 BHH"}="HH11 BHH";
$resn_exch{"1HH2 BHH"}="HH21 BHH"; $resn_exch{"1HH  BHH"}=" HH1 BHH"; $resn_exch{"1H10 BHI"}="H101 BHI";
$resn_exch{"2H10 BHI"}="H102 BHI"; $resn_exch{"1H11 BHI"}="H111 BHI"; $resn_exch{"2H11 BHI"}="H112 BHI";
$resn_exch{"3H11 BHI"}="H113 BHI"; $resn_exch{"1H12 BHI"}="H121 BHI"; $resn_exch{"2H12 BHI"}="H122 BHI";
$resn_exch{"1H13 BHI"}="H131 BHI"; $resn_exch{"2H13 BHI"}="H132 BHI"; $resn_exch{"1H1  BHL"}=" H11 BHL";
$resn_exch{"2H1  BHL"}=" H12 BHL"; $resn_exch{"1H2  BHL"}=" H21 BHL"; $resn_exch{"2H2  BHL"}=" H22 BHL";
$resn_exch{"1H3  BHL"}=" H31 BHL"; $resn_exch{"2H3  BHL"}=" H32 BHL"; $resn_exch{"1H4  BHL"}=" H41 BHL";
$resn_exch{"2H4  BHL"}=" H42 BHL"; $resn_exch{"1H5  BHL"}=" H51 BHL"; $resn_exch{"2H5  BHL"}=" H52 BHL";
$resn_exch{"1H6  BHL"}=" H61 BHL"; $resn_exch{"2H6  BHL"}=" H62 BHL"; $resn_exch{"1H13 BHM"}="H131 BHM";
$resn_exch{"2H13 BHM"}="H132 BHM"; $resn_exch{"1H12 BHM"}="H121 BHM"; $resn_exch{"2H12 BHM"}="H122 BHM";
$resn_exch{"3H12 BHM"}="H123 BHM"; $resn_exch{"1H9  BHP"}=" H91 BHP"; $resn_exch{"2H9  BHP"}=" H92 BHP";
$resn_exch{"1H8  BHP"}=" H81 BHP"; $resn_exch{"2H8  BHP"}=" H82 BHP"; $resn_exch{"1H7  BHP"}=" H71 BHP";
$resn_exch{"2H7  BHP"}=" H72 BHP"; $resn_exch{"1H12 BHP"}="H121 BHP"; $resn_exch{"2H12 BHP"}="H122 BHP";
$resn_exch{"1H11 BHP"}="H111 BHP"; $resn_exch{"2H11 BHP"}="H112 BHP"; $resn_exch{"1H10 BHP"}="H101 BHP";
$resn_exch{"2H10 BHP"}="H102 BHP"; $resn_exch{"1H15 BHP"}="H151 BHP"; $resn_exch{"2H15 BHP"}="H152 BHP";
$resn_exch{"1H29 BHP"}="H291 BHP"; $resn_exch{"2H29 BHP"}="H292 BHP"; $resn_exch{"1H30 BHP"}="H301 BHP";
$resn_exch{"2H30 BHP"}="H302 BHP"; $resn_exch{"1H22 BHP"}="H221 BHP"; $resn_exch{"2H22 BHP"}="H222 BHP";
$resn_exch{"1H8  BHQ"}=" H81 BHQ"; $resn_exch{"2H8  BHQ"}=" H82 BHQ"; $resn_exch{"3H8  BHQ"}=" H83 BHQ";
$resn_exch{"1H9  BHQ"}=" H91 BHQ"; $resn_exch{"2H9  BHQ"}=" H92 BHQ"; $resn_exch{"3H9  BHQ"}=" H93 BHQ";
$resn_exch{"1H10 BHQ"}="H101 BHQ"; $resn_exch{"2H10 BHQ"}="H102 BHQ"; $resn_exch{"3H10 BHQ"}="H103 BHQ";
$resn_exch{"1H12 BHQ"}="H121 BHQ"; $resn_exch{"2H12 BHQ"}="H122 BHQ"; $resn_exch{"3H12 BHQ"}="H123 BHQ";
$resn_exch{"1H13 BHQ"}="H131 BHQ"; $resn_exch{"2H13 BHQ"}="H132 BHQ"; $resn_exch{"3H13 BHQ"}="H133 BHQ";
$resn_exch{"1H14 BHQ"}="H141 BHQ"; $resn_exch{"2H14 BHQ"}="H142 BHQ"; $resn_exch{"3H14 BHQ"}="H143 BHQ";
$resn_exch{"1HN2 BHS"}="HN21 BHS"; $resn_exch{"2HN2 BHS"}="HN22 BHS"; $resn_exch{"1H7  BHS"}=" H71 BHS";
$resn_exch{"2H7  BHS"}=" H72 BHS"; $resn_exch{"1H3' BHS"}="H3'1 BHS"; $resn_exch{"2H3' BHS"}="H3'2 BHS";
$resn_exch{"3H3' BHS"}="H3'3 BHS"; $resn_exch{"1HBD BI1"}="HBD1 BI1"; $resn_exch{"2HBD BI1"}="HBD2 BI1";
$resn_exch{"3HBD BI1"}="HBD3 BI1"; $resn_exch{"1HBE BI1"}="HBE1 BI1"; $resn_exch{"2HBE BI1"}="HBE2 BI1";
$resn_exch{"3HBE BI1"}="HBE3 BI1"; $resn_exch{"1HBB BI1"}="HBB1 BI1"; $resn_exch{"2HBB BI1"}="HBB2 BI1";
$resn_exch{"1HBA BI1"}="HBA1 BI1"; $resn_exch{"2HBA BI1"}="HBA2 BI1"; $resn_exch{"1HAY BI1"}="HAY1 BI1";
$resn_exch{"2HAY BI1"}="HAY2 BI1"; $resn_exch{"1HBD BI2"}="HBD1 BI2"; $resn_exch{"2HBD BI2"}="HBD2 BI2";
$resn_exch{"3HBD BI2"}="HBD3 BI2"; $resn_exch{"1HBG BI2"}="HBG1 BI2"; $resn_exch{"2HBG BI2"}="HBG2 BI2";
$resn_exch{"1HBF BI2"}="HBF1 BI2"; $resn_exch{"2HBF BI2"}="HBF2 BI2"; $resn_exch{"1HBE BI2"}="HBE1 BI2";
$resn_exch{"2HBE BI2"}="HBE2 BI2"; $resn_exch{"1HBA BI2"}="HBA1 BI2"; $resn_exch{"2HBA BI2"}="HBA2 BI2";
$resn_exch{"1HAY BI2"}="HAY1 BI2"; $resn_exch{"2HAY BI2"}="HAY2 BI2"; $resn_exch{"1HBC BI3"}="HBC1 BI3";
$resn_exch{"2HBC BI3"}="HBC2 BI3"; $resn_exch{"1HBB BI3"}="HBB1 BI3"; $resn_exch{"2HBB BI3"}="HBB2 BI3";
$resn_exch{"1HBA BI3"}="HBA1 BI3"; $resn_exch{"2HBA BI3"}="HBA2 BI3"; $resn_exch{"1HAY BI3"}="HAY1 BI3";
$resn_exch{"2HAY BI3"}="HAY2 BI3"; $resn_exch{"1H24 BI4"}="H241 BI4"; $resn_exch{"2H24 BI4"}="H242 BI4";
$resn_exch{"1H25 BI4"}="H251 BI4"; $resn_exch{"2H25 BI4"}="H252 BI4"; $resn_exch{"1H27 BI4"}="H271 BI4";
$resn_exch{"2H27 BI4"}="H272 BI4"; $resn_exch{"1H28 BI4"}="H281 BI4"; $resn_exch{"2H28 BI4"}="H282 BI4";
$resn_exch{"1H29 BI4"}="H291 BI4"; $resn_exch{"2H29 BI4"}="H292 BI4"; $resn_exch{"1H31 BI4"}="H311 BI4";
$resn_exch{"2H31 BI4"}="H312 BI4"; $resn_exch{"3H31 BI4"}="H313 BI4"; $resn_exch{"1H6  BI5"}=" H61 BI5";
$resn_exch{"2H6  BI5"}=" H62 BI5"; $resn_exch{"1HB0 BI8"}="HB01 BI8"; $resn_exch{"2HB0 BI8"}="HB02 BI8";
$resn_exch{"3HB0 BI8"}="HB03 BI8"; $resn_exch{"1HBC BI8"}="HBC1 BI8"; $resn_exch{"2HBC BI8"}="HBC2 BI8";
$resn_exch{"1HBB BI8"}="HBB1 BI8"; $resn_exch{"2HBB BI8"}="HBB2 BI8"; $resn_exch{"1HBA BI8"}="HBA1 BI8";
$resn_exch{"2HBA BI8"}="HBA2 BI8"; $resn_exch{"1HAY BI8"}="HAY1 BI8"; $resn_exch{"2HAY BI8"}="HAY2 BI8";
$resn_exch{"1H12 BIA"}="H121 BIA"; $resn_exch{"2H12 BIA"}="H122 BIA"; $resn_exch{"1H16 BIA"}="H161 BIA";
$resn_exch{"2H16 BIA"}="H162 BIA"; $resn_exch{"1H21 BIA"}="H211 BIA"; $resn_exch{"2H21 BIA"}="H212 BIA";
$resn_exch{"1H22 BIA"}="H221 BIA"; $resn_exch{"2H22 BIA"}="H222 BIA"; $resn_exch{"1H25 BIA"}="H251 BIA";
$resn_exch{"2H25 BIA"}="H252 BIA"; $resn_exch{"1H27 BIA"}="H271 BIA"; $resn_exch{"2H27 BIA"}="H272 BIA";
$resn_exch{"1H2  BIB"}=" H21 BIB"; $resn_exch{"2H2  BIB"}=" H22 BIB"; $resn_exch{"3H2  BIB"}=" H23 BIB";
$resn_exch{"1H4  BIB"}=" H41 BIB"; $resn_exch{"2H4  BIB"}=" H42 BIB"; $resn_exch{"1HN1 BIB"}="HN11 BIB";
$resn_exch{"2HN1 BIB"}="HN12 BIB"; $resn_exch{"1H2  BIC"}=" H21 BIC"; $resn_exch{"2H2  BIC"}=" H22 BIC";
$resn_exch{"1H3  BIC"}=" H31 BIC"; $resn_exch{"2H3  BIC"}=" H32 BIC"; $resn_exch{"1H4  BIC"}=" H41 BIC";
$resn_exch{"2H4  BIC"}=" H42 BIC"; $resn_exch{"1H5  BIC"}=" H51 BIC"; $resn_exch{"2H5  BIC"}=" H52 BIC";
$resn_exch{"1HN3 BIC"}="HN31 BIC"; $resn_exch{"2HN3 BIC"}="HN32 BIC"; $resn_exch{"1H9  BIC"}=" H91 BIC";
$resn_exch{"2H9  BIC"}=" H92 BIC"; $resn_exch{"1H17 BID"}="H171 BID"; $resn_exch{"2H17 BID"}="H172 BID";
$resn_exch{"3H17 BID"}="H173 BID"; $resn_exch{"1H14 BID"}="H141 BID"; $resn_exch{"2H14 BID"}="H142 BID";
$resn_exch{"1H12 BID"}="H121 BID"; $resn_exch{"2H12 BID"}="H122 BID"; $resn_exch{"1H10 BID"}="H101 BID";
$resn_exch{"2H10 BID"}="H102 BID"; $resn_exch{"3H10 BID"}="H103 BID"; $resn_exch{"1H8  BID"}=" H81 BID";
$resn_exch{"2H8  BID"}=" H82 BID"; $resn_exch{"1H7  BID"}=" H71 BID"; $resn_exch{"2H7  BID"}=" H72 BID";
$resn_exch{"1H5  BID"}=" H51 BID"; $resn_exch{"2H5  BID"}=" H52 BID"; $resn_exch{"1H1  BID"}="H11A BID";
$resn_exch{"2H1  BID"}=" H12 BID"; $resn_exch{"3H1  BID"}=" H13 BID"; $resn_exch{"1H19 BID"}="H191 BID";
$resn_exch{"2H19 BID"}="H192 BID"; $resn_exch{"1H20 BID"}="H201 BID"; $resn_exch{"2H20 BID"}="H202 BID";
$resn_exch{"1H21 BID"}="H211 BID"; $resn_exch{"2H21 BID"}="H212 BID"; $resn_exch{"1H24 BID"}="H241 BID";
$resn_exch{"2H24 BID"}="H242 BID"; $resn_exch{"3H24 BID"}="H243 BID"; $resn_exch{"1H25 BID"}="H251 BID";
$resn_exch{"2H25 BID"}="H252 BID"; $resn_exch{"1H26 BID"}="H261 BID"; $resn_exch{"2H26 BID"}="H262 BID";
$resn_exch{"1H28 BID"}="H281 BID"; $resn_exch{"2H28 BID"}="H282 BID"; $resn_exch{"1H29 BID"}="H291 BID";
$resn_exch{"2H29 BID"}="H292 BID"; $resn_exch{"1H30 BID"}="H301 BID"; $resn_exch{"2H30 BID"}="H302 BID";
$resn_exch{"1H32 BID"}="H321 BID"; $resn_exch{"2H32 BID"}="H322 BID"; $resn_exch{"1H33 BID"}="H331 BID";
$resn_exch{"2H33 BID"}="H332 BID"; $resn_exch{"1H35 BID"}="H351 BID"; $resn_exch{"2H35 BID"}="H352 BID";
$resn_exch{"3H35 BID"}="H353 BID"; $resn_exch{"1H38 BID"}="H381 BID"; $resn_exch{"2H38 BID"}="H382 BID";
$resn_exch{"3H38 BID"}="H383 BID"; $resn_exch{"1H40 BID"}="H401 BID"; $resn_exch{"2H40 BID"}="H402 BID";
$resn_exch{"3H40 BID"}="H403 BID"; $resn_exch{"1HN  BIF"}=" H   BIF"; $resn_exch{"2HN  BIF"}=" H2  BIF";
$resn_exch{"1HB  BIF"}=" HB2 BIF"; $resn_exch{"2HB  BIF"}=" HB3 BIF"; $resn_exch{"1H27 BIM"}="H271 BIM";
$resn_exch{"2H27 BIM"}="H272 BIM"; $resn_exch{"1H35 BIM"}="H351 BIM"; $resn_exch{"2H35 BIM"}="H352 BIM";
$resn_exch{"1H36 BIM"}="H361 BIM"; $resn_exch{"2H36 BIM"}="H362 BIM"; $resn_exch{"1H37 BIM"}="H371 BIM";
$resn_exch{"2H37 BIM"}="H372 BIM"; $resn_exch{"1H38 BIM"}="H381 BIM"; $resn_exch{"2H38 BIM"}="H382 BIM";
$resn_exch{"1H40 BIM"}="H401 BIM"; $resn_exch{"2H40 BIM"}="H402 BIM"; $resn_exch{"3H40 BIM"}="H403 BIM";
$resn_exch{"1H26 BIM"}="H261 BIM"; $resn_exch{"2H26 BIM"}="H262 BIM"; $resn_exch{"1H28 BIM"}="H281 BIM";
$resn_exch{"2H28 BIM"}="H282 BIM"; $resn_exch{"1H41 BIM"}="H411 BIM"; $resn_exch{"2H41 BIM"}="H412 BIM";
$resn_exch{"3H41 BIM"}="H413 BIM"; $resn_exch{"1H15 BIN"}="H151 BIN"; $resn_exch{"2H15 BIN"}="H152 BIN";
$resn_exch{"3H15 BIN"}="H153 BIN"; $resn_exch{"1H16 BIN"}="H161 BIN"; $resn_exch{"2H16 BIN"}="H162 BIN";
$resn_exch{"3H16 BIN"}="H163 BIN"; $resn_exch{"1H17 BIN"}="H171 BIN"; $resn_exch{"2H17 BIN"}="H172 BIN";
$resn_exch{"1H20 BIN"}="H201 BIN"; $resn_exch{"2H20 BIN"}="H202 BIN"; $resn_exch{"3H20 BIN"}="H203 BIN";
$resn_exch{"1H21 BIN"}="H211 BIN"; $resn_exch{"2H21 BIN"}="H212 BIN"; $resn_exch{"3H21 BIN"}="H213 BIN";
$resn_exch{"3HOB BIN"}="HOB3 BIN"; $resn_exch{"4HOB BIN"}="HOB4 BIN"; $resn_exch{"1HN2 BIO"}="HN21 BIO";
$resn_exch{"2HN2 BIO"}="HN22 BIO"; $resn_exch{"1H11 BIO"}="H111 BIO"; $resn_exch{"2H11 BIO"}="H112 BIO";
$resn_exch{"3H11 BIO"}="H113 BIO"; $resn_exch{"1H3  BIP"}=" H31 BIP"; $resn_exch{"2H3  BIP"}=" H32 BIP";
$resn_exch{"1H   BIP"}=" H1  BIP"; $resn_exch{"2H   BIP"}=" H2A BIP"; $resn_exch{"1H3  BIQ"}=" H31 BIQ";
$resn_exch{"2H3  BIQ"}=" H32 BIQ"; $resn_exch{"1H12 BIQ"}="H121 BIQ"; $resn_exch{"2H12 BIQ"}="H122 BIQ";
$resn_exch{"1H14 BIQ"}="H141 BIQ"; $resn_exch{"2H14 BIQ"}="H142 BIQ"; $resn_exch{"1H23 BIQ"}="H231 BIQ";
$resn_exch{"2H23 BIQ"}="H232 BIQ"; $resn_exch{"1H24 BIQ"}="H241 BIQ"; $resn_exch{"2H24 BIQ"}="H242 BIQ";
$resn_exch{"1H25 BIQ"}="H251 BIQ"; $resn_exch{"2H25 BIQ"}="H252 BIQ"; $resn_exch{"3H25 BIQ"}="H253 BIQ";
$resn_exch{"1H26 BIQ"}="H261 BIQ"; $resn_exch{"2H26 BIQ"}="H262 BIQ"; $resn_exch{"1H41 BIQ"}="H411 BIQ";
$resn_exch{"2H41 BIQ"}="H412 BIQ"; $resn_exch{"1H3  BIR"}=" H31 BIR"; $resn_exch{"2H3  BIR"}=" H32 BIR";
$resn_exch{"3H3  BIR"}=" H33 BIR"; $resn_exch{"1HN1 BIR"}="HN11 BIR"; $resn_exch{"2HN1 BIR"}="HN12 BIR";
$resn_exch{"1H7  BIR"}=" H71 BIR"; $resn_exch{"2H7  BIR"}=" H72 BIR"; $resn_exch{"1H26 BIR"}="H261 BIR";
$resn_exch{"2H26 BIR"}="H262 BIR"; $resn_exch{"3H26 BIR"}="H263 BIR"; $resn_exch{"1H9  BIR"}=" H91 BIR";
$resn_exch{"2H9  BIR"}=" H92 BIR"; $resn_exch{"BHO2 BIS"}="HO2B BIS"; $resn_exch{"AHO1 BIS"}="HO1A BIS";
$resn_exch{"1H5* BIS"}="H5'1 BIS"; $resn_exch{"2H5* BIS"}="H5'2 BIS"; $resn_exch{" H4* BIS"}=" H4' BIS";
$resn_exch{" H3* BIS"}=" H3' BIS"; $resn_exch{"3HO* BIS"}="HO'3 BIS"; $resn_exch{" H2* BIS"}=" H2' BIS";
$resn_exch{"2HO* BIS"}="HO'2 BIS"; $resn_exch{" H1* BIS"}=" H1' BIS"; $resn_exch{"1HN6 BIS"}="HN61 BIS";
$resn_exch{"2HN6 BIS"}="HN62 BIS"; $resn_exch{"1H12 BIS"}="H121 BIS"; $resn_exch{"2H12 BIS"}="H122 BIS";
$resn_exch{"1H13 BIS"}="H131 BIS"; $resn_exch{"2H13 BIS"}="H132 BIS"; $resn_exch{"1H14 BIS"}="H141 BIS";
$resn_exch{"2H14 BIS"}="H142 BIS"; $resn_exch{"1H18 BIT"}="H181 BIT"; $resn_exch{"2H18 BIT"}="H182 BIT";
$resn_exch{"3H18 BIT"}="H183 BIT"; $resn_exch{"1H2  BIT"}=" H21 BIT"; $resn_exch{"2H2  BIT"}=" H22 BIT";
$resn_exch{"1H1  BIT"}=" H11 BIT"; $resn_exch{"2H1  BIT"}=" H12 BIT"; $resn_exch{"1H2  BIV"}=" H21 BIV";
$resn_exch{"2H2  BIV"}=" H22 BIV"; $resn_exch{"1H4  BIV"}=" H41 BIV"; $resn_exch{"2H4  BIV"}=" H42 BIV";
$resn_exch{"1H9  BIV"}=" H91 BIV"; $resn_exch{"2H9  BIV"}=" H92 BIV"; $resn_exch{"1H11 BIV"}="H111 BIV";
$resn_exch{"2H11 BIV"}="H112 BIV"; $resn_exch{"1H12 BIV"}="H121 BIV"; $resn_exch{"2H12 BIV"}="H122 BIV";
$resn_exch{"1H15 BIV"}="H151 BIV"; $resn_exch{"2H15 BIV"}="H152 BIV"; $resn_exch{"1H16 BIV"}="H161 BIV";
$resn_exch{"2H16 BIV"}="H162 BIV"; $resn_exch{"1H18 BIV"}="H181 BIV"; $resn_exch{"2H18 BIV"}="H182 BIV";
$resn_exch{"3H18 BIV"}="H183 BIV"; $resn_exch{"1H19 BIV"}="H191 BIV"; $resn_exch{"2H19 BIV"}="H192 BIV";
$resn_exch{"1H22 BIV"}="H221 BIV"; $resn_exch{"2H22 BIV"}="H222 BIV"; $resn_exch{"1H23 BIV"}="H231 BIV";
$resn_exch{"2H23 BIV"}="H232 BIV"; $resn_exch{"1H24 BIV"}="H241 BIV"; $resn_exch{"2H24 BIV"}="H242 BIV";
$resn_exch{"1H26 BIV"}="H261 BIV"; $resn_exch{"2H26 BIV"}="H262 BIV"; $resn_exch{"3H26 BIV"}="H263 BIV";
$resn_exch{"1H27 BIV"}="H271 BIV"; $resn_exch{"2H27 BIV"}="H272 BIV"; $resn_exch{"3H27 BIV"}="H273 BIV";
$resn_exch{"1H21 BIV"}="H211 BIV"; $resn_exch{"2H21 BIV"}="H212 BIV"; $resn_exch{"1H28 BIV"}="H281 BIV";
$resn_exch{"2H28 BIV"}="H282 BIV"; $resn_exch{"1H29 BIV"}="H291 BIV"; $resn_exch{"2H29 BIV"}="H292 BIV";
$resn_exch{"1H32 BIV"}="H321 BIV"; $resn_exch{"2H32 BIV"}="H322 BIV"; $resn_exch{"3H32 BIV"}="H323 BIV";
$resn_exch{"1H33 BIV"}="H331 BIV"; $resn_exch{"2H33 BIV"}="H332 BIV"; $resn_exch{"3H33 BIV"}="H333 BIV";
$resn_exch{"1H14 BIZ"}="H141 BIZ"; $resn_exch{"2H14 BIZ"}="H142 BIZ"; $resn_exch{"1H18 BIZ"}="H181 BIZ";
$resn_exch{"2H18 BIZ"}="H182 BIZ"; $resn_exch{"3H18 BIZ"}="H183 BIZ"; $resn_exch{"1H26 BIZ"}="H261 BIZ";
$resn_exch{"2H26 BIZ"}="H262 BIZ"; $resn_exch{"3H26 BIZ"}="H263 BIZ"; $resn_exch{"*HN1 BIZ"}="HN1' BIZ";
$resn_exch{" *H3 BIZ"}=" H3' BIZ"; $resn_exch{" *H6 BIZ"}=" H6' BIZ"; $resn_exch{" *H7 BIZ"}=" H7' BIZ";
$resn_exch{"*HN8 BIZ"}="HN8' BIZ"; $resn_exch{"*H10 BIZ"}="H10' BIZ"; $resn_exch{"1H4* BIZ"}="H4'1 BIZ";
$resn_exch{"2H4* BIZ"}="H4'2 BIZ"; $resn_exch{"*H15 BIZ"}="H15' BIZ"; $resn_exch{"1H8* BIZ"}="H8'1 BIZ";
$resn_exch{"2H8* BIZ"}="H8'2 BIZ"; $resn_exch{"3H8* BIZ"}="H8'3 BIZ"; $resn_exch{"*H19 BIZ"}="H19' BIZ";
$resn_exch{"*HO2 BIZ"}="HO2' BIZ"; $resn_exch{"*H25 BIZ"}="H25' BIZ"; $resn_exch{"1H6* BIZ"}="H6'1 BIZ";
$resn_exch{"2H6* BIZ"}="H6'2 BIZ"; $resn_exch{"3H6* BIZ"}="H6'3 BIZ"; $resn_exch{"*HN7 BIZ"}="HN7' BIZ";
$resn_exch{"1H2  BJH"}=" H21 BJH"; $resn_exch{"2H2  BJH"}=" H22 BJH"; $resn_exch{"1H4  BJH"}=" H41 BJH";
$resn_exch{"2H4  BJH"}=" H42 BJH"; $resn_exch{"3H4  BJH"}=" H43 BJH"; $resn_exch{"1HOB BJH"}="HOB1 BJH";
$resn_exch{"2HOB BJH"}="HOB2 BJH"; $resn_exch{"1H2  BJI"}=" H21 BJI"; $resn_exch{"2H2  BJI"}=" H22 BJI";
$resn_exch{"1H4  BJI"}=" H41 BJI"; $resn_exch{"2H4  BJI"}=" H42 BJI"; $resn_exch{"3H4  BJI"}=" H43 BJI";
$resn_exch{"1HOB BJI"}="HOB1 BJI"; $resn_exch{"2HOB BJI"}="HOB2 BJI"; $resn_exch{"1H2  BJP"}=" H21 BJP";
$resn_exch{"2H2  BJP"}=" H22 BJP"; $resn_exch{"1HOB BJP"}="HOB1 BJP"; $resn_exch{"2HOB BJP"}="HOB2 BJP";
$resn_exch{"1H20 BJP"}="H201 BJP"; $resn_exch{"1H21 BJP"}="H211 BJP"; $resn_exch{"1H22 BJP"}="H221 BJP";
$resn_exch{"1H23 BJP"}="H231 BJP"; $resn_exch{"1H24 BJP"}="H241 BJP"; $resn_exch{"1H25 BJP"}="H251 BJP";
$resn_exch{"2H25 BJP"}="H252 BJP"; $resn_exch{"1H11 BL5"}="H111 BL5"; $resn_exch{"2H11 BL5"}="H112 BL5";
$resn_exch{"1H12 BL5"}="H121 BL5"; $resn_exch{"2H12 BL5"}="H122 BL5"; $resn_exch{"1H13 BL5"}="H131 BL5";
$resn_exch{"2H13 BL5"}="H132 BL5"; $resn_exch{"1H14 BL5"}="H141 BL5"; $resn_exch{"2H14 BL5"}="H142 BL5";
$resn_exch{"1H15 BL5"}="H151 BL5"; $resn_exch{"2H15 BL5"}="H152 BL5"; $resn_exch{"1H20 BL5"}="H201 BL5";
$resn_exch{"2H20 BL5"}="H202 BL5"; $resn_exch{"1H21 BL5"}="H211 BL5"; $resn_exch{"2H21 BL5"}="H212 BL5";
$resn_exch{"1H22 BL5"}="H221 BL5"; $resn_exch{"2H22 BL5"}="H222 BL5"; $resn_exch{"1H23 BL5"}="H231 BL5";
$resn_exch{"2H23 BL5"}="H232 BL5"; $resn_exch{"3H23 BL5"}="H233 BL5"; $resn_exch{"1H24 BL5"}="H241 BL5";
$resn_exch{"2H24 BL5"}="H242 BL5"; $resn_exch{"1H27 BL5"}="H271 BL5"; $resn_exch{"2H27 BL5"}="H272 BL5";
$resn_exch{"1H31 BL5"}="H311 BL5"; $resn_exch{"2H31 BL5"}="H312 BL5"; $resn_exch{"1H30 BL5"}="H301 BL5";
$resn_exch{"2H30 BL5"}="H302 BL5"; $resn_exch{"1H29 BL5"}="H291 BL5"; $resn_exch{"2H29 BL5"}="H292 BL5";
$resn_exch{"1H28 BL5"}="H281 BL5"; $resn_exch{"2H28 BL5"}="H282 BL5"; $resn_exch{"3H28 BL5"}="H283 BL5";
$resn_exch{"1H40 BL5"}="H401 BL5"; $resn_exch{"2H40 BL5"}="H402 BL5"; $resn_exch{"1H39 BL5"}="H391 BL5";
$resn_exch{"2H39 BL5"}="H392 BL5"; $resn_exch{"1H38 BL5"}="H381 BL5"; $resn_exch{"2H38 BL5"}="H382 BL5";
$resn_exch{"1H37 BL5"}="H371 BL5"; $resn_exch{"2H37 BL5"}="H372 BL5"; $resn_exch{"1H36 BL5"}="H361 BL5";
$resn_exch{"2H36 BL5"}="H362 BL5"; $resn_exch{" H_A BLA"}=" HA  BLA"; $resn_exch{"1HMA BLA"}="HMA1 BLA";
$resn_exch{"2HMA BLA"}="HMA2 BLA"; $resn_exch{"3HMA BLA"}="HMA3 BLA"; $resn_exch{"1HAA BLA"}="HAA1 BLA";
$resn_exch{"2HAA BLA"}="HAA2 BLA"; $resn_exch{"1HBA BLA"}="HBA1 BLA"; $resn_exch{"2HBA BLA"}="HBA2 BLA";
$resn_exch{" H_B BLA"}=" HB  BLA"; $resn_exch{"1HMB BLA"}="HMB1 BLA"; $resn_exch{"2HMB BLA"}="HMB2 BLA";
$resn_exch{"3HMB BLA"}="HMB3 BLA"; $resn_exch{"1HBB BLA"}="HBB1 BLA"; $resn_exch{"2HBB BLA"}="HBB2 BLA";
$resn_exch{" H_C BLA"}=" HC  BLA"; $resn_exch{"1HMC BLA"}="HMC1 BLA"; $resn_exch{"2HMC BLA"}="HMC2 BLA";
$resn_exch{"3HMC BLA"}="HMC3 BLA"; $resn_exch{"1HBC BLA"}="HBC1 BLA"; $resn_exch{"2HBC BLA"}="HBC2 BLA";
$resn_exch{"1HMD BLA"}="HMD1 BLA"; $resn_exch{"2HMD BLA"}="HMD2 BLA"; $resn_exch{"3HMD BLA"}="HMD3 BLA";
$resn_exch{"1HAD BLA"}="HAD1 BLA"; $resn_exch{"2HAD BLA"}="HAD2 BLA"; $resn_exch{"1HBD BLA"}="HBD1 BLA";
$resn_exch{"2HBD BLA"}="HBD2 BLA"; $resn_exch{"1H50 BLB"}="H501 BLB"; $resn_exch{"2H50 BLB"}="H502 BLB";
$resn_exch{"1H51 BLB"}="H511 BLB"; $resn_exch{"2H51 BLB"}="H512 BLB"; $resn_exch{"1H52 BLB"}="H521 BLB";
$resn_exch{"2H52 BLB"}="H522 BLB"; $resn_exch{"1H53 BLB"}="H531 BLB"; $resn_exch{"2H53 BLB"}="H532 BLB";
$resn_exch{"1HNS BLB"}="HNS1 BLB"; $resn_exch{"2HNS BLB"}="HNS2 BLB"; $resn_exch{"1HNT BLB"}="HNT1 BLB";
$resn_exch{"2HNT BLB"}="HNT2 BLB"; $resn_exch{"1H61 BLB"}="H611 BLB"; $resn_exch{"2H61 BLB"}="H612 BLB";
$resn_exch{"1HNQ BLB"}="HNQ1 BLB"; $resn_exch{"2HNQ BLB"}="HNQ2 BLB"; $resn_exch{"1H66 BLB"}="H661 BLB";
$resn_exch{"2H66 BLB"}="H662 BLB"; $resn_exch{"1HB  BLE"}=" HB2 BLE"; $resn_exch{"2HB  BLE"}=" HB3 BLE";
$resn_exch{"1HD1 BLE"}="HD11 BLE"; $resn_exch{"2HD1 BLE"}="HD12 BLE"; $resn_exch{"3HD1 BLE"}="HD13 BLE";
$resn_exch{"1HD2 BLE"}="HD21 BLE"; $resn_exch{"2HD2 BLE"}="HD22 BLE"; $resn_exch{"3HD2 BLE"}="HD23 BLE";
$resn_exch{"1HN  BLE"}=" H   BLE"; $resn_exch{"2HN  BLE"}=" HN2 BLE"; $resn_exch{"1H8  BLG"}=" H81 BLG";
$resn_exch{"2H8  BLG"}=" H82 BLG"; $resn_exch{"3H8  BLG"}=" H83 BLG"; $resn_exch{"3HO4 BLG"}="HO43 BLG";
$resn_exch{"1H6  BLG"}=" H61 BLG"; $resn_exch{"2H6  BLG"}=" H62 BLG"; $resn_exch{"1HG  BLG"}=" HG1 BLG";
$resn_exch{"2HG  BLG"}=" HG2 BLG"; $resn_exch{"1HN  BLG"}=" HN1 BLG"; $resn_exch{"2HN  BLG"}="HN2A BLG";
$resn_exch{"1H9  BLG"}=" H91 BLG"; $resn_exch{"2H9  BLG"}=" H92 BLG"; $resn_exch{"1H11 BLG"}="H111 BLG";
$resn_exch{"2H11 BLG"}="H112 BLG"; $resn_exch{"1H12 BLG"}="H121 BLG"; $resn_exch{"2H12 BLG"}="H122 BLG";
$resn_exch{"3HOS BLG"}="HOS3 BLG"; $resn_exch{"1H1  BLI"}=" H11 BLI"; $resn_exch{"1H2  BLI"}=" H21 BLI";
$resn_exch{"2H2  BLI"}=" H22 BLI"; $resn_exch{"1H4  BLI"}=" H41 BLI"; $resn_exch{"2H4  BLI"}=" H42 BLI";
$resn_exch{"1H01 BLI"}="H011 BLI"; $resn_exch{"1H10 BLI"}="H101 BLI"; $resn_exch{"1H03 BLI"}="H031 BLI";
$resn_exch{"2H03 BLI"}="H032 BLI"; $resn_exch{"1H06 BLI"}="H061 BLI"; $resn_exch{"2H06 BLI"}="H062 BLI";
$resn_exch{"1H02 BLI"}="H021 BLI"; $resn_exch{"2H02 BLI"}="H022 BLI"; $resn_exch{"1H1' BLI"}="H1'1 BLI";
$resn_exch{"2H1' BLI"}="H1'2 BLI"; $resn_exch{"1H2' BLI"}="H2'1 BLI"; $resn_exch{"1H3' BLI"}="H3'1 BLI";
$resn_exch{"2H3' BLI"}="H3'2 BLI"; $resn_exch{"1H7' BLI"}="H7'1 BLI"; $resn_exch{"2H7' BLI"}="H7'2 BLI";
$resn_exch{"1H4' BLI"}="H4'1 BLI"; $resn_exch{"2H4' BLI"}="H4'2 BLI"; $resn_exch{"1H5' BLI"}="H5'1 BLI";
$resn_exch{"2H5' BLI"}="H5'2 BLI"; $resn_exch{"1HN2 BLI"}="HN21 BLI"; $resn_exch{"2HN2 BLI"}="HN22 BLI";
$resn_exch{"1HO3 BLI"}="HO31 BLI"; $resn_exch{"1H1* BLI"}="H1A1 BLI"; $resn_exch{"1H2* BLI"}="H2A1 BLI";
$resn_exch{"1H3* BLI"}="H3A1 BLI"; $resn_exch{"1H4* BLI"}="H4A1 BLI"; $resn_exch{"1H5* BLI"}="H5A1 BLI";
$resn_exch{"1H13 BLL"}="H131 BLL"; $resn_exch{"2H13 BLL"}="H132 BLL"; $resn_exch{"1H14 BLL"}="H141 BLL";
$resn_exch{"2H14 BLL"}="H142 BLL"; $resn_exch{"1H22 BLL"}="H221 BLL"; $resn_exch{"2H22 BLL"}="H222 BLL";
$resn_exch{"1H23 BLL"}="H231 BLL"; $resn_exch{"2H23 BLL"}="H232 BLL"; $resn_exch{"1H38 BLL"}="H381 BLL";
$resn_exch{"2H38 BLL"}="H382 BLL"; $resn_exch{"1H48 BLL"}="H481 BLL"; $resn_exch{"2H48 BLL"}="H482 BLL";
$resn_exch{"1H50 BLM"}="H501 BLM"; $resn_exch{"2H50 BLM"}="H502 BLM"; $resn_exch{"1H51 BLM"}="H511 BLM";
$resn_exch{"2H51 BLM"}="H512 BLM"; $resn_exch{"1H52 BLM"}="H521 BLM"; $resn_exch{"2H52 BLM"}="H522 BLM";
$resn_exch{"1H55 BLM"}="H551 BLM"; $resn_exch{"2H55 BLM"}="H552 BLM"; $resn_exch{"3H55 BLM"}="H553 BLM";
$resn_exch{"1H54 BLM"}="H541 BLM"; $resn_exch{"2H54 BLM"}="H542 BLM"; $resn_exch{"3H54 BLM"}="H543 BLM";
$resn_exch{"1H61 BLM"}="H611 BLM"; $resn_exch{"2H61 BLM"}="H612 BLM"; $resn_exch{"1HNQ BLM"}="HNQ1 BLM";
$resn_exch{"2HNQ BLM"}="HNQ2 BLM"; $resn_exch{"1H66 BLM"}="H661 BLM"; $resn_exch{"2H66 BLM"}="H662 BLM";
$resn_exch{"1HC2 BLN"}="HC21 BLN"; $resn_exch{"2HC2 BLN"}="HC22 BLN"; $resn_exch{"1HC3 BLN"}="HC31 BLN";
$resn_exch{"2HC3 BLN"}="HC32 BLN"; $resn_exch{"1HC5 BLN"}="HC51 BLN"; $resn_exch{"2HC5 BLN"}="HC52 BLN";
$resn_exch{"1HC6 BLN"}="HC61 BLN"; $resn_exch{"2HC6 BLN"}="HC62 BLN"; $resn_exch{"1HCB BLN"}="HCB1 BLN";
$resn_exch{"2HCB BLN"}="HCB2 BLN"; $resn_exch{"1HD1 BLN"}="HD11 BLN"; $resn_exch{"2HD1 BLN"}="HD12 BLN";
$resn_exch{"3HD1 BLN"}="HD13 BLN"; $resn_exch{"1HD2 BLN"}="HD21 BLN"; $resn_exch{"2HD2 BLN"}="HD22 BLN";
$resn_exch{"3HD2 BLN"}="HD23 BLN"; $resn_exch{"1HB' BLN"}="HB'1 BLN"; $resn_exch{"2HB' BLN"}="HB'2 BLN";
$resn_exch{"1HCD BLN"}="HCD1 BLN"; $resn_exch{"2HCD BLN"}="HCD2 BLN"; $resn_exch{"CH2' BLN"}="H2'C BLN";
$resn_exch{"CH3' BLN"}="H3'C BLN"; $resn_exch{"CH4' BLN"}="H4'C BLN"; $resn_exch{"CH5' BLN"}="H5'C BLN";
$resn_exch{"CH6' BLN"}="H6'C BLN"; $resn_exch{"CHA' BLN"}="HA'C BLN"; $resn_exch{"1H8  BLO"}=" H81 BLO";
$resn_exch{"2H8  BLO"}=" H82 BLO"; $resn_exch{"1H10 BLO"}="H101 BLO"; $resn_exch{"2H10 BLO"}="H102 BLO";
$resn_exch{"1H11 BLO"}="H111 BLO"; $resn_exch{"2H11 BLO"}="H112 BLO"; $resn_exch{"1H14 BLO"}="H141 BLO";
$resn_exch{"2H14 BLO"}="H142 BLO"; $resn_exch{"1H13 BLO"}="H131 BLO"; $resn_exch{"2H13 BLO"}="H132 BLO";
$resn_exch{"3H13 BLO"}="H133 BLO"; $resn_exch{"1HN9 BLO"}="HN91 BLO"; $resn_exch{"2HN9 BLO"}="HN92 BLO";
$resn_exch{"1H9  BLP"}=" H91 BLP"; $resn_exch{"2H9  BLP"}=" H92 BLP"; $resn_exch{"1H7  BLP"}=" H71 BLP";
$resn_exch{"2H7  BLP"}=" H72 BLP"; $resn_exch{"1H1  BLP"}=" H11 BLP"; $resn_exch{"2H1  BLP"}="H12A BLP";
$resn_exch{"3H1  BLP"}="H13A BLP"; $resn_exch{"1H5  BLP"}=" H51 BLP"; $resn_exch{"2H5  BLP"}=" H52 BLP";
$resn_exch{"1HN4 BLS"}="HN41 BLS"; $resn_exch{"2HN4 BLS"}="HN42 BLS"; $resn_exch{" H1* BLS"}=" H1' BLS";
$resn_exch{" H2* BLS"}=" H2' BLS"; $resn_exch{" H3* BLS"}=" H3' BLS"; $resn_exch{" H4* BLS"}=" H4' BLS";
$resn_exch{" H5* BLS"}=" H5' BLS"; $resn_exch{"1H8  BLS"}=" H81 BLS"; $resn_exch{"2H8  BLS"}=" H82 BLS";
$resn_exch{"1HN9 BLS"}="HN91 BLS"; $resn_exch{"2HN9 BLS"}="HN92 BLS"; $resn_exch{"1H10 BLS"}="H101 BLS";
$resn_exch{"2H10 BLS"}="H102 BLS"; $resn_exch{"1H11 BLS"}="H111 BLS"; $resn_exch{"2H11 BLS"}="H112 BLS";
$resn_exch{"1H13 BLS"}="H131 BLS"; $resn_exch{"2H13 BLS"}="H132 BLS"; $resn_exch{"3H13 BLS"}="H133 BLS";
$resn_exch{"1H15 BLS"}="H151 BLS"; $resn_exch{"2H15 BLS"}="H152 BLS"; $resn_exch{"1H3  BLT"}=" H31 BLT";
$resn_exch{"2H3  BLT"}=" H32 BLT"; $resn_exch{"1H6  BLT"}=" H61 BLT"; $resn_exch{"2H6  BLT"}=" H62 BLT";
$resn_exch{"1H7  BLT"}=" H71 BLT"; $resn_exch{"2H7  BLT"}=" H72 BLT"; $resn_exch{"1H10 BLT"}="H101 BLT";
$resn_exch{"2H10 BLT"}="H102 BLT"; $resn_exch{" H_A BLV"}=" HA  BLV"; $resn_exch{"1HMA BLV"}="HMA1 BLV";
$resn_exch{"2HMA BLV"}="HMA2 BLV"; $resn_exch{"3HMA BLV"}="HMA3 BLV"; $resn_exch{"1HAA BLV"}="HAA1 BLV";
$resn_exch{"2HAA BLV"}="HAA2 BLV"; $resn_exch{"1HBA BLV"}="HBA1 BLV"; $resn_exch{"2HBA BLV"}="HBA2 BLV";
$resn_exch{" H_B BLV"}=" HB  BLV"; $resn_exch{"1HMB BLV"}="HMB1 BLV"; $resn_exch{"2HMB BLV"}="HMB2 BLV";
$resn_exch{"3HMB BLV"}="HMB3 BLV"; $resn_exch{"1HBB BLV"}="HBB1 BLV"; $resn_exch{"2HBB BLV"}="HBB2 BLV";
$resn_exch{" H_C BLV"}=" HC  BLV"; $resn_exch{"1HMC BLV"}="HMC1 BLV"; $resn_exch{"2HMC BLV"}="HMC2 BLV";
$resn_exch{"3HMC BLV"}="HMC3 BLV"; $resn_exch{"1HBC BLV"}="HBC1 BLV"; $resn_exch{"2HBC BLV"}="HBC2 BLV";
$resn_exch{"1HMD BLV"}="HMD1 BLV"; $resn_exch{"2HMD BLV"}="HMD2 BLV"; $resn_exch{"3HMD BLV"}="HMD3 BLV";
$resn_exch{"1HAD BLV"}="HAD1 BLV"; $resn_exch{"2HAD BLV"}="HAD2 BLV"; $resn_exch{"1HBD BLV"}="HBD1 BLV";
$resn_exch{"2HBD BLV"}="HBD2 BLV"; $resn_exch{"1HN  BLY"}=" HN1 BLY"; $resn_exch{"2HN  BLY"}=" HN2 BLY";
$resn_exch{"1HB  BLY"}=" HB1 BLY"; $resn_exch{"2HB  BLY"}=" HB2 BLY"; $resn_exch{"1HG  BLY"}=" HG1 BLY";
$resn_exch{"2HG  BLY"}=" HG2 BLY"; $resn_exch{"1HD  BLY"}=" HD1 BLY"; $resn_exch{"2HD  BLY"}=" HD2 BLY";
$resn_exch{"1HE  BLY"}=" HE1 BLY"; $resn_exch{"2HE  BLY"}=" HE2 BLY"; $resn_exch{"1HZ  BLY"}=" HZ1 BLY";
$resn_exch{"2HZ  BLY"}=" HZ2 BLY"; $resn_exch{"3HZ  BLY"}=" HZ3 BLY"; $resn_exch{"1H1  BLZ"}=" H11 BLZ";
$resn_exch{"2H1  BLZ"}=" H12 BLZ"; $resn_exch{"3H1  BLZ"}="H13A BLZ"; $resn_exch{"1H14 BLZ"}="H141 BLZ";
$resn_exch{"2H14 BLZ"}="H142 BLZ"; $resn_exch{"3H14 BLZ"}="H143 BLZ"; $resn_exch{"1H26 BLZ"}="H261 BLZ";
$resn_exch{"2H26 BLZ"}="H262 BLZ"; $resn_exch{"1H31 BLZ"}="H311 BLZ"; $resn_exch{"2H31 BLZ"}="H312 BLZ";
$resn_exch{"3H31 BLZ"}="H313 BLZ"; $resn_exch{"1H2  BM1"}=" H21 BM1"; $resn_exch{"2H2  BM1"}=" H22 BM1";
$resn_exch{"1H3  BM1"}=" H31 BM1"; $resn_exch{"2H3  BM1"}=" H32 BM1"; $resn_exch{"1H8  BM1"}=" H81 BM1";
$resn_exch{"1H9  BM1"}=" H91 BM1"; $resn_exch{"1H10 BM1"}="H101 BM1"; $resn_exch{"1H11 BM1"}="H111 BM1";
$resn_exch{"1H14 BM1"}="H141 BM1"; $resn_exch{"1HA  BM1"}=" HA1 BM1"; $resn_exch{"2HA  BM1"}=" HA2 BM1";
$resn_exch{"3HA  BM1"}=" HA3 BM1"; $resn_exch{"1H16 BM1"}="H161 BM1"; $resn_exch{"1H17 BM1"}="H171 BM1";
$resn_exch{"1H18 BM1"}="H181 BM1"; $resn_exch{"1H16 BM2"}="H161 BM2"; $resn_exch{"2H16 BM2"}="H162 BM2";
$resn_exch{"1H21 BM2"}="H211 BM2"; $resn_exch{"2H21 BM2"}="H212 BM2"; $resn_exch{"1H22 BM2"}="H221 BM2";
$resn_exch{"2H22 BM2"}="H222 BM2"; $resn_exch{"1H23 BM2"}="H231 BM2"; $resn_exch{"2H23 BM2"}="H232 BM2";
$resn_exch{"1H25 BM2"}="H251 BM2"; $resn_exch{"2H25 BM2"}="H252 BM2"; $resn_exch{"1H29 BM2"}="H291 BM2";
$resn_exch{"2H29 BM2"}="H292 BM2"; $resn_exch{"1H30 BM2"}="H301 BM2"; $resn_exch{"2H30 BM2"}="H302 BM2";
$resn_exch{"1H31 BM2"}="H311 BM2"; $resn_exch{"2H31 BM2"}="H312 BM2"; $resn_exch{"1HN4 BM2"}="HN41 BM2";
$resn_exch{"2HN4 BM2"}="HN42 BM2"; $resn_exch{"1H8C BM3"}="H8C1 BM3"; $resn_exch{"2H8C BM3"}="H8C2 BM3";
$resn_exch{"3H8C BM3"}="H8C3 BM3"; $resn_exch{"1H6C BM3"}="H6C1 BM3"; $resn_exch{"2H6C BM3"}="H6C2 BM3";
$resn_exch{"1H2  BM5"}=" H21 BM5"; $resn_exch{"2H2  BM5"}=" H22 BM5"; $resn_exch{"1H8  BM5"}=" H81 BM5";
$resn_exch{"1H9  BM5"}=" H91 BM5"; $resn_exch{"1H10 BM5"}="H101 BM5"; $resn_exch{"1H11 BM5"}="H111 BM5";
$resn_exch{"1H14 BM5"}="H141 BM5"; $resn_exch{"1H15 BM5"}="H151 BM5"; $resn_exch{"1H16 BM5"}="H161 BM5";
$resn_exch{"1H17 BM5"}="H171 BM5"; $resn_exch{"1H18 BM5"}="H181 BM5"; $resn_exch{"1HD  BM5"}=" HD1 BM5";
$resn_exch{"2HD  BM5"}=" HD2 BM5"; $resn_exch{"3HD  BM5"}=" HD3 BM5"; $resn_exch{"1H16 BM6"}="H161 BM6";
$resn_exch{"2H16 BM6"}="H162 BM6"; $resn_exch{"1H17 BM6"}="H171 BM6"; $resn_exch{"2H17 BM6"}="H172 BM6";
$resn_exch{"1H19 BM6"}="H191 BM6"; $resn_exch{"2H19 BM6"}="H192 BM6"; $resn_exch{"1H20 BM6"}="H201 BM6";
$resn_exch{"2H20 BM6"}="H202 BM6"; $resn_exch{"1H21 BM6"}="H211 BM6"; $resn_exch{"2H21 BM6"}="H212 BM6";
$resn_exch{"3H21 BM6"}="H213 BM6"; $resn_exch{"1H22 BM6"}="H221 BM6"; $resn_exch{"2H22 BM6"}="H222 BM6";
$resn_exch{"3H22 BM6"}="H223 BM6"; $resn_exch{"1H23 BM6"}="H231 BM6"; $resn_exch{"2H23 BM6"}="H232 BM6";
$resn_exch{"3H23 BM6"}="H233 BM6"; $resn_exch{"1H24 BM6"}="H241 BM6"; $resn_exch{"2H24 BM6"}="H242 BM6";
$resn_exch{"3H24 BM6"}="H243 BM6"; $resn_exch{"1H16 BM9"}="H161 BM9"; $resn_exch{"2H16 BM9"}="H162 BM9";
$resn_exch{"1H21 BM9"}="H211 BM9"; $resn_exch{"2H21 BM9"}="H212 BM9"; $resn_exch{"1H22 BM9"}="H221 BM9";
$resn_exch{"2H22 BM9"}="H222 BM9"; $resn_exch{"1H23 BM9"}="H231 BM9"; $resn_exch{"2H23 BM9"}="H232 BM9";
$resn_exch{"1H25 BM9"}="H251 BM9"; $resn_exch{"2H25 BM9"}="H252 BM9"; $resn_exch{"1H30 BM9"}="H301 BM9";
$resn_exch{"2H30 BM9"}="H302 BM9"; $resn_exch{"1H32 BM9"}="H321 BM9"; $resn_exch{"2H32 BM9"}="H322 BM9";
$resn_exch{"1H33 BM9"}="H331 BM9"; $resn_exch{"2H33 BM9"}="H332 BM9"; $resn_exch{"1H34 BM9"}="H341 BM9";
$resn_exch{"2H34 BM9"}="H342 BM9"; $resn_exch{"1HN6 BM9"}="HN61 BM9"; $resn_exch{"2HN6 BM9"}="HN62 BM9";
$resn_exch{"1H6  BMA"}=" H61 BMA"; $resn_exch{"2H6  BMA"}=" H62 BMA"; $resn_exch{"3H3  BMD"}=" H33 BMD";
$resn_exch{"2H3  BMD"}=" H32 BMD"; $resn_exch{"1H3  BMD"}=" H31 BMD"; $resn_exch{"2H2  BMD"}=" H22 BMD";
$resn_exch{"1H2  BMD"}=" H21 BMD"; $resn_exch{"2H1  BMD"}=" H12 BMD"; $resn_exch{"1H1  BMD"}=" H11 BMD";
$resn_exch{"1HN  BMD"}=" HN1 BMD"; $resn_exch{"2HN  BMD"}=" HN2 BMD"; $resn_exch{"1H1  BME"}=" H11 BME";
$resn_exch{"2H1  BME"}=" H12 BME"; $resn_exch{"1H2  BME"}=" H21 BME"; $resn_exch{"2H2  BME"}=" H22 BME";
$resn_exch{"1H62 BMH"}="H621 BMH"; $resn_exch{"2H62 BMH"}="H622 BMH"; $resn_exch{"3H62 BMH"}="H623 BMH";
$resn_exch{"1H11 BMH"}="H111 BMH"; $resn_exch{"2H11 BMH"}="H112 BMH"; $resn_exch{"3H11 BMH"}="H113 BMH";
$resn_exch{"1H23 BMH"}="H231 BMH"; $resn_exch{"2H23 BMH"}="H232 BMH"; $resn_exch{"1H27 BMH"}="H271 BMH";
$resn_exch{"2H27 BMH"}="H272 BMH"; $resn_exch{"1H4  BMI"}=" H41 BMI"; $resn_exch{"2H4  BMI"}=" H42 BMI";
$resn_exch{"1H21 BMI"}="H211 BMI"; $resn_exch{"2H21 BMI"}="H212 BMI"; $resn_exch{"3H21 BMI"}="H213 BMI";
$resn_exch{"1H1  BMM"}=" H11 BMM"; $resn_exch{"2H1  BMM"}=" H12 BMM"; $resn_exch{"3H1  BMM"}=" H13 BMM";
$resn_exch{"1H17 BMO"}="H171 BMO"; $resn_exch{"1H19 BMO"}="H191 BMO"; $resn_exch{"2H19 BMO"}="H192 BMO";
$resn_exch{"1H36 BMO"}="H361 BMO"; $resn_exch{"2H36 BMO"}="H362 BMO"; $resn_exch{"1H22 BMO"}="H221 BMO";
$resn_exch{"2H22 BMO"}="H222 BMO"; $resn_exch{"1H1  BMO"}=" H11 BMO"; $resn_exch{"2H1  BMO"}="H12A BMO";
$resn_exch{"1H2  BMO"}=" H21 BMO"; $resn_exch{"2H2  BMO"}=" H22 BMO"; $resn_exch{"1H3  BMO"}=" H31 BMO";
$resn_exch{"2H3  BMO"}="H32A BMO"; $resn_exch{"2HOP BMP"}="HOP2 BMP"; $resn_exch{"3HOP BMP"}="HOP3 BMP";
$resn_exch{"1H5* BMP"}=" H5' BMP"; $resn_exch{"2H5* BMP"}="H5'' BMP"; $resn_exch{" H4* BMP"}=" H4' BMP";
$resn_exch{" H3* BMP"}=" H3' BMP"; $resn_exch{"*HO3 BMP"}="HO3' BMP"; $resn_exch{" H2* BMP"}=" H2' BMP";
$resn_exch{"*HO2 BMP"}="HO2' BMP"; $resn_exch{" H1* BMP"}=" H1' BMP"; $resn_exch{"1H5  BMP"}=" H51 BMP";
$resn_exch{"2HOP BMQ"}="HOP2 BMQ"; $resn_exch{"3HOP BMQ"}="HOP3 BMQ"; $resn_exch{"1H5* BMQ"}="H5'1 BMQ";
$resn_exch{"2H5* BMQ"}="H5'2 BMQ"; $resn_exch{" H4* BMQ"}=" H4' BMQ"; $resn_exch{" H3* BMQ"}=" H3' BMQ";
$resn_exch{"*HO3 BMQ"}="HO3' BMQ"; $resn_exch{" H2* BMQ"}=" H2' BMQ"; $resn_exch{"*HO2 BMQ"}="HO2' BMQ";
$resn_exch{" H1* BMQ"}=" H1' BMQ"; $resn_exch{"1H5  BMQ"}=" H51 BMQ"; $resn_exch{"2H5  BMQ"}=" H52 BMQ";
$resn_exch{"1H16 BMS"}="H161 BMS"; $resn_exch{"2H16 BMS"}="H162 BMS"; $resn_exch{"1H17 BMS"}="H171 BMS";
$resn_exch{"2H17 BMS"}="H172 BMS"; $resn_exch{"1H19 BMS"}="H191 BMS"; $resn_exch{"2H19 BMS"}="H192 BMS";
$resn_exch{"3H19 BMS"}="H193 BMS"; $resn_exch{"1H20 BMS"}="H201 BMS"; $resn_exch{"2H20 BMS"}="H202 BMS";
$resn_exch{"3H20 BMS"}="H203 BMS"; $resn_exch{" HN  BMT"}=" H   BMT"; $resn_exch{"1HCN BMT"}=" HN1 BMT";
$resn_exch{"2HCN BMT"}=" HN2 BMT"; $resn_exch{"3HCN BMT"}=" HN3 BMT"; $resn_exch{"1HD1 BMT"}="HD11 BMT";
$resn_exch{"2HD1 BMT"}="HD12 BMT"; $resn_exch{"3HD1 BMT"}="HD13 BMT"; $resn_exch{"1HD2 BMT"}="HD22 BMT";
$resn_exch{"2HD2 BMT"}="HD23 BMT"; $resn_exch{"1HH  BMT"}=" HH1 BMT"; $resn_exch{"2HH  BMT"}=" HH2 BMT";
$resn_exch{"3HH  BMT"}=" HH3 BMT"; $resn_exch{"1H15 BMU"}="H151 BMU"; $resn_exch{"2H15 BMU"}="H152 BMU";
$resn_exch{"3H15 BMU"}="H153 BMU"; $resn_exch{"1H17 BMU"}="H171 BMU"; $resn_exch{"2H17 BMU"}="H172 BMU";
$resn_exch{"3H17 BMU"}="H173 BMU"; $resn_exch{"1H18 BMU"}="H181 BMU"; $resn_exch{"2H18 BMU"}="H182 BMU";
$resn_exch{"3H18 BMU"}="H183 BMU"; $resn_exch{"1H19 BMU"}="H191 BMU"; $resn_exch{"2H19 BMU"}="H192 BMU";
$resn_exch{"3H19 BMU"}="H193 BMU"; $resn_exch{"1HAR BMV"}="HAR1 BMV"; $resn_exch{"2HAR BMV"}="HAR2 BMV";
$resn_exch{"1HAU BMV"}="HAU1 BMV"; $resn_exch{"2HAU BMV"}="HAU2 BMV"; $resn_exch{"1HAS BMV"}="HAS1 BMV";
$resn_exch{"2HAS BMV"}="HAS2 BMV"; $resn_exch{"1HAT BMV"}="HAT1 BMV"; $resn_exch{"2HAT BMV"}="HAT2 BMV";
$resn_exch{"1H28 BMY"}="H281 BMY"; $resn_exch{"2H28 BMY"}="H282 BMY"; $resn_exch{"1H31 BMY"}="H311 BMY";
$resn_exch{"2H31 BMY"}="H312 BMY"; $resn_exch{"8HO0 BMY"}="HO08 BMY"; $resn_exch{"1H2' BMY"}="H2'1 BMY";
$resn_exch{"2H2' BMY"}="H2'2 BMY"; $resn_exch{"1H7' BMY"}="H7'1 BMY"; $resn_exch{"2H7' BMY"}="H7'2 BMY";
$resn_exch{"3H7' BMY"}="H7'3 BMY"; $resn_exch{"1H8' BMY"}="H8'1 BMY"; $resn_exch{"2H8' BMY"}="H8'2 BMY";
$resn_exch{"9HO' BMY"}="HO'9 BMY"; $resn_exch{"1HO\" BMY"}="HO'1 BMY"; $resn_exch{"1H2\" BMY"}="H2A1 BMY";
$resn_exch{"2H2\" BMY"}="H2A2 BMY"; $resn_exch{"3H2\" BMY"}="H2'3 BMY"; $resn_exch{" H3\" BMY"}=" H3' BMY";
$resn_exch{" H4\" BMY"}=" H4' BMY"; $resn_exch{" H5\" BMY"}=" H5B BMY"; $resn_exch{" H6\" BMY"}=" H6' BMY";
$resn_exch{" H7\" BMY"}=" H7' BMY"; $resn_exch{" H9\" BMY"}=" H9' BMY"; $resn_exch{"1HOA BMY"}="HOA1 BMY";
$resn_exch{"2HOA BMY"}="HOA2 BMY"; $resn_exch{"1H3A BMY"}="H3A1 BMY"; $resn_exch{"2H3A BMY"}="H3A2 BMY";
$resn_exch{"1HH1 BMZ"}="HH11 BMZ"; $resn_exch{"2HH1 BMZ"}="HH12 BMZ"; $resn_exch{"1HH2 BMZ"}="HH21 BMZ";
$resn_exch{"2HH2 BMZ"}="HH22 BMZ"; $resn_exch{"1H15 BN2"}="H151 BN2"; $resn_exch{"2H15 BN2"}="H152 BN2";
$resn_exch{"1H16 BN2"}="H161 BN2"; $resn_exch{"2H16 BN2"}="H162 BN2"; $resn_exch{"1H17 BN2"}="H171 BN2";
$resn_exch{"2H17 BN2"}="H172 BN2"; $resn_exch{"1H18 BN2"}="H181 BN2"; $resn_exch{"2H18 BN2"}="H182 BN2";
$resn_exch{"1H19 BN2"}="H191 BN2"; $resn_exch{"2H19 BN2"}="H192 BN2"; $resn_exch{"1H20 BN2"}="H201 BN2";
$resn_exch{"2H20 BN2"}="H202 BN2"; $resn_exch{"1H15 BN3"}="H151 BN3"; $resn_exch{"2H15 BN3"}="H152 BN3";
$resn_exch{"1H16 BN3"}="H161 BN3"; $resn_exch{"2H16 BN3"}="H162 BN3"; $resn_exch{"1H17 BN3"}="H171 BN3";
$resn_exch{"2H17 BN3"}="H172 BN3"; $resn_exch{"1H15 BN4"}="H151 BN4"; $resn_exch{"2H15 BN4"}="H152 BN4";
$resn_exch{"1H16 BN4"}="H161 BN4"; $resn_exch{"2H16 BN4"}="H162 BN4"; $resn_exch{"1H17 BN4"}="H171 BN4";
$resn_exch{"2H17 BN4"}="H172 BN4"; $resn_exch{"1H19 BN4"}="H191 BN4"; $resn_exch{"2H19 BN4"}="H192 BN4";
$resn_exch{"3H19 BN4"}="H193 BN4"; $resn_exch{"1H20 BN4"}="H201 BN4"; $resn_exch{"2H20 BN4"}="H202 BN4";
$resn_exch{"3H20 BN4"}="H203 BN4"; $resn_exch{"1H15 BN5"}="H151 BN5"; $resn_exch{"2H15 BN5"}="H152 BN5";
$resn_exch{"3H15 BN5"}="H153 BN5"; $resn_exch{"1H16 BN5"}="H161 BN5"; $resn_exch{"2H16 BN5"}="H162 BN5";
$resn_exch{"1H17 BN5"}="H171 BN5"; $resn_exch{"2H17 BN5"}="H172 BN5"; $resn_exch{"1H18 BN5"}="H181 BN5";
$resn_exch{"2H18 BN5"}="H182 BN5"; $resn_exch{"1H19 BN5"}="H191 BN5"; $resn_exch{"2H19 BN5"}="H192 BN5";
$resn_exch{"1H4  BN6"}="H41A BN6"; $resn_exch{"2H4  BN6"}=" H42 BN6"; $resn_exch{"1H6  BN6"}=" H61 BN6";
$resn_exch{"2H6  BN6"}=" H62 BN6"; $resn_exch{"3H6  BN6"}=" H63 BN6"; $resn_exch{"1H7  BN6"}=" H71 BN6";
$resn_exch{"2H7  BN6"}=" H72 BN6"; $resn_exch{"3H7  BN6"}=" H73 BN6"; $resn_exch{"1H11 BN6"}="H111 BN6";
$resn_exch{"2H11 BN6"}="H112 BN6"; $resn_exch{"1H12 BN6"}="H121 BN6"; $resn_exch{"2H12 BN6"}="H122 BN6";
$resn_exch{"1H13 BN6"}="H131 BN6"; $resn_exch{"2H13 BN6"}="H132 BN6"; $resn_exch{"1H14 BN6"}="H141 BN6";
$resn_exch{"2H14 BN6"}="H142 BN6"; $resn_exch{"1H15 BN6"}="H151 BN6"; $resn_exch{"2H15 BN6"}="H152 BN6";
$resn_exch{"1H22 BN6"}="H221 BN6"; $resn_exch{"2H22 BN6"}="H222 BN6"; $resn_exch{"1H24 BN6"}="H241 BN6";
$resn_exch{"2H24 BN6"}="H242 BN6"; $resn_exch{"1H25 BN6"}="H251 BN6"; $resn_exch{"2H25 BN6"}="H252 BN6";
$resn_exch{"1H28 BN6"}="H281 BN6"; $resn_exch{"2H28 BN6"}="H282 BN6"; $resn_exch{"1H30 BN6"}="H301 BN6";
$resn_exch{"2H30 BN6"}="H302 BN6"; $resn_exch{"1H31 BN6"}="H311 BN6"; $resn_exch{"2H31 BN6"}="H312 BN6";
$resn_exch{"1H36 BN6"}="H361 BN6"; $resn_exch{"2H36 BN6"}="H362 BN6"; $resn_exch{"1HB1 BNA"}="HB11 BNA";
$resn_exch{"2HB1 BNA"}="HB12 BNA"; $resn_exch{"1HB2 BNA"}="HB21 BNA"; $resn_exch{"2HB2 BNA"}="HB22 BNA";
$resn_exch{"1HY1 BND"}="HY11 BND"; $resn_exch{"2HY1 BND"}="HY12 BND"; $resn_exch{"3HY1 BND"}="HY13 BND";
$resn_exch{"1HX3 BND"}="HX31 BND"; $resn_exch{"2HX3 BND"}="HX32 BND"; $resn_exch{"1H11 BND"}="H111 BND";
$resn_exch{"2H11 BND"}="H112 BND"; $resn_exch{"3H11 BND"}="H113 BND"; $resn_exch{"1HX9 BND"}="HX91 BND";
$resn_exch{"2HX9 BND"}="HX92 BND"; $resn_exch{"1H4  BNE"}=" H41 BNE"; $resn_exch{"2H4  BNE"}=" H42 BNE";
$resn_exch{"1H5  BNE"}=" H51 BNE"; $resn_exch{"2H5  BNE"}=" H52 BNE"; $resn_exch{"1H6  BNE"}=" H61 BNE";
$resn_exch{"1H7  BNE"}=" H71 BNE"; $resn_exch{"1H10 BNE"}="H101 BNE"; $resn_exch{"2H10 BNE"}="H102 BNE";
$resn_exch{"1H11 BNE"}="H111 BNE"; $resn_exch{"2H11 BNE"}="H112 BNE"; $resn_exch{"3H11 BNE"}="H113 BNE";
$resn_exch{"1H13 BNE"}="H131 BNE"; $resn_exch{"1H14 BNE"}="H141 BNE"; $resn_exch{"2H14 BNE"}="H142 BNE";
$resn_exch{"1H15 BNE"}="H151 BNE"; $resn_exch{"2H15 BNE"}="H152 BNE"; $resn_exch{"3H15 BNE"}="H153 BNE";
$resn_exch{"1H16 BNE"}="H161 BNE"; $resn_exch{"2H16 BNE"}="H162 BNE"; $resn_exch{"1H18 BNE"}="H181 BNE";
$resn_exch{"1H21 BNE"}="H211 BNE"; $resn_exch{"1H22 BNE"}="H221 BNE"; $resn_exch{"1H25 BNE"}="H251 BNE";
$resn_exch{"1H29 BNE"}="H291 BNE"; $resn_exch{"2H29 BNE"}="H292 BNE"; $resn_exch{"3H29 BNE"}="H293 BNE";
$resn_exch{"1H30 BNE"}="H301 BNE"; $resn_exch{"2H30 BNE"}="H302 BNE"; $resn_exch{"1H32 BNE"}="H321 BNE";
$resn_exch{"1H33 BNE"}="H331 BNE"; $resn_exch{"1H34 BNE"}="H341 BNE"; $resn_exch{"2H34 BNE"}="H342 BNE";
$resn_exch{"3H34 BNE"}="H343 BNE"; $resn_exch{"1H12 BNF"}="H121 BNF"; $resn_exch{"2H12 BNF"}="H122 BNF";
$resn_exch{"1H6  BNG"}=" H61 BNG"; $resn_exch{"2H6  BNG"}=" H62 BNG"; $resn_exch{"1H1' BNG"}="H1'1 BNG";
$resn_exch{"2H1' BNG"}="H1'2 BNG"; $resn_exch{"1H2' BNG"}="H2'1 BNG"; $resn_exch{"2H2' BNG"}="H2'2 BNG";
$resn_exch{"1H3' BNG"}="H3'1 BNG"; $resn_exch{"2H3' BNG"}="H3'2 BNG"; $resn_exch{"1H4' BNG"}="H4'1 BNG";
$resn_exch{"2H4' BNG"}="H4'2 BNG"; $resn_exch{"1H5' BNG"}="H5'1 BNG"; $resn_exch{"2H5' BNG"}="H5'2 BNG";
$resn_exch{"1H6' BNG"}="H6'1 BNG"; $resn_exch{"2H6' BNG"}="H6'2 BNG"; $resn_exch{"1H7' BNG"}="H7'1 BNG";
$resn_exch{"2H7' BNG"}="H7'2 BNG"; $resn_exch{"1H8' BNG"}="H8'1 BNG"; $resn_exch{"2H8' BNG"}="H8'2 BNG";
$resn_exch{"1H9' BNG"}="H9'1 BNG"; $resn_exch{"2H9' BNG"}="H9'2 BNG"; $resn_exch{"3H9' BNG"}="H9'3 BNG";
$resn_exch{"1H4  BNH"}="H41A BNH"; $resn_exch{"2H4  BNH"}=" H42 BNH"; $resn_exch{"1H6  BNH"}=" H61 BNH";
$resn_exch{"2H6  BNH"}=" H62 BNH"; $resn_exch{"3H6  BNH"}=" H63 BNH"; $resn_exch{"1H7  BNH"}=" H71 BNH";
$resn_exch{"2H7  BNH"}=" H72 BNH"; $resn_exch{"3H7  BNH"}=" H73 BNH"; $resn_exch{"1H11 BNH"}="H111 BNH";
$resn_exch{"2H11 BNH"}="H112 BNH"; $resn_exch{"1H12 BNH"}="H121 BNH"; $resn_exch{"2H12 BNH"}="H122 BNH";
$resn_exch{"1H13 BNH"}="H131 BNH"; $resn_exch{"2H13 BNH"}="H132 BNH"; $resn_exch{"1H14 BNH"}="H141 BNH";
$resn_exch{"2H14 BNH"}="H142 BNH"; $resn_exch{"1H15 BNH"}="H151 BNH"; $resn_exch{"2H15 BNH"}="H152 BNH";
$resn_exch{"1H22 BNH"}="H221 BNH"; $resn_exch{"2H22 BNH"}="H222 BNH"; $resn_exch{"1H24 BNH"}="H241 BNH";
$resn_exch{"2H24 BNH"}="H242 BNH"; $resn_exch{"1H25 BNH"}="H251 BNH"; $resn_exch{"2H25 BNH"}="H252 BNH";
$resn_exch{"1H28 BNH"}="H281 BNH"; $resn_exch{"2H28 BNH"}="H282 BNH"; $resn_exch{"1H30 BNH"}="H301 BNH";
$resn_exch{"2H30 BNH"}="H302 BNH"; $resn_exch{"1H31 BNH"}="H311 BNH"; $resn_exch{"2H31 BNH"}="H312 BNH";
$resn_exch{"1H37 BNH"}="H371 BNH"; $resn_exch{"2H37 BNH"}="H372 BNH"; $resn_exch{"1H10 BNI"}="H101 BNI";
$resn_exch{"2H10 BNI"}="H102 BNI"; $resn_exch{"1H9  BNI"}=" H91 BNI"; $resn_exch{"2H9  BNI"}=" H92 BNI";
$resn_exch{"1H8  BNI"}=" H81 BNI"; $resn_exch{"2H8  BNI"}=" H82 BNI"; $resn_exch{"1H7  BNI"}=" H71 BNI";
$resn_exch{"2H7  BNI"}=" H72 BNI"; $resn_exch{"1H2  BNI"}="H21A BNI"; $resn_exch{"1H6  BNI"}=" H61 BNI";
$resn_exch{"2H6  BNI"}=" H62 BNI"; $resn_exch{"1HN1 BNI"}="HN11 BNI"; $resn_exch{"1HN2 BNI"}="HN21 BNI";
$resn_exch{"1H17 BNI"}="H171 BNI"; $resn_exch{"1H17 BNL"}="H171 BNL"; $resn_exch{"2H17 BNL"}="H172 BNL";
$resn_exch{"1H15 BNL"}="H151 BNL"; $resn_exch{"2H15 BNL"}="H152 BNL"; $resn_exch{"1H14 BNL"}="H141 BNL";
$resn_exch{"2H14 BNL"}="H142 BNL"; $resn_exch{"1H13 BNL"}="H131 BNL"; $resn_exch{"2H13 BNL"}="H132 BNL";
$resn_exch{"1H12 BNL"}="H121 BNL"; $resn_exch{"2H12 BNL"}="H122 BNL"; $resn_exch{"1H1  BNL"}=" H11 BNL";
$resn_exch{"2H1  BNL"}=" H12 BNL"; $resn_exch{"1H3  BNL"}=" H31 BNL"; $resn_exch{"2H3  BNL"}=" H32 BNL";
$resn_exch{"1H4  BNL"}=" H41 BNL"; $resn_exch{"2H4  BNL"}=" H42 BNL"; $resn_exch{"1H5  BNL"}=" H51 BNL";
$resn_exch{"2H5  BNL"}=" H52 BNL"; $resn_exch{"1H6  BNL"}=" H61 BNL"; $resn_exch{"2H6  BNL"}=" H62 BNL";
$resn_exch{"1HH1 BNN"}="HH11 BNN"; $resn_exch{"2HH1 BNN"}="HH12 BNN"; $resn_exch{"3HH1 BNN"}="HH13 BNN";
$resn_exch{"1H16 BNN"}="H161 BNN"; $resn_exch{"2H16 BNN"}="H162 BNN"; $resn_exch{"1H7  BNN"}=" H71 BNN";
$resn_exch{"2H7  BNN"}=" H72 BNN"; $resn_exch{"1HN  BNO"}=" HN1 BNO"; $resn_exch{"2HN  BNO"}=" HN2 BNO";
$resn_exch{"1HB  BNO"}=" HB1 BNO"; $resn_exch{"2HB  BNO"}=" HB2 BNO"; $resn_exch{"1HG  BNO"}=" HG1 BNO";
$resn_exch{"2HG  BNO"}=" HG2 BNO"; $resn_exch{"1HD  BNO"}=" HD1 BNO"; $resn_exch{"2HD  BNO"}=" HD2 BNO";
$resn_exch{"1HE  BNO"}=" HE1 BNO"; $resn_exch{"2HE  BNO"}=" HE2 BNO"; $resn_exch{"3HE  BNO"}=" HE3 BNO";
$resn_exch{"1H8  BNP"}=" H81 BNP"; $resn_exch{"2H8  BNP"}=" H82 BNP"; $resn_exch{"1H9  BNP"}=" H91 BNP";
$resn_exch{"2H9  BNP"}=" H92 BNP"; $resn_exch{"1H10 BNP"}="H101 BNP"; $resn_exch{"2H10 BNP"}="H102 BNP";
$resn_exch{"1H11 BNP"}="H111 BNP"; $resn_exch{"2H11 BNP"}="H112 BNP"; $resn_exch{"1H20 BNP"}="H201 BNP";
$resn_exch{"2H20 BNP"}="H202 BNP"; $resn_exch{"1H21 BNP"}="H211 BNP"; $resn_exch{"2H21 BNP"}="H212 BNP";
$resn_exch{"1H22 BNP"}="H221 BNP"; $resn_exch{"2H22 BNP"}="H222 BNP"; $resn_exch{"1H23 BNP"}="H231 BNP";
$resn_exch{"2H23 BNP"}="H232 BNP"; $resn_exch{"1H11 BNR"}="H111 BNR"; $resn_exch{"2H11 BNR"}="H112 BNR";
$resn_exch{"1H14 BNR"}="H141 BNR"; $resn_exch{"2H14 BNR"}="H142 BNR"; $resn_exch{"3H14 BNR"}="H143 BNR";
$resn_exch{"1H15 BNR"}="H151 BNR"; $resn_exch{"2H15 BNR"}="H152 BNR"; $resn_exch{"1H21 BNR"}="H211 BNR";
$resn_exch{"2H21 BNR"}="H212 BNR"; $resn_exch{"3H21 BNR"}="H213 BNR"; $resn_exch{" H1* BNR"}=" H1' BNR";
$resn_exch{"1H2* BNR"}="H2'1 BNR"; $resn_exch{"2H2* BNR"}="H2'2 BNR"; $resn_exch{" H3* BNR"}=" H3' BNR";
$resn_exch{"*HO3 BNR"}="HO3' BNR"; $resn_exch{" H4* BNR"}=" H4' BNR"; $resn_exch{"1HN* BNR"}="HN'1 BNR";
$resn_exch{"2HN* BNR"}="HN'2 BNR"; $resn_exch{" H5* BNR"}=" H5' BNR"; $resn_exch{"1H6* BNR"}="H6'1 BNR";
$resn_exch{"2H6* BNR"}="H6'2 BNR"; $resn_exch{"3H6* BNR"}="H6'3 BNR"; $resn_exch{"1H7* BNR"}="H7'1 BNR";
$resn_exch{"2H7* BNR"}="H7'2 BNR"; $resn_exch{"'H1  BNR"}=" H1B BNR"; $resn_exch{"'H2  BNR"}=" H2' BNR";
$resn_exch{"'H3  BNR"}=" H3B BNR"; $resn_exch{"'HO8 BNR"}="HO8' BNR"; $resn_exch{"'H10 BNR"}="H10' BNR";
$resn_exch{"1HL' BNR"}="HL'1 BNR"; $resn_exch{"2HL' BNR"}="HL'2 BNR"; $resn_exch{"'H12 BNR"}="H12' BNR";
$resn_exch{"1HK' BNR"}="HK'1 BNR"; $resn_exch{"2HK' BNR"}="HK'2 BNR"; $resn_exch{"3HK' BNR"}="HK'3 BNR";
$resn_exch{"1HJ' BNR"}="HJ'1 BNR"; $resn_exch{"2HJ' BNR"}="HJ'2 BNR"; $resn_exch{"'H17 BNR"}="H17' BNR";
$resn_exch{"1HI' BNR"}="HI'1 BNR"; $resn_exch{"2HI' BNR"}="HI'2 BNR"; $resn_exch{"3HI' BNR"}="HI'3 BNR";
$resn_exch{"'H1* BNR"}=" H1D BNR"; $resn_exch{"1H2' BNR"}=" H2B BNR"; $resn_exch{"2H2' BNR"}=" H2D BNR";
$resn_exch{"'H3* BNR"}=" H3D BNR"; $resn_exch{"'HO3 BNR"}="HO3A BNR"; $resn_exch{"'H4* BNR"}=" H4D BNR";
$resn_exch{"1HN' BNR"}=" HNB BNR"; $resn_exch{"2HN' BNR"}=" HND BNR"; $resn_exch{"'H5* BNR"}=" H5D BNR";
$resn_exch{"1H6' BNR"}=" H6B BNR"; $resn_exch{"2H6' BNR"}=" H6D BNR"; $resn_exch{"3H6' BNR"}=" H6E BNR";
$resn_exch{"1H7' BNR"}=" H7B BNR"; $resn_exch{"2H7' BNR"}=" H7D BNR"; $resn_exch{"'H9A BNR"}="H9A' BNR";
$resn_exch{"'H9B BNR"}="H9B' BNR"; $resn_exch{"1HAA BNT"}="HAA1 BNT"; $resn_exch{"2HAA BNT"}="HAA2 BNT";
$resn_exch{"3HAA BNT"}="HAA3 BNT"; $resn_exch{"1HAC BNT"}="HAC1 BNT"; $resn_exch{"2HAC BNT"}="HAC2 BNT";
$resn_exch{"3HAC BNT"}="HAC3 BNT"; $resn_exch{"1HAM BNT"}="HAM1 BNT"; $resn_exch{"2HAM BNT"}="HAM2 BNT";
$resn_exch{"3HAM BNT"}="HAM3 BNT"; $resn_exch{"1HN1 BO1"}="HN11 BO1"; $resn_exch{"2HN1 BO1"}="HN12 BO1";
$resn_exch{"1H12 BO1"}="H121 BO1"; $resn_exch{"2H12 BO1"}="H122 BO1"; $resn_exch{"1H11 BO2"}="H111 BO2";
$resn_exch{"2H11 BO2"}="H112 BO2"; $resn_exch{"1H22 BO2"}="H221 BO2"; $resn_exch{"2H22 BO2"}="H222 BO2";
$resn_exch{"1H24 BO2"}="H241 BO2"; $resn_exch{"2H24 BO2"}="H242 BO2"; $resn_exch{"3H24 BO2"}="H243 BO2";
$resn_exch{"1H25 BO2"}="H251 BO2"; $resn_exch{"2H25 BO2"}="H252 BO2"; $resn_exch{"3H25 BO2"}="H253 BO2";
$resn_exch{"1H41 BO9"}="H411 BO9"; $resn_exch{"2H41 BO9"}="H412 BO9"; $resn_exch{"3H41 BO9"}="H413 BO9";
$resn_exch{"1H40 BO9"}="H401 BO9"; $resn_exch{"2H40 BO9"}="H402 BO9"; $resn_exch{"3H40 BO9"}="H403 BO9";
$resn_exch{"1H39 BO9"}="H391 BO9"; $resn_exch{"2H39 BO9"}="H392 BO9"; $resn_exch{"3H39 BO9"}="H393 BO9";
$resn_exch{"1H29 BO9"}="H291 BO9"; $resn_exch{"2H29 BO9"}="H292 BO9"; $resn_exch{"1H28 BO9"}="H281 BO9";
$resn_exch{"2H28 BO9"}="H282 BO9"; $resn_exch{"1H5  BO9"}=" H51 BO9"; $resn_exch{"2H5  BO9"}=" H52 BO9";
$resn_exch{"3H5  BO9"}=" H53 BO9"; $resn_exch{"1H10 BO9"}="H101 BO9"; $resn_exch{"2H10 BO9"}="H102 BO9";
$resn_exch{"1H11 BO9"}="H111 BO9"; $resn_exch{"2H11 BO9"}="H112 BO9"; $resn_exch{"1H12 BO9"}="H121 BO9";
$resn_exch{"2H12 BO9"}="H122 BO9"; $resn_exch{"1H13 BO9"}="H131 BO9"; $resn_exch{"2H13 BO9"}="H132 BO9";
$resn_exch{"1H18 BO9"}="H181 BO9"; $resn_exch{"2H18 BO9"}="H182 BO9"; $resn_exch{"1H1  BOC"}=" H11 BOC";
$resn_exch{"2H1  BOC"}=" H12 BOC"; $resn_exch{"3H1  BOC"}=" H13 BOC"; $resn_exch{"1H2  BOC"}=" H21 BOC";
$resn_exch{"2H2  BOC"}=" H22 BOC"; $resn_exch{"3H2  BOC"}=" H23 BOC"; $resn_exch{"1H3  BOC"}=" H31 BOC";
$resn_exch{"2H3  BOC"}=" H32 BOC"; $resn_exch{"3H3  BOC"}=" H33 BOC"; $resn_exch{"2HOP BOE"}="HOP2 BOE";
$resn_exch{"1H5* BOE"}=" H5' BOE"; $resn_exch{"2H5* BOE"}="H5'' BOE"; $resn_exch{" H4* BOE"}=" H4' BOE";
$resn_exch{" H1* BOE"}=" H1' BOE"; $resn_exch{"1H5M BOE"}=" H71 BOE"; $resn_exch{"2H5M BOE"}=" H72 BOE";
$resn_exch{"3H5M BOE"}=" H73 BOE"; $resn_exch{" H2* BOE"}=" H2' BOE"; $resn_exch{"1HB' BOE"}="HB'1 BOE";
$resn_exch{"2HB' BOE"}="HB'2 BOE"; $resn_exch{"1HC' BOE"}="HC'1 BOE"; $resn_exch{"2HC' BOE"}="HC'2 BOE";
$resn_exch{"1HE' BOE"}="HE'1 BOE"; $resn_exch{"2HE' BOE"}="HE'2 BOE"; $resn_exch{" H3* BOE"}=" H3' BOE";
$resn_exch{"H3T  BOE"}="HO3' BOE"; $resn_exch{"3HOP BOE"}="HOP3 BOE"; $resn_exch{"1H1' BOG"}="H1'1 BOG";
$resn_exch{"2H1' BOG"}="H1'2 BOG"; $resn_exch{"1H2' BOG"}="H2'1 BOG"; $resn_exch{"2H2' BOG"}="H2'2 BOG";
$resn_exch{"1H3' BOG"}="H3'1 BOG"; $resn_exch{"2H3' BOG"}="H3'2 BOG"; $resn_exch{"1H4' BOG"}="H4'1 BOG";
$resn_exch{"2H4' BOG"}="H4'2 BOG"; $resn_exch{"1H5' BOG"}="H5'1 BOG"; $resn_exch{"2H5' BOG"}="H5'2 BOG";
$resn_exch{"1H6' BOG"}="H6'1 BOG"; $resn_exch{"2H6' BOG"}="H6'2 BOG"; $resn_exch{"1H7' BOG"}="H7'1 BOG";
$resn_exch{"2H7' BOG"}="H7'2 BOG"; $resn_exch{"1H8' BOG"}="H8'1 BOG"; $resn_exch{"2H8' BOG"}="H8'2 BOG";
$resn_exch{"3H8' BOG"}="H8'3 BOG"; $resn_exch{"1H2  BOM"}=" H21 BOM"; $resn_exch{"2H2  BOM"}=" H22 BOM";
$resn_exch{"1H4  BOM"}=" H41 BOM"; $resn_exch{"2H4  BOM"}=" H42 BOM"; $resn_exch{"1H7  BOM"}=" H71 BOM";
$resn_exch{"2H7  BOM"}=" H72 BOM"; $resn_exch{"1H10 BOM"}="H101 BOM"; $resn_exch{"2H10 BOM"}="H102 BOM";
$resn_exch{"1H13 BOM"}="H131 BOM"; $resn_exch{"2H13 BOM"}="H132 BOM"; $resn_exch{"1H16 BOM"}="H161 BOM";
$resn_exch{"2H16 BOM"}="H162 BOM"; $resn_exch{"1H19 BOM"}="H191 BOM"; $resn_exch{"2H19 BOM"}="H192 BOM";
$resn_exch{"1H22 BOM"}="H221 BOM"; $resn_exch{"2H22 BOM"}="H222 BOM"; $resn_exch{"1H25 BOM"}="H251 BOM";
$resn_exch{"2H25 BOM"}="H252 BOM"; $resn_exch{"1H28 BOM"}="H281 BOM"; $resn_exch{"1H31 BOM"}="H311 BOM";
$resn_exch{"1H33 BOM"}="H331 BOM"; $resn_exch{"1H35 BOM"}="H351 BOM"; $resn_exch{"1H37 BOM"}="H371 BOM";
$resn_exch{"2H37 BOM"}="H372 BOM"; $resn_exch{"1H39 BOM"}="H391 BOM"; $resn_exch{"2H39 BOM"}="H392 BOM";
$resn_exch{"1H42 BOM"}="H421 BOM"; $resn_exch{"2H42 BOM"}="H422 BOM"; $resn_exch{"3H42 BOM"}="H423 BOM";
$resn_exch{"1H7  BOP"}=" H71 BOP"; $resn_exch{"2H7  BOP"}=" H72 BOP"; $resn_exch{"3H7  BOP"}=" H73 BOP";
$resn_exch{"1HN  BOR"}=" HN1 BOR"; $resn_exch{"2HN  BOR"}=" HN2 BOR"; $resn_exch{"1HB  BOR"}=" HB1 BOR";
$resn_exch{"2HB  BOR"}=" HB2 BOR"; $resn_exch{"1HG  BOR"}=" HG1 BOR"; $resn_exch{"2HG  BOR"}=" HG2 BOR";
$resn_exch{"1HD  BOR"}=" HD1 BOR"; $resn_exch{"2HD  BOR"}=" HD2 BOR"; $resn_exch{"1HH2 BOR"}="HH21 BOR";
$resn_exch{"2HH2 BOR"}="HH22 BOR"; $resn_exch{"1HN1 BOS"}="HN11 BOS"; $resn_exch{"2HN1 BOS"}="HN12 BOS";
$resn_exch{"1HN1 BOZ"}="HN11 BOZ"; $resn_exch{"2HN1 BOZ"}="HN12 BOZ"; $resn_exch{"1HN2 BOZ"}="HN21 BOZ";
$resn_exch{"2HN2 BOZ"}="HN22 BOZ"; $resn_exch{"1H1' BOZ"}="H1'1 BOZ"; $resn_exch{"2H1' BOZ"}="H1'2 BOZ";
$resn_exch{"1H2' BOZ"}="H2'1 BOZ"; $resn_exch{"2H2' BOZ"}="H2'2 BOZ"; $resn_exch{"'HN4 BOZ"}="HN4' BOZ";
$resn_exch{"1H10  BP"}="H101  BP"; $resn_exch{"2H10  BP"}="H102  BP"; $resn_exch{"1HC4 BP2"}="HC41 BP2";
$resn_exch{"2HC4 BP2"}="HC42 BP2"; $resn_exch{"1HC6 BP2"}="HC61 BP2"; $resn_exch{"2HC6 BP2"}="HC62 BP2";
$resn_exch{"1HC7 BP2"}="HC71 BP2"; $resn_exch{"2HC7 BP2"}="HC72 BP2"; $resn_exch{"1HC9 BP2"}="HC91 BP2";
$resn_exch{"2HC9 BP2"}="HC92 BP2"; $resn_exch{"3HC9 BP2"}="HC93 BP2"; $resn_exch{"1H10 BP2"}="H101 BP2";
$resn_exch{"2H10 BP2"}="H102 BP2"; $resn_exch{"3H10 BP2"}="H103 BP2"; $resn_exch{"1H11 BP2"}="H111 BP2";
$resn_exch{"2H11 BP2"}="H112 BP2"; $resn_exch{"3H11 BP2"}="H113 BP2"; $resn_exch{"1HA  BP4"}=" HA1 BP4";
$resn_exch{"2HA  BP4"}=" HA2 BP4"; $resn_exch{"1H12 BP5"}="H121 BP5"; $resn_exch{"2H12 BP5"}="H122 BP5";
$resn_exch{"1HN  BP5"}=" HN1 BP5"; $resn_exch{"2HN  BP5"}=" HN2 BP5"; $resn_exch{"1H10 BPA"}="H101 BPA";
$resn_exch{"2H10 BPA"}="H102 BPA"; $resn_exch{"1HMA BPB"}="HMA1 BPB"; $resn_exch{"2HMA BPB"}="HMA2 BPB";
$resn_exch{"3HMA BPB"}="HMA3 BPB"; $resn_exch{"1HAA BPB"}="HAA1 BPB"; $resn_exch{"2HAA BPB"}="HAA2 BPB";
$resn_exch{"1HBA BPB"}="HBA1 BPB"; $resn_exch{"2HBA BPB"}="HBA2 BPB"; $resn_exch{"1HMB BPB"}="HMB1 BPB";
$resn_exch{"2HMB BPB"}="HMB2 BPB"; $resn_exch{"3HMB BPB"}="HMB3 BPB"; $resn_exch{"1HBB BPB"}="HBB1 BPB";
$resn_exch{"2HBB BPB"}="HBB2 BPB"; $resn_exch{"3HBB BPB"}="HBB3 BPB"; $resn_exch{"1HMC BPB"}="HMC1 BPB";
$resn_exch{"2HMC BPB"}="HMC2 BPB"; $resn_exch{"3HMC BPB"}="HMC3 BPB"; $resn_exch{"1HAC BPB"}="HAC1 BPB";
$resn_exch{"2HAC BPB"}="HAC2 BPB"; $resn_exch{"1HBC BPB"}="HBC1 BPB"; $resn_exch{"2HBC BPB"}="HBC2 BPB";
$resn_exch{"3HBC BPB"}="HBC3 BPB"; $resn_exch{"1HMD BPB"}="HMD1 BPB"; $resn_exch{"2HMD BPB"}="HMD2 BPB";
$resn_exch{"3HMD BPB"}="HMD3 BPB"; $resn_exch{"1HED BPB"}="HED1 BPB"; $resn_exch{"2HED BPB"}="HED2 BPB";
$resn_exch{"3HED BPB"}="HED3 BPB"; $resn_exch{"1H1  BPB"}=" H11 BPB"; $resn_exch{"2H1  BPB"}=" H12 BPB";
$resn_exch{"1H4  BPB"}=" H41 BPB"; $resn_exch{"2H4  BPB"}=" H42 BPB"; $resn_exch{"3H4  BPB"}=" H43 BPB";
$resn_exch{"1H5  BPB"}=" H51 BPB"; $resn_exch{"2H5  BPB"}=" H52 BPB"; $resn_exch{"1H6  BPB"}=" H61 BPB";
$resn_exch{"2H6  BPB"}=" H62 BPB"; $resn_exch{"1H7  BPB"}=" H71 BPB"; $resn_exch{"2H7  BPB"}=" H72 BPB";
$resn_exch{"1H9  BPB"}=" H91 BPB"; $resn_exch{"2H9  BPB"}=" H92 BPB"; $resn_exch{"3H9  BPB"}=" H93 BPB";
$resn_exch{"1H10 BPB"}="H101 BPB"; $resn_exch{"2H10 BPB"}="H102 BPB"; $resn_exch{"1H11 BPB"}="H111 BPB";
$resn_exch{"2H11 BPB"}="H112 BPB"; $resn_exch{"1H12 BPB"}="H121 BPB"; $resn_exch{"2H12 BPB"}="H122 BPB";
$resn_exch{"1H14 BPB"}="H141 BPB"; $resn_exch{"2H14 BPB"}="H142 BPB"; $resn_exch{"3H14 BPB"}="H143 BPB";
$resn_exch{"1H15 BPB"}="H151 BPB"; $resn_exch{"2H15 BPB"}="H152 BPB"; $resn_exch{"1H16 BPB"}="H161 BPB";
$resn_exch{"2H16 BPB"}="H162 BPB"; $resn_exch{"1H17 BPB"}="H171 BPB"; $resn_exch{"2H17 BPB"}="H172 BPB";
$resn_exch{"1H19 BPB"}="H191 BPB"; $resn_exch{"2H19 BPB"}="H192 BPB"; $resn_exch{"3H19 BPB"}="H193 BPB";
$resn_exch{"1H20 BPB"}="H201 BPB"; $resn_exch{"2H20 BPB"}="H202 BPB"; $resn_exch{"3H20 BPB"}="H203 BPB";
$resn_exch{"1H4' BPC"}="H4'1 BPC"; $resn_exch{"2H4' BPC"}="H4'2 BPC"; $resn_exch{"1HN  BPE"}=" HN1 BPE";
$resn_exch{"2HN  BPE"}=" HN2 BPE"; $resn_exch{"1HB  BPE"}=" HB1 BPE"; $resn_exch{"2HB  BPE"}=" HB2 BPE";
$resn_exch{"1H05 BPE"}="H051 BPE"; $resn_exch{"2H05 BPE"}="H052 BPE"; $resn_exch{"1H   BPE"}=" H1  BPE";
$resn_exch{"2H   BPE"}=" H2  BPE"; $resn_exch{"1H08 BPE"}="H081 BPE"; $resn_exch{"2H08 BPE"}="H082 BPE";
$resn_exch{"1H09 BPE"}="H091 BPE"; $resn_exch{"2H09 BPE"}="H092 BPE"; $resn_exch{"1H10 BPE"}="H101 BPE";
$resn_exch{"2H10 BPE"}="H102 BPE"; $resn_exch{"1H9  BPF"}=" H91 BPF"; $resn_exch{"2H9  BPF"}=" H92 BPF";
$resn_exch{"3H9  BPF"}=" H93 BPF"; $resn_exch{"1HN1 BPF"}="HN11 BPF"; $resn_exch{"2HN1 BPF"}="HN12 BPF";
$resn_exch{"1H8' BPF"}="H8'1 BPF"; $resn_exch{"2H8' BPF"}="H8'2 BPF"; $resn_exch{"1H9' BPF"}="H9'1 BPF";
$resn_exch{"2H9' BPF"}="H9'2 BPF"; $resn_exch{"3H9' BPF"}="H9'3 BPF"; $resn_exch{"1H3* BPG"}="H3'1 BPG";
$resn_exch{"2H3* BPG"}="H3'2 BPG"; $resn_exch{" H3* BPG"}=" H3' BPG"; $resn_exch{"1H2* BPG"}="H2'1 BPG";
$resn_exch{"2H2* BPG"}="H2'2 BPG"; $resn_exch{"1H12 BPG"}="H121 BPG"; $resn_exch{"2H12 BPG"}="H122 BPG";
$resn_exch{"1H1* BPG"}="H1'1 BPG"; $resn_exch{"2H1* BPG"}="H1'2 BPG"; $resn_exch{"1HN2 BPG"}="HN21 BPG";
$resn_exch{"1H04 BPG"}="H041 BPG"; $resn_exch{"1H05 BPG"}="H051 BPG"; $resn_exch{"1H06 BPG"}="H061 BPG";
$resn_exch{"1H02 BPG"}="H021 BPG"; $resn_exch{"1H03 BPG"}="H031 BPG"; $resn_exch{"1HMA BPH"}="HMA1 BPH";
$resn_exch{"2HMA BPH"}="HMA2 BPH"; $resn_exch{"3HMA BPH"}="HMA3 BPH"; $resn_exch{"1HAA BPH"}="HAA1 BPH";
$resn_exch{"2HAA BPH"}="HAA2 BPH"; $resn_exch{"1HBA BPH"}="HBA1 BPH"; $resn_exch{"2HBA BPH"}="HBA2 BPH";
$resn_exch{"1HMB BPH"}="HMB1 BPH"; $resn_exch{"2HMB BPH"}="HMB2 BPH"; $resn_exch{"3HMB BPH"}="HMB3 BPH";
$resn_exch{"1HBB BPH"}="HBB1 BPH"; $resn_exch{"2HBB BPH"}="HBB2 BPH"; $resn_exch{"3HBB BPH"}="HBB3 BPH";
$resn_exch{"1HMC BPH"}="HMC1 BPH"; $resn_exch{"2HMC BPH"}="HMC2 BPH"; $resn_exch{"3HMC BPH"}="HMC3 BPH";
$resn_exch{"1HAC BPH"}="HAC1 BPH"; $resn_exch{"2HAC BPH"}="HAC2 BPH"; $resn_exch{"1HBC BPH"}="HBC1 BPH";
$resn_exch{"2HBC BPH"}="HBC2 BPH"; $resn_exch{"3HBC BPH"}="HBC3 BPH"; $resn_exch{"1HMD BPH"}="HMD1 BPH";
$resn_exch{"2HMD BPH"}="HMD2 BPH"; $resn_exch{"3HMD BPH"}="HMD3 BPH"; $resn_exch{"1HED BPH"}="HED1 BPH";
$resn_exch{"2HED BPH"}="HED2 BPH"; $resn_exch{"3HED BPH"}="HED3 BPH"; $resn_exch{"1H1  BPH"}=" H11 BPH";
$resn_exch{"2H1  BPH"}=" H12 BPH"; $resn_exch{"1H4  BPH"}=" H41 BPH"; $resn_exch{"2H4  BPH"}=" H42 BPH";
$resn_exch{"3H4  BPH"}=" H43 BPH"; $resn_exch{"1H5  BPH"}=" H51 BPH"; $resn_exch{"2H5  BPH"}=" H52 BPH";
$resn_exch{"1H6  BPH"}=" H61 BPH"; $resn_exch{"2H6  BPH"}=" H62 BPH"; $resn_exch{"1H7  BPH"}=" H71 BPH";
$resn_exch{"2H7  BPH"}=" H72 BPH"; $resn_exch{"1H9  BPH"}=" H91 BPH"; $resn_exch{"2H9  BPH"}=" H92 BPH";
$resn_exch{"3H9  BPH"}=" H93 BPH"; $resn_exch{"1H10 BPH"}="H101 BPH"; $resn_exch{"2H10 BPH"}="H102 BPH";
$resn_exch{"1H11 BPH"}="H111 BPH"; $resn_exch{"2H11 BPH"}="H112 BPH"; $resn_exch{"1H12 BPH"}="H121 BPH";
$resn_exch{"2H12 BPH"}="H122 BPH"; $resn_exch{"1H14 BPH"}="H141 BPH"; $resn_exch{"2H14 BPH"}="H142 BPH";
$resn_exch{"3H14 BPH"}="H143 BPH"; $resn_exch{"1H15 BPH"}="H151 BPH"; $resn_exch{"2H15 BPH"}="H152 BPH";
$resn_exch{"1H16 BPH"}="H161 BPH"; $resn_exch{"2H16 BPH"}="H162 BPH"; $resn_exch{"1H17 BPH"}="H171 BPH";
$resn_exch{"2H17 BPH"}="H172 BPH"; $resn_exch{"1H19 BPH"}="H191 BPH"; $resn_exch{"2H19 BPH"}="H192 BPH";
$resn_exch{"3H19 BPH"}="H193 BPH"; $resn_exch{"1H20 BPH"}="H201 BPH"; $resn_exch{"2H20 BPH"}="H202 BPH";
$resn_exch{"3H20 BPH"}="H203 BPH"; $resn_exch{"1H1  BPI"}="H11A BPI"; $resn_exch{"2H1  BPI"}="H12A BPI";
$resn_exch{"2HO  BPI"}=" HO2 BPI"; $resn_exch{"3HO  BPI"}=" HO3 BPI"; $resn_exch{"4HO  BPI"}=" HO4 BPI";
$resn_exch{"1H1  BPJ"}="H11A BPJ"; $resn_exch{"2H1  BPJ"}="H12A BPJ"; $resn_exch{"2HO  BPJ"}=" HO2 BPJ";
$resn_exch{"3HO  BPJ"}=" HO3 BPJ"; $resn_exch{"4HO  BPJ"}=" HO4 BPJ"; $resn_exch{"1HC0 BPM"}="HC01 BPM";
$resn_exch{"2HC0 BPM"}="HC02 BPM"; $resn_exch{"3HOA BPM"}="HOA3 BPM"; $resn_exch{"4HOA BPM"}="HOA4 BPM";
$resn_exch{"3HOB BPM"}="HOB3 BPM"; $resn_exch{"4HOB BPM"}="HOB4 BPM"; $resn_exch{"1H7  BPN"}=" H71 BPN";
$resn_exch{"2H7  BPN"}=" H72 BPN"; $resn_exch{"1HN1 BPO"}="HN11 BPO"; $resn_exch{"2HN1 BPO"}="HN12 BPO";
$resn_exch{"1H17 BPO"}="H171 BPO"; $resn_exch{"2H17 BPO"}="H172 BPO"; $resn_exch{"1H16 BPO"}="H161 BPO";
$resn_exch{"2H16 BPO"}="H162 BPO"; $resn_exch{"1H20 BPO"}="H201 BPO"; $resn_exch{"2H20 BPO"}="H202 BPO";
$resn_exch{"1H21 BPO"}="H211 BPO"; $resn_exch{"2H21 BPO"}="H212 BPO"; $resn_exch{"1H14 BPP"}="H141 BPP";
$resn_exch{"2H14 BPP"}="H142 BPP"; $resn_exch{"1H15 BPP"}="H151 BPP"; $resn_exch{"2H15 BPP"}="H152 BPP";
$resn_exch{"1H20 BPP"}="H201 BPP"; $resn_exch{"2H20 BPP"}="H202 BPP"; $resn_exch{"1H21 BPP"}="H211 BPP";
$resn_exch{"2H21 BPP"}="H212 BPP"; $resn_exch{"1H23 BPP"}="H231 BPP"; $resn_exch{"2H23 BPP"}="H232 BPP";
$resn_exch{"1H24 BPP"}="H241 BPP"; $resn_exch{"2H24 BPP"}="H242 BPP"; $resn_exch{"1H33 BPP"}="H331 BPP";
$resn_exch{"2H33 BPP"}="H332 BPP"; $resn_exch{"1HB  BPR"}=" HB1 BPR"; $resn_exch{"2HB  BPR"}=" HB2 BPR";
$resn_exch{"1HG  BPR"}=" HG1 BPR"; $resn_exch{"2HG  BPR"}=" HG2 BPR"; $resn_exch{"1HD  BPR"}=" HD1 BPR";
$resn_exch{"2HD  BPR"}=" HD2 BPR"; $resn_exch{"1H1  BPR"}=" H11 BPR"; $resn_exch{"2H1  BPR"}=" H12 BPR";
$resn_exch{"1H3  BPR"}=" H31 BPR"; $resn_exch{"2H3  BPR"}=" H32 BPR"; $resn_exch{"1H4  BPR"}=" H41 BPR";
$resn_exch{"2H4  BPR"}=" H42 BPR"; $resn_exch{"1HN1 BPT"}="HN11 BPT"; $resn_exch{"2HN1 BPT"}="HN12 BPT";
$resn_exch{"1HC1 BPT"}="HC11 BPT"; $resn_exch{"2HC1 BPT"}="HC12 BPT"; $resn_exch{"1HC2 BPT"}="HC21 BPT";
$resn_exch{"2HC2 BPT"}="HC22 BPT"; $resn_exch{"1HC3 BPT"}="HC31 BPT"; $resn_exch{"2HC3 BPT"}="HC32 BPT";
$resn_exch{"1HC4 BPT"}="HC41 BPT"; $resn_exch{"2HC4 BPT"}="HC42 BPT"; $resn_exch{"1HN2 BPT"}="HN21 BPT";
$resn_exch{"2HN2 BPT"}="HN22 BPT"; $resn_exch{"1HN3 BPT"}="HN31 BPT"; $resn_exch{"2HN3 BPT"}="HN32 BPT";
$resn_exch{"3HN3 BPT"}="HN33 BPT"; $resn_exch{"1HN4 BPT"}="HN41 BPT"; $resn_exch{"2HN4 BPT"}="HN42 BPT";
$resn_exch{"3HN4 BPT"}="HN43 BPT"; $resn_exch{"1HN5 BPT"}="HN51 BPT"; $resn_exch{"2HN5 BPT"}="HN52 BPT";
$resn_exch{"3HN5 BPT"}="HN53 BPT"; $resn_exch{"1HN6 BPT"}="HN61 BPT"; $resn_exch{"2HN6 BPT"}="HN62 BPT";
$resn_exch{"3HN6 BPT"}="HN63 BPT"; $resn_exch{"1HA2 BR3"}="HA21 BR3"; $resn_exch{"2HA2 BR3"}="HA22 BR3";
$resn_exch{"1HZ3 BR3"}="HZ31 BR3"; $resn_exch{"2HZ3 BR3"}="HZ32 BR3"; $resn_exch{"1HE3 BR3"}="HE31 BR3";
$resn_exch{"2HE3 BR3"}="HE32 BR3"; $resn_exch{"1HH2 BR3"}="HH21 BR3"; $resn_exch{"2HH2 BR3"}="HH22 BR3";
$resn_exch{"3HH2 BR3"}="HH23 BR3"; $resn_exch{"1H9  BR4"}=" H91 BR4"; $resn_exch{"2H9  BR4"}=" H92 BR4";
$resn_exch{"1H8  BR4"}=" H81 BR4"; $resn_exch{"2H8  BR4"}=" H82 BR4"; $resn_exch{"1H7  BR4"}=" H71 BR4";
$resn_exch{"2H7  BR4"}=" H72 BR4"; $resn_exch{"1H1  BR4"}=" H11 BR4"; $resn_exch{"1H5  BR4"}=" H51 BR4";
$resn_exch{"1H2  BR4"}=" H21 BR4"; $resn_exch{"1H4  BR4"}=" H41 BR4"; $resn_exch{"1H3  BR4"}=" H31 BR4";
$resn_exch{"1H12 BR4"}="H121 BR4"; $resn_exch{"2H12 BR4"}="H122 BR4"; $resn_exch{"1H11 BR4"}="H111 BR4";
$resn_exch{"2H11 BR4"}="H112 BR4"; $resn_exch{"1H10 BR4"}="H101 BR4"; $resn_exch{"2H10 BR4"}="H102 BR4";
$resn_exch{"1HN1 BR4"}="HN11 BR4"; $resn_exch{"1H23 BR4"}="H231 BR4"; $resn_exch{"2H23 BR4"}="H232 BR4";
$resn_exch{"1H14 BR4"}="H141 BR4"; $resn_exch{"1H15 BR4"}="H151 BR4"; $resn_exch{"2H15 BR4"}="H152 BR4";
$resn_exch{"1H16 BR4"}="H161 BR4"; $resn_exch{"2H16 BR4"}="H162 BR4"; $resn_exch{"1H18 BR4"}="H181 BR4";
$resn_exch{"1H22 BR4"}="H221 BR4"; $resn_exch{"1H19 BR4"}="H191 BR4"; $resn_exch{"1H21 BR4"}="H211 BR4";
$resn_exch{"1H20 BR4"}="H201 BR4"; $resn_exch{"1H24 BR4"}="H241 BR4"; $resn_exch{"2H24 BR4"}="H242 BR4";
$resn_exch{"1HO3 BR4"}="HO31 BR4"; $resn_exch{"1HA1 BR5"}="HA11 BR5"; $resn_exch{"2HA1 BR5"}="HA12 BR5";
$resn_exch{"3HA1 BR5"}="HA13 BR5"; $resn_exch{"1HB2 BR5"}="HB21 BR5"; $resn_exch{"2HB2 BR5"}="HB22 BR5";
$resn_exch{"1HZ2 BR5"}="HZ21 BR5"; $resn_exch{"2HZ2 BR5"}="HZ22 BR5"; $resn_exch{"1HA2 BR5"}="HA21 BR5";
$resn_exch{"2HA2 BR5"}="HA22 BR5"; $resn_exch{"1H7  BRB"}=" H71 BRB"; $resn_exch{"2H7  BRB"}=" H72 BRB";
$resn_exch{"1H2  BRC"}=" H21 BRC"; $resn_exch{"2H2  BRC"}=" H22 BRC"; $resn_exch{"1H3  BRC"}=" H31 BRC";
$resn_exch{"2H3  BRC"}=" H32 BRC"; $resn_exch{"1H4  BRC"}=" H41 BRC"; $resn_exch{"2H4  BRC"}=" H42 BRC";
$resn_exch{"1H5  BRC"}=" H51 BRC"; $resn_exch{"2H5  BRC"}=" H52 BRC"; $resn_exch{"1H6  BRC"}=" H61 BRC";
$resn_exch{"2H6  BRC"}=" H62 BRC"; $resn_exch{"1H7  BRC"}=" H71 BRC"; $resn_exch{"2H7  BRC"}=" H72 BRC";
$resn_exch{"1H8  BRC"}=" H81 BRC"; $resn_exch{"2H8  BRC"}=" H82 BRC"; $resn_exch{"1H9  BRC"}=" H91 BRC";
$resn_exch{"2H9  BRC"}=" H92 BRC"; $resn_exch{"1H10 BRC"}="H101 BRC"; $resn_exch{"2H10 BRC"}="H102 BRC";
$resn_exch{"1H11 BRC"}="H111 BRC"; $resn_exch{"2H11 BRC"}="H112 BRC"; $resn_exch{"1H12 BRC"}="H121 BRC";
$resn_exch{"2H12 BRC"}="H122 BRC"; $resn_exch{"1H3' BRD"}="H3'1 BRD"; $resn_exch{"1H2' BRD"}="H2'1 BRD";
$resn_exch{"1H5' BRD"}="H5'1 BRD"; $resn_exch{"2H5' BRD"}="H5'2 BRD"; $resn_exch{"1H1  BRE"}=" H11 BRE";
$resn_exch{"2H1  BRE"}=" H12 BRE"; $resn_exch{"3H1  BRE"}=" H13 BRE"; $resn_exch{"1H8  BRE"}=" H81 BRE";
$resn_exch{"1H9  BRE"}=" H91 BRE"; $resn_exch{"1H10 BRE"}="H101 BRE"; $resn_exch{"1H11 BRE"}="H111 BRE";
$resn_exch{"1H12 BRE"}="H121 BRE"; $resn_exch{"1H13 BRE"}="H131 BRE"; $resn_exch{"1H15 BRE"}="H151 BRE";
$resn_exch{"1H18 BRE"}="H181 BRE"; $resn_exch{"1H19 BRE"}="H191 BRE"; $resn_exch{"1H20 BRE"}="H201 BRE";
$resn_exch{"1H21 BRE"}="H211 BRE"; $resn_exch{"1H22 BRE"}="H221 BRE"; $resn_exch{"1H1C BRF"}="H1C1 BRF";
$resn_exch{"2H1C BRF"}="H1C2 BRF"; $resn_exch{"3H1C BRF"}="H1C3 BRF"; $resn_exch{"1H2  BRG"}=" H21 BRG";
$resn_exch{"2H2  BRG"}=" H22 BRG"; $resn_exch{"1H2  BRI"}=" H21 BRI"; $resn_exch{"2H2  BRI"}=" H22 BRI";
$resn_exch{"1H4  BRI"}=" H41 BRI"; $resn_exch{"2H4  BRI"}=" H42 BRI"; $resn_exch{"3H4  BRI"}=" H43 BRI";
$resn_exch{"1H6  BRI"}=" H61 BRI"; $resn_exch{"2H6  BRI"}=" H62 BRI"; $resn_exch{"3H6  BRI"}=" H63 BRI";
$resn_exch{"1HB1 BRJ"}="HB11 BRJ"; $resn_exch{"2HB1 BRJ"}="HB12 BRJ"; $resn_exch{"1HB  BRJ"}="HB1A BRJ";
$resn_exch{"2HB  BRJ"}=" HB2 BRJ"; $resn_exch{"1HAB BRK"}="HAB1 BRK"; $resn_exch{"2HAB BRK"}="HAB2 BRK";
$resn_exch{"3HAB BRK"}="HAB3 BRK"; $resn_exch{"1HAC BRK"}="HAC1 BRK"; $resn_exch{"2HAC BRK"}="HAC2 BRK";
$resn_exch{"1HAA BRK"}="HAA1 BRK"; $resn_exch{"2HAA BRK"}="HAA2 BRK"; $resn_exch{"3HAA BRK"}="HAA3 BRK";
$resn_exch{"1H6  BRL"}=" H61 BRL"; $resn_exch{"2H6  BRL"}=" H62 BRL"; $resn_exch{"1H14 BRL"}="H141 BRL";
$resn_exch{"2H14 BRL"}="H142 BRL"; $resn_exch{"1H15 BRL"}="H151 BRL"; $resn_exch{"2H15 BRL"}="H152 BRL";
$resn_exch{"1H16 BRL"}="H161 BRL"; $resn_exch{"2H16 BRL"}="H162 BRL"; $resn_exch{"3H16 BRL"}="H163 BRL";
$resn_exch{"1H2  BRM"}=" H21 BRM"; $resn_exch{"2H2  BRM"}=" H22 BRM"; $resn_exch{"1HB  BRN"}=" HB1 BRN";
$resn_exch{"2HB  BRN"}=" HB2 BRN"; $resn_exch{"1HB' BRN"}="HB'1 BRN"; $resn_exch{"2HB' BRN"}="HB'2 BRN";
$resn_exch{"1H1  BRP"}=" H11 BRP"; $resn_exch{"2H1  BRP"}=" H12 BRP"; $resn_exch{"1H3  BRP"}=" H31 BRP";
$resn_exch{"2H3  BRP"}=" H32 BRP"; $resn_exch{"1H   BRQ"}=" H1  BRQ"; $resn_exch{"2H   BRQ"}=" H2  BRQ";
$resn_exch{"1H10 BRR"}="H101 BRR"; $resn_exch{"2H10 BRR"}="H102 BRR"; $resn_exch{"3H10 BRR"}="H103 BRR";
$resn_exch{"1H9  BRR"}=" H91 BRR"; $resn_exch{"2H9  BRR"}=" H92 BRR"; $resn_exch{"1H8  BRR"}=" H81 BRR";
$resn_exch{"2H8  BRR"}=" H82 BRR"; $resn_exch{"1H7  BRR"}=" H71 BRR"; $resn_exch{"2H7  BRR"}=" H72 BRR";
$resn_exch{"1H5  BRR"}=" H51 BRR"; $resn_exch{"2H5  BRR"}=" H52 BRR"; $resn_exch{"1H22 BRS"}="H221 BRS";
$resn_exch{"2H22 BRS"}="H222 BRS"; $resn_exch{"3H22 BRS"}="H223 BRS"; $resn_exch{"1H8N BRT"}="H8N1 BRT";
$resn_exch{"2H8N BRT"}="H8N2 BRT"; $resn_exch{" H1* BRU"}=" H1' BRU"; $resn_exch{"1H2* BRU"}=" H2' BRU";
$resn_exch{"2H2* BRU"}="H2'' BRU"; $resn_exch{" H3* BRU"}=" H3' BRU"; $resn_exch{" H4* BRU"}=" H4' BRU";
$resn_exch{"*HO3 BRU"}="HO3' BRU"; $resn_exch{"1H5* BRU"}=" H5' BRU"; $resn_exch{"2H5* BRU"}="H5'' BRU";
$resn_exch{"2HOP BRU"}="HOP2 BRU"; $resn_exch{"3HOP BRU"}="HOP3 BRU"; $resn_exch{"1HC1 BRZ"}="HC11 BRZ";
$resn_exch{"2HC1 BRZ"}="HC1A BRZ"; $resn_exch{"1HC6 BRZ"}="HC61 BRZ"; $resn_exch{"2HC6 BRZ"}="HC62 BRZ";
$resn_exch{"3HC6 BRZ"}="HC63 BRZ"; $resn_exch{"1HC7 BRZ"}="HC71 BRZ"; $resn_exch{"2HC7 BRZ"}="HC72 BRZ";
$resn_exch{"3HC7 BRZ"}="HC73 BRZ"; $resn_exch{"1HC9 BRZ"}="HC91 BRZ"; $resn_exch{"2HC9 BRZ"}="HC92 BRZ";
$resn_exch{"1HC2 BRZ"}="HC21 BRZ"; $resn_exch{"2HC2 BRZ"}="HC22 BRZ"; $resn_exch{"3HC2 BRZ"}="HC23 BRZ";
$resn_exch{"1HC3 BRZ"}="HC31 BRZ"; $resn_exch{"2HC3 BRZ"}="HC32 BRZ"; $resn_exch{"3HC3 BRZ"}="HC33 BRZ";
$resn_exch{"1H7  BSA"}=" H71 BSA"; $resn_exch{"2H7  BSA"}=" H72 BSA"; $resn_exch{"1H10 BSA"}="H101 BSA";
$resn_exch{"2H10 BSA"}="H102 BSA"; $resn_exch{"CH01 BSB"}="H01C BSB"; $resn_exch{"CH02 BSB"}="H02C BSB";
$resn_exch{"CH04 BSB"}="H04C BSB"; $resn_exch{"CH05 BSB"}="H05C BSB"; $resn_exch{"NH09 BSB"}="H09N BSB";
$resn_exch{"1H10 BSB"}="H101 BSB"; $resn_exch{"2H10 BSB"}="H102 BSB"; $resn_exch{"1HNP BSB"}="HNP1 BSB";
$resn_exch{"2HNP BSB"}="HNP2 BSB"; $resn_exch{"CH16 BSB"}="H16C BSB"; $resn_exch{"CH17 BSB"}="H17C BSB";
$resn_exch{"CH18 BSB"}="H18C BSB"; $resn_exch{"CH19 BSB"}="H19C BSB"; $resn_exch{"CH20 BSB"}="H20C BSB";
$resn_exch{"1HAB BSC"}="HAB1 BSC"; $resn_exch{"2HAB BSC"}="HAB2 BSC"; $resn_exch{"3HAB BSC"}="HAB3 BSC";
$resn_exch{"1HAC BSC"}="HAC1 BSC"; $resn_exch{"2HAC BSC"}="HAC2 BSC"; $resn_exch{"1HAD BSC"}="HAD1 BSC";
$resn_exch{"2HAD BSC"}="HAD2 BSC"; $resn_exch{"1HAE BSC"}="HAE1 BSC"; $resn_exch{"2HAE BSC"}="HAE2 BSC";
$resn_exch{"1HAA BSC"}="HAA1 BSC"; $resn_exch{"2HAA BSC"}="HAA2 BSC"; $resn_exch{"1HAH BSC"}="HAH1 BSC";
$resn_exch{"2HAH BSC"}="HAH2 BSC"; $resn_exch{"1HB  BSC"}=" HB1 BSC"; $resn_exch{"2HB  BSC"}=" HB2 BSC";
$resn_exch{"HOXT BSC"}=" HXT BSC"; $resn_exch{"1HN  BSC"}=" HN1 BSC"; $resn_exch{"2HN  BSC"}=" HN2 BSC";
$resn_exch{" HN2 BSE"}=" H2  BSE"; $resn_exch{"1HC  BSE"}=" HC1 BSE"; $resn_exch{"2HC  BSE"}=" HC2 BSE";
$resn_exch{"1HB  BSE"}=" HB2 BSE"; $resn_exch{"2HB  BSE"}=" HB3 BSE"; $resn_exch{"1HC7 BSI"}="HC71 BSI";
$resn_exch{"2HC7 BSI"}="HC72 BSI"; $resn_exch{"1HC0 BSI"}="HC01 BSI"; $resn_exch{"2HC0 BSI"}="HC02 BSI";
$resn_exch{"1H53 BSJ"}="H531 BSJ"; $resn_exch{"2H53 BSJ"}="H532 BSJ"; $resn_exch{"1H63 BSJ"}="H631 BSJ";
$resn_exch{"2H63 BSJ"}="H632 BSJ"; $resn_exch{"1H52 BSJ"}="H521 BSJ"; $resn_exch{"2H52 BSJ"}="H522 BSJ";
$resn_exch{"1H50 BSJ"}="H501 BSJ"; $resn_exch{"2H50 BSJ"}="H502 BSJ"; $resn_exch{"1H58 BSJ"}="H581 BSJ";
$resn_exch{"2H58 BSJ"}="H582 BSJ"; $resn_exch{"1HA  BSJ"}=" HA1 BSJ"; $resn_exch{"2HB  BSJ"}=" HB2 BSJ";
$resn_exch{"1HB  BSJ"}=" HB1 BSJ"; $resn_exch{"2HC  BSJ"}=" HC2 BSJ"; $resn_exch{"1H51 BSJ"}="H511 BSJ";
$resn_exch{"2H51 BSJ"}="H512 BSJ"; $resn_exch{"1H48 BSJ"}="H481 BSJ"; $resn_exch{"2H48 BSJ"}="H482 BSJ";
$resn_exch{"1H47 BSJ"}="H471 BSJ"; $resn_exch{"2H47 BSJ"}="H472 BSJ"; $resn_exch{"1H43 BSJ"}="H431 BSJ";
$resn_exch{"2H43 BSJ"}="H432 BSJ"; $resn_exch{"3H43 BSJ"}="H433 BSJ"; $resn_exch{"1H44 BSJ"}="H441 BSJ";
$resn_exch{"2H44 BSJ"}="H442 BSJ"; $resn_exch{"3H44 BSJ"}="H443 BSJ"; $resn_exch{"1H41 BSJ"}="H411 BSJ";
$resn_exch{"2H41 BSJ"}="H412 BSJ"; $resn_exch{"1H15 BSJ"}="H151 BSJ"; $resn_exch{"2H15 BSJ"}="H152 BSJ";
$resn_exch{"1H3N BSJ"}="H3N1 BSJ"; $resn_exch{"2H3N BSJ"}="H3N2 BSJ"; $resn_exch{"1H22 BSM"}="H221 BSM";
$resn_exch{"2H22 BSM"}="H222 BSM"; $resn_exch{"3H22 BSM"}="H223 BSM"; $resn_exch{"1H26 BSM"}="H261 BSM";
$resn_exch{"2H26 BSM"}="H262 BSM"; $resn_exch{"1H27 BSM"}="H271 BSM"; $resn_exch{"2H27 BSM"}="H272 BSM";
$resn_exch{"3H27 BSM"}="H273 BSM"; $resn_exch{"1H10 BSO"}="H101 BSO"; $resn_exch{"2H10 BSO"}="H102 BSO";
$resn_exch{"1H9C BSO"}="H9C1 BSO"; $resn_exch{"2H9C BSO"}="H9C2 BSO"; $resn_exch{"1H8C BSO"}="H8C1 BSO";
$resn_exch{"2H8C BSO"}="H8C2 BSO"; $resn_exch{"1H7C BSO"}="H7C1 BSO"; $resn_exch{"2H7C BSO"}="H7C2 BSO";
$resn_exch{"1H6C BSO"}="H6C1 BSO"; $resn_exch{"2H6C BSO"}="H6C2 BSO"; $resn_exch{" H2* BSP"}=" H2B BSP";
$resn_exch{" H5* BSP"}=" H5B BSP"; $resn_exch{" H6* BSP"}=" H6B BSP"; $resn_exch{" H4* BSP"}=" H4B BSP";
$resn_exch{"1H8  BST"}=" H81 BST"; $resn_exch{"2H8  BST"}=" H82 BST"; $resn_exch{"1HN9 BST"}="HN91 BST";
$resn_exch{"2HN9 BST"}="HN92 BST"; $resn_exch{"1H10 BST"}="H101 BST"; $resn_exch{"2H10 BST"}="H102 BST";
$resn_exch{"1H11 BST"}="H111 BST"; $resn_exch{"2H11 BST"}="H112 BST"; $resn_exch{"1H13 BST"}="H131 BST";
$resn_exch{"2H13 BST"}="H132 BST"; $resn_exch{"3H13 BST"}="H133 BST"; $resn_exch{"1H15 BST"}="H151 BST";
$resn_exch{"2H15 BST"}="H152 BST"; $resn_exch{"1HN4 BST"}="HN41 BST"; $resn_exch{"2HN4 BST"}="HN42 BST";
$resn_exch{"1H4  BSU"}=" H41 BSU"; $resn_exch{"1H3  BSU"}=" H31 BSU"; $resn_exch{"1H2  BSU"}=" H21 BSU";
$resn_exch{"1H1  BSU"}=" H11 BSU"; $resn_exch{"1H6  BSU"}=" H61 BSU"; $resn_exch{"1H16 BSU"}="H161 BSU";
$resn_exch{"1H15 BSU"}="H151 BSU"; $resn_exch{"1H14 BSU"}="H141 BSU"; $resn_exch{"1H13 BSU"}="H131 BSU";
$resn_exch{"1H12 BSU"}="H121 BSU"; $resn_exch{"1H24 BT1"}="H241 BT1"; $resn_exch{"2H24 BT1"}="H242 BT1";
$resn_exch{"1H25 BT1"}="H251 BT1"; $resn_exch{"2H25 BT1"}="H252 BT1"; $resn_exch{"1H17 BT1"}="H171 BT1";
$resn_exch{"2H17 BT1"}="H172 BT1"; $resn_exch{"1H23 BT1"}="H231 BT1"; $resn_exch{"2H23 BT1"}="H232 BT1";
$resn_exch{"1H26 BT1"}="H261 BT1"; $resn_exch{"2H26 BT1"}="H262 BT1"; $resn_exch{"1H27 BT1"}="H271 BT1";
$resn_exch{"2H27 BT1"}="H272 BT1"; $resn_exch{"1H22 BT1"}="H221 BT1"; $resn_exch{"2H22 BT1"}="H222 BT1";
$resn_exch{"1H28 BT1"}="H281 BT1"; $resn_exch{"2H28 BT1"}="H282 BT1"; $resn_exch{"1H29 BT1"}="H291 BT1";
$resn_exch{"2H29 BT1"}="H292 BT1"; $resn_exch{"1H30 BT1"}="H301 BT1"; $resn_exch{"2H30 BT1"}="H302 BT1";
$resn_exch{"1H33 BT1"}="H331 BT1"; $resn_exch{"2H33 BT1"}="H332 BT1"; $resn_exch{"1H34 BT1"}="H341 BT1";
$resn_exch{"2H34 BT1"}="H342 BT1"; $resn_exch{"1H24 BT2"}="H241 BT2"; $resn_exch{"2H24 BT2"}="H242 BT2";
$resn_exch{"1H25 BT2"}="H251 BT2"; $resn_exch{"2H25 BT2"}="H252 BT2"; $resn_exch{"1H17 BT2"}="H171 BT2";
$resn_exch{"2H17 BT2"}="H172 BT2"; $resn_exch{"1H23 BT2"}="H231 BT2"; $resn_exch{"2H23 BT2"}="H232 BT2";
$resn_exch{"1H26 BT2"}="H261 BT2"; $resn_exch{"2H26 BT2"}="H262 BT2"; $resn_exch{"1H27 BT2"}="H271 BT2";
$resn_exch{"2H27 BT2"}="H272 BT2"; $resn_exch{"1H22 BT2"}="H221 BT2"; $resn_exch{"2H22 BT2"}="H222 BT2";
$resn_exch{"1H28 BT2"}="H281 BT2"; $resn_exch{"2H28 BT2"}="H282 BT2"; $resn_exch{"1H29 BT2"}="H291 BT2";
$resn_exch{"2H29 BT2"}="H292 BT2"; $resn_exch{"1H30 BT2"}="H301 BT2"; $resn_exch{"2H30 BT2"}="H302 BT2";
$resn_exch{"1H33 BT2"}="H331 BT2"; $resn_exch{"2H33 BT2"}="H332 BT2"; $resn_exch{"1H34 BT2"}="H341 BT2";
$resn_exch{"2H34 BT2"}="H342 BT2"; $resn_exch{"1H24 BT3"}="H241 BT3"; $resn_exch{"2H24 BT3"}="H242 BT3";
$resn_exch{"1H25 BT3"}="H251 BT3"; $resn_exch{"2H25 BT3"}="H252 BT3"; $resn_exch{"1H17 BT3"}="H171 BT3";
$resn_exch{"2H17 BT3"}="H172 BT3"; $resn_exch{"1H23 BT3"}="H231 BT3"; $resn_exch{"2H23 BT3"}="H232 BT3";
$resn_exch{"1H26 BT3"}="H261 BT3"; $resn_exch{"2H26 BT3"}="H262 BT3"; $resn_exch{"1H27 BT3"}="H271 BT3";
$resn_exch{"2H27 BT3"}="H272 BT3"; $resn_exch{"1H22 BT3"}="H221 BT3"; $resn_exch{"2H22 BT3"}="H222 BT3";
$resn_exch{"1H28 BT3"}="H281 BT3"; $resn_exch{"2H28 BT3"}="H282 BT3"; $resn_exch{"1H29 BT3"}="H291 BT3";
$resn_exch{"2H29 BT3"}="H292 BT3"; $resn_exch{"1H30 BT3"}="H301 BT3"; $resn_exch{"2H30 BT3"}="H302 BT3";
$resn_exch{"1H33 BT3"}="H331 BT3"; $resn_exch{"2H33 BT3"}="H332 BT3"; $resn_exch{"1H34 BT3"}="H341 BT3";
$resn_exch{"2H34 BT3"}="H342 BT3"; $resn_exch{"1H10 BT5"}="H101 BT5"; $resn_exch{"2H10 BT5"}="H102 BT5";
$resn_exch{"1H9B BT5"}="H9B1 BT5"; $resn_exch{"2H9B BT5"}="H9B2 BT5"; $resn_exch{"1H8B BT5"}="H8B1 BT5";
$resn_exch{"2H8B BT5"}="H8B2 BT5"; $resn_exch{"1H7B BT5"}="H7B1 BT5"; $resn_exch{"2H7B BT5"}="H7B2 BT5";
$resn_exch{"1H6B BT5"}="H6B1 BT5"; $resn_exch{"2H6B BT5"}="H6B2 BT5"; $resn_exch{"1H5* BT5"}=" H5' BT5";
$resn_exch{"2H5* BT5"}="H5'' BT5"; $resn_exch{" H4* BT5"}=" H4' BT5"; $resn_exch{" H3* BT5"}=" H3' BT5";
$resn_exch{" H2* BT5"}=" H2' BT5"; $resn_exch{" H1* BT5"}=" H1' BT5"; $resn_exch{"1HN6 BT5"}="HN61 BT5";
$resn_exch{"2HN6 BT5"}="HN62 BT5"; $resn_exch{"1HN  BTA"}=" HN1 BTA"; $resn_exch{"2HN  BTA"}=" HN2 BTA";
$resn_exch{"1HB  BTA"}=" HB1 BTA"; $resn_exch{"2HB  BTA"}=" HB2 BTA"; $resn_exch{"1HG  BTA"}=" HG1 BTA";
$resn_exch{"2HG  BTA"}=" HG2 BTA"; $resn_exch{"1HD  BTA"}=" HD1 BTA"; $resn_exch{"2HD  BTA"}=" HD2 BTA";
$resn_exch{"3HD  BTA"}=" HD3 BTA"; $resn_exch{"1H1  BTB"}=" H11 BTB"; $resn_exch{"2H1  BTB"}=" H12 BTB";
$resn_exch{"1H3  BTB"}=" H31 BTB"; $resn_exch{"2H3  BTB"}=" H32 BTB"; $resn_exch{"1H4  BTB"}=" H41 BTB";
$resn_exch{"2H4  BTB"}=" H42 BTB"; $resn_exch{"1H5  BTB"}=" H51 BTB"; $resn_exch{"2H5  BTB"}=" H52 BTB";
$resn_exch{"1H6  BTB"}=" H61 BTB"; $resn_exch{"2H6  BTB"}=" H62 BTB"; $resn_exch{"1H7  BTB"}=" H71 BTB";
$resn_exch{"2H7  BTB"}=" H72 BTB"; $resn_exch{"1H8  BTB"}=" H81 BTB"; $resn_exch{"2H8  BTB"}=" H82 BTB";
$resn_exch{"1HN  BTC"}=" H   BTC"; $resn_exch{"2HN  BTC"}=" H2  BTC"; $resn_exch{"1HB  BTC"}=" HB2 BTC";
$resn_exch{"2HB  BTC"}=" HB3 BTC"; $resn_exch{" HSG BTC"}=" HG  BTC"; $resn_exch{"*HO5 BTD"}="HO5' BTD";
$resn_exch{"1H5* BTD"}="H5'1 BTD"; $resn_exch{"2H5* BTD"}="H5'2 BTD"; $resn_exch{" H4* BTD"}=" H4' BTD";
$resn_exch{" H1* BTD"}=" H1' BTD"; $resn_exch{"1H2* BTD"}="H2'1 BTD"; $resn_exch{"2H2* BTD"}="H2'2 BTD";
$resn_exch{" H3* BTD"}=" H3' BTD"; $resn_exch{"*HO3 BTD"}="HO3' BTD"; $resn_exch{"1H31 BTH"}="H311 BTH";
$resn_exch{"2H31 BTH"}="H312 BTH"; $resn_exch{"3H31 BTH"}="H313 BTH"; $resn_exch{"1H35 BTH"}="H351 BTH";
$resn_exch{"2H35 BTH"}="H352 BTH"; $resn_exch{"1H23 BTH"}="H231 BTH"; $resn_exch{"2H23 BTH"}="H232 BTH";
$resn_exch{"1H22 BTH"}="H221 BTH"; $resn_exch{"2H22 BTH"}="H222 BTH"; $resn_exch{"1H9  BTH"}=" H91 BTH";
$resn_exch{"2H9  BTH"}=" H92 BTH"; $resn_exch{"1H10 BTH"}="H101 BTH"; $resn_exch{"2H10 BTH"}="H102 BTH";
$resn_exch{"1HA  BTL"}=" HA1 BTL"; $resn_exch{"2HA  BTL"}=" HA2 BTL"; $resn_exch{"1H3  BTL"}=" H31 BTL";
$resn_exch{"2H3  BTL"}=" H32 BTL"; $resn_exch{"3H3  BTL"}=" H33 BTL"; $resn_exch{"1H2  BTL"}=" H21 BTL";
$resn_exch{"2H2  BTL"}=" H22 BTL"; $resn_exch{"3H2  BTL"}=" H23 BTL"; $resn_exch{"1H1  BTL"}=" H11 BTL";
$resn_exch{"2H1  BTL"}=" H12 BTL"; $resn_exch{"3H1  BTL"}=" H13 BTL"; $resn_exch{"1H10 BTN"}="H101 BTN";
$resn_exch{"2H10 BTN"}="H102 BTN"; $resn_exch{"1H9  BTN"}=" H91 BTN"; $resn_exch{"2H9  BTN"}=" H92 BTN";
$resn_exch{"1H8  BTN"}=" H81 BTN"; $resn_exch{"2H8  BTN"}=" H82 BTN"; $resn_exch{"1H7  BTN"}=" H71 BTN";
$resn_exch{"2H7  BTN"}=" H72 BTN"; $resn_exch{"1H6  BTN"}=" H61 BTN"; $resn_exch{"2H6  BTN"}=" H62 BTN";
$resn_exch{"1H11 BTO"}="H111 BTO"; $resn_exch{"2H11 BTO"}="H112 BTO"; $resn_exch{"1H10 BTO"}="H101 BTO";
$resn_exch{"2H10 BTO"}="H102 BTO"; $resn_exch{"1H9  BTO"}=" H91 BTO"; $resn_exch{"2H9  BTO"}=" H92 BTO";
$resn_exch{"1H3  BTP"}=" H31 BTP"; $resn_exch{"2H3  BTP"}=" H32 BTP"; $resn_exch{"1H*  BTP"}=" H'1 BTP";
$resn_exch{"2H*  BTP"}=" H'2 BTP"; $resn_exch{" HS* BTP"}=" HS' BTP"; $resn_exch{"1H10 BTQ"}="H101 BTQ";
$resn_exch{"2H10 BTQ"}="H102 BTQ"; $resn_exch{"1H9  BTQ"}=" H91 BTQ"; $resn_exch{"2H9  BTQ"}=" H92 BTQ";
$resn_exch{"1H8  BTQ"}=" H81 BTQ"; $resn_exch{"2H8  BTQ"}=" H82 BTQ"; $resn_exch{"1H7  BTQ"}=" H71 BTQ";
$resn_exch{"2H7  BTQ"}=" H72 BTQ"; $resn_exch{"1H6  BTQ"}=" H61 BTQ"; $resn_exch{"2H6  BTQ"}=" H62 BTQ";
$resn_exch{"1HN  BTR"}=" H   BTR"; $resn_exch{"2HN  BTR"}=" H2  BTR"; $resn_exch{"1HB  BTR"}=" HB2 BTR";
$resn_exch{"2HB  BTR"}=" HB3 BTR"; $resn_exch{"1H1  BTS"}=" H11 BTS"; $resn_exch{"2H1  BTS"}=" H12 BTS";
$resn_exch{"1H8  BTS"}=" H81 BTS"; $resn_exch{"2H8  BTS"}=" H82 BTS"; $resn_exch{"1H9  BTS"}=" H91 BTS";
$resn_exch{"2H9  BTS"}=" H92 BTS"; $resn_exch{"1H5* BTX"}="H5'1 BTX"; $resn_exch{"2H5* BTX"}="H5'2 BTX";
$resn_exch{" H4* BTX"}=" H4' BTX"; $resn_exch{" H3* BTX"}=" H3' BTX"; $resn_exch{" H2* BTX"}=" H2' BTX";
$resn_exch{" H1* BTX"}=" H1' BTX"; $resn_exch{"1HN6 BTX"}="HN61 BTX"; $resn_exch{"2HN6 BTX"}="HN62 BTX";
$resn_exch{"1H11 BTX"}="H111 BTX"; $resn_exch{"2H11 BTX"}="H112 BTX"; $resn_exch{"1H10 BTX"}="H101 BTX";
$resn_exch{"2H10 BTX"}="H102 BTX"; $resn_exch{"1H9B BTX"}="H9B1 BTX"; $resn_exch{"2H9B BTX"}="H9B2 BTX";
$resn_exch{"1H8B BTX"}="H8B1 BTX"; $resn_exch{"2H8B BTX"}="H8B2 BTX"; $resn_exch{"1H7B BTX"}="H7B1 BTX";
$resn_exch{"2H7B BTX"}="H7B2 BTX"; $resn_exch{"1H6B BTX"}="H6B1 BTX"; $resn_exch{"2H6B BTX"}="H6B2 BTX";
$resn_exch{"1H10 BTY"}="H101 BTY"; $resn_exch{"2H10 BTY"}="H102 BTY"; $resn_exch{"3H10 BTY"}="H103 BTY";
$resn_exch{"1H11 BTY"}="H111 BTY"; $resn_exch{"2H11 BTY"}="H112 BTY"; $resn_exch{"1H1  BU1"}=" H11 BU1";
$resn_exch{"2H1  BU1"}=" H12 BU1"; $resn_exch{"1H2  BU1"}=" H21 BU1"; $resn_exch{"2H2  BU1"}=" H22 BU1";
$resn_exch{"1H3  BU1"}=" H31 BU1"; $resn_exch{"2H3  BU1"}=" H32 BU1"; $resn_exch{"1H4  BU1"}=" H41 BU1";
$resn_exch{"2H4  BU1"}=" H42 BU1"; $resn_exch{"1HC1 BU2"}="HC11 BU2"; $resn_exch{"2HC1 BU2"}="HC12 BU2";
$resn_exch{"1HC2 BU2"}="HC21 BU2"; $resn_exch{"2HC2 BU2"}="HC22 BU2"; $resn_exch{"1HC4 BU2"}="HC41 BU2";
$resn_exch{"2HC4 BU2"}="HC42 BU2"; $resn_exch{"3HC4 BU2"}="HC43 BU2"; $resn_exch{"1H4  BU3"}=" H41 BU3";
$resn_exch{"2H4  BU3"}=" H42 BU3"; $resn_exch{"3H4  BU3"}=" H43 BU3"; $resn_exch{"1H1  BU3"}=" H11 BU3";
$resn_exch{"2H1  BU3"}=" H12 BU3"; $resn_exch{"3H1  BU3"}=" H13 BU3"; $resn_exch{"1H1  BUA"}=" H11 BUA";
$resn_exch{"2H1  BUA"}=" H12 BUA"; $resn_exch{"3H1  BUA"}=" H13 BUA"; $resn_exch{"1H2  BUA"}=" H21 BUA";
$resn_exch{"2H2  BUA"}=" H22 BUA"; $resn_exch{"1H3  BUA"}=" H31 BUA"; $resn_exch{"2H3  BUA"}=" H32 BUA";
$resn_exch{"1H1  BUB"}=" H11 BUB"; $resn_exch{"2H1  BUB"}=" H12 BUB"; $resn_exch{"3H1  BUB"}=" H13 BUB";
$resn_exch{"1H2  BUB"}=" H21 BUB"; $resn_exch{"2H2  BUB"}=" H22 BUB"; $resn_exch{"1H6  BUB"}=" H61 BUB";
$resn_exch{"2H6  BUB"}=" H62 BUB"; $resn_exch{"1H9  BUB"}=" H91 BUB"; $resn_exch{"2H9  BUB"}=" H92 BUB";
$resn_exch{"1HN  BUC"}=" H   BUC"; $resn_exch{"2HN  BUC"}=" H2  BUC"; $resn_exch{"1HB  BUC"}=" HB2 BUC";
$resn_exch{"2HB  BUC"}=" HB3 BUC"; $resn_exch{"1H1  BUC"}=" H11 BUC"; $resn_exch{"2H1  BUC"}=" H12 BUC";
$resn_exch{"1H2  BUC"}=" H21 BUC"; $resn_exch{"2H2  BUC"}=" H22 BUC"; $resn_exch{"1H3  BUC"}=" H31 BUC";
$resn_exch{"2H3  BUC"}=" H32 BUC"; $resn_exch{"1H4  BUC"}=" H41 BUC"; $resn_exch{"2H4  BUC"}=" H42 BUC";
$resn_exch{"3H4  BUC"}=" H43 BUC"; $resn_exch{"1H1  BUG"}=" H   BUG"; $resn_exch{"2H1  BUG"}=" H2  BUG";
$resn_exch{"1HA  BUG"}=" HA1 BUG"; $resn_exch{"1HG1 BUG"}="HG11 BUG"; $resn_exch{"2HG1 BUG"}="HG12 BUG";
$resn_exch{"3HG1 BUG"}="HG13 BUG"; $resn_exch{"1HG2 BUG"}="HG21 BUG"; $resn_exch{"2HG2 BUG"}="HG22 BUG";
$resn_exch{"3HG2 BUG"}="HG23 BUG"; $resn_exch{"1HG3 BUG"}="HG31 BUG"; $resn_exch{"2HG3 BUG"}="HG32 BUG";
$resn_exch{"3HG3 BUG"}="HG33 BUG"; $resn_exch{"1HN2 BUG"}="HN21 BUG"; $resn_exch{"2HN2 BUG"}="HN22 BUG";
$resn_exch{"1H2  BUI"}=" H21 BUI"; $resn_exch{"2H2  BUI"}=" H22 BUI"; $resn_exch{"1H10 BUI"}="H101 BUI";
$resn_exch{"2H10 BUI"}="H102 BUI"; $resn_exch{"1H14 BUI"}="H141 BUI"; $resn_exch{"2H14 BUI"}="H142 BUI";
$resn_exch{"3H14 BUI"}="H143 BUI"; $resn_exch{"1H13 BUI"}="H131 BUI"; $resn_exch{"2H13 BUI"}="H132 BUI";
$resn_exch{"3H13 BUI"}="H133 BUI"; $resn_exch{"1H12 BUI"}="H121 BUI"; $resn_exch{"2H12 BUI"}="H122 BUI";
$resn_exch{"3H12 BUI"}="H123 BUI"; $resn_exch{"1H15 BUI"}="H151 BUI"; $resn_exch{"2H15 BUI"}="H152 BUI";
$resn_exch{"1H16 BUI"}="H161 BUI"; $resn_exch{"2H16 BUI"}="H162 BUI"; $resn_exch{"1H24 BUI"}="H241 BUI";
$resn_exch{"2H24 BUI"}="H242 BUI"; $resn_exch{"1H26 BUI"}="H261 BUI"; $resn_exch{"2H26 BUI"}="H262 BUI";
$resn_exch{"1H27 BUI"}="H271 BUI"; $resn_exch{"2H27 BUI"}="H272 BUI"; $resn_exch{"1H19 BUI"}="H191 BUI";
$resn_exch{"2H19 BUI"}="H192 BUI"; $resn_exch{"1H20 BUI"}="H201 BUI"; $resn_exch{"2H20 BUI"}="H202 BUI";
$resn_exch{"1H21 BUI"}="H211 BUI"; $resn_exch{"2H21 BUI"}="H212 BUI"; $resn_exch{"1H22 BUI"}="H221 BUI";
$resn_exch{"2H22 BUI"}="H222 BUI"; $resn_exch{"1H23 BUI"}="H231 BUI"; $resn_exch{"2H23 BUI"}="H232 BUI";
$resn_exch{"1H25 BUI"}="H251 BUI"; $resn_exch{"2H25 BUI"}="H252 BUI"; $resn_exch{"1H18 BUI"}="H181 BUI";
$resn_exch{"2H18 BUI"}="H182 BUI"; $resn_exch{"1H17 BUI"}="H171 BUI"; $resn_exch{"2H17 BUI"}="H172 BUI";
$resn_exch{"1H28 BUI"}="H281 BUI"; $resn_exch{"2H28 BUI"}="H282 BUI"; $resn_exch{"3H28 BUI"}="H283 BUI";
$resn_exch{"1H06 BUK"}="H061 BUK"; $resn_exch{"2H06 BUK"}="H062 BUK"; $resn_exch{"1H07 BUK"}="H071 BUK";
$resn_exch{"2H07 BUK"}="H072 BUK"; $resn_exch{"1H08 BUK"}="H081 BUK"; $resn_exch{"2H08 BUK"}="H082 BUK";
$resn_exch{"1H11 BUK"}="H111 BUK"; $resn_exch{"2H11 BUK"}="H112 BUK"; $resn_exch{"1H22 BUK"}="H221 BUK";
$resn_exch{"2H22 BUK"}="H222 BUK"; $resn_exch{"3H22 BUK"}="H223 BUK"; $resn_exch{"1H23 BUK"}="H231 BUK";
$resn_exch{"2H23 BUK"}="H232 BUK"; $resn_exch{"3H23 BUK"}="H233 BUK"; $resn_exch{"1H28 BUK"}="H281 BUK";
$resn_exch{"2H28 BUK"}="H282 BUK"; $resn_exch{"1H31 BUK"}="H311 BUK"; $resn_exch{"2H31 BUK"}="H312 BUK";
$resn_exch{"1H43 BUK"}="H431 BUK"; $resn_exch{"2H43 BUK"}="H432 BUK"; $resn_exch{"3H43 BUK"}="H433 BUK";
$resn_exch{"1H9  BUL"}=" H91 BUL"; $resn_exch{"2H9  BUL"}=" H92 BUL"; $resn_exch{"1H12 BUL"}="H121 BUL";
$resn_exch{"2H12 BUL"}="H122 BUL"; $resn_exch{"1H19 BUL"}="H191 BUL"; $resn_exch{"2H19 BUL"}="H192 BUL";
$resn_exch{"3H19 BUL"}="H193 BUL"; $resn_exch{"1H21 BUL"}="H211 BUL"; $resn_exch{"2H21 BUL"}="H212 BUL";
$resn_exch{"1H26 BUL"}="H261 BUL"; $resn_exch{"2H26 BUL"}="H262 BUL"; $resn_exch{"1H27 BUL"}="H271 BUL";
$resn_exch{"2H27 BUL"}="H272 BUL"; $resn_exch{"1HG2 BUM"}="HG21 BUM"; $resn_exch{"2HG2 BUM"}="HG22 BUM";
$resn_exch{"3HG2 BUM"}="HG23 BUM"; $resn_exch{"1HG3 BUM"}="HG31 BUM"; $resn_exch{"2HG3 BUM"}="HG32 BUM";
$resn_exch{"3HG3 BUM"}="HG33 BUM"; $resn_exch{"1HG1 BUM"}="HG11 BUM"; $resn_exch{"2HG1 BUM"}="HG12 BUM";
$resn_exch{"3HG1 BUM"}="HG13 BUM"; $resn_exch{"1HN  BUM"}=" HN1 BUM"; $resn_exch{"2HN  BUM"}=" HN2 BUM";
$resn_exch{"1HN2 BUM"}="HN21 BUM"; $resn_exch{"2HN2 BUM"}="HN22 BUM"; $resn_exch{"3HN2 BUM"}="HN23 BUM";
$resn_exch{"1H1  BUQ"}=" H11 BUQ"; $resn_exch{"2H1  BUQ"}=" H12 BUQ"; $resn_exch{"3H1  BUQ"}=" H13 BUQ";
$resn_exch{"1H3  BUQ"}=" H31 BUQ"; $resn_exch{"2H3  BUQ"}=" H32 BUQ"; $resn_exch{"1H4  BUQ"}=" H41 BUQ";
$resn_exch{"2H4  BUQ"}=" H42 BUQ"; $resn_exch{"1HO5 BUQ"}="HO51 BUQ"; $resn_exch{"1H1  BUT"}=" H11 BUT";
$resn_exch{"2H1  BUT"}=" H12 BUT"; $resn_exch{"3H1  BUT"}=" H13 BUT"; $resn_exch{"1H2  BUT"}=" H21 BUT";
$resn_exch{"2H2  BUT"}=" H22 BUT"; $resn_exch{"1H3  BUT"}=" H31 BUT"; $resn_exch{"2H3  BUT"}=" H32 BUT";
$resn_exch{"1H4  BUT"}=" H41 BUT"; $resn_exch{"2H4  BUT"}=" H42 BUT"; $resn_exch{"3H4  BUT"}=" H43 BUT";
$resn_exch{"1H50 BV1"}="H501 BV1"; $resn_exch{"2H50 BV1"}="H502 BV1"; $resn_exch{"1H1  BV1"}="H11A BV1";
$resn_exch{"2H1  BV1"}=" H12 BV1"; $resn_exch{"1H11 BV1"}="H111 BV1"; $resn_exch{"2H11 BV1"}="H112 BV1";
$resn_exch{"1H12 BV1"}="H121 BV1"; $resn_exch{"2H12 BV1"}="H122 BV1"; $resn_exch{"1H13 BV1"}="H131 BV1";
$resn_exch{"2H13 BV1"}="H132 BV1"; $resn_exch{"1H14 BV1"}="H141 BV1"; $resn_exch{"2H14 BV1"}="H142 BV1";
$resn_exch{"1H15 BV1"}="H151 BV1"; $resn_exch{"2H15 BV1"}="H152 BV1"; $resn_exch{"1H16 BV1"}="H161 BV1";
$resn_exch{"2H16 BV1"}="H162 BV1"; $resn_exch{"1H17 BV1"}="H171 BV1"; $resn_exch{"2H17 BV1"}="H172 BV1";
$resn_exch{"1H18 BV1"}="H181 BV1"; $resn_exch{"2H18 BV1"}="H182 BV1"; $resn_exch{"1H19 BV1"}="H191 BV1";
$resn_exch{"2H19 BV1"}="H192 BV1"; $resn_exch{"1H20 BV1"}="H201 BV1"; $resn_exch{"2H20 BV1"}="H202 BV1";
$resn_exch{"1H23 BV1"}="H231 BV1"; $resn_exch{"2H23 BV1"}="H232 BV1"; $resn_exch{"1H24 BV1"}="H241 BV1";
$resn_exch{"2H24 BV1"}="H242 BV1"; $resn_exch{"1H25 BV1"}="H251 BV1"; $resn_exch{"2H25 BV1"}="H252 BV1";
$resn_exch{"1H26 BV1"}="H261 BV1"; $resn_exch{"2H26 BV1"}="H262 BV1"; $resn_exch{"1H27 BV1"}="H271 BV1";
$resn_exch{"2H27 BV1"}="H272 BV1"; $resn_exch{"1H28 BV1"}="H281 BV1"; $resn_exch{"2H28 BV1"}="H282 BV1";
$resn_exch{"1H29 BV1"}="H291 BV1"; $resn_exch{"2H29 BV1"}="H292 BV1"; $resn_exch{"1H30 BV1"}="H301 BV1";
$resn_exch{"2H30 BV1"}="H302 BV1"; $resn_exch{"1H31 BV1"}="H311 BV1"; $resn_exch{"2H31 BV1"}="H312 BV1";
$resn_exch{"1H32 BV1"}="H321 BV1"; $resn_exch{"2H32 BV1"}="H322 BV1"; $resn_exch{"1H51 BV2"}="H511 BV2";
$resn_exch{"2H51 BV2"}="H512 BV2"; $resn_exch{"1H1  BV2"}="H11A BV2"; $resn_exch{"2H1  BV2"}="H12A BV2";
$resn_exch{"1H11 BV2"}="H111 BV2"; $resn_exch{"2H11 BV2"}="H112 BV2"; $resn_exch{"1H12 BV2"}="H121 BV2";
$resn_exch{"2H12 BV2"}="H122 BV2"; $resn_exch{"1H13 BV2"}="H131 BV2"; $resn_exch{"2H13 BV2"}="H132 BV2";
$resn_exch{"1H14 BV2"}="H141 BV2"; $resn_exch{"2H14 BV2"}="H142 BV2"; $resn_exch{"1H15 BV2"}="H151 BV2";
$resn_exch{"2H15 BV2"}="H152 BV2"; $resn_exch{"1H16 BV2"}="H161 BV2"; $resn_exch{"2H16 BV2"}="H162 BV2";
$resn_exch{"1H17 BV2"}="H171 BV2"; $resn_exch{"2H17 BV2"}="H172 BV2"; $resn_exch{"1H18 BV2"}="H181 BV2";
$resn_exch{"2H18 BV2"}="H182 BV2"; $resn_exch{"1H19 BV2"}="H191 BV2"; $resn_exch{"2H19 BV2"}="H192 BV2";
$resn_exch{"1H20 BV2"}="H201 BV2"; $resn_exch{"2H20 BV2"}="H202 BV2"; $resn_exch{"1H22 BV2"}="H221 BV2";
$resn_exch{"2H22 BV2"}="H222 BV2"; $resn_exch{"1HN6 BV2"}="HN61 BV2"; $resn_exch{"2HN6 BV2"}="HN62 BV2";
$resn_exch{"1H24 BV2"}="H241 BV2"; $resn_exch{"2H24 BV2"}="H242 BV2"; $resn_exch{"1H26 BV2"}="H261 BV2";
$resn_exch{"2H26 BV2"}="H262 BV2"; $resn_exch{"1H27 BV2"}="H271 BV2"; $resn_exch{"2H27 BV2"}="H272 BV2";
$resn_exch{"1H28 BV2"}="H281 BV2"; $resn_exch{"2H28 BV2"}="H282 BV2"; $resn_exch{"1H29 BV2"}="H291 BV2";
$resn_exch{"2H29 BV2"}="H292 BV2"; $resn_exch{"1H30 BV2"}="H301 BV2"; $resn_exch{"2H30 BV2"}="H302 BV2";
$resn_exch{"1H31 BV2"}="H311 BV2"; $resn_exch{"2H31 BV2"}="H312 BV2"; $resn_exch{"1H32 BV2"}="H321 BV2";
$resn_exch{"2H32 BV2"}="H322 BV2"; $resn_exch{"1H33 BV2"}="H331 BV2"; $resn_exch{"2H33 BV2"}="H332 BV2";
$resn_exch{"1H34 BV2"}="H341 BV2"; $resn_exch{"2H34 BV2"}="H342 BV2"; $resn_exch{"1H35 BV2"}="H351 BV2";
$resn_exch{"2H35 BV2"}="H352 BV2"; $resn_exch{"1H63 BV3"}="H631 BV3"; $resn_exch{"2H63 BV3"}="H632 BV3";
$resn_exch{"1H1  BV3"}="H11A BV3"; $resn_exch{"2H1  BV3"}=" H12 BV3"; $resn_exch{"1H11 BV3"}="H111 BV3";
$resn_exch{"2H11 BV3"}="H112 BV3"; $resn_exch{"1H12 BV3"}="H121 BV3"; $resn_exch{"2H12 BV3"}="H122 BV3";
$resn_exch{"1H13 BV3"}="H131 BV3"; $resn_exch{"2H13 BV3"}="H132 BV3"; $resn_exch{"1H14 BV3"}="H141 BV3";
$resn_exch{"2H14 BV3"}="H142 BV3"; $resn_exch{"1H15 BV3"}="H151 BV3"; $resn_exch{"2H15 BV3"}="H152 BV3";
$resn_exch{"1H16 BV3"}="H161 BV3"; $resn_exch{"2H16 BV3"}="H162 BV3"; $resn_exch{"1H17 BV3"}="H171 BV3";
$resn_exch{"2H17 BV3"}="H172 BV3"; $resn_exch{"1H18 BV3"}="H181 BV3"; $resn_exch{"2H18 BV3"}="H182 BV3";
$resn_exch{"1H19 BV3"}="H191 BV3"; $resn_exch{"2H19 BV3"}="H192 BV3"; $resn_exch{"1H20 BV3"}="H201 BV3";
$resn_exch{"2H20 BV3"}="H202 BV3"; $resn_exch{"1H23 BV3"}="H231 BV3"; $resn_exch{"2H23 BV3"}="H232 BV3";
$resn_exch{"1H24 BV3"}="H241 BV3"; $resn_exch{"2H24 BV3"}="H242 BV3"; $resn_exch{"1H26 BV3"}="H261 BV3";
$resn_exch{"2H26 BV3"}="H262 BV3"; $resn_exch{"1HN8 BV3"}="HN81 BV3"; $resn_exch{"2HN8 BV3"}="HN82 BV3";
$resn_exch{"1H28 BV3"}="H281 BV3"; $resn_exch{"2H28 BV3"}="H282 BV3"; $resn_exch{"1H30 BV3"}="H301 BV3";
$resn_exch{"2H30 BV3"}="H302 BV3"; $resn_exch{"1H31 BV3"}="H311 BV3"; $resn_exch{"2H31 BV3"}="H312 BV3";
$resn_exch{"1H34 BV3"}="H341 BV3"; $resn_exch{"2H34 BV3"}="H342 BV3"; $resn_exch{"1H35 BV3"}="H351 BV3";
$resn_exch{"2H35 BV3"}="H352 BV3"; $resn_exch{"1H36 BV3"}="H361 BV3"; $resn_exch{"2H36 BV3"}="H362 BV3";
$resn_exch{"1H37 BV3"}="H371 BV3"; $resn_exch{"2H37 BV3"}="H372 BV3"; $resn_exch{"1H38 BV3"}="H381 BV3";
$resn_exch{"2H38 BV3"}="H382 BV3"; $resn_exch{"1H39 BV3"}="H391 BV3"; $resn_exch{"2H39 BV3"}="H392 BV3";
$resn_exch{"1H40 BV3"}="H401 BV3"; $resn_exch{"2H40 BV3"}="H402 BV3"; $resn_exch{"1H41 BV3"}="H411 BV3";
$resn_exch{"2H41 BV3"}="H412 BV3"; $resn_exch{"1H42 BV3"}="H421 BV3"; $resn_exch{"2H42 BV3"}="H422 BV3";
$resn_exch{"1H43 BV3"}="H431 BV3"; $resn_exch{"2H43 BV3"}="H432 BV3"; $resn_exch{"1H75 BV4"}="H751 BV4";
$resn_exch{"2H75 BV4"}="H752 BV4"; $resn_exch{"1H1  BV4"}="H11A BV4"; $resn_exch{"2H1  BV4"}=" H12 BV4";
$resn_exch{"1H11 BV4"}="H111 BV4"; $resn_exch{"2H11 BV4"}="H112 BV4"; $resn_exch{"1H12 BV4"}="H121 BV4";
$resn_exch{"2H12 BV4"}="H122 BV4"; $resn_exch{"1H14 BV4"}="H141 BV4"; $resn_exch{"2H14 BV4"}="H142 BV4";
$resn_exch{"1H15 BV4"}="H151 BV4"; $resn_exch{"2H15 BV4"}="H152 BV4"; $resn_exch{"1H16 BV4"}="H161 BV4";
$resn_exch{"2H16 BV4"}="H162 BV4"; $resn_exch{"1H17 BV4"}="H171 BV4"; $resn_exch{"2H17 BV4"}="H172 BV4";
$resn_exch{"1H18 BV4"}="H181 BV4"; $resn_exch{"2H18 BV4"}="H182 BV4"; $resn_exch{"1H19 BV4"}="H191 BV4";
$resn_exch{"2H19 BV4"}="H192 BV4"; $resn_exch{"1H20 BV4"}="H201 BV4"; $resn_exch{"2H20 BV4"}="H202 BV4";
$resn_exch{"1H21 BV4"}="H211 BV4"; $resn_exch{"2H21 BV4"}="H212 BV4"; $resn_exch{"1H24 BV4"}="H241 BV4";
$resn_exch{"2H24 BV4"}="H242 BV4"; $resn_exch{"1H25 BV4"}="H251 BV4"; $resn_exch{"2H25 BV4"}="H252 BV4";
$resn_exch{"1H26 BV4"}="H261 BV4"; $resn_exch{"2H26 BV4"}="H262 BV4"; $resn_exch{"1H27 BV4"}="H271 BV4";
$resn_exch{"2H27 BV4"}="H272 BV4"; $resn_exch{"1H28 BV4"}="H281 BV4"; $resn_exch{"2H28 BV4"}="H282 BV4";
$resn_exch{"1H29 BV4"}="H291 BV4"; $resn_exch{"2H29 BV4"}="H292 BV4"; $resn_exch{"1H30 BV4"}="H301 BV4";
$resn_exch{"2H30 BV4"}="H302 BV4"; $resn_exch{"1H31 BV4"}="H311 BV4"; $resn_exch{"2H31 BV4"}="H312 BV4";
$resn_exch{"1H32 BV4"}="H321 BV4"; $resn_exch{"2H32 BV4"}="H322 BV4"; $resn_exch{"1H33 BV4"}="H331 BV4";
$resn_exch{"2H33 BV4"}="H332 BV4"; $resn_exch{"1H35 BV4"}="H351 BV4"; $resn_exch{"2H35 BV4"}="H352 BV4";
$resn_exch{"1HN8 BV4"}="HN81 BV4"; $resn_exch{"2HN8 BV4"}="HN82 BV4"; $resn_exch{"1H37 BV4"}="H371 BV4";
$resn_exch{"2H37 BV4"}="H372 BV4"; $resn_exch{"1H39 BV4"}="H391 BV4"; $resn_exch{"2H39 BV4"}="H392 BV4";
$resn_exch{"1H40 BV4"}="H401 BV4"; $resn_exch{"2H40 BV4"}="H402 BV4"; $resn_exch{"1H41 BV4"}="H411 BV4";
$resn_exch{"2H41 BV4"}="H412 BV4"; $resn_exch{"1H42 BV4"}="H421 BV4"; $resn_exch{"2H42 BV4"}="H422 BV4";
$resn_exch{"1H43 BV4"}="H431 BV4"; $resn_exch{"2H43 BV4"}="H432 BV4"; $resn_exch{"1H44 BV4"}="H441 BV4";
$resn_exch{"2H44 BV4"}="H442 BV4"; $resn_exch{"1H45 BV4"}="H451 BV4"; $resn_exch{"2H45 BV4"}="H452 BV4";
$resn_exch{"1H46 BV4"}="H461 BV4"; $resn_exch{"2H46 BV4"}="H462 BV4"; $resn_exch{"1H47 BV4"}="H471 BV4";
$resn_exch{"2H47 BV4"}="H472 BV4"; $resn_exch{"1H48 BV4"}="H481 BV4"; $resn_exch{"2H48 BV4"}="H482 BV4";
$resn_exch{"1H51 BV4"}="H511 BV4"; $resn_exch{"2H51 BV4"}="H512 BV4"; $resn_exch{"1H52 BV4"}="H521 BV4";
$resn_exch{"2H52 BV4"}="H522 BV4"; $resn_exch{"1H53 BV4"}="H531 BV4"; $resn_exch{"2H53 BV4"}="H532 BV4";
$resn_exch{"1H54 BV4"}="H541 BV4"; $resn_exch{"2H54 BV4"}="H542 BV4"; $resn_exch{"1H55 BV4"}="H551 BV4";
$resn_exch{"2H55 BV4"}="H552 BV4"; $resn_exch{"1H56 BV4"}="H561 BV4"; $resn_exch{"2H56 BV4"}="H562 BV4";
$resn_exch{"1H57 BV4"}="H571 BV4"; $resn_exch{"2H57 BV4"}="H572 BV4"; $resn_exch{"1H58 BV4"}="H581 BV4";
$resn_exch{"2H58 BV4"}="H582 BV4"; $resn_exch{"1H59 BV4"}="H591 BV4"; $resn_exch{"2H59 BV4"}="H592 BV4";
$resn_exch{"1H60 BV4"}="H601 BV4"; $resn_exch{"2H60 BV4"}="H602 BV4"; $resn_exch{"1H6  BVC"}=" H61 BVC";
$resn_exch{"2H6  BVC"}=" H62 BVC"; $resn_exch{"1HN7 BVC"}="HN71 BVC"; $resn_exch{"2HN7 BVC"}="HN72 BVC";
$resn_exch{"*HO5 BVD"}="HO5' BVD"; $resn_exch{"1H5* BVD"}="H5'1 BVD"; $resn_exch{"2H5* BVD"}="H5'2 BVD";
$resn_exch{" H4* BVD"}=" H4' BVD"; $resn_exch{" H1* BVD"}=" H1' BVD"; $resn_exch{"1H2* BVD"}="H2'1 BVD";
$resn_exch{"2H2* BVD"}="H2'2 BVD"; $resn_exch{" H3* BVD"}=" H3' BVD"; $resn_exch{"*HO3 BVD"}="HO3' BVD";
$resn_exch{"1HN7 BVF"}="HN71 BVF"; $resn_exch{"2HN7 BVF"}="HN72 BVF"; $resn_exch{"1H8  BVF"}=" H81 BVF";
$resn_exch{"2H8  BVF"}=" H82 BVF"; $resn_exch{"3H8  BVF"}=" H83 BVF"; $resn_exch{"1H6  BVG"}=" H61 BVG";
$resn_exch{"2H6  BVG"}=" H62 BVG"; $resn_exch{" H3* BVP"}=" H3' BVP"; $resn_exch{"1H2* BVP"}="H2'1 BVP";
$resn_exch{"2H2* BVP"}="H2'2 BVP"; $resn_exch{" H1* BVP"}=" H1' BVP"; $resn_exch{" H4* BVP"}=" H4' BVP";
$resn_exch{"1H5* BVP"}="H5'1 BVP"; $resn_exch{"2H5* BVP"}="H5'2 BVP"; $resn_exch{"1H21 BW2"}="H211 BW2";
$resn_exch{"2H21 BW2"}="H212 BW2"; $resn_exch{"1H20 BW2"}="H201 BW2"; $resn_exch{"2H20 BW2"}="H202 BW2";
$resn_exch{"1HN2 BW2"}="HN21 BW2"; $resn_exch{"2HN2 BW2"}="HN22 BW2"; $resn_exch{"1H7  BWD"}=" H71 BWD";
$resn_exch{"2H7  BWD"}=" H72 BWD"; $resn_exch{"1H8  BWD"}=" H81 BWD"; $resn_exch{"2H8  BWD"}=" H82 BWD";
$resn_exch{"OH92 BWD"}="H92O BWD"; $resn_exch{"1H22 BWP"}="H221 BWP"; $resn_exch{"2H22 BWP"}="H222 BWP";
$resn_exch{"1H24 BWP"}="H241 BWP"; $resn_exch{"2H24 BWP"}="H242 BWP"; $resn_exch{"1H26 BWP"}="H261 BWP";
$resn_exch{"2H26 BWP"}="H262 BWP"; $resn_exch{"3H26 BWP"}="H263 BWP"; $resn_exch{"1H27 BWP"}="H271 BWP";
$resn_exch{"2H27 BWP"}="H272 BWP"; $resn_exch{"3H27 BWP"}="H273 BWP"; $resn_exch{"1HN3 BX3"}="HN31 BX3";
$resn_exch{"2HN3 BX3"}="HN32 BX3"; $resn_exch{"1H14 BX3"}="H141 BX3"; $resn_exch{"2H14 BX3"}="H142 BX3";
$resn_exch{"1H27 BX3"}="H271 BX3"; $resn_exch{"2H27 BX3"}="H272 BX3"; $resn_exch{"1H29 BX3"}="H291 BX3";
$resn_exch{"2H29 BX3"}="H292 BX3"; $resn_exch{"1H30 BX3"}="H301 BX3"; $resn_exch{"2H30 BX3"}="H302 BX3";
$resn_exch{"1H33 BX3"}="H331 BX3"; $resn_exch{"2H33 BX3"}="H332 BX3"; $resn_exch{"3H33 BX3"}="H333 BX3";
$resn_exch{"1H34 BX3"}="H341 BX3"; $resn_exch{"2H34 BX3"}="H342 BX3"; $resn_exch{"1H5B BXP"}="H5B1 BXP";
$resn_exch{"2H5B BXP"}="H5B2 BXP"; $resn_exch{"1H5A BXP"}="H5A1 BXP"; $resn_exch{"2H5A BXP"}="H5A2 BXP";
$resn_exch{"1H1A BXP"}="H1A1 BXP"; $resn_exch{"2H1A BXP"}="H1A2 BXP"; $resn_exch{"1H22 BYP"}="H221 BYP";
$resn_exch{"2H22 BYP"}="H222 BYP"; $resn_exch{"1H24 BYP"}="H241 BYP"; $resn_exch{"2H24 BYP"}="H242 BYP";
$resn_exch{"1H26 BYP"}="H261 BYP"; $resn_exch{"2H26 BYP"}="H262 BYP"; $resn_exch{"3H26 BYP"}="H263 BYP";
$resn_exch{"1H27 BYP"}="H271 BYP"; $resn_exch{"2H27 BYP"}="H272 BYP"; $resn_exch{"3H27 BYP"}="H273 BYP";
$resn_exch{"1HC1 BYS"}="HC11 BYS"; $resn_exch{"1HXT BYS"}="HXT1 BYS"; $resn_exch{"1HC5 BYS"}="HC51 BYS";
$resn_exch{"2HC5 BYS"}="HC52 BYS"; $resn_exch{"1HC7 BYS"}="HC71 BYS"; $resn_exch{"1HC8 BYS"}="HC81 BYS";
$resn_exch{"1HC9 BYS"}="HC91 BYS"; $resn_exch{"1H10 BYS"}="H101 BYS"; $resn_exch{"1H11 BYS"}="H111 BYS";
$resn_exch{"1H12 BYS"}="H121 BYS"; $resn_exch{"2HXT BYS"}="HXT2 BYS"; $resn_exch{"1H16 BYS"}="H161 BYS";
$resn_exch{"2H16 BYS"}="H162 BYS"; $resn_exch{"1H18 BYS"}="H181 BYS"; $resn_exch{"1H21 BYS"}="H211 BYS";
$resn_exch{"2H21 BYS"}="H212 BYS"; $resn_exch{"1H24 BYS"}="H241 BYS"; $resn_exch{"1H25 BYS"}="H251 BYS";
$resn_exch{"1H3  BZ1"}=" H31 BZ1"; $resn_exch{"2H3  BZ1"}=" H32 BZ1"; $resn_exch{"1H12 BZ1"}="H121 BZ1";
$resn_exch{"2H12 BZ1"}="H122 BZ1"; $resn_exch{"1H4  BZ1"}=" H41 BZ1"; $resn_exch{"2H4  BZ1"}=" H42 BZ1";
$resn_exch{"1H5  BZ1"}=" H51 BZ1"; $resn_exch{"2H5  BZ1"}=" H52 BZ1"; $resn_exch{"3H5  BZ1"}=" H53 BZ1";
$resn_exch{"1H6  BZ1"}=" H61 BZ1"; $resn_exch{"2H6  BZ1"}=" H62 BZ1"; $resn_exch{"1HN2 BZ1"}="HN21 BZ1";
$resn_exch{"2HN2 BZ1"}="HN22 BZ1"; $resn_exch{"1H2  BZ1"}=" H21 BZ1"; $resn_exch{"2H2  BZ1"}=" H22 BZ1";
$resn_exch{"1H1  BZ1"}="H11A BZ1"; $resn_exch{"2H1  BZ1"}=" H12 BZ1"; $resn_exch{"3H1  BZ1"}=" H13 BZ1";
$resn_exch{"1H4C BZA"}="H4C1 BZA"; $resn_exch{"2H4C BZA"}="H4C2 BZA"; $resn_exch{"1HO1 BZB"}="HO11 BZB";
$resn_exch{"1HO2 BZB"}="HO21 BZB"; $resn_exch{"1HC2 BZB"}="HC21 BZB"; $resn_exch{"1HC7 BZB"}="HC71 BZB";
$resn_exch{"1HC4 BZB"}="HC41 BZB"; $resn_exch{"1HC6 BZB"}="HC61 BZB"; $resn_exch{"1HC5 BZB"}="HC51 BZB";
$resn_exch{"1HN1 BZC"}="HN11 BZC"; $resn_exch{"2HN1 BZC"}="HN12 BZC"; $resn_exch{"1H18 BZC"}="H181 BZC";
$resn_exch{"2H18 BZC"}="H182 BZC"; $resn_exch{"3H18 BZC"}="H183 BZC"; $resn_exch{"1H6  BZD"}=" H61 BZD";
$resn_exch{"2H6  BZD"}=" H62 BZD"; $resn_exch{"1H10 BZD"}="H101 BZD"; $resn_exch{"1H11 BZD"}="H111 BZD";
$resn_exch{"1H12 BZD"}="H121 BZD"; $resn_exch{"1H13 BZD"}="H131 BZD"; $resn_exch{"1H14 BZD"}="H141 BZD";
$resn_exch{"1H5* BZG"}="H5'1 BZG"; $resn_exch{"2H5* BZG"}="H5'2 BZG"; $resn_exch{"1HWC BZG"}="HWC1 BZG";
$resn_exch{"2HWC BZG"}="HWC2 BZG"; $resn_exch{"1HIN BZG"}="HIN1 BZG"; $resn_exch{"2HIN BZG"}="HIN2 BZG";
$resn_exch{" HT* BZG"}=" HT' BZG"; $resn_exch{"1HP* BZG"}="HP'1 BZG"; $resn_exch{"2HP* BZG"}="HP'2 BZG";
$resn_exch{" H4* BZG"}=" H4' BZG"; $resn_exch{" H3* BZG"}=" H3' BZG"; $resn_exch{"2H8  BZM"}=" H82 BZM";
$resn_exch{"1H8  BZM"}=" H81 BZM"; $resn_exch{"1H1  BZO"}=" H11 BZO"; $resn_exch{"1H2  BZO"}=" H21 BZO";
$resn_exch{"2H2  BZO"}=" H22 BZO"; $resn_exch{"1H2  BZP"}=" H21 BZP"; $resn_exch{"2H2  BZP"}=" H22 BZP";
$resn_exch{"1H3  BZP"}=" H31 BZP"; $resn_exch{"2H3  BZP"}=" H32 BZP"; $resn_exch{"1H5  BZP"}=" H51 BZP";
$resn_exch{"2H5  BZP"}=" H52 BZP"; $resn_exch{"1H6  BZP"}=" H61 BZP"; $resn_exch{"2H6  BZP"}=" H62 BZP";
$resn_exch{"1H4C BZR"}="H4C1 BZR"; $resn_exch{"2H4C BZR"}="H4C2 BZR"; $resn_exch{"1H2  BZS"}=" H21 BZS";
$resn_exch{"2H2  BZS"}=" H22 BZS"; $resn_exch{"1HB  BZS"}=" HB1 BZS"; $resn_exch{"2HB  BZS"}=" HB2 BZS";
$resn_exch{"1H24 BZT"}="H241 BZT"; $resn_exch{"2H24 BZT"}="H242 BZT"; $resn_exch{"1H31 BZT"}="H311 BZT";
$resn_exch{"2H31 BZT"}="H312 BZT"; $resn_exch{"1H17 BZT"}="H171 BZT"; $resn_exch{"2H17 BZT"}="H172 BZT";
$resn_exch{"1H23 BZT"}="H231 BZT"; $resn_exch{"2H23 BZT"}="H232 BZT"; $resn_exch{"1H26 BZT"}="H261 BZT";
$resn_exch{"2H26 BZT"}="H262 BZT"; $resn_exch{"1H27 BZT"}="H271 BZT"; $resn_exch{"2H27 BZT"}="H272 BZT";
$resn_exch{"1H22 BZT"}="H221 BZT"; $resn_exch{"2H22 BZT"}="H222 BZT"; $resn_exch{"1H28 BZT"}="H281 BZT";
$resn_exch{"2H28 BZT"}="H282 BZT"; $resn_exch{"1H29 BZT"}="H291 BZT"; $resn_exch{"2H29 BZT"}="H292 BZT";
$resn_exch{"1H30 BZT"}="H301 BZT"; $resn_exch{"2H30 BZT"}="H302 BZT"; $resn_exch{"1H34 BZT"}="H341 BZT";
$resn_exch{"2H34 BZT"}="H342 BZT"; $resn_exch{"1H9  BZU"}=" H91 BZU"; $resn_exch{"2H9  BZU"}=" H92 BZU";
$resn_exch{"1H11 BZU"}="H111 BZU"; $resn_exch{"2H11 BZU"}="H112 BZU"; $resn_exch{"1H12 BZU"}="H121 BZU";
$resn_exch{"2H12 BZU"}="H122 BZU"; $resn_exch{"1H14 BZU"}="H141 BZU"; $resn_exch{"2H14 BZU"}="H142 BZU";
$resn_exch{"1H15 BZU"}="H151 BZU"; $resn_exch{"2H15 BZU"}="H152 BZU"; $resn_exch{"3H15 BZU"}="H153 BZU";
$resn_exch{"1HN1 BZU"}="HN11 BZU"; $resn_exch{"2HN1 BZU"}="HN12 BZU"; $resn_exch{"1H17 BZU"}="H171 BZU";
$resn_exch{"2H17 BZU"}="H172 BZU"; $resn_exch{"1H18 BZU"}="H181 BZU"; $resn_exch{"2H18 BZU"}="H182 BZU";
$resn_exch{"3H18 BZU"}="H183 BZU"; $resn_exch{"1HX2 BZZ"}="HX21 BZZ"; $resn_exch{"2HX2 BZZ"}="HX22 BZZ";
$resn_exch{"1H1  BZZ"}=" H11 BZZ"; $resn_exch{"2H1  BZZ"}=" H12 BZZ"; $resn_exch{"3HOP   C"}="HOP3   C";
$resn_exch{"2HOP   C"}="HOP2   C"; $resn_exch{"1H5*   C"}=" H5'   C"; $resn_exch{"2H5*   C"}="H5''   C";
$resn_exch{" H4*   C"}=" H4'   C"; $resn_exch{" H3*   C"}=" H3'   C"; $resn_exch{"H3T    C"}="HO3'   C";
$resn_exch{" H2*   C"}=" H2'   C"; $resn_exch{"2HO*   C"}="HO2'   C"; $resn_exch{" H1*   C"}=" H1'   C";
$resn_exch{"1H4    C"}=" H41   C"; $resn_exch{"2H4    C"}=" H42   C"; $resn_exch{"1H11 C01"}="H111 C01";
$resn_exch{"2H11 C01"}="H112 C01"; $resn_exch{"3H11 C01"}="H113 C01"; $resn_exch{"1H16 C01"}="H161 C01";
$resn_exch{"2H16 C01"}="H162 C01"; $resn_exch{"1H17 C01"}="H171 C01"; $resn_exch{"2H17 C01"}="H172 C01";
$resn_exch{"1H20 C01"}="H201 C01"; $resn_exch{"2H20 C01"}="H202 C01"; $resn_exch{"3H20 C01"}="H203 C01";
$resn_exch{"1H26 C01"}="H261 C01"; $resn_exch{"2H26 C01"}="H262 C01"; $resn_exch{"1H27 C01"}="H271 C01";
$resn_exch{"2H27 C01"}="H272 C01"; $resn_exch{"1H30 C01"}="H301 C01"; $resn_exch{"2H30 C01"}="H302 C01";
$resn_exch{"3H30 C01"}="H303 C01"; $resn_exch{"1H50 C01"}="H501 C01"; $resn_exch{"2H50 C01"}="H502 C01";
$resn_exch{"3H50 C01"}="H503 C01"; $resn_exch{"1H54 C01"}="H541 C01"; $resn_exch{"2H54 C01"}="H542 C01";
$resn_exch{"3H54 C01"}="H543 C01"; $resn_exch{"1HN3 C02"}="HN31 C02"; $resn_exch{"2HN3 C02"}="HN32 C02";
$resn_exch{"1H10 C02"}="H101 C02"; $resn_exch{"2H10 C02"}="H102 C02"; $resn_exch{"1H16 C02"}="H161 C02";
$resn_exch{"2H16 C02"}="H162 C02"; $resn_exch{"3H16 C02"}="H163 C02"; $resn_exch{"NH21 C02"}="H21N C02";
$resn_exch{"1H14 C03"}="H141 C03"; $resn_exch{"2H14 C03"}="H142 C03"; $resn_exch{"3H14 C03"}="H143 C03";
$resn_exch{"1H18 C03"}="H181 C03"; $resn_exch{"2H18 C03"}="H182 C03"; $resn_exch{"1H38 C03"}="H381 C03";
$resn_exch{"2H38 C03"}="H382 C03"; $resn_exch{"3H38 C03"}="H383 C03"; $resn_exch{"1H58 C03"}="H581 C03";
$resn_exch{"2H58 C03"}="H582 C03"; $resn_exch{"3H58 C03"}="H583 C03"; $resn_exch{"1H1  C0M"}=" H11 C0M";
$resn_exch{"2H1  C0M"}=" H12 C0M"; $resn_exch{"1H27 C0M"}="H271 C0M"; $resn_exch{"2H27 C0M"}="H272 C0M";
$resn_exch{"1H28 C0M"}="H281 C0M"; $resn_exch{"2H28 C0M"}="H282 C0M"; $resn_exch{"1H34 C0M"}="H341 C0M";
$resn_exch{"2H34 C0M"}="H342 C0M"; $resn_exch{"1H33 C0M"}="H331 C0M"; $resn_exch{"2H33 C0M"}="H332 C0M";
$resn_exch{"1H32 C0M"}="H321 C0M"; $resn_exch{"2H32 C0M"}="H322 C0M"; $resn_exch{"1H31 C0M"}="H311 C0M";
$resn_exch{"2H31 C0M"}="H312 C0M"; $resn_exch{"1H30 C0M"}="H301 C0M"; $resn_exch{"2H30 C0M"}="H302 C0M";
$resn_exch{"1H6  C0M"}=" H61 C0M"; $resn_exch{"2H6  C0M"}=" H62 C0M"; $resn_exch{"1H4  C0M"}=" H41 C0M";
$resn_exch{"2H4  C0M"}=" H42 C0M"; $resn_exch{"1H1  C0R"}="H11A C0R"; $resn_exch{"2H1  C0R"}=" H12 C0R";
$resn_exch{"1H2  C0R"}=" H21 C0R"; $resn_exch{"2H2  C0R"}=" H22 C0R"; $resn_exch{"1H6  C0R"}=" H61 C0R";
$resn_exch{"2H6  C0R"}=" H62 C0R"; $resn_exch{"1H7  C0R"}=" H71 C0R"; $resn_exch{"2H7  C0R"}=" H72 C0R";
$resn_exch{"1H12 C0R"}="H121 C0R"; $resn_exch{"2H12 C0R"}="H122 C0R"; $resn_exch{"1H15 C0R"}="H151 C0R";
$resn_exch{"2H15 C0R"}="H152 C0R"; $resn_exch{"1H16 C0R"}="H161 C0R"; $resn_exch{"2H16 C0R"}="H162 C0R";
$resn_exch{"1H18 C0R"}="H181 C0R"; $resn_exch{"2H18 C0R"}="H182 C0R"; $resn_exch{"3H18 C0R"}="H183 C0R";
$resn_exch{"1H19 C0R"}="H191 C0R"; $resn_exch{"2H19 C0R"}="H192 C0R"; $resn_exch{"3H19 C0R"}="H193 C0R";
$resn_exch{"1H21 C0R"}="H211 C0R"; $resn_exch{"2H21 C0R"}="H212 C0R"; $resn_exch{"1H1C C10"}="H1C1 C10";
$resn_exch{"2H1C C10"}="H1C2 C10"; $resn_exch{"3H1C C10"}="H1C3 C10"; $resn_exch{"1H2C C10"}="H2C1 C10";
$resn_exch{"2H2C C10"}="H2C2 C10"; $resn_exch{"1H3C C10"}="H3C1 C10"; $resn_exch{"2H3C C10"}="H3C2 C10";
$resn_exch{"1H4C C10"}="H4C1 C10"; $resn_exch{"2H4C C10"}="H4C2 C10"; $resn_exch{"1H5C C10"}="H5C1 C10";
$resn_exch{"2H5C C10"}="H5C2 C10"; $resn_exch{"1H6C C10"}="H6C1 C10"; $resn_exch{"2H6C C10"}="H6C2 C10";
$resn_exch{"1H7C C10"}="H7C1 C10"; $resn_exch{"2H7C C10"}="H7C2 C10"; $resn_exch{"1H8C C10"}="H8C1 C10";
$resn_exch{"2H8C C10"}="H8C2 C10"; $resn_exch{"1H9C C10"}="H9C1 C10"; $resn_exch{"2H9C C10"}="H9C2 C10";
$resn_exch{"1H10 C10"}="H101 C10"; $resn_exch{"2H10 C10"}="H102 C10"; $resn_exch{"1H12 C10"}="H121 C10";
$resn_exch{"2H12 C10"}="H122 C10"; $resn_exch{"1H13 C10"}="H131 C10"; $resn_exch{"2H13 C10"}="H132 C10";
$resn_exch{"1H15 C10"}="H151 C10"; $resn_exch{"2H15 C10"}="H152 C10"; $resn_exch{"1H16 C10"}="H161 C10";
$resn_exch{"2H16 C10"}="H162 C10"; $resn_exch{"1H18 C10"}="H181 C10"; $resn_exch{"2H18 C10"}="H182 C10";
$resn_exch{"1H19 C10"}="H191 C10"; $resn_exch{"2H19 C10"}="H192 C10"; $resn_exch{"1H21 C10"}="H211 C10";
$resn_exch{"2H21 C10"}="H212 C10"; $resn_exch{"1H22 C10"}="H221 C10"; $resn_exch{"2H22 C10"}="H222 C10";
$resn_exch{"1H24 C10"}="H241 C10"; $resn_exch{"2H24 C10"}="H242 C10"; $resn_exch{"1H25 C10"}="H251 C10";
$resn_exch{"2H25 C10"}="H252 C10"; $resn_exch{"1H27 C10"}="H271 C10"; $resn_exch{"2H27 C10"}="H272 C10";
$resn_exch{"1H28 C10"}="H281 C10"; $resn_exch{"2H28 C10"}="H282 C10"; $resn_exch{"1H1  C11"}=" H11 C11";
$resn_exch{"2H1  C11"}=" H12 C11"; $resn_exch{"1H2  C11"}=" H21 C11"; $resn_exch{"2H2  C11"}=" H22 C11";
$resn_exch{"1H3  C11"}=" H31 C11"; $resn_exch{"2H3  C11"}=" H32 C11"; $resn_exch{"1H4  C11"}=" H41 C11";
$resn_exch{"2H4  C11"}=" H42 C11"; $resn_exch{"1H5  C11"}=" H51 C11"; $resn_exch{"2H5  C11"}=" H52 C11";
$resn_exch{"1H6  C11"}=" H61 C11"; $resn_exch{"2H6  C11"}=" H62 C11"; $resn_exch{"1H7  C11"}=" H71 C11";
$resn_exch{"2H7  C11"}=" H72 C11"; $resn_exch{"1H8  C11"}=" H81 C11"; $resn_exch{"2H8  C11"}=" H82 C11";
$resn_exch{"1H9  C11"}=" H91 C11"; $resn_exch{"2H9  C11"}=" H92 C11"; $resn_exch{"1H10 C11"}="H101 C11";
$resn_exch{"2H10 C11"}="H102 C11"; $resn_exch{"1H11 C11"}="H111 C11"; $resn_exch{"2H11 C11"}="H112 C11";
$resn_exch{"3H11 C11"}="H113 C11"; $resn_exch{"1HY1 C11"}="HY11 C11"; $resn_exch{"2HY1 C11"}="HY12 C11";
$resn_exch{"1HY2 C11"}="HY21 C11"; $resn_exch{"2HY2 C11"}="HY22 C11"; $resn_exch{"1HY3 C11"}="HY31 C11";
$resn_exch{"2HY3 C11"}="HY32 C11"; $resn_exch{"1HY4 C11"}="HY41 C11"; $resn_exch{"2HY4 C11"}="HY42 C11";
$resn_exch{"3HY4 C11"}="HY43 C11"; $resn_exch{"1HG1 C12"}="HG11 C12"; $resn_exch{"2HG1 C12"}="HG12 C12";
$resn_exch{"3HG1 C12"}="HG13 C12"; $resn_exch{"1HN1 C12"}="HN11 C12"; $resn_exch{"2HN1 C12"}="HN12 C12";
$resn_exch{"1HA3 C12"}="HA31 C12"; $resn_exch{"2HA3 C12"}="HA32 C12"; $resn_exch{"1HB2 C12"}="HB21 C12";
$resn_exch{"2HB2 C12"}="HB22 C12"; $resn_exch{"1H01 C14"}="H011 C14"; $resn_exch{"2H01 C14"}="H012 C14";
$resn_exch{"3H01 C14"}="H013 C14"; $resn_exch{"1H02 C14"}="H021 C14"; $resn_exch{"2H02 C14"}="H022 C14";
$resn_exch{"1H03 C14"}="H031 C14"; $resn_exch{"2H03 C14"}="H032 C14"; $resn_exch{"1H04 C14"}="H041 C14";
$resn_exch{"2H04 C14"}="H042 C14"; $resn_exch{"1H05 C14"}="H051 C14"; $resn_exch{"2H05 C14"}="H052 C14";
$resn_exch{"1H06 C14"}="H061 C14"; $resn_exch{"2H06 C14"}="H062 C14"; $resn_exch{"1H07 C14"}="H071 C14";
$resn_exch{"2H07 C14"}="H072 C14"; $resn_exch{"1H08 C14"}="H081 C14"; $resn_exch{"2H08 C14"}="H082 C14";
$resn_exch{"1H09 C14"}="H091 C14"; $resn_exch{"2H09 C14"}="H092 C14"; $resn_exch{"1H10 C14"}="H101 C14";
$resn_exch{"2H10 C14"}="H102 C14"; $resn_exch{"1H11 C14"}="H111 C14"; $resn_exch{"2H11 C14"}="H112 C14";
$resn_exch{"1H12 C14"}="H121 C14"; $resn_exch{"2H12 C14"}="H122 C14"; $resn_exch{"1H13 C14"}="H131 C14";
$resn_exch{"2H13 C14"}="H132 C14"; $resn_exch{"1H14 C14"}="H141 C14"; $resn_exch{"2H14 C14"}="H142 C14";
$resn_exch{"3H14 C14"}="H143 C14"; $resn_exch{"1H1C C15"}="H1C1 C15"; $resn_exch{"2H1C C15"}="H1C2 C15";
$resn_exch{"1H2C C15"}="H2C1 C15"; $resn_exch{"2H2C C15"}="H2C2 C15"; $resn_exch{"1H3C C15"}="H3C1 C15";
$resn_exch{"2H3C C15"}="H3C2 C15"; $resn_exch{"1H5C C15"}="H5C1 C15"; $resn_exch{"2H5C C15"}="H5C2 C15";
$resn_exch{"3H5C C15"}="H5C3 C15"; $resn_exch{"1H6C C15"}="H6C1 C15"; $resn_exch{"2H6C C15"}="H6C2 C15";
$resn_exch{"1H7C C15"}="H7C1 C15"; $resn_exch{"2H7C C15"}="H7C2 C15"; $resn_exch{"1H8C C15"}="H8C1 C15";
$resn_exch{"2H8C C15"}="H8C2 C15"; $resn_exch{"1HCN C15"}="HCN1 C15"; $resn_exch{"2HCN C15"}="HCN2 C15";
$resn_exch{"3HCN C15"}="HCN3 C15"; $resn_exch{"4HCN C15"}="HCN4 C15"; $resn_exch{"5HCN C15"}="HCN5 C15";
$resn_exch{"6HCN C15"}="HCN6 C15"; $resn_exch{"1H9C C15"}="H9C1 C15"; $resn_exch{"2H9C C15"}="H9C2 C15";
$resn_exch{"1H10 C15"}="H101 C15"; $resn_exch{"2H10 C15"}="H102 C15"; $resn_exch{"1H11 C15"}="H111 C15";
$resn_exch{"2H11 C15"}="H112 C15"; $resn_exch{"1H12 C15"}="H121 C15"; $resn_exch{"2H12 C15"}="H122 C15";
$resn_exch{"1H13 C15"}="H131 C15"; $resn_exch{"2H13 C15"}="H132 C15"; $resn_exch{"1H14 C15"}="H141 C15";
$resn_exch{"2H14 C15"}="H142 C15"; $resn_exch{"1H15 C15"}="H151 C15"; $resn_exch{"2H15 C15"}="H152 C15";
$resn_exch{"1H16 C15"}="H161 C15"; $resn_exch{"2H16 C15"}="H162 C15"; $resn_exch{"1H7B C17"}="H7B1 C17";
$resn_exch{"2H7B C17"}="H7B2 C17"; $resn_exch{"1H4C C18"}="H4C1 C18"; $resn_exch{"2H4C C18"}="H4C2 C18";
$resn_exch{"1H4N C18"}="H4N1 C18"; $resn_exch{"2H4N C18"}="H4N2 C18"; $resn_exch{"3H4N C18"}="H4N3 C18";
$resn_exch{"1H7B C18"}="H7B1 C18"; $resn_exch{"2H7B C18"}="H7B2 C18"; $resn_exch{"1H3  C1A"}=" H31 C1A";
$resn_exch{"2H3  C1A"}=" H32 C1A"; $resn_exch{"3H3  C1A"}=" H33 C1A"; $resn_exch{"1H5  C1A"}=" H51 C1A";
$resn_exch{"2H5  C1A"}=" H52 C1A"; $resn_exch{"1HN4 C1A"}="HN41 C1A"; $resn_exch{"2HN4 C1A"}="HN42 C1A";
$resn_exch{"1H9  C1A"}=" H91 C1A"; $resn_exch{"2H9  C1A"}=" H92 C1A"; $resn_exch{"3H9  C1A"}=" H93 C1A";
$resn_exch{"1H12 C1A"}="H121 C1A"; $resn_exch{"2H12 C1A"}="H122 C1A"; $resn_exch{"1HN8 C1A"}="HN81 C1A";
$resn_exch{"2HN8 C1A"}="HN82 C1A"; $resn_exch{"1HN2 C1B"}="HN21 C1B"; $resn_exch{"2HN2 C1B"}="HN22 C1B";
$resn_exch{"1H42 C1B"}="H421 C1B"; $resn_exch{"2H42 C1B"}="H422 C1B"; $resn_exch{"H3'' C1B"}=" H3B C1B";
$resn_exch{"H4'' C1B"}=" H4B C1B"; $resn_exch{"1H51 C1B"}="H511 C1B"; $resn_exch{"2H51 C1B"}="H512 C1B";
$resn_exch{"1H55 C1B"}="H551 C1B"; $resn_exch{"2H55 C1B"}="H552 C1B"; $resn_exch{"H6'' C1B"}=" H6B C1B";
$resn_exch{"H2'' C1B"}=" H2B C1B"; $resn_exch{"1H12 C1D"}="H121 C1D"; $resn_exch{"2H12 C1D"}="H122 C1D";
$resn_exch{"19H1 C1D"}="H119 C1D"; $resn_exch{"1H24 C1D"}="H241 C1D"; $resn_exch{"2H24 C1D"}="H242 C1D";
$resn_exch{"1H25 C1D"}="H251 C1D"; $resn_exch{"2H25 C1D"}="H252 C1D"; $resn_exch{"1H29 C1D"}="H291 C1D";
$resn_exch{"2H29 C1D"}="H292 C1D"; $resn_exch{"30H3 C1D"}="H330 C1D"; $resn_exch{"1H33 C1D"}="H331 C1D";
$resn_exch{"1H34 C1D"}="H341 C1D"; $resn_exch{"1H2C C1D"}="H2C1 C1D"; $resn_exch{"2H2C C1D"}="H2C2 C1D";
$resn_exch{"3H2C C1D"}="H2C3 C1D"; $resn_exch{"35H3 C1D"}="H335 C1D"; $resn_exch{"37H3 C1D"}="H337 C1D";
$resn_exch{"1H6N C1D"}="H6N1 C1D"; $resn_exch{"2H6N C1D"}="H6N2 C1D"; $resn_exch{"1H4C C1F"}="H4C1 C1F";
$resn_exch{"2H4C C1F"}="H4C2 C1F"; $resn_exch{"1H6C C1F"}="H6C1 C1F"; $resn_exch{"2H6C C1F"}="H6C2 C1F";
$resn_exch{"3H6C C1F"}="H6C3 C1F"; $resn_exch{"1H7C C1F"}="H7C1 C1F"; $resn_exch{"2H7C C1F"}="H7C2 C1F";
$resn_exch{"3H7C C1F"}="H7C3 C1F"; $resn_exch{"1H9C C1F"}="H9C1 C1F"; $resn_exch{"2H9C C1F"}="H9C2 C1F";
$resn_exch{"1H11 C1F"}="H111 C1F"; $resn_exch{"2H11 C1F"}="H112 C1F"; $resn_exch{"1H12 C1F"}="H121 C1F";
$resn_exch{"2H12 C1F"}="H122 C1F"; $resn_exch{"1H7  C1N"}=" H71 C1N"; $resn_exch{"2H7  C1N"}=" H72 C1N";
$resn_exch{"1H14 C1N"}="H141 C1N"; $resn_exch{"2H14 C1N"}="H142 C1N"; $resn_exch{"3H14 C1N"}="H143 C1N";
$resn_exch{"1H15 C1N"}="H151 C1N"; $resn_exch{"2H15 C1N"}="H152 C1N"; $resn_exch{"3H15 C1N"}="H153 C1N";
$resn_exch{"1H17 C1N"}="H171 C1N"; $resn_exch{"2H17 C1N"}="H172 C1N"; $resn_exch{"1H21 C1N"}="H211 C1N";
$resn_exch{"2H21 C1N"}="H212 C1N"; $resn_exch{"1H23 C1N"}="H231 C1N"; $resn_exch{"2H23 C1N"}="H232 C1N";
$resn_exch{"3H23 C1N"}="H233 C1N"; $resn_exch{"1H24 C1N"}="H241 C1N"; $resn_exch{"2H24 C1N"}="H242 C1N";
$resn_exch{"3H24 C1N"}="H243 C1N"; $resn_exch{"1H28 C1N"}="H281 C1N"; $resn_exch{"2H28 C1N"}="H282 C1N";
$resn_exch{"1H32 C1N"}="H321 C1N"; $resn_exch{"2H32 C1N"}="H322 C1N"; $resn_exch{"1H14 C1P"}="H141 C1P";
$resn_exch{"2H14 C1P"}="H142 C1P"; $resn_exch{"1H13 C1P"}="H131 C1P"; $resn_exch{"2H13 C1P"}="H132 C1P";
$resn_exch{"1H11 C1P"}="H111 C1P"; $resn_exch{"2H11 C1P"}="H112 C1P"; $resn_exch{"1H10 C1P"}="H101 C1P";
$resn_exch{"2H10 C1P"}="H102 C1P"; $resn_exch{"1H20 C1P"}="H201 C1P"; $resn_exch{"2H20 C1P"}="H202 C1P";
$resn_exch{"1H21 C1P"}="H211 C1P"; $resn_exch{"2H21 C1P"}="H212 C1P"; $resn_exch{"1H5  C1P"}=" H51 C1P";
$resn_exch{"2H5  C1P"}=" H52 C1P"; $resn_exch{"1H7  C1P"}=" H71 C1P"; $resn_exch{"2H7  C1P"}=" H72 C1P";
$resn_exch{"3H7  C1P"}=" H73 C1P"; $resn_exch{"1H8  C1P"}=" H81 C1P"; $resn_exch{"2H8  C1P"}=" H82 C1P";
$resn_exch{"3H8  C1P"}=" H83 C1P"; $resn_exch{"1H1  C1R"}=" H11 C1R"; $resn_exch{"2H1  C1R"}=" H12 C1R";
$resn_exch{"1H2  C1R"}=" H21 C1R"; $resn_exch{"2H2  C1R"}=" H22 C1R"; $resn_exch{"1H4  C1R"}=" H41 C1R";
$resn_exch{"2H4  C1R"}=" H42 C1R"; $resn_exch{"1H5  C1R"}=" H51 C1R"; $resn_exch{"2H5  C1R"}=" H52 C1R";
$resn_exch{"1H7  C1R"}=" H71 C1R"; $resn_exch{"2H7  C1R"}=" H72 C1R"; $resn_exch{"1H8  C1R"}=" H81 C1R";
$resn_exch{"2H8  C1R"}=" H82 C1R"; $resn_exch{"1H9  C1R"}=" H91 C1R"; $resn_exch{"2H9  C1R"}=" H92 C1R";
$resn_exch{"1H10 C1X"}="H101 C1X"; $resn_exch{"2H10 C1X"}="H102 C1X"; $resn_exch{"1H9C C1X"}="H9C1 C1X";
$resn_exch{"2H9C C1X"}="H9C2 C1X"; $resn_exch{"1H7C C1X"}="H7C1 C1X"; $resn_exch{"2H7C C1X"}="H7C2 C1X";
$resn_exch{"1H5C C1X"}="H5C1 C1X"; $resn_exch{"2H5C C1X"}="H5C2 C1X"; $resn_exch{"1H2C C1X"}="H2C1 C1X";
$resn_exch{"2H2C C1X"}="H2C2 C1X"; $resn_exch{"1HEC C1X"}="HEC1 C1X"; $resn_exch{"2HEC C1X"}="HEC2 C1X";
$resn_exch{"1HDC C1X"}="HDC1 C1X"; $resn_exch{"2HDC C1X"}="HDC2 C1X"; $resn_exch{"1HGC C1X"}="HGC1 C1X";
$resn_exch{"2HGC C1X"}="HGC2 C1X"; $resn_exch{"1HBC C1X"}="HBC1 C1X"; $resn_exch{"2HBC C1X"}="HBC2 C1X";
$resn_exch{"1HN  C1X"}=" HN1 C1X"; $resn_exch{"2HN  C1X"}=" HN2 C1X"; $resn_exch{"1H1  C20"}=" H11 C20";
$resn_exch{"2H1  C20"}="H12A C20"; $resn_exch{"3H1  C20"}=" H13 C20"; $resn_exch{"1H7  C20"}=" H71 C20";
$resn_exch{"2H7  C20"}=" H72 C20"; $resn_exch{"3H7  C20"}=" H73 C20"; $resn_exch{"1H8  C20"}=" H81 C20";
$resn_exch{"2H8  C20"}=" H82 C20"; $resn_exch{"3H8  C20"}=" H83 C20"; $resn_exch{"1H13 C20"}="H131 C20";
$resn_exch{"2H13 C20"}="H132 C20"; $resn_exch{"1H22 C20"}="H221 C20"; $resn_exch{"2H22 C20"}="H222 C20";
$resn_exch{"1H24 C20"}="H241 C20"; $resn_exch{"2H24 C20"}="H242 C20"; $resn_exch{"1H26 C20"}="H261 C20";
$resn_exch{"2H26 C20"}="H262 C20"; $resn_exch{"1H27 C20"}="H271 C20"; $resn_exch{"2H27 C20"}="H272 C20";
$resn_exch{"1H28 C20"}="H281 C20"; $resn_exch{"2H28 C20"}="H282 C20"; $resn_exch{"1H29 C20"}="H291 C20";
$resn_exch{"2H29 C20"}="H292 C20"; $resn_exch{"1H30 C20"}="H301 C20"; $resn_exch{"2H30 C20"}="H302 C20";
$resn_exch{"1H37 C20"}="H371 C20"; $resn_exch{"2H37 C20"}="H372 C20"; $resn_exch{"3H37 C20"}="H373 C20";
$resn_exch{"1H38 C20"}="H381 C20"; $resn_exch{"2H38 C20"}="H382 C20"; $resn_exch{"3H38 C20"}="H383 C20";
$resn_exch{"1H41 C20"}="H411 C20"; $resn_exch{"2H41 C20"}="H412 C20"; $resn_exch{"3H41 C20"}="H413 C20";
$resn_exch{"NH10 C24"}="H10N C24"; $resn_exch{"1H11 C24"}="H111 C24"; $resn_exch{"2H11 C24"}="H112 C24";
$resn_exch{"1H29 C24"}="H291 C24"; $resn_exch{"2H29 C24"}="H292 C24"; $resn_exch{"1H30 C24"}="H301 C24";
$resn_exch{"2H30 C24"}="H302 C24"; $resn_exch{"1H34 C24"}="H341 C24"; $resn_exch{"2H34 C24"}="H342 C24";
$resn_exch{"1H35 C24"}="H351 C24"; $resn_exch{"2H35 C24"}="H352 C24"; $resn_exch{"3H35 C24"}="H353 C24";
$resn_exch{"1H17 C24"}="H171 C24"; $resn_exch{"2H17 C24"}="H172 C24"; $resn_exch{"3H17 C24"}="H173 C24";
$resn_exch{"1HN8 C24"}="HN81 C24"; $resn_exch{"2HN8 C24"}="HN82 C24"; $resn_exch{"3HOP C25"}="HOP3 C25";
$resn_exch{"2HOP C25"}="HOP2 C25"; $resn_exch{"1H5* C25"}=" H5' C25"; $resn_exch{"2H5* C25"}="H5'' C25";
$resn_exch{" H4* C25"}=" H4' C25"; $resn_exch{" H3* C25"}=" H3' C25"; $resn_exch{"3HO* C25"}="HO'3 C25";
$resn_exch{" H2* C25"}=" H2' C25"; $resn_exch{"2HO* C25"}="HO2' C25"; $resn_exch{" H1* C25"}=" H1' C25";
$resn_exch{"1H4  C25"}=" H41 C25"; $resn_exch{"2H4  C25"}=" H42 C25"; $resn_exch{"1H1C C26"}="H1C1 C26";
$resn_exch{"2H1C C26"}="H1C2 C26"; $resn_exch{"1H7C C26"}="H7C1 C26"; $resn_exch{"2H7C C26"}="H7C2 C26";
$resn_exch{"1H8C C26"}="H8C1 C26"; $resn_exch{"2H8C C26"}="H8C2 C26"; $resn_exch{"3H8C C26"}="H8C3 C26";
$resn_exch{"1H3C C26"}="H3C1 C26"; $resn_exch{"2H3C C26"}="H3C2 C26"; $resn_exch{"1H4C C26"}="H4C1 C26";
$resn_exch{"2H4C C26"}="H4C2 C26"; $resn_exch{"1H5C C26"}="H5C1 C26"; $resn_exch{"2H5C C26"}="H5C2 C26";
$resn_exch{"1H6C C26"}="H6C1 C26"; $resn_exch{"2H6C C26"}="H6C2 C26"; $resn_exch{"3H6C C26"}="H6C3 C26";
$resn_exch{"1H1C C27"}="H1C1 C27"; $resn_exch{"2H1C C27"}="H1C2 C27"; $resn_exch{"3H1C C27"}="H1C3 C27";
$resn_exch{"1H14 C27"}="H141 C27"; $resn_exch{"2H14 C27"}="H142 C27"; $resn_exch{"1H15 C27"}="H151 C27";
$resn_exch{"2H15 C27"}="H152 C27"; $resn_exch{"1H17 C27"}="H171 C27"; $resn_exch{"2H17 C27"}="H172 C27";
$resn_exch{"3H17 C27"}="H173 C27"; $resn_exch{"1H18 C27"}="H181 C27"; $resn_exch{"2H18 C27"}="H182 C27";
$resn_exch{"1H22 C27"}="H221 C27"; $resn_exch{"2H22 C27"}="H222 C27"; $resn_exch{"3H22 C27"}="H223 C27";
$resn_exch{"1H25 C27"}="H251 C27"; $resn_exch{"2H25 C27"}="H252 C27"; $resn_exch{"1H13 C28"}="H131 C28";
$resn_exch{"2H13 C28"}="H132 C28"; $resn_exch{"1H19 C28"}="H191 C28"; $resn_exch{"2H19 C28"}="H192 C28";
$resn_exch{"1H20 C28"}="H201 C28"; $resn_exch{"2H20 C28"}="H202 C28"; $resn_exch{"1H26 C28"}="H261 C28";
$resn_exch{"2H26 C28"}="H262 C28"; $resn_exch{"1H27 C28"}="H271 C28"; $resn_exch{"2H27 C28"}="H272 C28";
$resn_exch{"1H29 C28"}="H291 C28"; $resn_exch{"2H29 C28"}="H292 C28"; $resn_exch{"1H30 C28"}="H301 C28";
$resn_exch{"2H30 C28"}="H302 C28"; $resn_exch{"1H32 C28"}="H321 C28"; $resn_exch{"2H32 C28"}="H322 C28";
$resn_exch{"1H31 C28"}="H311 C28"; $resn_exch{"2H31 C28"}="H312 C28"; $resn_exch{"1HN  C2A"}=" HN1 C2A";
$resn_exch{"2HN  C2A"}=" HN2 C2A"; $resn_exch{"1HC  C2A"}=" HC1 C2A"; $resn_exch{"2HC  C2A"}=" HC2 C2A";
$resn_exch{"1HN1 C2B"}="HN11 C2B"; $resn_exch{"2HN1 C2B"}="HN12 C2B"; $resn_exch{"1HB  C2B"}=" HB1 C2B";
$resn_exch{"2HB  C2B"}=" HB2 C2B"; $resn_exch{"1H13 C2D"}="H131 C2D"; $resn_exch{"2H13 C2D"}="H132 C2D";
$resn_exch{"1H15 C2D"}="H151 C2D"; $resn_exch{"2H15 C2D"}="H152 C2D"; $resn_exch{"1H16 C2D"}="H161 C2D";
$resn_exch{"2H16 C2D"}="H162 C2D"; $resn_exch{"17H1 C2D"}="H117 C2D"; $resn_exch{"1H19 C2D"}="H191 C2D";
$resn_exch{"20H2 C2D"}="H220 C2D"; $resn_exch{"1H24 C2D"}="H241 C2D"; $resn_exch{"2H24 C2D"}="H242 C2D";
$resn_exch{"1H25 C2D"}="H251 C2D"; $resn_exch{"2H25 C2D"}="H252 C2D"; $resn_exch{"1H26 C2D"}="H261 C2D";
$resn_exch{"27H2 C2D"}="H227 C2D"; $resn_exch{"1H28 C2D"}="H281 C2D"; $resn_exch{"1H30 C2D"}="H301 C2D";
$resn_exch{"1H31 C2D"}="H311 C2D"; $resn_exch{"1HN2 C2F"}="HN21 C2F"; $resn_exch{"2HN2 C2F"}="HN22 C2F";
$resn_exch{"1H7  C2F"}=" H71 C2F"; $resn_exch{"2H7  C2F"}=" H72 C2F"; $resn_exch{"1H9  C2F"}=" H91 C2F";
$resn_exch{"2H9  C2F"}=" H92 C2F"; $resn_exch{"1H11 C2F"}="H111 C2F"; $resn_exch{"2H11 C2F"}="H112 C2F";
$resn_exch{"3H11 C2F"}="H113 C2F"; $resn_exch{"1HB  C2F"}=" HB1 C2F"; $resn_exch{"2HB  C2F"}=" HB2 C2F";
$resn_exch{"1HG  C2F"}=" HG1 C2F"; $resn_exch{"2HG  C2F"}=" HG2 C2F"; $resn_exch{"2HOE C2F"}="HOE2 C2F";
$resn_exch{"1HN4 C2G"}="HN41 C2G"; $resn_exch{"2HN4 C2G"}="HN42 C2G"; $resn_exch{"1HC5 C2G"}="HC51 C2G";
$resn_exch{"2HC5 C2G"}="HC52 C2G"; $resn_exch{"1H1G C2G"}="H1G1 C2G"; $resn_exch{"2H1G C2G"}="H1G2 C2G";
$resn_exch{"1H3G C2G"}="H3G1 C2G"; $resn_exch{"2H3G C2G"}="H3G2 C2G"; $resn_exch{"3HOP C2L"}="HOP3 C2L";
$resn_exch{"2HOP C2L"}="HOP2 C2L"; $resn_exch{"1H5* C2L"}=" H5' C2L"; $resn_exch{"2H5* C2L"}="H5'' C2L";
$resn_exch{" H4* C2L"}=" H4' C2L"; $resn_exch{" H3* C2L"}=" H3' C2L"; $resn_exch{" H2* C2L"}=" H2' C2L";
$resn_exch{"2HO* C2L"}="HO2' C2L"; $resn_exch{" H1* C2L"}=" H1' C2L"; $resn_exch{"1H4  C2L"}=" H41 C2L";
$resn_exch{"2H4  C2L"}=" H42 C2L"; $resn_exch{"1HA* C2L"}="HA'1 C2L"; $resn_exch{"2HA* C2L"}="HA'2 C2L";
$resn_exch{"1HB* C2L"}="HB'1 C2L"; $resn_exch{"2HB* C2L"}="HB'2 C2L"; $resn_exch{"1HD* C2L"}="HD'1 C2L";
$resn_exch{"2HD* C2L"}="HD'2 C2L"; $resn_exch{"3HD* C2L"}="HD'3 C2L"; $resn_exch{"1H7  C2L"}=" H71 C2L";
$resn_exch{"2H7  C2L"}=" H72 C2L"; $resn_exch{"3H7  C2L"}=" H73 C2L"; $resn_exch{"1HN3 C2M"}="HN31 C2M";
$resn_exch{"2HN3 C2M"}="HN32 C2M"; $resn_exch{"1HN4 C2M"}="HN41 C2M"; $resn_exch{"2HN4 C2M"}="HN42 C2M";
$resn_exch{"1H12 C2M"}="H121 C2M"; $resn_exch{"2H12 C2M"}="H122 C2M"; $resn_exch{"3H12 C2M"}="H123 C2M";
$resn_exch{"1HB  C2N"}=" HB1 C2N"; $resn_exch{"2HB  C2N"}=" HB2 C2N"; $resn_exch{"1HN  C2N"}=" HN1 C2N";
$resn_exch{"2HN  C2N"}=" HN2 C2N"; $resn_exch{"3HN  C2N"}=" HN3 C2N"; $resn_exch{"1HN4 C2P"}="HN41 C2P";
$resn_exch{"2HN4 C2P"}="HN42 C2P"; $resn_exch{" H1* C2P"}=" H1' C2P"; $resn_exch{" H2* C2P"}=" H2' C2P";
$resn_exch{" H3* C2P"}=" H3' C2P"; $resn_exch{" H4* C2P"}=" H4' C2P"; $resn_exch{"*HO3 C2P"}="HO3' C2P";
$resn_exch{"1H5* C2P"}="H5'1 C2P"; $resn_exch{"2H5* C2P"}="H5'2 C2P"; $resn_exch{"*HO5 C2P"}="HO5' C2P";
$resn_exch{"2HOP C2P"}="HOP2 C2P"; $resn_exch{"3HOP C2P"}="HOP3 C2P"; $resn_exch{" H2* C2R"}=" H2' C2R";
$resn_exch{" H3* C2R"}=" H3' C2R"; $resn_exch{"1H5* C2R"}="H5'1 C2R"; $resn_exch{"2H5* C2R"}="H5'2 C2R";
$resn_exch{" H4* C2R"}=" H4' C2R"; $resn_exch{" H1* C2R"}=" H1' C2R"; $resn_exch{"1HN5 C2R"}="HN51 C2R";
$resn_exch{"2HN5 C2R"}="HN52 C2R"; $resn_exch{"3HOP C2S"}="HOP3 C2S"; $resn_exch{"2HOP C2S"}="HOP2 C2S";
$resn_exch{"1H5* C2S"}=" H5' C2S"; $resn_exch{"2H5* C2S"}="H5'' C2S"; $resn_exch{" H4* C2S"}=" H4' C2S";
$resn_exch{" H3* C2S"}=" H3' C2S"; $resn_exch{"H3T  C2S"}="HO3' C2S"; $resn_exch{"1H2* C2S"}="H2'1 C2S";
$resn_exch{" H1* C2S"}=" H1' C2S"; $resn_exch{"1H4  C2S"}=" H41 C2S"; $resn_exch{"2H4  C2S"}=" H42 C2S";
$resn_exch{"2H2* C2S"}="H2'2 C2S"; $resn_exch{"1H5* C31"}="H5'1 C31"; $resn_exch{"2H5* C31"}="H5'2 C31";
$resn_exch{" H4* C31"}=" H4' C31"; $resn_exch{" H3* C31"}=" H3' C31"; $resn_exch{" H2* C31"}=" H2' C31";
$resn_exch{" H1* C31"}=" H1' C31"; $resn_exch{"1H4N C31"}="H4N1 C31"; $resn_exch{"2H4N C31"}="H4N2 C31";
$resn_exch{"1HA* C31"}="HA'1 C31"; $resn_exch{"2HA* C31"}="HA'2 C31"; $resn_exch{"1HB* C31"}="HB'1 C31";
$resn_exch{"2HB* C31"}="HB'2 C31"; $resn_exch{"1HCC C31"}="HCC1 C31"; $resn_exch{"2HCC C31"}="HCC2 C31";
$resn_exch{"1HD* C31"}="HD'1 C31"; $resn_exch{"2HD* C31"}="HD'2 C31"; $resn_exch{"1H5* C32"}="H5'1 C32";
$resn_exch{"2H5* C32"}="H5'2 C32"; $resn_exch{" H4* C32"}=" H4' C32"; $resn_exch{" H3* C32"}=" H3' C32";
$resn_exch{"1H2* C32"}="H2'1 C32"; $resn_exch{"2H2* C32"}="H2'2 C32"; $resn_exch{" H1* C32"}=" H1' C32";
$resn_exch{"1H4N C32"}="H4N1 C32"; $resn_exch{"2H4N C32"}="H4N2 C32"; $resn_exch{"1H4  C33"}=" H41 C33";
$resn_exch{"2H4  C33"}=" H42 C33"; $resn_exch{"1H12 C33"}="H121 C33"; $resn_exch{"2H12 C33"}="H122 C33";
$resn_exch{"1H11 C33"}="H111 C33"; $resn_exch{"2H11 C33"}="H112 C33"; $resn_exch{"1H9  C33"}=" H91 C33";
$resn_exch{"2H9  C33"}=" H92 C33"; $resn_exch{"1H15 C33"}="H151 C33"; $resn_exch{"2H15 C33"}="H152 C33";
$resn_exch{"1H16 C33"}="H161 C33"; $resn_exch{"2H16 C33"}="H162 C33"; $resn_exch{"1H19 C33"}="H191 C33";
$resn_exch{"2H19 C33"}="H192 C33"; $resn_exch{"1H22 C33"}="H221 C33"; $resn_exch{"2H22 C33"}="H222 C33";
$resn_exch{"1H18 C33"}="H181 C33"; $resn_exch{"2H18 C33"}="H182 C33"; $resn_exch{"3H18 C33"}="H183 C33";
$resn_exch{"1H21 C33"}="H211 C33"; $resn_exch{"2H21 C33"}="H212 C33"; $resn_exch{"3H21 C33"}="H213 C33";
$resn_exch{"1H23 C33"}="H231 C33"; $resn_exch{"2H23 C33"}="H232 C33"; $resn_exch{"1H24 C33"}="H241 C33";
$resn_exch{"2H24 C33"}="H242 C33"; $resn_exch{"1H27 C33"}="H271 C33"; $resn_exch{"2H27 C33"}="H272 C33";
$resn_exch{"3H27 C33"}="H273 C33"; $resn_exch{"1H26 C33"}="H261 C33"; $resn_exch{"2H26 C33"}="H262 C33";
$resn_exch{"3H26 C33"}="H263 C33"; $resn_exch{"1H28 C33"}="H281 C33"; $resn_exch{"2H28 C33"}="H282 C33";
$resn_exch{"1H29 C33"}="H291 C33"; $resn_exch{"2H29 C33"}="H292 C33"; $resn_exch{"1H30 C33"}="H301 C33";
$resn_exch{"2H30 C33"}="H302 C33"; $resn_exch{"3H30 C33"}="H303 C33"; $resn_exch{"2HOP C34"}="HOP2 C34";
$resn_exch{"3HOP C34"}="HOP3 C34"; $resn_exch{"1H5* C34"}=" H5' C34"; $resn_exch{"2H5* C34"}="H5'' C34";
$resn_exch{" H4* C34"}=" H4' C34"; $resn_exch{" H1* C34"}=" H1' C34"; $resn_exch{"1H7C C34"}="H7C1 C34";
$resn_exch{"2H7C C34"}="H7C2 C34"; $resn_exch{"3H7C C34"}="H7C3 C34"; $resn_exch{"1H2* C34"}=" H2' C34";
$resn_exch{"2H2* C34"}="H2'' C34"; $resn_exch{" H3* C34"}=" H3' C34"; $resn_exch{"H3T  C34"}="HO3' C34";
$resn_exch{"1H31 C35"}="H311 C35"; $resn_exch{"2H31 C35"}="H312 C35"; $resn_exch{"3H31 C35"}="H313 C35";
$resn_exch{"1H24 C35"}="H241 C35"; $resn_exch{"2H24 C35"}="H242 C35"; $resn_exch{"1H25 C35"}="H251 C35";
$resn_exch{"2H25 C35"}="H252 C35"; $resn_exch{"1H26 C35"}="H261 C35"; $resn_exch{"2H26 C35"}="H262 C35";
$resn_exch{"1H6N C35"}="H6N1 C35"; $resn_exch{"2H6N C35"}="H6N2 C35"; $resn_exch{"1H7N C35"}="H7N1 C35";
$resn_exch{"2H7N C35"}="H7N2 C35"; $resn_exch{"1H18 C35"}="H181 C35"; $resn_exch{"2H18 C35"}="H182 C35";
$resn_exch{"1H20 C35"}="H201 C35"; $resn_exch{"2H20 C35"}="H202 C35"; $resn_exch{"3H20 C35"}="H203 C35";
$resn_exch{"1H21 C35"}="H211 C35"; $resn_exch{"2H21 C35"}="H212 C35"; $resn_exch{"3H21 C35"}="H213 C35";
$resn_exch{"1H15 C35"}="H151 C35"; $resn_exch{"2H15 C35"}="H152 C35"; $resn_exch{"3H15 C35"}="H153 C35";
$resn_exch{"1H13 C35"}="H131 C35"; $resn_exch{"2H13 C35"}="H132 C35"; $resn_exch{"1H14 C35"}="H141 C35";
$resn_exch{"2H14 C35"}="H142 C35"; $resn_exch{"3H14 C35"}="H143 C35"; $resn_exch{"1H7C C35"}="H7C1 C35";
$resn_exch{"2H7C C35"}="H7C2 C35"; $resn_exch{"1H1N C35"}="H1N1 C35"; $resn_exch{"2H1N C35"}="H1N2 C35";
$resn_exch{"1H5* C36"}="H5'1 C36"; $resn_exch{"2H5* C36"}="H5'2 C36"; $resn_exch{" H4* C36"}=" H4' C36";
$resn_exch{" H3* C36"}=" H3' C36"; $resn_exch{"1H2* C36"}="H2'1 C36"; $resn_exch{"2H2* C36"}="H2'2 C36";
$resn_exch{" H1* C36"}=" H1' C36"; $resn_exch{"1H4N C36"}="H4N1 C36"; $resn_exch{"2H4N C36"}="H4N2 C36";
$resn_exch{"1H5* C37"}="H5'1 C37"; $resn_exch{"2H5* C37"}="H5'2 C37"; $resn_exch{" H4* C37"}=" H4' C37";
$resn_exch{" H3* C37"}=" H3' C37"; $resn_exch{"1H2* C37"}="H2'1 C37"; $resn_exch{"2H2* C37"}="H2'2 C37";
$resn_exch{" H1* C37"}=" H1' C37"; $resn_exch{"1H4N C37"}="H4N1 C37"; $resn_exch{"2H4N C37"}="H4N2 C37";
$resn_exch{"1H5* C38"}="H5'1 C38"; $resn_exch{"2H5* C38"}="H5'2 C38"; $resn_exch{" H4* C38"}=" H4' C38";
$resn_exch{" H3* C38"}=" H3' C38"; $resn_exch{"1H2* C38"}="H2'1 C38"; $resn_exch{"2H2* C38"}="H2'2 C38";
$resn_exch{" H1* C38"}=" H1' C38"; $resn_exch{"1H4N C38"}="H4N1 C38"; $resn_exch{"2H4N C38"}="H4N2 C38";
$resn_exch{"1H1C C39"}="H1C1 C39"; $resn_exch{"2H1C C39"}="H1C2 C39"; $resn_exch{"3H1C C39"}="H1C3 C39";
$resn_exch{"1H3C C39"}="H3C1 C39"; $resn_exch{"2H3C C39"}="H3C2 C39"; $resn_exch{"1H4C C39"}="H4C1 C39";
$resn_exch{"2H4C C39"}="H4C2 C39"; $resn_exch{"1H5C C39"}="H5C1 C39"; $resn_exch{"2H5C C39"}="H5C2 C39";
$resn_exch{"1H16 C39"}="H161 C39"; $resn_exch{"2H16 C39"}="H162 C39"; $resn_exch{"1H21 C39"}="H211 C39";
$resn_exch{"2H21 C39"}="H212 C39"; $resn_exch{"1H19 C39"}="H191 C39"; $resn_exch{"2H19 C39"}="H192 C39";
$resn_exch{"3H19 C39"}="H193 C39"; $resn_exch{"1H20 C39"}="H201 C39"; $resn_exch{"2H20 C39"}="H202 C39";
$resn_exch{"3H20 C39"}="H203 C39"; $resn_exch{"1H23 C39"}="H231 C39"; $resn_exch{"2H23 C39"}="H232 C39";
$resn_exch{"1H26 C39"}="H261 C39"; $resn_exch{"2H26 C39"}="H262 C39"; $resn_exch{"1H28 C39"}="H281 C39";
$resn_exch{"2H28 C39"}="H282 C39"; $resn_exch{"3H28 C39"}="H283 C39"; $resn_exch{"1H32 C39"}="H321 C39";
$resn_exch{"2H32 C39"}="H322 C39"; $resn_exch{"1H33 C39"}="H331 C39"; $resn_exch{"2H33 C39"}="H332 C39";
$resn_exch{"1H34 C39"}="H341 C39"; $resn_exch{"2H34 C39"}="H342 C39"; $resn_exch{"1H35 C39"}="H351 C39";
$resn_exch{"2H35 C39"}="H352 C39"; $resn_exch{"3H35 C39"}="H353 C39"; $resn_exch{"1H1  C3A"}="H11A C3A";
$resn_exch{"2H1  C3A"}=" H12 C3A"; $resn_exch{"1H3  C3A"}="H31A C3A"; $resn_exch{"2H3  C3A"}="H32A C3A";
$resn_exch{"1H9  C3A"}=" H91 C3A"; $resn_exch{"2H9  C3A"}=" H92 C3A"; $resn_exch{"1H10 C3A"}="H101 C3A";
$resn_exch{"2H10 C3A"}="H102 C3A"; $resn_exch{"1H12 C3A"}="H121 C3A"; $resn_exch{"2H12 C3A"}="H122 C3A";
$resn_exch{"1H13 C3A"}="H131 C3A"; $resn_exch{"2H13 C3A"}="H132 C3A"; $resn_exch{"1H15 C3A"}="H151 C3A";
$resn_exch{"2H15 C3A"}="H152 C3A"; $resn_exch{"1H22 C3A"}="H221 C3A"; $resn_exch{"2H22 C3A"}="H222 C3A";
$resn_exch{"1H33 C3A"}="H331 C3A"; $resn_exch{"2H33 C3A"}="H332 C3A"; $resn_exch{"1H40 C3A"}="H401 C3A";
$resn_exch{"2H40 C3A"}="H402 C3A"; $resn_exch{"1H3  C3D"}=" H31 C3D"; $resn_exch{"2H3  C3D"}=" H32 C3D";
$resn_exch{"1H4  C3D"}=" H41 C3D"; $resn_exch{"2H4  C3D"}=" H42 C3D"; $resn_exch{"1H24 C3D"}="H241 C3D";
$resn_exch{"2H24 C3D"}="H242 C3D"; $resn_exch{"1H25 C3D"}="H251 C3D"; $resn_exch{"2H25 C3D"}="H252 C3D";
$resn_exch{"1H27 C3D"}="H271 C3D"; $resn_exch{"2H27 C3D"}="H272 C3D"; $resn_exch{"1H28 C3D"}="H281 C3D";
$resn_exch{"2H28 C3D"}="H282 C3D"; $resn_exch{"1H29 C3D"}="H291 C3D"; $resn_exch{"2H29 C3D"}="H292 C3D";
$resn_exch{"1H30 C3D"}="H301 C3D"; $resn_exch{"2H30 C3D"}="H302 C3D"; $resn_exch{"1H1N C3M"}="H1N1 C3M";
$resn_exch{"2H1N C3M"}="H1N2 C3M"; $resn_exch{"1H3C C3M"}="H3C1 C3M"; $resn_exch{"2H3C C3M"}="H3C2 C3M";
$resn_exch{"1H12 C3M"}="H121 C3M"; $resn_exch{"2H12 C3M"}="H122 C3M"; $resn_exch{"1H4  C3O"}=" H41 C3O";
$resn_exch{"2H4  C3O"}=" H42 C3O"; $resn_exch{"1H12 C3O"}="H121 C3O"; $resn_exch{"2H12 C3O"}="H122 C3O";
$resn_exch{"1H11 C3O"}="H111 C3O"; $resn_exch{"2H11 C3O"}="H112 C3O"; $resn_exch{"1H9  C3O"}=" H91 C3O";
$resn_exch{"2H9  C3O"}=" H92 C3O"; $resn_exch{"1H15 C3O"}="H151 C3O"; $resn_exch{"2H15 C3O"}="H152 C3O";
$resn_exch{"1H16 C3O"}="H161 C3O"; $resn_exch{"2H16 C3O"}="H162 C3O"; $resn_exch{"1H19 C3O"}="H191 C3O";
$resn_exch{"2H19 C3O"}="H192 C3O"; $resn_exch{"1H22 C3O"}="H221 C3O"; $resn_exch{"2H22 C3O"}="H222 C3O";
$resn_exch{"1H18 C3O"}="H181 C3O"; $resn_exch{"2H18 C3O"}="H182 C3O"; $resn_exch{"3H18 C3O"}="H183 C3O";
$resn_exch{"1H21 C3O"}="H211 C3O"; $resn_exch{"2H21 C3O"}="H212 C3O"; $resn_exch{"3H21 C3O"}="H213 C3O";
$resn_exch{"1H23 C3O"}="H231 C3O"; $resn_exch{"2H23 C3O"}="H232 C3O"; $resn_exch{"1H24 C3O"}="H241 C3O";
$resn_exch{"2H24 C3O"}="H242 C3O"; $resn_exch{"1H27 C3O"}="H271 C3O"; $resn_exch{"2H27 C3O"}="H272 C3O";
$resn_exch{"3H27 C3O"}="H273 C3O"; $resn_exch{"1H26 C3O"}="H261 C3O"; $resn_exch{"2H26 C3O"}="H262 C3O";
$resn_exch{"3H26 C3O"}="H263 C3O"; $resn_exch{"1H29 C3O"}="H291 C3O"; $resn_exch{"2H29 C3O"}="H292 C3O";
$resn_exch{"1H30 C3O"}="H301 C3O"; $resn_exch{"2H30 C3O"}="H302 C3O"; $resn_exch{"1H31 C3O"}="H311 C3O";
$resn_exch{"2H31 C3O"}="H312 C3O"; $resn_exch{"3H31 C3O"}="H313 C3O"; $resn_exch{"2HOP C3P"}="HOP2 C3P";
$resn_exch{"3HOP C3P"}="HOP3 C3P"; $resn_exch{"*HO5 C3P"}="HO5' C3P"; $resn_exch{"1H5* C3P"}="H5'1 C3P";
$resn_exch{"2H5* C3P"}="H5'2 C3P"; $resn_exch{" H4* C3P"}=" H4' C3P"; $resn_exch{" H3* C3P"}=" H3' C3P";
$resn_exch{"1H2* C3P"}="H2'1 C3P"; $resn_exch{"*HO2 C3P"}="HO2' C3P"; $resn_exch{" H1* C3P"}=" H1' C3P";
$resn_exch{"1HN4 C3P"}="HN41 C3P"; $resn_exch{"2HN4 C3P"}="HN42 C3P"; $resn_exch{"1H65 C3S"}="H651 C3S";
$resn_exch{"2H65 C3S"}="H652 C3S"; $resn_exch{"3H65 C3S"}="H653 C3S"; $resn_exch{"1H69 C3S"}="H691 C3S";
$resn_exch{"2H69 C3S"}="H692 C3S"; $resn_exch{"3H69 C3S"}="H693 C3S"; $resn_exch{"1H60 C3S"}="H601 C3S";
$resn_exch{"2H60 C3S"}="H602 C3S"; $resn_exch{"1H57 C3S"}="H571 C3S"; $resn_exch{"2H57 C3S"}="H572 C3S";
$resn_exch{"1H54 C3S"}="H541 C3S"; $resn_exch{"2H54 C3S"}="H542 C3S"; $resn_exch{"1H50 C3S"}="H501 C3S";
$resn_exch{"2H50 C3S"}="H502 C3S"; $resn_exch{"3H50 C3S"}="H503 C3S"; $resn_exch{"1H35 C3S"}="H351 C3S";
$resn_exch{"2H35 C3S"}="H352 C3S"; $resn_exch{"1H26 C3S"}="H261 C3S"; $resn_exch{"2H26 C3S"}="H262 C3S";
$resn_exch{"1H40 C3S"}="H401 C3S"; $resn_exch{"2H40 C3S"}="H402 C3S"; $resn_exch{"3H40 C3S"}="H403 C3S";
$resn_exch{"1H32 C3S"}="H321 C3S"; $resn_exch{"2H32 C3S"}="H322 C3S"; $resn_exch{"1H15 C3S"}="H151 C3S";
$resn_exch{"2H15 C3S"}="H152 C3S"; $resn_exch{"1H23 C3S"}="H231 C3S"; $resn_exch{"2H23 C3S"}="H232 C3S";
$resn_exch{"1H44 C3S"}="H441 C3S"; $resn_exch{"2H44 C3S"}="H442 C3S"; $resn_exch{"3H44 C3S"}="H443 C3S";
$resn_exch{"1H9  C3S"}=" H91 C3S"; $resn_exch{"2H9  C3S"}=" H92 C3S"; $resn_exch{"1H1  C3S"}=" H11 C3S";
$resn_exch{"2H1  C3S"}=" H12 C3S"; $resn_exch{"1H4  C3S"}=" H41 C3S"; $resn_exch{"2H4  C3S"}=" H42 C3S";
$resn_exch{"1H5  C3X"}=" H51 C3X"; $resn_exch{"2H5  C3X"}=" H52 C3X"; $resn_exch{"1H1' C3X"}="H1'1 C3X";
$resn_exch{"2H1' C3X"}="H1'2 C3X"; $resn_exch{"1H3' C3X"}="H3'1 C3X"; $resn_exch{"2H3' C3X"}="H3'2 C3X";
$resn_exch{"1HN  C3Y"}=" HN1 C3Y"; $resn_exch{"2HN  C3Y"}=" HN2 C3Y"; $resn_exch{"1HBC C3Y"}="HBC1 C3Y";
$resn_exch{"2HBC C3Y"}="HBC2 C3Y"; $resn_exch{"1H1C C3Y"}="H1C1 C3Y"; $resn_exch{"2H1C C3Y"}="H1C2 C3Y";
$resn_exch{"3H1C C3Y"}="H1C3 C3Y"; $resn_exch{"1H1C C40"}="H1C1 C40"; $resn_exch{"2H1C C40"}="H1C2 C40";
$resn_exch{"3H1C C40"}="H1C3 C40"; $resn_exch{"1H3C C40"}="H3C1 C40"; $resn_exch{"2H3C C40"}="H3C2 C40";
$resn_exch{"1H4C C40"}="H4C1 C40"; $resn_exch{"2H4C C40"}="H4C2 C40"; $resn_exch{"1H5C C40"}="H5C1 C40";
$resn_exch{"2H5C C40"}="H5C2 C40"; $resn_exch{"1H16 C40"}="H161 C40"; $resn_exch{"2H16 C40"}="H162 C40";
$resn_exch{"1H18 C40"}="H181 C40"; $resn_exch{"2H18 C40"}="H182 C40"; $resn_exch{"3H18 C40"}="H183 C40";
$resn_exch{"1H19 C40"}="H191 C40"; $resn_exch{"2H19 C40"}="H192 C40"; $resn_exch{"1H21 C40"}="H211 C40";
$resn_exch{"2H21 C40"}="H212 C40"; $resn_exch{"1H24 C40"}="H241 C40"; $resn_exch{"2H24 C40"}="H242 C40";
$resn_exch{"1H26 C40"}="H261 C40"; $resn_exch{"2H26 C40"}="H262 C40"; $resn_exch{"3H26 C40"}="H263 C40";
$resn_exch{"1H30 C40"}="H301 C40"; $resn_exch{"2H30 C40"}="H302 C40"; $resn_exch{"1H31 C40"}="H311 C40";
$resn_exch{"2H31 C40"}="H312 C40"; $resn_exch{"1H32 C40"}="H321 C40"; $resn_exch{"2H32 C40"}="H322 C40";
$resn_exch{"1H33 C40"}="H331 C40"; $resn_exch{"2H33 C40"}="H332 C40"; $resn_exch{"3H33 C40"}="H333 C40";
$resn_exch{"1H1C C41"}="H1C1 C41"; $resn_exch{"2H1C C41"}="H1C2 C41"; $resn_exch{"3H1C C41"}="H1C3 C41";
$resn_exch{"1H3C C41"}="H3C1 C41"; $resn_exch{"2H3C C41"}="H3C2 C41"; $resn_exch{"1H4C C41"}="H4C1 C41";
$resn_exch{"2H4C C41"}="H4C2 C41"; $resn_exch{"1H5C C41"}="H5C1 C41"; $resn_exch{"2H5C C41"}="H5C2 C41";
$resn_exch{"1H15 C41"}="H151 C41"; $resn_exch{"2H15 C41"}="H152 C41"; $resn_exch{"1H14 C41"}="H141 C41";
$resn_exch{"2H14 C41"}="H142 C41"; $resn_exch{"3H14 C41"}="H143 C41"; $resn_exch{"1H20 C41"}="H201 C41";
$resn_exch{"2H20 C41"}="H202 C41"; $resn_exch{"1H18 C41"}="H181 C41"; $resn_exch{"2H18 C41"}="H182 C41";
$resn_exch{"3H18 C41"}="H183 C41"; $resn_exch{"1H19 C41"}="H191 C41"; $resn_exch{"2H19 C41"}="H192 C41";
$resn_exch{"3H19 C41"}="H193 C41"; $resn_exch{"1H22 C41"}="H221 C41"; $resn_exch{"2H22 C41"}="H222 C41";
$resn_exch{"1H25 C41"}="H251 C41"; $resn_exch{"2H25 C41"}="H252 C41"; $resn_exch{"1H28 C41"}="H281 C41";
$resn_exch{"2H28 C41"}="H282 C41"; $resn_exch{"3H28 C41"}="H283 C41"; $resn_exch{"1H29 C41"}="H291 C41";
$resn_exch{"2H29 C41"}="H292 C41"; $resn_exch{"3H29 C41"}="H293 C41"; $resn_exch{"1H33 C41"}="H331 C41";
$resn_exch{"2H33 C41"}="H332 C41"; $resn_exch{"1H35 C41"}="H351 C41"; $resn_exch{"2H35 C41"}="H352 C41";
$resn_exch{"3H35 C41"}="H353 C41"; $resn_exch{"1H36 C41"}="H361 C41"; $resn_exch{"2H36 C41"}="H362 C41";
$resn_exch{"3H36 C41"}="H363 C41"; $resn_exch{"1H39 C41"}="H391 C41"; $resn_exch{"2H39 C41"}="H392 C41";
$resn_exch{"1H4N C42"}="H4N1 C42"; $resn_exch{"2H4N C42"}="H4N2 C42"; $resn_exch{" H1* C42"}=" H1' C42";
$resn_exch{"1H2* C42"}="H2'1 C42"; $resn_exch{"2H2* C42"}="H2'2 C42"; $resn_exch{" H3* C42"}=" H3' C42";
$resn_exch{" H4* C42"}=" H4' C42"; $resn_exch{"1H5* C42"}="H5'1 C42"; $resn_exch{"2H5* C42"}="H5'2 C42";
$resn_exch{"1HN  C42"}=" HN1 C42"; $resn_exch{"2HN  C42"}=" HN2 C42"; $resn_exch{"1H5* C43"}="H5'1 C43";
$resn_exch{"2H5* C43"}="H5'2 C43"; $resn_exch{" H4* C43"}=" H4' C43"; $resn_exch{" H3* C43"}=" H3' C43";
$resn_exch{" H2* C43"}=" H2' C43"; $resn_exch{" H1* C43"}=" H1' C43"; $resn_exch{"1H4N C43"}="H4N1 C43";
$resn_exch{"2H4N C43"}="H4N2 C43"; $resn_exch{"1HA* C43"}="HA'1 C43"; $resn_exch{"2HA* C43"}="HA'2 C43";
$resn_exch{"1HB* C43"}="HB'1 C43"; $resn_exch{"2HB* C43"}="HB'2 C43"; $resn_exch{"1HD* C43"}="HD'1 C43";
$resn_exch{"2HD* C43"}="HD'2 C43"; $resn_exch{"3HD* C43"}="HD'3 C43"; $resn_exch{"1H5C C44"}="H5C1 C44";
$resn_exch{"2H5C C44"}="H5C2 C44"; $resn_exch{"1H6C C44"}="H6C1 C44"; $resn_exch{"2H6C C44"}="H6C2 C44";
$resn_exch{"1H33 C44"}="H331 C44"; $resn_exch{"2H33 C44"}="H332 C44"; $resn_exch{"1H34 C44"}="H341 C44";
$resn_exch{"2H34 C44"}="H342 C44"; $resn_exch{"3H34 C44"}="H343 C44"; $resn_exch{"1H42 C44"}="H421 C44";
$resn_exch{"2H42 C44"}="H422 C44"; $resn_exch{"1H43 C44"}="H431 C44"; $resn_exch{"2H43 C44"}="H432 C44";
$resn_exch{"1H44 C44"}="H441 C44"; $resn_exch{"2H44 C44"}="H442 C44"; $resn_exch{"1H45 C44"}="H451 C44";
$resn_exch{"2H45 C44"}="H452 C44"; $resn_exch{"1H56 C44"}="H561 C44"; $resn_exch{"2H56 C44"}="H562 C44";
$resn_exch{"1H57 C44"}="H571 C44"; $resn_exch{"2H57 C44"}="H572 C44"; $resn_exch{"1H58 C44"}="H581 C44";
$resn_exch{"2H58 C44"}="H582 C44"; $resn_exch{" H1* C45"}=" H1' C45"; $resn_exch{"1H2* C45"}="H2'1 C45";
$resn_exch{"2H2* C45"}="H2'2 C45"; $resn_exch{"1HM2 C45"}="HM21 C45"; $resn_exch{"2HM2 C45"}="HM22 C45";
$resn_exch{"3HM2 C45"}="HM23 C45"; $resn_exch{" H3* C45"}=" H3' C45"; $resn_exch{" H4* C45"}=" H4' C45";
$resn_exch{"1H5* C45"}="H5'1 C45"; $resn_exch{"2H5* C45"}="H5'2 C45"; $resn_exch{"1H5* C46"}="H5'1 C46";
$resn_exch{"2H5* C46"}="H5'2 C46"; $resn_exch{" H4* C46"}=" H4' C46"; $resn_exch{" H3* C46"}=" H3' C46";
$resn_exch{"1H2* C46"}="H2'1 C46"; $resn_exch{"2H2* C46"}="H2'2 C46"; $resn_exch{" H1* C46"}=" H1' C46";
$resn_exch{"1HM5 C46"}="HM51 C46"; $resn_exch{"2HM5 C46"}="HM52 C46"; $resn_exch{"1H7C C46"}="H7C1 C46";
$resn_exch{"2H7C C46"}="H7C2 C46"; $resn_exch{"1H10 C48"}="H101 C48"; $resn_exch{"2H10 C48"}="H102 C48";
$resn_exch{"1H14 C48"}="H141 C48"; $resn_exch{"2H14 C48"}="H142 C48"; $resn_exch{"1H22 C48"}="H221 C48";
$resn_exch{"2H22 C48"}="H222 C48"; $resn_exch{"3H22 C48"}="H223 C48"; $resn_exch{"1H23 C48"}="H231 C48";
$resn_exch{"2H23 C48"}="H232 C48"; $resn_exch{"3H23 C48"}="H233 C48"; $resn_exch{"1H24 C48"}="H241 C48";
$resn_exch{"2H24 C48"}="H242 C48"; $resn_exch{"1H25 C48"}="H251 C48"; $resn_exch{"2H25 C48"}="H252 C48";
$resn_exch{"1H5* C49"}="H5'1 C49"; $resn_exch{"2H5* C49"}="H5'2 C49"; $resn_exch{" H4* C49"}=" H4' C49";
$resn_exch{" H3* C49"}=" H3' C49"; $resn_exch{"1H2* C49"}="H2'1 C49"; $resn_exch{"2H2* C49"}="H2'2 C49";
$resn_exch{" H1* C49"}=" H1' C49"; $resn_exch{"1HM5 C49"}="HM51 C49"; $resn_exch{"2HM5 C49"}="HM52 C49";
$resn_exch{"3HM5 C49"}="HM53 C49"; $resn_exch{"1HN1 C4A"}="HN11 C4A"; $resn_exch{"2HN1 C4A"}="HN12 C4A";
$resn_exch{"1H2  C4A"}="H21A C4A"; $resn_exch{"2H2  C4A"}="H22A C4A"; $resn_exch{"1H4  C4A"}=" H41 C4A";
$resn_exch{"2H4  C4A"}=" H42 C4A"; $resn_exch{"1H5  C4A"}=" H51 C4A"; $resn_exch{"2H5  C4A"}=" H52 C4A";
$resn_exch{"1H3  C4A"}=" H31 C4A"; $resn_exch{"2H3  C4A"}=" H32 C4A"; $resn_exch{"1H11 C4A"}="H111 C4A";
$resn_exch{"2H11 C4A"}="H112 C4A"; $resn_exch{"1H12 C4A"}="H121 C4A"; $resn_exch{"2H12 C4A"}="H122 C4A";
$resn_exch{"3H12 C4A"}="H123 C4A"; $resn_exch{"1H19 C4A"}="H191 C4A"; $resn_exch{"2H19 C4A"}="H192 C4A";
$resn_exch{"1H30 C4A"}="H301 C4A"; $resn_exch{"2H30 C4A"}="H302 C4A"; $resn_exch{"1H33 C4A"}="H331 C4A";
$resn_exch{"2H33 C4A"}="H332 C4A"; $resn_exch{"1H5  C4H"}=" H51 C4H"; $resn_exch{"2H5  C4H"}=" H52 C4H";
$resn_exch{"3H5  C4H"}=" H53 C4H"; $resn_exch{"1H6  C4H"}=" H61 C4H"; $resn_exch{"2H6  C4H"}=" H62 C4H";
$resn_exch{"3H6  C4H"}=" H63 C4H"; $resn_exch{"1H16 C4H"}="H161 C4H"; $resn_exch{"2H16 C4H"}="H162 C4H";
$resn_exch{"1H17 C4H"}="H171 C4H"; $resn_exch{"2H17 C4H"}="H172 C4H"; $resn_exch{"1H18 C4H"}="H181 C4H";
$resn_exch{"2H18 C4H"}="H182 C4H"; $resn_exch{"1H19 C4H"}="H191 C4H"; $resn_exch{"2H19 C4H"}="H192 C4H";
$resn_exch{"1H23 C4H"}="H231 C4H"; $resn_exch{"2H23 C4H"}="H232 C4H"; $resn_exch{"1H24 C4H"}="H241 C4H";
$resn_exch{"2H24 C4H"}="H242 C4H"; $resn_exch{"1H1C C4M"}="H1C1 C4M"; $resn_exch{"2H1C C4M"}="H1C2 C4M";
$resn_exch{"3H1C C4M"}="H1C3 C4M"; $resn_exch{"1H6C C4M"}="H6C1 C4M"; $resn_exch{"2H6C C4M"}="H6C2 C4M";
$resn_exch{"3H6C C4M"}="H6C3 C4M"; $resn_exch{"1H9C C4M"}="H9C1 C4M"; $resn_exch{"2H9C C4M"}="H9C2 C4M";
$resn_exch{"3H9C C4M"}="H9C3 C4M"; $resn_exch{"1H11 C4M"}="H111 C4M"; $resn_exch{"2H11 C4M"}="H112 C4M";
$resn_exch{"3H11 C4M"}="H113 C4M"; $resn_exch{"1H16 C4M"}="H161 C4M"; $resn_exch{"2H16 C4M"}="H162 C4M";
$resn_exch{"1H20 C4M"}="H201 C4M"; $resn_exch{"2H20 C4M"}="H202 C4M"; $resn_exch{"1H21 C4M"}="H211 C4M";
$resn_exch{"2H21 C4M"}="H212 C4M"; $resn_exch{"1H2  C4P"}=" H21 C4P"; $resn_exch{"2H2  C4P"}=" H22 C4P";
$resn_exch{"1H3  C4P"}=" H31 C4P"; $resn_exch{"2H3  C4P"}=" H32 C4P"; $resn_exch{"1H5  C4P"}="H51A C4P";
$resn_exch{"2H5  C4P"}="H52A C4P"; $resn_exch{"1H6  C4P"}=" H61 C4P"; $resn_exch{"2H6  C4P"}=" H62 C4P";
$resn_exch{"1H58 C4P"}="H581 C4P"; $resn_exch{"2H58 C4P"}="H582 C4P"; $resn_exch{"1H1  C4P"}=" H11 C4P";
$resn_exch{"2H1  C4P"}=" H12 C4P"; $resn_exch{"1H47 C4P"}="H471 C4P"; $resn_exch{"2H47 C4P"}="H472 C4P";
$resn_exch{"1H5  C4X"}=" H51 C4X"; $resn_exch{"2H5  C4X"}=" H52 C4X"; $resn_exch{"1H1' C4X"}="H1'1 C4X";
$resn_exch{"2H1' C4X"}="H1'2 C4X"; $resn_exch{"1H2' C4X"}="H2'1 C4X"; $resn_exch{"2H2' C4X"}="H2'2 C4X";
$resn_exch{"1H4' C4X"}="H4'1 C4X"; $resn_exch{"2H4' C4X"}="H4'2 C4X"; $resn_exch{"1H6C C53"}="H6C1 C53";
$resn_exch{"1H3N C53"}="H3N1 C53"; $resn_exch{"1HN  C5A"}=" HN1 C5A"; $resn_exch{"2HN  C5A"}=" HN2 C5A";
$resn_exch{"1HB1 C5A"}="HB11 C5A"; $resn_exch{"2HB1 C5A"}="HB12 C5A"; $resn_exch{"1HB2 C5A"}="HB21 C5A";
$resn_exch{"2HB2 C5A"}="HB22 C5A"; $resn_exch{"1HG2 C5A"}="HG21 C5A"; $resn_exch{"2HG2 C5A"}="HG22 C5A";
$resn_exch{"1HN  C5B"}=" HN1 C5B"; $resn_exch{"2HN  C5B"}=" HN2 C5B"; $resn_exch{"1HB1 C5B"}="HB11 C5B";
$resn_exch{"2HB1 C5B"}="HB12 C5B"; $resn_exch{"1HB2 C5B"}="HB21 C5B"; $resn_exch{"2HB2 C5B"}="HB22 C5B";
$resn_exch{"1HG2 C5B"}="HG21 C5B"; $resn_exch{"2HG2 C5B"}="HG22 C5B"; $resn_exch{"1HN  C5C"}=" H   C5C";
$resn_exch{"2HN  C5C"}=" H2  C5C"; $resn_exch{"1HB  C5C"}=" HB2 C5C"; $resn_exch{"2HB  C5C"}=" HB3 C5C";
$resn_exch{"1H2  C5C"}=" H21 C5C"; $resn_exch{"2H2  C5C"}=" H22 C5C"; $resn_exch{"1H3  C5C"}=" H31 C5C";
$resn_exch{"2H3  C5C"}=" H32 C5C"; $resn_exch{"1H4  C5C"}=" H41 C5C"; $resn_exch{"2H4  C5C"}=" H42 C5C";
$resn_exch{"1H5  C5C"}=" H51 C5C"; $resn_exch{"2H5  C5C"}=" H52 C5C"; $resn_exch{"1H4  C5G"}=" H41 C5G";
$resn_exch{"2H4  C5G"}=" H42 C5G"; $resn_exch{" H1* C5G"}=" H1D C5G"; $resn_exch{" H2* C5G"}=" H2D C5G";
$resn_exch{"2HO* C5G"}="HO'2 C5G"; $resn_exch{" H3* C5G"}=" H3D C5G"; $resn_exch{" H4* C5G"}=" H4D C5G";
$resn_exch{"1H5* C5G"}="H5'1 C5G"; $resn_exch{"2H5* C5G"}="H5'2 C5G"; $resn_exch{"2HO' C5G"}=" HOD C5G";
$resn_exch{"3HO' C5G"}="HO'3 C5G"; $resn_exch{"4HO' C5G"}="HO'4 C5G"; $resn_exch{"1H6' C5G"}="H6'1 C5G";
$resn_exch{"2H6' C5G"}="H6'2 C5G"; $resn_exch{"6HO' C5G"}="HO'6 C5G"; $resn_exch{"1H1C C5M"}="H1C1 C5M";
$resn_exch{"2H1C C5M"}="H1C2 C5M"; $resn_exch{"3H1C C5M"}="H1C3 C5M"; $resn_exch{"1H6C C5M"}="H6C1 C5M";
$resn_exch{"2H6C C5M"}="H6C2 C5M"; $resn_exch{"3H6C C5M"}="H6C3 C5M"; $resn_exch{"1H9C C5M"}="H9C1 C5M";
$resn_exch{"2H9C C5M"}="H9C2 C5M"; $resn_exch{"3H9C C5M"}="H9C3 C5M"; $resn_exch{"1H11 C5M"}="H111 C5M";
$resn_exch{"2H11 C5M"}="H112 C5M"; $resn_exch{"3H11 C5M"}="H113 C5M"; $resn_exch{"1H16 C5M"}="H161 C5M";
$resn_exch{"2H16 C5M"}="H162 C5M"; $resn_exch{"1H20 C5M"}="H201 C5M"; $resn_exch{"2H20 C5M"}="H202 C5M";
$resn_exch{"1H28 C5M"}="H281 C5M"; $resn_exch{"2H28 C5M"}="H282 C5M"; $resn_exch{"3HOP C5P"}="HOP3 C5P";
$resn_exch{"2HOP C5P"}="HOP2 C5P"; $resn_exch{"1H5* C5P"}="H5'1 C5P"; $resn_exch{"2H5* C5P"}="H5'2 C5P";
$resn_exch{" H4* C5P"}=" H4' C5P"; $resn_exch{" H3* C5P"}=" H3' C5P"; $resn_exch{"*HO3 C5P"}="HO3' C5P";
$resn_exch{"1H2* C5P"}="H2'1 C5P"; $resn_exch{"*HO2 C5P"}="HO2' C5P"; $resn_exch{" H1* C5P"}=" H1' C5P";
$resn_exch{"1HN4 C5P"}="HN41 C5P"; $resn_exch{"2HN4 C5P"}="HN42 C5P"; $resn_exch{"1H5  C5X"}=" H51 C5X";
$resn_exch{"2H5  C5X"}=" H52 C5X"; $resn_exch{"1H1' C5X"}="H1'1 C5X"; $resn_exch{"2H1' C5X"}="H1'2 C5X";
$resn_exch{"1H2' C5X"}="H2'1 C5X"; $resn_exch{"2H2' C5X"}="H2'2 C5X"; $resn_exch{"1H3' C5X"}="H3'1 C5X";
$resn_exch{"2H3' C5X"}="H3'2 C5X"; $resn_exch{"1H5' C5X"}="H5'1 C5X"; $resn_exch{"2H5' C5X"}="H5'2 C5X";
$resn_exch{"1HS2 C60"}="HS21 C60"; $resn_exch{"2HS2 C60"}="HS22 C60"; $resn_exch{"3HS2 C60"}="HS23 C60";
$resn_exch{"1HS3 C60"}="HS31 C60"; $resn_exch{"2HS3 C60"}="HS32 C60"; $resn_exch{"3HS3 C60"}="HS33 C60";
$resn_exch{"1HS4 C60"}="HS41 C60"; $resn_exch{"2HS4 C60"}="HS42 C60"; $resn_exch{"3HS4 C60"}="HS43 C60";
$resn_exch{"1HNF C60"}="HNF1 C60"; $resn_exch{"2HNF C60"}="HNF2 C60"; $resn_exch{"1HBF C60"}="HBF1 C60";
$resn_exch{"2HBF C60"}="HBF2 C60"; $resn_exch{"1HBH C60"}="HBH1 C60"; $resn_exch{"2HBH C60"}="HBH2 C60";
$resn_exch{"1HNH C60"}="HNH1 C60"; $resn_exch{"1HBC C60"}="HBC1 C60"; $resn_exch{"2HBC C60"}="HBC2 C60";
$resn_exch{"1H2C C60"}="H2C1 C60"; $resn_exch{"2H2C C60"}="H2C2 C60"; $resn_exch{"1H3C C60"}="H3C1 C60";
$resn_exch{"2H3C C60"}="H3C2 C60"; $resn_exch{"1H4C C60"}="H4C1 C60"; $resn_exch{"2H4C C60"}="H4C2 C60";
$resn_exch{"1H5C C60"}="H5C1 C60"; $resn_exch{"2H5C C60"}="H5C2 C60"; $resn_exch{"1H6C C60"}="H6C1 C60";
$resn_exch{"2H6C C60"}="H6C2 C60"; $resn_exch{"1HNV C60"}="HNV1 C60"; $resn_exch{"2HNV C60"}="HNV2 C60";
$resn_exch{"1H1V C60"}="H1V1 C60"; $resn_exch{"2H1V C60"}="H1V2 C60"; $resn_exch{"3H1V C60"}="H1V3 C60";
$resn_exch{"1H2V C60"}="H2V1 C60"; $resn_exch{"2H2V C60"}="H2V2 C60"; $resn_exch{"3H2V C60"}="H2V3 C60";
$resn_exch{"1H1T C60"}="H1T1 C60"; $resn_exch{"2H1T C60"}="H1T2 C60"; $resn_exch{"1H2T C60"}="H2T1 C60";
$resn_exch{"2H2T C60"}="H2T2 C60"; $resn_exch{"1H3T C60"}="H3T1 C60"; $resn_exch{"2H3T C60"}="H3T2 C60";
$resn_exch{"1H4T C60"}="H4T1 C60"; $resn_exch{"2H4T C60"}="H4T2 C60"; $resn_exch{"3H4T C60"}="H4T3 C60";
$resn_exch{"1H2C C61"}="H2C1 C61"; $resn_exch{"2H2C C61"}="H2C2 C61"; $resn_exch{"3H2C C61"}="H2C3 C61";
$resn_exch{"1H4C C61"}="H4C1 C61"; $resn_exch{"2H4C C61"}="H4C2 C61"; $resn_exch{"1H5C C61"}="H5C1 C61";
$resn_exch{"2H5C C61"}="H5C2 C61"; $resn_exch{"1H6C C61"}="H6C1 C61"; $resn_exch{"2H6C C61"}="H6C2 C61";
$resn_exch{"1H9C C61"}="H9C1 C61"; $resn_exch{"2H9C C61"}="H9C2 C61"; $resn_exch{"3H9C C61"}="H9C3 C61";
$resn_exch{"1H16 C61"}="H161 C61"; $resn_exch{"2H16 C61"}="H162 C61"; $resn_exch{"1H21 C61"}="H211 C61";
$resn_exch{"2H21 C61"}="H212 C61"; $resn_exch{"1H19 C61"}="H191 C61"; $resn_exch{"2H19 C61"}="H192 C61";
$resn_exch{"3H19 C61"}="H193 C61"; $resn_exch{"1H20 C61"}="H201 C61"; $resn_exch{"2H20 C61"}="H202 C61";
$resn_exch{"3H20 C61"}="H203 C61"; $resn_exch{"1H23 C61"}="H231 C61"; $resn_exch{"2H23 C61"}="H232 C61";
$resn_exch{"1H26 C61"}="H261 C61"; $resn_exch{"2H26 C61"}="H262 C61"; $resn_exch{"1H28 C61"}="H281 C61";
$resn_exch{"2H28 C61"}="H282 C61"; $resn_exch{"3H28 C61"}="H283 C61"; $resn_exch{"1H32 C61"}="H321 C61";
$resn_exch{"2H32 C61"}="H322 C61"; $resn_exch{"1H33 C61"}="H331 C61"; $resn_exch{"2H33 C61"}="H332 C61";
$resn_exch{"1H34 C61"}="H341 C61"; $resn_exch{"2H34 C61"}="H342 C61"; $resn_exch{"1H35 C61"}="H351 C61";
$resn_exch{"2H35 C61"}="H352 C61"; $resn_exch{"3H35 C61"}="H353 C61"; $resn_exch{"1H6N C62"}="H6N1 C62";
$resn_exch{"2H6N C62"}="H6N2 C62"; $resn_exch{"1H6C C62"}="H6C1 C62"; $resn_exch{"1H8' C66"}="H8'1 C66";
$resn_exch{"2H8' C66"}="H8'2 C66"; $resn_exch{"1H5' C66"}="H5'1 C66"; $resn_exch{"1H3' C66"}="H3'1 C66";
$resn_exch{"2H3' C66"}="H3'2 C66"; $resn_exch{"1H2' C66"}="H2'1 C66"; $resn_exch{"2H2' C66"}="H2'2 C66";
$resn_exch{"1H1' C66"}="H1'1 C66"; $resn_exch{"2H1' C66"}="H1'2 C66"; $resn_exch{"1HN4 C66"}="HN41 C66";
$resn_exch{"2HN4 C66"}="HN42 C66"; $resn_exch{"1HB  C66"}=" HB1 C66"; $resn_exch{"2HB  C66"}=" HB2 C66";
$resn_exch{"1HG  C66"}=" HG1 C66"; $resn_exch{"2HG  C66"}=" HG2 C66"; $resn_exch{"1HD  C66"}=" HD1 C66";
$resn_exch{"2HD  C66"}=" HD2 C66"; $resn_exch{"1HE  C66"}=" HE1 C66"; $resn_exch{"2HE  C66"}=" HE2 C66";
$resn_exch{"1HZ  C66"}=" HZ1 C66"; $resn_exch{"2HZ  C66"}=" HZ2 C66"; $resn_exch{"1HN  C6C"}=" H   C6C";
$resn_exch{"2HN  C6C"}=" H2  C6C"; $resn_exch{"1HB  C6C"}=" HB2 C6C"; $resn_exch{"2HB  C6C"}=" HB3 C6C";
$resn_exch{"1H2  C6C"}=" H21 C6C"; $resn_exch{"2H2  C6C"}=" H22 C6C"; $resn_exch{"1H3  C6C"}=" H31 C6C";
$resn_exch{"2H3  C6C"}=" H32 C6C"; $resn_exch{"1H4  C6C"}=" H41 C6C"; $resn_exch{"2H4  C6C"}=" H42 C6C";
$resn_exch{"1H5  C6C"}=" H51 C6C"; $resn_exch{"2H5  C6C"}=" H52 C6C"; $resn_exch{"1H6  C6C"}=" H61 C6C";
$resn_exch{"2H6  C6C"}=" H62 C6C"; $resn_exch{"1H5M C6P"}="H5M1 C6P"; $resn_exch{"2H5M C6P"}="H5M2 C6P";
$resn_exch{"1H2A C6P"}="H2A1 C6P"; $resn_exch{"2H2A C6P"}="H2A2 C6P"; $resn_exch{"3H2A C6P"}="H2A3 C6P";
$resn_exch{"1H4A C6P"}="H4A1 C6P"; $resn_exch{"2H4A C6P"}="H4A2 C6P"; $resn_exch{"1HB  C6P"}=" HB1 C6P";
$resn_exch{"2HB  C6P"}=" HB2 C6P"; $resn_exch{"1H2  C71"}=" H21 C71"; $resn_exch{"2H2  C71"}="H22A C71";
$resn_exch{"1H6  C71"}=" H61 C71"; $resn_exch{"2H6  C71"}=" H62 C71"; $resn_exch{"3H6  C71"}=" H63 C71";
$resn_exch{"1H17 C71"}="H171 C71"; $resn_exch{"2H17 C71"}="H172 C71"; $resn_exch{"1H18 C71"}="H181 C71";
$resn_exch{"2H18 C71"}="H182 C71"; $resn_exch{"1H20 C71"}="H201 C71"; $resn_exch{"2H20 C71"}="H202 C71";
$resn_exch{"1H21 C71"}="H211 C71"; $resn_exch{"2H21 C71"}="H212 C71"; $resn_exch{"1H27 C71"}="H271 C71";
$resn_exch{"2H27 C71"}="H272 C71"; $resn_exch{"1H28 C71"}="H281 C71"; $resn_exch{"2H28 C71"}="H282 C71";
$resn_exch{"1H29 C71"}="H291 C71"; $resn_exch{"2H29 C71"}="H292 C71"; $resn_exch{"1H30 C71"}="H301 C71";
$resn_exch{"2H30 C71"}="H302 C71"; $resn_exch{"1H31 C71"}="H311 C71"; $resn_exch{"2H31 C71"}="H312 C71";
$resn_exch{"3H31 C71"}="H313 C71"; $resn_exch{"1H34 C71"}="H341 C71"; $resn_exch{"2H34 C71"}="H342 C71";
$resn_exch{"1H16 C71"}="H161 C71"; $resn_exch{"2H16 C71"}="H162 C71"; $resn_exch{"1H61 C75"}="H611 C75";
$resn_exch{"2H61 C75"}="H612 C75"; $resn_exch{"3H61 C75"}="H613 C75"; $resn_exch{"1HN1 C78"}="HN11 C78";
$resn_exch{"2HN1 C78"}="HN12 C78"; $resn_exch{"1HC3 C78"}="HC31 C78"; $resn_exch{"1HC4 C78"}="HC41 C78";
$resn_exch{"1HC5 C78"}="HC51 C78"; $resn_exch{"1HC6 C78"}="HC61 C78"; $resn_exch{"1HC8 C78"}="HC81 C78";
$resn_exch{"2HC8 C78"}="HC82 C78"; $resn_exch{"1H12 C78"}="H121 C78"; $resn_exch{"1H13 C78"}="H131 C78";
$resn_exch{"1H14 C78"}="H141 C78"; $resn_exch{"2H14 C78"}="H142 C78"; $resn_exch{"1H16 C78"}="H161 C78";
$resn_exch{"1H17 C78"}="H171 C78"; $resn_exch{"1H18 C78"}="H181 C78"; $resn_exch{"1H19 C78"}="H191 C78";
$resn_exch{"1H25 C78"}="H251 C78"; $resn_exch{"1H27 C78"}="H271 C78"; $resn_exch{"1H28 C78"}="H281 C78";
$resn_exch{"1H30 C78"}="H301 C78"; $resn_exch{"2H30 C78"}="H302 C78"; $resn_exch{"1H31 C78"}="H311 C78";
$resn_exch{"2H31 C78"}="H312 C78"; $resn_exch{"1H32 C78"}="H321 C78"; $resn_exch{"2H32 C78"}="H322 C78";
$resn_exch{"1H33 C78"}="H331 C78"; $resn_exch{"2H33 C78"}="H332 C78"; $resn_exch{"1H34 C78"}="H341 C78";
$resn_exch{"2H34 C78"}="H342 C78"; $resn_exch{"1H37 C78"}="H371 C78"; $resn_exch{"1H38 C78"}="H381 C78";
$resn_exch{"1H39 C78"}="H391 C78"; $resn_exch{"2H39 C78"}="H392 C78"; $resn_exch{"1H40 C78"}="H401 C78";
$resn_exch{"2H40 C78"}="H402 C78"; $resn_exch{"1H41 C78"}="H411 C78"; $resn_exch{"2H41 C78"}="H412 C78";
$resn_exch{"1H42 C78"}="H421 C78"; $resn_exch{"2H42 C78"}="H422 C78"; $resn_exch{"1H43 C78"}="H431 C78";
$resn_exch{"1H45 C78"}="H451 C78"; $resn_exch{"2H45 C78"}="H452 C78"; $resn_exch{"1H7C C80"}="H7C1 C80";
$resn_exch{"2H7C C80"}="H7C2 C80"; $resn_exch{"1H20 C80"}="H201 C80"; $resn_exch{"2H20 C80"}="H202 C80";
$resn_exch{"1H22 C80"}="H221 C80"; $resn_exch{"2H22 C80"}="H222 C80"; $resn_exch{"3H22 C80"}="H223 C80";
$resn_exch{"1H23 C80"}="H231 C80"; $resn_exch{"2H23 C80"}="H232 C80"; $resn_exch{"1H25 C80"}="H251 C80";
$resn_exch{"2H25 C80"}="H252 C80"; $resn_exch{"1H28 C80"}="H281 C80"; $resn_exch{"2H28 C80"}="H282 C80";
$resn_exch{"1H30 C80"}="H301 C80"; $resn_exch{"2H30 C80"}="H302 C80"; $resn_exch{"3H30 C80"}="H303 C80";
$resn_exch{"1H34 C80"}="H341 C80"; $resn_exch{"2H34 C80"}="H342 C80"; $resn_exch{"1H35 C80"}="H351 C80";
$resn_exch{"2H35 C80"}="H352 C80"; $resn_exch{"1H36 C80"}="H361 C80"; $resn_exch{"2H36 C80"}="H362 C80";
$resn_exch{"1H37 C80"}="H371 C80"; $resn_exch{"2H37 C80"}="H372 C80"; $resn_exch{"3H37 C80"}="H373 C80";
$resn_exch{"1H6N C85"}="H6N1 C85"; $resn_exch{"2H6N C85"}="H6N2 C85"; $resn_exch{"1H3N C85"}="H3N1 C85";
$resn_exch{"2H3N C85"}="H3N2 C85"; $resn_exch{"1H1N C8C"}="H1N1 C8C"; $resn_exch{"2H1N C8C"}="H1N2 C8C";
$resn_exch{"1H7C C8C"}="H7C1 C8C"; $resn_exch{"2H7C C8C"}="H7C2 C8C"; $resn_exch{"1H8C C8C"}="H8C1 C8C";
$resn_exch{"2H8C C8C"}="H8C2 C8C"; $resn_exch{"1H1  C8E"}=" H11 C8E"; $resn_exch{"2H1  C8E"}=" H12 C8E";
$resn_exch{"3H1  C8E"}=" H13 C8E"; $resn_exch{"1H2  C8E"}=" H21 C8E"; $resn_exch{"2H2  C8E"}=" H22 C8E";
$resn_exch{"1H3  C8E"}=" H31 C8E"; $resn_exch{"2H3  C8E"}=" H32 C8E"; $resn_exch{"1H4  C8E"}=" H41 C8E";
$resn_exch{"2H4  C8E"}=" H42 C8E"; $resn_exch{"1H5  C8E"}=" H51 C8E"; $resn_exch{"2H5  C8E"}=" H52 C8E";
$resn_exch{"1H6  C8E"}=" H61 C8E"; $resn_exch{"2H6  C8E"}=" H62 C8E"; $resn_exch{"1H7  C8E"}=" H71 C8E";
$resn_exch{"2H7  C8E"}=" H72 C8E"; $resn_exch{"1H8  C8E"}=" H81 C8E"; $resn_exch{"2H8  C8E"}=" H82 C8E";
$resn_exch{"1H10 C8E"}="H101 C8E"; $resn_exch{"2H10 C8E"}="H102 C8E"; $resn_exch{"1H11 C8E"}="H111 C8E";
$resn_exch{"2H11 C8E"}="H112 C8E"; $resn_exch{"1H13 C8E"}="H131 C8E"; $resn_exch{"2H13 C8E"}="H132 C8E";
$resn_exch{"1H14 C8E"}="H141 C8E"; $resn_exch{"2H14 C8E"}="H142 C8E"; $resn_exch{"1H16 C8E"}="H161 C8E";
$resn_exch{"2H16 C8E"}="H162 C8E"; $resn_exch{"1H17 C8E"}="H171 C8E"; $resn_exch{"2H17 C8E"}="H172 C8E";
$resn_exch{"1H19 C8E"}="H191 C8E"; $resn_exch{"2H19 C8E"}="H192 C8E"; $resn_exch{"1H20 C8E"}="H201 C8E";
$resn_exch{"2H20 C8E"}="H202 C8E"; $resn_exch{"1H5  C90"}=" H51 C90"; $resn_exch{"2H5  C90"}=" H52 C90";
$resn_exch{"1H21 C90"}="H211 C90"; $resn_exch{"2H21 C90"}="H212 C90"; $resn_exch{"3H21 C90"}="H213 C90";
$resn_exch{"1HA1 C90"}="HA11 C90"; $resn_exch{"2HA1 C90"}="HA12 C90"; $resn_exch{"1HA2 C90"}="HA21 C90";
$resn_exch{"2HA2 C90"}="HA22 C90"; $resn_exch{"1HB1 C90"}="HB11 C90"; $resn_exch{"2HB1 C90"}="HB12 C90";
$resn_exch{"1HB2 C90"}="HB21 C90"; $resn_exch{"2HB2 C90"}="HB22 C90"; $resn_exch{"1H6N C94"}="H6N1 C94";
$resn_exch{"2H6N C94"}="H6N2 C94"; $resn_exch{"1H3N C94"}="H3N1 C94"; $resn_exch{"2H3N C94"}="H3N2 C94";
$resn_exch{"1H3N C95"}="H3N1 C95"; $resn_exch{"2H3N C95"}="H3N2 C95"; $resn_exch{"1H6N C96"}="H6N1 C96";
$resn_exch{"2H6N C96"}="H6N2 C96"; $resn_exch{"1H20 C96"}="H201 C96"; $resn_exch{"2H20 C96"}="H202 C96";
$resn_exch{"3H20 C96"}="H203 C96"; $resn_exch{"1H3N C96"}="H3N1 C96"; $resn_exch{"2H3N C96"}="H3N2 C96";
$resn_exch{"1HX5 CA1"}="HX51 CA1"; $resn_exch{"2HX5 CA1"}="HX52 CA1"; $resn_exch{"1H2C CA2"}="H2C1 CA2";
$resn_exch{"2H2C CA2"}="H2C2 CA2"; $resn_exch{"1H6C CA2"}="H6C1 CA2"; $resn_exch{"2H6C CA2"}="H6C2 CA2";
$resn_exch{"1H8C CA2"}="H8C1 CA2"; $resn_exch{"2H8C CA2"}="H8C2 CA2"; $resn_exch{"1H9C CA2"}="H9C1 CA2";
$resn_exch{"2H9C CA2"}="H9C2 CA2"; $resn_exch{"1H10 CA2"}="H101 CA2"; $resn_exch{"2H10 CA2"}="H102 CA2";
$resn_exch{"1H6  CA3"}=" H61 CA3"; $resn_exch{"2H6  CA3"}=" H62 CA3"; $resn_exch{"2HO' CA3"}="HO'2 CA3";
$resn_exch{"1H5' CA3"}="H5'1 CA3"; $resn_exch{"2H5' CA3"}="H5'2 CA3"; $resn_exch{"1HP9 CA3"}="HP91 CA3";
$resn_exch{"2HP9 CA3"}="HP92 CA3"; $resn_exch{"1HPA CA3"}="HPA1 CA3"; $resn_exch{"2HPA CA3"}="HPA2 CA3";
$resn_exch{"3HPA CA3"}="HPA3 CA3"; $resn_exch{"1HPB CA3"}="HPB1 CA3"; $resn_exch{"2HPB CA3"}="HPB2 CA3";
$resn_exch{"3HPB CA3"}="HPB3 CA3"; $resn_exch{"1HP5 CA3"}="HP51 CA3"; $resn_exch{"2HP5 CA3"}="HP52 CA3";
$resn_exch{"1HP4 CA3"}="HP41 CA3"; $resn_exch{"2HP4 CA3"}="HP42 CA3"; $resn_exch{"1HP2 CA3"}="HP21 CA3";
$resn_exch{"2HP2 CA3"}="HP22 CA3"; $resn_exch{"1HP1 CA3"}="HP11 CA3"; $resn_exch{"2HP1 CA3"}="HP12 CA3";
$resn_exch{"1HT2 CA3"}="HT21 CA3"; $resn_exch{"2HT2 CA3"}="HT22 CA3"; $resn_exch{"0HT1 CA3"}="HT10 CA3";
$resn_exch{"6HNT CA3"}="HNT6 CA3"; $resn_exch{"1HA1 CA3"}="HA11 CA3"; $resn_exch{"2HA1 CA3"}="HA12 CA3";
$resn_exch{"1HT1 CA3"}="HT11 CA3"; $resn_exch{"2HT1 CA3"}="HT12 CA3"; $resn_exch{"1HA3 CA3"}="HA31 CA3";
$resn_exch{"2HA3 CA3"}="HA32 CA3"; $resn_exch{"1HA4 CA3"}="HA41 CA3"; $resn_exch{"2HA4 CA3"}="HA42 CA3";
$resn_exch{"1HA5 CA3"}="HA51 CA3"; $resn_exch{"2HA5 CA3"}="HA52 CA3"; $resn_exch{"1H11 CA4"}="H111 CA4";
$resn_exch{"2H11 CA4"}="H112 CA4"; $resn_exch{"1H12 CA4"}="H121 CA4"; $resn_exch{"2H12 CA4"}="H122 CA4";
$resn_exch{"1H19 CA4"}="H191 CA4"; $resn_exch{"2H19 CA4"}="H192 CA4"; $resn_exch{"3H19 CA4"}="H193 CA4";
$resn_exch{"1H22 CA4"}="H221 CA4"; $resn_exch{"2H22 CA4"}="H222 CA4"; $resn_exch{"1H18 CA4"}="H181 CA4";
$resn_exch{"2H18 CA4"}="H182 CA4"; $resn_exch{"3H18 CA4"}="H183 CA4"; $resn_exch{"1H15 CA4"}="H151 CA4";
$resn_exch{"2H15 CA4"}="H152 CA4"; $resn_exch{"1H16 CA4"}="H161 CA4"; $resn_exch{"2H16 CA4"}="H162 CA4";
$resn_exch{"1H21 CA4"}="H211 CA4"; $resn_exch{"2H21 CA4"}="H212 CA4"; $resn_exch{"3H21 CA4"}="H213 CA4";
$resn_exch{"1H24 CA4"}="H241 CA4"; $resn_exch{"2H24 CA4"}="H242 CA4"; $resn_exch{"3H24 CA4"}="H243 CA4";
$resn_exch{"1HN6 CA5"}="HN61 CA5"; $resn_exch{"2HN6 CA5"}="HN62 CA5"; $resn_exch{"2HO' CA5"}="HO'2 CA5";
$resn_exch{"1H5' CA5"}="H5'1 CA5"; $resn_exch{"2H5' CA5"}="H5'2 CA5"; $resn_exch{"1HP9 CA5"}="HP91 CA5";
$resn_exch{"2HP9 CA5"}="HP92 CA5"; $resn_exch{"1HCA CA5"}="HCA1 CA5"; $resn_exch{"2HCA CA5"}="HCA2 CA5";
$resn_exch{"3HCA CA5"}="HCA3 CA5"; $resn_exch{"1HCB CA5"}="HCB1 CA5"; $resn_exch{"2HCB CA5"}="HCB2 CA5";
$resn_exch{"3HCB CA5"}="HCB3 CA5"; $resn_exch{"1HP5 CA5"}="HP51 CA5"; $resn_exch{"2HP5 CA5"}="HP52 CA5";
$resn_exch{"1HP4 CA5"}="HP41 CA5"; $resn_exch{"2HP4 CA5"}="HP42 CA5"; $resn_exch{"1HP2 CA5"}="HP21 CA5";
$resn_exch{"2HP2 CA5"}="HP22 CA5"; $resn_exch{"1HP1 CA5"}="HP11 CA5"; $resn_exch{"2HP1 CA5"}="HP12 CA5";
$resn_exch{"1HT2 CA5"}="HT21 CA5"; $resn_exch{"2HT2 CA5"}="HT22 CA5"; $resn_exch{"0HT1 CA5"}="HT10 CA5";
$resn_exch{"6HNT CA5"}="HNT6 CA5"; $resn_exch{"1HA1 CA5"}="HA11 CA5"; $resn_exch{"2HA1 CA5"}="HA12 CA5";
$resn_exch{"1HT1 CA5"}="HT11 CA5"; $resn_exch{"2HT1 CA5"}="HT12 CA5"; $resn_exch{" AH2 CAA"}=" H2A CAA";
$resn_exch{"AH61 CAA"}="H61A CAA"; $resn_exch{"AH62 CAA"}="H62A CAA"; $resn_exch{" AH8 CAA"}=" H8A CAA";
$resn_exch{"AH1* CAA"}=" H1B CAA"; $resn_exch{"AH2* CAA"}=" H2B CAA"; $resn_exch{"AHO2 CAA"}="HO2A CAA";
$resn_exch{"AH3* CAA"}=" H3B CAA"; $resn_exch{"8HOA CAA"}="HOA8 CAA"; $resn_exch{"9HOA CAA"}="HOA9 CAA";
$resn_exch{"AH4* CAA"}=" H4B CAA"; $resn_exch{"AH51 CAA"}="H51A CAA"; $resn_exch{"AH52 CAA"}="H52A CAA";
$resn_exch{"2HOA CAA"}="HOA2 CAA"; $resn_exch{"5HOA CAA"}="HOA5 CAA"; $resn_exch{"1H12 CAA"}="H121 CAA";
$resn_exch{"2H12 CAA"}="H122 CAA"; $resn_exch{"1H13 CAA"}="H131 CAA"; $resn_exch{"2H13 CAA"}="H132 CAA";
$resn_exch{"3H13 CAA"}="H133 CAA"; $resn_exch{"1H14 CAA"}="H141 CAA"; $resn_exch{"2H14 CAA"}="H142 CAA";
$resn_exch{"3H14 CAA"}="H143 CAA"; $resn_exch{"1H7  CAA"}=" H71 CAA"; $resn_exch{"2H7  CAA"}=" H72 CAA";
$resn_exch{"1H6  CAA"}=" H61 CAA"; $resn_exch{"2H6  CAA"}=" H62 CAA"; $resn_exch{"1H3  CAA"}=" H31 CAA";
$resn_exch{"2H3  CAA"}=" H32 CAA"; $resn_exch{"1H2  CAA"}=" H21 CAA"; $resn_exch{"2H2  CAA"}=" H22 CAA";
$resn_exch{"1H2' CAA"}="H2'1 CAA"; $resn_exch{"2H2' CAA"}="H2'2 CAA"; $resn_exch{"1H4' CAA"}="H4'1 CAA";
$resn_exch{"2H4' CAA"}="H4'2 CAA"; $resn_exch{"3H4' CAA"}="H4'3 CAA"; $resn_exch{"1H2  CAB"}=" H21 CAB";
$resn_exch{"2H2  CAB"}=" H22 CAB"; $resn_exch{"1H3  CAB"}=" H31 CAB"; $resn_exch{"2H3  CAB"}=" H32 CAB";
$resn_exch{"1HN4 CAB"}="HN41 CAB"; $resn_exch{"2HN4 CAB"}="HN42 CAB"; $resn_exch{"1H1  CAC"}=" H11 CAC";
$resn_exch{"2H1  CAC"}=" H12 CAC"; $resn_exch{"3H1  CAC"}=" H13 CAC"; $resn_exch{"1H2  CAC"}=" H21 CAC";
$resn_exch{"2H2  CAC"}=" H22 CAC"; $resn_exch{"3H2  CAC"}=" H23 CAC"; $resn_exch{"1H1  CAD"}=" H11 CAD";
$resn_exch{"2H1  CAD"}=" H12 CAD"; $resn_exch{"3H1  CAD"}=" H13 CAD"; $resn_exch{"1H2  CAD"}=" H21 CAD";
$resn_exch{"2H2  CAD"}=" H22 CAD"; $resn_exch{"3H2  CAD"}=" H23 CAD"; $resn_exch{"1H2  CAE"}=" H21 CAE";
$resn_exch{"2H2  CAE"}=" H22 CAE"; $resn_exch{"1H3  CAE"}=" H31 CAE"; $resn_exch{"2H3  CAE"}=" H32 CAE";
$resn_exch{"1H5  CAE"}=" H51 CAE"; $resn_exch{"2H5  CAE"}=" H52 CAE"; $resn_exch{"1H6  CAE"}=" H61 CAE";
$resn_exch{"2H6  CAE"}=" H62 CAE"; $resn_exch{"1H8  CAE"}=" H81 CAE"; $resn_exch{"2H8  CAE"}=" H82 CAE";
$resn_exch{"3H8  CAE"}=" H83 CAE"; $resn_exch{"1H9  CAE"}=" H91 CAE"; $resn_exch{"2H9  CAE"}=" H92 CAE";
$resn_exch{"3H9  CAE"}=" H93 CAE"; $resn_exch{"1H10 CAE"}="H101 CAE"; $resn_exch{"2H10 CAE"}="H102 CAE";
$resn_exch{"3H10 CAE"}="H103 CAE"; $resn_exch{"1HN  CAF"}=" H   CAF"; $resn_exch{"2HN  CAF"}=" H2  CAF";
$resn_exch{"1HB  CAF"}=" HB2 CAF"; $resn_exch{"2HB  CAF"}=" HB3 CAF"; $resn_exch{"1HE1 CAF"}="HE11 CAF";
$resn_exch{"2HE1 CAF"}="HE12 CAF"; $resn_exch{"3HE1 CAF"}="HE13 CAF"; $resn_exch{"1HE2 CAF"}="HE21 CAF";
$resn_exch{"2HE2 CAF"}="HE22 CAF"; $resn_exch{"3HE2 CAF"}="HE23 CAF"; $resn_exch{"2HOG CAG"}="HOG2 CAG";
$resn_exch{"2HOB CAG"}="HOB2 CAG"; $resn_exch{"2HOA CAG"}="HOA2 CAG"; $resn_exch{"1H5* CAG"}="H5'1 CAG";
$resn_exch{"2H5* CAG"}="H5'2 CAG"; $resn_exch{" H4* CAG"}=" H4B CAG"; $resn_exch{" H3* CAG"}=" H3B CAG";
$resn_exch{" H2* CAG"}=" H2' CAG"; $resn_exch{" H1* CAG"}=" H1' CAG"; $resn_exch{"1H2  CAG"}=" H21 CAG";
$resn_exch{"2H2  CAG"}=" H22 CAG"; $resn_exch{"1HCM CAG"}="HCM1 CAG"; $resn_exch{"2HCM CAG"}="HCM2 CAG";
$resn_exch{"3HCM CAG"}="HCM3 CAG"; $resn_exch{"1H3  CAH"}=" H31 CAH"; $resn_exch{"2H3  CAH"}=" H32 CAH";
$resn_exch{"1H6  CAH"}=" H61 CAH"; $resn_exch{"2H6  CAH"}=" H62 CAH"; $resn_exch{"1H8  CAH"}=" H81 CAH";
$resn_exch{"2H8  CAH"}=" H82 CAH"; $resn_exch{"3H8  CAH"}=" H83 CAH"; $resn_exch{"1H9  CAH"}=" H91 CAH";
$resn_exch{"2H9  CAH"}=" H92 CAH"; $resn_exch{"3H9  CAH"}=" H93 CAH"; $resn_exch{"1H10 CAH"}="H101 CAH";
$resn_exch{"2H10 CAH"}="H102 CAH"; $resn_exch{"3H10 CAH"}="H103 CAH"; $resn_exch{"1H20 CAI"}="H201 CAI";
$resn_exch{"2H20 CAI"}="H202 CAI"; $resn_exch{"1H2  CAI"}="H21A CAI"; $resn_exch{"2H2  CAI"}=" H22 CAI";
$resn_exch{"1H7  CAI"}=" H71 CAI"; $resn_exch{"2H7  CAI"}=" H72 CAI"; $resn_exch{"1H14 CAI"}="H141 CAI";
$resn_exch{"2H14 CAI"}="H142 CAI"; $resn_exch{"1H16 CAI"}="H161 CAI"; $resn_exch{"2H16 CAI"}="H162 CAI";
$resn_exch{"3H16 CAI"}="H163 CAI"; $resn_exch{"1H22 CAI"}="H221 CAI"; $resn_exch{"2H22 CAI"}="H222 CAI";
$resn_exch{"3H22 CAI"}="H223 CAI"; $resn_exch{"1H6A CAI"}="H6A1 CAI"; $resn_exch{"2H6A CAI"}="H6A2 CAI";
$resn_exch{"3H6A CAI"}="H6A3 CAI"; $resn_exch{"1H7A CAI"}="H7A1 CAI"; $resn_exch{"2H7A CAI"}="H7A2 CAI";
$resn_exch{"3H7A CAI"}="H7A3 CAI"; $resn_exch{"1H2B CAI"}="H2B1 CAI"; $resn_exch{"2H2B CAI"}="H2B2 CAI";
$resn_exch{"1H6B CAI"}="H6B1 CAI"; $resn_exch{"2H6B CAI"}="H6B2 CAI"; $resn_exch{"3H6B CAI"}="H6B3 CAI";
$resn_exch{"1H7B CAI"}="H7B1 CAI"; $resn_exch{"2H7B CAI"}="H7B2 CAI"; $resn_exch{"3H7B CAI"}="H7B3 CAI";
$resn_exch{"1H2C CAI"}="H2C1 CAI"; $resn_exch{"2H2C CAI"}="H2C2 CAI"; $resn_exch{"1H4C CAI"}="H4C1 CAI";
$resn_exch{"2H4C CAI"}="H4C2 CAI"; $resn_exch{"3H4C CAI"}="H4C3 CAI"; $resn_exch{"1H5C CAI"}="H5C1 CAI";
$resn_exch{"2H5C CAI"}="H5C2 CAI"; $resn_exch{"3H5C CAI"}="H5C3 CAI"; $resn_exch{"1H18 CAI"}="H181 CAI";
$resn_exch{"2H18 CAI"}="H182 CAI"; $resn_exch{"3H18 CAI"}="H183 CAI"; $resn_exch{"1H19 CAI"}="H191 CAI";
$resn_exch{"2H19 CAI"}="H192 CAI"; $resn_exch{"3H19 CAI"}="H193 CAI"; $resn_exch{"1H8A CAI"}="H8A1 CAI";
$resn_exch{"2H8A CAI"}="H8A2 CAI"; $resn_exch{"3H8A CAI"}="H8A3 CAI"; $resn_exch{"1HAK CAK"}="HAK1 CAK";
$resn_exch{"2HAK CAK"}="HAK2 CAK"; $resn_exch{"1HAE CAK"}="HAE1 CAK"; $resn_exch{"2HAE CAK"}="HAE2 CAK";
$resn_exch{"1HN  CAL"}=" H   CAL"; $resn_exch{"2HN  CAL"}=" H2  CAL"; $resn_exch{"1HB  CAL"}=" HB2 CAL";
$resn_exch{"2HB  CAL"}=" HB3 CAL"; $resn_exch{"1HD1 CAL"}="HD12 CAL"; $resn_exch{"2HD1 CAL"}="HD13 CAL";
$resn_exch{"1HD2 CAL"}="HD23 CAL"; $resn_exch{"2HD2 CAL"}="HD22 CAL"; $resn_exch{"1HE1 CAL"}="HE12 CAL";
$resn_exch{"2HE1 CAL"}="HE13 CAL"; $resn_exch{"1HE2 CAL"}="HE23 CAL"; $resn_exch{"2HE2 CAL"}="HE22 CAL";
$resn_exch{"1HZ  CAL"}=" HZ2 CAL"; $resn_exch{"2HZ  CAL"}=" HZ3 CAL"; $resn_exch{"1HM  CAL"}=" HM1 CAL";
$resn_exch{"2HM  CAL"}=" HM2 CAL"; $resn_exch{"1HB2 CAL"}="HB21 CAL"; $resn_exch{"2HB2 CAL"}="HB22 CAL";
$resn_exch{"1HD3 CAL"}="HD31 CAL"; $resn_exch{"2HD3 CAL"}="HD32 CAL"; $resn_exch{"3HD3 CAL"}="HD33 CAL";
$resn_exch{"1HD4 CAL"}="HD41 CAL"; $resn_exch{"2HD4 CAL"}="HD42 CAL"; $resn_exch{"3HD4 CAL"}="HD43 CAL";
$resn_exch{"1H3  CAM"}=" H31 CAM"; $resn_exch{"2H3  CAM"}=" H32 CAM"; $resn_exch{"1H5  CAM"}=" H51 CAM";
$resn_exch{"2H5  CAM"}=" H52 CAM"; $resn_exch{"1H6  CAM"}=" H61 CAM"; $resn_exch{"2H6  CAM"}=" H62 CAM";
$resn_exch{"1H8  CAM"}=" H81 CAM"; $resn_exch{"2H8  CAM"}=" H82 CAM"; $resn_exch{"3H8  CAM"}=" H83 CAM";
$resn_exch{"1H9  CAM"}=" H91 CAM"; $resn_exch{"2H9  CAM"}=" H92 CAM"; $resn_exch{"3H9  CAM"}=" H93 CAM";
$resn_exch{"1H10 CAM"}="H101 CAM"; $resn_exch{"2H10 CAM"}="H102 CAM"; $resn_exch{"3H10 CAM"}="H103 CAM";
$resn_exch{"1HN  CAN"}=" HN1 CAN"; $resn_exch{"2HN  CAN"}=" HN2 CAN"; $resn_exch{"1HB  CAN"}=" HB1 CAN";
$resn_exch{"2HB  CAN"}=" HB2 CAN"; $resn_exch{"1HG  CAN"}=" HG1 CAN"; $resn_exch{"2HG  CAN"}=" HG2 CAN";
$resn_exch{"1HNZ CAN"}="HNZ1 CAN"; $resn_exch{"2HNZ CAN"}="HNZ2 CAN"; $resn_exch{" AH2 CAO"}=" H2A CAO";
$resn_exch{"AH61 CAO"}="H61A CAO"; $resn_exch{"AH62 CAO"}="H62A CAO"; $resn_exch{" AH8 CAO"}=" H8A CAO";
$resn_exch{"AH1* CAO"}=" H1B CAO"; $resn_exch{"AH2* CAO"}=" H2B CAO"; $resn_exch{"AHO2 CAO"}="HO2A CAO";
$resn_exch{"AH3* CAO"}=" H3B CAO"; $resn_exch{"8HOA CAO"}="HOA8 CAO"; $resn_exch{"9HOA CAO"}="HOA9 CAO";
$resn_exch{"AH4* CAO"}=" H4B CAO"; $resn_exch{"AH51 CAO"}="H51A CAO"; $resn_exch{"AH52 CAO"}="H52A CAO";
$resn_exch{"2HOA CAO"}="HOA2 CAO"; $resn_exch{"5HOA CAO"}="HOA5 CAO"; $resn_exch{"1H12 CAO"}="H121 CAO";
$resn_exch{"2H12 CAO"}="H122 CAO"; $resn_exch{"1H13 CAO"}="H131 CAO"; $resn_exch{"2H13 CAO"}="H132 CAO";
$resn_exch{"3H13 CAO"}="H133 CAO"; $resn_exch{"1H14 CAO"}="H141 CAO"; $resn_exch{"2H14 CAO"}="H142 CAO";
$resn_exch{"3H14 CAO"}="H143 CAO"; $resn_exch{"1H7  CAO"}=" H71 CAO"; $resn_exch{"2H7  CAO"}=" H72 CAO";
$resn_exch{"1H6  CAO"}=" H61 CAO"; $resn_exch{"2H6  CAO"}=" H62 CAO"; $resn_exch{"1H3  CAO"}=" H31 CAO";
$resn_exch{"2H3  CAO"}=" H32 CAO"; $resn_exch{"1H2  CAO"}=" H21 CAO"; $resn_exch{"2H2  CAO"}=" H22 CAO";
$resn_exch{"1HOS CAO"}="HOS1 CAO"; $resn_exch{"1H1  CAP"}=" H11 CAP"; $resn_exch{"2H1  CAP"}=" H12 CAP";
$resn_exch{"1H5  CAP"}=" H51 CAP"; $resn_exch{"2H5  CAP"}=" H52 CAP"; $resn_exch{"2HOP CAP"}="HOP2 CAP";
$resn_exch{"3HOP CAP"}="HOP3 CAP"; $resn_exch{"5HOP CAP"}="HOP5 CAP"; $resn_exch{"6HOP CAP"}="HOP6 CAP";
$resn_exch{"2HOP CAR"}="HOP2 CAR"; $resn_exch{"3HOP CAR"}="HOP3 CAR"; $resn_exch{"1H5* CAR"}=" H5' CAR";
$resn_exch{"2H5* CAR"}="H5'' CAR"; $resn_exch{" H4* CAR"}=" H4' CAR"; $resn_exch{" H3* CAR"}=" H3' CAR";
$resn_exch{"H3T  CAR"}="HO3' CAR"; $resn_exch{" H2* CAR"}=" H2' CAR"; $resn_exch{"2HO* CAR"}="HO2' CAR";
$resn_exch{" H1* CAR"}=" H1' CAR"; $resn_exch{"1HN4 CAR"}="HN41 CAR"; $resn_exch{"2HN4 CAR"}="HN42 CAR";
$resn_exch{"1HN  CAS"}=" H   CAS"; $resn_exch{"2HN  CAS"}=" H2  CAS"; $resn_exch{"1HB  CAS"}=" HB2 CAS";
$resn_exch{"2HB  CAS"}=" HB3 CAS"; $resn_exch{"1HE1 CAS"}="HE11 CAS"; $resn_exch{"2HE1 CAS"}="HE12 CAS";
$resn_exch{"3HE1 CAS"}="HE13 CAS"; $resn_exch{"1HE2 CAS"}="HE21 CAS"; $resn_exch{"2HE2 CAS"}="HE22 CAS";
$resn_exch{"3HE2 CAS"}="HE23 CAS"; $resn_exch{"1H1  CAT"}=" H11 CAT"; $resn_exch{"2H1  CAT"}=" H12 CAT";
$resn_exch{"1H2  CAT"}=" H21 CAT"; $resn_exch{"2H2  CAT"}=" H22 CAT"; $resn_exch{"1H3  CAT"}=" H31 CAT";
$resn_exch{"2H3  CAT"}=" H32 CAT"; $resn_exch{"1H4  CAT"}=" H41 CAT"; $resn_exch{"2H4  CAT"}=" H42 CAT";
$resn_exch{"1H5  CAT"}=" H51 CAT"; $resn_exch{"2H5  CAT"}=" H52 CAT"; $resn_exch{"1H6  CAT"}=" H61 CAT";
$resn_exch{"2H6  CAT"}=" H62 CAT"; $resn_exch{"1H7  CAT"}=" H71 CAT"; $resn_exch{"2H7  CAT"}=" H72 CAT";
$resn_exch{"1H8  CAT"}=" H81 CAT"; $resn_exch{"2H8  CAT"}=" H82 CAT"; $resn_exch{"1H9  CAT"}=" H91 CAT";
$resn_exch{"2H9  CAT"}=" H92 CAT"; $resn_exch{"1H10 CAT"}="H101 CAT"; $resn_exch{"2H10 CAT"}="H102 CAT";
$resn_exch{"1H11 CAT"}="H111 CAT"; $resn_exch{"2H11 CAT"}="H112 CAT"; $resn_exch{"1H12 CAT"}="H121 CAT";
$resn_exch{"2H12 CAT"}="H122 CAT"; $resn_exch{"3H12 CAT"}="H123 CAT"; $resn_exch{"1HA1 CAT"}="HA11 CAT";
$resn_exch{"2HA1 CAT"}="HA12 CAT"; $resn_exch{"3HA1 CAT"}="HA13 CAT"; $resn_exch{"1HB1 CAT"}="HB11 CAT";
$resn_exch{"2HB1 CAT"}="HB12 CAT"; $resn_exch{"3HB1 CAT"}="HB13 CAT"; $resn_exch{"1HC1 CAT"}="HC11 CAT";
$resn_exch{"2HC1 CAT"}="HC12 CAT"; $resn_exch{"3HC1 CAT"}="HC13 CAT"; $resn_exch{"1HN  CAV"}=" H   CAV";
$resn_exch{"2HN  CAV"}=" H2  CAV"; $resn_exch{"1HB  CAV"}=" HB2 CAV"; $resn_exch{"2HB  CAV"}=" HB3 CAV";
$resn_exch{"1HD1 CAV"}="HD12 CAV"; $resn_exch{"2HD1 CAV"}="HD13 CAV"; $resn_exch{"1HD2 CAV"}="HD23 CAV";
$resn_exch{"2HD2 CAV"}="HD22 CAV"; $resn_exch{"1HE1 CAV"}="HE12 CAV"; $resn_exch{"2HE1 CAV"}="HE13 CAV";
$resn_exch{"1HE2 CAV"}="HE23 CAV"; $resn_exch{"2HE2 CAV"}="HE22 CAV"; $resn_exch{"1HZ  CAV"}=" HZ2 CAV";
$resn_exch{"2HZ  CAV"}=" HZ3 CAV"; $resn_exch{" HA* CAV"}=" HA' CAV"; $resn_exch{" HB* CAV"}=" HB' CAV";
$resn_exch{"1HG1 CAV"}="HG11 CAV"; $resn_exch{"2HG1 CAV"}="HG12 CAV"; $resn_exch{"3HG1 CAV"}="HG13 CAV";
$resn_exch{"1HG2 CAV"}="HG21 CAV"; $resn_exch{"2HG2 CAV"}="HG22 CAV"; $resn_exch{"3HG2 CAV"}="HG23 CAV";
$resn_exch{"1HOB CAV"}="HOB1 CAV"; $resn_exch{"1H9  CAX"}=" H91 CAX"; $resn_exch{"2H9  CAX"}=" H92 CAX";
$resn_exch{"1H8  CAX"}=" H81 CAX"; $resn_exch{"2H8  CAX"}=" H82 CAX"; $resn_exch{"3H8  CAX"}=" H83 CAX";
$resn_exch{"1H7  CAX"}=" H71 CAX"; $resn_exch{"2H7  CAX"}=" H72 CAX"; $resn_exch{"3H7  CAX"}=" H73 CAX";
$resn_exch{"1H6  CAX"}=" H61 CAX"; $resn_exch{"2H6  CAX"}=" H62 CAX"; $resn_exch{"3H6  CAX"}=" H63 CAX";
$resn_exch{"1H5  CAX"}=" H51 CAX"; $resn_exch{"2H5  CAX"}=" H52 CAX"; $resn_exch{"1HN  CAY"}=" HN1 CAY";
$resn_exch{"2HN  CAY"}=" HN2 CAY"; $resn_exch{"1HB  CAY"}=" HB1 CAY"; $resn_exch{"2HB  CAY"}=" HB2 CAY";
$resn_exch{"1HG  CAY"}=" HG1 CAY"; $resn_exch{"2HG  CAY"}=" HG2 CAY"; $resn_exch{"1HC2 CAZ"}="HC21 CAZ";
$resn_exch{"2HC2 CAZ"}="HC22 CAZ"; $resn_exch{"1HC3 CAZ"}="HC31 CAZ"; $resn_exch{"2HC3 CAZ"}="HC32 CAZ";
$resn_exch{"OH4A CAZ"}="H4AO CAZ"; $resn_exch{"NH10 CAZ"}="H10N CAZ"; $resn_exch{"CH15 CAZ"}="H15C CAZ";
$resn_exch{"1H18 CAZ"}="H181 CAZ"; $resn_exch{"2H18 CAZ"}="H182 CAZ"; $resn_exch{"1H19 CAZ"}="H191 CAZ";
$resn_exch{"2H19 CAZ"}="H192 CAZ"; $resn_exch{"3H19 CAZ"}="H193 CAZ"; $resn_exch{"1H20 CAZ"}="H201 CAZ";
$resn_exch{"2H20 CAZ"}="H202 CAZ"; $resn_exch{"3H20 CAZ"}="H203 CAZ"; $resn_exch{"OH2A CAZ"}="H2AO CAZ";
$resn_exch{"1H   CB1"}=" H1  CB1"; $resn_exch{"1H4  CB1"}=" H41 CB1"; $resn_exch{"1HN1 CB1"}="HN11 CB1";
$resn_exch{"2HN1 CB1"}="HN12 CB1"; $resn_exch{"1H7  CB1"}=" H71 CB1"; $resn_exch{"2H7  CB1"}=" H72 CB1";
$resn_exch{"1H9  CB1"}=" H91 CB1"; $resn_exch{"2H9  CB1"}=" H92 CB1"; $resn_exch{"H3T  CB2"}="HO3' CB2";
$resn_exch{" H3* CB2"}=" H3' CB2"; $resn_exch{"1H2* CB2"}=" H2' CB2"; $resn_exch{"2HO* CB2"}="HO2' CB2";
$resn_exch{" H1* CB2"}=" H1' CB2"; $resn_exch{" H4* CB2"}=" H4' CB2"; $resn_exch{"1H5* CB2"}=" H5' CB2";
$resn_exch{"2H5* CB2"}="H5'' CB2"; $resn_exch{"1HN4 CB2"}="HN41 CB2"; $resn_exch{"2HN4 CB2"}="HN42 CB2";
$resn_exch{"1HN2 CB3"}="HN21 CB3"; $resn_exch{"2HN2 CB3"}="HN22 CB3"; $resn_exch{"1H9  CB3"}=" H91 CB3";
$resn_exch{"2H9  CB3"}=" H92 CB3"; $resn_exch{"1HB  CB3"}=" HB1 CB3"; $resn_exch{"2HB  CB3"}=" HB2 CB3";
$resn_exch{"1HG  CB3"}=" HG1 CB3"; $resn_exch{"2HG  CB3"}=" HG2 CB3"; $resn_exch{"2HOE CB3"}="HOE2 CB3";
$resn_exch{"1HP1 CB3"}="HP11 CB3"; $resn_exch{"2HP1 CB3"}="HP12 CB3"; $resn_exch{"OHB1 CB4"}="HB1O CB4";
$resn_exch{"OHB2 CB4"}="HB2O CB4"; $resn_exch{"1HC7 CB4"}="HC71 CB4"; $resn_exch{"2HC7 CB4"}="HC72 CB4";
$resn_exch{"NH10 CB4"}="H10N CB4"; $resn_exch{"CH15 CB4"}="H15C CB4"; $resn_exch{"1H18 CB4"}="H181 CB4";
$resn_exch{"2H18 CB4"}="H182 CB4"; $resn_exch{"1H19 CB4"}="H191 CB4"; $resn_exch{"2H19 CB4"}="H192 CB4";
$resn_exch{"3H19 CB4"}="H193 CB4"; $resn_exch{"1H20 CB4"}="H201 CB4"; $resn_exch{"2H20 CB4"}="H202 CB4";
$resn_exch{"3H20 CB4"}="H203 CB4"; $resn_exch{"OH2B CB4"}="H2BO CB4"; $resn_exch{"1H2A CBA"}="H2A1 CBA";
$resn_exch{"2H2A CBA"}="H2A2 CBA"; $resn_exch{"3H2A CBA"}="H2A3 CBA"; $resn_exch{"1H4A CBA"}="H4A1 CBA";
$resn_exch{"2H4A CBA"}="H4A2 CBA"; $resn_exch{"1H5A CBA"}="H5A1 CBA"; $resn_exch{"2H5A CBA"}="H5A2 CBA";
$resn_exch{"2HOP CBA"}="HOP2 CBA"; $resn_exch{"3HOP CBA"}="HOP3 CBA"; $resn_exch{"2HOD CBA"}="HOD2 CBA";
$resn_exch{"1H10 CBB"}="H101 CBB"; $resn_exch{"2H10 CBB"}="H102 CBB"; $resn_exch{"1H16 CBB"}="H161 CBB";
$resn_exch{"18H1 CBB"}="H118 CBB"; $resn_exch{"1H20 CBB"}="H201 CBB"; $resn_exch{"2H20 CBB"}="H202 CBB";
$resn_exch{"1H33 CBB"}="H331 CBB"; $resn_exch{"2H33 CBB"}="H332 CBB"; $resn_exch{"34H3 CBB"}="H334 CBB";
$resn_exch{"1H37 CBB"}="H371 CBB"; $resn_exch{"1H38 CBB"}="H381 CBB"; $resn_exch{"1H39 CBB"}="H391 CBB";
$resn_exch{"1H41 CBB"}="H411 CBB"; $resn_exch{"2H41 CBB"}="H412 CBB"; $resn_exch{"1H43 CBB"}="H431 CBB";
$resn_exch{"2H43 CBB"}="H432 CBB"; $resn_exch{"3HOA CBD"}="HOA3 CBD"; $resn_exch{"1HN2 CBD"}="HN21 CBD";
$resn_exch{"2HN2 CBD"}="HN22 CBD"; $resn_exch{"3HOB CBD"}="HOB3 CBD"; $resn_exch{"3HOD CBD"}="HOD3 CBD";
$resn_exch{"1H1  CBE"}=" H11 CBE"; $resn_exch{"2H1  CBE"}="H12A CBE"; $resn_exch{"3H1  CBE"}="H13A CBE";
$resn_exch{"1H5  CBE"}=" H51 CBE"; $resn_exch{"2H5  CBE"}=" H52 CBE"; $resn_exch{"1H6  CBE"}=" H61 CBE";
$resn_exch{"2H6  CBE"}=" H62 CBE"; $resn_exch{"1H6  CBF"}=" H61 CBF"; $resn_exch{"2H6  CBF"}=" H62 CBF";
$resn_exch{"1HN1 CBF"}="HN11 CBF"; $resn_exch{"2HN1 CBF"}="HN12 CBF"; $resn_exch{"1H1  CBG"}=" H11 CBG";
$resn_exch{"1H2  CBG"}=" H21 CBG"; $resn_exch{"2H2  CBG"}=" H22 CBG"; $resn_exch{"1H3  CBG"}=" H31 CBG";
$resn_exch{"2H3  CBG"}=" H32 CBG"; $resn_exch{"3H3  CBG"}=" H33 CBG"; $resn_exch{"1H15 CBH"}="H151 CBH";
$resn_exch{"2H15 CBH"}="H152 CBH"; $resn_exch{"1H16 CBH"}="H161 CBH"; $resn_exch{"2H16 CBH"}="H162 CBH";
$resn_exch{"1H17 CBH"}="H171 CBH"; $resn_exch{"2H17 CBH"}="H172 CBH"; $resn_exch{"1H19 CBH"}="H191 CBH";
$resn_exch{"2H19 CBH"}="H192 CBH"; $resn_exch{"1H20 CBH"}="H201 CBH"; $resn_exch{"2H10 CBH"}="H102 CBH";
$resn_exch{"1H21 CBH"}="H211 CBH"; $resn_exch{"2H21 CBH"}="H212 CBH"; $resn_exch{"1H22 CBH"}="H221 CBH";
$resn_exch{"2H22 CBH"}="H222 CBH"; $resn_exch{"1H6  CBI"}=" H61 CBI"; $resn_exch{"2H6  CBI"}=" H62 CBI";
$resn_exch{"1H6' CBI"}="H6'1 CBI"; $resn_exch{"2H6' CBI"}="H6'2 CBI"; $resn_exch{"'HO1 CBI"}="HO1' CBI";
$resn_exch{"'HO2 CBI"}="HO2' CBI"; $resn_exch{"'HO3 CBI"}="HO3' CBI"; $resn_exch{"'HO6 CBI"}="HO6' CBI";
$resn_exch{"1H7  CBL"}=" H71 CBL"; $resn_exch{"2H7  CBL"}=" H72 CBL"; $resn_exch{"1H8  CBL"}=" H81 CBL";
$resn_exch{"2H8  CBL"}=" H82 CBL"; $resn_exch{"1H9  CBL"}=" H91 CBL"; $resn_exch{"2H9  CBL"}=" H92 CBL";
$resn_exch{"1H12 CBL"}="H121 CBL"; $resn_exch{"2H12 CBL"}="H122 CBL"; $resn_exch{"1H13 CBL"}="H131 CBL";
$resn_exch{"2H13 CBL"}="H132 CBL"; $resn_exch{"1H15 CBL"}="H151 CBL"; $resn_exch{"2H15 CBL"}="H152 CBL";
$resn_exch{"1H16 CBL"}="H161 CBL"; $resn_exch{"2H16 CBL"}="H162 CBL"; $resn_exch{"1H2  CBM"}=" H21 CBM";
$resn_exch{"2H2  CBM"}=" H22 CBM"; $resn_exch{"3H2  CBM"}=" H23 CBM"; $resn_exch{"1H7  CBN"}=" H71 CBN";
$resn_exch{"2H7  CBN"}=" H72 CBN"; $resn_exch{"3H7  CBN"}=" H73 CBN"; $resn_exch{"1H8  CBN"}=" H81 CBN";
$resn_exch{"2H8  CBN"}=" H82 CBN"; $resn_exch{"3H8  CBN"}=" H83 CBN"; $resn_exch{"1H12 CBN"}="H121 CBN";
$resn_exch{"2H12 CBN"}="H122 CBN"; $resn_exch{"3H12 CBN"}="H123 CBN"; $resn_exch{"1H25 CBN"}="H251 CBN";
$resn_exch{"2H25 CBN"}="H252 CBN"; $resn_exch{"1H30 CBN"}="H301 CBN"; $resn_exch{"2H30 CBN"}="H302 CBN";
$resn_exch{"3H30 CBN"}="H303 CBN"; $resn_exch{"1H34 CBN"}="H341 CBN"; $resn_exch{"2H34 CBN"}="H342 CBN";
$resn_exch{"3H34 CBN"}="H343 CBN"; $resn_exch{"1H6  CBN"}=" H61 CBN"; $resn_exch{"2H6  CBN"}=" H62 CBN";
$resn_exch{"3H6  CBN"}=" H63 CBN"; $resn_exch{"1H1  CBO"}=" H11 CBO"; $resn_exch{"2H1  CBO"}="H12A CBO";
$resn_exch{"1H2  CBO"}=" H21 CBO"; $resn_exch{"2H2  CBO"}=" H22 CBO"; $resn_exch{"1H6  CBO"}=" H61 CBO";
$resn_exch{"2H6  CBO"}=" H62 CBO"; $resn_exch{"1H7  CBO"}=" H71 CBO"; $resn_exch{"2H7  CBO"}=" H72 CBO";
$resn_exch{"1H15 CBO"}="H151 CBO"; $resn_exch{"2H15 CBO"}="H152 CBO"; $resn_exch{"1H16 CBO"}="H161 CBO";
$resn_exch{"2H16 CBO"}="H162 CBO"; $resn_exch{"1H19 CBO"}="H191 CBO"; $resn_exch{"2H19 CBO"}="H192 CBO";
$resn_exch{"3H19 CBO"}="H193 CBO"; $resn_exch{"1H20 CBO"}="H201 CBO"; $resn_exch{"2H20 CBO"}="H202 CBO";
$resn_exch{"1H22 CBO"}="H221 CBO"; $resn_exch{"2H22 CBO"}="H222 CBO"; $resn_exch{"1H23 CBO"}="H231 CBO";
$resn_exch{"2H23 CBO"}="H232 CBO"; $resn_exch{"1H24 CBO"}="H241 CBO"; $resn_exch{"2H24 CBO"}="H242 CBO";
$resn_exch{"3H24 CBO"}="H243 CBO"; $resn_exch{"1H25 CBO"}="H251 CBO"; $resn_exch{"2H25 CBO"}="H252 CBO";
$resn_exch{"3H25 CBO"}="H253 CBO"; $resn_exch{"1H26 CBO"}="H261 CBO"; $resn_exch{"2H26 CBO"}="H262 CBO";
$resn_exch{"3H26 CBO"}="H263 CBO"; $resn_exch{"1H27 CBO"}="H271 CBO"; $resn_exch{"2H27 CBO"}="H272 CBO";
$resn_exch{"3H27 CBO"}="H273 CBO"; $resn_exch{"1H28 CBO"}="H281 CBO"; $resn_exch{"2H28 CBO"}="H282 CBO";
$resn_exch{"3H28 CBO"}="H283 CBO"; $resn_exch{"1H30 CBO"}="H301 CBO"; $resn_exch{"2H30 CBO"}="H302 CBO";
$resn_exch{"1H31 CBO"}="H311 CBO"; $resn_exch{"2H31 CBO"}="H312 CBO"; $resn_exch{"1H34 CBO"}="H341 CBO";
$resn_exch{"2H34 CBO"}="H342 CBO"; $resn_exch{"3H34 CBO"}="H343 CBO"; $resn_exch{"IHO1 CBP"}="HO1I CBP";
$resn_exch{"1HN2 CBP"}="HN21 CBP"; $resn_exch{"1HC5 CBP"}="HC51 CBP"; $resn_exch{"2HC5 CBP"}="HC52 CBP";
$resn_exch{"1HC7 CBP"}="HC71 CBP"; $resn_exch{"2HC7 CBP"}="HC72 CBP"; $resn_exch{"1HC8 CBP"}="HC81 CBP";
$resn_exch{"2HC8 CBP"}="HC82 CBP"; $resn_exch{"1H10 CBP"}="H101 CBP"; $resn_exch{"2H10 CBP"}="H102 CBP";
$resn_exch{"1H11 CBP"}="H111 CBP"; $resn_exch{"2H11 CBP"}="H112 CBP"; $resn_exch{"1H16 CBP"}="H161 CBP";
$resn_exch{"1H17 CBP"}="H171 CBP"; $resn_exch{"1H19 CBP"}="H191 CBP"; $resn_exch{"1H20 CBP"}="H201 CBP";
$resn_exch{"1H23 CBP"}="H231 CBP"; $resn_exch{"1H24 CBP"}="H241 CBP"; $resn_exch{"1H26 CBP"}="H261 CBP";
$resn_exch{"1H27 CBP"}="H271 CBP"; $resn_exch{"2HOP CBR"}="HOP2 CBR"; $resn_exch{"1H4  CBR"}=" H41 CBR";
$resn_exch{"2H4  CBR"}=" H42 CBR"; $resn_exch{"1H2* CBR"}=" H2' CBR"; $resn_exch{"2H2* CBR"}="H2'' CBR";
$resn_exch{"1H5* CBR"}=" H5' CBR"; $resn_exch{"2H5* CBR"}="H5'' CBR"; $resn_exch{" H4* CBR"}=" H4' CBR";
$resn_exch{" H1* CBR"}=" H1' CBR"; $resn_exch{" H3* CBR"}=" H3' CBR"; $resn_exch{"H3T  CBR"}="HO3' CBR";
$resn_exch{"3HOP CBR"}="HOP3 CBR"; $resn_exch{" AH1 CBS"}=" H1A CBS"; $resn_exch{" AH2 CBS"}=" H2A CBS";
$resn_exch{" AH3 CBS"}=" H3A CBS"; $resn_exch{" AH4 CBS"}=" H4A CBS"; $resn_exch{" AH5 CBS"}=" H5A CBS";
$resn_exch{"AH61 CBS"}="H61A CBS"; $resn_exch{"AH62 CBS"}="H62A CBS"; $resn_exch{"AH81 CBS"}="H81A CBS";
$resn_exch{"AH82 CBS"}="H82A CBS"; $resn_exch{"AH83 CBS"}="H83A CBS"; $resn_exch{"AHN2 CBS"}="HN2A CBS";
$resn_exch{"AHO3 CBS"}="HO3A CBS"; $resn_exch{"AHO4 CBS"}="HO4A CBS"; $resn_exch{"AHO6 CBS"}="HO6A CBS";
$resn_exch{" BH1 CBS"}=" H1B CBS"; $resn_exch{" BH2 CBS"}=" H2B CBS"; $resn_exch{" BH3 CBS"}=" H3B CBS";
$resn_exch{" BH4 CBS"}=" H4B CBS"; $resn_exch{" BH5 CBS"}=" H5B CBS"; $resn_exch{"BH61 CBS"}="H61B CBS";
$resn_exch{"BH62 CBS"}="H62B CBS"; $resn_exch{"BH81 CBS"}="H81B CBS"; $resn_exch{"BH82 CBS"}="H82B CBS";
$resn_exch{"BH83 CBS"}="H83B CBS"; $resn_exch{"BHN2 CBS"}="HN2B CBS"; $resn_exch{"BHO1 CBS"}="HO1B CBS";
$resn_exch{"BHO3 CBS"}="HO3B CBS"; $resn_exch{"BHO6 CBS"}="HO6B CBS"; $resn_exch{"1H17 CBT"}="H171 CBT";
$resn_exch{"2H17 CBT"}="H172 CBT"; $resn_exch{"1H1  CBT"}=" H11 CBT"; $resn_exch{"2H1  CBT"}=" H12 CBT";
$resn_exch{"1H5* CBV"}="H5'1 CBV"; $resn_exch{"2H5* CBV"}="H5'2 CBV"; $resn_exch{" H4* CBV"}=" H4' CBV";
$resn_exch{" H3* CBV"}=" H3' CBV"; $resn_exch{"HO3* CBV"}="HO3' CBV"; $resn_exch{" H2* CBV"}=" H2' CBV";
$resn_exch{"HO2* CBV"}="HO2' CBV"; $resn_exch{" H1* CBV"}=" H1' CBV"; $resn_exch{"1HN4 CBV"}="HN41 CBV";
$resn_exch{"2HN4 CBV"}="HN42 CBV"; $resn_exch{"1H1' CBZ"}="H1'1 CBZ"; $resn_exch{"2H1' CBZ"}="H1'2 CBZ";
$resn_exch{"1HC1 CC0"}="HC11 CC0"; $resn_exch{"1HN1 CC0"}="HN11 CC0"; $resn_exch{"1HC0 CC0"}="HC01 CC0";
$resn_exch{"1HN2 CC0"}="HN21 CC0"; $resn_exch{"1HC4 CC0"}="HC41 CC0"; $resn_exch{"2HC4 CC0"}="HC42 CC0";
$resn_exch{"1HC6 CC0"}="HC61 CC0"; $resn_exch{"1HC7 CC0"}="HC71 CC0"; $resn_exch{"1HC9 CC0"}="HC91 CC0";
$resn_exch{"1H11 CC0"}="H111 CC0"; $resn_exch{"2H11 CC0"}="H112 CC0"; $resn_exch{"3H11 CC0"}="H113 CC0";
$resn_exch{"1H13 CC0"}="H131 CC0"; $resn_exch{"2H13 CC0"}="H132 CC0"; $resn_exch{"3H13 CC0"}="H133 CC0";
$resn_exch{"1H15 CC0"}="H151 CC0"; $resn_exch{"1H18 CC0"}="H181 CC0"; $resn_exch{"1H19 CC0"}="H191 CC0";
$resn_exch{"1H20 CC0"}="H201 CC0"; $resn_exch{"2H20 CC0"}="H202 CC0"; $resn_exch{"1H21 CC0"}="H211 CC0";
$resn_exch{"1H22 CC0"}="H221 CC0"; $resn_exch{"2H22 CC0"}="H222 CC0"; $resn_exch{"1H23 CC0"}="H231 CC0";
$resn_exch{"2H23 CC0"}="H232 CC0"; $resn_exch{"1H24 CC0"}="H241 CC0"; $resn_exch{"2H24 CC0"}="H242 CC0";
$resn_exch{"1H25 CC0"}="H251 CC0"; $resn_exch{"2H25 CC0"}="H252 CC0"; $resn_exch{"1H26 CC0"}="H261 CC0";
$resn_exch{"2H26 CC0"}="H262 CC0"; $resn_exch{"1HN3 CC0"}="HN31 CC0"; $resn_exch{"2HN3 CC0"}="HN32 CC0";
$resn_exch{"1H28 CC0"}="H281 CC0"; $resn_exch{"2H28 CC0"}="H282 CC0"; $resn_exch{"1HC1 CC1"}="HC11 CC1";
$resn_exch{"1HN1 CC1"}="HN11 CC1"; $resn_exch{"1HCO CC1"}="HCO1 CC1"; $resn_exch{"1HN2 CC1"}="HN21 CC1";
$resn_exch{"1H13 CC1"}="H131 CC1"; $resn_exch{"2H13 CC1"}="H132 CC1"; $resn_exch{"3H13 CC1"}="H133 CC1";
$resn_exch{"1HC4 CC1"}="HC41 CC1"; $resn_exch{"2HC4 CC1"}="HC42 CC1"; $resn_exch{"1HC6 CC1"}="HC61 CC1";
$resn_exch{"1HR1 CC1"}="HR11 CC1"; $resn_exch{"1HR3 CC1"}="HR31 CC1"; $resn_exch{"1HR4 CC1"}="HR41 CC1";
$resn_exch{"1HR5 CC1"}="HR51 CC1"; $resn_exch{"1HC9 CC1"}="HC91 CC1"; $resn_exch{"1HC7 CC1"}="HC71 CC1";
$resn_exch{"1H11 CC1"}="H111 CC1"; $resn_exch{"2H11 CC1"}="H112 CC1"; $resn_exch{"1H30 CC1"}="H301 CC1";
$resn_exch{"2H30 CC1"}="H302 CC1"; $resn_exch{"1H28 CC1"}="H281 CC1"; $resn_exch{"2H28 CC1"}="H282 CC1";
$resn_exch{"1H31 CC1"}="H311 CC1"; $resn_exch{"2H31 CC1"}="H312 CC1"; $resn_exch{"1H18 CC1"}="H181 CC1";
$resn_exch{"1H20 CC1"}="H201 CC1"; $resn_exch{"2H20 CC1"}="H202 CC1"; $resn_exch{"1H21 CC1"}="H211 CC1";
$resn_exch{"1H22 CC1"}="H221 CC1"; $resn_exch{"2H22 CC1"}="H222 CC1"; $resn_exch{"1H23 CC1"}="H231 CC1";
$resn_exch{"2H23 CC1"}="H232 CC1"; $resn_exch{"1H24 CC1"}="H241 CC1"; $resn_exch{"2H24 CC1"}="H242 CC1";
$resn_exch{"1H25 CC1"}="H251 CC1"; $resn_exch{"2H25 CC1"}="H252 CC1"; $resn_exch{"1H26 CC1"}="H261 CC1";
$resn_exch{"2H26 CC1"}="H262 CC1"; $resn_exch{"1H15 CC1"}="H151 CC1"; $resn_exch{"1HN3 CC1"}="HN31 CC1";
$resn_exch{"2HN3 CC1"}="HN32 CC1"; $resn_exch{"1H30 CC3"}="H301 CC3"; $resn_exch{"2H30 CC3"}="H302 CC3";
$resn_exch{"1H29 CC3"}="H291 CC3"; $resn_exch{"2H29 CC3"}="H292 CC3"; $resn_exch{"1H6N CC5"}="H6N1 CC5";
$resn_exch{"2H6N CC5"}="H6N2 CC5"; $resn_exch{"1H4' CC5"}="H4'1 CC5"; $resn_exch{"2H4' CC5"}="H4'2 CC5";
$resn_exch{"1H7  CCB"}=" H71 CCB"; $resn_exch{"2H7  CCB"}=" H72 CCB"; $resn_exch{"1H8  CCB"}=" H81 CCB";
$resn_exch{"2H8  CCB"}=" H82 CCB"; $resn_exch{"1H9  CCB"}=" H91 CCB"; $resn_exch{"2H9  CCB"}=" H92 CCB";
$resn_exch{"1H10 CCB"}="H101 CCB"; $resn_exch{"2H10 CCB"}="H102 CCB"; $resn_exch{"1H20 CCB"}="H201 CCB";
$resn_exch{"2H20 CCB"}="H202 CCB"; $resn_exch{"1H21 CCB"}="H211 CCB"; $resn_exch{"2H21 CCB"}="H212 CCB";
$resn_exch{"1H22 CCB"}="H221 CCB"; $resn_exch{"2H22 CCB"}="H222 CCB"; $resn_exch{"2HOC CCC"}="HOC2 CCC";
$resn_exch{"2HOP CCC"}="HOP2 CCC"; $resn_exch{"3HOP CCC"}="HOP3 CCC"; $resn_exch{"1H5* CCC"}=" H5' CCC";
$resn_exch{"2H5* CCC"}="H5'' CCC"; $resn_exch{" H4* CCC"}=" H4' CCC"; $resn_exch{" H3* CCC"}=" H3' CCC";
$resn_exch{" H2* CCC"}=" H2' CCC"; $resn_exch{" H1* CCC"}=" H1' CCC"; $resn_exch{"1H4  CCC"}=" H41 CCC";
$resn_exch{"2H4  CCC"}=" H42 CCC"; $resn_exch{"1H2C CCE"}="H2C1 CCE"; $resn_exch{"2H2C CCE"}="H2C2 CCE";
$resn_exch{"1H3C CCE"}="H3C1 CCE"; $resn_exch{"2H3C CCE"}="H3C2 CCE"; $resn_exch{"1H8C CCE"}="H8C1 CCE";
$resn_exch{"2H8C CCE"}="H8C2 CCE"; $resn_exch{"3H8C CCE"}="H8C3 CCE"; $resn_exch{"1H9C CCE"}="H9C1 CCE";
$resn_exch{"2H9C CCE"}="H9C2 CCE"; $resn_exch{"3H9C CCE"}="H9C3 CCE"; $resn_exch{"1H10 CCE"}="H101 CCE";
$resn_exch{"2H10 CCE"}="H102 CCE"; $resn_exch{"3H10 CCE"}="H103 CCE"; $resn_exch{"1H6N CCE"}="H6N1 CCE";
$resn_exch{"2H6N CCE"}="H6N2 CCE"; $resn_exch{"1HBC CCG"}="HBC1 CCG"; $resn_exch{"2HBC CCG"}="HBC2 CCG";
$resn_exch{"3HBC CCG"}="HBC3 CCG"; $resn_exch{"1HCV CCG"}="HCV1 CCG"; $resn_exch{"2HCV CCG"}="HCV2 CCG";
$resn_exch{"3HCV CCG"}="HCV3 CCG"; $resn_exch{"1HCU CCG"}="HCU1 CCG"; $resn_exch{"2HCU CCG"}="HCU2 CCG";
$resn_exch{"1HCX CCG"}="HCX1 CCG"; $resn_exch{"2HCX CCG"}="HCX2 CCG"; $resn_exch{"3HCX CCG"}="HCX3 CCG";
$resn_exch{"1HCO CCG"}="HCO1 CCG"; $resn_exch{"2HCO CCG"}="HCO2 CCG"; $resn_exch{"1HBW CCG"}="HBW1 CCG";
$resn_exch{"2HBW CCG"}="HBW2 CCG"; $resn_exch{"1HCC CCG"}="HCC1 CCG"; $resn_exch{"2HCC CCG"}="HCC2 CCG";
$resn_exch{"3HCC CCG"}="HCC3 CCG"; $resn_exch{"1HBZ CCG"}="HBZ1 CCG"; $resn_exch{"2HBZ CCG"}="HBZ2 CCG";
$resn_exch{"1HCA CCG"}="HCA1 CCG"; $resn_exch{"2HCA CCG"}="HCA2 CCG"; $resn_exch{"1HCB CCG"}="HCB1 CCG";
$resn_exch{"2HCB CCG"}="HCB2 CCG"; $resn_exch{"3HCB CCG"}="HCB3 CCG"; $resn_exch{"1HAF CCG"}="HAF1 CCG";
$resn_exch{"2HAF CCG"}="HAF2 CCG"; $resn_exch{"3HAF CCG"}="HAF3 CCG"; $resn_exch{"1HAM CCG"}="HAM1 CCG";
$resn_exch{"2HAM CCG"}="HAM2 CCG"; $resn_exch{"1HAQ CCG"}="HAQ1 CCG"; $resn_exch{"2HAQ CCG"}="HAQ2 CCG";
$resn_exch{"3HAQ CCG"}="HAQ3 CCG"; $resn_exch{"1HBE CCG"}="HBE1 CCG"; $resn_exch{"2HBE CCG"}="HBE2 CCG";
$resn_exch{"3HBE CCG"}="HBE3 CCG"; $resn_exch{"1HBD CCG"}="HBD1 CCG"; $resn_exch{"2HBD CCG"}="HBD2 CCG";
$resn_exch{"3HBD CCG"}="HBD3 CCG"; $resn_exch{"1HBO CCG"}="HBO1 CCG"; $resn_exch{"2HBO CCG"}="HBO2 CCG";
$resn_exch{"3HBO CCG"}="HBO3 CCG"; $resn_exch{"1HBP CCG"}="HBP1 CCG"; $resn_exch{"2HBP CCG"}="HBP2 CCG";
$resn_exch{"3HBP CCG"}="HBP3 CCG"; $resn_exch{"1HAA CCH"}="HAA1 CCH"; $resn_exch{"2HAA CCH"}="HAA2 CCH";
$resn_exch{"1HMA CCH"}="HMA1 CCH"; $resn_exch{"2HMA CCH"}="HMA2 CCH"; $resn_exch{"3HMA CCH"}="HMA3 CCH";
$resn_exch{"1HBA CCH"}="HBA1 CCH"; $resn_exch{"2HBA CCH"}="HBA2 CCH"; $resn_exch{"AHO1 CCH"}="HO1A CCH";
$resn_exch{"1HMB CCH"}="HMB1 CCH"; $resn_exch{"2HMB CCH"}="HMB2 CCH"; $resn_exch{"3HMB CCH"}="HMB3 CCH";
$resn_exch{"1HMC CCH"}="HMC1 CCH"; $resn_exch{"2HMC CCH"}="HMC2 CCH"; $resn_exch{"3HMC CCH"}="HMC3 CCH";
$resn_exch{"1HBC CCH"}="HBC1 CCH"; $resn_exch{"2HBC CCH"}="HBC2 CCH"; $resn_exch{"1HMD CCH"}="HMD1 CCH";
$resn_exch{"2HMD CCH"}="HMD2 CCH"; $resn_exch{"3HMD CCH"}="HMD3 CCH"; $resn_exch{"1HAD CCH"}="HAD1 CCH";
$resn_exch{"2HAD CCH"}="HAD2 CCH"; $resn_exch{"1HBD CCH"}="HBD1 CCH"; $resn_exch{"2HBD CCH"}="HBD2 CCH";
$resn_exch{"DHO1 CCH"}="HO1D CCH"; $resn_exch{"1H6D CCI"}="H6D1 CCI"; $resn_exch{"2H6D CCI"}="H6D2 CCI";
$resn_exch{"3H6D CCI"}="H6D3 CCI"; $resn_exch{"2HOD CCI"}="HOD2 CCI"; $resn_exch{"4HOD CCI"}="HOD4 CCI";
$resn_exch{"1H7D CCI"}="H7D1 CCI"; $resn_exch{"2H7D CCI"}="H7D2 CCI"; $resn_exch{"3H7D CCI"}="H7D3 CCI";
$resn_exch{"1H8C CCI"}="H8C1 CCI"; $resn_exch{"2H8C CCI"}="H8C2 CCI"; $resn_exch{"3H8C CCI"}="H8C3 CCI";
$resn_exch{"1H9C CCI"}="H9C1 CCI"; $resn_exch{"2H9C CCI"}="H9C2 CCI"; $resn_exch{"3H9C CCI"}="H9C3 CCI";
$resn_exch{"1H7C CCI"}="H7C1 CCI"; $resn_exch{"2H7C CCI"}="H7C2 CCI"; $resn_exch{"3H7C CCI"}="H7C3 CCI";
$resn_exch{"1H2B CCI"}="H2B1 CCI"; $resn_exch{"2H2B CCI"}="H2B2 CCI"; $resn_exch{"1H6B CCI"}="H6B1 CCI";
$resn_exch{"2H6B CCI"}="H6B2 CCI"; $resn_exch{"3H6B CCI"}="H6B3 CCI"; $resn_exch{"3HOB CCI"}="HOB3 CCI";
$resn_exch{"1H6A CCI"}="H6A1 CCI"; $resn_exch{"2H6A CCI"}="H6A2 CCI"; $resn_exch{"3H6A CCI"}="H6A3 CCI";
$resn_exch{"3HOA CCI"}="HOA3 CCI"; $resn_exch{"1HAN CCI"}="HAN1 CCI"; $resn_exch{"2HAN CCI"}="HAN2 CCI";
$resn_exch{"3HAN CCI"}="HAN3 CCI"; $resn_exch{"1H2E CCI"}="H2E1 CCI"; $resn_exch{"2H2E CCI"}="H2E2 CCI";
$resn_exch{"1H5E CCI"}="H5E1 CCI"; $resn_exch{"2H5E CCI"}="H5E2 CCI"; $resn_exch{"1H6E CCI"}="H6E1 CCI";
$resn_exch{"2H6E CCI"}="H6E2 CCI"; $resn_exch{"3H6E CCI"}="H6E3 CCI"; $resn_exch{"2HNE CCI"}="HNE2 CCI";
$resn_exch{"1H7E CCI"}="H7E1 CCI"; $resn_exch{"2H7E CCI"}="H7E2 CCI"; $resn_exch{"1H8E CCI"}="H8E1 CCI";
$resn_exch{"2H8E CCI"}="H8E2 CCI"; $resn_exch{"3H8E CCI"}="H8E3 CCI"; $resn_exch{"1HAK CCL"}="HAK1 CCL";
$resn_exch{"2HAK CCL"}="HAK2 CCL"; $resn_exch{"1HAG CCL"}="HAG1 CCL"; $resn_exch{"2HAG CCL"}="HAG2 CCL";
$resn_exch{"1HAH CCL"}="HAH1 CCL"; $resn_exch{"2HAH CCL"}="HAH2 CCL"; $resn_exch{"1HAL CCL"}="HAL1 CCL";
$resn_exch{"2HAL CCL"}="HAL2 CCL"; $resn_exch{"1HAI CCL"}="HAI1 CCL"; $resn_exch{"2HAI CCL"}="HAI2 CCL";
$resn_exch{"1HAE CCL"}="HAE1 CCL"; $resn_exch{"2HAE CCL"}="HAE2 CCL"; $resn_exch{"1HAF CCL"}="HAF1 CCL";
$resn_exch{"2HAF CCL"}="HAF2 CCL"; $resn_exch{"1HB  CCL"}=" HB1 CCL"; $resn_exch{"2HB  CCL"}=" HB2 CCL";
$resn_exch{"1HN  CCL"}=" HN1 CCL"; $resn_exch{"2HN  CCL"}=" HN2 CCL"; $resn_exch{"1HH2 CCM"}="HH21 CCM";
$resn_exch{"2HH2 CCM"}="HH22 CCM"; $resn_exch{"3HH2 CCM"}="HH23 CCM"; $resn_exch{"1HB1 CCM"}="HB11 CCM";
$resn_exch{"2HB1 CCM"}="HB12 CCM"; $resn_exch{"1HB2 CCM"}="HB21 CCM"; $resn_exch{"2HB2 CCM"}="HB22 CCM";
$resn_exch{"1HG1 CCM"}="HG11 CCM"; $resn_exch{"2HG1 CCM"}="HG12 CCM"; $resn_exch{"1HG2 CCM"}="HG21 CCM";
$resn_exch{"2HG2 CCM"}="HG22 CCM"; $resn_exch{"1H2  CCN"}=" H21 CCN"; $resn_exch{"2H2  CCN"}=" H22 CCN";
$resn_exch{"3H2  CCN"}=" H23 CCN"; $resn_exch{"1H1B CCO"}="H1B1 CCO"; $resn_exch{"2H1B CCO"}="H1B2 CCO";
$resn_exch{"2HOP CCP"}="HOP2 CCP"; $resn_exch{"3HOP CCP"}="HOP3 CCP"; $resn_exch{"1H1  CCP"}=" H11 CCP";
$resn_exch{"2H1  CCP"}=" H12 CCP"; $resn_exch{"1H2  CCP"}=" H21 CCP"; $resn_exch{"2H2  CCP"}=" H22 CCP";
$resn_exch{"1H3  CCP"}=" H31 CCP"; $resn_exch{"2H3  CCP"}=" H32 CCP"; $resn_exch{"1H4  CCP"}=" H41 CCP";
$resn_exch{"2H4  CCP"}=" H42 CCP"; $resn_exch{"3H4  CCP"}=" H43 CCP"; $resn_exch{"1H2  CCQ"}=" H21 CCQ";
$resn_exch{"2H2  CCQ"}=" H22 CCQ"; $resn_exch{"1H4  CCQ"}=" H41 CCQ"; $resn_exch{"2H4  CCQ"}=" H42 CCQ";
$resn_exch{"1H5C CCQ"}="H5C1 CCQ"; $resn_exch{"2H5C CCQ"}="H5C2 CCQ"; $resn_exch{"3H5C CCQ"}="H5C3 CCQ";
$resn_exch{"1H5B CCQ"}="H5B1 CCQ"; $resn_exch{"2H5B CCQ"}="H5B2 CCQ"; $resn_exch{"3H5B CCQ"}="H5B3 CCQ";
$resn_exch{"1H5A CCQ"}="H5A1 CCQ"; $resn_exch{"2H5A CCQ"}="H5A2 CCQ"; $resn_exch{"3H5A CCQ"}="H5A3 CCQ";
$resn_exch{"1H1  CCQ"}=" H11 CCQ"; $resn_exch{"2H1  CCQ"}=" H12 CCQ"; $resn_exch{"1H3  CCQ"}=" H31 CCQ";
$resn_exch{"2H3  CCQ"}=" H32 CCQ"; $resn_exch{"1H6  CCQ"}=" H61 CCQ"; $resn_exch{"2H6  CCQ"}=" H62 CCQ";
$resn_exch{"1H7  CCQ"}=" H71 CCQ"; $resn_exch{"2H7  CCQ"}=" H72 CCQ"; $resn_exch{"1H13 CCQ"}="H131 CCQ";
$resn_exch{"2H13 CCQ"}="H132 CCQ"; $resn_exch{"3H13 CCQ"}="H133 CCQ"; $resn_exch{"1H14 CCQ"}="H141 CCQ";
$resn_exch{"2H14 CCQ"}="H142 CCQ"; $resn_exch{"3H14 CCQ"}="H143 CCQ"; $resn_exch{"1H12 CCQ"}="H121 CCQ";
$resn_exch{"2H12 CCQ"}="H122 CCQ"; $resn_exch{"1H5* CCQ"}="H5'1 CCQ"; $resn_exch{"2H5* CCQ"}="H5'2 CCQ";
$resn_exch{" H4* CCQ"}=" H4' CCQ"; $resn_exch{" H3* CCQ"}=" H3' CCQ"; $resn_exch{" H2* CCQ"}=" H2' CCQ";
$resn_exch{" H1* CCQ"}=" H1' CCQ"; $resn_exch{"1HN6 CCQ"}="HN61 CCQ"; $resn_exch{"2HN6 CCQ"}="HN62 CCQ";
$resn_exch{"1H52 CCR"}="H521 CCR"; $resn_exch{"1H53 CCR"}="H531 CCR"; $resn_exch{"1H55 CCR"}="H551 CCR";
$resn_exch{"1H56 CCR"}="H561 CCR"; $resn_exch{"1H58 CCR"}="H581 CCR"; $resn_exch{"2H58 CCR"}="H582 CCR";
$resn_exch{"1H59 CCR"}="H591 CCR"; $resn_exch{"1H44 CCR"}="H441 CCR"; $resn_exch{"2H44 CCR"}="H442 CCR";
$resn_exch{"1H41 CCR"}="H411 CCR"; $resn_exch{"1H40 CCR"}="H401 CCR"; $resn_exch{"1H61 CCR"}="H611 CCR";
$resn_exch{"2H61 CCR"}="H612 CCR"; $resn_exch{"1H62 CCR"}="H621 CCR"; $resn_exch{"2H62 CCR"}="H622 CCR";
$resn_exch{"1H63 CCR"}="H631 CCR"; $resn_exch{"2H63 CCR"}="H632 CCR"; $resn_exch{"1H64 CCR"}="H641 CCR";
$resn_exch{"2H64 CCR"}="H642 CCR"; $resn_exch{"1H65 CCR"}="H651 CCR"; $resn_exch{"2H65 CCR"}="H652 CCR";
$resn_exch{"1H23 CCR"}="H231 CCR"; $resn_exch{"1H15 CCR"}="H151 CCR"; $resn_exch{"1H17 CCR"}="H171 CCR";
$resn_exch{"2H17 CCR"}="H172 CCR"; $resn_exch{"3H17 CCR"}="H173 CCR"; $resn_exch{"1H18 CCR"}="H181 CCR";
$resn_exch{"2H18 CCR"}="H182 CCR"; $resn_exch{"3H18 CCR"}="H183 CCR"; $resn_exch{"1H10 CCR"}="H101 CCR";
$resn_exch{"2H10 CCR"}="H102 CCR"; $resn_exch{"3H10 CCR"}="H103 CCR"; $resn_exch{"1H19 CCR"}="H191 CCR";
$resn_exch{"2H19 CCR"}="H192 CCR"; $resn_exch{"3H19 CCR"}="H193 CCR"; $resn_exch{"1H30 CCR"}="H301 CCR";
$resn_exch{"1H31 CCR"}="H311 CCR"; $resn_exch{"2H31 CCR"}="H312 CCR"; $resn_exch{"1H34 CCR"}="H341 CCR";
$resn_exch{"1HN  CCS"}=" H   CCS"; $resn_exch{"2HN  CCS"}=" H2  CCS"; $resn_exch{"1HB  CCS"}=" HB2 CCS";
$resn_exch{"2HB  CCS"}=" HB3 CCS"; $resn_exch{"1HD  CCS"}=" HD2 CCS"; $resn_exch{"2HD  CCS"}=" HD3 CCS";
$resn_exch{" HOZ CCS"}=" HZ2 CCS"; $resn_exch{"1H25 CCT"}="H251 CCT"; $resn_exch{"2H25 CCT"}="H252 CCT";
$resn_exch{"3H25 CCT"}="H253 CCT"; $resn_exch{"3H4  CCV"}=" H43 CCV"; $resn_exch{"2H4  CCV"}=" H42 CCV";
$resn_exch{"1H4  CCV"}=" H41 CCV"; $resn_exch{"2H11 CCV"}="H112 CCV"; $resn_exch{"1H11 CCV"}="H111 CCV";
$resn_exch{"2H13 CCV"}="H132 CCV"; $resn_exch{"1H13 CCV"}="H131 CCV"; $resn_exch{"2H14 CCV"}="H142 CCV";
$resn_exch{"1H14 CCV"}="H141 CCV"; $resn_exch{"1HA3 CCY"}="HA31 CCY"; $resn_exch{"2HA3 CCY"}="HA32 CCY";
$resn_exch{"1HN1 CCY"}="HN11 CCY"; $resn_exch{"2HN1 CCY"}="HN12 CCY"; $resn_exch{"1HB1 CCY"}="HB11 CCY";
$resn_exch{"2HB1 CCY"}="HB12 CCY"; $resn_exch{"1HB2 CCY"}="HB21 CCY"; $resn_exch{"2HB2 CCY"}="HB22 CCY";
$resn_exch{"1HO1 CD1"}="HO11 CD1"; $resn_exch{"2HO1 CD1"}="HO12 CD1"; $resn_exch{"1HO1 CD3"}="HO11 CD3";
$resn_exch{"2HO1 CD3"}="HO12 CD3"; $resn_exch{"1HO2 CD3"}="HO21 CD3"; $resn_exch{"2HO2 CD3"}="HO22 CD3";
$resn_exch{"1HO3 CD3"}="HO31 CD3"; $resn_exch{"2HO3 CD3"}="HO32 CD3"; $resn_exch{"1HO1 CD5"}="HO11 CD5";
$resn_exch{"2HO1 CD5"}="HO12 CD5"; $resn_exch{"1HO2 CD5"}="HO21 CD5"; $resn_exch{"2HO2 CD5"}="HO22 CD5";
$resn_exch{"1HO3 CD5"}="HO31 CD5"; $resn_exch{"2HO3 CD5"}="HO32 CD5"; $resn_exch{"1HO4 CD5"}="HO41 CD5";
$resn_exch{"2HO4 CD5"}="HO42 CD5"; $resn_exch{"1HO5 CD5"}="HO51 CD5"; $resn_exch{"2HO5 CD5"}="HO52 CD5";
$resn_exch{"1H12 CDA"}="H121 CDA"; $resn_exch{"2H12 CDA"}="H122 CDA"; $resn_exch{"1H29 CDA"}="H291 CDA";
$resn_exch{"2H29 CDA"}="H292 CDA"; $resn_exch{"3H29 CDA"}="H293 CDA"; $resn_exch{"1H33 CDA"}="H331 CDA";
$resn_exch{"2H33 CDA"}="H332 CDA"; $resn_exch{"1H39 CDA"}="H391 CDA"; $resn_exch{"2H39 CDA"}="H392 CDA";
$resn_exch{"1H12 CDB"}="H121 CDB"; $resn_exch{"2H12 CDB"}="H122 CDB"; $resn_exch{"1H29 CDB"}="H291 CDB";
$resn_exch{"2H29 CDB"}="H292 CDB"; $resn_exch{"3H29 CDB"}="H293 CDB"; $resn_exch{"1H33 CDB"}="H331 CDB";
$resn_exch{"2H33 CDB"}="H332 CDB"; $resn_exch{"1H39 CDB"}="H391 CDB"; $resn_exch{"2H39 CDB"}="H392 CDB";
$resn_exch{"1H1  CDB"}=" H11 CDB"; $resn_exch{"2H1  CDB"}=" H12 CDB"; $resn_exch{"3H1  CDB"}=" H13 CDB";
$resn_exch{"1H4  CDC"}=" H41 CDC"; $resn_exch{"2H4  CDC"}=" H42 CDC"; $resn_exch{" H1* CDC"}=" H1' CDC";
$resn_exch{" H2* CDC"}=" H2' CDC"; $resn_exch{"2HO* CDC"}="HO'2 CDC"; $resn_exch{" H3* CDC"}=" H3' CDC";
$resn_exch{" H4* CDC"}=" H4' CDC"; $resn_exch{"1H5* CDC"}="H5'1 CDC"; $resn_exch{"2H5* CDC"}="H5'2 CDC";
$resn_exch{"1H14 CDC"}="H141 CDC"; $resn_exch{"2H14 CDC"}="H142 CDC"; $resn_exch{"1H15 CDC"}="H151 CDC";
$resn_exch{"2H15 CDC"}="H152 CDC"; $resn_exch{"1H16 CDC"}="H161 CDC"; $resn_exch{"2H16 CDC"}="H162 CDC";
$resn_exch{"3H16 CDC"}="H163 CDC"; $resn_exch{"1H17 CDC"}="H171 CDC"; $resn_exch{"2H17 CDC"}="H172 CDC";
$resn_exch{"3H17 CDC"}="H173 CDC"; $resn_exch{"1H18 CDC"}="H181 CDC"; $resn_exch{"2H18 CDC"}="H182 CDC";
$resn_exch{"3H18 CDC"}="H183 CDC"; $resn_exch{"1H12 CDD"}="H121 CDD"; $resn_exch{"2H12 CDD"}="H122 CDD";
$resn_exch{"1H24 CDD"}="H241 CDD"; $resn_exch{"2H24 CDD"}="H242 CDD"; $resn_exch{"1H29 CDD"}="H291 CDD";
$resn_exch{"2H29 CDD"}="H292 CDD"; $resn_exch{"1HN  CDE"}=" H   CDE"; $resn_exch{"2HN  CDE"}=" H2  CDE";
$resn_exch{"1HG1 CDE"}="HG11 CDE"; $resn_exch{"2HG1 CDE"}="HG12 CDE"; $resn_exch{"3HG1 CDE"}="HG13 CDE";
$resn_exch{"1HG2 CDE"}="HG21 CDE"; $resn_exch{"2HG2 CDE"}="HG22 CDE"; $resn_exch{"3HG2 CDE"}="HG23 CDE";
$resn_exch{"1H   CDE"}=" HC1 CDE"; $resn_exch{"2H   CDE"}=" HC2 CDE"; $resn_exch{"3H   CDE"}=" HC3 CDE";
$resn_exch{"1H5* CDF"}="H5'1 CDF"; $resn_exch{"2H5* CDF"}="H5'2 CDF"; $resn_exch{" H4* CDF"}=" H4' CDF";
$resn_exch{" H3* CDF"}=" H3' CDF"; $resn_exch{" H2* CDF"}=" H2' CDF"; $resn_exch{" H1* CDF"}=" H1' CDF";
$resn_exch{"1H4N CDF"}="H4N1 CDF"; $resn_exch{"2H4N CDF"}="H4N2 CDF"; $resn_exch{"1H6C CDG"}="H6C1 CDG";
$resn_exch{"2H6C CDG"}="H6C2 CDG"; $resn_exch{"1H9C CDG"}="H9C1 CDG"; $resn_exch{"2H9C CDG"}="H9C2 CDG";
$resn_exch{"3H9C CDG"}="H9C3 CDG"; $resn_exch{"1H10 CDG"}="H101 CDG"; $resn_exch{"2H10 CDG"}="H102 CDG";
$resn_exch{"3H10 CDG"}="H103 CDG"; $resn_exch{"1H37 CDH"}="H371 CDH"; $resn_exch{"2H37 CDH"}="H372 CDH";
$resn_exch{"1H33 CDH"}="H331 CDH"; $resn_exch{"2H33 CDH"}="H332 CDH"; $resn_exch{"3H33 CDH"}="H333 CDH";
$resn_exch{"1H12 CDH"}="H121 CDH"; $resn_exch{"1H16 CDH"}="H161 CDH"; $resn_exch{"2H16 CDH"}="H162 CDH";
$resn_exch{"NH30 CDH"}="H30N CDH"; $resn_exch{"1H24 CDH"}="H241 CDH"; $resn_exch{"2H24 CDH"}="H242 CDH";
$resn_exch{"1H23 CDH"}="H231 CDH"; $resn_exch{"2H23 CDH"}="H232 CDH"; $resn_exch{"1H22 CDH"}="H221 CDH";
$resn_exch{"2H22 CDH"}="H222 CDH"; $resn_exch{"1H21 CDH"}="H211 CDH"; $resn_exch{"1H31 CDH"}="H311 CDH";
$resn_exch{"2H31 CDH"}="H312 CDH"; $resn_exch{"1HC4 CDI"}="HC41 CDI"; $resn_exch{"2HC4 CDI"}="HC42 CDI";
$resn_exch{"OHB2 CDI"}="HB2O CDI"; $resn_exch{"OHA1 CDI"}="HA1O CDI"; $resn_exch{"1HC1 CDI"}="HC11 CDI";
$resn_exch{"2HC1 CDI"}="HC12 CDI"; $resn_exch{"1HC5 CDI"}="HC51 CDI"; $resn_exch{"2HC5 CDI"}="HC52 CDI";
$resn_exch{"3HC5 CDI"}="HC53 CDI"; $resn_exch{"1H1O CDL"}="H1O1 CDL"; $resn_exch{"2HA2 CDL"}="HA22 CDL";
$resn_exch{"1HA2 CDL"}="HA21 CDL"; $resn_exch{"2HA3 CDL"}="HA32 CDL"; $resn_exch{"1HA3 CDL"}="HA31 CDL";
$resn_exch{"2H11 CDL"}="H112 CDL"; $resn_exch{"1H11 CDL"}="H111 CDL"; $resn_exch{"2H12 CDL"}="H122 CDL";
$resn_exch{"1H12 CDL"}="H121 CDL"; $resn_exch{"2H13 CDL"}="H132 CDL"; $resn_exch{"1H13 CDL"}="H131 CDL";
$resn_exch{"2H14 CDL"}="H142 CDL"; $resn_exch{"1H14 CDL"}="H141 CDL"; $resn_exch{"2H15 CDL"}="H152 CDL";
$resn_exch{"1H15 CDL"}="H151 CDL"; $resn_exch{"2H16 CDL"}="H162 CDL"; $resn_exch{"1H16 CDL"}="H161 CDL";
$resn_exch{"2H17 CDL"}="H172 CDL"; $resn_exch{"1H17 CDL"}="H171 CDL"; $resn_exch{"2H18 CDL"}="H182 CDL";
$resn_exch{"1H18 CDL"}="H181 CDL"; $resn_exch{"2H19 CDL"}="H192 CDL"; $resn_exch{"1H19 CDL"}="H191 CDL";
$resn_exch{"2H20 CDL"}="H202 CDL"; $resn_exch{"1H20 CDL"}="H201 CDL"; $resn_exch{"2H21 CDL"}="H212 CDL";
$resn_exch{"1H21 CDL"}="H211 CDL"; $resn_exch{"2H22 CDL"}="H222 CDL"; $resn_exch{"1H22 CDL"}="H221 CDL";
$resn_exch{"2H23 CDL"}="H232 CDL"; $resn_exch{"1H23 CDL"}="H231 CDL"; $resn_exch{"2H24 CDL"}="H242 CDL";
$resn_exch{"1H24 CDL"}="H241 CDL"; $resn_exch{"2H25 CDL"}="H252 CDL"; $resn_exch{"1H25 CDL"}="H251 CDL";
$resn_exch{"2H26 CDL"}="H262 CDL"; $resn_exch{"1H26 CDL"}="H261 CDL"; $resn_exch{"3H27 CDL"}="H273 CDL";
$resn_exch{"2H27 CDL"}="H272 CDL"; $resn_exch{"1H27 CDL"}="H271 CDL"; $resn_exch{"2HA6 CDL"}="HA62 CDL";
$resn_exch{"1HA6 CDL"}="HA61 CDL"; $resn_exch{"2H31 CDL"}="H312 CDL"; $resn_exch{"1H31 CDL"}="H311 CDL";
$resn_exch{"2H32 CDL"}="H322 CDL"; $resn_exch{"1H32 CDL"}="H321 CDL"; $resn_exch{"2H33 CDL"}="H332 CDL";
$resn_exch{"1H33 CDL"}="H331 CDL"; $resn_exch{"2H34 CDL"}="H342 CDL"; $resn_exch{"1H34 CDL"}="H341 CDL";
$resn_exch{"2H35 CDL"}="H352 CDL"; $resn_exch{"1H35 CDL"}="H351 CDL"; $resn_exch{"2H36 CDL"}="H362 CDL";
$resn_exch{"1H36 CDL"}="H361 CDL"; $resn_exch{"2H37 CDL"}="H372 CDL"; $resn_exch{"1H37 CDL"}="H371 CDL";
$resn_exch{"2H38 CDL"}="H382 CDL"; $resn_exch{"1H38 CDL"}="H381 CDL"; $resn_exch{"2H39 CDL"}="H392 CDL";
$resn_exch{"1H39 CDL"}="H391 CDL"; $resn_exch{"2H40 CDL"}="H402 CDL"; $resn_exch{"1H40 CDL"}="H401 CDL";
$resn_exch{"2H41 CDL"}="H412 CDL"; $resn_exch{"1H41 CDL"}="H411 CDL"; $resn_exch{"2H42 CDL"}="H422 CDL";
$resn_exch{"1H42 CDL"}="H421 CDL"; $resn_exch{"2H43 CDL"}="H432 CDL"; $resn_exch{"1H43 CDL"}="H431 CDL";
$resn_exch{"2H44 CDL"}="H442 CDL"; $resn_exch{"1H44 CDL"}="H441 CDL"; $resn_exch{"2H45 CDL"}="H452 CDL";
$resn_exch{"1H45 CDL"}="H451 CDL"; $resn_exch{"2H46 CDL"}="H462 CDL"; $resn_exch{"1H46 CDL"}="H461 CDL";
$resn_exch{"3H47 CDL"}="H473 CDL"; $resn_exch{"2H47 CDL"}="H472 CDL"; $resn_exch{"1H47 CDL"}="H471 CDL";
$resn_exch{"2HB2 CDL"}="HB22 CDL"; $resn_exch{"1HB2 CDL"}="HB21 CDL"; $resn_exch{"2HB3 CDL"}="HB32 CDL";
$resn_exch{"1HB3 CDL"}="HB31 CDL"; $resn_exch{"2H51 CDL"}="H512 CDL"; $resn_exch{"1H51 CDL"}="H511 CDL";
$resn_exch{"2H52 CDL"}="H522 CDL"; $resn_exch{"1H52 CDL"}="H521 CDL"; $resn_exch{"2H53 CDL"}="H532 CDL";
$resn_exch{"1H53 CDL"}="H531 CDL"; $resn_exch{"2H54 CDL"}="H542 CDL"; $resn_exch{"1H54 CDL"}="H541 CDL";
$resn_exch{"2H55 CDL"}="H552 CDL"; $resn_exch{"1H55 CDL"}="H551 CDL"; $resn_exch{"2H56 CDL"}="H562 CDL";
$resn_exch{"1H56 CDL"}="H561 CDL"; $resn_exch{"2H57 CDL"}="H572 CDL"; $resn_exch{"1H57 CDL"}="H571 CDL";
$resn_exch{"2H58 CDL"}="H582 CDL"; $resn_exch{"1H58 CDL"}="H581 CDL"; $resn_exch{"2H59 CDL"}="H592 CDL";
$resn_exch{"1H59 CDL"}="H591 CDL"; $resn_exch{"2H60 CDL"}="H602 CDL"; $resn_exch{"1H60 CDL"}="H601 CDL";
$resn_exch{"2H61 CDL"}="H612 CDL"; $resn_exch{"1H61 CDL"}="H611 CDL"; $resn_exch{"2H62 CDL"}="H622 CDL";
$resn_exch{"1H62 CDL"}="H621 CDL"; $resn_exch{"2H63 CDL"}="H632 CDL"; $resn_exch{"1H63 CDL"}="H631 CDL";
$resn_exch{"2H64 CDL"}="H642 CDL"; $resn_exch{"1H64 CDL"}="H641 CDL"; $resn_exch{"2H65 CDL"}="H652 CDL";
$resn_exch{"1H65 CDL"}="H651 CDL"; $resn_exch{"2H66 CDL"}="H662 CDL"; $resn_exch{"1H66 CDL"}="H661 CDL";
$resn_exch{"3H67 CDL"}="H673 CDL"; $resn_exch{"2H67 CDL"}="H672 CDL"; $resn_exch{"1H67 CDL"}="H671 CDL";
$resn_exch{"2HB6 CDL"}="HB62 CDL"; $resn_exch{"1HB6 CDL"}="HB61 CDL"; $resn_exch{"2H71 CDL"}="H712 CDL";
$resn_exch{"1H71 CDL"}="H711 CDL"; $resn_exch{"2H72 CDL"}="H722 CDL"; $resn_exch{"1H72 CDL"}="H721 CDL";
$resn_exch{"2H73 CDL"}="H732 CDL"; $resn_exch{"1H73 CDL"}="H731 CDL"; $resn_exch{"2H74 CDL"}="H742 CDL";
$resn_exch{"1H74 CDL"}="H741 CDL"; $resn_exch{"2H75 CDL"}="H752 CDL"; $resn_exch{"1H75 CDL"}="H751 CDL";
$resn_exch{"2H76 CDL"}="H762 CDL"; $resn_exch{"1H76 CDL"}="H761 CDL"; $resn_exch{"2H77 CDL"}="H772 CDL";
$resn_exch{"1H77 CDL"}="H771 CDL"; $resn_exch{"2H78 CDL"}="H782 CDL"; $resn_exch{"1H78 CDL"}="H781 CDL";
$resn_exch{"2H79 CDL"}="H792 CDL"; $resn_exch{"1H79 CDL"}="H791 CDL"; $resn_exch{"2H80 CDL"}="H802 CDL";
$resn_exch{"1H80 CDL"}="H801 CDL"; $resn_exch{"2H81 CDL"}="H812 CDL"; $resn_exch{"1H81 CDL"}="H811 CDL";
$resn_exch{"2H82 CDL"}="H822 CDL"; $resn_exch{"1H82 CDL"}="H821 CDL"; $resn_exch{"2H83 CDL"}="H832 CDL";
$resn_exch{"1H83 CDL"}="H831 CDL"; $resn_exch{"2H84 CDL"}="H842 CDL"; $resn_exch{"1H84 CDL"}="H841 CDL";
$resn_exch{"2H85 CDL"}="H852 CDL"; $resn_exch{"1H85 CDL"}="H851 CDL"; $resn_exch{"2H86 CDL"}="H862 CDL";
$resn_exch{"1H86 CDL"}="H861 CDL"; $resn_exch{"3H87 CDL"}="H873 CDL"; $resn_exch{"2H87 CDL"}="H872 CDL";
$resn_exch{"1H87 CDL"}="H871 CDL"; $resn_exch{"1H4  CDM"}=" H41 CDM"; $resn_exch{"2H4  CDM"}=" H42 CDM";
$resn_exch{" H1* CDM"}=" H1' CDM"; $resn_exch{" H2* CDM"}=" H2' CDM"; $resn_exch{"2HO* CDM"}="HO'2 CDM";
$resn_exch{" H3* CDM"}=" H3' CDM"; $resn_exch{" H4* CDM"}=" H4' CDM"; $resn_exch{"1H5* CDM"}="H5'1 CDM";
$resn_exch{"2H5* CDM"}="H5'2 CDM"; $resn_exch{"1H1M CDM"}="H1M1 CDM"; $resn_exch{"2H1M CDM"}="H1M2 CDM";
$resn_exch{"1H2M CDM"}="H2M1 CDM"; $resn_exch{"2H2M CDM"}="H2M2 CDM"; $resn_exch{"1H4M CDM"}="H4M1 CDM";
$resn_exch{"2H4M CDM"}="H4M2 CDM"; $resn_exch{"1H5M CDM"}="H5M1 CDM"; $resn_exch{"2H5M CDM"}="H5M2 CDM";
$resn_exch{"3H5M CDM"}="H5M3 CDM"; $resn_exch{"1HO1 CDN"}="HO11 CDN"; $resn_exch{"1HB3 CDN"}="HB31 CDN";
$resn_exch{"1HB2 CDN"}="HB21 CDN"; $resn_exch{"2HB2 CDN"}="HB22 CDN"; $resn_exch{"1HC1 CDN"}="HC11 CDN";
$resn_exch{"2HB3 CDN"}="HB32 CDN"; $resn_exch{"3HB3 CDN"}="HB33 CDN"; $resn_exch{"1HB4 CDN"}="HB41 CDN";
$resn_exch{"1HB6 CDN"}="HB61 CDN"; $resn_exch{"2HB6 CDN"}="HB62 CDN"; $resn_exch{"1HB9 CDN"}="HB91 CDN";
$resn_exch{"1HB7 CDN"}="HB71 CDN"; $resn_exch{"1H71 CDN"}="H711 CDN"; $resn_exch{"2H71 CDN"}="H712 CDN";
$resn_exch{"1H72 CDN"}="H721 CDN"; $resn_exch{"2H72 CDN"}="H722 CDN"; $resn_exch{"1H73 CDN"}="H731 CDN";
$resn_exch{"2H73 CDN"}="H732 CDN"; $resn_exch{"1H74 CDN"}="H741 CDN"; $resn_exch{"2H74 CDN"}="H742 CDN";
$resn_exch{"1H75 CDN"}="H751 CDN"; $resn_exch{"2H75 CDN"}="H752 CDN"; $resn_exch{"3H75 CDN"}="H753 CDN";
$resn_exch{"OHB7 CDN"}="HB7O CDN"; $resn_exch{"1HB5 CDN"}="HB51 CDN"; $resn_exch{"1H51 CDN"}="H511 CDN";
$resn_exch{"2H51 CDN"}="H512 CDN"; $resn_exch{"1H52 CDN"}="H521 CDN"; $resn_exch{"2H52 CDN"}="H522 CDN";
$resn_exch{"1H53 CDN"}="H531 CDN"; $resn_exch{"2H53 CDN"}="H532 CDN"; $resn_exch{"1H54 CDN"}="H541 CDN";
$resn_exch{"2H54 CDN"}="H542 CDN"; $resn_exch{"1H55 CDN"}="H551 CDN"; $resn_exch{"2H55 CDN"}="H552 CDN";
$resn_exch{"1H56 CDN"}="H561 CDN"; $resn_exch{"2H56 CDN"}="H562 CDN"; $resn_exch{"1H57 CDN"}="H571 CDN";
$resn_exch{"2H57 CDN"}="H572 CDN"; $resn_exch{"1H58 CDN"}="H581 CDN"; $resn_exch{"2H58 CDN"}="H582 CDN";
$resn_exch{"1H59 CDN"}="H591 CDN"; $resn_exch{"2H59 CDN"}="H592 CDN"; $resn_exch{"1H60 CDN"}="H601 CDN";
$resn_exch{"2H60 CDN"}="H602 CDN"; $resn_exch{"1H61 CDN"}="H611 CDN"; $resn_exch{"2H61 CDN"}="H612 CDN";
$resn_exch{"1H62 CDN"}="H621 CDN"; $resn_exch{"2H62 CDN"}="H622 CDN"; $resn_exch{"1H63 CDN"}="H631 CDN";
$resn_exch{"2H63 CDN"}="H632 CDN"; $resn_exch{"1H64 CDN"}="H641 CDN"; $resn_exch{"2H64 CDN"}="H642 CDN";
$resn_exch{"3H64 CDN"}="H643 CDN"; $resn_exch{"OHA3 CDN"}="HA3O CDN"; $resn_exch{"1HA2 CDN"}="HA21 CDN";
$resn_exch{"2HA2 CDN"}="HA22 CDN"; $resn_exch{"1HA3 CDN"}="HA31 CDN"; $resn_exch{"2HA3 CDN"}="HA32 CDN";
$resn_exch{"1HA4 CDN"}="HA41 CDN"; $resn_exch{"1HA6 CDN"}="HA61 CDN"; $resn_exch{"2HA6 CDN"}="HA62 CDN";
$resn_exch{"1HA9 CDN"}="HA91 CDN"; $resn_exch{"1HA7 CDN"}="HA71 CDN"; $resn_exch{"1H31 CDN"}="H311 CDN";
$resn_exch{"2H31 CDN"}="H312 CDN"; $resn_exch{"1H32 CDN"}="H321 CDN"; $resn_exch{"2H32 CDN"}="H322 CDN";
$resn_exch{"1H33 CDN"}="H331 CDN"; $resn_exch{"2H33 CDN"}="H332 CDN"; $resn_exch{"1H34 CDN"}="H341 CDN";
$resn_exch{"2H34 CDN"}="H342 CDN"; $resn_exch{"1H35 CDN"}="H351 CDN"; $resn_exch{"2H35 CDN"}="H352 CDN";
$resn_exch{"1H36 CDN"}="H361 CDN"; $resn_exch{"2H36 CDN"}="H362 CDN"; $resn_exch{"1H37 CDN"}="H371 CDN";
$resn_exch{"2H37 CDN"}="H372 CDN"; $resn_exch{"1H38 CDN"}="H381 CDN"; $resn_exch{"2H38 CDN"}="H382 CDN";
$resn_exch{"1H39 CDN"}="H391 CDN"; $resn_exch{"2H39 CDN"}="H392 CDN"; $resn_exch{"1H40 CDN"}="H401 CDN";
$resn_exch{"2H40 CDN"}="H402 CDN"; $resn_exch{"1H41 CDN"}="H411 CDN"; $resn_exch{"2H41 CDN"}="H412 CDN";
$resn_exch{"1H42 CDN"}="H421 CDN"; $resn_exch{"2H42 CDN"}="H422 CDN"; $resn_exch{"3H42 CDN"}="H423 CDN";
$resn_exch{"OHA7 CDN"}="HA7O CDN"; $resn_exch{"1HA5 CDN"}="HA51 CDN"; $resn_exch{"1H11 CDN"}="H111 CDN";
$resn_exch{"2H11 CDN"}="H112 CDN"; $resn_exch{"1H12 CDN"}="H121 CDN"; $resn_exch{"2H12 CDN"}="H122 CDN";
$resn_exch{"1H13 CDN"}="H131 CDN"; $resn_exch{"2H13 CDN"}="H132 CDN"; $resn_exch{"1H14 CDN"}="H141 CDN";
$resn_exch{"2H14 CDN"}="H142 CDN"; $resn_exch{"1H15 CDN"}="H151 CDN"; $resn_exch{"2H15 CDN"}="H152 CDN";
$resn_exch{"1H16 CDN"}="H161 CDN"; $resn_exch{"2H16 CDN"}="H162 CDN"; $resn_exch{"1H17 CDN"}="H171 CDN";
$resn_exch{"2H17 CDN"}="H172 CDN"; $resn_exch{"1H18 CDN"}="H181 CDN"; $resn_exch{"2H18 CDN"}="H182 CDN";
$resn_exch{"1H19 CDN"}="H191 CDN"; $resn_exch{"2H19 CDN"}="H192 CDN"; $resn_exch{"1H20 CDN"}="H201 CDN";
$resn_exch{"2H20 CDN"}="H202 CDN"; $resn_exch{"1H21 CDN"}="H211 CDN"; $resn_exch{"2H21 CDN"}="H212 CDN";
$resn_exch{"1H22 CDN"}="H221 CDN"; $resn_exch{"2H22 CDN"}="H222 CDN"; $resn_exch{"1H23 CDN"}="H231 CDN";
$resn_exch{"2H23 CDN"}="H232 CDN"; $resn_exch{"1H24 CDN"}="H241 CDN"; $resn_exch{"2H24 CDN"}="H242 CDN";
$resn_exch{"3H24 CDN"}="H243 CDN"; $resn_exch{"1H7  CDO"}=" H71 CDO"; $resn_exch{"2H7  CDO"}=" H72 CDO";
$resn_exch{"1H10 CDO"}="H101 CDO"; $resn_exch{"2H10 CDO"}="H102 CDO"; $resn_exch{"1H17 CDO"}="H171 CDO";
$resn_exch{"2H17 CDO"}="H172 CDO"; $resn_exch{"1H23 CDO"}="H231 CDO"; $resn_exch{"2H23 CDO"}="H232 CDO";
$resn_exch{"1H27 CDO"}="H271 CDO"; $resn_exch{"2H27 CDO"}="H272 CDO"; $resn_exch{"1H29 CDO"}="H291 CDO";
$resn_exch{"2H29 CDO"}="H292 CDO"; $resn_exch{"1H30 CDO"}="H301 CDO"; $resn_exch{"2H30 CDO"}="H302 CDO";
$resn_exch{"1H31 CDO"}="H311 CDO"; $resn_exch{"2H31 CDO"}="H312 CDO"; $resn_exch{"1H32 CDO"}="H321 CDO";
$resn_exch{"2H32 CDO"}="H322 CDO"; $resn_exch{"1H33 CDO"}="H331 CDO"; $resn_exch{"2H33 CDO"}="H332 CDO";
$resn_exch{"1H4  CDP"}=" H41 CDP"; $resn_exch{"2H4  CDP"}=" H42 CDP"; $resn_exch{" H1* CDP"}=" H1' CDP";
$resn_exch{" H2* CDP"}=" H2' CDP"; $resn_exch{"2HO* CDP"}="HO'2 CDP"; $resn_exch{" H3* CDP"}=" H3' CDP";
$resn_exch{" H4* CDP"}=" H4' CDP"; $resn_exch{"1H5* CDP"}="H5'1 CDP"; $resn_exch{"2H5* CDP"}="H5'2 CDP";
$resn_exch{"1H2  CDR"}=" H21 CDR"; $resn_exch{"2H2  CDR"}=" H22 CDR"; $resn_exch{"1H3  CDR"}=" H31 CDR";
$resn_exch{"2H3  CDR"}=" H32 CDR"; $resn_exch{"1H6  CDR"}=" H61 CDR"; $resn_exch{"2H6  CDR"}=" H62 CDR";
$resn_exch{"3H6  CDR"}=" H63 CDR"; $resn_exch{"1H33 CDS"}="H331 CDS"; $resn_exch{"2H33 CDS"}="H332 CDS";
$resn_exch{"1H27 CDS"}="H271 CDS"; $resn_exch{"2H27 CDS"}="H272 CDS"; $resn_exch{"3H27 CDS"}="H273 CDS";
$resn_exch{"1H2  CDS"}=" H21 CDS"; $resn_exch{"2H2  CDS"}=" H22 CDS"; $resn_exch{"1H20 CDS"}="H201 CDS";
$resn_exch{"2H20 CDS"}="H202 CDS"; $resn_exch{"3H20 CDS"}="H203 CDS"; $resn_exch{"1HT  CDT"}=" HT1 CDT";
$resn_exch{"2HT  CDT"}=" HT2 CDT"; $resn_exch{"1HN  CDT"}=" HN1 CDT"; $resn_exch{"1HB  CDT"}=" HB1 CDT";
$resn_exch{"2HB  CDT"}=" HB2 CDT"; $resn_exch{"1HG  CDT"}=" HG1 CDT"; $resn_exch{"2HG  CDT"}=" HG2 CDT";
$resn_exch{"1HE  CDT"}=" HE1 CDT"; $resn_exch{"2HE  CDT"}=" HE2 CDT"; $resn_exch{"3HE  CDT"}=" HE3 CDT";
$resn_exch{"1HC1 CDU"}="HC11 CDU"; $resn_exch{"2HC1 CDU"}="HC12 CDU"; $resn_exch{"1HC2 CDU"}="HC21 CDU";
$resn_exch{"2HC2 CDU"}="HC22 CDU"; $resn_exch{"1HC3 CDU"}="HC31 CDU"; $resn_exch{"2HC3 CDU"}="HC32 CDU";
$resn_exch{"1HC4 CDU"}="HC41 CDU"; $resn_exch{"2HC4 CDU"}="HC42 CDU"; $resn_exch{"1HC5 CDU"}="HC51 CDU";
$resn_exch{"1HC6 CDU"}="HC61 CDU"; $resn_exch{"2HC6 CDU"}="HC62 CDU"; $resn_exch{"1H11 CDU"}="H111 CDU";
$resn_exch{"2H11 CDU"}="H112 CDU"; $resn_exch{"1H12 CDU"}="H121 CDU"; $resn_exch{"2H12 CDU"}="H122 CDU";
$resn_exch{"1H13 CDU"}="H131 CDU"; $resn_exch{"2H13 CDU"}="H132 CDU"; $resn_exch{"1H14 CDU"}="H141 CDU";
$resn_exch{"2H14 CDU"}="H142 CDU"; $resn_exch{"1H15 CDU"}="H151 CDU"; $resn_exch{"2H15 CDU"}="H152 CDU";
$resn_exch{"1H16 CDU"}="H161 CDU"; $resn_exch{"2H16 CDU"}="H162 CDU"; $resn_exch{"1H17 CDU"}="H171 CDU";
$resn_exch{"2H17 CDU"}="H172 CDU"; $resn_exch{"1H18 CDU"}="H181 CDU"; $resn_exch{"2H18 CDU"}="H182 CDU";
$resn_exch{"1H19 CDU"}="H191 CDU"; $resn_exch{"2H19 CDU"}="H192 CDU"; $resn_exch{"1H20 CDU"}="H201 CDU";
$resn_exch{"2H20 CDU"}="H202 CDU"; $resn_exch{"3H20 CDU"}="H203 CDU"; $resn_exch{"1HT  CDV"}=" HT1 CDV";
$resn_exch{"2HT  CDV"}=" HT2 CDV"; $resn_exch{"1HN  CDV"}=" HN1 CDV"; $resn_exch{"1HG1 CDV"}="HG11 CDV";
$resn_exch{"2HG1 CDV"}="HG12 CDV"; $resn_exch{"3HG1 CDV"}="HG13 CDV"; $resn_exch{"1HG2 CDV"}="HG21 CDV";
$resn_exch{"2HG2 CDV"}="HG22 CDV"; $resn_exch{"3HG2 CDV"}="HG23 CDV"; $resn_exch{"1HAD CDX"}="HAD1 CDX";
$resn_exch{"2HAD CDX"}="HAD2 CDX"; $resn_exch{"1HAQ CDX"}="HAQ1 CDX"; $resn_exch{"2HAQ CDX"}="HAQ2 CDX";
$resn_exch{"1HAK CDX"}="HAK1 CDX"; $resn_exch{"2HAK CDX"}="HAK2 CDX"; $resn_exch{"1HAE CDX"}="HAE1 CDX";
$resn_exch{"2HAE CDX"}="HAE2 CDX"; $resn_exch{"3HAE CDX"}="HAE3 CDX"; $resn_exch{"1HAR CDX"}="HAR1 CDX";
$resn_exch{"2HAR CDX"}="HAR2 CDX"; $resn_exch{"1HAF CDX"}="HAF1 CDX"; $resn_exch{"2HAF CDX"}="HAF2 CDX";
$resn_exch{"1HC5 CDY"}="HC51 CDY"; $resn_exch{"2HC5 CDY"}="HC52 CDY"; $resn_exch{" H4* CDY"}=" H4' CDY";
$resn_exch{"1HC3 CDY"}="HC31 CDY"; $resn_exch{"2HC3 CDY"}="HC32 CDY"; $resn_exch{"1HC2 CDY"}="HC21 CDY";
$resn_exch{"2HC2 CDY"}="HC22 CDY"; $resn_exch{" H1* CDY"}=" H1' CDY"; $resn_exch{"1HN6 CDY"}="HN61 CDY";
$resn_exch{"2HN6 CDY"}="HN62 CDY"; $resn_exch{"1H1  CE1"}=" H11 CE1"; $resn_exch{"2H1  CE1"}=" H12 CE1";
$resn_exch{"3H1  CE1"}=" H13 CE1"; $resn_exch{"1H2  CE1"}=" H21 CE1"; $resn_exch{"2H2  CE1"}=" H22 CE1";
$resn_exch{"1H3  CE1"}=" H31 CE1"; $resn_exch{"2H3  CE1"}=" H32 CE1"; $resn_exch{"1H4  CE1"}=" H41 CE1";
$resn_exch{"2H4  CE1"}=" H42 CE1"; $resn_exch{"1H5  CE1"}=" H51 CE1"; $resn_exch{"2H5  CE1"}=" H52 CE1";
$resn_exch{"1H6  CE1"}=" H61 CE1"; $resn_exch{"2H6  CE1"}=" H62 CE1"; $resn_exch{"1H7  CE1"}=" H71 CE1";
$resn_exch{"2H7  CE1"}=" H72 CE1"; $resn_exch{"1H8  CE1"}=" H81 CE1"; $resn_exch{"2H8  CE1"}=" H82 CE1";
$resn_exch{"1H9  CE1"}=" H91 CE1"; $resn_exch{"2H9  CE1"}=" H92 CE1"; $resn_exch{"1H10 CE1"}="H101 CE1";
$resn_exch{"2H10 CE1"}="H102 CE1"; $resn_exch{"1H11 CE1"}="H111 CE1"; $resn_exch{"2H11 CE1"}="H112 CE1";
$resn_exch{"1H12 CE1"}="H121 CE1"; $resn_exch{"2H12 CE1"}="H122 CE1"; $resn_exch{"1H14 CE1"}="H141 CE1";
$resn_exch{"2H14 CE1"}="H142 CE1"; $resn_exch{"1H15 CE1"}="H151 CE1"; $resn_exch{"2H15 CE1"}="H152 CE1";
$resn_exch{"1H17 CE1"}="H171 CE1"; $resn_exch{"2H17 CE1"}="H172 CE1"; $resn_exch{"1H18 CE1"}="H181 CE1";
$resn_exch{"2H18 CE1"}="H182 CE1"; $resn_exch{"1H20 CE1"}="H201 CE1"; $resn_exch{"2H20 CE1"}="H202 CE1";
$resn_exch{"1H21 CE1"}="H211 CE1"; $resn_exch{"2H21 CE1"}="H212 CE1"; $resn_exch{"1H23 CE1"}="H231 CE1";
$resn_exch{"2H23 CE1"}="H232 CE1"; $resn_exch{"1H24 CE1"}="H241 CE1"; $resn_exch{"2H24 CE1"}="H242 CE1";
$resn_exch{"1H26 CE1"}="H261 CE1"; $resn_exch{"2H26 CE1"}="H262 CE1"; $resn_exch{"1H27 CE1"}="H271 CE1";
$resn_exch{"2H27 CE1"}="H272 CE1"; $resn_exch{"1H29 CE1"}="H291 CE1"; $resn_exch{"2H29 CE1"}="H292 CE1";
$resn_exch{"1H30 CE1"}="H301 CE1"; $resn_exch{"2H30 CE1"}="H302 CE1"; $resn_exch{"1H32 CE1"}="H321 CE1";
$resn_exch{"2H32 CE1"}="H322 CE1"; $resn_exch{"1H33 CE1"}="H331 CE1"; $resn_exch{"2H33 CE1"}="H332 CE1";
$resn_exch{"1H35 CE1"}="H351 CE1"; $resn_exch{"2H35 CE1"}="H352 CE1"; $resn_exch{"1H36 CE1"}="H361 CE1";
$resn_exch{"2H36 CE1"}="H362 CE1"; $resn_exch{"1H12 CE2"}="H121 CE2"; $resn_exch{"2H12 CE2"}="H122 CE2";
$resn_exch{"3H12 CE2"}="H123 CE2"; $resn_exch{"1H6  CE2"}=" H61 CE2"; $resn_exch{"2H6  CE2"}=" H62 CE2";
$resn_exch{"1H24 CE2"}="H241 CE2"; $resn_exch{"2H24 CE2"}="H242 CE2"; $resn_exch{"3H24 CE2"}="H243 CE2";
$resn_exch{"1H13 CE2"}="H131 CE2"; $resn_exch{"2H13 CE2"}="H132 CE2"; $resn_exch{"3H13 CE2"}="H133 CE2";
$resn_exch{"1H14 CE2"}="H141 CE2"; $resn_exch{"2H14 CE2"}="H142 CE2"; $resn_exch{"3H14 CE2"}="H143 CE2";
$resn_exch{"1H2  CE3"}=" H21 CE3"; $resn_exch{"2H2  CE3"}=" H22 CE3"; $resn_exch{"1H8  CE3"}=" H81 CE3";
$resn_exch{"2H8  CE3"}=" H82 CE3"; $resn_exch{"3H8  CE3"}=" H83 CE3"; $resn_exch{"1H4  CE3"}=" H41 CE3";
$resn_exch{"2H4  CE3"}=" H42 CE3"; $resn_exch{"1H9  CE3"}=" H91 CE3"; $resn_exch{"2H9  CE3"}=" H92 CE3";
$resn_exch{"1H11 CE3"}="H111 CE3"; $resn_exch{"2H11 CE3"}="H112 CE3"; $resn_exch{"3H11 CE3"}="H113 CE3";
$resn_exch{"1H6E CE5"}="H6E1 CE5"; $resn_exch{"2H6E CE5"}="H6E2 CE5"; $resn_exch{"1H6D CE5"}="H6D1 CE5";
$resn_exch{"2H6D CE5"}="H6D2 CE5"; $resn_exch{"1H6C CE5"}="H6C1 CE5"; $resn_exch{"2H6C CE5"}="H6C2 CE5";
$resn_exch{"1H6B CE5"}="H6B1 CE5"; $resn_exch{"2H6B CE5"}="H6B2 CE5"; $resn_exch{"1H6A CE5"}="H6A1 CE5";
$resn_exch{"2H6A CE5"}="H6A2 CE5"; $resn_exch{"1H6A CE6"}="H6A1 CE6"; $resn_exch{"2H6A CE6"}="H6A2 CE6";
$resn_exch{"1H6B CE6"}="H6B1 CE6"; $resn_exch{"2H6B CE6"}="H6B2 CE6"; $resn_exch{"1H6C CE6"}="H6C1 CE6";
$resn_exch{"2H6C CE6"}="H6C2 CE6"; $resn_exch{"1H6D CE6"}="H6D1 CE6"; $resn_exch{"2H6D CE6"}="H6D2 CE6";
$resn_exch{"1H6F CE6"}="H6F1 CE6"; $resn_exch{"2H6F CE6"}="H6F2 CE6"; $resn_exch{"1H6E CE6"}="H6E1 CE6";
$resn_exch{"2H6E CE6"}="H6E2 CE6"; $resn_exch{"1H6H CE8"}="H6H1 CE8"; $resn_exch{"2H6H CE8"}="H6H2 CE8";
$resn_exch{"1H6G CE8"}="H6G1 CE8"; $resn_exch{"2H6G CE8"}="H6G2 CE8"; $resn_exch{"1H6F CE8"}="H6F1 CE8";
$resn_exch{"2H6F CE8"}="H6F2 CE8"; $resn_exch{"1H6E CE8"}="H6E1 CE8"; $resn_exch{"2H6E CE8"}="H6E2 CE8";
$resn_exch{"1H6D CE8"}="H6D1 CE8"; $resn_exch{"2H6D CE8"}="H6D2 CE8"; $resn_exch{"1H6C CE8"}="H6C1 CE8";
$resn_exch{"2H6C CE8"}="H6C2 CE8"; $resn_exch{"1H6B CE8"}="H6B1 CE8"; $resn_exch{"2H6B CE8"}="H6B2 CE8";
$resn_exch{"1H6A CE8"}="H6A1 CE8"; $resn_exch{"2H6A CE8"}="H6A2 CE8"; $resn_exch{"1HN  CEA"}=" HN1 CEA";
$resn_exch{"2HN  CEA"}=" HN2 CEA"; $resn_exch{"1HB  CEA"}=" HB1 CEA"; $resn_exch{"2HB  CEA"}=" HB2 CEA";
$resn_exch{"1H10 CEB"}="H101 CEB"; $resn_exch{"2H10 CEB"}="H102 CEB"; $resn_exch{"1H11 CEB"}="H111 CEB";
$resn_exch{"2H11 CEB"}="H112 CEB"; $resn_exch{"3H11 CEB"}="H113 CEB"; $resn_exch{"1H12 CEB"}="H121 CEB";
$resn_exch{"2H12 CEB"}="H122 CEB"; $resn_exch{"3H12 CEB"}="H123 CEB"; $resn_exch{"1H14 CEB"}="H141 CEB";
$resn_exch{"2H14 CEB"}="H142 CEB"; $resn_exch{"1H15 CEB"}="H151 CEB"; $resn_exch{"2H15 CEB"}="H152 CEB";
$resn_exch{"3H15 CEB"}="H153 CEB"; $resn_exch{"1H1  CEC"}=" H11 CEC"; $resn_exch{"1H2  CEC"}=" H21 CEC";
$resn_exch{"2H2  CEC"}=" H22 CEC"; $resn_exch{"1H3  CEC"}=" H31 CEC"; $resn_exch{"2H3  CEC"}=" H32 CEC";
$resn_exch{"1H2  CED"}=" H21 CED"; $resn_exch{"2H2  CED"}=" H22 CED"; $resn_exch{"1H3' CED"}="H3'1 CED";
$resn_exch{"2H3' CED"}="H3'2 CED"; $resn_exch{"3H3' CED"}="H3'3 CED"; $resn_exch{"1H13 CED"}="H131 CED";
$resn_exch{"2H13 CED"}="H132 CED"; $resn_exch{"1H2  CEF"}=" H21 CEF"; $resn_exch{"2H2  CEF"}=" H22 CEF";
$resn_exch{"1H3' CEF"}="H3'1 CEF"; $resn_exch{"2H3' CEF"}="H3'2 CEF"; $resn_exch{"1H1' CEF"}="H1'1 CEF";
$resn_exch{"2H1' CEF"}="H1'2 CEF"; $resn_exch{"3H1' CEF"}="H1'3 CEF"; $resn_exch{"8H8  CEF"}=" H88 CEF";
$resn_exch{"1H18 CEF"}="H181 CEF"; $resn_exch{"2H18 CEF"}="H182 CEF"; $resn_exch{"3H18 CEF"}="H183 CEF";
$resn_exch{"1HN8 CEF"}="HN81 CEF"; $resn_exch{"2HN8 CEF"}="HN82 CEF"; $resn_exch{"8HOB CEG"}="HOB8 CEG";
$resn_exch{"1H1  CEH"}=" H11 CEH"; $resn_exch{"2H1  CEH"}="H12A CEH"; $resn_exch{"1H5  CEH"}=" H51 CEH";
$resn_exch{"2H5  CEH"}=" H52 CEH"; $resn_exch{"1H10 CEH"}="H101 CEH"; $resn_exch{"2H10 CEH"}="H102 CEH";
$resn_exch{"1H17 CEH"}="H171 CEH"; $resn_exch{"2H17 CEH"}="H172 CEH"; $resn_exch{"1H20 CEH"}="H201 CEH";
$resn_exch{"2H20 CEH"}="H202 CEH"; $resn_exch{"1H25 CEH"}="H251 CEH"; $resn_exch{"2H25 CEH"}="H252 CEH";
$resn_exch{"3H25 CEH"}="H253 CEH"; $resn_exch{"1H28 CEH"}="H281 CEH"; $resn_exch{"2H28 CEH"}="H282 CEH";
$resn_exch{"3H28 CEH"}="H283 CEH"; $resn_exch{"1H21 CEH"}="H211 CEH"; $resn_exch{"2H21 CEH"}="H212 CEH";
$resn_exch{"1H32 CEH"}="H321 CEH"; $resn_exch{"2H32 CEH"}="H322 CEH"; $resn_exch{"3H32 CEH"}="H323 CEH";
$resn_exch{"1H10 CEI"}="H101 CEI"; $resn_exch{"2H10 CEI"}="H102 CEI"; $resn_exch{"1H26 CEI"}="H261 CEI";
$resn_exch{"2H26 CEI"}="H262 CEI"; $resn_exch{"1H1C CEJ"}="H1C1 CEJ"; $resn_exch{"2H1C CEJ"}="H1C2 CEJ";
$resn_exch{"1H5C CEJ"}="H5C1 CEJ"; $resn_exch{"2H5C CEJ"}="H5C2 CEJ"; $resn_exch{"1H10 CEJ"}="H101 CEJ";
$resn_exch{"2H10 CEJ"}="H102 CEJ"; $resn_exch{"CH14 CEL"}="H14C CEL"; $resn_exch{"CH13 CEL"}="H13C CEL";
$resn_exch{"CH17 CEL"}="H17C CEL"; $resn_exch{"CH16 CEL"}="H16C CEL"; $resn_exch{"CH10 CEL"}="H10C CEL";
$resn_exch{"1H11 CEL"}="H111 CEL"; $resn_exch{"2H11 CEL"}="H112 CEL"; $resn_exch{"3H11 CEL"}="H113 CEL";
$resn_exch{"1HN3 CEL"}="HN31 CEL"; $resn_exch{"2HN3 CEL"}="HN32 CEL"; $resn_exch{"1H9  CEM"}=" H91 CEM";
$resn_exch{"2H9  CEM"}=" H92 CEM"; $resn_exch{"4HO1 CEM"}="HO14 CEM"; $resn_exch{"1HC2 CEN"}="HC21 CEN";
$resn_exch{"2HC2 CEN"}="HC22 CEN"; $resn_exch{"1H13 CEN"}="H131 CEN"; $resn_exch{"2H13 CEN"}="H132 CEN";
$resn_exch{"1H17 CEN"}="H171 CEN"; $resn_exch{"2H17 CEN"}="H172 CEN"; $resn_exch{"1HC  CEN"}=" HC1 CEN";
$resn_exch{"2HC  CEN"}=" HC2 CEN"; $resn_exch{"3HC  CEN"}=" HC3 CEN"; $resn_exch{"1HC5 CEN"}="HC51 CEN";
$resn_exch{"2HC5 CEN"}="HC52 CEN"; $resn_exch{"1HC9 CEN"}="HC91 CEN"; $resn_exch{"2HC9 CEN"}="HC92 CEN";
$resn_exch{"1HC2 CEO"}="HC21 CEO"; $resn_exch{"2HC2 CEO"}="HC22 CEO"; $resn_exch{"1H3' CEO"}="H3'1 CEO";
$resn_exch{"2H3' CEO"}="H3'2 CEO"; $resn_exch{"1H13 CEO"}="H131 CEO"; $resn_exch{"2H13 CEO"}="H132 CEO";
$resn_exch{"1H2  CEP"}=" H21 CEP"; $resn_exch{"2H2  CEP"}=" H22 CEP"; $resn_exch{"1H3' CEP"}="H3'1 CEP";
$resn_exch{"2H3' CEP"}="H3'2 CEP"; $resn_exch{"1H1' CEP"}="H1'1 CEP"; $resn_exch{"2H1' CEP"}="H1'2 CEP";
$resn_exch{"3H1' CEP"}="H1'3 CEP"; $resn_exch{"8H8  CEP"}=" H88 CEP"; $resn_exch{"1H13 CEP"}="H131 CEP";
$resn_exch{"2H13 CEP"}="H132 CEP"; $resn_exch{"1H2  CEQ"}=" H21 CEQ"; $resn_exch{"2H2  CEQ"}=" H22 CEQ";
$resn_exch{"3H2  CEQ"}=" H23 CEQ"; $resn_exch{"1H3  CEQ"}=" H31 CEQ"; $resn_exch{"2H3  CEQ"}=" H32 CEQ";
$resn_exch{"3H3  CEQ"}=" H33 CEQ"; $resn_exch{"1H4  CEQ"}=" H41 CEQ"; $resn_exch{"2H4  CEQ"}=" H42 CEQ";
$resn_exch{"3H4  CEQ"}=" H43 CEQ"; $resn_exch{"1H5  CEQ"}=" H51 CEQ"; $resn_exch{"2H5  CEQ"}=" H52 CEQ";
$resn_exch{"1H6  CEQ"}=" H61 CEQ"; $resn_exch{"2H6  CEQ"}=" H62 CEQ"; $resn_exch{"3H6  CEQ"}=" H63 CEQ";
$resn_exch{"1HN1 CER"}="HN11 CER"; $resn_exch{"2HN1 CER"}="HN12 CER"; $resn_exch{"1H2  CER"}=" H21 CER";
$resn_exch{"2H2  CER"}=" H22 CER"; $resn_exch{"1H3  CER"}=" H31 CER"; $resn_exch{"1H5  CER"}=" H51 CER";
$resn_exch{"2H5  CER"}=" H52 CER"; $resn_exch{"1H6  CER"}=" H61 CER"; $resn_exch{"2H6  CER"}=" H62 CER";
$resn_exch{"1H7  CER"}=" H71 CER"; $resn_exch{"1H8  CER"}=" H81 CER"; $resn_exch{"1H9  CER"}=" H91 CER";
$resn_exch{"2H9  CER"}=" H92 CER"; $resn_exch{"1H10 CER"}="H101 CER"; $resn_exch{"1H11 CER"}="H111 CER";
$resn_exch{"1H12 CER"}="H121 CER"; $resn_exch{"2H12 CER"}="H122 CER"; $resn_exch{"3H12 CER"}="H123 CER";
$resn_exch{"3H18 CES"}="H183 CES"; $resn_exch{"2H18 CES"}="H182 CES"; $resn_exch{"1H18 CES"}="H181 CES";
$resn_exch{"3H3' CES"}="H3'3 CES"; $resn_exch{"2H3' CES"}="H3'2 CES"; $resn_exch{"1H3' CES"}="H3'1 CES";
$resn_exch{"1H2  CES"}=" H21 CES"; $resn_exch{"2H2  CES"}=" H22 CES"; $resn_exch{"1H10 CET"}="H101 CET";
$resn_exch{"2H10 CET"}="H102 CET"; $resn_exch{"1H11 CET"}="H111 CET"; $resn_exch{"2H11 CET"}="H112 CET";
$resn_exch{"3H11 CET"}="H113 CET"; $resn_exch{"1H12 CET"}="H121 CET"; $resn_exch{"2H12 CET"}="H122 CET";
$resn_exch{"3H12 CET"}="H123 CET"; $resn_exch{"1H14 CET"}="H141 CET"; $resn_exch{"2H14 CET"}="H142 CET";
$resn_exch{"1H15 CET"}="H151 CET"; $resn_exch{"2H15 CET"}="H152 CET"; $resn_exch{"3H15 CET"}="H153 CET";
$resn_exch{"1H10 CF3"}="H101 CF3"; $resn_exch{"2H10 CF3"}="H102 CF3"; $resn_exch{"1H8  CF3"}=" H81 CF3";
$resn_exch{"2H8  CF3"}=" H82 CF3"; $resn_exch{"1H6  CF3"}=" H61 CF3"; $resn_exch{"2H6  CF3"}=" H62 CF3";
$resn_exch{"1H7  CF3"}=" H71 CF3"; $resn_exch{"2H7  CF3"}=" H72 CF3"; $resn_exch{"1H9  CF3"}=" H91 CF3";
$resn_exch{"2H9  CF3"}=" H92 CF3"; $resn_exch{"1H11 CF3"}="H111 CF3"; $resn_exch{"2H11 CF3"}="H112 CF3";
$resn_exch{"1H5S CF5"}="H5S1 CF5"; $resn_exch{"2H5S CF5"}="H5S2 CF5"; $resn_exch{"1H7  CF5"}=" H71 CF5";
$resn_exch{"2H7  CF5"}=" H72 CF5"; $resn_exch{"1H2C CFA"}="H2C1 CFA"; $resn_exch{"2H2C CFA"}="H2C2 CFA";
$resn_exch{"1H5' CFB"}="H5'1 CFB"; $resn_exch{"2H5' CFB"}="H5'2 CFB"; $resn_exch{"1HN6 CFB"}="HN61 CFB";
$resn_exch{"2HN6 CFB"}="HN62 CFB"; $resn_exch{"1H12 CFC"}="H121 CFC"; $resn_exch{"2H12 CFC"}="H122 CFC";
$resn_exch{"3H12 CFC"}="H123 CFC"; $resn_exch{"1H13 CFC"}="H131 CFC"; $resn_exch{"2H13 CFC"}="H132 CFC";
$resn_exch{"3H13 CFC"}="H133 CFC"; $resn_exch{"1H10 CFF"}="H101 CFF"; $resn_exch{"2H10 CFF"}="H102 CFF";
$resn_exch{"3H10 CFF"}="H103 CFF"; $resn_exch{"1H12 CFF"}="H121 CFF"; $resn_exch{"2H12 CFF"}="H122 CFF";
$resn_exch{"3H12 CFF"}="H123 CFF"; $resn_exch{"1H8  CFF"}=" H81 CFF"; $resn_exch{"1H14 CFF"}="H141 CFF";
$resn_exch{"2H14 CFF"}="H142 CFF"; $resn_exch{"3H14 CFF"}="H143 CFF"; $resn_exch{"1H5* CFL"}="H5'1 CFL";
$resn_exch{"2H5* CFL"}="H5'2 CFL"; $resn_exch{" H4* CFL"}=" H4' CFL"; $resn_exch{" H3* CFL"}=" H3' CFL";
$resn_exch{"HO3* CFL"}="HO3' CFL"; $resn_exch{" H2* CFL"}=" H2' CFL"; $resn_exch{" H1* CFL"}=" H1' CFL";
$resn_exch{"1HN4 CFL"}="HN41 CFL"; $resn_exch{"2HN4 CFL"}="HN42 CFL"; $resn_exch{"1H1C CFQ"}="H1C1 CFQ";
$resn_exch{"2H1C CFQ"}="H1C2 CFQ"; $resn_exch{"3H1C CFQ"}="H1C3 CFQ"; $resn_exch{"1H2C CFQ"}="H2C1 CFQ";
$resn_exch{"2H2C CFQ"}="H2C2 CFQ"; $resn_exch{"3H2C CFQ"}="H2C3 CFQ"; $resn_exch{"1H3C CFQ"}="H3C1 CFQ";
$resn_exch{"2H3C CFQ"}="H3C2 CFQ"; $resn_exch{"3H3C CFQ"}="H3C3 CFQ"; $resn_exch{"1H4C CFQ"}="H4C1 CFQ";
$resn_exch{"2H4C CFQ"}="H4C2 CFQ"; $resn_exch{"1H5C CFQ"}="H5C1 CFQ"; $resn_exch{"2H5C CFQ"}="H5C2 CFQ";
$resn_exch{"1HN  CFS"}=" HN1 CFS"; $resn_exch{"2HN  CFS"}=" HN2 CFS"; $resn_exch{"1HB  CFS"}=" HB1 CFS";
$resn_exch{"2HB  CFS"}=" HB2 CFS"; $resn_exch{"1HD1 CFS"}="HD11 CFS"; $resn_exch{"2HD1 CFS"}="HD12 CFS";
$resn_exch{"1HD2 CFS"}="HD21 CFS"; $resn_exch{"2HD2 CFS"}="HD22 CFS"; $resn_exch{"1HE1 CFS"}="HE11 CFS";
$resn_exch{"2HE1 CFS"}="HE12 CFS"; $resn_exch{"1HE2 CFS"}="HE21 CFS"; $resn_exch{"2HE2 CFS"}="HE22 CFS";
$resn_exch{"1HZ  CFS"}=" HZ1 CFS"; $resn_exch{"2HZ  CFS"}=" HZ2 CFS"; $resn_exch{"1HC9 CFX"}="HC91 CFX";
$resn_exch{"2HC9 CFX"}="HC92 CFX"; $resn_exch{"1H15 CFX"}="H151 CFX"; $resn_exch{"2H15 CFX"}="H152 CFX";
$resn_exch{"1H17 CFX"}="H171 CFX"; $resn_exch{"2H17 CFX"}="H172 CFX"; $resn_exch{"3H17 CFX"}="H173 CFX";
$resn_exch{"1H20 CFX"}="H201 CFX"; $resn_exch{"2H20 CFX"}="H202 CFX"; $resn_exch{"1H33 CFX"}="H331 CFX";
$resn_exch{"2H33 CFX"}="H332 CFX"; $resn_exch{"1H1   CG"}=" H11  CG"; $resn_exch{"1HN6  CG"}="HN61  CG";
$resn_exch{"1HN7  CG"}="HN71  CG"; $resn_exch{"2HN7  CG"}="HN72  CG"; $resn_exch{"1H8   CG"}=" H81  CG";
$resn_exch{"1H9   CG"}=" H91  CG"; $resn_exch{"1H10  CG"}="H101  CG"; $resn_exch{"2H10  CG"}="H102  CG";
$resn_exch{"1H11  CG"}="H111  CG"; $resn_exch{"2H11  CG"}="H112  CG"; $resn_exch{"1H14  CG"}="H141  CG";
$resn_exch{"1H16  CG"}="H161  CG"; $resn_exch{"1H17  CG"}="H171  CG"; $resn_exch{"2H17  CG"}="H172  CG";
$resn_exch{"1HNG CG2"}="HNG1 CG2"; $resn_exch{"1H2G CG2"}="H2G1 CG2"; $resn_exch{"2H2G CG2"}="H2G2 CG2";
$resn_exch{"1HC' CG2"}="HC'1 CG2"; $resn_exch{"2HC' CG2"}="HC'2 CG2"; $resn_exch{"1HCA CG2"}="HCA1 CG2";
$resn_exch{"2HCA CG2"}="HCA2 CG2"; $resn_exch{"1H4C CG2"}="H4C1 CG2"; $resn_exch{"2H4C CG2"}="H4C2 CG2";
$resn_exch{"1HN  CGA"}=" H   CGA"; $resn_exch{"2HN  CGA"}=" H2  CGA"; $resn_exch{"1HB  CGA"}=" HB1 CGA";
$resn_exch{"2HB  CGA"}=" HB2 CGA"; $resn_exch{"1HG  CGA"}=" HG1 CGA"; $resn_exch{"2HG  CGA"}=" HG2 CGA";
$resn_exch{"1H2  CGA"}=" H21 CGA"; $resn_exch{"2H2  CGA"}=" H22 CGA"; $resn_exch{"1H8C CGB"}="H8C1 CGB";
$resn_exch{"2H8C CGB"}="H8C2 CGB"; $resn_exch{"1H7C CGB"}="H7C1 CGB"; $resn_exch{"2H7C CGB"}="H7C2 CGB";
$resn_exch{"1HN7 CGF"}="HN71 CGF"; $resn_exch{"2HN7 CGF"}="HN72 CGF"; $resn_exch{"1H6  CGF"}=" H61 CGF";
$resn_exch{"2H6  CGF"}=" H62 CGF"; $resn_exch{"1HB  CGN"}=" HB1 CGN"; $resn_exch{"2HB  CGN"}=" HB2 CGN";
$resn_exch{"1HG  CGN"}=" HG1 CGN"; $resn_exch{"2HG  CGN"}=" HG2 CGN"; $resn_exch{"2HOP CGP"}="HOP2 CGP";
$resn_exch{"CHO5 CGP"}="HO5C CGP"; $resn_exch{"CH51 CGP"}="H51C CGP"; $resn_exch{"CH52 CGP"}="H52C CGP";
$resn_exch{"CH4* CGP"}=" H4B CGP"; $resn_exch{"CH3* CGP"}=" H3B CGP"; $resn_exch{"CH21 CGP"}="H21C CGP";
$resn_exch{"CH22 CGP"}="H22C CGP"; $resn_exch{"CH1* CGP"}=" H1B CGP"; $resn_exch{" CH5 CGP"}=" H5C CGP";
$resn_exch{" CH6 CGP"}=" H6C CGP"; $resn_exch{"CH41 CGP"}="H41C CGP"; $resn_exch{"CH42 CGP"}="H42C CGP";
$resn_exch{"GH51 CGP"}="H51G CGP"; $resn_exch{"GH52 CGP"}="H52G CGP"; $resn_exch{"GH4* CGP"}=" H4D CGP";
$resn_exch{"GH3* CGP"}=" H3D CGP"; $resn_exch{"GHO3 CGP"}="HO3G CGP"; $resn_exch{"GH21 CGP"}="H21G CGP";
$resn_exch{"GH22 CGP"}="H22G CGP"; $resn_exch{"GH1* CGP"}=" H1D CGP"; $resn_exch{" GH8 CGP"}=" H8G CGP";
$resn_exch{" GH1 CGP"}=" H1G CGP"; $resn_exch{"GHN1 CGP"}="HN1G CGP"; $resn_exch{"GHN2 CGP"}="HN2G CGP";
$resn_exch{"1H2  CGQ"}=" H21 CGQ"; $resn_exch{"1H4  CGQ"}=" H41 CGQ"; $resn_exch{"1H5  CGQ"}=" H51 CGQ";
$resn_exch{"1H6  CGQ"}=" H61 CGQ"; $resn_exch{"1H7  CGQ"}=" H71 CGQ"; $resn_exch{"1H10 CGQ"}="H101 CGQ";
$resn_exch{"2H10 CGQ"}="H102 CGQ"; $resn_exch{"1H11 CGQ"}="H111 CGQ"; $resn_exch{"2H11 CGQ"}="H112 CGQ";
$resn_exch{"1H12 CGQ"}="H121 CGQ"; $resn_exch{"1H15 CGQ"}="H151 CGQ"; $resn_exch{"2H15 CGQ"}="H152 CGQ";
$resn_exch{"1H19 CGQ"}="H191 CGQ"; $resn_exch{"1H21 CGQ"}="H211 CGQ"; $resn_exch{"1H23 CGQ"}="H231 CGQ";
$resn_exch{"1H24 CGQ"}="H241 CGQ"; $resn_exch{"1H25 CGQ"}="H251 CGQ"; $resn_exch{"1H27 CGQ"}="H271 CGQ";
$resn_exch{"1H29 CGQ"}="H291 CGQ"; $resn_exch{"2H29 CGQ"}="H292 CGQ"; $resn_exch{"1H30 CGQ"}="H301 CGQ";
$resn_exch{"2H30 CGQ"}="H302 CGQ"; $resn_exch{"1H5  CGR"}=" H51 CGR"; $resn_exch{"2H5  CGR"}=" H52 CGR";
$resn_exch{"HC4* CGR"}="HC4' CGR"; $resn_exch{"HC1* CGR"}="HC1' CGR"; $resn_exch{"HC2* CGR"}="HC2' CGR";
$resn_exch{"HC3* CGR"}="HC3' CGR"; $resn_exch{"1H8  CGR"}=" H81 CGR"; $resn_exch{"2H8  CGR"}=" H82 CGR";
$resn_exch{" H1* CGR"}=" H1' CGR"; $resn_exch{" H2* CGR"}=" H2' CGR"; $resn_exch{"HO2* CGR"}="HO2' CGR";
$resn_exch{" H3* CGR"}=" H3' CGR"; $resn_exch{"HO3* CGR"}="HO3' CGR"; $resn_exch{" H4* CGR"}=" H4' CGR";
$resn_exch{"1H5* CGR"}="H5'1 CGR"; $resn_exch{"2H5* CGR"}="H5'2 CGR"; $resn_exch{"1H2A CGR"}="H2A1 CGR";
$resn_exch{"2H2A CGR"}="H2A2 CGR"; $resn_exch{"1H7C CGT"}="H7C1 CGT"; $resn_exch{"2H7C CGT"}="H7C2 CGT";
$resn_exch{"1H6C CGT"}="H6C1 CGT"; $resn_exch{"2H6C CGT"}="H6C2 CGT"; $resn_exch{"1HBC CGT"}="HBC1 CGT";
$resn_exch{"2HBC CGT"}="HBC2 CGT"; $resn_exch{"1HN  CGU"}=" H   CGU"; $resn_exch{"2HN  CGU"}=" HN2 CGU";
$resn_exch{"1HB  CGU"}=" HB2 CGU"; $resn_exch{"2HB  CGU"}=" HB3 CGU"; $resn_exch{"HE2  CGU"}="HE12 CGU";
$resn_exch{"HE4  CGU"}="HE22 CGU"; $resn_exch{"3HOP  CH"}="HOP3  CH"; $resn_exch{"2HOP  CH"}="HOP2  CH";
$resn_exch{"1H5*  CH"}=" H5'  CH"; $resn_exch{"2H5*  CH"}="H5''  CH"; $resn_exch{" H4*  CH"}=" H4'  CH";
$resn_exch{" H3*  CH"}=" H3'  CH"; $resn_exch{"H3T   CH"}="HO3'  CH"; $resn_exch{" H2*  CH"}=" H2'  CH";
$resn_exch{"2HO*  CH"}="HO2'  CH"; $resn_exch{" H1*  CH"}=" H1'  CH"; $resn_exch{"1H4   CH"}=" H41  CH";
$resn_exch{"2H4   CH"}=" H42  CH"; $resn_exch{"1HN4 CH1"}="HN41 CH1"; $resn_exch{"2HN4 CH1"}="HN42 CH1";
$resn_exch{" H1* CH1"}=" H1' CH1"; $resn_exch{" H2* CH1"}=" H2' CH1"; $resn_exch{"*HO2 CH1"}="HO2' CH1";
$resn_exch{"2H3* CH1"}="H3'2 CH1"; $resn_exch{" H3* CH1"}=" H3' CH1"; $resn_exch{" H4* CH1"}=" H4' CH1";
$resn_exch{"1H5* CH1"}="H5'1 CH1"; $resn_exch{"2H5* CH1"}="H5'2 CH1"; $resn_exch{"2HOA CH1"}="HOA2 CH1";
$resn_exch{"2HOB CH1"}="HOB2 CH1"; $resn_exch{"2HOG CH1"}="HOG2 CH1"; $resn_exch{"3HOG CH1"}="HOG3 CH1";
$resn_exch{"1H   CH2"}=" H1  CH2"; $resn_exch{"2H   CH2"}=" H2  CH2"; $resn_exch{"1HC  CH3"}=" HC1 CH3";
$resn_exch{"2HC  CH3"}=" HC2 CH3"; $resn_exch{"3HC  CH3"}=" HC3 CH3"; $resn_exch{"4HC  CH3"}=" HC4 CH3";
$resn_exch{"2HB1 CH4"}="HB12 CH4"; $resn_exch{"3HB1 CH4"}="HB13 CH4"; $resn_exch{"1HG  CH4"}=" HG1 CH4";
$resn_exch{"2HD1 CH4"}="HD12 CH4"; $resn_exch{"3HD1 CH4"}="HD13 CH4"; $resn_exch{"2HE1 CH4"}="HE12 CH4";
$resn_exch{"3HE1 CH4"}="HE13 CH4"; $resn_exch{"1HZ1 CH4"}="HZ11 CH4"; $resn_exch{"2HZ1 CH4"}="HZ12 CH4";
$resn_exch{"2HB2 CH4"}="HB22 CH4"; $resn_exch{"3HB2 CH4"}="HB23 CH4"; $resn_exch{"2HG  CH4"}=" HG2 CH4";
$resn_exch{"2HD2 CH4"}="HD22 CH4"; $resn_exch{"3HD2 CH4"}="HD23 CH4"; $resn_exch{"2HE2 CH4"}="HE22 CH4";
$resn_exch{"3HE2 CH4"}="HE23 CH4"; $resn_exch{"1HZ2 CH4"}="HZ21 CH4"; $resn_exch{"2HZ2 CH4"}="HZ22 CH4";
$resn_exch{"1H1  CH5"}=" H11 CH5"; $resn_exch{"2H1  CH5"}=" H12 CH5"; $resn_exch{"1H3  CH5"}=" H31 CH5";
$resn_exch{"2H3  CH5"}=" H32 CH5"; $resn_exch{"1H4  CH5"}=" H41 CH5"; $resn_exch{"2H4  CH5"}=" H42 CH5";
$resn_exch{"1H5  CH5"}=" H51 CH5"; $resn_exch{"2H5  CH5"}=" H52 CH5"; $resn_exch{"1H6  CH5"}=" H61 CH5";
$resn_exch{"2H6  CH5"}=" H62 CH5"; $resn_exch{"3H6  CH5"}=" H63 CH5"; $resn_exch{"1H7  CH5"}=" H71 CH5";
$resn_exch{"2H7  CH5"}=" H72 CH5"; $resn_exch{"3H7  CH5"}=" H73 CH5"; $resn_exch{"1H8  CH5"}=" H81 CH5";
$resn_exch{"2H8  CH5"}=" H82 CH5"; $resn_exch{"3H8  CH5"}=" H83 CH5"; $resn_exch{"1HB  CH7"}=" HB1 CH7";
$resn_exch{"2HB  CH7"}="HB2A CH7"; $resn_exch{"1HG  CH7"}=" HG1 CH7"; $resn_exch{"2HG  CH7"}=" HG2 CH7";
$resn_exch{"1HD  CH7"}="HD1A CH7"; $resn_exch{"2HD  CH7"}="HD2A CH7"; $resn_exch{"1HE  CH7"}="HE1A CH7";
$resn_exch{"2HE  CH7"}="HE2A CH7"; $resn_exch{"1HA3 CH7"}="HA31 CH7"; $resn_exch{"2HA3 CH7"}="HA32 CH7";
$resn_exch{"1H1  CHA"}=" H11 CHA"; $resn_exch{"2H1  CHA"}=" H12 CHA"; $resn_exch{"1H3  CHA"}=" H31 CHA";
$resn_exch{"2H3  CHA"}=" H32 CHA"; $resn_exch{"1H2' CHA"}="H2'1 CHA"; $resn_exch{"2H2' CHA"}="H2'2 CHA";
$resn_exch{"1H3' CHA"}="H3'1 CHA"; $resn_exch{"2H3' CHA"}="H3'2 CHA"; $resn_exch{"1H4' CHA"}="H4'1 CHA";
$resn_exch{"2H4' CHA"}="H4'2 CHA"; $resn_exch{"1H5' CHA"}="H5'1 CHA"; $resn_exch{"2H5' CHA"}="H5'2 CHA";
$resn_exch{"1H6' CHA"}="H6'1 CHA"; $resn_exch{"2H6' CHA"}="H6'2 CHA"; $resn_exch{"1HN2 CHA"}="HN21 CHA";
$resn_exch{"2HN2 CHA"}="HN22 CHA"; $resn_exch{"1HC1 CHC"}="HC11 CHC"; $resn_exch{"2HC1 CHC"}="HC12 CHC";
$resn_exch{"1HC2 CHC"}="HC21 CHC"; $resn_exch{"2HC2 CHC"}="HC22 CHC"; $resn_exch{"1HC4 CHC"}="HC41 CHC";
$resn_exch{"2HC4 CHC"}="HC42 CHC"; $resn_exch{"1H11 CHC"}="H111 CHC"; $resn_exch{"2H11 CHC"}="H112 CHC";
$resn_exch{"1H12 CHC"}="H121 CHC"; $resn_exch{"2H12 CHC"}="H122 CHC"; $resn_exch{"CH14 CHC"}="H14C CHC";
$resn_exch{"1H15 CHC"}="H151 CHC"; $resn_exch{"2H15 CHC"}="H152 CHC"; $resn_exch{"1H16 CHC"}="H161 CHC";
$resn_exch{"2H16 CHC"}="H162 CHC"; $resn_exch{"CH17 CHC"}="H17C CHC"; $resn_exch{"1H18 CHC"}="H181 CHC";
$resn_exch{"2H18 CHC"}="H182 CHC"; $resn_exch{"3H18 CHC"}="H183 CHC"; $resn_exch{"1H19 CHC"}="H191 CHC";
$resn_exch{"2H19 CHC"}="H192 CHC"; $resn_exch{"3H19 CHC"}="H193 CHC"; $resn_exch{"CH20 CHC"}="H20C CHC";
$resn_exch{"1H21 CHC"}="H211 CHC"; $resn_exch{"2H21 CHC"}="H212 CHC"; $resn_exch{"3H21 CHC"}="H213 CHC";
$resn_exch{"1H22 CHC"}="H221 CHC"; $resn_exch{"2H22 CHC"}="H222 CHC"; $resn_exch{"1H23 CHC"}="H231 CHC";
$resn_exch{"2H23 CHC"}="H232 CHC"; $resn_exch{"1H25 CHC"}="H251 CHC"; $resn_exch{"2H25 CHC"}="H252 CHC";
$resn_exch{"3H25 CHC"}="H253 CHC"; $resn_exch{"1H26 CHC"}="H261 CHC"; $resn_exch{"2H26 CHC"}="H262 CHC";
$resn_exch{"OHT1 CHC"}="HT1O CHC"; $resn_exch{"1H1  CHD"}=" H11 CHD"; $resn_exch{"2H1  CHD"}="H12A CHD";
$resn_exch{"1H2  CHD"}=" H21 CHD"; $resn_exch{"2H2  CHD"}=" H22 CHD"; $resn_exch{"1H4  CHD"}=" H41 CHD";
$resn_exch{"2H4  CHD"}=" H42 CHD"; $resn_exch{"1H6  CHD"}=" H61 CHD"; $resn_exch{"2H6  CHD"}=" H62 CHD";
$resn_exch{"1H11 CHD"}="H111 CHD"; $resn_exch{"2H11 CHD"}="H112 CHD"; $resn_exch{"1H15 CHD"}="H151 CHD";
$resn_exch{"2H15 CHD"}="H152 CHD"; $resn_exch{"1H16 CHD"}="H161 CHD"; $resn_exch{"2H16 CHD"}="H162 CHD";
$resn_exch{"1H18 CHD"}="H181 CHD"; $resn_exch{"2H18 CHD"}="H182 CHD"; $resn_exch{"3H18 CHD"}="H183 CHD";
$resn_exch{"1H19 CHD"}="H191 CHD"; $resn_exch{"2H19 CHD"}="H192 CHD"; $resn_exch{"3H19 CHD"}="H193 CHD";
$resn_exch{"1H21 CHD"}="H211 CHD"; $resn_exch{"2H21 CHD"}="H212 CHD"; $resn_exch{"3H21 CHD"}="H213 CHD";
$resn_exch{"1H22 CHD"}="H221 CHD"; $resn_exch{"2H22 CHD"}="H222 CHD"; $resn_exch{"1H23 CHD"}="H231 CHD";
$resn_exch{"2H23 CHD"}="H232 CHD"; $resn_exch{"1HN  CHF"}=" H   CHF"; $resn_exch{"2HN  CHF"}=" H2  CHF";
$resn_exch{"1HB  CHF"}=" HB2 CHF"; $resn_exch{"2HB  CHF"}=" HB3 CHF"; $resn_exch{"1HD1 CHF"}="HD12 CHF";
$resn_exch{"2HD1 CHF"}="HD13 CHF"; $resn_exch{"1HD2 CHF"}="HD23 CHF"; $resn_exch{"2HD2 CHF"}="HD22 CHF";
$resn_exch{"1HE1 CHF"}="HE12 CHF"; $resn_exch{"2HE1 CHF"}="HE13 CHF"; $resn_exch{"1HE2 CHF"}="HE23 CHF";
$resn_exch{"2HE2 CHF"}="HE22 CHF"; $resn_exch{"1HZ  CHF"}=" HZ2 CHF"; $resn_exch{"2HZ  CHF"}=" HZ3 CHF";
$resn_exch{"1H2  CHG"}=" H21 CHG"; $resn_exch{"2H2  CHG"}=" H22 CHG"; $resn_exch{"1H3  CHG"}=" H31 CHG";
$resn_exch{"2H3  CHG"}=" H32 CHG"; $resn_exch{"1H4  CHG"}=" H41 CHG"; $resn_exch{"2H4  CHG"}=" H42 CHG";
$resn_exch{"1H5  CHG"}=" H51 CHG"; $resn_exch{"2H5  CHG"}=" H52 CHG"; $resn_exch{"1H6  CHG"}=" H61 CHG";
$resn_exch{"2H6  CHG"}=" H62 CHG"; $resn_exch{"1HN7 CHG"}="HN71 CHG"; $resn_exch{"2HN7 CHG"}="HN72 CHG";
$resn_exch{"1H6  CHH"}=" H61 CHH"; $resn_exch{"2H6  CHH"}=" H62 CHH"; $resn_exch{"3H6  CHH"}=" H63 CHH";
$resn_exch{"1H4  CHH"}=" H41 CHH"; $resn_exch{"2H4  CHH"}=" H42 CHH"; $resn_exch{"1H3  CHH"}=" H31 CHH";
$resn_exch{"2H3  CHH"}=" H32 CHH"; $resn_exch{"1H2  CHH"}=" H21 CHH"; $resn_exch{"2H2  CHH"}=" H22 CHH";
$resn_exch{"1H9  CHH"}=" H91 CHH"; $resn_exch{"2H9  CHH"}=" H92 CHH"; $resn_exch{"3H9  CHH"}=" H93 CHH";
$resn_exch{"1H10 CHH"}="H101 CHH"; $resn_exch{"2H10 CHH"}="H102 CHH"; $resn_exch{"3H10 CHH"}="H103 CHH";
$resn_exch{"1H8  CHH"}=" H81 CHH"; $resn_exch{"2H8  CHH"}=" H82 CHH"; $resn_exch{"3H8  CHH"}=" H83 CHH";
$resn_exch{"1H2  CHI"}=" H21 CHI"; $resn_exch{"1H3  CHI"}=" H31 CHI"; $resn_exch{"1H6  CHI"}=" H61 CHI";
$resn_exch{"1H10 CHI"}="H101 CHI"; $resn_exch{"1H12 CHI"}="H121 CHI"; $resn_exch{"2H12 CHI"}="H122 CHI";
$resn_exch{"1H13 CHI"}="H131 CHI"; $resn_exch{"2H13 CHI"}="H132 CHI"; $resn_exch{"1H15 CHI"}="H151 CHI";
$resn_exch{"2H15 CHI"}="H152 CHI"; $resn_exch{"1H16 CHI"}="H161 CHI"; $resn_exch{"2H16 CHI"}="H162 CHI";
$resn_exch{"1H17 CHI"}="H171 CHI"; $resn_exch{"2H17 CHI"}="H172 CHI"; $resn_exch{"1H19 CHI"}="H191 CHI";
$resn_exch{"1H20 CHI"}="H201 CHI"; $resn_exch{"1H22 CHI"}="H221 CHI"; $resn_exch{"1H23 CHI"}="H231 CHI";
$resn_exch{"1HX2 CHK"}="HX21 CHK"; $resn_exch{"2HX2 CHK"}="HX22 CHK"; $resn_exch{"1HX4 CHK"}="HX41 CHK";
$resn_exch{"2HX4 CHK"}="HX42 CHK"; $resn_exch{"1HX5 CHK"}="HX51 CHK"; $resn_exch{"2HX5 CHK"}="HX52 CHK";
$resn_exch{"1HX6 CHK"}="HX61 CHK"; $resn_exch{"2HX6 CHK"}="HX62 CHK"; $resn_exch{"1HX7 CHK"}="HX71 CHK";
$resn_exch{"2HX7 CHK"}="HX72 CHK"; $resn_exch{"1HX8 CHK"}="HX81 CHK"; $resn_exch{"2HX8 CHK"}="HX82 CHK";
$resn_exch{"1HX9 CHK"}="HX91 CHK"; $resn_exch{"2HX9 CHK"}="HX92 CHK"; $resn_exch{"1HXA CHK"}="HXA1 CHK";
$resn_exch{"2HXA CHK"}="HXA2 CHK"; $resn_exch{"1HXB CHK"}="HXB1 CHK"; $resn_exch{"2HXB CHK"}="HXB2 CHK";
$resn_exch{"1HXG CHK"}="HXG1 CHK"; $resn_exch{"2HXG CHK"}="HXG2 CHK"; $resn_exch{"1HMA CHL"}="HMA1 CHL";
$resn_exch{"2HMA CHL"}="HMA2 CHL"; $resn_exch{"3HMA CHL"}="HMA3 CHL"; $resn_exch{"1HAA CHL"}="HAA1 CHL";
$resn_exch{"2HAA CHL"}="HAA2 CHL"; $resn_exch{"1HBA CHL"}="HBA1 CHL"; $resn_exch{"2HBA CHL"}="HBA2 CHL";
$resn_exch{"1HMB CHL"}="HMB1 CHL"; $resn_exch{"2HMB CHL"}="HMB2 CHL"; $resn_exch{"3HMB CHL"}="HMB3 CHL";
$resn_exch{"1HBB CHL"}="HBB1 CHL"; $resn_exch{"2HBB CHL"}="HBB2 CHL"; $resn_exch{"1HAC CHL"}="HAC1 CHL";
$resn_exch{"2HAC CHL"}="HAC2 CHL"; $resn_exch{"1HBC CHL"}="HBC1 CHL"; $resn_exch{"2HBC CHL"}="HBC2 CHL";
$resn_exch{"3HBC CHL"}="HBC3 CHL"; $resn_exch{"1HMD CHL"}="HMD1 CHL"; $resn_exch{"2HMD CHL"}="HMD2 CHL";
$resn_exch{"3HMD CHL"}="HMD3 CHL"; $resn_exch{"1HED CHL"}="HED1 CHL"; $resn_exch{"2HED CHL"}="HED2 CHL";
$resn_exch{"3HED CHL"}="HED3 CHL"; $resn_exch{"1H1  CHL"}=" H11 CHL"; $resn_exch{"2H1  CHL"}=" H12 CHL";
$resn_exch{"1H4  CHL"}=" H41 CHL"; $resn_exch{"2H4  CHL"}=" H42 CHL"; $resn_exch{"3H4  CHL"}=" H43 CHL";
$resn_exch{"1H5  CHL"}=" H51 CHL"; $resn_exch{"2H5  CHL"}=" H52 CHL"; $resn_exch{"1H6  CHL"}=" H61 CHL";
$resn_exch{"2H6  CHL"}=" H62 CHL"; $resn_exch{"1H7  CHL"}=" H71 CHL"; $resn_exch{"2H7  CHL"}=" H72 CHL";
$resn_exch{"1H9  CHL"}=" H91 CHL"; $resn_exch{"2H9  CHL"}=" H92 CHL"; $resn_exch{"3H9  CHL"}=" H93 CHL";
$resn_exch{"1H10 CHL"}="H101 CHL"; $resn_exch{"2H10 CHL"}="H102 CHL"; $resn_exch{"1H11 CHL"}="H111 CHL";
$resn_exch{"2H11 CHL"}="H112 CHL"; $resn_exch{"1H12 CHL"}="H121 CHL"; $resn_exch{"2H12 CHL"}="H122 CHL";
$resn_exch{"1H14 CHL"}="H141 CHL"; $resn_exch{"2H14 CHL"}="H142 CHL"; $resn_exch{"3H14 CHL"}="H143 CHL";
$resn_exch{"1H15 CHL"}="H151 CHL"; $resn_exch{"2H15 CHL"}="H152 CHL"; $resn_exch{"1H16 CHL"}="H161 CHL";
$resn_exch{"2H16 CHL"}="H162 CHL"; $resn_exch{"1H17 CHL"}="H171 CHL"; $resn_exch{"2H17 CHL"}="H172 CHL";
$resn_exch{"1H19 CHL"}="H191 CHL"; $resn_exch{"2H19 CHL"}="H192 CHL"; $resn_exch{"3H19 CHL"}="H193 CHL";
$resn_exch{"1H20 CHL"}="H201 CHL"; $resn_exch{"2H20 CHL"}="H202 CHL"; $resn_exch{"3H20 CHL"}="H203 CHL";
$resn_exch{"1H1  CHM"}=" H11 CHM"; $resn_exch{"2H1  CHM"}=" H12 CHM"; $resn_exch{"1H3  CHM"}=" H31 CHM";
$resn_exch{"2H3  CHM"}=" H32 CHM"; $resn_exch{"1HN1 CHN"}="HN11 CHN"; $resn_exch{"2HN1 CHN"}="HN12 CHN";
$resn_exch{"1H1  CHN"}=" H11 CHN"; $resn_exch{"2H1  CHN"}=" H12 CHN"; $resn_exch{"1H2  CHN"}=" H21 CHN";
$resn_exch{"1H7  CHN"}=" H71 CHN"; $resn_exch{"2H7  CHN"}=" H72 CHN"; $resn_exch{"1H6  CHN"}=" H61 CHN";
$resn_exch{"2H6  CHN"}=" H62 CHN"; $resn_exch{"1H5  CHN"}=" H51 CHN"; $resn_exch{"1H4  CHN"}=" H41 CHN";
$resn_exch{"2H4  CHN"}=" H42 CHN"; $resn_exch{"1H3  CHN"}=" H31 CHN"; $resn_exch{"2H3  CHN"}=" H32 CHN";
$resn_exch{"1HN2 CHN"}="HN21 CHN"; $resn_exch{"2HN2 CHN"}="HN22 CHN"; $resn_exch{"1H1  CHO"}=" H11 CHO";
$resn_exch{"2H1  CHO"}=" H12 CHO"; $resn_exch{"1H2  CHO"}=" H21 CHO"; $resn_exch{"2H2  CHO"}=" H22 CHO";
$resn_exch{"1H4  CHO"}=" H41 CHO"; $resn_exch{"2H4  CHO"}=" H42 CHO"; $resn_exch{"1H6  CHO"}=" H61 CHO";
$resn_exch{"2H6  CHO"}=" H62 CHO"; $resn_exch{"1H11 CHO"}="H111 CHO"; $resn_exch{"2H11 CHO"}="H112 CHO";
$resn_exch{"1H12 CHO"}="H121 CHO"; $resn_exch{"2H12 CHO"}="H122 CHO"; $resn_exch{"1H15 CHO"}="H151 CHO";
$resn_exch{"2H15 CHO"}="H152 CHO"; $resn_exch{"1H16 CHO"}="H161 CHO"; $resn_exch{"2H16 CHO"}="H162 CHO";
$resn_exch{"1H18 CHO"}="H181 CHO"; $resn_exch{"2H18 CHO"}="H182 CHO"; $resn_exch{"3H18 CHO"}="H183 CHO";
$resn_exch{"1H19 CHO"}="H191 CHO"; $resn_exch{"2H19 CHO"}="H192 CHO"; $resn_exch{"3H19 CHO"}="H193 CHO";
$resn_exch{"1H21 CHO"}="H211 CHO"; $resn_exch{"2H21 CHO"}="H212 CHO"; $resn_exch{"3H21 CHO"}="H213 CHO";
$resn_exch{"1H22 CHO"}="H221 CHO"; $resn_exch{"2H22 CHO"}="H222 CHO"; $resn_exch{"1H23 CHO"}="H231 CHO";
$resn_exch{"2H23 CHO"}="H232 CHO"; $resn_exch{"1H26 CHO"}="H261 CHO"; $resn_exch{"2H26 CHO"}="H262 CHO";
$resn_exch{"1HN  CHP"}=" H   CHP"; $resn_exch{"2HN  CHP"}=" H2  CHP"; $resn_exch{" H2  CHP"}=" HC2 CHP";
$resn_exch{"1HBC CHQ"}="HBC1 CHQ"; $resn_exch{"2HBC CHQ"}="HBC2 CHQ"; $resn_exch{"1HBP CHQ"}="HBP1 CHQ";
$resn_exch{"2HBP CHQ"}="HBP2 CHQ"; $resn_exch{"1HGP CHQ"}="HGP1 CHQ"; $resn_exch{"2HGP CHQ"}="HGP2 CHQ";
$resn_exch{"1HDP CHQ"}="HDP1 CHQ"; $resn_exch{"2HDP CHQ"}="HDP2 CHQ"; $resn_exch{"1H19 CHR"}="H191 CHR";
$resn_exch{"2H19 CHR"}="H192 CHR"; $resn_exch{"3H19 CHR"}="H193 CHR"; $resn_exch{"1H18 CHR"}="H181 CHR";
$resn_exch{"2H18 CHR"}="H182 CHR"; $resn_exch{"3H18 CHR"}="H183 CHR"; $resn_exch{"1H31 CHR"}="H311 CHR";
$resn_exch{"2H31 CHR"}="H312 CHR"; $resn_exch{"3H31 CHR"}="H313 CHR"; $resn_exch{"1H32 CHR"}="H321 CHR";
$resn_exch{"2H32 CHR"}="H322 CHR"; $resn_exch{"3H32 CHR"}="H323 CHR"; $resn_exch{"1H35 CHR"}="H351 CHR";
$resn_exch{"2H35 CHR"}="H352 CHR"; $resn_exch{"1HN  CHS"}=" H   CHS"; $resn_exch{"2HN  CHS"}=" H2  CHS";
$resn_exch{"1HB  CHS"}=" HB2 CHS"; $resn_exch{"2HB  CHS"}=" HB3 CHS"; $resn_exch{"1HD1 CHS"}="HD12 CHS";
$resn_exch{"2HD1 CHS"}="HD13 CHS"; $resn_exch{"1HD2 CHS"}="HD23 CHS"; $resn_exch{"2HD2 CHS"}="HD22 CHS";
$resn_exch{"1HE1 CHS"}="HE12 CHS"; $resn_exch{"2HE1 CHS"}="HE13 CHS"; $resn_exch{"1HE2 CHS"}="HE23 CHS";
$resn_exch{"2HE2 CHS"}="HE22 CHS"; $resn_exch{"1HZ  CHS"}=" HZ2 CHS"; $resn_exch{"2HZ  CHS"}=" HZ3 CHS";
$resn_exch{"1HM  CHS"}=" HM1 CHS"; $resn_exch{"2HM  CHS"}=" HM2 CHS"; $resn_exch{"1HC4 CHT"}="HC41 CHT";
$resn_exch{"2HC4 CHT"}="HC42 CHT"; $resn_exch{"1HC5 CHT"}="HC51 CHT"; $resn_exch{"2HC5 CHT"}="HC52 CHT";
$resn_exch{"1H6  CHT"}=" H61 CHT"; $resn_exch{"2H6  CHT"}=" H62 CHT"; $resn_exch{"3H6  CHT"}=" H63 CHT";
$resn_exch{"1H7  CHT"}=" H71 CHT"; $resn_exch{"2H7  CHT"}=" H72 CHT"; $resn_exch{"3H7  CHT"}=" H73 CHT";
$resn_exch{"1H8  CHT"}=" H81 CHT"; $resn_exch{"2H8  CHT"}=" H82 CHT"; $resn_exch{"3H8  CHT"}=" H83 CHT";
$resn_exch{"6HO  CHT"}=" HO6 CHT"; $resn_exch{"1H1  CHX"}=" H11 CHX"; $resn_exch{"2H1  CHX"}=" H12 CHX";
$resn_exch{"1H2  CHX"}=" H21 CHX"; $resn_exch{"2H2  CHX"}=" H22 CHX"; $resn_exch{"1H3  CHX"}=" H31 CHX";
$resn_exch{"2H3  CHX"}=" H32 CHX"; $resn_exch{"1H4  CHX"}=" H41 CHX"; $resn_exch{"2H4  CHX"}=" H42 CHX";
$resn_exch{"1H5  CHX"}=" H51 CHX"; $resn_exch{"2H5  CHX"}=" H52 CHX"; $resn_exch{"1H6  CHX"}=" H61 CHX";
$resn_exch{"2H6  CHX"}=" H62 CHX"; $resn_exch{"1H7  CHY"}=" H71 CHY"; $resn_exch{"2H7  CHY"}=" H72 CHY";
$resn_exch{"1H12 CHY"}="H121 CHY"; $resn_exch{"2H12 CHY"}="H122 CHY"; $resn_exch{"1H14 CHY"}="H141 CHY";
$resn_exch{"2H14 CHY"}="H142 CHY"; $resn_exch{"3H14 CHY"}="H143 CHY"; $resn_exch{"1H15 CHY"}="H151 CHY";
$resn_exch{"2H15 CHY"}="H152 CHY"; $resn_exch{"3H15 CHY"}="H153 CHY"; $resn_exch{"1H19 CHY"}="H191 CHY";
$resn_exch{"2H19 CHY"}="H192 CHY"; $resn_exch{"1H20 CHY"}="H201 CHY"; $resn_exch{"2H20 CHY"}="H202 CHY";
$resn_exch{"1H25 CHY"}="H251 CHY"; $resn_exch{"2H25 CHY"}="H252 CHY"; $resn_exch{"1HN4 CHY"}="HN41 CHY";
$resn_exch{"2HN4 CHY"}="HN42 CHY"; $resn_exch{"1H16 CIA"}="H161 CIA"; $resn_exch{"2H16 CIA"}="H162 CIA";
$resn_exch{"1H10 CIA"}="H101 CIA"; $resn_exch{"2H10 CIA"}="H102 CIA"; $resn_exch{"1H18 CIA"}="H181 CIA";
$resn_exch{"2H18 CIA"}="H182 CIA"; $resn_exch{"3H18 CIA"}="H183 CIA"; $resn_exch{"1H29 CIA"}="H291 CIA";
$resn_exch{"2H29 CIA"}="H292 CIA"; $resn_exch{"1HB1 CIB"}="HB11 CIB"; $resn_exch{"2HB1 CIB"}="HB12 CIB";
$resn_exch{"1HD1 CIB"}="HD11 CIB"; $resn_exch{"2HD1 CIB"}="HD12 CIB"; $resn_exch{"3HD1 CIB"}="HD13 CIB";
$resn_exch{"1HD2 CIB"}="HD21 CIB"; $resn_exch{"2HD2 CIB"}="HD22 CIB"; $resn_exch{"3HD2 CIB"}="HD23 CIB";
$resn_exch{"1H25 CIB"}="H251 CIB"; $resn_exch{"2H25 CIB"}="H252 CIB"; $resn_exch{"3H25 CIB"}="H253 CIB";
$resn_exch{"1HB2 CIB"}="HB21 CIB"; $resn_exch{"2HB2 CIB"}="HB22 CIB"; $resn_exch{"1HD3 CIB"}="HD31 CIB";
$resn_exch{"2HD3 CIB"}="HD32 CIB"; $resn_exch{"3HD3 CIB"}="HD33 CIB"; $resn_exch{"1HD4 CIB"}="HD41 CIB";
$resn_exch{"2HD4 CIB"}="HD42 CIB"; $resn_exch{"3HD4 CIB"}="HD43 CIB"; $resn_exch{"1HB3 CIB"}="HB31 CIB";
$resn_exch{"2HB3 CIB"}="HB32 CIB"; $resn_exch{"1HG3 CIB"}="HG31 CIB"; $resn_exch{"2HG3 CIB"}="HG32 CIB";
$resn_exch{"1HE3 CIB"}="HE31 CIB"; $resn_exch{"2HE3 CIB"}="HE32 CIB"; $resn_exch{"3HE3 CIB"}="HE33 CIB";
$resn_exch{"1H19 CIB"}="H191 CIB"; $resn_exch{"2H19 CIB"}="H192 CIB"; $resn_exch{" AH2 CIC"}=" H2A CIC";
$resn_exch{"AH61 CIC"}="H61A CIC"; $resn_exch{"AH62 CIC"}="H62A CIC"; $resn_exch{" AH8 CIC"}=" H8A CIC";
$resn_exch{"AH1* CIC"}=" H1B CIC"; $resn_exch{"AH2* CIC"}=" H2B CIC"; $resn_exch{"AHO2 CIC"}="HO2A CIC";
$resn_exch{"AH3* CIC"}=" H3B CIC"; $resn_exch{"8HOA CIC"}="HOA8 CIC"; $resn_exch{"9HOA CIC"}="HOA9 CIC";
$resn_exch{"AH4* CIC"}=" H4B CIC"; $resn_exch{"AH51 CIC"}="H51A CIC"; $resn_exch{"AH52 CIC"}="H52A CIC";
$resn_exch{"2HOA CIC"}="HOA2 CIC"; $resn_exch{"5HOA CIC"}="HOA5 CIC"; $resn_exch{"1H12 CIC"}="H121 CIC";
$resn_exch{"2H12 CIC"}="H122 CIC"; $resn_exch{"1H13 CIC"}="H131 CIC"; $resn_exch{"2H13 CIC"}="H132 CIC";
$resn_exch{"3H13 CIC"}="H133 CIC"; $resn_exch{"1H14 CIC"}="H141 CIC"; $resn_exch{"2H14 CIC"}="H142 CIC";
$resn_exch{"3H14 CIC"}="H143 CIC"; $resn_exch{"1H7  CIC"}=" H71 CIC"; $resn_exch{"2H7  CIC"}=" H72 CIC";
$resn_exch{"1H6  CIC"}=" H61 CIC"; $resn_exch{"2H6  CIC"}=" H62 CIC"; $resn_exch{"1H3  CIC"}=" H31 CIC";
$resn_exch{"2H3  CIC"}=" H32 CIC"; $resn_exch{"1H2  CIC"}=" H21 CIC"; $resn_exch{"2H2  CIC"}=" H22 CIC";
$resn_exch{"1H1  CIC"}=" H11 CIC"; $resn_exch{"2H1  CIC"}=" H12 CIC"; $resn_exch{"H11  CIC"}="H111 CIC";
$resn_exch{"H12  CIC"}="H123 CIC"; $resn_exch{"2HO3 CIC"}="HO32 CIC"; $resn_exch{"2HO5 CIC"}="HO52 CIC";
$resn_exch{"1H18 CID"}="H181 CID"; $resn_exch{"2H18 CID"}="H182 CID"; $resn_exch{"1H9  CIE"}=" H91 CIE";
$resn_exch{"2H9  CIE"}=" H92 CIE"; $resn_exch{"1H10 CIE"}="H101 CIE"; $resn_exch{"2H10 CIE"}="H102 CIE";
$resn_exch{"3H10 CIE"}="H103 CIE"; $resn_exch{"1H8' CIE"}="H8'1 CIE"; $resn_exch{"2H8' CIE"}="H8'2 CIE";
$resn_exch{"3H8' CIE"}="H8'3 CIE"; $resn_exch{"1H01 CIG"}="H011 CIG"; $resn_exch{"2H01 CIG"}="H012 CIG";
$resn_exch{"1H1  CIL"}=" H11 CIL"; $resn_exch{"2H1  CIL"}="H12A CIL"; $resn_exch{"1H4  CIL"}=" H41 CIL";
$resn_exch{"2H4  CIL"}=" H42 CIL"; $resn_exch{"3H4  CIL"}=" H43 CIL"; $resn_exch{"1H5  CIL"}=" H51 CIL";
$resn_exch{"2H5  CIL"}=" H52 CIL"; $resn_exch{"3H5  CIL"}=" H53 CIL"; $resn_exch{"1H14 CIL"}="H141 CIL";
$resn_exch{"2H14 CIL"}="H142 CIL"; $resn_exch{"1H15 CIL"}="H151 CIL"; $resn_exch{"2H15 CIL"}="H152 CIL";
$resn_exch{"1H16 CIL"}="H161 CIL"; $resn_exch{"2H16 CIL"}="H162 CIL"; $resn_exch{"1H17 CIL"}="H171 CIL";
$resn_exch{"2H17 CIL"}="H172 CIL"; $resn_exch{"1H18 CIL"}="H181 CIL"; $resn_exch{"2H18 CIL"}="H182 CIL";
$resn_exch{"1H21 CIL"}="H211 CIL"; $resn_exch{"2H12 CIL"}="H122 CIL"; $resn_exch{"1H5  CIO"}=" H51 CIO";
$resn_exch{"2H5  CIO"}=" H52 CIO"; $resn_exch{"1H6  CIO"}=" H61 CIO"; $resn_exch{"2H6  CIO"}=" H62 CIO";
$resn_exch{"1H3  CIO"}=" H31 CIO"; $resn_exch{"2H3  CIO"}=" H32 CIO"; $resn_exch{"1H2  CIO"}=" H21 CIO";
$resn_exch{"2H2  CIO"}=" H22 CIO"; $resn_exch{"1H12 CIO"}="H121 CIO"; $resn_exch{"2H12 CIO"}="H122 CIO";
$resn_exch{"3H12 CIO"}="H123 CIO"; $resn_exch{"1H16 CIO"}="H161 CIO"; $resn_exch{"2H16 CIO"}="H162 CIO";
$resn_exch{"1H17 CIO"}="H171 CIO"; $resn_exch{"2H17 CIO"}="H172 CIO"; $resn_exch{"1H18 CIO"}="H181 CIO";
$resn_exch{"2H18 CIO"}="H182 CIO"; $resn_exch{"1H19 CIO"}="H191 CIO"; $resn_exch{"2H19 CIO"}="H192 CIO";
$resn_exch{"1H7  CIP"}=" H71 CIP"; $resn_exch{"2H7  CIP"}=" H72 CIP"; $resn_exch{"2HOP CIP"}="HOP2 CIP";
$resn_exch{"1H17 CIS"}="H171 CIS"; $resn_exch{"2H17 CIS"}="H172 CIS"; $resn_exch{"3H17 CIS"}="H173 CIS";
$resn_exch{"1H16 CIS"}="H161 CIS"; $resn_exch{"2H16 CIS"}="H162 CIS"; $resn_exch{"1H15 CIS"}="H151 CIS";
$resn_exch{"2H15 CIS"}="H152 CIS"; $resn_exch{"1H14 CIS"}="H141 CIS"; $resn_exch{"2H14 CIS"}="H142 CIS";
$resn_exch{"1H13 CIS"}="H131 CIS"; $resn_exch{"2H13 CIS"}="H132 CIS"; $resn_exch{"1H12 CIS"}="H121 CIS";
$resn_exch{"2H12 CIS"}="H122 CIS"; $resn_exch{"1H11 CIS"}="H111 CIS"; $resn_exch{"2H11 CIS"}="H112 CIS";
$resn_exch{"1H10 CIS"}="H101 CIS"; $resn_exch{"2H10 CIS"}="H102 CIS"; $resn_exch{"1H9  CIS"}=" H91 CIS";
$resn_exch{"2H9  CIS"}=" H92 CIS"; $resn_exch{"1H8  CIS"}=" H81 CIS"; $resn_exch{"2H8  CIS"}=" H82 CIS";
$resn_exch{"1H7  CIS"}=" H71 CIS"; $resn_exch{"2H7  CIS"}=" H72 CIS"; $resn_exch{"1H6  CIS"}=" H61 CIS";
$resn_exch{"2H6  CIS"}=" H62 CIS"; $resn_exch{"1H5  CIS"}=" H51 CIS"; $resn_exch{"2H5  CIS"}=" H52 CIS";
$resn_exch{"1H   CIS"}=" H1A CIS"; $resn_exch{"2H   CIS"}=" H2A CIS"; $resn_exch{"1H48 CIS"}="H481 CIS";
$resn_exch{"2H48 CIS"}="H482 CIS"; $resn_exch{"1H19 CIS"}="H191 CIS"; $resn_exch{"2H19 CIS"}="H192 CIS";
$resn_exch{"1H20 CIS"}="H201 CIS"; $resn_exch{"2H20 CIS"}="H202 CIS"; $resn_exch{"1H21 CIS"}="H211 CIS";
$resn_exch{"2H21 CIS"}="H212 CIS"; $resn_exch{"1H22 CIS"}="H221 CIS"; $resn_exch{"2H22 CIS"}="H222 CIS";
$resn_exch{"1H23 CIS"}="H231 CIS"; $resn_exch{"2H23 CIS"}="H232 CIS"; $resn_exch{"1H24 CIS"}="H241 CIS";
$resn_exch{"2H24 CIS"}="H242 CIS"; $resn_exch{"1H25 CIS"}="H251 CIS"; $resn_exch{"2H25 CIS"}="H252 CIS";
$resn_exch{"1H26 CIS"}="H261 CIS"; $resn_exch{"2H26 CIS"}="H262 CIS"; $resn_exch{"1H27 CIS"}="H271 CIS";
$resn_exch{"2H27 CIS"}="H272 CIS"; $resn_exch{"1H28 CIS"}="H281 CIS"; $resn_exch{"2H28 CIS"}="H282 CIS";
$resn_exch{"1H29 CIS"}="H291 CIS"; $resn_exch{"2H29 CIS"}="H292 CIS"; $resn_exch{"1H30 CIS"}="H301 CIS";
$resn_exch{"2H30 CIS"}="H302 CIS"; $resn_exch{"1H31 CIS"}="H311 CIS"; $resn_exch{"2H31 CIS"}="H312 CIS";
$resn_exch{"1H34 CIS"}="H341 CIS"; $resn_exch{"2H34 CIS"}="H342 CIS"; $resn_exch{"1H35 CIS"}="H351 CIS";
$resn_exch{"2H35 CIS"}="H352 CIS"; $resn_exch{"1H36 CIS"}="H361 CIS"; $resn_exch{"2H36 CIS"}="H362 CIS";
$resn_exch{"1H37 CIS"}="H371 CIS"; $resn_exch{"2H37 CIS"}="H372 CIS"; $resn_exch{"1H38 CIS"}="H381 CIS";
$resn_exch{"2H38 CIS"}="H382 CIS"; $resn_exch{"1H39 CIS"}="H391 CIS"; $resn_exch{"2H39 CIS"}="H392 CIS";
$resn_exch{"1H40 CIS"}="H401 CIS"; $resn_exch{"2H40 CIS"}="H402 CIS"; $resn_exch{"1H41 CIS"}="H411 CIS";
$resn_exch{"2H41 CIS"}="H412 CIS"; $resn_exch{"3H41 CIS"}="H413 CIS"; $resn_exch{"1H2  CIT"}=" H21 CIT";
$resn_exch{"2H2  CIT"}=" H22 CIT"; $resn_exch{"1H4  CIT"}=" H41 CIT"; $resn_exch{"2H4  CIT"}=" H42 CIT";
$resn_exch{"1HC1 CIU"}="HC11 CIU"; $resn_exch{"2HC1 CIU"}="HC12 CIU"; $resn_exch{"1HC2 CIU"}="HC21 CIU";
$resn_exch{"2HC2 CIU"}="HC22 CIU"; $resn_exch{"1HC3 CIU"}="HC31 CIU"; $resn_exch{"1HC4 CIU"}="HC41 CIU";
$resn_exch{"2HC4 CIU"}="HC42 CIU"; $resn_exch{"1HC5 CIU"}="HC51 CIU"; $resn_exch{"2HC5 CIU"}="HC52 CIU";
$resn_exch{"1HC6 CIU"}="HC61 CIU"; $resn_exch{"2HC6 CIU"}="HC62 CIU"; $resn_exch{"1HN7 CK1"}="HN71 CK1";
$resn_exch{"2HN7 CK1"}="HN72 CK1"; $resn_exch{"1H7A CK2"}="H7A1 CK2"; $resn_exch{"2H7A CK2"}="H7A2 CK2";
$resn_exch{"3H7A CK2"}="H7A3 CK2"; $resn_exch{"1H6A CK2"}="H6A1 CK2"; $resn_exch{"2H6A CK2"}="H6A2 CK2";
$resn_exch{"3H6A CK2"}="H6A3 CK2"; $resn_exch{"1HN7 CK2"}="HN71 CK2"; $resn_exch{"2HN7 CK2"}="HN72 CK2";
$resn_exch{"1H7A CK3"}="H7A1 CK3"; $resn_exch{"2H7A CK3"}="H7A2 CK3"; $resn_exch{"3H7A CK3"}="H7A3 CK3";
$resn_exch{"1H6A CK3"}="H6A1 CK3"; $resn_exch{"2H6A CK3"}="H6A2 CK3"; $resn_exch{"3H6A CK3"}="H6A3 CK3";
$resn_exch{"1H6A CK4"}="H6A1 CK4"; $resn_exch{"2H6A CK4"}="H6A2 CK4"; $resn_exch{"3H6A CK4"}="H6A3 CK4";
$resn_exch{"1H7A CK4"}="H7A1 CK4"; $resn_exch{"2H7A CK4"}="H7A2 CK4"; $resn_exch{"3H7A CK4"}="H7A3 CK4";
$resn_exch{"1H6A CK5"}="H6A1 CK5"; $resn_exch{"2H6A CK5"}="H6A2 CK5"; $resn_exch{"3H6A CK5"}="H6A3 CK5";
$resn_exch{"1H7A CK5"}="H7A1 CK5"; $resn_exch{"2H7A CK5"}="H7A2 CK5"; $resn_exch{"3H7A CK5"}="H7A3 CK5";
$resn_exch{"1H6A CK6"}="H6A1 CK6"; $resn_exch{"2H6A CK6"}="H6A2 CK6"; $resn_exch{"3H6A CK6"}="H6A3 CK6";
$resn_exch{"1H8A CK6"}="H8A1 CK6"; $resn_exch{"2H8A CK6"}="H8A2 CK6"; $resn_exch{"3H8A CK6"}="H8A3 CK6";
$resn_exch{"1H7A CK7"}="H7A1 CK7"; $resn_exch{"2H7A CK7"}="H7A2 CK7"; $resn_exch{"1H6A CK7"}="H6A1 CK7";
$resn_exch{"2H6A CK7"}="H6A2 CK7"; $resn_exch{"3H6A CK7"}="H6A3 CK7"; $resn_exch{"1H6A CK8"}="H6A1 CK8";
$resn_exch{"2H6A CK8"}="H6A2 CK8"; $resn_exch{"3H6A CK8"}="H6A3 CK8"; $resn_exch{"1H7A CK8"}="H7A1 CK8";
$resn_exch{"2H7A CK8"}="H7A2 CK8"; $resn_exch{"3H7A CK8"}="H7A3 CK8"; $resn_exch{"1H8B CK8"}="H8B1 CK8";
$resn_exch{"2H8B CK8"}="H8B2 CK8"; $resn_exch{"3H8B CK8"}="H8B3 CK8"; $resn_exch{"1H9B CK8"}="H9B1 CK8";
$resn_exch{"2H9B CK8"}="H9B2 CK8"; $resn_exch{"3H9B CK8"}="H9B3 CK8"; $resn_exch{"1H18 CK9"}="H181 CK9";
$resn_exch{"2H18 CK9"}="H182 CK9"; $resn_exch{"1H12 CK9"}="H121 CK9"; $resn_exch{"2H12 CK9"}="H122 CK9";
$resn_exch{"3H12 CK9"}="H123 CK9"; $resn_exch{"1H11 CK9"}="H111 CK9"; $resn_exch{"2H11 CK9"}="H112 CK9";
$resn_exch{"3H11 CK9"}="H113 CK9"; $resn_exch{"1H25 CK9"}="H251 CK9"; $resn_exch{"2H25 CK9"}="H252 CK9";
$resn_exch{"1H26 CK9"}="H261 CK9"; $resn_exch{"2H26 CK9"}="H262 CK9"; $resn_exch{"3H26 CK9"}="H263 CK9";
$resn_exch{"1H15 CK9"}="H151 CK9"; $resn_exch{"2H15 CK9"}="H152 CK9"; $resn_exch{"1H12 CKE"}="H121 CKE";
$resn_exch{"2H12 CKE"}="H122 CKE"; $resn_exch{"1H20 CKE"}="H201 CKE"; $resn_exch{"2H20 CKE"}="H202 CKE";
$resn_exch{"1H21 CKE"}="H211 CKE"; $resn_exch{"2H21 CKE"}="H212 CKE"; $resn_exch{"1H24 CKE"}="H241 CKE";
$resn_exch{"2H24 CKE"}="H242 CKE"; $resn_exch{"1H27 CKE"}="H271 CKE"; $resn_exch{"2H27 CKE"}="H272 CKE";
$resn_exch{"1H30 CKE"}="H301 CKE"; $resn_exch{"2H30 CKE"}="H302 CKE"; $resn_exch{"1H31 CKE"}="H311 CKE";
$resn_exch{"2H31 CKE"}="H312 CKE"; $resn_exch{"3H31 CKE"}="H313 CKE"; $resn_exch{"1H10 CKE"}="H101 CKE";
$resn_exch{"2H10 CKE"}="H102 CKE"; $resn_exch{"1H23 CKE"}="H231 CKE"; $resn_exch{"2H23 CKE"}="H232 CKE";
$resn_exch{"1H26 CKE"}="H261 CKE"; $resn_exch{"2H26 CKE"}="H262 CKE"; $resn_exch{"1H28 CKE"}="H281 CKE";
$resn_exch{"2H28 CKE"}="H282 CKE"; $resn_exch{"1H29 CKE"}="H291 CKE"; $resn_exch{"2H29 CKE"}="H292 CKE";
$resn_exch{"1H09 CKE"}="H091 CKE"; $resn_exch{"2H09 CKE"}="H092 CKE"; $resn_exch{"1H1' CKI"}="H1'1 CKI";
$resn_exch{"2H1' CKI"}="H1'2 CKI"; $resn_exch{"1H2' CKI"}="H2'1 CKI"; $resn_exch{"2H2' CKI"}="H2'2 CKI";
$resn_exch{"1HN2 CKI"}="HN21 CKI"; $resn_exch{"2HN2 CKI"}="HN22 CKI"; $resn_exch{"1H1  CKP"}=" H11 CKP";
$resn_exch{"2H1  CKP"}=" H12 CKP"; $resn_exch{"1HMA CL1"}="HMA1 CL1"; $resn_exch{"2HMA CL1"}="HMA2 CL1";
$resn_exch{"3HMA CL1"}="HMA3 CL1"; $resn_exch{"1HAA CL1"}="HAA1 CL1"; $resn_exch{"2HAA CL1"}="HAA2 CL1";
$resn_exch{"1HBA CL1"}="HBA1 CL1"; $resn_exch{"2HBA CL1"}="HBA2 CL1"; $resn_exch{"1HMB CL1"}="HMB1 CL1";
$resn_exch{"2HMB CL1"}="HMB2 CL1"; $resn_exch{"3HMB CL1"}="HMB3 CL1"; $resn_exch{"1HBB CL1"}="HBB1 CL1";
$resn_exch{"2HBB CL1"}="HBB2 CL1"; $resn_exch{"1HMC CL1"}="HMC1 CL1"; $resn_exch{"2HMC CL1"}="HMC2 CL1";
$resn_exch{"3HMC CL1"}="HMC3 CL1"; $resn_exch{"1HAC CL1"}="HAC1 CL1"; $resn_exch{"2HAC CL1"}="HAC2 CL1";
$resn_exch{"1HBC CL1"}="HBC1 CL1"; $resn_exch{"2HBC CL1"}="HBC2 CL1"; $resn_exch{"3HBC CL1"}="HBC3 CL1";
$resn_exch{"1HMD CL1"}="HMD1 CL1"; $resn_exch{"2HMD CL1"}="HMD2 CL1"; $resn_exch{"3HMD CL1"}="HMD3 CL1";
$resn_exch{"1HED CL1"}="HED1 CL1"; $resn_exch{"2HED CL1"}="HED2 CL1"; $resn_exch{"3HED CL1"}="HED3 CL1";
$resn_exch{"1H1  CL1"}=" H11 CL1"; $resn_exch{"2H1  CL1"}=" H12 CL1"; $resn_exch{"1H4  CL1"}=" H41 CL1";
$resn_exch{"2H4  CL1"}=" H42 CL1"; $resn_exch{"3H4  CL1"}=" H43 CL1"; $resn_exch{"1H5  CL1"}=" H51 CL1";
$resn_exch{"2H5  CL1"}=" H52 CL1"; $resn_exch{"1H6  CL1"}=" H61 CL1"; $resn_exch{"2H6  CL1"}=" H62 CL1";
$resn_exch{"1H7  CL1"}=" H71 CL1"; $resn_exch{"2H7  CL1"}=" H72 CL1"; $resn_exch{"1H9  CL1"}=" H91 CL1";
$resn_exch{"2H9  CL1"}=" H92 CL1"; $resn_exch{"3H9  CL1"}=" H93 CL1"; $resn_exch{"1H10 CL1"}="H101 CL1";
$resn_exch{"2H10 CL1"}="H102 CL1"; $resn_exch{"1H11 CL1"}="H111 CL1"; $resn_exch{"2H11 CL1"}="H112 CL1";
$resn_exch{"1H12 CL1"}="H121 CL1"; $resn_exch{"2H12 CL1"}="H122 CL1"; $resn_exch{"1H14 CL1"}="H141 CL1";
$resn_exch{"2H14 CL1"}="H142 CL1"; $resn_exch{"3H14 CL1"}="H143 CL1"; $resn_exch{"1H15 CL1"}="H151 CL1";
$resn_exch{"2H15 CL1"}="H152 CL1"; $resn_exch{"1H16 CL1"}="H161 CL1"; $resn_exch{"2H16 CL1"}="H162 CL1";
$resn_exch{"1H17 CL1"}="H171 CL1"; $resn_exch{"2H17 CL1"}="H172 CL1"; $resn_exch{"1H19 CL1"}="H191 CL1";
$resn_exch{"2H19 CL1"}="H192 CL1"; $resn_exch{"3H19 CL1"}="H193 CL1"; $resn_exch{"1H20 CL1"}="H201 CL1";
$resn_exch{"2H20 CL1"}="H202 CL1"; $resn_exch{"3H20 CL1"}="H203 CL1"; $resn_exch{"1HMA CL2"}="HMA1 CL2";
$resn_exch{"2HMA CL2"}="HMA2 CL2"; $resn_exch{"3HMA CL2"}="HMA3 CL2"; $resn_exch{"1HAA CL2"}="HAA1 CL2";
$resn_exch{"2HAA CL2"}="HAA2 CL2"; $resn_exch{"1HBA CL2"}="HBA1 CL2"; $resn_exch{"2HBA CL2"}="HBA2 CL2";
$resn_exch{"1HMB CL2"}="HMB1 CL2"; $resn_exch{"2HMB CL2"}="HMB2 CL2"; $resn_exch{"3HMB CL2"}="HMB3 CL2";
$resn_exch{"1HBB CL2"}="HBB1 CL2"; $resn_exch{"2HBB CL2"}="HBB2 CL2"; $resn_exch{"1HMC CL2"}="HMC1 CL2";
$resn_exch{"2HMC CL2"}="HMC2 CL2"; $resn_exch{"3HMC CL2"}="HMC3 CL2"; $resn_exch{"1HAC CL2"}="HAC1 CL2";
$resn_exch{"2HAC CL2"}="HAC2 CL2"; $resn_exch{"1HBC CL2"}="HBC1 CL2"; $resn_exch{"2HBC CL2"}="HBC2 CL2";
$resn_exch{"3HBC CL2"}="HBC3 CL2"; $resn_exch{"1HMD CL2"}="HMD1 CL2"; $resn_exch{"2HMD CL2"}="HMD2 CL2";
$resn_exch{"3HMD CL2"}="HMD3 CL2"; $resn_exch{"1HED CL2"}="HED1 CL2"; $resn_exch{"2HED CL2"}="HED2 CL2";
$resn_exch{"3HED CL2"}="HED3 CL2"; $resn_exch{"1H1  CL2"}=" H11 CL2"; $resn_exch{"2H1  CL2"}=" H12 CL2";
$resn_exch{"1H4  CL2"}=" H41 CL2"; $resn_exch{"2H4  CL2"}=" H42 CL2"; $resn_exch{"3H4  CL2"}=" H43 CL2";
$resn_exch{"1H5  CL2"}=" H51 CL2"; $resn_exch{"2H5  CL2"}=" H52 CL2"; $resn_exch{"1H6  CL2"}=" H61 CL2";
$resn_exch{"2H6  CL2"}=" H62 CL2"; $resn_exch{"1H7  CL2"}=" H71 CL2"; $resn_exch{"2H7  CL2"}=" H72 CL2";
$resn_exch{"1H9  CL2"}=" H91 CL2"; $resn_exch{"2H9  CL2"}=" H92 CL2"; $resn_exch{"3H9  CL2"}=" H93 CL2";
$resn_exch{"1H10 CL2"}="H101 CL2"; $resn_exch{"2H10 CL2"}="H102 CL2"; $resn_exch{"1H11 CL2"}="H111 CL2";
$resn_exch{"2H11 CL2"}="H112 CL2"; $resn_exch{"1H12 CL2"}="H121 CL2"; $resn_exch{"2H12 CL2"}="H122 CL2";
$resn_exch{"1H14 CL2"}="H141 CL2"; $resn_exch{"2H14 CL2"}="H142 CL2"; $resn_exch{"3H14 CL2"}="H143 CL2";
$resn_exch{"1H15 CL2"}="H151 CL2"; $resn_exch{"2H15 CL2"}="H152 CL2"; $resn_exch{"1H16 CL2"}="H161 CL2";
$resn_exch{"2H16 CL2"}="H162 CL2"; $resn_exch{"1H17 CL2"}="H171 CL2"; $resn_exch{"2H17 CL2"}="H172 CL2";
$resn_exch{"1H19 CL2"}="H191 CL2"; $resn_exch{"2H19 CL2"}="H192 CL2"; $resn_exch{"3H19 CL2"}="H193 CL2";
$resn_exch{"1H20 CL2"}="H201 CL2"; $resn_exch{"2H20 CL2"}="H202 CL2"; $resn_exch{"3H20 CL2"}="H203 CL2";
$resn_exch{"1H23 CL3"}="H231 CL3"; $resn_exch{"2H23 CL3"}="H232 CL3"; $resn_exch{"3H23 CL3"}="H233 CL3";
$resn_exch{"1H28 CL3"}="H281 CL3"; $resn_exch{"2H28 CL3"}="H282 CL3"; $resn_exch{"3H28 CL3"}="H283 CL3";
$resn_exch{"1HC2 CL4"}="HC21 CL4"; $resn_exch{"2HC2 CL4"}="HC22 CL4"; $resn_exch{"1H17 CL4"}="H171 CL4";
$resn_exch{"2H17 CL4"}="H172 CL4"; $resn_exch{"1H6  CL5"}=" H61 CL5"; $resn_exch{"2H6  CL5"}=" H62 CL5";
$resn_exch{"1H5  CL5"}=" H51 CL5"; $resn_exch{"2H5  CL5"}=" H52 CL5"; $resn_exch{"1H3  CL5"}=" H31 CL5";
$resn_exch{"2H3  CL5"}=" H32 CL5"; $resn_exch{"1HMA CLA"}="HMA1 CLA"; $resn_exch{"2HMA CLA"}="HMA2 CLA";
$resn_exch{"3HMA CLA"}="HMA3 CLA"; $resn_exch{"1HAA CLA"}="HAA1 CLA"; $resn_exch{"2HAA CLA"}="HAA2 CLA";
$resn_exch{"1HBA CLA"}="HBA1 CLA"; $resn_exch{"2HBA CLA"}="HBA2 CLA"; $resn_exch{"1HMB CLA"}="HMB1 CLA";
$resn_exch{"2HMB CLA"}="HMB2 CLA"; $resn_exch{"3HMB CLA"}="HMB3 CLA"; $resn_exch{"1HBB CLA"}="HBB1 CLA";
$resn_exch{"2HBB CLA"}="HBB2 CLA"; $resn_exch{"1HMC CLA"}="HMC1 CLA"; $resn_exch{"2HMC CLA"}="HMC2 CLA";
$resn_exch{"3HMC CLA"}="HMC3 CLA"; $resn_exch{"1HAC CLA"}="HAC1 CLA"; $resn_exch{"2HAC CLA"}="HAC2 CLA";
$resn_exch{"1HBC CLA"}="HBC1 CLA"; $resn_exch{"2HBC CLA"}="HBC2 CLA"; $resn_exch{"3HBC CLA"}="HBC3 CLA";
$resn_exch{"1HMD CLA"}="HMD1 CLA"; $resn_exch{"2HMD CLA"}="HMD2 CLA"; $resn_exch{"3HMD CLA"}="HMD3 CLA";
$resn_exch{"1HED CLA"}="HED1 CLA"; $resn_exch{"2HED CLA"}="HED2 CLA"; $resn_exch{"3HED CLA"}="HED3 CLA";
$resn_exch{"1H1  CLA"}=" H11 CLA"; $resn_exch{"2H1  CLA"}=" H12 CLA"; $resn_exch{"1H4  CLA"}=" H41 CLA";
$resn_exch{"2H4  CLA"}=" H42 CLA"; $resn_exch{"3H4  CLA"}=" H43 CLA"; $resn_exch{"1H5  CLA"}=" H51 CLA";
$resn_exch{"2H5  CLA"}=" H52 CLA"; $resn_exch{"1H6  CLA"}=" H61 CLA"; $resn_exch{"2H6  CLA"}=" H62 CLA";
$resn_exch{"1H7  CLA"}=" H71 CLA"; $resn_exch{"2H7  CLA"}=" H72 CLA"; $resn_exch{"1H9  CLA"}=" H91 CLA";
$resn_exch{"2H9  CLA"}=" H92 CLA"; $resn_exch{"3H9  CLA"}=" H93 CLA"; $resn_exch{"1H10 CLA"}="H101 CLA";
$resn_exch{"2H10 CLA"}="H102 CLA"; $resn_exch{"1H11 CLA"}="H111 CLA"; $resn_exch{"2H11 CLA"}="H112 CLA";
$resn_exch{"1H12 CLA"}="H121 CLA"; $resn_exch{"2H12 CLA"}="H122 CLA"; $resn_exch{"1H14 CLA"}="H141 CLA";
$resn_exch{"2H14 CLA"}="H142 CLA"; $resn_exch{"3H14 CLA"}="H143 CLA"; $resn_exch{"1H15 CLA"}="H151 CLA";
$resn_exch{"2H15 CLA"}="H152 CLA"; $resn_exch{"1H16 CLA"}="H161 CLA"; $resn_exch{"2H16 CLA"}="H162 CLA";
$resn_exch{"1H17 CLA"}="H171 CLA"; $resn_exch{"2H17 CLA"}="H172 CLA"; $resn_exch{"1H19 CLA"}="H191 CLA";
$resn_exch{"2H19 CLA"}="H192 CLA"; $resn_exch{"3H19 CLA"}="H193 CLA"; $resn_exch{"1H20 CLA"}="H201 CLA";
$resn_exch{"2H20 CLA"}="H202 CLA"; $resn_exch{"3H20 CLA"}="H203 CLA"; $resn_exch{"1HN  CLB"}=" H   CLB";
$resn_exch{"2HN  CLB"}=" H2  CLB"; $resn_exch{"1HB  CLB"}=" HB2 CLB"; $resn_exch{"2HB  CLB"}=" HB3 CLB";
$resn_exch{"1H7  CLB"}=" H71 CLB"; $resn_exch{"2H7  CLB"}=" H72 CLB"; $resn_exch{" H2  CLB"}=" HC2 CLB";
$resn_exch{"1HOB CLB"}="HOB1 CLB"; $resn_exch{"2HOB CLB"}="HOB2 CLB"; $resn_exch{"1H11 CLB"}="H111 CLB";
$resn_exch{"2H11 CLB"}="H112 CLB"; $resn_exch{"3H11 CLB"}="H113 CLB"; $resn_exch{"1H12 CLC"}="H121 CLC";
$resn_exch{"2H12 CLC"}="H122 CLC"; $resn_exch{"3H12 CLC"}="H123 CLC"; $resn_exch{"1H21 CLC"}="H211 CLC";
$resn_exch{"2H21 CLC"}="H212 CLC"; $resn_exch{"1HN  CLD"}=" H   CLD"; $resn_exch{"2HN  CLD"}=" H2  CLD";
$resn_exch{"1HB  CLD"}=" HB2 CLD"; $resn_exch{"2HB  CLD"}=" HB3 CLD"; $resn_exch{"1H7  CLD"}=" H71 CLD";
$resn_exch{"2H7  CLD"}=" H72 CLD"; $resn_exch{" H2  CLD"}=" HC2 CLD"; $resn_exch{"1HOB CLD"}="HOB1 CLD";
$resn_exch{"2HOB CLD"}="HOB2 CLD"; $resn_exch{"1H11 CLD"}="H111 CLD"; $resn_exch{"2H11 CLD"}="H112 CLD";
$resn_exch{"3H11 CLD"}="H113 CLD"; $resn_exch{" HN  CLE"}=" H2  CLE"; $resn_exch{" HB1 CLE"}=" HB2 CLE";
$resn_exch{" HB2 CLE"}=" HB3 CLE"; $resn_exch{"1HD1 CLE"}="HD11 CLE"; $resn_exch{"2HD1 CLE"}="HD12 CLE";
$resn_exch{"3HD1 CLE"}="HD13 CLE"; $resn_exch{"1HD2 CLE"}="HD21 CLE"; $resn_exch{"2HD2 CLE"}="HD22 CLE";
$resn_exch{"3HD2 CLE"}="HD23 CLE"; $resn_exch{"1HN2 CLE"}="HN21 CLE"; $resn_exch{"2HN2 CLE"}="HN22 CLE";
$resn_exch{" HN  CLG"}=" H2  CLG"; $resn_exch{"1HB  CLG"}=" HB2 CLG"; $resn_exch{"2HB  CLG"}=" HB3 CLG";
$resn_exch{"1HG  CLG"}=" HG2 CLG"; $resn_exch{"2HG  CLG"}=" HG3 CLG"; $resn_exch{"1HD  CLG"}=" HD2 CLG";
$resn_exch{"2HD  CLG"}=" HD3 CLG"; $resn_exch{"1HE  CLG"}=" HE2 CLG"; $resn_exch{"2HE  CLG"}=" HE3 CLG";
$resn_exch{" HNZ CLG"}=" HZ  CLG"; $resn_exch{"1HI  CLG"}=" HI1 CLG"; $resn_exch{"2HI  CLG"}=" HI2 CLG";
$resn_exch{"1HL  CLG"}=" HL1 CLG"; $resn_exch{"2HL  CLG"}=" HL2 CLG"; $resn_exch{"1HNM CLG"}="HNM1 CLG";
$resn_exch{"2HNM CLG"}="HNM2 CLG"; $resn_exch{" HN  CLH"}=" H2  CLH"; $resn_exch{"1HB  CLH"}=" HB2 CLH";
$resn_exch{"2HB  CLH"}=" HB3 CLH"; $resn_exch{"1HG  CLH"}=" HG2 CLH"; $resn_exch{"2HG  CLH"}=" HG3 CLH";
$resn_exch{"1HD  CLH"}=" HD2 CLH"; $resn_exch{"2HD  CLH"}=" HD3 CLH"; $resn_exch{"1HE  CLH"}=" HE2 CLH";
$resn_exch{"2HE  CLH"}=" HE3 CLH"; $resn_exch{" HNZ CLH"}=" HZ  CLH"; $resn_exch{"1HI  CLH"}=" HI1 CLH";
$resn_exch{"2HI  CLH"}=" HI2 CLH"; $resn_exch{"1H12 CLI"}="H121 CLI"; $resn_exch{"2H12 CLI"}="H122 CLI";
$resn_exch{"1H6  CLI"}=" H61 CLI"; $resn_exch{"2H6  CLI"}=" H62 CLI"; $resn_exch{"1H4C CLK"}="H4C1 CLK";
$resn_exch{"2H4C CLK"}="H4C2 CLK"; $resn_exch{"1H9N CLK"}="H9N1 CLK"; $resn_exch{"2H9N CLK"}="H9N2 CLK";
$resn_exch{"1H1  CLL"}=" H11 CLL"; $resn_exch{"2H1  CLL"}=" H12 CLL"; $resn_exch{"1H2  CLL"}=" H21 CLL";
$resn_exch{"2H2  CLL"}=" H22 CLL"; $resn_exch{"1H4  CLL"}=" H41 CLL"; $resn_exch{"2H4  CLL"}=" H42 CLL";
$resn_exch{"1H7  CLL"}=" H71 CLL"; $resn_exch{"2H7  CLL"}=" H72 CLL"; $resn_exch{"1H11 CLL"}="H111 CLL";
$resn_exch{"2H11 CLL"}="H112 CLL"; $resn_exch{"1H12 CLL"}="H121 CLL"; $resn_exch{"2H12 CLL"}="H122 CLL";
$resn_exch{"1H15 CLL"}="H151 CLL"; $resn_exch{"2H15 CLL"}="H152 CLL"; $resn_exch{"1H16 CLL"}="H161 CLL";
$resn_exch{"2H16 CLL"}="H162 CLL"; $resn_exch{"1H18 CLL"}="H181 CLL"; $resn_exch{"2H18 CLL"}="H182 CLL";
$resn_exch{"3H18 CLL"}="H183 CLL"; $resn_exch{"1H19 CLL"}="H191 CLL"; $resn_exch{"2H19 CLL"}="H192 CLL";
$resn_exch{"3H19 CLL"}="H193 CLL"; $resn_exch{"1H21 CLL"}="H211 CLL"; $resn_exch{"2H21 CLL"}="H212 CLL";
$resn_exch{"3H21 CLL"}="H213 CLL"; $resn_exch{"1H22 CLL"}="H221 CLL"; $resn_exch{"2H22 CLL"}="H222 CLL";
$resn_exch{"1H23 CLL"}="H231 CLL"; $resn_exch{"2H23 CLL"}="H232 CLL"; $resn_exch{"1H24 CLL"}="H241 CLL";
$resn_exch{"2H24 CLL"}="H242 CLL"; $resn_exch{"1H26 CLL"}="H261 CLL"; $resn_exch{"2H26 CLL"}="H262 CLL";
$resn_exch{"3H26 CLL"}="H263 CLL"; $resn_exch{"1H27 CLL"}="H271 CLL"; $resn_exch{"2H27 CLL"}="H272 CLL";
$resn_exch{"3H27 CLL"}="H273 CLL"; $resn_exch{"1H29 CLL"}="H291 CLL"; $resn_exch{"2H29 CLL"}="H292 CLL";
$resn_exch{"1H30 CLL"}="H301 CLL"; $resn_exch{"2H30 CLL"}="H302 CLL"; $resn_exch{"1H31 CLL"}="H311 CLL";
$resn_exch{"2H31 CLL"}="H312 CLL"; $resn_exch{"1H32 CLL"}="H321 CLL"; $resn_exch{"2H32 CLL"}="H322 CLL";
$resn_exch{"1H33 CLL"}="H331 CLL"; $resn_exch{"2H33 CLL"}="H332 CLL"; $resn_exch{"1H34 CLL"}="H341 CLL";
$resn_exch{"2H34 CLL"}="H342 CLL"; $resn_exch{"1H35 CLL"}="H351 CLL"; $resn_exch{"2H35 CLL"}="H352 CLL";
$resn_exch{"1H38 CLL"}="H381 CLL"; $resn_exch{"2H38 CLL"}="H382 CLL"; $resn_exch{"1H41 CLL"}="H411 CLL";
$resn_exch{"2H41 CLL"}="H412 CLL"; $resn_exch{"1H42 CLL"}="H421 CLL"; $resn_exch{"2H42 CLL"}="H422 CLL";
$resn_exch{"1H43 CLL"}="H431 CLL"; $resn_exch{"2H43 CLL"}="H432 CLL"; $resn_exch{"1H44 CLL"}="H441 CLL";
$resn_exch{"2H44 CLL"}="H442 CLL"; $resn_exch{"1H45 CLL"}="H451 CLL"; $resn_exch{"2H45 CLL"}="H452 CLL";
$resn_exch{"3H45 CLL"}="H453 CLL"; $resn_exch{"1H4  CLM"}=" H41 CLM"; $resn_exch{"2H4  CLM"}=" H42 CLM";
$resn_exch{"1HMA CLN"}="HMA1 CLN"; $resn_exch{"2HMA CLN"}="HMA2 CLN"; $resn_exch{"3HMA CLN"}="HMA3 CLN";
$resn_exch{"1HAA CLN"}="HAA1 CLN"; $resn_exch{"2HAA CLN"}="HAA2 CLN"; $resn_exch{"1HBA CLN"}="HBA1 CLN";
$resn_exch{"2HBA CLN"}="HBA2 CLN"; $resn_exch{"1HMB CLN"}="HMB1 CLN"; $resn_exch{"2HMB CLN"}="HMB2 CLN";
$resn_exch{"3HMB CLN"}="HMB3 CLN"; $resn_exch{"1HBB CLN"}="HBB1 CLN"; $resn_exch{"2HBB CLN"}="HBB2 CLN";
$resn_exch{"1HMC CLN"}="HMC1 CLN"; $resn_exch{"2HMC CLN"}="HMC2 CLN"; $resn_exch{"3HMC CLN"}="HMC3 CLN";
$resn_exch{"1HMD CLN"}="HMD1 CLN"; $resn_exch{"2HMD CLN"}="HMD2 CLN"; $resn_exch{"3HMD CLN"}="HMD3 CLN";
$resn_exch{"1HAD CLN"}="HAD1 CLN"; $resn_exch{"2HAD CLN"}="HAD2 CLN"; $resn_exch{"1HBD CLN"}="HBD1 CLN";
$resn_exch{"2HBD CLN"}="HBD2 CLN"; $resn_exch{"1H1  CLQ"}=" H11 CLQ"; $resn_exch{"1H2  CLQ"}=" H21 CLQ";
$resn_exch{"1H5  CLQ"}=" H51 CLQ"; $resn_exch{"1H6  CLQ"}=" H61 CLQ"; $resn_exch{"1H8  CLQ"}=" H81 CLQ";
$resn_exch{"1HN2 CLQ"}="HN21 CLQ"; $resn_exch{"1H10 CLQ"}="H101 CLQ"; $resn_exch{"1H11 CLQ"}="H111 CLQ";
$resn_exch{"2H11 CLQ"}="H112 CLQ"; $resn_exch{"1H12 CLQ"}="H121 CLQ"; $resn_exch{"2H12 CLQ"}="H122 CLQ";
$resn_exch{"1H13 CLQ"}="H131 CLQ"; $resn_exch{"2H13 CLQ"}="H132 CLQ"; $resn_exch{"1H14 CLQ"}="H141 CLQ";
$resn_exch{"2H14 CLQ"}="H142 CLQ"; $resn_exch{"1H15 CLQ"}="H151 CLQ"; $resn_exch{"2H15 CLQ"}="H152 CLQ";
$resn_exch{"3H15 CLQ"}="H153 CLQ"; $resn_exch{"1H16 CLQ"}="H161 CLQ"; $resn_exch{"2H16 CLQ"}="H162 CLQ";
$resn_exch{"1H17 CLQ"}="H171 CLQ"; $resn_exch{"2H17 CLQ"}="H172 CLQ"; $resn_exch{"3H17 CLQ"}="H173 CLQ";
$resn_exch{"1H18 CLQ"}="H181 CLQ"; $resn_exch{"2H18 CLQ"}="H182 CLQ"; $resn_exch{"3H18 CLQ"}="H183 CLQ";
$resn_exch{"1H1  CLR"}=" H11 CLR"; $resn_exch{"2H1  CLR"}=" H12 CLR"; $resn_exch{"1H2  CLR"}=" H21 CLR";
$resn_exch{"2H2  CLR"}=" H22 CLR"; $resn_exch{"1H4  CLR"}=" H41 CLR"; $resn_exch{"2H4  CLR"}=" H42 CLR";
$resn_exch{"1H7  CLR"}=" H71 CLR"; $resn_exch{"2H7  CLR"}=" H72 CLR"; $resn_exch{"1H11 CLR"}="H111 CLR";
$resn_exch{"2H11 CLR"}="H112 CLR"; $resn_exch{"1H12 CLR"}="H121 CLR"; $resn_exch{"2H12 CLR"}="H122 CLR";
$resn_exch{"1H15 CLR"}="H151 CLR"; $resn_exch{"2H15 CLR"}="H152 CLR"; $resn_exch{"1H16 CLR"}="H161 CLR";
$resn_exch{"2H16 CLR"}="H162 CLR"; $resn_exch{"1H18 CLR"}="H181 CLR"; $resn_exch{"2H18 CLR"}="H182 CLR";
$resn_exch{"3H18 CLR"}="H183 CLR"; $resn_exch{"1H19 CLR"}="H191 CLR"; $resn_exch{"2H19 CLR"}="H192 CLR";
$resn_exch{"3H19 CLR"}="H193 CLR"; $resn_exch{"1H21 CLR"}="H211 CLR"; $resn_exch{"2H21 CLR"}="H212 CLR";
$resn_exch{"3H21 CLR"}="H213 CLR"; $resn_exch{"1H22 CLR"}="H221 CLR"; $resn_exch{"2H22 CLR"}="H222 CLR";
$resn_exch{"1H23 CLR"}="H231 CLR"; $resn_exch{"2H23 CLR"}="H232 CLR"; $resn_exch{"1H24 CLR"}="H241 CLR";
$resn_exch{"2H24 CLR"}="H242 CLR"; $resn_exch{"1H26 CLR"}="H261 CLR"; $resn_exch{"2H26 CLR"}="H262 CLR";
$resn_exch{"3H26 CLR"}="H263 CLR"; $resn_exch{"1H27 CLR"}="H271 CLR"; $resn_exch{"2H27 CLR"}="H272 CLR";
$resn_exch{"3H27 CLR"}="H273 CLR"; $resn_exch{"1H2  CLS"}=" H21 CLS"; $resn_exch{"2H2  CLS"}=" H22 CLS";
$resn_exch{"1H3' CLS"}="H3'1 CLS"; $resn_exch{"2H3' CLS"}="H3'2 CLS"; $resn_exch{"1H23 CLS"}="H231 CLS";
$resn_exch{"2H23 CLS"}="H232 CLS"; $resn_exch{"3H23 CLS"}="H233 CLS"; $resn_exch{"1H13 CLS"}="H131 CLS";
$resn_exch{"2H13 CLS"}="H132 CLS"; $resn_exch{"1HA  CLT"}=" HA1 CLT"; $resn_exch{"2HA  CLT"}=" HA2 CLT";
$resn_exch{"1HB1 CLT"}="HB11 CLT"; $resn_exch{"2HB1 CLT"}="HB12 CLT"; $resn_exch{"1HG  CLT"}=" HG1 CLT";
$resn_exch{"2HG  CLT"}=" HG2 CLT"; $resn_exch{"1HAG CLU"}="HAG1 CLU"; $resn_exch{"2HAG CLU"}="HAG2 CLU";
$resn_exch{"1HAF CLU"}="HAF1 CLU"; $resn_exch{"2HAF CLU"}="HAF2 CLU"; $resn_exch{"1H1  CLW"}=" H11 CLW";
$resn_exch{"1H2  CLW"}=" H21 CLW"; $resn_exch{"1H5  CLW"}=" H51 CLW"; $resn_exch{"1HN1 CLW"}="HN11 CLW";
$resn_exch{"1H6  CLY"}=" H61 CLY"; $resn_exch{"2H6  CLY"}=" H62 CLY"; $resn_exch{"3H6  CLY"}=" H63 CLY";
$resn_exch{"1H9  CLY"}=" H91 CLY"; $resn_exch{"2H9  CLY"}=" H92 CLY"; $resn_exch{"3H9  CLY"}=" H93 CLY";
$resn_exch{"1H12 CLY"}="H121 CLY"; $resn_exch{"2H12 CLY"}="H122 CLY"; $resn_exch{"1H14 CLY"}="H141 CLY";
$resn_exch{"2H14 CLY"}="H142 CLY"; $resn_exch{"1H15 CLY"}="H151 CLY"; $resn_exch{"2H15 CLY"}="H152 CLY";
$resn_exch{"3H15 CLY"}="H153 CLY"; $resn_exch{"1H16 CLY"}="H161 CLY"; $resn_exch{"2H16 CLY"}="H162 CLY";
$resn_exch{"1H17 CLY"}="H171 CLY"; $resn_exch{"2H17 CLY"}="H172 CLY"; $resn_exch{"1H18 CLY"}="H181 CLY";
$resn_exch{"2H18 CLY"}="H182 CLY"; $resn_exch{"3H18 CLY"}="H183 CLY"; $resn_exch{"1H5  CLZ"}=" H51 CLZ";
$resn_exch{"2H5  CLZ"}=" H52 CLZ"; $resn_exch{"1H6  CLZ"}=" H61 CLZ"; $resn_exch{"2H6  CLZ"}=" H62 CLZ";
$resn_exch{"1H13 CLZ"}="H131 CLZ"; $resn_exch{"2H13 CLZ"}="H132 CLZ"; $resn_exch{"1H2   CM"}=" H21  CM";
$resn_exch{"2H2   CM"}=" H22  CM"; $resn_exch{"3H2   CM"}=" H23  CM"; $resn_exch{"1H5* CM0"}="H5'1 CM0";
$resn_exch{"2H5* CM0"}="H5'2 CM0"; $resn_exch{" H4* CM0"}=" H4' CM0"; $resn_exch{" H3* CM0"}=" H3' CM0";
$resn_exch{" H1* CM0"}=" H1' CM0"; $resn_exch{" H2* CM0"}=" H2' CM0"; $resn_exch{"1H7C CM0"}="H7C1 CM0";
$resn_exch{"2H7C CM0"}="H7C2 CM0"; $resn_exch{"1H13 CM1"}="H131 CM1"; $resn_exch{"2H13 CM1"}="H132 CM1";
$resn_exch{"1H23 CM1"}="H231 CM1"; $resn_exch{"2H23 CM1"}="H232 CM1"; $resn_exch{"3H23 CM1"}="H233 CM1";
$resn_exch{"1H25 CM1"}="H251 CM1"; $resn_exch{"2H25 CM1"}="H252 CM1"; $resn_exch{"1H31 CM1"}="H311 CM1";
$resn_exch{"2H31 CM1"}="H312 CM1"; $resn_exch{"1H32 CM1"}="H321 CM1"; $resn_exch{"2H32 CM1"}="H322 CM1";
$resn_exch{"1H33 CM1"}="H331 CM1"; $resn_exch{"2H33 CM1"}="H332 CM1"; $resn_exch{"1H34 CM1"}="H341 CM1";
$resn_exch{"2H34 CM1"}="H342 CM1"; $resn_exch{"1H39 CM1"}="H391 CM1"; $resn_exch{"2H39 CM1"}="H392 CM1";
$resn_exch{"1H40 CM1"}="H401 CM1"; $resn_exch{"2H40 CM1"}="H402 CM1"; $resn_exch{"1H41 CM1"}="H411 CM1";
$resn_exch{"2H41 CM1"}="H412 CM1"; $resn_exch{"1H42 CM1"}="H421 CM1"; $resn_exch{"2H42 CM1"}="H422 CM1";
$resn_exch{"1H49 CM1"}="H491 CM1"; $resn_exch{"2H49 CM1"}="H492 CM1"; $resn_exch{"1H50 CM1"}="H501 CM1";
$resn_exch{"2H50 CM1"}="H502 CM1"; $resn_exch{"1H51 CM1"}="H511 CM1"; $resn_exch{"2H51 CM1"}="H512 CM1";
$resn_exch{"1H52 CM1"}="H521 CM1"; $resn_exch{"2H52 CM1"}="H522 CM1"; $resn_exch{"1H53 CM1"}="H531 CM1";
$resn_exch{"2H53 CM1"}="H532 CM1"; $resn_exch{"1H54 CM1"}="H541 CM1"; $resn_exch{"2H54 CM1"}="H542 CM1";
$resn_exch{"1H13 CM2"}="H131 CM2"; $resn_exch{"2H13 CM2"}="H132 CM2"; $resn_exch{"1H22 CM2"}="H221 CM2";
$resn_exch{"2H22 CM2"}="H222 CM2"; $resn_exch{"1H25 CM2"}="H251 CM2"; $resn_exch{"2H25 CM2"}="H252 CM2";
$resn_exch{"1H31 CM2"}="H311 CM2"; $resn_exch{"2H31 CM2"}="H312 CM2"; $resn_exch{"1H32 CM2"}="H321 CM2";
$resn_exch{"2H32 CM2"}="H322 CM2"; $resn_exch{"1H33 CM2"}="H331 CM2"; $resn_exch{"2H33 CM2"}="H332 CM2";
$resn_exch{"1H34 CM2"}="H341 CM2"; $resn_exch{"2H34 CM2"}="H342 CM2"; $resn_exch{"1H39 CM2"}="H391 CM2";
$resn_exch{"2H39 CM2"}="H392 CM2"; $resn_exch{"1H40 CM2"}="H401 CM2"; $resn_exch{"2H40 CM2"}="H402 CM2";
$resn_exch{"1H41 CM2"}="H411 CM2"; $resn_exch{"2H41 CM2"}="H412 CM2"; $resn_exch{"1H42 CM2"}="H421 CM2";
$resn_exch{"2H42 CM2"}="H422 CM2"; $resn_exch{"1H49 CM2"}="H491 CM2"; $resn_exch{"2H49 CM2"}="H492 CM2";
$resn_exch{"1H50 CM2"}="H501 CM2"; $resn_exch{"2H50 CM2"}="H502 CM2"; $resn_exch{"1H51 CM2"}="H511 CM2";
$resn_exch{"2H51 CM2"}="H512 CM2"; $resn_exch{"1H52 CM2"}="H521 CM2"; $resn_exch{"2H52 CM2"}="H522 CM2";
$resn_exch{"1H53 CM2"}="H531 CM2"; $resn_exch{"2H53 CM2"}="H532 CM2"; $resn_exch{"1H54 CM2"}="H541 CM2";
$resn_exch{"2H54 CM2"}="H542 CM2"; $resn_exch{"1H13 CM3"}="H131 CM3"; $resn_exch{"2H13 CM3"}="H132 CM3";
$resn_exch{"3H13 CM3"}="H133 CM3"; $resn_exch{"1H28 CM3"}="H281 CM3"; $resn_exch{"2H28 CM3"}="H282 CM3";
$resn_exch{"1H29 CM3"}="H291 CM3"; $resn_exch{"2H29 CM3"}="H292 CM3"; $resn_exch{"1H31 CM3"}="H311 CM3";
$resn_exch{"2H31 CM3"}="H312 CM3"; $resn_exch{"1H32 CM3"}="H321 CM3"; $resn_exch{"2H32 CM3"}="H322 CM3";
$resn_exch{"1H33 CM3"}="H331 CM3"; $resn_exch{"2H33 CM3"}="H332 CM3"; $resn_exch{"1H34 CM3"}="H341 CM3";
$resn_exch{"2H34 CM3"}="H342 CM3"; $resn_exch{"1H35 CM3"}="H351 CM3"; $resn_exch{"2H35 CM3"}="H352 CM3";
$resn_exch{"1H12 CM4"}="H121 CM4"; $resn_exch{"2H12 CM4"}="H122 CM4"; $resn_exch{"3H12 CM4"}="H123 CM4";
$resn_exch{"1H27 CM4"}="H271 CM4"; $resn_exch{"2H27 CM4"}="H272 CM4"; $resn_exch{"1H28 CM4"}="H281 CM4";
$resn_exch{"2H28 CM4"}="H282 CM4"; $resn_exch{"1H30 CM4"}="H301 CM4"; $resn_exch{"2H30 CM4"}="H302 CM4";
$resn_exch{"1H31 CM4"}="H311 CM4"; $resn_exch{"2H31 CM4"}="H312 CM4"; $resn_exch{"1H32 CM4"}="H321 CM4";
$resn_exch{"2H32 CM4"}="H322 CM4"; $resn_exch{"1H33 CM4"}="H331 CM4"; $resn_exch{"2H33 CM4"}="H332 CM4";
$resn_exch{"1H7  CM5"}=" H71 CM5"; $resn_exch{"2H7  CM5"}=" H72 CM5"; $resn_exch{"1H8  CM5"}=" H81 CM5";
$resn_exch{"2H8  CM5"}=" H82 CM5"; $resn_exch{"1H9  CM5"}=" H91 CM5"; $resn_exch{"2H9  CM5"}=" H92 CM5";
$resn_exch{"1H10 CM5"}="H101 CM5"; $resn_exch{"2H10 CM5"}="H102 CM5"; $resn_exch{"1H11 CM5"}="H111 CM5";
$resn_exch{"2H11 CM5"}="H112 CM5"; $resn_exch{"1H5  CM5"}=" H51 CM5"; $resn_exch{"2H5  CM5"}=" H52 CM5";
$resn_exch{"1H4  CM5"}=" H41 CM5"; $resn_exch{"2H4  CM5"}=" H42 CM5"; $resn_exch{"1H3  CM5"}="H31A CM5";
$resn_exch{"2H3  CM5"}="H32A CM5"; $resn_exch{"1H2  CM5"}="H21A CM5"; $resn_exch{"2H2  CM5"}="H22A CM5";
$resn_exch{"1H1  CM5"}=" H11 CM5"; $resn_exch{"2H1  CM5"}=" H12 CM5"; $resn_exch{"1H19 CM5"}="H191 CM5";
$resn_exch{"2H19 CM5"}="H192 CM5"; $resn_exch{"1H30 CM5"}="H301 CM5"; $resn_exch{"2H30 CM5"}="H302 CM5";
$resn_exch{"1HB  CMA"}=" HB1 CMA"; $resn_exch{"2HB  CMA"}=" HB2 CMA"; $resn_exch{"1H1  CMA"}=" H11 CMA";
$resn_exch{"2H1  CMA"}=" H12 CMA"; $resn_exch{"1H2  CMA"}=" H21 CMA"; $resn_exch{"2H2  CMA"}=" H22 CMA";
$resn_exch{"1HG  CMA"}=" HG1 CMA"; $resn_exch{"2HG  CMA"}=" HG2 CMA"; $resn_exch{"1HD  CMA"}=" HD1 CMA";
$resn_exch{"2HD  CMA"}=" HD2 CMA"; $resn_exch{"1HH1 CMA"}="HH11 CMA"; $resn_exch{"2HH1 CMA"}="HH12 CMA";
$resn_exch{"1HH2 CMA"}="HH21 CMA"; $resn_exch{"1H10 CMB"}="H101 CMB"; $resn_exch{"2H10 CMB"}="H102 CMB";
$resn_exch{"3H10 CMB"}="H103 CMB"; $resn_exch{"1H13 CMB"}="H131 CMB"; $resn_exch{"2H13 CMB"}="H132 CMB";
$resn_exch{"1H17 CMB"}="H171 CMB"; $resn_exch{"2H17 CMB"}="H172 CMB"; $resn_exch{"18H1 CMB"}="H118 CMB";
$resn_exch{"1H21 CMB"}="H211 CMB"; $resn_exch{"1H22 CMB"}="H221 CMB"; $resn_exch{"1H23 CMB"}="H231 CMB";
$resn_exch{"24H2 CMB"}="H224 CMB"; $resn_exch{"1H25 CMB"}="H251 CMB"; $resn_exch{"2H25 CMB"}="H252 CMB";
$resn_exch{"1H28 CMB"}="H281 CMB"; $resn_exch{"1H53 CMB"}="H531 CMB"; $resn_exch{"2H53 CMB"}="H532 CMB";
$resn_exch{" AH2 CMC"}=" H2A CMC"; $resn_exch{"AH61 CMC"}="H61A CMC"; $resn_exch{"AH62 CMC"}="H62A CMC";
$resn_exch{" AH8 CMC"}=" H8A CMC"; $resn_exch{"AH1* CMC"}=" H1B CMC"; $resn_exch{"AH2* CMC"}=" H2B CMC";
$resn_exch{"AHO2 CMC"}="HO2A CMC"; $resn_exch{"AH3* CMC"}=" H3B CMC"; $resn_exch{"8HOA CMC"}="HOA8 CMC";
$resn_exch{"9HOA CMC"}="HOA9 CMC"; $resn_exch{"AH4* CMC"}=" H4B CMC"; $resn_exch{"AH51 CMC"}="H51A CMC";
$resn_exch{"AH52 CMC"}="H52A CMC"; $resn_exch{"2HOA CMC"}="HOA2 CMC"; $resn_exch{"5HOA CMC"}="HOA5 CMC";
$resn_exch{"1H12 CMC"}="H121 CMC"; $resn_exch{"2H12 CMC"}="H122 CMC"; $resn_exch{"1H13 CMC"}="H131 CMC";
$resn_exch{"2H13 CMC"}="H132 CMC"; $resn_exch{"3H13 CMC"}="H133 CMC"; $resn_exch{"1H14 CMC"}="H141 CMC";
$resn_exch{"2H14 CMC"}="H142 CMC"; $resn_exch{"3H14 CMC"}="H143 CMC"; $resn_exch{"1H7  CMC"}=" H71 CMC";
$resn_exch{"2H7  CMC"}=" H72 CMC"; $resn_exch{"1H6  CMC"}=" H61 CMC"; $resn_exch{"2H6  CMC"}=" H62 CMC";
$resn_exch{"1H3  CMC"}=" H31 CMC"; $resn_exch{"2H3  CMC"}=" H32 CMC"; $resn_exch{"1H2  CMC"}=" H21 CMC";
$resn_exch{"2H2  CMC"}=" H22 CMC"; $resn_exch{"2HO2 CMC"}="HO22 CMC"; $resn_exch{"1H11 CMD"}="H111 CMD";
$resn_exch{"2H11 CMD"}="H112 CMD"; $resn_exch{"2HO1 CMD"}="HO12 CMD"; $resn_exch{"1H14 CMD"}="H141 CMD";
$resn_exch{"2H14 CMD"}="H142 CMD"; $resn_exch{"4HO1 CMD"}="HO14 CMD"; $resn_exch{"1H15 CMD"}="H151 CMD";
$resn_exch{"2H15 CMD"}="H152 CMD"; $resn_exch{"7HO1 CMD"}="HO17 CMD"; $resn_exch{"1H21 CMD"}="H211 CMD";
$resn_exch{"2H21 CMD"}="H212 CMD"; $resn_exch{"3H21 CMD"}="H213 CMD"; $resn_exch{" H1* CMD"}=" H1' CMD";
$resn_exch{"1H2* CMD"}="H2'1 CMD"; $resn_exch{"2H2* CMD"}="H2'2 CMD"; $resn_exch{" H3* CMD"}=" H3' CMD";
$resn_exch{" H4* CMD"}=" H4' CMD"; $resn_exch{"*HO4 CMD"}="HO4' CMD"; $resn_exch{" H5* CMD"}=" H5' CMD";
$resn_exch{"1H6* CMD"}="H6'1 CMD"; $resn_exch{"2H6* CMD"}="H6'2 CMD"; $resn_exch{"3H6* CMD"}="H6'3 CMD";
$resn_exch{"1H23 CMD"}="H231 CMD"; $resn_exch{"2H23 CMD"}="H232 CMD"; $resn_exch{"1H63 CMD"}="H631 CMD";
$resn_exch{"2H63 CMD"}="H632 CMD"; $resn_exch{"1H53 CMD"}="H531 CMD"; $resn_exch{"2H53 CMD"}="H532 CMD";
$resn_exch{"1HN  CME"}=" H   CME"; $resn_exch{"2HN  CME"}=" H2  CME"; $resn_exch{"1HB  CME"}=" HB2 CME";
$resn_exch{"2HB  CME"}=" HB3 CME"; $resn_exch{"1HE  CME"}=" HE2 CME"; $resn_exch{"2HE  CME"}=" HE3 CME";
$resn_exch{"1HZ  CME"}=" HZ2 CME"; $resn_exch{"2HZ  CME"}=" HZ3 CME"; $resn_exch{" HO  CME"}=" HH  CME";
$resn_exch{"1H24 CMF"}="H241 CMF"; $resn_exch{"2H24 CMF"}="H242 CMF"; $resn_exch{"1H26 CMF"}="H261 CMF";
$resn_exch{"2H26 CMF"}="H262 CMF"; $resn_exch{"1H27 CMF"}="H271 CMF"; $resn_exch{"2H27 CMF"}="H272 CMF";
$resn_exch{"1H25 CMF"}="H251 CMF"; $resn_exch{"2H25 CMF"}="H252 CMF"; $resn_exch{"1H22 CMF"}="H221 CMF";
$resn_exch{"2H22 CMF"}="H222 CMF"; $resn_exch{"1H17 CMF"}="H171 CMF"; $resn_exch{"2H17 CMF"}="H172 CMF";
$resn_exch{"1H20 CMF"}="H201 CMF"; $resn_exch{"2H20 CMF"}="H202 CMF"; $resn_exch{"1H21 CMF"}="H211 CMF";
$resn_exch{"2H21 CMF"}="H212 CMF"; $resn_exch{"1H19 CMF"}="H191 CMF"; $resn_exch{"2H19 CMF"}="H192 CMF";
$resn_exch{"1H18 CMF"}="H181 CMF"; $resn_exch{"2H18 CMF"}="H182 CMF"; $resn_exch{"2H9  CMG"}=" H92 CMG";
$resn_exch{"1H9  CMG"}=" H91 CMG"; $resn_exch{"2H2  CMG"}=" H22 CMG"; $resn_exch{"1H2  CMG"}=" H21 CMG";
$resn_exch{"2H11 CMG"}="H112 CMG"; $resn_exch{"1H11 CMG"}="H111 CMG"; $resn_exch{"2H12 CMG"}="H122 CMG";
$resn_exch{"1H12 CMG"}="H121 CMG"; $resn_exch{"2H13 CMG"}="H132 CMG"; $resn_exch{"1H13 CMG"}="H131 CMG";
$resn_exch{"2H14 CMG"}="H142 CMG"; $resn_exch{"1H14 CMG"}="H141 CMG"; $resn_exch{"2H15 CMG"}="H152 CMG";
$resn_exch{"1H15 CMG"}="H151 CMG"; $resn_exch{"1HN  CMH"}=" H   CMH"; $resn_exch{"2HN  CMH"}=" H2  CMH";
$resn_exch{"1HBC CMH"}=" HB2 CMH"; $resn_exch{"2HBC CMH"}=" HB3 CMH"; $resn_exch{"1HM  CMH"}=" HM1 CMH";
$resn_exch{"2HM  CMH"}=" HM2 CMH"; $resn_exch{"3HM  CMH"}=" HM3 CMH"; $resn_exch{"1H10 CMI"}="H101 CMI";
$resn_exch{"2H10 CMI"}="H102 CMI"; $resn_exch{"3H10 CMI"}="H103 CMI"; $resn_exch{"1H13 CMI"}="H131 CMI";
$resn_exch{"2H13 CMI"}="H132 CMI"; $resn_exch{"1H17 CMI"}="H171 CMI"; $resn_exch{"2H17 CMI"}="H172 CMI";
$resn_exch{"1H32 CMI"}="H321 CMI"; $resn_exch{"2H32 CMI"}="H322 CMI"; $resn_exch{"1H33 CMI"}="H331 CMI";
$resn_exch{"2H33 CMI"}="H332 CMI"; $resn_exch{"3H33 CMI"}="H333 CMI"; $resn_exch{"1H37 CMI"}="H371 CMI";
$resn_exch{"2H37 CMI"}="H372 CMI"; $resn_exch{"3H37 CMI"}="H373 CMI"; $resn_exch{"1H4C CMK"}="H4C1 CMK";
$resn_exch{"2H4C CMK"}="H4C2 CMK"; $resn_exch{" H1* CMK"}=" H1' CMK"; $resn_exch{" H2* CMK"}=" H2' CMK";
$resn_exch{" H3* CMK"}=" H3' CMK"; $resn_exch{" H4* CMK"}=" H4' CMK"; $resn_exch{"1H5* CMK"}="H5'1 CMK";
$resn_exch{"2H5* CMK"}="H5'2 CMK"; $resn_exch{"1H3C CMK"}="H3C1 CMK"; $resn_exch{"2H3C CMK"}="H3C2 CMK";
$resn_exch{"1H8C CMK"}="H8C1 CMK"; $resn_exch{"2H8C CMK"}="H8C2 CMK"; $resn_exch{"1H15 CMM"}="H151 CMM";
$resn_exch{"2H15 CMM"}="H152 CMM"; $resn_exch{"1H16 CMM"}="H161 CMM"; $resn_exch{"2H16 CMM"}="H162 CMM";
$resn_exch{"1H23 CMM"}="H231 CMM"; $resn_exch{"2H23 CMM"}="H232 CMM"; $resn_exch{"1H27 CMM"}="H271 CMM";
$resn_exch{"2H27 CMM"}="H272 CMM"; $resn_exch{"3H27 CMM"}="H273 CMM"; $resn_exch{"1H31 CMM"}="H311 CMM";
$resn_exch{"2H31 CMM"}="H312 CMM"; $resn_exch{"1H33 CMM"}="H331 CMM"; $resn_exch{"2H33 CMM"}="H332 CMM";
$resn_exch{"1H37 CMM"}="H371 CMM"; $resn_exch{"2H37 CMM"}="H372 CMM"; $resn_exch{"1H36 CMM"}="H361 CMM";
$resn_exch{"2H36 CMM"}="H362 CMM"; $resn_exch{"1H35 CMM"}="H351 CMM"; $resn_exch{"2H35 CMM"}="H352 CMM";
$resn_exch{"1H40 CMM"}="H401 CMM"; $resn_exch{"2H40 CMM"}="H402 CMM"; $resn_exch{"2H6  CMN"}=" H62 CMN";
$resn_exch{"1H6  CMN"}=" H61 CMN"; $resn_exch{"2H1  CMN"}=" H12 CMN"; $resn_exch{"1H1  CMN"}=" H11 CMN";
$resn_exch{"1H2O CMN"}="H2O1 CMN"; $resn_exch{"1H3O CMN"}="H3O1 CMN"; $resn_exch{"1H4O CMN"}="H4O1 CMN";
$resn_exch{"2HOP CMP"}="HOP2 CMP"; $resn_exch{"1H5* CMP"}="H5'1 CMP"; $resn_exch{"2H5* CMP"}="H5'2 CMP";
$resn_exch{" H4* CMP"}=" H4' CMP"; $resn_exch{" H3* CMP"}=" H3' CMP"; $resn_exch{" H2* CMP"}=" H2' CMP";
$resn_exch{"*HO2 CMP"}="HO2' CMP"; $resn_exch{" H1* CMP"}=" H1' CMP"; $resn_exch{"1HN6 CMP"}="HN61 CMP";
$resn_exch{"2HN6 CMP"}="HN62 CMP"; $resn_exch{"1H2  CMQ"}="H21A CMQ"; $resn_exch{"2H2  CMQ"}="H22A CMQ";
$resn_exch{"1H11 CMQ"}="H111 CMQ"; $resn_exch{"2H11 CMQ"}="H112 CMQ"; $resn_exch{"1H13 CMQ"}="H131 CMQ";
$resn_exch{"2H13 CMQ"}="H132 CMQ"; $resn_exch{"3H13 CMQ"}="H133 CMQ"; $resn_exch{"1H14 CMQ"}="H141 CMQ";
$resn_exch{"2H14 CMQ"}="H142 CMQ"; $resn_exch{"3H14 CMQ"}="H143 CMQ"; $resn_exch{"1H24 CMQ"}="H241 CMQ";
$resn_exch{"2H24 CMQ"}="H242 CMQ"; $resn_exch{"3H24 CMQ"}="H243 CMQ"; $resn_exch{"1H17 CMQ"}="H171 CMQ";
$resn_exch{"2H17 CMQ"}="H172 CMQ"; $resn_exch{"1HMP CMR"}="HMP1 CMR"; $resn_exch{"2HMP CMR"}="HMP2 CMR";
$resn_exch{"3HMP CMR"}="HMP3 CMR"; $resn_exch{"1H5* CMR"}=" H5' CMR"; $resn_exch{"2H5* CMR"}="H5'' CMR";
$resn_exch{" H4* CMR"}=" H4' CMR"; $resn_exch{" H1* CMR"}=" H1' CMR"; $resn_exch{"1H4  CMR"}=" H41 CMR";
$resn_exch{"2H4  CMR"}=" H42 CMR"; $resn_exch{" H3* CMR"}=" H3' CMR"; $resn_exch{"1H2* CMR"}=" H2' CMR";
$resn_exch{"2H2* CMR"}="H2'' CMR"; $resn_exch{"H3T  CMR"}="HO3' CMR"; $resn_exch{"3HOP CMR"}="HOP3 CMR";
$resn_exch{"1H2  CMS"}=" H21 CMS"; $resn_exch{"2H2  CMS"}=" H22 CMS"; $resn_exch{"3H2  CMS"}=" H23 CMS";
$resn_exch{"1H3  CMS"}=" H31 CMS"; $resn_exch{"2H3  CMS"}=" H32 CMS"; $resn_exch{"1HN1 CMS"}="HN11 CMS";
$resn_exch{"2HN1 CMS"}="HN12 CMS"; $resn_exch{" HN2 CMT"}=" H2  CMT"; $resn_exch{"1HB  CMT"}=" HB2 CMT";
$resn_exch{"2HB  CMT"}=" HB3 CMT"; $resn_exch{"1H1  CMT"}=" H11 CMT"; $resn_exch{"2H1  CMT"}=" H12 CMT";
$resn_exch{"3H1  CMT"}=" H13 CMT"; $resn_exch{"1H4C CMU"}="H4C1 CMU"; $resn_exch{"2H4C CMU"}="H4C2 CMU";
$resn_exch{"1H6C CMU"}="H6C1 CMU"; $resn_exch{"2H6C CMU"}="H6C2 CMU"; $resn_exch{"1H7C CMU"}="H7C1 CMU";
$resn_exch{"2H7C CMU"}="H7C2 CMU"; $resn_exch{"1H8C CMU"}="H8C1 CMU"; $resn_exch{"2H8C CMU"}="H8C2 CMU";
$resn_exch{" AH2 CMX"}=" H2A CMX"; $resn_exch{"AH61 CMX"}="H61A CMX"; $resn_exch{"AH62 CMX"}="H62A CMX";
$resn_exch{" AH8 CMX"}=" H8A CMX"; $resn_exch{"AH1* CMX"}=" H1B CMX"; $resn_exch{"AH2* CMX"}=" H2B CMX";
$resn_exch{"AHO2 CMX"}="HO2A CMX"; $resn_exch{"AH3* CMX"}=" H3B CMX"; $resn_exch{"8HOA CMX"}="HOA8 CMX";
$resn_exch{"9HOA CMX"}="HOA9 CMX"; $resn_exch{"AH4* CMX"}=" H4B CMX"; $resn_exch{"AH51 CMX"}="H51A CMX";
$resn_exch{"AH52 CMX"}="H52A CMX"; $resn_exch{"2HOA CMX"}="HOA2 CMX"; $resn_exch{"5HOA CMX"}="HOA5 CMX";
$resn_exch{"1H12 CMX"}="H121 CMX"; $resn_exch{"2H12 CMX"}="H122 CMX"; $resn_exch{"1H13 CMX"}="H131 CMX";
$resn_exch{"2H13 CMX"}="H132 CMX"; $resn_exch{"3H13 CMX"}="H133 CMX"; $resn_exch{"1H14 CMX"}="H141 CMX";
$resn_exch{"2H14 CMX"}="H142 CMX"; $resn_exch{"3H14 CMX"}="H143 CMX"; $resn_exch{"1H7  CMX"}=" H71 CMX";
$resn_exch{"2H7  CMX"}=" H72 CMX"; $resn_exch{"1H6  CMX"}=" H61 CMX"; $resn_exch{"2H6  CMX"}=" H62 CMX";
$resn_exch{"1H3  CMX"}=" H31 CMX"; $resn_exch{"2H3  CMX"}=" H32 CMX"; $resn_exch{"1H2  CMX"}=" H21 CMX";
$resn_exch{"2H2  CMX"}=" H22 CMX"; $resn_exch{"1H1  CMX"}=" H11 CMX"; $resn_exch{"2H1  CMX"}=" H12 CMX";
$resn_exch{"1H2Y CMY"}="H2Y1 CMY"; $resn_exch{"2H2Y CMY"}="H2Y2 CMY"; $resn_exch{"1H3Y CMY"}="H3Y1 CMY";
$resn_exch{"2H3Y CMY"}="H3Y2 CMY"; $resn_exch{"1H4Y CMY"}="H4Y1 CMY"; $resn_exch{"2H4Y CMY"}="H4Y2 CMY";
$resn_exch{"1H5Y CMY"}="H5Y1 CMY"; $resn_exch{"2H5Y CMY"}="H5Y2 CMY"; $resn_exch{"1H6Y CMY"}="H6Y1 CMY";
$resn_exch{"2H6Y CMY"}="H6Y2 CMY"; $resn_exch{"1H7Y CMY"}="H7Y1 CMY"; $resn_exch{"2H7Y CMY"}="H7Y2 CMY";
$resn_exch{"1H1  CMY"}=" H11 CMY"; $resn_exch{"2H1  CMY"}=" H12 CMY"; $resn_exch{"1H1  CMZ"}=" H11 CMZ";
$resn_exch{"2H1  CMZ"}=" H12 CMZ"; $resn_exch{"3H1  CMZ"}=" H13 CMZ"; $resn_exch{"1H6  CMZ"}=" H61 CMZ";
$resn_exch{"2H6  CMZ"}=" H62 CMZ"; $resn_exch{"3H6  CMZ"}=" H63 CMZ"; $resn_exch{"1H9  CMZ"}=" H91 CMZ";
$resn_exch{"2H9  CMZ"}=" H92 CMZ"; $resn_exch{"1H12 CMZ"}="H121 CMZ"; $resn_exch{"2H12 CMZ"}="H122 CMZ";
$resn_exch{"1H14 CMZ"}="H141 CMZ"; $resn_exch{"2H14 CMZ"}="H142 CMZ"; $resn_exch{"1H15 CMZ"}="H151 CMZ";
$resn_exch{"2H15 CMZ"}="H152 CMZ"; $resn_exch{"1H17 CMZ"}="H171 CMZ"; $resn_exch{"2H17 CMZ"}="H172 CMZ";
$resn_exch{"1H18 CMZ"}="H181 CMZ"; $resn_exch{"2H18 CMZ"}="H182 CMZ"; $resn_exch{"1H13 CN2"}="H131 CN2";
$resn_exch{"2H13 CN2"}="H132 CN2"; $resn_exch{"1H10 CN2"}="H101 CN2"; $resn_exch{"2H10 CN2"}="H102 CN2";
$resn_exch{"1H9  CN2"}=" H91 CN2"; $resn_exch{"2H9  CN2"}=" H92 CN2"; $resn_exch{"1H2  CN2"}=" H21 CN2";
$resn_exch{"2H2  CN2"}=" H22 CN2"; $resn_exch{"3H2  CN2"}=" H23 CN2"; $resn_exch{"1H4  CN2"}=" H41 CN2";
$resn_exch{"2H4  CN2"}=" H42 CN2"; $resn_exch{"3H4  CN2"}=" H43 CN2"; $resn_exch{"1H6  CN2"}=" H61 CN2";
$resn_exch{"2H6  CN2"}=" H62 CN2"; $resn_exch{"3H6  CN2"}=" H63 CN2"; $resn_exch{"1H18 CN2"}="H181 CN2";
$resn_exch{"2H18 CN2"}="H182 CN2"; $resn_exch{"3H18 CN2"}="H183 CN2"; $resn_exch{"2HOA CNA"}="HOA2 CNA";
$resn_exch{"AH51 CNA"}="H51A CNA"; $resn_exch{"AH52 CNA"}="H52A CNA"; $resn_exch{"AH4* CNA"}=" H4B CNA";
$resn_exch{"AH3* CNA"}=" H3B CNA"; $resn_exch{"AHO3 CNA"}="HO3A CNA"; $resn_exch{"AH2* CNA"}=" H2B CNA";
$resn_exch{"AHO2 CNA"}="HO2A CNA"; $resn_exch{"AH1* CNA"}=" H1B CNA"; $resn_exch{" AH8 CNA"}=" H8A CNA";
$resn_exch{"AH61 CNA"}="H61A CNA"; $resn_exch{"AH62 CNA"}="H62A CNA"; $resn_exch{" AH2 CNA"}=" H2A CNA";
$resn_exch{"NH51 CNA"}="H51N CNA"; $resn_exch{"NH52 CNA"}="H52N CNA"; $resn_exch{"NH4* CNA"}=" H4D CNA";
$resn_exch{"1H4* CNA"}="H4'1 CNA"; $resn_exch{"2H4* CNA"}="H4'2 CNA"; $resn_exch{"NH3* CNA"}=" H3D CNA";
$resn_exch{"NHO3 CNA"}="HO3N CNA"; $resn_exch{"NH2* CNA"}=" H2D CNA"; $resn_exch{"NHO2 CNA"}="HO2N CNA";
$resn_exch{"NH1* CNA"}=" H1D CNA"; $resn_exch{" NH2 CNA"}=" H2N CNA"; $resn_exch{"NH71 CNA"}="H71N CNA";
$resn_exch{"NH72 CNA"}="H72N CNA"; $resn_exch{" NH4 CNA"}=" H4N CNA"; $resn_exch{" NH5 CNA"}=" H5N CNA";
$resn_exch{" NH6 CNA"}=" H6N CNA"; $resn_exch{"1H20 CNC"}="H201 CNC"; $resn_exch{"2H20 CNC"}="H202 CNC";
$resn_exch{"3H20 CNC"}="H203 CNC"; $resn_exch{"1H25 CNC"}="H251 CNC"; $resn_exch{"2H25 CNC"}="H252 CNC";
$resn_exch{"3H25 CNC"}="H253 CNC"; $resn_exch{"1H26 CNC"}="H261 CNC"; $resn_exch{"2H26 CNC"}="H262 CNC";
$resn_exch{"1H29 CNC"}="H291 CNC"; $resn_exch{"2H29 CNC"}="H292 CNC"; $resn_exch{"1H30 CNC"}="H301 CNC";
$resn_exch{"2H30 CNC"}="H302 CNC"; $resn_exch{"1H31 CNC"}="H311 CNC"; $resn_exch{"2H31 CNC"}="H312 CNC";
$resn_exch{"1H33 CNC"}="H331 CNC"; $resn_exch{"2H33 CNC"}="H332 CNC"; $resn_exch{"1H35 CNC"}="H351 CNC";
$resn_exch{"2H35 CNC"}="H352 CNC"; $resn_exch{"3H35 CNC"}="H353 CNC"; $resn_exch{"1H36 CNC"}="H361 CNC";
$resn_exch{"2H36 CNC"}="H362 CNC"; $resn_exch{"3H36 CNC"}="H363 CNC"; $resn_exch{"1H37 CNC"}="H371 CNC";
$resn_exch{"2H37 CNC"}="H372 CNC"; $resn_exch{"1H40 CNC"}="H401 CNC"; $resn_exch{"2H40 CNC"}="H402 CNC";
$resn_exch{"1H41 CNC"}="H411 CNC"; $resn_exch{"2H41 CNC"}="H412 CNC"; $resn_exch{"1H42 CNC"}="H421 CNC";
$resn_exch{"2H42 CNC"}="H422 CNC"; $resn_exch{"1H45 CNC"}="H451 CNC"; $resn_exch{"2H45 CNC"}="H452 CNC";
$resn_exch{"1H46 CNC"}="H461 CNC"; $resn_exch{"2H46 CNC"}="H462 CNC"; $resn_exch{"3H46 CNC"}="H463 CNC";
$resn_exch{"1H47 CNC"}="H471 CNC"; $resn_exch{"2H47 CNC"}="H472 CNC"; $resn_exch{"3H47 CNC"}="H473 CNC";
$resn_exch{"1H48 CNC"}="H481 CNC"; $resn_exch{"2H48 CNC"}="H482 CNC"; $resn_exch{"1H49 CNC"}="H491 CNC";
$resn_exch{"2H49 CNC"}="H492 CNC"; $resn_exch{"1H52 CNC"}="H521 CNC"; $resn_exch{"2H52 CNC"}="H522 CNC";
$resn_exch{"1H53 CNC"}="H531 CNC"; $resn_exch{"2H53 CNC"}="H532 CNC"; $resn_exch{"3H53 CNC"}="H533 CNC";
$resn_exch{"1H54 CNC"}="H541 CNC"; $resn_exch{"2H54 CNC"}="H542 CNC"; $resn_exch{"3H54 CNC"}="H543 CNC";
$resn_exch{"1H55 CNC"}="H551 CNC"; $resn_exch{"2H55 CNC"}="H552 CNC"; $resn_exch{"1H56 CNC"}="H561 CNC";
$resn_exch{"2H56 CNC"}="H562 CNC"; $resn_exch{"1H60 CNC"}="H601 CNC"; $resn_exch{"2H60 CNC"}="H602 CNC";
$resn_exch{"1H62 CNC"}="H621 CNC"; $resn_exch{"2H62 CNC"}="H622 CNC"; $resn_exch{"1H1P CNC"}="H1P1 CNC";
$resn_exch{"2H1P CNC"}="H1P2 CNC"; $resn_exch{"1H3P CNC"}="H3P1 CNC"; $resn_exch{"2H3P CNC"}="H3P2 CNC";
$resn_exch{"3H3P CNC"}="H3P3 CNC"; $resn_exch{"5HOP CNC"}="HOP5 CNC"; $resn_exch{"7HOR CNC"}="HOR7 CNC";
$resn_exch{"1H5R CNC"}="H5R1 CNC"; $resn_exch{"2H5R CNC"}="H5R2 CNC"; $resn_exch{"8HOR CNC"}="HOR8 CNC";
$resn_exch{"1HM5 CNC"}="HM51 CNC"; $resn_exch{"2HM5 CNC"}="HM52 CNC"; $resn_exch{"3HM5 CNC"}="HM53 CNC";
$resn_exch{"1HM6 CNC"}="HM61 CNC"; $resn_exch{"2HM6 CNC"}="HM62 CNC"; $resn_exch{"3HM6 CNC"}="HM63 CNC";
$resn_exch{"2HOA CND"}="HOA2 CND"; $resn_exch{"AH51 CND"}="H51A CND"; $resn_exch{"AH52 CND"}="H52A CND";
$resn_exch{"AH4* CND"}=" H4B CND"; $resn_exch{"AH3* CND"}=" H3B CND"; $resn_exch{"AHO3 CND"}="HO3A CND";
$resn_exch{"AH2* CND"}=" H2B CND"; $resn_exch{"AHO2 CND"}="HO2A CND"; $resn_exch{"AH1* CND"}=" H1B CND";
$resn_exch{" AH8 CND"}=" H8A CND"; $resn_exch{"AH61 CND"}="H61A CND"; $resn_exch{"AH62 CND"}="H62A CND";
$resn_exch{" AH2 CND"}=" H2A CND"; $resn_exch{"NH51 CND"}="H51N CND"; $resn_exch{"NH52 CND"}="H52N CND";
$resn_exch{"NH4* CND"}=" H4D CND"; $resn_exch{"NH3* CND"}=" H3D CND"; $resn_exch{"NHO3 CND"}="HO3N CND";
$resn_exch{"NH2* CND"}=" H2D CND"; $resn_exch{"NHO2 CND"}="HO2N CND"; $resn_exch{"NH1* CND"}=" H1D CND";
$resn_exch{" NH2 CND"}=" H2N CND"; $resn_exch{"NH71 CND"}="H71N CND"; $resn_exch{"NH72 CND"}="H72N CND";
$resn_exch{" NH4 CND"}=" H4N CND"; $resn_exch{" NH5 CND"}=" H5N CND"; $resn_exch{" NH6 CND"}=" H6N CND";
$resn_exch{"1H29 CNE"}="H291 CNE"; $resn_exch{"2H29 CNE"}="H292 CNE"; $resn_exch{"3H29 CNE"}="H293 CNE";
$resn_exch{"1H30 CNE"}="H301 CNE"; $resn_exch{"2H30 CNE"}="H302 CNE"; $resn_exch{"1H31 CNE"}="H311 CNE";
$resn_exch{"2H31 CNE"}="H312 CNE"; $resn_exch{"3H31 CNE"}="H313 CNE"; $resn_exch{"1H19 CNE"}="H191 CNE";
$resn_exch{"2H19 CNE"}="H192 CNE"; $resn_exch{"1H17 CNE"}="H171 CNE"; $resn_exch{"2H17 CNE"}="H172 CNE";
$resn_exch{"3H17 CNE"}="H173 CNE"; $resn_exch{"1H16 CNE"}="H161 CNE"; $resn_exch{"2H16 CNE"}="H162 CNE";
$resn_exch{"3H16 CNE"}="H163 CNE"; $resn_exch{"1H11 CNE"}="H111 CNE"; $resn_exch{"2H11 CNE"}="H112 CNE";
$resn_exch{"1H8  CNE"}=" H81 CNE"; $resn_exch{"2H8  CNE"}=" H82 CNE"; $resn_exch{"1H7  CNE"}=" H71 CNE";
$resn_exch{"2H7  CNE"}=" H72 CNE"; $resn_exch{"1HC2 CNH"}="HC21 CNH"; $resn_exch{"2HC2 CNH"}="HC22 CNH";
$resn_exch{"3HC2 CNH"}="HC23 CNH"; $resn_exch{"1HC3 CNH"}="HC31 CNH"; $resn_exch{"2HC3 CNH"}="HC32 CNH";
$resn_exch{"3HC3 CNH"}="HC33 CNH"; $resn_exch{"1H2  CNL"}=" H21 CNL"; $resn_exch{"2H2  CNL"}=" H22 CNL";
$resn_exch{"1H3  CNL"}=" H31 CNL"; $resn_exch{"2H3  CNL"}=" H32 CNL"; $resn_exch{"1H5  CNL"}=" H51 CNL";
$resn_exch{"2H5  CNL"}=" H52 CNL"; $resn_exch{"1H6  CNL"}=" H61 CNL"; $resn_exch{"2H6  CNL"}=" H62 CNL";
$resn_exch{"1H7  CNL"}=" H71 CNL"; $resn_exch{"2H7  CNL"}=" H72 CNL"; $resn_exch{"3H7  CNL"}=" H73 CNL";
$resn_exch{"1H9  CNL"}=" H91 CNL"; $resn_exch{"2H9  CNL"}=" H92 CNL"; $resn_exch{"3H9  CNL"}=" H93 CNL";
$resn_exch{"1H10 CNL"}="H101 CNL"; $resn_exch{"2H10 CNL"}="H102 CNL"; $resn_exch{"3H10 CNL"}="H103 CNL";
$resn_exch{"1HN  CNM"}=" HN1 CNM"; $resn_exch{"2HN  CNM"}=" HN2 CNM"; $resn_exch{"1H2  CNM"}=" H21 CNM";
$resn_exch{"2H2  CNM"}=" H22 CNM"; $resn_exch{"3H2  CNM"}=" H23 CNM"; $resn_exch{"1HN1 CNN"}="HN11 CNN";
$resn_exch{"2HN1 CNN"}="HN12 CNN"; $resn_exch{"1HC5 CNO"}="HC51 CNO"; $resn_exch{"2HC5 CNO"}="HC52 CNO";
$resn_exch{"3CH5 CNO"}="H53C CNO"; $resn_exch{"1HC6 CNO"}="HC61 CNO"; $resn_exch{"2HC6 CNO"}="HC62 CNO";
$resn_exch{"3HC6 CNO"}="HC63 CNO"; $resn_exch{"CH10 CNO"}="H10C CNO"; $resn_exch{"CH11 CNO"}="H11C CNO";
$resn_exch{"CH12 CNO"}="H12C CNO"; $resn_exch{"1H14 CNO"}="H141 CNO"; $resn_exch{"2H14 CNO"}="H142 CNO";
$resn_exch{"NH17 CNO"}="H17N CNO"; $resn_exch{"CH19 CNO"}="H19C CNO"; $resn_exch{"CH20 CNO"}="H20C CNO";
$resn_exch{"CH21 CNO"}="H21C CNO"; $resn_exch{"CH22 CNO"}="H22C CNO"; $resn_exch{"NH24 CNO"}="H24N CNO";
$resn_exch{"1H27 CNO"}="H271 CNO"; $resn_exch{"2H27 CNO"}="H272 CNO"; $resn_exch{"CH29 CNO"}="H29C CNO";
$resn_exch{"CH30 CNO"}="H30C CNO"; $resn_exch{"CH31 CNO"}="H31C CNO"; $resn_exch{"CH32 CNO"}="H32C CNO";
$resn_exch{"1H36 CNO"}="H361 CNO"; $resn_exch{"2H36 CNO"}="H362 CNO"; $resn_exch{"3H36 CNO"}="H363 CNO";
$resn_exch{"1H37 CNO"}="H371 CNO"; $resn_exch{"2H37 CNO"}="H372 CNO"; $resn_exch{"3H37 CNO"}="H373 CNO";
$resn_exch{"OH40 CNO"}="H40O CNO"; $resn_exch{"BHO1 CNP"}="HO1B CNP"; $resn_exch{"1H12 CNP"}="H121 CNP";
$resn_exch{"2H12 CNP"}="H122 CNP"; $resn_exch{"1H14 CNP"}="H141 CNP"; $resn_exch{"2H14 CNP"}="H142 CNP";
$resn_exch{"1H3  CNP"}=" H31 CNP"; $resn_exch{"2H3  CNP"}=" H32 CNP"; $resn_exch{"1H11 CNP"}="H111 CNP";
$resn_exch{"2H11 CNP"}="H112 CNP"; $resn_exch{"3H11 CNP"}="H113 CNP"; $resn_exch{"1H9  CNP"}=" H91 CNP";
$resn_exch{"2H9  CNP"}=" H92 CNP"; $resn_exch{"1H12 CNQ"}="H121 CNQ"; $resn_exch{"2H12 CNQ"}="H122 CNQ";
$resn_exch{"1H3  CNR"}=" H31 CNR"; $resn_exch{"2H3  CNR"}=" H32 CNR"; $resn_exch{"3H3  CNR"}=" H33 CNR";
$resn_exch{"1H24 CNS"}="H241 CNS"; $resn_exch{"2H24 CNS"}="H242 CNS"; $resn_exch{"3H24 CNS"}="H243 CNS";
$resn_exch{"1H23 CNS"}="H231 CNS"; $resn_exch{"2H23 CNS"}="H232 CNS"; $resn_exch{"1H22 CNS"}="H221 CNS";
$resn_exch{"2H22 CNS"}="H222 CNS"; $resn_exch{"1H21 CNS"}="H211 CNS"; $resn_exch{"2H21 CNS"}="H212 CNS";
$resn_exch{"1H20 CNS"}="H201 CNS"; $resn_exch{"2H20 CNS"}="H202 CNS"; $resn_exch{"1H19 CNS"}="H191 CNS";
$resn_exch{"2H19 CNS"}="H192 CNS"; $resn_exch{"1H18 CNS"}="H181 CNS"; $resn_exch{"2H18 CNS"}="H182 CNS";
$resn_exch{"1H17 CNS"}="H171 CNS"; $resn_exch{"2H17 CNS"}="H172 CNS"; $resn_exch{"1H16 CNS"}="H161 CNS";
$resn_exch{"2H16 CNS"}="H162 CNS"; $resn_exch{"1H15 CNS"}="H151 CNS"; $resn_exch{"2H15 CNS"}="H152 CNS";
$resn_exch{"1H14 CNS"}="H141 CNS"; $resn_exch{"2H14 CNS"}="H142 CNS"; $resn_exch{"1H13 CNS"}="H131 CNS";
$resn_exch{"2H13 CNS"}="H132 CNS"; $resn_exch{"1H12 CNS"}="H121 CNS"; $resn_exch{"2H12 CNS"}="H122 CNS";
$resn_exch{"1H11 CNS"}="H111 CNS"; $resn_exch{"2H11 CNS"}="H112 CNS"; $resn_exch{"1H10 CNS"}="H101 CNS";
$resn_exch{"2H10 CNS"}="H102 CNS"; $resn_exch{"1H9  CNS"}=" H91 CNS"; $resn_exch{"2H9  CNS"}=" H92 CNS";
$resn_exch{"1H7  CNS"}=" H71 CNS"; $resn_exch{"2H7  CNS"}=" H72 CNS"; $resn_exch{"1H6  CNS"}=" H61 CNS";
$resn_exch{"2H6  CNS"}=" H62 CNS"; $resn_exch{"1H3  CNS"}=" H31 CNS"; $resn_exch{"2H3  CNS"}=" H32 CNS";
$resn_exch{"1H4  CNS"}=" H41 CNS"; $resn_exch{"2H4  CNS"}=" H42 CNS"; $resn_exch{"1H5  CNS"}=" H51 CNS";
$resn_exch{"2H5  CNS"}=" H52 CNS"; $resn_exch{"1H2  CNS"}=" H21 CNS"; $resn_exch{"2H2  CNS"}=" H22 CNS";
$resn_exch{"1H1  CNS"}=" H11 CNS"; $resn_exch{"2H1  CNS"}=" H12 CNS"; $resn_exch{"3H1  CNS"}=" H13 CNS";
$resn_exch{"1HMC CNT"}="HMC1 CNT"; $resn_exch{"2HMC CNT"}="HMC2 CNT"; $resn_exch{"3HMC CNT"}="HMC3 CNT";
$resn_exch{"1HBC CNT"}="HBC1 CNT"; $resn_exch{"2HBC CNT"}="HBC2 CNT"; $resn_exch{"1H61 CNY"}="H611 CNY";
$resn_exch{"2H61 CNY"}="H612 CNY"; $resn_exch{"1H1  CNY"}="H11A CNY"; $resn_exch{"2H1  CNY"}="H12A CNY";
$resn_exch{"1H12 CNY"}="H121 CNY"; $resn_exch{"2H12 CNY"}="H122 CNY"; $resn_exch{"1H22 CNY"}="H221 CNY";
$resn_exch{"2H22 CNY"}="H222 CNY"; $resn_exch{"1H32 CNY"}="H321 CNY"; $resn_exch{"2H32 CNY"}="H322 CNY";
$resn_exch{"1H53 CNY"}="H531 CNY"; $resn_exch{"2H53 CNY"}="H532 CNY"; $resn_exch{"1H24 CNY"}="H241 CNY";
$resn_exch{"2H24 CNY"}="H242 CNY"; $resn_exch{"1H64 CNY"}="H641 CNY"; $resn_exch{"2H64 CNY"}="H642 CNY";
$resn_exch{"1H2  CNY"}="H21A CNY"; $resn_exch{"2H2  CNY"}=" H22 CNY"; $resn_exch{"1H18 CO1"}="H181 CO1";
$resn_exch{"2H18 CO1"}="H182 CO1"; $resn_exch{"1H4  CO1"}=" H41 CO1"; $resn_exch{"2H4  CO1"}=" H42 CO1";
$resn_exch{"1H8  CO1"}=" H81 CO1"; $resn_exch{"2H8  CO1"}=" H82 CO1"; $resn_exch{"1H11 CO1"}="H111 CO1";
$resn_exch{"2H11 CO1"}="H112 CO1"; $resn_exch{"1H12 CO1"}="H121 CO1"; $resn_exch{"2H12 CO1"}="H122 CO1";
$resn_exch{"3H12 CO1"}="H123 CO1"; $resn_exch{"1H13 CO1"}="H131 CO1"; $resn_exch{"2H13 CO1"}="H132 CO1";
$resn_exch{"1H6  CO1"}=" H61 CO1"; $resn_exch{"2H6  CO1"}=" H62 CO1"; $resn_exch{"1H21 CO1"}="H211 CO1";
$resn_exch{"2H21 CO1"}="H212 CO1"; $resn_exch{"3H21 CO1"}="H213 CO1"; $resn_exch{"1H20 CO1"}="H201 CO1";
$resn_exch{"2H20 CO1"}="H202 CO1"; $resn_exch{"1H23 CO1"}="H231 CO1"; $resn_exch{"2H23 CO1"}="H232 CO1";
$resn_exch{"1H28 CO1"}="H281 CO1"; $resn_exch{"2H28 CO1"}="H282 CO1"; $resn_exch{"3H28 CO1"}="H283 CO1";
$resn_exch{"1H29 CO1"}="H291 CO1"; $resn_exch{"2H29 CO1"}="H292 CO1"; $resn_exch{"3H29 CO1"}="H293 CO1";
$resn_exch{"1H16 CO1"}="H161 CO1"; $resn_exch{"2H16 CO1"}="H162 CO1"; $resn_exch{"3H16 CO1"}="H163 CO1";
$resn_exch{"1H15 CO1"}="H151 CO1"; $resn_exch{"2H15 CO1"}="H152 CO1"; $resn_exch{"1H19 CO1"}="H191 CO1";
$resn_exch{"2H19 CO1"}="H192 CO1"; $resn_exch{"1H2* CO4"}="H2'1 CO4"; $resn_exch{"2H2* CO4"}="H2'2 CO4";
$resn_exch{"1H4* CO4"}="H4'1 CO4"; $resn_exch{"2H4* CO4"}="H4'2 CO4"; $resn_exch{" H7* CO4"}=" H7' CO4";
$resn_exch{"1H8* CO4"}="H8'1 CO4"; $resn_exch{"2H8* CO4"}="H8'2 CO4"; $resn_exch{"1H5B CO4"}="H5B1 CO4";
$resn_exch{"2H5B CO4"}="H5B2 CO4"; $resn_exch{"3H5B CO4"}="H5B3 CO4"; $resn_exch{"1H4B CO4"}="H4B1 CO4";
$resn_exch{"2H4B CO4"}="H4B2 CO4"; $resn_exch{"3H4B CO4"}="H4B3 CO4"; $resn_exch{"1H3B CO4"}="H3B1 CO4";
$resn_exch{"2H3B CO4"}="H3B2 CO4"; $resn_exch{"3H3B CO4"}="H3B3 CO4"; $resn_exch{"1H9* CO4"}="H9'1 CO4";
$resn_exch{"2H9* CO4"}="H9'2 CO4"; $resn_exch{"3H9* CO4"}="H9'3 CO4"; $resn_exch{"1H51 CO4"}="H511 CO4";
$resn_exch{"2H51 CO4"}="H512 CO4"; $resn_exch{"3H51 CO4"}="H513 CO4"; $resn_exch{"1HO1 CO5"}="HO11 CO5";
$resn_exch{"2HO1 CO5"}="HO12 CO5"; $resn_exch{"1HO2 CO5"}="HO21 CO5"; $resn_exch{"2HO2 CO5"}="HO22 CO5";
$resn_exch{"1HO3 CO5"}="HO31 CO5"; $resn_exch{"2HO3 CO5"}="HO32 CO5"; $resn_exch{"1HO4 CO5"}="HO41 CO5";
$resn_exch{"2HO4 CO5"}="HO42 CO5"; $resn_exch{"1HO5 CO5"}="HO51 CO5"; $resn_exch{"2HO5 CO5"}="HO52 CO5";
$resn_exch{"1HN6 CO6"}="HN61 CO6"; $resn_exch{"2HN6 CO6"}="HN62 CO6"; $resn_exch{" H1* CO6"}=" H1' CO6";
$resn_exch{" H2* CO6"}=" H2' CO6"; $resn_exch{" H3* CO6"}=" H3' CO6"; $resn_exch{" H4* CO6"}=" H4' CO6";
$resn_exch{"1H5* CO6"}="H5'1 CO6"; $resn_exch{"2H5* CO6"}="H5'2 CO6"; $resn_exch{"1H12 CO6"}="H121 CO6";
$resn_exch{"2H12 CO6"}="H122 CO6"; $resn_exch{"1H13 CO6"}="H131 CO6"; $resn_exch{"2H13 CO6"}="H132 CO6";
$resn_exch{"3H13 CO6"}="H133 CO6"; $resn_exch{"1H14 CO6"}="H141 CO6"; $resn_exch{"2H14 CO6"}="H142 CO6";
$resn_exch{"3H14 CO6"}="H143 CO6"; $resn_exch{"1H7  CO6"}=" H71 CO6"; $resn_exch{"2H7  CO6"}=" H72 CO6";
$resn_exch{"1H6  CO6"}=" H61 CO6"; $resn_exch{"2H6  CO6"}=" H62 CO6"; $resn_exch{"1H3  CO6"}=" H31 CO6";
$resn_exch{"2H3  CO6"}=" H32 CO6"; $resn_exch{"1H2  CO6"}=" H21 CO6"; $resn_exch{"2H2  CO6"}=" H22 CO6";
$resn_exch{"1H1  CO6"}=" H11 CO6"; $resn_exch{"2H1  CO6"}=" H12 CO6"; $resn_exch{"3H1  CO6"}=" H13 CO6";
$resn_exch{"1H4  CO6"}=" H41 CO6"; $resn_exch{"2H4  CO6"}=" H42 CO6"; $resn_exch{"3H4  CO6"}=" H43 CO6";
$resn_exch{" AH2 CO8"}=" H2A CO8"; $resn_exch{"AH61 CO8"}="H61A CO8"; $resn_exch{"AH62 CO8"}="H62A CO8";
$resn_exch{" AH8 CO8"}=" H8A CO8"; $resn_exch{"AH1* CO8"}=" H1B CO8"; $resn_exch{"AH2* CO8"}=" H2B CO8";
$resn_exch{"AHO2 CO8"}="HO2A CO8"; $resn_exch{"AH3* CO8"}=" H3B CO8"; $resn_exch{"8HOA CO8"}="HOA8 CO8";
$resn_exch{"9HOA CO8"}="HOA9 CO8"; $resn_exch{"AH4* CO8"}=" H4B CO8"; $resn_exch{"AH51 CO8"}="H51A CO8";
$resn_exch{"AH52 CO8"}="H52A CO8"; $resn_exch{"2HOA CO8"}="HOA2 CO8"; $resn_exch{"5HOA CO8"}="HOA5 CO8";
$resn_exch{"1H12 CO8"}="H121 CO8"; $resn_exch{"2H12 CO8"}="H122 CO8"; $resn_exch{"1H13 CO8"}="H131 CO8";
$resn_exch{"2H13 CO8"}="H132 CO8"; $resn_exch{"3H13 CO8"}="H133 CO8"; $resn_exch{"1H14 CO8"}="H141 CO8";
$resn_exch{"2H14 CO8"}="H142 CO8"; $resn_exch{"3H14 CO8"}="H143 CO8"; $resn_exch{"1H7  CO8"}=" H71 CO8";
$resn_exch{"2H7  CO8"}=" H72 CO8"; $resn_exch{"1H6  CO8"}=" H61 CO8"; $resn_exch{"2H6  CO8"}=" H62 CO8";
$resn_exch{"1H3  CO8"}=" H31 CO8"; $resn_exch{"2H3  CO8"}=" H32 CO8"; $resn_exch{"1H2  CO8"}=" H21 CO8";
$resn_exch{"2H2  CO8"}=" H22 CO8"; $resn_exch{"1H2* CO8"}="H2'1 CO8"; $resn_exch{"2H2* CO8"}="H2'2 CO8";
$resn_exch{"1H3* CO8"}="H3'1 CO8"; $resn_exch{"2H3* CO8"}="H3'2 CO8"; $resn_exch{"1H4* CO8"}="H4'1 CO8";
$resn_exch{"2H4* CO8"}="H4'2 CO8"; $resn_exch{"1H5* CO8"}="H5'1 CO8"; $resn_exch{"2H5* CO8"}="H5'2 CO8";
$resn_exch{"1H6* CO8"}="H6'1 CO8"; $resn_exch{"2H6* CO8"}="H6'2 CO8"; $resn_exch{"1H7* CO8"}="H7'1 CO8";
$resn_exch{"2H7* CO8"}="H7'2 CO8"; $resn_exch{"1H8* CO8"}="H8'1 CO8"; $resn_exch{"2H8* CO8"}="H8'2 CO8";
$resn_exch{"3H8* CO8"}="H8'3 CO8"; $resn_exch{"1H1  CO9"}="H11A CO9"; $resn_exch{"2H1  CO9"}=" H12 CO9";
$resn_exch{"3H1  CO9"}=" H13 CO9"; $resn_exch{"1HN1 CO9"}="HN11 CO9"; $resn_exch{"2HN1 CO9"}="HN12 CO9";
$resn_exch{"1H6  CO9"}=" H61 CO9"; $resn_exch{"2H6  CO9"}=" H62 CO9"; $resn_exch{"1H7  CO9"}=" H71 CO9";
$resn_exch{"2H7  CO9"}=" H72 CO9"; $resn_exch{"1H8  CO9"}=" H81 CO9"; $resn_exch{"2H8  CO9"}=" H82 CO9";
$resn_exch{"1H12 CO9"}="H121 CO9"; $resn_exch{"2H12 CO9"}="H122 CO9"; $resn_exch{" AH2 COA"}=" H2A COA";
$resn_exch{"AH61 COA"}="H61A COA"; $resn_exch{"AH62 COA"}="H62A COA"; $resn_exch{" AH8 COA"}=" H8A COA";
$resn_exch{"AH1* COA"}=" H1B COA"; $resn_exch{"AH2* COA"}=" H2B COA"; $resn_exch{"AHO2 COA"}="HO2A COA";
$resn_exch{"AH3* COA"}=" H3B COA"; $resn_exch{"8HOA COA"}="HOA8 COA"; $resn_exch{"9HOA COA"}="HOA9 COA";
$resn_exch{"AH4* COA"}=" H4B COA"; $resn_exch{"AH51 COA"}="H51A COA"; $resn_exch{"AH52 COA"}="H52A COA";
$resn_exch{"2HOA COA"}="HOA2 COA"; $resn_exch{"5HOA COA"}="HOA5 COA"; $resn_exch{"1H12 COA"}="H121 COA";
$resn_exch{"2H12 COA"}="H122 COA"; $resn_exch{"1H13 COA"}="H131 COA"; $resn_exch{"2H13 COA"}="H132 COA";
$resn_exch{"3H13 COA"}="H133 COA"; $resn_exch{"1H14 COA"}="H141 COA"; $resn_exch{"2H14 COA"}="H142 COA";
$resn_exch{"3H14 COA"}="H143 COA"; $resn_exch{"1H7  COA"}=" H71 COA"; $resn_exch{"2H7  COA"}=" H72 COA";
$resn_exch{"1H6  COA"}=" H61 COA"; $resn_exch{"2H6  COA"}=" H62 COA"; $resn_exch{"1H3  COA"}=" H31 COA";
$resn_exch{"2H3  COA"}=" H32 COA"; $resn_exch{"1H2  COA"}=" H21 COA"; $resn_exch{"2H2  COA"}=" H22 COA";
$resn_exch{"1HN7 COD"}="HN71 COD"; $resn_exch{"2HN7 COD"}="HN72 COD"; $resn_exch{"1H18 COD"}="H181 COD";
$resn_exch{"2H18 COD"}="H182 COD"; $resn_exch{"1H28 COD"}="H281 COD"; $resn_exch{"2H28 COD"}="H282 COD";
$resn_exch{"1H30 COD"}="H301 COD"; $resn_exch{"2H30 COD"}="H302 COD"; $resn_exch{"3H30 COD"}="H303 COD";
$resn_exch{"1H31 COD"}="H311 COD"; $resn_exch{"2H31 COD"}="H312 COD"; $resn_exch{"3H31 COD"}="H313 COD";
$resn_exch{"1H37 COD"}="H371 COD"; $resn_exch{"2H37 COD"}="H372 COD"; $resn_exch{"1H38 COD"}="H381 COD";
$resn_exch{"2H38 COD"}="H382 COD"; $resn_exch{"1H42 COD"}="H421 COD"; $resn_exch{"2H42 COD"}="H422 COD";
$resn_exch{"1H43 COD"}="H431 COD"; $resn_exch{"2H43 COD"}="H432 COD"; $resn_exch{"1HN2 COE"}="HN21 COE";
$resn_exch{"2HN2 COE"}="HN22 COE"; $resn_exch{"1HN4 COE"}="HN41 COE"; $resn_exch{"2HN4 COE"}="HN42 COE";
$resn_exch{"1H9  COE"}=" H91 COE"; $resn_exch{"2H9  COE"}=" H92 COE"; $resn_exch{"1H10 COE"}="H101 COE";
$resn_exch{"2H10 COE"}="H102 COE"; $resn_exch{"3H10 COE"}="H103 COE"; $resn_exch{"1HB  COE"}=" HB1 COE";
$resn_exch{"2HB  COE"}=" HB2 COE"; $resn_exch{"1HG  COE"}=" HG1 COE"; $resn_exch{"2HG  COE"}=" HG2 COE";
$resn_exch{"2HOE COE"}="HOE2 COE"; $resn_exch{" AH2 COF"}=" H2A COF"; $resn_exch{"AH61 COF"}="H61A COF";
$resn_exch{"AH62 COF"}="H62A COF"; $resn_exch{" AH8 COF"}=" H8A COF"; $resn_exch{"AH1* COF"}=" H1B COF";
$resn_exch{"AH2* COF"}=" H2B COF"; $resn_exch{"AHO2 COF"}="HO2A COF"; $resn_exch{"AH3* COF"}=" H3B COF";
$resn_exch{"8HOA COF"}="HOA8 COF"; $resn_exch{"9HOA COF"}="HOA9 COF"; $resn_exch{"AH4* COF"}=" H4B COF";
$resn_exch{"AH51 COF"}="H51A COF"; $resn_exch{"AH52 COF"}="H52A COF"; $resn_exch{"2HOA COF"}="HOA2 COF";
$resn_exch{"5HOA COF"}="HOA5 COF"; $resn_exch{"1H12 COF"}="H121 COF"; $resn_exch{"2H12 COF"}="H122 COF";
$resn_exch{"1H13 COF"}="H131 COF"; $resn_exch{"2H13 COF"}="H132 COF"; $resn_exch{"3H13 COF"}="H133 COF";
$resn_exch{"1H14 COF"}="H141 COF"; $resn_exch{"2H14 COF"}="H142 COF"; $resn_exch{"3H14 COF"}="H143 COF";
$resn_exch{"1H7  COF"}=" H71 COF"; $resn_exch{"2H7  COF"}=" H72 COF"; $resn_exch{"1H6  COF"}=" H61 COF";
$resn_exch{"2H6  COF"}=" H62 COF"; $resn_exch{"1H3  COF"}=" H31 COF"; $resn_exch{"2H3  COF"}=" H32 COF";
$resn_exch{"1H2  COF"}=" H21 COF"; $resn_exch{"2H2  COF"}=" H22 COF"; $resn_exch{"1HI1 COF"}="HI11 COF";
$resn_exch{"2HI1 COF"}="HI12 COF"; $resn_exch{" H5* COG"}=" H5' COG"; $resn_exch{" H7* COG"}=" H7' COG";
$resn_exch{"1H21 COG"}="H211 COG"; $resn_exch{"2H21 COG"}="H212 COG"; $resn_exch{"3H21 COG"}="H213 COG";
$resn_exch{"1H7' COG"}="H7'1 COG"; $resn_exch{"2H7' COG"}="H7'2 COG"; $resn_exch{"1H61 COG"}="H611 COG";
$resn_exch{"2H61 COG"}="H612 COG"; $resn_exch{"3H61 COG"}="H613 COG"; $resn_exch{"1H51 COG"}="H511 COG";
$resn_exch{"2H51 COG"}="H512 COG"; $resn_exch{"3H51 COG"}="H513 COG"; $resn_exch{"1HN2 COG"}="HN21 COG";
$resn_exch{"2HN2 COG"}="HN22 COG"; $resn_exch{"1HN4 COG"}="HN41 COG"; $resn_exch{"2HN4 COG"}="HN42 COG";
$resn_exch{" H8* COG"}=" H8' COG"; $resn_exch{"1HMA COH"}="HMA1 COH"; $resn_exch{"2HMA COH"}="HMA2 COH";
$resn_exch{"3HMA COH"}="HMA3 COH"; $resn_exch{"1HAA COH"}="HAA1 COH"; $resn_exch{"2HAA COH"}="HAA2 COH";
$resn_exch{"1HBA COH"}="HBA1 COH"; $resn_exch{"2HBA COH"}="HBA2 COH"; $resn_exch{"1HMB COH"}="HMB1 COH";
$resn_exch{"2HMB COH"}="HMB2 COH"; $resn_exch{"3HMB COH"}="HMB3 COH"; $resn_exch{"1HBB COH"}="HBB1 COH";
$resn_exch{"2HBB COH"}="HBB2 COH"; $resn_exch{"1HMC COH"}="HMC1 COH"; $resn_exch{"2HMC COH"}="HMC2 COH";
$resn_exch{"3HMC COH"}="HMC3 COH"; $resn_exch{"1HBC COH"}="HBC1 COH"; $resn_exch{"2HBC COH"}="HBC2 COH";
$resn_exch{"1HMD COH"}="HMD1 COH"; $resn_exch{"2HMD COH"}="HMD2 COH"; $resn_exch{"3HMD COH"}="HMD3 COH";
$resn_exch{"1HAD COH"}="HAD1 COH"; $resn_exch{"2HAD COH"}="HAD2 COH"; $resn_exch{"1HBD COH"}="HBD1 COH";
$resn_exch{"2HBD COH"}="HBD2 COH"; $resn_exch{"1H3  COI"}=" H31 COI"; $resn_exch{"2H3  COI"}=" H32 COI";
$resn_exch{"1H5  COI"}=" H51 COI"; $resn_exch{"2H5  COI"}=" H52 COI"; $resn_exch{"3H5  COI"}=" H53 COI";
$resn_exch{"1H6  COI"}=" H61 COI"; $resn_exch{"2H6  COI"}=" H62 COI"; $resn_exch{"3H6  COI"}=" H63 COI";
$resn_exch{"1H48 COJ"}="H481 COJ"; $resn_exch{"2H48 COJ"}="H482 COJ"; $resn_exch{"1H49 COJ"}="H491 COJ";
$resn_exch{"2H49 COJ"}="H492 COJ"; $resn_exch{"1H46 COJ"}="H461 COJ"; $resn_exch{"2H46 COJ"}="H462 COJ";
$resn_exch{"3H46 COJ"}="H463 COJ"; $resn_exch{"1H47 COJ"}="H471 COJ"; $resn_exch{"2H47 COJ"}="H472 COJ";
$resn_exch{"3H47 COJ"}="H473 COJ"; $resn_exch{"1H60 COJ"}="H601 COJ"; $resn_exch{"2H60 COJ"}="H602 COJ";
$resn_exch{"1H54 COJ"}="H541 COJ"; $resn_exch{"2H54 COJ"}="H542 COJ"; $resn_exch{"3H54 COJ"}="H543 COJ";
$resn_exch{"1H55 COJ"}="H551 COJ"; $resn_exch{"2H55 COJ"}="H552 COJ"; $resn_exch{"1H56 COJ"}="H561 COJ";
$resn_exch{"2H56 COJ"}="H562 COJ"; $resn_exch{"1H42 COJ"}="H421 COJ"; $resn_exch{"2H42 COJ"}="H422 COJ";
$resn_exch{"1H25 COJ"}="H251 COJ"; $resn_exch{"2H25 COJ"}="H252 COJ"; $resn_exch{"3H25 COJ"}="H253 COJ";
$resn_exch{"1H26 COJ"}="H261 COJ"; $resn_exch{"2H26 COJ"}="H262 COJ"; $resn_exch{"1H3  COJ"}=" H31 COJ";
$resn_exch{"1H30 COJ"}="H301 COJ"; $resn_exch{"2H30 COJ"}="H302 COJ"; $resn_exch{"1H64 COJ"}="H641 COJ";
$resn_exch{"2H64 COJ"}="H642 COJ"; $resn_exch{"3H64 COJ"}="H643 COJ"; $resn_exch{"1H20 COJ"}="H201 COJ";
$resn_exch{"2H20 COJ"}="H202 COJ"; $resn_exch{"3H20 COJ"}="H203 COJ"; $resn_exch{"1H31 COJ"}="H311 COJ";
$resn_exch{"2H31 COJ"}="H312 COJ"; $resn_exch{"1H36 COJ"}="H361 COJ"; $resn_exch{"2H36 COJ"}="H362 COJ";
$resn_exch{"3H36 COJ"}="H363 COJ"; $resn_exch{"1H37 COJ"}="H371 COJ"; $resn_exch{"2H37 COJ"}="H372 COJ";
$resn_exch{"1H41 COJ"}="H411 COJ"; $resn_exch{"2H41 COJ"}="H412 COJ"; $resn_exch{"1H35 COJ"}="H351 COJ";
$resn_exch{"2H35 COJ"}="H352 COJ"; $resn_exch{"3H35 COJ"}="H353 COJ"; $resn_exch{"1HN6 COK"}="HN61 COK";
$resn_exch{"2HN6 COK"}="HN62 COK"; $resn_exch{" H1* COK"}=" H1' COK"; $resn_exch{" H2* COK"}=" H2' COK";
$resn_exch{" H3* COK"}=" H3' COK"; $resn_exch{" H4* COK"}=" H4' COK"; $resn_exch{"1H5* COK"}="H5'1 COK";
$resn_exch{"2H5* COK"}="H5'2 COK"; $resn_exch{"1H12 COK"}="H121 COK"; $resn_exch{"2H12 COK"}="H122 COK";
$resn_exch{"1H13 COK"}="H131 COK"; $resn_exch{"2H13 COK"}="H132 COK"; $resn_exch{"3H13 COK"}="H133 COK";
$resn_exch{"1H14 COK"}="H141 COK"; $resn_exch{"2H14 COK"}="H142 COK"; $resn_exch{"3H14 COK"}="H143 COK";
$resn_exch{"1H7  COK"}=" H71 COK"; $resn_exch{"2H7  COK"}=" H72 COK"; $resn_exch{"1H6  COK"}=" H61 COK";
$resn_exch{"2H6  COK"}=" H62 COK"; $resn_exch{"1H3  COK"}=" H31 COK"; $resn_exch{"2H3  COK"}=" H32 COK";
$resn_exch{"1H2  COK"}=" H21 COK"; $resn_exch{"2H2  COK"}=" H22 COK"; $resn_exch{"1H50 COK"}="H501 COK";
$resn_exch{"2H50 COK"}="H502 COK"; $resn_exch{"1H51 COK"}="H511 COK"; $resn_exch{"2H51 COK"}="H512 COK";
$resn_exch{"1H2  COL"}=" H21 COL"; $resn_exch{"2H2  COL"}="H22A COL"; $resn_exch{"1H5  COL"}=" H51 COL";
$resn_exch{"2H5  COL"}=" H52 COL"; $resn_exch{"1H6  COL"}=" H61 COL"; $resn_exch{"2H6  COL"}=" H62 COL";
$resn_exch{"1H1  COM"}=" H11 COM"; $resn_exch{"2H1  COM"}=" H12 COM"; $resn_exch{"1H2  COM"}=" H21 COM";
$resn_exch{"2H2  COM"}=" H22 COM"; $resn_exch{"3HOS COM"}="HOS3 COM"; $resn_exch{"1HN1 CON"}="HN11 CON";
$resn_exch{"2HN1 CON"}="HN12 CON"; $resn_exch{"3HN1 CON"}="HN13 CON"; $resn_exch{"1HN2 CON"}="HN21 CON";
$resn_exch{"2HN2 CON"}="HN22 CON"; $resn_exch{"3HN2 CON"}="HN23 CON"; $resn_exch{"1HN3 CON"}="HN31 CON";
$resn_exch{"2HN3 CON"}="HN32 CON"; $resn_exch{"3HN3 CON"}="HN33 CON"; $resn_exch{"1HN4 CON"}="HN41 CON";
$resn_exch{"2HN4 CON"}="HN42 CON"; $resn_exch{"3HN4 CON"}="HN43 CON"; $resn_exch{"1H2  COO"}=" H21 COO";
$resn_exch{"2H2  COO"}=" H22 COO"; $resn_exch{"1H3  COO"}=" H31 COO"; $resn_exch{"2H3  COO"}=" H32 COO";
$resn_exch{"1H5  COO"}=" H51 COO"; $resn_exch{"2H5  COO"}=" H52 COO"; $resn_exch{"1H6  COO"}=" H61 COO";
$resn_exch{"2H6  COO"}=" H62 COO"; $resn_exch{"1H10 COO"}="H101 COO"; $resn_exch{"2H10 COO"}="H102 COO";
$resn_exch{"3H10 COO"}="H103 COO"; $resn_exch{"1H15 COO"}="H151 COO"; $resn_exch{"2H15 COO"}="H152 COO";
$resn_exch{"3H15 COO"}="H153 COO"; $resn_exch{"1H14 COO"}="H141 COO"; $resn_exch{"2H14 COO"}="H142 COO";
$resn_exch{"3H14 COO"}="H143 COO"; $resn_exch{"1H12 COO"}="H121 COO"; $resn_exch{"2H12 COO"}="H122 COO";
$resn_exch{"1H5X COO"}="H5X1 COO"; $resn_exch{"2H5X COO"}="H5X2 COO"; $resn_exch{"1H6A COO"}="H6A1 COO";
$resn_exch{"2H6A COO"}="H6A2 COO"; $resn_exch{"1H11 COP"}="H111 COP"; $resn_exch{"2H11 COP"}="H112 COP";
$resn_exch{"1H27 COP"}="H271 COP"; $resn_exch{"2H27 COP"}="H272 COP"; $resn_exch{"1H28 COP"}="H281 COP";
$resn_exch{"2H28 COP"}="H282 COP"; $resn_exch{"1H29 COP"}="H291 COP"; $resn_exch{"2H29 COP"}="H292 COP";
$resn_exch{"1HN3 COP"}="HN31 COP"; $resn_exch{"2HN3 COP"}="HN32 COP"; $resn_exch{"1HN1 COP"}="HN11 COP";
$resn_exch{"2HN1 COP"}="HN12 COP"; $resn_exch{" H5* COQ"}=" H5' COQ"; $resn_exch{" H7* COQ"}=" H7' COQ";
$resn_exch{"1H9* COQ"}="H9'1 COQ"; $resn_exch{"2H9* COQ"}="H9'2 COQ"; $resn_exch{"1H31 COQ"}="H311 COQ";
$resn_exch{"2H31 COQ"}="H312 COQ"; $resn_exch{"3H31 COQ"}="H313 COQ"; $resn_exch{"1H51 COQ"}="H511 COQ";
$resn_exch{"2H51 COQ"}="H512 COQ"; $resn_exch{"3H51 COQ"}="H513 COQ"; $resn_exch{"1HCN COQ"}="HCN1 COQ";
$resn_exch{"2HCN COQ"}="HCN2 COQ"; $resn_exch{"3HCN COQ"}="HCN3 COQ"; $resn_exch{"1HN2 COQ"}="HN21 COQ";
$resn_exch{"2HN2 COQ"}="HN22 COQ"; $resn_exch{"1HN4 COQ"}="HN41 COQ"; $resn_exch{"2HN4 COQ"}="HN42 COQ";
$resn_exch{"1H1  COR"}=" H11 COR"; $resn_exch{"2H1  COR"}=" H12 COR"; $resn_exch{"1HN2 COR"}="HN21 COR";
$resn_exch{"2HN2 COR"}="HN22 COR"; $resn_exch{"1HN4 COR"}="HN41 COR"; $resn_exch{"2HN4 COR"}="HN42 COR";
$resn_exch{"1H5  COR"}=" H51 COR"; $resn_exch{"2H5  COR"}=" H52 COR"; $resn_exch{" AH2 COS"}=" H2A COS";
$resn_exch{"AH61 COS"}="H61A COS"; $resn_exch{"AH62 COS"}="H62A COS"; $resn_exch{" AH8 COS"}=" H8A COS";
$resn_exch{"AH1* COS"}=" H1B COS"; $resn_exch{"AH2* COS"}=" H2B COS"; $resn_exch{"AHO2 COS"}="HO2A COS";
$resn_exch{"AH3* COS"}=" H3B COS"; $resn_exch{"8HOA COS"}="HOA8 COS"; $resn_exch{"9HOA COS"}="HOA9 COS";
$resn_exch{"AH4* COS"}=" H4B COS"; $resn_exch{"AH51 COS"}="H51A COS"; $resn_exch{"AH52 COS"}="H52A COS";
$resn_exch{"2HOA COS"}="HOA2 COS"; $resn_exch{"5HOA COS"}="HOA5 COS"; $resn_exch{"1H12 COS"}="H121 COS";
$resn_exch{"2H12 COS"}="H122 COS"; $resn_exch{"1H13 COS"}="H131 COS"; $resn_exch{"2H13 COS"}="H132 COS";
$resn_exch{"3H13 COS"}="H133 COS"; $resn_exch{"1H14 COS"}="H141 COS"; $resn_exch{"2H14 COS"}="H142 COS";
$resn_exch{"3H14 COS"}="H143 COS"; $resn_exch{"1H7  COS"}=" H71 COS"; $resn_exch{"2H7  COS"}=" H72 COS";
$resn_exch{"1H6  COS"}=" H61 COS"; $resn_exch{"2H6  COS"}=" H62 COS"; $resn_exch{"1H3  COS"}=" H31 COS";
$resn_exch{"2H3  COS"}=" H32 COS"; $resn_exch{"1H2  COS"}=" H21 COS"; $resn_exch{"2H2  COS"}=" H22 COS";
$resn_exch{"PHS' COS"}="HS'P COS"; $resn_exch{"1H1T COT"}="H1T1 COT"; $resn_exch{"2H1T COT"}="H1T2 COT";
$resn_exch{"1H6T COT"}="H6T1 COT"; $resn_exch{"2H6T COT"}="H6T2 COT"; $resn_exch{"1H2T COT"}="H2T1 COT";
$resn_exch{"2H2T COT"}="H2T2 COT"; $resn_exch{"1HC5 COT"}="HC51 COT"; $resn_exch{"2HC5 COT"}="HC52 COT";
$resn_exch{"1HN6 COT"}="HN61 COT"; $resn_exch{"2HN6 COT"}="HN62 COT"; $resn_exch{"2HCA COT"}="HCA2 COT";
$resn_exch{"1H6P COT"}="H6P1 COT"; $resn_exch{"2H6P COT"}="H6P2 COT"; $resn_exch{"1H7P COT"}="H7P1 COT";
$resn_exch{"2H7P COT"}="H7P2 COT"; $resn_exch{"1H16 COT"}="H161 COT"; $resn_exch{"2H16 COT"}="H162 COT";
$resn_exch{"1H14 COT"}="H141 COT"; $resn_exch{"2H14 COT"}="H142 COT"; $resn_exch{"3H14 COT"}="H143 COT";
$resn_exch{"1H15 COT"}="H151 COT"; $resn_exch{"2H15 COT"}="H152 COT"; $resn_exch{"3H15 COT"}="H153 COT";
$resn_exch{"1H2P COT"}="H2P1 COT"; $resn_exch{"2H2P COT"}="H2P2 COT"; $resn_exch{"1H1P COT"}="H1P1 COT";
$resn_exch{"2H1P COT"}="H1P2 COT"; $resn_exch{"1HN1 COX"}="HN11 COX"; $resn_exch{"2HN1 COX"}="HN12 COX";
$resn_exch{"1H9  COX"}=" H91 COX"; $resn_exch{"2H9  COX"}=" H92 COX"; $resn_exch{"3H9  COX"}=" H93 COX";
$resn_exch{"1H20 COY"}="H201 COY"; $resn_exch{"2H20 COY"}="H202 COY"; $resn_exch{"3H20 COY"}="H203 COY";
$resn_exch{"1H25 COY"}="H251 COY"; $resn_exch{"2H25 COY"}="H252 COY"; $resn_exch{"3H25 COY"}="H253 COY";
$resn_exch{"1H26 COY"}="H261 COY"; $resn_exch{"2H26 COY"}="H262 COY"; $resn_exch{"1H29 COY"}="H291 COY";
$resn_exch{"2H29 COY"}="H292 COY"; $resn_exch{"1H30 COY"}="H301 COY"; $resn_exch{"2H30 COY"}="H302 COY";
$resn_exch{"1H31 COY"}="H311 COY"; $resn_exch{"2H31 COY"}="H312 COY"; $resn_exch{"1H33 COY"}="H331 COY";
$resn_exch{"2H33 COY"}="H332 COY"; $resn_exch{"1H35 COY"}="H351 COY"; $resn_exch{"2H35 COY"}="H352 COY";
$resn_exch{"3H35 COY"}="H353 COY"; $resn_exch{"6H6  COY"}=" H66 COY"; $resn_exch{"1H36 COY"}="H361 COY";
$resn_exch{"2H36 COY"}="H362 COY"; $resn_exch{"3H36 COY"}="H363 COY"; $resn_exch{"1H37 COY"}="H371 COY";
$resn_exch{"2H37 COY"}="H372 COY"; $resn_exch{"1H40 COY"}="H401 COY"; $resn_exch{"2H40 COY"}="H402 COY";
$resn_exch{"1H41 COY"}="H411 COY"; $resn_exch{"2H41 COY"}="H412 COY"; $resn_exch{"1H42 COY"}="H421 COY";
$resn_exch{"2H42 COY"}="H422 COY"; $resn_exch{"1H45 COY"}="H451 COY"; $resn_exch{"2H45 COY"}="H452 COY";
$resn_exch{"11H1 COY"}="H111 COY"; $resn_exch{"1H46 COY"}="H461 COY"; $resn_exch{"2H46 COY"}="H462 COY";
$resn_exch{"3H46 COY"}="H463 COY"; $resn_exch{"1H47 COY"}="H471 COY"; $resn_exch{"2H47 COY"}="H472 COY";
$resn_exch{"3H47 COY"}="H473 COY"; $resn_exch{"1H48 COY"}="H481 COY"; $resn_exch{"2H48 COY"}="H482 COY";
$resn_exch{"1H49 COY"}="H491 COY"; $resn_exch{"2H49 COY"}="H492 COY"; $resn_exch{"1H52 COY"}="H521 COY";
$resn_exch{"2H52 COY"}="H522 COY"; $resn_exch{"1H53 COY"}="H531 COY"; $resn_exch{"2H53 COY"}="H532 COY";
$resn_exch{"3H53 COY"}="H533 COY"; $resn_exch{"16H1 COY"}="H116 COY"; $resn_exch{"1H54 COY"}="H541 COY";
$resn_exch{"2H54 COY"}="H542 COY"; $resn_exch{"3H54 COY"}="H543 COY"; $resn_exch{"1H55 COY"}="H551 COY";
$resn_exch{"2H55 COY"}="H552 COY"; $resn_exch{"1H56 COY"}="H561 COY"; $resn_exch{"2H56 COY"}="H562 COY";
$resn_exch{"1H60 COY"}="H601 COY"; $resn_exch{"2H60 COY"}="H602 COY"; $resn_exch{"1H62 COY"}="H621 COY";
$resn_exch{"2H62 COY"}="H622 COY"; $resn_exch{"19H1 COY"}="H119 COY"; $resn_exch{"1H1P COY"}="H1P1 COY";
$resn_exch{"2H1P COY"}="H1P2 COY"; $resn_exch{"1H3P COY"}="H3P1 COY"; $resn_exch{"2H3P COY"}="H3P2 COY";
$resn_exch{"3H3P COY"}="H3P3 COY"; $resn_exch{"5HOP COY"}="HOP5 COY"; $resn_exch{"7HOR COY"}="HOR7 COY";
$resn_exch{"1H5R COY"}="H5R1 COY"; $resn_exch{"2H5R COY"}="H5R2 COY"; $resn_exch{"8HOR COY"}="HOR8 COY";
$resn_exch{"1HM5 COY"}="HM51 COY"; $resn_exch{"2HM5 COY"}="HM52 COY"; $resn_exch{"3HM5 COY"}="HM53 COY";
$resn_exch{"1HM6 COY"}="HM61 COY"; $resn_exch{"2HM6 COY"}="HM62 COY"; $resn_exch{"3HM6 COY"}="HM63 COY";
$resn_exch{"1H71 COY"}="H711 COY"; $resn_exch{"2H71 COY"}="H712 COY"; $resn_exch{"1H72 COY"}="H721 COY";
$resn_exch{"2H72 COY"}="H722 COY"; $resn_exch{"1H73 COY"}="H731 COY"; $resn_exch{"2H73 COY"}="H732 COY";
$resn_exch{"1H74 COY"}="H741 COY"; $resn_exch{"2H74 COY"}="H742 COY"; $resn_exch{"1H75 COY"}="H751 COY";
$resn_exch{"2H75 COY"}="H752 COY"; $resn_exch{"1H6A COY"}="H6A1 COY"; $resn_exch{"2H6A COY"}="H6A2 COY";
$resn_exch{" AH2 COZ"}=" H2A COZ"; $resn_exch{"AH61 COZ"}="H61A COZ"; $resn_exch{"AH62 COZ"}="H62A COZ";
$resn_exch{" AH8 COZ"}=" H8A COZ"; $resn_exch{"AH1* COZ"}=" H1B COZ"; $resn_exch{"AH2* COZ"}=" H2B COZ";
$resn_exch{"AHO2 COZ"}="HO2A COZ"; $resn_exch{"AH3* COZ"}=" H3B COZ"; $resn_exch{"8HOA COZ"}="HOA8 COZ";
$resn_exch{"9HOA COZ"}="HOA9 COZ"; $resn_exch{"AH4* COZ"}=" H4B COZ"; $resn_exch{"AH51 COZ"}="H51A COZ";
$resn_exch{"AH52 COZ"}="H52A COZ"; $resn_exch{"2HOA COZ"}="HOA2 COZ"; $resn_exch{"5HOA COZ"}="HOA5 COZ";
$resn_exch{"1H12 COZ"}="H121 COZ"; $resn_exch{"2H12 COZ"}="H122 COZ"; $resn_exch{"1H13 COZ"}="H131 COZ";
$resn_exch{"2H13 COZ"}="H132 COZ"; $resn_exch{"3H13 COZ"}="H133 COZ"; $resn_exch{"1H14 COZ"}="H141 COZ";
$resn_exch{"2H14 COZ"}="H142 COZ"; $resn_exch{"3H14 COZ"}="H143 COZ"; $resn_exch{"1H7  COZ"}=" H71 COZ";
$resn_exch{"2H7  COZ"}=" H72 COZ"; $resn_exch{"1H6  COZ"}=" H61 COZ"; $resn_exch{"2H6  COZ"}=" H62 COZ";
$resn_exch{"1H3  COZ"}=" H31 COZ"; $resn_exch{"2H3  COZ"}=" H32 COZ"; $resn_exch{"1H2  COZ"}=" H21 COZ";
$resn_exch{"2H2  COZ"}=" H22 COZ"; $resn_exch{"1HN   CP"}=" HN1  CP"; $resn_exch{"2HN   CP"}=" HN2  CP";
$resn_exch{"2HOP  CP"}="HOP2  CP"; $resn_exch{"3HOP  CP"}="HOP3  CP"; $resn_exch{"1H17 CP0"}="H171 CP0";
$resn_exch{"2H17 CP0"}="H172 CP0"; $resn_exch{"1H5  CP0"}=" H51 CP0"; $resn_exch{"2H5  CP0"}=" H52 CP0";
$resn_exch{"1H28 CP0"}="H281 CP0"; $resn_exch{"2H28 CP0"}="H282 CP0"; $resn_exch{"1H31 CP0"}="H311 CP0";
$resn_exch{"2H31 CP0"}="H312 CP0"; $resn_exch{"1H32 CP0"}="H321 CP0"; $resn_exch{"2H32 CP0"}="H322 CP0";
$resn_exch{"3H32 CP0"}="H323 CP0"; $resn_exch{"1H33 CP0"}="H331 CP0"; $resn_exch{"2H33 CP0"}="H332 CP0";
$resn_exch{"3H33 CP0"}="H333 CP0"; $resn_exch{"1H25 CP0"}="H251 CP0"; $resn_exch{"2H25 CP0"}="H252 CP0";
$resn_exch{"1H22 CP0"}="H221 CP0"; $resn_exch{"2H22 CP0"}="H222 CP0"; $resn_exch{"1H26 CP0"}="H261 CP0";
$resn_exch{"2H26 CP0"}="H262 CP0"; $resn_exch{"1H29 CP0"}="H291 CP0"; $resn_exch{"2H29 CP0"}="H292 CP0";
$resn_exch{"1H34 CP0"}="H341 CP0"; $resn_exch{"2H34 CP0"}="H342 CP0"; $resn_exch{"1H35 CP0"}="H351 CP0";
$resn_exch{"2H35 CP0"}="H352 CP0"; $resn_exch{"1H27 CP0"}="H271 CP0"; $resn_exch{"2H27 CP0"}="H272 CP0";
$resn_exch{"1H30 CP0"}="H301 CP0"; $resn_exch{"2H30 CP0"}="H302 CP0"; $resn_exch{"1H1  CP0"}="H11A CP0";
$resn_exch{"2H1  CP0"}="H12A CP0"; $resn_exch{"1H8' CP1"}="H8'1 CP1"; $resn_exch{"2H8' CP1"}="H8'2 CP1";
$resn_exch{"1H5' CP1"}="H5'1 CP1"; $resn_exch{"2H5' CP1"}="H5'2 CP1"; $resn_exch{"1H3' CP1"}="H3'1 CP1";
$resn_exch{"2H3' CP1"}="H3'2 CP1"; $resn_exch{"1H2' CP1"}="H2'1 CP1"; $resn_exch{"2H2' CP1"}="H2'2 CP1";
$resn_exch{"1H1' CP1"}="H1'1 CP1"; $resn_exch{"2H1' CP1"}="H1'2 CP1"; $resn_exch{"3H1' CP1"}="H1'3 CP1";
$resn_exch{"1HN4 CP1"}="HN41 CP1"; $resn_exch{"2HN4 CP1"}="HN42 CP1"; $resn_exch{"1HC2 CP2"}="HC21 CP2";
$resn_exch{"2HC2 CP2"}="HC22 CP2"; $resn_exch{"1HC3 CP2"}="HC31 CP2"; $resn_exch{"1HC4 CP2"}="HC41 CP2";
$resn_exch{"2HC4 CP2"}="HC42 CP2"; $resn_exch{"3HC4 CP2"}="HC43 CP2"; $resn_exch{"1H62 CP3"}="H621 CP3";
$resn_exch{"2H62 CP3"}="H622 CP3"; $resn_exch{"1H61 CP3"}="H611 CP3"; $resn_exch{"2H61 CP3"}="H612 CP3";
$resn_exch{"1H20 CP3"}="H201 CP3"; $resn_exch{"2H20 CP3"}="H202 CP3"; $resn_exch{"1H86 CP3"}="H861 CP3";
$resn_exch{"2H86 CP3"}="H862 CP3"; $resn_exch{"3H86 CP3"}="H863 CP3"; $resn_exch{"1H15 CP3"}="H151 CP3";
$resn_exch{"2H15 CP3"}="H152 CP3"; $resn_exch{"1H51 CP3"}="H511 CP3"; $resn_exch{"2H51 CP3"}="H512 CP3";
$resn_exch{"1H52 CP3"}="H521 CP3"; $resn_exch{"2H52 CP3"}="H522 CP3"; $resn_exch{"1H81 CP3"}="H811 CP3";
$resn_exch{"2H81 CP3"}="H812 CP3"; $resn_exch{"3H81 CP3"}="H813 CP3"; $resn_exch{"1H10 CP3"}="H101 CP3";
$resn_exch{"2H10 CP3"}="H102 CP3"; $resn_exch{"1H7A CP3"}="H7A1 CP3"; $resn_exch{"2H7A CP3"}="H7A2 CP3";
$resn_exch{"3H7A CP3"}="H7A3 CP3"; $resn_exch{"1H41 CP3"}="H411 CP3"; $resn_exch{"2H41 CP3"}="H412 CP3";
$resn_exch{"1H42 CP3"}="H421 CP3"; $resn_exch{"2H42 CP3"}="H422 CP3"; $resn_exch{"1H5  CP3"}=" H51 CP3";
$resn_exch{"2H5  CP3"}=" H52 CP3"; $resn_exch{"1H2A CP3"}="H2A1 CP3"; $resn_exch{"2H2A CP3"}="H2A2 CP3";
$resn_exch{"3H2A CP3"}="H2A3 CP3"; $resn_exch{"1H31 CP3"}="H311 CP3"; $resn_exch{"2H31 CP3"}="H312 CP3";
$resn_exch{"1H32 CP3"}="H321 CP3"; $resn_exch{"2H32 CP3"}="H322 CP3"; $resn_exch{"1H13 CP4"}="H131 CP4";
$resn_exch{"2H13 CP4"}="H132 CP4"; $resn_exch{"1H14 CP4"}="H141 CP4"; $resn_exch{"2H14 CP4"}="H142 CP4";
$resn_exch{"1H16 CP4"}="H161 CP4"; $resn_exch{"2H16 CP4"}="H162 CP4"; $resn_exch{"1H17 CP4"}="H171 CP4";
$resn_exch{"2H17 CP4"}="H172 CP4"; $resn_exch{"1H19 CP4"}="H191 CP4"; $resn_exch{"2H19 CP4"}="H192 CP4";
$resn_exch{"1H20 CP4"}="H201 CP4"; $resn_exch{"2H20 CP4"}="H202 CP4"; $resn_exch{"1H13 CP6"}="H131 CP6";
$resn_exch{"2H13 CP6"}="H132 CP6"; $resn_exch{"1H14 CP6"}="H141 CP6"; $resn_exch{"2H14 CP6"}="H142 CP6";
$resn_exch{"1H15 CP6"}="H151 CP6"; $resn_exch{"2H15 CP6"}="H152 CP6"; $resn_exch{"1H16 CP6"}="H161 CP6";
$resn_exch{"2H16 CP6"}="H162 CP6"; $resn_exch{"3H16 CP6"}="H163 CP6"; $resn_exch{"1H13 CP7"}="H131 CP7";
$resn_exch{"2H13 CP7"}="H132 CP7"; $resn_exch{"1H14 CP7"}="H141 CP7"; $resn_exch{"2H14 CP7"}="H142 CP7";
$resn_exch{"1H15 CP7"}="H151 CP7"; $resn_exch{"2H15 CP7"}="H152 CP7"; $resn_exch{"1H16 CP7"}="H161 CP7";
$resn_exch{"2H16 CP7"}="H162 CP7"; $resn_exch{"3H16 CP7"}="H163 CP7"; $resn_exch{"1H6C CP8"}="H6C1 CP8";
$resn_exch{"2H6C CP8"}="H6C2 CP8"; $resn_exch{"1H8C CP8"}="H8C1 CP8"; $resn_exch{"2H8C CP8"}="H8C2 CP8";
$resn_exch{"3H8C CP8"}="H8C3 CP8"; $resn_exch{"1H14 CP8"}="H141 CP8"; $resn_exch{"2H14 CP8"}="H142 CP8";
$resn_exch{"1H17 CP8"}="H171 CP8"; $resn_exch{"2H17 CP8"}="H172 CP8"; $resn_exch{"1H18 CP8"}="H181 CP8";
$resn_exch{"2H18 CP8"}="H182 CP8"; $resn_exch{"1H10 CP9"}="H101 CP9"; $resn_exch{"2H10 CP9"}="H102 CP9";
$resn_exch{"1H26 CP9"}="H261 CP9"; $resn_exch{"2H26 CP9"}="H262 CP9"; $resn_exch{"3H26 CP9"}="H263 CP9";
$resn_exch{"2HOP CPA"}="HOP2 CPA"; $resn_exch{"CHO5 CPA"}="HO5C CPA"; $resn_exch{"CH51 CPA"}="H51C CPA";
$resn_exch{"CH52 CPA"}="H52C CPA"; $resn_exch{"CH4' CPA"}=" H4D CPA"; $resn_exch{"CH3' CPA"}=" H3D CPA";
$resn_exch{"CH21 CPA"}="H21C CPA"; $resn_exch{"CH22 CPA"}="H22C CPA"; $resn_exch{"CH1' CPA"}=" H1D CPA";
$resn_exch{" CH5 CPA"}=" H5C CPA"; $resn_exch{" CH6 CPA"}=" H6C CPA"; $resn_exch{"CH41 CPA"}="H41C CPA";
$resn_exch{"CH42 CPA"}="H42C CPA"; $resn_exch{"AH51 CPA"}="H51A CPA"; $resn_exch{"AH52 CPA"}="H52A CPA";
$resn_exch{"AH4' CPA"}=" H4B CPA"; $resn_exch{"AH3' CPA"}=" H3B CPA"; $resn_exch{"AHO3 CPA"}="HO3A CPA";
$resn_exch{"AH21 CPA"}="H21A CPA"; $resn_exch{"AH22 CPA"}="H22A CPA"; $resn_exch{"AH1' CPA"}=" H1B CPA";
$resn_exch{" AH8 CPA"}=" H8A CPA"; $resn_exch{"AH61 CPA"}="H61A CPA"; $resn_exch{"AH62 CPA"}="H62A CPA";
$resn_exch{" AH2 CPA"}=" H2A CPA"; $resn_exch{"1H1  CPB"}=" H11 CPB"; $resn_exch{"2H1  CPB"}=" H12 CPB";
$resn_exch{"3H1  CPB"}=" H13 CPB"; $resn_exch{"1H16 CPB"}="H161 CPB"; $resn_exch{"2H16 CPB"}="H162 CPB";
$resn_exch{"1H15 CPB"}="H151 CPB"; $resn_exch{"2H15 CPB"}="H152 CPB"; $resn_exch{"1H14 CPB"}="H141 CPB";
$resn_exch{"1H13 CPB"}="H131 CPB"; $resn_exch{"1H12 CPB"}="H121 CPB"; $resn_exch{"2H12 CPB"}="H122 CPB";
$resn_exch{"1H3  CPB"}=" H31 CPB"; $resn_exch{"1H6  CPB"}=" H61 CPB"; $resn_exch{"1H23 CPB"}="H231 CPB";
$resn_exch{"1H24 CPB"}="H241 CPB"; $resn_exch{"1H25 CPB"}="H251 CPB"; $resn_exch{"1H26 CPB"}="H261 CPB";
$resn_exch{" HN  CPC"}=" H   CPC"; $resn_exch{"1HCN CPC"}="HCN1 CPC"; $resn_exch{"2HCN CPC"}="HCN2 CPC";
$resn_exch{"3HCN CPC"}="HCN3 CPC"; $resn_exch{"1HG1 CPC"}="HG13 CPC"; $resn_exch{"2HG1 CPC"}="HG12 CPC";
$resn_exch{"1HG2 CPC"}="HG21 CPC"; $resn_exch{"2HG2 CPC"}="HG22 CPC"; $resn_exch{"3HG2 CPC"}="HG23 CPC";
$resn_exch{"1H4  CPD"}=" H41 CPD"; $resn_exch{"2H4  CPD"}=" H42 CPD"; $resn_exch{"1H12 CPD"}="H121 CPD";
$resn_exch{"2H12 CPD"}="H122 CPD"; $resn_exch{"1H22 CPD"}="H221 CPD"; $resn_exch{"2H22 CPD"}="H222 CPD";
$resn_exch{"1H23 CPD"}="H231 CPD"; $resn_exch{"2H23 CPD"}="H232 CPD"; $resn_exch{"3H23 CPD"}="H233 CPD";
$resn_exch{"1H24 CPD"}="H241 CPD"; $resn_exch{"2H24 CPD"}="H242 CPD"; $resn_exch{"1H25 CPD"}="H251 CPD";
$resn_exch{"2H25 CPD"}="H252 CPD"; $resn_exch{"1H26 CPD"}="H261 CPD"; $resn_exch{"2H26 CPD"}="H262 CPD";
$resn_exch{"1H27 CPD"}="H271 CPD"; $resn_exch{"2H27 CPD"}="H272 CPD"; $resn_exch{"3H27 CPD"}="H273 CPD";
$resn_exch{"1H28 CPD"}="H281 CPD"; $resn_exch{"2H28 CPD"}="H282 CPD"; $resn_exch{"3H28 CPD"}="H283 CPD";
$resn_exch{"1H17 CPF"}="H171 CPF"; $resn_exch{"2H17 CPF"}="H172 CPF"; $resn_exch{"1H16 CPF"}="H161 CPF";
$resn_exch{"2H16 CPF"}="H162 CPF"; $resn_exch{"1H15 CPF"}="H151 CPF"; $resn_exch{"2H15 CPF"}="H152 CPF";
$resn_exch{"1H14 CPF"}="H141 CPF"; $resn_exch{"2H14 CPF"}="H142 CPF"; $resn_exch{"1H12 CPF"}="H121 CPF";
$resn_exch{"2H12 CPF"}="H122 CPF"; $resn_exch{"1H13 CPF"}="H131 CPF"; $resn_exch{"2H13 CPF"}="H132 CPF";
$resn_exch{"2HOP CPG"}="HOP2 CPG"; $resn_exch{"3HOP CPG"}="HOP3 CPG"; $resn_exch{"1H5* CPG"}="H5'1 CPG";
$resn_exch{"2H5* CPG"}="H5'2 CPG"; $resn_exch{" H4* CPG"}=" H4' CPG"; $resn_exch{" H3* CPG"}=" H3' CPG";
$resn_exch{"*HO3 CPG"}="HO3' CPG"; $resn_exch{" H2* CPG"}=" H2' CPG"; $resn_exch{"*HO2 CPG"}="HO2' CPG";
$resn_exch{" H1* CPG"}=" H1' CPG"; $resn_exch{"1HN2 CPG"}="HN21 CPG"; $resn_exch{"2HN2 CPG"}="HN22 CPG";
$resn_exch{"1H4  CPH"}=" H41 CPH"; $resn_exch{"2H4  CPH"}=" H42 CPH"; $resn_exch{"1H10 CPH"}="H101 CPH";
$resn_exch{"2H10 CPH"}="H102 CPH"; $resn_exch{"1H7  CPH"}=" H71 CPH"; $resn_exch{"2H7  CPH"}=" H72 CPH";
$resn_exch{"3H7  CPH"}=" H73 CPH"; $resn_exch{" H1* CPH"}=" H1' CPH"; $resn_exch{" H3* CPH"}=" H3' CPH";
$resn_exch{" H4* CPH"}=" H4' CPH"; $resn_exch{"1H5  CPH"}=" H51 CPH"; $resn_exch{"2H5  CPH"}=" H52 CPH";
$resn_exch{"3H5  CPH"}=" H53 CPH"; $resn_exch{"1H2  CPI"}=" H21 CPI"; $resn_exch{"2H2  CPI"}=" H22 CPI";
$resn_exch{"1H3  CPI"}=" H31 CPI"; $resn_exch{"2H3  CPI"}=" H32 CPI"; $resn_exch{"1H4  CPI"}=" H41 CPI";
$resn_exch{"2H4  CPI"}=" H42 CPI"; $resn_exch{"1H5  CPI"}=" H51 CPI"; $resn_exch{"2H5  CPI"}=" H52 CPI";
$resn_exch{"1H28 CPJ"}="H281 CPJ"; $resn_exch{"2H28 CPJ"}="H282 CPJ"; $resn_exch{"1H32 CPJ"}="H321 CPJ";
$resn_exch{"2H32 CPJ"}="H322 CPJ"; $resn_exch{"3H32 CPJ"}="H323 CPJ"; $resn_exch{"1H24 CPJ"}="H241 CPJ";
$resn_exch{"2H24 CPJ"}="H242 CPJ"; $resn_exch{"1H16 CPJ"}="H161 CPJ"; $resn_exch{"2H16 CPJ"}="H162 CPJ";
$resn_exch{"1H15 CPJ"}="H151 CPJ"; $resn_exch{"2H15 CPJ"}="H152 CPJ"; $resn_exch{"1H2  CPJ"}=" H21 CPJ";
$resn_exch{"2H2  CPJ"}=" H22 CPJ"; $resn_exch{"1H4  CPJ"}=" H41 CPJ"; $resn_exch{"2H4  CPJ"}=" H42 CPJ";
$resn_exch{"1H10 CPJ"}="H101 CPJ"; $resn_exch{"2H10 CPJ"}="H102 CPJ"; $resn_exch{"3H10 CPJ"}="H103 CPJ";
$resn_exch{"1H28 CPK"}="H281 CPK"; $resn_exch{"2H28 CPK"}="H282 CPK"; $resn_exch{"1H32 CPK"}="H321 CPK";
$resn_exch{"2H32 CPK"}="H322 CPK"; $resn_exch{"3H32 CPK"}="H323 CPK"; $resn_exch{"1H24 CPK"}="H241 CPK";
$resn_exch{"2H24 CPK"}="H242 CPK"; $resn_exch{"1H16 CPK"}="H161 CPK"; $resn_exch{"2H16 CPK"}="H162 CPK";
$resn_exch{"1H15 CPK"}="H151 CPK"; $resn_exch{"2H15 CPK"}="H152 CPK"; $resn_exch{"1H18 CPK"}="H181 CPK";
$resn_exch{"2H18 CPK"}="H182 CPK"; $resn_exch{"1H19 CPK"}="H191 CPK"; $resn_exch{"2H19 CPK"}="H192 CPK";
$resn_exch{"1H2  CPK"}=" H21 CPK"; $resn_exch{"2H2  CPK"}=" H22 CPK"; $resn_exch{"1H4  CPK"}=" H41 CPK";
$resn_exch{"2H4  CPK"}=" H42 CPK"; $resn_exch{"1H10 CPK"}="H101 CPK"; $resn_exch{"2H10 CPK"}="H102 CPK";
$resn_exch{"3H10 CPK"}="H103 CPK"; $resn_exch{"1HC1 CPL"}="HC11 CPL"; $resn_exch{"2HC1 CPL"}="HC12 CPL";
$resn_exch{"1HC3 CPL"}="HC31 CPL"; $resn_exch{"2HC3 CPL"}="HC32 CPL"; $resn_exch{"1HC4 CPL"}="HC41 CPL";
$resn_exch{"2HC4 CPL"}="HC42 CPL"; $resn_exch{"1HC5 CPL"}="HC51 CPL"; $resn_exch{"2HC5 CPL"}="HC52 CPL";
$resn_exch{"1HC6 CPL"}="HC61 CPL"; $resn_exch{"2HC6 CPL"}="HC62 CPL"; $resn_exch{"3HC6 CPL"}="HC63 CPL";
$resn_exch{"1HC7 CPL"}="HC71 CPL"; $resn_exch{"2HC7 CPL"}="HC72 CPL"; $resn_exch{"3HC7 CPL"}="HC73 CPL";
$resn_exch{"1HC8 CPL"}="HC81 CPL"; $resn_exch{"2HC8 CPL"}="HC82 CPL"; $resn_exch{"3HC8 CPL"}="HC83 CPL";
$resn_exch{"1H12 CPL"}="H121 CPL"; $resn_exch{"2H12 CPL"}="H122 CPL"; $resn_exch{"1H13 CPL"}="H131 CPL";
$resn_exch{"2H13 CPL"}="H132 CPL"; $resn_exch{"1H14 CPL"}="H141 CPL"; $resn_exch{"2H14 CPL"}="H142 CPL";
$resn_exch{"1H15 CPL"}="H151 CPL"; $resn_exch{"2H15 CPL"}="H152 CPL"; $resn_exch{"1H16 CPL"}="H161 CPL";
$resn_exch{"2H16 CPL"}="H162 CPL"; $resn_exch{"1H17 CPL"}="H171 CPL"; $resn_exch{"2H17 CPL"}="H172 CPL";
$resn_exch{"1H18 CPL"}="H181 CPL"; $resn_exch{"2H18 CPL"}="H182 CPL"; $resn_exch{"1H19 CPL"}="H191 CPL";
$resn_exch{"2H19 CPL"}="H192 CPL"; $resn_exch{"1H20 CPL"}="H201 CPL"; $resn_exch{"2H20 CPL"}="H202 CPL";
$resn_exch{"1H21 CPL"}="H211 CPL"; $resn_exch{"2H21 CPL"}="H212 CPL"; $resn_exch{"1H22 CPL"}="H221 CPL";
$resn_exch{"2H22 CPL"}="H222 CPL"; $resn_exch{"1H23 CPL"}="H231 CPL"; $resn_exch{"2H23 CPL"}="H232 CPL";
$resn_exch{"1H24 CPL"}="H241 CPL"; $resn_exch{"2H24 CPL"}="H242 CPL"; $resn_exch{"1H25 CPL"}="H251 CPL";
$resn_exch{"2H25 CPL"}="H252 CPL"; $resn_exch{"1H26 CPL"}="H261 CPL"; $resn_exch{"2H26 CPL"}="H262 CPL";
$resn_exch{"3H26 CPL"}="H263 CPL"; $resn_exch{"1H32 CPL"}="H321 CPL"; $resn_exch{"2H32 CPL"}="H322 CPL";
$resn_exch{"1H33 CPL"}="H331 CPL"; $resn_exch{"2H33 CPL"}="H332 CPL"; $resn_exch{"1H34 CPL"}="H341 CPL";
$resn_exch{"2H34 CPL"}="H342 CPL"; $resn_exch{"1H35 CPL"}="H351 CPL"; $resn_exch{"2H35 CPL"}="H352 CPL";
$resn_exch{"1H36 CPL"}="H361 CPL"; $resn_exch{"2H36 CPL"}="H362 CPL"; $resn_exch{"1H37 CPL"}="H371 CPL";
$resn_exch{"2H37 CPL"}="H372 CPL"; $resn_exch{"1H38 CPL"}="H381 CPL"; $resn_exch{"2H38 CPL"}="H382 CPL";
$resn_exch{"1H41 CPL"}="H411 CPL"; $resn_exch{"2H41 CPL"}="H412 CPL"; $resn_exch{"1H44 CPL"}="H441 CPL";
$resn_exch{"2H44 CPL"}="H442 CPL"; $resn_exch{"1H45 CPL"}="H451 CPL"; $resn_exch{"2H45 CPL"}="H452 CPL";
$resn_exch{"1H46 CPL"}="H461 CPL"; $resn_exch{"2H46 CPL"}="H462 CPL"; $resn_exch{"1H47 CPL"}="H471 CPL";
$resn_exch{"2H47 CPL"}="H472 CPL"; $resn_exch{"1H48 CPL"}="H481 CPL"; $resn_exch{"2H48 CPL"}="H482 CPL";
$resn_exch{"3H48 CPL"}="H483 CPL"; $resn_exch{"1HA1 CPM"}="HA11 CPM"; $resn_exch{"2HA1 CPM"}="HA12 CPM";
$resn_exch{"1HB  CPM"}=" HB1 CPM"; $resn_exch{"2HB  CPM"}=" HB2 CPM"; $resn_exch{"1HM  CPM"}=" HM1 CPM";
$resn_exch{"2HM  CPM"}=" HM2 CPM"; $resn_exch{"3HM  CPM"}=" HM3 CPM"; $resn_exch{"1H8' CPN"}="H8'1 CPN";
$resn_exch{"2H8' CPN"}="H8'2 CPN"; $resn_exch{"1H5' CPN"}="H5'1 CPN"; $resn_exch{"2H5' CPN"}="H5'2 CPN";
$resn_exch{"1H3' CPN"}="H3'1 CPN"; $resn_exch{"2H3' CPN"}="H3'2 CPN"; $resn_exch{"1H2' CPN"}="H2'1 CPN";
$resn_exch{"2H2' CPN"}="H2'2 CPN"; $resn_exch{"1H1' CPN"}="H1'1 CPN"; $resn_exch{"2H1' CPN"}="H1'2 CPN";
$resn_exch{"3H1' CPN"}="H1'3 CPN"; $resn_exch{"1HN4 CPN"}="HN41 CPN"; $resn_exch{"2HN4 CPN"}="HN42 CPN";
$resn_exch{"1H2  CPO"}=" H21 CPO"; $resn_exch{"1H3  CPO"}=" H31 CPO"; $resn_exch{"2H3  CPO"}=" H32 CPO";
$resn_exch{"1H4  CPO"}=" H41 CPO"; $resn_exch{"2H4  CPO"}=" H42 CPO"; $resn_exch{"1H5  CPO"}=" H51 CPO";
$resn_exch{"2H5  CPO"}=" H52 CPO"; $resn_exch{"1HN2 CPO"}="HN21 CPO"; $resn_exch{"1H8  CPO"}=" H81 CPO";
$resn_exch{"1H9  CPO"}=" H91 CPO"; $resn_exch{"2H9  CPO"}=" H92 CPO"; $resn_exch{"1H10 CPO"}="H101 CPO";
$resn_exch{"2H10 CPO"}="H102 CPO"; $resn_exch{"1H11 CPO"}="H111 CPO"; $resn_exch{"2H11 CPO"}="H112 CPO";
$resn_exch{"1HN3 CPO"}="HN31 CPO"; $resn_exch{"1H12 CPO"}="H121 CPO"; $resn_exch{"2H12 CPO"}="H122 CPO";
$resn_exch{"3H12 CPO"}="H123 CPO"; $resn_exch{"1H14 CPO"}="H141 CPO"; $resn_exch{"2H14 CPO"}="H142 CPO";
$resn_exch{"1H15 CPO"}="H151 CPO"; $resn_exch{"2H15 CPO"}="H152 CPO"; $resn_exch{"1H16 CPO"}="H161 CPO";
$resn_exch{"1H18 CPO"}="H181 CPO"; $resn_exch{"1H20 CPO"}="H201 CPO"; $resn_exch{"2H20 CPO"}="H202 CPO";
$resn_exch{"1H21 CPO"}="H211 CPO"; $resn_exch{"2H21 CPO"}="H212 CPO"; $resn_exch{"1H24 CPO"}="H241 CPO";
$resn_exch{"1H25 CPO"}="H251 CPO"; $resn_exch{"2H25 CPO"}="H252 CPO"; $resn_exch{"1H26 CPO"}="H261 CPO";
$resn_exch{"2H26 CPO"}="H262 CPO"; $resn_exch{"1H27 CPO"}="H271 CPO"; $resn_exch{"2H27 CPO"}="H272 CPO";
$resn_exch{"2H29 CPO"}="H292 CPO"; $resn_exch{"1H31 CPO"}="H311 CPO"; $resn_exch{"2H31 CPO"}="H312 CPO";
$resn_exch{"1H32 CPO"}="H321 CPO"; $resn_exch{"2H32 CPO"}="H322 CPO"; $resn_exch{"1H33 CPO"}="H331 CPO";
$resn_exch{"2H33 CPO"}="H332 CPO"; $resn_exch{"3H33 CPO"}="H333 CPO"; $resn_exch{"1HN6 CPO"}="HN61 CPO";
$resn_exch{"1H35 CPO"}="H351 CPO"; $resn_exch{"2H35 CPO"}="H352 CPO"; $resn_exch{"3H35 CPO"}="H353 CPO";
$resn_exch{"1H19 CPO"}="H191 CPO"; $resn_exch{"2H19 CPO"}="H192 CPO"; $resn_exch{"3H19 CPO"}="H193 CPO";
$resn_exch{"1H3  CPP"}=" H31 CPP"; $resn_exch{"2H3  CPP"}=" H32 CPP"; $resn_exch{"3H3  CPP"}=" H33 CPP";
$resn_exch{"1H2' CPP"}="H2'1 CPP"; $resn_exch{"2H2' CPP"}="H2'2 CPP"; $resn_exch{"1H3' CPP"}="H3'1 CPP";
$resn_exch{"2H3' CPP"}="H3'2 CPP"; $resn_exch{"1H4' CPP"}="H4'1 CPP"; $resn_exch{"2H4' CPP"}="H4'2 CPP";
$resn_exch{"1H1  CPQ"}=" H11 CPQ"; $resn_exch{"2H1  CPQ"}=" H12 CPQ"; $resn_exch{"1H3  CPQ"}=" H31 CPQ";
$resn_exch{"2H3  CPQ"}=" H32 CPQ"; $resn_exch{"1H7  CPQ"}=" H71 CPQ"; $resn_exch{"2H7  CPQ"}=" H72 CPQ";
$resn_exch{"1H8  CPQ"}=" H81 CPQ"; $resn_exch{"2H8  CPQ"}=" H82 CPQ"; $resn_exch{"1H10 CPQ"}="H101 CPQ";
$resn_exch{"2H10 CPQ"}="H102 CPQ"; $resn_exch{"3H10 CPQ"}="H103 CPQ"; $resn_exch{"1H11 CPQ"}="H111 CPQ";
$resn_exch{"2H11 CPQ"}="H112 CPQ"; $resn_exch{"3H11 CPQ"}="H113 CPQ"; $resn_exch{"1H12 CPQ"}="H121 CPQ";
$resn_exch{"2H12 CPQ"}="H122 CPQ"; $resn_exch{"1H14 CPQ"}="H141 CPQ"; $resn_exch{"2H14 CPQ"}="H142 CPQ";
$resn_exch{"1H15 CPQ"}="H151 CPQ"; $resn_exch{"1H16 CPQ"}="H161 CPQ"; $resn_exch{"2H16 CPQ"}="H162 CPQ";
$resn_exch{"1H17 CPQ"}="H171 CPQ"; $resn_exch{"2H17 CPQ"}="H172 CPQ"; $resn_exch{"1H21 CPQ"}="H211 CPQ";
$resn_exch{"2H21 CPQ"}="H212 CPQ"; $resn_exch{"3H21 CPQ"}="H213 CPQ"; $resn_exch{"1H22 CPQ"}="H221 CPQ";
$resn_exch{"2H22 CPQ"}="H222 CPQ"; $resn_exch{"1H23 CPQ"}="H231 CPQ"; $resn_exch{"2H23 CPQ"}="H232 CPQ";
$resn_exch{"1H74 CPQ"}="H741 CPQ"; $resn_exch{"2H74 CPQ"}="H742 CPQ"; $resn_exch{"1H75 CPQ"}="H751 CPQ";
$resn_exch{"2H75 CPQ"}="H752 CPQ"; $resn_exch{"1H76 CPQ"}="H761 CPQ"; $resn_exch{"2H76 CPQ"}="H762 CPQ";
$resn_exch{"1H77 CPQ"}="H771 CPQ"; $resn_exch{"1H80 CPQ"}="H801 CPQ"; $resn_exch{"1H86 CPQ"}="H861 CPQ";
$resn_exch{"1H81 CPQ"}="H811 CPQ"; $resn_exch{"1H87 CPQ"}="H871 CPQ"; $resn_exch{"1H82 CPQ"}="H821 CPQ";
$resn_exch{"1H88 CPQ"}="H881 CPQ"; $resn_exch{"1H83 CPQ"}="H831 CPQ"; $resn_exch{"1H89 CPQ"}="H891 CPQ";
$resn_exch{"1H84 CPQ"}="H841 CPQ"; $resn_exch{"2H84 CPQ"}="H842 CPQ"; $resn_exch{"1H85 CPQ"}="H851 CPQ";
$resn_exch{"1H68 CPQ"}="H681 CPQ"; $resn_exch{"2H68 CPQ"}="H682 CPQ"; $resn_exch{"1H69 CPQ"}="H691 CPQ";
$resn_exch{"2H69 CPQ"}="H692 CPQ"; $resn_exch{"1H70 CPQ"}="H701 CPQ"; $resn_exch{"2H70 CPQ"}="H702 CPQ";
$resn_exch{"1H71 CPQ"}="H711 CPQ"; $resn_exch{"1H90 CPQ"}="H901 CPQ"; $resn_exch{"1H96 CPQ"}="H961 CPQ";
$resn_exch{"1H91 CPQ"}="H911 CPQ"; $resn_exch{"1H97 CPQ"}="H971 CPQ"; $resn_exch{"1H92 CPQ"}="H921 CPQ";
$resn_exch{"1H98 CPQ"}="H981 CPQ"; $resn_exch{"1H93 CPQ"}="H931 CPQ"; $resn_exch{"1H99 CPQ"}="H991 CPQ";
$resn_exch{"1H94 CPQ"}="H941 CPQ"; $resn_exch{"2H94 CPQ"}="H942 CPQ"; $resn_exch{"1H95 CPQ"}="H951 CPQ";
$resn_exch{"2HOP CPR"}="HOP2 CPR"; $resn_exch{"3HOP CPR"}="HOP3 CPR"; $resn_exch{"1H5* CPR"}="H5'1 CPR";
$resn_exch{"2H5* CPR"}="H5'2 CPR"; $resn_exch{" H4* CPR"}=" H4' CPR"; $resn_exch{" H3* CPR"}=" H3' CPR";
$resn_exch{"*HO3 CPR"}="HO3' CPR"; $resn_exch{" H2* CPR"}=" H2' CPR"; $resn_exch{"*HO2 CPR"}="HO2' CPR";
$resn_exch{" H1* CPR"}=" H1' CPR"; $resn_exch{"1H1  CPS"}=" H11 CPS"; $resn_exch{"2H1  CPS"}=" H12 CPS";
$resn_exch{"1H3  CPS"}=" H31 CPS"; $resn_exch{"2H3  CPS"}=" H32 CPS"; $resn_exch{"1H7  CPS"}=" H71 CPS";
$resn_exch{"2H7  CPS"}=" H72 CPS"; $resn_exch{"1H8  CPS"}=" H81 CPS"; $resn_exch{"2H8  CPS"}=" H82 CPS";
$resn_exch{"1H10 CPS"}="H101 CPS"; $resn_exch{"2H10 CPS"}="H102 CPS"; $resn_exch{"3H10 CPS"}="H103 CPS";
$resn_exch{"1H11 CPS"}="H111 CPS"; $resn_exch{"2H11 CPS"}="H112 CPS"; $resn_exch{"3H11 CPS"}="H113 CPS";
$resn_exch{"1H12 CPS"}="H121 CPS"; $resn_exch{"2H12 CPS"}="H122 CPS"; $resn_exch{"1H14 CPS"}="H141 CPS";
$resn_exch{"2H14 CPS"}="H142 CPS"; $resn_exch{"1H16 CPS"}="H161 CPS"; $resn_exch{"2H16 CPS"}="H162 CPS";
$resn_exch{"1H21 CPS"}="H211 CPS"; $resn_exch{"2H21 CPS"}="H212 CPS"; $resn_exch{"3H21 CPS"}="H213 CPS";
$resn_exch{"1H22 CPS"}="H221 CPS"; $resn_exch{"2H22 CPS"}="H222 CPS"; $resn_exch{"1H23 CPS"}="H231 CPS";
$resn_exch{"2H23 CPS"}="H232 CPS"; $resn_exch{"1H25 CPS"}="H251 CPS"; $resn_exch{"2H25 CPS"}="H252 CPS";
$resn_exch{"1H26 CPS"}="H261 CPS"; $resn_exch{"2H26 CPS"}="H262 CPS"; $resn_exch{"1H27 CPS"}="H271 CPS";
$resn_exch{"2H27 CPS"}="H272 CPS"; $resn_exch{"1H28 CPS"}="H281 CPS"; $resn_exch{"2H28 CPS"}="H282 CPS";
$resn_exch{"3H28 CPS"}="H283 CPS"; $resn_exch{"1H29 CPS"}="H291 CPS"; $resn_exch{"2H29 CPS"}="H292 CPS";
$resn_exch{"3H29 CPS"}="H293 CPS"; $resn_exch{"1H30 CPS"}="H301 CPS"; $resn_exch{"2H30 CPS"}="H302 CPS";
$resn_exch{"1H31 CPS"}="H311 CPS"; $resn_exch{"2H31 CPS"}="H312 CPS"; $resn_exch{"1H32 CPS"}="H321 CPS";
$resn_exch{"2H32 CPS"}="H322 CPS"; $resn_exch{"1H1  CPT"}=" H11 CPT"; $resn_exch{"2H1  CPT"}=" H12 CPT";
$resn_exch{"3H1  CPT"}=" H13 CPT"; $resn_exch{"1H2  CPT"}=" H21 CPT"; $resn_exch{"2H2  CPT"}=" H22 CPT";
$resn_exch{"3H2  CPT"}=" H23 CPT"; $resn_exch{"1HN  CPV"}=" H   CPV"; $resn_exch{"2HN  CPV"}=" H2  CPV";
$resn_exch{"1HB  CPV"}=" HB2 CPV"; $resn_exch{"2HB  CPV"}=" HB3 CPV"; $resn_exch{"1HD1 CPV"}="HD12 CPV";
$resn_exch{"2HD1 CPV"}="HD13 CPV"; $resn_exch{"1HD2 CPV"}="HD23 CPV"; $resn_exch{"2HD2 CPV"}="HD22 CPV";
$resn_exch{"1HE1 CPV"}="HE12 CPV"; $resn_exch{"2HE1 CPV"}="HE13 CPV"; $resn_exch{"1HE2 CPV"}="HE23 CPV";
$resn_exch{"2HE2 CPV"}="HE22 CPV"; $resn_exch{"1HZ  CPV"}=" HZ2 CPV"; $resn_exch{"2HZ  CPV"}=" HZ3 CPV";
$resn_exch{"1HB1 CPV"}="HB11 CPV"; $resn_exch{"2HB1 CPV"}="HB12 CPV"; $resn_exch{" HA* CPV"}=" HA' CPV";
$resn_exch{" HB* CPV"}=" HB' CPV"; $resn_exch{"1HG1 CPV"}="HG11 CPV"; $resn_exch{"2HG1 CPV"}="HG12 CPV";
$resn_exch{"3HG1 CPV"}="HG13 CPV"; $resn_exch{"1HG2 CPV"}="HG21 CPV"; $resn_exch{"2HG2 CPV"}="HG22 CPV";
$resn_exch{"3HG2 CPV"}="HG23 CPV"; $resn_exch{"1H1  CPW"}=" H11 CPW"; $resn_exch{"2H1  CPW"}=" H12 CPW";
$resn_exch{"1H6  CPW"}=" H61 CPW"; $resn_exch{"2H6  CPW"}=" H62 CPW"; $resn_exch{"1H7  CPW"}=" H71 CPW";
$resn_exch{"2H7  CPW"}=" H72 CPW"; $resn_exch{"1H8  CPW"}=" H81 CPW"; $resn_exch{"2H8  CPW"}=" H82 CPW";
$resn_exch{"1HN3 CPW"}="HN31 CPW"; $resn_exch{"2HN3 CPW"}="HN32 CPW"; $resn_exch{"1H17 CPX"}="H171 CPX";
$resn_exch{"2H17 CPX"}="H172 CPX"; $resn_exch{"1H16 CPX"}="H161 CPX"; $resn_exch{"2H16 CPX"}="H162 CPX";
$resn_exch{"1H30 CPX"}="H301 CPX"; $resn_exch{"2H30 CPX"}="H302 CPX"; $resn_exch{"1H32 CPX"}="H321 CPX";
$resn_exch{"2H32 CPX"}="H322 CPX"; $resn_exch{"1H31 CPX"}="H311 CPX"; $resn_exch{"2H31 CPX"}="H312 CPX";
$resn_exch{"1H28 CPX"}="H281 CPX"; $resn_exch{"2H28 CPX"}="H282 CPX"; $resn_exch{"1H22 CPX"}="H221 CPX";
$resn_exch{"2H22 CPX"}="H222 CPX"; $resn_exch{"3H22 CPX"}="H223 CPX"; $resn_exch{"1H18 CPX"}="H181 CPX";
$resn_exch{"2H18 CPX"}="H182 CPX"; $resn_exch{"3H18 CPX"}="H183 CPX"; $resn_exch{"1H9  CPX"}=" H91 CPX";
$resn_exch{"2H9  CPX"}=" H92 CPX"; $resn_exch{"1H7  CPX"}=" H71 CPX"; $resn_exch{"2H7  CPX"}=" H72 CPX";
$resn_exch{"1H23 CPX"}="H231 CPX"; $resn_exch{"2H23 CPX"}="H232 CPX"; $resn_exch{"1H21 CPX"}="H211 CPX";
$resn_exch{"2H21 CPX"}="H212 CPX"; $resn_exch{"1H20 CPX"}="H201 CPX"; $resn_exch{"2H20 CPX"}="H202 CPX";
$resn_exch{"1HN  CPY"}=" HN1 CPY"; $resn_exch{"2HN  CPY"}=" HN2 CPY"; $resn_exch{"1HB  CPY"}=" HB1 CPY";
$resn_exch{"2HB  CPY"}=" HB2 CPY"; $resn_exch{"1HD1 CPY"}="HD11 CPY"; $resn_exch{"2HD1 CPY"}="HD12 CPY";
$resn_exch{"1HD2 CPY"}="HD21 CPY"; $resn_exch{"2HD2 CPY"}="HD22 CPY"; $resn_exch{"1HE1 CPY"}="HE11 CPY";
$resn_exch{"2HE1 CPY"}="HE12 CPY"; $resn_exch{"1HE2 CPY"}="HE21 CPY"; $resn_exch{"2HE2 CPY"}="HE22 CPY";
$resn_exch{"1HCZ CPY"}="HCZ1 CPY"; $resn_exch{"2HCZ CPY"}="HCZ2 CPY"; $resn_exch{"1HM  CPY"}=" HM1 CPY";
$resn_exch{"2HM  CPY"}=" HM2 CPY"; $resn_exch{"1H16 CQA"}="H161 CQA"; $resn_exch{"2H16 CQA"}="H162 CQA";
$resn_exch{"1H17 CQA"}="H171 CQA"; $resn_exch{"2H17 CQA"}="H172 CQA"; $resn_exch{"1H18 CQA"}="H181 CQA";
$resn_exch{"2H18 CQA"}="H182 CQA"; $resn_exch{"3H18 CQA"}="H183 CQA"; $resn_exch{"1H19 CQA"}="H191 CQA";
$resn_exch{"2H19 CQA"}="H192 CQA"; $resn_exch{"1H20 CQA"}="H201 CQA"; $resn_exch{"2H20 CQA"}="H202 CQA";
$resn_exch{"3H20 CQA"}="H203 CQA"; $resn_exch{"1H6C CQP"}="H6C1 CQP"; $resn_exch{"2H6C CQP"}="H6C2 CQP";
$resn_exch{"1H10 CQP"}="H101 CQP"; $resn_exch{"2H10 CQP"}="H102 CQP"; $resn_exch{"1H14 CQP"}="H141 CQP";
$resn_exch{"2H14 CQP"}="H142 CQP"; $resn_exch{"1H31 CQP"}="H311 CQP"; $resn_exch{"2H31 CQP"}="H312 CQP";
$resn_exch{"1HN  CQR"}=" HN1 CQR"; $resn_exch{"2HN  CQR"}=" HN2 CQR"; $resn_exch{"1HA1 CQR"}="HA11 CQR";
$resn_exch{"2HA1 CQR"}="HA12 CQR"; $resn_exch{"1HA3 CQR"}="HA31 CQR"; $resn_exch{"2HA3 CQR"}="HA32 CQR";
$resn_exch{"1HN1 CR0"}="HN11 CR0"; $resn_exch{"2HN1 CR0"}="HN12 CR0"; $resn_exch{"1HG1 CR0"}="HG11 CR0";
$resn_exch{"2HG1 CR0"}="HG12 CR0"; $resn_exch{"3HG1 CR0"}="HG13 CR0"; $resn_exch{"1HA3 CR0"}="HA31 CR0";
$resn_exch{"2HA3 CR0"}="HA32 CR0"; $resn_exch{"1HB2 CR0"}="HB21 CR0"; $resn_exch{"2HB2 CR0"}="HB22 CR0";
$resn_exch{"1HD1 CR0"}="HD11 CR0"; $resn_exch{"2HD1 CR0"}="HD12 CR0"; $resn_exch{"3HD1 CR0"}="HD13 CR0";
$resn_exch{"1HD2 CR0"}="HD21 CR0"; $resn_exch{"2HD2 CR0"}="HD22 CR0"; $resn_exch{"3HD2 CR0"}="HD23 CR0";
$resn_exch{"1H2  CR1"}=" H21 CR1"; $resn_exch{"1H4  CR1"}=" H41 CR1"; $resn_exch{"1H5  CR1"}=" H51 CR1";
$resn_exch{"1H6  CR1"}=" H61 CR1"; $resn_exch{"2H6  CR1"}=" H62 CR1"; $resn_exch{"1H1  CR1"}=" H11 CR1";
$resn_exch{"1H3  CR1"}=" H31 CR1"; $resn_exch{"1H8  CR1"}=" H81 CR1"; $resn_exch{"2H8  CR1"}=" H82 CR1";
$resn_exch{"3H8  CR1"}=" H83 CR1"; $resn_exch{"1HN1 CR2"}="HN11 CR2"; $resn_exch{"2HN1 CR2"}="HN12 CR2";
$resn_exch{"1HA1 CR2"}="HA11 CR2"; $resn_exch{"2HA1 CR2"}="HA12 CR2"; $resn_exch{"1HA3 CR2"}="HA31 CR2";
$resn_exch{"2HA3 CR2"}="HA32 CR2"; $resn_exch{"1HN  CR5"}=" HN1 CR5"; $resn_exch{"2HN  CR5"}="HN2A CR5";
$resn_exch{"1HA1 CR5"}="HA11 CR5"; $resn_exch{"2HA1 CR5"}="HA12 CR5"; $resn_exch{"1HA3 CR5"}="HA31 CR5";
$resn_exch{"2HA3 CR5"}="HA32 CR5"; $resn_exch{"1H2  CR6"}=" H21 CR6"; $resn_exch{"1H4  CR6"}=" H41 CR6";
$resn_exch{"1H5  CR6"}=" H51 CR6"; $resn_exch{"1H6  CR6"}=" H61 CR6"; $resn_exch{"2H6  CR6"}=" H62 CR6";
$resn_exch{"1H3  CR6"}=" H31 CR6"; $resn_exch{"1H8  CR6"}=" H81 CR6"; $resn_exch{"2H8  CR6"}=" H82 CR6";
$resn_exch{"3H8  CR6"}=" H83 CR6"; $resn_exch{"1HN1 CR7"}="HN11 CR7"; $resn_exch{"2HN1 CR7"}="HN12 CR7";
$resn_exch{"1HB1 CR7"}="HB11 CR7"; $resn_exch{"2HB1 CR7"}="HB12 CR7"; $resn_exch{"1HG1 CR7"}="HG11 CR7";
$resn_exch{"2HG1 CR7"}="HG12 CR7"; $resn_exch{"1HA3 CR7"}="HA31 CR7"; $resn_exch{"2HA3 CR7"}="HA32 CR7";
$resn_exch{"1HD  CR7"}="HD1A CR7"; $resn_exch{"2HD  CR7"}="HD2A CR7"; $resn_exch{"1HE  CR7"}="HE1A CR7";
$resn_exch{"2HE  CR7"}="HE2A CR7"; $resn_exch{"1HNZ CR7"}="HNZ1 CR7"; $resn_exch{"2HNZ CR7"}="HNZ2 CR7";
$resn_exch{"1H20 CR8"}="H201 CR8"; $resn_exch{"2H20 CR8"}="H202 CR8"; $resn_exch{"1H   CR8"}=" H1  CR8";
$resn_exch{"2H   CR8"}=" H2A CR8"; $resn_exch{"1H17 CR8"}="H171 CR8"; $resn_exch{"2H17 CR8"}="H172 CR8";
$resn_exch{"H6'' CR9"}=" H6B CR9"; $resn_exch{"1H2  CRA"}=" H21 CRA"; $resn_exch{"1H4  CRA"}=" H41 CRA";
$resn_exch{"1H5  CRA"}=" H51 CRA"; $resn_exch{"1H6  CRA"}=" H61 CRA"; $resn_exch{"2H6  CRA"}=" H62 CRA";
$resn_exch{"1H3  CRA"}=" H31 CRA"; $resn_exch{"1H8  CRA"}=" H81 CRA"; $resn_exch{"2H8  CRA"}=" H82 CRA";
$resn_exch{"3H8  CRA"}=" H83 CRA"; $resn_exch{"1H3  CRB"}=" H31 CRB"; $resn_exch{"2H3  CRB"}=" H32 CRB";
$resn_exch{"1H7  CRB"}=" H71 CRB"; $resn_exch{"2H7  CRB"}=" H72 CRB"; $resn_exch{"1H8  CRB"}=" H81 CRB";
$resn_exch{"2H8  CRB"}=" H82 CRB"; $resn_exch{"2HOP CRB"}="HOP2 CRB"; $resn_exch{"3HOP CRB"}="HOP3 CRB";
$resn_exch{"1H2  CRC"}=" H21 CRC"; $resn_exch{"2H2  CRC"}=" H22 CRC"; $resn_exch{"1H3  CRC"}=" H31 CRC";
$resn_exch{"2H3  CRC"}=" H32 CRC"; $resn_exch{"1H4  CRC"}=" H41 CRC"; $resn_exch{"2H4  CRC"}=" H42 CRC";
$resn_exch{"1H5  CRC"}=" H51 CRC"; $resn_exch{"2H5  CRC"}=" H52 CRC"; $resn_exch{"1H6  CRC"}=" H61 CRC";
$resn_exch{"2H6  CRC"}=" H62 CRC"; $resn_exch{"1H7  CRC"}=" H71 CRC"; $resn_exch{"2H7  CRC"}=" H72 CRC";
$resn_exch{"1H8  CRC"}=" H81 CRC"; $resn_exch{"2H8  CRC"}=" H82 CRC"; $resn_exch{"1H9  CRC"}=" H91 CRC";
$resn_exch{"2H9  CRC"}=" H92 CRC"; $resn_exch{"1H10 CRC"}="H101 CRC"; $resn_exch{"2H10 CRC"}="H102 CRC";
$resn_exch{"3H10 CRC"}="H103 CRC"; $resn_exch{"1HC1 CRD"}="HC11 CRD"; $resn_exch{"2HC1 CRD"}="HC12 CRD";
$resn_exch{"3HC1 CRD"}="HC13 CRD"; $resn_exch{"1H1  CRE"}=" H11 CRE"; $resn_exch{"2H1  CRE"}="H12A CRE";
$resn_exch{"1H2  CRE"}=" H21 CRE"; $resn_exch{"2H2  CRE"}=" H22 CRE"; $resn_exch{"1H4  CRE"}=" H41 CRE";
$resn_exch{"2H4  CRE"}=" H42 CRE"; $resn_exch{"3H4  CRE"}=" H43 CRE"; $resn_exch{"1H5  CRE"}=" H51 CRE";
$resn_exch{"1H6  CRE"}=" H61 CRE"; $resn_exch{"2H6  CRE"}=" H62 CRE"; $resn_exch{"1H7  CRE"}=" H71 CRE";
$resn_exch{"2H7  CRE"}=" H72 CRE"; $resn_exch{"1H9  CRE"}=" H91 CRE"; $resn_exch{"2H9  CRE"}=" H92 CRE";
$resn_exch{"3H9  CRE"}=" H93 CRE"; $resn_exch{"1H11 CRE"}="H111 CRE"; $resn_exch{"2H11 CRE"}="H112 CRE";
$resn_exch{"1H14 CRE"}="H141 CRE"; $resn_exch{"2H14 CRE"}="H142 CRE"; $resn_exch{"3H14 CRE"}="H143 CRE";
$resn_exch{"1H15 CRE"}="H151 CRE"; $resn_exch{"2H15 CRE"}="H152 CRE"; $resn_exch{"1HN1 CRF"}="HN11 CRF";
$resn_exch{"2HN1 CRF"}="HN12 CRF"; $resn_exch{"1HG1 CRF"}="HG11 CRF"; $resn_exch{"2HG1 CRF"}="HG12 CRF";
$resn_exch{"3HG1 CRF"}="HG13 CRF"; $resn_exch{"1HA3 CRF"}="HA31 CRF"; $resn_exch{"2HA3 CRF"}="HA32 CRF";
$resn_exch{"1HN1 CRG"}="HN11 CRG"; $resn_exch{"2HN1 CRG"}="HN12 CRG"; $resn_exch{"1HG1 CRG"}="HG11 CRG";
$resn_exch{"2HG1 CRG"}="HG12 CRG"; $resn_exch{"3HG1 CRG"}="HG13 CRG"; $resn_exch{"OHG1 CRG"}="HG1O CRG";
$resn_exch{"1HA3 CRG"}="HA31 CRG"; $resn_exch{"2HA3 CRG"}="HA32 CRG"; $resn_exch{"1H2  CRH"}=" H21 CRH";
$resn_exch{"2H2  CRH"}=" H22 CRH"; $resn_exch{"1H4  CRH"}=" H41 CRH"; $resn_exch{"2H4  CRH"}=" H42 CRH";
$resn_exch{"1H15 CRH"}="H151 CRH"; $resn_exch{"2H15 CRH"}="H152 CRH"; $resn_exch{"3H15 CRH"}="H153 CRH";
$resn_exch{" H1* CRH"}=" H1' CRH"; $resn_exch{" H3* CRH"}=" H3' CRH"; $resn_exch{" H4* CRH"}=" H4' CRH";
$resn_exch{"1H1* CRH"}="H1'1 CRH"; $resn_exch{"2H1* CRH"}="H1'2 CRH"; $resn_exch{"3H1* CRH"}="H1'3 CRH";
$resn_exch{"1H4* CRH"}="H4'1 CRH"; $resn_exch{"2H4* CRH"}="H4'2 CRH"; $resn_exch{"3H4* CRH"}="H4'3 CRH";
$resn_exch{"1H13 CRJ"}="H131 CRJ"; $resn_exch{"2H13 CRJ"}="H132 CRJ"; $resn_exch{"1H15 CRJ"}="H151 CRJ";
$resn_exch{"2H15 CRJ"}="H152 CRJ"; $resn_exch{"1H19 CRJ"}="H191 CRJ"; $resn_exch{"2H19 CRJ"}="H192 CRJ";
$resn_exch{"1H20 CRJ"}="H201 CRJ"; $resn_exch{"2H20 CRJ"}="H202 CRJ"; $resn_exch{"1H25 CRJ"}="H251 CRJ";
$resn_exch{"2H25 CRJ"}="H252 CRJ"; $resn_exch{"1H26 CRJ"}="H261 CRJ"; $resn_exch{"2H26 CRJ"}="H262 CRJ";
$resn_exch{"1HA3 CRK"}="HA31 CRK"; $resn_exch{"2HA3 CRK"}="HA32 CRK"; $resn_exch{"1HB  CRK"}=" HB1 CRK";
$resn_exch{"2HB  CRK"}="HB2A CRK"; $resn_exch{"1HE  CRK"}="HE1A CRK"; $resn_exch{"2HE  CRK"}="HE2A CRK";
$resn_exch{"3HE  CRK"}=" HE3 CRK"; $resn_exch{"1HG  CRK"}=" HG1 CRK"; $resn_exch{"2HG  CRK"}=" HG2 CRK";
$resn_exch{"1H2  CRL"}=" H21 CRL"; $resn_exch{"2H2  CRL"}=" H22 CRL"; $resn_exch{"1H3  CRL"}=" H31 CRL";
$resn_exch{"2H3  CRL"}=" H32 CRL"; $resn_exch{"1H4  CRL"}=" H41 CRL"; $resn_exch{"2H4  CRL"}=" H42 CRL";
$resn_exch{"1H5  CRL"}=" H51 CRL"; $resn_exch{"2H5  CRL"}=" H52 CRL"; $resn_exch{"1H10 CRL"}="H101 CRL";
$resn_exch{"2H10 CRL"}="H102 CRL"; $resn_exch{"1H11 CRL"}="H111 CRL"; $resn_exch{"2H11 CRL"}="H112 CRL";
$resn_exch{"1H12 CRL"}="H121 CRL"; $resn_exch{"2H12 CRL"}="H122 CRL"; $resn_exch{"1H13 CRL"}="H131 CRL";
$resn_exch{"2H13 CRL"}="H132 CRL"; $resn_exch{"1H14 CRL"}="H141 CRL"; $resn_exch{"2H14 CRL"}="H142 CRL";
$resn_exch{"1H15 CRL"}="H151 CRL"; $resn_exch{"2H15 CRL"}="H152 CRL"; $resn_exch{"3H15 CRL"}="H153 CRL";
$resn_exch{"1H16 CRL"}="H161 CRL"; $resn_exch{"2H16 CRL"}="H162 CRL"; $resn_exch{"3H16 CRL"}="H163 CRL";
$resn_exch{"1H17 CRL"}="H171 CRL"; $resn_exch{"2H17 CRL"}="H172 CRL"; $resn_exch{"3H17 CRL"}="H173 CRL";
$resn_exch{"1H17 CRM"}="H171 CRM"; $resn_exch{"2H17 CRM"}="H172 CRM"; $resn_exch{"1H25 CRM"}="H251 CRM";
$resn_exch{"2H25 CRM"}="H252 CRM"; $resn_exch{"1H11 CRM"}="H111 CRM"; $resn_exch{"2H11 CRM"}="H112 CRM";
$resn_exch{"1H12 CRM"}="H121 CRM"; $resn_exch{"2H12 CRM"}="H122 CRM"; $resn_exch{"1H5  CRN"}=" H51 CRN";
$resn_exch{"2H5  CRN"}=" H52 CRN"; $resn_exch{"1H4  CRN"}=" H41 CRN"; $resn_exch{"2H4  CRN"}=" H42 CRN";
$resn_exch{"3H4  CRN"}=" H43 CRN"; $resn_exch{"1HN3 CRN"}="HN31 CRN"; $resn_exch{"2HN3 CRN"}="HN32 CRN";
$resn_exch{"1HN1 CRO"}="HN11 CRO"; $resn_exch{" HN2 CRO"}=" H2  CRO"; $resn_exch{"1HG1 CRO"}="HG11 CRO";
$resn_exch{"2HG1 CRO"}="HG12 CRO"; $resn_exch{"3HG1 CRO"}="HG13 CRO"; $resn_exch{"1HA3 CRO"}="HA31 CRO";
$resn_exch{"2HA3 CRO"}="HA32 CRO"; $resn_exch{"1H8' CRP"}="H8'1 CRP"; $resn_exch{"2H8' CRP"}="H8'2 CRP";
$resn_exch{"3H8' CRP"}="H8'3 CRP"; $resn_exch{"1H4  CRP"}=" H41 CRP"; $resn_exch{"2H4  CRP"}=" H42 CRP";
$resn_exch{"3H4  CRP"}=" H43 CRP"; $resn_exch{"1H5  CRP"}=" H51 CRP"; $resn_exch{"2H5  CRP"}=" H52 CRP";
$resn_exch{"1H6  CRP"}=" H61 CRP"; $resn_exch{"2H6  CRP"}=" H62 CRP"; $resn_exch{"3H6  CRP"}=" H63 CRP";
$resn_exch{"1HB1 CRQ"}="HB11 CRQ"; $resn_exch{"2HB1 CRQ"}="HB12 CRQ"; $resn_exch{"1HG1 CRQ"}="HG11 CRQ";
$resn_exch{"2HG1 CRQ"}="HG12 CRQ"; $resn_exch{"1HA3 CRQ"}="HA31 CRQ"; $resn_exch{"2HA3 CRQ"}="HA32 CRQ";
$resn_exch{"1HB  CRQ"}=" HB1 CRQ"; $resn_exch{"1HE1 CRQ"}="HE11 CRQ"; $resn_exch{"2HE1 CRQ"}="HE12 CRQ";
$resn_exch{"1H7  CRS"}=" H71 CRS"; $resn_exch{"2H7  CRS"}=" H72 CRS"; $resn_exch{"3H7  CRS"}=" H73 CRS";
$resn_exch{"1H1M CRT"}="H1M1 CRT"; $resn_exch{"2H1M CRT"}="H1M2 CRT"; $resn_exch{"3H1M CRT"}="H1M3 CRT";
$resn_exch{"1H2  CRT"}="H21A CRT"; $resn_exch{"2H2  CRT"}="H22A CRT"; $resn_exch{"3H2  CRT"}=" H23 CRT";
$resn_exch{"1H3  CRT"}="H31A CRT"; $resn_exch{"2H3  CRT"}="H32A CRT"; $resn_exch{"3H3  CRT"}=" H33 CRT";
$resn_exch{"1H4  CRT"}=" H41 CRT"; $resn_exch{"2H4  CRT"}=" H42 CRT"; $resn_exch{"1H8  CRT"}=" H81 CRT";
$resn_exch{"2H8  CRT"}=" H82 CRT"; $resn_exch{"3H8  CRT"}=" H83 CRT"; $resn_exch{"1H13 CRT"}="H131 CRT";
$resn_exch{"2H13 CRT"}="H132 CRT"; $resn_exch{"3H13 CRT"}="H133 CRT"; $resn_exch{"1H18 CRT"}="H181 CRT";
$resn_exch{"2H18 CRT"}="H182 CRT"; $resn_exch{"3H18 CRT"}="H183 CRT"; $resn_exch{"1H24 CRT"}="H241 CRT";
$resn_exch{"2H24 CRT"}="H242 CRT"; $resn_exch{"3H24 CRT"}="H243 CRT"; $resn_exch{"1H29 CRT"}="H291 CRT";
$resn_exch{"2H29 CRT"}="H292 CRT"; $resn_exch{"3H29 CRT"}="H293 CRT"; $resn_exch{"1H34 CRT"}="H341 CRT";
$resn_exch{"2H34 CRT"}="H342 CRT"; $resn_exch{"3H34 CRT"}="H343 CRT"; $resn_exch{"1H37 CRT"}="H371 CRT";
$resn_exch{"2H37 CRT"}="H372 CRT"; $resn_exch{"1H39 CRT"}="H391 CRT"; $resn_exch{"2H39 CRT"}="H392 CRT";
$resn_exch{"3H39 CRT"}="H393 CRT"; $resn_exch{"1H40 CRT"}="H401 CRT"; $resn_exch{"2H40 CRT"}="H402 CRT";
$resn_exch{"3H40 CRT"}="H403 CRT"; $resn_exch{"1H2M CRT"}="H2M1 CRT"; $resn_exch{"2H2M CRT"}="H2M2 CRT";
$resn_exch{"3H2M CRT"}="H2M3 CRT"; $resn_exch{"1HE2 CRU"}="HE21 CRU"; $resn_exch{"1HG1 CRU"}="HG11 CRU";
$resn_exch{"2HG1 CRU"}="HG12 CRU"; $resn_exch{"1HB1 CRU"}="HB11 CRU"; $resn_exch{"2HB1 CRU"}="HB12 CRU";
$resn_exch{"1HA3 CRU"}="HA31 CRU"; $resn_exch{"2HA3 CRU"}="HA32 CRU"; $resn_exch{"1HB  CRU"}=" HB1 CRU";
$resn_exch{"1H10 CRV"}="H101 CRV"; $resn_exch{"2H10 CRV"}="H102 CRV"; $resn_exch{"1H12 CRV"}="H121 CRV";
$resn_exch{"2H12 CRV"}="H122 CRV"; $resn_exch{"1H17 CRV"}="H171 CRV"; $resn_exch{"2H17 CRV"}="H172 CRV";
$resn_exch{"1H18 CRV"}="H181 CRV"; $resn_exch{"2H18 CRV"}="H182 CRV"; $resn_exch{"1H19 CRV"}="H191 CRV";
$resn_exch{"2H19 CRV"}="H192 CRV"; $resn_exch{"1H16 CRV"}="H161 CRV"; $resn_exch{"2H16 CRV"}="H162 CRV";
$resn_exch{"1HN1 CRW"}="HN11 CRW"; $resn_exch{"2HN1 CRW"}="HN12 CRW"; $resn_exch{"1HB1 CRW"}="HB11 CRW";
$resn_exch{"2HB1 CRW"}="HB12 CRW"; $resn_exch{"3HB1 CRW"}="HB13 CRW"; $resn_exch{"1HA3 CRW"}="HA31 CRW";
$resn_exch{"2HA3 CRW"}="HA32 CRW"; $resn_exch{"1HB2 CRW"}="HB21 CRW"; $resn_exch{"2HB2 CRW"}="HB22 CRW";
$resn_exch{"1HN1 CRX"}="HN11 CRX"; $resn_exch{"2HN1 CRX"}="HN12 CRX"; $resn_exch{"1HB1 CRX"}="HB11 CRX";
$resn_exch{"2HB1 CRX"}="HB12 CRX"; $resn_exch{"3HB1 CRX"}="HB13 CRX"; $resn_exch{"1HA3 CRX"}="HA31 CRX";
$resn_exch{"2HA3 CRX"}="HA32 CRX"; $resn_exch{"1HB2 CRX"}="HB21 CRX"; $resn_exch{"2HB2 CRX"}="HB22 CRX";
$resn_exch{"1H1  CRY"}=" H11 CRY"; $resn_exch{"2H1  CRY"}=" H12 CRY"; $resn_exch{"1H2  CRY"}=" H21 CRY";
$resn_exch{"1H3  CRY"}=" H31 CRY"; $resn_exch{"2H3  CRY"}=" H32 CRY"; $resn_exch{"1H2  CRZ"}=" H21 CRZ";
$resn_exch{"2H2  CRZ"}=" H22 CRZ"; $resn_exch{"1H3  CRZ"}=" H31 CRZ"; $resn_exch{"2H3  CRZ"}=" H32 CRZ";
$resn_exch{"1H4  CRZ"}=" H41 CRZ"; $resn_exch{"2H4  CRZ"}=" H42 CRZ"; $resn_exch{" HN1 CS1"}=" H   CS1";
$resn_exch{" HN2 CS1"}=" H2  CS1"; $resn_exch{" HB1 CS1"}=" HB2 CS1"; $resn_exch{" HB2 CS1"}=" HB3 CS1";
$resn_exch{" H2  CS1"}=" HC2 CS1"; $resn_exch{"1H6  CS2"}=" H61 CS2"; $resn_exch{"2H6  CS2"}=" H62 CS2";
$resn_exch{"1HN  CS3"}=" H   CS3"; $resn_exch{"2HN  CS3"}=" H2  CS3"; $resn_exch{"1HB  CS3"}=" HB2 CS3";
$resn_exch{"2HB  CS3"}=" HB3 CS3"; $resn_exch{"1H1H CS3"}="H1H1 CS3"; $resn_exch{"2H1H CS3"}="H1H2 CS3";
$resn_exch{"1H1I CS3"}="H1I1 CS3"; $resn_exch{"2H1I CS3"}="H1I2 CS3"; $resn_exch{"1HN  CS4"}=" H   CS4";
$resn_exch{"2HN  CS4"}=" H2  CS4"; $resn_exch{"1HB  CS4"}=" HB2 CS4"; $resn_exch{"2HB  CS4"}=" HB3 CS4";
$resn_exch{"1H1J CS4"}="H1J1 CS4"; $resn_exch{"2H1J CS4"}="H1J2 CS4"; $resn_exch{"1H1K CS4"}="H1K1 CS4";
$resn_exch{"2H1K CS4"}="H1K2 CS4"; $resn_exch{"1H8  CS5"}=" H81 CS5"; $resn_exch{"2H8  CS5"}=" H82 CS5";
$resn_exch{"1H9  CS5"}=" H91 CS5"; $resn_exch{"2H9  CS5"}=" H92 CS5"; $resn_exch{"1H10 CS5"}="H101 CS5";
$resn_exch{"2H10 CS5"}="H102 CS5"; $resn_exch{"3H10 CS5"}="H103 CS5"; $resn_exch{"1H11 CS5"}="H111 CS5";
$resn_exch{"2H11 CS5"}="H112 CS5"; $resn_exch{"1H12 CS5"}="H121 CS5"; $resn_exch{"2H12 CS5"}="H122 CS5";
$resn_exch{"1H13 CS5"}="H131 CS5"; $resn_exch{"2H13 CS5"}="H132 CS5"; $resn_exch{"3H13 CS5"}="H133 CS5";
$resn_exch{"1H17 CS5"}="H171 CS5"; $resn_exch{"2H17 CS5"}="H172 CS5"; $resn_exch{"1H18 CS5"}="H181 CS5";
$resn_exch{"2H18 CS5"}="H182 CS5"; $resn_exch{"1H19 CS5"}="H191 CS5"; $resn_exch{"2H19 CS5"}="H192 CS5";
$resn_exch{"1H32 CS5"}="H321 CS5"; $resn_exch{"2H32 CS5"}="H322 CS5"; $resn_exch{"3H32 CS5"}="H323 CS5";
$resn_exch{"1H33 CS5"}="H331 CS5"; $resn_exch{"2H33 CS5"}="H332 CS5"; $resn_exch{"3H33 CS5"}="H333 CS5";
$resn_exch{"1H4  CS7"}=" H41 CS7"; $resn_exch{"2H4  CS7"}=" H42 CS7"; $resn_exch{"1H6  CS7"}=" H61 CS7";
$resn_exch{"2H6  CS7"}=" H62 CS7"; $resn_exch{"1H7  CS7"}=" H71 CS7"; $resn_exch{"2H7  CS7"}=" H72 CS7";
$resn_exch{"1H22 CS7"}="H221 CS7"; $resn_exch{"2H22 CS7"}="H222 CS7"; $resn_exch{"3H22 CS7"}="H223 CS7";
$resn_exch{"1H23 CS7"}="H231 CS7"; $resn_exch{"2H23 CS7"}="H232 CS7"; $resn_exch{"1H30 CS7"}="H301 CS7";
$resn_exch{"2H30 CS7"}="H302 CS7"; $resn_exch{"1H31 CS7"}="H311 CS7"; $resn_exch{"2H31 CS7"}="H312 CS7";
$resn_exch{"1H32 CS7"}="H321 CS7"; $resn_exch{"2H32 CS7"}="H322 CS7"; $resn_exch{"3H32 CS7"}="H323 CS7";
$resn_exch{"1H33 CS7"}="H331 CS7"; $resn_exch{"2H33 CS7"}="H332 CS7"; $resn_exch{"1H34 CS7"}="H341 CS7";
$resn_exch{"2H34 CS7"}="H342 CS7"; $resn_exch{"1H35 CS7"}="H351 CS7"; $resn_exch{"2H35 CS7"}="H352 CS7";
$resn_exch{"3H35 CS7"}="H353 CS7"; $resn_exch{"1H7  CS8"}=" H71 CS8"; $resn_exch{"2H7  CS8"}=" H72 CS8";
$resn_exch{"1H6  CS8"}=" H61 CS8"; $resn_exch{"2H6  CS8"}=" H62 CS8"; $resn_exch{"1H3  CS8"}=" H31 CS8";
$resn_exch{"2H3  CS8"}=" H32 CS8"; $resn_exch{"1H2  CS8"}=" H21 CS8"; $resn_exch{"2H2  CS8"}=" H22 CS8";
$resn_exch{"1H2* CS8"}="H2'1 CS8"; $resn_exch{"2H2* CS8"}="H2'2 CS8"; $resn_exch{"1H4* CS8"}="H4'1 CS8";
$resn_exch{"2H4* CS8"}="H4'2 CS8"; $resn_exch{"1H5* CS8"}="H5'1 CS8"; $resn_exch{"2H5* CS8"}="H5'2 CS8";
$resn_exch{"1H6* CS8"}="H6'1 CS8"; $resn_exch{"2H6* CS8"}="H6'2 CS8"; $resn_exch{"1H7* CS8"}="H7'1 CS8";
$resn_exch{"2H7* CS8"}="H7'2 CS8"; $resn_exch{"1H8* CS8"}="H8'1 CS8"; $resn_exch{"2H8* CS8"}="H8'2 CS8";
$resn_exch{"3H8* CS8"}="H8'3 CS8"; $resn_exch{"1H13 CS8"}="H131 CS8"; $resn_exch{"2H13 CS8"}="H132 CS8";
$resn_exch{"3H13 CS8"}="H133 CS8"; $resn_exch{"1H14 CS8"}="H141 CS8"; $resn_exch{"2H14 CS8"}="H142 CS8";
$resn_exch{"3H14 CS8"}="H143 CS8"; $resn_exch{"1H12 CS8"}="H121 CS8"; $resn_exch{"2H12 CS8"}="H122 CS8";
$resn_exch{"1H1  CS8"}=" H11 CS8"; $resn_exch{"2H1  CS8"}=" H12 CS8"; $resn_exch{" H4* CS8"}=" H4' CS8";
$resn_exch{" H1* CS8"}=" H1' CS8"; $resn_exch{"1HN6 CS8"}="HN61 CS8"; $resn_exch{"2HN6 CS8"}="HN62 CS8";
$resn_exch{" H2* CS8"}=" H2' CS8"; $resn_exch{" H3* CS8"}=" H3' CS8"; $resn_exch{"1H8  CS9"}=" H81 CS9";
$resn_exch{"2H8  CS9"}=" H82 CS9"; $resn_exch{"1H9  CS9"}=" H91 CS9"; $resn_exch{"2H9  CS9"}=" H92 CS9";
$resn_exch{"1H10 CS9"}="H101 CS9"; $resn_exch{"2H10 CS9"}="H102 CS9"; $resn_exch{"3H10 CS9"}="H103 CS9";
$resn_exch{"1H11 CS9"}="H111 CS9"; $resn_exch{"2H11 CS9"}="H112 CS9"; $resn_exch{"1H12 CS9"}="H121 CS9";
$resn_exch{"2H12 CS9"}="H122 CS9"; $resn_exch{"1H13 CS9"}="H131 CS9"; $resn_exch{"2H13 CS9"}="H132 CS9";
$resn_exch{"3H13 CS9"}="H133 CS9"; $resn_exch{"1H30 CS9"}="H301 CS9"; $resn_exch{"2H30 CS9"}="H302 CS9";
$resn_exch{"3H30 CS9"}="H303 CS9"; $resn_exch{"1H17 CS9"}="H171 CS9"; $resn_exch{"2H17 CS9"}="H172 CS9";
$resn_exch{"1H18 CS9"}="H181 CS9"; $resn_exch{"2H18 CS9"}="H182 CS9"; $resn_exch{"1H20 CS9"}="H201 CS9";
$resn_exch{"2H20 CS9"}="H202 CS9"; $resn_exch{"1HN  CSA"}=" H   CSA"; $resn_exch{"2HN  CSA"}=" H2  CSA";
$resn_exch{"1HB  CSA"}=" HB2 CSA"; $resn_exch{"2HB  CSA"}=" HB3 CSA"; $resn_exch{"1H3  CSA"}=" H31 CSA";
$resn_exch{"2H3  CSA"}=" H32 CSA"; $resn_exch{"1H1  CSA"}=" H11 CSA"; $resn_exch{"2H1  CSA"}=" H12 CSA";
$resn_exch{"3H1  CSA"}=" H13 CSA"; $resn_exch{"1HN  CSB"}=" H   CSB"; $resn_exch{"2HN  CSB"}=" H2  CSB";
$resn_exch{"1HB  CSB"}=" HB2 CSB"; $resn_exch{"2HB  CSB"}=" HB3 CSB"; $resn_exch{"1H1  CSC"}=" H11 CSC";
$resn_exch{"2H1  CSC"}=" H12 CSC"; $resn_exch{"1H3' CSC"}="H3'1 CSC"; $resn_exch{"2H3' CSC"}="H3'2 CSC";
$resn_exch{"1H13 CSC"}="H131 CSC"; $resn_exch{"2H13 CSC"}="H132 CSC"; $resn_exch{"1H14 CSC"}="H141 CSC";
$resn_exch{"2H14 CSC"}="H142 CSC"; $resn_exch{"1H15 CSC"}="H151 CSC"; $resn_exch{"2H15 CSC"}="H152 CSC";
$resn_exch{"1H17 CSC"}="H171 CSC"; $resn_exch{"2H17 CSC"}="H172 CSC"; $resn_exch{"3H17 CSC"}="H173 CSC";
$resn_exch{"1H20 CSC"}="H201 CSC"; $resn_exch{"2H20 CSC"}="H202 CSC"; $resn_exch{"3H20 CSC"}="H203 CSC";
$resn_exch{"1HN  CSD"}=" H   CSD"; $resn_exch{"2HN  CSD"}=" H2  CSD"; $resn_exch{"1HB  CSD"}=" HB2 CSD";
$resn_exch{"2HB  CSD"}=" HB3 CSD"; $resn_exch{"1HN  CSE"}=" H   CSE"; $resn_exch{"2HN  CSE"}=" H2  CSE";
$resn_exch{"1HB  CSE"}=" HB2 CSE"; $resn_exch{"2HB  CSE"}=" HB3 CSE"; $resn_exch{"1H9A CSF"}="H9A1 CSF";
$resn_exch{"2H9A CSF"}="H9A2 CSF"; $resn_exch{" H3* CSF"}=" H3' CSF"; $resn_exch{" H2* CSF"}=" H2' CSF";
$resn_exch{" H1* CSF"}=" H1' CSF"; $resn_exch{" H4* CSF"}=" H4' CSF"; $resn_exch{"1H5* CSF"}=" H5' CSF";
$resn_exch{"2H5* CSF"}="H5'' CSF"; $resn_exch{"H3T  CSF"}="HO3' CSF"; $resn_exch{"1H4  CSF"}=" H41 CSF";
$resn_exch{"2H4  CSF"}=" H42 CSF"; $resn_exch{"1H11 CSF"}="H111 CSF"; $resn_exch{"2H11 CSF"}="H112 CSF";
$resn_exch{"3H11 CSF"}="H113 CSF"; $resn_exch{"1HA3 CSH"}="HA31 CSH"; $resn_exch{"2HA3 CSH"}="HA32 CSH";
$resn_exch{"1HN1 CSH"}="HN11 CSH"; $resn_exch{"2HN1 CSH"}="HN12 CSH"; $resn_exch{"1HB1 CSH"}="HB11 CSH";
$resn_exch{"2HB1 CSH"}="HB12 CSH"; $resn_exch{"1HB2 CSH"}="HB21 CSH"; $resn_exch{"2HB2 CSH"}="HB22 CSH";
$resn_exch{" HNE CSI"}=" HE  CSI"; $resn_exch{"1HCD CSI"}="HD13 CSI"; $resn_exch{"2HCD CSI"}="HD12 CSI";
$resn_exch{" HNH CSI"}=" HH  CSI"; $resn_exch{" HCB CSI"}=" HB  CSI"; $resn_exch{"1HCG CSI"}="HG13 CSI";
$resn_exch{"2HCG CSI"}="HG12 CSI"; $resn_exch{" HNG CSI"}=" HG2 CSI"; $resn_exch{"1HN  CSI"}=" H   CSI";
$resn_exch{"2HN  CSI"}=" H2  CSI"; $resn_exch{"2HOP CSL"}="HOP2 CSL"; $resn_exch{"1H5* CSL"}="H5'1 CSL";
$resn_exch{"2H5* CSL"}="H5'2 CSL"; $resn_exch{" H4* CSL"}=" H4' CSL"; $resn_exch{" H1* CSL"}=" H1' CSL";
$resn_exch{"1HN4 CSL"}="HN41 CSL"; $resn_exch{"2HN4 CSL"}="HN42 CSL"; $resn_exch{" H2* CSL"}=" H2' CSL";
$resn_exch{"1HA* CSL"}="HA'1 CSL"; $resn_exch{"2HA* CSL"}="HA'2 CSL"; $resn_exch{"3HA* CSL"}="HA'3 CSL";
$resn_exch{" H3* CSL"}=" H3' CSL"; $resn_exch{"3HOP CSL"}="HOP3 CSL"; $resn_exch{"1HN  CSO"}=" H   CSO";
$resn_exch{"2HN  CSO"}=" HN2 CSO"; $resn_exch{"1HB  CSO"}=" HB2 CSO"; $resn_exch{"2HB  CSO"}=" HB3 CSO";
$resn_exch{"1HN  CSP"}=" H   CSP"; $resn_exch{"2HN  CSP"}=" H2  CSP"; $resn_exch{"1HB  CSP"}=" HB2 CSP";
$resn_exch{"2HB  CSP"}=" HB3 CSP"; $resn_exch{"PHO2 CSP"}="HO2P CSP"; $resn_exch{"PHO3 CSP"}="HO3P CSP";
$resn_exch{" HN2 CSR"}=" H2  CSR"; $resn_exch{"1HB  CSR"}=" HB2 CSR"; $resn_exch{"2HB  CSR"}=" HB3 CSR";
$resn_exch{"1HN  CSS"}=" H   CSS"; $resn_exch{"2HN  CSS"}=" HN2 CSS"; $resn_exch{"1HB  CSS"}=" HB2 CSS";
$resn_exch{"2HB  CSS"}=" HB3 CSS"; $resn_exch{"1H7  CST"}=" H71 CST"; $resn_exch{"2H7  CST"}=" H72 CST";
$resn_exch{"1H12 CST"}="H121 CST"; $resn_exch{"2H12 CST"}="H122 CST"; $resn_exch{"1H14 CST"}="H141 CST";
$resn_exch{"2H14 CST"}="H142 CST"; $resn_exch{"3H14 CST"}="H143 CST"; $resn_exch{"1H15 CST"}="H151 CST";
$resn_exch{"2H15 CST"}="H152 CST"; $resn_exch{"3H15 CST"}="H153 CST"; $resn_exch{"1H19 CST"}="H191 CST";
$resn_exch{"2H19 CST"}="H192 CST"; $resn_exch{"1H20 CST"}="H201 CST"; $resn_exch{"2H20 CST"}="H202 CST";
$resn_exch{"1H25 CST"}="H251 CST"; $resn_exch{"2H25 CST"}="H252 CST"; $resn_exch{"1HN4 CST"}="HN41 CST";
$resn_exch{"2HN4 CST"}="HN42 CST"; $resn_exch{"1HN  CSU"}=" H   CSU"; $resn_exch{"2HN  CSU"}=" H2  CSU";
$resn_exch{"1HB  CSU"}=" HB2 CSU"; $resn_exch{"2HB  CSU"}=" HB3 CSU"; $resn_exch{"1HN  CSW"}=" H   CSW";
$resn_exch{"2HN  CSW"}=" H2  CSW"; $resn_exch{"1HB  CSW"}=" HB2 CSW"; $resn_exch{"2HB  CSW"}=" HB3 CSW";
$resn_exch{" HS  CSW"}=" HG  CSW"; $resn_exch{"1HN  CSX"}=" H   CSX"; $resn_exch{"2HN  CSX"}=" H2  CSX";
$resn_exch{"1HB  CSX"}=" HB2 CSX"; $resn_exch{"2HB  CSX"}=" HB3 CSX"; $resn_exch{" HS  CSX"}=" HG  CSX";
$resn_exch{"1HA3 CSY"}="HA31 CSY"; $resn_exch{"2HA3 CSY"}="HA32 CSY"; $resn_exch{"1HN1 CSY"}="HN11 CSY";
$resn_exch{"2HN1 CSY"}="HN12 CSY"; $resn_exch{"1HB1 CSY"}="HB11 CSY"; $resn_exch{"2HB1 CSY"}="HB12 CSY";
$resn_exch{"1HB2 CSY"}="HB21 CSY"; $resn_exch{"2HB2 CSY"}="HB22 CSY"; $resn_exch{" HN2 CSZ"}=" H2  CSZ";
$resn_exch{"1HB  CSZ"}=" HB2 CSZ"; $resn_exch{"2HB  CSZ"}=" HB3 CSZ"; $resn_exch{"HSE1 CSZ"}=" HE  CSZ";
$resn_exch{"1H9  CT0"}=" H91 CT0"; $resn_exch{"2H9  CT0"}=" H92 CT0"; $resn_exch{"1H10 CT0"}="H101 CT0";
$resn_exch{"2H10 CT0"}="H102 CT0"; $resn_exch{"1H11 CT0"}="H111 CT0"; $resn_exch{"2H11 CT0"}="H112 CT0";
$resn_exch{"1H12 CT0"}="H121 CT0"; $resn_exch{"2H12 CT0"}="H122 CT0"; $resn_exch{"1H1  CT1"}=" H11 CT1";
$resn_exch{"2H1  CT1"}="H12A CT1"; $resn_exch{"3H1  CT1"}=" H13 CT1"; $resn_exch{"1H2  CT1"}=" H21 CT1";
$resn_exch{"2H2  CT1"}=" H22 CT1"; $resn_exch{"1H3  CT1"}=" H31 CT1"; $resn_exch{"2H3  CT1"}=" H32 CT1";
$resn_exch{"1H4  CT1"}=" H41 CT1"; $resn_exch{"2H4  CT1"}=" H42 CT1"; $resn_exch{"1H13 CT1"}="H131 CT1";
$resn_exch{"2H13 CT1"}="H132 CT1"; $resn_exch{"1H21 CT1"}="H211 CT1"; $resn_exch{"2H21 CT1"}="H212 CT1";
$resn_exch{"3H21 CT1"}="H213 CT1"; $resn_exch{"1H22 CT1"}="H221 CT1"; $resn_exch{"2H22 CT1"}="H222 CT1";
$resn_exch{"3H22 CT1"}="H223 CT1"; $resn_exch{"1H1  CT2"}=" H11 CT2"; $resn_exch{"2H1  CT2"}=" H12 CT2";
$resn_exch{"3H1  CT2"}=" H13 CT2"; $resn_exch{"1H2  CT2"}=" H21 CT2"; $resn_exch{"2H2  CT2"}=" H22 CT2";
$resn_exch{"1H3  CT2"}=" H31 CT2"; $resn_exch{"2H3  CT2"}=" H32 CT2"; $resn_exch{"1H4  CT2"}=" H41 CT2";
$resn_exch{"2H4  CT2"}=" H42 CT2"; $resn_exch{"1H13 CT2"}="H131 CT2"; $resn_exch{"2H13 CT2"}="H132 CT2";
$resn_exch{"1H20 CT2"}="H201 CT2"; $resn_exch{"2H20 CT2"}="H202 CT2"; $resn_exch{"1H21 CT2"}="H211 CT2";
$resn_exch{"2H21 CT2"}="H212 CT2"; $resn_exch{"1H22 CT2"}="H221 CT2"; $resn_exch{"2H22 CT2"}="H222 CT2";
$resn_exch{"1H23 CT2"}="H231 CT2"; $resn_exch{"2H23 CT2"}="H232 CT2"; $resn_exch{"1H6A CT3"}="H6A1 CT3";
$resn_exch{"2H6A CT3"}="H6A2 CT3"; $resn_exch{"2HOA CT3"}="HOA2 CT3"; $resn_exch{"3HOA CT3"}="HOA3 CT3";
$resn_exch{"6HOA CT3"}="HOA6 CT3"; $resn_exch{"1H6B CT3"}="H6B1 CT3"; $resn_exch{"2H6B CT3"}="H6B2 CT3";
$resn_exch{"2HOB CT3"}="HOB2 CT3"; $resn_exch{"3HOB CT3"}="HOB3 CT3"; $resn_exch{"6HOB CT3"}="HOB6 CT3";
$resn_exch{"1H6C CT3"}="H6C1 CT3"; $resn_exch{"2H6C CT3"}="H6C2 CT3"; $resn_exch{"2HOC CT3"}="HOC2 CT3";
$resn_exch{"3HOC CT3"}="HOC3 CT3"; $resn_exch{"4HOC CT3"}="HOC4 CT3"; $resn_exch{"6HOC CT3"}="HOC6 CT3";
$resn_exch{"1H7C CT5"}="H7C1 CT5"; $resn_exch{"2H7C CT5"}="H7C2 CT5"; $resn_exch{"3H7C CT5"}="H7C3 CT5";
$resn_exch{"1H26 CT5"}="H261 CT5"; $resn_exch{"2H26 CT5"}="H262 CT5"; $resn_exch{"1H27 CT5"}="H271 CT5";
$resn_exch{"2H27 CT5"}="H272 CT5"; $resn_exch{"3H27 CT5"}="H273 CT5"; $resn_exch{"1H17 CT5"}="H171 CT5";
$resn_exch{"2H17 CT5"}="H172 CT5"; $resn_exch{"1H18 CT5"}="H181 CT5"; $resn_exch{"2H18 CT5"}="H182 CT5";
$resn_exch{"1H13 CT6"}="H131 CT6"; $resn_exch{"2H13 CT6"}="H132 CT6"; $resn_exch{"1H20 CT7"}="H201 CT7";
$resn_exch{"2H20 CT7"}="H202 CT7"; $resn_exch{"3H20 CT7"}="H203 CT7"; $resn_exch{"1H13 CT8"}="H131 CT8";
$resn_exch{"2H13 CT8"}="H132 CT8"; $resn_exch{"1HN8 CT9"}="HN81 CT9"; $resn_exch{"2HN8 CT9"}="HN82 CT9";
$resn_exch{"1H61 CT9"}="H611 CT9"; $resn_exch{"2H61 CT9"}="H612 CT9"; $resn_exch{"3H61 CT9"}="H613 CT9";
$resn_exch{"1H62 CT9"}="H621 CT9"; $resn_exch{"2H62 CT9"}="H622 CT9"; $resn_exch{"3H62 CT9"}="H623 CT9";
$resn_exch{"1H5  CT9"}=" H51 CT9"; $resn_exch{"2H5  CT9"}=" H52 CT9"; $resn_exch{"1H6  CT9"}=" H61 CT9";
$resn_exch{"2H6  CT9"}=" H62 CT9"; $resn_exch{"1H3  CT9"}=" H31 CT9"; $resn_exch{"2H3  CT9"}=" H32 CT9";
$resn_exch{"1H2  CT9"}=" H21 CT9"; $resn_exch{"2H2  CT9"}=" H22 CT9"; $resn_exch{"1H21 CT9"}="H211 CT9";
$resn_exch{"2H21 CT9"}="H212 CT9"; $resn_exch{"3H21 CT9"}="H213 CT9"; $resn_exch{"1H22 CT9"}="H221 CT9";
$resn_exch{"2H22 CT9"}="H222 CT9"; $resn_exch{"3H22 CT9"}="H223 CT9"; $resn_exch{"1H15 CTA"}="H151 CTA";
$resn_exch{"2H15 CTA"}="H152 CTA"; $resn_exch{"1H16 CTA"}="H161 CTA"; $resn_exch{"2H16 CTA"}="H162 CTA";
$resn_exch{"1H17 CTA"}="H171 CTA"; $resn_exch{"2H17 CTA"}="H172 CTA"; $resn_exch{"1H21 CTA"}="H211 CTA";
$resn_exch{"2H21 CTA"}="H212 CTA"; $resn_exch{"1H24 CTA"}="H241 CTA"; $resn_exch{"2H24 CTA"}="H242 CTA";
$resn_exch{"1H33 CTA"}="H331 CTA"; $resn_exch{"2H33 CTA"}="H332 CTA"; $resn_exch{"1H42 CTA"}="H421 CTA";
$resn_exch{"2H42 CTA"}="H422 CTA"; $resn_exch{"1H43 CTA"}="H431 CTA"; $resn_exch{"2H43 CTA"}="H432 CTA";
$resn_exch{"1H44 CTA"}="H441 CTA"; $resn_exch{"2H44 CTA"}="H442 CTA"; $resn_exch{"1H47 CTA"}="H471 CTA";
$resn_exch{"2H47 CTA"}="H472 CTA"; $resn_exch{"1H3  CTB"}=" H31 CTB"; $resn_exch{"2H3  CTB"}=" H32 CTB";
$resn_exch{"1H7  CTB"}=" H71 CTB"; $resn_exch{"2H7  CTB"}=" H72 CTB"; $resn_exch{"1HN2 CTC"}="HN21 CTC";
$resn_exch{"2HN2 CTC"}="HN22 CTC"; $resn_exch{"1H4' CTC"}="H4'1 CTC"; $resn_exch{"2H4' CTC"}="H4'2 CTC";
$resn_exch{"3H4' CTC"}="H4'3 CTC"; $resn_exch{"1H4\" CTC"}="H4A1 CTC"; $resn_exch{"2H4\" CTC"}="H4A2 CTC";
$resn_exch{"3H4\" CTC"}="H4A3 CTC"; $resn_exch{"1H5  CTC"}=" H51 CTC"; $resn_exch{"2H5  CTC"}=" H52 CTC";
$resn_exch{"1H6' CTC"}="H6'1 CTC"; $resn_exch{"2H6' CTC"}="H6'2 CTC"; $resn_exch{"3H6' CTC"}="H6'3 CTC";
$resn_exch{"1HN4 CTD"}="HN41 CTD"; $resn_exch{"2HN4 CTD"}="HN42 CTD"; $resn_exch{" H1* CTD"}=" H1' CTD";
$resn_exch{" H2* CTD"}=" H2' CTD"; $resn_exch{"*HO2 CTD"}="HO2' CTD"; $resn_exch{" H3* CTD"}=" H3' CTD";
$resn_exch{" H4* CTD"}=" H4' CTD"; $resn_exch{"*HO3 CTD"}="HO3' CTD"; $resn_exch{"1H5* CTD"}="H5'1 CTD";
$resn_exch{"2H5* CTD"}="H5'2 CTD"; $resn_exch{"*HO5 CTD"}="HO5' CTD"; $resn_exch{"1HN  CTE"}=" HN1 CTE";
$resn_exch{"2HN  CTE"}=" HN2 CTE"; $resn_exch{"1HB  CTE"}=" HB1 CTE"; $resn_exch{"2HB  CTE"}=" HB2 CTE";
$resn_exch{"1H5M CTG"}=" H71 CTG"; $resn_exch{"2H5M CTG"}=" H72 CTG"; $resn_exch{"3H5M CTG"}=" H73 CTG";
$resn_exch{" H1* CTG"}=" H1' CTG"; $resn_exch{"1H2* CTG"}=" H2' CTG"; $resn_exch{"2H2* CTG"}="H2'' CTG";
$resn_exch{" H3* CTG"}=" H3' CTG"; $resn_exch{"H3T  CTG"}="HO3' CTG"; $resn_exch{" H4* CTG"}=" H4' CTG";
$resn_exch{"1H5* CTG"}=" H5' CTG"; $resn_exch{"2H5* CTG"}="H5'' CTG"; $resn_exch{"3HOP CTG"}="HOP3 CTG";
$resn_exch{"2HOP CTG"}="HOP2 CTG"; $resn_exch{"1HN  CTH"}=" H   CTH"; $resn_exch{"2HN  CTH"}=" H2  CTH";
$resn_exch{"1HG2 CTH"}="HG22 CTH"; $resn_exch{"2HG2 CTH"}="HG23 CTH"; $resn_exch{" HO1 CTH"}=" HG1 CTH";
$resn_exch{"1H3  CTL"}=" H31 CTL"; $resn_exch{"2H3  CTL"}=" H32 CTL"; $resn_exch{"1H4  CTL"}=" H41 CTL";
$resn_exch{"2H4  CTL"}=" H42 CTL"; $resn_exch{"1H12 CTL"}="H121 CTL"; $resn_exch{"2H12 CTL"}="H122 CTL";
$resn_exch{" H5* CTN"}=" H5' CTN"; $resn_exch{"1H5* CTN"}="H5'1 CTN"; $resn_exch{"2H5* CTN"}="H5'2 CTN";
$resn_exch{" H4* CTN"}=" H4' CTN"; $resn_exch{" H1* CTN"}=" H1' CTN"; $resn_exch{"1HN4 CTN"}="HN41 CTN";
$resn_exch{"2HN4 CTN"}="HN42 CTN"; $resn_exch{" H2* CTN"}=" H2' CTN"; $resn_exch{" H3* CTN"}=" H3' CTN";
$resn_exch{"1H81 CTO"}="H811 CTO"; $resn_exch{"2H81 CTO"}="H812 CTO"; $resn_exch{"3H81 CTO"}="H813 CTO";
$resn_exch{"1H61 CTO"}="H611 CTO"; $resn_exch{"2H61 CTO"}="H612 CTO"; $resn_exch{"1H82 CTO"}="H821 CTO";
$resn_exch{"2H82 CTO"}="H822 CTO"; $resn_exch{"3H82 CTO"}="H823 CTO"; $resn_exch{"1H62 CTO"}="H621 CTO";
$resn_exch{"2H62 CTO"}="H622 CTO"; $resn_exch{"1H83 CTO"}="H831 CTO"; $resn_exch{"2H83 CTO"}="H832 CTO";
$resn_exch{"3H83 CTO"}="H833 CTO"; $resn_exch{"1H63 CTO"}="H631 CTO"; $resn_exch{"2H63 CTO"}="H632 CTO";
$resn_exch{"1HN4 CTP"}="HN41 CTP"; $resn_exch{"2HN4 CTP"}="HN42 CTP"; $resn_exch{" H1* CTP"}=" H1' CTP";
$resn_exch{" H2* CTP"}=" H2' CTP"; $resn_exch{"*HO2 CTP"}="HO2' CTP"; $resn_exch{" H3* CTP"}=" H3' CTP";
$resn_exch{" H4* CTP"}=" H4' CTP"; $resn_exch{"*HO3 CTP"}="HO3' CTP"; $resn_exch{"1H5* CTP"}="H5'1 CTP";
$resn_exch{"2H5* CTP"}="H5'2 CTP"; $resn_exch{"2HOA CTP"}="HOA2 CTP"; $resn_exch{"2HOB CTP"}="HOB2 CTP";
$resn_exch{"2HOG CTP"}="HOG2 CTP"; $resn_exch{"3HOG CTP"}="HOG3 CTP"; $resn_exch{"1H6A CTR"}="H6A1 CTR";
$resn_exch{"2H6A CTR"}="H6A2 CTR"; $resn_exch{"AHO2 CTR"}="HO2A CTR"; $resn_exch{"AHO3 CTR"}="HO3A CTR";
$resn_exch{"AHO4 CTR"}="HO4A CTR"; $resn_exch{"AHO6 CTR"}="HO6A CTR"; $resn_exch{"1H6B CTR"}="H6B1 CTR";
$resn_exch{"2H6B CTR"}="H6B2 CTR"; $resn_exch{"BHO2 CTR"}="HO2B CTR"; $resn_exch{"BHO3 CTR"}="HO3B CTR";
$resn_exch{"BHO6 CTR"}="HO6B CTR"; $resn_exch{"1H6C CTR"}="H6C1 CTR"; $resn_exch{"2H6C CTR"}="H6C2 CTR";
$resn_exch{"CHO1 CTR"}="HO1C CTR"; $resn_exch{"CHO2 CTR"}="HO2C CTR"; $resn_exch{"CHO3 CTR"}="HO3C CTR";
$resn_exch{"CHO6 CTR"}="HO6C CTR"; $resn_exch{"1H1  CTS"}=" H11 CTS"; $resn_exch{"2H1  CTS"}=" H12 CTS";
$resn_exch{"1H7  CTS"}=" H71 CTS"; $resn_exch{"2H7  CTS"}=" H72 CTS"; $resn_exch{"1H8  CTS"}=" H81 CTS";
$resn_exch{"2H8  CTS"}=" H82 CTS"; $resn_exch{"1H6A CTT"}="H6A1 CTT"; $resn_exch{"2H6A CTT"}="H6A2 CTT";
$resn_exch{"2HOA CTT"}="HOA2 CTT"; $resn_exch{"3HOA CTT"}="HOA3 CTT"; $resn_exch{"6HOA CTT"}="HOA6 CTT";
$resn_exch{"1H6B CTT"}="H6B1 CTT"; $resn_exch{"2H6B CTT"}="H6B2 CTT"; $resn_exch{"2HOB CTT"}="HOB2 CTT";
$resn_exch{"3HOB CTT"}="HOB3 CTT"; $resn_exch{"6HOB CTT"}="HOB6 CTT"; $resn_exch{"1H6C CTT"}="H6C1 CTT";
$resn_exch{"2H6C CTT"}="H6C2 CTT"; $resn_exch{"2HOC CTT"}="HOC2 CTT"; $resn_exch{"3HOC CTT"}="HOC3 CTT";
$resn_exch{"6HOC CTT"}="HOC6 CTT"; $resn_exch{"1H6D CTT"}="H6D1 CTT"; $resn_exch{"2H6D CTT"}="H6D2 CTT";
$resn_exch{"2HOD CTT"}="HOD2 CTT"; $resn_exch{"3HOD CTT"}="HOD3 CTT"; $resn_exch{"4HOD CTT"}="HOD4 CTT";
$resn_exch{"6HOD CTT"}="HOD6 CTT"; $resn_exch{"1HA  CTX"}=" HA1 CTX"; $resn_exch{"2HA  CTX"}=" HA2 CTX";
$resn_exch{"3HA  CTX"}=" HA3 CTX"; $resn_exch{"1HB  CTX"}=" HB1 CTX"; $resn_exch{"2HB  CTX"}=" HB2 CTX";
$resn_exch{"1HA1 CTX"}="HA11 CTX"; $resn_exch{"2HA1 CTX"}="HA12 CTX"; $resn_exch{"1HB1 CTX"}="HB11 CTX";
$resn_exch{"2HB1 CTX"}="HB12 CTX"; $resn_exch{"1H21 CTX"}="H211 CTX"; $resn_exch{"2H21 CTX"}="H212 CTX";
$resn_exch{"3H21 CTX"}="H213 CTX"; $resn_exch{"1H22 CTX"}="H221 CTX"; $resn_exch{"2H22 CTX"}="H222 CTX";
$resn_exch{"3H22 CTX"}="H223 CTX"; $resn_exch{"1H7  CTY"}=" H71 CTY"; $resn_exch{"2H7  CTY"}=" H72 CTY";
$resn_exch{"1H34 CTY"}="H341 CTY"; $resn_exch{"2H34 CTY"}="H342 CTY"; $resn_exch{"3H34 CTY"}="H343 CTY";
$resn_exch{"1H33 CTY"}="H331 CTY"; $resn_exch{"2H33 CTY"}="H332 CTY"; $resn_exch{"3H33 CTY"}="H333 CTY";
$resn_exch{"1H35 CTY"}="H351 CTY"; $resn_exch{"2H35 CTY"}="H352 CTY"; $resn_exch{"3H35 CTY"}="H353 CTY";
$resn_exch{"1H36 CTY"}="H361 CTY"; $resn_exch{"2H36 CTY"}="H362 CTY"; $resn_exch{"1H30 CTY"}="H301 CTY";
$resn_exch{"2H30 CTY"}="H302 CTY"; $resn_exch{"3H30 CTY"}="H303 CTY"; $resn_exch{"1H32 CTY"}="H321 CTY";
$resn_exch{"2H32 CTY"}="H322 CTY"; $resn_exch{"3H32 CTY"}="H323 CTY"; $resn_exch{"1H25 CTY"}="H251 CTY";
$resn_exch{"2H25 CTY"}="H252 CTY"; $resn_exch{"1H27 CTY"}="H271 CTY"; $resn_exch{"2H27 CTY"}="H272 CTY";
$resn_exch{"3H27 CTY"}="H273 CTY"; $resn_exch{"1H28 CTY"}="H281 CTY"; $resn_exch{"2H28 CTY"}="H282 CTY";
$resn_exch{"3H28 CTY"}="H283 CTY"; $resn_exch{"1H15 CTY"}="H151 CTY"; $resn_exch{"2H15 CTY"}="H152 CTY";
$resn_exch{"1H20 CTY"}="H201 CTY"; $resn_exch{"2H20 CTY"}="H202 CTY"; $resn_exch{"3H20 CTY"}="H203 CTY";
$resn_exch{"1H29 CTY"}="H291 CTY"; $resn_exch{"2H29 CTY"}="H292 CTY"; $resn_exch{"3H29 CTY"}="H293 CTY";
$resn_exch{"1H21 CTY"}="H211 CTY"; $resn_exch{"2H21 CTY"}="H212 CTY"; $resn_exch{"3H21 CTY"}="H213 CTY";
$resn_exch{"1H37 CTY"}="H371 CTY"; $resn_exch{"2H37 CTY"}="H372 CTY"; $resn_exch{"3H37 CTY"}="H373 CTY";
$resn_exch{"1H31 CTY"}="H311 CTY"; $resn_exch{"2H31 CTY"}="H312 CTY"; $resn_exch{"3H31 CTY"}="H313 CTY";
$resn_exch{"1H19 CTY"}="H191 CTY"; $resn_exch{"2H19 CTY"}="H192 CTY"; $resn_exch{"3H19 CTY"}="H193 CTY";
$resn_exch{"1H38 CTY"}="H381 CTY"; $resn_exch{"2H38 CTY"}="H382 CTY"; $resn_exch{"3H38 CTY"}="H383 CTY";
$resn_exch{"1H10 CTZ"}="H101 CTZ"; $resn_exch{"2H10 CTZ"}="H102 CTZ"; $resn_exch{"1H26 CTZ"}="H261 CTZ";
$resn_exch{"2H26 CTZ"}="H262 CTZ"; $resn_exch{"1H3  CUB"}=" H31 CUB"; $resn_exch{"2H3  CUB"}=" H32 CUB";
$resn_exch{"1H2  CUB"}=" H21 CUB"; $resn_exch{"2H2  CUB"}=" H22 CUB"; $resn_exch{"1H1  CUB"}=" H11 CUB";
$resn_exch{"2H1  CUB"}=" H12 CUB"; $resn_exch{"1H4  CUB"}=" H41 CUB"; $resn_exch{"2H4  CUB"}=" H42 CUB";
$resn_exch{"3H4  CUB"}=" H43 CUB"; $resn_exch{"1H10 CUP"}="H101 CUP"; $resn_exch{"2H10 CUP"}="H102 CUP";
$resn_exch{"1H10 CUS"}="H101 CUS"; $resn_exch{"2H10 CUS"}="H102 CUS"; $resn_exch{"1H16 CUS"}="H161 CUS";
$resn_exch{"2H16 CUS"}="H162 CUS"; $resn_exch{"3H16 CUS"}="H163 CUS"; $resn_exch{"1H12 CUS"}="H121 CUS";
$resn_exch{"2H12 CUS"}="H122 CUS"; $resn_exch{"3H12 CUS"}="H123 CUS"; $resn_exch{"1HO1 CVB"}="HO11 CVB";
$resn_exch{"1HO2 CVB"}="HO21 CVB"; $resn_exch{"1HC2 CVB"}="HC21 CVB"; $resn_exch{"1HC3 CVB"}="HC31 CVB";
$resn_exch{"1HX1 CVB"}="HX11 CVB"; $resn_exch{"1HX2 CVB"}="HX21 CVB"; $resn_exch{"1H1  CVB"}=" H11 CVB";
$resn_exch{"1HC5 CVB"}="HC51 CVB"; $resn_exch{"1HC6 CVB"}="HC61 CVB"; $resn_exch{"1H3  CVI"}=" H31 CVI";
$resn_exch{"1H4  CVI"}=" H41 CVI"; $resn_exch{"1H6  CVI"}=" H61 CVI"; $resn_exch{"1H7  CVI"}=" H71 CVI";
$resn_exch{"1H9  CVI"}=" H91 CVI"; $resn_exch{"1H10 CVI"}="H101 CVI"; $resn_exch{"1H12 CVI"}="H121 CVI";
$resn_exch{"1H13 CVI"}="H131 CVI"; $resn_exch{"1H15 CVI"}="H151 CVI"; $resn_exch{"1H16 CVI"}="H161 CVI";
$resn_exch{"1H18 CVI"}="H181 CVI"; $resn_exch{"1H19 CVI"}="H191 CVI"; $resn_exch{"1H20 CVI"}="H201 CVI";
$resn_exch{"2H20 CVI"}="H202 CVI"; $resn_exch{"3H20 CVI"}="H203 CVI"; $resn_exch{"1H21 CVI"}="H211 CVI";
$resn_exch{"2H21 CVI"}="H212 CVI"; $resn_exch{"3H21 CVI"}="H213 CVI"; $resn_exch{"1H22 CVI"}="H221 CVI";
$resn_exch{"2H22 CVI"}="H222 CVI"; $resn_exch{"3H22 CVI"}="H223 CVI"; $resn_exch{"1H23 CVI"}="H231 CVI";
$resn_exch{"2H23 CVI"}="H232 CVI"; $resn_exch{"3H23 CVI"}="H233 CVI"; $resn_exch{"1H24 CVI"}="H241 CVI";
$resn_exch{"2H24 CVI"}="H242 CVI"; $resn_exch{"3H24 CVI"}="H243 CVI"; $resn_exch{"1H25 CVI"}="H251 CVI";
$resn_exch{"2H25 CVI"}="H252 CVI"; $resn_exch{"3H25 CVI"}="H253 CVI"; $resn_exch{"1H2  CX6"}=" H21 CX6";
$resn_exch{"2H2  CX6"}=" H22 CX6"; $resn_exch{"1H3  CX6"}=" H31 CX6"; $resn_exch{"2H3  CX6"}=" H32 CX6";
$resn_exch{"1H4  CX6"}=" H41 CX6"; $resn_exch{"2H4  CX6"}=" H42 CX6"; $resn_exch{"1H12 CX6"}="H121 CX6";
$resn_exch{"2H12 CX6"}="H122 CX6"; $resn_exch{"1H13 CX6"}="H131 CX6"; $resn_exch{"2H13 CX6"}="H132 CX6";
$resn_exch{"1H12 CXA"}="H121 CXA"; $resn_exch{"2H12 CXA"}="H122 CXA"; $resn_exch{"1H19 CXA"}="H191 CXA";
$resn_exch{"2H19 CXA"}="H192 CXA"; $resn_exch{"1H18 CXB"}="H181 CXB"; $resn_exch{"2H18 CXB"}="H182 CXB";
$resn_exch{"3H18 CXB"}="H183 CXB"; $resn_exch{"1H7  CXB"}=" H71 CXB"; $resn_exch{"2H7  CXB"}=" H72 CXB";
$resn_exch{"1H1  CXB"}=" H11 CXB"; $resn_exch{"2H1  CXB"}=" H12 CXB"; $resn_exch{"1H01 CXE"}="H011 CXE";
$resn_exch{"2H01 CXE"}="H012 CXE"; $resn_exch{"3H01 CXE"}="H013 CXE"; $resn_exch{"1H02 CXE"}="H021 CXE";
$resn_exch{"2H02 CXE"}="H022 CXE"; $resn_exch{"1H03 CXE"}="H031 CXE"; $resn_exch{"2H03 CXE"}="H032 CXE";
$resn_exch{"1H04 CXE"}="H041 CXE"; $resn_exch{"2H04 CXE"}="H042 CXE"; $resn_exch{"1H05 CXE"}="H051 CXE";
$resn_exch{"2H05 CXE"}="H052 CXE"; $resn_exch{"1H06 CXE"}="H061 CXE"; $resn_exch{"2H06 CXE"}="H062 CXE";
$resn_exch{"1H07 CXE"}="H071 CXE"; $resn_exch{"2H07 CXE"}="H072 CXE"; $resn_exch{"1H08 CXE"}="H081 CXE";
$resn_exch{"2H08 CXE"}="H082 CXE"; $resn_exch{"1H09 CXE"}="H091 CXE"; $resn_exch{"2H09 CXE"}="H092 CXE";
$resn_exch{"1H10 CXE"}="H101 CXE"; $resn_exch{"2H10 CXE"}="H102 CXE"; $resn_exch{"1H12 CXE"}="H121 CXE";
$resn_exch{"2H12 CXE"}="H122 CXE"; $resn_exch{"1H13 CXE"}="H131 CXE"; $resn_exch{"2H13 CXE"}="H132 CXE";
$resn_exch{"1H15 CXE"}="H151 CXE"; $resn_exch{"2H15 CXE"}="H152 CXE"; $resn_exch{"1H16 CXE"}="H161 CXE";
$resn_exch{"2H16 CXE"}="H162 CXE"; $resn_exch{"1H18 CXE"}="H181 CXE"; $resn_exch{"2H18 CXE"}="H182 CXE";
$resn_exch{"1H19 CXE"}="H191 CXE"; $resn_exch{"2H19 CXE"}="H192 CXE"; $resn_exch{"1H21 CXE"}="H211 CXE";
$resn_exch{"2H21 CXE"}="H212 CXE"; $resn_exch{"1H22 CXE"}="H221 CXE"; $resn_exch{"2H22 CXE"}="H222 CXE";
$resn_exch{"1H24 CXE"}="H241 CXE"; $resn_exch{"2H24 CXE"}="H242 CXE"; $resn_exch{"1H25 CXE"}="H251 CXE";
$resn_exch{"2H25 CXE"}="H252 CXE"; $resn_exch{"1H2  CXF"}=" H21 CXF"; $resn_exch{"2H2  CXF"}=" H22 CXF";
$resn_exch{"1H3  CXF"}=" H31 CXF"; $resn_exch{"2H3  CXF"}=" H32 CXF"; $resn_exch{"1H4  CXF"}=" H41 CXF";
$resn_exch{"2H4  CXF"}=" H42 CXF"; $resn_exch{"1H5  CXF"}=" H51 CXF"; $resn_exch{"2H5  CXF"}=" H52 CXF";
$resn_exch{"1H6  CXF"}=" H61 CXF"; $resn_exch{"2H6  CXF"}=" H62 CXF"; $resn_exch{"1H2  CXL"}=" H21 CXL";
$resn_exch{"2H2  CXL"}=" H22 CXL"; $resn_exch{"1H3  CXL"}=" H31 CXL"; $resn_exch{"2H3  CXL"}=" H32 CXL";
$resn_exch{"1H4  CXL"}=" H41 CXL"; $resn_exch{"2H4  CXL"}=" H42 CXL"; $resn_exch{"1H5  CXL"}=" H51 CXL";
$resn_exch{"2H5  CXL"}=" H52 CXL"; $resn_exch{"1H6  CXL"}=" H61 CXL"; $resn_exch{"2H6  CXL"}=" H62 CXL";
$resn_exch{" HN  CXM"}=" H   CXM"; $resn_exch{"1HB  CXM"}=" HB2 CXM"; $resn_exch{"2HB  CXM"}=" HB3 CXM";
$resn_exch{"1HG  CXM"}=" HG2 CXM"; $resn_exch{"2HG  CXM"}=" HG3 CXM"; $resn_exch{"1HE  CXM"}=" HE1 CXM";
$resn_exch{"2HE  CXM"}=" HE2 CXM"; $resn_exch{"3HE  CXM"}=" HE3 CXM"; $resn_exch{"1H51 CXN"}="H511 CXN";
$resn_exch{"2H51 CXN"}="H512 CXN"; $resn_exch{"3H51 CXN"}="H513 CXN"; $resn_exch{"1H52 CXN"}="H521 CXN";
$resn_exch{"2H52 CXN"}="H522 CXN"; $resn_exch{"3H52 CXN"}="H523 CXN"; $resn_exch{"1H27 CXN"}="H271 CXN";
$resn_exch{"2H27 CXN"}="H272 CXN"; $resn_exch{"3H27 CXN"}="H273 CXN"; $resn_exch{"1H3  CXO"}=" H31 CXO";
$resn_exch{"2H3  CXO"}=" H32 CXO"; $resn_exch{"1H4  CXO"}=" H41 CXO"; $resn_exch{"2H4  CXO"}=" H42 CXO";
$resn_exch{"1H5  CXO"}=" H51 CXO"; $resn_exch{"2H5  CXO"}=" H52 CXO"; $resn_exch{"1H6  CXO"}=" H61 CXO";
$resn_exch{"2H6  CXO"}=" H62 CXO"; $resn_exch{"1H2  CXP"}=" H21 CXP"; $resn_exch{"2H2  CXP"}=" H22 CXP";
$resn_exch{"1H3  CXP"}=" H31 CXP"; $resn_exch{"2H3  CXP"}=" H32 CXP"; $resn_exch{"1H4  CXP"}=" H41 CXP";
$resn_exch{"2H4  CXP"}=" H42 CXP"; $resn_exch{"1H5  CXP"}=" H51 CXP"; $resn_exch{"2H5  CXP"}=" H52 CXP";
$resn_exch{"1H6  CXP"}=" H61 CXP"; $resn_exch{"2H6  CXP"}=" H62 CXP"; $resn_exch{"1H7  CXP"}=" H71 CXP";
$resn_exch{"2H7  CXP"}=" H72 CXP"; $resn_exch{"1H8  CXP"}=" H81 CXP"; $resn_exch{"2H8  CXP"}=" H82 CXP";
$resn_exch{"1H1  CXR"}=" H11 CXR"; $resn_exch{"2H1  CXR"}=" H12 CXR"; $resn_exch{" H4* CXR"}=" H4' CXR";
$resn_exch{" H3* CXR"}=" H3' CXR"; $resn_exch{" H2* CXR"}=" H2' CXR"; $resn_exch{" H1* CXR"}=" H1' CXR";
$resn_exch{"1H5* CXR"}="H5'1 CXR"; $resn_exch{"2H5* CXR"}="H5'2 CXR"; $resn_exch{"1H1  CXS"}=" H11 CXS";
$resn_exch{"2H1  CXS"}=" H12 CXS"; $resn_exch{"1H2  CXS"}=" H21 CXS"; $resn_exch{"2H2  CXS"}=" H22 CXS";
$resn_exch{"1H3  CXS"}=" H31 CXS"; $resn_exch{"2H3  CXS"}=" H32 CXS"; $resn_exch{"1H5  CXS"}=" H51 CXS";
$resn_exch{"2H5  CXS"}=" H52 CXS"; $resn_exch{"1H6  CXS"}=" H61 CXS"; $resn_exch{"2H6  CXS"}=" H62 CXS";
$resn_exch{"1H7  CXS"}=" H71 CXS"; $resn_exch{"2H7  CXS"}=" H72 CXS"; $resn_exch{"1H8  CXS"}=" H81 CXS";
$resn_exch{"2H8  CXS"}=" H82 CXS"; $resn_exch{"1H9  CXS"}=" H91 CXS"; $resn_exch{"2H9  CXS"}=" H92 CXS";
$resn_exch{"1H6  CXT"}=" H61 CXT"; $resn_exch{"2H6  CXT"}=" H62 CXT"; $resn_exch{"1H8C CXT"}="H8C1 CXT";
$resn_exch{"2H8C CXT"}="H8C2 CXT"; $resn_exch{"1H10 CXT"}="H101 CXT"; $resn_exch{"2H10 CXT"}="H102 CXT";
$resn_exch{"3H10 CXT"}="H103 CXT"; $resn_exch{"1H11 CXT"}="H111 CXT"; $resn_exch{"2H11 CXT"}="H112 CXT";
$resn_exch{"3H11 CXT"}="H113 CXT"; $resn_exch{"1H12 CXT"}="H121 CXT"; $resn_exch{"1H21 CXT"}="H211 CXT";
$resn_exch{"2H21 CXT"}="H212 CXT"; $resn_exch{"1H23 CXT"}="H231 CXT"; $resn_exch{"2H23 CXT"}="H232 CXT";
$resn_exch{"1H26 CXT"}="H261 CXT"; $resn_exch{"2H26 CXT"}="H262 CXT"; $resn_exch{"1H27 CXT"}="H271 CXT";
$resn_exch{"2H27 CXT"}="H272 CXT"; $resn_exch{"1H31 CXT"}="H311 CXT"; $resn_exch{"2H31 CXT"}="H312 CXT";
$resn_exch{"1H32 CXT"}="H321 CXT"; $resn_exch{"2H32 CXT"}="H322 CXT"; $resn_exch{"1H34 CXT"}="H341 CXT";
$resn_exch{"2H34 CXT"}="H342 CXT"; $resn_exch{"1H39 CXT"}="H391 CXT"; $resn_exch{"2H39 CXT"}="H392 CXT";
$resn_exch{"3H39 CXT"}="H393 CXT"; $resn_exch{"1H40 CXT"}="H401 CXT"; $resn_exch{"2H40 CXT"}="H402 CXT";
$resn_exch{"1H5  CXX"}=" H51 CXX"; $resn_exch{"1H6  CXX"}=" H61 CXX"; $resn_exch{"1H17 CXX"}="H171 CXX";
$resn_exch{"2H17 CXX"}="H172 CXX"; $resn_exch{"1H18 CXX"}="H181 CXX"; $resn_exch{"2H18 CXX"}="H182 CXX";
$resn_exch{"3H18 CXX"}="H183 CXX"; $resn_exch{"1H15 CXX"}="H151 CXX"; $resn_exch{"2H15 CXX"}="H152 CXX";
$resn_exch{"1H16 CXX"}="H161 CXX"; $resn_exch{"2H16 CXX"}="H162 CXX"; $resn_exch{"1H19 CXX"}="H191 CXX";
$resn_exch{"2H19 CXX"}="H192 CXX"; $resn_exch{"3H19 CXX"}="H193 CXX"; $resn_exch{"2H5  CXX"}=" H52 CXX";
$resn_exch{"2H6  CXX"}=" H62 CXX"; $resn_exch{"1H5* CXY"}="H5'1 CXY"; $resn_exch{"2H5* CXY"}="H5'2 CXY";
$resn_exch{" H4* CXY"}=" H4D CXY"; $resn_exch{" H3* CXY"}=" H3D CXY"; $resn_exch{" H2* CXY"}=" H2D CXY";
$resn_exch{" H1* CXY"}=" H1D CXY"; $resn_exch{"1HN4 CXY"}="HN41 CXY"; $resn_exch{"2HN4 CXY"}="HN42 CXY";
$resn_exch{"1H5' CXY"}=" H5D CXY"; $resn_exch{"2H5' CXY"}=" H5B CXY"; $resn_exch{"1HAO CY0"}="HAO1 CY0";
$resn_exch{"2HAO CY0"}="HAO2 CY0"; $resn_exch{"1HAN CY0"}="HAN1 CY0"; $resn_exch{"2HAN CY0"}="HAN2 CY0";
$resn_exch{"1HBC CY0"}="HBC1 CY0"; $resn_exch{"2HBC CY0"}="HBC2 CY0"; $resn_exch{"1HN  CY0"}=" HN1 CY0";
$resn_exch{"2HN  CY0"}=" HN2 CY0"; $resn_exch{" HN1 CY1"}=" H2  CY1"; $resn_exch{" HB1 CY1"}=" HB2 CY1";
$resn_exch{" HB2 CY1"}=" HB3 CY1"; $resn_exch{" HD1 CY1"}=" HD2 CY1"; $resn_exch{" HD2 CY1"}=" HD3 CY1";
$resn_exch{" HE1 CY1"}=" HE  CY1"; $resn_exch{"1HM  CY1"}=" HM1 CY1"; $resn_exch{"2HM  CY1"}=" HM2 CY1";
$resn_exch{"3HM  CY1"}=" HM3 CY1"; $resn_exch{" HN2 CY3"}=" H2  CY3"; $resn_exch{"1HB  CY3"}=" HB2 CY3";
$resn_exch{"2HB  CY3"}=" HB3 CY3"; $resn_exch{"1HN1 CY3"}="HN11 CY3"; $resn_exch{"2HN1 CY3"}="HN12 CY3";
$resn_exch{" HN2 CY4"}=" H2  CY4"; $resn_exch{"1HB  CY4"}=" HB2 CY4"; $resn_exch{"2HB  CY4"}=" HB3 CY4";
$resn_exch{"1HA2 CY4"}="HA22 CY4"; $resn_exch{"2HA2 CY4"}="HA23 CY4"; $resn_exch{"1HA3 CY4"}="HA32 CY4";
$resn_exch{"2HA3 CY4"}="HA33 CY4"; $resn_exch{"1HA4 CY4"}="HA41 CY4"; $resn_exch{"2HA4 CY4"}="HA42 CY4";
$resn_exch{"3HA4 CY4"}="HA43 CY4"; $resn_exch{"1H5  CY6"}=" H51 CY6"; $resn_exch{"2H5  CY6"}=" H52 CY6";
$resn_exch{"3H5  CY6"}=" H53 CY6"; $resn_exch{"1H16 CY6"}="H161 CY6"; $resn_exch{"2H16 CY6"}="H162 CY6";
$resn_exch{"3H16 CY6"}="H163 CY6"; $resn_exch{"1H17 CY6"}="H171 CY6"; $resn_exch{"2H17 CY6"}="H172 CY6";
$resn_exch{"3H17 CY6"}="H173 CY6"; $resn_exch{"1H20 CY6"}="H201 CY6"; $resn_exch{"2H20 CY6"}="H202 CY6";
$resn_exch{"1H22 CY6"}="H221 CY6"; $resn_exch{"2H22 CY6"}="H222 CY6"; $resn_exch{"1H25 CY6"}="H251 CY6";
$resn_exch{"2H25 CY6"}="H252 CY6"; $resn_exch{"3H25 CY6"}="H253 CY6"; $resn_exch{"1H26 CY6"}="H261 CY6";
$resn_exch{"2H26 CY6"}="H262 CY6"; $resn_exch{"3H26 CY6"}="H263 CY6"; $resn_exch{"1H38 CY6"}="H381 CY6";
$resn_exch{"2H38 CY6"}="H382 CY6"; $resn_exch{"1H39 CY6"}="H391 CY6"; $resn_exch{"2H39 CY6"}="H392 CY6";
$resn_exch{"3H39 CY6"}="H393 CY6"; $resn_exch{"1H40 CY6"}="H401 CY6"; $resn_exch{"2H40 CY6"}="H402 CY6";
$resn_exch{"1H42 CY6"}="H421 CY6"; $resn_exch{"2H42 CY6"}="H422 CY6"; $resn_exch{"1H43 CY6"}="H431 CY6";
$resn_exch{"2H43 CY6"}="H432 CY6"; $resn_exch{"1HAV CY7"}="HAV1 CY7"; $resn_exch{"2HAV CY7"}="HAV2 CY7";
$resn_exch{"1HAU CY7"}="HAU1 CY7"; $resn_exch{"2HAU CY7"}="HAU2 CY7"; $resn_exch{"1HAA CY7"}="HAA1 CY7";
$resn_exch{"2HAA CY7"}="HAA2 CY7"; $resn_exch{"3HAA CY7"}="HAA3 CY7"; $resn_exch{"1HAX CY7"}="HAX1 CY7";
$resn_exch{"2HAX CY7"}="HAX2 CY7"; $resn_exch{"1HAY CY7"}="HAY1 CY7"; $resn_exch{"2HAY CY7"}="HAY2 CY7";
$resn_exch{"1HAB CY7"}="HAB1 CY7"; $resn_exch{"2HAB CY7"}="HAB2 CY7"; $resn_exch{"3HAB CY7"}="HAB3 CY7";
$resn_exch{"1HAC CY7"}="HAC1 CY7"; $resn_exch{"2HAC CY7"}="HAC2 CY7"; $resn_exch{"3HAC CY7"}="HAC3 CY7";
$resn_exch{"1HBC CY7"}="HBC1 CY7"; $resn_exch{"2HBC CY7"}="HBC2 CY7"; $resn_exch{"1HN  CY7"}=" HN1 CY7";
$resn_exch{"2HN  CY7"}=" HN2 CY7"; $resn_exch{"1HMA CYB"}="HMA1 CYB"; $resn_exch{"2HMA CYB"}="HMA2 CYB";
$resn_exch{"3HMA CYB"}="HMA3 CYB"; $resn_exch{"1HAA CYB"}="HAA1 CYB"; $resn_exch{"2HAA CYB"}="HAA2 CYB";
$resn_exch{"1HBA CYB"}="HBA1 CYB"; $resn_exch{"2HBA CYB"}="HBA2 CYB"; $resn_exch{"1HHB CYB"}="HHB1 CYB";
$resn_exch{"2HHB CYB"}="HHB2 CYB"; $resn_exch{" H_B CYB"}=" HB  CYB"; $resn_exch{"1HMB CYB"}="HMB1 CYB";
$resn_exch{"2HMB CYB"}="HMB2 CYB"; $resn_exch{"3HMB CYB"}="HMB3 CYB"; $resn_exch{"1HAB CYB"}="HAB1 CYB";
$resn_exch{"2HAB CYB"}="HAB2 CYB"; $resn_exch{"1HBB CYB"}="HBB1 CYB"; $resn_exch{"2HBB CYB"}="HBB2 CYB";
$resn_exch{"3HBB CYB"}="HBB3 CYB"; $resn_exch{" H_C CYB"}=" HC  CYB"; $resn_exch{"1HAC CYB"}="HAC1 CYB";
$resn_exch{"2HAC CYB"}="HAC2 CYB"; $resn_exch{"1HBC CYB"}="HBC1 CYB"; $resn_exch{"2HBC CYB"}="HBC2 CYB";
$resn_exch{"3HBC CYB"}="HBC3 CYB"; $resn_exch{"1HHD CYB"}="HHD1 CYB"; $resn_exch{"2HHD CYB"}="HHD2 CYB";
$resn_exch{" H_D CYB"}=" HD  CYB"; $resn_exch{"1HMD CYB"}="HMD1 CYB"; $resn_exch{"2HMD CYB"}="HMD2 CYB";
$resn_exch{"3HMD CYB"}="HMD3 CYB"; $resn_exch{"1HAD CYB"}="HAD1 CYB"; $resn_exch{"2HAD CYB"}="HAD2 CYB";
$resn_exch{"1HBD CYB"}="HBD1 CYB"; $resn_exch{"2HBD CYB"}="HBD2 CYB"; $resn_exch{"1HMA CYC"}="HMA1 CYC";
$resn_exch{"2HMA CYC"}="HMA2 CYC"; $resn_exch{"3HMA CYC"}="HMA3 CYC"; $resn_exch{"1HAA CYC"}="HAA1 CYC";
$resn_exch{"2HAA CYC"}="HAA2 CYC"; $resn_exch{"1HBA CYC"}="HBA1 CYC"; $resn_exch{"2HBA CYC"}="HBA2 CYC";
$resn_exch{" H_B CYC"}=" HB  CYC"; $resn_exch{"1HMB CYC"}="HMB1 CYC"; $resn_exch{"2HMB CYC"}="HMB2 CYC";
$resn_exch{"3HMB CYC"}="HMB3 CYC"; $resn_exch{"1HAB CYC"}="HAB1 CYC"; $resn_exch{"2HAB CYC"}="HAB2 CYC";
$resn_exch{"1HBB CYC"}="HBB1 CYC"; $resn_exch{"2HBB CYC"}="HBB2 CYC"; $resn_exch{"3HBB CYC"}="HBB3 CYC";
$resn_exch{" H_C CYC"}=" HC  CYC"; $resn_exch{"1HMC CYC"}="HMC1 CYC"; $resn_exch{"2HMC CYC"}="HMC2 CYC";
$resn_exch{"3HMC CYC"}="HMC3 CYC"; $resn_exch{"1HAC CYC"}="HAC1 CYC"; $resn_exch{"2HAC CYC"}="HAC2 CYC";
$resn_exch{"1HBC CYC"}="HBC1 CYC"; $resn_exch{"2HBC CYC"}="HBC2 CYC"; $resn_exch{"3HBC CYC"}="HBC3 CYC";
$resn_exch{" H_D CYC"}=" HD  CYC"; $resn_exch{"1HMD CYC"}="HMD1 CYC"; $resn_exch{"2HMD CYC"}="HMD2 CYC";
$resn_exch{"3HMD CYC"}="HMD3 CYC"; $resn_exch{"1HAD CYC"}="HAD1 CYC"; $resn_exch{"2HAD CYC"}="HAD2 CYC";
$resn_exch{"1HBD CYC"}="HBD1 CYC"; $resn_exch{"2HBD CYC"}="HBD2 CYC"; $resn_exch{" HN2 CYD"}=" H2  CYD";
$resn_exch{"1HB  CYD"}=" HB2 CYD"; $resn_exch{"2HB  CYD"}=" HB3 CYD"; $resn_exch{"1H1  CYD"}=" H11 CYD";
$resn_exch{"2H1  CYD"}=" H12 CYD"; $resn_exch{"1HN1 CYD"}="HN11 CYD"; $resn_exch{"2HN1 CYD"}="HN12 CYD";
$resn_exch{"1H4  CYD"}=" H41 CYD"; $resn_exch{"2H4  CYD"}=" H42 CYD"; $resn_exch{"1H3  CYD"}=" H31 CYD";
$resn_exch{"2H3  CYD"}=" H32 CYD"; $resn_exch{"2H4  CYE"}=" H42 CYE"; $resn_exch{"1H4  CYE"}=" H41 CYE";
$resn_exch{"2H2  CYE"}=" H22 CYE"; $resn_exch{" H2  CYF"}=" HC2 CYF"; $resn_exch{"1HAL CYF"}="HAL3 CYF";
$resn_exch{"2HAL CYF"}="HAL2 CYF"; $resn_exch{"1HB  CYF"}=" HB3 CYF"; $resn_exch{"2HB  CYF"}=" HB2 CYF";
$resn_exch{"1HN  CYF"}=" H   CYF"; $resn_exch{"2HN  CYF"}=" H2  CYF"; $resn_exch{"1HNT CYF"}="HNT1 CYF";
$resn_exch{"2HNT CYF"}="HNT2 CYF"; $resn_exch{"1HN  CYG"}=" H   CYG"; $resn_exch{"2HN  CYG"}=" H2  CYG";
$resn_exch{"1HB  CYG"}=" HB2 CYG"; $resn_exch{"2HB  CYG"}=" HB3 CYG"; $resn_exch{"1HN1 CYG"}="HN11 CYG";
$resn_exch{"2HN1 CYG"}="HN12 CYG"; $resn_exch{"1HB1 CYG"}="HB13 CYG"; $resn_exch{"2HB1 CYG"}="HB12 CYG";
$resn_exch{"1HG1 CYG"}="HG13 CYG"; $resn_exch{"2HG1 CYG"}="HG12 CYG"; $resn_exch{"1H2  CYH"}=" H21 CYH";
$resn_exch{"2H2  CYH"}=" H22 CYH"; $resn_exch{"1H3  CYH"}=" H31 CYH"; $resn_exch{"2H3  CYH"}=" H32 CYH";
$resn_exch{"1H4  CYH"}=" H41 CYH"; $resn_exch{"2H4  CYH"}=" H42 CYH"; $resn_exch{"1H5  CYH"}=" H51 CYH";
$resn_exch{"2H5  CYH"}=" H52 CYH"; $resn_exch{"1H6  CYH"}=" H61 CYH"; $resn_exch{"2H6  CYH"}=" H62 CYH";
$resn_exch{"1H1  CYI"}=" H11 CYI"; $resn_exch{"2H1  CYI"}=" H12 CYI"; $resn_exch{"1H2  CYI"}=" H21 CYI";
$resn_exch{"2H2  CYI"}=" H22 CYI"; $resn_exch{"1H3  CYI"}=" H31 CYI"; $resn_exch{"2H3  CYI"}=" H32 CYI";
$resn_exch{"1H5  CYI"}=" H51 CYI"; $resn_exch{"2H5  CYI"}=" H52 CYI"; $resn_exch{"1H6  CYI"}=" H61 CYI";
$resn_exch{"2H6  CYI"}=" H62 CYI"; $resn_exch{"1H10 CYJ"}="H101 CYJ"; $resn_exch{"2H10 CYJ"}="H102 CYJ";
$resn_exch{"1H9C CYJ"}="H9C1 CYJ"; $resn_exch{"2H9C CYJ"}="H9C2 CYJ"; $resn_exch{"1H7C CYJ"}="H7C1 CYJ";
$resn_exch{"2H7C CYJ"}="H7C2 CYJ"; $resn_exch{"1H5C CYJ"}="H5C1 CYJ"; $resn_exch{"2H5C CYJ"}="H5C2 CYJ";
$resn_exch{"1H2C CYJ"}="H2C1 CYJ"; $resn_exch{"2H2C CYJ"}="H2C2 CYJ"; $resn_exch{"1HEC CYJ"}="HEC1 CYJ";
$resn_exch{"2HEC CYJ"}="HEC2 CYJ"; $resn_exch{"1HDC CYJ"}="HDC1 CYJ"; $resn_exch{"2HDC CYJ"}="HDC2 CYJ";
$resn_exch{"1HGC CYJ"}="HGC1 CYJ"; $resn_exch{"2HGC CYJ"}="HGC2 CYJ"; $resn_exch{"1HBC CYJ"}="HBC1 CYJ";
$resn_exch{"2HBC CYJ"}="HBC2 CYJ"; $resn_exch{"1HN  CYJ"}=" HN1 CYJ"; $resn_exch{"2HN  CYJ"}=" HN2 CYJ";
$resn_exch{"1H6  CYL"}=" H61 CYL"; $resn_exch{"2H6  CYL"}=" H62 CYL"; $resn_exch{"1HN1 CYL"}="HN11 CYL";
$resn_exch{"2HN1 CYL"}="HN12 CYL"; $resn_exch{"1HN  CYM"}=" HN1 CYM"; $resn_exch{"2HN  CYM"}=" HN2 CYM";
$resn_exch{"1HB  CYM"}=" HB1 CYM"; $resn_exch{"2HB  CYM"}=" HB2 CYM"; $resn_exch{"1HD  CYM"}=" HD1 CYM";
$resn_exch{"2HD  CYM"}=" HD2 CYM"; $resn_exch{"3HD  CYM"}=" HD3 CYM"; $resn_exch{"1HN1 CYP"}="HN11 CYP";
$resn_exch{"2HN1 CYP"}="HN12 CYP"; $resn_exch{"1HB1 CYP"}="HB11 CYP"; $resn_exch{"2HB1 CYP"}="HB12 CYP";
$resn_exch{"1HG1 CYP"}="HG11 CYP"; $resn_exch{"2HG1 CYP"}="HG12 CYP"; $resn_exch{"1HB2 CYP"}="HB21 CYP";
$resn_exch{"2HB2 CYP"}="HB22 CYP"; $resn_exch{"1HA3 CYP"}="HA31 CYP"; $resn_exch{"2HA3 CYP"}="HA32 CYP";
$resn_exch{"1HN  CYQ"}=" H   CYQ"; $resn_exch{"2HN  CYQ"}=" H2  CYQ"; $resn_exch{"1HB  CYQ"}=" HB2 CYQ";
$resn_exch{"2HB  CYQ"}=" HB3 CYQ"; $resn_exch{"1HD  CYQ"}=" HD2 CYQ"; $resn_exch{"2HD  CYQ"}=" HD3 CYQ";
$resn_exch{"2HOP CYQ"}="HOP2 CYQ"; $resn_exch{"3HOP CYQ"}="HOP3 CYQ"; $resn_exch{" H2  CYR"}=" HC2 CYR";
$resn_exch{"1HN2 CYR"}="HN21 CYR"; $resn_exch{"2HN2 CYR"}="HN22 CYR"; $resn_exch{"1H3  CYR"}=" H31 CYR";
$resn_exch{"2H3  CYR"}=" H32 CYR"; $resn_exch{"1H4  CYR"}=" H41 CYR"; $resn_exch{"2H4  CYR"}=" H42 CYR";
$resn_exch{"1H5  CYR"}=" H51 CYR"; $resn_exch{"2H5  CYR"}=" H52 CYR"; $resn_exch{"1HN7 CYR"}="HN71 CYR";
$resn_exch{"2HN7 CYR"}="HN72 CYR"; $resn_exch{"1HB  CYR"}=" HB3 CYR"; $resn_exch{"2HB  CYR"}=" HB2 CYR";
$resn_exch{"1HN  CYR"}=" H   CYR"; $resn_exch{"2HN  CYR"}=" H2  CYR"; $resn_exch{" HN2 CYS"}=" H2  CYS";
$resn_exch{"1HB  CYS"}=" HB2 CYS"; $resn_exch{"2HB  CYS"}=" HB3 CYS"; $resn_exch{"1HN4 CYT"}="HN41 CYT";
$resn_exch{"2HN4 CYT"}="HN42 CYT"; $resn_exch{"1H14 CYU"}="H141 CYU"; $resn_exch{"2H14 CYU"}="H142 CYU";
$resn_exch{"1H20 CYU"}="H201 CYU"; $resn_exch{"2H20 CYU"}="H202 CYU"; $resn_exch{"1H24 CYU"}="H241 CYU";
$resn_exch{"2H24 CYU"}="H242 CYU"; $resn_exch{"1H31 CYU"}="H311 CYU"; $resn_exch{"2H31 CYU"}="H312 CYU";
$resn_exch{"1H32 CYU"}="H321 CYU"; $resn_exch{"2H32 CYU"}="H322 CYU"; $resn_exch{"1H41 CYU"}="H411 CYU";
$resn_exch{"2H41 CYU"}="H412 CYU"; $resn_exch{"3H41 CYU"}="H413 CYU"; $resn_exch{"1H42 CYU"}="H421 CYU";
$resn_exch{"2H42 CYU"}="H422 CYU"; $resn_exch{"3H42 CYU"}="H423 CYU"; $resn_exch{"1H43 CYU"}="H431 CYU";
$resn_exch{"2H43 CYU"}="H432 CYU"; $resn_exch{"3H43 CYU"}="H433 CYU"; $resn_exch{"1H44 CYU"}="H441 CYU";
$resn_exch{"2H44 CYU"}="H442 CYU"; $resn_exch{"3H44 CYU"}="H443 CYU"; $resn_exch{"1H45 CYU"}="H451 CYU";
$resn_exch{"2H45 CYU"}="H452 CYU"; $resn_exch{"3H45 CYU"}="H453 CYU"; $resn_exch{"1H46 CYU"}="H461 CYU";
$resn_exch{"2H46 CYU"}="H462 CYU"; $resn_exch{"3H46 CYU"}="H463 CYU"; $resn_exch{"1H47 CYU"}="H471 CYU";
$resn_exch{"2H47 CYU"}="H472 CYU"; $resn_exch{"3H47 CYU"}="H473 CYU"; $resn_exch{"1H48 CYU"}="H481 CYU";
$resn_exch{"2H48 CYU"}="H482 CYU"; $resn_exch{"3H48 CYU"}="H483 CYU"; $resn_exch{"1H49 CYU"}="H491 CYU";
$resn_exch{"2H49 CYU"}="H492 CYU"; $resn_exch{"3H49 CYU"}="H493 CYU"; $resn_exch{"1H50 CYU"}="H501 CYU";
$resn_exch{"2H50 CYU"}="H502 CYU"; $resn_exch{"3H50 CYU"}="H503 CYU"; $resn_exch{"1H37 CYU"}="H371 CYU";
$resn_exch{"2H37 CYU"}="H372 CYU"; $resn_exch{"1H38 CYU"}="H381 CYU"; $resn_exch{"2H38 CYU"}="H382 CYU";
$resn_exch{"3H38 CYU"}="H383 CYU"; $resn_exch{"1H39 CYU"}="H391 CYU"; $resn_exch{"2H39 CYU"}="H392 CYU";
$resn_exch{"3H39 CYU"}="H393 CYU"; $resn_exch{"1H40 CYU"}="H401 CYU"; $resn_exch{"2H40 CYU"}="H402 CYU";
$resn_exch{"3H40 CYU"}="H403 CYU"; $resn_exch{"1H49 CYV"}="H491 CYV"; $resn_exch{"2H49 CYV"}="H492 CYV";
$resn_exch{"3H49 CYV"}="H493 CYV"; $resn_exch{"1H48 CYV"}="H481 CYV"; $resn_exch{"2H48 CYV"}="H482 CYV";
$resn_exch{"3H48 CYV"}="H483 CYV"; $resn_exch{"1H47 CYV"}="H471 CYV"; $resn_exch{"2H47 CYV"}="H472 CYV";
$resn_exch{"3H47 CYV"}="H473 CYV"; $resn_exch{"1H40 CYV"}="H401 CYV"; $resn_exch{"2H40 CYV"}="H402 CYV";
$resn_exch{"1H35 CYV"}="H351 CYV"; $resn_exch{"2H35 CYV"}="H352 CYV"; $resn_exch{"3H35 CYV"}="H353 CYV";
$resn_exch{"1H34 CYV"}="H341 CYV"; $resn_exch{"2H34 CYV"}="H342 CYV"; $resn_exch{"3H34 CYV"}="H343 CYV";
$resn_exch{"1H29 CYV"}="H291 CYV"; $resn_exch{"2H29 CYV"}="H292 CYV"; $resn_exch{"1H23 CYV"}="H231 CYV";
$resn_exch{"2H23 CYV"}="H232 CYV"; $resn_exch{"1H26 CYV"}="H261 CYV"; $resn_exch{"2H26 CYV"}="H262 CYV";
$resn_exch{"3H26 CYV"}="H263 CYV"; $resn_exch{"1H27 CYV"}="H271 CYV"; $resn_exch{"2H27 CYV"}="H272 CYV";
$resn_exch{"3H27 CYV"}="H273 CYV"; $resn_exch{"1H5  CYV"}=" H51 CYV"; $resn_exch{"2H5  CYV"}=" H52 CYV";
$resn_exch{"1H6  CYV"}=" H61 CYV"; $resn_exch{"2H6  CYV"}=" H62 CYV"; $resn_exch{"1H9  CYV"}=" H91 CYV";
$resn_exch{"2H9  CYV"}=" H92 CYV"; $resn_exch{"1H10 CYV"}="H101 CYV"; $resn_exch{"2H10 CYV"}="H102 CYV";
$resn_exch{"3H10 CYV"}="H103 CYV"; $resn_exch{"1H19 CYV"}="H191 CYV"; $resn_exch{"2H19 CYV"}="H192 CYV";
$resn_exch{"1H14 CYV"}="H141 CYV"; $resn_exch{"2H14 CYV"}="H142 CYV"; $resn_exch{"1H15 CYV"}="H151 CYV";
$resn_exch{"2H15 CYV"}="H152 CYV"; $resn_exch{"1H1  CYX"}=" H11 CYX"; $resn_exch{"1H4  CYX"}=" H41 CYX";
$resn_exch{"2H4  CYX"}=" H42 CYX"; $resn_exch{"1H5  CYX"}=" H51 CYX"; $resn_exch{"2H5  CYX"}=" H52 CYX";
$resn_exch{"1H6  CYX"}=" H61 CYX"; $resn_exch{"2H6  CYX"}=" H62 CYX"; $resn_exch{"1H8  CYX"}=" H81 CYX";
$resn_exch{"1H9  CYX"}=" H91 CYX"; $resn_exch{"1H2  CYY"}=" H21 CYY"; $resn_exch{"2H2  CYY"}=" H22 CYY";
$resn_exch{"1HN1 CYY"}="HN11 CYY"; $resn_exch{"2HN1 CYY"}="HN12 CYY"; $resn_exch{"1HN3 CYY"}="HN31 CYY";
$resn_exch{"2HN3 CYY"}="HN32 CYY"; $resn_exch{"1H1  CYZ"}="H11A CYZ"; $resn_exch{"1H2  CYZ"}=" H21 CYZ";
$resn_exch{"1H3  CYZ"}="H31A CYZ"; $resn_exch{"1H4  CYZ"}=" H41 CYZ"; $resn_exch{"1H5  CYZ"}=" H51 CYZ";
$resn_exch{"1H6  CYZ"}=" H61 CYZ"; $resn_exch{"2H6  CYZ"}=" H62 CYZ"; $resn_exch{"1H7  CYZ"}=" H71 CYZ";
$resn_exch{"2H7  CYZ"}=" H72 CYZ"; $resn_exch{"1H8  CYZ"}=" H81 CYZ"; $resn_exch{" HN2 CZ2"}=" H2  CZ2";
$resn_exch{" HN1 CZ2"}=" H   CZ2"; $resn_exch{"1HCB CZ2"}=" HB2 CZ2"; $resn_exch{"2HCB CZ2"}=" HB3 CZ2";
$resn_exch{"1H1  CZA"}=" H11 CZA"; $resn_exch{"2H1  CZA"}=" H12 CZA"; $resn_exch{"3H1  CZA"}=" H13 CZA";
$resn_exch{"1H16 CZA"}="H161 CZA"; $resn_exch{"1H12 CZA"}="H121 CZA"; $resn_exch{"2H12 CZA"}="H122 CZA";
$resn_exch{"1H10 CZA"}="H101 CZA"; $resn_exch{"2H10 CZA"}="H102 CZA"; $resn_exch{"3H10 CZA"}="H103 CZA";
$resn_exch{"1H11 CZA"}="H111 CZA"; $resn_exch{"2H11 CZA"}="H112 CZA"; $resn_exch{"3H11 CZA"}="H113 CZA";
$resn_exch{"1H26 CZB"}="H261 CZB"; $resn_exch{"2H26 CZB"}="H262 CZB"; $resn_exch{"1H10 CZB"}="H101 CZB";
$resn_exch{"2H10 CZB"}="H102 CZB"; $resn_exch{"1H10 CZH"}="H101 CZH"; $resn_exch{"2H10 CZH"}="H102 CZH";
$resn_exch{"1H26 CZH"}="H261 CZH"; $resn_exch{"2H26 CZH"}="H262 CZH"; $resn_exch{"1H10 CZI"}="H101 CZI";
$resn_exch{"2H10 CZI"}="H102 CZI"; $resn_exch{"1H26 CZI"}="H261 CZI"; $resn_exch{"2H26 CZI"}="H262 CZI";
$resn_exch{"1HB7 CZM"}="HB71 CZM"; $resn_exch{"2HB7 CZM"}="HB72 CZM"; $resn_exch{"3HB7 CZM"}="HB73 CZM";
$resn_exch{"1HA7 CZM"}="HA71 CZM"; $resn_exch{"2HA7 CZM"}="HA72 CZM"; $resn_exch{"3HA7 CZM"}="HA73 CZM";
$resn_exch{"1H29 CZN"}="H291 CZN"; $resn_exch{"2H29 CZN"}="H292 CZN"; $resn_exch{"1H10 CZN"}="H101 CZN";
$resn_exch{"2H10 CZN"}="H102 CZN"; $resn_exch{"1H26 CZP"}="H261 CZP"; $resn_exch{"2H26 CZP"}="H262 CZP";
$resn_exch{"1H28 CZP"}="H281 CZP"; $resn_exch{"2H28 CZP"}="H282 CZP"; $resn_exch{"1H29 CZP"}="H291 CZP";
$resn_exch{"2H29 CZP"}="H292 CZP"; $resn_exch{"1H30 CZP"}="H301 CZP"; $resn_exch{"2H30 CZP"}="H302 CZP";
$resn_exch{"1H31 CZP"}="H311 CZP"; $resn_exch{"2H31 CZP"}="H312 CZP"; $resn_exch{"1H10 CZP"}="H101 CZP";
$resn_exch{"2H10 CZP"}="H102 CZP"; $resn_exch{" HN1 CZZ"}=" H   CZZ"; $resn_exch{" HN2 CZZ"}=" H2  CZZ";
$resn_exch{"1HCB CZZ"}="HCB1 CZZ"; $resn_exch{"2HCB CZZ"}="HCB2 CZZ"; $resn_exch{"1H9  D01"}=" H91 D01";
$resn_exch{"2H9  D01"}=" H92 D01"; $resn_exch{"1H10 D01"}="H101 D01"; $resn_exch{"2H10 D01"}="H102 D01";
$resn_exch{"1H19 D01"}="H191 D01"; $resn_exch{"2H19 D01"}="H192 D01"; $resn_exch{"1H20 D01"}="H201 D01";
$resn_exch{"2H20 D01"}="H202 D01"; $resn_exch{"1H1  D05"}=" H11 D05"; $resn_exch{"2H1  D05"}=" H12 D05";
$resn_exch{"1H2  D05"}=" H21 D05"; $resn_exch{"2H2  D05"}="H22A D05"; $resn_exch{"3H2  D05"}="H23A D05";
$resn_exch{"1H20 D05"}="H201 D05"; $resn_exch{"2H20 D05"}="H202 D05"; $resn_exch{"1H1  D10"}=" H11 D10";
$resn_exch{"2H1  D10"}=" H12 D10"; $resn_exch{"3H1  D10"}=" H13 D10"; $resn_exch{"1H2  D10"}=" H21 D10";
$resn_exch{"2H2  D10"}=" H22 D10"; $resn_exch{"1H3  D10"}=" H31 D10"; $resn_exch{"2H3  D10"}=" H32 D10";
$resn_exch{"1H4  D10"}=" H41 D10"; $resn_exch{"2H4  D10"}=" H42 D10"; $resn_exch{"1H5  D10"}=" H51 D10";
$resn_exch{"2H5  D10"}=" H52 D10"; $resn_exch{"1H6  D10"}=" H61 D10"; $resn_exch{"2H6  D10"}=" H62 D10";
$resn_exch{"1H7  D10"}=" H71 D10"; $resn_exch{"2H7  D10"}=" H72 D10"; $resn_exch{"1H8  D10"}=" H81 D10";
$resn_exch{"2H8  D10"}=" H82 D10"; $resn_exch{"1H9  D10"}=" H91 D10"; $resn_exch{"2H9  D10"}=" H92 D10";
$resn_exch{"1H10 D10"}="H101 D10"; $resn_exch{"2H10 D10"}="H102 D10"; $resn_exch{"3H10 D10"}="H103 D10";
$resn_exch{"1HN  D11"}=" H   D11"; $resn_exch{"2HN  D11"}=" H2  D11"; $resn_exch{"1HG2 D11"}="HG21 D11";
$resn_exch{"2HG2 D11"}="HG22 D11"; $resn_exch{"3HG2 D11"}="HG23 D11"; $resn_exch{"HOXT D11"}=" HXT D11";
$resn_exch{"1H1  D12"}=" H11 D12"; $resn_exch{"2H1  D12"}=" H12 D12"; $resn_exch{"3H1  D12"}=" H13 D12";
$resn_exch{"1H2  D12"}=" H21 D12"; $resn_exch{"2H2  D12"}=" H22 D12"; $resn_exch{"1H3  D12"}=" H31 D12";
$resn_exch{"2H3  D12"}=" H32 D12"; $resn_exch{"1H4  D12"}=" H41 D12"; $resn_exch{"2H4  D12"}=" H42 D12";
$resn_exch{"1H5  D12"}=" H51 D12"; $resn_exch{"2H5  D12"}=" H52 D12"; $resn_exch{"1H6  D12"}=" H61 D12";
$resn_exch{"2H6  D12"}=" H62 D12"; $resn_exch{"1H7  D12"}=" H71 D12"; $resn_exch{"2H7  D12"}=" H72 D12";
$resn_exch{"1H8  D12"}=" H81 D12"; $resn_exch{"2H8  D12"}=" H82 D12"; $resn_exch{"1H9  D12"}=" H91 D12";
$resn_exch{"2H9  D12"}=" H92 D12"; $resn_exch{"1H10 D12"}="H101 D12"; $resn_exch{"2H10 D12"}="H102 D12";
$resn_exch{"1H11 D12"}="H111 D12"; $resn_exch{"2H11 D12"}="H112 D12"; $resn_exch{"1H12 D12"}="H121 D12";
$resn_exch{"2H12 D12"}="H122 D12"; $resn_exch{"3H12 D12"}="H123 D12"; $resn_exch{"1H25 D13"}="H251 D13";
$resn_exch{"2H25 D13"}="H252 D13"; $resn_exch{"3H25 D13"}="H253 D13"; $resn_exch{"1HM2 D16"}="HM21 D16";
$resn_exch{"2HM2 D16"}="HM22 D16"; $resn_exch{"3HM2 D16"}="HM23 D16"; $resn_exch{"1H9  D16"}=" H91 D16";
$resn_exch{"2H9  D16"}=" H92 D16"; $resn_exch{"1HB  D16"}=" HB1 D16"; $resn_exch{"2HB  D16"}=" HB2 D16";
$resn_exch{"1HG  D16"}=" HG1 D16"; $resn_exch{"2HG  D16"}=" HG2 D16"; $resn_exch{"2HOE D16"}="HOE2 D16";
$resn_exch{"1HP1 D16"}="HP11 D16"; $resn_exch{"2HP1 D16"}="HP12 D16"; $resn_exch{"3HP1 D16"}="HP13 D16";
$resn_exch{"1H10 D18"}="H101 D18"; $resn_exch{"2H10 D18"}="H102 D18"; $resn_exch{"3H10 D18"}="H103 D18";
$resn_exch{"1H9  D18"}=" H91 D18"; $resn_exch{"2H9  D18"}=" H92 D18"; $resn_exch{"3H9  D18"}=" H93 D18";
$resn_exch{"1HN1 D18"}="HN11 D18"; $resn_exch{"2HN1 D18"}="HN12 D18"; $resn_exch{"1HN' D18"}="HN'1 D18";
$resn_exch{"2HN' D18"}="HN'2 D18"; $resn_exch{"'HN2 D18"}="HN2' D18"; $resn_exch{"1H9' D18"}="H9'1 D18";
$resn_exch{"2H9' D18"}="H9'2 D18"; $resn_exch{"3H9' D18"}="H9'3 D18"; $resn_exch{"1H1' D18"}="H1'1 D18";
$resn_exch{"2H1' D18"}="H1'2 D18"; $resn_exch{"3H1' D18"}="H1'3 D18"; $resn_exch{"1H10 D19"}="H101 D19";
$resn_exch{"2H10 D19"}="H102 D19"; $resn_exch{"1H9  D19"}=" H91 D19"; $resn_exch{"2H9  D19"}=" H92 D19";
$resn_exch{"1HN1 D19"}="HN11 D19"; $resn_exch{"2HN1 D19"}="HN12 D19"; $resn_exch{"1HN' D19"}="HN'1 D19";
$resn_exch{"2HN' D19"}="HN'2 D19"; $resn_exch{"'HN2 D19"}="HN2' D19"; $resn_exch{"1H9' D19"}="H9'1 D19";
$resn_exch{"2H9' D19"}="H9'2 D19"; $resn_exch{"1H1' D19"}="H1'1 D19"; $resn_exch{"2H1' D19"}="H1'2 D19";
$resn_exch{"1H22 D1B"}="H221 D1B"; $resn_exch{"2H22 D1B"}="H222 D1B"; $resn_exch{"1H26 D1B"}="H261 D1B";
$resn_exch{"2H26 D1B"}="H262 D1B"; $resn_exch{"1H1C D1D"}="H1C1 D1D"; $resn_exch{"2H1C D1D"}="H1C2 D1D";
$resn_exch{"1H4C D1D"}="H4C1 D1D"; $resn_exch{"2H4C D1D"}="H4C2 D1D"; $resn_exch{"1H10 D1G"}="H101 D1G";
$resn_exch{"2H10 D1G"}="H102 D1G"; $resn_exch{"1H11 D1G"}="H111 D1G"; $resn_exch{"2H11 D1G"}="H112 D1G";
$resn_exch{"3H11 D1G"}="H113 D1G"; $resn_exch{"1H12 D1G"}="H121 D1G"; $resn_exch{"2H12 D1G"}="H122 D1G";
$resn_exch{"3H12 D1G"}="H123 D1G"; $resn_exch{"1HAA D1H"}="HAA1 D1H"; $resn_exch{"2HAA D1H"}="HAA2 D1H";
$resn_exch{"3HAA D1H"}="HAA3 D1H"; $resn_exch{"1HAC D1H"}="HAC1 D1H"; $resn_exch{"2HAC D1H"}="HAC2 D1H";
$resn_exch{"3HAC D1H"}="HAC3 D1H"; $resn_exch{"1HAK D1H"}="HAK1 D1H"; $resn_exch{"2HAK D1H"}="HAK2 D1H";
$resn_exch{"1HAL D1H"}="HAL1 D1H"; $resn_exch{"2HAL D1H"}="HAL2 D1H"; $resn_exch{"1HAB D1H"}="HAB1 D1H";
$resn_exch{"2HAB D1H"}="HAB2 D1H"; $resn_exch{"3HAB D1H"}="HAB3 D1H"; $resn_exch{"1HAD D1H"}="HAD1 D1H";
$resn_exch{"2HAD D1H"}="HAD2 D1H"; $resn_exch{"3HAD D1H"}="HAD3 D1H"; $resn_exch{"1H2A D1L"}="H2A1 D1L";
$resn_exch{"2H2A D1L"}="H2A2 D1L"; $resn_exch{"3H2A D1L"}="H2A3 D1L"; $resn_exch{"3HOP D1P"}="HOP3 D1P";
$resn_exch{"2HOP D1P"}="HOP2 D1P"; $resn_exch{"1H5* D1P"}=" H5' D1P"; $resn_exch{"2H5* D1P"}="H5'' D1P";
$resn_exch{" H4* D1P"}=" H4' D1P"; $resn_exch{" H3* D1P"}=" H3' D1P"; $resn_exch{"3HO* D1P"}="HO'3 D1P";
$resn_exch{"1H2* D1P"}=" H2' D1P"; $resn_exch{"2H2* D1P"}="H2'' D1P"; $resn_exch{" H1* D1P"}=" H1' D1P";
$resn_exch{"HO1* D1P"}="HO1' D1P"; $resn_exch{"1H39 D1R"}="H391 D1R"; $resn_exch{"2H39 D1R"}="H392 D1R";
$resn_exch{"1H40 D1R"}="H401 D1R"; $resn_exch{"2H40 D1R"}="H402 D1R"; $resn_exch{"1H33 D1R"}="H331 D1R";
$resn_exch{"2H33 D1R"}="H332 D1R"; $resn_exch{"3H33 D1R"}="H333 D1R"; $resn_exch{"1H49 D1R"}="H491 D1R";
$resn_exch{"2H49 D1R"}="H492 D1R"; $resn_exch{"1H48 D1R"}="H481 D1R"; $resn_exch{"2H48 D1R"}="H482 D1R";
$resn_exch{"1H10 D1R"}="H101 D1R"; $resn_exch{"2H10 D1R"}="H102 D1R"; $resn_exch{"1H19 D1R"}="H191 D1R";
$resn_exch{"2H19 D1R"}="H192 D1R"; $resn_exch{"1H20 D1R"}="H201 D1R"; $resn_exch{"2H20 D1R"}="H202 D1R";
$resn_exch{"1H27 D1R"}="H271 D1R"; $resn_exch{"2H27 D1R"}="H272 D1R"; $resn_exch{"1H28 D1R"}="H281 D1R";
$resn_exch{"2H28 D1R"}="H282 D1R"; $resn_exch{"1H10 D1T"}="H101 D1T"; $resn_exch{"2H10 D1T"}="H102 D1T";
$resn_exch{"3H10 D1T"}="H103 D1T"; $resn_exch{"1H9  D1T"}=" H91 D1T"; $resn_exch{"2H9  D1T"}=" H92 D1T";
$resn_exch{"1H8  D1T"}=" H81 D1T"; $resn_exch{"2H8  D1T"}=" H82 D1T"; $resn_exch{"1H7  D1T"}=" H71 D1T";
$resn_exch{"2H7  D1T"}=" H72 D1T"; $resn_exch{"1H6  D1T"}=" H61 D1T"; $resn_exch{"2H6  D1T"}=" H62 D1T";
$resn_exch{"1H5  D1T"}=" H51 D1T"; $resn_exch{"2H5  D1T"}=" H52 D1T"; $resn_exch{"1H4  D1T"}=" H41 D1T";
$resn_exch{"2H4  D1T"}=" H42 D1T"; $resn_exch{"1H3  D1T"}=" H31 D1T"; $resn_exch{"2H3  D1T"}=" H32 D1T";
$resn_exch{"1H2  D1T"}=" H21 D1T"; $resn_exch{"2H2  D1T"}=" H22 D1T"; $resn_exch{"1H1  D1T"}=" H11 D1T";
$resn_exch{"2H1  D1T"}=" H12 D1T"; $resn_exch{"1HN  D20"}=" HN1 D20"; $resn_exch{"2HN  D20"}=" HN2 D20";
$resn_exch{"1H2C D20"}="H2C1 D20"; $resn_exch{"2H2C D20"}="H2C2 D20"; $resn_exch{"1H3C D20"}="H3C1 D20";
$resn_exch{"2H3C D20"}="H3C2 D20"; $resn_exch{"1H4C D20"}="H4C1 D20"; $resn_exch{"2H4C D20"}="H4C2 D20";
$resn_exch{"1H7C D20"}="H7C1 D20"; $resn_exch{"2H7C D20"}="H7C2 D20"; $resn_exch{"1H10 D20"}="H101 D20";
$resn_exch{"2H10 D20"}="H102 D20"; $resn_exch{"1H8C D20"}="H8C1 D20"; $resn_exch{"2H8C D20"}="H8C2 D20";
$resn_exch{"3H8C D20"}="H8C3 D20"; $resn_exch{"1HN7 D23"}="HN71 D23"; $resn_exch{"2HN7 D23"}="HN72 D23";
$resn_exch{"1H20 D23"}="H201 D23"; $resn_exch{"2H20 D23"}="H202 D23"; $resn_exch{"3H20 D23"}="H203 D23";
$resn_exch{"1H21 D23"}="H211 D23"; $resn_exch{"2H21 D23"}="H212 D23"; $resn_exch{"3H21 D23"}="H213 D23";
$resn_exch{"1H22 D23"}="H221 D23"; $resn_exch{"2H22 D23"}="H222 D23"; $resn_exch{"3H22 D23"}="H223 D23";
$resn_exch{"1H1  D24"}="H11A D24"; $resn_exch{"2H1  D24"}="H12A D24"; $resn_exch{"1H1' D24"}="H1'1 D24";
$resn_exch{"2H1' D24"}="H1'2 D24"; $resn_exch{"1H21 D27"}="H211 D27"; $resn_exch{"2H21 D27"}="H212 D27";
$resn_exch{"3H21 D27"}="H213 D27"; $resn_exch{"1H22 D27"}="H221 D27"; $resn_exch{"2H22 D27"}="H222 D27";
$resn_exch{"3H22 D27"}="H223 D27"; $resn_exch{"1H13 D28"}="H131 D28"; $resn_exch{"2H13 D28"}="H132 D28";
$resn_exch{"3H13 D28"}="H133 D28"; $resn_exch{"1H23 D2A"}="H231 D2A"; $resn_exch{"2H23 D2A"}="H232 D2A";
$resn_exch{"1H26 D2A"}="H261 D2A"; $resn_exch{"2H26 D2A"}="H262 D2A"; $resn_exch{"1H1  D2C"}="H11A D2C";
$resn_exch{"2H1  D2C"}="H12A D2C"; $resn_exch{"3H1  D2C"}=" H13 D2C"; $resn_exch{"1H2  D2C"}=" H21 D2C";
$resn_exch{"2H2  D2C"}=" H22 D2C"; $resn_exch{"3H2  D2C"}=" H23 D2C"; $resn_exch{"1H6  D2C"}=" H61 D2C";
$resn_exch{"2H6  D2C"}=" H62 D2C"; $resn_exch{"1HN2 D2C"}="HN21 D2C"; $resn_exch{"2HN2 D2C"}="HN22 D2C";
$resn_exch{"1H10 D2G"}="H101 D2G"; $resn_exch{"2H10 D2G"}="H102 D2G"; $resn_exch{"1H11 D2G"}="H111 D2G";
$resn_exch{"2H11 D2G"}="H112 D2G"; $resn_exch{"3H11 D2G"}="H113 D2G"; $resn_exch{"1H2C D2P"}="H2C1 D2P";
$resn_exch{"2H2C D2P"}="H2C2 D2P"; $resn_exch{"1H4C D2P"}="H4C1 D2P"; $resn_exch{"2H4C D2P"}="H4C2 D2P";
$resn_exch{"2HOP  D3"}="HOP2  D3"; $resn_exch{"3HOP  D3"}="HOP3  D3"; $resn_exch{"1H5*  D3"}=" H5'  D3";
$resn_exch{"2H5*  D3"}="H5''  D3"; $resn_exch{" H4*  D3"}=" H4'  D3"; $resn_exch{" H3*  D3"}=" H3'  D3";
$resn_exch{"*HO3  D3"}="HO3'  D3"; $resn_exch{"1H2*  D3"}=" H2'  D3"; $resn_exch{"2H2*  D3"}="H2''  D3";
$resn_exch{" H1*  D3"}=" H1'  D3"; $resn_exch{"1HN1 D31"}="HN11 D31"; $resn_exch{"2HN1 D31"}="HN12 D31";
$resn_exch{"1H2  D31"}=" H21 D31"; $resn_exch{"2H2  D31"}=" H22 D31"; $resn_exch{"1H6  D31"}=" H61 D31";
$resn_exch{"2H6  D31"}=" H62 D31"; $resn_exch{"1H7  D31"}=" H71 D31"; $resn_exch{"2H7  D31"}=" H72 D31";
$resn_exch{"1H9  D31"}=" H91 D31"; $resn_exch{"2H9  D31"}=" H92 D31"; $resn_exch{"1H10 D31"}="H101 D31";
$resn_exch{"2H10 D31"}="H102 D31"; $resn_exch{"1H20 D31"}="H201 D31"; $resn_exch{"2H20 D31"}="H202 D31";
$resn_exch{"1H37 D31"}="H371 D31"; $resn_exch{"2H37 D31"}="H372 D31"; $resn_exch{"1H38 D31"}="H381 D31";
$resn_exch{"2H38 D31"}="H382 D31"; $resn_exch{"1H39 D31"}="H391 D31"; $resn_exch{"2H39 D31"}="H392 D31";
$resn_exch{"1H40 D31"}="H401 D31"; $resn_exch{"2H40 D31"}="H402 D31"; $resn_exch{"1H41 D31"}="H411 D31";
$resn_exch{"2H41 D31"}="H412 D31"; $resn_exch{"7H7  D34"}=" H77 D34"; $resn_exch{"1HN  D34"}=" HN1 D34";
$resn_exch{"2HN  D34"}="HN2A D34"; $resn_exch{"1H9  D34"}=" H91 D34"; $resn_exch{"2H9  D34"}=" H92 D34";
$resn_exch{"1H0  D34"}=" H01 D34"; $resn_exch{"2H0  D34"}=" H02 D34"; $resn_exch{"1H1  D34"}=" H11 D34";
$resn_exch{"2H1  D34"}=" H12 D34"; $resn_exch{"'H7' D34"}=" H7B D34"; $resn_exch{"1HN' D34"}="HN'1 D34";
$resn_exch{"2HN' D34"}="HN'2 D34"; $resn_exch{"1H9' D34"}="H9'1 D34"; $resn_exch{"2H9' D34"}="H9'2 D34";
$resn_exch{"1H0' D34"}="H0'1 D34"; $resn_exch{"2H0' D34"}="H0'2 D34"; $resn_exch{"1H1' D34"}="H1'1 D34";
$resn_exch{"2H1' D34"}="H1'2 D34"; $resn_exch{"7H7  D35"}=" H77 D35"; $resn_exch{"1HN  D35"}=" HN1 D35";
$resn_exch{"2HN  D35"}="HN2A D35"; $resn_exch{"1H9  D35"}=" H91 D35"; $resn_exch{"2H9  D35"}=" H92 D35";
$resn_exch{"1H0  D35"}=" H01 D35"; $resn_exch{"2H0  D35"}=" H02 D35"; $resn_exch{"1H1  D35"}=" H11 D35";
$resn_exch{"2H1  D35"}=" H12 D35"; $resn_exch{"1H2  D35"}=" H21 D35"; $resn_exch{"2H2  D35"}=" H22 D35";
$resn_exch{"1H3  D35"}=" H31 D35"; $resn_exch{"2H3  D35"}=" H32 D35"; $resn_exch{"'H7' D35"}=" H7B D35";
$resn_exch{"1HN' D35"}="HN'1 D35"; $resn_exch{"2HN' D35"}="HN'2 D35"; $resn_exch{"1H9' D35"}="H9'1 D35";
$resn_exch{"2H9' D35"}="H9'2 D35"; $resn_exch{"1H0' D35"}="H0'1 D35"; $resn_exch{"2H0' D35"}="H0'2 D35";
$resn_exch{"1H1' D35"}="H1'1 D35"; $resn_exch{"2H1' D35"}="H1'2 D35"; $resn_exch{"1H2' D35"}="H2'1 D35";
$resn_exch{"2H2' D35"}="H2'2 D35"; $resn_exch{"1H3' D35"}="H3'1 D35"; $resn_exch{"2H3' D35"}="H3'2 D35";
$resn_exch{"1H7  D3F"}=" H71 D3F"; $resn_exch{"2H7  D3F"}=" H72 D3F"; $resn_exch{"3H7  D3F"}=" H73 D3F";
$resn_exch{"1H10 D3G"}="H101 D3G"; $resn_exch{"2H10 D3G"}="H102 D3G"; $resn_exch{"1H_2 D3G"}="H_21 D3G";
$resn_exch{"2H_2 D3G"}="H_22 D3G"; $resn_exch{"1HN  D3P"}=" HN1 D3P"; $resn_exch{"2HN  D3P"}=" HN2 D3P";
$resn_exch{"2HOA D3T"}="HOA2 D3T"; $resn_exch{"1H5* D3T"}="H5'1 D3T"; $resn_exch{"2H5* D3T"}="H5'2 D3T";
$resn_exch{" H4* D3T"}=" H4' D3T"; $resn_exch{" H1* D3T"}=" H1' D3T"; $resn_exch{"1H5M D3T"}="H5M1 D3T";
$resn_exch{"2H5M D3T"}="H5M2 D3T"; $resn_exch{"3H5M D3T"}="H5M3 D3T"; $resn_exch{"1H2* D3T"}="H2'1 D3T";
$resn_exch{"2H2* D3T"}="H2'2 D3T"; $resn_exch{"1H3* D3T"}="H3'1 D3T"; $resn_exch{"2H3* D3T"}="H3'2 D3T";
$resn_exch{"2HOB D3T"}="HOB2 D3T"; $resn_exch{"3HOG D3T"}="HOG3 D3T"; $resn_exch{"1H1  D42"}=" H11 D42";
$resn_exch{"2H1  D42"}=" H12 D42"; $resn_exch{"1H2  D42"}="H21A D42"; $resn_exch{"2H2  D42"}=" H22 D42";
$resn_exch{"1H4  D42"}=" H41 D42"; $resn_exch{"2H4  D42"}=" H42 D42"; $resn_exch{"1H5  D42"}=" H51 D42";
$resn_exch{"2H5  D42"}=" H52 D42"; $resn_exch{"1H15 D42"}="H151 D42"; $resn_exch{"2H15 D42"}="H152 D42";
$resn_exch{"3H15 D42"}="H153 D42"; $resn_exch{"1H42 D42"}="H421 D42"; $resn_exch{"2H42 D42"}="H422 D42";
$resn_exch{"3H42 D42"}="H423 D42"; $resn_exch{"1H46 D42"}="H461 D42"; $resn_exch{"2H46 D42"}="H462 D42";
$resn_exch{"3H46 D42"}="H463 D42"; $resn_exch{"OH2A D4D"}="H2AO D4D"; $resn_exch{"OH1B D4D"}="H1BO D4D";
$resn_exch{"OH3B D4D"}="H3BO D4D"; $resn_exch{"1H5' D4D"}="H5'1 D4D"; $resn_exch{"2H5' D4D"}="H5'2 D4D";
$resn_exch{"CH4' D4D"}="H4'C D4D"; $resn_exch{"CH3' D4D"}="H3'C D4D"; $resn_exch{"CH2' D4D"}="H2'C D4D";
$resn_exch{"CH1' D4D"}="H1'C D4D"; $resn_exch{"1H5M D4D"}="H5M1 D4D"; $resn_exch{"2H5M D4D"}="H5M2 D4D";
$resn_exch{"3H5M D4D"}="H5M3 D4D"; $resn_exch{"6H6  D4D"}=" H66 D4D"; $resn_exch{"2HOP D4M"}="HOP2 D4M";
$resn_exch{"3HOP D4M"}="HOP3 D4M"; $resn_exch{"1H5* D4M"}=" H5' D4M"; $resn_exch{"2H5* D4M"}="H5'' D4M";
$resn_exch{" H4* D4M"}=" H4' D4M"; $resn_exch{" H3* D4M"}=" H3' D4M"; $resn_exch{" H2* D4M"}=" H2' D4M";
$resn_exch{" H1* D4M"}=" H1' D4M"; $resn_exch{"1H5M D4M"}=" H71 D4M"; $resn_exch{"2H5M D4M"}=" H72 D4M";
$resn_exch{"3H5M D4M"}=" H73 D4M"; $resn_exch{"1HN  D4P"}=" HN1 D4P"; $resn_exch{"2HN  D4P"}=" HN2 D4P";
$resn_exch{"OH1C D4T"}="H1CO D4T"; $resn_exch{"OH2C D4T"}="H2CO D4T"; $resn_exch{"OH2B D4T"}="H2BO D4T";
$resn_exch{"OH2A D4T"}="H2AO D4T"; $resn_exch{"1H5' D4T"}="H5'1 D4T"; $resn_exch{"2H5' D4T"}="H5'2 D4T";
$resn_exch{"CH4' D4T"}="H4'C D4T"; $resn_exch{"CH1' D4T"}="H1'C D4T"; $resn_exch{"6H6  D4T"}=" H66 D4T";
$resn_exch{"1HCA D4T"}="HCA1 D4T"; $resn_exch{"2HCA D4T"}="HCA2 D4T"; $resn_exch{"3HCA D4T"}="HCA3 D4T";
$resn_exch{"CH2' D4T"}="H2'C D4T"; $resn_exch{"CH3' D4T"}="H3'C D4T"; $resn_exch{"1H2C D55"}="H2C1 D55";
$resn_exch{"2H2C D55"}="H2C2 D55"; $resn_exch{"3H2C D55"}="H2C3 D55"; $resn_exch{"1H3C D55"}="H3C1 D55";
$resn_exch{"2H3C D55"}="H3C2 D55"; $resn_exch{"1H5C D55"}="H5C1 D55"; $resn_exch{"2H5C D55"}="H5C2 D55";
$resn_exch{"1H6C D55"}="H6C1 D55"; $resn_exch{"2H6C D55"}="H6C2 D55"; $resn_exch{"3H6C D55"}="H6C3 D55";
$resn_exch{"1H19 D55"}="H191 D55"; $resn_exch{"2H19 D55"}="H192 D55"; $resn_exch{"3H19 D55"}="H193 D55";
$resn_exch{"1H22 D55"}="H221 D55"; $resn_exch{"2H22 D55"}="H222 D55"; $resn_exch{"1H23 D55"}="H231 D55";
$resn_exch{"2H23 D55"}="H232 D55"; $resn_exch{"1H24 D55"}="H241 D55"; $resn_exch{"2H24 D55"}="H242 D55";
$resn_exch{"1H29 D55"}="H291 D55"; $resn_exch{"2H29 D55"}="H292 D55"; $resn_exch{"3H29 D55"}="H293 D55";
$resn_exch{"1H2C D56"}="H2C1 D56"; $resn_exch{"2H2C D56"}="H2C2 D56"; $resn_exch{"1H6C D56"}="H6C1 D56";
$resn_exch{"2H6C D56"}="H6C2 D56"; $resn_exch{"1H3C D56"}="H3C1 D56"; $resn_exch{"2H3C D56"}="H3C2 D56";
$resn_exch{"1H4C D56"}="H4C1 D56"; $resn_exch{"2H4C D56"}="H4C2 D56"; $resn_exch{"1H5C D56"}="H5C1 D56";
$resn_exch{"2H5C D56"}="H5C2 D56"; $resn_exch{"1H19 D56"}="H191 D56"; $resn_exch{"2H19 D56"}="H192 D56";
$resn_exch{"3H19 D56"}="H193 D56"; $resn_exch{"1H10 D56"}="H101 D56"; $resn_exch{"2H10 D56"}="H102 D56";
$resn_exch{"3H10 D56"}="H103 D56"; $resn_exch{"1H13 D56"}="H131 D56"; $resn_exch{"2H13 D56"}="H132 D56";
$resn_exch{"1H22 D56"}="H221 D56"; $resn_exch{"2H22 D56"}="H222 D56"; $resn_exch{"1H23 D56"}="H231 D56";
$resn_exch{"2H23 D56"}="H232 D56"; $resn_exch{"1H24 D56"}="H241 D56"; $resn_exch{"2H24 D56"}="H242 D56";
$resn_exch{"1H29 D56"}="H291 D56"; $resn_exch{"2H29 D56"}="H292 D56"; $resn_exch{"3H29 D56"}="H293 D56";
$resn_exch{"1HB  D5G"}=" HB1 D5G"; $resn_exch{"2HB  D5G"}=" HB2 D5G"; $resn_exch{"1HD2 D5G"}="HD21 D5G";
$resn_exch{"2HD2 D5G"}="HD22 D5G"; $resn_exch{"3HD2 D5G"}="HD23 D5G"; $resn_exch{"1HD1 D5G"}="HD11 D5G";
$resn_exch{"2HD1 D5G"}="HD12 D5G"; $resn_exch{"3HD1 D5G"}="HD13 D5G"; $resn_exch{"1HAX D5G"}="HAX1 D5G";
$resn_exch{"2HAX D5G"}="HAX2 D5G"; $resn_exch{"1HAN D5G"}="HAN1 D5G"; $resn_exch{"2HAN D5G"}="HAN2 D5G";
$resn_exch{"1H5* D5M"}="H5'1 D5M"; $resn_exch{"2H5* D5M"}="H5'2 D5M"; $resn_exch{" H4* D5M"}=" H4' D5M";
$resn_exch{" H3* D5M"}=" H3' D5M"; $resn_exch{"1H2* D5M"}="H2'1 D5M"; $resn_exch{"2H2* D5M"}="H2'2 D5M";
$resn_exch{" H1* D5M"}=" H1' D5M"; $resn_exch{"1HN6 D5M"}="HN61 D5M"; $resn_exch{"2HN6 D5M"}="HN62 D5M";
$resn_exch{"1H2  D6G"}=" H21 D6G"; $resn_exch{"2H2  D6G"}=" H22 D6G"; $resn_exch{"1H6  D6G"}=" H61 D6G";
$resn_exch{"2H6  D6G"}=" H62 D6G"; $resn_exch{"2HOP D6G"}="HOP2 D6G"; $resn_exch{"3HOP D6G"}="HOP3 D6G";
$resn_exch{"1H1  D6P"}=" H11 D6P"; $resn_exch{"2H1  D6P"}=" H12 D6P"; $resn_exch{"1H2  D6P"}=" H21 D6P";
$resn_exch{"2H2  D6P"}=" H22 D6P"; $resn_exch{"1HZ7 D76"}="HZ71 D76"; $resn_exch{"2HZ7 D76"}="HZ72 D76";
$resn_exch{"3HZ7 D76"}="HZ73 D76"; $resn_exch{"1HZ3 D76"}="HZ31 D76"; $resn_exch{"2HZ3 D76"}="HZ32 D76";
$resn_exch{"1HZ4 D76"}="HZ41 D76"; $resn_exch{"2HZ4 D76"}="HZ42 D76"; $resn_exch{"1HZ5 D76"}="HZ51 D76";
$resn_exch{"2HZ5 D76"}="HZ52 D76"; $resn_exch{"1HP3 D76"}="HP31 D76"; $resn_exch{"2HP3 D76"}="HP32 D76";
$resn_exch{"1HP4 D76"}="HP41 D76"; $resn_exch{"2HP4 D76"}="HP42 D76"; $resn_exch{"1HC2 D76"}="HC21 D76";
$resn_exch{"2HC2 D76"}="HC22 D76"; $resn_exch{"1HP1 D76"}="HP11 D76"; $resn_exch{"2HP1 D76"}="HP12 D76";
$resn_exch{"1H4  D7G"}=" H41 D7G"; $resn_exch{"2H4  D7G"}=" H42 D7G"; $resn_exch{"1H5  D7G"}=" H51 D7G";
$resn_exch{"2H5  D7G"}=" H52 D7G"; $resn_exch{"1H10 D7G"}="H101 D7G"; $resn_exch{"2H10 D7G"}="H102 D7G";
$resn_exch{"1H11 D7G"}="H111 D7G"; $resn_exch{"2H11 D7G"}="H112 D7G"; $resn_exch{"1H12 D7G"}="H121 D7G";
$resn_exch{"2H12 D7G"}="H122 D7G"; $resn_exch{"1H13 D7G"}="H131 D7G"; $resn_exch{"2H13 D7G"}="H132 D7G";
$resn_exch{"1H14 D7G"}="H141 D7G"; $resn_exch{"2H14 D7G"}="H142 D7G"; $resn_exch{"3H14 D7G"}="H143 D7G";
$resn_exch{"1H15 D7G"}="H151 D7G"; $resn_exch{"2H15 D7G"}="H152 D7G"; $resn_exch{"1H17 D7G"}="H171 D7G";
$resn_exch{"2H17 D7G"}="H172 D7G"; $resn_exch{"3H17 D7G"}="H173 D7G"; $resn_exch{"1H18 D7G"}="H181 D7G";
$resn_exch{"2H18 D7G"}="H182 D7G"; $resn_exch{"3H18 D7G"}="H183 D7G"; $resn_exch{"1H19 D7G"}="H191 D7G";
$resn_exch{"2H19 D7G"}="H192 D7G"; $resn_exch{"1H1' D7P"}="H1'1 D7P"; $resn_exch{"2H1' D7P"}="H1'2 D7P";
$resn_exch{"1HN' D7P"}="HN'1 D7P"; $resn_exch{"2HN' D7P"}="HN'2 D7P"; $resn_exch{"1HB' D7P"}="HB'1 D7P";
$resn_exch{"2HB' D7P"}="HB'2 D7P"; $resn_exch{"1HB  D7P"}=" HB1 D7P"; $resn_exch{"2HB  D7P"}=" HB2 D7P";
$resn_exch{"1HG  D7P"}=" HG1 D7P"; $resn_exch{"2HG  D7P"}=" HG2 D7P"; $resn_exch{"1HD  D7P"}="HD1A D7P";
$resn_exch{"2HD  D7P"}="HD2A D7P"; $resn_exch{"1H19 D91"}="H191 D91"; $resn_exch{"2H19 D91"}="H192 D91";
$resn_exch{"1H16 D91"}="H161 D91"; $resn_exch{"2H16 D91"}="H162 D91"; $resn_exch{"1H15 D91"}="H151 D91";
$resn_exch{"2H15 D91"}="H152 D91"; $resn_exch{"1H7  D91"}=" H71 D91"; $resn_exch{"2H7  D91"}=" H72 D91";
$resn_exch{"1H11 D91"}="H111 D91"; $resn_exch{"2H11 D91"}="H112 D91"; $resn_exch{"3H11 D91"}="H113 D91";
$resn_exch{"1H12 D91"}="H121 D91"; $resn_exch{"2H12 D91"}="H122 D91"; $resn_exch{"3H12 D91"}="H123 D91";
$resn_exch{"1H6  D92"}=" H61 D92"; $resn_exch{"2H6  D92"}=" H62 D92"; $resn_exch{"1H5  D92"}=" H51 D92";
$resn_exch{"2H5  D92"}=" H52 D92"; $resn_exch{"1H4  D92"}=" H41 D92"; $resn_exch{"2H4  D92"}=" H42 D92";
$resn_exch{"1H3  D92"}=" H31 D92"; $resn_exch{"2H3  D92"}=" H32 D92"; $resn_exch{"1H26 D92"}="H261 D92";
$resn_exch{"2H26 D92"}="H262 D92"; $resn_exch{"1H30 D92"}="H301 D92"; $resn_exch{"2H30 D92"}="H302 D92";
$resn_exch{"3H30 D92"}="H303 D92"; $resn_exch{"1H28 D92"}="H281 D92"; $resn_exch{"2H28 D92"}="H282 D92";
$resn_exch{"1H29 D92"}="H291 D92"; $resn_exch{"2H29 D92"}="H292 D92"; $resn_exch{"1H6  D93"}=" H61 D93";
$resn_exch{"2H6  D93"}=" H62 D93"; $resn_exch{"1H5  D93"}=" H51 D93"; $resn_exch{"2H5  D93"}=" H52 D93";
$resn_exch{"1H4  D93"}=" H41 D93"; $resn_exch{"2H4  D93"}=" H42 D93"; $resn_exch{"1H3  D93"}=" H31 D93";
$resn_exch{"2H3  D93"}=" H32 D93"; $resn_exch{"1H26 D93"}="H261 D93"; $resn_exch{"2H26 D93"}="H262 D93";
$resn_exch{"1H30 D93"}="H301 D93"; $resn_exch{"2H30 D93"}="H302 D93"; $resn_exch{"3H30 D93"}="H303 D93";
$resn_exch{"1H28 D93"}="H281 D93"; $resn_exch{"2H28 D93"}="H282 D93"; $resn_exch{"1H29 D93"}="H291 D93";
$resn_exch{"2H29 D93"}="H292 D93"; $resn_exch{"1HAC D9G"}="HAC1 D9G"; $resn_exch{"2HAC D9G"}="HAC2 D9G";
$resn_exch{"3HAC D9G"}="HAC3 D9G"; $resn_exch{"1HAD D9G"}="HAD1 D9G"; $resn_exch{"2HAD D9G"}="HAD2 D9G";
$resn_exch{"1HAE D9G"}="HAE1 D9G"; $resn_exch{"2HAE D9G"}="HAE2 D9G"; $resn_exch{"1HAF D9G"}="HAF1 D9G";
$resn_exch{"2HAF D9G"}="HAF2 D9G"; $resn_exch{"1HAG D9G"}="HAG1 D9G"; $resn_exch{"2HAG D9G"}="HAG2 D9G";
$resn_exch{"1HAH D9G"}="HAH1 D9G"; $resn_exch{"2HAH D9G"}="HAH2 D9G"; $resn_exch{"1HAI D9G"}="HAI1 D9G";
$resn_exch{"2HAI D9G"}="HAI2 D9G"; $resn_exch{"1HAJ D9G"}="HAJ1 D9G"; $resn_exch{"2HAJ D9G"}="HAJ2 D9G";
$resn_exch{"1HAK D9G"}="HAK1 D9G"; $resn_exch{"2HAK D9G"}="HAK2 D9G"; $resn_exch{"1HAL D9G"}="HAL1 D9G";
$resn_exch{"2HAL D9G"}="HAL2 D9G"; $resn_exch{"1HAM D9G"}="HAM1 D9G"; $resn_exch{"2HAM D9G"}="HAM2 D9G";
$resn_exch{"1HAN D9G"}="HAN1 D9G"; $resn_exch{"2HAN D9G"}="HAN2 D9G"; $resn_exch{"1HAS D9G"}="HAS1 D9G";
$resn_exch{"2HAS D9G"}="HAS2 D9G"; $resn_exch{"3HAS D9G"}="HAS3 D9G"; $resn_exch{"1HAA D9G"}="HAA1 D9G";
$resn_exch{"2HAA D9G"}="HAA2 D9G"; $resn_exch{"3HAA D9G"}="HAA3 D9G"; $resn_exch{"1HAP D9G"}="HAP1 D9G";
$resn_exch{"2HAP D9G"}="HAP2 D9G"; $resn_exch{"3HOP  DA"}="HOP3  DA"; $resn_exch{"2HOP  DA"}="HOP2  DA";
$resn_exch{"1H5*  DA"}=" H5'  DA"; $resn_exch{"2H5*  DA"}="H5''  DA"; $resn_exch{" H4*  DA"}=" H4'  DA";
$resn_exch{" H3*  DA"}=" H3'  DA"; $resn_exch{"H3T   DA"}="HO3'  DA"; $resn_exch{"1H2*  DA"}=" H2'  DA";
$resn_exch{"2H2*  DA"}="H2''  DA"; $resn_exch{" H1*  DA"}=" H1'  DA"; $resn_exch{"1H6   DA"}=" H61  DA";
$resn_exch{"2H6   DA"}=" H62  DA"; $resn_exch{"1HN2 DA1"}="HN21 DA1"; $resn_exch{"2HN2 DA1"}="HN22 DA1";
$resn_exch{"1HN3 DA1"}="HN31 DA1"; $resn_exch{"2HN3 DA1"}="HN32 DA1"; $resn_exch{"1HC1 DA2"}="HC11 DA2";
$resn_exch{"2HC1 DA2"}="HC12 DA2"; $resn_exch{"3HC1 DA2"}="HC13 DA2"; $resn_exch{"1HC2 DA2"}="HC21 DA2";
$resn_exch{"2HC2 DA2"}="HC22 DA2"; $resn_exch{"3HC2 DA2"}="HC23 DA2"; $resn_exch{"1HN  DA2"}=" HN1 DA2";
$resn_exch{"2HN  DA2"}=" HN2 DA2"; $resn_exch{"1HCB DA2"}="HCB1 DA2"; $resn_exch{"2HCB DA2"}="HCB2 DA2";
$resn_exch{"1HCG DA2"}="HCG1 DA2"; $resn_exch{"2HCG DA2"}="HCG2 DA2"; $resn_exch{"1HCD DA2"}="HCD1 DA2";
$resn_exch{"2HCD DA2"}="HCD2 DA2"; $resn_exch{"1HN1 DA3"}="HN11 DA3"; $resn_exch{"2HN1 DA3"}="HN12 DA3";
$resn_exch{"1H3  DA3"}=" H31 DA3"; $resn_exch{"2H3  DA3"}=" H32 DA3"; $resn_exch{"1H5  DA3"}=" H51 DA3";
$resn_exch{"2H5  DA3"}=" H52 DA3"; $resn_exch{"1HN9 DA5"}="HN91 DA5"; $resn_exch{"2HN9 DA5"}="HN92 DA5";
$resn_exch{"3HN9 DA5"}="HN93 DA5"; $resn_exch{"1HND DA5"}="HND1 DA5"; $resn_exch{"1HD2 DA5"}="HD21 DA5";
$resn_exch{"2HD2 DA5"}="HD22 DA5"; $resn_exch{"1HD3 DA5"}="HD31 DA5"; $resn_exch{"2HD3 DA5"}="HD32 DA5";
$resn_exch{"1HD7 DA5"}="HD71 DA5"; $resn_exch{"2HD7 DA5"}="HD72 DA5"; $resn_exch{"3HD7 DA5"}="HD73 DA5";
$resn_exch{"1HD8 DA5"}="HD81 DA5"; $resn_exch{"2HD8 DA5"}="HD82 DA5"; $resn_exch{"3HD8 DA5"}="HD83 DA5";
$resn_exch{"1HN9 DA6"}="HN91 DA6"; $resn_exch{"2HN9 DA6"}="HN92 DA6"; $resn_exch{"3HN9 DA6"}="HN93 DA6";
$resn_exch{"1HND DA6"}="HND1 DA6"; $resn_exch{"1HD2 DA6"}="HD21 DA6"; $resn_exch{"2HD2 DA6"}="HD22 DA6";
$resn_exch{"1HD3 DA6"}="HD31 DA6"; $resn_exch{"2HD3 DA6"}="HD32 DA6"; $resn_exch{"1HD7 DA6"}="HD71 DA6";
$resn_exch{"2HD7 DA6"}="HD72 DA6"; $resn_exch{"3HD7 DA6"}="HD73 DA6"; $resn_exch{"1HD8 DA6"}="HD81 DA6";
$resn_exch{"2HD8 DA6"}="HD82 DA6"; $resn_exch{"3HD8 DA6"}="HD83 DA6"; $resn_exch{"1HN9 DA7"}="HN91 DA7";
$resn_exch{"2HN9 DA7"}="HN92 DA7"; $resn_exch{"1H17 DA7"}="H171 DA7"; $resn_exch{"2H17 DA7"}="H172 DA7";
$resn_exch{"1H18 DA7"}="H181 DA7"; $resn_exch{"2H18 DA7"}="H182 DA7"; $resn_exch{"1H20 DA7"}="H201 DA7";
$resn_exch{"2H20 DA7"}="H202 DA7"; $resn_exch{"3H20 DA7"}="H203 DA7"; $resn_exch{"1H21 DA7"}="H211 DA7";
$resn_exch{"2H21 DA7"}="H212 DA7"; $resn_exch{"3H21 DA7"}="H213 DA7"; $resn_exch{"1HCA DAA"}="HCA1 DAA";
$resn_exch{"2HCA DAA"}="HCA2 DAA"; $resn_exch{"1HCB DAA"}="HCB1 DAA"; $resn_exch{"2HCB DAA"}="HCB2 DAA";
$resn_exch{"1HCG DAA"}="HCG1 DAA"; $resn_exch{"2HCG DAA"}="HCG2 DAA"; $resn_exch{"1HCD DAA"}="HCD1 DAA";
$resn_exch{"2HCD DAA"}="HCD2 DAA"; $resn_exch{"1HCE DAA"}="HCE1 DAA"; $resn_exch{"2HCE DAA"}="HCE2 DAA";
$resn_exch{"1HCS DAA"}="HCS1 DAA"; $resn_exch{"2HCS DAA"}="HCS2 DAA"; $resn_exch{"3HCS DAA"}="HCS3 DAA";
$resn_exch{"1HN2 DAA"}="HN21 DAA"; $resn_exch{"2HN2 DAA"}="HN22 DAA"; $resn_exch{"1HN  DAB"}=" H   DAB";
$resn_exch{"2HN  DAB"}=" H2  DAB"; $resn_exch{"1HB  DAB"}=" HB2 DAB"; $resn_exch{"2HB  DAB"}=" HB3 DAB";
$resn_exch{"1HG  DAB"}=" HG2 DAB"; $resn_exch{"2HG  DAB"}=" HG3 DAB"; $resn_exch{"1HD  DAB"}=" HD1 DAB";
$resn_exch{"2HD  DAB"}=" HD2 DAB"; $resn_exch{"1H4  DAC"}=" H41 DAC"; $resn_exch{"2H4  DAC"}=" H42 DAC";
$resn_exch{"1H5  DAC"}=" H51 DAC"; $resn_exch{"2H5  DAC"}=" H52 DAC"; $resn_exch{"1H6  DAC"}=" H61 DAC";
$resn_exch{"2H6  DAC"}=" H62 DAC"; $resn_exch{"1H7  DAC"}=" H71 DAC"; $resn_exch{"2H7  DAC"}=" H72 DAC";
$resn_exch{"1H8  DAC"}=" H81 DAC"; $resn_exch{"2H8  DAC"}=" H82 DAC"; $resn_exch{"1H9  DAC"}=" H91 DAC";
$resn_exch{"2H9  DAC"}=" H92 DAC"; $resn_exch{"1H10 DAC"}="H101 DAC"; $resn_exch{"2H10 DAC"}="H102 DAC";
$resn_exch{"3H10 DAC"}="H103 DAC"; $resn_exch{"1H1' DAC"}="H1'1 DAC"; $resn_exch{"2H1' DAC"}="H1'2 DAC";
$resn_exch{"1H2' DAC"}="H2'1 DAC"; $resn_exch{"2H2' DAC"}="H2'2 DAC"; $resn_exch{"1H5' DAC"}="H5'1 DAC";
$resn_exch{"2H5' DAC"}="H5'2 DAC"; $resn_exch{"3H5' DAC"}="H5'3 DAC"; $resn_exch{"2HOG DAD"}="HOG2 DAD";
$resn_exch{"3HOG DAD"}="HOG3 DAD"; $resn_exch{"2HOB DAD"}="HOB2 DAD"; $resn_exch{"2HOA DAD"}="HOA2 DAD";
$resn_exch{"1H5* DAD"}="H5'1 DAD"; $resn_exch{"2H5* DAD"}="H5'2 DAD"; $resn_exch{" H4* DAD"}=" H4' DAD";
$resn_exch{"1H3* DAD"}="H3'1 DAD"; $resn_exch{"2H3* DAD"}="H3'2 DAD"; $resn_exch{"1H2* DAD"}="H2'1 DAD";
$resn_exch{"2H2* DAD"}="H2'2 DAD"; $resn_exch{" H1* DAD"}=" H1' DAD"; $resn_exch{"1HN6 DAD"}="HN61 DAD";
$resn_exch{"2HN6 DAD"}="HN62 DAD"; $resn_exch{"2HOB DAE"}="HOB2 DAE"; $resn_exch{"2HOA DAE"}="HOA2 DAE";
$resn_exch{"1HNA DAE"}="HNA1 DAE"; $resn_exch{"1HA2 DAE"}="HA21 DAE"; $resn_exch{"2HA2 DAE"}="HA22 DAE";
$resn_exch{"1HA1 DAE"}="HA11 DAE"; $resn_exch{"2HA1 DAE"}="HA12 DAE"; $resn_exch{"1H1  DAF"}=" H11 DAF";
$resn_exch{"2H1  DAF"}=" H12 DAF"; $resn_exch{"1H6  DAF"}=" H61 DAF"; $resn_exch{"2H6  DAF"}=" H62 DAF";
$resn_exch{"3H6  DAF"}=" H63 DAF"; $resn_exch{"1H7H DAF"}="H7H1 DAF"; $resn_exch{"2H7H DAF"}="H7H2 DAF";
$resn_exch{"'HO2 DAF"}="HO2' DAF"; $resn_exch{"'HO3 DAF"}="HO3' DAF"; $resn_exch{"'HO4 DAF"}="HO4' DAF";
$resn_exch{"1H6  DAG"}=" H61 DAG"; $resn_exch{"2H6  DAG"}=" H62 DAG"; $resn_exch{"3H6  DAG"}=" H63 DAG";
$resn_exch{"1HN4 DAG"}="HN41 DAG"; $resn_exch{"2HN4 DAG"}="HN42 DAG"; $resn_exch{" HN2 DAH"}=" H2  DAH";
$resn_exch{"1HB  DAH"}=" HB2 DAH"; $resn_exch{"2HB  DAH"}=" HB3 DAH"; $resn_exch{" HOE DAH"}=" HE2 DAH";
$resn_exch{" HOZ DAH"}=" HZ  DAH"; $resn_exch{"1H5  DAI"}=" H51 DAI"; $resn_exch{"2H5  DAI"}=" H52 DAI";
$resn_exch{"1H21 DAI"}="H211 DAI"; $resn_exch{"2H21 DAI"}="H212 DAI"; $resn_exch{"1H20 DAI"}="H201 DAI";
$resn_exch{"2H20 DAI"}="H202 DAI"; $resn_exch{"1H3  DAI"}=" H31 DAI"; $resn_exch{"2H3  DAI"}=" H32 DAI";
$resn_exch{"1H2  DAI"}=" H21 DAI"; $resn_exch{"2H2  DAI"}=" H22 DAI"; $resn_exch{"1H1  DAI"}=" H11 DAI";
$resn_exch{"2H1  DAI"}=" H12 DAI"; $resn_exch{"3H1  DAI"}="H13A DAI"; $resn_exch{"1H1  DAJ"}=" H11 DAJ";
$resn_exch{"2H1  DAJ"}=" H12 DAJ"; $resn_exch{"1HN  DAJ"}=" HN1 DAJ"; $resn_exch{"2HN  DAJ"}=" HN2 DAJ";
$resn_exch{"3HN  DAJ"}=" HN3 DAJ"; $resn_exch{"1H3  DAJ"}=" H31 DAJ"; $resn_exch{"2H3  DAJ"}=" H32 DAJ";
$resn_exch{"1H13 DAJ"}="H131 DAJ"; $resn_exch{"2H13 DAJ"}="H132 DAJ"; $resn_exch{"3H13 DAJ"}="H133 DAJ";
$resn_exch{"4HN  DAJ"}=" HN4 DAJ"; $resn_exch{"5HN  DAJ"}=" HN5 DAJ"; $resn_exch{"1H10 DAJ"}="H101 DAJ";
$resn_exch{"2H10 DAJ"}="H102 DAJ"; $resn_exch{"3H10 DAJ"}="H103 DAJ"; $resn_exch{"2H2  DAK"}="H22A DAK";
$resn_exch{"1HN6 DAK"}="HN61 DAK"; $resn_exch{"2HN6 DAK"}="HN62 DAK"; $resn_exch{"1H8  DAK"}=" H81 DAK";
$resn_exch{"1H1' DAK"}="H1'1 DAK"; $resn_exch{"1H2' DAK"}="H2'1 DAK"; $resn_exch{"1HO2 DAK"}="HO21 DAK";
$resn_exch{"1H3' DAK"}="H3'1 DAK"; $resn_exch{"1H4' DAK"}="H4'1 DAK"; $resn_exch{"1H5' DAK"}="H5'1 DAK";
$resn_exch{"2H5' DAK"}="H5'2 DAK"; $resn_exch{"1HP9 DAK"}="HP91 DAK"; $resn_exch{"2HP9 DAK"}="HP92 DAK";
$resn_exch{"1HPA DAK"}="HPA1 DAK"; $resn_exch{"2HPA DAK"}="HPA2 DAK"; $resn_exch{"3HPA DAK"}="HPA3 DAK";
$resn_exch{"1HPB DAK"}="HPB1 DAK"; $resn_exch{"2HPB DAK"}="HPB2 DAK"; $resn_exch{"3HPB DAK"}="HPB3 DAK";
$resn_exch{"1HP7 DAK"}="HP71 DAK"; $resn_exch{"1HO3 DAK"}="HO31 DAK"; $resn_exch{"1HN2 DAK"}="HN21 DAK";
$resn_exch{"1HP5 DAK"}="HP51 DAK"; $resn_exch{"2HP5 DAK"}="HP52 DAK"; $resn_exch{"1HP4 DAK"}="HP41 DAK";
$resn_exch{"2HP4 DAK"}="HP42 DAK"; $resn_exch{"1HN1 DAK"}="HN11 DAK"; $resn_exch{"1HP2 DAK"}="HP21 DAK";
$resn_exch{"2HP2 DAK"}="HP22 DAK"; $resn_exch{"1HP1 DAK"}="HP11 DAK"; $resn_exch{"2HP1 DAK"}="HP12 DAK";
$resn_exch{"1HD6 DAK"}="HD61 DAK"; $resn_exch{"1HD5 DAK"}="HD51 DAK"; $resn_exch{"1HD9 DAK"}="HD91 DAK";
$resn_exch{"1HD8 DAK"}="HD81 DAK"; $resn_exch{"1HDA DAK"}="HDA1 DAK"; $resn_exch{"2HDA DAK"}="HDA2 DAK";
$resn_exch{"3HDA DAK"}="HDA3 DAK"; $resn_exch{"1HDB DAK"}="HDB1 DAK"; $resn_exch{"2HDB DAK"}="HDB2 DAK";
$resn_exch{"3HDB DAK"}="HDB3 DAK"; $resn_exch{"1HD3 DAK"}="HD31 DAK"; $resn_exch{"1HD2 DAK"}="HD21 DAK";
$resn_exch{" HN2 DAL"}=" H2  DAL"; $resn_exch{"1HB  DAL"}=" HB1 DAL"; $resn_exch{"2HB  DAL"}=" HB2 DAL";
$resn_exch{"3HB  DAL"}=" HB3 DAL"; $resn_exch{" HN  DAM"}=" H   DAM"; $resn_exch{"1HM  DAM"}=" HM1 DAM";
$resn_exch{"2HM  DAM"}=" HM2 DAM"; $resn_exch{"3HM  DAM"}=" HM3 DAM"; $resn_exch{"1HB  DAM"}=" HB1 DAM";
$resn_exch{"2HB  DAM"}=" HB2 DAM"; $resn_exch{"1H9  DAN"}=" H91 DAN"; $resn_exch{"2H9  DAN"}=" H92 DAN";
$resn_exch{"1H11 DAN"}="H111 DAN"; $resn_exch{"2H11 DAN"}="H112 DAN"; $resn_exch{"3H11 DAN"}="H113 DAN";
$resn_exch{"1HOB DAN"}="HOB1 DAN"; $resn_exch{"1H2  DAO"}=" H21 DAO"; $resn_exch{"2H2  DAO"}=" H22 DAO";
$resn_exch{"1H3  DAO"}=" H31 DAO"; $resn_exch{"2H3  DAO"}=" H32 DAO"; $resn_exch{"1H4  DAO"}=" H41 DAO";
$resn_exch{"2H4  DAO"}=" H42 DAO"; $resn_exch{"1H5  DAO"}=" H51 DAO"; $resn_exch{"2H5  DAO"}=" H52 DAO";
$resn_exch{"1H6  DAO"}=" H61 DAO"; $resn_exch{"2H6  DAO"}=" H62 DAO"; $resn_exch{"1H7  DAO"}=" H71 DAO";
$resn_exch{"2H7  DAO"}=" H72 DAO"; $resn_exch{"1H8  DAO"}=" H81 DAO"; $resn_exch{"2H8  DAO"}=" H82 DAO";
$resn_exch{"1H9  DAO"}=" H91 DAO"; $resn_exch{"2H9  DAO"}=" H92 DAO"; $resn_exch{"1H10 DAO"}="H101 DAO";
$resn_exch{"2H10 DAO"}="H102 DAO"; $resn_exch{"1H11 DAO"}="H111 DAO"; $resn_exch{"2H11 DAO"}="H112 DAO";
$resn_exch{"1H12 DAO"}="H121 DAO"; $resn_exch{"2H12 DAO"}="H122 DAO"; $resn_exch{"3H12 DAO"}="H123 DAO";
$resn_exch{"1HN3 DAP"}="HN31 DAP"; $resn_exch{"2HN3 DAP"}="HN32 DAP"; $resn_exch{" H2* DAP"}=" H2' DAP";
$resn_exch{" H3* DAP"}=" H3' DAP"; $resn_exch{" H5* DAP"}=" H5' DAP"; $resn_exch{" H6* DAP"}=" H6' DAP";
$resn_exch{"1HN5 DAP"}="HN51 DAP"; $resn_exch{"2HN5 DAP"}="HN52 DAP"; $resn_exch{"2HOB DAQ"}="HOB2 DAQ";
$resn_exch{"2HOA DAQ"}="HOA2 DAQ"; $resn_exch{"1HNA DAQ"}="HNA1 DAQ"; $resn_exch{"1HA2 DAQ"}="HA21 DAQ";
$resn_exch{"2HA2 DAQ"}="HA22 DAQ"; $resn_exch{"1HA1 DAQ"}="HA11 DAQ"; $resn_exch{"2HA1 DAQ"}="HA12 DAQ";
$resn_exch{"1HA3 DAQ"}="HA31 DAQ"; $resn_exch{"2HA3 DAQ"}="HA32 DAQ"; $resn_exch{" HN2 DAR"}=" H2  DAR";
$resn_exch{"1HB  DAR"}=" HB2 DAR"; $resn_exch{"2HB  DAR"}=" HB3 DAR"; $resn_exch{"1HG  DAR"}=" HG2 DAR";
$resn_exch{"2HG  DAR"}=" HG3 DAR"; $resn_exch{"1HD  DAR"}=" HD2 DAR"; $resn_exch{"2HD  DAR"}=" HD3 DAR";
$resn_exch{" HNE DAR"}=" HE  DAR"; $resn_exch{"1HH1 DAR"}="HH11 DAR"; $resn_exch{"2HH1 DAR"}="HH12 DAR";
$resn_exch{"1HH2 DAR"}="HH21 DAR"; $resn_exch{"2HH2 DAR"}="HH22 DAR"; $resn_exch{" HN2 DAS"}=" H2  DAS";
$resn_exch{"1HB  DAS"}=" HB2 DAS"; $resn_exch{"2HB  DAS"}=" HB3 DAS"; $resn_exch{"2HOB DAT"}="HOB2 DAT";
$resn_exch{"3HOB DAT"}="HOB3 DAT"; $resn_exch{"2HOA DAT"}="HOA2 DAT"; $resn_exch{"1H5* DAT"}="H5'1 DAT";
$resn_exch{"2H5* DAT"}="H5'2 DAT"; $resn_exch{" H4* DAT"}=" H4' DAT"; $resn_exch{" H3* DAT"}=" H3' DAT";
$resn_exch{"*HO3 DAT"}="HO3' DAT"; $resn_exch{"1H2* DAT"}="H2'1 DAT"; $resn_exch{"2H2* DAT"}="H2'2 DAT";
$resn_exch{" H1* DAT"}=" H1' DAT"; $resn_exch{"1HN6 DAT"}="HN61 DAT"; $resn_exch{"2HN6 DAT"}="HN62 DAT";
$resn_exch{"1HC6 DAU"}="HC61 DAU"; $resn_exch{"2HC6 DAU"}="HC62 DAU"; $resn_exch{"OH4P DAU"}="H4PO DAU";
$resn_exch{"OH2P DAU"}="H2PO DAU"; $resn_exch{"1HC5 DAU"}="HC51 DAU"; $resn_exch{"2HC5 DAU"}="HC52 DAU";
$resn_exch{"CH4, DAU"}="H4,C DAU"; $resn_exch{"CH3, DAU"}="H3,C DAU"; $resn_exch{"OH3, DAU"}="H3,O DAU";
$resn_exch{"1HC2 DAU"}="HC21 DAU"; $resn_exch{"2HC2 DAU"}="HC22 DAU"; $resn_exch{"CH1, DAU"}="H1,C DAU";
$resn_exch{"CH61 DAU"}="H61C DAU"; $resn_exch{"1H5A DAU"}="H5A1 DAU"; $resn_exch{"2H5A DAU"}="H5A2 DAU";
$resn_exch{"3H5A DAU"}="H5A3 DAU"; $resn_exch{"NH31 DAU"}="H31N DAU"; $resn_exch{"1HCB DAV"}="HCB1 DAV";
$resn_exch{"2HCB DAV"}="HCB2 DAV"; $resn_exch{"1HCG DAV"}="HCG1 DAV"; $resn_exch{"2HCG DAV"}="HCG2 DAV";
$resn_exch{"1HCD DAV"}="HCD1 DAV"; $resn_exch{"2HCD DAV"}="HCD2 DAV"; $resn_exch{"1HCA DAV"}="HCA1 DAV";
$resn_exch{"2HCA DAV"}="HCA2 DAV"; $resn_exch{"1H9  DAX"}=" H91 DAX"; $resn_exch{"2H9  DAX"}=" H92 DAX";
$resn_exch{"1HD2 DAX"}="HD21 DAX"; $resn_exch{"2HD2 DAX"}="HD22 DAX"; $resn_exch{"1HD3 DAX"}="HD31 DAX";
$resn_exch{"2HD3 DAX"}="HD32 DAX"; $resn_exch{"1HD7 DAX"}="HD71 DAX"; $resn_exch{"2HD7 DAX"}="HD72 DAX";
$resn_exch{"3HD7 DAX"}="HD73 DAX"; $resn_exch{"1HD8 DAX"}="HD81 DAX"; $resn_exch{"2HD8 DAX"}="HD82 DAX";
$resn_exch{"3HD8 DAX"}="HD83 DAX"; $resn_exch{"1HAF DB0"}="HAF1 DB0"; $resn_exch{"2HAF DB0"}="HAF2 DB0";
$resn_exch{"1HAA DB0"}="HAA1 DB0"; $resn_exch{"2HAA DB0"}="HAA2 DB0"; $resn_exch{"3HAA DB0"}="HAA3 DB0";
$resn_exch{"1HAC DB0"}="HAC1 DB0"; $resn_exch{"2HAC DB0"}="HAC2 DB0"; $resn_exch{"1HN1 DB1"}="HN11 DB1";
$resn_exch{"2HN1 DB1"}="HN12 DB1"; $resn_exch{"1H14 DB9"}="H141 DB9"; $resn_exch{"2H14 DB9"}="H142 DB9";
$resn_exch{"1H26 DB9"}="H261 DB9"; $resn_exch{"2H26 DB9"}="H262 DB9"; $resn_exch{"1H2  DBA"}=" H21 DBA";
$resn_exch{"2H2  DBA"}=" H22 DBA"; $resn_exch{"1HM2 DBA"}="HM21 DBA"; $resn_exch{"2HM2 DBA"}="HM22 DBA";
$resn_exch{"3HM2 DBA"}="HM23 DBA"; $resn_exch{"1HM6 DBA"}="HM61 DBA"; $resn_exch{"2HM6 DBA"}="HM62 DBA";
$resn_exch{"3HM6 DBA"}="HM63 DBA"; $resn_exch{"1H5  DBD"}=" H51 DBD"; $resn_exch{"2H5  DBD"}=" H52 DBD";
$resn_exch{"1H6  DBD"}=" H61 DBD"; $resn_exch{"2H6  DBD"}=" H62 DBD"; $resn_exch{"1H19 DBD"}="H191 DBD";
$resn_exch{"2H19 DBD"}="H192 DBD"; $resn_exch{"1H15 DBG"}="H151 DBG"; $resn_exch{"2H15 DBG"}="H152 DBG";
$resn_exch{"3HNG DBG"}="HNG3 DBG"; $resn_exch{"1HOG DBG"}="HOG1 DBG"; $resn_exch{"3HNB DBG"}="HNB3 DBG";
$resn_exch{"1HOB DBG"}="HOB1 DBG"; $resn_exch{"1HOA DBG"}="HOA1 DBG"; $resn_exch{"1H5* DBG"}="H5'1 DBG";
$resn_exch{"2H5* DBG"}="H5'2 DBG"; $resn_exch{" H4* DBG"}=" H4' DBG"; $resn_exch{" H2* DBG"}=" H2' DBG";
$resn_exch{"HO2* DBG"}="HO2' DBG"; $resn_exch{" H3* DBG"}=" H3' DBG"; $resn_exch{"HO3* DBG"}="HO3' DBG";
$resn_exch{" H1* DBG"}=" H1' DBG"; $resn_exch{"1HN2 DBG"}="HN21 DBG"; $resn_exch{"2HN2 DBG"}="HN22 DBG";
$resn_exch{"1H5' DBI"}="H5'1 DBI"; $resn_exch{"2H5' DBI"}="H5'2 DBI"; $resn_exch{"1H11 DBI"}="H111 DBI";
$resn_exch{"2H11 DBI"}="H112 DBI"; $resn_exch{"3H11 DBI"}="H113 DBI"; $resn_exch{"1H10 DBI"}="H101 DBI";
$resn_exch{"2H10 DBI"}="H102 DBI"; $resn_exch{"3H10 DBI"}="H103 DBI"; $resn_exch{"1HO2 DBI"}="HO21 DBI";
$resn_exch{"1HC  DBL"}=" HC1 DBL"; $resn_exch{"2HC  DBL"}=" HC2 DBL"; $resn_exch{"3HC  DBL"}=" HC3 DBL";
$resn_exch{"2HN  DBL"}=" HN2 DBL"; $resn_exch{"1HC6 DBL"}="HC61 DBL"; $resn_exch{"2HC6 DBL"}="HC62 DBL";
$resn_exch{"7HC  DBL"}=" HC7 DBL"; $resn_exch{"1HA  DBL"}=" HA1 DBL"; $resn_exch{"2HB  DBL"}=" HB2 DBL";
$resn_exch{"3HD  DBL"}=" HD3 DBL"; $resn_exch{"1HB  DBL"}=" HB1 DBL"; $resn_exch{"2HD  DBL"}=" HD2 DBL";
$resn_exch{"3HE  DBL"}=" HE3 DBL"; $resn_exch{"1HN0 DBL"}="HN01 DBL"; $resn_exch{"1HC1 DBL"}="HC11 DBL";
$resn_exch{"1HC4 DBL"}="HC41 DBL"; $resn_exch{"1HO5 DBL"}="HO51 DBL"; $resn_exch{"1HC7 DBL"}="HC71 DBL";
$resn_exch{"1HC8 DBL"}="HC81 DBL"; $resn_exch{"2HC0 DBL"}="HC02 DBL"; $resn_exch{"2HN4 DBL"}="HN42 DBL";
$resn_exch{"2HC5 DBL"}="HC52 DBL"; $resn_exch{"1H28 DBL"}="H281 DBL"; $resn_exch{"2H28 DBL"}="H282 DBL";
$resn_exch{"1H31 DBL"}="H311 DBL"; $resn_exch{"2H31 DBL"}="H312 DBL"; $resn_exch{"3HN2 DBL"}="HN23 DBL";
$resn_exch{"3HC3 DBL"}="HC33 DBL"; $resn_exch{"3HC7 DBL"}="HC73 DBL"; $resn_exch{"3HC8 DBL"}="HC83 DBL";
$resn_exch{"4HO2 DBL"}="HO24 DBL"; $resn_exch{"4HN3 DBL"}="HN34 DBL"; $resn_exch{"4HC4 DBL"}="HC44 DBL";
$resn_exch{"4HC8 DBL"}="HC84 DBL"; $resn_exch{"4HC9 DBL"}="HC94 DBL"; $resn_exch{"5HC1 DBL"}="HC15 DBL";
$resn_exch{"5HO3 DBL"}="HO35 DBL"; $resn_exch{"5HN4 DBL"}="HN45 DBL"; $resn_exch{"5HC5 DBL"}="HC55 DBL";
$resn_exch{"5HC8 DBL"}="HC85 DBL"; $resn_exch{"6HC1 DBL"}="HC16 DBL"; $resn_exch{"6HC2 DBL"}="HC26 DBL";
$resn_exch{"6HC3 DBL"}="HC36 DBL"; $resn_exch{"6HN8 DBL"}="HN86 DBL"; $resn_exch{"6HC9 DBL"}="HC96 DBL";
$resn_exch{"7HC4 DBL"}="HC47 DBL"; $resn_exch{"7HC6 DBL"}="HC67 DBL"; $resn_exch{"7HO8 DBL"}="HO87 DBL";
$resn_exch{"7HO9 DBL"}="HO97 DBL"; $resn_exch{"8HO0 DBL"}="HO08 DBL"; $resn_exch{"1HC' DBL"}="HC'1 DBL";
$resn_exch{"1H2' DBL"}="H2'1 DBL"; $resn_exch{"2H2' DBL"}="H2'2 DBL"; $resn_exch{"5HC' DBL"}="HC'5 DBL";
$resn_exch{"1H7' DBL"}="H7'1 DBL"; $resn_exch{"2H7' DBL"}="H7'2 DBL"; $resn_exch{"3H7' DBL"}="H7'3 DBL";
$resn_exch{"1H8' DBL"}="H8'1 DBL"; $resn_exch{"2H8' DBL"}="H8'2 DBL"; $resn_exch{"OH9' DBL"}="H9'O DBL";
$resn_exch{"1HO\" DBL"}="HO'1 DBL"; $resn_exch{"1H2\" DBL"}="H2A1 DBL"; $resn_exch{"2H2\" DBL"}="H2A2 DBL";
$resn_exch{"3H2\" DBL"}="H2'3 DBL"; $resn_exch{"1H6' DBM"}="H6'1 DBM"; $resn_exch{"2H6' DBM"}="H6'2 DBM";
$resn_exch{"3H6' DBM"}="H6'3 DBM"; $resn_exch{"1H7  DBM"}=" H71 DBM"; $resn_exch{"2H7  DBM"}=" H72 DBM";
$resn_exch{"3H7  DBM"}=" H73 DBM"; $resn_exch{"1H27 DBN"}="H271 DBN"; $resn_exch{"2H27 DBN"}="H272 DBN";
$resn_exch{"1H28 DBN"}="H281 DBN"; $resn_exch{"2H28 DBN"}="H282 DBN"; $resn_exch{"1H29 DBN"}="H291 DBN";
$resn_exch{"2H29 DBN"}="H292 DBN"; $resn_exch{"1H30 DBN"}="H301 DBN"; $resn_exch{"2H30 DBN"}="H302 DBN";
$resn_exch{"1H9  DBO"}=" H91 DBO"; $resn_exch{"2H9  DBO"}=" H92 DBO"; $resn_exch{"1H14 DBO"}="H141 DBO";
$resn_exch{"2H14 DBO"}="H142 DBO"; $resn_exch{"1H27 DBO"}="H271 DBO"; $resn_exch{"2H27 DBO"}="H272 DBO";
$resn_exch{"3H27 DBO"}="H273 DBO"; $resn_exch{"1H31 DBO"}="H311 DBO"; $resn_exch{"2H31 DBO"}="H312 DBO";
$resn_exch{"1HN  DBP"}=" HN1 DBP"; $resn_exch{"1HN2 DBP"}="HN21 DBP"; $resn_exch{"1HB1 DBP"}="HB11 DBP";
$resn_exch{"1HB2 DBP"}="HB21 DBP"; $resn_exch{"1HD1 DBP"}="HD11 DBP"; $resn_exch{"1HC1 DBP"}="HC11 DBP";
$resn_exch{"1HC2 DBP"}="HC21 DBP"; $resn_exch{"1HCZ DBP"}="HCZ1 DBP"; $resn_exch{"1HE2 DBP"}="HE21 DBP";
$resn_exch{"2HE2 DBP"}="HE22 DBP"; $resn_exch{"1H6  DBQ"}=" H61 DBQ"; $resn_exch{"2H6  DBQ"}=" H62 DBQ";
$resn_exch{"1H7  DBQ"}=" H71 DBQ"; $resn_exch{"2H7  DBQ"}=" H72 DBQ"; $resn_exch{"1HN5 DBQ"}="HN51 DBQ";
$resn_exch{"2HN5 DBQ"}="HN52 DBQ"; $resn_exch{"1H1  DBR"}="H11A DBR"; $resn_exch{"2H1  DBR"}="H12A DBR";
$resn_exch{"1H3  DBR"}="H31A DBR"; $resn_exch{"2H3  DBR"}=" H32 DBR"; $resn_exch{"1H4  DBR"}=" H41 DBR";
$resn_exch{"2H4  DBR"}=" H42 DBR"; $resn_exch{"1H6  DBR"}=" H61 DBR"; $resn_exch{"2H6  DBR"}=" H62 DBR";
$resn_exch{"1H7  DBR"}=" H71 DBR"; $resn_exch{"2H7  DBR"}=" H72 DBR"; $resn_exch{"1H10 DBR"}="H101 DBR";
$resn_exch{"2H10 DBR"}="H102 DBR"; $resn_exch{"1H13 DBR"}="H131 DBR"; $resn_exch{"2H13 DBR"}="H132 DBR";
$resn_exch{"1H14 DBR"}="H141 DBR"; $resn_exch{"2H14 DBR"}="H142 DBR"; $resn_exch{"1H15 DBR"}="H151 DBR";
$resn_exch{"2H15 DBR"}="H152 DBR"; $resn_exch{"1H16 DBR"}="H161 DBR"; $resn_exch{"2H16 DBR"}="H162 DBR";
$resn_exch{"1H28 DBR"}="H281 DBR"; $resn_exch{"2H28 DBR"}="H282 DBR"; $resn_exch{"3H28 DBR"}="H283 DBR";
$resn_exch{"1H29 DBR"}="H291 DBR"; $resn_exch{"2H29 DBR"}="H292 DBR"; $resn_exch{"3H29 DBR"}="H293 DBR";
$resn_exch{"1H28 DBS"}="H281 DBS"; $resn_exch{"2H28 DBS"}="H282 DBS"; $resn_exch{"1H8  DBT"}=" H81 DBT";
$resn_exch{"2H8  DBT"}=" H82 DBT"; $resn_exch{"1H9  DBT"}=" H91 DBT"; $resn_exch{"2H9  DBT"}=" H92 DBT";
$resn_exch{"1H10 DBT"}="H101 DBT"; $resn_exch{"2H10 DBT"}="H102 DBT"; $resn_exch{"1H11 DBT"}="H111 DBT";
$resn_exch{"2H11 DBT"}="H112 DBT"; $resn_exch{"1H12 DBT"}="H121 DBT"; $resn_exch{"2H12 DBT"}="H122 DBT";
$resn_exch{"1H13 DBT"}="H131 DBT"; $resn_exch{"2H13 DBT"}="H132 DBT"; $resn_exch{"1H14 DBT"}="H141 DBT";
$resn_exch{"2H14 DBT"}="H142 DBT"; $resn_exch{"3H14 DBT"}="H143 DBT"; $resn_exch{"1HB  DBU"}=" HB1 DBU";
$resn_exch{"1HG  DBU"}=" HG1 DBU"; $resn_exch{"2HG  DBU"}=" HG2 DBU"; $resn_exch{"3HG  DBU"}=" HG3 DBU";
$resn_exch{"1HMA DBV"}="HMA1 DBV"; $resn_exch{"2HMA DBV"}="HMA2 DBV"; $resn_exch{"3HMA DBV"}="HMA3 DBV";
$resn_exch{"1HBA DBV"}="HBA1 DBV"; $resn_exch{"2HBA DBV"}="HBA2 DBV"; $resn_exch{"1HHA DBV"}="HHA1 DBV";
$resn_exch{"1HAA DBV"}="HAA1 DBV"; $resn_exch{"1HHB DBV"}="HHB1 DBV"; $resn_exch{"1HMB DBV"}="HMB1 DBV";
$resn_exch{"2HMB DBV"}="HMB2 DBV"; $resn_exch{"3HMB DBV"}="HMB3 DBV"; $resn_exch{"1HAB DBV"}="HAB1 DBV";
$resn_exch{"2HAB DBV"}="HAB2 DBV"; $resn_exch{"1HBB DBV"}="HBB1 DBV"; $resn_exch{"2HBB DBV"}="HBB2 DBV";
$resn_exch{"1H2B DBV"}="H2B1 DBV"; $resn_exch{"1HHC DBV"}="HHC1 DBV"; $resn_exch{"2HHC DBV"}="HHC2 DBV";
$resn_exch{"1HMC DBV"}="HMC1 DBV"; $resn_exch{"2HMC DBV"}="HMC2 DBV"; $resn_exch{"3HMC DBV"}="HMC3 DBV";
$resn_exch{"1HAC DBV"}="HAC1 DBV"; $resn_exch{"2HAC DBV"}="HAC2 DBV"; $resn_exch{"1HBC DBV"}="HBC1 DBV";
$resn_exch{"2HBC DBV"}="HBC2 DBV"; $resn_exch{"1H2C DBV"}="H2C1 DBV"; $resn_exch{"1H1D DBV"}="H1D1 DBV";
$resn_exch{"1HMD DBV"}="HMD1 DBV"; $resn_exch{"2HMD DBV"}="HMD2 DBV"; $resn_exch{"3HMD DBV"}="HMD3 DBV";
$resn_exch{"1HAD DBV"}="HAD1 DBV"; $resn_exch{"1HBD DBV"}="HBD1 DBV"; $resn_exch{"2HBD DBV"}="HBD2 DBV";
$resn_exch{"1HN  DBY"}=" H   DBY"; $resn_exch{"2HN  DBY"}=" H2  DBY"; $resn_exch{"1HB  DBY"}=" HB2 DBY";
$resn_exch{"2HB  DBY"}=" HB3 DBY"; $resn_exch{"1HN  DBZ"}=" HN1 DBZ"; $resn_exch{"2HN  DBZ"}=" HN2 DBZ";
$resn_exch{"1HB  DBZ"}=" HB1 DBZ"; $resn_exch{"2HB  DBZ"}=" HB2 DBZ"; $resn_exch{"3HOP  DC"}="HOP3  DC";
$resn_exch{"2HOP  DC"}="HOP2  DC"; $resn_exch{"1H5*  DC"}=" H5'  DC"; $resn_exch{"2H5*  DC"}="H5''  DC";
$resn_exch{" H4*  DC"}=" H4'  DC"; $resn_exch{" H3*  DC"}=" H3'  DC"; $resn_exch{"H3T   DC"}="HO3'  DC";
$resn_exch{"1H2*  DC"}=" H2'  DC"; $resn_exch{"2H2*  DC"}="H2''  DC"; $resn_exch{" H1*  DC"}=" H1'  DC";
$resn_exch{"1H4   DC"}=" H41  DC"; $resn_exch{"2H4   DC"}=" H42  DC"; $resn_exch{"1H10 DC8"}="H101 DC8";
$resn_exch{"2H10 DC8"}="H102 DC8"; $resn_exch{"1H14 DC8"}="H141 DC8"; $resn_exch{"2H14 DC8"}="H142 DC8";
$resn_exch{" AH2 DCA"}=" H2A DCA"; $resn_exch{"AH61 DCA"}="H61A DCA"; $resn_exch{"AH62 DCA"}="H62A DCA";
$resn_exch{" AH8 DCA"}=" H8A DCA"; $resn_exch{"AH1* DCA"}=" H1B DCA"; $resn_exch{"AH2* DCA"}=" H2B DCA";
$resn_exch{"AHO2 DCA"}="HO2A DCA"; $resn_exch{"AH3* DCA"}=" H3B DCA"; $resn_exch{"8HOA DCA"}="HOA8 DCA";
$resn_exch{"9HOA DCA"}="HOA9 DCA"; $resn_exch{"AH4* DCA"}=" H4B DCA"; $resn_exch{"AH51 DCA"}="H51A DCA";
$resn_exch{"AH52 DCA"}="H52A DCA"; $resn_exch{"2HOA DCA"}="HOA2 DCA"; $resn_exch{"5HOA DCA"}="HOA5 DCA";
$resn_exch{"1H12 DCA"}="H121 DCA"; $resn_exch{"2H12 DCA"}="H122 DCA"; $resn_exch{"1H13 DCA"}="H131 DCA";
$resn_exch{"2H13 DCA"}="H132 DCA"; $resn_exch{"3H13 DCA"}="H133 DCA"; $resn_exch{"1H14 DCA"}="H141 DCA";
$resn_exch{"2H14 DCA"}="H142 DCA"; $resn_exch{"3H14 DCA"}="H143 DCA"; $resn_exch{"1H7  DCA"}=" H71 DCA";
$resn_exch{"2H7  DCA"}=" H72 DCA"; $resn_exch{"1H6  DCA"}=" H61 DCA"; $resn_exch{"2H6  DCA"}=" H62 DCA";
$resn_exch{"1H3  DCA"}=" H31 DCA"; $resn_exch{"2H3  DCA"}=" H32 DCA"; $resn_exch{"1H2  DCA"}=" H21 DCA";
$resn_exch{"2H2  DCA"}=" H22 DCA"; $resn_exch{"3H2  DCA"}=" H23 DCA"; $resn_exch{"1H6A DCB"}="H6A1 DCB";
$resn_exch{"2HCA DCB"}="HCA2 DCB"; $resn_exch{"1H6B DCB"}="H6B1 DCB"; $resn_exch{"2H6B DCB"}="H6B2 DCB";
$resn_exch{"1H10 DCC"}="H101 DCC"; $resn_exch{"2H10 DCC"}="H102 DCC"; $resn_exch{"1H11 DCC"}="H111 DCC";
$resn_exch{"2H11 DCC"}="H112 DCC"; $resn_exch{"1H12 DCC"}="H121 DCC"; $resn_exch{"2H12 DCC"}="H122 DCC";
$resn_exch{"3H12 DCC"}="H123 DCC"; $resn_exch{"1HN6 DCC"}="HN61 DCC"; $resn_exch{"2HN6 DCC"}="HN62 DCC";
$resn_exch{"1H5' DCC"}="H5'1 DCC"; $resn_exch{"2H5' DCC"}="H5'2 DCC"; $resn_exch{"1HPA DCC"}="HPA1 DCC";
$resn_exch{"2HPA DCC"}="HPA2 DCC"; $resn_exch{"1HP7 DCC"}="HP71 DCC"; $resn_exch{"2HP7 DCC"}="HP72 DCC";
$resn_exch{"3HP7 DCC"}="HP73 DCC"; $resn_exch{"1HP9 DCC"}="HP91 DCC"; $resn_exch{"2HP9 DCC"}="HP92 DCC";
$resn_exch{"3HP9 DCC"}="HP93 DCC"; $resn_exch{"1HP5 DCC"}="HP51 DCC"; $resn_exch{"2HP5 DCC"}="HP52 DCC";
$resn_exch{"1HP4 DCC"}="HP41 DCC"; $resn_exch{"2HP4 DCC"}="HP42 DCC"; $resn_exch{"1HP2 DCC"}="HP21 DCC";
$resn_exch{"2HP2 DCC"}="HP22 DCC"; $resn_exch{"1HP1 DCC"}="HP11 DCC"; $resn_exch{"2HP1 DCC"}="HP12 DCC";
$resn_exch{"1HA2 DCC"}="HA21 DCC"; $resn_exch{"2HA2 DCC"}="HA22 DCC"; $resn_exch{"1HA3 DCC"}="HA31 DCC";
$resn_exch{"2HA3 DCC"}="HA32 DCC"; $resn_exch{"1HA4 DCC"}="HA41 DCC"; $resn_exch{"2HA4 DCC"}="HA42 DCC";
$resn_exch{"1HA5 DCC"}="HA51 DCC"; $resn_exch{"2HA5 DCC"}="HA52 DCC"; $resn_exch{"1HA6 DCC"}="HA61 DCC";
$resn_exch{"2HA6 DCC"}="HA62 DCC"; $resn_exch{"1HA7 DCC"}="HA71 DCC"; $resn_exch{"2HA7 DCC"}="HA72 DCC";
$resn_exch{"1HA8 DCC"}="HA81 DCC"; $resn_exch{"2HA8 DCC"}="HA82 DCC"; $resn_exch{"1HA9 DCC"}="HA91 DCC";
$resn_exch{"2HA9 DCC"}="HA92 DCC"; $resn_exch{"1H5C DCD"}="H5C1 DCD"; $resn_exch{"2H5C DCD"}="H5C2 DCD";
$resn_exch{"3H5C DCD"}="H5C3 DCD"; $resn_exch{"1H3C DCD"}="H3C1 DCD"; $resn_exch{"2H3C DCD"}="H3C2 DCD";
$resn_exch{"1H2C DCD"}="H2C1 DCD"; $resn_exch{"2H2C DCD"}="H2C2 DCD"; $resn_exch{"1H4C DCD"}="H4C1 DCD";
$resn_exch{"2H4C DCD"}="H4C2 DCD"; $resn_exch{"3H4C DCD"}="H4C3 DCD"; $resn_exch{"1H1  DCE"}=" H11 DCE";
$resn_exch{"2H1  DCE"}=" H12 DCE"; $resn_exch{"1H2  DCE"}=" H21 DCE"; $resn_exch{"2H2  DCE"}=" H22 DCE";
$resn_exch{"1H7  DCF"}=" H71 DCF"; $resn_exch{"2H7  DCF"}=" H72 DCF"; $resn_exch{"1H2S DCF"}="H2S1 DCF";
$resn_exch{"2H2S DCF"}="H2S2 DCF"; $resn_exch{"1H5S DCF"}="H5S1 DCF"; $resn_exch{"2H5S DCF"}="H5S2 DCF";
$resn_exch{"2HOP DCG"}="HOP2 DCG"; $resn_exch{"3HOP DCG"}="HOP3 DCG"; $resn_exch{"1H5* DCG"}=" H5' DCG";
$resn_exch{"2H5* DCG"}="H5'' DCG"; $resn_exch{" H4* DCG"}=" H4' DCG"; $resn_exch{" H3* DCG"}=" H3' DCG";
$resn_exch{"*HO3 DCG"}="HO3' DCG"; $resn_exch{"1H2* DCG"}=" H2' DCG"; $resn_exch{"2H2* DCG"}="H2'' DCG";
$resn_exch{" H1* DCG"}=" H1' DCG"; $resn_exch{"1HN2 DCG"}="HN21 DCG"; $resn_exch{"2HN2 DCG"}="HN22 DCG";
$resn_exch{"1HZ1 DCH"}="HZ11 DCH"; $resn_exch{"2HZ1 DCH"}="HZ12 DCH"; $resn_exch{"1HZ2 DCH"}="HZ21 DCH";
$resn_exch{"2HZ2 DCH"}="HZ22 DCH"; $resn_exch{"1HA1 DCH"}="HA11 DCH"; $resn_exch{"2HA1 DCH"}="HA12 DCH";
$resn_exch{"1HA5 DCH"}="HA51 DCH"; $resn_exch{"2HA5 DCH"}="HA52 DCH"; $resn_exch{"1HA6 DCH"}="HA61 DCH";
$resn_exch{"2HA6 DCH"}="HA62 DCH"; $resn_exch{"3HA6 DCH"}="HA63 DCH"; $resn_exch{"1H32 DCH"}="H321 DCH";
$resn_exch{"2H32 DCH"}="H322 DCH"; $resn_exch{"1H34 DCH"}="H341 DCH"; $resn_exch{"2H34 DCH"}="H342 DCH";
$resn_exch{"1H35 DCH"}="H351 DCH"; $resn_exch{"2H35 DCH"}="H352 DCH"; $resn_exch{"1HN  DCI"}=" HN1 DCI";
$resn_exch{"2HN  DCI"}=" HN2 DCI"; $resn_exch{"1HA  DCI"}=" HA1 DCI"; $resn_exch{"2HA  DCI"}=" HA2 DCI";
$resn_exch{"1HG1 DCI"}="HG11 DCI"; $resn_exch{"2HG1 DCI"}="HG12 DCI"; $resn_exch{"1HG2 DCI"}="HG21 DCI";
$resn_exch{"2HG2 DCI"}="HG22 DCI"; $resn_exch{"3HG2 DCI"}="HG23 DCI"; $resn_exch{"1HD  DCI"}=" HD1 DCI";
$resn_exch{"2HD  DCI"}=" HD2 DCI"; $resn_exch{"3HD  DCI"}=" HD3 DCI"; $resn_exch{"1HN  DCL"}=" HN1 DCL";
$resn_exch{"2HN  DCL"}=" HN2 DCL"; $resn_exch{"1HC  DCL"}=" HC1 DCL"; $resn_exch{"2HC  DCL"}=" HC2 DCL";
$resn_exch{"1HB  DCL"}=" HB1 DCL"; $resn_exch{"2HB  DCL"}=" HB2 DCL"; $resn_exch{"1HD1 DCL"}="HD11 DCL";
$resn_exch{"2HD1 DCL"}="HD12 DCL"; $resn_exch{"3HD1 DCL"}="HD13 DCL"; $resn_exch{"1HD2 DCL"}="HD21 DCL";
$resn_exch{"2HD2 DCL"}="HD22 DCL"; $resn_exch{"3HD2 DCL"}="HD23 DCL"; $resn_exch{"1HN4 DCM"}="HN41 DCM";
$resn_exch{"2HN4 DCM"}="HN42 DCM"; $resn_exch{" H1* DCM"}=" H1' DCM"; $resn_exch{"1H2* DCM"}="H2'1 DCM";
$resn_exch{"2H2* DCM"}="H2'2 DCM"; $resn_exch{" H3* DCM"}=" H3' DCM"; $resn_exch{" H4* DCM"}=" H4' DCM";
$resn_exch{"*HO3 DCM"}="HO3' DCM"; $resn_exch{"1H5* DCM"}="H5'1 DCM"; $resn_exch{"2H5* DCM"}="H5'2 DCM";
$resn_exch{"2HOP DCM"}="HOP2 DCM"; $resn_exch{"3HOP DCM"}="HOP3 DCM"; $resn_exch{"1H3  DCO"}=" H31 DCO";
$resn_exch{"2H3  DCO"}=" H32 DCO"; $resn_exch{"1HN4 DCP"}="HN41 DCP"; $resn_exch{"2HN4 DCP"}="HN42 DCP";
$resn_exch{" H1* DCP"}=" H1' DCP"; $resn_exch{"1H2* DCP"}="H2'1 DCP"; $resn_exch{"2H2* DCP"}="H2'2 DCP";
$resn_exch{" H3* DCP"}=" H3' DCP"; $resn_exch{" H4* DCP"}=" H4' DCP"; $resn_exch{"*HO3 DCP"}="HO3' DCP";
$resn_exch{"1H5* DCP"}="H5'1 DCP"; $resn_exch{"2H5* DCP"}="H5'2 DCP"; $resn_exch{"2HOA DCP"}="HOA2 DCP";
$resn_exch{"2HOB DCP"}="HOB2 DCP"; $resn_exch{"2HOG DCP"}="HOG2 DCP"; $resn_exch{"3HOG DCP"}="HOG3 DCP";
$resn_exch{"1H2A DCS"}="H2A1 DCS"; $resn_exch{"2H2A DCS"}="H2A2 DCS"; $resn_exch{"3H2A DCS"}="H2A3 DCS";
$resn_exch{"1H4A DCS"}="H4A1 DCS"; $resn_exch{"2H4A DCS"}="H4A2 DCS"; $resn_exch{"1H5A DCS"}="H5A1 DCS";
$resn_exch{"2H5A DCS"}="H5A2 DCS"; $resn_exch{"2HOP DCS"}="HOP2 DCS"; $resn_exch{"3HOP DCS"}="HOP3 DCS";
$resn_exch{"1HB  DCS"}=" HB1 DCS"; $resn_exch{"2HB  DCS"}=" HB2 DCS"; $resn_exch{"1HN4 DCT"}="HN41 DCT";
$resn_exch{"2HN4 DCT"}="HN42 DCT"; $resn_exch{" H1* DCT"}=" H1' DCT"; $resn_exch{"1H2* DCT"}=" H2' DCT";
$resn_exch{"2H2* DCT"}="H2'' DCT"; $resn_exch{"1H3* DCT"}="H3'1 DCT"; $resn_exch{"2H3* DCT"}="H3'2 DCT";
$resn_exch{" H4* DCT"}=" H4' DCT"; $resn_exch{"1H5* DCT"}=" H5' DCT"; $resn_exch{"2H5* DCT"}="H5'' DCT";
$resn_exch{"2HOA DCT"}="HOA2 DCT"; $resn_exch{"2HOB DCT"}="HOB2 DCT"; $resn_exch{"2HOG DCT"}="HOG2 DCT";
$resn_exch{"3HOG DCT"}="HOG3 DCT"; $resn_exch{"1H2  DCU"}="H21A DCU"; $resn_exch{"2H2  DCU"}=" H22 DCU";
$resn_exch{"1H3  DCU"}="H31A DCU"; $resn_exch{"2H3  DCU"}="H32A DCU"; $resn_exch{"1H4  DCU"}=" H41 DCU";
$resn_exch{"2H4  DCU"}=" H42 DCU"; $resn_exch{"1H5  DCU"}=" H51 DCU"; $resn_exch{"2H5  DCU"}=" H52 DCU";
$resn_exch{"1H6  DCU"}=" H61 DCU"; $resn_exch{"2H6  DCU"}=" H62 DCU"; $resn_exch{"1H7  DCU"}=" H71 DCU";
$resn_exch{"2H7  DCU"}=" H72 DCU"; $resn_exch{"1H8  DCU"}=" H81 DCU"; $resn_exch{"2H8  DCU"}=" H82 DCU";
$resn_exch{"1H9  DCU"}=" H91 DCU"; $resn_exch{"2H9  DCU"}=" H92 DCU"; $resn_exch{"1H10 DCU"}="H101 DCU";
$resn_exch{"2H10 DCU"}="H102 DCU"; $resn_exch{"1H11 DCU"}="H111 DCU"; $resn_exch{"2H11 DCU"}="H112 DCU";
$resn_exch{"1H13 DCU"}="H131 DCU"; $resn_exch{"2H13 DCU"}="H132 DCU"; $resn_exch{"3H13 DCU"}="H133 DCU";
$resn_exch{"1H14 DCU"}="H141 DCU"; $resn_exch{"2H14 DCU"}="H142 DCU"; $resn_exch{"3H14 DCU"}="H143 DCU";
$resn_exch{"1H15 DCU"}="H151 DCU"; $resn_exch{"2H15 DCU"}="H152 DCU"; $resn_exch{"3H15 DCU"}="H153 DCU";
$resn_exch{"1H36 DCU"}="H361 DCU"; $resn_exch{"2H36 DCU"}="H362 DCU"; $resn_exch{"1H37 DCU"}="H371 DCU";
$resn_exch{"2H37 DCU"}="H372 DCU"; $resn_exch{"1H3  DCX"}=" H31 DCX"; $resn_exch{"2H3  DCX"}=" H32 DCX";
$resn_exch{"3H3  DCX"}=" H33 DCX"; $resn_exch{"1H6  DCX"}=" H61 DCX"; $resn_exch{"2H6  DCX"}=" H62 DCX";
$resn_exch{"1H14 DCX"}="H141 DCX"; $resn_exch{"2H14 DCX"}="H142 DCX"; $resn_exch{"3H14 DCX"}="H143 DCX";
$resn_exch{"1H13 DCX"}="H131 DCX"; $resn_exch{"2H13 DCX"}="H132 DCX"; $resn_exch{"1H12 DCX"}="H121 DCX";
$resn_exch{"2H12 DCX"}="H122 DCX"; $resn_exch{"1H11 DCX"}="H111 DCX"; $resn_exch{"2H11 DCX"}="H112 DCX";
$resn_exch{"1H10 DCX"}="H101 DCX"; $resn_exch{"2H10 DCX"}="H102 DCX"; $resn_exch{"1H9  DCX"}=" H91 DCX";
$resn_exch{"2H9  DCX"}=" H92 DCX"; $resn_exch{"1H8  DCX"}=" H81 DCX"; $resn_exch{"2H8  DCX"}=" H82 DCX";
$resn_exch{"1H7  DCX"}=" H71 DCX"; $resn_exch{"2H7  DCX"}=" H72 DCX"; $resn_exch{"1H4  DCX"}=" H41 DCX";
$resn_exch{"2H4  DCX"}=" H42 DCX"; $resn_exch{"1H2  DCX"}=" H21 DCX"; $resn_exch{"2H2  DCX"}=" H22 DCX";
$resn_exch{"3H2  DCX"}=" H23 DCX"; $resn_exch{" HN2 DCY"}=" H2  DCY"; $resn_exch{"1HB  DCY"}=" HB2 DCY";
$resn_exch{"2HB  DCY"}=" HB3 DCY"; $resn_exch{"1HN4 DCZ"}="HN41 DCZ"; $resn_exch{"2HN4 DCZ"}="HN42 DCZ";
$resn_exch{" H1* DCZ"}=" H1' DCZ"; $resn_exch{"1H2* DCZ"}="H2'1 DCZ"; $resn_exch{"2H2* DCZ"}="H2'2 DCZ";
$resn_exch{" H3* DCZ"}=" H3' DCZ"; $resn_exch{" H4* DCZ"}=" H4' DCZ"; $resn_exch{"*HO3 DCZ"}="HO3' DCZ";
$resn_exch{"1H5* DCZ"}="H5'1 DCZ"; $resn_exch{"2H5* DCZ"}="H5'2 DCZ"; $resn_exch{"1HO5 DCZ"}="HO51 DCZ";
$resn_exch{"1H2  DDA"}=" H21 DDA"; $resn_exch{"2H2  DDA"}=" H22 DDA"; $resn_exch{"1H6  DDA"}=" H61 DDA";
$resn_exch{"2H6  DDA"}=" H62 DDA"; $resn_exch{"3H6  DDA"}=" H63 DDA"; $resn_exch{"1H2  DDB"}=" H21 DDB";
$resn_exch{"2H2  DDB"}=" H22 DDB"; $resn_exch{"1H3M DDB"}="H3M1 DDB"; $resn_exch{"2H3M DDB"}="H3M2 DDB";
$resn_exch{"3H3M DDB"}="H3M3 DDB"; $resn_exch{"1H6  DDB"}=" H61 DDB"; $resn_exch{"2H6  DDB"}=" H62 DDB";
$resn_exch{"3H6  DDB"}=" H63 DDB"; $resn_exch{"1H8  DDC"}=" H81 DDC"; $resn_exch{"2H8  DDC"}=" H82 DDC";
$resn_exch{"1HB2 DDD"}="HB21 DDD"; $resn_exch{"2HB2 DDD"}="HB22 DDD"; $resn_exch{"1HG2 DDD"}="HG21 DDD";
$resn_exch{"2HG2 DDD"}="HG22 DDD"; $resn_exch{"1HA3 DDD"}="HA31 DDD"; $resn_exch{"2HA3 DDD"}="HA32 DDD";
$resn_exch{"1HD2 DDD"}="HD21 DDD"; $resn_exch{"2HD2 DDD"}="HD22 DDD"; $resn_exch{"1HB1 DDD"}="HB11 DDD";
$resn_exch{"2HB1 DDD"}="HB12 DDD"; $resn_exch{"1HA1 DDD"}="HA11 DDD"; $resn_exch{"2HA1 DDD"}="HA12 DDD";
$resn_exch{"1HN  DDE"}=" H   DDE"; $resn_exch{"2HN  DDE"}=" H2  DDE"; $resn_exch{"1HB  DDE"}=" HB2 DDE";
$resn_exch{"2HB  DDE"}=" HB3 DDE"; $resn_exch{"1HAD DDE"}="HAD1 DDE"; $resn_exch{"2HAD DDE"}="HAD2 DDE";
$resn_exch{"1HAB DDE"}="HAB1 DDE"; $resn_exch{"2HAB DDE"}="HAB2 DDE"; $resn_exch{"3HAB DDE"}="HAB3 DDE";
$resn_exch{"1HAC DDE"}="HAC1 DDE"; $resn_exch{"2HAC DDE"}="HAC2 DDE"; $resn_exch{"3HAC DDE"}="HAC3 DDE";
$resn_exch{"1HAA DDE"}="HAA1 DDE"; $resn_exch{"2HAA DDE"}="HAA2 DDE"; $resn_exch{"3HAA DDE"}="HAA3 DDE";
$resn_exch{"1HAU DDE"}="HAU3 DDE"; $resn_exch{"2HAU DDE"}="HAU2 DDE"; $resn_exch{"1HAT DDE"}="HAT2 DDE";
$resn_exch{"2HAT DDE"}="HAT3 DDE"; $resn_exch{"1HN2 DDF"}="HN21 DDF"; $resn_exch{"2HN2 DDF"}="HN22 DDF";
$resn_exch{"1H5  DDF"}=" H51 DDF"; $resn_exch{"2H5  DDF"}=" H52 DDF"; $resn_exch{"1H7  DDF"}=" H71 DDF";
$resn_exch{"2H7  DDF"}=" H72 DDF"; $resn_exch{"1H9  DDF"}=" H91 DDF"; $resn_exch{"2H9  DDF"}=" H92 DDF";
$resn_exch{"1H10 DDF"}="H101 DDF"; $resn_exch{"2H10 DDF"}="H102 DDF"; $resn_exch{"1HB  DDF"}=" HB1 DDF";
$resn_exch{"2HB  DDF"}=" HB2 DDF"; $resn_exch{"1HG  DDF"}=" HG1 DDF"; $resn_exch{"2HG  DDF"}=" HG2 DDF";
$resn_exch{"2HOE DDF"}="HOE2 DDF"; $resn_exch{"3HOP DDG"}="HOP3 DDG"; $resn_exch{"2HOP DDG"}="HOP2 DDG";
$resn_exch{"1H5* DDG"}=" H5' DDG"; $resn_exch{"2H5* DDG"}="H5'' DDG"; $resn_exch{" H4* DDG"}=" H4' DDG";
$resn_exch{"1H3* DDG"}="H3'1 DDG"; $resn_exch{"2H3* DDG"}="H3'2 DDG"; $resn_exch{"1H2* DDG"}=" H2' DDG";
$resn_exch{"2H2* DDG"}="H2'' DDG"; $resn_exch{" H1* DDG"}=" H1' DDG"; $resn_exch{"1H2  DDG"}=" H21 DDG";
$resn_exch{"2H2  DDG"}=" H22 DDG"; $resn_exch{"1HAA DDH"}="HAA1 DDH"; $resn_exch{"2HAA DDH"}="HAA2 DDH";
$resn_exch{"1HMA DDH"}="HMA1 DDH"; $resn_exch{"2HMA DDH"}="HMA2 DDH"; $resn_exch{"3HMA DDH"}="HMA3 DDH";
$resn_exch{"1HBA DDH"}="HBA1 DDH"; $resn_exch{"2HBA DDH"}="HBA2 DDH"; $resn_exch{"AHO2 DDH"}="HO2A DDH";
$resn_exch{"1HMB DDH"}="HMB1 DDH"; $resn_exch{"2HMB DDH"}="HMB2 DDH"; $resn_exch{"3HMB DDH"}="HMB3 DDH";
$resn_exch{"BHOC DDH"}="HOCB DDH"; $resn_exch{"1HBB DDH"}="HBB1 DDH"; $resn_exch{"2HBB DDH"}="HBB2 DDH";
$resn_exch{"1HMC DDH"}="HMC1 DDH"; $resn_exch{"2HMC DDH"}="HMC2 DDH"; $resn_exch{"3HMC DDH"}="HMC3 DDH";
$resn_exch{"1HBC DDH"}="HBC1 DDH"; $resn_exch{"2HBC DDH"}="HBC2 DDH"; $resn_exch{"3HBC DDH"}="HBC3 DDH";
$resn_exch{"1HMD DDH"}="HMD1 DDH"; $resn_exch{"2HMD DDH"}="HMD2 DDH"; $resn_exch{"3HMD DDH"}="HMD3 DDH";
$resn_exch{"1HAD DDH"}="HAD1 DDH"; $resn_exch{"2HAD DDH"}="HAD2 DDH"; $resn_exch{"1HBD DDH"}="HBD1 DDH";
$resn_exch{"2HBD DDH"}="HBD2 DDH"; $resn_exch{"DHO1 DDH"}="HO1D DDH"; $resn_exch{"1HN  DDI"}=" HN1 DDI";
$resn_exch{"2HN  DDI"}=" HN2 DDI"; $resn_exch{"1H2  DDL"}=" H21 DDL"; $resn_exch{"2H2  DDL"}=" H22 DDL";
$resn_exch{"1H6  DDL"}=" H61 DDL"; $resn_exch{"2H6  DDL"}=" H62 DDL"; $resn_exch{"3H6  DDL"}=" H63 DDL";
$resn_exch{"1HC1 DDM"}="HC11 DDM"; $resn_exch{"2HC1 DDM"}="HC12 DDM"; $resn_exch{"1HC6 DDM"}="HC61 DDM";
$resn_exch{"2HC6 DDM"}="HC62 DDM"; $resn_exch{"1H5* DDN"}=" H5' DDN"; $resn_exch{"2H5* DDN"}="H5'' DDN";
$resn_exch{" H4* DDN"}=" H4' DDN"; $resn_exch{" H3* DDN"}=" H3' DDN"; $resn_exch{"1H2* DDN"}=" H2' DDN";
$resn_exch{"2H2* DDN"}="H2'' DDN"; $resn_exch{" H1* DDN"}=" H1' DDN"; $resn_exch{"1HN  DDO"}=" HN1 DDO";
$resn_exch{"2HN  DDO"}=" HN2 DDO"; $resn_exch{"1HA  DDO"}=" HA1 DDO"; $resn_exch{"1HB  DDO"}=" HB1 DDO";
$resn_exch{"2HB  DDO"}=" HB2 DDO"; $resn_exch{"1HG  DDO"}=" HG1 DDO"; $resn_exch{"2HG  DDO"}=" HG2 DDO";
$resn_exch{"1HD  DDO"}=" HD1 DDO"; $resn_exch{"2HD  DDO"}=" HD2 DDO"; $resn_exch{"1HE  DDO"}=" HE1 DDO";
$resn_exch{"2HE  DDO"}=" HE2 DDO"; $resn_exch{"1H7  DDP"}=" H71 DDP"; $resn_exch{"2H7  DDP"}=" H72 DDP";
$resn_exch{"1H2  DDP"}=" H21 DDP"; $resn_exch{"2H2  DDP"}=" H22 DDP"; $resn_exch{"1H6  DDP"}=" H61 DDP";
$resn_exch{"1H9  DDP"}=" H91 DDP"; $resn_exch{"1HM1 DDQ"}="HM11 DDQ"; $resn_exch{"2HM1 DDQ"}="HM12 DDQ";
$resn_exch{"3HM1 DDQ"}="HM13 DDQ"; $resn_exch{"1HM2 DDQ"}="HM21 DDQ"; $resn_exch{"2HM2 DDQ"}="HM22 DDQ";
$resn_exch{"3HM2 DDQ"}="HM23 DDQ"; $resn_exch{"1H1  DDQ"}=" H11 DDQ"; $resn_exch{"2H1  DDQ"}=" H12 DDQ";
$resn_exch{"1H2  DDQ"}=" H21 DDQ"; $resn_exch{"2H2  DDQ"}=" H22 DDQ"; $resn_exch{"1H3  DDQ"}=" H31 DDQ";
$resn_exch{"2H3  DDQ"}=" H32 DDQ"; $resn_exch{"1H4  DDQ"}=" H41 DDQ"; $resn_exch{"2H4  DDQ"}=" H42 DDQ";
$resn_exch{"1H5  DDQ"}=" H51 DDQ"; $resn_exch{"2H5  DDQ"}=" H52 DDQ"; $resn_exch{"1H6  DDQ"}=" H61 DDQ";
$resn_exch{"2H6  DDQ"}=" H62 DDQ"; $resn_exch{"1H7  DDQ"}=" H71 DDQ"; $resn_exch{"2H7  DDQ"}=" H72 DDQ";
$resn_exch{"1H8  DDQ"}=" H81 DDQ"; $resn_exch{"2H8  DDQ"}=" H82 DDQ"; $resn_exch{"1H9  DDQ"}=" H91 DDQ";
$resn_exch{"2H9  DDQ"}=" H92 DDQ"; $resn_exch{"1H10 DDQ"}="H101 DDQ"; $resn_exch{"2H10 DDQ"}="H102 DDQ";
$resn_exch{"3H10 DDQ"}="H103 DDQ"; $resn_exch{"1H22 DDR"}="H221 DDR"; $resn_exch{"2H22 DDR"}="H222 DDR";
$resn_exch{"1H23 DDR"}="H231 DDR"; $resn_exch{"2H23 DDR"}="H232 DDR"; $resn_exch{"1H24 DDR"}="H241 DDR";
$resn_exch{"2H24 DDR"}="H242 DDR"; $resn_exch{"1H25 DDR"}="H251 DDR"; $resn_exch{"2H25 DDR"}="H252 DDR";
$resn_exch{"1H26 DDR"}="H261 DDR"; $resn_exch{"2H26 DDR"}="H262 DDR"; $resn_exch{"1H27 DDR"}="H271 DDR";
$resn_exch{"2H27 DDR"}="H272 DDR"; $resn_exch{"1H28 DDR"}="H281 DDR"; $resn_exch{"2H28 DDR"}="H282 DDR";
$resn_exch{"1H29 DDR"}="H291 DDR"; $resn_exch{"2H29 DDR"}="H292 DDR"; $resn_exch{"1H30 DDR"}="H301 DDR";
$resn_exch{"2H30 DDR"}="H302 DDR"; $resn_exch{"3H30 DDR"}="H303 DDR"; $resn_exch{"1H53 DDR"}="H531 DDR";
$resn_exch{"2H53 DDR"}="H532 DDR"; $resn_exch{"1H51 DDR"}="H511 DDR"; $resn_exch{"2H51 DDR"}="H512 DDR";
$resn_exch{"1H2  DDR"}=" H21 DDR"; $resn_exch{"2H2  DDR"}=" H22 DDR"; $resn_exch{"1H3  DDR"}=" H31 DDR";
$resn_exch{"2H3  DDR"}=" H32 DDR"; $resn_exch{"1H4  DDR"}=" H41 DDR"; $resn_exch{"2H4  DDR"}=" H42 DDR";
$resn_exch{"1H5  DDR"}=" H51 DDR"; $resn_exch{"2H5  DDR"}="H52A DDR"; $resn_exch{"1H6  DDR"}=" H61 DDR";
$resn_exch{"2H6  DDR"}=" H62 DDR"; $resn_exch{"1H7  DDR"}=" H71 DDR"; $resn_exch{"2H7  DDR"}=" H72 DDR";
$resn_exch{"1H8  DDR"}=" H81 DDR"; $resn_exch{"2H8  DDR"}=" H82 DDR"; $resn_exch{"1H9  DDR"}=" H91 DDR";
$resn_exch{"2H9  DDR"}=" H92 DDR"; $resn_exch{"1H10 DDR"}="H101 DDR"; $resn_exch{"2H10 DDR"}="H102 DDR";
$resn_exch{"3H10 DDR"}="H103 DDR"; $resn_exch{"1HC9 DDT"}="HC91 DDT"; $resn_exch{"2HC9 DDT"}="HC92 DDT";
$resn_exch{"3HC9 DDT"}="HC93 DDT"; $resn_exch{"1H10 DDT"}="H101 DDT"; $resn_exch{"2H10 DDT"}="H102 DDT";
$resn_exch{"3H10 DDT"}="H103 DDT"; $resn_exch{"1H13 DDT"}="H131 DDT"; $resn_exch{"2H13 DDT"}="H132 DDT";
$resn_exch{"1H2* DDT"}="H2'1 DDT"; $resn_exch{"1H3* DDT"}="H3'1 DDT"; $resn_exch{"1H4* DDT"}="H4'1 DDT";
$resn_exch{"1H6* DDT"}="H6'1 DDT"; $resn_exch{"1H7* DDT"}="H7'1 DDT"; $resn_exch{"1H8* DDT"}="H8'1 DDT";
$resn_exch{"1H9* DDT"}="H9'1 DDT"; $resn_exch{"2H9* DDT"}="H9'2 DDT"; $resn_exch{"3H9* DDT"}="H9'3 DDT";
$resn_exch{"1H** DDT"}=" HB1 DDT"; $resn_exch{"2H** DDT"}=" HB2 DDT"; $resn_exch{"3H** DDT"}=" HB3 DDT";
$resn_exch{" H1* DDU"}=" H1' DDU"; $resn_exch{"1H2* DDU"}="H2'1 DDU"; $resn_exch{"2H2* DDU"}="H2'2 DDU";
$resn_exch{" H3* DDU"}=" H3' DDU"; $resn_exch{" H4* DDU"}=" H4' DDU"; $resn_exch{"*HO3 DDU"}="HO3' DDU";
$resn_exch{"1H5* DDU"}="H5'1 DDU"; $resn_exch{"2H5* DDU"}="H5'2 DDU"; $resn_exch{"3H5* DDU"}="H5'3 DDU";
$resn_exch{"2HOP DDX"}="HOP2 DDX"; $resn_exch{"3HOP DDX"}="HOP3 DDX"; $resn_exch{"1H5* DDX"}=" H5' DDX";
$resn_exch{"2H5* DDX"}="H5'' DDX"; $resn_exch{" H4* DDX"}=" H4' DDX"; $resn_exch{" H1* DDX"}=" H1' DDX";
$resn_exch{" H3* DDX"}=" H3' DDX"; $resn_exch{" H2* DDX"}=" H2' DDX"; $resn_exch{"HO1* DDX"}="HO1' DDX";
$resn_exch{"1H5* DDY"}="H5'1 DDY"; $resn_exch{"2H5* DDY"}="H5'2 DDY"; $resn_exch{" H4* DDY"}=" H4' DDY";
$resn_exch{"1H3* DDY"}="H3'1 DDY"; $resn_exch{"2H3* DDY"}="H3'2 DDY"; $resn_exch{"1H2* DDY"}="H2'1 DDY";
$resn_exch{"2H2* DDY"}="H2'2 DDY"; $resn_exch{" H1* DDY"}=" H1' DDY"; $resn_exch{"1HN4 DDY"}="HN41 DDY";
$resn_exch{"2HN4 DDY"}="HN42 DDY"; $resn_exch{"1HAA DE1"}="HAA1 DE1"; $resn_exch{"2HAA DE1"}="HAA2 DE1";
$resn_exch{"3HAA DE1"}="HAA3 DE1"; $resn_exch{"1HAC DE1"}="HAC1 DE1"; $resn_exch{"2HAC DE1"}="HAC2 DE1";
$resn_exch{"1HAE DE1"}="HAE1 DE1"; $resn_exch{"2HAE DE1"}="HAE2 DE1"; $resn_exch{"1HAG DE1"}="HAG1 DE1";
$resn_exch{"2HAG DE1"}="HAG2 DE1"; $resn_exch{"1HAI DE1"}="HAI1 DE1"; $resn_exch{"2HAI DE1"}="HAI2 DE1";
$resn_exch{"1HAK DE1"}="HAK1 DE1"; $resn_exch{"2HAK DE1"}="HAK2 DE1"; $resn_exch{"1HAJ DE1"}="HAJ1 DE1";
$resn_exch{"2HAJ DE1"}="HAJ2 DE1"; $resn_exch{"1HAH DE1"}="HAH1 DE1"; $resn_exch{"2HAH DE1"}="HAH2 DE1";
$resn_exch{"1HAF DE1"}="HAF1 DE1"; $resn_exch{"2HAF DE1"}="HAF2 DE1"; $resn_exch{"1HAD DE1"}="HAD1 DE1";
$resn_exch{"2HAD DE1"}="HAD2 DE1"; $resn_exch{"1HN6 DEA"}="HN61 DEA"; $resn_exch{"2HN6 DEA"}="HN62 DEA";
$resn_exch{"1H1* DEA"}="H1'1 DEA"; $resn_exch{"2H1* DEA"}="H1'2 DEA"; $resn_exch{" H2* DEA"}=" H2' DEA";
$resn_exch{"2HO* DEA"}="HO'2 DEA"; $resn_exch{" H3* DEA"}=" H3' DEA"; $resn_exch{"3HO* DEA"}="HO'3 DEA";
$resn_exch{"1H7  DEB"}=" H71 DEB"; $resn_exch{"2H7  DEB"}=" H72 DEB"; $resn_exch{"1H14 DEB"}="H141 DEB";
$resn_exch{"2H14 DEB"}="H142 DEB"; $resn_exch{"1H15 DEB"}="H151 DEB"; $resn_exch{"2H15 DEB"}="H152 DEB";
$resn_exch{"3H15 DEB"}="H153 DEB"; $resn_exch{"1H18 DEB"}="H181 DEB"; $resn_exch{"2H18 DEB"}="H182 DEB";
$resn_exch{"3H18 DEB"}="H183 DEB"; $resn_exch{"1H20 DEB"}="H201 DEB"; $resn_exch{"2H20 DEB"}="H202 DEB";
$resn_exch{"3H20 DEB"}="H203 DEB"; $resn_exch{"1H22 DEB"}="H221 DEB"; $resn_exch{"2H22 DEB"}="H222 DEB";
$resn_exch{"3H22 DEB"}="H223 DEB"; $resn_exch{"1H23 DEB"}="H231 DEB"; $resn_exch{"2H23 DEB"}="H232 DEB";
$resn_exch{"3H23 DEB"}="H233 DEB"; $resn_exch{"1H25 DEB"}="H251 DEB"; $resn_exch{"2H25 DEB"}="H252 DEB";
$resn_exch{"3H25 DEB"}="H253 DEB"; $resn_exch{"1H27 DEB"}="H271 DEB"; $resn_exch{"2H27 DEB"}="H272 DEB";
$resn_exch{"3H27 DEB"}="H273 DEB"; $resn_exch{"1H2  DEC"}=" H21 DEC"; $resn_exch{"2H2  DEC"}=" H22 DEC";
$resn_exch{"1H3  DEC"}=" H31 DEC"; $resn_exch{"2H3  DEC"}=" H32 DEC"; $resn_exch{"1H4  DEC"}=" H41 DEC";
$resn_exch{"2H4  DEC"}=" H42 DEC"; $resn_exch{"1H5  DEC"}=" H51 DEC"; $resn_exch{"2H5  DEC"}=" H52 DEC";
$resn_exch{"1H6  DEC"}=" H61 DEC"; $resn_exch{"2H6  DEC"}=" H62 DEC"; $resn_exch{"1H7  DEC"}=" H71 DEC";
$resn_exch{"2H7  DEC"}=" H72 DEC"; $resn_exch{"1H8  DEC"}=" H81 DEC"; $resn_exch{"2H8  DEC"}=" H82 DEC";
$resn_exch{"1H9  DEC"}=" H91 DEC"; $resn_exch{"2H9  DEC"}=" H92 DEC"; $resn_exch{"1H10 DED"}="H101 DED";
$resn_exch{"2H10 DED"}="H102 DED"; $resn_exch{"1H11 DED"}="H111 DED"; $resn_exch{"2H11 DED"}="H112 DED";
$resn_exch{"1H13 DED"}="H131 DED"; $resn_exch{"2H13 DED"}="H132 DED"; $resn_exch{"3H13 DED"}="H133 DED";
$resn_exch{"1H14 DED"}="H141 DED"; $resn_exch{"2H14 DED"}="H142 DED"; $resn_exch{"3H14 DED"}="H143 DED";
$resn_exch{"1H11 DEE"}="H111 DEE"; $resn_exch{"2H11 DEE"}="H112 DEE"; $resn_exch{"1H12 DEE"}="H121 DEE";
$resn_exch{"2H12 DEE"}="H122 DEE"; $resn_exch{"3H12 DEE"}="H123 DEE"; $resn_exch{"1H9  DEE"}=" H91 DEE";
$resn_exch{"2H9  DEE"}=" H92 DEE"; $resn_exch{"3H9  DEE"}=" H93 DEE"; $resn_exch{"1H10 DEE"}="H101 DEE";
$resn_exch{"2H10 DEE"}="H102 DEE"; $resn_exch{"3H10 DEE"}="H103 DEE"; $resn_exch{"1H12 DEF"}="H121 DEF";
$resn_exch{"2H12 DEF"}="H122 DEF"; $resn_exch{"1H13 DEF"}="H131 DEF"; $resn_exch{"2H13 DEF"}="H132 DEF";
$resn_exch{"NH12 DEF"}="H12N DEF"; $resn_exch{"1H15 DEF"}="H151 DEF"; $resn_exch{"2H15 DEF"}="H152 DEF";
$resn_exch{"1H16 DEF"}="H161 DEF"; $resn_exch{"2H16 DEF"}="H162 DEF"; $resn_exch{"1H18 DEF"}="H181 DEF";
$resn_exch{"2H18 DEF"}="H182 DEF"; $resn_exch{"1H17 DEF"}="H171 DEF"; $resn_exch{"2H17 DEF"}="H172 DEF";
$resn_exch{"1H19 DEF"}="H191 DEF"; $resn_exch{"2H19 DEF"}="H192 DEF"; $resn_exch{"1H22 DEF"}="H221 DEF";
$resn_exch{"2H22 DEF"}="H222 DEF"; $resn_exch{"1H23 DEF"}="H231 DEF"; $resn_exch{"2H23 DEF"}="H232 DEF";
$resn_exch{"NH22 DEF"}="H22N DEF"; $resn_exch{"1H25 DEF"}="H251 DEF"; $resn_exch{"2H25 DEF"}="H252 DEF";
$resn_exch{"1H26 DEF"}="H261 DEF"; $resn_exch{"2H26 DEF"}="H262 DEF"; $resn_exch{"1H27 DEF"}="H271 DEF";
$resn_exch{"2H27 DEF"}="H272 DEF"; $resn_exch{"1H28 DEF"}="H281 DEF"; $resn_exch{"2H28 DEF"}="H282 DEF";
$resn_exch{"1H29 DEF"}="H291 DEF"; $resn_exch{"2H29 DEF"}="H292 DEF"; $resn_exch{"1H32 DEF"}="H321 DEF";
$resn_exch{"2H32 DEF"}="H322 DEF"; $resn_exch{"1H39 DEF"}="H391 DEF"; $resn_exch{"2H39 DEF"}="H392 DEF";
$resn_exch{"1H33 DEF"}="H331 DEF"; $resn_exch{"2H33 DEF"}="H332 DEF"; $resn_exch{"3H33 DEF"}="H333 DEF";
$resn_exch{"1H40 DEF"}="H401 DEF"; $resn_exch{"2H40 DEF"}="H402 DEF"; $resn_exch{"1H41 DEF"}="H411 DEF";
$resn_exch{"2H41 DEF"}="H412 DEF"; $resn_exch{"1H42 DEF"}="H421 DEF"; $resn_exch{"2H42 DEF"}="H422 DEF";
$resn_exch{"1H43 DEF"}="H431 DEF"; $resn_exch{"2H43 DEF"}="H432 DEF"; $resn_exch{"NH44 DEF"}="H44N DEF";
$resn_exch{"1H49 DEF"}="H491 DEF"; $resn_exch{"2H49 DEF"}="H492 DEF"; $resn_exch{"3H49 DEF"}="H493 DEF";
$resn_exch{"1H3' DEG"}="H3'1 DEG"; $resn_exch{"2H3' DEG"}="H3'2 DEG"; $resn_exch{"1H4' DEG"}="H4'1 DEG";
$resn_exch{"2H4' DEG"}="H4'2 DEG"; $resn_exch{"3H4' DEG"}="H4'3 DEG"; $resn_exch{"1H1' DEG"}="H1'1 DEG";
$resn_exch{"2H1' DEG"}="H1'2 DEG"; $resn_exch{"1H2' DEG"}="H2'1 DEG"; $resn_exch{"2H2' DEG"}="H2'2 DEG";
$resn_exch{"1H6C DEG"}="H6C1 DEG"; $resn_exch{"2H6C DEG"}="H6C2 DEG"; $resn_exch{"1HC4 DEL"}="HC41 DEL";
$resn_exch{"2HC4 DEL"}="HC42 DEL"; $resn_exch{"1H6  DEL"}=" H61 DEL"; $resn_exch{"2H6  DEL"}=" H62 DEL";
$resn_exch{"1H6' DEL"}="H6'1 DEL"; $resn_exch{"2H6' DEL"}="H6'2 DEL"; $resn_exch{"'HO1 DEL"}="HO1' DEL";
$resn_exch{"'HO2 DEL"}="HO2' DEL"; $resn_exch{"'HO3 DEL"}="HO3' DEL"; $resn_exch{"'HO6 DEL"}="HO6' DEL";
$resn_exch{"1HCM DEM"}="HCM1 DEM"; $resn_exch{"2HCM DEM"}="HCM2 DEM"; $resn_exch{"1HC1 DEM"}="HC11 DEM";
$resn_exch{"2HC1 DEM"}="HC12 DEM"; $resn_exch{"1HC2 DEM"}="HC21 DEM"; $resn_exch{"2HC2 DEM"}="HC22 DEM";
$resn_exch{"1HC3 DEM"}="HC31 DEM"; $resn_exch{"2HC3 DEM"}="HC32 DEM"; $resn_exch{"1HC4 DEM"}="HC41 DEM";
$resn_exch{"2HC4 DEM"}="HC42 DEM"; $resn_exch{"1HC5 DEM"}="HC51 DEM"; $resn_exch{"2HC5 DEM"}="HC52 DEM";
$resn_exch{"1HC6 DEM"}="HC61 DEM"; $resn_exch{"2HC6 DEM"}="HC62 DEM"; $resn_exch{"1HC7 DEM"}="HC71 DEM";
$resn_exch{"2HC7 DEM"}="HC72 DEM"; $resn_exch{"1HC8 DEM"}="HC81 DEM"; $resn_exch{"2HC8 DEM"}="HC82 DEM";
$resn_exch{"1HC9 DEM"}="HC91 DEM"; $resn_exch{"2HC9 DEM"}="HC92 DEM"; $resn_exch{"1H10 DEM"}="H101 DEM";
$resn_exch{"2H10 DEM"}="H102 DEM"; $resn_exch{"3H10 DEM"}="H103 DEM"; $resn_exch{"1H1  DEN"}=" H11 DEN";
$resn_exch{"2H1  DEN"}=" H12 DEN"; $resn_exch{"1H9  DEO"}=" H91 DEO"; $resn_exch{"2H9  DEO"}=" H92 DEO";
$resn_exch{"1H10 DEO"}="H101 DEO"; $resn_exch{"2H10 DEO"}="H102 DEO"; $resn_exch{"1H12 DEO"}="H121 DEO";
$resn_exch{"2H12 DEO"}="H122 DEO"; $resn_exch{"1H26 DEO"}="H261 DEO"; $resn_exch{"2H26 DEO"}="H262 DEO";
$resn_exch{"1H27 DEO"}="H271 DEO"; $resn_exch{"2H27 DEO"}="H272 DEO"; $resn_exch{"3H27 DEO"}="H273 DEO";
$resn_exch{"1H1  DEP"}=" H11 DEP"; $resn_exch{"2H1  DEP"}=" H12 DEP"; $resn_exch{"1H2  DEP"}=" H21 DEP";
$resn_exch{"2H2  DEP"}=" H22 DEP"; $resn_exch{"3H2  DEP"}=" H23 DEP"; $resn_exch{"1H3  DEP"}=" H31 DEP";
$resn_exch{"2H3  DEP"}=" H32 DEP"; $resn_exch{"1H4  DEP"}=" H41 DEP"; $resn_exch{"2H4  DEP"}=" H42 DEP";
$resn_exch{"3H4  DEP"}=" H43 DEP"; $resn_exch{"1H1  DEQ"}=" H11 DEQ"; $resn_exch{"1H2  DEQ"}=" H21 DEQ";
$resn_exch{"1H3  DEQ"}=" H31 DEQ"; $resn_exch{"1H4  DEQ"}=" H41 DEQ"; $resn_exch{"1H8  DEQ"}=" H81 DEQ";
$resn_exch{"1H13 DEQ"}="H131 DEQ"; $resn_exch{"1H15 DEQ"}="H151 DEQ"; $resn_exch{"1H16 DEQ"}="H161 DEQ";
$resn_exch{"1H17 DEQ"}="H171 DEQ"; $resn_exch{"1H18 DEQ"}="H181 DEQ"; $resn_exch{"1H19 DEQ"}="H191 DEQ";
$resn_exch{"2H19 DEQ"}="H192 DEQ"; $resn_exch{"1H20 DEQ"}="H201 DEQ"; $resn_exch{"2H20 DEQ"}="H202 DEQ";
$resn_exch{"1H21 DEQ"}="H211 DEQ"; $resn_exch{"2H21 DEQ"}="H212 DEQ"; $resn_exch{"1H22 DEQ"}="H221 DEQ";
$resn_exch{"2H22 DEQ"}="H222 DEQ"; $resn_exch{"1H23 DEQ"}="H231 DEQ"; $resn_exch{"2H23 DEQ"}="H232 DEQ";
$resn_exch{"1H24 DEQ"}="H241 DEQ"; $resn_exch{"2H24 DEQ"}="H242 DEQ"; $resn_exch{"1H25 DEQ"}="H251 DEQ";
$resn_exch{"2H25 DEQ"}="H252 DEQ"; $resn_exch{"1H26 DEQ"}="H261 DEQ"; $resn_exch{"2H26 DEQ"}="H262 DEQ";
$resn_exch{"1H27 DEQ"}="H271 DEQ"; $resn_exch{"2H27 DEQ"}="H272 DEQ"; $resn_exch{"1H28 DEQ"}="H281 DEQ";
$resn_exch{"2H28 DEQ"}="H282 DEQ"; $resn_exch{"1H30 DEQ"}="H301 DEQ"; $resn_exch{"2H30 DEQ"}="H302 DEQ";
$resn_exch{"3H30 DEQ"}="H303 DEQ"; $resn_exch{"1H29 DEQ"}="H291 DEQ"; $resn_exch{"2H29 DEQ"}="H292 DEQ";
$resn_exch{"3H29 DEQ"}="H293 DEQ"; $resn_exch{"1HN4 DEQ"}="HN41 DEQ"; $resn_exch{"2HN4 DEQ"}="HN42 DEQ";
$resn_exch{"1HN3 DEQ"}="HN31 DEQ"; $resn_exch{"2HN3 DEQ"}="HN32 DEQ"; $resn_exch{"1H4  DER"}=" H41 DER";
$resn_exch{"2H4  DER"}=" H42 DER"; $resn_exch{"1HP9 DES"}="HP91 DES"; $resn_exch{"2HP9 DES"}="HP92 DES";
$resn_exch{"3HP9 DES"}="HP93 DES"; $resn_exch{"1HP8 DES"}="HP81 DES"; $resn_exch{"2HP8 DES"}="HP82 DES";
$resn_exch{"3HOP DES"}="HOP3 DES"; $resn_exch{"3HO  DES"}=" HO3 DES"; $resn_exch{"1H8  DES"}=" H81 DES";
$resn_exch{"2H8  DES"}=" H82 DES"; $resn_exch{"1H9  DES"}=" H91 DES"; $resn_exch{"2H9  DES"}=" H92 DES";
$resn_exch{"3H9  DES"}=" H93 DES"; $resn_exch{"1HM1 DET"}="HM11 DET"; $resn_exch{"2HM1 DET"}="HM12 DET";
$resn_exch{"3HM1 DET"}="HM13 DET"; $resn_exch{"1HM2 DET"}="HM21 DET"; $resn_exch{"2HM2 DET"}="HM22 DET";
$resn_exch{"3HM2 DET"}="HM23 DET"; $resn_exch{"1H1  DET"}=" H11 DET"; $resn_exch{"2H1  DET"}=" H12 DET";
$resn_exch{"1H2  DET"}=" H21 DET"; $resn_exch{"2H2  DET"}=" H22 DET"; $resn_exch{"1H3  DET"}=" H31 DET";
$resn_exch{"2H3  DET"}=" H32 DET"; $resn_exch{"1H4  DET"}=" H41 DET"; $resn_exch{"2H4  DET"}=" H42 DET";
$resn_exch{"1H5  DET"}=" H51 DET"; $resn_exch{"2H5  DET"}=" H52 DET"; $resn_exch{"1H6  DET"}=" H61 DET";
$resn_exch{"2H6  DET"}=" H62 DET"; $resn_exch{"1H7  DET"}=" H71 DET"; $resn_exch{"2H7  DET"}=" H72 DET";
$resn_exch{"1H8  DET"}=" H81 DET"; $resn_exch{"2H8  DET"}=" H82 DET"; $resn_exch{"1H9  DET"}=" H91 DET";
$resn_exch{"2H9  DET"}=" H92 DET"; $resn_exch{"1H10 DET"}="H101 DET"; $resn_exch{"2H10 DET"}="H102 DET";
$resn_exch{"1H11 DET"}="H111 DET"; $resn_exch{"2H11 DET"}="H112 DET"; $resn_exch{"3H11 DET"}="H113 DET";
$resn_exch{"1HMB DEU"}="HMB1 DEU"; $resn_exch{"2HMB DEU"}="HMB2 DEU"; $resn_exch{"3HMB DEU"}="HMB3 DEU";
$resn_exch{"1HMC DEU"}="HMC1 DEU"; $resn_exch{"2HMC DEU"}="HMC2 DEU"; $resn_exch{"3HMC DEU"}="HMC3 DEU";
$resn_exch{"1HMA DEU"}="HMA1 DEU"; $resn_exch{"2HMA DEU"}="HMA2 DEU"; $resn_exch{"3HMA DEU"}="HMA3 DEU";
$resn_exch{"1HMD DEU"}="HMD1 DEU"; $resn_exch{"2HMD DEU"}="HMD2 DEU"; $resn_exch{"3HMD DEU"}="HMD3 DEU";
$resn_exch{"1HAA DEU"}="HAA1 DEU"; $resn_exch{"2HAA DEU"}="HAA2 DEU"; $resn_exch{"1HBA DEU"}="HBA1 DEU";
$resn_exch{"2HBA DEU"}="HBA2 DEU"; $resn_exch{"1HAD DEU"}="HAD1 DEU"; $resn_exch{"2HAD DEU"}="HAD2 DEU";
$resn_exch{"1HBD DEU"}="HBD1 DEU"; $resn_exch{"2HBD DEU"}="HBD2 DEU"; $resn_exch{"1H21 DEX"}="H211 DEX";
$resn_exch{"1H22 DEX"}="H221 DEX"; $resn_exch{"1H51 DEX"}="H511 DEX"; $resn_exch{"1H52 DEX"}="H521 DEX";
$resn_exch{"1H81 DEX"}="H811 DEX"; $resn_exch{"1H82 DEX"}="H821 DEX"; $resn_exch{"1H83 DEX"}="H831 DEX";
$resn_exch{"1H91 DEX"}="H911 DEX"; $resn_exch{"1H92 DEX"}="H921 DEX"; $resn_exch{"1H93 DEX"}="H931 DEX";
$resn_exch{"2H11 DEX"}="H112 DEX"; $resn_exch{"2H12 DEX"}="H122 DEX"; $resn_exch{"2H21 DEX"}="H212 DEX";
$resn_exch{"2H22 DEX"}="H222 DEX"; $resn_exch{"2H23 DEX"}="H232 DEX"; $resn_exch{"1H4C DEZ"}="H4C1 DEZ";
$resn_exch{"2H4C DEZ"}="H4C2 DEZ"; $resn_exch{"1H11 DF1"}="H111 DF1"; $resn_exch{"2H11 DF1"}="H112 DF1";
$resn_exch{"1H14 DF1"}="H141 DF1"; $resn_exch{"2H14 DF1"}="H142 DF1"; $resn_exch{"3H14 DF1"}="H143 DF1";
$resn_exch{"1H26 DF2"}="H261 DF2"; $resn_exch{"2H26 DF2"}="H262 DF2"; $resn_exch{"1H8C DF2"}="H8C1 DF2";
$resn_exch{"2H8C DF2"}="H8C2 DF2"; $resn_exch{"1HC7 DFB"}="HC71 DFB"; $resn_exch{"2HC7 DFB"}="HC72 DFB";
$resn_exch{"3HOP DFC"}="HOP3 DFC"; $resn_exch{"2HOP DFC"}="HOP2 DFC"; $resn_exch{"1H5* DFC"}=" H5' DFC";
$resn_exch{"2H5* DFC"}="H5'' DFC"; $resn_exch{" H4* DFC"}=" H4' DFC"; $resn_exch{" H3* DFC"}=" H3' DFC";
$resn_exch{"H3T  DFC"}="HO3' DFC"; $resn_exch{"1H2* DFC"}=" H2' DFC"; $resn_exch{"2H2* DFC"}="H2'' DFC";
$resn_exch{" H1* DFC"}=" H1' DFC"; $resn_exch{"1H4  DFC"}=" H41 DFC"; $resn_exch{"2H4  DFC"}=" H42 DFC";
$resn_exch{"1H11 DFD"}="H111 DFD"; $resn_exch{"2H11 DFD"}="H112 DFD"; $resn_exch{"3H11 DFD"}="H113 DFD";
$resn_exch{"1H10 DFD"}="H101 DFD"; $resn_exch{"2H10 DFD"}="H102 DFD"; $resn_exch{"1H9  DFD"}=" H91 DFD";
$resn_exch{"2H9  DFD"}=" H92 DFD"; $resn_exch{"1H8  DFD"}=" H81 DFD"; $resn_exch{"2H8  DFD"}=" H82 DFD";
$resn_exch{"1H7  DFD"}=" H71 DFD"; $resn_exch{"2H7  DFD"}=" H72 DFD"; $resn_exch{"1H6  DFD"}=" H61 DFD";
$resn_exch{"2H6  DFD"}=" H62 DFD"; $resn_exch{"1H5  DFD"}=" H51 DFD"; $resn_exch{"2H5  DFD"}=" H52 DFD";
$resn_exch{"1H4  DFD"}=" H41 DFD"; $resn_exch{"2H4  DFD"}=" H42 DFD"; $resn_exch{"1H3  DFD"}=" H31 DFD";
$resn_exch{"2H3  DFD"}=" H32 DFD"; $resn_exch{"1H2  DFD"}=" H21 DFD"; $resn_exch{"2H2  DFD"}=" H22 DFD";
$resn_exch{"1HAA DFE"}="HAA1 DFE"; $resn_exch{"2HAA DFE"}="HAA2 DFE"; $resn_exch{"3HAA DFE"}="HAA3 DFE";
$resn_exch{"1HAG DFE"}="HAG1 DFE"; $resn_exch{"2HAG DFE"}="HAG2 DFE"; $resn_exch{"3HOP DFG"}="HOP3 DFG";
$resn_exch{"2HOP DFG"}="HOP2 DFG"; $resn_exch{"1H5* DFG"}=" H5' DFG"; $resn_exch{"2H5* DFG"}="H5'' DFG";
$resn_exch{" H4* DFG"}=" H4' DFG"; $resn_exch{" H3* DFG"}=" H3' DFG"; $resn_exch{"H3T  DFG"}="HO3' DFG";
$resn_exch{"1H2* DFG"}=" H2' DFG"; $resn_exch{"2H2* DFG"}="H2'' DFG"; $resn_exch{" H1* DFG"}=" H1' DFG";
$resn_exch{"1H2  DFG"}=" H21 DFG"; $resn_exch{"2H2  DFG"}=" H22 DFG"; $resn_exch{"1HN  DFI"}=" H   DFI";
$resn_exch{"2HN  DFI"}=" H2  DFI"; $resn_exch{"1HB  DFI"}=" HB2 DFI"; $resn_exch{"2HB  DFI"}=" HB3 DFI";
$resn_exch{"1HD1 DFI"}="HD11 DFI"; $resn_exch{"2HD1 DFI"}="HD12 DFI"; $resn_exch{"3HD1 DFI"}="HD13 DFI";
$resn_exch{"1HD2 DFI"}="HD21 DFI"; $resn_exch{"2HD2 DFI"}="HD22 DFI"; $resn_exch{"3HD2 DFI"}="HD23 DFI";
$resn_exch{"1H20 DFJ"}="H201 DFJ"; $resn_exch{"2H20 DFJ"}="H202 DFJ"; $resn_exch{"1H35 DFJ"}="H351 DFJ";
$resn_exch{"2H35 DFJ"}="H352 DFJ"; $resn_exch{"1H40 DFJ"}="H401 DFJ"; $resn_exch{"2H40 DFJ"}="H402 DFJ";
$resn_exch{"1H7C DFJ"}="H7C1 DFJ"; $resn_exch{"2H7C DFJ"}="H7C2 DFJ"; $resn_exch{"1H8  DFL"}=" H81 DFL";
$resn_exch{"2H8  DFL"}=" H82 DFL"; $resn_exch{"1H21 DFM"}="H211 DFM"; $resn_exch{"2H21 DFM"}="H212 DFM";
$resn_exch{"1H27 DFM"}="H271 DFM"; $resn_exch{"2H27 DFM"}="H272 DFM"; $resn_exch{"1H4C DFM"}="H4C1 DFM";
$resn_exch{"2H4C DFM"}="H4C2 DFM"; $resn_exch{"1H71 DFM"}="H711 DFM"; $resn_exch{"2H71 DFM"}="H712 DFM";
$resn_exch{"3H71 DFM"}="H713 DFM"; $resn_exch{"1H28 DFN"}="H281 DFN"; $resn_exch{"2H28 DFN"}="H282 DFN";
$resn_exch{"3H28 DFN"}="H283 DFN"; $resn_exch{"1H26 DFN"}="H261 DFN"; $resn_exch{"2H26 DFN"}="H262 DFN";
$resn_exch{"1H40 DFN"}="H401 DFN"; $resn_exch{"2H40 DFN"}="H402 DFN"; $resn_exch{"1HN  DFO"}=" H   DFO";
$resn_exch{"2HN  DFO"}=" H2  DFO"; $resn_exch{"1HB  DFO"}=" HB2 DFO"; $resn_exch{"2HB  DFO"}=" HB3 DFO";
$resn_exch{"1HD1 DFO"}="HD11 DFO"; $resn_exch{"2HD1 DFO"}="HD12 DFO"; $resn_exch{"3HD1 DFO"}="HD13 DFO";
$resn_exch{"1HD2 DFO"}="HD21 DFO"; $resn_exch{"2HD2 DFO"}="HD22 DFO"; $resn_exch{"3HD2 DFO"}="HD23 DFO";
$resn_exch{"1H2  DFP"}=" H21 DFP"; $resn_exch{"2H2  DFP"}=" H22 DFP"; $resn_exch{"3H2  DFP"}=" H23 DFP";
$resn_exch{"1H3  DFP"}=" H31 DFP"; $resn_exch{"2H3  DFP"}=" H32 DFP"; $resn_exch{"3H3  DFP"}=" H33 DFP";
$resn_exch{"1H2' DFP"}="H2'1 DFP"; $resn_exch{"2H2' DFP"}="H2'2 DFP"; $resn_exch{"3H2' DFP"}="H2'3 DFP";
$resn_exch{"1H3' DFP"}="H3'1 DFP"; $resn_exch{"2H3' DFP"}="H3'2 DFP"; $resn_exch{"3H3' DFP"}="H3'3 DFP";
$resn_exch{"1H5  DFR"}=" H51 DFR"; $resn_exch{"2H5  DFR"}=" H52 DFR"; $resn_exch{"1H6  DFR"}=" H61 DFR";
$resn_exch{"2H6  DFR"}=" H62 DFR"; $resn_exch{"1H7  DFR"}=" H71 DFR"; $resn_exch{"2H7  DFR"}=" H72 DFR";
$resn_exch{"3H7  DFR"}=" H73 DFR"; $resn_exch{"2HOP DFT"}="HOP2 DFT"; $resn_exch{"3HOP DFT"}="HOP3 DFT";
$resn_exch{"1H5* DFT"}=" H5' DFT"; $resn_exch{"2H5* DFT"}="H5'' DFT"; $resn_exch{" H4* DFT"}=" H4' DFT";
$resn_exch{" H3* DFT"}=" H3' DFT"; $resn_exch{"*HO3 DFT"}="HO3' DFT"; $resn_exch{"1H2* DFT"}=" H2' DFT";
$resn_exch{"2H2* DFT"}="H2'' DFT"; $resn_exch{" H1* DFT"}=" H1' DFT"; $resn_exch{"1H5M DFT"}=" H71 DFT";
$resn_exch{"2H5M DFT"}=" H72 DFT"; $resn_exch{"3H5M DFT"}=" H73 DFT"; $resn_exch{"1H1  DFU"}=" H11 DFU";
$resn_exch{"2H1  DFU"}=" H12 DFU"; $resn_exch{"1H6  DFU"}=" H61 DFU"; $resn_exch{"2H6  DFU"}=" H62 DFU";
$resn_exch{"3H6  DFU"}=" H63 DFU"; $resn_exch{"1H8  DFV"}=" H81 DFV"; $resn_exch{"2H8  DFV"}=" H82 DFV";
$resn_exch{"1H8C DFW"}="H8C1 DFW"; $resn_exch{"2H8C DFW"}="H8C2 DFW"; $resn_exch{"1H1  DFX"}=" H11 DFX";
$resn_exch{"2H1  DFX"}=" H12 DFX"; $resn_exch{"1H5  DFX"}=" H51 DFX"; $resn_exch{"2H5  DFX"}=" H52 DFX";
$resn_exch{"1H23 DFY"}="H231 DFY"; $resn_exch{"2H23 DFY"}="H232 DFY"; $resn_exch{"1H8C DFY"}="H8C1 DFY";
$resn_exch{"2H8C DFY"}="H8C2 DFY"; $resn_exch{"1HAA DFY"}="HAA1 DFY"; $resn_exch{"2HAA DFY"}="HAA2 DFY";
$resn_exch{"3HAA DFY"}="HAA3 DFY"; $resn_exch{"1H23 DFZ"}="H231 DFZ"; $resn_exch{"2H23 DFZ"}="H232 DFZ";
$resn_exch{"1H8C DFZ"}="H8C1 DFZ"; $resn_exch{"2H8C DFZ"}="H8C2 DFZ"; $resn_exch{"3HOP  DG"}="HOP3  DG";
$resn_exch{"2HOP  DG"}="HOP2  DG"; $resn_exch{"1H5*  DG"}=" H5'  DG"; $resn_exch{"2H5*  DG"}="H5''  DG";
$resn_exch{" H4*  DG"}=" H4'  DG"; $resn_exch{" H3*  DG"}=" H3'  DG"; $resn_exch{"H3T   DG"}="HO3'  DG";
$resn_exch{"1H2*  DG"}=" H2'  DG"; $resn_exch{"2H2*  DG"}="H2''  DG"; $resn_exch{" H1*  DG"}=" H1'  DG";
$resn_exch{"1H2   DG"}=" H21  DG"; $resn_exch{"2H2   DG"}=" H22  DG"; $resn_exch{"1HAA DG1"}="HAA1 DG1";
$resn_exch{"2HAA DG1"}="HAA2 DG1"; $resn_exch{" H1* DG1"}=" H1' DG1"; $resn_exch{" H2* DG1"}=" H2' DG1";
$resn_exch{"HO2* DG1"}="HO2' DG1"; $resn_exch{" H3* DG1"}=" H3' DG1"; $resn_exch{"HO3* DG1"}="HO3' DG1";
$resn_exch{" H4* DG1"}=" H4' DG1"; $resn_exch{"1H5* DG1"}="H5'1 DG1"; $resn_exch{"2H5* DG1"}="H5'2 DG1";
$resn_exch{"1HAY DG1"}="HAY1 DG1"; $resn_exch{"2HAY DG1"}="HAY2 DG1"; $resn_exch{"1HN6 DG1"}="HN61 DG1";
$resn_exch{"2HN6 DG1"}="HN62 DG1"; $resn_exch{"1H4  DG2"}=" H41 DG2"; $resn_exch{"2H4  DG2"}=" H42 DG2";
$resn_exch{"2HOG DG3"}="HOG2 DG3"; $resn_exch{"3HOG DG3"}="HOG3 DG3"; $resn_exch{"2HOB DG3"}="HOB2 DG3";
$resn_exch{"2HOA DG3"}="HOA2 DG3"; $resn_exch{"1H5* DG3"}="H5'1 DG3"; $resn_exch{"2H5* DG3"}="H5'2 DG3";
$resn_exch{" H4* DG3"}=" H4' DG3"; $resn_exch{"1H3* DG3"}="H3'1 DG3"; $resn_exch{"2H3* DG3"}="H3'2 DG3";
$resn_exch{"1H2* DG3"}="H2'1 DG3"; $resn_exch{"2H2* DG3"}="H2'2 DG3"; $resn_exch{" H1* DG3"}=" H1' DG3";
$resn_exch{"1HN2 DG3"}="HN21 DG3"; $resn_exch{"2HN2 DG3"}="HN22 DG3"; $resn_exch{"1HC1 DG6"}="HC11 DG6";
$resn_exch{"2HC1 DG6"}="HC12 DG6"; $resn_exch{"1HC2 DG6"}="HC21 DG6"; $resn_exch{"2HC2 DG6"}="HC22 DG6";
$resn_exch{"1HC6 DG6"}="HC61 DG6"; $resn_exch{"2HC6 DG6"}="HC62 DG6"; $resn_exch{"1HA2 DGA"}="HA21 DGA";
$resn_exch{"2HA2 DGA"}="HA22 DGA"; $resn_exch{"1HA3 DGA"}="HA31 DGA"; $resn_exch{"2HA3 DGA"}="HA32 DGA";
$resn_exch{"1HA4 DGA"}="HA41 DGA"; $resn_exch{"2HA4 DGA"}="HA42 DGA"; $resn_exch{"1HA5 DGA"}="HA51 DGA";
$resn_exch{"2HA5 DGA"}="HA52 DGA"; $resn_exch{"1HA6 DGA"}="HA61 DGA"; $resn_exch{"2HA6 DGA"}="HA62 DGA";
$resn_exch{"1HA7 DGA"}="HA71 DGA"; $resn_exch{"2HA7 DGA"}="HA72 DGA"; $resn_exch{"1HA8 DGA"}="HA81 DGA";
$resn_exch{"2HA8 DGA"}="HA82 DGA"; $resn_exch{"1HA9 DGA"}="HA91 DGA"; $resn_exch{"2HA9 DGA"}="HA92 DGA";
$resn_exch{"1HAT DGA"}="HAT1 DGA"; $resn_exch{"2HAT DGA"}="HAT2 DGA"; $resn_exch{"1HAE DGA"}="HAE1 DGA";
$resn_exch{"2HAE DGA"}="HAE2 DGA"; $resn_exch{"1HAW DGA"}="HAW1 DGA"; $resn_exch{"2HAW DGA"}="HAW2 DGA";
$resn_exch{"1HAH DGA"}="HAH1 DGA"; $resn_exch{"2HAH DGA"}="HAH2 DGA"; $resn_exch{"1HAF DGA"}="HAF1 DGA";
$resn_exch{"2HAF DGA"}="HAF2 DGA"; $resn_exch{"1HAN DGA"}="HAN1 DGA"; $resn_exch{"2HAN DGA"}="HAN2 DGA";
$resn_exch{"1HAS DGA"}="HAS1 DGA"; $resn_exch{"2HAS DGA"}="HAS2 DGA"; $resn_exch{"1HAV DGA"}="HAV1 DGA";
$resn_exch{"2HAV DGA"}="HAV2 DGA"; $resn_exch{"1HAG DGA"}="HAG1 DGA"; $resn_exch{"2HAG DGA"}="HAG2 DGA";
$resn_exch{"3HAG DGA"}="HAG3 DGA"; $resn_exch{"1HB2 DGA"}="HB21 DGA"; $resn_exch{"2HB2 DGA"}="HB22 DGA";
$resn_exch{"1HB3 DGA"}="HB31 DGA"; $resn_exch{"2HB3 DGA"}="HB32 DGA"; $resn_exch{"1HB4 DGA"}="HB41 DGA";
$resn_exch{"2HB4 DGA"}="HB42 DGA"; $resn_exch{"1HB5 DGA"}="HB51 DGA"; $resn_exch{"2HB5 DGA"}="HB52 DGA";
$resn_exch{"1HB6 DGA"}="HB61 DGA"; $resn_exch{"2HB6 DGA"}="HB62 DGA"; $resn_exch{"1HB7 DGA"}="HB71 DGA";
$resn_exch{"2HB7 DGA"}="HB72 DGA"; $resn_exch{"1HB8 DGA"}="HB81 DGA"; $resn_exch{"2HB8 DGA"}="HB82 DGA";
$resn_exch{"1HB9 DGA"}="HB91 DGA"; $resn_exch{"2HB9 DGA"}="HB92 DGA"; $resn_exch{"1HBT DGA"}="HBT1 DGA";
$resn_exch{"2HBT DGA"}="HBT2 DGA"; $resn_exch{"1HBE DGA"}="HBE1 DGA"; $resn_exch{"2HBE DGA"}="HBE2 DGA";
$resn_exch{"1HBW DGA"}="HBW1 DGA"; $resn_exch{"2HBW DGA"}="HBW2 DGA"; $resn_exch{"1HBH DGA"}="HBH1 DGA";
$resn_exch{"2HBH DGA"}="HBH2 DGA"; $resn_exch{"1HBF DGA"}="HBF1 DGA"; $resn_exch{"2HBF DGA"}="HBF2 DGA";
$resn_exch{"1HBN DGA"}="HBN1 DGA"; $resn_exch{"2HBN DGA"}="HBN2 DGA"; $resn_exch{"1HBS DGA"}="HBS1 DGA";
$resn_exch{"2HBS DGA"}="HBS2 DGA"; $resn_exch{"1HBV DGA"}="HBV1 DGA"; $resn_exch{"2HBV DGA"}="HBV2 DGA";
$resn_exch{"1HBG DGA"}="HBG1 DGA"; $resn_exch{"2HBG DGA"}="HBG2 DGA"; $resn_exch{"3HBG DGA"}="HBG3 DGA";
$resn_exch{"1HG1 DGA"}="HG11 DGA"; $resn_exch{"2HG1 DGA"}="HG12 DGA"; $resn_exch{"1HG3 DGA"}="HG31 DGA";
$resn_exch{"2HG3 DGA"}="HG32 DGA"; $resn_exch{"1HAU DGB"}="HAU1 DGB"; $resn_exch{"2HAU DGB"}="HAU2 DGB";
$resn_exch{"1HAS DGB"}="HAS1 DGB"; $resn_exch{"2HAS DGB"}="HAS2 DGB"; $resn_exch{"1HAT DGB"}="HAT1 DGB";
$resn_exch{"2HAT DGB"}="HAT2 DGB"; $resn_exch{"1HAR DGB"}="HAR1 DGB"; $resn_exch{"2HAR DGB"}="HAR2 DGB";
$resn_exch{"1HAQ DGB"}="HAQ1 DGB"; $resn_exch{"2HAQ DGB"}="HAQ2 DGB"; $resn_exch{"1HAO DGB"}="HAO1 DGB";
$resn_exch{"2HAO DGB"}="HAO2 DGB"; $resn_exch{"1HAN DGB"}="HAN1 DGB"; $resn_exch{"2HAN DGB"}="HAN2 DGB";
$resn_exch{"1HAP DGB"}="HAP1 DGB"; $resn_exch{"2HAP DGB"}="HAP2 DGB"; $resn_exch{"1HAC DGB"}="HAC1 DGB";
$resn_exch{"2HAC DGB"}="HAC2 DGB"; $resn_exch{"1HAD DGB"}="HAD1 DGB"; $resn_exch{"2HAD DGB"}="HAD2 DGB";
$resn_exch{"1HA2 DGD"}="HA21 DGD"; $resn_exch{"2HA2 DGD"}="HA22 DGD"; $resn_exch{"1HA3 DGD"}="HA31 DGD";
$resn_exch{"2HA3 DGD"}="HA32 DGD"; $resn_exch{"1HA4 DGD"}="HA41 DGD"; $resn_exch{"2HA4 DGD"}="HA42 DGD";
$resn_exch{"1HA5 DGD"}="HA51 DGD"; $resn_exch{"2HA5 DGD"}="HA52 DGD"; $resn_exch{"1HA6 DGD"}="HA61 DGD";
$resn_exch{"2HA6 DGD"}="HA62 DGD"; $resn_exch{"1HA7 DGD"}="HA71 DGD"; $resn_exch{"2HA7 DGD"}="HA72 DGD";
$resn_exch{"1HA8 DGD"}="HA81 DGD"; $resn_exch{"2HA8 DGD"}="HA82 DGD"; $resn_exch{"1HA9 DGD"}="HA91 DGD";
$resn_exch{"2HA9 DGD"}="HA92 DGD"; $resn_exch{"1HAT DGD"}="HAT1 DGD"; $resn_exch{"2HAT DGD"}="HAT2 DGD";
$resn_exch{"1HAE DGD"}="HAE1 DGD"; $resn_exch{"2HAE DGD"}="HAE2 DGD"; $resn_exch{"1HAW DGD"}="HAW1 DGD";
$resn_exch{"2HAW DGD"}="HAW2 DGD"; $resn_exch{"1HAH DGD"}="HAH1 DGD"; $resn_exch{"2HAH DGD"}="HAH2 DGD";
$resn_exch{"1HAF DGD"}="HAF1 DGD"; $resn_exch{"2HAF DGD"}="HAF2 DGD"; $resn_exch{"1HAN DGD"}="HAN1 DGD";
$resn_exch{"2HAN DGD"}="HAN2 DGD"; $resn_exch{"1HAS DGD"}="HAS1 DGD"; $resn_exch{"2HAS DGD"}="HAS2 DGD";
$resn_exch{"1HAV DGD"}="HAV1 DGD"; $resn_exch{"2HAV DGD"}="HAV2 DGD"; $resn_exch{"1HAG DGD"}="HAG1 DGD";
$resn_exch{"2HAG DGD"}="HAG2 DGD"; $resn_exch{"3HAG DGD"}="HAG3 DGD"; $resn_exch{"1HB2 DGD"}="HB21 DGD";
$resn_exch{"2HB2 DGD"}="HB22 DGD"; $resn_exch{"1HB3 DGD"}="HB31 DGD"; $resn_exch{"2HB3 DGD"}="HB32 DGD";
$resn_exch{"1HB4 DGD"}="HB41 DGD"; $resn_exch{"2HB4 DGD"}="HB42 DGD"; $resn_exch{"1HB5 DGD"}="HB51 DGD";
$resn_exch{"2HB5 DGD"}="HB52 DGD"; $resn_exch{"1HB6 DGD"}="HB61 DGD"; $resn_exch{"2HB6 DGD"}="HB62 DGD";
$resn_exch{"1HB7 DGD"}="HB71 DGD"; $resn_exch{"2HB7 DGD"}="HB72 DGD"; $resn_exch{"1HB8 DGD"}="HB81 DGD";
$resn_exch{"2HB8 DGD"}="HB82 DGD"; $resn_exch{"1HB9 DGD"}="HB91 DGD"; $resn_exch{"2HB9 DGD"}="HB92 DGD";
$resn_exch{"1HBT DGD"}="HBT1 DGD"; $resn_exch{"2HBT DGD"}="HBT2 DGD"; $resn_exch{"1HBE DGD"}="HBE1 DGD";
$resn_exch{"2HBE DGD"}="HBE2 DGD"; $resn_exch{"1HBW DGD"}="HBW1 DGD"; $resn_exch{"2HBW DGD"}="HBW2 DGD";
$resn_exch{"1HBH DGD"}="HBH1 DGD"; $resn_exch{"2HBH DGD"}="HBH2 DGD"; $resn_exch{"1HBF DGD"}="HBF1 DGD";
$resn_exch{"2HBF DGD"}="HBF2 DGD"; $resn_exch{"1HBN DGD"}="HBN1 DGD"; $resn_exch{"2HBN DGD"}="HBN2 DGD";
$resn_exch{"1HBS DGD"}="HBS1 DGD"; $resn_exch{"2HBS DGD"}="HBS2 DGD"; $resn_exch{"1HBV DGD"}="HBV1 DGD";
$resn_exch{"2HBV DGD"}="HBV2 DGD"; $resn_exch{"1HBG DGD"}="HBG1 DGD"; $resn_exch{"2HBG DGD"}="HBG2 DGD";
$resn_exch{"3HBG DGD"}="HBG3 DGD"; $resn_exch{"1HG1 DGD"}="HG11 DGD"; $resn_exch{"2HG1 DGD"}="HG12 DGD";
$resn_exch{"1HG3 DGD"}="HG31 DGD"; $resn_exch{"2HG3 DGD"}="HG32 DGD"; $resn_exch{"DHO2 DGD"}="HO2D DGD";
$resn_exch{"DHO3 DGD"}="HO3D DGD"; $resn_exch{"DHO4 DGD"}="HO4D DGD"; $resn_exch{"1HD6 DGD"}="HD61 DGD";
$resn_exch{"2HD6 DGD"}="HD62 DGD"; $resn_exch{"EHO2 DGD"}="HO2E DGD"; $resn_exch{"EHO3 DGD"}="HO3E DGD";
$resn_exch{"EHO4 DGD"}="HO4E DGD"; $resn_exch{"1HE6 DGD"}="HE61 DGD"; $resn_exch{"2HE6 DGD"}="HE62 DGD";
$resn_exch{"EHO5 DGD"}="HO5E DGD"; $resn_exch{"1H16 DGG"}="H161 DGG"; $resn_exch{"2H16 DGG"}="H162 DGG";
$resn_exch{"3H16 DGG"}="H163 DGG"; $resn_exch{"1H15 DGG"}="H151 DGG"; $resn_exch{"2H15 DGG"}="H152 DGG";
$resn_exch{"1H14 DGG"}="H141 DGG"; $resn_exch{"2H14 DGG"}="H142 DGG"; $resn_exch{"1H13 DGG"}="H131 DGG";
$resn_exch{"2H13 DGG"}="H132 DGG"; $resn_exch{"1H12 DGG"}="H121 DGG"; $resn_exch{"2H12 DGG"}="H122 DGG";
$resn_exch{"1H11 DGG"}="H111 DGG"; $resn_exch{"2H11 DGG"}="H112 DGG"; $resn_exch{"1H10 DGG"}="H101 DGG";
$resn_exch{"2H10 DGG"}="H102 DGG"; $resn_exch{"1HC9 DGG"}="HC91 DGG"; $resn_exch{"2HC9 DGG"}="HC92 DGG";
$resn_exch{"1HC8 DGG"}="HC81 DGG"; $resn_exch{"2HC8 DGG"}="HC82 DGG"; $resn_exch{"1HC7 DGG"}="HC71 DGG";
$resn_exch{"2HC7 DGG"}="HC72 DGG"; $resn_exch{"1HC6 DGG"}="HC61 DGG"; $resn_exch{"2HC6 DGG"}="HC62 DGG";
$resn_exch{"1HC5 DGG"}="HC51 DGG"; $resn_exch{"2HC5 DGG"}="HC52 DGG"; $resn_exch{"1HC4 DGG"}="HC41 DGG";
$resn_exch{"2HC4 DGG"}="HC42 DGG"; $resn_exch{"1HC3 DGG"}="HC31 DGG"; $resn_exch{"2HC3 DGG"}="HC32 DGG";
$resn_exch{"1HC2 DGG"}="HC21 DGG"; $resn_exch{"2HC2 DGG"}="HC22 DGG"; $resn_exch{"1H17 DGG"}="H171 DGG";
$resn_exch{"2H17 DGG"}="H172 DGG"; $resn_exch{"1H19 DGG"}="H191 DGG"; $resn_exch{"1H22 DGG"}="H221 DGG";
$resn_exch{"2H22 DGG"}="H222 DGG"; $resn_exch{"1H23 DGG"}="H231 DGG"; $resn_exch{"2H23 DGG"}="H232 DGG";
$resn_exch{"1H24 DGG"}="H241 DGG"; $resn_exch{"2H24 DGG"}="H242 DGG"; $resn_exch{"1H25 DGG"}="H251 DGG";
$resn_exch{"2H25 DGG"}="H252 DGG"; $resn_exch{"1H26 DGG"}="H261 DGG"; $resn_exch{"2H26 DGG"}="H262 DGG";
$resn_exch{"1H27 DGG"}="H271 DGG"; $resn_exch{"2H27 DGG"}="H272 DGG"; $resn_exch{"1H28 DGG"}="H281 DGG";
$resn_exch{"2H28 DGG"}="H282 DGG"; $resn_exch{"1H29 DGG"}="H291 DGG"; $resn_exch{"1H30 DGG"}="H301 DGG";
$resn_exch{"1H39 DGG"}="H391 DGG"; $resn_exch{"2H39 DGG"}="H392 DGG"; $resn_exch{"1H31 DGG"}="H311 DGG";
$resn_exch{"2H31 DGG"}="H312 DGG"; $resn_exch{"1H32 DGG"}="H321 DGG"; $resn_exch{"2H32 DGG"}="H322 DGG";
$resn_exch{"1H33 DGG"}="H331 DGG"; $resn_exch{"2H33 DGG"}="H332 DGG"; $resn_exch{"1H34 DGG"}="H341 DGG";
$resn_exch{"2H34 DGG"}="H342 DGG"; $resn_exch{"1H35 DGG"}="H351 DGG"; $resn_exch{"2H35 DGG"}="H352 DGG";
$resn_exch{"1H36 DGG"}="H361 DGG"; $resn_exch{"2H36 DGG"}="H362 DGG"; $resn_exch{"3H36 DGG"}="H363 DGG";
$resn_exch{"1H18 DGG"}="H181 DGG"; $resn_exch{"2H18 DGG"}="H182 DGG"; $resn_exch{"1HOP DGG"}="HOP1 DGG";
$resn_exch{"1H40 DGG"}="H401 DGG"; $resn_exch{"2H40 DGG"}="H402 DGG"; $resn_exch{"1H41 DGG"}="H411 DGG";
$resn_exch{"1H42 DGG"}="H421 DGG"; $resn_exch{"1H43 DGG"}="H431 DGG"; $resn_exch{"2H43 DGG"}="H432 DGG";
$resn_exch{"1H44 DGG"}="H441 DGG"; $resn_exch{"1HN  DGH"}=" HN1 DGH"; $resn_exch{"2HA  DGH"}=" HA2 DGH";
$resn_exch{"2HOB DGI"}="HOB2 DGI"; $resn_exch{"3HOB DGI"}="HOB3 DGI"; $resn_exch{"2HOA DGI"}="HOA2 DGI";
$resn_exch{"1H5* DGI"}=" H5' DGI"; $resn_exch{"2H5* DGI"}="H5'' DGI"; $resn_exch{" H4* DGI"}=" H4' DGI";
$resn_exch{" H3* DGI"}=" H3' DGI"; $resn_exch{"*HO3 DGI"}="HO3' DGI"; $resn_exch{"1H2* DGI"}=" H2' DGI";
$resn_exch{"2H2* DGI"}="H2'' DGI"; $resn_exch{" H1* DGI"}=" H1' DGI"; $resn_exch{"1HN2 DGI"}="HN21 DGI";
$resn_exch{"2HN2 DGI"}="HN22 DGI"; $resn_exch{" HN2 DGL"}=" H2  DGL"; $resn_exch{"1HB  DGL"}=" HB2 DGL";
$resn_exch{"2HB  DGL"}=" HB3 DGL"; $resn_exch{"1HG  DGL"}=" HG2 DGL"; $resn_exch{"2HG  DGL"}=" HG3 DGL";
$resn_exch{"1H6  DGM"}=" H61 DGM"; $resn_exch{"2H6  DGM"}=" H62 DGM"; $resn_exch{" HN2 DGN"}=" H2  DGN";
$resn_exch{"1HB  DGN"}=" HB2 DGN"; $resn_exch{"2HB  DGN"}=" HB3 DGN"; $resn_exch{"1HG  DGN"}=" HG2 DGN";
$resn_exch{"2HG  DGN"}=" HG3 DGN"; $resn_exch{"1HE2 DGN"}="HE21 DGN"; $resn_exch{"2HE2 DGN"}="HE22 DGN";
$resn_exch{"2HOP DGP"}="HOP2 DGP"; $resn_exch{"3HOP DGP"}="HOP3 DGP"; $resn_exch{"1H5* DGP"}=" H5' DGP";
$resn_exch{"2H5* DGP"}="H5'' DGP"; $resn_exch{" H4* DGP"}=" H4' DGP"; $resn_exch{" H3* DGP"}=" H3' DGP";
$resn_exch{"*HO3 DGP"}="HO3' DGP"; $resn_exch{"1H2* DGP"}=" H2' DGP"; $resn_exch{"2H2* DGP"}="H2'' DGP";
$resn_exch{" H1* DGP"}=" H1' DGP"; $resn_exch{"1HN2 DGP"}="HN21 DGP"; $resn_exch{"2HN2 DGP"}="HN22 DGP";
$resn_exch{"1H6  DGS"}=" H61 DGS"; $resn_exch{"2H6  DGS"}=" H62 DGS"; $resn_exch{"2HOG DGT"}="HOG2 DGT";
$resn_exch{"3HOG DGT"}="HOG3 DGT"; $resn_exch{"2HOB DGT"}="HOB2 DGT"; $resn_exch{"2HOA DGT"}="HOA2 DGT";
$resn_exch{"1H5* DGT"}="H5'1 DGT"; $resn_exch{"2H5* DGT"}="H5'2 DGT"; $resn_exch{" H4* DGT"}=" H4' DGT";
$resn_exch{" H3* DGT"}=" H3' DGT"; $resn_exch{"*HO3 DGT"}="HO3' DGT"; $resn_exch{"1H2* DGT"}="H2'1 DGT";
$resn_exch{"2H2* DGT"}="H2'2 DGT"; $resn_exch{" H1* DGT"}=" H1' DGT"; $resn_exch{"1HN2 DGT"}="HN21 DGT";
$resn_exch{"2HN2 DGT"}="HN22 DGT"; $resn_exch{"1H1  DGX"}=" H11 DGX"; $resn_exch{"2H1  DGX"}="H12A DGX";
$resn_exch{"1H2  DGX"}=" H21 DGX"; $resn_exch{"2H2  DGX"}="H22A DGX"; $resn_exch{"1H4  DGX"}=" H41 DGX";
$resn_exch{"2H4  DGX"}=" H42 DGX"; $resn_exch{"1H6  DGX"}=" H61 DGX"; $resn_exch{"2H6  DGX"}=" H62 DGX";
$resn_exch{"1H7  DGX"}=" H71 DGX"; $resn_exch{"2H7  DGX"}=" H72 DGX"; $resn_exch{"1H11 DGX"}="H111 DGX";
$resn_exch{"2H11 DGX"}="H112 DGX"; $resn_exch{"2HO1 DGX"}="HO12 DGX"; $resn_exch{"4HO1 DGX"}="HO14 DGX";
$resn_exch{"1H15 DGX"}="H151 DGX"; $resn_exch{"2H15 DGX"}="H152 DGX"; $resn_exch{"1H16 DGX"}="H161 DGX";
$resn_exch{"2H16 DGX"}="H162 DGX"; $resn_exch{"1H18 DGX"}="H181 DGX"; $resn_exch{"2H18 DGX"}="H182 DGX";
$resn_exch{"3H18 DGX"}="H183 DGX"; $resn_exch{"1H19 DGX"}="H191 DGX"; $resn_exch{"2H19 DGX"}="H192 DGX";
$resn_exch{"3H19 DGX"}="H193 DGX"; $resn_exch{"1H21 DGX"}="H211 DGX"; $resn_exch{"2H21 DGX"}="H212 DGX";
$resn_exch{"1H2X DGX"}="H2X1 DGX"; $resn_exch{"2H2X DGX"}="H2X2 DGX"; $resn_exch{"3HOX DGX"}="HOX3 DGX";
$resn_exch{"1H6X DGX"}="H6X1 DGX"; $resn_exch{"2H6X DGX"}="H6X2 DGX"; $resn_exch{"3H6X DGX"}="H6X3 DGX";
$resn_exch{"1H2Y DGX"}="H2Y1 DGX"; $resn_exch{"2H2Y DGX"}="H2Y2 DGX"; $resn_exch{"3HOY DGX"}="HOY3 DGX";
$resn_exch{"1H6Y DGX"}="H6Y1 DGX"; $resn_exch{"2H6Y DGX"}="H6Y2 DGX"; $resn_exch{"3H6Y DGX"}="H6Y3 DGX";
$resn_exch{"1H2Z DGX"}="H2Z1 DGX"; $resn_exch{"2H2Z DGX"}="H2Z2 DGX"; $resn_exch{"3HOZ DGX"}="HOZ3 DGX";
$resn_exch{"4HOZ DGX"}="HOZ4 DGX"; $resn_exch{"1H6Z DGX"}="H6Z1 DGX"; $resn_exch{"2H6Z DGX"}="H6Z2 DGX";
$resn_exch{"3H6Z DGX"}="H6Z3 DGX"; $resn_exch{"1H3C DGY"}="H3C1 DGY"; $resn_exch{"2H3C DGY"}="H3C2 DGY";
$resn_exch{"1H16 DH1"}="H161 DH1"; $resn_exch{"2H16 DH1"}="H162 DH1"; $resn_exch{"3H16 DH1"}="H163 DH1";
$resn_exch{"1H52 DH1"}="H521 DH1"; $resn_exch{"2H52 DH1"}="H522 DH1"; $resn_exch{"1H1  DH1"}=" H11 DH1";
$resn_exch{"2H1  DH1"}=" H12 DH1"; $resn_exch{"1H3  DH1"}=" H31 DH1"; $resn_exch{"2H3  DH1"}=" H32 DH1";
$resn_exch{"1H7  DH1"}=" H71 DH1"; $resn_exch{"2H7  DH1"}=" H72 DH1"; $resn_exch{"1H51 DH1"}="H511 DH1";
$resn_exch{"2H51 DH1"}="H512 DH1"; $resn_exch{"1HN2 DH1"}="HN21 DH1"; $resn_exch{"2HN2 DH1"}="HN22 DH1";
$resn_exch{"1HN4 DH1"}="HN41 DH1"; $resn_exch{"2HN4 DH1"}="HN42 DH1"; $resn_exch{"1HN4 DH3"}="HN41 DH3";
$resn_exch{"2HN4 DH3"}="HN42 DH3"; $resn_exch{"1HN2 DH3"}="HN21 DH3"; $resn_exch{"2HN2 DH3"}="HN22 DH3";
$resn_exch{"1H51 DH3"}="H511 DH3"; $resn_exch{"2H51 DH3"}="H512 DH3"; $resn_exch{"1H1  DH3"}=" H11 DH3";
$resn_exch{"2H1  DH3"}=" H12 DH3"; $resn_exch{"1H3  DH3"}=" H31 DH3"; $resn_exch{"2H3  DH3"}=" H32 DH3";
$resn_exch{"1H7  DH3"}=" H71 DH3"; $resn_exch{"2H7  DH3"}=" H72 DH3"; $resn_exch{"1H5* DH3"}="H5'1 DH3";
$resn_exch{"2H5* DH3"}="H5'2 DH3"; $resn_exch{"3H5* DH3"}="H5'3 DH3"; $resn_exch{"1H4* DH3"}="H4'1 DH3";
$resn_exch{"2H4* DH3"}="H4'2 DH3"; $resn_exch{"3H4* DH3"}="H4'3 DH3"; $resn_exch{"1H22 DH9"}="H221 DH9";
$resn_exch{"2H22 DH9"}="H222 DH9"; $resn_exch{"3H22 DH9"}="H223 DH9"; $resn_exch{"1H21 DH9"}="H211 DH9";
$resn_exch{"2H21 DH9"}="H212 DH9"; $resn_exch{"1H20 DH9"}="H201 DH9"; $resn_exch{"2H20 DH9"}="H202 DH9";
$resn_exch{"1H19 DH9"}="H191 DH9"; $resn_exch{"2H19 DH9"}="H192 DH9"; $resn_exch{"1H18 DH9"}="H181 DH9";
$resn_exch{"2H18 DH9"}="H182 DH9"; $resn_exch{"1H17 DH9"}="H171 DH9"; $resn_exch{"2H17 DH9"}="H172 DH9";
$resn_exch{" H2* DH9"}=" H2' DH9"; $resn_exch{"HO2* DH9"}="HO2' DH9"; $resn_exch{" H3* DH9"}=" H3' DH9";
$resn_exch{"HO3* DH9"}="HO3' DH9"; $resn_exch{"1H4* DH9"}="H4'1 DH9"; $resn_exch{"2H4* DH9"}="H4'2 DH9";
$resn_exch{" H5* DH9"}=" H5' DH9"; $resn_exch{"1H26 DH9"}="H261 DH9"; $resn_exch{"2H26 DH9"}="H262 DH9";
$resn_exch{"1H28 DH9"}="H281 DH9"; $resn_exch{"2H28 DH9"}="H282 DH9"; $resn_exch{"3H28 DH9"}="H283 DH9";
$resn_exch{"1H29 DH9"}="H291 DH9"; $resn_exch{"2H29 DH9"}="H292 DH9"; $resn_exch{"3H29 DH9"}="H293 DH9";
$resn_exch{"HN1* DH9"}="HN1' DH9"; $resn_exch{"1H6* DH9"}="H6'1 DH9"; $resn_exch{"2H6* DH9"}="H6'2 DH9";
$resn_exch{"1H7* DH9"}="H7'1 DH9"; $resn_exch{"2H7* DH9"}="H7'2 DH9"; $resn_exch{"1H8* DH9"}="H8'1 DH9";
$resn_exch{"2H8* DH9"}="H8'2 DH9"; $resn_exch{"1H9* DH9"}="H9'1 DH9"; $resn_exch{"2H9* DH9"}="H9'2 DH9";
$resn_exch{"1H10 DH9"}="H101 DH9"; $resn_exch{"2H10 DH9"}="H102 DH9"; $resn_exch{"1H11 DH9"}="H111 DH9";
$resn_exch{"2H11 DH9"}="H112 DH9"; $resn_exch{"1H12 DH9"}="H121 DH9"; $resn_exch{"2H12 DH9"}="H122 DH9";
$resn_exch{"1H13 DH9"}="H131 DH9"; $resn_exch{"2H13 DH9"}="H132 DH9"; $resn_exch{"1H14 DH9"}="H141 DH9";
$resn_exch{"2H14 DH9"}="H142 DH9"; $resn_exch{"1H15 DH9"}="H151 DH9"; $resn_exch{"2H15 DH9"}="H152 DH9";
$resn_exch{"1H16 DH9"}="H161 DH9"; $resn_exch{"2H16 DH9"}="H162 DH9"; $resn_exch{"3H16 DH9"}="H163 DH9";
$resn_exch{"1HN  DHA"}=" H   DHA"; $resn_exch{"2HN  DHA"}=" H2  DHA"; $resn_exch{"2HB  DHA"}=" HB1 DHA";
$resn_exch{"3HB  DHA"}=" HB2 DHA"; $resn_exch{"1H3  DHD"}=" H31 DHD"; $resn_exch{"2H3  DHD"}=" H32 DHD";
$resn_exch{"1HMA DHE"}="HMA1 DHE"; $resn_exch{"2HMA DHE"}="HMA2 DHE"; $resn_exch{"3HMA DHE"}="HMA3 DHE";
$resn_exch{"1HAA DHE"}="HAA1 DHE"; $resn_exch{"2HAA DHE"}="HAA2 DHE"; $resn_exch{"1HBA DHE"}="HBA1 DHE";
$resn_exch{"2HBA DHE"}="HBA2 DHE"; $resn_exch{"1HGB DHE"}="HGB1 DHE"; $resn_exch{"2HGB DHE"}="HGB2 DHE";
$resn_exch{"3HGB DHE"}="HGB3 DHE"; $resn_exch{"1HAB DHE"}="HAB1 DHE"; $resn_exch{"2HAB DHE"}="HAB2 DHE";
$resn_exch{"1HGC DHE"}="HGC1 DHE"; $resn_exch{"2HGC DHE"}="HGC2 DHE"; $resn_exch{"3HGC DHE"}="HGC3 DHE";
$resn_exch{"1HAC DHE"}="HAC1 DHE"; $resn_exch{"2HAC DHE"}="HAC2 DHE"; $resn_exch{"1HMD DHE"}="HMD1 DHE";
$resn_exch{"2HMD DHE"}="HMD2 DHE"; $resn_exch{"3HMD DHE"}="HMD3 DHE"; $resn_exch{"1HAD DHE"}="HAD1 DHE";
$resn_exch{"2HAD DHE"}="HAD2 DHE"; $resn_exch{"1HBD DHE"}="HBD1 DHE"; $resn_exch{"2HBD DHE"}="HBD2 DHE";
$resn_exch{"1HN2 DHF"}="HN21 DHF"; $resn_exch{"2HN2 DHF"}="HN22 DHF"; $resn_exch{"1H7  DHF"}=" H71 DHF";
$resn_exch{"2H7  DHF"}=" H72 DHF"; $resn_exch{"1H9  DHF"}=" H91 DHF"; $resn_exch{"2H9  DHF"}=" H92 DHF";
$resn_exch{"1HB  DHF"}=" HB1 DHF"; $resn_exch{"2HB  DHF"}=" HB2 DHF"; $resn_exch{"1HG  DHF"}=" HG1 DHF";
$resn_exch{"2HG  DHF"}=" HG2 DHF"; $resn_exch{"2HOE DHF"}="HOE2 DHF"; $resn_exch{"1H2  DHG"}=" H21 DHG";
$resn_exch{"2H2  DHG"}=" H22 DHG"; $resn_exch{"1H3  DHG"}=" H31 DHG"; $resn_exch{"2H3  DHG"}=" H32 DHG";
$resn_exch{"1H4  DHG"}=" H41 DHG"; $resn_exch{"2H4  DHG"}=" H42 DHG"; $resn_exch{"1H5  DHG"}=" H51 DHG";
$resn_exch{"2H5  DHG"}=" H52 DHG"; $resn_exch{"1H6  DHG"}=" H61 DHG"; $resn_exch{"2H6  DHG"}=" H62 DHG";
$resn_exch{"1H7  DHG"}=" H71 DHG"; $resn_exch{"2H7  DHG"}=" H72 DHG"; $resn_exch{"1H8  DHG"}=" H81 DHG";
$resn_exch{"2H8  DHG"}=" H82 DHG"; $resn_exch{"1H9  DHG"}=" H91 DHG"; $resn_exch{"2H9  DHG"}=" H92 DHG";
$resn_exch{"1H10 DHG"}="H101 DHG"; $resn_exch{"2H10 DHG"}="H102 DHG"; $resn_exch{"1H11 DHG"}="H111 DHG";
$resn_exch{"2H11 DHG"}="H112 DHG"; $resn_exch{"1H12 DHG"}="H121 DHG"; $resn_exch{"2H12 DHG"}="H122 DHG";
$resn_exch{"3H12 DHG"}="H123 DHG"; $resn_exch{"1HH1 DHG"}="HH11 DHG"; $resn_exch{"2HH1 DHG"}="HH12 DHG";
$resn_exch{"1HH3 DHG"}="HH31 DHG"; $resn_exch{"2HH3 DHG"}="HH32 DHG"; $resn_exch{"1HH4 DHG"}="HH41 DHG";
$resn_exch{"2HH4 DHG"}="HH42 DHG"; $resn_exch{"1HH5 DHG"}="HH51 DHG"; $resn_exch{"2HH5 DHG"}="HH52 DHG";
$resn_exch{"1HH6 DHG"}="HH61 DHG"; $resn_exch{"2HH6 DHG"}="HH62 DHG"; $resn_exch{"3HH6 DHG"}="HH63 DHG";
$resn_exch{"2HOP DHG"}="HOP2 DHG"; $resn_exch{"1HG1 DHG"}="HG11 DHG"; $resn_exch{"2HG1 DHG"}="HG12 DHG";
$resn_exch{"1HG2 DHG"}="HG21 DHG"; $resn_exch{"2HG2 DHG"}="HG22 DHG"; $resn_exch{"2HOG DHG"}="HOG2 DHG";
$resn_exch{"1HP1 DHH"}="HP11 DHH"; $resn_exch{"2HP1 DHH"}="HP12 DHH"; $resn_exch{"1H06 DHH"}="H061 DHH";
$resn_exch{"2H06 DHH"}="H062 DHH"; $resn_exch{"1H07 DHH"}="H071 DHH"; $resn_exch{"2H07 DHH"}="H072 DHH";
$resn_exch{"1H08 DHH"}="H081 DHH"; $resn_exch{"2H08 DHH"}="H082 DHH"; $resn_exch{"1H09 DHH"}="H091 DHH";
$resn_exch{"2H09 DHH"}="H092 DHH"; $resn_exch{" HN2 DHI"}=" H2  DHI"; $resn_exch{"1HB  DHI"}=" HB2 DHI";
$resn_exch{"2HB  DHI"}=" HB3 DHI"; $resn_exch{"1H6  DHJ"}=" H61 DHJ"; $resn_exch{"2H6  DHJ"}=" H62 DHJ";
$resn_exch{"1H4  DHJ"}=" H41 DHJ"; $resn_exch{"2H4  DHJ"}=" H42 DHJ"; $resn_exch{"1H6C DHK"}="H6C1 DHK";
$resn_exch{"2H6C DHK"}="H6C2 DHK"; $resn_exch{"1HN  DHL"}=" HN1 DHL"; $resn_exch{"2HN  DHL"}=" HN2 DHL";
$resn_exch{"2HA  DHL"}=" HA2 DHL"; $resn_exch{"3HA  DHL"}=" HA3 DHL"; $resn_exch{"2HB  DHL"}=" HB2 DHL";
$resn_exch{"3HB  DHL"}=" HB3 DHL"; $resn_exch{"2H7  DHM"}=" H72 DHM"; $resn_exch{"1H7  DHM"}=" H71 DHM";
$resn_exch{"1H6  DHM"}=" H61 DHM"; $resn_exch{"2H4  DHM"}=" H42 DHM"; $resn_exch{"1H4  DHM"}=" H41 DHM";
$resn_exch{"2H3  DHM"}=" H32 DHM"; $resn_exch{"1H3  DHM"}=" H31 DHM"; $resn_exch{"2H2  DHM"}=" H22 DHM";
$resn_exch{"1H2  DHM"}=" H21 DHM"; $resn_exch{"3H8  DHM"}=" H83 DHM"; $resn_exch{"2H8  DHM"}=" H82 DHM";
$resn_exch{"1H8  DHM"}=" H81 DHM"; $resn_exch{"3H9  DHM"}=" H93 DHM"; $resn_exch{"2H9  DHM"}=" H92 DHM";
$resn_exch{"1H9  DHM"}=" H91 DHM"; $resn_exch{"3H10 DHM"}="H103 DHM"; $resn_exch{"2H10 DHM"}="H102 DHM";
$resn_exch{"1H10 DHM"}="H101 DHM"; $resn_exch{"1HN  DHN"}=" HN1 DHN"; $resn_exch{"2HN  DHN"}=" HN2 DHN";
$resn_exch{"1HB  DHN"}=" HB1 DHN"; $resn_exch{"2HB  DHN"}=" HB2 DHN"; $resn_exch{"1HG  DHN"}=" HG1 DHN";
$resn_exch{"2HG  DHN"}=" HG2 DHN"; $resn_exch{"1HD  DHN"}=" HD1 DHN"; $resn_exch{"2HD  DHN"}=" HD2 DHN";
$resn_exch{"1H1  DHO"}=" H11 DHO"; $resn_exch{"2H1  DHO"}="H12A DHO"; $resn_exch{"1H2  DHO"}=" H21 DHO";
$resn_exch{"2H2  DHO"}=" H22 DHO"; $resn_exch{"1H4  DHO"}=" H41 DHO"; $resn_exch{"2H4  DHO"}=" H42 DHO";
$resn_exch{"1H6  DHO"}=" H61 DHO"; $resn_exch{"2H6  DHO"}=" H62 DHO"; $resn_exch{"1H11 DHO"}="H111 DHO";
$resn_exch{"2H11 DHO"}="H112 DHO"; $resn_exch{"1H15 DHO"}="H151 DHO"; $resn_exch{"2H15 DHO"}="H152 DHO";
$resn_exch{"1H16 DHO"}="H161 DHO"; $resn_exch{"2H16 DHO"}="H162 DHO"; $resn_exch{"1H18 DHO"}="H181 DHO";
$resn_exch{"2H18 DHO"}="H182 DHO"; $resn_exch{"3H18 DHO"}="H183 DHO"; $resn_exch{"1H19 DHO"}="H191 DHO";
$resn_exch{"2H19 DHO"}="H192 DHO"; $resn_exch{"3H19 DHO"}="H193 DHO"; $resn_exch{"1H21 DHO"}="H211 DHO";
$resn_exch{"2H21 DHO"}="H212 DHO"; $resn_exch{"3H21 DHO"}="H213 DHO"; $resn_exch{"1H22 DHO"}="H221 DHO";
$resn_exch{"2H22 DHO"}="H222 DHO"; $resn_exch{"1H23 DHO"}="H231 DHO"; $resn_exch{"2H23 DHO"}="H232 DHO";
$resn_exch{"1H5  DHP"}=" H51 DHP"; $resn_exch{"2H5  DHP"}=" H52 DHP"; $resn_exch{"1H6  DHP"}=" H61 DHP";
$resn_exch{"2H6  DHP"}=" H62 DHP"; $resn_exch{"1H7  DHP"}=" H71 DHP"; $resn_exch{"2H7  DHP"}=" H72 DHP";
$resn_exch{"1H8  DHP"}=" H81 DHP"; $resn_exch{"2H8  DHP"}=" H82 DHP"; $resn_exch{"1H9  DHP"}=" H91 DHP";
$resn_exch{"2H9  DHP"}=" H92 DHP"; $resn_exch{"1H10 DHP"}="H101 DHP"; $resn_exch{"2H10 DHP"}="H102 DHP";
$resn_exch{"1H11 DHP"}="H111 DHP"; $resn_exch{"2H11 DHP"}="H112 DHP"; $resn_exch{"1H12 DHP"}="H121 DHP";
$resn_exch{"2H12 DHP"}="H122 DHP"; $resn_exch{"1H13 DHP"}="H131 DHP"; $resn_exch{"2H13 DHP"}="H132 DHP";
$resn_exch{"1H14 DHP"}="H141 DHP"; $resn_exch{"2H14 DHP"}="H142 DHP"; $resn_exch{"3H14 DHP"}="H143 DHP";
$resn_exch{"1H3  DHQ"}=" H31 DHQ"; $resn_exch{"2H3  DHQ"}=" H32 DHQ"; $resn_exch{"1H4  DHQ"}=" H41 DHQ";
$resn_exch{"2H4  DHQ"}=" H42 DHQ"; $resn_exch{"1H9  DHQ"}=" H91 DHQ"; $resn_exch{"2H9  DHQ"}=" H92 DHQ";
$resn_exch{"3H9  DHQ"}=" H93 DHQ"; $resn_exch{"1HN  DHS"}=" HN1 DHS"; $resn_exch{"2HN  DHS"}=" HN2 DHS";
$resn_exch{"1H6  DHS"}=" H61 DHS"; $resn_exch{"2H6  DHS"}=" H62 DHS"; $resn_exch{"1H1  DHT"}=" H11 DHT";
$resn_exch{"2H1  DHT"}=" H12 DHT"; $resn_exch{"1H2  DHT"}=" H21 DHT"; $resn_exch{"2H2  DHT"}=" H22 DHT";
$resn_exch{"1H4  DHT"}=" H41 DHT"; $resn_exch{"2H4  DHT"}=" H42 DHT"; $resn_exch{"1H6  DHT"}=" H61 DHT";
$resn_exch{"2H6  DHT"}=" H62 DHT"; $resn_exch{"1H7  DHT"}=" H71 DHT"; $resn_exch{"2H7  DHT"}=" H72 DHT";
$resn_exch{"1H11 DHT"}="H111 DHT"; $resn_exch{"2H11 DHT"}="H112 DHT"; $resn_exch{"1H12 DHT"}="H121 DHT";
$resn_exch{"2H12 DHT"}="H122 DHT"; $resn_exch{"1H15 DHT"}="H151 DHT"; $resn_exch{"2H15 DHT"}="H152 DHT";
$resn_exch{"1H16 DHT"}="H161 DHT"; $resn_exch{"2H16 DHT"}="H162 DHT"; $resn_exch{"1H18 DHT"}="H181 DHT";
$resn_exch{"2H18 DHT"}="H182 DHT"; $resn_exch{"3H18 DHT"}="H183 DHT"; $resn_exch{"1H19 DHT"}="H191 DHT";
$resn_exch{"2H19 DHT"}="H192 DHT"; $resn_exch{"3H19 DHT"}="H193 DHT"; $resn_exch{"2HOP DHU"}="HOP2 DHU";
$resn_exch{"3HOP DHU"}="HOP3 DHU"; $resn_exch{"1H5* DHU"}=" H5' DHU"; $resn_exch{"2H5* DHU"}="H5'' DHU";
$resn_exch{" H4* DHU"}=" H4' DHU"; $resn_exch{" H3* DHU"}=" H3' DHU"; $resn_exch{"*HO3 DHU"}="HO3' DHU";
$resn_exch{" H2* DHU"}=" H2' DHU"; $resn_exch{"*HO2 DHU"}="HO2' DHU"; $resn_exch{" H1* DHU"}=" H1' DHU";
$resn_exch{"1H5  DHU"}=" H51 DHU"; $resn_exch{"2H5  DHU"}=" H52 DHU"; $resn_exch{"1H6  DHU"}=" H61 DHU";
$resn_exch{"2H6  DHU"}=" H62 DHU"; $resn_exch{"1H7  DHY"}=" H71 DHY"; $resn_exch{"2H7  DHY"}=" H72 DHY";
$resn_exch{"1H4  DHZ"}=" H41 DHZ"; $resn_exch{"2H4  DHZ"}=" H42 DHZ"; $resn_exch{"1H5' DHZ"}="H5'1 DHZ";
$resn_exch{"2H5' DHZ"}="H5'2 DHZ"; $resn_exch{"3HOP  DI"}="HOP3  DI"; $resn_exch{"2HOP  DI"}="HOP2  DI";
$resn_exch{"1H5*  DI"}=" H5'  DI"; $resn_exch{"2H5*  DI"}="H5''  DI"; $resn_exch{" H4*  DI"}=" H4'  DI";
$resn_exch{" H3*  DI"}=" H3'  DI"; $resn_exch{"H3T   DI"}="HO3'  DI"; $resn_exch{"1H2*  DI"}=" H2'  DI";
$resn_exch{"2H2*  DI"}="H2''  DI"; $resn_exch{" H1*  DI"}=" H1'  DI"; $resn_exch{"1H2  DI2"}="H21A DI2";
$resn_exch{"2H2  DI2"}=" H22 DI2"; $resn_exch{"1H3  DI2"}=" H31 DI2"; $resn_exch{"2H3  DI2"}=" H32 DI2";
$resn_exch{"1H4  DI2"}=" H41 DI2"; $resn_exch{"2H4  DI2"}=" H42 DI2"; $resn_exch{"1H5  DI2"}=" H51 DI2";
$resn_exch{"2H5  DI2"}=" H52 DI2"; $resn_exch{"1H8  DI2"}=" H81 DI2"; $resn_exch{"2H8  DI2"}=" H82 DI2";
$resn_exch{"1H9  DI2"}=" H91 DI2"; $resn_exch{"2H9  DI2"}=" H92 DI2"; $resn_exch{"1H10 DI2"}="H101 DI2";
$resn_exch{"2H10 DI2"}="H102 DI2"; $resn_exch{"1H12 DI2"}="H121 DI2"; $resn_exch{"2H12 DI2"}="H122 DI2";
$resn_exch{"1H15 DI2"}="H151 DI2"; $resn_exch{"2H15 DI2"}="H152 DI2"; $resn_exch{"3H15 DI2"}="H153 DI2";
$resn_exch{"1HN1 DI2"}="HN11 DI2"; $resn_exch{"2HN1 DI2"}="HN12 DI2"; $resn_exch{"1H2  DI3"}=" H21 DI3";
$resn_exch{"2H2  DI3"}="H22A DI3"; $resn_exch{"1H3  DI3"}=" H31 DI3"; $resn_exch{"2H3  DI3"}=" H32 DI3";
$resn_exch{"1H4  DI3"}=" H41 DI3"; $resn_exch{"2H4  DI3"}=" H42 DI3"; $resn_exch{"1H5  DI3"}=" H51 DI3";
$resn_exch{"2H5  DI3"}=" H52 DI3"; $resn_exch{"1H9  DI3"}=" H91 DI3"; $resn_exch{"2H9  DI3"}=" H92 DI3";
$resn_exch{"1H10 DI3"}="H101 DI3"; $resn_exch{"2H10 DI3"}="H102 DI3"; $resn_exch{"1H11 DI3"}="H111 DI3";
$resn_exch{"2H11 DI3"}="H112 DI3"; $resn_exch{"1H13 DI3"}="H131 DI3"; $resn_exch{"2H13 DI3"}="H132 DI3";
$resn_exch{"1H16 DI3"}="H161 DI3"; $resn_exch{"2H16 DI3"}="H162 DI3"; $resn_exch{"3H16 DI3"}="H163 DI3";
$resn_exch{"1HN2 DI3"}="HN21 DI3"; $resn_exch{"2HN2 DI3"}="HN22 DI3"; $resn_exch{"1H2  DI4"}=" H21 DI4";
$resn_exch{"2H2  DI4"}="H22A DI4"; $resn_exch{"1H3  DI4"}=" H31 DI4"; $resn_exch{"2H3  DI4"}=" H32 DI4";
$resn_exch{"1H4  DI4"}=" H41 DI4"; $resn_exch{"2H4  DI4"}=" H42 DI4"; $resn_exch{"1H5  DI4"}=" H51 DI4";
$resn_exch{"2H5  DI4"}=" H52 DI4"; $resn_exch{"1H6  DI4"}=" H61 DI4"; $resn_exch{"2H6  DI4"}=" H62 DI4";
$resn_exch{"1H9  DI4"}=" H91 DI4"; $resn_exch{"2H9  DI4"}=" H92 DI4"; $resn_exch{"1H10 DI4"}="H101 DI4";
$resn_exch{"2H10 DI4"}="H102 DI4"; $resn_exch{"1H11 DI4"}="H111 DI4"; $resn_exch{"2H11 DI4"}="H112 DI4";
$resn_exch{"1H13 DI4"}="H131 DI4"; $resn_exch{"2H13 DI4"}="H132 DI4"; $resn_exch{"1H16 DI4"}="H161 DI4";
$resn_exch{"2H16 DI4"}="H162 DI4"; $resn_exch{"3H16 DI4"}="H163 DI4"; $resn_exch{"1HN1 DI4"}="HN11 DI4";
$resn_exch{"2HN1 DI4"}="HN12 DI4"; $resn_exch{"1H2  DI5"}=" H21 DI5"; $resn_exch{"2H2  DI5"}=" H22 DI5";
$resn_exch{"1H3  DI5"}=" H31 DI5"; $resn_exch{"2H3  DI5"}=" H32 DI5"; $resn_exch{"1H4  DI5"}=" H41 DI5";
$resn_exch{"2H4  DI5"}=" H42 DI5"; $resn_exch{"1H7  DI5"}=" H71 DI5"; $resn_exch{"2H7  DI5"}=" H72 DI5";
$resn_exch{"1H8  DI5"}=" H81 DI5"; $resn_exch{"2H8  DI5"}=" H82 DI5"; $resn_exch{"1H9  DI5"}=" H91 DI5";
$resn_exch{"2H9  DI5"}=" H92 DI5"; $resn_exch{"1H11 DI5"}="H111 DI5"; $resn_exch{"2H11 DI5"}="H112 DI5";
$resn_exch{"1H14 DI5"}="H141 DI5"; $resn_exch{"2H14 DI5"}="H142 DI5"; $resn_exch{"3H14 DI5"}="H143 DI5";
$resn_exch{"1HN1 DI5"}="HN11 DI5"; $resn_exch{"2HN1 DI5"}="HN12 DI5"; $resn_exch{"1H6  DI6"}=" H61 DI6";
$resn_exch{"2H6  DI6"}=" H62 DI6"; $resn_exch{"1H7  DI6"}=" H71 DI6"; $resn_exch{"2H7  DI6"}=" H72 DI6";
$resn_exch{"1HAB DI9"}="HAB1 DI9"; $resn_exch{"2HAB DI9"}="HAB2 DI9"; $resn_exch{"1HAJ DI9"}="HAJ1 DI9";
$resn_exch{"2HAJ DI9"}="HAJ2 DI9"; $resn_exch{"1HAK DI9"}="HAK1 DI9"; $resn_exch{"2HAK DI9"}="HAK2 DI9";
$resn_exch{"1HAL DI9"}="HAL1 DI9"; $resn_exch{"2HAL DI9"}="HAL2 DI9"; $resn_exch{"1HAM DI9"}="HAM1 DI9";
$resn_exch{"2HAM DI9"}="HAM2 DI9"; $resn_exch{"1HAH DI9"}="HAH1 DI9"; $resn_exch{"2HAH DI9"}="HAH2 DI9";
$resn_exch{"1HAI DI9"}="HAI1 DI9"; $resn_exch{"2HAI DI9"}="HAI2 DI9"; $resn_exch{"1HAO DI9"}="HAO1 DI9";
$resn_exch{"2HAO DI9"}="HAO2 DI9"; $resn_exch{"1HAN DI9"}="HAN1 DI9"; $resn_exch{"2HAN DI9"}="HAN2 DI9";
$resn_exch{"1HN1 DIA"}="HN11 DIA"; $resn_exch{"2HN1 DIA"}="HN12 DIA"; $resn_exch{"1HC1 DIA"}="HC11 DIA";
$resn_exch{"2HC1 DIA"}="HC12 DIA"; $resn_exch{"1HC2 DIA"}="HC21 DIA"; $resn_exch{"2HC2 DIA"}="HC22 DIA";
$resn_exch{"1HC3 DIA"}="HC31 DIA"; $resn_exch{"2HC3 DIA"}="HC32 DIA"; $resn_exch{"1HC4 DIA"}="HC41 DIA";
$resn_exch{"2HC4 DIA"}="HC42 DIA"; $resn_exch{"1HC5 DIA"}="HC51 DIA"; $resn_exch{"2HC5 DIA"}="HC52 DIA";
$resn_exch{"1HC6 DIA"}="HC61 DIA"; $resn_exch{"2HC6 DIA"}="HC62 DIA"; $resn_exch{"1HC7 DIA"}="HC71 DIA";
$resn_exch{"2HC7 DIA"}="HC72 DIA"; $resn_exch{"1HC8 DIA"}="HC81 DIA"; $resn_exch{"2HC8 DIA"}="HC82 DIA";
$resn_exch{"1HN8 DIA"}="HN81 DIA"; $resn_exch{"2HN8 DIA"}="HN82 DIA"; $resn_exch{"1HA  DIB"}=" HA1 DIB";
$resn_exch{"2HA  DIB"}=" HA2 DIB"; $resn_exch{"1HB  DIB"}=" HB1 DIB"; $resn_exch{"2HB  DIB"}=" HB2 DIB";
$resn_exch{"1HG  DIB"}=" HG1 DIB"; $resn_exch{"2HG  DIB"}=" HG2 DIB"; $resn_exch{"1HE1 DIB"}="HE11 DIB";
$resn_exch{"2HE1 DIB"}="HE12 DIB"; $resn_exch{"3HE1 DIB"}="HE13 DIB"; $resn_exch{"1HE2 DIB"}="HE21 DIB";
$resn_exch{"2HE2 DIB"}="HE22 DIB"; $resn_exch{"3HE2 DIB"}="HE23 DIB"; $resn_exch{"1HN1 DID"}="HN11 DID";
$resn_exch{"2HN1 DID"}="HN12 DID"; $resn_exch{"1H7  DID"}=" H71 DID"; $resn_exch{"2H7  DID"}=" H72 DID";
$resn_exch{"1H8  DID"}=" H81 DID"; $resn_exch{"2H8  DID"}=" H82 DID"; $resn_exch{"1H10 DID"}="H101 DID";
$resn_exch{"2H10 DID"}="H102 DID"; $resn_exch{"1H11 DID"}="H111 DID"; $resn_exch{"2H11 DID"}="H112 DID";
$resn_exch{"1H8' DID"}="H8'1 DID"; $resn_exch{"2H8' DID"}="H8'2 DID"; $resn_exch{"1H7' DID"}="H7'1 DID";
$resn_exch{"2H7' DID"}="H7'2 DID"; $resn_exch{"1H1' DID"}="H1'1 DID"; $resn_exch{"2H1' DID"}="H1'2 DID";
$resn_exch{"1H13 DIF"}="H131 DIF"; $resn_exch{"2H13 DIF"}="H132 DIF"; $resn_exch{"1H1  DIG"}=" H11 DIG";
$resn_exch{"2H1  DIG"}=" H12 DIG"; $resn_exch{"1H6  DIG"}=" H61 DIG"; $resn_exch{"2H6  DIG"}=" H62 DIG";
$resn_exch{"*HO5 DIH"}="HO5' DIH"; $resn_exch{"1H5* DIH"}="H5'1 DIH"; $resn_exch{"2H5* DIH"}="H5'2 DIH";
$resn_exch{" H4* DIH"}=" H4' DIH"; $resn_exch{"1H6* DIH"}="H6'1 DIH"; $resn_exch{"2H6* DIH"}="H6'2 DIH";
$resn_exch{" H3* DIH"}=" H3' DIH"; $resn_exch{"*HO3 DIH"}="HO3' DIH"; $resn_exch{"1H2* DIH"}="H2'1 DIH";
$resn_exch{"2H2* DIH"}="H2'2 DIH"; $resn_exch{" H1* DIH"}=" H1' DIH"; $resn_exch{"1H10 DIH"}="H101 DIH";
$resn_exch{"2H10 DIH"}="H102 DIH"; $resn_exch{" HN2 DIL"}=" H2  DIL"; $resn_exch{"1HG1 DIL"}="HG12 DIL";
$resn_exch{"2HG1 DIL"}="HG13 DIL"; $resn_exch{"1HG2 DIL"}="HG21 DIL"; $resn_exch{"2HG2 DIL"}="HG22 DIL";
$resn_exch{"3HG2 DIL"}="HG23 DIL"; $resn_exch{"1HD1 DIL"}="HD11 DIL"; $resn_exch{"2HD1 DIL"}="HD12 DIL";
$resn_exch{"3HD1 DIL"}="HD13 DIL"; $resn_exch{"1H5  DIM"}=" H51 DIM"; $resn_exch{"2H5  DIM"}=" H52 DIM";
$resn_exch{"3H5  DIM"}=" H53 DIM"; $resn_exch{"1H10 DIM"}="H101 DIM"; $resn_exch{"2H10 DIM"}="H102 DIM";
$resn_exch{"3H10 DIM"}="H103 DIM"; $resn_exch{"1H12 DIM"}="H121 DIM"; $resn_exch{"2H12 DIM"}="H122 DIM";
$resn_exch{"1H13 DIM"}="H131 DIM"; $resn_exch{"2H13 DIM"}="H132 DIM"; $resn_exch{"1HN9 DIM"}="HN91 DIM";
$resn_exch{"2HN9 DIM"}="HN92 DIM"; $resn_exch{"1HN8 DIM"}="HN81 DIM"; $resn_exch{"2HN8 DIM"}="HN82 DIM";
$resn_exch{"1H1  DIO"}=" H11 DIO"; $resn_exch{"2H1  DIO"}=" H12 DIO"; $resn_exch{"1H2  DIO"}=" H21 DIO";
$resn_exch{"2H2  DIO"}=" H22 DIO"; $resn_exch{"1H1' DIO"}="H1'1 DIO"; $resn_exch{"2H1' DIO"}="H1'2 DIO";
$resn_exch{"1H2' DIO"}="H2'1 DIO"; $resn_exch{"2H2' DIO"}="H2'2 DIO"; $resn_exch{"1H1  DIP"}=" H11 DIP";
$resn_exch{"2H1  DIP"}=" H12 DIP"; $resn_exch{"1H2  DIP"}=" H21 DIP"; $resn_exch{"2H2  DIP"}=" H22 DIP";
$resn_exch{"1H3  DIP"}=" H31 DIP"; $resn_exch{"2H3  DIP"}=" H32 DIP"; $resn_exch{"1H4  DIP"}=" H41 DIP";
$resn_exch{"2H4  DIP"}=" H42 DIP"; $resn_exch{"1H5  DIP"}=" H51 DIP"; $resn_exch{"2H5  DIP"}=" H52 DIP";
$resn_exch{"3H5  DIP"}=" H53 DIP"; $resn_exch{"1H1' DIP"}="H1'1 DIP"; $resn_exch{"2H1' DIP"}="H1'2 DIP";
$resn_exch{"1H2' DIP"}="H2'1 DIP"; $resn_exch{"2H2' DIP"}="H2'2 DIP"; $resn_exch{"1H3' DIP"}="H3'1 DIP";
$resn_exch{"2H3' DIP"}="H3'2 DIP"; $resn_exch{"1H4' DIP"}="H4'1 DIP"; $resn_exch{"2H4' DIP"}="H4'2 DIP";
$resn_exch{"1H5' DIP"}="H5'1 DIP"; $resn_exch{"2H5' DIP"}="H5'2 DIP"; $resn_exch{"3H5' DIP"}="H5'3 DIP";
$resn_exch{"1HM  DIQ"}=" HM1 DIQ"; $resn_exch{"2HM  DIQ"}=" HM2 DIQ"; $resn_exch{"3HM  DIQ"}=" HM3 DIQ";
$resn_exch{"1H3  DIQ"}=" H31 DIQ"; $resn_exch{"2H3  DIQ"}=" H32 DIQ"; $resn_exch{"1H4  DIQ"}=" H41 DIQ";
$resn_exch{"2H4  DIQ"}=" H42 DIQ"; $resn_exch{"1H5  DIQ"}=" H51 DIQ"; $resn_exch{"2H5  DIQ"}=" H52 DIQ";
$resn_exch{"1H6  DIQ"}=" H61 DIQ"; $resn_exch{"2H6  DIQ"}=" H62 DIQ"; $resn_exch{"1H7  DIQ"}=" H71 DIQ";
$resn_exch{"2H7  DIQ"}=" H72 DIQ"; $resn_exch{"1H8  DIQ"}=" H81 DIQ"; $resn_exch{"2H8  DIQ"}=" H82 DIQ";
$resn_exch{"1HB  DIR"}=" HB2 DIR"; $resn_exch{"2HB  DIR"}=" HB3 DIR"; $resn_exch{"1HN  DIR"}=" H   DIR";
$resn_exch{"2HN  DIR"}=" H2  DIR"; $resn_exch{" HNG DIR"}=" HG  DIR"; $resn_exch{"1HH2 DIR"}="HH21 DIR";
$resn_exch{"2HH2 DIR"}="HH22 DIR"; $resn_exch{"1H   DIS"}=" H1  DIS"; $resn_exch{"2H   DIS"}=" H2  DIS";
$resn_exch{"1H18 DIT"}="H181 DIT"; $resn_exch{"2H18 DIT"}="H182 DIT"; $resn_exch{"3H18 DIT"}="H183 DIT";
$resn_exch{"1H19 DIT"}="H191 DIT"; $resn_exch{"2H19 DIT"}="H192 DIT"; $resn_exch{"1H20 DIT"}="H201 DIT";
$resn_exch{"2H20 DIT"}="H202 DIT"; $resn_exch{"1H22 DIT"}="H221 DIT"; $resn_exch{"2H22 DIT"}="H222 DIT";
$resn_exch{"1H23 DIT"}="H231 DIT"; $resn_exch{"2H23 DIT"}="H232 DIT"; $resn_exch{"1H25 DIT"}="H251 DIT";
$resn_exch{"2H25 DIT"}="H252 DIT"; $resn_exch{"1H26 DIT"}="H261 DIT"; $resn_exch{"2H26 DIT"}="H262 DIT";
$resn_exch{" XH1 DIT"}=" H1X DIT"; $resn_exch{" XH3 DIT"}=" H3X DIT"; $resn_exch{" XH4 DIT"}=" H4X DIT";
$resn_exch{" XH5 DIT"}=" H5X DIT"; $resn_exch{" XH6 DIT"}=" H6X DIT"; $resn_exch{"XHN7 DIT"}="HN7X DIT";
$resn_exch{" XH8 DIT"}=" H8X DIT"; $resn_exch{" XH9 DIT"}=" H9X DIT"; $resn_exch{"XH11 DIT"}="H11X DIT";
$resn_exch{"1HX8 DIT"}="HX81 DIT"; $resn_exch{"2HX8 DIT"}="HX82 DIT"; $resn_exch{"3HX8 DIT"}="HX83 DIT";
$resn_exch{"1HX9 DIT"}="HX91 DIT"; $resn_exch{"2HX9 DIT"}="HX92 DIT"; $resn_exch{"1HX0 DIT"}="HX01 DIT";
$resn_exch{"2HX0 DIT"}="HX02 DIT"; $resn_exch{"1HX2 DIT"}="HX21 DIT"; $resn_exch{"2HX2 DIT"}="HX22 DIT";
$resn_exch{"1HX3 DIT"}="HX31 DIT"; $resn_exch{"2HX3 DIT"}="HX32 DIT"; $resn_exch{"XH24 DIT"}="H24X DIT";
$resn_exch{"1HX5 DIT"}="HX51 DIT"; $resn_exch{"2HX5 DIT"}="HX52 DIT"; $resn_exch{"1HX6 DIT"}="HX61 DIT";
$resn_exch{"2HX6 DIT"}="HX62 DIT"; $resn_exch{"1HN  DIV"}=" H   DIV"; $resn_exch{"2HN  DIV"}=" H2  DIV";
$resn_exch{"1HB1 DIV"}="HB11 DIV"; $resn_exch{"2HB1 DIV"}="HB12 DIV"; $resn_exch{"1HG1 DIV"}="HG11 DIV";
$resn_exch{"2HG1 DIV"}="HG12 DIV"; $resn_exch{"3HG1 DIV"}="HG13 DIV"; $resn_exch{"1HB2 DIV"}="HB21 DIV";
$resn_exch{"2HB2 DIV"}="HB22 DIV"; $resn_exch{"3HB2 DIV"}="HB23 DIV"; $resn_exch{"1HM  DIX"}=" HM1 DIX";
$resn_exch{"2HM  DIX"}=" HM2 DIX"; $resn_exch{"3HM  DIX"}=" HM3 DIX"; $resn_exch{"1H3  DIX"}=" H31 DIX";
$resn_exch{"2H3  DIX"}=" H32 DIX"; $resn_exch{"1H2  DIX"}=" H21 DIX"; $resn_exch{"2H2  DIX"}=" H22 DIX";
$resn_exch{"1H1  DIX"}=" H11 DIX"; $resn_exch{"2H1  DIX"}=" H12 DIX"; $resn_exch{"1H2' DIX"}="H2'1 DIX";
$resn_exch{"2H2' DIX"}="H2'2 DIX"; $resn_exch{"1H5' DIX"}="H5'1 DIX"; $resn_exch{"2H5' DIX"}="H5'2 DIX";
$resn_exch{"1H4' DIX"}="H4'1 DIX"; $resn_exch{"2H4' DIX"}="H4'2 DIX"; $resn_exch{"1H3' DIX"}="H3'1 DIX";
$resn_exch{"2H3' DIX"}="H3'2 DIX"; $resn_exch{"1H2  DIY"}=" H21 DIY"; $resn_exch{"2H2  DIY"}=" H22 DIY";
$resn_exch{"1H6  DIY"}=" H61 DIY"; $resn_exch{"2H6  DIY"}=" H62 DIY"; $resn_exch{"1H3  DIY"}=" H31 DIY";
$resn_exch{"2H3  DIY"}=" H32 DIY"; $resn_exch{"1H4  DIY"}=" H41 DIY"; $resn_exch{"2H4  DIY"}=" H42 DIY";
$resn_exch{"1H1' DIY"}="H1'1 DIY"; $resn_exch{"2H1' DIY"}="H1'2 DIY"; $resn_exch{"1H2' DIY"}="H2'1 DIY";
$resn_exch{"2H2' DIY"}="H2'2 DIY"; $resn_exch{"1H3' DIY"}="H3'1 DIY"; $resn_exch{"2H3' DIY"}="H3'2 DIY";
$resn_exch{"1H4' DIY"}="H4'1 DIY"; $resn_exch{"2H4' DIY"}="H4'2 DIY"; $resn_exch{"3H4' DIY"}="H4'3 DIY";
$resn_exch{"1H51 DJK"}="H511 DJK"; $resn_exch{"2H51 DJK"}="H512 DJK"; $resn_exch{"1H11 DJK"}="H111 DJK";
$resn_exch{"1H2  DJR"}=" H21 DJR"; $resn_exch{"2H2  DJR"}="H22A DJR"; $resn_exch{"1H1  DJR"}="H11A DJR";
$resn_exch{"2H1  DJR"}="H12A DJR"; $resn_exch{"1H3  DJR"}=" H31 DJR"; $resn_exch{"2H3  DJR"}=" H32 DJR";
$resn_exch{"1H8  DJR"}=" H81 DJR"; $resn_exch{"2H8  DJR"}=" H82 DJR"; $resn_exch{"1H15 DJR"}="H151 DJR";
$resn_exch{"2H15 DJR"}="H152 DJR"; $resn_exch{"1H26 DJR"}="H261 DJR"; $resn_exch{"2H26 DJR"}="H262 DJR";
$resn_exch{"3H26 DJR"}="H263 DJR"; $resn_exch{"1H16 DJR"}="H161 DJR"; $resn_exch{"2H16 DJR"}="H162 DJR";
$resn_exch{"1H25 DJR"}="H251 DJR"; $resn_exch{"2H25 DJR"}="H252 DJR"; $resn_exch{"3H25 DJR"}="H253 DJR";
$resn_exch{"1H24 DJR"}="H241 DJR"; $resn_exch{"2H24 DJR"}="H242 DJR"; $resn_exch{"3H24 DJR"}="H243 DJR";
$resn_exch{"1H16 DK2"}="H161 DK2"; $resn_exch{"2H16 DK2"}="H162 DK2"; $resn_exch{"1H2  DKA"}=" H21 DKA";
$resn_exch{"2H2  DKA"}=" H22 DKA"; $resn_exch{"1H3  DKA"}=" H31 DKA"; $resn_exch{"2H3  DKA"}=" H32 DKA";
$resn_exch{"1H4  DKA"}=" H41 DKA"; $resn_exch{"2H4  DKA"}=" H42 DKA"; $resn_exch{"1H5  DKA"}=" H51 DKA";
$resn_exch{"2H5  DKA"}=" H52 DKA"; $resn_exch{"1H6  DKA"}=" H61 DKA"; $resn_exch{"2H6  DKA"}=" H62 DKA";
$resn_exch{"1H7  DKA"}=" H71 DKA"; $resn_exch{"2H7  DKA"}=" H72 DKA"; $resn_exch{"1H8  DKA"}=" H81 DKA";
$resn_exch{"2H8  DKA"}=" H82 DKA"; $resn_exch{"1H9  DKA"}=" H91 DKA"; $resn_exch{"2H9  DKA"}=" H92 DKA";
$resn_exch{"1H10 DKA"}="H101 DKA"; $resn_exch{"2H10 DKA"}="H102 DKA"; $resn_exch{"3H10 DKA"}="H103 DKA";
$resn_exch{"1HAA DKI"}="HAA1 DKI"; $resn_exch{"2HAA DKI"}="HAA2 DKI"; $resn_exch{"1HAB DKI"}="HAB1 DKI";
$resn_exch{"2HAB DKI"}="HAB2 DKI"; $resn_exch{"1HC2 DKT"}="HC21 DKT"; $resn_exch{"2HC2 DKT"}="HC22 DKT";
$resn_exch{"1HB1 DKT"}="HB11 DKT"; $resn_exch{"2HB1 DKT"}="HB12 DKT"; $resn_exch{"1HD1 DKT"}="HD11 DKT";
$resn_exch{"2HD1 DKT"}="HD12 DKT"; $resn_exch{"1HD2 DKT"}="HD21 DKT"; $resn_exch{"2HD2 DKT"}="HD22 DKT";
$resn_exch{"1HE1 DKT"}="HE11 DKT"; $resn_exch{"2HE1 DKT"}="HE12 DKT"; $resn_exch{"1HE2 DKT"}="HE21 DKT";
$resn_exch{"2HE2 DKT"}="HE22 DKT"; $resn_exch{"1HZ1 DKT"}="HZ11 DKT"; $resn_exch{"2HZ1 DKT"}="HZ12 DKT";
$resn_exch{"1HB2 DKT"}="HB21 DKT"; $resn_exch{"2HB2 DKT"}="HB22 DKT"; $resn_exch{"1HG2 DKT"}="HG21 DKT";
$resn_exch{"2HG2 DKT"}="HG22 DKT"; $resn_exch{"1HD3 DKT"}="HD31 DKT"; $resn_exch{"2HD3 DKT"}="HD32 DKT";
$resn_exch{"1HH1 DKT"}="HH11 DKT"; $resn_exch{"2HH1 DKT"}="HH12 DKT"; $resn_exch{"1HH2 DKT"}="HH21 DKT";
$resn_exch{"1HB3 DKT"}="HB31 DKT"; $resn_exch{"2HB3 DKT"}="HB32 DKT"; $resn_exch{"1HG3 DKT"}="HG31 DKT";
$resn_exch{"2HG3 DKT"}="HG32 DKT"; $resn_exch{"1HB4 DKT"}="HB41 DKT"; $resn_exch{"2HB4 DKT"}="HB42 DKT";
$resn_exch{"1HD5 DKT"}="HD51 DKT"; $resn_exch{"2HD5 DKT"}="HD52 DKT"; $resn_exch{"1HD6 DKT"}="HD61 DKT";
$resn_exch{"2HD6 DKT"}="HD62 DKT"; $resn_exch{"1HE6 DKT"}="HE61 DKT"; $resn_exch{"2HE6 DKT"}="HE62 DKT";
$resn_exch{"1HE7 DKT"}="HE71 DKT"; $resn_exch{"2HE7 DKT"}="HE72 DKT"; $resn_exch{"1HZ3 DKT"}="HZ31 DKT";
$resn_exch{"2HZ3 DKT"}="HZ32 DKT"; $resn_exch{"1H6  DL6"}=" H61 DL6"; $resn_exch{"2H6  DL6"}=" H62 DL6";
$resn_exch{"1H8  DL6"}=" H81 DL6"; $resn_exch{"2H8  DL6"}=" H82 DL6"; $resn_exch{"1H8  DL8"}=" H81 DL8";
$resn_exch{"2H8  DL8"}=" H82 DL8"; $resn_exch{"1H6  DL8"}=" H61 DL8"; $resn_exch{"2H6  DL8"}=" H62 DL8";
$resn_exch{"1H3  DLA"}=" H31 DLA"; $resn_exch{"2H3  DLA"}=" H32 DLA"; $resn_exch{"3H3  DLA"}=" H33 DLA";
$resn_exch{" HN2 DLE"}=" H2  DLE"; $resn_exch{"1HB  DLE"}=" HB2 DLE"; $resn_exch{"2HB  DLE"}=" HB3 DLE";
$resn_exch{"1HD1 DLE"}="HD11 DLE"; $resn_exch{"2HD1 DLE"}="HD12 DLE"; $resn_exch{"3HD1 DLE"}="HD13 DLE";
$resn_exch{"1HD2 DLE"}="HD21 DLE"; $resn_exch{"2HD2 DLE"}="HD22 DLE"; $resn_exch{"3HD2 DLE"}="HD23 DLE";
$resn_exch{"1H2  DLF"}=" H21 DLF"; $resn_exch{"2H2  DLF"}=" H22 DLF"; $resn_exch{"1H6  DLF"}=" H61 DLF";
$resn_exch{"2H6  DLF"}=" H62 DLF"; $resn_exch{"3H6  DLF"}=" H63 DLF"; $resn_exch{"1H3  DLG"}=" H31 DLG";
$resn_exch{"2H3  DLG"}=" H32 DLG"; $resn_exch{"1H6  DLG"}=" H61 DLG"; $resn_exch{"2H6  DLG"}=" H62 DLG";
$resn_exch{"1H11 DLG"}="H111 DLG"; $resn_exch{"2H11 DLG"}="H112 DLG"; $resn_exch{"1H12 DLG"}="H121 DLG";
$resn_exch{"2H12 DLG"}="H122 DLG"; $resn_exch{"1H13 DLG"}="H131 DLG"; $resn_exch{"2H13 DLG"}="H132 DLG";
$resn_exch{"1H14 DLG"}="H141 DLG"; $resn_exch{"2H14 DLG"}="H142 DLG"; $resn_exch{"1H15 DLG"}="H151 DLG";
$resn_exch{"2H15 DLG"}="H152 DLG"; $resn_exch{"1H16 DLG"}="H161 DLG"; $resn_exch{"2H16 DLG"}="H162 DLG";
$resn_exch{"3H16 DLG"}="H163 DLG"; $resn_exch{"1H8  DLI"}=" H81 DLI"; $resn_exch{"2H8  DLI"}=" H82 DLI";
$resn_exch{"1H10 DLI"}="H101 DLI"; $resn_exch{"2H10 DLI"}="H102 DLI"; $resn_exch{"3H10 DLI"}="H103 DLI";
$resn_exch{"1H12 DLI"}="H121 DLI"; $resn_exch{"2H12 DLI"}="H122 DLI"; $resn_exch{"1H22 DLI"}="H221 DLI";
$resn_exch{"2H22 DLI"}="H222 DLI"; $resn_exch{"1HC1 DLP"}="HC11 DLP"; $resn_exch{"2HC1 DLP"}="HC12 DLP";
$resn_exch{"1HC3 DLP"}="HC31 DLP"; $resn_exch{"2HC3 DLP"}="HC32 DLP"; $resn_exch{"1HC4 DLP"}="HC41 DLP";
$resn_exch{"2HC4 DLP"}="HC42 DLP"; $resn_exch{"1HC5 DLP"}="HC51 DLP"; $resn_exch{"2HC5 DLP"}="HC52 DLP";
$resn_exch{"1HC6 DLP"}="HC61 DLP"; $resn_exch{"2HC6 DLP"}="HC62 DLP"; $resn_exch{"3HC6 DLP"}="HC63 DLP";
$resn_exch{"1HC7 DLP"}="HC71 DLP"; $resn_exch{"2HC7 DLP"}="HC72 DLP"; $resn_exch{"3HC7 DLP"}="HC73 DLP";
$resn_exch{"1HC8 DLP"}="HC81 DLP"; $resn_exch{"2HC8 DLP"}="HC82 DLP"; $resn_exch{"3HC8 DLP"}="HC83 DLP";
$resn_exch{"1H12 DLP"}="H121 DLP"; $resn_exch{"2H12 DLP"}="H122 DLP"; $resn_exch{"1H13 DLP"}="H131 DLP";
$resn_exch{"2H13 DLP"}="H132 DLP"; $resn_exch{"1H14 DLP"}="H141 DLP"; $resn_exch{"2H14 DLP"}="H142 DLP";
$resn_exch{"1H15 DLP"}="H151 DLP"; $resn_exch{"2H15 DLP"}="H152 DLP"; $resn_exch{"1H16 DLP"}="H161 DLP";
$resn_exch{"2H16 DLP"}="H162 DLP"; $resn_exch{"1H17 DLP"}="H171 DLP"; $resn_exch{"2H17 DLP"}="H172 DLP";
$resn_exch{"1H18 DLP"}="H181 DLP"; $resn_exch{"2H18 DLP"}="H182 DLP"; $resn_exch{"1H21 DLP"}="H211 DLP";
$resn_exch{"2H21 DLP"}="H212 DLP"; $resn_exch{"1H24 DLP"}="H241 DLP"; $resn_exch{"2H24 DLP"}="H242 DLP";
$resn_exch{"1H25 DLP"}="H251 DLP"; $resn_exch{"2H25 DLP"}="H252 DLP"; $resn_exch{"1H26 DLP"}="H261 DLP";
$resn_exch{"2H26 DLP"}="H262 DLP"; $resn_exch{"1H27 DLP"}="H271 DLP"; $resn_exch{"2H27 DLP"}="H272 DLP";
$resn_exch{"1H28 DLP"}="H281 DLP"; $resn_exch{"2H28 DLP"}="H282 DLP"; $resn_exch{"3H28 DLP"}="H283 DLP";
$resn_exch{"1H32 DLP"}="H321 DLP"; $resn_exch{"2H32 DLP"}="H322 DLP"; $resn_exch{"1H33 DLP"}="H331 DLP";
$resn_exch{"2H33 DLP"}="H332 DLP"; $resn_exch{"1H34 DLP"}="H341 DLP"; $resn_exch{"2H34 DLP"}="H342 DLP";
$resn_exch{"1H35 DLP"}="H351 DLP"; $resn_exch{"2H35 DLP"}="H352 DLP"; $resn_exch{"1H36 DLP"}="H361 DLP";
$resn_exch{"2H36 DLP"}="H362 DLP"; $resn_exch{"1H37 DLP"}="H371 DLP"; $resn_exch{"2H37 DLP"}="H372 DLP";
$resn_exch{"1H38 DLP"}="H381 DLP"; $resn_exch{"2H38 DLP"}="H382 DLP"; $resn_exch{"1H41 DLP"}="H411 DLP";
$resn_exch{"2H41 DLP"}="H412 DLP"; $resn_exch{"1H44 DLP"}="H441 DLP"; $resn_exch{"2H44 DLP"}="H442 DLP";
$resn_exch{"1H45 DLP"}="H451 DLP"; $resn_exch{"2H45 DLP"}="H452 DLP"; $resn_exch{"1H46 DLP"}="H461 DLP";
$resn_exch{"2H46 DLP"}="H462 DLP"; $resn_exch{"1H47 DLP"}="H471 DLP"; $resn_exch{"2H47 DLP"}="H472 DLP";
$resn_exch{"1H48 DLP"}="H481 DLP"; $resn_exch{"2H48 DLP"}="H482 DLP"; $resn_exch{"3H48 DLP"}="H483 DLP";
$resn_exch{"1H3  DLQ"}=" H31 DLQ"; $resn_exch{"2H3  DLQ"}=" H32 DLQ"; $resn_exch{"3H3  DLQ"}="H33A DLQ";
$resn_exch{"1H5  DLQ"}=" H51 DLQ"; $resn_exch{"2H5  DLQ"}=" H52 DLQ"; $resn_exch{"1H7  DLQ"}=" H71 DLQ";
$resn_exch{"2H7  DLQ"}=" H72 DLQ"; $resn_exch{"3H7  DLQ"}=" H73 DLQ"; $resn_exch{"1H8  DLQ"}=" H81 DLQ";
$resn_exch{"2H8  DLQ"}=" H82 DLQ"; $resn_exch{"3H8  DLQ"}=" H83 DLQ"; $resn_exch{"1H14 DLQ"}="H141 DLQ";
$resn_exch{"2H14 DLQ"}="H142 DLQ"; $resn_exch{"3H14 DLQ"}="H143 DLQ"; $resn_exch{"1H17 DLQ"}="H171 DLQ";
$resn_exch{"2H17 DLQ"}="H172 DLQ"; $resn_exch{"3H17 DLQ"}="H173 DLQ"; $resn_exch{"1H18 DLQ"}="H181 DLQ";
$resn_exch{"2H18 DLQ"}="H182 DLQ"; $resn_exch{"3H18 DLQ"}="H183 DLQ"; $resn_exch{"1H19 DLQ"}="H191 DLQ";
$resn_exch{"2H19 DLQ"}="H192 DLQ"; $resn_exch{"3H19 DLQ"}="H193 DLQ"; $resn_exch{"1H20 DLQ"}="H201 DLQ";
$resn_exch{"2H20 DLQ"}="H202 DLQ"; $resn_exch{"3H20 DLQ"}="H203 DLQ"; $resn_exch{"1H21 DLQ"}="H211 DLQ";
$resn_exch{"2H21 DLQ"}="H212 DLQ"; $resn_exch{"3H21 DLQ"}="H213 DLQ"; $resn_exch{"1H25 DLQ"}="H251 DLQ";
$resn_exch{"2H25 DLQ"}="H252 DLQ"; $resn_exch{"1H26 DLQ"}="H261 DLQ"; $resn_exch{"2H26 DLQ"}="H262 DLQ";
$resn_exch{"1H28 DLQ"}="H281 DLQ"; $resn_exch{"2H28 DLQ"}="H282 DLQ"; $resn_exch{"1H31 DLQ"}="H311 DLQ";
$resn_exch{"2H31 DLQ"}="H312 DLQ"; $resn_exch{"1H40 DLQ"}="H401 DLQ"; $resn_exch{"2H40 DLQ"}="H402 DLQ";
$resn_exch{"1H41 DLQ"}="H411 DLQ"; $resn_exch{"2H41 DLQ"}="H412 DLQ"; $resn_exch{"1H45 DLQ"}="H451 DLQ";
$resn_exch{"2H45 DLQ"}="H452 DLQ"; $resn_exch{"3H45 DLQ"}="H453 DLQ"; $resn_exch{"1H46 DLQ"}="H461 DLQ";
$resn_exch{"2H46 DLQ"}="H462 DLQ"; $resn_exch{"3H46 DLQ"}="H463 DLQ"; $resn_exch{"1H2  DLS"}=" H21 DLS";
$resn_exch{"2H2  DLS"}=" H22 DLS"; $resn_exch{"3H2  DLS"}=" H23 DLS"; $resn_exch{" HN  DLS"}=" H   DLS";
$resn_exch{"1HCB DLS"}=" HB2 DLS"; $resn_exch{"2HCB DLS"}=" HB3 DLS"; $resn_exch{"1HCG DLS"}=" HG2 DLS";
$resn_exch{"2HCG DLS"}=" HG3 DLS"; $resn_exch{"1HCD DLS"}=" HD2 DLS"; $resn_exch{"2HCD DLS"}=" HD3 DLS";
$resn_exch{"1HCE DLS"}=" HE2 DLS"; $resn_exch{"2HCE DLS"}=" HE3 DLS"; $resn_exch{" HNZ DLS"}=" HZ  DLS";
$resn_exch{"1HH3 DLS"}="HH31 DLS"; $resn_exch{"2HH3 DLS"}="HH32 DLS"; $resn_exch{"3HH3 DLS"}="HH33 DLS";
$resn_exch{" HN2 DLY"}=" H2  DLY"; $resn_exch{"1HB  DLY"}=" HB2 DLY"; $resn_exch{"2HB  DLY"}=" HB3 DLY";
$resn_exch{"1HG  DLY"}=" HG2 DLY"; $resn_exch{"2HG  DLY"}=" HG3 DLY"; $resn_exch{"1HD  DLY"}=" HD2 DLY";
$resn_exch{"2HD  DLY"}=" HD3 DLY"; $resn_exch{"1HE  DLY"}=" HE2 DLY"; $resn_exch{"2HE  DLY"}=" HE3 DLY";
$resn_exch{"1HZ  DLY"}=" HZ1 DLY"; $resn_exch{"2HZ  DLY"}=" HZ2 DLY"; $resn_exch{"1H11 DM1"}="H111 DM1";
$resn_exch{"2H11 DM1"}="H112 DM1"; $resn_exch{"2HO1 DM1"}="HO12 DM1"; $resn_exch{"1H14 DM1"}="H141 DM1";
$resn_exch{"2H14 DM1"}="H142 DM1"; $resn_exch{"3H14 DM1"}="H143 DM1"; $resn_exch{"1H15 DM1"}="H151 DM1";
$resn_exch{"2H15 DM1"}="H152 DM1"; $resn_exch{"7HO1 DM1"}="HO17 DM1"; $resn_exch{"1H21 DM1"}="H211 DM1";
$resn_exch{"2H21 DM1"}="H212 DM1"; $resn_exch{"3H21 DM1"}="H213 DM1"; $resn_exch{" H1* DM1"}=" H1' DM1";
$resn_exch{"1H2* DM1"}="H2'1 DM1"; $resn_exch{"2H2* DM1"}="H2'2 DM1"; $resn_exch{" H3* DM1"}=" H3' DM1";
$resn_exch{"1HN* DM1"}="HN'1 DM1"; $resn_exch{"2HN* DM1"}="HN'2 DM1"; $resn_exch{" H4* DM1"}=" H4' DM1";
$resn_exch{"*HO4 DM1"}="HO4' DM1"; $resn_exch{" H5* DM1"}=" H5' DM1"; $resn_exch{"1H6* DM1"}="H6'1 DM1";
$resn_exch{"2H6* DM1"}="H6'2 DM1"; $resn_exch{"3H6* DM1"}="H6'3 DM1"; $resn_exch{"1H11 DM2"}="H111 DM2";
$resn_exch{"2H11 DM2"}="H112 DM2"; $resn_exch{"2HO1 DM2"}="HO12 DM2"; $resn_exch{"1H14 DM2"}="H141 DM2";
$resn_exch{"2H14 DM2"}="H142 DM2"; $resn_exch{"4HO1 DM2"}="HO14 DM2"; $resn_exch{"1H15 DM2"}="H151 DM2";
$resn_exch{"2H15 DM2"}="H152 DM2"; $resn_exch{"7HO1 DM2"}="HO17 DM2"; $resn_exch{"1H21 DM2"}="H211 DM2";
$resn_exch{"2H21 DM2"}="H212 DM2"; $resn_exch{"3H21 DM2"}="H213 DM2"; $resn_exch{" H1* DM2"}=" H1' DM2";
$resn_exch{"1H2* DM2"}="H2'1 DM2"; $resn_exch{"2H2* DM2"}="H2'2 DM2"; $resn_exch{" H3* DM2"}=" H3' DM2";
$resn_exch{"1HN* DM2"}="HN'1 DM2"; $resn_exch{"2HN* DM2"}="HN'2 DM2"; $resn_exch{" H4* DM2"}=" H4' DM2";
$resn_exch{"*HO4 DM2"}="HO4' DM2"; $resn_exch{" H5* DM2"}=" H5' DM2"; $resn_exch{"1H6* DM2"}="H6'1 DM2";
$resn_exch{"2H6* DM2"}="H6'2 DM2"; $resn_exch{"3H6* DM2"}="H6'3 DM2"; $resn_exch{"1H11 DM3"}="H111 DM3";
$resn_exch{"2H11 DM3"}="H112 DM3"; $resn_exch{"2HO1 DM3"}="HO12 DM3"; $resn_exch{"1H14 DM3"}="H141 DM3";
$resn_exch{"2H14 DM3"}="H142 DM3"; $resn_exch{"3H14 DM3"}="H143 DM3"; $resn_exch{"1H15 DM3"}="H151 DM3";
$resn_exch{"2H15 DM3"}="H152 DM3"; $resn_exch{"1H21 DM3"}="H211 DM3"; $resn_exch{"2H21 DM3"}="H212 DM3";
$resn_exch{"3H21 DM3"}="H213 DM3"; $resn_exch{" H1* DM3"}=" H1' DM3"; $resn_exch{"1H2* DM3"}="H2'1 DM3";
$resn_exch{"2H2* DM3"}="H2'2 DM3"; $resn_exch{" H3* DM3"}=" H3' DM3"; $resn_exch{"1HN* DM3"}="HN'1 DM3";
$resn_exch{"2HN* DM3"}="HN'2 DM3"; $resn_exch{" H4* DM3"}=" H4' DM3"; $resn_exch{"*HO4 DM3"}="HO4' DM3";
$resn_exch{" H5* DM3"}=" H5' DM3"; $resn_exch{"1H6* DM3"}="H6'1 DM3"; $resn_exch{"2H6* DM3"}="H6'2 DM3";
$resn_exch{"3H6* DM3"}="H6'3 DM3"; $resn_exch{"1H11 DM4"}="H111 DM4"; $resn_exch{"2H11 DM4"}="H112 DM4";
$resn_exch{"2HO1 DM4"}="HO12 DM4"; $resn_exch{"1H14 DM4"}="H141 DM4"; $resn_exch{"2H14 DM4"}="H142 DM4";
$resn_exch{"4HO1 DM4"}="HO14 DM4"; $resn_exch{"1H15 DM4"}="H151 DM4"; $resn_exch{"2H15 DM4"}="H152 DM4";
$resn_exch{" H1* DM4"}=" H1' DM4"; $resn_exch{"1H2* DM4"}="H2'1 DM4"; $resn_exch{"2H2* DM4"}="H2'2 DM4";
$resn_exch{" H3* DM4"}=" H3' DM4"; $resn_exch{"1HN* DM4"}="HN'1 DM4"; $resn_exch{"2HN* DM4"}="HN'2 DM4";
$resn_exch{" H4* DM4"}=" H4' DM4"; $resn_exch{"*HO4 DM4"}="HO4' DM4"; $resn_exch{" H5* DM4"}=" H5' DM4";
$resn_exch{"1H6* DM4"}="H6'1 DM4"; $resn_exch{"2H6* DM4"}="H6'2 DM4"; $resn_exch{"3H6* DM4"}="H6'3 DM4";
$resn_exch{"1H11 DM5"}="H111 DM5"; $resn_exch{"2H11 DM5"}="H112 DM5"; $resn_exch{"2HO1 DM5"}="HO12 DM5";
$resn_exch{"1H14 DM5"}="H141 DM5"; $resn_exch{"2H14 DM5"}="H142 DM5"; $resn_exch{"3H14 DM5"}="H143 DM5";
$resn_exch{"1H15 DM5"}="H151 DM5"; $resn_exch{"2H15 DM5"}="H152 DM5"; $resn_exch{"7HO1 DM5"}="HO17 DM5";
$resn_exch{" H1* DM5"}=" H1' DM5"; $resn_exch{"1H2* DM5"}="H2'1 DM5"; $resn_exch{"2H2* DM5"}="H2'2 DM5";
$resn_exch{" H3* DM5"}=" H3' DM5"; $resn_exch{"1HN* DM5"}="HN'1 DM5"; $resn_exch{"2HN* DM5"}="HN'2 DM5";
$resn_exch{" H4* DM5"}=" H4' DM5"; $resn_exch{"*HO4 DM5"}="HO4' DM5"; $resn_exch{" H5* DM5"}=" H5' DM5";
$resn_exch{"1H6* DM5"}="H6'1 DM5"; $resn_exch{"2H6* DM5"}="H6'2 DM5"; $resn_exch{"3H6* DM5"}="H6'3 DM5";
$resn_exch{"1H8  DM6"}=" H81 DM6"; $resn_exch{"2H8  DM6"}=" H82 DM6"; $resn_exch{"1H10 DM6"}="H101 DM6";
$resn_exch{"2H10 DM6"}="H102 DM6"; $resn_exch{"1H14 DM6"}="H141 DM6"; $resn_exch{"2H14 DM6"}="H142 DM6";
$resn_exch{"1H21 DM6"}="H211 DM6"; $resn_exch{"2H21 DM6"}="H212 DM6"; $resn_exch{"3H21 DM6"}="H213 DM6";
$resn_exch{" H1* DM6"}=" H1' DM6"; $resn_exch{"1H2* DM6"}="H2'1 DM6"; $resn_exch{"2H2* DM6"}="H2'2 DM6";
$resn_exch{" H3* DM6"}=" H3' DM6"; $resn_exch{" H4* DM6"}=" H4' DM6"; $resn_exch{" H5* DM6"}=" H5' DM6";
$resn_exch{"1H6* DM6"}="H6'1 DM6"; $resn_exch{"2H6* DM6"}="H6'2 DM6"; $resn_exch{"3H6* DM6"}="H6'3 DM6";
$resn_exch{"4HO* DM6"}="HO'4 DM6"; $resn_exch{"1HN* DM6"}="HN'1 DM6"; $resn_exch{"2HN* DM6"}="HN'2 DM6";
$resn_exch{"3HN* DM6"}="HN'3 DM6"; $resn_exch{"1H8  DM7"}=" H81 DM7"; $resn_exch{"2H8  DM7"}=" H82 DM7";
$resn_exch{"1H10 DM7"}="H101 DM7"; $resn_exch{"2H10 DM7"}="H102 DM7"; $resn_exch{"1H14 DM7"}="H141 DM7";
$resn_exch{"2H14 DM7"}="H142 DM7"; $resn_exch{"1H21 DM7"}="H211 DM7"; $resn_exch{"2H21 DM7"}="H212 DM7";
$resn_exch{"3H21 DM7"}="H213 DM7"; $resn_exch{" H1* DM7"}=" H1' DM7"; $resn_exch{"1H2* DM7"}="H2'1 DM7";
$resn_exch{"2H2* DM7"}="H2'2 DM7"; $resn_exch{" H3* DM7"}=" H3' DM7"; $resn_exch{" H4* DM7"}=" H4' DM7";
$resn_exch{" H5* DM7"}=" H5' DM7"; $resn_exch{"1H6* DM7"}="H6'1 DM7"; $resn_exch{"2H6* DM7"}="H6'2 DM7";
$resn_exch{"3H6* DM7"}="H6'3 DM7"; $resn_exch{"1HN* DM7"}="HN'1 DM7"; $resn_exch{"2HN* DM7"}="HN'2 DM7";
$resn_exch{"3HN* DM7"}="HN'3 DM7"; $resn_exch{"1H8  DM8"}=" H81 DM8"; $resn_exch{"2H8  DM8"}=" H82 DM8";
$resn_exch{"1H10 DM8"}="H101 DM8"; $resn_exch{"2H10 DM8"}="H102 DM8"; $resn_exch{"1H14 DM8"}="H141 DM8";
$resn_exch{"2H14 DM8"}="H142 DM8"; $resn_exch{"3H14 DM8"}="H143 DM8"; $resn_exch{"1H21 DM8"}="H211 DM8";
$resn_exch{"2H21 DM8"}="H212 DM8"; $resn_exch{"3H21 DM8"}="H213 DM8"; $resn_exch{" H1* DM8"}=" H1' DM8";
$resn_exch{" H2* DM8"}=" H2' DM8"; $resn_exch{" H3* DM8"}=" H3' DM8"; $resn_exch{" H4* DM8"}=" H4' DM8";
$resn_exch{" H5* DM8"}=" H5' DM8"; $resn_exch{"1H6* DM8"}="H6'1 DM8"; $resn_exch{"2H6* DM8"}="H6'2 DM8";
$resn_exch{"3H6* DM8"}="H6'3 DM8"; $resn_exch{"*HO4 DM8"}="HO4' DM8"; $resn_exch{"1HN* DM8"}="HN'1 DM8";
$resn_exch{"2HN* DM8"}="HN'2 DM8"; $resn_exch{"1H8  DM9"}=" H81 DM9"; $resn_exch{"2H8  DM9"}=" H82 DM9";
$resn_exch{"1H10 DM9"}="H101 DM9"; $resn_exch{"2H10 DM9"}="H102 DM9"; $resn_exch{"1H14 DM9"}="H141 DM9";
$resn_exch{"2H14 DM9"}="H142 DM9"; $resn_exch{"3H14 DM9"}="H143 DM9"; $resn_exch{"1H21 DM9"}="H211 DM9";
$resn_exch{"2H21 DM9"}="H212 DM9"; $resn_exch{"3H21 DM9"}="H213 DM9"; $resn_exch{" H1* DM9"}=" H1' DM9";
$resn_exch{"1H2* DM9"}="H2'1 DM9"; $resn_exch{"2H2* DM9"}="H2'2 DM9"; $resn_exch{" H3* DM9"}=" H3' DM9";
$resn_exch{" H4* DM9"}=" H4' DM9"; $resn_exch{"HO4* DM9"}="HO4' DM9"; $resn_exch{" H5* DM9"}=" H5' DM9";
$resn_exch{"1H6* DM9"}="H6'1 DM9"; $resn_exch{"2H6* DM9"}="H6'2 DM9"; $resn_exch{"3H6* DM9"}="H6'3 DM9";
$resn_exch{"1H31 DM9"}="H311 DM9"; $resn_exch{"2H31 DM9"}="H312 DM9"; $resn_exch{"1H41 DM9"}="H411 DM9";
$resn_exch{"2H41 DM9"}="H412 DM9"; $resn_exch{"1H42 DM9"}="H421 DM9"; $resn_exch{"2H42 DM9"}="H422 DM9";
$resn_exch{"1H43 DM9"}="H431 DM9"; $resn_exch{"2H43 DM9"}="H432 DM9"; $resn_exch{"3H43 DM9"}="H433 DM9";
$resn_exch{"1H1  DMA"}=" H11 DMA"; $resn_exch{"2H1  DMA"}=" H12 DMA"; $resn_exch{"1H4  DMA"}=" H41 DMA";
$resn_exch{"2H4  DMA"}=" H42 DMA"; $resn_exch{"3H4  DMA"}=" H43 DMA"; $resn_exch{"1H5  DMA"}=" H51 DMA";
$resn_exch{"2H5  DMA"}=" H52 DMA"; $resn_exch{"3H5  DMA"}=" H53 DMA"; $resn_exch{"2HOA DMA"}="HOA2 DMA";
$resn_exch{"2HOB DMA"}="HOB2 DMA"; $resn_exch{"3HOB DMA"}="HOB3 DMA"; $resn_exch{"1HM3 DMB"}="HM31 DMB";
$resn_exch{"2HM3 DMB"}="HM32 DMB"; $resn_exch{"3HM3 DMB"}="HM33 DMB"; $resn_exch{"1HM5 DMB"}="HM51 DMB";
$resn_exch{"2HM5 DMB"}="HM52 DMB"; $resn_exch{"3HM5 DMB"}="HM53 DMB"; $resn_exch{"'HO4 DMB"}="HO4' DMB";
$resn_exch{"1HB2 DMC"}="HB21 DMC"; $resn_exch{"2HB2 DMC"}="HB22 DMC"; $resn_exch{"3HB2 DMC"}="HB23 DMC";
$resn_exch{"1HH1 DMC"}="HH11 DMC"; $resn_exch{"2HH1 DMC"}="HH12 DMC"; $resn_exch{"1HH2 DMC"}="HH21 DMC";
$resn_exch{"2HH2 DMC"}="HH22 DMC"; $resn_exch{"1HM1 DMC"}="HM11 DMC"; $resn_exch{"2HM1 DMC"}="HM12 DMC";
$resn_exch{"3HM1 DMC"}="HM13 DMC"; $resn_exch{"1HM2 DMC"}="HM21 DMC"; $resn_exch{"2HM2 DMC"}="HM22 DMC";
$resn_exch{"3HM2 DMC"}="HM23 DMC"; $resn_exch{"1HN3 DMD"}="HN31 DMD"; $resn_exch{"1H2  DMD"}=" H21 DMD";
$resn_exch{"1H4  DMD"}=" H41 DMD"; $resn_exch{"1H7  DMD"}=" H71 DMD"; $resn_exch{"1H8  DMD"}=" H81 DMD";
$resn_exch{"2H8  DMD"}=" H82 DMD"; $resn_exch{"3H8  DMD"}=" H83 DMD"; $resn_exch{"1H9  DMD"}=" H91 DMD";
$resn_exch{"2H9  DMD"}=" H92 DMD"; $resn_exch{"3H9  DMD"}=" H93 DMD"; $resn_exch{"1H2  DME"}=" H21 DME";
$resn_exch{"2H2  DME"}=" H22 DME"; $resn_exch{"1H3  DME"}=" H31 DME"; $resn_exch{"2H3  DME"}=" H32 DME";
$resn_exch{"1H4  DME"}=" H41 DME"; $resn_exch{"2H4  DME"}=" H42 DME"; $resn_exch{"1H5  DME"}=" H51 DME";
$resn_exch{"2H5  DME"}=" H52 DME"; $resn_exch{"1H6  DME"}=" H61 DME"; $resn_exch{"2H6  DME"}=" H62 DME";
$resn_exch{"1H7  DME"}=" H71 DME"; $resn_exch{"2H7  DME"}=" H72 DME"; $resn_exch{"1H8  DME"}=" H81 DME";
$resn_exch{"2H8  DME"}=" H82 DME"; $resn_exch{"1H9  DME"}=" H91 DME"; $resn_exch{"2H9  DME"}=" H92 DME";
$resn_exch{"1H10 DME"}="H101 DME"; $resn_exch{"2H10 DME"}="H102 DME"; $resn_exch{"1H11 DME"}="H111 DME";
$resn_exch{"2H11 DME"}="H112 DME"; $resn_exch{"1H13 DME"}="H131 DME"; $resn_exch{"2H13 DME"}="H132 DME";
$resn_exch{"3H13 DME"}="H133 DME"; $resn_exch{"1H14 DME"}="H141 DME"; $resn_exch{"2H14 DME"}="H142 DME";
$resn_exch{"3H14 DME"}="H143 DME"; $resn_exch{"1H15 DME"}="H151 DME"; $resn_exch{"2H15 DME"}="H152 DME";
$resn_exch{"3H15 DME"}="H153 DME"; $resn_exch{"1H16 DME"}="H161 DME"; $resn_exch{"2H16 DME"}="H162 DME";
$resn_exch{"3H16 DME"}="H163 DME"; $resn_exch{"1H17 DME"}="H171 DME"; $resn_exch{"2H17 DME"}="H172 DME";
$resn_exch{"3H17 DME"}="H173 DME"; $resn_exch{"1H18 DME"}="H181 DME"; $resn_exch{"2H18 DME"}="H182 DME";
$resn_exch{"3H18 DME"}="H183 DME"; $resn_exch{"1H1  DMF"}=" H11 DMF"; $resn_exch{"2H1  DMF"}=" H12 DMF";
$resn_exch{"3H1  DMF"}=" H13 DMF"; $resn_exch{"1H2  DMF"}=" H21 DMF"; $resn_exch{"2H2  DMF"}=" H22 DMF";
$resn_exch{"3H2  DMF"}=" H23 DMF"; $resn_exch{"1HA  DMG"}=" HA1 DMG"; $resn_exch{"2HA  DMG"}=" HA2 DMG";
$resn_exch{"1H4  DMG"}=" H41 DMG"; $resn_exch{"2H4  DMG"}=" H42 DMG"; $resn_exch{"3H4  DMG"}=" H43 DMG";
$resn_exch{"1H5  DMG"}=" H51 DMG"; $resn_exch{"2H5  DMG"}=" H52 DMG"; $resn_exch{"3H5  DMG"}=" H53 DMG";
$resn_exch{" HN2 DMH"}=" H2  DMH"; $resn_exch{"1HA  DMH"}=" HA1 DMH"; $resn_exch{"1HB  DMH"}=" HB2 DMH";
$resn_exch{"2HB  DMH"}=" HB3 DMH"; $resn_exch{"1HE1 DMH"}="HE11 DMH"; $resn_exch{"2HE1 DMH"}="HE12 DMH";
$resn_exch{"3HE1 DMH"}="HE13 DMH"; $resn_exch{"1HE2 DMH"}="HE21 DMH"; $resn_exch{"2HE2 DMH"}="HE22 DMH";
$resn_exch{"3HE2 DMH"}="HE23 DMH"; $resn_exch{"1HM2 DMI"}="HM21 DMI"; $resn_exch{"2HM2 DMI"}="HM22 DMI";
$resn_exch{"3HM2 DMI"}="HM23 DMI"; $resn_exch{"1HM3 DMI"}="HM31 DMI"; $resn_exch{"2HM3 DMI"}="HM32 DMI";
$resn_exch{"3HM3 DMI"}="HM33 DMI"; $resn_exch{"1H1  DMJ"}=" H11 DMJ"; $resn_exch{"2H1  DMJ"}=" H12 DMJ";
$resn_exch{"1H6  DMJ"}=" H61 DMJ"; $resn_exch{"2H6  DMJ"}=" H62 DMJ"; $resn_exch{"1HN  DMK"}=" H   DMK";
$resn_exch{"2HN  DMK"}=" H2  DMK"; $resn_exch{"1HG2 DMK"}="HG21 DMK"; $resn_exch{"2HG2 DMK"}="HG22 DMK";
$resn_exch{"3HG2 DMK"}="HG23 DMK"; $resn_exch{"1HG3 DMK"}="HG31 DMK"; $resn_exch{"2HG3 DMK"}="HG32 DMK";
$resn_exch{"3HG3 DMK"}="HG33 DMK"; $resn_exch{"OHD1 DMK"}=" HD1 DMK"; $resn_exch{"1HM1 DML"}="HM11 DML";
$resn_exch{"2HM1 DML"}="HM12 DML"; $resn_exch{"3HM1 DML"}="HM13 DML"; $resn_exch{"1HM2 DML"}="HM21 DML";
$resn_exch{"2HM2 DML"}="HM22 DML"; $resn_exch{"3HM2 DML"}="HM23 DML"; $resn_exch{"1H11 DMM"}="H111 DMM";
$resn_exch{"2H11 DMM"}="H112 DMM"; $resn_exch{"2HO1 DMM"}="HO12 DMM"; $resn_exch{"1H14 DMM"}="H141 DMM";
$resn_exch{"2H14 DMM"}="H142 DMM"; $resn_exch{"4HO1 DMM"}="HO14 DMM"; $resn_exch{"1H15 DMM"}="H151 DMM";
$resn_exch{"2H15 DMM"}="H152 DMM"; $resn_exch{"7HO1 DMM"}="HO17 DMM"; $resn_exch{"1H21 DMM"}="H211 DMM";
$resn_exch{"2H21 DMM"}="H212 DMM"; $resn_exch{"3H21 DMM"}="H213 DMM"; $resn_exch{" H1* DMM"}=" H1' DMM";
$resn_exch{"1H2* DMM"}="H2'1 DMM"; $resn_exch{"2H2* DMM"}="H2'2 DMM"; $resn_exch{" H3* DMM"}=" H3' DMM";
$resn_exch{" H4* DMM"}=" H4' DMM"; $resn_exch{"*HO4 DMM"}="HO4' DMM"; $resn_exch{" H5* DMM"}=" H5' DMM";
$resn_exch{"1H6* DMM"}="H6'1 DMM"; $resn_exch{"2H6* DMM"}="H6'2 DMM"; $resn_exch{"3H6* DMM"}="H6'3 DMM";
$resn_exch{"1H33 DMM"}="H331 DMM"; $resn_exch{"2H33 DMM"}="H332 DMM"; $resn_exch{"1H63 DMM"}="H631 DMM";
$resn_exch{"2H63 DMM"}="H632 DMM"; $resn_exch{"1H53 DMM"}="H531 DMM"; $resn_exch{"2H53 DMM"}="H532 DMM";
$resn_exch{"1H73 DMM"}="H731 DMM"; $resn_exch{"2H73 DMM"}="H732 DMM"; $resn_exch{"3H73 DMM"}="H733 DMM";
$resn_exch{"1H2  DMN"}=" H21 DMN"; $resn_exch{"2H2  DMN"}=" H22 DMN"; $resn_exch{"3H2  DMN"}=" H23 DMN";
$resn_exch{"1H3  DMN"}=" H31 DMN"; $resn_exch{"2H3  DMN"}=" H32 DMN"; $resn_exch{"3H3  DMN"}=" H33 DMN";
$resn_exch{"1HCO DMO"}="HCO1 DMO"; $resn_exch{"1HNO DMO"}="HNO1 DMO"; $resn_exch{"2HNO DMO"}="HNO2 DMO";
$resn_exch{"1HBO DMO"}="HBO1 DMO"; $resn_exch{"2HBO DMO"}="HBO2 DMO"; $resn_exch{"1HGO DMO"}="HGO1 DMO";
$resn_exch{"2HGO DMO"}="HGO2 DMO"; $resn_exch{"1HDO DMO"}="HDO1 DMO"; $resn_exch{"2HDO DMO"}="HDO2 DMO";
$resn_exch{"1HEO DMO"}="HEO1 DMO"; $resn_exch{"2HEO DMO"}="HEO2 DMO"; $resn_exch{"1H20 DMP"}="H201 DMP";
$resn_exch{"2H20 DMP"}="H202 DMP"; $resn_exch{"1H27 DMP"}="H271 DMP"; $resn_exch{"2H27 DMP"}="H272 DMP";
$resn_exch{"1H30 DMP"}="H301 DMP"; $resn_exch{"2H30 DMP"}="H302 DMP"; $resn_exch{"1H60 DMP"}="H601 DMP";
$resn_exch{"2H60 DMP"}="H602 DMP"; $resn_exch{"1H70 DMP"}="H701 DMP"; $resn_exch{"2H70 DMP"}="H702 DMP";
$resn_exch{"1H77 DMP"}="H771 DMP"; $resn_exch{"2H77 DMP"}="H772 DMP"; $resn_exch{"7HO2 DMP"}="HO27 DMP";
$resn_exch{"7HO7 DMP"}="HO77 DMP"; $resn_exch{"1H2  DMQ"}="H21A DMQ"; $resn_exch{"2H2  DMQ"}="H22A DMQ";
$resn_exch{"1H7  DMQ"}="H71A DMQ"; $resn_exch{"2H7  DMQ"}=" H72 DMQ"; $resn_exch{"1HN2 DMQ"}="HN21 DMQ";
$resn_exch{"2HN2 DMQ"}="HN22 DMQ"; $resn_exch{"1H31 DMQ"}="H311 DMQ"; $resn_exch{"2H31 DMQ"}="H312 DMQ";
$resn_exch{"1H61 DMQ"}="H611 DMQ"; $resn_exch{"2H61 DMQ"}="H612 DMQ"; $resn_exch{"1HN7 DMQ"}="HN71 DMQ";
$resn_exch{"2HN7 DMQ"}="HN72 DMQ"; $resn_exch{"1H1  DMS"}=" H11 DMS"; $resn_exch{"2H1  DMS"}=" H12 DMS";
$resn_exch{"3H1  DMS"}=" H13 DMS"; $resn_exch{"1H2  DMS"}=" H21 DMS"; $resn_exch{"2H2  DMS"}=" H22 DMS";
$resn_exch{"3H2  DMS"}=" H23 DMS"; $resn_exch{" HN  DMT"}=" H   DMT"; $resn_exch{"1HCN DMT"}="HCN1 DMT";
$resn_exch{"2HCN DMT"}="HCN2 DMT"; $resn_exch{"3HCN DMT"}="HCN3 DMT"; $resn_exch{"1HD1 DMT"}="HD12 DMT";
$resn_exch{"2HD1 DMT"}="HD13 DMT"; $resn_exch{"1HD2 DMT"}="HD21 DMT"; $resn_exch{"2HD2 DMT"}="HD22 DMT";
$resn_exch{"3HD2 DMT"}="HD23 DMT"; $resn_exch{"1HD3 DMT"}="HD31 DMT"; $resn_exch{"2HD3 DMT"}="HD32 DMT";
$resn_exch{"3HD3 DMT"}="HD33 DMT"; $resn_exch{"1HH  DMT"}=" HH1 DMT"; $resn_exch{"2HH  DMT"}=" HH2 DMT";
$resn_exch{"3HH  DMT"}=" HH3 DMT"; $resn_exch{"1H18 DMU"}="H181 DMU"; $resn_exch{"2H18 DMU"}="H182 DMU";
$resn_exch{"1H19 DMU"}="H191 DMU"; $resn_exch{"2H19 DMU"}="H192 DMU"; $resn_exch{"1H22 DMU"}="H221 DMU";
$resn_exch{"2H22 DMU"}="H222 DMU"; $resn_exch{"1H25 DMU"}="H251 DMU"; $resn_exch{"2H25 DMU"}="H252 DMU";
$resn_exch{"1H28 DMU"}="H281 DMU"; $resn_exch{"2H28 DMU"}="H282 DMU"; $resn_exch{"1H31 DMU"}="H311 DMU";
$resn_exch{"2H31 DMU"}="H312 DMU"; $resn_exch{"1H34 DMU"}="H341 DMU"; $resn_exch{"2H34 DMU"}="H342 DMU";
$resn_exch{"1H37 DMU"}="H371 DMU"; $resn_exch{"2H37 DMU"}="H372 DMU"; $resn_exch{"1H40 DMU"}="H401 DMU";
$resn_exch{"2H40 DMU"}="H402 DMU"; $resn_exch{"1H43 DMU"}="H431 DMU"; $resn_exch{"2H43 DMU"}="H432 DMU";
$resn_exch{"3H43 DMU"}="H433 DMU"; $resn_exch{"1H57 DMU"}="H571 DMU"; $resn_exch{"2H57 DMU"}="H572 DMU";
$resn_exch{"1H11 DMU"}="H111 DMU"; $resn_exch{"2H11 DMU"}="H112 DMU"; $resn_exch{"3H32 DMV"}="H323 DMV";
$resn_exch{"2H32 DMV"}="H322 DMV"; $resn_exch{"1H32 DMV"}="H321 DMV"; $resn_exch{"2H4  DMV"}=" H42 DMV";
$resn_exch{"1H4  DMV"}=" H41 DMV"; $resn_exch{"3H5  DMV"}=" H53 DMV"; $resn_exch{"2H5  DMV"}=" H52 DMV";
$resn_exch{"1H5  DMV"}=" H51 DMV"; $resn_exch{"1H5M DMW"}="H5M1 DMW"; $resn_exch{"2H5M DMW"}="H5M2 DMW";
$resn_exch{"3H5M DMW"}="H5M3 DMW"; $resn_exch{"1H6M DMW"}="H6M1 DMW"; $resn_exch{"2H6M DMW"}="H6M2 DMW";
$resn_exch{"3H6M DMW"}="H6M3 DMW"; $resn_exch{"1H7  DMX"}=" H71 DMX"; $resn_exch{"2H7  DMX"}=" H72 DMX";
$resn_exch{"1H9  DMX"}=" H91 DMX"; $resn_exch{"2H9  DMX"}=" H92 DMX"; $resn_exch{"1H10 DMX"}="H101 DMX";
$resn_exch{"2H10 DMX"}="H102 DMX"; $resn_exch{"1H12 DMX"}="H121 DMX"; $resn_exch{"2H12 DMX"}="H122 DMX";
$resn_exch{"3H12 DMX"}="H123 DMX"; $resn_exch{"1H13 DMX"}="H131 DMX"; $resn_exch{"2H13 DMX"}="H132 DMX";
$resn_exch{"3H13 DMX"}="H133 DMX"; $resn_exch{"1H17 DMX"}="H171 DMX"; $resn_exch{"2H17 DMX"}="H172 DMX";
$resn_exch{"1H6  DMY"}=" H61 DMY"; $resn_exch{"2H6  DMY"}=" H62 DMY"; $resn_exch{"3H6  DMY"}=" H63 DMY";
$resn_exch{"1H12 DMY"}="H121 DMY"; $resn_exch{"2H12 DMY"}="H122 DMY"; $resn_exch{"3H12 DMY"}="H123 DMY";
$resn_exch{"1H18 DMY"}="H181 DMY"; $resn_exch{"2H18 DMY"}="H182 DMY"; $resn_exch{"3H18 DMY"}="H183 DMY";
$resn_exch{"1H20 DMY"}="H201 DMY"; $resn_exch{"2H20 DMY"}="H202 DMY"; $resn_exch{"1H21 DMY"}="H211 DMY";
$resn_exch{"2H21 DMY"}="H212 DMY"; $resn_exch{"1HN9 DMY"}="HN91 DMY"; $resn_exch{"2HN9 DMY"}="HN92 DMY";
$resn_exch{"1H41 DMZ"}="H411 DMZ"; $resn_exch{"2H41 DMZ"}="H412 DMZ"; $resn_exch{"3H41 DMZ"}="H413 DMZ";
$resn_exch{"1H31 DMZ"}="H311 DMZ"; $resn_exch{"2H31 DMZ"}="H312 DMZ"; $resn_exch{"3H31 DMZ"}="H313 DMZ";
$resn_exch{"1H5C DN1"}="H5C1 DN1"; $resn_exch{"2H5C DN1"}="H5C2 DN1"; $resn_exch{"1H6C DN1"}="H6C1 DN1";
$resn_exch{"2H6C DN1"}="H6C2 DN1"; $resn_exch{"3H6C DN1"}="H6C3 DN1"; $resn_exch{"1H7C DN1"}="H7C1 DN1";
$resn_exch{"2H7C DN1"}="H7C2 DN1"; $resn_exch{"3H7C DN1"}="H7C3 DN1"; $resn_exch{"1H22 DN1"}="H221 DN1";
$resn_exch{"2H22 DN1"}="H222 DN1"; $resn_exch{"1H23 DN1"}="H231 DN1"; $resn_exch{"2H23 DN1"}="H232 DN1";
$resn_exch{"1H5C DN2"}="H5C1 DN2"; $resn_exch{"2H5C DN2"}="H5C2 DN2"; $resn_exch{"1H6C DN2"}="H6C1 DN2";
$resn_exch{"2H6C DN2"}="H6C2 DN2"; $resn_exch{"3H6C DN2"}="H6C3 DN2"; $resn_exch{"1H7C DN2"}="H7C1 DN2";
$resn_exch{"2H7C DN2"}="H7C2 DN2"; $resn_exch{"3H7C DN2"}="H7C3 DN2"; $resn_exch{"1H13 DN2"}="H131 DN2";
$resn_exch{"2H13 DN2"}="H132 DN2"; $resn_exch{"1H22 DN2"}="H221 DN2"; $resn_exch{"2H22 DN2"}="H222 DN2";
$resn_exch{"1H23 DN2"}="H231 DN2"; $resn_exch{"2H23 DN2"}="H232 DN2"; $resn_exch{"OHPN DND"}="HPNO DND";
$resn_exch{"1H5N DND"}="H5N1 DND"; $resn_exch{"2H5N DND"}="H5N2 DND"; $resn_exch{"1H4N DND"}="H4N1 DND";
$resn_exch{"CH3N DND"}="H3NC DND"; $resn_exch{"OH3N DND"}="H3NO DND"; $resn_exch{"CH2' DND"}=" H2B DND";
$resn_exch{"OH2' DND"}="H2'O DND"; $resn_exch{"CH1N DND"}="H1NC DND"; $resn_exch{"CH6N DND"}="H6NC DND";
$resn_exch{"CH5N DND"}="H5NC DND"; $resn_exch{"CH4N DND"}="H4NC DND"; $resn_exch{"CH2N DND"}="H2NC DND";
$resn_exch{"OH7N DND"}="H7NO DND"; $resn_exch{"OHPA DND"}="HPAO DND"; $resn_exch{"1H5A DND"}="H5A1 DND";
$resn_exch{"2H5A DND"}="H5A2 DND"; $resn_exch{"4H'A DND"}=" HB4 DND"; $resn_exch{"CH3A DND"}="H3AC DND";
$resn_exch{"3H'A DND"}=" HB3 DND"; $resn_exch{"2H'A DND"}=" HB2 DND"; $resn_exch{"OH'A DND"}=" HBO DND";
$resn_exch{"1H'A DND"}=" HB1 DND"; $resn_exch{"CH2A DND"}="H2AC DND"; $resn_exch{"CH8A DND"}="H8AC DND";
$resn_exch{"1HN6 DND"}="HN61 DND"; $resn_exch{"2HN6 DND"}="HN62 DND"; $resn_exch{"1HN  DNE"}=" H   DNE";
$resn_exch{"2HN  DNE"}=" H2  DNE"; $resn_exch{"1HB  DNE"}=" HB2 DNE"; $resn_exch{"2HB  DNE"}=" HB3 DNE";
$resn_exch{"1HG  DNE"}=" HG2 DNE"; $resn_exch{"2HG  DNE"}=" HG3 DNE"; $resn_exch{"1HD  DNE"}=" HD2 DNE";
$resn_exch{"2HD  DNE"}=" HD3 DNE"; $resn_exch{"1HE  DNE"}=" HE1 DNE"; $resn_exch{"2HE  DNE"}=" HE2 DNE";
$resn_exch{"3HE  DNE"}=" HE3 DNE"; $resn_exch{"1HN  DNG"}=" H   DNG"; $resn_exch{"1HB  DNG"}=" HB2 DNG";
$resn_exch{"2HB  DNG"}=" HB3 DNG"; $resn_exch{"1HG  DNG"}=" HG2 DNG"; $resn_exch{"2HG  DNG"}=" HG3 DNG";
$resn_exch{"1HD  DNG"}=" HD2 DNG"; $resn_exch{"2HD  DNG"}=" HD3 DNG"; $resn_exch{"1HE  DNG"}=" HE1 DNG";
$resn_exch{"2HE  DNG"}=" HE2 DNG"; $resn_exch{"3HE  DNG"}=" HE3 DNG"; $resn_exch{"1H7  DNH"}=" H71 DNH";
$resn_exch{"2H7  DNH"}=" H72 DNH"; $resn_exch{"1H8  DNH"}=" H81 DNH"; $resn_exch{"2H8  DNH"}=" H82 DNH";
$resn_exch{"1H5  DNH"}=" H51 DNH"; $resn_exch{"2H5  DNH"}=" H52 DNH"; $resn_exch{"1H6  DNH"}=" H61 DNH";
$resn_exch{"2H6  DNH"}=" H62 DNH"; $resn_exch{"1H1  DNH"}=" H11 DNH"; $resn_exch{"2H1  DNH"}=" H12 DNH";
$resn_exch{"1H2  DNH"}=" H21 DNH"; $resn_exch{"2H2  DNH"}=" H22 DNH"; $resn_exch{"1H1  DNJ"}=" H11 DNJ";
$resn_exch{"2H1  DNJ"}=" H12 DNJ"; $resn_exch{"1H6  DNJ"}=" H61 DNJ"; $resn_exch{"2H6  DNJ"}=" H62 DNJ";
$resn_exch{"1HB  DNL"}=" HB1 DNL"; $resn_exch{"2HB  DNL"}=" HB2 DNL"; $resn_exch{"1HG  DNL"}=" HG1 DNL";
$resn_exch{"2HG  DNL"}=" HG2 DNL"; $resn_exch{"1HD  DNL"}=" HD1 DNL"; $resn_exch{"2HD  DNL"}=" HD2 DNL";
$resn_exch{"1HE  DNL"}=" HE1 DNL"; $resn_exch{"2HE  DNL"}=" HE2 DNL"; $resn_exch{"1HZ  DNL"}=" HZ1 DNL";
$resn_exch{"2HZ  DNL"}=" HZ2 DNL"; $resn_exch{" HN  DNM"}=" H   DNM"; $resn_exch{"1HB  DNM"}=" HB2 DNM";
$resn_exch{"2HB  DNM"}=" HB3 DNM"; $resn_exch{"1HG  DNM"}=" HG2 DNM"; $resn_exch{"2HG  DNM"}=" HG3 DNM";
$resn_exch{"1HD  DNM"}=" HD2 DNM"; $resn_exch{"2HD  DNM"}=" HD3 DNM"; $resn_exch{"1HE  DNM"}=" HE1 DNM";
$resn_exch{"2HE  DNM"}=" HE2 DNM"; $resn_exch{"3HE  DNM"}=" HE3 DNM"; $resn_exch{"1H2  DNN"}=" H21 DNN";
$resn_exch{"2H2  DNN"}=" H22 DNN"; $resn_exch{"1H3  DNN"}=" H31 DNN"; $resn_exch{"2H3  DNN"}=" H32 DNN";
$resn_exch{"1H4  DNN"}=" H41 DNN"; $resn_exch{"2H4  DNN"}=" H42 DNN"; $resn_exch{"1H5  DNN"}=" H51 DNN";
$resn_exch{"2H5  DNN"}=" H52 DNN"; $resn_exch{"1H6  DNN"}=" H61 DNN"; $resn_exch{"2H6  DNN"}=" H62 DNN";
$resn_exch{"1H9  DNN"}=" H91 DNN"; $resn_exch{"2H9  DNN"}=" H92 DNN"; $resn_exch{"3H9  DNN"}=" H93 DNN";
$resn_exch{"1HN7 DNN"}="HN71 DNN"; $resn_exch{"2HN7 DNN"}="HN72 DNN"; $resn_exch{"1HN8 DNN"}="HN81 DNN";
$resn_exch{"2HN8 DNN"}="HN82 DNN"; $resn_exch{"1HN  DNP"}=" H   DNP"; $resn_exch{"2HN  DNP"}=" H2  DNP";
$resn_exch{"1HB  DNP"}=" HB2 DNP"; $resn_exch{"2HB  DNP"}=" HB3 DNP"; $resn_exch{"1HG  DNP"}=" HG1 DNP";
$resn_exch{"2HG  DNP"}=" HG2 DNP"; $resn_exch{"3HG  DNP"}=" HG3 DNP"; $resn_exch{"1H5  DNQ"}=" H51 DNQ";
$resn_exch{"1H6  DNQ"}=" H61 DNQ"; $resn_exch{"2HOP DNR"}="HOP2 DNR"; $resn_exch{"1H5* DNR"}=" H5' DNR";
$resn_exch{"2H5* DNR"}="H5'' DNR"; $resn_exch{" H4* DNR"}=" H4' DNR"; $resn_exch{" H1* DNR"}=" H1' DNR";
$resn_exch{"1H2* DNR"}=" H2' DNR"; $resn_exch{"2H2* DNR"}="H2'' DNR"; $resn_exch{" H3* DNR"}=" H3' DNR";
$resn_exch{"H3T  DNR"}="HO3' DNR"; $resn_exch{"3HOP DNR"}="HOP3 DNR"; $resn_exch{"1H1  DNS"}="H11A DNS";
$resn_exch{"2H1  DNS"}=" H12 DNS"; $resn_exch{"3H1  DNS"}=" H13 DNS"; $resn_exch{"1H2  DNS"}=" H21 DNS";
$resn_exch{"2H2  DNS"}=" H22 DNS"; $resn_exch{"3H2  DNS"}=" H23 DNS"; $resn_exch{" HNZ DNS"}=" HZ  DNS";
$resn_exch{"1HE  DNS"}=" HE3 DNS"; $resn_exch{"2HE  DNS"}=" HE2 DNS"; $resn_exch{"1HD  DNS"}=" HD3 DNS";
$resn_exch{"2HD  DNS"}=" HD2 DNS"; $resn_exch{"1HG  DNS"}=" HG3 DNS"; $resn_exch{"2HG  DNS"}=" HG2 DNS";
$resn_exch{"1HB  DNS"}=" HB3 DNS"; $resn_exch{"2HB  DNS"}=" HB2 DNS"; $resn_exch{"1HN  DNS"}=" H   DNS";
$resn_exch{"2HN  DNS"}=" H2  DNS"; $resn_exch{"1H13 DNT"}="H131 DNT"; $resn_exch{"2H13 DNT"}="H132 DNT";
$resn_exch{"3H13 DNT"}="H133 DNT"; $resn_exch{"1H12 DNT"}="H121 DNT"; $resn_exch{"2H12 DNT"}="H122 DNT";
$resn_exch{"1H11 DNT"}="H111 DNT"; $resn_exch{"2H11 DNT"}="H112 DNT"; $resn_exch{"1H9  DNT"}=" H91 DNT";
$resn_exch{"2H9  DNT"}=" H92 DNT"; $resn_exch{"1H8  DNT"}=" H81 DNT"; $resn_exch{"2H8  DNT"}=" H82 DNT";
$resn_exch{"1H10 DNT"}="H101 DNT"; $resn_exch{"2H10 DNT"}="H102 DNT"; $resn_exch{"3H10 DNT"}="H103 DNT";
$resn_exch{"1HB  DO2"}=" HB1 DO2"; $resn_exch{"2HB  DO2"}=" HB2 DO2"; $resn_exch{"1HG  DO2"}=" HG1 DO2";
$resn_exch{"2HG  DO2"}=" HG2 DO2"; $resn_exch{"1HD  DO2"}=" HD1 DO2"; $resn_exch{"2HD  DO2"}=" HD2 DO2";
$resn_exch{"1HE  DO2"}=" HE1 DO2"; $resn_exch{"1HO  DO3"}=" HO1 DO3"; $resn_exch{"3HO  DO3"}=" HO3 DO3";
$resn_exch{"5HO  DO3"}=" HO5 DO3"; $resn_exch{"7HO  DO3"}=" HO7 DO3"; $resn_exch{"1HC1 DO3"}="HC11 DO3";
$resn_exch{"2HC1 DO3"}="HC12 DO3"; $resn_exch{"1HC2 DO3"}="HC21 DO3"; $resn_exch{"2HC2 DO3"}="HC22 DO3";
$resn_exch{"1HC3 DO3"}="HC31 DO3"; $resn_exch{"2HC3 DO3"}="HC32 DO3"; $resn_exch{"1HC4 DO3"}="HC41 DO3";
$resn_exch{"2HC4 DO3"}="HC42 DO3"; $resn_exch{"1HC5 DO3"}="HC51 DO3"; $resn_exch{"2HC5 DO3"}="HC52 DO3";
$resn_exch{"1HC6 DO3"}="HC61 DO3"; $resn_exch{"2HC6 DO3"}="HC62 DO3"; $resn_exch{"1HC7 DO3"}="HC71 DO3";
$resn_exch{"2HC7 DO3"}="HC72 DO3"; $resn_exch{"1HC8 DO3"}="HC81 DO3"; $resn_exch{"2HC8 DO3"}="HC82 DO3";
$resn_exch{"1H10 DO3"}="H101 DO3"; $resn_exch{"2H10 DO3"}="H102 DO3"; $resn_exch{"1H12 DO3"}="H121 DO3";
$resn_exch{"2H12 DO3"}="H122 DO3"; $resn_exch{"1H14 DO3"}="H141 DO3"; $resn_exch{"2H14 DO3"}="H142 DO3";
$resn_exch{"1H16 DO3"}="H161 DO3"; $resn_exch{"2H16 DO3"}="H162 DO3"; $resn_exch{"1H17 DO3"}="H171 DO3";
$resn_exch{"2H17 DO3"}="H172 DO3"; $resn_exch{"3H17 DO3"}="H173 DO3"; $resn_exch{"1H4  DO4"}=" H41 DO4";
$resn_exch{"2H4  DO4"}=" H42 DO4"; $resn_exch{"1H3  DO4"}=" H31 DO4"; $resn_exch{"2H3  DO4"}=" H32 DO4";
$resn_exch{"1H3  DO8"}=" H31 DO8"; $resn_exch{"2H3  DO8"}=" H32 DO8"; $resn_exch{"1H8  DO8"}=" H81 DO8";
$resn_exch{"2H8  DO8"}=" H82 DO8"; $resn_exch{"2HOP DO8"}="HOP2 DO8"; $resn_exch{"3HOP DO8"}="HOP3 DO8";
$resn_exch{" HN2 DOA"}=" H2  DOA"; $resn_exch{"1HA  DOA"}=" HA1 DOA"; $resn_exch{"2HA  DOA"}=" HA2 DOA";
$resn_exch{"1H2  DOA"}=" H21 DOA"; $resn_exch{"2H2  DOA"}=" H22 DOA"; $resn_exch{"1H3  DOA"}=" H31 DOA";
$resn_exch{"2H3  DOA"}=" H32 DOA"; $resn_exch{"1H4  DOA"}=" H41 DOA"; $resn_exch{"2H4  DOA"}=" H42 DOA";
$resn_exch{"1H5  DOA"}=" H51 DOA"; $resn_exch{"2H5  DOA"}=" H52 DOA"; $resn_exch{"1H6  DOA"}=" H61 DOA";
$resn_exch{"2H6  DOA"}=" H62 DOA"; $resn_exch{"1H7  DOA"}=" H71 DOA"; $resn_exch{"2H7  DOA"}=" H72 DOA";
$resn_exch{"1H8  DOA"}=" H81 DOA"; $resn_exch{"2H8  DOA"}=" H82 DOA"; $resn_exch{"1H9  DOA"}=" H91 DOA";
$resn_exch{"2H9  DOA"}=" H92 DOA"; $resn_exch{"1H10 DOA"}="H101 DOA"; $resn_exch{"2H10 DOA"}="H102 DOA";
$resn_exch{"1HA2 DOA"}="HA21 DOA"; $resn_exch{"2HA2 DOA"}="HA22 DOA"; $resn_exch{"'HO2 DOB"}="HO2' DOB";
$resn_exch{"1HN4 DOC"}="HN41 DOC"; $resn_exch{"2HN4 DOC"}="HN42 DOC"; $resn_exch{" H1* DOC"}=" H1' DOC";
$resn_exch{"1H2* DOC"}=" H2' DOC"; $resn_exch{"2H2* DOC"}="H2'' DOC"; $resn_exch{"1H3* DOC"}="H3'1 DOC";
$resn_exch{"2H3* DOC"}="H3'2 DOC"; $resn_exch{" H4* DOC"}=" H4' DOC"; $resn_exch{"1H5* DOC"}=" H5' DOC";
$resn_exch{"2H5* DOC"}="H5'' DOC"; $resn_exch{"2HOP DOC"}="HOP2 DOC"; $resn_exch{"3HOP DOC"}="HOP3 DOC";
$resn_exch{"1HC2 DOE"}="HC21 DOE"; $resn_exch{"2HC2 DOE"}="HC22 DOE"; $resn_exch{"1HC3 DOE"}="HC31 DOE";
$resn_exch{"2HC3 DOE"}="HC32 DOE"; $resn_exch{"1HC4 DOE"}="HC41 DOE"; $resn_exch{"2HC4 DOE"}="HC42 DOE";
$resn_exch{"1HC5 DOE"}="HC51 DOE"; $resn_exch{"2HC5 DOE"}="HC52 DOE"; $resn_exch{"1HC6 DOE"}="HC61 DOE";
$resn_exch{"2HC6 DOE"}="HC62 DOE"; $resn_exch{"1HC7 DOE"}="HC71 DOE"; $resn_exch{"2HC7 DOE"}="HC72 DOE";
$resn_exch{"1HC8 DOE"}="HC81 DOE"; $resn_exch{"2HC8 DOE"}="HC82 DOE"; $resn_exch{"1HC9 DOE"}="HC91 DOE";
$resn_exch{"2HC9 DOE"}="HC92 DOE"; $resn_exch{"1H11 DOE"}="H111 DOE"; $resn_exch{"2H11 DOE"}="H112 DOE";
$resn_exch{"1H13 DOE"}="H131 DOE"; $resn_exch{"2H13 DOE"}="H132 DOE"; $resn_exch{"1H15 DOE"}="H151 DOE";
$resn_exch{"2H15 DOE"}="H152 DOE"; $resn_exch{"1H17 DOE"}="H171 DOE"; $resn_exch{"2H17 DOE"}="H172 DOE";
$resn_exch{"1H25 DOE"}="H251 DOE"; $resn_exch{"2H25 DOE"}="H252 DOE"; $resn_exch{"1H26 DOE"}="H261 DOE";
$resn_exch{"2H26 DOE"}="H262 DOE"; $resn_exch{"1H27 DOE"}="H271 DOE"; $resn_exch{"2H27 DOE"}="H272 DOE";
$resn_exch{"1HC2 DOF"}="HC21 DOF"; $resn_exch{"2HC2 DOF"}="HC22 DOF"; $resn_exch{"1HC3 DOF"}="HC31 DOF";
$resn_exch{"2HC3 DOF"}="HC32 DOF"; $resn_exch{"1HC4 DOF"}="HC41 DOF"; $resn_exch{"2HC4 DOF"}="HC42 DOF";
$resn_exch{"1HC5 DOF"}="HC51 DOF"; $resn_exch{"2HC5 DOF"}="HC52 DOF"; $resn_exch{"1HC6 DOF"}="HC61 DOF";
$resn_exch{"2HC6 DOF"}="HC62 DOF"; $resn_exch{"1HC7 DOF"}="HC71 DOF"; $resn_exch{"2HC7 DOF"}="HC72 DOF";
$resn_exch{"1HC8 DOF"}="HC81 DOF"; $resn_exch{"2HC8 DOF"}="HC82 DOF"; $resn_exch{"1HC9 DOF"}="HC91 DOF";
$resn_exch{"2HC9 DOF"}="HC92 DOF"; $resn_exch{"1H11 DOF"}="H111 DOF"; $resn_exch{"2H11 DOF"}="H112 DOF";
$resn_exch{"1H13 DOF"}="H131 DOF"; $resn_exch{"2H13 DOF"}="H132 DOF"; $resn_exch{"1H15 DOF"}="H151 DOF";
$resn_exch{"2H15 DOF"}="H152 DOF"; $resn_exch{"1H17 DOF"}="H171 DOF"; $resn_exch{"2H17 DOF"}="H172 DOF";
$resn_exch{"1H21 DOG"}="H211 DOG"; $resn_exch{"2H21 DOG"}="H212 DOG"; $resn_exch{"1H16 DOG"}="H161 DOG";
$resn_exch{"2H16 DOG"}="H162 DOG"; $resn_exch{"1H18 DOG"}="H181 DOG"; $resn_exch{"2H18 DOG"}="H182 DOG";
$resn_exch{"3H18 DOG"}="H183 DOG"; $resn_exch{"OH12 DOG"}="H12O DOG"; $resn_exch{"1H15 DOG"}="H151 DOG";
$resn_exch{"2H15 DOG"}="H152 DOG"; $resn_exch{"OH14 DOG"}="H14O DOG"; $resn_exch{"1HC7 DOG"}="HC71 DOG";
$resn_exch{"2HC7 DOG"}="HC72 DOG"; $resn_exch{"1H11 DOG"}="H111 DOG"; $resn_exch{"2H11 DOG"}="H112 DOG";
$resn_exch{"1H19 DOG"}="H191 DOG"; $resn_exch{"2H19 DOG"}="H192 DOG"; $resn_exch{"3H19 DOG"}="H193 DOG";
$resn_exch{"1HC6 DOG"}="HC61 DOG"; $resn_exch{"2HC6 DOG"}="HC62 DOG"; $resn_exch{"1HC4 DOG"}="HC41 DOG";
$resn_exch{"2HC4 DOG"}="HC42 DOG"; $resn_exch{"OH32 DOG"}="H32O DOG"; $resn_exch{"1HC2 DOG"}="HC21 DOG";
$resn_exch{"2HC2 DOG"}="HC22 DOG"; $resn_exch{"1HC1 DOG"}="HC11 DOG"; $resn_exch{"2HC1 DOG"}="HC12 DOG";
$resn_exch{"1HN  DOH"}=" HN1 DOH"; $resn_exch{"2HN  DOH"}=" HN2 DOH"; $resn_exch{"1H5* DOI"}="H5'1 DOI";
$resn_exch{"2H5* DOI"}="H5'2 DOI"; $resn_exch{" H4* DOI"}=" H4' DOI"; $resn_exch{" H1* DOI"}=" H1' DOI";
$resn_exch{"1H2* DOI"}="H2'1 DOI"; $resn_exch{"2H2* DOI"}="H2'2 DOI"; $resn_exch{" H3* DOI"}=" H3' DOI";
$resn_exch{"*HO3 DOI"}="HO3' DOI"; $resn_exch{"1HC4 DOL"}="HC41 DOL"; $resn_exch{"2HC4 DOL"}="HC42 DOL";
$resn_exch{"1HC3 DOL"}="HC31 DOL"; $resn_exch{"2HC3 DOL"}="HC3A DOL"; $resn_exch{"1H42 DOL"}="H421 DOL";
$resn_exch{"2H42 DOL"}="H422 DOL"; $resn_exch{"1H43 DOL"}="H431 DOL"; $resn_exch{"2H43 DOL"}="H432 DOL";
$resn_exch{"1H47 DOL"}="H471 DOL"; $resn_exch{"2H47 DOL"}="H472 DOL"; $resn_exch{"1H48 DOL"}="H481 DOL";
$resn_exch{"2H48 DOL"}="H482 DOL"; $resn_exch{"3H48 DOL"}="H483 DOL"; $resn_exch{"1H45 DOL"}="H451 DOL";
$resn_exch{"2H45 DOL"}="H452 DOL"; $resn_exch{"1H46 DOL"}="H461 DOL"; $resn_exch{"2H46 DOL"}="H462 DOL";
$resn_exch{"3H46 DOL"}="H463 DOL"; $resn_exch{"1H13 DOL"}="H131 DOL"; $resn_exch{"2H13 DOL"}="H132 DOL";
$resn_exch{"1H16 DOL"}="H161 DOL"; $resn_exch{"2H16 DOL"}="H162 DOL"; $resn_exch{"1H21 DOL"}="H211 DOL";
$resn_exch{"2H21 DOL"}="H212 DOL"; $resn_exch{"3H21 DOL"}="H213 DOL"; $resn_exch{"1H24 DOL"}="H241 DOL";
$resn_exch{"2H24 DOL"}="H242 DOL"; $resn_exch{"1H31 DOL"}="H311 DOL"; $resn_exch{"2H31 DOL"}="H312 DOL";
$resn_exch{"3H31 DOL"}="H313 DOL"; $resn_exch{"1H35 DOL"}="H351 DOL"; $resn_exch{"2H35 DOL"}="H352 DOL";
$resn_exch{"3H35 DOL"}="H353 DOL"; $resn_exch{"1H34 DOL"}="H341 DOL"; $resn_exch{"2H34 DOL"}="H342 DOL";
$resn_exch{"3H34 DOL"}="H343 DOL"; $resn_exch{"1H6  DOM"}=" H61 DOM"; $resn_exch{"2H6  DOM"}=" H62 DOM";
$resn_exch{"1H2' DOM"}="H2'1 DOM"; $resn_exch{"2H2' DOM"}="H2'2 DOM"; $resn_exch{"1H6' DOM"}="H6'1 DOM";
$resn_exch{"2H6' DOM"}="H6'2 DOM"; $resn_exch{"'HO1 DOM"}="HO1' DOM"; $resn_exch{"'HO3 DOM"}="HO3' DOM";
$resn_exch{"'HO6 DOM"}="HO6' DOM"; $resn_exch{"HOXT DON"}=" HXT DON"; $resn_exch{"1HN  DON"}=" H   DON";
$resn_exch{"2HN  DON"}=" H2  DON"; $resn_exch{"1HB  DON"}=" HB2 DON"; $resn_exch{"2HB  DON"}=" HB3 DON";
$resn_exch{"1H1E DON"}="H1E1 DON"; $resn_exch{"2H1E DON"}="H1E2 DON"; $resn_exch{"1H1C DON"}="H1C1 DON";
$resn_exch{"2H1C DON"}="H1C2 DON"; $resn_exch{"1H1A DON"}="H1A1 DON"; $resn_exch{"1H1  DOP"}=" H11 DOP";
$resn_exch{"2H1  DOP"}=" H12 DOP"; $resn_exch{"1H2  DOP"}=" H21 DOP"; $resn_exch{"2H2  DOP"}=" H22 DOP";
$resn_exch{"1H3  DOP"}=" H31 DOP"; $resn_exch{"2H3  DOP"}=" H32 DOP"; $resn_exch{"1H4  DOP"}=" H41 DOP";
$resn_exch{"2H4  DOP"}=" H42 DOP"; $resn_exch{"1H5  DOP"}=" H51 DOP"; $resn_exch{"2H5  DOP"}=" H52 DOP";
$resn_exch{"1H6  DOP"}=" H61 DOP"; $resn_exch{"2H6  DOP"}=" H62 DOP"; $resn_exch{"1H7  DOP"}=" H71 DOP";
$resn_exch{"2H7  DOP"}=" H72 DOP"; $resn_exch{"1H8  DOP"}=" H81 DOP"; $resn_exch{"2H8  DOP"}=" H82 DOP";
$resn_exch{"3H8  DOP"}=" H83 DOP"; $resn_exch{"1H1' DOP"}="H1'1 DOP"; $resn_exch{"2H1' DOP"}="H1'2 DOP";
$resn_exch{"1H2' DOP"}="H2'1 DOP"; $resn_exch{"2H2' DOP"}="H2'2 DOP"; $resn_exch{"1H3' DOP"}="H3'1 DOP";
$resn_exch{"2H3' DOP"}="H3'2 DOP"; $resn_exch{"1H4' DOP"}="H4'1 DOP"; $resn_exch{"2H4' DOP"}="H4'2 DOP";
$resn_exch{"1H5' DOP"}="H5'1 DOP"; $resn_exch{"2H5' DOP"}="H5'2 DOP"; $resn_exch{"1H6' DOP"}="H6'1 DOP";
$resn_exch{"2H6' DOP"}="H6'2 DOP"; $resn_exch{"1H7' DOP"}="H7'1 DOP"; $resn_exch{"2H7' DOP"}="H7'2 DOP";
$resn_exch{"1H8' DOP"}="H8'1 DOP"; $resn_exch{"2H8' DOP"}="H8'2 DOP"; $resn_exch{"3H8' DOP"}="H8'3 DOP";
$resn_exch{"1HAB DOQ"}="HAB1 DOQ"; $resn_exch{"2HAB DOQ"}="HAB2 DOQ"; $resn_exch{"3HAB DOQ"}="HAB3 DOQ";
$resn_exch{"1HAA DOQ"}="HAA1 DOQ"; $resn_exch{"2HAA DOQ"}="HAA2 DOQ"; $resn_exch{"3HAA DOQ"}="HAA3 DOQ";
$resn_exch{"1HAL DOQ"}="HAL1 DOQ"; $resn_exch{"2HAL DOQ"}="HAL2 DOQ"; $resn_exch{"1HG  DOQ"}=" HG1 DOQ";
$resn_exch{"2HG  DOQ"}=" HG2 DOQ"; $resn_exch{"1H5  DOR"}=" H51 DOR"; $resn_exch{"2H5  DOR"}=" H52 DOR";
$resn_exch{"1H3  DOS"}=" H31 DOS"; $resn_exch{"2H3  DOS"}=" H32 DOS"; $resn_exch{"1H4  DOS"}=" H41 DOS";
$resn_exch{"2H4  DOS"}=" H42 DOS"; $resn_exch{"1H5  DOS"}=" H51 DOS"; $resn_exch{"1H6  DOS"}=" H61 DOS";
$resn_exch{"7H7  DOS"}=" H77 DOS"; $resn_exch{"8H8  DOS"}=" H88 DOS"; $resn_exch{"1H9  DOS"}=" H91 DOS";
$resn_exch{"2H9  DOS"}=" H92 DOS"; $resn_exch{"1H10 DOS"}="H101 DOS"; $resn_exch{"2H10 DOS"}="H102 DOS";
$resn_exch{"1H11 DOS"}="H111 DOS"; $resn_exch{"2H11 DOS"}="H112 DOS"; $resn_exch{"1H12 DOS"}="H121 DOS";
$resn_exch{"2H12 DOS"}="H122 DOS"; $resn_exch{"1H27 DOS"}="H271 DOS"; $resn_exch{"2H27 DOS"}="H272 DOS";
$resn_exch{"1H28 DOS"}="H281 DOS"; $resn_exch{"2H28 DOS"}="H282 DOS"; $resn_exch{"1H29 DOS"}="H291 DOS";
$resn_exch{"2H29 DOS"}="H292 DOS"; $resn_exch{"1H30 DOS"}="H301 DOS"; $resn_exch{"2H30 DOS"}="H302 DOS";
$resn_exch{"31H3 DOS"}="H331 DOS"; $resn_exch{"32H3 DOS"}="H332 DOS"; $resn_exch{"1H33 DOS"}="H33A DOS";
$resn_exch{"2H33 DOS"}="H33B DOS"; $resn_exch{"1H34 DOS"}="H341 DOS"; $resn_exch{"2H34 DOS"}="H342 DOS";
$resn_exch{"1H35 DOS"}="H351 DOS"; $resn_exch{"2H35 DOS"}="H352 DOS"; $resn_exch{"1H36 DOS"}="H361 DOS";
$resn_exch{"2H36 DOS"}="H362 DOS"; $resn_exch{"1HG  DOS"}=" HG1 DOS"; $resn_exch{"2HG  DOS"}=" HG2 DOS";
$resn_exch{"1HD2 DOS"}="HD21 DOS"; $resn_exch{"2HD2 DOS"}="HD22 DOS"; $resn_exch{"1HE1 DOS"}="HE11 DOS";
$resn_exch{"2HE1 DOS"}="HE12 DOS"; $resn_exch{"1HE2 DOS"}="HE21 DOS"; $resn_exch{"1HOG DOT"}="HOG1 DOT";
$resn_exch{"3HOG DOT"}="HOG3 DOT"; $resn_exch{"2HOB DOT"}="HOB2 DOT"; $resn_exch{"2HOA DOT"}="HOA2 DOT";
$resn_exch{"1H5* DOT"}="H5'1 DOT"; $resn_exch{"2H5* DOT"}="H5'2 DOT"; $resn_exch{" H4* DOT"}=" H4B DOT";
$resn_exch{" H3* DOT"}=" H3B DOT"; $resn_exch{"1H2* DOT"}="H2'1 DOT"; $resn_exch{"2H2* DOT"}="H2'2 DOT";
$resn_exch{" H1* DOT"}=" H1' DOT"; $resn_exch{"1HN6 DOT"}="HN61 DOT"; $resn_exch{"2HN6 DOT"}="HN62 DOT";
$resn_exch{"1H2' DOT"}=" H2B DOT"; $resn_exch{"2H2' DOT"}=" H2D DOT"; $resn_exch{"1H1  DOX"}=" H11 DOX";
$resn_exch{"2H1  DOX"}=" H12 DOX"; $resn_exch{"1H2  DOX"}=" H21 DOX"; $resn_exch{"2H2  DOX"}=" H22 DOX";
$resn_exch{"1H3  DOX"}=" H31 DOX"; $resn_exch{"2H3  DOX"}=" H32 DOX"; $resn_exch{"1H4  DOX"}=" H41 DOX";
$resn_exch{"2H4  DOX"}=" H42 DOX"; $resn_exch{"1HD  DP1"}=" HD1 DP1"; $resn_exch{"2HD  DP1"}=" HD2 DP1";
$resn_exch{"1HG  DP1"}=" HG1 DP1"; $resn_exch{"2HG  DP1"}=" HG2 DP1"; $resn_exch{"1HB  DP1"}=" HB1 DP1";
$resn_exch{"2HB  DP1"}=" HB2 DP1"; $resn_exch{"1HN  DP1"}=" HN1 DP1"; $resn_exch{"2HN  DP1"}=" HN2 DP1";
$resn_exch{"1H1' DP1"}="H1'1 DP1"; $resn_exch{"2H1' DP1"}="H1'2 DP1"; $resn_exch{"1HB' DP1"}="HB'1 DP1";
$resn_exch{"2HB' DP1"}="HB'2 DP1"; $resn_exch{"1H1  DP1"}=" H11 DP1"; $resn_exch{"2H1  DP1"}=" H12 DP1";
$resn_exch{"1HN1 DP1"}="HN11 DP1"; $resn_exch{"2HN1 DP1"}="HN12 DP1"; $resn_exch{"1HN2 DP2"}="HN21 DP2";
$resn_exch{"2HN2 DP2"}="HN22 DP2"; $resn_exch{"1HB  DP2"}=" HB1 DP2"; $resn_exch{"2HB  DP2"}=" HB2 DP2";
$resn_exch{"1HG  DP2"}=" HG1 DP2"; $resn_exch{"2HG  DP2"}=" HG2 DP2"; $resn_exch{"1HD  DP2"}=" HD1 DP2";
$resn_exch{"2HD  DP2"}=" HD2 DP2"; $resn_exch{"1HN' DP2"}="HN'1 DP2"; $resn_exch{"2HN' DP2"}="HN'2 DP2";
$resn_exch{"1HB' DP2"}="HB'1 DP2"; $resn_exch{"2HB' DP2"}="HB'2 DP2"; $resn_exch{"1HG' DP2"}="HG'1 DP2";
$resn_exch{"2HG' DP2"}="HG'2 DP2"; $resn_exch{"1HD' DP2"}="HD'1 DP2"; $resn_exch{"2HD' DP2"}="HD'2 DP2";
$resn_exch{"1HE' DP2"}="HE'1 DP2"; $resn_exch{"2HE' DP2"}="HE'2 DP2"; $resn_exch{"1HZ' DP2"}="HZ'1 DP2";
$resn_exch{"2HZ' DP2"}="HZ'2 DP2"; $resn_exch{"1H2' DP3"}="H2'1 DP3"; $resn_exch{"2H2' DP3"}="H2'2 DP3";
$resn_exch{"1H1  DP3"}=" H11 DP3"; $resn_exch{"2H1  DP3"}=" H12 DP3"; $resn_exch{"1H1' DP3"}="H1'1 DP3";
$resn_exch{"2H1' DP3"}="H1'2 DP3"; $resn_exch{"1H   DP3"}=" H1  DP3"; $resn_exch{"2H   DP3"}=" H2  DP3";
$resn_exch{"1HN  DP3"}=" HN1 DP3"; $resn_exch{"2HN  DP3"}=" HN2 DP3"; $resn_exch{"1HB  DP3"}=" HB1 DP3";
$resn_exch{"2HB  DP3"}=" HB2 DP3"; $resn_exch{"1HG  DP3"}=" HG1 DP3"; $resn_exch{"2HG  DP3"}=" HG2 DP3";
$resn_exch{"1HD  DP3"}=" HD1 DP3"; $resn_exch{"2HD  DP3"}=" HD2 DP3"; $resn_exch{"1HC7 DP4"}="HC71 DP4";
$resn_exch{"2HC7 DP4"}="HC72 DP4"; $resn_exch{"3HC7 DP4"}="HC73 DP4"; $resn_exch{"1HC8 DP4"}="HC81 DP4";
$resn_exch{"2HC8 DP4"}="HC82 DP4"; $resn_exch{"3HC8 DP4"}="HC83 DP4"; $resn_exch{"1H10 DP4"}="H101 DP4";
$resn_exch{"2H10 DP4"}="H102 DP4"; $resn_exch{"1H11 DP4"}="H111 DP4"; $resn_exch{"2H11 DP4"}="H112 DP4";
$resn_exch{"1H12 DP4"}="H121 DP4"; $resn_exch{"2H12 DP4"}="H122 DP4"; $resn_exch{"1H13 DP4"}="H131 DP4";
$resn_exch{"2H13 DP4"}="H132 DP4"; $resn_exch{"1H14 DP4"}="H141 DP4"; $resn_exch{"2H14 DP4"}="H142 DP4";
$resn_exch{"3H14 DP4"}="H143 DP4"; $resn_exch{"1H6A DP5"}="H6A1 DP5"; $resn_exch{"2H6A DP5"}="H6A2 DP5";
$resn_exch{"AHO2 DP5"}="HO2A DP5"; $resn_exch{"AHO3 DP5"}="HO3A DP5"; $resn_exch{"AHS4 DP5"}="HS4A DP5";
$resn_exch{"AHO6 DP5"}="HO6A DP5"; $resn_exch{"1H6B DP5"}="H6B1 DP5"; $resn_exch{"2H6B DP5"}="H6B2 DP5";
$resn_exch{"BHO2 DP5"}="HO2B DP5"; $resn_exch{"BHO3 DP5"}="HO3B DP5"; $resn_exch{"BHO6 DP5"}="HO6B DP5";
$resn_exch{"1H6C DP5"}="H6C1 DP5"; $resn_exch{"2H6C DP5"}="H6C2 DP5"; $resn_exch{"CHS1 DP5"}="HS1C DP5";
$resn_exch{"CHO2 DP5"}="HO2C DP5"; $resn_exch{"CHO3 DP5"}="HO3C DP5"; $resn_exch{"CHO6 DP5"}="HO6C DP5";
$resn_exch{"1H3A DP6"}="H3A1 DP6"; $resn_exch{"2H3A DP6"}="H3A2 DP6"; $resn_exch{"3H3A DP6"}="H3A3 DP6";
$resn_exch{"1H5  DP6"}=" H51 DP6"; $resn_exch{"2H5  DP6"}=" H52 DP6"; $resn_exch{"1H4  DP6"}=" H41 DP6";
$resn_exch{"2H4  DP6"}=" H42 DP6"; $resn_exch{"1H2  DP6"}=" H21 DP6"; $resn_exch{"2H2  DP6"}=" H22 DP6";
$resn_exch{"1H3  DP7"}=" H31 DP7"; $resn_exch{"2H3  DP7"}=" H32 DP7"; $resn_exch{"1H4  DP7"}=" H41 DP7";
$resn_exch{"2H4  DP7"}=" H42 DP7"; $resn_exch{"1H5  DP7"}=" H51 DP7"; $resn_exch{"2H5  DP7"}=" H52 DP7";
$resn_exch{"1H9  DP7"}=" H91 DP7"; $resn_exch{"2H9  DP7"}=" H92 DP7"; $resn_exch{"1H10 DP7"}="H101 DP7";
$resn_exch{"2H10 DP7"}="H102 DP7"; $resn_exch{"1H11 DP7"}="H111 DP7"; $resn_exch{"2H11 DP7"}="H112 DP7";
$resn_exch{"1H13 DP7"}="H131 DP7"; $resn_exch{"2H13 DP7"}="H132 DP7"; $resn_exch{"1H16 DP7"}="H161 DP7";
$resn_exch{"2H16 DP7"}="H162 DP7"; $resn_exch{"3H16 DP7"}="H163 DP7"; $resn_exch{"1HN3 DP7"}="HN31 DP7";
$resn_exch{"2HN3 DP7"}="HN32 DP7"; $resn_exch{"1HD  DP9"}=" HD1 DP9"; $resn_exch{"2HD  DP9"}=" HD2 DP9";
$resn_exch{"1HG  DP9"}=" HG1 DP9"; $resn_exch{"2HG  DP9"}=" HG2 DP9"; $resn_exch{"1HB  DP9"}=" HB1 DP9";
$resn_exch{"2HB  DP9"}=" HB2 DP9"; $resn_exch{"1HN  DP9"}=" HN1 DP9"; $resn_exch{"2HN  DP9"}=" HN2 DP9";
$resn_exch{"1HD' DP9"}="HD'1 DP9"; $resn_exch{"2HD' DP9"}="HD'2 DP9"; $resn_exch{"1H1' DP9"}="H1'1 DP9";
$resn_exch{"2H1' DP9"}="H1'2 DP9"; $resn_exch{"1HB' DP9"}="HB'1 DP9"; $resn_exch{"2HB' DP9"}="HB'2 DP9";
$resn_exch{"1H10 DPA"}="H101 DPA"; $resn_exch{"2H10 DPA"}="H102 DPA"; $resn_exch{"1H11 DPA"}="H111 DPA";
$resn_exch{"2H11 DPA"}="H112 DPA"; $resn_exch{"3H11 DPA"}="H113 DPA"; $resn_exch{"1H12 DPA"}="H121 DPA";
$resn_exch{"2H12 DPA"}="H122 DPA"; $resn_exch{"3H12 DPA"}="H123 DPA"; $resn_exch{"1H13 DPA"}="H131 DPA";
$resn_exch{"2H13 DPA"}="H132 DPA"; $resn_exch{"3H13 DPA"}="H133 DPA"; $resn_exch{"1H5M DPB"}=" H71 DPB";
$resn_exch{"2H5M DPB"}=" H72 DPB"; $resn_exch{"3H5M DPB"}=" H73 DPB"; $resn_exch{" H1* DPB"}=" H1' DPB";
$resn_exch{" H4* DPB"}=" H4' DPB"; $resn_exch{" H3* DPB"}=" H3' DPB"; $resn_exch{"1H2* DPB"}=" H2' DPB";
$resn_exch{"2H2* DPB"}="H2'' DPB"; $resn_exch{"1H5* DPB"}=" H5' DPB"; $resn_exch{"2H5* DPB"}="H5'' DPB";
$resn_exch{"2HOP DPB"}="HOP2 DPB"; $resn_exch{"3HOP DPB"}="HOP3 DPB"; $resn_exch{"1H11 DPC"}="H111 DPC";
$resn_exch{"2H11 DPC"}="H112 DPC"; $resn_exch{"3H11 DPC"}="H113 DPC"; $resn_exch{"1H81 DPC"}="H811 DPC";
$resn_exch{"2H81 DPC"}="H812 DPC"; $resn_exch{"1H82 DPC"}="H821 DPC"; $resn_exch{"2H82 DPC"}="H822 DPC";
$resn_exch{"1H9  DPC"}=" H91 DPC"; $resn_exch{"2H9  DPC"}=" H92 DPC"; $resn_exch{"1H91 DPC"}="H911 DPC";
$resn_exch{"2H91 DPC"}="H912 DPC"; $resn_exch{"1H92 DPC"}="H921 DPC"; $resn_exch{"2H92 DPC"}="H922 DPC";
$resn_exch{"3H92 DPC"}="H923 DPC"; $resn_exch{"1HNE DPC"}="HNE1 DPC"; $resn_exch{"2HNE DPC"}="HNE2 DPC";
$resn_exch{"2H2  DPD"}="H22A DPD"; $resn_exch{"1H2  DPD"}=" H21 DPD"; $resn_exch{"2H3  DPD"}="H32A DPD";
$resn_exch{"1H3  DPD"}=" H31 DPD"; $resn_exch{"2H5  DPD"}=" H52 DPD"; $resn_exch{"1H5  DPD"}=" H51 DPD";
$resn_exch{"2H6  DPD"}=" H62 DPD"; $resn_exch{"1H6  DPD"}=" H61 DPD"; $resn_exch{"2H7  DPD"}=" H72 DPD";
$resn_exch{"1H7  DPD"}=" H71 DPD"; $resn_exch{"3H19 DPD"}="H193 DPD"; $resn_exch{"2H19 DPD"}="H192 DPD";
$resn_exch{"1H19 DPD"}="H191 DPD"; $resn_exch{"2H41 DPD"}="H412 DPD"; $resn_exch{"1H41 DPD"}="H411 DPD";
$resn_exch{"2H42 DPD"}="H422 DPD"; $resn_exch{"1H42 DPD"}="H421 DPD"; $resn_exch{"2H43 DPD"}="H432 DPD";
$resn_exch{"1H43 DPD"}="H431 DPD"; $resn_exch{"3H44 DPD"}="H443 DPD"; $resn_exch{"2H44 DPD"}="H442 DPD";
$resn_exch{"1H44 DPD"}="H441 DPD"; $resn_exch{"2H51 DPD"}="H512 DPD"; $resn_exch{"1H51 DPD"}="H511 DPD";
$resn_exch{"2H52 DPD"}="H522 DPD"; $resn_exch{"1H52 DPD"}="H521 DPD"; $resn_exch{"2H53 DPD"}="H532 DPD";
$resn_exch{"1H53 DPD"}="H531 DPD"; $resn_exch{"3H54 DPD"}="H543 DPD"; $resn_exch{"2H54 DPD"}="H542 DPD";
$resn_exch{"1H54 DPD"}="H541 DPD"; $resn_exch{"1H1  DPE"}=" H11 DPE"; $resn_exch{"2H1  DPE"}=" H12 DPE";
$resn_exch{"1H2  DPE"}=" H21 DPE"; $resn_exch{"2H2  DPE"}=" H22 DPE"; $resn_exch{"1H3  DPE"}=" H31 DPE";
$resn_exch{"2H3  DPE"}=" H32 DPE"; $resn_exch{"1H4  DPE"}=" H41 DPE"; $resn_exch{"2H4  DPE"}=" H42 DPE";
$resn_exch{"1H5  DPE"}=" H51 DPE"; $resn_exch{"2H5  DPE"}=" H52 DPE"; $resn_exch{"1H6  DPE"}=" H61 DPE";
$resn_exch{"2H6  DPE"}=" H62 DPE"; $resn_exch{"1H7  DPE"}=" H71 DPE"; $resn_exch{"2H7  DPE"}=" H72 DPE";
$resn_exch{"1H8  DPE"}=" H81 DPE"; $resn_exch{"2H8  DPE"}=" H82 DPE"; $resn_exch{"1H9  DPE"}=" H91 DPE";
$resn_exch{"2H9  DPE"}=" H92 DPE"; $resn_exch{"1H10 DPE"}="H101 DPE"; $resn_exch{"2H10 DPE"}="H102 DPE";
$resn_exch{"1H11 DPE"}="H111 DPE"; $resn_exch{"2H11 DPE"}="H112 DPE"; $resn_exch{"3H11 DPE"}="H113 DPE";
$resn_exch{"1HM  DPE"}=" HM1 DPE"; $resn_exch{"2HM  DPE"}=" HM2 DPE"; $resn_exch{"3HM  DPE"}=" HM3 DPE";
$resn_exch{"1H4  DPF"}=" H41 DPF"; $resn_exch{"2H4  DPF"}=" H42 DPF"; $resn_exch{"3H4  DPF"}=" H43 DPF";
$resn_exch{"1H1  DPF"}=" H11 DPF"; $resn_exch{"2H1  DPF"}=" H12 DPF"; $resn_exch{"1H2  DPF"}=" H21 DPF";
$resn_exch{"2H2  DPF"}=" H22 DPF"; $resn_exch{"1H3  DPF"}=" H31 DPF"; $resn_exch{"2H3  DPF"}=" H32 DPF";
$resn_exch{"3H3  DPF"}=" H33 DPF"; $resn_exch{"1H1  DPG"}=" H11 DPG"; $resn_exch{"2H1  DPG"}=" H12 DPG";
$resn_exch{"1H3  DPG"}=" H31 DPG"; $resn_exch{"2H3  DPG"}=" H32 DPG"; $resn_exch{"1H4  DPG"}=" H41 DPG";
$resn_exch{"2H4  DPG"}=" H42 DPG"; $resn_exch{"1H6  DPG"}=" H61 DPG"; $resn_exch{"2H6  DPG"}=" H62 DPG";
$resn_exch{"1H11 DPG"}="H111 DPG"; $resn_exch{"2H11 DPG"}="H112 DPG"; $resn_exch{"1H12 DPG"}="H121 DPG";
$resn_exch{"2H12 DPG"}="H122 DPG"; $resn_exch{"1H14 DPG"}="H141 DPG"; $resn_exch{"2H14 DPG"}="H142 DPG";
$resn_exch{"3H14 DPG"}="H143 DPG"; $resn_exch{"1H15 DPG"}="H151 DPG"; $resn_exch{"2H15 DPG"}="H152 DPG";
$resn_exch{"1H16 DPG"}="H161 DPG"; $resn_exch{"2H16 DPG"}="H162 DPG"; $resn_exch{"1H17 DPG"}="H171 DPG";
$resn_exch{"2H17 DPG"}="H172 DPG"; $resn_exch{"1H19 DPG"}="H191 DPG"; $resn_exch{"2H19 DPG"}="H192 DPG";
$resn_exch{"3H19 DPG"}="H193 DPG"; $resn_exch{"1H20 DPG"}="H201 DPG"; $resn_exch{"2H20 DPG"}="H202 DPG";
$resn_exch{"1H21 DPG"}="H211 DPG"; $resn_exch{"2H21 DPG"}="H212 DPG"; $resn_exch{"1H22 DPG"}="H221 DPG";
$resn_exch{"2H22 DPG"}="H222 DPG"; $resn_exch{"1H24 DPG"}="H241 DPG"; $resn_exch{"2H24 DPG"}="H242 DPG";
$resn_exch{"3H24 DPG"}="H243 DPG"; $resn_exch{"1H25 DPG"}="H251 DPG"; $resn_exch{"2H25 DPG"}="H252 DPG";
$resn_exch{"1H26 DPG"}="H261 DPG"; $resn_exch{"2H26 DPG"}="H262 DPG"; $resn_exch{"1H27 DPG"}="H271 DPG";
$resn_exch{"2H27 DPG"}="H272 DPG"; $resn_exch{"1H29 DPG"}="H291 DPG"; $resn_exch{"2H29 DPG"}="H292 DPG";
$resn_exch{"3H29 DPG"}="H293 DPG"; $resn_exch{"1H30 DPG"}="H301 DPG"; $resn_exch{"2H30 DPG"}="H302 DPG";
$resn_exch{"3H30 DPG"}="H303 DPG"; $resn_exch{"1H41 DPG"}="H411 DPG"; $resn_exch{"2H41 DPG"}="H412 DPG";
$resn_exch{"1H42 DPG"}="H421 DPG"; $resn_exch{"2H42 DPG"}="H422 DPG"; $resn_exch{"1H44 DPG"}="H441 DPG";
$resn_exch{"2H44 DPG"}="H442 DPG"; $resn_exch{"3H44 DPG"}="H443 DPG"; $resn_exch{"1H45 DPG"}="H451 DPG";
$resn_exch{"2H45 DPG"}="H452 DPG"; $resn_exch{"1H46 DPG"}="H461 DPG"; $resn_exch{"2H46 DPG"}="H462 DPG";
$resn_exch{"1H47 DPG"}="H471 DPG"; $resn_exch{"2H47 DPG"}="H472 DPG"; $resn_exch{"1H49 DPG"}="H491 DPG";
$resn_exch{"2H49 DPG"}="H492 DPG"; $resn_exch{"3H49 DPG"}="H493 DPG"; $resn_exch{"1H50 DPG"}="H501 DPG";
$resn_exch{"2H50 DPG"}="H502 DPG"; $resn_exch{"1H51 DPG"}="H511 DPG"; $resn_exch{"2H51 DPG"}="H512 DPG";
$resn_exch{"1H52 DPG"}="H521 DPG"; $resn_exch{"2H52 DPG"}="H522 DPG"; $resn_exch{"1H54 DPG"}="H541 DPG";
$resn_exch{"2H54 DPG"}="H542 DPG"; $resn_exch{"3H54 DPG"}="H543 DPG"; $resn_exch{"1H55 DPG"}="H551 DPG";
$resn_exch{"2H55 DPG"}="H552 DPG"; $resn_exch{"1H56 DPG"}="H561 DPG"; $resn_exch{"2H56 DPG"}="H562 DPG";
$resn_exch{"1H57 DPG"}="H571 DPG"; $resn_exch{"2H57 DPG"}="H572 DPG"; $resn_exch{"1H59 DPG"}="H591 DPG";
$resn_exch{"2H59 DPG"}="H592 DPG"; $resn_exch{"3H59 DPG"}="H593 DPG"; $resn_exch{"1H60 DPG"}="H601 DPG";
$resn_exch{"2H60 DPG"}="H602 DPG"; $resn_exch{"3H60 DPG"}="H603 DPG"; $resn_exch{"1HM  DPH"}=" HM1 DPH";
$resn_exch{"2HM  DPH"}=" HM2 DPH"; $resn_exch{"3HM  DPH"}=" HM3 DPH"; $resn_exch{"1HB  DPH"}=" HB1 DPH";
$resn_exch{"2HB  DPH"}=" HB2 DPH"; $resn_exch{"1HC  DPI"}=" HC1 DPI"; $resn_exch{"2HC  DPI"}=" HC2 DPI";
$resn_exch{"3HC  DPI"}=" HC3 DPI"; $resn_exch{"2HN  DPI"}=" HN2 DPI"; $resn_exch{"1HC6 DPI"}="HC61 DPI";
$resn_exch{"2HC6 DPI"}="HC62 DPI"; $resn_exch{"7HC  DPI"}=" HC7 DPI"; $resn_exch{"1HA  DPI"}=" HA1 DPI";
$resn_exch{"2HB  DPI"}=" HB2 DPI"; $resn_exch{"3HD  DPI"}=" HD3 DPI"; $resn_exch{"1HB  DPI"}=" HB1 DPI";
$resn_exch{"2HD  DPI"}=" HD2 DPI"; $resn_exch{"3HE  DPI"}=" HE3 DPI"; $resn_exch{"1HN0 DPI"}="HN01 DPI";
$resn_exch{"1HC1 DPI"}="HC11 DPI"; $resn_exch{"1HC4 DPI"}="HC41 DPI"; $resn_exch{"1HO5 DPI"}="HO51 DPI";
$resn_exch{"1HC7 DPI"}="HC71 DPI"; $resn_exch{"1HC8 DPI"}="HC81 DPI"; $resn_exch{"2HC0 DPI"}="HC02 DPI";
$resn_exch{"2HN4 DPI"}="HN42 DPI"; $resn_exch{"2HC5 DPI"}="HC52 DPI"; $resn_exch{"3HN2 DPI"}="HN23 DPI";
$resn_exch{"3HC3 DPI"}="HC33 DPI"; $resn_exch{"3HC7 DPI"}="HC73 DPI"; $resn_exch{"3HC8 DPI"}="HC83 DPI";
$resn_exch{"4HN3 DPI"}="HN34 DPI"; $resn_exch{"4HC4 DPI"}="HC44 DPI"; $resn_exch{"4HC8 DPI"}="HC84 DPI";
$resn_exch{"4HC9 DPI"}="HC94 DPI"; $resn_exch{"5HC1 DPI"}="HC15 DPI"; $resn_exch{"5HO3 DPI"}="HO35 DPI";
$resn_exch{"5HN4 DPI"}="HN45 DPI"; $resn_exch{"5HC5 DPI"}="HC55 DPI"; $resn_exch{"5HC8 DPI"}="HC85 DPI";
$resn_exch{"6HC1 DPI"}="HC16 DPI"; $resn_exch{"6HC2 DPI"}="HC26 DPI"; $resn_exch{"6HC3 DPI"}="HC36 DPI";
$resn_exch{"6HC4 DPI"}="HC46 DPI"; $resn_exch{"6HN8 DPI"}="HN86 DPI"; $resn_exch{"6HC9 DPI"}="HC96 DPI";
$resn_exch{"7HO1 DPI"}="HO17 DPI"; $resn_exch{"7HC4 DPI"}="HC47 DPI"; $resn_exch{"7HC6 DPI"}="HC67 DPI";
$resn_exch{"7HO8 DPI"}="HO87 DPI"; $resn_exch{"7HO9 DPI"}="HO97 DPI"; $resn_exch{"1HC' DPI"}="HC'1 DPI";
$resn_exch{"1H2' DPI"}="H2'1 DPI"; $resn_exch{"2H2' DPI"}="H2'2 DPI"; $resn_exch{"4HC' DPI"}="HC'4 DPI";
$resn_exch{"5HC' DPI"}="HC'5 DPI"; $resn_exch{"1H7' DPI"}="H7'1 DPI"; $resn_exch{"2H7' DPI"}="H7'2 DPI";
$resn_exch{"3H7' DPI"}="H7'3 DPI"; $resn_exch{"1H8' DPI"}="H8'1 DPI"; $resn_exch{"2H8' DPI"}="H8'2 DPI";
$resn_exch{"9HO' DPI"}="HO'9 DPI"; $resn_exch{"1H1\" DPI"}="H1'1 DPI"; $resn_exch{"2H1\" DPI"}="H1'2 DPI";
$resn_exch{"3H1\" DPI"}="H1'3 DPI"; $resn_exch{"2HC\" DPI"}="HC'2 DPI"; $resn_exch{"3HC\" DPI"}="HC'3 DPI";
$resn_exch{"4HC\" DPI"}="HCA4 DPI"; $resn_exch{"5HC\" DPI"}="HCA5 DPI"; $resn_exch{"6HC\" DPI"}="HC'6 DPI";
$resn_exch{"9HO\" DPI"}="HOA9 DPI"; $resn_exch{"1HOA DPI"}="HOA1 DPI"; $resn_exch{"4HCA DPI"}="HCAA DPI";
$resn_exch{"5HCA DPI"}="HCAB DPI"; $resn_exch{"6HCA DPI"}="HCA6 DPI"; $resn_exch{"7HCA DPI"}="HCA7 DPI";
$resn_exch{"8HCA DPI"}="HCA8 DPI"; $resn_exch{"1HOB DPI"}="HOB1 DPI"; $resn_exch{"2HOB DPI"}="HOB2 DPI";
$resn_exch{"3HOB DPI"}="HOB3 DPI"; $resn_exch{"1H4B DPI"}="H4B1 DPI"; $resn_exch{"2H4B DPI"}="H4B2 DPI";
$resn_exch{"3H4B DPI"}="H4B3 DPI"; $resn_exch{"1H28 DPI"}="H281 DPI"; $resn_exch{"2H28 DPI"}="H282 DPI";
$resn_exch{"1H31 DPI"}="H311 DPI"; $resn_exch{"2H31 DPI"}="H312 DPI"; $resn_exch{"1H2A DPI"}="H2A1 DPI";
$resn_exch{"2H2A DPI"}="H2A2 DPI"; $resn_exch{"3HOA DPI"}="HOA3 DPI"; $resn_exch{"1H9  DPJ"}=" H91 DPJ";
$resn_exch{"2H9  DPJ"}=" H92 DPJ"; $resn_exch{"3H9  DPJ"}=" H93 DPJ"; $resn_exch{"1H7  DPJ"}=" H71 DPJ";
$resn_exch{"2H7  DPJ"}=" H72 DPJ"; $resn_exch{"1H6  DPJ"}=" H61 DPJ"; $resn_exch{"2H6  DPJ"}=" H62 DPJ";
$resn_exch{"1H8  DPJ"}=" H81 DPJ"; $resn_exch{"2H8  DPJ"}=" H82 DPJ"; $resn_exch{"3H8  DPJ"}=" H83 DPJ";
$resn_exch{"1H12 DPK"}="H121 DPK"; $resn_exch{"2H12 DPK"}="H122 DPK"; $resn_exch{"1H10 DPK"}="H101 DPK";
$resn_exch{"2H10 DPK"}="H102 DPK"; $resn_exch{"1H9N DPK"}="H9N1 DPK"; $resn_exch{"2H9N DPK"}="H9N2 DPK";
$resn_exch{"3H9N DPK"}="H9N3 DPK"; $resn_exch{"1H8C DPK"}="H8C1 DPK"; $resn_exch{"2H8C DPK"}="H8C2 DPK";
$resn_exch{"3H8C DPK"}="H8C3 DPK"; $resn_exch{"1H7C DPK"}="H7C1 DPK"; $resn_exch{"2H7C DPK"}="H7C2 DPK";
$resn_exch{"1HB  DPL"}=" HB2 DPL"; $resn_exch{"2HB  DPL"}=" HB3 DPL"; $resn_exch{"1HD  DPL"}=" HD3 DPL";
$resn_exch{"2HD  DPL"}=" HD2 DPL"; $resn_exch{"1H5A DPM"}="H5A1 DPM"; $resn_exch{"2H5A DPM"}="H5A2 DPM";
$resn_exch{"1H7A DPM"}="H7A1 DPM"; $resn_exch{"2H7A DPM"}="H7A2 DPM"; $resn_exch{"1H8A DPM"}="H8A1 DPM";
$resn_exch{"2H8A DPM"}="H8A2 DPM"; $resn_exch{"1HCA DPM"}="HCA1 DPM"; $resn_exch{"2HCA DPM"}="HCA2 DPM";
$resn_exch{"3HCA DPM"}="HCA3 DPM"; $resn_exch{"2HOA DPM"}="HOA2 DPM"; $resn_exch{"4HOA DPM"}="HOA4 DPM";
$resn_exch{"1H4B DPM"}="H4B1 DPM"; $resn_exch{"2H4B DPM"}="H4B2 DPM"; $resn_exch{"1H5B DPM"}="H5B1 DPM";
$resn_exch{"2H5B DPM"}="H5B2 DPM"; $resn_exch{"1H7B DPM"}="H7B1 DPM"; $resn_exch{"2H7B DPM"}="H7B2 DPM";
$resn_exch{"1H8B DPM"}="H8B1 DPM"; $resn_exch{"2H8B DPM"}="H8B2 DPM"; $resn_exch{"1HCB DPM"}="HCB1 DPM";
$resn_exch{"2HCB DPM"}="HCB2 DPM"; $resn_exch{"2HOB DPM"}="HOB2 DPM"; $resn_exch{"4HOB DPM"}="HOB4 DPM";
$resn_exch{" HN2 DPN"}=" H2  DPN"; $resn_exch{"1HB  DPN"}=" HB2 DPN"; $resn_exch{"2HB  DPN"}=" HB3 DPN";
$resn_exch{"1HN  DPP"}=" H   DPP"; $resn_exch{"2HN  DPP"}=" H2  DPP"; $resn_exch{"1HB  DPP"}=" HB2 DPP";
$resn_exch{"2HB  DPP"}=" HB3 DPP"; $resn_exch{"1HG  DPP"}=" HG1 DPP"; $resn_exch{"2HG  DPP"}=" HG2 DPP";
$resn_exch{"1HB  DPQ"}=" HB1 DPQ"; $resn_exch{"2HB  DPQ"}=" HB2 DPQ"; $resn_exch{"1HD1 DPQ"}="HD11 DPQ";
$resn_exch{"2HD1 DPQ"}="HD12 DPQ"; $resn_exch{" HT1 DPR"}=" H   DPR"; $resn_exch{"1HB  DPR"}=" HB2 DPR";
$resn_exch{"2HB  DPR"}=" HB3 DPR"; $resn_exch{"1HG  DPR"}=" HG2 DPR"; $resn_exch{"2HG  DPR"}=" HG3 DPR";
$resn_exch{"1HD  DPR"}=" HD2 DPR"; $resn_exch{"2HD  DPR"}=" HD3 DPR"; $resn_exch{"1H7  DPS"}=" H71 DPS";
$resn_exch{"2H7  DPS"}=" H72 DPS"; $resn_exch{"1H13 DPS"}="H131 DPS"; $resn_exch{"2H13 DPS"}="H132 DPS";
$resn_exch{"1H15 DPS"}="H151 DPS"; $resn_exch{"2H15 DPS"}="H152 DPS"; $resn_exch{"1H18 DPS"}="H181 DPS";
$resn_exch{"2H18 DPS"}="H182 DPS"; $resn_exch{"1H20 DPS"}="H201 DPS"; $resn_exch{"2H20 DPS"}="H202 DPS";
$resn_exch{"1H11 DPT"}="H111 DPT"; $resn_exch{"2H11 DPT"}="H112 DPT"; $resn_exch{"3H11 DPT"}="H113 DPT";
$resn_exch{"1H12 DPT"}="H121 DPT"; $resn_exch{"2H12 DPT"}="H122 DPT"; $resn_exch{"3H12 DPT"}="H123 DPT";
$resn_exch{"1HA  DPU"}=" HA1 DPU"; $resn_exch{"2HA  DPU"}=" HA2 DPU"; $resn_exch{"1HB  DPU"}=" HB1 DPU";
$resn_exch{"2HB  DPU"}=" HB2 DPU"; $resn_exch{"1HG  DPU"}=" HG1 DPU"; $resn_exch{"2HG  DPU"}=" HG2 DPU";
$resn_exch{"1HD  DPU"}=" HD1 DPU"; $resn_exch{"2HD  DPU"}=" HD2 DPU"; $resn_exch{"1HE  DPU"}=" HE1 DPU";
$resn_exch{"2HE  DPU"}=" HE2 DPU"; $resn_exch{"1HS  DPU"}=" HS1 DPU"; $resn_exch{"2HS  DPU"}=" HS2 DPU";
$resn_exch{"3HS  DPU"}=" HS3 DPU"; $resn_exch{"1HN2 DPU"}="HN21 DPU"; $resn_exch{"2HN2 DPU"}="HN22 DPU";
$resn_exch{"1H17 DPX"}="H171 DPX"; $resn_exch{"2H17 DPX"}="H172 DPX"; $resn_exch{"1H15 DPX"}="H151 DPX";
$resn_exch{"2H15 DPX"}="H152 DPX"; $resn_exch{"3H15 DPX"}="H153 DPX"; $resn_exch{"1H6  DPX"}=" H61 DPX";
$resn_exch{"2H6  DPX"}=" H62 DPX"; $resn_exch{"1HND DPX"}="HND1 DPX"; $resn_exch{"2HND DPX"}="HND2 DPX";
$resn_exch{"1H14 DPX"}="H141 DPX"; $resn_exch{"2H14 DPX"}="H142 DPX"; $resn_exch{"3H14 DPX"}="H143 DPX";
$resn_exch{"2HOP DPY"}="HOP2 DPY"; $resn_exch{"1H2* DPY"}="H2'1 DPY"; $resn_exch{"2H2* DPY"}="H2'2 DPY";
$resn_exch{"1H5* DPY"}="H5'1 DPY"; $resn_exch{"2H5* DPY"}="H5'2 DPY"; $resn_exch{" H4* DPY"}=" H4' DPY";
$resn_exch{" H1* DPY"}=" H1' DPY"; $resn_exch{" H3* DPY"}=" H3' DPY"; $resn_exch{"3HOP DPY"}="HOP3 DPY";
$resn_exch{"1H13 DPZ"}="H131 DPZ"; $resn_exch{"2H13 DPZ"}="H132 DPZ"; $resn_exch{"1H14 DPZ"}="H141 DPZ";
$resn_exch{"2H14 DPZ"}="H142 DPZ"; $resn_exch{"1H2  DQA"}=" H21 DQA"; $resn_exch{"2H2  DQA"}=" H22 DQA";
$resn_exch{"1H6  DQA"}=" H61 DQA"; $resn_exch{"2H6  DQA"}=" H62 DQA"; $resn_exch{"1HA  DQB"}=" HA1 DQB";
$resn_exch{"2HA  DQB"}=" HA2 DQB"; $resn_exch{"1HB  DQB"}=" HB1 DQB"; $resn_exch{"2HB  DQB"}=" HB2 DQB";
$resn_exch{"1HG  DQB"}=" HG1 DQB"; $resn_exch{"2HG  DQB"}=" HG2 DQB"; $resn_exch{"1HA2 DQB"}="HA21 DQB";
$resn_exch{"2HA2 DQB"}="HA22 DQB"; $resn_exch{"1H9  DQB"}=" H91 DQB"; $resn_exch{"2H9  DQB"}=" H92 DQB";
$resn_exch{"1H2M DQN"}="H2M1 DQN"; $resn_exch{"2H2M DQN"}="H2M2 DQN"; $resn_exch{"3H2M DQN"}="H2M3 DQN";
$resn_exch{"1H3M DQN"}="H3M1 DQN"; $resn_exch{"2H3M DQN"}="H3M2 DQN"; $resn_exch{"3H3M DQN"}="H3M3 DQN";
$resn_exch{"1H5M DQN"}="H5M1 DQN"; $resn_exch{"2H5M DQN"}="H5M2 DQN"; $resn_exch{"3H5M DQN"}="H5M3 DQN";
$resn_exch{"1H6M DQN"}="H6M1 DQN"; $resn_exch{"2H6M DQN"}="H6M2 DQN"; $resn_exch{"3H6M DQN"}="H6M3 DQN";
$resn_exch{"1H14 DQO"}="H141 DQO"; $resn_exch{"2H14 DQO"}="H142 DQO"; $resn_exch{"1H40 DQO"}="H401 DQO";
$resn_exch{"2H40 DQO"}="H402 DQO"; $resn_exch{"1H41 DQO"}="H411 DQO"; $resn_exch{"2H41 DQO"}="H412 DQO";
$resn_exch{"1H42 DQO"}="H421 DQO"; $resn_exch{"1H43 DQO"}="H431 DQO"; $resn_exch{"2H43 DQO"}="H432 DQO";
$resn_exch{"1H44 DQO"}="H441 DQO"; $resn_exch{"2H44 DQO"}="H442 DQO"; $resn_exch{"1H5  DQQ"}=" H51 DQQ";
$resn_exch{"2H5  DQQ"}=" H52 DQQ"; $resn_exch{"1H6  DQQ"}=" H61 DQQ"; $resn_exch{"2H6  DQQ"}=" H62 DQQ";
$resn_exch{"1H10 DQR"}="H101 DQR"; $resn_exch{"2H10 DQR"}="H102 DQR"; $resn_exch{"1H11 DQR"}="H111 DQR";
$resn_exch{"2H11 DQR"}="H112 DQR"; $resn_exch{"1H6  DQR"}="H61A DQR"; $resn_exch{"2H6  DQR"}=" H62 DQR";
$resn_exch{"1H60 DQR"}="H601 DQR"; $resn_exch{"2H60 DQR"}="H602 DQR"; $resn_exch{"1H61 DQR"}="H611 DQR";
$resn_exch{"2H61 DQR"}="H612 DQR"; $resn_exch{"1HN4 DQU"}="HN41 DQU"; $resn_exch{"2HN4 DQU"}="HN42 DQU";
$resn_exch{"1HN2 DQU"}="HN21 DQU"; $resn_exch{"2HN2 DQU"}="HN22 DQU"; $resn_exch{"1HAJ DR0"}="HAJ1 DR0";
$resn_exch{"2HAJ DR0"}="HAJ2 DR0"; $resn_exch{"3HAJ DR0"}="HAJ3 DR0"; $resn_exch{"1HAI DR0"}="HAI1 DR0";
$resn_exch{"2HAI DR0"}="HAI2 DR0"; $resn_exch{"1HAH DR0"}="HAH1 DR0"; $resn_exch{"2HAH DR0"}="HAH2 DR0";
$resn_exch{"1HAG DR0"}="HAG1 DR0"; $resn_exch{"2HAG DR0"}="HAG2 DR0"; $resn_exch{"1HAF DR0"}="HAF1 DR0";
$resn_exch{"2HAF DR0"}="HAF2 DR0"; $resn_exch{"1HAE DR0"}="HAE1 DR0"; $resn_exch{"2HAE DR0"}="HAE2 DR0";
$resn_exch{"1HAC DR0"}="HAC1 DR0"; $resn_exch{"2HAC DR0"}="HAC2 DR0"; $resn_exch{"3HAC DR0"}="HAC3 DR0";
$resn_exch{"1HAK DR0"}="HAK1 DR0"; $resn_exch{"2HAK DR0"}="HAK2 DR0"; $resn_exch{"3HAK DR0"}="HAK3 DR0";
$resn_exch{"1HAB DR0"}="HAB1 DR0"; $resn_exch{"2HAB DR0"}="HAB2 DR0"; $resn_exch{"1H18 DR1"}="H181 DR1";
$resn_exch{"2H18 DR1"}="H182 DR1"; $resn_exch{"3H18 DR1"}="H183 DR1"; $resn_exch{"1HAB DR2"}="HAB1 DR2";
$resn_exch{"2HAB DR2"}="HAB2 DR2"; $resn_exch{"3HAB DR2"}="HAB3 DR2"; $resn_exch{"1HAV DR2"}="HAV1 DR2";
$resn_exch{"2HAV DR2"}="HAV2 DR2"; $resn_exch{" H1* DR2"}=" H1' DR2"; $resn_exch{" H5* DR2"}=" H5' DR2";
$resn_exch{"1H6* DR2"}="H6'1 DR2"; $resn_exch{"2H6* DR2"}="H6'2 DR2"; $resn_exch{" H6* DR2"}=" H6' DR2";
$resn_exch{" H4* DR2"}=" H4' DR2"; $resn_exch{" H3* DR2"}=" H3' DR2"; $resn_exch{" H2* DR2"}=" H2' DR2";
$resn_exch{"1HAC DR2"}="HAC1 DR2"; $resn_exch{"2HAC DR2"}="HAC2 DR2"; $resn_exch{"3HAC DR2"}="HAC3 DR2";
$resn_exch{"1HAC DR3"}="HAC1 DR3"; $resn_exch{"2HAC DR3"}="HAC2 DR3"; $resn_exch{"3HAC DR3"}="HAC3 DR3";
$resn_exch{" H2* DR3"}=" H2' DR3"; $resn_exch{" H3* DR3"}=" H3' DR3"; $resn_exch{" H4* DR3"}=" H4' DR3";
$resn_exch{" H5* DR3"}=" H5' DR3"; $resn_exch{"1H6* DR3"}="H6'1 DR3"; $resn_exch{"2H6* DR3"}="H6'2 DR3";
$resn_exch{" H6* DR3"}=" H6' DR3"; $resn_exch{" H1* DR3"}=" H1' DR3"; $resn_exch{"1HAV DR3"}="HAV1 DR3";
$resn_exch{"2HAV DR3"}="HAV2 DR3"; $resn_exch{"1HAB DR3"}="HAB1 DR3"; $resn_exch{"2HAB DR3"}="HAB2 DR3";
$resn_exch{"3HAB DR3"}="HAB3 DR3"; $resn_exch{"1HAA DR4"}="HAA1 DR4"; $resn_exch{"2HAA DR4"}="HAA2 DR4";
$resn_exch{"3HAA DR4"}="HAA3 DR4"; $resn_exch{"1HAO DR4"}="HAO1 DR4"; $resn_exch{"2HAO DR4"}="HAO2 DR4";
$resn_exch{"1HAL DR4"}="HAL1 DR4"; $resn_exch{"2HAL DR4"}="HAL2 DR4"; $resn_exch{"1HAJ DR4"}="HAJ1 DR4";
$resn_exch{"2HAJ DR4"}="HAJ2 DR4"; $resn_exch{"1HAH DR4"}="HAH1 DR4"; $resn_exch{"2HAH DR4"}="HAH2 DR4";
$resn_exch{"1HAG DR4"}="HAG1 DR4"; $resn_exch{"2HAG DR4"}="HAG2 DR4"; $resn_exch{"1HAI DR4"}="HAI1 DR4";
$resn_exch{"2HAI DR4"}="HAI2 DR4"; $resn_exch{"1HAK DR4"}="HAK1 DR4"; $resn_exch{"2HAK DR4"}="HAK2 DR4";
$resn_exch{"1HAM DR4"}="HAM1 DR4"; $resn_exch{"2HAM DR4"}="HAM2 DR4"; $resn_exch{"1H6  DR4"}=" H61 DR4";
$resn_exch{"2H6  DR4"}=" H62 DR4"; $resn_exch{"1H6  DR5"}=" H61 DR5"; $resn_exch{"2H6  DR5"}=" H62 DR5";
$resn_exch{"1HAA DR5"}="HAA1 DR5"; $resn_exch{"2HAA DR5"}="HAA2 DR5"; $resn_exch{"3HAA DR5"}="HAA3 DR5";
$resn_exch{"1HAJ DR5"}="HAJ1 DR5"; $resn_exch{"2HAJ DR5"}="HAJ2 DR5"; $resn_exch{"1HCY DR6"}="HCY1 DR6";
$resn_exch{"2HCY DR6"}="HCY2 DR6"; $resn_exch{"1HAA DR7"}="HAA1 DR7"; $resn_exch{"2HAA DR7"}="HAA2 DR7";
$resn_exch{"3HAA DR7"}="HAA3 DR7"; $resn_exch{"1HAE DR7"}="HAE1 DR7"; $resn_exch{"2HAE DR7"}="HAE2 DR7";
$resn_exch{"3HAE DR7"}="HAE3 DR7"; $resn_exch{"1HG2 DR7"}="HG21 DR7"; $resn_exch{"2HG2 DR7"}="HG22 DR7";
$resn_exch{"3HG2 DR7"}="HG23 DR7"; $resn_exch{"1HG1 DR7"}="HG11 DR7"; $resn_exch{"2HG1 DR7"}="HG12 DR7";
$resn_exch{"3HG1 DR7"}="HG13 DR7"; $resn_exch{"1HBA DR7"}="HBA1 DR7"; $resn_exch{"2HBA DR7"}="HBA2 DR7";
$resn_exch{"1HBC DR7"}="HBC1 DR7"; $resn_exch{"2HBC DR7"}="HBC2 DR7"; $resn_exch{"1HAH DR7"}="HAH1 DR7";
$resn_exch{"2HAH DR7"}="HAH2 DR7"; $resn_exch{"3HAH DR7"}="HAH3 DR7"; $resn_exch{"1HAF DR7"}="HAF1 DR7";
$resn_exch{"2HAF DR7"}="HAF2 DR7"; $resn_exch{"3HAF DR7"}="HAF3 DR7"; $resn_exch{"1HAG DR7"}="HAG1 DR7";
$resn_exch{"2HAG DR7"}="HAG2 DR7"; $resn_exch{"3HAG DR7"}="HAG3 DR7"; $resn_exch{"1HAB DR7"}="HAB1 DR7";
$resn_exch{"2HAB DR7"}="HAB2 DR7"; $resn_exch{"3HAB DR7"}="HAB3 DR7"; $resn_exch{"1HBB DR7"}="HBB1 DR7";
$resn_exch{"2HBB DR7"}="HBB2 DR7"; $resn_exch{"1HAJ DR8"}="HAJ1 DR8"; $resn_exch{"2HAJ DR8"}="HAJ2 DR8";
$resn_exch{"3HAJ DR8"}="HAJ3 DR8"; $resn_exch{"1HAK DR8"}="HAK1 DR8"; $resn_exch{"2HAK DR8"}="HAK2 DR8";
$resn_exch{"3HAK DR8"}="HAK3 DR8"; $resn_exch{"1HAA DR8"}="HAA1 DR8"; $resn_exch{"2HAA DR8"}="HAA2 DR8";
$resn_exch{"3HAA DR8"}="HAA3 DR8"; $resn_exch{"1HAB DR8"}="HAB1 DR8"; $resn_exch{"2HAB DR8"}="HAB2 DR8";
$resn_exch{"3HAB DR8"}="HAB3 DR8"; $resn_exch{"1HBX DR9"}="HBX1 DR9"; $resn_exch{"2HBX DR9"}="HBX2 DR9";
$resn_exch{"3HBX DR9"}="HBX3 DR9"; $resn_exch{"1HBV DR9"}="HBV1 DR9"; $resn_exch{"2HBV DR9"}="HBV2 DR9";
$resn_exch{"1HBS DR9"}="HBS1 DR9"; $resn_exch{"2HBS DR9"}="HBS2 DR9"; $resn_exch{"1HBH DR9"}="HBH1 DR9";
$resn_exch{"2HBH DR9"}="HBH2 DR9"; $resn_exch{"1HAQ DR9"}="HAQ1 DR9"; $resn_exch{"2HAQ DR9"}="HAQ2 DR9";
$resn_exch{"1HAP DR9"}="HAP1 DR9"; $resn_exch{"2HAP DR9"}="HAP2 DR9"; $resn_exch{"1HAO DR9"}="HAO1 DR9";
$resn_exch{"2HAO DR9"}="HAO2 DR9"; $resn_exch{"1HAN DR9"}="HAN1 DR9"; $resn_exch{"2HAN DR9"}="HAN2 DR9";
$resn_exch{"1HAK DR9"}="HAK1 DR9"; $resn_exch{"2HAK DR9"}="HAK2 DR9"; $resn_exch{"1HAJ DR9"}="HAJ1 DR9";
$resn_exch{"2HAJ DR9"}="HAJ2 DR9"; $resn_exch{"1HAI DR9"}="HAI1 DR9"; $resn_exch{"2HAI DR9"}="HAI2 DR9";
$resn_exch{"1HAH DR9"}="HAH1 DR9"; $resn_exch{"2HAH DR9"}="HAH2 DR9"; $resn_exch{"1HAG DR9"}="HAG1 DR9";
$resn_exch{"2HAG DR9"}="HAG2 DR9"; $resn_exch{"1HAF DR9"}="HAF1 DR9"; $resn_exch{"2HAF DR9"}="HAF2 DR9";
$resn_exch{"1HAE DR9"}="HAE1 DR9"; $resn_exch{"2HAE DR9"}="HAE2 DR9"; $resn_exch{"1HAB DR9"}="HAB1 DR9";
$resn_exch{"2HAB DR9"}="HAB2 DR9"; $resn_exch{"1HBK DR9"}="HBK1 DR9"; $resn_exch{"2HBK DR9"}="HBK2 DR9";
$resn_exch{"1HBO DR9"}="HBO1 DR9"; $resn_exch{"2HBO DR9"}="HBO2 DR9"; $resn_exch{"1HBQ DR9"}="HBQ1 DR9";
$resn_exch{"2HBQ DR9"}="HBQ2 DR9"; $resn_exch{"1HBB DR9"}="HBB1 DR9"; $resn_exch{"2HBB DR9"}="HBB2 DR9";
$resn_exch{"1HBA DR9"}="HBA1 DR9"; $resn_exch{"2HBA DR9"}="HBA2 DR9"; $resn_exch{"1HAZ DR9"}="HAZ1 DR9";
$resn_exch{"2HAZ DR9"}="HAZ2 DR9"; $resn_exch{"1HAY DR9"}="HAY1 DR9"; $resn_exch{"2HAY DR9"}="HAY2 DR9";
$resn_exch{"1HAX DR9"}="HAX1 DR9"; $resn_exch{"2HAX DR9"}="HAX2 DR9"; $resn_exch{"1HAW DR9"}="HAW1 DR9";
$resn_exch{"2HAW DR9"}="HAW2 DR9"; $resn_exch{"1HAV DR9"}="HAV1 DR9"; $resn_exch{"2HAV DR9"}="HAV2 DR9";
$resn_exch{"1HAS DR9"}="HAS1 DR9"; $resn_exch{"2HAS DR9"}="HAS2 DR9"; $resn_exch{"1HAR DR9"}="HAR1 DR9";
$resn_exch{"2HAR DR9"}="HAR2 DR9"; $resn_exch{"1HBI DR9"}="HBI1 DR9"; $resn_exch{"2HBI DR9"}="HBI2 DR9";
$resn_exch{"1HBT DR9"}="HBT1 DR9"; $resn_exch{"2HBT DR9"}="HBT2 DR9"; $resn_exch{"1HBW DR9"}="HBW1 DR9";
$resn_exch{"2HBW DR9"}="HBW2 DR9"; $resn_exch{"1HBY DR9"}="HBY1 DR9"; $resn_exch{"2HBY DR9"}="HBY2 DR9";
$resn_exch{"3HBY DR9"}="HBY3 DR9"; $resn_exch{"1H13 DRA"}="H131 DRA"; $resn_exch{"2H13 DRA"}="H132 DRA";
$resn_exch{"1H14 DRA"}="H141 DRA"; $resn_exch{"2H14 DRA"}="H142 DRA"; $resn_exch{"3H14 DRA"}="H143 DRA";
$resn_exch{"1H8  DRA"}=" H81 DRA"; $resn_exch{"2H8  DRA"}=" H82 DRA"; $resn_exch{" H1* DRA"}=" H1' DRA";
$resn_exch{"1H2* DRA"}="H2'1 DRA"; $resn_exch{"2H2* DRA"}="H2'2 DRA"; $resn_exch{" H3* DRA"}=" H3' DRA";
$resn_exch{"1H7* DRA"}="H7'1 DRA"; $resn_exch{"2H7* DRA"}="H7'2 DRA"; $resn_exch{"3H7* DRA"}="H7'3 DRA";
$resn_exch{"1H8* DRA"}="H8'1 DRA"; $resn_exch{"2H8* DRA"}="H8'2 DRA"; $resn_exch{"3H8* DRA"}="H8'3 DRA";
$resn_exch{" H5* DRA"}=" H5' DRA"; $resn_exch{"1H6* DRA"}="H6'1 DRA"; $resn_exch{"2H6* DRA"}="H6'2 DRA";
$resn_exch{"3H6* DRA"}="H6'3 DRA"; $resn_exch{" H4* DRA"}=" H4' DRA"; $resn_exch{"1H32 DRA"}="H321 DRA";
$resn_exch{"2H32 DRA"}="H322 DRA"; $resn_exch{"6HO1 DRA"}="HO16 DRA"; $resn_exch{"1H36 DRA"}="H361 DRA";
$resn_exch{"2H36 DRA"}="H362 DRA"; $resn_exch{"3H36 DRA"}="H363 DRA"; $resn_exch{"1H41 DRA"}="H411 DRA";
$resn_exch{"2H41 DRA"}="H412 DRA"; $resn_exch{"3H41 DRA"}="H413 DRA"; $resn_exch{"1H38 DRA"}="H381 DRA";
$resn_exch{"2H38 DRA"}="H382 DRA"; $resn_exch{"1H40 DRA"}="H401 DRA"; $resn_exch{"2H40 DRA"}="H402 DRA";
$resn_exch{"3H38 DRB"}="H383 DRB"; $resn_exch{"2H38 DRB"}="H382 DRB"; $resn_exch{"1H38 DRB"}="H381 DRB";
$resn_exch{"2H39 DRB"}="H392 DRB"; $resn_exch{"1H39 DRB"}="H391 DRB"; $resn_exch{"2H40 DRB"}="H402 DRB";
$resn_exch{"1H40 DRB"}="H401 DRB"; $resn_exch{"2H41 DRB"}="H412 DRB"; $resn_exch{"1H41 DRB"}="H411 DRB";
$resn_exch{"2H42 DRB"}="H422 DRB"; $resn_exch{"1H42 DRB"}="H421 DRB"; $resn_exch{"2H43 DRB"}="H432 DRB";
$resn_exch{"1H43 DRB"}="H431 DRB"; $resn_exch{"2H44 DRB"}="H442 DRB"; $resn_exch{"1H44 DRB"}="H441 DRB";
$resn_exch{"2H45 DRB"}="H452 DRB"; $resn_exch{"1H45 DRB"}="H451 DRB"; $resn_exch{"2H46 DRB"}="H462 DRB";
$resn_exch{"1H46 DRB"}="H461 DRB"; $resn_exch{"2H51 DRB"}="H512 DRB"; $resn_exch{"1H51 DRB"}="H511 DRB";
$resn_exch{"2H55 DRB"}="H552 DRB"; $resn_exch{"1H55 DRB"}="H551 DRB"; $resn_exch{"2H57 DRB"}="H572 DRB";
$resn_exch{"1H57 DRB"}="H571 DRB"; $resn_exch{"2H58 DRB"}="H582 DRB"; $resn_exch{"1H58 DRB"}="H581 DRB";
$resn_exch{"2H59 DRB"}="H592 DRB"; $resn_exch{"1H59 DRB"}="H591 DRB"; $resn_exch{"1H20 DRC"}="H201 DRC";
$resn_exch{"2H20 DRC"}="H202 DRC"; $resn_exch{"1H21 DRC"}="H211 DRC"; $resn_exch{"2H21 DRC"}="H212 DRC";
$resn_exch{"1H22 DRC"}="H221 DRC"; $resn_exch{"2H22 DRC"}="H222 DRC"; $resn_exch{"1H23 DRC"}="H231 DRC";
$resn_exch{"2H23 DRC"}="H232 DRC"; $resn_exch{"1H18 DRC"}="H181 DRC"; $resn_exch{"2H18 DRC"}="H182 DRC";
$resn_exch{"1H17 DRC"}="H171 DRC"; $resn_exch{"2H17 DRC"}="H172 DRC"; $resn_exch{"1HN9 DRC"}="HN91 DRC";
$resn_exch{"2HN9 DRC"}="HN92 DRC"; $resn_exch{"1HAA DRD"}="HAA1 DRD"; $resn_exch{"2HAA DRD"}="HAA2 DRD";
$resn_exch{"1HAB DRD"}="HAB1 DRD"; $resn_exch{"2HAB DRD"}="HAB2 DRD"; $resn_exch{"1HAC DRD"}="HAC1 DRD";
$resn_exch{"2HAC DRD"}="HAC2 DRD"; $resn_exch{"1HBL DRD"}="HBL1 DRD"; $resn_exch{"2HBL DRD"}="HBL2 DRD";
$resn_exch{"1HBM DRD"}="HBM1 DRD"; $resn_exch{"2HBM DRD"}="HBM2 DRD"; $resn_exch{"1HBN DRD"}="HBN1 DRD";
$resn_exch{"2HBN DRD"}="HBN2 DRD"; $resn_exch{"3HBN DRD"}="HBN3 DRD"; $resn_exch{"1HAL DRD"}="HAL1 DRD";
$resn_exch{"2HAL DRD"}="HAL2 DRD"; $resn_exch{"3HAL DRD"}="HAL3 DRD"; $resn_exch{"1HAK DRD"}="HAK1 DRD";
$resn_exch{"2HAK DRD"}="HAK2 DRD"; $resn_exch{"3HAK DRD"}="HAK3 DRD"; $resn_exch{"1HAA DRE"}="HAA1 DRE";
$resn_exch{"2HAA DRE"}="HAA2 DRE"; $resn_exch{"1HAB DRE"}="HAB1 DRE"; $resn_exch{"2HAB DRE"}="HAB2 DRE";
$resn_exch{"1HAC DRE"}="HAC1 DRE"; $resn_exch{"2HAC DRE"}="HAC2 DRE"; $resn_exch{"1HAD DRE"}="HAD1 DRE";
$resn_exch{"2HAD DRE"}="HAD2 DRE"; $resn_exch{"1HAE DRE"}="HAE1 DRE"; $resn_exch{"2HAE DRE"}="HAE2 DRE";
$resn_exch{"1HAF DRE"}="HAF1 DRE"; $resn_exch{"2HAF DRE"}="HAF2 DRE"; $resn_exch{"1HAG DRE"}="HAG1 DRE";
$resn_exch{"2HAG DRE"}="HAG2 DRE"; $resn_exch{"1HAI DRE"}="HAI1 DRE"; $resn_exch{"2HAI DRE"}="HAI2 DRE";
$resn_exch{"1HAJ DRE"}="HAJ1 DRE"; $resn_exch{"2HAJ DRE"}="HAJ2 DRE"; $resn_exch{"1HAK DRE"}="HAK1 DRE";
$resn_exch{"2HAK DRE"}="HAK2 DRE"; $resn_exch{"1HAL DRE"}="HAL1 DRE"; $resn_exch{"2HAL DRE"}="HAL2 DRE";
$resn_exch{"1HAM DRE"}="HAM1 DRE"; $resn_exch{"2HAM DRE"}="HAM2 DRE"; $resn_exch{"1HAN DRE"}="HAN1 DRE";
$resn_exch{"2HAN DRE"}="HAN2 DRE"; $resn_exch{"1HAO DRE"}="HAO1 DRE"; $resn_exch{"2HAO DRE"}="HAO2 DRE";
$resn_exch{"1H48 DRF"}="H481 DRF"; $resn_exch{"2H48 DRF"}="H482 DRF"; $resn_exch{"1H49 DRF"}="H491 DRF";
$resn_exch{"2H49 DRF"}="H492 DRF"; $resn_exch{"3H49 DRF"}="H493 DRF"; $resn_exch{"1H40 DRF"}="H401 DRF";
$resn_exch{"2H40 DRF"}="H402 DRF"; $resn_exch{"1H24 DRF"}="H241 DRF"; $resn_exch{"2H24 DRF"}="H242 DRF";
$resn_exch{"1H23 DRF"}="H231 DRF"; $resn_exch{"2H23 DRF"}="H232 DRF"; $resn_exch{"1HC5 DRG"}="HC51 DRG";
$resn_exch{"2HC5 DRG"}="HC52 DRG"; $resn_exch{"1HC6 DRG"}="HC61 DRG"; $resn_exch{"2HC6 DRG"}="HC62 DRG";
$resn_exch{"CH10 DRG"}="H10C DRG"; $resn_exch{"1HN3 DRG"}="HN31 DRG"; $resn_exch{"2HN3 DRG"}="HN32 DRG";
$resn_exch{"1HAB DRH"}="HAB1 DRH"; $resn_exch{"2HAB DRH"}="HAB2 DRH"; $resn_exch{"3HAB DRH"}="HAB3 DRH";
$resn_exch{"1HAO DRH"}="HAO1 DRH"; $resn_exch{"2HAO DRH"}="HAO2 DRH"; $resn_exch{"1HAC DRH"}="HAC1 DRH";
$resn_exch{"2HAC DRH"}="HAC2 DRH"; $resn_exch{"3HAC DRH"}="HAC3 DRH"; $resn_exch{"1HAT DRH"}="HAT1 DRH";
$resn_exch{"2HAT DRH"}="HAT2 DRH"; $resn_exch{"1HAV DRH"}="HAV1 DRH"; $resn_exch{"2HAV DRH"}="HAV2 DRH";
$resn_exch{"1HAU DRH"}="HAU1 DRH"; $resn_exch{"2HAU DRH"}="HAU2 DRH"; $resn_exch{"1HAS DRH"}="HAS1 DRH";
$resn_exch{"2HAS DRH"}="HAS2 DRH"; $resn_exch{"1HAR DRH"}="HAR1 DRH"; $resn_exch{"2HAR DRH"}="HAR2 DRH";
$resn_exch{"1HAQ DRH"}="HAQ1 DRH"; $resn_exch{"2HAQ DRH"}="HAQ2 DRH"; $resn_exch{"1HAP DRH"}="HAP1 DRH";
$resn_exch{"2HAP DRH"}="HAP2 DRH"; $resn_exch{"1HAN DRH"}="HAN1 DRH"; $resn_exch{"2HAN DRH"}="HAN2 DRH";
$resn_exch{"1HAA DRH"}="HAA1 DRH"; $resn_exch{"2HAA DRH"}="HAA2 DRH"; $resn_exch{"3HAA DRH"}="HAA3 DRH";
$resn_exch{"1H2  DRI"}=" H21 DRI"; $resn_exch{"2H2  DRI"}=" H22 DRI"; $resn_exch{"1H4  DRI"}=" H41 DRI";
$resn_exch{"2H4  DRI"}=" H42 DRI"; $resn_exch{"3H4  DRI"}=" H43 DRI"; $resn_exch{"1H6  DRI"}=" H61 DRI";
$resn_exch{"2H6  DRI"}=" H62 DRI"; $resn_exch{"3H6  DRI"}=" H63 DRI"; $resn_exch{"1HAB DRJ"}="HAB1 DRJ";
$resn_exch{"2HAB DRJ"}="HAB2 DRJ"; $resn_exch{"3HAB DRJ"}="HAB3 DRJ"; $resn_exch{"1HAO DRJ"}="HAO1 DRJ";
$resn_exch{"2HAO DRJ"}="HAO2 DRJ"; $resn_exch{"1HAC DRJ"}="HAC1 DRJ"; $resn_exch{"2HAC DRJ"}="HAC2 DRJ";
$resn_exch{"3HAC DRJ"}="HAC3 DRJ"; $resn_exch{"1HAT DRJ"}="HAT1 DRJ"; $resn_exch{"2HAT DRJ"}="HAT2 DRJ";
$resn_exch{"1HAV DRJ"}="HAV1 DRJ"; $resn_exch{"2HAV DRJ"}="HAV2 DRJ"; $resn_exch{"1HAU DRJ"}="HAU1 DRJ";
$resn_exch{"2HAU DRJ"}="HAU2 DRJ"; $resn_exch{"1HAS DRJ"}="HAS1 DRJ"; $resn_exch{"2HAS DRJ"}="HAS2 DRJ";
$resn_exch{"1HAR DRJ"}="HAR1 DRJ"; $resn_exch{"2HAR DRJ"}="HAR2 DRJ"; $resn_exch{"1HAQ DRJ"}="HAQ1 DRJ";
$resn_exch{"2HAQ DRJ"}="HAQ2 DRJ"; $resn_exch{"1HAP DRJ"}="HAP1 DRJ"; $resn_exch{"2HAP DRJ"}="HAP2 DRJ";
$resn_exch{"1HAN DRJ"}="HAN1 DRJ"; $resn_exch{"2HAN DRJ"}="HAN2 DRJ"; $resn_exch{"1HAA DRJ"}="HAA1 DRJ";
$resn_exch{"2HAA DRJ"}="HAA2 DRJ"; $resn_exch{"3HAA DRJ"}="HAA3 DRJ"; $resn_exch{"2HOP DRM"}="HOP2 DRM";
$resn_exch{"3HOP DRM"}="HOP3 DRM"; $resn_exch{"1H6* DRM"}="H6'1 DRM"; $resn_exch{"2H6* DRM"}="H6'2 DRM";
$resn_exch{" H5* DRM"}=" H5' DRM"; $resn_exch{"1H4* DRM"}="H4'1 DRM"; $resn_exch{"2H4* DRM"}="H4'2 DRM";
$resn_exch{"1H3* DRM"}="H3'1 DRM"; $resn_exch{"2H3* DRM"}="H3'2 DRM"; $resn_exch{"1H2* DRM"}=" H2' DRM";
$resn_exch{"2H2* DRM"}="H2'' DRM"; $resn_exch{" H1* DRM"}=" H1' DRM"; $resn_exch{"1HAB DRN"}="HAB1 DRN";
$resn_exch{"2HAB DRN"}="HAB2 DRN"; $resn_exch{"1HBF DRN"}="HBF1 DRN"; $resn_exch{"2HBF DRN"}="HBF2 DRN";
$resn_exch{"1HBE DRN"}="HBE1 DRN"; $resn_exch{"2HBE DRN"}="HBE2 DRN"; $resn_exch{"1HA0 DRN"}="HA01 DRN";
$resn_exch{"2HA0 DRN"}="HA02 DRN"; $resn_exch{"1HAY DRN"}="HAY1 DRN"; $resn_exch{"2HAY DRN"}="HAY2 DRN";
$resn_exch{"3HAY DRN"}="HAY3 DRN"; $resn_exch{"2HOP DRP"}="HOP2 DRP"; $resn_exch{"1H2* DRP"}="H2'1 DRP";
$resn_exch{"2H2* DRP"}="H2'2 DRP"; $resn_exch{"1H5* DRP"}="H5'1 DRP"; $resn_exch{"2H5* DRP"}="H5'2 DRP";
$resn_exch{" H4* DRP"}=" H4' DRP"; $resn_exch{" H1* DRP"}=" H1' DRP"; $resn_exch{" H3* DRP"}=" H3' DRP";
$resn_exch{"3HOP DRP"}="HOP3 DRP"; $resn_exch{"1HAX DRQ"}="HAX1 DRQ"; $resn_exch{"2HAX DRQ"}="HAX2 DRQ";
$resn_exch{"3HAX DRQ"}="HAX3 DRQ"; $resn_exch{"1HAW DRQ"}="HAW1 DRQ"; $resn_exch{"2HAW DRQ"}="HAW2 DRQ";
$resn_exch{"1HAR DRQ"}="HAR1 DRQ"; $resn_exch{"2HAR DRQ"}="HAR2 DRQ"; $resn_exch{"1HAQ DRQ"}="HAQ1 DRQ";
$resn_exch{"2HAQ DRQ"}="HAQ2 DRQ"; $resn_exch{"1HAP DRQ"}="HAP1 DRQ"; $resn_exch{"2HAP DRQ"}="HAP2 DRQ";
$resn_exch{"3HAP DRQ"}="HAP3 DRQ"; $resn_exch{"1HAL DRQ"}="HAL1 DRQ"; $resn_exch{"2HAL DRQ"}="HAL2 DRQ";
$resn_exch{"1HAK DRQ"}="HAK1 DRQ"; $resn_exch{"2HAK DRQ"}="HAK2 DRQ"; $resn_exch{"1HAH DRQ"}="HAH1 DRQ";
$resn_exch{"2HAH DRQ"}="HAH2 DRQ"; $resn_exch{"1HAG DRQ"}="HAG1 DRQ"; $resn_exch{"2HAG DRQ"}="HAG2 DRQ";
$resn_exch{"3HOP DRT"}="HOP3 DRT"; $resn_exch{"2HOP DRT"}="HOP2 DRT"; $resn_exch{"1H5* DRT"}=" H5' DRT";
$resn_exch{"2H5* DRT"}="H5'' DRT"; $resn_exch{" H4* DRT"}=" H4' DRT"; $resn_exch{" H3* DRT"}=" H3' DRT";
$resn_exch{"H3T  DRT"}="HO3' DRT"; $resn_exch{"1H2* DRT"}=" H2' DRT"; $resn_exch{"2H2* DRT"}="H2'' DRT";
$resn_exch{" H1* DRT"}=" H1' DRT"; $resn_exch{"1H5M DRT"}=" H71 DRT"; $resn_exch{"2H5M DRT"}=" H72 DRT";
$resn_exch{"3H5M DRT"}=" H73 DRT"; $resn_exch{"1H3  DRU"}=" H31 DRU"; $resn_exch{"2H3  DRU"}=" H32 DRU";
$resn_exch{"1H4  DRU"}=" H41 DRU"; $resn_exch{"2H4  DRU"}=" H42 DRU"; $resn_exch{"1H5  DRU"}=" H51 DRU";
$resn_exch{"1H6  DRU"}=" H61 DRU"; $resn_exch{"7H7  DRU"}=" H77 DRU"; $resn_exch{"8H8  DRU"}=" H88 DRU";
$resn_exch{"1H9  DRU"}=" H91 DRU"; $resn_exch{"2H9  DRU"}=" H92 DRU"; $resn_exch{"1H10 DRU"}="H101 DRU";
$resn_exch{"2H10 DRU"}="H102 DRU"; $resn_exch{"1H11 DRU"}="H111 DRU"; $resn_exch{"2H11 DRU"}="H112 DRU";
$resn_exch{"1H12 DRU"}="H121 DRU"; $resn_exch{"2H12 DRU"}="H122 DRU"; $resn_exch{"1H27 DRU"}="H271 DRU";
$resn_exch{"2H27 DRU"}="H272 DRU"; $resn_exch{"1H28 DRU"}="H281 DRU"; $resn_exch{"2H28 DRU"}="H282 DRU";
$resn_exch{"1H29 DRU"}="H291 DRU"; $resn_exch{"2H29 DRU"}="H292 DRU"; $resn_exch{"1H30 DRU"}="H301 DRU";
$resn_exch{"2H30 DRU"}="H302 DRU"; $resn_exch{"31H3 DRU"}="H331 DRU"; $resn_exch{"32H3 DRU"}="H332 DRU";
$resn_exch{"1H33 DRU"}="H33A DRU"; $resn_exch{"2H33 DRU"}="H33B DRU"; $resn_exch{"1H34 DRU"}="H341 DRU";
$resn_exch{"2H34 DRU"}="H342 DRU"; $resn_exch{"1H35 DRU"}="H351 DRU"; $resn_exch{"2H35 DRU"}="H352 DRU";
$resn_exch{"1H36 DRU"}="H361 DRU"; $resn_exch{"2H36 DRU"}="H362 DRU"; $resn_exch{"1HG  DRU"}=" HG1 DRU";
$resn_exch{"2HG  DRU"}=" HG2 DRU"; $resn_exch{"1HD2 DRU"}="HD21 DRU"; $resn_exch{"2HD2 DRU"}="HD22 DRU";
$resn_exch{"1HE1 DRU"}="HE11 DRU"; $resn_exch{"2HE1 DRU"}="HE12 DRU"; $resn_exch{"1HE2 DRU"}="HE21 DRU";
$resn_exch{"1HBJ DRY"}="HBJ1 DRY"; $resn_exch{"2HBJ DRY"}="HBJ2 DRY"; $resn_exch{"1HAA DRY"}="HAA1 DRY";
$resn_exch{"2HAA DRY"}="HAA2 DRY"; $resn_exch{"1HAB DRY"}="HAB1 DRY"; $resn_exch{"2HAB DRY"}="HAB2 DRY";
$resn_exch{"1HAC DRY"}="HAC1 DRY"; $resn_exch{"2HAC DRY"}="HAC2 DRY"; $resn_exch{"1HBE DRY"}="HBE1 DRY";
$resn_exch{"2HBE DRY"}="HBE2 DRY"; $resn_exch{"1HBF DRY"}="HBF1 DRY"; $resn_exch{"2HBF DRY"}="HBF2 DRY";
$resn_exch{"1HBG DRY"}="HBG1 DRY"; $resn_exch{"2HBG DRY"}="HBG2 DRY"; $resn_exch{"3HBG DRY"}="HBG3 DRY";
$resn_exch{"2HOP DRZ"}="HOP2 DRZ"; $resn_exch{"3HOP DRZ"}="HOP3 DRZ"; $resn_exch{"1H5* DRZ"}=" H5' DRZ";
$resn_exch{"2H5* DRZ"}="H5'' DRZ"; $resn_exch{" H4* DRZ"}=" H4' DRZ"; $resn_exch{"OH4* DRZ"}="H4'O DRZ";
$resn_exch{" H3* DRZ"}=" H3' DRZ"; $resn_exch{"1H2* DRZ"}=" H2' DRZ"; $resn_exch{"2H2* DRZ"}="H2'' DRZ";
$resn_exch{" H1* DRZ"}=" H1' DRZ"; $resn_exch{"H3T  DRZ"}="HO3' DRZ"; $resn_exch{"1H11 DSA"}="H111 DSA";
$resn_exch{"2H11 DSA"}="H112 DSA"; $resn_exch{"1H13 DSA"}="H131 DSA"; $resn_exch{"2H13 DSA"}="H132 DSA";
$resn_exch{"3H13 DSA"}="H133 DSA"; $resn_exch{"1H26 DSA"}="H261 DSA"; $resn_exch{"2H26 DSA"}="H262 DSA";
$resn_exch{"3H26 DSA"}="H263 DSA"; $resn_exch{"1H27 DSA"}="H271 DSA"; $resn_exch{"2H27 DSA"}="H272 DSA";
$resn_exch{"3H27 DSA"}="H273 DSA"; $resn_exch{"1H28 DSA"}="H281 DSA"; $resn_exch{"2H28 DSA"}="H282 DSA";
$resn_exch{"3H28 DSA"}="H283 DSA"; $resn_exch{"1H15 DSA"}="H151 DSA"; $resn_exch{"2H15 DSA"}="H152 DSA";
$resn_exch{"3H15 DSA"}="H153 DSA"; $resn_exch{"1H2  DSB"}=" H21 DSB"; $resn_exch{"2H2  DSB"}=" H22 DSB";
$resn_exch{"1H3  DSB"}=" H31 DSB"; $resn_exch{"2H3  DSB"}=" H32 DSB"; $resn_exch{"1H5  DSB"}=" H51 DSB";
$resn_exch{"2H5  DSB"}=" H52 DSB"; $resn_exch{"1H1  DSB"}=" H11 DSB"; $resn_exch{"1H6  DSB"}=" H61 DSB";
$resn_exch{"2H6  DSB"}=" H62 DSB"; $resn_exch{"1H8  DSB"}=" H81 DSB"; $resn_exch{"2H8  DSB"}=" H82 DSB";
$resn_exch{"1H9  DSB"}=" H91 DSB"; $resn_exch{"2H9  DSB"}=" H92 DSB"; $resn_exch{"1H11 DSB"}="H111 DSB";
$resn_exch{"1H1  DSC"}=" H11 DSC"; $resn_exch{"2H1  DSC"}=" H12 DSC"; $resn_exch{"1H2  DSC"}=" H21 DSC";
$resn_exch{"2H2  DSC"}=" H22 DSC"; $resn_exch{"1H3  DSC"}=" H31 DSC"; $resn_exch{"2H3  DSC"}=" H32 DSC";
$resn_exch{"1H4  DSC"}=" H41 DSC"; $resn_exch{"2H4  DSC"}=" H42 DSC"; $resn_exch{"1H5  DSC"}=" H51 DSC";
$resn_exch{"2H5  DSC"}=" H52 DSC"; $resn_exch{"1H6  DSC"}=" H61 DSC"; $resn_exch{"2H6  DSC"}=" H62 DSC";
$resn_exch{"1H7  DSC"}=" H71 DSC"; $resn_exch{"2H7  DSC"}=" H72 DSC"; $resn_exch{"1H8  DSC"}=" H81 DSC";
$resn_exch{"2H8  DSC"}=" H82 DSC"; $resn_exch{"1H9  DSC"}=" H91 DSC"; $resn_exch{"2H9  DSC"}=" H92 DSC";
$resn_exch{"1H10 DSC"}="H101 DSC"; $resn_exch{"2H10 DSC"}="H102 DSC"; $resn_exch{"1H11 DSC"}="H111 DSC";
$resn_exch{"2H11 DSC"}="H112 DSC"; $resn_exch{"1H12 DSC"}="H121 DSC"; $resn_exch{"2H12 DSC"}="H122 DSC";
$resn_exch{"3H12 DSC"}="H123 DSC"; $resn_exch{"1H2  DSD"}=" H21 DSD"; $resn_exch{"2H2  DSD"}=" H22 DSD";
$resn_exch{"1H3  DSD"}=" H31 DSD"; $resn_exch{"2H3  DSD"}=" H32 DSD"; $resn_exch{"1H4  DSD"}=" H41 DSD";
$resn_exch{"2H4  DSD"}=" H42 DSD"; $resn_exch{"1H5  DSD"}=" H51 DSD"; $resn_exch{"2H5  DSD"}=" H52 DSD";
$resn_exch{"1H6  DSD"}=" H61 DSD"; $resn_exch{"2H6  DSD"}=" H62 DSD"; $resn_exch{"1H9  DSD"}=" H91 DSD";
$resn_exch{"2H9  DSD"}=" H92 DSD"; $resn_exch{"3H9  DSD"}=" H93 DSD"; $resn_exch{"1HN8 DSD"}="HN81 DSD";
$resn_exch{"2HN8 DSD"}="HN82 DSD"; $resn_exch{" HN  DSE"}=" H   DSE"; $resn_exch{"1HB  DSE"}=" HB2 DSE";
$resn_exch{"2HB  DSE"}=" HB3 DSE"; $resn_exch{"1HN  DSE"}=" HN1 DSE"; $resn_exch{"2HN  DSE"}=" HN2 DSE";
$resn_exch{"3HN  DSE"}=" HN3 DSE"; $resn_exch{" HOG DSE"}=" HG  DSE"; $resn_exch{" HN2 DSG"}=" H2  DSG";
$resn_exch{"1HB  DSG"}=" HB2 DSG"; $resn_exch{"2HB  DSG"}=" HB3 DSG"; $resn_exch{"1HD2 DSG"}="HD21 DSG";
$resn_exch{"2HD2 DSG"}="HD22 DSG"; $resn_exch{"1H11 DSI"}="H111 DSI"; $resn_exch{"2H11 DSI"}="H112 DSI";
$resn_exch{"1H13 DSI"}="H131 DSI"; $resn_exch{"2H13 DSI"}="H132 DSI"; $resn_exch{"3H13 DSI"}="H133 DSI";
$resn_exch{"1H15 DSI"}="H151 DSI"; $resn_exch{"2H15 DSI"}="H152 DSI"; $resn_exch{"3H15 DSI"}="H153 DSI";
$resn_exch{"1H18 DSM"}="H181 DSM"; $resn_exch{"2H18 DSM"}="H182 DSM"; $resn_exch{"3H18 DSM"}="H183 DSM";
$resn_exch{"1H17 DSM"}="H171 DSM"; $resn_exch{"2H17 DSM"}="H172 DSM"; $resn_exch{"1H16 DSM"}="H161 DSM";
$resn_exch{"2H16 DSM"}="H162 DSM"; $resn_exch{"1H15 DSM"}="H151 DSM"; $resn_exch{"2H15 DSM"}="H152 DSM";
$resn_exch{"1H5  DSM"}=" H51 DSM"; $resn_exch{"2H5  DSM"}=" H52 DSM"; $resn_exch{"1H6  DSM"}=" H61 DSM";
$resn_exch{"2H6  DSM"}=" H62 DSM"; $resn_exch{" HN2 DSN"}=" H2  DSN"; $resn_exch{"1HB  DSN"}=" HB2 DSN";
$resn_exch{"2HB  DSN"}=" HB3 DSN"; $resn_exch{" HOG DSN"}=" HG  DSN"; $resn_exch{"1H1C DSO"}="H1C1 DSO";
$resn_exch{"2H1C DSO"}="H1C2 DSO"; $resn_exch{"1H3C DSO"}="H3C1 DSO"; $resn_exch{"2H3C DSO"}="H3C2 DSO";
$resn_exch{"1H4C DSO"}="H4C1 DSO"; $resn_exch{"2H4C DSO"}="H4C2 DSO"; $resn_exch{"1H6C DSO"}="H6C1 DSO";
$resn_exch{"2H6C DSO"}="H6C2 DSO"; $resn_exch{"1H7C DSO"}="H7C1 DSO"; $resn_exch{"2H7C DSO"}="H7C2 DSO";
$resn_exch{"1H10 DSO"}="H101 DSO"; $resn_exch{"2H10 DSO"}="H102 DSO"; $resn_exch{"1H22 DSO"}="H221 DSO";
$resn_exch{"2H22 DSO"}="H222 DSO"; $resn_exch{"3H22 DSO"}="H223 DSO"; $resn_exch{"1H23 DSO"}="H231 DSO";
$resn_exch{"2H23 DSO"}="H232 DSO"; $resn_exch{"3H23 DSO"}="H233 DSO"; $resn_exch{"1H26 DSO"}="H261 DSO";
$resn_exch{"2H26 DSO"}="H262 DSO"; $resn_exch{"1H27 DSO"}="H271 DSO"; $resn_exch{"2H27 DSO"}="H272 DSO";
$resn_exch{"1H28 DSO"}="H281 DSO"; $resn_exch{"2H28 DSO"}="H282 DSO"; $resn_exch{"1H29 DSO"}="H291 DSO";
$resn_exch{"2H29 DSO"}="H292 DSO"; $resn_exch{"1H30 DSO"}="H301 DSO"; $resn_exch{"2H30 DSO"}="H302 DSO";
$resn_exch{"1H31 DSO"}="H311 DSO"; $resn_exch{"2H31 DSO"}="H312 DSO"; $resn_exch{"1H32 DSO"}="H321 DSO";
$resn_exch{"2H32 DSO"}="H322 DSO"; $resn_exch{"1H33 DSO"}="H331 DSO"; $resn_exch{"2H33 DSO"}="H332 DSO";
$resn_exch{"1HB  DSP"}=" HB1 DSP"; $resn_exch{"2HB  DSP"}=" HB2 DSP"; $resn_exch{"1H2  DSR"}=" H21 DSR";
$resn_exch{"2H2  DSR"}=" H22 DSR"; $resn_exch{"1H6  DSR"}=" H61 DSR"; $resn_exch{"2H6  DSR"}=" H62 DSR";
$resn_exch{"3H6  DSR"}=" H63 DSR"; $resn_exch{"1H1  DSS"}=" H11 DSS"; $resn_exch{"2H1  DSS"}=" H12 DSS";
$resn_exch{"3H1  DSS"}=" H13 DSS"; $resn_exch{"1H2  DSS"}=" H21 DSS"; $resn_exch{"2H2  DSS"}=" H22 DSS";
$resn_exch{"1H4  DSS"}=" H41 DSS"; $resn_exch{"2H4  DSS"}=" H42 DSS"; $resn_exch{"3H4  DSS"}=" H43 DSS";
$resn_exch{"1H10 DST"}="H101 DST"; $resn_exch{"2H10 DST"}="H102 DST"; $resn_exch{"1H11 DST"}="H111 DST";
$resn_exch{"2H11 DST"}="H112 DST"; $resn_exch{"1H13 DST"}="H131 DST"; $resn_exch{"2H13 DST"}="H132 DST";
$resn_exch{"3H13 DST"}="H133 DST"; $resn_exch{"1H14 DST"}="H141 DST"; $resn_exch{"2H14 DST"}="H142 DST";
$resn_exch{"3H14 DST"}="H143 DST"; $resn_exch{"1H2N DSU"}="H2N1 DSU"; $resn_exch{"2H2N DSU"}="H2N2 DSU";
$resn_exch{"1H3N DSU"}="H3N1 DSU"; $resn_exch{"2H3N DSU"}="H3N2 DSU"; $resn_exch{"1H4N DSU"}="H4N1 DSU";
$resn_exch{"2H4N DSU"}="H4N2 DSU"; $resn_exch{"1H5N DSU"}="H5N1 DSU"; $resn_exch{"2H5N DSU"}="H5N2 DSU";
$resn_exch{"1H6N DSU"}="H6N1 DSU"; $resn_exch{"2H6N DSU"}="H6N2 DSU"; $resn_exch{"1H7N DSU"}="H7N1 DSU";
$resn_exch{"2H7N DSU"}="H7N2 DSU"; $resn_exch{"1H8N DSU"}="H8N1 DSU"; $resn_exch{"2H8N DSU"}="H8N2 DSU";
$resn_exch{"1H9N DSU"}="H9N1 DSU"; $resn_exch{"2H9N DSU"}="H9N2 DSU"; $resn_exch{"1H10 DSU"}="H101 DSU";
$resn_exch{"2H10 DSU"}="H102 DSU"; $resn_exch{"3H10 DSU"}="H103 DSU"; $resn_exch{"1H6* DSU"}="H6'1 DSU";
$resn_exch{"2H6* DSU"}="H6'2 DSU"; $resn_exch{" H5* DSU"}=" H5' DSU"; $resn_exch{" H4* DSU"}=" H4' DSU";
$resn_exch{" H3* DSU"}=" H3' DSU"; $resn_exch{"1H1* DSU"}="H1'1 DSU"; $resn_exch{"2H1* DSU"}="H1'2 DSU";
$resn_exch{" H1* DSU"}=" H1' DSU"; $resn_exch{"1H6  DSU"}=" H61 DSU"; $resn_exch{"2H6  DSU"}=" H62 DSU";
$resn_exch{"3H6  DSU"}=" H63 DSU"; $resn_exch{"1H10 DSX"}="H101 DSX"; $resn_exch{"2H10 DSX"}="H102 DSX";
$resn_exch{"1H17 DSX"}="H171 DSX"; $resn_exch{"2H17 DSX"}="H172 DSX"; $resn_exch{"3H17 DSX"}="H173 DSX";
$resn_exch{"1H22 DSX"}="H221 DSX"; $resn_exch{"2H22 DSX"}="H222 DSX"; $resn_exch{"1H24 DSX"}="H241 DSX";
$resn_exch{"2H24 DSX"}="H242 DSX"; $resn_exch{"3H24 DSX"}="H243 DSX"; $resn_exch{"1H25 DSX"}="H251 DSX";
$resn_exch{"2H25 DSX"}="H252 DSX"; $resn_exch{"3H25 DSX"}="H253 DSX"; $resn_exch{"1H28 DSX"}="H281 DSX";
$resn_exch{"2H28 DSX"}="H282 DSX"; $resn_exch{"1HM1 DSY"}="HM11 DSY"; $resn_exch{"2HM1 DSY"}="HM12 DSY";
$resn_exch{"3HM1 DSY"}="HM13 DSY"; $resn_exch{"1HM2 DSY"}="HM21 DSY"; $resn_exch{"2HM2 DSY"}="HM22 DSY";
$resn_exch{"3HM2 DSY"}="HM23 DSY"; $resn_exch{"2HOS DSY"}="HOS2 DSY"; $resn_exch{"3HOP  DT"}="HOP3  DT";
$resn_exch{"2HOP  DT"}="HOP2  DT"; $resn_exch{"1H5*  DT"}=" H5'  DT"; $resn_exch{"2H5*  DT"}="H5''  DT";
$resn_exch{" H4*  DT"}=" H4'  DT"; $resn_exch{" H3*  DT"}=" H3'  DT"; $resn_exch{"H3T   DT"}="HO3'  DT";
$resn_exch{"1H2*  DT"}=" H2'  DT"; $resn_exch{"2H2*  DT"}="H2''  DT"; $resn_exch{" H1*  DT"}=" H1'  DT";
$resn_exch{"1H5M  DT"}=" H71  DT"; $resn_exch{"2H5M  DT"}=" H72  DT"; $resn_exch{"3H5M  DT"}=" H73  DT";
$resn_exch{"1H6N DT1"}="H6N1 DT1"; $resn_exch{"2H6N DT1"}="H6N2 DT1"; $resn_exch{"1H20 DT1"}="H201 DT1";
$resn_exch{"2H20 DT1"}="H202 DT1"; $resn_exch{"1H2C DT1"}="H2C1 DT1"; $resn_exch{"2H2C DT1"}="H2C2 DT1";
$resn_exch{"1H3C DT1"}="H3C1 DT1"; $resn_exch{"2H3C DT1"}="H3C2 DT1"; $resn_exch{"1H4C DT1"}="H4C1 DT1";
$resn_exch{"2H4C DT1"}="H4C2 DT1"; $resn_exch{"1H5C DT1"}="H5C1 DT1"; $resn_exch{"2H5C DT1"}="H5C2 DT1";
$resn_exch{"1H6C DT1"}="H6C1 DT1"; $resn_exch{"2H6C DT1"}="H6C2 DT1"; $resn_exch{"1H6N DT2"}="H6N1 DT2";
$resn_exch{"2H6N DT2"}="H6N2 DT2"; $resn_exch{"1H2C DT2"}="H2C1 DT2"; $resn_exch{"2H2C DT2"}="H2C2 DT2";
$resn_exch{"1H3C DT2"}="H3C1 DT2"; $resn_exch{"2H3C DT2"}="H3C2 DT2"; $resn_exch{"1H4C DT2"}="H4C1 DT2";
$resn_exch{"2H4C DT2"}="H4C2 DT2"; $resn_exch{"1H5C DT2"}="H5C1 DT2"; $resn_exch{"2H5C DT2"}="H5C2 DT2";
$resn_exch{"1H6C DT2"}="H6C1 DT2"; $resn_exch{"2H6C DT2"}="H6C2 DT2"; $resn_exch{"1H4  DT3"}=" H41 DT3";
$resn_exch{"2H4  DT3"}=" H42 DT3"; $resn_exch{"1H1  DT3"}=" H11 DT3"; $resn_exch{"2H1  DT3"}=" H12 DT3";
$resn_exch{"1H6N DT4"}="H6N1 DT4"; $resn_exch{"2H6N DT4"}="H6N2 DT4"; $resn_exch{"1H2C DT4"}="H2C1 DT4";
$resn_exch{"2H2C DT4"}="H2C2 DT4"; $resn_exch{"1H3C DT4"}="H3C1 DT4"; $resn_exch{"2H3C DT4"}="H3C2 DT4";
$resn_exch{"1H8N DT4"}="H8N1 DT4"; $resn_exch{"2H8N DT4"}="H8N2 DT4"; $resn_exch{"1H5C DT4"}="H5C1 DT4";
$resn_exch{"2H5C DT4"}="H5C2 DT4"; $resn_exch{"1H6C DT4"}="H6C1 DT4"; $resn_exch{"2H6C DT4"}="H6C2 DT4";
$resn_exch{"1H2C DT5"}="H2C1 DT5"; $resn_exch{"2H2C DT5"}="H2C2 DT5"; $resn_exch{"1H3C DT5"}="H3C1 DT5";
$resn_exch{"2H3C DT5"}="H3C2 DT5"; $resn_exch{"1H4C DT5"}="H4C1 DT5"; $resn_exch{"2H4C DT5"}="H4C2 DT5";
$resn_exch{"1H5C DT5"}="H5C1 DT5"; $resn_exch{"2H5C DT5"}="H5C2 DT5"; $resn_exch{"1H6C DT5"}="H6C1 DT5";
$resn_exch{"2H6C DT5"}="H6C2 DT5"; $resn_exch{"1H1  DT6"}=" H11 DT6"; $resn_exch{"2H1  DT6"}=" H12 DT6";
$resn_exch{"1H8  DT6"}=" H81 DT6"; $resn_exch{"2H8  DT6"}=" H82 DT6"; $resn_exch{"3H8  DT6"}=" H83 DT6";
$resn_exch{"1H10 DT6"}="H101 DT6"; $resn_exch{"2H10 DT6"}="H102 DT6"; $resn_exch{"3H10 DT6"}="H103 DT6";
$resn_exch{"1H6  DT6"}=" H61 DT6"; $resn_exch{"2H6  DT6"}=" H62 DT6"; $resn_exch{"1H37 DT8"}="H371 DT8";
$resn_exch{"2H37 DT8"}="H372 DT8"; $resn_exch{"3H37 DT8"}="H373 DT8"; $resn_exch{"1H36 DT8"}="H361 DT8";
$resn_exch{"2H36 DT8"}="H362 DT8"; $resn_exch{"3H36 DT8"}="H363 DT8"; $resn_exch{"1H18 DT8"}="H181 DT8";
$resn_exch{"2H18 DT8"}="H182 DT8"; $resn_exch{"1H19 DT8"}="H191 DT8"; $resn_exch{"2H19 DT8"}="H192 DT8";
$resn_exch{"1H16 DT8"}="H161 DT8"; $resn_exch{"2H16 DT8"}="H162 DT8"; $resn_exch{"1H28 DT8"}="H281 DT8";
$resn_exch{"2H28 DT8"}="H282 DT8"; $resn_exch{"1H33 DT8"}="H331 DT8"; $resn_exch{"2H33 DT8"}="H332 DT8";
$resn_exch{"3H33 DT8"}="H333 DT8"; $resn_exch{"1H30 DT8"}="H301 DT8"; $resn_exch{"2H30 DT8"}="H302 DT8";
$resn_exch{"1H31 DT8"}="H311 DT8"; $resn_exch{"2H31 DT8"}="H312 DT8"; $resn_exch{"1H62 DTA"}="H621 DTA";
$resn_exch{"2H62 DTA"}="H622 DTA"; $resn_exch{" H1* DTA"}=" H1' DTA"; $resn_exch{" H2* DTA"}=" H2' DTA";
$resn_exch{"HO2* DTA"}="HO2' DTA"; $resn_exch{" H3* DTA"}=" H3' DTA"; $resn_exch{"HO3* DTA"}="HO3' DTA";
$resn_exch{" H4* DTA"}=" H4' DTA"; $resn_exch{"1H5* DTA"}="H5'1 DTA"; $resn_exch{"2H5* DTA"}="H5'2 DTA";
$resn_exch{"1H5R DTA"}="H5R1 DTA"; $resn_exch{"2H5R DTA"}="H5R2 DTA"; $resn_exch{"1H61 DTA"}="H611 DTA";
$resn_exch{"2H61 DTA"}="H612 DTA"; $resn_exch{"1HCT DTB"}="HCT1 DTB"; $resn_exch{"2HCT DTB"}="HCT2 DTB";
$resn_exch{"3HCT DTB"}="HCT3 DTB"; $resn_exch{"1HCE DTB"}="HCE1 DTB"; $resn_exch{"2HCE DTB"}="HCE2 DTB";
$resn_exch{"1HCD DTB"}="HCD1 DTB"; $resn_exch{"2HCD DTB"}="HCD2 DTB"; $resn_exch{"1HCG DTB"}="HCG1 DTB";
$resn_exch{"2HCG DTB"}="HCG2 DTB"; $resn_exch{"1HCB DTB"}="HCB1 DTB"; $resn_exch{"2HCB DTB"}="HCB2 DTB";
$resn_exch{"1HCA DTB"}="HCA1 DTB"; $resn_exch{"2HCA DTB"}="HCA2 DTB"; $resn_exch{"1H15 DTC"}="H151 DTC";
$resn_exch{"2H15 DTC"}="H152 DTC"; $resn_exch{"1HN  DTE"}=" HN1 DTE"; $resn_exch{"2HN  DTE"}=" HN2 DTE";
$resn_exch{"1HBC DTE"}="HBC1 DTE"; $resn_exch{"2HBC DTE"}="HBC2 DTE"; $resn_exch{"1HAU DTF"}="HAU1 DTF";
$resn_exch{"1HAL DTF"}="HAL1 DTF"; $resn_exch{"1HAK DTF"}="HAK1 DTF"; $resn_exch{"1HAS DTF"}="HAS1 DTF";
$resn_exch{"1HAT DTF"}="HAT1 DTF"; $resn_exch{"1HAM DTF"}="HAM1 DTF"; $resn_exch{"1HAP DTF"}="HAP1 DTF";
$resn_exch{"1HBE DTF"}="HBE1 DTF"; $resn_exch{"2HBE DTF"}="HBE2 DTF"; $resn_exch{"1HBT DTF"}="HBT1 DTF";
$resn_exch{"1HAY DTF"}="HAY1 DTF"; $resn_exch{"2HAY DTF"}="HAY2 DTF"; $resn_exch{"1HAI DTF"}="HAI1 DTF";
$resn_exch{"1HAJ DTF"}="HAJ1 DTF"; $resn_exch{"1HBB DTF"}="HBB1 DTF"; $resn_exch{"2HBB DTF"}="HBB2 DTF";
$resn_exch{"1HA  DTF"}=" HA1 DTF"; $resn_exch{"1HB  DTF"}=" HB1 DTF"; $resn_exch{"2HB  DTF"}=" HB2 DTF";
$resn_exch{"1HAZ DTF"}="HAZ1 DTF"; $resn_exch{"2HAZ DTF"}="HAZ2 DTF"; $resn_exch{"1HAW DTF"}="HAW1 DTF";
$resn_exch{"2HAW DTF"}="HAW2 DTF"; $resn_exch{"1HAV DTF"}="HAV1 DTF"; $resn_exch{"2HAV DTF"}="HAV2 DTF";
$resn_exch{"1HAX DTF"}="HAX1 DTF"; $resn_exch{"2HAX DTF"}="HAX2 DTF"; $resn_exch{"1HBA DTF"}="HBA1 DTF";
$resn_exch{"2HBA DTF"}="HBA2 DTF"; $resn_exch{"1HBC DTF"}="HBC1 DTF"; $resn_exch{"2HBC DTF"}="HBC2 DTF";
$resn_exch{"1HBS DTF"}="HBS1 DTF"; $resn_exch{"1HAR DTF"}="HAR1 DTF"; $resn_exch{"1HAO DTF"}="HAO1 DTF";
$resn_exch{"1HAQ DTF"}="HAQ1 DTF"; $resn_exch{"1HAN DTF"}="HAN1 DTF"; $resn_exch{"1HBF DTF"}="HBF1 DTF";
$resn_exch{"2HBF DTF"}="HBF2 DTF"; $resn_exch{"1HA  DTG"}=" HA1 DTG"; $resn_exch{"2HA  DTG"}=" HA2 DTG";
$resn_exch{"1H1  DTG"}=" H11 DTG"; $resn_exch{"2H1  DTG"}=" H12 DTG"; $resn_exch{"3H1  DTG"}=" H13 DTG";
$resn_exch{"1H2  DTG"}=" H21 DTG"; $resn_exch{"2H2  DTG"}=" H22 DTG"; $resn_exch{"3H2  DTG"}=" H23 DTG";
$resn_exch{"1H3  DTG"}=" H31 DTG"; $resn_exch{"2H3  DTG"}=" H32 DTG"; $resn_exch{"3H3  DTG"}=" H33 DTG";
$resn_exch{"1HG2 DTH"}="HG21 DTH"; $resn_exch{"2HG2 DTH"}="HG22 DTH"; $resn_exch{"3HG2 DTH"}="HG23 DTH";
$resn_exch{" HOG DTH"}=" HG1 DTH"; $resn_exch{"1H1  DTI"}="H11A DTI"; $resn_exch{"2H1  DTI"}=" H12 DTI";
$resn_exch{"3H1  DTI"}=" H13 DTI"; $resn_exch{"1H5  DTI"}=" H51 DTI"; $resn_exch{"2H5  DTI"}=" H52 DTI";
$resn_exch{"3H5  DTI"}=" H53 DTI"; $resn_exch{"1H1C DTL"}="H1C1 DTL"; $resn_exch{"2H1C DTL"}="H1C2 DTL";
$resn_exch{"1H4C DTL"}="H4C1 DTL"; $resn_exch{"2H4C DTL"}="H4C2 DTL"; $resn_exch{" H5* DTM"}=" H5' DTM";
$resn_exch{" H7* DTM"}=" H7' DTM"; $resn_exch{"1HC9 DTM"}="HC91 DTM"; $resn_exch{"2HC9 DTM"}="HC92 DTM";
$resn_exch{"1H31 DTM"}="H311 DTM"; $resn_exch{"2H31 DTM"}="H312 DTM"; $resn_exch{"3H31 DTM"}="H313 DTM";
$resn_exch{"1H41 DTM"}="H411 DTM"; $resn_exch{"2H41 DTM"}="H412 DTM"; $resn_exch{"3H41 DTM"}="H413 DTM";
$resn_exch{"1H51 DTM"}="H511 DTM"; $resn_exch{"2H51 DTM"}="H512 DTM"; $resn_exch{"3H51 DTM"}="H513 DTM";
$resn_exch{"1HCN DTM"}="HCN1 DTM"; $resn_exch{"2HCN DTM"}="HCN2 DTM"; $resn_exch{"3HCN DTM"}="HCN3 DTM";
$resn_exch{"1HN2 DTM"}="HN21 DTM"; $resn_exch{"2HN2 DTM"}="HN22 DTM"; $resn_exch{"1HN4 DTM"}="HN41 DTM";
$resn_exch{"2HN4 DTM"}="HN42 DTM"; $resn_exch{"2H1  DTO"}=" H12 DTO"; $resn_exch{"1H1  DTO"}=" H11 DTO";
$resn_exch{"1H2O DTO"}="H2O1 DTO"; $resn_exch{"1H3O DTO"}="H3O1 DTO"; $resn_exch{"2H4  DTO"}=" H42 DTO";
$resn_exch{"1H4  DTO"}=" H41 DTO"; $resn_exch{"2HOG DTP"}="HOG2 DTP"; $resn_exch{"3HOG DTP"}="HOG3 DTP";
$resn_exch{"2HOB DTP"}="HOB2 DTP"; $resn_exch{"2HOA DTP"}="HOA2 DTP"; $resn_exch{"1H5* DTP"}="H5'1 DTP";
$resn_exch{"2H5* DTP"}="H5'2 DTP"; $resn_exch{" H4* DTP"}=" H4' DTP"; $resn_exch{" H3* DTP"}=" H3' DTP";
$resn_exch{"*HO3 DTP"}="HO3' DTP"; $resn_exch{"1H2* DTP"}="H2'1 DTP"; $resn_exch{"2H2* DTP"}="H2'2 DTP";
$resn_exch{" H1* DTP"}=" H1' DTP"; $resn_exch{"1HN6 DTP"}="HN61 DTP"; $resn_exch{"2HN6 DTP"}="HN62 DTP";
$resn_exch{"1H2  DTQ"}=" H21 DTQ"; $resn_exch{"1H7  DTQ"}=" H71 DTQ"; $resn_exch{"1H10 DTQ"}="H101 DTQ";
$resn_exch{"1H12 DTQ"}="H121 DTQ"; $resn_exch{"1H15 DTQ"}="H151 DTQ"; $resn_exch{"1H17 DTQ"}="H171 DTQ";
$resn_exch{"1H18 DTQ"}="H181 DTQ"; $resn_exch{"1H19 DTQ"}="H191 DTQ"; $resn_exch{"1H21 DTQ"}="H211 DTQ";
$resn_exch{"1H27 DTQ"}="H271 DTQ"; $resn_exch{"2H27 DTQ"}="H272 DTQ"; $resn_exch{"3H27 DTQ"}="H273 DTQ";
$resn_exch{"1H32 DTQ"}="H321 DTQ"; $resn_exch{"2H32 DTQ"}="H322 DTQ"; $resn_exch{"3H32 DTQ"}="H323 DTQ";
$resn_exch{"1HB  DTR"}=" HB2 DTR"; $resn_exch{"2HB  DTR"}=" HB3 DTR"; $resn_exch{"1H1  DTT"}=" H11 DTT";
$resn_exch{"2H1  DTT"}=" H12 DTT"; $resn_exch{"1H4  DTT"}=" H41 DTT"; $resn_exch{"2H4  DTT"}=" H42 DTT";
$resn_exch{"1H1C DTU"}="H1C1 DTU"; $resn_exch{"2H1C DTU"}="H1C2 DTU"; $resn_exch{"1H4C DTU"}="H4C1 DTU";
$resn_exch{"2H4C DTU"}="H4C2 DTU"; $resn_exch{"1H1C DTV"}="H1C1 DTV"; $resn_exch{"2H1C DTV"}="H1C2 DTV";
$resn_exch{"1H4C DTV"}="H4C1 DTV"; $resn_exch{"2H4C DTV"}="H4C2 DTV"; $resn_exch{"1H21 DTX"}="H211 DTX";
$resn_exch{"2H21 DTX"}="H212 DTX"; $resn_exch{"1H16 DTX"}="H161 DTX"; $resn_exch{"2H16 DTX"}="H162 DTX";
$resn_exch{"1H12 DTX"}="H121 DTX"; $resn_exch{"2H12 DTX"}="H122 DTX"; $resn_exch{"1H11 DTX"}="H111 DTX";
$resn_exch{"2H11 DTX"}="H112 DTX"; $resn_exch{"1H18 DTX"}="H181 DTX"; $resn_exch{"2H18 DTX"}="H182 DTX";
$resn_exch{"3H18 DTX"}="H183 DTX"; $resn_exch{"1H15 DTX"}="H151 DTX"; $resn_exch{"2H15 DTX"}="H152 DTX";
$resn_exch{"OH14 DTX"}="H14O DTX"; $resn_exch{"1HC7 DTX"}="HC71 DTX"; $resn_exch{"2HC7 DTX"}="HC72 DTX";
$resn_exch{"1HC1 DTX"}="HC11 DTX"; $resn_exch{"2HC1 DTX"}="HC12 DTX"; $resn_exch{"1H19 DTX"}="H191 DTX";
$resn_exch{"2H19 DTX"}="H192 DTX"; $resn_exch{"3H19 DTX"}="H193 DTX"; $resn_exch{"1HC6 DTX"}="HC61 DTX";
$resn_exch{"2HC6 DTX"}="HC62 DTX"; $resn_exch{"1HC4 DTX"}="HC41 DTX"; $resn_exch{"2HC4 DTX"}="HC42 DTX";
$resn_exch{"1HC2 DTX"}="HC21 DTX"; $resn_exch{"2HC2 DTX"}="HC22 DTX"; $resn_exch{"OH32 DTX"}="H32O DTX";
$resn_exch{" HN2 DTY"}=" H2  DTY"; $resn_exch{"1HB  DTY"}=" HB2 DTY"; $resn_exch{"2HB  DTY"}=" HB3 DTY";
$resn_exch{"3HOP  DU"}="HOP3  DU"; $resn_exch{"2HOP  DU"}="HOP2  DU"; $resn_exch{"1H5*  DU"}=" H5'  DU";
$resn_exch{"2H5*  DU"}="H5''  DU"; $resn_exch{" H4*  DU"}=" H4'  DU"; $resn_exch{" H3*  DU"}=" H3'  DU";
$resn_exch{"H3T   DU"}="HO3'  DU"; $resn_exch{"1H2*  DU"}=" H2'  DU"; $resn_exch{"2H2*  DU"}="H2''  DU";
$resn_exch{" H1*  DU"}=" H1'  DU"; $resn_exch{"1H6  DUC"}=" H61 DUC"; $resn_exch{"2H6  DUC"}=" H62 DUC";
$resn_exch{"1H5  DUC"}=" H51 DUC"; $resn_exch{"2H5  DUC"}=" H52 DUC"; $resn_exch{" H1* DUD"}=" H1' DUD";
$resn_exch{"1H2* DUD"}="H2'1 DUD"; $resn_exch{"2H2* DUD"}="H2'2 DUD"; $resn_exch{" H3* DUD"}=" H3' DUD";
$resn_exch{" H4* DUD"}=" H4' DUD"; $resn_exch{"*HO3 DUD"}="HO3' DUD"; $resn_exch{"1H5* DUD"}="H5'1 DUD";
$resn_exch{"2H5* DUD"}="H5'2 DUD"; $resn_exch{"2HOA DUD"}="HOA2 DUD"; $resn_exch{"2HOB DUD"}="HOB2 DUD";
$resn_exch{"3HOB DUD"}="HOB3 DUD"; $resn_exch{" H1* DUN"}=" H1' DUN"; $resn_exch{"1H2* DUN"}="H2'1 DUN";
$resn_exch{"2H2* DUN"}="H2'2 DUN"; $resn_exch{" H3* DUN"}=" H3' DUN"; $resn_exch{" H4* DUN"}=" H4' DUN";
$resn_exch{"1H5* DUN"}="H5'1 DUN"; $resn_exch{"2H5* DUN"}="H5'2 DUN"; $resn_exch{"1H10 DUO"}="H101 DUO";
$resn_exch{"2H10 DUO"}="H102 DUO"; $resn_exch{"3H10 DUO"}="H103 DUO"; $resn_exch{"1H12 DUO"}="H121 DUO";
$resn_exch{"2H12 DUO"}="H122 DUO"; $resn_exch{"1H25 DUO"}="H251 DUO"; $resn_exch{"2H25 DUO"}="H252 DUO";
$resn_exch{"3H25 DUO"}="H253 DUO"; $resn_exch{"1H27 DUO"}="H271 DUO"; $resn_exch{"2H27 DUO"}="H272 DUO";
$resn_exch{"3H27 DUO"}="H273 DUO"; $resn_exch{"1H29 DUO"}="H291 DUO"; $resn_exch{"2H29 DUO"}="H292 DUO";
$resn_exch{"3H29 DUO"}="H293 DUO"; $resn_exch{"1H33 DUO"}="H331 DUO"; $resn_exch{"2H33 DUO"}="H332 DUO";
$resn_exch{"3H33 DUO"}="H333 DUO"; $resn_exch{"1H37 DUO"}="H371 DUO"; $resn_exch{"2H37 DUO"}="H372 DUO";
$resn_exch{"3H37 DUO"}="H373 DUO"; $resn_exch{" H1* DUP"}=" H1' DUP"; $resn_exch{"1H2* DUP"}="H2'1 DUP";
$resn_exch{"2H2* DUP"}="H2'2 DUP"; $resn_exch{" H3* DUP"}=" H3' DUP"; $resn_exch{" H4* DUP"}=" H4' DUP";
$resn_exch{"1H5* DUP"}="H5'1 DUP"; $resn_exch{"2H5* DUP"}="H5'2 DUP"; $resn_exch{" H1* DUR"}=" H1' DUR";
$resn_exch{"1H2* DUR"}="H2'1 DUR"; $resn_exch{"2H2* DUR"}="H2'2 DUR"; $resn_exch{" H3* DUR"}=" H3' DUR";
$resn_exch{" H4* DUR"}=" H4' DUR"; $resn_exch{"*HO3 DUR"}="HO3' DUR"; $resn_exch{"1H5* DUR"}="H5'1 DUR";
$resn_exch{"2H5* DUR"}="H5'2 DUR"; $resn_exch{"*HO5 DUR"}="HO5' DUR"; $resn_exch{" H1* DUT"}=" H1' DUT";
$resn_exch{"1H2* DUT"}="H2'1 DUT"; $resn_exch{"2H2* DUT"}="H2'2 DUT"; $resn_exch{" H3* DUT"}=" H3' DUT";
$resn_exch{" H4* DUT"}=" H4' DUT"; $resn_exch{"*HO3 DUT"}="HO3' DUT"; $resn_exch{"1H5* DUT"}="H5'1 DUT";
$resn_exch{"2H5* DUT"}="H5'2 DUT"; $resn_exch{"2HOA DUT"}="HOA2 DUT"; $resn_exch{"2HOB DUT"}="HOB2 DUT";
$resn_exch{"2HOG DUT"}="HOG2 DUT"; $resn_exch{"3HOG DUT"}="HOG3 DUT"; $resn_exch{"1H2' DUX"}="H2'1 DUX";
$resn_exch{"2H2' DUX"}="H2'2 DUX"; $resn_exch{"1H5' DUX"}="H5'1 DUX"; $resn_exch{"2H5' DUX"}="H5'2 DUX";
$resn_exch{"1HN  DVA"}=" H   DVA"; $resn_exch{"2HN  DVA"}=" H2  DVA"; $resn_exch{"1HG1 DVA"}="HG11 DVA";
$resn_exch{"2HG1 DVA"}="HG12 DVA"; $resn_exch{"3HG1 DVA"}="HG13 DVA"; $resn_exch{"1HG2 DVA"}="HG21 DVA";
$resn_exch{"2HG2 DVA"}="HG22 DVA"; $resn_exch{"3HG2 DVA"}="HG23 DVA"; $resn_exch{"1HC2 DVR"}="HC21 DVR";
$resn_exch{"2HC2 DVR"}="HC22 DVR"; $resn_exch{"1H12 DVR"}="H121 DVR"; $resn_exch{"2H12 DVR"}="H122 DVR";
$resn_exch{"1H1  DVV"}="H11A DVV"; $resn_exch{"2H1  DVV"}=" H12 DVV"; $resn_exch{"3H1  DVV"}=" H13 DVV";
$resn_exch{"1H6  DVV"}="H61A DVV"; $resn_exch{"2H6  DVV"}="H62A DVV"; $resn_exch{"1H8  DVV"}=" H81 DVV";
$resn_exch{"2H8  DVV"}=" H82 DVV"; $resn_exch{"3H8  DVV"}=" H83 DVV"; $resn_exch{"1H9  DVV"}=" H91 DVV";
$resn_exch{"2H9  DVV"}=" H92 DVV"; $resn_exch{"3H9  DVV"}=" H93 DVV"; $resn_exch{"1H6' DVV"}="H6'1 DVV";
$resn_exch{"2H6' DVV"}="H6'2 DVV"; $resn_exch{"1H28 DVV"}="H281 DVV"; $resn_exch{"2H28 DVV"}="H282 DVV";
$resn_exch{"1H31 DVV"}="H311 DVV"; $resn_exch{"2H31 DVV"}="H312 DVV"; $resn_exch{"1H12 DW2"}="H121 DW2";
$resn_exch{"1H13 DW2"}="H131 DW2"; $resn_exch{"1H14 DW2"}="H141 DW2"; $resn_exch{"1H16 DW2"}="H161 DW2";
$resn_exch{"1H6  DW2"}=" H61 DW2"; $resn_exch{"1H5  DW2"}=" H51 DW2"; $resn_exch{"23H2 DW2"}="H223 DW2";
$resn_exch{"28H2 DW2"}="H228 DW2"; $resn_exch{"1H27 DW2"}="H271 DW2"; $resn_exch{"26H2 DW2"}="H226 DW2";
$resn_exch{"1H25 DW2"}="H251 DW2"; $resn_exch{"1H24 DW2"}="H241 DW2"; $resn_exch{"1H1  DX5"}=" H11 DX5";
$resn_exch{"2H1  DX5"}=" H12 DX5"; $resn_exch{"1H5  DX5"}=" H51 DX5"; $resn_exch{"2H5  DX5"}=" H52 DX5";
$resn_exch{"2H27 DX9"}="H227 DX9"; $resn_exch{"1HN2 DX9"}="HN21 DX9"; $resn_exch{"2HN1 DX9"}="HN12 DX9";
$resn_exch{"1HN1 DX9"}="HN11 DX9"; $resn_exch{"1H14 DX9"}="H141 DX9"; $resn_exch{"2H14 DX9"}="H142 DX9";
$resn_exch{"1H27 DX9"}="H127 DX9"; $resn_exch{"1H30 DX9"}="H130 DX9"; $resn_exch{"1H29 DX9"}="H129 DX9";
$resn_exch{"2H29 DX9"}="H229 DX9"; $resn_exch{"1HN3 DX9"}="HN31 DX9"; $resn_exch{"2H30 DX9"}="H230 DX9";
$resn_exch{"1H33 DX9"}="H331 DX9"; $resn_exch{"2H33 DX9"}="H332 DX9"; $resn_exch{"3H33 DX9"}="H333 DX9";
$resn_exch{"1H2  DXA"}=" H21 DXA"; $resn_exch{"2H2  DXA"}=" H22 DXA"; $resn_exch{"1H1  DXA"}=" H11 DXA";
$resn_exch{"2H1  DXA"}=" H12 DXA"; $resn_exch{"3H1  DXA"}=" H13 DXA"; $resn_exch{"1H5  DXA"}=" H51 DXA";
$resn_exch{"2H5  DXA"}=" H52 DXA"; $resn_exch{"3H5  DXA"}=" H53 DXA"; $resn_exch{"1H7  DXA"}=" H71 DXA";
$resn_exch{"2H7  DXA"}=" H72 DXA"; $resn_exch{"3H7  DXA"}=" H73 DXA"; $resn_exch{"1H8  DXA"}=" H81 DXA";
$resn_exch{"2H8  DXA"}=" H82 DXA"; $resn_exch{"1H9  DXA"}=" H91 DXA"; $resn_exch{"2H9  DXA"}=" H92 DXA";
$resn_exch{"3H9  DXA"}=" H93 DXA"; $resn_exch{"1H2  DXB"}=" H21 DXB"; $resn_exch{"2H2  DXB"}=" H22 DXB";
$resn_exch{"2H2  DXC"}=" H22 DXC"; $resn_exch{"1H2  DXC"}=" H21 DXC"; $resn_exch{"2H5  DXC"}=" H52 DXC";
$resn_exch{"1H5  DXC"}=" H51 DXC"; $resn_exch{"2H6  DXC"}=" H62 DXC"; $resn_exch{"1H6  DXC"}=" H61 DXC";
$resn_exch{"2H7  DXC"}=" H72 DXC"; $resn_exch{"1H7  DXC"}=" H71 DXC"; $resn_exch{"2H8  DXC"}=" H82 DXC";
$resn_exch{"1H8  DXC"}=" H81 DXC"; $resn_exch{"2H14 DXC"}="H142 DXC"; $resn_exch{"1H14 DXC"}="H141 DXC";
$resn_exch{"2H15 DXC"}="H152 DXC"; $resn_exch{"1H15 DXC"}="H151 DXC"; $resn_exch{"2H16 DXC"}="H162 DXC";
$resn_exch{"1H16 DXC"}="H161 DXC"; $resn_exch{"3H18 DXC"}="H183 DXC"; $resn_exch{"2H18 DXC"}="H182 DXC";
$resn_exch{"1H18 DXC"}="H181 DXC"; $resn_exch{"1H1O DXC"}="H1O1 DXC"; $resn_exch{"3H20 DXC"}="H203 DXC";
$resn_exch{"2H20 DXC"}="H202 DXC"; $resn_exch{"1H20 DXC"}="H201 DXC"; $resn_exch{"2H21 DXC"}="H212 DXC";
$resn_exch{"1H21 DXC"}="H211 DXC"; $resn_exch{"2H22 DXC"}="H222 DXC"; $resn_exch{"1H22 DXC"}="H221 DXC";
$resn_exch{"3H24 DXC"}="H243 DXC"; $resn_exch{"2H24 DXC"}="H242 DXC"; $resn_exch{"1H24 DXC"}="H241 DXC";
$resn_exch{"2HOP DXD"}="HOP2 DXD"; $resn_exch{"1H5* DXD"}="H5'1 DXD"; $resn_exch{"2H5* DXD"}="H5'2 DXD";
$resn_exch{" H4* DXD"}=" H4' DXD"; $resn_exch{" H1* DXD"}=" H1' DXD"; $resn_exch{"1H6* DXD"}="H6'1 DXD";
$resn_exch{"2H6* DXD"}="H6'2 DXD"; $resn_exch{" H7* DXD"}=" H7' DXD"; $resn_exch{"1H2* DXD"}="H2'1 DXD";
$resn_exch{"2H2* DXD"}="H2'2 DXD"; $resn_exch{" H3* DXD"}=" H3' DXD"; $resn_exch{"H3T  DXD"}="HO3' DXD";
$resn_exch{"3HOP DXD"}="HOP3 DXD"; $resn_exch{"1H4  DXE"}=" H41 DXE"; $resn_exch{"2H4  DXE"}=" H42 DXE";
$resn_exch{"3H4  DXE"}=" H43 DXE"; $resn_exch{"1H3  DXE"}=" H31 DXE"; $resn_exch{"2H3  DXE"}=" H32 DXE";
$resn_exch{"1H2  DXE"}=" H21 DXE"; $resn_exch{"2H2  DXE"}=" H22 DXE"; $resn_exch{"1H1  DXE"}=" H11 DXE";
$resn_exch{"2H1  DXE"}=" H12 DXE"; $resn_exch{"3H1  DXE"}=" H13 DXE"; $resn_exch{"1H2  DXG"}=" H21 DXG";
$resn_exch{"1HO2 DXG"}="HO21 DXG"; $resn_exch{"1H3  DXG"}=" H31 DXG"; $resn_exch{"1HO3 DXG"}="HO31 DXG";
$resn_exch{"1H4  DXG"}=" H41 DXG"; $resn_exch{"2H4  DXG"}=" H42 DXG"; $resn_exch{"1HO5 DXG"}="HO51 DXG";
$resn_exch{"1H5  DXG"}=" H51 DXG"; $resn_exch{"2HOP DXN"}="HOP2 DXN"; $resn_exch{"1H5* DXN"}="H5'1 DXN";
$resn_exch{"2H5* DXN"}="H5'2 DXN"; $resn_exch{" H4* DXN"}=" H4' DXN"; $resn_exch{" H1* DXN"}=" H1' DXN";
$resn_exch{"1H6* DXN"}="H6'1 DXN"; $resn_exch{"2H6* DXN"}="H6'2 DXN"; $resn_exch{" H7* DXN"}=" H7' DXN";
$resn_exch{"1H2* DXN"}="H2'1 DXN"; $resn_exch{"2H2* DXN"}="H2'2 DXN"; $resn_exch{" H3* DXN"}=" H3' DXN";
$resn_exch{"H3T  DXN"}="HO3' DXN"; $resn_exch{"3HOP DXN"}="HOP3 DXN"; $resn_exch{"1HC1 DXP"}="HC11 DXP";
$resn_exch{"2HC1 DXP"}="HC12 DXP"; $resn_exch{"3HC1 DXP"}="HC13 DXP"; $resn_exch{"2HOP DXP"}="HOP2 DXP";
$resn_exch{"1HOP DXP"}="HOP1 DXP"; $resn_exch{"1H21 DXT"}="H211 DXT"; $resn_exch{"2H21 DXT"}="H212 DXT";
$resn_exch{"1H41 DXT"}="H411 DXT"; $resn_exch{"2H41 DXT"}="H412 DXT"; $resn_exch{"3H41 DXT"}="H413 DXT";
$resn_exch{"1H42 DXT"}="H421 DXT"; $resn_exch{"2H42 DXT"}="H422 DXT"; $resn_exch{"3H42 DXT"}="H423 DXT";
$resn_exch{"1H61 DXT"}="H611 DXT"; $resn_exch{"2H61 DXT"}="H612 DXT"; $resn_exch{"3H61 DXT"}="H613 DXT";
$resn_exch{"1H   DXX"}=" H1  DXX"; $resn_exch{"1HB  DXX"}=" HB1 DXX"; $resn_exch{"2HB  DXX"}=" HB2 DXX";
$resn_exch{"3HB  DXX"}=" HB3 DXX"; $resn_exch{"2HZ2 DYB"}="HZ22 DYB"; $resn_exch{"3HZ2 DYB"}="HZ23 DYB";
$resn_exch{"2HZ3 DYB"}="HZ32 DYB"; $resn_exch{"3HZ3 DYB"}="HZ33 DYB"; $resn_exch{"2HZ1 DYB"}="HZ12 DYB";
$resn_exch{"3HZ1 DYB"}="HZ13 DYB"; $resn_exch{"2HH  DYB"}=" HH2 DYB"; $resn_exch{"3HH  DYB"}=" HH3 DYB";
$resn_exch{"2HQ  DYB"}=" HQ2 DYB"; $resn_exch{"3HQ  DYB"}=" HQ3 DYB"; $resn_exch{"2HI  DYB"}=" HI2 DYB";
$resn_exch{"3HI  DYB"}=" HI3 DYB"; $resn_exch{"1HK  DYB"}=" HK1 DYB"; $resn_exch{"2HK  DYB"}=" HK2 DYB";
$resn_exch{"3HK  DYB"}=" HK3 DYB"; $resn_exch{"1HB1 DYG"}="HB11 DYG"; $resn_exch{"2HB1 DYG"}="HB12 DYG";
$resn_exch{"1HN  DYG"}=" HN1 DYG"; $resn_exch{"2HN  DYG"}=" HN2 DYG"; $resn_exch{"1HA3 DYG"}="HA31 DYG";
$resn_exch{"2HA3 DYG"}="HA32 DYG"; $resn_exch{"1H7  DYM"}=" H71 DYM"; $resn_exch{"2H7  DYM"}=" H72 DYM";
$resn_exch{"1H9  DYM"}=" H91 DYM"; $resn_exch{"2H9  DYM"}=" H92 DYM"; $resn_exch{"1H11 DYM"}="H111 DYM";
$resn_exch{"2H11 DYM"}="H112 DYM"; $resn_exch{"3H11 DYM"}="H113 DYM"; $resn_exch{"1HAY DYP"}="HAY1 DYP";
$resn_exch{"2HAY DYP"}="HAY2 DYP"; $resn_exch{"3HAY DYP"}="HAY3 DYP"; $resn_exch{"1HAX DYP"}="HAX1 DYP";
$resn_exch{"2HAX DYP"}="HAX2 DYP"; $resn_exch{"1HAU DYP"}="HAU1 DYP"; $resn_exch{"2HAU DYP"}="HAU2 DYP";
$resn_exch{"1HAT DYP"}="HAT1 DYP"; $resn_exch{"2HAT DYP"}="HAT2 DYP"; $resn_exch{"1HAI DYP"}="HAI1 DYP";
$resn_exch{"2HAI DYP"}="HAI2 DYP"; $resn_exch{"1HAH DYP"}="HAH1 DYP"; $resn_exch{"2HAH DYP"}="HAH2 DYP";
$resn_exch{"1HAF DYP"}="HAF1 DYP"; $resn_exch{"2HAF DYP"}="HAF2 DYP"; $resn_exch{"1H9  DYP"}=" H91 DYP";
$resn_exch{"2H9  DYP"}=" H92 DYP"; $resn_exch{"1HN2 DYP"}="HN21 DYP"; $resn_exch{"2HN2 DYP"}="HN22 DYP";
$resn_exch{"1HN4 DYP"}="HN41 DYP"; $resn_exch{"2HN4 DYP"}="HN42 DYP"; $resn_exch{"1H96 DZ8"}="H961 DZ8";
$resn_exch{"2H96 DZ8"}="H962 DZ8"; $resn_exch{"3H96 DZ8"}="H963 DZ8"; $resn_exch{"1H95 DZ8"}="H951 DZ8";
$resn_exch{"2H95 DZ8"}="H952 DZ8"; $resn_exch{"3H95 DZ8"}="H953 DZ8"; $resn_exch{"1H93 DZ8"}="H931 DZ8";
$resn_exch{"2H93 DZ8"}="H932 DZ8"; $resn_exch{"1H92 DZ8"}="H921 DZ8"; $resn_exch{"2H92 DZ8"}="H922 DZ8";
$resn_exch{"1H91 DZ8"}="H911 DZ8"; $resn_exch{"2H91 DZ8"}="H912 DZ8"; $resn_exch{"1HN6 DZ8"}="HN61 DZ8";
$resn_exch{"2HN6 DZ8"}="HN62 DZ8"; $resn_exch{"1H81 DZ8"}="H811 DZ8"; $resn_exch{"2H81 DZ8"}="H812 DZ8";
$resn_exch{"1H7' DZ8"}="H7'1 DZ8"; $resn_exch{"2H7' DZ8"}="H7'2 DZ8"; $resn_exch{"1HN2 DZF"}="HN21 DZF";
$resn_exch{"2HN2 DZF"}="HN22 DZF"; $resn_exch{"1H9  DZF"}=" H91 DZF"; $resn_exch{"2H9  DZF"}=" H92 DZF";
$resn_exch{"1HB  DZF"}=" HB1 DZF"; $resn_exch{"2HB  DZF"}=" HB2 DZF"; $resn_exch{"1HG  DZF"}=" HG1 DZF";
$resn_exch{"2HG  DZF"}=" HG2 DZF"; $resn_exch{"2HOE DZF"}="HOE2 DZF"; $resn_exch{"1H6C DZN"}="H6C1 DZN";
$resn_exch{"2H6C DZN"}="H6C2 DZN"; $resn_exch{"1H17 DZP"}="H171 DZP"; $resn_exch{"2H17 DZP"}="H172 DZP";
$resn_exch{"1H20 DZP"}="H201 DZP"; $resn_exch{"2H20 DZP"}="H202 DZP"; $resn_exch{"3H20 DZP"}="H203 DZP";
$resn_exch{"1H1  DZZ"}=" H11 DZZ"; $resn_exch{"2H1  DZZ"}=" H12 DZZ"; $resn_exch{"3H1  DZZ"}=" H13 DZZ";
$resn_exch{"1H2  DZZ"}=" H21 DZZ"; $resn_exch{"2H2  DZZ"}=" H22 DZZ"; $resn_exch{"3H2  DZZ"}=" H23 DZZ";
$resn_exch{"2HOP   E"}="HOP2   E"; $resn_exch{"3HOP   E"}="HOP3   E"; $resn_exch{"1H5*   E"}=" H5'   E";
$resn_exch{"2H5*   E"}="H5''   E"; $resn_exch{" H4*   E"}=" H4'   E"; $resn_exch{" H3*   E"}=" H3'   E";
$resn_exch{"*HO3   E"}="HO3'   E"; $resn_exch{"1H2*   E"}=" H2'   E"; $resn_exch{"2H2*   E"}="H2''   E";
$resn_exch{" H1*   E"}=" H1'   E"; $resn_exch{"1H25 E04"}="H251 E04"; $resn_exch{"2H25 E04"}="H252 E04";
$resn_exch{"3H25 E04"}="H253 E04"; $resn_exch{"1H24 E04"}="H241 E04"; $resn_exch{"2H24 E04"}="H242 E04";
$resn_exch{"3H24 E04"}="H243 E04"; $resn_exch{"1H22 E04"}="H221 E04"; $resn_exch{"2H22 E04"}="H222 E04";
$resn_exch{"1H23 E04"}="H231 E04"; $resn_exch{"2H23 E04"}="H232 E04"; $resn_exch{"1H18 E04"}="H181 E04";
$resn_exch{"2H18 E04"}="H182 E04"; $resn_exch{"3H18 E04"}="H183 E04"; $resn_exch{"1H15 E04"}="H151 E04";
$resn_exch{"2H15 E04"}="H152 E04"; $resn_exch{"1H16 E04"}="H161 E04"; $resn_exch{"2H16 E04"}="H162 E04";
$resn_exch{"1H11 E04"}="H111 E04"; $resn_exch{"2H11 E04"}="H112 E04"; $resn_exch{"1H12 E04"}="H121 E04";
$resn_exch{"2H12 E04"}="H122 E04"; $resn_exch{"1H28 E04"}="H281 E04"; $resn_exch{"2H28 E04"}="H282 E04";
$resn_exch{"1H6  E04"}=" H61 E04"; $resn_exch{"2H6  E04"}=" H62 E04"; $resn_exch{"1H7  E04"}=" H71 E04";
$resn_exch{"2H7  E04"}=" H72 E04"; $resn_exch{"1H5  E09"}=" H51 E09"; $resn_exch{"1H11 E09"}="H111 E09";
$resn_exch{"2H11 E09"}="H112 E09"; $resn_exch{"1H12 E09"}="H121 E09"; $resn_exch{"2H12 E09"}="H122 E09";
$resn_exch{"1H20 E09"}="H201 E09"; $resn_exch{"2H20 E09"}="H202 E09"; $resn_exch{"1H25 E09"}="H251 E09";
$resn_exch{"2H25 E09"}="H252 E09"; $resn_exch{"1H26 E09"}="H261 E09"; $resn_exch{"2H26 E09"}="H262 E09";
$resn_exch{"1H7  E09"}=" H71 E09"; $resn_exch{"2H7  E09"}=" H72 E09"; $resn_exch{"1H31 E09"}="H311 E09";
$resn_exch{"2H31 E09"}="H312 E09"; $resn_exch{"3H31 E09"}="H313 E09"; $resn_exch{"1H9C E10"}="H9C1 E10";
$resn_exch{"2H9C E10"}="H9C2 E10"; $resn_exch{"1H13 E10"}="H131 E10"; $resn_exch{"2H13 E10"}="H132 E10";
$resn_exch{"1H14 E10"}="H141 E10"; $resn_exch{"2H14 E10"}="H142 E10"; $resn_exch{"1H24 E10"}="H241 E10";
$resn_exch{"2H24 E10"}="H242 E10"; $resn_exch{"1H36 E10"}="H361 E10"; $resn_exch{"2H36 E10"}="H362 E10";
$resn_exch{"1H38 E10"}="H381 E10"; $resn_exch{"2H38 E10"}="H382 E10"; $resn_exch{"1H39 E10"}="H391 E10";
$resn_exch{"2H39 E10"}="H392 E10"; $resn_exch{"1H41 E10"}="H411 E10"; $resn_exch{"2H41 E10"}="H412 E10";
$resn_exch{"1H63 E10"}="H631 E10"; $resn_exch{"2H63 E10"}="H632 E10"; $resn_exch{"1H64 E10"}="H641 E10";
$resn_exch{"2H64 E10"}="H642 E10"; $resn_exch{"1H65 E10"}="H651 E10"; $resn_exch{"2H65 E10"}="H652 E10";
$resn_exch{"1H66 E10"}="H661 E10"; $resn_exch{"2H66 E10"}="H662 E10"; $resn_exch{"1H67 E10"}="H671 E10";
$resn_exch{"2H67 E10"}="H672 E10"; $resn_exch{"1H68 E10"}="H681 E10"; $resn_exch{"2H68 E10"}="H682 E10";
$resn_exch{"1H69 E10"}="H691 E10"; $resn_exch{"2H69 E10"}="H692 E10"; $resn_exch{"1H70 E10"}="H701 E10";
$resn_exch{"2H70 E10"}="H702 E10"; $resn_exch{"1H14 E12"}="H141 E12"; $resn_exch{"2H14 E12"}="H142 E12";
$resn_exch{"1H24 E12"}="H241 E12"; $resn_exch{"2H24 E12"}="H242 E12"; $resn_exch{"1H9C E12"}="H9C1 E12";
$resn_exch{"2H9C E12"}="H9C2 E12"; $resn_exch{"1H13 E12"}="H131 E12"; $resn_exch{"2H13 E12"}="H132 E12";
$resn_exch{"1H63 E12"}="H631 E12"; $resn_exch{"2H63 E12"}="H632 E12"; $resn_exch{"1H36 E12"}="H361 E12";
$resn_exch{"2H36 E12"}="H362 E12"; $resn_exch{"1H38 E12"}="H381 E12"; $resn_exch{"2H38 E12"}="H382 E12";
$resn_exch{"1H39 E12"}="H391 E12"; $resn_exch{"2H39 E12"}="H392 E12"; $resn_exch{"1H41 E12"}="H411 E12";
$resn_exch{"2H41 E12"}="H412 E12"; $resn_exch{"1H64 E12"}="H641 E12"; $resn_exch{"2H64 E12"}="H642 E12";
$resn_exch{"1H65 E12"}="H651 E12"; $resn_exch{"2H65 E12"}="H652 E12"; $resn_exch{"1H66 E12"}="H661 E12";
$resn_exch{"2H66 E12"}="H662 E12"; $resn_exch{"1H67 E12"}="H671 E12"; $resn_exch{"2H67 E12"}="H672 E12";
$resn_exch{"1H68 E12"}="H681 E12"; $resn_exch{"2H68 E12"}="H682 E12"; $resn_exch{"1H69 E12"}="H691 E12";
$resn_exch{"2H69 E12"}="H692 E12"; $resn_exch{"1H70 E12"}="H701 E12"; $resn_exch{"2H70 E12"}="H702 E12";
$resn_exch{"1H71 E12"}="H711 E12"; $resn_exch{"2H71 E12"}="H712 E12"; $resn_exch{"1H72 E12"}="H721 E12";
$resn_exch{"2H72 E12"}="H722 E12"; $resn_exch{"1HN1 E1H"}="HN11 E1H"; $resn_exch{"2HA1 E1H"}="HA12 E1H";
$resn_exch{"1HB1 E1H"}="HB11 E1H"; $resn_exch{"2HB1 E1H"}="HB12 E1H"; $resn_exch{"3HB1 E1H"}="HB13 E1H";
$resn_exch{"1H3C E1P"}="H3C1 E1P"; $resn_exch{"2H3C E1P"}="H3C2 E1P"; $resn_exch{"1H2C E1P"}="H2C1 E1P";
$resn_exch{"2H2C E1P"}="H2C2 E1P"; $resn_exch{"1H1C E1P"}="H1C1 E1P"; $resn_exch{"2H1C E1P"}="H1C2 E1P";
$resn_exch{"1H5C E1P"}="H5C1 E1P"; $resn_exch{"2H5C E1P"}="H5C2 E1P"; $resn_exch{"1H4C E1P"}="H4C1 E1P";
$resn_exch{"2H4C E1P"}="H4C2 E1P"; $resn_exch{"1H8C E1P"}="H8C1 E1P"; $resn_exch{"2H8C E1P"}="H8C2 E1P";
$resn_exch{"1H9C E1P"}="H9C1 E1P"; $resn_exch{"2H9C E1P"}="H9C2 E1P"; $resn_exch{"3H9C E1P"}="H9C3 E1P";
$resn_exch{"2HOP E1X"}="HOP2 E1X"; $resn_exch{"1H10 E1X"}="H101 E1X"; $resn_exch{"2H10 E1X"}="H102 E1X";
$resn_exch{"1H11 E1X"}="H111 E1X"; $resn_exch{"2H11 E1X"}="H112 E1X"; $resn_exch{"3H11 E1X"}="H113 E1X";
$resn_exch{"1H2* E1X"}=" H2' E1X"; $resn_exch{"2H2* E1X"}="H2'' E1X"; $resn_exch{"1H5* E1X"}=" H5' E1X";
$resn_exch{"2H5* E1X"}="H5'' E1X"; $resn_exch{" H4* E1X"}=" H4' E1X"; $resn_exch{" H1* E1X"}=" H1' E1X";
$resn_exch{"H3T  E1X"}="HO3' E1X"; $resn_exch{"3HOP E1X"}="HOP3 E1X"; $resn_exch{"1H9  E20"}=" H91 E20";
$resn_exch{"2H9  E20"}=" H92 E20"; $resn_exch{"1H10 E20"}="H101 E20"; $resn_exch{"2H10 E20"}="H102 E20";
$resn_exch{"1H12 E20"}="H121 E20"; $resn_exch{"2H12 E20"}="H122 E20"; $resn_exch{"1H13 E20"}="H131 E20";
$resn_exch{"2H13 E20"}="H132 E20"; $resn_exch{"1H15 E20"}="H151 E20"; $resn_exch{"2H15 E20"}="H152 E20";
$resn_exch{"1H16 E20"}="H161 E20"; $resn_exch{"2H16 E20"}="H162 E20"; $resn_exch{"1H17 E20"}="H171 E20";
$resn_exch{"2H17 E20"}="H172 E20"; $resn_exch{"1H26 E20"}="H261 E20"; $resn_exch{"2H26 E20"}="H262 E20";
$resn_exch{"3H26 E20"}="H263 E20"; $resn_exch{"1H28 E20"}="H281 E20"; $resn_exch{"2H28 E20"}="H282 E20";
$resn_exch{"3H28 E20"}="H283 E20"; $resn_exch{"1H6  E3G"}=" H61 E3G"; $resn_exch{"2H6  E3G"}=" H62 E3G";
$resn_exch{"1H7  E3G"}=" H71 E3G"; $resn_exch{"2H7  E3G"}=" H72 E3G"; $resn_exch{"1H11 E3G"}="H111 E3G";
$resn_exch{"2H11 E3G"}="H112 E3G"; $resn_exch{"1H12 E3G"}="H121 E3G"; $resn_exch{"2H12 E3G"}="H122 E3G";
$resn_exch{"1H15 E3G"}="H151 E3G"; $resn_exch{"2H15 E3G"}="H152 E3G"; $resn_exch{"1H16 E3G"}="H161 E3G";
$resn_exch{"2H16 E3G"}="H162 E3G"; $resn_exch{"1H18 E3G"}="H181 E3G"; $resn_exch{"2H18 E3G"}="H182 E3G";
$resn_exch{"3H18 E3G"}="H183 E3G"; $resn_exch{"1H3C E3O"}="H3C1 E3O"; $resn_exch{"2H3C E3O"}="H3C2 E3O";
$resn_exch{"1H4C E3O"}="H4C1 E3O"; $resn_exch{"2H4C E3O"}="H4C2 E3O"; $resn_exch{"1H6C E3O"}="H6C1 E3O";
$resn_exch{"2H6C E3O"}="H6C2 E3O"; $resn_exch{"3H6C E3O"}="H6C3 E3O"; $resn_exch{"1H9C E3O"}="H9C1 E3O";
$resn_exch{"2H9C E3O"}="H9C2 E3O"; $resn_exch{"1H14 E3O"}="H141 E3O"; $resn_exch{"2H14 E3O"}="H142 E3O";
$resn_exch{"1H15 E3O"}="H151 E3O"; $resn_exch{"2H15 E3O"}="H152 E3O"; $resn_exch{"1H26 E4D"}="H261 E4D";
$resn_exch{"2H26 E4D"}="H262 E4D"; $resn_exch{"1H27 E4D"}="H271 E4D"; $resn_exch{"2H27 E4D"}="H272 E4D";
$resn_exch{"1H29 E4D"}="H291 E4D"; $resn_exch{"2H29 E4D"}="H292 E4D"; $resn_exch{"1H30 E4D"}="H301 E4D";
$resn_exch{"2H30 E4D"}="H302 E4D"; $resn_exch{"1H31 E4D"}="H311 E4D"; $resn_exch{"2H31 E4D"}="H312 E4D";
$resn_exch{"1H32 E4D"}="H321 E4D"; $resn_exch{"2H32 E4D"}="H322 E4D"; $resn_exch{"1H33 E4D"}="H331 E4D";
$resn_exch{"2H33 E4D"}="H332 E4D"; $resn_exch{"1HAD E4H"}="HAD1 E4H"; $resn_exch{"2HAD E4H"}="HAD2 E4H";
$resn_exch{"3HAD E4H"}="HAD3 E4H"; $resn_exch{"1HAL E4H"}="HAL1 E4H"; $resn_exch{"2HAL E4H"}="HAL2 E4H";
$resn_exch{"1HAC E4H"}="HAC1 E4H"; $resn_exch{"2HAC E4H"}="HAC2 E4H"; $resn_exch{"3HAC E4H"}="HAC3 E4H";
$resn_exch{"1HAB E4H"}="HAB1 E4H"; $resn_exch{"2HAB E4H"}="HAB2 E4H"; $resn_exch{"3HAB E4H"}="HAB3 E4H";
$resn_exch{"1HAK E4H"}="HAK1 E4H"; $resn_exch{"2HAK E4H"}="HAK2 E4H"; $resn_exch{"1HAA E4H"}="HAA1 E4H";
$resn_exch{"2HAA E4H"}="HAA2 E4H"; $resn_exch{"3HAA E4H"}="HAA3 E4H"; $resn_exch{"1HAE E4H"}="HAE1 E4H";
$resn_exch{"2HAE E4H"}="HAE2 E4H"; $resn_exch{"3HAE E4H"}="HAE3 E4H"; $resn_exch{"1H11 E4N"}="H111 E4N";
$resn_exch{"2H11 E4N"}="H112 E4N"; $resn_exch{"1H12 E4N"}="H121 E4N"; $resn_exch{"2H12 E4N"}="H122 E4N";
$resn_exch{"3H12 E4N"}="H123 E4N"; $resn_exch{"1H21 E4N"}="H211 E4N"; $resn_exch{"2H21 E4N"}="H212 E4N";
$resn_exch{"1H22 E4N"}="H221 E4N"; $resn_exch{"2H22 E4N"}="H222 E4N"; $resn_exch{"3H22 E4N"}="H223 E4N";
$resn_exch{"1H31 E4N"}="H311 E4N"; $resn_exch{"2H31 E4N"}="H312 E4N"; $resn_exch{"1H32 E4N"}="H321 E4N";
$resn_exch{"2H32 E4N"}="H322 E4N"; $resn_exch{"3H32 E4N"}="H323 E4N"; $resn_exch{"1H41 E4N"}="H411 E4N";
$resn_exch{"2H41 E4N"}="H412 E4N"; $resn_exch{"1H42 E4N"}="H421 E4N"; $resn_exch{"2H42 E4N"}="H422 E4N";
$resn_exch{"3H42 E4N"}="H423 E4N"; $resn_exch{"1H4  E4P"}=" H41 E4P"; $resn_exch{"2H4  E4P"}=" H42 E4P";
$resn_exch{"2HOP E4P"}="HOP2 E4P"; $resn_exch{"3HOP E4P"}="HOP3 E4P"; $resn_exch{"1HAA E55"}="HAA1 E55";
$resn_exch{"2HAA E55"}="HAA2 E55"; $resn_exch{"3HAA E55"}="HAA3 E55"; $resn_exch{"1HAS E55"}="HAS1 E55";
$resn_exch{"2HAS E55"}="HAS2 E55"; $resn_exch{"1HAY E55"}="HAY1 E55"; $resn_exch{"2HAY E55"}="HAY2 E55";
$resn_exch{"1HBE E55"}="HBE1 E55"; $resn_exch{"2HBE E55"}="HBE2 E55"; $resn_exch{"1HBC E55"}="HBC1 E55";
$resn_exch{"2HBC E55"}="HBC2 E55"; $resn_exch{"1HAW E55"}="HAW1 E55"; $resn_exch{"2HAW E55"}="HAW2 E55";
$resn_exch{"1HAX E55"}="HAX1 E55"; $resn_exch{"2HAX E55"}="HAX2 E55"; $resn_exch{"1HBD E55"}="HBD1 E55";
$resn_exch{"2HBD E55"}="HBD2 E55"; $resn_exch{"1HBI E55"}="HBI1 E55"; $resn_exch{"2HBI E55"}="HBI2 E55";
$resn_exch{"1HBL E55"}="HBL1 E55"; $resn_exch{"2HBL E55"}="HBL2 E55"; $resn_exch{"1HBO E55"}="HBO1 E55";
$resn_exch{"2HBO E55"}="HBO2 E55"; $resn_exch{"1HBS E55"}="HBS1 E55"; $resn_exch{"2HBS E55"}="HBS2 E55";
$resn_exch{"1HBV E55"}="HBV1 E55"; $resn_exch{"2HBV E55"}="HBV2 E55"; $resn_exch{"1HBZ E55"}="HBZ1 E55";
$resn_exch{"2HBZ E55"}="HBZ2 E55"; $resn_exch{"1HCE E55"}="HCE1 E55"; $resn_exch{"2HCE E55"}="HCE2 E55";
$resn_exch{"1H6  E55"}=" H61 E55"; $resn_exch{"2H6  E55"}=" H62 E55"; $resn_exch{"1HCH E55"}="HCH1 E55";
$resn_exch{"2HCH E55"}="HCH2 E55"; $resn_exch{"1HCD E55"}="HCD1 E55"; $resn_exch{"2HCD E55"}="HCD2 E55";
$resn_exch{"1HBY E55"}="HBY1 E55"; $resn_exch{"2HBY E55"}="HBY2 E55"; $resn_exch{"1HBU E55"}="HBU1 E55";
$resn_exch{"2HBU E55"}="HBU2 E55"; $resn_exch{"1HBR E55"}="HBR1 E55"; $resn_exch{"2HBR E55"}="HBR2 E55";
$resn_exch{"1HBP E55"}="HBP1 E55"; $resn_exch{"2HBP E55"}="HBP2 E55"; $resn_exch{"1HBM E55"}="HBM1 E55";
$resn_exch{"2HBM E55"}="HBM2 E55"; $resn_exch{"1HBJ E55"}="HBJ1 E55"; $resn_exch{"2HBJ E55"}="HBJ2 E55";
$resn_exch{"1HBF E55"}="HBF1 E55"; $resn_exch{"2HBF E55"}="HBF2 E55"; $resn_exch{"1HAZ E55"}="HAZ1 E55";
$resn_exch{"2HAZ E55"}="HAZ2 E55"; $resn_exch{"1HAT E55"}="HAT1 E55"; $resn_exch{"2HAT E55"}="HAT2 E55";
$resn_exch{"1HAB E55"}="HAB1 E55"; $resn_exch{"2HAB E55"}="HAB2 E55"; $resn_exch{"3HAB E55"}="HAB3 E55";
$resn_exch{"1HCB E55"}="HCB1 E55"; $resn_exch{"2HCB E55"}="HCB2 E55"; $resn_exch{"1HBX E55"}="HBX1 E55";
$resn_exch{"2HBX E55"}="HBX2 E55"; $resn_exch{"1HBT E55"}="HBT1 E55"; $resn_exch{"2HBT E55"}="HBT2 E55";
$resn_exch{"1HBQ E55"}="HBQ1 E55"; $resn_exch{"2HBQ E55"}="HBQ2 E55"; $resn_exch{"1HBN E55"}="HBN1 E55";
$resn_exch{"2HBN E55"}="HBN2 E55"; $resn_exch{"1HBK E55"}="HBK1 E55"; $resn_exch{"2HBK E55"}="HBK2 E55";
$resn_exch{"1HBG E55"}="HBG1 E55"; $resn_exch{"2HBG E55"}="HBG2 E55"; $resn_exch{"1HBA E55"}="HBA1 E55";
$resn_exch{"2HBA E55"}="HBA2 E55"; $resn_exch{"1HAU E55"}="HAU1 E55"; $resn_exch{"2HAU E55"}="HAU2 E55";
$resn_exch{"1HAC E55"}="HAC1 E55"; $resn_exch{"2HAC E55"}="HAC2 E55"; $resn_exch{"3HAC E55"}="HAC3 E55";
$resn_exch{"1HCI E55"}="HCI1 E55"; $resn_exch{"2HCI E55"}="HCI2 E55"; $resn_exch{"1HAE E55"}="HAE1 E55";
$resn_exch{"2HAE E55"}="HAE2 E55"; $resn_exch{"3HAE E55"}="HAE3 E55"; $resn_exch{"1HCC E55"}="HCC1 E55";
$resn_exch{"2HCC E55"}="HCC2 E55"; $resn_exch{"1HCG E55"}="HCG1 E55"; $resn_exch{"2HCG E55"}="HCG2 E55";
$resn_exch{"1HCF E55"}="HCF1 E55"; $resn_exch{"2HCF E55"}="HCF2 E55"; $resn_exch{"1HCA E55"}="HCA1 E55";
$resn_exch{"2HCA E55"}="HCA2 E55"; $resn_exch{"1HBW E55"}="HBW1 E55"; $resn_exch{"2HBW E55"}="HBW2 E55";
$resn_exch{"1HBH E55"}="HBH1 E55"; $resn_exch{"2HBH E55"}="HBH2 E55"; $resn_exch{"1HBB E55"}="HBB1 E55";
$resn_exch{"2HBB E55"}="HBB2 E55"; $resn_exch{"1HAV E55"}="HAV1 E55"; $resn_exch{"2HAV E55"}="HAV2 E55";
$resn_exch{"1HAD E55"}="HAD1 E55"; $resn_exch{"2HAD E55"}="HAD2 E55"; $resn_exch{"3HAD E55"}="HAD3 E55";
$resn_exch{"1HAF E55"}="HAF1 E55"; $resn_exch{"2HAF E55"}="HAF2 E55"; $resn_exch{"3HAF E55"}="HAF3 E55";
$resn_exch{"1H2  E64"}=" H21 E64"; $resn_exch{"2H2  E64"}=" H22 E64"; $resn_exch{"1H7  E64"}=" H71 E64";
$resn_exch{"2H7  E64"}=" H72 E64"; $resn_exch{"1H9  E64"}=" H91 E64"; $resn_exch{"2H9  E64"}=" H92 E64";
$resn_exch{"3H9  E64"}=" H93 E64"; $resn_exch{"1H10 E64"}="H101 E64"; $resn_exch{"2H10 E64"}="H102 E64";
$resn_exch{"3H10 E64"}="H103 E64"; $resn_exch{"1H12 E64"}="H121 E64"; $resn_exch{"2H12 E64"}="H122 E64";
$resn_exch{"1H13 E64"}="H131 E64"; $resn_exch{"2H13 E64"}="H132 E64"; $resn_exch{"1H14 E64"}="H141 E64";
$resn_exch{"2H14 E64"}="H142 E64"; $resn_exch{"1H15 E64"}="H151 E64"; $resn_exch{"2H15 E64"}="H152 E64";
$resn_exch{"1HN4 E64"}="HN41 E64"; $resn_exch{"2HN4 E64"}="HN42 E64"; $resn_exch{"1HN5 E64"}="HN51 E64";
$resn_exch{"2HN5 E64"}="HN52 E64"; $resn_exch{"1H2  E6C"}=" H21 E6C"; $resn_exch{"2H2  E6C"}=" H22 E6C";
$resn_exch{"1H11 E6C"}="H111 E6C"; $resn_exch{"2H11 E6C"}="H112 E6C"; $resn_exch{"1H12 E6C"}="H121 E6C";
$resn_exch{"2H12 E6C"}="H122 E6C"; $resn_exch{"1H14 E6C"}="H141 E6C"; $resn_exch{"2H14 E6C"}="H142 E6C";
$resn_exch{"3H14 E6C"}="H143 E6C"; $resn_exch{"1H15 E6C"}="H151 E6C"; $resn_exch{"2H15 E6C"}="H152 E6C";
$resn_exch{"3H15 E6C"}="H153 E6C"; $resn_exch{"1H19 E6C"}="H191 E6C"; $resn_exch{"2H19 E6C"}="H192 E6C";
$resn_exch{"1H21 E6C"}="H211 E6C"; $resn_exch{"2H21 E6C"}="H212 E6C"; $resn_exch{"3H21 E6C"}="H213 E6C";
$resn_exch{"1H22 E6C"}="H221 E6C"; $resn_exch{"2H22 E6C"}="H222 E6C"; $resn_exch{"3H22 E6C"}="H223 E6C";
$resn_exch{"1H25 E89"}="H251 E89"; $resn_exch{"2H25 E89"}="H252 E89"; $resn_exch{"1H14 E89"}="H141 E89";
$resn_exch{"2H14 E89"}="H142 E89"; $resn_exch{"1H15 E89"}="H151 E89"; $resn_exch{"2H15 E89"}="H152 E89";
$resn_exch{"1H23 E89"}="H231 E89"; $resn_exch{"2H23 E89"}="H232 E89"; $resn_exch{"3H23 E89"}="H233 E89";
$resn_exch{"1H32 E97"}="H321 E97"; $resn_exch{"2H32 E97"}="H322 E97"; $resn_exch{"1H33 E97"}="H331 E97";
$resn_exch{"2H33 E97"}="H332 E97"; $resn_exch{"1H34 E97"}="H341 E97"; $resn_exch{"2H34 E97"}="H342 E97";
$resn_exch{"1H37 E97"}="H371 E97"; $resn_exch{"2H37 E97"}="H372 E97"; $resn_exch{"1H38 E97"}="H381 E97";
$resn_exch{"2H38 E97"}="H382 E97"; $resn_exch{"1H39 E97"}="H391 E97"; $resn_exch{"2H39 E97"}="H392 E97";
$resn_exch{"1H41 E97"}="H411 E97"; $resn_exch{"2H41 E97"}="H412 E97"; $resn_exch{"3H41 E97"}="H413 E97";
$resn_exch{"1H42 E97"}="H421 E97"; $resn_exch{"2H42 E97"}="H422 E97"; $resn_exch{"3H42 E97"}="H423 E97";
$resn_exch{"1H43 E97"}="H431 E97"; $resn_exch{"2H43 E97"}="H432 E97"; $resn_exch{"3H43 E97"}="H433 E97";
$resn_exch{"1H44 E97"}="H441 E97"; $resn_exch{"2H44 E97"}="H442 E97"; $resn_exch{"3H44 E97"}="H443 E97";
$resn_exch{"1H1  EA2"}=" H11 EA2"; $resn_exch{"2H1  EA2"}=" H12 EA2"; $resn_exch{"1H2  EA2"}=" H21 EA2";
$resn_exch{"2H2  EA2"}=" H22 EA2"; $resn_exch{"1HN3 EA2"}="HN31 EA2"; $resn_exch{"2HN3 EA2"}="HN32 EA2";
$resn_exch{"1H5A EA5"}="H5A1 EA5"; $resn_exch{"2H5A EA5"}="H5A2 EA5"; $resn_exch{"1H2A EA5"}="H2A1 EA5";
$resn_exch{"2H2A EA5"}="H2A2 EA5"; $resn_exch{"3H2A EA5"}="H2A3 EA5"; $resn_exch{"1H4A EA5"}="H4A1 EA5";
$resn_exch{"2H4A EA5"}="H4A2 EA5"; $resn_exch{"1HN4 EA5"}="HN41 EA5"; $resn_exch{"2HN4 EA5"}="HN42 EA5";
$resn_exch{"1H   EA5"}=" H1  EA5"; $resn_exch{"2H   EA5"}=" H2  EA5"; $resn_exch{"1HB  EA5"}=" HB1 EA5";
$resn_exch{"2HB  EA5"}=" HB2 EA5"; $resn_exch{"1HG  EA5"}=" HG1 EA5"; $resn_exch{"2HG  EA5"}=" HG2 EA5";
$resn_exch{"1H9  EAA"}=" H91 EAA"; $resn_exch{"2H9  EAA"}=" H92 EAA"; $resn_exch{"1H10 EAA"}="H101 EAA";
$resn_exch{"2H10 EAA"}="H102 EAA"; $resn_exch{"3H10 EAA"}="H103 EAA"; $resn_exch{"1H11 EAA"}="H111 EAA";
$resn_exch{"2H11 EAA"}="H112 EAA"; $resn_exch{"1H12 EAA"}="H121 EAA"; $resn_exch{"2H12 EAA"}="H122 EAA";
$resn_exch{"1HN  EAB"}=" HN1 EAB"; $resn_exch{"2HN  EAB"}=" HN2 EAB"; $resn_exch{"2HA  EAB"}=" HA2 EAB";
$resn_exch{"3HA  EAB"}=" HA3 EAB"; $resn_exch{"3HM  EAB"}=" HM3 EAB"; $resn_exch{"2HM  EAB"}=" HM2 EAB";
$resn_exch{"1H1  EAD"}=" H11 EAD"; $resn_exch{"2H1  EAD"}="H12A EAD"; $resn_exch{"1HN3 EAD"}="HN31 EAD";
$resn_exch{"2HN3 EAD"}="HN32 EAD"; $resn_exch{"1H11 EAD"}="H111 EAD"; $resn_exch{"2H11 EAD"}="H112 EAD";
$resn_exch{"1HN7 EAD"}="HN71 EAD"; $resn_exch{"2HN7 EAD"}="HN72 EAD"; $resn_exch{"1H28 EAD"}="H281 EAD";
$resn_exch{"2H28 EAD"}="H282 EAD"; $resn_exch{"1H29 EAD"}="H291 EAD"; $resn_exch{"2H29 EAD"}="H292 EAD";
$resn_exch{"3H29 EAD"}="H293 EAD"; $resn_exch{"1H6  EAL"}=" H61 EAL"; $resn_exch{"2H6  EAL"}=" H62 EAL";
$resn_exch{"1H7  EAL"}=" H71 EAL"; $resn_exch{"2H7  EAL"}=" H72 EAL"; $resn_exch{"1H8C EAL"}="H8C1 EAL";
$resn_exch{"2H8C EAL"}="H8C2 EAL"; $resn_exch{"1H10 EAL"}="H101 EAL"; $resn_exch{"2H10 EAL"}="H102 EAL";
$resn_exch{"3H10 EAL"}="H103 EAL"; $resn_exch{"1H14 EAL"}="H141 EAL"; $resn_exch{"2H14 EAL"}="H142 EAL";
$resn_exch{"1H15 EAL"}="H151 EAL"; $resn_exch{"2H15 EAL"}="H152 EAL"; $resn_exch{"1H1  EAP"}=" H11 EAP";
$resn_exch{"1H2  EAP"}=" H21 EAP"; $resn_exch{"1HN3 EAP"}="HN31 EAP"; $resn_exch{"2HN3 EAP"}="HN32 EAP";
$resn_exch{"1H2  EB1"}=" H21 EB1"; $resn_exch{"2H2  EB1"}="H22A EB1"; $resn_exch{"1H4  EB1"}=" H41 EB1";
$resn_exch{"2H4  EB1"}=" H42 EB1"; $resn_exch{"1H9  EB1"}=" H91 EB1"; $resn_exch{"2H9  EB1"}=" H92 EB1";
$resn_exch{"1H11 EB1"}="H111 EB1"; $resn_exch{"2H11 EB1"}="H112 EB1"; $resn_exch{"1H12 EB1"}="H121 EB1";
$resn_exch{"2H12 EB1"}="H122 EB1"; $resn_exch{"1H15 EB1"}="H151 EB1"; $resn_exch{"2H15 EB1"}="H152 EB1";
$resn_exch{"1H16 EB1"}="H161 EB1"; $resn_exch{"2H16 EB1"}="H162 EB1"; $resn_exch{"1H18 EB1"}="H181 EB1";
$resn_exch{"2H18 EB1"}="H182 EB1"; $resn_exch{"3H18 EB1"}="H183 EB1"; $resn_exch{"1H19 EB1"}="H191 EB1";
$resn_exch{"2H19 EB1"}="H192 EB1"; $resn_exch{"1H21 EB1"}="H211 EB1"; $resn_exch{"2H21 EB1"}="H212 EB1";
$resn_exch{"3H21 EB1"}="H213 EB1"; $resn_exch{"1H26 EB1"}="H261 EB1"; $resn_exch{"2H26 EB1"}="H262 EB1";
$resn_exch{"1H28 EB1"}="H281 EB1"; $resn_exch{"2H28 EB1"}="H282 EB1"; $resn_exch{"1H30 EB1"}="H301 EB1";
$resn_exch{"2H30 EB1"}="H302 EB1"; $resn_exch{"3H30 EB1"}="H303 EB1"; $resn_exch{"1H31 EB1"}="H311 EB1";
$resn_exch{"2H31 EB1"}="H312 EB1"; $resn_exch{"3H31 EB1"}="H313 EB1"; $resn_exch{"1H6  EBG"}=" H61 EBG";
$resn_exch{"2H6  EBG"}=" H62 EBG"; $resn_exch{"1H7  EBG"}=" H71 EBG"; $resn_exch{"2H7  EBG"}=" H72 EBG";
$resn_exch{"1H8  EBG"}=" H81 EBG"; $resn_exch{"2H8  EBG"}=" H82 EBG"; $resn_exch{"1H10 EBG"}="H101 EBG";
$resn_exch{"2H10 EBG"}="H102 EBG"; $resn_exch{"1H7  EBP"}=" H71 EBP"; $resn_exch{"2H7  EBP"}=" H72 EBP";
$resn_exch{"1H8  EBP"}=" H81 EBP"; $resn_exch{"2H8  EBP"}=" H82 EBP"; $resn_exch{"3H8  EBP"}=" H83 EBP";
$resn_exch{"1H9  EBP"}=" H91 EBP"; $resn_exch{"2H9  EBP"}=" H92 EBP"; $resn_exch{"1H10 EBP"}="H101 EBP";
$resn_exch{"2H10 EBP"}="H102 EBP"; $resn_exch{"3H10 EBP"}="H103 EBP"; $resn_exch{"1H11 EBP"}="H111 EBP";
$resn_exch{"2H11 EBP"}="H112 EBP"; $resn_exch{"1H12 EBP"}="H121 EBP"; $resn_exch{"2H12 EBP"}="H122 EBP";
$resn_exch{"3H12 EBP"}="H123 EBP"; $resn_exch{"1H21 EBS"}="H211 EBS"; $resn_exch{"2H21 EBS"}="H212 EBS";
$resn_exch{"1H22 EBS"}="H221 EBS"; $resn_exch{"2H22 EBS"}="H222 EBS"; $resn_exch{"3H22 EBS"}="H223 EBS";
$resn_exch{"1H24 EBS"}="H241 EBS"; $resn_exch{"2H24 EBS"}="H242 EBS"; $resn_exch{"1H25 EBS"}="H251 EBS";
$resn_exch{"2H25 EBS"}="H252 EBS"; $resn_exch{"3H25 EBS"}="H253 EBS"; $resn_exch{"2H2  EBW"}=" H22 EBW";
$resn_exch{"1H2  EBW"}=" H21 EBW"; $resn_exch{"2H5  EBW"}=" H52 EBW"; $resn_exch{"1H5  EBW"}=" H51 EBW";
$resn_exch{"2H6  EBW"}=" H62 EBW"; $resn_exch{"1H6  EBW"}=" H61 EBW"; $resn_exch{"2H13 EBW"}="H132 EBW";
$resn_exch{"1H13 EBW"}="H131 EBW"; $resn_exch{"3H20 EBW"}="H203 EBW"; $resn_exch{"2H20 EBW"}="H202 EBW";
$resn_exch{"1H20 EBW"}="H201 EBW"; $resn_exch{"3H21 EBW"}="H213 EBW"; $resn_exch{"2H21 EBW"}="H212 EBW";
$resn_exch{"1H21 EBW"}="H211 EBW"; $resn_exch{"2H22 EBW"}="H222 EBW"; $resn_exch{"1H22 EBW"}="H221 EBW";
$resn_exch{"2H24 EBW"}="H242 EBW"; $resn_exch{"1H24 EBW"}="H241 EBW"; $resn_exch{"3H25 EBW"}="H253 EBW";
$resn_exch{"2H25 EBW"}="H252 EBW"; $resn_exch{"1H25 EBW"}="H251 EBW"; $resn_exch{"3H26 EBW"}="H263 EBW";
$resn_exch{"2H26 EBW"}="H262 EBW"; $resn_exch{"1H26 EBW"}="H261 EBW"; $resn_exch{"2H28 EBW"}="H282 EBW";
$resn_exch{"1H28 EBW"}="H281 EBW"; $resn_exch{"2H29 EBW"}="H292 EBW"; $resn_exch{"1H29 EBW"}="H291 EBW";
$resn_exch{"1H9C ECA"}="H9C1 ECA"; $resn_exch{"2H9C ECA"}="H9C2 ECA"; $resn_exch{"1H8C ECA"}="H8C1 ECA";
$resn_exch{"2H8C ECA"}="H8C2 ECA"; $resn_exch{"1H7C ECA"}="H7C1 ECA"; $resn_exch{"2H7C ECA"}="H7C2 ECA";
$resn_exch{"1H21 ECG"}="H211 ECG"; $resn_exch{"2H21 ECG"}="H212 ECG"; $resn_exch{"1H26 ECG"}="H261 ECG";
$resn_exch{"2H26 ECG"}="H262 ECG"; $resn_exch{"1H18 ECG"}="H181 ECG"; $resn_exch{"2H18 ECG"}="H182 ECG";
$resn_exch{"1H1  ECG"}=" H11 ECG"; $resn_exch{"2H1  ECG"}=" H12 ECG"; $resn_exch{"3H1  ECG"}=" H13 ECG";
$resn_exch{"1H14 ECG"}="H141 ECG"; $resn_exch{"2H14 ECG"}="H142 ECG"; $resn_exch{"3H14 ECG"}="H143 ECG";
$resn_exch{"1H8C ECN"}="H8C1 ECN"; $resn_exch{"2H8C ECN"}="H8C2 ECN"; $resn_exch{"1H19 ECN"}="H191 ECN";
$resn_exch{"2H19 ECN"}="H192 ECN"; $resn_exch{"1HC7 ECO"}="HC71 ECO"; $resn_exch{"2HC7 ECO"}="HC72 ECO";
$resn_exch{"1H11 ECO"}="H111 ECO"; $resn_exch{"2H11 ECO"}="H112 ECO"; $resn_exch{"1H12 ECO"}="H121 ECO";
$resn_exch{"2H12 ECO"}="H122 ECO"; $resn_exch{"1H15 ECO"}="H151 ECO"; $resn_exch{"2H15 ECO"}="H152 ECO";
$resn_exch{"1H16 ECO"}="H161 ECO"; $resn_exch{"2H16 ECO"}="H162 ECO"; $resn_exch{"OH17 ECO"}="H17O ECO";
$resn_exch{"1H18 ECO"}="H181 ECO"; $resn_exch{"2H18 ECO"}="H182 ECO"; $resn_exch{"3H18 ECO"}="H183 ECO";
$resn_exch{"1H19 ECO"}="H191 ECO"; $resn_exch{"2H19 ECO"}="H192 ECO"; $resn_exch{"1HN  ECQ"}=" HN1 ECQ";
$resn_exch{"2HN  ECQ"}=" HN2 ECQ"; $resn_exch{"1HMK ECQ"}="HMK1 ECQ"; $resn_exch{"2HMK ECQ"}="HMK2 ECQ";
$resn_exch{"3HMK ECQ"}="HMK3 ECQ"; $resn_exch{"1HB  ECQ"}=" HB1 ECQ"; $resn_exch{"2HB  ECQ"}=" HB2 ECQ";
$resn_exch{"1HAE ECQ"}="HAE1 ECQ"; $resn_exch{"2HAE ECQ"}="HAE2 ECQ"; $resn_exch{"1HD1 ECQ"}="HD11 ECQ";
$resn_exch{"2HD1 ECQ"}="HD12 ECQ"; $resn_exch{"1H11 ECS"}="H111 ECS"; $resn_exch{"2H11 ECS"}="H112 ECS";
$resn_exch{"1H12 ECS"}="H121 ECS"; $resn_exch{"2H12 ECS"}="H122 ECS"; $resn_exch{"1H18 ECS"}="H181 ECS";
$resn_exch{"2H18 ECS"}="H182 ECS"; $resn_exch{"3H18 ECS"}="H183 ECS"; $resn_exch{"1H15 ECS"}="H151 ECS";
$resn_exch{"2H15 ECS"}="H152 ECS"; $resn_exch{"1H16 ECS"}="H161 ECS"; $resn_exch{"2H16 ECS"}="H162 ECS";
$resn_exch{"1H6C ECS"}="H6C1 ECS"; $resn_exch{"2H6C ECS"}="H6C2 ECS"; $resn_exch{"1H7C ECS"}="H7C1 ECS";
$resn_exch{"2H7C ECS"}="H7C2 ECS"; $resn_exch{"1H1  ECT"}="H11A ECT"; $resn_exch{"2H1  ECT"}="H12A ECT";
$resn_exch{"1H6  ECT"}=" H61 ECT"; $resn_exch{"2H6  ECT"}=" H62 ECT"; $resn_exch{"1H18 ECT"}="H181 ECT";
$resn_exch{"2H18 ECT"}="H182 ECT"; $resn_exch{"3H18 ECT"}="H183 ECT"; $resn_exch{"1H23 ECT"}="H231 ECT";
$resn_exch{"2H23 ECT"}="H232 ECT"; $resn_exch{"1H32 ECT"}="H321 ECT"; $resn_exch{"2H32 ECT"}="H322 ECT";
$resn_exch{"3H32 ECT"}="H323 ECT"; $resn_exch{"1H37 ECT"}="H371 ECT"; $resn_exch{"2H37 ECT"}="H372 ECT";
$resn_exch{"1H38 ECT"}="H381 ECT"; $resn_exch{"2H38 ECT"}="H382 ECT"; $resn_exch{"1H43 ECT"}="H431 ECT";
$resn_exch{"2H43 ECT"}="H432 ECT"; $resn_exch{"3H43 ECT"}="H433 ECT"; $resn_exch{"1H49 ECT"}="H491 ECT";
$resn_exch{"2H49 ECT"}="H492 ECT"; $resn_exch{"3H49 ECT"}="H493 ECT"; $resn_exch{"1H51 ECT"}="H511 ECT";
$resn_exch{"2H51 ECT"}="H512 ECT"; $resn_exch{"3H51 ECT"}="H513 ECT"; $resn_exch{"1H54 ECT"}="H541 ECT";
$resn_exch{"2H54 ECT"}="H542 ECT"; $resn_exch{"2HOP EDA"}="HOP2 EDA"; $resn_exch{"3HOP EDA"}="HOP3 EDA";
$resn_exch{"1H2* EDA"}=" H2' EDA"; $resn_exch{"2H2* EDA"}="H2'' EDA"; $resn_exch{"1H5* EDA"}=" H5' EDA";
$resn_exch{"2H5* EDA"}="H5'' EDA"; $resn_exch{" H4* EDA"}=" H4' EDA"; $resn_exch{" H1* EDA"}=" H1' EDA";
$resn_exch{" H3* EDA"}=" H3' EDA"; $resn_exch{"H3T  EDA"}="HO3' EDA"; $resn_exch{"2HOP EDC"}="HOP2 EDC";
$resn_exch{"3HOP EDC"}="HOP3 EDC"; $resn_exch{"1H5* EDC"}=" H5' EDC"; $resn_exch{"2H5* EDC"}="H5'' EDC";
$resn_exch{" H4* EDC"}=" H4' EDC"; $resn_exch{" H1* EDC"}=" H1' EDC"; $resn_exch{"1H2* EDC"}=" H2' EDC";
$resn_exch{"2H2* EDC"}="H2'' EDC"; $resn_exch{" H3* EDC"}=" H3' EDC"; $resn_exch{"*HO3 EDC"}="HO3' EDC";
$resn_exch{"1H2  EDE"}=" H21 EDE"; $resn_exch{"2H2  EDE"}=" H22 EDE"; $resn_exch{"1H10 EDE"}="H101 EDE";
$resn_exch{"2H10 EDE"}="H102 EDE"; $resn_exch{"1H11 EDE"}="H111 EDE"; $resn_exch{"2H11 EDE"}="H112 EDE";
$resn_exch{"1H12 EDE"}="H121 EDE"; $resn_exch{"2H12 EDE"}="H122 EDE"; $resn_exch{"1H15 EDE"}="H151 EDE";
$resn_exch{"2H15 EDE"}="H152 EDE"; $resn_exch{"1H18 EDE"}="H181 EDE"; $resn_exch{"2H18 EDE"}="H182 EDE";
$resn_exch{"1H21 EDE"}="H211 EDE"; $resn_exch{"2H21 EDE"}="H212 EDE"; $resn_exch{"1H22 EDE"}="H221 EDE";
$resn_exch{"2H22 EDE"}="H222 EDE"; $resn_exch{"1H23 EDE"}="H231 EDE"; $resn_exch{"2H23 EDE"}="H232 EDE";
$resn_exch{"1H25 EDE"}="H251 EDE"; $resn_exch{"2H25 EDE"}="H252 EDE"; $resn_exch{"1H26 EDE"}="H261 EDE";
$resn_exch{"2H26 EDE"}="H262 EDE"; $resn_exch{"1H27 EDE"}="H271 EDE"; $resn_exch{"2H27 EDE"}="H272 EDE";
$resn_exch{"1H28 EDE"}="H281 EDE"; $resn_exch{"2H28 EDE"}="H282 EDE"; $resn_exch{"1H31 EDE"}="H311 EDE";
$resn_exch{"2H31 EDE"}="H312 EDE"; $resn_exch{"1H33 EDE"}="H331 EDE"; $resn_exch{"2H33 EDE"}="H332 EDE";
$resn_exch{"1H45 EDE"}="H451 EDE"; $resn_exch{"2H45 EDE"}="H452 EDE"; $resn_exch{"1H46 EDE"}="H461 EDE";
$resn_exch{"2H46 EDE"}="H462 EDE"; $resn_exch{"1H50 EDE"}="H501 EDE"; $resn_exch{"2H50 EDE"}="H502 EDE";
$resn_exch{"1H55 EDE"}="H551 EDE"; $resn_exch{"2H55 EDE"}="H552 EDE"; $resn_exch{"1H1  EDO"}=" H11 EDO";
$resn_exch{"2H1  EDO"}=" H12 EDO"; $resn_exch{"1H2  EDO"}=" H21 EDO"; $resn_exch{"2H2  EDO"}=" H22 EDO";
$resn_exch{"1H7  EDR"}=" H71 EDR"; $resn_exch{"2H7  EDR"}=" H72 EDR"; $resn_exch{"3H7  EDR"}=" H73 EDR";
$resn_exch{"1H8  EDR"}=" H81 EDR"; $resn_exch{"2H8  EDR"}=" H82 EDR"; $resn_exch{"3H8  EDR"}=" H83 EDR";
$resn_exch{"1H9  EDR"}=" H91 EDR"; $resn_exch{"2H9  EDR"}=" H92 EDR"; $resn_exch{"1H10 EDR"}="H101 EDR";
$resn_exch{"2H10 EDR"}="H102 EDR"; $resn_exch{"3H10 EDR"}="H103 EDR"; $resn_exch{"1H04 EDT"}="H041 EDT";
$resn_exch{"2H04 EDT"}="H042 EDT"; $resn_exch{"1H02 EDT"}="H021 EDT"; $resn_exch{"2H02 EDT"}="H022 EDT";
$resn_exch{"1H06 EDT"}="H061 EDT"; $resn_exch{"2H06 EDT"}="H062 EDT"; $resn_exch{"1H07 EDT"}="H071 EDT";
$resn_exch{"2H07 EDT"}="H072 EDT"; $resn_exch{"1H09 EDT"}="H091 EDT"; $resn_exch{"2H09 EDT"}="H092 EDT";
$resn_exch{"1H11 EDT"}="H111 EDT"; $resn_exch{"2H11 EDT"}="H112 EDT"; $resn_exch{"UHN3 EEB"}="HN3U EEB";
$resn_exch{" UH5 EEB"}=" H5U EEB"; $resn_exch{" UH6 EEB"}=" H6U EEB"; $resn_exch{"UH1* EEB"}=" H1D EEB";
$resn_exch{"UH2* EEB"}=" H2D EEB"; $resn_exch{"UHO2 EEB"}="HO2U EEB"; $resn_exch{"UH3* EEB"}=" H3D EEB";
$resn_exch{"UH4* EEB"}=" H4D EEB"; $resn_exch{"UHO3 EEB"}="HO3U EEB"; $resn_exch{"1H5* EEB"}="H5'1 EEB";
$resn_exch{"2H5* EEB"}="H5'2 EEB"; $resn_exch{"2HOA EEB"}="HOA2 EEB"; $resn_exch{"2HOB EEB"}="HOB2 EEB";
$resn_exch{"1H6  EEB"}=" H61 EEB"; $resn_exch{"2H6  EEB"}=" H62 EEB"; $resn_exch{"1H8  EEB"}=" H81 EEB";
$resn_exch{"2H8  EEB"}=" H82 EEB"; $resn_exch{"3H8  EEB"}=" H83 EEB"; $resn_exch{" EH2 EEB"}=" H2E EEB";
$resn_exch{"1HE3 EEB"}="HE31 EEB"; $resn_exch{"2HE3 EEB"}="HE32 EEB"; $resn_exch{"1HE4 EEB"}="HE41 EEB";
$resn_exch{"2HE4 EEB"}="HE42 EEB"; $resn_exch{"3HE4 EEB"}="HE43 EEB"; $resn_exch{"EHO2 EEB"}="HO2E EEB";
$resn_exch{"1H6  EED"}=" H61 EED"; $resn_exch{"2H6  EED"}=" H62 EED"; $resn_exch{"1H7  EED"}=" H71 EED";
$resn_exch{"2H7  EED"}=" H72 EED"; $resn_exch{"1H15 EED"}="H151 EED"; $resn_exch{"2H15 EED"}="H152 EED";
$resn_exch{"1H16 EED"}="H161 EED"; $resn_exch{"2H16 EED"}="H162 EED"; $resn_exch{"1H18 EED"}="H181 EED";
$resn_exch{"2H18 EED"}="H182 EED"; $resn_exch{"3H18 EED"}="H183 EED"; $resn_exch{"1H12 EED"}="H121 EED";
$resn_exch{"2H12 EED"}="H122 EED"; $resn_exch{"1H19 EED"}="H191 EED"; $resn_exch{"2H19 EED"}="H192 EED";
$resn_exch{"1H21 EED"}="H211 EED"; $resn_exch{"2H21 EED"}="H212 EED"; $resn_exch{"3H21 EED"}="H213 EED";
$resn_exch{"1H2  EEE"}=" H21 EEE"; $resn_exch{"2H2  EEE"}=" H22 EEE"; $resn_exch{"3H2  EEE"}=" H23 EEE";
$resn_exch{"1H3  EEE"}=" H31 EEE"; $resn_exch{"2H3  EEE"}=" H32 EEE"; $resn_exch{"1H4  EEE"}=" H41 EEE";
$resn_exch{"2H4  EEE"}=" H42 EEE"; $resn_exch{"3H4  EEE"}=" H43 EEE"; $resn_exch{"1HN  EFC"}=" H   EFC";
$resn_exch{"2HN  EFC"}=" H2  EFC"; $resn_exch{"1HB  EFC"}=" HB2 EFC"; $resn_exch{"2HB  EFC"}=" HB3 EFC";
$resn_exch{"1H1  EFC"}=" H11 EFC"; $resn_exch{"2H1  EFC"}=" H12 EFC"; $resn_exch{"1H2  EFC"}=" H21 EFC";
$resn_exch{"2H2  EFC"}=" H22 EFC"; $resn_exch{"1H12 EFS"}="H121 EFS"; $resn_exch{"2H12 EFS"}="H122 EFS";
$resn_exch{"3H12 EFS"}="H123 EFS"; $resn_exch{"1H11 EFS"}="H111 EFS"; $resn_exch{"2H11 EFS"}="H112 EFS";
$resn_exch{"1H10 EFZ"}="H101 EFZ"; $resn_exch{"1H11 EFZ"}="H111 EFZ"; $resn_exch{"2H11 EFZ"}="H112 EFZ";
$resn_exch{"1H12 EFZ"}="H121 EFZ"; $resn_exch{"2H12 EFZ"}="H122 EFZ"; $resn_exch{"1HN1 EG1"}="HN11 EG1";
$resn_exch{"2HN1 EG1"}="HN12 EG1"; $resn_exch{"1H8  EG1"}=" H81 EG1"; $resn_exch{"2H8  EG1"}=" H82 EG1";
$resn_exch{"1H9  EG1"}=" H91 EG1"; $resn_exch{"2H9  EG1"}=" H92 EG1"; $resn_exch{"1H10 EG1"}="H101 EG1";
$resn_exch{"2H10 EG1"}="H102 EG1"; $resn_exch{"1H11 EG1"}="H111 EG1"; $resn_exch{"2H11 EG1"}="H112 EG1";
$resn_exch{"1H12 EG1"}="H121 EG1"; $resn_exch{"2H12 EG1"}="H122 EG1"; $resn_exch{"1H13 EG1"}="H131 EG1";
$resn_exch{"2H13 EG1"}="H132 EG1"; $resn_exch{"1H15 EG1"}="H151 EG1"; $resn_exch{"2H15 EG1"}="H152 EG1";
$resn_exch{"1HN4 EG1"}="HN41 EG1"; $resn_exch{"2HN4 EG1"}="HN42 EG1"; $resn_exch{"1HN1 EG2"}="HN11 EG2";
$resn_exch{"2HN1 EG2"}="HN12 EG2"; $resn_exch{"1H8  EG2"}=" H81 EG2"; $resn_exch{"2H8  EG2"}=" H82 EG2";
$resn_exch{"1H9  EG2"}=" H91 EG2"; $resn_exch{"2H9  EG2"}=" H92 EG2"; $resn_exch{"1H10 EG2"}="H101 EG2";
$resn_exch{"2H10 EG2"}="H102 EG2"; $resn_exch{"1H11 EG2"}="H111 EG2"; $resn_exch{"2H11 EG2"}="H112 EG2";
$resn_exch{"1H12 EG2"}="H121 EG2"; $resn_exch{"2H12 EG2"}="H122 EG2"; $resn_exch{"1H13 EG2"}="H131 EG2";
$resn_exch{"2H13 EG2"}="H132 EG2"; $resn_exch{"1HN3 EG2"}="HN31 EG2"; $resn_exch{"2HN3 EG2"}="HN32 EG2";
$resn_exch{"1HN1 EG3"}="HN11 EG3"; $resn_exch{"2HN1 EG3"}="HN12 EG3"; $resn_exch{"1H8  EG3"}=" H81 EG3";
$resn_exch{"2H8  EG3"}=" H82 EG3"; $resn_exch{"1H9  EG3"}=" H91 EG3"; $resn_exch{"2H9  EG3"}=" H92 EG3";
$resn_exch{"1H10 EG3"}="H101 EG3"; $resn_exch{"2H10 EG3"}="H102 EG3"; $resn_exch{"1H11 EG3"}="H111 EG3";
$resn_exch{"2H11 EG3"}="H112 EG3"; $resn_exch{"1H12 EG3"}="H121 EG3"; $resn_exch{"2H12 EG3"}="H122 EG3";
$resn_exch{"1H13 EG3"}="H131 EG3"; $resn_exch{"2H13 EG3"}="H132 EG3"; $resn_exch{"1HN4 EG3"}="HN41 EG3";
$resn_exch{"2HN4 EG3"}="HN42 EG3"; $resn_exch{"1H16 EG3"}="H161 EG3"; $resn_exch{"2H16 EG3"}="H162 EG3";
$resn_exch{"1H2  EGC"}=" H21 EGC"; $resn_exch{"2H2  EGC"}=" H22 EGC"; $resn_exch{"1H3  EGC"}="H31A EGC";
$resn_exch{"2H3  EGC"}=" H32 EGC"; $resn_exch{"1H5  EGC"}=" H51 EGC"; $resn_exch{"2H5  EGC"}=" H52 EGC";
$resn_exch{"1H6  EGC"}=" H61 EGC"; $resn_exch{"2H6  EGC"}=" H62 EGC"; $resn_exch{"1H8  EGC"}=" H81 EGC";
$resn_exch{"2H8  EGC"}=" H82 EGC"; $resn_exch{"1H9  EGC"}=" H91 EGC"; $resn_exch{"2H9  EGC"}=" H92 EGC";
$resn_exch{"1H11 EGC"}="H111 EGC"; $resn_exch{"2H11 EGC"}="H112 EGC"; $resn_exch{"1H12 EGC"}="H121 EGC";
$resn_exch{"2H12 EGC"}="H122 EGC"; $resn_exch{"1H14 EGC"}="H141 EGC"; $resn_exch{"2H14 EGC"}="H142 EGC";
$resn_exch{"1H15 EGC"}="H151 EGC"; $resn_exch{"2H15 EGC"}="H152 EGC"; $resn_exch{"1H17 EGC"}="H171 EGC";
$resn_exch{"2H17 EGC"}="H172 EGC"; $resn_exch{"1H18 EGC"}="H181 EGC"; $resn_exch{"2H18 EGC"}="H182 EGC";
$resn_exch{"1H20 EGC"}="H201 EGC"; $resn_exch{"2H20 EGC"}="H202 EGC"; $resn_exch{"1H21 EGC"}="H211 EGC";
$resn_exch{"2H21 EGC"}="H212 EGC"; $resn_exch{"1H23 EGC"}="H231 EGC"; $resn_exch{"2H23 EGC"}="H232 EGC";
$resn_exch{"1H24 EGC"}="H241 EGC"; $resn_exch{"2H24 EGC"}="H242 EGC"; $resn_exch{"1H26 EGC"}="H261 EGC";
$resn_exch{"2H26 EGC"}="H262 EGC"; $resn_exch{"1H27 EGC"}="H271 EGC"; $resn_exch{"2H27 EGC"}="H272 EGC";
$resn_exch{"1H36 EGC"}="H361 EGC"; $resn_exch{"2H36 EGC"}="H362 EGC"; $resn_exch{"3H36 EGC"}="H363 EGC";
$resn_exch{"1H37 EGC"}="H371 EGC"; $resn_exch{"2H37 EGC"}="H372 EGC"; $resn_exch{"3H37 EGC"}="H373 EGC";
$resn_exch{"1H38 EGC"}="H381 EGC"; $resn_exch{"2H38 EGC"}="H382 EGC"; $resn_exch{"1H40 EGC"}="H401 EGC";
$resn_exch{"2H40 EGC"}="H402 EGC"; $resn_exch{"3H40 EGC"}="H403 EGC"; $resn_exch{"1H41 EGC"}="H411 EGC";
$resn_exch{"2H41 EGC"}="H412 EGC"; $resn_exch{"3H41 EGC"}="H413 EGC"; $resn_exch{"1H42 EGC"}="H421 EGC";
$resn_exch{"2H42 EGC"}="H422 EGC"; $resn_exch{"3H42 EGC"}="H423 EGC"; $resn_exch{"1HG  EGD"}=" HG1 EGD";
$resn_exch{"2HG  EGD"}=" HG2 EGD"; $resn_exch{"3HG  EGD"}=" HG3 EGD"; $resn_exch{"1HD  EGD"}=" HD1 EGD";
$resn_exch{"2HD  EGD"}=" HD2 EGD"; $resn_exch{"1HH2 EGD"}="HH21 EGD"; $resn_exch{"2HH2 EGD"}="HH22 EGD";
$resn_exch{"1H1  EGL"}=" H11 EGL"; $resn_exch{"2H1  EGL"}=" H12 EGL"; $resn_exch{"1H2  EGL"}=" H21 EGL";
$resn_exch{"2H2  EGL"}=" H22 EGL"; $resn_exch{"1HC4 EGT"}="HC41 EGT"; $resn_exch{"2HC4 EGT"}="HC42 EGT";
$resn_exch{"1HA  EH5"}=" HA1 EH5"; $resn_exch{"1HB  EH5"}=" HB1 EH5"; $resn_exch{"2HB  EH5"}=" HB2 EH5";
$resn_exch{"1HD1 EH5"}="HD11 EH5"; $resn_exch{"1HD2 EH5"}="HD21 EH5"; $resn_exch{"1HE1 EH5"}="HE11 EH5";
$resn_exch{"1HE2 EH5"}="HE21 EH5"; $resn_exch{"1HZ  EH5"}=" HZ1 EH5"; $resn_exch{"1HN2 EH5"}="HN21 EH5";
$resn_exch{"1H1  EH5"}=" H11 EH5"; $resn_exch{"1HB1 EH5"}="HB11 EH5"; $resn_exch{"2HB1 EH5"}="HB12 EH5";
$resn_exch{"1HO1 EH5"}="HO11 EH5"; $resn_exch{"1H3  EH5"}=" H31 EH5"; $resn_exch{"2H3  EH5"}=" H32 EH5";
$resn_exch{"1H4  EH5"}=" H41 EH5"; $resn_exch{"2H4  EH5"}=" H42 EH5"; $resn_exch{"1H6  EH5"}=" H61 EH5";
$resn_exch{"2H6  EH5"}=" H62 EH5"; $resn_exch{"1H7  EH5"}=" H71 EH5"; $resn_exch{"2H7  EH5"}=" H72 EH5";
$resn_exch{"1H10 EH5"}="H101 EH5"; $resn_exch{"1H22 EH5"}="H221 EH5"; $resn_exch{"1H32 EH5"}="H321 EH5";
$resn_exch{"1H26 EH5"}="H261 EH5"; $resn_exch{"1H11 EH5"}="H111 EH5"; $resn_exch{"2H11 EH5"}="H112 EH5";
$resn_exch{"1H23 EH5"}="H231 EH5"; $resn_exch{"2H23 EH5"}="H232 EH5"; $resn_exch{"1H15 EH5"}="H151 EH5";
$resn_exch{"1H19 EH5"}="H191 EH5"; $resn_exch{"1H33 EH5"}="H331 EH5"; $resn_exch{"2H33 EH5"}="H332 EH5";
$resn_exch{"1H27 EH5"}="H271 EH5"; $resn_exch{"1H8  EH5"}=" H81 EH5"; $resn_exch{"2H8  EH5"}=" H82 EH5";
$resn_exch{"1H16 EH5"}="H161 EH5"; $resn_exch{"1H34 EH5"}="H341 EH5"; $resn_exch{"1H31 EH5"}="H311 EH5";
$resn_exch{"2H31 EH5"}="H312 EH5"; $resn_exch{"3H31 EH5"}="H313 EH5"; $resn_exch{"1H37 EH5"}="H371 EH5";
$resn_exch{"2H37 EH5"}="H372 EH5"; $resn_exch{"3H37 EH5"}="H373 EH5"; $resn_exch{"1H36 EH5"}="H361 EH5";
$resn_exch{"2H36 EH5"}="H362 EH5"; $resn_exch{"1H13 EH5"}="H131 EH5"; $resn_exch{"1H25 EH5"}="H251 EH5";
$resn_exch{"1H17 EH5"}="H171 EH5"; $resn_exch{"1H35 EH5"}="H351 EH5"; $resn_exch{"1H29 EH5"}="H291 EH5";
$resn_exch{"1HBK EHA"}="HBK1 EHA"; $resn_exch{"2HBK EHA"}="HBK2 EHA"; $resn_exch{"1HBL EHA"}="HBL1 EHA";
$resn_exch{"2HBL EHA"}="HBL2 EHA"; $resn_exch{"1HBM EHA"}="HBM1 EHA"; $resn_exch{"2HBM EHA"}="HBM2 EHA";
$resn_exch{"3HBM EHA"}="HBM3 EHA"; $resn_exch{"1HBI EHA"}="HBI1 EHA"; $resn_exch{"2HBI EHA"}="HBI2 EHA";
$resn_exch{"1HBH EHA"}="HBH1 EHA"; $resn_exch{"2HBH EHA"}="HBH2 EHA"; $resn_exch{"1HBG EHA"}="HBG1 EHA";
$resn_exch{"2HBG EHA"}="HBG2 EHA"; $resn_exch{"1HA  EHA"}=" HA1 EHA"; $resn_exch{"2HA  EHA"}=" HA2 EHA";
$resn_exch{"1H19 EHD"}="H191 EHD"; $resn_exch{"2H19 EHD"}="H192 EHD"; $resn_exch{"1H25 EHD"}="H251 EHD";
$resn_exch{"2H25 EHD"}="H252 EHD"; $resn_exch{"1H31 EHD"}="H311 EHD"; $resn_exch{"2H31 EHD"}="H312 EHD";
$resn_exch{"3H31 EHD"}="H313 EHD"; $resn_exch{"1H11 EHD"}="H111 EHD"; $resn_exch{"2H11 EHD"}="H112 EHD";
$resn_exch{"1H4  EHM"}=" H41 EHM"; $resn_exch{"2H4  EHM"}=" H42 EHM"; $resn_exch{"1HN  EHP"}=" HN1 EHP";
$resn_exch{"2HN  EHP"}=" HN2 EHP"; $resn_exch{"1HB  EHP"}=" HB1 EHP"; $resn_exch{"2HB  EHP"}=" HB2 EHP";
$resn_exch{"1HAA EI1"}="HAA1 EI1"; $resn_exch{"2HAA EI1"}="HAA2 EI1"; $resn_exch{"3HAA EI1"}="HAA3 EI1";
$resn_exch{"1HAK EI1"}="HAK1 EI1"; $resn_exch{"2HAK EI1"}="HAK2 EI1"; $resn_exch{"1H2  EIC"}=" H21 EIC";
$resn_exch{"2H2  EIC"}=" H22 EIC"; $resn_exch{"1H3  EIC"}=" H31 EIC"; $resn_exch{"2H3  EIC"}=" H32 EIC";
$resn_exch{"1H4  EIC"}=" H41 EIC"; $resn_exch{"2H4  EIC"}=" H42 EIC"; $resn_exch{"1H5  EIC"}=" H51 EIC";
$resn_exch{"2H5  EIC"}=" H52 EIC"; $resn_exch{"1H6  EIC"}=" H61 EIC"; $resn_exch{"2H6  EIC"}=" H62 EIC";
$resn_exch{"1H7  EIC"}=" H71 EIC"; $resn_exch{"2H7  EIC"}=" H72 EIC"; $resn_exch{"1H8  EIC"}=" H81 EIC";
$resn_exch{"2H8  EIC"}=" H82 EIC"; $resn_exch{"1H9  EIC"}=" H91 EIC"; $resn_exch{"1H1O EIC"}="H1O1 EIC";
$resn_exch{"1H11 EIC"}="H111 EIC"; $resn_exch{"2H11 EIC"}="H112 EIC"; $resn_exch{"1H12 EIC"}="H121 EIC";
$resn_exch{"1H13 EIC"}="H131 EIC"; $resn_exch{"1H14 EIC"}="H141 EIC"; $resn_exch{"2H14 EIC"}="H142 EIC";
$resn_exch{"1H15 EIC"}="H151 EIC"; $resn_exch{"2H15 EIC"}="H152 EIC"; $resn_exch{"1H16 EIC"}="H161 EIC";
$resn_exch{"2H16 EIC"}="H162 EIC"; $resn_exch{"1H17 EIC"}="H171 EIC"; $resn_exch{"2H17 EIC"}="H172 EIC";
$resn_exch{"1H18 EIC"}="H181 EIC"; $resn_exch{"2H18 EIC"}="H182 EIC"; $resn_exch{"3H18 EIC"}="H183 EIC";
$resn_exch{"1H3  EIN"}=" H31 EIN"; $resn_exch{"2H3  EIN"}=" H32 EIN"; $resn_exch{"3H3  EIN"}=" H33 EIN";
$resn_exch{"1H12 EIN"}="H121 EIN"; $resn_exch{"2H12 EIN"}="H122 EIN"; $resn_exch{"3H12 EIN"}="H123 EIN";
$resn_exch{"1H13 EIN"}="H131 EIN"; $resn_exch{"2H13 EIN"}="H132 EIN"; $resn_exch{"3H13 EIN"}="H133 EIN";
$resn_exch{"1H10 EIP"}="H101 EIP"; $resn_exch{"2H10 EIP"}="H102 EIP"; $resn_exch{"1H11 EIP"}="H111 EIP";
$resn_exch{"2H11 EIP"}="H112 EIP"; $resn_exch{"1H13 EIP"}="H131 EIP"; $resn_exch{"2H13 EIP"}="H132 EIP";
$resn_exch{"1H14 EIP"}="H141 EIP"; $resn_exch{"2H14 EIP"}="H142 EIP"; $resn_exch{"3H14 EIP"}="H143 EIP";
$resn_exch{"3HOP EIT"}="HOP3 EIT"; $resn_exch{"2HOP EIT"}="HOP2 EIT"; $resn_exch{"1H5* EIT"}=" H5' EIT";
$resn_exch{"2H5* EIT"}="H5'' EIT"; $resn_exch{" H4* EIT"}=" H4' EIT"; $resn_exch{" H1* EIT"}=" H1' EIT";
$resn_exch{"1H5M EIT"}=" H71 EIT"; $resn_exch{"2H5M EIT"}=" H72 EIT"; $resn_exch{"3H5M EIT"}=" H73 EIT";
$resn_exch{" H2* EIT"}=" H2' EIT"; $resn_exch{"1HB' EIT"}="HB'1 EIT"; $resn_exch{"2HB' EIT"}="HB'2 EIT";
$resn_exch{"1HC' EIT"}="HC'1 EIT"; $resn_exch{"2HC' EIT"}="HC'2 EIT"; $resn_exch{" H3* EIT"}=" H3' EIT";
$resn_exch{"H3T  EIT"}="HO3' EIT"; $resn_exch{"1HC6 EJT"}="HC61 EJT"; $resn_exch{"2HC6 EJT"}="HC62 EJT";
$resn_exch{"1HX6 EJT"}="HX61 EJT"; $resn_exch{"2HX6 EJT"}="HX62 EJT"; $resn_exch{"1HCB EJT"}="HCB1 EJT";
$resn_exch{"2HCB EJT"}="HCB2 EJT"; $resn_exch{"1HCG EJT"}="HCG1 EJT"; $resn_exch{"2HCG EJT"}="HCG2 EJT";
$resn_exch{"1HCD EJT"}="HCDA EJT"; $resn_exch{"2HCD EJT"}="HCDB EJT"; $resn_exch{"1HCE EJT"}="HCE1 EJT";
$resn_exch{"2HCE EJT"}="HCE2 EJT"; $resn_exch{"1HXG EJT"}="HXG1 EJT"; $resn_exch{"2HXG EJT"}="HXG2 EJT";
$resn_exch{"1HXD EJT"}="HXD1 EJT"; $resn_exch{"2HXD EJT"}="HXD2 EJT"; $resn_exch{"1HXE EJT"}="HXE1 EJT";
$resn_exch{"2HXE EJT"}="HXE2 EJT"; $resn_exch{"1HN  EKE"}=" HN1 EKE"; $resn_exch{"2HN  EKE"}=" HN2 EKE";
$resn_exch{"1HB  EKE"}=" HB1 EKE"; $resn_exch{"2HB  EKE"}=" HB2 EKE"; $resn_exch{"1HG  EKE"}=" HG1 EKE";
$resn_exch{"2HG  EKE"}=" HG2 EKE"; $resn_exch{"1HMK EKE"}="HMK1 EKE"; $resn_exch{"2HMK EKE"}="HMK2 EKE";
$resn_exch{"3HMK EKE"}="HMK3 EKE"; $resn_exch{"1H11  EL"}="H111  EL"; $resn_exch{"2H11  EL"}="H112  EL";
$resn_exch{"3H11  EL"}="H113  EL"; $resn_exch{"1H15  EL"}="H151  EL"; $resn_exch{"2H15  EL"}="H152  EL";
$resn_exch{"3H15  EL"}="H153  EL"; $resn_exch{"1HA  ELA"}=" HA1 ELA"; $resn_exch{"2HA  ELA"}=" HA2 ELA";
$resn_exch{"1H3  ELA"}=" H31 ELA"; $resn_exch{"2H3  ELA"}=" H32 ELA"; $resn_exch{"1H4  ELA"}=" H41 ELA";
$resn_exch{"2H4  ELA"}=" H42 ELA"; $resn_exch{"1H5  ELA"}=" H51 ELA"; $resn_exch{"2H5  ELA"}=" H52 ELA";
$resn_exch{"1H6  ELA"}=" H61 ELA"; $resn_exch{"2H6  ELA"}=" H62 ELA"; $resn_exch{"1H7  ELA"}=" H71 ELA";
$resn_exch{"2H7  ELA"}=" H72 ELA"; $resn_exch{"1H8  ELA"}=" H81 ELA"; $resn_exch{"2H8  ELA"}=" H82 ELA";
$resn_exch{"1H11 ELA"}="H111 ELA"; $resn_exch{"2H11 ELA"}="H112 ELA"; $resn_exch{"1H12 ELA"}="H121 ELA";
$resn_exch{"2H12 ELA"}="H122 ELA"; $resn_exch{"1H13 ELA"}="H131 ELA"; $resn_exch{"2H13 ELA"}="H132 ELA";
$resn_exch{"1H14 ELA"}="H141 ELA"; $resn_exch{"2H14 ELA"}="H142 ELA"; $resn_exch{"1H15 ELA"}="H151 ELA";
$resn_exch{"2H15 ELA"}="H152 ELA"; $resn_exch{"1H16 ELA"}="H161 ELA"; $resn_exch{"2H16 ELA"}="H162 ELA";
$resn_exch{"1H17 ELA"}="H171 ELA"; $resn_exch{"2H17 ELA"}="H172 ELA"; $resn_exch{"1H18 ELA"}="H181 ELA";
$resn_exch{"2H18 ELA"}="H182 ELA"; $resn_exch{"3H18 ELA"}="H183 ELA"; $resn_exch{"1HN1 ELD"}="HN11 ELD";
$resn_exch{"2HN1 ELD"}="HN12 ELD"; $resn_exch{"1H2  ELD"}=" H21 ELD"; $resn_exch{"2H2  ELD"}=" H22 ELD";
$resn_exch{"1H3  ELD"}=" H31 ELD"; $resn_exch{"2H3  ELD"}=" H32 ELD"; $resn_exch{"1H4  ELD"}=" H41 ELD";
$resn_exch{"2H4  ELD"}=" H42 ELD"; $resn_exch{"1H5  ELD"}=" H51 ELD"; $resn_exch{"2H5  ELD"}=" H52 ELD";
$resn_exch{"1H6  ELD"}=" H61 ELD"; $resn_exch{"2H6  ELD"}=" H62 ELD"; $resn_exch{"1H7  ELD"}=" H71 ELD";
$resn_exch{"2H7  ELD"}=" H72 ELD"; $resn_exch{"1H8  ELD"}=" H81 ELD"; $resn_exch{"2H8  ELD"}=" H82 ELD";
$resn_exch{"1H11 ELD"}="H111 ELD"; $resn_exch{"2H11 ELD"}="H112 ELD"; $resn_exch{"1H12 ELD"}="H121 ELD";
$resn_exch{"2H12 ELD"}="H122 ELD"; $resn_exch{"1H13 ELD"}="H131 ELD"; $resn_exch{"2H13 ELD"}="H132 ELD";
$resn_exch{"1H14 ELD"}="H141 ELD"; $resn_exch{"2H14 ELD"}="H142 ELD"; $resn_exch{"1H15 ELD"}="H151 ELD";
$resn_exch{"2H15 ELD"}="H152 ELD"; $resn_exch{"1H16 ELD"}="H161 ELD"; $resn_exch{"2H16 ELD"}="H162 ELD";
$resn_exch{"1H17 ELD"}="H171 ELD"; $resn_exch{"2H17 ELD"}="H172 ELD"; $resn_exch{"1H18 ELD"}="H181 ELD";
$resn_exch{"2H18 ELD"}="H182 ELD"; $resn_exch{"3H18 ELD"}="H183 ELD"; $resn_exch{"1H17 ELI"}="H171 ELI";
$resn_exch{"2H17 ELI"}="H172 ELI"; $resn_exch{"3H17 ELI"}="H173 ELI"; $resn_exch{"1H18 ELI"}="H181 ELI";
$resn_exch{"2H18 ELI"}="H182 ELI"; $resn_exch{"1H19 ELI"}="H191 ELI"; $resn_exch{"2H19 ELI"}="H192 ELI";
$resn_exch{"1H20 ELI"}="H201 ELI"; $resn_exch{"2H20 ELI"}="H202 ELI"; $resn_exch{"1H23 ELI"}="H231 ELI";
$resn_exch{"2H23 ELI"}="H232 ELI"; $resn_exch{"1H26 ELI"}="H261 ELI"; $resn_exch{"2H26 ELI"}="H262 ELI";
$resn_exch{"1HA1 ELP"}="HA11 ELP"; $resn_exch{"2HA1 ELP"}="HA12 ELP"; $resn_exch{"3HA1 ELP"}="HA13 ELP";
$resn_exch{"1H2A ELP"}="H2A1 ELP"; $resn_exch{"2H2A ELP"}="H2A2 ELP"; $resn_exch{"3H2A ELP"}="H2A3 ELP";
$resn_exch{"1H5A ELP"}="H5A1 ELP"; $resn_exch{"2H5A ELP"}="H5A2 ELP"; $resn_exch{"1H5* EMA"}="H5'1 EMA";
$resn_exch{"2H5* EMA"}="H5'2 EMA"; $resn_exch{"1H4* EMA"}="H4'1 EMA"; $resn_exch{"2H4* EMA"}="H4'2 EMA";
$resn_exch{"1H1* EMA"}="H1'1 EMA"; $resn_exch{"2H1* EMA"}="H1'2 EMA"; $resn_exch{"1HN6 EMA"}="HN61 EMA";
$resn_exch{"2HN6 EMA"}="HN62 EMA"; $resn_exch{"3H1  EMB"}=" H13 EMB"; $resn_exch{"2H1  EMB"}=" H12 EMB";
$resn_exch{"1H1  EMB"}=" H11 EMB"; $resn_exch{"2H2  EMB"}=" H22 EMB"; $resn_exch{"1H2  EMB"}=" H21 EMB";
$resn_exch{"3H4  EMB"}=" H43 EMB"; $resn_exch{"2H4  EMB"}=" H42 EMB"; $resn_exch{"1H4  EMB"}=" H41 EMB";
$resn_exch{"1H1  EMC"}=" H11 EMC"; $resn_exch{"2H1  EMC"}=" H12 EMC"; $resn_exch{"1H2  EMC"}=" H21 EMC";
$resn_exch{"2H2  EMC"}=" H22 EMC"; $resn_exch{"3H2  EMC"}=" H23 EMC"; $resn_exch{"1H7  EMD"}=" H71 EMD";
$resn_exch{"2H7  EMD"}=" H72 EMD"; $resn_exch{"3H7  EMD"}=" H73 EMD"; $resn_exch{"1H15 EMD"}="H151 EMD";
$resn_exch{"2H15 EMD"}="H152 EMD"; $resn_exch{"1H16 EMD"}="H161 EMD"; $resn_exch{"2H16 EMD"}="H162 EMD";
$resn_exch{"1H17 EMD"}="H171 EMD"; $resn_exch{"2H17 EMD"}="H172 EMD"; $resn_exch{"1H25 EMD"}="H251 EMD";
$resn_exch{"2H25 EMD"}="H252 EMD"; $resn_exch{"3H25 EMD"}="H253 EMD"; $resn_exch{"1H26 EMD"}="H261 EMD";
$resn_exch{"2H26 EMD"}="H262 EMD"; $resn_exch{"3H26 EMD"}="H263 EMD"; $resn_exch{"1H11 EMF"}="H111 EMF";
$resn_exch{"2H11 EMF"}="H112 EMF"; $resn_exch{"1H12 EMF"}="H121 EMF"; $resn_exch{"2H12 EMF"}="H122 EMF";
$resn_exch{"3H12 EMF"}="H123 EMF"; $resn_exch{"1H14 EMF"}="H141 EMF"; $resn_exch{"2H14 EMF"}="H142 EMF";
$resn_exch{"1H13 EMF"}="H131 EMF"; $resn_exch{"2H13 EMF"}="H132 EMF"; $resn_exch{"3H13 EMF"}="H133 EMF";
$resn_exch{"1H2C EMM"}="H2C1 EMM"; $resn_exch{"2H2C EMM"}="H2C2 EMM"; $resn_exch{"3H2C EMM"}="H2C3 EMM";
$resn_exch{"1H3C EMM"}="H3C1 EMM"; $resn_exch{"2H3C EMM"}="H3C2 EMM"; $resn_exch{"1H10 EMO"}="H101 EMO";
$resn_exch{"2H10 EMO"}="H102 EMO"; $resn_exch{"3H10 EMO"}="H103 EMO"; $resn_exch{"1H2  EMP"}=" H21 EMP";
$resn_exch{"2H2  EMP"}=" H22 EMP"; $resn_exch{"1H5  EMP"}=" H51 EMP"; $resn_exch{"2H5  EMP"}=" H52 EMP";
$resn_exch{"1H6  EMP"}=" H61 EMP"; $resn_exch{"2H6  EMP"}=" H62 EMP"; $resn_exch{"1H7  EMP"}=" H71 EMP";
$resn_exch{"2H7  EMP"}=" H72 EMP"; $resn_exch{"3H7  EMP"}=" H73 EMP"; $resn_exch{"1H8  EMP"}=" H81 EMP";
$resn_exch{"2H8  EMP"}=" H82 EMP"; $resn_exch{"3H8  EMP"}=" H83 EMP"; $resn_exch{"1H2  EMR"}=" H21 EMR";
$resn_exch{"2H2  EMR"}=" H22 EMR"; $resn_exch{"1H3  EMR"}=" H31 EMR"; $resn_exch{"2H3  EMR"}=" H32 EMR";
$resn_exch{"1H5  EMR"}=" H51 EMR"; $resn_exch{"2H5  EMR"}=" H52 EMR"; $resn_exch{"1H6  EMR"}=" H61 EMR";
$resn_exch{"2H6  EMR"}=" H62 EMR"; $resn_exch{"1H1' EMR"}="H1'1 EMR"; $resn_exch{"2H1' EMR"}="H1'2 EMR";
$resn_exch{"1H2' EMR"}="H2'1 EMR"; $resn_exch{"2H2' EMR"}="H2'2 EMR"; $resn_exch{"1HN2 EMR"}="HN21 EMR";
$resn_exch{"2HN2 EMR"}="HN22 EMR"; $resn_exch{"1H1  EMT"}=" H11 EMT"; $resn_exch{"2H1  EMT"}=" H12 EMT";
$resn_exch{"1H2  EMT"}=" H21 EMT"; $resn_exch{"2H2  EMT"}=" H22 EMT"; $resn_exch{"3H2  EMT"}=" H23 EMT";
$resn_exch{"1H9C EMU"}="H9C1 EMU"; $resn_exch{"2H9C EMU"}="H9C2 EMU"; $resn_exch{" H1* ENA"}=" H1' ENA";
$resn_exch{" H2* ENA"}=" H2' ENA"; $resn_exch{" H4* ENA"}=" H4' ENA"; $resn_exch{" H3* ENA"}=" H3' ENA";
$resn_exch{"4HO* ENA"}="HO'4 ENA"; $resn_exch{"1H5* ENA"}=" H5' ENA"; $resn_exch{"2H5* ENA"}="H5'' ENA";
$resn_exch{"3HO* ENA"}="HO'3 ENA"; $resn_exch{"1H6* ENA"}="H6'1 ENA"; $resn_exch{"2H6* ENA"}="H6'2 ENA";
$resn_exch{" H7* ENA"}=" H7' ENA"; $resn_exch{" H8* ENA"}=" H8' ENA"; $resn_exch{" H9* ENA"}=" H9' ENA";
$resn_exch{" H0* ENA"}=" H0' ENA"; $resn_exch{"7HO* ENA"}="HO'7 ENA"; $resn_exch{"8HO* ENA"}="HO'8 ENA";
$resn_exch{"1H15 ENB"}="H151 ENB"; $resn_exch{"2H15 ENB"}="H152 ENB"; $resn_exch{"1H17 ENB"}="H171 ENB";
$resn_exch{"2H17 ENB"}="H172 ENB"; $resn_exch{"1H18 ENB"}="H181 ENB"; $resn_exch{"2H18 ENB"}="H182 ENB";
$resn_exch{"1H21 ENB"}="H211 ENB"; $resn_exch{"2H21 ENB"}="H212 ENB"; $resn_exch{"3H21 ENB"}="H213 ENB";
$resn_exch{"1H22 ENB"}="H221 ENB"; $resn_exch{"2H22 ENB"}="H222 ENB"; $resn_exch{"3H22 ENB"}="H223 ENB";
$resn_exch{"1H23 ENB"}="H231 ENB"; $resn_exch{"2H23 ENB"}="H232 ENB"; $resn_exch{"3H23 ENB"}="H233 ENB";
$resn_exch{"1H24 ENB"}="H241 ENB"; $resn_exch{"2H24 ENB"}="H242 ENB"; $resn_exch{"3H24 ENB"}="H243 ENB";
$resn_exch{"1H25 ENB"}="H251 ENB"; $resn_exch{"2H25 ENB"}="H252 ENB"; $resn_exch{"3H25 ENB"}="H253 ENB";
$resn_exch{"1H28 ENB"}="H281 ENB"; $resn_exch{"2H28 ENB"}="H282 ENB"; $resn_exch{"1H29 ENB"}="H291 ENB";
$resn_exch{"2H29 ENB"}="H292 ENB"; $resn_exch{"3H29 ENB"}="H293 ENB"; $resn_exch{"1H1  ENC"}=" H11 ENC";
$resn_exch{"2H1  ENC"}=" H12 ENC"; $resn_exch{"1H2  ENC"}=" H21 ENC"; $resn_exch{"2H2  ENC"}=" H22 ENC";
$resn_exch{"3H2  ENC"}=" H23 ENC"; $resn_exch{"1HAI END"}="HAI1 END"; $resn_exch{"2HAI END"}="HAI2 END";
$resn_exch{"1HAK END"}="HAK1 END"; $resn_exch{"2HAK END"}="HAK2 END"; $resn_exch{"1HAJ END"}="HAJ1 END";
$resn_exch{"2HAJ END"}="HAJ2 END"; $resn_exch{"1HAH END"}="HAH1 END"; $resn_exch{"2HAH END"}="HAH2 END";
$resn_exch{"1HAK ENE"}="HAK1 ENE"; $resn_exch{"2HAK ENE"}="HAK2 ENE"; $resn_exch{"1HAJ ENE"}="HAJ1 ENE";
$resn_exch{"2HAJ ENE"}="HAJ2 ENE"; $resn_exch{"1HAL ENE"}="HAL1 ENE"; $resn_exch{"2HAL ENE"}="HAL2 ENE";
$resn_exch{"1HAN ENE"}="HAN1 ENE"; $resn_exch{"2HAN ENE"}="HAN2 ENE"; $resn_exch{"1HAM ENE"}="HAM1 ENE";
$resn_exch{"2HAM ENE"}="HAM2 ENE"; $resn_exch{"1H   ENH"}=" H1  ENH"; $resn_exch{"1H9  ENH"}=" H91 ENH";
$resn_exch{"1H1  ENH"}=" H11 ENH"; $resn_exch{"2H1  ENH"}=" H12 ENH"; $resn_exch{"3H1  ENH"}=" H13 ENH";
$resn_exch{"1H21 ENM"}="H211 ENM"; $resn_exch{"2H21 ENM"}="H212 ENM"; $resn_exch{"1H20 ENM"}="H201 ENM";
$resn_exch{"2H20 ENM"}="H202 ENM"; $resn_exch{"1H19 ENM"}="H191 ENM"; $resn_exch{"2H19 ENM"}="H192 ENM";
$resn_exch{"1H13 ENM"}="H131 ENM"; $resn_exch{"2H13 ENM"}="H132 ENM"; $resn_exch{"1H14 ENM"}="H141 ENM";
$resn_exch{"2H14 ENM"}="H142 ENM"; $resn_exch{"1H18 ENM"}="H181 ENM"; $resn_exch{"2H18 ENM"}="H182 ENM";
$resn_exch{"3H18 ENM"}="H183 ENM"; $resn_exch{"1H06 ENM"}="H061 ENM"; $resn_exch{"2H06 ENM"}="H062 ENM";
$resn_exch{"1H01 ENM"}="H011 ENM"; $resn_exch{"2H01 ENM"}="H012 ENM"; $resn_exch{"1H03 ENM"}="H031 ENM";
$resn_exch{"2H03 ENM"}="H032 ENM"; $resn_exch{"1H07 ENM"}="H071 ENM"; $resn_exch{"2H07 ENM"}="H072 ENM";
$resn_exch{"1H08 ENM"}="H081 ENM"; $resn_exch{"2H08 ENM"}="H082 ENM"; $resn_exch{"1H15 ENM"}="H151 ENM";
$resn_exch{"2H15 ENM"}="H152 ENM"; $resn_exch{"1H16 ENM"}="H161 ENM"; $resn_exch{"2H16 ENM"}="H162 ENM";
$resn_exch{"1H3  ENO"}=" H31 ENO"; $resn_exch{"2H3  ENO"}=" H32 ENO"; $resn_exch{"1H5* ENP"}=" H5' ENP";
$resn_exch{"2H5* ENP"}="H5'' ENP"; $resn_exch{" H4* ENP"}=" H4' ENP"; $resn_exch{" H3* ENP"}=" H3' ENP";
$resn_exch{" H2* ENP"}=" H2' ENP"; $resn_exch{" H1* ENP"}=" H1' ENP"; $resn_exch{"1H0* ENP"}="H0'1 ENP";
$resn_exch{"2H0* ENP"}="H0'2 ENP"; $resn_exch{" H9* ENP"}=" H9' ENP"; $resn_exch{" H8* ENP"}=" H8' ENP";
$resn_exch{"7HO* ENP"}="HO'7 ENP"; $resn_exch{" H7* ENP"}=" H7' ENP"; $resn_exch{"6HO* ENP"}="HO'6 ENP";
$resn_exch{" H6* ENP"}=" H6' ENP"; $resn_exch{"1H8  ENT"}=" H81 ENT"; $resn_exch{"2H8  ENT"}=" H82 ENT";
$resn_exch{"1H16 ENT"}="H161 ENT"; $resn_exch{"2H16 ENT"}="H162 ENT"; $resn_exch{"3H16 ENT"}="H163 ENT";
$resn_exch{"1H7  ENT"}=" H71 ENT"; $resn_exch{"2H7  ENT"}=" H72 ENT"; $resn_exch{"1H15 ENT"}="H151 ENT";
$resn_exch{"2H15 ENT"}="H152 ENT"; $resn_exch{"3H15 ENT"}="H153 ENT"; $resn_exch{"1H14 ENT"}="H141 ENT";
$resn_exch{"2H14 ENT"}="H142 ENT"; $resn_exch{"3H14 ENT"}="H143 ENT"; $resn_exch{"1H13 ENT"}="H131 ENT";
$resn_exch{"2H13 ENT"}="H132 ENT"; $resn_exch{"3H13 ENT"}="H133 ENT"; $resn_exch{"1H1C ENX"}="H1C1 ENX";
$resn_exch{"2H1C ENX"}="H1C2 ENX"; $resn_exch{"3H1C ENX"}="H1C3 ENX"; $resn_exch{"1H2C ENX"}="H2C1 ENX";
$resn_exch{"2H2C ENX"}="H2C2 ENX"; $resn_exch{"1H8C ENX"}="H8C1 ENX"; $resn_exch{"2H8C ENX"}="H8C2 ENX";
$resn_exch{"1H26 ENX"}="H261 ENX"; $resn_exch{"2H26 ENX"}="H262 ENX"; $resn_exch{"1H34 ENX"}="H341 ENX";
$resn_exch{"2H34 ENX"}="H342 ENX"; $resn_exch{"3H34 ENX"}="H343 ENX"; $resn_exch{"1H36 ENX"}="H361 ENX";
$resn_exch{"2H36 ENX"}="H362 ENX"; $resn_exch{"3H36 ENX"}="H363 ENX"; $resn_exch{"1H39 ENX"}="H391 ENX";
$resn_exch{"2H39 ENX"}="H392 ENX"; $resn_exch{"1H40 ENX"}="H401 ENX"; $resn_exch{"2H40 ENX"}="H402 ENX";
$resn_exch{"1H43 ENX"}="H431 ENX"; $resn_exch{"2H43 ENX"}="H432 ENX"; $resn_exch{"1H7  EOA"}=" H71 EOA";
$resn_exch{"2H7  EOA"}=" H72 EOA"; $resn_exch{"1H8  EOA"}=" H81 EOA"; $resn_exch{"2H8  EOA"}=" H82 EOA";
$resn_exch{"1HAM EOB"}="HAM1 EOB"; $resn_exch{"2HAM EOB"}="HAM2 EOB"; $resn_exch{"1HAN EOB"}="HAN1 EOB";
$resn_exch{"2HAN EOB"}="HAN2 EOB"; $resn_exch{"1H2  EOD"}=" H21 EOD"; $resn_exch{"2H2  EOD"}=" H22 EOD";
$resn_exch{"1H3  EOD"}=" H31 EOD"; $resn_exch{"2H3  EOD"}=" H32 EOD"; $resn_exch{"1H4  EOD"}=" H41 EOD";
$resn_exch{"2H4  EOD"}=" H42 EOD"; $resn_exch{"1H5  EOD"}=" H51 EOD"; $resn_exch{"2H5  EOD"}=" H52 EOD";
$resn_exch{"1H6  EOD"}=" H61 EOD"; $resn_exch{"2H6  EOD"}=" H62 EOD"; $resn_exch{"1H7  EOD"}=" H71 EOD";
$resn_exch{"2H7  EOD"}=" H72 EOD"; $resn_exch{"1H8  EOD"}=" H81 EOD"; $resn_exch{"2H8  EOD"}=" H82 EOD";
$resn_exch{"1H9  EOD"}=" H91 EOD"; $resn_exch{"2H9  EOD"}=" H92 EOD"; $resn_exch{"1H10 EOD"}="H101 EOD";
$resn_exch{"2H10 EOD"}="H102 EOD"; $resn_exch{"1H11 EOD"}="H111 EOD"; $resn_exch{"2H11 EOD"}="H112 EOD";
$resn_exch{"1H14 EOD"}="H141 EOD"; $resn_exch{"2H14 EOD"}="H142 EOD"; $resn_exch{"1H15 EOD"}="H151 EOD";
$resn_exch{"2H15 EOD"}="H152 EOD"; $resn_exch{"1H16 EOD"}="H161 EOD"; $resn_exch{"2H16 EOD"}="H162 EOD";
$resn_exch{"1H17 EOD"}="H171 EOD"; $resn_exch{"2H17 EOD"}="H172 EOD"; $resn_exch{"1H18 EOD"}="H181 EOD";
$resn_exch{"2H18 EOD"}="H182 EOD"; $resn_exch{"3H18 EOD"}="H183 EOD"; $resn_exch{"1H1  EOH"}=" H11 EOH";
$resn_exch{"2H1  EOH"}=" H12 EOH"; $resn_exch{"1H2  EOH"}=" H21 EOH"; $resn_exch{"2H2  EOH"}=" H22 EOH";
$resn_exch{"3H2  EOH"}=" H23 EOH"; $resn_exch{"1HA* EOM"}="HA'1 EOM"; $resn_exch{"2HA* EOM"}="HA'2 EOM";
$resn_exch{"1HC* EOM"}="HC'1 EOM"; $resn_exch{"2HC* EOM"}="HC'2 EOM"; $resn_exch{"1HD* EOM"}="HD'1 EOM";
$resn_exch{"2HD* EOM"}="HD'2 EOM"; $resn_exch{"3HD* EOM"}="HD'3 EOM"; $resn_exch{"1HAL EOP"}="HAL1 EOP";
$resn_exch{"2HAL EOP"}="HAL2 EOP"; $resn_exch{"1HAJ EOP"}="HAJ1 EOP"; $resn_exch{"2HAJ EOP"}="HAJ2 EOP";
$resn_exch{"1HAI EOP"}="HAI1 EOP"; $resn_exch{"2HAI EOP"}="HAI2 EOP"; $resn_exch{"1HAK EOP"}="HAK1 EOP";
$resn_exch{"2HAK EOP"}="HAK2 EOP"; $resn_exch{"1H19 EOT"}="H191 EOT"; $resn_exch{"2H19 EOT"}="H192 EOT";
$resn_exch{"1H20 EOT"}="H201 EOT"; $resn_exch{"2H20 EOT"}="H202 EOT"; $resn_exch{"1H11 EOT"}="H111 EOT";
$resn_exch{"2H11 EOT"}="H112 EOT"; $resn_exch{"1H2  EOT"}=" H21 EOT"; $resn_exch{"2H2  EOT"}=" H22 EOT";
$resn_exch{"1H9  EOT"}=" H91 EOT"; $resn_exch{"2H9  EOT"}=" H92 EOT"; $resn_exch{"1H7  EOT"}=" H71 EOT";
$resn_exch{"2H7  EOT"}=" H72 EOT"; $resn_exch{"1H5  EOT"}=" H51 EOT"; $resn_exch{"2H5  EOT"}=" H52 EOT";
$resn_exch{"1H3  EOT"}=" H31 EOT"; $resn_exch{"2H3  EOT"}=" H32 EOT"; $resn_exch{"1H1  EOX"}=" H11 EOX";
$resn_exch{"2H1  EOX"}=" H12 EOX"; $resn_exch{"1H2  EOX"}=" H21 EOX"; $resn_exch{"2H2  EOX"}=" H22 EOX";
$resn_exch{"3H2  EOX"}=" H23 EOX"; $resn_exch{"1HAN EOZ"}="HAN1 EOZ"; $resn_exch{"2HAN EOZ"}="HAN2 EOZ";
$resn_exch{"1HAO EOZ"}="HAO1 EOZ"; $resn_exch{"2HAO EOZ"}="HAO2 EOZ"; $resn_exch{"1H16  EP"}="H161  EP";
$resn_exch{"2H16  EP"}="H162  EP"; $resn_exch{"3H16  EP"}="H163  EP"; $resn_exch{"1H6   EP"}=" H61  EP";
$resn_exch{"2H6   EP"}=" H62  EP"; $resn_exch{"3H6   EP"}=" H63  EP"; $resn_exch{"1H21  EP"}="H211  EP";
$resn_exch{"2H21  EP"}="H212  EP"; $resn_exch{"1H32  EP"}="H321  EP"; $resn_exch{"2H32  EP"}="H322  EP";
$resn_exch{"1H35  EP"}="H351  EP"; $resn_exch{"2H35  EP"}="H352  EP"; $resn_exch{"1H38  EP"}="H381  EP";
$resn_exch{"2H38  EP"}="H382  EP"; $resn_exch{"1H43  EP"}="H431  EP"; $resn_exch{"2H43  EP"}="H432  EP";
$resn_exch{"3H43  EP"}="H433  EP"; $resn_exch{"1H53  EP"}="H531  EP"; $resn_exch{"2H53  EP"}="H532  EP";
$resn_exch{"3H53  EP"}="H533  EP"; $resn_exch{"1H64  EP"}="H641  EP"; $resn_exch{"2H64  EP"}="H642  EP";
$resn_exch{"3H64  EP"}="H643  EP"; $resn_exch{"1H60  EP"}="H601  EP"; $resn_exch{"2H60  EP"}="H602  EP";
$resn_exch{"3H60  EP"}="H603  EP"; $resn_exch{"1H72  EP"}="H721  EP"; $resn_exch{"2H72  EP"}="H722  EP";
$resn_exch{"1H11 EP1"}="H111 EP1"; $resn_exch{"2H11 EP1"}="H112 EP1"; $resn_exch{"1H10 EP1"}="H101 EP1";
$resn_exch{"2H10 EP1"}="H102 EP1"; $resn_exch{"1H9  EP1"}=" H91 EP1"; $resn_exch{"2H9  EP1"}=" H92 EP1";
$resn_exch{"1H2  EP1"}=" H21 EP1"; $resn_exch{"2H2  EP1"}=" H22 EP1"; $resn_exch{"1H3  EP1"}=" H31 EP1";
$resn_exch{"2H3  EP1"}=" H32 EP1"; $resn_exch{"1H5  EP1"}=" H51 EP1"; $resn_exch{"2H5  EP1"}=" H52 EP1";
$resn_exch{"1H6  EP1"}=" H61 EP1"; $resn_exch{"2H6  EP1"}=" H62 EP1"; $resn_exch{"1H7  EP1"}=" H71 EP1";
$resn_exch{"2H7  EP1"}=" H72 EP1"; $resn_exch{"1H8  EP1"}=" H81 EP1"; $resn_exch{"2H8  EP1"}=" H82 EP1";
$resn_exch{"1H2  EPA"}=" H21 EPA"; $resn_exch{"2H2  EPA"}=" H22 EPA"; $resn_exch{"1H3  EPA"}=" H31 EPA";
$resn_exch{"2H3  EPA"}=" H32 EPA"; $resn_exch{"1H4  EPA"}=" H41 EPA"; $resn_exch{"2H4  EPA"}=" H42 EPA";
$resn_exch{"1H5  EPA"}=" H51 EPA"; $resn_exch{"1H6  EPA"}=" H61 EPA"; $resn_exch{"1H7  EPA"}=" H71 EPA";
$resn_exch{"2H7  EPA"}=" H72 EPA"; $resn_exch{"1H8  EPA"}=" H81 EPA"; $resn_exch{"1H9  EPA"}=" H91 EPA";
$resn_exch{"1H10 EPA"}="H101 EPA"; $resn_exch{"2H10 EPA"}="H102 EPA"; $resn_exch{"1H11 EPA"}="H111 EPA";
$resn_exch{"1H12 EPA"}="H121 EPA"; $resn_exch{"1H13 EPA"}="H131 EPA"; $resn_exch{"2H13 EPA"}="H132 EPA";
$resn_exch{"1H14 EPA"}="H141 EPA"; $resn_exch{"1H15 EPA"}="H151 EPA"; $resn_exch{"1H16 EPA"}="H161 EPA";
$resn_exch{"2H16 EPA"}="H162 EPA"; $resn_exch{"1H17 EPA"}="H171 EPA"; $resn_exch{"1H18 EPA"}="H181 EPA";
$resn_exch{"1H19 EPA"}="H191 EPA"; $resn_exch{"2H19 EPA"}="H192 EPA"; $resn_exch{"1H20 EPA"}="H201 EPA";
$resn_exch{"2H20 EPA"}="H202 EPA"; $resn_exch{"3H20 EPA"}="H203 EPA"; $resn_exch{"1H51 EPB"}="H511 EPB";
$resn_exch{"2H51 EPB"}="H512 EPB"; $resn_exch{"3H51 EPB"}="H513 EPB"; $resn_exch{"1H3  EPB"}=" H31 EPB";
$resn_exch{"2H3  EPB"}=" H32 EPB"; $resn_exch{"1H21 EPB"}="H211 EPB"; $resn_exch{"2H21 EPB"}="H212 EPB";
$resn_exch{"3H21 EPB"}="H213 EPB"; $resn_exch{"1H27 EPB"}="H271 EPB"; $resn_exch{"2H27 EPB"}="H272 EPB";
$resn_exch{"3H27 EPB"}="H273 EPB"; $resn_exch{"1H15 EPB"}="H151 EPB"; $resn_exch{"2H15 EPB"}="H152 EPB";
$resn_exch{"1H29 EPB"}="H291 EPB"; $resn_exch{"2H29 EPB"}="H292 EPB"; $resn_exch{"3H29 EPB"}="H293 EPB";
$resn_exch{"1H30 EPB"}="H301 EPB"; $resn_exch{"2H30 EPB"}="H302 EPB"; $resn_exch{"3H30 EPB"}="H303 EPB";
$resn_exch{"1H28 EPB"}="H281 EPB"; $resn_exch{"2H28 EPB"}="H282 EPB"; $resn_exch{"3H28 EPB"}="H283 EPB";
$resn_exch{"1H91 EPB"}="H911 EPB"; $resn_exch{"2H91 EPB"}="H912 EPB"; $resn_exch{"3H91 EPB"}="H913 EPB";
$resn_exch{"1H8  EPB"}=" H81 EPB"; $resn_exch{"2H8  EPB"}=" H82 EPB"; $resn_exch{"1H7  EPB"}=" H71 EPB";
$resn_exch{"2H7  EPB"}=" H72 EPB"; $resn_exch{"1H6  EPB"}=" H61 EPB"; $resn_exch{"2H6  EPB"}=" H62 EPB";
$resn_exch{"1H1C EPC"}="H1C1 EPC"; $resn_exch{"1H2A EPC"}="H2A1 EPC"; $resn_exch{"2H2A EPC"}="H2A2 EPC";
$resn_exch{"3H2A EPC"}="H2A3 EPC"; $resn_exch{"1H4A EPC"}="H4A1 EPC"; $resn_exch{"1HC6 EPC"}="HC61 EPC";
$resn_exch{"1H5A EPC"}="H5A1 EPC"; $resn_exch{"2H5A EPC"}="H5A2 EPC"; $resn_exch{"1HCA EPC"}="HCA1 EPC";
$resn_exch{"2HCA EPC"}="HCA2 EPC"; $resn_exch{"3HCA EPC"}="HCA3 EPC"; $resn_exch{"1H51 EPD"}="H511 EPD";
$resn_exch{"2H51 EPD"}="H512 EPD"; $resn_exch{"3H51 EPD"}="H513 EPD"; $resn_exch{"1H3  EPD"}=" H31 EPD";
$resn_exch{"2H3  EPD"}=" H32 EPD"; $resn_exch{"1H21 EPD"}="H211 EPD"; $resn_exch{"2H21 EPD"}="H212 EPD";
$resn_exch{"3H21 EPD"}="H213 EPD"; $resn_exch{"1H27 EPD"}="H271 EPD"; $resn_exch{"2H27 EPD"}="H272 EPD";
$resn_exch{"3H27 EPD"}="H273 EPD"; $resn_exch{"1H6  EPD"}=" H61 EPD"; $resn_exch{"2H6  EPD"}=" H62 EPD";
$resn_exch{"1H7  EPD"}=" H71 EPD"; $resn_exch{"2H7  EPD"}=" H72 EPD"; $resn_exch{"1H8  EPD"}=" H81 EPD";
$resn_exch{"2H8  EPD"}=" H82 EPD"; $resn_exch{"1H91 EPD"}="H911 EPD"; $resn_exch{"2H91 EPD"}="H912 EPD";
$resn_exch{"3H91 EPD"}="H913 EPD"; $resn_exch{"1H28 EPD"}="H281 EPD"; $resn_exch{"2H28 EPD"}="H282 EPD";
$resn_exch{"3H28 EPD"}="H283 EPD"; $resn_exch{"1H30 EPD"}="H301 EPD"; $resn_exch{"2H30 EPD"}="H302 EPD";
$resn_exch{"3H30 EPD"}="H303 EPD"; $resn_exch{"1H29 EPD"}="H291 EPD"; $resn_exch{"2H29 EPD"}="H292 EPD";
$resn_exch{"3H29 EPD"}="H293 EPD"; $resn_exch{"1H15 EPD"}="H151 EPD"; $resn_exch{"2H15 EPD"}="H152 EPD";
$resn_exch{"1H2  EPE"}=" H21 EPE"; $resn_exch{"2H2  EPE"}=" H22 EPE"; $resn_exch{"1H3  EPE"}=" H31 EPE";
$resn_exch{"2H3  EPE"}=" H32 EPE"; $resn_exch{"1H5  EPE"}=" H51 EPE"; $resn_exch{"2H5  EPE"}=" H52 EPE";
$resn_exch{"1H6  EPE"}=" H61 EPE"; $resn_exch{"2H6  EPE"}=" H62 EPE"; $resn_exch{"1H7  EPE"}=" H71 EPE";
$resn_exch{"2H7  EPE"}=" H72 EPE"; $resn_exch{"1H8  EPE"}=" H81 EPE"; $resn_exch{"2H8  EPE"}=" H82 EPE";
$resn_exch{"1H9  EPE"}=" H91 EPE"; $resn_exch{"2H9  EPE"}=" H92 EPE"; $resn_exch{"1H10 EPE"}="H101 EPE";
$resn_exch{"2H10 EPE"}="H102 EPE"; $resn_exch{"3HOS EPE"}="HOS3 EPE"; $resn_exch{"1H6  EPG"}=" H61 EPG";
$resn_exch{"2H6  EPG"}=" H62 EPG"; $resn_exch{"1H7  EPG"}=" H71 EPG"; $resn_exch{"2H7  EPG"}=" H72 EPG";
$resn_exch{"1H9  EPG"}=" H91 EPG"; $resn_exch{"2H9  EPG"}=" H92 EPG"; $resn_exch{"1H1  EPH"}=" H11 EPH";
$resn_exch{"2H1  EPH"}="H12A EPH"; $resn_exch{"1H18 EPH"}="H181 EPH"; $resn_exch{"2H18 EPH"}="H182 EPH";
$resn_exch{"1H19 EPH"}="H191 EPH"; $resn_exch{"2H19 EPH"}="H192 EPH"; $resn_exch{"1H20 EPH"}="H201 EPH";
$resn_exch{"2H20 EPH"}="H202 EPH"; $resn_exch{"1H21 EPH"}="H211 EPH"; $resn_exch{"2H21 EPH"}="H212 EPH";
$resn_exch{"1H22 EPH"}="H221 EPH"; $resn_exch{"2H22 EPH"}="H222 EPH"; $resn_exch{"1H23 EPH"}="H231 EPH";
$resn_exch{"2H23 EPH"}="H232 EPH"; $resn_exch{"1H26 EPH"}="H261 EPH"; $resn_exch{"2H26 EPH"}="H262 EPH";
$resn_exch{"1H27 EPH"}="H271 EPH"; $resn_exch{"2H27 EPH"}="H272 EPH"; $resn_exch{"1H30 EPH"}="H301 EPH";
$resn_exch{"2H30 EPH"}="H302 EPH"; $resn_exch{"1H31 EPH"}="H311 EPH"; $resn_exch{"2H31 EPH"}="H312 EPH";
$resn_exch{"1H34 EPH"}="H341 EPH"; $resn_exch{"2H34 EPH"}="H342 EPH"; $resn_exch{"3H34 EPH"}="H343 EPH";
$resn_exch{"1H37 EPH"}="H371 EPH"; $resn_exch{"2H37 EPH"}="H372 EPH"; $resn_exch{"1H5  EPH"}=" H51 EPH";
$resn_exch{"2H5  EPH"}=" H52 EPH"; $resn_exch{"1H6  EPH"}=" H61 EPH"; $resn_exch{"2H6  EPH"}=" H62 EPH";
$resn_exch{"1H7  EPH"}=" H71 EPH"; $resn_exch{"2H7  EPH"}=" H72 EPH"; $resn_exch{"1H8  EPH"}=" H81 EPH";
$resn_exch{"2H8  EPH"}=" H82 EPH"; $resn_exch{"1H9  EPH"}=" H91 EPH"; $resn_exch{"2H9  EPH"}=" H92 EPH";
$resn_exch{"1H10 EPH"}="H101 EPH"; $resn_exch{"2H10 EPH"}="H102 EPH"; $resn_exch{"1H11 EPH"}="H111 EPH";
$resn_exch{"2H11 EPH"}="H112 EPH"; $resn_exch{"1H14 EPH"}="H141 EPH"; $resn_exch{"2H14 EPH"}="H142 EPH";
$resn_exch{"1H17 EPH"}="H171 EPH"; $resn_exch{"2H17 EPH"}="H172 EPH"; $resn_exch{"1H35 EPH"}="H351 EPH";
$resn_exch{"2H35 EPH"}="H352 EPH"; $resn_exch{"1H36 EPH"}="H361 EPH"; $resn_exch{"2H36 EPH"}="H362 EPH";
$resn_exch{"3H36 EPH"}="H363 EPH"; $resn_exch{"1H38 EPH"}="H381 EPH"; $resn_exch{"2H38 EPH"}="H382 EPH";
$resn_exch{"1H39 EPH"}="H391 EPH"; $resn_exch{"2H39 EPH"}="H392 EPH"; $resn_exch{"1HN1 EPH"}="HN11 EPH";
$resn_exch{"2HN1 EPH"}="HN12 EPH"; $resn_exch{"1H2  EPI"}=" H21 EPI"; $resn_exch{"2H2  EPI"}=" H22 EPI";
$resn_exch{"1H3  EPI"}=" H31 EPI"; $resn_exch{"2H3  EPI"}=" H32 EPI"; $resn_exch{"1H5  EPI"}=" H51 EPI";
$resn_exch{"2H5  EPI"}=" H52 EPI"; $resn_exch{"1H6  EPI"}=" H61 EPI"; $resn_exch{"2H6  EPI"}=" H62 EPI";
$resn_exch{"1H1' EPI"}="H1'1 EPI"; $resn_exch{"2H1' EPI"}="H1'2 EPI"; $resn_exch{"1H2' EPI"}="H2'1 EPI";
$resn_exch{"2H2' EPI"}="H2'2 EPI"; $resn_exch{"3H2' EPI"}="H2'3 EPI"; $resn_exch{"1H2C EPJ"}="H2C1 EPJ";
$resn_exch{"2H2C EPJ"}="H2C2 EPJ"; $resn_exch{"1H4C EPJ"}="H4C1 EPJ"; $resn_exch{"2H4C EPJ"}="H4C2 EPJ";
$resn_exch{"1H5C EPJ"}="H5C1 EPJ"; $resn_exch{"2H5C EPJ"}="H5C2 EPJ"; $resn_exch{"1H9  EPL"}=" H91 EPL";
$resn_exch{"2H9  EPL"}=" H92 EPL"; $resn_exch{"3H9  EPL"}=" H93 EPL"; $resn_exch{"1H8  EPL"}=" H81 EPL";
$resn_exch{"2H8  EPL"}=" H82 EPL"; $resn_exch{"1H10 EPL"}="H101 EPL"; $resn_exch{"2H10 EPL"}="H102 EPL";
$resn_exch{"1H11 EPL"}="H111 EPL"; $resn_exch{"2H11 EPL"}="H112 EPL"; $resn_exch{"3H11 EPL"}="H113 EPL";
$resn_exch{"1H7  EPL"}=" H71 EPL"; $resn_exch{"2H7  EPL"}=" H72 EPL"; $resn_exch{"3H7  EPL"}=" H73 EPL";
$resn_exch{"1H3  EPM"}=" H31 EPM"; $resn_exch{"2H3  EPM"}=" H32 EPM"; $resn_exch{"1H4  EPM"}=" H41 EPM";
$resn_exch{"2H4  EPM"}=" H42 EPM"; $resn_exch{"1H5  EPM"}=" H51 EPM"; $resn_exch{"2H5  EPM"}=" H52 EPM";
$resn_exch{"3H5  EPM"}=" H53 EPM"; $resn_exch{"1H7  EPM"}=" H71 EPM"; $resn_exch{"2H7  EPM"}=" H72 EPM";
$resn_exch{"1H8  EPM"}=" H81 EPM"; $resn_exch{"2H8  EPM"}=" H82 EPM"; $resn_exch{"1H9  EPM"}=" H91 EPM";
$resn_exch{"2H9  EPM"}=" H92 EPM"; $resn_exch{"1H10 EPM"}="H101 EPM"; $resn_exch{"2H10 EPM"}="H102 EPM";
$resn_exch{"1H11 EPM"}="H111 EPM"; $resn_exch{"2H11 EPM"}="H112 EPM"; $resn_exch{"1H12 EPM"}="H121 EPM";
$resn_exch{"2H12 EPM"}="H122 EPM"; $resn_exch{"1H13 EPM"}="H131 EPM"; $resn_exch{"2H13 EPM"}="H132 EPM";
$resn_exch{"1H14 EPM"}="H141 EPM"; $resn_exch{"2H14 EPM"}="H142 EPM"; $resn_exch{"1H15 EPM"}="H151 EPM";
$resn_exch{"2H15 EPM"}="H152 EPM"; $resn_exch{"1H16 EPM"}="H161 EPM"; $resn_exch{"2H16 EPM"}="H162 EPM";
$resn_exch{"1H17 EPM"}="H171 EPM"; $resn_exch{"2H17 EPM"}="H172 EPM"; $resn_exch{"1H18 EPM"}="H181 EPM";
$resn_exch{"2H18 EPM"}="H182 EPM"; $resn_exch{"1H19 EPM"}="H191 EPM"; $resn_exch{"2H19 EPM"}="H192 EPM";
$resn_exch{"1H20 EPM"}="H201 EPM"; $resn_exch{"2H20 EPM"}="H202 EPM"; $resn_exch{"1H21 EPM"}="H211 EPM";
$resn_exch{"2H21 EPM"}="H212 EPM"; $resn_exch{"3H21 EPM"}="H213 EPM"; $resn_exch{"1H1  EPN"}=" H11 EPN";
$resn_exch{"2H1  EPN"}=" H12 EPN"; $resn_exch{"1H2  EPN"}=" H21 EPN"; $resn_exch{"2H2  EPN"}=" H22 EPN";
$resn_exch{"1H3  EPN"}=" H31 EPN"; $resn_exch{"2H3  EPN"}=" H32 EPN"; $resn_exch{"1H1  EPO"}=" H11 EPO";
$resn_exch{"1H3  EPO"}=" H31 EPO"; $resn_exch{"2H3  EPO"}=" H32 EPO"; $resn_exch{"4H4  EPO"}=" H44 EPO";
$resn_exch{"1HNQ EPQ"}="HNQ1 EPQ"; $resn_exch{"2HNQ EPQ"}="HNQ2 EPQ"; $resn_exch{"1HQB EPQ"}="HQB1 EPQ";
$resn_exch{"2HQB EPQ"}="HQB2 EPQ"; $resn_exch{"1HMK EPQ"}="HMK1 EPQ"; $resn_exch{"2HMK EPQ"}="HMK2 EPQ";
$resn_exch{"3HMK EPQ"}="HMK3 EPQ"; $resn_exch{"1HG  EPQ"}=" HG1 EPQ"; $resn_exch{"2HG  EPQ"}=" HG2 EPQ";
$resn_exch{"1HH1 EPQ"}="HH11 EPQ"; $resn_exch{"2HH1 EPQ"}="HH12 EPQ"; $resn_exch{"3HH1 EPQ"}="HH13 EPQ";
$resn_exch{"1HH2 EPQ"}="HH21 EPQ"; $resn_exch{"2HH2 EPQ"}="HH22 EPQ"; $resn_exch{"3HH2 EPQ"}="HH23 EPQ";
$resn_exch{"1H6  EPS"}=" H61 EPS"; $resn_exch{"2H6  EPS"}=" H62 EPS"; $resn_exch{"1H8  EPS"}=" H81 EPS";
$resn_exch{"2H8  EPS"}=" H82 EPS"; $resn_exch{"1H7  EPT"}=" H71 EPT"; $resn_exch{"2H7  EPT"}=" H72 EPT";
$resn_exch{"1H8  EPT"}=" H81 EPT"; $resn_exch{"2H8  EPT"}=" H82 EPT"; $resn_exch{"1H9  EPT"}=" H91 EPT";
$resn_exch{"2H9  EPT"}=" H92 EPT"; $resn_exch{"1H10 EPT"}="H101 EPT"; $resn_exch{"2H10 EPT"}="H102 EPT";
$resn_exch{"1H11 EPT"}="H111 EPT"; $resn_exch{"2H11 EPT"}="H112 EPT"; $resn_exch{"1H12 EPT"}="H121 EPT";
$resn_exch{"2H12 EPT"}="H122 EPT"; $resn_exch{"1H13 EPT"}="H131 EPT"; $resn_exch{"2H13 EPT"}="H132 EPT";
$resn_exch{"3H13 EPT"}="H133 EPT"; $resn_exch{"UHN3 EPU"}="HN3U EPU"; $resn_exch{" UH5 EPU"}=" H5U EPU";
$resn_exch{" UH6 EPU"}=" H6U EPU"; $resn_exch{"UH1* EPU"}=" H1D EPU"; $resn_exch{"UH2* EPU"}=" H2D EPU";
$resn_exch{"UHO2 EPU"}="HO2U EPU"; $resn_exch{"UH3* EPU"}=" H3D EPU"; $resn_exch{"UH4* EPU"}=" H4D EPU";
$resn_exch{"UHO3 EPU"}="HO3U EPU"; $resn_exch{"1H5* EPU"}="H5'1 EPU"; $resn_exch{"2H5* EPU"}="H5'2 EPU";
$resn_exch{"2HOA EPU"}="HOA2 EPU"; $resn_exch{"2HOB EPU"}="HOB2 EPU"; $resn_exch{"1H6  EPU"}=" H61 EPU";
$resn_exch{"2H6  EPU"}=" H62 EPU"; $resn_exch{"1H8  EPU"}=" H81 EPU"; $resn_exch{"2H8  EPU"}=" H82 EPU";
$resn_exch{"3H8  EPU"}=" H83 EPU"; $resn_exch{"EHO2 EPU"}="HO2E EPU"; $resn_exch{"1HE3 EPU"}="HE31 EPU";
$resn_exch{"2HE3 EPU"}="HE32 EPU"; $resn_exch{"1H1  EPX"}=" H11 EPX"; $resn_exch{"1H3  EPX"}="H31A EPX";
$resn_exch{"2H3  EPX"}="H32A EPX"; $resn_exch{"3H3  EPX"}=" H33 EPX"; $resn_exch{"1H5  EPX"}=" H51 EPX";
$resn_exch{"1H6  EPX"}=" H61 EPX"; $resn_exch{"2H6  EPX"}=" H62 EPX"; $resn_exch{"3H6  EPX"}=" H63 EPX";
$resn_exch{"1H7  EPX"}=" H71 EPX"; $resn_exch{"2H7  EPX"}=" H72 EPX"; $resn_exch{"1H8  EPX"}=" H81 EPX";
$resn_exch{"2H8  EPX"}=" H82 EPX"; $resn_exch{"3H8  EPX"}=" H83 EPX"; $resn_exch{"1H9  EPX"}=" H91 EPX";
$resn_exch{"2H9  EPX"}=" H92 EPX"; $resn_exch{"3H9  EPX"}=" H93 EPX"; $resn_exch{"1HN2 EPX"}="HN21 EPX";
$resn_exch{"1H13 EPX"}="H131 EPX"; $resn_exch{"2H13 EPX"}="H132 EPX"; $resn_exch{"3H13 EPX"}="H133 EPX";
$resn_exch{"1H14 EPX"}="H141 EPX"; $resn_exch{"2H14 EPX"}="H142 EPX"; $resn_exch{"1H15 EPX"}="H151 EPX";
$resn_exch{"2H15 EPX"}="H152 EPX"; $resn_exch{"3H15 EPX"}="H153 EPX"; $resn_exch{"1HN3 EPX"}="HN31 EPX";
$resn_exch{"1H19 EPX"}="H191 EPX"; $resn_exch{"2H19 EPX"}="H192 EPX"; $resn_exch{"3H19 EPX"}="H193 EPX";
$resn_exch{"1HN5 EPX"}="HN51 EPX"; $resn_exch{"1H26 EPX"}="H261 EPX"; $resn_exch{"2H26 EPX"}="H262 EPX";
$resn_exch{"1H25 EPX"}="H251 EPX"; $resn_exch{"2H25 EPX"}="H252 EPX"; $resn_exch{"3H25 EPX"}="H253 EPX";
$resn_exch{"1H24 EPX"}="H241 EPX"; $resn_exch{"2H24 EPX"}="H242 EPX"; $resn_exch{"3H24 EPX"}="H243 EPX";
$resn_exch{"1HO6 EPX"}="HO61 EPX"; $resn_exch{"1HN4 EPX"}="HN41 EPX"; $resn_exch{"1H30 EPX"}="H301 EPX";
$resn_exch{"2H30 EPX"}="H302 EPX"; $resn_exch{"3H30 EPX"}="H303 EPX"; $resn_exch{"1H28 EPX"}="H281 EPX";
$resn_exch{"2H28 EPX"}="H282 EPX"; $resn_exch{"3H28 EPX"}="H283 EPX"; $resn_exch{"1H27 EPX"}="H271 EPX";
$resn_exch{"2H27 EPX"}="H272 EPX"; $resn_exch{"1HO7 EPX"}="HO71 EPX"; $resn_exch{"1H3A EPY"}="H3A1 EPY";
$resn_exch{"4HNA EPY"}="HNA4 EPY"; $resn_exch{"1H1A EPY"}="H1A1 EPY"; $resn_exch{"2H1A EPY"}="H1A2 EPY";
$resn_exch{"1H5A EPY"}="H5A1 EPY"; $resn_exch{"2H5A EPY"}="H5A2 EPY"; $resn_exch{"6HOA EPY"}="HOA6 EPY";
$resn_exch{"6HOB EPY"}="HOB6 EPY"; $resn_exch{"1H5B EPY"}="H5B1 EPY"; $resn_exch{"1HN5 EPY"}="HN51 EPY";
$resn_exch{"2HN5 EPY"}="HN52 EPY"; $resn_exch{"1H4B EPY"}="H4B1 EPY"; $resn_exch{"2H4B EPY"}="H4B2 EPY";
$resn_exch{"1H3B EPY"}="H3B1 EPY"; $resn_exch{"2H3B EPY"}="H3B2 EPY"; $resn_exch{"1HNB EPY"}="HNB1 EPY";
$resn_exch{"1H1C EPY"}="H1C1 EPY"; $resn_exch{"1H3C EPY"}="H3C1 EPY"; $resn_exch{"2H3C EPY"}="H3C2 EPY";
$resn_exch{"2HOC EPY"}="HOC2 EPY"; $resn_exch{"1H2C EPY"}="H2C1 EPY"; $resn_exch{"2H2C EPY"}="H2C2 EPY";
$resn_exch{"1H9C EPY"}="H9C1 EPY"; $resn_exch{"1H8C EPY"}="H8C1 EPY"; $resn_exch{"1H6C EPY"}="H6C1 EPY";
$resn_exch{"1H5C EPY"}="H5C1 EPY"; $resn_exch{"2H10 EQI"}="H102 EQI"; $resn_exch{"1H13 EQI"}="H131 EQI";
$resn_exch{"1H18 EQI"}="H181 EQI"; $resn_exch{"2H18 EQI"}="H182 EQI"; $resn_exch{"1H19 EQI"}="H191 EQI";
$resn_exch{"2H19 EQI"}="H192 EQI"; $resn_exch{"1H24 EQI"}="H241 EQI"; $resn_exch{"2H24 EQI"}="H242 EQI";
$resn_exch{"1H25 EQI"}="H251 EQI"; $resn_exch{"2H25 EQI"}="H252 EQI"; $resn_exch{"1H27 EQI"}="H271 EQI";
$resn_exch{"2H27 EQI"}="H272 EQI"; $resn_exch{"3H27 EQI"}="H273 EQI"; $resn_exch{"1H3  EQP"}=" H31 EQP";
$resn_exch{"2H3  EQP"}=" H32 EQP"; $resn_exch{"1H9  EQP"}=" H91 EQP"; $resn_exch{"2H9  EQP"}=" H92 EQP";
$resn_exch{"1H11 EQP"}="H111 EQP"; $resn_exch{"2H11 EQP"}="H112 EQP"; $resn_exch{"3H11 EQP"}="H113 EQP";
$resn_exch{"2HOP EQP"}="HOP2 EQP"; $resn_exch{"3HOP EQP"}="HOP3 EQP"; $resn_exch{"1H18 EQU"}="H181 EQU";
$resn_exch{"2H18 EQU"}="H182 EQU"; $resn_exch{"1H19 EQU"}="H191 EQU"; $resn_exch{"2H19 EQU"}="H192 EQU";
$resn_exch{"1H24 EQU"}="H241 EQU"; $resn_exch{"2H24 EQU"}="H242 EQU"; $resn_exch{"1H25 EQU"}="H251 EQU";
$resn_exch{"2H25 EQU"}="H252 EQU"; $resn_exch{"1H27 EQU"}="H271 EQU"; $resn_exch{"2H27 EQU"}="H272 EQU";
$resn_exch{"3H27 EQU"}="H273 EQU"; $resn_exch{"1H10 ERD"}="H101 ERD"; $resn_exch{"2H10 ERD"}="H102 ERD";
$resn_exch{"1H1  ERG"}=" H11 ERG"; $resn_exch{"2H1  ERG"}=" H12 ERG"; $resn_exch{"1H2  ERG"}=" H21 ERG";
$resn_exch{"2H2  ERG"}="H22A ERG"; $resn_exch{"1H4  ERG"}=" H41 ERG"; $resn_exch{"2H4  ERG"}=" H42 ERG";
$resn_exch{"1H11 ERG"}="H111 ERG"; $resn_exch{"2H11 ERG"}="H112 ERG"; $resn_exch{"1H12 ERG"}="H121 ERG";
$resn_exch{"2H12 ERG"}="H122 ERG"; $resn_exch{"1H15 ERG"}="H151 ERG"; $resn_exch{"2H15 ERG"}="H152 ERG";
$resn_exch{"1H16 ERG"}="H161 ERG"; $resn_exch{"2H16 ERG"}="H162 ERG"; $resn_exch{"1H18 ERG"}="H181 ERG";
$resn_exch{"2H18 ERG"}="H182 ERG"; $resn_exch{"3H18 ERG"}="H183 ERG"; $resn_exch{"1H19 ERG"}="H191 ERG";
$resn_exch{"2H19 ERG"}="H192 ERG"; $resn_exch{"3H19 ERG"}="H193 ERG"; $resn_exch{"1H21 ERG"}="H211 ERG";
$resn_exch{"2H21 ERG"}="H212 ERG"; $resn_exch{"3H21 ERG"}="H213 ERG"; $resn_exch{"1H26 ERG"}="H261 ERG";
$resn_exch{"2H26 ERG"}="H262 ERG"; $resn_exch{"3H26 ERG"}="H263 ERG"; $resn_exch{"1H27 ERG"}="H271 ERG";
$resn_exch{"2H27 ERG"}="H272 ERG"; $resn_exch{"3H27 ERG"}="H273 ERG"; $resn_exch{"1H28 ERG"}="H281 ERG";
$resn_exch{"2H28 ERG"}="H282 ERG"; $resn_exch{"3H28 ERG"}="H283 ERG"; $resn_exch{"1H2  ERI"}=" H21 ERI";
$resn_exch{"2H2  ERI"}=" H22 ERI"; $resn_exch{"1H3  ERI"}=" H31 ERI"; $resn_exch{"2H3  ERI"}=" H32 ERI";
$resn_exch{"3H3  ERI"}=" H33 ERI"; $resn_exch{"1H4  ERI"}=" H41 ERI"; $resn_exch{"2H4  ERI"}=" H42 ERI";
$resn_exch{"3H4  ERI"}=" H43 ERI"; $resn_exch{"1H6  ERI"}=" H61 ERI"; $resn_exch{"2H6  ERI"}=" H62 ERI";
$resn_exch{"3H6  ERI"}=" H63 ERI"; $resn_exch{"1HN2 ERN"}="HN21 ERN"; $resn_exch{"2HN2 ERN"}="HN22 ERN";
$resn_exch{"1H34 ERN"}="H341 ERN"; $resn_exch{"2H34 ERN"}="H342 ERN"; $resn_exch{"3H34 ERN"}="H343 ERN";
$resn_exch{"1H36 ERN"}="H361 ERN"; $resn_exch{"2H36 ERN"}="H362 ERN"; $resn_exch{"1H37 ERN"}="H371 ERN";
$resn_exch{"2H37 ERN"}="H372 ERN"; $resn_exch{"3H37 ERN"}="H373 ERN"; $resn_exch{"1H30 ERN"}="H301 ERN";
$resn_exch{"2H30 ERN"}="H302 ERN"; $resn_exch{"3H30 ERN"}="H303 ERN"; $resn_exch{"1H32 ERN"}="H321 ERN";
$resn_exch{"2H32 ERN"}="H322 ERN"; $resn_exch{"3H32 ERN"}="H323 ERN"; $resn_exch{"1H7  ERN"}=" H71 ERN";
$resn_exch{"2H7  ERN"}=" H72 ERN"; $resn_exch{"1H33 ERN"}="H331 ERN"; $resn_exch{"2H33 ERN"}="H332 ERN";
$resn_exch{"3H33 ERN"}="H333 ERN"; $resn_exch{"1H35 ERN"}="H351 ERN"; $resn_exch{"2H35 ERN"}="H352 ERN";
$resn_exch{"3H35 ERN"}="H353 ERN"; $resn_exch{"1H31 ERN"}="H311 ERN"; $resn_exch{"2H31 ERN"}="H312 ERN";
$resn_exch{"3H31 ERN"}="H313 ERN"; $resn_exch{"1H25 ERN"}="H251 ERN"; $resn_exch{"2H25 ERN"}="H252 ERN";
$resn_exch{"1H27 ERN"}="H271 ERN"; $resn_exch{"2H27 ERN"}="H272 ERN"; $resn_exch{"3H27 ERN"}="H273 ERN";
$resn_exch{"1H28 ERN"}="H281 ERN"; $resn_exch{"2H28 ERN"}="H282 ERN"; $resn_exch{"3H28 ERN"}="H283 ERN";
$resn_exch{"1H29 ERN"}="H291 ERN"; $resn_exch{"2H29 ERN"}="H292 ERN"; $resn_exch{"3H29 ERN"}="H293 ERN";
$resn_exch{"1H15 ERN"}="H151 ERN"; $resn_exch{"2H15 ERN"}="H152 ERN"; $resn_exch{"1H20 ERN"}="H201 ERN";
$resn_exch{"2H20 ERN"}="H202 ERN"; $resn_exch{"3H20 ERN"}="H203 ERN"; $resn_exch{"1H19 ERN"}="H191 ERN";
$resn_exch{"2H19 ERN"}="H192 ERN"; $resn_exch{"3H19 ERN"}="H193 ERN"; $resn_exch{"1H21 ERN"}="H211 ERN";
$resn_exch{"2H21 ERN"}="H212 ERN"; $resn_exch{"3H21 ERN"}="H213 ERN"; $resn_exch{"1H40 ERT"}="H401 ERT";
$resn_exch{"2H40 ERT"}="H402 ERT"; $resn_exch{"3H40 ERT"}="H403 ERT"; $resn_exch{"1H22 ERT"}="H221 ERT";
$resn_exch{"2H22 ERT"}="H222 ERT"; $resn_exch{"3H22 ERT"}="H223 ERT"; $resn_exch{"1H8  ERT"}=" H81 ERT";
$resn_exch{"2H8  ERT"}=" H82 ERT"; $resn_exch{"1H13 ERT"}="H131 ERT"; $resn_exch{"2H13 ERT"}="H132 ERT";
$resn_exch{"1H14 ERT"}="H141 ERT"; $resn_exch{"2H14 ERT"}="H142 ERT"; $resn_exch{"3H14 ERT"}="H143 ERT";
$resn_exch{" H1* ERT"}=" H1' ERT"; $resn_exch{"1H2* ERT"}="H2'1 ERT"; $resn_exch{"2H2* ERT"}="H2'2 ERT";
$resn_exch{" H3* ERT"}=" H3' ERT"; $resn_exch{"1H7* ERT"}="H7'1 ERT"; $resn_exch{"2H7* ERT"}="H7'2 ERT";
$resn_exch{"3H7* ERT"}="H7'3 ERT"; $resn_exch{"1H8* ERT"}="H8'1 ERT"; $resn_exch{"2H8* ERT"}="H8'2 ERT";
$resn_exch{"3H8* ERT"}="H8'3 ERT"; $resn_exch{" H4* ERT"}=" H4' ERT"; $resn_exch{" H5* ERT"}=" H5' ERT";
$resn_exch{"1H6* ERT"}="H6'1 ERT"; $resn_exch{"2H6* ERT"}="H6'2 ERT"; $resn_exch{"3H6* ERT"}="H6'3 ERT";
$resn_exch{"1H7  ERY"}=" H71 ERY"; $resn_exch{"2H7  ERY"}=" H72 ERY"; $resn_exch{"1H34 ERY"}="H341 ERY";
$resn_exch{"2H34 ERY"}="H342 ERY"; $resn_exch{"3H34 ERY"}="H343 ERY"; $resn_exch{"1H33 ERY"}="H331 ERY";
$resn_exch{"2H33 ERY"}="H332 ERY"; $resn_exch{"3H33 ERY"}="H333 ERY"; $resn_exch{"1H35 ERY"}="H351 ERY";
$resn_exch{"2H35 ERY"}="H352 ERY"; $resn_exch{"3H35 ERY"}="H353 ERY"; $resn_exch{"1H36 ERY"}="H361 ERY";
$resn_exch{"2H36 ERY"}="H362 ERY"; $resn_exch{"1H30 ERY"}="H301 ERY"; $resn_exch{"2H30 ERY"}="H302 ERY";
$resn_exch{"3H30 ERY"}="H303 ERY"; $resn_exch{"1H32 ERY"}="H321 ERY"; $resn_exch{"2H32 ERY"}="H322 ERY";
$resn_exch{"3H32 ERY"}="H323 ERY"; $resn_exch{"1H25 ERY"}="H251 ERY"; $resn_exch{"2H25 ERY"}="H252 ERY";
$resn_exch{"1H27 ERY"}="H271 ERY"; $resn_exch{"2H27 ERY"}="H272 ERY"; $resn_exch{"3H27 ERY"}="H273 ERY";
$resn_exch{"1H28 ERY"}="H281 ERY"; $resn_exch{"2H28 ERY"}="H282 ERY"; $resn_exch{"3H28 ERY"}="H283 ERY";
$resn_exch{"1H15 ERY"}="H151 ERY"; $resn_exch{"2H15 ERY"}="H152 ERY"; $resn_exch{"1H20 ERY"}="H201 ERY";
$resn_exch{"2H20 ERY"}="H202 ERY"; $resn_exch{"3H20 ERY"}="H203 ERY"; $resn_exch{"1H29 ERY"}="H291 ERY";
$resn_exch{"2H29 ERY"}="H292 ERY"; $resn_exch{"3H29 ERY"}="H293 ERY"; $resn_exch{"1H21 ERY"}="H211 ERY";
$resn_exch{"2H21 ERY"}="H212 ERY"; $resn_exch{"3H21 ERY"}="H213 ERY"; $resn_exch{"1H37 ERY"}="H371 ERY";
$resn_exch{"2H37 ERY"}="H372 ERY"; $resn_exch{"3H37 ERY"}="H373 ERY"; $resn_exch{"1H31 ERY"}="H311 ERY";
$resn_exch{"2H31 ERY"}="H312 ERY"; $resn_exch{"3H31 ERY"}="H313 ERY"; $resn_exch{"1H19 ERY"}="H191 ERY";
$resn_exch{"2H19 ERY"}="H192 ERY"; $resn_exch{"3H19 ERY"}="H193 ERY"; $resn_exch{"1H1  ESA"}=" H11 ESA";
$resn_exch{"2H1  ESA"}=" H12 ESA"; $resn_exch{"3H1  ESA"}=" H13 ESA"; $resn_exch{"1H2  ESA"}=" H21 ESA";
$resn_exch{"2H2  ESA"}=" H22 ESA"; $resn_exch{"1HN  ESC"}=" H   ESC"; $resn_exch{"2HN  ESC"}=" H2  ESC";
$resn_exch{"1HCA ESC"}="HCA1 ESC"; $resn_exch{"1HCB ESC"}=" HB2 ESC"; $resn_exch{"2HCB ESC"}=" HB3 ESC";
$resn_exch{"1HCG ESC"}=" HG2 ESC"; $resn_exch{"2HCG ESC"}=" HG3 ESC"; $resn_exch{"1HCD ESC"}=" HD2 ESC";
$resn_exch{"2HCD ESC"}=" HD3 ESC"; $resn_exch{"1HCE ESC"}=" HE1 ESC"; $resn_exch{"2HCE ESC"}=" HE2 ESC";
$resn_exch{"3HCE ESC"}=" HE3 ESC"; $resn_exch{"1HG  ESD"}=" HG1 ESD"; $resn_exch{"2HG  ESD"}=" HG2 ESD";
$resn_exch{"2HD  ESD"}=" HD2 ESD"; $resn_exch{"1HD  ESD"}=" HD1 ESD"; $resn_exch{"2HA  ESD"}=" HA2 ESD";
$resn_exch{"1HA  ESD"}=" HA1 ESD"; $resn_exch{"1H12 ESH"}="H121 ESH"; $resn_exch{"2H12 ESH"}="H122 ESH";
$resn_exch{"2H13 ESH"}="H132 ESH"; $resn_exch{"1H13 ESH"}="H131 ESH"; $resn_exch{"1H14 ESH"}="H141 ESH";
$resn_exch{"2H14 ESH"}="H142 ESH"; $resn_exch{"1H23 ESH"}="H231 ESH"; $resn_exch{"2H23 ESH"}="H232 ESH";
$resn_exch{"3H32 ESH"}="H323 ESH"; $resn_exch{"2H32 ESH"}="H322 ESH"; $resn_exch{"1H32 ESH"}="H321 ESH";
$resn_exch{"3H33 ESH"}="H333 ESH"; $resn_exch{"2H33 ESH"}="H332 ESH"; $resn_exch{"1H33 ESH"}="H331 ESH";
$resn_exch{"1HH1 ESI"}="HH11 ESI"; $resn_exch{"2HH1 ESI"}="HH12 ESI"; $resn_exch{"1HH2 ESI"}="HH21 ESI";
$resn_exch{"2HH2 ESI"}="HH22 ESI"; $resn_exch{"1H3  ESL"}=" H31 ESL"; $resn_exch{"2H3  ESL"}=" H32 ESL";
$resn_exch{"1H4  ESL"}=" H41 ESL"; $resn_exch{"2H4  ESL"}=" H42 ESL"; $resn_exch{"1H6  ESL"}=" H61 ESL";
$resn_exch{"2H6  ESL"}=" H62 ESL"; $resn_exch{"3H6  ESL"}=" H63 ESL"; $resn_exch{"1H9  ESL"}=" H91 ESL";
$resn_exch{"2H9  ESL"}=" H92 ESL"; $resn_exch{"1H14 ESL"}="H141 ESL"; $resn_exch{"2H14 ESL"}="H142 ESL";
$resn_exch{"1H15 ESL"}="H151 ESL"; $resn_exch{"2H15 ESL"}="H152 ESL"; $resn_exch{"1H11 ESM"}="H111 ESM";
$resn_exch{"2H11 ESM"}="H112 ESM"; $resn_exch{"1H12 ESM"}="H121 ESM"; $resn_exch{"2H12 ESM"}="H122 ESM";
$resn_exch{"CH14 ESM"}="H14C ESM"; $resn_exch{"1H15 ESM"}="H151 ESM"; $resn_exch{"2H15 ESM"}="H152 ESM";
$resn_exch{"1H16 ESM"}="H161 ESM"; $resn_exch{"2H16 ESM"}="H162 ESM"; $resn_exch{"CH17 ESM"}="H17C ESM";
$resn_exch{"1H18 ESM"}="H181 ESM"; $resn_exch{"2H18 ESM"}="H182 ESM"; $resn_exch{"3H18 ESM"}="H183 ESM";
$resn_exch{"1HC6 ESM"}="HC61 ESM"; $resn_exch{"2HC6 ESM"}="HC62 ESM"; $resn_exch{"1HC7 ESM"}="HC71 ESM";
$resn_exch{"2HC7 ESM"}="HC72 ESM"; $resn_exch{"OH17 ESM"}="H17O ESM"; $resn_exch{"1H19 ESM"}="H191 ESM";
$resn_exch{"2H19 ESM"}="H192 ESM"; $resn_exch{"3H19 ESM"}="H193 ESM"; $resn_exch{"2HOP ESO"}="HOP2 ESO";
$resn_exch{"3HOP ESO"}="HOP3 ESO"; $resn_exch{"1H6  ESO"}=" H61 ESO"; $resn_exch{"2H6  ESO"}=" H62 ESO";
$resn_exch{"1H7  ESO"}=" H71 ESO"; $resn_exch{"2H7  ESO"}=" H72 ESO"; $resn_exch{"1H11 ESO"}="H111 ESO";
$resn_exch{"2H11 ESO"}="H112 ESO"; $resn_exch{"1H12 ESO"}="H121 ESO"; $resn_exch{"2H12 ESO"}="H122 ESO";
$resn_exch{"1H15 ESO"}="H151 ESO"; $resn_exch{"2H15 ESO"}="H152 ESO"; $resn_exch{"1H16 ESO"}="H161 ESO";
$resn_exch{"2H16 ESO"}="H162 ESO"; $resn_exch{"1H18 ESO"}="H181 ESO"; $resn_exch{"2H18 ESO"}="H182 ESO";
$resn_exch{"3H18 ESO"}="H183 ESO"; $resn_exch{"1HH1 ESP"}="HH11 ESP"; $resn_exch{"2HH1 ESP"}="HH12 ESP";
$resn_exch{"1HH2 ESP"}="HH21 ESP"; $resn_exch{"2HH2 ESP"}="HH22 ESP"; $resn_exch{"1H2C ESR"}="H2C1 ESR";
$resn_exch{"2H2C ESR"}="H2C2 ESR"; $resn_exch{"1H5C ESR"}="H5C1 ESR"; $resn_exch{"2H5C ESR"}="H5C2 ESR";
$resn_exch{"1H6C ESR"}="H6C1 ESR"; $resn_exch{"2H6C ESR"}="H6C2 ESR"; $resn_exch{"1H10 ESR"}="H101 ESR";
$resn_exch{"2H10 ESR"}="H102 ESR"; $resn_exch{"1H11 ESR"}="H111 ESR"; $resn_exch{"2H11 ESR"}="H112 ESR";
$resn_exch{"1H19 ESR"}="H191 ESR"; $resn_exch{"2H19 ESR"}="H192 ESR"; $resn_exch{"1H24 ESR"}="H241 ESR";
$resn_exch{"2H24 ESR"}="H242 ESR"; $resn_exch{"1H25 ESR"}="H251 ESR"; $resn_exch{"2H25 ESR"}="H252 ESR";
$resn_exch{"1H27 ESR"}="H271 ESR"; $resn_exch{"2H27 ESR"}="H272 ESR"; $resn_exch{"3H27 ESR"}="H273 ESR";
$resn_exch{"1H18 ESR"}="H181 ESR"; $resn_exch{"2H18 ESR"}="H182 ESR"; $resn_exch{"1H6  EST"}=" H61 EST";
$resn_exch{"2H6  EST"}=" H62 EST"; $resn_exch{"1H7  EST"}=" H71 EST"; $resn_exch{"2H7  EST"}=" H72 EST";
$resn_exch{"1H11 EST"}="H111 EST"; $resn_exch{"2H11 EST"}="H112 EST"; $resn_exch{"1H12 EST"}="H121 EST";
$resn_exch{"2H12 EST"}="H122 EST"; $resn_exch{"1H15 EST"}="H151 EST"; $resn_exch{"2H15 EST"}="H152 EST";
$resn_exch{"1H16 EST"}="H161 EST"; $resn_exch{"2H16 EST"}="H162 EST"; $resn_exch{"1H18 EST"}="H181 EST";
$resn_exch{"2H18 EST"}="H182 EST"; $resn_exch{"3H18 EST"}="H183 EST"; $resn_exch{"1HH1 ESX"}="HH11 ESX";
$resn_exch{"2HH1 ESX"}="HH12 ESX"; $resn_exch{"1HH2 ESX"}="HH21 ESX"; $resn_exch{"2HH2 ESX"}="HH22 ESX";
$resn_exch{"1H5  ESY"}=" H51 ESY"; $resn_exch{"2H5  ESY"}=" H52 ESY"; $resn_exch{"1H13 ESY"}="H131 ESY";
$resn_exch{"2H13 ESY"}="H132 ESY"; $resn_exch{"1H14 ESY"}="H141 ESY"; $resn_exch{"2H14 ESY"}="H142 ESY";
$resn_exch{"1H15 ESY"}="H151 ESY"; $resn_exch{"2H15 ESY"}="H152 ESY"; $resn_exch{"1H16 ESY"}="H161 ESY";
$resn_exch{"2H16 ESY"}="H162 ESY"; $resn_exch{"1H24 ESY"}="H241 ESY"; $resn_exch{"2H24 ESY"}="H242 ESY";
$resn_exch{"1H25 ESY"}="H251 ESY"; $resn_exch{"2H25 ESY"}="H252 ESY"; $resn_exch{"3H25 ESY"}="H253 ESY";
$resn_exch{"1H26 ESY"}="H261 ESY"; $resn_exch{"2H26 ESY"}="H262 ESY"; $resn_exch{"3H26 ESY"}="H263 ESY";
$resn_exch{"1H31 ESY"}="H311 ESY"; $resn_exch{"2H31 ESY"}="H312 ESY"; $resn_exch{"1H40 ESY"}="H401 ESY";
$resn_exch{"2H40 ESY"}="H402 ESY"; $resn_exch{"3H40 ESY"}="H403 ESY"; $resn_exch{"1H21  ET"}="H211  ET";
$resn_exch{"2H21  ET"}="H212  ET"; $resn_exch{"1H22  ET"}="H221  ET"; $resn_exch{"2H22  ET"}="H222  ET";
$resn_exch{"3H22  ET"}="H223  ET"; $resn_exch{"1H23  ET"}="H231  ET"; $resn_exch{"2H23  ET"}="H232  ET";
$resn_exch{"1H24  ET"}="H241  ET"; $resn_exch{"2H24  ET"}="H242  ET"; $resn_exch{"1HA  ETA"}=" HA1 ETA";
$resn_exch{"2HA  ETA"}=" HA2 ETA"; $resn_exch{"1HN  ETA"}=" HN1 ETA"; $resn_exch{"2HN  ETA"}=" HN2 ETA";
$resn_exch{"1HB  ETA"}=" HB1 ETA"; $resn_exch{"2HB  ETA"}=" HB2 ETA"; $resn_exch{"1H6A ETB"}="H6A1 ETB";
$resn_exch{"2H6A ETB"}="H6A2 ETB"; $resn_exch{"1H5B ETB"}="H5B1 ETB"; $resn_exch{"2H5B ETB"}="H5B2 ETB";
$resn_exch{"1HCP ETB"}="HCP1 ETB"; $resn_exch{"2HCP ETB"}="HCP2 ETB"; $resn_exch{"1HDP ETB"}="HDP1 ETB";
$resn_exch{"2HDP ETB"}="HDP2 ETB"; $resn_exch{"3HDP ETB"}="HDP3 ETB"; $resn_exch{"1HEP ETB"}="HEP1 ETB";
$resn_exch{"2HEP ETB"}="HEP2 ETB"; $resn_exch{"3HEP ETB"}="HEP3 ETB"; $resn_exch{"1H7P ETB"}="H7P1 ETB";
$resn_exch{"2H7P ETB"}="H7P2 ETB"; $resn_exch{"1H6P ETB"}="H6P1 ETB"; $resn_exch{"2H6P ETB"}="H6P2 ETB";
$resn_exch{"1H3P ETB"}="H3P1 ETB"; $resn_exch{"2H3P ETB"}="H3P2 ETB"; $resn_exch{"3H3P ETB"}="H3P3 ETB";
$resn_exch{"1H22 ETC"}="H221 ETC"; $resn_exch{"2H22 ETC"}="H222 ETC"; $resn_exch{"3H22 ETC"}="H223 ETC";
$resn_exch{"1H21 ETC"}="H211 ETC"; $resn_exch{"2H21 ETC"}="H212 ETC"; $resn_exch{"1H12 ETC"}="H121 ETC";
$resn_exch{"2H12 ETC"}="H122 ETC"; $resn_exch{"1HC6 ETC"}="HC61 ETC"; $resn_exch{"2HC6 ETC"}="HC62 ETC";
$resn_exch{"1H20 ETC"}="H201 ETC"; $resn_exch{"2H20 ETC"}="H202 ETC"; $resn_exch{"1H19 ETC"}="H191 ETC";
$resn_exch{"2H19 ETC"}="H192 ETC"; $resn_exch{"3H19 ETC"}="H193 ETC"; $resn_exch{"1H12 ETE"}="H121 ETE";
$resn_exch{"2H12 ETE"}="H122 ETE"; $resn_exch{"1H22 ETE"}="H221 ETE"; $resn_exch{"2H22 ETE"}="H222 ETE";
$resn_exch{"1H13 ETE"}="H131 ETE"; $resn_exch{"2H13 ETE"}="H132 ETE"; $resn_exch{"1H23 ETE"}="H231 ETE";
$resn_exch{"2H23 ETE"}="H232 ETE"; $resn_exch{"1H14 ETE"}="H141 ETE"; $resn_exch{"2H14 ETE"}="H142 ETE";
$resn_exch{"1H24 ETE"}="H241 ETE"; $resn_exch{"2H24 ETE"}="H242 ETE"; $resn_exch{"1H15 ETE"}="H151 ETE";
$resn_exch{"2H15 ETE"}="H152 ETE"; $resn_exch{"1H25 ETE"}="H251 ETE"; $resn_exch{"2H25 ETE"}="H252 ETE";
$resn_exch{"1H26 ETE"}="H261 ETE"; $resn_exch{"2H26 ETE"}="H262 ETE"; $resn_exch{"3H26 ETE"}="H263 ETE";
$resn_exch{"1H2  ETF"}=" H21 ETF"; $resn_exch{"2H2  ETF"}=" H22 ETF"; $resn_exch{"1H1  ETH"}=" H11 ETH";
$resn_exch{"2H1  ETH"}=" H12 ETH"; $resn_exch{"1H2  ETH"}=" H21 ETH"; $resn_exch{"2H2  ETH"}=" H22 ETH";
$resn_exch{"3H2  ETH"}=" H23 ETH"; $resn_exch{"1H1  ETI"}=" H11 ETI"; $resn_exch{"2H1  ETI"}=" H12 ETI";
$resn_exch{"1H2  ETI"}=" H21 ETI"; $resn_exch{"2H2  ETI"}=" H22 ETI"; $resn_exch{"3H2  ETI"}=" H23 ETI";
$resn_exch{"1H1  ETM"}=" H11 ETM"; $resn_exch{"2H1  ETM"}=" H12 ETM"; $resn_exch{"1H2  ETM"}=" H21 ETM";
$resn_exch{"2H2  ETM"}=" H22 ETM"; $resn_exch{"1H3  ETM"}=" H31 ETM"; $resn_exch{"2H3  ETM"}=" H32 ETM";
$resn_exch{"3H3  ETM"}=" H33 ETM"; $resn_exch{"1H4  ETM"}=" H41 ETM"; $resn_exch{"2H4  ETM"}=" H42 ETM";
$resn_exch{"3H4  ETM"}=" H43 ETM"; $resn_exch{"1H5  ETM"}=" H51 ETM"; $resn_exch{"2H5  ETM"}=" H52 ETM";
$resn_exch{"3H5  ETM"}=" H53 ETM"; $resn_exch{"1H   ETN"}=" H1  ETN"; $resn_exch{"2H   ETN"}=" H2  ETN";
$resn_exch{"3H   ETN"}=" H3  ETN"; $resn_exch{"1H1  ETN"}=" H11 ETN"; $resn_exch{"2H1  ETN"}=" H12 ETN";
$resn_exch{"1H2  ETN"}=" H21 ETN"; $resn_exch{"2H2  ETN"}=" H22 ETN"; $resn_exch{"3H2  ETN"}=" H23 ETN";
$resn_exch{"1HB  ETO"}=" HB1 ETO"; $resn_exch{"2HB  ETO"}=" HB2 ETO"; $resn_exch{"3HB  ETO"}=" HB3 ETO";
$resn_exch{"1HA  ETO"}=" HA1 ETO"; $resn_exch{"2HA  ETO"}=" HA2 ETO"; $resn_exch{"1H2  ETR"}=" H21 ETR";
$resn_exch{"2H2  ETR"}=" H22 ETR"; $resn_exch{"1H3  ETR"}=" H31 ETR"; $resn_exch{"2H3  ETR"}=" H32 ETR";
$resn_exch{"1H4  ETR"}=" H41 ETR"; $resn_exch{"2H4  ETR"}=" H42 ETR"; $resn_exch{"1H16 ETR"}="H161 ETR";
$resn_exch{"2H16 ETR"}="H162 ETR"; $resn_exch{"3H16 ETR"}="H163 ETR"; $resn_exch{"1H17 ETR"}="H171 ETR";
$resn_exch{"2H17 ETR"}="H172 ETR"; $resn_exch{"3H17 ETR"}="H173 ETR"; $resn_exch{"1H18 ETR"}="H181 ETR";
$resn_exch{"2H18 ETR"}="H182 ETR"; $resn_exch{"3H18 ETR"}="H183 ETR"; $resn_exch{"1H19 ETR"}="H191 ETR";
$resn_exch{"2H19 ETR"}="H192 ETR"; $resn_exch{"3H19 ETR"}="H193 ETR"; $resn_exch{"1H20 ETR"}="H201 ETR";
$resn_exch{"2H20 ETR"}="H202 ETR"; $resn_exch{"3H20 ETR"}="H203 ETR"; $resn_exch{"1H21 ETR"}="H211 ETR";
$resn_exch{"2H21 ETR"}="H212 ETR"; $resn_exch{"1H22 ETR"}="H221 ETR"; $resn_exch{"2H22 ETR"}="H222 ETR";
$resn_exch{"3H22 ETR"}="H223 ETR"; $resn_exch{"1HN3 ETS"}="HN31 ETS"; $resn_exch{"2HN3 ETS"}="HN32 ETS";
$resn_exch{"1H15 ETS"}="H151 ETS"; $resn_exch{"2H15 ETS"}="H152 ETS"; $resn_exch{"3H15 ETS"}="H153 ETS";
$resn_exch{"1H18 ETS"}="H181 ETS"; $resn_exch{"2H18 ETS"}="H182 ETS"; $resn_exch{"1H19 ETS"}="H191 ETS";
$resn_exch{"2H19 ETS"}="H192 ETS"; $resn_exch{"3H19 ETS"}="H193 ETS"; $resn_exch{"1HN6 ETV"}="HN61 ETV";
$resn_exch{"2HN6 ETV"}="HN62 ETV"; $resn_exch{"1H16 ETV"}="H161 ETV"; $resn_exch{"2H16 ETV"}="H162 ETV";
$resn_exch{"1H14 ETV"}="H141 ETV"; $resn_exch{"2H14 ETV"}="H142 ETV"; $resn_exch{"1H2  ETX"}=" H21 ETX";
$resn_exch{"2H2  ETX"}=" H22 ETX"; $resn_exch{"1H3  ETX"}=" H31 ETX"; $resn_exch{"2H3  ETX"}=" H32 ETX";
$resn_exch{"1H4  ETX"}=" H41 ETX"; $resn_exch{"2H4  ETX"}=" H42 ETX"; $resn_exch{"3H4  ETX"}=" H43 ETX";
$resn_exch{"1H1  ETX"}=" H11 ETX"; $resn_exch{"2H1  ETX"}=" H12 ETX"; $resn_exch{"1H8  ETY"}=" H81 ETY";
$resn_exch{"2H8  ETY"}=" H82 ETY"; $resn_exch{"3H8  ETY"}=" H83 ETY"; $resn_exch{"1H7  ETY"}=" H71 ETY";
$resn_exch{"2H7  ETY"}=" H72 ETY"; $resn_exch{"1H1  EUD"}=" H11 EUD"; $resn_exch{"2H1  EUD"}=" H12 EUD";
$resn_exch{"3H3  EUD"}=" H33 EUD"; $resn_exch{"1H4  EUD"}=" H41 EUD"; $resn_exch{"2H4  EUD"}=" H42 EUD";
$resn_exch{"3H4  EUD"}=" H43 EUD"; $resn_exch{"1H5  EUD"}=" H51 EUD"; $resn_exch{"2H5  EUD"}=" H52 EUD";
$resn_exch{"1H6  EUD"}=" H61 EUD"; $resn_exch{"2H6  EUD"}=" H62 EUD"; $resn_exch{"1H7  EUD"}=" H71 EUD";
$resn_exch{"2H7  EUD"}=" H72 EUD"; $resn_exch{"1H9  EUD"}=" H91 EUD"; $resn_exch{"2H9  EUD"}=" H92 EUD";
$resn_exch{"3H9  EUD"}=" H93 EUD"; $resn_exch{"1H10 EUD"}="H101 EUD"; $resn_exch{"2H10 EUD"}="H102 EUD";
$resn_exch{"1H11 EUD"}="H111 EUD"; $resn_exch{"2H11 EUD"}="H112 EUD"; $resn_exch{"1H12 EUD"}="H121 EUD";
$resn_exch{"1H14 EUD"}="H141 EUD"; $resn_exch{"2H14 EUD"}="H142 EUD"; $resn_exch{"3H14 EUD"}="H143 EUD";
$resn_exch{"1H15 EUD"}="H151 EUD"; $resn_exch{"2H15 EUD"}="H152 EUD"; $resn_exch{"1H8  EUG"}=" H81 EUG";
$resn_exch{"2H8  EUG"}=" H82 EUG"; $resn_exch{"1H9  EUG"}=" H91 EUG"; $resn_exch{"2H9  EUG"}=" H92 EUG";
$resn_exch{"3H9  EUG"}=" H93 EUG"; $resn_exch{"1HBH EXO"}="HBH1 EXO"; $resn_exch{"2HBH EXO"}="HBH2 EXO";
$resn_exch{"1HBE EXO"}="HBE1 EXO"; $resn_exch{"2HBE EXO"}="HBE2 EXO"; $resn_exch{"3H   EYS"}=" H3  EYS";
$resn_exch{"2H   EYS"}=" H2  EYS"; $resn_exch{"1H   EYS"}=" H   EYS"; $resn_exch{"2HB  EYS"}=" HB2 EYS";
$resn_exch{"1HB  EYS"}=" HB3 EYS"; $resn_exch{" H   EYS"}=" HC  EYS"; $resn_exch{"1H18 EZT"}="H181 EZT";
$resn_exch{"2H18 EZT"}="H182 EZT"; $resn_exch{"3H18 EZT"}="H183 EZT"; $resn_exch{"1H12 EZT"}="H121 EZT";
$resn_exch{"2H12 EZT"}="H122 EZT"; $resn_exch{"1H11 EZT"}="H111 EZT"; $resn_exch{"2H11 EZT"}="H112 EZT";
$resn_exch{"1H6  EZT"}=" H61 EZT"; $resn_exch{"2H6  EZT"}=" H62 EZT"; $resn_exch{"1H7  EZT"}=" H71 EZT";
$resn_exch{"2H7  EZT"}=" H72 EZT"; $resn_exch{"1H15 EZT"}="H151 EZT"; $resn_exch{"2H15 EZT"}="H152 EZT";
$resn_exch{"1H16 EZT"}="H161 EZT"; $resn_exch{"2H16 EZT"}="H162 EZT"; $resn_exch{"1H1  F09"}=" H11 F09";
$resn_exch{"2H1  F09"}=" H12 F09"; $resn_exch{"3H1  F09"}=" H13 F09"; $resn_exch{"1H2  F09"}=" H21 F09";
$resn_exch{"2H2  F09"}=" H22 F09"; $resn_exch{"1H3  F09"}=" H31 F09"; $resn_exch{"2H3  F09"}=" H32 F09";
$resn_exch{"1H4  F09"}=" H41 F09"; $resn_exch{"2H4  F09"}=" H42 F09"; $resn_exch{"1H5  F09"}=" H51 F09";
$resn_exch{"2H5  F09"}=" H52 F09"; $resn_exch{"1H6  F09"}=" H61 F09"; $resn_exch{"2H6  F09"}=" H62 F09";
$resn_exch{"1H7  F09"}=" H71 F09"; $resn_exch{"2H7  F09"}=" H72 F09"; $resn_exch{"1H8  F09"}=" H81 F09";
$resn_exch{"2H8  F09"}=" H82 F09"; $resn_exch{"1H9  F09"}=" H91 F09"; $resn_exch{"2H9  F09"}=" H92 F09";
$resn_exch{"1H17 F10"}="H171 F10"; $resn_exch{"2H17 F10"}="H172 F10"; $resn_exch{"1H18 F10"}="H181 F10";
$resn_exch{"2H18 F10"}="H182 F10"; $resn_exch{"1H3C F11"}="H3C1 F11"; $resn_exch{"2H3C F11"}="H3C2 F11";
$resn_exch{"1H4C F11"}="H4C1 F11"; $resn_exch{"2H4C F11"}="H4C2 F11"; $resn_exch{"1H5C F11"}="H5C1 F11";
$resn_exch{"2H5C F11"}="H5C2 F11"; $resn_exch{"1H6C F11"}="H6C1 F11"; $resn_exch{"2H6C F11"}="H6C2 F11";
$resn_exch{"1H19 F11"}="H191 F11"; $resn_exch{"2H19 F11"}="H192 F11"; $resn_exch{"1H20 F11"}="H201 F11";
$resn_exch{"2H20 F11"}="H202 F11"; $resn_exch{"1H21 F11"}="H211 F11"; $resn_exch{"2H21 F11"}="H212 F11";
$resn_exch{"1H24 F11"}="H241 F11"; $resn_exch{"2H24 F11"}="H242 F11"; $resn_exch{"1H25 F11"}="H251 F11";
$resn_exch{"2H25 F11"}="H252 F11"; $resn_exch{"1H28 F11"}="H281 F11"; $resn_exch{"2H28 F11"}="H282 F11";
$resn_exch{"1H29 F11"}="H291 F11"; $resn_exch{"2H29 F11"}="H292 F11"; $resn_exch{"1H30 F11"}="H301 F11";
$resn_exch{"2H30 F11"}="H302 F11"; $resn_exch{"1H31 F11"}="H311 F11"; $resn_exch{"2H31 F11"}="H312 F11";
$resn_exch{"1H12 F12"}="H121 F12"; $resn_exch{"2H12 F12"}="H122 F12"; $resn_exch{"3H12 F12"}="H123 F12";
$resn_exch{"1H20 F17"}="H201 F17"; $resn_exch{"2H20 F17"}="H202 F17"; $resn_exch{"1H35 F17"}="H351 F17";
$resn_exch{"2H35 F17"}="H352 F17"; $resn_exch{"1H40 F17"}="H401 F17"; $resn_exch{"2H40 F17"}="H402 F17";
$resn_exch{"1H7C F17"}="H7C1 F17"; $resn_exch{"2H7C F17"}="H7C2 F17"; $resn_exch{"1H16 F18"}="H161 F18";
$resn_exch{"2H16 F18"}="H162 F18"; $resn_exch{"1H18 F18"}="H181 F18"; $resn_exch{"2H18 F18"}="H182 F18";
$resn_exch{"1H16 F19"}="H161 F19"; $resn_exch{"2H16 F19"}="H162 F19"; $resn_exch{"1H18 F19"}="H181 F19";
$resn_exch{"2H18 F19"}="H182 F19"; $resn_exch{"1H19 F1G"}="H191 F1G"; $resn_exch{"2H19 F1G"}="H192 F1G";
$resn_exch{"3H19 F1G"}="H193 F1G"; $resn_exch{"1H3  F1G"}=" H31 F1G"; $resn_exch{"2H3  F1G"}=" H32 F1G";
$resn_exch{"1H2  F1G"}=" H21 F1G"; $resn_exch{"2H2  F1G"}=" H22 F1G"; $resn_exch{"1H1  F1P"}=" H11 F1P";
$resn_exch{"2H1  F1P"}=" H12 F1P"; $resn_exch{"1H6  F1P"}=" H61 F1P"; $resn_exch{"2H6  F1P"}=" H62 F1P";
$resn_exch{"1H2C F20"}="H2C1 F20"; $resn_exch{"2H2C F20"}="H2C2 F20"; $resn_exch{"1H21 F20"}="H211 F20";
$resn_exch{"2H21 F20"}="H212 F20"; $resn_exch{"1H27 F20"}="H271 F20"; $resn_exch{"2H27 F20"}="H272 F20";
$resn_exch{"1H4C F20"}="H4C1 F20"; $resn_exch{"2H4C F20"}="H4C2 F20"; $resn_exch{"1H12 F21"}="H121 F21";
$resn_exch{"2H12 F21"}="H122 F21"; $resn_exch{"1H13 F21"}="H131 F21"; $resn_exch{"2H13 F21"}="H132 F21";
$resn_exch{"1H15 F21"}="H151 F21"; $resn_exch{"2H15 F21"}="H152 F21"; $resn_exch{"1H14 F21"}="H141 F21";
$resn_exch{"2H14 F21"}="H142 F21"; $resn_exch{"1H1  F21"}=" H11 F21"; $resn_exch{"2H1  F21"}=" H12 F21";
$resn_exch{"1H11 F21"}="H111 F21"; $resn_exch{"2H11 F21"}="H112 F21"; $resn_exch{"1H4  F21"}=" H41 F21";
$resn_exch{"2H4  F21"}=" H42 F21"; $resn_exch{"1H10 F23"}="H101 F23"; $resn_exch{"2H10 F23"}="H102 F23";
$resn_exch{"1H11 F23"}="H111 F23"; $resn_exch{"2H11 F23"}="H112 F23"; $resn_exch{"1H12 F23"}="H121 F23";
$resn_exch{"2H12 F23"}="H122 F23"; $resn_exch{"1H13 F23"}="H131 F23"; $resn_exch{"2H13 F23"}="H132 F23";
$resn_exch{"1H14 F23"}="H141 F23"; $resn_exch{"2H14 F23"}="H142 F23"; $resn_exch{"1H15 F23"}="H151 F23";
$resn_exch{"2H15 F23"}="H152 F23"; $resn_exch{"1H16 F23"}="H161 F23"; $resn_exch{"2H16 F23"}="H162 F23";
$resn_exch{"1H17 F23"}="H171 F23"; $resn_exch{"2H17 F23"}="H172 F23"; $resn_exch{"1H18 F23"}="H181 F23";
$resn_exch{"2H18 F23"}="H182 F23"; $resn_exch{"1H19 F23"}="H191 F23"; $resn_exch{"2H19 F23"}="H192 F23";
$resn_exch{"1H20 F23"}="H201 F23"; $resn_exch{"2H20 F23"}="H202 F23"; $resn_exch{"1H21 F23"}="H211 F23";
$resn_exch{"2H21 F23"}="H212 F23"; $resn_exch{"1H22 F23"}="H221 F23"; $resn_exch{"2H22 F23"}="H222 F23";
$resn_exch{"1H23 F23"}="H231 F23"; $resn_exch{"2H23 F23"}="H232 F23"; $resn_exch{"3H23 F23"}="H233 F23";
$resn_exch{"1H6  F24"}=" H61 F24"; $resn_exch{"2H6  F24"}=" H62 F24"; $resn_exch{"1H21 F24"}="H211 F24";
$resn_exch{"2H21 F24"}="H212 F24"; $resn_exch{"3H21 F24"}="H213 F24"; $resn_exch{"1H12 F24"}="H121 F24";
$resn_exch{"2H12 F24"}="H122 F24"; $resn_exch{"1H7  F24"}=" H71 F24"; $resn_exch{"2H7  F24"}=" H72 F24";
$resn_exch{"3H7  F24"}=" H73 F24"; $resn_exch{"1H16 F24"}="H161 F24"; $resn_exch{"2H16 F24"}="H162 F24";
$resn_exch{"1H13 F24"}="H131 F24"; $resn_exch{"2H13 F24"}="H132 F24"; $resn_exch{"3H13 F24"}="H133 F24";
$resn_exch{"1H20 F24"}="H201 F24"; $resn_exch{"2H20 F24"}="H202 F24"; $resn_exch{"1H11 F24"}="H111 F24";
$resn_exch{"2H11 F24"}="H112 F24"; $resn_exch{"1H10 F24"}="H101 F24"; $resn_exch{"2H10 F24"}="H102 F24";
$resn_exch{"1H5  F24"}=" H51 F24"; $resn_exch{"2H5  F24"}=" H52 F24"; $resn_exch{"1H4  F24"}=" H41 F24";
$resn_exch{"2H4  F24"}=" H42 F24"; $resn_exch{"1H23 F25"}="H231 F25"; $resn_exch{"2H23 F25"}="H232 F25";
$resn_exch{"1H12 F25"}="H121 F25"; $resn_exch{"2H12 F25"}="H122 F25"; $resn_exch{"1H13 F25"}="H131 F25";
$resn_exch{"2H13 F25"}="H132 F25"; $resn_exch{"2H14 F25"}="H142 F25"; $resn_exch{"1H14 F25"}="H141 F25";
$resn_exch{"3H3  F25"}=" H33 F25"; $resn_exch{"1H30 F29"}="H301 F29"; $resn_exch{"2H30 F29"}="H302 F29";
$resn_exch{"1H3  F29"}=" H31 F29"; $resn_exch{"2H3  F29"}=" H32 F29"; $resn_exch{"1H17 F29"}="H171 F29";
$resn_exch{"2H17 F29"}="H172 F29"; $resn_exch{"CH01 F2B"}="H01C F2B"; $resn_exch{"CH02 F2B"}="H02C F2B";
$resn_exch{"CH04 F2B"}="H04C F2B"; $resn_exch{"CH05 F2B"}="H05C F2B"; $resn_exch{"NH09 F2B"}="H09N F2B";
$resn_exch{"1H10 F2B"}="H101 F2B"; $resn_exch{"2H10 F2B"}="H102 F2B"; $resn_exch{"1HNP F2B"}="HNP1 F2B";
$resn_exch{"2HNP F2B"}="HNP2 F2B"; $resn_exch{"CH16 F2B"}="H16C F2B"; $resn_exch{"CH17 F2B"}="H17C F2B";
$resn_exch{"CH18 F2B"}="H18C F2B"; $resn_exch{"1H24 F2G"}="H241 F2G"; $resn_exch{"2H24 F2G"}="H242 F2G";
$resn_exch{"1H26 F2G"}="H261 F2G"; $resn_exch{"2H26 F2G"}="H262 F2G"; $resn_exch{"1H27 F2G"}="H271 F2G";
$resn_exch{"2H27 F2G"}="H272 F2G"; $resn_exch{"1H28 F2G"}="H281 F2G"; $resn_exch{"2H28 F2G"}="H282 F2G";
$resn_exch{"1H29 F2G"}="H291 F2G"; $resn_exch{"2H29 F2G"}="H292 F2G"; $resn_exch{"1H30 F2G"}="H301 F2G";
$resn_exch{"2H30 F2G"}="H302 F2G"; $resn_exch{"1H5  F2G"}=" H51 F2G"; $resn_exch{"2H5  F2G"}=" H52 F2G";
$resn_exch{"3H5  F2G"}=" H53 F2G"; $resn_exch{"1H4  F2G"}=" H41 F2G"; $resn_exch{"2H4  F2G"}=" H42 F2G";
$resn_exch{"3H4  F2G"}=" H43 F2G"; $resn_exch{"1H10 F2G"}="H101 F2G"; $resn_exch{"2H10 F2G"}="H102 F2G";
$resn_exch{"3H10 F2G"}="H103 F2G"; $resn_exch{"1H31 F2G"}="H311 F2G"; $resn_exch{"2H31 F2G"}="H312 F2G";
$resn_exch{"3H31 F2G"}="H313 F2G"; $resn_exch{"1H15 F2G"}="H151 F2G"; $resn_exch{"2H15 F2G"}="H152 F2G";
$resn_exch{"1H7  F2I"}=" H71 F2I"; $resn_exch{"2H7  F2I"}=" H72 F2I"; $resn_exch{"3H7  F2I"}=" H73 F2I";
$resn_exch{"1H10 F2I"}="H101 F2I"; $resn_exch{"2H10 F2I"}="H102 F2I"; $resn_exch{"1H13 F2I"}="H131 F2I";
$resn_exch{"2H13 F2I"}="H132 F2I"; $resn_exch{"1H17 F2I"}="H171 F2I"; $resn_exch{"2H17 F2I"}="H172 F2I";
$resn_exch{"1H9  F2I"}=" H91 F2I"; $resn_exch{"2H9  F2I"}=" H92 F2I"; $resn_exch{"1H11 F2I"}="H111 F2I";
$resn_exch{"2H11 F2I"}="H112 F2I"; $resn_exch{"1H12 F2I"}="H121 F2I"; $resn_exch{"2H12 F2I"}="H122 F2I";
$resn_exch{"3H12 F2I"}="H123 F2I"; $resn_exch{"1H14 F2I"}="H141 F2I"; $resn_exch{"2H14 F2I"}="H142 F2I";
$resn_exch{"3H14 F2I"}="H143 F2I"; $resn_exch{"1H18 F2I"}="H181 F2I"; $resn_exch{"2H18 F2I"}="H182 F2I";
$resn_exch{"1H19 F2I"}="H191 F2I"; $resn_exch{"2H19 F2I"}="H192 F2I"; $resn_exch{"1HCD F2N"}="HCD1 F2N";
$resn_exch{"2HCD F2N"}="HCD2 F2N"; $resn_exch{"1HCE F2N"}="HCE1 F2N"; $resn_exch{"2HCE F2N"}="HCE2 F2N";
$resn_exch{"1HBJ F2N"}="HBJ1 F2N"; $resn_exch{"2HBJ F2N"}="HBJ2 F2N"; $resn_exch{"3HBJ F2N"}="HBJ3 F2N";
$resn_exch{"1HBH F2N"}="HBH1 F2N"; $resn_exch{"2HBH F2N"}="HBH2 F2N"; $resn_exch{"3HBH F2N"}="HBH3 F2N";
$resn_exch{"1HBO F2N"}="HBO1 F2N"; $resn_exch{"2HBO F2N"}="HBO2 F2N"; $resn_exch{"1HBV F2N"}="HBV1 F2N";
$resn_exch{"2HBV F2N"}="HBV2 F2N"; $resn_exch{"1H5* F2N"}="H5'1 F2N"; $resn_exch{"2H5* F2N"}="H5'2 F2N";
$resn_exch{" H4* F2N"}=" H4' F2N"; $resn_exch{" H3* F2N"}=" H3' F2N"; $resn_exch{"HO3* F2N"}="HO3' F2N";
$resn_exch{" H2* F2N"}=" H2' F2N"; $resn_exch{"HO2* F2N"}="HO2' F2N"; $resn_exch{" H1* F2N"}=" H1' F2N";
$resn_exch{"1HN6 F2N"}="HN61 F2N"; $resn_exch{"2HN6 F2N"}="HN62 F2N"; $resn_exch{"1H1C F2P"}="H1C1 F2P";
$resn_exch{"2H1C F2P"}="H1C2 F2P"; $resn_exch{"1H6C F2P"}="H6C1 F2P"; $resn_exch{"2H6C F2P"}="H6C2 F2P";
$resn_exch{"1H2C F32"}="H2C1 F32"; $resn_exch{"2H2C F32"}="H2C2 F32"; $resn_exch{"1H21 F32"}="H211 F32";
$resn_exch{"2H21 F32"}="H212 F32"; $resn_exch{"1H24 F3G"}="H241 F3G"; $resn_exch{"2H24 F3G"}="H242 F3G";
$resn_exch{"1H5  F3G"}=" H51 F3G"; $resn_exch{"2H5  F3G"}=" H52 F3G"; $resn_exch{"3H5  F3G"}=" H53 F3G";
$resn_exch{"1H4  F3G"}=" H41 F3G"; $resn_exch{"2H4  F3G"}=" H42 F3G"; $resn_exch{"3H4  F3G"}=" H43 F3G";
$resn_exch{"1H10 F3G"}="H101 F3G"; $resn_exch{"2H10 F3G"}="H102 F3G"; $resn_exch{"3H10 F3G"}="H103 F3G";
$resn_exch{"1H31 F3G"}="H311 F3G"; $resn_exch{"2H31 F3G"}="H312 F3G"; $resn_exch{"3H31 F3G"}="H313 F3G";
$resn_exch{"1H15 F3G"}="H151 F3G"; $resn_exch{"2H15 F3G"}="H152 F3G"; $resn_exch{"1H1  F3P"}=" H11 F3P";
$resn_exch{"2H1  F3P"}=" H12 F3P"; $resn_exch{"3H1  F3P"}=" H13 F3P"; $resn_exch{"1H4  F3P"}=" H41 F3P";
$resn_exch{"2H4  F3P"}=" H42 F3P"; $resn_exch{"1H5  F3P"}=" H51 F3P"; $resn_exch{"2H5  F3P"}=" H52 F3P";
$resn_exch{"1H6  F3P"}=" H61 F3P"; $resn_exch{"2H6  F3P"}=" H62 F3P"; $resn_exch{"1H8  F3P"}=" H81 F3P";
$resn_exch{"2H8  F3P"}=" H82 F3P"; $resn_exch{"3H8  F3P"}=" H83 F3P"; $resn_exch{"1H9  F3P"}=" H91 F3P";
$resn_exch{"2H9  F3P"}=" H92 F3P"; $resn_exch{"3H9  F3P"}=" H93 F3P"; $resn_exch{"1H10 F3P"}="H101 F3P";
$resn_exch{"2H10 F3P"}="H102 F3P"; $resn_exch{"3H10 F3P"}="H103 F3P"; $resn_exch{"1HM8 F42"}="HM81 F42";
$resn_exch{"1H1* F42"}="H1'1 F42"; $resn_exch{"2H1* F42"}="H1'2 F42"; $resn_exch{" H2* F42"}=" H2' F42";
$resn_exch{"*HO2 F42"}="HO2' F42"; $resn_exch{" H3* F42"}=" H3' F42"; $resn_exch{"*HO3 F42"}="HO3' F42";
$resn_exch{" H4* F42"}=" H4' F42"; $resn_exch{"*HO4 F42"}="HO4' F42"; $resn_exch{"1H5* F42"}="H5'1 F42";
$resn_exch{"2H5* F42"}="H5'2 F42"; $resn_exch{"2HOP F42"}="HOP2 F42"; $resn_exch{"1H1I F42"}="H1I1 F42";
$resn_exch{"1H2I F42"}="H2I1 F42"; $resn_exch{"2H2I F42"}="H2I2 F42"; $resn_exch{"3H2I F42"}="H2I3 F42";
$resn_exch{"1HNH F42"}="HNH1 F42"; $resn_exch{"1H1H F42"}="H1H1 F42"; $resn_exch{"1H3H F42"}="H3H1 F42";
$resn_exch{"2H3H F42"}="H3H2 F42"; $resn_exch{"1H4H F42"}="H4H1 F42"; $resn_exch{"2H4H F42"}="H4H2 F42";
$resn_exch{"1HNG F42"}="HNG1 F42"; $resn_exch{"1H1G F42"}="H1G1 F42"; $resn_exch{"1H3G F42"}="H3G1 F42";
$resn_exch{"2H3G F42"}="H3G2 F42"; $resn_exch{"1H4G F42"}="H4G1 F42"; $resn_exch{"2H4G F42"}="H4G2 F42";
$resn_exch{"1HHA F43"}="HHA1 F43"; $resn_exch{"2HHA F43"}="HHA2 F43"; $resn_exch{"1H5A F43"}="H5A1 F43";
$resn_exch{"2H5A F43"}="H5A2 F43"; $resn_exch{"1HN8 F43"}="HN81 F43"; $resn_exch{"2HN8 F43"}="HN82 F43";
$resn_exch{"1H9A F43"}="H9A1 F43"; $resn_exch{"2H9A F43"}="H9A2 F43"; $resn_exch{"3H9A F43"}="H9A3 F43";
$resn_exch{"1HAA F43"}="HAA1 F43"; $resn_exch{"2HAA F43"}="HAA2 F43"; $resn_exch{"1HBA F43"}="HBA1 F43";
$resn_exch{"2HBA F43"}="HBA2 F43"; $resn_exch{"1HHB F43"}="HHB1 F43"; $resn_exch{"2HHB F43"}="HHB2 F43";
$resn_exch{"1H8B F43"}="H8B1 F43"; $resn_exch{"2H8B F43"}="H8B2 F43"; $resn_exch{"1H9B F43"}="H9B1 F43";
$resn_exch{"2H9B F43"}="H9B2 F43"; $resn_exch{"3H9B F43"}="H9B3 F43"; $resn_exch{"1HAB F43"}="HAB1 F43";
$resn_exch{"2HAB F43"}="HAB2 F43"; $resn_exch{"1HBB F43"}="HBB1 F43"; $resn_exch{"2HBB F43"}="HBB2 F43";
$resn_exch{"1H5C F43"}="H5C1 F43"; $resn_exch{"2H5C F43"}="H5C2 F43"; $resn_exch{"1H8C F43"}="H8C1 F43";
$resn_exch{"2H8C F43"}="H8C2 F43"; $resn_exch{"1H9C F43"}="H9C1 F43"; $resn_exch{"2H9C F43"}="H9C2 F43";
$resn_exch{"1H5D F43"}="H5D1 F43"; $resn_exch{"2H5D F43"}="H5D2 F43"; $resn_exch{"1H6D F43"}="H6D1 F43";
$resn_exch{"2H6D F43"}="H6D2 F43"; $resn_exch{"1H9D F43"}="H9D1 F43"; $resn_exch{"2H9D F43"}="H9D2 F43";
$resn_exch{"1HH3 F50"}="HH31 F50"; $resn_exch{"2HH3 F50"}="HH32 F50"; $resn_exch{"3HH3 F50"}="HH33 F50";
$resn_exch{"CH01 F6B"}="H01C F6B"; $resn_exch{"CH02 F6B"}="H02C F6B"; $resn_exch{"CH04 F6B"}="H04C F6B";
$resn_exch{"CH05 F6B"}="H05C F6B"; $resn_exch{"NH09 F6B"}="H09N F6B"; $resn_exch{"1H10 F6B"}="H101 F6B";
$resn_exch{"2H10 F6B"}="H102 F6B"; $resn_exch{"1HNP F6B"}="HNP1 F6B"; $resn_exch{"2HNP F6B"}="HNP2 F6B";
$resn_exch{"CH17 F6B"}="H17C F6B"; $resn_exch{"CH18 F6B"}="H18C F6B"; $resn_exch{"CH19 F6B"}="H19C F6B";
$resn_exch{"1H15 F6F"}="H151 F6F"; $resn_exch{"2H15 F6F"}="H152 F6F"; $resn_exch{"1H16 F6F"}="H161 F6F";
$resn_exch{"2H16 F6F"}="H162 F6F"; $resn_exch{"1H1  F6P"}=" H11 F6P"; $resn_exch{"2H1  F6P"}=" H12 F6P";
$resn_exch{"1H6  F6P"}=" H61 F6P"; $resn_exch{"2H6  F6P"}=" H62 F6P"; $resn_exch{"2HOP F6P"}="HOP2 F6P";
$resn_exch{"3HOP F6P"}="HOP3 F6P"; $resn_exch{"1H1C F6R"}="H1C1 F6R"; $resn_exch{"2H1C F6R"}="H1C2 F6R";
$resn_exch{"1H6C F6R"}="H6C1 F6R"; $resn_exch{"2H6C F6R"}="H6C2 F6R"; $resn_exch{"1H1  F77"}=" H11 F77";
$resn_exch{"2H1  F77"}=" H12 F77"; $resn_exch{"3H1  F77"}=" H13 F77"; $resn_exch{"1H2  F77"}="H21A F77";
$resn_exch{"2H2  F77"}="H22A F77"; $resn_exch{"1H9  F77"}=" H91 F77"; $resn_exch{"2H9  F77"}=" H92 F77";
$resn_exch{"3H9  F77"}=" H93 F77"; $resn_exch{"1H7  F79"}=" H71 F79"; $resn_exch{"2H7  F79"}=" H72 F79";
$resn_exch{"1H8  F79"}=" H81 F79"; $resn_exch{"2H8  F79"}=" H82 F79"; $resn_exch{"3H8  F79"}=" H83 F79";
$resn_exch{"1H10 F79"}="H101 F79"; $resn_exch{"2H10 F79"}="H102 F79"; $resn_exch{"1H11 F79"}="H111 F79";
$resn_exch{"2H11 F79"}="H112 F79"; $resn_exch{"1H12 F79"}="H121 F79"; $resn_exch{"2H12 F79"}="H122 F79";
$resn_exch{"1H1  F83"}=" H11 F83"; $resn_exch{"2H1  F83"}="H12A F83"; $resn_exch{"1H3  F83"}=" H31 F83";
$resn_exch{"2H3  F83"}=" H32 F83"; $resn_exch{"1H10 F83"}="H101 F83"; $resn_exch{"2H10 F83"}="H102 F83";
$resn_exch{"1H3M F89"}="H3M1 F89"; $resn_exch{"2H3M F89"}="H3M2 F89"; $resn_exch{"3H3M F89"}="H3M3 F89";
$resn_exch{"1H11 F89"}="H111 F89"; $resn_exch{"2H11 F89"}="H112 F89"; $resn_exch{"2HN1 F89"}="HN12 F89";
$resn_exch{"1H19 F89"}="H191 F89"; $resn_exch{"2H19 F89"}="H192 F89"; $resn_exch{"1HB  F89"}=" HB1 F89";
$resn_exch{"2HB  F89"}=" HB2 F89"; $resn_exch{"1HG  F89"}=" HG1 F89"; $resn_exch{"2HG  F89"}=" HG2 F89";
$resn_exch{"2HOE F89"}="HOE2 F89"; $resn_exch{"1H14 F9F"}="H141 F9F"; $resn_exch{"2H14 F9F"}="H142 F9F";
$resn_exch{"1H15 F9F"}="H151 F9F"; $resn_exch{"2H15 F9F"}="H152 F9F"; $resn_exch{"1H9   FA"}=" H91  FA";
$resn_exch{"2H9   FA"}=" H92  FA"; $resn_exch{"1HB1  FA"}="HB11  FA"; $resn_exch{"2HB1  FA"}="HB12  FA";
$resn_exch{"1HG1  FA"}="HG11  FA"; $resn_exch{"2HG1  FA"}="HG12  FA"; $resn_exch{"1HA2  FA"}="HA21  FA";
$resn_exch{"2HA2  FA"}="HA22  FA"; $resn_exch{"1HOE  FA"}="HOE1  FA"; $resn_exch{"1H2C FA1"}="H2C1 FA1";
$resn_exch{"2H2C FA1"}="H2C2 FA1"; $resn_exch{"2HOP FA2"}="HOP2 FA2"; $resn_exch{" H2* FA2"}=" H2' FA2";
$resn_exch{"1H4* FA2"}="H4'1 FA2"; $resn_exch{"2H4* FA2"}="H4'2 FA2"; $resn_exch{" H1* FA2"}=" H1' FA2";
$resn_exch{" H3* FA2"}=" H3' FA2"; $resn_exch{"2HO* FA2"}="HO2' FA2"; $resn_exch{"3HOP FA2"}="HOP3 FA2";
$resn_exch{"1H2C FA3"}="H2C1 FA3"; $resn_exch{"2H2C FA3"}="H2C2 FA3"; $resn_exch{"1H7  FA4"}=" H71 FA4";
$resn_exch{"2H7  FA4"}=" H72 FA4"; $resn_exch{"1H8  FA4"}=" H81 FA4"; $resn_exch{"2H8  FA4"}=" H82 FA4";
$resn_exch{"1H9  FA4"}=" H91 FA4"; $resn_exch{"2H9  FA4"}=" H92 FA4"; $resn_exch{"1H10 FA4"}="H101 FA4";
$resn_exch{"2H10 FA4"}="H102 FA4"; $resn_exch{"1H11 FA4"}="H111 FA4"; $resn_exch{"2H11 FA4"}="H112 FA4";
$resn_exch{"1H12 FA4"}="H121 FA4"; $resn_exch{"2H12 FA4"}="H122 FA4"; $resn_exch{"1H13 FA4"}="H131 FA4";
$resn_exch{"2H13 FA4"}="H132 FA4"; $resn_exch{"1H14 FA4"}="H141 FA4"; $resn_exch{"2H14 FA4"}="H142 FA4";
$resn_exch{"1H15 FA4"}="H151 FA4"; $resn_exch{"2H15 FA4"}="H152 FA4"; $resn_exch{"1H20 FA4"}="H201 FA4";
$resn_exch{"2H20 FA4"}="H202 FA4"; $resn_exch{"1H23 FA4"}="H231 FA4"; $resn_exch{"2H23 FA4"}="H232 FA4";
$resn_exch{"1H24 FA4"}="H241 FA4"; $resn_exch{"2H24 FA4"}="H242 FA4"; $resn_exch{"1H25 FA4"}="H251 FA4";
$resn_exch{"2H25 FA4"}="H252 FA4"; $resn_exch{"1H26 FA4"}="H261 FA4"; $resn_exch{"2H26 FA4"}="H262 FA4";
$resn_exch{"1H27 FA4"}="H271 FA4"; $resn_exch{"2H27 FA4"}="H272 FA4"; $resn_exch{"1H30 FA4"}="H301 FA4";
$resn_exch{"2H30 FA4"}="H302 FA4"; $resn_exch{"1HN  FA5"}=" HN1 FA5"; $resn_exch{"2HN  FA5"}=" HN2 FA5";
$resn_exch{"1HB  FA5"}=" HB1 FA5"; $resn_exch{"2HB  FA5"}=" HB2 FA5"; $resn_exch{"2HOP FA5"}="HOP2 FA5";
$resn_exch{"1H5* FA5"}=" H5' FA5"; $resn_exch{"2H5* FA5"}="H5'' FA5"; $resn_exch{" H4* FA5"}=" H4' FA5";
$resn_exch{" H3* FA5"}=" H3' FA5"; $resn_exch{"3HO* FA5"}="HO'3 FA5"; $resn_exch{" H2* FA5"}=" H2' FA5";
$resn_exch{"2HO* FA5"}="HO2' FA5"; $resn_exch{" H1* FA5"}=" H1' FA5"; $resn_exch{"1H6  FA5"}=" H61 FA5";
$resn_exch{"2H6  FA5"}=" H62 FA5"; $resn_exch{"1H2C FA6"}="H2C1 FA6"; $resn_exch{"2H2C FA6"}="H2C2 FA6";
$resn_exch{"1H6C FA6"}="H6C1 FA6"; $resn_exch{"2H6C FA6"}="H6C2 FA6"; $resn_exch{"1HB' FA9"}="HB1' FA9";
$resn_exch{"2HB' FA9"}="HB2' FA9"; $resn_exch{"1H7M FA9"}="H7M1 FA9"; $resn_exch{"2H7M FA9"}="H7M2 FA9";
$resn_exch{"3H7M FA9"}="H7M3 FA9"; $resn_exch{"1H8M FA9"}="H8M1 FA9"; $resn_exch{"2H8M FA9"}="H8M2 FA9";
$resn_exch{"3H8M FA9"}="H8M3 FA9"; $resn_exch{"1H1* FA9"}="H11' FA9"; $resn_exch{"2H1* FA9"}="H12' FA9";
$resn_exch{" H2* FA9"}=" H2' FA9"; $resn_exch{" H3* FA9"}=" H3' FA9"; $resn_exch{" H4* FA9"}=" H4' FA9";
$resn_exch{"1H5* FA9"}="H51' FA9"; $resn_exch{"2H5* FA9"}="H52' FA9"; $resn_exch{"1HA' FA9"}="HA1' FA9";
$resn_exch{"2HA' FA9"}="HA2' FA9"; $resn_exch{"1HAA FA9"}="HAA1 FA9"; $resn_exch{"2HAA FA9"}="HAA2 FA9";
$resn_exch{"2HOA FAA"}="HOA2 FAA"; $resn_exch{"AH51 FAA"}="H51A FAA"; $resn_exch{"AH52 FAA"}="H52A FAA";
$resn_exch{"AH4* FAA"}=" H4B FAA"; $resn_exch{"AH3* FAA"}=" H3B FAA"; $resn_exch{"AHO3 FAA"}="HO3A FAA";
$resn_exch{"AH2* FAA"}=" H2B FAA"; $resn_exch{"AHO2 FAA"}="HO2A FAA"; $resn_exch{"AH1* FAA"}=" H1B FAA";
$resn_exch{" AH8 FAA"}=" H8A FAA"; $resn_exch{"AH61 FAA"}="H61A FAA"; $resn_exch{"AH62 FAA"}="H62A FAA";
$resn_exch{" AH2 FAA"}=" H2A FAA"; $resn_exch{"1HM7 FAA"}="HM71 FAA"; $resn_exch{"2HM7 FAA"}="HM72 FAA";
$resn_exch{"3HM7 FAA"}="HM73 FAA"; $resn_exch{"1HM8 FAA"}="HM81 FAA"; $resn_exch{"2HM8 FAA"}="HM82 FAA";
$resn_exch{"3HM8 FAA"}="HM83 FAA"; $resn_exch{"1H1* FAA"}="H1'1 FAA"; $resn_exch{"2H1* FAA"}="H1'2 FAA";
$resn_exch{" H2* FAA"}=" H2' FAA"; $resn_exch{"*HO2 FAA"}="HO2' FAA"; $resn_exch{" H3* FAA"}=" H3' FAA";
$resn_exch{"*HO3 FAA"}="HO3' FAA"; $resn_exch{" H4* FAA"}=" H4' FAA"; $resn_exch{"*HO4 FAA"}="HO4' FAA";
$resn_exch{"1H5* FAA"}="H5'1 FAA"; $resn_exch{"2H5* FAA"}="H5'2 FAA"; $resn_exch{"1H7P FAA"}="H7P1 FAA";
$resn_exch{"2H7P FAA"}="H7P2 FAA"; $resn_exch{"4HOP FAA"}="HOP4 FAA"; $resn_exch{"2HOA FAB"}="HOA2 FAB";
$resn_exch{"AH51 FAB"}="H51A FAB"; $resn_exch{"AH52 FAB"}="H52A FAB"; $resn_exch{"AH4* FAB"}=" H4B FAB";
$resn_exch{"AH3* FAB"}=" H3B FAB"; $resn_exch{"AHO3 FAB"}="HO3A FAB"; $resn_exch{"AH2* FAB"}=" H2B FAB";
$resn_exch{"AHO2 FAB"}="HO2A FAB"; $resn_exch{"AH1* FAB"}=" H1B FAB"; $resn_exch{" AH8 FAB"}=" H8A FAB";
$resn_exch{"AH61 FAB"}="H61A FAB"; $resn_exch{"AH62 FAB"}="H62A FAB"; $resn_exch{" AH2 FAB"}=" H2A FAB";
$resn_exch{"1HM7 FAB"}="HM71 FAB"; $resn_exch{"2HM7 FAB"}="HM72 FAB"; $resn_exch{"3HM7 FAB"}="HM73 FAB";
$resn_exch{"1HM8 FAB"}="HM81 FAB"; $resn_exch{"2HM8 FAB"}="HM82 FAB"; $resn_exch{"3HM8 FAB"}="HM83 FAB";
$resn_exch{"1H1* FAB"}="H1'1 FAB"; $resn_exch{"2H1* FAB"}="H1'2 FAB"; $resn_exch{" H2* FAB"}=" H2' FAB";
$resn_exch{"*HO2 FAB"}="HO2' FAB"; $resn_exch{" H3* FAB"}=" H3D FAB"; $resn_exch{"*HO3 FAB"}="HO3' FAB";
$resn_exch{" H4* FAB"}=" H4' FAB"; $resn_exch{"*HO4 FAB"}="HO4' FAB"; $resn_exch{"1H5* FAB"}="H5'1 FAB";
$resn_exch{"2H5* FAB"}="H5'2 FAB"; $resn_exch{"1H4' FAB"}="H4'1 FAB"; $resn_exch{"2H4' FAB"}="H4'2 FAB";
$resn_exch{"3H4' FAB"}="H4'3 FAB"; $resn_exch{"1H5' FAB"}="H51X FAB"; $resn_exch{"2H5' FAB"}="H52X FAB";
$resn_exch{"3H5' FAB"}="H5'3 FAB"; $resn_exch{"2HOA FAD"}="HOA2 FAD"; $resn_exch{"AH51 FAD"}="H51A FAD";
$resn_exch{"AH52 FAD"}="H52A FAD"; $resn_exch{"AH4* FAD"}=" H4B FAD"; $resn_exch{"AH3* FAD"}=" H3B FAD";
$resn_exch{"AHO3 FAD"}="HO3A FAD"; $resn_exch{"AH2* FAD"}=" H2B FAD"; $resn_exch{"AHO2 FAD"}="HO2A FAD";
$resn_exch{"AH1* FAD"}=" H1B FAD"; $resn_exch{" AH8 FAD"}=" H8A FAD"; $resn_exch{"AH61 FAD"}="H61A FAD";
$resn_exch{"AH62 FAD"}="H62A FAD"; $resn_exch{" AH2 FAD"}=" H2A FAD"; $resn_exch{"1HM7 FAD"}="HM71 FAD";
$resn_exch{"2HM7 FAD"}="HM72 FAD"; $resn_exch{"3HM7 FAD"}="HM73 FAD"; $resn_exch{"1HM8 FAD"}="HM81 FAD";
$resn_exch{"2HM8 FAD"}="HM82 FAD"; $resn_exch{"3HM8 FAD"}="HM83 FAD"; $resn_exch{"1H1* FAD"}="H1'1 FAD";
$resn_exch{"2H1* FAD"}="H1'2 FAD"; $resn_exch{" H2* FAD"}=" H2' FAD"; $resn_exch{"*HO2 FAD"}="HO2' FAD";
$resn_exch{" H3* FAD"}=" H3' FAD"; $resn_exch{"*HO3 FAD"}="HO3' FAD"; $resn_exch{" H4* FAD"}=" H4' FAD";
$resn_exch{"*HO4 FAD"}="HO4' FAD"; $resn_exch{"1H5* FAD"}="H5'1 FAD"; $resn_exch{"2H5* FAD"}="H5'2 FAD";
$resn_exch{"2HOP FAD"}="HOP2 FAD"; $resn_exch{"2HOA FAE"}="HOA2 FAE"; $resn_exch{"AH51 FAE"}="H51A FAE";
$resn_exch{"AH52 FAE"}="H52A FAE"; $resn_exch{"AH4* FAE"}=" H4B FAE"; $resn_exch{"AH3* FAE"}=" H3B FAE";
$resn_exch{"AHO3 FAE"}="HO3A FAE"; $resn_exch{"AH2* FAE"}=" H2B FAE"; $resn_exch{"AHO2 FAE"}="HO2A FAE";
$resn_exch{"AH1* FAE"}=" H1B FAE"; $resn_exch{" AH8 FAE"}=" H8A FAE"; $resn_exch{" AH7 FAE"}=" H7A FAE";
$resn_exch{"AH61 FAE"}="H61A FAE"; $resn_exch{"AH62 FAE"}="H62A FAE"; $resn_exch{" AH2 FAE"}=" H2A FAE";
$resn_exch{"1HM7 FAE"}="HM71 FAE"; $resn_exch{"2HM7 FAE"}="HM72 FAE"; $resn_exch{"3HM7 FAE"}="HM73 FAE";
$resn_exch{"1HM8 FAE"}="HM81 FAE"; $resn_exch{"2HM8 FAE"}="HM82 FAE"; $resn_exch{"3HM8 FAE"}="HM83 FAE";
$resn_exch{"1H1* FAE"}="H1'1 FAE"; $resn_exch{"2H1* FAE"}="H1'2 FAE"; $resn_exch{" H2* FAE"}=" H2' FAE";
$resn_exch{"*HO2 FAE"}="HO2' FAE"; $resn_exch{" H3* FAE"}=" H3' FAE"; $resn_exch{"*HO3 FAE"}="HO3' FAE";
$resn_exch{" H4* FAE"}=" H4' FAE"; $resn_exch{"*HO4 FAE"}="HO4' FAE"; $resn_exch{"1H5* FAE"}="H5'1 FAE";
$resn_exch{"2H5* FAE"}="H5'2 FAE"; $resn_exch{"2HOP FAE"}="HOP2 FAE"; $resn_exch{"1HAX FAF"}="HAX1 FAF";
$resn_exch{"2HAX FAF"}="HAX2 FAF"; $resn_exch{"3HAX FAF"}="HAX3 FAF"; $resn_exch{"1HBL FAF"}="HBL1 FAF";
$resn_exch{"2HBL FAF"}="HBL2 FAF"; $resn_exch{"1HDL FAF"}="HDL1 FAF"; $resn_exch{"2HDL FAF"}="HDL2 FAF";
$resn_exch{"3HDL FAF"}="HDL3 FAF"; $resn_exch{"1HDM FAF"}="HDM1 FAF"; $resn_exch{"2HDM FAF"}="HDM2 FAF";
$resn_exch{"3HDM FAF"}="HDM3 FAF"; $resn_exch{"2H3  FAG"}=" H32 FAG"; $resn_exch{"1H2A FAG"}="H2A1 FAG";
$resn_exch{"2H2A FAG"}="H2A2 FAG"; $resn_exch{"2HOP FAG"}="HOP2 FAG"; $resn_exch{"1H5* FAG"}="H5'1 FAG";
$resn_exch{"2H5* FAG"}="H5'2 FAG"; $resn_exch{" H4* FAG"}=" H4' FAG"; $resn_exch{" H1* FAG"}=" H1' FAG";
$resn_exch{"1HN2 FAG"}="HN21 FAG"; $resn_exch{"2HN2 FAG"}="HN22 FAG"; $resn_exch{"1H2* FAG"}="H2'1 FAG";
$resn_exch{"2H2* FAG"}="H2'2 FAG"; $resn_exch{" H3* FAG"}=" H3' FAG"; $resn_exch{"*HO3 FAG"}="HO3' FAG";
$resn_exch{"1HM  FAG"}=" HM1 FAG"; $resn_exch{"2HM  FAG"}=" HM2 FAG"; $resn_exch{"3HM  FAG"}=" HM3 FAG";
$resn_exch{"3HOP FAG"}="HOP3 FAG"; $resn_exch{"1H4  FAI"}=" H41 FAI"; $resn_exch{"2H4  FAI"}=" H42 FAI";
$resn_exch{"1HN2 FAI"}="HN21 FAI"; $resn_exch{"2HN2 FAI"}="HN22 FAI"; $resn_exch{"1H22 FAL"}="H221 FAL";
$resn_exch{"2H22 FAL"}="H222 FAL"; $resn_exch{"1H24 FAL"}="H241 FAL"; $resn_exch{"2H24 FAL"}="H242 FAL";
$resn_exch{"1H26 FAL"}="H261 FAL"; $resn_exch{"2H26 FAL"}="H262 FAL"; $resn_exch{"3H26 FAL"}="H263 FAL";
$resn_exch{"1H27 FAL"}="H271 FAL"; $resn_exch{"2H27 FAL"}="H272 FAL"; $resn_exch{"3H27 FAL"}="H273 FAL";
$resn_exch{" AH2 FAM"}=" H2A FAM"; $resn_exch{"AH61 FAM"}="H61A FAM"; $resn_exch{"AH62 FAM"}="H62A FAM";
$resn_exch{" AH8 FAM"}=" H8A FAM"; $resn_exch{"AH1* FAM"}=" H1B FAM"; $resn_exch{"AH2* FAM"}=" H2B FAM";
$resn_exch{"AHO2 FAM"}="HO2A FAM"; $resn_exch{"AH3* FAM"}=" H3B FAM"; $resn_exch{"8HOA FAM"}="HOA8 FAM";
$resn_exch{"9HOA FAM"}="HOA9 FAM"; $resn_exch{"AH4* FAM"}=" H4B FAM"; $resn_exch{"AH51 FAM"}="H51A FAM";
$resn_exch{"AH52 FAM"}="H52A FAM"; $resn_exch{"2HOA FAM"}="HOA2 FAM"; $resn_exch{"5HOA FAM"}="HOA5 FAM";
$resn_exch{"1H12 FAM"}="H121 FAM"; $resn_exch{"2H12 FAM"}="H122 FAM"; $resn_exch{"1H13 FAM"}="H131 FAM";
$resn_exch{"2H13 FAM"}="H132 FAM"; $resn_exch{"3H13 FAM"}="H133 FAM"; $resn_exch{"1H14 FAM"}="H141 FAM";
$resn_exch{"2H14 FAM"}="H142 FAM"; $resn_exch{"3H14 FAM"}="H143 FAM"; $resn_exch{"1H7  FAM"}=" H71 FAM";
$resn_exch{"2H7  FAM"}=" H72 FAM"; $resn_exch{"1H6  FAM"}=" H61 FAM"; $resn_exch{"2H6  FAM"}=" H62 FAM";
$resn_exch{"1H3  FAM"}=" H31 FAM"; $resn_exch{"2H3  FAM"}=" H32 FAM"; $resn_exch{"1H2  FAM"}=" H21 FAM";
$resn_exch{"2H2  FAM"}=" H22 FAM"; $resn_exch{"1HN2 FAM"}="HN21 FAM"; $resn_exch{"2HN2 FAM"}="HN22 FAM";
$resn_exch{"1HC' FAO"}="HC1' FAO"; $resn_exch{"2HC' FAO"}="HC2' FAO"; $resn_exch{"1H6A FAO"}="H6A1 FAO";
$resn_exch{"2H6A FAO"}="H6A2 FAO"; $resn_exch{"1H7M FAO"}="H7M1 FAO"; $resn_exch{"2H7M FAO"}="H7M2 FAO";
$resn_exch{"3H7M FAO"}="H7M3 FAO"; $resn_exch{"1H8M FAO"}="H8M1 FAO"; $resn_exch{"2H8M FAO"}="H8M2 FAO";
$resn_exch{"3H8M FAO"}="H8M3 FAO"; $resn_exch{"1H1' FAO"}="H11' FAO"; $resn_exch{"2H1' FAO"}="H12' FAO";
$resn_exch{"1H5' FAO"}="H51' FAO"; $resn_exch{"2H5' FAO"}="H52' FAO"; $resn_exch{"1H24 FAP"}="H241 FAP";
$resn_exch{"2H24 FAP"}="H242 FAP"; $resn_exch{"1H26 FAP"}="H261 FAP"; $resn_exch{"2H26 FAP"}="H262 FAP";
$resn_exch{"1H28 FAP"}="H281 FAP"; $resn_exch{"2H28 FAP"}="H282 FAP"; $resn_exch{"3H28 FAP"}="H283 FAP";
$resn_exch{"1H29 FAP"}="H291 FAP"; $resn_exch{"2H29 FAP"}="H292 FAP"; $resn_exch{"3H29 FAP"}="H293 FAP";
$resn_exch{"1H1  FAR"}=" H11 FAR"; $resn_exch{"2H1  FAR"}="H12A FAR"; $resn_exch{"3H1  FAR"}=" H13 FAR";
$resn_exch{"1H5  FAR"}=" H51 FAR"; $resn_exch{"2H5  FAR"}=" H52 FAR"; $resn_exch{"1H6  FAR"}=" H61 FAR";
$resn_exch{"2H6  FAR"}=" H62 FAR"; $resn_exch{"1H9  FAR"}=" H91 FAR"; $resn_exch{"2H9  FAR"}=" H92 FAR";
$resn_exch{"1H11 FAR"}="H111 FAR"; $resn_exch{"2H11 FAR"}="H112 FAR"; $resn_exch{"1H14 FAR"}="H141 FAR";
$resn_exch{"2H14 FAR"}="H142 FAR"; $resn_exch{"3H14 FAR"}="H143 FAR"; $resn_exch{"1H15 FAR"}="H151 FAR";
$resn_exch{"2H15 FAR"}="H152 FAR"; $resn_exch{"3H15 FAR"}="H153 FAR"; $resn_exch{"1H4  FAR"}=" H41 FAR";
$resn_exch{"2H4  FAR"}=" H42 FAR"; $resn_exch{"3H4  FAR"}=" H43 FAR"; $resn_exch{"1H10 FAR"}="H101 FAR";
$resn_exch{"2H10 FAR"}="H102 FAR"; $resn_exch{"3H10 FAR"}="H103 FAR"; $resn_exch{"2HOA FAS"}="HOA2 FAS";
$resn_exch{"AH51 FAS"}="H51A FAS"; $resn_exch{"AH52 FAS"}="H52A FAS"; $resn_exch{"AH4* FAS"}=" H4B FAS";
$resn_exch{"AH3* FAS"}=" H3B FAS"; $resn_exch{"AHO3 FAS"}="HO3A FAS"; $resn_exch{"AH2* FAS"}=" H2B FAS";
$resn_exch{"AHO2 FAS"}="HO2A FAS"; $resn_exch{"AH1* FAS"}=" H1B FAS"; $resn_exch{" AH8 FAS"}=" H8A FAS";
$resn_exch{"AH61 FAS"}="H61A FAS"; $resn_exch{"AH62 FAS"}="H62A FAS"; $resn_exch{" AH2 FAS"}=" H2A FAS";
$resn_exch{"1HM7 FAS"}="HM71 FAS"; $resn_exch{"2HM7 FAS"}="HM72 FAS"; $resn_exch{"3HM7 FAS"}="HM73 FAS";
$resn_exch{"1HM8 FAS"}="HM81 FAS"; $resn_exch{"2HM8 FAS"}="HM82 FAS"; $resn_exch{"3HM8 FAS"}="HM83 FAS";
$resn_exch{"1H1* FAS"}="H1'1 FAS"; $resn_exch{"2H1* FAS"}="H1'2 FAS"; $resn_exch{" H2* FAS"}=" H2' FAS";
$resn_exch{"*HO2 FAS"}="HO2' FAS"; $resn_exch{" H3* FAS"}=" H3' FAS"; $resn_exch{"*HO3 FAS"}="HO3' FAS";
$resn_exch{" H4* FAS"}=" H4' FAS"; $resn_exch{"*HO4 FAS"}="HO4' FAS"; $resn_exch{"1H5* FAS"}="H5'1 FAS";
$resn_exch{"2H5* FAS"}="H5'2 FAS"; $resn_exch{"2HOP FAS"}="HOP2 FAS"; $resn_exch{"1H2  FAT"}=" H21 FAT";
$resn_exch{"2H2  FAT"}=" H22 FAT"; $resn_exch{"1H3  FAT"}=" H31 FAT"; $resn_exch{"2H3  FAT"}=" H32 FAT";
$resn_exch{"1H4  FAT"}=" H41 FAT"; $resn_exch{"2H4  FAT"}=" H42 FAT"; $resn_exch{"1H5  FAT"}=" H51 FAT";
$resn_exch{"2H5  FAT"}=" H52 FAT"; $resn_exch{"1H6  FAT"}=" H61 FAT"; $resn_exch{"2H6  FAT"}=" H62 FAT";
$resn_exch{"1H7  FAT"}=" H71 FAT"; $resn_exch{"2H7  FAT"}=" H72 FAT"; $resn_exch{"1H8  FAT"}=" H81 FAT";
$resn_exch{"2H8  FAT"}=" H82 FAT"; $resn_exch{"1H9  FAT"}=" H91 FAT"; $resn_exch{"2H9  FAT"}=" H92 FAT";
$resn_exch{"1H10 FAT"}="H101 FAT"; $resn_exch{"2H10 FAT"}="H102 FAT"; $resn_exch{"1H11 FAT"}="H111 FAT";
$resn_exch{"2H11 FAT"}="H112 FAT"; $resn_exch{"1H12 FAT"}="H121 FAT"; $resn_exch{"2H12 FAT"}="H122 FAT";
$resn_exch{"1H13 FAT"}="H131 FAT"; $resn_exch{"2H13 FAT"}="H132 FAT"; $resn_exch{"1H14 FAT"}="H141 FAT";
$resn_exch{"2H14 FAT"}="H142 FAT"; $resn_exch{"1H15 FAT"}="H151 FAT"; $resn_exch{"2H15 FAT"}="H152 FAT";
$resn_exch{"1H16 FAT"}="H161 FAT"; $resn_exch{"2H16 FAT"}="H162 FAT"; $resn_exch{"3H16 FAT"}="H163 FAT";
$resn_exch{"1H4   FB"}=" H41  FB"; $resn_exch{"2H4   FB"}=" H42  FB"; $resn_exch{"1H3   FB"}=" H31  FB";
$resn_exch{"2H3   FB"}=" H32  FB"; $resn_exch{"1HB   FB"}=" HB1  FB"; $resn_exch{"2HB   FB"}=" HB2  FB";
$resn_exch{"3HB   FB"}=" HB3  FB"; $resn_exch{"1HA   FB"}=" HA1  FB"; $resn_exch{"2HA   FB"}=" HA2  FB";
$resn_exch{"1H   FBA"}=" H1  FBA"; $resn_exch{"2H   FBA"}=" H2A FBA"; $resn_exch{"1HN  FBA"}=" HN1 FBA";
$resn_exch{"2HN  FBA"}=" HN2 FBA"; $resn_exch{"3HN  FBA"}=" HN3 FBA"; $resn_exch{"1H1  FBE"}=" H11 FBE";
$resn_exch{"2H1  FBE"}=" H12 FBE"; $resn_exch{"1H4  FBE"}=" H41 FBE"; $resn_exch{"2H4  FBE"}=" H42 FBE";
$resn_exch{"1HN  FBE"}=" HN1 FBE"; $resn_exch{"2HN  FBE"}=" HN2 FBE"; $resn_exch{"1H12 FBI"}="H121 FBI";
$resn_exch{"2H12 FBI"}="H122 FBI"; $resn_exch{"3H12 FBI"}="H123 FBI"; $resn_exch{"1H13 FBI"}="H131 FBI";
$resn_exch{"2H13 FBI"}="H132 FBI"; $resn_exch{"3H13 FBI"}="H133 FBI"; $resn_exch{"1H2  FBI"}=" H21 FBI";
$resn_exch{"2H2  FBI"}=" H22 FBI"; $resn_exch{"1H4  FBI"}=" H41 FBI"; $resn_exch{"2H4  FBI"}=" H42 FBI";
$resn_exch{"1H6  FBI"}=" H61 FBI"; $resn_exch{"2H6  FBI"}=" H62 FBI"; $resn_exch{"1H7  FBI"}=" H71 FBI";
$resn_exch{"2H7  FBI"}=" H72 FBI"; $resn_exch{"1H92 FBI"}="H921 FBI"; $resn_exch{"2H92 FBI"}="H922 FBI";
$resn_exch{"3H92 FBI"}="H923 FBI"; $resn_exch{"1H93 FBI"}="H931 FBI"; $resn_exch{"2H93 FBI"}="H932 FBI";
$resn_exch{"3H93 FBI"}="H933 FBI"; $resn_exch{"1H22 FBL"}="H221 FBL"; $resn_exch{"2H22 FBL"}="H222 FBL";
$resn_exch{"1H24 FBL"}="H241 FBL"; $resn_exch{"2H24 FBL"}="H242 FBL"; $resn_exch{"1H26 FBL"}="H261 FBL";
$resn_exch{"2H26 FBL"}="H262 FBL"; $resn_exch{"3H26 FBL"}="H263 FBL"; $resn_exch{"1H27 FBL"}="H271 FBL";
$resn_exch{"2H27 FBL"}="H272 FBL"; $resn_exch{"3H27 FBL"}="H273 FBL"; $resn_exch{"2HOP FBP"}="HOP2 FBP";
$resn_exch{"3HOP FBP"}="HOP3 FBP"; $resn_exch{"1H1  FBP"}=" H11 FBP"; $resn_exch{"2H1  FBP"}=" H12 FBP";
$resn_exch{"1H6  FBP"}=" H61 FBP"; $resn_exch{"2H6  FBP"}=" H62 FBP"; $resn_exch{"5HOP FBP"}="HOP5 FBP";
$resn_exch{"6HOP FBP"}="HOP6 FBP"; $resn_exch{"8HC  FBQ"}=" HC8 FBQ"; $resn_exch{"9HC  FBQ"}=" HC9 FBQ";
$resn_exch{"1HC0 FBQ"}="HC01 FBQ"; $resn_exch{"1H11 FBQ"}="H111 FBQ"; $resn_exch{"2H11 FBQ"}="H112 FBQ";
$resn_exch{"1H12 FBQ"}="H121 FBQ"; $resn_exch{"2H12 FBQ"}="H122 FBQ"; $resn_exch{"1H14 FBQ"}="H141 FBQ";
$resn_exch{"2H14 FBQ"}="H142 FBQ"; $resn_exch{"1HC6 FBQ"}="HC61 FBQ"; $resn_exch{"1HC8 FBQ"}="HC81 FBQ";
$resn_exch{"1HC9 FBQ"}="HC91 FBQ"; $resn_exch{"2HC0 FBQ"}="HC02 FBQ"; $resn_exch{"2HO6 FBQ"}="HO62 FBQ";
$resn_exch{"1H28 FBQ"}="H281 FBQ"; $resn_exch{"2H28 FBQ"}="H282 FBQ"; $resn_exch{"3H28 FBQ"}="H283 FBQ";
$resn_exch{"OH6S FBQ"}="H6SO FBQ"; $resn_exch{"1H2  FBR"}=" H21 FBR"; $resn_exch{"2H2  FBR"}=" H22 FBR";
$resn_exch{"1H3  FBR"}=" H31 FBR"; $resn_exch{"2H3  FBR"}=" H32 FBR"; $resn_exch{"1H16 FBR"}="H161 FBR";
$resn_exch{"2H16 FBR"}="H162 FBR"; $resn_exch{"1H17 FBR"}="H171 FBR"; $resn_exch{"2H17 FBR"}="H172 FBR";
$resn_exch{"1H18 FBR"}="H181 FBR"; $resn_exch{"2H18 FBR"}="H182 FBR"; $resn_exch{"1H19 FBR"}="H191 FBR";
$resn_exch{"2H19 FBR"}="H192 FBR"; $resn_exch{"1H20 FBR"}="H201 FBR"; $resn_exch{"2H20 FBR"}="H202 FBR";
$resn_exch{"3H20 FBR"}="H203 FBR"; $resn_exch{"1HN0 FBS"}="HN01 FBS"; $resn_exch{"2HN0 FBS"}="HN02 FBS";
$resn_exch{"1HN0 FBT"}="HN01 FBT"; $resn_exch{"2HN0 FBT"}="HN02 FBT"; $resn_exch{"1HN0 FBU"}="HN01 FBU";
$resn_exch{"2HN0 FBU"}="HN02 FBU"; $resn_exch{"1H17 FC1"}="H171 FC1"; $resn_exch{"2H17 FC1"}="H172 FC1";
$resn_exch{"3H17 FC1"}="H173 FC1"; $resn_exch{"1H18 FC1"}="H181 FC1"; $resn_exch{"2H18 FC1"}="H182 FC1";
$resn_exch{"3H18 FC1"}="H183 FC1"; $resn_exch{"1H11 FC1"}="H111 FC1"; $resn_exch{"2H11 FC1"}="H112 FC1";
$resn_exch{"1H14 FC1"}="H141 FC1"; $resn_exch{"2H14 FC1"}="H142 FC1"; $resn_exch{"1H15 FC1"}="H151 FC1";
$resn_exch{"2H15 FC1"}="H152 FC1"; $resn_exch{"1H6  FCA"}=" H61 FCA"; $resn_exch{"2H6  FCA"}=" H62 FCA";
$resn_exch{"3H6  FCA"}=" H63 FCA"; $resn_exch{"1H6  FCB"}=" H61 FCB"; $resn_exch{"2H6  FCB"}=" H62 FCB";
$resn_exch{"3H6  FCB"}=" H63 FCB"; $resn_exch{"1H1  FCG"}=" H11 FCG"; $resn_exch{"2H1  FCG"}=" H12 FCG";
$resn_exch{" H1* FCG"}=" H1' FCG"; $resn_exch{"1HN6 FCG"}="HN61 FCG"; $resn_exch{"2HN6 FCG"}="HN62 FCG";
$resn_exch{"H11  FCG"}="H111 FCG"; $resn_exch{"H12  FCG"}="H122 FCG"; $resn_exch{"1H5* FCG"}="H5'1 FCG";
$resn_exch{"2H5* FCG"}="H5'2 FCG"; $resn_exch{" H4* FCG"}=" H4' FCG"; $resn_exch{" H3* FCG"}=" H3' FCG";
$resn_exch{" H2* FCG"}=" H2' FCG"; $resn_exch{"1H1* FCG"}="H1'1 FCG"; $resn_exch{"2H1* FCG"}="H1'2 FCG";
$resn_exch{"1H7M FCG"}="H7M1 FCG"; $resn_exch{"2H7M FCG"}="H7M2 FCG"; $resn_exch{"3H7M FCG"}="H7M3 FCG";
$resn_exch{"1H11 FCG"}="H141 FCG"; $resn_exch{"2H11 FCG"}="H112 FCG"; $resn_exch{"1H12 FCG"}="H121 FCG";
$resn_exch{"2H12 FCG"}="H142 FCG"; $resn_exch{"1H8M FCG"}="H8M1 FCG"; $resn_exch{"2H8M FCG"}="H8M2 FCG";
$resn_exch{"3H8M FCG"}="H8M3 FCG"; $resn_exch{"1H14 FCI"}="H141 FCI"; $resn_exch{"2H14 FCI"}="H142 FCI";
$resn_exch{"1H17 FCI"}="H171 FCI"; $resn_exch{"2H17 FCI"}="H172 FCI"; $resn_exch{"1H18 FCI"}="H181 FCI";
$resn_exch{"2H18 FCI"}="H182 FCI"; $resn_exch{"1H21 FCI"}="H211 FCI"; $resn_exch{"2H21 FCI"}="H212 FCI";
$resn_exch{"1H24 FCI"}="H241 FCI"; $resn_exch{"2H24 FCI"}="H242 FCI"; $resn_exch{"1H25 FCI"}="H251 FCI";
$resn_exch{"2H25 FCI"}="H252 FCI"; $resn_exch{"1H27 FCI"}="H271 FCI"; $resn_exch{"2H27 FCI"}="H272 FCI";
$resn_exch{"1H29 FCI"}="H291 FCI"; $resn_exch{"2H29 FCI"}="H292 FCI"; $resn_exch{"1H30 FCI"}="H301 FCI";
$resn_exch{"2H30 FCI"}="H302 FCI"; $resn_exch{"1H32 FCI"}="H321 FCI"; $resn_exch{"2H32 FCI"}="H322 FCI";
$resn_exch{"1H34 FCI"}="H341 FCI"; $resn_exch{"2H34 FCI"}="H342 FCI"; $resn_exch{"1H35 FCI"}="H351 FCI";
$resn_exch{"2H35 FCI"}="H352 FCI"; $resn_exch{"3H35 FCI"}="H353 FCI"; $resn_exch{"1H36 FCI"}="H361 FCI";
$resn_exch{"2H36 FCI"}="H362 FCI"; $resn_exch{"3H36 FCI"}="H363 FCI"; $resn_exch{"1H37 FCI"}="H371 FCI";
$resn_exch{"2H37 FCI"}="H372 FCI"; $resn_exch{"3H37 FCI"}="H373 FCI"; $resn_exch{"1H38 FCI"}="H381 FCI";
$resn_exch{"2H38 FCI"}="H382 FCI"; $resn_exch{"1HN  FCL"}=" HN1 FCL"; $resn_exch{"2HN  FCL"}=" HN2 FCL";
$resn_exch{"1HBC FCL"}="HBC1 FCL"; $resn_exch{"2HBC FCL"}="HBC2 FCL"; $resn_exch{"1H3  FCN"}=" H31 FCN";
$resn_exch{"2H3  FCN"}=" H32 FCN"; $resn_exch{"3H3  FCN"}=" H33 FCN"; $resn_exch{"2HOP FCN"}="HOP2 FCN";
$resn_exch{"3HOP FCN"}="HOP3 FCN"; $resn_exch{"1HA  FCP"}=" HA1 FCP"; $resn_exch{"2HB  FCP"}=" HB2 FCP";
$resn_exch{"1HB  FCP"}=" HB1 FCP"; $resn_exch{"2HC  FCP"}=" HC2 FCP"; $resn_exch{"1HC  FCP"}=" HC1 FCP";
$resn_exch{"2HD  FCP"}=" HD2 FCP"; $resn_exch{"3HE  FCP"}=" HE3 FCP"; $resn_exch{"1HD  FCP"}=" HD1 FCP";
$resn_exch{"2HE  FCP"}=" HE2 FCP"; $resn_exch{"3HF  FCP"}=" HF3 FCP"; $resn_exch{"1H6A FCT"}="H6A1 FCT";
$resn_exch{"2H6A FCT"}="H6A2 FCT"; $resn_exch{"3HOA FCT"}="HOA3 FCT"; $resn_exch{"6HOA FCT"}="HOA6 FCT";
$resn_exch{"1H6B FCT"}="H6B1 FCT"; $resn_exch{"2H6B FCT"}="H6B2 FCT"; $resn_exch{"2HOB FCT"}="HOB2 FCT";
$resn_exch{"3HOB FCT"}="HOB3 FCT"; $resn_exch{"6HOB FCT"}="HOB6 FCT"; $resn_exch{"1H6C FCT"}="H6C1 FCT";
$resn_exch{"2H6C FCT"}="H6C2 FCT"; $resn_exch{"2HOC FCT"}="HOC2 FCT"; $resn_exch{"3HOC FCT"}="HOC3 FCT";
$resn_exch{"4HOC FCT"}="HOC4 FCT"; $resn_exch{"6HOC FCT"}="HOC6 FCT"; $resn_exch{" AH2 FCX"}=" H2A FCX";
$resn_exch{"AH61 FCX"}="H61A FCX"; $resn_exch{"AH62 FCX"}="H62A FCX"; $resn_exch{" AH8 FCX"}=" H8A FCX";
$resn_exch{"AH1* FCX"}=" H1B FCX"; $resn_exch{"AH2* FCX"}=" H2B FCX"; $resn_exch{"AHO2 FCX"}="HO2A FCX";
$resn_exch{"AH3* FCX"}=" H3B FCX"; $resn_exch{"8HOA FCX"}="HOA8 FCX"; $resn_exch{"9HOA FCX"}="HOA9 FCX";
$resn_exch{"AH4* FCX"}=" H4B FCX"; $resn_exch{"AH51 FCX"}="H51A FCX"; $resn_exch{"AH52 FCX"}="H52A FCX";
$resn_exch{"2HOA FCX"}="HOA2 FCX"; $resn_exch{"5HOA FCX"}="HOA5 FCX"; $resn_exch{"1H12 FCX"}="H121 FCX";
$resn_exch{"2H12 FCX"}="H122 FCX"; $resn_exch{"1H13 FCX"}="H131 FCX"; $resn_exch{"2H13 FCX"}="H132 FCX";
$resn_exch{"3H13 FCX"}="H133 FCX"; $resn_exch{"1H14 FCX"}="H141 FCX"; $resn_exch{"2H14 FCX"}="H142 FCX";
$resn_exch{"3H14 FCX"}="H143 FCX"; $resn_exch{"1H7  FCX"}=" H71 FCX"; $resn_exch{"2H7  FCX"}=" H72 FCX";
$resn_exch{"1H6  FCX"}=" H61 FCX"; $resn_exch{"2H6  FCX"}=" H62 FCX"; $resn_exch{"1H3  FCX"}=" H31 FCX";
$resn_exch{"2H3  FCX"}=" H32 FCX"; $resn_exch{"1H2  FCX"}=" H21 FCX"; $resn_exch{"2H2  FCX"}=" H22 FCX";
$resn_exch{"1HB  FCY"}=" HB1 FCY"; $resn_exch{"2HB  FCY"}=" HB2 FCY"; $resn_exch{"1H66 FD1"}="H661 FD1";
$resn_exch{"1H64 FD1"}="H641 FD1"; $resn_exch{"1H63 FD1"}="H631 FD1"; $resn_exch{"1H62 FD1"}="H621 FD1";
$resn_exch{"1H43 FD1"}="H431 FD1"; $resn_exch{"2H43 FD1"}="H432 FD1"; $resn_exch{"1H54 FD1"}="H541 FD1";
$resn_exch{"1H34 FD1"}="H341 FD1"; $resn_exch{"2H34 FD1"}="H342 FD1"; $resn_exch{"1H31 FD1"}="H311 FD1";
$resn_exch{"1H71 FD1"}="H711 FD1"; $resn_exch{"1H72 FD1"}="H721 FD1"; $resn_exch{"2H72 FD1"}="H722 FD1";
$resn_exch{"1H74 FD1"}="H741 FD1"; $resn_exch{"2H74 FD1"}="H742 FD1"; $resn_exch{"1H75 FD1"}="H751 FD1";
$resn_exch{"2H75 FD1"}="H752 FD1"; $resn_exch{"1H40 FD1"}="H401 FD1"; $resn_exch{"2H40 FD1"}="H402 FD1";
$resn_exch{"1H92 FD1"}="H921 FD1"; $resn_exch{"1H29 FD1"}="H291 FD1"; $resn_exch{"1H3  FD1"}=" H31 FD1";
$resn_exch{"1H4  FD1"}=" H41 FD1"; $resn_exch{"1H1  FD1"}=" H11 FD1"; $resn_exch{"1H5  FD1"}=" H51 FD1";
$resn_exch{"1H6  FD1"}=" H61 FD1"; $resn_exch{"1H7  FD1"}=" H71 FD1"; $resn_exch{"1H8  FD1"}=" H81 FD1";
$resn_exch{"1H66 FD2"}="H661 FD2"; $resn_exch{"1H64 FD2"}="H641 FD2"; $resn_exch{"1H63 FD2"}="H631 FD2";
$resn_exch{"1H62 FD2"}="H621 FD2"; $resn_exch{"1H43 FD2"}="H431 FD2"; $resn_exch{"2H43 FD2"}="H432 FD2";
$resn_exch{"1H54 FD2"}="H541 FD2"; $resn_exch{"1H34 FD2"}="H341 FD2"; $resn_exch{"2H34 FD2"}="H342 FD2";
$resn_exch{"1H31 FD2"}="H311 FD2"; $resn_exch{"1H71 FD2"}="H711 FD2"; $resn_exch{"2H71 FD2"}="H712 FD2";
$resn_exch{"1H72 FD2"}="H721 FD2"; $resn_exch{"2H72 FD2"}="H722 FD2"; $resn_exch{"1H74 FD2"}="H741 FD2";
$resn_exch{"2H74 FD2"}="H742 FD2"; $resn_exch{"1H75 FD2"}="H751 FD2"; $resn_exch{"2H75 FD2"}="H752 FD2";
$resn_exch{"1H40 FD2"}="H401 FD2"; $resn_exch{"1H99 FD2"}="H991 FD2"; $resn_exch{"2H99 FD2"}="H992 FD2";
$resn_exch{"3H99 FD2"}="H993 FD2"; $resn_exch{"1H29 FD2"}="H291 FD2"; $resn_exch{"1H3  FD2"}=" H31 FD2";
$resn_exch{"1H4  FD2"}=" H41 FD2"; $resn_exch{"1H1  FD2"}=" H11 FD2"; $resn_exch{"1H5  FD2"}=" H51 FD2";
$resn_exch{"1H6  FD2"}=" H61 FD2"; $resn_exch{"1H7  FD2"}=" H71 FD2"; $resn_exch{"1H8  FD2"}=" H81 FD2";
$resn_exch{"1H66 FD3"}="H661 FD3"; $resn_exch{"1H64 FD3"}="H641 FD3"; $resn_exch{"1H63 FD3"}="H631 FD3";
$resn_exch{"1H62 FD3"}="H621 FD3"; $resn_exch{"1H43 FD3"}="H431 FD3"; $resn_exch{"2H43 FD3"}="H432 FD3";
$resn_exch{"1H54 FD3"}="H541 FD3"; $resn_exch{"1H34 FD3"}="H341 FD3"; $resn_exch{"2H34 FD3"}="H342 FD3";
$resn_exch{"1H31 FD3"}="H311 FD3"; $resn_exch{"1H71 FD3"}="H711 FD3"; $resn_exch{"2H71 FD3"}="H712 FD3";
$resn_exch{"1H72 FD3"}="H721 FD3"; $resn_exch{"2H72 FD3"}="H722 FD3"; $resn_exch{"1H74 FD3"}="H741 FD3";
$resn_exch{"2H74 FD3"}="H742 FD3"; $resn_exch{"1H75 FD3"}="H751 FD3"; $resn_exch{"2H75 FD3"}="H752 FD3";
$resn_exch{"1H40 FD3"}="H401 FD3"; $resn_exch{"1H29 FD3"}="H291 FD3"; $resn_exch{"1H3  FD3"}=" H31 FD3";
$resn_exch{"1H4  FD3"}=" H41 FD3"; $resn_exch{"1H1  FD3"}=" H11 FD3"; $resn_exch{"1H5  FD3"}=" H51 FD3";
$resn_exch{"1H6  FD3"}=" H61 FD3"; $resn_exch{"1H7  FD3"}=" H71 FD3"; $resn_exch{"1H8  FD3"}=" H81 FD3";
$resn_exch{"1H66 FD4"}="H661 FD4"; $resn_exch{"1H64 FD4"}="H641 FD4"; $resn_exch{"1H63 FD4"}="H631 FD4";
$resn_exch{"1H62 FD4"}="H621 FD4"; $resn_exch{"1H43 FD4"}="H431 FD4"; $resn_exch{"2H43 FD4"}="H432 FD4";
$resn_exch{"1H54 FD4"}="H541 FD4"; $resn_exch{"1H34 FD4"}="H341 FD4"; $resn_exch{"2H34 FD4"}="H342 FD4";
$resn_exch{"1H31 FD4"}="H311 FD4"; $resn_exch{"1H71 FD4"}="H711 FD4"; $resn_exch{"2H71 FD4"}="H712 FD4";
$resn_exch{"1H72 FD4"}="H721 FD4"; $resn_exch{"2H72 FD4"}="H722 FD4"; $resn_exch{"1H74 FD4"}="H741 FD4";
$resn_exch{"2H74 FD4"}="H742 FD4"; $resn_exch{"1H75 FD4"}="H751 FD4"; $resn_exch{"2H75 FD4"}="H752 FD4";
$resn_exch{"1H29 FD4"}="H291 FD4"; $resn_exch{"1H3  FD4"}=" H31 FD4"; $resn_exch{"1H4  FD4"}=" H41 FD4";
$resn_exch{"1H1  FD4"}=" H11 FD4"; $resn_exch{"1H5  FD4"}=" H51 FD4"; $resn_exch{"1H6  FD4"}=" H61 FD4";
$resn_exch{"1H7  FD4"}=" H71 FD4"; $resn_exch{"1H8  FD4"}=" H81 FD4"; $resn_exch{"1H55 FD4"}="H551 FD4";
$resn_exch{"2H55 FD4"}="H552 FD4"; $resn_exch{"3H55 FD4"}="H553 FD4"; $resn_exch{"2HOA FDA"}="HOA2 FDA";
$resn_exch{"AH51 FDA"}="H51A FDA"; $resn_exch{"AH52 FDA"}="H52A FDA"; $resn_exch{"AH4* FDA"}=" H4B FDA";
$resn_exch{"AH3* FDA"}=" H3B FDA"; $resn_exch{"AHO3 FDA"}="HO3A FDA"; $resn_exch{"AH2* FDA"}=" H2B FDA";
$resn_exch{"AHO2 FDA"}="HO2A FDA"; $resn_exch{"AH1* FDA"}=" H1B FDA"; $resn_exch{" AH8 FDA"}=" H8A FDA";
$resn_exch{"AH61 FDA"}="H61A FDA"; $resn_exch{"AH62 FDA"}="H62A FDA"; $resn_exch{" AH2 FDA"}=" H2A FDA";
$resn_exch{"1HM7 FDA"}="HM71 FDA"; $resn_exch{"2HM7 FDA"}="HM72 FDA"; $resn_exch{"3HM7 FDA"}="HM73 FDA";
$resn_exch{"1HM8 FDA"}="HM81 FDA"; $resn_exch{"2HM8 FDA"}="HM82 FDA"; $resn_exch{"3HM8 FDA"}="HM83 FDA";
$resn_exch{"1H1* FDA"}="H1'1 FDA"; $resn_exch{"2H1* FDA"}="H1'2 FDA"; $resn_exch{" H2* FDA"}=" H2' FDA";
$resn_exch{"*HO2 FDA"}="HO2' FDA"; $resn_exch{" H3* FDA"}=" H3' FDA"; $resn_exch{"*HO3 FDA"}="HO3' FDA";
$resn_exch{" H4* FDA"}=" H4' FDA"; $resn_exch{"*HO4 FDA"}="HO4' FDA"; $resn_exch{"1H5* FDA"}="H5'1 FDA";
$resn_exch{"2H5* FDA"}="H5'2 FDA"; $resn_exch{"2HOP FDA"}="HOP2 FDA"; $resn_exch{"1H2  FDC"}=" H21 FDC";
$resn_exch{"2H2  FDC"}=" H22 FDC"; $resn_exch{"1H4  FDC"}=" H41 FDC"; $resn_exch{"2H4  FDC"}=" H42 FDC";
$resn_exch{"1H8  FDC"}=" H81 FDC"; $resn_exch{"2H8  FDC"}=" H82 FDC"; $resn_exch{"OH10 FDC"}="H10O FDC";
$resn_exch{"OH11 FDC"}="H11O FDC"; $resn_exch{"1H10 FDC"}="H101 FDC"; $resn_exch{"2H10 FDC"}="H102 FDC";
$resn_exch{"1HBD FDD"}="HBD1 FDD"; $resn_exch{"2HBD FDD"}="HBD2 FDD"; $resn_exch{"1HAD FDD"}="HAD1 FDD";
$resn_exch{"2HAD FDD"}="HAD2 FDD"; $resn_exch{"1HMD FDD"}="HMD1 FDD"; $resn_exch{"2HMD FDD"}="HMD2 FDD";
$resn_exch{"3HMD FDD"}="HMD3 FDD"; $resn_exch{"1HMF FDD"}="HMF1 FDD"; $resn_exch{"2HMF FDD"}="HMF2 FDD";
$resn_exch{"3HMF FDD"}="HMF3 FDD"; $resn_exch{"1HMC FDD"}="HMC1 FDD"; $resn_exch{"2HMC FDD"}="HMC2 FDD";
$resn_exch{"3HMC FDD"}="HMC3 FDD"; $resn_exch{"1HME FDD"}="HME1 FDD"; $resn_exch{"2HME FDD"}="HME2 FDD";
$resn_exch{"3HME FDD"}="HME3 FDD"; $resn_exch{"1HMB FDD"}="HMB1 FDD"; $resn_exch{"2HMB FDD"}="HMB2 FDD";
$resn_exch{"3HMB FDD"}="HMB3 FDD"; $resn_exch{"1HMA FDD"}="HMA1 FDD"; $resn_exch{"2HMA FDD"}="HMA2 FDD";
$resn_exch{"3HMA FDD"}="HMA3 FDD"; $resn_exch{"1HAA FDD"}="HAA1 FDD"; $resn_exch{"2HAA FDD"}="HAA2 FDD";
$resn_exch{"1HBA FDD"}="HBA1 FDD"; $resn_exch{"2HBA FDD"}="HBA2 FDD"; $resn_exch{"1HAA FDE"}="HAA1 FDE";
$resn_exch{"2HAA FDE"}="HAA2 FDE"; $resn_exch{"1HBA FDE"}="HBA1 FDE"; $resn_exch{"2HBA FDE"}="HBA2 FDE";
$resn_exch{"1HMA FDE"}="HMA1 FDE"; $resn_exch{"2HMA FDE"}="HMA2 FDE"; $resn_exch{"3HMA FDE"}="HMA3 FDE";
$resn_exch{"1HME FDE"}="HME1 FDE"; $resn_exch{"2HME FDE"}="HME2 FDE"; $resn_exch{"3HME FDE"}="HME3 FDE";
$resn_exch{"1HMF FDE"}="HMF1 FDE"; $resn_exch{"2HMF FDE"}="HMF2 FDE"; $resn_exch{"3HMF FDE"}="HMF3 FDE";
$resn_exch{"1HMD FDE"}="HMD1 FDE"; $resn_exch{"2HMD FDE"}="HMD2 FDE"; $resn_exch{"3HMD FDE"}="HMD3 FDE";
$resn_exch{"1HAD FDE"}="HAD1 FDE"; $resn_exch{"2HAD FDE"}="HAD2 FDE"; $resn_exch{"1HBD FDE"}="HBD1 FDE";
$resn_exch{"2HBD FDE"}="HBD2 FDE"; $resn_exch{"1H17 FDI"}="H171 FDI"; $resn_exch{"2H17 FDI"}="H172 FDI";
$resn_exch{"1H18 FDI"}="H181 FDI"; $resn_exch{"2H18 FDI"}="H182 FDI"; $resn_exch{"3H18 FDI"}="H183 FDI";
$resn_exch{"1H24 FDI"}="H241 FDI"; $resn_exch{"2H24 FDI"}="H242 FDI"; $resn_exch{"1H26 FDI"}="H261 FDI";
$resn_exch{"2H26 FDI"}="H262 FDI"; $resn_exch{"3H26 FDI"}="H263 FDI"; $resn_exch{"1H34 FDI"}="H341 FDI";
$resn_exch{"2H34 FDI"}="H342 FDI"; $resn_exch{"3H34 FDI"}="H343 FDI"; $resn_exch{"1HC6 FDM"}="HC61 FDM";
$resn_exch{"1HN3 FDM"}="HN31 FDM"; $resn_exch{"1HC7 FDM"}="HC71 FDM"; $resn_exch{"2HC7 FDM"}="HC72 FDM";
$resn_exch{"3HC7 FDM"}="HC73 FDM"; $resn_exch{"1H1* FDM"}="H1'1 FDM"; $resn_exch{"1H2* FDM"}="H2'1 FDM";
$resn_exch{"2H2* FDM"}="H2'2 FDM"; $resn_exch{"1H3* FDM"}="H3'1 FDM"; $resn_exch{"1H4* FDM"}="H4'1 FDM";
$resn_exch{"1H5* FDM"}="H5'1 FDM"; $resn_exch{"2H5* FDM"}="H5'2 FDM"; $resn_exch{"1HP3 FDM"}="HP31 FDM";
$resn_exch{"1HP2 FDM"}="HP21 FDM"; $resn_exch{"1H7  FDN"}=" H71 FDN"; $resn_exch{"2H7  FDN"}=" H72 FDN";
$resn_exch{"3H7  FDN"}=" H73 FDN"; $resn_exch{"2HOP FDP"}="HOP2 FDP"; $resn_exch{"3HOP FDP"}="HOP3 FDP";
$resn_exch{"1H1  FDP"}=" H11 FDP"; $resn_exch{"2H1  FDP"}=" H12 FDP"; $resn_exch{"1H6  FDP"}=" H61 FDP";
$resn_exch{"2H6  FDP"}=" H62 FDP"; $resn_exch{"5HOP FDP"}="HOP5 FDP"; $resn_exch{"6HOP FDP"}="HOP6 FDP";
$resn_exch{"2HOP FDQ"}="HOP2 FDQ"; $resn_exch{"3HOP FDQ"}="HOP3 FDQ"; $resn_exch{"1H21 FDQ"}="H211 FDQ";
$resn_exch{"2H21 FDQ"}="H212 FDQ"; $resn_exch{"1H6  FDQ"}=" H61 FDQ"; $resn_exch{"2H6  FDQ"}=" H62 FDQ";
$resn_exch{"5HOP FDQ"}="HOP5 FDQ"; $resn_exch{"6HOP FDQ"}="HOP6 FDQ"; $resn_exch{"1H19 FDT"}="H191 FDT";
$resn_exch{"2H19 FDT"}="H192 FDT"; $resn_exch{"3H19 FDT"}="H193 FDT"; $resn_exch{"1HAW FE1"}="HAW1 FE1";
$resn_exch{"2HAW FE1"}="HAW2 FE1"; $resn_exch{"1HAX FE1"}="HAX1 FE1"; $resn_exch{"2HAX FE1"}="HAX2 FE1";
$resn_exch{"1HBA FE1"}="HBA1 FE1"; $resn_exch{"2HBA FE1"}="HBA2 FE1"; $resn_exch{"1HBB FE1"}="HBB1 FE1";
$resn_exch{"2HBB FE1"}="HBB2 FE1"; $resn_exch{"1HBF FE1"}="HBF1 FE1"; $resn_exch{"2HBF FE1"}="HBF2 FE1";
$resn_exch{"3HBF FE1"}="HBF3 FE1"; $resn_exch{"1HAB FE1"}="HAB1 FE1"; $resn_exch{"2HAB FE1"}="HAB2 FE1";
$resn_exch{"3HAB FE1"}="HAB3 FE1"; $resn_exch{"1H9  FE1"}=" H91 FE1"; $resn_exch{"2H9  FE1"}=" H92 FE1";
$resn_exch{"1HN4 FE1"}="HN41 FE1"; $resn_exch{"2HN4 FE1"}="HN42 FE1"; $resn_exch{"1HN2 FE1"}="HN21 FE1";
$resn_exch{"2HN2 FE1"}="HN22 FE1"; $resn_exch{"1HBA FEC"}="HBA1 FEC"; $resn_exch{"2HBA FEC"}="HBA2 FEC";
$resn_exch{"1HAA FEC"}="HAA1 FEC"; $resn_exch{"2HAA FEC"}="HAA2 FEC"; $resn_exch{"1HMB FEC"}="HMB1 FEC";
$resn_exch{"2HMB FEC"}="HMB2 FEC"; $resn_exch{"3HMB FEC"}="HMB3 FEC"; $resn_exch{"1HAB FEC"}="HAB1 FEC";
$resn_exch{"2HAB FEC"}="HAB2 FEC"; $resn_exch{"1HBB FEC"}="HBB1 FEC"; $resn_exch{"2HBB FEC"}="HBB2 FEC";
$resn_exch{"1HAC FEC"}="HAC1 FEC"; $resn_exch{"2HAC FEC"}="HAC2 FEC"; $resn_exch{"1HBC FEC"}="HBC1 FEC";
$resn_exch{"2HBC FEC"}="HBC2 FEC"; $resn_exch{"1HMC FEC"}="HMC1 FEC"; $resn_exch{"2HMC FEC"}="HMC2 FEC";
$resn_exch{"3HMC FEC"}="HMC3 FEC"; $resn_exch{"1HAD FEC"}="HAD1 FEC"; $resn_exch{"2HAD FEC"}="HAD2 FEC";
$resn_exch{"1HBD FEC"}="HBD1 FEC"; $resn_exch{"2HBD FEC"}="HBD2 FEC"; $resn_exch{"1HMD FEC"}="HMD1 FEC";
$resn_exch{"2HMD FEC"}="HMD2 FEC"; $resn_exch{"3HMD FEC"}="HMD3 FEC"; $resn_exch{"1HMA FEC"}="HMA1 FEC";
$resn_exch{"2HMA FEC"}="HMA2 FEC"; $resn_exch{"3HMA FEC"}="HMA3 FEC"; $resn_exch{"1HO1 FEL"}="HO11 FEL";
$resn_exch{"2HO1 FEL"}="HO12 FEL"; $resn_exch{"1HO2 FEL"}="HO21 FEL"; $resn_exch{"2HO2 FEL"}="HO22 FEL";
$resn_exch{"1HO3 FEL"}="HO31 FEL"; $resn_exch{"2HO3 FEL"}="HO32 FEL"; $resn_exch{"1H11 FEM"}="H111 FEM";
$resn_exch{"2H11 FEM"}="H112 FEM"; $resn_exch{"1H12 FEM"}="H121 FEM"; $resn_exch{"2H12 FEM"}="H122 FEM";
$resn_exch{"1H15 FEM"}="H151 FEM"; $resn_exch{"2H15 FEM"}="H152 FEM"; $resn_exch{"1H16 FEM"}="H161 FEM";
$resn_exch{"2H16 FEM"}="H162 FEM"; $resn_exch{"1H2  FEN"}=" H21 FEN"; $resn_exch{"2H2  FEN"}=" H22 FEN";
$resn_exch{"1H3  FEN"}=" H31 FEN"; $resn_exch{"2H3  FEN"}=" H32 FEN"; $resn_exch{"1H4  FEN"}=" H41 FEN";
$resn_exch{"2H4  FEN"}=" H42 FEN"; $resn_exch{"1H16 FEN"}="H161 FEN"; $resn_exch{"2H16 FEN"}="H162 FEN";
$resn_exch{"3H16 FEN"}="H163 FEN"; $resn_exch{"1H17 FEN"}="H171 FEN"; $resn_exch{"2H17 FEN"}="H172 FEN";
$resn_exch{"3H17 FEN"}="H173 FEN"; $resn_exch{"1H18 FEN"}="H181 FEN"; $resn_exch{"2H18 FEN"}="H182 FEN";
$resn_exch{"3H18 FEN"}="H183 FEN"; $resn_exch{"1H19 FEN"}="H191 FEN"; $resn_exch{"2H19 FEN"}="H192 FEN";
$resn_exch{"3H19 FEN"}="H193 FEN"; $resn_exch{"1H20 FEN"}="H201 FEN"; $resn_exch{"2H20 FEN"}="H202 FEN";
$resn_exch{"3H20 FEN"}="H203 FEN"; $resn_exch{"'HO4 FEN"}="HO4' FEN"; $resn_exch{"1HC1 FEP"}="HC11 FEP";
$resn_exch{"2HC1 FEP"}="HC12 FEP"; $resn_exch{"1HC2 FEP"}="HC21 FEP"; $resn_exch{"2HC2 FEP"}="HC22 FEP";
$resn_exch{"1HC3 FEP"}="HC31 FEP"; $resn_exch{"2HC3 FEP"}="HC32 FEP"; $resn_exch{"1HC4 FEP"}="HC41 FEP";
$resn_exch{"2HC4 FEP"}="HC42 FEP"; $resn_exch{"1H10 FER"}="H101 FER"; $resn_exch{"2H10 FER"}="H102 FER";
$resn_exch{"3H10 FER"}="H103 FER"; $resn_exch{"1H26 FEX"}="H261 FEX"; $resn_exch{"2H26 FEX"}="H262 FEX";
$resn_exch{"3H26 FEX"}="H263 FEX"; $resn_exch{"1H24 FEX"}="H241 FEX"; $resn_exch{"2H24 FEX"}="H242 FEX";
$resn_exch{"1H23 FEX"}="H231 FEX"; $resn_exch{"2H23 FEX"}="H232 FEX"; $resn_exch{"1H15 FEX"}="H151 FEX";
$resn_exch{"2H15 FEX"}="H152 FEX"; $resn_exch{"1H2  FEX"}=" H21 FEX"; $resn_exch{"2H2  FEX"}="H22A FEX";
$resn_exch{"3H2  FEX"}=" H23 FEX"; $resn_exch{"1H3  FEX"}="H31A FEX"; $resn_exch{"2H3  FEX"}="H32A FEX";
$resn_exch{"3H3  FEX"}=" H33 FEX"; $resn_exch{"1HN4 FF3"}="HN41 FF3"; $resn_exch{"2HN4 FF3"}="HN42 FF3";
$resn_exch{"1HN3 FF3"}="HN31 FF3"; $resn_exch{"2HN3 FF3"}="HN32 FF3"; $resn_exch{"1H1  FFA"}=" H11 FFA";
$resn_exch{"2H1  FFA"}=" H12 FFA"; $resn_exch{"1H2  FFA"}=" H21 FFA"; $resn_exch{"2H2  FFA"}=" H22 FFA";
$resn_exch{"1H9  FFA"}=" H91 FFA"; $resn_exch{"2H9  FFA"}=" H92 FFA"; $resn_exch{"1H10 FFA"}="H101 FFA";
$resn_exch{"2H10 FFA"}="H102 FFA"; $resn_exch{"1H11 FFA"}="H111 FFA"; $resn_exch{"2H11 FFA"}="H112 FFA";
$resn_exch{"1H12 FFA"}="H121 FFA"; $resn_exch{"2H12 FFA"}="H122 FFA"; $resn_exch{"1H15 FFA"}="H151 FFA";
$resn_exch{"2H15 FFA"}="H152 FFA"; $resn_exch{"1H16 FFA"}="H161 FFA"; $resn_exch{"2H16 FFA"}="H162 FFA";
$resn_exch{"1H18 FFA"}="H181 FFA"; $resn_exch{"2H18 FFA"}="H182 FFA"; $resn_exch{"3H18 FFA"}="H183 FFA";
$resn_exch{"1H19 FFA"}="H191 FFA"; $resn_exch{"2H19 FFA"}="H192 FFA"; $resn_exch{"3H19 FFA"}="H193 FFA";
$resn_exch{"CH01 FFB"}="H01C FFB"; $resn_exch{"CH02 FFB"}="H02C FFB"; $resn_exch{"CH04 FFB"}="H04C FFB";
$resn_exch{"CH05 FFB"}="H05C FFB"; $resn_exch{"NH09 FFB"}="H09N FFB"; $resn_exch{"1H10 FFB"}="H101 FFB";
$resn_exch{"2H10 FFB"}="H102 FFB"; $resn_exch{"1HNP FFB"}="HNP1 FFB"; $resn_exch{"2HNP FFB"}="HNP2 FFB";
$resn_exch{"1H1A FFC"}="H1A1 FFC"; $resn_exch{"2H1A FFC"}="H1A2 FFC"; $resn_exch{"1H6A FFC"}="H6A1 FFC";
$resn_exch{"2H6A FFC"}="H6A2 FFC"; $resn_exch{"AHO3 FFC"}="HO3A FFC"; $resn_exch{"AHO6 FFC"}="HO6A FFC";
$resn_exch{"1H6B FFC"}="H6B1 FFC"; $resn_exch{"2H6B FFC"}="H6B2 FFC"; $resn_exch{"BHO2 FFC"}="HO2B FFC";
$resn_exch{"BHO3 FFC"}="HO3B FFC"; $resn_exch{"BHO4 FFC"}="HO4B FFC"; $resn_exch{"BHO6 FFC"}="HO6B FFC";
$resn_exch{"3HOP FFD"}="HOP3 FFD"; $resn_exch{" H1* FFD"}=" H1' FFD"; $resn_exch{"1H2* FFD"}="H2'1 FFD";
$resn_exch{"2H2* FFD"}="H2'2 FFD"; $resn_exch{" H3* FFD"}=" H3' FFD"; $resn_exch{" H4* FFD"}=" H4' FFD";
$resn_exch{"1H5* FFD"}="H5'1 FFD"; $resn_exch{"2H5* FFD"}="H5'2 FFD"; $resn_exch{"2HOP FFD"}="HOP2 FFD";
$resn_exch{"2HOA FFF"}="HOA2 FFF"; $resn_exch{"2HOB FFF"}="HOB2 FFF"; $resn_exch{"3HOB FFF"}="HOB3 FFF";
$resn_exch{"1H1  FFF"}=" H11 FFF"; $resn_exch{"2H1  FFF"}=" H12 FFF"; $resn_exch{"1H4  FFF"}=" H41 FFF";
$resn_exch{"2H4  FFF"}=" H42 FFF"; $resn_exch{"1H5  FFF"}=" H51 FFF"; $resn_exch{"2H5  FFF"}=" H52 FFF";
$resn_exch{"1H8  FFF"}=" H81 FFF"; $resn_exch{"2H8  FFF"}=" H82 FFF"; $resn_exch{"1H9  FFF"}=" H91 FFF";
$resn_exch{"2H9  FFF"}=" H92 FFF"; $resn_exch{"1H12 FFF"}="H121 FFF"; $resn_exch{"2H12 FFF"}="H122 FFF";
$resn_exch{"3H12 FFF"}="H123 FFF"; $resn_exch{"1H15 FFF"}="H151 FFF"; $resn_exch{"2H15 FFF"}="H152 FFF";
$resn_exch{"3H15 FFF"}="H153 FFF"; $resn_exch{"1H13 FFF"}="H131 FFF"; $resn_exch{"2H13 FFF"}="H132 FFF";
$resn_exch{"3H13 FFF"}="H133 FFF"; $resn_exch{"1HN2 FFO"}="HN21 FFO"; $resn_exch{"2HN2 FFO"}="HN22 FFO";
$resn_exch{"1H9  FFO"}=" H91 FFO"; $resn_exch{"2H9  FFO"}=" H92 FFO"; $resn_exch{"1HB  FFO"}=" HB1 FFO";
$resn_exch{"2HB  FFO"}=" HB2 FFO"; $resn_exch{"1HG  FFO"}=" HG1 FFO"; $resn_exch{"2HG  FFO"}=" HG2 FFO";
$resn_exch{"2HOE FFO"}="HOE2 FFO"; $resn_exch{"1H18 FG1"}="H181 FG1"; $resn_exch{"2H18 FG1"}="H182 FG1";
$resn_exch{"1H17 FG1"}="H171 FG1"; $resn_exch{"2H17 FG1"}="H172 FG1"; $resn_exch{"1H7  FG1"}=" H71 FG1";
$resn_exch{"2H7  FG1"}=" H72 FG1"; $resn_exch{"1H6  FG1"}=" H61 FG1"; $resn_exch{"2H6  FG1"}=" H62 FG1";
$resn_exch{"1H2  FG1"}="H21A FG1"; $resn_exch{"2H2  FG1"}=" H22 FG1"; $resn_exch{"1H14 FG1"}="H141 FG1";
$resn_exch{"2H14 FG1"}="H142 FG1"; $resn_exch{"2H   FGL"}=" H2  FGL"; $resn_exch{"1H   FGL"}=" H   FGL";
$resn_exch{"1HN  FGP"}=" HN1 FGP"; $resn_exch{"2HN  FGP"}=" HN2 FGP"; $resn_exch{"1H10 FGR"}="H101 FGR";
$resn_exch{"2H10 FGR"}="H102 FGR"; $resn_exch{"1H23 FGR"}="H231 FGR"; $resn_exch{"2H23 FGR"}="H232 FGR";
$resn_exch{"1H2C FHL"}="H2C1 FHL"; $resn_exch{"2H2C FHL"}="H2C2 FHL"; $resn_exch{"1H3C FHL"}="H3C1 FHL";
$resn_exch{"2H3C FHL"}="H3C2 FHL"; $resn_exch{"1H5C FHL"}="H5C1 FHL"; $resn_exch{"2H5C FHL"}="H5C2 FHL";
$resn_exch{"1HEC FHL"}="HEC1 FHL"; $resn_exch{"2HEC FHL"}="HEC2 FHL"; $resn_exch{"1HDC FHL"}="HDC1 FHL";
$resn_exch{"2HDC FHL"}="HDC2 FHL"; $resn_exch{"1HGC FHL"}="HGC1 FHL"; $resn_exch{"2HGC FHL"}="HGC2 FHL";
$resn_exch{"1HBC FHL"}="HBC1 FHL"; $resn_exch{"2HBC FHL"}="HBC2 FHL"; $resn_exch{"1HN  FHL"}=" HN1 FHL";
$resn_exch{"2HN  FHL"}=" HN2 FHL"; $resn_exch{"1H13 FHM"}="H131 FHM"; $resn_exch{"2H13 FHM"}="H132 FHM";
$resn_exch{"1H12 FHM"}="H121 FHM"; $resn_exch{"2H12 FHM"}="H122 FHM"; $resn_exch{"3H12 FHM"}="H123 FHM";
$resn_exch{"1H4  FHP"}=" H41 FHP"; $resn_exch{"2H4  FHP"}=" H42 FHP"; $resn_exch{"1H5  FHP"}=" H51 FHP";
$resn_exch{"2H5  FHP"}=" H52 FHP"; $resn_exch{"1H8  FHP"}=" H81 FHP"; $resn_exch{"2H8  FHP"}=" H82 FHP";
$resn_exch{"1H9  FHP"}=" H91 FHP"; $resn_exch{"2H9  FHP"}=" H92 FHP"; $resn_exch{"1H12 FHP"}="H121 FHP";
$resn_exch{"2H12 FHP"}="H122 FHP"; $resn_exch{"3H12 FHP"}="H123 FHP"; $resn_exch{"1H13 FHP"}="H131 FHP";
$resn_exch{"2H13 FHP"}="H132 FHP"; $resn_exch{"3H13 FHP"}="H133 FHP"; $resn_exch{"1H14 FHP"}="H141 FHP";
$resn_exch{"2H14 FHP"}="H142 FHP"; $resn_exch{"3H14 FHP"}="H143 FHP"; $resn_exch{"1H15 FHP"}="H151 FHP";
$resn_exch{"2H15 FHP"}="H152 FHP"; $resn_exch{"3H15 FHP"}="H153 FHP"; $resn_exch{"2HOA FHP"}="HOA2 FHP";
$resn_exch{"3HOA FHP"}="HOA3 FHP"; $resn_exch{" H1* FHU"}=" H1' FHU"; $resn_exch{" H2* FHU"}=" H2' FHU";
$resn_exch{"2HO* FHU"}="HO2' FHU"; $resn_exch{" H3* FHU"}=" H3' FHU"; $resn_exch{" H4* FHU"}=" H4' FHU";
$resn_exch{"H3T  FHU"}="HO3' FHU"; $resn_exch{"1H5* FHU"}=" H5' FHU"; $resn_exch{"2H5* FHU"}="H5'' FHU";
$resn_exch{"2HOP FHU"}="HOP2 FHU"; $resn_exch{"3HOP FHU"}="HOP3 FHU"; $resn_exch{"1H21 FID"}="H211 FID";
$resn_exch{"2H21 FID"}="H212 FID"; $resn_exch{"1H8I FID"}="H8I1 FID"; $resn_exch{"2H8I FID"}="H8I2 FID";
$resn_exch{"1H25 FIH"}="H251 FIH"; $resn_exch{"2H25 FIH"}="H252 FIH"; $resn_exch{"3H25 FIH"}="H253 FIH";
$resn_exch{"1H1  FII"}=" H11 FII"; $resn_exch{"2H1  FII"}=" H12 FII"; $resn_exch{"3H1  FII"}=" H13 FII";
$resn_exch{"1H6  FII"}=" H61 FII"; $resn_exch{"2H6  FII"}=" H62 FII"; $resn_exch{"3H6  FII"}=" H63 FII";
$resn_exch{"1H10 FII"}="H101 FII"; $resn_exch{"1H11 FII"}="H111 FII"; $resn_exch{"2H11 FII"}="H112 FII";
$resn_exch{"1H12 FII"}="H121 FII"; $resn_exch{"2H12 FII"}="H122 FII"; $resn_exch{"1H18 FII"}="H181 FII";
$resn_exch{"2H18 FII"}="H182 FII"; $resn_exch{"3H18 FII"}="H183 FII"; $resn_exch{"1H22 FII"}="H221 FII";
$resn_exch{"1H23 FII"}="H231 FII"; $resn_exch{"2H23 FII"}="H232 FII"; $resn_exch{"1H24 FII"}="H241 FII";
$resn_exch{"2H24 FII"}="H242 FII"; $resn_exch{"1H30 FII"}="H301 FII"; $resn_exch{"2H30 FII"}="H302 FII";
$resn_exch{"3H30 FII"}="H303 FII"; $resn_exch{"1H34 FII"}="H341 FII"; $resn_exch{"1H35 FII"}="H351 FII";
$resn_exch{"2H35 FII"}="H352 FII"; $resn_exch{"1HN  FII"}=" HN1 FII"; $resn_exch{"1H45 FII"}="H451 FII";
$resn_exch{"2H45 FII"}="H452 FII"; $resn_exch{"1H50 FII"}="H501 FII"; $resn_exch{"1H51 FII"}="H511 FII";
$resn_exch{"1H20 FIL"}="H201 FIL"; $resn_exch{"2H20 FIL"}="H202 FIL"; $resn_exch{"1H17 FIL"}="H171 FIL";
$resn_exch{"2H17 FIL"}="H172 FIL"; $resn_exch{"3H17 FIL"}="H173 FIL"; $resn_exch{"1H8  FIL"}=" H81 FIL";
$resn_exch{"2H8  FIL"}=" H82 FIL"; $resn_exch{"3H8  FIL"}=" H83 FIL"; $resn_exch{"1H12 FIL"}="H121 FIL";
$resn_exch{"2H12 FIL"}="H122 FIL"; $resn_exch{"1H13 FIL"}="H131 FIL"; $resn_exch{"2H13 FIL"}="H132 FIL";
$resn_exch{"1H14 FIL"}="H141 FIL"; $resn_exch{"2H14 FIL"}="H142 FIL"; $resn_exch{"1H15 FIL"}="H151 FIL";
$resn_exch{"2H15 FIL"}="H152 FIL"; $resn_exch{"1H3  FIN"}=" H31 FIN"; $resn_exch{"2H3  FIN"}=" H32 FIN";
$resn_exch{"3H3  FIN"}=" H33 FIN"; $resn_exch{"1H12 FIN"}="H121 FIN"; $resn_exch{"2H12 FIN"}="H122 FIN";
$resn_exch{"3H12 FIN"}="H123 FIN"; $resn_exch{"1H13 FIN"}="H131 FIN"; $resn_exch{"2H13 FIN"}="H132 FIN";
$resn_exch{"3H13 FIN"}="H133 FIN"; $resn_exch{"1H3P FIP"}="H3P1 FIP"; $resn_exch{"2H3P FIP"}="H3P2 FIP";
$resn_exch{"1H2P FIP"}="H2P1 FIP"; $resn_exch{"2H2P FIP"}="H2P2 FIP"; $resn_exch{"1H1P FIP"}="H1P1 FIP";
$resn_exch{"2H1P FIP"}="H1P2 FIP"; $resn_exch{"1H21 FIR"}="H211 FIR"; $resn_exch{"2H21 FIR"}="H212 FIR";
$resn_exch{"1H8I FIR"}="H8I1 FIR"; $resn_exch{"2H8I FIR"}="H8I2 FIR"; $resn_exch{"1H21 FIS"}="H211 FIS";
$resn_exch{"2H21 FIS"}="H212 FIS"; $resn_exch{"1H8I FIS"}="H8I1 FIS"; $resn_exch{"2H8I FIS"}="H8I2 FIS";
$resn_exch{"1H3  FK5"}="H31A FK5"; $resn_exch{"2H3  FK5"}="H32A FK5"; $resn_exch{"1H4  FK5"}=" H41 FK5";
$resn_exch{"2H4  FK5"}=" H42 FK5"; $resn_exch{"1H5  FK5"}=" H51 FK5"; $resn_exch{"2H5  FK5"}=" H52 FK5";
$resn_exch{"1H6  FK5"}=" H61 FK5"; $resn_exch{"2H6  FK5"}=" H62 FK5"; $resn_exch{"1H12 FK5"}="H121 FK5";
$resn_exch{"2H12 FK5"}="H122 FK5"; $resn_exch{"1H16 FK5"}="H161 FK5"; $resn_exch{"2H16 FK5"}="H162 FK5";
$resn_exch{"1H18 FK5"}="H181 FK5"; $resn_exch{"2H18 FK5"}="H182 FK5"; $resn_exch{"1H23 FK5"}="H231 FK5";
$resn_exch{"2H23 FK5"}="H232 FK5"; $resn_exch{"1H30 FK5"}="H301 FK5"; $resn_exch{"2H30 FK5"}="H302 FK5";
$resn_exch{"1H33 FK5"}="H331 FK5"; $resn_exch{"2H33 FK5"}="H332 FK5"; $resn_exch{"1H34 FK5"}="H341 FK5";
$resn_exch{"2H34 FK5"}="H342 FK5"; $resn_exch{"1H35 FK5"}="H351 FK5"; $resn_exch{"2H35 FK5"}="H352 FK5";
$resn_exch{"3H35 FK5"}="H353 FK5"; $resn_exch{"1H36 FK5"}="H361 FK5"; $resn_exch{"2H36 FK5"}="H362 FK5";
$resn_exch{"3H36 FK5"}="H363 FK5"; $resn_exch{"1H37 FK5"}="H371 FK5"; $resn_exch{"2H37 FK5"}="H372 FK5";
$resn_exch{"3H37 FK5"}="H373 FK5"; $resn_exch{"1H38 FK5"}="H381 FK5"; $resn_exch{"2H38 FK5"}="H382 FK5";
$resn_exch{"1H40 FK5"}="H401 FK5"; $resn_exch{"2H40 FK5"}="H402 FK5"; $resn_exch{"1H41 FK5"}="H411 FK5";
$resn_exch{"2H41 FK5"}="H412 FK5"; $resn_exch{"3H41 FK5"}="H413 FK5"; $resn_exch{"1H42 FK5"}="H421 FK5";
$resn_exch{"2H42 FK5"}="H422 FK5"; $resn_exch{"3H42 FK5"}="H423 FK5"; $resn_exch{"1H43 FK5"}="H431 FK5";
$resn_exch{"2H43 FK5"}="H432 FK5"; $resn_exch{"3H43 FK5"}="H433 FK5"; $resn_exch{"1H44 FK5"}="H441 FK5";
$resn_exch{"2H44 FK5"}="H442 FK5"; $resn_exch{"3H44 FK5"}="H443 FK5"; $resn_exch{"1H45 FK5"}="H451 FK5";
$resn_exch{"2H45 FK5"}="H452 FK5"; $resn_exch{"3H45 FK5"}="H453 FK5"; $resn_exch{"0HO1 FK5"}="HO10 FK5";
$resn_exch{"2HO1 FK5"}="HO12 FK5"; $resn_exch{"1H3  FKA"}="H31A FKA"; $resn_exch{"2H3  FKA"}=" H32 FKA";
$resn_exch{"1H4  FKA"}=" H41 FKA"; $resn_exch{"2H4  FKA"}=" H42 FKA"; $resn_exch{"1H5  FKA"}=" H51 FKA";
$resn_exch{"2H5  FKA"}=" H52 FKA"; $resn_exch{"1H6  FKA"}=" H61 FKA"; $resn_exch{"2H6  FKA"}=" H62 FKA";
$resn_exch{"1H11 FKA"}="H111 FKA"; $resn_exch{"2H11 FKA"}="H112 FKA"; $resn_exch{"1H34 FKA"}="H341 FKA";
$resn_exch{"2H34 FKA"}="H342 FKA"; $resn_exch{"3H34 FKA"}="H343 FKA"; $resn_exch{"1H42 FKA"}="H421 FKA";
$resn_exch{"2H42 FKA"}="H422 FKA"; $resn_exch{"3H42 FKA"}="H423 FKA"; $resn_exch{"1H43 FKA"}="H431 FKA";
$resn_exch{"2H43 FKA"}="H432 FKA"; $resn_exch{"3H43 FKA"}="H433 FKA"; $resn_exch{"1H15 FKA"}="H151 FKA";
$resn_exch{"2H15 FKA"}="H152 FKA"; $resn_exch{"1H35 FKA"}="H351 FKA"; $resn_exch{"2H35 FKA"}="H352 FKA";
$resn_exch{"3H35 FKA"}="H353 FKA"; $resn_exch{"1H17 FKA"}="H171 FKA"; $resn_exch{"2H17 FKA"}="H172 FKA";
$resn_exch{"1H36 FKA"}="H361 FKA"; $resn_exch{"2H36 FKA"}="H362 FKA"; $resn_exch{"3H36 FKA"}="H363 FKA";
$resn_exch{"1H37 FKA"}="H371 FKA"; $resn_exch{"2H37 FKA"}="H372 FKA"; $resn_exch{"1H22 FKA"}="H221 FKA";
$resn_exch{"2H22 FKA"}="H222 FKA"; $resn_exch{"1H40 FKA"}="H401 FKA"; $resn_exch{"2H40 FKA"}="H402 FKA";
$resn_exch{"3H40 FKA"}="H403 FKA"; $resn_exch{"1H41 FKA"}="H411 FKA"; $resn_exch{"2H41 FKA"}="H412 FKA";
$resn_exch{"3H41 FKA"}="H413 FKA"; $resn_exch{"1H29 FKA"}="H291 FKA"; $resn_exch{"2H29 FKA"}="H292 FKA";
$resn_exch{"1H32 FKA"}="H321 FKA"; $resn_exch{"2H32 FKA"}="H322 FKA"; $resn_exch{"1H33 FKA"}="H331 FKA";
$resn_exch{"2H33 FKA"}="H332 FKA"; $resn_exch{"1H44 FKA"}="H441 FKA"; $resn_exch{"2H44 FKA"}="H442 FKA";
$resn_exch{"3H44 FKA"}="H443 FKA"; $resn_exch{"1H51 FKA"}="H511 FKA"; $resn_exch{"2H51 FKA"}="H512 FKA";
$resn_exch{"1H49 FKA"}="H491 FKA"; $resn_exch{"2H49 FKA"}="H492 FKA"; $resn_exch{"1H9  FKD"}=" H91 FKD";
$resn_exch{"2H9  FKD"}=" H92 FKD"; $resn_exch{"2H   FKI"}=" H2  FKI"; $resn_exch{"1H   FKI"}=" H1  FKI";
$resn_exch{"2HB  FKI"}=" HB2 FKI"; $resn_exch{"1HB  FKI"}=" HB1 FKI"; $resn_exch{"1H2  FKP"}=" H21 FKP";
$resn_exch{"2H2  FKP"}=" H22 FKP"; $resn_exch{"1H3  FKP"}=" H31 FKP"; $resn_exch{"2H3  FKP"}=" H32 FKP";
$resn_exch{"1H12 FKP"}="H121 FKP"; $resn_exch{"2H12 FKP"}="H122 FKP"; $resn_exch{"1H15 FKP"}="H151 FKP";
$resn_exch{"2H15 FKP"}="H152 FKP"; $resn_exch{"1H16 FKP"}="H161 FKP"; $resn_exch{"2H16 FKP"}="H162 FKP";
$resn_exch{"3H16 FKP"}="H163 FKP"; $resn_exch{"1H17 FKP"}="H171 FKP"; $resn_exch{"2H17 FKP"}="H172 FKP";
$resn_exch{"3H17 FKP"}="H173 FKP"; $resn_exch{"1H18 FKP"}="H181 FKP"; $resn_exch{"2H18 FKP"}="H182 FKP";
$resn_exch{"3H18 FKP"}="H183 FKP"; $resn_exch{"1H19 FKP"}="H191 FKP"; $resn_exch{"2H19 FKP"}="H192 FKP";
$resn_exch{"3H19 FKP"}="H193 FKP"; $resn_exch{"1H20 FKP"}="H201 FKP"; $resn_exch{"2H20 FKP"}="H202 FKP";
$resn_exch{"3H20 FKP"}="H203 FKP"; $resn_exch{"1H22 FKP"}="H221 FKP"; $resn_exch{"2H22 FKP"}="H222 FKP";
$resn_exch{"1H23 FKP"}="H231 FKP"; $resn_exch{"2H23 FKP"}="H232 FKP"; $resn_exch{"1H24 FKP"}="H241 FKP";
$resn_exch{"2H24 FKP"}="H242 FKP"; $resn_exch{"1H25 FKP"}="H251 FKP"; $resn_exch{"2H25 FKP"}="H252 FKP";
$resn_exch{"1H26 FKP"}="H261 FKP"; $resn_exch{"2H26 FKP"}="H262 FKP"; $resn_exch{"1H27 FKP"}="H271 FKP";
$resn_exch{"2H27 FKP"}="H272 FKP"; $resn_exch{"1H28 FKP"}="H281 FKP"; $resn_exch{"2H28 FKP"}="H282 FKP";
$resn_exch{"1H29 FKP"}="H291 FKP"; $resn_exch{"2H29 FKP"}="H292 FKP"; $resn_exch{"1H30 FKP"}="H301 FKP";
$resn_exch{"2H30 FKP"}="H302 FKP"; $resn_exch{"3H30 FKP"}="H303 FKP"; $resn_exch{"1HN1 FL1"}="HN11 FL1";
$resn_exch{"2HN1 FL1"}="HN12 FL1"; $resn_exch{"1H7  FL1"}=" H71 FL1"; $resn_exch{"2H7  FL1"}=" H72 FL1";
$resn_exch{"1H29 FL1"}="H291 FL1"; $resn_exch{"2H29 FL1"}="H292 FL1"; $resn_exch{"1H13 FL2"}="H131 FL2";
$resn_exch{"2H13 FL2"}="H132 FL2"; $resn_exch{"3H13 FL2"}="H133 FL2"; $resn_exch{"1H16 FL2"}="H161 FL2";
$resn_exch{"2H16 FL2"}="H162 FL2"; $resn_exch{"3H16 FL2"}="H163 FL2"; $resn_exch{"1H5  FL8"}=" H51 FL8";
$resn_exch{"1H7  FL8"}=" H71 FL8"; $resn_exch{"1H8  FL8"}=" H81 FL8"; $resn_exch{"1H2' FL8"}="H2'1 FL8";
$resn_exch{"1H6' FL8"}="H6'1 FL8"; $resn_exch{"OH4' FL8"}="H4'O FL8"; $resn_exch{"1H3* FL8"}="H3'1 FL8";
$resn_exch{"2H3* FL8"}="H3'2 FL8"; $resn_exch{"3H3* FL8"}="H3'3 FL8"; $resn_exch{"1HO6 FL8"}="HO61 FL8";
$resn_exch{" H6* FL9"}=" H6' FL9"; $resn_exch{"*HO2 FL9"}="HO2' FL9"; $resn_exch{"*HO4 FL9"}="HO4' FL9";
$resn_exch{" HN2 FLA"}=" H2  FLA"; $resn_exch{"1HA  FLC"}=" HA1 FLC"; $resn_exch{"2HA  FLC"}=" HA2 FLC";
$resn_exch{"1HG  FLC"}=" HG1 FLC"; $resn_exch{"2HG  FLC"}=" HG2 FLC"; $resn_exch{"1H18 FLD"}="H181 FLD";
$resn_exch{"2H18 FLD"}="H182 FLD"; $resn_exch{"3H18 FLD"}="H183 FLD"; $resn_exch{"1H19 FLD"}="H191 FLD";
$resn_exch{"2H19 FLD"}="H192 FLD"; $resn_exch{"1H20 FLD"}="H201 FLD"; $resn_exch{"2H20 FLD"}="H202 FLD";
$resn_exch{"1HN2 FLD"}="HN21 FLD"; $resn_exch{"2HN2 FLD"}="HN22 FLD"; $resn_exch{"1H22 FLD"}="H221 FLD";
$resn_exch{"2H22 FLD"}="H222 FLD"; $resn_exch{"1H23 FLD"}="H231 FLD"; $resn_exch{"2H23 FLD"}="H232 FLD";
$resn_exch{"1H24 FLD"}="H241 FLD"; $resn_exch{"2H24 FLD"}="H242 FLD"; $resn_exch{" XH1 FLD"}=" H1X FLD";
$resn_exch{" XH3 FLD"}=" H3X FLD"; $resn_exch{" XH4 FLD"}=" H4X FLD"; $resn_exch{" XH5 FLD"}=" H5X FLD";
$resn_exch{" XH6 FLD"}=" H6X FLD"; $resn_exch{"XHN7 FLD"}="HN7X FLD"; $resn_exch{" XH8 FLD"}=" H8X FLD";
$resn_exch{" XH9 FLD"}=" H9X FLD"; $resn_exch{"XH11 FLD"}="H11X FLD"; $resn_exch{"1HX8 FLD"}="HX81 FLD";
$resn_exch{"2HX8 FLD"}="HX82 FLD"; $resn_exch{"3HX8 FLD"}="HX83 FLD"; $resn_exch{"1HX9 FLD"}="HX91 FLD";
$resn_exch{"2HX9 FLD"}="HX92 FLD"; $resn_exch{"1HX0 FLD"}="HX01 FLD"; $resn_exch{"2HX0 FLD"}="HX02 FLD";
$resn_exch{"XHN1 FLD"}="HN1X FLD"; $resn_exch{"XHN2 FLD"}="HN2X FLD"; $resn_exch{"1HX2 FLD"}="HX21 FLD";
$resn_exch{"2HX2 FLD"}="HX22 FLD"; $resn_exch{"1HX3 FLD"}="HX31 FLD"; $resn_exch{"2HX3 FLD"}="HX32 FLD";
$resn_exch{"1HX4 FLD"}="HX41 FLD"; $resn_exch{"2HX4 FLD"}="HX42 FLD"; $resn_exch{" HN  FLE"}=" H   FLE";
$resn_exch{"1HB  FLE"}=" HB2 FLE"; $resn_exch{"2HB  FLE"}=" HB3 FLE"; $resn_exch{"1HD1 FLE"}="HD11 FLE";
$resn_exch{"2HD1 FLE"}="HD12 FLE"; $resn_exch{"3HD1 FLE"}="HD13 FLE"; $resn_exch{"1HD2 FLE"}="HD21 FLE";
$resn_exch{"2HD2 FLE"}="HD22 FLE"; $resn_exch{"3HD2 FLE"}="HD23 FLE"; $resn_exch{"1H22 FLG"}="H221 FLG";
$resn_exch{"2H22 FLG"}="H222 FLG"; $resn_exch{"1H23 FLG"}="H231 FLG"; $resn_exch{"2H23 FLG"}="H232 FLG";
$resn_exch{"3H23 FLG"}="H233 FLG"; $resn_exch{"1H1  FLH"}=" H11 FLH"; $resn_exch{"2H1  FLH"}=" H12 FLH";
$resn_exch{"1HC4 FLM"}="HC41 FLM"; $resn_exch{"2HC4 FLM"}="HC42 FLM"; $resn_exch{"3HC4 FLM"}="HC43 FLM";
$resn_exch{"1HN6 FLM"}="HN61 FLM"; $resn_exch{"2HN6 FLM"}="HN62 FLM"; $resn_exch{"3H   FLN"}=" H3  FLN";
$resn_exch{" H2* FLN"}=" H2' FLN"; $resn_exch{" H3* FLN"}=" H3' FLN"; $resn_exch{" H4* FLN"}=" H4' FLN";
$resn_exch{" H5* FLN"}=" H5' FLN"; $resn_exch{" H6* FLN"}=" H6' FLN"; $resn_exch{"5H   FLN"}=" H5  FLN";
$resn_exch{"6H   FLN"}=" H6  FLN"; $resn_exch{"7H   FLN"}=" H7  FLN"; $resn_exch{"8H   FLN"}=" H8  FLN";
$resn_exch{"1H13 FLP"}="H131 FLP"; $resn_exch{"2H13 FLP"}="H132 FLP"; $resn_exch{"3H13 FLP"}="H133 FLP";
$resn_exch{"1H16 FLQ"}="H161 FLQ"; $resn_exch{"2H16 FLQ"}="H162 FLQ"; $resn_exch{"1H17 FLQ"}="H171 FLQ";
$resn_exch{"2H17 FLQ"}="H172 FLQ"; $resn_exch{"1H18 FLQ"}="H181 FLQ"; $resn_exch{"2H18 FLQ"}="H182 FLQ";
$resn_exch{"1H19 FLQ"}="H191 FLQ"; $resn_exch{"2H19 FLQ"}="H192 FLQ"; $resn_exch{" H2* FLQ"}=" H2' FLQ";
$resn_exch{" H3* FLQ"}=" H3' FLQ"; $resn_exch{" H6* FLQ"}=" H6' FLQ"; $resn_exch{"1H20 FLQ"}="H201 FLQ";
$resn_exch{"2H20 FLQ"}="H202 FLQ"; $resn_exch{"1H21 FLQ"}="H211 FLQ"; $resn_exch{"2H21 FLQ"}="H212 FLQ";
$resn_exch{"1H23 FLQ"}="H231 FLQ"; $resn_exch{"2H23 FLQ"}="H232 FLQ"; $resn_exch{"3H23 FLQ"}="H233 FLQ";
$resn_exch{"1HN  FLT"}=" H   FLT"; $resn_exch{"2HN  FLT"}=" H2  FLT"; $resn_exch{"1HA  FLT"}=" HA1 FLT";
$resn_exch{"1HB  FLT"}=" HB2 FLT"; $resn_exch{"2HB  FLT"}=" HB3 FLT"; $resn_exch{"1HD1 FLT"}=" HD1 FLT";
$resn_exch{"1HE1 FLT"}=" HE1 FLT"; $resn_exch{"1HD2 FLT"}=" HD2 FLT"; $resn_exch{"1HE2 FLT"}=" HE2 FLT";
$resn_exch{"1HO1 FLT"}="HO11 FLT"; $resn_exch{"1HO3 FLT"}="HO31 FLT"; $resn_exch{"1HOX FLT"}=" HXT FLT";
$resn_exch{"1HB  FLX"}=" HB1 FLX"; $resn_exch{"2HB  FLX"}=" HB2 FLX"; $resn_exch{"1HD1 FLX"}="HD11 FLX";
$resn_exch{"2HD1 FLX"}="HD12 FLX"; $resn_exch{"3HD1 FLX"}="HD13 FLX"; $resn_exch{"1HD2 FLX"}="HD21 FLX";
$resn_exch{"2HD2 FLX"}="HD22 FLX"; $resn_exch{"3HD2 FLX"}="HD23 FLX"; $resn_exch{"1H2  FLX"}="H21A FLX";
$resn_exch{"2H2  FLX"}=" H22 FLX"; $resn_exch{"2HOP FLX"}="HOP2 FLX"; $resn_exch{"3HOP FLX"}="HOP3 FLX";
$resn_exch{"1HN6 FM1"}="HN61 FM1"; $resn_exch{"'HO2 FM1"}="HO2' FM1"; $resn_exch{"'HO3 FM1"}="HO3' FM1";
$resn_exch{"1H5' FM1"}="H5'1 FM1"; $resn_exch{"2H5' FM1"}="H5'2 FM1"; $resn_exch{"'HO5 FM1"}="HO5' FM1";
$resn_exch{"1H66 FM1"}="H661 FM1"; $resn_exch{"2H66 FM1"}="H662 FM1"; $resn_exch{"3H66 FM1"}="H663 FM1";
$resn_exch{"1HN6 FM2"}="HN61 FM2"; $resn_exch{"2HN6 FM2"}="HN62 FM2"; $resn_exch{"'HO2 FM2"}="HO2' FM2";
$resn_exch{"'HO3 FM2"}="HO3' FM2"; $resn_exch{"1H5' FM2"}="H5'1 FM2"; $resn_exch{"2H5' FM2"}="H5'2 FM2";
$resn_exch{"'HO5 FM2"}="HO5' FM2"; $resn_exch{"1H11 FM2"}="H111 FM2"; $resn_exch{"2H11 FM2"}="H112 FM2";
$resn_exch{"3H11 FM2"}="H113 FM2"; $resn_exch{"1HM7 FMA"}="HM71 FMA"; $resn_exch{"2HM7 FMA"}="HM72 FMA";
$resn_exch{"3HM7 FMA"}="HM73 FMA"; $resn_exch{"1HM8 FMA"}="HM81 FMA"; $resn_exch{"2HM8 FMA"}="HM82 FMA";
$resn_exch{"3HM8 FMA"}="HM83 FMA"; $resn_exch{"1H1* FMA"}="H1'1 FMA"; $resn_exch{"2H1* FMA"}="H1'2 FMA";
$resn_exch{" H2* FMA"}=" H2' FMA"; $resn_exch{"*HO2 FMA"}="HO2' FMA"; $resn_exch{" H3* FMA"}=" H3B FMA";
$resn_exch{"*HO3 FMA"}="HO3' FMA"; $resn_exch{" H4* FMA"}=" H4' FMA"; $resn_exch{"*HO4 FMA"}="HO4' FMA";
$resn_exch{"1H5* FMA"}="H5'1 FMA"; $resn_exch{"2H5* FMA"}="H5'2 FMA"; $resn_exch{"2HOP FMA"}="HOP2 FMA";
$resn_exch{"3HOP FMA"}="HOP3 FMA"; $resn_exch{"'HO2 FMA"}="HO2A FMA"; $resn_exch{"1H2' FMA"}="H2'1 FMA";
$resn_exch{"2H2' FMA"}="H2'2 FMA"; $resn_exch{"1H4' FMA"}="H4'1 FMA"; $resn_exch{"2H4' FMA"}="H4'2 FMA";
$resn_exch{"1H5' FMA"}=" H5B FMA"; $resn_exch{"2H5' FMA"}=" H5D FMA"; $resn_exch{"1H6' FMA"}="H6'1 FMA";
$resn_exch{"2H6' FMA"}="H6'2 FMA"; $resn_exch{"1H7' FMA"}="H7'1 FMA"; $resn_exch{"2H7' FMA"}="H7'2 FMA";
$resn_exch{"1H8' FMA"}="H8'1 FMA"; $resn_exch{"2H8' FMA"}="H8'2 FMA"; $resn_exch{"1H9' FMA"}="H9'1 FMA";
$resn_exch{"2H9' FMA"}="H9'2 FMA"; $resn_exch{"1H11 FMA"}="H111 FMA"; $resn_exch{"2H11 FMA"}="H112 FMA";
$resn_exch{"1H12 FMA"}="H121 FMA"; $resn_exch{"2H12 FMA"}="H122 FMA"; $resn_exch{"1H13 FMA"}="H131 FMA";
$resn_exch{"2H13 FMA"}="H132 FMA"; $resn_exch{"1H14 FMA"}="H141 FMA"; $resn_exch{"2H14 FMA"}="H142 FMA";
$resn_exch{"1H15 FMA"}="H151 FMA"; $resn_exch{"2H15 FMA"}="H152 FMA"; $resn_exch{"3H15 FMA"}="H153 FMA";
$resn_exch{"1H5' FMB"}="H5'1 FMB"; $resn_exch{"2H5' FMB"}="H5'2 FMB"; $resn_exch{"1HN6 FMC"}="HN61 FMC";
$resn_exch{"2HN6 FMC"}="HN62 FMC"; $resn_exch{"'HO2 FMC"}="HO2' FMC"; $resn_exch{"'HO3 FMC"}="HO3' FMC";
$resn_exch{"1H5' FMC"}="H5'1 FMC"; $resn_exch{"2H5' FMC"}="H5'2 FMC"; $resn_exch{"'HO5 FMC"}="HO5' FMC";
$resn_exch{"1H28 FMD"}="H281 FMD"; $resn_exch{"2H28 FMD"}="H282 FMD"; $resn_exch{"1H25 FMD"}="H251 FMD";
$resn_exch{"2H25 FMD"}="H252 FMD"; $resn_exch{"3H25 FMD"}="H253 FMD"; $resn_exch{"1H22 FMD"}="H221 FMD";
$resn_exch{"2H22 FMD"}="H222 FMD"; $resn_exch{"3H22 FMD"}="H223 FMD"; $resn_exch{" HN  FME"}=" H   FME";
$resn_exch{"1HB  FME"}=" HB2 FME"; $resn_exch{"2HB  FME"}=" HB3 FME"; $resn_exch{"1HG  FME"}=" HG2 FME";
$resn_exch{"2HG  FME"}=" HG3 FME"; $resn_exch{"1HE  FME"}=" HE1 FME"; $resn_exch{"2HE  FME"}=" HE2 FME";
$resn_exch{"3HE  FME"}=" HE3 FME"; $resn_exch{"1H6  FMF"}=" H61 FMF"; $resn_exch{"2H6  FMF"}=" H62 FMF";
$resn_exch{"1HAA FMI"}="HAA1 FMI"; $resn_exch{"2HAA FMI"}="HAA2 FMI"; $resn_exch{"1HMA FMI"}="HMA1 FMI";
$resn_exch{"2HMA FMI"}="HMA2 FMI"; $resn_exch{"3HMA FMI"}="HMA3 FMI"; $resn_exch{"1HBA FMI"}="HBA1 FMI";
$resn_exch{"2HBA FMI"}="HBA2 FMI"; $resn_exch{"1HLB FMI"}="HLB1 FMI"; $resn_exch{"2HLB FMI"}="HLB2 FMI";
$resn_exch{"3HLB FMI"}="HLB3 FMI"; $resn_exch{"1HMB FMI"}="HMB1 FMI"; $resn_exch{"2HMB FMI"}="HMB2 FMI";
$resn_exch{"1HNB FMI"}="HNB1 FMI"; $resn_exch{"2HNB FMI"}="HNB2 FMI"; $resn_exch{"3HNB FMI"}="HNB3 FMI";
$resn_exch{"1HMC FMI"}="HMC1 FMI"; $resn_exch{"2HMC FMI"}="HMC2 FMI"; $resn_exch{"3HMC FMI"}="HMC3 FMI";
$resn_exch{"1HAC FMI"}="HAC1 FMI"; $resn_exch{"2HAC FMI"}="HAC2 FMI"; $resn_exch{"1HBC FMI"}="HBC1 FMI";
$resn_exch{"2HBC FMI"}="HBC2 FMI"; $resn_exch{"3HBC FMI"}="HBC3 FMI"; $resn_exch{"1HMD FMI"}="HMD1 FMI";
$resn_exch{"2HMD FMI"}="HMD2 FMI"; $resn_exch{"3HMD FMI"}="HMD3 FMI"; $resn_exch{"1HAD FMI"}="HAD1 FMI";
$resn_exch{"2HAD FMI"}="HAD2 FMI"; $resn_exch{"1HBD FMI"}="HBD1 FMI"; $resn_exch{"2HBD FMI"}="HBD2 FMI";
$resn_exch{"1H1  FMM"}="H11A FMM"; $resn_exch{"2H1  FMM"}=" H12 FMM"; $resn_exch{"3H1  FMM"}=" H13 FMM";
$resn_exch{"1H5  FMM"}=" H51 FMM"; $resn_exch{"2H5  FMM"}=" H52 FMM"; $resn_exch{"1H6  FMM"}=" H61 FMM";
$resn_exch{"2H6  FMM"}=" H62 FMM"; $resn_exch{"1H8  FMM"}=" H81 FMM"; $resn_exch{"2H8  FMM"}=" H82 FMM";
$resn_exch{"1H28 FMM"}="H281 FMM"; $resn_exch{"2H28 FMM"}="H282 FMM"; $resn_exch{"1HM7 FMN"}="HM71 FMN";
$resn_exch{"2HM7 FMN"}="HM72 FMN"; $resn_exch{"3HM7 FMN"}="HM73 FMN"; $resn_exch{"1HM8 FMN"}="HM81 FMN";
$resn_exch{"2HM8 FMN"}="HM82 FMN"; $resn_exch{"3HM8 FMN"}="HM83 FMN"; $resn_exch{"1H1* FMN"}="H1'1 FMN";
$resn_exch{"2H1* FMN"}="H1'2 FMN"; $resn_exch{" H2* FMN"}=" H2' FMN"; $resn_exch{"*HO2 FMN"}="HO2' FMN";
$resn_exch{" H3* FMN"}=" H3' FMN"; $resn_exch{"*HO3 FMN"}="HO3' FMN"; $resn_exch{" H4* FMN"}=" H4' FMN";
$resn_exch{"*HO4 FMN"}="HO4' FMN"; $resn_exch{"1H5* FMN"}="H5'1 FMN"; $resn_exch{"2H5* FMN"}="H5'2 FMN";
$resn_exch{"2HOP FMN"}="HOP2 FMN"; $resn_exch{"3HOP FMN"}="HOP3 FMN"; $resn_exch{"2HOP FMP"}="HOP2 FMP";
$resn_exch{"3HOP FMP"}="HOP3 FMP"; $resn_exch{"1H5* FMP"}="H5'1 FMP"; $resn_exch{"2H5* FMP"}="H5'2 FMP";
$resn_exch{" H4* FMP"}=" H4' FMP"; $resn_exch{" H3* FMP"}=" H3' FMP"; $resn_exch{"*HO3 FMP"}="HO3' FMP";
$resn_exch{" H2* FMP"}=" H2' FMP"; $resn_exch{"*HO2 FMP"}="HO2' FMP"; $resn_exch{" H1* FMP"}=" H1' FMP";
$resn_exch{"1HN6 FMP"}="HN61 FMP"; $resn_exch{"2HN6 FMP"}="HN62 FMP"; $resn_exch{"1HN  FMS"}=" HN1 FMS";
$resn_exch{"2HN  FMS"}=" HN2 FMS"; $resn_exch{"1H5M FMU"}="H5M1 FMU"; $resn_exch{"2H5M FMU"}="H5M2 FMU";
$resn_exch{"3H5M FMU"}="H5M3 FMU"; $resn_exch{"1H6C FMU"}="H6C1 FMU"; $resn_exch{"2H6C FMU"}="H6C2 FMU";
$resn_exch{" H1* FMU"}=" H1' FMU"; $resn_exch{" H2* FMU"}=" H2' FMU"; $resn_exch{" H3* FMU"}=" H3' FMU";
$resn_exch{" H4* FMU"}=" H4' FMU"; $resn_exch{"1H5* FMU"}="H5'1 FMU"; $resn_exch{"2H5* FMU"}="H5'2 FMU";
$resn_exch{"1HN1 FMX"}="HN11 FMX"; $resn_exch{"1H22 FMX"}="H221 FMX"; $resn_exch{"1H23 FMX"}="H231 FMX";
$resn_exch{"1H24 FMX"}="H241 FMX"; $resn_exch{"1H25 FMX"}="H251 FMX"; $resn_exch{"1H26 FMX"}="H261 FMX";
$resn_exch{"1H7  FMX"}=" H71 FMX"; $resn_exch{"2H7  FMX"}=" H72 FMX"; $resn_exch{"3H7  FMX"}=" H73 FMX";
$resn_exch{"1H9  FMX"}=" H91 FMX"; $resn_exch{"1H10 FMX"}="H101 FMX"; $resn_exch{"1H12 FMX"}="H121 FMX";
$resn_exch{"1H13 FMX"}="H131 FMX"; $resn_exch{"1H16 FMX"}="H161 FMX"; $resn_exch{"1H17 FMX"}="H171 FMX";
$resn_exch{"1H18 FMX"}="H181 FMX"; $resn_exch{"1H19 FMX"}="H191 FMX"; $resn_exch{"1H20 FMX"}="H201 FMX";
$resn_exch{"1H1  FNG"}=" H11 FNG"; $resn_exch{"1H2  FNG"}=" H21 FNG"; $resn_exch{"1HO2 FNG"}="HO21 FNG";
$resn_exch{"1H3  FNG"}=" H31 FNG"; $resn_exch{"1HO3 FNG"}="HO31 FNG"; $resn_exch{"1H4  FNG"}=" H41 FNG";
$resn_exch{"1HO4 FNG"}="HO41 FNG"; $resn_exch{"1H5  FNG"}=" H51 FNG"; $resn_exch{"1H6  FNG"}=" H61 FNG";
$resn_exch{"2H6  FNG"}=" H62 FNG"; $resn_exch{"1HO6 FNG"}="HO61 FNG"; $resn_exch{"1H12 FNG"}="H121 FNG";
$resn_exch{"1H10 FNG"}="H101 FNG"; $resn_exch{"1H8  FNG"}=" H81 FNG"; $resn_exch{"1H15 FNG"}="H151 FNG";
$resn_exch{"2H15 FNG"}="H152 FNG"; $resn_exch{"1H3  FNH"}="H31A FNH"; $resn_exch{"2H3  FNH"}="H32A FNH";
$resn_exch{"1H4  FNH"}=" H41 FNH"; $resn_exch{"2H4  FNH"}=" H42 FNH"; $resn_exch{"1H5  FNH"}="H51A FNH";
$resn_exch{"2H5  FNH"}=" H52 FNH"; $resn_exch{"1H6  FNH"}=" H61 FNH"; $resn_exch{"2H6  FNH"}=" H62 FNH";
$resn_exch{"1H7  FNH"}=" H71 FNH"; $resn_exch{"2H7  FNH"}=" H72 FNH"; $resn_exch{"1H8  FNH"}=" H81 FNH";
$resn_exch{"2H8  FNH"}=" H82 FNH"; $resn_exch{"1H9  FNH"}=" H91 FNH"; $resn_exch{"2H9  FNH"}=" H92 FNH";
$resn_exch{"1H13 FNH"}="H131 FNH"; $resn_exch{"2H13 FNH"}="H132 FNH"; $resn_exch{"1H17 FNH"}="H171 FNH";
$resn_exch{"2H17 FNH"}="H172 FNH"; $resn_exch{"1H14 FNH"}="H141 FNH"; $resn_exch{"2H14 FNH"}="H142 FNH";
$resn_exch{"1H16 FNH"}="H161 FNH"; $resn_exch{"2H16 FNH"}="H162 FNH"; $resn_exch{"1H15 FNH"}="H151 FNH";
$resn_exch{"2H15 FNH"}="H152 FNH"; $resn_exch{"1H22 FNH"}="H221 FNH"; $resn_exch{"2H22 FNH"}="H222 FNH";
$resn_exch{"1H33 FNH"}="H331 FNH"; $resn_exch{"2H33 FNH"}="H332 FNH"; $resn_exch{"1H34 FNH"}="H341 FNH";
$resn_exch{"2H34 FNH"}="H342 FNH"; $resn_exch{"1H35 FNH"}="H351 FNH"; $resn_exch{"2H35 FNH"}="H352 FNH";
$resn_exch{"3H35 FNH"}="H353 FNH"; $resn_exch{"1H40 FNH"}="H401 FNH"; $resn_exch{"2H40 FNH"}="H402 FNH";
$resn_exch{"1H7M FNR"}="H7M1 FNR"; $resn_exch{"2H7M FNR"}="H7M2 FNR"; $resn_exch{"3H7M FNR"}="H7M3 FNR";
$resn_exch{"1H8M FNR"}="H8M1 FNR"; $resn_exch{"2H8M FNR"}="H8M2 FNR"; $resn_exch{"3H8M FNR"}="H8M3 FNR";
$resn_exch{"1H1* FNR"}="H1'1 FNR"; $resn_exch{"2H1* FNR"}="H1'2 FNR"; $resn_exch{" H2* FNR"}=" H2' FNR";
$resn_exch{" H3* FNR"}=" H3' FNR"; $resn_exch{" H4* FNR"}=" H4' FNR"; $resn_exch{"1H5* FNR"}="H5'1 FNR";
$resn_exch{"2H5* FNR"}="H5'2 FNR"; $resn_exch{"1HM7 FNS"}="HM71 FNS"; $resn_exch{"2HM7 FNS"}="HM72 FNS";
$resn_exch{"3HM7 FNS"}="HM73 FNS"; $resn_exch{"1HM8 FNS"}="HM81 FNS"; $resn_exch{"2HM8 FNS"}="HM82 FNS";
$resn_exch{"3HM8 FNS"}="HM83 FNS"; $resn_exch{"1H1* FNS"}="H1'1 FNS"; $resn_exch{"2H1* FNS"}="H1'2 FNS";
$resn_exch{" H2* FNS"}=" H2' FNS"; $resn_exch{"*HO2 FNS"}="HO2' FNS"; $resn_exch{" H3* FNS"}=" H3' FNS";
$resn_exch{"*HO3 FNS"}="HO3' FNS"; $resn_exch{" H4* FNS"}=" H4' FNS"; $resn_exch{"*HO4 FNS"}="HO4' FNS";
$resn_exch{"1H5* FNS"}="H5'1 FNS"; $resn_exch{"2H5* FNS"}="H5'2 FNS"; $resn_exch{"2HOP FNS"}="HOP2 FNS";
$resn_exch{"3HOP FNS"}="HOP3 FNS"; $resn_exch{"1H3C FNZ"}="H3C1 FNZ"; $resn_exch{"2H3C FNZ"}="H3C2 FNZ";
$resn_exch{"1H1  FOC"}=" H11 FOC"; $resn_exch{"2H1  FOC"}=" H12 FOC"; $resn_exch{"1H6  FOC"}=" H61 FOC";
$resn_exch{"2H6  FOC"}=" H62 FOC"; $resn_exch{"3H6  FOC"}=" H63 FOC"; $resn_exch{"1HN  FOE"}=" H   FOE";
$resn_exch{"2HN  FOE"}=" H2  FOE"; $resn_exch{"1HB  FOE"}=" HB2 FOE"; $resn_exch{"2HB  FOE"}=" HB3 FOE";
$resn_exch{"1H1  FOE"}="H11A FOE"; $resn_exch{"2H1  FOE"}="H12A FOE"; $resn_exch{"1H5  FOE"}=" H51 FOE";
$resn_exch{"2H5  FOE"}=" H52 FOE"; $resn_exch{"3H5  FOE"}=" H53 FOE"; $resn_exch{"1H7  FOE"}=" H71 FOE";
$resn_exch{"2H7  FOE"}=" H72 FOE"; $resn_exch{"3H7  FOE"}=" H73 FOE"; $resn_exch{"1H   FOG"}=" HC1 FOG";
$resn_exch{"2H   FOG"}=" HC2 FOG"; $resn_exch{"1HN  FOG"}=" H   FOG"; $resn_exch{"2HN  FOG"}=" H2  FOG";
$resn_exch{"1HB  FOG"}=" HB2 FOG"; $resn_exch{"2HB  FOG"}=" HB3 FOG"; $resn_exch{"1H10 FOH"}="H101 FOH";
$resn_exch{"2H10 FOH"}="H102 FOH"; $resn_exch{"1H11 FOH"}="H111 FOH"; $resn_exch{"2H11 FOH"}="H112 FOH";
$resn_exch{"3H13 FOH"}="H133 FOH"; $resn_exch{"2H13 FOH"}="H132 FOH"; $resn_exch{"1H13 FOH"}="H131 FOH";
$resn_exch{"1H15 FOH"}="H151 FOH"; $resn_exch{"2H15 FOH"}="H152 FOH"; $resn_exch{"1H2  FOK"}=" H21 FOK";
$resn_exch{"2H2  FOK"}=" H22 FOK"; $resn_exch{"1H3  FOK"}=" H31 FOK"; $resn_exch{"2H3  FOK"}=" H32 FOK";
$resn_exch{"1H12 FOK"}="H121 FOK"; $resn_exch{"2H12 FOK"}="H122 FOK"; $resn_exch{"1H15 FOK"}="H151 FOK";
$resn_exch{"2H15 FOK"}="H152 FOK"; $resn_exch{"1H16 FOK"}="H161 FOK"; $resn_exch{"2H16 FOK"}="H162 FOK";
$resn_exch{"3H16 FOK"}="H163 FOK"; $resn_exch{"1H17 FOK"}="H171 FOK"; $resn_exch{"2H17 FOK"}="H172 FOK";
$resn_exch{"3H17 FOK"}="H173 FOK"; $resn_exch{"1H18 FOK"}="H181 FOK"; $resn_exch{"2H18 FOK"}="H182 FOK";
$resn_exch{"3H18 FOK"}="H183 FOK"; $resn_exch{"1H19 FOK"}="H191 FOK"; $resn_exch{"2H19 FOK"}="H192 FOK";
$resn_exch{"3H19 FOK"}="H193 FOK"; $resn_exch{"1H20 FOK"}="H201 FOK"; $resn_exch{"2H20 FOK"}="H202 FOK";
$resn_exch{"3H20 FOK"}="H203 FOK"; $resn_exch{"1H22 FOK"}="H221 FOK"; $resn_exch{"2H22 FOK"}="H222 FOK";
$resn_exch{"3H22 FOK"}="H223 FOK"; $resn_exch{"1HN2 FOL"}="HN21 FOL"; $resn_exch{"2HN2 FOL"}="HN22 FOL";
$resn_exch{"1H9  FOL"}=" H91 FOL"; $resn_exch{"2H9  FOL"}=" H92 FOL"; $resn_exch{"1HB  FOL"}=" HB1 FOL";
$resn_exch{"2HB  FOL"}=" HB2 FOL"; $resn_exch{"1HG  FOL"}=" HG1 FOL"; $resn_exch{"2HG  FOL"}=" HG2 FOL";
$resn_exch{"2HOE FOL"}="HOE2 FOL"; $resn_exch{"1H2  FOM"}=" H21 FOM"; $resn_exch{"2H2  FOM"}=" H22 FOM";
$resn_exch{"1H3  FOM"}=" H31 FOM"; $resn_exch{"2H3  FOM"}=" H32 FOM"; $resn_exch{"1H4  FOM"}=" H41 FOM";
$resn_exch{"2H4  FOM"}=" H42 FOM"; $resn_exch{"1HN2 FON"}="HN21 FON"; $resn_exch{"2HN2 FON"}="HN22 FON";
$resn_exch{"1H6  FON"}=" H61 FON"; $resn_exch{"2HC7 FON"}="HC72 FON"; $resn_exch{"1H9  FON"}=" H91 FON";
$resn_exch{"2H9  FON"}=" H92 FON"; $resn_exch{"1HB  FON"}=" HB1 FON"; $resn_exch{"2HB  FON"}=" HB2 FON";
$resn_exch{"1HG  FON"}=" HG1 FON"; $resn_exch{"2HG  FON"}=" HG2 FON"; $resn_exch{"2HOE FON"}="HOE2 FON";
$resn_exch{"1H2A FOO"}="H2A1 FOO"; $resn_exch{"2H2A FOO"}="H2A2 FOO"; $resn_exch{"3H2A FOO"}="H2A3 FOO";
$resn_exch{"1H5A FOO"}="H5A1 FOO"; $resn_exch{"2H5A FOO"}="H5A2 FOO"; $resn_exch{"1H   FOR"}=" H1  FOR";
$resn_exch{"2H   FOR"}=" H2  FOR"; $resn_exch{"1H   FOS"}=" H1  FOS"; $resn_exch{"2H   FOS"}=" H2A FOS";
$resn_exch{"1HM  FOS"}=" HM1 FOS"; $resn_exch{"2HM  FOS"}=" HM2 FOS"; $resn_exch{"2HOP FOS"}="HOP2 FOS";
$resn_exch{"3HOP FOS"}="HOP3 FOS"; $resn_exch{"2HOP FOX"}="HOP2 FOX"; $resn_exch{"1H5* FOX"}="H5'1 FOX";
$resn_exch{"2H5* FOX"}="H5'2 FOX"; $resn_exch{" H4* FOX"}=" H4' FOX"; $resn_exch{"1H6* FOX"}="H6'1 FOX";
$resn_exch{"2H6* FOX"}="H6'2 FOX"; $resn_exch{" H3* FOX"}=" H3' FOX"; $resn_exch{"1H2* FOX"}="H2'1 FOX";
$resn_exch{"2H2* FOX"}="H2'2 FOX"; $resn_exch{" H1* FOX"}=" H1' FOX"; $resn_exch{"1H2  FOX"}=" H21 FOX";
$resn_exch{"2H2  FOX"}=" H22 FOX"; $resn_exch{"3HOP FOX"}="HOP3 FOX"; $resn_exch{"1H7C FP1"}="H7C1 FP1";
$resn_exch{"2H7C FP1"}="H7C2 FP1"; $resn_exch{"3H7C FP1"}="H7C3 FP1"; $resn_exch{"1H3  FPA"}=" H31 FPA";
$resn_exch{"2H3  FPA"}=" H32 FPA"; $resn_exch{"1H4  FPA"}=" H41 FPA"; $resn_exch{"2H4  FPA"}=" H42 FPA";
$resn_exch{"1H11 FPB"}="H111 FPB"; $resn_exch{"2H11 FPB"}="H112 FPB"; $resn_exch{"1H18 FPB"}="H181 FPB";
$resn_exch{"2H18 FPB"}="H182 FPB"; $resn_exch{"1H9C FPB"}="H9C1 FPB"; $resn_exch{"2H9C FPB"}="H9C2 FPB";
$resn_exch{"1H21 FPB"}="H211 FPB"; $resn_exch{"2H21 FPB"}="H212 FPB"; $resn_exch{"1H20 FPB"}="H201 FPB";
$resn_exch{"2H20 FPB"}="H202 FPB"; $resn_exch{"1H19 FPB"}="H191 FPB"; $resn_exch{"2H19 FPB"}="H192 FPB";
$resn_exch{"1H15 FPB"}="H151 FPB"; $resn_exch{"2H15 FPB"}="H152 FPB"; $resn_exch{"1H6  FPC"}=" H61 FPC";
$resn_exch{"2H6  FPC"}=" H62 FPC"; $resn_exch{"1H1  FPC"}=" H11 FPC"; $resn_exch{"2H1  FPC"}=" H12 FPC";
$resn_exch{"2HOP FPC"}="HOP2 FPC"; $resn_exch{"3HOP FPC"}="HOP3 FPC"; $resn_exch{"1H3  FPD"}=" H31 FPD";
$resn_exch{"2H3  FPD"}=" H32 FPD"; $resn_exch{"3H3  FPD"}=" H33 FPD"; $resn_exch{"1H2  FPD"}=" H21 FPD";
$resn_exch{"2H2  FPD"}=" H22 FPD"; $resn_exch{"1H7  FPD"}=" H71 FPD"; $resn_exch{"2H7  FPD"}=" H72 FPD";
$resn_exch{"1H8  FPD"}=" H81 FPD"; $resn_exch{"2H8  FPD"}=" H82 FPD"; $resn_exch{"1H12 FPD"}="H121 FPD";
$resn_exch{"2H12 FPD"}="H122 FPD"; $resn_exch{"1H3  FPE"}=" H31 FPE"; $resn_exch{"2H3  FPE"}=" H32 FPE";
$resn_exch{"1H9  FPG"}=" H91 FPG"; $resn_exch{"2H9  FPG"}=" H92 FPG"; $resn_exch{"3H9  FPG"}=" H93 FPG";
$resn_exch{"1H6  FPG"}=" H61 FPG"; $resn_exch{"2H6  FPG"}=" H62 FPG"; $resn_exch{"1H8  FPG"}=" H81 FPG";
$resn_exch{"2H8  FPG"}=" H82 FPG"; $resn_exch{"3H8  FPG"}=" H83 FPG"; $resn_exch{"1H1  FPG"}=" H11 FPG";
$resn_exch{"2H1  FPG"}=" H12 FPG"; $resn_exch{"3H1  FPG"}=" H13 FPG"; $resn_exch{"1H10 FPG"}="H101 FPG";
$resn_exch{"2H10 FPG"}="H102 FPG"; $resn_exch{"3H10 FPG"}="H103 FPG"; $resn_exch{"1H5  FPG"}=" H51 FPG";
$resn_exch{"2H5  FPG"}=" H52 FPG"; $resn_exch{"1H4  FPG"}=" H41 FPG"; $resn_exch{"2H4  FPG"}=" H42 FPG";
$resn_exch{"1H2  FPI"}=" H21 FPI"; $resn_exch{"2H2  FPI"}=" H22 FPI"; $resn_exch{"1H3  FPI"}=" H31 FPI";
$resn_exch{"2H3  FPI"}=" H32 FPI"; $resn_exch{"1H4  FPI"}=" H41 FPI"; $resn_exch{"2H4  FPI"}=" H42 FPI";
$resn_exch{"1H5  FPI"}=" H51 FPI"; $resn_exch{"2H5  FPI"}=" H52 FPI"; $resn_exch{"1H6  FPI"}=" H61 FPI";
$resn_exch{"2H6  FPI"}=" H62 FPI"; $resn_exch{"1H1  FPP"}=" H11 FPP"; $resn_exch{"2H1  FPP"}="H12A FPP";
$resn_exch{"1H4  FPP"}=" H41 FPP"; $resn_exch{"2H4  FPP"}=" H42 FPP"; $resn_exch{"3H4  FPP"}=" H43 FPP";
$resn_exch{"1H5  FPP"}=" H51 FPP"; $resn_exch{"2H5  FPP"}=" H52 FPP"; $resn_exch{"1H6  FPP"}=" H61 FPP";
$resn_exch{"2H6  FPP"}=" H62 FPP"; $resn_exch{"1H10 FPP"}="H101 FPP"; $resn_exch{"2H10 FPP"}="H102 FPP";
$resn_exch{"3H10 FPP"}="H103 FPP"; $resn_exch{"1H9  FPP"}=" H91 FPP"; $resn_exch{"2H9  FPP"}=" H92 FPP";
$resn_exch{"1H11 FPP"}="H111 FPP"; $resn_exch{"2H11 FPP"}="H112 FPP"; $resn_exch{"1H14 FPP"}="H141 FPP";
$resn_exch{"2H14 FPP"}="H142 FPP"; $resn_exch{"3H14 FPP"}="H143 FPP"; $resn_exch{"1H15 FPP"}="H151 FPP";
$resn_exch{"2H15 FPP"}="H152 FPP"; $resn_exch{"3H15 FPP"}="H153 FPP"; $resn_exch{"2HOA FPP"}="HOA2 FPP";
$resn_exch{"2HOB FPP"}="HOB2 FPP"; $resn_exch{"3HOB FPP"}="HOB3 FPP"; $resn_exch{"1H1  FPR"}=" H11 FPR";
$resn_exch{"2H1  FPR"}=" H12 FPR"; $resn_exch{"3H1  FPR"}=" H13 FPR"; $resn_exch{"1H2  FPR"}=" H21 FPR";
$resn_exch{"2H2  FPR"}=" H22 FPR"; $resn_exch{"1H3  FPR"}=" H31 FPR"; $resn_exch{"2H3  FPR"}=" H32 FPR";
$resn_exch{"1H1  FPS"}=" H11 FPS"; $resn_exch{"2H1  FPS"}="H12A FPS"; $resn_exch{"1H4  FPS"}=" H41 FPS";
$resn_exch{"2H4  FPS"}=" H42 FPS"; $resn_exch{"3H4  FPS"}=" H43 FPS"; $resn_exch{"1H5  FPS"}=" H51 FPS";
$resn_exch{"2H5  FPS"}=" H52 FPS"; $resn_exch{"1H6  FPS"}=" H61 FPS"; $resn_exch{"2H6  FPS"}=" H62 FPS";
$resn_exch{"1H10 FPS"}="H101 FPS"; $resn_exch{"2H10 FPS"}="H102 FPS"; $resn_exch{"1H9  FPS"}=" H91 FPS";
$resn_exch{"2H9  FPS"}=" H92 FPS"; $resn_exch{"3H9  FPS"}=" H93 FPS"; $resn_exch{"1H11 FPS"}="H111 FPS";
$resn_exch{"2H11 FPS"}="H112 FPS"; $resn_exch{"1H14 FPS"}="H141 FPS"; $resn_exch{"2H14 FPS"}="H142 FPS";
$resn_exch{"3H14 FPS"}="H143 FPS"; $resn_exch{"1H15 FPS"}="H151 FPS"; $resn_exch{"2H15 FPS"}="H152 FPS";
$resn_exch{"3H15 FPS"}="H153 FPS"; $resn_exch{"1H11 FPT"}="H111 FPT"; $resn_exch{"2H11 FPT"}="H112 FPT";
$resn_exch{"1H12 FPT"}="H121 FPT"; $resn_exch{"2H12 FPT"}="H122 FPT"; $resn_exch{"1HA  FPT"}=" HA1 FPT";
$resn_exch{"2HA  FPT"}=" HA2 FPT"; $resn_exch{"1HB  FPT"}=" HB1 FPT"; $resn_exch{"2HB  FPT"}=" HB2 FPT";
$resn_exch{"3HB  FPT"}=" HB3 FPT"; $resn_exch{"1H4A FQP"}="H4A1 FQP"; $resn_exch{"2H4A FQP"}="H4A2 FQP";
$resn_exch{"1H6A FQP"}="H6A1 FQP"; $resn_exch{"1H7A FQP"}="H7A1 FQP"; $resn_exch{"2H7A FQP"}="H7A2 FQP";
$resn_exch{"1HO2 FQP"}="HO21 FQP"; $resn_exch{"1HO5 FQP"}="HO51 FQP"; $resn_exch{"1HO6 FQP"}="HO61 FQP";
$resn_exch{"1H11 FR0"}="H111 FR0"; $resn_exch{"2H11 FR0"}="H112 FR0"; $resn_exch{"1H28 FR0"}="H281 FR0";
$resn_exch{"2H28 FR0"}="H282 FR0"; $resn_exch{"1H29 FR0"}="H291 FR0"; $resn_exch{"2H29 FR0"}="H292 FR0";
$resn_exch{"1H15 FR1"}="H151 FR1"; $resn_exch{"2H15 FR1"}="H152 FR1"; $resn_exch{"3H15 FR1"}="H153 FR1";
$resn_exch{"1H16 FR1"}="H161 FR1"; $resn_exch{"2H16 FR1"}="H162 FR1"; $resn_exch{"3H16 FR1"}="H163 FR1";
$resn_exch{"1H21 FR1"}="H211 FR1"; $resn_exch{"2H21 FR1"}="H212 FR1"; $resn_exch{"1H22 FR1"}="H221 FR1";
$resn_exch{"2H22 FR1"}="H222 FR1"; $resn_exch{"1H23 FR1"}="H231 FR1"; $resn_exch{"2H23 FR1"}="H232 FR1";
$resn_exch{"1H29 FR1"}="H291 FR1"; $resn_exch{"2H29 FR1"}="H292 FR1"; $resn_exch{"3H29 FR1"}="H293 FR1";
$resn_exch{"1H30 FR1"}="H301 FR1"; $resn_exch{"2H30 FR1"}="H302 FR1"; $resn_exch{"3H30 FR1"}="H303 FR1";
$resn_exch{"1H37 FR1"}="H371 FR1"; $resn_exch{"2H37 FR1"}="H372 FR1"; $resn_exch{"1H10 FR2"}="H101 FR2";
$resn_exch{"2H10 FR2"}="H102 FR2"; $resn_exch{"1H14 FR2"}="H141 FR2"; $resn_exch{"2H14 FR2"}="H142 FR2";
$resn_exch{"1H18 FR2"}="H181 FR2"; $resn_exch{"2H18 FR2"}="H182 FR2"; $resn_exch{"1H19 FR2"}="H191 FR2";
$resn_exch{"2H19 FR2"}="H192 FR2"; $resn_exch{"1H10 FR3"}="H101 FR3"; $resn_exch{"2H10 FR3"}="H102 FR3";
$resn_exch{"1H14 FR3"}="H141 FR3"; $resn_exch{"2H14 FR3"}="H142 FR3"; $resn_exch{"1H18 FR3"}="H181 FR3";
$resn_exch{"2H18 FR3"}="H182 FR3"; $resn_exch{"1H19 FR3"}="H191 FR3"; $resn_exch{"2H19 FR3"}="H192 FR3";
$resn_exch{"1H10 FR4"}="H101 FR4"; $resn_exch{"2H10 FR4"}="H102 FR4"; $resn_exch{"1H14 FR4"}="H141 FR4";
$resn_exch{"2H14 FR4"}="H142 FR4"; $resn_exch{"1H18 FR4"}="H181 FR4"; $resn_exch{"2H18 FR4"}="H182 FR4";
$resn_exch{"1H19 FR4"}="H191 FR4"; $resn_exch{"2H19 FR4"}="H192 FR4"; $resn_exch{"1H4  FR4"}=" H41 FR4";
$resn_exch{"2H4  FR4"}=" H42 FR4"; $resn_exch{"1H6  FR4"}=" H61 FR4"; $resn_exch{"2H6  FR4"}=" H62 FR4";
$resn_exch{"1H10 FR5"}="H101 FR5"; $resn_exch{"2H10 FR5"}="H102 FR5"; $resn_exch{"1H14 FR5"}="H141 FR5";
$resn_exch{"2H14 FR5"}="H142 FR5"; $resn_exch{"OH15 FR5"}="H15O FR5"; $resn_exch{"1H18 FR5"}="H181 FR5";
$resn_exch{"2H18 FR5"}="H182 FR5"; $resn_exch{"1H19 FR5"}="H191 FR5"; $resn_exch{"2H19 FR5"}="H192 FR5";
$resn_exch{"1H4  FR5"}=" H41 FR5"; $resn_exch{"2H4  FR5"}=" H42 FR5"; $resn_exch{"1H6  FR5"}=" H61 FR5";
$resn_exch{"2H6  FR5"}=" H62 FR5"; $resn_exch{"1H7  FR5"}=" H71 FR5"; $resn_exch{"2H7  FR5"}=" H72 FR5";
$resn_exch{"3H7  FR5"}=" H73 FR5"; $resn_exch{"1H10 FR6"}="H101 FR6"; $resn_exch{"2H10 FR6"}="H102 FR6";
$resn_exch{"1H18 FR6"}="H181 FR6"; $resn_exch{"2H18 FR6"}="H182 FR6"; $resn_exch{"1H19 FR6"}="H191 FR6";
$resn_exch{"2H19 FR6"}="H192 FR6"; $resn_exch{"1H9  FR6"}=" H91 FR6"; $resn_exch{"2H9  FR6"}=" H92 FR6";
$resn_exch{"3H9  FR6"}=" H93 FR6"; $resn_exch{"1H10 FR7"}="H101 FR7"; $resn_exch{"2H10 FR7"}="H102 FR7";
$resn_exch{"1H18 FR7"}="H181 FR7"; $resn_exch{"2H18 FR7"}="H182 FR7"; $resn_exch{"1H2  FR7"}=" H21 FR7";
$resn_exch{"2H2  FR7"}=" H22 FR7"; $resn_exch{"1H9  FR7"}=" H91 FR7"; $resn_exch{"2H9  FR7"}=" H92 FR7";
$resn_exch{"3H9  FR7"}=" H93 FR7"; $resn_exch{"1H10 FR8"}="H101 FR8"; $resn_exch{"2H10 FR8"}="H102 FR8";
$resn_exch{"1H14 FR8"}="H141 FR8"; $resn_exch{"2H14 FR8"}="H142 FR8"; $resn_exch{"1H18 FR8"}="H181 FR8";
$resn_exch{"2H18 FR8"}="H182 FR8"; $resn_exch{"1H19 FR8"}="H191 FR8"; $resn_exch{"2H19 FR8"}="H192 FR8";
$resn_exch{"1H4  FR8"}=" H41 FR8"; $resn_exch{"2H4  FR8"}=" H42 FR8"; $resn_exch{"1H6  FR8"}=" H61 FR8";
$resn_exch{"2H6  FR8"}=" H62 FR8"; $resn_exch{"1H16 FR8"}="H161 FR8"; $resn_exch{"2H16 FR8"}="H162 FR8";
$resn_exch{"1H17 FR8"}="H171 FR8"; $resn_exch{"2H17 FR8"}="H172 FR8"; $resn_exch{"1H10 FR9"}="H101 FR9";
$resn_exch{"2H10 FR9"}="H102 FR9"; $resn_exch{"1H14 FR9"}="H141 FR9"; $resn_exch{"2H14 FR9"}="H142 FR9";
$resn_exch{"1H18 FR9"}="H181 FR9"; $resn_exch{"2H18 FR9"}="H182 FR9"; $resn_exch{"1H19 FR9"}="H191 FR9";
$resn_exch{"2H19 FR9"}="H192 FR9"; $resn_exch{"1H6  FR9"}=" H61 FR9"; $resn_exch{"2H6  FR9"}=" H62 FR9";
$resn_exch{"1H22 FRA"}="H221 FRA"; $resn_exch{"2H22 FRA"}="H222 FRA"; $resn_exch{"3H22 FRA"}="H223 FRA";
$resn_exch{"1H17 FRA"}="H171 FRA"; $resn_exch{"2H17 FRA"}="H172 FRA"; $resn_exch{"1H16 FRA"}="H161 FRA";
$resn_exch{"2H16 FRA"}="H162 FRA"; $resn_exch{"1H14 FRA"}="H141 FRA"; $resn_exch{"2H14 FRA"}="H142 FRA";
$resn_exch{"1H13 FRA"}="H131 FRA"; $resn_exch{"2H13 FRA"}="H132 FRA"; $resn_exch{"1H19 FRA"}="H191 FRA";
$resn_exch{"2H19 FRA"}="H192 FRA"; $resn_exch{"1HN3 FRB"}="HN31 FRB"; $resn_exch{"2HN3 FRB"}="HN32 FRB";
$resn_exch{"1H7  FRB"}=" H71 FRB"; $resn_exch{"2H7  FRB"}=" H72 FRB"; $resn_exch{"1H8  FRB"}=" H81 FRB";
$resn_exch{"2H8  FRB"}=" H82 FRB"; $resn_exch{"1H9  FRB"}=" H91 FRB"; $resn_exch{"2H9  FRB"}=" H92 FRB";
$resn_exch{"1H10 FRB"}="H101 FRB"; $resn_exch{"2H10 FRB"}="H102 FRB"; $resn_exch{"1H11 FRB"}="H111 FRB";
$resn_exch{"2H11 FRB"}="H112 FRB"; $resn_exch{"1H15 FRB"}="H151 FRB"; $resn_exch{"2H15 FRB"}="H152 FRB";
$resn_exch{"1H19 FRB"}="H191 FRB"; $resn_exch{"2H19 FRB"}="H192 FRB"; $resn_exch{"1H20 FRB"}="H201 FRB";
$resn_exch{"2H20 FRB"}="H202 FRB"; $resn_exch{"1H23 FRB"}="H231 FRB"; $resn_exch{"2H23 FRB"}="H232 FRB";
$resn_exch{"1H22 FRB"}="H221 FRB"; $resn_exch{"2H22 FRB"}="H222 FRB"; $resn_exch{"1H10 FRC"}="H101 FRC";
$resn_exch{"2H10 FRC"}="H102 FRC"; $resn_exch{"1H18 FRC"}="H181 FRC"; $resn_exch{"2H18 FRC"}="H182 FRC";
$resn_exch{"1H2  FRC"}=" H21 FRC"; $resn_exch{"2H2  FRC"}=" H22 FRC"; $resn_exch{"1H9  FRC"}=" H91 FRC";
$resn_exch{"2H9  FRC"}=" H92 FRC"; $resn_exch{"3H9  FRC"}=" H93 FRC"; $resn_exch{" HN2 FRD"}=" H2  FRD";
$resn_exch{"1H   FRD"}=" HC1 FRD"; $resn_exch{"2H   FRD"}=" HC2 FRD"; $resn_exch{"3H   FRD"}=" HC3 FRD";
$resn_exch{"1HB  FRD"}=" HB2 FRD"; $resn_exch{"2HB  FRD"}=" HB3 FRD"; $resn_exch{"1HN6 FRE"}="HN61 FRE";
$resn_exch{"2HN6 FRE"}="HN62 FRE"; $resn_exch{" H1* FRE"}=" H1' FRE"; $resn_exch{" H2* FRE"}=" H2' FRE";
$resn_exch{" H3* FRE"}=" H3' FRE"; $resn_exch{" H4* FRE"}=" H4' FRE"; $resn_exch{"1H5* FRE"}="H5'1 FRE";
$resn_exch{"2H5* FRE"}="H5'2 FRE"; $resn_exch{"1H12 FRE"}="H121 FRE"; $resn_exch{"2H12 FRE"}="H122 FRE";
$resn_exch{"1H1  FRE"}=" H11 FRE"; $resn_exch{"2H1  FRE"}=" H12 FRE"; $resn_exch{"3H1  FRE"}=" H13 FRE";
$resn_exch{"1H14 FRE"}="H141 FRE"; $resn_exch{"2H14 FRE"}="H142 FRE"; $resn_exch{"3H14 FRE"}="H143 FRE";
$resn_exch{"1H7  FRE"}=" H71 FRE"; $resn_exch{"2H7  FRE"}=" H72 FRE"; $resn_exch{"1H6  FRE"}=" H61 FRE";
$resn_exch{"2H6  FRE"}=" H62 FRE"; $resn_exch{"1H3  FRE"}=" H31 FRE"; $resn_exch{"2H3  FRE"}=" H32 FRE";
$resn_exch{"1H2  FRE"}=" H21 FRE"; $resn_exch{"2H2  FRE"}=" H22 FRE"; $resn_exch{"H11  FRE"}="H111 FRE";
$resn_exch{"H12  FRE"}="H112 FRE"; $resn_exch{"1H13 FRE"}="H131 FRE"; $resn_exch{"2H13 FRE"}="H132 FRE";
$resn_exch{"3H13 FRE"}="H133 FRE"; $resn_exch{"1HN  FRF"}=" HN1 FRF"; $resn_exch{"2HN  FRF"}=" HN2 FRF";
$resn_exch{"1HTC FRF"}="HTC1 FRF"; $resn_exch{"2HTC FRF"}="HTC2 FRF"; $resn_exch{"1HBC FRF"}="HBC1 FRF";
$resn_exch{"2HBC FRF"}="HBC2 FRF"; $resn_exch{"1HB1 FRF"}="HB11 FRF"; $resn_exch{"2HB1 FRF"}="HB12 FRF";
$resn_exch{"1H32 FRG"}="H321 FRG"; $resn_exch{"2H32 FRG"}="H322 FRG"; $resn_exch{"1H30 FRG"}="H301 FRG";
$resn_exch{"2H30 FRG"}="H302 FRG"; $resn_exch{"1H28 FRG"}="H281 FRG"; $resn_exch{"2H28 FRG"}="H282 FRG";
$resn_exch{"1H27 FRG"}="H271 FRG"; $resn_exch{"2H27 FRG"}="H272 FRG"; $resn_exch{"1H26 FRG"}="H261 FRG";
$resn_exch{"2H26 FRG"}="H262 FRG"; $resn_exch{"1H23 FRG"}="H231 FRG"; $resn_exch{"2H23 FRG"}="H232 FRG";
$resn_exch{"1H20 FRG"}="H201 FRG"; $resn_exch{"2H20 FRG"}="H202 FRG"; $resn_exch{"3H20 FRG"}="H203 FRG";
$resn_exch{"1H15 FRG"}="H151 FRG"; $resn_exch{"2H15 FRG"}="H152 FRG"; $resn_exch{"1H32 FRH"}="H321 FRH";
$resn_exch{"2H32 FRH"}="H322 FRH"; $resn_exch{"1H38 FRH"}="H381 FRH"; $resn_exch{"2H38 FRH"}="H382 FRH";
$resn_exch{"3H38 FRH"}="H383 FRH"; $resn_exch{"1H37 FRH"}="H371 FRH"; $resn_exch{"2H37 FRH"}="H372 FRH";
$resn_exch{"3H37 FRH"}="H373 FRH"; $resn_exch{"1HN4 FRH"}="HN41 FRH"; $resn_exch{"2HN4 FRH"}="HN42 FRH";
$resn_exch{"1H28 FRH"}="H281 FRH"; $resn_exch{"2H28 FRH"}="H282 FRH"; $resn_exch{"1H22 FRH"}="H221 FRH";
$resn_exch{"2H22 FRH"}="H222 FRH"; $resn_exch{"1H15 FRH"}="H151 FRH"; $resn_exch{"2H15 FRH"}="H152 FRH";
$resn_exch{"1H27 FRH"}="H271 FRH"; $resn_exch{"2H27 FRH"}="H272 FRH"; $resn_exch{"1H29 FRH"}="H291 FRH";
$resn_exch{"2H29 FRH"}="H292 FRH"; $resn_exch{"1H34 FRH"}="H341 FRH"; $resn_exch{"2H34 FRH"}="H342 FRH";
$resn_exch{"3H34 FRH"}="H343 FRH"; $resn_exch{"1H35 FRH"}="H351 FRH"; $resn_exch{"2H35 FRH"}="H352 FRH";
$resn_exch{"1H29 FRI"}="H291 FRI"; $resn_exch{"2H29 FRI"}="H292 FRI"; $resn_exch{"1H41 FRI"}="H411 FRI";
$resn_exch{"2H41 FRI"}="H412 FRI"; $resn_exch{"1H37 FRI"}="H371 FRI"; $resn_exch{"2H37 FRI"}="H372 FRI";
$resn_exch{"3H37 FRI"}="H373 FRI"; $resn_exch{"1H36 FRI"}="H361 FRI"; $resn_exch{"2H36 FRI"}="H362 FRI";
$resn_exch{"3H36 FRI"}="H363 FRI"; $resn_exch{"1HN4 FRI"}="HN41 FRI"; $resn_exch{"2HN4 FRI"}="HN42 FRI";
$resn_exch{"1H31 FRI"}="H311 FRI"; $resn_exch{"2H31 FRI"}="H312 FRI"; $resn_exch{"1H39 FRI"}="H391 FRI";
$resn_exch{"2H39 FRI"}="H392 FRI"; $resn_exch{"1H21 FRI"}="H211 FRI"; $resn_exch{"2H21 FRI"}="H212 FRI";
$resn_exch{"1H43 FRI"}="H431 FRI"; $resn_exch{"2H43 FRI"}="H432 FRI"; $resn_exch{"1H38 FRI"}="H381 FRI";
$resn_exch{"2H38 FRI"}="H382 FRI"; $resn_exch{"3H38 FRI"}="H383 FRI"; $resn_exch{"1H40 FRI"}="H401 FRI";
$resn_exch{"2H40 FRI"}="H402 FRI"; $resn_exch{"1H42 FRI"}="H421 FRI"; $resn_exch{"2H42 FRI"}="H422 FRI";
$resn_exch{"3H42 FRI"}="H423 FRI"; $resn_exch{"1H13 FRJ"}="H131 FRJ"; $resn_exch{"2H13 FRJ"}="H132 FRJ";
$resn_exch{"1H14 FRJ"}="H141 FRJ"; $resn_exch{"2H14 FRJ"}="H142 FRJ"; $resn_exch{"3H14 FRJ"}="H143 FRJ";
$resn_exch{"1H9  FRK"}=" H91 FRK"; $resn_exch{"2H9  FRK"}=" H92 FRK"; $resn_exch{"1H11 FRK"}="H111 FRK";
$resn_exch{"2H11 FRK"}="H112 FRK"; $resn_exch{"1H12 FRK"}="H121 FRK"; $resn_exch{"2H12 FRK"}="H122 FRK";
$resn_exch{"1H15 FRK"}="H151 FRK"; $resn_exch{"2H15 FRK"}="H152 FRK"; $resn_exch{"1H18 FRK"}="H181 FRK";
$resn_exch{"2H18 FRK"}="H182 FRK"; $resn_exch{"3H18 FRK"}="H183 FRK"; $resn_exch{"1H17 FRL"}="H171 FRL";
$resn_exch{"2H17 FRL"}="H172 FRL"; $resn_exch{"1H18 FRL"}="H181 FRL"; $resn_exch{"2H18 FRL"}="H182 FRL";
$resn_exch{"1H23 FRL"}="H231 FRL"; $resn_exch{"2H23 FRL"}="H232 FRL"; $resn_exch{"3H23 FRL"}="H233 FRL";
$resn_exch{"1H30 FRL"}="H301 FRL"; $resn_exch{"2H30 FRL"}="H302 FRL"; $resn_exch{"1H17 FRM"}="H171 FRM";
$resn_exch{"2H17 FRM"}="H172 FRM"; $resn_exch{"1H20 FRM"}="H201 FRM"; $resn_exch{"2H20 FRM"}="H202 FRM";
$resn_exch{"1H23 FRM"}="H231 FRM"; $resn_exch{"2H23 FRM"}="H232 FRM"; $resn_exch{"1H29 FRM"}="H291 FRM";
$resn_exch{"2H29 FRM"}="H292 FRM"; $resn_exch{"1H32 FRM"}="H321 FRM"; $resn_exch{"2H32 FRM"}="H322 FRM";
$resn_exch{"1H33 FRM"}="H331 FRM"; $resn_exch{"2H33 FRM"}="H332 FRM"; $resn_exch{"1H7  FRM"}=" H71 FRM";
$resn_exch{"2H7  FRM"}=" H72 FRM"; $resn_exch{"3H7  FRM"}=" H73 FRM"; $resn_exch{"1HN  FRN"}="HN1A FRN";
$resn_exch{"2HN  FRN"}="HN2A FRN"; $resn_exch{"3HN  FRN"}="HN3A FRN"; $resn_exch{"1HG2 FRN"}="HG21 FRN";
$resn_exch{"2HG2 FRN"}="HG22 FRN"; $resn_exch{"3HG2 FRN"}="HG23 FRN"; $resn_exch{"1H31 FRN"}="H311 FRN";
$resn_exch{"2H31 FRN"}="H312 FRN"; $resn_exch{"3H31 FRN"}="H313 FRN"; $resn_exch{"1H22 FRN"}="H221 FRN";
$resn_exch{"2H22 FRN"}="H222 FRN"; $resn_exch{"1HD1 FRN"}="HD11 FRN"; $resn_exch{"2HD1 FRN"}="HD12 FRN";
$resn_exch{"3HD1 FRN"}="HD13 FRN"; $resn_exch{"1HD2 FRN"}="HD21 FRN"; $resn_exch{"2HD2 FRN"}="HD22 FRN";
$resn_exch{"3HD2 FRN"}="HD23 FRN"; $resn_exch{"1H23 FRN"}="H231 FRN"; $resn_exch{"2H23 FRN"}="H232 FRN";
$resn_exch{"1H32 FRN"}="H321 FRN"; $resn_exch{"2H32 FRN"}="H322 FRN"; $resn_exch{"1HD  FRN"}=" HD1 FRN";
$resn_exch{"2HD  FRN"}=" HD2 FRN"; $resn_exch{"1HE  FRN"}="HE1A FRN"; $resn_exch{"2HE  FRN"}="HE2A FRN";
$resn_exch{"1H24 FRN"}="H241 FRN"; $resn_exch{"2H24 FRN"}="H242 FRN"; $resn_exch{"1H25 FRN"}="H251 FRN";
$resn_exch{"2H25 FRN"}="H252 FRN"; $resn_exch{"1H34 FRN"}="H341 FRN"; $resn_exch{"2H34 FRN"}="H342 FRN";
$resn_exch{"1H42 FRN"}="H421 FRN"; $resn_exch{"2H42 FRN"}="H422 FRN"; $resn_exch{"1H26 FRN"}="H261 FRN";
$resn_exch{"2H26 FRN"}="H262 FRN"; $resn_exch{"1H27 FRN"}="H271 FRN"; $resn_exch{"2H27 FRN"}="H272 FRN";
$resn_exch{"1H28 FRN"}="H281 FRN"; $resn_exch{"2H28 FRN"}="H282 FRN"; $resn_exch{"1H29 FRN"}="H291 FRN";
$resn_exch{"2H29 FRN"}="H292 FRN"; $resn_exch{"1H44 FRN"}="H441 FRN"; $resn_exch{"2H44 FRN"}="H442 FRN";
$resn_exch{"1H8  FRO"}=" H81 FRO"; $resn_exch{"2H8  FRO"}=" H82 FRO"; $resn_exch{"1H9  FRO"}=" H91 FRO";
$resn_exch{"2H9  FRO"}=" H92 FRO"; $resn_exch{"1H10 FRO"}="H101 FRO"; $resn_exch{"2H10 FRO"}="H102 FRO";
$resn_exch{"1H18 FRO"}="H181 FRO"; $resn_exch{"2H18 FRO"}="H182 FRO"; $resn_exch{"3H18 FRO"}="H183 FRO";
$resn_exch{"1H17 FRP"}="H171 FRP"; $resn_exch{"2H17 FRP"}="H172 FRP"; $resn_exch{"1H13 FRP"}="H131 FRP";
$resn_exch{"2H13 FRP"}="H132 FRP"; $resn_exch{"3H13 FRP"}="H133 FRP"; $resn_exch{"1H14 FRP"}="H141 FRP";
$resn_exch{"2H14 FRP"}="H142 FRP"; $resn_exch{"3H14 FRP"}="H143 FRP"; $resn_exch{"1H11 FRP"}="H111 FRP";
$resn_exch{"2H11 FRP"}="H112 FRP"; $resn_exch{"1H10 FRP"}="H101 FRP"; $resn_exch{"2H10 FRP"}="H102 FRP";
$resn_exch{"1H9  FRP"}=" H91 FRP"; $resn_exch{"2H9  FRP"}=" H92 FRP"; $resn_exch{"1H8  FRP"}=" H81 FRP";
$resn_exch{"2H8  FRP"}=" H82 FRP"; $resn_exch{"1H21 FRP"}="H211 FRP"; $resn_exch{"2H21 FRP"}="H212 FRP";
$resn_exch{"3H21 FRP"}="H213 FRP"; $resn_exch{"1H18 FRP"}="H181 FRP"; $resn_exch{"2H18 FRP"}="H182 FRP";
$resn_exch{"1HN4 FRP"}="HN41 FRP"; $resn_exch{"2HN4 FRP"}="HN42 FRP"; $resn_exch{"1H16 FRQ"}="H161 FRQ";
$resn_exch{"2H16 FRQ"}="H162 FRQ"; $resn_exch{"1H17 FRQ"}="H171 FRQ"; $resn_exch{"2H17 FRQ"}="H172 FRQ";
$resn_exch{"1H20 FRQ"}="H201 FRQ"; $resn_exch{"2H20 FRQ"}="H202 FRQ"; $resn_exch{"1H23 FRQ"}="H231 FRQ";
$resn_exch{"2H23 FRQ"}="H232 FRQ"; $resn_exch{"1H29 FRQ"}="H291 FRQ"; $resn_exch{"2H29 FRQ"}="H292 FRQ";
$resn_exch{"1H30 FRQ"}="H301 FRQ"; $resn_exch{"2H30 FRQ"}="H302 FRQ"; $resn_exch{"1H33 FRQ"}="H331 FRQ";
$resn_exch{"2H33 FRQ"}="H332 FRQ"; $resn_exch{"1H59 FRR"}="H591 FRR"; $resn_exch{"2H59 FRR"}="H592 FRR";
$resn_exch{"3H59 FRR"}="H593 FRR"; $resn_exch{"1H34 FRR"}="H341 FRR"; $resn_exch{"2H34 FRR"}="H342 FRR";
$resn_exch{"1H18 FRR"}="H181 FRR"; $resn_exch{"2H18 FRR"}="H182 FRR"; $resn_exch{"1H62 FRR"}="H621 FRR";
$resn_exch{"2H62 FRR"}="H622 FRR"; $resn_exch{"3H62 FRR"}="H623 FRR"; $resn_exch{"1H1  FRU"}=" H11 FRU";
$resn_exch{"2H1  FRU"}=" H12 FRU"; $resn_exch{"1H6  FRU"}=" H61 FRU"; $resn_exch{"2H6  FRU"}=" H62 FRU";
$resn_exch{"1H24 FRX"}="H241 FRX"; $resn_exch{"2H24 FRX"}="H242 FRX"; $resn_exch{"1H25 FRX"}="H251 FRX";
$resn_exch{"2H25 FRX"}="H252 FRX"; $resn_exch{"1H21 FRX"}="H211 FRX"; $resn_exch{"2H21 FRX"}="H212 FRX";
$resn_exch{"1H15 FRX"}="H151 FRX"; $resn_exch{"2H15 FRX"}="H152 FRX"; $resn_exch{"1H14 FRY"}="H141 FRY";
$resn_exch{"2H14 FRY"}="H142 FRY"; $resn_exch{"1H20 FRY"}="H201 FRY"; $resn_exch{"2H20 FRY"}="H202 FRY";
$resn_exch{"1H23 FRY"}="H231 FRY"; $resn_exch{"2H23 FRY"}="H232 FRY"; $resn_exch{"1H30 FRZ"}="H301 FRZ";
$resn_exch{"2H30 FRZ"}="H302 FRZ"; $resn_exch{"CH01 FSB"}="H01C FSB"; $resn_exch{"CH02 FSB"}="H02C FSB";
$resn_exch{"CH04 FSB"}="H04C FSB"; $resn_exch{"CH05 FSB"}="H05C FSB"; $resn_exch{"NH09 FSB"}="H09N FSB";
$resn_exch{"1H10 FSB"}="H101 FSB"; $resn_exch{"2H10 FSB"}="H102 FSB"; $resn_exch{"1HNP FSB"}="HNP1 FSB";
$resn_exch{"2HNP FSB"}="HNP2 FSB"; $resn_exch{"CH16 FSB"}="H16C FSB"; $resn_exch{"CH17 FSB"}="H17C FSB";
$resn_exch{"CH18 FSB"}="H18C FSB"; $resn_exch{"CH19 FSB"}="H19C FSB"; $resn_exch{"1H18 FSC"}="H181 FSC";
$resn_exch{"2H18 FSC"}="H182 FSC"; $resn_exch{"3H18 FSC"}="H183 FSC"; $resn_exch{"1H17 FSC"}="H171 FSC";
$resn_exch{"2H17 FSC"}="H172 FSC"; $resn_exch{"1H36 FSC"}="H361 FSC"; $resn_exch{"2H36 FSC"}="H362 FSC";
$resn_exch{"3H36 FSC"}="H363 FSC"; $resn_exch{"1H10 FSC"}="H101 FSC"; $resn_exch{"2H10 FSC"}="H102 FSC";
$resn_exch{"1H7C FSC"}="H7C1 FSC"; $resn_exch{"2H7C FSC"}="H7C2 FSC"; $resn_exch{"3H7C FSC"}="H7C3 FSC";
$resn_exch{"1H27 FSC"}="H271 FSC"; $resn_exch{"2H27 FSC"}="H272 FSC"; $resn_exch{"1H38 FSC"}="H381 FSC";
$resn_exch{"2H38 FSC"}="H382 FSC"; $resn_exch{"3H38 FSC"}="H383 FSC"; $resn_exch{"1H26 FSC"}="H261 FSC";
$resn_exch{"2H26 FSC"}="H262 FSC"; $resn_exch{"1H25 FSC"}="H251 FSC"; $resn_exch{"2H25 FSC"}="H252 FSC";
$resn_exch{"1H23 FSC"}="H231 FSC"; $resn_exch{"2H23 FSC"}="H232 FSC"; $resn_exch{"3H23 FSC"}="H233 FSC";
$resn_exch{"1H42 FSC"}="H421 FSC"; $resn_exch{"2H42 FSC"}="H422 FSC"; $resn_exch{"3H42 FSC"}="H423 FSC";
$resn_exch{"1H35 FSC"}="H351 FSC"; $resn_exch{"2H35 FSC"}="H352 FSC"; $resn_exch{"1H47 FSC"}="H471 FSC";
$resn_exch{"2H47 FSC"}="H472 FSC"; $resn_exch{"3H47 FSC"}="H473 FSC"; $resn_exch{"1H46 FSC"}="H461 FSC";
$resn_exch{"2H46 FSC"}="H462 FSC"; $resn_exch{"3H46 FSC"}="H463 FSC"; $resn_exch{"1H48 FSC"}="H481 FSC";
$resn_exch{"2H48 FSC"}="H482 FSC"; $resn_exch{"1H11 FSI"}="H111 FSI"; $resn_exch{"2H11 FSI"}="H112 FSI";
$resn_exch{"3H11 FSI"}="H113 FSI"; $resn_exch{"1H9  FSI"}=" H91 FSI"; $resn_exch{"2H9  FSI"}=" H92 FSI";
$resn_exch{"1HAA FSM"}="HAA1 FSM"; $resn_exch{"2HAA FSM"}="HAA2 FSM"; $resn_exch{"1HA' FSM"}="HA'1 FSM";
$resn_exch{"2HA' FSM"}="HA'2 FSM"; $resn_exch{"3HA' FSM"}="HA'3 FSM"; $resn_exch{"1HB' FSM"}="HB'1 FSM";
$resn_exch{"2HB' FSM"}="HB'2 FSM"; $resn_exch{"1HBA FSM"}="HBA1 FSM"; $resn_exch{"2HBA FSM"}="HBA2 FSM";
$resn_exch{"3HBA FSM"}="HBA3 FSM"; $resn_exch{"1H23 FSN"}="H231 FSN"; $resn_exch{"2H23 FSN"}="H232 FSN";
$resn_exch{"1H12 FSN"}="H121 FSN"; $resn_exch{"2H12 FSN"}="H122 FSN"; $resn_exch{"1H13 FSN"}="H131 FSN";
$resn_exch{"2H13 FSN"}="H132 FSN"; $resn_exch{"1H14 FSN"}="H141 FSN"; $resn_exch{"2H14 FSN"}="H142 FSN";
$resn_exch{"1HN1 FSN"}="HN11 FSN"; $resn_exch{"2HN1 FSN"}="HN12 FSN"; $resn_exch{"1HN2 FSN"}="HN21 FSN";
$resn_exch{"2HN2 FSN"}="HN22 FSN"; $resn_exch{"1H1  FSP"}=" H11 FSP"; $resn_exch{"2H1  FSP"}=" H12 FSP";
$resn_exch{"3H1  FSP"}=" H13 FSP"; $resn_exch{"1H2  FSP"}=" H21 FSP"; $resn_exch{"2H2  FSP"}=" H22 FSP";
$resn_exch{"1H3  FSP"}=" H31 FSP"; $resn_exch{"2H3  FSP"}="H32A FSP"; $resn_exch{"1H4  FSP"}=" H41 FSP";
$resn_exch{"2H4  FSP"}=" H42 FSP"; $resn_exch{"1H10 FSP"}="H101 FSP"; $resn_exch{"2H10 FSP"}="H102 FSP";
$resn_exch{"1H12 FSP"}="H121 FSP"; $resn_exch{"2H12 FSP"}="H122 FSP"; $resn_exch{"1H20 FSP"}="H201 FSP";
$resn_exch{"2H20 FSP"}="H202 FSP"; $resn_exch{"1H21 FSP"}="H211 FSP"; $resn_exch{"2H21 FSP"}="H212 FSP";
$resn_exch{"1H22 FSP"}="H221 FSP"; $resn_exch{"2H22 FSP"}="H222 FSP"; $resn_exch{"1HN  FT6"}=" H   FT6";
$resn_exch{"2HN  FT6"}=" H2  FT6"; $resn_exch{"1HB  FT6"}=" HB2 FT6"; $resn_exch{"2HB  FT6"}=" HB3 FT6";
$resn_exch{"HNE1 FT6"}=" HE1 FT6"; $resn_exch{"1H34 FTB"}="H341 FTB"; $resn_exch{"2H34 FTB"}="H342 FTB";
$resn_exch{"3H34 FTB"}="H343 FTB"; $resn_exch{"1H24 FTB"}="H241 FTB"; $resn_exch{"2H24 FTB"}="H242 FTB";
$resn_exch{"1H12 FTB"}="H121 FTB"; $resn_exch{"2H12 FTB"}="H122 FTB"; $resn_exch{"1H13 FTB"}="H131 FTB";
$resn_exch{"2H13 FTB"}="H132 FTB"; $resn_exch{"1H14 FTB"}="H141 FTB"; $resn_exch{"2H14 FTB"}="H142 FTB";
$resn_exch{"1H15 FTB"}="H151 FTB"; $resn_exch{"2H15 FTB"}="H152 FTB"; $resn_exch{"3H15 FTB"}="H153 FTB";
$resn_exch{"1H16 FTB"}="H161 FTB"; $resn_exch{"2H16 FTB"}="H162 FTB"; $resn_exch{"1H11 FTC"}="H111 FTC";
$resn_exch{"2H11 FTC"}="H112 FTC"; $resn_exch{"1H12 FTC"}="H121 FTC"; $resn_exch{"2H12 FTC"}="H122 FTC";
$resn_exch{"1HA  FTC"}=" HA1 FTC"; $resn_exch{"2HA  FTC"}=" HA2 FTC"; $resn_exch{"1HB  FTC"}=" HB1 FTC";
$resn_exch{"2HB  FTC"}=" HB2 FTC"; $resn_exch{"3HB  FTC"}=" HB3 FTC"; $resn_exch{"1HC1 FTH"}="HC11 FTH";
$resn_exch{"2HC1 FTH"}="HC1A FTH"; $resn_exch{"3HC1 FTH"}="HC1B FTH"; $resn_exch{"1HC7 FTH"}="HC71 FTH";
$resn_exch{"2HC7 FTH"}="HC72 FTH"; $resn_exch{"1H10 FTH"}="H101 FTH"; $resn_exch{"2H10 FTH"}="H102 FTH";
$resn_exch{"1H22 FTH"}="H221 FTH"; $resn_exch{"2H22 FTH"}="H222 FTH"; $resn_exch{"1H23 FTH"}="H231 FTH";
$resn_exch{"2H23 FTH"}="H232 FTH"; $resn_exch{"1H26 FTH"}="H261 FTH"; $resn_exch{"2H26 FTH"}="H262 FTH";
$resn_exch{"1HC1 FTI"}="HC1A FTI"; $resn_exch{"2HC1 FTI"}="HC12 FTI"; $resn_exch{"1HC2 FTI"}="HC2A FTI";
$resn_exch{"2HC2 FTI"}="HC22 FTI"; $resn_exch{"1HC3 FTI"}="HC31 FTI"; $resn_exch{"2HC3 FTI"}="HC32 FTI";
$resn_exch{"1H16 FTI"}="H161 FTI"; $resn_exch{"2H16 FTI"}="H162 FTI"; $resn_exch{"1H20 FTI"}="H201 FTI";
$resn_exch{"2H20 FTI"}="H202 FTI"; $resn_exch{"1HC8 FTL"}="HC81 FTL"; $resn_exch{"2HC8 FTL"}="HC82 FTL";
$resn_exch{"1H15 FTL"}="H151 FTL"; $resn_exch{"2H15 FTL"}="H152 FTL"; $resn_exch{"3H15 FTL"}="H153 FTL";
$resn_exch{"1H18 FTL"}="H181 FTL"; $resn_exch{"2H18 FTL"}="H182 FTL"; $resn_exch{"1H4A FTP"}="H4A1 FTP";
$resn_exch{"2H4A FTP"}="H4A2 FTP"; $resn_exch{"1H6A FTP"}="H6A1 FTP"; $resn_exch{"1H7A FTP"}="H7A1 FTP";
$resn_exch{"2H7A FTP"}="H7A2 FTP"; $resn_exch{"1H2  FTP"}=" H21 FTP"; $resn_exch{"1HM4 FTP"}="HM41 FTP";
$resn_exch{"2HM4 FTP"}="HM42 FTP"; $resn_exch{"3HM4 FTP"}="HM43 FTP"; $resn_exch{"1H6  FTP"}=" H61 FTP";
$resn_exch{"2H6  FTP"}=" H62 FTP"; $resn_exch{"1H7  FTP"}=" H71 FTP"; $resn_exch{"2H7  FTP"}=" H72 FTP";
$resn_exch{"1HN  FTR"}=" H   FTR"; $resn_exch{"2HN  FTR"}=" H2  FTR"; $resn_exch{"1HB  FTR"}=" HB2 FTR";
$resn_exch{"2HB  FTR"}=" HB3 FTR"; $resn_exch{" HNE FTR"}=" HE1 FTR"; $resn_exch{"1H1  FTS"}=" H11 FTS";
$resn_exch{"2H1  FTS"}=" H12 FTS"; $resn_exch{"1H10 FTS"}="H101 FTS"; $resn_exch{"2H10 FTS"}="H102 FTS";
$resn_exch{"1H3  FTS"}=" H31 FTS"; $resn_exch{"2H3  FTS"}=" H32 FTS"; $resn_exch{"1H11 FTS"}="H111 FTS";
$resn_exch{"2H11 FTS"}="H112 FTS"; $resn_exch{"1H12 FTS"}="H121 FTS"; $resn_exch{"2H12 FTS"}="H122 FTS";
$resn_exch{"1H2  FTT"}=" H21 FTT"; $resn_exch{"2H2  FTT"}=" H22 FTT"; $resn_exch{"1H4  FTT"}=" H41 FTT";
$resn_exch{"2H4  FTT"}=" H42 FTT"; $resn_exch{"1H5  FTT"}=" H51 FTT"; $resn_exch{"2H5  FTT"}=" H52 FTT";
$resn_exch{"1H6  FTT"}=" H61 FTT"; $resn_exch{"2H6  FTT"}=" H62 FTT"; $resn_exch{"1H7  FTT"}=" H71 FTT";
$resn_exch{"2H7  FTT"}=" H72 FTT"; $resn_exch{"1H8  FTT"}=" H81 FTT"; $resn_exch{"2H8  FTT"}=" H82 FTT";
$resn_exch{"1H9  FTT"}=" H91 FTT"; $resn_exch{"2H9  FTT"}=" H92 FTT"; $resn_exch{"1H10 FTT"}="H101 FTT";
$resn_exch{"2H10 FTT"}="H102 FTT"; $resn_exch{"1H11 FTT"}="H111 FTT"; $resn_exch{"2H11 FTT"}="H112 FTT";
$resn_exch{"1H12 FTT"}="H121 FTT"; $resn_exch{"2H12 FTT"}="H122 FTT"; $resn_exch{"1H13 FTT"}="H131 FTT";
$resn_exch{"2H13 FTT"}="H132 FTT"; $resn_exch{"1H14 FTT"}="H141 FTT"; $resn_exch{"2H14 FTT"}="H142 FTT";
$resn_exch{"3H14 FTT"}="H143 FTT"; $resn_exch{"1HN  FTY"}=" H   FTY"; $resn_exch{"2HN  FTY"}=" H2  FTY";
$resn_exch{"1HB  FTY"}=" HB2 FTY"; $resn_exch{"2HB  FTY"}=" HB3 FTY"; $resn_exch{"PHO2 FTY"}="HO2P FTY";
$resn_exch{"PHO3 FTY"}="HO3P FTY"; $resn_exch{"1H1C FU4"}="H1C1 FU4"; $resn_exch{"2H1C FU4"}="H1C2 FU4";
$resn_exch{"1H6C FU4"}="H6C1 FU4"; $resn_exch{"2H6C FU4"}="H6C2 FU4"; $resn_exch{"3H6C FU4"}="H6C3 FU4";
$resn_exch{"1H1  FUA"}="H11A FUA"; $resn_exch{"2H1  FUA"}=" H12 FUA"; $resn_exch{"1H2  FUA"}=" H21 FUA";
$resn_exch{"2H2  FUA"}=" H22 FUA"; $resn_exch{"1H6  FUA"}=" H61 FUA"; $resn_exch{"2H6  FUA"}=" H62 FUA";
$resn_exch{"1H7  FUA"}=" H71 FUA"; $resn_exch{"2H7  FUA"}=" H72 FUA"; $resn_exch{"1H12 FUA"}="H121 FUA";
$resn_exch{"2H12 FUA"}="H122 FUA"; $resn_exch{"1H15 FUA"}="H151 FUA"; $resn_exch{"2H15 FUA"}="H152 FUA";
$resn_exch{"1H18 FUA"}="H181 FUA"; $resn_exch{"2H18 FUA"}="H182 FUA"; $resn_exch{"3H18 FUA"}="H183 FUA";
$resn_exch{"1H19 FUA"}="H191 FUA"; $resn_exch{"2H19 FUA"}="H192 FUA"; $resn_exch{"3H19 FUA"}="H193 FUA";
$resn_exch{"1H20 FUA"}="H201 FUA"; $resn_exch{"2H20 FUA"}="H202 FUA"; $resn_exch{"3H20 FUA"}="H203 FUA";
$resn_exch{"1H21 FUA"}="H211 FUA"; $resn_exch{"2H21 FUA"}="H212 FUA"; $resn_exch{"3H21 FUA"}="H213 FUA";
$resn_exch{"1H23 FUA"}="H231 FUA"; $resn_exch{"2H23 FUA"}="H232 FUA"; $resn_exch{"1H24 FUA"}="H241 FUA";
$resn_exch{"2H24 FUA"}="H242 FUA"; $resn_exch{"1H27 FUA"}="H271 FUA"; $resn_exch{"2H27 FUA"}="H272 FUA";
$resn_exch{"3H27 FUA"}="H273 FUA"; $resn_exch{"1H28 FUA"}="H281 FUA"; $resn_exch{"2H28 FUA"}="H282 FUA";
$resn_exch{"3H28 FUA"}="H283 FUA"; $resn_exch{"1H32 FUA"}="H321 FUA"; $resn_exch{"2H32 FUA"}="H322 FUA";
$resn_exch{"3H32 FUA"}="H323 FUA"; $resn_exch{"1H6  FUC"}=" H61 FUC"; $resn_exch{"2H6  FUC"}=" H62 FUC";
$resn_exch{"3H6  FUC"}=" H63 FUC"; $resn_exch{"1H1  FUD"}=" H11 FUD"; $resn_exch{"2H1  FUD"}=" H12 FUD";
$resn_exch{"1H6  FUD"}=" H61 FUD"; $resn_exch{"2H6  FUD"}=" H62 FUD"; $resn_exch{"1H6C FUF"}="H6C1 FUF";
$resn_exch{"2H6C FUF"}="H6C2 FUF"; $resn_exch{"3H6C FUF"}="H6C3 FUF"; $resn_exch{"1H5  FUG"}=" H51 FUG";
$resn_exch{"2H5  FUG"}=" H52 FUG"; $resn_exch{"1H6  FUG"}=" H61 FUG"; $resn_exch{"2H6  FUG"}=" H62 FUG";
$resn_exch{"1H11 FUG"}="H111 FUG"; $resn_exch{"2H11 FUG"}="H112 FUG"; $resn_exch{"3H11 FUG"}="H113 FUG";
$resn_exch{"1H23 FUG"}="H231 FUG"; $resn_exch{"2H23 FUG"}="H232 FUG"; $resn_exch{"1H2A FUG"}="H2A1 FUG";
$resn_exch{"2H2A FUG"}="H2A2 FUG"; $resn_exch{"3H2A FUG"}="H2A3 FUG"; $resn_exch{"1H2B FUG"}="H2B1 FUG";
$resn_exch{"2H2B FUG"}="H2B2 FUG"; $resn_exch{"3H2B FUG"}="H2B3 FUG"; $resn_exch{"1H2C FUG"}="H2C1 FUG";
$resn_exch{"2H2C FUG"}="H2C2 FUG"; $resn_exch{"3H2C FUG"}="H2C3 FUG"; $resn_exch{"1H31 FUG"}="H311 FUG";
$resn_exch{"2H31 FUG"}="H312 FUG"; $resn_exch{"3H31 FUG"}="H313 FUG"; $resn_exch{"4HOB FUG"}="HOB4 FUG";
$resn_exch{"1H6  FUL"}=" H61 FUL"; $resn_exch{"2H6  FUL"}=" H62 FUL"; $resn_exch{"3H6  FUL"}=" H63 FUL";
$resn_exch{"1HN1 FUN"}="HN11 FUN"; $resn_exch{"2HN1 FUN"}="HN12 FUN"; $resn_exch{"1H5  FUN"}=" H51 FUN";
$resn_exch{"2H5  FUN"}=" H52 FUN"; $resn_exch{"2HOB FUP"}="HOB2 FUP"; $resn_exch{"3HOB FUP"}="HOB3 FUP";
$resn_exch{"2HOA FUP"}="HOA2 FUP"; $resn_exch{"1H5* FUP"}="H5'1 FUP"; $resn_exch{"2H5* FUP"}="H5'2 FUP";
$resn_exch{" H4* FUP"}=" H4' FUP"; $resn_exch{" H3* FUP"}=" H3' FUP"; $resn_exch{"1H2* FUP"}="H2'1 FUP";
$resn_exch{"2H2* FUP"}="H2'2 FUP"; $resn_exch{" H1* FUP"}=" H1' FUP"; $resn_exch{"1H7  FUX"}=" H71 FUX";
$resn_exch{"2H7  FUX"}=" H72 FUX"; $resn_exch{"1HB1 FVF"}="HB11 FVF"; $resn_exch{"1HB2 FVF"}="HB21 FVF";
$resn_exch{"1HD1 FVF"}="HD11 FVF"; $resn_exch{"1HD2 FVF"}="HD21 FVF"; $resn_exch{"1HE1 FVF"}="HE11 FVF";
$resn_exch{"1HE2 FVF"}="HE21 FVF"; $resn_exch{"1HZ  FVF"}=" HZ1 FVF"; $resn_exch{"2HN  FVF"}=" HN2 FVF";
$resn_exch{"2HA  FVF"}=" HA2 FVF"; $resn_exch{"2HB1 FVF"}="HB12 FVF"; $resn_exch{"2HB2 FVF"}="HB22 FVF";
$resn_exch{"2HD1 FVF"}="HD12 FVF"; $resn_exch{"2HD2 FVF"}="HD22 FVF"; $resn_exch{"2HE1 FVF"}="HE12 FVF";
$resn_exch{"2HE2 FVF"}="HE22 FVF"; $resn_exch{"2HZ  FVF"}=" HZ2 FVF"; $resn_exch{"3HN  FVF"}=" HN3 FVF";
$resn_exch{"3HA  FVF"}=" HA3 FVF"; $resn_exch{"3HB  FVF"}=" HB3 FVF"; $resn_exch{"3H11 FVF"}="H113 FVF";
$resn_exch{"3H12 FVF"}="H123 FVF"; $resn_exch{"3H13 FVF"}="H133 FVF"; $resn_exch{"3H21 FVF"}="H213 FVF";
$resn_exch{"3H22 FVF"}="H223 FVF"; $resn_exch{"3H23 FVF"}="H233 FVF"; $resn_exch{"2HOP FVF"}="HOP2 FVF";
$resn_exch{"4HA  FVF"}=" HA4 FVF"; $resn_exch{"4HXT FVF"}="HXT4 FVF"; $resn_exch{"4HB1 FVF"}="HB14 FVF";
$resn_exch{"4HB2 FVF"}="HB24 FVF"; $resn_exch{"4HD1 FVF"}="HD14 FVF"; $resn_exch{"4HD2 FVF"}="HD24 FVF";
$resn_exch{"4HE1 FVF"}="HE14 FVF"; $resn_exch{"4HE2 FVF"}="HE24 FVF"; $resn_exch{"4HZ  FVF"}=" HZ4 FVF";
$resn_exch{"1H7  FWD"}=" H71 FWD"; $resn_exch{"2H7  FWD"}=" H72 FWD"; $resn_exch{"1H8  FWD"}=" H81 FWD";
$resn_exch{"2H8  FWD"}=" H82 FWD"; $resn_exch{"OH92 FWD"}="H92O FWD"; $resn_exch{"1H10 FX3"}="H101 FX3";
$resn_exch{"2H10 FX3"}="H102 FX3"; $resn_exch{"3H10 FX3"}="H103 FX3"; $resn_exch{"1H5* FX3"}="H5'1 FX3";
$resn_exch{"2H5* FX3"}="H5'2 FX3"; $resn_exch{" H4* FX3"}=" H4' FX3"; $resn_exch{" H3* FX3"}=" H3' FX3";
$resn_exch{" H1* FX3"}=" H1' FX3"; $resn_exch{" H2* FX3"}=" H2' FX3"; $resn_exch{"1H8  FXI"}=" H81 FXI";
$resn_exch{"2H8  FXI"}=" H82 FXI"; $resn_exch{"1H9  FXI"}=" H91 FXI"; $resn_exch{"2H9  FXI"}=" H92 FXI";
$resn_exch{"1H11 FXI"}="H111 FXI"; $resn_exch{"2H11 FXI"}="H112 FXI"; $resn_exch{"1H13 FXI"}="H131 FXI";
$resn_exch{"2H13 FXI"}="H132 FXI"; $resn_exch{"3H13 FXI"}="H133 FXI"; $resn_exch{"1H26 FXI"}="H261 FXI";
$resn_exch{"2H26 FXI"}="H262 FXI"; $resn_exch{"1H27 FXI"}="H271 FXI"; $resn_exch{"2H27 FXI"}="H272 FXI";
$resn_exch{"1H29 FXI"}="H291 FXI"; $resn_exch{"2H29 FXI"}="H292 FXI"; $resn_exch{"3H29 FXI"}="H293 FXI";
$resn_exch{"1H31 FXM"}="H311 FXM"; $resn_exch{"2H31 FXM"}="H312 FXM"; $resn_exch{"3H31 FXM"}="H313 FXM";
$resn_exch{"1H32 FXM"}="H321 FXM"; $resn_exch{"2H32 FXM"}="H322 FXM"; $resn_exch{"3H32 FXM"}="H323 FXM";
$resn_exch{"1H33 FXM"}="H331 FXM"; $resn_exch{"2H33 FXM"}="H332 FXM"; $resn_exch{"3H33 FXM"}="H333 FXM";
$resn_exch{"1H26 FXM"}="H261 FXM"; $resn_exch{"2H26 FXM"}="H262 FXM"; $resn_exch{"1H16 FXM"}="H161 FXM";
$resn_exch{"2H16 FXM"}="H162 FXM"; $resn_exch{"1H19 FXM"}="H191 FXM"; $resn_exch{"2H19 FXM"}="H192 FXM";
$resn_exch{"1H6  FXM"}=" H61 FXM"; $resn_exch{"2H6  FXM"}=" H62 FXM"; $resn_exch{"1H36 FXM"}="H361 FXM";
$resn_exch{"2H36 FXM"}="H362 FXM"; $resn_exch{"1H37 FXM"}="H371 FXM"; $resn_exch{"2H37 FXM"}="H372 FXM";
$resn_exch{"3H37 FXM"}="H373 FXM"; $resn_exch{"1H38 FXM"}="H381 FXM"; $resn_exch{"2H38 FXM"}="H382 FXM";
$resn_exch{"1H14 FXN"}="H141 FXN"; $resn_exch{"2H14 FXN"}="H142 FXN"; $resn_exch{"1H9  FXN"}=" H91 FXN";
$resn_exch{"2H9  FXN"}=" H92 FXN"; $resn_exch{"1H1A FXP"}="H1A1 FXP"; $resn_exch{"2H1A FXP"}="H1A2 FXP";
$resn_exch{"1H5A FXP"}="H5A1 FXP"; $resn_exch{"2H5A FXP"}="H5A2 FXP"; $resn_exch{"1H5B FXP"}="H5B1 FXP";
$resn_exch{"2H5B FXP"}="H5B2 FXP"; $resn_exch{"1H16 FXV"}="H161 FXV"; $resn_exch{"2H16 FXV"}="H162 FXV";
$resn_exch{"3H16 FXV"}="H163 FXV"; $resn_exch{"1H20 FXV"}="H201 FXV"; $resn_exch{"2H20 FXV"}="H202 FXV";
$resn_exch{"1H30 FXV"}="H301 FXV"; $resn_exch{"2H30 FXV"}="H302 FXV"; $resn_exch{"1H31 FXV"}="H311 FXV";
$resn_exch{"2H31 FXV"}="H312 FXV"; $resn_exch{"1H34 FXV"}="H341 FXV"; $resn_exch{"2H34 FXV"}="H342 FXV";
$resn_exch{"3H34 FXV"}="H343 FXV"; $resn_exch{"1H25 FXY"}="H251 FXY"; $resn_exch{"2H25 FXY"}="H252 FXY";
$resn_exch{"3H25 FXY"}="H253 FXY"; $resn_exch{"1H2  FXY"}=" H21 FXY"; $resn_exch{"2H2  FXY"}=" H22 FXY";
$resn_exch{"1H3  FXY"}=" H31 FXY"; $resn_exch{"2H3  FXY"}=" H32 FXY"; $resn_exch{"1H4  FXY"}=" H41 FXY";
$resn_exch{"2H4  FXY"}=" H42 FXY"; $resn_exch{"1H5  FXY"}=" H51 FXY"; $resn_exch{"2H5  FXY"}=" H52 FXY";
$resn_exch{"1H6  FXY"}=" H61 FXY"; $resn_exch{"2H6  FXY"}=" H62 FXY"; $resn_exch{"1H7  FXY"}=" H71 FXY";
$resn_exch{"2H7  FXY"}=" H72 FXY"; $resn_exch{"3H7  FXY"}=" H73 FXY"; $resn_exch{"1H   FYA"}=" H1  FYA";
$resn_exch{"2H   FYA"}=" H2A FYA"; $resn_exch{"1HN  FYA"}=" HN1 FYA"; $resn_exch{"2HN  FYA"}=" HN2 FYA";
$resn_exch{"1HB  FYA"}=" HB1 FYA"; $resn_exch{"2HB  FYA"}=" HB2 FYA"; $resn_exch{"2HOP FYA"}="HOP2 FYA";
$resn_exch{"1H5* FYA"}="H5'1 FYA"; $resn_exch{"2H5* FYA"}="H5'2 FYA"; $resn_exch{" H4* FYA"}=" H4' FYA";
$resn_exch{" H3* FYA"}=" H3' FYA"; $resn_exch{"3HO* FYA"}="HO'3 FYA"; $resn_exch{" H2* FYA"}=" H2' FYA";
$resn_exch{"2HO* FYA"}="HO'2 FYA"; $resn_exch{" H1* FYA"}=" H1' FYA"; $resn_exch{"1H6  FYA"}=" H61 FYA";
$resn_exch{"2H6  FYA"}=" H62 FYA"; $resn_exch{"1H6N FYN"}="H6N1 FYN"; $resn_exch{"2H6N FYN"}="H6N2 FYN";
$resn_exch{"1H5' FYN"}="H5'1 FYN"; $resn_exch{"2H5' FYN"}="H5'2 FYN"; $resn_exch{"1HPB FYN"}="HPB1 FYN";
$resn_exch{"2HPB FYN"}="HPB2 FYN"; $resn_exch{"1HP9 FYN"}="HP91 FYN"; $resn_exch{"2HP9 FYN"}="HP92 FYN";
$resn_exch{"3HP9 FYN"}="HP93 FYN"; $resn_exch{"1HP8 FYN"}="HP81 FYN"; $resn_exch{"2HP8 FYN"}="HP82 FYN";
$resn_exch{"3HP8 FYN"}="HP83 FYN"; $resn_exch{"1HP5 FYN"}="HP51 FYN"; $resn_exch{"2HP5 FYN"}="HP52 FYN";
$resn_exch{"1HP4 FYN"}="HP41 FYN"; $resn_exch{"2HP4 FYN"}="HP42 FYN"; $resn_exch{"1HP2 FYN"}="HP21 FYN";
$resn_exch{"2HP2 FYN"}="HP22 FYN"; $resn_exch{"1HP1 FYN"}="HP11 FYN"; $resn_exch{"2HP1 FYN"}="HP12 FYN";
$resn_exch{"3HOP   G"}="HOP3   G"; $resn_exch{"2HOP   G"}="HOP2   G"; $resn_exch{"1H5*   G"}=" H5'   G";
$resn_exch{"2H5*   G"}="H5''   G"; $resn_exch{" H4*   G"}=" H4'   G"; $resn_exch{" H3*   G"}=" H3'   G";
$resn_exch{"H3T    G"}="HO3'   G"; $resn_exch{" H2*   G"}=" H2'   G"; $resn_exch{"2HO*   G"}="HO2'   G";
$resn_exch{" H1*   G"}=" H1'   G"; $resn_exch{"1H2    G"}=" H21   G"; $resn_exch{"2H2    G"}=" H22   G";
$resn_exch{"1H7  G0G"}=" H71 G0G"; $resn_exch{"2H7  G0G"}=" H72 G0G"; $resn_exch{"1H21 G0G"}="H211 G0G";
$resn_exch{"2H21 G0G"}="H212 G0G"; $resn_exch{"1H25 G0G"}="H251 G0G"; $resn_exch{"2H25 G0G"}="H252 G0G";
$resn_exch{"1H33 G0G"}="H331 G0G"; $resn_exch{"2H33 G0G"}="H332 G0G"; $resn_exch{"1H13 G12"}="H131 G12";
$resn_exch{"2H13 G12"}="H132 G12"; $resn_exch{"3H13 G12"}="H133 G12"; $resn_exch{"1H27 G15"}="H271 G15";
$resn_exch{"2H27 G15"}="H272 G15"; $resn_exch{"1H28 G15"}="H281 G15"; $resn_exch{"2H28 G15"}="H282 G15";
$resn_exch{"1H29 G15"}="H291 G15"; $resn_exch{"2H29 G15"}="H292 G15"; $resn_exch{"1H30 G15"}="H301 G15";
$resn_exch{"2H30 G15"}="H302 G15"; $resn_exch{"1H24 G15"}="H241 G15"; $resn_exch{"2H24 G15"}="H242 G15";
$resn_exch{"3H24 G15"}="H243 G15"; $resn_exch{"1H21 G15"}="H211 G15"; $resn_exch{"2H21 G15"}="H212 G15";
$resn_exch{"1H18 G15"}="H181 G15"; $resn_exch{"2H18 G15"}="H182 G15"; $resn_exch{"1H6  G16"}=" H61 G16";
$resn_exch{"2H6  G16"}=" H62 G16"; $resn_exch{"2HOP G16"}="HOP2 G16"; $resn_exch{"3HOP G16"}="HOP3 G16";
$resn_exch{"HO1* G16"}="HO1' G16"; $resn_exch{"1HN4 G19"}="HN41 G19"; $resn_exch{"2HN4 G19"}="HN42 G19";
$resn_exch{"1H1  G19"}="H11A G19"; $resn_exch{"2H1  G19"}=" H12 G19"; $resn_exch{"1H13 G19"}="H131 G19";
$resn_exch{"2H13 G19"}="H132 G19"; $resn_exch{"1H15 G19"}="H151 G19"; $resn_exch{"2H15 G19"}="H152 G19";
$resn_exch{"3H15 G19"}="H153 G19"; $resn_exch{"1H16 G19"}="H161 G19"; $resn_exch{"2H16 G19"}="H162 G19";
$resn_exch{"3H16 G19"}="H163 G19"; $resn_exch{"1H17 G19"}="H171 G19"; $resn_exch{"2H17 G19"}="H172 G19";
$resn_exch{"3H17 G19"}="H173 G19"; $resn_exch{"1H18 G19"}="H181 G19"; $resn_exch{"2H18 G19"}="H182 G19";
$resn_exch{"3H18 G19"}="H183 G19"; $resn_exch{"1H20 G19"}="H201 G19"; $resn_exch{"2H20 G19"}="H202 G19";
$resn_exch{"1H32 G1G"}="H321 G1G"; $resn_exch{"2H32 G1G"}="H322 G1G"; $resn_exch{"1H46 G1G"}="H461 G1G";
$resn_exch{"2H46 G1G"}="H462 G1G"; $resn_exch{"3H46 G1G"}="H463 G1G"; $resn_exch{"1H45 G1G"}="H451 G1G";
$resn_exch{"2H45 G1G"}="H452 G1G"; $resn_exch{"1H25 G1G"}="H251 G1G"; $resn_exch{"2H25 G1G"}="H252 G1G";
$resn_exch{"1H12 G1G"}="H121 G1G"; $resn_exch{"2H12 G1G"}="H122 G1G"; $resn_exch{"1H37 G1G"}="H371 G1G";
$resn_exch{"2H37 G1G"}="H372 G1G"; $resn_exch{"3H37 G1G"}="H373 G1G"; $resn_exch{"1H14 G1L"}="H141 G1L";
$resn_exch{"2H14 G1L"}="H142 G1L"; $resn_exch{"3H14 G1L"}="H143 G1L"; $resn_exch{"1H15 G1L"}="H151 G1L";
$resn_exch{"2H15 G1L"}="H152 G1L"; $resn_exch{"3H15 G1L"}="H153 G1L"; $resn_exch{"1H16 G1L"}="H161 G1L";
$resn_exch{"2H16 G1L"}="H162 G1L"; $resn_exch{"1H6  G1P"}=" H61 G1P"; $resn_exch{"2H6  G1P"}=" H62 G1P";
$resn_exch{"2HOP G1P"}="HOP2 G1P"; $resn_exch{"3HOP G1P"}="HOP3 G1P"; $resn_exch{"1HN2 G1R"}="HN21 G1R";
$resn_exch{"2HN2 G1R"}="HN22 G1R"; $resn_exch{" H1* G1R"}=" H1' G1R"; $resn_exch{" H2* G1R"}=" H2' G1R";
$resn_exch{"HO2* G1R"}="HO2' G1R"; $resn_exch{" H3* G1R"}=" H3' G1R"; $resn_exch{"HO3* G1R"}="HO3' G1R";
$resn_exch{" H4* G1R"}=" H4' G1R"; $resn_exch{"1H5* G1R"}="H5'1 G1R"; $resn_exch{"2H5* G1R"}="H5'2 G1R";
$resn_exch{"1H5X G1R"}="H5X1 G1R"; $resn_exch{"2H5X G1R"}="H5X2 G1R"; $resn_exch{"1H11 G20"}="H111 G20";
$resn_exch{"2H11 G20"}="H112 G20"; $resn_exch{"3H11 G20"}="H113 G20"; $resn_exch{"1H81 G20"}="H811 G20";
$resn_exch{"2H81 G20"}="H812 G20"; $resn_exch{"3H81 G20"}="H813 G20"; $resn_exch{"1H9  G20"}=" H91 G20";
$resn_exch{"2H9  G20"}=" H92 G20"; $resn_exch{"1H91 G20"}="H911 G20"; $resn_exch{"2H91 G20"}="H912 G20";
$resn_exch{"1H92 G20"}="H921 G20"; $resn_exch{"2H92 G20"}="H922 G20"; $resn_exch{"3H92 G20"}="H923 G20";
$resn_exch{"1HN2 G20"}="HN21 G20"; $resn_exch{"2HN2 G20"}="HN22 G20"; $resn_exch{"1H3  G21"}=" H31 G21";
$resn_exch{"2H3  G21"}=" H32 G21"; $resn_exch{"1H11 G21"}="H111 G21"; $resn_exch{"2H11 G21"}="H112 G21";
$resn_exch{"3H11 G21"}="H113 G21"; $resn_exch{"1H81 G21"}="H811 G21"; $resn_exch{"2H81 G21"}="H812 G21";
$resn_exch{"1H82 G21"}="H821 G21"; $resn_exch{"2H82 G21"}="H822 G21"; $resn_exch{"1H9  G21"}=" H91 G21";
$resn_exch{"2H9  G21"}=" H92 G21"; $resn_exch{"1H91 G21"}="H911 G21"; $resn_exch{"2H91 G21"}="H912 G21";
$resn_exch{"1H92 G21"}="H921 G21"; $resn_exch{"2H92 G21"}="H922 G21"; $resn_exch{"3H92 G21"}="H923 G21";
$resn_exch{"1HNE G21"}="HNE1 G21"; $resn_exch{"2HNE G21"}="HNE2 G21"; $resn_exch{"1H5  G23"}=" H51 G23";
$resn_exch{"2H5  G23"}=" H52 G23"; $resn_exch{"3H5  G23"}=" H53 G23"; $resn_exch{"1H6  G23"}=" H61 G23";
$resn_exch{"2H6  G23"}=" H62 G23"; $resn_exch{"3H6  G23"}=" H63 G23"; $resn_exch{"1H9  G23"}=" H91 G23";
$resn_exch{"2H9  G23"}=" H92 G23"; $resn_exch{"1H11 G23"}="H111 G23"; $resn_exch{"2H11 G23"}="H112 G23";
$resn_exch{"1HAQ G24"}="HAQ1 G24"; $resn_exch{"2HAQ G24"}="HAQ2 G24"; $resn_exch{"1HAM G24"}="HAM1 G24";
$resn_exch{"2HAM G24"}="HAM2 G24"; $resn_exch{"1HBB G24"}="HBB1 G24"; $resn_exch{"2HBB G24"}="HBB2 G24";
$resn_exch{"3HBB G24"}="HBB3 G24"; $resn_exch{"1HAE G24"}="HAE1 G24"; $resn_exch{"2HAE G24"}="HAE2 G24";
$resn_exch{"3HAE G24"}="HAE3 G24"; $resn_exch{"1HAT G24"}="HAT1 G24"; $resn_exch{"2HAT G24"}="HAT2 G24";
$resn_exch{"3HOP G25"}="HOP3 G25"; $resn_exch{"2HOP G25"}="HOP2 G25"; $resn_exch{"1H5* G25"}=" H5' G25";
$resn_exch{"2H5* G25"}="H5'' G25"; $resn_exch{" H4* G25"}=" H4' G25"; $resn_exch{" H3* G25"}=" H3' G25";
$resn_exch{"3HO* G25"}="HO'3 G25"; $resn_exch{" H2* G25"}=" H2' G25"; $resn_exch{"2HO* G25"}="HO2' G25";
$resn_exch{" H1* G25"}=" H1' G25"; $resn_exch{"1H2  G25"}=" H21 G25"; $resn_exch{"2H2  G25"}=" H22 G25";
$resn_exch{"1H5  G26"}=" H51 G26"; $resn_exch{"2H5  G26"}=" H52 G26"; $resn_exch{"3H5  G26"}=" H53 G26";
$resn_exch{"1H6  G26"}=" H61 G26"; $resn_exch{"2H6  G26"}=" H62 G26"; $resn_exch{"3H6  G26"}=" H63 G26";
$resn_exch{"1H9  G26"}=" H91 G26"; $resn_exch{"2H9  G26"}=" H92 G26"; $resn_exch{"1H10 G26"}="H101 G26";
$resn_exch{"2H10 G26"}="H102 G26"; $resn_exch{"1H12 G26"}="H121 G26"; $resn_exch{"2H12 G26"}="H122 G26";
$resn_exch{"1H19 G26"}="H191 G26"; $resn_exch{"2H19 G26"}="H192 G26"; $resn_exch{"1H10 G27"}="H101 G27";
$resn_exch{"2H10 G27"}="H102 G27"; $resn_exch{"1H9  G27"}=" H91 G27"; $resn_exch{"2H9  G27"}=" H92 G27";
$resn_exch{"1H8  G27"}=" H81 G27"; $resn_exch{"2H8  G27"}=" H82 G27"; $resn_exch{"1H7  G27"}=" H71 G27";
$resn_exch{"2H7  G27"}=" H72 G27"; $resn_exch{"1H2  G27"}=" H21 G27"; $resn_exch{"2H2  G27"}=" H22 G27";
$resn_exch{"1H6  G27"}=" H61 G27"; $resn_exch{"2H6  G27"}=" H62 G27"; $resn_exch{"1H3  G28"}=" H31 G28";
$resn_exch{"2H3  G28"}=" H32 G28"; $resn_exch{"1H11 G28"}="H111 G28"; $resn_exch{"2H11 G28"}="H112 G28";
$resn_exch{"3H11 G28"}="H113 G28"; $resn_exch{"1H81 G28"}="H811 G28"; $resn_exch{"2H81 G28"}="H812 G28";
$resn_exch{"1H82 G28"}="H821 G28"; $resn_exch{"2H82 G28"}="H822 G28"; $resn_exch{"3H82 G28"}="H823 G28";
$resn_exch{"1H9  G28"}=" H91 G28"; $resn_exch{"2H9  G28"}=" H92 G28"; $resn_exch{"1H91 G28"}="H911 G28";
$resn_exch{"2H91 G28"}="H912 G28"; $resn_exch{"3H91 G28"}="H913 G28"; $resn_exch{"1HN4 G28"}="HN41 G28";
$resn_exch{"2HN4 G28"}="HN42 G28"; $resn_exch{"1H6  G2F"}=" H61 G2F"; $resn_exch{"2H6  G2F"}=" H62 G2F";
$resn_exch{"1H1  G2H"}="H11A G2H"; $resn_exch{"2H1  G2H"}=" H12 G2H"; $resn_exch{"1H3  G2H"}="H31A G2H";
$resn_exch{"2H3  G2H"}=" H32 G2H"; $resn_exch{"3HOP G2L"}="HOP3 G2L"; $resn_exch{"2HOP G2L"}="HOP2 G2L";
$resn_exch{"1H5* G2L"}=" H5' G2L"; $resn_exch{"2H5* G2L"}="H5'' G2L"; $resn_exch{" H4* G2L"}=" H4' G2L";
$resn_exch{" H3* G2L"}=" H3' G2L"; $resn_exch{" H2* G2L"}=" H2' G2L"; $resn_exch{"2HO* G2L"}="HO2' G2L";
$resn_exch{" H1* G2L"}=" H1' G2L"; $resn_exch{"1H2  G2L"}=" H21 G2L"; $resn_exch{"2H2  G2L"}=" H22 G2L";
$resn_exch{"1HA* G2L"}="HA'1 G2L"; $resn_exch{"2HA* G2L"}="HA'2 G2L"; $resn_exch{"1HB* G2L"}="HB'1 G2L";
$resn_exch{"2HB* G2L"}="HB'2 G2L"; $resn_exch{"1HD* G2L"}="HD'1 G2L"; $resn_exch{"2HD* G2L"}="HD'2 G2L";
$resn_exch{"3HD* G2L"}="HD'3 G2L"; $resn_exch{"1H3A G2P"}="H3A1 G2P"; $resn_exch{"2H3A G2P"}="H3A2 G2P";
$resn_exch{"1H5* G2P"}="H5'1 G2P"; $resn_exch{"2H5* G2P"}="H5'2 G2P"; $resn_exch{" H4* G2P"}=" H4' G2P";
$resn_exch{" H3* G2P"}=" H3' G2P"; $resn_exch{" H2* G2P"}=" H2' G2P"; $resn_exch{" H1* G2P"}=" H1' G2P";
$resn_exch{"1H2N G2P"}="H2N1 G2P"; $resn_exch{"2H2N G2P"}="H2N2 G2P"; $resn_exch{"1HN2 G2R"}="HN21 G2R";
$resn_exch{"2HN2 G2R"}="HN22 G2R"; $resn_exch{" H1* G2R"}=" H1' G2R"; $resn_exch{" H2* G2R"}=" H2' G2R";
$resn_exch{"HO2* G2R"}="HO2' G2R"; $resn_exch{" H3* G2R"}=" H3' G2R"; $resn_exch{"HO3* G2R"}="HO3' G2R";
$resn_exch{" H4* G2R"}=" H4' G2R"; $resn_exch{"1H5* G2R"}="H5'1 G2R"; $resn_exch{"2H5* G2R"}="H5'2 G2R";
$resn_exch{"1H5X G2R"}="H5X1 G2R"; $resn_exch{"2H5X G2R"}="H5X2 G2R"; $resn_exch{"1H1X G2R"}="H1X1 G2R";
$resn_exch{"2H1X G2R"}="H1X2 G2R"; $resn_exch{"3HOP G2S"}="HOP3 G2S"; $resn_exch{"2HOP G2S"}="HOP2 G2S";
$resn_exch{"1H5* G2S"}=" H5' G2S"; $resn_exch{"2H5* G2S"}="H5'' G2S"; $resn_exch{" H4* G2S"}=" H4' G2S";
$resn_exch{" H3* G2S"}=" H3' G2S"; $resn_exch{"H3T  G2S"}="HO3' G2S"; $resn_exch{"1H2* G2S"}="H2'1 G2S";
$resn_exch{" H1* G2S"}=" H1' G2S"; $resn_exch{"1H2  G2S"}=" H21 G2S"; $resn_exch{"2H2  G2S"}=" H22 G2S";
$resn_exch{"2H2* G2S"}="H2'2 G2S"; $resn_exch{"1H5* G31"}="H5'1 G31"; $resn_exch{"2H5* G31"}="H5'2 G31";
$resn_exch{" H4* G31"}=" H4' G31"; $resn_exch{" H3* G31"}=" H3' G31"; $resn_exch{"1H2* G31"}="H2'1 G31";
$resn_exch{"2H2* G31"}="H2'2 G31"; $resn_exch{" H1* G31"}=" H1' G31"; $resn_exch{"1H2N G31"}="H2N1 G31";
$resn_exch{"2H2N G31"}="H2N2 G31"; $resn_exch{"1H3M G31"}="H3M1 G31"; $resn_exch{"2H3M G31"}="H3M2 G31";
$resn_exch{"3H3M G31"}="H3M3 G31"; $resn_exch{"1H5* G32"}="H5'1 G32"; $resn_exch{"2H5* G32"}="H5'2 G32";
$resn_exch{" H4* G32"}=" H4' G32"; $resn_exch{" H3* G32"}=" H3' G32"; $resn_exch{"1H2* G32"}="H2'1 G32";
$resn_exch{"2H2* G32"}="H2'2 G32"; $resn_exch{" H1* G32"}=" H1' G32"; $resn_exch{"1H2N G32"}="H2N1 G32";
$resn_exch{"2H2N G32"}="H2N2 G32"; $resn_exch{"1H6M G32"}="H6M1 G32"; $resn_exch{"2H6M G32"}="H6M2 G32";
$resn_exch{"3H6M G32"}="H6M3 G32"; $resn_exch{"1H5* G33"}="H5'1 G33"; $resn_exch{"2H5* G33"}="H5'2 G33";
$resn_exch{" H4* G33"}=" H4' G33"; $resn_exch{" H3* G33"}=" H3' G33"; $resn_exch{"1H2* G33"}="H2'1 G33";
$resn_exch{"2H2* G33"}="H2'2 G33"; $resn_exch{" H1* G33"}=" H1' G33"; $resn_exch{"1H2  G33"}=" H21 G33";
$resn_exch{"2H2  G33"}=" H22 G33"; $resn_exch{"1H8M G33"}="H8M1 G33"; $resn_exch{"2H8M G33"}="H8M2 G33";
$resn_exch{"3H8M G33"}="H8M3 G33"; $resn_exch{"1H8  G34"}=" H81 G34"; $resn_exch{"2H8  G34"}=" H82 G34";
$resn_exch{"1H2  G34"}=" H21 G34"; $resn_exch{"2H2  G34"}=" H22 G34"; $resn_exch{"1H1  G34"}="H11A G34";
$resn_exch{"2H1  G34"}=" H12 G34"; $resn_exch{"1H7  G34"}=" H71 G34"; $resn_exch{"2H7  G34"}=" H72 G34";
$resn_exch{"1H13 G34"}="H131 G34"; $resn_exch{"2H13 G34"}="H132 G34"; $resn_exch{"1H15 G34"}="H151 G34";
$resn_exch{"2H15 G34"}="H152 G34"; $resn_exch{"3H15 G34"}="H153 G34"; $resn_exch{"1H16 G34"}="H161 G34";
$resn_exch{"2H16 G34"}="H162 G34"; $resn_exch{"3H16 G34"}="H163 G34"; $resn_exch{"1H17 G34"}="H171 G34";
$resn_exch{"2H17 G34"}="H172 G34"; $resn_exch{"3H17 G34"}="H173 G34"; $resn_exch{"1H18 G34"}="H181 G34";
$resn_exch{"2H18 G34"}="H182 G34"; $resn_exch{"3H18 G34"}="H183 G34"; $resn_exch{"1H20 G34"}="H201 G34";
$resn_exch{"2H20 G34"}="H202 G34"; $resn_exch{"1H22 G34"}="H221 G34"; $resn_exch{"2H22 G34"}="H222 G34";
$resn_exch{"1H24 G34"}="H241 G34"; $resn_exch{"2H24 G34"}="H242 G34"; $resn_exch{"1H26 G34"}="H261 G34";
$resn_exch{"2H26 G34"}="H262 G34"; $resn_exch{"1H27 G34"}="H271 G34"; $resn_exch{"2H27 G34"}="H272 G34";
$resn_exch{"1H29 G34"}="H291 G34"; $resn_exch{"2H29 G34"}="H292 G34"; $resn_exch{"1H30 G34"}="H301 G34";
$resn_exch{"2H30 G34"}="H302 G34"; $resn_exch{"3H30 G34"}="H303 G34"; $resn_exch{"1H5* G36"}="H5'1 G36";
$resn_exch{"2H5* G36"}="H5'2 G36"; $resn_exch{" H4* G36"}=" H4' G36"; $resn_exch{" H3* G36"}=" H3' G36";
$resn_exch{"1H2* G36"}="H2'1 G36"; $resn_exch{"2H2* G36"}="H2'2 G36"; $resn_exch{"1HM2 G36"}="HM21 G36";
$resn_exch{"2HM2 G36"}="HM22 G36"; $resn_exch{"3HM2 G36"}="HM23 G36"; $resn_exch{" H1* G36"}=" H1' G36";
$resn_exch{"1H2N G36"}="H2N1 G36"; $resn_exch{"2H2N G36"}="H2N2 G36"; $resn_exch{"1H1M G36"}="H1M1 G36";
$resn_exch{"2H1M G36"}="H1M2 G36"; $resn_exch{"1H1  G37"}=" H11 G37"; $resn_exch{"2H1  G37"}=" H12 G37";
$resn_exch{"1H7  G37"}=" H71 G37"; $resn_exch{"2H7  G37"}=" H72 G37"; $resn_exch{"3H7  G37"}=" H73 G37";
$resn_exch{"1H8  G37"}=" H81 G37"; $resn_exch{"2H8  G37"}=" H82 G37"; $resn_exch{"3H8  G37"}=" H83 G37";
$resn_exch{"1H11 G37"}="H111 G37"; $resn_exch{"2H11 G37"}="H112 G37"; $resn_exch{"1H19 G37"}="H191 G37";
$resn_exch{"2H19 G37"}="H192 G37"; $resn_exch{"1H34 G37"}="H341 G37"; $resn_exch{"2H34 G37"}="H342 G37";
$resn_exch{"1H41 G37"}="H411 G37"; $resn_exch{"2H41 G37"}="H412 G37"; $resn_exch{"1H5* G38"}="H5'1 G38";
$resn_exch{"2H5* G38"}="H5'2 G38"; $resn_exch{" H4* G38"}=" H4' G38"; $resn_exch{" H3* G38"}=" H3' G38";
$resn_exch{"1H2* G38"}="H2'1 G38"; $resn_exch{"2H2* G38"}="H2'2 G38"; $resn_exch{" H1* G38"}=" H1' G38";
$resn_exch{"3HN  G38"}=" HN3 G38"; $resn_exch{"1H2N G38"}="H2N1 G38"; $resn_exch{"2H2N G38"}="H2N2 G38";
$resn_exch{"1HN  G38"}=" HN1 G38"; $resn_exch{"2HN  G38"}=" HN2 G38"; $resn_exch{"1H3  G39"}=" H31 G39";
$resn_exch{"2H3  G39"}=" H32 G39"; $resn_exch{"1H11 G39"}="H111 G39"; $resn_exch{"2H11 G39"}="H112 G39";
$resn_exch{"3H11 G39"}="H113 G39"; $resn_exch{"1H9  G39"}=" H91 G39"; $resn_exch{"2H9  G39"}=" H92 G39";
$resn_exch{"1H81 G39"}="H811 G39"; $resn_exch{"2H81 G39"}="H812 G39"; $resn_exch{"1H82 G39"}="H821 G39";
$resn_exch{"2H82 G39"}="H822 G39"; $resn_exch{"3H82 G39"}="H823 G39"; $resn_exch{"1H91 G39"}="H911 G39";
$resn_exch{"2H91 G39"}="H912 G39"; $resn_exch{"3H91 G39"}="H913 G39"; $resn_exch{"1HN4 G39"}="HN41 G39";
$resn_exch{"2HN4 G39"}="HN42 G39"; $resn_exch{"1H32 G3A"}="H321 G3A"; $resn_exch{"2H32 G3A"}="H322 G3A";
$resn_exch{"1H45 G3A"}="H451 G3A"; $resn_exch{"2H45 G3A"}="H452 G3A"; $resn_exch{"1H25 G3A"}="H251 G3A";
$resn_exch{"2H25 G3A"}="H252 G3A"; $resn_exch{"1H16 G3A"}="H161 G3A"; $resn_exch{"2H16 G3A"}="H162 G3A";
$resn_exch{"2HOB G3D"}="HOB2 G3D"; $resn_exch{"3HOB G3D"}="HOB3 G3D"; $resn_exch{"2HOA G3D"}="HOA2 G3D";
$resn_exch{"1H5* G3D"}="H5'1 G3D"; $resn_exch{"2H5* G3D"}="H5'2 G3D"; $resn_exch{" H4* G3D"}=" H4' G3D";
$resn_exch{" H3* G3D"}=" H3' G3D"; $resn_exch{" H2* G3D"}=" H2' G3D"; $resn_exch{"*HO2 G3D"}="HO2' G3D";
$resn_exch{" H1* G3D"}=" H1' G3D"; $resn_exch{"1HN2 G3D"}="HN21 G3D"; $resn_exch{"2HN2 G3D"}="HN22 G3D";
$resn_exch{"1H33 G3G"}="H331 G3G"; $resn_exch{"2H33 G3G"}="H332 G3G"; $resn_exch{"1H25 G3G"}="H251 G3G";
$resn_exch{"2H25 G3G"}="H252 G3G"; $resn_exch{"1H21 G3G"}="H211 G3G"; $resn_exch{"2H21 G3G"}="H212 G3G";
$resn_exch{"1H7  G3G"}=" H71 G3G"; $resn_exch{"2H7  G3G"}=" H72 G3G"; $resn_exch{"1H42 G3G"}="H421 G3G";
$resn_exch{"2H42 G3G"}="H422 G3G"; $resn_exch{"1H43 G3G"}="H431 G3G"; $resn_exch{"2H43 G3G"}="H432 G3G";
$resn_exch{"1H1  G3H"}=" H11 G3H"; $resn_exch{"1H3  G3H"}=" H31 G3H"; $resn_exch{"2H3  G3H"}=" H32 G3H";
$resn_exch{"3HOP G3H"}="HOP3 G3H"; $resn_exch{"4HOP G3H"}="HOP4 G3H"; $resn_exch{"1H1  G3P"}=" H11 G3P";
$resn_exch{"2H1  G3P"}=" H12 G3P"; $resn_exch{"1H3  G3P"}=" H31 G3P"; $resn_exch{"2H3  G3P"}=" H32 G3P";
$resn_exch{"3HOP G3P"}="HOP3 G3P"; $resn_exch{"4HOP G3P"}="HOP4 G3P"; $resn_exch{"1H5* G42"}="H5'1 G42";
$resn_exch{"2H5* G42"}="H5'2 G42"; $resn_exch{" H4* G42"}=" H4' G42"; $resn_exch{" H3* G42"}=" H3' G42";
$resn_exch{"1H2* G42"}="H2'1 G42"; $resn_exch{"2H2* G42"}="H2'2 G42"; $resn_exch{" H1* G42"}=" H1' G42";
$resn_exch{"1H2N G42"}="H2N1 G42"; $resn_exch{"2H2N G42"}="H2N2 G42"; $resn_exch{"1H1  G44"}="H11A G44";
$resn_exch{"2H1  G44"}="H12A G44"; $resn_exch{"3H1  G44"}=" H13 G44"; $resn_exch{"1H16 G44"}="H161 G44";
$resn_exch{"2H16 G44"}="H162 G44"; $resn_exch{"1H24 G44"}="H241 G44"; $resn_exch{"2H24 G44"}="H242 G44";
$resn_exch{"1H5* G46"}="H5'1 G46"; $resn_exch{"2H5* G46"}="H5'2 G46"; $resn_exch{" H4* G46"}=" H4' G46";
$resn_exch{" H3* G46"}=" H3' G46"; $resn_exch{" H2* G46"}=" H2' G46"; $resn_exch{" H1* G46"}=" H1' G46";
$resn_exch{"1H2N G46"}="H2N1 G46"; $resn_exch{"2H2N G46"}="H2N2 G46"; $resn_exch{"1H5* G47"}="H5'1 G47";
$resn_exch{"2H5* G47"}="H5'2 G47"; $resn_exch{" H4* G47"}=" H4' G47"; $resn_exch{" H3* G47"}=" H3' G47";
$resn_exch{"1H2* G47"}="H2'1 G47"; $resn_exch{"2H2* G47"}="H2'2 G47"; $resn_exch{" H1* G47"}=" H1' G47";
$resn_exch{"1H6A G47"}="H6A1 G47"; $resn_exch{"2H6A G47"}="H6A2 G47"; $resn_exch{"1H7A G47"}="H7A1 G47";
$resn_exch{"2H7A G47"}="H7A2 G47"; $resn_exch{"1H5* G48"}="H5'1 G48"; $resn_exch{"2H5* G48"}="H5'2 G48";
$resn_exch{" H4* G48"}=" H4' G48"; $resn_exch{" H3* G48"}=" H3' G48"; $resn_exch{" H2* G48"}=" H2' G48";
$resn_exch{" H1* G48"}=" H1' G48"; $resn_exch{"1H2N G48"}="H2N1 G48"; $resn_exch{"2H2N G48"}="H2N2 G48";
$resn_exch{"1HA* G48"}="HA'1 G48"; $resn_exch{"2HA* G48"}="HA'2 G48"; $resn_exch{"1HB* G48"}="HB'1 G48";
$resn_exch{"2HB* G48"}="HB'2 G48"; $resn_exch{"1HD* G48"}="HD'1 G48"; $resn_exch{"2HD* G48"}="HD'2 G48";
$resn_exch{"3HD* G48"}="HD'3 G48"; $resn_exch{"1H5* G49"}="H5'1 G49"; $resn_exch{"2H5* G49"}="H5'2 G49";
$resn_exch{" H4* G49"}=" H4' G49"; $resn_exch{" H3* G49"}=" H3' G49"; $resn_exch{"1H2* G49"}="H2'1 G49";
$resn_exch{"2H2* G49"}="H2'2 G49"; $resn_exch{" H1* G49"}=" H1' G49"; $resn_exch{"1HM2 G49"}="HM21 G49";
$resn_exch{"2HM2 G49"}="HM22 G49"; $resn_exch{"3HM2 G49"}="HM23 G49"; $resn_exch{"1H4  G4D"}=" H41 G4D";
$resn_exch{"2H4  G4D"}=" H42 G4D"; $resn_exch{"1H6  G4D"}=" H61 G4D"; $resn_exch{"2H6  G4D"}=" H62 G4D";
$resn_exch{"1HN2 G4G"}="HN21 G4G"; $resn_exch{"2HN2 G4G"}="HN2A G4G"; $resn_exch{"1HN1 G4G"}="HN11 G4G";
$resn_exch{"2HN1 G4G"}="HN12 G4G"; $resn_exch{"1H7  G4G"}=" H71 G4G"; $resn_exch{"2H7  G4G"}=" H72 G4G";
$resn_exch{"1H21 G4G"}="H211 G4G"; $resn_exch{"2H21 G4G"}="H212 G4G"; $resn_exch{"1H25 G4G"}="H251 G4G";
$resn_exch{"2H25 G4G"}="H252 G4G"; $resn_exch{"1H33 G4G"}="H331 G4G"; $resn_exch{"2H33 G4G"}="H332 G4G";
$resn_exch{"1H5B G4M"}="H5B1 G4M"; $resn_exch{"2H5B G4M"}="H5B2 G4M"; $resn_exch{"1H2A G4M"}="H2A1 G4M";
$resn_exch{"2H2A G4M"}="H2A2 G4M"; $resn_exch{"3H2A G4M"}="H2A3 G4M"; $resn_exch{"1H4A G4M"}="H4A1 G4M";
$resn_exch{"2H4A G4M"}="H4A2 G4M"; $resn_exch{"1H6G G4M"}="H6G1 G4M"; $resn_exch{"2H6G G4M"}="H6G2 G4M";
$resn_exch{"3H6G G4M"}="H6G3 G4M"; $resn_exch{"1H5* G4M"}="H5'1 G4M"; $resn_exch{"2H5* G4M"}="H5'2 G4M";
$resn_exch{" H4* G4M"}=" H4' G4M"; $resn_exch{" H3* G4M"}=" H3' G4M"; $resn_exch{"HO3* G4M"}="HO3' G4M";
$resn_exch{" H2* G4M"}=" H2' G4M"; $resn_exch{"HO2* G4M"}="HO2' G4M"; $resn_exch{" H1* G4M"}=" H1' G4M";
$resn_exch{"1HN2 G4M"}="HN21 G4M"; $resn_exch{"2HN2 G4M"}="HN22 G4M"; $resn_exch{"2HOB G4P"}="HOB2 G4P";
$resn_exch{"3HOB G4P"}="HOB3 G4P"; $resn_exch{"2HOA G4P"}="HOA2 G4P"; $resn_exch{"1H5* G4P"}=" H5' G4P";
$resn_exch{"2H5* G4P"}="H5'' G4P"; $resn_exch{" H4* G4P"}=" H4' G4P"; $resn_exch{" H3* G4P"}=" H3' G4P";
$resn_exch{" H2* G4P"}=" H2' G4P"; $resn_exch{"*HO2 G4P"}="HO2' G4P"; $resn_exch{" H1* G4P"}=" H1' G4P";
$resn_exch{"1HN2 G4P"}="HN21 G4P"; $resn_exch{"2HN2 G4P"}="HN22 G4P"; $resn_exch{"1H6  G4S"}=" H61 G4S";
$resn_exch{"2H6  G4S"}=" H62 G4S"; $resn_exch{"1H8C G50"}="H8C1 G50"; $resn_exch{"2H8C G50"}="H8C2 G50";
$resn_exch{"1H9C G50"}="H9C1 G50"; $resn_exch{"2H9C G50"}="H9C2 G50"; $resn_exch{"1H5* G5P"}="H5'1 G5P";
$resn_exch{"2H5* G5P"}="H5'2 G5P"; $resn_exch{" H4* G5P"}=" H4' G5P"; $resn_exch{" H1* G5P"}=" H1' G5P";
$resn_exch{"1HN2 G5P"}="HN21 G5P"; $resn_exch{"2HN2 G5P"}="HN22 G5P"; $resn_exch{"1H1  G5P"}="H11X G5P";
$resn_exch{"2H1  G5P"}=" H12 G5P"; $resn_exch{" H2* G5P"}=" H2' G5P"; $resn_exch{" H3* G5P"}=" H3' G5P";
$resn_exch{"1HN6 G5P"}="HN61 G5P"; $resn_exch{"2HN6 G5P"}="HN62 G5P"; $resn_exch{"3HCA G6D"}="HCA3 G6D";
$resn_exch{"3HOA G6D"}="HOA3 G6D"; $resn_exch{"4HCA G6D"}="HCA4 G6D"; $resn_exch{"4HOA G6D"}="HOA4 G6D";
$resn_exch{"5HCA G6D"}="HCA5 G6D"; $resn_exch{"1HC6 G6D"}="HC61 G6D"; $resn_exch{"2HC6 G6D"}="HC62 G6D";
$resn_exch{"3HC6 G6D"}="HC63 G6D"; $resn_exch{"1H6  G6P"}=" H61 G6P"; $resn_exch{"2H6  G6P"}=" H62 G6P";
$resn_exch{"1H6  G6Q"}=" H61 G6Q"; $resn_exch{"2H6  G6Q"}=" H62 G6Q"; $resn_exch{"2HOP G6Q"}="HOP2 G6Q";
$resn_exch{"3HOP G6Q"}="HOP3 G6Q"; $resn_exch{"1H6  G6S"}=" H61 G6S"; $resn_exch{"2H6  G6S"}=" H62 G6S";
$resn_exch{"3HOP G7M"}="HOP3 G7M"; $resn_exch{"2HOP G7M"}="HOP2 G7M"; $resn_exch{"1H5* G7M"}=" H5' G7M";
$resn_exch{"2H5* G7M"}="H5'' G7M"; $resn_exch{" H4* G7M"}=" H4' G7M"; $resn_exch{" H3* G7M"}=" H3' G7M";
$resn_exch{"H3T  G7M"}="HO3' G7M"; $resn_exch{" H2* G7M"}=" H2' G7M"; $resn_exch{"2HO* G7M"}="HO2' G7M";
$resn_exch{" H1* G7M"}=" H1' G7M"; $resn_exch{"1HN7 G7M"}="HN71 G7M"; $resn_exch{"2HN7 G7M"}="HN72 G7M";
$resn_exch{"3HN7 G7M"}="HN73 G7M"; $resn_exch{"1H2  G7M"}=" H21 G7M"; $resn_exch{"2H2  G7M"}=" H22 G7M";
$resn_exch{"1H1  G80"}="H11A G80"; $resn_exch{"2H1  G80"}=" H12 G80"; $resn_exch{"1H2  G80"}=" H21 G80";
$resn_exch{"2H2  G80"}=" H22 G80"; $resn_exch{"1H7  G80"}=" H71 G80"; $resn_exch{"2H7  G80"}=" H72 G80";
$resn_exch{"1H8  G80"}=" H81 G80"; $resn_exch{"2H8  G80"}=" H82 G80"; $resn_exch{"1H13 G80"}="H131 G80";
$resn_exch{"2H13 G80"}="H132 G80"; $resn_exch{"1H15 G80"}="H151 G80"; $resn_exch{"2H15 G80"}="H152 G80";
$resn_exch{"3H15 G80"}="H153 G80"; $resn_exch{"1H16 G80"}="H161 G80"; $resn_exch{"2H16 G80"}="H162 G80";
$resn_exch{"3H16 G80"}="H163 G80"; $resn_exch{"1H17 G80"}="H171 G80"; $resn_exch{"2H17 G80"}="H172 G80";
$resn_exch{"3H17 G80"}="H173 G80"; $resn_exch{"1H18 G80"}="H181 G80"; $resn_exch{"2H18 G80"}="H182 G80";
$resn_exch{"3H18 G80"}="H183 G80"; $resn_exch{"1H20 G80"}="H201 G80"; $resn_exch{"2H20 G80"}="H202 G80";
$resn_exch{"1H22 G80"}="H221 G80"; $resn_exch{"2H22 G80"}="H222 G80"; $resn_exch{"1H24 G80"}="H241 G80";
$resn_exch{"2H24 G80"}="H242 G80"; $resn_exch{"1H25 G80"}="H251 G80"; $resn_exch{"2H25 G80"}="H252 G80";
$resn_exch{"1H27 G80"}="H271 G80"; $resn_exch{"2H27 G80"}="H272 G80"; $resn_exch{"1H26 G80"}="H261 G80";
$resn_exch{"2H26 G80"}="H262 G80"; $resn_exch{"1H5C G88"}="H5C1 G88"; $resn_exch{"2H5C G88"}="H5C2 G88";
$resn_exch{"1H4C G88"}="H4C1 G88"; $resn_exch{"2H4C G88"}="H4C2 G88"; $resn_exch{"1H1C G88"}="H1C1 G88";
$resn_exch{"2H1C G88"}="H1C2 G88"; $resn_exch{"1HN1 GA0"}="HN11 GA0"; $resn_exch{"2HN1 GA0"}="HN12 GA0";
$resn_exch{"1HA1 GA0"}="HA11 GA0"; $resn_exch{"2HA1 GA0"}="HA12 GA0"; $resn_exch{"1HB2 GA0"}="HB21 GA0";
$resn_exch{"2HB2 GA0"}="HB22 GA0"; $resn_exch{"3HB2 GA0"}="HB23 GA0"; $resn_exch{"1HD3 GA0"}="HD31 GA0";
$resn_exch{"2HD3 GA0"}="HD32 GA0"; $resn_exch{"1HB3 GA0"}="HB31 GA0"; $resn_exch{"2HB3 GA0"}="HB32 GA0";
$resn_exch{"1HG3 GA0"}="HG31 GA0"; $resn_exch{"2HG3 GA0"}="HG32 GA0"; $resn_exch{"1H4* GA2"}="H4'1 GA2";
$resn_exch{"2H4* GA2"}="H4'2 GA2"; $resn_exch{"*HO4 GA2"}="HO4' GA2"; $resn_exch{"1H3* GA2"}="H3'1 GA2";
$resn_exch{"2H3* GA2"}="H3'2 GA2"; $resn_exch{"*HO3 GA2"}="HO3' GA2"; $resn_exch{" H2* GA2"}=" H2' GA2";
$resn_exch{"1H1* GA2"}="H1'1 GA2"; $resn_exch{"2H1* GA2"}="H1'2 GA2"; $resn_exch{"1HN2 GA2"}="HN21 GA2";
$resn_exch{"2HN2 GA2"}="HN22 GA2"; $resn_exch{"1H1  GA4"}=" H11 GA4"; $resn_exch{"2H1  GA4"}=" H12 GA4";
$resn_exch{"1H2  GA4"}=" H21 GA4"; $resn_exch{"2H2  GA4"}=" H22 GA4"; $resn_exch{"1H11 GA4"}="H111 GA4";
$resn_exch{"2H11 GA4"}="H112 GA4"; $resn_exch{"1H12 GA4"}="H121 GA4"; $resn_exch{"2H12 GA4"}="H122 GA4";
$resn_exch{"1H14 GA4"}="H141 GA4"; $resn_exch{"2H14 GA4"}="H142 GA4"; $resn_exch{"1H15 GA4"}="H151 GA4";
$resn_exch{"2H15 GA4"}="H152 GA4"; $resn_exch{"1H17 GA4"}="H171 GA4"; $resn_exch{"2H17 GA4"}="H172 GA4";
$resn_exch{"1H18 GA4"}="H181 GA4"; $resn_exch{"2H18 GA4"}="H182 GA4"; $resn_exch{"3H18 GA4"}="H183 GA4";
$resn_exch{"1H6  GAA"}=" H61 GAA"; $resn_exch{"2H6  GAA"}=" H62 GAA"; $resn_exch{"'HO2 GAB"}="HO2' GAB";
$resn_exch{"1HN3 GAB"}="HN31 GAB"; $resn_exch{"2HN3 GAB"}="HN32 GAB"; $resn_exch{"1H6A GAC"}="H6A1 GAC";
$resn_exch{"2H6A GAC"}="H6A2 GAC"; $resn_exch{"1H7A GAC"}="H7A1 GAC"; $resn_exch{"2H7A GAC"}="H7A2 GAC";
$resn_exch{"2HOA GAC"}="HOA2 GAC"; $resn_exch{"3HOA GAC"}="HOA3 GAC"; $resn_exch{"4HOA GAC"}="HOA4 GAC";
$resn_exch{"6HOA GAC"}="HOA6 GAC"; $resn_exch{"1H6B GAC"}="H6B1 GAC"; $resn_exch{"2H6B GAC"}="H6B2 GAC";
$resn_exch{"3H6B GAC"}="H6B3 GAC"; $resn_exch{"2HOB GAC"}="HOB2 GAC"; $resn_exch{"3HOB GAC"}="HOB3 GAC";
$resn_exch{"1H6C GAC"}="H6C1 GAC"; $resn_exch{"2H6C GAC"}="H6C2 GAC"; $resn_exch{"2HOC GAC"}="HOC2 GAC";
$resn_exch{"3HOC GAC"}="HOC3 GAC"; $resn_exch{"6HOC GAC"}="HOC6 GAC"; $resn_exch{"1H6D GAC"}="H6D1 GAC";
$resn_exch{"2H6D GAC"}="H6D2 GAC"; $resn_exch{"1HOD GAC"}="HOD1 GAC"; $resn_exch{"2HOD GAC"}="HOD2 GAC";
$resn_exch{"3HOD GAC"}="HOD3 GAC"; $resn_exch{"6HOD GAC"}="HOD6 GAC"; $resn_exch{"1H1  GAD"}=" H11 GAD";
$resn_exch{"2H1  GAD"}=" H12 GAD"; $resn_exch{"1H1C GAF"}="H1C1 GAF"; $resn_exch{"1H6C GAF"}="H6C1 GAF";
$resn_exch{"2H6C GAF"}="H6C2 GAF"; $resn_exch{"1H1  GAH"}="H11A GAH"; $resn_exch{"2H1  GAH"}=" H12 GAH";
$resn_exch{"1H7  GAH"}=" H71 GAH"; $resn_exch{"2H7  GAH"}=" H72 GAH"; $resn_exch{"1H10 GAH"}="H101 GAH";
$resn_exch{"2H10 GAH"}="H102 GAH"; $resn_exch{"1H5  GAH"}=" H51 GAH"; $resn_exch{"2H5  GAH"}="H52A GAH";
$resn_exch{"1H4  GAH"}=" H41 GAH"; $resn_exch{"2H4  GAH"}=" H42 GAH"; $resn_exch{"1H48 GAH"}="H481 GAH";
$resn_exch{"2H48 GAH"}="H482 GAH"; $resn_exch{"1H49 GAH"}="H491 GAH"; $resn_exch{"2H49 GAH"}="H492 GAH";
$resn_exch{"1H50 GAH"}="H501 GAH"; $resn_exch{"2H50 GAH"}="H502 GAH"; $resn_exch{"1H51 GAH"}="H511 GAH";
$resn_exch{"2H51 GAH"}="H512 GAH"; $resn_exch{"1H54 GAH"}="H541 GAH"; $resn_exch{"2H54 GAH"}="H542 GAH";
$resn_exch{"1H47 GAH"}="H471 GAH"; $resn_exch{"2H47 GAH"}="H472 GAH"; $resn_exch{"1H33 GAH"}="H331 GAH";
$resn_exch{"2H33 GAH"}="H332 GAH"; $resn_exch{"1H27 GAH"}="H271 GAH"; $resn_exch{"2H27 GAH"}="H272 GAH";
$resn_exch{"1H26 GAH"}="H261 GAH"; $resn_exch{"2H26 GAH"}="H262 GAH"; $resn_exch{"1H16 GAH"}="H161 GAH";
$resn_exch{"2H16 GAH"}="H162 GAH"; $resn_exch{"1H2  GAH"}=" H21 GAH"; $resn_exch{"2H2  GAH"}=" H22 GAH";
$resn_exch{"1H23 GAH"}="H231 GAH"; $resn_exch{"2H23 GAH"}="H232 GAH"; $resn_exch{"1H22 GAH"}="H221 GAH";
$resn_exch{"2H22 GAH"}="H222 GAH"; $resn_exch{"1H21 GAH"}="H211 GAH"; $resn_exch{"2H21 GAH"}="H212 GAH";
$resn_exch{"1H20 GAH"}="H201 GAH"; $resn_exch{"2H20 GAH"}="H202 GAH"; $resn_exch{"1H19 GAH"}="H191 GAH";
$resn_exch{"2H19 GAH"}="H192 GAH"; $resn_exch{"1HN2 GAI"}="HN21 GAI"; $resn_exch{"2HN2 GAI"}="HN22 GAI";
$resn_exch{"1HN3 GAI"}="HN31 GAI"; $resn_exch{"2HN3 GAI"}="HN32 GAI"; $resn_exch{"1H6  GAL"}=" H61 GAL";
$resn_exch{"2H6  GAL"}=" H62 GAL"; $resn_exch{"1H2  GAM"}=" H21 GAM"; $resn_exch{"2H2  GAM"}=" H22 GAM";
$resn_exch{"3H2  GAM"}=" H23 GAM"; $resn_exch{"1H1  GAM"}=" H11 GAM"; $resn_exch{"2H1  GAM"}=" H12 GAM";
$resn_exch{"1HG  GAM"}=" HG1 GAM"; $resn_exch{"2HG  GAM"}=" HG2 GAM"; $resn_exch{"1HN  GAM"}=" HN1 GAM";
$resn_exch{"2HN  GAM"}=" HN2 GAM"; $resn_exch{"1HN1 GAN"}="HN11 GAN"; $resn_exch{"2HN1 GAN"}="HN12 GAN";
$resn_exch{"1H3  GAN"}=" H31 GAN"; $resn_exch{"2H3  GAN"}=" H32 GAN"; $resn_exch{"3H3  GAN"}=" H33 GAN";
$resn_exch{"1H8  GAN"}=" H81 GAN"; $resn_exch{"2H8  GAN"}=" H82 GAN"; $resn_exch{"3H8  GAN"}=" H83 GAN";
$resn_exch{"1H12 GAN"}="H121 GAN"; $resn_exch{"2H12 GAN"}="H122 GAN"; $resn_exch{"1H19 GAN"}="H191 GAN";
$resn_exch{"2H19 GAN"}="H192 GAN"; $resn_exch{"1H20 GAN"}="H201 GAN"; $resn_exch{"2H20 GAN"}="H202 GAN";
$resn_exch{"1H21 GAN"}="H211 GAN"; $resn_exch{"2H21 GAN"}="H212 GAN"; $resn_exch{"1H24 GAN"}="H241 GAN";
$resn_exch{"2H24 GAN"}="H242 GAN"; $resn_exch{"3H24 GAN"}="H243 GAN"; $resn_exch{"1H30 GAN"}="H301 GAN";
$resn_exch{"2H30 GAN"}="H302 GAN"; $resn_exch{"3H30 GAN"}="H303 GAN"; $resn_exch{"1H31 GAN"}="H311 GAN";
$resn_exch{"2H31 GAN"}="H312 GAN"; $resn_exch{"3H31 GAN"}="H313 GAN"; $resn_exch{"1H38 GAN"}="H381 GAN";
$resn_exch{"2H38 GAN"}="H382 GAN"; $resn_exch{"3H38 GAN"}="H383 GAN"; $resn_exch{"1H39 GAN"}="H391 GAN";
$resn_exch{"2H39 GAN"}="H392 GAN"; $resn_exch{"3H39 GAN"}="H393 GAN"; $resn_exch{"1H42 GAN"}="H421 GAN";
$resn_exch{"2H42 GAN"}="H422 GAN"; $resn_exch{"3H42 GAN"}="H423 GAN"; $resn_exch{"3HOP GAO"}="HOP3 GAO";
$resn_exch{"2HOP GAO"}="HOP2 GAO"; $resn_exch{"1H5* GAO"}=" H5' GAO"; $resn_exch{"2H5* GAO"}="H5'' GAO";
$resn_exch{" H4* GAO"}=" H4' GAO"; $resn_exch{" H3* GAO"}=" H3' GAO"; $resn_exch{"H3T  GAO"}="HO3' GAO";
$resn_exch{" H2* GAO"}=" H2' GAO"; $resn_exch{"2HO* GAO"}="HO2' GAO"; $resn_exch{" H1* GAO"}=" H1' GAO";
$resn_exch{"1H2  GAO"}=" H21 GAO"; $resn_exch{"2H2  GAO"}=" H22 GAO"; $resn_exch{"1H   GAP"}=" H1  GAP";
$resn_exch{"2H   GAP"}=" H2A GAP"; $resn_exch{"3H   GAP"}=" H3  GAP"; $resn_exch{"1HA  GAP"}=" HA1 GAP";
$resn_exch{"2HA  GAP"}=" HA2 GAP"; $resn_exch{"1H5* GAP"}="H5'1 GAP"; $resn_exch{"2H5* GAP"}="H5'2 GAP";
$resn_exch{" H4* GAP"}=" H4' GAP"; $resn_exch{" H3* GAP"}=" H3' GAP"; $resn_exch{"3HO* GAP"}="HO'3 GAP";
$resn_exch{" H2* GAP"}=" H2' GAP"; $resn_exch{"2HO* GAP"}="HO'2 GAP"; $resn_exch{" H1* GAP"}=" H1' GAP";
$resn_exch{"5H5  GAQ"}=" H55 GAQ"; $resn_exch{"1H4  GAQ"}=" H41 GAQ"; $resn_exch{"2H4  GAQ"}=" H42 GAQ";
$resn_exch{"1H3  GAQ"}=" H31 GAQ"; $resn_exch{"2H3  GAQ"}=" H32 GAQ"; $resn_exch{"1H2  GAQ"}=" H21 GAQ";
$resn_exch{"2H2  GAQ"}=" H22 GAQ"; $resn_exch{"1H1  GAQ"}=" H11 GAQ"; $resn_exch{"1H10 GAR"}="H101 GAR";
$resn_exch{"2H10 GAR"}="H102 GAR"; $resn_exch{"1H23 GAR"}="H231 GAR"; $resn_exch{"2H23 GAR"}="H232 GAR";
$resn_exch{"1H24 GAR"}="H241 GAR"; $resn_exch{"2H24 GAR"}="H242 GAR"; $resn_exch{"1H9  GAS"}=" H91 GAS";
$resn_exch{"2H9  GAS"}=" H92 GAS"; $resn_exch{"1H6  GAT"}=" H61 GAT"; $resn_exch{"2H6  GAT"}=" H62 GAT";
$resn_exch{"1HN1 GAT"}="HN11 GAT"; $resn_exch{"2HN1 GAT"}="HN12 GAT"; $resn_exch{"1HN  GAU"}=" H   GAU";
$resn_exch{"2HN  GAU"}=" H2  GAU"; $resn_exch{"1HB  GAU"}=" HB2 GAU"; $resn_exch{"2HB  GAU"}=" HB3 GAU";
$resn_exch{"1HG  GAU"}=" HG2 GAU"; $resn_exch{"2HG  GAU"}=" HG3 GAU"; $resn_exch{"1HO  GAU"}=" HO1 GAU";
$resn_exch{"2HO  GAU"}=" HO2 GAU"; $resn_exch{"1H2N GAV"}="H2N1 GAV"; $resn_exch{"2H2N GAV"}="H2N2 GAV";
$resn_exch{" H1* GAV"}=" H1' GAV"; $resn_exch{" H2* GAV"}=" H2' GAV"; $resn_exch{" H3* GAV"}=" H3' GAV";
$resn_exch{" H4* GAV"}=" H4' GAV"; $resn_exch{"1H5* GAV"}="H5'1 GAV"; $resn_exch{"2H5* GAV"}="H5'2 GAV";
$resn_exch{"1H    GB"}=" H1   GB"; $resn_exch{"2H    GB"}=" H2   GB"; $resn_exch{"3H    GB"}=" H3   GB";
$resn_exch{"1H9  GB1"}=" H91 GB1"; $resn_exch{"2H9  GB1"}=" H92 GB1"; $resn_exch{"1H6  GB1"}=" H61 GB1";
$resn_exch{"2H6  GB1"}=" H62 GB1"; $resn_exch{"1H2  GB1"}=" H21 GB1"; $resn_exch{"2H2  GB1"}=" H22 GB1";
$resn_exch{"1H6  GB2"}=" H61 GB2"; $resn_exch{"2H6  GB2"}=" H62 GB2"; $resn_exch{"1H2  GB2"}=" H21 GB2";
$resn_exch{"2H2  GB2"}=" H22 GB2"; $resn_exch{"1H9  GB2"}=" H91 GB2"; $resn_exch{"2H9  GB2"}=" H92 GB2";
$resn_exch{"1H9  GB3"}=" H91 GB3"; $resn_exch{"2H9  GB3"}=" H92 GB3"; $resn_exch{"1H6  GB3"}=" H61 GB3";
$resn_exch{"2H6  GB3"}=" H62 GB3"; $resn_exch{"1H21 GB3"}="H211 GB3"; $resn_exch{"2H21 GB3"}="H212 GB3";
$resn_exch{"3H21 GB3"}="H213 GB3"; $resn_exch{"1H4  GBC"}=" H41 GBC"; $resn_exch{"2H4  GBC"}=" H42 GBC";
$resn_exch{"1H5  GBC"}=" H51 GBC"; $resn_exch{"2H5  GBC"}=" H52 GBC"; $resn_exch{"1H6  GBC"}=" H61 GBC";
$resn_exch{"2H6  GBC"}=" H62 GBC"; $resn_exch{"1HN3 GBC"}="HN31 GBC"; $resn_exch{"2HN3 GBC"}="HN32 GBC";
$resn_exch{"1HN1 GBI"}="HN11 GBI"; $resn_exch{"2HN1 GBI"}="HN12 GBI"; $resn_exch{"1HB1 GBI"}="HB11 GBI";
$resn_exch{"2HB1 GBI"}="HB12 GBI"; $resn_exch{"1HG1 GBI"}="HG11 GBI"; $resn_exch{"2HG1 GBI"}="HG12 GBI";
$resn_exch{"1HB2 GBI"}="HB21 GBI"; $resn_exch{"2HB2 GBI"}="HB22 GBI"; $resn_exch{"1HA3 GBI"}="HA31 GBI";
$resn_exch{"2HA3 GBI"}="HA32 GBI"; $resn_exch{"1H'  GBI"}=" H'1 GBI"; $resn_exch{"2H'  GBI"}=" H'2 GBI";
$resn_exch{"1HAC GBL"}="HAC1 GBL"; $resn_exch{"2HAC GBL"}="HAC2 GBL"; $resn_exch{"1HBC GBL"}="HBC1 GBL";
$resn_exch{"2HBC GBL"}="HBC2 GBL"; $resn_exch{"1HGC GBL"}="HGC1 GBL"; $resn_exch{"2HGC GBL"}="HGC2 GBL";
$resn_exch{"1HN1 GBN"}="HN11 GBN"; $resn_exch{"2HN1 GBN"}="HN12 GBN"; $resn_exch{"1H2  GBN"}=" H21 GBN";
$resn_exch{"2H2  GBN"}=" H22 GBN"; $resn_exch{"1H3  GBN"}=" H31 GBN"; $resn_exch{"2H3  GBN"}=" H32 GBN";
$resn_exch{"1H2R GBN"}="H2R1 GBN"; $resn_exch{"2H2R GBN"}="H2R2 GBN"; $resn_exch{"1H3R GBN"}="H3R1 GBN";
$resn_exch{"2H3R GBN"}="H3R2 GBN"; $resn_exch{"1H4R GBN"}="H4R1 GBN"; $resn_exch{"2H4R GBN"}="H4R2 GBN";
$resn_exch{"1H5R GBN"}="H5R1 GBN"; $resn_exch{"2H5R GBN"}="H5R2 GBN"; $resn_exch{"1H6R GBN"}="H6R1 GBN";
$resn_exch{"2H6R GBN"}="H6R2 GBN"; $resn_exch{"1HOT GBP"}="HOT1 GBP"; $resn_exch{"1NH1 GBP"}="H11N GBP";
$resn_exch{"2NH1 GBP"}="H12N GBP"; $resn_exch{"1HA  GBP"}=" HA1 GBP"; $resn_exch{"1HB  GBP"}=" HB1 GBP";
$resn_exch{"2HB  GBP"}=" HB2 GBP"; $resn_exch{"1HB2 GBP"}="HB21 GBP"; $resn_exch{"2HB2 GBP"}="HB22 GBP";
$resn_exch{"1HG  GBP"}=" HG1 GBP"; $resn_exch{"2HG  GBP"}=" HG2 GBP"; $resn_exch{"1HN2 GBP"}="HN21 GBP";
$resn_exch{"1HA3 GBP"}="HA31 GBP"; $resn_exch{"2HA3 GBP"}="HA32 GBP"; $resn_exch{"1HO3 GBP"}="HO31 GBP";
$resn_exch{"1HH1 GBS"}="HH11 GBS"; $resn_exch{"2HH1 GBS"}="HH12 GBS"; $resn_exch{"1HN1 GBX"}="HN11 GBX";
$resn_exch{"2HN1 GBX"}="HN12 GBX"; $resn_exch{"1HB1 GBX"}="HB11 GBX"; $resn_exch{"2HB1 GBX"}="HB12 GBX";
$resn_exch{"1HG1 GBX"}="HG11 GBX"; $resn_exch{"2HG1 GBX"}="HG12 GBX"; $resn_exch{"1HB2 GBX"}="HB21 GBX";
$resn_exch{"2HB2 GBX"}="HB22 GBX"; $resn_exch{"1HA3 GBX"}="HA31 GBX"; $resn_exch{"2HA3 GBX"}="HA32 GBX";
$resn_exch{"1H1  GC1"}=" H11 GC1"; $resn_exch{"1H4  GC4"}=" H41 GC4"; $resn_exch{"2H4  GC4"}=" H42 GC4";
$resn_exch{"1HN1 GC7"}="HN11 GC7"; $resn_exch{"2HN1 GC7"}="HN12 GC7"; $resn_exch{"1H1  GC7"}=" H11 GC7";
$resn_exch{"2H1  GC7"}=" H12 GC7"; $resn_exch{"1H2  GC7"}=" H21 GC7"; $resn_exch{"2H2  GC7"}=" H22 GC7";
$resn_exch{"1H3  GC7"}=" H31 GC7"; $resn_exch{"2H3  GC7"}=" H32 GC7"; $resn_exch{"1H4  GC7"}=" H41 GC7";
$resn_exch{"2H4  GC7"}=" H42 GC7"; $resn_exch{"1H5  GC7"}=" H51 GC7"; $resn_exch{"2H5  GC7"}=" H52 GC7";
$resn_exch{"1H6  GC7"}=" H61 GC7"; $resn_exch{"2H6  GC7"}=" H62 GC7"; $resn_exch{"1H7  GC7"}=" H71 GC7";
$resn_exch{"2H7  GC7"}=" H72 GC7"; $resn_exch{"1HH1 GC7"}="HH11 GC7"; $resn_exch{"2HH1 GC7"}="HH12 GC7";
$resn_exch{"1HH2 GC7"}="HH21 GC7"; $resn_exch{"2HH2 GC7"}="HH22 GC7"; $resn_exch{"1H2  GCA"}=" H21 GCA";
$resn_exch{"1H4  GCA"}=" H41 GCA"; $resn_exch{"1H6  GCA"}=" H61 GCA"; $resn_exch{"1HP  GCA"}=" HP1 GCA";
$resn_exch{"2HP  GCA"}=" HP2 GCA"; $resn_exch{"3HP  GCA"}=" HP3 GCA"; $resn_exch{"1HQ  GCA"}=" HQ1 GCA";
$resn_exch{"2HQ  GCA"}=" HQ2 GCA"; $resn_exch{"3HQ  GCA"}=" HQ3 GCA"; $resn_exch{"1HG  GCA"}=" HG1 GCA";
$resn_exch{"2HG  GCA"}=" HG2 GCA"; $resn_exch{"1HF  GCA"}=" HF1 GCA"; $resn_exch{"1HH  GCA"}=" HH1 GCA";
$resn_exch{"2HH  GCA"}=" HH2 GCA"; $resn_exch{"3HH  GCA"}=" HH3 GCA"; $resn_exch{"1HI  GCA"}=" HI1 GCA";
$resn_exch{"2HI  GCA"}=" HI2 GCA"; $resn_exch{"3HI  GCA"}=" HI3 GCA"; $resn_exch{"1HN  GCA"}=" HN1 GCA";
$resn_exch{"1HA  GCA"}=" HA1 GCA"; $resn_exch{"2HA  GCA"}=" HA2 GCA"; $resn_exch{"1HC  GCA"}=" HC1 GCA";
$resn_exch{"2HC  GCA"}=" HC2 GCA"; $resn_exch{"1HD  GCA"}=" HD1 GCA"; $resn_exch{"2HD  GCA"}=" HD2 GCA";
$resn_exch{"3HD  GCA"}=" HD3 GCA"; $resn_exch{"1HB1 GCG"}="HB11 GCG"; $resn_exch{"2HB1 GCG"}="HB12 GCG";
$resn_exch{"1HN1 GCG"}="HN11 GCG"; $resn_exch{"2HN1 GCG"}="HN12 GCG"; $resn_exch{"1HG1 GCG"}="HG11 GCG";
$resn_exch{"2HG1 GCG"}="HG12 GCG"; $resn_exch{"1HB2 GCG"}="HB21 GCG"; $resn_exch{"2HB2 GCG"}="HB22 GCG";
$resn_exch{"1HA3 GCG"}="HA31 GCG"; $resn_exch{"2HA3 GCG"}="HA32 GCG"; $resn_exch{"1HNS GCG"}="HNS1 GCG";
$resn_exch{"1H2S GCG"}="H2S1 GCG"; $resn_exch{"2H2S GCG"}="H2S2 GCG"; $resn_exch{"1H3S GCG"}="H3S1 GCG";
$resn_exch{"2H3S GCG"}="H3S2 GCG"; $resn_exch{"1H4S GCG"}="H4S1 GCG"; $resn_exch{"2H4S GCG"}="H4S2 GCG";
$resn_exch{"1H5S GCG"}="H5S1 GCG"; $resn_exch{"2H5S GCG"}="H5S2 GCG"; $resn_exch{"1H7S GCG"}="H7S1 GCG";
$resn_exch{"2H7S GCG"}="H7S2 GCG"; $resn_exch{"6HNS GCG"}="HNS6 GCG"; $resn_exch{"1H8S GCG"}="H8S1 GCG";
$resn_exch{"2H8S GCG"}="H8S2 GCG"; $resn_exch{"1H9S GCG"}="H9S1 GCG"; $resn_exch{"2H9S GCG"}="H9S2 GCG";
$resn_exch{"1HA5 GCG"}="HA51 GCG"; $resn_exch{"2HA5 GCG"}="HA52 GCG"; $resn_exch{"1HB6 GCG"}="HB61 GCG";
$resn_exch{"2HB6 GCG"}="HB62 GCG"; $resn_exch{"1HB7 GCG"}="HB71 GCG"; $resn_exch{"2HB7 GCG"}="HB72 GCG";
$resn_exch{"1HN7 GCG"}="HN71 GCG"; $resn_exch{"2HN7 GCG"}="HN72 GCG"; $resn_exch{"1HG7 GCG"}="HG71 GCG";
$resn_exch{"2HG7 GCG"}="HG72 GCG"; $resn_exch{"2HOP GCK"}="HOP2 GCK"; $resn_exch{"1H16 GCK"}="H161 GCK";
$resn_exch{"2H16 GCK"}="H162 GCK"; $resn_exch{"1H17 GCK"}="H171 GCK"; $resn_exch{"2H17 GCK"}="H172 GCK";
$resn_exch{"1H20 GCK"}="H201 GCK"; $resn_exch{"2H20 GCK"}="H202 GCK"; $resn_exch{"1H21 GCK"}="H211 GCK";
$resn_exch{"2H21 GCK"}="H212 GCK"; $resn_exch{"1H2* GCK"}="H2'1 GCK"; $resn_exch{"2H2* GCK"}="H2'2 GCK";
$resn_exch{"1H5* GCK"}="H5'1 GCK"; $resn_exch{"2H5* GCK"}="H5'2 GCK"; $resn_exch{" H4* GCK"}=" H4' GCK";
$resn_exch{" H1* GCK"}=" H1' GCK"; $resn_exch{" H3* GCK"}=" H3' GCK"; $resn_exch{"3HOP GCK"}="HOP3 GCK";
$resn_exch{"1H1  GCL"}=" H11 GCL"; $resn_exch{"2H1  GCL"}=" H12 GCL"; $resn_exch{"1H5  GCL"}=" H51 GCL";
$resn_exch{"2H5  GCL"}=" H52 GCL"; $resn_exch{"1H7  GCL"}=" H71 GCL"; $resn_exch{"2H7  GCL"}=" H72 GCL";
$resn_exch{"3H7  GCL"}=" H73 GCL"; $resn_exch{"1H8  GCL"}=" H81 GCL"; $resn_exch{"2H8  GCL"}=" H82 GCL";
$resn_exch{"3H8  GCL"}=" H83 GCL"; $resn_exch{"1H2' GCL"}="H2'1 GCL"; $resn_exch{"2H2' GCL"}="H2'2 GCL";
$resn_exch{"1H3' GCL"}="H3'1 GCL"; $resn_exch{"2H3' GCL"}="H3'2 GCL"; $resn_exch{"1H4' GCL"}="H4'1 GCL";
$resn_exch{"2H4' GCL"}="H4'2 GCL"; $resn_exch{"1H5' GCL"}="H5'1 GCL"; $resn_exch{"2H5' GCL"}="H5'2 GCL";
$resn_exch{"1H6' GCL"}="H6'1 GCL"; $resn_exch{"2H6' GCL"}="H6'2 GCL"; $resn_exch{"1HN2 GCL"}="HN21 GCL";
$resn_exch{"2HN2 GCL"}="HN22 GCL"; $resn_exch{"1HN  GCM"}=" H   GCM"; $resn_exch{"2HN  GCM"}=" H2  GCM";
$resn_exch{"1HA  GCM"}=" HA1 GCM"; $resn_exch{"2HA  GCM"}=" HA2 GCM"; $resn_exch{"1HM  GCM"}=" HM1 GCM";
$resn_exch{"2HM  GCM"}=" HM2 GCM"; $resn_exch{"3HM  GCM"}=" HM3 GCM"; $resn_exch{"1H3  GCN"}=" H31 GCN";
$resn_exch{"2H3  GCN"}=" H32 GCN"; $resn_exch{"1H6  GCN"}=" H61 GCN"; $resn_exch{"2H6  GCN"}=" H62 GCN";
$resn_exch{"1HN2 GCN"}="HN21 GCN"; $resn_exch{"2HN2 GCN"}="HN22 GCN"; $resn_exch{"1H6  GCO"}=" H61 GCO";
$resn_exch{"2H6  GCO"}=" H62 GCO"; $resn_exch{"2HOG GCP"}="HOG2 GCP"; $resn_exch{"3HOG GCP"}="HOG3 GCP";
$resn_exch{"1H3B GCP"}="H3B1 GCP"; $resn_exch{"2H3B GCP"}="H3B2 GCP"; $resn_exch{"2HOB GCP"}="HOB2 GCP";
$resn_exch{"2HOA GCP"}="HOA2 GCP"; $resn_exch{"2H5* GCP"}="H5'2 GCP"; $resn_exch{"1H5* GCP"}="H5'1 GCP";
$resn_exch{" H4* GCP"}=" H4' GCP"; $resn_exch{" H3* GCP"}=" H3' GCP"; $resn_exch{"*HO3 GCP"}="HO3' GCP";
$resn_exch{" H2* GCP"}=" H2' GCP"; $resn_exch{"*HO2 GCP"}="HO2' GCP"; $resn_exch{" H1* GCP"}=" H1' GCP";
$resn_exch{"1HN2 GCP"}="HN21 GCP"; $resn_exch{"2HN2 GCP"}="HN22 GCP"; $resn_exch{"1H5* GCQ"}="H5'1 GCQ";
$resn_exch{"2H5* GCQ"}="H5'2 GCQ"; $resn_exch{" H4* GCQ"}=" H4' GCQ"; $resn_exch{" H3* GCQ"}=" H3' GCQ";
$resn_exch{" H1* GCQ"}=" H1' GCQ"; $resn_exch{"1HN4 GCQ"}="HN41 GCQ"; $resn_exch{"2HN4 GCQ"}="HN42 GCQ";
$resn_exch{"1H14 GCR"}="H141 GCR"; $resn_exch{"2H14 GCR"}="H142 GCR"; $resn_exch{"1H17 GCR"}="H171 GCR";
$resn_exch{"2H17 GCR"}="H172 GCR"; $resn_exch{"1H18 GCR"}="H181 GCR"; $resn_exch{"2H18 GCR"}="H182 GCR";
$resn_exch{"1H21 GCR"}="H211 GCR"; $resn_exch{"2H21 GCR"}="H212 GCR"; $resn_exch{"1H24 GCR"}="H241 GCR";
$resn_exch{"2H24 GCR"}="H242 GCR"; $resn_exch{"1H25 GCR"}="H251 GCR"; $resn_exch{"2H25 GCR"}="H252 GCR";
$resn_exch{"1H27 GCR"}="H271 GCR"; $resn_exch{"2H27 GCR"}="H272 GCR"; $resn_exch{"1H29 GCR"}="H291 GCR";
$resn_exch{"2H29 GCR"}="H292 GCR"; $resn_exch{"1H30 GCR"}="H301 GCR"; $resn_exch{"2H30 GCR"}="H302 GCR";
$resn_exch{"1H32 GCR"}="H321 GCR"; $resn_exch{"2H32 GCR"}="H322 GCR"; $resn_exch{"1H34 GCR"}="H341 GCR";
$resn_exch{"2H34 GCR"}="H342 GCR"; $resn_exch{"1H35 GCR"}="H351 GCR"; $resn_exch{"2H35 GCR"}="H352 GCR";
$resn_exch{"3H35 GCR"}="H353 GCR"; $resn_exch{"1H36 GCR"}="H361 GCR"; $resn_exch{"2H36 GCR"}="H362 GCR";
$resn_exch{"3H36 GCR"}="H363 GCR"; $resn_exch{"1H37 GCR"}="H371 GCR"; $resn_exch{"2H37 GCR"}="H372 GCR";
$resn_exch{"3H37 GCR"}="H373 GCR"; $resn_exch{"1H38 GCR"}="H381 GCR"; $resn_exch{"2H38 GCR"}="H382 GCR";
$resn_exch{"1H3  GCS"}=" H31 GCS"; $resn_exch{"1H6  GCS"}=" H61 GCS"; $resn_exch{"2H6  GCS"}=" H62 GCS";
$resn_exch{"1HN2 GCS"}="HN21 GCS"; $resn_exch{"2HN2 GCS"}="HN22 GCS"; $resn_exch{"2H3  GCS"}=" H32 GCS";
$resn_exch{"1HC1 GCT"}="HC11 GCT"; $resn_exch{"2HC1 GCT"}="HC12 GCT"; $resn_exch{"1HC4 GCT"}="HC41 GCT";
$resn_exch{"2HC4 GCT"}="HC42 GCT"; $resn_exch{"1H7  GCV"}=" H71 GCV"; $resn_exch{"2H7  GCV"}=" H72 GCV";
$resn_exch{"3H7  GCV"}=" H73 GCV"; $resn_exch{"1H7  GCW"}=" H71 GCW"; $resn_exch{"2H7  GCW"}=" H72 GCW";
$resn_exch{"3H7  GCW"}=" H73 GCW"; $resn_exch{"1H1  GD7"}=" H11 GD7"; $resn_exch{"2H1  GD7"}=" H12 GD7";
$resn_exch{"3H1  GD7"}=" H13 GD7"; $resn_exch{"1H2  GD7"}=" H21 GD7"; $resn_exch{"2H2  GD7"}=" H22 GD7";
$resn_exch{"3H2  GD7"}=" H23 GD7"; $resn_exch{"1H3  GD7"}=" H31 GD7"; $resn_exch{"2H3  GD7"}=" H32 GD7";
$resn_exch{"3H3  GD7"}=" H33 GD7"; $resn_exch{"1H4  GD7"}=" H41 GD7"; $resn_exch{"2H4  GD7"}=" H42 GD7";
$resn_exch{"3H4  GD7"}=" H43 GD7"; $resn_exch{"1H7  GD7"}=" H71 GD7"; $resn_exch{"2H7  GD7"}=" H72 GD7";
$resn_exch{"3H7  GD7"}=" H73 GD7"; $resn_exch{"1H6C GDA"}="H6C1 GDA"; $resn_exch{"2H6C GDA"}="H6C2 GDA";
$resn_exch{"1H4N GDA"}="H4N1 GDA"; $resn_exch{"2H4N GDA"}="H4N2 GDA"; $resn_exch{"1HN1 GDB"}="HN11 GDB";
$resn_exch{"2HN1 GDB"}="HN12 GDB"; $resn_exch{"1HB1 GDB"}="HB11 GDB"; $resn_exch{"2HB1 GDB"}="HB12 GDB";
$resn_exch{"1HG1 GDB"}="HG11 GDB"; $resn_exch{"2HG1 GDB"}="HG12 GDB"; $resn_exch{"1HB2 GDB"}="HB21 GDB";
$resn_exch{"2HB2 GDB"}="HB22 GDB"; $resn_exch{"1HA3 GDB"}="HA31 GDB"; $resn_exch{"2HA3 GDB"}="HA32 GDB";
$resn_exch{"1H5* GDC"}="H5'1 GDC"; $resn_exch{"2H5* GDC"}="H5'2 GDC"; $resn_exch{" H4* GDC"}=" H4' GDC";
$resn_exch{" H3* GDC"}=" H3' GDC"; $resn_exch{" H2* GDC"}=" H2' GDC"; $resn_exch{" H1* GDC"}=" H1' GDC";
$resn_exch{"1H2N GDC"}="H2N1 GDC"; $resn_exch{"2H2N GDC"}="H2N2 GDC"; $resn_exch{"1H6A GDC"}="H6A1 GDC";
$resn_exch{"2H6A GDC"}="H6A2 GDC"; $resn_exch{"1H2N GDD"}="H2N1 GDD"; $resn_exch{"2H2N GDD"}="H2N2 GDD";
$resn_exch{" H1* GDD"}=" H1' GDD"; $resn_exch{" H2* GDD"}=" H2' GDD"; $resn_exch{" H3* GDD"}=" H3' GDD";
$resn_exch{" H4* GDD"}=" H4' GDD"; $resn_exch{"1H5* GDD"}="H5'1 GDD"; $resn_exch{"2H5* GDD"}="H5'2 GDD";
$resn_exch{"1H61 GDD"}="H611 GDD"; $resn_exch{"2H61 GDD"}="H612 GDD"; $resn_exch{"1H6C GDL"}="H6C1 GDL";
$resn_exch{"2H6C GDL"}="H6C2 GDL"; $resn_exch{"1H8C GDL"}="H8C1 GDL"; $resn_exch{"2H8C GDL"}="H8C2 GDL";
$resn_exch{"3H8C GDL"}="H8C3 GDL"; $resn_exch{"1HN2 GDM"}="HN21 GDM"; $resn_exch{"2HN2 GDM"}="HN22 GDM";
$resn_exch{"1H13 GDM"}="H131 GDM"; $resn_exch{"2H13 GDM"}="H132 GDM"; $resn_exch{"1H15 GDM"}="H151 GDM";
$resn_exch{"2H15 GDM"}="H152 GDM"; $resn_exch{"1H22 GDM"}="H221 GDM"; $resn_exch{"2H22 GDM"}="H222 GDM";
$resn_exch{"3H22 GDM"}="H223 GDM"; $resn_exch{"1H23 GDM"}="H231 GDM"; $resn_exch{"2H23 GDM"}="H232 GDM";
$resn_exch{"3H23 GDM"}="H233 GDM"; $resn_exch{"1H25 GDM"}="H251 GDM"; $resn_exch{"2H25 GDM"}="H252 GDM";
$resn_exch{"3H25 GDM"}="H253 GDM"; $resn_exch{"1H26 GDM"}="H261 GDM"; $resn_exch{"2H26 GDM"}="H262 GDM";
$resn_exch{"3H26 GDM"}="H263 GDM"; $resn_exch{"1H27 GDM"}="H271 GDM"; $resn_exch{"2H27 GDM"}="H272 GDM";
$resn_exch{"3H27 GDM"}="H273 GDM"; $resn_exch{"1H28 GDM"}="H281 GDM"; $resn_exch{"2H28 GDM"}="H282 GDM";
$resn_exch{"3H28 GDM"}="H283 GDM"; $resn_exch{"1H29 GDM"}="H291 GDM"; $resn_exch{"2H29 GDM"}="H292 GDM";
$resn_exch{"3H29 GDM"}="H293 GDM"; $resn_exch{"1HN  GDN"}=" HN1 GDN"; $resn_exch{"2HN  GDN"}="HN2A GDN";
$resn_exch{"1HB1 GDN"}="HB11 GDN"; $resn_exch{"2HB1 GDN"}="HB12 GDN"; $resn_exch{"1HG1 GDN"}="HG11 GDN";
$resn_exch{"2HG1 GDN"}="HG12 GDN"; $resn_exch{"1HB2 GDN"}="HB21 GDN"; $resn_exch{"2HB2 GDN"}="HB22 GDN";
$resn_exch{"1HA3 GDN"}="HA31 GDN"; $resn_exch{"2HA3 GDN"}="HA32 GDN"; $resn_exch{"2HOB GDP"}="HOB2 GDP";
$resn_exch{"3HOB GDP"}="HOB3 GDP"; $resn_exch{"2HOA GDP"}="HOA2 GDP"; $resn_exch{"1H5* GDP"}=" H5' GDP";
$resn_exch{"2H5* GDP"}="H5'' GDP"; $resn_exch{" H4* GDP"}=" H4' GDP"; $resn_exch{" H3* GDP"}=" H3' GDP";
$resn_exch{"*HO3 GDP"}="HO3' GDP"; $resn_exch{" H2* GDP"}=" H2' GDP"; $resn_exch{"*HO2 GDP"}="HO2' GDP";
$resn_exch{" H1* GDP"}=" H1' GDP"; $resn_exch{"1HN2 GDP"}="HN21 GDP"; $resn_exch{"2HN2 GDP"}="HN22 GDP";
$resn_exch{"2HOB GDR"}="HOB2 GDR"; $resn_exch{"2HOA GDR"}="HOA2 GDR"; $resn_exch{"1H5* GDR"}=" H5' GDR";
$resn_exch{"2H5* GDR"}="H5'' GDR"; $resn_exch{" H4* GDR"}=" H4' GDR"; $resn_exch{" H1* GDR"}=" H1' GDR";
$resn_exch{"1HN2 GDR"}="HN21 GDR"; $resn_exch{"2HN2 GDR"}="HN22 GDR"; $resn_exch{" H2* GDR"}=" H2' GDR";
$resn_exch{"HO2* GDR"}="HO2A GDR"; $resn_exch{" H3* GDR"}=" H3' GDR"; $resn_exch{"HO3* GDR"}="HO3A GDR";
$resn_exch{"1HC6 GDR"}="HC61 GDR"; $resn_exch{"2HC6 GDR"}="HC62 GDR"; $resn_exch{"3HC6 GDR"}="HC63 GDR";
$resn_exch{"1HN1 GDS"}="HN11 GDS"; $resn_exch{"2HN1 GDS"}="HN12 GDS"; $resn_exch{"1HB1 GDS"}="HB11 GDS";
$resn_exch{"2HB1 GDS"}="HB12 GDS"; $resn_exch{"1HG1 GDS"}="HG11 GDS"; $resn_exch{"2HG1 GDS"}="HG12 GDS";
$resn_exch{"1HB2 GDS"}="HB21 GDS"; $resn_exch{"2HB2 GDS"}="HB22 GDS"; $resn_exch{"1HA3 GDS"}="HA31 GDS";
$resn_exch{"2HA3 GDS"}="HA32 GDS"; $resn_exch{"1HA4 GDS"}="HA41 GDS"; $resn_exch{"2HA4 GDS"}="HA42 GDS";
$resn_exch{"1HB5 GDS"}="HB51 GDS"; $resn_exch{"2HB5 GDS"}="HB52 GDS"; $resn_exch{"1HN6 GDS"}="HN61 GDS";
$resn_exch{"2HN6 GDS"}="HN62 GDS"; $resn_exch{"1HB6 GDS"}="HB61 GDS"; $resn_exch{"2HB6 GDS"}="HB62 GDS";
$resn_exch{"1HG6 GDS"}="HG61 GDS"; $resn_exch{"2HG6 GDS"}="HG62 GDS"; $resn_exch{" H1* GDU"}=" H1D GDU";
$resn_exch{" H2* GDU"}=" H2D GDU"; $resn_exch{"*HO2 GDU"}="HO2' GDU"; $resn_exch{" H3* GDU"}=" H3D GDU";
$resn_exch{" H4* GDU"}=" H4D GDU"; $resn_exch{"*HO3 GDU"}="HO3' GDU"; $resn_exch{"1H5* GDU"}="H5'1 GDU";
$resn_exch{"2H5* GDU"}="H5'2 GDU"; $resn_exch{"2HOA GDU"}="HOA2 GDU"; $resn_exch{"2HOB GDU"}="HOB2 GDU";
$resn_exch{"1H6' GDU"}="H6'1 GDU"; $resn_exch{"2H6' GDU"}="H6'2 GDU"; $resn_exch{"'HO2 GDU"}="HO2A GDU";
$resn_exch{"'HO3 GDU"}="HO3A GDU"; $resn_exch{"'HO4 GDU"}="HO4' GDU"; $resn_exch{"'HO6 GDU"}="HO6' GDU";
$resn_exch{"1H12 GDV"}="H121 GDV"; $resn_exch{"2H12 GDV"}="H122 GDV"; $resn_exch{"3H12 GDV"}="H123 GDV";
$resn_exch{"1H6C GDV"}="H6C1 GDV"; $resn_exch{"2H6C GDV"}="H6C2 GDV"; $resn_exch{"1HN2 GDX"}="HN21 GDX";
$resn_exch{"2HN2 GDX"}="HN22 GDX"; $resn_exch{" H1* GDX"}=" H1' GDX"; $resn_exch{" H2* GDX"}=" H2' GDX";
$resn_exch{"*HO2 GDX"}="HO2A GDX"; $resn_exch{" H3* GDX"}=" H3' GDX"; $resn_exch{"*HO3 GDX"}="HO3A GDX";
$resn_exch{" H4* GDX"}=" H4' GDX"; $resn_exch{"2H5* GDX"}="H5'2 GDX"; $resn_exch{"1H5* GDX"}="H5'1 GDX";
$resn_exch{"2HOA GDX"}="HOA2 GDX"; $resn_exch{"2HOB GDX"}="HOB2 GDX"; $resn_exch{"1HN2 GE1"}="HN21 GE1";
$resn_exch{"2HN2 GE1"}="HN22 GE1"; $resn_exch{"1H3  GE1"}=" H31 GE1"; $resn_exch{"2H3  GE1"}=" H32 GE1";
$resn_exch{"1H4  GE1"}=" H41 GE1"; $resn_exch{"2H4  GE1"}=" H42 GE1"; $resn_exch{"1H6  GE1"}=" H61 GE1";
$resn_exch{"2H6  GE1"}=" H62 GE1"; $resn_exch{"1HN1 GE1"}="HN11 GE1"; $resn_exch{"2HN1 GE1"}="HN12 GE1";
$resn_exch{"1H   GE2"}=" H1A GE2"; $resn_exch{"2H   GE2"}=" H2  GE2"; $resn_exch{"1HN1 GE2"}="HN11 GE2";
$resn_exch{"2HN1 GE2"}="HN12 GE2"; $resn_exch{"1H2  GE2"}=" H21 GE2"; $resn_exch{"2H2  GE2"}=" H22 GE2";
$resn_exch{"1H4  GE2"}=" H41 GE2"; $resn_exch{"2H4  GE2"}=" H42 GE2"; $resn_exch{"1HN6 GE2"}="HN61 GE2";
$resn_exch{"2HN6 GE2"}="HN62 GE2"; $resn_exch{"1H   GE3"}=" H1  GE3"; $resn_exch{"2H   GE3"}=" H2A GE3";
$resn_exch{"1H31 GE3"}="H311 GE3"; $resn_exch{"2H31 GE3"}="H312 GE3"; $resn_exch{"3H31 GE3"}="H313 GE3";
$resn_exch{"1H41 GE3"}="H411 GE3"; $resn_exch{"2H41 GE3"}="H412 GE3"; $resn_exch{"3H41 GE3"}="H413 GE3";
$resn_exch{"1HN1 GEA"}="HN11 GEA"; $resn_exch{"2HN1 GEA"}="HN12 GEA"; $resn_exch{"1HC4 GEA"}="HC41 GEA";
$resn_exch{"1HC5 GEA"}="HC51 GEA"; $resn_exch{"2HC5 GEA"}="HC52 GEA"; $resn_exch{"1HC6 GEA"}="HC61 GEA";
$resn_exch{"2HC6 GEA"}="HC62 GEA"; $resn_exch{"1HC7 GEA"}="HC71 GEA"; $resn_exch{"2HC7 GEA"}="HC72 GEA";
$resn_exch{"1H11 GEA"}="H111 GEA"; $resn_exch{"1H12 GEA"}="H121 GEA"; $resn_exch{"2H12 GEA"}="H122 GEA";
$resn_exch{"1H17 GEA"}="H171 GEA"; $resn_exch{"1H22 GEA"}="H221 GEA"; $resn_exch{"1H23 GEA"}="H231 GEA";
$resn_exch{"1H28 GEA"}="H281 GEA"; $resn_exch{"1H33 GEA"}="H331 GEA"; $resn_exch{"1H37 GEA"}="H371 GEA";
$resn_exch{"1H38 GEA"}="H381 GEA"; $resn_exch{"1H41 GEA"}="H411 GEA"; $resn_exch{"1H42 GEA"}="H421 GEA";
$resn_exch{"1H43 GEA"}="H431 GEA"; $resn_exch{"1H44 GEA"}="H441 GEA"; $resn_exch{"1H45 GEA"}="H451 GEA";
$resn_exch{"1H49 GEA"}="H491 GEA"; $resn_exch{"2H49 GEA"}="H492 GEA"; $resn_exch{"1H50 GEA"}="H501 GEA";
$resn_exch{"1H58 GEA"}="H581 GEA"; $resn_exch{"2H58 GEA"}="H582 GEA"; $resn_exch{"1H60 GEA"}="H601 GEA";
$resn_exch{"2H60 GEA"}="H602 GEA"; $resn_exch{"3H60 GEA"}="H603 GEA"; $resn_exch{"1H61 GEA"}="H611 GEA";
$resn_exch{"1H62 GEA"}="H621 GEA"; $resn_exch{"1H63 GEA"}="H631 GEA"; $resn_exch{"2H63 GEA"}="H632 GEA";
$resn_exch{"3H63 GEA"}="H633 GEA"; $resn_exch{"1H64 GEA"}="H641 GEA"; $resn_exch{"2H64 GEA"}="H642 GEA";
$resn_exch{"3H64 GEA"}="H643 GEA"; $resn_exch{"1H65 GEA"}="H651 GEA"; $resn_exch{"1H73 GEA"}="H731 GEA";
$resn_exch{"2H73 GEA"}="H732 GEA"; $resn_exch{"3H73 GEA"}="H733 GEA"; $resn_exch{"1H74 GEA"}="H741 GEA";
$resn_exch{"1H75 GEA"}="H751 GEA"; $resn_exch{"2H75 GEA"}="H752 GEA"; $resn_exch{"1H78 GEA"}="H781 GEA";
$resn_exch{"1H79 GEA"}="H791 GEA"; $resn_exch{"2H79 GEA"}="H792 GEA"; $resn_exch{"3H79 GEA"}="H793 GEA";
$resn_exch{"1H80 GEA"}="H801 GEA"; $resn_exch{"1H84 GEA"}="H841 GEA"; $resn_exch{"1H1C GEG"}="H1C1 GEG";
$resn_exch{"2H1C GEG"}="H1C2 GEG"; $resn_exch{"1H2C GEG"}="H2C1 GEG"; $resn_exch{"2H2C GEG"}="H2C2 GEG";
$resn_exch{"1H5C GEG"}="H5C1 GEG"; $resn_exch{"2H5C GEG"}="H5C2 GEG"; $resn_exch{"3H5C GEG"}="H5C3 GEG";
$resn_exch{"1H3N GEG"}="H3N1 GEG"; $resn_exch{"2H3N GEG"}="H3N2 GEG"; $resn_exch{"1H1  GEL"}=" H11 GEL";
$resn_exch{"2H1  GEL"}=" H12 GEL"; $resn_exch{"1H11 GEL"}="H111 GEL"; $resn_exch{"2H11 GEL"}="H112 GEL";
$resn_exch{"1H12 GEL"}="H121 GEL"; $resn_exch{"2H12 GEL"}="H122 GEL"; $resn_exch{"1H13 GEL"}="H131 GEL";
$resn_exch{"2H13 GEL"}="H132 GEL"; $resn_exch{"1H14 GEL"}="H141 GEL"; $resn_exch{"2H14 GEL"}="H142 GEL";
$resn_exch{"1H15 GEL"}="H151 GEL"; $resn_exch{"2H15 GEL"}="H152 GEL"; $resn_exch{"1H16 GEL"}="H161 GEL";
$resn_exch{"2H16 GEL"}="H162 GEL"; $resn_exch{"1H17 GEL"}="H171 GEL"; $resn_exch{"2H17 GEL"}="H172 GEL";
$resn_exch{"1H18 GEL"}="H181 GEL"; $resn_exch{"2H18 GEL"}="H182 GEL"; $resn_exch{"3H18 GEL"}="H183 GEL";
$resn_exch{"2HOP GEL"}="HOP2 GEL"; $resn_exch{"1H22 GEL"}="H221 GEL"; $resn_exch{"2H22 GEL"}="H222 GEL";
$resn_exch{"1H23 GEL"}="H231 GEL"; $resn_exch{"2H23 GEL"}="H232 GEL"; $resn_exch{"1H24 GEL"}="H241 GEL";
$resn_exch{"2H24 GEL"}="H242 GEL"; $resn_exch{"1H25 GEL"}="H251 GEL"; $resn_exch{"2H25 GEL"}="H252 GEL";
$resn_exch{"1H26 GEL"}="H261 GEL"; $resn_exch{"2H26 GEL"}="H262 GEL"; $resn_exch{"1H27 GEL"}="H271 GEL";
$resn_exch{"2H27 GEL"}="H272 GEL"; $resn_exch{"1H28 GEL"}="H281 GEL"; $resn_exch{"2H28 GEL"}="H282 GEL";
$resn_exch{"3H28 GEL"}="H283 GEL"; $resn_exch{"1H3  GEL"}=" H31 GEL"; $resn_exch{"2H3  GEL"}=" H32 GEL";
$resn_exch{"4HOP GEL"}="HOP4 GEL"; $resn_exch{"1H31 GEL"}="H311 GEL"; $resn_exch{"2H31 GEL"}="H312 GEL";
$resn_exch{"1H32 GEL"}="H321 GEL"; $resn_exch{"2H32 GEL"}="H322 GEL"; $resn_exch{"1HN3 GEL"}="HN31 GEL";
$resn_exch{"2HN3 GEL"}="HN32 GEL"; $resn_exch{"8HC  GEM"}=" HC8 GEM"; $resn_exch{"1HC6 GEM"}="HC61 GEM";
$resn_exch{"2HC6 GEM"}="HC62 GEM"; $resn_exch{"1HC5 GEM"}="HC51 GEM"; $resn_exch{"2HC5 GEM"}="HC52 GEM";
$resn_exch{"3HN  GEM"}=" HN3 GEM"; $resn_exch{"1HN4 GEM"}="HN41 GEM"; $resn_exch{"2HN4 GEM"}="HN42 GEM";
$resn_exch{"1HN2 GEM"}="HN21 GEM"; $resn_exch{"1HO4 GEM"}="HO41 GEM"; $resn_exch{"1H10 GEM"}="H101 GEM";
$resn_exch{"2H10 GEM"}="H102 GEM"; $resn_exch{"1HO3 GEM"}="HO31 GEM"; $resn_exch{"1H5' GEO"}="H5'1 GEO";
$resn_exch{"2H5' GEO"}="H5'2 GEO"; $resn_exch{"1HN4 GEO"}="HN41 GEO"; $resn_exch{"2HN4 GEO"}="HN42 GEO";
$resn_exch{"1H2  GEP"}=" H21 GEP"; $resn_exch{"2H2  GEP"}=" H22 GEP"; $resn_exch{"1H3  GEP"}=" H31 GEP";
$resn_exch{"2H3  GEP"}=" H32 GEP"; $resn_exch{"1H4  GEP"}=" H41 GEP"; $resn_exch{"2H4  GEP"}=" H42 GEP";
$resn_exch{"1H5  GEP"}=" H51 GEP"; $resn_exch{"2H5  GEP"}=" H52 GEP"; $resn_exch{"1H6  GEP"}=" H61 GEP";
$resn_exch{"2H6  GEP"}=" H62 GEP"; $resn_exch{"1H7  GEP"}=" H71 GEP"; $resn_exch{"2H7  GEP"}=" H72 GEP";
$resn_exch{"3H7  GEP"}=" H73 GEP"; $resn_exch{"1H8  GEP"}=" H81 GEP"; $resn_exch{"2H8  GEP"}=" H82 GEP";
$resn_exch{"1H9  GEP"}=" H91 GEP"; $resn_exch{"2H9  GEP"}=" H92 GEP"; $resn_exch{"1H19 GEP"}="H191 GEP";
$resn_exch{"2H19 GEP"}="H192 GEP"; $resn_exch{"1H20 GEP"}="H201 GEP"; $resn_exch{"2H20 GEP"}="H202 GEP";
$resn_exch{"1H21 GEP"}="H211 GEP"; $resn_exch{"2H21 GEP"}="H212 GEP"; $resn_exch{"1H2C GEQ"}="H2C1 GEQ";
$resn_exch{"2H2C GEQ"}="H2C2 GEQ"; $resn_exch{"1H3C GEQ"}="H3C1 GEQ"; $resn_exch{"2H3C GEQ"}="H3C2 GEQ";
$resn_exch{"1H5C GEQ"}="H5C1 GEQ"; $resn_exch{"2H5C GEQ"}="H5C2 GEQ"; $resn_exch{"1H6C GEQ"}="H6C1 GEQ";
$resn_exch{"2H6C GEQ"}="H6C2 GEQ"; $resn_exch{"1H1  GER"}=" H11 GER"; $resn_exch{"2H1  GER"}=" H12 GER";
$resn_exch{"3H1  GER"}=" H13 GER"; $resn_exch{"1H2  GER"}=" H21 GER"; $resn_exch{"1H4  GER"}=" H41 GER";
$resn_exch{"2H4  GER"}=" H42 GER"; $resn_exch{"3H4  GER"}=" H43 GER"; $resn_exch{"1H5  GER"}=" H51 GER";
$resn_exch{"2H5  GER"}=" H52 GER"; $resn_exch{"1H6  GER"}=" H61 GER"; $resn_exch{"2H6  GER"}=" H62 GER";
$resn_exch{"1H7  GER"}=" H71 GER"; $resn_exch{"1H9  GER"}=" H91 GER"; $resn_exch{"2H9  GER"}=" H92 GER";
$resn_exch{"3H9  GER"}=" H93 GER"; $resn_exch{"1H10 GER"}="H101 GER"; $resn_exch{"2H10 GER"}="H102 GER";
$resn_exch{"1H11 GER"}="H111 GER"; $resn_exch{"2H11 GER"}="H112 GER"; $resn_exch{"1H12 GER"}="H121 GER";
$resn_exch{"1H14 GER"}="H141 GER"; $resn_exch{"2H14 GER"}="H142 GER"; $resn_exch{"3H14 GER"}="H143 GER";
$resn_exch{"1H15 GER"}="H151 GER"; $resn_exch{"2H15 GER"}="H152 GER"; $resn_exch{"1H16 GER"}="H161 GER";
$resn_exch{"2H16 GER"}="H162 GER"; $resn_exch{"1H17 GER"}="H171 GER"; $resn_exch{"1H19 GER"}="H191 GER";
$resn_exch{"2H19 GER"}="H192 GER"; $resn_exch{"3H19 GER"}="H193 GER"; $resn_exch{"1H20 GER"}="H201 GER";
$resn_exch{"2H20 GER"}="H202 GER"; $resn_exch{"3H20 GER"}="H203 GER"; $resn_exch{"1H11 GET"}="H111 GET";
$resn_exch{"1H21 GET"}="H211 GET"; $resn_exch{"2H21 GET"}="H212 GET"; $resn_exch{"1H31 GET"}="H311 GET";
$resn_exch{"1H41 GET"}="H411 GET"; $resn_exch{"1H51 GET"}="H511 GET"; $resn_exch{"1H61 GET"}="H611 GET";
$resn_exch{"1H71 GET"}="H711 GET"; $resn_exch{"2H71 GET"}="H712 GET"; $resn_exch{"3H71 GET"}="H713 GET";
$resn_exch{"1H12 GET"}="H121 GET"; $resn_exch{"2H12 GET"}="H122 GET"; $resn_exch{"1H22 GET"}="H221 GET";
$resn_exch{"2H22 GET"}="H222 GET"; $resn_exch{"1H32 GET"}="H321 GET"; $resn_exch{"2H32 GET"}="H322 GET";
$resn_exch{"1H42 GET"}="H421 GET"; $resn_exch{"1H52 GET"}="H521 GET"; $resn_exch{"1H62 GET"}="H621 GET";
$resn_exch{"1H13 GET"}="H131 GET"; $resn_exch{"1H23 GET"}="H231 GET"; $resn_exch{"1H33 GET"}="H331 GET";
$resn_exch{"1H93 GET"}="H931 GET"; $resn_exch{"2H93 GET"}="H932 GET"; $resn_exch{"3H93 GET"}="H933 GET";
$resn_exch{"1H83 GET"}="H831 GET"; $resn_exch{"2H83 GET"}="H832 GET"; $resn_exch{"3H83 GET"}="H833 GET";
$resn_exch{"1H53 GET"}="H531 GET"; $resn_exch{"2H53 GET"}="H532 GET"; $resn_exch{"1HN2 GFF"}="HN21 GFF";
$resn_exch{"2HN2 GFF"}="HN22 GFF"; $resn_exch{"1H2' GFF"}="H2'1 GFF"; $resn_exch{"2H2' GFF"}="H2'2 GFF";
$resn_exch{"1H5' GFF"}="H5'1 GFF"; $resn_exch{"2H5' GFF"}="H5'2 GFF"; $resn_exch{"1H63 GFG"}="H631 GFG";
$resn_exch{"2H63 GFG"}="H632 GFG"; $resn_exch{"1H62 GFG"}="H621 GFG"; $resn_exch{"2H62 GFG"}="H622 GFG";
$resn_exch{"1H12 GFG"}="H121 GFG"; $resn_exch{"2H12 GFG"}="H122 GFG"; $resn_exch{"1H6  GFG"}=" H61 GFG";
$resn_exch{"2H6  GFG"}=" H62 GFG"; $resn_exch{"1H5' GFH"}="H5'1 GFH"; $resn_exch{"2H5' GFH"}="H5'2 GFH";
$resn_exch{"1HN2 GFH"}="HN21 GFH"; $resn_exch{"2HN2 GFH"}="HN22 GFH"; $resn_exch{"1H2' GFH"}="H2'1 GFH";
$resn_exch{"2H2' GFH"}="H2'2 GFH"; $resn_exch{"1H5* GFL"}="H5'1 GFL"; $resn_exch{"2H5* GFL"}="H5'2 GFL";
$resn_exch{" H4* GFL"}=" H4' GFL"; $resn_exch{" H3* GFL"}=" H3' GFL"; $resn_exch{"HO3* GFL"}="HO3' GFL";
$resn_exch{" H2* GFL"}=" H2' GFL"; $resn_exch{" H1* GFL"}=" H1' GFL"; $resn_exch{"1HN2 GFL"}="HN21 GFL";
$resn_exch{"2HN2 GFL"}="HN22 GFL"; $resn_exch{"1H6  GFP"}=" H61 GFP"; $resn_exch{"2H6  GFP"}=" H62 GFP";
$resn_exch{"2HOP GFP"}="HOP2 GFP"; $resn_exch{"3HOP GFP"}="HOP3 GFP"; $resn_exch{"1H4  GG1"}=" H41 GG1";
$resn_exch{"2H4  GG1"}=" H42 GG1"; $resn_exch{"1H25 GG1"}="H251 GG1"; $resn_exch{"2H25 GG1"}="H252 GG1";
$resn_exch{"3H25 GG1"}="H253 GG1"; $resn_exch{"1H11 GG1"}="H111 GG1"; $resn_exch{"2H11 GG1"}="H112 GG1";
$resn_exch{"1H15 GG2"}="H151 GG2"; $resn_exch{"2H15 GG2"}="H152 GG2"; $resn_exch{"3H15 GG2"}="H153 GG2";
$resn_exch{"1H20 GG2"}="H201 GG2"; $resn_exch{"2H20 GG2"}="H202 GG2"; $resn_exch{"1H21 GG2"}="H211 GG2";
$resn_exch{"2H21 GG2"}="H212 GG2"; $resn_exch{"1H24 GG2"}="H241 GG2"; $resn_exch{"2H24 GG2"}="H242 GG2";
$resn_exch{"1H25 GG2"}="H251 GG2"; $resn_exch{"2H25 GG2"}="H252 GG2"; $resn_exch{"1H23 GG2"}="H231 GG2";
$resn_exch{"2H23 GG2"}="H232 GG2"; $resn_exch{"1HN3 GG2"}="HN31 GG2"; $resn_exch{"2HN3 GG2"}="HN32 GG2";
$resn_exch{"1H17 GG2"}="H171 GG2"; $resn_exch{"2H17 GG2"}="H172 GG2"; $resn_exch{"1H2  GG3"}=" H21 GG3";
$resn_exch{"2H2  GG3"}=" H22 GG3"; $resn_exch{"1H6  GG3"}=" H61 GG3"; $resn_exch{"2H6  GG3"}=" H62 GG3";
$resn_exch{"1H7  GG3"}=" H71 GG3"; $resn_exch{"2H7  GG3"}=" H72 GG3"; $resn_exch{"1H21 GG3"}="H211 GG3";
$resn_exch{"2H21 GG3"}="H212 GG3"; $resn_exch{"3H21 GG3"}="H213 GG3"; $resn_exch{"1H22 GG3"}="H221 GG3";
$resn_exch{"2H22 GG3"}="H222 GG3"; $resn_exch{"1H26 GG3"}="H261 GG3"; $resn_exch{"2H26 GG3"}="H262 GG3";
$resn_exch{"1H27 GG3"}="H271 GG3"; $resn_exch{"2H27 GG3"}="H272 GG3"; $resn_exch{"1H3  GG4"}="H31A GG4";
$resn_exch{"2H3  GG4"}=" H32 GG4"; $resn_exch{"1H5  GG4"}=" H51 GG4"; $resn_exch{"2H5  GG4"}=" H52 GG4";
$resn_exch{"1H6  GG4"}=" H61 GG4"; $resn_exch{"2H6  GG4"}=" H62 GG4"; $resn_exch{"1H7  GG4"}=" H71 GG4";
$resn_exch{"2H7  GG4"}=" H72 GG4"; $resn_exch{"1H12 GG4"}="H121 GG4"; $resn_exch{"2H12 GG4"}="H122 GG4";
$resn_exch{"1H16 GG4"}="H161 GG4"; $resn_exch{"2H16 GG4"}="H162 GG4"; $resn_exch{"1H19 GG4"}="H191 GG4";
$resn_exch{"2H19 GG4"}="H192 GG4"; $resn_exch{"1H20 GG4"}="H201 GG4"; $resn_exch{"2H20 GG4"}="H202 GG4";
$resn_exch{"1H21 GG4"}="H211 GG4"; $resn_exch{"2H21 GG4"}="H212 GG4"; $resn_exch{"3H21 GG4"}="H213 GG4";
$resn_exch{"1H22 GG4"}="H221 GG4"; $resn_exch{"2H22 GG4"}="H222 GG4"; $resn_exch{"3H22 GG4"}="H223 GG4";
$resn_exch{"1H23 GG4"}="H231 GG4"; $resn_exch{"2H23 GG4"}="H232 GG4"; $resn_exch{"3H23 GG4"}="H233 GG4";
$resn_exch{"1H24 GG4"}="H241 GG4"; $resn_exch{"2H24 GG4"}="H242 GG4"; $resn_exch{"1H25 GG4"}="H251 GG4";
$resn_exch{"2H25 GG4"}="H252 GG4"; $resn_exch{"1H27 GG4"}="H271 GG4"; $resn_exch{"2H27 GG4"}="H272 GG4";
$resn_exch{"1H28 GG4"}="H281 GG4"; $resn_exch{"2H28 GG4"}="H282 GG4"; $resn_exch{"1H33 GG4"}="H331 GG4";
$resn_exch{"2H33 GG4"}="H332 GG4"; $resn_exch{"3H33 GG4"}="H333 GG4"; $resn_exch{"1H29 GG4"}="H291 GG4";
$resn_exch{"2H29 GG4"}="H292 GG4"; $resn_exch{"1H30 GG4"}="H301 GG4"; $resn_exch{"2H30 GG4"}="H302 GG4";
$resn_exch{"1H3  GG6"}=" H31 GG6"; $resn_exch{"2H3  GG6"}=" H32 GG6"; $resn_exch{"3H3  GG6"}=" H33 GG6";
$resn_exch{"1HB  GG7"}=" HB1 GG7"; $resn_exch{"2HB  GG7"}=" HB2 GG7"; $resn_exch{"3HB  GG7"}=" HB3 GG7";
$resn_exch{"1HN  GG7"}=" HN1 GG7"; $resn_exch{"2HN  GG7"}=" HN2 GG7"; $resn_exch{"1H6  GG9"}=" H61 GG9";
$resn_exch{"2H6  GG9"}=" H62 GG9"; $resn_exch{"1HN8 GGA"}="HN81 GGA"; $resn_exch{"2HN8 GGA"}="HN82 GGA";
$resn_exch{"1H10 GGA"}="H101 GGA"; $resn_exch{"2H10 GGA"}="H102 GGA"; $resn_exch{"1H12 GGA"}="H121 GGA";
$resn_exch{"2H12 GGA"}="H122 GGA"; $resn_exch{"1H22 GGA"}="H221 GGA"; $resn_exch{"2H22 GGA"}="H222 GGA";
$resn_exch{"3H22 GGA"}="H223 GGA"; $resn_exch{"1H29 GGA"}="H291 GGA"; $resn_exch{"2H29 GGA"}="H292 GGA";
$resn_exch{"1H33 GGA"}="H331 GGA"; $resn_exch{"2H33 GGA"}="H332 GGA"; $resn_exch{"1H35 GGA"}="H351 GGA";
$resn_exch{"2H35 GGA"}="H352 GGA"; $resn_exch{"1H37 GGA"}="H371 GGA"; $resn_exch{"2H37 GGA"}="H372 GGA";
$resn_exch{"1H39 GGA"}="H391 GGA"; $resn_exch{"2H39 GGA"}="H392 GGA"; $resn_exch{"1H43 GGA"}="H431 GGA";
$resn_exch{"2H43 GGA"}="H432 GGA"; $resn_exch{"1H45 GGA"}="H451 GGA"; $resn_exch{"2H45 GGA"}="H452 GGA";
$resn_exch{"1H47 GGA"}="H471 GGA"; $resn_exch{"2H47 GGA"}="H472 GGA"; $resn_exch{"1H49 GGA"}="H491 GGA";
$resn_exch{"2H49 GGA"}="H492 GGA"; $resn_exch{"1H54 GGA"}="H541 GGA"; $resn_exch{"2H54 GGA"}="H542 GGA";
$resn_exch{"1HN  GGB"}="HN1A GGB"; $resn_exch{"2HN  GGB"}=" HN2 GGB"; $resn_exch{"1HCB GGB"}="HCB1 GGB";
$resn_exch{"2HCB GGB"}="HCB2 GGB"; $resn_exch{"1HCG GGB"}="HCG1 GGB"; $resn_exch{"2HCG GGB"}="HCG2 GGB";
$resn_exch{"1HN2 GGB"}="HN21 GGB"; $resn_exch{"2HN2 GGB"}="HN22 GGB"; $resn_exch{"1HB1 GGC"}="HB11 GGC";
$resn_exch{"2HB1 GGC"}="HB12 GGC"; $resn_exch{"1HG1 GGC"}="HG11 GGC"; $resn_exch{"2HG1 GGC"}="HG12 GGC";
$resn_exch{"1HN1 GGC"}="HN11 GGC"; $resn_exch{"2HN1 GGC"}="HN12 GGC"; $resn_exch{"1HB2 GGC"}="HB21 GGC";
$resn_exch{"2HB2 GGC"}="HB22 GGC"; $resn_exch{"1H10 GGC"}="H101 GGC"; $resn_exch{"2H10 GGC"}="H102 GGC";
$resn_exch{"1HA3 GGC"}="HA31 GGC"; $resn_exch{"2HA3 GGC"}="HA32 GGC"; $resn_exch{"1HA1 GGD"}="HA11 GGD";
$resn_exch{"1HA2 GGD"}="HA21 GGD"; $resn_exch{"1HA3 GGD"}="HA31 GGD"; $resn_exch{"1HA4 GGD"}="HA41 GGD";
$resn_exch{"1HA5 GGD"}="HA51 GGD"; $resn_exch{"1HA6 GGD"}="HA61 GGD"; $resn_exch{"2HA6 GGD"}="HA62 GGD";
$resn_exch{"1HO2 GGD"}="HO21 GGD"; $resn_exch{"1HO4 GGD"}="HO41 GGD"; $resn_exch{"1HO6 GGD"}="HO61 GGD";
$resn_exch{"1HB1 GGD"}="HB11 GGD"; $resn_exch{"1HB2 GGD"}="HB21 GGD"; $resn_exch{"1HB3 GGD"}="HB31 GGD";
$resn_exch{"1HB4 GGD"}="HB41 GGD"; $resn_exch{"1HB5 GGD"}="HB51 GGD"; $resn_exch{"1HB6 GGD"}="HB61 GGD";
$resn_exch{"2HB6 GGD"}="HB62 GGD"; $resn_exch{"1HC3 GGD"}="HC31 GGD"; $resn_exch{"2HC3 GGD"}="HC32 GGD";
$resn_exch{"1HC4 GGD"}="HC41 GGD"; $resn_exch{"1HC6 GGD"}="HC61 GGD"; $resn_exch{"2HC6 GGD"}="HC62 GGD";
$resn_exch{"1H14 GGD"}="H141 GGD"; $resn_exch{"2H14 GGD"}="H142 GGD"; $resn_exch{"1H15 GGD"}="H151 GGD";
$resn_exch{"2H15 GGD"}="H152 GGD"; $resn_exch{"1H16 GGD"}="H161 GGD"; $resn_exch{"2H16 GGD"}="H162 GGD";
$resn_exch{"1H17 GGD"}="H171 GGD"; $resn_exch{"2H17 GGD"}="H172 GGD"; $resn_exch{"1H18 GGD"}="H181 GGD";
$resn_exch{"2H18 GGD"}="H182 GGD"; $resn_exch{"1H19 GGD"}="H191 GGD"; $resn_exch{"2H19 GGD"}="H192 GGD";
$resn_exch{"1H20 GGD"}="H201 GGD"; $resn_exch{"2H20 GGD"}="H202 GGD"; $resn_exch{"1H21 GGD"}="H211 GGD";
$resn_exch{"2H21 GGD"}="H212 GGD"; $resn_exch{"1H22 GGD"}="H221 GGD"; $resn_exch{"1H23 GGD"}="H231 GGD";
$resn_exch{"1H24 GGD"}="H241 GGD"; $resn_exch{"2H24 GGD"}="H242 GGD"; $resn_exch{"1H31 GGD"}="H311 GGD";
$resn_exch{"2H31 GGD"}="H312 GGD"; $resn_exch{"1H32 GGD"}="H321 GGD"; $resn_exch{"2H32 GGD"}="H322 GGD";
$resn_exch{"1H33 GGD"}="H331 GGD"; $resn_exch{"2H33 GGD"}="H332 GGD"; $resn_exch{"1H34 GGD"}="H341 GGD";
$resn_exch{"2H34 GGD"}="H342 GGD"; $resn_exch{"1H35 GGD"}="H351 GGD"; $resn_exch{"2H35 GGD"}="H352 GGD";
$resn_exch{"1H36 GGD"}="H361 GGD"; $resn_exch{"2H36 GGD"}="H362 GGD"; $resn_exch{"1H37 GGD"}="H371 GGD";
$resn_exch{"2H37 GGD"}="H372 GGD"; $resn_exch{"1H38 GGD"}="H381 GGD"; $resn_exch{"1H39 GGD"}="H391 GGD";
$resn_exch{"1H40 GGD"}="H401 GGD"; $resn_exch{"2H40 GGD"}="H402 GGD"; $resn_exch{"1H41 GGD"}="H411 GGD";
$resn_exch{"2H41 GGD"}="H412 GGD"; $resn_exch{"1H42 GGD"}="H421 GGD"; $resn_exch{"2H42 GGD"}="H422 GGD";
$resn_exch{"1H43 GGD"}="H431 GGD"; $resn_exch{"2H43 GGD"}="H432 GGD"; $resn_exch{"1H44 GGD"}="H441 GGD";
$resn_exch{"2H44 GGD"}="H442 GGD"; $resn_exch{"1H25 GGD"}="H251 GGD"; $resn_exch{"2H25 GGD"}="H252 GGD";
$resn_exch{"1H26 GGD"}="H261 GGD"; $resn_exch{"2H26 GGD"}="H262 GGD"; $resn_exch{"1H27 GGD"}="H271 GGD";
$resn_exch{"2H27 GGD"}="H272 GGD"; $resn_exch{"1H28 GGD"}="H281 GGD"; $resn_exch{"2H28 GGD"}="H282 GGD";
$resn_exch{"1H29 GGD"}="H291 GGD"; $resn_exch{"2H29 GGD"}="H292 GGD"; $resn_exch{"1H30 GGD"}="H301 GGD";
$resn_exch{"2H30 GGD"}="H302 GGD"; $resn_exch{"1H48 GGD"}="H481 GGD"; $resn_exch{"2H48 GGD"}="H482 GGD";
$resn_exch{"3H48 GGD"}="H483 GGD"; $resn_exch{"1H45 GGD"}="H451 GGD"; $resn_exch{"2H45 GGD"}="H452 GGD";
$resn_exch{"1H46 GGD"}="H461 GGD"; $resn_exch{"2H46 GGD"}="H462 GGD"; $resn_exch{"1H47 GGD"}="H471 GGD";
$resn_exch{"2H47 GGD"}="H472 GGD"; $resn_exch{"3H47 GGD"}="H473 GGD"; $resn_exch{"1H3B GGH"}="H3B1 GGH";
$resn_exch{"2H3B GGH"}="H3B2 GGH"; $resn_exch{"1H5' GGH"}="H5'1 GGH"; $resn_exch{"2H5' GGH"}="H5'2 GGH";
$resn_exch{"1HN2 GGH"}="HN21 GGH"; $resn_exch{"2HN2 GGH"}="HN22 GGH"; $resn_exch{"1H2' GGH"}="H2'1 GGH";
$resn_exch{"2H2' GGH"}="H2'2 GGH"; $resn_exch{"1HN  GGL"}=" HN1 GGL"; $resn_exch{"2HN  GGL"}=" HN2 GGL";
$resn_exch{"1HB  GGL"}=" HB1 GGL"; $resn_exch{"2HB  GGL"}=" HB2 GGL"; $resn_exch{"1HG  GGL"}=" HG1 GGL";
$resn_exch{"2HG  GGL"}=" HG2 GGL"; $resn_exch{"1H5  GGO"}=" H51 GGO"; $resn_exch{"2H5  GGO"}=" H52 GGO";
$resn_exch{"1H6  GGO"}=" H61 GGO"; $resn_exch{"2H6  GGO"}=" H62 GGO"; $resn_exch{"1HN  GGO"}=" HN1 GGO";
$resn_exch{"2HN  GGO"}=" HN2 GGO"; $resn_exch{"1H3  GGO"}=" H31 GGO"; $resn_exch{"2H3  GGO"}=" H32 GGO";
$resn_exch{"1H12 GGO"}="H121 GGO"; $resn_exch{"2H12 GGO"}="H122 GGO"; $resn_exch{"1H18 GGO"}="H181 GGO";
$resn_exch{"2H18 GGO"}="H182 GGO"; $resn_exch{"1H30 GGP"}="H301 GGP"; $resn_exch{"2H30 GGP"}="H302 GGP";
$resn_exch{" H3* GGZ"}=" H3' GGZ"; $resn_exch{" H4* GGZ"}=" H4' GGZ"; $resn_exch{"1H5* GGZ"}="H5'1 GGZ";
$resn_exch{"2H5* GGZ"}="H5'2 GGZ"; $resn_exch{" H2* GGZ"}=" H2' GGZ"; $resn_exch{" H1* GGZ"}=" H1' GGZ";
$resn_exch{"1HN6 GGZ"}="HN61 GGZ"; $resn_exch{"2HN6 GGZ"}="HN62 GGZ"; $resn_exch{"2HOG GH3"}="HOG2 GH3";
$resn_exch{"3HOG GH3"}="HOG3 GH3"; $resn_exch{"2HOB GH3"}="HOB2 GH3"; $resn_exch{"2HOA GH3"}="HOA2 GH3";
$resn_exch{"1H5* GH3"}=" H5' GH3"; $resn_exch{"2H5* GH3"}="H5'' GH3"; $resn_exch{" H4* GH3"}=" H4' GH3";
$resn_exch{"2H3* GH3"}="H3'2 GH3"; $resn_exch{" H3* GH3"}=" H3' GH3"; $resn_exch{" H2* GH3"}=" H2' GH3";
$resn_exch{"*HO2 GH3"}="HO2' GH3"; $resn_exch{" H1* GH3"}=" H1' GH3"; $resn_exch{"1HN2 GH3"}="HN21 GH3";
$resn_exch{"2HN2 GH3"}="HN22 GH3"; $resn_exch{"1H3  GHA"}=" H31 GHA"; $resn_exch{"2H3  GHA"}=" H32 GHA";
$resn_exch{"1H6  GHA"}=" H61 GHA"; $resn_exch{"2H6  GHA"}=" H62 GHA"; $resn_exch{"1H7  GHA"}=" H71 GHA";
$resn_exch{"2H7  GHA"}=" H72 GHA"; $resn_exch{"1H10 GHA"}="H101 GHA"; $resn_exch{"2H10 GHA"}="H102 GHA";
$resn_exch{"1HN  GHG"}=" H   GHG"; $resn_exch{"2HN  GHG"}=" H2  GHG"; $resn_exch{"1HB  GHG"}=" HB2 GHG";
$resn_exch{"2HB  GHG"}=" HB3 GHG"; $resn_exch{"1HE2 GHG"}="HE21 GHG"; $resn_exch{"2HE2 GHG"}="HE22 GHG";
$resn_exch{"1HN  GHP"}=" H   GHP"; $resn_exch{"2HN  GHP"}=" H2  GHP"; $resn_exch{" H2  GHP"}=" HC2 GHP";
$resn_exch{"1H6C GI1"}="H6C1 GI1"; $resn_exch{"2H6C GI1"}="H6C2 GI1"; $resn_exch{"1H9C GI1"}="H9C1 GI1";
$resn_exch{"2H9C GI1"}="H9C2 GI1"; $resn_exch{"3H9C GI1"}="H9C3 GI1"; $resn_exch{"1H1C GI2"}="H1C1 GI2";
$resn_exch{"2H1C GI2"}="H1C2 GI2"; $resn_exch{"1H4C GI2"}="H4C1 GI2"; $resn_exch{"2H4C GI2"}="H4C2 GI2";
$resn_exch{"3H4C GI2"}="H4C3 GI2"; $resn_exch{"1H6B GI2"}="H6B1 GI2"; $resn_exch{"2H6B GI2"}="H6B2 GI2";
$resn_exch{"1H6C GI3"}="H6C1 GI3"; $resn_exch{"2H6C GI3"}="H6C2 GI3"; $resn_exch{"1H1C GI4"}="H1C1 GI4";
$resn_exch{"2H1C GI4"}="H1C2 GI4"; $resn_exch{"1H6B GI4"}="H6B1 GI4"; $resn_exch{"2H6B GI4"}="H6B2 GI4";
$resn_exch{"1H36 GIG"}="H361 GIG"; $resn_exch{"2H36 GIG"}="H362 GIG"; $resn_exch{"3H36 GIG"}="H363 GIG";
$resn_exch{"1H1N GIL"}="H1N1 GIL"; $resn_exch{"2H1N GIL"}="H1N2 GIL"; $resn_exch{"1H17 GIL"}="H171 GIL";
$resn_exch{"2H17 GIL"}="H172 GIL"; $resn_exch{"3H17 GIL"}="H173 GIL"; $resn_exch{"1H18 GIL"}="H181 GIL";
$resn_exch{"2H18 GIL"}="H182 GIL"; $resn_exch{"3H18 GIL"}="H183 GIL"; $resn_exch{"1H19 GIL"}="H191 GIL";
$resn_exch{"2H19 GIL"}="H192 GIL"; $resn_exch{"3H19 GIL"}="H193 GIL"; $resn_exch{"1H20 GIL"}="H201 GIL";
$resn_exch{"2H20 GIL"}="H202 GIL"; $resn_exch{"3H20 GIL"}="H203 GIL"; $resn_exch{"1H9C GIL"}="H9C1 GIL";
$resn_exch{"2H9C GIL"}="H9C2 GIL"; $resn_exch{"1H28 GIL"}="H281 GIL"; $resn_exch{"2H28 GIL"}="H282 GIL";
$resn_exch{"3H28 GIL"}="H283 GIL"; $resn_exch{"1H30 GIL"}="H301 GIL"; $resn_exch{"2H30 GIL"}="H302 GIL";
$resn_exch{"3H30 GIL"}="H303 GIL"; $resn_exch{"1H6C GIM"}="H6C1 GIM"; $resn_exch{"2H6C GIM"}="H6C2 GIM";
$resn_exch{"1H24 GIN"}="H241 GIN"; $resn_exch{"2H24 GIN"}="H242 GIN"; $resn_exch{"1H30 GIN"}="H301 GIN";
$resn_exch{"2H30 GIN"}="H302 GIN"; $resn_exch{"1H31 GIN"}="H311 GIN"; $resn_exch{"2H31 GIN"}="H312 GIN";
$resn_exch{"3H31 GIN"}="H313 GIN"; $resn_exch{"1H32 GIN"}="H321 GIN"; $resn_exch{"2H32 GIN"}="H322 GIN";
$resn_exch{"1HBP GIO"}="HBP1 GIO"; $resn_exch{"2HBP GIO"}="HBP2 GIO"; $resn_exch{"1HGP GIO"}="HGP1 GIO";
$resn_exch{"2HGP GIO"}="HGP2 GIO"; $resn_exch{"1HDP GIO"}="HDP1 GIO"; $resn_exch{"2HDP GIO"}="HDP2 GIO";
$resn_exch{"1HAC GIO"}="HAC1 GIO"; $resn_exch{"2HAC GIO"}="HAC2 GIO"; $resn_exch{"1HOT GIP"}="HOT1 GIP";
$resn_exch{"1NH1 GIP"}="H11N GIP"; $resn_exch{"2NH1 GIP"}="H12N GIP"; $resn_exch{"1HA  GIP"}=" HA1 GIP";
$resn_exch{"1HB  GIP"}=" HB1 GIP"; $resn_exch{"2HB  GIP"}=" HB2 GIP"; $resn_exch{"1HB2 GIP"}="HB21 GIP";
$resn_exch{"2HB2 GIP"}="HB22 GIP"; $resn_exch{"1HG  GIP"}=" HG1 GIP"; $resn_exch{"2HG  GIP"}=" HG2 GIP";
$resn_exch{"1HN2 GIP"}="HN21 GIP"; $resn_exch{"1HA3 GIP"}="HA31 GIP"; $resn_exch{"2HA3 GIP"}="HA32 GIP";
$resn_exch{"1HO3 GIP"}="HO31 GIP"; $resn_exch{"1H10 GIQ"}="H101 GIQ"; $resn_exch{"2H10 GIQ"}="H102 GIQ";
$resn_exch{"1HN1 GIR"}="HN11 GIR"; $resn_exch{"2HN1 GIR"}="HN12 GIR"; $resn_exch{"1H2  GIR"}=" H21 GIR";
$resn_exch{"2H2  GIR"}=" H22 GIR"; $resn_exch{"1H11 GIR"}="H111 GIR"; $resn_exch{"2H11 GIR"}="H112 GIR";
$resn_exch{"1H11 GIS"}="H111 GIS"; $resn_exch{"2H11 GIS"}="H112 GIS"; $resn_exch{"1H12 GIS"}="H121 GIS";
$resn_exch{"2H12 GIS"}="H122 GIS"; $resn_exch{"3H12 GIS"}="H123 GIS"; $resn_exch{"1HN3 GIS"}="HN31 GIS";
$resn_exch{"2HN3 GIS"}="HN32 GIS"; $resn_exch{"1H2N GKD"}="H2N1 GKD"; $resn_exch{"2H2N GKD"}="H2N2 GKD";
$resn_exch{" H1* GKD"}=" H1' GKD"; $resn_exch{" H2* GKD"}=" H2' GKD"; $resn_exch{" H3* GKD"}=" H3' GKD";
$resn_exch{" H4* GKD"}=" H4' GKD"; $resn_exch{"1H5* GKD"}="H5'1 GKD"; $resn_exch{"2H5* GKD"}="H5'2 GKD";
$resn_exch{"1H6G GKD"}="H6G1 GKD"; $resn_exch{"2H6G GKD"}="H6G2 GKD"; $resn_exch{"1H2N GKE"}="H2N1 GKE";
$resn_exch{"2HA  GKE"}=" HA2 GKE"; $resn_exch{" H2* GKE"}=" H2' GKE"; $resn_exch{" H3* GKE"}=" H3' GKE";
$resn_exch{"1HA  GKE"}=" HA1 GKE"; $resn_exch{"2HB  GKE"}=" HB2 GKE"; $resn_exch{"1HB  GKE"}=" HB1 GKE";
$resn_exch{"2HC  GKE"}=" HC2 GKE"; $resn_exch{"1H2  GKR"}=" H21 GKR"; $resn_exch{"1HO2 GKR"}="HO21 GKR";
$resn_exch{"1H3  GKR"}=" H31 GKR"; $resn_exch{"1HO3 GKR"}="HO31 GKR"; $resn_exch{"2H4  GKR"}=" H42 GKR";
$resn_exch{"1H4  GKR"}=" H41 GKR"; $resn_exch{"1HO5 GKR"}="HO51 GKR"; $resn_exch{"1H5  GKR"}=" H51 GKR";
$resn_exch{"1H6  GL1"}=" H61 GL1"; $resn_exch{"2H6  GL1"}=" H62 GL1"; $resn_exch{"1H6  GL2"}=" H61 GL2";
$resn_exch{"2H6  GL2"}=" H62 GL2"; $resn_exch{"1HN3 GL2"}="HN31 GL2"; $resn_exch{"2HN3 GL2"}="HN32 GL2";
$resn_exch{"1HA  GL3"}=" HA1 GL3"; $resn_exch{"2HA  GL3"}=" HA2 GL3"; $resn_exch{"1HN  GL3"}=" H   GL3";
$resn_exch{"2HN  GL3"}=" H2  GL3"; $resn_exch{"1H6  GL4"}=" H61 GL4"; $resn_exch{"2H6  GL4"}=" H62 GL4";
$resn_exch{"1H6  GL5"}=" H61 GL5"; $resn_exch{"2H6  GL5"}=" H62 GL5"; $resn_exch{"1H15 GL6"}="H151 GL6";
$resn_exch{"2H15 GL6"}="H152 GL6"; $resn_exch{"3H15 GL6"}="H153 GL6"; $resn_exch{"1H16 GL6"}="H161 GL6";
$resn_exch{"2H16 GL6"}="H162 GL6"; $resn_exch{"3H16 GL6"}="H163 GL6"; $resn_exch{"1H6  GL6"}=" H61 GL6";
$resn_exch{"2H6  GL6"}=" H62 GL6"; $resn_exch{"1H6  GL7"}=" H61 GL7"; $resn_exch{"2H6  GL7"}=" H62 GL7";
$resn_exch{"1H9  GL7"}=" H91 GL7"; $resn_exch{"2H9  GL7"}=" H92 GL7"; $resn_exch{"3H9  GL7"}=" H93 GL7";
$resn_exch{"1H4C GL8"}="H4C1 GL8"; $resn_exch{"2H4C GL8"}="H4C2 GL8"; $resn_exch{"1H11 GL8"}="H111 GL8";
$resn_exch{"2H11 GL8"}="H112 GL8"; $resn_exch{"1H12 GL8"}="H121 GL8"; $resn_exch{"2H12 GL8"}="H122 GL8";
$resn_exch{"1H16 GL8"}="H161 GL8"; $resn_exch{"2H16 GL8"}="H162 GL8"; $resn_exch{"3H16 GL8"}="H163 GL8";
$resn_exch{"1H17 GL8"}="H171 GL8"; $resn_exch{"2H17 GL8"}="H172 GL8"; $resn_exch{"1H18 GL8"}="H181 GL8";
$resn_exch{"2H18 GL8"}="H182 GL8"; $resn_exch{"1H19 GL8"}="H191 GL8"; $resn_exch{"2H19 GL8"}="H192 GL8";
$resn_exch{"1H20 GL8"}="H201 GL8"; $resn_exch{"2H20 GL8"}="H202 GL8"; $resn_exch{"1H21 GL8"}="H211 GL8";
$resn_exch{"2H21 GL8"}="H212 GL8"; $resn_exch{"1H22 GL8"}="H221 GL8"; $resn_exch{"2H22 GL8"}="H222 GL8";
$resn_exch{"1H23 GL8"}="H231 GL8"; $resn_exch{"2H23 GL8"}="H232 GL8"; $resn_exch{"1H24 GL8"}="H241 GL8";
$resn_exch{"2H24 GL8"}="H242 GL8"; $resn_exch{"1H6  GL9"}=" H61 GL9"; $resn_exch{"2H6  GL9"}=" H62 GL9";
$resn_exch{"1H10 GL9"}="H101 GL9"; $resn_exch{"2H10 GL9"}="H102 GL9"; $resn_exch{"3H10 GL9"}="H103 GL9";
$resn_exch{"1H6  GLA"}=" H61 GLA"; $resn_exch{"2H6  GLA"}=" H62 GLA"; $resn_exch{"1H6  GLB"}=" H61 GLB";
$resn_exch{"2H6  GLB"}=" H62 GLB"; $resn_exch{"1H4  GLD"}=" H41 GLD"; $resn_exch{"2H4  GLD"}=" H42 GLD";
$resn_exch{"1H6  GLD"}=" H61 GLD"; $resn_exch{"2H6  GLD"}=" H62 GLD"; $resn_exch{"3H6  GLD"}=" H63 GLD";
$resn_exch{"1H1  GLE"}=" H11 GLE"; $resn_exch{"2H1  GLE"}=" H12 GLE"; $resn_exch{"1H3  GLE"}=" H31 GLE";
$resn_exch{"2H3  GLE"}=" H32 GLE"; $resn_exch{"1H31 GLE"}="H311 GLE"; $resn_exch{"2H31 GLE"}="H312 GLE";
$resn_exch{"1H2P GLE"}="H2P1 GLE"; $resn_exch{"2H2P GLE"}="H2P2 GLE"; $resn_exch{"3H2P GLE"}="H2P3 GLE";
$resn_exch{"1H11 GLE"}="H111 GLE"; $resn_exch{"2H11 GLE"}="H112 GLE"; $resn_exch{"1H12 GLE"}="H121 GLE";
$resn_exch{"2H12 GLE"}="H122 GLE"; $resn_exch{"1H13 GLE"}="H131 GLE"; $resn_exch{"2H13 GLE"}="H132 GLE";
$resn_exch{"1H14 GLE"}="H141 GLE"; $resn_exch{"2H14 GLE"}="H142 GLE"; $resn_exch{"1H15 GLE"}="H151 GLE";
$resn_exch{"2H15 GLE"}="H152 GLE"; $resn_exch{"1H16 GLE"}="H161 GLE"; $resn_exch{"2H16 GLE"}="H162 GLE";
$resn_exch{"1H17 GLE"}="H171 GLE"; $resn_exch{"2H17 GLE"}="H172 GLE"; $resn_exch{"1H18 GLE"}="H181 GLE";
$resn_exch{"2H18 GLE"}="H182 GLE"; $resn_exch{"1H19 GLE"}="H191 GLE"; $resn_exch{"2H19 GLE"}="H192 GLE";
$resn_exch{"1H20 GLE"}="H201 GLE"; $resn_exch{"2H20 GLE"}="H202 GLE"; $resn_exch{"3H20 GLE"}="H203 GLE";
$resn_exch{"1H6  GLF"}=" H61 GLF"; $resn_exch{"2H6  GLF"}=" H62 GLF"; $resn_exch{"1H6  GLG"}=" H61 GLG";
$resn_exch{"2H6  GLG"}=" H62 GLG"; $resn_exch{"1HN  GLG"}=" HN1 GLG"; $resn_exch{"2HN  GLG"}=" HN2 GLG";
$resn_exch{"1HN  GLH"}=" H   GLH"; $resn_exch{"2HN  GLH"}=" H2  GLH"; $resn_exch{"1HCB GLH"}=" HB2 GLH";
$resn_exch{"2HCB GLH"}=" HB3 GLH"; $resn_exch{"1HCG GLH"}=" HG2 GLH"; $resn_exch{"2HCG GLH"}=" HG3 GLH";
$resn_exch{"1HC3 GLH"}="HC31 GLH"; $resn_exch{"2HC3 GLH"}="HC32 GLH"; $resn_exch{"1HC7 GLH"}="HC71 GLH";
$resn_exch{"2HC7 GLH"}="HC72 GLH"; $resn_exch{"1HC4 GLH"}="HC41 GLH"; $resn_exch{"2HC4 GLH"}="HC42 GLH";
$resn_exch{"1HC5 GLH"}="HC51 GLH"; $resn_exch{"2HC5 GLH"}="HC52 GLH"; $resn_exch{"1HC6 GLH"}="HC61 GLH";
$resn_exch{"2HC6 GLH"}="HC62 GLH"; $resn_exch{"1HC9 GLH"}="HC91 GLH"; $resn_exch{"2HC9 GLH"}="HC92 GLH";
$resn_exch{"1H13 GLH"}="H131 GLH"; $resn_exch{"2H13 GLH"}="H132 GLH"; $resn_exch{"1H10 GLH"}="H101 GLH";
$resn_exch{"2H10 GLH"}="H102 GLH"; $resn_exch{"1H11 GLH"}="H111 GLH"; $resn_exch{"2H11 GLH"}="H112 GLH";
$resn_exch{"1H12 GLH"}="H121 GLH"; $resn_exch{"2H12 GLH"}="H122 GLH"; $resn_exch{"1HN  GLI"}=" HN1 GLI";
$resn_exch{"2HN  GLI"}="HN2A GLI"; $resn_exch{"1HB1 GLI"}="HB11 GLI"; $resn_exch{"2HB1 GLI"}="HB12 GLI";
$resn_exch{"3HB1 GLI"}="HB13 GLI"; $resn_exch{"1HB2 GLI"}="HB21 GLI"; $resn_exch{"2HB2 GLI"}="HB22 GLI";
$resn_exch{"3HB2 GLI"}="HB23 GLI"; $resn_exch{"1HB3 GLI"}="HB31 GLI"; $resn_exch{"2HB3 GLI"}="HB32 GLI";
$resn_exch{"1HD3 GLI"}="HD31 GLI"; $resn_exch{"2HD3 GLI"}="HD32 GLI"; $resn_exch{"1HE3 GLI"}="HE31 GLI";
$resn_exch{"2HE3 GLI"}="HE32 GLI"; $resn_exch{"1HX4 GLI"}="HX41 GLI"; $resn_exch{"2HX4 GLI"}="HX42 GLI";
$resn_exch{"1HA4 GLI"}="HA41 GLI"; $resn_exch{"2HA4 GLI"}="HA42 GLI"; $resn_exch{"1H51 GLI"}="H511 GLI";
$resn_exch{"2H51 GLI"}="H512 GLI"; $resn_exch{"3H51 GLI"}="H513 GLI"; $resn_exch{"1H52 GLI"}="H521 GLI";
$resn_exch{"2H52 GLI"}="H522 GLI"; $resn_exch{"3H52 GLI"}="H523 GLI"; $resn_exch{"1H61 GLI"}="H611 GLI";
$resn_exch{"2H61 GLI"}="H612 GLI"; $resn_exch{"3H61 GLI"}="H613 GLI"; $resn_exch{"1H62 GLI"}="H621 GLI";
$resn_exch{"2H62 GLI"}="H622 GLI"; $resn_exch{"3H62 GLI"}="H623 GLI"; $resn_exch{"1HM6 GLI"}="HM61 GLI";
$resn_exch{"2HM6 GLI"}="HM62 GLI"; $resn_exch{"3HM6 GLI"}="HM63 GLI"; $resn_exch{"'HN1 GLL"}="HN1' GLL";
$resn_exch{"'HN2 GLL"}="HN2' GLL"; $resn_exch{"1HN  GLM"}=" H   GLM"; $resn_exch{"2HN  GLM"}=" H2  GLM";
$resn_exch{"1HA  GLM"}=" HA1 GLM"; $resn_exch{"2HA  GLM"}=" HA2 GLM"; $resn_exch{"1HM  GLM"}=" HM1 GLM";
$resn_exch{"2HM  GLM"}=" HM2 GLM"; $resn_exch{"3HM  GLM"}=" HM3 GLM"; $resn_exch{" HN2 GLN"}=" H2  GLN";
$resn_exch{"1HB  GLN"}=" HB2 GLN"; $resn_exch{"2HB  GLN"}=" HB3 GLN"; $resn_exch{"1HG  GLN"}=" HG2 GLN";
$resn_exch{"2HG  GLN"}=" HG3 GLN"; $resn_exch{"1HE2 GLN"}="HE21 GLN"; $resn_exch{"2HE2 GLN"}="HE22 GLN";
$resn_exch{"1H6  GLO"}=" H61 GLO"; $resn_exch{"2H6  GLO"}=" H62 GLO"; $resn_exch{"1H6  GLP"}=" H61 GLP";
$resn_exch{"2H6  GLP"}=" H62 GLP"; $resn_exch{"1HN2 GLP"}="HN21 GLP"; $resn_exch{"2HN2 GLP"}="HN22 GLP";
$resn_exch{"2HOP GLP"}="HOP2 GLP"; $resn_exch{"3HOP GLP"}="HOP3 GLP"; $resn_exch{" HN2 GLQ"}=" H2  GLQ";
$resn_exch{"1HB  GLQ"}=" HB1 GLQ"; $resn_exch{"2HB  GLQ"}=" HB2 GLQ"; $resn_exch{"1HG  GLQ"}=" HG1 GLQ";
$resn_exch{"2HG  GLQ"}=" HG2 GLQ"; $resn_exch{"1H2  GLR"}=" H21 GLR"; $resn_exch{"1HO2 GLR"}="HO21 GLR";
$resn_exch{"1H3  GLR"}=" H31 GLR"; $resn_exch{"1HO3 GLR"}="HO31 GLR"; $resn_exch{"1H4  GLR"}=" H41 GLR";
$resn_exch{"2H4  GLR"}=" H42 GLR"; $resn_exch{"1H6  GLS"}=" H61 GLS"; $resn_exch{"2H6  GLS"}=" H62 GLS";
$resn_exch{"1H6  GLT"}=" H61 GLT"; $resn_exch{"2H6  GLT"}=" H62 GLT"; $resn_exch{" HN2 GLU"}=" H2  GLU";
$resn_exch{"1HB  GLU"}=" HB2 GLU"; $resn_exch{"2HB  GLU"}=" HB3 GLU"; $resn_exch{"1HG  GLU"}=" HG2 GLU";
$resn_exch{"2HG  GLU"}=" HG3 GLU"; $resn_exch{"1H6  GLW"}=" H61 GLW"; $resn_exch{"2H6  GLW"}=" H62 GLW";
$resn_exch{"3H6  GLW"}=" H63 GLW"; $resn_exch{"1HB  GLX"}=" HB1 GLX"; $resn_exch{"2HB  GLX"}=" HB2 GLX";
$resn_exch{"1HG  GLX"}=" HG1 GLX"; $resn_exch{"2HG  GLX"}=" HG2 GLX"; $resn_exch{" HN2 GLY"}=" H2  GLY";
$resn_exch{"1HA  GLY"}=" HA2 GLY"; $resn_exch{"2HA  GLY"}=" HA3 GLY"; $resn_exch{" HN2 GLZ"}=" H2  GLZ";
$resn_exch{"1HA  GLZ"}=" HA1 GLZ"; $resn_exch{"2HA  GLZ"}=" HA2 GLZ"; $resn_exch{"1HN1 GM1"}="HN11 GM1";
$resn_exch{"2HN1 GM1"}="HN12 GM1"; $resn_exch{"1H2  GM1"}=" H21 GM1"; $resn_exch{"2H2  GM1"}=" H22 GM1";
$resn_exch{"1HN2 GM1"}="HN21 GM1"; $resn_exch{"2HN2 GM1"}="HN22 GM1"; $resn_exch{"1HB1 GM2"}="HB11 GM2";
$resn_exch{"2HB1 GM2"}="HB12 GM2"; $resn_exch{"1H6C GM2"}="H6C1 GM2"; $resn_exch{"2H6C GM2"}="H6C2 GM2";
$resn_exch{"1HL6 GM2"}="HL61 GM2"; $resn_exch{"2HL6 GM2"}="HL62 GM2"; $resn_exch{"1HL5 GM2"}="HL51 GM2";
$resn_exch{"2HL5 GM2"}="HL52 GM2"; $resn_exch{"1HL7 GM2"}="HL71 GM2"; $resn_exch{"2HL7 GM2"}="HL72 GM2";
$resn_exch{"1HL4 GM2"}="HL41 GM2"; $resn_exch{"2HL4 GM2"}="HL42 GM2"; $resn_exch{"1HL3 GM2"}="HL31 GM2";
$resn_exch{"2HL3 GM2"}="HL32 GM2"; $resn_exch{"1HL2 GM2"}="HL21 GM2"; $resn_exch{"2HL2 GM2"}="HL22 GM2";
$resn_exch{"1HR2 GM2"}="HR21 GM2"; $resn_exch{"2HR2 GM2"}="HR22 GM2"; $resn_exch{"1HR3 GM2"}="HR31 GM2";
$resn_exch{"2HR3 GM2"}="HR32 GM2"; $resn_exch{"1HR4 GM2"}="HR41 GM2"; $resn_exch{"2HR4 GM2"}="HR42 GM2";
$resn_exch{"1HR5 GM2"}="HR51 GM2"; $resn_exch{"2HR5 GM2"}="HR52 GM2"; $resn_exch{"1HR6 GM2"}="HR61 GM2";
$resn_exch{"2HR6 GM2"}="HR62 GM2"; $resn_exch{"1HR7 GM2"}="HR71 GM2"; $resn_exch{"2HR7 GM2"}="HR72 GM2";
$resn_exch{"1HR8 GM2"}="HR81 GM2"; $resn_exch{"2HR8 GM2"}="HR82 GM2"; $resn_exch{"1HR9 GM2"}="HR91 GM2";
$resn_exch{"2HR9 GM2"}="HR92 GM2"; $resn_exch{"1HR1 GM2"}="HR11 GM2"; $resn_exch{"1HL8 GM2"}="HL81 GM2";
$resn_exch{"2HL8 GM2"}="HL82 GM2"; $resn_exch{"1HL9 GM2"}="HL91 GM2"; $resn_exch{"2HL9 GM2"}="HL92 GM2";
$resn_exch{"1HL1 GM2"}="HL11 GM2"; $resn_exch{"2HL1 GM2"}="HL12 GM2"; $resn_exch{"1HBC GM3"}="HBC1 GM3";
$resn_exch{"2HBC GM3"}="HBC2 GM3"; $resn_exch{"1HBD GM3"}="HBD1 GM3"; $resn_exch{"2HBD GM3"}="HBD2 GM3";
$resn_exch{"1HBE GM3"}="HBE1 GM3"; $resn_exch{"2HBE GM3"}="HBE2 GM3"; $resn_exch{"1HBF GM3"}="HBF1 GM3";
$resn_exch{"2HBF GM3"}="HBF2 GM3"; $resn_exch{"1HBG GM3"}="HBG1 GM3"; $resn_exch{"2HBG GM3"}="HBG2 GM3";
$resn_exch{"1HBH GM3"}="HBH1 GM3"; $resn_exch{"2HBH GM3"}="HBH2 GM3"; $resn_exch{"1HBI GM3"}="HBI1 GM3";
$resn_exch{"2HBI GM3"}="HBI2 GM3"; $resn_exch{"1HBJ GM3"}="HBJ1 GM3"; $resn_exch{"2HBJ GM3"}="HBJ2 GM3";
$resn_exch{"1HBK GM3"}="HBK1 GM3"; $resn_exch{"2HBK GM3"}="HBK2 GM3"; $resn_exch{"1HBL GM3"}="HBL1 GM3";
$resn_exch{"2HBL GM3"}="HBL2 GM3"; $resn_exch{"1HBM GM3"}="HBM1 GM3"; $resn_exch{"2HBM GM3"}="HBM2 GM3";
$resn_exch{"1HBN GM3"}="HBN1 GM3"; $resn_exch{"2HBN GM3"}="HBN2 GM3"; $resn_exch{"1HBO GM3"}="HBO1 GM3";
$resn_exch{"2HBO GM3"}="HBO2 GM3"; $resn_exch{"1HBP GM3"}="HBP1 GM3"; $resn_exch{"2HBP GM3"}="HBP2 GM3";
$resn_exch{"1HBQ GM3"}="HBQ1 GM3"; $resn_exch{"2HBQ GM3"}="HBQ2 GM3"; $resn_exch{"1HB1 GM3"}="HB11 GM3";
$resn_exch{"2HB1 GM3"}="HB12 GM3"; $resn_exch{"1HB2 GM3"}="HB21 GM3"; $resn_exch{"2HB2 GM3"}="HB22 GM3";
$resn_exch{"3HB2 GM3"}="HB23 GM3"; $resn_exch{"1HAX GM3"}="HAX1 GM3"; $resn_exch{"2HAX GM3"}="HAX2 GM3";
$resn_exch{"1HAV GM3"}="HAV1 GM3"; $resn_exch{"2HAV GM3"}="HAV2 GM3"; $resn_exch{"1HBS GM3"}="HBS1 GM3";
$resn_exch{"2HBS GM3"}="HBS2 GM3"; $resn_exch{"1HBU GM3"}="HBU1 GM3"; $resn_exch{"2HBU GM3"}="HBU2 GM3";
$resn_exch{"1HBV GM3"}="HBV1 GM3"; $resn_exch{"2HBV GM3"}="HBV2 GM3"; $resn_exch{"1HBW GM3"}="HBW1 GM3";
$resn_exch{"2HBW GM3"}="HBW2 GM3"; $resn_exch{"1HBX GM3"}="HBX1 GM3"; $resn_exch{"2HBX GM3"}="HBX2 GM3";
$resn_exch{"1HBY GM3"}="HBY1 GM3"; $resn_exch{"2HBY GM3"}="HBY2 GM3"; $resn_exch{"1HBZ GM3"}="HBZ1 GM3";
$resn_exch{"2HBZ GM3"}="HBZ2 GM3"; $resn_exch{"1HCA GM3"}="HCA1 GM3"; $resn_exch{"2HCA GM3"}="HCA2 GM3";
$resn_exch{"1HCB GM3"}="HCB1 GM3"; $resn_exch{"2HCB GM3"}="HCB2 GM3"; $resn_exch{"1HCC GM3"}="HCC1 GM3";
$resn_exch{"2HCC GM3"}="HCC2 GM3"; $resn_exch{"1HCD GM3"}="HCD1 GM3"; $resn_exch{"2HCD GM3"}="HCD2 GM3";
$resn_exch{"1HCE GM3"}="HCE1 GM3"; $resn_exch{"2HCE GM3"}="HCE2 GM3"; $resn_exch{"1HCF GM3"}="HCF1 GM3";
$resn_exch{"2HCF GM3"}="HCF2 GM3"; $resn_exch{"1HCG GM3"}="HCG1 GM3"; $resn_exch{"2HCG GM3"}="HCG2 GM3";
$resn_exch{"1HCH GM3"}="HCH1 GM3"; $resn_exch{"2HCH GM3"}="HCH2 GM3"; $resn_exch{"1HCI GM3"}="HCI1 GM3";
$resn_exch{"2HCI GM3"}="HCI2 GM3"; $resn_exch{"1HCJ GM3"}="HCJ1 GM3"; $resn_exch{"2HCJ GM3"}="HCJ2 GM3";
$resn_exch{"1HCK GM3"}="HCK1 GM3"; $resn_exch{"2HCK GM3"}="HCK2 GM3"; $resn_exch{"1HCL GM3"}="HCL1 GM3";
$resn_exch{"2HCL GM3"}="HCL2 GM3"; $resn_exch{"1HCM GM3"}="HCM1 GM3"; $resn_exch{"2HCM GM3"}="HCM2 GM3";
$resn_exch{"1HCN GM3"}="HCN1 GM3"; $resn_exch{"2HCN GM3"}="HCN2 GM3"; $resn_exch{"1HCO GM3"}="HCO1 GM3";
$resn_exch{"2HCO GM3"}="HCO2 GM3"; $resn_exch{"1HCP GM3"}="HCP1 GM3"; $resn_exch{"2HCP GM3"}="HCP2 GM3";
$resn_exch{"3HCP GM3"}="HCP3 GM3"; $resn_exch{"1HAC GM6"}="HAC1 GM6"; $resn_exch{"2HAC GM6"}="HAC2 GM6";
$resn_exch{"1HBA GM6"}="HBA1 GM6"; $resn_exch{"2HBA GM6"}="HBA2 GM6"; $resn_exch{"3HBA GM6"}="HBA3 GM6";
$resn_exch{"1HAO GM6"}="HAO1 GM6"; $resn_exch{"2HAO GM6"}="HAO2 GM6"; $resn_exch{"1HAA GM6"}="HAA1 GM6";
$resn_exch{"2HAA GM6"}="HAA2 GM6"; $resn_exch{"1HAJ GM6"}="HAJ1 GM6"; $resn_exch{"2HAJ GM6"}="HAJ2 GM6";
$resn_exch{"3HAJ GM6"}="HAJ3 GM6"; $resn_exch{"1HAI GM6"}="HAI1 GM6"; $resn_exch{"2HAI GM6"}="HAI2 GM6";
$resn_exch{"3HAI GM6"}="HAI3 GM6"; $resn_exch{" HN2 GMA"}=" H2  GMA"; $resn_exch{"1HB  GMA"}=" HB2 GMA";
$resn_exch{"2HB  GMA"}=" HB3 GMA"; $resn_exch{"1HG  GMA"}=" HG2 GMA"; $resn_exch{"2HG  GMA"}=" HG3 GMA";
$resn_exch{" HO2 GMA"}=" HE2 GMA"; $resn_exch{"2HN  GMA"}="HN2A GMA"; $resn_exch{"1HN  GMA"}=" HN1 GMA";
$resn_exch{"1H10 GMC"}="H101 GMC"; $resn_exch{"2H10 GMC"}="H102 GMC"; $resn_exch{"3H10 GMC"}="H103 GMC";
$resn_exch{"1HC  GMC"}=" HC1 GMC"; $resn_exch{"2HC  GMC"}=" HC2 GMC"; $resn_exch{"3HC  GMC"}=" HC3 GMC";
$resn_exch{" H1* GMC"}=" H1' GMC"; $resn_exch{" H4* GMC"}=" H4' GMC"; $resn_exch{"1H5* GMC"}="H5'1 GMC";
$resn_exch{"2H5* GMC"}="H5'2 GMC"; $resn_exch{" H5* GMC"}=" H5' GMC"; $resn_exch{" H2* GMC"}=" H2' GMC";
$resn_exch{" H3* GMC"}=" H3' GMC"; $resn_exch{"1H3* GMC"}="H3'1 GMC"; $resn_exch{"2H3* GMC"}="H3'2 GMC";
$resn_exch{"1H7  GMH"}=" H71 GMH"; $resn_exch{"2H7  GMH"}=" H72 GMH"; $resn_exch{"1HAH GMM"}="HAH1 GMM";
$resn_exch{"2HAH GMM"}="HAH2 GMM"; $resn_exch{"3HAH GMM"}="HAH3 GMM"; $resn_exch{"1HAI GMM"}="HAI1 GMM";
$resn_exch{"2HAI GMM"}="HAI2 GMM"; $resn_exch{"1HAJ GMM"}="HAJ1 GMM"; $resn_exch{"2HAJ GMM"}="HAJ2 GMM";
$resn_exch{"1HAK GMM"}="HAK1 GMM"; $resn_exch{"2HAK GMM"}="HAK2 GMM"; $resn_exch{"1HAL GMM"}="HAL1 GMM";
$resn_exch{"2HAL GMM"}="HAL2 GMM"; $resn_exch{"1HAM GMM"}="HAM1 GMM"; $resn_exch{"2HAM GMM"}="HAM2 GMM";
$resn_exch{"1HAN GMM"}="HAN1 GMM"; $resn_exch{"2HAN GMM"}="HAN2 GMM"; $resn_exch{"1HAO GMM"}="HAO1 GMM";
$resn_exch{"2HAO GMM"}="HAO2 GMM"; $resn_exch{"1HAP GMM"}="HAP1 GMM"; $resn_exch{"2HAP GMM"}="HAP2 GMM";
$resn_exch{"1HAQ GMM"}="HAQ1 GMM"; $resn_exch{"2HAQ GMM"}="HAQ2 GMM"; $resn_exch{"1HAR GMM"}="HAR1 GMM";
$resn_exch{"2HAR GMM"}="HAR2 GMM"; $resn_exch{"1HAS GMM"}="HAS1 GMM"; $resn_exch{"2HAS GMM"}="HAS2 GMM";
$resn_exch{"1HAT GMM"}="HAT1 GMM"; $resn_exch{"2HAT GMM"}="HAT2 GMM"; $resn_exch{"1HAU GMM"}="HAU1 GMM";
$resn_exch{"2HAU GMM"}="HAU2 GMM"; $resn_exch{"1HAV GMM"}="HAV1 GMM"; $resn_exch{"2HAV GMM"}="HAV2 GMM";
$resn_exch{"1HAW GMM"}="HAW1 GMM"; $resn_exch{"2HAW GMM"}="HAW2 GMM"; $resn_exch{"1HAX GMM"}="HAX1 GMM";
$resn_exch{"2HAX GMM"}="HAX2 GMM"; $resn_exch{"1HAY GMM"}="HAY1 GMM"; $resn_exch{"2HAY GMM"}="HAY2 GMM";
$resn_exch{"1HBB GMM"}="HBB1 GMM"; $resn_exch{"2HBB GMM"}="HBB2 GMM"; $resn_exch{"1HBC GMM"}="HBC1 GMM";
$resn_exch{"2HBC GMM"}="HBC2 GMM"; $resn_exch{"1HBD GMM"}="HBD1 GMM"; $resn_exch{"2HBD GMM"}="HBD2 GMM";
$resn_exch{"1HBE GMM"}="HBE1 GMM"; $resn_exch{"2HBE GMM"}="HBE2 GMM"; $resn_exch{"1HBF GMM"}="HBF1 GMM";
$resn_exch{"2HBF GMM"}="HBF2 GMM"; $resn_exch{"1HBG GMM"}="HBG1 GMM"; $resn_exch{"2HBG GMM"}="HBG2 GMM";
$resn_exch{"1HBH GMM"}="HBH1 GMM"; $resn_exch{"2HBH GMM"}="HBH2 GMM"; $resn_exch{"1HBI GMM"}="HBI1 GMM";
$resn_exch{"2HBI GMM"}="HBI2 GMM"; $resn_exch{"1HBL GMM"}="HBL1 GMM"; $resn_exch{"2HBL GMM"}="HBL2 GMM";
$resn_exch{"1HBM GMM"}="HBM1 GMM"; $resn_exch{"2HBM GMM"}="HBM2 GMM"; $resn_exch{"1HBN GMM"}="HBN1 GMM";
$resn_exch{"2HBN GMM"}="HBN2 GMM"; $resn_exch{"1HBO GMM"}="HBO1 GMM"; $resn_exch{"2HBO GMM"}="HBO2 GMM";
$resn_exch{"1HBP GMM"}="HBP1 GMM"; $resn_exch{"2HBP GMM"}="HBP2 GMM"; $resn_exch{"1HBQ GMM"}="HBQ1 GMM";
$resn_exch{"2HBQ GMM"}="HBQ2 GMM"; $resn_exch{"1HBR GMM"}="HBR1 GMM"; $resn_exch{"2HBR GMM"}="HBR2 GMM";
$resn_exch{"1HBS GMM"}="HBS1 GMM"; $resn_exch{"2HBS GMM"}="HBS2 GMM"; $resn_exch{"1HBT GMM"}="HBT1 GMM";
$resn_exch{"2HBT GMM"}="HBT2 GMM"; $resn_exch{"1HBU GMM"}="HBU1 GMM"; $resn_exch{"2HBU GMM"}="HBU2 GMM";
$resn_exch{"1HBV GMM"}="HBV1 GMM"; $resn_exch{"2HBV GMM"}="HBV2 GMM"; $resn_exch{"1HBW GMM"}="HBW1 GMM";
$resn_exch{"2HBW GMM"}="HBW2 GMM"; $resn_exch{"1HBX GMM"}="HBX1 GMM"; $resn_exch{"2HBX GMM"}="HBX2 GMM";
$resn_exch{"1HBY GMM"}="HBY1 GMM"; $resn_exch{"2HBY GMM"}="HBY2 GMM"; $resn_exch{"1HBZ GMM"}="HBZ1 GMM";
$resn_exch{"2HBZ GMM"}="HBZ2 GMM"; $resn_exch{"1HCA GMM"}="HCA1 GMM"; $resn_exch{"2HCA GMM"}="HCA2 GMM";
$resn_exch{"1HCB GMM"}="HCB1 GMM"; $resn_exch{"2HCB GMM"}="HCB2 GMM"; $resn_exch{"1HCC GMM"}="HCC1 GMM";
$resn_exch{"2HCC GMM"}="HCC2 GMM"; $resn_exch{"1HCD GMM"}="HCD1 GMM"; $resn_exch{"2HCD GMM"}="HCD2 GMM";
$resn_exch{"1HCH GMM"}="HCH1 GMM"; $resn_exch{"2HCH GMM"}="HCH2 GMM"; $resn_exch{"1HCI GMM"}="HCI1 GMM";
$resn_exch{"2HCI GMM"}="HCI2 GMM"; $resn_exch{"1HCJ GMM"}="HCJ1 GMM"; $resn_exch{"2HCJ GMM"}="HCJ2 GMM";
$resn_exch{"1HCK GMM"}="HCK1 GMM"; $resn_exch{"2HCK GMM"}="HCK2 GMM"; $resn_exch{"1HCL GMM"}="HCL1 GMM";
$resn_exch{"2HCL GMM"}="HCL2 GMM"; $resn_exch{"1HCM GMM"}="HCM1 GMM"; $resn_exch{"2HCM GMM"}="HCM2 GMM";
$resn_exch{"1HCN GMM"}="HCN1 GMM"; $resn_exch{"2HCN GMM"}="HCN2 GMM"; $resn_exch{"1HCO GMM"}="HCO1 GMM";
$resn_exch{"2HCO GMM"}="HCO2 GMM"; $resn_exch{"3HCO GMM"}="HCO3 GMM"; $resn_exch{"1HCT GMM"}="HCT1 GMM";
$resn_exch{"2HCT GMM"}="HCT2 GMM"; $resn_exch{"1H1  GMN"}=" H11 GMN"; $resn_exch{"2H1  GMN"}=" H12 GMN";
$resn_exch{"1H2  GMN"}=" H21 GMN"; $resn_exch{"2H2  GMN"}=" H22 GMN"; $resn_exch{"1H4  GMN"}=" H41 GMN";
$resn_exch{"2H4  GMN"}=" H42 GMN"; $resn_exch{"1H5  GMN"}=" H51 GMN"; $resn_exch{"2H5  GMN"}=" H52 GMN";
$resn_exch{"3H5  GMN"}=" H53 GMN"; $resn_exch{"1H6  GMN"}=" H61 GMN"; $resn_exch{"2H6  GMN"}=" H62 GMN";
$resn_exch{"1H7  GMN"}=" H71 GMN"; $resn_exch{"2H7  GMN"}=" H72 GMN"; $resn_exch{"3H7  GMN"}=" H73 GMN";
$resn_exch{"1H8  GMN"}=" H81 GMN"; $resn_exch{"2H8  GMN"}=" H82 GMN"; $resn_exch{"1H9  GMN"}=" H91 GMN";
$resn_exch{"2H9  GMN"}=" H92 GMN"; $resn_exch{"3H9  GMN"}=" H93 GMN"; $resn_exch{"1H10 GMN"}="H101 GMN";
$resn_exch{"2H10 GMN"}="H102 GMN"; $resn_exch{"1H11 GMN"}="H111 GMN"; $resn_exch{"2H11 GMN"}="H112 GMN";
$resn_exch{"1H13 GMN"}="H131 GMN"; $resn_exch{"2H13 GMN"}="H132 GMN"; $resn_exch{"1H14 GMN"}="H141 GMN";
$resn_exch{"2H14 GMN"}="H142 GMN"; $resn_exch{"3H14 GMN"}="H143 GMN"; $resn_exch{"1H15 GMN"}="H151 GMN";
$resn_exch{"2H15 GMN"}="H152 GMN"; $resn_exch{"1H16 GMN"}="H161 GMN"; $resn_exch{"2H16 GMN"}="H162 GMN";
$resn_exch{"3H16 GMN"}="H163 GMN"; $resn_exch{"1H17 GMN"}="H171 GMN"; $resn_exch{"2H17 GMN"}="H172 GMN";
$resn_exch{"1H18 GMN"}="H181 GMN"; $resn_exch{"2H18 GMN"}="H182 GMN"; $resn_exch{"3H18 GMN"}="H183 GMN";
$resn_exch{"1H25 GMN"}="H251 GMN"; $resn_exch{"2H25 GMN"}="H252 GMN"; $resn_exch{"1H26 GMN"}="H261 GMN";
$resn_exch{"2H26 GMN"}="H262 GMN"; $resn_exch{"1H28 GMN"}="H281 GMN"; $resn_exch{"2H28 GMN"}="H282 GMN";
$resn_exch{"1H29 GMN"}="H291 GMN"; $resn_exch{"2H29 GMN"}="H292 GMN"; $resn_exch{"3H29 GMN"}="H293 GMN";
$resn_exch{"1H30 GMN"}="H301 GMN"; $resn_exch{"2H30 GMN"}="H302 GMN"; $resn_exch{"1H31 GMN"}="H311 GMN";
$resn_exch{"2H31 GMN"}="H312 GMN"; $resn_exch{"3H31 GMN"}="H313 GMN"; $resn_exch{"1H32 GMN"}="H321 GMN";
$resn_exch{"2H32 GMN"}="H322 GMN"; $resn_exch{"1H33 GMN"}="H331 GMN"; $resn_exch{"2H33 GMN"}="H332 GMN";
$resn_exch{"3H33 GMN"}="H333 GMN"; $resn_exch{"*HO5 GMP"}="HO5' GMP"; $resn_exch{"1H5* GMP"}="H5'1 GMP";
$resn_exch{"2H5* GMP"}="H5'2 GMP"; $resn_exch{" H4* GMP"}=" H4' GMP"; $resn_exch{" H3* GMP"}=" H3' GMP";
$resn_exch{"*HO3 GMP"}="HO3' GMP"; $resn_exch{" H2* GMP"}=" H2' GMP"; $resn_exch{"*HO2 GMP"}="HO2' GMP";
$resn_exch{" H1* GMP"}=" H1' GMP"; $resn_exch{"1HN2 GMP"}="HN21 GMP"; $resn_exch{"2HN2 GMP"}="HN22 GMP";
$resn_exch{"1H2  GMS"}=" H21 GMS"; $resn_exch{"2H2  GMS"}=" H22 GMS"; $resn_exch{"1H2* GMS"}=" H2' GMS";
$resn_exch{"2H2* GMS"}="H2'' GMS"; $resn_exch{"1H5* GMS"}=" H5' GMS"; $resn_exch{"2H5* GMS"}="H5'' GMS";
$resn_exch{" H4* GMS"}=" H4' GMS"; $resn_exch{" H1* GMS"}=" H1' GMS"; $resn_exch{" H3* GMS"}=" H3' GMS";
$resn_exch{"H3T  GMS"}="HO3' GMS"; $resn_exch{"2HOP GMS"}="HOP2 GMS"; $resn_exch{"1H5* GMU"}=" H5' GMU";
$resn_exch{"2H5* GMU"}="H5'' GMU"; $resn_exch{" H4* GMU"}=" H4' GMU"; $resn_exch{" H3* GMU"}=" H3' GMU";
$resn_exch{" H2* GMU"}=" H2' GMU"; $resn_exch{" H1* GMU"}=" H1' GMU"; $resn_exch{"1H5M GMU"}=" H71 GMU";
$resn_exch{"2H5M GMU"}=" H72 GMU"; $resn_exch{"3H5M GMU"}=" H73 GMU"; $resn_exch{"1HA* GMU"}="HA'1 GMU";
$resn_exch{"2HA* GMU"}="HA'2 GMU"; $resn_exch{"1HB* GMU"}="HB'1 GMU"; $resn_exch{"2HB* GMU"}="HB'2 GMU";
$resn_exch{" HC* GMU"}=" HC' GMU"; $resn_exch{" HD* GMU"}=" HD' GMU"; $resn_exch{"1HE* GMU"}="HE'1 GMU";
$resn_exch{"2HE* GMU"}="HE'2 GMU"; $resn_exch{"1HF* GMU"}="HF'1 GMU"; $resn_exch{"2HF* GMU"}="HF'2 GMU";
$resn_exch{"1H7  GMY"}=" H71 GMY"; $resn_exch{"2H7  GMY"}=" H72 GMY"; $resn_exch{"1H9  GMY"}=" H91 GMY";
$resn_exch{"2H9  GMY"}=" H92 GMY"; $resn_exch{"1H26 GMY"}="H261 GMY"; $resn_exch{"2H26 GMY"}="H262 GMY";
$resn_exch{"3H26 GMY"}="H263 GMY"; $resn_exch{"1H27 GMY"}="H271 GMY"; $resn_exch{"2H27 GMY"}="H272 GMY";
$resn_exch{"3H27 GMY"}="H273 GMY"; $resn_exch{"1H29 GMY"}="H291 GMY"; $resn_exch{"2H29 GMY"}="H292 GMY";
$resn_exch{"3H29 GMY"}="H293 GMY"; $resn_exch{"1H31 GMY"}="H311 GMY"; $resn_exch{"2H31 GMY"}="H312 GMY";
$resn_exch{"3H31 GMY"}="H313 GMY"; $resn_exch{"1H32 GMY"}="H321 GMY"; $resn_exch{"2H32 GMY"}="H322 GMY";
$resn_exch{"3H32 GMY"}="H323 GMY"; $resn_exch{"1HN6 GMY"}="HN61 GMY"; $resn_exch{"2HN6 GMY"}="HN62 GMY";
$resn_exch{"1H38 GMY"}="H381 GMY"; $resn_exch{"2H38 GMY"}="H382 GMY"; $resn_exch{"3H38 GMY"}="H383 GMY";
$resn_exch{"1H39 GMY"}="H391 GMY"; $resn_exch{"2H39 GMY"}="H392 GMY"; $resn_exch{"3H39 GMY"}="H393 GMY";
$resn_exch{"1H6' GN1"}="H6'1 GN1"; $resn_exch{"2H6' GN1"}="H6'2 GN1"; $resn_exch{"1H8' GN1"}="H8'1 GN1";
$resn_exch{"2H8' GN1"}="H8'2 GN1"; $resn_exch{"3H8' GN1"}="H8'3 GN1"; $resn_exch{"2HOP GN7"}="HOP2 GN7";
$resn_exch{"3HOP GN7"}="HOP3 GN7"; $resn_exch{"1H5* GN7"}=" H5' GN7"; $resn_exch{"2H5* GN7"}="H5'' GN7";
$resn_exch{" H4* GN7"}=" H4' GN7"; $resn_exch{" H3* GN7"}=" H3' GN7"; $resn_exch{"*HO3 GN7"}="HO3' GN7";
$resn_exch{"1H2* GN7"}=" H2' GN7"; $resn_exch{"2H2* GN7"}="H2'' GN7"; $resn_exch{" H1* GN7"}=" H1' GN7";
$resn_exch{"1HN2 GN7"}="HN21 GN7"; $resn_exch{"2HN2 GN7"}="HN22 GN7"; $resn_exch{"1HAO GN8"}="HAO1 GN8";
$resn_exch{"2HAO GN8"}="HAO2 GN8"; $resn_exch{"1H3  GNA"}=" H31 GNA"; $resn_exch{"2H3  GNA"}=" H32 GNA";
$resn_exch{"1H9  GNA"}=" H91 GNA"; $resn_exch{"2H9  GNA"}=" H92 GNA"; $resn_exch{"1H11 GNA"}="H111 GNA";
$resn_exch{"2H11 GNA"}="H112 GNA"; $resn_exch{"3H11 GNA"}="H113 GNA"; $resn_exch{"1HN7 GNA"}="HN71 GNA";
$resn_exch{"2HN7 GNA"}="HN72 GNA"; $resn_exch{"1HOB GNA"}="HOB1 GNA"; $resn_exch{"1HOT GNB"}="HOT1 GNB";
$resn_exch{"1NH1 GNB"}="H11N GNB"; $resn_exch{"2NH1 GNB"}="H12N GNB"; $resn_exch{"1HA  GNB"}=" HA1 GNB";
$resn_exch{"1HB  GNB"}=" HB1 GNB"; $resn_exch{"2HB  GNB"}=" HB2 GNB"; $resn_exch{"1HB2 GNB"}="HB21 GNB";
$resn_exch{"2HB2 GNB"}="HB22 GNB"; $resn_exch{"1HG  GNB"}=" HG1 GNB"; $resn_exch{"2HG  GNB"}=" HG2 GNB";
$resn_exch{"1HN2 GNB"}="HN21 GNB"; $resn_exch{"1HA3 GNB"}="HA31 GNB"; $resn_exch{"2HA3 GNB"}="HA32 GNB";
$resn_exch{"1HO3 GNB"}="HO31 GNB"; $resn_exch{"1HC  GNB"}=" HC1 GNB"; $resn_exch{"2HC  GNB"}=" HC2 GNB";
$resn_exch{"1HB  GND"}=" HB1 GND"; $resn_exch{"2HB  GND"}=" HB2 GND"; $resn_exch{"1HG  GND"}=" HG1 GND";
$resn_exch{"2HG  GND"}=" HG2 GND"; $resn_exch{"1HD  GND"}=" HD1 GND"; $resn_exch{"2HD  GND"}=" HD2 GND";
$resn_exch{"1HH1 GND"}="HH11 GND"; $resn_exch{"1HH2 GND"}="HH21 GND"; $resn_exch{"2HH2 GND"}="HH22 GND";
$resn_exch{"1HN  GND"}=" HN1 GND"; $resn_exch{"2HN  GND"}=" HN2 GND"; $resn_exch{"3HN  GND"}=" HN3 GND";
$resn_exch{"1H5* GNE"}=" H5' GNE"; $resn_exch{"2H5* GNE"}="H5'' GNE"; $resn_exch{" H4* GNE"}=" H4' GNE";
$resn_exch{" H3* GNE"}=" H3' GNE"; $resn_exch{"1H2* GNE"}=" H2' GNE"; $resn_exch{"2H2* GNE"}="H2'' GNE";
$resn_exch{" H1* GNE"}=" H1' GNE"; $resn_exch{"1H13 GNF"}="H131 GNF"; $resn_exch{"2H13 GNF"}="H132 GNF";
$resn_exch{"1H19 GNF"}="H191 GNF"; $resn_exch{"2H19 GNF"}="H192 GNF"; $resn_exch{"1H18 GNF"}="H181 GNF";
$resn_exch{"2H18 GNF"}="H182 GNF"; $resn_exch{"1H17 GNF"}="H171 GNF"; $resn_exch{"2H17 GNF"}="H172 GNF";
$resn_exch{"1H16 GNF"}="H161 GNF"; $resn_exch{"2H16 GNF"}="H162 GNF"; $resn_exch{"1H15 GNF"}="H151 GNF";
$resn_exch{"2H15 GNF"}="H152 GNF"; $resn_exch{"1H8  GNF"}=" H81 GNF"; $resn_exch{"2H8  GNF"}=" H82 GNF";
$resn_exch{"1H7  GNF"}=" H71 GNF"; $resn_exch{"2H7  GNF"}=" H72 GNF"; $resn_exch{"1H   GNF"}=" H1  GNF";
$resn_exch{"2H   GNF"}=" H2  GNF"; $resn_exch{"3H   GNF"}=" H3A GNF"; $resn_exch{" H5* GNG"}=" H5' GNG";
$resn_exch{"1H5* GNG"}="H5'1 GNG"; $resn_exch{"2H5* GNG"}="H5'2 GNG"; $resn_exch{" H4* GNG"}=" H4' GNG";
$resn_exch{" H1* GNG"}=" H1' GNG"; $resn_exch{"1HN2 GNG"}="HN21 GNG"; $resn_exch{"2HN2 GNG"}="HN22 GNG";
$resn_exch{"1H2* GNG"}="H2'1 GNG"; $resn_exch{"2H2* GNG"}="H2'2 GNG"; $resn_exch{" H3* GNG"}=" H3' GNG";
$resn_exch{"1HN3 GNH"}="HN31 GNH"; $resn_exch{"2HN3 GNH"}="HN32 GNH"; $resn_exch{"2HOB GNH"}="HOB2 GNH";
$resn_exch{"2HOA GNH"}="HOA2 GNH"; $resn_exch{"2H5* GNH"}="H5'2 GNH"; $resn_exch{"1H5* GNH"}="H5'1 GNH";
$resn_exch{" H4* GNH"}=" H4' GNH"; $resn_exch{" H3* GNH"}=" H3' GNH"; $resn_exch{"*HO3 GNH"}="HO3' GNH";
$resn_exch{" H2* GNH"}=" H2' GNH"; $resn_exch{"*HO2 GNH"}="HO2' GNH"; $resn_exch{" H1* GNH"}=" H1' GNH";
$resn_exch{"1HN2 GNH"}="HN21 GNH"; $resn_exch{"2HN2 GNH"}="HN22 GNH"; $resn_exch{"1H8C GNI"}="H8C1 GNI";
$resn_exch{"2H8C GNI"}="H8C2 GNI"; $resn_exch{"1H7C GNI"}="H7C1 GNI"; $resn_exch{"2H7C GNI"}="H7C2 GNI";
$resn_exch{"1H2C GNI"}="H2C1 GNI"; $resn_exch{"2H2C GNI"}="H2C2 GNI"; $resn_exch{"2HOG GNP"}="HOG2 GNP";
$resn_exch{"3HOG GNP"}="HOG3 GNP"; $resn_exch{"3HNB GNP"}="HNB3 GNP"; $resn_exch{"2HOB GNP"}="HOB2 GNP";
$resn_exch{"2HOA GNP"}="HOA2 GNP"; $resn_exch{"2H5* GNP"}="H5'2 GNP"; $resn_exch{"1H5* GNP"}="H5'1 GNP";
$resn_exch{" H4* GNP"}=" H4' GNP"; $resn_exch{" H3* GNP"}=" H3' GNP"; $resn_exch{"*HO3 GNP"}="HO3' GNP";
$resn_exch{" H2* GNP"}=" H2' GNP"; $resn_exch{"*HO2 GNP"}="HO2' GNP"; $resn_exch{" H1* GNP"}=" H1' GNP";
$resn_exch{"1HN2 GNP"}="HN21 GNP"; $resn_exch{"2HN2 GNP"}="HN22 GNP"; $resn_exch{"1H17 GNQ"}="H171 GNQ";
$resn_exch{"2H17 GNQ"}="H172 GNQ"; $resn_exch{"1H16 GNQ"}="H161 GNQ"; $resn_exch{"2H16 GNQ"}="H162 GNQ";
$resn_exch{"1H15 GNQ"}="H151 GNQ"; $resn_exch{"2H15 GNQ"}="H152 GNQ"; $resn_exch{"1H14 GNQ"}="H141 GNQ";
$resn_exch{"2H14 GNQ"}="H142 GNQ"; $resn_exch{"1H13 GNQ"}="H131 GNQ"; $resn_exch{"2H13 GNQ"}="H132 GNQ";
$resn_exch{"1H12 GNQ"}="H121 GNQ"; $resn_exch{"2H12 GNQ"}="H122 GNQ"; $resn_exch{"1H10 GNQ"}="H101 GNQ";
$resn_exch{"2H10 GNQ"}="H102 GNQ"; $resn_exch{"3H10 GNQ"}="H103 GNQ"; $resn_exch{"1H8  GNQ"}=" H81 GNQ";
$resn_exch{"2H8  GNQ"}=" H82 GNQ"; $resn_exch{"1HAK GNR"}="HAK1 GNR"; $resn_exch{"2HAK GNR"}="HAK2 GNR";
$resn_exch{"1H6  GNS"}=" H61 GNS"; $resn_exch{"2H6  GNS"}=" H62 GNS"; $resn_exch{"1H4  GNT"}="H41A GNT";
$resn_exch{"2H4  GNT"}=" H42 GNT"; $resn_exch{"1H9  GNT"}=" H91 GNT"; $resn_exch{"2H9  GNT"}=" H92 GNT";
$resn_exch{"1H11 GNT"}="H111 GNT"; $resn_exch{"2H11 GNT"}="H112 GNT"; $resn_exch{"1H12 GNT"}="H121 GNT";
$resn_exch{"2H12 GNT"}="H122 GNT"; $resn_exch{"1H16 GNT"}="H161 GNT"; $resn_exch{"2H16 GNT"}="H162 GNT";
$resn_exch{"3H16 GNT"}="H163 GNT"; $resn_exch{"1H19 GNT"}="H191 GNT"; $resn_exch{"2H19 GNT"}="H192 GNT";
$resn_exch{"3H19 GNT"}="H193 GNT"; $resn_exch{"2H2  GOA"}=" H22 GOA"; $resn_exch{"1H2  GOA"}=" H21 GOA";
$resn_exch{"1H1  GOL"}=" H11 GOL"; $resn_exch{"2H1  GOL"}=" H12 GOL"; $resn_exch{"1H3  GOL"}=" H31 GOL";
$resn_exch{"2H3  GOL"}=" H32 GOL"; $resn_exch{"1HC  GOM"}=" HC1 GOM"; $resn_exch{"2HC  GOM"}=" HC2 GOM";
$resn_exch{"1HB  GOM"}=" HB1 GOM"; $resn_exch{"2HB  GOM"}=" HB2 GOM"; $resn_exch{"1HG  GOM"}=" HG1 GOM";
$resn_exch{"2HG  GOM"}=" HG2 GOM"; $resn_exch{"1H5* GOM"}=" H5' GOM"; $resn_exch{"2H5* GOM"}="H5'' GOM";
$resn_exch{" H4* GOM"}=" H4' GOM"; $resn_exch{" H1* GOM"}=" H1' GOM"; $resn_exch{"1HN6 GOM"}="HN61 GOM";
$resn_exch{"2HN6 GOM"}="HN62 GOM"; $resn_exch{" H2* GOM"}=" H2' GOM"; $resn_exch{"*HO2 GOM"}="HO2' GOM";
$resn_exch{" H3* GOM"}=" H3' GOM"; $resn_exch{"*HO3 GOM"}="HO3' GOM"; $resn_exch{"1HC6 GOX"}="HC61 GOX";
$resn_exch{"2HC6 GOX"}="HC62 GOX"; $resn_exch{"1H6  GP1"}=" H61 GP1"; $resn_exch{"2H6  GP1"}=" H62 GP1";
$resn_exch{"1HN2 GP1"}="HN21 GP1"; $resn_exch{"2HN2 GP1"}="HN22 GP1"; $resn_exch{"1HOP GP1"}="HOP1 GP1";
$resn_exch{"2HOP GP1"}="HOP2 GP1"; $resn_exch{"1HN2 GP2"}="HN21 GP2"; $resn_exch{"2HN2 GP2"}="HN22 GP2";
$resn_exch{" H1* GP2"}=" H1' GP2"; $resn_exch{" H2* GP2"}=" H2' GP2"; $resn_exch{" H3* GP2"}=" H3' GP2";
$resn_exch{" H4* GP2"}=" H4' GP2"; $resn_exch{"1H5* GP2"}="H5'1 GP2"; $resn_exch{"2H5* GP2"}="H5'2 GP2";
$resn_exch{"1H3A GP2"}="H3A1 GP2"; $resn_exch{"2H3A GP2"}="H3A2 GP2"; $resn_exch{" AH8 GP3"}=" H8A GP3";
$resn_exch{" AH1 GP3"}=" H1A GP3"; $resn_exch{"AH21 GP3"}="H21A GP3"; $resn_exch{"AH22 GP3"}="H22A GP3";
$resn_exch{"AH51 GP3"}="H51A GP3"; $resn_exch{"AH52 GP3"}="H52A GP3"; $resn_exch{"AH4* GP3"}=" H4D GP3";
$resn_exch{"AH3* GP3"}=" H3D GP3"; $resn_exch{"AHO3 GP3"}="HO3A GP3"; $resn_exch{"AH2* GP3"}=" H2D GP3";
$resn_exch{"AHO2 GP3"}="HO2A GP3"; $resn_exch{"AH1* GP3"}=" H1D GP3"; $resn_exch{"2HOA GP3"}="HOA2 GP3";
$resn_exch{"2HOB GP3"}="HOB2 GP3"; $resn_exch{"2HOG GP3"}="HOG2 GP3"; $resn_exch{"BH51 GP3"}="H51B GP3";
$resn_exch{"BH52 GP3"}="H52B GP3"; $resn_exch{"BH4* GP3"}=" H4E GP3"; $resn_exch{"BH3* GP3"}=" H3E GP3";
$resn_exch{"BHO3 GP3"}="HO3B GP3"; $resn_exch{"BH2* GP3"}=" H2E GP3"; $resn_exch{"BHO2 GP3"}="HO2B GP3";
$resn_exch{"BH1* GP3"}=" H1E GP3"; $resn_exch{" BH8 GP3"}=" H8B GP3"; $resn_exch{" BH1 GP3"}=" H1B GP3";
$resn_exch{"BH21 GP3"}="H21B GP3"; $resn_exch{"BH22 GP3"}="H22B GP3"; $resn_exch{"1H6  GP4"}=" H61 GP4";
$resn_exch{"2H6  GP4"}=" H62 GP4"; $resn_exch{"1HN2 GP4"}="HN21 GP4"; $resn_exch{"2HN2 GP4"}="HN22 GP4";
$resn_exch{"1HOP GP4"}="HOP1 GP4"; $resn_exch{"2HOP GP4"}="HOP2 GP4"; $resn_exch{"1HN2 GP5"}="HN21 GP5";
$resn_exch{"2HN2 GP5"}="HN22 GP5"; $resn_exch{" H1* GP5"}=" H1' GP5"; $resn_exch{" H2* GP5"}=" H2' GP5";
$resn_exch{" H3* GP5"}=" H3' GP5"; $resn_exch{" H4* GP5"}=" H4' GP5"; $resn_exch{"1H5* GP5"}="H5'1 GP5";
$resn_exch{"2H5* GP5"}="H5'2 GP5"; $resn_exch{"1HAV GP5"}="HAV1 GP5"; $resn_exch{"2HAV GP5"}="HAV2 GP5";
$resn_exch{"1HAB GP5"}="HAB1 GP5"; $resn_exch{"2HAB GP5"}="HAB2 GP5"; $resn_exch{"1HN2 GP6"}="HN21 GP6";
$resn_exch{"2HN2 GP6"}="HN22 GP6"; $resn_exch{"1HN1 GP8"}="HN11 GP8"; $resn_exch{"2HN1 GP8"}="HN12 GP8";
$resn_exch{"2HA2 GPB"}="HA22 GPB"; $resn_exch{"1HA2 GPB"}="HA21 GPB"; $resn_exch{"3H5  GPB"}=" H53 GPB";
$resn_exch{"2H5  GPB"}=" H52 GPB"; $resn_exch{"1H5  GPB"}=" H51 GPB"; $resn_exch{"1H9  GPB"}=" H91 GPB";
$resn_exch{"2H9  GPB"}=" H92 GPB"; $resn_exch{"1H10 GPB"}="H101 GPB"; $resn_exch{"2H10 GPB"}="H102 GPB";
$resn_exch{"1HA  GPB"}=" HA1 GPB"; $resn_exch{"2HB  GPB"}=" HB2 GPB"; $resn_exch{"1HB  GPB"}=" HB1 GPB";
$resn_exch{"2HG  GPB"}=" HG2 GPB"; $resn_exch{"1HG  GPB"}=" HG1 GPB"; $resn_exch{"OHE2 GPB"}="HE2O GPB";
$resn_exch{" OH1 GPB"}=" H1O GPB"; $resn_exch{" GH1 GPC"}=" H1G GPC"; $resn_exch{"GH21 GPC"}="H21G GPC";
$resn_exch{"GH22 GPC"}="H22G GPC"; $resn_exch{" GH8 GPC"}=" H8G GPC"; $resn_exch{"GH1* GPC"}=" H1D GPC";
$resn_exch{"GH2* GPC"}=" H2D GPC"; $resn_exch{"GH3* GPC"}=" H3D GPC"; $resn_exch{"GH4* GPC"}=" H4D GPC";
$resn_exch{"GH51 GPC"}="H51G GPC"; $resn_exch{"GH52 GPC"}="H52G GPC"; $resn_exch{"GHO5 GPC"}="HO5G GPC";
$resn_exch{"2HOC GPC"}="HOC2 GPC"; $resn_exch{"CH51 GPC"}="H51C GPC"; $resn_exch{"CH52 GPC"}="H52C GPC";
$resn_exch{"CH4* GPC"}=" H4B GPC"; $resn_exch{"CH3* GPC"}=" H3B GPC"; $resn_exch{"CHO3 GPC"}="HO3C GPC";
$resn_exch{"CH2* GPC"}=" H2B GPC"; $resn_exch{"CHO2 GPC"}="HO2C GPC"; $resn_exch{"CH1* GPC"}=" H1B GPC";
$resn_exch{"CH41 GPC"}="H41C GPC"; $resn_exch{"CH42 GPC"}="H42C GPC"; $resn_exch{" CH5 GPC"}=" H5C GPC";
$resn_exch{" CH6 GPC"}=" H6C GPC"; $resn_exch{"1H1  GPE"}=" H11 GPE"; $resn_exch{"2H1  GPE"}=" H12 GPE";
$resn_exch{"1H3  GPE"}=" H31 GPE"; $resn_exch{"2H3  GPE"}=" H32 GPE"; $resn_exch{"1H11 GPE"}="H111 GPE";
$resn_exch{"2H11 GPE"}="H112 GPE"; $resn_exch{"1H12 GPE"}="H121 GPE"; $resn_exch{"2H12 GPE"}="H122 GPE";
$resn_exch{"1HN  GPE"}=" HN1 GPE"; $resn_exch{"2HN  GPE"}=" HN2 GPE"; $resn_exch{" AH8 GPG"}=" H8A GPG";
$resn_exch{" AH1 GPG"}=" H1A GPG"; $resn_exch{"AH21 GPG"}="H21A GPG"; $resn_exch{"AH22 GPG"}="H22A GPG";
$resn_exch{"AHO5 GPG"}="HO5A GPG"; $resn_exch{"AH51 GPG"}="H51A GPG"; $resn_exch{"AH52 GPG"}="H52A GPG";
$resn_exch{"AH4* GPG"}=" H4D GPG"; $resn_exch{"AH3* GPG"}=" H3D GPG"; $resn_exch{"AHO3 GPG"}="HO3A GPG";
$resn_exch{"AH2* GPG"}=" H2D GPG"; $resn_exch{"AH1* GPG"}=" H1D GPG"; $resn_exch{"2HOP GPG"}="HOP2 GPG";
$resn_exch{"BH51 GPG"}="H51B GPG"; $resn_exch{"BH52 GPG"}="H52B GPG"; $resn_exch{"BH4* GPG"}=" H4E GPG";
$resn_exch{"BH3* GPG"}=" H3E GPG"; $resn_exch{"BHO3 GPG"}="HO3B GPG"; $resn_exch{"BH2* GPG"}=" H2E GPG";
$resn_exch{"BHO2 GPG"}="HO2B GPG"; $resn_exch{"BH1* GPG"}=" H1E GPG"; $resn_exch{" BH8 GPG"}=" H8B GPG";
$resn_exch{" BH1 GPG"}=" H1B GPG"; $resn_exch{"BH21 GPG"}="H21B GPG"; $resn_exch{"BH22 GPG"}="H22B GPG";
$resn_exch{"1H5  GPI"}=" H51 GPI"; $resn_exch{"2H5  GPI"}=" H52 GPI"; $resn_exch{"1H3  GPI"}=" H31 GPI";
$resn_exch{"2H3  GPI"}=" H32 GPI"; $resn_exch{"1H4  GPI"}=" H41 GPI"; $resn_exch{"2H4  GPI"}=" H42 GPI";
$resn_exch{"1H14 GPI"}="H141 GPI"; $resn_exch{"2H14 GPI"}="H142 GPI"; $resn_exch{"1H15 GPI"}="H151 GPI";
$resn_exch{"2H15 GPI"}="H152 GPI"; $resn_exch{"1H16 GPI"}="H161 GPI"; $resn_exch{"2H16 GPI"}="H162 GPI";
$resn_exch{"1H12 GPI"}="H121 GPI"; $resn_exch{"2H12 GPI"}="H122 GPI"; $resn_exch{"3H12 GPI"}="H123 GPI";
$resn_exch{"1H13 GPI"}="H131 GPI"; $resn_exch{"2H13 GPI"}="H132 GPI"; $resn_exch{"3H13 GPI"}="H133 GPI";
$resn_exch{"1H10 GPI"}="H101 GPI"; $resn_exch{"2H10 GPI"}="H102 GPI"; $resn_exch{"1H11 GPI"}="H111 GPI";
$resn_exch{"2H11 GPI"}="H112 GPI"; $resn_exch{"3H11 GPI"}="H113 GPI"; $resn_exch{"1H1  GPJ"}=" H11 GPJ";
$resn_exch{"2H1  GPJ"}=" H12 GPJ"; $resn_exch{"1H2  GPJ"}=" H21 GPJ"; $resn_exch{"2H2  GPJ"}=" H22 GPJ";
$resn_exch{"1HN1 GPJ"}="HN11 GPJ"; $resn_exch{"2HOP GPL"}="HOP2 GPL"; $resn_exch{"1H5* GPL"}="H5'1 GPL";
$resn_exch{"2H5* GPL"}="H5'2 GPL"; $resn_exch{" H4* GPL"}=" H4' GPL"; $resn_exch{" H3* GPL"}=" H3' GPL";
$resn_exch{"*HO3 GPL"}="HO3' GPL"; $resn_exch{" H2* GPL"}=" H2' GPL"; $resn_exch{"*HO2 GPL"}="HO2' GPL";
$resn_exch{" H1* GPL"}=" H1' GPL"; $resn_exch{"1HN2 GPL"}="HN21 GPL"; $resn_exch{"2HN2 GPL"}="HN22 GPL";
$resn_exch{"1HN  GPL"}=" H   GPL"; $resn_exch{"2HN  GPL"}=" H2  GPL"; $resn_exch{"1HB  GPL"}=" HB2 GPL";
$resn_exch{"2HB  GPL"}=" HB3 GPL"; $resn_exch{"1HG  GPL"}=" HG2 GPL"; $resn_exch{"2HG  GPL"}=" HG3 GPL";
$resn_exch{"1HD  GPL"}=" HD2 GPL"; $resn_exch{"2HD  GPL"}=" HD3 GPL"; $resn_exch{"1HE  GPL"}=" HE2 GPL";
$resn_exch{"2HE  GPL"}=" HE3 GPL"; $resn_exch{" HNZ GPL"}=" HZ  GPL"; $resn_exch{"1H6  GPM"}=" H61 GPM";
$resn_exch{"2H6  GPM"}=" H62 GPM"; $resn_exch{"1H7  GPM"}=" H71 GPM"; $resn_exch{"2H7  GPM"}=" H72 GPM";
$resn_exch{"2HOP GPM"}="HOP2 GPM"; $resn_exch{"3HOP GPM"}="HOP3 GPM"; $resn_exch{"1H8' GPN"}="H8'1 GPN";
$resn_exch{"2H8' GPN"}="H8'2 GPN"; $resn_exch{"1H5' GPN"}="H5'1 GPN"; $resn_exch{"2H5' GPN"}="H5'2 GPN";
$resn_exch{"1H3' GPN"}="H3'1 GPN"; $resn_exch{"2H3' GPN"}="H3'2 GPN"; $resn_exch{"1H2' GPN"}="H2'1 GPN";
$resn_exch{"2H2' GPN"}="H2'2 GPN"; $resn_exch{"1H1' GPN"}="H1'1 GPN"; $resn_exch{"2H1' GPN"}="H1'2 GPN";
$resn_exch{"3H1' GPN"}="H1'3 GPN"; $resn_exch{"1HN2 GPN"}="HN21 GPN"; $resn_exch{"2HN2 GPN"}="HN22 GPN";
$resn_exch{"1H1  GPP"}=" H11 GPP"; $resn_exch{"2H1  GPP"}=" H12 GPP"; $resn_exch{"1H4  GPP"}=" H41 GPP";
$resn_exch{"2H4  GPP"}=" H42 GPP"; $resn_exch{"3H4  GPP"}=" H43 GPP"; $resn_exch{"1H5  GPP"}=" H51 GPP";
$resn_exch{"2H5  GPP"}=" H52 GPP"; $resn_exch{"1H6  GPP"}=" H61 GPP"; $resn_exch{"2H6  GPP"}=" H62 GPP";
$resn_exch{"1H9  GPP"}=" H91 GPP"; $resn_exch{"2H9  GPP"}=" H92 GPP"; $resn_exch{"3H9  GPP"}=" H93 GPP";
$resn_exch{"1H10 GPP"}="H101 GPP"; $resn_exch{"2H10 GPP"}="H102 GPP"; $resn_exch{"3H10 GPP"}="H103 GPP";
$resn_exch{"2HOA GPP"}="HOA2 GPP"; $resn_exch{"2HOB GPP"}="HOB2 GPP"; $resn_exch{"3HOB GPP"}="HOB3 GPP";
$resn_exch{"1HN1 GPR"}="HN11 GPR"; $resn_exch{"2HN1 GPR"}="HN12 GPR"; $resn_exch{"1HB1 GPR"}="HB11 GPR";
$resn_exch{"2HB1 GPR"}="HB12 GPR"; $resn_exch{"1HG1 GPR"}="HG11 GPR"; $resn_exch{"2HG1 GPR"}="HG12 GPR";
$resn_exch{"1HB2 GPR"}="HB21 GPR"; $resn_exch{"2HB2 GPR"}="HB22 GPR"; $resn_exch{"1HA3 GPR"}="HA31 GPR";
$resn_exch{"2HA3 GPR"}="HA32 GPR"; $resn_exch{"1HN1 GPS"}="HN11 GPS"; $resn_exch{"2HN1 GPS"}="HN12 GPS";
$resn_exch{"1HB1 GPS"}="HB11 GPS"; $resn_exch{"2HB1 GPS"}="HB12 GPS"; $resn_exch{"1HG1 GPS"}="HG11 GPS";
$resn_exch{"2HG1 GPS"}="HG12 GPS"; $resn_exch{"1HB2 GPS"}="HB21 GPS"; $resn_exch{"2HB2 GPS"}="HB22 GPS";
$resn_exch{"1HA3 GPS"}="HA31 GPS"; $resn_exch{"2HA3 GPS"}="HA32 GPS"; $resn_exch{"1HN2 GPX"}="HN21 GPX";
$resn_exch{"2HN2 GPX"}="HN22 GPX"; $resn_exch{"2HOA GPX"}="HOA2 GPX"; $resn_exch{"2HOB GPX"}="HOB2 GPX";
$resn_exch{"3HOB GPX"}="HOB3 GPX"; $resn_exch{"1H5' GPX"}="H5'1 GPX"; $resn_exch{"2H5' GPX"}="H5'2 GPX";
$resn_exch{"2H7  GR1"}=" H72 GR1"; $resn_exch{"1H7  GR1"}=" H71 GR1"; $resn_exch{"2H17 GR1"}="H172 GR1";
$resn_exch{"1H17 GR1"}="H171 GR1"; $resn_exch{"2H18 GR1"}="H182 GR1"; $resn_exch{"1H18 GR1"}="H181 GR1";
$resn_exch{"2H19 GR1"}="H192 GR1"; $resn_exch{"1H19 GR1"}="H191 GR1"; $resn_exch{"2H20 GR1"}="H202 GR1";
$resn_exch{"1H20 GR1"}="H201 GR1"; $resn_exch{"2H3  GR1"}=" H32 GR1"; $resn_exch{"1H3  GR1"}=" H31 GR1";
$resn_exch{"2H14 GR1"}="H142 GR1"; $resn_exch{"1H14 GR1"}="H141 GR1"; $resn_exch{"1H3C GR1"}="H3C1 GR1";
$resn_exch{"3H27 GR1"}="H273 GR1"; $resn_exch{"2H27 GR1"}="H272 GR1"; $resn_exch{"1H27 GR1"}="H271 GR1";
$resn_exch{"2H1  GR1"}=" H12 GR1"; $resn_exch{"1H1  GR1"}=" H11 GR1"; $resn_exch{"3H2  GR1"}="H23A GR1";
$resn_exch{"2H2  GR1"}="H22A GR1"; $resn_exch{"1H2  GR1"}=" H21 GR1"; $resn_exch{"1H1  GR3"}=" H11 GR3";
$resn_exch{"2H1  GR3"}=" H12 GR3"; $resn_exch{"1H2  GR3"}="H21A GR3"; $resn_exch{"2H2  GR3"}=" H22 GR3";
$resn_exch{"1H6  GR3"}=" H61 GR3"; $resn_exch{"2H6  GR3"}=" H62 GR3"; $resn_exch{"1H11 GR3"}="H111 GR3";
$resn_exch{"2H11 GR3"}="H112 GR3"; $resn_exch{"1H12 GR3"}="H121 GR3"; $resn_exch{"2H12 GR3"}="H122 GR3";
$resn_exch{"1H15 GR3"}="H151 GR3"; $resn_exch{"2H15 GR3"}="H152 GR3"; $resn_exch{"1H18 GR3"}="H181 GR3";
$resn_exch{"2H18 GR3"}="H182 GR3"; $resn_exch{"3H18 GR3"}="H183 GR3"; $resn_exch{"1H19 GR3"}="H191 GR3";
$resn_exch{"2H19 GR3"}="H192 GR3"; $resn_exch{"3H19 GR3"}="H193 GR3"; $resn_exch{"1H22 GR3"}="H221 GR3";
$resn_exch{"2H22 GR3"}="H222 GR3"; $resn_exch{"1H26 GR3"}="H261 GR3"; $resn_exch{"2H26 GR3"}="H262 GR3";
$resn_exch{"3H26 GR3"}="H263 GR3"; $resn_exch{"1H27 GR3"}="H271 GR3"; $resn_exch{"2H27 GR3"}="H272 GR3";
$resn_exch{"3H27 GR3"}="H273 GR3"; $resn_exch{"1H28 GR3"}="H281 GR3"; $resn_exch{"2H28 GR3"}="H282 GR3";
$resn_exch{"3H28 GR3"}="H283 GR3"; $resn_exch{"1H30 GR3"}="H301 GR3"; $resn_exch{"2H30 GR3"}="H302 GR3";
$resn_exch{"3H30 GR3"}="H303 GR3"; $resn_exch{"1H38 GR3"}="H381 GR3"; $resn_exch{"2H38 GR3"}="H382 GR3";
$resn_exch{"3H38 GR3"}="H383 GR3"; $resn_exch{"1H1  GR4"}=" H11 GR4"; $resn_exch{"2H1  GR4"}="H12A GR4";
$resn_exch{"1H2  GR4"}="H21A GR4"; $resn_exch{"2H2  GR4"}=" H22 GR4"; $resn_exch{"1H6  GR4"}=" H61 GR4";
$resn_exch{"2H6  GR4"}=" H62 GR4"; $resn_exch{"1H11 GR4"}="H111 GR4"; $resn_exch{"2H11 GR4"}="H112 GR4";
$resn_exch{"1H15 GR4"}="H151 GR4"; $resn_exch{"2H15 GR4"}="H152 GR4"; $resn_exch{"1H18 GR4"}="H181 GR4";
$resn_exch{"2H18 GR4"}="H182 GR4"; $resn_exch{"3H18 GR4"}="H183 GR4"; $resn_exch{"1H19 GR4"}="H191 GR4";
$resn_exch{"2H19 GR4"}="H192 GR4"; $resn_exch{"3H19 GR4"}="H193 GR4"; $resn_exch{"1H22 GR4"}="H221 GR4";
$resn_exch{"2H22 GR4"}="H222 GR4"; $resn_exch{"1H26 GR4"}="H261 GR4"; $resn_exch{"2H26 GR4"}="H262 GR4";
$resn_exch{"3H26 GR4"}="H263 GR4"; $resn_exch{"1H27 GR4"}="H271 GR4"; $resn_exch{"2H27 GR4"}="H272 GR4";
$resn_exch{"3H27 GR4"}="H273 GR4"; $resn_exch{"1H28 GR4"}="H281 GR4"; $resn_exch{"2H28 GR4"}="H282 GR4";
$resn_exch{"1H30 GR4"}="H301 GR4"; $resn_exch{"2H30 GR4"}="H302 GR4"; $resn_exch{"3H30 GR4"}="H303 GR4";
$resn_exch{"1H38 GR4"}="H381 GR4"; $resn_exch{"2H38 GR4"}="H382 GR4"; $resn_exch{"3H38 GR4"}="H383 GR4";
$resn_exch{"1H31 GR4"}="H311 GR4"; $resn_exch{"2H31 GR4"}="H312 GR4"; $resn_exch{"1H10 GRF"}="H101 GRF";
$resn_exch{"2H10 GRF"}="H102 GRF"; $resn_exch{"1H19 GRF"}="H191 GRF"; $resn_exch{"2H19 GRF"}="H192 GRF";
$resn_exch{"1H20 GRG"}="H201 GRG"; $resn_exch{"2H20 GRG"}="H202 GRG"; $resn_exch{"3H20 GRG"}="H203 GRG";
$resn_exch{"1H19 GRG"}="H191 GRG"; $resn_exch{"2H19 GRG"}="H192 GRG"; $resn_exch{"3H19 GRG"}="H193 GRG";
$resn_exch{"1H16 GRG"}="H161 GRG"; $resn_exch{"2H16 GRG"}="H162 GRG"; $resn_exch{"1H14 GRG"}="H141 GRG";
$resn_exch{"2H14 GRG"}="H142 GRG"; $resn_exch{"3H14 GRG"}="H143 GRG"; $resn_exch{"1H15 GRG"}="H151 GRG";
$resn_exch{"2H15 GRG"}="H152 GRG"; $resn_exch{"1H11 GRG"}="H111 GRG"; $resn_exch{"2H11 GRG"}="H112 GRG";
$resn_exch{"1HC9 GRG"}="HC91 GRG"; $resn_exch{"2HC9 GRG"}="HC92 GRG"; $resn_exch{"1H10 GRG"}="H101 GRG";
$resn_exch{"2H10 GRG"}="H102 GRG"; $resn_exch{"3H10 GRG"}="H103 GRG"; $resn_exch{"1HC6 GRG"}="HC61 GRG";
$resn_exch{"2HC6 GRG"}="HC62 GRG"; $resn_exch{"1HC5 GRG"}="HC51 GRG"; $resn_exch{"2HC5 GRG"}="HC52 GRG";
$resn_exch{"1HC4 GRG"}="HC41 GRG"; $resn_exch{"2HC4 GRG"}="HC42 GRG"; $resn_exch{"3HC4 GRG"}="HC43 GRG";
$resn_exch{"1HC1 GRG"}="HC11 GRG"; $resn_exch{"2HC1 GRG"}="HC12 GRG"; $resn_exch{"OH1A GRG"}="H1AO GRG";
$resn_exch{"OH1B GRG"}="H1BO GRG"; $resn_exch{"OH2B GRG"}="H2BO GRG"; $resn_exch{"1H12 GRL"}="H121 GRL";
$resn_exch{"2H12 GRL"}="H122 GRL"; $resn_exch{"1H14 GRL"}="H141 GRL"; $resn_exch{"2H14 GRL"}="H142 GRL";
$resn_exch{"1H16 GRL"}="H161 GRL"; $resn_exch{"2H16 GRL"}="H162 GRL"; $resn_exch{"1H2  GRL"}=" H21 GRL";
$resn_exch{"2H2  GRL"}="H22A GRL"; $resn_exch{"1H23 GRL"}="H231 GRL"; $resn_exch{"2H23 GRL"}="H232 GRL";
$resn_exch{"1H32 GRL"}="H321 GRL"; $resn_exch{"2H32 GRL"}="H322 GRL"; $resn_exch{"1H33 GRL"}="H331 GRL";
$resn_exch{"2H33 GRL"}="H332 GRL"; $resn_exch{"1H35 GRL"}="H351 GRL"; $resn_exch{"2H35 GRL"}="H352 GRL";
$resn_exch{"3H35 GRL"}="H353 GRL"; $resn_exch{"1H36 GRL"}="H361 GRL"; $resn_exch{"2H36 GRL"}="H362 GRL";
$resn_exch{"3H36 GRL"}="H363 GRL"; $resn_exch{"1H3  GRL"}="H31A GRL"; $resn_exch{"2H3  GRL"}=" H32 GRL";
$resn_exch{"1HC3 GRO"}="HC31 GRO"; $resn_exch{"2HC3 GRO"}="HC32 GRO"; $resn_exch{"3HC3 GRO"}="HC33 GRO";
$resn_exch{"2HSP  GS"}="HSP2  GS"; $resn_exch{"3HOP  GS"}="HOP3  GS"; $resn_exch{"1H5*  GS"}=" H5'  GS";
$resn_exch{"2H5*  GS"}="H5''  GS"; $resn_exch{" H4*  GS"}=" H4'  GS"; $resn_exch{" H3*  GS"}=" H3'  GS";
$resn_exch{"*HO3  GS"}="HO3'  GS"; $resn_exch{"1H2*  GS"}=" H2'  GS"; $resn_exch{"2H2*  GS"}="H2''  GS";
$resn_exch{" H1*  GS"}=" H1'  GS"; $resn_exch{"1HN2  GS"}="HN21  GS"; $resn_exch{"2HN2  GS"}="HN22  GS";
$resn_exch{"1H6  GS1"}=" H61 GS1"; $resn_exch{"2H6  GS1"}=" H62 GS1"; $resn_exch{"1H6  GS4"}=" H61 GS4";
$resn_exch{"2H6  GS4"}=" H62 GS4"; $resn_exch{"1H16 GS5"}="H161 GS5"; $resn_exch{"2H16 GS5"}="H162 GS5";
$resn_exch{"1H20 GS5"}="H201 GS5"; $resn_exch{"2H20 GS5"}="H202 GS5"; $resn_exch{"1H23 GS5"}="H231 GS5";
$resn_exch{"2H23 GS5"}="H232 GS5"; $resn_exch{"3H23 GS5"}="H233 GS5"; $resn_exch{"1H26 GS5"}="H261 GS5";
$resn_exch{"2H26 GS5"}="H262 GS5"; $resn_exch{"1H27 GS5"}="H271 GS5"; $resn_exch{"2H27 GS5"}="H272 GS5";
$resn_exch{"1H28 GS5"}="H281 GS5"; $resn_exch{"2H28 GS5"}="H282 GS5"; $resn_exch{"1H29 GS5"}="H291 GS5";
$resn_exch{"2H29 GS5"}="H292 GS5"; $resn_exch{"1H13 GS6"}="H131 GS6"; $resn_exch{"2H13 GS6"}="H132 GS6";
$resn_exch{"1H14 GS6"}="H141 GS6"; $resn_exch{"2H14 GS6"}="H142 GS6"; $resn_exch{"1H17 GS6"}="H171 GS6";
$resn_exch{"2H17 GS6"}="H172 GS6"; $resn_exch{"3H17 GS6"}="H173 GS6"; $resn_exch{"1H19 GS6"}="H191 GS6";
$resn_exch{"2H19 GS6"}="H192 GS6"; $resn_exch{"1H23 GS6"}="H231 GS6"; $resn_exch{"2H23 GS6"}="H232 GS6";
$resn_exch{"1H20 GS6"}="H201 GS6"; $resn_exch{"2H20 GS6"}="H202 GS6"; $resn_exch{"1H22 GS6"}="H221 GS6";
$resn_exch{"2H22 GS6"}="H222 GS6"; $resn_exch{"1H6  GSA"}=" H61 GSA"; $resn_exch{"2H6  GSA"}=" H62 GSA";
$resn_exch{"3HOS GSA"}="HOS3 GSA"; $resn_exch{"1HN1 GSB"}="HN11 GSB"; $resn_exch{"2HN1 GSB"}="HN12 GSB";
$resn_exch{"1HB1 GSB"}="HB11 GSB"; $resn_exch{"2HB1 GSB"}="HB12 GSB"; $resn_exch{"1HG1 GSB"}="HG11 GSB";
$resn_exch{"2HG1 GSB"}="HG12 GSB"; $resn_exch{"1HA3 GSB"}="HA31 GSB"; $resn_exch{"2HA3 GSB"}="HA32 GSB";
$resn_exch{"1HB2 GSB"}="HB21 GSB"; $resn_exch{"2HB2 GSB"}="HB22 GSB"; $resn_exch{"1H'  GSB"}=" H'1 GSB";
$resn_exch{"2H'  GSB"}=" H'2 GSB"; $resn_exch{"1HN  GSC"}=" H   GSC"; $resn_exch{"2HN  GSC"}=" H2  GSC";
$resn_exch{"1H1  GSC"}=" H11 GSC"; $resn_exch{"2H1  GSC"}=" H12 GSC"; $resn_exch{"1H2  GSC"}=" H21 GSC";
$resn_exch{"2H2  GSC"}=" H22 GSC"; $resn_exch{"3H2  GSC"}=" H23 GSC"; $resn_exch{"1H6  GSD"}=" H61 GSD";
$resn_exch{"2H6  GSD"}=" H62 GSD"; $resn_exch{"1H1  GSE"}=" H11 GSE"; $resn_exch{"2H1  GSE"}="H12A GSE";
$resn_exch{"1H3  GSE"}=" H31 GSE"; $resn_exch{"2H3  GSE"}=" H32 GSE"; $resn_exch{"1H11 GSE"}="H111 GSE";
$resn_exch{"2H11 GSE"}="H112 GSE"; $resn_exch{"1HN  GSE"}=" HN1 GSE"; $resn_exch{"2HN  GSE"}=" HN2 GSE";
$resn_exch{"1HB1 GSF"}="HB11 GSF"; $resn_exch{"2HB1 GSF"}="HB12 GSF"; $resn_exch{"1HG1 GSF"}="HG11 GSF";
$resn_exch{"2HG1 GSF"}="HG12 GSF"; $resn_exch{"1HN1 GSF"}="HN11 GSF"; $resn_exch{"2HN1 GSF"}="HN12 GSF";
$resn_exch{"1HB2 GSF"}="HB21 GSF"; $resn_exch{"2HB2 GSF"}="HB22 GSF"; $resn_exch{"1HA3 GSF"}="HA31 GSF";
$resn_exch{"2HA3 GSF"}="HA32 GSF"; $resn_exch{"1H8  GSG"}=" H81 GSG"; $resn_exch{"2H8  GSG"}=" H82 GSG";
$resn_exch{"1H9  GSG"}=" H91 GSG"; $resn_exch{"2H9  GSG"}=" H92 GSG"; $resn_exch{"3H9  GSG"}=" H93 GSG";
$resn_exch{"1H14 GSG"}="H141 GSG"; $resn_exch{"2H14 GSG"}="H142 GSG"; $resn_exch{"3H14 GSG"}="H143 GSG";
$resn_exch{"1H19 GSG"}="H191 GSG"; $resn_exch{"2H19 GSG"}="H192 GSG"; $resn_exch{"3H19 GSG"}="H193 GSG";
$resn_exch{"1H20 GSG"}="H201 GSG"; $resn_exch{"2H20 GSG"}="H202 GSG"; $resn_exch{"1H21 GSG"}="H211 GSG";
$resn_exch{"2H21 GSG"}="H212 GSG"; $resn_exch{"1H22 GSG"}="H221 GSG"; $resn_exch{"2H22 GSG"}="H222 GSG";
$resn_exch{"3H22 GSG"}="H223 GSG"; $resn_exch{"1HN1 GSH"}="HN11 GSH"; $resn_exch{"2HN1 GSH"}="HN12 GSH";
$resn_exch{"2HB1 GSH"}="HB12 GSH"; $resn_exch{"3HB1 GSH"}="HB13 GSH"; $resn_exch{"2HG1 GSH"}="HG12 GSH";
$resn_exch{"3HG1 GSH"}="HG13 GSH"; $resn_exch{"2HB2 GSH"}="HB22 GSH"; $resn_exch{"3HB2 GSH"}="HB23 GSH";
$resn_exch{"1HA3 GSH"}="HA31 GSH"; $resn_exch{"2HA3 GSH"}="HA32 GSH"; $resn_exch{"1H23 GSJ"}="H231 GSJ";
$resn_exch{"2H23 GSJ"}="H232 GSJ"; $resn_exch{"3H23 GSJ"}="H233 GSJ"; $resn_exch{"1H22 GSJ"}="H221 GSJ";
$resn_exch{"2H22 GSJ"}="H222 GSJ"; $resn_exch{"1H11 GSJ"}="H111 GSJ"; $resn_exch{"2H11 GSJ"}="H112 GSJ";
$resn_exch{"1H20 GSJ"}="H201 GSJ"; $resn_exch{"2H20 GSJ"}="H202 GSJ"; $resn_exch{"3H20 GSJ"}="H203 GSJ";
$resn_exch{"1H21 GSJ"}="H211 GSJ"; $resn_exch{"2H21 GSJ"}="H212 GSJ"; $resn_exch{"3H21 GSJ"}="H213 GSJ";
$resn_exch{"1H17 GSJ"}="H171 GSJ"; $resn_exch{"2H17 GSJ"}="H172 GSJ"; $resn_exch{"3H17 GSJ"}="H173 GSJ";
$resn_exch{"1H14 GSJ"}="H141 GSJ"; $resn_exch{"2H14 GSJ"}="H142 GSJ"; $resn_exch{"1H13 GSJ"}="H131 GSJ";
$resn_exch{"2H13 GSJ"}="H132 GSJ"; $resn_exch{"1H13 GSK"}="H131 GSK"; $resn_exch{"2H13 GSK"}="H132 GSK";
$resn_exch{"1H14 GSK"}="H141 GSK"; $resn_exch{"2H14 GSK"}="H142 GSK"; $resn_exch{"1H17 GSK"}="H171 GSK";
$resn_exch{"2H17 GSK"}="H172 GSK"; $resn_exch{"3H17 GSK"}="H173 GSK"; $resn_exch{"1H19 GSK"}="H191 GSK";
$resn_exch{"2H19 GSK"}="H192 GSK"; $resn_exch{"1H20 GSK"}="H201 GSK"; $resn_exch{"2H20 GSK"}="H202 GSK";
$resn_exch{"1H22 GSK"}="H221 GSK"; $resn_exch{"2H22 GSK"}="H222 GSK"; $resn_exch{"1H23 GSK"}="H231 GSK";
$resn_exch{"2H23 GSK"}="H232 GSK"; $resn_exch{"1H17 GSL"}="H171 GSL"; $resn_exch{"2H17 GSL"}="H172 GSL";
$resn_exch{"3H17 GSL"}="H173 GSL"; $resn_exch{"1H16 GSL"}="H161 GSL"; $resn_exch{"2H16 GSL"}="H162 GSL";
$resn_exch{"1H15 GSL"}="H151 GSL"; $resn_exch{"2H15 GSL"}="H152 GSL"; $resn_exch{"1H14 GSL"}="H141 GSL";
$resn_exch{"2H14 GSL"}="H142 GSL"; $resn_exch{"1H13 GSL"}="H131 GSL"; $resn_exch{"2H13 GSL"}="H132 GSL";
$resn_exch{"1H12 GSL"}="H121 GSL"; $resn_exch{"2H12 GSL"}="H122 GSL"; $resn_exch{"1H11 GSL"}="H111 GSL";
$resn_exch{"2H11 GSL"}="H112 GSL"; $resn_exch{"1H10 GSL"}="H101 GSL"; $resn_exch{"2H10 GSL"}="H102 GSL";
$resn_exch{"1H9  GSL"}=" H91 GSL"; $resn_exch{"2H9  GSL"}=" H92 GSL"; $resn_exch{"1H8  GSL"}=" H81 GSL";
$resn_exch{"2H8  GSL"}=" H82 GSL"; $resn_exch{"1H7  GSL"}=" H71 GSL"; $resn_exch{"2H7  GSL"}=" H72 GSL";
$resn_exch{"1H6  GSL"}=" H61 GSL"; $resn_exch{"2H6  GSL"}=" H62 GSL"; $resn_exch{"1H5  GSL"}=" H51 GSL";
$resn_exch{"2H5  GSL"}=" H52 GSL"; $resn_exch{"1H4  GSL"}=" H41 GSL"; $resn_exch{"2H4  GSL"}=" H42 GSL";
$resn_exch{"1H3  GSL"}="H31A GSL"; $resn_exch{"2H3  GSL"}="H32A GSL"; $resn_exch{"1H   GSL"}=" H1A GSL";
$resn_exch{"2H   GSL"}=" H2A GSL"; $resn_exch{"1H19 GSL"}="H191 GSL"; $resn_exch{"2H19 GSL"}="H192 GSL";
$resn_exch{"1H20 GSL"}="H201 GSL"; $resn_exch{"2H20 GSL"}="H202 GSL"; $resn_exch{"1H21 GSL"}="H211 GSL";
$resn_exch{"2H21 GSL"}="H212 GSL"; $resn_exch{"1H22 GSL"}="H221 GSL"; $resn_exch{"2H22 GSL"}="H222 GSL";
$resn_exch{"1H23 GSL"}="H231 GSL"; $resn_exch{"2H23 GSL"}="H232 GSL"; $resn_exch{"1H24 GSL"}="H241 GSL";
$resn_exch{"2H24 GSL"}="H242 GSL"; $resn_exch{"1H25 GSL"}="H251 GSL"; $resn_exch{"2H25 GSL"}="H252 GSL";
$resn_exch{"1H26 GSL"}="H261 GSL"; $resn_exch{"2H26 GSL"}="H262 GSL"; $resn_exch{"1H27 GSL"}="H271 GSL";
$resn_exch{"2H27 GSL"}="H272 GSL"; $resn_exch{"1H38 GSL"}="H381 GSL"; $resn_exch{"2H38 GSL"}="H382 GSL";
$resn_exch{"1H37 GSL"}="H371 GSL"; $resn_exch{"2H37 GSL"}="H372 GSL"; $resn_exch{"1H36 GSL"}="H361 GSL";
$resn_exch{"2H36 GSL"}="H362 GSL"; $resn_exch{"1H35 GSL"}="H351 GSL"; $resn_exch{"2H35 GSL"}="H352 GSL";
$resn_exch{"3H35 GSL"}="H353 GSL"; $resn_exch{"1HA3 GSM"}="HA31 GSM"; $resn_exch{"2HA3 GSM"}="HA32 GSM";
$resn_exch{"1HB2 GSM"}="HB21 GSM"; $resn_exch{"2HB2 GSM"}="HB22 GSM"; $resn_exch{"1H4  GSM"}=" H41 GSM";
$resn_exch{"2H4  GSM"}=" H42 GSM"; $resn_exch{"3H4  GSM"}=" H43 GSM"; $resn_exch{"1HG1 GSM"}="HG11 GSM";
$resn_exch{"2HG1 GSM"}="HG12 GSM"; $resn_exch{"1HB1 GSM"}="HB11 GSM"; $resn_exch{"2HB1 GSM"}="HB12 GSM";
$resn_exch{"1HN1 GSM"}="HN11 GSM"; $resn_exch{"2HN1 GSM"}="HN12 GSM"; $resn_exch{"1HN  GSN"}=" HN1 GSN";
$resn_exch{"2HN  GSN"}=" HN2 GSN"; $resn_exch{"1HB  GSN"}=" HB1 GSN"; $resn_exch{"2HB  GSN"}=" HB2 GSN";
$resn_exch{"1HAG GSN"}="HAG1 GSN"; $resn_exch{"2HAG GSN"}="HAG2 GSN"; $resn_exch{"1HAL GSN"}="HAL1 GSN";
$resn_exch{"2HAL GSN"}="HAL2 GSN"; $resn_exch{"1HAR GSN"}="HAR1 GSN"; $resn_exch{"2HAR GSN"}="HAR2 GSN";
$resn_exch{"1HN  GSO"}=" HN1 GSO"; $resn_exch{"2HN  GSO"}=" HN2 GSO"; $resn_exch{"1HBC GSO"}="HBC1 GSO";
$resn_exch{"2HBC GSO"}="HBC2 GSO"; $resn_exch{"1HB2 GSO"}="HB21 GSO"; $resn_exch{"2HB2 GSO"}="HB22 GSO";
$resn_exch{"1HGC GSO"}="HGC1 GSO"; $resn_exch{"2HGC GSO"}="HGC2 GSO"; $resn_exch{"1HA3 GSO"}="HA31 GSO";
$resn_exch{"2HA3 GSO"}="HA32 GSO"; $resn_exch{"1HD2 GSO"}="HD21 GSO"; $resn_exch{"2HD2 GSO"}="HD22 GSO";
$resn_exch{"2HOG GSP"}="HOG2 GSP"; $resn_exch{"3HOG GSP"}="HOG3 GSP"; $resn_exch{"2HOB GSP"}="HOB2 GSP";
$resn_exch{"2HOA GSP"}="HOA2 GSP"; $resn_exch{"1H5* GSP"}="H5'1 GSP"; $resn_exch{"2H5* GSP"}="H5'2 GSP";
$resn_exch{" H4* GSP"}=" H4' GSP"; $resn_exch{" H3* GSP"}=" H3' GSP"; $resn_exch{"*HO3 GSP"}="HO3' GSP";
$resn_exch{" H2* GSP"}=" H2' GSP"; $resn_exch{"*HO2 GSP"}="HO2' GSP"; $resn_exch{" H1* GSP"}=" H1' GSP";
$resn_exch{"1HN2 GSP"}="HN21 GSP"; $resn_exch{"2HN2 GSP"}="HN22 GSP"; $resn_exch{"1H16 GSQ"}="H161 GSQ";
$resn_exch{"2H16 GSQ"}="H162 GSQ"; $resn_exch{"1H20 GSQ"}="H201 GSQ"; $resn_exch{"2H20 GSQ"}="H202 GSQ";
$resn_exch{"1H23 GSQ"}="H231 GSQ"; $resn_exch{"2H23 GSQ"}="H232 GSQ"; $resn_exch{"3H23 GSQ"}="H233 GSQ";
$resn_exch{"1H26 GSQ"}="H261 GSQ"; $resn_exch{"2H26 GSQ"}="H262 GSQ"; $resn_exch{"1H27 GSQ"}="H271 GSQ";
$resn_exch{"2H27 GSQ"}="H272 GSQ"; $resn_exch{"1H28 GSQ"}="H281 GSQ"; $resn_exch{"2H28 GSQ"}="H282 GSQ";
$resn_exch{"1H29 GSQ"}="H291 GSQ"; $resn_exch{"2H29 GSQ"}="H292 GSQ"; $resn_exch{"2HOP GSR"}="HOP2 GSR";
$resn_exch{"3HOP GSR"}="HOP3 GSR"; $resn_exch{"1H5* GSR"}=" H5' GSR"; $resn_exch{"2H5* GSR"}="H5'' GSR";
$resn_exch{" H4* GSR"}=" H4' GSR"; $resn_exch{" H3* GSR"}=" H3' GSR"; $resn_exch{"*HO3 GSR"}="HO3' GSR";
$resn_exch{"1H2* GSR"}=" H2' GSR"; $resn_exch{"2H2* GSR"}="H2'' GSR"; $resn_exch{" H1* GSR"}=" H1' GSR";
$resn_exch{"1HB  GSR"}=" HB1 GSR"; $resn_exch{"2HB  GSR"}=" HB2 GSR"; $resn_exch{"2HOP GSS"}="HOP2 GSS";
$resn_exch{"3HOP GSS"}="HOP3 GSS"; $resn_exch{"1H5* GSS"}=" H5' GSS"; $resn_exch{"2H5* GSS"}="H5'' GSS";
$resn_exch{" H4* GSS"}=" H4' GSS"; $resn_exch{" H3* GSS"}=" H3' GSS"; $resn_exch{"*HO3 GSS"}="HO3' GSS";
$resn_exch{"1H2* GSS"}=" H2' GSS"; $resn_exch{"2H2* GSS"}="H2'' GSS"; $resn_exch{" H1* GSS"}=" H1' GSS";
$resn_exch{"1HB  GSS"}=" HB1 GSS"; $resn_exch{"2HB  GSS"}=" HB2 GSS"; $resn_exch{"1H1  GST"}=" H11 GST";
$resn_exch{"2H1  GST"}=" H12 GST"; $resn_exch{"1H10 GST"}="H101 GST"; $resn_exch{"2H10 GST"}="H102 GST";
$resn_exch{"3H10 GST"}="H103 GST"; $resn_exch{"1H4  GST"}=" H41 GST"; $resn_exch{"2H4  GST"}=" H42 GST";
$resn_exch{"1H5  GST"}=" H51 GST"; $resn_exch{"2H5  GST"}=" H52 GST"; $resn_exch{"1H9  GST"}=" H91 GST";
$resn_exch{"2H9  GST"}=" H92 GST"; $resn_exch{"3H9  GST"}=" H93 GST"; $resn_exch{"1H8  GST"}=" H81 GST";
$resn_exch{"2H8  GST"}=" H82 GST"; $resn_exch{"3H8  GST"}=" H83 GST"; $resn_exch{" H2  GSU"}=" HC2 GSU";
$resn_exch{"1HN  GSU"}=" H   GSU"; $resn_exch{"2HN  GSU"}=" H2  GSU"; $resn_exch{"1HB  GSU"}=" HB2 GSU";
$resn_exch{"2HB  GSU"}=" HB3 GSU"; $resn_exch{"1HG  GSU"}=" HG2 GSU"; $resn_exch{"2HG  GSU"}=" HG3 GSU";
$resn_exch{"1H5* GSU"}="H5'1 GSU"; $resn_exch{"2H5* GSU"}="H5'2 GSU"; $resn_exch{" H4* GSU"}=" H4' GSU";
$resn_exch{" H1* GSU"}=" H1' GSU"; $resn_exch{"1HN6 GSU"}="HN61 GSU"; $resn_exch{"2HN6 GSU"}="HN62 GSU";
$resn_exch{" H2* GSU"}=" H2' GSU"; $resn_exch{"*HO2 GSU"}="HO2' GSU"; $resn_exch{" H3* GSU"}=" H3' GSU";
$resn_exch{"*HO3 GSU"}="HO3' GSU"; $resn_exch{"1H1N GSW"}="H1N1 GSW"; $resn_exch{"2H1N GSW"}="H1N2 GSW";
$resn_exch{"1HB1 GSW"}="HB11 GSW"; $resn_exch{"2HB1 GSW"}="HB12 GSW"; $resn_exch{"1HG1 GSW"}="HG11 GSW";
$resn_exch{"2HG1 GSW"}="HG12 GSW"; $resn_exch{"1HB2 GSW"}="HB21 GSW"; $resn_exch{"2HB2 GSW"}="HB22 GSW";
$resn_exch{"1HA3 GSW"}="HA31 GSW"; $resn_exch{"2HA3 GSW"}="HA32 GSW"; $resn_exch{"1H13 GSX"}="H131 GSX";
$resn_exch{"2H13 GSX"}="H132 GSX"; $resn_exch{"1H14 GSX"}="H141 GSX"; $resn_exch{"2H14 GSX"}="H142 GSX";
$resn_exch{"1H17 GSX"}="H171 GSX"; $resn_exch{"2H17 GSX"}="H172 GSX"; $resn_exch{"3H17 GSX"}="H173 GSX";
$resn_exch{"1H19 GSX"}="H191 GSX"; $resn_exch{"2H19 GSX"}="H192 GSX"; $resn_exch{"1H23 GSX"}="H231 GSX";
$resn_exch{"2H23 GSX"}="H232 GSX"; $resn_exch{"1H20 GSX"}="H201 GSX"; $resn_exch{"2H20 GSX"}="H202 GSX";
$resn_exch{"1H22 GSX"}="H221 GSX"; $resn_exch{"2H22 GSX"}="H222 GSX"; $resn_exch{"1HAZ GSZ"}="HAZ1 GSZ";
$resn_exch{"2HAZ GSZ"}="HAZ2 GSZ"; $resn_exch{"3HAZ GSZ"}="HAZ3 GSZ"; $resn_exch{"1HBA GSZ"}="HBA1 GSZ";
$resn_exch{"2HBA GSZ"}="HBA2 GSZ"; $resn_exch{"3HBA GSZ"}="HBA3 GSZ"; $resn_exch{"1HAI GSZ"}="HAI1 GSZ";
$resn_exch{"2HAI GSZ"}="HAI2 GSZ"; $resn_exch{"1HAR GSZ"}="HAR1 GSZ"; $resn_exch{"2HAR GSZ"}="HAR2 GSZ";
$resn_exch{"1HAS GSZ"}="HAS1 GSZ"; $resn_exch{"2HAS GSZ"}="HAS2 GSZ"; $resn_exch{"1HN  GT9"}=" H   GT9";
$resn_exch{"2HN  GT9"}=" H2  GT9"; $resn_exch{"1HB  GT9"}=" HB2 GT9"; $resn_exch{"2HB  GT9"}=" HB3 GT9";
$resn_exch{"1H1  GT9"}=" H11 GT9"; $resn_exch{"2H1  GT9"}=" H12 GT9"; $resn_exch{"1H2  GT9"}=" H21 GT9";
$resn_exch{"2H2  GT9"}=" H22 GT9"; $resn_exch{"1H3  GT9"}=" H31 GT9"; $resn_exch{"2H3  GT9"}=" H32 GT9";
$resn_exch{"1H4  GT9"}=" H41 GT9"; $resn_exch{"2H4  GT9"}=" H42 GT9"; $resn_exch{"1H5  GT9"}=" H51 GT9";
$resn_exch{"2H5  GT9"}=" H52 GT9"; $resn_exch{"1H6  GT9"}=" H61 GT9"; $resn_exch{"2H6  GT9"}=" H62 GT9";
$resn_exch{"1H7  GT9"}=" H71 GT9"; $resn_exch{"2H7  GT9"}=" H72 GT9"; $resn_exch{"1H8  GT9"}=" H81 GT9";
$resn_exch{"2H8  GT9"}=" H82 GT9"; $resn_exch{"1H9  GT9"}=" H91 GT9"; $resn_exch{"2H9  GT9"}=" H92 GT9";
$resn_exch{"3H9  GT9"}=" H93 GT9"; $resn_exch{"1HN2 GTA"}="HN21 GTA"; $resn_exch{"2HN2 GTA"}="HN22 GTA";
$resn_exch{"1H7  GTA"}=" H71 GTA"; $resn_exch{"2H7  GTA"}=" H72 GTA"; $resn_exch{"3H7  GTA"}=" H73 GTA";
$resn_exch{"AHO3 GTA"}="HO3A GTA"; $resn_exch{"1H5A GTA"}="H5A1 GTA"; $resn_exch{"2H5A GTA"}="H5A2 GTA";
$resn_exch{"AHO2 GTA"}="HO2A GTA"; $resn_exch{"2H12 GTA"}="H122 GTA"; $resn_exch{"2H22 GTA"}="H222 GTA";
$resn_exch{"2H32 GTA"}="H322 GTA"; $resn_exch{"1H5B GTA"}="H5B1 GTA"; $resn_exch{"2H5B GTA"}="H5B2 GTA";
$resn_exch{"BHO3 GTA"}="HO3B GTA"; $resn_exch{"BHO2 GTA"}="HO2B GTA"; $resn_exch{"1HN6 GTA"}="HN61 GTA";
$resn_exch{"2HN6 GTA"}="HN62 GTA"; $resn_exch{"1HN1 GTB"}="HN11 GTB"; $resn_exch{"2HN1 GTB"}="HN12 GTB";
$resn_exch{"1HB1 GTB"}="HB11 GTB"; $resn_exch{"2HB1 GTB"}="HB12 GTB"; $resn_exch{"1HG1 GTB"}="HG11 GTB";
$resn_exch{"2HG1 GTB"}="HG12 GTB"; $resn_exch{"1HB2 GTB"}="HB21 GTB"; $resn_exch{"2HB2 GTB"}="HB22 GTB";
$resn_exch{"1HA3 GTB"}="HA31 GTB"; $resn_exch{"2HA3 GTB"}="HA32 GTB"; $resn_exch{"1H'  GTB"}=" H'1 GTB";
$resn_exch{"2H'  GTB"}=" H'2 GTB"; $resn_exch{"1HN1 GTD"}="HN11 GTD"; $resn_exch{"2HN1 GTD"}="HN12 GTD";
$resn_exch{"1HB1 GTD"}="HB11 GTD"; $resn_exch{"2HB1 GTD"}="HB12 GTD"; $resn_exch{"1HG1 GTD"}="HG11 GTD";
$resn_exch{"2HG1 GTD"}="HG12 GTD"; $resn_exch{"1HB2 GTD"}="HB21 GTD"; $resn_exch{"2HB2 GTD"}="HB22 GTD";
$resn_exch{"1HA3 GTD"}="HA31 GTD"; $resn_exch{"2HA3 GTD"}="HA32 GTD"; $resn_exch{"2HOA GTG"}="HOA2 GTG";
$resn_exch{"2HOB GTG"}="HOB2 GTG"; $resn_exch{"1HC7 GTG"}="HC71 GTG"; $resn_exch{"2HC7 GTG"}="HC72 GTG";
$resn_exch{"3HC7 GTG"}="HC73 GTG"; $resn_exch{"1HN2 GTG"}="HN21 GTG"; $resn_exch{"2HN2 GTG"}="HN22 GTG";
$resn_exch{"1H5* GTG"}="H5'1 GTG"; $resn_exch{"2H5* GTG"}="H5'2 GTG"; $resn_exch{" H4* GTG"}=" H4' GTG";
$resn_exch{" H3* GTG"}=" H3' GTG"; $resn_exch{"*HO3 GTG"}="HO3' GTG"; $resn_exch{" H2* GTG"}=" H2' GTG";
$resn_exch{"*HO2 GTG"}="HO2' GTG"; $resn_exch{" H1* GTG"}=" H1' GTG"; $resn_exch{"2HOG GTG"}="HOG2 GTG";
$resn_exch{"1H5B GTG"}="H5B1 GTG"; $resn_exch{"2H5B GTG"}="H5B2 GTG"; $resn_exch{"OH3B GTG"}="H3BO GTG";
$resn_exch{"OH2B GTG"}="H2BO GTG"; $resn_exch{"NH1B GTG"}="H1BN GTG"; $resn_exch{"1H2B GTG"}="H2B1 GTG";
$resn_exch{"2H2B GTG"}="H2B2 GTG"; $resn_exch{"1HN1 GTH"}="HN11 GTH"; $resn_exch{"2HN1 GTH"}="HN12 GTH";
$resn_exch{"1HG  GTH"}=" HG1 GTH"; $resn_exch{"2HG  GTH"}=" HG2 GTH"; $resn_exch{"3HG  GTH"}=" HG3 GTH";
$resn_exch{"1H6  GTH"}=" H61 GTH"; $resn_exch{"2H6  GTH"}=" H62 GTH"; $resn_exch{"1H12 GTH"}="H121 GTH";
$resn_exch{"2H12 GTH"}="H122 GTH"; $resn_exch{"1H14 GTH"}="H141 GTH"; $resn_exch{"2H14 GTH"}="H142 GTH";
$resn_exch{"3H14 GTH"}="H143 GTH"; $resn_exch{"1H6C GTL"}="H6C1 GTL"; $resn_exch{"2H6C GTL"}="H6C2 GTL";
$resn_exch{"1H6C GTM"}="H6C1 GTM"; $resn_exch{"2H6C GTM"}="H6C2 GTM"; $resn_exch{"1H7C GTM"}="H7C1 GTM";
$resn_exch{"2H7C GTM"}="H7C2 GTM"; $resn_exch{"3H7C GTM"}="H7C3 GTM"; $resn_exch{"2HOG GTN"}="HOG2 GTN";
$resn_exch{"3HOG GTN"}="HOG3 GTN"; $resn_exch{"2HOB GTN"}="HOB2 GTN"; $resn_exch{"2HOA GTN"}="HOA2 GTN";
$resn_exch{"1H5* GTN"}="H5'1 GTN"; $resn_exch{"2H5* GTN"}="H5'2 GTN"; $resn_exch{" H4* GTN"}=" H4' GTN";
$resn_exch{"1H3* GTN"}="H3'1 GTN"; $resn_exch{"2H3* GTN"}="H3'2 GTN"; $resn_exch{"1H2* GTN"}="H2'1 GTN";
$resn_exch{"2H2* GTN"}="H2'2 GTN"; $resn_exch{" H1* GTN"}=" H1' GTN"; $resn_exch{"1H2  GTN"}=" H21 GTN";
$resn_exch{"2H2  GTN"}=" H22 GTN"; $resn_exch{"2HOG GTO"}="HOG2 GTO"; $resn_exch{"3HOG GTO"}="HOG3 GTO";
$resn_exch{"1H3B GTO"}="H3B1 GTO"; $resn_exch{"2H3B GTO"}="H3B2 GTO"; $resn_exch{"2HOB GTO"}="HOB2 GTO";
$resn_exch{"2HOA GTO"}="HOA2 GTO"; $resn_exch{"1H5* GTO"}="H5'1 GTO"; $resn_exch{"2H5* GTO"}="H5'2 GTO";
$resn_exch{" H4* GTO"}=" H4' GTO"; $resn_exch{" H3* GTO"}=" H3' GTO"; $resn_exch{"*HO3 GTO"}="HO3' GTO";
$resn_exch{" H2* GTO"}=" H2' GTO"; $resn_exch{"*HO2 GTO"}="HO2' GTO"; $resn_exch{" H1* GTO"}=" H1' GTO";
$resn_exch{"1HN2 GTO"}="HN21 GTO"; $resn_exch{"2HN2 GTO"}="HN22 GTO"; $resn_exch{"2HOG GTP"}="HOG2 GTP";
$resn_exch{"3HOG GTP"}="HOG3 GTP"; $resn_exch{"2HOB GTP"}="HOB2 GTP"; $resn_exch{"2HOA GTP"}="HOA2 GTP";
$resn_exch{"1H5* GTP"}=" H5' GTP"; $resn_exch{"2H5* GTP"}="H5'' GTP"; $resn_exch{" H4* GTP"}=" H4' GTP";
$resn_exch{" H3* GTP"}=" H3' GTP"; $resn_exch{"*HO3 GTP"}="HO3' GTP"; $resn_exch{" H2* GTP"}=" H2' GTP";
$resn_exch{"*HO2 GTP"}="HO2' GTP"; $resn_exch{" H1* GTP"}=" H1' GTP"; $resn_exch{"1HN2 GTP"}="HN21 GTP";
$resn_exch{"2HN2 GTP"}="HN22 GTP"; $resn_exch{"1HN1 GTS"}="HN11 GTS"; $resn_exch{"2HN1 GTS"}="HN12 GTS";
$resn_exch{"1HB1 GTS"}="HB11 GTS"; $resn_exch{"2HB1 GTS"}="HB12 GTS"; $resn_exch{"1HG1 GTS"}="HG11 GTS";
$resn_exch{"2HG1 GTS"}="HG12 GTS"; $resn_exch{"1HB2 GTS"}="HB21 GTS"; $resn_exch{"2HB2 GTS"}="HB22 GTS";
$resn_exch{"3HOS GTS"}="HOS3 GTS"; $resn_exch{"1HA3 GTS"}="HA31 GTS"; $resn_exch{"2HA3 GTS"}="HA32 GTS";
$resn_exch{"1HN1 GTT"}="HN11 GTT"; $resn_exch{"2HN1 GTT"}="HN12 GTT"; $resn_exch{"3HN1 GTT"}="HN13 GTT";
$resn_exch{"1HB1 GTT"}="HB11 GTT"; $resn_exch{"2HB1 GTT"}="HB12 GTT"; $resn_exch{"1HG1 GTT"}="HG11 GTT";
$resn_exch{"2HG1 GTT"}="HG12 GTT"; $resn_exch{"1HB2 GTT"}="HB21 GTT"; $resn_exch{"2HB2 GTT"}="HB22 GTT";
$resn_exch{"1HA3 GTT"}="HA31 GTT"; $resn_exch{"2HA3 GTT"}="HA32 GTT"; $resn_exch{"1HN1 GTX"}="HN11 GTX";
$resn_exch{"2HN1 GTX"}="HN12 GTX"; $resn_exch{"3HN1 GTX"}="HN13 GTX"; $resn_exch{"1HB1 GTX"}="HB11 GTX";
$resn_exch{"2HB1 GTX"}="HB12 GTX"; $resn_exch{"1HG1 GTX"}="HG11 GTX"; $resn_exch{"2HG1 GTX"}="HG12 GTX";
$resn_exch{"1HB2 GTX"}="HB21 GTX"; $resn_exch{"2HB2 GTX"}="HB22 GTX"; $resn_exch{"1HS1 GTX"}="HS11 GTX";
$resn_exch{"2HS1 GTX"}="HS12 GTX"; $resn_exch{"1HS2 GTX"}="HS21 GTX"; $resn_exch{"2HS2 GTX"}="HS22 GTX";
$resn_exch{"1HS3 GTX"}="HS31 GTX"; $resn_exch{"2HS3 GTX"}="HS32 GTX"; $resn_exch{"1HS4 GTX"}="HS41 GTX";
$resn_exch{"2HS4 GTX"}="HS42 GTX"; $resn_exch{"1HS5 GTX"}="HS51 GTX"; $resn_exch{"2HS5 GTX"}="HS52 GTX";
$resn_exch{"1HS6 GTX"}="HS61 GTX"; $resn_exch{"2HS6 GTX"}="HS62 GTX"; $resn_exch{"3HS6 GTX"}="HS63 GTX";
$resn_exch{"1HA3 GTX"}="HA31 GTX"; $resn_exch{"2HA3 GTX"}="HA32 GTX"; $resn_exch{"1HN1 GTY"}="HN11 GTY";
$resn_exch{"2HN1 GTY"}="HN12 GTY"; $resn_exch{"1HB1 GTY"}="HB11 GTY"; $resn_exch{"2HB1 GTY"}="HB12 GTY";
$resn_exch{"1HG1 GTY"}="HG11 GTY"; $resn_exch{"2HG1 GTY"}="HG12 GTY"; $resn_exch{"1HB2 GTY"}="HB21 GTY";
$resn_exch{"2HB2 GTY"}="HB22 GTY"; $resn_exch{"1H1S GTY"}="H1S1 GTY"; $resn_exch{"2H1S GTY"}="H1S2 GTY";
$resn_exch{"1H2S GTY"}="H2S1 GTY"; $resn_exch{"2H2S GTY"}="H2S2 GTY"; $resn_exch{"1H3S GTY"}="H3S1 GTY";
$resn_exch{"2H3S GTY"}="H3S2 GTY"; $resn_exch{"1H4S GTY"}="H4S1 GTY"; $resn_exch{"2H4S GTY"}="H4S2 GTY";
$resn_exch{"1H5S GTY"}="H5S1 GTY"; $resn_exch{"2H5S GTY"}="H5S2 GTY"; $resn_exch{"1H6S GTY"}="H6S1 GTY";
$resn_exch{"2H6S GTY"}="H6S2 GTY"; $resn_exch{"1H7S GTY"}="H7S1 GTY"; $resn_exch{"2H7S GTY"}="H7S2 GTY";
$resn_exch{"1H8S GTY"}="H8S1 GTY"; $resn_exch{"2H8S GTY"}="H8S2 GTY"; $resn_exch{"3H8S GTY"}="H8S3 GTY";
$resn_exch{"1HA3 GTY"}="HA31 GTY"; $resn_exch{"2HA3 GTY"}="HA32 GTY"; $resn_exch{"1H6  GTZ"}=" H61 GTZ";
$resn_exch{"2H6  GTZ"}=" H62 GTZ"; $resn_exch{"1H6  GU0"}=" H61 GU0"; $resn_exch{"2H6  GU0"}=" H62 GU0";
$resn_exch{"1H7  GU1"}=" H71 GU1"; $resn_exch{"2H7  GU1"}=" H72 GU1"; $resn_exch{"3H7  GU1"}=" H73 GU1";
$resn_exch{"1H8  GU1"}=" H81 GU1"; $resn_exch{"2H8  GU1"}=" H82 GU1"; $resn_exch{"3H8  GU1"}=" H83 GU1";
$resn_exch{"1H7  GU2"}=" H71 GU2"; $resn_exch{"2H7  GU2"}=" H72 GU2"; $resn_exch{"3H7  GU2"}=" H73 GU2";
$resn_exch{"1H8  GU2"}=" H81 GU2"; $resn_exch{"2H8  GU2"}=" H82 GU2"; $resn_exch{"3H8  GU2"}=" H83 GU2";
$resn_exch{"1H6  GU3"}=" H61 GU3"; $resn_exch{"2H6  GU3"}=" H62 GU3"; $resn_exch{"1H8  GU3"}=" H81 GU3";
$resn_exch{"2H8  GU3"}=" H82 GU3"; $resn_exch{"3H8  GU3"}=" H83 GU3"; $resn_exch{"1H7  GU3"}=" H71 GU3";
$resn_exch{"2H7  GU3"}=" H72 GU3"; $resn_exch{"3H7  GU3"}=" H73 GU3"; $resn_exch{"1H6  GU4"}=" H61 GU4";
$resn_exch{"2H6  GU4"}=" H62 GU4"; $resn_exch{"1H7  GU5"}=" H71 GU5"; $resn_exch{"2H7  GU5"}=" H72 GU5";
$resn_exch{"3H7  GU5"}=" H73 GU5"; $resn_exch{"1H8  GU5"}=" H81 GU5"; $resn_exch{"2H8  GU5"}=" H82 GU5";
$resn_exch{"3H8  GU5"}=" H83 GU5"; $resn_exch{"1H6  GU5"}=" H61 GU5"; $resn_exch{"2H6  GU5"}=" H62 GU5";
$resn_exch{"1H6  GU6"}=" H61 GU6"; $resn_exch{"2H6  GU6"}=" H62 GU6"; $resn_exch{"1HN2 GU7"}="HN21 GU7";
$resn_exch{"2HN2 GU7"}="HN22 GU7"; $resn_exch{"1H10 GU7"}="H101 GU7"; $resn_exch{"2H10 GU7"}="H102 GU7";
$resn_exch{"1H11 GU7"}="H111 GU7"; $resn_exch{"2H11 GU7"}="H112 GU7"; $resn_exch{"1H14 GU7"}="H141 GU7";
$resn_exch{"2H14 GU7"}="H142 GU7"; $resn_exch{"1H16 GU7"}="H161 GU7"; $resn_exch{"2H16 GU7"}="H162 GU7";
$resn_exch{"1H9  GU8"}=" H91 GU8"; $resn_exch{"2H9  GU8"}=" H92 GU8"; $resn_exch{"3H9  GU8"}=" H93 GU8";
$resn_exch{"1H6  GU8"}=" H61 GU8"; $resn_exch{"2H6  GU8"}=" H62 GU8"; $resn_exch{"1H7  GU8"}=" H71 GU8";
$resn_exch{"2H7  GU8"}=" H72 GU8"; $resn_exch{"3H7  GU8"}=" H73 GU8"; $resn_exch{"1H8  GU8"}=" H81 GU8";
$resn_exch{"2H8  GU8"}=" H82 GU8"; $resn_exch{"3H8  GU8"}=" H83 GU8"; $resn_exch{"1H6  GU9"}=" H61 GU9";
$resn_exch{"2H6  GU9"}=" H62 GU9"; $resn_exch{"1H9  GU9"}=" H91 GU9"; $resn_exch{"2H9  GU9"}=" H92 GU9";
$resn_exch{"3H9  GU9"}=" H93 GU9"; $resn_exch{"1H8  GU9"}=" H81 GU9"; $resn_exch{"2H8  GU9"}=" H82 GU9";
$resn_exch{"3H8  GU9"}=" H83 GU9"; $resn_exch{"1H7  GU9"}=" H71 GU9"; $resn_exch{"2H7  GU9"}=" H72 GU9";
$resn_exch{"3H7  GU9"}=" H73 GU9"; $resn_exch{"1H2  GUA"}=" H21 GUA"; $resn_exch{"2H2  GUA"}=" H22 GUA";
$resn_exch{"1H3  GUA"}=" H31 GUA"; $resn_exch{"2H3  GUA"}=" H32 GUA"; $resn_exch{"1H4  GUA"}=" H41 GUA";
$resn_exch{"2H4  GUA"}=" H42 GUA"; $resn_exch{" H1* GUD"}=" H1D GUD"; $resn_exch{" H2* GUD"}=" H2D GUD";
$resn_exch{"*HO2 GUD"}="HO2' GUD"; $resn_exch{" H3* GUD"}=" H3D GUD"; $resn_exch{" H4* GUD"}=" H4D GUD";
$resn_exch{"*HO3 GUD"}="HO3' GUD"; $resn_exch{"1H5* GUD"}="H5'1 GUD"; $resn_exch{"2H5* GUD"}="H5'2 GUD";
$resn_exch{"2HOA GUD"}="HOA2 GUD"; $resn_exch{"2HOB GUD"}="HOB2 GUD"; $resn_exch{"1H6' GUD"}="H6'1 GUD";
$resn_exch{"2H6' GUD"}="H6'2 GUD"; $resn_exch{"'HO2 GUD"}="HO2A GUD"; $resn_exch{"'HO3 GUD"}="HO3A GUD";
$resn_exch{"'HO4 GUD"}="HO4' GUD"; $resn_exch{"'HO6 GUD"}="HO6' GUD"; $resn_exch{"1H6  GUL"}=" H61 GUL";
$resn_exch{"2H6  GUL"}=" H62 GUL"; $resn_exch{"1H10 GUM"}="H101 GUM"; $resn_exch{"2H10 GUM"}="H102 GUM";
$resn_exch{"3H10 GUM"}="H103 GUM"; $resn_exch{"1H16 GUM"}="H161 GUM"; $resn_exch{"2H16 GUM"}="H162 GUM";
$resn_exch{"1H18 GUM"}="H181 GUM"; $resn_exch{"2H18 GUM"}="H182 GUM"; $resn_exch{"3H18 GUM"}="H183 GUM";
$resn_exch{"1H24 GUM"}="H241 GUM"; $resn_exch{"2H24 GUM"}="H242 GUM"; $resn_exch{"1H26 GUM"}="H261 GUM";
$resn_exch{"2H26 GUM"}="H262 GUM"; $resn_exch{"3H26 GUM"}="H263 GUM"; $resn_exch{"1HN2 GUN"}="HN21 GUN";
$resn_exch{"2HN2 GUN"}="HN22 GUN"; $resn_exch{"1H6  GUP"}=" H61 GUP"; $resn_exch{"2H6  GUP"}=" H62 GUP";
$resn_exch{"'HN1 GUR"}="HN1' GUR"; $resn_exch{"'HN2 GUR"}="HN2' GUR"; $resn_exch{"1HA  GVA"}=" HA1 GVA";
$resn_exch{"2HA  GVA"}=" HA2 GVA"; $resn_exch{"1HB  GVA"}=" HB1 GVA"; $resn_exch{"2HB  GVA"}=" HB2 GVA";
$resn_exch{"1HG  GVA"}=" HG1 GVA"; $resn_exch{"2HG  GVA"}=" HG2 GVA"; $resn_exch{"1HD  GVA"}=" HD1 GVA";
$resn_exch{"2HD  GVA"}=" HD2 GVA"; $resn_exch{"1HH2 GVA"}="HH21 GVA"; $resn_exch{"2HH2 GVA"}="HH22 GVA";
$resn_exch{"1H36 GVC"}="H361 GVC"; $resn_exch{"2H36 GVC"}="H362 GVC"; $resn_exch{"3H36 GVC"}="H363 GVC";
$resn_exch{"1HAL GVD"}="HAL1 GVD"; $resn_exch{"2HAL GVD"}="HAL2 GVD"; $resn_exch{"1HAM GVD"}="HAM1 GVD";
$resn_exch{"2HAM GVD"}="HAM2 GVD"; $resn_exch{"1HA0 GVD"}="HA01 GVD"; $resn_exch{"2HA0 GVD"}="HA02 GVD";
$resn_exch{"1HN  GVE"}=" HN1 GVE"; $resn_exch{"2HN  GVE"}=" HN2 GVE"; $resn_exch{"1HA  GVE"}=" HA1 GVE";
$resn_exch{"2HA  GVE"}=" HA2 GVE"; $resn_exch{"1HB  GVE"}=" HB1 GVE"; $resn_exch{"2HB  GVE"}=" HB2 GVE";
$resn_exch{"1HG  GVE"}=" HG1 GVE"; $resn_exch{"2HG  GVE"}=" HG2 GVE"; $resn_exch{"1HH3 GVE"}="HH31 GVE";
$resn_exch{"2HH3 GVE"}="HH32 GVE"; $resn_exch{"3HH3 GVE"}="HH33 GVE"; $resn_exch{"1H2C GVG"}="H2C1 GVG";
$resn_exch{"2H2C GVG"}="H2C2 GVG"; $resn_exch{"3H2C GVG"}="H2C3 GVG"; $resn_exch{"1H9C GVI"}="H9C1 GVI";
$resn_exch{"2H9C GVI"}="H9C2 GVI"; $resn_exch{"1H1C GVI"}="H1C1 GVI"; $resn_exch{"2H1C GVI"}="H1C2 GVI";
$resn_exch{"3H1C GVI"}="H1C3 GVI"; $resn_exch{"1H1N GVJ"}="H1N1 GVJ"; $resn_exch{"2H1N GVJ"}="H1N2 GVJ";
$resn_exch{"1H9C GVK"}="H9C1 GVK"; $resn_exch{"2H9C GVK"}="H9C2 GVK"; $resn_exch{"1H15 GVK"}="H151 GVK";
$resn_exch{"2H15 GVK"}="H152 GVK"; $resn_exch{"1H10 GVK"}="H101 GVK"; $resn_exch{"2H10 GVK"}="H102 GVK";
$resn_exch{"1H14 GVK"}="H141 GVK"; $resn_exch{"2H14 GVK"}="H142 GVK"; $resn_exch{"1HN  GVL"}=" HN1 GVL";
$resn_exch{"2HN  GVL"}=" HN2 GVL"; $resn_exch{"1HBC GVL"}="HBC1 GVL"; $resn_exch{"2HBC GVL"}="HBC2 GVL";
$resn_exch{"1H28 GVL"}="H281 GVL"; $resn_exch{"2H28 GVL"}="H282 GVL"; $resn_exch{"1H30 GVL"}="H301 GVL";
$resn_exch{"2H30 GVL"}="H302 GVL"; $resn_exch{"3H30 GVL"}="H303 GVL"; $resn_exch{"1H31 GVL"}="H311 GVL";
$resn_exch{"2H31 GVL"}="H312 GVL"; $resn_exch{"3H31 GVL"}="H313 GVL"; $resn_exch{"1H36 GVL"}="H361 GVL";
$resn_exch{"2H36 GVL"}="H362 GVL"; $resn_exch{"1H2C GVM"}="H2C1 GVM"; $resn_exch{"2H2C GVM"}="H2C2 GVM";
$resn_exch{"1H4C GVM"}="H4C1 GVM"; $resn_exch{"2H4C GVM"}="H4C2 GVM"; $resn_exch{"1H5A GVM"}="H5A1 GVM";
$resn_exch{"2H5A GVM"}="H5A2 GVM"; $resn_exch{"3H5A GVM"}="H5A3 GVM"; $resn_exch{"1H5B GVM"}="H5B1 GVM";
$resn_exch{"2H5B GVM"}="H5B2 GVM"; $resn_exch{"3H5B GVM"}="H5B3 GVM"; $resn_exch{"1H5C GVM"}="H5C1 GVM";
$resn_exch{"2H5C GVM"}="H5C2 GVM"; $resn_exch{"3H5C GVM"}="H5C3 GVM"; $resn_exch{"1H1N GVN"}="H1N1 GVN";
$resn_exch{"2H1N GVN"}="H1N2 GVN"; $resn_exch{"1H2C GVN"}="H2C1 GVN"; $resn_exch{"2H2C GVN"}="H2C2 GVN";
$resn_exch{"1H1N GVO"}="H1N1 GVO"; $resn_exch{"2H1N GVO"}="H1N2 GVO"; $resn_exch{"1H2C GVO"}="H2C1 GVO";
$resn_exch{"2H2C GVO"}="H2C2 GVO"; $resn_exch{"1H9C GVP"}="H9C1 GVP"; $resn_exch{"2H9C GVP"}="H9C2 GVP";
$resn_exch{"1H13 GVP"}="H131 GVP"; $resn_exch{"2H13 GVP"}="H132 GVP"; $resn_exch{"1H10 GVP"}="H101 GVP";
$resn_exch{"2H10 GVP"}="H102 GVP"; $resn_exch{"1H12 GVP"}="H121 GVP"; $resn_exch{"2H12 GVP"}="H122 GVP";
$resn_exch{"1H15 GVQ"}="H151 GVQ"; $resn_exch{"2H15 GVQ"}="H152 GVQ"; $resn_exch{"1H1N GVQ"}="H1N1 GVQ";
$resn_exch{"2H1N GVQ"}="H1N2 GVQ"; $resn_exch{"1H25 GVR"}="H251 GVR"; $resn_exch{"2H25 GVR"}="H252 GVR";
$resn_exch{"1HN7 GW3"}="HN71 GW3"; $resn_exch{"2HN7 GW3"}="HN72 GW3"; $resn_exch{"1HN9 GW3"}="HN91 GW3";
$resn_exch{"2HN9 GW3"}="HN92 GW3"; $resn_exch{"1H27 GW3"}="H271 GW3"; $resn_exch{"2H27 GW3"}="H272 GW3";
$resn_exch{"1H30 GW3"}="H301 GW3"; $resn_exch{"2H30 GW3"}="H302 GW3"; $resn_exch{"3H30 GW3"}="H303 GW3";
$resn_exch{"1H34 GW3"}="H341 GW3"; $resn_exch{"2H34 GW3"}="H342 GW3"; $resn_exch{"1H37 GW3"}="H371 GW3";
$resn_exch{"2H37 GW3"}="H372 GW3"; $resn_exch{"3H37 GW3"}="H373 GW3"; $resn_exch{"1H3  GW4"}=" H31 GW4";
$resn_exch{"2H3  GW4"}=" H32 GW4"; $resn_exch{"3H3  GW4"}=" H33 GW4"; $resn_exch{"1H11 GW4"}="H111 GW4";
$resn_exch{"2H11 GW4"}="H112 GW4"; $resn_exch{"1H12 GW4"}="H121 GW4"; $resn_exch{"2H12 GW4"}="H122 GW4";
$resn_exch{"1H32 GW4"}="H321 GW4"; $resn_exch{"2H32 GW4"}="H322 GW4"; $resn_exch{"1H33 GW4"}="H331 GW4";
$resn_exch{"2H33 GW4"}="H332 GW4"; $resn_exch{"1H35 GW4"}="H351 GW4"; $resn_exch{"2H35 GW4"}="H352 GW4";
$resn_exch{"3H35 GW4"}="H353 GW4"; $resn_exch{"1H2  GW5"}="H21A GW5"; $resn_exch{"2H2  GW5"}="H22A GW5";
$resn_exch{"1H1  GW5"}=" H11 GW5"; $resn_exch{"2H1  GW5"}="H12A GW5"; $resn_exch{"3H1  GW5"}="H13A GW5";
$resn_exch{"1HD  GWB"}=" HD1 GWB"; $resn_exch{"2HD  GWB"}=" HD2 GWB"; $resn_exch{"1HE  GWB"}=" HE1 GWB";
$resn_exch{"2HE  GWB"}=" HE2 GWB"; $resn_exch{"1H12 GWB"}="H121 GWB"; $resn_exch{"2H12 GWB"}="H122 GWB";
$resn_exch{"3H12 GWB"}="H123 GWB"; $resn_exch{"1H13 GWB"}="H131 GWB"; $resn_exch{"2H13 GWB"}="H132 GWB";
$resn_exch{"3H13 GWB"}="H133 GWB"; $resn_exch{"1H6C GXL"}="H6C1 GXL"; $resn_exch{"2H6C GXL"}="H6C2 GXL";
$resn_exch{"1HN  GYC"}=" HN1 GYC"; $resn_exch{"2HN  GYC"}=" HN2 GYC"; $resn_exch{"1HB1 GYC"}="HB11 GYC";
$resn_exch{"2HB1 GYC"}="HB12 GYC"; $resn_exch{"1HA3 GYC"}="HA31 GYC"; $resn_exch{"2HA3 GYC"}="HA32 GYC";
$resn_exch{"1H6C GYP"}="H6C1 GYP"; $resn_exch{"2H6C GYP"}="H6C2 GYP"; $resn_exch{"1H7C GYP"}="H7C1 GYP";
$resn_exch{"2H7C GYP"}="H7C2 GYP"; $resn_exch{"3H7C GYP"}="H7C3 GYP"; $resn_exch{"1HN  GYS"}=" HN1 GYS";
$resn_exch{"2HN  GYS"}=" HN2 GYS"; $resn_exch{"1HB1 GYS"}="HB11 GYS"; $resn_exch{"2HB1 GYS"}="HB12 GYS";
$resn_exch{"1HA3 GYS"}="HA31 GYS"; $resn_exch{"2HA3 GYS"}="HA32 GYS"; $resn_exch{"1HH2 GZZ"}="HH21 GZZ";
$resn_exch{"2HH2 GZZ"}="HH22 GZZ"; $resn_exch{"EHN1 GZZ"}="HN1E GZZ"; $resn_exch{"1HC1 GZZ"}="HC11 GZZ";
$resn_exch{"2HC1 GZZ"}="HC12 GZZ"; $resn_exch{"1HC2 GZZ"}="HC21 GZZ"; $resn_exch{"2HC2 GZZ"}="HC22 GZZ";
$resn_exch{"1HC3 GZZ"}="HC31 GZZ"; $resn_exch{"2HC3 GZZ"}="HC32 GZZ"; $resn_exch{"1HC4 GZZ"}="HC41 GZZ";
$resn_exch{"2HC4 GZZ"}="HC42 GZZ"; $resn_exch{"1HC5 GZZ"}="HC51 GZZ"; $resn_exch{"2HC5 GZZ"}="HC52 GZZ";
$resn_exch{"1HC6 GZZ"}="HC61 GZZ"; $resn_exch{"2HC6 GZZ"}="HC62 GZZ"; $resn_exch{"1HC7 GZZ"}="HC71 GZZ";
$resn_exch{"2HC7 GZZ"}="HC72 GZZ"; $resn_exch{"1HC8 GZZ"}="HC81 GZZ"; $resn_exch{"2HC8 GZZ"}="HC82 GZZ";
$resn_exch{"9HN  GZZ"}=" HN9 GZZ"; $resn_exch{"1H10 GZZ"}="H101 GZZ"; $resn_exch{"2H10 GZZ"}="H102 GZZ";
$resn_exch{"1H11 GZZ"}="H111 GZZ"; $resn_exch{"2H11 GZZ"}="H112 GZZ"; $resn_exch{"1H12 GZZ"}="H121 GZZ";
$resn_exch{"2H12 GZZ"}="H122 GZZ"; $resn_exch{"1H13 GZZ"}="H131 GZZ"; $resn_exch{"2H13 GZZ"}="H132 GZZ";
$resn_exch{"1H14 GZZ"}="H141 GZZ"; $resn_exch{"2H14 GZZ"}="H142 GZZ"; $resn_exch{"1H15 GZZ"}="H151 GZZ";
$resn_exch{"2H15 GZZ"}="H152 GZZ"; $resn_exch{"1H16 GZZ"}="H161 GZZ"; $resn_exch{"2H16 GZZ"}="H162 GZZ";
$resn_exch{"1H17 GZZ"}="H171 GZZ"; $resn_exch{"2H17 GZZ"}="H172 GZZ"; $resn_exch{"EHN2 GZZ"}="HN2E GZZ";
$resn_exch{"1HH3 GZZ"}="HH31 GZZ"; $resn_exch{"2HH3 GZZ"}="HH32 GZZ"; $resn_exch{"1H16 H01"}="H161 H01";
$resn_exch{"2H16 H01"}="H162 H01"; $resn_exch{"3H16 H01"}="H163 H01"; $resn_exch{"1H26 H01"}="H261 H01";
$resn_exch{"2H26 H01"}="H262 H01"; $resn_exch{"3H26 H01"}="H263 H01"; $resn_exch{"1H48 H01"}="H481 H01";
$resn_exch{"2H48 H01"}="H482 H01"; $resn_exch{"1H47 H01"}="H471 H01"; $resn_exch{"2H47 H01"}="H472 H01";
$resn_exch{"1H46 H01"}="H461 H01"; $resn_exch{"2H46 H01"}="H462 H01"; $resn_exch{"3H46 H01"}="H463 H01";
$resn_exch{"1H37 H01"}="H371 H01"; $resn_exch{"2H37 H01"}="H372 H01"; $resn_exch{"1H38 H01"}="H381 H01";
$resn_exch{"2H38 H01"}="H382 H01"; $resn_exch{"1H36 H01"}="H361 H01"; $resn_exch{"2H36 H01"}="H362 H01";
$resn_exch{"3H36 H01"}="H363 H01"; $resn_exch{"1HBA H02"}="HBA1 H02"; $resn_exch{"2HBA H02"}="HBA2 H02";
$resn_exch{"3HBA H02"}="HBA3 H02"; $resn_exch{"1HAA H02"}="HAA1 H02"; $resn_exch{"2HAA H02"}="HAA2 H02";
$resn_exch{"1HMA H02"}="HMA1 H02"; $resn_exch{"2HMA H02"}="HMA2 H02"; $resn_exch{"3HMA H02"}="HMA3 H02";
$resn_exch{"HN_A H02"}=" HNA H02"; $resn_exch{"1HCB H02"}="HCB1 H02"; $resn_exch{"2HCB H02"}="HCB2 H02";
$resn_exch{"3HCB H02"}="HCB3 H02"; $resn_exch{"1HMB H02"}="HMB1 H02"; $resn_exch{"2HMB H02"}="HMB2 H02";
$resn_exch{"3HMB H02"}="HMB3 H02"; $resn_exch{"1HAB H02"}="HAB1 H02"; $resn_exch{"2HAB H02"}="HAB2 H02";
$resn_exch{"1HBB H02"}="HBB1 H02"; $resn_exch{"2HBB H02"}="HBB2 H02"; $resn_exch{"3HBB H02"}="HBB3 H02";
$resn_exch{"HN_C H02"}=" HNC H02"; $resn_exch{"1HMC H02"}="HMC1 H02"; $resn_exch{"2HMC H02"}="HMC2 H02";
$resn_exch{"3HMC H02"}="HMC3 H02"; $resn_exch{"1HAC H02"}="HAC1 H02"; $resn_exch{"2HAC H02"}="HAC2 H02";
$resn_exch{"1HBC H02"}="HBC1 H02"; $resn_exch{"2HBC H02"}="HBC2 H02"; $resn_exch{"HN_D H02"}=" HND H02";
$resn_exch{"1HMD H02"}="HMD1 H02"; $resn_exch{"2HMD H02"}="HMD2 H02"; $resn_exch{"3HMD H02"}="HMD3 H02";
$resn_exch{"1HAD H02"}="HAD1 H02"; $resn_exch{"2HAD H02"}="HAD2 H02"; $resn_exch{"1HBD H02"}="HBD1 H02";
$resn_exch{"2HBD H02"}="HBD2 H02"; $resn_exch{"1HB  H12"}=" HB1 H12"; $resn_exch{"2HB  H12"}=" HB2 H12";
$resn_exch{"1HC  H12"}=" HC1 H12"; $resn_exch{"2HC  H12"}=" HC2 H12"; $resn_exch{"1HD  H12"}=" HD1 H12";
$resn_exch{"2HD  H12"}=" HD2 H12"; $resn_exch{"1HE  H12"}=" HE1 H12"; $resn_exch{"2HE  H12"}=" HE2 H12";
$resn_exch{"1HF  H12"}=" HF1 H12"; $resn_exch{"2HF  H12"}=" HF2 H12"; $resn_exch{"1H11 H12"}="H111 H12";
$resn_exch{"2H11 H12"}="H112 H12"; $resn_exch{"1H12 H12"}="H121 H12"; $resn_exch{"2H12 H12"}="H122 H12";
$resn_exch{"1H13 H12"}="H131 H12"; $resn_exch{"2H13 H12"}="H132 H12"; $resn_exch{"3H13 H12"}="H133 H12";
$resn_exch{"1HB  H16"}=" HB1 H16"; $resn_exch{"2HB  H16"}=" HB2 H16"; $resn_exch{"1HC  H16"}=" HC1 H16";
$resn_exch{"2HC  H16"}=" HC2 H16"; $resn_exch{"1HD  H16"}=" HD1 H16"; $resn_exch{"2HD  H16"}=" HD2 H16";
$resn_exch{"1HE  H16"}=" HE1 H16"; $resn_exch{"2HE  H16"}=" HE2 H16"; $resn_exch{"1HF  H16"}=" HF1 H16";
$resn_exch{"2HF  H16"}=" HF2 H16"; $resn_exch{"1H11 H16"}="H111 H16"; $resn_exch{"2H11 H16"}="H112 H16";
$resn_exch{"1H12 H16"}="H121 H16"; $resn_exch{"2H12 H16"}="H122 H16"; $resn_exch{"1H13 H16"}="H131 H16";
$resn_exch{"2H13 H16"}="H132 H16"; $resn_exch{"3H13 H16"}="H133 H16"; $resn_exch{"1HB  H18"}=" HB1 H18";
$resn_exch{"2HB  H18"}=" HB2 H18"; $resn_exch{"1HC  H18"}=" HC1 H18"; $resn_exch{"2HC  H18"}=" HC2 H18";
$resn_exch{"1HD  H18"}=" HD1 H18"; $resn_exch{"2HD  H18"}=" HD2 H18"; $resn_exch{"1HE  H18"}=" HE1 H18";
$resn_exch{"2HE  H18"}=" HE2 H18"; $resn_exch{"1HF  H18"}=" HF1 H18"; $resn_exch{"2HF  H18"}=" HF2 H18";
$resn_exch{"1H11 H18"}="H111 H18"; $resn_exch{"2H11 H18"}="H112 H18"; $resn_exch{"1H12 H18"}="H121 H18";
$resn_exch{"2H12 H18"}="H122 H18"; $resn_exch{"1H13 H18"}="H131 H18"; $resn_exch{"2H13 H18"}="H132 H18";
$resn_exch{"3H13 H18"}="H133 H18"; $resn_exch{"1HN  H1D"}="HN1A H1D"; $resn_exch{"2HN  H1D"}=" HN2 H1D";
$resn_exch{"1HB  H1D"}=" HB1 H1D"; $resn_exch{"2HB  H1D"}=" HB2 H1D"; $resn_exch{"1HG  H1D"}=" HG1 H1D";
$resn_exch{"2HG  H1D"}=" HG2 H1D"; $resn_exch{"1H5  H1D"}=" H51 H1D"; $resn_exch{"2H5  H1D"}=" H52 H1D";
$resn_exch{"1H2A H1L"}="H2A1 H1L"; $resn_exch{"2H2A H1L"}="H2A2 H1L"; $resn_exch{"3H2A H1L"}="H2A3 H1L";
$resn_exch{"1HAJ H1N"}="HAJ1 H1N"; $resn_exch{"2HAJ H1N"}="HAJ2 H1N"; $resn_exch{"1HAL H1N"}="HAL1 H1N";
$resn_exch{"2HAL H1N"}="HAL2 H1N"; $resn_exch{"1HAW H1N"}="HAW1 H1N"; $resn_exch{"2HAW H1N"}="HAW2 H1N";
$resn_exch{"1HAV H1N"}="HAV1 H1N"; $resn_exch{"2HAV H1N"}="HAV2 H1N"; $resn_exch{"1HAK H1N"}="HAK1 H1N";
$resn_exch{"2HAK H1N"}="HAK2 H1N"; $resn_exch{"1HAH H1N"}="HAH1 H1N"; $resn_exch{"2HAH H1N"}="HAH2 H1N";
$resn_exch{"1HAX H1N"}="HAX1 H1N"; $resn_exch{"2HAX H1N"}="HAX2 H1N"; $resn_exch{"1HAT H1N"}="HAT1 H1N";
$resn_exch{"2HAT H1N"}="HAT2 H1N"; $resn_exch{"3HAT H1N"}="HAT3 H1N"; $resn_exch{"1H6  H1S"}=" H61 H1S";
$resn_exch{"2H6  H1S"}=" H62 H1S"; $resn_exch{"1HB  H20"}=" HB1 H20"; $resn_exch{"2HB  H20"}=" HB2 H20";
$resn_exch{"1HC  H20"}=" HC1 H20"; $resn_exch{"2HC  H20"}=" HC2 H20"; $resn_exch{"1HD  H20"}=" HD1 H20";
$resn_exch{"2HD  H20"}=" HD2 H20"; $resn_exch{"1HE  H20"}=" HE1 H20"; $resn_exch{"2HE  H20"}=" HE2 H20";
$resn_exch{"1HF  H20"}=" HF1 H20"; $resn_exch{"2HF  H20"}=" HF2 H20"; $resn_exch{"1H12 H20"}="H121 H20";
$resn_exch{"2H12 H20"}="H122 H20"; $resn_exch{"3H12 H20"}="H123 H20"; $resn_exch{"1H13 H20"}="H131 H20";
$resn_exch{"2H13 H20"}="H132 H20"; $resn_exch{"3H13 H20"}="H133 H20"; $resn_exch{"1H1C H24"}="H1C1 H24";
$resn_exch{"2H1C H24"}="H1C2 H24"; $resn_exch{"3H1C H24"}="H1C3 H24"; $resn_exch{"1H15 H24"}="H151 H24";
$resn_exch{"2H15 H24"}="H152 H24"; $resn_exch{"3H15 H24"}="H153 H24"; $resn_exch{"1H18 H24"}="H181 H24";
$resn_exch{"2H18 H24"}="H182 H24"; $resn_exch{"1H22 H24"}="H221 H24"; $resn_exch{"2H22 H24"}="H222 H24";
$resn_exch{"3H22 H24"}="H223 H24"; $resn_exch{"1H24 H24"}="H241 H24"; $resn_exch{"2H24 H24"}="H242 H24";
$resn_exch{"1H2  H2A"}="H21A H2A"; $resn_exch{"2H2  H2A"}=" H22 H2A"; $resn_exch{"1H3' H2A"}="H3'1 H2A";
$resn_exch{"2H3' H2A"}="H3'2 H2A"; $resn_exch{"1H13 H2A"}="H131 H2A"; $resn_exch{"2H13 H2A"}="H132 H2A";
$resn_exch{"1H14 H2A"}="H141 H2A"; $resn_exch{"2H14 H2A"}="H142 H2A"; $resn_exch{"1H15 H2A"}="H151 H2A";
$resn_exch{"2H15 H2A"}="H152 H2A"; $resn_exch{"1H16 H2A"}="H161 H2A"; $resn_exch{"2H16 H2A"}="H162 H2A";
$resn_exch{"1H24 H2A"}="H241 H2A"; $resn_exch{"2H24 H2A"}="H242 H2A"; $resn_exch{"1H25 H2A"}="H251 H2A";
$resn_exch{"2H25 H2A"}="H252 H2A"; $resn_exch{"3H25 H2A"}="H253 H2A"; $resn_exch{"1H26 H2A"}="H261 H2A";
$resn_exch{"2H26 H2A"}="H262 H2A"; $resn_exch{"3H26 H2A"}="H263 H2A"; $resn_exch{"2H2  H2B"}=" H22 H2B";
$resn_exch{"1H2  H2B"}=" H21 H2B"; $resn_exch{"1H7  H2B"}=" H71 H2B"; $resn_exch{"2H7  H2B"}=" H72 H2B";
$resn_exch{"1H9O H2B"}="H9O1 H2B"; $resn_exch{"3H11 H2B"}="H113 H2B"; $resn_exch{"2H11 H2B"}="H112 H2B";
$resn_exch{"1H11 H2B"}="H111 H2B"; $resn_exch{"1H1O H2B"}="H1O1 H2B"; $resn_exch{"1H6  H2P"}=" H61 H2P";
$resn_exch{"2H6  H2P"}=" H62 H2P"; $resn_exch{"1H7  H2P"}=" H71 H2P"; $resn_exch{"2H7  H2P"}=" H72 H2P";
$resn_exch{"3H7  H2P"}=" H73 H2P"; $resn_exch{"2HOP H2P"}="HOP2 H2P"; $resn_exch{"3HOP H2P"}="HOP3 H2P";
$resn_exch{"1HS  H2S"}=" HS1 H2S"; $resn_exch{"2HS  H2S"}=" HS2 H2S"; $resn_exch{"2HOP H2U"}="HOP2 H2U";
$resn_exch{"3HOP H2U"}="HOP3 H2U"; $resn_exch{"1H5* H2U"}=" H5' H2U"; $resn_exch{"2H5* H2U"}="H5'' H2U";
$resn_exch{" H4* H2U"}=" H4' H2U"; $resn_exch{" H3* H2U"}=" H3' H2U"; $resn_exch{"*HO3 H2U"}="HO3' H2U";
$resn_exch{" H1* H2U"}=" H1' H2U"; $resn_exch{" H2* H2U"}=" H2' H2U"; $resn_exch{"*HO2 H2U"}="HO2' H2U";
$resn_exch{"1H5  H2U"}=" H51 H2U"; $resn_exch{"2H5  H2U"}=" H52 H2U"; $resn_exch{"1H6  H2U"}=" H61 H2U";
$resn_exch{"2H6  H2U"}=" H62 H2U"; $resn_exch{"1H15 H33"}="H151 H33"; $resn_exch{"2H15 H33"}="H152 H33";
$resn_exch{"3H15 H33"}="H153 H33"; $resn_exch{"1H11 H33"}="H111 H33"; $resn_exch{"2H11 H33"}="H112 H33";
$resn_exch{"3H11 H33"}="H113 H33"; $resn_exch{"1HAP H35"}="HAP1 H35"; $resn_exch{"2HAP H35"}="HAP2 H35";
$resn_exch{"1H6  H3S"}=" H61 H3S"; $resn_exch{"2H6  H3S"}=" H62 H3S"; $resn_exch{"1HN2 H4B"}="HN21 H4B";
$resn_exch{"2HN2 H4B"}="HN22 H4B"; $resn_exch{"1H7  H4B"}=" H71 H4B"; $resn_exch{"2H7  H4B"}=" H72 H4B";
$resn_exch{"1H11 H4B"}="H111 H4B"; $resn_exch{"2H11 H4B"}="H112 H4B"; $resn_exch{"3H11 H4B"}="H113 H4B";
$resn_exch{"1HA2 H4M"}="HA21 H4M"; $resn_exch{"2HA2 H4M"}="HA22 H4M"; $resn_exch{"1H7M H4M"}="H7M1 H4M";
$resn_exch{"2H7M H4M"}="H7M2 H4M"; $resn_exch{"3H7M H4M"}="H7M3 H4M"; $resn_exch{"1H9M H4M"}="H9M1 H4M";
$resn_exch{"2H9M H4M"}="H9M2 H4M"; $resn_exch{"3H9M H4M"}="H9M3 H4M"; $resn_exch{"1H10 H4M"}="H101 H4M";
$resn_exch{"2H10 H4M"}="H102 H4M"; $resn_exch{"1HX1 H4M"}="HX11 H4M"; $resn_exch{"2HX1 H4M"}="HX12 H4M";
$resn_exch{"1HX5 H4M"}="HX51 H4M"; $resn_exch{"2HX5 H4M"}="HX52 H4M"; $resn_exch{"2HOX H4M"}="HOX2 H4M";
$resn_exch{"3HOX H4M"}="HOX3 H4M"; $resn_exch{"4HOX H4M"}="HOX4 H4M"; $resn_exch{"1H5J H4M"}="H5J1 H4M";
$resn_exch{"2H5J H4M"}="H5J2 H4M"; $resn_exch{"3HOJ H4M"}="HOJ3 H4M"; $resn_exch{"2HOJ H4M"}="HOJ2 H4M";
$resn_exch{"1HP3 H4M"}="HP31 H4M"; $resn_exch{"2HP3 H4M"}="HP32 H4M"; $resn_exch{"1HP4 H4M"}="HP41 H4M";
$resn_exch{"2HP4 H4M"}="HP42 H4M"; $resn_exch{"1HA5 H4P"}="HA51 H4P"; $resn_exch{"2HA5 H4P"}="HA52 H4P";
$resn_exch{"1H3  H4P"}=" H31 H4P"; $resn_exch{"2H3  H4P"}=" H32 H4P"; $resn_exch{"1H4  H4P"}=" H41 H4P";
$resn_exch{"2H4  H4P"}=" H42 H4P"; $resn_exch{"1H2  H4Z"}=" H21 H4Z"; $resn_exch{"2H2  H4Z"}=" H22 H4Z";
$resn_exch{"1H3  H4Z"}="H31A H4Z"; $resn_exch{"2H3  H4Z"}=" H32 H4Z"; $resn_exch{"1H5  H4Z"}=" H51 H4Z";
$resn_exch{"2H5  H4Z"}=" H52 H4Z"; $resn_exch{"1H8  H4Z"}=" H81 H4Z"; $resn_exch{"2H8  H4Z"}=" H82 H4Z";
$resn_exch{"1H12 H4Z"}="H121 H4Z"; $resn_exch{"2H12 H4Z"}="H122 H4Z"; $resn_exch{"1H18 H4Z"}="H181 H4Z";
$resn_exch{"2H18 H4Z"}="H182 H4Z"; $resn_exch{"3H18 H4Z"}="H183 H4Z"; $resn_exch{"1HN4 H4Z"}="HN41 H4Z";
$resn_exch{"2HN4 H4Z"}="HN42 H4Z"; $resn_exch{"1H25 H4Z"}="H251 H4Z"; $resn_exch{"2H25 H4Z"}="H252 H4Z";
$resn_exch{"3H25 H4Z"}="H253 H4Z"; $resn_exch{"1H2M H52"}="H2M1 H52"; $resn_exch{"2H2M H52"}="H2M2 H52";
$resn_exch{"3H2M H52"}="H2M3 H52"; $resn_exch{"1H23 H52"}="H231 H52"; $resn_exch{"2H23 H52"}="H232 H52";
$resn_exch{"1H25 H52"}="H251 H52"; $resn_exch{"2H25 H52"}="H252 H52"; $resn_exch{"1H26 H52"}="H261 H52";
$resn_exch{"2H26 H52"}="H262 H52"; $resn_exch{"1H27 H52"}="H271 H52"; $resn_exch{"2H27 H52"}="H272 H52";
$resn_exch{"1HM  H52"}=" HM1 H52"; $resn_exch{"2HM  H52"}=" HM2 H52"; $resn_exch{"3HM  H52"}=" HM3 H52";
$resn_exch{"1H6  H53"}=" H61 H53"; $resn_exch{"2H6  H53"}=" H62 H53"; $resn_exch{" HN  H5M"}=" H   H5M";
$resn_exch{"1HG  H5M"}=" HG2 H5M"; $resn_exch{"2HG  H5M"}=" HG3 H5M"; $resn_exch{"1HD1 H5M"}="HD11 H5M";
$resn_exch{"2HD1 H5M"}="HD12 H5M"; $resn_exch{"3HD1 H5M"}="HD13 H5M"; $resn_exch{"'HO2 H5P"}="HO2' H5P";
$resn_exch{"'HO3 H5P"}="HO3' H5P"; $resn_exch{"1H5' H5P"}="H5'1 H5P"; $resn_exch{"2H5' H5P"}="H5'2 H5P";
$resn_exch{"2HOP H5P"}="HOP2 H5P"; $resn_exch{"3HOP H5P"}="HOP3 H5P"; $resn_exch{"1H97 H64"}="H971 H64";
$resn_exch{"2H97 H64"}="H972 H64"; $resn_exch{"3H97 H64"}="H973 H64"; $resn_exch{"1H96 H64"}="H961 H64";
$resn_exch{"2H96 H64"}="H962 H64"; $resn_exch{"3H96 H64"}="H963 H64"; $resn_exch{"1H93 H64"}="H931 H64";
$resn_exch{"2H93 H64"}="H932 H64"; $resn_exch{"1H92 H64"}="H921 H64"; $resn_exch{"2H92 H64"}="H922 H64";
$resn_exch{"1H91 H64"}="H911 H64"; $resn_exch{"2H91 H64"}="H912 H64"; $resn_exch{"1HN6 H64"}="HN61 H64";
$resn_exch{"2HN6 H64"}="HN62 H64"; $resn_exch{"1H'2 H64"}="H'21 H64"; $resn_exch{"2H'2 H64"}="H'22 H64";
$resn_exch{"1H96 H71"}="H961 H71"; $resn_exch{"2H96 H71"}="H962 H71"; $resn_exch{"3H96 H71"}="H963 H71";
$resn_exch{"1H97 H71"}="H971 H71"; $resn_exch{"2H97 H71"}="H972 H71"; $resn_exch{"3H97 H71"}="H973 H71";
$resn_exch{"1H93 H71"}="H931 H71"; $resn_exch{"2H93 H71"}="H932 H71"; $resn_exch{"1H92 H71"}="H921 H71";
$resn_exch{"2H92 H71"}="H922 H71"; $resn_exch{"1H91 H71"}="H911 H71"; $resn_exch{"2H91 H71"}="H912 H71";
$resn_exch{"1HN6 H71"}="HN61 H71"; $resn_exch{"2HN6 H71"}="HN62 H71"; $resn_exch{"1H'2 H71"}="H'21 H71";
$resn_exch{"2H'2 H71"}="H'22 H71"; $resn_exch{"1H16 H7J"}="H161 H7J"; $resn_exch{"2H16 H7J"}="H162 H7J";
$resn_exch{"3H16 H7J"}="H163 H7J"; $resn_exch{"1H27 H7J"}="H271 H7J"; $resn_exch{"2H27 H7J"}="H272 H7J";
$resn_exch{"3H27 H7J"}="H273 H7J"; $resn_exch{"1H26 H7J"}="H261 H7J"; $resn_exch{"2H26 H7J"}="H262 H7J";
$resn_exch{"3H26 H7J"}="H263 H7J"; $resn_exch{"1H   H7J"}=" H1  H7J"; $resn_exch{"2H   H7J"}=" H2  H7J";
$resn_exch{"3H   H7J"}=" H3  H7J"; $resn_exch{"1H5  H7J"}=" H51 H7J"; $resn_exch{"2H5  H7J"}=" H52 H7J";
$resn_exch{"1H9  H7J"}=" H91 H7J"; $resn_exch{"2H9  H7J"}=" H92 H7J"; $resn_exch{"3H9  H7J"}=" H93 H7J";
$resn_exch{"1H1  H8H"}=" H11 H8H"; $resn_exch{"2H1  H8H"}=" H12 H8H"; $resn_exch{"3H1  H8H"}="H13A H8H";
$resn_exch{"1H3  H8H"}=" H31 H8H"; $resn_exch{"2H3  H8H"}=" H32 H8H"; $resn_exch{"1H4  H8H"}=" H41 H8H";
$resn_exch{"2H4  H8H"}=" H42 H8H"; $resn_exch{"1H6  H8H"}=" H61 H8H"; $resn_exch{"2H6  H8H"}=" H62 H8H";
$resn_exch{"1H7  H8H"}=" H71 H8H"; $resn_exch{"2H7  H8H"}=" H72 H8H"; $resn_exch{"1H14 H8H"}="H141 H8H";
$resn_exch{"2H14 H8H"}="H142 H8H"; $resn_exch{"1H15 H8H"}="H151 H8H"; $resn_exch{"2H15 H8H"}="H152 H8H";
$resn_exch{"1H17 H8H"}="H171 H8H"; $resn_exch{"2H17 H8H"}="H172 H8H"; $resn_exch{"1H18 H8H"}="H181 H8H";
$resn_exch{"2H18 H8H"}="H182 H8H"; $resn_exch{"1H25 H8H"}="H251 H8H"; $resn_exch{"2H25 H8H"}="H252 H8H";
$resn_exch{"1H37 H8H"}="H371 H8H"; $resn_exch{"2H37 H8H"}="H372 H8H"; $resn_exch{"1H38 H8H"}="H381 H8H";
$resn_exch{"2H38 H8H"}="H382 H8H"; $resn_exch{"1HN2 HA1"}="HN21 HA1"; $resn_exch{"2HN2 HA1"}="HN22 HA1";
$resn_exch{"1H10 HA1"}="H101 HA1"; $resn_exch{"2H10 HA1"}="H102 HA1"; $resn_exch{"1H11 HA1"}="H111 HA1";
$resn_exch{"2H11 HA1"}="H112 HA1"; $resn_exch{"1H12 HA1"}="H121 HA1"; $resn_exch{"2H12 HA1"}="H122 HA1";
$resn_exch{"1H13 HA1"}="H131 HA1"; $resn_exch{"2H13 HA1"}="H132 HA1"; $resn_exch{"1H7C HA2"}="H7C1 HA2";
$resn_exch{"2H7C HA2"}="H7C2 HA2"; $resn_exch{"1H14 HA2"}="H141 HA2"; $resn_exch{"2H14 HA2"}="H142 HA2";
$resn_exch{"1H2N HA2"}="H2N1 HA2"; $resn_exch{"2H2N HA2"}="H2N2 HA2"; $resn_exch{"1H16 HA2"}="H161 HA2";
$resn_exch{"2H16 HA2"}="H162 HA2"; $resn_exch{"1H18 HA2"}="H181 HA2"; $resn_exch{"2H18 HA2"}="H182 HA2";
$resn_exch{"1H19 HA2"}="H191 HA2"; $resn_exch{"2H19 HA2"}="H192 HA2"; $resn_exch{"1H20 HA2"}="H201 HA2";
$resn_exch{"2H20 HA2"}="H202 HA2"; $resn_exch{"1H21 HA2"}="H211 HA2"; $resn_exch{"2H21 HA2"}="H212 HA2";
$resn_exch{"1H2  HAA"}=" H21 HAA"; $resn_exch{"2H2  HAA"}=" H22 HAA"; $resn_exch{"'HO4 HAB"}="HO4' HAB";
$resn_exch{"1H2  HAC"}=" H21 HAC"; $resn_exch{"2H2  HAC"}=" H22 HAC"; $resn_exch{"1H3  HAC"}=" H31 HAC";
$resn_exch{"2H3  HAC"}=" H32 HAC"; $resn_exch{"1H4  HAC"}=" H41 HAC"; $resn_exch{"2H4  HAC"}=" H42 HAC";
$resn_exch{"1H5  HAC"}=" H51 HAC"; $resn_exch{"2H5  HAC"}=" H52 HAC"; $resn_exch{"1H6  HAC"}=" H61 HAC";
$resn_exch{"2H6  HAC"}=" H62 HAC"; $resn_exch{"1H7  HAC"}=" H71 HAC"; $resn_exch{"2H7  HAC"}=" H72 HAC";
$resn_exch{"1HN8 HAC"}="HN81 HAC"; $resn_exch{"2HN8 HAC"}="HN82 HAC"; $resn_exch{"1H1  HAD"}=" H11 HAD";
$resn_exch{"2H1  HAD"}=" H12 HAD"; $resn_exch{"1H1  HAE"}=" H11 HAE"; $resn_exch{"2H1  HAE"}=" H12 HAE";
$resn_exch{"3H1  HAE"}=" H13 HAE"; $resn_exch{"1HN1 HAG"}="HN11 HAG"; $resn_exch{"2HN1 HAG"}="HN12 HAG";
$resn_exch{"1HB1 HAG"}="HB11 HAG"; $resn_exch{"2HB1 HAG"}="HB12 HAG"; $resn_exch{"2HG1 HAG"}="HG12 HAG";
$resn_exch{"1HG1 HAG"}="HG11 HAG"; $resn_exch{"1HN2 HAG"}="HN21 HAG"; $resn_exch{"1HB2 HAG"}="HB21 HAG";
$resn_exch{"2HB2 HAG"}="HB22 HAG"; $resn_exch{"1HN3 HAG"}="HN31 HAG"; $resn_exch{"1HA3 HAG"}="HA31 HAG";
$resn_exch{"2HA3 HAG"}="HA32 HAG"; $resn_exch{"1H42 HAG"}="H421 HAG"; $resn_exch{"2H42 HAG"}="H422 HAG";
$resn_exch{"1H45 HAG"}="H451 HAG"; $resn_exch{"2H45 HAG"}="H452 HAG"; $resn_exch{"1H46 HAG"}="H461 HAG";
$resn_exch{"2H46 HAG"}="H462 HAG"; $resn_exch{"1H47 HAG"}="H471 HAG"; $resn_exch{"2H47 HAG"}="H472 HAG";
$resn_exch{"1H48 HAG"}="H481 HAG"; $resn_exch{"2H48 HAG"}="H482 HAG"; $resn_exch{"1H49 HAG"}="H491 HAG";
$resn_exch{"2H49 HAG"}="H492 HAG"; $resn_exch{"3H49 HAG"}="H493 HAG"; $resn_exch{"1H2  HAI"}=" H21 HAI";
$resn_exch{"2H2  HAI"}=" H22 HAI"; $resn_exch{"1H3  HAI"}=" H31 HAI"; $resn_exch{"2H3  HAI"}=" H32 HAI";
$resn_exch{"1H4  HAI"}=" H41 HAI"; $resn_exch{"2H4  HAI"}=" H42 HAI"; $resn_exch{"1H5  HAI"}=" H51 HAI";
$resn_exch{"2H5  HAI"}=" H52 HAI"; $resn_exch{"1H6  HAI"}=" H61 HAI"; $resn_exch{"2H6  HAI"}=" H62 HAI";
$resn_exch{"1HN  HAI"}=" HN1 HAI"; $resn_exch{"2HN  HAI"}=" HN2 HAI"; $resn_exch{"3HN  HAI"}=" HN3 HAI";
$resn_exch{"1HC1 HAL"}="HC11 HAL"; $resn_exch{"2HC1 HAL"}="HC12 HAL"; $resn_exch{"1HC2 HAL"}="HC21 HAL";
$resn_exch{"2HC2 HAL"}="HC22 HAL"; $resn_exch{"1HC4 HAL"}="HC41 HAL"; $resn_exch{"2HC4 HAL"}="HC42 HAL";
$resn_exch{"1HC6 HAL"}="HC61 HAL"; $resn_exch{"2HC6 HAL"}="HC62 HAL"; $resn_exch{"1HC8 HAL"}="HC81 HAL";
$resn_exch{"2HC8 HAL"}="HC82 HAL"; $resn_exch{"1HC9 HAL"}="HC91 HAL"; $resn_exch{"2HC9 HAL"}="HC92 HAL";
$resn_exch{"1H10 HAL"}="H101 HAL"; $resn_exch{"2H10 HAL"}="H102 HAL"; $resn_exch{"1H5* HAM"}="H5'1 HAM";
$resn_exch{"2H5* HAM"}="H5'2 HAM"; $resn_exch{" H4* HAM"}=" H4' HAM"; $resn_exch{" H3* HAM"}=" H3' HAM";
$resn_exch{"*HO3 HAM"}="HO3' HAM"; $resn_exch{" H2* HAM"}=" H2' HAM"; $resn_exch{"*HO2 HAM"}="HO2' HAM";
$resn_exch{" H1* HAM"}=" H1' HAM"; $resn_exch{"1HN6 HAM"}="HN61 HAM"; $resn_exch{"2HN6 HAM"}="HN62 HAM";
$resn_exch{"1HN  HAM"}=" HN1 HAM"; $resn_exch{"2HN  HAM"}=" HN2 HAM"; $resn_exch{"3HN  HAM"}=" HN3 HAM";
$resn_exch{"1HB  HAM"}=" HB1 HAM"; $resn_exch{"2HB  HAM"}=" HB2 HAM"; $resn_exch{"1H10 HAN"}="H101 HAN";
$resn_exch{"2H10 HAN"}="H102 HAN"; $resn_exch{"3H10 HAN"}="H103 HAN"; $resn_exch{"1H11 HAN"}="H111 HAN";
$resn_exch{"2H11 HAN"}="H112 HAN"; $resn_exch{"3H11 HAN"}="H113 HAN"; $resn_exch{"1H12 HAN"}="H121 HAN";
$resn_exch{"2H12 HAN"}="H122 HAN"; $resn_exch{"1H13 HAN"}="H131 HAN"; $resn_exch{"2H13 HAN"}="H132 HAN";
$resn_exch{"1H14 HAN"}="H141 HAN"; $resn_exch{"2H14 HAN"}="H142 HAN"; $resn_exch{"1H15 HAN"}="H151 HAN";
$resn_exch{"2H15 HAN"}="H152 HAN"; $resn_exch{"1H16 HAN"}="H161 HAN"; $resn_exch{"2H16 HAN"}="H162 HAN";
$resn_exch{"1H2  HAP"}=" H21 HAP"; $resn_exch{"2H2  HAP"}=" H22 HAP"; $resn_exch{"1H4  HAP"}=" H41 HAP";
$resn_exch{"2H4  HAP"}=" H42 HAP"; $resn_exch{"1H6  HAP"}=" H61 HAP"; $resn_exch{"2H6  HAP"}=" H62 HAP";
$resn_exch{"3H6  HAP"}=" H63 HAP"; $resn_exch{"1H7  HAP"}=" H71 HAP"; $resn_exch{"2H7  HAP"}=" H72 HAP";
$resn_exch{"3H7  HAP"}=" H73 HAP"; $resn_exch{"1H11 HAP"}="H111 HAP"; $resn_exch{"2H11 HAP"}="H112 HAP";
$resn_exch{"3H11 HAP"}="H113 HAP"; $resn_exch{"1H12 HAP"}="H121 HAP"; $resn_exch{"2H12 HAP"}="H122 HAP";
$resn_exch{"1HN  HAQ"}=" H   HAQ"; $resn_exch{"2HN  HAQ"}=" H2  HAQ"; $resn_exch{"1HA  HAQ"}=" HA1 HAQ";
$resn_exch{"1HB  HAQ"}=" HB2 HAQ"; $resn_exch{"2HB  HAQ"}=" HB3 HAQ"; $resn_exch{"1HG  HAQ"}=" HG2 HAQ";
$resn_exch{"2HG  HAQ"}=" HG3 HAQ"; $resn_exch{"1HA2 HAQ"}=" HA2 HAQ"; $resn_exch{"1HB2 HAQ"}="HB23 HAQ";
$resn_exch{"2HB2 HAQ"}="HB22 HAQ"; $resn_exch{"1HD2 HAQ"}=" HD2 HAQ"; $resn_exch{"1HE2 HAQ"}=" HE2 HAQ";
$resn_exch{"1HZ  HAQ"}=" HZ  HAQ"; $resn_exch{" HN2 HAR"}=" H2  HAR"; $resn_exch{"1HB  HAR"}=" HB2 HAR";
$resn_exch{"2HB  HAR"}=" HB3 HAR"; $resn_exch{"1HG  HAR"}=" HG2 HAR"; $resn_exch{"2HG  HAR"}=" HG3 HAR";
$resn_exch{"1HD  HAR"}=" HD2 HAR"; $resn_exch{"2HD  HAR"}=" HD3 HAR"; $resn_exch{"1HMA HAS"}="HMA1 HAS";
$resn_exch{"2HMA HAS"}="HMA2 HAS"; $resn_exch{"3HMA HAS"}="HMA3 HAS"; $resn_exch{"1HAA HAS"}="HAA1 HAS";
$resn_exch{"2HAA HAS"}="HAA2 HAS"; $resn_exch{"1HBA HAS"}="HBA1 HAS"; $resn_exch{"2HBA HAS"}="HBA2 HAS";
$resn_exch{"1HMB HAS"}="HMB1 HAS"; $resn_exch{"2HMB HAS"}="HMB2 HAS"; $resn_exch{"3HMB HAS"}="HMB3 HAS";
$resn_exch{"1HMC HAS"}="HMC1 HAS"; $resn_exch{"2HMC HAS"}="HMC2 HAS"; $resn_exch{"3HMC HAS"}="HMC3 HAS";
$resn_exch{"1HBC HAS"}="HBC1 HAS"; $resn_exch{"2HBC HAS"}="HBC2 HAS"; $resn_exch{"1HAD HAS"}="HAD1 HAS";
$resn_exch{"2HAD HAS"}="HAD2 HAS"; $resn_exch{"1HBD HAS"}="HBD1 HAS"; $resn_exch{"2HBD HAS"}="HBD2 HAS";
$resn_exch{"1H12 HAS"}="H121 HAS"; $resn_exch{"2H12 HAS"}="H122 HAS"; $resn_exch{"1H13 HAS"}="H131 HAS";
$resn_exch{"2H13 HAS"}="H132 HAS"; $resn_exch{"1H16 HAS"}="H161 HAS"; $resn_exch{"2H16 HAS"}="H162 HAS";
$resn_exch{"1H17 HAS"}="H171 HAS"; $resn_exch{"2H17 HAS"}="H172 HAS"; $resn_exch{"1H20 HAS"}="H201 HAS";
$resn_exch{"2H20 HAS"}="H202 HAS"; $resn_exch{"1H21 HAS"}="H211 HAS"; $resn_exch{"2H21 HAS"}="H212 HAS";
$resn_exch{"1H24 HAS"}="H241 HAS"; $resn_exch{"2H24 HAS"}="H242 HAS"; $resn_exch{"1H25 HAS"}="H251 HAS";
$resn_exch{"2H25 HAS"}="H252 HAS"; $resn_exch{"3H25 HAS"}="H253 HAS"; $resn_exch{"1H26 HAS"}="H261 HAS";
$resn_exch{"2H26 HAS"}="H262 HAS"; $resn_exch{"3H26 HAS"}="H263 HAS"; $resn_exch{"1H27 HAS"}="H271 HAS";
$resn_exch{"2H27 HAS"}="H272 HAS"; $resn_exch{"3H27 HAS"}="H273 HAS"; $resn_exch{"1H28 HAS"}="H281 HAS";
$resn_exch{"2H28 HAS"}="H282 HAS"; $resn_exch{"1H31 HAS"}="H311 HAS"; $resn_exch{"2H31 HAS"}="H312 HAS";
$resn_exch{"3H31 HAS"}="H313 HAS"; $resn_exch{"1H32 HAS"}="H321 HAS"; $resn_exch{"2H32 HAS"}="H322 HAS";
$resn_exch{"3H32 HAS"}="H323 HAS"; $resn_exch{"1HNA HAV"}="HNA1 HAV"; $resn_exch{"2HNA HAV"}="HNA2 HAV";
$resn_exch{"1HG1 HAV"}="HG11 HAV"; $resn_exch{"2HG1 HAV"}="HG12 HAV"; $resn_exch{"3HG1 HAV"}="HG13 HAV";
$resn_exch{"1HG2 HAV"}="HG21 HAV"; $resn_exch{"2HG2 HAV"}="HG22 HAV"; $resn_exch{"3HG2 HAV"}="HG23 HAV";
$resn_exch{" AH2 HAX"}=" H2A HAX"; $resn_exch{"AH61 HAX"}="H61A HAX"; $resn_exch{"AH62 HAX"}="H62A HAX";
$resn_exch{" AH8 HAX"}=" H8A HAX"; $resn_exch{"AH1* HAX"}=" H1B HAX"; $resn_exch{"AH2* HAX"}=" H2B HAX";
$resn_exch{"AHO2 HAX"}="HO2A HAX"; $resn_exch{"AH3* HAX"}=" H3B HAX"; $resn_exch{"8HOA HAX"}="HOA8 HAX";
$resn_exch{"9HOA HAX"}="HOA9 HAX"; $resn_exch{"AH4* HAX"}=" H4B HAX"; $resn_exch{"AH51 HAX"}="H51A HAX";
$resn_exch{"AH52 HAX"}="H52A HAX"; $resn_exch{"2HOA HAX"}="HOA2 HAX"; $resn_exch{"5HOA HAX"}="HOA5 HAX";
$resn_exch{"1H12 HAX"}="H121 HAX"; $resn_exch{"2H12 HAX"}="H122 HAX"; $resn_exch{"1H13 HAX"}="H131 HAX";
$resn_exch{"2H13 HAX"}="H132 HAX"; $resn_exch{"3H13 HAX"}="H133 HAX"; $resn_exch{"1H14 HAX"}="H141 HAX";
$resn_exch{"2H14 HAX"}="H142 HAX"; $resn_exch{"3H14 HAX"}="H143 HAX"; $resn_exch{"1H7  HAX"}=" H71 HAX";
$resn_exch{"2H7  HAX"}=" H72 HAX"; $resn_exch{"1H6  HAX"}=" H61 HAX"; $resn_exch{"2H6  HAX"}=" H62 HAX";
$resn_exch{"1H3  HAX"}=" H31 HAX"; $resn_exch{"2H3  HAX"}=" H32 HAX"; $resn_exch{"1H2  HAX"}=" H21 HAX";
$resn_exch{"2H2  HAX"}=" H22 HAX"; $resn_exch{"1H1  HAX"}=" H11 HAX"; $resn_exch{"2H1  HAX"}=" H12 HAX";
$resn_exch{"1H22 HAZ"}="H221 HAZ"; $resn_exch{"2H22 HAZ"}="H222 HAZ"; $resn_exch{"1H23 HAZ"}="H231 HAZ";
$resn_exch{"2H23 HAZ"}="H232 HAZ"; $resn_exch{"1H24 HAZ"}="H241 HAZ"; $resn_exch{"2H24 HAZ"}="H242 HAZ";
$resn_exch{"1H1  HAZ"}=" H11 HAZ"; $resn_exch{"2H1  HAZ"}=" H12 HAZ"; $resn_exch{"1H11 HAZ"}="H111 HAZ";
$resn_exch{"2H11 HAZ"}="H112 HAZ"; $resn_exch{"3H11 HAZ"}="H113 HAZ"; $resn_exch{"1H12 HAZ"}="H121 HAZ";
$resn_exch{"2H12 HAZ"}="H122 HAZ"; $resn_exch{"3H12 HAZ"}="H123 HAZ"; $resn_exch{"1H2  HAZ"}=" H21 HAZ";
$resn_exch{"2H2  HAZ"}=" H22 HAZ"; $resn_exch{"1H3  HAZ"}=" H31 HAZ"; $resn_exch{"2H3  HAZ"}=" H32 HAZ";
$resn_exch{"1H13 HAZ"}="H131 HAZ"; $resn_exch{"2H13 HAZ"}="H132 HAZ"; $resn_exch{"1H6  HAZ"}=" H61 HAZ";
$resn_exch{"2H6  HAZ"}=" H62 HAZ"; $resn_exch{"1H7  HAZ"}=" H71 HAZ"; $resn_exch{"2H7  HAZ"}=" H72 HAZ";
$resn_exch{"1H6C HB1"}="H6C1 HB1"; $resn_exch{"1H7C HB1"}="H7C1 HB1"; $resn_exch{"1H8C HB1"}="H8C1 HB1";
$resn_exch{"1H9C HB1"}="H9C1 HB1"; $resn_exch{"1H2  HBB"}=" H21 HBB"; $resn_exch{"2H2  HBB"}="H22A HBB";
$resn_exch{"1H10 HBB"}="H101 HBB"; $resn_exch{"2H10 HBB"}="H102 HBB"; $resn_exch{"1H11 HBB"}="H111 HBB";
$resn_exch{"2H11 HBB"}="H112 HBB"; $resn_exch{"1H12 HBB"}="H121 HBB"; $resn_exch{"2H12 HBB"}="H122 HBB";
$resn_exch{"1H19 HBB"}="H191 HBB"; $resn_exch{"2H19 HBB"}="H192 HBB"; $resn_exch{"3H19 HBB"}="H193 HBB";
$resn_exch{"1H27 HBB"}="H271 HBB"; $resn_exch{"2H27 HBB"}="H272 HBB"; $resn_exch{"1H30 HBB"}="H301 HBB";
$resn_exch{"2H30 HBB"}="H302 HBB"; $resn_exch{"3H30 HBB"}="H303 HBB"; $resn_exch{"1H31 HBB"}="H311 HBB";
$resn_exch{"2H31 HBB"}="H312 HBB"; $resn_exch{"3H31 HBB"}="H313 HBB"; $resn_exch{"1H18 HBB"}="H181 HBB";
$resn_exch{"2H18 HBB"}="H182 HBB"; $resn_exch{"3H18 HBB"}="H183 HBB"; $resn_exch{"1H25 HBB"}="H251 HBB";
$resn_exch{"2H25 HBB"}="H252 HBB"; $resn_exch{"1H28 HBB"}="H281 HBB"; $resn_exch{"2H28 HBB"}="H282 HBB";
$resn_exch{"1H2  HBC"}=" H21 HBC"; $resn_exch{"1H3  HBC"}=" H31 HBC"; $resn_exch{"1H4  HBC"}=" H41 HBC";
$resn_exch{"1H5  HBC"}=" H51 HBC"; $resn_exch{"1H6  HBC"}=" H61 HBC"; $resn_exch{"1H7  HBC"}=" H71 HBC";
$resn_exch{"1H8  HBC"}=" H81 HBC"; $resn_exch{"2H8  HBC"}=" H82 HBC"; $resn_exch{"1H9  HBC"}=" H91 HBC";
$resn_exch{"2H9  HBC"}=" H92 HBC"; $resn_exch{"1H11 HBC"}="H111 HBC"; $resn_exch{"2H11 HBC"}="H112 HBC";
$resn_exch{"1HN1 HBC"}="HN11 HBC"; $resn_exch{"2HN1 HBC"}="HN12 HBC"; $resn_exch{"1HN' HBD"}="HN'1 HBD";
$resn_exch{"2HN' HBD"}="HN'2 HBD"; $resn_exch{"1H2  HBH"}=" H21 HBH"; $resn_exch{"2H2  HBH"}="H22A HBH";
$resn_exch{"1H10 HBH"}="H101 HBH"; $resn_exch{"2H10 HBH"}="H102 HBH"; $resn_exch{"1H11 HBH"}="H111 HBH";
$resn_exch{"2H11 HBH"}="H112 HBH"; $resn_exch{"1H12 HBH"}="H121 HBH"; $resn_exch{"2H12 HBH"}="H122 HBH";
$resn_exch{"1H27 HBH"}="H271 HBH"; $resn_exch{"2H27 HBH"}="H272 HBH"; $resn_exch{"1H30 HBH"}="H301 HBH";
$resn_exch{"2H30 HBH"}="H302 HBH"; $resn_exch{"3H30 HBH"}="H303 HBH"; $resn_exch{"1H31 HBH"}="H311 HBH";
$resn_exch{"2H31 HBH"}="H312 HBH"; $resn_exch{"3H31 HBH"}="H313 HBH"; $resn_exch{"1H17 HBH"}="H171 HBH";
$resn_exch{"2H17 HBH"}="H172 HBH"; $resn_exch{"1H19 HBH"}="H191 HBH"; $resn_exch{"2H19 HBH"}="H192 HBH";
$resn_exch{"1H25 HBH"}="H251 HBH"; $resn_exch{"2H25 HBH"}="H252 HBH"; $resn_exch{"1H28 HBH"}="H281 HBH";
$resn_exch{"2H28 HBH"}="H282 HBH"; $resn_exch{"1HN2 HBI"}="HN21 HBI"; $resn_exch{"2HN2 HBI"}="HN22 HBI";
$resn_exch{"1H7  HBI"}=" H71 HBI"; $resn_exch{"2H7  HBI"}=" H72 HBI"; $resn_exch{"1H11 HBI"}="H111 HBI";
$resn_exch{"2H11 HBI"}="H112 HBI"; $resn_exch{"3H11 HBI"}="H113 HBI"; $resn_exch{"1HN2 HBL"}="HN21 HBL";
$resn_exch{"2HN2 HBL"}="HN22 HBL"; $resn_exch{"1HC7 HBL"}="HC71 HBL"; $resn_exch{"2HC7 HBL"}="HC72 HBL";
$resn_exch{"1H10 HBL"}="H101 HBL"; $resn_exch{"2H10 HBL"}="H102 HBL"; $resn_exch{"3H10 HBL"}="H103 HBL";
$resn_exch{"OH10 HBL"}="H10O HBL"; $resn_exch{"1H20 HBM"}="H201 HBM"; $resn_exch{"2H20 HBM"}="H202 HBM";
$resn_exch{"1H21 HBM"}="H211 HBM"; $resn_exch{"2H21 HBM"}="H212 HBM"; $resn_exch{"1H30 HBM"}="H301 HBM";
$resn_exch{"2H30 HBM"}="H302 HBM"; $resn_exch{"3H30 HBM"}="H303 HBM"; $resn_exch{" H2  HBN"}=" HC2 HBN";
$resn_exch{"1HN  HBN"}=" H   HBN"; $resn_exch{"2HN  HBN"}=" H2  HBN"; $resn_exch{"1HB  HBN"}=" HB2 HBN";
$resn_exch{"2HB  HBN"}=" HB3 HBN"; $resn_exch{"3H9B HBO"}="H9B3 HBO"; $resn_exch{"2H9B HBO"}="H9B2 HBO";
$resn_exch{"1H9B HBO"}="H9B1 HBO"; $resn_exch{"AH10 HBP"}="H10A HBP"; $resn_exch{"1H9  HBQ"}=" H91 HBQ";
$resn_exch{"2H9  HBQ"}=" H92 HBQ"; $resn_exch{"1H10 HBQ"}="H101 HBQ"; $resn_exch{"2H10 HBQ"}="H102 HBQ";
$resn_exch{"3H10 HBQ"}="H103 HBQ"; $resn_exch{"1H13 HBQ"}="H131 HBQ"; $resn_exch{"2H13 HBQ"}="H132 HBQ";
$resn_exch{"3H13 HBQ"}="H133 HBQ"; $resn_exch{"1H14 HBQ"}="H141 HBQ"; $resn_exch{"2H14 HBQ"}="H142 HBQ";
$resn_exch{"3H14 HBQ"}="H143 HBQ"; $resn_exch{"1H4  HBR"}=" H41 HBR"; $resn_exch{"2H4  HBR"}=" H42 HBR";
$resn_exch{"3H4  HBR"}=" H43 HBR"; $resn_exch{"1H1  HBR"}=" H11 HBR"; $resn_exch{"2H1  HBR"}=" H12 HBR";
$resn_exch{"3H1  HBR"}=" H13 HBR"; $resn_exch{"1H4  HBS"}=" H41 HBS"; $resn_exch{"2H4  HBS"}=" H42 HBS";
$resn_exch{"3H4  HBS"}=" H43 HBS"; $resn_exch{"1H1  HBS"}=" H11 HBS"; $resn_exch{"2H1  HBS"}=" H12 HBS";
$resn_exch{"3H1  HBS"}=" H13 HBS"; $resn_exch{"1HO1 HBU"}="HO11 HBU"; $resn_exch{"1HC4 HBU"}="HC41 HBU";
$resn_exch{"2HC4 HBU"}="HC42 HBU"; $resn_exch{"1HC9 HBU"}="HC91 HBU"; $resn_exch{"2HC9 HBU"}="HC92 HBU";
$resn_exch{"1H14 HBU"}="H141 HBU"; $resn_exch{"1H15 HBU"}="H151 HBU"; $resn_exch{"2H15 HBU"}="H152 HBU";
$resn_exch{"3H15 HBU"}="H153 HBU"; $resn_exch{"1H12 HBY"}="H121 HBY"; $resn_exch{"2H12 HBY"}="H122 HBY";
$resn_exch{"3H12 HBY"}="H123 HBY"; $resn_exch{"1H11 HBY"}="H111 HBY"; $resn_exch{"2H11 HBY"}="H112 HBY";
$resn_exch{"3H11 HBY"}="H113 HBY"; $resn_exch{"1H13 HBY"}="H131 HBY"; $resn_exch{"2H13 HBY"}="H132 HBY";
$resn_exch{"1H14 HBY"}="H141 HBY"; $resn_exch{"2H14 HBY"}="H142 HBY"; $resn_exch{"3H14 HBY"}="H143 HBY";
$resn_exch{"1H15 HBY"}="H151 HBY"; $resn_exch{"2H15 HBY"}="H152 HBY"; $resn_exch{"3H15 HBY"}="H153 HBY";
$resn_exch{"1HO1 HC0"}="HO11 HC0"; $resn_exch{"2HO1 HC0"}="HO12 HC0"; $resn_exch{"3H3  HC0"}=" H33 HC0";
$resn_exch{"4H4  HC0"}=" H44 HC0"; $resn_exch{"6H6  HC0"}=" H66 HC0"; $resn_exch{"7H7  HC0"}=" H77 HC0";
$resn_exch{"8H8  HC0"}=" H88 HC0"; $resn_exch{"1HO1 HC1"}="HO11 HC1"; $resn_exch{"2HO1 HC1"}="HO12 HC1";
$resn_exch{"1HO2 HC1"}="HO21 HC1"; $resn_exch{"2HO2 HC1"}="HO22 HC1"; $resn_exch{"3H3  HC1"}=" H33 HC1";
$resn_exch{"4H4  HC1"}=" H44 HC1"; $resn_exch{"6H6  HC1"}=" H66 HC1"; $resn_exch{"7H7  HC1"}=" H77 HC1";
$resn_exch{"1H1  HC2"}=" H11 HC2"; $resn_exch{"2H1  HC2"}=" H12 HC2"; $resn_exch{"1H2  HC2"}=" H21 HC2";
$resn_exch{"2H2  HC2"}=" H22 HC2"; $resn_exch{"1H4  HC2"}=" H41 HC2"; $resn_exch{"2H4  HC2"}=" H42 HC2";
$resn_exch{"1H7  HC2"}=" H71 HC2"; $resn_exch{"2H7  HC2"}=" H72 HC2"; $resn_exch{"1H11 HC2"}="H111 HC2";
$resn_exch{"2H11 HC2"}="H112 HC2"; $resn_exch{"1H12 HC2"}="H121 HC2"; $resn_exch{"2H12 HC2"}="H122 HC2";
$resn_exch{"1H15 HC2"}="H151 HC2"; $resn_exch{"2H15 HC2"}="H152 HC2"; $resn_exch{"1H16 HC2"}="H161 HC2";
$resn_exch{"2H16 HC2"}="H162 HC2"; $resn_exch{"1H18 HC2"}="H181 HC2"; $resn_exch{"2H18 HC2"}="H182 HC2";
$resn_exch{"3H18 HC2"}="H183 HC2"; $resn_exch{"1H19 HC2"}="H191 HC2"; $resn_exch{"2H19 HC2"}="H192 HC2";
$resn_exch{"3H19 HC2"}="H193 HC2"; $resn_exch{"1H21 HC2"}="H211 HC2"; $resn_exch{"2H21 HC2"}="H212 HC2";
$resn_exch{"3H21 HC2"}="H213 HC2"; $resn_exch{"1H22 HC2"}="H221 HC2"; $resn_exch{"2H22 HC2"}="H222 HC2";
$resn_exch{"1H23 HC2"}="H231 HC2"; $resn_exch{"2H23 HC2"}="H232 HC2"; $resn_exch{"1H24 HC2"}="H241 HC2";
$resn_exch{"2H24 HC2"}="H242 HC2"; $resn_exch{"1H26 HC2"}="H261 HC2"; $resn_exch{"2H26 HC2"}="H262 HC2";
$resn_exch{"3H26 HC2"}="H263 HC2"; $resn_exch{"1H27 HC2"}="H271 HC2"; $resn_exch{"2H27 HC2"}="H272 HC2";
$resn_exch{"3H27 HC2"}="H273 HC2"; $resn_exch{"1H18 HC3"}="H181 HC3"; $resn_exch{"2H18 HC3"}="H182 HC3";
$resn_exch{"3H18 HC3"}="H183 HC3"; $resn_exch{"1H12 HC3"}="H121 HC3"; $resn_exch{"2H12 HC3"}="H122 HC3";
$resn_exch{"1H11 HC3"}="H111 HC3"; $resn_exch{"2H11 HC3"}="H112 HC3"; $resn_exch{"1H19 HC3"}="H191 HC3";
$resn_exch{"2H19 HC3"}="H192 HC3"; $resn_exch{"3H19 HC3"}="H193 HC3"; $resn_exch{"1H1  HC3"}=" H11 HC3";
$resn_exch{"2H1  HC3"}=" H12 HC3"; $resn_exch{"1H2  HC3"}=" H21 HC3"; $resn_exch{"2H2  HC3"}=" H22 HC3";
$resn_exch{"1H4  HC3"}=" H41 HC3"; $resn_exch{"2H4  HC3"}=" H42 HC3"; $resn_exch{"1H7  HC3"}=" H71 HC3";
$resn_exch{"2H7  HC3"}=" H72 HC3"; $resn_exch{"1H15 HC3"}="H151 HC3"; $resn_exch{"2H15 HC3"}="H152 HC3";
$resn_exch{"1H16 HC3"}="H161 HC3"; $resn_exch{"2H16 HC3"}="H162 HC3"; $resn_exch{"1H21 HC3"}="H211 HC3";
$resn_exch{"2H21 HC3"}="H212 HC3"; $resn_exch{"3H21 HC3"}="H213 HC3"; $resn_exch{"1H22 HC3"}="H221 HC3";
$resn_exch{"2H22 HC3"}="H222 HC3"; $resn_exch{"1H23 HC3"}="H231 HC3"; $resn_exch{"2H23 HC3"}="H232 HC3";
$resn_exch{"1H24 HC3"}="H241 HC3"; $resn_exch{"2H24 HC3"}="H242 HC3"; $resn_exch{"1H26 HC3"}="H261 HC3";
$resn_exch{"2H26 HC3"}="H262 HC3"; $resn_exch{"3H26 HC3"}="H263 HC3"; $resn_exch{"1H27 HC3"}="H271 HC3";
$resn_exch{"2H27 HC3"}="H272 HC3"; $resn_exch{"3H27 HC3"}="H273 HC3"; $resn_exch{"'HO4 HC4"}="HO4' HC4";
$resn_exch{"1HAC HC5"}="HAC1 HC5"; $resn_exch{"2HAC HC5"}="HAC2 HC5"; $resn_exch{"3HAC HC5"}="HAC3 HC5";
$resn_exch{"1HAD HC5"}="HAD1 HC5"; $resn_exch{"2HAD HC5"}="HAD2 HC5"; $resn_exch{"1HAE HC5"}="HAE1 HC5";
$resn_exch{"2HAE HC5"}="HAE2 HC5"; $resn_exch{"1HAF HC5"}="HAF1 HC5"; $resn_exch{"2HAF HC5"}="HAF2 HC5";
$resn_exch{"1HAG HC5"}="HAG1 HC5"; $resn_exch{"2HAG HC5"}="HAG2 HC5"; $resn_exch{"1HAQ HC5"}="HAQ1 HC5";
$resn_exch{"2HAQ HC5"}="HAQ2 HC5"; $resn_exch{"1HAO HC5"}="HAO1 HC5"; $resn_exch{"2HAO HC5"}="HAO2 HC5";
$resn_exch{"1HAT HC5"}="HAT1 HC5"; $resn_exch{"2HAT HC5"}="HAT2 HC5"; $resn_exch{"3HAT HC5"}="HAT3 HC5";
$resn_exch{"1HAJ HC5"}="HAJ1 HC5"; $resn_exch{"2HAJ HC5"}="HAJ2 HC5"; $resn_exch{"3HAJ HC5"}="HAJ3 HC5";
$resn_exch{"1HAM HC5"}="HAM1 HC5"; $resn_exch{"2HAM HC5"}="HAM2 HC5"; $resn_exch{"3HAM HC5"}="HAM3 HC5";
$resn_exch{"1H2  HCA"}=" H21 HCA"; $resn_exch{"2H2  HCA"}=" H22 HCA"; $resn_exch{"1H4  HCA"}=" H41 HCA";
$resn_exch{"2H4  HCA"}=" H42 HCA"; $resn_exch{"1H5  HCA"}=" H51 HCA"; $resn_exch{"2H5  HCA"}=" H52 HCA";
$resn_exch{"1H3C HCG"}="H3C1 HCG"; $resn_exch{"2H3C HCG"}="H3C2 HCG"; $resn_exch{"1H4C HCG"}="H4C1 HCG";
$resn_exch{"2H4C HCG"}="H4C2 HCG"; $resn_exch{"1H7C HCG"}="H7C1 HCG"; $resn_exch{"2H7C HCG"}="H7C2 HCG";
$resn_exch{"1H14 HCG"}="H141 HCG"; $resn_exch{"2H14 HCG"}="H142 HCG"; $resn_exch{"1H16 HCG"}="H161 HCG";
$resn_exch{"2H16 HCG"}="H162 HCG"; $resn_exch{"1H2  HCI"}=" H21 HCI"; $resn_exch{"2H2  HCI"}=" H22 HCI";
$resn_exch{"1H3  HCI"}=" H31 HCI"; $resn_exch{"2H3  HCI"}=" H32 HCI"; $resn_exch{"1HMA HCO"}="HMA1 HCO";
$resn_exch{"2HMA HCO"}="HMA2 HCO"; $resn_exch{"3HMA HCO"}="HMA3 HCO"; $resn_exch{"1HAA HCO"}="HAA1 HCO";
$resn_exch{"2HAA HCO"}="HAA2 HCO"; $resn_exch{"1HBA HCO"}="HBA1 HCO"; $resn_exch{"2HBA HCO"}="HBA2 HCO";
$resn_exch{"3H3B HCO"}="H3B3 HCO"; $resn_exch{"1HMB HCO"}="HMB1 HCO"; $resn_exch{"2HMB HCO"}="HMB2 HCO";
$resn_exch{"3HMB HCO"}="HMB3 HCO"; $resn_exch{"1HBB HCO"}="HBB1 HCO"; $resn_exch{"2HBB HCO"}="HBB2 HCO";
$resn_exch{"3H3C HCO"}="H3C3 HCO"; $resn_exch{"1HMC HCO"}="HMC1 HCO"; $resn_exch{"2HMC HCO"}="HMC2 HCO";
$resn_exch{"3HMC HCO"}="HMC3 HCO"; $resn_exch{"1HBC HCO"}="HBC1 HCO"; $resn_exch{"2HBC HCO"}="HBC2 HCO";
$resn_exch{"1HMD HCO"}="HMD1 HCO"; $resn_exch{"2HMD HCO"}="HMD2 HCO"; $resn_exch{"3HMD HCO"}="HMD3 HCO";
$resn_exch{"1HAD HCO"}="HAD1 HCO"; $resn_exch{"2HAD HCO"}="HAD2 HCO"; $resn_exch{"1HBD HCO"}="HBD1 HCO";
$resn_exch{"2HBD HCO"}="HBD2 HCO"; $resn_exch{"1HA1 HCP"}="HA11 HCP"; $resn_exch{"2HA1 HCP"}="HA12 HCP";
$resn_exch{"1HB1 HCP"}="HB11 HCP"; $resn_exch{"2HB1 HCP"}="HB12 HCP"; $resn_exch{"1HB2 HCP"}="HB21 HCP";
$resn_exch{"2HB2 HCP"}="HB22 HCP"; $resn_exch{"1HA2 HCP"}="HA21 HCP"; $resn_exch{"2HA2 HCP"}="HA22 HCP";
$resn_exch{"1HC2 HCP"}="HC21 HCP"; $resn_exch{"2HC2 HCP"}="HC22 HCP"; $resn_exch{"3HC2 HCP"}="HC23 HCP";
$resn_exch{"1HC5 HCP"}="HC51 HCP"; $resn_exch{"2HC5 HCP"}="HC52 HCP"; $resn_exch{"1H18 HCR"}="H181 HCR";
$resn_exch{"2H18 HCR"}="H182 HCR"; $resn_exch{"3H18 HCR"}="H183 HCR"; $resn_exch{"1H12 HCR"}="H121 HCR";
$resn_exch{"2H12 HCR"}="H122 HCR"; $resn_exch{"1H11 HCR"}="H111 HCR"; $resn_exch{"2H11 HCR"}="H112 HCR";
$resn_exch{"1H19 HCR"}="H191 HCR"; $resn_exch{"2H19 HCR"}="H192 HCR"; $resn_exch{"3H19 HCR"}="H193 HCR";
$resn_exch{"1H1  HCR"}=" H11 HCR"; $resn_exch{"2H1  HCR"}=" H12 HCR"; $resn_exch{"1H2  HCR"}=" H21 HCR";
$resn_exch{"2H2  HCR"}=" H22 HCR"; $resn_exch{"1H4  HCR"}=" H41 HCR"; $resn_exch{"2H4  HCR"}=" H42 HCR";
$resn_exch{"1H15 HCR"}="H151 HCR"; $resn_exch{"2H15 HCR"}="H152 HCR"; $resn_exch{"1H16 HCR"}="H161 HCR";
$resn_exch{"2H16 HCR"}="H162 HCR"; $resn_exch{"1H21 HCR"}="H211 HCR"; $resn_exch{"2H21 HCR"}="H212 HCR";
$resn_exch{"3H21 HCR"}="H213 HCR"; $resn_exch{"1H22 HCR"}="H221 HCR"; $resn_exch{"2H22 HCR"}="H222 HCR";
$resn_exch{"1H23 HCR"}="H231 HCR"; $resn_exch{"2H23 HCR"}="H232 HCR"; $resn_exch{"1H24 HCR"}="H241 HCR";
$resn_exch{"2H24 HCR"}="H242 HCR"; $resn_exch{"1H27 HCR"}="H271 HCR"; $resn_exch{"2H27 HCR"}="H272 HCR";
$resn_exch{"3H27 HCR"}="H273 HCR"; $resn_exch{"1H26 HCR"}="H261 HCR"; $resn_exch{"2H26 HCR"}="H262 HCR";
$resn_exch{"3H26 HCR"}="H263 HCR"; $resn_exch{" HN  HCS"}=" H   HCS"; $resn_exch{"2HN  HCS"}=" H2  HCS";
$resn_exch{"1HCB HCS"}=" HB2 HCS"; $resn_exch{"2HCB HCS"}=" HB3 HCS"; $resn_exch{"1HCG HCS"}=" HG2 HCS";
$resn_exch{"2HCG HCS"}=" HG3 HCS"; $resn_exch{" HSD HCS"}=" HD  HCS"; $resn_exch{"1H7  HCX"}=" H71 HCX";
$resn_exch{"2H7  HCX"}=" H72 HCX"; $resn_exch{"1H2* HCX"}="H2'1 HCX"; $resn_exch{"2H2* HCX"}="H2'2 HCX";
$resn_exch{"1H5* HCX"}="H5'1 HCX"; $resn_exch{"2H5* HCX"}="H5'2 HCX"; $resn_exch{" H4* HCX"}=" H4' HCX";
$resn_exch{" H1* HCX"}=" H1' HCX"; $resn_exch{" H3* HCX"}=" H3' HCX"; $resn_exch{"1HB  HDA"}=" HB1 HDA";
$resn_exch{"2HB  HDA"}=" HB2 HDA"; $resn_exch{"1H07 HDB"}="H071 HDB"; $resn_exch{"2H07 HDB"}="H072 HDB";
$resn_exch{"1H11 HDB"}="H111 HDB"; $resn_exch{"2H11 HDB"}="H112 HDB"; $resn_exch{"1H16 HDB"}="H161 HDB";
$resn_exch{"2H16 HDB"}="H162 HDB"; $resn_exch{"1H17 HDB"}="H171 HDB"; $resn_exch{"2H17 HDB"}="H172 HDB";
$resn_exch{"1H7  HDC"}=" H71 HDC"; $resn_exch{"2H7  HDC"}=" H72 HDC"; $resn_exch{"1H6  HDC"}=" H61 HDC";
$resn_exch{"2H6  HDC"}=" H62 HDC"; $resn_exch{"1H3  HDC"}=" H31 HDC"; $resn_exch{"2H3  HDC"}=" H32 HDC";
$resn_exch{"1H2  HDC"}=" H21 HDC"; $resn_exch{"2H2  HDC"}=" H22 HDC"; $resn_exch{"1H2* HDC"}="H2'1 HDC";
$resn_exch{"2H2* HDC"}="H2'2 HDC"; $resn_exch{" H3* HDC"}=" H3' HDC"; $resn_exch{" H2* HDC"}=" H2' HDC";
$resn_exch{"1H4* HDC"}="H4'1 HDC"; $resn_exch{"2H4* HDC"}="H4'2 HDC"; $resn_exch{"1H5* HDC"}="H5'1 HDC";
$resn_exch{"2H5* HDC"}="H5'2 HDC"; $resn_exch{"1H6* HDC"}="H6'1 HDC"; $resn_exch{"2H6* HDC"}="H6'2 HDC";
$resn_exch{"1H7* HDC"}="H7'1 HDC"; $resn_exch{"2H7* HDC"}="H7'2 HDC"; $resn_exch{"1H8* HDC"}="H8'1 HDC";
$resn_exch{"2H8* HDC"}="H8'2 HDC"; $resn_exch{"1H9* HDC"}="H9'1 HDC"; $resn_exch{"2H9* HDC"}="H9'2 HDC";
$resn_exch{"1H10 HDC"}="H101 HDC"; $resn_exch{"2H10 HDC"}="H102 HDC"; $resn_exch{"3H10 HDC"}="H103 HDC";
$resn_exch{"1H13 HDC"}="H131 HDC"; $resn_exch{"2H13 HDC"}="H132 HDC"; $resn_exch{"3H13 HDC"}="H133 HDC";
$resn_exch{"1H14 HDC"}="H141 HDC"; $resn_exch{"2H14 HDC"}="H142 HDC"; $resn_exch{"3H14 HDC"}="H143 HDC";
$resn_exch{"1H12 HDC"}="H121 HDC"; $resn_exch{"2H12 HDC"}="H122 HDC"; $resn_exch{"1H1  HDC"}=" H11 HDC";
$resn_exch{"2H1  HDC"}=" H12 HDC"; $resn_exch{" H4* HDC"}=" H4' HDC"; $resn_exch{" H1* HDC"}=" H1' HDC";
$resn_exch{"1HN6 HDC"}="HN61 HDC"; $resn_exch{"2HN6 HDC"}="HN62 HDC"; $resn_exch{"1HMA HDD"}="HMA1 HDD";
$resn_exch{"2HMA HDD"}="HMA2 HDD"; $resn_exch{"3HMA HDD"}="HMA3 HDD"; $resn_exch{"1HAA HDD"}="HAA1 HDD";
$resn_exch{"2HAA HDD"}="HAA2 HDD"; $resn_exch{"1HBA HDD"}="HBA1 HDD"; $resn_exch{"2HBA HDD"}="HBA2 HDD";
$resn_exch{"1HMB HDD"}="HMB1 HDD"; $resn_exch{"2HMB HDD"}="HMB2 HDD"; $resn_exch{"3HMB HDD"}="HMB3 HDD";
$resn_exch{"1HBB HDD"}="HBB1 HDD"; $resn_exch{"2HBB HDD"}="HBB2 HDD"; $resn_exch{"1HMC HDD"}="HMC1 HDD";
$resn_exch{"2HMC HDD"}="HMC2 HDD"; $resn_exch{"3HMC HDD"}="HMC3 HDD"; $resn_exch{"1HBC HDD"}="HBC1 HDD";
$resn_exch{"2HBC HDD"}="HBC2 HDD"; $resn_exch{"1HMD HDD"}="HMD1 HDD"; $resn_exch{"2HMD HDD"}="HMD2 HDD";
$resn_exch{"3HMD HDD"}="HMD3 HDD"; $resn_exch{"1HAD HDD"}="HAD1 HDD"; $resn_exch{"2HAD HDD"}="HAD2 HDD";
$resn_exch{"1HBD HDD"}="HBD1 HDD"; $resn_exch{"2HBD HDD"}="HBD2 HDD"; $resn_exch{"1H1* HDF"}="H1'1 HDF";
$resn_exch{"2H1* HDF"}="H1'2 HDF"; $resn_exch{" H2* HDF"}=" H2' HDF"; $resn_exch{"*HO2 HDF"}="HO2' HDF";
$resn_exch{" H3* HDF"}=" H3' HDF"; $resn_exch{"*HO3 HDF"}="HO3' HDF"; $resn_exch{" H4* HDF"}=" H4' HDF";
$resn_exch{"*HO4 HDF"}="HO4' HDF"; $resn_exch{"1H5* HDF"}="H5'1 HDF"; $resn_exch{"2H5* HDF"}="H5'2 HDF";
$resn_exch{"*HO5 HDF"}="HO5' HDF"; $resn_exch{"1H12 HDI"}="H121 HDI"; $resn_exch{"2H12 HDI"}="H122 HDI";
$resn_exch{"3H12 HDI"}="H123 HDI"; $resn_exch{"1H10 HDI"}="H101 HDI"; $resn_exch{"2H10 HDI"}="H102 HDI";
$resn_exch{"1HC8 HDI"}="HC81 HDI"; $resn_exch{"2HC8 HDI"}="HC82 HDI"; $resn_exch{"2HOP HDP"}="HOP2 HDP";
$resn_exch{"1H5* HDP"}="H5'1 HDP"; $resn_exch{"2H5* HDP"}="H5'' HDP"; $resn_exch{"H3T  HDP"}="HO3' HDP";
$resn_exch{"3HOP HDP"}="HOP3 HDP"; $resn_exch{"1H1  HDS"}=" H11 HDS"; $resn_exch{"2H1  HDS"}=" H12 HDS";
$resn_exch{"1H2  HDS"}=" H21 HDS"; $resn_exch{"2H2  HDS"}=" H22 HDS"; $resn_exch{"1H3  HDS"}=" H31 HDS";
$resn_exch{"2H3  HDS"}=" H32 HDS"; $resn_exch{"1H4  HDS"}=" H41 HDS"; $resn_exch{"2H4  HDS"}=" H42 HDS";
$resn_exch{"1H5  HDS"}=" H51 HDS"; $resn_exch{"2H5  HDS"}=" H52 HDS"; $resn_exch{"1H6  HDS"}=" H61 HDS";
$resn_exch{"2H6  HDS"}=" H62 HDS"; $resn_exch{"1H7  HDS"}=" H71 HDS"; $resn_exch{"2H7  HDS"}=" H72 HDS";
$resn_exch{"1H8  HDS"}=" H81 HDS"; $resn_exch{"2H8  HDS"}=" H82 HDS"; $resn_exch{"1H9  HDS"}=" H91 HDS";
$resn_exch{"2H9  HDS"}=" H92 HDS"; $resn_exch{"1H10 HDS"}="H101 HDS"; $resn_exch{"2H10 HDS"}="H102 HDS";
$resn_exch{"1H11 HDS"}="H111 HDS"; $resn_exch{"2H11 HDS"}="H112 HDS"; $resn_exch{"1H12 HDS"}="H121 HDS";
$resn_exch{"2H12 HDS"}="H122 HDS"; $resn_exch{"1H13 HDS"}="H131 HDS"; $resn_exch{"2H13 HDS"}="H132 HDS";
$resn_exch{"1H14 HDS"}="H141 HDS"; $resn_exch{"2H14 HDS"}="H142 HDS"; $resn_exch{"1H15 HDS"}="H151 HDS";
$resn_exch{"2H15 HDS"}="H152 HDS"; $resn_exch{"1H16 HDS"}="H161 HDS"; $resn_exch{"2H16 HDS"}="H162 HDS";
$resn_exch{"3H16 HDS"}="H163 HDS"; $resn_exch{"3HOS HDS"}="HOS3 HDS"; $resn_exch{"1H24 HDT"}="H241 HDT";
$resn_exch{"2H24 HDT"}="H242 HDT"; $resn_exch{"1H19 HDU"}="H191 HDU"; $resn_exch{"2H19 HDU"}="H192 HDU";
$resn_exch{"3H19 HDU"}="H193 HDU"; $resn_exch{"1H10 HDU"}="H101 HDU"; $resn_exch{"2H10 HDU"}="H102 HDU";
$resn_exch{"3H10 HDU"}="H103 HDU"; $resn_exch{"1H10 HDY"}="H101 HDY"; $resn_exch{"2H10 HDY"}="H102 HDY";
$resn_exch{"3H10 HDY"}="H103 HDY"; $resn_exch{"1H25 HDY"}="H251 HDY"; $resn_exch{"2H25 HDY"}="H252 HDY";
$resn_exch{"1H27 HDY"}="H271 HDY"; $resn_exch{"2H27 HDY"}="H272 HDY"; $resn_exch{"1H30 HDY"}="H301 HDY";
$resn_exch{"2H30 HDY"}="H302 HDY"; $resn_exch{"3H30 HDY"}="H303 HDY"; $resn_exch{"1H29 HDY"}="H291 HDY";
$resn_exch{"2H29 HDY"}="H292 HDY"; $resn_exch{"3H29 HDY"}="H293 HDY"; $resn_exch{"1H3  HE1"}=" H31 HE1";
$resn_exch{"2H3  HE1"}=" H32 HE1"; $resn_exch{"1H4  HE1"}=" H41 HE1"; $resn_exch{"2H4  HE1"}=" H42 HE1";
$resn_exch{"1HAA HE2"}="HAA1 HE2"; $resn_exch{"2HAA HE2"}="HAA2 HE2"; $resn_exch{"3HAA HE2"}="HAA3 HE2";
$resn_exch{"1HAC HE2"}="HAC1 HE2"; $resn_exch{"2HAC HE2"}="HAC2 HE2"; $resn_exch{"1HAE HE2"}="HAE1 HE2";
$resn_exch{"2HAE HE2"}="HAE2 HE2"; $resn_exch{"1HAG HE2"}="HAG1 HE2"; $resn_exch{"2HAG HE2"}="HAG2 HE2";
$resn_exch{"1HAF HE2"}="HAF1 HE2"; $resn_exch{"2HAF HE2"}="HAF2 HE2"; $resn_exch{"1HAD HE2"}="HAD1 HE2";
$resn_exch{"2HAD HE2"}="HAD2 HE2"; $resn_exch{"1H9  HE3"}=" H91 HE3"; $resn_exch{"2H9  HE3"}=" H92 HE3";
$resn_exch{"3H9  HE3"}=" H93 HE3"; $resn_exch{"1HM2 HE3"}="HM21 HE3"; $resn_exch{"2HM2 HE3"}="HM22 HE3";
$resn_exch{"3HM2 HE3"}="HM23 HE3"; $resn_exch{"1H7' HE3"}="H7'1 HE3"; $resn_exch{"2H7' HE3"}="H7'2 HE3";
$resn_exch{"1HM4 HE3"}="HM41 HE3"; $resn_exch{"2HM4 HE3"}="HM42 HE3"; $resn_exch{"3HM4 HE3"}="HM43 HE3";
$resn_exch{"1H6  HE3"}=" H61 HE3"; $resn_exch{"2H6  HE3"}=" H62 HE3"; $resn_exch{"1H7  HE3"}=" H71 HE3";
$resn_exch{"2H7  HE3"}=" H72 HE3"; $resn_exch{"1HAA HE4"}="HAA1 HE4"; $resn_exch{"2HAA HE4"}="HAA2 HE4";
$resn_exch{"3HAA HE4"}="HAA3 HE4"; $resn_exch{"1HAC HE4"}="HAC1 HE4"; $resn_exch{"2HAC HE4"}="HAC2 HE4";
$resn_exch{"1HAE HE4"}="HAE1 HE4"; $resn_exch{"2HAE HE4"}="HAE2 HE4"; $resn_exch{"1HAG HE4"}="HAG1 HE4";
$resn_exch{"2HAG HE4"}="HAG2 HE4"; $resn_exch{"1HAH HE4"}="HAH1 HE4"; $resn_exch{"2HAH HE4"}="HAH2 HE4";
$resn_exch{"1HAF HE4"}="HAF1 HE4"; $resn_exch{"2HAF HE4"}="HAF2 HE4"; $resn_exch{"1HAD HE4"}="HAD1 HE4";
$resn_exch{"2HAD HE4"}="HAD2 HE4"; $resn_exch{"1H40 HE5"}="H401 HE5"; $resn_exch{"2H40 HE5"}="H402 HE5";
$resn_exch{"3H40 HE5"}="H403 HE5"; $resn_exch{"1HBA HE5"}="HBA1 HE5"; $resn_exch{"2HBA HE5"}="HBA2 HE5";
$resn_exch{"1HAA HE5"}="HAA1 HE5"; $resn_exch{"2HAA HE5"}="HAA2 HE5"; $resn_exch{"1HMA HE5"}="HMA1 HE5";
$resn_exch{"2HMA HE5"}="HMA2 HE5"; $resn_exch{"3HMA HE5"}="HMA3 HE5"; $resn_exch{"1H2B HE5"}="H2B1 HE5";
$resn_exch{"2H2B HE5"}="H2B2 HE5"; $resn_exch{"1HAB HE5"}="HAB1 HE5"; $resn_exch{"2HAB HE5"}="HAB2 HE5";
$resn_exch{"3HAB HE5"}="HAB3 HE5"; $resn_exch{"1HAC HE5"}="HAC1 HE5"; $resn_exch{"2HAC HE5"}="HAC2 HE5";
$resn_exch{"3HAC HE5"}="HAC3 HE5"; $resn_exch{"1HMD HE5"}="HMD1 HE5"; $resn_exch{"2HMD HE5"}="HMD2 HE5";
$resn_exch{"3HMD HE5"}="HMD3 HE5"; $resn_exch{"1HAD HE5"}="HAD1 HE5"; $resn_exch{"2HAD HE5"}="HAD2 HE5";
$resn_exch{"1HBD HE5"}="HBD1 HE5"; $resn_exch{"2HBD HE5"}="HBD2 HE5"; $resn_exch{"1H41 HE5"}="H411 HE5";
$resn_exch{"2H41 HE5"}="H412 HE5"; $resn_exch{"3H41 HE5"}="H413 HE5"; $resn_exch{"1HMA HE6"}="HMA1 HE6";
$resn_exch{"2HMA HE6"}="HMA2 HE6"; $resn_exch{"3HMA HE6"}="HMA3 HE6"; $resn_exch{"1HMB HE6"}="HMB1 HE6";
$resn_exch{"2HMB HE6"}="HMB2 HE6"; $resn_exch{"3HMB HE6"}="HMB3 HE6"; $resn_exch{"1HNB HE6"}="HNB1 HE6";
$resn_exch{"2HNB HE6"}="HNB2 HE6"; $resn_exch{"3HNB HE6"}="HNB3 HE6"; $resn_exch{"1HMC HE6"}="HMC1 HE6";
$resn_exch{"2HMC HE6"}="HMC2 HE6"; $resn_exch{"3HMC HE6"}="HMC3 HE6"; $resn_exch{"1HNC HE6"}="HNC1 HE6";
$resn_exch{"2HNC HE6"}="HNC2 HE6"; $resn_exch{"3HNC HE6"}="HNC3 HE6"; $resn_exch{"1HMD HE6"}="HMD1 HE6";
$resn_exch{"2HMD HE6"}="HMD2 HE6"; $resn_exch{"3HMD HE6"}="HMD3 HE6"; $resn_exch{"1HAI HE7"}="HAI1 HE7";
$resn_exch{"2HAI HE7"}="HAI2 HE7"; $resn_exch{"1HAK HE7"}="HAK1 HE7"; $resn_exch{"2HAK HE7"}="HAK2 HE7";
$resn_exch{"1HAA HE7"}="HAA1 HE7"; $resn_exch{"2HAA HE7"}="HAA2 HE7"; $resn_exch{"3HAA HE7"}="HAA3 HE7";
$resn_exch{"1HAM HE7"}="HAM1 HE7"; $resn_exch{"2HAM HE7"}="HAM2 HE7"; $resn_exch{"1HAO HE7"}="HAO1 HE7";
$resn_exch{"2HAO HE7"}="HAO2 HE7"; $resn_exch{"1HAN HE7"}="HAN1 HE7"; $resn_exch{"2HAN HE7"}="HAN2 HE7";
$resn_exch{"1HAP HE7"}="HAP1 HE7"; $resn_exch{"2HAP HE7"}="HAP2 HE7"; $resn_exch{"1HAL HE7"}="HAL1 HE7";
$resn_exch{"2HAL HE7"}="HAL2 HE7"; $resn_exch{"1HAJ HE7"}="HAJ1 HE7"; $resn_exch{"2HAJ HE7"}="HAJ2 HE7";
$resn_exch{"1HAA HEA"}="HAA1 HEA"; $resn_exch{"2HAA HEA"}="HAA2 HEA"; $resn_exch{"1HBA HEA"}="HBA1 HEA";
$resn_exch{"2HBA HEA"}="HBA2 HEA"; $resn_exch{"1HMB HEA"}="HMB1 HEA"; $resn_exch{"2HMB HEA"}="HMB2 HEA";
$resn_exch{"3HMB HEA"}="HMB3 HEA"; $resn_exch{"1HMC HEA"}="HMC1 HEA"; $resn_exch{"2HMC HEA"}="HMC2 HEA";
$resn_exch{"3HMC HEA"}="HMC3 HEA"; $resn_exch{"1HBC HEA"}="HBC1 HEA"; $resn_exch{"2HBC HEA"}="HBC2 HEA";
$resn_exch{"1HMD HEA"}="HMD1 HEA"; $resn_exch{"2HMD HEA"}="HMD2 HEA"; $resn_exch{"3HMD HEA"}="HMD3 HEA";
$resn_exch{"1HAD HEA"}="HAD1 HEA"; $resn_exch{"2HAD HEA"}="HAD2 HEA"; $resn_exch{"1HBD HEA"}="HBD1 HEA";
$resn_exch{"2HBD HEA"}="HBD2 HEA"; $resn_exch{"1H12 HEA"}="H121 HEA"; $resn_exch{"2H12 HEA"}="H122 HEA";
$resn_exch{"1H13 HEA"}="H131 HEA"; $resn_exch{"2H13 HEA"}="H132 HEA"; $resn_exch{"1H16 HEA"}="H161 HEA";
$resn_exch{"2H16 HEA"}="H162 HEA"; $resn_exch{"1H17 HEA"}="H171 HEA"; $resn_exch{"2H17 HEA"}="H172 HEA";
$resn_exch{"1H20 HEA"}="H201 HEA"; $resn_exch{"2H20 HEA"}="H202 HEA"; $resn_exch{"1H21 HEA"}="H211 HEA";
$resn_exch{"2H21 HEA"}="H212 HEA"; $resn_exch{"1H24 HEA"}="H241 HEA"; $resn_exch{"2H24 HEA"}="H242 HEA";
$resn_exch{"3H24 HEA"}="H243 HEA"; $resn_exch{"1H25 HEA"}="H251 HEA"; $resn_exch{"2H25 HEA"}="H252 HEA";
$resn_exch{"3H25 HEA"}="H253 HEA"; $resn_exch{"1H26 HEA"}="H261 HEA"; $resn_exch{"2H26 HEA"}="H262 HEA";
$resn_exch{"3H26 HEA"}="H263 HEA"; $resn_exch{"1H27 HEA"}="H271 HEA"; $resn_exch{"2H27 HEA"}="H272 HEA";
$resn_exch{"3H27 HEA"}="H273 HEA"; $resn_exch{"1HMA HEB"}="HMA1 HEB"; $resn_exch{"2HMA HEB"}="HMA2 HEB";
$resn_exch{"3HMA HEB"}="HMA3 HEB"; $resn_exch{"1HAA HEB"}="HAA1 HEB"; $resn_exch{"2HAA HEB"}="HAA2 HEB";
$resn_exch{"1HBA HEB"}="HBA1 HEB"; $resn_exch{"2HBA HEB"}="HBA2 HEB"; $resn_exch{"1HMB HEB"}="HMB1 HEB";
$resn_exch{"2HMB HEB"}="HMB2 HEB"; $resn_exch{"3HMB HEB"}="HMB3 HEB"; $resn_exch{"2HAB HEB"}="HAB2 HEB";
$resn_exch{"1HBB HEB"}="HBB1 HEB"; $resn_exch{"2HBB HEB"}="HBB2 HEB"; $resn_exch{"3HBB HEB"}="HBB3 HEB";
$resn_exch{"1HMC HEB"}="HMC1 HEB"; $resn_exch{"2HMC HEB"}="HMC2 HEB"; $resn_exch{"3HMC HEB"}="HMC3 HEB";
$resn_exch{"1HBC HEB"}="HBC1 HEB"; $resn_exch{"2HBC HEB"}="HBC2 HEB"; $resn_exch{"1HMD HEB"}="HMD1 HEB";
$resn_exch{"2HMD HEB"}="HMD2 HEB"; $resn_exch{"3HMD HEB"}="HMD3 HEB"; $resn_exch{"1HAD HEB"}="HAD1 HEB";
$resn_exch{"2HAD HEB"}="HAD2 HEB"; $resn_exch{"1HBD HEB"}="HBD1 HEB"; $resn_exch{"2HBD HEB"}="HBD2 HEB";
$resn_exch{"1HMA HEC"}="HMA1 HEC"; $resn_exch{"2HMA HEC"}="HMA2 HEC"; $resn_exch{"3HMA HEC"}="HMA3 HEC";
$resn_exch{"1HAA HEC"}="HAA1 HEC"; $resn_exch{"2HAA HEC"}="HAA2 HEC"; $resn_exch{"1HBA HEC"}="HBA1 HEC";
$resn_exch{"2HBA HEC"}="HBA2 HEC"; $resn_exch{"1HMB HEC"}="HMB1 HEC"; $resn_exch{"2HMB HEC"}="HMB2 HEC";
$resn_exch{"3HMB HEC"}="HMB3 HEC"; $resn_exch{"1HBB HEC"}="HBB1 HEC"; $resn_exch{"2HBB HEC"}="HBB2 HEC";
$resn_exch{"3HBB HEC"}="HBB3 HEC"; $resn_exch{"1HMC HEC"}="HMC1 HEC"; $resn_exch{"2HMC HEC"}="HMC2 HEC";
$resn_exch{"3HMC HEC"}="HMC3 HEC"; $resn_exch{"1HBC HEC"}="HBC1 HEC"; $resn_exch{"2HBC HEC"}="HBC2 HEC";
$resn_exch{"3HBC HEC"}="HBC3 HEC"; $resn_exch{"1HMD HEC"}="HMD1 HEC"; $resn_exch{"2HMD HEC"}="HMD2 HEC";
$resn_exch{"3HMD HEC"}="HMD3 HEC"; $resn_exch{"1HAD HEC"}="HAD1 HEC"; $resn_exch{"2HAD HEC"}="HAD2 HEC";
$resn_exch{"1HBD HEC"}="HBD1 HEC"; $resn_exch{"2HBD HEC"}="HBD2 HEC"; $resn_exch{"1H1  HED"}=" H11 HED";
$resn_exch{"2H1  HED"}=" H12 HED"; $resn_exch{"1H2  HED"}=" H21 HED"; $resn_exch{"2H2  HED"}=" H22 HED";
$resn_exch{"1H5  HED"}=" H51 HED"; $resn_exch{"2H5  HED"}=" H52 HED"; $resn_exch{"1H6  HED"}=" H61 HED";
$resn_exch{"2H6  HED"}=" H62 HED"; $resn_exch{"1H1  HEE"}=" H11 HEE"; $resn_exch{"2H1  HEE"}=" H12 HEE";
$resn_exch{"1H2  HEE"}=" H21 HEE"; $resn_exch{"2H2  HEE"}=" H22 HEE"; $resn_exch{"1H3  HEE"}=" H31 HEE";
$resn_exch{"2H3  HEE"}=" H32 HEE"; $resn_exch{"1H4  HEE"}=" H41 HEE"; $resn_exch{"2H4  HEE"}=" H42 HEE";
$resn_exch{"1H5  HEE"}=" H51 HEE"; $resn_exch{"2H5  HEE"}=" H52 HEE"; $resn_exch{"1H6  HEE"}=" H61 HEE";
$resn_exch{"2H6  HEE"}=" H62 HEE"; $resn_exch{"3H6  HEE"}=" H63 HEE"; $resn_exch{"3HOP HEE"}="HOP3 HEE";
$resn_exch{"1H1' HEE"}="H1'1 HEE"; $resn_exch{"2H1' HEE"}="H1'2 HEE"; $resn_exch{"1H2' HEE"}="H2'1 HEE";
$resn_exch{"2H2' HEE"}="H2'2 HEE"; $resn_exch{"3H2' HEE"}="H2'3 HEE"; $resn_exch{"1HM5 HEF"}="HM51 HEF";
$resn_exch{"2HM5 HEF"}="HM52 HEF"; $resn_exch{"3HM5 HEF"}="HM53 HEF"; $resn_exch{"1HM  HEF"}=" HM1 HEF";
$resn_exch{"2HM  HEF"}=" HM2 HEF"; $resn_exch{"1HE1 HEF"}="HE11 HEF"; $resn_exch{"2HE1 HEF"}="HE12 HEF";
$resn_exch{"1HE2 HEF"}="HE21 HEF"; $resn_exch{"2HE2 HEF"}="HE22 HEF"; $resn_exch{"1HMA HEG"}="HMA1 HEG";
$resn_exch{"2HMA HEG"}="HMA2 HEG"; $resn_exch{"3HMA HEG"}="HMA3 HEG"; $resn_exch{"1HAA HEG"}="HAA1 HEG";
$resn_exch{"2HAA HEG"}="HAA2 HEG"; $resn_exch{"1HBA HEG"}="HBA1 HEG"; $resn_exch{"2HBA HEG"}="HBA2 HEG";
$resn_exch{"1HMB HEG"}="HMB1 HEG"; $resn_exch{"2HMB HEG"}="HMB2 HEG"; $resn_exch{"3HMB HEG"}="HMB3 HEG";
$resn_exch{"1HBB HEG"}="HBB1 HEG"; $resn_exch{"2HBB HEG"}="HBB2 HEG"; $resn_exch{"1HMC HEG"}="HMC1 HEG";
$resn_exch{"2HMC HEG"}="HMC2 HEG"; $resn_exch{"3HMC HEG"}="HMC3 HEG"; $resn_exch{"1HBC HEG"}="HBC1 HEG";
$resn_exch{"2HBC HEG"}="HBC2 HEG"; $resn_exch{"1HMD HEG"}="HMD1 HEG"; $resn_exch{"2HMD HEG"}="HMD2 HEG";
$resn_exch{"3HMD HEG"}="HMD3 HEG"; $resn_exch{"1HAD HEG"}="HAD1 HEG"; $resn_exch{"2HAD HEG"}="HAD2 HEG";
$resn_exch{"1HBD HEG"}="HBD1 HEG"; $resn_exch{"2HBD HEG"}="HBD2 HEG"; $resn_exch{"1HMA HEH"}="HMA1 HEH";
$resn_exch{"2HMA HEH"}="HMA2 HEH"; $resn_exch{"3HMA HEH"}="HMA3 HEH"; $resn_exch{"HE6* HEH"}="HE6' HEH";
$resn_exch{"HE2* HEH"}="HE2' HEH"; $resn_exch{"1HM1 HEH"}="HM11 HEH"; $resn_exch{"2HM1 HEH"}="HM12 HEH";
$resn_exch{"3HM1 HEH"}="HM13 HEH"; $resn_exch{"HE3* HEH"}="HE3' HEH"; $resn_exch{"HE4* HEH"}="HE4' HEH";
$resn_exch{"1HG2 HEH"}="HG21 HEH"; $resn_exch{"2HG2 HEH"}="HG22 HEH"; $resn_exch{"3HG2 HEH"}="HG23 HEH";
$resn_exch{"1HG3 HEH"}="HG31 HEH"; $resn_exch{"2HG3 HEH"}="HG32 HEH"; $resn_exch{"3HG3 HEH"}="HG33 HEH";
$resn_exch{"HE5* HEH"}="HE5' HEH"; $resn_exch{"HF6* HEH"}="HF6' HEH"; $resn_exch{"HF2* HEH"}="HF2' HEH";
$resn_exch{"HF3* HEH"}="HF3' HEH"; $resn_exch{"1HK5 HEH"}="HK51 HEH"; $resn_exch{"2HK5 HEH"}="HK52 HEH";
$resn_exch{"3HK5 HEH"}="HK53 HEH"; $resn_exch{"1HK6 HEH"}="HK61 HEH"; $resn_exch{"2HK6 HEH"}="HK62 HEH";
$resn_exch{"3HK6 HEH"}="HK63 HEH"; $resn_exch{"1HM4 HEH"}="HM41 HEH"; $resn_exch{"2HM4 HEH"}="HM42 HEH";
$resn_exch{"3HM4 HEH"}="HM43 HEH"; $resn_exch{"HF5* HEH"}="HF5' HEH"; $resn_exch{"1HMB HEH"}="HMB1 HEH";
$resn_exch{"2HMB HEH"}="HMB2 HEH"; $resn_exch{"3HMB HEH"}="HMB3 HEH"; $resn_exch{"1HMC HEH"}="HMC1 HEH";
$resn_exch{"2HMC HEH"}="HMC2 HEH"; $resn_exch{"3HMC HEH"}="HMC3 HEH"; $resn_exch{"1H11 HEL"}="H111 HEL";
$resn_exch{"2H11 HEL"}="H112 HEL"; $resn_exch{"1H12 HEL"}="H121 HEL"; $resn_exch{"2H12 HEL"}="H122 HEL";
$resn_exch{"1H13 HEL"}="H131 HEL"; $resn_exch{"2H13 HEL"}="H132 HEL"; $resn_exch{"1H14 HEL"}="H141 HEL";
$resn_exch{"2H14 HEL"}="H142 HEL"; $resn_exch{"1H22 HEL"}="H221 HEL"; $resn_exch{"2H22 HEL"}="H222 HEL";
$resn_exch{"1H23 HEL"}="H231 HEL"; $resn_exch{"2H23 HEL"}="H232 HEL"; $resn_exch{"3H23 HEL"}="H233 HEL";
$resn_exch{"1H24 HEL"}="H241 HEL"; $resn_exch{"2H24 HEL"}="H242 HEL"; $resn_exch{"3H24 HEL"}="H243 HEL";
$resn_exch{"1H25 HEL"}="H251 HEL"; $resn_exch{"2H25 HEL"}="H252 HEL"; $resn_exch{"3H25 HEL"}="H253 HEL";
$resn_exch{"1HMA HEM"}="HMA1 HEM"; $resn_exch{"2HMA HEM"}="HMA2 HEM"; $resn_exch{"3HMA HEM"}="HMA3 HEM";
$resn_exch{"1HAA HEM"}="HAA1 HEM"; $resn_exch{"2HAA HEM"}="HAA2 HEM"; $resn_exch{"1HBA HEM"}="HBA1 HEM";
$resn_exch{"2HBA HEM"}="HBA2 HEM"; $resn_exch{"1HMB HEM"}="HMB1 HEM"; $resn_exch{"2HMB HEM"}="HMB2 HEM";
$resn_exch{"3HMB HEM"}="HMB3 HEM"; $resn_exch{"1HBB HEM"}="HBB1 HEM"; $resn_exch{"2HBB HEM"}="HBB2 HEM";
$resn_exch{"1HMC HEM"}="HMC1 HEM"; $resn_exch{"2HMC HEM"}="HMC2 HEM"; $resn_exch{"3HMC HEM"}="HMC3 HEM";
$resn_exch{"1HBC HEM"}="HBC1 HEM"; $resn_exch{"2HBC HEM"}="HBC2 HEM"; $resn_exch{"1HMD HEM"}="HMD1 HEM";
$resn_exch{"2HMD HEM"}="HMD2 HEM"; $resn_exch{"3HMD HEM"}="HMD3 HEM"; $resn_exch{"1HAD HEM"}="HAD1 HEM";
$resn_exch{"2HAD HEM"}="HAD2 HEM"; $resn_exch{"1HBD HEM"}="HBD1 HEM"; $resn_exch{"2HBD HEM"}="HBD2 HEM";
$resn_exch{"1H2A HEN"}="H2A1 HEN"; $resn_exch{"2H2A HEN"}="H2A2 HEN"; $resn_exch{"3H2A HEN"}="H2A3 HEN";
$resn_exch{"1H4A HEN"}="H4A1 HEN"; $resn_exch{"2H4A HEN"}="H4A2 HEN"; $resn_exch{"1H5A HEN"}="H5A1 HEN";
$resn_exch{"2H5A HEN"}="H5A2 HEN"; $resn_exch{"OHP1 HEN"}="HP1O HEN"; $resn_exch{"OHP3 HEN"}="HP3O HEN";
$resn_exch{"OH2B HEN"}="H2BO HEN"; $resn_exch{"CHBI HEN"}="HBIC HEN"; $resn_exch{"CHGI HEN"}="HGIC HEN";
$resn_exch{"1HCE HEN"}="HCE1 HEN"; $resn_exch{"CHCE HEN"}="HCEC HEN"; $resn_exch{"OHG1 HEN"}="HG1O HEN";
$resn_exch{"OHG2 HEN"}="HG2O HEN"; $resn_exch{"1HA  HEO"}=" HA1 HEO"; $resn_exch{"2HA  HEO"}=" HA2 HEO";
$resn_exch{"3HA  HEO"}=" HA3 HEO"; $resn_exch{"1HAA HEO"}="HAA1 HEO"; $resn_exch{"2HAA HEO"}="HAA2 HEO";
$resn_exch{"1HBA HEO"}="HBA1 HEO"; $resn_exch{"2HBA HEO"}="HBA2 HEO"; $resn_exch{"1HMB HEO"}="HMB1 HEO";
$resn_exch{"2HMB HEO"}="HMB2 HEO"; $resn_exch{"3HMB HEO"}="HMB3 HEO"; $resn_exch{"1HMC HEO"}="HMC1 HEO";
$resn_exch{"2HMC HEO"}="HMC2 HEO"; $resn_exch{"3HMC HEO"}="HMC3 HEO"; $resn_exch{"1HBC HEO"}="HBC1 HEO";
$resn_exch{"2HBC HEO"}="HBC2 HEO"; $resn_exch{"1HMD HEO"}="HMD1 HEO"; $resn_exch{"2HMD HEO"}="HMD2 HEO";
$resn_exch{"3HMD HEO"}="HMD3 HEO"; $resn_exch{"1HAD HEO"}="HAD1 HEO"; $resn_exch{"2HAD HEO"}="HAD2 HEO";
$resn_exch{"1HBD HEO"}="HBD1 HEO"; $resn_exch{"2HBD HEO"}="HBD2 HEO"; $resn_exch{"1H12 HEO"}="H121 HEO";
$resn_exch{"2H12 HEO"}="H122 HEO"; $resn_exch{"1H13 HEO"}="H131 HEO"; $resn_exch{"2H13 HEO"}="H132 HEO";
$resn_exch{"1H16 HEO"}="H161 HEO"; $resn_exch{"2H16 HEO"}="H162 HEO"; $resn_exch{"1H17 HEO"}="H171 HEO";
$resn_exch{"2H17 HEO"}="H172 HEO"; $resn_exch{"1H20 HEO"}="H201 HEO"; $resn_exch{"2H20 HEO"}="H202 HEO";
$resn_exch{"1H21 HEO"}="H211 HEO"; $resn_exch{"2H21 HEO"}="H212 HEO"; $resn_exch{"1H24 HEO"}="H241 HEO";
$resn_exch{"2H24 HEO"}="H242 HEO"; $resn_exch{"3H24 HEO"}="H243 HEO"; $resn_exch{"1H25 HEO"}="H251 HEO";
$resn_exch{"2H25 HEO"}="H252 HEO"; $resn_exch{"3H25 HEO"}="H253 HEO"; $resn_exch{"1H26 HEO"}="H261 HEO";
$resn_exch{"2H26 HEO"}="H262 HEO"; $resn_exch{"3H26 HEO"}="H263 HEO"; $resn_exch{"1H27 HEO"}="H271 HEO";
$resn_exch{"2H27 HEO"}="H272 HEO"; $resn_exch{"3H27 HEO"}="H273 HEO"; $resn_exch{"1H2  HEP"}=" H21 HEP";
$resn_exch{"2H2  HEP"}=" H22 HEP"; $resn_exch{"1H3  HEP"}=" H31 HEP"; $resn_exch{"2H3  HEP"}=" H32 HEP";
$resn_exch{"1H4  HEP"}=" H41 HEP"; $resn_exch{"2H4  HEP"}=" H42 HEP"; $resn_exch{"1H5  HEP"}=" H51 HEP";
$resn_exch{"2H5  HEP"}=" H52 HEP"; $resn_exch{"3H5  HEP"}=" H53 HEP"; $resn_exch{"1H13 HEP"}="H131 HEP";
$resn_exch{"2H13 HEP"}="H132 HEP"; $resn_exch{"1H14 HEP"}="H141 HEP"; $resn_exch{"2H14 HEP"}="H142 HEP";
$resn_exch{"2HOP HEP"}="HOP2 HEP"; $resn_exch{"1H32 HEQ"}="H321 HEQ"; $resn_exch{"2H32 HEQ"}="H322 HEQ";
$resn_exch{"3H32 HEQ"}="H323 HEQ"; $resn_exch{"1H31 HEQ"}="H311 HEQ"; $resn_exch{"2H31 HEQ"}="H312 HEQ";
$resn_exch{"3H31 HEQ"}="H313 HEQ"; $resn_exch{"1HC2 HEQ"}="HC21 HEQ"; $resn_exch{"2HC2 HEQ"}="HC22 HEQ";
$resn_exch{"1HC3 HEQ"}="HC31 HEQ"; $resn_exch{"1HO3 HEQ"}="HO31 HEQ"; $resn_exch{"1HC4 HEQ"}="HC41 HEQ";
$resn_exch{"2HC4 HEQ"}="HC42 HEQ"; $resn_exch{"1H33 HEQ"}="H331 HEQ"; $resn_exch{"2H33 HEQ"}="H332 HEQ";
$resn_exch{"3H33 HEQ"}="H333 HEQ"; $resn_exch{"1HC7 HEQ"}="HC71 HEQ"; $resn_exch{"1HC8 HEQ"}="HC81 HEQ";
$resn_exch{"1H34 HEQ"}="H341 HEQ"; $resn_exch{"2H34 HEQ"}="H342 HEQ"; $resn_exch{"3H34 HEQ"}="H343 HEQ";
$resn_exch{"1H10 HEQ"}="H101 HEQ"; $resn_exch{"1H11 HEQ"}="H111 HEQ"; $resn_exch{"1H12 HEQ"}="H121 HEQ";
$resn_exch{"1H35 HEQ"}="H351 HEQ"; $resn_exch{"2H35 HEQ"}="H352 HEQ"; $resn_exch{"3H35 HEQ"}="H353 HEQ";
$resn_exch{"1H14 HEQ"}="H141 HEQ"; $resn_exch{"1H15 HEQ"}="H151 HEQ"; $resn_exch{"1H16 HEQ"}="H161 HEQ";
$resn_exch{"1H17 HEQ"}="H171 HEQ"; $resn_exch{"1H36 HEQ"}="H361 HEQ"; $resn_exch{"2H36 HEQ"}="H362 HEQ";
$resn_exch{"3H36 HEQ"}="H363 HEQ"; $resn_exch{"1H19 HEQ"}="H191 HEQ"; $resn_exch{"1H20 HEQ"}="H201 HEQ";
$resn_exch{"1H21 HEQ"}="H211 HEQ"; $resn_exch{"1H37 HEQ"}="H371 HEQ"; $resn_exch{"2H37 HEQ"}="H372 HEQ";
$resn_exch{"3H37 HEQ"}="H373 HEQ"; $resn_exch{"1H23 HEQ"}="H231 HEQ"; $resn_exch{"2H24 HEQ"}="H242 HEQ";
$resn_exch{"1H38 HEQ"}="H381 HEQ"; $resn_exch{"2H38 HEQ"}="H382 HEQ"; $resn_exch{"3H38 HEQ"}="H383 HEQ";
$resn_exch{"1H28 HEQ"}="H281 HEQ"; $resn_exch{"2H28 HEQ"}="H282 HEQ"; $resn_exch{"1H29 HEQ"}="H291 HEQ";
$resn_exch{"2H29 HEQ"}="H292 HEQ"; $resn_exch{"1H40 HEQ"}="H401 HEQ"; $resn_exch{"2H40 HEQ"}="H402 HEQ";
$resn_exch{"3H40 HEQ"}="H403 HEQ"; $resn_exch{"1H39 HEQ"}="H391 HEQ"; $resn_exch{"2H39 HEQ"}="H392 HEQ";
$resn_exch{"3H39 HEQ"}="H393 HEQ"; $resn_exch{"1HMA HES"}="HMA1 HES"; $resn_exch{"2HMA HES"}="HMA2 HES";
$resn_exch{"3HMA HES"}="HMA3 HES"; $resn_exch{"1HAA HES"}="HAA1 HES"; $resn_exch{"2HAA HES"}="HAA2 HES";
$resn_exch{"1HBA HES"}="HBA1 HES"; $resn_exch{"2HBA HES"}="HBA2 HES"; $resn_exch{"1HMB HES"}="HMB1 HES";
$resn_exch{"2HMB HES"}="HMB2 HES"; $resn_exch{"3HMB HES"}="HMB3 HES"; $resn_exch{"1HBB HES"}="HBB1 HES";
$resn_exch{"2HBB HES"}="HBB2 HES"; $resn_exch{"3HBB HES"}="HBB3 HES"; $resn_exch{"1HMC HES"}="HMC1 HES";
$resn_exch{"2HMC HES"}="HMC2 HES"; $resn_exch{"3HMC HES"}="HMC3 HES"; $resn_exch{"1HBC HES"}="HBC1 HES";
$resn_exch{"2HBC HES"}="HBC2 HES"; $resn_exch{"3HBC HES"}="HBC3 HES"; $resn_exch{"1HMD HES"}="HMD1 HES";
$resn_exch{"2HMD HES"}="HMD2 HES"; $resn_exch{"3HMD HES"}="HMD3 HES"; $resn_exch{"1HAD HES"}="HAD1 HES";
$resn_exch{"2HAD HES"}="HAD2 HES"; $resn_exch{"1HBD HES"}="HBD1 HES"; $resn_exch{"2HBD HES"}="HBD2 HES";
$resn_exch{"1H1A HET"}="H1A1 HET"; $resn_exch{"2H1A HET"}="H1A2 HET"; $resn_exch{"1HT0 HET"}="HT01 HET";
$resn_exch{"2HT0 HET"}="HT02 HET"; $resn_exch{"1HA2 HET"}="HA21 HET"; $resn_exch{"2HA2 HET"}="HA22 HET";
$resn_exch{"3HA2 HET"}="HA23 HET"; $resn_exch{"1HA3 HET"}="HA31 HET"; $resn_exch{"2HA3 HET"}="HA32 HET";
$resn_exch{"3HA3 HET"}="HA33 HET"; $resn_exch{"1HS2 HET"}="HS21 HET"; $resn_exch{"2HS2 HET"}="HS22 HET";
$resn_exch{"1HS3 HET"}="HS31 HET"; $resn_exch{"2HS3 HET"}="HS32 HET"; $resn_exch{"1HS4 HET"}="HS41 HET";
$resn_exch{"2HS4 HET"}="HS42 HET"; $resn_exch{"1HS5 HET"}="HS51 HET"; $resn_exch{"2HS5 HET"}="HS52 HET";
$resn_exch{"2HOP HEU"}="HOP2 HEU"; $resn_exch{"1H5* HEU"}=" H5' HEU"; $resn_exch{"2H5* HEU"}="H5'' HEU";
$resn_exch{" H4* HEU"}=" H4' HEU"; $resn_exch{" H1* HEU"}=" H1' HEU"; $resn_exch{"1H7  HEU"}=" H71 HEU";
$resn_exch{"2H7  HEU"}=" H72 HEU"; $resn_exch{"1H8  HEU"}=" H81 HEU"; $resn_exch{"2H8  HEU"}=" H82 HEU";
$resn_exch{" H3* HEU"}=" H3' HEU"; $resn_exch{"1H2* HEU"}=" H2' HEU"; $resn_exch{"2H2* HEU"}="H2'' HEU";
$resn_exch{"H3T  HEU"}="HO3' HEU"; $resn_exch{"3HOP HEU"}="HOP3 HEU"; $resn_exch{"1HMA HEV"}="HMA1 HEV";
$resn_exch{"2HMA HEV"}="HMA2 HEV"; $resn_exch{"3HMA HEV"}="HMA3 HEV"; $resn_exch{"1HAA HEV"}="HAA1 HEV";
$resn_exch{"2HAA HEV"}="HAA2 HEV"; $resn_exch{"1HBA HEV"}="HBA1 HEV"; $resn_exch{"2HBA HEV"}="HBA2 HEV";
$resn_exch{"1HM1 HEV"}="HM11 HEV"; $resn_exch{"2HM1 HEV"}="HM12 HEV"; $resn_exch{"1HBB HEV"}="HBB1 HEV";
$resn_exch{"2HBB HEV"}="HBB2 HEV"; $resn_exch{"1HM2 HEV"}="HM21 HEV"; $resn_exch{"2HM2 HEV"}="HM22 HEV";
$resn_exch{"1HBC HEV"}="HBC1 HEV"; $resn_exch{"2HBC HEV"}="HBC2 HEV"; $resn_exch{"1HMD HEV"}="HMD1 HEV";
$resn_exch{"2HMD HEV"}="HMD2 HEV"; $resn_exch{"3HMD HEV"}="HMD3 HEV"; $resn_exch{"1HAD HEV"}="HAD1 HEV";
$resn_exch{"2HAD HEV"}="HAD2 HEV"; $resn_exch{"1HBD HEV"}="HBD1 HEV"; $resn_exch{"2HBD HEV"}="HBD2 HEV";
$resn_exch{"1H1  HEX"}=" H11 HEX"; $resn_exch{"2H1  HEX"}=" H12 HEX"; $resn_exch{"3H1  HEX"}=" H13 HEX";
$resn_exch{"1H2  HEX"}=" H21 HEX"; $resn_exch{"2H2  HEX"}=" H22 HEX"; $resn_exch{"1H3  HEX"}=" H31 HEX";
$resn_exch{"2H3  HEX"}=" H32 HEX"; $resn_exch{"1H4  HEX"}=" H41 HEX"; $resn_exch{"2H4  HEX"}=" H42 HEX";
$resn_exch{"1H5  HEX"}=" H51 HEX"; $resn_exch{"2H5  HEX"}=" H52 HEX"; $resn_exch{"1H6  HEX"}=" H61 HEX";
$resn_exch{"2H6  HEX"}=" H62 HEX"; $resn_exch{"3H6  HEX"}=" H63 HEX"; $resn_exch{"1H2A HEY"}="H2A1 HEY";
$resn_exch{"2H2A HEY"}="H2A2 HEY"; $resn_exch{"3H2A HEY"}="H2A3 HEY"; $resn_exch{"1H4A HEY"}="H4A1 HEY";
$resn_exch{"2H4A HEY"}="H4A2 HEY"; $resn_exch{"1H5A HEY"}="H5A1 HEY"; $resn_exch{"2H5A HEY"}="H5A2 HEY";
$resn_exch{"1HGI HEY"}="HGI1 HEY"; $resn_exch{"2HGI HEY"}="HGI2 HEY"; $resn_exch{"1HEI HEY"}="HEI1 HEY";
$resn_exch{"2HEI HEY"}="HEI2 HEY"; $resn_exch{"1H1  HEZ"}=" H11 HEZ"; $resn_exch{"2H1  HEZ"}=" H12 HEZ";
$resn_exch{"1H2  HEZ"}=" H21 HEZ"; $resn_exch{"2H2  HEZ"}=" H22 HEZ"; $resn_exch{"1H3  HEZ"}=" H31 HEZ";
$resn_exch{"2H3  HEZ"}=" H32 HEZ"; $resn_exch{"1H4  HEZ"}=" H41 HEZ"; $resn_exch{"2H4  HEZ"}=" H42 HEZ";
$resn_exch{"1H5  HEZ"}=" H51 HEZ"; $resn_exch{"2H5  HEZ"}=" H52 HEZ"; $resn_exch{"1H6  HEZ"}=" H61 HEZ";
$resn_exch{"2H6  HEZ"}=" H62 HEZ"; $resn_exch{"1H3  HF1"}=" H31 HF1"; $resn_exch{"2H3  HF1"}=" H32 HF1";
$resn_exch{"1H4  HF1"}=" H41 HF1"; $resn_exch{"2H4  HF1"}=" H42 HF1"; $resn_exch{"1HB3 HF3"}="HB31 HF3";
$resn_exch{"1HB1 HF3"}="HB11 HF3"; $resn_exch{"1HB2 HF3"}="HB21 HF3"; $resn_exch{"1HA2 HF3"}="HA21 HF3";
$resn_exch{"1HA1 HF3"}="HA11 HF3"; $resn_exch{"1HC1 HF3"}="HC11 HF3"; $resn_exch{"1HC2 HF3"}="HC21 HF3";
$resn_exch{"1HA1 HF5"}="HA11 HF5"; $resn_exch{"1HB1 HF5"}="HB11 HF5"; $resn_exch{"1HB2 HF5"}="HB21 HF5";
$resn_exch{"1HB3 HF5"}="HB31 HF5"; $resn_exch{"1HC1 HF5"}="HC11 HF5"; $resn_exch{"1HC2 HF5"}="HC21 HF5";
$resn_exch{"1HD1 HF5"}="HD11 HF5"; $resn_exch{"1HD2 HF5"}="HD21 HF5"; $resn_exch{"1HE1 HF5"}="HE11 HF5";
$resn_exch{"1HE2 HF5"}="HE21 HF5"; $resn_exch{"1HE3 HF5"}="HE31 HF5"; $resn_exch{"1HE4 HF5"}="HE41 HF5";
$resn_exch{"1HB  HFA"}=" HB1 HFA"; $resn_exch{"2HB  HFA"}=" HB2 HFA"; $resn_exch{"1H15 HFL"}="H151 HFL";
$resn_exch{"2H15 HFL"}="H152 HFL"; $resn_exch{"3H15 HFL"}="H153 HFL"; $resn_exch{"1HMA HFM"}="HMA1 HFM";
$resn_exch{"2HMA HFM"}="HMA2 HFM"; $resn_exch{"3HMA HFM"}="HMA3 HFM"; $resn_exch{"1HAA HFM"}="HAA1 HFM";
$resn_exch{"2HAA HFM"}="HAA2 HFM"; $resn_exch{"1HBA HFM"}="HBA1 HFM"; $resn_exch{"2HBA HFM"}="HBA2 HFM";
$resn_exch{"1HMB HFM"}="HMB1 HFM"; $resn_exch{"2HMB HFM"}="HMB2 HFM"; $resn_exch{"3HMB HFM"}="HMB3 HFM";
$resn_exch{"3H3C HFM"}="H3C3 HFM"; $resn_exch{"1HMC HFM"}="HMC1 HFM"; $resn_exch{"2HMC HFM"}="HMC2 HFM";
$resn_exch{"3HMC HFM"}="HMC3 HFM"; $resn_exch{"1HBC HFM"}="HBC1 HFM"; $resn_exch{"2HBC HFM"}="HBC2 HFM";
$resn_exch{"1HMD HFM"}="HMD1 HFM"; $resn_exch{"2HMD HFM"}="HMD2 HFM"; $resn_exch{"3HMD HFM"}="HMD3 HFM";
$resn_exch{"1HAD HFM"}="HAD1 HFM"; $resn_exch{"2HAD HFM"}="HAD2 HFM"; $resn_exch{"1HBD HFM"}="HBD1 HFM";
$resn_exch{"2HBD HFM"}="HBD2 HFM"; $resn_exch{"1H2  HFP"}=" H21 HFP"; $resn_exch{"2H2  HFP"}=" H22 HFP";
$resn_exch{"1H4  HFP"}=" H41 HFP"; $resn_exch{"2H4  HFP"}=" H42 HFP"; $resn_exch{"3H4  HFP"}=" H43 HFP";
$resn_exch{"1H5  HFP"}=" H51 HFP"; $resn_exch{"2H5  HFP"}=" H52 HFP"; $resn_exch{"1H6  HFP"}=" H61 HFP";
$resn_exch{"2H6  HFP"}=" H62 HFP"; $resn_exch{"1H7  HFP"}=" H71 HFP"; $resn_exch{"2H7  HFP"}=" H72 HFP";
$resn_exch{"1H9  HFP"}=" H91 HFP"; $resn_exch{"2H9  HFP"}=" H92 HFP"; $resn_exch{"3H9  HFP"}=" H93 HFP";
$resn_exch{"1H10 HFP"}="H101 HFP"; $resn_exch{"2H10 HFP"}="H102 HFP"; $resn_exch{"1H11 HFP"}="H111 HFP";
$resn_exch{"2H11 HFP"}="H112 HFP"; $resn_exch{"1H12 HFP"}="H121 HFP"; $resn_exch{"2H12 HFP"}="H122 HFP";
$resn_exch{"1H14 HFP"}="H141 HFP"; $resn_exch{"2H14 HFP"}="H142 HFP"; $resn_exch{"3H14 HFP"}="H143 HFP";
$resn_exch{"1H15 HFP"}="H151 HFP"; $resn_exch{"2H15 HFP"}="H152 HFP"; $resn_exch{"3H15 HFP"}="H153 HFP";
$resn_exch{"1H10 HFS"}="H101 HFS"; $resn_exch{"2H10 HFS"}="H102 HFS"; $resn_exch{"1H11 HFS"}="H111 HFS";
$resn_exch{"2H11 HFS"}="H112 HFS"; $resn_exch{"1H12 HFS"}="H121 HFS"; $resn_exch{"2H12 HFS"}="H122 HFS";
$resn_exch{"1H13 HFS"}="H131 HFS"; $resn_exch{"2H13 HFS"}="H132 HFS"; $resn_exch{"1H14 HFS"}="H141 HFS";
$resn_exch{"2H14 HFS"}="H142 HFS"; $resn_exch{"1H12 HFT"}="H121 HFT"; $resn_exch{"2H12 HFT"}="H122 HFT";
$resn_exch{"3H12 HFT"}="H123 HFT"; $resn_exch{"1H13 HFT"}="H131 HFT"; $resn_exch{"2H13 HFT"}="H132 HFT";
$resn_exch{"3H13 HFT"}="H133 HFT"; $resn_exch{"1H14 HFV"}="H141 HFV"; $resn_exch{"2H14 HFV"}="H142 HFV";
$resn_exch{"1H3C HFV"}="H3C1 HFV"; $resn_exch{"2H3C HFV"}="H3C2 HFV"; $resn_exch{"1H4C HFV"}="H4C1 HFV";
$resn_exch{"2H4C HFV"}="H4C2 HFV"; $resn_exch{"1H7C HFV"}="H7C1 HFV"; $resn_exch{"2H7C HFV"}="H7C2 HFV";
$resn_exch{"1H16 HFV"}="H161 HFV"; $resn_exch{"2H16 HFV"}="H162 HFV"; $resn_exch{"1H9  HG9"}=" H91 HG9";
$resn_exch{"2H9  HG9"}=" H92 HG9"; $resn_exch{"1H12 HG9"}="H121 HG9"; $resn_exch{"2H12 HG9"}="H122 HG9";
$resn_exch{"1H15 HG9"}="H151 HG9"; $resn_exch{"2H15 HG9"}="H152 HG9"; $resn_exch{"1H18 HG9"}="H181 HG9";
$resn_exch{"2H18 HG9"}="H182 HG9"; $resn_exch{"1H21 HG9"}="H211 HG9"; $resn_exch{"2H21 HG9"}="H212 HG9";
$resn_exch{"1H24 HG9"}="H241 HG9"; $resn_exch{"2H24 HG9"}="H242 HG9"; $resn_exch{"1H27 HG9"}="H271 HG9";
$resn_exch{"2H27 HG9"}="H272 HG9"; $resn_exch{"1H35 HG9"}="H351 HG9"; $resn_exch{"2H35 HG9"}="H352 HG9";
$resn_exch{"1H36 HG9"}="H361 HG9"; $resn_exch{"2H36 HG9"}="H362 HG9"; $resn_exch{"1H43 HG9"}="H431 HG9";
$resn_exch{"2H43 HG9"}="H432 HG9"; $resn_exch{"1H60 HG9"}="H601 HG9"; $resn_exch{"2H60 HG9"}="H602 HG9";
$resn_exch{"1H1  HG9"}=" H11 HG9"; $resn_exch{"2H1  HG9"}=" H12 HG9"; $resn_exch{"3H1  HG9"}=" H13 HG9";
$resn_exch{"1HN  HGA"}=" HN1 HGA"; $resn_exch{"2HN  HGA"}=" HN2 HGA"; $resn_exch{"1HB  HGA"}=" HB1 HGA";
$resn_exch{"2HB  HGA"}=" HB2 HGA"; $resn_exch{"1HG  HGA"}=" HG1 HGA"; $resn_exch{"2HG  HGA"}=" HG2 HGA";
$resn_exch{"1HO1 HGB"}="HO11 HGB"; $resn_exch{"1HO3 HGB"}="HO31 HGB"; $resn_exch{"1H12 HGP"}="H121 HGP";
$resn_exch{"2H12 HGP"}="H122 HGP"; $resn_exch{"1H13 HGP"}="H131 HGP"; $resn_exch{"2H13 HGP"}="H132 HGP";
$resn_exch{"1H14 HGP"}="H141 HGP"; $resn_exch{"2H14 HGP"}="H142 HGP"; $resn_exch{"1H15 HGP"}="H151 HGP";
$resn_exch{"2H15 HGP"}="H152 HGP"; $resn_exch{"1H16 HGP"}="H161 HGP"; $resn_exch{"2H16 HGP"}="H162 HGP";
$resn_exch{"1H17 HGP"}="H171 HGP"; $resn_exch{"2H17 HGP"}="H172 HGP"; $resn_exch{"1H18 HGP"}="H181 HGP";
$resn_exch{"2H18 HGP"}="H182 HGP"; $resn_exch{"1H19 HGP"}="H191 HGP"; $resn_exch{"2H19 HGP"}="H192 HGP";
$resn_exch{"1H20 HGP"}="H201 HGP"; $resn_exch{"2H20 HGP"}="H202 HGP"; $resn_exch{"1H21 HGP"}="H211 HGP";
$resn_exch{"2H21 HGP"}="H212 HGP"; $resn_exch{"1H22 HGP"}="H221 HGP"; $resn_exch{"2H22 HGP"}="H222 HGP";
$resn_exch{"1H23 HGP"}="H231 HGP"; $resn_exch{"2H23 HGP"}="H232 HGP"; $resn_exch{"1H24 HGP"}="H241 HGP";
$resn_exch{"2H24 HGP"}="H242 HGP"; $resn_exch{"1H25 HGP"}="H251 HGP"; $resn_exch{"2H25 HGP"}="H252 HGP";
$resn_exch{"1H26 HGP"}="H261 HGP"; $resn_exch{"2H26 HGP"}="H262 HGP"; $resn_exch{"3H26 HGP"}="H263 HGP";
$resn_exch{"1H3  HGP"}=" H31 HGP"; $resn_exch{"2H3  HGP"}=" H32 HGP"; $resn_exch{"1H1  HGP"}=" H11 HGP";
$resn_exch{"2H1  HGP"}=" H12 HGP"; $resn_exch{"1H32 HGP"}="H321 HGP"; $resn_exch{"2H32 HGP"}="H322 HGP";
$resn_exch{"1H33 HGP"}="H331 HGP"; $resn_exch{"2H33 HGP"}="H332 HGP"; $resn_exch{"1H34 HGP"}="H341 HGP";
$resn_exch{"2H34 HGP"}="H342 HGP"; $resn_exch{"1H35 HGP"}="H351 HGP"; $resn_exch{"2H35 HGP"}="H352 HGP";
$resn_exch{"1H36 HGP"}="H361 HGP"; $resn_exch{"2H36 HGP"}="H362 HGP"; $resn_exch{"1H37 HGP"}="H371 HGP";
$resn_exch{"2H37 HGP"}="H372 HGP"; $resn_exch{"1H38 HGP"}="H381 HGP"; $resn_exch{"2H38 HGP"}="H382 HGP";
$resn_exch{"1H39 HGP"}="H391 HGP"; $resn_exch{"2H39 HGP"}="H392 HGP"; $resn_exch{"1H40 HGP"}="H401 HGP";
$resn_exch{"2H40 HGP"}="H402 HGP"; $resn_exch{"1H42 HGP"}="H421 HGP"; $resn_exch{"2H42 HGP"}="H422 HGP";
$resn_exch{"1H43 HGP"}="H431 HGP"; $resn_exch{"2H43 HGP"}="H432 HGP"; $resn_exch{"1H44 HGP"}="H441 HGP";
$resn_exch{"2H44 HGP"}="H442 HGP"; $resn_exch{"1H4  HGP"}=" H41 HGP"; $resn_exch{"2H4  HGP"}=" H42 HGP";
$resn_exch{"1H5  HGP"}=" H51 HGP"; $resn_exch{"2H5  HGP"}=" H52 HGP"; $resn_exch{"1H6  HGP"}=" H61 HGP";
$resn_exch{"2H6  HGP"}=" H62 HGP"; $resn_exch{"3H6  HGP"}=" H63 HGP"; $resn_exch{"1H7  HGP"}=" H71 HGP";
$resn_exch{"2H7  HGP"}=" H72 HGP"; $resn_exch{"3H7  HGP"}=" H73 HGP"; $resn_exch{"1H8  HGP"}=" H81 HGP";
$resn_exch{"2H8  HGP"}=" H82 HGP"; $resn_exch{"3H8  HGP"}=" H83 HGP"; $resn_exch{"1H9  HGP"}=" H91 HGP";
$resn_exch{"2H9  HGP"}=" H92 HGP"; $resn_exch{"3H9  HGP"}=" H93 HGP"; $resn_exch{"1HN2 HGT"}="HN21 HGT";
$resn_exch{"2HN2 HGT"}="HN22 HGT"; $resn_exch{"1H26 HGT"}="H261 HGT"; $resn_exch{"2H26 HGT"}="H262 HGT";
$resn_exch{"1H22 HGT"}="H221 HGT"; $resn_exch{"2H22 HGT"}="H222 HGT"; $resn_exch{"1H21 HGT"}="H211 HGT";
$resn_exch{"2H21 HGT"}="H212 HGT"; $resn_exch{"1HN2 HGU"}="HN21 HGU"; $resn_exch{"2HN2 HGU"}="HN22 HGU";
$resn_exch{"1H12 HGX"}="H121 HGX"; $resn_exch{"2H12 HGX"}="H122 HGX"; $resn_exch{"1H13 HGX"}="H131 HGX";
$resn_exch{"2H13 HGX"}="H132 HGX"; $resn_exch{"1H14 HGX"}="H141 HGX"; $resn_exch{"2H14 HGX"}="H142 HGX";
$resn_exch{"1H15 HGX"}="H151 HGX"; $resn_exch{"2H15 HGX"}="H152 HGX"; $resn_exch{"1H16 HGX"}="H161 HGX";
$resn_exch{"2H16 HGX"}="H162 HGX"; $resn_exch{"1H17 HGX"}="H171 HGX"; $resn_exch{"2H17 HGX"}="H172 HGX";
$resn_exch{"1H18 HGX"}="H181 HGX"; $resn_exch{"2H18 HGX"}="H182 HGX"; $resn_exch{"1H19 HGX"}="H191 HGX";
$resn_exch{"2H19 HGX"}="H192 HGX"; $resn_exch{"1H20 HGX"}="H201 HGX"; $resn_exch{"2H20 HGX"}="H202 HGX";
$resn_exch{"1H21 HGX"}="H211 HGX"; $resn_exch{"2H21 HGX"}="H212 HGX"; $resn_exch{"1H22 HGX"}="H221 HGX";
$resn_exch{"2H22 HGX"}="H222 HGX"; $resn_exch{"1H23 HGX"}="H231 HGX"; $resn_exch{"2H23 HGX"}="H232 HGX";
$resn_exch{"3H23 HGX"}="H233 HGX"; $resn_exch{"1H3  HGX"}=" H31 HGX"; $resn_exch{"2H3  HGX"}=" H32 HGX";
$resn_exch{"1H1  HGX"}=" H11 HGX"; $resn_exch{"2H1  HGX"}=" H12 HGX"; $resn_exch{"1H32 HGX"}="H321 HGX";
$resn_exch{"2H32 HGX"}="H322 HGX"; $resn_exch{"1H33 HGX"}="H331 HGX"; $resn_exch{"2H33 HGX"}="H332 HGX";
$resn_exch{"1H34 HGX"}="H341 HGX"; $resn_exch{"2H34 HGX"}="H342 HGX"; $resn_exch{"1H35 HGX"}="H351 HGX";
$resn_exch{"2H35 HGX"}="H352 HGX"; $resn_exch{"1H36 HGX"}="H361 HGX"; $resn_exch{"2H36 HGX"}="H362 HGX";
$resn_exch{"1H37 HGX"}="H371 HGX"; $resn_exch{"2H37 HGX"}="H372 HGX"; $resn_exch{"1H38 HGX"}="H381 HGX";
$resn_exch{"2H38 HGX"}="H382 HGX"; $resn_exch{"1H39 HGX"}="H391 HGX"; $resn_exch{"2H39 HGX"}="H392 HGX";
$resn_exch{"1H40 HGX"}="H401 HGX"; $resn_exch{"2H40 HGX"}="H402 HGX"; $resn_exch{"1H42 HGX"}="H421 HGX";
$resn_exch{"2H42 HGX"}="H422 HGX"; $resn_exch{"1H43 HGX"}="H431 HGX"; $resn_exch{"2H43 HGX"}="H432 HGX";
$resn_exch{"1H44 HGX"}="H441 HGX"; $resn_exch{"2H44 HGX"}="H442 HGX"; $resn_exch{"1H45 HGX"}="H451 HGX";
$resn_exch{"2H45 HGX"}="H452 HGX"; $resn_exch{"1H46 HGX"}="H461 HGX"; $resn_exch{"2H46 HGX"}="H462 HGX";
$resn_exch{"1H47 HGX"}="H471 HGX"; $resn_exch{"2H47 HGX"}="H472 HGX"; $resn_exch{"1H48 HGX"}="H481 HGX";
$resn_exch{"2H48 HGX"}="H482 HGX"; $resn_exch{"3H48 HGX"}="H483 HGX"; $resn_exch{"1H4  HGX"}=" H41 HGX";
$resn_exch{"2H4  HGX"}=" H42 HGX"; $resn_exch{"1H5  HGX"}=" H51 HGX"; $resn_exch{"2H5  HGX"}=" H52 HGX";
$resn_exch{"1H6  HGX"}=" H61 HGX"; $resn_exch{"2H6  HGX"}=" H62 HGX"; $resn_exch{"3H6  HGX"}=" H63 HGX";
$resn_exch{"1H7  HGX"}=" H71 HGX"; $resn_exch{"2H7  HGX"}=" H72 HGX"; $resn_exch{"3H7  HGX"}=" H73 HGX";
$resn_exch{"1H8  HGX"}=" H81 HGX"; $resn_exch{"2H8  HGX"}=" H82 HGX"; $resn_exch{"3H8  HGX"}=" H83 HGX";
$resn_exch{"1H08 HH1"}="H081 HH1"; $resn_exch{"2H08 HH1"}="H082 HH1"; $resn_exch{"1H10 HH1"}="H101 HH1";
$resn_exch{"2H10 HH1"}="H102 HH1"; $resn_exch{"1H20 HH1"}="H201 HH1"; $resn_exch{"2H20 HH1"}="H202 HH1";
$resn_exch{"1H49 HH1"}="H491 HH1"; $resn_exch{"2H49 HH1"}="H492 HH1"; $resn_exch{"1H36 HH1"}="H361 HH1";
$resn_exch{"2H36 HH1"}="H362 HH1"; $resn_exch{"1H45 HH1"}="H451 HH1"; $resn_exch{"2H45 HH1"}="H452 HH1";
$resn_exch{"1H47 HH1"}="H471 HH1"; $resn_exch{"2H47 HH1"}="H472 HH1"; $resn_exch{"1H48 HH1"}="H481 HH1";
$resn_exch{"2H48 HH1"}="H482 HH1"; $resn_exch{"1HN1 HH1"}="HN11 HH1"; $resn_exch{"2HN1 HH1"}="HN12 HH1";
$resn_exch{"1H3  HH2"}=" H31 HH2"; $resn_exch{"1HN6 HH2"}="HN61 HH2"; $resn_exch{"2HN6 HH2"}="HN62 HH2";
$resn_exch{"1H11 HH2"}="H111 HH2"; $resn_exch{"2H11 HH2"}="H112 HH2"; $resn_exch{"2HOP HH2"}="HOP2 HH2";
$resn_exch{"5HOP HH2"}="HOP5 HH2"; $resn_exch{"6HOP HH2"}="HOP6 HH2"; $resn_exch{"1HN2 HHA"}="HN21 HHA";
$resn_exch{"2HN2 HHA"}="HN22 HHA"; $resn_exch{"1H1  HHG"}=" H11 HHG"; $resn_exch{"2H1  HHG"}=" H12 HHG";
$resn_exch{"1H3  HHG"}=" H31 HHG"; $resn_exch{"2H3  HHG"}=" H32 HHG"; $resn_exch{"1H5  HHG"}=" H51 HHG";
$resn_exch{"2H5  HHG"}=" H52 HHG"; $resn_exch{"1H6  HHG"}=" H61 HHG"; $resn_exch{"2H6  HHG"}=" H62 HHG";
$resn_exch{"1H7  HHG"}=" H71 HHG"; $resn_exch{"2H7  HHG"}=" H72 HHG"; $resn_exch{"1H8  HHG"}=" H81 HHG";
$resn_exch{"2H8  HHG"}=" H82 HHG"; $resn_exch{"1H9  HHG"}=" H91 HHG"; $resn_exch{"2H9  HHG"}=" H92 HHG";
$resn_exch{"3H9  HHG"}=" H93 HHG"; $resn_exch{" HN2 HHK"}=" H2  HHK"; $resn_exch{"2HB  HHK"}=" HB2 HHK";
$resn_exch{"1HB  HHK"}=" HB3 HHK"; $resn_exch{"2HG  HHK"}=" HG2 HHK"; $resn_exch{"1HG  HHK"}=" HG3 HHK";
$resn_exch{"2HD  HHK"}=" HD2 HHK"; $resn_exch{"1HD  HHK"}=" HD3 HHK"; $resn_exch{"2HE  HHK"}=" HE2 HHK";
$resn_exch{"3HE  HHK"}=" HE3 HHK"; $resn_exch{"2HZ  HHK"}=" HZ2 HHK"; $resn_exch{"1HZ  HHK"}=" HZ3 HHK";
$resn_exch{"2HT  HHK"}=" HT2 HHK"; $resn_exch{"1HT  HHK"}=" HT1 HHK"; $resn_exch{"1HH  HHK"}=" HH1 HHK";
$resn_exch{"2HH  HHK"}=" HH2 HHK"; $resn_exch{"1H   HHO"}=" H1  HHO"; $resn_exch{"2H   HHO"}=" H2  HHO";
$resn_exch{"1H1  HHO"}=" H11 HHO"; $resn_exch{"2H1  HHO"}=" H12 HHO"; $resn_exch{"1H2  HHO"}=" H21 HHO";
$resn_exch{"2H2  HHO"}=" H22 HHO"; $resn_exch{"1H3  HHO"}=" H31 HHO"; $resn_exch{"2H3  HHO"}=" H32 HHO";
$resn_exch{"1H4  HHO"}=" H41 HHO"; $resn_exch{"2H4  HHO"}=" H42 HHO"; $resn_exch{"1H5  HHO"}=" H51 HHO";
$resn_exch{"2H5  HHO"}=" H52 HHO"; $resn_exch{"1H6  HHO"}=" H61 HHO"; $resn_exch{"2H6  HHO"}=" H62 HHO";
$resn_exch{"1H7  HHO"}=" H71 HHO"; $resn_exch{"2H7  HHO"}=" H72 HHO"; $resn_exch{"1H8  HHO"}=" H81 HHO";
$resn_exch{"2H8  HHO"}=" H82 HHO"; $resn_exch{"1H7  HHP"}=" H71 HHP"; $resn_exch{"2H7  HHP"}=" H72 HHP";
$resn_exch{"1HN2 HHP"}="HN21 HHP"; $resn_exch{"2HN2 HHP"}="HN22 HHP"; $resn_exch{"1H6A HHP"}="H6A1 HHP";
$resn_exch{"2H6A HHP"}="H6A2 HHP"; $resn_exch{"1H7  HHR"}=" H71 HHR"; $resn_exch{"1HN2 HHR"}="HN21 HHR";
$resn_exch{"2HN2 HHR"}="HN22 HHR"; $resn_exch{"1H6A HHR"}="H6A1 HHR"; $resn_exch{"2H6A HHR"}="H6A2 HHR";
$resn_exch{"1H1C HI5"}="H1C1 HI5"; $resn_exch{"2H1C HI5"}="H1C2 HI5"; $resn_exch{"1H3C HI5"}="H3C1 HI5";
$resn_exch{"2H3C HI5"}="H3C2 HI5"; $resn_exch{"1H12 HI5"}="H121 HI5"; $resn_exch{"2H12 HI5"}="H122 HI5";
$resn_exch{"1H13 HI5"}="H131 HI5"; $resn_exch{"2H13 HI5"}="H132 HI5"; $resn_exch{"1H14 HI5"}="H141 HI5";
$resn_exch{"2H14 HI5"}="H142 HI5"; $resn_exch{"3H14 HI5"}="H143 HI5"; $resn_exch{"1H32 HI5"}="H321 HI5";
$resn_exch{"2H32 HI5"}="H322 HI5"; $resn_exch{"1H33 HI5"}="H331 HI5"; $resn_exch{"2H33 HI5"}="H332 HI5";
$resn_exch{"1H34 HI5"}="H341 HI5"; $resn_exch{"2H34 HI5"}="H342 HI5"; $resn_exch{"3H34 HI5"}="H343 HI5";
$resn_exch{"1HN4 HI6"}="HN41 HI6"; $resn_exch{"2HN4 HI6"}="HN42 HI6"; $resn_exch{"1H8  HI6"}=" H81 HI6";
$resn_exch{"2H8  HI6"}=" H82 HI6"; $resn_exch{"1H7  HI6"}=" H71 HI6"; $resn_exch{"2H7  HI6"}=" H72 HI6";
$resn_exch{"1HN  HIA"}=" H   HIA"; $resn_exch{"2HN  HIA"}=" H2  HIA"; $resn_exch{"1HBC HIA"}=" HB2 HIA";
$resn_exch{"2HBC HIA"}=" HB3 HIA"; $resn_exch{"1HXT HIA"}="HXT1 HIA"; $resn_exch{"2HXT HIA"}="HXT2 HIA";
$resn_exch{"1H8  HIB"}=" H81 HIB"; $resn_exch{"2H8  HIB"}=" H82 HIB"; $resn_exch{"3H8  HIB"}=" H83 HIB";
$resn_exch{"1H9  HIB"}=" H91 HIB"; $resn_exch{"2H9  HIB"}=" H92 HIB"; $resn_exch{"3H9  HIB"}=" H93 HIB";
$resn_exch{"1HX  HIB"}=" HX1 HIB"; $resn_exch{"2HX  HIB"}=" HX2 HIB"; $resn_exch{"3HX  HIB"}=" HX3 HIB";
$resn_exch{" HN2 HIC"}=" H2  HIC"; $resn_exch{"1HB  HIC"}=" HB2 HIC"; $resn_exch{"2HB  HIC"}=" HB3 HIC";
$resn_exch{"1HZ  HIC"}=" HZ1 HIC"; $resn_exch{"2HZ  HIC"}=" HZ2 HIC"; $resn_exch{"3HZ  HIC"}=" HZ3 HIC";
$resn_exch{"1HAA HIF"}="HAA1 HIF"; $resn_exch{"2HAA HIF"}="HAA2 HIF"; $resn_exch{"1HMA HIF"}="HMA1 HIF";
$resn_exch{"2HMA HIF"}="HMA2 HIF"; $resn_exch{"3HMA HIF"}="HMA3 HIF"; $resn_exch{"1HBA HIF"}="HBA1 HIF";
$resn_exch{"2HBA HIF"}="HBA2 HIF"; $resn_exch{"1HMB HIF"}="HMB1 HIF"; $resn_exch{"2HMB HIF"}="HMB2 HIF";
$resn_exch{"3HMB HIF"}="HMB3 HIF"; $resn_exch{"1HAB HIF"}="HAB1 HIF"; $resn_exch{"2HAB HIF"}="HAB2 HIF";
$resn_exch{"1HBB HIF"}="HBB1 HIF"; $resn_exch{"2HBB HIF"}="HBB2 HIF"; $resn_exch{"3HBB HIF"}="HBB3 HIF";
$resn_exch{"1HAC HIF"}="HAC1 HIF"; $resn_exch{"2HAC HIF"}="HAC2 HIF"; $resn_exch{"1HBC HIF"}="HBC1 HIF";
$resn_exch{"2HBC HIF"}="HBC2 HIF"; $resn_exch{"3HBC HIF"}="HBC3 HIF"; $resn_exch{"1HIF HIF"}="HIF1 HIF";
$resn_exch{"2HIF HIF"}="HIF2 HIF"; $resn_exch{"3HIF HIF"}="HIF3 HIF"; $resn_exch{"1HMD HIF"}="HMD1 HIF";
$resn_exch{"2HMD HIF"}="HMD2 HIF"; $resn_exch{"3HMD HIF"}="HMD3 HIF"; $resn_exch{"1HAD HIF"}="HAD1 HIF";
$resn_exch{"2HAD HIF"}="HAD2 HIF"; $resn_exch{"1HBD HIF"}="HBD1 HIF"; $resn_exch{"2HBD HIF"}="HBD2 HIF";
$resn_exch{"1HN  HII"}=" HN1 HII"; $resn_exch{"2HN  HII"}=" HN2 HII"; $resn_exch{"3HN  HII"}=" HN3 HII";
$resn_exch{"1HB  HII"}=" HB1 HII"; $resn_exch{"2HB  HII"}=" HB2 HII"; $resn_exch{"1HNZ HII"}="HNZ1 HII";
$resn_exch{"2HNZ HII"}="HNZ2 HII"; $resn_exch{"1H1  HIN"}=" H11 HIN"; $resn_exch{"2H1  HIN"}=" H12 HIN";
$resn_exch{"1H2  HIN"}=" H21 HIN"; $resn_exch{"2H2  HIN"}=" H22 HIN"; $resn_exch{"1H10 HIN"}="H101 HIN";
$resn_exch{"2H10 HIN"}="H102 HIN"; $resn_exch{"3H10 HIN"}="H103 HIN"; $resn_exch{"1H11 HIN"}="H111 HIN";
$resn_exch{"2H11 HIN"}="H112 HIN"; $resn_exch{"3H11 HIN"}="H113 HIN"; $resn_exch{"1H12 HIN"}="H121 HIN";
$resn_exch{"2H12 HIN"}="H122 HIN"; $resn_exch{"1H41 HIO"}="H411 HIO"; $resn_exch{"2H41 HIO"}="H412 HIO";
$resn_exch{"3H41 HIO"}="H413 HIO"; $resn_exch{"1H42 HIO"}="H421 HIO"; $resn_exch{"2H42 HIO"}="H422 HIO";
$resn_exch{"3H42 HIO"}="H423 HIO"; $resn_exch{"1HN  HIP"}=" H   HIP"; $resn_exch{"2HN  HIP"}=" H2  HIP";
$resn_exch{"1HB  HIP"}=" HB2 HIP"; $resn_exch{"2HB  HIP"}=" HB3 HIP"; $resn_exch{"2HOP HIP"}="HOP2 HIP";
$resn_exch{"3HOP HIP"}="HOP3 HIP"; $resn_exch{"1HN  HIQ"}=" HN1 HIQ"; $resn_exch{"2HN  HIQ"}=" HN2 HIQ";
$resn_exch{"1HB  HIQ"}=" HB1 HIQ"; $resn_exch{"2HB  HIQ"}=" HB2 HIQ"; $resn_exch{"1H1C HIQ"}="H1C1 HIQ";
$resn_exch{"2H1C HIQ"}="H1C2 HIQ"; $resn_exch{"1H3C HIQ"}="H3C1 HIQ"; $resn_exch{"2H3C HIQ"}="H3C2 HIQ";
$resn_exch{" HN2 HIS"}=" H2  HIS"; $resn_exch{"1HB  HIS"}=" HB2 HIS"; $resn_exch{"2HB  HIS"}=" HB3 HIS";
$resn_exch{"1H5  HIT"}=" H51 HIT"; $resn_exch{"2H5  HIT"}=" H52 HIT"; $resn_exch{"3H5  HIT"}=" H53 HIT";
$resn_exch{"1H9  HIT"}=" H91 HIT"; $resn_exch{"2H9  HIT"}=" H92 HIT"; $resn_exch{"3H9  HIT"}=" H93 HIT";
$resn_exch{"1H17 HIT"}="H171 HIT"; $resn_exch{"2H17 HIT"}="H172 HIT"; $resn_exch{"3H17 HIT"}="H173 HIT";
$resn_exch{"1H3  HIU"}=" H31 HIU"; $resn_exch{"2H3  HIU"}=" H32 HIU"; $resn_exch{"3H3  HIU"}=" H33 HIU";
$resn_exch{"1H4  HIU"}=" H41 HIU"; $resn_exch{"2H4  HIU"}=" H42 HIU"; $resn_exch{"1H3C HKV"}="H3C1 HKV";
$resn_exch{"2H3C HKV"}="H3C2 HKV"; $resn_exch{"1H4  HKV"}=" H41 HKV"; $resn_exch{"1HB  HLE"}=" HB1 HLE";
$resn_exch{"2HB  HLE"}=" HB2 HLE"; $resn_exch{"1HD1 HLE"}="HD11 HLE"; $resn_exch{"2HD1 HLE"}="HD12 HLE";
$resn_exch{"3HD1 HLE"}="HD13 HLE"; $resn_exch{"1HD2 HLE"}="HD21 HLE"; $resn_exch{"2HD2 HLE"}="HD22 HLE";
$resn_exch{"3HD2 HLE"}="HD23 HLE"; $resn_exch{"1H4N HLO"}="H4N1 HLO"; $resn_exch{"2H4N HLO"}="H4N2 HLO";
$resn_exch{"1H8C HLO"}="H8C1 HLO"; $resn_exch{"2H8C HLO"}="H8C2 HLO"; $resn_exch{"1H7C HLO"}="H7C1 HLO";
$resn_exch{"2H7C HLO"}="H7C2 HLO"; $resn_exch{"1HN  HLU"}=" H   HLU"; $resn_exch{"2HN  HLU"}=" H2  HLU";
$resn_exch{"1HD1 HLU"}="HD11 HLU"; $resn_exch{"2HD1 HLU"}="HD12 HLU"; $resn_exch{"3HD1 HLU"}="HD13 HLU";
$resn_exch{"1HD2 HLU"}="HD21 HLU"; $resn_exch{"2HD2 HLU"}="HD22 HLU"; $resn_exch{"3HD2 HLU"}="HD23 HLU";
$resn_exch{" HO  HLU"}=" HH  HLU"; $resn_exch{"1H10 HM2"}="H101 HM2"; $resn_exch{"2H10 HM2"}="H102 HM2";
$resn_exch{"1H11 HM2"}="H111 HM2"; $resn_exch{"2H11 HM2"}="H112 HM2"; $resn_exch{"1H18 HM2"}="H181 HM2";
$resn_exch{"2H18 HM2"}="H182 HM2"; $resn_exch{"3H18 HM2"}="H183 HM2"; $resn_exch{"1H12 HM4"}="H121 HM4";
$resn_exch{"2H12 HM4"}="H122 HM4"; $resn_exch{"3H12 HM4"}="H123 HM4"; $resn_exch{"1H13 HM4"}="H131 HM4";
$resn_exch{"2H13 HM4"}="H132 HM4"; $resn_exch{"3H13 HM4"}="H133 HM4"; $resn_exch{"1H14 HM4"}="H141 HM4";
$resn_exch{"2H14 HM4"}="H142 HM4"; $resn_exch{"3H14 HM4"}="H143 HM4"; $resn_exch{"1H12 HM5"}="H121 HM5";
$resn_exch{"2H12 HM5"}="H122 HM5"; $resn_exch{"3H12 HM5"}="H123 HM5"; $resn_exch{"1H13 HM5"}="H131 HM5";
$resn_exch{"2H13 HM5"}="H132 HM5"; $resn_exch{"3H13 HM5"}="H133 HM5"; $resn_exch{"1H14 HM5"}="H141 HM5";
$resn_exch{"2H14 HM5"}="H142 HM5"; $resn_exch{"3H14 HM5"}="H143 HM5"; $resn_exch{"1HN  HMA"}=" HN1 HMA";
$resn_exch{"2HN  HMA"}="HN2A HMA"; $resn_exch{"1HB  HMA"}=" HB1 HMA"; $resn_exch{"2HB  HMA"}=" HB2 HMA";
$resn_exch{"3HB  HMA"}=" HB3 HMA"; $resn_exch{"1HA  HMB"}=" HA1 HMB"; $resn_exch{"2HA  HMB"}=" HA2 HMB";
$resn_exch{"1HB  HMB"}=" HB1 HMB"; $resn_exch{"2HB  HMB"}=" HB2 HMB"; $resn_exch{"1HE  HMB"}=" HE1 HMB";
$resn_exch{"2HE  HMB"}=" HE2 HMB"; $resn_exch{"3HE  HMB"}=" HE3 HMB"; $resn_exch{"1H7  HMC"}=" H71 HMC";
$resn_exch{"2H7  HMC"}=" H72 HMC"; $resn_exch{"1HN5 HMD"}="HN51 HMD"; $resn_exch{"2HN5 HMD"}="HN52 HMD";
$resn_exch{"1HAC HME"}="HAC1 HME"; $resn_exch{"2HAC HME"}="HAC2 HME"; $resn_exch{"1HBC HME"}="HBC1 HME";
$resn_exch{"2HBC HME"}="HBC2 HME"; $resn_exch{"3HBC HME"}="HBC3 HME"; $resn_exch{"1HMC HME"}="HMC1 HME";
$resn_exch{"2HMC HME"}="HMC2 HME"; $resn_exch{"3HMC HME"}="HMC3 HME"; $resn_exch{"1HMB HME"}="HMB1 HME";
$resn_exch{"2HMB HME"}="HMB2 HME"; $resn_exch{"3HMB HME"}="HMB3 HME"; $resn_exch{"1HAB HME"}="HAB1 HME";
$resn_exch{"2HAB HME"}="HAB2 HME"; $resn_exch{"1HBB HME"}="HBB1 HME"; $resn_exch{"2HBB HME"}="HBB2 HME";
$resn_exch{"3HBB HME"}="HBB3 HME"; $resn_exch{"1HMA HME"}="HMA1 HME"; $resn_exch{"2HMA HME"}="HMA2 HME";
$resn_exch{"3HMA HME"}="HMA3 HME"; $resn_exch{"1HAA HME"}="HAA1 HME"; $resn_exch{"2HAA HME"}="HAA2 HME";
$resn_exch{"1HBA HME"}="HBA1 HME"; $resn_exch{"2HBA HME"}="HBA2 HME"; $resn_exch{"1HAD HME"}="HAD1 HME";
$resn_exch{"2HAD HME"}="HAD2 HME"; $resn_exch{"1HBD HME"}="HBD1 HME"; $resn_exch{"2HBD HME"}="HBD2 HME";
$resn_exch{"1HMD HME"}="HMD1 HME"; $resn_exch{"2HMD HME"}="HMD2 HME"; $resn_exch{"3HMD HME"}="HMD3 HME";
$resn_exch{"1HN  HMF"}=" H   HMF"; $resn_exch{"2HN  HMF"}=" H2  HMF"; $resn_exch{"1HB  HMF"}=" HB2 HMF";
$resn_exch{"2HB  HMF"}=" HB3 HMF"; $resn_exch{"1HG  HMF"}=" HG2 HMF"; $resn_exch{"2HG  HMF"}=" HG3 HMF";
$resn_exch{" H2  HMF"}=" HC2 HMF"; $resn_exch{"1H5* HMG"}="H5'1 HMG"; $resn_exch{"2H5* HMG"}="H5'2 HMG";
$resn_exch{"H4*A HMG"}=" H4B HMG"; $resn_exch{"H3*A HMG"}=" H3B HMG"; $resn_exch{" H2* HMG"}=" H2' HMG";
$resn_exch{"H2*A HMG"}=" H2B HMG"; $resn_exch{"H1*A HMG"}=" H1B HMG"; $resn_exch{"1H2A HMG"}="H2A1 HMG";
$resn_exch{"1H8A HMG"}="H8A1 HMG"; $resn_exch{"1H12 HMG"}="H121 HMG"; $resn_exch{"2H12 HMG"}="H122 HMG";
$resn_exch{"1H13 HMG"}="H131 HMG"; $resn_exch{"2H13 HMG"}="H132 HMG"; $resn_exch{"3H13 HMG"}="H133 HMG";
$resn_exch{"1H14 HMG"}="H141 HMG"; $resn_exch{"2H14 HMG"}="H142 HMG"; $resn_exch{"3H14 HMG"}="H143 HMG";
$resn_exch{"1H10 HMG"}="H101 HMG"; $resn_exch{"1H7P HMG"}="H7P1 HMG"; $resn_exch{"2H7P HMG"}="H7P2 HMG";
$resn_exch{"1H6P HMG"}="H6P1 HMG"; $resn_exch{"2H6P HMG"}="H6P2 HMG"; $resn_exch{"1H3P HMG"}="H3P1 HMG";
$resn_exch{"2H3P HMG"}="H3P2 HMG"; $resn_exch{"1H2P HMG"}="H2P1 HMG"; $resn_exch{"2H2P HMG"}="H2P2 HMG";
$resn_exch{"1H2  HMG"}=" H21 HMG"; $resn_exch{"2H2  HMG"}=" H22 HMG"; $resn_exch{"1H4  HMG"}=" H41 HMG";
$resn_exch{"2H4  HMG"}=" H42 HMG"; $resn_exch{"1H6  HMG"}=" H61 HMG"; $resn_exch{"2H6  HMG"}=" H62 HMG";
$resn_exch{"3H6  HMG"}=" H63 HMG"; $resn_exch{"1HCM HMH"}="HCM1 HMH"; $resn_exch{"2HCM HMH"}="HCM2 HMH";
$resn_exch{"3HCM HMH"}="HCM3 HMH"; $resn_exch{"1HN4 HMH"}="HN41 HMH"; $resn_exch{"2HN4 HMH"}="HN42 HMH";
$resn_exch{"1HC7 HMH"}="HC71 HMH"; $resn_exch{"2HC7 HMH"}="HC72 HMH"; $resn_exch{"1HB  HMI"}=" HB1 HMI";
$resn_exch{"2HB  HMI"}=" HB2 HMI"; $resn_exch{"1HD1 HMI"}="HD11 HMI"; $resn_exch{"2HD1 HMI"}="HD12 HMI";
$resn_exch{"3HD1 HMI"}="HD13 HMI"; $resn_exch{"1HD2 HMI"}="HD21 HMI"; $resn_exch{"2HD2 HMI"}="HD22 HMI";
$resn_exch{"3HD2 HMI"}="HD23 HMI"; $resn_exch{"1H1  HMK"}=" H11 HMK"; $resn_exch{"2H1  HMK"}="H12A HMK";
$resn_exch{"1H16 HMK"}="H161 HMK"; $resn_exch{"2H16 HMK"}="H162 HMK"; $resn_exch{"1H31 HMM"}="H311 HMM";
$resn_exch{"2H31 HMM"}="H312 HMM"; $resn_exch{"3H31 HMM"}="H313 HMM"; $resn_exch{"1H14 HMM"}="H141 HMM";
$resn_exch{"2H14 HMM"}="H142 HMM"; $resn_exch{"1H15 HMM"}="H151 HMM"; $resn_exch{"1H16 HMM"}="H161 HMM";
$resn_exch{"1H17 HMM"}="H171 HMM"; $resn_exch{"1H18 HMM"}="H181 HMM"; $resn_exch{"2H18 HMM"}="H182 HMM";
$resn_exch{"1HC3 HMN"}="HC31 HMN"; $resn_exch{"2HC3 HMN"}="HC32 HMN"; $resn_exch{"1H11 HMN"}="H111 HMN";
$resn_exch{"2H11 HMN"}="H112 HMN"; $resn_exch{"3H11 HMN"}="H113 HMN"; $resn_exch{"1HC9 HMN"}="HC91 HMN";
$resn_exch{"2HC9 HMN"}="HC92 HMN"; $resn_exch{"1H20 HMO"}="H201 HMO"; $resn_exch{"2H20 HMO"}="H202 HMO";
$resn_exch{"3H20 HMO"}="H203 HMO"; $resn_exch{"1H1' HMP"}="H1'1 HMP"; $resn_exch{"2H1' HMP"}="H1'2 HMP";
$resn_exch{"1H3' HMP"}="H3'1 HMP"; $resn_exch{"2H3' HMP"}="H3'2 HMP"; $resn_exch{"3H3' HMP"}="H3'3 HMP";
$resn_exch{"1H4' HMP"}="H4'1 HMP"; $resn_exch{"2H4' HMP"}="H4'2 HMP"; $resn_exch{"3H4' HMP"}="H4'3 HMP";
$resn_exch{"1HN  HMR"}=" H   HMR"; $resn_exch{"2HN  HMR"}=" H2  HMR"; $resn_exch{"1HB  HMR"}=" HB2 HMR";
$resn_exch{"2HB  HMR"}=" HB3 HMR"; $resn_exch{"1HG  HMR"}=" HG2 HMR"; $resn_exch{"2HG  HMR"}=" HG3 HMR";
$resn_exch{"1HD  HMR"}=" HD2 HMR"; $resn_exch{"2HD  HMR"}=" HD3 HMR"; $resn_exch{" HNE HMR"}=" HE  HMR";
$resn_exch{" HN1 HMR"}=" HH1 HMR"; $resn_exch{"1HN2 HMR"}="HH21 HMR"; $resn_exch{"2HN2 HMR"}="HH22 HMR";
$resn_exch{"1HC  HMR"}=" HC1 HMR"; $resn_exch{"2HC  HMR"}=" HC2 HMR"; $resn_exch{"1HM5 HMU"}="HM51 HMU";
$resn_exch{"2HM5 HMU"}="HM52 HMU"; $resn_exch{"1H5C HNE"}="H5C1 HNE"; $resn_exch{"2H5C HNE"}="H5C2 HNE";
$resn_exch{"1H6C HNE"}="H6C1 HNE"; $resn_exch{"2H6C HNE"}="H6C2 HNE"; $resn_exch{"1H7C HNE"}="H7C1 HNE";
$resn_exch{"2H7C HNE"}="H7C2 HNE"; $resn_exch{"1H8C HNE"}="H8C1 HNE"; $resn_exch{"2H8C HNE"}="H8C2 HNE";
$resn_exch{"1H9C HNE"}="H9C1 HNE"; $resn_exch{"2H9C HNE"}="H9C2 HNE"; $resn_exch{"3H9C HNE"}="H9C3 HNE";
$resn_exch{"1HMA HNI"}="HMA1 HNI"; $resn_exch{"2HMA HNI"}="HMA2 HNI"; $resn_exch{"3HMA HNI"}="HMA3 HNI";
$resn_exch{"1HAA HNI"}="HAA1 HNI"; $resn_exch{"2HAA HNI"}="HAA2 HNI"; $resn_exch{"1HBA HNI"}="HBA1 HNI";
$resn_exch{"2HBA HNI"}="HBA2 HNI"; $resn_exch{"1HMB HNI"}="HMB1 HNI"; $resn_exch{"2HMB HNI"}="HMB2 HNI";
$resn_exch{"3HMB HNI"}="HMB3 HNI"; $resn_exch{"1HBB HNI"}="HBB1 HNI"; $resn_exch{"2HBB HNI"}="HBB2 HNI";
$resn_exch{"1HMC HNI"}="HMC1 HNI"; $resn_exch{"2HMC HNI"}="HMC2 HNI"; $resn_exch{"3HMC HNI"}="HMC3 HNI";
$resn_exch{"1HBC HNI"}="HBC1 HNI"; $resn_exch{"2HBC HNI"}="HBC2 HNI"; $resn_exch{"1HMD HNI"}="HMD1 HNI";
$resn_exch{"2HMD HNI"}="HMD2 HNI"; $resn_exch{"3HMD HNI"}="HMD3 HNI"; $resn_exch{"1HAD HNI"}="HAD1 HNI";
$resn_exch{"2HAD HNI"}="HAD2 HNI"; $resn_exch{"1HBD HNI"}="HBD1 HNI"; $resn_exch{"2HBD HNI"}="HBD2 HNI";
$resn_exch{"1H1  HNP"}=" H11 HNP"; $resn_exch{"2H1  HNP"}=" H12 HNP"; $resn_exch{"1H1  HNT"}=" H11 HNT";
$resn_exch{"2H1  HNT"}=" H12 HNT"; $resn_exch{"1H4  HNT"}="H41A HNT"; $resn_exch{"2H4  HNT"}=" H42 HNT";
$resn_exch{"1H22 HNT"}="H221 HNT"; $resn_exch{"2H22 HNT"}="H222 HNT"; $resn_exch{"1HN  HOA"}=" HN1 HOA";
$resn_exch{"2HN  HOA"}=" HN2 HOA"; $resn_exch{"2HOP HOB"}="HOP2 HOB"; $resn_exch{"1H5* HOB"}=" H5' HOB";
$resn_exch{"2H5* HOB"}="H5'' HOB"; $resn_exch{" H4* HOB"}=" H4' HOB"; $resn_exch{" H3* HOB"}=" H3' HOB";
$resn_exch{"1H2* HOB"}=" H2' HOB"; $resn_exch{"2H2* HOB"}="H2'' HOB"; $resn_exch{" H1* HOB"}=" H1' HOB";
$resn_exch{"1H2  HOB"}=" H21 HOB"; $resn_exch{"2H2  HOB"}=" H22 HOB"; $resn_exch{"1H3  HOB"}=" H31 HOB";
$resn_exch{"2H3  HOB"}=" H32 HOB"; $resn_exch{"1H4  HOB"}=" H41 HOB"; $resn_exch{"2H4  HOB"}=" H42 HOB";
$resn_exch{"9H   HOB"}=" H9  HOB"; $resn_exch{"1H14 HOB"}="H141 HOB"; $resn_exch{"2H14 HOB"}="H142 HOB";
$resn_exch{"1H13 HOB"}="H131 HOB"; $resn_exch{"2H13 HOB"}="H132 HOB"; $resn_exch{"1H10 HOB"}="H101 HOB";
$resn_exch{"2H10 HOB"}="H102 HOB"; $resn_exch{"6H1  HOB"}=" H16 HOB"; $resn_exch{"1H17 HOB"}="H171 HOB";
$resn_exch{"2H17 HOB"}="H172 HOB"; $resn_exch{"8H1  HOB"}=" H18 HOB"; $resn_exch{"9H1  HOB"}=" H19 HOB";
$resn_exch{"1H20 HOB"}="H201 HOB"; $resn_exch{"2H20 HOB"}="H202 HOB"; $resn_exch{"1H21 HOB"}="H211 HOB";
$resn_exch{"2H21 HOB"}="H212 HOB"; $resn_exch{"3H2  HOB"}=" H23 HOB"; $resn_exch{"1H25 HOB"}="H251 HOB";
$resn_exch{"2H25 HOB"}="H252 HOB"; $resn_exch{"1H26 HOB"}="H261 HOB"; $resn_exch{"2H26 HOB"}="H262 HOB";
$resn_exch{"7H2  HOB"}=" H27 HOB"; $resn_exch{"8H2  HOB"}=" H28 HOB"; $resn_exch{"1H29 HOB"}="H291 HOB";
$resn_exch{"2H29 HOB"}="H292 HOB"; $resn_exch{"3H29 HOB"}="H293 HOB"; $resn_exch{"1H30 HOB"}="H301 HOB";
$resn_exch{"2H30 HOB"}="H302 HOB"; $resn_exch{"1H31 HOB"}="H311 HOB"; $resn_exch{"2H31 HOB"}="H312 HOB";
$resn_exch{"1H32 HOB"}="H321 HOB"; $resn_exch{"2H32 HOB"}="H322 HOB"; $resn_exch{"3H3  HOB"}=" H33 HOB";
$resn_exch{"1H34 HOB"}="H341 HOB"; $resn_exch{"2H34 HOB"}="H342 HOB"; $resn_exch{"3H34 HOB"}="H343 HOB";
$resn_exch{"1H35 HOB"}="H351 HOB"; $resn_exch{"2H35 HOB"}="H352 HOB"; $resn_exch{"3H35 HOB"}="H353 HOB";
$resn_exch{"1H24 HOB"}="H241 HOB"; $resn_exch{"2H24 HOB"}="H242 HOB"; $resn_exch{"3H24 HOB"}="H243 HOB";
$resn_exch{"1H15 HOB"}="H151 HOB"; $resn_exch{"2H15 HOB"}="H152 HOB"; $resn_exch{"3H15 HOB"}="H153 HOB";
$resn_exch{"H3T  HOB"}="HO3' HOB"; $resn_exch{"3HOP HOB"}="HOP3 HOB"; $resn_exch{"1H8  HOC"}=" H81 HOC";
$resn_exch{"2H8  HOC"}=" H82 HOC"; $resn_exch{"3H8  HOC"}=" H83 HOC"; $resn_exch{"1H7  HOC"}=" H71 HOC";
$resn_exch{"2H7  HOC"}=" H72 HOC"; $resn_exch{"1H3  HOC"}=" H31 HOC"; $resn_exch{"2H3  HOC"}=" H32 HOC";
$resn_exch{"1H4  HOC"}=" H41 HOC"; $resn_exch{"2H4  HOC"}=" H42 HOC"; $resn_exch{"1H5  HOC"}=" H51 HOC";
$resn_exch{"2H5  HOC"}=" H52 HOC"; $resn_exch{"1H6  HOC"}=" H61 HOC"; $resn_exch{"2H6  HOC"}=" H62 HOC";
$resn_exch{"1H21 HOE"}="H211 HOE"; $resn_exch{"2H21 HOE"}="H212 HOE"; $resn_exch{"1H22 HOE"}="H221 HOE";
$resn_exch{"2H22 HOE"}="H222 HOE"; $resn_exch{"1H23 HOE"}="H231 HOE"; $resn_exch{"2H23 HOE"}="H232 HOE";
$resn_exch{"1H24 HOE"}="H241 HOE"; $resn_exch{"2H24 HOE"}="H242 HOE"; $resn_exch{"1H25 HOE"}="H251 HOE";
$resn_exch{"2H25 HOE"}="H252 HOE"; $resn_exch{"3H25 HOE"}="H253 HOE"; $resn_exch{"1H   HOH"}=" H1  HOH";
$resn_exch{"2H   HOH"}=" H2  HOH"; $resn_exch{"2HOP HOL"}="HOP2 HOL"; $resn_exch{"1H5* HOL"}=" H5' HOL";
$resn_exch{"2H5* HOL"}="H5'' HOL"; $resn_exch{" H4* HOL"}=" H4' HOL"; $resn_exch{" H3* HOL"}=" H3' HOL";
$resn_exch{"1H2* HOL"}=" H2' HOL"; $resn_exch{"2H2* HOL"}="H2'' HOL"; $resn_exch{" H1* HOL"}=" H1' HOL";
$resn_exch{"1H2  HOL"}=" H21 HOL"; $resn_exch{"2H2  HOL"}=" H22 HOL"; $resn_exch{"1H3  HOL"}=" H31 HOL";
$resn_exch{"2H3  HOL"}=" H32 HOL"; $resn_exch{"1H4  HOL"}=" H41 HOL"; $resn_exch{"2H4  HOL"}=" H42 HOL";
$resn_exch{"9H   HOL"}=" H9  HOL"; $resn_exch{"1H14 HOL"}="H141 HOL"; $resn_exch{"2H14 HOL"}="H142 HOL";
$resn_exch{"1H13 HOL"}="H131 HOL"; $resn_exch{"2H13 HOL"}="H132 HOL"; $resn_exch{"1H10 HOL"}="H101 HOL";
$resn_exch{"2H10 HOL"}="H102 HOL"; $resn_exch{"6H1  HOL"}=" H16 HOL"; $resn_exch{"1H17 HOL"}="H171 HOL";
$resn_exch{"2H17 HOL"}="H172 HOL"; $resn_exch{"8H1  HOL"}=" H18 HOL"; $resn_exch{"9H1  HOL"}=" H19 HOL";
$resn_exch{"1H20 HOL"}="H201 HOL"; $resn_exch{"2H20 HOL"}="H202 HOL"; $resn_exch{"1H21 HOL"}="H211 HOL";
$resn_exch{"2H21 HOL"}="H212 HOL"; $resn_exch{"3H2  HOL"}=" H23 HOL"; $resn_exch{"1H25 HOL"}="H251 HOL";
$resn_exch{"2H25 HOL"}="H252 HOL"; $resn_exch{"1H26 HOL"}="H261 HOL"; $resn_exch{"2H26 HOL"}="H262 HOL";
$resn_exch{"7H2  HOL"}=" H27 HOL"; $resn_exch{"8H2  HOL"}=" H28 HOL"; $resn_exch{"1H29 HOL"}="H291 HOL";
$resn_exch{"2H29 HOL"}="H292 HOL"; $resn_exch{"3H29 HOL"}="H293 HOL"; $resn_exch{"1H30 HOL"}="H301 HOL";
$resn_exch{"2H30 HOL"}="H302 HOL"; $resn_exch{"1H31 HOL"}="H311 HOL"; $resn_exch{"2H31 HOL"}="H312 HOL";
$resn_exch{"1H32 HOL"}="H321 HOL"; $resn_exch{"2H32 HOL"}="H322 HOL"; $resn_exch{"3H3  HOL"}=" H33 HOL";
$resn_exch{"1H34 HOL"}="H341 HOL"; $resn_exch{"2H34 HOL"}="H342 HOL"; $resn_exch{"3H34 HOL"}="H343 HOL";
$resn_exch{"1H35 HOL"}="H351 HOL"; $resn_exch{"2H35 HOL"}="H352 HOL"; $resn_exch{"3H35 HOL"}="H353 HOL";
$resn_exch{"1H24 HOL"}="H241 HOL"; $resn_exch{"2H24 HOL"}="H242 HOL"; $resn_exch{"3H24 HOL"}="H243 HOL";
$resn_exch{"1H15 HOL"}="H151 HOL"; $resn_exch{"2H15 HOL"}="H152 HOL"; $resn_exch{"3H15 HOL"}="H153 HOL";
$resn_exch{"H3T  HOL"}="HO3' HOL"; $resn_exch{"3HOP HOL"}="HOP3 HOL"; $resn_exch{"1HA4 HOM"}="HA41 HOM";
$resn_exch{"1HA6 HOM"}="HA61 HOM"; $resn_exch{"1HA7 HOM"}="HA71 HOM"; $resn_exch{"1HB  HOM"}=" HB1 HOM";
$resn_exch{"2HB  HOM"}=" HB2 HOM"; $resn_exch{"1HB7 HOM"}="HB71 HOM"; $resn_exch{"2HB7 HOM"}="HB72 HOM";
$resn_exch{"1HB8 HOM"}="HB81 HOM"; $resn_exch{"2HB8 HOM"}="HB82 HOM"; $resn_exch{"3HB8 HOM"}="HB83 HOM";
$resn_exch{"1HO4 HOM"}="HO41 HOM"; $resn_exch{"1HO1 HOM"}="HO11 HOM"; $resn_exch{"1H12 HOP"}="H121 HOP";
$resn_exch{"2H12 HOP"}="H122 HOP"; $resn_exch{"1H9  HOP"}=" H91 HOP"; $resn_exch{"2H9  HOP"}=" H92 HOP";
$resn_exch{"1H8  HOP"}=" H81 HOP"; $resn_exch{"2H8  HOP"}=" H82 HOP"; $resn_exch{"1H23 HOP"}="H231 HOP";
$resn_exch{"2H23 HOP"}="H232 HOP"; $resn_exch{"1H24 HOP"}="H241 HOP"; $resn_exch{"2H24 HOP"}="H242 HOP";
$resn_exch{"1H25 HOP"}="H251 HOP"; $resn_exch{"2H25 HOP"}="H252 HOP"; $resn_exch{"1H51 HOQ"}="H511 HOQ";
$resn_exch{"2H51 HOQ"}="H512 HOQ"; $resn_exch{"3H51 HOQ"}="H513 HOQ"; $resn_exch{"1H52 HOQ"}="H521 HOQ";
$resn_exch{"2H52 HOQ"}="H522 HOQ"; $resn_exch{"3H52 HOQ"}="H523 HOQ"; $resn_exch{"1H9  HOQ"}=" H91 HOQ";
$resn_exch{"2H9  HOQ"}=" H92 HOQ"; $resn_exch{"3H9  HOQ"}=" H93 HOQ"; $resn_exch{"1H10 HOQ"}="H101 HOQ";
$resn_exch{"2H10 HOQ"}="H102 HOQ"; $resn_exch{"3H10 HOQ"}="H103 HOQ"; $resn_exch{"1HN  HOZ"}=" HN1 HOZ";
$resn_exch{"2HN  HOZ"}=" HN2 HOZ"; $resn_exch{"1H   HOZ"}=" H1  HOZ"; $resn_exch{"2H   HOZ"}=" H2  HOZ";
$resn_exch{"1HN1 HOZ"}="HN11 HOZ"; $resn_exch{"2HN1 HOZ"}="HN12 HOZ"; $resn_exch{"1HB  HOZ"}=" HB1 HOZ";
$resn_exch{"2HB  HOZ"}=" HB2 HOZ"; $resn_exch{"1HG  HOZ"}=" HG1 HOZ"; $resn_exch{"2HG  HOZ"}=" HG2 HOZ";
$resn_exch{"1H3  HP1"}=" H31 HP1"; $resn_exch{"2H3  HP1"}=" H32 HP1"; $resn_exch{"3H3  HP1"}=" H33 HP1";
$resn_exch{"1H9  HP1"}=" H91 HP1"; $resn_exch{"2H9  HP1"}=" H92 HP1"; $resn_exch{"3H9  HP1"}=" H93 HP1";
$resn_exch{"1H15 HP1"}="H151 HP1"; $resn_exch{"2H15 HP1"}="H152 HP1"; $resn_exch{"3H15 HP1"}="H153 HP1";
$resn_exch{"1H18 HP1"}="H181 HP1"; $resn_exch{"2H18 HP1"}="H182 HP1"; $resn_exch{"1H19 HP1"}="H191 HP1";
$resn_exch{"2H19 HP1"}="H192 HP1"; $resn_exch{"1H20 HP1"}="H201 HP1"; $resn_exch{"2H20 HP1"}="H202 HP1";
$resn_exch{"1H21 HP1"}="H211 HP1"; $resn_exch{"2H21 HP1"}="H212 HP1"; $resn_exch{"3H21 HP1"}="H213 HP1";
$resn_exch{"1H22 HP1"}="H221 HP1"; $resn_exch{"2H22 HP1"}="H222 HP1"; $resn_exch{"3H22 HP1"}="H223 HP1";
$resn_exch{"1H28 HP1"}="H281 HP1"; $resn_exch{"2H28 HP1"}="H282 HP1"; $resn_exch{"3H28 HP1"}="H283 HP1";
$resn_exch{"1H29 HP1"}="H291 HP1"; $resn_exch{"2H29 HP1"}="H292 HP1"; $resn_exch{"1H30 HP1"}="H301 HP1";
$resn_exch{"2H30 HP1"}="H302 HP1"; $resn_exch{"1H9  HP2"}=" H91 HP2"; $resn_exch{"2H9  HP2"}=" H92 HP2";
$resn_exch{"3H9  HP2"}=" H93 HP2"; $resn_exch{"1H15 HP2"}="H151 HP2"; $resn_exch{"2H15 HP2"}="H152 HP2";
$resn_exch{"3H15 HP2"}="H153 HP2"; $resn_exch{"1H21 HP2"}="H211 HP2"; $resn_exch{"2H21 HP2"}="H212 HP2";
$resn_exch{"3H21 HP2"}="H213 HP2"; $resn_exch{"1H22 HP2"}="H221 HP2"; $resn_exch{"2H22 HP2"}="H222 HP2";
$resn_exch{"3H22 HP2"}="H223 HP2"; $resn_exch{"1H20 HP2"}="H201 HP2"; $resn_exch{"2H20 HP2"}="H202 HP2";
$resn_exch{"1H18 HP2"}="H181 HP2"; $resn_exch{"2H18 HP2"}="H182 HP2"; $resn_exch{"1H19 HP2"}="H191 HP2";
$resn_exch{"2H19 HP2"}="H192 HP2"; $resn_exch{"1H3  HP2"}=" H31 HP2"; $resn_exch{"2H3  HP2"}="H32A HP2";
$resn_exch{"3H3  HP2"}=" H33 HP2"; $resn_exch{"1H28 HP2"}="H281 HP2"; $resn_exch{"2H28 HP2"}="H282 HP2";
$resn_exch{"3H28 HP2"}="H283 HP2"; $resn_exch{"1H30 HP2"}="H301 HP2"; $resn_exch{"2H30 HP2"}="H302 HP2";
$resn_exch{"1H29 HP2"}="H291 HP2"; $resn_exch{"2H29 HP2"}="H292 HP2"; $resn_exch{"1H1  HP3"}=" H11 HP3";
$resn_exch{"2H1  HP3"}=" H12 HP3"; $resn_exch{"3H1  HP3"}=" H13 HP3"; $resn_exch{"1H3  HP3"}=" H31 HP3";
$resn_exch{"2H3  HP3"}=" H32 HP3"; $resn_exch{"1H20 HP6"}="H201 HP6"; $resn_exch{"2H20 HP6"}="H202 HP6";
$resn_exch{"3H20 HP6"}="H203 HP6"; $resn_exch{"1H21 HP6"}="H211 HP6"; $resn_exch{"2H21 HP6"}="H212 HP6";
$resn_exch{"1H22 HP6"}="H221 HP6"; $resn_exch{"2H22 HP6"}="H222 HP6"; $resn_exch{"1H23 HP6"}="H231 HP6";
$resn_exch{"2H23 HP6"}="H232 HP6"; $resn_exch{"1H24 HP6"}="H241 HP6"; $resn_exch{"2H24 HP6"}="H242 HP6";
$resn_exch{"1H25 HP6"}="H251 HP6"; $resn_exch{"2H25 HP6"}="H252 HP6"; $resn_exch{"1H26 HP6"}="H261 HP6";
$resn_exch{"2H26 HP6"}="H262 HP6"; $resn_exch{"3H26 HP6"}="H263 HP6"; $resn_exch{"1H1  HPB"}=" H11 HPB";
$resn_exch{"2H1  HPB"}=" H12 HPB"; $resn_exch{"3H1  HPB"}=" H13 HPB"; $resn_exch{"1H4  HPB"}=" H41 HPB";
$resn_exch{"2H4  HPB"}=" H42 HPB"; $resn_exch{"1HN3 HPB"}="HN31 HPB"; $resn_exch{"2HN3 HPB"}="HN32 HPB";
$resn_exch{"1HB  HPC"}=" HB1 HPC"; $resn_exch{"2HB  HPC"}=" HB2 HPC"; $resn_exch{"1HCH HPC"}="HCH1 HPC";
$resn_exch{"2HCH HPC"}="HCH2 HPC"; $resn_exch{"3HCH HPC"}="HCH3 HPC"; $resn_exch{"2HOP HPD"}="HOP2 HPD";
$resn_exch{"3HOP HPD"}="HOP3 HPD"; $resn_exch{"1H5* HPD"}="H5'1 HPD"; $resn_exch{"2H5* HPD"}="H5'2 HPD";
$resn_exch{"4HO* HPD"}="HO'4 HPD"; $resn_exch{"3HO* HPD"}="HO'3 HPD"; $resn_exch{"1H1* HPD"}="H1'1 HPD";
$resn_exch{"2H1* HPD"}="H1'2 HPD"; $resn_exch{"1HO* HPD"}="HO'1 HPD"; $resn_exch{" H3* HPD"}=" H3' HPD";
$resn_exch{" H4* HPD"}=" H4' HPD"; $resn_exch{"1H2* HPD"}="H2'1 HPD"; $resn_exch{"2H2* HPD"}="H2'2 HPD";
$resn_exch{"1HN  HPE"}=" H   HPE"; $resn_exch{"2HN  HPE"}=" H2  HPE"; $resn_exch{"1HB  HPE"}=" HB2 HPE";
$resn_exch{"2HB  HPE"}=" HB3 HPE"; $resn_exch{"1HG  HPE"}=" HG2 HPE"; $resn_exch{"2HG  HPE"}=" HG3 HPE";
$resn_exch{"1HE  HPE"}=" HE1 HPE"; $resn_exch{"2HE  HPE"}=" HE2 HPE"; $resn_exch{"1HZ  HPE"}=" HZ1 HPE";
$resn_exch{"2HZ  HPE"}=" HZ2 HPE"; $resn_exch{"1HH  HPE"}=" HH  HPE"; $resn_exch{"1H11 HPF"}="H111 HPF";
$resn_exch{"2H11 HPF"}="H112 HPF"; $resn_exch{"1H1  HPG"}=" H11 HPG"; $resn_exch{"2H1  HPG"}=" H12 HPG";
$resn_exch{"1H2  HPG"}=" H21 HPG"; $resn_exch{"2H2  HPG"}=" H22 HPG"; $resn_exch{"1H3  HPG"}=" H31 HPG";
$resn_exch{"2H3  HPG"}=" H32 HPG"; $resn_exch{" H   HPH"}=" HC  HPH"; $resn_exch{"1HN  HPH"}=" H   HPH";
$resn_exch{"2HN  HPH"}=" H2  HPH"; $resn_exch{"1HB  HPH"}=" HB2 HPH"; $resn_exch{"2HB  HPH"}=" HB3 HPH";
$resn_exch{"1H07 HPI"}="H071 HPI"; $resn_exch{"2H07 HPI"}="H072 HPI"; $resn_exch{"1H11 HPI"}="H111 HPI";
$resn_exch{"2H11 HPI"}="H112 HPI"; $resn_exch{"1H12 HPI"}="H121 HPI"; $resn_exch{"2H12 HPI"}="H122 HPI";
$resn_exch{"1H28 HPI"}="H281 HPI"; $resn_exch{"2H28 HPI"}="H282 HPI"; $resn_exch{"1H29 HPI"}="H291 HPI";
$resn_exch{"2H29 HPI"}="H292 HPI"; $resn_exch{"1HA5 HPK"}="HA51 HPK"; $resn_exch{"2HA5 HPK"}="HA52 HPK";
$resn_exch{"1H1  HPL"}=" H11 HPL"; $resn_exch{"2H1  HPL"}=" H12 HPL"; $resn_exch{"1H2  HPL"}=" H21 HPL";
$resn_exch{"2H2  HPL"}=" H22 HPL"; $resn_exch{"1H3  HPL"}=" H31 HPL"; $resn_exch{"2H3  HPL"}=" H32 HPL";
$resn_exch{"1H4  HPL"}=" H41 HPL"; $resn_exch{"2H4  HPL"}=" H42 HPL"; $resn_exch{"1H5  HPL"}=" H51 HPL";
$resn_exch{"2H5  HPL"}=" H52 HPL"; $resn_exch{"1H6  HPL"}=" H61 HPL"; $resn_exch{"2H6  HPL"}=" H62 HPL";
$resn_exch{"3H6  HPL"}=" H63 HPL"; $resn_exch{"1H12 HPL"}="H121 HPL"; $resn_exch{"2H12 HPL"}="H122 HPL";
$resn_exch{"1H1C HPM"}="H1C1 HPM"; $resn_exch{"2H1C HPM"}="H1C2 HPM"; $resn_exch{"1H2C HPM"}="H2C1 HPM";
$resn_exch{"2H2C HPM"}="H2C2 HPM"; $resn_exch{"1H3C HPM"}="H3C1 HPM"; $resn_exch{"2H3C HPM"}="H3C2 HPM";
$resn_exch{"1H4C HPM"}="H4C1 HPM"; $resn_exch{"2H4C HPM"}="H4C2 HPM"; $resn_exch{"1H5C HPM"}="H5C1 HPM";
$resn_exch{"2H5C HPM"}="H5C2 HPM"; $resn_exch{"1H7C HPM"}="H7C1 HPM"; $resn_exch{"2H7C HPM"}="H7C2 HPM";
$resn_exch{"1H9C HPM"}="H9C1 HPM"; $resn_exch{"2H9C HPM"}="H9C2 HPM"; $resn_exch{"1H38 HPM"}="H381 HPM";
$resn_exch{"2H38 HPM"}="H382 HPM"; $resn_exch{"3H38 HPM"}="H383 HPM"; $resn_exch{"1HN1 HPN"}="HN11 HPN";
$resn_exch{"2HN1 HPN"}="HN12 HPN"; $resn_exch{"1H2  HPN"}=" H21 HPN"; $resn_exch{"2H2  HPN"}=" H22 HPN";
$resn_exch{"1H3  HPN"}=" H31 HPN"; $resn_exch{"2H3  HPN"}=" H32 HPN"; $resn_exch{"1H4  HPN"}=" H41 HPN";
$resn_exch{"2H4  HPN"}=" H42 HPN"; $resn_exch{"1H5  HPN"}=" H51 HPN"; $resn_exch{"2H5  HPN"}=" H52 HPN";
$resn_exch{"1H6  HPN"}=" H61 HPN"; $resn_exch{"2H6  HPN"}=" H62 HPN"; $resn_exch{"1H7  HPN"}=" H71 HPN";
$resn_exch{"2H7  HPN"}=" H72 HPN"; $resn_exch{"3H7  HPN"}=" H73 HPN"; $resn_exch{"1H1  HPO"}=" H11 HPO";
$resn_exch{"2H1  HPO"}=" H12 HPO"; $resn_exch{"3H1  HPO"}=" H13 HPO"; $resn_exch{"1H2  HPO"}=" H21 HPO";
$resn_exch{"2H2  HPO"}=" H22 HPO"; $resn_exch{"1H3  HPO"}=" H31 HPO"; $resn_exch{"2H3  HPO"}=" H32 HPO";
$resn_exch{"1H4  HPO"}=" H41 HPO"; $resn_exch{"2H4  HPO"}=" H42 HPO"; $resn_exch{"1H5  HPO"}=" H51 HPO";
$resn_exch{"2H5  HPO"}=" H52 HPO"; $resn_exch{"1H7  HPO"}=" H71 HPO"; $resn_exch{"2H7  HPO"}=" H72 HPO";
$resn_exch{"1H7  HPP"}=" H71 HPP"; $resn_exch{"2H7  HPP"}=" H72 HPP"; $resn_exch{"1H8  HPP"}=" H81 HPP";
$resn_exch{"2H8  HPP"}=" H82 HPP"; $resn_exch{"1HN  HPQ"}=" H   HPQ"; $resn_exch{"2HN  HPQ"}=" H2  HPQ";
$resn_exch{"1HB  HPQ"}=" HB1 HPQ"; $resn_exch{"2HB  HPQ"}=" HB2 HPQ"; $resn_exch{"1HG  HPQ"}=" HG1 HPQ";
$resn_exch{"2HG  HPQ"}=" HG2 HPQ"; $resn_exch{"1HE  HPQ"}=" HE1 HPQ"; $resn_exch{"2HE  HPQ"}=" HE2 HPQ";
$resn_exch{"1HZ  HPQ"}=" HZ1 HPQ"; $resn_exch{"2HZ  HPQ"}=" HZ2 HPQ"; $resn_exch{"1HH  HPQ"}=" HH1 HPQ";
$resn_exch{"1HM  HPQ"}=" HM1 HPQ"; $resn_exch{"2HM  HPQ"}=" HM2 HPQ"; $resn_exch{"3HM  HPQ"}=" HM3 HPQ";
$resn_exch{"1H8  HPR"}=" H81 HPR"; $resn_exch{"2H8  HPR"}=" H82 HPR"; $resn_exch{" H1* HPR"}=" H1' HPR";
$resn_exch{" H2* HPR"}=" H2' HPR"; $resn_exch{"*HO2 HPR"}="HO2' HPR"; $resn_exch{" H3* HPR"}=" H3' HPR";
$resn_exch{"*HO3 HPR"}="HO3' HPR"; $resn_exch{" H4* HPR"}=" H4' HPR"; $resn_exch{"1H5* HPR"}="H5'1 HPR";
$resn_exch{"2H5* HPR"}="H5'2 HPR"; $resn_exch{"*HO5 HPR"}="HO5' HPR"; $resn_exch{"3H4  HPT"}=" H43 HPT";
$resn_exch{"2H4  HPT"}=" H42 HPT"; $resn_exch{"1H4  HPT"}=" H41 HPT"; $resn_exch{"2H11 HPT"}="H112 HPT";
$resn_exch{"1H11 HPT"}="H111 HPT"; $resn_exch{"2H12 HPT"}="H122 HPT"; $resn_exch{"1H12 HPT"}="H121 HPT";
$resn_exch{"2H13 HPT"}="H132 HPT"; $resn_exch{"1H13 HPT"}="H131 HPT"; $resn_exch{"1H11  HQ"}="H111  HQ";
$resn_exch{"2H11  HQ"}="H112  HQ"; $resn_exch{"1H12  HQ"}="H121  HQ"; $resn_exch{"2H12  HQ"}="H122  HQ";
$resn_exch{"1H13  HQ"}="H131  HQ"; $resn_exch{"2H13  HQ"}="H132  HQ"; $resn_exch{"1H14  HQ"}="H141  HQ";
$resn_exch{"2H14  HQ"}="H142  HQ"; $resn_exch{"1H15  HQ"}="H151  HQ"; $resn_exch{"2H15  HQ"}="H152  HQ";
$resn_exch{"1H16  HQ"}="H161  HQ"; $resn_exch{"2H16  HQ"}="H162  HQ"; $resn_exch{"1H17  HQ"}="H171  HQ";
$resn_exch{"2H17  HQ"}="H172  HQ"; $resn_exch{"3H17  HQ"}="H173  HQ"; $resn_exch{"1H9  HQC"}=" H91 HQC";
$resn_exch{"2H9  HQC"}=" H92 HQC"; $resn_exch{"1H12 HQC"}="H121 HQC"; $resn_exch{"2H12 HQC"}="H122 HQC";
$resn_exch{"1H13 HQC"}="H131 HQC"; $resn_exch{"2H13 HQC"}="H132 HQC"; $resn_exch{"1H6  HQC"}=" H61 HQC";
$resn_exch{"2H6  HQC"}=" H62 HQC"; $resn_exch{"1H16 HQC"}="H161 HQC"; $resn_exch{"2H16 HQC"}="H162 HQC";
$resn_exch{"1H7  HQL"}=" H71 HQL"; $resn_exch{"2H7  HQL"}=" H72 HQL"; $resn_exch{"1H8  HQL"}=" H81 HQL";
$resn_exch{"2H8  HQL"}=" H82 HQL"; $resn_exch{"1H9  HQL"}=" H91 HQL"; $resn_exch{"2H9  HQL"}=" H92 HQL";
$resn_exch{"1H10 HQL"}="H101 HQL"; $resn_exch{"2H10 HQL"}="H102 HQL"; $resn_exch{"1H11 HQL"}="H111 HQL";
$resn_exch{"2H11 HQL"}="H112 HQL"; $resn_exch{"1H13 HQL"}="H131 HQL"; $resn_exch{"2H13 HQL"}="H132 HQL";
$resn_exch{"1H14 HQL"}="H141 HQL"; $resn_exch{"2H14 HQL"}="H142 HQL"; $resn_exch{"1HO1 HQO"}="HO11 HQO";
$resn_exch{"1H11 HQO"}="H111 HQO"; $resn_exch{"2H11 HQO"}="H112 HQO"; $resn_exch{"1H12 HQO"}="H121 HQO";
$resn_exch{"2H12 HQO"}="H122 HQO"; $resn_exch{"1H13 HQO"}="H131 HQO"; $resn_exch{"2H13 HQO"}="H132 HQO";
$resn_exch{"1H14 HQO"}="H141 HQO"; $resn_exch{"2H14 HQO"}="H142 HQO"; $resn_exch{"1H15 HQO"}="H151 HQO";
$resn_exch{"2H15 HQO"}="H152 HQO"; $resn_exch{"1H16 HQO"}="H161 HQO"; $resn_exch{"2H16 HQO"}="H162 HQO";
$resn_exch{"1H17 HQO"}="H171 HQO"; $resn_exch{"2H17 HQO"}="H172 HQO"; $resn_exch{"3H17 HQO"}="H173 HQO";
$resn_exch{"1HAL HQP"}="HAL1 HQP"; $resn_exch{"2HAL HQP"}="HAL2 HQP"; $resn_exch{"1HAM HQP"}="HAM1 HQP";
$resn_exch{"2HAM HQP"}="HAM2 HQP"; $resn_exch{"1H7  HQQ"}=" H71 HQQ"; $resn_exch{"2H7  HQQ"}=" H72 HQQ";
$resn_exch{"3H7  HQQ"}=" H73 HQQ"; $resn_exch{"1H7  HR1"}=" H71 HR1"; $resn_exch{"2H7  HR1"}=" H72 HR1";
$resn_exch{"1H8  HR1"}=" H81 HR1"; $resn_exch{"2H8  HR1"}=" H82 HR1"; $resn_exch{"1H10 HR1"}="H101 HR1";
$resn_exch{"2H10 HR1"}="H102 HR1"; $resn_exch{"1H13 HR1"}="H131 HR1"; $resn_exch{"2H13 HR1"}="H132 HR1";
$resn_exch{"3H13 HR1"}="H133 HR1"; $resn_exch{"1H14 HR1"}="H141 HR1"; $resn_exch{"2H14 HR1"}="H142 HR1";
$resn_exch{"3H14 HR1"}="H143 HR1"; $resn_exch{"1H35 HR1"}="H351 HR1"; $resn_exch{"2H35 HR1"}="H352 HR1";
$resn_exch{"1H6  HR2"}=" H61 HR2"; $resn_exch{"2H6  HR2"}=" H62 HR2"; $resn_exch{"1H20 HR2"}="H201 HR2";
$resn_exch{"2H20 HR2"}="H202 HR2"; $resn_exch{"1H22 HR2"}="H221 HR2"; $resn_exch{"2H22 HR2"}="H222 HR2";
$resn_exch{"1H24 HR2"}="H241 HR2"; $resn_exch{"2H24 HR2"}="H242 HR2"; $resn_exch{"1H19 HR2"}="H191 HR2";
$resn_exch{"2H19 HR2"}="H192 HR2"; $resn_exch{"1H12 HR2"}="H121 HR2"; $resn_exch{"2H12 HR2"}="H122 HR2";
$resn_exch{"3H12 HR2"}="H123 HR2"; $resn_exch{"1H11 HR2"}="H111 HR2"; $resn_exch{"2H11 HR2"}="H112 HR2";
$resn_exch{"3H11 HR2"}="H113 HR2"; $resn_exch{"1H2  HR2"}="H21A HR2"; $resn_exch{"2H2  HR2"}=" H22 HR2";
$resn_exch{"1H16 HR2"}="H161 HR2"; $resn_exch{"2H16 HR2"}="H162 HR2"; $resn_exch{"1HP5 HRB"}="HP51 HRB";
$resn_exch{"2HP5 HRB"}="HP52 HRB"; $resn_exch{"3HP5 HRB"}="HP53 HRB"; $resn_exch{"1HP4 HRB"}="HP41 HRB";
$resn_exch{"2HP4 HRB"}="HP42 HRB"; $resn_exch{"1HP3 HRB"}="HP31 HRB"; $resn_exch{"2HP3 HRB"}="HP32 HRB";
$resn_exch{"1HP2 HRB"}="HP21 HRB"; $resn_exch{"2HP2 HRB"}="HP22 HRB"; $resn_exch{"1HP1 HRB"}="HP11 HRB";
$resn_exch{"2HP1 HRB"}="HP12 HRB"; $resn_exch{"1H3  HRB"}=" H31 HRB"; $resn_exch{"2H3  HRB"}=" H32 HRB";
$resn_exch{"1HB  HRB"}=" HB1 HRB"; $resn_exch{"2HB  HRB"}=" HB2 HRB"; $resn_exch{"1HA2 HRB"}="HA21 HRB";
$resn_exch{"2HA2 HRB"}="HA22 HRB"; $resn_exch{"3HA2 HRB"}="HA23 HRB"; $resn_exch{"1HN  HRG"}="HN1A HRG";
$resn_exch{"2HN  HRG"}=" HN2 HRG"; $resn_exch{"1HB  HRG"}=" HB1 HRG"; $resn_exch{"2HB  HRG"}=" HB2 HRG";
$resn_exch{"1HG  HRG"}=" HG1 HRG"; $resn_exch{"2HG  HRG"}=" HG2 HRG"; $resn_exch{"1HG' HRG"}="HG'1 HRG";
$resn_exch{"2HG' HRG"}="HG'2 HRG"; $resn_exch{"1HD  HRG"}=" HD1 HRG"; $resn_exch{"2HD  HRG"}=" HD2 HRG";
$resn_exch{"1HN2 HRG"}="HN21 HRG"; $resn_exch{"2HN2 HRG"}="HN22 HRG"; $resn_exch{"1HCH HRI"}="HCH1 HRI";
$resn_exch{"2HCH HRI"}="HCH2 HRI"; $resn_exch{"1HCF HRI"}="HCF1 HRI"; $resn_exch{"1HI1 HRI"}="HI11 HRI";
$resn_exch{"2HI1 HRI"}="HI12 HRI"; $resn_exch{"3HI1 HRI"}="HI13 HRI"; $resn_exch{"1HI2 HRI"}="HI21 HRI";
$resn_exch{"2HI2 HRI"}="HI22 HRI"; $resn_exch{"3HI2 HRI"}="HI23 HRI"; $resn_exch{"1HAB HRM"}="HAB1 HRM";
$resn_exch{"2HAB HRM"}="HAB2 HRM"; $resn_exch{"3HAB HRM"}="HAB3 HRM"; $resn_exch{"1HAA HRM"}="HAA1 HRM";
$resn_exch{"2HAA HRM"}="HAA2 HRM"; $resn_exch{"3HAA HRM"}="HAA3 HRM"; $resn_exch{"1HN  HRP"}=" H   HRP";
$resn_exch{"2HN  HRP"}=" H2  HRP"; $resn_exch{"1HB  HRP"}=" HB2 HRP"; $resn_exch{"2HB  HRP"}=" HB3 HRP";
$resn_exch{" HO1 HRP"}=" HXT HRP"; $resn_exch{"1HB  HSA"}=" HB1 HSA"; $resn_exch{"2HB  HSA"}=" HB2 HSA";
$resn_exch{"1H4  HSA"}=" H41 HSA"; $resn_exch{"2H4  HSA"}=" H42 HSA"; $resn_exch{"2HO  HSA"}=" HO2 HSA";
$resn_exch{"3HO  HSA"}=" HO3 HSA"; $resn_exch{"1HN  HSA"}=" HN1 HSA"; $resn_exch{"2HN  HSA"}=" HN2 HSA";
$resn_exch{"1H3  HSE"}=" H31 HSE"; $resn_exch{"2H3  HSE"}=" H32 HSE"; $resn_exch{"1H4  HSE"}=" H41 HSE";
$resn_exch{"2H4  HSE"}=" H42 HSE"; $resn_exch{"1H1  HSF"}=" H11 HSF"; $resn_exch{"2H1  HSF"}=" H12 HSF";
$resn_exch{"1H2  HSF"}=" H21 HSF"; $resn_exch{"2H2  HSF"}=" H22 HSF"; $resn_exch{"1H3  HSF"}=" H31 HSF";
$resn_exch{"2H3  HSF"}=" H32 HSF"; $resn_exch{"1H4  HSF"}=" H41 HSF"; $resn_exch{"2H4  HSF"}=" H42 HSF";
$resn_exch{"1H5  HSF"}=" H51 HSF"; $resn_exch{"2H5  HSF"}=" H52 HSF"; $resn_exch{"1H6  HSF"}=" H61 HSF";
$resn_exch{"2H6  HSF"}=" H62 HSF"; $resn_exch{"1H7  HSF"}=" H71 HSF"; $resn_exch{"2H7  HSF"}=" H72 HSF";
$resn_exch{"1H8  HSF"}=" H81 HSF"; $resn_exch{"2H8  HSF"}=" H82 HSF"; $resn_exch{"1H9  HSF"}=" H91 HSF";
$resn_exch{"2H9  HSF"}=" H92 HSF"; $resn_exch{"1H10 HSF"}="H101 HSF"; $resn_exch{"2H10 HSF"}="H102 HSF";
$resn_exch{"1H11 HSF"}="H111 HSF"; $resn_exch{"2H11 HSF"}="H112 HSF"; $resn_exch{"1H12 HSF"}="H121 HSF";
$resn_exch{"2H12 HSF"}="H122 HSF"; $resn_exch{"1H13 HSF"}="H131 HSF"; $resn_exch{"2H13 HSF"}="H132 HSF";
$resn_exch{"1H14 HSF"}="H141 HSF"; $resn_exch{"2H14 HSF"}="H142 HSF"; $resn_exch{"1H15 HSF"}="H151 HSF";
$resn_exch{"2H15 HSF"}="H152 HSF"; $resn_exch{"1H16 HSF"}="H161 HSF"; $resn_exch{"2H16 HSF"}="H162 HSF";
$resn_exch{"3H16 HSF"}="H163 HSF"; $resn_exch{"1H60 HSI"}="H601 HSI"; $resn_exch{"2H60 HSI"}="H602 HSI";
$resn_exch{"3H60 HSI"}="H603 HSI"; $resn_exch{"1H56 HSI"}="H561 HSI"; $resn_exch{"2H56 HSI"}="H562 HSI";
$resn_exch{"1H52 HSI"}="H521 HSI"; $resn_exch{"2H52 HSI"}="H522 HSI"; $resn_exch{"1H48 HSI"}="H481 HSI";
$resn_exch{"2H48 HSI"}="H482 HSI"; $resn_exch{"1H44 HSI"}="H441 HSI"; $resn_exch{"2H44 HSI"}="H442 HSI";
$resn_exch{"1H40 HSI"}="H401 HSI"; $resn_exch{"2H40 HSI"}="H402 HSI"; $resn_exch{"1H5  HSI"}=" H51 HSI";
$resn_exch{"2H5  HSI"}=" H52 HSI"; $resn_exch{" HN2 HSL"}=" H2  HSL"; $resn_exch{"1HB  HSL"}=" HB2 HSL";
$resn_exch{"2HB  HSL"}=" HB3 HSL"; $resn_exch{"1HG  HSL"}=" HG2 HSL"; $resn_exch{"2HG  HSL"}=" HG3 HSL";
$resn_exch{"1H   HSM"}=" H1  HSM"; $resn_exch{"2H   HSM"}=" H2  HSM"; $resn_exch{"1HA  HSM"}=" HA1 HSM";
$resn_exch{"2HA  HSM"}=" HA2 HSM"; $resn_exch{"1HB  HSM"}=" HB1 HSM"; $resn_exch{"2HB  HSM"}=" HB2 HSM";
$resn_exch{"1HN  HSO"}=" HN1 HSO"; $resn_exch{"2HN  HSO"}=" HN2 HSO"; $resn_exch{"1HB  HSO"}=" HB1 HSO";
$resn_exch{"2HB  HSO"}=" HB2 HSO"; $resn_exch{"1H7  HSP"}=" H71 HSP"; $resn_exch{"2H7  HSP"}=" H72 HSP";
$resn_exch{"1H8  HSP"}=" H81 HSP"; $resn_exch{"2H8  HSP"}=" H82 HSP"; $resn_exch{"1H9  HSP"}=" H91 HSP";
$resn_exch{"2H9  HSP"}=" H92 HSP"; $resn_exch{"1H10 HSP"}="H101 HSP"; $resn_exch{"2H10 HSP"}="H102 HSP";
$resn_exch{"1HB  HSS"}=" HB1 HSS"; $resn_exch{"2HB  HSS"}=" HB2 HSS"; $resn_exch{"1HN  HSS"}=" HN1 HSS";
$resn_exch{"2HN  HSS"}=" HN2 HSS"; $resn_exch{"1H5* HSS"}="H5'1 HSS"; $resn_exch{"2H5* HSS"}="H5'2 HSS";
$resn_exch{" H4* HSS"}=" H4' HSS"; $resn_exch{" H3* HSS"}=" H3' HSS"; $resn_exch{"HO3* HSS"}="HO3' HSS";
$resn_exch{" H2* HSS"}=" H2' HSS"; $resn_exch{"HO2* HSS"}="HO2' HSS"; $resn_exch{" H1* HSS"}=" H1' HSS";
$resn_exch{"1HN6 HSS"}="HN61 HSS"; $resn_exch{"2HN6 HSS"}="HN62 HSS"; $resn_exch{"1H1  HST"}=" H11 HST";
$resn_exch{"2H1  HST"}=" H12 HST"; $resn_exch{"3H1  HST"}=" H13 HST"; $resn_exch{"1H2  HST"}=" H21 HST";
$resn_exch{"2H2  HST"}=" H22 HST"; $resn_exch{"1H3  HST"}=" H31 HST"; $resn_exch{"2H3  HST"}=" H32 HST";
$resn_exch{"1H4  HST"}=" H41 HST"; $resn_exch{"2H4  HST"}=" H42 HST"; $resn_exch{"1H5  HST"}=" H51 HST";
$resn_exch{"2H5  HST"}=" H52 HST"; $resn_exch{"1H7  HST"}=" H71 HST"; $resn_exch{"2H7  HST"}=" H72 HST";
$resn_exch{"1H8  HST"}=" H81 HST"; $resn_exch{"2H8  HST"}=" H82 HST"; $resn_exch{"1H9  HST"}=" H91 HST";
$resn_exch{"2H9  HST"}=" H92 HST"; $resn_exch{"1H10 HST"}="H101 HST"; $resn_exch{"2H10 HST"}="H102 HST";
$resn_exch{"1H11 HST"}="H111 HST"; $resn_exch{"2H11 HST"}="H112 HST"; $resn_exch{"3H11 HST"}="H113 HST";
$resn_exch{"1HN1 HSW"}="HN11 HSW"; $resn_exch{"2HN1 HSW"}="HN12 HSW"; $resn_exch{"1H21  HT"}="H211  HT";
$resn_exch{"2H21  HT"}="H212  HT"; $resn_exch{"1H22  HT"}="H221  HT"; $resn_exch{"2H22  HT"}="H222  HT";
$resn_exch{"1H23  HT"}="H231  HT"; $resn_exch{"2H23  HT"}="H232  HT"; $resn_exch{"1H24  HT"}="H241  HT";
$resn_exch{"2H24  HT"}="H242  HT"; $resn_exch{"3H25  HT"}="H253  HT"; $resn_exch{"2H25  HT"}="H252  HT";
$resn_exch{"1H25  HT"}="H251  HT"; $resn_exch{"1H21 HT1"}="H211 HT1"; $resn_exch{"2H21 HT1"}="H212 HT1";
$resn_exch{"1H22 HT1"}="H221 HT1"; $resn_exch{"2H22 HT1"}="H222 HT1"; $resn_exch{"1H23 HT1"}="H231 HT1";
$resn_exch{"2H23 HT1"}="H232 HT1"; $resn_exch{"1H24 HT1"}="H241 HT1"; $resn_exch{"2H24 HT1"}="H242 HT1";
$resn_exch{"1H25 HT1"}="H251 HT1"; $resn_exch{"2H25 HT1"}="H252 HT1"; $resn_exch{"3H25 HT1"}="H253 HT1";
$resn_exch{"1H26 HT1"}="H261 HT1"; $resn_exch{"2H26 HT1"}="H262 HT1"; $resn_exch{"1H27 HT1"}="H271 HT1";
$resn_exch{"2H27 HT1"}="H272 HT1"; $resn_exch{"3H27 HT1"}="H273 HT1"; $resn_exch{"1HN1 HT2"}="HN11 HT2";
$resn_exch{"2HN1 HT2"}="HN12 HT2"; $resn_exch{"1HN7 HT2"}="HN71 HT2"; $resn_exch{"2HN7 HT2"}="HN72 HT2";
$resn_exch{"1H14 HT5"}="H141 HT5"; $resn_exch{"2H14 HT5"}="H142 HT5"; $resn_exch{"1H13 HT5"}="H131 HT5";
$resn_exch{"2H13 HT5"}="H132 HT5"; $resn_exch{"1H9C HT5"}="H9C1 HT5"; $resn_exch{"2H9C HT5"}="H9C2 HT5";
$resn_exch{"1H2C HT5"}="H2C1 HT5"; $resn_exch{"2H2C HT5"}="H2C2 HT5"; $resn_exch{"1H1C HT5"}="H1C1 HT5";
$resn_exch{"2H1C HT5"}="H1C2 HT5"; $resn_exch{"1H50 HT5"}="H501 HT5"; $resn_exch{"2H50 HT5"}="H502 HT5";
$resn_exch{"1H3C HT5"}="H3C1 HT5"; $resn_exch{"2H3C HT5"}="H3C2 HT5"; $resn_exch{"1H4C HT5"}="H4C1 HT5";
$resn_exch{"2H4C HT5"}="H4C2 HT5"; $resn_exch{"1H5C HT5"}="H5C1 HT5"; $resn_exch{"2H5C HT5"}="H5C2 HT5";
$resn_exch{"1H6C HT5"}="H6C1 HT5"; $resn_exch{"2H6C HT5"}="H6C2 HT5"; $resn_exch{"1H8C HT5"}="H8C1 HT5";
$resn_exch{"2H8C HT5"}="H8C2 HT5"; $resn_exch{"3H8C HT5"}="H8C3 HT5"; $resn_exch{"1H2  HTA"}=" H21 HTA";
$resn_exch{"2H2  HTA"}=" H22 HTA"; $resn_exch{"1H5  HTA"}=" H51 HTA"; $resn_exch{"2H5  HTA"}=" H52 HTA";
$resn_exch{"1H7  HTA"}=" H71 HTA"; $resn_exch{"2H7  HTA"}=" H72 HTA"; $resn_exch{"3H7  HTA"}=" H73 HTA";
$resn_exch{"1H8  HTA"}=" H81 HTA"; $resn_exch{"2H8  HTA"}=" H82 HTA"; $resn_exch{"3H8  HTA"}=" H83 HTA";
$resn_exch{"1H11 HTA"}="H111 HTA"; $resn_exch{"2H11 HTA"}="H112 HTA"; $resn_exch{"1H18 HTA"}="H181 HTA";
$resn_exch{"2H18 HTA"}="H182 HTA"; $resn_exch{"3H18 HTA"}="H183 HTA"; $resn_exch{"1H19 HTA"}="H191 HTA";
$resn_exch{"2H19 HTA"}="H192 HTA"; $resn_exch{"3H19 HTA"}="H193 HTA"; $resn_exch{"1H8  HTF"}=" H81 HTF";
$resn_exch{"2H8  HTF"}=" H82 HTF"; $resn_exch{"3H8  HTF"}=" H83 HTF"; $resn_exch{"2H7  HTF"}=" H72 HTF";
$resn_exch{"3H7  HTF"}=" H73 HTF"; $resn_exch{"2H6  HTF"}=" H62 HTF"; $resn_exch{"3H6  HTF"}=" H63 HTF";
$resn_exch{"2H5  HTF"}=" H52 HTF"; $resn_exch{"3H5  HTF"}=" H53 HTF"; $resn_exch{"2H4  HTF"}=" H42 HTF";
$resn_exch{"3H4  HTF"}=" H43 HTF"; $resn_exch{"2H3  HTF"}=" H32 HTF"; $resn_exch{"3H3  HTF"}=" H33 HTF";
$resn_exch{"2H2  HTF"}=" H22 HTF"; $resn_exch{"3H2  HTF"}=" H23 HTF"; $resn_exch{"2HG  HTF"}=" HG2 HTF";
$resn_exch{"3HG  HTF"}=" HG3 HTF"; $resn_exch{"2HB  HTF"}=" HB2 HTF"; $resn_exch{"3HB  HTF"}=" HB3 HTF";
$resn_exch{"1H6  HTG"}=" H61 HTG"; $resn_exch{"2H6  HTG"}=" H62 HTG"; $resn_exch{"1H1' HTG"}="H1'1 HTG";
$resn_exch{"2H1' HTG"}="H1'2 HTG"; $resn_exch{"1H2' HTG"}="H2'1 HTG"; $resn_exch{"2H2' HTG"}="H2'2 HTG";
$resn_exch{"1H3' HTG"}="H3'1 HTG"; $resn_exch{"2H3' HTG"}="H3'2 HTG"; $resn_exch{"1H4' HTG"}="H4'1 HTG";
$resn_exch{"2H4' HTG"}="H4'2 HTG"; $resn_exch{"1H5' HTG"}="H5'1 HTG"; $resn_exch{"2H5' HTG"}="H5'2 HTG";
$resn_exch{"1H6' HTG"}="H6'1 HTG"; $resn_exch{"2H6' HTG"}="H6'2 HTG"; $resn_exch{"1H7' HTG"}="H7'1 HTG";
$resn_exch{"2H7' HTG"}="H7'2 HTG"; $resn_exch{"3H7' HTG"}="H7'3 HTG"; $resn_exch{"1HN  HTI"}=" H   HTI";
$resn_exch{"2HN  HTI"}=" H2  HTI"; $resn_exch{"1HB  HTI"}=" HB2 HTI"; $resn_exch{"2HB  HTI"}=" HB3 HTI";
$resn_exch{"1HAJ HTI"}="HAJ1 HTI"; $resn_exch{"2HAJ HTI"}="HAJ2 HTI"; $resn_exch{"1HAL HTI"}="HAL1 HTI";
$resn_exch{"2HAL HTI"}="HAL2 HTI"; $resn_exch{"1H2A HTL"}="H2A1 HTL"; $resn_exch{"2H2A HTL"}="H2A2 HTL";
$resn_exch{"3H2A HTL"}="H2A3 HTL"; $resn_exch{"1H4* HTL"}="H4'1 HTL"; $resn_exch{"2H4* HTL"}="H4'2 HTL";
$resn_exch{" H6* HTL"}=" H6' HTL"; $resn_exch{"1H35 HTL"}="H351 HTL"; $resn_exch{"2H35 HTL"}="H352 HTL";
$resn_exch{"1H4A HTL"}="H4A1 HTL"; $resn_exch{"2H4A HTL"}="H4A2 HTL"; $resn_exch{"3H4A HTL"}="H4A3 HTL";
$resn_exch{"1H5A HTL"}="H5A1 HTL"; $resn_exch{"2H5A HTL"}="H5A2 HTL"; $resn_exch{"1H5B HTL"}="H5B1 HTL";
$resn_exch{"2H5B HTL"}="H5B2 HTL"; $resn_exch{"OH22 HTL"}="H22O HTL"; $resn_exch{"OH23 HTL"}="H23O HTL";
$resn_exch{"1H3' HTL"}="H3'1 HTL"; $resn_exch{"2H3' HTL"}="H3'2 HTL"; $resn_exch{"3H3' HTL"}="H3'3 HTL";
$resn_exch{"1H7  HTM"}=" H71 HTM"; $resn_exch{"2H7  HTM"}=" H72 HTM"; $resn_exch{"1H8  HTM"}=" H81 HTM";
$resn_exch{"2H8  HTM"}=" H82 HTM"; $resn_exch{"1H9  HTM"}=" H91 HTM"; $resn_exch{"2H9  HTM"}=" H92 HTM";
$resn_exch{"1H11 HTM"}="H111 HTM"; $resn_exch{"2H11 HTM"}="H112 HTM"; $resn_exch{"3H11 HTM"}="H113 HTM";
$resn_exch{"1H1  HTO"}=" H11 HTO"; $resn_exch{"2H1  HTO"}=" H12 HTO"; $resn_exch{"1H4  HTO"}=" H41 HTO";
$resn_exch{"2H4  HTO"}=" H42 HTO"; $resn_exch{"1H5  HTO"}=" H51 HTO"; $resn_exch{"2H5  HTO"}=" H52 HTO";
$resn_exch{"1H6  HTO"}=" H61 HTO"; $resn_exch{"2H6  HTO"}=" H62 HTO"; $resn_exch{"1H7  HTO"}=" H71 HTO";
$resn_exch{"2H7  HTO"}=" H72 HTO"; $resn_exch{"3H7  HTO"}=" H73 HTO"; $resn_exch{"1H6  HTP"}=" H61 HTP";
$resn_exch{"2H6  HTP"}=" H62 HTP"; $resn_exch{"1H1  HTQ"}=" H11 HTQ"; $resn_exch{"1H2  HTQ"}=" H21 HTQ";
$resn_exch{"2H2  HTQ"}=" H22 HTQ"; $resn_exch{"1H3  HTQ"}=" H31 HTQ"; $resn_exch{"1H4  HTQ"}=" H41 HTQ";
$resn_exch{"2H4  HTQ"}=" H42 HTQ"; $resn_exch{"1H5  HTQ"}=" H51 HTQ"; $resn_exch{"1H6  HTQ"}=" H61 HTQ";
$resn_exch{"2H6  HTQ"}=" H62 HTQ"; $resn_exch{"1H7  HTQ"}=" H71 HTQ"; $resn_exch{"2H7  HTQ"}=" H72 HTQ";
$resn_exch{"1H9  HTQ"}=" H91 HTQ"; $resn_exch{"2H9  HTQ"}=" H92 HTQ"; $resn_exch{"3H9  HTQ"}=" H93 HTQ";
$resn_exch{"1H13 HTQ"}="H131 HTQ"; $resn_exch{"1H15 HTQ"}="H151 HTQ"; $resn_exch{"1H16 HTQ"}="H161 HTQ";
$resn_exch{"1H17 HTQ"}="H171 HTQ"; $resn_exch{"1H18 HTQ"}="H181 HTQ"; $resn_exch{"1H19 HTQ"}="H191 HTQ";
$resn_exch{"1H20 HTQ"}="H201 HTQ"; $resn_exch{"1HN  HTR"}=" H   HTR"; $resn_exch{"2HN  HTR"}=" H2  HTR";
$resn_exch{" HO  HTR"}=" HH  HTR"; $resn_exch{"1H5  HU1"}=" H51 HU1"; $resn_exch{"2H5  HU1"}=" H52 HU1";
$resn_exch{"3H5  HU1"}=" H53 HU1"; $resn_exch{"1H6  HU1"}=" H61 HU1"; $resn_exch{"2H6  HU1"}=" H62 HU1";
$resn_exch{"3H6  HU1"}=" H63 HU1"; $resn_exch{"1H7  HU1"}=" H71 HU1"; $resn_exch{"2H7  HU1"}=" H72 HU1";
$resn_exch{"3H7  HU1"}=" H73 HU1"; $resn_exch{"1H11 HU1"}="H111 HU1"; $resn_exch{"2H11 HU1"}="H112 HU1";
$resn_exch{"1H12 HU1"}="H121 HU1"; $resn_exch{"2H12 HU1"}="H122 HU1"; $resn_exch{"1H13 HU1"}="H131 HU1";
$resn_exch{"2H13 HU1"}="H132 HU1"; $resn_exch{"1H14 HU1"}="H141 HU1"; $resn_exch{"2H14 HU1"}="H142 HU1";
$resn_exch{"1H15 HU1"}="H151 HU1"; $resn_exch{"2H15 HU1"}="H152 HU1"; $resn_exch{"1H19 HU1"}="H191 HU1";
$resn_exch{"2H19 HU1"}="H192 HU1"; $resn_exch{"1H24 HU1"}="H241 HU1"; $resn_exch{"2H24 HU1"}="H242 HU1";
$resn_exch{"3H24 HU1"}="H243 HU1"; $resn_exch{"1H25 HU1"}="H251 HU1"; $resn_exch{"2H25 HU1"}="H252 HU1";
$resn_exch{"3H25 HU1"}="H253 HU1"; $resn_exch{"1H30 HU1"}="H301 HU1"; $resn_exch{"2H30 HU1"}="H302 HU1";
$resn_exch{"1H32 HU1"}="H321 HU1"; $resn_exch{"2H32 HU1"}="H322 HU1"; $resn_exch{"1H33 HU1"}="H331 HU1";
$resn_exch{"2H33 HU1"}="H332 HU1"; $resn_exch{"1H34 HU1"}="H341 HU1"; $resn_exch{"2H34 HU1"}="H342 HU1";
$resn_exch{"1H38 HU1"}="H381 HU1"; $resn_exch{"2H38 HU1"}="H382 HU1"; $resn_exch{"1H3  HU2"}=" H31 HU2";
$resn_exch{"2H3  HU2"}=" H32 HU2"; $resn_exch{"1H4  HU2"}="H41A HU2"; $resn_exch{"1H5  HU2"}=" H51 HU2";
$resn_exch{"1H6  HU2"}=" H61 HU2"; $resn_exch{"2H6  HU2"}=" H62 HU2"; $resn_exch{"1H8  HU2"}=" H81 HU2";
$resn_exch{"2H8  HU2"}=" H82 HU2"; $resn_exch{"3H8  HU2"}=" H83 HU2"; $resn_exch{"1H9  HU2"}=" H91 HU2";
$resn_exch{"2H9  HU2"}=" H92 HU2"; $resn_exch{"3H9  HU2"}=" H93 HU2"; $resn_exch{"1H11 HU2"}="H111 HU2";
$resn_exch{"2H11 HU2"}="H112 HU2"; $resn_exch{"3H11 HU2"}="H113 HU2"; $resn_exch{"1H12 HU2"}="H121 HU2";
$resn_exch{"2H12 HU2"}="H122 HU2"; $resn_exch{"3H12 HU2"}="H123 HU2"; $resn_exch{"1H16 HU2"}="H161 HU2";
$resn_exch{"2H16 HU2"}="H162 HU2"; $resn_exch{"1H18 HU2"}="H181 HU2"; $resn_exch{"2H18 HU2"}="H182 HU2";
$resn_exch{"3H18 HU2"}="H183 HU2"; $resn_exch{"1H19 HU2"}="H191 HU2"; $resn_exch{"2H19 HU2"}="H192 HU2";
$resn_exch{"3H19 HU2"}="H193 HU2"; $resn_exch{"1H24 HU2"}="H241 HU2"; $resn_exch{"2H24 HU2"}="H242 HU2";
$resn_exch{"1H25 HU2"}="H251 HU2"; $resn_exch{"2H25 HU2"}="H252 HU2"; $resn_exch{"1H26 HU2"}="H261 HU2";
$resn_exch{"2H26 HU2"}="H262 HU2"; $resn_exch{"3H26 HU2"}="H263 HU2"; $resn_exch{"1H31 HU2"}="H311 HU2";
$resn_exch{"2H31 HU2"}="H312 HU2"; $resn_exch{"1H5  HU4"}=" H51 HU4"; $resn_exch{"2H5  HU4"}=" H52 HU4";
$resn_exch{"3H5  HU4"}=" H53 HU4"; $resn_exch{"1H6  HU4"}=" H61 HU4"; $resn_exch{"2H6  HU4"}=" H62 HU4";
$resn_exch{"3H6  HU4"}=" H63 HU4"; $resn_exch{"1H7  HU4"}=" H71 HU4"; $resn_exch{"2H7  HU4"}=" H72 HU4";
$resn_exch{"3H7  HU4"}=" H73 HU4"; $resn_exch{"1H11 HU4"}="H111 HU4"; $resn_exch{"2H11 HU4"}="H112 HU4";
$resn_exch{"1H12 HU4"}="H121 HU4"; $resn_exch{"2H12 HU4"}="H122 HU4"; $resn_exch{"1H13 HU4"}="H131 HU4";
$resn_exch{"2H13 HU4"}="H132 HU4"; $resn_exch{"1H14 HU4"}="H141 HU4"; $resn_exch{"2H14 HU4"}="H142 HU4";
$resn_exch{"1H15 HU4"}="H151 HU4"; $resn_exch{"2H15 HU4"}="H152 HU4"; $resn_exch{"1H19 HU4"}="H191 HU4";
$resn_exch{"2H19 HU4"}="H192 HU4"; $resn_exch{"1H25 HU4"}="H251 HU4"; $resn_exch{"2H25 HU4"}="H252 HU4";
$resn_exch{"3H25 HU4"}="H253 HU4"; $resn_exch{"1H26 HU4"}="H261 HU4"; $resn_exch{"2H26 HU4"}="H262 HU4";
$resn_exch{"3H26 HU4"}="H263 HU4"; $resn_exch{"1H31 HU4"}="H311 HU4"; $resn_exch{"2H31 HU4"}="H312 HU4";
$resn_exch{"1H33 HU4"}="H331 HU4"; $resn_exch{"2H33 HU4"}="H332 HU4"; $resn_exch{"1H34 HU4"}="H341 HU4";
$resn_exch{"2H34 HU4"}="H342 HU4"; $resn_exch{"1H35 HU4"}="H351 HU4"; $resn_exch{"2H35 HU4"}="H352 HU4";
$resn_exch{"1H39 HU4"}="H391 HU4"; $resn_exch{"2H39 HU4"}="H392 HU4"; $resn_exch{"1H5  HU5"}=" H51 HU5";
$resn_exch{"2H5  HU5"}=" H52 HU5"; $resn_exch{"3H5  HU5"}=" H53 HU5"; $resn_exch{"1H6  HU5"}=" H61 HU5";
$resn_exch{"2H6  HU5"}=" H62 HU5"; $resn_exch{"3H6  HU5"}=" H63 HU5"; $resn_exch{"1H7  HU5"}=" H71 HU5";
$resn_exch{"2H7  HU5"}=" H72 HU5"; $resn_exch{"3H7  HU5"}=" H73 HU5"; $resn_exch{"1H11 HU5"}="H111 HU5";
$resn_exch{"2H11 HU5"}="H112 HU5"; $resn_exch{"3H11 HU5"}="H113 HU5"; $resn_exch{"1H12 HU5"}="H121 HU5";
$resn_exch{"2H12 HU5"}="H122 HU5"; $resn_exch{"3H12 HU5"}="H123 HU5"; $resn_exch{"1H13 HU5"}="H131 HU5";
$resn_exch{"2H13 HU5"}="H132 HU5"; $resn_exch{"3H13 HU5"}="H133 HU5"; $resn_exch{"1H17 HU5"}="H171 HU5";
$resn_exch{"2H17 HU5"}="H172 HU5"; $resn_exch{"1H22 HU5"}="H221 HU5"; $resn_exch{"2H22 HU5"}="H222 HU5";
$resn_exch{"3H22 HU5"}="H223 HU5"; $resn_exch{"1H23 HU5"}="H231 HU5"; $resn_exch{"2H23 HU5"}="H232 HU5";
$resn_exch{"3H23 HU5"}="H233 HU5"; $resn_exch{"1H28 HU5"}="H281 HU5"; $resn_exch{"2H28 HU5"}="H282 HU5";
$resn_exch{"1H30 HU5"}="H301 HU5"; $resn_exch{"2H30 HU5"}="H302 HU5"; $resn_exch{"1H31 HU5"}="H311 HU5";
$resn_exch{"2H31 HU5"}="H312 HU5"; $resn_exch{"1H32 HU5"}="H321 HU5"; $resn_exch{"2H32 HU5"}="H322 HU5";
$resn_exch{"1H36 HU5"}="H361 HU5"; $resn_exch{"2H36 HU5"}="H362 HU5"; $resn_exch{"1H6C HUB"}="H6C1 HUB";
$resn_exch{"2H6C HUB"}="H6C2 HUB"; $resn_exch{"1H9C HUB"}="H9C1 HUB"; $resn_exch{"2H9C HUB"}="H9C2 HUB";
$resn_exch{"1H10 HUB"}="H101 HUB"; $resn_exch{"2H10 HUB"}="H102 HUB"; $resn_exch{"1H13 HUB"}="H131 HUB";
$resn_exch{"2H13 HUB"}="H132 HUB"; $resn_exch{"1H15 HUB"}="H151 HUB"; $resn_exch{"2H15 HUB"}="H152 HUB";
$resn_exch{"3H15 HUB"}="H153 HUB"; $resn_exch{"1H90 HUB"}="H901 HUB"; $resn_exch{"2H90 HUB"}="H902 HUB";
$resn_exch{"1H5  HUD"}=" H51 HUD"; $resn_exch{"2H5  HUD"}=" H52 HUD"; $resn_exch{"3H5  HUD"}=" H53 HUD";
$resn_exch{"1H6  HUD"}=" H61 HUD"; $resn_exch{"2H6  HUD"}=" H62 HUD"; $resn_exch{"3H6  HUD"}=" H63 HUD";
$resn_exch{"1H7  HUD"}=" H71 HUD"; $resn_exch{"2H7  HUD"}=" H72 HUD"; $resn_exch{"3H7  HUD"}=" H73 HUD";
$resn_exch{"1H11 HUD"}="H111 HUD"; $resn_exch{"2H11 HUD"}="H112 HUD"; $resn_exch{"3H11 HUD"}="H113 HUD";
$resn_exch{"1H12 HUD"}="H121 HUD"; $resn_exch{"2H12 HUD"}="H122 HUD"; $resn_exch{"3H12 HUD"}="H123 HUD";
$resn_exch{"1H13 HUD"}="H131 HUD"; $resn_exch{"2H13 HUD"}="H132 HUD"; $resn_exch{"3H13 HUD"}="H133 HUD";
$resn_exch{"1H17 HUD"}="H171 HUD"; $resn_exch{"2H17 HUD"}="H172 HUD"; $resn_exch{"1H22 HUD"}="H221 HUD";
$resn_exch{"2H22 HUD"}="H222 HUD"; $resn_exch{"3H22 HUD"}="H223 HUD"; $resn_exch{"1H23 HUD"}="H231 HUD";
$resn_exch{"2H23 HUD"}="H232 HUD"; $resn_exch{"3H23 HUD"}="H233 HUD"; $resn_exch{"1H28 HUD"}="H281 HUD";
$resn_exch{"2H28 HUD"}="H282 HUD"; $resn_exch{"1H30 HUD"}="H301 HUD"; $resn_exch{"2H30 HUD"}="H302 HUD";
$resn_exch{"1H31 HUD"}="H311 HUD"; $resn_exch{"2H31 HUD"}="H312 HUD"; $resn_exch{"1H35 HUD"}="H351 HUD";
$resn_exch{"2H35 HUD"}="H352 HUD"; $resn_exch{"1H6  HUP"}=" H61 HUP"; $resn_exch{"2H6  HUP"}=" H62 HUP";
$resn_exch{"1H8  HUP"}=" H81 HUP"; $resn_exch{"2H8  HUP"}=" H82 HUP"; $resn_exch{"1H9  HUP"}=" H91 HUP";
$resn_exch{"2H9  HUP"}=" H92 HUP"; $resn_exch{"3H9  HUP"}=" H93 HUP"; $resn_exch{"1HN2 HUP"}="HN21 HUP";
$resn_exch{"2HN2 HUP"}="HN22 HUP"; $resn_exch{"1H13 HUP"}="H131 HUP"; $resn_exch{"2H13 HUP"}="H132 HUP";
$resn_exch{"1H15 HUP"}="H151 HUP"; $resn_exch{"2H15 HUP"}="H152 HUP"; $resn_exch{"3H15 HUP"}="H153 HUP";
$resn_exch{"1HN2 HUX"}="HN21 HUX"; $resn_exch{"2HN2 HUX"}="HN22 HUX"; $resn_exch{"1HC5 HUX"}="HC51 HUX";
$resn_exch{"2HC5 HUX"}="HC52 HUX"; $resn_exch{"1H12 HUX"}="H121 HUX"; $resn_exch{"2H12 HUX"}="H122 HUX";
$resn_exch{"1HC9 HUX"}="HC91 HUX"; $resn_exch{"2HC9 HUX"}="HC92 HUX"; $resn_exch{"1H18 HUX"}="H181 HUX";
$resn_exch{"2H18 HUX"}="H182 HUX"; $resn_exch{"1H10 HUX"}="H101 HUX"; $resn_exch{"2H10 HUX"}="H102 HUX";
$resn_exch{"3H10 HUX"}="H103 HUX"; $resn_exch{"1HN  HV5"}=" HN1 HV5"; $resn_exch{"2HN  HV5"}=" HN2 HV5";
$resn_exch{"1HG1 HV5"}="HG11 HV5"; $resn_exch{"2HG1 HV5"}="HG12 HV5"; $resn_exch{"3HG1 HV5"}="HG13 HV5";
$resn_exch{"1HG2 HV5"}="HG21 HV5"; $resn_exch{"2HG2 HV5"}="HG22 HV5"; $resn_exch{"3HG2 HV5"}="HG23 HV5";
$resn_exch{"1HG3 HV5"}="HG31 HV5"; $resn_exch{"2HG3 HV5"}="HG32 HV5"; $resn_exch{"3HG3 HV5"}="HG33 HV5";
$resn_exch{"1HN  HV7"}=" HN1 HV7"; $resn_exch{"2HN  HV7"}=" HN2 HV7"; $resn_exch{"1HB  HV7"}=" HB1 HV7";
$resn_exch{"2HB  HV7"}=" HB2 HV7"; $resn_exch{"1HM  HV7"}=" HM1 HV7"; $resn_exch{"2HM  HV7"}=" HM2 HV7";
$resn_exch{"3HM  HV7"}=" HM3 HV7"; $resn_exch{"1HN  HV8"}=" HN1 HV8"; $resn_exch{"2HN  HV8"}="HN2A HV8";
$resn_exch{"1HB1 HV8"}="HB11 HV8"; $resn_exch{"2HB1 HV8"}="HB12 HV8"; $resn_exch{"1HB2 HV8"}="HB21 HV8";
$resn_exch{"2HB2 HV8"}="HB22 HV8"; $resn_exch{"1HM2 HV8"}="HM21 HV8"; $resn_exch{"2HM2 HV8"}="HM22 HV8";
$resn_exch{"3HM2 HV8"}="HM23 HV8"; $resn_exch{"1H7  HWD"}=" H71 HWD"; $resn_exch{"2H7  HWD"}=" H72 HWD";
$resn_exch{"1H8  HWD"}=" H81 HWD"; $resn_exch{"2H8  HWD"}=" H82 HWD"; $resn_exch{"OH92 HWD"}="H92O HWD";
$resn_exch{"1H14 HWG"}="H141 HWG"; $resn_exch{"2H14 HWG"}="H142 HWG"; $resn_exch{"1H15 HWG"}="H151 HWG";
$resn_exch{"2H15 HWG"}="H152 HWG"; $resn_exch{"1H26 HWG"}="H261 HWG"; $resn_exch{"2H26 HWG"}="H262 HWG";
$resn_exch{"3H26 HWG"}="H263 HWG"; $resn_exch{"1H30 HWG"}="H301 HWG"; $resn_exch{"2H30 HWG"}="H302 HWG";
$resn_exch{"3H30 HWG"}="H303 HWG"; $resn_exch{"1H34 HWG"}="H341 HWG"; $resn_exch{"2H34 HWG"}="H342 HWG";
$resn_exch{"3H34 HWG"}="H343 HWG"; $resn_exch{"1H46 HWG"}="H461 HWG"; $resn_exch{"2H46 HWG"}="H462 HWG";
$resn_exch{"3H46 HWG"}="H463 HWG"; $resn_exch{"1H50 HWG"}="H501 HWG"; $resn_exch{"2H50 HWG"}="H502 HWG";
$resn_exch{"3H50 HWG"}="H503 HWG"; $resn_exch{"1H54 HWG"}="H541 HWG"; $resn_exch{"2H54 HWG"}="H542 HWG";
$resn_exch{"3H54 HWG"}="H543 HWG"; $resn_exch{"1H2  HXA"}=" H21 HXA"; $resn_exch{"2H2  HXA"}=" H22 HXA";
$resn_exch{"1H3  HXA"}=" H31 HXA"; $resn_exch{"2H3  HXA"}=" H32 HXA"; $resn_exch{"1H6  HXA"}=" H61 HXA";
$resn_exch{"2H6  HXA"}=" H62 HXA"; $resn_exch{"1H9  HXA"}=" H91 HXA"; $resn_exch{"2H9  HXA"}=" H92 HXA";
$resn_exch{"1H12 HXA"}="H121 HXA"; $resn_exch{"2H12 HXA"}="H122 HXA"; $resn_exch{"1H15 HXA"}="H151 HXA";
$resn_exch{"2H15 HXA"}="H152 HXA"; $resn_exch{"1H18 HXA"}="H181 HXA"; $resn_exch{"2H18 HXA"}="H182 HXA";
$resn_exch{"1H21 HXA"}="H211 HXA"; $resn_exch{"2H21 HXA"}="H212 HXA"; $resn_exch{"1H22 HXA"}="H221 HXA";
$resn_exch{"2H22 HXA"}="H222 HXA"; $resn_exch{"3H22 HXA"}="H223 HXA"; $resn_exch{"1HN6 HXC"}="HN61 HXC";
$resn_exch{"2HN6 HXC"}="HN62 HXC"; $resn_exch{"1H5' HXC"}="H5'1 HXC"; $resn_exch{"2H5' HXC"}="H5'2 HXC";
$resn_exch{"1HPB HXC"}="HPB1 HXC"; $resn_exch{"2HPB HXC"}="HPB2 HXC"; $resn_exch{"1HP9 HXC"}="HP91 HXC";
$resn_exch{"2HP9 HXC"}="HP92 HXC"; $resn_exch{"3HP9 HXC"}="HP93 HXC"; $resn_exch{"1HP8 HXC"}="HP81 HXC";
$resn_exch{"2HP8 HXC"}="HP82 HXC"; $resn_exch{"3HP8 HXC"}="HP83 HXC"; $resn_exch{"1HP5 HXC"}="HP51 HXC";
$resn_exch{"2HP5 HXC"}="HP52 HXC"; $resn_exch{"1HP4 HXC"}="HP41 HXC"; $resn_exch{"2HP4 HXC"}="HP42 HXC";
$resn_exch{"1HP2 HXC"}="HP21 HXC"; $resn_exch{"2HP2 HXC"}="HP22 HXC"; $resn_exch{"1HP1 HXC"}="HP11 HXC";
$resn_exch{"2HP1 HXC"}="HP12 HXC"; $resn_exch{"1HM2 HXC"}="HM21 HXC"; $resn_exch{"2HM2 HXC"}="HM22 HXC";
$resn_exch{"1HM3 HXC"}="HM31 HXC"; $resn_exch{"2HM3 HXC"}="HM32 HXC"; $resn_exch{"1HM4 HXC"}="HM41 HXC";
$resn_exch{"2HM4 HXC"}="HM42 HXC"; $resn_exch{"1HM5 HXC"}="HM51 HXC"; $resn_exch{"2HM5 HXC"}="HM52 HXC";
$resn_exch{"1HM6 HXC"}="HM61 HXC"; $resn_exch{"2HM6 HXC"}="HM62 HXC"; $resn_exch{"3HM6 HXC"}="HM63 HXC";
$resn_exch{"1H11 HXD"}="H111 HXD"; $resn_exch{"2H11 HXD"}="H112 HXD"; $resn_exch{"1H10 HXD"}="H101 HXD";
$resn_exch{"2H10 HXD"}="H102 HXD"; $resn_exch{"1H12 HXD"}="H121 HXD"; $resn_exch{"2H12 HXD"}="H122 HXD";
$resn_exch{"3H12 HXD"}="H123 HXD"; $resn_exch{"1H9  HXD"}=" H91 HXD"; $resn_exch{"2H9  HXD"}=" H92 HXD";
$resn_exch{"1H8  HXD"}=" H81 HXD"; $resn_exch{"2H8  HXD"}=" H82 HXD"; $resn_exch{"1H7  HXD"}=" H71 HXD";
$resn_exch{"2H7  HXD"}=" H72 HXD"; $resn_exch{"1H6  HXD"}=" H61 HXD"; $resn_exch{"2H6  HXD"}=" H62 HXD";
$resn_exch{"1H5  HXD"}=" H51 HXD"; $resn_exch{"2H5  HXD"}=" H52 HXD"; $resn_exch{"1H4  HXD"}=" H41 HXD";
$resn_exch{"2H4  HXD"}=" H42 HXD"; $resn_exch{"1H2  HXD"}=" H21 HXD"; $resn_exch{"2H2  HXD"}=" H22 HXD";
$resn_exch{"1HN2 HXL"}="HN21 HXL"; $resn_exch{"2HN2 HXL"}="HN22 HXL"; $resn_exch{"1HN6 HXL"}="HN61 HXL";
$resn_exch{"2HN6 HXL"}="HN62 HXL"; $resn_exch{"1H1' HXP"}="H1'1 HXP"; $resn_exch{"2H1' HXP"}="H1'2 HXP";
$resn_exch{"1H2' HXP"}="H2'1 HXP"; $resn_exch{"2H2' HXP"}="H2'2 HXP"; $resn_exch{"'HO2 HXP"}="HO2' HXP";
$resn_exch{" H   HY3"}=" HC  HY3"; $resn_exch{" HN  HY3"}=" H   HY3"; $resn_exch{"1HG  HY3"}=" HG2 HY3";
$resn_exch{"2HG  HY3"}=" HG3 HY3"; $resn_exch{"1HD  HY3"}=" HD3 HY3"; $resn_exch{"2HD  HY3"}=" HD2 HY3";
$resn_exch{"1H4  HYA"}=" H41 HYA"; $resn_exch{"2H4  HYA"}=" H42 HYA"; $resn_exch{"1H15 HYB"}="H151 HYB";
$resn_exch{"2H15 HYB"}="H152 HYB"; $resn_exch{"1H18 HYB"}="H181 HYB"; $resn_exch{"2H18 HYB"}="H182 HYB";
$resn_exch{"1H30 HYB"}="H301 HYB"; $resn_exch{"2H30 HYB"}="H302 HYB"; $resn_exch{"3H30 HYB"}="H303 HYB";
$resn_exch{"1H31 HYB"}="H311 HYB"; $resn_exch{"2H31 HYB"}="H312 HYB"; $resn_exch{"3H31 HYB"}="H313 HYB";
$resn_exch{"1H35 HYB"}="H351 HYB"; $resn_exch{"2H35 HYB"}="H352 HYB"; $resn_exch{"1H36 HYB"}="H361 HYB";
$resn_exch{"2H36 HYB"}="H362 HYB"; $resn_exch{"1H37 HYB"}="H371 HYB"; $resn_exch{"2H37 HYB"}="H372 HYB";
$resn_exch{"3H37 HYB"}="H373 HYB"; $resn_exch{"1H39 HYB"}="H391 HYB"; $resn_exch{"2H39 HYB"}="H392 HYB";
$resn_exch{"3H39 HYB"}="H393 HYB"; $resn_exch{"1H40 HYB"}="H401 HYB"; $resn_exch{"2H40 HYB"}="H402 HYB";
$resn_exch{"3H40 HYB"}="H403 HYB"; $resn_exch{"1H6  HYC"}=" H61 HYC"; $resn_exch{"2H6  HYC"}=" H62 HYC";
$resn_exch{"1H7  HYC"}=" H71 HYC"; $resn_exch{"2H7  HYC"}=" H72 HYC"; $resn_exch{"1H11 HYC"}="H111 HYC";
$resn_exch{"2H11 HYC"}="H112 HYC"; $resn_exch{"1H12 HYC"}="H121 HYC"; $resn_exch{"2H12 HYC"}="H122 HYC";
$resn_exch{"1H15 HYC"}="H151 HYC"; $resn_exch{"2H15 HYC"}="H152 HYC"; $resn_exch{"1H16 HYC"}="H161 HYC";
$resn_exch{"1H18 HYC"}="H181 HYC"; $resn_exch{"2H18 HYC"}="H182 HYC"; $resn_exch{"3H18 HYC"}="H183 HYC";
$resn_exch{"1H21 HYC"}="H211 HYC"; $resn_exch{"2H21 HYC"}="H212 HYC"; $resn_exch{"1H22 HYC"}="H221 HYC";
$resn_exch{"2H22 HYC"}="H222 HYC"; $resn_exch{"1H23 HYC"}="H231 HYC"; $resn_exch{"2H23 HYC"}="H232 HYC";
$resn_exch{"1H24 HYC"}="H241 HYC"; $resn_exch{"2H24 HYC"}="H242 HYC"; $resn_exch{"1H25 HYC"}="H251 HYC";
$resn_exch{"2H25 HYC"}="H252 HYC"; $resn_exch{"1H26 HYC"}="H261 HYC"; $resn_exch{"2H26 HYC"}="H262 HYC";
$resn_exch{"1H27 HYC"}="H271 HYC"; $resn_exch{"2H27 HYC"}="H272 HYC"; $resn_exch{"1H28 HYC"}="H281 HYC";
$resn_exch{"2H28 HYC"}="H282 HYC"; $resn_exch{"1HO2 HYC"}="HO21 HYC"; $resn_exch{"1HO3 HYC"}="HO31 HYC";
$resn_exch{"1H35 HYC"}="H351 HYC"; $resn_exch{"2H35 HYC"}="H352 HYC"; $resn_exch{"1HN4 HYC"}="HN41 HYC";
$resn_exch{"2HN4 HYC"}="HN42 HYC"; $resn_exch{"1H02 HYF"}="H021 HYF"; $resn_exch{"2H02 HYF"}="H022 HYF";
$resn_exch{"3H02 HYF"}="H023 HYF"; $resn_exch{"1H03 HYF"}="H031 HYF"; $resn_exch{"2H03 HYF"}="H032 HYF";
$resn_exch{"3H03 HYF"}="H033 HYF"; $resn_exch{"1H04 HYF"}="H041 HYF"; $resn_exch{"2H04 HYF"}="H042 HYF";
$resn_exch{"3H04 HYF"}="H043 HYF"; $resn_exch{"1H05 HYF"}="H051 HYF"; $resn_exch{"1H06 HYF"}="H061 HYF";
$resn_exch{"2H06 HYF"}="H062 HYF"; $resn_exch{"3H06 HYF"}="H063 HYF"; $resn_exch{"1H09 HYF"}="H091 HYF";
$resn_exch{"1H11 HYF"}="H111 HYF"; $resn_exch{"2H11 HYF"}="H112 HYF"; $resn_exch{"1H12 HYF"}="H121 HYF";
$resn_exch{"2H12 HYF"}="H122 HYF"; $resn_exch{"3H12 HYF"}="H123 HYF"; $resn_exch{"1H13 HYF"}="H131 HYF";
$resn_exch{"2H13 HYF"}="H132 HYF"; $resn_exch{"3H13 HYF"}="H133 HYF"; $resn_exch{"1H14 HYF"}="H141 HYF";
$resn_exch{"2H14 HYF"}="H142 HYF"; $resn_exch{"3H14 HYF"}="H143 HYF"; $resn_exch{"1H16 HYF"}="H161 HYF";
$resn_exch{"1H17 HYF"}="H171 HYF"; $resn_exch{"2H17 HYF"}="H172 HYF"; $resn_exch{"1H21 HYF"}="H211 HYF";
$resn_exch{"1H22 HYF"}="H221 HYF"; $resn_exch{"2H22 HYF"}="H222 HYF"; $resn_exch{"3H22 HYF"}="H223 HYF";
$resn_exch{"1H23 HYF"}="H231 HYF"; $resn_exch{"2H23 HYF"}="H232 HYF"; $resn_exch{"3H23 HYF"}="H233 HYF";
$resn_exch{"1H25 HYF"}="H251 HYF"; $resn_exch{"1H26 HYF"}="H261 HYF"; $resn_exch{"2H26 HYF"}="H262 HYF";
$resn_exch{"1H32 HYF"}="H321 HYF"; $resn_exch{"2H32 HYF"}="H322 HYF"; $resn_exch{"1H33 HYF"}="H331 HYF";
$resn_exch{"1H35 HYF"}="H351 HYF"; $resn_exch{"2H35 HYF"}="H352 HYF"; $resn_exch{"1H36 HYF"}="H361 HYF";
$resn_exch{"2H36 HYF"}="H362 HYF"; $resn_exch{"1H37 HYF"}="H371 HYF"; $resn_exch{"1H38 HYF"}="H381 HYF";
$resn_exch{"2H38 HYF"}="H382 HYF"; $resn_exch{"3H38 HYF"}="H383 HYF"; $resn_exch{"1H39 HYF"}="H391 HYF";
$resn_exch{"2H39 HYF"}="H392 HYF"; $resn_exch{"3H39 HYF"}="H393 HYF"; $resn_exch{"1H3  HYG"}="H31A HYG";
$resn_exch{"2H3  HYG"}="H32A HYG"; $resn_exch{"1H7  HYG"}=" H71 HYG"; $resn_exch{"2H7  HYG"}=" H72 HYG";
$resn_exch{"1H10 HYG"}="H101 HYG"; $resn_exch{"2H10 HYG"}="H102 HYG"; $resn_exch{"3H10 HYG"}="H103 HYG";
$resn_exch{"1H19 HYG"}="H191 HYG"; $resn_exch{"2H19 HYG"}="H192 HYG"; $resn_exch{"1H34 HYG"}="H341 HYG";
$resn_exch{"2H34 HYG"}="H342 HYG"; $resn_exch{"1H36 HYG"}="H361 HYG"; $resn_exch{"2H36 HYG"}="H362 HYG";
$resn_exch{"1HN  HYI"}="HN1A HYI"; $resn_exch{"2HN  HYI"}=" HN2 HYI"; $resn_exch{"1HB  HYI"}=" HB1 HYI";
$resn_exch{"2HB  HYI"}=" HB2 HYI"; $resn_exch{"1HG  HYI"}=" HG1 HYI"; $resn_exch{"2HG  HYI"}=" HG2 HYI";
$resn_exch{"1H5  HYI"}=" H51 HYI"; $resn_exch{"2H5  HYI"}=" H52 HYI"; $resn_exch{"1HAI HYM"}="HAI1 HYM";
$resn_exch{"2HAI HYM"}="HAI2 HYM"; $resn_exch{"1HAH HYM"}="HAH1 HYM"; $resn_exch{"2HAH HYM"}="HAH2 HYM";
$resn_exch{"1HAQ HYM"}="HAQ1 HYM"; $resn_exch{"2HAQ HYM"}="HAQ2 HYM"; $resn_exch{"1HB  HYP"}=" HB2 HYP";
$resn_exch{"2HB  HYP"}=" HB3 HYP"; $resn_exch{"1HD  HYP"}="HD22 HYP"; $resn_exch{"2HD  HYP"}="HD23 HYP";
$resn_exch{" HOD HYP"}=" HD1 HYP"; $resn_exch{"1H12 HYQ"}="H121 HYQ"; $resn_exch{"2H12 HYQ"}="H122 HYQ";
$resn_exch{"1H8  HYQ"}=" H81 HYQ"; $resn_exch{"2H8  HYQ"}=" H82 HYQ"; $resn_exch{"1H7  HYQ"}=" H71 HYQ";
$resn_exch{"2H7  HYQ"}=" H72 HYQ"; $resn_exch{"1H11 HYQ"}="H111 HYQ"; $resn_exch{"2H11 HYQ"}="H112 HYQ";
$resn_exch{"1HAA HZ3"}="HAA1 HZ3"; $resn_exch{"2HAA HZ3"}="HAA2 HZ3"; $resn_exch{"3HAA HZ3"}="HAA3 HZ3";
$resn_exch{"1HAB HZ3"}="HAB1 HZ3"; $resn_exch{"2HAB HZ3"}="HAB2 HZ3"; $resn_exch{"3HAB HZ3"}="HAB3 HZ3";
$resn_exch{"3HOP   I"}="HOP3   I"; $resn_exch{"2HOP   I"}="HOP2   I"; $resn_exch{"1H5*   I"}=" H5'   I";
$resn_exch{"2H5*   I"}="H5''   I"; $resn_exch{" H4*   I"}=" H4'   I"; $resn_exch{" H3*   I"}=" H3'   I";
$resn_exch{"H3T    I"}="HO3'   I"; $resn_exch{" H2*   I"}=" H2'   I"; $resn_exch{"2HO*   I"}="HO2'   I";
$resn_exch{" H1*   I"}=" H1'   I"; $resn_exch{"1H98 I01"}="H981 I01"; $resn_exch{"2H98 I01"}="H982 I01";
$resn_exch{"3H98 I01"}="H983 I01"; $resn_exch{"1H36 I01"}="H361 I01"; $resn_exch{"2H36 I01"}="H362 I01";
$resn_exch{"1H35 I01"}="H351 I01"; $resn_exch{"2H35 I01"}="H352 I01"; $resn_exch{"1H34 I01"}="H341 I01";
$resn_exch{"2H34 I01"}="H342 I01"; $resn_exch{"1H32 I01"}="H321 I01"; $resn_exch{"2H32 I01"}="H322 I01";
$resn_exch{"1H15 I02"}="H151 I02"; $resn_exch{"2H15 I02"}="H152 I02"; $resn_exch{"3H15 I02"}="H153 I02";
$resn_exch{"1H16 I02"}="H161 I02"; $resn_exch{"2H16 I02"}="H162 I02"; $resn_exch{"3H16 I02"}="H163 I02";
$resn_exch{"1H17 I02"}="H171 I02"; $resn_exch{"2H17 I02"}="H172 I02"; $resn_exch{"3H17 I02"}="H173 I02";
$resn_exch{"1HN3 I02"}="HN31 I02"; $resn_exch{"2HN3 I02"}="HN32 I02"; $resn_exch{"1H21 I02"}="H211 I02";
$resn_exch{"2H21 I02"}="H212 I02"; $resn_exch{"3H21 I02"}="H213 I02"; $resn_exch{"1H22 I02"}="H221 I02";
$resn_exch{"2H22 I02"}="H222 I02"; $resn_exch{"1H15 I03"}="H151 I03"; $resn_exch{"2H15 I03"}="H152 I03";
$resn_exch{"3H15 I03"}="H153 I03"; $resn_exch{"1H16 I03"}="H161 I03"; $resn_exch{"2H16 I03"}="H162 I03";
$resn_exch{"3H16 I03"}="H163 I03"; $resn_exch{"1H17 I03"}="H171 I03"; $resn_exch{"2H17 I03"}="H172 I03";
$resn_exch{"3H17 I03"}="H173 I03"; $resn_exch{"1H19 I03"}="H191 I03"; $resn_exch{"2H19 I03"}="H192 I03";
$resn_exch{"1HN3 I03"}="HN31 I03"; $resn_exch{"2HN3 I03"}="HN32 I03"; $resn_exch{"1H21 I03"}="H211 I03";
$resn_exch{"2H21 I03"}="H212 I03"; $resn_exch{"1H22 I03"}="H221 I03"; $resn_exch{"2H22 I03"}="H222 I03";
$resn_exch{"1H98 I04"}="H981 I04"; $resn_exch{"2H98 I04"}="H982 I04"; $resn_exch{"3H98 I04"}="H983 I04";
$resn_exch{"1H36 I04"}="H361 I04"; $resn_exch{"2H36 I04"}="H362 I04"; $resn_exch{"1H35 I04"}="H351 I04";
$resn_exch{"2H35 I04"}="H352 I04"; $resn_exch{"1H34 I04"}="H341 I04"; $resn_exch{"2H34 I04"}="H342 I04";
$resn_exch{"1H32 I04"}="H321 I04"; $resn_exch{"2H32 I04"}="H322 I04"; $resn_exch{"1H97 I05"}="H971 I05";
$resn_exch{"2H97 I05"}="H972 I05"; $resn_exch{"3H97 I05"}="H973 I05"; $resn_exch{"1H42 I05"}="H421 I05";
$resn_exch{"2H42 I05"}="H422 I05"; $resn_exch{"1H36 I05"}="H361 I05"; $resn_exch{"2H36 I05"}="H362 I05";
$resn_exch{"1H35 I05"}="H351 I05"; $resn_exch{"2H35 I05"}="H352 I05"; $resn_exch{"1H34 I05"}="H341 I05";
$resn_exch{"2H34 I05"}="H342 I05"; $resn_exch{"1H32 I05"}="H321 I05"; $resn_exch{"2H32 I05"}="H322 I05";
$resn_exch{"1H7  I06"}=" H71 I06"; $resn_exch{"1H8  I06"}=" H81 I06"; $resn_exch{"1H11 I06"}="H111 I06";
$resn_exch{"1H12 I06"}="H121 I06"; $resn_exch{"1HA  I06"}=" HA1 I06"; $resn_exch{"2HA  I06"}=" HA2 I06";
$resn_exch{"1HB  I06"}=" HB1 I06"; $resn_exch{"2HB  I06"}=" HB2 I06"; $resn_exch{"1HD1 I06"}="HD11 I06";
$resn_exch{"1HD2 I06"}="HD21 I06"; $resn_exch{"1HE1 I06"}="HE11 I06"; $resn_exch{"1HE2 I06"}="HE21 I06";
$resn_exch{"1HZ  I06"}=" HZ1 I06"; $resn_exch{"1H98 I08"}="H981 I08"; $resn_exch{"2H98 I08"}="H982 I08";
$resn_exch{"3H98 I08"}="H983 I08"; $resn_exch{"1H36 I08"}="H361 I08"; $resn_exch{"2H36 I08"}="H362 I08";
$resn_exch{"1H35 I08"}="H351 I08"; $resn_exch{"2H35 I08"}="H352 I08"; $resn_exch{"1H34 I08"}="H341 I08";
$resn_exch{"2H34 I08"}="H342 I08"; $resn_exch{"1H32 I08"}="H321 I08"; $resn_exch{"2H32 I08"}="H322 I08";
$resn_exch{"1H24 I0G"}="H241 I0G"; $resn_exch{"2H24 I0G"}="H242 I0G"; $resn_exch{"1H1  I0G"}="H11A I0G";
$resn_exch{"2H1  I0G"}=" H12 I0G"; $resn_exch{"1H23 I0G"}="H231 I0G"; $resn_exch{"2H23 I0G"}="H232 I0G";
$resn_exch{"1H16 I10"}="H161 I10"; $resn_exch{"2H16 I10"}="H162 I10"; $resn_exch{"1H20 I10"}="H201 I10";
$resn_exch{"2H20 I10"}="H202 I10"; $resn_exch{"3H20 I10"}="H203 I10"; $resn_exch{"1H24 I10"}="H241 I10";
$resn_exch{"2H24 I10"}="H242 I10"; $resn_exch{"3H24 I10"}="H243 I10"; $resn_exch{"1H30 I10"}="H301 I10";
$resn_exch{"2H30 I10"}="H302 I10"; $resn_exch{"1H35 I10"}="H351 I10"; $resn_exch{"2H35 I10"}="H352 I10";
$resn_exch{"1H1  I11"}=" H11 I11"; $resn_exch{"2H1  I11"}=" H12 I11"; $resn_exch{"1H2  I11"}=" H21 I11";
$resn_exch{"2H2  I11"}=" H22 I11"; $resn_exch{"1H4  I11"}=" H41 I11"; $resn_exch{"2H4  I11"}=" H42 I11";
$resn_exch{"1H5  I11"}=" H51 I11"; $resn_exch{"2H5  I11"}=" H52 I11"; $resn_exch{"1H6  I11"}=" H61 I11";
$resn_exch{"2H6  I11"}=" H62 I11"; $resn_exch{"1H21 I11"}="H211 I11"; $resn_exch{"2H21 I11"}="H212 I11";
$resn_exch{"1H25 I11"}="H251 I11"; $resn_exch{"2H25 I11"}="H252 I11"; $resn_exch{"1H26 I11"}="H261 I11";
$resn_exch{"2H26 I11"}="H262 I11"; $resn_exch{"1H36 I11"}="H361 I11"; $resn_exch{"2H36 I11"}="H362 I11";
$resn_exch{"1H37 I11"}="H371 I11"; $resn_exch{"2H37 I11"}="H372 I11"; $resn_exch{"1H38 I11"}="H381 I11";
$resn_exch{"2H38 I11"}="H382 I11"; $resn_exch{"1H39 I11"}="H391 I11"; $resn_exch{"2H39 I11"}="H392 I11";
$resn_exch{"1H40 I11"}="H401 I11"; $resn_exch{"2H40 I11"}="H402 I11"; $resn_exch{"1H4  I12"}=" H41 I12";
$resn_exch{"2H4  I12"}=" H42 I12"; $resn_exch{"3H4  I12"}=" H43 I12"; $resn_exch{"1H8  I12"}=" H81 I12";
$resn_exch{"2H8  I12"}=" H82 I12"; $resn_exch{"3H8  I12"}=" H83 I12"; $resn_exch{"1H11 I12"}="H111 I12";
$resn_exch{"2H11 I12"}="H112 I12"; $resn_exch{"3H11 I12"}="H113 I12"; $resn_exch{"1H12 I12"}="H121 I12";
$resn_exch{"2H12 I12"}="H122 I12"; $resn_exch{"3H12 I12"}="H123 I12"; $resn_exch{"1H15 I12"}="H151 I12";
$resn_exch{"2H15 I12"}="H152 I12"; $resn_exch{"1H17 I12"}="H171 I12"; $resn_exch{"2H17 I12"}="H172 I12";
$resn_exch{"3H17 I12"}="H173 I12"; $resn_exch{"1H18 I12"}="H181 I12"; $resn_exch{"2H18 I12"}="H182 I12";
$resn_exch{"3H18 I12"}="H183 I12"; $resn_exch{"1H24 I12"}="H241 I12"; $resn_exch{"2H24 I12"}="H242 I12";
$resn_exch{"1H25 I12"}="H251 I12"; $resn_exch{"2H25 I12"}="H252 I12"; $resn_exch{"3H25 I12"}="H253 I12";
$resn_exch{"1H26 I12"}="H261 I12"; $resn_exch{"2H26 I12"}="H262 I12"; $resn_exch{"1H28 I12"}="H281 I12";
$resn_exch{"2H28 I12"}="H282 I12"; $resn_exch{"1H29 I12"}="H291 I12"; $resn_exch{"2H29 I12"}="H292 I12";
$resn_exch{"1H26 I17"}="H261 I17"; $resn_exch{"1H8  I17"}=" H81 I17"; $resn_exch{"1H18 I17"}="H181 I17";
$resn_exch{"1H19 I17"}="H191 I17"; $resn_exch{"2H19 I17"}="H192 I17"; $resn_exch{"1H25 I17"}="H251 I17";
$resn_exch{"1H24 I17"}="H241 I17"; $resn_exch{"1H23 I17"}="H231 I17"; $resn_exch{"1H22 I17"}="H221 I17";
$resn_exch{"1H21 I17"}="H211 I17"; $resn_exch{"1H10 I17"}="H101 I17"; $resn_exch{"1H11 I17"}="H111 I17";
$resn_exch{"1H14 I17"}="H141 I17"; $resn_exch{"1H15 I17"}="H151 I17"; $resn_exch{"2H15 I17"}="H152 I17";
$resn_exch{"1H16 I17"}="H161 I17"; $resn_exch{"2H16 I17"}="H162 I17"; $resn_exch{"1H12 I17"}="H121 I17";
$resn_exch{"2H12 I17"}="H122 I17"; $resn_exch{"1H13 I17"}="H131 I17"; $resn_exch{"2H13 I17"}="H132 I17";
$resn_exch{"1H17 I17"}="H171 I17"; $resn_exch{"2H17 I17"}="H172 I17"; $resn_exch{"1H30 I17"}="H301 I17";
$resn_exch{"2H30 I17"}="H302 I17"; $resn_exch{"1H29 I17"}="H291 I17"; $resn_exch{"2H29 I17"}="H292 I17";
$resn_exch{"1H27 I17"}="H271 I17"; $resn_exch{"2H27 I17"}="H272 I17"; $resn_exch{"1H28 I17"}="H281 I17";
$resn_exch{"2H28 I17"}="H282 I17"; $resn_exch{"1H9C I1H"}="H9C1 I1H"; $resn_exch{"2H9C I1H"}="H9C2 I1H";
$resn_exch{"1H10 I1H"}="H101 I1H"; $resn_exch{"2H10 I1H"}="H102 I1H"; $resn_exch{"1H25 I1H"}="H251 I1H";
$resn_exch{"2H25 I1H"}="H252 I1H"; $resn_exch{"1H29 I1H"}="H291 I1H"; $resn_exch{"2H29 I1H"}="H292 I1H";
$resn_exch{"1H30 I1H"}="H301 I1H"; $resn_exch{"2H30 I1H"}="H302 I1H"; $resn_exch{"1H32 I1H"}="H321 I1H";
$resn_exch{"2H32 I1H"}="H322 I1H"; $resn_exch{"1H33 I1H"}="H331 I1H"; $resn_exch{"2H33 I1H"}="H332 I1H";
$resn_exch{"1H7C I1H"}="H7C1 I1H"; $resn_exch{"2H7C I1H"}="H7C2 I1H"; $resn_exch{"3H7C I1H"}="H7C3 I1H";
$resn_exch{"1H25 I1P"}="H251 I1P"; $resn_exch{"2H25 I1P"}="H252 I1P"; $resn_exch{"1H26 I1P"}="H261 I1P";
$resn_exch{"2H26 I1P"}="H262 I1P"; $resn_exch{"1H27 I1P"}="H271 I1P"; $resn_exch{"2H27 I1P"}="H272 I1P";
$resn_exch{"1H29 I1P"}="H291 I1P"; $resn_exch{"2H29 I1P"}="H292 I1P"; $resn_exch{"3H29 I1P"}="H293 I1P";
$resn_exch{"1H30 I1P"}="H301 I1P"; $resn_exch{"2H30 I1P"}="H302 I1P"; $resn_exch{"3H30 I1P"}="H303 I1P";
$resn_exch{"1H1  I20"}=" H11 I20"; $resn_exch{"2H1  I20"}="H12A I20"; $resn_exch{"3H1  I20"}="H13A I20";
$resn_exch{"1H7  I20"}=" H71 I20"; $resn_exch{"2H7  I20"}=" H72 I20"; $resn_exch{"1H9  I20"}=" H91 I20";
$resn_exch{"2H9  I20"}=" H92 I20"; $resn_exch{"1H27 I20"}="H271 I20"; $resn_exch{"2H27 I20"}="H272 I20";
$resn_exch{"1H29 I20"}="H291 I20"; $resn_exch{"2H29 I20"}="H292 I20"; $resn_exch{"1H32 I20"}="H321 I20";
$resn_exch{"2H32 I20"}="H322 I20"; $resn_exch{"1HG  I20"}=" HG1 I20"; $resn_exch{"2HG  I20"}=" HG2 I20";
$resn_exch{"1HB  I20"}=" HB1 I20"; $resn_exch{"2HB  I20"}=" HB2 I20"; $resn_exch{"1HA  I20"}=" HA1 I20";
$resn_exch{"2HA  I20"}=" HA2 I20"; $resn_exch{"1H36 I20"}="H361 I20"; $resn_exch{"2H36 I20"}="H362 I20";
$resn_exch{"3H36 I20"}="H363 I20"; $resn_exch{"1H10 I21"}="H101 I21"; $resn_exch{"2H10 I21"}="H102 I21";
$resn_exch{"1HN3 I21"}="HN31 I21"; $resn_exch{"2HN3 I21"}="HN32 I21"; $resn_exch{"1H14 I21"}="H141 I21";
$resn_exch{"2H14 I21"}="H142 I21"; $resn_exch{"1H30 I21"}="H301 I21"; $resn_exch{"2H30 I21"}="H302 I21";
$resn_exch{"3H30 I21"}="H303 I21"; $resn_exch{"1H7  I22"}=" H71 I22"; $resn_exch{"2H7  I22"}=" H72 I22";
$resn_exch{"1H2  I22"}=" H21 I22"; $resn_exch{"2H2  I22"}=" H22 I22"; $resn_exch{"2HO2 I2P"}="HO22 I2P";
$resn_exch{"3HO2 I2P"}="HO23 I2P"; $resn_exch{"2HO4 I2P"}="HO42 I2P"; $resn_exch{"3HO4 I2P"}="HO43 I2P";
$resn_exch{"2HO5 I2P"}="HO52 I2P"; $resn_exch{"3HO5 I2P"}="HO53 I2P"; $resn_exch{"1H1  I38"}=" H11 I38";
$resn_exch{"2H1  I38"}=" H12 I38"; $resn_exch{"1H2  I38"}=" H21 I38"; $resn_exch{"2H2  I38"}=" H22 I38";
$resn_exch{"1H51 I38"}="H511 I38"; $resn_exch{"2H51 I38"}="H512 I38"; $resn_exch{"3H51 I38"}="H513 I38";
$resn_exch{" H3* I3A"}=" H3' I3A"; $resn_exch{"1H1  I3N"}=" H11 I3N"; $resn_exch{"2H1  I3N"}=" H12 I3N";
$resn_exch{"3H1  I3N"}=" H13 I3N"; $resn_exch{"1H12 I3N"}="H121 I3N"; $resn_exch{"2H12 I3N"}="H122 I3N";
$resn_exch{"1H16 I3N"}="H161 I3N"; $resn_exch{"2H16 I3N"}="H162 I3N"; $resn_exch{"3H16 I3N"}="H163 I3N";
$resn_exch{"1H17 I3N"}="H171 I3N"; $resn_exch{"2H17 I3N"}="H172 I3N"; $resn_exch{"2HO1 I3P"}="HO12 I3P";
$resn_exch{"3HO1 I3P"}="HO13 I3P"; $resn_exch{"2HO4 I3P"}="HO42 I3P"; $resn_exch{"3HO4 I3P"}="HO43 I3P";
$resn_exch{"2HO5 I3P"}="HO52 I3P"; $resn_exch{"3HO5 I3P"}="HO53 I3P"; $resn_exch{"2H1  I40"}=" H12 I40";
$resn_exch{"1H1  I40"}=" H11 I40"; $resn_exch{"2H2  I40"}=" H22 I40"; $resn_exch{"1H2  I40"}=" H21 I40";
$resn_exch{"2H3  I40"}=" H32 I40"; $resn_exch{"1H3  I40"}=" H31 I40"; $resn_exch{"2H4  I40"}=" H42 I40";
$resn_exch{"1H4  I40"}=" H41 I40"; $resn_exch{"1H2N I40"}="H2N1 I40"; $resn_exch{"2H14 I40"}="H142 I40";
$resn_exch{"1H14 I40"}="H141 I40"; $resn_exch{"1HA2 I48"}="HA21 I48"; $resn_exch{"2HA2 I48"}="HA22 I48";
$resn_exch{"1HA3 I48"}="HA31 I48"; $resn_exch{"2HA3 I48"}="HA32 I48"; $resn_exch{"1H27 I48"}="H271 I48";
$resn_exch{"2H27 I48"}="H272 I48"; $resn_exch{"3H27 I48"}="H273 I48"; $resn_exch{"1H1' I4B"}="H1'1 I4B";
$resn_exch{"2H1' I4B"}="H1'2 I4B"; $resn_exch{"1H3' I4B"}="H3'1 I4B"; $resn_exch{"2H3' I4B"}="H3'2 I4B";
$resn_exch{"3H3' I4B"}="H3'3 I4B"; $resn_exch{"1H4' I4B"}="H4'1 I4B"; $resn_exch{"2H4' I4B"}="H4'2 I4B";
$resn_exch{"3H4' I4B"}="H4'3 I4B"; $resn_exch{"1H38 I52"}="H381 I52"; $resn_exch{"2H38 I52"}="H382 I52";
$resn_exch{"1H39 I52"}="H391 I52"; $resn_exch{"2H39 I52"}="H392 I52"; $resn_exch{"3H39 I52"}="H393 I52";
$resn_exch{"1H37 I52"}="H371 I52"; $resn_exch{"2H37 I52"}="H372 I52"; $resn_exch{"1H35 I52"}="H351 I52";
$resn_exch{"2H35 I52"}="H352 I52"; $resn_exch{"1H36 I52"}="H361 I52"; $resn_exch{"2H36 I52"}="H362 I52";
$resn_exch{"1H25 I52"}="H251 I52"; $resn_exch{"2H25 I52"}="H252 I52"; $resn_exch{"3H25 I52"}="H253 I52";
$resn_exch{"1H24 I52"}="H241 I52"; $resn_exch{"2H24 I52"}="H242 I52"; $resn_exch{"3H24 I52"}="H243 I52";
$resn_exch{"1H7  I52"}=" H71 I52"; $resn_exch{"2H7  I52"}=" H72 I52"; $resn_exch{"1H9  I52"}=" H91 I52";
$resn_exch{"2H9  I52"}=" H92 I52"; $resn_exch{"1H13 I52"}="H131 I52"; $resn_exch{"2H13 I52"}="H132 I52";
$resn_exch{"1H12 I52"}="H121 I52"; $resn_exch{"2H12 I52"}="H122 I52"; $resn_exch{"1H10 I52"}="H101 I52";
$resn_exch{"2H10 I52"}="H102 I52"; $resn_exch{"1H6  I52"}=" H61 I52"; $resn_exch{"2H6  I52"}=" H62 I52";
$resn_exch{"1HN  I58"}=" H   I58"; $resn_exch{"2HN  I58"}=" H2  I58"; $resn_exch{"1HB  I58"}=" HB2 I58";
$resn_exch{"2HB  I58"}=" HB3 I58"; $resn_exch{"1HD  I58"}=" HD2 I58"; $resn_exch{"2HD  I58"}=" HD3 I58";
$resn_exch{"1HE  I58"}=" HE2 I58"; $resn_exch{"2HE  I58"}=" HE3 I58"; $resn_exch{"1HZ  I58"}=" HZ  I58";
$resn_exch{"1HT  I58"}=" HT1 I58"; $resn_exch{"2HT  I58"}=" HT2 I58"; $resn_exch{"3HT  I58"}=" HT3 I58";
$resn_exch{"2HOP I5C"}="HOP2 I5C"; $resn_exch{"1H4  I5C"}=" H41 I5C"; $resn_exch{"2H4  I5C"}=" H42 I5C";
$resn_exch{"1H2* I5C"}=" H2' I5C"; $resn_exch{"2H2* I5C"}="H2'' I5C"; $resn_exch{"1H5* I5C"}=" H5' I5C";
$resn_exch{"2H5* I5C"}="H5'' I5C"; $resn_exch{" H4* I5C"}=" H4' I5C"; $resn_exch{" H1* I5C"}=" H1' I5C";
$resn_exch{" H3* I5C"}=" H3' I5C"; $resn_exch{"H3T  I5C"}="HO3' I5C"; $resn_exch{"3HOP I5C"}="HOP3 I5C";
$resn_exch{"1H6C I5S"}="H6C1 I5S"; $resn_exch{"2H6C I5S"}="H6C2 I5S"; $resn_exch{"1H8C I5S"}="H8C1 I5S";
$resn_exch{"2H8C I5S"}="H8C2 I5S"; $resn_exch{"1H9C I5S"}="H9C1 I5S"; $resn_exch{"2H9C I5S"}="H9C2 I5S";
$resn_exch{"1H13 I5S"}="H131 I5S"; $resn_exch{"2H13 I5S"}="H132 I5S"; $resn_exch{"1H14 I5S"}="H141 I5S";
$resn_exch{"2H14 I5S"}="H142 I5S"; $resn_exch{"1HN1 I7A"}="HN11 I7A"; $resn_exch{"2HN1 I7A"}="HN12 I7A";
$resn_exch{"1HN2 I7A"}="HN21 I7A"; $resn_exch{"2HN2 I7A"}="HN22 I7A"; $resn_exch{"1HN1 I7B"}="HN11 I7B";
$resn_exch{"2HN1 I7B"}="HN12 I7B"; $resn_exch{"1HN2 I7B"}="HN21 I7B"; $resn_exch{"2HN2 I7B"}="HN22 I7B";
$resn_exch{"1HN3 I7B"}="HN31 I7B"; $resn_exch{"2HN3 I7B"}="HN32 I7B"; $resn_exch{"1HN1 I7C"}="HN11 I7C";
$resn_exch{"2HN1 I7C"}="HN12 I7C"; $resn_exch{"1HN2 I7C"}="HN21 I7C"; $resn_exch{"2HN2 I7C"}="HN22 I7C";
$resn_exch{"1HN3 I7C"}="HN31 I7C"; $resn_exch{"2HN3 I7C"}="HN32 I7C"; $resn_exch{"1H11 I84"}="H111 I84";
$resn_exch{"2H11 I84"}="H112 I84"; $resn_exch{"3H11 I84"}="H113 I84"; $resn_exch{"1H15 I84"}="H151 I84";
$resn_exch{"2H15 I84"}="H152 I84"; $resn_exch{"1H29 I84"}="H291 I84"; $resn_exch{"2H29 I84"}="H292 I84";
$resn_exch{"3H29 I84"}="H293 I84"; $resn_exch{"1H33 I84"}="H331 I84"; $resn_exch{"2H33 I84"}="H332 I84";
$resn_exch{"3H33 I84"}="H333 I84"; $resn_exch{"1H39 I84"}="H391 I84"; $resn_exch{"2H39 I84"}="H392 I84";
$resn_exch{"1H10 I96"}="H101 I96"; $resn_exch{"2H10 I96"}="H102 I96"; $resn_exch{"3H10 I96"}="H103 I96";
$resn_exch{"1H11 I96"}="H111 I96"; $resn_exch{"2H11 I96"}="H112 I96"; $resn_exch{"3H11 I96"}="H113 I96";
$resn_exch{"1H17 I96"}="H171 I96"; $resn_exch{"2H17 I96"}="H172 I96"; $resn_exch{"1H20  IA"}="H201  IA";
$resn_exch{"2H20  IA"}="H202  IA"; $resn_exch{"1H21  IA"}="H211  IA"; $resn_exch{"2H21  IA"}="H212  IA";
$resn_exch{"1H22  IA"}="H221  IA"; $resn_exch{"2H22  IA"}="H222  IA"; $resn_exch{"1H23  IA"}="H231  IA";
$resn_exch{"2H23  IA"}="H232  IA"; $resn_exch{"1H24  IA"}="H241  IA"; $resn_exch{"2H24  IA"}="H242  IA";
$resn_exch{"3H24  IA"}="H243  IA"; $resn_exch{"1H17 IAA"}="H171 IAA"; $resn_exch{"2H17 IAA"}="H172 IAA";
$resn_exch{"1H2  IAA"}=" H21 IAA"; $resn_exch{"2H2  IAA"}=" H22 IAA"; $resn_exch{"1H16 IAA"}="H161 IAA";
$resn_exch{"2H16 IAA"}="H162 IAA"; $resn_exch{"1H6X IAB"}="H6X1 IAB"; $resn_exch{"2H6X IAB"}="H6X2 IAB";
$resn_exch{"1H6Y IAB"}="H6Y1 IAB"; $resn_exch{"2H6Y IAB"}="H6Y2 IAB"; $resn_exch{"3H6Y IAB"}="H6Y3 IAB";
$resn_exch{"1H7Z IAB"}="H7Z1 IAB"; $resn_exch{"2H7Z IAB"}="H7Z2 IAB"; $resn_exch{"1H6W IAB"}="H6W1 IAB";
$resn_exch{"2H6W IAB"}="H6W2 IAB"; $resn_exch{"1H17 IAC"}="H171 IAC"; $resn_exch{"2H17 IAC"}="H172 IAC";
$resn_exch{"1HB  IAD"}=" HB1 IAD"; $resn_exch{"2HB  IAD"}=" HB2 IAD"; $resn_exch{"1H   IAD"}=" H1  IAD";
$resn_exch{"2H   IAD"}=" H2A IAD"; $resn_exch{"1H11 IAG"}="H111 IAG"; $resn_exch{"2H11 IAG"}="H112 IAG";
$resn_exch{"1HC9 IAG"}="HC91 IAG"; $resn_exch{"2HC9 IAG"}="HC92 IAG"; $resn_exch{"1HB  IAM"}=" HB1 IAM";
$resn_exch{"1HT  IAM"}=" HT1 IAM"; $resn_exch{"2HT  IAM"}=" HT2 IAM"; $resn_exch{"1HK1 IAM"}="HK11 IAM";
$resn_exch{"2HK1 IAM"}="HK12 IAM"; $resn_exch{"3HK1 IAM"}="HK13 IAM"; $resn_exch{"1HK2 IAM"}="HK21 IAM";
$resn_exch{"2HK2 IAM"}="HK22 IAM"; $resn_exch{"3HK2 IAM"}="HK23 IAM"; $resn_exch{"1HO1 IAP"}="HO11 IAP";
$resn_exch{"1HO2 IAP"}="HO21 IAP"; $resn_exch{"1H2  IAP"}=" H21 IAP"; $resn_exch{"1H3  IAP"}=" H31 IAP";
$resn_exch{"1H4  IAP"}=" H41 IAP"; $resn_exch{"1H6  IAP"}=" H61 IAP"; $resn_exch{"1HN1 IAP"}="HN11 IAP";
$resn_exch{"1H8  IAP"}=" H81 IAP"; $resn_exch{"2H8  IAP"}=" H82 IAP"; $resn_exch{"1HN  IAS"}=" H   IAS";
$resn_exch{"2HN  IAS"}=" H2  IAS"; $resn_exch{"1HB  IAS"}=" HB2 IAS"; $resn_exch{"2HB  IAS"}=" HB3 IAS";
$resn_exch{"1H14 IAV"}="H141 IAV"; $resn_exch{"2H14 IAV"}="H142 IAV"; $resn_exch{"3H14 IAV"}="H143 IAV";
$resn_exch{"1H15 IAV"}="H151 IAV"; $resn_exch{"2H15 IAV"}="H152 IAV"; $resn_exch{"3H15 IAV"}="H153 IAV";
$resn_exch{"1HC9 IAV"}="HC91 IAV"; $resn_exch{"2HC9 IAV"}="HC92 IAV"; $resn_exch{"1H20  IB"}="H201  IB";
$resn_exch{"2H20  IB"}="H202  IB"; $resn_exch{"1H21  IB"}="H211  IB"; $resn_exch{"2H21  IB"}="H212  IB";
$resn_exch{"1H22  IB"}="H221  IB"; $resn_exch{"2H22  IB"}="H222  IB"; $resn_exch{"1H23  IB"}="H231  IB";
$resn_exch{"2H23  IB"}="H232  IB"; $resn_exch{"1H24  IB"}="H241  IB"; $resn_exch{"2H24  IB"}="H242  IB";
$resn_exch{"3H24  IB"}="H243  IB"; $resn_exch{"1HN2  IB"}="HN21  IB"; $resn_exch{"2HN2  IB"}="HN22  IB";
$resn_exch{"3HN2  IB"}="HN23  IB"; $resn_exch{"2HOA IB2"}="HOA2 IB2"; $resn_exch{"1H3A IB2"}="H3A1 IB2";
$resn_exch{"2H3A IB2"}="H3A2 IB2"; $resn_exch{"2HOB IB2"}="HOB2 IB2"; $resn_exch{"2HSG IB2"}="HSG2 IB2";
$resn_exch{"1H5* IB2"}="H5'1 IB2"; $resn_exch{"2H5* IB2"}="H5'2 IB2"; $resn_exch{" H4* IB2"}=" H4D IB2";
$resn_exch{" H3* IB2"}=" H3D IB2"; $resn_exch{"*HO3 IB2"}="HO3' IB2"; $resn_exch{" H2* IB2"}=" H2D IB2";
$resn_exch{"*HO2 IB2"}="HO2' IB2"; $resn_exch{" H1* IB2"}=" H1D IB2"; $resn_exch{"1H5' IB2"}="H51X IB2";
$resn_exch{"2H5' IB2"}="H52X IB2"; $resn_exch{"'HO3 IB2"}="HO3X IB2"; $resn_exch{"'HO2 IB2"}="HO2X IB2";
$resn_exch{"1HN6 IB2"}="HN61 IB2"; $resn_exch{"2HN6 IB2"}="HN62 IB2"; $resn_exch{" AH8 IB2"}=" H8A IB2";
$resn_exch{"1HNA IB2"}="HNA1 IB2"; $resn_exch{"2HNA IB2"}="HNA2 IB2"; $resn_exch{" AH2 IB2"}=" H2A IB2";
$resn_exch{"1H15 IBA"}="H151 IBA"; $resn_exch{"2H15 IBA"}="H152 IBA"; $resn_exch{"1H14 IBA"}="H141 IBA";
$resn_exch{"2H14 IBA"}="H142 IBA"; $resn_exch{"1H18 IBA"}="H181 IBA"; $resn_exch{"2H18 IBA"}="H182 IBA";
$resn_exch{"1H17 IBA"}="H171 IBA"; $resn_exch{"2H17 IBA"}="H172 IBA"; $resn_exch{"1H1  IBA"}=" H11 IBA";
$resn_exch{"2H1  IBA"}=" H12 IBA"; $resn_exch{"1H11 IBA"}="H111 IBA"; $resn_exch{"2H11 IBA"}="H112 IBA";
$resn_exch{"1H12 IBA"}="H121 IBA"; $resn_exch{"2H12 IBA"}="H122 IBA"; $resn_exch{"3H12 IBA"}="H123 IBA";
$resn_exch{"1H9  IBA"}=" H91 IBA"; $resn_exch{"2H9  IBA"}=" H92 IBA"; $resn_exch{"1H10 IBA"}="H101 IBA";
$resn_exch{"2H10 IBA"}="H102 IBA"; $resn_exch{"3H10 IBA"}="H103 IBA"; $resn_exch{"1H2  IBB"}="H21A IBB";
$resn_exch{"2H2  IBB"}=" H22 IBB"; $resn_exch{"1H3  IBB"}=" H31 IBB"; $resn_exch{"2H3  IBB"}=" H32 IBB";
$resn_exch{"1HN2 IBC"}="HN21 IBC"; $resn_exch{"2HN2 IBC"}="HN22 IBC"; $resn_exch{"1H9  IBC"}=" H91 IBC";
$resn_exch{"2H9  IBC"}=" H92 IBC"; $resn_exch{"1H6  IBC"}=" H61 IBC"; $resn_exch{"2H6  IBC"}=" H62 IBC";
$resn_exch{"1H7  IBC"}=" H71 IBC"; $resn_exch{"2H7  IBC"}=" H72 IBC"; $resn_exch{"1H8  IBC"}=" H81 IBC";
$resn_exch{"2H8  IBC"}=" H82 IBC"; $resn_exch{"1HN1 IBG"}="HN11 IBG"; $resn_exch{"2HN1 IBG"}="HN12 IBG";
$resn_exch{"CHA1 IBG"}="HA1C IBG"; $resn_exch{"1HCB IBG"}="HCB1 IBG"; $resn_exch{"2HCB IBG"}="HCB2 IBG";
$resn_exch{"1HCG IBG"}="HCG1 IBG"; $resn_exch{"2HCG IBG"}="HCG2 IBG"; $resn_exch{"CHA2 IBG"}="HA2C IBG";
$resn_exch{"1HB2 IBG"}="HB21 IBG"; $resn_exch{"2HB2 IBG"}="HB22 IBG"; $resn_exch{"1HC3 IBG"}="HC31 IBG";
$resn_exch{"2HC3 IBG"}="HC32 IBG"; $resn_exch{"1HCS IBG"}="HCS1 IBG"; $resn_exch{"2HCS IBG"}="HCS2 IBG";
$resn_exch{"1H10 IBM"}="H101 IBM"; $resn_exch{"2H10 IBM"}="H102 IBM"; $resn_exch{"3H10 IBM"}="H103 IBM";
$resn_exch{"1H11 IBM"}="H111 IBM"; $resn_exch{"2H11 IBM"}="H112 IBM"; $resn_exch{"1H13 IBM"}="H131 IBM";
$resn_exch{"2H13 IBM"}="H132 IBM"; $resn_exch{"3H13 IBM"}="H133 IBM"; $resn_exch{"1H14 IBM"}="H141 IBM";
$resn_exch{"2H14 IBM"}="H142 IBM"; $resn_exch{"3H14 IBM"}="H143 IBM"; $resn_exch{"1HAN IBO"}="HAN1 IBO";
$resn_exch{"2HAN IBO"}="HAN2 IBO"; $resn_exch{"1HB2 IBO"}="HB21 IBO"; $resn_exch{"2HB2 IBO"}="HB22 IBO";
$resn_exch{"3HB2 IBO"}="HB23 IBO"; $resn_exch{"1HB1 IBO"}="HB11 IBO"; $resn_exch{"2HB1 IBO"}="HB12 IBO";
$resn_exch{"3HB1 IBO"}="HB13 IBO"; $resn_exch{"1H2  IBP"}=" H21 IBP"; $resn_exch{"2H2  IBP"}=" H22 IBP";
$resn_exch{"1H3  IBP"}=" H31 IBP"; $resn_exch{"1H4  IBP"}=" H41 IBP"; $resn_exch{"2H4  IBP"}=" H42 IBP";
$resn_exch{"3H4  IBP"}=" H43 IBP"; $resn_exch{"1H5  IBP"}=" H51 IBP"; $resn_exch{"2H5  IBP"}=" H52 IBP";
$resn_exch{"3H5  IBP"}=" H53 IBP"; $resn_exch{"1H6  IBP"}=" H61 IBP"; $resn_exch{"1H7  IBP"}=" H71 IBP";
$resn_exch{"2H7  IBP"}=" H72 IBP"; $resn_exch{"3H7  IBP"}=" H73 IBP"; $resn_exch{"1H9  IBP"}=" H91 IBP";
$resn_exch{"1H10 IBP"}="H101 IBP"; $resn_exch{"1H12 IBP"}="H121 IBP"; $resn_exch{"1H13 IBP"}="H131 IBP";
$resn_exch{"1HN  IBR"}=" HN1 IBR"; $resn_exch{"2HN  IBR"}=" HN2 IBR"; $resn_exch{"1H2  IBR"}=" H21 IBR";
$resn_exch{"2H2  IBR"}=" H22 IBR"; $resn_exch{"1HE1 IBR"}="HE11 IBR"; $resn_exch{"2HE1 IBR"}="HE12 IBR";
$resn_exch{"1HE2 IBR"}="HE21 IBR"; $resn_exch{"2HE2 IBR"}="HE22 IBR"; $resn_exch{"1H21 IBS"}="H211 IBS";
$resn_exch{"2H21 IBS"}="H212 IBS"; $resn_exch{"1H25 IBS"}="H251 IBS"; $resn_exch{"3HO1 IBS"}="HO13 IBS";
$resn_exch{"2HO4 IBS"}="HO42 IBS"; $resn_exch{"3HO4 IBS"}="HO43 IBS"; $resn_exch{"2HO5 IBS"}="HO52 IBS";
$resn_exch{"3HO5 IBS"}="HO53 IBS"; $resn_exch{"1H   IBZ"}=" H1  IBZ"; $resn_exch{"2H   IBZ"}=" H2  IBZ";
$resn_exch{"2HOP  IC"}="HOP2  IC"; $resn_exch{"3HOP  IC"}="HOP3  IC"; $resn_exch{"H3T   IC"}="HO3'  IC";
$resn_exch{" H1*  IC"}=" H1'  IC"; $resn_exch{" H2*  IC"}=" H2'  IC"; $resn_exch{" H3*  IC"}=" H3'  IC";
$resn_exch{" H4*  IC"}=" H4'  IC"; $resn_exch{"1H5*  IC"}=" H5'  IC"; $resn_exch{"2H5*  IC"}="H5''  IC";
$resn_exch{"2HO*  IC"}="HO2'  IC"; $resn_exch{"1H2   IC"}=" H21  IC"; $resn_exch{"2H2   IC"}=" H22  IC";
$resn_exch{"1H4  IC1"}=" H41 IC1"; $resn_exch{"1H5  IC1"}=" H51 IC1"; $resn_exch{"1H6  IC1"}=" H61 IC1";
$resn_exch{"1H7  IC1"}=" H71 IC1"; $resn_exch{"1H10 IC1"}="H101 IC1"; $resn_exch{"1H3' IC1"}="H3'1 IC1";
$resn_exch{"1H5' IC1"}="H5'1 IC1"; $resn_exch{"1H11 IC1"}="H111 IC1"; $resn_exch{"2H11 IC1"}="H112 IC1";
$resn_exch{"3H11 IC1"}="H113 IC1"; $resn_exch{"1H12 IC1"}="H121 IC1"; $resn_exch{"2H12 IC1"}="H122 IC1";
$resn_exch{"3H12 IC1"}="H123 IC1"; $resn_exch{"1H13 IC1"}="H131 IC1"; $resn_exch{"2H13 IC1"}="H132 IC1";
$resn_exch{"3H13 IC1"}="H133 IC1"; $resn_exch{"1H4  ICA"}=" H41 ICA"; $resn_exch{"2H4  ICA"}=" H42 ICA";
$resn_exch{"1H4  ICI"}=" H41 ICI"; $resn_exch{"2H4  ICI"}=" H42 ICI"; $resn_exch{"1H11 ICL"}="H111 ICL";
$resn_exch{"2H11 ICL"}="H112 ICL"; $resn_exch{"3H11 ICL"}="H113 ICL"; $resn_exch{"1H12 ICL"}="H121 ICL";
$resn_exch{"2H12 ICL"}="H122 ICL"; $resn_exch{"3H12 ICL"}="H123 ICL"; $resn_exch{"1H15 ICL"}="H151 ICL";
$resn_exch{"2H15 ICL"}="H152 ICL"; $resn_exch{"3H15 ICL"}="H153 ICL"; $resn_exch{"1H16 ICL"}="H161 ICL";
$resn_exch{"2H16 ICL"}="H162 ICL"; $resn_exch{"3H16 ICL"}="H163 ICL"; $resn_exch{"1H17 ICL"}="H171 ICL";
$resn_exch{"2H17 ICL"}="H172 ICL"; $resn_exch{"3H17 ICL"}="H173 ICL"; $resn_exch{"1H1  ICN"}=" H11 ICN";
$resn_exch{"2H1  ICN"}=" H12 ICN"; $resn_exch{"3H1  ICN"}=" H13 ICN"; $resn_exch{"1H3  ICN"}=" H31 ICN";
$resn_exch{"2H3  ICN"}=" H32 ICN"; $resn_exch{"3H3  ICN"}=" H33 ICN"; $resn_exch{"1H5  ICN"}=" H51 ICN";
$resn_exch{"2H5  ICN"}=" H52 ICN"; $resn_exch{"3H5  ICN"}=" H53 ICN"; $resn_exch{"1HM2 ICP"}="HM21 ICP";
$resn_exch{"2HM2 ICP"}="HM22 ICP"; $resn_exch{"3HM2 ICP"}="HM23 ICP"; $resn_exch{"1H6A ICP"}="H6A1 ICP";
$resn_exch{"1H7A ICP"}="H7A1 ICP"; $resn_exch{"2H7A ICP"}="H7A2 ICP"; $resn_exch{"1H5' ICR"}="H5'1 ICR";
$resn_exch{"2H5' ICR"}="H5'2 ICR"; $resn_exch{"1H4  ICT"}=" H41 ICT"; $resn_exch{"2H4  ICT"}=" H42 ICT";
$resn_exch{"1HN1 ICU"}="HN11 ICU"; $resn_exch{"2HN1 ICU"}="HN12 ICU"; $resn_exch{"1H10 ICU"}="H101 ICU";
$resn_exch{"2H10 ICU"}="H102 ICU"; $resn_exch{"3H10 ICU"}="H103 ICU"; $resn_exch{"1H12 ICU"}="H121 ICU";
$resn_exch{"2H12 ICU"}="H122 ICU"; $resn_exch{"3H12 ICU"}="H123 ICU"; $resn_exch{"*HO5 ID2"}="HO5' ID2";
$resn_exch{"1H5* ID2"}="H5'1 ID2"; $resn_exch{"2H5* ID2"}="H5'2 ID2"; $resn_exch{" H4* ID2"}=" H4' ID2";
$resn_exch{" H1* ID2"}=" H1' ID2"; $resn_exch{"1H2* ID2"}="H2'1 ID2"; $resn_exch{"2H2* ID2"}="H2'2 ID2";
$resn_exch{" H3* ID2"}=" H3' ID2"; $resn_exch{"*HO3 ID2"}="HO3' ID2"; $resn_exch{"1H18 ID5"}="H181 ID5";
$resn_exch{"2H18 ID5"}="H182 ID5"; $resn_exch{"1H20 ID5"}="H201 ID5"; $resn_exch{"2H20 ID5"}="H202 ID5";
$resn_exch{"1HN2 IDA"}="HN21 IDA"; $resn_exch{"2HN2 IDA"}="HN22 IDA"; $resn_exch{"1HC9 IDA"}="HC91 IDA";
$resn_exch{"2HC9 IDA"}="HC92 IDA"; $resn_exch{"1H12 IDA"}="H121 IDA"; $resn_exch{"2H12 IDA"}="H122 IDA";
$resn_exch{"1H13 IDA"}="H131 IDA"; $resn_exch{"2H13 IDA"}="H132 IDA"; $resn_exch{"1H14 IDA"}="H141 IDA";
$resn_exch{"2H14 IDA"}="H142 IDA"; $resn_exch{"3H14 IDA"}="H143 IDA"; $resn_exch{"1H11 IDA"}="H111 IDA";
$resn_exch{"2H11 IDA"}="H112 IDA"; $resn_exch{"1H8C IDB"}="H8C1 IDB"; $resn_exch{"2H8C IDB"}="H8C2 IDB";
$resn_exch{"1H10 IDB"}="H101 IDB"; $resn_exch{"2H10 IDB"}="H102 IDB"; $resn_exch{"1H11 IDB"}="H111 IDB";
$resn_exch{"2H11 IDB"}="H112 IDB"; $resn_exch{"1H19 IDB"}="H191 IDB"; $resn_exch{"2H19 IDB"}="H192 IDB";
$resn_exch{"1HCA IDC"}="HCA1 IDC"; $resn_exch{"2HCA IDC"}="HCA2 IDC"; $resn_exch{"1HCB IDC"}="HCB1 IDC";
$resn_exch{"2HCB IDC"}="HCB2 IDC"; $resn_exch{"1H2  IDD"}=" H21 IDD"; $resn_exch{"1H3  IDD"}=" H31 IDD";
$resn_exch{"1H4  IDD"}=" H41 IDD"; $resn_exch{"1H5  IDD"}=" H51 IDD"; $resn_exch{"1H6  IDD"}=" H61 IDD";
$resn_exch{"1H2B IDD"}="H2B1 IDD"; $resn_exch{"1H3B IDD"}="H3B1 IDD"; $resn_exch{"1H4B IDD"}="H4B1 IDD";
$resn_exch{"1H5B IDD"}="H5B1 IDD"; $resn_exch{"1H6B IDD"}="H6B1 IDD"; $resn_exch{"2H6B IDD"}="H6B2 IDD";
$resn_exch{"1HO2 IDD"}="HO21 IDD"; $resn_exch{"1HO3 IDD"}="HO31 IDD"; $resn_exch{"1HO4 IDD"}="HO41 IDD";
$resn_exch{"1HO6 IDD"}="HO61 IDD"; $resn_exch{"1H8B IDD"}="H8B1 IDD"; $resn_exch{"1H6B IDE"}="H6B1 IDE";
$resn_exch{"2H6B IDE"}="H6B2 IDE"; $resn_exch{"1H7  IDE"}=" H71 IDE"; $resn_exch{"2H7  IDE"}=" H72 IDE";
$resn_exch{"1HN2 IDG"}="HN21 IDG"; $resn_exch{"2HN2 IDG"}="HN22 IDG"; $resn_exch{"1H6  IDG"}=" H61 IDG";
$resn_exch{"2H6  IDG"}=" H62 IDG"; $resn_exch{"1HN6 IDG"}="HN61 IDG"; $resn_exch{"2HN6 IDG"}="HN62 IDG";
$resn_exch{"1H6C IDH"}="H6C1 IDH"; $resn_exch{"2H6C IDH"}="H6C2 IDH"; $resn_exch{"1HC4 IDI"}="HC41 IDI";
$resn_exch{"1HC5 IDI"}="HC51 IDI"; $resn_exch{"1HC7 IDI"}="HC71 IDI"; $resn_exch{"1HC1 IDI"}="HC11 IDI";
$resn_exch{"2HC1 IDI"}="HC12 IDI"; $resn_exch{"1HC2 IDI"}="HC21 IDI"; $resn_exch{"2HC2 IDI"}="HC22 IDI";
$resn_exch{"1HC3 IDI"}="HC31 IDI"; $resn_exch{"2HC3 IDI"}="HC32 IDI"; $resn_exch{"1HN1 IDI"}="HN11 IDI";
$resn_exch{"1H2  IDM"}=" H21 IDM"; $resn_exch{"2H2  IDM"}=" H22 IDM"; $resn_exch{"1H3  IDM"}=" H31 IDM";
$resn_exch{"2H3  IDM"}=" H32 IDM"; $resn_exch{"1H12 IDN"}="H121 IDN"; $resn_exch{"2H12 IDN"}="H122 IDN";
$resn_exch{"3H12 IDN"}="H123 IDN"; $resn_exch{"1H13 IDN"}="H131 IDN"; $resn_exch{"2H13 IDN"}="H132 IDN";
$resn_exch{"1H20 IDN"}="H201 IDN"; $resn_exch{"2H20 IDN"}="H202 IDN"; $resn_exch{"3H20 IDN"}="H203 IDN";
$resn_exch{"1H25 IDN"}="H251 IDN"; $resn_exch{"2H25 IDN"}="H252 IDN"; $resn_exch{"1H26 IDN"}="H261 IDN";
$resn_exch{"2H26 IDN"}="H262 IDN"; $resn_exch{"2HOB IDP"}="HOB2 IDP"; $resn_exch{"3HOB IDP"}="HOB3 IDP";
$resn_exch{"2HOA IDP"}="HOA2 IDP"; $resn_exch{"1H5* IDP"}="H5'1 IDP"; $resn_exch{"2H5* IDP"}="H5'2 IDP";
$resn_exch{" H4* IDP"}=" H4' IDP"; $resn_exch{" H3* IDP"}=" H3' IDP"; $resn_exch{"*HO3 IDP"}="HO3' IDP";
$resn_exch{" H2* IDP"}=" H2' IDP"; $resn_exch{"*HO2 IDP"}="HO2' IDP"; $resn_exch{" H1* IDP"}=" H1' IDP";
$resn_exch{"3HOS IDS"}="HOS3 IDS"; $resn_exch{"1H1  IDU"}=" H11 IDU"; $resn_exch{"2H1  IDU"}=" H12 IDU";
$resn_exch{"1H4  IDU"}=" H41 IDU"; $resn_exch{"2H4  IDU"}=" H42 IDU"; $resn_exch{"3HOS IDU"}="HOS3 IDU";
$resn_exch{"1H7  IEM"}=" H71 IEM"; $resn_exch{"2H7  IEM"}=" H72 IEM"; $resn_exch{"1H9  IEM"}=" H91 IEM";
$resn_exch{"2H9  IEM"}=" H92 IEM"; $resn_exch{"3H9  IEM"}=" H93 IEM"; $resn_exch{"1H10 IEM"}="H101 IEM";
$resn_exch{"2H10 IEM"}="H102 IEM"; $resn_exch{"3H10 IEM"}="H103 IEM"; $resn_exch{"1H12 IEM"}="H121 IEM";
$resn_exch{"2H12 IEM"}="H122 IEM"; $resn_exch{"3H12 IEM"}="H123 IEM"; $resn_exch{"1H7  IET"}=" H71 IET";
$resn_exch{"2H7  IET"}=" H72 IET"; $resn_exch{"1HA3 IEY"}="HA31 IEY"; $resn_exch{"2HA3 IEY"}="HA32 IEY";
$resn_exch{"1HB2 IEY"}="HB21 IEY"; $resn_exch{"2HB2 IEY"}="HB22 IEY"; $resn_exch{"1HC6 IFG"}="HC61 IFG";
$resn_exch{"2HC6 IFG"}="HC62 IFG"; $resn_exch{"1HC8 IFG"}="HC81 IFG"; $resn_exch{"2HC8 IFG"}="HC82 IFG";
$resn_exch{"3HC8 IFG"}="HC83 IFG"; $resn_exch{"1HC9 IFG"}="HC91 IFG"; $resn_exch{"2HC9 IFG"}="HC92 IFG";
$resn_exch{"1H7C IFL"}="H7C1 IFL"; $resn_exch{"2H7C IFL"}="H7C2 IFL"; $resn_exch{"1H6C IFL"}="H6C1 IFL";
$resn_exch{"2H6C IFL"}="H6C2 IFL"; $resn_exch{"1H2C IFM"}="H2C1 IFM"; $resn_exch{"2H2C IFM"}="H2C2 IFM";
$resn_exch{"1H1C IFM"}="H1C1 IFM"; $resn_exch{"2H1C IFM"}="H1C2 IFM"; $resn_exch{"1H6C IFM"}="H6C1 IFM";
$resn_exch{"2H6C IFM"}="H6C2 IFM"; $resn_exch{"1H6A IFP"}="H6A1 IFP"; $resn_exch{"1H7A IFP"}="H7A1 IFP";
$resn_exch{"2H7A IFP"}="H7A2 IFP"; $resn_exch{"2HOP  IG"}="HOP2  IG"; $resn_exch{"3HOP  IG"}="HOP3  IG";
$resn_exch{"H3T   IG"}="HO3'  IG"; $resn_exch{" H1*  IG"}=" H1'  IG"; $resn_exch{" H2*  IG"}=" H2'  IG";
$resn_exch{" H3*  IG"}=" H3'  IG"; $resn_exch{" H4*  IG"}=" H4'  IG"; $resn_exch{"1H5*  IG"}=" H5'  IG";
$resn_exch{"2H5*  IG"}="H5''  IG"; $resn_exch{"2HO*  IG"}="HO2'  IG"; $resn_exch{"1H6   IG"}=" H61  IG";
$resn_exch{"2H6   IG"}=" H62  IG"; $resn_exch{"1H1  IGC"}="H11A IGC"; $resn_exch{"2H1  IGC"}="H12A IGC";
$resn_exch{"3H1  IGC"}="H13A IGC"; $resn_exch{"1H2  IGC"}=" H21 IGC"; $resn_exch{"2H2  IGC"}="H22A IGC";
$resn_exch{"1H3  IGC"}="H31A IGC"; $resn_exch{"2H3  IGC"}="H32A IGC"; $resn_exch{"1H4  IGC"}=" H41 IGC";
$resn_exch{"2H4  IGC"}=" H42 IGC"; $resn_exch{"1H5  IGC"}=" H51 IGC"; $resn_exch{"2H5  IGC"}=" H52 IGC";
$resn_exch{"1H6  IGC"}=" H61 IGC"; $resn_exch{"2H6  IGC"}=" H62 IGC"; $resn_exch{"1H7  IGC"}=" H71 IGC";
$resn_exch{"2H7  IGC"}=" H72 IGC"; $resn_exch{"1H8  IGC"}=" H81 IGC"; $resn_exch{"2H8  IGC"}=" H82 IGC";
$resn_exch{"1H9  IGC"}=" H91 IGC"; $resn_exch{"2H9  IGC"}=" H92 IGC"; $resn_exch{"1H10 IGC"}="H101 IGC";
$resn_exch{"2H10 IGC"}="H102 IGC"; $resn_exch{"1H11 IGC"}="H111 IGC"; $resn_exch{"2H11 IGC"}="H112 IGC";
$resn_exch{"1H12 IGC"}="H121 IGC"; $resn_exch{"2H12 IGC"}="H122 IGC"; $resn_exch{"1H13 IGC"}="H131 IGC";
$resn_exch{"2H13 IGC"}="H132 IGC"; $resn_exch{"1H18 IGC"}="H181 IGC"; $resn_exch{"2H18 IGC"}="H182 IGC";
$resn_exch{"1H21 IGC"}="H211 IGC"; $resn_exch{"2H21 IGC"}="H212 IGC"; $resn_exch{"1H25 IGC"}="H251 IGC";
$resn_exch{"2H25 IGC"}="H252 IGC"; $resn_exch{"1H30 IGC"}="H301 IGC"; $resn_exch{"2H30 IGC"}="H302 IGC";
$resn_exch{"1H38 IGC"}="H381 IGC"; $resn_exch{"2H38 IGC"}="H382 IGC"; $resn_exch{"1H39 IGC"}="H391 IGC";
$resn_exch{"2H39 IGC"}="H392 IGC"; $resn_exch{"1H40 IGC"}="H401 IGC"; $resn_exch{"2H40 IGC"}="H402 IGC";
$resn_exch{"1H41 IGC"}="H411 IGC"; $resn_exch{"2H41 IGC"}="H412 IGC"; $resn_exch{"1H42 IGC"}="H421 IGC";
$resn_exch{"2H42 IGC"}="H422 IGC"; $resn_exch{"1H43 IGC"}="H431 IGC"; $resn_exch{"2H43 IGC"}="H432 IGC";
$resn_exch{"1H44 IGC"}="H441 IGC"; $resn_exch{"2H44 IGC"}="H442 IGC"; $resn_exch{"3H44 IGC"}="H443 IGC";
$resn_exch{"1HN  IGL"}=" H   IGL"; $resn_exch{"2HN  IGL"}=" H2  IGL"; $resn_exch{"1HG1 IGL"}="HG12 IGL";
$resn_exch{"2HG1 IGL"}="HG13 IGL"; $resn_exch{"1HG2 IGL"}="HG22 IGL"; $resn_exch{"2HG2 IGL"}="HG23 IGL";
$resn_exch{"1H3  IGN"}=" H31 IGN"; $resn_exch{"1H4  IGN"}=" H41 IGN"; $resn_exch{"2H4  IGN"}=" H42 IGN";
$resn_exch{"1H5  IGN"}=" H51 IGN"; $resn_exch{"2H5  IGN"}=" H52 IGN"; $resn_exch{"1H6  IGN"}=" H61 IGN";
$resn_exch{"2H6  IGN"}=" H62 IGN"; $resn_exch{"1H7  IGN"}=" H71 IGN"; $resn_exch{"1H9  IGN"}=" H91 IGN";
$resn_exch{"1H10 IGN"}="H101 IGN"; $resn_exch{"2H10 IGN"}="H102 IGN"; $resn_exch{"1H12 IGN"}="H121 IGN";
$resn_exch{"1H13 IGN"}="H131 IGN"; $resn_exch{"2H13 IGN"}="H132 IGN"; $resn_exch{"1H14 IGN"}="H141 IGN";
$resn_exch{"2H14 IGN"}="H142 IGN"; $resn_exch{"1H15 IGN"}="H151 IGN"; $resn_exch{"2H15 IGN"}="H152 IGN";
$resn_exch{"1H16 IGN"}="H161 IGN"; $resn_exch{"2H16 IGN"}="H162 IGN"; $resn_exch{"1H33 IGN"}="H331 IGN";
$resn_exch{"1H34 IGN"}="H341 IGN"; $resn_exch{"2H34 IGN"}="H342 IGN"; $resn_exch{"1H51 IGN"}="H511 IGN";
$resn_exch{"2H51 IGN"}="H512 IGN"; $resn_exch{"1H52 IGN"}="H521 IGN"; $resn_exch{"1H53 IGN"}="H531 IGN";
$resn_exch{"2H53 IGN"}="H532 IGN"; $resn_exch{"1H54 IGN"}="H541 IGN"; $resn_exch{"2H54 IGN"}="H542 IGN";
$resn_exch{"1H55 IGN"}="H551 IGN"; $resn_exch{"2H55 IGN"}="H552 IGN"; $resn_exch{"1H56 IGN"}="H561 IGN";
$resn_exch{"2H56 IGN"}="H562 IGN"; $resn_exch{"1H21 IGN"}="H211 IGN"; $resn_exch{"1H22 IGN"}="H221 IGN";
$resn_exch{"2H22 IGN"}="H222 IGN"; $resn_exch{"1H25 IGN"}="H251 IGN"; $resn_exch{"1H1  IGP"}=" H11 IGP";
$resn_exch{"2H1  IGP"}=" H12 IGP"; $resn_exch{"1HOP IGP"}="HOP1 IGP"; $resn_exch{"2HOP IGP"}="HOP2 IGP";
$resn_exch{"2HOP IGU"}="HOP2 IGU"; $resn_exch{"3HOP IGU"}="HOP3 IGU"; $resn_exch{"1H5* IGU"}=" H5' IGU";
$resn_exch{"2H5* IGU"}="H5'' IGU"; $resn_exch{" H4* IGU"}=" H4' IGU"; $resn_exch{" H3* IGU"}=" H3' IGU";
$resn_exch{"*HO3 IGU"}="HO3' IGU"; $resn_exch{"1H2* IGU"}=" H2' IGU"; $resn_exch{"2H2* IGU"}="H2'' IGU";
$resn_exch{" H1* IGU"}=" H1' IGU"; $resn_exch{"1HN6 IGU"}="HN61 IGU"; $resn_exch{"2HN6 IGU"}="HN62 IGU";
$resn_exch{"1H23 IH1"}="H231 IH1"; $resn_exch{"2H23 IH1"}="H232 IH1"; $resn_exch{"1H22 IH1"}="H221 IH1";
$resn_exch{"2H22 IH1"}="H222 IH1"; $resn_exch{"1H16 IH1"}="H161 IH1"; $resn_exch{"2H16 IH1"}="H162 IH1";
$resn_exch{"1HC1 IH1"}="HC11 IH1"; $resn_exch{"2HC1 IH1"}="HC12 IH1"; $resn_exch{"1HC3 IH1"}="HC31 IH1";
$resn_exch{"2HC3 IH1"}="HC32 IH1"; $resn_exch{"1HC4 IH1"}="HC41 IH1"; $resn_exch{"2HC4 IH1"}="HC42 IH1";
$resn_exch{"1HC6 IH1"}="HC61 IH1"; $resn_exch{"2HC6 IH1"}="HC62 IH1"; $resn_exch{"1HC7 IH1"}="HC71 IH1";
$resn_exch{"2HC7 IH1"}="HC72 IH1"; $resn_exch{"1H11 IH1"}="H111 IH1"; $resn_exch{"2H11 IH1"}="H112 IH1";
$resn_exch{"1H1  IH2"}=" H11 IH2"; $resn_exch{"2H1  IH2"}=" H12 IH2"; $resn_exch{"1H10 IH2"}="H101 IH2";
$resn_exch{"2H10 IH2"}="H102 IH2"; $resn_exch{"1H7  IH2"}=" H71 IH2"; $resn_exch{"2H7  IH2"}=" H72 IH2";
$resn_exch{"1H6  IH2"}=" H61 IH2"; $resn_exch{"2H6  IH2"}=" H62 IH2"; $resn_exch{"1H4  IH2"}=" H41 IH2";
$resn_exch{"2H4  IH2"}=" H42 IH2"; $resn_exch{"1H3  IH2"}=" H31 IH2"; $resn_exch{"2H3  IH2"}="H32A IH2";
$resn_exch{"1H17 IH2"}="H171 IH2"; $resn_exch{"2H17 IH2"}="H172 IH2"; $resn_exch{"1H16 IH2"}="H161 IH2";
$resn_exch{"2H16 IH2"}="H162 IH2"; $resn_exch{"3H16 IH2"}="H163 IH2"; $resn_exch{"1H23 IH2"}="H231 IH2";
$resn_exch{"2H23 IH2"}="H232 IH2"; $resn_exch{"1HNB IH3"}="HNB1 IH3"; $resn_exch{"2HNB IH3"}="HNB2 IH3";
$resn_exch{"1HC7 IH3"}="HC71 IH3"; $resn_exch{"2HC7 IH3"}="HC72 IH3"; $resn_exch{"1HC9 IH3"}="HC91 IH3";
$resn_exch{"2HC9 IH3"}="HC92 IH3"; $resn_exch{"1HC4 IH3"}="HC41 IH3"; $resn_exch{"2HC4 IH3"}="HC42 IH3";
$resn_exch{"1HC6 IH3"}="HC61 IH3"; $resn_exch{"2HC6 IH3"}="HC62 IH3"; $resn_exch{"1H16 IH3"}="H161 IH3";
$resn_exch{"2H16 IH3"}="H162 IH3"; $resn_exch{"1H15 IH3"}="H151 IH3"; $resn_exch{"2H15 IH3"}="H152 IH3";
$resn_exch{"3H15 IH3"}="H153 IH3"; $resn_exch{"1H22 IH3"}="H221 IH3"; $resn_exch{"2H22 IH3"}="H222 IH3";
$resn_exch{"1H29 IH4"}="H291 IH4"; $resn_exch{"2H29 IH4"}="H292 IH4"; $resn_exch{"3H29 IH4"}="H293 IH4";
$resn_exch{"1H25 IH4"}="H251 IH4"; $resn_exch{"2H25 IH4"}="H252 IH4"; $resn_exch{"1H16 IH4"}="H161 IH4";
$resn_exch{"2H16 IH4"}="H162 IH4"; $resn_exch{"1H38 IH4"}="H381 IH4"; $resn_exch{"2H38 IH4"}="H382 IH4";
$resn_exch{"1H39 IH4"}="H391 IH4"; $resn_exch{"2H39 IH4"}="H392 IH4"; $resn_exch{"1H40 IH4"}="H401 IH4";
$resn_exch{"2H40 IH4"}="H402 IH4"; $resn_exch{"1H41 IH4"}="H411 IH4"; $resn_exch{"2H41 IH4"}="H412 IH4";
$resn_exch{"1H42 IH4"}="H421 IH4"; $resn_exch{"2H42 IH4"}="H422 IH4"; $resn_exch{"3H42 IH4"}="H423 IH4";
$resn_exch{"1H5C IH4"}="H5C1 IH4"; $resn_exch{"2H5C IH4"}="H5C2 IH4"; $resn_exch{"1H6C IH4"}="H6C1 IH4";
$resn_exch{"2H6C IH4"}="H6C2 IH4"; $resn_exch{"1H14 IH4"}="H141 IH4"; $resn_exch{"2H14 IH4"}="H142 IH4";
$resn_exch{"1H15 IH4"}="H151 IH4"; $resn_exch{"2H15 IH4"}="H152 IH4"; $resn_exch{"1H8C IH4"}="H8C1 IH4";
$resn_exch{"2H8C IH4"}="H8C2 IH4"; $resn_exch{"1H13 IH5"}="H131 IH5"; $resn_exch{"2H13 IH5"}="H132 IH5";
$resn_exch{"1H16 IH5"}="H161 IH5"; $resn_exch{"2H16 IH5"}="H162 IH5"; $resn_exch{"3H16 IH5"}="H163 IH5";
$resn_exch{"1H18 IH5"}="H181 IH5"; $resn_exch{"2H18 IH5"}="H182 IH5"; $resn_exch{"3H18 IH5"}="H183 IH5";
$resn_exch{"1HL5 IH5"}="HL51 IH5"; $resn_exch{"2HL5 IH5"}="HL52 IH5"; $resn_exch{"3HL5 IH5"}="HL53 IH5";
$resn_exch{"1HL6 IH5"}="HL61 IH5"; $resn_exch{"2HL6 IH5"}="HL62 IH5"; $resn_exch{"3HL6 IH5"}="HL63 IH5";
$resn_exch{"1H1  IH6"}=" H11 IH6"; $resn_exch{"2H1  IH6"}=" H12 IH6"; $resn_exch{"3H1  IH6"}=" H13 IH6";
$resn_exch{"1H4  IH6"}=" H41 IH6"; $resn_exch{"2H4  IH6"}=" H42 IH6"; $resn_exch{"1H11 IH6"}="H111 IH6";
$resn_exch{"2H11 IH6"}="H112 IH6"; $resn_exch{"1H12 IH6"}="H121 IH6"; $resn_exch{"2H12 IH6"}="H122 IH6";
$resn_exch{"1H16 IH6"}="H161 IH6"; $resn_exch{"2H16 IH6"}="H162 IH6"; $resn_exch{"3H16 IH6"}="H163 IH6";
$resn_exch{"1H17 IH6"}="H171 IH6"; $resn_exch{"2H17 IH6"}="H172 IH6"; $resn_exch{"3H17 IH6"}="H173 IH6";
$resn_exch{"1H17 IHD"}="H171 IHD"; $resn_exch{"2H17 IHD"}="H172 IHD"; $resn_exch{"3H17 IHD"}="H173 IHD";
$resn_exch{"1H16 IHD"}="H161 IHD"; $resn_exch{"2H16 IHD"}="H162 IHD"; $resn_exch{"1H15 IHD"}="H151 IHD";
$resn_exch{"2H15 IHD"}="H152 IHD"; $resn_exch{"1H14 IHD"}="H141 IHD"; $resn_exch{"2H14 IHD"}="H142 IHD";
$resn_exch{"1H13 IHD"}="H131 IHD"; $resn_exch{"2H13 IHD"}="H132 IHD"; $resn_exch{"1H12 IHD"}="H121 IHD";
$resn_exch{"2H12 IHD"}="H122 IHD"; $resn_exch{"1H11 IHD"}="H111 IHD"; $resn_exch{"2H11 IHD"}="H112 IHD";
$resn_exch{"1H10 IHD"}="H101 IHD"; $resn_exch{"2H10 IHD"}="H102 IHD"; $resn_exch{"1H9  IHD"}=" H91 IHD";
$resn_exch{"2H9  IHD"}=" H92 IHD"; $resn_exch{"1H8  IHD"}=" H81 IHD"; $resn_exch{"2H8  IHD"}=" H82 IHD";
$resn_exch{"1H7  IHD"}=" H71 IHD"; $resn_exch{"2H7  IHD"}=" H72 IHD"; $resn_exch{"1H6  IHD"}=" H61 IHD";
$resn_exch{"2H6  IHD"}=" H62 IHD"; $resn_exch{"1H5  IHD"}=" H51 IHD"; $resn_exch{"2H5  IHD"}=" H52 IHD";
$resn_exch{"1H4  IHD"}=" H41 IHD"; $resn_exch{"2H4  IHD"}=" H42 IHD"; $resn_exch{"1H3  IHD"}=" H31 IHD";
$resn_exch{"2H3  IHD"}=" H32 IHD"; $resn_exch{"1H2  IHD"}=" H21 IHD"; $resn_exch{"2H2  IHD"}=" H22 IHD";
$resn_exch{"1H01 IHE"}="H011 IHE"; $resn_exch{"2H01 IHE"}="H012 IHE"; $resn_exch{"1H02 IHE"}="H021 IHE";
$resn_exch{"2H02 IHE"}="H022 IHE"; $resn_exch{"1H04 IHE"}="H041 IHE"; $resn_exch{"2H04 IHE"}="H042 IHE";
$resn_exch{"1H05 IHE"}="H051 IHE"; $resn_exch{"2H05 IHE"}="H052 IHE"; $resn_exch{"1H15 IHE"}="H151 IHE";
$resn_exch{"2H15 IHE"}="H152 IHE"; $resn_exch{"3H15 IHE"}="H153 IHE"; $resn_exch{"1H16 IHE"}="H161 IHE";
$resn_exch{"2H16 IHE"}="H162 IHE"; $resn_exch{"1H17 IHE"}="H171 IHE"; $resn_exch{"2H17 IHE"}="H172 IHE";
$resn_exch{"1H30 IHE"}="H301 IHE"; $resn_exch{"2H30 IHE"}="H302 IHE"; $resn_exch{"1H31 IHE"}="H311 IHE";
$resn_exch{"2H31 IHE"}="H312 IHE"; $resn_exch{"1H32 IHE"}="H321 IHE"; $resn_exch{"2H32 IHE"}="H322 IHE";
$resn_exch{"1H33 IHE"}="H331 IHE"; $resn_exch{"2H33 IHE"}="H332 IHE"; $resn_exch{"1H34 IHE"}="H341 IHE";
$resn_exch{"2H34 IHE"}="H342 IHE"; $resn_exch{"1H1  IHG"}=" H11 IHG"; $resn_exch{"2H1  IHG"}=" H12 IHG";
$resn_exch{"3H1  IHG"}=" H13 IHG"; $resn_exch{"1H2  IHG"}=" H21 IHG"; $resn_exch{"1H3  IHG"}=" H31 IHG";
$resn_exch{"2H3  IHG"}=" H32 IHG"; $resn_exch{"3H3  IHG"}=" H33 IHG"; $resn_exch{"1HE  IHG"}=" HE1 IHG";
$resn_exch{"1HH1 IHG"}="HH11 IHG"; $resn_exch{"1HH2 IHG"}="HH21 IHG"; $resn_exch{"1HH  IHG"}=" HH1 IHG";
$resn_exch{"1H13 IHI"}="H131 IHI"; $resn_exch{"2H13 IHI"}="H132 IHI"; $resn_exch{"1H14 IHI"}="H141 IHI";
$resn_exch{"2H14 IHI"}="H142 IHI"; $resn_exch{"1H15 IHI"}="H151 IHI"; $resn_exch{"2H15 IHI"}="H152 IHI";
$resn_exch{"1H16 IHI"}="H161 IHI"; $resn_exch{"2H16 IHI"}="H162 IHI"; $resn_exch{"1H39 IHI"}="H391 IHI";
$resn_exch{"2H39 IHI"}="H392 IHI"; $resn_exch{"1H40 IHI"}="H401 IHI"; $resn_exch{"2H40 IHI"}="H402 IHI";
$resn_exch{"1H41 IHI"}="H411 IHI"; $resn_exch{"2H41 IHI"}="H412 IHI"; $resn_exch{"1H17 IHJ"}="H171 IHJ";
$resn_exch{"2H17 IHJ"}="H172 IHJ"; $resn_exch{"1H18 IHJ"}="H181 IHJ"; $resn_exch{"2H18 IHJ"}="H182 IHJ";
$resn_exch{"1H19 IHJ"}="H191 IHJ"; $resn_exch{"2H19 IHJ"}="H192 IHJ"; $resn_exch{"1H20 IHJ"}="H201 IHJ";
$resn_exch{"2H20 IHJ"}="H202 IHJ"; $resn_exch{"1H41 IHJ"}="H411 IHJ"; $resn_exch{"2H41 IHJ"}="H412 IHJ";
$resn_exch{"1H42 IHJ"}="H421 IHJ"; $resn_exch{"2H42 IHJ"}="H422 IHJ"; $resn_exch{"1H43 IHJ"}="H431 IHJ";
$resn_exch{"2H43 IHJ"}="H432 IHJ"; $resn_exch{"1H52 IHJ"}="H521 IHJ"; $resn_exch{"2H52 IHJ"}="H522 IHJ";
$resn_exch{"1H53 IHJ"}="H531 IHJ"; $resn_exch{"2H53 IHJ"}="H532 IHJ"; $resn_exch{"1H55 IHJ"}="H551 IHJ";
$resn_exch{"2H55 IHJ"}="H552 IHJ"; $resn_exch{"1H56 IHJ"}="H561 IHJ"; $resn_exch{"2H56 IHJ"}="H562 IHJ";
$resn_exch{"1H61 IHJ"}="H611 IHJ"; $resn_exch{"2H61 IHJ"}="H612 IHJ"; $resn_exch{"3H61 IHJ"}="H613 IHJ";
$resn_exch{"1H2  IHN"}=" H21 IHN"; $resn_exch{"2H2  IHN"}=" H22 IHN"; $resn_exch{"1H10 IHN"}="H101 IHN";
$resn_exch{"2H10 IHN"}="H102 IHN"; $resn_exch{"1H13 IHN"}="H131 IHN"; $resn_exch{"2H13 IHN"}="H132 IHN";
$resn_exch{"1H22 IHN"}="H221 IHN"; $resn_exch{"2H22 IHN"}="H222 IHN"; $resn_exch{"1H24 IHN"}="H241 IHN";
$resn_exch{"2H24 IHN"}="H242 IHN"; $resn_exch{"3H24 IHN"}="H243 IHN"; $resn_exch{"1H25 IHN"}="H251 IHN";
$resn_exch{"2H25 IHN"}="H252 IHN"; $resn_exch{"1H27 IHN"}="H271 IHN"; $resn_exch{"2H27 IHN"}="H272 IHN";
$resn_exch{"3H27 IHN"}="H273 IHN"; $resn_exch{"1H28 IHN"}="H281 IHN"; $resn_exch{"2H28 IHN"}="H282 IHN";
$resn_exch{"3H28 IHN"}="H283 IHN"; $resn_exch{"1H29 IHN"}="H291 IHN"; $resn_exch{"2H29 IHN"}="H292 IHN";
$resn_exch{"3H29 IHN"}="H293 IHN"; $resn_exch{"1H31 IHN"}="H311 IHN"; $resn_exch{"2H31 IHN"}="H312 IHN";
$resn_exch{"3H31 IHN"}="H313 IHN"; $resn_exch{"1H32 IHN"}="H321 IHN"; $resn_exch{"2H32 IHN"}="H322 IHN";
$resn_exch{"3H32 IHN"}="H323 IHN"; $resn_exch{"1H34 IHN"}="H341 IHN"; $resn_exch{"2H34 IHN"}="H342 IHN";
$resn_exch{"3H34 IHN"}="H343 IHN"; $resn_exch{"1H35 IHN"}="H351 IHN"; $resn_exch{"2H35 IHN"}="H352 IHN";
$resn_exch{"3H35 IHN"}="H353 IHN"; $resn_exch{"1H37 IHN"}="H371 IHN"; $resn_exch{"2H37 IHN"}="H372 IHN";
$resn_exch{"3H37 IHN"}="H373 IHN"; $resn_exch{"1H38 IHN"}="H381 IHN"; $resn_exch{"2H38 IHN"}="H382 IHN";
$resn_exch{"3H38 IHN"}="H383 IHN"; $resn_exch{"1H39 IHN"}="H391 IHN"; $resn_exch{"2H39 IHN"}="H392 IHN";
$resn_exch{"3H39 IHN"}="H393 IHN"; $resn_exch{"2HO4 IHS"}="HO42 IHS"; $resn_exch{"3HO4 IHS"}="HO43 IHS";
$resn_exch{"4HO4 IHS"}="HO44 IHS"; $resn_exch{"5HO4 IHS"}="HO45 IHS"; $resn_exch{"6HO4 IHS"}="HO46 IHS";
$resn_exch{"1H51 IHU"}="H511 IHU"; $resn_exch{"2H51 IHU"}="H512 IHU"; $resn_exch{"3H51 IHU"}="H513 IHU";
$resn_exch{"1H13 IIA"}="H131 IIA"; $resn_exch{"2H13 IIA"}="H132 IIA"; $resn_exch{"1H14 IIA"}="H141 IIA";
$resn_exch{"2H14 IIA"}="H142 IIA"; $resn_exch{"1H16 IIA"}="H161 IIA"; $resn_exch{"2H16 IIA"}="H162 IIA";
$resn_exch{"1H17 IIA"}="H171 IIA"; $resn_exch{"2H17 IIA"}="H172 IIA"; $resn_exch{"1H28 IIA"}="H281 IIA";
$resn_exch{"2H28 IIA"}="H282 IIA"; $resn_exch{"3H28 IIA"}="H283 IIA"; $resn_exch{"1H32 IIA"}="H321 IIA";
$resn_exch{"2H32 IIA"}="H322 IIA"; $resn_exch{"3H32 IIA"}="H323 IIA"; $resn_exch{"1H37 IIA"}="H371 IIA";
$resn_exch{"2H37 IIA"}="H372 IIA"; $resn_exch{"1H13 IIB"}="H131 IIB"; $resn_exch{"2H13 IIB"}="H132 IIB";
$resn_exch{"1H14 IIB"}="H141 IIB"; $resn_exch{"2H14 IIB"}="H142 IIB"; $resn_exch{"1H16 IIB"}="H161 IIB";
$resn_exch{"2H16 IIB"}="H162 IIB"; $resn_exch{"1H17 IIB"}="H171 IIB"; $resn_exch{"2H17 IIB"}="H172 IIB";
$resn_exch{"1H28 IIB"}="H281 IIB"; $resn_exch{"2H28 IIB"}="H282 IIB"; $resn_exch{"3H28 IIB"}="H283 IIB";
$resn_exch{"1H32 IIB"}="H321 IIB"; $resn_exch{"2H32 IIB"}="H322 IIB"; $resn_exch{"3H32 IIB"}="H323 IIB";
$resn_exch{"1H37 IIB"}="H371 IIB"; $resn_exch{"2H37 IIB"}="H372 IIB"; $resn_exch{"1H11 IIB"}="H111 IIB";
$resn_exch{"2H11 IIB"}="H112 IIB"; $resn_exch{"3H11 IIB"}="H113 IIB"; $resn_exch{"H21  IIC"}="HB21 IIC";
$resn_exch{"1H14 IID"}="H141 IID"; $resn_exch{"2H14 IID"}="H142 IID"; $resn_exch{"1H17 IID"}="H171 IID";
$resn_exch{"2H17 IID"}="H172 IID"; $resn_exch{"1H26 IID"}="H261 IID"; $resn_exch{"2H26 IID"}="H262 IID";
$resn_exch{"1H7C IID"}="H7C1 IID"; $resn_exch{"2H7C IID"}="H7C2 IID"; $resn_exch{"3H7C IID"}="H7C3 IID";
$resn_exch{"1H10 IID"}="H101 IID"; $resn_exch{"2H10 IID"}="H102 IID"; $resn_exch{"1H12 IID"}="H121 IID";
$resn_exch{"2H12 IID"}="H122 IID"; $resn_exch{"1H19 IID"}="H191 IID"; $resn_exch{"2H19 IID"}="H192 IID";
$resn_exch{"3H19 IID"}="H193 IID"; $resn_exch{"1H20 IID"}="H201 IID"; $resn_exch{"2H20 IID"}="H202 IID";
$resn_exch{"3H20 IID"}="H203 IID"; $resn_exch{"1H13 IIE"}="H131 IIE"; $resn_exch{"2H13 IIE"}="H132 IIE";
$resn_exch{"1H14 IIE"}="H141 IIE"; $resn_exch{"2H14 IIE"}="H142 IIE"; $resn_exch{"1H16 IIE"}="H161 IIE";
$resn_exch{"2H16 IIE"}="H162 IIE"; $resn_exch{"1H17 IIE"}="H171 IIE"; $resn_exch{"2H17 IIE"}="H172 IIE";
$resn_exch{"1H28 IIE"}="H281 IIE"; $resn_exch{"2H28 IIE"}="H282 IIE"; $resn_exch{"3H28 IIE"}="H283 IIE";
$resn_exch{"1H32 IIE"}="H321 IIE"; $resn_exch{"2H32 IIE"}="H322 IIE"; $resn_exch{"3H32 IIE"}="H323 IIE";
$resn_exch{"1H37 IIE"}="H371 IIE"; $resn_exch{"2H37 IIE"}="H372 IIE"; $resn_exch{" HN2 IIL"}=" H2  IIL";
$resn_exch{"1HG2 IIL"}="HG21 IIL"; $resn_exch{"2HG2 IIL"}="HG22 IIL"; $resn_exch{"3HG2 IIL"}="HG23 IIL";
$resn_exch{"1HG1 IIL"}="HG12 IIL"; $resn_exch{"2HG1 IIL"}="HG13 IIL"; $resn_exch{"1HD1 IIL"}="HD11 IIL";
$resn_exch{"2HD1 IIL"}="HD12 IIL"; $resn_exch{"3HD1 IIL"}="HD13 IIL"; $resn_exch{"1HM2 IIN"}="HM21 IIN";
$resn_exch{"2HM2 IIN"}="HM22 IIN"; $resn_exch{"3HM2 IIN"}="HM23 IIN"; $resn_exch{"1HN3 IIN"}="HN31 IIN";
$resn_exch{"2HN3 IIN"}="HN32 IIN"; $resn_exch{"1H3  IIP"}=" H31 IIP"; $resn_exch{"2H3  IIP"}=" H32 IIP";
$resn_exch{"3H3  IIP"}=" H33 IIP"; $resn_exch{"1H9  IIP"}=" H91 IIP"; $resn_exch{"2H9  IIP"}=" H92 IIP";
$resn_exch{"3H9  IIP"}=" H93 IIP"; $resn_exch{"1H15 IIP"}="H151 IIP"; $resn_exch{"2H15 IIP"}="H152 IIP";
$resn_exch{"3H15 IIP"}="H153 IIP"; $resn_exch{"1H18 IIP"}="H181 IIP"; $resn_exch{"2H18 IIP"}="H182 IIP";
$resn_exch{"1H19 IIP"}="H191 IIP"; $resn_exch{"2H19 IIP"}="H192 IIP"; $resn_exch{"1H20 IIP"}="H201 IIP";
$resn_exch{"2H20 IIP"}="H202 IIP"; $resn_exch{"1H21 IIP"}="H211 IIP"; $resn_exch{"2H21 IIP"}="H212 IIP";
$resn_exch{"3H21 IIP"}="H213 IIP"; $resn_exch{"1H22 IIP"}="H221 IIP"; $resn_exch{"2H22 IIP"}="H222 IIP";
$resn_exch{"3H22 IIP"}="H223 IIP"; $resn_exch{"1H28 IIP"}="H281 IIP"; $resn_exch{"2H28 IIP"}="H282 IIP";
$resn_exch{"3H28 IIP"}="H283 IIP"; $resn_exch{"1H29 IIP"}="H291 IIP"; $resn_exch{"2H29 IIP"}="H292 IIP";
$resn_exch{"1H30 IIP"}="H301 IIP"; $resn_exch{"2H30 IIP"}="H302 IIP"; $resn_exch{"1H2A IK2"}="H2A1 IK2";
$resn_exch{"2H2A IK2"}="H2A2 IK2"; $resn_exch{"3H2A IK2"}="H2A3 IK2"; $resn_exch{"1H4A IK2"}="H4A1 IK2";
$resn_exch{"2H4A IK2"}="H4A2 IK2"; $resn_exch{"1H1' IK2"}="H1'1 IK2"; $resn_exch{"2H1' IK2"}="H1'2 IK2";
$resn_exch{"1H5A IK2"}="H5A1 IK2"; $resn_exch{"2H5A IK2"}="H5A2 IK2"; $resn_exch{"2HOP IK2"}="HOP2 IK2";
$resn_exch{"3HOP IK2"}="HOP3 IK2"; $resn_exch{"1HN1 IK8"}="HN11 IK8"; $resn_exch{"2HN1 IK8"}="HN12 IK8";
$resn_exch{"1H11 IK8"}="H111 IK8"; $resn_exch{"2H11 IK8"}="H112 IK8"; $resn_exch{"1H20 IK8"}="H201 IK8";
$resn_exch{"2H20 IK8"}="H202 IK8"; $resn_exch{"3H20 IK8"}="H203 IK8"; $resn_exch{"1H23 IK8"}="H231 IK8";
$resn_exch{"2H23 IK8"}="H232 IK8"; $resn_exch{"3H23 IK8"}="H233 IK8"; $resn_exch{"1H2  IKT"}=" H21 IKT";
$resn_exch{"2H2  IKT"}=" H22 IKT"; $resn_exch{"1H4  IKT"}=" H41 IKT"; $resn_exch{"2H4  IKT"}=" H42 IKT";
$resn_exch{"1H5  IKT"}=" H51 IKT"; $resn_exch{"2H5  IKT"}=" H52 IKT"; $resn_exch{"1H6  IKT"}=" H61 IKT";
$resn_exch{"2H6  IKT"}=" H62 IKT"; $resn_exch{"1H7  IKT"}=" H71 IKT"; $resn_exch{"2H7  IKT"}=" H72 IKT";
$resn_exch{"1H8  IKT"}=" H81 IKT"; $resn_exch{"2H8  IKT"}=" H82 IKT"; $resn_exch{"1H2' IKT"}="H2'1 IKT";
$resn_exch{"2H2' IKT"}="H2'2 IKT"; $resn_exch{"3H2' IKT"}="H2'3 IKT"; $resn_exch{"1HN1 IKT"}="HN11 IKT";
$resn_exch{"2HN1 IKT"}="HN12 IKT"; $resn_exch{"1HN  ILA"}=" HN1 ILA"; $resn_exch{"2HN  ILA"}=" HN2 ILA";
$resn_exch{"1HA  ILA"}=" HA1 ILA"; $resn_exch{"1HG2 ILA"}="HG21 ILA"; $resn_exch{"2HG2 ILA"}="HG22 ILA";
$resn_exch{"3HG2 ILA"}="HG23 ILA"; $resn_exch{"1HG1 ILA"}="HG11 ILA"; $resn_exch{"2HG1 ILA"}="HG12 ILA";
$resn_exch{"1HD  ILA"}=" HD1 ILA"; $resn_exch{"2HD  ILA"}=" HD2 ILA"; $resn_exch{"3HD  ILA"}=" HD3 ILA";
$resn_exch{"1H5  ILA"}=" H51 ILA"; $resn_exch{"2H5  ILA"}=" H52 ILA"; $resn_exch{"1H6  ILA"}=" H61 ILA";
$resn_exch{"2H6  ILA"}=" H62 ILA"; $resn_exch{"1H20 ILB"}="H201 ILB"; $resn_exch{"2H20 ILB"}="H202 ILB";
$resn_exch{"1H21 ILB"}="H211 ILB"; $resn_exch{"2H21 ILB"}="H212 ILB"; $resn_exch{"1H22 ILB"}="H221 ILB";
$resn_exch{"2H22 ILB"}="H222 ILB"; $resn_exch{"1H4  ILC"}=" H41 ILC"; $resn_exch{"2H4  ILC"}=" H42 ILC";
$resn_exch{"3H4  ILC"}=" H43 ILC"; $resn_exch{" HN2 ILE"}=" H2  ILE"; $resn_exch{"1HG1 ILE"}="HG12 ILE";
$resn_exch{"2HG1 ILE"}="HG13 ILE"; $resn_exch{"1HG2 ILE"}="HG21 ILE"; $resn_exch{"2HG2 ILE"}="HG22 ILE";
$resn_exch{"3HG2 ILE"}="HG23 ILE"; $resn_exch{"1HD1 ILE"}="HD11 ILE"; $resn_exch{"2HD1 ILE"}="HD12 ILE";
$resn_exch{"3HD1 ILE"}="HD13 ILE"; $resn_exch{"1HN  ILG"}=" H   ILG"; $resn_exch{"2HN  ILG"}=" H2  ILG";
$resn_exch{"3HN  ILG"}=" H3  ILG"; $resn_exch{"1HB  ILG"}=" HB2 ILG"; $resn_exch{"2HB  ILG"}=" HB3 ILG";
$resn_exch{"1HG  ILG"}=" HG2 ILG"; $resn_exch{"2HG  ILG"}=" HG3 ILG"; $resn_exch{"1H6  ILH"}=" H61 ILH";
$resn_exch{"2H6  ILH"}=" H62 ILH"; $resn_exch{"1H26 ILI"}="H261 ILI"; $resn_exch{"2H26 ILI"}="H262 ILI";
$resn_exch{"3H26 ILI"}="H263 ILI"; $resn_exch{"1H11 ILI"}="H111 ILI"; $resn_exch{"2H11 ILI"}="H112 ILI";
$resn_exch{"1H18 ILI"}="H181 ILI"; $resn_exch{"2H18 ILI"}="H182 ILI"; $resn_exch{"3H18 ILI"}="H183 ILI";
$resn_exch{"1H23 ILI"}="H231 ILI"; $resn_exch{"2H23 ILI"}="H232 ILI"; $resn_exch{"1H25 ILI"}="H251 ILI";
$resn_exch{"2H25 ILI"}="H252 ILI"; $resn_exch{"3H25 ILI"}="H253 ILI"; $resn_exch{"1HN  ILO"}=" HN1 ILO";
$resn_exch{"2HN  ILO"}="HN2A ILO"; $resn_exch{"1HCB ILO"}="HCB1 ILO"; $resn_exch{"2HCB ILO"}="HCB2 ILO";
$resn_exch{"1HCG ILO"}="HCG1 ILO"; $resn_exch{"2HCG ILO"}="HCG2 ILO"; $resn_exch{"1HCD ILO"}="HCD1 ILO";
$resn_exch{"2HCD ILO"}="HCD2 ILO"; $resn_exch{"1HC1 ILO"}="HC11 ILO"; $resn_exch{"2HC1 ILO"}="HC12 ILO";
$resn_exch{"3HC1 ILO"}="HC13 ILO"; $resn_exch{"2HOP ILP"}="HOP2 ILP"; $resn_exch{"3HOP ILP"}="HOP3 ILP";
$resn_exch{"1H5A ILP"}="H5A1 ILP"; $resn_exch{"2H5A ILP"}="H5A2 ILP"; $resn_exch{"1H2A ILP"}="H2A1 ILP";
$resn_exch{"2H2A ILP"}="H2A2 ILP"; $resn_exch{"3H2A ILP"}="H2A3 ILP"; $resn_exch{"1H4A ILP"}="H4A1 ILP";
$resn_exch{"2H4A ILP"}="H4A2 ILP"; $resn_exch{"1HG2 ILP"}="HG21 ILP"; $resn_exch{"2HG2 ILP"}="HG22 ILP";
$resn_exch{"3HG2 ILP"}="HG23 ILP"; $resn_exch{"1HG1 ILP"}="HG11 ILP"; $resn_exch{"2HG1 ILP"}="HG12 ILP";
$resn_exch{"1HD1 ILP"}="HD11 ILP"; $resn_exch{"2HD1 ILP"}="HD12 ILP"; $resn_exch{"3HD1 ILP"}="HD13 ILP";
$resn_exch{"1HN1 ILT"}="HN11 ILT"; $resn_exch{"2HN1 ILT"}="HN12 ILT"; $resn_exch{"1HN2 ILT"}="HN21 ILT";
$resn_exch{"2HN2 ILT"}="HN22 ILT"; $resn_exch{"1H2  ILT"}=" H21 ILT"; $resn_exch{"2H2  ILT"}=" H22 ILT";
$resn_exch{"1H8  ILT"}=" H81 ILT"; $resn_exch{"2H8  ILT"}=" H82 ILT"; $resn_exch{"3H8  ILT"}=" H83 ILT";
$resn_exch{"1H14 ILT"}="H141 ILT"; $resn_exch{"2H14 ILT"}="H142 ILT"; $resn_exch{"3H14 ILT"}="H143 ILT";
$resn_exch{"1H16 ILT"}="H161 ILT"; $resn_exch{"2H16 ILT"}="H162 ILT"; $resn_exch{"1H17 ILT"}="H171 ILT";
$resn_exch{"2H17 ILT"}="H172 ILT"; $resn_exch{"1HN9 ILT"}="HN91 ILT"; $resn_exch{"2HN9 ILT"}="HN92 ILT";
$resn_exch{"1HN0 ILT"}="HN01 ILT"; $resn_exch{"2HN0 ILT"}="HN02 ILT"; $resn_exch{" HN2 ILX"}=" H2  ILX";
$resn_exch{"1HG1 ILX"}="HG11 ILX"; $resn_exch{"1HG2 ILX"}="HG21 ILX"; $resn_exch{"2HG2 ILX"}="HG22 ILX";
$resn_exch{"3HG2 ILX"}="HG23 ILX"; $resn_exch{"1HD1 ILX"}="HD12 ILX"; $resn_exch{"2HD1 ILX"}="HD13 ILX";
$resn_exch{"1H7  IM2"}=" H71 IM2"; $resn_exch{"1H6  IM2"}="H61A IM2"; $resn_exch{"1H5  IM2"}=" H51 IM2";
$resn_exch{"1H62 IM2"}="H621 IM2"; $resn_exch{"2H62 IM2"}="H622 IM2"; $resn_exch{"3H62 IM2"}="H623 IM2";
$resn_exch{"1H4  IM2"}=" H41 IM2"; $resn_exch{"1H32 IM2"}="H321 IM2"; $resn_exch{"1H22 IM2"}="H221 IM2";
$resn_exch{"2H22 IM2"}="H222 IM2"; $resn_exch{"1H23 IM2"}="H231 IM2"; $resn_exch{"2H23 IM2"}="H232 IM2";
$resn_exch{"1H24 IM2"}="H241 IM2"; $resn_exch{"1H1  IM2"}=" H11 IM2"; $resn_exch{"2H1  IM2"}=" H12 IM2";
$resn_exch{"1HB  IM3"}=" HB1 IM3"; $resn_exch{"2HB  IM3"}=" HB2 IM3"; $resn_exch{"1H6  IM8"}=" H61 IM8";
$resn_exch{"2H6  IM8"}=" H62 IM8"; $resn_exch{"3H6  IM8"}=" H63 IM8"; $resn_exch{"1H16 IM8"}="H161 IM8";
$resn_exch{"2H16 IM8"}="H162 IM8"; $resn_exch{"3H16 IM8"}="H163 IM8"; $resn_exch{"1H17 IM8"}="H171 IM8";
$resn_exch{"2H17 IM8"}="H172 IM8"; $resn_exch{"1H20 IM8"}="H201 IM8"; $resn_exch{"2H20 IM8"}="H202 IM8";
$resn_exch{"1H21 IM8"}="H211 IM8"; $resn_exch{"2H21 IM8"}="H212 IM8"; $resn_exch{"1H22 IM8"}="H221 IM8";
$resn_exch{"2H22 IM8"}="H222 IM8"; $resn_exch{"3H22 IM8"}="H223 IM8"; $resn_exch{"1H20 IMA"}="H201 IMA";
$resn_exch{"2H20 IMA"}="H202 IMA"; $resn_exch{"1H23 IMA"}="H231 IMA"; $resn_exch{"2H23 IMA"}="H232 IMA";
$resn_exch{"1H37 IMA"}="H371 IMA"; $resn_exch{"2H37 IMA"}="H372 IMA"; $resn_exch{"1H45 IMA"}="H451 IMA";
$resn_exch{"2H45 IMA"}="H452 IMA"; $resn_exch{"3H45 IMA"}="H453 IMA"; $resn_exch{"1H49 IMA"}="H491 IMA";
$resn_exch{"2H49 IMA"}="H492 IMA"; $resn_exch{"3H49 IMA"}="H493 IMA"; $resn_exch{"1H53 IMA"}="H531 IMA";
$resn_exch{"2H53 IMA"}="H532 IMA"; $resn_exch{"3H53 IMA"}="H533 IMA"; $resn_exch{"1HC1 IMA"}="HC11 IMA";
$resn_exch{"2HC1 IMA"}="HC12 IMA"; $resn_exch{"2HOP IMC"}="HOP2 IMC"; $resn_exch{"3HOP IMC"}="HOP3 IMC";
$resn_exch{"1H5* IMC"}=" H5' IMC"; $resn_exch{"2H5* IMC"}="H5'' IMC"; $resn_exch{" H4* IMC"}=" H4' IMC";
$resn_exch{" H1* IMC"}=" H1' IMC"; $resn_exch{"1HN2 IMC"}="HN21 IMC"; $resn_exch{"2HN2 IMC"}="HN22 IMC";
$resn_exch{"1H5M IMC"}=" H71 IMC"; $resn_exch{"2H5M IMC"}=" H72 IMC"; $resn_exch{"3H5M IMC"}=" H73 IMC";
$resn_exch{"1H2* IMC"}=" H2' IMC"; $resn_exch{"2H2* IMC"}="H2'' IMC"; $resn_exch{" H3* IMC"}=" H3' IMC";
$resn_exch{"*HO3 IMC"}="HO3' IMC"; $resn_exch{"1HG  IME"}=" HG1 IME"; $resn_exch{"1HD2 IME"}="HD21 IME";
$resn_exch{"1HE1 IME"}="HE11 IME"; $resn_exch{"2HG  IME"}=" HG2 IME"; $resn_exch{"2HD2 IME"}="HD22 IME";
$resn_exch{"2HE1 IME"}="HE12 IME"; $resn_exch{"3HG  IME"}=" HG3 IME"; $resn_exch{"3HD2 IME"}="HD23 IME";
$resn_exch{"3HE1 IME"}="HE13 IME"; $resn_exch{"4HG  IME"}=" HG4 IME"; $resn_exch{"4HD2 IME"}="HD24 IME";
$resn_exch{"4HE1 IME"}="HE14 IME"; $resn_exch{"1HO1 IMF"}="HO11 IMF"; $resn_exch{"2HO1 IMF"}="HO12 IMF";
$resn_exch{"1HO2 IMF"}="HO21 IMF"; $resn_exch{"2HO2 IMF"}="HO22 IMF"; $resn_exch{"1HG  IMF"}=" HG1 IMF";
$resn_exch{"1HD2 IMF"}="HD21 IMF"; $resn_exch{"1HD1 IMF"}="HD11 IMF"; $resn_exch{"2H1E IMF"}="H1E2 IMF";
$resn_exch{"2HG  IMF"}=" HG2 IMF"; $resn_exch{"2HD2 IMF"}="HD22 IMF"; $resn_exch{"2HD1 IMF"}="HD12 IMF";
$resn_exch{"1H2E IMF"}="H2E1 IMF"; $resn_exch{"3HG  IMF"}=" HG3 IMF"; $resn_exch{"3HD2 IMF"}="HD23 IMF";
$resn_exch{"3HD1 IMF"}="HD13 IMF"; $resn_exch{"4HG  IMF"}=" HG4 IMF"; $resn_exch{"4HD2 IMF"}="HD24 IMF";
$resn_exch{"4HD1 IMF"}="HD14 IMF"; $resn_exch{"1H4E IMF"}="H4E1 IMF"; $resn_exch{"5HO* IMG"}="HO'5 IMG";
$resn_exch{"1H5* IMG"}="H5'1 IMG"; $resn_exch{"2H5* IMG"}="H5'2 IMG"; $resn_exch{" H4* IMG"}=" H4' IMG";
$resn_exch{"4HN* IMG"}="HN'4 IMG"; $resn_exch{" H3* IMG"}=" H3' IMG"; $resn_exch{"3HO* IMG"}="HO'3 IMG";
$resn_exch{" H2* IMG"}=" H2' IMG"; $resn_exch{"2HO* IMG"}="HO'2 IMG"; $resn_exch{" H1* IMG"}=" H1' IMG";
$resn_exch{"1H2  IMG"}=" H21 IMG"; $resn_exch{"2H2  IMG"}=" H22 IMG"; $resn_exch{"5HO* IMH"}="HO'5 IMH";
$resn_exch{"1H5* IMH"}="H5'1 IMH"; $resn_exch{"2H5* IMH"}="H5'2 IMH"; $resn_exch{" H4* IMH"}=" H4' IMH";
$resn_exch{"4HN* IMH"}="HN'4 IMH"; $resn_exch{" H3* IMH"}=" H3' IMH"; $resn_exch{"3HO* IMH"}="HO'3 IMH";
$resn_exch{" H2* IMH"}=" H2' IMH"; $resn_exch{"2HO* IMH"}="HO'2 IMH"; $resn_exch{" H1* IMH"}=" H1' IMH";
$resn_exch{"1H10 IMI"}="H101 IMI"; $resn_exch{"2H10 IMI"}="H102 IMI"; $resn_exch{"1H9  IMI"}=" H91 IMI";
$resn_exch{"2H9  IMI"}=" H92 IMI"; $resn_exch{"1H8  IMI"}=" H81 IMI"; $resn_exch{"2H8  IMI"}=" H82 IMI";
$resn_exch{"1H7  IMI"}=" H71 IMI"; $resn_exch{"2H7  IMI"}=" H72 IMI"; $resn_exch{"1H6  IMI"}=" H61 IMI";
$resn_exch{"2H6  IMI"}=" H62 IMI"; $resn_exch{"1H6  IMK"}=" H61 IMK"; $resn_exch{"2H6  IMK"}=" H62 IMK";
$resn_exch{"1H14 IMK"}="H141 IMK"; $resn_exch{"2H14 IMK"}="H142 IMK"; $resn_exch{"3H14 IMK"}="H143 IMK";
$resn_exch{" HN  IML"}=" H   IML"; $resn_exch{"1HN  IML"}=" HN1 IML"; $resn_exch{"2HN  IML"}=" HN2 IML";
$resn_exch{"3HN  IML"}=" HN3 IML"; $resn_exch{"1HG2 IML"}="HG21 IML"; $resn_exch{"2HG2 IML"}="HG22 IML";
$resn_exch{"3HG2 IML"}="HG23 IML"; $resn_exch{"1HG1 IML"}="HG12 IML"; $resn_exch{"2HG1 IML"}="HG13 IML";
$resn_exch{"1HD1 IML"}="HD11 IML"; $resn_exch{"2HD1 IML"}="HD12 IML"; $resn_exch{"3HD1 IML"}="HD13 IML";
$resn_exch{"1H17 IMM"}="H171 IMM"; $resn_exch{"2H17 IMM"}="H172 IMM"; $resn_exch{"3H17 IMM"}="H173 IMM";
$resn_exch{"1H18 IMM"}="H181 IMM"; $resn_exch{"2H18 IMM"}="H182 IMM"; $resn_exch{"1H20 IMM"}="H201 IMM";
$resn_exch{"2H20 IMM"}="H202 IMM"; $resn_exch{"3H20 IMM"}="H203 IMM"; $resn_exch{"1H6  IMN"}=" H61 IMN";
$resn_exch{"2H6  IMN"}=" H62 IMN"; $resn_exch{"3H6  IMN"}=" H63 IMN"; $resn_exch{"1H16 IMN"}="H161 IMN";
$resn_exch{"2H16 IMN"}="H162 IMN"; $resn_exch{"3H16 IMN"}="H163 IMN"; $resn_exch{"1H17 IMN"}="H171 IMN";
$resn_exch{"2H17 IMN"}="H172 IMN"; $resn_exch{"1H5' IMO"}="H5'1 IMO"; $resn_exch{"2H5' IMO"}="H5'2 IMO";
$resn_exch{"1HOA IMO"}="HOA1 IMO"; $resn_exch{"2HOA IMO"}="HOA2 IMO"; $resn_exch{"1H8  IMO"}=" H81 IMO";
$resn_exch{"2HO' IMO"}="HO'2 IMO"; $resn_exch{"3HO' IMO"}="HO'3 IMO"; $resn_exch{"2HOP IMP"}="HOP2 IMP";
$resn_exch{"3HOP IMP"}="HOP3 IMP"; $resn_exch{"1H5* IMP"}="H5'1 IMP"; $resn_exch{"2H5* IMP"}="H5'2 IMP";
$resn_exch{" H4* IMP"}=" H4' IMP"; $resn_exch{" H3* IMP"}=" H3' IMP"; $resn_exch{"*HO3 IMP"}="HO3' IMP";
$resn_exch{" H2* IMP"}=" H2' IMP"; $resn_exch{"*HO2 IMP"}="HO2' IMP"; $resn_exch{" H1* IMP"}=" H1' IMP";
$resn_exch{"5HO* IMR"}="HO'5 IMR"; $resn_exch{"1H5* IMR"}="H5'1 IMR"; $resn_exch{"2H5* IMR"}="H5'2 IMR";
$resn_exch{" H4* IMR"}=" H4' IMR"; $resn_exch{"4HN* IMR"}="HN'4 IMR"; $resn_exch{" H3* IMR"}=" H3' IMR";
$resn_exch{"3HO* IMR"}="HO'3 IMR"; $resn_exch{" H2* IMR"}=" H2' IMR"; $resn_exch{"2HO* IMR"}="HO'2 IMR";
$resn_exch{"1H1* IMR"}="H1'1 IMR"; $resn_exch{"2H1* IMR"}="H1'2 IMR"; $resn_exch{"1H6  IMS"}=" H61 IMS";
$resn_exch{"2H6  IMS"}=" H62 IMS"; $resn_exch{"3H6  IMS"}=" H63 IMS"; $resn_exch{"1H21 IMS"}="H211 IMS";
$resn_exch{"2H21 IMS"}="H212 IMS"; $resn_exch{"1H22 IMS"}="H221 IMS"; $resn_exch{"2H22 IMS"}="H222 IMS";
$resn_exch{"3H22 IMS"}="H223 IMS"; $resn_exch{"1H20 IMS"}="H201 IMS"; $resn_exch{"2H20 IMS"}="H202 IMS";
$resn_exch{"1H17 IMS"}="H171 IMS"; $resn_exch{"2H17 IMS"}="H172 IMS"; $resn_exch{"1H16 IMS"}="H161 IMS";
$resn_exch{"2H16 IMS"}="H162 IMS"; $resn_exch{"3H16 IMS"}="H163 IMS"; $resn_exch{"1HD  IMT"}=" HD1 IMT";
$resn_exch{"2HD  IMT"}=" HD2 IMT"; $resn_exch{"3HD  IMT"}=" HD3 IMT"; $resn_exch{"PHO1 IMU"}="HO1P IMU";
$resn_exch{"PHO3 IMU"}="HO3P IMU"; $resn_exch{"1H5* IMU"}="H5'1 IMU"; $resn_exch{"2H5* IMU"}="H5'2 IMU";
$resn_exch{" H4* IMU"}=" H4' IMU"; $resn_exch{"*HN4 IMU"}="HN4' IMU"; $resn_exch{" H3* IMU"}=" H3' IMU";
$resn_exch{"*HO3 IMU"}="HO3' IMU"; $resn_exch{" H2* IMU"}=" H2' IMU"; $resn_exch{"*HO2 IMU"}="HO2' IMU";
$resn_exch{" H1* IMU"}=" H1' IMU"; $resn_exch{"1HN2 IMU"}="HN21 IMU"; $resn_exch{"2HN2 IMU"}="HN22 IMU";
$resn_exch{"1H32 IMY"}="H321 IMY"; $resn_exch{"2H32 IMY"}="H322 IMY"; $resn_exch{"3H32 IMY"}="H323 IMY";
$resn_exch{"1H33 IMY"}="H331 IMY"; $resn_exch{"2H33 IMY"}="H332 IMY"; $resn_exch{"3H33 IMY"}="H333 IMY";
$resn_exch{"1H30 IMY"}="H301 IMY"; $resn_exch{"2H30 IMY"}="H302 IMY"; $resn_exch{"1H5  IMY"}=" H51 IMY";
$resn_exch{"2H5  IMY"}=" H52 IMY"; $resn_exch{"1H6  IMY"}=" H61 IMY"; $resn_exch{"2H6  IMY"}=" H62 IMY";
$resn_exch{"1H7  IMY"}=" H71 IMY"; $resn_exch{"2H7  IMY"}=" H72 IMY"; $resn_exch{"1H8  IMY"}=" H81 IMY";
$resn_exch{"2H8  IMY"}=" H82 IMY"; $resn_exch{"1H27 IMZ"}="H271 IMZ"; $resn_exch{"2H27 IMZ"}="H272 IMZ";
$resn_exch{"1H28 IMZ"}="H281 IMZ"; $resn_exch{"2H28 IMZ"}="H282 IMZ"; $resn_exch{"3H28 IMZ"}="H283 IMZ";
$resn_exch{"1H24 IMZ"}="H241 IMZ"; $resn_exch{"2H24 IMZ"}="H242 IMZ"; $resn_exch{"3H24 IMZ"}="H243 IMZ";
$resn_exch{"1H15 IMZ"}="H151 IMZ"; $resn_exch{"2H15 IMZ"}="H152 IMZ"; $resn_exch{"1H14 IMZ"}="H141 IMZ";
$resn_exch{"2H14 IMZ"}="H142 IMZ"; $resn_exch{"1H13 IMZ"}="H131 IMZ"; $resn_exch{"2H13 IMZ"}="H132 IMZ";
$resn_exch{"1H16 IMZ"}="H161 IMZ"; $resn_exch{"2H16 IMZ"}="H162 IMZ"; $resn_exch{"1H11 IMZ"}="H111 IMZ";
$resn_exch{"2H11 IMZ"}="H112 IMZ"; $resn_exch{"1H12 IMZ"}="H121 IMZ"; $resn_exch{"2H12 IMZ"}="H122 IMZ";
$resn_exch{"1H14 IN0"}="H141 IN0"; $resn_exch{"2H14 IN0"}="H142 IN0"; $resn_exch{"1H61 IN0"}="H611 IN0";
$resn_exch{"2H61 IN0"}="H612 IN0"; $resn_exch{"1H43 IN0"}="H431 IN0"; $resn_exch{"2H43 IN0"}="H432 IN0";
$resn_exch{"1H32 IN0"}="H321 IN0"; $resn_exch{"2H32 IN0"}="H322 IN0"; $resn_exch{"3H32 IN0"}="H323 IN0";
$resn_exch{"1H25 IN0"}="H251 IN0"; $resn_exch{"2H25 IN0"}="H252 IN0"; $resn_exch{"3H25 IN0"}="H253 IN0";
$resn_exch{"1H24 IN0"}="H241 IN0"; $resn_exch{"2H24 IN0"}="H242 IN0"; $resn_exch{"3H24 IN0"}="H243 IN0";
$resn_exch{"1H1  IN1"}=" H11 IN1"; $resn_exch{"2H1  IN1"}=" H12 IN1"; $resn_exch{"1H2  IN1"}=" H21 IN1";
$resn_exch{"2H2  IN1"}=" H22 IN1"; $resn_exch{"1H3  IN1"}=" H31 IN1"; $resn_exch{"2H3  IN1"}=" H32 IN1";
$resn_exch{"1H7  IN1"}=" H71 IN1"; $resn_exch{"2H7  IN1"}=" H72 IN1"; $resn_exch{"3H7  IN1"}=" H73 IN1";
$resn_exch{"1H6B IN1"}="H6B1 IN1"; $resn_exch{"2H6B IN1"}="H6B2 IN1"; $resn_exch{"1H5  IN1"}=" H51 IN1";
$resn_exch{"2H5  IN1"}=" H52 IN1"; $resn_exch{"4HOS IN1"}="HOS4 IN1"; $resn_exch{"1HN1 IN2"}="HN11 IN2";
$resn_exch{"2HN1 IN2"}="HN12 IN2"; $resn_exch{"1H2  IN2"}=" H21 IN2"; $resn_exch{"2H2  IN2"}=" H22 IN2";
$resn_exch{"1H3  IN2"}=" H31 IN2"; $resn_exch{"2H3  IN2"}=" H32 IN2"; $resn_exch{"1H4  IN2"}=" H41 IN2";
$resn_exch{"2H4  IN2"}=" H42 IN2"; $resn_exch{"1H5  IN2"}=" H51 IN2"; $resn_exch{"2H5  IN2"}=" H52 IN2";
$resn_exch{"1H10 IN2"}="H101 IN2"; $resn_exch{"2H10 IN2"}="H102 IN2"; $resn_exch{"3H10 IN2"}="H103 IN2";
$resn_exch{"1H11 IN2"}="H111 IN2"; $resn_exch{"2H11 IN2"}="H112 IN2"; $resn_exch{"3H11 IN2"}="H113 IN2";
$resn_exch{"1H7  IN3"}=" H71 IN3"; $resn_exch{"2H7  IN3"}=" H72 IN3"; $resn_exch{"1H7' IN3"}="H7'1 IN3";
$resn_exch{"2H7' IN3"}="H7'2 IN3"; $resn_exch{"'H11 IN3"}="H11' IN3"; $resn_exch{"AH12 IN3"}="H12A IN3";
$resn_exch{"BH12 IN3"}="H12B IN3"; $resn_exch{"'H14 IN3"}="H14' IN3"; $resn_exch{"AH15 IN3"}="H15A IN3";
$resn_exch{"BH15 IN3"}="H15B IN3"; $resn_exch{"CH15 IN3"}="H15C IN3"; $resn_exch{"AH16 IN3"}="H16A IN3";
$resn_exch{"BH16 IN3"}="H16B IN3"; $resn_exch{"CH16 IN3"}="H16C IN3"; $resn_exch{"1H9  IN4"}=" H91 IN4";
$resn_exch{"2H9  IN4"}=" H92 IN4"; $resn_exch{"1H11 IN4"}="H111 IN4"; $resn_exch{"2H11 IN4"}="H112 IN4";
$resn_exch{"1HN4 IN4"}="HN41 IN4"; $resn_exch{"2HN4 IN4"}="HN42 IN4"; $resn_exch{"1HN1 IN4"}="HN11 IN4";
$resn_exch{"2HN1 IN4"}="HN12 IN4"; $resn_exch{"1H14 IN4"}="H141 IN4"; $resn_exch{"2H14 IN4"}="H142 IN4";
$resn_exch{"1H8  IN4"}=" H81 IN4"; $resn_exch{"2H8  IN4"}=" H82 IN4"; $resn_exch{"3H8  IN4"}=" H83 IN4";
$resn_exch{"1H7  IN5"}=" H71 IN5"; $resn_exch{"2H7  IN5"}=" H72 IN5"; $resn_exch{"1H8  IN5"}=" H81 IN5";
$resn_exch{"2H8  IN5"}=" H82 IN5"; $resn_exch{"1H10 IN5"}="H101 IN5"; $resn_exch{"2H10 IN5"}="H102 IN5";
$resn_exch{"3H10 IN5"}="H103 IN5"; $resn_exch{"1H6  IN5"}=" H61 IN5"; $resn_exch{"2H6  IN5"}=" H62 IN5";
$resn_exch{"3H6  IN5"}=" H63 IN5"; $resn_exch{"1H7  IN6"}=" H71 IN6"; $resn_exch{"2H7  IN6"}=" H72 IN6";
$resn_exch{"1H12 IN6"}="H121 IN6"; $resn_exch{"2H12 IN6"}="H122 IN6"; $resn_exch{"1H15 IN6"}="H151 IN6";
$resn_exch{"2H15 IN6"}="H152 IN6"; $resn_exch{"3H15 IN6"}="H153 IN6"; $resn_exch{"1H16 IN6"}="H161 IN6";
$resn_exch{"2H16 IN6"}="H162 IN6"; $resn_exch{"3H16 IN6"}="H163 IN6"; $resn_exch{"1H7' IN6"}="H7'1 IN6";
$resn_exch{"2H7' IN6"}="H7'2 IN6"; $resn_exch{"'H11 IN6"}="H11' IN6"; $resn_exch{"AH12 IN6"}="H12A IN6";
$resn_exch{"BH12 IN6"}="H12B IN6"; $resn_exch{"'H14 IN6"}="H14' IN6"; $resn_exch{"AH15 IN6"}="H15A IN6";
$resn_exch{"BH15 IN6"}="H15B IN6"; $resn_exch{"CH15 IN6"}="H15C IN6"; $resn_exch{"AH16 IN6"}="H16A IN6";
$resn_exch{"BH16 IN6"}="H16B IN6"; $resn_exch{"CH16 IN6"}="H16C IN6"; $resn_exch{"1H7  IN7"}=" H71 IN7";
$resn_exch{"2H7  IN7"}=" H72 IN7"; $resn_exch{"1H13 IN7"}="H131 IN7"; $resn_exch{"2H13 IN7"}="H132 IN7";
$resn_exch{"1H15 IN7"}="H151 IN7"; $resn_exch{"2H15 IN7"}="H152 IN7"; $resn_exch{"1H18 IN7"}="H181 IN7";
$resn_exch{"2H18 IN7"}="H182 IN7"; $resn_exch{"1H8  IN7"}=" H81 IN7"; $resn_exch{"2H8  IN7"}=" H82 IN7";
$resn_exch{"1H13 IN8"}="H131 IN8"; $resn_exch{"2H13 IN8"}="H132 IN8"; $resn_exch{"1H4  IN8"}=" H41 IN8";
$resn_exch{"2H4  IN8"}=" H42 IN8"; $resn_exch{"1H7  IN9"}=" H71 IN9"; $resn_exch{"2H7  IN9"}=" H72 IN9";
$resn_exch{"1H14 IN9"}="H141 IN9"; $resn_exch{"2H14 IN9"}="H142 IN9"; $resn_exch{"3H14 IN9"}="H143 IN9";
$resn_exch{"1H7  INA"}=" H71 INA"; $resn_exch{"2H7  INA"}=" H72 INA"; $resn_exch{"1H12 INA"}="H121 INA";
$resn_exch{"2H12 INA"}="H122 INA"; $resn_exch{"1H14 INA"}="H141 INA"; $resn_exch{"2H14 INA"}="H142 INA";
$resn_exch{"3H14 INA"}="H143 INA"; $resn_exch{"1H15 INA"}="H151 INA"; $resn_exch{"2H15 INA"}="H152 INA";
$resn_exch{"3H15 INA"}="H153 INA"; $resn_exch{"1H29 INA"}="H291 INA"; $resn_exch{"2H29 INA"}="H292 INA";
$resn_exch{"1H34 INA"}="H341 INA"; $resn_exch{"2H34 INA"}="H342 INA"; $resn_exch{"1H36 INA"}="H361 INA";
$resn_exch{"2H36 INA"}="H362 INA"; $resn_exch{"3H36 INA"}="H363 INA"; $resn_exch{"1H37 INA"}="H371 INA";
$resn_exch{"2H37 INA"}="H372 INA"; $resn_exch{"3H37 INA"}="H373 INA"; $resn_exch{"1H1' INB"}="H1'1 INB";
$resn_exch{"2H1' INB"}="H1'2 INB"; $resn_exch{"1H3' INB"}="H3'1 INB"; $resn_exch{"2H3' INB"}="H3'2 INB";
$resn_exch{"'HN2 INB"}="HN2' INB"; $resn_exch{"1HM' INB"}="HM'1 INB"; $resn_exch{"2HM' INB"}="HM'2 INB";
$resn_exch{"3HM' INB"}="HM'3 INB"; $resn_exch{"2HOP INB"}="HOP2 INB"; $resn_exch{"1H1\" INB"}="H1A1 INB";
$resn_exch{"2H1\" INB"}="H1A2 INB"; $resn_exch{"1H2\" INB"}="H2'1 INB"; $resn_exch{"2H2\" INB"}="H2'2 INB";
$resn_exch{"1HM\" INB"}="HMA1 INB"; $resn_exch{"2HM\" INB"}="HMA2 INB"; $resn_exch{"3HM\" INB"}="HMA3 INB";
$resn_exch{"1H1  INB"}=" H11 INB"; $resn_exch{"2H1  INB"}=" H12 INB"; $resn_exch{"1H2  INB"}=" H21 INB";
$resn_exch{"2H2  INB"}=" H22 INB"; $resn_exch{"1H3  INB"}=" H31 INB"; $resn_exch{"2H3  INB"}=" H32 INB";
$resn_exch{"1H4  INB"}=" H41 INB"; $resn_exch{"2H4  INB"}=" H42 INB"; $resn_exch{"1H5  INB"}=" H51 INB";
$resn_exch{"2H5  INB"}=" H52 INB"; $resn_exch{"1H6  INB"}=" H61 INB"; $resn_exch{"2H6  INB"}=" H62 INB";
$resn_exch{"1H7  INB"}=" H71 INB"; $resn_exch{"2H7  INB"}=" H72 INB"; $resn_exch{"1H8  INB"}=" H81 INB";
$resn_exch{"2H8  INB"}=" H82 INB"; $resn_exch{"1H9  INB"}=" H91 INB"; $resn_exch{"2H9  INB"}=" H92 INB";
$resn_exch{"1H10 INB"}="H101 INB"; $resn_exch{"2H10 INB"}="H102 INB"; $resn_exch{"1H11 INB"}="H111 INB";
$resn_exch{"2H11 INB"}="H112 INB"; $resn_exch{"1H12 INB"}="H121 INB"; $resn_exch{"2H12 INB"}="H122 INB";
$resn_exch{"1H13 INB"}="H131 INB"; $resn_exch{"2H13 INB"}="H132 INB"; $resn_exch{"1H14 INB"}="H141 INB";
$resn_exch{"2H14 INB"}="H142 INB"; $resn_exch{"1H15 INB"}="H151 INB"; $resn_exch{"2H15 INB"}="H152 INB";
$resn_exch{"1H16 INB"}="H161 INB"; $resn_exch{"2H16 INB"}="H162 INB"; $resn_exch{"1H17 INB"}="H171 INB";
$resn_exch{"2H17 INB"}="H172 INB"; $resn_exch{"1H18 INB"}="H181 INB"; $resn_exch{"2H18 INB"}="H182 INB";
$resn_exch{"3H18 INB"}="H183 INB"; $resn_exch{"1HA1 INC"}="HA11 INC"; $resn_exch{"2HA1 INC"}="HA12 INC";
$resn_exch{"3HA1 INC"}="HA13 INC"; $resn_exch{"1HB2 INC"}="HB21 INC"; $resn_exch{"2HB2 INC"}="HB22 INC";
$resn_exch{"1HD1 INC"}="HD11 INC"; $resn_exch{"2HD1 INC"}="HD12 INC"; $resn_exch{"3HD1 INC"}="HD13 INC";
$resn_exch{"1HD2 INC"}="HD21 INC"; $resn_exch{"2HD2 INC"}="HD22 INC"; $resn_exch{"3HD2 INC"}="HD23 INC";
$resn_exch{"1H3  INC"}=" H31 INC"; $resn_exch{"2H3  INC"}=" H32 INC"; $resn_exch{"3H3  INC"}=" H33 INC";
$resn_exch{"1H1  INE"}=" H11 INE"; $resn_exch{"1H4  INE"}=" H41 INE"; $resn_exch{"1H5  INE"}=" H51 INE";
$resn_exch{"1H6  INE"}=" H61 INE"; $resn_exch{"1HB  INF"}=" HB1 INF"; $resn_exch{"2HB  INF"}=" HB2 INF";
$resn_exch{"1HB  ING"}=" HB1 ING"; $resn_exch{"2HB  ING"}=" HB2 ING"; $resn_exch{"1HN1 ING"}="HN11 ING";
$resn_exch{"2HN1 ING"}="HN12 ING"; $resn_exch{"1H8  INI"}=" H81 INI"; $resn_exch{"2H8  INI"}=" H82 INI";
$resn_exch{"1H12 INI"}="H121 INI"; $resn_exch{"2H12 INI"}="H122 INI"; $resn_exch{"1H8  INJ"}=" H81 INJ";
$resn_exch{"2H8  INJ"}=" H82 INJ"; $resn_exch{"1H9  INJ"}=" H91 INJ"; $resn_exch{"1H10 INJ"}="H101 INJ";
$resn_exch{"1H11 INJ"}="H111 INJ"; $resn_exch{"1H12 INJ"}="H121 INJ"; $resn_exch{"2H12 INJ"}="H122 INJ";
$resn_exch{"1H13 INJ"}="H131 INJ"; $resn_exch{"2H13 INJ"}="H132 INJ"; $resn_exch{"1H14 INJ"}="H141 INJ";
$resn_exch{"2H14 INJ"}="H142 INJ"; $resn_exch{"1H15 INJ"}="H151 INJ"; $resn_exch{"2H15 INJ"}="H152 INJ";
$resn_exch{"1H16 INJ"}="H161 INJ"; $resn_exch{"2H16 INJ"}="H162 INJ"; $resn_exch{"1H17 INJ"}="H171 INJ";
$resn_exch{"2H17 INJ"}="H172 INJ"; $resn_exch{"2HOP INJ"}="HOP2 INJ"; $resn_exch{"3HOP INJ"}="HOP3 INJ";
$resn_exch{"1HC1 INK"}="HC11 INK"; $resn_exch{"2HC1 INK"}="HC12 INK"; $resn_exch{"3HC1 INK"}="HC13 INK";
$resn_exch{"1HC3 INK"}="HC31 INK"; $resn_exch{"2HC3 INK"}="HC32 INK"; $resn_exch{"1HC4 INK"}="HC41 INK";
$resn_exch{"2HC4 INK"}="HC42 INK"; $resn_exch{"3HC4 INK"}="HC43 INK"; $resn_exch{"1HC5 INK"}="HC51 INK";
$resn_exch{"2HC5 INK"}="HC52 INK"; $resn_exch{"1HN4 INL"}="HN41 INL"; $resn_exch{"2HN4 INL"}="HN42 INL";
$resn_exch{"1H6  INL"}=" H61 INL"; $resn_exch{"1H10 INL"}="H101 INL"; $resn_exch{"1H16 INL"}="H161 INL";
$resn_exch{"2H16 INL"}="H162 INL"; $resn_exch{"1H17 INL"}="H171 INL"; $resn_exch{"2H17 INL"}="H172 INL";
$resn_exch{"1H18 INL"}="H181 INL"; $resn_exch{"2H18 INL"}="H182 INL"; $resn_exch{"1H20 INL"}="H201 INL";
$resn_exch{"2H20 INL"}="H202 INL"; $resn_exch{"1H21 INL"}="H211 INL"; $resn_exch{"2H21 INL"}="H212 INL";
$resn_exch{"1H23 INL"}="H231 INL"; $resn_exch{"1H25 INL"}="H251 INL"; $resn_exch{"1H26 INL"}="H261 INL";
$resn_exch{"1H27 INL"}="H271 INL"; $resn_exch{"1H30 INL"}="H301 INL"; $resn_exch{"2H30 INL"}="H302 INL";
$resn_exch{"3H30 INL"}="H303 INL"; $resn_exch{"1HC2 INM"}="HC21 INM"; $resn_exch{"1H7  INM"}=" H71 INM";
$resn_exch{"2H7  INM"}=" H72 INM"; $resn_exch{"1H8  INM"}=" H81 INM"; $resn_exch{"2H8  INM"}=" H82 INM";
$resn_exch{"1H9  INM"}=" H91 INM"; $resn_exch{"2H9  INM"}=" H92 INM"; $resn_exch{"1H10 INM"}="H101 INM";
$resn_exch{"2H10 INM"}="H102 INM"; $resn_exch{"1H12 INM"}="H121 INM"; $resn_exch{"2H12 INM"}="H122 INM";
$resn_exch{"3H12 INM"}="H123 INM"; $resn_exch{"1H13 INM"}="H131 INM"; $resn_exch{"1H19 INM"}="H191 INM";
$resn_exch{"2H19 INM"}="H192 INM"; $resn_exch{"1H2  INN"}=" H21 INN"; $resn_exch{"2H2  INN"}=" H22 INN";
$resn_exch{"3H2  INN"}=" H23 INN"; $resn_exch{"1H8  INN"}=" H81 INN"; $resn_exch{"2H8  INN"}=" H82 INN";
$resn_exch{"3H8  INN"}=" H83 INN"; $resn_exch{"1H9  INN"}=" H91 INN"; $resn_exch{"2H9  INN"}=" H92 INN";
$resn_exch{"3H9  INN"}=" H93 INN"; $resn_exch{"1H12 INN"}="H121 INN"; $resn_exch{"2H12 INN"}="H122 INN";
$resn_exch{"3H12 INN"}="H123 INN"; $resn_exch{"1H0  INN"}=" H01 INN"; $resn_exch{"2H0  INN"}=" H02 INN";
$resn_exch{"1HB  INN"}=" HB1 INN"; $resn_exch{"2HB  INN"}=" HB2 INN"; $resn_exch{"1H3  INN"}=" H31 INN";
$resn_exch{"2H3  INN"}=" H32 INN"; $resn_exch{"3H3  INN"}=" H33 INN"; $resn_exch{"1H7  INN"}=" H71 INN";
$resn_exch{"2H7  INN"}=" H72 INN"; $resn_exch{"3H7  INN"}=" H73 INN"; $resn_exch{"1H14 INN"}="H141 INN";
$resn_exch{"2H14 INN"}="H142 INN"; $resn_exch{"1H15 INN"}="H151 INN"; $resn_exch{"2H15 INN"}="H152 INN";
$resn_exch{"1HN4 INN"}="HN41 INN"; $resn_exch{"2HN4 INN"}="HN42 INN"; $resn_exch{"1H7  INP"}=" H71 INP";
$resn_exch{"2H7  INP"}=" H72 INP"; $resn_exch{"1H13 INP"}="H131 INP"; $resn_exch{"2H13 INP"}="H132 INP";
$resn_exch{"1H15 INP"}="H151 INP"; $resn_exch{"2H15 INP"}="H152 INP"; $resn_exch{"1H16 INP"}="H161 INP";
$resn_exch{"2H16 INP"}="H162 INP"; $resn_exch{"1H19 INP"}="H191 INP"; $resn_exch{"2H19 INP"}="H192 INP";
$resn_exch{"1H21 INP"}="H211 INP"; $resn_exch{"2H21 INP"}="H212 INP"; $resn_exch{"3H21 INP"}="H213 INP";
$resn_exch{"1H22 INP"}="H221 INP"; $resn_exch{"2H22 INP"}="H222 INP"; $resn_exch{"3H22 INP"}="H223 INP";
$resn_exch{"1H27 INP"}="H271 INP"; $resn_exch{"2H27 INP"}="H272 INP"; $resn_exch{"1H29 INP"}="H291 INP";
$resn_exch{"2H29 INP"}="H292 INP"; $resn_exch{"3H29 INP"}="H293 INP"; $resn_exch{"1H30 INP"}="H301 INP";
$resn_exch{"2H30 INP"}="H302 INP"; $resn_exch{"3H30 INP"}="H303 INP"; $resn_exch{"1H35 INP"}="H351 INP";
$resn_exch{"2H35 INP"}="H352 INP"; $resn_exch{"1H03 INQ"}="H031 INQ"; $resn_exch{"2H03 INQ"}="H032 INQ";
$resn_exch{"1H05 INQ"}="H051 INQ"; $resn_exch{"1H09 INQ"}="H091 INQ"; $resn_exch{"1H15 INQ"}="H151 INQ";
$resn_exch{"2H15 INQ"}="H152 INQ"; $resn_exch{"1H16 INQ"}="H161 INQ"; $resn_exch{"2H16 INQ"}="H162 INQ";
$resn_exch{"1H17 INQ"}="H171 INQ"; $resn_exch{"2H17 INQ"}="H172 INQ"; $resn_exch{"1H19 INQ"}="H191 INQ";
$resn_exch{"2H19 INQ"}="H192 INQ"; $resn_exch{"1H20 INQ"}="H201 INQ"; $resn_exch{"2H20 INQ"}="H202 INQ";
$resn_exch{"1H22 INQ"}="H221 INQ"; $resn_exch{"1H24 INQ"}="H241 INQ"; $resn_exch{"1H25 INQ"}="H251 INQ";
$resn_exch{"1H26 INQ"}="H261 INQ"; $resn_exch{"1H28 INQ"}="H281 INQ"; $resn_exch{"1H2  INT"}=" H21 INT";
$resn_exch{"2H2  INT"}=" H22 INT"; $resn_exch{"1H3  INT"}=" H31 INT"; $resn_exch{"2H3  INT"}=" H32 INT";
$resn_exch{"1HG1 INT"}="HG11 INT"; $resn_exch{"2HG1 INT"}="HG12 INT"; $resn_exch{"3HG1 INT"}="HG13 INT";
$resn_exch{"1HG2 INT"}="HG21 INT"; $resn_exch{"2HG2 INT"}="HG22 INT"; $resn_exch{"3HG2 INT"}="HG23 INT";
$resn_exch{"1H20 INT"}="H201 INT"; $resn_exch{"2H20 INT"}="H202 INT"; $resn_exch{"3H20 INT"}="H203 INT";
$resn_exch{"1HA  INT"}=" HA1 INT"; $resn_exch{"2HA  INT"}=" HA2 INT"; $resn_exch{"1HG3 INU"}="HG31 INU";
$resn_exch{"2HG3 INU"}="HG32 INU"; $resn_exch{"1HG4 INU"}="HG41 INU"; $resn_exch{"2HG4 INU"}="HG42 INU";
$resn_exch{"1HD1 INU"}="HD11 INU"; $resn_exch{"2HD1 INU"}="HD12 INU"; $resn_exch{"1HD2 INU"}="HD21 INU";
$resn_exch{"2HD2 INU"}="HD22 INU"; $resn_exch{"1HD3 INU"}="HD31 INU"; $resn_exch{"2HD3 INU"}="HD32 INU";
$resn_exch{"1HD4 INU"}="HD41 INU"; $resn_exch{"2HD4 INU"}="HD42 INU"; $resn_exch{"1HD5 INU"}="HD51 INU";
$resn_exch{"2HD5 INU"}="HD52 INU"; $resn_exch{"1HD6 INU"}="HD61 INU"; $resn_exch{"2HD6 INU"}="HD62 INU";
$resn_exch{"1H01 INV"}="H011 INV"; $resn_exch{"1H02 INV"}="H021 INV"; $resn_exch{"1H04 INV"}="H041 INV";
$resn_exch{"1H05 INV"}="H051 INV"; $resn_exch{"1H09 INV"}="H091 INV"; $resn_exch{"1H10 INV"}="H101 INV";
$resn_exch{"2H10 INV"}="H102 INV"; $resn_exch{"1HP2 INV"}="HP21 INV"; $resn_exch{"2HP2 INV"}="HP22 INV";
$resn_exch{"1H16 INV"}="H161 INV"; $resn_exch{"1H17 INV"}="H171 INV"; $resn_exch{"1H19 INV"}="H191 INV";
$resn_exch{"1H20 INV"}="H201 INV"; $resn_exch{"1H01 INW"}="H011 INW"; $resn_exch{"1H02 INW"}="H021 INW";
$resn_exch{"1H04 INW"}="H041 INW"; $resn_exch{"1H05 INW"}="H051 INW"; $resn_exch{"1H09 INW"}="H091 INW";
$resn_exch{"1H10 INW"}="H101 INW"; $resn_exch{"2H10 INW"}="H102 INW"; $resn_exch{"1HP2 INW"}="HP21 INW";
$resn_exch{"2HP2 INW"}="HP22 INW"; $resn_exch{"1H17 INW"}="H171 INW"; $resn_exch{"1H19 INW"}="H191 INW";
$resn_exch{"1H20 INW"}="H201 INW"; $resn_exch{"1HC7 INX"}="HC71 INX"; $resn_exch{"2HC7 INX"}="HC72 INX";
$resn_exch{"1H15 INX"}="H151 INX"; $resn_exch{"2H15 INX"}="H152 INX"; $resn_exch{"1H16 INX"}="H161 INX";
$resn_exch{"2H16 INX"}="H162 INX"; $resn_exch{"1H17 INX"}="H171 INX"; $resn_exch{"2H17 INX"}="H172 INX";
$resn_exch{"1H18 INX"}="H181 INX"; $resn_exch{"2H18 INX"}="H182 INX"; $resn_exch{"1H19 INX"}="H191 INX";
$resn_exch{"2H19 INX"}="H192 INX"; $resn_exch{"3H19 INX"}="H193 INX"; $resn_exch{"1H23 INX"}="H231 INX";
$resn_exch{"2H23 INX"}="H232 INX"; $resn_exch{"1H30 INX"}="H301 INX"; $resn_exch{"2H30 INX"}="H302 INX";
$resn_exch{"1H38 INX"}="H381 INX"; $resn_exch{"2H38 INX"}="H382 INX"; $resn_exch{"1H39 INX"}="H391 INX";
$resn_exch{"2H39 INX"}="H392 INX"; $resn_exch{"1H6  INY"}=" H61 INY"; $resn_exch{"2H6  INY"}=" H62 INY";
$resn_exch{"3H6  INY"}=" H63 INY"; $resn_exch{"1H17 INY"}="H171 INY"; $resn_exch{"2H17 INY"}="H172 INY";
$resn_exch{"1H12 INY"}="H121 INY"; $resn_exch{"2H12 INY"}="H122 INY"; $resn_exch{"3H12 INY"}="H123 INY";
$resn_exch{"1H11 INZ"}="H111 INZ"; $resn_exch{"2H11 INZ"}="H112 INZ"; $resn_exch{"1H22 INZ"}="H221 INZ";
$resn_exch{"2H22 INZ"}="H222 INZ"; $resn_exch{"1H23 INZ"}="H231 INZ"; $resn_exch{"2H23 INZ"}="H232 INZ";
$resn_exch{"1H24 INZ"}="H241 INZ"; $resn_exch{"2H24 INZ"}="H242 INZ"; $resn_exch{"1H25 INZ"}="H251 INZ";
$resn_exch{"2H25 INZ"}="H252 INZ"; $resn_exch{"1H26 INZ"}="H261 INZ"; $resn_exch{"2H26 INZ"}="H262 INZ";
$resn_exch{"3H26 INZ"}="H263 INZ"; $resn_exch{"1H30 INZ"}="H301 INZ"; $resn_exch{"2H30 INZ"}="H302 INZ";
$resn_exch{"1H01 IOA"}="H011 IOA"; $resn_exch{"1H02 IOA"}="H021 IOA"; $resn_exch{"1H04 IOA"}="H041 IOA";
$resn_exch{"1H05 IOA"}="H051 IOA"; $resn_exch{"1H09 IOA"}="H091 IOA"; $resn_exch{"1H10 IOA"}="H101 IOA";
$resn_exch{"2H10 IOA"}="H102 IOA"; $resn_exch{"1HP2 IOA"}="HP21 IOA"; $resn_exch{"2HP2 IOA"}="HP22 IOA";
$resn_exch{"1H16 IOA"}="H161 IOA"; $resn_exch{"1H18 IOA"}="H181 IOA"; $resn_exch{"1H19 IOA"}="H191 IOA";
$resn_exch{"2HB  IOB"}=" HB2 IOB"; $resn_exch{"1HB  IOB"}=" HB1 IOB"; $resn_exch{"1H01 IOC"}="H011 IOC";
$resn_exch{"1H02 IOC"}="H021 IOC"; $resn_exch{"1H04 IOC"}="H041 IOC"; $resn_exch{"1H05 IOC"}="H051 IOC";
$resn_exch{"1H09 IOC"}="H091 IOC"; $resn_exch{"1H10 IOC"}="H101 IOC"; $resn_exch{"2H10 IOC"}="H102 IOC";
$resn_exch{"1HP2 IOC"}="HP21 IOC"; $resn_exch{"2HP2 IOC"}="HP22 IOC"; $resn_exch{"1H19 IOC"}="H191 IOC";
$resn_exch{"1H20 IOC"}="H201 IOC"; $resn_exch{"1H01 IOE"}="H011 IOE"; $resn_exch{"1H02 IOE"}="H021 IOE";
$resn_exch{"1H04 IOE"}="H041 IOE"; $resn_exch{"1H05 IOE"}="H051 IOE"; $resn_exch{"1H09 IOE"}="H091 IOE";
$resn_exch{"1H10 IOE"}="H101 IOE"; $resn_exch{"2H10 IOE"}="H102 IOE"; $resn_exch{"1HP2 IOE"}="HP21 IOE";
$resn_exch{"2HP2 IOE"}="HP22 IOE"; $resn_exch{"1H17 IOE"}="H171 IOE"; $resn_exch{"1H19 IOE"}="H191 IOE";
$resn_exch{"1H01 IOF"}="H011 IOF"; $resn_exch{"1H02 IOF"}="H021 IOF"; $resn_exch{"1H04 IOF"}="H041 IOF";
$resn_exch{"1H05 IOF"}="H051 IOF"; $resn_exch{"1H09 IOF"}="H091 IOF"; $resn_exch{"1H10 IOF"}="H101 IOF";
$resn_exch{"2H10 IOF"}="H102 IOF"; $resn_exch{"1HP2 IOF"}="HP21 IOF"; $resn_exch{"2HP2 IOF"}="HP22 IOF";
$resn_exch{"1H16 IOF"}="H161 IOF"; $resn_exch{"1H20 IOF"}="H201 IOF"; $resn_exch{"1H8  IOG"}=" H81 IOG";
$resn_exch{"2H8  IOG"}=" H82 IOG"; $resn_exch{"1H9  IOG"}=" H91 IOG"; $resn_exch{"2H9  IOG"}=" H92 IOG";
$resn_exch{"1H11 IOG"}="H111 IOG"; $resn_exch{"2H11 IOG"}="H112 IOG"; $resn_exch{"3H11 IOG"}="H113 IOG";
$resn_exch{"1H15 IOG"}="H151 IOG"; $resn_exch{"2H15 IOG"}="H152 IOG"; $resn_exch{"1H32 IOG"}="H321 IOG";
$resn_exch{"2H32 IOG"}="H322 IOG"; $resn_exch{"1H33 IOG"}="H331 IOG"; $resn_exch{"2H33 IOG"}="H332 IOG";
$resn_exch{"1H35 IOG"}="H351 IOG"; $resn_exch{"2H35 IOG"}="H352 IOG"; $resn_exch{"1H36 IOG"}="H361 IOG";
$resn_exch{"2H36 IOG"}="H362 IOG"; $resn_exch{"1H37 IOG"}="H371 IOG"; $resn_exch{"2H37 IOG"}="H372 IOG";
$resn_exch{"1H38 IOG"}="H381 IOG"; $resn_exch{"2H38 IOG"}="H382 IOG"; $resn_exch{"1H39 IOG"}="H391 IOG";
$resn_exch{"2H39 IOG"}="H392 IOG"; $resn_exch{"1HB1 IOH"}="HB11 IOH"; $resn_exch{"2HB1 IOH"}="HB12 IOH";
$resn_exch{"3HB1 IOH"}="HB13 IOH"; $resn_exch{"1HB2 IOH"}="HB21 IOH"; $resn_exch{"2HB2 IOH"}="HB22 IOH";
$resn_exch{"3HB2 IOH"}="HB23 IOH"; $resn_exch{"1H8  IOK"}=" H81 IOK"; $resn_exch{"2H8  IOK"}=" H82 IOK";
$resn_exch{"1H9  IOK"}=" H91 IOK"; $resn_exch{"2H9  IOK"}=" H92 IOK"; $resn_exch{"1H11 IOK"}="H111 IOK";
$resn_exch{"2H11 IOK"}="H112 IOK"; $resn_exch{"3H11 IOK"}="H113 IOK"; $resn_exch{"1H15 IOK"}="H151 IOK";
$resn_exch{"2H15 IOK"}="H152 IOK"; $resn_exch{"1HNA IOM"}="HNA1 IOM"; $resn_exch{"2HNA IOM"}="HNA2 IOM";
$resn_exch{"1HNB IOM"}="HNB1 IOM"; $resn_exch{"2HNB IOM"}="HNB2 IOM"; $resn_exch{"1HCB IOM"}="HCB1 IOM";
$resn_exch{"2HCB IOM"}="HCB2 IOM"; $resn_exch{"3HCB IOM"}="HCB3 IOM"; $resn_exch{"1HCC IOM"}="HCC1 IOM";
$resn_exch{"2HCC IOM"}="HCC2 IOM"; $resn_exch{"1H1  ION"}=" H11 ION"; $resn_exch{"2H1  ION"}=" H12 ION";
$resn_exch{"1H4  ION"}=" H41 ION"; $resn_exch{"2H4  ION"}=" H42 ION"; $resn_exch{"3H4  ION"}=" H43 ION";
$resn_exch{"1H5  ION"}=" H51 ION"; $resn_exch{"2H5  ION"}=" H52 ION"; $resn_exch{"1H6  ION"}=" H61 ION";
$resn_exch{"2H6  ION"}=" H62 ION"; $resn_exch{"1H7  ION"}=" H71 ION"; $resn_exch{"2H7  ION"}=" H72 ION";
$resn_exch{"1H9  ION"}=" H91 ION"; $resn_exch{"2H9  ION"}=" H92 ION"; $resn_exch{"3H9  ION"}=" H93 ION";
$resn_exch{"1H11 ION"}="H111 ION"; $resn_exch{"2H11 ION"}="H112 ION"; $resn_exch{"1H12 ION"}="H121 ION";
$resn_exch{"1H14 ION"}="H141 ION"; $resn_exch{"2H14 ION"}="H142 ION"; $resn_exch{"3H14 ION"}="H143 ION";
$resn_exch{"1H15 ION"}="H151 ION"; $resn_exch{"2H15 ION"}="H152 ION"; $resn_exch{"1H2' IOP"}="H2'1 IOP";
$resn_exch{"2H2' IOP"}="H2'2 IOP"; $resn_exch{"1H3' IOP"}="H3'1 IOP"; $resn_exch{"2H3' IOP"}="H3'2 IOP";
$resn_exch{"1H5* IOT"}="H5'1 IOT"; $resn_exch{"2H5* IOT"}="H5'2 IOT"; $resn_exch{" H4* IOT"}=" H4' IOT";
$resn_exch{" H3* IOT"}=" H3' IOT"; $resn_exch{"3HO* IOT"}="HO'3 IOT"; $resn_exch{" H2* IOT"}=" H2' IOT";
$resn_exch{"2HO* IOT"}="HO'2 IOT"; $resn_exch{" H1* IOT"}=" H1' IOT"; $resn_exch{"1H6  IOT"}=" H61 IOT";
$resn_exch{"2H6  IOT"}=" H62 IOT"; $resn_exch{"1HCX IOT"}="HCX1 IOT"; $resn_exch{"2HCX IOT"}="HCX2 IOT";
$resn_exch{"1HB  IOT"}=" HB1 IOT"; $resn_exch{"2HB  IOT"}=" HB2 IOT"; $resn_exch{"1HG  IOT"}=" HG1 IOT";
$resn_exch{"2HG  IOT"}=" HG2 IOT"; $resn_exch{"1HD  IOT"}=" HD1 IOT"; $resn_exch{"2HD  IOT"}=" HD2 IOT";
$resn_exch{"1HH1 IOT"}="HH11 IOT"; $resn_exch{"1HH2 IOT"}="HH21 IOT"; $resn_exch{"2HH2 IOT"}="HH22 IOT";
$resn_exch{"1HB  IOY"}=" HB3 IOY"; $resn_exch{"2HB  IOY"}=" HB2 IOY"; $resn_exch{"1HN  IOY"}=" H   IOY";
$resn_exch{"2HN  IOY"}=" H2  IOY"; $resn_exch{"2H3  IP1"}=" H32 IP1"; $resn_exch{"1H3  IP1"}=" H31 IP1";
$resn_exch{"2H4  IP1"}="H42A IP1"; $resn_exch{"1H4  IP1"}=" H41 IP1"; $resn_exch{"2H7  IP1"}=" H72 IP1";
$resn_exch{"1H7  IP1"}=" H71 IP1"; $resn_exch{"2H14 IP1"}="H142 IP1"; $resn_exch{"1H14 IP1"}="H141 IP1";
$resn_exch{"3H33 IP1"}="H333 IP1"; $resn_exch{"2H33 IP1"}="H332 IP1"; $resn_exch{"1H33 IP1"}="H331 IP1";
$resn_exch{"3H37 IP1"}="H373 IP1"; $resn_exch{"2H37 IP1"}="H372 IP1"; $resn_exch{"1H37 IP1"}="H371 IP1";
$resn_exch{"2HO4 IP2"}="HO42 IP2"; $resn_exch{"3HO4 IP2"}="HO43 IP2"; $resn_exch{"2HO5 IP2"}="HO52 IP2";
$resn_exch{"3HO5 IP2"}="HO53 IP2"; $resn_exch{"1H1  IP3"}=" H11 IP3"; $resn_exch{"2H1  IP3"}=" H12 IP3";
$resn_exch{"1H2  IP3"}=" H21 IP3"; $resn_exch{"2H2  IP3"}=" H22 IP3"; $resn_exch{"1H4  IP3"}=" H41 IP3";
$resn_exch{"2H4  IP3"}=" H42 IP3"; $resn_exch{"3H4  IP3"}=" H43 IP3"; $resn_exch{"1H5  IP3"}=" H51 IP3";
$resn_exch{"2H5  IP3"}=" H52 IP3"; $resn_exch{"3H5  IP3"}=" H53 IP3"; $resn_exch{"1H3  IP4"}=" H31 IP4";
$resn_exch{"2H3  IP4"}=" H32 IP4"; $resn_exch{"1H4  IP4"}=" H41 IP4"; $resn_exch{"2H4  IP4"}=" H42 IP4";
$resn_exch{"1HM1 IP4"}="HM11 IP4"; $resn_exch{"2HM1 IP4"}="HM12 IP4"; $resn_exch{"3HM1 IP4"}="HM13 IP4";
$resn_exch{"1HM2 IP4"}="HM21 IP4"; $resn_exch{"2HM2 IP4"}="HM22 IP4"; $resn_exch{"3HM2 IP4"}="HM23 IP4";
$resn_exch{"1HM3 IP4"}="HM31 IP4"; $resn_exch{"2HM3 IP4"}="HM32 IP4"; $resn_exch{"3HM3 IP4"}="HM33 IP4";
$resn_exch{"1HN1 IP6"}="HN11 IP6"; $resn_exch{"2HN1 IP6"}="HN12 IP6"; $resn_exch{"1H9  IP6"}=" H91 IP6";
$resn_exch{"2H9  IP6"}=" H92 IP6"; $resn_exch{"1HN1 IP7"}="HN11 IP7"; $resn_exch{"2HN1 IP7"}="HN12 IP7";
$resn_exch{"1H9  IP7"}=" H91 IP7"; $resn_exch{"2H9  IP7"}=" H92 IP7"; $resn_exch{"1H17 IP7"}="H171 IP7";
$resn_exch{"2H17 IP7"}="H172 IP7"; $resn_exch{"1H1  IPA"}=" H11 IPA"; $resn_exch{"2H1  IPA"}=" H12 IPA";
$resn_exch{"3H1  IPA"}=" H13 IPA"; $resn_exch{"1H3  IPA"}=" H31 IPA"; $resn_exch{"2H3  IPA"}=" H32 IPA";
$resn_exch{"3H3  IPA"}=" H33 IPA"; $resn_exch{"3H7  IPB"}=" H73 IPB"; $resn_exch{"2H7  IPB"}=" H72 IPB";
$resn_exch{"1H7  IPB"}=" H71 IPB"; $resn_exch{"3H9  IPB"}=" H93 IPB"; $resn_exch{"2H9  IPB"}=" H92 IPB";
$resn_exch{"1H9  IPB"}=" H91 IPB"; $resn_exch{"3H10 IPB"}="H103 IPB"; $resn_exch{"2H10 IPB"}="H102 IPB";
$resn_exch{"1H10 IPB"}="H101 IPB"; $resn_exch{"1H3  IPC"}=" H31 IPC"; $resn_exch{"2H3  IPC"}=" H32 IPC";
$resn_exch{"3H3  IPC"}=" H33 IPC"; $resn_exch{"1H4  IPC"}=" H41 IPC"; $resn_exch{"2H4  IPC"}=" H42 IPC";
$resn_exch{"3H4  IPC"}=" H43 IPC"; $resn_exch{"1H13 IPC"}="H131 IPC"; $resn_exch{"2H13 IPC"}="H132 IPC";
$resn_exch{"3H13 IPC"}="H133 IPC"; $resn_exch{"1H1  IPE"}=" H11 IPE"; $resn_exch{"2H1  IPE"}=" H12 IPE";
$resn_exch{"1H2  IPE"}=" H21 IPE"; $resn_exch{"2H2  IPE"}=" H22 IPE"; $resn_exch{"1H4  IPE"}=" H41 IPE";
$resn_exch{"2H4  IPE"}=" H42 IPE"; $resn_exch{"1H5  IPE"}=" H51 IPE"; $resn_exch{"2H5  IPE"}=" H52 IPE";
$resn_exch{"3H5  IPE"}=" H53 IPE"; $resn_exch{"1H2  IPF"}=" H21 IPF"; $resn_exch{"2H2  IPF"}="H22A IPF";
$resn_exch{"3H2  IPF"}="H23A IPF"; $resn_exch{"1H4  IPF"}="H41A IPF"; $resn_exch{"2H4  IPF"}=" H42 IPF";
$resn_exch{"1H16 IPF"}="H161 IPF"; $resn_exch{"2H16 IPF"}="H162 IPF"; $resn_exch{"3H16 IPF"}="H163 IPF";
$resn_exch{"1H17 IPF"}="H171 IPF"; $resn_exch{"2H17 IPF"}="H172 IPF"; $resn_exch{"3H17 IPF"}="H173 IPF";
$resn_exch{"1H20 IPF"}="H201 IPF"; $resn_exch{"2H20 IPF"}="H202 IPF"; $resn_exch{"1H29 IPF"}="H291 IPF";
$resn_exch{"2H29 IPF"}="H292 IPF"; $resn_exch{"1H30 IPF"}="H301 IPF"; $resn_exch{"2H30 IPF"}="H302 IPF";
$resn_exch{"1H31 IPF"}="H311 IPF"; $resn_exch{"2H31 IPF"}="H312 IPF"; $resn_exch{"1H35 IPF"}="H351 IPF";
$resn_exch{"2H35 IPF"}="H352 IPF"; $resn_exch{"3H35 IPF"}="H353 IPF"; $resn_exch{"1H36 IPF"}="H361 IPF";
$resn_exch{"2H36 IPF"}="H362 IPF"; $resn_exch{"3H36 IPF"}="H363 IPF"; $resn_exch{"1H39 IPF"}="H391 IPF";
$resn_exch{"2H39 IPF"}="H392 IPF"; $resn_exch{"1H50 IPF"}="H501 IPF"; $resn_exch{"2H50 IPF"}="H502 IPF";
$resn_exch{"3H50 IPF"}="H503 IPF"; $resn_exch{"1HA  IPG"}=" HA1 IPG"; $resn_exch{"2HA  IPG"}=" HA2 IPG";
$resn_exch{"1HG1 IPG"}="HG11 IPG"; $resn_exch{"2HG1 IPG"}="HG12 IPG"; $resn_exch{"3HG1 IPG"}="HG13 IPG";
$resn_exch{"1HG2 IPG"}="HG21 IPG"; $resn_exch{"2HG2 IPG"}="HG22 IPG"; $resn_exch{"3HG2 IPG"}="HG23 IPG";
$resn_exch{"1H1' IPL"}="H1'1 IPL"; $resn_exch{"2H1' IPL"}="H1'2 IPL"; $resn_exch{"1H2' IPL"}="H2'1 IPL";
$resn_exch{"2H2' IPL"}="H2'2 IPL"; $resn_exch{"1H3' IPL"}="H3'1 IPL"; $resn_exch{"2H3' IPL"}="H3'2 IPL";
$resn_exch{"2HOP IPL"}="HOP2 IPL"; $resn_exch{"3HOP IPL"}="HOP3 IPL"; $resn_exch{"1H6  IPM"}=" H61 IPM";
$resn_exch{"2H6  IPM"}=" H62 IPM"; $resn_exch{"3H6  IPM"}=" H63 IPM"; $resn_exch{"1H7  IPM"}=" H71 IPM";
$resn_exch{"2H7  IPM"}=" H72 IPM"; $resn_exch{"3H7  IPM"}=" H73 IPM"; $resn_exch{"1H8' IPN"}="H8'1 IPN";
$resn_exch{"2H8' IPN"}="H8'2 IPN"; $resn_exch{"1H5' IPN"}="H5'1 IPN"; $resn_exch{"2H5' IPN"}="H5'2 IPN";
$resn_exch{"'HO2 IPN"}="HO2' IPN"; $resn_exch{"1H3' IPN"}="H3'1 IPN"; $resn_exch{"2H3' IPN"}="H3'2 IPN";
$resn_exch{"1H2' IPN"}="H2'1 IPN"; $resn_exch{"2H2' IPN"}="H2'2 IPN"; $resn_exch{"1H1' IPN"}="H1'1 IPN";
$resn_exch{"2H1' IPN"}="H1'2 IPN"; $resn_exch{"3H1' IPN"}="H1'3 IPN"; $resn_exch{"1H1' IPO"}="H1'1 IPO";
$resn_exch{"2H1' IPO"}="H1'2 IPO"; $resn_exch{"1HN2 IPO"}="HN21 IPO"; $resn_exch{"2HN2 IPO"}="HN22 IPO";
$resn_exch{"1HOP IPP"}="HOP1 IPP"; $resn_exch{"1H   IPP"}=" H1  IPP"; $resn_exch{"2H   IPP"}=" H2  IPP";
$resn_exch{"1H2  IPP"}=" H21 IPP"; $resn_exch{"2H2  IPP"}=" H22 IPP"; $resn_exch{"1HC5 IPR"}="HC51 IPR";
$resn_exch{"2HC5 IPR"}="HC52 IPR"; $resn_exch{"1HC4 IPR"}="HC41 IPR"; $resn_exch{"2HC4 IPR"}="HC42 IPR";
$resn_exch{"1HC1 IPR"}="HC11 IPR"; $resn_exch{"2HC1 IPR"}="HC12 IPR"; $resn_exch{"3HC1 IPR"}="HC13 IPR";
$resn_exch{"1HC3 IPR"}="HC31 IPR"; $resn_exch{"2HC3 IPR"}="HC32 IPR"; $resn_exch{"3HC3 IPR"}="HC33 IPR";
$resn_exch{"1H6  IPT"}=" H61 IPT"; $resn_exch{"2H6  IPT"}=" H62 IPT"; $resn_exch{"1H2' IPT"}="H2'1 IPT";
$resn_exch{"2H2' IPT"}="H2'2 IPT"; $resn_exch{"3H2' IPT"}="H2'3 IPT"; $resn_exch{"1H3' IPT"}="H3'1 IPT";
$resn_exch{"2H3' IPT"}="H3'2 IPT"; $resn_exch{"3H3' IPT"}="H3'3 IPT"; $resn_exch{"1H1  IPU"}=" H11 IPU";
$resn_exch{"2H1  IPU"}=" H12 IPU"; $resn_exch{"3H1  IPU"}=" H13 IPU"; $resn_exch{"1H3  IPU"}=" H31 IPU";
$resn_exch{"2H3  IPU"}=" H32 IPU"; $resn_exch{"3H3  IPU"}=" H33 IPU"; $resn_exch{"1HN2 IPU"}="HN21 IPU";
$resn_exch{"2HN2 IPU"}="HN22 IPU"; $resn_exch{"1H3  IPY"}=" H31 IPY"; $resn_exch{"2H3  IPY"}="H32A IPY";
$resn_exch{"3H3  IPY"}=" H33 IPY"; $resn_exch{"1H9  IPY"}=" H91 IPY"; $resn_exch{"2H9  IPY"}=" H92 IPY";
$resn_exch{"3H9  IPY"}=" H93 IPY"; $resn_exch{"1H15 IPY"}="H151 IPY"; $resn_exch{"2H15 IPY"}="H152 IPY";
$resn_exch{"3H15 IPY"}="H153 IPY"; $resn_exch{"1H18 IPY"}="H181 IPY"; $resn_exch{"2H18 IPY"}="H182 IPY";
$resn_exch{"1H19 IPY"}="H191 IPY"; $resn_exch{"2H19 IPY"}="H192 IPY"; $resn_exch{"1H20 IPY"}="H201 IPY";
$resn_exch{"2H20 IPY"}="H202 IPY"; $resn_exch{"1H21 IPY"}="H211 IPY"; $resn_exch{"2H21 IPY"}="H212 IPY";
$resn_exch{"3H21 IPY"}="H213 IPY"; $resn_exch{"1H22 IPY"}="H221 IPY"; $resn_exch{"2H22 IPY"}="H222 IPY";
$resn_exch{"3H22 IPY"}="H223 IPY"; $resn_exch{"1H28 IPY"}="H281 IPY"; $resn_exch{"2H28 IPY"}="H282 IPY";
$resn_exch{"3H28 IPY"}="H283 IPY"; $resn_exch{"1H29 IPY"}="H291 IPY"; $resn_exch{"2H29 IPY"}="H292 IPY";
$resn_exch{"1H30 IPY"}="H301 IPY"; $resn_exch{"2H30 IPY"}="H302 IPY"; $resn_exch{"1H9  IPZ"}=" H91 IPZ";
$resn_exch{"2H9  IPZ"}=" H92 IPZ"; $resn_exch{"3H9  IPZ"}=" H93 IPZ"; $resn_exch{"1H10 IPZ"}="H101 IPZ";
$resn_exch{"2H10 IPZ"}="H102 IPZ"; $resn_exch{"3H10 IPZ"}="H103 IPZ"; $resn_exch{"1H11 IPZ"}="H111 IPZ";
$resn_exch{"2H11 IPZ"}="H112 IPZ"; $resn_exch{"3H11 IPZ"}="H113 IPZ"; $resn_exch{"1H18 IQA"}="H181 IQA";
$resn_exch{"2H18 IQA"}="H182 IQA"; $resn_exch{"1H2' IQB"}="H2'1 IQB"; $resn_exch{"2H2' IQB"}="H2'2 IQB";
$resn_exch{"1H3' IQB"}="H3'1 IQB"; $resn_exch{"2H3' IQB"}="H3'2 IQB"; $resn_exch{"1H5' IQB"}="H5'1 IQB";
$resn_exch{"2H5' IQB"}="H5'2 IQB"; $resn_exch{" H2* IQB"}=" H2' IQB"; $resn_exch{" H3* IQB"}=" H3' IQB";
$resn_exch{" H5* IQB"}=" H5' IQB"; $resn_exch{" H6* IQB"}=" H6B IQB"; $resn_exch{"1HM2 IQP"}="HM21 IQP";
$resn_exch{"2HM2 IQP"}="HM22 IQP"; $resn_exch{"3HM2 IQP"}="HM23 IQP"; $resn_exch{"1H3' IQP"}="H3'1 IQP";
$resn_exch{"2H3' IQP"}="H3'2 IQP"; $resn_exch{"1H5' IQP"}="H5'1 IQP"; $resn_exch{"2H5' IQP"}="H5'2 IQP";
$resn_exch{"1H6' IQP"}="H6'1 IQP"; $resn_exch{"2H6' IQP"}="H6'2 IQP"; $resn_exch{"1H2' IQS"}="H2'1 IQS";
$resn_exch{"2H2' IQS"}="H2'2 IQS"; $resn_exch{"1H3' IQS"}="H3'1 IQS"; $resn_exch{"2H3' IQS"}="H3'2 IQS";
$resn_exch{"1H5' IQS"}="H5'1 IQS"; $resn_exch{"2H5' IQS"}="H5'2 IQS"; $resn_exch{"3H5' IQS"}="H5'3 IQS";
$resn_exch{"1H2' IQU"}="H2'1 IQU"; $resn_exch{"2H2' IQU"}="H2'2 IQU"; $resn_exch{"1H3' IQU"}="H3'1 IQU";
$resn_exch{"2H3' IQU"}="H3'2 IQU"; $resn_exch{"1H4' IQU"}="H4'1 IQU"; $resn_exch{"2H4' IQU"}="H4'2 IQU";
$resn_exch{"1H12 IQZ"}="H121 IQZ"; $resn_exch{"2H12 IQZ"}="H122 IQZ"; $resn_exch{"1H16 IQZ"}="H161 IQZ";
$resn_exch{"2H16 IQZ"}="H162 IQZ"; $resn_exch{"1H17 IQZ"}="H171 IQZ"; $resn_exch{"2H17 IQZ"}="H172 IQZ";
$resn_exch{"1H18 IQZ"}="H181 IQZ"; $resn_exch{"2H18 IQZ"}="H182 IQZ"; $resn_exch{"1H19 IQZ"}="H191 IQZ";
$resn_exch{"2H19 IQZ"}="H192 IQZ"; $resn_exch{"1HAO IRE"}="HAO1 IRE"; $resn_exch{"2HAO IRE"}="HAO2 IRE";
$resn_exch{"1HAL IRE"}="HAL1 IRE"; $resn_exch{"2HAL IRE"}="HAL2 IRE"; $resn_exch{"1HAM IRE"}="HAM1 IRE";
$resn_exch{"2HAM IRE"}="HAM2 IRE"; $resn_exch{"1HAP IRE"}="HAP1 IRE"; $resn_exch{"2HAP IRE"}="HAP2 IRE";
$resn_exch{"1HAN IRE"}="HAN1 IRE"; $resn_exch{"2HAN IRE"}="HAN2 IRE"; $resn_exch{"1HAJ IRE"}="HAJ1 IRE";
$resn_exch{"2HAJ IRE"}="HAJ2 IRE"; $resn_exch{"1HAK IRE"}="HAK1 IRE"; $resn_exch{"2HAK IRE"}="HAK2 IRE";
$resn_exch{"1HAA IRE"}="HAA1 IRE"; $resn_exch{"2HAA IRE"}="HAA2 IRE"; $resn_exch{"3HAA IRE"}="HAA3 IRE";
$resn_exch{"1H7M IRF"}="H7M1 IRF"; $resn_exch{"2H7M IRF"}="H7M2 IRF"; $resn_exch{"3H7M IRF"}="H7M3 IRF";
$resn_exch{"1H1* IRF"}="H1'1 IRF"; $resn_exch{"2H1* IRF"}="H1'2 IRF"; $resn_exch{" H2* IRF"}=" H2' IRF";
$resn_exch{" H3* IRF"}=" H3' IRF"; $resn_exch{" H4* IRF"}=" H4' IRF"; $resn_exch{"1H5* IRF"}="H5'1 IRF";
$resn_exch{"2H5* IRF"}="H5'2 IRF"; $resn_exch{" H5* IRF"}=" H5' IRF"; $resn_exch{"1HN1 IRI"}="HN11 IRI";
$resn_exch{"2HN1 IRI"}="HN12 IRI"; $resn_exch{"3HN1 IRI"}="HN13 IRI"; $resn_exch{"1HN2 IRI"}="HN21 IRI";
$resn_exch{"2HN2 IRI"}="HN22 IRI"; $resn_exch{"3HN2 IRI"}="HN23 IRI"; $resn_exch{"1HN3 IRI"}="HN31 IRI";
$resn_exch{"2HN3 IRI"}="HN32 IRI"; $resn_exch{"3HN3 IRI"}="HN33 IRI"; $resn_exch{"1HN4 IRI"}="HN41 IRI";
$resn_exch{"2HN4 IRI"}="HN42 IRI"; $resn_exch{"3HN4 IRI"}="HN43 IRI"; $resn_exch{"1HN5 IRI"}="HN51 IRI";
$resn_exch{"2HN5 IRI"}="HN52 IRI"; $resn_exch{"3HN5 IRI"}="HN53 IRI"; $resn_exch{"1HN6 IRI"}="HN61 IRI";
$resn_exch{"2HN6 IRI"}="HN62 IRI"; $resn_exch{"3HN6 IRI"}="HN63 IRI"; $resn_exch{"1H5* IRP"}="H5'1 IRP";
$resn_exch{"2H5* IRP"}="H5'2 IRP"; $resn_exch{" H4* IRP"}=" H4' IRP"; $resn_exch{"4HN* IRP"}="HN'4 IRP";
$resn_exch{" H3* IRP"}=" H3' IRP"; $resn_exch{"3HO* IRP"}="HO'3 IRP"; $resn_exch{" H2* IRP"}=" H2' IRP";
$resn_exch{"2HO* IRP"}="HO'2 IRP"; $resn_exch{" H1* IRP"}=" H1' IRP"; $resn_exch{"1H18 IS2"}="H181 IS2";
$resn_exch{"2H18 IS2"}="H182 IS2"; $resn_exch{"1H19 IS2"}="H191 IS2"; $resn_exch{"2H19 IS2"}="H192 IS2";
$resn_exch{"3H19 IS2"}="H193 IS2"; $resn_exch{"1H4  ISA"}=" H41 ISA"; $resn_exch{"2H4  ISA"}=" H42 ISA";
$resn_exch{"1HB1 ISB"}="HB11 ISB"; $resn_exch{"2HB1 ISB"}="HB12 ISB"; $resn_exch{"3HB1 ISB"}="HB13 ISB";
$resn_exch{"1HB2 ISB"}="HB21 ISB"; $resn_exch{"2HB2 ISB"}="HB22 ISB"; $resn_exch{"3HB2 ISB"}="HB23 ISB";
$resn_exch{"1H3' ISC"}="H3'1 ISC"; $resn_exch{"2H3' ISC"}="H3'2 ISC"; $resn_exch{"1H13 ISD"}="H131 ISD";
$resn_exch{"2H13 ISD"}="H132 ISD"; $resn_exch{"1H15 ISF"}="H151 ISF"; $resn_exch{"2H15 ISF"}="H152 ISF";
$resn_exch{"3H15 ISF"}="H153 ISF"; $resn_exch{"1HN1 ISO"}="HN11 ISO"; $resn_exch{"2HN1 ISO"}="HN12 ISO";
$resn_exch{"1H2' ISO"}="H2'1 ISO"; $resn_exch{"2H2' ISO"}="H2'2 ISO"; $resn_exch{"3H2' ISO"}="H2'3 ISO";
$resn_exch{"1H3' ISO"}="H3'1 ISO"; $resn_exch{"2H3' ISO"}="H3'2 ISO"; $resn_exch{"3H3' ISO"}="H3'3 ISO";
$resn_exch{"1H1  ISP"}=" H11 ISP"; $resn_exch{"2H1  ISP"}=" H12 ISP"; $resn_exch{"3H1  ISP"}=" H13 ISP";
$resn_exch{"1H3  ISP"}=" H31 ISP"; $resn_exch{"2H3  ISP"}=" H32 ISP"; $resn_exch{"3H3  ISP"}=" H33 ISP";
$resn_exch{"2HOP ISP"}="HOP2 ISP"; $resn_exch{"3HOP ISP"}="HOP3 ISP"; $resn_exch{"1H2  IST"}=" H21 IST";
$resn_exch{"3H2  IST"}=" H23 IST"; $resn_exch{"2H2  IST"}=" H22 IST"; $resn_exch{"1H5  IST"}=" H51 IST";
$resn_exch{"2H5  IST"}=" H52 IST"; $resn_exch{"1H10 IST"}="H101 IST"; $resn_exch{"2H10 IST"}="H102 IST";
$resn_exch{"3H10 IST"}="H103 IST"; $resn_exch{"1HA2 IST"}="HA21 IST"; $resn_exch{"2HA2 IST"}="HA22 IST";
$resn_exch{"1H1  IST"}=" H11 IST"; $resn_exch{"2H1  IST"}=" H12 IST"; $resn_exch{"1HC1 ISU"}="HC11 ISU";
$resn_exch{"2HC1 ISU"}="HC12 ISU"; $resn_exch{"3HC1 ISU"}="HC13 ISU"; $resn_exch{"1HC2 ISU"}="HC21 ISU";
$resn_exch{"2HC2 ISU"}="HC22 ISU"; $resn_exch{"1HN1 ISU"}="HN11 ISU"; $resn_exch{"2HN1 ISU"}="HN12 ISU";
$resn_exch{"1H5C ISX"}="H5C1 ISX"; $resn_exch{"2H5C ISX"}="H5C2 ISX"; $resn_exch{"1H1C ISX"}="H1C1 ISX";
$resn_exch{"2H1C ISX"}="H1C2 ISX"; $resn_exch{"1H6C ISX"}="H6C1 ISX"; $resn_exch{"2H6C ISX"}="H6C2 ISX";
$resn_exch{"1H6A ISX"}="H6A1 ISX"; $resn_exch{"2H6A ISX"}="H6A2 ISX"; $resn_exch{"1H21 ITA"}="H211 ITA";
$resn_exch{"2H21 ITA"}="H212 ITA"; $resn_exch{"1H10 ITA"}="H101 ITA"; $resn_exch{"2H10 ITA"}="H102 ITA";
$resn_exch{"1H20 ITB"}="H201 ITB"; $resn_exch{"2H20 ITB"}="H202 ITB"; $resn_exch{"1H4  ITM"}=" H41 ITM";
$resn_exch{"2H4  ITM"}=" H42 ITM"; $resn_exch{"1HB  ITR"}=" HB1 ITR"; $resn_exch{"2HB  ITR"}=" HB2 ITR";
$resn_exch{"1H5* ITT"}="H5'1 ITT"; $resn_exch{"2H5* ITT"}="H5'2 ITT"; $resn_exch{" H4* ITT"}=" H4' ITT";
$resn_exch{" H3* ITT"}=" H3' ITT"; $resn_exch{" H2* ITT"}=" H2' ITT"; $resn_exch{" H1* ITT"}=" H1' ITT";
$resn_exch{"1H1  ITU"}=" H11 ITU"; $resn_exch{"2H1  ITU"}=" H12 ITU"; $resn_exch{"3H1  ITU"}=" H13 ITU";
$resn_exch{"1H2  ITU"}=" H21 ITU"; $resn_exch{"2H2  ITU"}=" H22 ITU"; $resn_exch{"1HN1 ITU"}="HN11 ITU";
$resn_exch{"2HN1 ITU"}="HN12 ITU"; $resn_exch{"2HOP  IU"}="HOP2  IU"; $resn_exch{"3HOP  IU"}="HOP3  IU";
$resn_exch{"1H5*  IU"}=" H5'  IU"; $resn_exch{"2H5*  IU"}="H5''  IU"; $resn_exch{" H4*  IU"}=" H4'  IU";
$resn_exch{" H3*  IU"}=" H3'  IU"; $resn_exch{"H3T   IU"}="HO3'  IU"; $resn_exch{" H2*  IU"}=" H2'  IU";
$resn_exch{"2HO*  IU"}="HO2'  IU"; $resn_exch{" H1*  IU"}=" H1'  IU"; $resn_exch{"1HC1 IU5"}="HC11 IU5";
$resn_exch{"2HC1 IU5"}="HC12 IU5"; $resn_exch{"1H11 IU5"}="H111 IU5"; $resn_exch{"2H11 IU5"}="H112 IU5";
$resn_exch{"1H12 IU5"}="H121 IU5"; $resn_exch{"2H12 IU5"}="H122 IU5"; $resn_exch{"1H14 IU5"}="H141 IU5";
$resn_exch{"1H15 IU5"}="H151 IU5"; $resn_exch{"2H15 IU5"}="H152 IU5"; $resn_exch{"1H16 IU5"}="H161 IU5";
$resn_exch{"2H16 IU5"}="H162 IU5"; $resn_exch{"1H17 IU5"}="H171 IU5"; $resn_exch{"1H18 IU5"}="H181 IU5";
$resn_exch{"2H18 IU5"}="H182 IU5"; $resn_exch{"3H18 IU5"}="H183 IU5"; $resn_exch{"1H19 IU5"}="H191 IU5";
$resn_exch{"2H19 IU5"}="H192 IU5"; $resn_exch{"3H19 IU5"}="H193 IU5"; $resn_exch{"1HOA IU5"}="HOA1 IU5";
$resn_exch{"1HOB IU5"}="HOB1 IU5"; $resn_exch{"1HC2 IU5"}="HC21 IU5"; $resn_exch{"2HC2 IU5"}="HC22 IU5";
$resn_exch{"1H20 IU5"}="H201 IU5"; $resn_exch{"1H21 IU5"}="H211 IU5"; $resn_exch{"2H21 IU5"}="H212 IU5";
$resn_exch{"3H21 IU5"}="H213 IU5"; $resn_exch{"1H22 IU5"}="H221 IU5"; $resn_exch{"2H22 IU5"}="H222 IU5";
$resn_exch{"1H23 IU5"}="H231 IU5"; $resn_exch{"2H23 IU5"}="H232 IU5"; $resn_exch{"1HC3 IU5"}="HC31 IU5";
$resn_exch{"1HC4 IU5"}="HC41 IU5"; $resn_exch{"2HC4 IU5"}="HC42 IU5"; $resn_exch{"4HOA IU5"}="HOA4 IU5";
$resn_exch{"1HC5 IU5"}="HC51 IU5"; $resn_exch{"1HC6 IU5"}="HC61 IU5"; $resn_exch{"2HC6 IU5"}="HC62 IU5";
$resn_exch{"1HC7 IU5"}="HC71 IU5"; $resn_exch{"1HC8 IU5"}="HC81 IU5"; $resn_exch{"1HC9 IU5"}="HC91 IU5";
$resn_exch{"1HC1 IU6"}="HC11 IU6"; $resn_exch{"2HC1 IU6"}="HC12 IU6"; $resn_exch{"1H11 IU6"}="H111 IU6";
$resn_exch{"2H11 IU6"}="H112 IU6"; $resn_exch{"1H12 IU6"}="H121 IU6"; $resn_exch{"2H12 IU6"}="H122 IU6";
$resn_exch{"1H14 IU6"}="H141 IU6"; $resn_exch{"1H15 IU6"}="H151 IU6"; $resn_exch{"2H15 IU6"}="H152 IU6";
$resn_exch{"1H16 IU6"}="H161 IU6"; $resn_exch{"2H16 IU6"}="H162 IU6"; $resn_exch{"1H17 IU6"}="H171 IU6";
$resn_exch{"1H18 IU6"}="H181 IU6"; $resn_exch{"2H18 IU6"}="H182 IU6"; $resn_exch{"3H18 IU6"}="H183 IU6";
$resn_exch{"1H19 IU6"}="H191 IU6"; $resn_exch{"2H19 IU6"}="H192 IU6"; $resn_exch{"3H19 IU6"}="H193 IU6";
$resn_exch{"1HOA IU6"}="HOA1 IU6"; $resn_exch{"1HOB IU6"}="HOB1 IU6"; $resn_exch{"1HC2 IU6"}="HC21 IU6";
$resn_exch{"2HC2 IU6"}="HC22 IU6"; $resn_exch{"1H20 IU6"}="H201 IU6"; $resn_exch{"1H21 IU6"}="H211 IU6";
$resn_exch{"2H21 IU6"}="H212 IU6"; $resn_exch{"3H21 IU6"}="H213 IU6"; $resn_exch{"1H22 IU6"}="H221 IU6";
$resn_exch{"2H22 IU6"}="H222 IU6"; $resn_exch{"1H23 IU6"}="H231 IU6"; $resn_exch{"2H23 IU6"}="H232 IU6";
$resn_exch{"1HC3 IU6"}="HC31 IU6"; $resn_exch{"1HC4 IU6"}="HC41 IU6"; $resn_exch{"2HC4 IU6"}="HC42 IU6";
$resn_exch{"4HOA IU6"}="HOA4 IU6"; $resn_exch{"1HC5 IU6"}="HC51 IU6"; $resn_exch{"1HC6 IU6"}="HC61 IU6";
$resn_exch{"1H26 IU6"}="H261 IU6"; $resn_exch{"2H26 IU6"}="H262 IU6"; $resn_exch{"1H25 IU6"}="H251 IU6";
$resn_exch{"2H25 IU6"}="H252 IU6"; $resn_exch{"3H25 IU6"}="H253 IU6"; $resn_exch{"1HC7 IU6"}="HC71 IU6";
$resn_exch{"1HC8 IU6"}="HC81 IU6"; $resn_exch{"1HC9 IU6"}="HC91 IU6"; $resn_exch{"1HA  IVA"}=" HA1 IVA";
$resn_exch{"2HA  IVA"}=" HA2 IVA"; $resn_exch{"1HG1 IVA"}="HG11 IVA"; $resn_exch{"2HG1 IVA"}="HG12 IVA";
$resn_exch{"3HG1 IVA"}="HG13 IVA"; $resn_exch{"1HG2 IVA"}="HG21 IVA"; $resn_exch{"2HG2 IVA"}="HG22 IVA";
$resn_exch{"3HG2 IVA"}="HG23 IVA"; $resn_exch{"1H57 IVS"}="H571 IVS"; $resn_exch{"2H57 IVS"}="H572 IVS";
$resn_exch{"1H52 IVS"}="H521 IVS"; $resn_exch{"2H52 IVS"}="H522 IVS"; $resn_exch{"1H51 IVS"}="H511 IVS";
$resn_exch{"1H23 IVS"}="H231 IVS"; $resn_exch{"1H58 IVS"}="H581 IVS"; $resn_exch{"2H58 IVS"}="H582 IVS";
$resn_exch{"1H22 IVS"}="H221 IVS"; $resn_exch{"1H20 IVS"}="H201 IVS"; $resn_exch{"1H19 IVS"}="H191 IVS";
$resn_exch{"1H17 IVS"}="H171 IVS"; $resn_exch{"1H16 IVS"}="H161 IVS"; $resn_exch{"1H10 IVS"}="H101 IVS";
$resn_exch{"2H10 IVS"}="H102 IVS"; $resn_exch{"1HC2 IVS"}="HC21 IVS"; $resn_exch{"1HC6 IVS"}="HC61 IVS";
$resn_exch{"2HC6 IVS"}="HC62 IVS"; $resn_exch{"3HC6 IVS"}="HC63 IVS"; $resn_exch{"1H76 IVS"}="H761 IVS";
$resn_exch{"2H76 IVS"}="H762 IVS"; $resn_exch{"3H76 IVS"}="H763 IVS"; $resn_exch{"1H60 IVS"}="H601 IVS";
$resn_exch{"1H67 IVS"}="H671 IVS"; $resn_exch{"2H67 IVS"}="H672 IVS"; $resn_exch{"3H67 IVS"}="H673 IVS";
$resn_exch{"1H63 IVS"}="H631 IVS"; $resn_exch{"2H63 IVS"}="H632 IVS"; $resn_exch{"3H63 IVS"}="H633 IVS";
$resn_exch{"1H38 IVS"}="H381 IVS"; $resn_exch{"1H40 IVS"}="H401 IVS"; $resn_exch{"2H40 IVS"}="H402 IVS";
$resn_exch{"3H40 IVS"}="H403 IVS"; $resn_exch{"1H44 IVS"}="H441 IVS"; $resn_exch{"2H44 IVS"}="H442 IVS";
$resn_exch{"3H44 IVS"}="H443 IVS"; $resn_exch{"1H25 IVS"}="H251 IVS"; $resn_exch{"1H27 IVS"}="H271 IVS";
$resn_exch{"2H27 IVS"}="H272 IVS"; $resn_exch{"3H27 IVS"}="H273 IVS"; $resn_exch{"1H31 IVS"}="H311 IVS";
$resn_exch{"2H31 IVS"}="H312 IVS"; $resn_exch{"3H31 IVS"}="H313 IVS"; $resn_exch{"1HC1 IVS"}="HC11 IVS";
$resn_exch{"2HC1 IVS"}="HC12 IVS"; $resn_exch{"3HC1 IVS"}="HC13 IVS"; $resn_exch{"1H7  IWD"}=" H71 IWD";
$resn_exch{"2H7  IWD"}=" H72 IWD"; $resn_exch{"1H8  IWD"}=" H81 IWD"; $resn_exch{"2H8  IWD"}=" H82 IWD";
$resn_exch{"OH92 IWD"}="H92O IWD"; $resn_exch{"1H39 IX1"}="H391 IX1"; $resn_exch{"2H39 IX1"}="H392 IX1";
$resn_exch{"3H39 IX1"}="H393 IX1"; $resn_exch{"1H40 IX1"}="H401 IX1"; $resn_exch{"2H40 IX1"}="H402 IX1";
$resn_exch{"1H9  IX1"}=" H91 IX1"; $resn_exch{"2H9  IX1"}=" H92 IX1"; $resn_exch{"1H22 IXS"}="H221 IXS";
$resn_exch{"2H22 IXS"}="H222 IXS"; $resn_exch{"3H22 IXS"}="H223 IXS"; $resn_exch{"1H29 IXS"}="H291 IXS";
$resn_exch{"2H29 IXS"}="H292 IXS"; $resn_exch{"3H29 IXS"}="H293 IXS"; $resn_exch{"1H50 IXS"}="H501 IXS";
$resn_exch{"2H50 IXS"}="H502 IXS"; $resn_exch{"3H50 IXS"}="H503 IXS"; $resn_exch{"1H56 IXS"}="H561 IXS";
$resn_exch{"2H56 IXS"}="H562 IXS"; $resn_exch{"1H37 IXS"}="H371 IXS"; $resn_exch{"2H37 IXS"}="H372 IXS";
$resn_exch{"1H8  IXS"}=" H81 IXS"; $resn_exch{"2H8  IXS"}=" H82 IXS"; $resn_exch{"3H8  IXS"}=" H83 IXS";
$resn_exch{"1H10 IXS"}="H101 IXS"; $resn_exch{"2H10 IXS"}="H102 IXS"; $resn_exch{"1H52 IXS"}="H521 IXS";
$resn_exch{"2H52 IXS"}="H522 IXS"; $resn_exch{"3H52 IXS"}="H523 IXS"; $resn_exch{"1H53 IXS"}="H531 IXS";
$resn_exch{"2H53 IXS"}="H532 IXS"; $resn_exch{"3H53 IXS"}="H533 IXS"; $resn_exch{"1H6  IXX"}=" H61 IXX";
$resn_exch{"1H5  IXX"}=" H51 IXX"; $resn_exch{"1H15 IXX"}="H151 IXX"; $resn_exch{"2H15 IXX"}="H152 IXX";
$resn_exch{"1H16 IXX"}="H161 IXX"; $resn_exch{"2H16 IXX"}="H162 IXX"; $resn_exch{"1H17 IXX"}="H171 IXX";
$resn_exch{"2H17 IXX"}="H172 IXX"; $resn_exch{"1H18 IXX"}="H181 IXX"; $resn_exch{"2H18 IXX"}="H182 IXX";
$resn_exch{"3H18 IXX"}="H183 IXX"; $resn_exch{"1H19 IXX"}="H191 IXX"; $resn_exch{"2H19 IXX"}="H192 IXX";
$resn_exch{"3H19 IXX"}="H193 IXX"; $resn_exch{"2H5  IXX"}=" H52 IXX"; $resn_exch{"2H6  IXX"}=" H62 IXX";
$resn_exch{"1HAN IYG"}="HAN1 IYG"; $resn_exch{"2HAN IYG"}="HAN2 IYG"; $resn_exch{"3HAN IYG"}="HAN3 IYG";
$resn_exch{"1OH  IYG"}=" H1O IYG"; $resn_exch{"1HT  IYG"}=" HT1 IYG"; $resn_exch{"1HAT IYG"}="HAT1 IYG";
$resn_exch{"2HAT IYG"}="HAT2 IYG"; $resn_exch{" HN2 IYR"}=" H2  IYR"; $resn_exch{"1HB  IYR"}=" HB2 IYR";
$resn_exch{"2HB  IYR"}=" HB3 IYR"; $resn_exch{"1HAN IYT"}="HAN1 IYT"; $resn_exch{"2HAN IYT"}="HAN2 IYT";
$resn_exch{"3HAN IYT"}="HAN3 IYT"; $resn_exch{" HN  IYT"}=" H   IYT"; $resn_exch{"1HB  IYT"}=" HB2 IYT";
$resn_exch{"2HB  IYT"}=" HB3 IYT"; $resn_exch{" HOH IYT"}=" HH  IYT"; $resn_exch{"1HGT IYT"}="HGT1 IYT";
$resn_exch{"2HGT IYT"}="HGT2 IYT"; $resn_exch{"3HGT IYT"}="HGT3 IYT"; $resn_exch{"1HAJ IYZ"}="HAJ1 IYZ";
$resn_exch{"2HAJ IYZ"}="HAJ2 IYZ"; $resn_exch{"1HAK IYZ"}="HAK1 IYZ"; $resn_exch{"2HAK IYZ"}="HAK2 IYZ";
$resn_exch{"1HAL IYZ"}="HAL1 IYZ"; $resn_exch{"2HAL IYZ"}="HAL2 IYZ"; $resn_exch{"1HAA IYZ"}="HAA1 IYZ";
$resn_exch{"2HAA IYZ"}="HAA2 IYZ"; $resn_exch{"3HAA IYZ"}="HAA3 IYZ"; $resn_exch{"1H2C IZ1"}="H2C1 IZ1";
$resn_exch{"2H2C IZ1"}="H2C2 IZ1"; $resn_exch{"1H21 IZ1"}="H211 IZ1"; $resn_exch{"2H21 IZ1"}="H212 IZ1";
$resn_exch{"1H2C IZ2"}="H2C1 IZ2"; $resn_exch{"2H2C IZ2"}="H2C2 IZ2"; $resn_exch{"1H21 IZ2"}="H211 IZ2";
$resn_exch{"2H21 IZ2"}="H212 IZ2"; $resn_exch{"1H5C IZ2"}="H5C1 IZ2"; $resn_exch{"2H5C IZ2"}="H5C2 IZ2";
$resn_exch{"1H2C IZ3"}="H2C1 IZ3"; $resn_exch{"2H2C IZ3"}="H2C2 IZ3"; $resn_exch{"1H21 IZ3"}="H211 IZ3";
$resn_exch{"2H21 IZ3"}="H212 IZ3"; $resn_exch{"1H18 IZ3"}="H181 IZ3"; $resn_exch{"2H18 IZ3"}="H182 IZ3";
$resn_exch{"1H2C IZ4"}="H2C1 IZ4"; $resn_exch{"2H2C IZ4"}="H2C2 IZ4"; $resn_exch{"1H21 IZ4"}="H211 IZ4";
$resn_exch{"2H21 IZ4"}="H212 IZ4"; $resn_exch{"1H4C IZ4"}="H4C1 IZ4"; $resn_exch{"2H4C IZ4"}="H4C2 IZ4";
$resn_exch{"1H18 IZ4"}="H181 IZ4"; $resn_exch{"2H18 IZ4"}="H182 IZ4"; $resn_exch{"1H27 IZ4"}="H271 IZ4";
$resn_exch{"2H27 IZ4"}="H272 IZ4"; $resn_exch{"1H2C IZ5"}="H2C1 IZ5"; $resn_exch{"2H2C IZ5"}="H2C2 IZ5";
$resn_exch{"1H21 IZ5"}="H211 IZ5"; $resn_exch{"2H21 IZ5"}="H212 IZ5"; $resn_exch{"1H5C IZ5"}="H5C1 IZ5";
$resn_exch{"2H5C IZ5"}="H5C2 IZ5"; $resn_exch{"1H30 IZ5"}="H301 IZ5"; $resn_exch{"2H30 IZ5"}="H302 IZ5";
$resn_exch{"1H29 IZ5"}="H291 IZ5"; $resn_exch{"2H29 IZ5"}="H292 IZ5"; $resn_exch{"1H17 IZA"}="H171 IZA";
$resn_exch{"2H17 IZA"}="H172 IZA"; $resn_exch{"3H17 IZA"}="H173 IZA"; $resn_exch{"1H16 IZA"}="H161 IZA";
$resn_exch{"2H16 IZA"}="H162 IZA"; $resn_exch{"3H16 IZA"}="H163 IZA"; $resn_exch{"1H15 IZA"}="H151 IZA";
$resn_exch{"2H15 IZA"}="H152 IZA"; $resn_exch{"3H15 IZA"}="H153 IZA"; $resn_exch{"1H2C IZB"}="H2C1 IZB";
$resn_exch{"2H2C IZB"}="H2C2 IZB"; $resn_exch{"1H21 IZB"}="H211 IZB"; $resn_exch{"2H21 IZB"}="H212 IZB";
$resn_exch{"1H18 IZB"}="H181 IZB"; $resn_exch{"2H18 IZB"}="H182 IZB"; $resn_exch{"1H27 IZB"}="H271 IZB";
$resn_exch{"2H27 IZB"}="H272 IZB"; $resn_exch{"1H20 IZB"}="H201 IZB"; $resn_exch{"2H20 IZB"}="H202 IZB";
$resn_exch{"3H20 IZB"}="H203 IZB"; $resn_exch{"1H2  IZC"}=" H21 IZC"; $resn_exch{"2H2  IZC"}=" H22 IZC";
$resn_exch{"1H6  IZC"}=" H61 IZC"; $resn_exch{"2H6  IZC"}=" H62 IZC"; $resn_exch{"1H2C IZD"}="H2C1 IZD";
$resn_exch{"2H2C IZD"}="H2C2 IZD"; $resn_exch{"1H21 IZD"}="H211 IZD"; $resn_exch{"2H21 IZD"}="H212 IZD";
$resn_exch{"1H27 IZD"}="H271 IZD"; $resn_exch{"2H27 IZD"}="H272 IZD"; $resn_exch{"1H4C IZD"}="H4C1 IZD";
$resn_exch{"2H4C IZD"}="H4C2 IZD"; $resn_exch{"1H71 IZD"}="H711 IZD"; $resn_exch{"2H71 IZD"}="H712 IZD";
$resn_exch{"3H71 IZD"}="H713 IZD"; $resn_exch{"1H2C IZE"}="H2C1 IZE"; $resn_exch{"2H2C IZE"}="H2C2 IZE";
$resn_exch{"1H21 IZE"}="H211 IZE"; $resn_exch{"2H21 IZE"}="H212 IZE"; $resn_exch{"1H2C IZF"}="H2C1 IZF";
$resn_exch{"2H2C IZF"}="H2C2 IZF"; $resn_exch{"1H21 IZF"}="H211 IZF"; $resn_exch{"2H21 IZF"}="H212 IZF";
$resn_exch{"1H23 IZF"}="H231 IZF"; $resn_exch{"2H23 IZF"}="H232 IZF"; $resn_exch{"1H27 IZF"}="H271 IZF";
$resn_exch{"2H27 IZF"}="H272 IZF"; $resn_exch{"1H18 IZF"}="H181 IZF"; $resn_exch{"2H18 IZF"}="H182 IZF";
$resn_exch{"1H4C IZF"}="H4C1 IZF"; $resn_exch{"2H4C IZF"}="H4C2 IZF"; $resn_exch{"1H5C IZF"}="H5C1 IZF";
$resn_exch{"2H5C IZF"}="H5C2 IZF"; $resn_exch{"1H39 IZF"}="H391 IZF"; $resn_exch{"2H39 IZF"}="H392 IZF";
$resn_exch{"3H39 IZF"}="H393 IZF"; $resn_exch{"1H4C J01"}="H4C1 J01"; $resn_exch{"2H4C J01"}="H4C2 J01";
$resn_exch{"1H7C J01"}="H7C1 J01"; $resn_exch{"2H7C J01"}="H7C2 J01"; $resn_exch{"1H01 J07"}="H011 J07";
$resn_exch{"2H01 J07"}="H012 J07"; $resn_exch{"3H01 J07"}="H013 J07"; $resn_exch{"1H02 J07"}="H021 J07";
$resn_exch{"2H02 J07"}="H022 J07"; $resn_exch{"1H07 J07"}="H071 J07"; $resn_exch{"2H07 J07"}="H072 J07";
$resn_exch{"1H08 J07"}="H081 J07"; $resn_exch{"2H08 J07"}="H082 J07"; $resn_exch{"1H10 J07"}="H101 J07";
$resn_exch{"2H10 J07"}="H102 J07"; $resn_exch{"1H11 J07"}="H111 J07"; $resn_exch{"2H11 J07"}="H112 J07";
$resn_exch{"1H19 J12"}="H191 J12"; $resn_exch{"2H19 J12"}="H192 J12"; $resn_exch{"1H20 J12"}="H201 J12";
$resn_exch{"2H20 J12"}="H202 J12"; $resn_exch{"1H21 J12"}="H211 J12"; $resn_exch{"2H21 J12"}="H212 J12";
$resn_exch{"1H23 J12"}="H231 J12"; $resn_exch{"2H23 J12"}="H232 J12"; $resn_exch{"1H24 J12"}="H241 J12";
$resn_exch{"2H24 J12"}="H242 J12"; $resn_exch{"1H26 J12"}="H261 J12"; $resn_exch{"2H26 J12"}="H262 J12";
$resn_exch{"1H27 J12"}="H271 J12"; $resn_exch{"2H27 J12"}="H272 J12"; $resn_exch{"1H28 J12"}="H281 J12";
$resn_exch{"2H28 J12"}="H282 J12"; $resn_exch{"1H35 J12"}="H351 J12"; $resn_exch{"2H35 J12"}="H352 J12";
$resn_exch{"1H36 J12"}="H361 J12"; $resn_exch{"2H36 J12"}="H362 J12"; $resn_exch{"1H37 J12"}="H371 J12";
$resn_exch{"2H37 J12"}="H372 J12"; $resn_exch{"1H6  J12"}=" H61 J12"; $resn_exch{"2H6  J12"}=" H62 J12";
$resn_exch{"1H42 J12"}="H421 J12"; $resn_exch{"2H42 J12"}="H422 J12"; $resn_exch{"1H19 J15"}="H191 J15";
$resn_exch{"2H19 J15"}="H192 J15"; $resn_exch{"1H20 J15"}="H201 J15"; $resn_exch{"2H20 J15"}="H202 J15";
$resn_exch{"1H21 J15"}="H211 J15"; $resn_exch{"2H21 J15"}="H212 J15"; $resn_exch{"1H23 J15"}="H231 J15";
$resn_exch{"2H23 J15"}="H232 J15"; $resn_exch{"1H24 J15"}="H241 J15"; $resn_exch{"2H24 J15"}="H242 J15";
$resn_exch{"1H28 J15"}="H281 J15"; $resn_exch{"2H28 J15"}="H282 J15"; $resn_exch{"1H35 J15"}="H351 J15";
$resn_exch{"2H35 J15"}="H352 J15"; $resn_exch{"1H36 J15"}="H361 J15"; $resn_exch{"2H36 J15"}="H362 J15";
$resn_exch{"1H37 J15"}="H371 J15"; $resn_exch{"2H37 J15"}="H372 J15"; $resn_exch{"1H26 J15"}="H261 J15";
$resn_exch{"2H26 J15"}="H262 J15"; $resn_exch{"1H27 J15"}="H271 J15"; $resn_exch{"2H27 J15"}="H272 J15";
$resn_exch{"1H6  J15"}=" H61 J15"; $resn_exch{"2H6  J15"}=" H62 J15"; $resn_exch{"1HBA J60"}="HBA1 J60";
$resn_exch{"2HBA J60"}="HBA2 J60"; $resn_exch{"3HBA J60"}="HBA3 J60"; $resn_exch{"1HAZ J60"}="HAZ1 J60";
$resn_exch{"2HAZ J60"}="HAZ2 J60"; $resn_exch{"1HAY J60"}="HAY1 J60"; $resn_exch{"2HAY J60"}="HAY2 J60";
$resn_exch{"1HAW J60"}="HAW1 J60"; $resn_exch{"2HAW J60"}="HAW2 J60"; $resn_exch{"1HBB J60"}="HBB1 J60";
$resn_exch{"2HBB J60"}="HBB2 J60"; $resn_exch{"3HBB J60"}="HBB3 J60"; $resn_exch{"1HAV J60"}="HAV1 J60";
$resn_exch{"2HAV J60"}="HAV2 J60"; $resn_exch{"1HAQ J60"}="HAQ1 J60"; $resn_exch{"2HAQ J60"}="HAQ2 J60";
$resn_exch{"3HAQ J60"}="HAQ3 J60"; $resn_exch{"1HAR J60"}="HAR1 J60"; $resn_exch{"2HAR J60"}="HAR2 J60";
$resn_exch{"3HAR J60"}="HAR3 J60"; $resn_exch{"1H1  J77"}=" H11 J77"; $resn_exch{"2H1  J77"}=" H12 J77";
$resn_exch{"3H1  J77"}=" H13 J77"; $resn_exch{"1H7  J77"}=" H71 J77"; $resn_exch{"2H7  J77"}=" H72 J77";
$resn_exch{"1H8  J77"}=" H81 J77"; $resn_exch{"2H8  J77"}=" H82 J77"; $resn_exch{"1H10 J77"}="H101 J77";
$resn_exch{"2H10 J77"}="H102 J77"; $resn_exch{"1H11 J77"}="H111 J77"; $resn_exch{"2H11 J77"}="H112 J77";
$resn_exch{"1H12 J77"}="H121 J77"; $resn_exch{"2H12 J77"}="H122 J77"; $resn_exch{"1H13 J77"}="H131 J77";
$resn_exch{"2H13 J77"}="H132 J77"; $resn_exch{"1H24 J77"}="H241 J77"; $resn_exch{"2H24 J77"}="H242 J77";
$resn_exch{"1H25 J77"}="H251 J77"; $resn_exch{"2H25 J77"}="H252 J77"; $resn_exch{"3H25 J77"}="H253 J77";
$resn_exch{"1H1  J78"}=" H11 J78"; $resn_exch{"2H1  J78"}=" H12 J78"; $resn_exch{"3H1  J78"}=" H13 J78";
$resn_exch{"1H7  J78"}=" H71 J78"; $resn_exch{"2H7  J78"}=" H72 J78"; $resn_exch{"1H8  J78"}=" H81 J78";
$resn_exch{"2H8  J78"}=" H82 J78"; $resn_exch{"1H10 J78"}="H101 J78"; $resn_exch{"2H10 J78"}="H102 J78";
$resn_exch{"1H11 J78"}="H111 J78"; $resn_exch{"2H11 J78"}="H112 J78"; $resn_exch{"1H12 J78"}="H121 J78";
$resn_exch{"2H12 J78"}="H122 J78"; $resn_exch{"1H13 J78"}="H131 J78"; $resn_exch{"2H13 J78"}="H132 J78";
$resn_exch{"1H14 J78"}="H141 J78"; $resn_exch{"2H14 J78"}="H142 J78"; $resn_exch{"1H25 J78"}="H251 J78";
$resn_exch{"2H25 J78"}="H252 J78"; $resn_exch{"1H26 J78"}="H261 J78"; $resn_exch{"2H26 J78"}="H262 J78";
$resn_exch{"3H26 J78"}="H263 J78"; $resn_exch{"1H1  J80"}=" H11 J80"; $resn_exch{"2H1  J80"}=" H12 J80";
$resn_exch{"3H1  J80"}=" H13 J80"; $resn_exch{"1H7  J80"}=" H71 J80"; $resn_exch{"2H7  J80"}=" H72 J80";
$resn_exch{"1H8  J80"}=" H81 J80"; $resn_exch{"2H8  J80"}=" H82 J80"; $resn_exch{"1H10 J80"}="H101 J80";
$resn_exch{"2H10 J80"}="H102 J80"; $resn_exch{"1H11 J80"}="H111 J80"; $resn_exch{"2H11 J80"}="H112 J80";
$resn_exch{"1H12 J80"}="H121 J80"; $resn_exch{"2H12 J80"}="H122 J80"; $resn_exch{"1H13 J80"}="H131 J80";
$resn_exch{"2H13 J80"}="H132 J80"; $resn_exch{"1H14 J80"}="H141 J80"; $resn_exch{"2H14 J80"}="H142 J80";
$resn_exch{"1H15 J80"}="H151 J80"; $resn_exch{"2H15 J80"}="H152 J80"; $resn_exch{"1H26 J80"}="H261 J80";
$resn_exch{"2H26 J80"}="H262 J80"; $resn_exch{"1H27 J80"}="H271 J80"; $resn_exch{"2H27 J80"}="H272 J80";
$resn_exch{"3H27 J80"}="H273 J80"; $resn_exch{"1H1  JAN"}="H11A JAN"; $resn_exch{"2H1  JAN"}=" H12 JAN";
$resn_exch{"3H1  JAN"}="H13A JAN"; $resn_exch{"1HN3 JAN"}="HN31 JAN"; $resn_exch{"2HN3 JAN"}="HN32 JAN";
$resn_exch{"1H18 JAN"}="H181 JAN"; $resn_exch{"2H18 JAN"}="H182 JAN"; $resn_exch{"3H18 JAN"}="H183 JAN";
$resn_exch{"1H40 JAS"}="H401 JAS"; $resn_exch{"2H40 JAS"}="H402 JAS"; $resn_exch{"1H41 JAS"}="H411 JAS";
$resn_exch{"2H41 JAS"}="H412 JAS"; $resn_exch{"3H41 JAS"}="H413 JAS"; $resn_exch{"1H33 JAS"}="H331 JAS";
$resn_exch{"2H33 JAS"}="H332 JAS"; $resn_exch{"1H42 JAS"}="H421 JAS"; $resn_exch{"2H42 JAS"}="H422 JAS";
$resn_exch{"3H42 JAS"}="H423 JAS"; $resn_exch{"1H39 JAS"}="H391 JAS"; $resn_exch{"2H39 JAS"}="H392 JAS";
$resn_exch{"3H39 JAS"}="H393 JAS"; $resn_exch{"1H29 JAS"}="H291 JAS"; $resn_exch{"2H29 JAS"}="H292 JAS";
$resn_exch{"1H28 JAS"}="H281 JAS"; $resn_exch{"2H28 JAS"}="H282 JAS"; $resn_exch{"1H38 JAS"}="H381 JAS";
$resn_exch{"2H38 JAS"}="H382 JAS"; $resn_exch{"3H38 JAS"}="H383 JAS"; $resn_exch{"1H43 JAS"}="H431 JAS";
$resn_exch{"2H43 JAS"}="H432 JAS"; $resn_exch{"3H43 JAS"}="H433 JAS"; $resn_exch{"1H25 JAS"}="H251 JAS";
$resn_exch{"2H25 JAS"}="H252 JAS"; $resn_exch{"1H2  JAS"}=" H21 JAS"; $resn_exch{"2H2  JAS"}=" H22 JAS";
$resn_exch{"1H4  JAS"}=" H41 JAS"; $resn_exch{"2H4  JAS"}=" H42 JAS"; $resn_exch{"1H6  JAS"}=" H61 JAS";
$resn_exch{"2H6  JAS"}=" H62 JAS"; $resn_exch{"1H37 JAS"}="H371 JAS"; $resn_exch{"2H37 JAS"}="H372 JAS";
$resn_exch{"3H37 JAS"}="H373 JAS"; $resn_exch{"1H22 JAS"}="H221 JAS"; $resn_exch{"2H22 JAS"}="H222 JAS";
$resn_exch{"1H21 JAS"}="H211 JAS"; $resn_exch{"2H21 JAS"}="H212 JAS"; $resn_exch{"1H44 JAS"}="H441 JAS";
$resn_exch{"2H44 JAS"}="H442 JAS"; $resn_exch{"3H44 JAS"}="H443 JAS"; $resn_exch{"1H36 JAS"}="H361 JAS";
$resn_exch{"2H36 JAS"}="H362 JAS"; $resn_exch{"3H36 JAS"}="H363 JAS"; $resn_exch{"1H7C JCL"}="H7C1 JCL";
$resn_exch{"2H7C JCL"}="H7C2 JCL"; $resn_exch{"1H8C JCL"}="H8C1 JCL"; $resn_exch{"2H8C JCL"}="H8C2 JCL";
$resn_exch{"1H9C JCL"}="H9C1 JCL"; $resn_exch{"2H9C JCL"}="H9C2 JCL"; $resn_exch{"1H10 JCL"}="H101 JCL";
$resn_exch{"2H10 JCL"}="H102 JCL"; $resn_exch{"1HC8 JE2"}="HC81 JE2"; $resn_exch{"2HC8 JE2"}="HC82 JE2";
$resn_exch{"3HC8 JE2"}="HC83 JE2"; $resn_exch{"1H13 JE2"}="H131 JE2"; $resn_exch{"2H13 JE2"}="H132 JE2";
$resn_exch{"1H25 JE2"}="H251 JE2"; $resn_exch{"2H25 JE2"}="H252 JE2"; $resn_exch{"1H28 JE2"}="H281 JE2";
$resn_exch{"2H28 JE2"}="H282 JE2"; $resn_exch{"3H28 JE2"}="H283 JE2"; $resn_exch{"1H29 JE2"}="H291 JE2";
$resn_exch{"2H29 JE2"}="H292 JE2"; $resn_exch{"3H29 JE2"}="H293 JE2"; $resn_exch{"1H34 JE2"}="H341 JE2";
$resn_exch{"2H34 JE2"}="H342 JE2"; $resn_exch{"1H41 JE2"}="H411 JE2"; $resn_exch{"2H41 JE2"}="H412 JE2";
$resn_exch{"3H41 JE2"}="H413 JE2"; $resn_exch{"1H38 JEF"}="H381 JEF"; $resn_exch{"2H38 JEF"}="H382 JEF";
$resn_exch{"3H38 JEF"}="H383 JEF"; $resn_exch{"1H37 JEF"}="H371 JEF"; $resn_exch{"2H37 JEF"}="H372 JEF";
$resn_exch{"1H36 JEF"}="H361 JEF"; $resn_exch{"2H36 JEF"}="H362 JEF"; $resn_exch{"1H35 JEF"}="H351 JEF";
$resn_exch{"2H35 JEF"}="H352 JEF"; $resn_exch{"1H34 JEF"}="H341 JEF"; $resn_exch{"2H34 JEF"}="H342 JEF";
$resn_exch{"3H34 JEF"}="H343 JEF"; $resn_exch{"1H32 JEF"}="H321 JEF"; $resn_exch{"2H32 JEF"}="H322 JEF";
$resn_exch{"1H40 JEF"}="H401 JEF"; $resn_exch{"2H40 JEF"}="H402 JEF"; $resn_exch{"3H40 JEF"}="H403 JEF";
$resn_exch{"1H20 JEF"}="H201 JEF"; $resn_exch{"2H20 JEF"}="H202 JEF"; $resn_exch{"1H18 JEF"}="H181 JEF";
$resn_exch{"2H18 JEF"}="H182 JEF"; $resn_exch{"3H18 JEF"}="H183 JEF"; $resn_exch{"1H17 JEF"}="H171 JEF";
$resn_exch{"2H17 JEF"}="H172 JEF"; $resn_exch{"1H1  JEF"}="H11A JEF"; $resn_exch{"2H1  JEF"}=" H12 JEF";
$resn_exch{"3H1  JEF"}="H13A JEF"; $resn_exch{"1H3  JEF"}=" H31 JEF"; $resn_exch{"2H3  JEF"}=" H32 JEF";
$resn_exch{"1H6  JEF"}=" H61 JEF"; $resn_exch{"2H6  JEF"}=" H62 JEF"; $resn_exch{"3H6  JEF"}=" H63 JEF";
$resn_exch{"1H4  JEF"}=" H41 JEF"; $resn_exch{"2H4  JEF"}=" H42 JEF"; $resn_exch{"1H9  JEF"}=" H91 JEF";
$resn_exch{"2H9  JEF"}=" H92 JEF"; $resn_exch{"3H9  JEF"}=" H93 JEF"; $resn_exch{"1H8  JEF"}=" H81 JEF";
$resn_exch{"2H8  JEF"}=" H82 JEF"; $resn_exch{"1H12 JEF"}="H121 JEF"; $resn_exch{"2H12 JEF"}="H122 JEF";
$resn_exch{"3H12 JEF"}="H123 JEF"; $resn_exch{"1H10 JEF"}="H101 JEF"; $resn_exch{"2H10 JEF"}="H102 JEF";
$resn_exch{"1H15 JEF"}="H151 JEF"; $resn_exch{"2H15 JEF"}="H152 JEF"; $resn_exch{"3H15 JEF"}="H153 JEF";
$resn_exch{"1H14 JEF"}="H141 JEF"; $resn_exch{"2H14 JEF"}="H142 JEF"; $resn_exch{"1H16 JEF"}="H161 JEF";
$resn_exch{"2H16 JEF"}="H162 JEF"; $resn_exch{"1H31 JEF"}="H311 JEF"; $resn_exch{"2H31 JEF"}="H312 JEF";
$resn_exch{"3H31 JEF"}="H313 JEF"; $resn_exch{"1HN1 JEF"}="HN11 JEF"; $resn_exch{"2HN1 JEF"}="HN12 JEF";
$resn_exch{"1H1  JEN"}=" H11 JEN"; $resn_exch{"2H1  JEN"}=" H12 JEN"; $resn_exch{"3H1  JEN"}=" H13 JEN";
$resn_exch{"1H10 JEN"}="H101 JEN"; $resn_exch{"2H10 JEN"}="H102 JEN"; $resn_exch{"1H11 JEN"}="H111 JEN";
$resn_exch{"2H11 JEN"}="H112 JEN"; $resn_exch{"1H13 JEN"}="H131 JEN"; $resn_exch{"2H13 JEN"}="H132 JEN";
$resn_exch{"1H14 JEN"}="H141 JEN"; $resn_exch{"2H14 JEN"}="H142 JEN"; $resn_exch{"1H21 JEN"}="H211 JEN";
$resn_exch{"2H21 JEN"}="H212 JEN"; $resn_exch{"3H21 JEN"}="H213 JEN"; $resn_exch{"1H12 JH0"}="H121 JH0";
$resn_exch{"2H12 JH0"}="H122 JH0"; $resn_exch{"1H13 JH0"}="H131 JH0"; $resn_exch{"2H13 JH0"}="H132 JH0";
$resn_exch{"1H14 JH0"}="H141 JH0"; $resn_exch{"2H14 JH0"}="H142 JH0"; $resn_exch{"1H15 JH0"}="H151 JH0";
$resn_exch{"2H15 JH0"}="H152 JH0"; $resn_exch{"1H16 JH0"}="H161 JH0"; $resn_exch{"2H16 JH0"}="H162 JH0";
$resn_exch{"1H17 JH0"}="H171 JH0"; $resn_exch{"2H17 JH0"}="H172 JH0"; $resn_exch{"1H18 JH0"}="H181 JH0";
$resn_exch{"2H18 JH0"}="H182 JH0"; $resn_exch{"1H19 JH0"}="H191 JH0"; $resn_exch{"2H19 JH0"}="H192 JH0";
$resn_exch{"1H20 JH0"}="H201 JH0"; $resn_exch{"2H20 JH0"}="H202 JH0"; $resn_exch{"1H21 JH0"}="H211 JH0";
$resn_exch{"2H21 JH0"}="H212 JH0"; $resn_exch{"1H22 JH0"}="H221 JH0"; $resn_exch{"2H22 JH0"}="H222 JH0";
$resn_exch{"1H23 JH0"}="H231 JH0"; $resn_exch{"2H23 JH0"}="H232 JH0"; $resn_exch{"1H24 JH0"}="H241 JH0";
$resn_exch{"2H24 JH0"}="H242 JH0"; $resn_exch{"1H25 JH0"}="H251 JH0"; $resn_exch{"2H25 JH0"}="H252 JH0";
$resn_exch{"1H26 JH0"}="H261 JH0"; $resn_exch{"2H26 JH0"}="H262 JH0"; $resn_exch{"3H26 JH0"}="H263 JH0";
$resn_exch{"1H1  JH0"}=" H11 JH0"; $resn_exch{"2H1  JH0"}=" H12 JH0"; $resn_exch{"1H2  JH0"}=" H21 JH0";
$resn_exch{"2H2  JH0"}=" H22 JH0"; $resn_exch{"1H3  JH0"}=" H31 JH0"; $resn_exch{"2H3  JH0"}=" H32 JH0";
$resn_exch{"1H4  JH0"}="H41A JH0"; $resn_exch{"2H4  JH0"}="H42A JH0"; $resn_exch{"1H37 JH0"}="H371 JH0";
$resn_exch{"2H37 JH0"}="H372 JH0"; $resn_exch{"1H9  JH0"}=" H91 JH0"; $resn_exch{"2H9  JH0"}=" H92 JH0";
$resn_exch{"3H9  JH0"}=" H93 JH0"; $resn_exch{"1H8  JH0"}=" H81 JH0"; $resn_exch{"2H8  JH0"}=" H82 JH0";
$resn_exch{"1H34 JH0"}="H341 JH0"; $resn_exch{"2H34 JH0"}="H342 JH0"; $resn_exch{"1H33 JH0"}="H331 JH0";
$resn_exch{"2H33 JH0"}="H332 JH0"; $resn_exch{"1H32 JH0"}="H321 JH0"; $resn_exch{"2H32 JH0"}="H322 JH0";
$resn_exch{"1H31 JH0"}="H311 JH0"; $resn_exch{"2H31 JH0"}="H312 JH0"; $resn_exch{"1H28 JH0"}="H281 JH0";
$resn_exch{"2H28 JH0"}="H282 JH0"; $resn_exch{"3H28 JH0"}="H283 JH0"; $resn_exch{"1H29 JH0"}="H291 JH0";
$resn_exch{"2H29 JH0"}="H292 JH0"; $resn_exch{"3H29 JH0"}="H293 JH0"; $resn_exch{"1H30 JH0"}="H301 JH0";
$resn_exch{"2H30 JH0"}="H302 JH0"; $resn_exch{"3H30 JH0"}="H303 JH0"; $resn_exch{"1H13 JIN"}="H131 JIN";
$resn_exch{"2H13 JIN"}="H132 JIN"; $resn_exch{"3H13 JIN"}="H133 JIN"; $resn_exch{"1H34 JIN"}="H341 JIN";
$resn_exch{"2H34 JIN"}="H342 JIN"; $resn_exch{"3H34 JIN"}="H343 JIN"; $resn_exch{"1H24 JJ3"}="H241 JJ3";
$resn_exch{"2H24 JJ3"}="H242 JJ3"; $resn_exch{"1H10 JJ3"}="H101 JJ3"; $resn_exch{"2H10 JJ3"}="H102 JJ3";
$resn_exch{"1H11 JJ3"}="H111 JJ3"; $resn_exch{"2H11 JJ3"}="H112 JJ3"; $resn_exch{"1H12 JJ3"}="H121 JJ3";
$resn_exch{"2H12 JJ3"}="H122 JJ3"; $resn_exch{"1H26 JJ3"}="H261 JJ3"; $resn_exch{"2H26 JJ3"}="H262 JJ3";
$resn_exch{"3H26 JJ3"}="H263 JJ3"; $resn_exch{"1H1  JN3"}=" H11 JN3"; $resn_exch{"2H1  JN3"}=" H12 JN3";
$resn_exch{"1H2  JN3"}=" H21 JN3"; $resn_exch{"2H2  JN3"}=" H22 JN3"; $resn_exch{"1H4  JN3"}=" H41 JN3";
$resn_exch{"2H4  JN3"}=" H42 JN3"; $resn_exch{"1H6  JN3"}=" H61 JN3"; $resn_exch{"2H6  JN3"}=" H62 JN3";
$resn_exch{"1H11 JN3"}="H111 JN3"; $resn_exch{"2H11 JN3"}="H112 JN3"; $resn_exch{"1H12 JN3"}="H121 JN3";
$resn_exch{"2H12 JN3"}="H122 JN3"; $resn_exch{"1H15 JN3"}="H151 JN3"; $resn_exch{"2H15 JN3"}="H152 JN3";
$resn_exch{"1H16 JN3"}="H161 JN3"; $resn_exch{"2H16 JN3"}="H162 JN3"; $resn_exch{"1H18 JN3"}="H181 JN3";
$resn_exch{"2H18 JN3"}="H182 JN3"; $resn_exch{"3H18 JN3"}="H183 JN3"; $resn_exch{"1H19 JN3"}="H191 JN3";
$resn_exch{"2H19 JN3"}="H192 JN3"; $resn_exch{"3H19 JN3"}="H193 JN3"; $resn_exch{"1H21 JN3"}="H211 JN3";
$resn_exch{"2H21 JN3"}="H212 JN3"; $resn_exch{"3H21 JN3"}="H213 JN3"; $resn_exch{"1H22 JN3"}="H221 JN3";
$resn_exch{"2H22 JN3"}="H222 JN3"; $resn_exch{"1H23 JN3"}="H231 JN3"; $resn_exch{"2H23 JN3"}="H232 JN3";
$resn_exch{"1H2  JNH"}=" H21 JNH"; $resn_exch{"2H2  JNH"}=" H22 JNH"; $resn_exch{"1H3  JNH"}=" H31 JNH";
$resn_exch{"2H3  JNH"}=" H32 JNH"; $resn_exch{"1H4  JNH"}=" H41 JNH"; $resn_exch{"2H4  JNH"}=" H42 JNH";
$resn_exch{"1HN2 JNH"}="HN21 JNH"; $resn_exch{"2HN2 JNH"}="HN22 JNH"; $resn_exch{"1H7  JNH"}=" H71 JNH";
$resn_exch{"2H7  JNH"}=" H72 JNH"; $resn_exch{"1H20 JNH"}="H201 JNH"; $resn_exch{"2H20 JNH"}="H202 JNH";
$resn_exch{"1HN3 JNH"}="HN31 JNH"; $resn_exch{"2HN3 JNH"}="HN32 JNH"; $resn_exch{"1H23 JNK"}="H231 JNK";
$resn_exch{"2H23 JNK"}="H232 JNK"; $resn_exch{"1H24 JNK"}="H241 JNK"; $resn_exch{"2H24 JNK"}="H242 JNK";
$resn_exch{"1H26 JNK"}="H261 JNK"; $resn_exch{"2H26 JNK"}="H262 JNK"; $resn_exch{"1H27 JNK"}="H271 JNK";
$resn_exch{"2H27 JNK"}="H272 JNK"; $resn_exch{"1H30 JNK"}="H301 JNK"; $resn_exch{"2H30 JNK"}="H302 JNK";
$resn_exch{"3H30 JNK"}="H303 JNK"; $resn_exch{"1H10 JOS"}="H101 JOS"; $resn_exch{"2H10 JOS"}="H102 JOS";
$resn_exch{"1H16 JOS"}="H161 JOS"; $resn_exch{"2H16 JOS"}="H162 JOS"; $resn_exch{"3H16 JOS"}="H163 JOS";
$resn_exch{"1H2  JOS"}="H21A JOS"; $resn_exch{"2H2  JOS"}=" H22 JOS"; $resn_exch{"1H18 JOS"}="H181 JOS";
$resn_exch{"2H18 JOS"}="H182 JOS"; $resn_exch{"3H18 JOS"}="H183 JOS"; $resn_exch{"1H19 JOS"}="H191 JOS";
$resn_exch{"2H19 JOS"}="H192 JOS"; $resn_exch{"3H19 JOS"}="H193 JOS"; $resn_exch{"1H20 JOS"}="H201 JOS";
$resn_exch{"2H20 JOS"}="H202 JOS"; $resn_exch{"1H7  JOS"}=" H71 JOS"; $resn_exch{"2H7  JOS"}=" H72 JOS";
$resn_exch{"1H22 JOS"}="H221 JOS"; $resn_exch{"2H22 JOS"}="H222 JOS"; $resn_exch{"3H22 JOS"}="H223 JOS";
$resn_exch{"1H6A JOS"}="H6A1 JOS"; $resn_exch{"2H6A JOS"}="H6A2 JOS"; $resn_exch{"3H6A JOS"}="H6A3 JOS";
$resn_exch{"1H7A JOS"}="H7A1 JOS"; $resn_exch{"2H7A JOS"}="H7A2 JOS"; $resn_exch{"3H7A JOS"}="H7A3 JOS";
$resn_exch{"1H2B JOS"}="H2B1 JOS"; $resn_exch{"2H2B JOS"}="H2B2 JOS"; $resn_exch{"1H6B JOS"}="H6B1 JOS";
$resn_exch{"2H6B JOS"}="H6B2 JOS"; $resn_exch{"3H6B JOS"}="H6B3 JOS"; $resn_exch{"1H7B JOS"}="H7B1 JOS";
$resn_exch{"2H7B JOS"}="H7B2 JOS"; $resn_exch{"3H7B JOS"}="H7B3 JOS"; $resn_exch{"1H2C JOS"}="H2C1 JOS";
$resn_exch{"2H2C JOS"}="H2C2 JOS"; $resn_exch{"1H4C JOS"}="H4C1 JOS"; $resn_exch{"2H4C JOS"}="H4C2 JOS";
$resn_exch{"3H4C JOS"}="H4C3 JOS"; $resn_exch{"1H5C JOS"}="H5C1 JOS"; $resn_exch{"2H5C JOS"}="H5C2 JOS";
$resn_exch{"3H5C JOS"}="H5C3 JOS"; $resn_exch{"1H8A JOS"}="H8A1 JOS"; $resn_exch{"2H8A JOS"}="H8A2 JOS";
$resn_exch{"3H8A JOS"}="H8A3 JOS"; $resn_exch{"1H23 JP1"}="H231 JP1"; $resn_exch{"2H23 JP1"}="H232 JP1";
$resn_exch{"1H25 JP1"}="H251 JP1"; $resn_exch{"2H25 JP1"}="H252 JP1"; $resn_exch{"1H26 JP1"}="H261 JP1";
$resn_exch{"2H26 JP1"}="H262 JP1"; $resn_exch{"1H22 JP1"}="H221 JP1"; $resn_exch{"2H22 JP1"}="H222 JP1";
$resn_exch{"1H23 JPA"}="H231 JPA"; $resn_exch{"2H23 JPA"}="H232 JPA"; $resn_exch{"3H23 JPA"}="H233 JPA";
$resn_exch{"1H3  JPC"}=" H31 JPC"; $resn_exch{"2H3  JPC"}=" H32 JPC"; $resn_exch{"3H3  JPC"}=" H33 JPC";
$resn_exch{"1H4  JPC"}=" H41 JPC"; $resn_exch{"2H4  JPC"}=" H42 JPC"; $resn_exch{"3H4  JPC"}=" H43 JPC";
$resn_exch{"1H7  JPJ"}=" H71 JPJ"; $resn_exch{"2H7  JPJ"}=" H72 JPJ"; $resn_exch{"1H18 JPJ"}="H181 JPJ";
$resn_exch{"2H18 JPJ"}="H182 JPJ"; $resn_exch{"1H9  JPN"}=" H91 JPN"; $resn_exch{"2H9  JPN"}=" H92 JPN";
$resn_exch{"1H8  JPN"}=" H81 JPN"; $resn_exch{"2H8  JPN"}=" H82 JPN"; $resn_exch{"1H7  JPN"}=" H71 JPN";
$resn_exch{"2H7  JPN"}=" H72 JPN"; $resn_exch{"1H1  JPR"}=" H11 JPR"; $resn_exch{"2H1  JPR"}=" H12 JPR";
$resn_exch{"3H1  JPR"}="H13A JPR"; $resn_exch{"1H9  JPR"}=" H91 JPR"; $resn_exch{"2H9  JPR"}=" H92 JPR";
$resn_exch{"1H10 JPR"}="H101 JPR"; $resn_exch{"2H10 JPR"}="H102 JPR"; $resn_exch{"1H11 JPR"}="H111 JPR";
$resn_exch{"2H11 JPR"}="H112 JPR"; $resn_exch{"1H16 JPR"}="H161 JPR"; $resn_exch{"2H16 JPR"}="H162 JPR";
$resn_exch{"1H17 JPR"}="H171 JPR"; $resn_exch{"2H17 JPR"}="H172 JPR"; $resn_exch{"3H17 JPR"}="H173 JPR";
$resn_exch{"1H18 JPR"}="H181 JPR"; $resn_exch{"2H18 JPR"}="H182 JPR"; $resn_exch{"3H18 JPR"}="H183 JPR";
$resn_exch{"1H21 JS4"}="H211 JS4"; $resn_exch{"2H21 JS4"}="H212 JS4"; $resn_exch{"1H61 JS4"}="H611 JS4";
$resn_exch{"2H61 JS4"}="H612 JS4"; $resn_exch{"1H12 JS4"}="H121 JS4"; $resn_exch{"2H12 JS4"}="H122 JS4";
$resn_exch{"1H22 JS4"}="H221 JS4"; $resn_exch{"2H22 JS4"}="H222 JS4"; $resn_exch{"1H32 JS4"}="H321 JS4";
$resn_exch{"2H32 JS4"}="H322 JS4"; $resn_exch{"1H53 JS4"}="H531 JS4"; $resn_exch{"2H53 JS4"}="H532 JS4";
$resn_exch{"1H63 JS4"}="H631 JS4"; $resn_exch{"2H63 JS4"}="H632 JS4"; $resn_exch{"1H73 JS4"}="H731 JS4";
$resn_exch{"2H73 JS4"}="H732 JS4"; $resn_exch{"1H83 JS4"}="H831 JS4"; $resn_exch{"2H83 JS4"}="H832 JS4";
$resn_exch{"1H15 JS4"}="H151 JS4"; $resn_exch{"2H15 JS4"}="H152 JS4"; $resn_exch{"1H1  JS4"}="H11A JS4";
$resn_exch{"2H1  JS4"}="H12A JS4"; $resn_exch{"1H25 JS4"}="H251 JS4"; $resn_exch{"2H25 JS4"}="H252 JS4";
$resn_exch{"1H24 JS4"}="H241 JS4"; $resn_exch{"2H24 JS4"}="H242 JS4"; $resn_exch{"1H64 JS4"}="H641 JS4";
$resn_exch{"2H64 JS4"}="H642 JS4"; $resn_exch{"1H2  JS4"}="H21A JS4"; $resn_exch{"2H2  JS4"}=" H22 JS4";
$resn_exch{"1H21 JS5"}="H211 JS5"; $resn_exch{"2H21 JS5"}="H212 JS5"; $resn_exch{"1H61 JS5"}="H611 JS5";
$resn_exch{"2H61 JS5"}="H612 JS5"; $resn_exch{"1H12 JS5"}="H121 JS5"; $resn_exch{"2H12 JS5"}="H122 JS5";
$resn_exch{"1H22 JS5"}="H221 JS5"; $resn_exch{"2H22 JS5"}="H222 JS5"; $resn_exch{"1H32 JS5"}="H321 JS5";
$resn_exch{"2H32 JS5"}="H322 JS5"; $resn_exch{"1H53 JS5"}="H531 JS5"; $resn_exch{"2H53 JS5"}="H532 JS5";
$resn_exch{"1H63 JS5"}="H631 JS5"; $resn_exch{"2H63 JS5"}="H632 JS5"; $resn_exch{"1H73 JS5"}="H731 JS5";
$resn_exch{"2H73 JS5"}="H732 JS5"; $resn_exch{"1H83 JS5"}="H831 JS5"; $resn_exch{"2H83 JS5"}="H832 JS5";
$resn_exch{"1H25 JS5"}="H251 JS5"; $resn_exch{"2H25 JS5"}="H252 JS5"; $resn_exch{"1H35 JS5"}="H351 JS5";
$resn_exch{"2H35 JS5"}="H352 JS5"; $resn_exch{"1H45 JS5"}="H451 JS5"; $resn_exch{"2H45 JS5"}="H452 JS5";
$resn_exch{"1H55 JS5"}="H551 JS5"; $resn_exch{"2H55 JS5"}="H552 JS5"; $resn_exch{"1H24 JS5"}="H241 JS5";
$resn_exch{"2H24 JS5"}="H242 JS5"; $resn_exch{"1H1  JS5"}="H11A JS5"; $resn_exch{"2H1  JS5"}="H12A JS5";
$resn_exch{"1H64 JS5"}="H641 JS5"; $resn_exch{"2H64 JS5"}="H642 JS5"; $resn_exch{"2H8  JST"}=" H82 JST";
$resn_exch{"1H8  JST"}=" H81 JST"; $resn_exch{"1H10 JST"}="H101 JST"; $resn_exch{"2H10 JST"}="H102 JST";
$resn_exch{"1H11 JST"}="H111 JST"; $resn_exch{"2H11 JST"}="H112 JST"; $resn_exch{"1H13 JST"}="H131 JST";
$resn_exch{"2H13 JST"}="H132 JST"; $resn_exch{"1H1  JTP"}=" H11 JTP"; $resn_exch{"2H1  JTP"}=" H12 JTP";
$resn_exch{"3H1  JTP"}=" H13 JTP"; $resn_exch{"1H33 JTP"}="H331 JTP"; $resn_exch{"2H33 JTP"}="H332 JTP";
$resn_exch{"3H33 JTP"}="H333 JTP"; $resn_exch{"1H20 JTP"}="H201 JTP"; $resn_exch{"2H20 JTP"}="H202 JTP";
$resn_exch{"1H19 JTP"}="H191 JTP"; $resn_exch{"2H19 JTP"}="H192 JTP"; $resn_exch{"1H27 JTP"}="H271 JTP";
$resn_exch{"2H27 JTP"}="H272 JTP"; $resn_exch{"1H26 JTP"}="H261 JTP"; $resn_exch{"2H26 JTP"}="H262 JTP";
$resn_exch{"1H25 JTP"}="H251 JTP"; $resn_exch{"2H25 JTP"}="H252 JTP"; $resn_exch{"1H24 JTP"}="H241 JTP";
$resn_exch{"2H24 JTP"}="H242 JTP"; $resn_exch{"1H23 JTP"}="H231 JTP"; $resn_exch{"2H23 JTP"}="H232 JTP";
$resn_exch{"1H7  JW5"}=" H71 JW5"; $resn_exch{"2H7  JW5"}=" H72 JW5"; $resn_exch{" H1* JW5"}=" H1' JW5";
$resn_exch{" H2* JW5"}=" H2' JW5"; $resn_exch{"2HO* JW5"}="HO'2 JW5"; $resn_exch{" H3* JW5"}=" H3' JW5";
$resn_exch{" H4* JW5"}=" H4' JW5"; $resn_exch{"1H5* JW5"}="H5'1 JW5"; $resn_exch{"2H5* JW5"}="H5'2 JW5";
$resn_exch{"3HOP JW5"}="HOP3 JW5"; $resn_exch{"2HOP JW5"}="HOP2 JW5"; $resn_exch{"1HO2  JY"}="HO21  JY";
$resn_exch{"1H1   JY"}=" H11  JY"; $resn_exch{"2H1   JY"}=" H12  JY"; $resn_exch{"1H2   JY"}=" H21  JY";
$resn_exch{"2H2   JY"}=" H22  JY"; $resn_exch{"1H3   JY"}=" H31  JY"; $resn_exch{"1H4   JY"}=" H41  JY";
$resn_exch{"2H4   JY"}=" H42  JY"; $resn_exch{"1H6   JY"}=" H61  JY"; $resn_exch{"1H7   JY"}=" H71  JY";
$resn_exch{"1H9   JY"}=" H91  JY"; $resn_exch{"2H9   JY"}=" H92  JY"; $resn_exch{"1H11  JY"}="H111  JY";
$resn_exch{"2H11  JY"}="H112  JY"; $resn_exch{"1H12  JY"}="H121  JY"; $resn_exch{"2H12  JY"}="H122  JY";
$resn_exch{"1H14  JY"}="H141  JY"; $resn_exch{"1H15  JY"}="H151  JY"; $resn_exch{"2H15  JY"}="H152  JY";
$resn_exch{"1H16  JY"}="H161  JY"; $resn_exch{"2H16  JY"}="H162  JY"; $resn_exch{"1H17  JY"}="H171  JY";
$resn_exch{"1H18  JY"}="H181  JY"; $resn_exch{"2H18  JY"}="H182  JY"; $resn_exch{"3H18  JY"}="H183  JY";
$resn_exch{"1H19  JY"}="H191  JY"; $resn_exch{"2H19  JY"}="H192  JY"; $resn_exch{"1H20  JY"}="H201  JY";
$resn_exch{"1H21  JY"}="H211  JY"; $resn_exch{"2H21  JY"}="H212  JY"; $resn_exch{"3H21  JY"}="H213  JY";
$resn_exch{"1H22  JY"}="H221  JY"; $resn_exch{"2H22  JY"}="H222  JY"; $resn_exch{"1H23  JY"}="H231  JY";
$resn_exch{"1H24  JY"}="H241  JY"; $resn_exch{"1H26  JY"}="H261  JY"; $resn_exch{"1H28  JY"}="H281  JY";
$resn_exch{"1HO1  JY"}="HO11  JY"; $resn_exch{"1H11 K01"}="H111 K01"; $resn_exch{"2H11 K01"}="H112 K01";
$resn_exch{"1H10 K01"}="H101 K01"; $resn_exch{"2H10 K01"}="H102 K01"; $resn_exch{"1H9  K01"}=" H91 K01";
$resn_exch{"2H9  K01"}=" H92 K01"; $resn_exch{"1H8  K01"}=" H81 K01"; $resn_exch{"2H8  K01"}=" H82 K01";
$resn_exch{"1H2  K01"}=" H21 K01"; $resn_exch{"2H2  K01"}=" H22 K01"; $resn_exch{"1H1  K01"}=" H11 K01";
$resn_exch{"2H1  K01"}=" H12 K01"; $resn_exch{"3H1  K01"}=" H13 K01"; $resn_exch{"1H23 K01"}="H231 K01";
$resn_exch{"2H23 K01"}="H232 K01"; $resn_exch{"1H24 K01"}="H241 K01"; $resn_exch{"2H24 K01"}="H242 K01";
$resn_exch{"3H24 K01"}="H243 K01"; $resn_exch{"1H7C K02"}="H7C1 K02"; $resn_exch{"2H7C K02"}="H7C2 K02";
$resn_exch{"1H11 K02"}="H111 K02"; $resn_exch{"2H11 K02"}="H112 K02"; $resn_exch{"1H10 K02"}="H101 K02";
$resn_exch{"2H10 K02"}="H102 K02"; $resn_exch{"3H10 K02"}="H103 K02"; $resn_exch{"1H8C K02"}="H8C1 K02";
$resn_exch{"2H8C K02"}="H8C2 K02"; $resn_exch{"1H2C K02"}="H2C1 K02"; $resn_exch{"2H2C K02"}="H2C2 K02";
$resn_exch{"1H18 K02"}="H181 K02"; $resn_exch{"2H18 K02"}="H182 K02"; $resn_exch{"1H1C K02"}="H1C1 K02";
$resn_exch{"2H1C K02"}="H1C2 K02"; $resn_exch{"3H1C K02"}="H1C3 K02"; $resn_exch{"1H19 K02"}="H191 K02";
$resn_exch{"2H19 K02"}="H192 K02"; $resn_exch{"3H19 K02"}="H193 K02"; $resn_exch{"1H20 K03"}="H201 K03";
$resn_exch{"2H20 K03"}="H202 K03"; $resn_exch{"1H25 K03"}="H251 K03"; $resn_exch{"2H25 K03"}="H252 K03";
$resn_exch{"1H24 K03"}="H241 K03"; $resn_exch{"2H24 K03"}="H242 K03"; $resn_exch{"3H24 K03"}="H243 K03";
$resn_exch{"1H23 K03"}="H231 K03"; $resn_exch{"2H23 K03"}="H232 K03"; $resn_exch{"3H23 K03"}="H233 K03";
$resn_exch{"1H21 K03"}="H211 K03"; $resn_exch{"2H21 K03"}="H212 K03"; $resn_exch{"1H4C K03"}="H4C1 K03";
$resn_exch{"2H4C K03"}="H4C2 K03"; $resn_exch{"1H2C K03"}="H2C1 K03"; $resn_exch{"2H2C K03"}="H2C2 K03";
$resn_exch{"1H5C K03"}="H5C1 K03"; $resn_exch{"2H5C K03"}="H5C2 K03"; $resn_exch{"3H5C K03"}="H5C3 K03";
$resn_exch{"1H1C K03"}="H1C1 K03"; $resn_exch{"2H1C K03"}="H1C2 K03"; $resn_exch{"3H1C K03"}="H1C3 K03";
$resn_exch{" H2* K05"}=" H2' K05"; $resn_exch{" H5* K05"}=" H5' K05"; $resn_exch{" H6* K05"}=" H6' K05";
$resn_exch{"2H11 K05"}="H112 K05"; $resn_exch{"1H10 K05"}="H101 K05"; $resn_exch{"2H10 K05"}="H102 K05";
$resn_exch{"3H10 K05"}="H103 K05"; $resn_exch{"1HN1 K10"}="HN11 K10"; $resn_exch{"2HN1 K10"}="HN12 K10";
$resn_exch{"1H7  K10"}=" H71 K10"; $resn_exch{"2H7  K10"}=" H72 K10"; $resn_exch{"3H7  K10"}=" H73 K10";
$resn_exch{"1H8  K10"}=" H81 K10"; $resn_exch{"2H8  K10"}=" H82 K10"; $resn_exch{"3H8  K10"}=" H83 K10";
$resn_exch{"1H9  K10"}=" H91 K10"; $resn_exch{"2H9  K10"}=" H92 K10"; $resn_exch{"3H9  K10"}=" H93 K10";
$resn_exch{"1H10 K10"}="H101 K10"; $resn_exch{"2H10 K10"}="H102 K10"; $resn_exch{"1H17 K10"}="H171 K10";
$resn_exch{"2H17 K10"}="H172 K10"; $resn_exch{"3H17 K10"}="H173 K10"; $resn_exch{"1H18 K10"}="H181 K10";
$resn_exch{"2H18 K10"}="H182 K10"; $resn_exch{"3H18 K10"}="H183 K10"; $resn_exch{"1H1N K1R"}="H1N1 K1R";
$resn_exch{"2H1N K1R"}="H1N2 K1R"; $resn_exch{"1HB1 K1R"}="HB11 K1R"; $resn_exch{"2HB1 K1R"}="HB12 K1R";
$resn_exch{"1HG1 K1R"}="HG11 K1R"; $resn_exch{"2HG1 K1R"}="HG12 K1R"; $resn_exch{"1H2N K1R"}="H2N1 K1R";
$resn_exch{"2H2N K1R"}="H2N2 K1R"; $resn_exch{"1HB2 K1R"}="HB21 K1R"; $resn_exch{"2HB2 K1R"}="HB22 K1R";
$resn_exch{"1H10 K21"}="H101 K21"; $resn_exch{"2H10 K21"}="H102 K21"; $resn_exch{"1H12 K21"}="H121 K21";
$resn_exch{"2H12 K21"}="H122 K21"; $resn_exch{"1H13 K21"}="H131 K21"; $resn_exch{"2H13 K21"}="H132 K21";
$resn_exch{"1H22 K21"}="H221 K21"; $resn_exch{"2H22 K21"}="H222 K21"; $resn_exch{"1H24 K21"}="H241 K21";
$resn_exch{"2H24 K21"}="H242 K21"; $resn_exch{"1H30 K21"}="H301 K21"; $resn_exch{"2H30 K21"}="H302 K21";
$resn_exch{"1H31 K21"}="H311 K21"; $resn_exch{"2H31 K21"}="H312 K21"; $resn_exch{"1H33 K21"}="H331 K21";
$resn_exch{"2H33 K21"}="H332 K21"; $resn_exch{"1H34 K21"}="H341 K21"; $resn_exch{"2H34 K21"}="H342 K21";
$resn_exch{"1H43 K21"}="H431 K21"; $resn_exch{"2H43 K21"}="H432 K21"; $resn_exch{"1H59 K21"}="H591 K21";
$resn_exch{"2H59 K21"}="H592 K21"; $resn_exch{"3H59 K21"}="H593 K21"; $resn_exch{"1H16 K25"}="H161 K25";
$resn_exch{"2H16 K25"}="H162 K25"; $resn_exch{"3H16 K25"}="H163 K25"; $resn_exch{"1H17 K25"}="H171 K25";
$resn_exch{"2H17 K25"}="H172 K25"; $resn_exch{"3H17 K25"}="H173 K25"; $resn_exch{"1H7  K2C"}=" H71 K2C";
$resn_exch{"2H7  K2C"}=" H72 K2C"; $resn_exch{"1H11 K37"}="H111 K37"; $resn_exch{"2H11 K37"}="H112 K37";
$resn_exch{"3H11 K37"}="H113 K37"; $resn_exch{"1H11 K44"}="H111 K44"; $resn_exch{"2H11 K44"}="H112 K44";
$resn_exch{"3H11 K44"}="H113 K44"; $resn_exch{"1H12 K44"}="H121 K44"; $resn_exch{"2H12 K44"}="H122 K44";
$resn_exch{"1H14 K44"}="H141 K44"; $resn_exch{"2H14 K44"}="H142 K44"; $resn_exch{"1H15 K51"}="H151 K51";
$resn_exch{"2H15 K51"}="H152 K51"; $resn_exch{"1H17 K51"}="H171 K51"; $resn_exch{"2H17 K51"}="H172 K51";
$resn_exch{"3H17 K51"}="H173 K51"; $resn_exch{"1H18 K51"}="H181 K51"; $resn_exch{"2H18 K51"}="H182 K51";
$resn_exch{"3H18 K51"}="H183 K51"; $resn_exch{"1H6  KAB"}=" H61 KAB"; $resn_exch{"2H6  KAB"}=" H62 KAB";
$resn_exch{"1H36 KAB"}="H361 KAB"; $resn_exch{"2H36 KAB"}="H362 KAB"; $resn_exch{"3H36 KAB"}="H363 KAB";
$resn_exch{"1H4  KAB"}=" H41 KAB"; $resn_exch{"2H4  KAB"}=" H42 KAB"; $resn_exch{"1HN1 KAB"}="HN11 KAB";
$resn_exch{"2HN1 KAB"}="HN12 KAB"; $resn_exch{"1H2  KAB"}=" H21 KAB"; $resn_exch{"2H2  KAB"}="H22A KAB";
$resn_exch{"1H37 KAB"}="H371 KAB"; $resn_exch{"2H37 KAB"}="H372 KAB"; $resn_exch{"3H37 KAB"}="H373 KAB";
$resn_exch{"1H47 KAB"}="H471 KAB"; $resn_exch{"2H47 KAB"}="H472 KAB"; $resn_exch{"3H47 KAB"}="H473 KAB";
$resn_exch{"11H1 KAB"}="H111 KAB"; $resn_exch{"1H21 KAB"}="H211 KAB"; $resn_exch{"2H21 KAB"}="H212 KAB";
$resn_exch{"1H46 KAB"}="H461 KAB"; $resn_exch{"2H46 KAB"}="H462 KAB"; $resn_exch{"3H46 KAB"}="H463 KAB";
$resn_exch{"1H38 KAB"}="H381 KAB"; $resn_exch{"2H38 KAB"}="H382 KAB"; $resn_exch{"3H38 KAB"}="H383 KAB";
$resn_exch{"1H25 KAB"}="H251 KAB"; $resn_exch{"2H25 KAB"}="H252 KAB"; $resn_exch{"1H45 KAB"}="H451 KAB";
$resn_exch{"2H45 KAB"}="H452 KAB"; $resn_exch{"3H45 KAB"}="H453 KAB"; $resn_exch{"1H39 KAB"}="H391 KAB";
$resn_exch{"2H39 KAB"}="H392 KAB"; $resn_exch{"3H39 KAB"}="H393 KAB"; $resn_exch{"1H28 KAB"}="H281 KAB";
$resn_exch{"2H28 KAB"}="H282 KAB"; $resn_exch{"1H29 KAB"}="H291 KAB"; $resn_exch{"2H29 KAB"}="H292 KAB";
$resn_exch{"1H40 KAB"}="H401 KAB"; $resn_exch{"2H40 KAB"}="H402 KAB"; $resn_exch{"3H40 KAB"}="H403 KAB";
$resn_exch{"1H44 KAB"}="H441 KAB"; $resn_exch{"2H44 KAB"}="H442 KAB"; $resn_exch{"3H44 KAB"}="H443 KAB";
$resn_exch{"1H41 KAB"}="H411 KAB"; $resn_exch{"2H41 KAB"}="H412 KAB"; $resn_exch{"3H41 KAB"}="H413 KAB";
$resn_exch{"1H43 KAB"}="H431 KAB"; $resn_exch{"2H43 KAB"}="H432 KAB"; $resn_exch{"3H43 KAB"}="H433 KAB";
$resn_exch{"1H42 KAB"}="H421 KAB"; $resn_exch{"2H42 KAB"}="H422 KAB"; $resn_exch{"1H5* KAG"}=" H5' KAG";
$resn_exch{"2H5* KAG"}="H5'' KAG"; $resn_exch{" H4* KAG"}=" H4' KAG"; $resn_exch{" H3* KAG"}=" H3' KAG";
$resn_exch{"2H2* KAG"}="H2'' KAG"; $resn_exch{"1H2* KAG"}=" H2' KAG"; $resn_exch{" H1* KAG"}=" H1' KAG";
$resn_exch{"1HN5 KAH"}="HN51 KAH"; $resn_exch{"2HN5 KAH"}="HN52 KAH"; $resn_exch{"1HC  KAH"}=" HC1 KAH";
$resn_exch{"2HC  KAH"}=" HC2 KAH"; $resn_exch{"3HC  KAH"}=" HC3 KAH"; $resn_exch{"1HC3 KAH"}="HC31 KAH";
$resn_exch{"2HC3 KAH"}="HC32 KAH"; $resn_exch{"1HC2 KAH"}="HC21 KAH"; $resn_exch{"2HC2 KAH"}="HC22 KAH";
$resn_exch{"1HA2 KAM"}="HA21 KAM"; $resn_exch{"2HA2 KAM"}="HA22 KAM"; $resn_exch{"3HA2 KAM"}="HA23 KAM";
$resn_exch{"1H4A KAM"}="H4A1 KAM"; $resn_exch{"2H4A KAM"}="H4A2 KAM"; $resn_exch{"1HA5 KAM"}="HA51 KAM";
$resn_exch{"2HA5 KAM"}="HA52 KAM"; $resn_exch{"2HOP KAM"}="HOP2 KAM"; $resn_exch{"3HOP KAM"}="HOP3 KAM";
$resn_exch{"9HNC KAM"}="HNC9 KAM"; $resn_exch{"1HC9 KAM"}="HC91 KAM"; $resn_exch{"2HC9 KAM"}="HC92 KAM";
$resn_exch{"3HC9 KAM"}="HC93 KAM"; $resn_exch{"1HC6 KAM"}="HC61 KAM"; $resn_exch{"2HC6 KAM"}="HC62 KAM";
$resn_exch{"1HC5 KAM"}="HC51 KAM"; $resn_exch{"2HC5 KAM"}="HC52 KAM"; $resn_exch{"1HC4 KAM"}="HC41 KAM";
$resn_exch{"2HC4 KAM"}="HC42 KAM"; $resn_exch{"1HC3 KAM"}="HC31 KAM"; $resn_exch{"2HC3 KAM"}="HC32 KAM";
$resn_exch{"1HC2 KAM"}="HC21 KAM"; $resn_exch{"2HC2 KAM"}="HC22 KAM"; $resn_exch{"1HOC KAM"}="HOC1 KAM";
$resn_exch{"1H6  KAN"}=" H61 KAN"; $resn_exch{"2H6  KAN"}=" H62 KAN"; $resn_exch{"1H12 KAN"}="H121 KAN";
$resn_exch{"2H12 KAN"}="H122 KAN"; $resn_exch{"1H18 KAN"}="H181 KAN"; $resn_exch{"2H18 KAN"}="H182 KAN";
$resn_exch{"1HN1 KAN"}="HN11 KAN"; $resn_exch{"2HN1 KAN"}="HN12 KAN"; $resn_exch{"1HN2 KAN"}="HN21 KAN";
$resn_exch{"2HN2 KAN"}="HN22 KAN"; $resn_exch{"1HN3 KAN"}="HN31 KAN"; $resn_exch{"2HN3 KAN"}="HN32 KAN";
$resn_exch{"1HN4 KAN"}="HN41 KAN"; $resn_exch{"2HN4 KAN"}="HN42 KAN"; $resn_exch{"2HA  KAP"}=" HA2 KAP";
$resn_exch{"1HA  KAP"}=" HA1 KAP"; $resn_exch{"2HB  KAP"}=" HB2 KAP"; $resn_exch{"1HB  KAP"}=" HB1 KAP";
$resn_exch{"2HG  KAP"}=" HG2 KAP"; $resn_exch{"1HG  KAP"}=" HG1 KAP"; $resn_exch{"2HD  KAP"}=" HD2 KAP";
$resn_exch{"1HD  KAP"}=" HD1 KAP"; $resn_exch{"2HE  KAP"}=" HE2 KAP"; $resn_exch{"1HE  KAP"}=" HE1 KAP";
$resn_exch{"3HS  KAP"}=" HS3 KAP"; $resn_exch{"2HS  KAP"}=" HS2 KAP"; $resn_exch{"1HS  KAP"}=" HS1 KAP";
$resn_exch{"1H   KAP"}=" H1  KAP"; $resn_exch{"2H   KAP"}=" H2  KAP"; $resn_exch{"1H9  KAR"}=" H91 KAR";
$resn_exch{"2H9  KAR"}=" H92 KAR"; $resn_exch{"3H9  KAR"}=" H93 KAR"; $resn_exch{"1H20 KAR"}="H201 KAR";
$resn_exch{"2H20 KAR"}="H202 KAR"; $resn_exch{"3H20 KAR"}="H203 KAR"; $resn_exch{"1H25 KAR"}="H251 KAR";
$resn_exch{"2H25 KAR"}="H252 KAR"; $resn_exch{"3H25 KAR"}="H253 KAR"; $resn_exch{"1H23 KAR"}="H231 KAR";
$resn_exch{"2H23 KAR"}="H232 KAR"; $resn_exch{"1H34 KAR"}="H341 KAR"; $resn_exch{"2H34 KAR"}="H342 KAR";
$resn_exch{"1H31 KAR"}="H311 KAR"; $resn_exch{"2H31 KAR"}="H312 KAR"; $resn_exch{"1H35 KAR"}="H351 KAR";
$resn_exch{"2H35 KAR"}="H352 KAR"; $resn_exch{"1H36 KAR"}="H361 KAR"; $resn_exch{"2H36 KAR"}="H362 KAR";
$resn_exch{"3H36 KAR"}="H363 KAR"; $resn_exch{"1H28 KAR"}="H281 KAR"; $resn_exch{"2H28 KAR"}="H282 KAR";
$resn_exch{"1H19 KAR"}="H191 KAR"; $resn_exch{"2H19 KAR"}="H192 KAR"; $resn_exch{"1H30 KAR"}="H301 KAR";
$resn_exch{"2H30 KAR"}="H302 KAR"; $resn_exch{"1H13 KAR"}="H131 KAR"; $resn_exch{"2H13 KAR"}="H132 KAR";
$resn_exch{"1H91 KAR"}="H911 KAR"; $resn_exch{"2H91 KAR"}="H912 KAR"; $resn_exch{"1H92 KAR"}="H921 KAR";
$resn_exch{"2H92 KAR"}="H922 KAR"; $resn_exch{"1H94 KAR"}="H941 KAR"; $resn_exch{"2H94 KAR"}="H942 KAR";
$resn_exch{"1H45 KAR"}="H451 KAR"; $resn_exch{"2H45 KAR"}="H452 KAR"; $resn_exch{"3H45 KAR"}="H453 KAR";
$resn_exch{"1H95 KAR"}="H951 KAR"; $resn_exch{"2H95 KAR"}="H952 KAR"; $resn_exch{"1H90 KAR"}="H901 KAR";
$resn_exch{"2H90 KAR"}="H902 KAR"; $resn_exch{"1H89 KAR"}="H891 KAR"; $resn_exch{"2H89 KAR"}="H892 KAR";
$resn_exch{"1HAE KBG"}="HAE1 KBG"; $resn_exch{"2HAE KBG"}="HAE2 KBG"; $resn_exch{"1H2  KCP"}=" H21 KCP";
$resn_exch{"2H2  KCP"}=" H22 KCP"; $resn_exch{"1H3' KCP"}="H3'1 KCP"; $resn_exch{"2H3' KCP"}="H3'2 KCP";
$resn_exch{"3H3' KCP"}="H3'3 KCP"; $resn_exch{"1H13 KCP"}="H131 KCP"; $resn_exch{"2H13 KCP"}="H132 KCP";
$resn_exch{"1HN  KCQ"}=" HN1 KCQ"; $resn_exch{"2HN  KCQ"}=" HN2 KCQ"; $resn_exch{"1HMK KCQ"}="HMK1 KCQ";
$resn_exch{"2HMK KCQ"}="HMK2 KCQ"; $resn_exch{"3HMK KCQ"}="HMK3 KCQ"; $resn_exch{"1HB  KCQ"}=" HB1 KCQ";
$resn_exch{"2HB  KCQ"}=" HB2 KCQ"; $resn_exch{"1HAE KCQ"}="HAE1 KCQ"; $resn_exch{"2HAE KCQ"}="HAE2 KCQ";
$resn_exch{"1HD1 KCQ"}="HD11 KCQ"; $resn_exch{"2HD1 KCQ"}="HD12 KCQ"; $resn_exch{"1HN  KCX"}=" H   KCX";
$resn_exch{"2HN  KCX"}=" HN2 KCX"; $resn_exch{"1HB  KCX"}=" HB2 KCX"; $resn_exch{"2HB  KCX"}=" HB3 KCX";
$resn_exch{"1HG  KCX"}=" HG2 KCX"; $resn_exch{"2HG  KCX"}=" HG3 KCX"; $resn_exch{"1HD  KCX"}=" HD2 KCX";
$resn_exch{"2HD  KCX"}=" HD3 KCX"; $resn_exch{"1HE  KCX"}=" HE2 KCX"; $resn_exch{"2HE  KCX"}=" HE3 KCX";
$resn_exch{"1HZ  KCX"}=" HZ  KCX"; $resn_exch{" HX2 KCX"}=" HQ2 KCX"; $resn_exch{"1H3  KDA"}=" H31 KDA";
$resn_exch{"2H3  KDA"}=" H32 KDA"; $resn_exch{"1H8  KDA"}=" H81 KDA"; $resn_exch{"2H8  KDA"}=" H82 KDA";
$resn_exch{"1H9  KDA"}=" H91 KDA"; $resn_exch{"2H9  KDA"}=" H92 KDA"; $resn_exch{"1H11 KDA"}="H111 KDA";
$resn_exch{"2H11 KDA"}="H112 KDA"; $resn_exch{"1H1C KDF"}="H1C1 KDF"; $resn_exch{"2H1C KDF"}="H1C2 KDF";
$resn_exch{"1H4C KDF"}="H4C1 KDF"; $resn_exch{"2H4C KDF"}="H4C2 KDF"; $resn_exch{"1H1C KDG"}="H1C1 KDG";
$resn_exch{"2H1C KDG"}="H1C2 KDG"; $resn_exch{"1H4C KDG"}="H4C1 KDG"; $resn_exch{"2H4C KDG"}="H4C2 KDG";
$resn_exch{"1H3  KDO"}=" H31 KDO"; $resn_exch{"2H3  KDO"}=" H32 KDO"; $resn_exch{"1H8  KDO"}=" H81 KDO";
$resn_exch{"2H8  KDO"}=" H82 KDO"; $resn_exch{"1HAI KDP"}="HAI1 KDP"; $resn_exch{"2HAI KDP"}="HAI2 KDP";
$resn_exch{"1HAJ KDP"}="HAJ1 KDP"; $resn_exch{"2HAJ KDP"}="HAJ2 KDP"; $resn_exch{"1H5A KE4"}="H5A1 KE4";
$resn_exch{"2H5A KE4"}="H5A2 KE4"; $resn_exch{"1H2A KE4"}="H2A1 KE4"; $resn_exch{"2H2A KE4"}="H2A2 KE4";
$resn_exch{"3H2A KE4"}="H2A3 KE4"; $resn_exch{"1H4A KE4"}="H4A1 KE4"; $resn_exch{"2H4A KE4"}="H4A2 KE4";
$resn_exch{"1H   KE4"}=" H1  KE4"; $resn_exch{"2H   KE4"}=" H2  KE4"; $resn_exch{"1HN5 KE4"}="HN51 KE4";
$resn_exch{"2HN5 KE4"}="HN52 KE4"; $resn_exch{"1HB  KE4"}=" HB1 KE4"; $resn_exch{"2HB  KE4"}=" HB2 KE4";
$resn_exch{"1HG  KE4"}=" HG1 KE4"; $resn_exch{"2HG  KE4"}=" HG2 KE4"; $resn_exch{"3H18 KEF"}="H183 KEF";
$resn_exch{"2H18 KEF"}="H182 KEF"; $resn_exch{"1H18 KEF"}="H181 KEF"; $resn_exch{"2H3' KEF"}="H3'2 KEF";
$resn_exch{"1H3' KEF"}="H3'1 KEF"; $resn_exch{"1HN3 KEF"}="HN31 KEF"; $resn_exch{"2HN3 KEF"}="HN32 KEF";
$resn_exch{"1H2A KET"}="H2A1 KET"; $resn_exch{"2H2A KET"}="H2A2 KET"; $resn_exch{"3H2A KET"}="H2A3 KET";
$resn_exch{"1H5A KET"}="H5A1 KET"; $resn_exch{"2H5A KET"}="H5A2 KET"; $resn_exch{"2HOP KET"}="HOP2 KET";
$resn_exch{"3HOP KET"}="HOP3 KET"; $resn_exch{"1HB  KET"}=" HB1 KET"; $resn_exch{"2HB  KET"}=" HB2 KET";
$resn_exch{"1H20 KEU"}="H201 KEU"; $resn_exch{"2H20 KEU"}="H202 KEU"; $resn_exch{"1H19 KEU"}="H191 KEU";
$resn_exch{"2H19 KEU"}="H192 KEU"; $resn_exch{"1H1  KEU"}=" H11 KEU"; $resn_exch{"2H1  KEU"}="H12A KEU";
$resn_exch{"1H2  KEU"}=" H21 KEU"; $resn_exch{"2H2  KEU"}=" H22 KEU"; $resn_exch{"1H3  KEU"}=" H31 KEU";
$resn_exch{"2H3  KEU"}=" H32 KEU"; $resn_exch{"1HN2 KEU"}="HN21 KEU"; $resn_exch{"2HN2 KEU"}="HN22 KEU";
$resn_exch{"1HN6 KEU"}="HN61 KEU"; $resn_exch{"2HN6 KEU"}="HN62 KEU"; $resn_exch{"1HN  KGC"}=" H   KGC";
$resn_exch{"2HN  KGC"}=" H2  KGC"; $resn_exch{"1HB  KGC"}=" HB2 KGC"; $resn_exch{"2HB  KGC"}=" HB3 KGC";
$resn_exch{"1HG  KGC"}=" HG2 KGC"; $resn_exch{"2HG  KGC"}=" HG3 KGC"; $resn_exch{"1HD  KGC"}=" HD2 KGC";
$resn_exch{"2HD  KGC"}=" HD3 KGC"; $resn_exch{"1HE  KGC"}=" HE2 KGC"; $resn_exch{"2HE  KGC"}=" HE3 KGC";
$resn_exch{" HNZ KGC"}=" HZ  KGC"; $resn_exch{"1H27 KGC"}="H271 KGC"; $resn_exch{"2H27 KGC"}="H272 KGC";
$resn_exch{"1H12 KGC"}="H121 KGC"; $resn_exch{"2H12 KGC"}="H122 KGC"; $resn_exch{"1H8  KGQ"}=" H81 KGQ";
$resn_exch{"2H8  KGQ"}=" H82 KGQ"; $resn_exch{"1H15 KGQ"}="H151 KGQ"; $resn_exch{"2H15 KGQ"}="H152 KGQ";
$resn_exch{"1H18 KGQ"}="H181 KGQ"; $resn_exch{"2H18 KGQ"}="H182 KGQ"; $resn_exch{"1H19 KGQ"}="H191 KGQ";
$resn_exch{"2H19 KGQ"}="H192 KGQ"; $resn_exch{"3H19 KGQ"}="H193 KGQ"; $resn_exch{"1H16 KGQ"}="H161 KGQ";
$resn_exch{"2H16 KGQ"}="H162 KGQ"; $resn_exch{"1H17 KGQ"}="H171 KGQ"; $resn_exch{"2H17 KGQ"}="H172 KGQ";
$resn_exch{"3H17 KGQ"}="H173 KGQ"; $resn_exch{"1H20 KGQ"}="H201 KGQ"; $resn_exch{"2H20 KGQ"}="H202 KGQ";
$resn_exch{"1H21 KGQ"}="H211 KGQ"; $resn_exch{"2H21 KGQ"}="H212 KGQ"; $resn_exch{"1H30 KGQ"}="H301 KGQ";
$resn_exch{"2H30 KGQ"}="H302 KGQ"; $resn_exch{"3H30 KGQ"}="H303 KGQ"; $resn_exch{"1H29 KGQ"}="H291 KGQ";
$resn_exch{"2H29 KGQ"}="H292 KGQ"; $resn_exch{"3H29 KGQ"}="H293 KGQ"; $resn_exch{"1H31 KGQ"}="H311 KGQ";
$resn_exch{"2H31 KGQ"}="H312 KGQ"; $resn_exch{"3H31 KGQ"}="H313 KGQ"; $resn_exch{"1H3  KGQ"}=" H31 KGQ";
$resn_exch{"2H3  KGQ"}="H32A KGQ"; $resn_exch{"1H2  KGQ"}=" H21 KGQ"; $resn_exch{"2H2  KGQ"}="H22A KGQ";
$resn_exch{"1H1  KGQ"}="H11A KGQ"; $resn_exch{"2H1  KGQ"}="H12A KGQ"; $resn_exch{"1H2  KGR"}=" H21 KGR";
$resn_exch{"1HO2 KGR"}="HO21 KGR"; $resn_exch{"1H3  KGR"}=" H31 KGR"; $resn_exch{"1HO3 KGR"}="HO31 KGR";
$resn_exch{"1H4  KGR"}=" H41 KGR"; $resn_exch{"2H4  KGR"}=" H42 KGR"; $resn_exch{"1H8  KGY"}=" H81 KGY";
$resn_exch{"2H8  KGY"}=" H82 KGY"; $resn_exch{"3H8  KGY"}=" H83 KGY"; $resn_exch{"1H10 KGY"}="H101 KGY";
$resn_exch{"2H10 KGY"}="H102 KGY"; $resn_exch{"3H10 KGY"}="H103 KGY"; $resn_exch{"1H12 KGY"}="H121 KGY";
$resn_exch{"2H12 KGY"}="H122 KGY"; $resn_exch{"1H13 KGY"}="H131 KGY"; $resn_exch{"2H13 KGY"}="H132 KGY";
$resn_exch{"1H15 KGY"}="H151 KGY"; $resn_exch{"2H15 KGY"}="H152 KGY"; $resn_exch{"1H16 KGY"}="H161 KGY";
$resn_exch{"2H16 KGY"}="H162 KGY"; $resn_exch{"1H18 KGY"}="H181 KGY"; $resn_exch{"2H18 KGY"}="H182 KGY";
$resn_exch{"1H31 KGY"}="H311 KGY"; $resn_exch{"2H31 KGY"}="H312 KGY"; $resn_exch{"1H34 KGY"}="H341 KGY";
$resn_exch{"2H34 KGY"}="H342 KGY"; $resn_exch{"3H34 KGY"}="H343 KGY"; $resn_exch{"1HC2 KH1"}="HC21 KH1";
$resn_exch{"2HC2 KH1"}="HC22 KH1"; $resn_exch{"1HC4 KH1"}="HC41 KH1"; $resn_exch{"2HC4 KH1"}="HC42 KH1";
$resn_exch{"1HC9 KH1"}="HC91 KH1"; $resn_exch{"2HC9 KH1"}="HC92 KH1"; $resn_exch{"1H11 KH1"}="H111 KH1";
$resn_exch{"2H11 KH1"}="H112 KH1"; $resn_exch{"1H12 KH1"}="H121 KH1"; $resn_exch{"2H12 KH1"}="H122 KH1";
$resn_exch{"1H15 KH1"}="H151 KH1"; $resn_exch{"2H15 KH1"}="H152 KH1"; $resn_exch{"1H16 KH1"}="H161 KH1";
$resn_exch{"2H16 KH1"}="H162 KH1"; $resn_exch{"1H18 KH1"}="H181 KH1"; $resn_exch{"2H18 KH1"}="H182 KH1";
$resn_exch{"3H18 KH1"}="H183 KH1"; $resn_exch{"1H19 KH1"}="H191 KH1"; $resn_exch{"2H19 KH1"}="H192 KH1";
$resn_exch{"1H22 KH1"}="H221 KH1"; $resn_exch{"2H22 KH1"}="H222 KH1"; $resn_exch{"1H23 KH1"}="H231 KH1";
$resn_exch{"2H23 KH1"}="H232 KH1"; $resn_exch{"1H26 KH1"}="H261 KH1"; $resn_exch{"2H26 KH1"}="H262 KH1";
$resn_exch{"1H27 KH1"}="H271 KH1"; $resn_exch{"2H27 KH1"}="H272 KH1"; $resn_exch{"1H21 KH1"}="H211 KH1";
$resn_exch{"2H21 KH1"}="H212 KH1"; $resn_exch{"3H21 KH1"}="H213 KH1"; $resn_exch{"1H24 KH1"}="H241 KH1";
$resn_exch{"2H24 KH1"}="H242 KH1"; $resn_exch{"1H28 KH1"}="H281 KH1"; $resn_exch{"2H28 KH1"}="H282 KH1";
$resn_exch{"3H28 KH1"}="H283 KH1"; $resn_exch{"1H29 KH1"}="H291 KH1"; $resn_exch{"2H29 KH1"}="H292 KH1";
$resn_exch{"3H29 KH1"}="H293 KH1"; $resn_exch{"1H9C KHA"}="H9C1 KHA"; $resn_exch{"2H9C KHA"}="H9C2 KHA";
$resn_exch{"1H10 KHA"}="H101 KHA"; $resn_exch{"2H10 KHA"}="H102 KHA"; $resn_exch{"1H7C KHA"}="H7C1 KHA";
$resn_exch{"2H7C KHA"}="H7C2 KHA"; $resn_exch{"1H11 KHA"}="H111 KHA"; $resn_exch{"2H11 KHA"}="H112 KHA";
$resn_exch{"3H11 KHA"}="H113 KHA"; $resn_exch{"1H16 KHA"}="H161 KHA"; $resn_exch{"2H16 KHA"}="H162 KHA";
$resn_exch{"1H15 KHA"}="H151 KHA"; $resn_exch{"2H15 KHA"}="H152 KHA"; $resn_exch{"1H13 KHA"}="H131 KHA";
$resn_exch{"2H13 KHA"}="H132 KHA"; $resn_exch{"1H14 KHA"}="H141 KHA"; $resn_exch{"2H14 KHA"}="H142 KHA";
$resn_exch{"1H12 KHA"}="H121 KHA"; $resn_exch{"2H12 KHA"}="H122 KHA"; $resn_exch{" H1* KHO"}=" H1' KHO";
$resn_exch{" H2* KHO"}=" H2' KHO"; $resn_exch{" H3* KHO"}=" H3' KHO"; $resn_exch{" H4* KHO"}=" H4' KHO";
$resn_exch{"1H5* KHO"}="H5'1 KHO"; $resn_exch{"2H5* KHO"}="H5'2 KHO"; $resn_exch{" H5* KHO"}=" H5' KHO";
$resn_exch{"1H5  KHO"}=" H51 KHO"; $resn_exch{"2H5  KHO"}=" H52 KHO"; $resn_exch{" H5* KHP"}=" H5B KHP";
$resn_exch{"1H5* KHP"}="H5'1 KHP"; $resn_exch{"2H5* KHP"}="H5'2 KHP"; $resn_exch{" H4* KHP"}=" H4' KHP";
$resn_exch{" H1* KHP"}=" H1' KHP"; $resn_exch{" H2* KHP"}=" H2B KHP"; $resn_exch{" H3* KHP"}=" H3B KHP";
$resn_exch{"1H12 KI2"}="H121 KI2"; $resn_exch{"2H12 KI2"}="H122 KI2"; $resn_exch{"1H19 KI2"}="H191 KI2";
$resn_exch{"2H19 KI2"}="H192 KI2"; $resn_exch{"1H6  KIF"}=" H61 KIF"; $resn_exch{"2H6  KIF"}=" H62 KIF";
$resn_exch{"1H8  KIQ"}=" H81 KIQ"; $resn_exch{"2H8  KIQ"}=" H82 KIQ"; $resn_exch{"1H21 KIQ"}="H211 KIQ";
$resn_exch{"2H21 KIQ"}="H212 KIQ"; $resn_exch{"1HN  KIQ"}=" HN1 KIQ"; $resn_exch{"2HN  KIQ"}=" HN2 KIQ";
$resn_exch{"1H3  KIQ"}=" H31 KIQ"; $resn_exch{"2H3  KIQ"}=" H32 KIQ"; $resn_exch{"1H12 KIQ"}="H121 KIQ";
$resn_exch{"2H12 KIQ"}="H122 KIQ"; $resn_exch{"1H9  KIQ"}=" H91 KIQ"; $resn_exch{"2H9  KIQ"}=" H92 KIQ";
$resn_exch{"1H5  KIR"}=" H51 KIR"; $resn_exch{"1H6  KIR"}=" H61 KIR"; $resn_exch{"1H20 KIR"}="H201 KIR";
$resn_exch{"1H24 KIR"}="H241 KIR"; $resn_exch{"1H25 KIR"}="H251 KIR"; $resn_exch{"2H25 KIR"}="H252 KIR";
$resn_exch{"1H4  KIR"}=" H41 KIR"; $resn_exch{"1H9  KIR"}=" H91 KIR"; $resn_exch{"1H10 KIR"}="H101 KIR";
$resn_exch{"1H11 KIR"}="H111 KIR"; $resn_exch{"1H12 KIR"}="H121 KIR"; $resn_exch{"1H13 KIR"}="H131 KIR";
$resn_exch{"1H14 KIR"}="H141 KIR"; $resn_exch{"1H15 KIR"}="H151 KIR"; $resn_exch{"1H16 KIR"}="H161 KIR";
$resn_exch{"1H17 KIR"}="H171 KIR"; $resn_exch{"1H19 KIR"}="H191 KIR"; $resn_exch{"1H22 KIR"}="H221 KIR";
$resn_exch{"1H23 KIR"}="H231 KIR"; $resn_exch{"1HO5 KIR"}="HO51 KIR"; $resn_exch{"1H1  KIR"}=" H11 KIR";
$resn_exch{"1H41 KIR"}="H411 KIR"; $resn_exch{"2H41 KIR"}="H412 KIR"; $resn_exch{"3H41 KIR"}="H413 KIR";
$resn_exch{"1HO6 KIR"}="HO61 KIR"; $resn_exch{"1H42 KIR"}="H421 KIR"; $resn_exch{"2H42 KIR"}="H422 KIR";
$resn_exch{"3H42 KIR"}="H423 KIR"; $resn_exch{"1H43 KIR"}="H431 KIR"; $resn_exch{"2H43 KIR"}="H432 KIR";
$resn_exch{"3H43 KIR"}="H433 KIR"; $resn_exch{"1H44 KIR"}="H441 KIR"; $resn_exch{"2H44 KIR"}="H442 KIR";
$resn_exch{"3H44 KIR"}="H443 KIR"; $resn_exch{"1H26 KIR"}="H261 KIR"; $resn_exch{"1H45 KIR"}="H451 KIR";
$resn_exch{"2H45 KIR"}="H452 KIR"; $resn_exch{"1H46 KIR"}="H461 KIR"; $resn_exch{"2H46 KIR"}="H462 KIR";
$resn_exch{"3H46 KIR"}="H463 KIR"; $resn_exch{"1H28 KIR"}="H281 KIR"; $resn_exch{"1H29 KIR"}="H291 KIR";
$resn_exch{"1H30 KIR"}="H301 KIR"; $resn_exch{"1HO0 KIR"}="HO01 KIR"; $resn_exch{"1H31 KIR"}="H311 KIR";
$resn_exch{"1HO1 KIR"}="HO11 KIR"; $resn_exch{"1H47 KIR"}="H471 KIR"; $resn_exch{"2H47 KIR"}="H472 KIR";
$resn_exch{"3H47 KIR"}="H473 KIR"; $resn_exch{"1H48 KIR"}="H481 KIR"; $resn_exch{"2H48 KIR"}="H482 KIR";
$resn_exch{"3H48 KIR"}="H483 KIR"; $resn_exch{"1H33 KIR"}="H331 KIR"; $resn_exch{"1H35 KIR"}="H351 KIR";
$resn_exch{"1H36 KIR"}="H361 KIR"; $resn_exch{"1H37 KIR"}="H371 KIR"; $resn_exch{"1H38 KIR"}="H381 KIR";
$resn_exch{"1H39 KIR"}="H391 KIR"; $resn_exch{"2H39 KIR"}="H392 KIR"; $resn_exch{"3H39 KIR"}="H393 KIR";
$resn_exch{"1HO1 KIV"}="HO11 KIV"; $resn_exch{"1HC3 KIV"}="HC31 KIV"; $resn_exch{"1HC4 KIV"}="HC41 KIV";
$resn_exch{"2HC4 KIV"}="HC42 KIV"; $resn_exch{"3HC4 KIV"}="HC43 KIV"; $resn_exch{"1HC5 KIV"}="HC51 KIV";
$resn_exch{"2HC5 KIV"}="HC52 KIV"; $resn_exch{"3HC5 KIV"}="HC53 KIV"; $resn_exch{"1H22 KJ2"}="H221 KJ2";
$resn_exch{"2H22 KJ2"}="H222 KJ2"; $resn_exch{"1H23 KJ2"}="H231 KJ2"; $resn_exch{"2H23 KJ2"}="H232 KJ2";
$resn_exch{"1H24 KJ2"}="H241 KJ2"; $resn_exch{"2H24 KJ2"}="H242 KJ2"; $resn_exch{"1H25 KJ2"}="H251 KJ2";
$resn_exch{"2H25 KJ2"}="H252 KJ2"; $resn_exch{"1H4C KLN"}="H4C1 KLN"; $resn_exch{"2H4C KLN"}="H4C2 KLN";
$resn_exch{"1H7C KLN"}="H7C1 KLN"; $resn_exch{"2H7C KLN"}="H7C2 KLN"; $resn_exch{"1H14 KLN"}="H141 KLN";
$resn_exch{"2H14 KLN"}="H142 KLN"; $resn_exch{"1H21 KLN"}="H211 KLN"; $resn_exch{"2H21 KLN"}="H212 KLN";
$resn_exch{"1H24 KLN"}="H241 KLN"; $resn_exch{"2H24 KLN"}="H242 KLN"; $resn_exch{"1H22 KLN"}="H221 KLN";
$resn_exch{"2H22 KLN"}="H222 KLN"; $resn_exch{"1H23 KLN"}="H231 KLN"; $resn_exch{"2H23 KLN"}="H232 KLN";
$resn_exch{"1H26 KLN"}="H261 KLN"; $resn_exch{"2H26 KLN"}="H262 KLN"; $resn_exch{"3H26 KLN"}="H263 KLN";
$resn_exch{"1H4  KMB"}=" H41 KMB"; $resn_exch{"2H4  KMB"}=" H42 KMB"; $resn_exch{"1HN4 KMB"}="HN41 KMB";
$resn_exch{"2HN4 KMB"}="HN42 KMB"; $resn_exch{"1HN2 KMB"}="HN21 KMB"; $resn_exch{"2HN2 KMB"}="HN22 KMB";
$resn_exch{"1H9  KMB"}=" H91 KMB"; $resn_exch{"2H9  KMB"}=" H92 KMB"; $resn_exch{"1H3  KMT"}=" H31 KMT";
$resn_exch{"2H3  KMT"}=" H32 KMT"; $resn_exch{"1H4  KMT"}=" H41 KMT"; $resn_exch{"2H4  KMT"}=" H42 KMT";
$resn_exch{"1H5  KMT"}=" H51 KMT"; $resn_exch{"2H5  KMT"}=" H52 KMT"; $resn_exch{"3H5  KMT"}=" H53 KMT";
$resn_exch{"1H10 KNI"}="H101 KNI"; $resn_exch{"2H10 KNI"}="H102 KNI"; $resn_exch{"1H12 KNI"}="H121 KNI";
$resn_exch{"2H12 KNI"}="H122 KNI"; $resn_exch{"1H16 KNI"}="H161 KNI"; $resn_exch{"2H16 KNI"}="H162 KNI";
$resn_exch{"1H19 KNI"}="H191 KNI"; $resn_exch{"2H19 KNI"}="H192 KNI"; $resn_exch{"1H20 KNI"}="H201 KNI";
$resn_exch{"2H20 KNI"}="H202 KNI"; $resn_exch{"1H23 KNI"}="H231 KNI"; $resn_exch{"2H23 KNI"}="H232 KNI";
$resn_exch{"3H23 KNI"}="H233 KNI"; $resn_exch{"1H24 KNI"}="H241 KNI"; $resn_exch{"2H24 KNI"}="H242 KNI";
$resn_exch{"3H24 KNI"}="H243 KNI"; $resn_exch{"1H25 KNI"}="H251 KNI"; $resn_exch{"2H25 KNI"}="H252 KNI";
$resn_exch{"3H25 KNI"}="H253 KNI"; $resn_exch{"1H27 KNI"}="H271 KNI"; $resn_exch{"2H27 KNI"}="H272 KNI";
$resn_exch{"3H27 KNI"}="H273 KNI"; $resn_exch{"1H1  KO4"}=" H11 KO4"; $resn_exch{"2H1  KO4"}=" H12 KO4";
$resn_exch{"1H2  KO4"}=" H21 KO4"; $resn_exch{"2H2  KO4"}=" H22 KO4"; $resn_exch{"1H3  KO4"}=" H31 KO4";
$resn_exch{"2H3  KO4"}=" H32 KO4"; $resn_exch{"1H4  KO4"}=" H41 KO4"; $resn_exch{"2H4  KO4"}=" H42 KO4";
$resn_exch{"1H6C KOJ"}="H6C1 KOJ"; $resn_exch{"2H6C KOJ"}="H6C2 KOJ"; $resn_exch{"1HAK KOR"}="HAK1 KOR";
$resn_exch{"2HAK KOR"}="HAK2 KOR"; $resn_exch{"1HAI KOR"}="HAI3 KOR"; $resn_exch{"2HAI KOR"}="HAI2 KOR";
$resn_exch{"1HN  KOR"}=" H   KOR"; $resn_exch{"2HN  KOR"}=" H2  KOR"; $resn_exch{"1HAH KOR"}="HAH3 KOR";
$resn_exch{"2HAH KOR"}="HAH2 KOR"; $resn_exch{"1HBC KOR"}=" HB3 KOR"; $resn_exch{"2HBC KOR"}=" HB2 KOR";
$resn_exch{"1H27 KOS"}="H271 KOS"; $resn_exch{"2H27 KOS"}="H272 KOS"; $resn_exch{"3H27 KOS"}="H273 KOS";
$resn_exch{"1H13 KOS"}="H131 KOS"; $resn_exch{"2H13 KOS"}="H132 KOS"; $resn_exch{"1H28 KOS"}="H281 KOS";
$resn_exch{"2H28 KOS"}="H282 KOS"; $resn_exch{"3H28 KOS"}="H283 KOS"; $resn_exch{"1H15 KOS"}="H151 KOS";
$resn_exch{"2H15 KOS"}="H152 KOS"; $resn_exch{"1H30 KOS"}="H301 KOS"; $resn_exch{"2H30 KOS"}="H302 KOS";
$resn_exch{"1H31 KOS"}="H311 KOS"; $resn_exch{"2H31 KOS"}="H312 KOS"; $resn_exch{"1H33 KOS"}="H331 KOS";
$resn_exch{"2H33 KOS"}="H332 KOS"; $resn_exch{"3H33 KOS"}="H333 KOS"; $resn_exch{"1H34 KOS"}="H341 KOS";
$resn_exch{"2H34 KOS"}="H342 KOS"; $resn_exch{"3H34 KOS"}="H343 KOS"; $resn_exch{"1H22 KOS"}="H221 KOS";
$resn_exch{"2H22 KOS"}="H222 KOS"; $resn_exch{"3H22 KOS"}="H223 KOS"; $resn_exch{"1H23 KOS"}="H231 KOS";
$resn_exch{"2H23 KOS"}="H232 KOS"; $resn_exch{"3H23 KOS"}="H233 KOS"; $resn_exch{"1HN2 KOS"}="HN21 KOS";
$resn_exch{"2HN2 KOS"}="HN22 KOS"; $resn_exch{"1H25 KOS"}="H251 KOS"; $resn_exch{"2H25 KOS"}="H252 KOS";
$resn_exch{"3H25 KOS"}="H253 KOS"; $resn_exch{"1H26 KOS"}="H261 KOS"; $resn_exch{"2H26 KOS"}="H262 KOS";
$resn_exch{"3H26 KOS"}="H263 KOS"; $resn_exch{"1HB  KPA"}=" HB1 KPA"; $resn_exch{"2HB  KPA"}=" HB2 KPA";
$resn_exch{"1HG  KPA"}=" HG1 KPA"; $resn_exch{"2HG  KPA"}=" HG2 KPA"; $resn_exch{"1HP  KPA"}=" HP1 KPA";
$resn_exch{"2HP  KPA"}=" HP2 KPA"; $resn_exch{"1HAK KPC"}="HAK1 KPC"; $resn_exch{"2HAK KPC"}="HAK2 KPC";
$resn_exch{"3HAK KPC"}="HAK3 KPC"; $resn_exch{"1HAI KPC"}="HAI1 KPC"; $resn_exch{"2HAI KPC"}="HAI2 KPC";
$resn_exch{"1HAG KPC"}="HAG1 KPC"; $resn_exch{"2HAG KPC"}="HAG2 KPC"; $resn_exch{"1HAF KPC"}="HAF1 KPC";
$resn_exch{"2HAF KPC"}="HAF2 KPC"; $resn_exch{"1HAL KPC"}="HAL1 KPC"; $resn_exch{"1HN  KPH"}=" HN1 KPH";
$resn_exch{"2HN  KPH"}=" HN2 KPH"; $resn_exch{"2HOP KPH"}="HOP2 KPH"; $resn_exch{"1HB  KPH"}=" HB1 KPH";
$resn_exch{"2HB  KPH"}=" HB2 KPH"; $resn_exch{"1HG  KPH"}=" HG1 KPH"; $resn_exch{"2HG  KPH"}=" HG2 KPH";
$resn_exch{"1HD  KPH"}=" HD1 KPH"; $resn_exch{"2HD  KPH"}=" HD2 KPH"; $resn_exch{"1HE  KPH"}=" HE1 KPH";
$resn_exch{"2HE  KPH"}=" HE2 KPH"; $resn_exch{"1HNZ KPH"}="HNZ1 KPH"; $resn_exch{"2HNZ KPH"}="HNZ2 KPH";
$resn_exch{"1H3  KPL"}=" H31 KPL"; $resn_exch{"2H3  KPL"}=" H32 KPL"; $resn_exch{"3H3  KPL"}=" H33 KPL";
$resn_exch{"1H1  KPL"}=" H11 KPL"; $resn_exch{"2H1  KPL"}=" H12 KPL"; $resn_exch{"3H1  KPL"}=" H13 KPL";
$resn_exch{"1H4  KPL"}=" H41 KPL"; $resn_exch{"2H4  KPL"}=" H42 KPL"; $resn_exch{"1H7  KPV"}=" H71 KPV";
$resn_exch{"2H7  KPV"}=" H72 KPV"; $resn_exch{"1H8  KPV"}=" H81 KPV"; $resn_exch{"2H8  KPV"}=" H82 KPV";
$resn_exch{"1H9  KPV"}=" H91 KPV"; $resn_exch{"2H9  KPV"}=" H92 KPV"; $resn_exch{"1HAM KR2"}="HAM1 KR2";
$resn_exch{"2HAM KR2"}="HAM2 KR2"; $resn_exch{"1HAN KR2"}="HAN1 KR2"; $resn_exch{"2HAN KR2"}="HAN2 KR2";
$resn_exch{"1HAV KR2"}="HAV1 KR2"; $resn_exch{"2HAV KR2"}="HAV2 KR2"; $resn_exch{"1HDD KR2"}="HDD1 KR2";
$resn_exch{"2HDD KR2"}="HDD2 KR2"; $resn_exch{"1HNN KR2"}="HNN1 KR2"; $resn_exch{"2HNN KR2"}="HNN2 KR2";
$resn_exch{"1HAT KR2"}="HAT1 KR2"; $resn_exch{"2HAT KR2"}="HAT2 KR2"; $resn_exch{"1HAS KR2"}="HAS1 KR2";
$resn_exch{"2HAS KR2"}="HAS2 KR2"; $resn_exch{"1HAK KR2"}="HAK1 KR2"; $resn_exch{"2HAK KR2"}="HAK2 KR2";
$resn_exch{"1H7  KRA"}=" H71 KRA"; $resn_exch{"2H7  KRA"}=" H72 KRA"; $resn_exch{"1H10 KRA"}="H101 KRA";
$resn_exch{"2H10 KRA"}="H102 KRA"; $resn_exch{"1H16 KRB"}="H161 KRB"; $resn_exch{"2H16 KRB"}="H162 KRB";
$resn_exch{"1H25 KRB"}="H251 KRB"; $resn_exch{"2H25 KRB"}="H252 KRB"; $resn_exch{"1HN  KRI"}=" HN1 KRI";
$resn_exch{"2HN  KRI"}=" HN2 KRI"; $resn_exch{"1HB  KRI"}=" HB1 KRI"; $resn_exch{"2HB  KRI"}=" HB2 KRI";
$resn_exch{"1HG  KRI"}=" HG1 KRI"; $resn_exch{"2HG  KRI"}=" HG2 KRI"; $resn_exch{"1H5  KRI"}=" H51 KRI";
$resn_exch{"2H5  KRI"}=" H52 KRI"; $resn_exch{"1H1  KRI"}=" H11 KRI"; $resn_exch{"2H1  KRI"}=" H12 KRI";
$resn_exch{"1H26 KSA"}="H261 KSA"; $resn_exch{"2H26 KSA"}="H262 KSA"; $resn_exch{"3H26 KSA"}="H263 KSA";
$resn_exch{"1H25 KSA"}="H251 KSA"; $resn_exch{"2H25 KSA"}="H252 KSA"; $resn_exch{"3H25 KSA"}="H253 KSA";
$resn_exch{"1H27 KSA"}="H271 KSA"; $resn_exch{"2H27 KSA"}="H272 KSA"; $resn_exch{"1H23 KSA"}="H231 KSA";
$resn_exch{"2H23 KSA"}="H232 KSA"; $resn_exch{"1H9  KSG"}=" H91 KSG"; $resn_exch{"2H9  KSG"}=" H92 KSG";
$resn_exch{"3H9  KSG"}=" H93 KSG"; $resn_exch{"1H11 KSG"}="H111 KSG"; $resn_exch{"2H11 KSG"}="H112 KSG";
$resn_exch{"1HN1 KSG"}="HN11 KSG"; $resn_exch{"2HN1 KSG"}="HN12 KSG"; $resn_exch{"1H4  KST"}=" H41 KST";
$resn_exch{"2H4  KST"}=" H42 KST"; $resn_exch{"1H3  KST"}=" H31 KST"; $resn_exch{"2H3  KST"}=" H32 KST";
$resn_exch{"1H2  KST"}=" H21 KST"; $resn_exch{"2H2  KST"}=" H22 KST"; $resn_exch{"1H1  KST"}=" H11 KST";
$resn_exch{"2H1  KST"}=" H12 KST"; $resn_exch{"1HN  KST"}="HN1A KST"; $resn_exch{"2HN  KST"}=" HN2 KST";
$resn_exch{"2H   KT3"}=" H2  KT3"; $resn_exch{"1HG1 KT3"}="HG11 KT3"; $resn_exch{"1HG2 KT3"}="HG21 KT3";
$resn_exch{"1HB1 KT3"}="HB11 KT3"; $resn_exch{"1HB2 KT3"}="HB21 KT3"; $resn_exch{"1HA  KT3"}="HA1A KT3";
$resn_exch{"1HXT KT3"}="HXT1 KT3"; $resn_exch{"1H   KT3"}=" H1  KT3"; $resn_exch{"1H1  KT3"}=" H11 KT3";
$resn_exch{"2H1  KT3"}="H12A KT3"; $resn_exch{"1H2  KT3"}=" H21 KT3"; $resn_exch{"2H2  KT3"}=" H22 KT3";
$resn_exch{"1H3  KT3"}=" H31 KT3"; $resn_exch{"2H3  KT3"}=" H32 KT3"; $resn_exch{"1HN8 KT3"}="HN81 KT3";
$resn_exch{"2HN8 KT3"}="HN82 KT3"; $resn_exch{"1HN2 KT3"}="HN21 KT3"; $resn_exch{"2HN2 KT3"}="HN22 KT3";
$resn_exch{"3HE2 KT3"}="HE23 KT3"; $resn_exch{"3HG1 KT3"}="HG13 KT3"; $resn_exch{"3HG2 KT3"}="HG23 KT3";
$resn_exch{"3HB1 KT3"}="HB13 KT3"; $resn_exch{"3HB2 KT3"}="HB23 KT3"; $resn_exch{"3HA  KT3"}=" HA3 KT3";
$resn_exch{"3HXT KT3"}="HXT3 KT3"; $resn_exch{"3H   KT3"}=" H3  KT3"; $resn_exch{"2HG1 KT3"}="HG12 KT3";
$resn_exch{"2HG2 KT3"}="HG22 KT3"; $resn_exch{"2HB1 KT3"}="HB12 KT3"; $resn_exch{"2HB2 KT3"}="HB22 KT3";
$resn_exch{"2HA  KT3"}="HA2A KT3"; $resn_exch{"2HXT KT3"}="HXT2 KT3"; $resn_exch{"2H   KT5"}=" H2  KT5";
$resn_exch{"1HG1 KT5"}="HG11 KT5"; $resn_exch{"1HG2 KT5"}="HG21 KT5"; $resn_exch{"1HB1 KT5"}="HB11 KT5";
$resn_exch{"1HB2 KT5"}="HB21 KT5"; $resn_exch{"1HA  KT5"}="HA1A KT5"; $resn_exch{"1HXT KT5"}="HXT1 KT5";
$resn_exch{"1H   KT5"}=" H1  KT5"; $resn_exch{"1H1  KT5"}=" H11 KT5"; $resn_exch{"2H1  KT5"}="H12A KT5";
$resn_exch{"1H2  KT5"}=" H21 KT5"; $resn_exch{"2H2  KT5"}=" H22 KT5"; $resn_exch{"1H3  KT5"}=" H31 KT5";
$resn_exch{"2H3  KT5"}=" H32 KT5"; $resn_exch{"1HN8 KT5"}="HN81 KT5"; $resn_exch{"2HN8 KT5"}="HN82 KT5";
$resn_exch{"1HN2 KT5"}="HN21 KT5"; $resn_exch{"2HN2 KT5"}="HN22 KT5"; $resn_exch{"5HE2 KT5"}="HE25 KT5";
$resn_exch{"5HG1 KT5"}="HG15 KT5"; $resn_exch{"5HG2 KT5"}="HG25 KT5"; $resn_exch{"5HB1 KT5"}="HB15 KT5";
$resn_exch{"5HB2 KT5"}="HB25 KT5"; $resn_exch{"5HA  KT5"}=" HA5 KT5"; $resn_exch{"5HXT KT5"}="HXT5 KT5";
$resn_exch{"5H   KT5"}=" H5  KT5"; $resn_exch{"4HG1 KT5"}="HG14 KT5"; $resn_exch{"4HG2 KT5"}="HG24 KT5";
$resn_exch{"4HB1 KT5"}="HB14 KT5"; $resn_exch{"4HB2 KT5"}="HB24 KT5"; $resn_exch{"4HA  KT5"}=" HA4 KT5";
$resn_exch{"4HXT KT5"}="HXT4 KT5"; $resn_exch{"4H   KT5"}=" H4  KT5"; $resn_exch{"3HG1 KT5"}="HG13 KT5";
$resn_exch{"3HG2 KT5"}="HG23 KT5"; $resn_exch{"3HB1 KT5"}="HB13 KT5"; $resn_exch{"3HB2 KT5"}="HB23 KT5";
$resn_exch{"3HA  KT5"}=" HA3 KT5"; $resn_exch{"3HXT KT5"}="HXT3 KT5"; $resn_exch{"3H   KT5"}=" H3  KT5";
$resn_exch{"2HG1 KT5"}="HG12 KT5"; $resn_exch{"2HG2 KT5"}="HG22 KT5"; $resn_exch{"2HB1 KT5"}="HB12 KT5";
$resn_exch{"2HB2 KT5"}="HB22 KT5"; $resn_exch{"2HA  KT5"}="HA2A KT5"; $resn_exch{"2HXT KT5"}="HXT2 KT5";
$resn_exch{"1H6C KTA"}="H6C1 KTA"; $resn_exch{"2H6C KTA"}="H6C2 KTA"; $resn_exch{"1H5C KTA"}="H5C1 KTA";
$resn_exch{"2H5C KTA"}="H5C2 KTA"; $resn_exch{"1H4C KTA"}="H4C1 KTA"; $resn_exch{"2H4C KTA"}="H4C2 KTA";
$resn_exch{"1H2C KTA"}="H2C1 KTA"; $resn_exch{"2H2C KTA"}="H2C2 KTA"; $resn_exch{"1H7C KTA"}="H7C1 KTA";
$resn_exch{"2H7C KTA"}="H7C2 KTA"; $resn_exch{"1HC4 KTN"}="HC41 KTN"; $resn_exch{"2HC4 KTN"}="HC42 KTN";
$resn_exch{"1HC7 KTN"}="HC71 KTN"; $resn_exch{"2HC7 KTN"}="HC72 KTN"; $resn_exch{"1H14 KTN"}="H141 KTN";
$resn_exch{"2H14 KTN"}="H142 KTN"; $resn_exch{"1H21 KTN"}="H211 KTN"; $resn_exch{"2H21 KTN"}="H212 KTN";
$resn_exch{"1H22 KTN"}="H221 KTN"; $resn_exch{"2H22 KTN"}="H222 KTN"; $resn_exch{"1H23 KTN"}="H231 KTN";
$resn_exch{"2H23 KTN"}="H232 KTN"; $resn_exch{"1H24 KTN"}="H241 KTN"; $resn_exch{"2H24 KTN"}="H242 KTN";
$resn_exch{"1H26 KTN"}="H261 KTN"; $resn_exch{"2H26 KTN"}="H262 KTN"; $resn_exch{"3H26 KTN"}="H263 KTN";
$resn_exch{"1H23 KWT"}="H231 KWT"; $resn_exch{"2H23 KWT"}="H232 KWT"; $resn_exch{"1H10 KWT"}="H101 KWT";
$resn_exch{"2H10 KWT"}="H102 KWT"; $resn_exch{"3H10 KWT"}="H103 KWT"; $resn_exch{"1H12 KWT"}="H121 KWT";
$resn_exch{"2H12 KWT"}="H122 KWT"; $resn_exch{"1H14 KWT"}="H141 KWT"; $resn_exch{"2H14 KWT"}="H142 KWT";
$resn_exch{"3H14 KWT"}="H143 KWT"; $resn_exch{"1H16 KWT"}="H161 KWT"; $resn_exch{"2H16 KWT"}="H162 KWT";
$resn_exch{"1H17 KWT"}="H171 KWT"; $resn_exch{"2H17 KWT"}="H172 KWT"; $resn_exch{"1H22 KWT"}="H221 KWT";
$resn_exch{"2H22 KWT"}="H222 KWT"; $resn_exch{"3H22 KWT"}="H223 KWT"; $resn_exch{"1H24 KWT"}="H241 KWT";
$resn_exch{"2H24 KWT"}="H242 KWT"; $resn_exch{"3H24 KWT"}="H243 KWT"; $resn_exch{"1HAC KY1"}="HAC1 KY1";
$resn_exch{"2HAC KY1"}="HAC2 KY1"; $resn_exch{"1HBC KY1"}="HBC1 KY1"; $resn_exch{"2HBC KY1"}="HBC2 KY1";
$resn_exch{"1H1N KY1"}="H1N1 KY1"; $resn_exch{"2H1N KY1"}="H1N2 KY1"; $resn_exch{" HN2 KYN"}=" H2  KYN";
$resn_exch{"1HB  KYN"}=" HB2 KYN"; $resn_exch{"2HB  KYN"}=" HB3 KYN"; $resn_exch{"1HD  KYN"}=" HD1 KYN";
$resn_exch{"1HE  KYN"}=" HE1 KYN"; $resn_exch{"2HE  KYN"}=" HE2 KYN"; $resn_exch{"1H1  KYN"}=" H11 KYN";
$resn_exch{"2H1  KYN"}=" H12 KYN"; $resn_exch{"1H51 KYS"}="H511 KYS"; $resn_exch{"2H51 KYS"}="H512 KYS";
$resn_exch{"1H47 KYS"}="H471 KYS"; $resn_exch{"2H47 KYS"}="H472 KYS"; $resn_exch{"1H46 KYS"}="H461 KYS";
$resn_exch{"2H46 KYS"}="H462 KYS"; $resn_exch{"1H45 KYS"}="H451 KYS"; $resn_exch{"2H45 KYS"}="H452 KYS";
$resn_exch{"1H40 KYS"}="H401 KYS"; $resn_exch{"2H40 KYS"}="H402 KYS"; $resn_exch{"1H28 KYS"}="H281 KYS";
$resn_exch{"2H28 KYS"}="H282 KYS"; $resn_exch{"1H29 KYS"}="H291 KYS"; $resn_exch{"2H29 KYS"}="H292 KYS";
$resn_exch{"1H15 L00"}="H151 L00"; $resn_exch{"2H15 L00"}="H152 L00"; $resn_exch{"3H15 L00"}="H153 L00";
$resn_exch{"1H16 L00"}="H161 L00"; $resn_exch{"2H16 L00"}="H162 L00"; $resn_exch{"3H16 L00"}="H163 L00";
$resn_exch{"1HN6 L00"}="HN61 L00"; $resn_exch{"2HN6 L00"}="HN62 L00"; $resn_exch{"1H21 L00"}="H211 L00";
$resn_exch{"2H21 L00"}="H212 L00"; $resn_exch{"3H21 L00"}="H213 L00"; $resn_exch{"1H22 L00"}="H221 L00";
$resn_exch{"2H22 L00"}="H222 L00"; $resn_exch{"1H1  L00"}=" H11 L00"; $resn_exch{"2H1  L00"}=" H12 L00";
$resn_exch{"1H2  L00"}=" H21 L00"; $resn_exch{"2H2  L00"}=" H22 L00"; $resn_exch{"1H3  L00"}=" H31 L00";
$resn_exch{"2H3  L00"}=" H32 L00"; $resn_exch{"1H4  L00"}=" H41 L00"; $resn_exch{"2H4  L00"}=" H42 L00";
$resn_exch{"3H4  L00"}=" H43 L00"; $resn_exch{"1H7  L00"}=" H71 L00"; $resn_exch{"2H7  L00"}=" H72 L00";
$resn_exch{"1H8  L00"}=" H81 L00"; $resn_exch{"2H8  L00"}=" H82 L00"; $resn_exch{"3H8  L00"}=" H83 L00";
$resn_exch{"1H1C L01"}="H1C1 L01"; $resn_exch{"2H1C L01"}="H1C2 L01"; $resn_exch{"3H1C L01"}="H1C3 L01";
$resn_exch{"1H2C L01"}="H2C1 L01"; $resn_exch{"2H2C L01"}="H2C2 L01"; $resn_exch{"1H3C L01"}="H3C1 L01";
$resn_exch{"2H3C L01"}="H3C2 L01"; $resn_exch{"1H5C L01"}="H5C1 L01"; $resn_exch{"2H5C L01"}="H5C2 L01";
$resn_exch{"1H6C L01"}="H6C1 L01"; $resn_exch{"2H6C L01"}="H6C2 L01"; $resn_exch{"1H7C L01"}="H7C1 L01";
$resn_exch{"2H7C L01"}="H7C2 L01"; $resn_exch{"3H7C L01"}="H7C3 L01"; $resn_exch{"1H21 L01"}="H211 L01";
$resn_exch{"2H21 L01"}="H212 L01"; $resn_exch{"1H31 L01"}="H311 L01"; $resn_exch{"2H31 L01"}="H312 L01";
$resn_exch{"1H35 L01"}="H351 L01"; $resn_exch{"2H35 L01"}="H352 L01"; $resn_exch{"1H42 L01"}="H421 L01";
$resn_exch{"2H42 L01"}="H422 L01"; $resn_exch{"3H42 L01"}="H423 L01"; $resn_exch{"1H8C L02"}="H8C1 L02";
$resn_exch{"2H8C L02"}="H8C2 L02"; $resn_exch{"1H10 L02"}="H101 L02"; $resn_exch{"2H10 L02"}="H102 L02";
$resn_exch{"1H13 L02"}="H131 L02"; $resn_exch{"2H13 L02"}="H132 L02"; $resn_exch{"1H15 L02"}="H151 L02";
$resn_exch{"2H15 L02"}="H152 L02"; $resn_exch{"1H1C L03"}="H1C1 L03"; $resn_exch{"2H1C L03"}="H1C2 L03";
$resn_exch{"3H1C L03"}="H1C3 L03"; $resn_exch{"1H1  L04"}="H11A L04"; $resn_exch{"2H1  L04"}="H12A L04";
$resn_exch{"1H4  L04"}=" H41 L04"; $resn_exch{"2H4  L04"}="H42A L04"; $resn_exch{"3H4  L04"}="H43A L04";
$resn_exch{"1H8  L04"}=" H81 L04"; $resn_exch{"2H8  L04"}=" H82 L04"; $resn_exch{"1H9  L04"}=" H91 L04";
$resn_exch{"2H9  L04"}=" H92 L04"; $resn_exch{"1H18 L04"}="H181 L04"; $resn_exch{"2H18 L04"}="H182 L04";
$resn_exch{"1H20 L04"}="H201 L04"; $resn_exch{"2H20 L04"}="H202 L04"; $resn_exch{"1H22 L04"}="H221 L04";
$resn_exch{"2H22 L04"}="H222 L04"; $resn_exch{"3H22 L04"}="H223 L04"; $resn_exch{"1H23 L04"}="H231 L04";
$resn_exch{"2H23 L04"}="H232 L04"; $resn_exch{"1H24 L04"}="H241 L04"; $resn_exch{"2H24 L04"}="H242 L04";
$resn_exch{"1H25 L04"}="H251 L04"; $resn_exch{"2H25 L04"}="H252 L04"; $resn_exch{"1H28 L04"}="H281 L04";
$resn_exch{"2H28 L04"}="H282 L04"; $resn_exch{"1H1  L05"}=" H11 L05"; $resn_exch{"2H1  L05"}=" H12 L05";
$resn_exch{"3H1  L05"}=" H13 L05"; $resn_exch{"1H13 L05"}="H131 L05"; $resn_exch{"2H13 L05"}="H132 L05";
$resn_exch{"1H1  L06"}=" H11 L06"; $resn_exch{"2H1  L06"}="H12A L06"; $resn_exch{"1H3  L06"}=" H31 L06";
$resn_exch{"2H3  L06"}=" H32 L06"; $resn_exch{"1H4  L06"}=" H41 L06"; $resn_exch{"2H4  L06"}=" H42 L06";
$resn_exch{"1H5  L06"}=" H51 L06"; $resn_exch{"2H5  L06"}=" H52 L06"; $resn_exch{"1HN9 L06"}="HN91 L06";
$resn_exch{"2HN9 L06"}="HN92 L06"; $resn_exch{"1H19 L08"}="H191 L08"; $resn_exch{"2H19 L08"}="H192 L08";
$resn_exch{"1H20 L08"}="H201 L08"; $resn_exch{"2H20 L08"}="H202 L08"; $resn_exch{"1H21 L08"}="H211 L08";
$resn_exch{"2H21 L08"}="H212 L08"; $resn_exch{"1H22 L08"}="H221 L08"; $resn_exch{"2H22 L08"}="H222 L08";
$resn_exch{"1H24 L08"}="H241 L08"; $resn_exch{"2H24 L08"}="H242 L08"; $resn_exch{"3H24 L08"}="H243 L08";
$resn_exch{"1H9  L08"}=" H91 L08"; $resn_exch{"2H9  L08"}=" H92 L08"; $resn_exch{"1H33 L08"}="H331 L08";
$resn_exch{"2H33 L08"}="H332 L08"; $resn_exch{"3H33 L08"}="H333 L08"; $resn_exch{"1H1C L09"}="H1C1 L09";
$resn_exch{"2H1C L09"}="H1C2 L09"; $resn_exch{"3H1C L09"}="H1C3 L09"; $resn_exch{"1H3C L09"}="H3C1 L09";
$resn_exch{"2H3C L09"}="H3C2 L09"; $resn_exch{"3H3C L09"}="H3C3 L09"; $resn_exch{"1H4C L09"}="H4C1 L09";
$resn_exch{"2H4C L09"}="H4C2 L09"; $resn_exch{"3H4C L09"}="H4C3 L09"; $resn_exch{"1H18 L09"}="H181 L09";
$resn_exch{"2H18 L09"}="H182 L09"; $resn_exch{"1H4C L10"}="H4C1 L10"; $resn_exch{"2H4C L10"}="H4C2 L10";
$resn_exch{"3H4C L10"}="H4C3 L10"; $resn_exch{"1H3C L10"}="H3C1 L10"; $resn_exch{"2H3C L10"}="H3C2 L10";
$resn_exch{"3H3C L10"}="H3C3 L10"; $resn_exch{"1H1C L10"}="H1C1 L10"; $resn_exch{"2H1C L10"}="H1C2 L10";
$resn_exch{"3H1C L10"}="H1C3 L10"; $resn_exch{"1H17 L11"}="H171 L11"; $resn_exch{"2H17 L11"}="H172 L11";
$resn_exch{"1H3C L11"}="H3C1 L11"; $resn_exch{"2H3C L11"}="H3C2 L11"; $resn_exch{"1H31 L11"}="H311 L11";
$resn_exch{"2H31 L11"}="H312 L11"; $resn_exch{"1H30 L11"}="H301 L11"; $resn_exch{"2H30 L11"}="H302 L11";
$resn_exch{"1H28 L11"}="H281 L11"; $resn_exch{"2H28 L11"}="H282 L11"; $resn_exch{"1H27 L11"}="H271 L11";
$resn_exch{"2H27 L11"}="H272 L11"; $resn_exch{"1H5C L12"}="H5C1 L12"; $resn_exch{"2H5C L12"}="H5C2 L12";
$resn_exch{"1H6C L12"}="H6C1 L12"; $resn_exch{"2H6C L12"}="H6C2 L12"; $resn_exch{"1H14 L13"}="H141 L13";
$resn_exch{"2H14 L13"}="H142 L13"; $resn_exch{"1H13 L13"}="H131 L13"; $resn_exch{"2H13 L13"}="H132 L13";
$resn_exch{"3H13 L13"}="H133 L13"; $resn_exch{"1H1C L15"}="H1C1 L15"; $resn_exch{"2H1C L15"}="H1C2 L15";
$resn_exch{"3H1C L15"}="H1C3 L15"; $resn_exch{"1H11 L15"}="H111 L15"; $resn_exch{"2H11 L15"}="H112 L15";
$resn_exch{"1H12 L15"}="H121 L15"; $resn_exch{"2H12 L15"}="H122 L15"; $resn_exch{"1H13 L15"}="H131 L15";
$resn_exch{"2H13 L15"}="H132 L15"; $resn_exch{"1H1  L17"}=" H11 L17"; $resn_exch{"2H1  L17"}=" H12 L17";
$resn_exch{"1H33 L17"}="H331 L17"; $resn_exch{"2H33 L17"}="H332 L17"; $resn_exch{"1H7  L17"}=" H71 L17";
$resn_exch{"2H7  L17"}=" H72 L17"; $resn_exch{"3H7  L17"}=" H73 L17"; $resn_exch{"1H9  L1A"}=" H91 L1A";
$resn_exch{"2H9  L1A"}=" H92 L1A"; $resn_exch{"1H13 L1A"}="H131 L1A"; $resn_exch{"2H13 L1A"}="H132 L1A";
$resn_exch{"1HN4 L1A"}="HN41 L1A"; $resn_exch{"2HN4 L1A"}="HN42 L1A"; $resn_exch{"1H17 L1A"}="H171 L1A";
$resn_exch{"2H17 L1A"}="H172 L1A"; $resn_exch{"1H20 L1A"}="H201 L1A"; $resn_exch{"2H20 L1A"}="H202 L1A";
$resn_exch{"1H22 L1A"}="H221 L1A"; $resn_exch{"2H22 L1A"}="H222 L1A"; $resn_exch{"3H22 L1A"}="H223 L1A";
$resn_exch{"1H23 L1A"}="H231 L1A"; $resn_exch{"2H23 L1A"}="H232 L1A"; $resn_exch{"3H23 L1A"}="H233 L1A";
$resn_exch{"1H24 L1A"}="H241 L1A"; $resn_exch{"2H24 L1A"}="H242 L1A"; $resn_exch{"1H25 L1A"}="H251 L1A";
$resn_exch{"2H25 L1A"}="H252 L1A"; $resn_exch{"1H26 L1A"}="H261 L1A"; $resn_exch{"2H26 L1A"}="H262 L1A";
$resn_exch{"1H27 L1A"}="H271 L1A"; $resn_exch{"2H27 L1A"}="H272 L1A"; $resn_exch{"1H28 L1A"}="H281 L1A";
$resn_exch{"2H28 L1A"}="H282 L1A"; $resn_exch{"1H30 L1A"}="H301 L1A"; $resn_exch{"2H30 L1A"}="H302 L1A";
$resn_exch{"1H33 L1A"}="H331 L1A"; $resn_exch{"2H33 L1A"}="H332 L1A"; $resn_exch{"1HN3 L1B"}="HN31 L1B";
$resn_exch{"2HN3 L1B"}="HN32 L1B"; $resn_exch{"1HN4 L1B"}="HN41 L1B"; $resn_exch{"2HN4 L1B"}="HN42 L1B";
$resn_exch{"1H5  L1B"}=" H51 L1B"; $resn_exch{"2H5  L1B"}=" H52 L1B"; $resn_exch{"1H6  L1B"}=" H61 L1B";
$resn_exch{"2H6  L1B"}=" H62 L1B"; $resn_exch{"3H6  L1B"}=" H63 L1B"; $resn_exch{"1H16 L1B"}="H161 L1B";
$resn_exch{"2H16 L1B"}="H162 L1B"; $resn_exch{"1H17 L1B"}="H171 L1B"; $resn_exch{"2H17 L1B"}="H172 L1B";
$resn_exch{"1H18 L1B"}="H181 L1B"; $resn_exch{"2H18 L1B"}="H182 L1B"; $resn_exch{"1H19 L1B"}="H191 L1B";
$resn_exch{"2H19 L1B"}="H192 L1B"; $resn_exch{"3H19 L1B"}="H193 L1B"; $resn_exch{"1H5N L1G"}="H5N1 L1G";
$resn_exch{"2H5N L1G"}="H5N2 L1G"; $resn_exch{"1H13 L1G"}="H131 L1G"; $resn_exch{"2H13 L1G"}="H132 L1G";
$resn_exch{"1H14 L1G"}="H141 L1G"; $resn_exch{"2H14 L1G"}="H142 L1G"; $resn_exch{"1H15 L1G"}="H151 L1G";
$resn_exch{"2H15 L1G"}="H152 L1G"; $resn_exch{"3H15 L1G"}="H153 L1G"; $resn_exch{"1H16 L1G"}="H161 L1G";
$resn_exch{"2H16 L1G"}="H162 L1G"; $resn_exch{"1H17 L1G"}="H171 L1G"; $resn_exch{"2H17 L1G"}="H172 L1G";
$resn_exch{"1H22 L1G"}="H221 L1G"; $resn_exch{"2H22 L1G"}="H222 L1G"; $resn_exch{"1H23 L1G"}="H231 L1G";
$resn_exch{"2H23 L1G"}="H232 L1G"; $resn_exch{"1H25 L1G"}="H251 L1G"; $resn_exch{"2H25 L1G"}="H252 L1G";
$resn_exch{"3H25 L1G"}="H253 L1G"; $resn_exch{"1H27 L1G"}="H271 L1G"; $resn_exch{"2H27 L1G"}="H272 L1G";
$resn_exch{"1H28 L1G"}="H281 L1G"; $resn_exch{"2H28 L1G"}="H282 L1G"; $resn_exch{"1H33 L1G"}="H331 L1G";
$resn_exch{"2H33 L1G"}="H332 L1G"; $resn_exch{"3H33 L1G"}="H333 L1G"; $resn_exch{"1H1N L1O"}="H1N1 L1O";
$resn_exch{"2H1N L1O"}="H1N2 L1O"; $resn_exch{"1H2C L1O"}="H2C1 L1O"; $resn_exch{"2H2C L1O"}="H2C2 L1O";
$resn_exch{"1H3C L1O"}="H3C1 L1O"; $resn_exch{"2H3C L1O"}="H3C2 L1O"; $resn_exch{"2H1  L1P"}=" H12 L1P";
$resn_exch{"1H1  L1P"}=" H11 L1P"; $resn_exch{"2H3  L1P"}=" H32 L1P"; $resn_exch{"1H3  L1P"}=" H31 L1P";
$resn_exch{"2HOP L1P"}="HOP2 L1P"; $resn_exch{"3HOP L1P"}="HOP3 L1P"; $resn_exch{"2H11 L1P"}="H112 L1P";
$resn_exch{"1H11 L1P"}="H111 L1P"; $resn_exch{"2H12 L1P"}="H122 L1P"; $resn_exch{"1H12 L1P"}="H121 L1P";
$resn_exch{"3H14 L1P"}="H143 L1P"; $resn_exch{"2H14 L1P"}="H142 L1P"; $resn_exch{"1H14 L1P"}="H141 L1P";
$resn_exch{"2H15 L1P"}="H152 L1P"; $resn_exch{"1H15 L1P"}="H151 L1P"; $resn_exch{"2H16 L1P"}="H162 L1P";
$resn_exch{"1H16 L1P"}="H161 L1P"; $resn_exch{"2H17 L1P"}="H172 L1P"; $resn_exch{"1H17 L1P"}="H171 L1P";
$resn_exch{"3H19 L1P"}="H193 L1P"; $resn_exch{"2H19 L1P"}="H192 L1P"; $resn_exch{"1H19 L1P"}="H191 L1P";
$resn_exch{"2H20 L1P"}="H202 L1P"; $resn_exch{"1H20 L1P"}="H201 L1P"; $resn_exch{"2H21 L1P"}="H212 L1P";
$resn_exch{"1H21 L1P"}="H211 L1P"; $resn_exch{"2H22 L1P"}="H222 L1P"; $resn_exch{"1H22 L1P"}="H221 L1P";
$resn_exch{"3H24 L1P"}="H243 L1P"; $resn_exch{"2H24 L1P"}="H242 L1P"; $resn_exch{"1H24 L1P"}="H241 L1P";
$resn_exch{"2H25 L1P"}="H252 L1P"; $resn_exch{"1H25 L1P"}="H251 L1P"; $resn_exch{"2H26 L1P"}="H262 L1P";
$resn_exch{"1H26 L1P"}="H261 L1P"; $resn_exch{"2H27 L1P"}="H272 L1P"; $resn_exch{"1H27 L1P"}="H271 L1P";
$resn_exch{"3H29 L1P"}="H293 L1P"; $resn_exch{"2H29 L1P"}="H292 L1P"; $resn_exch{"1H29 L1P"}="H291 L1P";
$resn_exch{"3H30 L1P"}="H303 L1P"; $resn_exch{"2H30 L1P"}="H302 L1P"; $resn_exch{"1H30 L1P"}="H301 L1P";
$resn_exch{"2H41 L1P"}="H412 L1P"; $resn_exch{"1H41 L1P"}="H411 L1P"; $resn_exch{"2H42 L1P"}="H422 L1P";
$resn_exch{"1H42 L1P"}="H421 L1P"; $resn_exch{"3H44 L1P"}="H443 L1P"; $resn_exch{"2H44 L1P"}="H442 L1P";
$resn_exch{"1H44 L1P"}="H441 L1P"; $resn_exch{"2H45 L1P"}="H452 L1P"; $resn_exch{"1H45 L1P"}="H451 L1P";
$resn_exch{"2H46 L1P"}="H462 L1P"; $resn_exch{"1H46 L1P"}="H461 L1P"; $resn_exch{"2H47 L1P"}="H472 L1P";
$resn_exch{"1H47 L1P"}="H471 L1P"; $resn_exch{"3H49 L1P"}="H493 L1P"; $resn_exch{"2H49 L1P"}="H492 L1P";
$resn_exch{"1H49 L1P"}="H491 L1P"; $resn_exch{"2H50 L1P"}="H502 L1P"; $resn_exch{"1H50 L1P"}="H501 L1P";
$resn_exch{"2H51 L1P"}="H512 L1P"; $resn_exch{"1H51 L1P"}="H511 L1P"; $resn_exch{"2H52 L1P"}="H522 L1P";
$resn_exch{"1H52 L1P"}="H521 L1P"; $resn_exch{"3H54 L1P"}="H543 L1P"; $resn_exch{"2H54 L1P"}="H542 L1P";
$resn_exch{"1H54 L1P"}="H541 L1P"; $resn_exch{"2H55 L1P"}="H552 L1P"; $resn_exch{"1H55 L1P"}="H551 L1P";
$resn_exch{"2H56 L1P"}="H562 L1P"; $resn_exch{"1H56 L1P"}="H561 L1P"; $resn_exch{"2H57 L1P"}="H572 L1P";
$resn_exch{"1H57 L1P"}="H571 L1P"; $resn_exch{"3H59 L1P"}="H593 L1P"; $resn_exch{"2H59 L1P"}="H592 L1P";
$resn_exch{"1H59 L1P"}="H591 L1P"; $resn_exch{"3H60 L1P"}="H603 L1P"; $resn_exch{"2H60 L1P"}="H602 L1P";
$resn_exch{"1H60 L1P"}="H601 L1P"; $resn_exch{"1H1C L1R"}="H1C1 L1R"; $resn_exch{"2H1C L1R"}="H1C2 L1R";
$resn_exch{"3H1C L1R"}="H1C3 L1R"; $resn_exch{"1H3C L1R"}="H3C1 L1R"; $resn_exch{"2H3C L1R"}="H3C2 L1R";
$resn_exch{"3H3C L1R"}="H3C3 L1R"; $resn_exch{"1H19 L1R"}="H191 L1R"; $resn_exch{"2H19 L1R"}="H192 L1R";
$resn_exch{"1H20 L1R"}="H201 L1R"; $resn_exch{"2H20 L1R"}="H202 L1R"; $resn_exch{"1H21 L1R"}="H211 L1R";
$resn_exch{"2H21 L1R"}="H212 L1R"; $resn_exch{"1H1C L20"}="H1C1 L20"; $resn_exch{"2H1C L20"}="H1C2 L20";
$resn_exch{"3H1C L20"}="H1C3 L20"; $resn_exch{"1H18 L20"}="H181 L20"; $resn_exch{"2H18 L20"}="H182 L20";
$resn_exch{"1H21 L20"}="H211 L20"; $resn_exch{"2H21 L20"}="H212 L20"; $resn_exch{"1H25 L20"}="H251 L20";
$resn_exch{"2H25 L20"}="H252 L20"; $resn_exch{"1HN2 L24"}="HN21 L24"; $resn_exch{"2HN2 L24"}="HN22 L24";
$resn_exch{"1H8  L24"}=" H81 L24"; $resn_exch{"2H8  L24"}=" H82 L24"; $resn_exch{"1H10 L24"}="H101 L24";
$resn_exch{"2H10 L24"}="H102 L24"; $resn_exch{"1H11 L24"}="H111 L24"; $resn_exch{"2H11 L24"}="H112 L24";
$resn_exch{"1H12 L24"}="H121 L24"; $resn_exch{"2H12 L24"}="H122 L24"; $resn_exch{"1HB  L24"}=" HB1 L24";
$resn_exch{"2HB  L24"}=" HB2 L24"; $resn_exch{"1HG  L24"}=" HG1 L24"; $resn_exch{"2HG  L24"}=" HG2 L24";
$resn_exch{"1H12 L2C"}="H121 L2C"; $resn_exch{"2H12 L2C"}="H122 L2C"; $resn_exch{"1H13 L2C"}="H131 L2C";
$resn_exch{"2H13 L2C"}="H132 L2C"; $resn_exch{"1H14 L2C"}="H141 L2C"; $resn_exch{"2H14 L2C"}="H142 L2C";
$resn_exch{"1H15 L2C"}="H151 L2C"; $resn_exch{"2H15 L2C"}="H152 L2C"; $resn_exch{"1H16 L2C"}="H161 L2C";
$resn_exch{"2H16 L2C"}="H162 L2C"; $resn_exch{"1H17 L2C"}="H171 L2C"; $resn_exch{"2H17 L2C"}="H172 L2C";
$resn_exch{"1H18 L2C"}="H181 L2C"; $resn_exch{"2H18 L2C"}="H182 L2C"; $resn_exch{"1H19 L2C"}="H191 L2C";
$resn_exch{"2H19 L2C"}="H192 L2C"; $resn_exch{"3H19 L2C"}="H193 L2C"; $resn_exch{"1H22 L2C"}="H221 L2C";
$resn_exch{"2H22 L2C"}="H222 L2C"; $resn_exch{"1H23 L2C"}="H231 L2C"; $resn_exch{"2H23 L2C"}="H232 L2C";
$resn_exch{"1H24 L2C"}="H241 L2C"; $resn_exch{"2H24 L2C"}="H242 L2C"; $resn_exch{"1H25 L2C"}="H251 L2C";
$resn_exch{"2H25 L2C"}="H252 L2C"; $resn_exch{"1H26 L2C"}="H261 L2C"; $resn_exch{"2H26 L2C"}="H262 L2C";
$resn_exch{"1H27 L2C"}="H271 L2C"; $resn_exch{"2H27 L2C"}="H272 L2C"; $resn_exch{"1H28 L2C"}="H281 L2C";
$resn_exch{"2H28 L2C"}="H282 L2C"; $resn_exch{"1H29 L2C"}="H291 L2C"; $resn_exch{"2H29 L2C"}="H292 L2C";
$resn_exch{"1H30 L2C"}="H301 L2C"; $resn_exch{"2H30 L2C"}="H302 L2C"; $resn_exch{"1H31 L2C"}="H311 L2C";
$resn_exch{"2H31 L2C"}="H312 L2C"; $resn_exch{"1H32 L2C"}="H321 L2C"; $resn_exch{"2H32 L2C"}="H322 L2C";
$resn_exch{"3H32 L2C"}="H323 L2C"; $resn_exch{"1H41 L2C"}="H411 L2C"; $resn_exch{"2H41 L2C"}="H412 L2C";
$resn_exch{"1H43 L2C"}="H431 L2C"; $resn_exch{"2H43 L2C"}="H432 L2C"; $resn_exch{"1H5N L2G"}="H5N1 L2G";
$resn_exch{"2H5N L2G"}="H5N2 L2G"; $resn_exch{"1H13 L2G"}="H131 L2G"; $resn_exch{"2H13 L2G"}="H132 L2G";
$resn_exch{"1H14 L2G"}="H141 L2G"; $resn_exch{"2H14 L2G"}="H142 L2G"; $resn_exch{"1H15 L2G"}="H151 L2G";
$resn_exch{"2H15 L2G"}="H152 L2G"; $resn_exch{"3H15 L2G"}="H153 L2G"; $resn_exch{"1H16 L2G"}="H161 L2G";
$resn_exch{"2H16 L2G"}="H162 L2G"; $resn_exch{"1H17 L2G"}="H171 L2G"; $resn_exch{"2H17 L2G"}="H172 L2G";
$resn_exch{"1H22 L2G"}="H221 L2G"; $resn_exch{"2H22 L2G"}="H222 L2G"; $resn_exch{"1H23 L2G"}="H231 L2G";
$resn_exch{"2H23 L2G"}="H232 L2G"; $resn_exch{"1H25 L2G"}="H251 L2G"; $resn_exch{"2H25 L2G"}="H252 L2G";
$resn_exch{"3H25 L2G"}="H253 L2G"; $resn_exch{"1H27 L2G"}="H271 L2G"; $resn_exch{"2H27 L2G"}="H272 L2G";
$resn_exch{"1H28 L2G"}="H281 L2G"; $resn_exch{"2H28 L2G"}="H282 L2G"; $resn_exch{"1H34 L2G"}="H341 L2G";
$resn_exch{"2H34 L2G"}="H342 L2G"; $resn_exch{"3H34 L2G"}="H343 L2G"; $resn_exch{"2H1  L2P"}=" H12 L2P";
$resn_exch{"1H1  L2P"}=" H11 L2P"; $resn_exch{"2H3  L2P"}=" H32 L2P"; $resn_exch{"1H3  L2P"}=" H31 L2P";
$resn_exch{"2H11 L2P"}="H112 L2P"; $resn_exch{"1H11 L2P"}="H111 L2P"; $resn_exch{"2H12 L2P"}="H122 L2P";
$resn_exch{"1H12 L2P"}="H121 L2P"; $resn_exch{"3H14 L2P"}="H143 L2P"; $resn_exch{"2H14 L2P"}="H142 L2P";
$resn_exch{"1H14 L2P"}="H141 L2P"; $resn_exch{"2H15 L2P"}="H152 L2P"; $resn_exch{"1H15 L2P"}="H151 L2P";
$resn_exch{"2H16 L2P"}="H162 L2P"; $resn_exch{"1H16 L2P"}="H161 L2P"; $resn_exch{"2H17 L2P"}="H172 L2P";
$resn_exch{"1H17 L2P"}="H171 L2P"; $resn_exch{"3H19 L2P"}="H193 L2P"; $resn_exch{"2H19 L2P"}="H192 L2P";
$resn_exch{"1H19 L2P"}="H191 L2P"; $resn_exch{"2H20 L2P"}="H202 L2P"; $resn_exch{"1H20 L2P"}="H201 L2P";
$resn_exch{"2H21 L2P"}="H212 L2P"; $resn_exch{"1H21 L2P"}="H211 L2P"; $resn_exch{"2H22 L2P"}="H222 L2P";
$resn_exch{"1H22 L2P"}="H221 L2P"; $resn_exch{"3H24 L2P"}="H243 L2P"; $resn_exch{"2H24 L2P"}="H242 L2P";
$resn_exch{"1H24 L2P"}="H241 L2P"; $resn_exch{"2H25 L2P"}="H252 L2P"; $resn_exch{"1H25 L2P"}="H251 L2P";
$resn_exch{"2H26 L2P"}="H262 L2P"; $resn_exch{"1H26 L2P"}="H261 L2P"; $resn_exch{"2H27 L2P"}="H272 L2P";
$resn_exch{"1H27 L2P"}="H271 L2P"; $resn_exch{"3H29 L2P"}="H293 L2P"; $resn_exch{"2H29 L2P"}="H292 L2P";
$resn_exch{"1H29 L2P"}="H291 L2P"; $resn_exch{"3H30 L2P"}="H303 L2P"; $resn_exch{"2H30 L2P"}="H302 L2P";
$resn_exch{"1H30 L2P"}="H301 L2P"; $resn_exch{"2H41 L2P"}="H412 L2P"; $resn_exch{"1H41 L2P"}="H411 L2P";
$resn_exch{"2H42 L2P"}="H422 L2P"; $resn_exch{"1H42 L2P"}="H421 L2P"; $resn_exch{"3H44 L2P"}="H443 L2P";
$resn_exch{"2H44 L2P"}="H442 L2P"; $resn_exch{"1H44 L2P"}="H441 L2P"; $resn_exch{"2H45 L2P"}="H452 L2P";
$resn_exch{"1H45 L2P"}="H451 L2P"; $resn_exch{"2H46 L2P"}="H462 L2P"; $resn_exch{"1H46 L2P"}="H461 L2P";
$resn_exch{"2H47 L2P"}="H472 L2P"; $resn_exch{"1H47 L2P"}="H471 L2P"; $resn_exch{"3H49 L2P"}="H493 L2P";
$resn_exch{"2H49 L2P"}="H492 L2P"; $resn_exch{"1H49 L2P"}="H491 L2P"; $resn_exch{"2H50 L2P"}="H502 L2P";
$resn_exch{"1H50 L2P"}="H501 L2P"; $resn_exch{"2H51 L2P"}="H512 L2P"; $resn_exch{"1H51 L2P"}="H511 L2P";
$resn_exch{"2H52 L2P"}="H522 L2P"; $resn_exch{"1H52 L2P"}="H521 L2P"; $resn_exch{"3H54 L2P"}="H543 L2P";
$resn_exch{"2H54 L2P"}="H542 L2P"; $resn_exch{"1H54 L2P"}="H541 L2P"; $resn_exch{"2H55 L2P"}="H552 L2P";
$resn_exch{"1H55 L2P"}="H551 L2P"; $resn_exch{"2H56 L2P"}="H562 L2P"; $resn_exch{"1H56 L2P"}="H561 L2P";
$resn_exch{"2H57 L2P"}="H572 L2P"; $resn_exch{"1H57 L2P"}="H571 L2P"; $resn_exch{"3H59 L2P"}="H593 L2P";
$resn_exch{"2H59 L2P"}="H592 L2P"; $resn_exch{"1H59 L2P"}="H591 L2P"; $resn_exch{"3H60 L2P"}="H603 L2P";
$resn_exch{"2H60 L2P"}="H602 L2P"; $resn_exch{"1H60 L2P"}="H601 L2P"; $resn_exch{"1HN2 L34"}="HN21 L34";
$resn_exch{"2HN2 L34"}="HN22 L34"; $resn_exch{"1H7  L34"}=" H71 L34"; $resn_exch{"2H7  L34"}=" H72 L34";
$resn_exch{"1H9  L34"}=" H91 L34"; $resn_exch{"2H9  L34"}=" H92 L34"; $resn_exch{"1HB  L34"}=" HB1 L34";
$resn_exch{"2HB  L34"}=" HB2 L34"; $resn_exch{"1HG  L34"}=" HG1 L34"; $resn_exch{"2HG  L34"}=" HG2 L34";
$resn_exch{"1H15 L35"}="H151 L35"; $resn_exch{"2H15 L35"}="H152 L35"; $resn_exch{"3H15 L35"}="H153 L35";
$resn_exch{"1H16 L35"}="H161 L35"; $resn_exch{"2H16 L35"}="H162 L35"; $resn_exch{"3H16 L35"}="H163 L35";
$resn_exch{"1HN2 L37"}="HN21 L37"; $resn_exch{"2HN2 L37"}="HN22 L37"; $resn_exch{"1HN5 L37"}="HN51 L37";
$resn_exch{"2HN5 L37"}="HN52 L37"; $resn_exch{"1HB  L37"}=" HB1 L37"; $resn_exch{"2HB  L37"}=" HB2 L37";
$resn_exch{"1HG  L37"}=" HG1 L37"; $resn_exch{"2HG  L37"}=" HG2 L37"; $resn_exch{"1H5N L3G"}="H5N1 L3G";
$resn_exch{"2H5N L3G"}="H5N2 L3G"; $resn_exch{"1H13 L3G"}="H131 L3G"; $resn_exch{"2H13 L3G"}="H132 L3G";
$resn_exch{"1H14 L3G"}="H141 L3G"; $resn_exch{"2H14 L3G"}="H142 L3G"; $resn_exch{"1H15 L3G"}="H151 L3G";
$resn_exch{"2H15 L3G"}="H152 L3G"; $resn_exch{"3H15 L3G"}="H153 L3G"; $resn_exch{"1H16 L3G"}="H161 L3G";
$resn_exch{"2H16 L3G"}="H162 L3G"; $resn_exch{"1H17 L3G"}="H171 L3G"; $resn_exch{"2H17 L3G"}="H172 L3G";
$resn_exch{"1H22 L3G"}="H221 L3G"; $resn_exch{"2H22 L3G"}="H222 L3G"; $resn_exch{"1H23 L3G"}="H231 L3G";
$resn_exch{"2H23 L3G"}="H232 L3G"; $resn_exch{"1H25 L3G"}="H251 L3G"; $resn_exch{"2H25 L3G"}="H252 L3G";
$resn_exch{"3H25 L3G"}="H253 L3G"; $resn_exch{"1H27 L3G"}="H271 L3G"; $resn_exch{"2H27 L3G"}="H272 L3G";
$resn_exch{"1H28 L3G"}="H281 L3G"; $resn_exch{"2H28 L3G"}="H282 L3G"; $resn_exch{"2H1  L3P"}=" H12 L3P";
$resn_exch{"1H1  L3P"}=" H11 L3P"; $resn_exch{"2H3  L3P"}=" H32 L3P"; $resn_exch{"1H3  L3P"}=" H31 L3P";
$resn_exch{"2H4  L3P"}=" H42 L3P"; $resn_exch{"1H4  L3P"}=" H41 L3P"; $resn_exch{"2H6  L3P"}=" H62 L3P";
$resn_exch{"1H6  L3P"}=" H61 L3P"; $resn_exch{"6HOP L3P"}="HOP6 L3P"; $resn_exch{"2H11 L3P"}="H112 L3P";
$resn_exch{"1H11 L3P"}="H111 L3P"; $resn_exch{"2H12 L3P"}="H122 L3P"; $resn_exch{"1H12 L3P"}="H121 L3P";
$resn_exch{"3H14 L3P"}="H143 L3P"; $resn_exch{"2H14 L3P"}="H142 L3P"; $resn_exch{"1H14 L3P"}="H141 L3P";
$resn_exch{"2H15 L3P"}="H152 L3P"; $resn_exch{"1H15 L3P"}="H151 L3P"; $resn_exch{"2H16 L3P"}="H162 L3P";
$resn_exch{"1H16 L3P"}="H161 L3P"; $resn_exch{"2H17 L3P"}="H172 L3P"; $resn_exch{"1H17 L3P"}="H171 L3P";
$resn_exch{"3H19 L3P"}="H193 L3P"; $resn_exch{"2H19 L3P"}="H192 L3P"; $resn_exch{"1H19 L3P"}="H191 L3P";
$resn_exch{"2H20 L3P"}="H202 L3P"; $resn_exch{"1H20 L3P"}="H201 L3P"; $resn_exch{"2H21 L3P"}="H212 L3P";
$resn_exch{"1H21 L3P"}="H211 L3P"; $resn_exch{"2H22 L3P"}="H222 L3P"; $resn_exch{"1H22 L3P"}="H221 L3P";
$resn_exch{"3H24 L3P"}="H243 L3P"; $resn_exch{"2H24 L3P"}="H242 L3P"; $resn_exch{"1H24 L3P"}="H241 L3P";
$resn_exch{"2H25 L3P"}="H252 L3P"; $resn_exch{"1H25 L3P"}="H251 L3P"; $resn_exch{"2H26 L3P"}="H262 L3P";
$resn_exch{"1H26 L3P"}="H261 L3P"; $resn_exch{"2H27 L3P"}="H272 L3P"; $resn_exch{"1H27 L3P"}="H271 L3P";
$resn_exch{"3H29 L3P"}="H293 L3P"; $resn_exch{"2H29 L3P"}="H292 L3P"; $resn_exch{"1H29 L3P"}="H291 L3P";
$resn_exch{"3H30 L3P"}="H303 L3P"; $resn_exch{"2H30 L3P"}="H302 L3P"; $resn_exch{"1H30 L3P"}="H301 L3P";
$resn_exch{"2H41 L3P"}="H412 L3P"; $resn_exch{"1H41 L3P"}="H411 L3P"; $resn_exch{"2H42 L3P"}="H422 L3P";
$resn_exch{"1H42 L3P"}="H421 L3P"; $resn_exch{"3H44 L3P"}="H443 L3P"; $resn_exch{"2H44 L3P"}="H442 L3P";
$resn_exch{"1H44 L3P"}="H441 L3P"; $resn_exch{"2H45 L3P"}="H452 L3P"; $resn_exch{"1H45 L3P"}="H451 L3P";
$resn_exch{"2H46 L3P"}="H462 L3P"; $resn_exch{"1H46 L3P"}="H461 L3P"; $resn_exch{"2H47 L3P"}="H472 L3P";
$resn_exch{"1H47 L3P"}="H471 L3P"; $resn_exch{"3H49 L3P"}="H493 L3P"; $resn_exch{"2H49 L3P"}="H492 L3P";
$resn_exch{"1H49 L3P"}="H491 L3P"; $resn_exch{"2H50 L3P"}="H502 L3P"; $resn_exch{"1H50 L3P"}="H501 L3P";
$resn_exch{"2H51 L3P"}="H512 L3P"; $resn_exch{"1H51 L3P"}="H511 L3P"; $resn_exch{"2H52 L3P"}="H522 L3P";
$resn_exch{"1H52 L3P"}="H521 L3P"; $resn_exch{"3H54 L3P"}="H543 L3P"; $resn_exch{"2H54 L3P"}="H542 L3P";
$resn_exch{"1H54 L3P"}="H541 L3P"; $resn_exch{"2H55 L3P"}="H552 L3P"; $resn_exch{"1H55 L3P"}="H551 L3P";
$resn_exch{"2H56 L3P"}="H562 L3P"; $resn_exch{"1H56 L3P"}="H561 L3P"; $resn_exch{"2H57 L3P"}="H572 L3P";
$resn_exch{"1H57 L3P"}="H571 L3P"; $resn_exch{"3H59 L3P"}="H593 L3P"; $resn_exch{"2H59 L3P"}="H592 L3P";
$resn_exch{"1H59 L3P"}="H591 L3P"; $resn_exch{"3H60 L3P"}="H603 L3P"; $resn_exch{"2H60 L3P"}="H602 L3P";
$resn_exch{"1H60 L3P"}="H601 L3P"; $resn_exch{"1H15 L47"}="H151 L47"; $resn_exch{"2H15 L47"}="H152 L47";
$resn_exch{"1H21 L47"}="H211 L47"; $resn_exch{"2H21 L47"}="H212 L47"; $resn_exch{"3H21 L47"}="H213 L47";
$resn_exch{"1H10 L4C"}="H101 L4C"; $resn_exch{"2H10 L4C"}="H102 L4C"; $resn_exch{"1H11 L4C"}="H111 L4C";
$resn_exch{"2H11 L4C"}="H112 L4C"; $resn_exch{"1H5C L4C"}="H5C1 L4C"; $resn_exch{"2H5C L4C"}="H5C2 L4C";
$resn_exch{"1H7C L4C"}="H7C1 L4C"; $resn_exch{"2H7C L4C"}="H7C2 L4C"; $resn_exch{"3H7C L4C"}="H7C3 L4C";
$resn_exch{"1H27 L4G"}="H271 L4G"; $resn_exch{"2H27 L4G"}="H272 L4G"; $resn_exch{"3H27 L4G"}="H273 L4G";
$resn_exch{"1H30 L4G"}="H301 L4G"; $resn_exch{"2H30 L4G"}="H302 L4G"; $resn_exch{"1H31 L4G"}="H311 L4G";
$resn_exch{"2H31 L4G"}="H312 L4G"; $resn_exch{"1H33 L4G"}="H331 L4G"; $resn_exch{"2H33 L4G"}="H332 L4G";
$resn_exch{"1H34 L4G"}="H341 L4G"; $resn_exch{"2H34 L4G"}="H342 L4G"; $resn_exch{"1H35 L4G"}="H351 L4G";
$resn_exch{"2H35 L4G"}="H352 L4G"; $resn_exch{"1H36 L4G"}="H361 L4G"; $resn_exch{"2H36 L4G"}="H362 L4G";
$resn_exch{"1H37 L4G"}="H371 L4G"; $resn_exch{"2H37 L4G"}="H372 L4G"; $resn_exch{"2H1  L4P"}=" H12 L4P";
$resn_exch{"1H1  L4P"}=" H11 L4P"; $resn_exch{"2H3  L4P"}=" H32 L4P"; $resn_exch{"1H3  L4P"}=" H31 L4P";
$resn_exch{"2H4  L4P"}=" H42 L4P"; $resn_exch{"1H4  L4P"}=" H41 L4P"; $resn_exch{"2H6  L4P"}=" H62 L4P";
$resn_exch{"1H6  L4P"}=" H61 L4P"; $resn_exch{"2HOP L4P"}="HOP2 L4P"; $resn_exch{"2H11 L4P"}="H112 L4P";
$resn_exch{"1H11 L4P"}="H111 L4P"; $resn_exch{"2H12 L4P"}="H122 L4P"; $resn_exch{"1H12 L4P"}="H121 L4P";
$resn_exch{"3H14 L4P"}="H143 L4P"; $resn_exch{"2H14 L4P"}="H142 L4P"; $resn_exch{"1H14 L4P"}="H141 L4P";
$resn_exch{"2H15 L4P"}="H152 L4P"; $resn_exch{"1H15 L4P"}="H151 L4P"; $resn_exch{"2H16 L4P"}="H162 L4P";
$resn_exch{"1H16 L4P"}="H161 L4P"; $resn_exch{"2H17 L4P"}="H172 L4P"; $resn_exch{"1H17 L4P"}="H171 L4P";
$resn_exch{"3H19 L4P"}="H193 L4P"; $resn_exch{"2H19 L4P"}="H192 L4P"; $resn_exch{"1H19 L4P"}="H191 L4P";
$resn_exch{"2H20 L4P"}="H202 L4P"; $resn_exch{"1H20 L4P"}="H201 L4P"; $resn_exch{"2H21 L4P"}="H212 L4P";
$resn_exch{"1H21 L4P"}="H211 L4P"; $resn_exch{"2H22 L4P"}="H222 L4P"; $resn_exch{"1H22 L4P"}="H221 L4P";
$resn_exch{"3H24 L4P"}="H243 L4P"; $resn_exch{"2H24 L4P"}="H242 L4P"; $resn_exch{"1H24 L4P"}="H241 L4P";
$resn_exch{"2H25 L4P"}="H252 L4P"; $resn_exch{"1H25 L4P"}="H251 L4P"; $resn_exch{"2H26 L4P"}="H262 L4P";
$resn_exch{"1H26 L4P"}="H261 L4P"; $resn_exch{"2H27 L4P"}="H272 L4P"; $resn_exch{"1H27 L4P"}="H271 L4P";
$resn_exch{"3H29 L4P"}="H293 L4P"; $resn_exch{"2H29 L4P"}="H292 L4P"; $resn_exch{"1H29 L4P"}="H291 L4P";
$resn_exch{"3H30 L4P"}="H303 L4P"; $resn_exch{"2H30 L4P"}="H302 L4P"; $resn_exch{"1H30 L4P"}="H301 L4P";
$resn_exch{"2H41 L4P"}="H412 L4P"; $resn_exch{"1H41 L4P"}="H411 L4P"; $resn_exch{"2H42 L4P"}="H422 L4P";
$resn_exch{"1H42 L4P"}="H421 L4P"; $resn_exch{"3H44 L4P"}="H443 L4P"; $resn_exch{"2H44 L4P"}="H442 L4P";
$resn_exch{"1H44 L4P"}="H441 L4P"; $resn_exch{"2H45 L4P"}="H452 L4P"; $resn_exch{"1H45 L4P"}="H451 L4P";
$resn_exch{"2H46 L4P"}="H462 L4P"; $resn_exch{"1H46 L4P"}="H461 L4P"; $resn_exch{"2H47 L4P"}="H472 L4P";
$resn_exch{"1H47 L4P"}="H471 L4P"; $resn_exch{"3H49 L4P"}="H493 L4P"; $resn_exch{"2H49 L4P"}="H492 L4P";
$resn_exch{"1H49 L4P"}="H491 L4P"; $resn_exch{"2H50 L4P"}="H502 L4P"; $resn_exch{"1H50 L4P"}="H501 L4P";
$resn_exch{"2H51 L4P"}="H512 L4P"; $resn_exch{"1H51 L4P"}="H511 L4P"; $resn_exch{"2H52 L4P"}="H522 L4P";
$resn_exch{"1H52 L4P"}="H521 L4P"; $resn_exch{"3H54 L4P"}="H543 L4P"; $resn_exch{"2H54 L4P"}="H542 L4P";
$resn_exch{"1H54 L4P"}="H541 L4P"; $resn_exch{"2H55 L4P"}="H552 L4P"; $resn_exch{"1H55 L4P"}="H551 L4P";
$resn_exch{"2H56 L4P"}="H562 L4P"; $resn_exch{"1H56 L4P"}="H561 L4P"; $resn_exch{"2H57 L4P"}="H572 L4P";
$resn_exch{"1H57 L4P"}="H571 L4P"; $resn_exch{"3H59 L4P"}="H593 L4P"; $resn_exch{"2H59 L4P"}="H592 L4P";
$resn_exch{"1H59 L4P"}="H591 L4P"; $resn_exch{"3H60 L4P"}="H603 L4P"; $resn_exch{"2H60 L4P"}="H602 L4P";
$resn_exch{"1H60 L4P"}="H601 L4P"; $resn_exch{"1H1  L75"}=" H11 L75"; $resn_exch{"2H1  L75"}=" H12 L75";
$resn_exch{"1H2  L75"}=" H21 L75"; $resn_exch{"1H5  L75"}=" H51 L75"; $resn_exch{"2H5  L75"}=" H52 L75";
$resn_exch{"3H5  L75"}=" H53 L75"; $resn_exch{"1H6  L75"}=" H61 L75"; $resn_exch{"2H6  L75"}=" H62 L75";
$resn_exch{"3H6  L75"}=" H63 L75"; $resn_exch{"1H7  L75"}=" H71 L75"; $resn_exch{"2H7  L75"}=" H72 L75";
$resn_exch{"3H7  L75"}=" H73 L75"; $resn_exch{"1H8  L75"}=" H81 L75"; $resn_exch{"2H8  L75"}=" H82 L75";
$resn_exch{"1H9  L75"}=" H91 L75"; $resn_exch{"2H9  L75"}=" H92 L75"; $resn_exch{"1H10 L75"}="H101 L75";
$resn_exch{"2H10 L75"}="H102 L75"; $resn_exch{"1H11 L75"}="H111 L75"; $resn_exch{"1H12 L75"}="H121 L75";
$resn_exch{"2H12 L75"}="H122 L75"; $resn_exch{"1H13 L75"}="H131 L75"; $resn_exch{"1H14 L75"}="H141 L75";
$resn_exch{"2H14 L75"}="H142 L75"; $resn_exch{"1H16 L75"}="H161 L75"; $resn_exch{"1H17 L75"}="H171 L75";
$resn_exch{"1H18 L75"}="H181 L75"; $resn_exch{"1H19 L75"}="H191 L75"; $resn_exch{"1H20 L75"}="H201 L75";
$resn_exch{"1H22 L75"}="H221 L75"; $resn_exch{"1H23 L75"}="H231 L75"; $resn_exch{"1H24 L75"}="H241 L75";
$resn_exch{"2H24 L75"}="H242 L75"; $resn_exch{"1H26 L75"}="H261 L75"; $resn_exch{"1H27 L75"}="H271 L75";
$resn_exch{"1H28 L75"}="H281 L75"; $resn_exch{"1H29 L75"}="H291 L75"; $resn_exch{"1H31 L75"}="H311 L75";
$resn_exch{"2H31 L75"}="H312 L75"; $resn_exch{"1H32 L75"}="H321 L75"; $resn_exch{"1H33 L75"}="H331 L75";
$resn_exch{"1H36 L75"}="H361 L75"; $resn_exch{"1H37 L75"}="H371 L75"; $resn_exch{"1H39 L75"}="H391 L75";
$resn_exch{"1H   L79"}=" H1A L79"; $resn_exch{"2H   L79"}=" H2A L79"; $resn_exch{"1H16 L79"}="H161 L79";
$resn_exch{"2H16 L79"}="H162 L79"; $resn_exch{"1H17 L79"}="H171 L79"; $resn_exch{"2H17 L79"}="H172 L79";
$resn_exch{"3H17 L79"}="H173 L79"; $resn_exch{"1H18 L79"}="H181 L79"; $resn_exch{"2H18 L79"}="H182 L79";
$resn_exch{"3H18 L79"}="H183 L79"; $resn_exch{"1H21 L79"}="H211 L79"; $resn_exch{"2H21 L79"}="H212 L79";
$resn_exch{"3H21 L79"}="H213 L79"; $resn_exch{"1H22 L79"}="H221 L79"; $resn_exch{"2H22 L79"}="H222 L79";
$resn_exch{"1H23 L79"}="H231 L79"; $resn_exch{"2H23 L79"}="H232 L79"; $resn_exch{"3H23 L79"}="H233 L79";
$resn_exch{"1H11 L86"}="H111 L86"; $resn_exch{"2H11 L86"}="H112 L86"; $resn_exch{"1H33 L86"}="H331 L86";
$resn_exch{"2H33 L86"}="H332 L86"; $resn_exch{"1H39 L86"}="H391 L86"; $resn_exch{"2H39 L86"}="H392 L86";
$resn_exch{"1H9  L86"}=" H91 L86"; $resn_exch{"2H9  L86"}=" H92 L86"; $resn_exch{"1H3  L86"}=" H31 L86";
$resn_exch{"2H3  L86"}=" H32 L86"; $resn_exch{"1H7  L86"}=" H71 L86"; $resn_exch{"2H7  L86"}=" H72 L86";
$resn_exch{"1H12 L86"}="H121 L86"; $resn_exch{"2H12 L86"}="H122 L86"; $resn_exch{"1H13 L86"}="H131 L86";
$resn_exch{"2H13 L86"}="H132 L86"; $resn_exch{"1H14 L86"}="H141 L86"; $resn_exch{"2H14 L86"}="H142 L86";
$resn_exch{"1H1  L98"}="H11A L98"; $resn_exch{"2H1  L98"}=" H12 L98"; $resn_exch{"1H12 L98"}="H121 L98";
$resn_exch{"2H12 L98"}="H122 L98"; $resn_exch{"1H73 LA1"}="H731 LA1"; $resn_exch{"2H73 LA1"}="H732 LA1";
$resn_exch{"3H73 LA1"}="H733 LA1"; $resn_exch{"1H49 LA1"}="H491 LA1"; $resn_exch{"2H49 LA1"}="H492 LA1";
$resn_exch{"1H3  LA1"}=" H31 LA1"; $resn_exch{"2H3  LA1"}=" H32 LA1"; $resn_exch{"1H6  LA1"}="H61A LA1";
$resn_exch{"2H6  LA1"}=" H62 LA1"; $resn_exch{"1H14 LA1"}="H141 LA1"; $resn_exch{"2H14 LA1"}="H142 LA1";
$resn_exch{"1H23 LA1"}="H231 LA1"; $resn_exch{"2H23 LA1"}="H232 LA1"; $resn_exch{"3H23 LA1"}="H233 LA1";
$resn_exch{"1H19 LA1"}="H191 LA1"; $resn_exch{"2H19 LA1"}="H192 LA1"; $resn_exch{"3H19 LA1"}="H193 LA1";
$resn_exch{"1H27 LA1"}="H271 LA1"; $resn_exch{"2H27 LA1"}="H272 LA1"; $resn_exch{"1H17 LAB"}="H171 LAB";
$resn_exch{"2H17 LAB"}="H172 LAB"; $resn_exch{"1H14 LAB"}="H141 LAB"; $resn_exch{"2H14 LAB"}="H142 LAB";
$resn_exch{"1H12 LAB"}="H121 LAB"; $resn_exch{"2H12 LAB"}="H122 LAB"; $resn_exch{"1H10 LAB"}="H101 LAB";
$resn_exch{"2H10 LAB"}="H102 LAB"; $resn_exch{"1H9  LAB"}=" H91 LAB"; $resn_exch{"2H9  LAB"}=" H92 LAB";
$resn_exch{"1H20 LAB"}="H201 LAB"; $resn_exch{"2H20 LAB"}="H202 LAB"; $resn_exch{"3H20 LAB"}="H203 LAB";
$resn_exch{"1H5  LAB"}=" H51 LAB"; $resn_exch{"2H5  LAB"}=" H52 LAB"; $resn_exch{"1H4  LAB"}=" H41 LAB";
$resn_exch{"2H4  LAB"}=" H42 LAB"; $resn_exch{"1H19 LAB"}="H191 LAB"; $resn_exch{"2H19 LAB"}="H192 LAB";
$resn_exch{"3H19 LAB"}="H193 LAB"; $resn_exch{"1H3  LAC"}=" H31 LAC"; $resn_exch{"2H3  LAC"}=" H32 LAC";
$resn_exch{"3H3  LAC"}=" H33 LAC"; $resn_exch{"2H   LAD"}=" H2A LAD"; $resn_exch{"1H   LAD"}=" H1  LAD";
$resn_exch{"2HB  LAD"}=" HB2 LAD"; $resn_exch{"1HB  LAD"}=" HB1 LAD"; $resn_exch{"2HG  LAD"}=" HG2 LAD";
$resn_exch{"1HG  LAD"}=" HG1 LAD"; $resn_exch{"2HD  LAD"}=" HD2 LAD"; $resn_exch{"1HD  LAD"}=" HD1 LAD";
$resn_exch{"2HE  LAD"}=" HE2 LAD"; $resn_exch{"1HE  LAD"}=" HE1 LAD"; $resn_exch{"2HZ  LAD"}=" HZ2 LAD";
$resn_exch{"1HZ  LAD"}=" HZ1 LAD"; $resn_exch{"2H5' LAD"}="H5'2 LAD"; $resn_exch{"1H5' LAD"}="H5'1 LAD";
$resn_exch{"2H6  LAD"}=" H62 LAD"; $resn_exch{"1H6  LAD"}=" H61 LAD"; $resn_exch{"1H2O LAD"}="H2O1 LAD";
$resn_exch{"1H3O LAD"}="H3O1 LAD"; $resn_exch{"OH2A LAD"}="H2AO LAD"; $resn_exch{"1H1  LAE"}=" H11 LAE";
$resn_exch{"1H4  LAE"}=" H41 LAE"; $resn_exch{"2H4  LAE"}=" H42 LAE"; $resn_exch{"1H5  LAE"}=" H51 LAE";
$resn_exch{"2H5  LAE"}=" H52 LAE"; $resn_exch{"1H11 LAE"}="H111 LAE"; $resn_exch{"1H14 LAE"}="H141 LAE";
$resn_exch{"2H14 LAE"}="H142 LAE"; $resn_exch{"1H18 LAE"}="H181 LAE"; $resn_exch{"2H18 LAE"}="H182 LAE";
$resn_exch{"1H19 LAE"}="H191 LAE"; $resn_exch{"2H19 LAE"}="H192 LAE"; $resn_exch{"1H22 LAE"}="H221 LAE";
$resn_exch{"2H22 LAE"}="H222 LAE"; $resn_exch{"1H25 LAE"}="H251 LAE"; $resn_exch{"2H25 LAE"}="H252 LAE";
$resn_exch{"1H28 LAE"}="H281 LAE"; $resn_exch{"2H28 LAE"}="H282 LAE"; $resn_exch{"3H28 LAE"}="H283 LAE";
$resn_exch{"1H5C LAF"}="H5C1 LAF"; $resn_exch{"2H5C LAF"}="H5C2 LAF"; $resn_exch{"1H3C LAF"}="H3C1 LAF";
$resn_exch{"2H3C LAF"}="H3C2 LAF"; $resn_exch{"1H2C LAF"}="H2C1 LAF"; $resn_exch{"2H2C LAF"}="H2C2 LAF";
$resn_exch{"1HA6 LAG"}="HA61 LAG"; $resn_exch{"2HA6 LAG"}="HA62 LAG"; $resn_exch{"1HB6 LAG"}="HB61 LAG";
$resn_exch{"2HB6 LAG"}="HB62 LAG"; $resn_exch{"1HD6 LAG"}="HD61 LAG"; $resn_exch{"2HD6 LAG"}="HD62 LAG";
$resn_exch{"1H6  LAK"}=" H61 LAK"; $resn_exch{"2H6  LAK"}=" H62 LAK"; $resn_exch{"1H6' LAK"}="H6'1 LAK";
$resn_exch{"2H6' LAK"}="H6'2 LAK"; $resn_exch{"'HO1 LAK"}="HO1' LAK"; $resn_exch{"'HO2 LAK"}="HO2' LAK";
$resn_exch{"'HO3 LAK"}="HO3' LAK"; $resn_exch{"'HO4 LAK"}="HO4' LAK"; $resn_exch{"1HB  LAL"}=" HB1 LAL";
$resn_exch{"2HB  LAL"}=" HB2 LAL"; $resn_exch{"3HB  LAL"}=" HB3 LAL"; $resn_exch{"1HH1 LAL"}="HH11 LAL";
$resn_exch{"2HH1 LAL"}="HH12 LAL"; $resn_exch{"3HH1 LAL"}="HH13 LAL"; $resn_exch{"1HH2 LAL"}="HH21 LAL";
$resn_exch{"2HH2 LAL"}="HH22 LAL"; $resn_exch{"3HH2 LAL"}="HH23 LAL"; $resn_exch{"1H1  LAM"}=" H11 LAM";
$resn_exch{"1H2  LAM"}=" H21 LAM"; $resn_exch{"1HO2 LAM"}="HO21 LAM"; $resn_exch{"1H3  LAM"}=" H31 LAM";
$resn_exch{"1HO3 LAM"}="HO31 LAM"; $resn_exch{"1H4  LAM"}=" H41 LAM"; $resn_exch{"1HO4 LAM"}="HO41 LAM";
$resn_exch{"1H5  LAM"}=" H51 LAM"; $resn_exch{"1H6  LAM"}=" H61 LAM"; $resn_exch{"2H6  LAM"}=" H62 LAM";
$resn_exch{"1HO6 LAM"}="HO61 LAM"; $resn_exch{"1H1A LAM"}="H1A1 LAM"; $resn_exch{"1H2A LAM"}="H2A1 LAM";
$resn_exch{"1H3A LAM"}="H3A1 LAM"; $resn_exch{"1H4A LAM"}="H4A1 LAM"; $resn_exch{"1H5A LAM"}="H5A1 LAM";
$resn_exch{"1H6A LAM"}="H6A1 LAM"; $resn_exch{"2H6A LAM"}="H6A2 LAM"; $resn_exch{"1HA2 LAM"}="HA21 LAM";
$resn_exch{"1HA4 LAM"}="HA41 LAM"; $resn_exch{"1HA6 LAM"}="HA61 LAM"; $resn_exch{"1H1B LAM"}="H1B1 LAM";
$resn_exch{"1H2B LAM"}="H2B1 LAM"; $resn_exch{"1H3B LAM"}="H3B1 LAM"; $resn_exch{"1H4B LAM"}="H4B1 LAM";
$resn_exch{"1H5B LAM"}="H5B1 LAM"; $resn_exch{"1H6B LAM"}="H6B1 LAM"; $resn_exch{"2H6B LAM"}="H6B2 LAM";
$resn_exch{"1HB2 LAM"}="HB21 LAM"; $resn_exch{"1HB4 LAM"}="HB41 LAM"; $resn_exch{"1HB6 LAM"}="HB61 LAM";
$resn_exch{"1H2' LAM"}="H2'1 LAM"; $resn_exch{"1H3' LAM"}="H3'1 LAM"; $resn_exch{"1H5' LAM"}="H5'1 LAM";
$resn_exch{"1H6' LAM"}="H6'1 LAM"; $resn_exch{"1H5C LAN"}="H5C1 LAN"; $resn_exch{"2H5C LAN"}="H5C2 LAN";
$resn_exch{"1H4C LAN"}="H4C1 LAN"; $resn_exch{"2H4C LAN"}="H4C2 LAN"; $resn_exch{"1H26 LAN"}="H261 LAN";
$resn_exch{"2H26 LAN"}="H262 LAN"; $resn_exch{"3H26 LAN"}="H263 LAN"; $resn_exch{"1H27 LAN"}="H271 LAN";
$resn_exch{"2H27 LAN"}="H272 LAN"; $resn_exch{"3H27 LAN"}="H273 LAN"; $resn_exch{"1H30 LAN"}="H301 LAN";
$resn_exch{"2H30 LAN"}="H302 LAN"; $resn_exch{"1H31 LAN"}="H311 LAN"; $resn_exch{"2H31 LAN"}="H312 LAN";
$resn_exch{"1H2C LAN"}="H2C1 LAN"; $resn_exch{"2H2C LAN"}="H2C2 LAN"; $resn_exch{"3H2C LAN"}="H2C3 LAN";
$resn_exch{"1H23 LAN"}="H231 LAN"; $resn_exch{"2H23 LAN"}="H232 LAN"; $resn_exch{"1H22 LAN"}="H221 LAN";
$resn_exch{"2H22 LAN"}="H222 LAN"; $resn_exch{"1H11 LAN"}="H111 LAN"; $resn_exch{"2H11 LAN"}="H112 LAN";
$resn_exch{"3H11 LAN"}="H113 LAN"; $resn_exch{"1H8C LAN"}="H8C1 LAN"; $resn_exch{"2H8C LAN"}="H8C2 LAN";
$resn_exch{"3H8C LAN"}="H8C3 LAN"; $resn_exch{"1H9C LAN"}="H9C1 LAN"; $resn_exch{"2H9C LAN"}="H9C2 LAN";
$resn_exch{"1H21 LAN"}="H211 LAN"; $resn_exch{"2H21 LAN"}="H212 LAN"; $resn_exch{"1H20 LAN"}="H201 LAN";
$resn_exch{"2H20 LAN"}="H202 LAN"; $resn_exch{"3H20 LAN"}="H203 LAN"; $resn_exch{"1H14 LAN"}="H141 LAN";
$resn_exch{"2H14 LAN"}="H142 LAN"; $resn_exch{"1H15 LAN"}="H151 LAN"; $resn_exch{"2H15 LAN"}="H152 LAN";
$resn_exch{"1H19 LAN"}="H191 LAN"; $resn_exch{"2H19 LAN"}="H192 LAN"; $resn_exch{"3H19 LAN"}="H193 LAN";
$resn_exch{"1H18 LAN"}="H181 LAN"; $resn_exch{"2H18 LAN"}="H182 LAN"; $resn_exch{"3H18 LAN"}="H183 LAN";
$resn_exch{"1H2  LAP"}=" H21 LAP"; $resn_exch{"2H2  LAP"}=" H22 LAP"; $resn_exch{"1H3  LAP"}=" H31 LAP";
$resn_exch{"2H3  LAP"}=" H32 LAP"; $resn_exch{"1H4  LAP"}=" H41 LAP"; $resn_exch{"2H4  LAP"}=" H42 LAP";
$resn_exch{"1H5  LAP"}=" H51 LAP"; $resn_exch{"2H5  LAP"}=" H52 LAP"; $resn_exch{"1H6  LAP"}=" H61 LAP";
$resn_exch{"2H6  LAP"}=" H62 LAP"; $resn_exch{"1H7  LAP"}=" H71 LAP"; $resn_exch{"2H7  LAP"}=" H72 LAP";
$resn_exch{"1H8  LAP"}=" H81 LAP"; $resn_exch{"2H8  LAP"}=" H82 LAP"; $resn_exch{"1H9  LAP"}=" H91 LAP";
$resn_exch{"2H9  LAP"}=" H92 LAP"; $resn_exch{"1H10 LAP"}="H101 LAP"; $resn_exch{"2H10 LAP"}="H102 LAP";
$resn_exch{"1H11 LAP"}="H111 LAP"; $resn_exch{"2H11 LAP"}="H112 LAP"; $resn_exch{"1H12 LAP"}="H121 LAP";
$resn_exch{"2H12 LAP"}="H122 LAP"; $resn_exch{"3H12 LAP"}="H123 LAP"; $resn_exch{"1H13 LAP"}="H131 LAP";
$resn_exch{"2H13 LAP"}="H132 LAP"; $resn_exch{"1H15 LAP"}="H151 LAP"; $resn_exch{"2H15 LAP"}="H152 LAP";
$resn_exch{"1H16 LAP"}="H161 LAP"; $resn_exch{"2H16 LAP"}="H162 LAP"; $resn_exch{"1H17 LAP"}="H171 LAP";
$resn_exch{"2H17 LAP"}="H172 LAP"; $resn_exch{"1H18 LAP"}="H181 LAP"; $resn_exch{"2H18 LAP"}="H182 LAP";
$resn_exch{"3H18 LAP"}="H183 LAP"; $resn_exch{"1H19 LAP"}="H191 LAP"; $resn_exch{"2H19 LAP"}="H192 LAP";
$resn_exch{"3H19 LAP"}="H193 LAP"; $resn_exch{"1H20 LAP"}="H201 LAP"; $resn_exch{"2H20 LAP"}="H202 LAP";
$resn_exch{"3H20 LAP"}="H203 LAP"; $resn_exch{"1H11 LAQ"}="H111 LAQ"; $resn_exch{"2H11 LAQ"}="H112 LAQ";
$resn_exch{"1H12 LAQ"}="H121 LAQ"; $resn_exch{"2H12 LAQ"}="H122 LAQ"; $resn_exch{"1H13 LAQ"}="H131 LAQ";
$resn_exch{"2H13 LAQ"}="H132 LAQ"; $resn_exch{"1H14 LAQ"}="H141 LAQ"; $resn_exch{"2H14 LAQ"}="H142 LAQ";
$resn_exch{"1H17 LAQ"}="H171 LAQ"; $resn_exch{"2H17 LAQ"}="H172 LAQ"; $resn_exch{"1H16 LAQ"}="H161 LAQ";
$resn_exch{"2H16 LAQ"}="H162 LAQ"; $resn_exch{"1H5* LAQ"}="H5'1 LAQ"; $resn_exch{"2H5* LAQ"}="H5'2 LAQ";
$resn_exch{" H4* LAQ"}=" H4' LAQ"; $resn_exch{" H3* LAQ"}=" H3' LAQ"; $resn_exch{"HO3* LAQ"}="HO3' LAQ";
$resn_exch{" H2* LAQ"}=" H2' LAQ"; $resn_exch{"HO2* LAQ"}="HO2' LAQ"; $resn_exch{" H1* LAQ"}=" H1' LAQ";
$resn_exch{"1HN6 LAQ"}="HN61 LAQ"; $resn_exch{"2HN6 LAQ"}="HN62 LAQ"; $resn_exch{"1H2  LAR"}=" H21 LAR";
$resn_exch{"1H4  LAR"}=" H41 LAR"; $resn_exch{"2H4  LAR"}=" H42 LAR"; $resn_exch{"1H5  LAR"}=" H51 LAR";
$resn_exch{"2H5  LAR"}=" H52 LAR"; $resn_exch{"1H6  LAR"}=" H61 LAR"; $resn_exch{"1H7  LAR"}=" H71 LAR";
$resn_exch{"1H8  LAR"}=" H81 LAR"; $resn_exch{"1H9  LAR"}=" H91 LAR"; $resn_exch{"1H10 LAR"}="H101 LAR";
$resn_exch{"1H11 LAR"}="H111 LAR"; $resn_exch{"2H11 LAR"}="H112 LAR"; $resn_exch{"1H12 LAR"}="H121 LAR";
$resn_exch{"2H12 LAR"}="H122 LAR"; $resn_exch{"1H13 LAR"}="H131 LAR"; $resn_exch{"1H14 LAR"}="H141 LAR";
$resn_exch{"2H14 LAR"}="H142 LAR"; $resn_exch{"1H15 LAR"}="H151 LAR"; $resn_exch{"1H16 LAR"}="H161 LAR";
$resn_exch{"2H16 LAR"}="H162 LAR"; $resn_exch{"1H18 LAR"}="H181 LAR"; $resn_exch{"1H19 LAR"}="H191 LAR";
$resn_exch{"2H19 LAR"}="H192 LAR"; $resn_exch{"1H21 LAR"}="H211 LAR"; $resn_exch{"2H21 LAR"}="H212 LAR";
$resn_exch{"3H21 LAR"}="H213 LAR"; $resn_exch{"1H22 LAR"}="H221 LAR"; $resn_exch{"2H22 LAR"}="H222 LAR";
$resn_exch{"3H22 LAR"}="H223 LAR"; $resn_exch{"1H6  LAT"}=" H61 LAT"; $resn_exch{"2H6  LAT"}=" H62 LAT";
$resn_exch{"1H6' LAT"}="H6'1 LAT"; $resn_exch{"2H6' LAT"}="H6'2 LAT"; $resn_exch{"'HO1 LAT"}="HO1' LAT";
$resn_exch{"'HO2 LAT"}="HO2' LAT"; $resn_exch{"'HO3 LAT"}="HO3' LAT"; $resn_exch{"'HO6 LAT"}="HO6' LAT";
$resn_exch{"1H2  LAU"}=" H21 LAU"; $resn_exch{"2H2  LAU"}=" H22 LAU"; $resn_exch{"1H3  LAU"}=" H31 LAU";
$resn_exch{"2H3  LAU"}=" H32 LAU"; $resn_exch{"1H4  LAU"}=" H41 LAU"; $resn_exch{"2H4  LAU"}=" H42 LAU";
$resn_exch{"1H5  LAU"}=" H51 LAU"; $resn_exch{"2H5  LAU"}=" H52 LAU"; $resn_exch{"1H6  LAU"}=" H61 LAU";
$resn_exch{"2H6  LAU"}=" H62 LAU"; $resn_exch{"1H7  LAU"}=" H71 LAU"; $resn_exch{"2H7  LAU"}=" H72 LAU";
$resn_exch{"1H8  LAU"}=" H81 LAU"; $resn_exch{"2H8  LAU"}=" H82 LAU"; $resn_exch{"1H9  LAU"}=" H91 LAU";
$resn_exch{"2H9  LAU"}=" H92 LAU"; $resn_exch{"1H10 LAU"}="H101 LAU"; $resn_exch{"2H10 LAU"}="H102 LAU";
$resn_exch{"1H11 LAU"}="H111 LAU"; $resn_exch{"2H11 LAU"}="H112 LAU"; $resn_exch{"1H12 LAU"}="H121 LAU";
$resn_exch{"2H12 LAU"}="H122 LAU"; $resn_exch{"3H12 LAU"}="H123 LAU"; $resn_exch{"1H2  LAX"}=" H21 LAX";
$resn_exch{"2H2  LAX"}=" H22 LAX"; $resn_exch{"1H3  LAX"}=" H31 LAX"; $resn_exch{"2H3  LAX"}=" H32 LAX";
$resn_exch{"1H4  LAX"}=" H41 LAX"; $resn_exch{"2H4  LAX"}=" H42 LAX"; $resn_exch{"1H5  LAX"}=" H51 LAX";
$resn_exch{"2H5  LAX"}=" H52 LAX"; $resn_exch{"1H6  LAX"}=" H61 LAX"; $resn_exch{"2H6  LAX"}=" H62 LAX";
$resn_exch{"1H7  LAX"}=" H71 LAX"; $resn_exch{"2H7  LAX"}=" H72 LAX"; $resn_exch{"1H10 LAX"}="H101 LAX";
$resn_exch{"2H10 LAX"}="H102 LAX"; $resn_exch{"1H13 LAX"}="H131 LAX"; $resn_exch{"2H13 LAX"}="H132 LAX";
$resn_exch{"1H16 LAX"}="H161 LAX"; $resn_exch{"2H16 LAX"}="H162 LAX"; $resn_exch{"1H17 LAX"}="H171 LAX";
$resn_exch{"2H17 LAX"}="H172 LAX"; $resn_exch{"1H18 LAX"}="H181 LAX"; $resn_exch{"2H18 LAX"}="H182 LAX";
$resn_exch{"1H19 LAX"}="H191 LAX"; $resn_exch{"2H19 LAX"}="H192 LAX"; $resn_exch{"1H20 LAX"}="H201 LAX";
$resn_exch{"2H20 LAX"}="H202 LAX"; $resn_exch{"3H20 LAX"}="H203 LAX"; $resn_exch{"1H9C LAZ"}="H9C1 LAZ";
$resn_exch{"2H9C LAZ"}="H9C2 LAZ"; $resn_exch{"1H10 LAZ"}="H101 LAZ"; $resn_exch{"2H10 LAZ"}="H102 LAZ";
$resn_exch{"1H6C LBT"}="H6C1 LBT"; $resn_exch{"2H6C LBT"}="H6C2 LBT"; $resn_exch{"1H6' LBT"}="H6'1 LBT";
$resn_exch{"2H6' LBT"}="H6'2 LBT"; $resn_exch{"1HBC LBV"}="HBC1 LBV"; $resn_exch{"2HBC LBV"}="HBC2 LBV";
$resn_exch{"3HBC LBV"}="HBC3 LBV"; $resn_exch{"1HAC LBV"}="HAC1 LBV"; $resn_exch{"1HMC LBV"}="HMC1 LBV";
$resn_exch{"2HMC LBV"}="HMC2 LBV"; $resn_exch{"3HMC LBV"}="HMC3 LBV"; $resn_exch{"1HHD LBV"}="HHD1 LBV";
$resn_exch{"1HAD LBV"}="HAD1 LBV"; $resn_exch{"2HAD LBV"}="HAD2 LBV"; $resn_exch{"1HBD LBV"}="HBD1 LBV";
$resn_exch{"2HBD LBV"}="HBD2 LBV"; $resn_exch{"1HMD LBV"}="HMD1 LBV"; $resn_exch{"2HMD LBV"}="HMD2 LBV";
$resn_exch{"3HMD LBV"}="HMD3 LBV"; $resn_exch{"1HHA LBV"}="HHA1 LBV"; $resn_exch{"1HAA LBV"}="HAA1 LBV";
$resn_exch{"2HAA LBV"}="HAA2 LBV"; $resn_exch{"1HBA LBV"}="HBA1 LBV"; $resn_exch{"2HBA LBV"}="HBA2 LBV";
$resn_exch{"1HMA LBV"}="HMA1 LBV"; $resn_exch{"2HMA LBV"}="HMA2 LBV"; $resn_exch{"3HMA LBV"}="HMA3 LBV";
$resn_exch{"1HHB LBV"}="HHB1 LBV"; $resn_exch{"1HMB LBV"}="HMB1 LBV"; $resn_exch{"2HMB LBV"}="HMB2 LBV";
$resn_exch{"3HMB LBV"}="HMB3 LBV"; $resn_exch{"1HBB LBV"}="HBB1 LBV"; $resn_exch{"2HBB LBV"}="HBB2 LBV";
$resn_exch{"3HOP  LC"}="HOP3  LC"; $resn_exch{"2HOP  LC"}="HOP2  LC"; $resn_exch{"1H5*  LC"}=" H5'  LC";
$resn_exch{"2H5*  LC"}="H5''  LC"; $resn_exch{" H4*  LC"}=" H4'  LC"; $resn_exch{" H3*  LC"}=" H3'  LC";
$resn_exch{"H3T   LC"}="HO3'  LC"; $resn_exch{" H2*  LC"}=" H2'  LC"; $resn_exch{"2HO*  LC"}="HO2'  LC";
$resn_exch{" H1*  LC"}=" H1'  LC"; $resn_exch{"1H4   LC"}=" H41  LC"; $resn_exch{"2H4   LC"}=" H42  LC";
$resn_exch{"1H5* LCA"}="H5'1 LCA"; $resn_exch{"2H5* LCA"}="H5'2 LCA"; $resn_exch{" H3* LCA"}=" H3' LCA";
$resn_exch{"1H6* LCA"}="H6'1 LCA"; $resn_exch{"2H6* LCA"}="H6'2 LCA"; $resn_exch{"1H2* LCA"}="H2'1 LCA";
$resn_exch{"1H6  LCA"}=" H61 LCA"; $resn_exch{"2H6  LCA"}=" H62 LCA"; $resn_exch{" H1* LCA"}=" H1' LCA";
$resn_exch{"1H5* LCC"}="H5'1 LCC"; $resn_exch{"2H5* LCC"}="H5'2 LCC"; $resn_exch{" H1* LCC"}=" H1' LCC";
$resn_exch{"1H5M LCC"}="H5M1 LCC"; $resn_exch{"2H5M LCC"}="H5M2 LCC"; $resn_exch{"3H5M LCC"}="H5M3 LCC";
$resn_exch{"1H4  LCC"}=" H41 LCC"; $resn_exch{"2H4  LCC"}=" H42 LCC"; $resn_exch{" H3* LCC"}=" H3' LCC";
$resn_exch{"1H2* LCC"}="H2'1 LCC"; $resn_exch{"1H6* LCC"}="H6'1 LCC"; $resn_exch{"2H6* LCC"}="H6'2 LCC";
$resn_exch{"1H2  LCF"}=" H21 LCF"; $resn_exch{"2H2  LCF"}="H22A LCF"; $resn_exch{"1H4  LCF"}=" H41 LCF";
$resn_exch{"2H4  LCF"}=" H42 LCF"; $resn_exch{"1H8  LCF"}=" H81 LCF"; $resn_exch{"2H8  LCF"}=" H82 LCF";
$resn_exch{"1H10 LCF"}="H101 LCF"; $resn_exch{"2H10 LCF"}="H102 LCF"; $resn_exch{"3H10 LCF"}="H103 LCF";
$resn_exch{"1H11 LCF"}="H111 LCF"; $resn_exch{"2H11 LCF"}="H112 LCF"; $resn_exch{"3H11 LCF"}="H113 LCF";
$resn_exch{"1H5* LCG"}=" H5' LCG"; $resn_exch{"2H5* LCG"}="H5'' LCG"; $resn_exch{" H3* LCG"}=" H3' LCG";
$resn_exch{"1H6* LCG"}="H6'1 LCG"; $resn_exch{"2H6* LCG"}="H6'2 LCG"; $resn_exch{"1H2* LCG"}=" H2' LCG";
$resn_exch{" H1* LCG"}=" H1' LCG"; $resn_exch{"2HOP LCG"}="HOP2 LCG"; $resn_exch{"1H2  LCG"}=" H21 LCG";
$resn_exch{"2H2  LCG"}=" H22 LCG"; $resn_exch{"H3T  LCG"}="HO3' LCG"; $resn_exch{"3HOP LCG"}="HOP3 LCG";
$resn_exch{"1H5* LCH"}="H5'1 LCH"; $resn_exch{"2H5* LCH"}="H5'2 LCH"; $resn_exch{" H1* LCH"}=" H1' LCH";
$resn_exch{"1H5M LCH"}="H5M1 LCH"; $resn_exch{"2H5M LCH"}="H5M2 LCH"; $resn_exch{"3H5M LCH"}="H5M3 LCH";
$resn_exch{"1H4  LCH"}=" H41 LCH"; $resn_exch{"2H4  LCH"}=" H42 LCH"; $resn_exch{" H3* LCH"}=" H3' LCH";
$resn_exch{"1H2* LCH"}="H2'1 LCH"; $resn_exch{"1H6* LCH"}="H6'1 LCH"; $resn_exch{"2H6* LCH"}="H6'2 LCH";
$resn_exch{"1H2A LCS"}="H2A1 LCS"; $resn_exch{"2H2A LCS"}="H2A2 LCS"; $resn_exch{"3H2A LCS"}="H2A3 LCS";
$resn_exch{"1H4A LCS"}="H4A1 LCS"; $resn_exch{"2H4A LCS"}="H4A2 LCS"; $resn_exch{"1H5A LCS"}="H5A1 LCS";
$resn_exch{"2H5A LCS"}="H5A2 LCS"; $resn_exch{"2HOP LCS"}="HOP2 LCS"; $resn_exch{"3HOP LCS"}="HOP3 LCS";
$resn_exch{"1HB  LCS"}=" HB1 LCS"; $resn_exch{"2HB  LCS"}=" HB2 LCS"; $resn_exch{"1HN  LCX"}=" H   LCX";
$resn_exch{"2HN  LCX"}=" H2  LCX"; $resn_exch{"1H   LCX"}=" HC1 LCX"; $resn_exch{"2H   LCX"}=" HC2 LCX";
$resn_exch{"1HB  LCX"}=" HB2 LCX"; $resn_exch{"2HB  LCX"}=" HB3 LCX"; $resn_exch{"1HG  LCX"}=" HG2 LCX";
$resn_exch{"2HG  LCX"}=" HG3 LCX"; $resn_exch{"1HD  LCX"}=" HD2 LCX"; $resn_exch{"2HD  LCX"}=" HD3 LCX";
$resn_exch{"1HE  LCX"}=" HE2 LCX"; $resn_exch{"2HE  LCX"}=" HE3 LCX"; $resn_exch{" HNZ LCX"}=" HZ  LCX";
$resn_exch{"1HM1 LDA"}="HM11 LDA"; $resn_exch{"2HM1 LDA"}="HM12 LDA"; $resn_exch{"3HM1 LDA"}="HM13 LDA";
$resn_exch{"1HM2 LDA"}="HM21 LDA"; $resn_exch{"2HM2 LDA"}="HM22 LDA"; $resn_exch{"3HM2 LDA"}="HM23 LDA";
$resn_exch{"1H1  LDA"}=" H11 LDA"; $resn_exch{"2H1  LDA"}=" H12 LDA"; $resn_exch{"1H2  LDA"}=" H21 LDA";
$resn_exch{"2H2  LDA"}=" H22 LDA"; $resn_exch{"1H3  LDA"}=" H31 LDA"; $resn_exch{"2H3  LDA"}=" H32 LDA";
$resn_exch{"1H4  LDA"}=" H41 LDA"; $resn_exch{"2H4  LDA"}=" H42 LDA"; $resn_exch{"1H5  LDA"}=" H51 LDA";
$resn_exch{"2H5  LDA"}=" H52 LDA"; $resn_exch{"1H6  LDA"}=" H61 LDA"; $resn_exch{"2H6  LDA"}=" H62 LDA";
$resn_exch{"1H7  LDA"}=" H71 LDA"; $resn_exch{"2H7  LDA"}=" H72 LDA"; $resn_exch{"1H8  LDA"}=" H81 LDA";
$resn_exch{"2H8  LDA"}=" H82 LDA"; $resn_exch{"1H9  LDA"}=" H91 LDA"; $resn_exch{"2H9  LDA"}=" H92 LDA";
$resn_exch{"1H10 LDA"}="H101 LDA"; $resn_exch{"2H10 LDA"}="H102 LDA"; $resn_exch{"1H11 LDA"}="H111 LDA";
$resn_exch{"2H11 LDA"}="H112 LDA"; $resn_exch{"1H12 LDA"}="H121 LDA"; $resn_exch{"2H12 LDA"}="H122 LDA";
$resn_exch{"3H12 LDA"}="H123 LDA"; $resn_exch{"1HN4 LDC"}="HN41 LDC"; $resn_exch{"2HN4 LDC"}="HN42 LDC";
$resn_exch{"1H9  LDC"}=" H91 LDC"; $resn_exch{"2H9  LDC"}=" H92 LDC"; $resn_exch{"1H12 LDC"}="H121 LDC";
$resn_exch{"2H12 LDC"}="H122 LDC"; $resn_exch{"1HN  LDH"}=" HN1 LDH"; $resn_exch{"2HN  LDH"}=" HN2 LDH";
$resn_exch{"1HB  LDH"}=" HB1 LDH"; $resn_exch{"2HB  LDH"}=" HB2 LDH"; $resn_exch{"1HG  LDH"}=" HG1 LDH";
$resn_exch{"2HG  LDH"}=" HG2 LDH"; $resn_exch{"1HD  LDH"}=" HD1 LDH"; $resn_exch{"2HD  LDH"}=" HD2 LDH";
$resn_exch{"1HE  LDH"}=" HE1 LDH"; $resn_exch{"2HE  LDH"}=" HE2 LDH"; $resn_exch{"1HH  LDH"}=" HH1 LDH";
$resn_exch{"2HH  LDH"}=" HH2 LDH"; $resn_exch{"1H1  LDH"}=" H11 LDH"; $resn_exch{"2H1  LDH"}=" H12 LDH";
$resn_exch{"3H1  LDH"}=" H13 LDH"; $resn_exch{"1H1C LDM"}="H1C1 LDM"; $resn_exch{"2H1C LDM"}="H1C2 LDM";
$resn_exch{"1H3C LDM"}="H3C1 LDM"; $resn_exch{"2H3C LDM"}="H3C2 LDM"; $resn_exch{"1H5C LDM"}="H5C1 LDM";
$resn_exch{"2H5C LDM"}="H5C2 LDM"; $resn_exch{"1H7C LDM"}="H7C1 LDM"; $resn_exch{"2H7C LDM"}="H7C2 LDM";
$resn_exch{"1H8C LDM"}="H8C1 LDM"; $resn_exch{"2H8C LDM"}="H8C2 LDM"; $resn_exch{"1H10 LDM"}="H101 LDM";
$resn_exch{"2H10 LDM"}="H102 LDM"; $resn_exch{"1H11 LDM"}="H111 LDM"; $resn_exch{"2H11 LDM"}="H112 LDM";
$resn_exch{"1H12 LDM"}="H121 LDM"; $resn_exch{"2H12 LDM"}="H122 LDM"; $resn_exch{"1HM1 LDM"}="HM11 LDM";
$resn_exch{"2HM1 LDM"}="HM12 LDM"; $resn_exch{"3HM1 LDM"}="HM13 LDM"; $resn_exch{"1HM2 LDM"}="HM21 LDM";
$resn_exch{"2HM2 LDM"}="HM22 LDM"; $resn_exch{"3HM2 LDM"}="HM23 LDM"; $resn_exch{"1H2C LDM"}="H2C1 LDM";
$resn_exch{"2H2C LDM"}="H2C2 LDM"; $resn_exch{"1H9C LDM"}="H9C1 LDM"; $resn_exch{"2H9C LDM"}="H9C2 LDM";
$resn_exch{"1H14 LDM"}="H141 LDM"; $resn_exch{"2H14 LDM"}="H142 LDM"; $resn_exch{"1H15 LDM"}="H151 LDM";
$resn_exch{"2H15 LDM"}="H152 LDM"; $resn_exch{"3H15 LDM"}="H153 LDM"; $resn_exch{"1H6C LDM"}="H6C1 LDM";
$resn_exch{"2H6C LDM"}="H6C2 LDM"; $resn_exch{"1H13 LDM"}="H131 LDM"; $resn_exch{"2H13 LDM"}="H132 LDM";
$resn_exch{"1HN  LDO"}=" HN1 LDO"; $resn_exch{"2HN  LDO"}=" HN2 LDO"; $resn_exch{"1HA  LDO"}=" HA1 LDO";
$resn_exch{"1HB  LDO"}=" HB1 LDO"; $resn_exch{"2HB  LDO"}=" HB2 LDO"; $resn_exch{"1HG  LDO"}=" HG1 LDO";
$resn_exch{"2HG  LDO"}=" HG2 LDO"; $resn_exch{"1HD  LDO"}=" HD1 LDO"; $resn_exch{"2HD  LDO"}=" HD2 LDO";
$resn_exch{"1HE  LDO"}=" HE1 LDO"; $resn_exch{"2HE  LDO"}=" HE2 LDO"; $resn_exch{"1H7  LDP"}=" H71 LDP";
$resn_exch{"2H7  LDP"}=" H72 LDP"; $resn_exch{"1H8  LDP"}=" H81 LDP"; $resn_exch{"2H8  LDP"}=" H82 LDP";
$resn_exch{"1HN1 LDP"}="HN11 LDP"; $resn_exch{"2HN1 LDP"}="HN12 LDP"; $resn_exch{"1H13 LDT"}="H131 LDT";
$resn_exch{"2H13 LDT"}="H132 LDT"; $resn_exch{"1H20 LDT"}="H201 LDT"; $resn_exch{"2H20 LDT"}="H202 LDT";
$resn_exch{"1H3  LEA"}=" H31 LEA"; $resn_exch{"2H3  LEA"}=" H32 LEA"; $resn_exch{"1H4  LEA"}=" H41 LEA";
$resn_exch{"2H4  LEA"}=" H42 LEA"; $resn_exch{"1H5  LEA"}=" H51 LEA"; $resn_exch{"2H5  LEA"}=" H52 LEA";
$resn_exch{"1H6  LEA"}=" H61 LEA"; $resn_exch{"2H6  LEA"}=" H62 LEA"; $resn_exch{"3H6  LEA"}=" H63 LEA";
$resn_exch{"1HN  LED"}=" HN1 LED"; $resn_exch{"2HN  LED"}=" HN2 LED"; $resn_exch{"1HBC LED"}="HBC1 LED";
$resn_exch{"2HBC LED"}="HBC2 LED"; $resn_exch{"1HD1 LED"}="HD11 LED"; $resn_exch{"2HD1 LED"}="HD12 LED";
$resn_exch{"3HD1 LED"}="HD13 LED"; $resn_exch{"1HD1 LEF"}="HD12 LEF"; $resn_exch{"2HD1 LEF"}="HD13 LEF";
$resn_exch{"1HD2 LEF"}="HD21 LEF"; $resn_exch{"2HD2 LEF"}="HD22 LEF"; $resn_exch{"3HD2 LEF"}="HD23 LEF";
$resn_exch{"1HBC LEF"}=" HB3 LEF"; $resn_exch{"2HBC LEF"}=" HB2 LEF"; $resn_exch{"1HN  LEF"}=" H   LEF";
$resn_exch{"2HN  LEF"}=" H2  LEF"; $resn_exch{"1H27 LEG"}="H271 LEG"; $resn_exch{"2H27 LEG"}="H272 LEG";
$resn_exch{"3H27 LEG"}="H273 LEG"; $resn_exch{"1H24 LEG"}="H241 LEG"; $resn_exch{"2H24 LEG"}="H242 LEG";
$resn_exch{"1H21 LEG"}="H211 LEG"; $resn_exch{"2H21 LEG"}="H212 LEG"; $resn_exch{"1H18 LEG"}="H181 LEG";
$resn_exch{"2H18 LEG"}="H182 LEG"; $resn_exch{"1H15 LEG"}="H151 LEG"; $resn_exch{"2H15 LEG"}="H152 LEG";
$resn_exch{"1H14 LEG"}="H141 LEG"; $resn_exch{"2H14 LEG"}="H142 LEG"; $resn_exch{"1H33 LEG"}="H331 LEG";
$resn_exch{"2H33 LEG"}="H332 LEG"; $resn_exch{"1HD2 LEN"}="HD21 LEN"; $resn_exch{"2HD2 LEN"}="HD22 LEN";
$resn_exch{"3HD2 LEN"}="HD23 LEN"; $resn_exch{"1HD1 LEN"}="HD11 LEN"; $resn_exch{"2HD1 LEN"}="HD12 LEN";
$resn_exch{"3HD1 LEN"}="HD13 LEN"; $resn_exch{"1HB  LEN"}=" HB1 LEN"; $resn_exch{"2HB  LEN"}=" HB2 LEN";
$resn_exch{"1HA  LEN"}=" HA1 LEN"; $resn_exch{"2HA  LEN"}=" HA2 LEN"; $resn_exch{"1HN  LEN"}=" HN1 LEN";
$resn_exch{"2HN  LEN"}=" HN2 LEN"; $resn_exch{"1H2  LEO"}=" H21 LEO"; $resn_exch{"2H2  LEO"}=" H22 LEO";
$resn_exch{"1H4  LEO"}=" H41 LEO"; $resn_exch{"2H4  LEO"}=" H42 LEO"; $resn_exch{"1H5  LEO"}=" H51 LEO";
$resn_exch{"2H5  LEO"}=" H52 LEO"; $resn_exch{"1H8  LEO"}=" H81 LEO"; $resn_exch{"2H8  LEO"}=" H82 LEO";
$resn_exch{"3H8  LEO"}=" H83 LEO"; $resn_exch{"1H21 LEO"}="H211 LEO"; $resn_exch{"2H21 LEO"}="H212 LEO";
$resn_exch{"1H22 LEO"}="H221 LEO"; $resn_exch{"2H22 LEO"}="H222 LEO"; $resn_exch{"3H22 LEO"}="H223 LEO";
$resn_exch{"1H10 LET"}="H101 LET"; $resn_exch{"2H10 LET"}="H102 LET"; $resn_exch{"1H9C LET"}="H9C1 LET";
$resn_exch{"2H9C LET"}="H9C2 LET"; $resn_exch{"1H7C LET"}="H7C1 LET"; $resn_exch{"2H7C LET"}="H7C2 LET";
$resn_exch{"1H5C LET"}="H5C1 LET"; $resn_exch{"2H5C LET"}="H5C2 LET"; $resn_exch{"1H3C LET"}="H3C1 LET";
$resn_exch{"2H3C LET"}="H3C2 LET"; $resn_exch{"1H2C LET"}="H2C1 LET"; $resn_exch{"2H2C LET"}="H2C2 LET";
$resn_exch{"1HEC LET"}="HEC1 LET"; $resn_exch{"2HEC LET"}="HEC2 LET"; $resn_exch{"1HDC LET"}="HDC1 LET";
$resn_exch{"2HDC LET"}="HDC2 LET"; $resn_exch{"1HGC LET"}="HGC1 LET"; $resn_exch{"2HGC LET"}="HGC2 LET";
$resn_exch{"1HBC LET"}="HBC1 LET"; $resn_exch{"2HBC LET"}="HBC2 LET"; $resn_exch{"1HN  LET"}=" HN1 LET";
$resn_exch{"2HN  LET"}=" HN2 LET"; $resn_exch{" HN2 LEU"}=" H2  LEU"; $resn_exch{"1HB  LEU"}=" HB2 LEU";
$resn_exch{"2HB  LEU"}=" HB3 LEU"; $resn_exch{"1HD1 LEU"}="HD11 LEU"; $resn_exch{"2HD1 LEU"}="HD12 LEU";
$resn_exch{"3HD1 LEU"}="HD13 LEU"; $resn_exch{"1HD2 LEU"}="HD21 LEU"; $resn_exch{"2HD2 LEU"}="HD22 LEU";
$resn_exch{"3HD2 LEU"}="HD23 LEU"; $resn_exch{"1H1  LFA"}=" H11 LFA"; $resn_exch{"2H1  LFA"}=" H12 LFA";
$resn_exch{"3H1  LFA"}=" H13 LFA"; $resn_exch{"1H2  LFA"}=" H21 LFA"; $resn_exch{"2H2  LFA"}=" H22 LFA";
$resn_exch{"1H3  LFA"}=" H31 LFA"; $resn_exch{"2H3  LFA"}=" H32 LFA"; $resn_exch{"1H4  LFA"}=" H41 LFA";
$resn_exch{"2H4  LFA"}=" H42 LFA"; $resn_exch{"1H5  LFA"}=" H51 LFA"; $resn_exch{"2H5  LFA"}=" H52 LFA";
$resn_exch{"1H6  LFA"}=" H61 LFA"; $resn_exch{"2H6  LFA"}=" H62 LFA"; $resn_exch{"1H7  LFA"}=" H71 LFA";
$resn_exch{"2H7  LFA"}=" H72 LFA"; $resn_exch{"1H8  LFA"}=" H81 LFA"; $resn_exch{"2H8  LFA"}=" H82 LFA";
$resn_exch{"1H9  LFA"}=" H91 LFA"; $resn_exch{"2H9  LFA"}=" H92 LFA"; $resn_exch{"1H10 LFA"}="H101 LFA";
$resn_exch{"2H10 LFA"}="H102 LFA"; $resn_exch{"1H11 LFA"}="H111 LFA"; $resn_exch{"2H11 LFA"}="H112 LFA";
$resn_exch{"1H12 LFA"}="H121 LFA"; $resn_exch{"2H12 LFA"}="H122 LFA"; $resn_exch{"1H13 LFA"}="H131 LFA";
$resn_exch{"2H13 LFA"}="H132 LFA"; $resn_exch{"1H14 LFA"}="H141 LFA"; $resn_exch{"2H14 LFA"}="H142 LFA";
$resn_exch{"1H15 LFA"}="H151 LFA"; $resn_exch{"2H15 LFA"}="H152 LFA"; $resn_exch{"1H16 LFA"}="H161 LFA";
$resn_exch{"2H16 LFA"}="H162 LFA"; $resn_exch{"1H17 LFA"}="H171 LFA"; $resn_exch{"2H17 LFA"}="H172 LFA";
$resn_exch{"1H18 LFA"}="H181 LFA"; $resn_exch{"2H18 LFA"}="H182 LFA"; $resn_exch{"1H19 LFA"}="H191 LFA";
$resn_exch{"2H19 LFA"}="H192 LFA"; $resn_exch{"1H20 LFA"}="H201 LFA"; $resn_exch{"2H20 LFA"}="H202 LFA";
$resn_exch{"3H20 LFA"}="H203 LFA"; $resn_exch{"1H6  LFC"}=" H61 LFC"; $resn_exch{"2H6  LFC"}=" H62 LFC";
$resn_exch{"3H6  LFC"}=" H63 LFC"; $resn_exch{"1H7M LFN"}="H7M1 LFN"; $resn_exch{"2H7M LFN"}="H7M2 LFN";
$resn_exch{"3H7M LFN"}="H7M3 LFN"; $resn_exch{"1H8M LFN"}="H8M1 LFN"; $resn_exch{"2H8M LFN"}="H8M2 LFN";
$resn_exch{"3H8M LFN"}="H8M3 LFN"; $resn_exch{"1H1* LFN"}="H1'1 LFN"; $resn_exch{"2H1* LFN"}="H1'2 LFN";
$resn_exch{"3H1* LFN"}="H1'3 LFN"; $resn_exch{"1H1C LFR"}="H1C1 LFR"; $resn_exch{"2H1C LFR"}="H1C2 LFR";
$resn_exch{"1H6C LFR"}="H6C1 LFR"; $resn_exch{"2H6C LFR"}="H6C2 LFR"; $resn_exch{"3HOP  LG"}="HOP3  LG";
$resn_exch{"2HOP  LG"}="HOP2  LG"; $resn_exch{"1H5*  LG"}=" H5'  LG"; $resn_exch{"2H5*  LG"}="H5''  LG";
$resn_exch{" H4*  LG"}=" H4'  LG"; $resn_exch{" H3*  LG"}=" H3'  LG"; $resn_exch{"H3T   LG"}="HO3'  LG";
$resn_exch{" H2*  LG"}=" H2'  LG"; $resn_exch{"2HO*  LG"}="HO2'  LG"; $resn_exch{" H1*  LG"}=" H1'  LG";
$resn_exch{"1H2   LG"}=" H21  LG"; $resn_exch{"2H2   LG"}=" H22  LG"; $resn_exch{"2HC  LG2"}=" HC2 LG2";
$resn_exch{"5HC  LG2"}=" HC5 LG2"; $resn_exch{"1HC8 LG2"}="HC81 LG2"; $resn_exch{"2HC8 LG2"}="HC82 LG2";
$resn_exch{"1HC9 LG2"}="HC91 LG2"; $resn_exch{"2HC9 LG2"}="HC92 LG2"; $resn_exch{"1HC4 LG2"}="HC41 LG2";
$resn_exch{"1HC6 LG2"}="HC61 LG2"; $resn_exch{"1HC7 LG2"}="HC71 LG2"; $resn_exch{"2HO0 LG2"}="HO02 LG2";
$resn_exch{"1H21 LG2"}="H211 LG2"; $resn_exch{"2H21 LG2"}="H212 LG2"; $resn_exch{"1H22 LG2"}="H221 LG2";
$resn_exch{"2H22 LG2"}="H222 LG2"; $resn_exch{"1H23 LG2"}="H231 LG2"; $resn_exch{"2H23 LG2"}="H232 LG2";
$resn_exch{"3H23 LG2"}="H233 LG2"; $resn_exch{"1H24 LG2"}="H241 LG2"; $resn_exch{"2H24 LG2"}="H242 LG2";
$resn_exch{"3H24 LG2"}="H243 LG2"; $resn_exch{"1H25 LG2"}="H251 LG2"; $resn_exch{"2H25 LG2"}="H252 LG2";
$resn_exch{"3H25 LG2"}="H253 LG2"; $resn_exch{"1H26 LG2"}="H261 LG2"; $resn_exch{"2H26 LG2"}="H262 LG2";
$resn_exch{"3H26 LG2"}="H263 LG2"; $resn_exch{"1H27 LG2"}="H271 LG2"; $resn_exch{"2H27 LG2"}="H272 LG2";
$resn_exch{"3H27 LG2"}="H273 LG2"; $resn_exch{"1HN7 LG3"}="HN71 LG3"; $resn_exch{"2HN7 LG3"}="HN72 LG3";
$resn_exch{"1HN8 LG3"}="HN81 LG3"; $resn_exch{"2HN8 LG3"}="HN82 LG3"; $resn_exch{"1HN7 LG4"}="HN71 LG4";
$resn_exch{"2HN7 LG4"}="HN72 LG4"; $resn_exch{"1H8  LG4"}=" H81 LG4"; $resn_exch{"2H8  LG4"}=" H82 LG4";
$resn_exch{"3H8  LG4"}=" H83 LG4"; $resn_exch{"1H2  LG5"}=" H21 LG5"; $resn_exch{"2H2  LG5"}=" H22 LG5";
$resn_exch{"1H7  LG5"}=" H71 LG5"; $resn_exch{"2H7  LG5"}=" H72 LG5"; $resn_exch{"3H7  LG5"}=" H73 LG5";
$resn_exch{"1HN8 LG5"}="HN81 LG5"; $resn_exch{"2HN8 LG5"}="HN82 LG5"; $resn_exch{"1H6  LG6"}=" H61 LG6";
$resn_exch{"2H6  LG6"}=" H62 LG6"; $resn_exch{"2HOP LG6"}="HOP2 LG6"; $resn_exch{"3HOP LG6"}="HOP3 LG6";
$resn_exch{"1H3  LG7"}=" H31 LG7"; $resn_exch{"2H3  LG7"}=" H32 LG7"; $resn_exch{"1H4  LG7"}=" H41 LG7";
$resn_exch{"2H4  LG7"}=" H42 LG7"; $resn_exch{"1HBD LG8"}="HBD1 LG8"; $resn_exch{"2HBD LG8"}="HBD2 LG8";
$resn_exch{"1HBB LG8"}="HBB1 LG8"; $resn_exch{"2HBB LG8"}="HBB2 LG8"; $resn_exch{"1H6C LG8"}="H6C1 LG8";
$resn_exch{"2H6C LG8"}="H6C2 LG8"; $resn_exch{"1HBA LG8"}="HBA1 LG8"; $resn_exch{"2HBA LG8"}="HBA2 LG8";
$resn_exch{"3HBA LG8"}="HBA3 LG8"; $resn_exch{"1HBG LG8"}="HBG1 LG8"; $resn_exch{"2HBG LG8"}="HBG2 LG8";
$resn_exch{"1HBJ LG8"}="HBJ1 LG8"; $resn_exch{"2HBJ LG8"}="HBJ2 LG8"; $resn_exch{"3HBJ LG8"}="HBJ3 LG8";
$resn_exch{"1HBI LG8"}="HBI1 LG8"; $resn_exch{"2HBI LG8"}="HBI2 LG8"; $resn_exch{"3HBI LG8"}="HBI3 LG8";
$resn_exch{"1HC6 LGC"}="HC61 LGC"; $resn_exch{"2HC6 LGC"}="HC62 LGC"; $resn_exch{"1H12 LGD"}="H121 LGD";
$resn_exch{"2H12 LGD"}="H122 LGD"; $resn_exch{"1H14 LGD"}="H141 LGD"; $resn_exch{"2H14 LGD"}="H142 LGD";
$resn_exch{"1H4  LGF"}=" H41 LGF"; $resn_exch{"2H4  LGF"}=" H42 LGF"; $resn_exch{"1H2  LGF"}=" H21 LGF";
$resn_exch{"2H2  LGF"}=" H22 LGF"; $resn_exch{"1H1  LGF"}=" H11 LGF"; $resn_exch{"2H1  LGF"}="H12A LGF";
$resn_exch{"1H35 LGF"}="H351 LGF"; $resn_exch{"2H35 LGF"}="H352 LGF"; $resn_exch{"3H35 LGF"}="H353 LGF";
$resn_exch{"1H36 LGF"}="H361 LGF"; $resn_exch{"2H36 LGF"}="H362 LGF"; $resn_exch{"1H38 LGF"}="H381 LGF";
$resn_exch{"2H38 LGF"}="H382 LGF"; $resn_exch{"1H39 LGF"}="H391 LGF"; $resn_exch{"2H39 LGF"}="H392 LGF";
$resn_exch{"1H17 LGG"}="H171 LGG"; $resn_exch{"2H17 LGG"}="H172 LGG"; $resn_exch{"1H20 LGG"}="H201 LGG";
$resn_exch{"2H20 LGG"}="H202 LGG"; $resn_exch{"3H20 LGG"}="H203 LGG"; $resn_exch{"1H21 LGG"}="H211 LGG";
$resn_exch{"2H21 LGG"}="H212 LGG"; $resn_exch{"3H21 LGG"}="H213 LGG"; $resn_exch{"1H28 LGG"}="H281 LGG";
$resn_exch{"2H28 LGG"}="H282 LGG"; $resn_exch{"1H12 LGG"}="H121 LGG"; $resn_exch{"2H12 LGG"}="H122 LGG";
$resn_exch{"1H3  LGG"}=" H31 LGG"; $resn_exch{"2H3  LGG"}=" H32 LGG"; $resn_exch{"1H2  LGG"}=" H21 LGG";
$resn_exch{"2H2  LGG"}="H22A LGG"; $resn_exch{"1H34 LGG"}="H341 LGG"; $resn_exch{"2H34 LGG"}="H342 LGG";
$resn_exch{"3H34 LGG"}="H343 LGG"; $resn_exch{"1H16 LGG"}="H161 LGG"; $resn_exch{"2H16 LGG"}="H162 LGG";
$resn_exch{"1HN4 LGG"}="HN41 LGG"; $resn_exch{"2HN4 LGG"}="HN42 LGG"; $resn_exch{"1H23 LGG"}="H231 LGG";
$resn_exch{"2H23 LGG"}="H232 LGG"; $resn_exch{"1H30 LGG"}="H301 LGG"; $resn_exch{"2H30 LGG"}="H302 LGG";
$resn_exch{"1H31 LGG"}="H311 LGG"; $resn_exch{"2H31 LGG"}="H312 LGG"; $resn_exch{"1H27 LGG"}="H271 LGG";
$resn_exch{"2H27 LGG"}="H272 LGG"; $resn_exch{"1HN7 LGG"}="HN71 LGG"; $resn_exch{"2HN7 LGG"}="HN72 LGG";
$resn_exch{"2HOP LGP"}="HOP2 LGP"; $resn_exch{"3HOP LGP"}="HOP3 LGP"; $resn_exch{"1H5* LGP"}=" H5' LGP";
$resn_exch{"2H5* LGP"}="H5'' LGP"; $resn_exch{" H4* LGP"}=" H4' LGP"; $resn_exch{"1H3* LGP"}="H3'1 LGP";
$resn_exch{"2H3* LGP"}="H3'2 LGP"; $resn_exch{"*HO3 LGP"}="HO3' LGP"; $resn_exch{"1H1* LGP"}="H1'1 LGP";
$resn_exch{"2H1* LGP"}="H1'2 LGP"; $resn_exch{"1HN2 LGP"}="HN21 LGP"; $resn_exch{"2HN2 LGP"}="HN22 LGP";
$resn_exch{"1H21 LHA"}="H211 LHA"; $resn_exch{"2H21 LHA"}="H212 LHA"; $resn_exch{"1H61 LHA"}="H611 LHA";
$resn_exch{"2H61 LHA"}="H612 LHA"; $resn_exch{"1H22 LHA"}="H221 LHA"; $resn_exch{"2H22 LHA"}="H222 LHA";
$resn_exch{"1H32 LHA"}="H321 LHA"; $resn_exch{"2H32 LHA"}="H322 LHA"; $resn_exch{"1H53 LHA"}="H531 LHA";
$resn_exch{"2H53 LHA"}="H532 LHA"; $resn_exch{"1H24 LHA"}="H241 LHA"; $resn_exch{"2H24 LHA"}="H242 LHA";
$resn_exch{"1H64 LHA"}="H641 LHA"; $resn_exch{"2H64 LHA"}="H642 LHA"; $resn_exch{"1HN4 LHA"}="HN41 LHA";
$resn_exch{"2HN4 LHA"}="HN42 LHA"; $resn_exch{"1H15 LHA"}="H151 LHA"; $resn_exch{"2H15 LHA"}="H152 LHA";
$resn_exch{"1H25 LHA"}="H251 LHA"; $resn_exch{"2H25 LHA"}="H252 LHA"; $resn_exch{"1H35 LHA"}="H351 LHA";
$resn_exch{"2H35 LHA"}="H352 LHA"; $resn_exch{"1H45 LHA"}="H451 LHA"; $resn_exch{"2H45 LHA"}="H452 LHA";
$resn_exch{"1H37 LHA"}="H371 LHA"; $resn_exch{"2H37 LHA"}="H372 LHA"; $resn_exch{"1H47 LHA"}="H471 LHA";
$resn_exch{"2H47 LHA"}="H472 LHA"; $resn_exch{"1HN7 LHA"}="HN71 LHA"; $resn_exch{"2HN7 LHA"}="HN72 LHA";
$resn_exch{"1HC1 LHG"}="HC11 LHG"; $resn_exch{"2HC1 LHG"}="HC12 LHG"; $resn_exch{"1HC3 LHG"}="HC31 LHG";
$resn_exch{"2HC3 LHG"}="HC32 LHG"; $resn_exch{"1HC4 LHG"}="HC41 LHG"; $resn_exch{"2HC4 LHG"}="HC42 LHG";
$resn_exch{"1HC6 LHG"}="HC61 LHG"; $resn_exch{"2HC6 LHG"}="HC62 LHG"; $resn_exch{"1HC8 LHG"}="HC81 LHG";
$resn_exch{"2HC8 LHG"}="HC82 LHG"; $resn_exch{"1HC9 LHG"}="HC91 LHG"; $resn_exch{"2HC9 LHG"}="HC92 LHG";
$resn_exch{"1H10 LHG"}="H101 LHG"; $resn_exch{"2H10 LHG"}="H102 LHG"; $resn_exch{"1H24 LHG"}="H241 LHG";
$resn_exch{"2H24 LHG"}="H242 LHG"; $resn_exch{"1H11 LHG"}="H111 LHG"; $resn_exch{"2H11 LHG"}="H112 LHG";
$resn_exch{"1H12 LHG"}="H121 LHG"; $resn_exch{"2H12 LHG"}="H122 LHG"; $resn_exch{"1H13 LHG"}="H131 LHG";
$resn_exch{"2H13 LHG"}="H132 LHG"; $resn_exch{"1H14 LHG"}="H141 LHG"; $resn_exch{"2H14 LHG"}="H142 LHG";
$resn_exch{"1H15 LHG"}="H151 LHG"; $resn_exch{"2H15 LHG"}="H152 LHG"; $resn_exch{"1H16 LHG"}="H161 LHG";
$resn_exch{"2H16 LHG"}="H162 LHG"; $resn_exch{"1H17 LHG"}="H171 LHG"; $resn_exch{"2H17 LHG"}="H172 LHG";
$resn_exch{"1H18 LHG"}="H181 LHG"; $resn_exch{"2H18 LHG"}="H182 LHG"; $resn_exch{"1H19 LHG"}="H191 LHG";
$resn_exch{"2H19 LHG"}="H192 LHG"; $resn_exch{"1H20 LHG"}="H201 LHG"; $resn_exch{"2H20 LHG"}="H202 LHG";
$resn_exch{"1H21 LHG"}="H211 LHG"; $resn_exch{"2H21 LHG"}="H212 LHG"; $resn_exch{"1H22 LHG"}="H221 LHG";
$resn_exch{"2H22 LHG"}="H222 LHG"; $resn_exch{"3H22 LHG"}="H223 LHG"; $resn_exch{"1H25 LHG"}="H251 LHG";
$resn_exch{"2H25 LHG"}="H252 LHG"; $resn_exch{"1H26 LHG"}="H261 LHG"; $resn_exch{"2H26 LHG"}="H262 LHG";
$resn_exch{"1H27 LHG"}="H271 LHG"; $resn_exch{"2H27 LHG"}="H272 LHG"; $resn_exch{"1H28 LHG"}="H281 LHG";
$resn_exch{"2H28 LHG"}="H282 LHG"; $resn_exch{"1H29 LHG"}="H291 LHG"; $resn_exch{"2H29 LHG"}="H292 LHG";
$resn_exch{"1H30 LHG"}="H301 LHG"; $resn_exch{"2H30 LHG"}="H302 LHG"; $resn_exch{"1H31 LHG"}="H311 LHG";
$resn_exch{"2H31 LHG"}="H312 LHG"; $resn_exch{"1H32 LHG"}="H321 LHG"; $resn_exch{"2H32 LHG"}="H322 LHG";
$resn_exch{"1H33 LHG"}="H331 LHG"; $resn_exch{"2H33 LHG"}="H332 LHG"; $resn_exch{"1H34 LHG"}="H341 LHG";
$resn_exch{"2H34 LHG"}="H342 LHG"; $resn_exch{"1H35 LHG"}="H351 LHG"; $resn_exch{"2H35 LHG"}="H352 LHG";
$resn_exch{"1H36 LHG"}="H361 LHG"; $resn_exch{"2H36 LHG"}="H362 LHG"; $resn_exch{"1H37 LHG"}="H371 LHG";
$resn_exch{"2H37 LHG"}="H372 LHG"; $resn_exch{"1H38 LHG"}="H381 LHG"; $resn_exch{"2H38 LHG"}="H382 LHG";
$resn_exch{"3H38 LHG"}="H383 LHG"; $resn_exch{"3HOP LHU"}="HOP3 LHU"; $resn_exch{"2HOP LHU"}="HOP2 LHU";
$resn_exch{"1H5* LHU"}=" H5' LHU"; $resn_exch{"2H5* LHU"}="H5'' LHU"; $resn_exch{" H4* LHU"}=" H4' LHU";
$resn_exch{" H3* LHU"}=" H3' LHU"; $resn_exch{"H3T  LHU"}="HO3' LHU"; $resn_exch{" H2* LHU"}=" H2' LHU";
$resn_exch{"2HO* LHU"}="HO2' LHU"; $resn_exch{" H1* LHU"}=" H1' LHU"; $resn_exch{"1HB  LHY"}=" HB1 LHY";
$resn_exch{"2HB  LHY"}=" HB2 LHY"; $resn_exch{"1H1  LI0"}=" H11 LI0"; $resn_exch{"2H1  LI0"}="H12A LI0";
$resn_exch{"1H3  LI0"}="H31A LI0"; $resn_exch{"2H3  LI0"}="H32A LI0"; $resn_exch{"1H4  LI0"}=" H41 LI0";
$resn_exch{"2H4  LI0"}=" H42 LI0"; $resn_exch{"1H5  LI0"}=" H51 LI0"; $resn_exch{"2H5  LI0"}=" H52 LI0";
$resn_exch{"1H18 LI0"}="H181 LI0"; $resn_exch{"2H18 LI0"}="H182 LI0"; $resn_exch{"1H25 LI0"}="H251 LI0";
$resn_exch{"2H25 LI0"}="H252 LI0"; $resn_exch{"3H25 LI0"}="H253 LI0"; $resn_exch{"1H26 LI0"}="H261 LI0";
$resn_exch{"2H26 LI0"}="H262 LI0"; $resn_exch{"3H26 LI0"}="H263 LI0"; $resn_exch{"1H29 LI0"}="H291 LI0";
$resn_exch{"2H29 LI0"}="H292 LI0"; $resn_exch{"3H29 LI0"}="H293 LI0"; $resn_exch{"1H30 LI0"}="H301 LI0";
$resn_exch{"2H30 LI0"}="H302 LI0"; $resn_exch{"1H1  LI1"}=" H11 LI1"; $resn_exch{"2H1  LI1"}=" H12 LI1";
$resn_exch{"1H3  LI1"}=" H31 LI1"; $resn_exch{"2H3  LI1"}=" H32 LI1"; $resn_exch{"1H41 LI1"}="H411 LI1";
$resn_exch{"2H41 LI1"}="H412 LI1"; $resn_exch{"1H42 LI1"}="H421 LI1"; $resn_exch{"2H42 LI1"}="H422 LI1";
$resn_exch{"1H44 LI1"}="H441 LI1"; $resn_exch{"2H44 LI1"}="H442 LI1"; $resn_exch{"3H44 LI1"}="H443 LI1";
$resn_exch{"1H45 LI1"}="H451 LI1"; $resn_exch{"2H45 LI1"}="H452 LI1"; $resn_exch{"1H46 LI1"}="H461 LI1";
$resn_exch{"2H46 LI1"}="H462 LI1"; $resn_exch{"1H47 LI1"}="H471 LI1"; $resn_exch{"2H47 LI1"}="H472 LI1";
$resn_exch{"1H49 LI1"}="H491 LI1"; $resn_exch{"2H49 LI1"}="H492 LI1"; $resn_exch{"3H49 LI1"}="H493 LI1";
$resn_exch{"1H50 LI1"}="H501 LI1"; $resn_exch{"2H50 LI1"}="H502 LI1"; $resn_exch{"1H51 LI1"}="H511 LI1";
$resn_exch{"2H51 LI1"}="H512 LI1"; $resn_exch{"1H52 LI1"}="H521 LI1"; $resn_exch{"2H52 LI1"}="H522 LI1";
$resn_exch{"1H53 LI1"}="H531 LI1"; $resn_exch{"2H53 LI1"}="H532 LI1"; $resn_exch{"1H55 LI1"}="H551 LI1";
$resn_exch{"2H55 LI1"}="H552 LI1"; $resn_exch{"1H56 LI1"}="H561 LI1"; $resn_exch{"2H56 LI1"}="H562 LI1";
$resn_exch{"1H57 LI1"}="H571 LI1"; $resn_exch{"2H57 LI1"}="H572 LI1"; $resn_exch{"1H59 LI1"}="H591 LI1";
$resn_exch{"2H59 LI1"}="H592 LI1"; $resn_exch{"3H59 LI1"}="H593 LI1"; $resn_exch{"1H60 LI1"}="H601 LI1";
$resn_exch{"2H60 LI1"}="H602 LI1"; $resn_exch{"3H60 LI1"}="H603 LI1"; $resn_exch{"1H11 LI1"}="H111 LI1";
$resn_exch{"2H11 LI1"}="H112 LI1"; $resn_exch{"1H12 LI1"}="H121 LI1"; $resn_exch{"2H12 LI1"}="H122 LI1";
$resn_exch{"1H14 LI1"}="H141 LI1"; $resn_exch{"2H14 LI1"}="H142 LI1"; $resn_exch{"3H14 LI1"}="H143 LI1";
$resn_exch{"1H15 LI1"}="H151 LI1"; $resn_exch{"2H15 LI1"}="H152 LI1"; $resn_exch{"1H16 LI1"}="H161 LI1";
$resn_exch{"2H16 LI1"}="H162 LI1"; $resn_exch{"1H17 LI1"}="H171 LI1"; $resn_exch{"2H17 LI1"}="H172 LI1";
$resn_exch{"1H19 LI1"}="H191 LI1"; $resn_exch{"2H19 LI1"}="H192 LI1"; $resn_exch{"3H19 LI1"}="H193 LI1";
$resn_exch{"1H20 LI1"}="H201 LI1"; $resn_exch{"2H20 LI1"}="H202 LI1"; $resn_exch{"1H21 LI1"}="H211 LI1";
$resn_exch{"2H21 LI1"}="H212 LI1"; $resn_exch{"1H22 LI1"}="H221 LI1"; $resn_exch{"2H22 LI1"}="H222 LI1";
$resn_exch{"1H24 LI1"}="H241 LI1"; $resn_exch{"2H24 LI1"}="H242 LI1"; $resn_exch{"3H24 LI1"}="H243 LI1";
$resn_exch{"1H25 LI1"}="H251 LI1"; $resn_exch{"2H25 LI1"}="H252 LI1"; $resn_exch{"1H26 LI1"}="H261 LI1";
$resn_exch{"2H26 LI1"}="H262 LI1"; $resn_exch{"1H27 LI1"}="H271 LI1"; $resn_exch{"2H27 LI1"}="H272 LI1";
$resn_exch{"1H29 LI1"}="H291 LI1"; $resn_exch{"2H29 LI1"}="H292 LI1"; $resn_exch{"3H29 LI1"}="H293 LI1";
$resn_exch{"1H30 LI1"}="H301 LI1"; $resn_exch{"2H30 LI1"}="H302 LI1"; $resn_exch{"3H30 LI1"}="H303 LI1";
$resn_exch{"1H19 LI2"}="H191 LI2"; $resn_exch{"2H19 LI2"}="H192 LI2"; $resn_exch{"1H20 LI2"}="H201 LI2";
$resn_exch{"2H20 LI2"}="H202 LI2"; $resn_exch{"1H30 LI2"}="H301 LI2"; $resn_exch{"2H30 LI2"}="H302 LI2";
$resn_exch{"1H31 LI2"}="H311 LI2"; $resn_exch{"2H31 LI2"}="H312 LI2"; $resn_exch{"1H33 LI2"}="H331 LI2";
$resn_exch{"2H33 LI2"}="H332 LI2"; $resn_exch{"1H34 LI2"}="H341 LI2"; $resn_exch{"2H34 LI2"}="H342 LI2";
$resn_exch{"1H22 LI3"}="H221 LI3"; $resn_exch{"2H22 LI3"}="H222 LI3"; $resn_exch{"1H23 LI3"}="H231 LI3";
$resn_exch{"2H23 LI3"}="H232 LI3"; $resn_exch{"1H25 LI3"}="H251 LI3"; $resn_exch{"2H25 LI3"}="H252 LI3";
$resn_exch{"1H26 LI3"}="H261 LI3"; $resn_exch{"2H26 LI3"}="H262 LI3"; $resn_exch{"1H12 LI4"}="H121 LI4";
$resn_exch{"2H12 LI4"}="H122 LI4"; $resn_exch{"1H9C LI4"}="H9C1 LI4"; $resn_exch{"2H9C LI4"}="H9C2 LI4";
$resn_exch{"1H1  LI5"}=" H11 LI5"; $resn_exch{"2H1  LI5"}=" H12 LI5"; $resn_exch{"1H3  LI5"}=" H31 LI5";
$resn_exch{"2H3  LI5"}=" H32 LI5"; $resn_exch{"1H31 LI5"}="H311 LI5"; $resn_exch{"2H31 LI5"}="H312 LI5";
$resn_exch{"1H32 LI5"}="H321 LI5"; $resn_exch{"2H32 LI5"}="H322 LI5"; $resn_exch{"3H32 LI5"}="H323 LI5";
$resn_exch{"1H11 LI6"}="H111 LI6"; $resn_exch{"2H11 LI6"}="H112 LI6"; $resn_exch{"3H11 LI6"}="H113 LI6";
$resn_exch{"1H21 LI8"}="H211 LI8"; $resn_exch{"2H21 LI8"}="H212 LI8"; $resn_exch{"1H22 LI8"}="H221 LI8";
$resn_exch{"2H22 LI8"}="H222 LI8"; $resn_exch{"1H23 LI8"}="H231 LI8"; $resn_exch{"2H23 LI8"}="H232 LI8";
$resn_exch{"1H24 LI8"}="H241 LI8"; $resn_exch{"2H24 LI8"}="H242 LI8"; $resn_exch{"1H27 LI8"}="H271 LI8";
$resn_exch{"2H27 LI8"}="H272 LI8"; $resn_exch{"1H28 LI8"}="H281 LI8"; $resn_exch{"2H28 LI8"}="H282 LI8";
$resn_exch{"1H33 LI8"}="H331 LI8"; $resn_exch{"2H33 LI8"}="H332 LI8"; $resn_exch{"1H35 LI8"}="H351 LI8";
$resn_exch{"2H35 LI8"}="H352 LI8"; $resn_exch{"3H35 LI8"}="H353 LI8"; $resn_exch{"1H36 LI8"}="H361 LI8";
$resn_exch{"2H36 LI8"}="H362 LI8"; $resn_exch{"3H36 LI8"}="H363 LI8"; $resn_exch{"1H39 LI8"}="H391 LI8";
$resn_exch{"2H39 LI8"}="H392 LI8"; $resn_exch{"1H12 LI9"}="H121 LI9"; $resn_exch{"2H12 LI9"}="H122 LI9";
$resn_exch{"3H12 LI9"}="H123 LI9"; $resn_exch{"1H18 LI9"}="H181 LI9"; $resn_exch{"2H18 LI9"}="H182 LI9";
$resn_exch{"1H19 LI9"}="H191 LI9"; $resn_exch{"2H19 LI9"}="H192 LI9"; $resn_exch{"1H20 LI9"}="H201 LI9";
$resn_exch{"2H20 LI9"}="H202 LI9"; $resn_exch{"1H21 LI9"}="H211 LI9"; $resn_exch{"2H21 LI9"}="H212 LI9";
$resn_exch{"1H31 LI9"}="H311 LI9"; $resn_exch{"2H31 LI9"}="H312 LI9"; $resn_exch{"3H31 LI9"}="H313 LI9";
$resn_exch{"1H2  LIA"}=" H21 LIA"; $resn_exch{"2H2  LIA"}=" H22 LIA"; $resn_exch{"1H3  LIA"}=" H31 LIA";
$resn_exch{"2H3  LIA"}=" H32 LIA"; $resn_exch{"1H5  LIA"}=" H51 LIA"; $resn_exch{"2H5  LIA"}=" H52 LIA";
$resn_exch{"1H6  LIA"}=" H61 LIA"; $resn_exch{"2H6  LIA"}=" H62 LIA"; $resn_exch{"1H23 LIA"}="H231 LIA";
$resn_exch{"2H23 LIA"}="H232 LIA"; $resn_exch{"3H23 LIA"}="H233 LIA"; $resn_exch{"1H26 LIA"}="H261 LIA";
$resn_exch{"2H26 LIA"}="H262 LIA"; $resn_exch{"1H28 LIA"}="H281 LIA"; $resn_exch{"2H28 LIA"}="H282 LIA";
$resn_exch{"3H28 LIA"}="H283 LIA"; $resn_exch{"1H14 LIB"}="H141 LIB"; $resn_exch{"2H14 LIB"}="H142 LIB";
$resn_exch{"3H14 LIB"}="H143 LIB"; $resn_exch{"1H43 LIB"}="H431 LIB"; $resn_exch{"2H43 LIB"}="H432 LIB";
$resn_exch{"3H43 LIB"}="H433 LIB"; $resn_exch{"1H47 LIB"}="H471 LIB"; $resn_exch{"2H47 LIB"}="H472 LIB";
$resn_exch{"3H47 LIB"}="H473 LIB"; $resn_exch{"1H46 LIB"}="H461 LIB"; $resn_exch{"2H46 LIB"}="H462 LIB";
$resn_exch{"3H46 LIB"}="H463 LIB"; $resn_exch{"1H7  LIC"}=" H71 LIC"; $resn_exch{"2H7  LIC"}=" H72 LIC";
$resn_exch{"1H31 LIC"}="H311 LIC"; $resn_exch{"2H31 LIC"}="H312 LIC"; $resn_exch{"3H31 LIC"}="H313 LIC";
$resn_exch{"1H45 LIC"}="H451 LIC"; $resn_exch{"2H45 LIC"}="H452 LIC"; $resn_exch{"3H45 LIC"}="H453 LIC";
$resn_exch{"1H53 LIC"}="H531 LIC"; $resn_exch{"2H53 LIC"}="H532 LIC"; $resn_exch{"3H53 LIC"}="H533 LIC";
$resn_exch{"1H51 LIC"}="H511 LIC"; $resn_exch{"2H51 LIC"}="H512 LIC"; $resn_exch{"3H51 LIC"}="H513 LIC";
$resn_exch{"1H10 LID"}="H101 LID"; $resn_exch{"2H10 LID"}="H102 LID"; $resn_exch{"1H11 LID"}="H111 LID";
$resn_exch{"2H11 LID"}="H112 LID"; $resn_exch{"3H11 LID"}="H113 LID"; $resn_exch{"1H26 LIE"}="H261 LIE";
$resn_exch{"2H26 LIE"}="H262 LIE"; $resn_exch{"3H26 LIE"}="H263 LIE"; $resn_exch{"1H25 LIE"}="H251 LIE";
$resn_exch{"2H25 LIE"}="H252 LIE"; $resn_exch{"3H25 LIE"}="H253 LIE"; $resn_exch{"1H39 LIF"}="H391 LIF";
$resn_exch{"2H39 LIF"}="H392 LIF"; $resn_exch{"3H39 LIF"}="H393 LIF"; $resn_exch{"1H31 LIF"}="H311 LIF";
$resn_exch{"2H31 LIF"}="H312 LIF"; $resn_exch{"1H17 LIG"}="H171 LIG"; $resn_exch{"2H17 LIG"}="H172 LIG";
$resn_exch{"1H7  LIH"}=" H71 LIH"; $resn_exch{"1H5A LIH"}="H5A1 LIH"; $resn_exch{"2H5A LIH"}="H5A2 LIH";
$resn_exch{"3H5A LIH"}="H5A3 LIH"; $resn_exch{"1H6' LIH"}="H6'1 LIH"; $resn_exch{"1H7' LIH"}="H7'1 LIH";
$resn_exch{"1H8' LIH"}="H8'1 LIH"; $resn_exch{"1H9  LIH"}=" H91 LIH"; $resn_exch{"2H9  LIH"}=" H92 LIH";
$resn_exch{"1H4' LIH"}="H4'1 LIH"; $resn_exch{"1H2' LIH"}="H2'1 LIH"; $resn_exch{"1H3' LIH"}="H3'1 LIH";
$resn_exch{"1H10 LIH"}="H101 LIH"; $resn_exch{"1H4  LIH"}=" H41 LIH"; $resn_exch{"2H4  LIH"}=" H42 LIH";
$resn_exch{"1H2  LIH"}=" H21 LIH"; $resn_exch{"2H2  LIH"}=" H22 LIH"; $resn_exch{"1H7  LII"}=" H71 LII";
$resn_exch{"1H5  LII"}=" H51 LII"; $resn_exch{"2H5  LII"}=" H52 LII"; $resn_exch{"3H5  LII"}=" H53 LII";
$resn_exch{"1H9  LII"}=" H91 LII"; $resn_exch{"1H3' LII"}="H3'1 LII"; $resn_exch{"1H4' LII"}="H4'1 LII";
$resn_exch{"1H6' LII"}="H6'1 LII"; $resn_exch{"1H10 LII"}="H101 LII"; $resn_exch{"1H2A LII"}="H2A1 LII";
$resn_exch{"2H2A LII"}="H2A2 LII"; $resn_exch{"3H2A LII"}="H2A3 LII"; $resn_exch{"1H5' LII"}="H5'1 LII";
$resn_exch{"2H5' LII"}="H5'2 LII"; $resn_exch{"3H5' LII"}="H5'3 LII"; $resn_exch{"1H4  LII"}=" H41 LII";
$resn_exch{"2H4  LII"}=" H42 LII"; $resn_exch{"1H16 LII"}="H161 LII"; $resn_exch{"2H16 LII"}="H162 LII";
$resn_exch{"1H4  LIJ"}=" H41 LIJ"; $resn_exch{"2H4  LIJ"}=" H42 LIJ"; $resn_exch{"3H4  LIJ"}=" H43 LIJ";
$resn_exch{"1H5  LIJ"}=" H51 LIJ"; $resn_exch{"2H5  LIJ"}=" H52 LIJ"; $resn_exch{"1H6  LIJ"}=" H61 LIJ";
$resn_exch{"2H6  LIJ"}=" H62 LIJ"; $resn_exch{"1H22 LIJ"}="H221 LIJ"; $resn_exch{"2H22 LIJ"}="H222 LIJ";
$resn_exch{"1H24 LIJ"}="H241 LIJ"; $resn_exch{"2H24 LIJ"}="H242 LIJ"; $resn_exch{"1H1  LIJ"}=" H11 LIJ";
$resn_exch{"2H1  LIJ"}=" H12 LIJ"; $resn_exch{"3H1  LIJ"}="H13A LIJ"; $resn_exch{"1H2  LIJ"}="H21A LIJ";
$resn_exch{"2H2  LIJ"}=" H22 LIJ"; $resn_exch{"1H3  LIJ"}=" H31 LIJ"; $resn_exch{"2H3  LIJ"}=" H32 LIJ";
$resn_exch{"1H9  LIJ"}=" H91 LIJ"; $resn_exch{"2H9  LIJ"}=" H92 LIJ"; $resn_exch{"1H14 LIJ"}="H141 LIJ";
$resn_exch{"2H14 LIJ"}="H142 LIJ"; $resn_exch{"1H31 LIJ"}="H311 LIJ"; $resn_exch{"2H31 LIJ"}="H312 LIJ";
$resn_exch{"3H31 LIJ"}="H313 LIJ"; $resn_exch{"1H44 LIJ"}="H441 LIJ"; $resn_exch{"2H44 LIJ"}="H442 LIJ";
$resn_exch{"1H14 LIK"}="H141 LIK"; $resn_exch{"2H14 LIK"}="H142 LIK"; $resn_exch{"1HC1 LIK"}="HC11 LIK";
$resn_exch{"2HC1 LIK"}="HC12 LIK"; $resn_exch{"1H7  LIK"}=" H71 LIK"; $resn_exch{"2H7  LIK"}=" H72 LIK";
$resn_exch{"1H1  LIK"}=" H11 LIK"; $resn_exch{"2H1  LIK"}=" H12 LIK"; $resn_exch{"1HC3 LIK"}="HC31 LIK";
$resn_exch{"2HC3 LIK"}="HC32 LIK"; $resn_exch{"1H3  LIK"}=" H31 LIK"; $resn_exch{"2H3  LIK"}=" H32 LIK";
$resn_exch{"1H4  LIK"}=" H41 LIK"; $resn_exch{"2H4  LIK"}=" H42 LIK"; $resn_exch{"1H5  LIK"}=" H51 LIK";
$resn_exch{"2H5  LIK"}=" H52 LIK"; $resn_exch{"1H6  LIK"}=" H61 LIK"; $resn_exch{"2H6  LIK"}=" H62 LIK";
$resn_exch{"3H6  LIK"}=" H63 LIK"; $resn_exch{"1H2E LIL"}="H2E1 LIL"; $resn_exch{"2H2E LIL"}="H2E2 LIL";
$resn_exch{"1H3E LIL"}="H3E1 LIL"; $resn_exch{"1H4E LIL"}="H4E1 LIL"; $resn_exch{"2H4E LIL"}="H4E2 LIL";
$resn_exch{"1H5E LIL"}="H5E1 LIL"; $resn_exch{"2H5E LIL"}="H5E2 LIL"; $resn_exch{"1H6E LIL"}="H6E1 LIL";
$resn_exch{"2H6E LIL"}="H6E2 LIL"; $resn_exch{"1H7E LIL"}="H7E1 LIL"; $resn_exch{"2H7E LIL"}="H7E2 LIL";
$resn_exch{"1H8E LIL"}="H8E1 LIL"; $resn_exch{"2H8E LIL"}="H8E2 LIL"; $resn_exch{"1H9E LIL"}="H9E1 LIL";
$resn_exch{"2H9E LIL"}="H9E2 LIL"; $resn_exch{"1H0S LIL"}="H0S1 LIL"; $resn_exch{"2H0S LIL"}="H0S2 LIL";
$resn_exch{"1H1S LIL"}="H1S1 LIL"; $resn_exch{"2H1S LIL"}="H1S2 LIL"; $resn_exch{"1H2S LIL"}="H2S1 LIL";
$resn_exch{"2H2S LIL"}="H2S2 LIL"; $resn_exch{"1H3S LIL"}="H3S1 LIL"; $resn_exch{"2H3S LIL"}="H3S2 LIL";
$resn_exch{"1H4S LIL"}="H4S1 LIL"; $resn_exch{"2H4S LIL"}="H4S2 LIL"; $resn_exch{"1H5S LIL"}="H5S1 LIL";
$resn_exch{"2H5S LIL"}="H5S2 LIL"; $resn_exch{"3H5S LIL"}="H5S3 LIL"; $resn_exch{"1H2F LIL"}="H2F1 LIL";
$resn_exch{"2H2F LIL"}="H2F2 LIL"; $resn_exch{"1H3F LIL"}="H3F1 LIL"; $resn_exch{"2H3F LIL"}="H3F2 LIL";
$resn_exch{"1H4F LIL"}="H4F1 LIL"; $resn_exch{"2H4F LIL"}="H4F2 LIL"; $resn_exch{"1H5F LIL"}="H5F1 LIL";
$resn_exch{"2H5F LIL"}="H5F2 LIL"; $resn_exch{"1H6F LIL"}="H6F1 LIL"; $resn_exch{"2H6F LIL"}="H6F2 LIL";
$resn_exch{"1H7F LIL"}="H7F1 LIL"; $resn_exch{"2H7F LIL"}="H7F2 LIL"; $resn_exch{"1H8F LIL"}="H8F1 LIL";
$resn_exch{"2H8F LIL"}="H8F2 LIL"; $resn_exch{"1H9F LIL"}="H9F1 LIL"; $resn_exch{"2H9F LIL"}="H9F2 LIL";
$resn_exch{"1H0T LIL"}="H0T1 LIL"; $resn_exch{"2H0T LIL"}="H0T2 LIL"; $resn_exch{"1H1T LIL"}="H1T1 LIL";
$resn_exch{"2H1T LIL"}="H1T2 LIL"; $resn_exch{"1H2T LIL"}="H2T1 LIL"; $resn_exch{"2H2T LIL"}="H2T2 LIL";
$resn_exch{"1H3T LIL"}="H3T1 LIL"; $resn_exch{"2H3T LIL"}="H3T2 LIL"; $resn_exch{"3H3T LIL"}="H3T3 LIL";
$resn_exch{"1H2J LIM"}="H2J1 LIM"; $resn_exch{"2H2J LIM"}="H2J2 LIM"; $resn_exch{"1H4J LIM"}="H4J1 LIM";
$resn_exch{"2H4J LIM"}="H4J2 LIM"; $resn_exch{"1H5J LIM"}="H5J1 LIM"; $resn_exch{"2H5J LIM"}="H5J2 LIM";
$resn_exch{"1H6J LIM"}="H6J1 LIM"; $resn_exch{"2H6J LIM"}="H6J2 LIM"; $resn_exch{"1H7J LIM"}="H7J1 LIM";
$resn_exch{"2H7J LIM"}="H7J2 LIM"; $resn_exch{"1H8J LIM"}="H8J1 LIM"; $resn_exch{"2H8J LIM"}="H8J2 LIM";
$resn_exch{"1H9J LIM"}="H9J1 LIM"; $resn_exch{"2H9J LIM"}="H9J2 LIM"; $resn_exch{"1H0X LIM"}="H0X1 LIM";
$resn_exch{"2H0X LIM"}="H0X2 LIM"; $resn_exch{"1H1X LIM"}="H1X1 LIM"; $resn_exch{"2H1X LIM"}="H1X2 LIM";
$resn_exch{"1H2X LIM"}="H2X1 LIM"; $resn_exch{"2H2X LIM"}="H2X2 LIM"; $resn_exch{"1H3X LIM"}="H3X1 LIM";
$resn_exch{"2H3X LIM"}="H3X2 LIM"; $resn_exch{"1H4X LIM"}="H4X1 LIM"; $resn_exch{"2H4X LIM"}="H4X2 LIM";
$resn_exch{"1H5X LIM"}="H5X1 LIM"; $resn_exch{"2H5X LIM"}="H5X2 LIM"; $resn_exch{"3H5X LIM"}="H5X3 LIM";
$resn_exch{"1H2I LIN"}="H2I1 LIN"; $resn_exch{"2H2I LIN"}="H2I2 LIN"; $resn_exch{"1H4I LIN"}="H4I1 LIN";
$resn_exch{"2H4I LIN"}="H4I2 LIN"; $resn_exch{"3H4I LIN"}="H4I3 LIN"; $resn_exch{"1H1  LIO"}=" H11 LIO";
$resn_exch{"2H1  LIO"}=" H12 LIO"; $resn_exch{"1H2  LIO"}=" H21 LIO"; $resn_exch{"1H3  LIO"}=" H31 LIO";
$resn_exch{"2H3  LIO"}=" H32 LIO"; $resn_exch{"1H4  LIO"}=" H41 LIO"; $resn_exch{"2H4  LIO"}=" H42 LIO";
$resn_exch{"1H5  LIO"}=" H51 LIO"; $resn_exch{"2H5  LIO"}=" H52 LIO"; $resn_exch{"1H6  LIO"}=" H61 LIO";
$resn_exch{"2H6  LIO"}=" H62 LIO"; $resn_exch{"3H6  LIO"}=" H63 LIO"; $resn_exch{"1H7  LIO"}=" H71 LIO";
$resn_exch{"2H7  LIO"}=" H72 LIO"; $resn_exch{"3H7  LIO"}=" H73 LIO"; $resn_exch{"1H8  LIO"}=" H81 LIO";
$resn_exch{"2H8  LIO"}=" H82 LIO"; $resn_exch{"3H8  LIO"}=" H83 LIO"; $resn_exch{"1H12 LIO"}="H121 LIO";
$resn_exch{"2H12 LIO"}="H122 LIO"; $resn_exch{"1H13 LIO"}="H131 LIO"; $resn_exch{"2H13 LIO"}="H132 LIO";
$resn_exch{"1H14 LIO"}="H141 LIO"; $resn_exch{"2H14 LIO"}="H142 LIO"; $resn_exch{"1H15 LIO"}="H151 LIO";
$resn_exch{"2H15 LIO"}="H152 LIO"; $resn_exch{"1H16 LIO"}="H161 LIO"; $resn_exch{"2H16 LIO"}="H162 LIO";
$resn_exch{"1H17 LIO"}="H171 LIO"; $resn_exch{"2H17 LIO"}="H172 LIO"; $resn_exch{"1H18 LIO"}="H181 LIO";
$resn_exch{"2H18 LIO"}="H182 LIO"; $resn_exch{"1H19 LIO"}="H191 LIO"; $resn_exch{"2H19 LIO"}="H192 LIO";
$resn_exch{"1H20 LIO"}="H201 LIO"; $resn_exch{"2H20 LIO"}="H202 LIO"; $resn_exch{"1H21 LIO"}="H211 LIO";
$resn_exch{"2H21 LIO"}="H212 LIO"; $resn_exch{"1H22 LIO"}="H221 LIO"; $resn_exch{"2H22 LIO"}="H222 LIO";
$resn_exch{"1H23 LIO"}="H231 LIO"; $resn_exch{"2H23 LIO"}="H232 LIO"; $resn_exch{"1H24 LIO"}="H241 LIO";
$resn_exch{"2H24 LIO"}="H242 LIO"; $resn_exch{"1H25 LIO"}="H251 LIO"; $resn_exch{"2H25 LIO"}="H252 LIO";
$resn_exch{"3H25 LIO"}="H253 LIO"; $resn_exch{"1H32 LIO"}="H321 LIO"; $resn_exch{"2H32 LIO"}="H322 LIO";
$resn_exch{"1H33 LIO"}="H331 LIO"; $resn_exch{"2H33 LIO"}="H332 LIO"; $resn_exch{"1H34 LIO"}="H341 LIO";
$resn_exch{"2H34 LIO"}="H342 LIO"; $resn_exch{"1H35 LIO"}="H351 LIO"; $resn_exch{"2H35 LIO"}="H352 LIO";
$resn_exch{"1H36 LIO"}="H361 LIO"; $resn_exch{"2H36 LIO"}="H362 LIO"; $resn_exch{"1H37 LIO"}="H371 LIO";
$resn_exch{"2H37 LIO"}="H372 LIO"; $resn_exch{"1H38 LIO"}="H381 LIO"; $resn_exch{"2H38 LIO"}="H382 LIO";
$resn_exch{"1H39 LIO"}="H391 LIO"; $resn_exch{"2H39 LIO"}="H392 LIO"; $resn_exch{"1H40 LIO"}="H401 LIO";
$resn_exch{"2H40 LIO"}="H402 LIO"; $resn_exch{"3H40 LIO"}="H403 LIO"; $resn_exch{"1HOP LIO"}="HOP1 LIO";
$resn_exch{"1H7  LIQ"}=" H71 LIQ"; $resn_exch{"2H7  LIQ"}=" H72 LIQ"; $resn_exch{"1H22 LIQ"}="H221 LIQ";
$resn_exch{"2H22 LIQ"}="H222 LIQ"; $resn_exch{"1H24 LIQ"}="H241 LIQ"; $resn_exch{"2H24 LIQ"}="H242 LIQ";
$resn_exch{"1H28 LIQ"}="H281 LIQ"; $resn_exch{"2H28 LIQ"}="H282 LIQ"; $resn_exch{"1H1  LIQ"}=" H11 LIQ";
$resn_exch{"2H1  LIQ"}=" H12 LIQ"; $resn_exch{"1H1A LIQ"}="H1A1 LIQ"; $resn_exch{"2H1A LIQ"}="H1A2 LIQ";
$resn_exch{"3H1A LIQ"}="H1A3 LIQ"; $resn_exch{"1H2  LIQ"}="H21A LIQ"; $resn_exch{"2H2  LIQ"}=" H22 LIQ";
$resn_exch{"1H3  LIQ"}="H31A LIQ"; $resn_exch{"2H3  LIQ"}=" H32 LIQ"; $resn_exch{"1H9  LIQ"}=" H91 LIQ";
$resn_exch{"2H9  LIQ"}=" H92 LIQ"; $resn_exch{"1H14 LIQ"}="H141 LIQ"; $resn_exch{"2H14 LIQ"}="H142 LIQ";
$resn_exch{"1H32 LIQ"}="H321 LIQ"; $resn_exch{"2H32 LIQ"}="H322 LIQ"; $resn_exch{"3H32 LIQ"}="H323 LIQ";
$resn_exch{"1H13 LIR"}="H131 LIR"; $resn_exch{"2H13 LIR"}="H132 LIR"; $resn_exch{"1H15 LIR"}="H151 LIR";
$resn_exch{"2H15 LIR"}="H152 LIR"; $resn_exch{"1H17 LIR"}="H171 LIR"; $resn_exch{"2H17 LIR"}="H172 LIR";
$resn_exch{"1H11 LIR"}="H111 LIR"; $resn_exch{"2H11 LIR"}="H112 LIR"; $resn_exch{"3H11 LIR"}="H113 LIR";
$resn_exch{"1H16 LIR"}="H161 LIR"; $resn_exch{"2H16 LIR"}="H162 LIR"; $resn_exch{"1H18 LIR"}="H181 LIR";
$resn_exch{"2H18 LIR"}="H182 LIR"; $resn_exch{"1H19 LIR"}="H191 LIR"; $resn_exch{"2H19 LIR"}="H192 LIR";
$resn_exch{"1HN  LIS"}=" HN1 LIS"; $resn_exch{"2HN  LIS"}=" HN2 LIS"; $resn_exch{"1HBC LIS"}="HBC1 LIS";
$resn_exch{"2HBC LIS"}="HBC2 LIS"; $resn_exch{"1HDC LIS"}="HDC1 LIS"; $resn_exch{"2HDC LIS"}="HDC2 LIS";
$resn_exch{"1H10 LIT"}="H101 LIT"; $resn_exch{"2H10 LIT"}="H102 LIT"; $resn_exch{"1H1  LIU"}="H11A LIU";
$resn_exch{"2H1  LIU"}=" H12 LIU"; $resn_exch{"1H3  LIU"}="H31A LIU"; $resn_exch{"2H3  LIU"}="H32A LIU";
$resn_exch{"1H4  LIU"}=" H41 LIU"; $resn_exch{"2H4  LIU"}=" H42 LIU"; $resn_exch{"1H6  LIU"}=" H61 LIU";
$resn_exch{"2H6  LIU"}=" H62 LIU"; $resn_exch{"1H7  LIU"}=" H71 LIU"; $resn_exch{"2H7  LIU"}=" H72 LIU";
$resn_exch{"3H7  LIU"}=" H73 LIU"; $resn_exch{"1H8  LIU"}=" H81 LIU"; $resn_exch{"2H8  LIU"}=" H82 LIU";
$resn_exch{"3H8  LIU"}=" H83 LIU"; $resn_exch{"1H27 LIU"}="H271 LIU"; $resn_exch{"2H27 LIU"}="H272 LIU";
$resn_exch{"1H40 LIU"}="H401 LIU"; $resn_exch{"2H40 LIU"}="H402 LIU"; $resn_exch{"3H40 LIU"}="H403 LIU";
$resn_exch{"1H41 LIU"}="H411 LIU"; $resn_exch{"2H41 LIU"}="H412 LIU"; $resn_exch{"3H41 LIU"}="H413 LIU";
$resn_exch{"1H21 LIV"}="H211 LIV"; $resn_exch{"2H21 LIV"}="H212 LIV"; $resn_exch{"1H31 LIV"}="H311 LIV";
$resn_exch{"2H31 LIV"}="H312 LIV"; $resn_exch{"1H61 LIV"}="H611 LIV"; $resn_exch{"2H61 LIV"}="H612 LIV";
$resn_exch{"1H12 LIV"}="H121 LIV"; $resn_exch{"2H12 LIV"}="H122 LIV"; $resn_exch{"1H32 LIV"}="H321 LIV";
$resn_exch{"2H32 LIV"}="H322 LIV"; $resn_exch{"1H22 LIV"}="H221 LIV"; $resn_exch{"2H22 LIV"}="H222 LIV";
$resn_exch{"1H53 LIV"}="H531 LIV"; $resn_exch{"2H53 LIV"}="H532 LIV"; $resn_exch{"1H24 LIV"}="H241 LIV";
$resn_exch{"2H24 LIV"}="H242 LIV"; $resn_exch{"1H1  LIV"}="H11A LIV"; $resn_exch{"2H1  LIV"}="H12A LIV";
$resn_exch{"1H64 LIV"}="H641 LIV"; $resn_exch{"2H64 LIV"}="H642 LIV"; $resn_exch{"1H65 LIV"}="H651 LIV";
$resn_exch{"2H65 LIV"}="H652 LIV"; $resn_exch{"1H8  LIW"}=" H81 LIW"; $resn_exch{"2H8  LIW"}=" H82 LIW";
$resn_exch{"1H9  LIW"}=" H91 LIW"; $resn_exch{"2H9  LIW"}=" H92 LIW"; $resn_exch{"1H10 LIW"}="H101 LIW";
$resn_exch{"2H10 LIW"}="H102 LIW"; $resn_exch{"1H11 LIW"}="H111 LIW"; $resn_exch{"2H11 LIW"}="H112 LIW";
$resn_exch{"1H12 LIW"}="H121 LIW"; $resn_exch{"2H12 LIW"}="H122 LIW"; $resn_exch{"1H19 LIW"}="H191 LIW";
$resn_exch{"2H19 LIW"}="H192 LIW"; $resn_exch{"3H19 LIW"}="H193 LIW"; $resn_exch{"1H20 LIW"}="H201 LIW";
$resn_exch{"2H20 LIW"}="H202 LIW"; $resn_exch{"1H27 LIW"}="H271 LIW"; $resn_exch{"2H27 LIW"}="H272 LIW";
$resn_exch{"3H27 LIW"}="H273 LIW"; $resn_exch{"1H5  LIX"}=" H51 LIX"; $resn_exch{"2H5  LIX"}=" H52 LIX";
$resn_exch{"1H6  LIX"}=" H61 LIX"; $resn_exch{"2H6  LIX"}=" H62 LIX"; $resn_exch{"3H6  LIX"}=" H63 LIX";
$resn_exch{"1H13 LIX"}="H131 LIX"; $resn_exch{"2H13 LIX"}="H132 LIX"; $resn_exch{"3H13 LIX"}="H133 LIX";
$resn_exch{"1H17 LIX"}="H171 LIX"; $resn_exch{"2H17 LIX"}="H172 LIX"; $resn_exch{"1H20 LIX"}="H201 LIX";
$resn_exch{"2H20 LIX"}="H202 LIX"; $resn_exch{"1H21 LIX"}="H211 LIX"; $resn_exch{"2H21 LIX"}="H212 LIX";
$resn_exch{"1H22 LIX"}="H221 LIX"; $resn_exch{"2H22 LIX"}="H222 LIX"; $resn_exch{"3H22 LIX"}="H223 LIX";
$resn_exch{"1HN4 LIX"}="HN41 LIX"; $resn_exch{"2HN4 LIX"}="HN42 LIX"; $resn_exch{"1H16 LIX"}="H161 LIX";
$resn_exch{"2H16 LIX"}="H162 LIX"; $resn_exch{"1H18 LIX"}="H181 LIX"; $resn_exch{"2H18 LIX"}="H182 LIX";
$resn_exch{"1H19 LIX"}="H191 LIX"; $resn_exch{"2H19 LIX"}="H192 LIX"; $resn_exch{"3H19 LIX"}="H193 LIX";
$resn_exch{"1HN3 LIY"}="HN31 LIY"; $resn_exch{"2HN3 LIY"}="HN32 LIY"; $resn_exch{"1H6  LIY"}=" H61 LIY";
$resn_exch{"2H6  LIY"}=" H62 LIY"; $resn_exch{"1H13 LIY"}="H131 LIY"; $resn_exch{"2H13 LIY"}="H132 LIY";
$resn_exch{"1H17 LIY"}="H171 LIY"; $resn_exch{"2H17 LIY"}="H172 LIY"; $resn_exch{"1H20 LIY"}="H201 LIY";
$resn_exch{"2H20 LIY"}="H202 LIY"; $resn_exch{"3H20 LIY"}="H203 LIY"; $resn_exch{"1HN4 LIY"}="HN41 LIY";
$resn_exch{"2HN4 LIY"}="HN42 LIY"; $resn_exch{"1H14 LIY"}="H141 LIY"; $resn_exch{"2H14 LIY"}="H142 LIY";
$resn_exch{"1H16 LIY"}="H161 LIY"; $resn_exch{"2H16 LIY"}="H162 LIY"; $resn_exch{"1H18 LIY"}="H181 LIY";
$resn_exch{"2H18 LIY"}="H182 LIY"; $resn_exch{"1H19 LIY"}="H191 LIY"; $resn_exch{"2H19 LIY"}="H192 LIY";
$resn_exch{"3H19 LIY"}="H193 LIY"; $resn_exch{"1HBC LK1"}="HBC1 LK1"; $resn_exch{"2HBC LK1"}="HBC2 LK1";
$resn_exch{"1HGC LK1"}="HGC1 LK1"; $resn_exch{"2HGC LK1"}="HGC2 LK1"; $resn_exch{"1H23 LK1"}="H231 LK1";
$resn_exch{"2H23 LK1"}="H232 LK1"; $resn_exch{"1H24 LK1"}="H241 LK1"; $resn_exch{"2H24 LK1"}="H242 LK1";
$resn_exch{"1H25 LK1"}="H251 LK1"; $resn_exch{"2H25 LK1"}="H252 LK1"; $resn_exch{"1H26 LK1"}="H261 LK1";
$resn_exch{"2H26 LK1"}="H262 LK1"; $resn_exch{"3H26 LK1"}="H263 LK1"; $resn_exch{"1HBC LK2"}="HBC1 LK2";
$resn_exch{"2HBC LK2"}="HBC2 LK2"; $resn_exch{"1HGC LK2"}="HGC1 LK2"; $resn_exch{"2HGC LK2"}="HGC2 LK2";
$resn_exch{"1H23 LK2"}="H231 LK2"; $resn_exch{"2H23 LK2"}="H232 LK2"; $resn_exch{"1H24 LK2"}="H241 LK2";
$resn_exch{"2H24 LK2"}="H242 LK2"; $resn_exch{"1H29 LK2"}="H291 LK2"; $resn_exch{"2H29 LK2"}="H292 LK2";
$resn_exch{"1H44 LK2"}="H441 LK2"; $resn_exch{"2H44 LK2"}="H442 LK2"; $resn_exch{"3H44 LK2"}="H443 LK2";
$resn_exch{"1HBC LK3"}="HBC1 LK3"; $resn_exch{"2HBC LK3"}="HBC2 LK3"; $resn_exch{"1HGC LK3"}="HGC1 LK3";
$resn_exch{"2HGC LK3"}="HGC2 LK3"; $resn_exch{"1H23 LK3"}="H231 LK3"; $resn_exch{"2H23 LK3"}="H232 LK3";
$resn_exch{"1H24 LK3"}="H241 LK3"; $resn_exch{"2H24 LK3"}="H242 LK3"; $resn_exch{"1H25 LK3"}="H251 LK3";
$resn_exch{"2H25 LK3"}="H252 LK3"; $resn_exch{"1H26 LK3"}="H261 LK3"; $resn_exch{"2H26 LK3"}="H262 LK3";
$resn_exch{"1H30 LK3"}="H301 LK3"; $resn_exch{"2H30 LK3"}="H302 LK3"; $resn_exch{"3H30 LK3"}="H303 LK3";
$resn_exch{"1H23 LK4"}="H231 LK4"; $resn_exch{"2H23 LK4"}="H232 LK4"; $resn_exch{"1HBC LK4"}="HBC1 LK4";
$resn_exch{"2HBC LK4"}="HBC2 LK4"; $resn_exch{"1HGC LK4"}="HGC1 LK4"; $resn_exch{"2HGC LK4"}="HGC2 LK4";
$resn_exch{"1H1  LK5"}=" H11 LK5"; $resn_exch{"2H1  LK5"}=" H12 LK5"; $resn_exch{"3H1  LK5"}=" H13 LK5";
$resn_exch{"1H3  LK5"}=" H31 LK5"; $resn_exch{"2H3  LK5"}=" H32 LK5"; $resn_exch{"1H4  LK5"}=" H41 LK5";
$resn_exch{"2H4  LK5"}=" H42 LK5"; $resn_exch{"1H5  LK5"}=" H51 LK5"; $resn_exch{"2H5  LK5"}=" H52 LK5";
$resn_exch{"1H7  LK5"}=" H71 LK5"; $resn_exch{"2H7  LK5"}=" H72 LK5"; $resn_exch{"1H8  LK5"}=" H81 LK5";
$resn_exch{"2H8  LK5"}=" H82 LK5"; $resn_exch{"1H9  LK5"}=" H91 LK5"; $resn_exch{"2H9  LK5"}=" H92 LK5";
$resn_exch{"1H16 LK5"}="H161 LK5"; $resn_exch{"2H16 LK5"}="H162 LK5"; $resn_exch{"1H2  LK5"}=" H21 LK5";
$resn_exch{"2H2  LK5"}=" H22 LK5"; $resn_exch{"3H2  LK5"}=" H23 LK5"; $resn_exch{"1H1  LK7"}=" H11 LK7";
$resn_exch{"2H1  LK7"}=" H12 LK7"; $resn_exch{"3H1  LK7"}="H13A LK7"; $resn_exch{"1H10 LK7"}="H101 LK7";
$resn_exch{"2H10 LK7"}="H102 LK7"; $resn_exch{"1H11 LK7"}="H111 LK7"; $resn_exch{"2H11 LK7"}="H112 LK7";
$resn_exch{"1H12 LK7"}="H121 LK7"; $resn_exch{"2H12 LK7"}="H122 LK7"; $resn_exch{"1H14 LK7"}="H141 LK7";
$resn_exch{"2H14 LK7"}="H142 LK7"; $resn_exch{"3H14 LK7"}="H143 LK7"; $resn_exch{"1HN1 LKA"}="HN11 LKA";
$resn_exch{"2HN1 LKA"}="HN12 LKA"; $resn_exch{"1H6  LKA"}=" H61 LKA"; $resn_exch{"2H6  LKA"}=" H62 LKA";
$resn_exch{"1H4  LKC"}=" H41 LKC"; $resn_exch{"2H4  LKC"}=" H42 LKC"; $resn_exch{" H1* LKC"}=" H1' LKC";
$resn_exch{"1H2* LKC"}="H2'1 LKC"; $resn_exch{" H3* LKC"}=" H3' LKC"; $resn_exch{"1H5* LKC"}="H5'1 LKC";
$resn_exch{"2H5* LKC"}="H5'2 LKC"; $resn_exch{"1H5M LKC"}="H5M1 LKC"; $resn_exch{"2H5M LKC"}="H5M2 LKC";
$resn_exch{"3H5M LKC"}="H5M3 LKC"; $resn_exch{"1H6* LKC"}="H6'1 LKC"; $resn_exch{"2H6* LKC"}="H6'2 LKC";
$resn_exch{"1H6  LKS"}=" H61 LKS"; $resn_exch{"2H6  LKS"}=" H62 LKS"; $resn_exch{"1H8  LKS"}=" H81 LKS";
$resn_exch{"2H8  LKS"}=" H82 LKS"; $resn_exch{"1H7  LLB"}=" H71 LLB"; $resn_exch{"2H7  LLB"}=" H72 LLB";
$resn_exch{"3H7  LLB"}=" H73 LLB"; $resn_exch{"1H28 LLB"}="H281 LLB"; $resn_exch{"2H28 LLB"}="H282 LLB";
$resn_exch{"1H29 LLB"}="H291 LLB"; $resn_exch{"2H29 LLB"}="H292 LLB"; $resn_exch{"1H66 LLB"}="H661 LLB";
$resn_exch{"2H66 LLB"}="H662 LLB"; $resn_exch{"1H65 LLB"}="H651 LLB"; $resn_exch{"2H65 LLB"}="H652 LLB";
$resn_exch{"1H63 LLB"}="H631 LLB"; $resn_exch{"2H63 LLB"}="H632 LLB"; $resn_exch{"1H62 LLB"}="H621 LLB";
$resn_exch{"2H62 LLB"}="H622 LLB"; $resn_exch{"1H26 LLC"}="H261 LLC"; $resn_exch{"2H26 LLC"}="H262 LLC";
$resn_exch{"1H28 LLC"}="H281 LLC"; $resn_exch{"2H28 LLC"}="H282 LLC"; $resn_exch{"1H25 LLC"}="H251 LLC";
$resn_exch{"2H25 LLC"}="H252 LLC"; $resn_exch{"1H31 LLC"}="H311 LLC"; $resn_exch{"2H31 LLC"}="H312 LLC";
$resn_exch{"1H30 LLC"}="H301 LLC"; $resn_exch{"2H30 LLC"}="H302 LLC"; $resn_exch{"1H29 LLC"}="H291 LLC";
$resn_exch{"2H29 LLC"}="H292 LLC"; $resn_exch{"1HAF LLF"}="HAF1 LLF"; $resn_exch{"2HAF LLF"}="HAF2 LLF";
$resn_exch{"1H2A LLF"}="H2A1 LLF"; $resn_exch{"2H2A LLF"}="H2A2 LLF"; $resn_exch{"3H2A LLF"}="H2A3 LLF";
$resn_exch{"1H5M LLF"}="H5M1 LLF"; $resn_exch{"2H5M LLF"}="H5M2 LLF"; $resn_exch{"1H21 LLL"}="H211 LLL";
$resn_exch{"2H21 LLL"}="H212 LLL"; $resn_exch{"1H31 LLL"}="H311 LLL"; $resn_exch{"2H31 LLL"}="H312 LLL";
$resn_exch{"1H41 LLL"}="H411 LLL"; $resn_exch{"2H41 LLL"}="H412 LLL"; $resn_exch{"1H61 LLL"}="H611 LLL";
$resn_exch{"2H61 LLL"}="H612 LLL"; $resn_exch{"1H1  LLL"}="H11A LLL"; $resn_exch{"2H1  LLL"}="H12A LLL";
$resn_exch{"1H12 LLL"}="H121 LLL"; $resn_exch{"2H12 LLL"}="H122 LLL"; $resn_exch{"1H22 LLL"}="H221 LLL";
$resn_exch{"2H22 LLL"}="H222 LLL"; $resn_exch{"1H32 LLL"}="H321 LLL"; $resn_exch{"2H32 LLL"}="H322 LLL";
$resn_exch{"1H53 LLL"}="H531 LLL"; $resn_exch{"2H53 LLL"}="H532 LLL"; $resn_exch{"1H83 LLL"}="H831 LLL";
$resn_exch{"2H83 LLL"}="H832 LLL"; $resn_exch{"3H83 LLL"}="H833 LLL"; $resn_exch{"1H93 LLL"}="H931 LLL";
$resn_exch{"2H93 LLL"}="H932 LLL"; $resn_exch{"3H93 LLL"}="H933 LLL"; $resn_exch{"1H2A LLP"}="H2'1 LLP";
$resn_exch{"2H2A LLP"}="H2'2 LLP"; $resn_exch{"3H2A LLP"}="H2'3 LLP"; $resn_exch{"1H4A LLP"}="H4'1 LLP";
$resn_exch{"2H4A LLP"}="H4'2 LLP"; $resn_exch{"1H5A LLP"}="H5'1 LLP"; $resn_exch{"2H5A LLP"}="H5'2 LLP";
$resn_exch{"2HOP LLP"}="HOP2 LLP"; $resn_exch{"3HOP LLP"}="HOP3 LLP"; $resn_exch{"1HN  LLP"}=" H   LLP";
$resn_exch{"2HN  LLP"}=" HN2 LLP"; $resn_exch{"1HB  LLP"}=" HB2 LLP"; $resn_exch{"2HB  LLP"}=" HB3 LLP";
$resn_exch{"1HG  LLP"}=" HG2 LLP"; $resn_exch{"2HG  LLP"}=" HG3 LLP"; $resn_exch{"1HD  LLP"}=" HD2 LLP";
$resn_exch{"2HD  LLP"}=" HD3 LLP"; $resn_exch{"1HE  LLP"}=" HE2 LLP"; $resn_exch{"2HE  LLP"}=" HE3 LLP";
$resn_exch{" HNZ LLP"}=" HZ1 LLP"; $resn_exch{"1HN  LLY"}=" H   LLY"; $resn_exch{"2HN  LLY"}=" H2  LLY";
$resn_exch{"1HB  LLY"}=" HB2 LLY"; $resn_exch{"2HB  LLY"}=" HB3 LLY"; $resn_exch{"1HG  LLY"}=" HG2 LLY";
$resn_exch{"2HG  LLY"}=" HG3 LLY"; $resn_exch{"1HD  LLY"}=" HD2 LLY"; $resn_exch{"2HD  LLY"}=" HD3 LLY";
$resn_exch{"1HE  LLY"}=" HE2 LLY"; $resn_exch{"2HE  LLY"}=" HE3 LLY"; $resn_exch{" HNZ LLY"}=" HZ  LLY";
$resn_exch{"1HA6 LM2"}="HA61 LM2"; $resn_exch{"2HA6 LM2"}="HA62 LM2"; $resn_exch{"1HB6 LM2"}="HB61 LM2";
$resn_exch{"2HB6 LM2"}="HB62 LM2"; $resn_exch{"1HD6 LM2"}="HD61 LM2"; $resn_exch{"2HD6 LM2"}="HD62 LM2";
$resn_exch{"1HD7 LM2"}="HD71 LM2"; $resn_exch{"2HD7 LM2"}="HD72 LM2"; $resn_exch{"3HD7 LM2"}="HD73 LM2";
$resn_exch{" HN2 LME"}=" H2  LME"; $resn_exch{" HN  LME"}=" H   LME"; $resn_exch{"1H1  LME"}=" H11 LME";
$resn_exch{"2H1  LME"}=" H12 LME"; $resn_exch{"3H1  LME"}=" H13 LME"; $resn_exch{"1HG  LME"}=" HG2 LME";
$resn_exch{"2HG  LME"}=" HG3 LME"; $resn_exch{"1HC6 LMG"}="HC61 LMG"; $resn_exch{"2HC6 LMG"}="HC62 LMG";
$resn_exch{"1HC7 LMG"}="HC71 LMG"; $resn_exch{"2HC7 LMG"}="HC72 LMG"; $resn_exch{"1HC9 LMG"}="HC91 LMG";
$resn_exch{"2HC9 LMG"}="HC92 LMG"; $resn_exch{"1H11 LMG"}="H111 LMG"; $resn_exch{"2H11 LMG"}="H112 LMG";
$resn_exch{"1H12 LMG"}="H121 LMG"; $resn_exch{"2H12 LMG"}="H122 LMG"; $resn_exch{"1H13 LMG"}="H131 LMG";
$resn_exch{"2H13 LMG"}="H132 LMG"; $resn_exch{"1H14 LMG"}="H141 LMG"; $resn_exch{"2H14 LMG"}="H142 LMG";
$resn_exch{"1H15 LMG"}="H151 LMG"; $resn_exch{"2H15 LMG"}="H152 LMG"; $resn_exch{"1H16 LMG"}="H161 LMG";
$resn_exch{"2H16 LMG"}="H162 LMG"; $resn_exch{"1H17 LMG"}="H171 LMG"; $resn_exch{"2H17 LMG"}="H172 LMG";
$resn_exch{"1H18 LMG"}="H181 LMG"; $resn_exch{"2H18 LMG"}="H182 LMG"; $resn_exch{"1H19 LMG"}="H191 LMG";
$resn_exch{"2H19 LMG"}="H192 LMG"; $resn_exch{"1H20 LMG"}="H201 LMG"; $resn_exch{"2H20 LMG"}="H202 LMG";
$resn_exch{"1H21 LMG"}="H211 LMG"; $resn_exch{"2H21 LMG"}="H212 LMG"; $resn_exch{"1H22 LMG"}="H221 LMG";
$resn_exch{"2H22 LMG"}="H222 LMG"; $resn_exch{"1H23 LMG"}="H231 LMG"; $resn_exch{"2H23 LMG"}="H232 LMG";
$resn_exch{"1H24 LMG"}="H241 LMG"; $resn_exch{"2H24 LMG"}="H242 LMG"; $resn_exch{"1H25 LMG"}="H251 LMG";
$resn_exch{"2H25 LMG"}="H252 LMG"; $resn_exch{"1H26 LMG"}="H261 LMG"; $resn_exch{"2H26 LMG"}="H262 LMG";
$resn_exch{"1H27 LMG"}="H271 LMG"; $resn_exch{"2H27 LMG"}="H272 LMG"; $resn_exch{"3H27 LMG"}="H273 LMG";
$resn_exch{"1H29 LMG"}="H291 LMG"; $resn_exch{"2H29 LMG"}="H292 LMG"; $resn_exch{"1H30 LMG"}="H301 LMG";
$resn_exch{"2H30 LMG"}="H302 LMG"; $resn_exch{"1H31 LMG"}="H311 LMG"; $resn_exch{"2H31 LMG"}="H312 LMG";
$resn_exch{"1H32 LMG"}="H321 LMG"; $resn_exch{"2H32 LMG"}="H322 LMG"; $resn_exch{"1H33 LMG"}="H331 LMG";
$resn_exch{"2H33 LMG"}="H332 LMG"; $resn_exch{"1H34 LMG"}="H341 LMG"; $resn_exch{"2H34 LMG"}="H342 LMG";
$resn_exch{"1H35 LMG"}="H351 LMG"; $resn_exch{"2H35 LMG"}="H352 LMG"; $resn_exch{"1H36 LMG"}="H361 LMG";
$resn_exch{"2H36 LMG"}="H362 LMG"; $resn_exch{"1H37 LMG"}="H371 LMG"; $resn_exch{"2H37 LMG"}="H372 LMG";
$resn_exch{"1H38 LMG"}="H381 LMG"; $resn_exch{"2H38 LMG"}="H382 LMG"; $resn_exch{"1H39 LMG"}="H391 LMG";
$resn_exch{"2H39 LMG"}="H392 LMG"; $resn_exch{"1H40 LMG"}="H401 LMG"; $resn_exch{"2H40 LMG"}="H402 LMG";
$resn_exch{"1H41 LMG"}="H411 LMG"; $resn_exch{"2H41 LMG"}="H412 LMG"; $resn_exch{"1H42 LMG"}="H421 LMG";
$resn_exch{"2H42 LMG"}="H422 LMG"; $resn_exch{"1H43 LMG"}="H431 LMG"; $resn_exch{"2H43 LMG"}="H432 LMG";
$resn_exch{"1H44 LMG"}="H441 LMG"; $resn_exch{"2H44 LMG"}="H442 LMG"; $resn_exch{"1H45 LMG"}="H451 LMG";
$resn_exch{"2H45 LMG"}="H452 LMG"; $resn_exch{"3H45 LMG"}="H453 LMG"; $resn_exch{"1HB  LML"}=" HB1 LML";
$resn_exch{"2HB  LML"}=" HB2 LML"; $resn_exch{"1HD1 LML"}="HD11 LML"; $resn_exch{"2HD1 LML"}="HD12 LML";
$resn_exch{"3HD1 LML"}="HD13 LML"; $resn_exch{"1HD2 LML"}="HD21 LML"; $resn_exch{"2HD2 LML"}="HD22 LML";
$resn_exch{"3HD2 LML"}="HD23 LML"; $resn_exch{" O3* LMS"}=" O3' LMS"; $resn_exch{" C2* LMS"}=" C2' LMS";
$resn_exch{" O2* LMS"}=" O2' LMS"; $resn_exch{" C1* LMS"}=" C1' LMS"; $resn_exch{"1H5* LMS"}="H5'1 LMS";
$resn_exch{"2H5* LMS"}="H5'2 LMS"; $resn_exch{" H4* LMS"}=" H4' LMS"; $resn_exch{" H3* LMS"}=" H3' LMS";
$resn_exch{" H2* LMS"}=" H2' LMS"; $resn_exch{" H1* LMS"}=" H1' LMS"; $resn_exch{"1H6N LMS"}="H6N1 LMS";
$resn_exch{"2H6N LMS"}="H6N2 LMS"; $resn_exch{"1HN  LMS"}=" HN1 LMS"; $resn_exch{"2HN  LMS"}=" HN2 LMS";
$resn_exch{" H1* LMT"}=" H1B LMT"; $resn_exch{" H2* LMT"}=" H2B LMT"; $resn_exch{" H3* LMT"}=" H3B LMT";
$resn_exch{" H4* LMT"}=" H4B LMT"; $resn_exch{" H5* LMT"}=" H5B LMT"; $resn_exch{"2H6* LMT"}="H6'2 LMT";
$resn_exch{"1H6* LMT"}="H6'1 LMT"; $resn_exch{"1H2O LMT"}="H2O1 LMT"; $resn_exch{"1H3O LMT"}="H3O1 LMT";
$resn_exch{"1H4O LMT"}="H4O1 LMT"; $resn_exch{" H6* LMT"}=" H6B LMT"; $resn_exch{"2H6' LMT"}=" H6D LMT";
$resn_exch{"1H6' LMT"}=" H6E LMT"; $resn_exch{"2H2O LMT"}="H2O2 LMT"; $resn_exch{"2H3O LMT"}="H3O2 LMT";
$resn_exch{"2H1  LMT"}=" H12 LMT"; $resn_exch{"1H1  LMT"}=" H11 LMT"; $resn_exch{"2H2  LMT"}=" H22 LMT";
$resn_exch{"1H2  LMT"}=" H21 LMT"; $resn_exch{"2H3  LMT"}=" H32 LMT"; $resn_exch{"1H3  LMT"}=" H31 LMT";
$resn_exch{"2H4  LMT"}=" H42 LMT"; $resn_exch{"1H4  LMT"}=" H41 LMT"; $resn_exch{"2H5  LMT"}=" H52 LMT";
$resn_exch{"1H5  LMT"}=" H51 LMT"; $resn_exch{"2H6  LMT"}=" H62 LMT"; $resn_exch{"1H6  LMT"}=" H61 LMT";
$resn_exch{"2H7  LMT"}=" H72 LMT"; $resn_exch{"1H7  LMT"}=" H71 LMT"; $resn_exch{"2H8  LMT"}=" H82 LMT";
$resn_exch{"1H8  LMT"}=" H81 LMT"; $resn_exch{"2H9  LMT"}=" H92 LMT"; $resn_exch{"1H9  LMT"}=" H91 LMT";
$resn_exch{"2H10 LMT"}="H102 LMT"; $resn_exch{"1H10 LMT"}="H101 LMT"; $resn_exch{"2H11 LMT"}="H112 LMT";
$resn_exch{"1H11 LMT"}="H111 LMT"; $resn_exch{"3H12 LMT"}="H123 LMT"; $resn_exch{"2H12 LMT"}="H122 LMT";
$resn_exch{"1H12 LMT"}="H121 LMT"; $resn_exch{" H1* LMU"}=" H1B LMU"; $resn_exch{" H2* LMU"}=" H2B LMU";
$resn_exch{" H3* LMU"}=" H3B LMU"; $resn_exch{" H4* LMU"}=" H4B LMU"; $resn_exch{" H5* LMU"}=" H5B LMU";
$resn_exch{"2H6* LMU"}="H6'2 LMU"; $resn_exch{"1H6* LMU"}="H6'1 LMU"; $resn_exch{"1H2O LMU"}="H2O1 LMU";
$resn_exch{"1H3O LMU"}="H3O1 LMU"; $resn_exch{"1H4O LMU"}="H4O1 LMU"; $resn_exch{" H6* LMU"}=" H6B LMU";
$resn_exch{"2H6' LMU"}=" H6D LMU"; $resn_exch{"1H6' LMU"}=" H6E LMU"; $resn_exch{"2H2O LMU"}="H2O2 LMU";
$resn_exch{"2H3O LMU"}="H3O2 LMU"; $resn_exch{"2H1  LMU"}=" H12 LMU"; $resn_exch{"1H1  LMU"}=" H11 LMU";
$resn_exch{"2H2  LMU"}=" H22 LMU"; $resn_exch{"1H2  LMU"}=" H21 LMU"; $resn_exch{"2H3  LMU"}=" H32 LMU";
$resn_exch{"1H3  LMU"}=" H31 LMU"; $resn_exch{"2H4  LMU"}=" H42 LMU"; $resn_exch{"1H4  LMU"}=" H41 LMU";
$resn_exch{"2H5  LMU"}=" H52 LMU"; $resn_exch{"1H5  LMU"}=" H51 LMU"; $resn_exch{"2H6  LMU"}=" H62 LMU";
$resn_exch{"1H6  LMU"}=" H61 LMU"; $resn_exch{"2H7  LMU"}=" H72 LMU"; $resn_exch{"1H7  LMU"}=" H71 LMU";
$resn_exch{"2H8  LMU"}=" H82 LMU"; $resn_exch{"1H8  LMU"}=" H81 LMU"; $resn_exch{"2H9  LMU"}=" H92 LMU";
$resn_exch{"1H9  LMU"}=" H91 LMU"; $resn_exch{"2H10 LMU"}="H102 LMU"; $resn_exch{"1H10 LMU"}="H101 LMU";
$resn_exch{"2H11 LMU"}="H112 LMU"; $resn_exch{"1H11 LMU"}="H111 LMU"; $resn_exch{"3H12 LMU"}="H123 LMU";
$resn_exch{"2H12 LMU"}="H122 LMU"; $resn_exch{"1H12 LMU"}="H121 LMU"; $resn_exch{"1H8  LMZ"}=" H81 LMZ";
$resn_exch{"2H8  LMZ"}=" H82 LMZ"; $resn_exch{"1H12 LMZ"}="H121 LMZ"; $resn_exch{"2H12 LMZ"}="H122 LMZ";
$resn_exch{"1HN7 LNC"}="HN71 LNC"; $resn_exch{"2HN7 LNC"}="HN72 LNC"; $resn_exch{"1HGS LNC"}="HGS1 LNC";
$resn_exch{"2HGS LNC"}="HGS2 LNC"; $resn_exch{"1HBS LNC"}="HBS1 LNC"; $resn_exch{"2HBS LNC"}="HBS2 LNC";
$resn_exch{" H1* LNC"}=" H1' LNC"; $resn_exch{" H4* LNC"}=" H4' LNC"; $resn_exch{"1H5* LNC"}="H5'1 LNC";
$resn_exch{"2H5* LNC"}="H5'2 LNC"; $resn_exch{"1H1  LNC"}=" H11 LNC"; $resn_exch{"2H1  LNC"}=" H12 LNC";
$resn_exch{" H2* LNC"}=" H2' LNC"; $resn_exch{" H3* LNC"}=" H3' LNC"; $resn_exch{"1HN6 LNC"}="HN61 LNC";
$resn_exch{"2HN6 LNC"}="HN62 LNC"; $resn_exch{"H11  LNC"}="H111 LNC"; $resn_exch{"H12  LNC"}="H122 LNC";
$resn_exch{"1H2C LNG"}="H2C1 LNG"; $resn_exch{"2H2C LNG"}="H2C2 LNG"; $resn_exch{"1H3C LNG"}="H3C1 LNG";
$resn_exch{"2H3C LNG"}="H3C2 LNG"; $resn_exch{"1H4C LNG"}="H4C1 LNG"; $resn_exch{"2H4C LNG"}="H4C2 LNG";
$resn_exch{"1H5C LNG"}="H5C1 LNG"; $resn_exch{"2H5C LNG"}="H5C2 LNG"; $resn_exch{"1H6C LNG"}="H6C1 LNG";
$resn_exch{"2H6C LNG"}="H6C2 LNG"; $resn_exch{"1H7C LNG"}="H7C1 LNG"; $resn_exch{"2H7C LNG"}="H7C2 LNG";
$resn_exch{"1H8C LNG"}="H8C1 LNG"; $resn_exch{"2H8C LNG"}="H8C2 LNG"; $resn_exch{"1H9C LNG"}="H9C1 LNG";
$resn_exch{"2H9C LNG"}="H9C2 LNG"; $resn_exch{"1H10 LNG"}="H101 LNG"; $resn_exch{"2H10 LNG"}="H102 LNG";
$resn_exch{"1H11 LNG"}="H111 LNG"; $resn_exch{"2H11 LNG"}="H112 LNG"; $resn_exch{"1H13 LNG"}="H131 LNG";
$resn_exch{"2H13 LNG"}="H132 LNG"; $resn_exch{"3H13 LNG"}="H133 LNG"; $resn_exch{"1H14 LNG"}="H141 LNG";
$resn_exch{"2H14 LNG"}="H142 LNG"; $resn_exch{"3H14 LNG"}="H143 LNG"; $resn_exch{"1H2  LNL"}=" H21 LNL";
$resn_exch{"2H2  LNL"}=" H22 LNL"; $resn_exch{"1H3  LNL"}=" H31 LNL"; $resn_exch{"2H3  LNL"}=" H32 LNL";
$resn_exch{"1H4  LNL"}=" H41 LNL"; $resn_exch{"2H4  LNL"}=" H42 LNL"; $resn_exch{"1H5  LNL"}=" H51 LNL";
$resn_exch{"2H5  LNL"}=" H52 LNL"; $resn_exch{"1H6  LNL"}=" H61 LNL"; $resn_exch{"2H6  LNL"}=" H62 LNL";
$resn_exch{"1H7  LNL"}=" H71 LNL"; $resn_exch{"2H7  LNL"}=" H72 LNL"; $resn_exch{"1H8  LNL"}=" H81 LNL";
$resn_exch{"2H8  LNL"}=" H82 LNL"; $resn_exch{"1H11 LNL"}="H111 LNL"; $resn_exch{"2H11 LNL"}="H112 LNL";
$resn_exch{"1H14 LNL"}="H141 LNL"; $resn_exch{"2H14 LNL"}="H142 LNL"; $resn_exch{"1H17 LNL"}="H171 LNL";
$resn_exch{"2H17 LNL"}="H172 LNL"; $resn_exch{"1H18 LNL"}="H181 LNL"; $resn_exch{"2H18 LNL"}="H182 LNL";
$resn_exch{"3H18 LNL"}="H183 LNL"; $resn_exch{"1HN  LNO"}=" HN1 LNO"; $resn_exch{"2HN  LNO"}="HN2A LNO";
$resn_exch{"1HB  LNO"}=" HB1 LNO"; $resn_exch{"2HB  LNO"}=" HB2 LNO"; $resn_exch{"1HD1 LNO"}="HD11 LNO";
$resn_exch{"2HD1 LNO"}="HD12 LNO"; $resn_exch{"3HD1 LNO"}="HD13 LNO"; $resn_exch{"1HD2 LNO"}="HD21 LNO";
$resn_exch{"2HD2 LNO"}="HD22 LNO"; $resn_exch{"3HD2 LNO"}="HD23 LNO"; $resn_exch{"1H1  LNQ"}=" H11 LNQ";
$resn_exch{"2H1  LNQ"}=" H12 LNQ"; $resn_exch{"3H1  LNQ"}=" H13 LNQ"; $resn_exch{"1H2  LNQ"}=" H21 LNQ";
$resn_exch{"2H2  LNQ"}=" H22 LNQ"; $resn_exch{"1H3  LNQ"}=" H31 LNQ"; $resn_exch{"2H3  LNQ"}=" H32 LNQ";
$resn_exch{"1H5  LNQ"}=" H51 LNQ"; $resn_exch{"2H5  LNQ"}=" H52 LNQ"; $resn_exch{"1H6  LNQ"}=" H61 LNQ";
$resn_exch{"2H6  LNQ"}=" H62 LNQ"; $resn_exch{"1H8  LNQ"}=" H81 LNQ"; $resn_exch{"2H8  LNQ"}=" H82 LNQ";
$resn_exch{"1H9  LNQ"}=" H91 LNQ"; $resn_exch{"2H9  LNQ"}=" H92 LNQ"; $resn_exch{"1H11 LNQ"}="H111 LNQ";
$resn_exch{"2H11 LNQ"}="H112 LNQ"; $resn_exch{"1H12 LNQ"}="H121 LNQ"; $resn_exch{"2H12 LNQ"}="H122 LNQ";
$resn_exch{"1H13 LNQ"}="H131 LNQ"; $resn_exch{"2H13 LNQ"}="H132 LNQ"; $resn_exch{"1H14 LNQ"}="H141 LNQ";
$resn_exch{"1H21 LNQ"}="H211 LNQ"; $resn_exch{"2H21 LNQ"}="H212 LNQ"; $resn_exch{"1H8  LNR"}=" H81 LNR";
$resn_exch{"2H8  LNR"}=" H82 LNR"; $resn_exch{"1HN1 LNR"}="HN11 LNR"; $resn_exch{"2HN1 LNR"}="HN12 LNR";
$resn_exch{"1HC  LO1"}=" HC1 LO1"; $resn_exch{"2HC  LO1"}=" HC2 LO1"; $resn_exch{"1H1N LO1"}="H1N1 LO1";
$resn_exch{"2H1N LO1"}="H1N2 LO1"; $resn_exch{"1H8C LOB"}="H8C1 LOB"; $resn_exch{"2H8C LOB"}="H8C2 LOB";
$resn_exch{"1H12 LOB"}="H121 LOB"; $resn_exch{"2H12 LOB"}="H122 LOB"; $resn_exch{"1H15 LOB"}="H151 LOB";
$resn_exch{"2H15 LOB"}="H152 LOB"; $resn_exch{"1H13 LOB"}="H131 LOB"; $resn_exch{"2H13 LOB"}="H132 LOB";
$resn_exch{"1H22 LOB"}="H221 LOB"; $resn_exch{"2H22 LOB"}="H222 LOB"; $resn_exch{"3H22 LOB"}="H223 LOB";
$resn_exch{"1H14 LOB"}="H141 LOB"; $resn_exch{"2H14 LOB"}="H142 LOB"; $resn_exch{"1H3  LOF"}=" H31 LOF";
$resn_exch{"2H3  LOF"}=" H32 LOF"; $resn_exch{"1HD1 LOL"}="HD11 LOL"; $resn_exch{"2HD1 LOL"}="HD12 LOL";
$resn_exch{"3HD1 LOL"}="HD13 LOL"; $resn_exch{"1HD2 LOL"}="HD21 LOL"; $resn_exch{"2HD2 LOL"}="HD22 LOL";
$resn_exch{"3HD2 LOL"}="HD23 LOL"; $resn_exch{"1HB  LOL"}=" HB3 LOL"; $resn_exch{"2HB  LOL"}=" HB2 LOL";
$resn_exch{"1HN  LOL"}=" H   LOL"; $resn_exch{"2HN  LOL"}=" H2  LOL"; $resn_exch{" H   LOL"}=" HC  LOL";
$resn_exch{"1HN1 LOP"}="HN11 LOP"; $resn_exch{"2HN1 LOP"}="HN12 LOP"; $resn_exch{"1H1  LOP"}=" H11 LOP";
$resn_exch{"2H1  LOP"}=" H12 LOP"; $resn_exch{"1H2  LOP"}=" H21 LOP"; $resn_exch{"2H2  LOP"}=" H22 LOP";
$resn_exch{"1H3  LOP"}=" H31 LOP"; $resn_exch{"2H3  LOP"}=" H32 LOP"; $resn_exch{"1H5  LOP"}=" H51 LOP";
$resn_exch{"2H5  LOP"}=" H52 LOP"; $resn_exch{"1H7  LOP"}=" H71 LOP"; $resn_exch{"2H7  LOP"}=" H72 LOP";
$resn_exch{"1H8  LOP"}=" H81 LOP"; $resn_exch{"2H8  LOP"}=" H82 LOP"; $resn_exch{"1H9  LOP"}=" H91 LOP";
$resn_exch{"2H9  LOP"}=" H92 LOP"; $resn_exch{"1H10 LOP"}="H101 LOP"; $resn_exch{"2H10 LOP"}="H102 LOP";
$resn_exch{"1H11 LOP"}="H111 LOP"; $resn_exch{"2H11 LOP"}="H112 LOP"; $resn_exch{"1H12 LOP"}="H121 LOP";
$resn_exch{"2H12 LOP"}="H122 LOP"; $resn_exch{"1H13 LOP"}="H131 LOP"; $resn_exch{"2H13 LOP"}="H132 LOP";
$resn_exch{"1H16 LOP"}="H161 LOP"; $resn_exch{"2H16 LOP"}="H162 LOP"; $resn_exch{"1H17 LOP"}="H171 LOP";
$resn_exch{"2H17 LOP"}="H172 LOP"; $resn_exch{"1H18 LOP"}="H181 LOP"; $resn_exch{"2H18 LOP"}="H182 LOP";
$resn_exch{"1H19 LOP"}="H191 LOP"; $resn_exch{"2H19 LOP"}="H192 LOP"; $resn_exch{"1H20 LOP"}="H201 LOP";
$resn_exch{"2H20 LOP"}="H202 LOP"; $resn_exch{"1H21 LOP"}="H211 LOP"; $resn_exch{"2H21 LOP"}="H212 LOP";
$resn_exch{"1H22 LOP"}="H221 LOP"; $resn_exch{"2H22 LOP"}="H222 LOP"; $resn_exch{"1H23 LOP"}="H231 LOP";
$resn_exch{"2H23 LOP"}="H232 LOP"; $resn_exch{"3H23 LOP"}="H233 LOP"; $resn_exch{"1H25 LOP"}="H251 LOP";
$resn_exch{"2H25 LOP"}="H252 LOP"; $resn_exch{"1H26 LOP"}="H261 LOP"; $resn_exch{"2H26 LOP"}="H262 LOP";
$resn_exch{"1H27 LOP"}="H271 LOP"; $resn_exch{"2H27 LOP"}="H272 LOP"; $resn_exch{"1H28 LOP"}="H281 LOP";
$resn_exch{"2H28 LOP"}="H282 LOP"; $resn_exch{"1H29 LOP"}="H291 LOP"; $resn_exch{"2H29 LOP"}="H292 LOP";
$resn_exch{"1H30 LOP"}="H301 LOP"; $resn_exch{"2H30 LOP"}="H302 LOP"; $resn_exch{"1H31 LOP"}="H311 LOP";
$resn_exch{"2H31 LOP"}="H312 LOP"; $resn_exch{"1H32 LOP"}="H321 LOP"; $resn_exch{"2H32 LOP"}="H322 LOP";
$resn_exch{"1H33 LOP"}="H331 LOP"; $resn_exch{"2H33 LOP"}="H332 LOP"; $resn_exch{"1H34 LOP"}="H341 LOP";
$resn_exch{"2H34 LOP"}="H342 LOP"; $resn_exch{"1H35 LOP"}="H351 LOP"; $resn_exch{"2H35 LOP"}="H352 LOP";
$resn_exch{"3H35 LOP"}="H353 LOP"; $resn_exch{"1H5  LOR"}=" H51 LOR"; $resn_exch{"2H5  LOR"}=" H52 LOR";
$resn_exch{"1H4  LOR"}=" H41 LOR"; $resn_exch{"2H4  LOR"}=" H42 LOR"; $resn_exch{"1H22 LOR"}="H221 LOR";
$resn_exch{"2H22 LOR"}="H222 LOR"; $resn_exch{"1H3  LOS"}=" H31 LOS"; $resn_exch{"2H3  LOS"}=" H32 LOS";
$resn_exch{"1H4  LOS"}=" H41 LOS"; $resn_exch{"2H4  LOS"}=" H42 LOS"; $resn_exch{"1H5  LOS"}=" H51 LOS";
$resn_exch{"1H6  LOS"}=" H61 LOS"; $resn_exch{"7H7  LOS"}=" H77 LOS"; $resn_exch{"8H8  LOS"}=" H88 LOS";
$resn_exch{"1H9  LOS"}=" H91 LOS"; $resn_exch{"2H9  LOS"}=" H92 LOS"; $resn_exch{"1H10 LOS"}="H101 LOS";
$resn_exch{"2H10 LOS"}="H102 LOS"; $resn_exch{"1H11 LOS"}="H111 LOS"; $resn_exch{"2H11 LOS"}="H112 LOS";
$resn_exch{"1H12 LOS"}="H121 LOS"; $resn_exch{"2H12 LOS"}="H122 LOS"; $resn_exch{"1H27 LOS"}="H271 LOS";
$resn_exch{"2H27 LOS"}="H272 LOS"; $resn_exch{"1H28 LOS"}="H281 LOS"; $resn_exch{"2H28 LOS"}="H282 LOS";
$resn_exch{"1H29 LOS"}="H291 LOS"; $resn_exch{"2H29 LOS"}="H292 LOS"; $resn_exch{"1H30 LOS"}="H301 LOS";
$resn_exch{"2H30 LOS"}="H302 LOS"; $resn_exch{"31H3 LOS"}="H331 LOS"; $resn_exch{"32H3 LOS"}="H332 LOS";
$resn_exch{"1H33 LOS"}="H33A LOS"; $resn_exch{"2H33 LOS"}="H33B LOS"; $resn_exch{"1H34 LOS"}="H341 LOS";
$resn_exch{"2H34 LOS"}="H342 LOS"; $resn_exch{"1H35 LOS"}="H351 LOS"; $resn_exch{"2H35 LOS"}="H352 LOS";
$resn_exch{"1H36 LOS"}="H361 LOS"; $resn_exch{"2H36 LOS"}="H362 LOS"; $resn_exch{"1HG  LOS"}=" HG1 LOS";
$resn_exch{"2HG  LOS"}=" HG2 LOS"; $resn_exch{"1HD2 LOS"}="HD21 LOS"; $resn_exch{"2HD2 LOS"}="HD22 LOS";
$resn_exch{"1HE1 LOS"}="HE11 LOS"; $resn_exch{"2HE1 LOS"}="HE12 LOS"; $resn_exch{"1HE2 LOS"}="HE21 LOS";
$resn_exch{"1HD1 LOV"}="HD11 LOV"; $resn_exch{"2HD1 LOV"}="HD12 LOV"; $resn_exch{"3HD1 LOV"}="HD13 LOV";
$resn_exch{"1HD2 LOV"}="HD21 LOV"; $resn_exch{"2HD2 LOV"}="HD22 LOV"; $resn_exch{"3HD2 LOV"}="HD23 LOV";
$resn_exch{"1H1B LOV"}="H1B1 LOV"; $resn_exch{"2H1B LOV"}="H1B2 LOV"; $resn_exch{"1HS  LOV"}=" HS1 LOV";
$resn_exch{"1HN  LOV"}=" H   LOV"; $resn_exch{"2HN  LOV"}=" H2  LOV"; $resn_exch{"1HT  LOV"}=" HT1 LOV";
$resn_exch{"2HT  LOV"}=" HT2 LOV"; $resn_exch{"1HG1 LOV"}="HG11 LOV"; $resn_exch{"2HG1 LOV"}="HG12 LOV";
$resn_exch{"3HG1 LOV"}="HG13 LOV"; $resn_exch{"1HG2 LOV"}="HG21 LOV"; $resn_exch{"2HG2 LOV"}="HG22 LOV";
$resn_exch{"3HG2 LOV"}="HG23 LOV"; $resn_exch{"1H5  LOX"}=" H51 LOX"; $resn_exch{"2H5  LOX"}=" H52 LOX";
$resn_exch{"1H1  LP1"}=" H11 LP1"; $resn_exch{"2H1  LP1"}=" H12 LP1"; $resn_exch{"3H1  LP1"}="H13A LP1";
$resn_exch{"1H24 LP1"}="H241 LP1"; $resn_exch{"2H24 LP1"}="H242 LP1"; $resn_exch{"3H24 LP1"}="H243 LP1";
$resn_exch{"1H30 LP1"}="H301 LP1"; $resn_exch{"2H30 LP1"}="H302 LP1"; $resn_exch{"3H30 LP1"}="H303 LP1";
$resn_exch{"1H31 LP1"}="H311 LP1"; $resn_exch{"2H31 LP1"}="H312 LP1"; $resn_exch{"3H31 LP1"}="H313 LP1";
$resn_exch{"1H33 LP1"}="H331 LP1"; $resn_exch{"2H33 LP1"}="H332 LP1"; $resn_exch{"1H6  LP1"}=" H61 LP1";
$resn_exch{"2H6  LP1"}=" H62 LP1"; $resn_exch{"1H25 LP1"}="H251 LP1"; $resn_exch{"2H25 LP1"}="H252 LP1";
$resn_exch{"3H25 LP1"}="H253 LP1"; $resn_exch{"1H28 LP1"}="H281 LP1"; $resn_exch{"2H28 LP1"}="H282 LP1";
$resn_exch{"1H39 LP1"}="H391 LP1"; $resn_exch{"2H39 LP1"}="H392 LP1"; $resn_exch{"1H40 LP1"}="H401 LP1";
$resn_exch{"2H40 LP1"}="H402 LP1"; $resn_exch{"3H40 LP1"}="H403 LP1"; $resn_exch{"1H45 LP1"}="H451 LP1";
$resn_exch{"2H45 LP1"}="H452 LP1"; $resn_exch{"1HN  LP1"}=" HN1 LP1"; $resn_exch{"2HN  LP1"}=" HN2 LP1";
$resn_exch{"1H1  LP2"}=" H11 LP2"; $resn_exch{"2H1  LP2"}=" H12 LP2"; $resn_exch{"3H1  LP2"}="H13A LP2";
$resn_exch{"1H6  LP2"}=" H61 LP2"; $resn_exch{"2H6  LP2"}=" H62 LP2"; $resn_exch{"1H24 LP2"}="H241 LP2";
$resn_exch{"2H24 LP2"}="H242 LP2"; $resn_exch{"3H24 LP2"}="H243 LP2"; $resn_exch{"1H25 LP2"}="H251 LP2";
$resn_exch{"2H25 LP2"}="H252 LP2"; $resn_exch{"3H25 LP2"}="H253 LP2"; $resn_exch{"1H28 LP2"}="H281 LP2";
$resn_exch{"2H28 LP2"}="H282 LP2"; $resn_exch{"1H30 LP2"}="H301 LP2"; $resn_exch{"2H30 LP2"}="H302 LP2";
$resn_exch{"3H30 LP2"}="H303 LP2"; $resn_exch{"1H31 LP2"}="H311 LP2"; $resn_exch{"2H31 LP2"}="H312 LP2";
$resn_exch{"3H31 LP2"}="H313 LP2"; $resn_exch{"1H33 LP2"}="H331 LP2"; $resn_exch{"2H33 LP2"}="H332 LP2";
$resn_exch{"1H39 LP2"}="H391 LP2"; $resn_exch{"2H39 LP2"}="H392 LP2"; $resn_exch{"1H40 LP2"}="H401 LP2";
$resn_exch{"2H40 LP2"}="H402 LP2"; $resn_exch{"3H40 LP2"}="H403 LP2"; $resn_exch{"1H45 LP2"}="H451 LP2";
$resn_exch{"2H45 LP2"}="H452 LP2"; $resn_exch{"1HN  LP2"}=" HN1 LP2"; $resn_exch{"2HN  LP2"}=" HN2 LP2";
$resn_exch{"1H1  LP3"}=" H11 LP3"; $resn_exch{"2H1  LP3"}=" H12 LP3"; $resn_exch{"1H3  LP3"}=" H31 LP3";
$resn_exch{"2H3  LP3"}=" H32 LP3"; $resn_exch{"1H4  LP3"}=" H41 LP3"; $resn_exch{"2H4  LP3"}=" H42 LP3";
$resn_exch{"1H5  LP3"}=" H51 LP3"; $resn_exch{"2H5  LP3"}=" H52 LP3"; $resn_exch{"1H6  LP3"}=" H61 LP3";
$resn_exch{"2H6  LP3"}=" H62 LP3"; $resn_exch{"3H6  LP3"}=" H63 LP3"; $resn_exch{"1H7  LP3"}=" H71 LP3";
$resn_exch{"2H7  LP3"}=" H72 LP3"; $resn_exch{"3H7  LP3"}=" H73 LP3"; $resn_exch{"1H8  LP3"}=" H81 LP3";
$resn_exch{"2H8  LP3"}=" H82 LP3"; $resn_exch{"3H8  LP3"}=" H83 LP3"; $resn_exch{"1H12 LP3"}="H121 LP3";
$resn_exch{"2H12 LP3"}="H122 LP3"; $resn_exch{"1H13 LP3"}="H131 LP3"; $resn_exch{"2H13 LP3"}="H132 LP3";
$resn_exch{"1H14 LP3"}="H141 LP3"; $resn_exch{"2H14 LP3"}="H142 LP3"; $resn_exch{"1H15 LP3"}="H151 LP3";
$resn_exch{"2H15 LP3"}="H152 LP3"; $resn_exch{"1H16 LP3"}="H161 LP3"; $resn_exch{"2H16 LP3"}="H162 LP3";
$resn_exch{"1H17 LP3"}="H171 LP3"; $resn_exch{"2H17 LP3"}="H172 LP3"; $resn_exch{"1H18 LP3"}="H181 LP3";
$resn_exch{"2H18 LP3"}="H182 LP3"; $resn_exch{"1H19 LP3"}="H191 LP3"; $resn_exch{"2H19 LP3"}="H192 LP3";
$resn_exch{"1H20 LP3"}="H201 LP3"; $resn_exch{"2H20 LP3"}="H202 LP3"; $resn_exch{"1H21 LP3"}="H211 LP3";
$resn_exch{"2H21 LP3"}="H212 LP3"; $resn_exch{"1H22 LP3"}="H221 LP3"; $resn_exch{"2H22 LP3"}="H222 LP3";
$resn_exch{"1H23 LP3"}="H231 LP3"; $resn_exch{"2H23 LP3"}="H232 LP3"; $resn_exch{"1H24 LP3"}="H241 LP3";
$resn_exch{"2H24 LP3"}="H242 LP3"; $resn_exch{"1H25 LP3"}="H251 LP3"; $resn_exch{"2H25 LP3"}="H252 LP3";
$resn_exch{"1H26 LP3"}="H261 LP3"; $resn_exch{"2H26 LP3"}="H262 LP3"; $resn_exch{"1H27 LP3"}="H271 LP3";
$resn_exch{"2H27 LP3"}="H272 LP3"; $resn_exch{"1H28 LP3"}="H281 LP3"; $resn_exch{"2H28 LP3"}="H282 LP3";
$resn_exch{"3H28 LP3"}="H283 LP3"; $resn_exch{"1H6  LP4"}=" H61 LP4"; $resn_exch{"2H6  LP4"}=" H62 LP4";
$resn_exch{"1H29 LP4"}="H291 LP4"; $resn_exch{"2H29 LP4"}="H292 LP4"; $resn_exch{"1H31 LP4"}="H311 LP4";
$resn_exch{"2H31 LP4"}="H312 LP4"; $resn_exch{"1H32 LP4"}="H321 LP4"; $resn_exch{"2H32 LP4"}="H322 LP4";
$resn_exch{"1H33 LP4"}="H331 LP4"; $resn_exch{"2H33 LP4"}="H332 LP4"; $resn_exch{"1H34 LP4"}="H341 LP4";
$resn_exch{"2H34 LP4"}="H342 LP4"; $resn_exch{"1H35 LP4"}="H351 LP4"; $resn_exch{"2H35 LP4"}="H352 LP4";
$resn_exch{"1H36 LP4"}="H361 LP4"; $resn_exch{"2H36 LP4"}="H362 LP4"; $resn_exch{"1H37 LP4"}="H371 LP4";
$resn_exch{"2H37 LP4"}="H372 LP4"; $resn_exch{"1H38 LP4"}="H381 LP4"; $resn_exch{"2H38 LP4"}="H382 LP4";
$resn_exch{"1H39 LP4"}="H391 LP4"; $resn_exch{"2H39 LP4"}="H392 LP4"; $resn_exch{"1H40 LP4"}="H401 LP4";
$resn_exch{"2H40 LP4"}="H402 LP4"; $resn_exch{"1H41 LP4"}="H411 LP4"; $resn_exch{"2H41 LP4"}="H412 LP4";
$resn_exch{"3H41 LP4"}="H413 LP4"; $resn_exch{"1H1  LP4"}=" H11 LP4"; $resn_exch{"2H1  LP4"}=" H12 LP4";
$resn_exch{"1H8  LP4"}=" H81 LP4"; $resn_exch{"2H8  LP4"}=" H82 LP4"; $resn_exch{"1H17 LP4"}="H171 LP4";
$resn_exch{"2H17 LP4"}="H172 LP4"; $resn_exch{"1H18 LP4"}="H181 LP4"; $resn_exch{"2H18 LP4"}="H182 LP4";
$resn_exch{"1H19 LP4"}="H191 LP4"; $resn_exch{"2H19 LP4"}="H192 LP4"; $resn_exch{"1H20 LP4"}="H201 LP4";
$resn_exch{"2H20 LP4"}="H202 LP4"; $resn_exch{"1H21 LP4"}="H211 LP4"; $resn_exch{"2H21 LP4"}="H212 LP4";
$resn_exch{"1H22 LP4"}="H221 LP4"; $resn_exch{"2H22 LP4"}="H222 LP4"; $resn_exch{"1H23 LP4"}="H231 LP4";
$resn_exch{"2H23 LP4"}="H232 LP4"; $resn_exch{"1H24 LP4"}="H241 LP4"; $resn_exch{"2H24 LP4"}="H242 LP4";
$resn_exch{"1H25 LP4"}="H251 LP4"; $resn_exch{"2H25 LP4"}="H252 LP4"; $resn_exch{"1H26 LP4"}="H261 LP4";
$resn_exch{"2H26 LP4"}="H262 LP4"; $resn_exch{"1H27 LP4"}="H271 LP4"; $resn_exch{"2H27 LP4"}="H272 LP4";
$resn_exch{"3H27 LP4"}="H273 LP4"; $resn_exch{"1H8  LP5"}=" H81 LP5"; $resn_exch{"2H8  LP5"}=" H82 LP5";
$resn_exch{"1H17 LP5"}="H171 LP5"; $resn_exch{"2H17 LP5"}="H172 LP5"; $resn_exch{"1H18 LP5"}="H181 LP5";
$resn_exch{"2H18 LP5"}="H182 LP5"; $resn_exch{"1H19 LP5"}="H191 LP5"; $resn_exch{"2H19 LP5"}="H192 LP5";
$resn_exch{"1H20 LP5"}="H201 LP5"; $resn_exch{"2H20 LP5"}="H202 LP5"; $resn_exch{"1H21 LP5"}="H211 LP5";
$resn_exch{"2H21 LP5"}="H212 LP5"; $resn_exch{"1H22 LP5"}="H221 LP5"; $resn_exch{"2H22 LP5"}="H222 LP5";
$resn_exch{"1H23 LP5"}="H231 LP5"; $resn_exch{"2H23 LP5"}="H232 LP5"; $resn_exch{"1H24 LP5"}="H241 LP5";
$resn_exch{"2H24 LP5"}="H242 LP5"; $resn_exch{"1H25 LP5"}="H251 LP5"; $resn_exch{"2H25 LP5"}="H252 LP5";
$resn_exch{"1H26 LP5"}="H261 LP5"; $resn_exch{"2H26 LP5"}="H262 LP5"; $resn_exch{"1H27 LP5"}="H271 LP5";
$resn_exch{"2H27 LP5"}="H272 LP5"; $resn_exch{"3H27 LP5"}="H273 LP5"; $resn_exch{"1H6  LP5"}=" H61 LP5";
$resn_exch{"2H6  LP5"}=" H62 LP5"; $resn_exch{"1H29 LP5"}="H291 LP5"; $resn_exch{"2H29 LP5"}="H292 LP5";
$resn_exch{"1H31 LP5"}="H311 LP5"; $resn_exch{"2H31 LP5"}="H312 LP5"; $resn_exch{"1H32 LP5"}="H321 LP5";
$resn_exch{"2H32 LP5"}="H322 LP5"; $resn_exch{"1H33 LP5"}="H331 LP5"; $resn_exch{"2H33 LP5"}="H332 LP5";
$resn_exch{"1H34 LP5"}="H341 LP5"; $resn_exch{"2H34 LP5"}="H342 LP5"; $resn_exch{"1H35 LP5"}="H351 LP5";
$resn_exch{"2H35 LP5"}="H352 LP5"; $resn_exch{"1H36 LP5"}="H361 LP5"; $resn_exch{"2H36 LP5"}="H362 LP5";
$resn_exch{"1H37 LP5"}="H371 LP5"; $resn_exch{"2H37 LP5"}="H372 LP5"; $resn_exch{"1H38 LP5"}="H381 LP5";
$resn_exch{"2H38 LP5"}="H382 LP5"; $resn_exch{"1H39 LP5"}="H391 LP5"; $resn_exch{"2H39 LP5"}="H392 LP5";
$resn_exch{"1H40 LP5"}="H401 LP5"; $resn_exch{"2H40 LP5"}="H402 LP5"; $resn_exch{"1H41 LP5"}="H411 LP5";
$resn_exch{"2H41 LP5"}="H412 LP5"; $resn_exch{"3H41 LP5"}="H413 LP5"; $resn_exch{"1H2  LPA"}=" H21 LPA";
$resn_exch{"2H2  LPA"}=" H22 LPA"; $resn_exch{"1H3  LPA"}=" H31 LPA"; $resn_exch{"2H3  LPA"}=" H32 LPA";
$resn_exch{"1H4  LPA"}=" H41 LPA"; $resn_exch{"2H4  LPA"}=" H42 LPA"; $resn_exch{"1H5  LPA"}=" H51 LPA";
$resn_exch{"2H5  LPA"}=" H52 LPA"; $resn_exch{"1H7  LPA"}=" H71 LPA"; $resn_exch{"2H7  LPA"}=" H72 LPA";
$resn_exch{"1H8  LPA"}=" H81 LPA"; $resn_exch{"2H8  LPA"}=" H82 LPA"; $resn_exch{"1H0A LPC"}="H0A1 LPC";
$resn_exch{"2H0A LPC"}="H0A2 LPC"; $resn_exch{"3H0A LPC"}="H0A3 LPC"; $resn_exch{"1H0B LPC"}="H0B1 LPC";
$resn_exch{"2H0B LPC"}="H0B2 LPC"; $resn_exch{"3H0B LPC"}="H0B3 LPC"; $resn_exch{"1H0C LPC"}="H0C1 LPC";
$resn_exch{"2H0C LPC"}="H0C2 LPC"; $resn_exch{"3H0C LPC"}="H0C3 LPC"; $resn_exch{"1H2  LPC"}=" H21 LPC";
$resn_exch{"2H2  LPC"}=" H22 LPC"; $resn_exch{"1H3  LPC"}=" H31 LPC"; $resn_exch{"2H3  LPC"}=" H32 LPC";
$resn_exch{"AHO5 LPC"}="HO5A LPC"; $resn_exch{"1H7  LPC"}=" H71 LPC"; $resn_exch{"2H7  LPC"}=" H72 LPC";
$resn_exch{"1H9  LPC"}=" H91 LPC"; $resn_exch{"2H9  LPC"}=" H92 LPC"; $resn_exch{"1HB  LPC"}=" HB1 LPC";
$resn_exch{"2HB  LPC"}=" HB2 LPC"; $resn_exch{"1HC  LPC"}=" HC1 LPC"; $resn_exch{"2HC  LPC"}=" HC2 LPC";
$resn_exch{"1HD  LPC"}=" HD1 LPC"; $resn_exch{"2HD  LPC"}=" HD2 LPC"; $resn_exch{"1HE  LPC"}=" HE1 LPC";
$resn_exch{"2HE  LPC"}=" HE2 LPC"; $resn_exch{"1HF  LPC"}=" HF1 LPC"; $resn_exch{"2HF  LPC"}=" HF2 LPC";
$resn_exch{"1HG  LPC"}=" HG1 LPC"; $resn_exch{"2HG  LPC"}=" HG2 LPC"; $resn_exch{"1HH  LPC"}=" HH1 LPC";
$resn_exch{"2HH  LPC"}=" HH2 LPC"; $resn_exch{"1HI  LPC"}=" HI1 LPC"; $resn_exch{"2HI  LPC"}=" HI2 LPC";
$resn_exch{"1HJ  LPC"}=" HJ1 LPC"; $resn_exch{"2HJ  LPC"}=" HJ2 LPC"; $resn_exch{"1HK  LPC"}=" HK1 LPC";
$resn_exch{"2HK  LPC"}=" HK2 LPC"; $resn_exch{"1HL  LPC"}=" HL1 LPC"; $resn_exch{"2HL  LPC"}=" HL2 LPC";
$resn_exch{"1HM  LPC"}=" HM1 LPC"; $resn_exch{"2HM  LPC"}=" HM2 LPC"; $resn_exch{"1HN  LPC"}=" HN1 LPC";
$resn_exch{"2HN  LPC"}=" HN2 LPC"; $resn_exch{"3HN  LPC"}=" HN3 LPC"; $resn_exch{"1HN2 LPD"}="HN21 LPD";
$resn_exch{"2HN2 LPD"}="HN22 LPD"; $resn_exch{" HN  LPD"}=" H   LPD"; $resn_exch{"1HD  LPD"}=" HD3 LPD";
$resn_exch{"2HD  LPD"}=" HD2 LPD"; $resn_exch{"1HG  LPD"}=" HG3 LPD"; $resn_exch{"2HG  LPD"}=" HG2 LPD";
$resn_exch{"1HB  LPD"}=" HB2 LPD"; $resn_exch{"2HB  LPD"}=" HB3 LPD"; $resn_exch{"1H1  LPE"}=" H11 LPE";
$resn_exch{"2H1  LPE"}=" H12 LPE"; $resn_exch{"1H2  LPE"}=" H21 LPE"; $resn_exch{"1HO2 LPE"}="HO21 LPE";
$resn_exch{"1H3  LPE"}=" H31 LPE"; $resn_exch{"2H3  LPE"}=" H32 LPE"; $resn_exch{"1H11 LPE"}="H111 LPE";
$resn_exch{"2H11 LPE"}="H112 LPE"; $resn_exch{"1H12 LPE"}="H121 LPE"; $resn_exch{"2H12 LPE"}="H122 LPE";
$resn_exch{"1HO3 LPE"}="HO31 LPE"; $resn_exch{"1H31 LPE"}="H311 LPE"; $resn_exch{"2H31 LPE"}="H312 LPE";
$resn_exch{"1H32 LPE"}="H321 LPE"; $resn_exch{"2H32 LPE"}="H322 LPE"; $resn_exch{"1H1N LPE"}="H1N1 LPE";
$resn_exch{"2H1N LPE"}="H1N2 LPE"; $resn_exch{"3H1N LPE"}="H1N3 LPE"; $resn_exch{"1H2N LPE"}="H2N1 LPE";
$resn_exch{"2H2N LPE"}="H2N2 LPE"; $resn_exch{"3H2N LPE"}="H2N3 LPE"; $resn_exch{"1H3N LPE"}="H3N1 LPE";
$resn_exch{"2H3N LPE"}="H3N2 LPE"; $resn_exch{"3H3N LPE"}="H3N3 LPE"; $resn_exch{"1H13 LPE"}="H131 LPE";
$resn_exch{"2H13 LPE"}="H132 LPE"; $resn_exch{"1H14 LPE"}="H141 LPE"; $resn_exch{"2H14 LPE"}="H142 LPE";
$resn_exch{"1H15 LPE"}="H151 LPE"; $resn_exch{"2H15 LPE"}="H152 LPE"; $resn_exch{"1H16 LPE"}="H161 LPE";
$resn_exch{"2H16 LPE"}="H162 LPE"; $resn_exch{"1H17 LPE"}="H171 LPE"; $resn_exch{"2H17 LPE"}="H172 LPE";
$resn_exch{"1H18 LPE"}="H181 LPE"; $resn_exch{"2H18 LPE"}="H182 LPE"; $resn_exch{"1H19 LPE"}="H191 LPE";
$resn_exch{"2H19 LPE"}="H192 LPE"; $resn_exch{"1H20 LPE"}="H201 LPE"; $resn_exch{"2H20 LPE"}="H202 LPE";
$resn_exch{"1H21 LPE"}="H211 LPE"; $resn_exch{"2H21 LPE"}="H212 LPE"; $resn_exch{"1H22 LPE"}="H221 LPE";
$resn_exch{"2H22 LPE"}="H222 LPE"; $resn_exch{"1H23 LPE"}="H231 LPE"; $resn_exch{"2H23 LPE"}="H232 LPE";
$resn_exch{"1H24 LPE"}="H241 LPE"; $resn_exch{"2H24 LPE"}="H242 LPE"; $resn_exch{"1H25 LPE"}="H251 LPE";
$resn_exch{"2H25 LPE"}="H252 LPE"; $resn_exch{"1H26 LPE"}="H261 LPE"; $resn_exch{"2H26 LPE"}="H262 LPE";
$resn_exch{"1H27 LPE"}="H271 LPE"; $resn_exch{"2H27 LPE"}="H272 LPE"; $resn_exch{"1H28 LPE"}="H281 LPE";
$resn_exch{"2H28 LPE"}="H282 LPE"; $resn_exch{"3H28 LPE"}="H283 LPE"; $resn_exch{"1H4  LPF"}=" H41 LPF";
$resn_exch{"2H4  LPF"}=" H42 LPF"; $resn_exch{"1HL2 LPF"}="HL21 LPF"; $resn_exch{"2HL2 LPF"}="HL22 LPF";
$resn_exch{"3HL2 LPF"}="HL23 LPF"; $resn_exch{"1HA1 LPF"}="HA11 LPF"; $resn_exch{"2HA1 LPF"}="HA12 LPF";
$resn_exch{"1HA3 LPF"}="HA31 LPF"; $resn_exch{"2HA3 LPF"}="HA32 LPF"; $resn_exch{"3HA3 LPF"}="HA33 LPF";
$resn_exch{"1HA4 LPF"}="HA41 LPF"; $resn_exch{"2HA4 LPF"}="HA42 LPF"; $resn_exch{"3HA4 LPF"}="HA43 LPF";
$resn_exch{"1HN  LPG"}=" H   LPG"; $resn_exch{"2HN  LPG"}=" H2  LPG"; $resn_exch{"1HN  LPL"}=" H   LPL";
$resn_exch{"2HN  LPL"}=" H2  LPL"; $resn_exch{"1HB  LPL"}=" HB2 LPL"; $resn_exch{"2HB  LPL"}=" HB3 LPL";
$resn_exch{"1HD1 LPL"}="HD11 LPL"; $resn_exch{"2HD1 LPL"}="HD12 LPL"; $resn_exch{"3HD1 LPL"}="HD13 LPL";
$resn_exch{"1HD2 LPL"}="HD21 LPL"; $resn_exch{"2HD2 LPL"}="HD22 LPL"; $resn_exch{"3HD2 LPL"}="HD23 LPL";
$resn_exch{"1HM  LPL"}=" HM1 LPL"; $resn_exch{"2HM  LPL"}=" HM2 LPL"; $resn_exch{"1HB1 LPL"}="HB11 LPL";
$resn_exch{"2HB1 LPL"}="HB12 LPL"; $resn_exch{"1HD3 LPL"}="HD31 LPL"; $resn_exch{"2HD3 LPL"}="HD32 LPL";
$resn_exch{"3HD3 LPL"}="HD33 LPL"; $resn_exch{"1HD4 LPL"}="HD41 LPL"; $resn_exch{"2HD4 LPL"}="HD42 LPL";
$resn_exch{"3HD4 LPL"}="HD43 LPL"; $resn_exch{"1HN  LPM"}=" HN1 LPM"; $resn_exch{"2HN  LPM"}=" HN2 LPM";
$resn_exch{"1H2  LPM"}=" H21 LPM"; $resn_exch{"2H2  LPM"}=" H22 LPM"; $resn_exch{"1H3  LPM"}=" H31 LPM";
$resn_exch{"2H3  LPM"}=" H32 LPM"; $resn_exch{"1H4  LPM"}=" H41 LPM"; $resn_exch{"2H4  LPM"}=" H42 LPM";
$resn_exch{"1H5  LPM"}=" H51 LPM"; $resn_exch{"2H5  LPM"}=" H52 LPM"; $resn_exch{"1H7  LPM"}=" H71 LPM";
$resn_exch{"2H7  LPM"}=" H72 LPM"; $resn_exch{"1H8  LPM"}=" H81 LPM"; $resn_exch{"2H8  LPM"}=" H82 LPM";
$resn_exch{"1HC6 LPP"}="HC61 LPP"; $resn_exch{"2HC6 LPP"}="HC62 LPP"; $resn_exch{"1HC8 LPP"}="HC81 LPP";
$resn_exch{"2HC8 LPP"}="HC82 LPP"; $resn_exch{"1H12 LPP"}="H121 LPP"; $resn_exch{"2H12 LPP"}="H122 LPP";
$resn_exch{"1H13 LPP"}="H131 LPP"; $resn_exch{"2H13 LPP"}="H132 LPP"; $resn_exch{"1H14 LPP"}="H141 LPP";
$resn_exch{"2H14 LPP"}="H142 LPP"; $resn_exch{"1H15 LPP"}="H151 LPP"; $resn_exch{"2H15 LPP"}="H152 LPP";
$resn_exch{"1H16 LPP"}="H161 LPP"; $resn_exch{"2H16 LPP"}="H162 LPP"; $resn_exch{"1H17 LPP"}="H171 LPP";
$resn_exch{"2H17 LPP"}="H172 LPP"; $resn_exch{"1H18 LPP"}="H181 LPP"; $resn_exch{"2H18 LPP"}="H182 LPP";
$resn_exch{"1H19 LPP"}="H191 LPP"; $resn_exch{"2H19 LPP"}="H192 LPP"; $resn_exch{"1H20 LPP"}="H201 LPP";
$resn_exch{"2H20 LPP"}="H202 LPP"; $resn_exch{"1H21 LPP"}="H211 LPP"; $resn_exch{"2H21 LPP"}="H212 LPP";
$resn_exch{"1H22 LPP"}="H221 LPP"; $resn_exch{"2H22 LPP"}="H222 LPP"; $resn_exch{"1H23 LPP"}="H231 LPP";
$resn_exch{"2H23 LPP"}="H232 LPP"; $resn_exch{"1H24 LPP"}="H241 LPP"; $resn_exch{"2H24 LPP"}="H242 LPP";
$resn_exch{"1H25 LPP"}="H251 LPP"; $resn_exch{"2H25 LPP"}="H252 LPP"; $resn_exch{"1H26 LPP"}="H261 LPP";
$resn_exch{"2H26 LPP"}="H262 LPP"; $resn_exch{"3H26 LPP"}="H263 LPP"; $resn_exch{"1H30 LPP"}="H301 LPP";
$resn_exch{"2H30 LPP"}="H302 LPP"; $resn_exch{"1H31 LPP"}="H311 LPP"; $resn_exch{"2H31 LPP"}="H312 LPP";
$resn_exch{"1H32 LPP"}="H321 LPP"; $resn_exch{"2H32 LPP"}="H322 LPP"; $resn_exch{"1H33 LPP"}="H331 LPP";
$resn_exch{"2H33 LPP"}="H332 LPP"; $resn_exch{"1H34 LPP"}="H341 LPP"; $resn_exch{"2H34 LPP"}="H342 LPP";
$resn_exch{"1H35 LPP"}="H351 LPP"; $resn_exch{"2H35 LPP"}="H352 LPP"; $resn_exch{"1H36 LPP"}="H361 LPP";
$resn_exch{"2H36 LPP"}="H362 LPP"; $resn_exch{"1H37 LPP"}="H371 LPP"; $resn_exch{"2H37 LPP"}="H372 LPP";
$resn_exch{"1H38 LPP"}="H381 LPP"; $resn_exch{"2H38 LPP"}="H382 LPP"; $resn_exch{"1H39 LPP"}="H391 LPP";
$resn_exch{"2H39 LPP"}="H392 LPP"; $resn_exch{"1H40 LPP"}="H401 LPP"; $resn_exch{"2H40 LPP"}="H402 LPP";
$resn_exch{"1H41 LPP"}="H411 LPP"; $resn_exch{"2H41 LPP"}="H412 LPP"; $resn_exch{"1H42 LPP"}="H421 LPP";
$resn_exch{"2H42 LPP"}="H422 LPP"; $resn_exch{"1H43 LPP"}="H431 LPP"; $resn_exch{"2H43 LPP"}="H432 LPP";
$resn_exch{"1H44 LPP"}="H441 LPP"; $resn_exch{"2H44 LPP"}="H442 LPP"; $resn_exch{"3H44 LPP"}="H443 LPP";
$resn_exch{"1H3N LPR"}="H3N1 LPR"; $resn_exch{"2H3N LPR"}="H3N2 LPR"; $resn_exch{"2H6  LPR"}=" H62 LPR";
$resn_exch{"2H7  LPR"}=" H72 LPR"; $resn_exch{"1H8C LPR"}="H8C1 LPR"; $resn_exch{"2H8C LPR"}="H8C2 LPR";
$resn_exch{"1H10 LPR"}="H101 LPR"; $resn_exch{"2H10 LPR"}="H102 LPR"; $resn_exch{"1H11 LPR"}="H111 LPR";
$resn_exch{"2H11 LPR"}="H112 LPR"; $resn_exch{"1H12 LPR"}="H121 LPR"; $resn_exch{"2H12 LPR"}="H122 LPR";
$resn_exch{"1H13 LPR"}="H131 LPR"; $resn_exch{"2H13 LPR"}="H132 LPR"; $resn_exch{"1H14 LPR"}="H141 LPR";
$resn_exch{"2H14 LPR"}="H142 LPR"; $resn_exch{"1H15 LPR"}="H151 LPR"; $resn_exch{"2H15 LPR"}="H152 LPR";
$resn_exch{"1HN  LPS"}=" HN1 LPS"; $resn_exch{"2HN  LPS"}=" HN2 LPS"; $resn_exch{"1H11 LPS"}="H111 LPS";
$resn_exch{"2H11 LPS"}="H112 LPS"; $resn_exch{"1H1  LPS"}=" H11 LPS"; $resn_exch{"2H1  LPS"}="H12A LPS";
$resn_exch{"1H3  LPS"}="H31A LPS"; $resn_exch{"2H3  LPS"}="H32A LPS"; $resn_exch{"1H32 LPS"}="H321 LPS";
$resn_exch{"2H32 LPS"}="H322 LPS"; $resn_exch{"1H33 LPS"}="H331 LPS"; $resn_exch{"2H33 LPS"}="H332 LPS";
$resn_exch{"1H34 LPS"}="H341 LPS"; $resn_exch{"2H34 LPS"}="H342 LPS"; $resn_exch{"1H35 LPS"}="H351 LPS";
$resn_exch{"2H35 LPS"}="H352 LPS"; $resn_exch{"1H36 LPS"}="H361 LPS"; $resn_exch{"2H36 LPS"}="H362 LPS";
$resn_exch{"1H37 LPS"}="H371 LPS"; $resn_exch{"2H37 LPS"}="H372 LPS"; $resn_exch{"1H38 LPS"}="H381 LPS";
$resn_exch{"2H38 LPS"}="H382 LPS"; $resn_exch{"1H39 LPS"}="H391 LPS"; $resn_exch{"2H39 LPS"}="H392 LPS";
$resn_exch{"1H40 LPS"}="H401 LPS"; $resn_exch{"2H40 LPS"}="H402 LPS"; $resn_exch{"1H41 LPS"}="H411 LPS";
$resn_exch{"2H41 LPS"}="H412 LPS"; $resn_exch{"1H42 LPS"}="H421 LPS"; $resn_exch{"2H42 LPS"}="H422 LPS";
$resn_exch{"1H43 LPS"}="H431 LPS"; $resn_exch{"2H43 LPS"}="H432 LPS"; $resn_exch{"1H44 LPS"}="H441 LPS";
$resn_exch{"2H44 LPS"}="H442 LPS"; $resn_exch{"1H45 LPS"}="H451 LPS"; $resn_exch{"2H45 LPS"}="H452 LPS";
$resn_exch{"3H45 LPS"}="H453 LPS"; $resn_exch{"1H8  LPT"}=" H81 LPT"; $resn_exch{"2H8  LPT"}=" H82 LPT";
$resn_exch{"1H7  LPT"}=" H71 LPT"; $resn_exch{"2H7  LPT"}=" H72 LPT"; $resn_exch{"1H6  LPT"}=" H61 LPT";
$resn_exch{"2H6  LPT"}=" H62 LPT"; $resn_exch{"1H5  LPT"}=" H51 LPT"; $resn_exch{"2H5  LPT"}=" H52 LPT";
$resn_exch{"1H4  LPT"}=" H41 LPT"; $resn_exch{"2H4  LPT"}=" H42 LPT"; $resn_exch{"1HN2 LPT"}="HN21 LPT";
$resn_exch{"2HN2 LPT"}="HN22 LPT"; $resn_exch{"1HN1 LPT"}="HN11 LPT"; $resn_exch{"2HN1 LPT"}="HN12 LPT";
$resn_exch{"3HN1 LPT"}="HN13 LPT"; $resn_exch{"1H01 LQQ"}="H011 LQQ"; $resn_exch{"2H01 LQQ"}="H012 LQQ";
$resn_exch{"3H01 LQQ"}="H013 LQQ"; $resn_exch{"1H05 LQQ"}="H051 LQQ"; $resn_exch{"2H05 LQQ"}="H052 LQQ";
$resn_exch{"3H05 LQQ"}="H053 LQQ"; $resn_exch{"1H11 LQQ"}="H111 LQQ"; $resn_exch{"2H11 LQQ"}="H112 LQQ";
$resn_exch{"1H12 LQQ"}="H121 LQQ"; $resn_exch{"2H12 LQQ"}="H122 LQQ"; $resn_exch{"1H13 LQQ"}="H131 LQQ";
$resn_exch{"2H13 LQQ"}="H132 LQQ"; $resn_exch{"1H14 LQQ"}="H141 LQQ"; $resn_exch{"2H14 LQQ"}="H142 LQQ";
$resn_exch{"1H21 LQQ"}="H211 LQQ"; $resn_exch{"2H21 LQQ"}="H212 LQQ"; $resn_exch{"1H22 LQQ"}="H221 LQQ";
$resn_exch{"2H22 LQQ"}="H222 LQQ"; $resn_exch{"1H23 LQQ"}="H231 LQQ"; $resn_exch{"2H23 LQQ"}="H232 LQQ";
$resn_exch{"1H24 LQQ"}="H241 LQQ"; $resn_exch{"2H24 LQQ"}="H242 LQQ"; $resn_exch{"3H38 LRB"}="H383 LRB";
$resn_exch{"2H38 LRB"}="H382 LRB"; $resn_exch{"1H38 LRB"}="H381 LRB"; $resn_exch{"2H39 LRB"}="H392 LRB";
$resn_exch{"1H39 LRB"}="H391 LRB"; $resn_exch{"2H40 LRB"}="H402 LRB"; $resn_exch{"1H40 LRB"}="H401 LRB";
$resn_exch{"2H41 LRB"}="H412 LRB"; $resn_exch{"1H41 LRB"}="H411 LRB"; $resn_exch{"2H42 LRB"}="H422 LRB";
$resn_exch{"1H42 LRB"}="H421 LRB"; $resn_exch{"2H43 LRB"}="H432 LRB"; $resn_exch{"1H43 LRB"}="H431 LRB";
$resn_exch{"2H44 LRB"}="H442 LRB"; $resn_exch{"1H44 LRB"}="H441 LRB"; $resn_exch{"2H45 LRB"}="H452 LRB";
$resn_exch{"1H45 LRB"}="H451 LRB"; $resn_exch{"2H46 LRB"}="H462 LRB"; $resn_exch{"1H46 LRB"}="H461 LRB";
$resn_exch{"2H51 LRB"}="H512 LRB"; $resn_exch{"1H51 LRB"}="H511 LRB"; $resn_exch{"2H55 LRB"}="H552 LRB";
$resn_exch{"1H55 LRB"}="H551 LRB"; $resn_exch{"2H57 LRB"}="H572 LRB"; $resn_exch{"1H57 LRB"}="H571 LRB";
$resn_exch{"2H58 LRB"}="H582 LRB"; $resn_exch{"1H58 LRB"}="H581 LRB"; $resn_exch{"2H59 LRB"}="H592 LRB";
$resn_exch{"1H59 LRB"}="H591 LRB"; $resn_exch{"1H1C LRH"}="H1C1 LRH"; $resn_exch{"2H1C LRH"}="H1C2 LRH";
$resn_exch{"1H6C LRH"}="H6C1 LRH"; $resn_exch{"2H6C LRH"}="H6C2 LRH"; $resn_exch{"3H6C LRH"}="H6C3 LRH";
$resn_exch{"1H3  LRU"}=" H31 LRU"; $resn_exch{"2H3  LRU"}=" H32 LRU"; $resn_exch{"1H4  LRU"}=" H41 LRU";
$resn_exch{"2H4  LRU"}=" H42 LRU"; $resn_exch{"1H5  LRU"}=" H51 LRU"; $resn_exch{"1H6  LRU"}=" H61 LRU";
$resn_exch{"7H7  LRU"}=" H77 LRU"; $resn_exch{"8H8  LRU"}=" H88 LRU"; $resn_exch{"1H9  LRU"}=" H91 LRU";
$resn_exch{"2H9  LRU"}=" H92 LRU"; $resn_exch{"1H10 LRU"}="H101 LRU"; $resn_exch{"2H10 LRU"}="H102 LRU";
$resn_exch{"1H11 LRU"}="H111 LRU"; $resn_exch{"2H11 LRU"}="H112 LRU"; $resn_exch{"1H12 LRU"}="H121 LRU";
$resn_exch{"2H12 LRU"}="H122 LRU"; $resn_exch{"1H27 LRU"}="H271 LRU"; $resn_exch{"2H27 LRU"}="H272 LRU";
$resn_exch{"1H28 LRU"}="H281 LRU"; $resn_exch{"2H28 LRU"}="H282 LRU"; $resn_exch{"1H29 LRU"}="H291 LRU";
$resn_exch{"2H29 LRU"}="H292 LRU"; $resn_exch{"1H30 LRU"}="H301 LRU"; $resn_exch{"2H30 LRU"}="H302 LRU";
$resn_exch{"31H3 LRU"}="H331 LRU"; $resn_exch{"32H3 LRU"}="H332 LRU"; $resn_exch{"1H33 LRU"}="H33A LRU";
$resn_exch{"2H33 LRU"}="H33B LRU"; $resn_exch{"1H34 LRU"}="H341 LRU"; $resn_exch{"2H34 LRU"}="H342 LRU";
$resn_exch{"1H35 LRU"}="H351 LRU"; $resn_exch{"2H35 LRU"}="H352 LRU"; $resn_exch{"1H36 LRU"}="H361 LRU";
$resn_exch{"2H36 LRU"}="H362 LRU"; $resn_exch{"1HG  LRU"}=" HG1 LRU"; $resn_exch{"2HG  LRU"}=" HG2 LRU";
$resn_exch{"1HD2 LRU"}="HD21 LRU"; $resn_exch{"2HD2 LRU"}="HD22 LRU"; $resn_exch{"1HE1 LRU"}="HE11 LRU";
$resn_exch{"2HE1 LRU"}="HE12 LRU"; $resn_exch{"1HE2 LRU"}="HE21 LRU"; $resn_exch{" NH1 LS1"}=" H1N LS1";
$resn_exch{"1H4  LS1"}=" H41 LS1"; $resn_exch{"1H5  LS1"}=" H51 LS1"; $resn_exch{"1H6  LS1"}=" H61 LS1";
$resn_exch{"1H7  LS1"}=" H71 LS1"; $resn_exch{"1H12 LS1"}="H121 LS1"; $resn_exch{"NH17 LS1"}="H17N LS1";
$resn_exch{"1H21 LS1"}="H211 LS1"; $resn_exch{"1H22 LS1"}="H221 LS1"; $resn_exch{"1H24 LS1"}="H241 LS1";
$resn_exch{"1H25 LS1"}="H251 LS1"; $resn_exch{"NH33 LS1"}="H33N LS1"; $resn_exch{"1H35 LS1"}="H351 LS1";
$resn_exch{"2H35 LS1"}="H352 LS1"; $resn_exch{"3H35 LS1"}="H353 LS1"; $resn_exch{" NH1 LS2"}=" H1N LS2";
$resn_exch{"1H6  LS2"}=" H61 LS2"; $resn_exch{"1H7  LS2"}=" H71 LS2"; $resn_exch{"1H14 LS2"}="H141 LS2";
$resn_exch{"NH19 LS2"}="H19N LS2"; $resn_exch{"1H22 LS2"}="H221 LS2"; $resn_exch{"1H23 LS2"}="H231 LS2";
$resn_exch{"1H25 LS2"}="H251 LS2"; $resn_exch{"1H26 LS2"}="H261 LS2"; $resn_exch{"1H29 LS2"}="H291 LS2";
$resn_exch{"2H29 LS2"}="H292 LS2"; $resn_exch{"NH37 LS2"}="H37N LS2"; $resn_exch{"1H39 LS2"}="H391 LS2";
$resn_exch{"2H39 LS2"}="H392 LS2"; $resn_exch{"3H39 LS2"}="H393 LS2"; $resn_exch{" NH7 LS3"}=" H7N LS3";
$resn_exch{"1H9  LS3"}=" H91 LS3"; $resn_exch{"1H10 LS3"}="H101 LS3"; $resn_exch{"1H14 LS3"}="H141 LS3";
$resn_exch{"1H16 LS3"}="H161 LS3"; $resn_exch{"1H17 LS3"}="H171 LS3"; $resn_exch{"1H20 LS3"}="H201 LS3";
$resn_exch{"NH21 LS3"}="H21N LS3"; $resn_exch{"1H23 LS3"}="H231 LS3"; $resn_exch{"1H25 LS3"}="H251 LS3";
$resn_exch{"2H25 LS3"}="H252 LS3"; $resn_exch{"1H29 LS3"}="H291 LS3"; $resn_exch{"2H29 LS3"}="H292 LS3";
$resn_exch{"1H31 LS3"}="H311 LS3"; $resn_exch{"1H32 LS3"}="H321 LS3"; $resn_exch{"1H7  LS4"}=" H71 LS4";
$resn_exch{"1H9  LS4"}=" H91 LS4"; $resn_exch{"1H10 LS4"}="H101 LS4"; $resn_exch{"1H11 LS4"}="H111 LS4";
$resn_exch{"1H12 LS4"}="H121 LS4"; $resn_exch{"1H15 LS4"}="H151 LS4"; $resn_exch{"NH16 LS4"}="H16N LS4";
$resn_exch{"1H18 LS4"}="H181 LS4"; $resn_exch{"1H19 LS4"}="H191 LS4"; $resn_exch{"1H21 LS4"}="H211 LS4";
$resn_exch{"1H22 LS4"}="H221 LS4"; $resn_exch{"NH26 LS4"}="H26N LS4"; $resn_exch{"1H29 LS4"}="H291 LS4";
$resn_exch{"1H30 LS4"}="H301 LS4"; $resn_exch{" NH1 LS5"}=" H1N LS5"; $resn_exch{"1H4  LS5"}=" H41 LS5";
$resn_exch{"1H5  LS5"}=" H51 LS5"; $resn_exch{"1H6  LS5"}=" H61 LS5"; $resn_exch{"1H7  LS5"}=" H71 LS5";
$resn_exch{"1H12 LS5"}="H121 LS5"; $resn_exch{"NH17 LS5"}="H17N LS5"; $resn_exch{"1H21 LS5"}="H211 LS5";
$resn_exch{"1H22 LS5"}="H221 LS5"; $resn_exch{"1H24 LS5"}="H241 LS5"; $resn_exch{"1H25 LS5"}="H251 LS5";
$resn_exch{"NH33 LS5"}="H33N LS5"; $resn_exch{"NH36 LS5"}="H36N LS5"; $resn_exch{"1H37 LS5"}="H371 LS5";
$resn_exch{"2H37 LS5"}="H372 LS5"; $resn_exch{"1HN  LSO"}=" HN1 LSO"; $resn_exch{"2HN  LSO"}=" HN2 LSO";
$resn_exch{"1HBC LSO"}="HBC1 LSO"; $resn_exch{"2HBC LSO"}="HBC2 LSO"; $resn_exch{"1HGC LSO"}="HGC1 LSO";
$resn_exch{"2HGC LSO"}="HGC2 LSO"; $resn_exch{"1HDC LSO"}="HDC1 LSO"; $resn_exch{"2HDC LSO"}="HDC2 LSO";
$resn_exch{"1HEC LSO"}="HEC1 LSO"; $resn_exch{"2HEC LSO"}="HEC2 LSO"; $resn_exch{"1H3C LSO"}="H3C1 LSO";
$resn_exch{"2H3C LSO"}="H3C2 LSO"; $resn_exch{"1H2C LSO"}="H2C1 LSO"; $resn_exch{"2H2C LSO"}="H2C2 LSO";
$resn_exch{"1H5C LSO"}="H5C1 LSO"; $resn_exch{"2H5C LSO"}="H5C2 LSO"; $resn_exch{"1H7C LSO"}="H7C1 LSO";
$resn_exch{"2H7C LSO"}="H7C2 LSO"; $resn_exch{"1H9C LSO"}="H9C1 LSO"; $resn_exch{"2H9C LSO"}="H9C2 LSO";
$resn_exch{"1H10 LSO"}="H101 LSO"; $resn_exch{"2H10 LSO"}="H102 LSO"; $resn_exch{"1HN  LSP"}=" HN1 LSP";
$resn_exch{"2HN  LSP"}=" HN2 LSP"; $resn_exch{"3HN  LSP"}=" HN3 LSP"; $resn_exch{"1H12 LSP"}="H121 LSP";
$resn_exch{"2H12 LSP"}="H122 LSP"; $resn_exch{"1H11 LSP"}="H111 LSP"; $resn_exch{"2H11 LSP"}="H112 LSP";
$resn_exch{"1H32 LSP"}="H321 LSP"; $resn_exch{"2H32 LSP"}="H322 LSP"; $resn_exch{"3H32 LSP"}="H323 LSP";
$resn_exch{"1HN  LTA"}=" H   LTA"; $resn_exch{"2HN  LTA"}=" H2  LTA"; $resn_exch{"1HB  LTA"}=" HB2 LTA";
$resn_exch{"2HB  LTA"}=" HB3 LTA"; $resn_exch{"1HG  LTA"}=" HG2 LTA"; $resn_exch{"2HG  LTA"}=" HG3 LTA";
$resn_exch{"1HD  LTA"}=" HD2 LTA"; $resn_exch{"2HD  LTA"}=" HD3 LTA"; $resn_exch{"1HE  LTA"}=" HE2 LTA";
$resn_exch{"2HE  LTA"}=" HE3 LTA"; $resn_exch{"1HZ  LTA"}=" HZ1 LTA"; $resn_exch{"2HZ  LTA"}=" HZ2 LTA";
$resn_exch{" H   LTA"}=" HC  LTA"; $resn_exch{"1H2  LTA"}=" H21 LTA"; $resn_exch{"2H2  LTA"}=" H22 LTA";
$resn_exch{"1H9  LTA"}=" H91 LTA"; $resn_exch{"2H9  LTA"}=" H92 LTA"; $resn_exch{"1H10 LTA"}="H101 LTA";
$resn_exch{"2H10 LTA"}="H102 LTA"; $resn_exch{"3H10 LTA"}="H103 LTA"; $resn_exch{"1HC1 LTL"}="HC11 LTL";
$resn_exch{"2HC1 LTL"}="HC12 LTL"; $resn_exch{"3HC1 LTL"}="HC13 LTL"; $resn_exch{"1HC6 LTL"}="HC61 LTL";
$resn_exch{"2HC6 LTL"}="HC62 LTL"; $resn_exch{"1HC7 LTL"}="HC71 LTL"; $resn_exch{"2HC7 LTL"}="HC72 LTL";
$resn_exch{"1H13 LTL"}="H131 LTL"; $resn_exch{"2H13 LTL"}="H132 LTL"; $resn_exch{"1H14 LTL"}="H141 LTL";
$resn_exch{"2H14 LTL"}="H142 LTL"; $resn_exch{"3H14 LTL"}="H143 LTL"; $resn_exch{"1H22 LTL"}="H221 LTL";
$resn_exch{"2H22 LTL"}="H222 LTL"; $resn_exch{"3H22 LTL"}="H223 LTL"; $resn_exch{"1HH3 LTN"}="HH31 LTN";
$resn_exch{"2HH3 LTN"}="HH32 LTN"; $resn_exch{"1HB  LTN"}=" HB1 LTN"; $resn_exch{"2HB  LTN"}=" HB2 LTN";
$resn_exch{"1HN  LTN"}=" HN1 LTN"; $resn_exch{"2HN  LTN"}=" HN2 LTN"; $resn_exch{"1HB  LTR"}=" HB1 LTR";
$resn_exch{"2HB  LTR"}=" HB2 LTR"; $resn_exch{"1HN3 LTT"}="HN31 LTT"; $resn_exch{"2HN3 LTT"}="HN32 LTT";
$resn_exch{"1H8  LTT"}=" H81 LTT"; $resn_exch{"2H8  LTT"}=" H82 LTT"; $resn_exch{"1H6  LTT"}=" H61 LTT";
$resn_exch{"2H6  LTT"}=" H62 LTT"; $resn_exch{"1H7M LUM"}="H7M1 LUM"; $resn_exch{"2H7M LUM"}="H7M2 LUM";
$resn_exch{"3H7M LUM"}="H7M3 LUM"; $resn_exch{"1H8M LUM"}="H8M1 LUM"; $resn_exch{"2H8M LUM"}="H8M2 LUM";
$resn_exch{"3H8M LUM"}="H8M3 LUM"; $resn_exch{"1H2  LUT"}=" H21 LUT"; $resn_exch{"2H2  LUT"}=" H22 LUT";
$resn_exch{"1H4  LUT"}=" H41 LUT"; $resn_exch{"2H4  LUT"}=" H42 LUT"; $resn_exch{"1H16 LUT"}="H161 LUT";
$resn_exch{"2H16 LUT"}="H162 LUT"; $resn_exch{"3H16 LUT"}="H163 LUT"; $resn_exch{"1H17 LUT"}="H171 LUT";
$resn_exch{"2H17 LUT"}="H172 LUT"; $resn_exch{"3H17 LUT"}="H173 LUT"; $resn_exch{"1H18 LUT"}="H181 LUT";
$resn_exch{"2H18 LUT"}="H182 LUT"; $resn_exch{"3H18 LUT"}="H183 LUT"; $resn_exch{"1H19 LUT"}="H191 LUT";
$resn_exch{"2H19 LUT"}="H192 LUT"; $resn_exch{"3H19 LUT"}="H193 LUT"; $resn_exch{"1H20 LUT"}="H201 LUT";
$resn_exch{"2H20 LUT"}="H202 LUT"; $resn_exch{"3H20 LUT"}="H203 LUT"; $resn_exch{"1H22 LUT"}="H221 LUT";
$resn_exch{"2H22 LUT"}="H222 LUT"; $resn_exch{"1H36 LUT"}="H361 LUT"; $resn_exch{"2H36 LUT"}="H362 LUT";
$resn_exch{"3H36 LUT"}="H363 LUT"; $resn_exch{"1H37 LUT"}="H371 LUT"; $resn_exch{"2H37 LUT"}="H372 LUT";
$resn_exch{"3H37 LUT"}="H373 LUT"; $resn_exch{"1H38 LUT"}="H381 LUT"; $resn_exch{"2H38 LUT"}="H382 LUT";
$resn_exch{"3H38 LUT"}="H383 LUT"; $resn_exch{"1H39 LUT"}="H391 LUT"; $resn_exch{"2H39 LUT"}="H392 LUT";
$resn_exch{"3H39 LUT"}="H393 LUT"; $resn_exch{"1H40 LUT"}="H401 LUT"; $resn_exch{"2H40 LUT"}="H402 LUT";
$resn_exch{"3H40 LUT"}="H403 LUT"; $resn_exch{"1H2C LUX"}="H2C1 LUX"; $resn_exch{"2H2C LUX"}="H2C2 LUX";
$resn_exch{"1H4C LUX"}="H4C1 LUX"; $resn_exch{"2H4C LUX"}="H4C2 LUX"; $resn_exch{"1H10 LUX"}="H101 LUX";
$resn_exch{"2H10 LUX"}="H102 LUX"; $resn_exch{"1H11 LUX"}="H111 LUX"; $resn_exch{"2H11 LUX"}="H112 LUX";
$resn_exch{"1H12 LUX"}="H121 LUX"; $resn_exch{"2H12 LUX"}="H122 LUX"; $resn_exch{"1H14 LUX"}="H141 LUX";
$resn_exch{"2H14 LUX"}="H142 LUX"; $resn_exch{"1H15 LUX"}="H151 LUX"; $resn_exch{"2H15 LUX"}="H152 LUX";
$resn_exch{"1H16 LUX"}="H161 LUX"; $resn_exch{"2H16 LUX"}="H162 LUX"; $resn_exch{"3H16 LUX"}="H163 LUX";
$resn_exch{"1H17 LUX"}="H171 LUX"; $resn_exch{"2H17 LUX"}="H172 LUX"; $resn_exch{"3H17 LUX"}="H173 LUX";
$resn_exch{"1H18 LUX"}="H181 LUX"; $resn_exch{"2H18 LUX"}="H182 LUX"; $resn_exch{"1H19 LUX"}="H191 LUX";
$resn_exch{"2H19 LUX"}="H192 LUX"; $resn_exch{"3H19 LUX"}="H193 LUX"; $resn_exch{"1H20 LUX"}="H201 LUX";
$resn_exch{"2H20 LUX"}="H202 LUX"; $resn_exch{"3H20 LUX"}="H203 LUX"; $resn_exch{"1H22 LUX"}="H221 LUX";
$resn_exch{"2H22 LUX"}="H222 LUX"; $resn_exch{"1H24 LUX"}="H241 LUX"; $resn_exch{"1H27 LUX"}="H271 LUX";
$resn_exch{"1H28 LUX"}="H281 LUX"; $resn_exch{"2H28 LUX"}="H282 LUX"; $resn_exch{"1H30 LUX"}="H301 LUX";
$resn_exch{"2H30 LUX"}="H302 LUX"; $resn_exch{"1H31 LUX"}="H311 LUX"; $resn_exch{"2H31 LUX"}="H312 LUX";
$resn_exch{"1H32 LUX"}="H321 LUX"; $resn_exch{"2H32 LUX"}="H322 LUX"; $resn_exch{"1H34 LUX"}="H341 LUX";
$resn_exch{"2H34 LUX"}="H342 LUX"; $resn_exch{"1H35 LUX"}="H351 LUX"; $resn_exch{"2H35 LUX"}="H352 LUX";
$resn_exch{"1H36 LUX"}="H361 LUX"; $resn_exch{"2H36 LUX"}="H362 LUX"; $resn_exch{"3H36 LUX"}="H363 LUX";
$resn_exch{"1H37 LUX"}="H371 LUX"; $resn_exch{"2H37 LUX"}="H372 LUX"; $resn_exch{"3H37 LUX"}="H373 LUX";
$resn_exch{"1H38 LUX"}="H381 LUX"; $resn_exch{"2H38 LUX"}="H382 LUX"; $resn_exch{"3H38 LUX"}="H383 LUX";
$resn_exch{"1H39 LUX"}="H391 LUX"; $resn_exch{"2H39 LUX"}="H392 LUX"; $resn_exch{"3H39 LUX"}="H393 LUX";
$resn_exch{"1H40 LUX"}="H401 LUX"; $resn_exch{"2H40 LUX"}="H402 LUX"; $resn_exch{"3H40 LUX"}="H403 LUX";
$resn_exch{"1H16 LVA"}="H161 LVA"; $resn_exch{"2H16 LVA"}="H162 LVA"; $resn_exch{"1H17 LVA"}="H171 LVA";
$resn_exch{"2H17 LVA"}="H172 LVA"; $resn_exch{"1H18 LVA"}="H181 LVA"; $resn_exch{"2H18 LVA"}="H182 LVA";
$resn_exch{"1H19 LVA"}="H191 LVA"; $resn_exch{"2H19 LVA"}="H192 LVA"; $resn_exch{"1H20 LVA"}="H201 LVA";
$resn_exch{"2H20 LVA"}="H202 LVA"; $resn_exch{"3H20 LVA"}="H203 LVA"; $resn_exch{"1H21 LVA"}="H211 LVA";
$resn_exch{"2H21 LVA"}="H212 LVA"; $resn_exch{"1H23 LVA"}="H231 LVA"; $resn_exch{"2H23 LVA"}="H232 LVA";
$resn_exch{"3H23 LVA"}="H233 LVA"; $resn_exch{"1H27 LVA"}="H271 LVA"; $resn_exch{"2H27 LVA"}="H272 LVA";
$resn_exch{"1H28 LVA"}="H281 LVA"; $resn_exch{"2H28 LVA"}="H282 LVA"; $resn_exch{"3H28 LVA"}="H283 LVA";
$resn_exch{"1H29 LVA"}="H291 LVA"; $resn_exch{"2H29 LVA"}="H292 LVA"; $resn_exch{"3H29 LVA"}="H293 LVA";
$resn_exch{"1HN  LVG"}=" H   LVG"; $resn_exch{"2HN  LVG"}=" H2  LVG"; $resn_exch{"1HG1 LVG"}="HG11 LVG";
$resn_exch{"2HG1 LVG"}="HG12 LVG"; $resn_exch{"1HB1 LVS"}="HB11 LVS"; $resn_exch{"2HB1 LVS"}="HB12 LVS";
$resn_exch{"1HD1 LVS"}="HD11 LVS"; $resn_exch{"2HD1 LVS"}="HD12 LVS"; $resn_exch{"3HD1 LVS"}="HD13 LVS";
$resn_exch{"1HD2 LVS"}="HD21 LVS"; $resn_exch{"2HD2 LVS"}="HD22 LVS"; $resn_exch{"3HD2 LVS"}="HD23 LVS";
$resn_exch{"1HB2 LVS"}="HB21 LVS"; $resn_exch{"2HB2 LVS"}="HB22 LVS"; $resn_exch{"1HD3 LVS"}="HD31 LVS";
$resn_exch{"2HD3 LVS"}="HD32 LVS"; $resn_exch{"3HD3 LVS"}="HD33 LVS"; $resn_exch{"1HD4 LVS"}="HD41 LVS";
$resn_exch{"2HD4 LVS"}="HD42 LVS"; $resn_exch{"3HD4 LVS"}="HD43 LVS"; $resn_exch{"1HB3 LVS"}="HB31 LVS";
$resn_exch{"2HB3 LVS"}="HB32 LVS"; $resn_exch{"1HD5 LVS"}="HD51 LVS"; $resn_exch{"2HD5 LVS"}="HD52 LVS";
$resn_exch{"3HD5 LVS"}="HD53 LVS"; $resn_exch{"1HD6 LVS"}="HD61 LVS"; $resn_exch{"2HD6 LVS"}="HD62 LVS";
$resn_exch{"3HD6 LVS"}="HD63 LVS"; $resn_exch{"1H1* LVS"}="H1'1 LVS"; $resn_exch{"2H1* LVS"}="H1'2 LVS";
$resn_exch{"3H1* LVS"}="H1'3 LVS"; $resn_exch{" H2* LVS"}=" H2' LVS"; $resn_exch{"1HC4 LVS"}="HC41 LVS";
$resn_exch{"2HC4 LVS"}="HC42 LVS"; $resn_exch{"1H5  LX1"}=" H51 LX1"; $resn_exch{"2H5  LX1"}=" H52 LX1";
$resn_exch{"1H1  LX1"}=" H11 LX1"; $resn_exch{"2H1  LX1"}=" H12 LX1"; $resn_exch{"3H1  LX1"}=" H13 LX1";
$resn_exch{"1H5C LXC"}="H5C1 LXC"; $resn_exch{"2H5C LXC"}="H5C2 LXC"; $resn_exch{"1H5  LXP"}=" H51 LXP";
$resn_exch{"2H5  LXP"}=" H52 LXP"; $resn_exch{"1H1  LXP"}=" H11 LXP"; $resn_exch{"2H1  LXP"}=" H12 LXP";
$resn_exch{"1HC1 LY1"}="HC11 LY1"; $resn_exch{"2HC1 LY1"}="HC12 LY1"; $resn_exch{"1HN1 LY1"}="HN11 LY1";
$resn_exch{"1HC2 LY1"}="HC21 LY1"; $resn_exch{"2HC2 LY1"}="HC22 LY1"; $resn_exch{"1HC3 LY1"}="HC31 LY1";
$resn_exch{"2HC3 LY1"}="HC32 LY1"; $resn_exch{"1HC4 LY1"}="HC41 LY1"; $resn_exch{"2HC4 LY1"}="HC42 LY1";
$resn_exch{"1HC5 LY1"}="HC51 LY1"; $resn_exch{"1HC6 LY1"}="HC61 LY1"; $resn_exch{"1H15 LY2"}="H151 LY2";
$resn_exch{"2H15 LY2"}="H152 LY2"; $resn_exch{"1H19 LY2"}="H191 LY2"; $resn_exch{"2H19 LY2"}="H192 LY2";
$resn_exch{"1H16 LY2"}="H161 LY2"; $resn_exch{"2H16 LY2"}="H162 LY2"; $resn_exch{"1H18 LY2"}="H181 LY2";
$resn_exch{"2H18 LY2"}="H182 LY2"; $resn_exch{"1HC7 LY3"}="HC71 LY3"; $resn_exch{"2HC7 LY3"}="HC72 LY3";
$resn_exch{"1HC8 LY3"}="HC81 LY3"; $resn_exch{"2HC8 LY3"}="HC82 LY3"; $resn_exch{"1HN4 LY3"}="HN41 LY3";
$resn_exch{"2HN4 LY3"}="HN42 LY3"; $resn_exch{"1H17 LY3"}="H171 LY3"; $resn_exch{"2H17 LY3"}="H172 LY3";
$resn_exch{"1H19 LY3"}="H191 LY3"; $resn_exch{"2H19 LY3"}="H192 LY3"; $resn_exch{"1HBD LY4"}="HBD1 LY4";
$resn_exch{"2HBD LY4"}="HBD2 LY4"; $resn_exch{"1HBB LY4"}="HBB1 LY4"; $resn_exch{"2HBB LY4"}="HBB2 LY4";
$resn_exch{"1HBA LY4"}="HBA1 LY4"; $resn_exch{"2HBA LY4"}="HBA2 LY4"; $resn_exch{"1HBC LY4"}="HBC1 LY4";
$resn_exch{"2HBC LY4"}="HBC2 LY4"; $resn_exch{"1HBG LY4"}="HBG1 LY4"; $resn_exch{"2HBG LY4"}="HBG2 LY4";
$resn_exch{"1HBI LY4"}="HBI1 LY4"; $resn_exch{"2HBI LY4"}="HBI2 LY4"; $resn_exch{"3HBI LY4"}="HBI3 LY4";
$resn_exch{"1HBJ LY4"}="HBJ1 LY4"; $resn_exch{"2HBJ LY4"}="HBJ2 LY4"; $resn_exch{"3HBJ LY4"}="HBJ3 LY4";
$resn_exch{"1HC7 LYA"}="HC71 LYA"; $resn_exch{"2HC7 LYA"}="HC72 LYA"; $resn_exch{"1HC8 LYA"}="HC81 LYA";
$resn_exch{"2HC8 LYA"}="HC82 LYA"; $resn_exch{"1H19 LYA"}="H191 LYA"; $resn_exch{"2H19 LYA"}="H192 LYA";
$resn_exch{"1H24 LYA"}="H241 LYA"; $resn_exch{"2H24 LYA"}="H242 LYA"; $resn_exch{"1H25 LYA"}="H251 LYA";
$resn_exch{"2H25 LYA"}="H252 LYA"; $resn_exch{"1HC8 LYB"}="HC81 LYB"; $resn_exch{"2HC8 LYB"}="HC82 LYB";
$resn_exch{"1HC9 LYB"}="HC91 LYB"; $resn_exch{"2HC9 LYB"}="HC92 LYB"; $resn_exch{"1HN4 LYB"}="HN41 LYB";
$resn_exch{"2HN4 LYB"}="HN42 LYB"; $resn_exch{"1H18 LYB"}="H181 LYB"; $resn_exch{"2H18 LYB"}="H182 LYB";
$resn_exch{"1H19 LYB"}="H191 LYB"; $resn_exch{"2H19 LYB"}="H192 LYB"; $resn_exch{"1H23 LYB"}="H231 LYB";
$resn_exch{"2H23 LYB"}="H232 LYB"; $resn_exch{"1H24 LYB"}="H241 LYB"; $resn_exch{"2H24 LYB"}="H242 LYB";
$resn_exch{"1H28 LYB"}="H281 LYB"; $resn_exch{"2H28 LYB"}="H282 LYB"; $resn_exch{"1H29 LYB"}="H291 LYB";
$resn_exch{"2H29 LYB"}="H292 LYB"; $resn_exch{"1H33 LYB"}="H331 LYB"; $resn_exch{"2H33 LYB"}="H332 LYB";
$resn_exch{"1H34 LYB"}="H341 LYB"; $resn_exch{"2H34 LYB"}="H342 LYB"; $resn_exch{"1H1  LYC"}="H11A LYC";
$resn_exch{"2H1  LYC"}=" H12 LYC"; $resn_exch{"3H1  LYC"}=" H13 LYC"; $resn_exch{"1H3  LYC"}=" H31 LYC";
$resn_exch{"2H3  LYC"}=" H32 LYC"; $resn_exch{"3H3  LYC"}=" H33 LYC"; $resn_exch{"1H5  LYC"}=" H51 LYC";
$resn_exch{"2H5  LYC"}=" H52 LYC"; $resn_exch{"1H6  LYC"}=" H61 LYC"; $resn_exch{"2H6  LYC"}=" H62 LYC";
$resn_exch{"1H8  LYC"}=" H81 LYC"; $resn_exch{"2H8  LYC"}=" H82 LYC"; $resn_exch{"3H8  LYC"}=" H83 LYC";
$resn_exch{"1H13 LYC"}="H131 LYC"; $resn_exch{"2H13 LYC"}="H132 LYC"; $resn_exch{"3H13 LYC"}="H133 LYC";
$resn_exch{"1H18 LYC"}="H181 LYC"; $resn_exch{"2H18 LYC"}="H182 LYC"; $resn_exch{"3H18 LYC"}="H183 LYC";
$resn_exch{"1H52 LYC"}="H521 LYC"; $resn_exch{"2H52 LYC"}="H522 LYC"; $resn_exch{"3H52 LYC"}="H523 LYC";
$resn_exch{"1H57 LYC"}="H571 LYC"; $resn_exch{"2H57 LYC"}="H572 LYC"; $resn_exch{"3H57 LYC"}="H573 LYC";
$resn_exch{"1H62 LYC"}="H621 LYC"; $resn_exch{"2H62 LYC"}="H622 LYC"; $resn_exch{"3H62 LYC"}="H623 LYC";
$resn_exch{"1H63 LYC"}="H631 LYC"; $resn_exch{"2H63 LYC"}="H632 LYC"; $resn_exch{"1H64 LYC"}="H641 LYC";
$resn_exch{"2H64 LYC"}="H642 LYC"; $resn_exch{"1H67 LYC"}="H671 LYC"; $resn_exch{"2H67 LYC"}="H672 LYC";
$resn_exch{"3H67 LYC"}="H673 LYC"; $resn_exch{"1H68 LYC"}="H681 LYC"; $resn_exch{"2H68 LYC"}="H682 LYC";
$resn_exch{"3H68 LYC"}="H683 LYC"; $resn_exch{"1HG1 LYD"}="HG11 LYD"; $resn_exch{"2HG1 LYD"}="HG12 LYD";
$resn_exch{"3HG1 LYD"}="HG13 LYD"; $resn_exch{"1HG2 LYD"}="HG21 LYD"; $resn_exch{"2HG2 LYD"}="HG22 LYD";
$resn_exch{"3HG2 LYD"}="HG23 LYD"; $resn_exch{"1HC8 LYD"}="HC81 LYD"; $resn_exch{"2HC8 LYD"}="HC82 LYD";
$resn_exch{"1HC7 LYD"}="HC71 LYD"; $resn_exch{"2HC7 LYD"}="HC72 LYD"; $resn_exch{"1HN2 LYD"}="HN21 LYD";
$resn_exch{"2HN2 LYD"}="HN22 LYD"; $resn_exch{"1HC4 LYL"}="HC41 LYL"; $resn_exch{"2HC4 LYL"}="HC42 LYL";
$resn_exch{"1H15 LYL"}="H151 LYL"; $resn_exch{"2H15 LYL"}="H152 LYL"; $resn_exch{"3H15 LYL"}="H153 LYL";
$resn_exch{"1H14 LYL"}="H141 LYL"; $resn_exch{"2H14 LYL"}="H142 LYL"; $resn_exch{"1HN  LYM"}=" H   LYM";
$resn_exch{"2HN  LYM"}=" H2  LYM"; $resn_exch{"1HB  LYM"}=" HB2 LYM"; $resn_exch{"2HB  LYM"}=" HB3 LYM";
$resn_exch{"1HG  LYM"}=" HG2 LYM"; $resn_exch{"2HG  LYM"}=" HG3 LYM"; $resn_exch{"1HD  LYM"}=" HD2 LYM";
$resn_exch{"2HD  LYM"}=" HD3 LYM"; $resn_exch{"1HE  LYM"}=" HE2 LYM"; $resn_exch{"2HE  LYM"}=" HE3 LYM";
$resn_exch{"1HZ  LYM"}=" HZ1 LYM"; $resn_exch{"2HZ  LYM"}=" HZ2 LYM"; $resn_exch{"1HM  LYM"}=" HM1 LYM";
$resn_exch{"2HM  LYM"}=" HM2 LYM"; $resn_exch{"3HM  LYM"}=" HM3 LYM"; $resn_exch{" HN  LYN"}=" H   LYN";
$resn_exch{" HN2 LYN"}=" H2  LYN"; $resn_exch{" HB1 LYN"}=" HB2 LYN"; $resn_exch{" HB2 LYN"}=" HB3 LYN";
$resn_exch{" HG1 LYN"}=" HG2 LYN"; $resn_exch{" HG2 LYN"}=" HG3 LYN"; $resn_exch{" HD1 LYN"}=" HD2 LYN";
$resn_exch{" HD2 LYN"}=" HD3 LYN"; $resn_exch{" HE1 LYN"}=" HE2 LYN"; $resn_exch{" HE2 LYN"}=" HE3 LYN";
$resn_exch{"H1   LYN"}="HNT1 LYN"; $resn_exch{"H2   LYN"}="HNT2 LYN"; $resn_exch{"1HN  LYP"}=" HN1 LYP";
$resn_exch{"2HN  LYP"}=" HN2 LYP"; $resn_exch{"1HBC LYP"}="HBC1 LYP"; $resn_exch{"2HBC LYP"}="HBC2 LYP";
$resn_exch{"1HGC LYP"}="HGC1 LYP"; $resn_exch{"2HGC LYP"}="HGC2 LYP"; $resn_exch{"1HDC LYP"}="HDC1 LYP";
$resn_exch{"2HDC LYP"}="HDC2 LYP"; $resn_exch{"1HEC LYP"}="HEC1 LYP"; $resn_exch{"2HEC LYP"}="HEC2 LYP";
$resn_exch{"1HMC LYP"}="HMC1 LYP"; $resn_exch{"2HMC LYP"}="HMC2 LYP"; $resn_exch{"3HMC LYP"}="HMC3 LYP";
$resn_exch{"1HCD LYP"}="HCD1 LYP"; $resn_exch{"2HCD LYP"}="HCD2 LYP"; $resn_exch{"1HCA LYP"}="HCA1 LYP";
$resn_exch{"2HCA LYP"}="HCA2 LYP"; $resn_exch{"3HCA LYP"}="HCA3 LYP"; $resn_exch{"1HCC LYP"}="HCC1 LYP";
$resn_exch{"2HCC LYP"}="HCC2 LYP"; $resn_exch{" HN2 LYR"}=" H2  LYR"; $resn_exch{" HB1 LYR"}=" HB2 LYR";
$resn_exch{" HB2 LYR"}=" HB3 LYR"; $resn_exch{" HG1 LYR"}=" HG2 LYR"; $resn_exch{" HG2 LYR"}=" HG3 LYR";
$resn_exch{" HD1 LYR"}=" HD2 LYR"; $resn_exch{" HD2 LYR"}=" HD3 LYR"; $resn_exch{" HE1 LYR"}=" HE2 LYR";
$resn_exch{" HE2 LYR"}=" HE3 LYR"; $resn_exch{"1H1  LYR"}=" H11 LYR"; $resn_exch{" H2  LYR"}=" HC2 LYR";
$resn_exch{"1H4  LYR"}=" H41 LYR"; $resn_exch{"2H4  LYR"}=" H42 LYR"; $resn_exch{"3H4  LYR"}=" H43 LYR";
$resn_exch{"1H8  LYR"}=" H81 LYR"; $resn_exch{"2H8  LYR"}=" H82 LYR"; $resn_exch{"3H8  LYR"}=" H83 LYR";
$resn_exch{"1H13 LYR"}="H131 LYR"; $resn_exch{"2H13 LYR"}="H132 LYR"; $resn_exch{"3H13 LYR"}="H133 LYR";
$resn_exch{"1H14 LYR"}="H141 LYR"; $resn_exch{"2H14 LYR"}="H142 LYR"; $resn_exch{"1H15 LYR"}="H151 LYR";
$resn_exch{"2H15 LYR"}="H152 LYR"; $resn_exch{"1H16 LYR"}="H161 LYR"; $resn_exch{"2H16 LYR"}="H162 LYR";
$resn_exch{"1H18 LYR"}="H181 LYR"; $resn_exch{"2H18 LYR"}="H182 LYR"; $resn_exch{"3H18 LYR"}="H183 LYR";
$resn_exch{"1H19 LYR"}="H191 LYR"; $resn_exch{"2H19 LYR"}="H192 LYR"; $resn_exch{"3H19 LYR"}="H193 LYR";
$resn_exch{" HN2 LYS"}=" H2  LYS"; $resn_exch{"1HB  LYS"}=" HB2 LYS"; $resn_exch{"2HB  LYS"}=" HB3 LYS";
$resn_exch{"1HG  LYS"}=" HG2 LYS"; $resn_exch{"2HG  LYS"}=" HG3 LYS"; $resn_exch{"1HD  LYS"}=" HD2 LYS";
$resn_exch{"2HD  LYS"}=" HD3 LYS"; $resn_exch{"1HE  LYS"}=" HE2 LYS"; $resn_exch{"2HE  LYS"}=" HE3 LYS";
$resn_exch{"1HZ  LYS"}=" HZ1 LYS"; $resn_exch{"2HZ  LYS"}=" HZ2 LYS"; $resn_exch{"3HZ  LYS"}=" HZ3 LYS";
$resn_exch{"1H1  LYT"}=" H11 LYT"; $resn_exch{"2H1  LYT"}=" H12 LYT"; $resn_exch{"1HN  LYT"}=" HN1 LYT";
$resn_exch{"2HN  LYT"}=" HN2 LYT"; $resn_exch{"1H2  LYT"}=" H21 LYT"; $resn_exch{"2H2  LYT"}=" H22 LYT";
$resn_exch{"1H3  LYT"}=" H31 LYT"; $resn_exch{"2H3  LYT"}=" H32 LYT"; $resn_exch{"1H4  LYT"}=" H41 LYT";
$resn_exch{"2H4  LYT"}=" H42 LYT"; $resn_exch{"3H4  LYT"}=" H43 LYT"; $resn_exch{"1HN  LYW"}=" HN1 LYW";
$resn_exch{"2HN  LYW"}=" HN2 LYW"; $resn_exch{"1H2  LYW"}=" H21 LYW"; $resn_exch{"2H2  LYW"}=" H22 LYW";
$resn_exch{"1H3  LYW"}=" H31 LYW"; $resn_exch{"2H3  LYW"}=" H32 LYW"; $resn_exch{"1H4  LYW"}=" H41 LYW";
$resn_exch{"2H4  LYW"}=" H42 LYW"; $resn_exch{"1H5  LYW"}=" H51 LYW"; $resn_exch{"2H5  LYW"}=" H52 LYW";
$resn_exch{"3H5  LYW"}=" H53 LYW"; $resn_exch{"1H6  LYX"}=" H61 LYX"; $resn_exch{"2H6  LYX"}=" H62 LYX";
$resn_exch{"1H8  LYX"}=" H81 LYX"; $resn_exch{"2H8  LYX"}=" H82 LYX"; $resn_exch{"3H8  LYX"}=" H83 LYX";
$resn_exch{"1H9  LYX"}=" H91 LYX"; $resn_exch{"2H9  LYX"}=" H92 LYX"; $resn_exch{"3H9  LYX"}=" H93 LYX";
$resn_exch{"1H14 LYX"}="H141 LYX"; $resn_exch{"2H14 LYX"}="H142 LYX"; $resn_exch{"1H15 LYX"}="H151 LYX";
$resn_exch{"2H15 LYX"}="H152 LYX"; $resn_exch{"1H18 LYX"}="H181 LYX"; $resn_exch{"2H18 LYX"}="H182 LYX";
$resn_exch{"1H19 LYX"}="H191 LYX"; $resn_exch{"2H19 LYX"}="H192 LYX"; $resn_exch{"1H21 LYX"}="H211 LYX";
$resn_exch{"1H22 LYX"}="H221 LYX"; $resn_exch{"2H22 LYX"}="H222 LYX"; $resn_exch{"3H22 LYX"}="H223 LYX";
$resn_exch{"1HE  LYX"}=" HE3 LYX"; $resn_exch{"2HE  LYX"}=" HE2 LYX"; $resn_exch{"1HD  LYX"}=" HD3 LYX";
$resn_exch{"2HD  LYX"}=" HD2 LYX"; $resn_exch{"1HG  LYX"}=" HG3 LYX"; $resn_exch{"2HG  LYX"}=" HG2 LYX";
$resn_exch{"1HB  LYX"}=" HB3 LYX"; $resn_exch{"2HB  LYX"}=" HB2 LYX"; $resn_exch{" HN2 LYX"}=" H2  LYX";
$resn_exch{"1H40 LYX"}="H401 LYX"; $resn_exch{"2H40 LYX"}="H402 LYX"; $resn_exch{"1H60 LYX"}="H601 LYX";
$resn_exch{"2H60 LYX"}="H602 LYX"; $resn_exch{" HN2 LYZ"}=" H2  LYZ"; $resn_exch{"1HB  LYZ"}=" HB2 LYZ";
$resn_exch{"2HB  LYZ"}=" HB3 LYZ"; $resn_exch{"1HG  LYZ"}=" HG2 LYZ"; $resn_exch{"2HG  LYZ"}=" HG3 LYZ";
$resn_exch{"1HE  LYZ"}=" HE2 LYZ"; $resn_exch{"2HE  LYZ"}=" HE3 LYZ"; $resn_exch{"1HZ  LYZ"}=" HZ1 LYZ";
$resn_exch{"2HZ  LYZ"}=" HZ2 LYZ"; $resn_exch{" HO  LYZ"}=" HH  LYZ"; $resn_exch{"1H9C M01"}="H9C1 M01";
$resn_exch{"2H9C M01"}="H9C2 M01"; $resn_exch{"1H10 M01"}="H101 M01"; $resn_exch{"2H10 M01"}="H102 M01";
$resn_exch{"1H11 M01"}="H111 M01"; $resn_exch{"2H11 M01"}="H112 M01"; $resn_exch{"1H15 M01"}="H151 M01";
$resn_exch{"2H15 M01"}="H152 M01"; $resn_exch{"1H12 M01"}="H121 M01"; $resn_exch{"2H12 M01"}="H122 M01";
$resn_exch{"1H14 M01"}="H141 M01"; $resn_exch{"2H14 M01"}="H142 M01"; $resn_exch{"1HAO M02"}="HAO1 M02";
$resn_exch{"2HAO M02"}="HAO2 M02"; $resn_exch{"1HAK M02"}="HAK1 M02"; $resn_exch{"2HAK M02"}="HAK2 M02";
$resn_exch{"1H10 M02"}="H101 M02"; $resn_exch{"2H10 M02"}="H102 M02"; $resn_exch{"1H7C M02"}="H7C1 M02";
$resn_exch{"2H7C M02"}="H7C2 M02"; $resn_exch{"1HAP M02"}="HAP1 M02"; $resn_exch{"2HAP M02"}="HAP2 M02";
$resn_exch{"1H2C M03"}="H2C1 M03"; $resn_exch{"2HA  M03"}=" HA2 M03"; $resn_exch{"1H8C M03"}="H8C1 M03";
$resn_exch{"2H8C M03"}="H8C2 M03"; $resn_exch{"1HA  M03"}=" HA1 M03"; $resn_exch{"2HB  M03"}=" HB2 M03";
$resn_exch{"1HB  M03"}=" HB1 M03"; $resn_exch{"2HC  M03"}=" HC2 M03"; $resn_exch{"1H7C M03"}="H7C1 M03";
$resn_exch{"2H7C M03"}="H7C2 M03"; $resn_exch{"1HC  M03"}=" HC1 M03"; $resn_exch{"2HD  M03"}=" HD2 M03";
$resn_exch{"1H2C M04"}="H2C1 M04"; $resn_exch{"2H2C M04"}="H2C2 M04"; $resn_exch{"1H8C M04"}="H8C1 M04";
$resn_exch{"2H8C M04"}="H8C2 M04"; $resn_exch{"1H3C M04"}="H3C1 M04"; $resn_exch{"2H3C M04"}="H3C2 M04";
$resn_exch{"1H5C M04"}="H5C1 M04"; $resn_exch{"2H5C M04"}="H5C2 M04"; $resn_exch{"1H6N M04"}="H6N1 M04";
$resn_exch{"2H6N M04"}="H6N2 M04"; $resn_exch{"1H7C M04"}="H7C1 M04"; $resn_exch{"2H7C M04"}="H7C2 M04";
$resn_exch{"1H13 M04"}="H131 M04"; $resn_exch{"2H13 M04"}="H132 M04"; $resn_exch{"1H8C M05"}="H8C1 M05";
$resn_exch{"2H8C M05"}="H8C2 M05"; $resn_exch{"1H2C M05"}="H2C1 M05"; $resn_exch{"2H2C M05"}="H2C2 M05";
$resn_exch{"1H3C M05"}="H3C1 M05"; $resn_exch{"2H3C M05"}="H3C2 M05"; $resn_exch{"1H4C M05"}="H4C1 M05";
$resn_exch{"2H4C M05"}="H4C2 M05"; $resn_exch{"1H6N M05"}="H6N1 M05"; $resn_exch{"2H6N M05"}="H6N2 M05";
$resn_exch{"1H17 M05"}="H171 M05"; $resn_exch{"2H17 M05"}="H172 M05"; $resn_exch{"1HDK M0E"}="HDK1 M0E";
$resn_exch{"2HDK M0E"}="HDK2 M0E"; $resn_exch{"1HAU M0E"}="HAU1 M0E"; $resn_exch{"2HAU M0E"}="HAU2 M0E";
$resn_exch{"1HAS M0E"}="HAS1 M0E"; $resn_exch{"2HAS M0E"}="HAS2 M0E"; $resn_exch{"3HAS M0E"}="HAS3 M0E";
$resn_exch{"1HAT M0E"}="HAT1 M0E"; $resn_exch{"2HAT M0E"}="HAT2 M0E"; $resn_exch{"1HAH M0E"}="HAH1 M0E";
$resn_exch{"2HAH M0E"}="HAH2 M0E"; $resn_exch{"3HAH M0E"}="HAH3 M0E"; $resn_exch{"1H6  M0E"}=" H61 M0E";
$resn_exch{"2H6  M0E"}=" H62 M0E"; $resn_exch{"1HBO M0E"}="HBO1 M0E"; $resn_exch{"2HBO M0E"}="HBO2 M0E";
$resn_exch{"1HCB M0E"}="HCB1 M0E"; $resn_exch{"2HCB M0E"}="HCB2 M0E"; $resn_exch{"3HCB M0E"}="HCB3 M0E";
$resn_exch{"1HBZ M0E"}="HBZ1 M0E"; $resn_exch{"2HBZ M0E"}="HBZ2 M0E"; $resn_exch{"3HBZ M0E"}="HBZ3 M0E";
$resn_exch{"1H1  M0E"}=" H11 M0E"; $resn_exch{"1H4  M0E"}=" H41 M0E"; $resn_exch{"2H4  M0E"}=" H42 M0E";
$resn_exch{"3H4  M0E"}=" H43 M0E"; $resn_exch{"1H5  M0E"}=" H51 M0E"; $resn_exch{"2H5  M0E"}=" H52 M0E";
$resn_exch{"1H36 M0E"}="H361 M0E"; $resn_exch{"2H36 M0E"}="H362 M0E"; $resn_exch{"1H10 M0E"}="H101 M0E";
$resn_exch{"2H10 M0E"}="H102 M0E"; $resn_exch{"3H10 M0E"}="H103 M0E"; $resn_exch{"1H11 M0E"}="H111 M0E";
$resn_exch{"2H11 M0E"}="H112 M0E"; $resn_exch{"3H11 M0E"}="H113 M0E"; $resn_exch{"1H12 M0E"}="H121 M0E";
$resn_exch{"2H12 M0E"}="H122 M0E"; $resn_exch{"1H13 M0E"}="H131 M0E"; $resn_exch{"2H13 M0E"}="H132 M0E";
$resn_exch{"1H15 M0E"}="H151 M0E"; $resn_exch{"2H15 M0E"}="H152 M0E"; $resn_exch{"1H16 M0E"}="H161 M0E";
$resn_exch{"2H16 M0E"}="H162 M0E"; $resn_exch{"1H19 M0E"}="H191 M0E"; $resn_exch{"2H19 M0E"}="H192 M0E";
$resn_exch{"3H19 M0E"}="H193 M0E"; $resn_exch{"1H20 M0E"}="H201 M0E"; $resn_exch{"2H20 M0E"}="H202 M0E";
$resn_exch{"1H21 M0E"}="H211 M0E"; $resn_exch{"2H21 M0E"}="H212 M0E"; $resn_exch{"1H24 M0E"}="H241 M0E";
$resn_exch{"2H24 M0E"}="H242 M0E"; $resn_exch{"3H24 M0E"}="H243 M0E"; $resn_exch{"1H25 M0E"}="H251 M0E";
$resn_exch{"2H25 M0E"}="H252 M0E"; $resn_exch{"3H25 M0E"}="H253 M0E"; $resn_exch{"1H2  M0E"}=" H21 M0E";
$resn_exch{"1HN  M0H"}=" HN1 M0H"; $resn_exch{"2HN  M0H"}=" HN2 M0H"; $resn_exch{"1HB  M0H"}=" HB1 M0H";
$resn_exch{"2HB  M0H"}=" HB2 M0H"; $resn_exch{"1HD  M0H"}=" HD1 M0H"; $resn_exch{"2HD  M0H"}=" HD2 M0H";
$resn_exch{"1H8  M0N"}=" H81 M0N"; $resn_exch{"2H8  M0N"}=" H82 M0N"; $resn_exch{"1H12 M13"}="H121 M13";
$resn_exch{"2H12 M13"}="H122 M13"; $resn_exch{"1HM  M13"}=" HM1 M13"; $resn_exch{"2HM  M13"}=" HM2 M13";
$resn_exch{"3HM  M13"}=" HM3 M13"; $resn_exch{"1H6  M13"}=" H61 M13"; $resn_exch{"2H6  M13"}=" H62 M13";
$resn_exch{"1HN6 M1A"}="HN61 M1A"; $resn_exch{"2HN6 M1A"}="HN62 M1A"; $resn_exch{"1H1  M1A"}=" H11 M1A";
$resn_exch{"2H1  M1A"}=" H12 M1A"; $resn_exch{"3H1  M1A"}=" H13 M1A"; $resn_exch{"1H16 M1B"}="H161 M1B";
$resn_exch{"2H16 M1B"}="H162 M1B"; $resn_exch{"1H26 M1B"}="H261 M1B"; $resn_exch{"2H26 M1B"}="H262 M1B";
$resn_exch{"1H13 M1C"}="H131 M1C"; $resn_exch{"2H13 M1C"}="H132 M1C"; $resn_exch{"1H12 M1C"}="H121 M1C";
$resn_exch{"2H12 M1C"}="H122 M1C"; $resn_exch{"1HN3 M1C"}="HN31 M1C"; $resn_exch{"2HN3 M1C"}="HN32 M1C";
$resn_exch{"1H9  M1C"}=" H91 M1C"; $resn_exch{"2H9  M1C"}=" H92 M1C"; $resn_exch{"1H4  M1C"}=" H41 M1C";
$resn_exch{"2H4  M1C"}=" H42 M1C"; $resn_exch{"1H5  M1C"}=" H51 M1C"; $resn_exch{"2H5  M1C"}=" H52 M1C";
$resn_exch{"1H6  M1C"}=" H61 M1C"; $resn_exch{"2H6  M1C"}=" H62 M1C"; $resn_exch{"1H7  M1C"}=" H71 M1C";
$resn_exch{"2H7  M1C"}=" H72 M1C"; $resn_exch{"3H7  M1C"}=" H73 M1C"; $resn_exch{"2HOP M1G"}="HOP2 M1G";
$resn_exch{"3HOP M1G"}="HOP3 M1G"; $resn_exch{"1H5* M1G"}=" H5' M1G"; $resn_exch{"2H5* M1G"}="H5'' M1G";
$resn_exch{" H4* M1G"}=" H4' M1G"; $resn_exch{" H3* M1G"}=" H3' M1G"; $resn_exch{"H3T  M1G"}="HO3' M1G";
$resn_exch{"1H2* M1G"}=" H2' M1G"; $resn_exch{"2H2* M1G"}="H2'' M1G"; $resn_exch{" H1* M1G"}=" H1' M1G";
$resn_exch{"1H1  M1L"}=" H11 M1L"; $resn_exch{"2H1  M1L"}=" H12 M1L"; $resn_exch{"1H8  M1L"}=" H81 M1L";
$resn_exch{"2H8  M1L"}=" H82 M1L"; $resn_exch{"1H10 M1L"}="H101 M1L"; $resn_exch{"2H10 M1L"}="H102 M1L";
$resn_exch{"1H13 M1L"}="H131 M1L"; $resn_exch{"2H13 M1L"}="H132 M1L"; $resn_exch{"3H13 M1L"}="H133 M1L";
$resn_exch{"1H20 M1L"}="H201 M1L"; $resn_exch{"2H20 M1L"}="H202 M1L"; $resn_exch{"3H20 M1L"}="H203 M1L";
$resn_exch{"1H21 M1L"}="H211 M1L"; $resn_exch{"2H21 M1L"}="H212 M1L"; $resn_exch{"3H21 M1L"}="H213 M1L";
$resn_exch{"1H23 M1L"}="H231 M1L"; $resn_exch{"2H23 M1L"}="H232 M1L"; $resn_exch{"1H24 M1L"}="H241 M1L";
$resn_exch{"2H24 M1L"}="H242 M1L"; $resn_exch{"1H22 M1N"}="H221 M1N"; $resn_exch{"2H22 M1N"}="H222 M1N";
$resn_exch{"1H25 M1N"}="H251 M1N"; $resn_exch{"2H25 M1N"}="H252 M1N"; $resn_exch{"3H25 M1N"}="H253 M1N";
$resn_exch{"1H24 M1N"}="H241 M1N"; $resn_exch{"2H24 M1N"}="H242 M1N"; $resn_exch{"3H24 M1N"}="H243 M1N";
$resn_exch{"1H14 M1N"}="H141 M1N"; $resn_exch{"2H14 M1N"}="H142 M1N"; $resn_exch{"1H13 M1N"}="H131 M1N";
$resn_exch{"2H13 M1N"}="H132 M1N"; $resn_exch{"1H11 M1N"}="H111 M1N"; $resn_exch{"2H11 M1N"}="H112 M1N";
$resn_exch{"1H10 M1N"}="H101 M1N"; $resn_exch{"2H10 M1N"}="H102 M1N"; $resn_exch{"1H5  M1N"}=" H51 M1N";
$resn_exch{"2H5  M1N"}=" H52 M1N"; $resn_exch{"1H6  M1P"}=" H61 M1P"; $resn_exch{"2H6  M1P"}=" H62 M1P";
$resn_exch{"1H8C M1S"}="H8C1 M1S"; $resn_exch{"2H8C M1S"}="H8C2 M1S"; $resn_exch{"1H10 M1S"}="H101 M1S";
$resn_exch{"2H10 M1S"}="H102 M1S"; $resn_exch{"1H11 M1S"}="H111 M1S"; $resn_exch{"2H11 M1S"}="H112 M1S";
$resn_exch{"1H12 M1S"}="H121 M1S"; $resn_exch{"2H12 M1S"}="H122 M1S"; $resn_exch{"1H13 M1S"}="H131 M1S";
$resn_exch{"2H13 M1S"}="H132 M1S"; $resn_exch{"1H14 M1S"}="H141 M1S"; $resn_exch{"2H14 M1S"}="H142 M1S";
$resn_exch{"1H17 M1S"}="H171 M1S"; $resn_exch{"2H17 M1S"}="H172 M1S"; $resn_exch{"1H18 M1S"}="H181 M1S";
$resn_exch{"2H18 M1S"}="H182 M1S"; $resn_exch{"1HN1 M25"}="HN11 M25"; $resn_exch{"2HN1 M25"}="HN12 M25";
$resn_exch{"1H7  M25"}=" H71 M25"; $resn_exch{"2H7  M25"}=" H72 M25"; $resn_exch{"1H8  M25"}=" H81 M25";
$resn_exch{"2H8  M25"}=" H82 M25"; $resn_exch{"1H10 M25"}="H101 M25"; $resn_exch{"2H10 M25"}="H102 M25";
$resn_exch{"3H10 M25"}="H103 M25"; $resn_exch{"1HN3 M28"}="HN31 M28"; $resn_exch{"2HN3 M28"}="HN32 M28";
$resn_exch{"1H7  M28"}=" H71 M28"; $resn_exch{"2H7  M28"}=" H72 M28"; $resn_exch{"1H8  M28"}=" H81 M28";
$resn_exch{"2H8  M28"}=" H82 M28"; $resn_exch{"1HN  M29"}=" HN1 M29"; $resn_exch{"2HN  M29"}=" HN2 M29";
$resn_exch{"1H7  M29"}=" H71 M29"; $resn_exch{"2H7  M29"}=" H72 M29"; $resn_exch{"1H8  M29"}=" H81 M29";
$resn_exch{"2H8  M29"}=" H82 M29"; $resn_exch{"1H10 M29"}="H101 M29"; $resn_exch{"2H10 M29"}="H102 M29";
$resn_exch{"1H11 M29"}="H111 M29"; $resn_exch{"2H11 M29"}="H112 M29"; $resn_exch{"3H11 M29"}="H113 M29";
$resn_exch{"1H11 M2C"}="H111 M2C"; $resn_exch{"2H11 M2C"}="H112 M2C"; $resn_exch{"1H12 M2C"}="H121 M2C";
$resn_exch{"2H12 M2C"}="H122 M2C"; $resn_exch{"1H14 M2C"}="H141 M2C"; $resn_exch{"2H14 M2C"}="H142 M2C";
$resn_exch{"3H14 M2C"}="H143 M2C"; $resn_exch{"1H10 M2C"}="H101 M2C"; $resn_exch{"2H10 M2C"}="H102 M2C";
$resn_exch{"2HOP M2G"}="HOP2 M2G"; $resn_exch{"3HOP M2G"}="HOP3 M2G"; $resn_exch{"1H5* M2G"}=" H5' M2G";
$resn_exch{"2H5* M2G"}="H5'' M2G"; $resn_exch{" H4* M2G"}=" H4' M2G"; $resn_exch{" H3* M2G"}=" H3' M2G";
$resn_exch{"*HO3 M2G"}="HO3' M2G"; $resn_exch{" H2* M2G"}=" H2' M2G"; $resn_exch{"*HO2 M2G"}="HO2' M2G";
$resn_exch{" H1* M2G"}=" H1' M2G"; $resn_exch{"1HM1 M2G"}="HM11 M2G"; $resn_exch{"2HM1 M2G"}="HM12 M2G";
$resn_exch{"3HM1 M2G"}="HM13 M2G"; $resn_exch{"1HM2 M2G"}="HM21 M2G"; $resn_exch{"2HM2 M2G"}="HM22 M2G";
$resn_exch{"3HM2 M2G"}="HM23 M2G"; $resn_exch{"1H1  M2M"}=" H11 M2M"; $resn_exch{"2H1  M2M"}=" H12 M2M";
$resn_exch{"3H1  M2M"}=" H13 M2M"; $resn_exch{"1H2  M2M"}=" H21 M2M"; $resn_exch{"2H2  M2M"}=" H22 M2M";
$resn_exch{"1H3  M2M"}=" H31 M2M"; $resn_exch{"2H3  M2M"}=" H32 M2M"; $resn_exch{"1H4  M2M"}=" H41 M2M";
$resn_exch{"2H4  M2M"}=" H42 M2M"; $resn_exch{"1H5  M2M"}=" H51 M2M"; $resn_exch{"2H5  M2M"}=" H52 M2M";
$resn_exch{"1H6  M2M"}=" H61 M2M"; $resn_exch{"2H6  M2M"}=" H62 M2M"; $resn_exch{"3H6  M2M"}=" H63 M2M";
$resn_exch{"1H1  M2P"}="H11A M2P"; $resn_exch{"2H1  M2P"}=" H12 M2P"; $resn_exch{"1H6  M2P"}=" H61 M2P";
$resn_exch{"2H6  M2P"}="H62A M2P"; $resn_exch{"1H3B M33"}="H3B1 M33"; $resn_exch{"2H3B M33"}="H3B2 M33";
$resn_exch{"3H3B M33"}="H3B3 M33"; $resn_exch{"1H5' M33"}="H5'1 M33"; $resn_exch{"2H5' M33"}="H5'2 M33";
$resn_exch{"1HN6 M33"}="HN61 M33"; $resn_exch{"2HN6 M33"}="HN62 M33"; $resn_exch{"1H22 M38"}="H221 M38";
$resn_exch{"2H22 M38"}="H222 M38"; $resn_exch{"1H23 M38"}="H231 M38"; $resn_exch{"2H23 M38"}="H232 M38";
$resn_exch{"1H25 M38"}="H251 M38"; $resn_exch{"2H25 M38"}="H252 M38"; $resn_exch{"1HN1 M3C"}="HN11 M3C";
$resn_exch{"2HN1 M3C"}="HN12 M3C"; $resn_exch{"1H10 M3C"}="H101 M3C"; $resn_exch{"2H10 M3C"}="H102 M3C";
$resn_exch{"1H11 M3C"}="H111 M3C"; $resn_exch{"2H11 M3C"}="H112 M3C"; $resn_exch{"1H13 M3C"}="H131 M3C";
$resn_exch{"2H13 M3C"}="H132 M3C"; $resn_exch{"3H13 M3C"}="H133 M3C"; $resn_exch{"1HN  M3L"}=" H   M3L";
$resn_exch{"2HN  M3L"}=" H2  M3L"; $resn_exch{"1HB  M3L"}=" HB2 M3L"; $resn_exch{"2HB  M3L"}=" HB3 M3L";
$resn_exch{"1HG  M3L"}=" HG2 M3L"; $resn_exch{"2HG  M3L"}=" HG3 M3L"; $resn_exch{"1HD  M3L"}=" HD2 M3L";
$resn_exch{"2HD  M3L"}=" HD3 M3L"; $resn_exch{"1HE  M3L"}=" HE2 M3L"; $resn_exch{"2HE  M3L"}=" HE3 M3L";
$resn_exch{"1HM1 M3L"}="HM11 M3L"; $resn_exch{"2HM1 M3L"}="HM12 M3L"; $resn_exch{"3HM1 M3L"}="HM13 M3L";
$resn_exch{"1HM2 M3L"}="HM21 M3L"; $resn_exch{"2HM2 M3L"}="HM22 M3L"; $resn_exch{"3HM2 M3L"}="HM23 M3L";
$resn_exch{"1HM3 M3L"}="HM31 M3L"; $resn_exch{"2HM3 M3L"}="HM32 M3L"; $resn_exch{"3HM3 M3L"}="HM33 M3L";
$resn_exch{"1H18 M54"}="H181 M54"; $resn_exch{"2H18 M54"}="H182 M54"; $resn_exch{"1H19 M54"}="H191 M54";
$resn_exch{"2H19 M54"}="H192 M54"; $resn_exch{"1H20 M54"}="H201 M54"; $resn_exch{"2H20 M54"}="H202 M54";
$resn_exch{"1H21 M54"}="H211 M54"; $resn_exch{"2H21 M54"}="H212 M54"; $resn_exch{"2HN* M5M"}="HN'2 M5M";
$resn_exch{"1H7* M5M"}="H7'1 M5M"; $resn_exch{"2H7* M5M"}="H7'2 M5M"; $resn_exch{"3H7* M5M"}="H7'3 M5M";
$resn_exch{"2HOP M5M"}="HOP2 M5M"; $resn_exch{"1H5* M5M"}=" H5' M5M"; $resn_exch{"2H5* M5M"}="H5'' M5M";
$resn_exch{" H4* M5M"}=" H4' M5M"; $resn_exch{" H1* M5M"}=" H1' M5M"; $resn_exch{"1HN4 M5M"}="HN41 M5M";
$resn_exch{"2HN4 M5M"}="HN42 M5M"; $resn_exch{" H2* M5M"}=" H2' M5M"; $resn_exch{" H3* M5M"}=" H3' M5M";
$resn_exch{"H3T  M5M"}="HO3' M5M"; $resn_exch{"3HOP M5M"}="HOP3 M5M"; $resn_exch{"1H7  M5P"}=" H71 M5P";
$resn_exch{"2H7  M5P"}=" H72 M5P"; $resn_exch{"1H8  M5P"}=" H81 M5P"; $resn_exch{"2H8  M5P"}=" H82 M5P";
$resn_exch{"1H9  M5P"}=" H91 M5P"; $resn_exch{"2H9  M5P"}=" H92 M5P"; $resn_exch{"1H11 M5P"}="H111 M5P";
$resn_exch{"2H11 M5P"}="H112 M5P"; $resn_exch{"1H6  M6P"}=" H61 M6P"; $resn_exch{"2H6  M6P"}=" H62 M6P";
$resn_exch{"2HOP M6P"}="HOP2 M6P"; $resn_exch{"3HOP M6P"}="HOP3 M6P"; $resn_exch{"1H6  M6R"}=" H61 M6R";
$resn_exch{"2H6  M6R"}=" H62 M6R"; $resn_exch{"1HN2 M6R"}="HN21 M6R"; $resn_exch{"2HN2 M6R"}="HN22 M6R";
$resn_exch{"1H1  M6R"}=" H11 M6R"; $resn_exch{"2H1  M6R"}=" H12 M6R"; $resn_exch{"1HM2 M6T"}="HM21 M6T";
$resn_exch{"2HM2 M6T"}="HM22 M6T"; $resn_exch{"3HM2 M6T"}="HM23 M6T"; $resn_exch{"1HN4 M6T"}="HN41 M6T";
$resn_exch{"2HN4 M6T"}="HN42 M6T"; $resn_exch{"1H7' M6T"}="H7'1 M6T"; $resn_exch{"2H7' M6T"}="H7'2 M6T";
$resn_exch{"1HM4 M6T"}="HM41 M6T"; $resn_exch{"2HM4 M6T"}="HM42 M6T"; $resn_exch{"3HM4 M6T"}="HM43 M6T";
$resn_exch{"1HM6 M6T"}="HM61 M6T"; $resn_exch{"2HM6 M6T"}="HM62 M6T"; $resn_exch{"3HM6 M6T"}="HM63 M6T";
$resn_exch{"1H6  M6T"}=" H61 M6T"; $resn_exch{"2H6  M6T"}=" H62 M6T"; $resn_exch{"1H7  M6T"}=" H71 M6T";
$resn_exch{"2H7  M6T"}=" H72 M6T"; $resn_exch{"2HOA M6T"}="HOA2 M6T"; $resn_exch{"2HOB M6T"}="HOB2 M6T";
$resn_exch{"1H22 M77"}="H221 M77"; $resn_exch{"2H22 M77"}="H222 M77"; $resn_exch{"1H21 M77"}="H211 M77";
$resn_exch{"2H21 M77"}="H212 M77"; $resn_exch{"1H16 M77"}="H161 M77"; $resn_exch{"2H16 M77"}="H162 M77";
$resn_exch{"1H20 M77"}="H201 M77"; $resn_exch{"2H20 M77"}="H202 M77"; $resn_exch{"1H15 M77"}="H151 M77";
$resn_exch{"2H15 M77"}="H152 M77"; $resn_exch{"2HOA M7G"}="HOA2 M7G"; $resn_exch{"2HOB M7G"}="HOB2 M7G";
$resn_exch{"3HOB M7G"}="HOB3 M7G"; $resn_exch{"1H5* M7G"}="H5'1 M7G"; $resn_exch{"2H5* M7G"}="H5'2 M7G";
$resn_exch{" H4* M7G"}=" H4' M7G"; $resn_exch{" H3* M7G"}=" H3' M7G"; $resn_exch{"*HO3 M7G"}="HO3' M7G";
$resn_exch{" H2* M7G"}=" H2' M7G"; $resn_exch{"*HO2 M7G"}="HO2' M7G"; $resn_exch{" H1* M7G"}=" H1' M7G";
$resn_exch{"1H8  M7G"}=" H81 M7G"; $resn_exch{"2H8  M7G"}=" H82 M7G"; $resn_exch{"1HM7 M7G"}="HM71 M7G";
$resn_exch{"2HM7 M7G"}="HM72 M7G"; $resn_exch{"3HM7 M7G"}="HM73 M7G"; $resn_exch{"1HN2 M7G"}="HN21 M7G";
$resn_exch{"2HN2 M7G"}="HN22 M7G"; $resn_exch{"1H7  M7P"}=" H71 M7P"; $resn_exch{"2H7  M7P"}=" H72 M7P";
$resn_exch{"1HH3 M8C"}="HH31 M8C"; $resn_exch{"2HH3 M8C"}="HH32 M8C"; $resn_exch{"3HH3 M8C"}="HH33 M8C";
$resn_exch{"1H12 M98"}="H121 M98"; $resn_exch{"2H12 M98"}="H122 M98"; $resn_exch{"1H13 M98"}="H131 M98";
$resn_exch{"2H13 M98"}="H132 M98"; $resn_exch{"1H18 M98"}="H181 M98"; $resn_exch{"2H18 M98"}="H182 M98";
$resn_exch{"1H1  M99"}=" H11 M99"; $resn_exch{"1H12 M99"}="H121 M99"; $resn_exch{"2H12 M99"}="H122 M99";
$resn_exch{"1H13 M99"}="H131 M99"; $resn_exch{"2H13 M99"}="H132 M99"; $resn_exch{"8H8  M99"}=" H88 M99";
$resn_exch{"4H4  M99"}=" H44 M99"; $resn_exch{"6H6  M99"}=" H66 M99"; $resn_exch{"7H7  M99"}=" H77 M99";
$resn_exch{"9H9  M99"}=" H99 M99"; $resn_exch{"11H1 M99"}="H111 M99"; $resn_exch{"1H18 M99"}="H181 M99";
$resn_exch{"2H18 M99"}="H182 M99"; $resn_exch{"20H2 M99"}="H220 M99"; $resn_exch{"21H2 M99"}="H221 M99";
$resn_exch{"22H2 M99"}="H222 M99"; $resn_exch{"23H2 M99"}="H223 M99"; $resn_exch{"1H13 MA0"}="H131 MA0";
$resn_exch{"2H13 MA0"}="H132 MA0"; $resn_exch{"1H1C MA0"}="H1C1 MA0"; $resn_exch{"2H1C MA0"}="H1C2 MA0";
$resn_exch{"1H2C MA0"}="H2C1 MA0"; $resn_exch{"2H2C MA0"}="H2C2 MA0"; $resn_exch{"1H10 MA0"}="H101 MA0";
$resn_exch{"2H10 MA0"}="H102 MA0"; $resn_exch{"3H10 MA0"}="H103 MA0"; $resn_exch{"1H6  MA1"}=" H61 MA1";
$resn_exch{"2H6  MA1"}=" H62 MA1"; $resn_exch{"1H6  MA2"}=" H61 MA2"; $resn_exch{"2H6  MA2"}=" H62 MA2";
$resn_exch{"1H7  MA2"}=" H71 MA2"; $resn_exch{"2H7  MA2"}=" H72 MA2"; $resn_exch{"3H7  MA2"}=" H73 MA2";
$resn_exch{"1H6  MA3"}=" H61 MA3"; $resn_exch{"2H6  MA3"}=" H62 MA3"; $resn_exch{"1H7  MA3"}=" H71 MA3";
$resn_exch{"2H7  MA3"}=" H72 MA3"; $resn_exch{"3H7  MA3"}=" H73 MA3"; $resn_exch{"1H1  MA4"}=" H11 MA4";
$resn_exch{"1H2  MA4"}=" H21 MA4"; $resn_exch{"1H3  MA4"}=" H31 MA4"; $resn_exch{"1H4  MA4"}=" H41 MA4";
$resn_exch{"1H5  MA4"}=" H51 MA4"; $resn_exch{"1H6  MA4"}=" H61 MA4"; $resn_exch{"2H6  MA4"}=" H62 MA4";
$resn_exch{"1H10 MA4"}="H101 MA4"; $resn_exch{"1H20 MA4"}="H201 MA4"; $resn_exch{"1H30 MA4"}="H301 MA4";
$resn_exch{"1H40 MA4"}="H401 MA4"; $resn_exch{"1H50 MA4"}="H501 MA4"; $resn_exch{"1H60 MA4"}="H601 MA4";
$resn_exch{"2H60 MA4"}="H602 MA4"; $resn_exch{"1H11 MA4"}="H111 MA4"; $resn_exch{"2H11 MA4"}="H112 MA4";
$resn_exch{"1H21 MA4"}="H211 MA4"; $resn_exch{"2H21 MA4"}="H212 MA4"; $resn_exch{"1H31 MA4"}="H311 MA4";
$resn_exch{"2H31 MA4"}="H312 MA4"; $resn_exch{"1H41 MA4"}="H411 MA4"; $resn_exch{"2H41 MA4"}="H412 MA4";
$resn_exch{"1H51 MA4"}="H511 MA4"; $resn_exch{"2H51 MA4"}="H512 MA4"; $resn_exch{"1H61 MA4"}="H611 MA4";
$resn_exch{"2H61 MA4"}="H612 MA4"; $resn_exch{"1H12 MA4"}="H121 MA4"; $resn_exch{"1H22 MA4"}="H221 MA4";
$resn_exch{"2H22 MA4"}="H222 MA4"; $resn_exch{"1H32 MA4"}="H321 MA4"; $resn_exch{"2H32 MA4"}="H322 MA4";
$resn_exch{"1H42 MA4"}="H421 MA4"; $resn_exch{"2H42 MA4"}="H422 MA4"; $resn_exch{"1H52 MA4"}="H521 MA4";
$resn_exch{"2H52 MA4"}="H522 MA4"; $resn_exch{"1H62 MA4"}="H621 MA4"; $resn_exch{"2H62 MA4"}="H622 MA4";
$resn_exch{"1H42 MA5"}="H421 MA5"; $resn_exch{"2H42 MA5"}="H422 MA5"; $resn_exch{"1H32 MA5"}="H321 MA5";
$resn_exch{"2H32 MA5"}="H322 MA5"; $resn_exch{"1H22 MA5"}="H221 MA5"; $resn_exch{"2H22 MA5"}="H222 MA5";
$resn_exch{"1H12 MA5"}="H121 MA5"; $resn_exch{"2H12 MA5"}="H122 MA5"; $resn_exch{"1H62 MA5"}="H621 MA5";
$resn_exch{"2H62 MA5"}="H622 MA5"; $resn_exch{"1H61 MA5"}="H611 MA5"; $resn_exch{"2H61 MA5"}="H612 MA5";
$resn_exch{"1H11 MA5"}="H111 MA5"; $resn_exch{"2H11 MA5"}="H112 MA5"; $resn_exch{"1H60 MA5"}="H601 MA5";
$resn_exch{"2H60 MA5"}="H602 MA5"; $resn_exch{"1H6  MA5"}=" H61 MA5"; $resn_exch{"2H6  MA5"}=" H62 MA5";
$resn_exch{"1HOP MA6"}="HOP1 MA6"; $resn_exch{"2HOP MA6"}="HOP2 MA6"; $resn_exch{"1H5* MA6"}=" H5' MA6";
$resn_exch{"2H5* MA6"}="H5'' MA6"; $resn_exch{" H4* MA6"}=" H4' MA6"; $resn_exch{" H1* MA6"}=" H1' MA6";
$resn_exch{"1H9  MA6"}=" H91 MA6"; $resn_exch{"2H9  MA6"}=" H92 MA6"; $resn_exch{"3H9  MA6"}=" H93 MA6";
$resn_exch{"1H10 MA6"}="H101 MA6"; $resn_exch{"2H10 MA6"}="H102 MA6"; $resn_exch{"3H10 MA6"}="H103 MA6";
$resn_exch{" H2* MA6"}=" H2' MA6"; $resn_exch{"*HO2 MA6"}="HO2' MA6"; $resn_exch{" H3* MA6"}=" H3' MA6";
$resn_exch{"*HO3 MA6"}="HO3' MA6"; $resn_exch{"2HOP MA7"}="HOP2 MA7"; $resn_exch{"1H6  MA7"}=" H61 MA7";
$resn_exch{"2H6  MA7"}=" H62 MA7"; $resn_exch{"1HN  MA7"}=" HN1 MA7"; $resn_exch{"2HN  MA7"}=" HN2 MA7";
$resn_exch{"3HN  MA7"}=" HN3 MA7"; $resn_exch{"1H2* MA7"}=" H2' MA7"; $resn_exch{"2H2* MA7"}="H2'' MA7";
$resn_exch{"1H5* MA7"}=" H5' MA7"; $resn_exch{"2H5* MA7"}="H5'' MA7"; $resn_exch{" H4* MA7"}=" H4' MA7";
$resn_exch{" H1* MA7"}=" H1' MA7"; $resn_exch{" H3* MA7"}=" H3' MA7"; $resn_exch{"H3T  MA7"}="HO3' MA7";
$resn_exch{"3HOP MA7"}="HOP3 MA7"; $resn_exch{"1H4C MA8"}="H4C1 MA8"; $resn_exch{"2H4C MA8"}="H4C2 MA8";
$resn_exch{"1H2N MA8"}="H2N1 MA8"; $resn_exch{"2H2N MA8"}="H2N2 MA8"; $resn_exch{"1H15 MA8"}="H151 MA8";
$resn_exch{"2H15 MA8"}="H152 MA8"; $resn_exch{"1H15 MA9"}="H151 MA9"; $resn_exch{"2H15 MA9"}="H152 MA9";
$resn_exch{"1H28 MA9"}="H281 MA9"; $resn_exch{"2H28 MA9"}="H282 MA9"; $resn_exch{"1H14 MA9"}="H141 MA9";
$resn_exch{"2H14 MA9"}="H142 MA9"; $resn_exch{"1H27 MA9"}="H271 MA9"; $resn_exch{"2H27 MA9"}="H272 MA9";
$resn_exch{"1H   MA9"}=" H1  MA9"; $resn_exch{"2H   MA9"}=" H2  MA9"; $resn_exch{"3H   MA9"}=" H3  MA9";
$resn_exch{" HN  MAA"}=" H   MAA"; $resn_exch{"1HM  MAA"}=" HM1 MAA"; $resn_exch{"2HM  MAA"}=" HM2 MAA";
$resn_exch{"3HM  MAA"}=" HM3 MAA"; $resn_exch{"1HB  MAA"}=" HB1 MAA"; $resn_exch{"2HB  MAA"}=" HB2 MAA";
$resn_exch{"3HB  MAA"}=" HB3 MAA"; $resn_exch{"2H6A MAB"}="H6A2 MAB"; $resn_exch{"1H6A MAB"}="H6A1 MAB";
$resn_exch{"1H2O MAB"}="H2O1 MAB"; $resn_exch{"1H3O MAB"}="H3O1 MAB"; $resn_exch{"2H6B MAB"}="H6B2 MAB";
$resn_exch{"1H6B MAB"}="H6B1 MAB"; $resn_exch{"2H2O MAB"}="H2O2 MAB"; $resn_exch{"2H3O MAB"}="H3O2 MAB";
$resn_exch{"1H4O MAB"}="H4O1 MAB"; $resn_exch{"1H1O MAB"}="H1O1 MAB"; $resn_exch{"1H2  MAC"}=" H21 MAC";
$resn_exch{"2H2  MAC"}=" H22 MAC"; $resn_exch{"3H2  MAC"}=" H23 MAC"; $resn_exch{"1H5* MAD"}=" H5' MAD";
$resn_exch{"2H5* MAD"}="H5'' MAD"; $resn_exch{" H4* MAD"}=" H4' MAD"; $resn_exch{" H3* MAD"}=" H3' MAD";
$resn_exch{" H2* MAD"}=" H2' MAD"; $resn_exch{" H1* MAD"}=" H1' MAD"; $resn_exch{"1HM1 MAD"}="HM11 MAD";
$resn_exch{"2HM1 MAD"}="HM12 MAD"; $resn_exch{"3HM1 MAD"}="HM13 MAD"; $resn_exch{"1H6C MAF"}="H6C1 MAF";
$resn_exch{"2H6C MAF"}="H6C2 MAF"; $resn_exch{"1H6  MAG"}=" H61 MAG"; $resn_exch{"2H6  MAG"}=" H62 MAG";
$resn_exch{"1H8  MAG"}=" H81 MAG"; $resn_exch{"2H8  MAG"}=" H82 MAG"; $resn_exch{"3H8  MAG"}=" H83 MAG";
$resn_exch{"1HM  MAG"}=" HM1 MAG"; $resn_exch{"2HM  MAG"}=" HM2 MAG"; $resn_exch{"3HM  MAG"}=" HM3 MAG";
$resn_exch{"1H2  MAH"}=" H21 MAH"; $resn_exch{"2H2  MAH"}=" H22 MAH"; $resn_exch{"1H4  MAH"}=" H41 MAH";
$resn_exch{"2H4  MAH"}=" H42 MAH"; $resn_exch{"1H6  MAH"}=" H61 MAH"; $resn_exch{"2H6  MAH"}=" H62 MAH";
$resn_exch{"3H6  MAH"}=" H63 MAH"; $resn_exch{"1HN  MAI"}="HN1A MAI"; $resn_exch{"2HN  MAI"}=" HN2 MAI";
$resn_exch{"1HJ  MAI"}=" HJ1 MAI"; $resn_exch{"2HJ  MAI"}=" HJ2 MAI"; $resn_exch{"3HJ  MAI"}=" HJ3 MAI";
$resn_exch{"1HB  MAI"}=" HB1 MAI"; $resn_exch{"2HB  MAI"}=" HB2 MAI"; $resn_exch{"1HG  MAI"}=" HG1 MAI";
$resn_exch{"2HG  MAI"}=" HG2 MAI"; $resn_exch{"1HD  MAI"}=" HD1 MAI"; $resn_exch{"2HD  MAI"}=" HD2 MAI";
$resn_exch{"1HN2 MAI"}="HN21 MAI"; $resn_exch{"2HN2 MAI"}="HN22 MAI"; $resn_exch{"1H7  MAJ"}=" H71 MAJ";
$resn_exch{"2H7  MAJ"}=" H72 MAJ"; $resn_exch{"1H8  MAJ"}=" H81 MAJ"; $resn_exch{"2H8  MAJ"}=" H82 MAJ";
$resn_exch{"1H9  MAJ"}=" H91 MAJ"; $resn_exch{"2H9  MAJ"}=" H92 MAJ"; $resn_exch{"1HN1 MAJ"}="HN11 MAJ";
$resn_exch{"2HN1 MAJ"}="HN12 MAJ"; $resn_exch{"1H6  MAL"}=" H61 MAL"; $resn_exch{"2H6  MAL"}=" H62 MAL";
$resn_exch{"1H6' MAL"}="H6'1 MAL"; $resn_exch{"2H6' MAL"}="H6'2 MAL"; $resn_exch{"'HO1 MAL"}="HO1' MAL";
$resn_exch{"'HO2 MAL"}="HO2' MAL"; $resn_exch{"'HO3 MAL"}="HO3' MAL"; $resn_exch{"'HO6 MAL"}="HO6' MAL";
$resn_exch{"1H6  MAM"}=" H61 MAM"; $resn_exch{"2H6  MAM"}=" H62 MAM"; $resn_exch{"1H7  MAM"}=" H71 MAM";
$resn_exch{"2H7  MAM"}=" H72 MAM"; $resn_exch{"3H7  MAM"}=" H73 MAM"; $resn_exch{"1H6  MAN"}=" H61 MAN";
$resn_exch{"2H6  MAN"}=" H62 MAN"; $resn_exch{"1HN  MAO"}=" HN1 MAO"; $resn_exch{"2HN  MAO"}=" HN2 MAO";
$resn_exch{"1HB  MAO"}=" HB1 MAO"; $resn_exch{"2HB  MAO"}=" HB2 MAO"; $resn_exch{"1HG  MAO"}=" HG1 MAO";
$resn_exch{"2HG  MAO"}=" HG2 MAO"; $resn_exch{"1HE  MAO"}=" HE1 MAO"; $resn_exch{"2HE  MAO"}=" HE2 MAO";
$resn_exch{"3HE  MAO"}=" HE3 MAO"; $resn_exch{"1H5* MAO"}="H5'1 MAO"; $resn_exch{"2H5* MAO"}="H5'2 MAO";
$resn_exch{" H4* MAO"}=" H4' MAO"; $resn_exch{" H1* MAO"}=" H1' MAO"; $resn_exch{"1HN6 MAO"}="HN61 MAO";
$resn_exch{"2HN6 MAO"}="HN62 MAO"; $resn_exch{" H2* MAO"}=" H2' MAO"; $resn_exch{"*HO2 MAO"}="HO2' MAO";
$resn_exch{" H3* MAO"}=" H3' MAO"; $resn_exch{"*HO3 MAO"}="HO3' MAO"; $resn_exch{"2HOG MAP"}="HOG2 MAP";
$resn_exch{"3HOG MAP"}="HOG3 MAP"; $resn_exch{"2HOB MAP"}="HOB2 MAP"; $resn_exch{"1HNB MAP"}="HNB1 MAP";
$resn_exch{"1H5* MAP"}="H5'1 MAP"; $resn_exch{"2H5* MAP"}="H5'2 MAP"; $resn_exch{" H4* MAP"}=" H4' MAP";
$resn_exch{" H3* MAP"}=" H3' MAP"; $resn_exch{"*HO3 MAP"}="HO3' MAP"; $resn_exch{" H2* MAP"}=" H2' MAP";
$resn_exch{"*HO2 MAP"}="HO2' MAP"; $resn_exch{" H1* MAP"}=" H1' MAP"; $resn_exch{"1HN6 MAP"}="HN61 MAP";
$resn_exch{"2HN6 MAP"}="HN62 MAP"; $resn_exch{"1H9  MAQ"}=" H91 MAQ"; $resn_exch{"2H9  MAQ"}=" H92 MAQ";
$resn_exch{"3H9  MAQ"}=" H93 MAQ"; $resn_exch{"1H2  MAQ"}=" H21 MAQ"; $resn_exch{"2H2  MAQ"}=" H22 MAQ";
$resn_exch{"1H11 MAR"}="H111 MAR"; $resn_exch{"2H11 MAR"}="H112 MAR"; $resn_exch{"2HO1 MAR"}="HO12 MAR";
$resn_exch{"1H14 MAR"}="H141 MAR"; $resn_exch{"2H14 MAR"}="H142 MAR"; $resn_exch{"3H14 MAR"}="H143 MAR";
$resn_exch{"1H15 MAR"}="H151 MAR"; $resn_exch{"2H15 MAR"}="H152 MAR"; $resn_exch{"7HO1 MAR"}="HO17 MAR";
$resn_exch{"1H21 MAR"}="H211 MAR"; $resn_exch{"2H21 MAR"}="H212 MAR"; $resn_exch{"3H21 MAR"}="H213 MAR";
$resn_exch{" H1* MAR"}=" H1' MAR"; $resn_exch{"1H2* MAR"}="H2'1 MAR"; $resn_exch{"2H2* MAR"}="H2'2 MAR";
$resn_exch{" H3* MAR"}=" H3' MAR"; $resn_exch{"1HN* MAR"}="HN'1 MAR"; $resn_exch{"2HN* MAR"}="HN'2 MAR";
$resn_exch{" H4* MAR"}=" H4' MAR"; $resn_exch{" H5* MAR"}=" H5' MAR"; $resn_exch{"1H6* MAR"}="H6'1 MAR";
$resn_exch{"2H6* MAR"}="H6'2 MAR"; $resn_exch{"3H6* MAR"}="H6'3 MAR"; $resn_exch{"1HB2 MAR"}="HB21 MAR";
$resn_exch{"2HB2 MAR"}="HB22 MAR"; $resn_exch{"3HOB MAR"}="HOB3 MAR"; $resn_exch{"4HOB MAR"}="HOB4 MAR";
$resn_exch{"1HB6 MAR"}="HB61 MAR"; $resn_exch{"2HB6 MAR"}="HB62 MAR"; $resn_exch{"3HB6 MAR"}="HB63 MAR";
$resn_exch{"1H1  MAS"}=" H11 MAS"; $resn_exch{"2H1  MAS"}=" H12 MAS"; $resn_exch{"3H1  MAS"}=" H13 MAS";
$resn_exch{"1H2  MAS"}=" H21 MAS"; $resn_exch{"2H2  MAS"}=" H22 MAS"; $resn_exch{"3H2  MAS"}=" H23 MAS";
$resn_exch{"1H2  MAT"}=" H21 MAT"; $resn_exch{"2H2  MAT"}=" H22 MAT"; $resn_exch{"1H5  MAT"}=" H51 MAT";
$resn_exch{"2H5  MAT"}=" H52 MAT"; $resn_exch{"1H7  MAT"}=" H71 MAT"; $resn_exch{"2H7  MAT"}=" H72 MAT";
$resn_exch{"3H7  MAT"}=" H73 MAT"; $resn_exch{"1H8  MAT"}=" H81 MAT"; $resn_exch{"2H8  MAT"}=" H82 MAT";
$resn_exch{"3H8  MAT"}=" H83 MAT"; $resn_exch{"1H9  MAT"}=" H91 MAT"; $resn_exch{"2H9  MAT"}=" H92 MAT";
$resn_exch{"3H9  MAT"}=" H93 MAT"; $resn_exch{"5HC  MAU"}=" HC5 MAU"; $resn_exch{"6HC  MAU"}=" HC6 MAU";
$resn_exch{"2HC0 MAU"}="HC02 MAU"; $resn_exch{"2HC4 MAU"}="HC42 MAU"; $resn_exch{"1H25 MAU"}="H251 MAU";
$resn_exch{"2H25 MAU"}="H252 MAU"; $resn_exch{"4HO  MAU"}=" HO4 MAU"; $resn_exch{"9HC  MAU"}=" HC9 MAU";
$resn_exch{"1HC0 MAU"}="HC01 MAU"; $resn_exch{"1HC1 MAU"}="HC11 MAU"; $resn_exch{"1HC2 MAU"}="HC21 MAU";
$resn_exch{"1HC3 MAU"}="HC31 MAU"; $resn_exch{"1HC4 MAU"}="HC41 MAU"; $resn_exch{"1HC5 MAU"}="HC51 MAU";
$resn_exch{"1HC6 MAU"}="HC61 MAU"; $resn_exch{"1HC7 MAU"}="HC71 MAU"; $resn_exch{"1HC9 MAU"}="HC91 MAU";
$resn_exch{"2HC2 MAU"}="HC22 MAU"; $resn_exch{"2HC3 MAU"}="HC32 MAU"; $resn_exch{"1HO5 MAU"}="HO51 MAU";
$resn_exch{"1H40 MAU"}="H401 MAU"; $resn_exch{"2H40 MAU"}="H402 MAU"; $resn_exch{"3H40 MAU"}="H403 MAU";
$resn_exch{"1H41 MAU"}="H411 MAU"; $resn_exch{"2H41 MAU"}="H412 MAU"; $resn_exch{"3H41 MAU"}="H413 MAU";
$resn_exch{"1HO6 MAU"}="HO61 MAU"; $resn_exch{"1H42 MAU"}="H421 MAU"; $resn_exch{"2H42 MAU"}="H422 MAU";
$resn_exch{"3H42 MAU"}="H423 MAU"; $resn_exch{"1H43 MAU"}="H431 MAU"; $resn_exch{"2H43 MAU"}="H432 MAU";
$resn_exch{"3H43 MAU"}="H433 MAU"; $resn_exch{"1H44 MAU"}="H441 MAU"; $resn_exch{"2H44 MAU"}="H442 MAU";
$resn_exch{"3H44 MAU"}="H443 MAU"; $resn_exch{"2HN6 MAU"}="HN62 MAU"; $resn_exch{"1H45 MAU"}="H451 MAU";
$resn_exch{"2H45 MAU"}="H452 MAU"; $resn_exch{"1H46 MAU"}="H461 MAU"; $resn_exch{"2H46 MAU"}="H462 MAU";
$resn_exch{"3H46 MAU"}="H463 MAU"; $resn_exch{"2HC8 MAU"}="HC82 MAU"; $resn_exch{"2HO9 MAU"}="HO92 MAU";
$resn_exch{"3HO  MAU"}=" HO3 MAU"; $resn_exch{"3HC1 MAU"}="HC13 MAU"; $resn_exch{"3HO1 MAU"}="HO13 MAU";
$resn_exch{"1H47 MAU"}="H471 MAU"; $resn_exch{"2H47 MAU"}="H472 MAU"; $resn_exch{"3H47 MAU"}="H473 MAU";
$resn_exch{"1H48 MAU"}="H481 MAU"; $resn_exch{"2H48 MAU"}="H482 MAU"; $resn_exch{"3H48 MAU"}="H483 MAU";
$resn_exch{"3HC3 MAU"}="HC33 MAU"; $resn_exch{"3HC5 MAU"}="HC53 MAU"; $resn_exch{"3HC6 MAU"}="HC63 MAU";
$resn_exch{"3HC7 MAU"}="HC73 MAU"; $resn_exch{"3HC8 MAU"}="HC83 MAU"; $resn_exch{"1H39 MAU"}="H391 MAU";
$resn_exch{"2H39 MAU"}="H392 MAU"; $resn_exch{"3H39 MAU"}="H393 MAU"; $resn_exch{"1H7C MAX"}="H7C1 MAX";
$resn_exch{"2H7C MAX"}="H7C2 MAX"; $resn_exch{"1H9C MAX"}="H9C1 MAX"; $resn_exch{"2H9C MAX"}="H9C2 MAX";
$resn_exch{"1H14 MAX"}="H141 MAX"; $resn_exch{"2H14 MAX"}="H142 MAX"; $resn_exch{"1H22 MAX"}="H221 MAX";
$resn_exch{"2H22 MAX"}="H222 MAX"; $resn_exch{"3H22 MAX"}="H223 MAX"; $resn_exch{"1H26 MAX"}="H261 MAX";
$resn_exch{"2H26 MAX"}="H262 MAX"; $resn_exch{"3H26 MAX"}="H263 MAX"; $resn_exch{"1HCM MAY"}="HCM1 MAY";
$resn_exch{"2HCM MAY"}="HCM2 MAY"; $resn_exch{"3HCM MAY"}="HCM3 MAY"; $resn_exch{"1H1  MAY"}=" H11 MAY";
$resn_exch{"2H1  MAY"}=" H12 MAY"; $resn_exch{"1H2  MAY"}=" H21 MAY"; $resn_exch{"2H2  MAY"}=" H22 MAY";
$resn_exch{"1H3  MAY"}=" H31 MAY"; $resn_exch{"2H3  MAY"}=" H32 MAY"; $resn_exch{"1H4  MAY"}=" H41 MAY";
$resn_exch{"2H4  MAY"}=" H42 MAY"; $resn_exch{"1H5  MAY"}=" H51 MAY"; $resn_exch{"1H6  MAY"}=" H61 MAY";
$resn_exch{"1H7  MAY"}=" H71 MAY"; $resn_exch{"2H7  MAY"}=" H72 MAY"; $resn_exch{"1H8  MAY"}=" H81 MAY";
$resn_exch{"1H9  MAY"}=" H91 MAY"; $resn_exch{"1H10 MAY"}="H101 MAY"; $resn_exch{"2H10 MAY"}="H102 MAY";
$resn_exch{"1H11 MAY"}="H111 MAY"; $resn_exch{"1H12 MAY"}="H121 MAY"; $resn_exch{"1H13 MAY"}="H131 MAY";
$resn_exch{"2H13 MAY"}="H132 MAY"; $resn_exch{"1H14 MAY"}="H141 MAY"; $resn_exch{"1H15 MAY"}="H151 MAY";
$resn_exch{"1H16 MAY"}="H161 MAY"; $resn_exch{"2H16 MAY"}="H162 MAY"; $resn_exch{"1H17 MAY"}="H171 MAY";
$resn_exch{"2H17 MAY"}="H172 MAY"; $resn_exch{"1H18 MAY"}="H181 MAY"; $resn_exch{"2H18 MAY"}="H182 MAY";
$resn_exch{"1H19 MAY"}="H191 MAY"; $resn_exch{"2H19 MAY"}="H192 MAY"; $resn_exch{"1H20 MAY"}="H201 MAY";
$resn_exch{"2H20 MAY"}="H202 MAY"; $resn_exch{"3H20 MAY"}="H203 MAY"; $resn_exch{"1H12 MAZ"}="H121 MAZ";
$resn_exch{"2H12 MAZ"}="H122 MAZ"; $resn_exch{"1H11 MAZ"}="H111 MAZ"; $resn_exch{"2H11 MAZ"}="H112 MAZ";
$resn_exch{"1HN  MBA"}=" HN1 MBA"; $resn_exch{"2HN  MBA"}=" HN2 MBA"; $resn_exch{"1H1  MBA"}=" H11 MBA";
$resn_exch{"2H1  MBA"}=" H12 MBA"; $resn_exch{"1H2' MBA"}="H2'1 MBA"; $resn_exch{"2H2' MBA"}="H2'2 MBA";
$resn_exch{"3H2' MBA"}="H2'3 MBA"; $resn_exch{"1H3  MBA"}=" H31 MBA"; $resn_exch{"2H3  MBA"}=" H32 MBA";
$resn_exch{"1H4  MBA"}=" H41 MBA"; $resn_exch{"2H4  MBA"}=" H42 MBA"; $resn_exch{"3H4  MBA"}=" H43 MBA";
$resn_exch{"1H20 MBB"}="H201 MBB"; $resn_exch{"2H20 MBB"}="H202 MBB"; $resn_exch{"1H21 MBB"}="H211 MBB";
$resn_exch{"2H21 MBB"}="H212 MBB"; $resn_exch{"1H22 MBB"}="H221 MBB"; $resn_exch{"2H22 MBB"}="H222 MBB";
$resn_exch{"1H23 MBB"}="H231 MBB"; $resn_exch{"2H23 MBB"}="H232 MBB"; $resn_exch{"1H24 MBB"}="H241 MBB";
$resn_exch{"2H24 MBB"}="H242 MBB"; $resn_exch{"3H24 MBB"}="H243 MBB"; $resn_exch{"1HN1 MBB"}="HN11 MBB";
$resn_exch{"2HN1 MBB"}="HN12 MBB"; $resn_exch{"3HN1 MBB"}="HN13 MBB"; $resn_exch{"1HN2 MBB"}="HN21 MBB";
$resn_exch{"2HN2 MBB"}="HN22 MBB"; $resn_exch{"3HN2 MBB"}="HN23 MBB"; $resn_exch{"1H25 MBB"}="H251 MBB";
$resn_exch{"2H25 MBB"}="H252 MBB"; $resn_exch{"3H25 MBB"}="H253 MBB"; $resn_exch{"1H24 MBC"}="H241 MBC";
$resn_exch{"2H24 MBC"}="H242 MBC"; $resn_exch{"1H26 MBC"}="H261 MBC"; $resn_exch{"2H26 MBC"}="H262 MBC";
$resn_exch{"1HB3 MBD"}="HB31 MBD"; $resn_exch{"2HB3 MBD"}="HB32 MBD"; $resn_exch{"3HB3 MBD"}="HB33 MBD";
$resn_exch{"1H6C MBF"}="H6C1 MBF"; $resn_exch{"2H6C MBF"}="H6C2 MBF"; $resn_exch{"1H6  MBG"}=" H61 MBG";
$resn_exch{"2H6  MBG"}=" H62 MBG"; $resn_exch{"1H7  MBG"}=" H71 MBG"; $resn_exch{"2H7  MBG"}=" H72 MBG";
$resn_exch{"3H7  MBG"}=" H73 MBG"; $resn_exch{"1HN2 MBH"}="HN21 MBH"; $resn_exch{"2HN2 MBH"}="HN22 MBH";
$resn_exch{"1H   MBH"}=" H1  MBH"; $resn_exch{"2H   MBH"}=" H2A MBH"; $resn_exch{"1H1M MBH"}="H1M1 MBH";
$resn_exch{"2H1M MBH"}="H1M2 MBH"; $resn_exch{"3H1M MBH"}="H1M3 MBH"; $resn_exch{"1HA2 MBH"}="HA21 MBH";
$resn_exch{"2HA2 MBH"}="HA22 MBH"; $resn_exch{"1H   MBN"}=" H1  MBN"; $resn_exch{"2H   MBN"}=" H2A MBN";
$resn_exch{"3H   MBN"}=" H3A MBN"; $resn_exch{"1H3* MBP"}="H3'1 MBP"; $resn_exch{"2H3* MBP"}="H3'2 MBP";
$resn_exch{"3H3* MBP"}="H3'3 MBP"; $resn_exch{" H2* MBP"}=" H2' MBP"; $resn_exch{"1H12 MBP"}="H121 MBP";
$resn_exch{"2H12 MBP"}="H122 MBP"; $resn_exch{"3H12 MBP"}="H123 MBP"; $resn_exch{"1H2N MBP"}="H2N1 MBP";
$resn_exch{"2H2N MBP"}="H2N2 MBP"; $resn_exch{"1H60 MBP"}="H601 MBP"; $resn_exch{"2H60 MBP"}="H602 MBP";
$resn_exch{" HN  MBQ"}=" H2  MBQ"; $resn_exch{" HB1 MBQ"}=" HB2 MBQ"; $resn_exch{" HB2 MBQ"}=" HB3 MBQ";
$resn_exch{"1HAE MBQ"}="HAE1 MBQ"; $resn_exch{"2HAE MBQ"}="HAE2 MBQ"; $resn_exch{"3HAK MBQ"}="HAK3 MBQ";
$resn_exch{"2HAK MBQ"}="HAK2 MBQ"; $resn_exch{"1HAK MBQ"}="HAK1 MBQ"; $resn_exch{"1H5  MBS"}=" H51 MBS";
$resn_exch{"2H5  MBS"}=" H52 MBS"; $resn_exch{"3H5  MBS"}=" H53 MBS"; $resn_exch{"1H16 MBS"}="H161 MBS";
$resn_exch{"2H16 MBS"}="H162 MBS"; $resn_exch{"1H3  MBS"}=" H31 MBS"; $resn_exch{"2H3  MBS"}=" H32 MBS";
$resn_exch{"1H2  MBS"}=" H21 MBS"; $resn_exch{"1HO1 MBS"}="HO11 MBS"; $resn_exch{"1H22 MBS"}="H221 MBS";
$resn_exch{"1H23 MBS"}="H231 MBS"; $resn_exch{"1H25 MBS"}="H251 MBS"; $resn_exch{"1H26 MBS"}="H261 MBS";
$resn_exch{"1H32 MBS"}="H321 MBS"; $resn_exch{"1H33 MBS"}="H331 MBS"; $resn_exch{"1H35 MBS"}="H351 MBS";
$resn_exch{"1H36 MBS"}="H361 MBS"; $resn_exch{"1H7  MBS"}=" H71 MBS"; $resn_exch{"2H7  MBS"}=" H72 MBS";
$resn_exch{"3H7  MBS"}=" H73 MBS"; $resn_exch{"1H19 MBT"}="H191 MBT"; $resn_exch{"2H19 MBT"}="H192 MBT";
$resn_exch{"3H19 MBT"}="H193 MBT"; $resn_exch{"1H20 MBT"}="H201 MBT"; $resn_exch{"2H20 MBT"}="H202 MBT";
$resn_exch{"3H20 MBT"}="H203 MBT"; $resn_exch{"1H16 MBT"}="H161 MBT"; $resn_exch{"2H16 MBT"}="H162 MBT";
$resn_exch{"3H16 MBT"}="H163 MBT"; $resn_exch{"1H17 MBT"}="H171 MBT"; $resn_exch{"2H17 MBT"}="H172 MBT";
$resn_exch{"3H17 MBT"}="H173 MBT"; $resn_exch{"1H1A MBV"}="H1A1 MBV"; $resn_exch{"2H1A MBV"}="H1A2 MBV";
$resn_exch{"3H1A MBV"}="H1A3 MBV"; $resn_exch{"1H2A MBV"}="H2A1 MBV"; $resn_exch{"2H2A MBV"}="H2A2 MBV";
$resn_exch{"1H1B MBV"}="H1B1 MBV"; $resn_exch{"2H1B MBV"}="H1B2 MBV"; $resn_exch{"1H6B MBV"}="H6B1 MBV";
$resn_exch{"2H6B MBV"}="H6B2 MBV"; $resn_exch{"3H6B MBV"}="H6B3 MBV"; $resn_exch{"1H6C MBV"}="H6C1 MBV";
$resn_exch{"2H6C MBV"}="H6C2 MBV"; $resn_exch{"3H6C MBV"}="H6C3 MBV"; $resn_exch{"1H1C MBV"}="H1C1 MBV";
$resn_exch{"2H1C MBV"}="H1C2 MBV"; $resn_exch{"1H2D MBV"}="H2D1 MBV"; $resn_exch{"2H2D MBV"}="H2D2 MBV";
$resn_exch{"1H1D MBV"}="H1D1 MBV"; $resn_exch{"2H1D MBV"}="H1D2 MBV"; $resn_exch{"3H1D MBV"}="H1D3 MBV";
$resn_exch{"1H3A MBV"}="H3A1 MBV"; $resn_exch{"2H3A MBV"}="H3A2 MBV"; $resn_exch{"3H3A MBV"}="H3A3 MBV";
$resn_exch{"1H2B MBV"}="H2B1 MBV"; $resn_exch{"2H2B MBV"}="H2B2 MBV"; $resn_exch{"1H2C MBV"}="H2C1 MBV";
$resn_exch{"2H2C MBV"}="H2C2 MBV"; $resn_exch{"1H3D MBV"}="H3D1 MBV"; $resn_exch{"2H3D MBV"}="H3D2 MBV";
$resn_exch{"3H3D MBV"}="H3D3 MBV"; $resn_exch{"2HOP MBZ"}="HOP2 MBZ"; $resn_exch{"3HOP MBZ"}="HOP3 MBZ";
$resn_exch{"3HO* MBZ"}="HO'3 MBZ"; $resn_exch{" H1* MBZ"}=" H1' MBZ"; $resn_exch{"1H2* MBZ"}=" H2' MBZ";
$resn_exch{"2H2* MBZ"}="H2'' MBZ"; $resn_exch{" H3* MBZ"}=" H3' MBZ"; $resn_exch{" H4* MBZ"}=" H4' MBZ";
$resn_exch{"1H5* MBZ"}=" H5' MBZ"; $resn_exch{"2H5* MBZ"}="H5'' MBZ"; $resn_exch{"1H5M MBZ"}=" H71 MBZ";
$resn_exch{"2H5M MBZ"}=" H72 MBZ"; $resn_exch{"3H5M MBZ"}=" H73 MBZ"; $resn_exch{"1H11  MC"}="H111  MC";
$resn_exch{"2H11  MC"}="H112  MC"; $resn_exch{"3H11  MC"}="H113  MC"; $resn_exch{"1H7   MC"}=" H71  MC";
$resn_exch{"2H7   MC"}=" H72  MC"; $resn_exch{"1H10  MC"}="H101  MC"; $resn_exch{"2H10  MC"}="H102  MC";
$resn_exch{"1H14  MC"}="H141  MC"; $resn_exch{"2H14  MC"}="H142  MC"; $resn_exch{"1H3   MC"}=" H31  MC";
$resn_exch{"2H3   MC"}=" H32  MC"; $resn_exch{"1H1   MC"}=" H11  MC"; $resn_exch{"2H1   MC"}=" H12  MC";
$resn_exch{"H1   MC1"}="H241 MC1"; $resn_exch{"H2   MC1"}="H242 MC1"; $resn_exch{"H3   MC1"}="H243 MC1";
$resn_exch{" H25 MC1"}=" H   MC1"; $resn_exch{" H26 MC1"}=" H2  MC1"; $resn_exch{" H22 MC1"}=" HB2 MC1";
$resn_exch{" H23 MC1"}=" HB3 MC1"; $resn_exch{"1H12 MC2"}="H121 MC2"; $resn_exch{"2H12 MC2"}="H122 MC2";
$resn_exch{"1H13 MC2"}="H131 MC2"; $resn_exch{"2H13 MC2"}="H132 MC2"; $resn_exch{"1H14 MC2"}="H141 MC2";
$resn_exch{"2H14 MC2"}="H142 MC2"; $resn_exch{"1HN4 MC2"}="HN41 MC2"; $resn_exch{"2HN4 MC2"}="HN42 MC2";
$resn_exch{"1HN1 MC2"}="HN11 MC2"; $resn_exch{"2HN1 MC2"}="HN12 MC2"; $resn_exch{"1H3  MC2"}=" H31 MC2";
$resn_exch{"2H3  MC2"}=" H32 MC2"; $resn_exch{"1H1  MC3"}=" H11 MC3"; $resn_exch{"2H1  MC3"}=" H12 MC3";
$resn_exch{"1H3  MC3"}=" H31 MC3"; $resn_exch{"2H3  MC3"}=" H32 MC3"; $resn_exch{"1H4  MC3"}=" H41 MC3";
$resn_exch{"2H4  MC3"}=" H42 MC3"; $resn_exch{"1H5  MC3"}=" H51 MC3"; $resn_exch{"2H5  MC3"}=" H52 MC3";
$resn_exch{"1H6  MC3"}=" H61 MC3"; $resn_exch{"2H6  MC3"}=" H62 MC3"; $resn_exch{"3H6  MC3"}=" H63 MC3";
$resn_exch{"1H7  MC3"}=" H71 MC3"; $resn_exch{"2H7  MC3"}=" H72 MC3"; $resn_exch{"3H7  MC3"}=" H73 MC3";
$resn_exch{"1H8  MC3"}=" H81 MC3"; $resn_exch{"2H8  MC3"}=" H82 MC3"; $resn_exch{"3H8  MC3"}=" H83 MC3";
$resn_exch{"1H12 MC3"}="H121 MC3"; $resn_exch{"2H12 MC3"}="H122 MC3"; $resn_exch{"1H13 MC3"}="H131 MC3";
$resn_exch{"2H13 MC3"}="H132 MC3"; $resn_exch{"1H14 MC3"}="H141 MC3"; $resn_exch{"2H14 MC3"}="H142 MC3";
$resn_exch{"1H15 MC3"}="H151 MC3"; $resn_exch{"2H15 MC3"}="H152 MC3"; $resn_exch{"1H16 MC3"}="H161 MC3";
$resn_exch{"2H16 MC3"}="H162 MC3"; $resn_exch{"1H17 MC3"}="H171 MC3"; $resn_exch{"2H17 MC3"}="H172 MC3";
$resn_exch{"1H18 MC3"}="H181 MC3"; $resn_exch{"2H18 MC3"}="H182 MC3"; $resn_exch{"1H19 MC3"}="H191 MC3";
$resn_exch{"2H19 MC3"}="H192 MC3"; $resn_exch{"1H20 MC3"}="H201 MC3"; $resn_exch{"2H20 MC3"}="H202 MC3";
$resn_exch{"1H21 MC3"}="H211 MC3"; $resn_exch{"2H21 MC3"}="H212 MC3"; $resn_exch{"1H22 MC3"}="H221 MC3";
$resn_exch{"2H22 MC3"}="H222 MC3"; $resn_exch{"1H23 MC3"}="H231 MC3"; $resn_exch{"2H23 MC3"}="H232 MC3";
$resn_exch{"1H24 MC3"}="H241 MC3"; $resn_exch{"2H24 MC3"}="H242 MC3"; $resn_exch{"3H24 MC3"}="H243 MC3";
$resn_exch{"1H32 MC3"}="H321 MC3"; $resn_exch{"2H32 MC3"}="H322 MC3"; $resn_exch{"1H33 MC3"}="H331 MC3";
$resn_exch{"2H33 MC3"}="H332 MC3"; $resn_exch{"1H34 MC3"}="H341 MC3"; $resn_exch{"2H34 MC3"}="H342 MC3";
$resn_exch{"1H35 MC3"}="H351 MC3"; $resn_exch{"2H35 MC3"}="H352 MC3"; $resn_exch{"1H36 MC3"}="H361 MC3";
$resn_exch{"2H36 MC3"}="H362 MC3"; $resn_exch{"1H37 MC3"}="H371 MC3"; $resn_exch{"2H37 MC3"}="H372 MC3";
$resn_exch{"1H38 MC3"}="H381 MC3"; $resn_exch{"2H38 MC3"}="H382 MC3"; $resn_exch{"1H39 MC3"}="H391 MC3";
$resn_exch{"2H39 MC3"}="H392 MC3"; $resn_exch{"1H40 MC3"}="H401 MC3"; $resn_exch{"2H40 MC3"}="H402 MC3";
$resn_exch{"1H41 MC3"}="H411 MC3"; $resn_exch{"2H41 MC3"}="H412 MC3"; $resn_exch{"1H42 MC3"}="H421 MC3";
$resn_exch{"2H42 MC3"}="H422 MC3"; $resn_exch{"1H43 MC3"}="H431 MC3"; $resn_exch{"2H43 MC3"}="H432 MC3";
$resn_exch{"1H44 MC3"}="H441 MC3"; $resn_exch{"2H44 MC3"}="H442 MC3"; $resn_exch{"3H44 MC3"}="H443 MC3";
$resn_exch{"1H2  MC9"}=" H21 MC9"; $resn_exch{"2H2  MC9"}="H22A MC9"; $resn_exch{"1H4  MC9"}=" H41 MC9";
$resn_exch{"2H4  MC9"}=" H42 MC9"; $resn_exch{"1H9  MC9"}=" H91 MC9"; $resn_exch{"2H9  MC9"}=" H92 MC9";
$resn_exch{"1H11 MC9"}="H111 MC9"; $resn_exch{"2H11 MC9"}="H112 MC9"; $resn_exch{"1H12 MC9"}="H121 MC9";
$resn_exch{"2H12 MC9"}="H122 MC9"; $resn_exch{"1H15 MC9"}="H151 MC9"; $resn_exch{"2H15 MC9"}="H152 MC9";
$resn_exch{"1H16 MC9"}="H161 MC9"; $resn_exch{"2H16 MC9"}="H162 MC9"; $resn_exch{"1H18 MC9"}="H181 MC9";
$resn_exch{"2H18 MC9"}="H182 MC9"; $resn_exch{"3H18 MC9"}="H183 MC9"; $resn_exch{"1H19 MC9"}="H191 MC9";
$resn_exch{"2H19 MC9"}="H192 MC9"; $resn_exch{"1H21 MC9"}="H211 MC9"; $resn_exch{"2H21 MC9"}="H212 MC9";
$resn_exch{"3H21 MC9"}="H213 MC9"; $resn_exch{"1H26 MC9"}="H261 MC9"; $resn_exch{"2H26 MC9"}="H262 MC9";
$resn_exch{"1H27 MC9"}="H271 MC9"; $resn_exch{"2H27 MC9"}="H272 MC9"; $resn_exch{"1HN6 MCA"}="HN61 MCA";
$resn_exch{"2HN6 MCA"}="HN62 MCA"; $resn_exch{"'HO2 MCA"}="HO2' MCA"; $resn_exch{"1H5' MCA"}="H5'1 MCA";
$resn_exch{"2H5' MCA"}="H5'2 MCA"; $resn_exch{"1HPB MCA"}="HPB1 MCA"; $resn_exch{"2HPB MCA"}="HPB2 MCA";
$resn_exch{"1H9  MCA"}=" H91 MCA"; $resn_exch{"2H9  MCA"}=" H92 MCA"; $resn_exch{"3H9  MCA"}=" H93 MCA";
$resn_exch{"1H8  MCA"}=" H81 MCA"; $resn_exch{"2H8  MCA"}=" H82 MCA"; $resn_exch{"3H8  MCA"}=" H83 MCA";
$resn_exch{"1H5  MCA"}=" H51 MCA"; $resn_exch{"2H5  MCA"}=" H52 MCA"; $resn_exch{"1H4  MCA"}=" H41 MCA";
$resn_exch{"2H4  MCA"}=" H42 MCA"; $resn_exch{"1H2  MCA"}=" H21 MCA"; $resn_exch{"2H2  MCA"}=" H22 MCA";
$resn_exch{"1H1  MCA"}=" H11 MCA"; $resn_exch{"2H1  MCA"}=" H12 MCA"; $resn_exch{"1H3  MCA"}=" H31 MCA";
$resn_exch{"2H3  MCA"}=" H32 MCA"; $resn_exch{"3H3  MCA"}=" H33 MCA"; $resn_exch{"1HA  MCB"}="HA1A MCB";
$resn_exch{"1HN6 MCD"}="HN61 MCD"; $resn_exch{"2HN6 MCD"}="HN62 MCD"; $resn_exch{"'HO2 MCD"}="HO2' MCD";
$resn_exch{"1H5' MCD"}="H5'1 MCD"; $resn_exch{"2H5' MCD"}="H5'2 MCD"; $resn_exch{"1HPB MCD"}="HPB1 MCD";
$resn_exch{"2HPB MCD"}="HPB2 MCD"; $resn_exch{"1HP9 MCD"}="HP91 MCD"; $resn_exch{"2HP9 MCD"}="HP92 MCD";
$resn_exch{"3HP9 MCD"}="HP93 MCD"; $resn_exch{"1HP8 MCD"}="HP81 MCD"; $resn_exch{"2HP8 MCD"}="HP82 MCD";
$resn_exch{"3HP8 MCD"}="HP83 MCD"; $resn_exch{"3HOP MCD"}="HOP3 MCD"; $resn_exch{"2HNP MCD"}="HNP2 MCD";
$resn_exch{"1HP5 MCD"}="HP51 MCD"; $resn_exch{"2HP5 MCD"}="HP52 MCD"; $resn_exch{"1HP4 MCD"}="HP41 MCD";
$resn_exch{"2HP4 MCD"}="HP42 MCD"; $resn_exch{"1HNP MCD"}="HNP1 MCD"; $resn_exch{"1HP2 MCD"}="HP21 MCD";
$resn_exch{"2HP2 MCD"}="HP22 MCD"; $resn_exch{"1HP1 MCD"}="HP11 MCD"; $resn_exch{"2HP1 MCD"}="HP12 MCD";
$resn_exch{"1HPS MCD"}="HPS1 MCD"; $resn_exch{"2HPS MCD"}="HPS2 MCD"; $resn_exch{"1HS3 MCD"}="HS31 MCD";
$resn_exch{"2HS3 MCD"}="HS32 MCD"; $resn_exch{"3HS3 MCD"}="HS33 MCD"; $resn_exch{"5HOS MCD"}="HOS5 MCD";
$resn_exch{"1H1  MCE"}=" H11 MCE"; $resn_exch{"2H1  MCE"}=" H12 MCE"; $resn_exch{"3H1  MCE"}=" H13 MCE";
$resn_exch{"1H2  MCE"}=" H21 MCE"; $resn_exch{"2H2  MCE"}=" H22 MCE"; $resn_exch{"1HN  MCG"}=" H   MCG";
$resn_exch{"2HN  MCG"}=" H2  MCG"; $resn_exch{" HO2 MCG"}=" HH2 MCG"; $resn_exch{"1HM  MCG"}=" HM1 MCG";
$resn_exch{"2HM  MCG"}=" HM2 MCG"; $resn_exch{"3HM  MCG"}=" HM3 MCG"; $resn_exch{"1H1  MCI"}=" H11 MCI";
$resn_exch{"1H3  MCI"}=" H31 MCI"; $resn_exch{"1H4  MCI"}=" H41 MCI"; $resn_exch{"1H5  MCI"}=" H51 MCI";
$resn_exch{"1H6  MCI"}=" H61 MCI"; $resn_exch{"1H12 MCI"}="H121 MCI"; $resn_exch{"2H12 MCI"}="H122 MCI";
$resn_exch{"1H13 MCI"}="H131 MCI"; $resn_exch{"2H13 MCI"}="H132 MCI"; $resn_exch{"1H16 MCI"}="H161 MCI";
$resn_exch{"1H19 MCI"}="H191 MCI"; $resn_exch{"2H19 MCI"}="H192 MCI"; $resn_exch{"1H33 MCI"}="H331 MCI";
$resn_exch{"1H34 MCI"}="H341 MCI"; $resn_exch{"1H37 MCI"}="H371 MCI"; $resn_exch{"2H37 MCI"}="H372 MCI";
$resn_exch{"1H41 MCI"}="H411 MCI"; $resn_exch{"1HN  MCL"}=" H   MCL"; $resn_exch{"2HN  MCL"}=" H2  MCL";
$resn_exch{"1HB  MCL"}=" HB2 MCL"; $resn_exch{"2HB  MCL"}=" HB3 MCL"; $resn_exch{"1HG  MCL"}=" HG2 MCL";
$resn_exch{"2HG  MCL"}=" HG3 MCL"; $resn_exch{"1HD  MCL"}=" HD2 MCL"; $resn_exch{"2HD  MCL"}=" HD3 MCL";
$resn_exch{"1HE  MCL"}=" HE2 MCL"; $resn_exch{"2HE  MCL"}=" HE3 MCL"; $resn_exch{" HNZ MCL"}=" HZ  MCL";
$resn_exch{"1H1  MCL"}=" H11 MCL"; $resn_exch{"2H1  MCL"}=" H12 MCL"; $resn_exch{"3H1  MCL"}=" H13 MCL";
$resn_exch{"1HN  MCM"}=" HN1 MCM"; $resn_exch{"2HN  MCM"}=" HN2 MCM"; $resn_exch{"1H10 MCM"}="H101 MCM";
$resn_exch{"2H10 MCM"}="H102 MCM"; $resn_exch{"3H10 MCM"}="H103 MCM"; $resn_exch{"1HN4 MCN"}="HN41 MCN";
$resn_exch{"2HN4 MCN"}="HN42 MCN"; $resn_exch{" H1* MCN"}=" H1' MCN"; $resn_exch{" H2* MCN"}=" H2' MCN";
$resn_exch{"*HO2 MCN"}="HO2' MCN"; $resn_exch{" H3* MCN"}=" H3' MCN"; $resn_exch{" H4* MCN"}=" H4' MCN";
$resn_exch{"*HO3 MCN"}="HO3' MCN"; $resn_exch{"1H5* MCN"}="H5'1 MCN"; $resn_exch{"2H5* MCN"}="H5'2 MCN";
$resn_exch{"2HOA MCN"}="HOA2 MCN"; $resn_exch{"2HOB MCN"}="HOB2 MCN"; $resn_exch{"1H10 MCN"}="H101 MCN";
$resn_exch{"2H10 MCN"}="H102 MCN"; $resn_exch{"1HN2 MCN"}="HN21 MCN"; $resn_exch{"2HN2 MCN"}="HN22 MCN";
$resn_exch{"1HO3 MCO"}="HO31 MCO"; $resn_exch{"1HC8 MCO"}="HC81 MCO"; $resn_exch{"1HC7 MCO"}="HC71 MCO";
$resn_exch{"2HC7 MCO"}="HC72 MCO"; $resn_exch{"1HC6 MCO"}="HC61 MCO"; $resn_exch{"2HC6 MCO"}="HC62 MCO";
$resn_exch{"1HC5 MCO"}="HC51 MCO"; $resn_exch{"2HC5 MCO"}="HC52 MCO"; $resn_exch{"1HC2 MCO"}="HC21 MCO";
$resn_exch{"1HC3 MCO"}="HC31 MCO"; $resn_exch{"2HC3 MCO"}="HC32 MCO"; $resn_exch{"3HC3 MCO"}="HC33 MCO";
$resn_exch{"1HC1 MCO"}="HC11 MCO"; $resn_exch{"2HC1 MCO"}="HC12 MCO"; $resn_exch{"1HS  MCO"}=" HS1 MCO";
$resn_exch{"1H2  MCP"}=" H21 MCP"; $resn_exch{"2H2  MCP"}=" H22 MCP"; $resn_exch{"1H4  MCP"}=" H41 MCP";
$resn_exch{"2H4  MCP"}=" H42 MCP"; $resn_exch{"1H5  MCP"}=" H51 MCP"; $resn_exch{"2H5  MCP"}=" H52 MCP";
$resn_exch{"1H7  MCP"}=" H71 MCP"; $resn_exch{"2H7  MCP"}=" H72 MCP"; $resn_exch{"3H7  MCP"}=" H73 MCP";
$resn_exch{"1H2  MCR"}=" H21 MCR"; $resn_exch{"2H2  MCR"}=" H22 MCR"; $resn_exch{"1HN  MCS"}=" HN1 MCS";
$resn_exch{"2HN  MCS"}=" HN2 MCS"; $resn_exch{"1HBC MCS"}="HBC1 MCS"; $resn_exch{"2HBC MCS"}="HBC2 MCS";
$resn_exch{"1HAG MCS"}="HAG1 MCS"; $resn_exch{"2HAG MCS"}="HAG2 MCS"; $resn_exch{"1H   MCT"}=" H1  MCT";
$resn_exch{"2H   MCT"}=" H2A MCT"; $resn_exch{"3H   MCT"}=" H3  MCT"; $resn_exch{"1HN4 MCY"}="HN41 MCY";
$resn_exch{"2HN4 MCY"}="HN42 MCY"; $resn_exch{" H1* MCY"}=" H1' MCY"; $resn_exch{"1H2* MCY"}=" H2' MCY";
$resn_exch{"2H2* MCY"}="H2'' MCY"; $resn_exch{" H3* MCY"}=" H3' MCY"; $resn_exch{" H4* MCY"}=" H4' MCY";
$resn_exch{"*HO3 MCY"}="HO3' MCY"; $resn_exch{"1H5* MCY"}=" H5' MCY"; $resn_exch{"2H5* MCY"}="H5'' MCY";
$resn_exch{"*HO5 MCY"}="HO5' MCY"; $resn_exch{"1H5A MCY"}="H5A1 MCY"; $resn_exch{"2H5A MCY"}="H5A2 MCY";
$resn_exch{"3H5A MCY"}="H5A3 MCY"; $resn_exch{"1H10 MD1"}="H101 MD1"; $resn_exch{"2H10 MD1"}="H102 MD1";
$resn_exch{"1H16 MD1"}="H161 MD1"; $resn_exch{"2H16 MD1"}="H162 MD1"; $resn_exch{"1H5* MD1"}="H5'1 MD1";
$resn_exch{"2H5* MD1"}="H5'2 MD1"; $resn_exch{" H4* MD1"}=" H4' MD1"; $resn_exch{" H1* MD1"}=" H1' MD1";
$resn_exch{"1HN2 MD1"}="HN21 MD1"; $resn_exch{"2HN2 MD1"}="HN22 MD1"; $resn_exch{" H2* MD1"}=" H2' MD1";
$resn_exch{" H3* MD1"}=" H3' MD1"; $resn_exch{"1HC1 MD2"}="HC11 MD2"; $resn_exch{"2HC1 MD2"}="HC1A MD2";
$resn_exch{"1HC6 MD2"}="HC61 MD2"; $resn_exch{"2HC6 MD2"}="HC62 MD2"; $resn_exch{"1HC7 MD2"}="HC71 MD2";
$resn_exch{"2HC7 MD2"}="HC72 MD2"; $resn_exch{"1H11 MD2"}="H111 MD2"; $resn_exch{"2H11 MD2"}="H112 MD2";
$resn_exch{"1HC4 MD2"}="HC41 MD2"; $resn_exch{"2HC4 MD2"}="HC42 MD2"; $resn_exch{"1HC8 MD2"}="HC81 MD2";
$resn_exch{"2HC8 MD2"}="HC82 MD2"; $resn_exch{"1HC9 MD2"}="HC91 MD2"; $resn_exch{"2HC9 MD2"}="HC92 MD2";
$resn_exch{"1H14 MD2"}="H141 MD2"; $resn_exch{"2H14 MD2"}="H142 MD2"; $resn_exch{"1H2  MDA"}=" H21 MDA";
$resn_exch{"2H2  MDA"}=" H22 MDA"; $resn_exch{"1H6  MDA"}=" H61 MDA"; $resn_exch{"2H6  MDA"}=" H62 MDA";
$resn_exch{"3H6  MDA"}=" H63 MDA"; $resn_exch{"1H3' MDA"}="H3'1 MDA"; $resn_exch{"2H3' MDA"}="H3'2 MDA";
$resn_exch{"3H3' MDA"}="H3'3 MDA"; $resn_exch{"1H1  MDC"}=" H11 MDC"; $resn_exch{"2H1  MDC"}=" H12 MDC";
$resn_exch{"1H2  MDC"}=" H21 MDC"; $resn_exch{"2H2  MDC"}=" H22 MDC"; $resn_exch{"1H19 MDC"}="H191 MDC";
$resn_exch{"2H19 MDC"}="H192 MDC"; $resn_exch{"1H20 MDC"}="H201 MDC"; $resn_exch{"2H20 MDC"}="H202 MDC";
$resn_exch{"3H20 MDC"}="H203 MDC"; $resn_exch{"1H21 MDC"}="H211 MDC"; $resn_exch{"2H21 MDC"}="H212 MDC";
$resn_exch{"1H22 MDC"}="H221 MDC"; $resn_exch{"2H22 MDC"}="H222 MDC"; $resn_exch{"3H22 MDC"}="H223 MDC";
$resn_exch{"1HA6 MDD"}="HA61 MDD"; $resn_exch{"2HA6 MDD"}="HA62 MDD"; $resn_exch{"1H5* MDE"}="H5'1 MDE";
$resn_exch{"2H5* MDE"}="H5'2 MDE"; $resn_exch{" H4* MDE"}=" H4' MDE"; $resn_exch{" H1* MDE"}=" H1' MDE";
$resn_exch{"1HN6 MDE"}="HN61 MDE"; $resn_exch{"2HN6 MDE"}="HN62 MDE"; $resn_exch{" H7* MDE"}=" H7' MDE";
$resn_exch{" H8* MDE"}=" H8' MDE"; $resn_exch{"1H12 MDE"}="H121 MDE"; $resn_exch{"2H12 MDE"}="H122 MDE";
$resn_exch{"1H13 MDE"}="H131 MDE"; $resn_exch{"2H13 MDE"}="H132 MDE"; $resn_exch{"3H13 MDE"}="H133 MDE";
$resn_exch{"1H14 MDE"}="H141 MDE"; $resn_exch{"2H14 MDE"}="H142 MDE"; $resn_exch{"3H14 MDE"}="H143 MDE";
$resn_exch{"1H7  MDE"}=" H71 MDE"; $resn_exch{"2H7  MDE"}=" H72 MDE"; $resn_exch{"1H6  MDE"}=" H61 MDE";
$resn_exch{"2H6  MDE"}=" H62 MDE"; $resn_exch{"1H3  MDE"}=" H31 MDE"; $resn_exch{"2H3  MDE"}=" H32 MDE";
$resn_exch{"1H2  MDE"}=" H21 MDE"; $resn_exch{"2H2  MDE"}=" H22 MDE"; $resn_exch{" H2* MDE"}=" H2' MDE";
$resn_exch{" H3* MDE"}=" H3' MDE"; $resn_exch{"1H4* MDE"}="H4'1 MDE"; $resn_exch{"2H4* MDE"}="H4'2 MDE";
$resn_exch{" H5* MDE"}=" H5' MDE"; $resn_exch{"1HB  MDE"}=" HB1 MDE"; $resn_exch{"2HB  MDE"}=" HB2 MDE";
$resn_exch{"HB1  MDE"}="HB11 MDE"; $resn_exch{"1H6* MDE"}="H6'1 MDE"; $resn_exch{"2H6* MDE"}="H6'2 MDE";
$resn_exch{"1H7* MDE"}="H7'1 MDE"; $resn_exch{"2H7* MDE"}="H7'2 MDE"; $resn_exch{"1H8* MDE"}="H8'1 MDE";
$resn_exch{"2H8* MDE"}="H8'2 MDE"; $resn_exch{"1H9* MDE"}="H9'1 MDE"; $resn_exch{"2H9* MDE"}="H9'2 MDE";
$resn_exch{"1H10 MDE"}="H101 MDE"; $resn_exch{"2H10 MDE"}="H102 MDE"; $resn_exch{"3H10 MDE"}="H103 MDE";
$resn_exch{"2HOB MDG"}="HOB2 MDG"; $resn_exch{"3HOB MDG"}="HOB3 MDG"; $resn_exch{"2HOA MDG"}="HOA2 MDG";
$resn_exch{"1H5* MDG"}="H5'1 MDG"; $resn_exch{"2H5* MDG"}="H5'2 MDG"; $resn_exch{" H4* MDG"}=" H4' MDG";
$resn_exch{" H3* MDG"}=" H3' MDG"; $resn_exch{"*HO3 MDG"}="HO3' MDG"; $resn_exch{" H2* MDG"}=" H2' MDG";
$resn_exch{"*HO2 MDG"}="HO2' MDG"; $resn_exch{" H1* MDG"}=" H1' MDG"; $resn_exch{"1H8  MDG"}=" H81 MDG";
$resn_exch{"2H8  MDG"}=" H82 MDG"; $resn_exch{"1HM7 MDG"}="HM71 MDG"; $resn_exch{"2HM7 MDG"}="HM72 MDG";
$resn_exch{"3HM7 MDG"}="HM73 MDG"; $resn_exch{"1HN2 MDG"}="HN21 MDG"; $resn_exch{"2HN2 MDG"}="HN22 MDG";
$resn_exch{" HN  MDH"}=" H   MDH"; $resn_exch{"1HM  MDH"}=" HM1 MDH"; $resn_exch{"2HM  MDH"}=" HM2 MDH";
$resn_exch{"3HM  MDH"}=" HM3 MDH"; $resn_exch{"1HG  MDH"}=" HG1 MDH"; $resn_exch{"2HG  MDH"}=" HG2 MDH";
$resn_exch{"3HG  MDH"}=" HG3 MDH"; $resn_exch{"1H1  MDL"}=" H11 MDL"; $resn_exch{"2H1  MDL"}=" H12 MDL";
$resn_exch{"3H1  MDL"}=" H13 MDL"; $resn_exch{"1H11 MDL"}="H111 MDL"; $resn_exch{"2H11 MDL"}="H112 MDL";
$resn_exch{"1H12 MDL"}="H121 MDL"; $resn_exch{"2H12 MDL"}="H122 MDL"; $resn_exch{"1H15 MDL"}="H151 MDL";
$resn_exch{"2H15 MDL"}="H152 MDL"; $resn_exch{"1H26 MDL"}="H261 MDL"; $resn_exch{"2H26 MDL"}="H262 MDL";
$resn_exch{"3HN3 MDL"}="HN33 MDL"; $resn_exch{"1H55 MDL"}="H551 MDL"; $resn_exch{"2H55 MDL"}="H552 MDL";
$resn_exch{"1H6  MDM"}=" H61 MDM"; $resn_exch{"2H6  MDM"}=" H62 MDM"; $resn_exch{"1H6' MDM"}="H6'1 MDM";
$resn_exch{"2H6' MDM"}="H6'2 MDM"; $resn_exch{"1H7' MDM"}="H7'1 MDM"; $resn_exch{"2H7' MDM"}="H7'2 MDM";
$resn_exch{"3H7' MDM"}="H7'3 MDM"; $resn_exch{"'HO2 MDM"}="HO2' MDM"; $resn_exch{"'HO4 MDM"}="HO4' MDM";
$resn_exch{"'HO6 MDM"}="HO6' MDM"; $resn_exch{"3HO  MDN"}=" HO3 MDN"; $resn_exch{"2HO  MDN"}=" HO2 MDN";
$resn_exch{"1H4  MDN"}=" H41 MDN"; $resn_exch{"2H4  MDN"}=" H42 MDN"; $resn_exch{"7HO  MDN"}=" HO7 MDN";
$resn_exch{"8HO  MDN"}=" HO8 MDN"; $resn_exch{"1HN  MDO"}=" H   MDO"; $resn_exch{"2HN  MDO"}=" H2  MDO";
$resn_exch{"1HC  MDO"}=" HB1 MDO"; $resn_exch{"2HC  MDO"}=" HB2 MDO"; $resn_exch{"3HC  MDO"}=" HB3 MDO";
$resn_exch{"1HB2 MDO"}="HB21 MDO"; $resn_exch{"2HB2 MDO"}="HB22 MDO"; $resn_exch{"3HB2 MDO"}="HB23 MDO";
$resn_exch{"1HA3 MDO"}="HA31 MDO"; $resn_exch{"2HA3 MDO"}="HA32 MDO"; $resn_exch{"1H6  MDP"}=" H61 MDP";
$resn_exch{"2H6  MDP"}=" H62 MDP"; $resn_exch{"1H9  MDP"}=" H91 MDP"; $resn_exch{"2H9  MDP"}=" H92 MDP";
$resn_exch{"3H9  MDP"}=" H93 MDP"; $resn_exch{"1H11 MDP"}="H111 MDP"; $resn_exch{"2H11 MDP"}="H112 MDP";
$resn_exch{"3H11 MDP"}="H113 MDP"; $resn_exch{"1H6' MDR"}="H6'1 MDR"; $resn_exch{"2H6' MDR"}="H6'2 MDR";
$resn_exch{"3H6' MDR"}="H6'3 MDR"; $resn_exch{"1H2' MDR"}="H2'1 MDR"; $resn_exch{"2H2' MDR"}="H2'2 MDR";
$resn_exch{"1H5' MDR"}="H5'1 MDR"; $resn_exch{"2H5' MDR"}="H5'2 MDR"; $resn_exch{"8H2  MDW"}=" H28 MDW";
$resn_exch{"7H2  MDW"}=" H27 MDW"; $resn_exch{"5H3  MDW"}=" H35 MDW"; $resn_exch{"6H3  MDW"}=" H36 MDW";
$resn_exch{"7H3  MDW"}=" H37 MDW"; $resn_exch{"9H2  MDW"}=" H29 MDW"; $resn_exch{"0H3  MDW"}=" H30 MDW";
$resn_exch{"8H3  MDW"}=" H38 MDW"; $resn_exch{"9H3  MDW"}=" H39 MDW"; $resn_exch{"1H3  MDW"}=" H31 MDW";
$resn_exch{"2H3  MDW"}=" H32 MDW"; $resn_exch{"3H3  MDW"}=" H33 MDW"; $resn_exch{"4H3  MDW"}=" H34 MDW";
$resn_exch{"8H4  MDW"}=" H48 MDW"; $resn_exch{"2H4  MDW"}=" H42 MDW"; $resn_exch{"5H4  MDW"}=" H45 MDW";
$resn_exch{"6H4  MDW"}=" H46 MDW"; $resn_exch{"7H4  MDW"}=" H47 MDW"; $resn_exch{"3H4  MDW"}=" H43 MDW";
$resn_exch{"4H4  MDW"}=" H44 MDW"; $resn_exch{"1H10 MDX"}="H101 MDX"; $resn_exch{"2H10 MDX"}="H102 MDX";
$resn_exch{"1H9  MDX"}=" H91 MDX"; $resn_exch{"2H9  MDX"}=" H92 MDX"; $resn_exch{"1H8  MDX"}=" H81 MDX";
$resn_exch{"2H8  MDX"}=" H82 MDX"; $resn_exch{"1H7  MDX"}=" H71 MDX"; $resn_exch{"2H7  MDX"}=" H72 MDX";
$resn_exch{"1H6  MDX"}=" H61 MDX"; $resn_exch{"2H6  MDX"}=" H62 MDX"; $resn_exch{"1H5  MDX"}=" H51 MDX";
$resn_exch{"2H5  MDX"}=" H52 MDX"; $resn_exch{"1H4  MDX"}=" H41 MDX"; $resn_exch{"2H4  MDX"}=" H42 MDX";
$resn_exch{"1H3  MDX"}=" H31 MDX"; $resn_exch{"2H3  MDX"}=" H32 MDX"; $resn_exch{"1H2  MDX"}=" H21 MDX";
$resn_exch{"2H2  MDX"}=" H22 MDX"; $resn_exch{"1H1  MDX"}=" H11 MDX"; $resn_exch{"2H1  MDX"}=" H12 MDX";
$resn_exch{"1H25 MDZ"}="H251 MDZ"; $resn_exch{"2H25 MDZ"}="H252 MDZ"; $resn_exch{"3H25 MDZ"}="H253 MDZ";
$resn_exch{"1H7C MDZ"}="H7C1 MDZ"; $resn_exch{"2H7C MDZ"}="H7C2 MDZ"; $resn_exch{"1H4C MDZ"}="H4C1 MDZ";
$resn_exch{"2H4C MDZ"}="H4C2 MDZ"; $resn_exch{"1H3C MDZ"}="H3C1 MDZ"; $resn_exch{"2H3C MDZ"}="H3C2 MDZ";
$resn_exch{"1H14 MDZ"}="H141 MDZ"; $resn_exch{"2H14 MDZ"}="H142 MDZ"; $resn_exch{"1H37 MDZ"}="H371 MDZ";
$resn_exch{"2H37 MDZ"}="H372 MDZ"; $resn_exch{"3H37 MDZ"}="H373 MDZ"; $resn_exch{"1H33 MDZ"}="H331 MDZ";
$resn_exch{"2H33 MDZ"}="H332 MDZ"; $resn_exch{"3H33 MDZ"}="H333 MDZ"; $resn_exch{"1H9  ME1"}=" H91 ME1";
$resn_exch{"2H9  ME1"}=" H92 ME1"; $resn_exch{"1H10 ME1"}="H101 ME1"; $resn_exch{"2H10 ME1"}="H102 ME1";
$resn_exch{"1H7  ME2"}=" H71 ME2"; $resn_exch{"2H7  ME2"}=" H72 ME2"; $resn_exch{"3H7  ME2"}=" H73 ME2";
$resn_exch{"1H6  ME2"}=" H61 ME2"; $resn_exch{"2H6  ME2"}=" H62 ME2"; $resn_exch{"1H5  ME2"}=" H51 ME2";
$resn_exch{"2H5  ME2"}=" H52 ME2"; $resn_exch{"1H4  ME2"}=" H41 ME2"; $resn_exch{"2H4  ME2"}=" H42 ME2";
$resn_exch{"1H3  ME2"}=" H31 ME2"; $resn_exch{"2H3  ME2"}=" H32 ME2"; $resn_exch{"1H2  ME2"}=" H21 ME2";
$resn_exch{"2H2  ME2"}=" H22 ME2"; $resn_exch{"1H1  ME2"}=" H11 ME2"; $resn_exch{"2H1  ME2"}=" H12 ME2";
$resn_exch{"3H1  ME2"}=" H13 ME2"; $resn_exch{"1H25 ME3"}="H251 ME3"; $resn_exch{"2H25 ME3"}="H252 ME3";
$resn_exch{"1H26 ME3"}="H261 ME3"; $resn_exch{"2H26 ME3"}="H262 ME3"; $resn_exch{"3H26 ME3"}="H263 ME3";
$resn_exch{"1H8C ME3"}="H8C1 ME3"; $resn_exch{"1H9C ME3"}="H9C1 ME3"; $resn_exch{"1H5C ME3"}="H5C1 ME3";
$resn_exch{"1H4C ME3"}="H4C1 ME3"; $resn_exch{"1H2  ME4"}=" H21 ME4"; $resn_exch{"2H2  ME4"}=" H22 ME4";
$resn_exch{"1H3  ME4"}="H31A ME4"; $resn_exch{"2H3  ME4"}="H32A ME4"; $resn_exch{"1H4  ME4"}=" H41 ME4";
$resn_exch{"2H4  ME4"}=" H42 ME4"; $resn_exch{"1H5  ME4"}=" H51 ME4"; $resn_exch{"2H5  ME4"}=" H52 ME4";
$resn_exch{"1H8  ME5"}=" H81 ME5"; $resn_exch{"2H8  ME5"}=" H82 ME5"; $resn_exch{"1H10 ME5"}="H101 ME5";
$resn_exch{"2H10 ME5"}="H102 ME5"; $resn_exch{"1H12 ME5"}="H121 ME5"; $resn_exch{"2H12 ME5"}="H122 ME5";
$resn_exch{"1H1C MEA"}="H1C1 MEA"; $resn_exch{"2H1C MEA"}="H1C2 MEA"; $resn_exch{"3H1C MEA"}="H1C3 MEA";
$resn_exch{"1H'C MEA"}="H'C1 MEA"; $resn_exch{"2H'C MEA"}="H'C2 MEA"; $resn_exch{"1H2  MEB"}=" H21 MEB";
$resn_exch{"1H3  MEB"}=" H31 MEB"; $resn_exch{"2H3  MEB"}=" H32 MEB"; $resn_exch{"1H1' MEB"}="H1'1 MEB";
$resn_exch{"2H1' MEB"}="H1'2 MEB"; $resn_exch{"3H1' MEB"}="H1'3 MEB"; $resn_exch{"3H2  MEC"}=" H23 MEC";
$resn_exch{"2H2  MEC"}=" H22 MEC"; $resn_exch{"1H2  MEC"}=" H21 MEC"; $resn_exch{"2H4  MEC"}=" H42 MEC";
$resn_exch{"1H4  MEC"}=" H41 MEC"; $resn_exch{"3H5  MEC"}=" H53 MEC"; $resn_exch{"2H5  MEC"}=" H52 MEC";
$resn_exch{"1H5  MEC"}=" H51 MEC"; $resn_exch{" HN2 MED"}=" H2  MED"; $resn_exch{"1HB  MED"}=" HB2 MED";
$resn_exch{"2HB  MED"}=" HB3 MED"; $resn_exch{"1HG  MED"}=" HG2 MED"; $resn_exch{"2HG  MED"}=" HG3 MED";
$resn_exch{"1HE  MED"}=" HE1 MED"; $resn_exch{"2HE  MED"}=" HE2 MED"; $resn_exch{"3HE  MED"}=" HE3 MED";
$resn_exch{"1H   MEE"}=" H1  MEE"; $resn_exch{"2H   MEE"}=" H2  MEE"; $resn_exch{"3H   MEE"}=" H3  MEE";
$resn_exch{" HN2 MEG"}=" H2  MEG"; $resn_exch{"1HG  MEG"}=" HG2 MEG"; $resn_exch{"2HG  MEG"}=" HG3 MEG";
$resn_exch{"1H1  MEG"}=" H11 MEG"; $resn_exch{"2H1  MEG"}=" H12 MEG"; $resn_exch{"3H1  MEG"}=" H13 MEG";
$resn_exch{" HO1 MEG"}=" HE2 MEG"; $resn_exch{"1H1  MEI"}="H11A MEI"; $resn_exch{"2H1  MEI"}="H12A MEI";
$resn_exch{"3H1  MEI"}=" H13 MEI"; $resn_exch{"1H3  MEI"}=" H31 MEI"; $resn_exch{"2H3  MEI"}=" H32 MEI";
$resn_exch{"3H3  MEI"}=" H33 MEI"; $resn_exch{"1H4  MEI"}=" H41 MEI"; $resn_exch{"2H4  MEI"}=" H42 MEI";
$resn_exch{"3H4  MEI"}=" H43 MEI"; $resn_exch{"1H5  MEI"}=" H51 MEI"; $resn_exch{"2H5  MEI"}=" H52 MEI";
$resn_exch{"1H6  MEI"}=" H61 MEI"; $resn_exch{"2H6  MEI"}=" H62 MEI"; $resn_exch{"1H7  MEI"}=" H71 MEI";
$resn_exch{"2H7  MEI"}=" H72 MEI"; $resn_exch{"1H9  MEI"}=" H91 MEI"; $resn_exch{"2H9  MEI"}=" H92 MEI";
$resn_exch{"3H9  MEI"}=" H93 MEI"; $resn_exch{"1H10 MEI"}="H101 MEI"; $resn_exch{"2H10 MEI"}="H102 MEI";
$resn_exch{"1H14 MEI"}="H141 MEI"; $resn_exch{"2H14 MEI"}="H142 MEI"; $resn_exch{"3H14 MEI"}="H143 MEI";
$resn_exch{"1H1  MEL"}=" H11 MEL"; $resn_exch{"1H2  MEL"}=" H21 MEL"; $resn_exch{"2H2  MEL"}=" H22 MEL";
$resn_exch{"1H3  MEL"}=" H31 MEL"; $resn_exch{"2H3  MEL"}=" H32 MEL"; $resn_exch{"1H4  MEL"}=" H41 MEL";
$resn_exch{"2H4  MEL"}=" H42 MEL"; $resn_exch{"1H5  MEL"}=" H51 MEL"; $resn_exch{"2H5  MEL"}=" H52 MEL";
$resn_exch{"1H6  MEL"}=" H61 MEL"; $resn_exch{"2H6  MEL"}=" H62 MEL"; $resn_exch{"1H7  MEL"}=" H71 MEL";
$resn_exch{"1H8  MEL"}=" H81 MEL"; $resn_exch{"1H12 MEL"}="H121 MEL"; $resn_exch{"1H15 MEL"}="H151 MEL";
$resn_exch{"1H16 MEL"}="H161 MEL"; $resn_exch{"2H16 MEL"}="H162 MEL"; $resn_exch{"1H18 MEL"}="H181 MEL";
$resn_exch{"1H19 MEL"}="H191 MEL"; $resn_exch{"1H21 MEL"}="H211 MEL"; $resn_exch{"1H22 MEL"}="H221 MEL";
$resn_exch{"1H24 MEL"}="H241 MEL"; $resn_exch{"1H25 MEL"}="H251 MEL"; $resn_exch{"2H25 MEL"}="H252 MEL";
$resn_exch{"1H26 MEL"}="H261 MEL"; $resn_exch{"2H26 MEL"}="H262 MEL"; $resn_exch{"1H27 MEL"}="H271 MEL";
$resn_exch{"2H27 MEL"}="H272 MEL"; $resn_exch{"1H28 MEL"}="H281 MEL"; $resn_exch{"2H28 MEL"}="H282 MEL";
$resn_exch{"1H30 MEL"}="H301 MEL"; $resn_exch{" HN2 MEN"}=" H2  MEN"; $resn_exch{"1HB  MEN"}=" HB2 MEN";
$resn_exch{"2HB  MEN"}=" HB3 MEN"; $resn_exch{"1HE2 MEN"}="HE21 MEN"; $resn_exch{"2HE2 MEN"}="HE22 MEN";
$resn_exch{"3HE2 MEN"}="HE23 MEN"; $resn_exch{"2HOP MEP"}="HOP2 MEP"; $resn_exch{"3HOP MEP"}="HOP3 MEP";
$resn_exch{"1H5* MEP"}=" H5' MEP"; $resn_exch{"2H5* MEP"}="H5'' MEP"; $resn_exch{" H4* MEP"}=" H4' MEP";
$resn_exch{" H3* MEP"}=" H3' MEP"; $resn_exch{"H3T  MEP"}="HO3' MEP"; $resn_exch{"2H3T MEP"}="H3T2 MEP";
$resn_exch{"3H3T MEP"}="H3T3 MEP"; $resn_exch{" H2* MEP"}=" H2' MEP"; $resn_exch{" H1* MEP"}=" H1' MEP";
$resn_exch{"1H5M MEP"}=" H71 MEP"; $resn_exch{"2H5M MEP"}=" H72 MEP"; $resn_exch{"3H5M MEP"}=" H73 MEP";
$resn_exch{"1HA* MEP"}="HA'1 MEP"; $resn_exch{"2HA* MEP"}="HA'2 MEP"; $resn_exch{"3HA* MEP"}="HA'3 MEP";
$resn_exch{" HN2 MEQ"}=" H2  MEQ"; $resn_exch{"1HB  MEQ"}=" HB2 MEQ"; $resn_exch{"2HB  MEQ"}=" HB3 MEQ";
$resn_exch{"1HG  MEQ"}=" HG2 MEQ"; $resn_exch{"2HG  MEQ"}=" HG3 MEQ"; $resn_exch{"1HE2 MEQ"}="HE21 MEQ";
$resn_exch{"1HE  MEQ"}=" HE1 MEQ"; $resn_exch{"2HE  MEQ"}=" HE2 MEQ"; $resn_exch{"3HE  MEQ"}=" HE3 MEQ";
$resn_exch{"2HC  MER"}=" HC2 MER"; $resn_exch{"6HC  MER"}=" HC6 MER"; $resn_exch{"5HC  MER"}=" HC5 MER";
$resn_exch{"6HC1 MER"}="HC16 MER"; $resn_exch{"6HO2 MER"}="HO26 MER"; $resn_exch{"1H62 MER"}="H621 MER";
$resn_exch{"2H62 MER"}="H622 MER"; $resn_exch{"3H62 MER"}="H623 MER"; $resn_exch{"3HO2 MER"}="HO23 MER";
$resn_exch{"1HCA MER"}="HCA1 MER"; $resn_exch{"1H4A MER"}="H4A1 MER"; $resn_exch{"2H4A MER"}="H4A2 MER";
$resn_exch{"5HCA MER"}="HCA5 MER"; $resn_exch{"1HC  MER"}=" HC1 MER"; $resn_exch{"3HNA MER"}="HNA3 MER";
$resn_exch{"1H11 MER"}="H111 MER"; $resn_exch{"2H11 MER"}="H112 MER"; $resn_exch{"3H11 MER"}="H113 MER";
$resn_exch{"1H2A MER"}="H2A1 MER"; $resn_exch{"2H2A MER"}="H2A2 MER"; $resn_exch{"1H8A MER"}="H8A1 MER";
$resn_exch{"2H8A MER"}="H8A2 MER"; $resn_exch{"3H8A MER"}="H8A3 MER"; $resn_exch{"1H9A MER"}="H9A1 MER";
$resn_exch{"2H9A MER"}="H9A2 MER"; $resn_exch{"3H9A MER"}="H9A3 MER"; $resn_exch{"1H2  MES"}=" H21 MES";
$resn_exch{"2H2  MES"}=" H22 MES"; $resn_exch{"1H3  MES"}=" H31 MES"; $resn_exch{"2H3  MES"}=" H32 MES";
$resn_exch{"1H5  MES"}=" H51 MES"; $resn_exch{"2H5  MES"}=" H52 MES"; $resn_exch{"1H6  MES"}=" H61 MES";
$resn_exch{"2H6  MES"}=" H62 MES"; $resn_exch{"1H7  MES"}=" H71 MES"; $resn_exch{"2H7  MES"}=" H72 MES";
$resn_exch{"1H8  MES"}=" H81 MES"; $resn_exch{"2H8  MES"}=" H82 MES"; $resn_exch{" HN2 MET"}=" H2  MET";
$resn_exch{"1HB  MET"}=" HB2 MET"; $resn_exch{"2HB  MET"}=" HB3 MET"; $resn_exch{"1HG  MET"}=" HG2 MET";
$resn_exch{"2HG  MET"}=" HG3 MET"; $resn_exch{"1HE  MET"}=" HE1 MET"; $resn_exch{"2HE  MET"}=" HE2 MET";
$resn_exch{"3HE  MET"}=" HE3 MET"; $resn_exch{"1HB  MEU"}=" HB1 MEU"; $resn_exch{"2HB  MEU"}=" HB2 MEU";
$resn_exch{"3HB  MEU"}=" HB3 MEU"; $resn_exch{"1HN  MEU"}=" H   MEU"; $resn_exch{"2HN  MEU"}=" H2  MEU";
$resn_exch{"1HA  MEU"}=" HA1 MEU"; $resn_exch{"2HA  MEU"}=" HA2 MEU"; $resn_exch{"1H8  MEV"}=" H81 MEV";
$resn_exch{"2H8  MEV"}=" H82 MEV"; $resn_exch{"1H2  MEV"}=" H21 MEV"; $resn_exch{"2H2  MEV"}=" H22 MEV";
$resn_exch{"1H4  MEV"}=" H41 MEV"; $resn_exch{"2H4  MEV"}=" H42 MEV"; $resn_exch{"1H6  MEV"}=" H61 MEV";
$resn_exch{"2H6  MEV"}=" H62 MEV"; $resn_exch{"3H6  MEV"}=" H63 MEV"; $resn_exch{"1H4  MEZ"}=" H41 MEZ";
$resn_exch{"2H4  MEZ"}=" H42 MEZ"; $resn_exch{"3H4  MEZ"}=" H43 MEZ"; $resn_exch{"1H2  MF2"}=" H21 MF2";
$resn_exch{"2H2  MF2"}=" H22 MF2"; $resn_exch{"1H7  MF2"}=" H71 MF2"; $resn_exch{"2H7  MF2"}=" H72 MF2";
$resn_exch{"3H7  MF2"}=" H73 MF2"; $resn_exch{"1H8  MF2"}=" H81 MF2"; $resn_exch{"2H8  MF2"}=" H82 MF2";
$resn_exch{"3H8  MF2"}=" H83 MF2"; $resn_exch{"1H9  MF2"}=" H91 MF2"; $resn_exch{"2H9  MF2"}=" H92 MF2";
$resn_exch{"1H10 MF2"}="H101 MF2"; $resn_exch{"2H10 MF2"}="H102 MF2"; $resn_exch{"1H11 MF2"}="H111 MF2";
$resn_exch{"2H11 MF2"}="H112 MF2"; $resn_exch{"1H12 MF2"}="H121 MF2"; $resn_exch{"2H12 MF2"}="H122 MF2";
$resn_exch{"1H13 MF2"}="H131 MF2"; $resn_exch{"2H13 MF2"}="H132 MF2"; $resn_exch{"1H14 MF2"}="H141 MF2";
$resn_exch{"2H14 MF2"}="H142 MF2"; $resn_exch{"1H15 MF2"}="H151 MF2"; $resn_exch{"2H15 MF2"}="H152 MF2";
$resn_exch{"1H16 MF2"}="H161 MF2"; $resn_exch{"2H16 MF2"}="H162 MF2"; $resn_exch{"1H6  MF2"}=" H61 MF2";
$resn_exch{"2H6  MF2"}=" H62 MF2"; $resn_exch{" HN2 MF3"}=" H2  MF3"; $resn_exch{"1HB  MF3"}=" HB2 MF3";
$resn_exch{"2HB  MF3"}=" HB3 MF3"; $resn_exch{"1HG  MF3"}=" HG2 MF3"; $resn_exch{"2HG  MF3"}=" HG3 MF3";
$resn_exch{"1HM  MFA"}=" HM1 MFA"; $resn_exch{"2HM  MFA"}=" HM2 MFA"; $resn_exch{"3HM  MFA"}=" HM3 MFA";
$resn_exch{"1H6  MFA"}=" H61 MFA"; $resn_exch{"2H6  MFA"}=" H62 MFA"; $resn_exch{"3H6  MFA"}=" H63 MFA";
$resn_exch{"1H6  MFB"}=" H61 MFB"; $resn_exch{"2H6  MFB"}=" H62 MFB"; $resn_exch{"3H6  MFB"}=" H63 MFB";
$resn_exch{"1HM  MFB"}=" HM1 MFB"; $resn_exch{"2HM  MFB"}=" HM2 MFB"; $resn_exch{"3HM  MFB"}=" HM3 MFB";
$resn_exch{"1HG1 MFC"}="HG11 MFC"; $resn_exch{"2HG1 MFC"}="HG12 MFC"; $resn_exch{"3HG1 MFC"}="HG13 MFC";
$resn_exch{"1HA3 MFC"}="HA31 MFC"; $resn_exch{"2HA3 MFC"}="HA32 MFC"; $resn_exch{"1HN3 MFD"}="HN31 MFD";
$resn_exch{"2HN3 MFD"}="HN32 MFD"; $resn_exch{"1HM2 MFD"}="HM21 MFD"; $resn_exch{"2HM2 MFD"}="HM22 MFD";
$resn_exch{"3HM2 MFD"}="HM23 MFD"; $resn_exch{"1H'  MFD"}=" H'1 MFD"; $resn_exch{"2H'  MFD"}=" H'2 MFD";
$resn_exch{"1HM9 MFD"}="HM91 MFD"; $resn_exch{"2HM9 MFD"}="HM92 MFD"; $resn_exch{"3HM9 MFD"}="HM93 MFD";
$resn_exch{"1HM8 MFD"}="HM81 MFD"; $resn_exch{"2HM8 MFD"}="HM82 MFD"; $resn_exch{"3HM8 MFD"}="HM83 MFD";
$resn_exch{"1HM6 MFD"}="HM61 MFD"; $resn_exch{"2HM6 MFD"}="HM62 MFD"; $resn_exch{"3HM6 MFD"}="HM63 MFD";
$resn_exch{"1H15 MFM"}="H151 MFM"; $resn_exch{"2H15 MFM"}="H152 MFM"; $resn_exch{"1H16 MFM"}="H161 MFM";
$resn_exch{"2H16 MFM"}="H162 MFM"; $resn_exch{"1H1  MFN"}="H11A MFN"; $resn_exch{"2H1  MFN"}="H12A MFN";
$resn_exch{"1H22 MFN"}="H221 MFN"; $resn_exch{"2H22 MFN"}="H222 MFN"; $resn_exch{"1H14 MFN"}="H141 MFN";
$resn_exch{"2H14 MFN"}="H142 MFN"; $resn_exch{"1H26 MFN"}="H261 MFN"; $resn_exch{"2H26 MFN"}="H262 MFN";
$resn_exch{"1H27 MFN"}="H271 MFN"; $resn_exch{"2H27 MFN"}="H272 MFN"; $resn_exch{"2H28 MFN"}="H282 MFN";
$resn_exch{"1H17 MFN"}="H171 MFN"; $resn_exch{"2H17 MFN"}="H172 MFN"; $resn_exch{"1H18 MFN"}="H181 MFN";
$resn_exch{"2H18 MFN"}="H182 MFN"; $resn_exch{"1H15 MFN"}="H151 MFN"; $resn_exch{"2H15 MFN"}="H152 MFN";
$resn_exch{"1H12 MFN"}="H121 MFN"; $resn_exch{"2H12 MFN"}="H122 MFN"; $resn_exch{"1H13 MFN"}="H131 MFN";
$resn_exch{"2H13 MFN"}="H132 MFN"; $resn_exch{"1HN1 MFN"}="HN11 MFN"; $resn_exch{"1H31 MFN"}="H311 MFN";
$resn_exch{"2H31 MFN"}="H312 MFN"; $resn_exch{"1H32 MFN"}="H321 MFN"; $resn_exch{"2H32 MFN"}="H322 MFN";
$resn_exch{"1H16 MFP"}="H161 MFP"; $resn_exch{"2H16 MFP"}="H162 MFP"; $resn_exch{"1H15 MFP"}="H151 MFP";
$resn_exch{"2H15 MFP"}="H152 MFP"; $resn_exch{"3H15 MFP"}="H153 MFP"; $resn_exch{"1H27 MFP"}="H271 MFP";
$resn_exch{"2H27 MFP"}="H272 MFP"; $resn_exch{"1H29 MFP"}="H291 MFP"; $resn_exch{"2H29 MFP"}="H292 MFP";
$resn_exch{"1H31 MFP"}="H311 MFP"; $resn_exch{"2H31 MFP"}="H312 MFP"; $resn_exch{"3H31 MFP"}="H313 MFP";
$resn_exch{"1H32 MFP"}="H321 MFP"; $resn_exch{"2H32 MFP"}="H322 MFP"; $resn_exch{"3H32 MFP"}="H323 MFP";
$resn_exch{"1H16 MFQ"}="H161 MFQ"; $resn_exch{"2H16 MFQ"}="H162 MFQ"; $resn_exch{"1H15 MFQ"}="H151 MFQ";
$resn_exch{"2H15 MFQ"}="H152 MFQ"; $resn_exch{"3H15 MFQ"}="H153 MFQ"; $resn_exch{"1H27 MFQ"}="H271 MFQ";
$resn_exch{"2H27 MFQ"}="H272 MFQ"; $resn_exch{"1H29 MFQ"}="H291 MFQ"; $resn_exch{"2H29 MFQ"}="H292 MFQ";
$resn_exch{"1H31 MFQ"}="H311 MFQ"; $resn_exch{"2H31 MFQ"}="H312 MFQ"; $resn_exch{"3H31 MFQ"}="H313 MFQ";
$resn_exch{"1H32 MFQ"}="H321 MFQ"; $resn_exch{"2H32 MFQ"}="H322 MFQ"; $resn_exch{"3H32 MFQ"}="H323 MFQ";
$resn_exch{"1H6  MFU"}=" H61 MFU"; $resn_exch{"2H6  MFU"}=" H62 MFU"; $resn_exch{"3H6  MFU"}=" H63 MFU";
$resn_exch{"1HM  MFU"}=" HM1 MFU"; $resn_exch{"2HM  MFU"}=" HM2 MFU"; $resn_exch{"3HM  MFU"}=" HM3 MFU";
$resn_exch{"3HOP MG1"}="HOP3 MG1"; $resn_exch{"2HOP MG1"}="HOP2 MG1"; $resn_exch{"1H5* MG1"}=" H5' MG1";
$resn_exch{"2H5* MG1"}="H5'' MG1"; $resn_exch{" H4* MG1"}=" H4' MG1"; $resn_exch{" H3* MG1"}=" H3' MG1";
$resn_exch{"H3T  MG1"}="HO3' MG1"; $resn_exch{"1H2* MG1"}=" H2' MG1"; $resn_exch{"2H2* MG1"}="H2'' MG1";
$resn_exch{" H1* MG1"}=" H1' MG1"; $resn_exch{"1HM1 MG1"}="HM11 MG1"; $resn_exch{"2HM1 MG1"}="HM12 MG1";
$resn_exch{"3HM1 MG1"}="HM13 MG1"; $resn_exch{"1H2  MG1"}=" H21 MG1"; $resn_exch{"2H2  MG1"}=" H22 MG1";
$resn_exch{"1HN2 MG7"}="HN21 MG7"; $resn_exch{"2HN2 MG7"}="HN22 MG7"; $resn_exch{"'HO5 MG7"}="HO5' MG7";
$resn_exch{"1H5' MG7"}="H5'1 MG7"; $resn_exch{"2H5' MG7"}="H5'2 MG7"; $resn_exch{"1H7  MG7"}=" H71 MG7";
$resn_exch{"2H7  MG7"}=" H72 MG7"; $resn_exch{"3H7  MG7"}=" H73 MG7"; $resn_exch{"'HO2 MG7"}="HO2' MG7";
$resn_exch{"'HO3 MG7"}="HO3' MG7"; $resn_exch{"1H1  MG8"}="H11A MG8"; $resn_exch{"2H1  MG8"}="H12A MG8";
$resn_exch{"3H1  MG8"}="H13A MG8"; $resn_exch{"1H2  MG8"}=" H21 MG8"; $resn_exch{"2H2  MG8"}=" H22 MG8";
$resn_exch{"1H3  MG8"}=" H31 MG8"; $resn_exch{"2H3  MG8"}=" H32 MG8"; $resn_exch{"1H4  MG8"}=" H41 MG8";
$resn_exch{"2H4  MG8"}=" H42 MG8"; $resn_exch{"1H5  MG8"}=" H51 MG8"; $resn_exch{"2H5  MG8"}=" H52 MG8";
$resn_exch{"1H6  MG8"}=" H61 MG8"; $resn_exch{"2H6  MG8"}=" H62 MG8"; $resn_exch{"1H7  MG8"}=" H71 MG8";
$resn_exch{"2H7  MG8"}=" H72 MG8"; $resn_exch{"1H9  MG8"}=" H91 MG8"; $resn_exch{"2H9  MG8"}=" H92 MG8";
$resn_exch{"1H14 MG8"}="H141 MG8"; $resn_exch{"2H14 MG8"}="H142 MG8"; $resn_exch{"1H15 MG8"}="H151 MG8";
$resn_exch{"2H15 MG8"}="H152 MG8"; $resn_exch{"3H15 MG8"}="H153 MG8"; $resn_exch{"1H6  MGA"}=" H61 MGA";
$resn_exch{"2H6  MGA"}=" H62 MGA"; $resn_exch{"1H7  MGA"}=" H71 MGA"; $resn_exch{"2H7  MGA"}=" H72 MGA";
$resn_exch{"3H7  MGA"}=" H73 MGA"; $resn_exch{"1H1  MGB"}=" H11 MGB"; $resn_exch{"2H1  MGB"}=" H12 MGB";
$resn_exch{"1H2  MGB"}=" H21 MGB"; $resn_exch{"1H4  MGB"}=" H41 MGB"; $resn_exch{"1H6  MGB"}=" H61 MGB";
$resn_exch{"1H8  MGB"}=" H81 MGB"; $resn_exch{"2H8  MGB"}=" H82 MGB"; $resn_exch{"3H8  MGB"}=" H83 MGB";
$resn_exch{"1H10 MGB"}="H101 MGB"; $resn_exch{"1H12 MGB"}="H121 MGB"; $resn_exch{"1H13 MGB"}="H131 MGB";
$resn_exch{"2H13 MGB"}="H132 MGB"; $resn_exch{"1H6  MGC"}=" H61 MGC"; $resn_exch{"2H6  MGC"}=" H62 MGC";
$resn_exch{"1HM  MGC"}=" HM1 MGC"; $resn_exch{"2HM  MGC"}=" HM2 MGC"; $resn_exch{"3HM  MGC"}=" HM3 MGC";
$resn_exch{"1H8  MGC"}=" H81 MGC"; $resn_exch{"2H8  MGC"}=" H82 MGC"; $resn_exch{"3H8  MGC"}=" H83 MGC";
$resn_exch{"2HOB MGD"}="HOB2 MGD"; $resn_exch{"2HOA MGD"}="HOA2 MGD"; $resn_exch{"1H5' MGD"}="H5'1 MGD";
$resn_exch{"2H5' MGD"}="H5'2 MGD"; $resn_exch{"'HO3 MGD"}="HO3' MGD"; $resn_exch{"'HO2 MGD"}="HO2' MGD";
$resn_exch{"1HN2 MGD"}="HN21 MGD"; $resn_exch{"2HN2 MGD"}="HN22 MGD"; $resn_exch{"1H10 MGD"}="H101 MGD";
$resn_exch{"2H10 MGD"}="H102 MGD"; $resn_exch{"1H19 MGD"}="H191 MGD"; $resn_exch{"2H19 MGD"}="H192 MGD";
$resn_exch{"1H2A MGE"}="H2A1 MGE"; $resn_exch{"2H2A MGE"}="H2A2 MGE"; $resn_exch{"1H3A MGE"}="H3A1 MGE";
$resn_exch{"2H3A MGE"}="H3A2 MGE"; $resn_exch{"1H4A MGE"}="H4A1 MGE"; $resn_exch{"2H4A MGE"}="H4A2 MGE";
$resn_exch{"1H5A MGE"}="H5A1 MGE"; $resn_exch{"2H5A MGE"}="H5A2 MGE"; $resn_exch{"1H6A MGE"}="H6A1 MGE";
$resn_exch{"2H6A MGE"}="H6A2 MGE"; $resn_exch{"1H7A MGE"}="H7A1 MGE"; $resn_exch{"2H7A MGE"}="H7A2 MGE";
$resn_exch{"1H8A MGE"}="H8A1 MGE"; $resn_exch{"2H8A MGE"}="H8A2 MGE"; $resn_exch{"1H9A MGE"}="H9A1 MGE";
$resn_exch{"2H9A MGE"}="H9A2 MGE"; $resn_exch{"1H10 MGE"}="H101 MGE"; $resn_exch{"2H10 MGE"}="H102 MGE";
$resn_exch{"1H11 MGE"}="H111 MGE"; $resn_exch{"2H11 MGE"}="H112 MGE"; $resn_exch{"1H12 MGE"}="H121 MGE";
$resn_exch{"2H12 MGE"}="H122 MGE"; $resn_exch{"1H13 MGE"}="H131 MGE"; $resn_exch{"2H13 MGE"}="H132 MGE";
$resn_exch{"3H13 MGE"}="H133 MGE"; $resn_exch{"1H2B MGE"}="H2B1 MGE"; $resn_exch{"2H2B MGE"}="H2B2 MGE";
$resn_exch{"1H3B MGE"}="H3B1 MGE"; $resn_exch{"2H3B MGE"}="H3B2 MGE"; $resn_exch{"1H4B MGE"}="H4B1 MGE";
$resn_exch{"2H4B MGE"}="H4B2 MGE"; $resn_exch{"1H5B MGE"}="H5B1 MGE"; $resn_exch{"2H5B MGE"}="H5B2 MGE";
$resn_exch{"1H6B MGE"}="H6B1 MGE"; $resn_exch{"2H6B MGE"}="H6B2 MGE"; $resn_exch{"1H7B MGE"}="H7B1 MGE";
$resn_exch{"2H7B MGE"}="H7B2 MGE"; $resn_exch{"1H8B MGE"}="H8B1 MGE"; $resn_exch{"2H8B MGE"}="H8B2 MGE";
$resn_exch{"1H9B MGE"}="H9B1 MGE"; $resn_exch{"2H9B MGE"}="H9B2 MGE"; $resn_exch{"1H20 MGE"}="H201 MGE";
$resn_exch{"2H20 MGE"}="H202 MGE"; $resn_exch{"1H21 MGE"}="H211 MGE"; $resn_exch{"2H21 MGE"}="H212 MGE";
$resn_exch{"1H22 MGE"}="H221 MGE"; $resn_exch{"2H22 MGE"}="H222 MGE"; $resn_exch{"1H23 MGE"}="H231 MGE";
$resn_exch{"2H23 MGE"}="H232 MGE"; $resn_exch{"1H24 MGE"}="H241 MGE"; $resn_exch{"2H24 MGE"}="H242 MGE";
$resn_exch{"1H25 MGE"}="H251 MGE"; $resn_exch{"2H25 MGE"}="H252 MGE"; $resn_exch{"1H26 MGE"}="H261 MGE";
$resn_exch{"2H26 MGE"}="H262 MGE"; $resn_exch{"3H26 MGE"}="H263 MGE"; $resn_exch{"1H1G MGE"}="H1G1 MGE";
$resn_exch{"2H1G MGE"}="H1G2 MGE"; $resn_exch{"1H3G MGE"}="H3G1 MGE"; $resn_exch{"2H3G MGE"}="H3G2 MGE";
$resn_exch{"1H6D MGE"}="H6D1 MGE"; $resn_exch{"2H6D MGE"}="H6D2 MGE"; $resn_exch{"1HB  MGG"}=" HB2 MGG";
$resn_exch{"2HB  MGG"}=" HB3 MGG"; $resn_exch{"1HG  MGG"}=" HG2 MGG"; $resn_exch{"2HG  MGG"}=" HG3 MGG";
$resn_exch{"1HD  MGG"}=" HD2 MGG"; $resn_exch{"2HD  MGG"}=" HD3 MGG"; $resn_exch{" HNE MGG"}=" HE  MGG";
$resn_exch{"1HH1 MGG"}="HH11 MGG"; $resn_exch{"2HH1 MGG"}="HH12 MGG"; $resn_exch{"1HH2 MGG"}="HH21 MGG";
$resn_exch{"2HH2 MGG"}="HH22 MGG"; $resn_exch{"1H2  MGG"}=" H21 MGG"; $resn_exch{"2H2  MGG"}=" H22 MGG";
$resn_exch{"1H   MGG"}=" H1  MGG"; $resn_exch{"1H6  MGL"}=" H61 MGL"; $resn_exch{"2H6  MGL"}=" H62 MGL";
$resn_exch{"1H7  MGL"}=" H71 MGL"; $resn_exch{"2H7  MGL"}=" H72 MGL"; $resn_exch{"3H7  MGL"}=" H73 MGL";
$resn_exch{"1H20 MGM"}="H201 MGM"; $resn_exch{"2H20 MGM"}="H202 MGM"; $resn_exch{"3H20 MGM"}="H203 MGM";
$resn_exch{"1H19 MGM"}="H191 MGM"; $resn_exch{"2H19 MGM"}="H192 MGM"; $resn_exch{"3H19 MGM"}="H193 MGM";
$resn_exch{"1H16 MGM"}="H161 MGM"; $resn_exch{"2H16 MGM"}="H162 MGM"; $resn_exch{"1H14 MGM"}="H141 MGM";
$resn_exch{"2H14 MGM"}="H142 MGM"; $resn_exch{"3H14 MGM"}="H143 MGM"; $resn_exch{"1H15 MGM"}="H151 MGM";
$resn_exch{"2H15 MGM"}="H152 MGM"; $resn_exch{"1H11 MGM"}="H111 MGM"; $resn_exch{"2H11 MGM"}="H112 MGM";
$resn_exch{"1HC9 MGM"}="HC91 MGM"; $resn_exch{"2HC9 MGM"}="HC92 MGM"; $resn_exch{"1H10 MGM"}="H101 MGM";
$resn_exch{"2H10 MGM"}="H102 MGM"; $resn_exch{"3H10 MGM"}="H103 MGM"; $resn_exch{"1HC6 MGM"}="HC61 MGM";
$resn_exch{"2HC6 MGM"}="HC62 MGM"; $resn_exch{"1HC5 MGM"}="HC51 MGM"; $resn_exch{"2HC5 MGM"}="HC52 MGM";
$resn_exch{"1HC4 MGM"}="HC41 MGM"; $resn_exch{"2HC4 MGM"}="HC42 MGM"; $resn_exch{"3HC4 MGM"}="HC43 MGM";
$resn_exch{"1HC2 MGM"}="HC21 MGM"; $resn_exch{"2HC2 MGM"}="HC22 MGM"; $resn_exch{"1HC1 MGM"}="HC11 MGM";
$resn_exch{"2HC1 MGM"}="HC12 MGM"; $resn_exch{"OH1A MGM"}="H1AO MGM"; $resn_exch{"OH1B MGM"}="H1BO MGM";
$resn_exch{"OH2B MGM"}="H2BO MGM"; $resn_exch{"1HN  MGN"}=" H   MGN"; $resn_exch{"2HN  MGN"}=" H2  MGN";
$resn_exch{"1HB1 MGN"}="HB11 MGN"; $resn_exch{"2HB1 MGN"}="HB12 MGN"; $resn_exch{"1HB2 MGN"}="HB21 MGN";
$resn_exch{"2HB2 MGN"}="HB22 MGN"; $resn_exch{"3HB2 MGN"}="HB23 MGN"; $resn_exch{"1HG  MGN"}=" HG1 MGN";
$resn_exch{"2HG  MGN"}=" HG2 MGN"; $resn_exch{"1HN2 MGN"}="HN21 MGN"; $resn_exch{"2HN2 MGN"}="HN22 MGN";
$resn_exch{"1H5* MGO"}=" H51 MGO"; $resn_exch{"2H5* MGO"}=" H52 MGO"; $resn_exch{" H4* MGO"}=" H4  MGO";
$resn_exch{" H3* MGO"}=" H3  MGO"; $resn_exch{" H1* MGO"}=" H1  MGO"; $resn_exch{" H2* MGO"}=" H2  MGO";
$resn_exch{"1HAA MGO"}="HAA1 MGO"; $resn_exch{"2HAA MGO"}="HAA2 MGO"; $resn_exch{"3HAA MGO"}="HAA3 MGO";
$resn_exch{"1HAB MGO"}="HAB1 MGO"; $resn_exch{"2HAB MGO"}="HAB2 MGO"; $resn_exch{" H2  MGQ"}=" H2' MGQ";
$resn_exch{" H4  MGQ"}=" H4' MGQ"; $resn_exch{" H3  MGQ"}=" H3' MGQ"; $resn_exch{"H51  MGQ"}="H5'1 MGQ";
$resn_exch{"H52  MGQ"}="H5'2 MGQ"; $resn_exch{"1H3  MGR"}=" H31 MGR"; $resn_exch{"1H4  MGR"}=" H41 MGR";
$resn_exch{"1H5  MGR"}=" H51 MGR"; $resn_exch{"1H6  MGR"}=" H61 MGR"; $resn_exch{"1H7  MGR"}=" H71 MGR";
$resn_exch{"1H9  MGR"}=" H91 MGR"; $resn_exch{"1H10 MGR"}="H101 MGR"; $resn_exch{"1H12 MGR"}="H121 MGR";
$resn_exch{"1H13 MGR"}="H131 MGR"; $resn_exch{"1H15 MGR"}="H151 MGR"; $resn_exch{"1H16 MGR"}="H161 MGR";
$resn_exch{"1H18 MGR"}="H181 MGR"; $resn_exch{"1H19 MGR"}="H191 MGR"; $resn_exch{"1H22 MGR"}="H221 MGR";
$resn_exch{"2H22 MGR"}="H222 MGR"; $resn_exch{"3H22 MGR"}="H223 MGR"; $resn_exch{"1H23 MGR"}="H231 MGR";
$resn_exch{"2H23 MGR"}="H232 MGR"; $resn_exch{"3H23 MGR"}="H233 MGR"; $resn_exch{"1H24 MGR"}="H241 MGR";
$resn_exch{"2H24 MGR"}="H242 MGR"; $resn_exch{"3H24 MGR"}="H243 MGR"; $resn_exch{"1H25 MGR"}="H251 MGR";
$resn_exch{"2H25 MGR"}="H252 MGR"; $resn_exch{"3H25 MGR"}="H253 MGR"; $resn_exch{"1HC7 MGS"}="HC71 MGS";
$resn_exch{"2HC7 MGS"}="HC72 MGS"; $resn_exch{"3HC7 MGS"}="HC73 MGS"; $resn_exch{"1HC8 MGS"}="HC81 MGS";
$resn_exch{"2HC8 MGS"}="HC82 MGS"; $resn_exch{"3HC8 MGS"}="HC83 MGS"; $resn_exch{"1HC6 MGS"}="HC61 MGS";
$resn_exch{"2HC6 MGS"}="HC62 MGS"; $resn_exch{"3HC6 MGS"}="HC63 MGS"; $resn_exch{"1H12 MGS"}="H121 MGS";
$resn_exch{"2H12 MGS"}="H122 MGS"; $resn_exch{"1H11 MGS"}="H111 MGS"; $resn_exch{"2H11 MGS"}="H112 MGS";
$resn_exch{"2HOG MGT"}="HOG2 MGT"; $resn_exch{"3HOG MGT"}="HOG3 MGT"; $resn_exch{"2HOB MGT"}="HOB2 MGT";
$resn_exch{"2HOA MGT"}="HOA2 MGT"; $resn_exch{"1H5* MGT"}="H5'1 MGT"; $resn_exch{"2H5* MGT"}="H5'2 MGT";
$resn_exch{" H4* MGT"}=" H4' MGT"; $resn_exch{" H3* MGT"}=" H3' MGT"; $resn_exch{"*HO3 MGT"}="HO3' MGT";
$resn_exch{" H2* MGT"}=" H2' MGT"; $resn_exch{"*HO2 MGT"}="HO2' MGT"; $resn_exch{" H1* MGT"}=" H1' MGT";
$resn_exch{"1H8  MGT"}=" H81 MGT"; $resn_exch{"2H8  MGT"}=" H82 MGT"; $resn_exch{"1HM7 MGT"}="HM71 MGT";
$resn_exch{"2HM7 MGT"}="HM72 MGT"; $resn_exch{"3HM7 MGT"}="HM73 MGT"; $resn_exch{"1HN2 MGT"}="HN21 MGT";
$resn_exch{"2HN2 MGT"}="HN22 MGT"; $resn_exch{"1HC8 MGU"}="HC81 MGU"; $resn_exch{"2HC8 MGU"}="HC82 MGU";
$resn_exch{"3HC8 MGU"}="HC83 MGU"; $resn_exch{"1HC6 MGU"}="HC61 MGU"; $resn_exch{"2HC6 MGU"}="HC62 MGU";
$resn_exch{"3HC6 MGU"}="HC63 MGU"; $resn_exch{"1H12 MGU"}="H121 MGU"; $resn_exch{"2H12 MGU"}="H122 MGU";
$resn_exch{"1H11 MGU"}="H111 MGU"; $resn_exch{"2H11 MGU"}="H112 MGU"; $resn_exch{"1H27 MGU"}="H271 MGU";
$resn_exch{"2H27 MGU"}="H272 MGU"; $resn_exch{"3H27 MGU"}="H273 MGU"; $resn_exch{"1H26 MGU"}="H261 MGU";
$resn_exch{"2H26 MGU"}="H262 MGU"; $resn_exch{"3H26 MGU"}="H263 MGU"; $resn_exch{"1H32 MGU"}="H321 MGU";
$resn_exch{"2H32 MGU"}="H322 MGU"; $resn_exch{"1H31 MGU"}="H311 MGU"; $resn_exch{"2H31 MGU"}="H312 MGU";
$resn_exch{"H51  MGV"}="H5'1 MGV"; $resn_exch{"H52  MGV"}="H5'2 MGV"; $resn_exch{"1HD  MGX"}=" HD1 MGX";
$resn_exch{"2HD  MGX"}=" HD2 MGX"; $resn_exch{"3HD  MGX"}=" HD3 MGX"; $resn_exch{"1HH1 MGX"}="HH11 MGX";
$resn_exch{"2HH1 MGX"}="HH12 MGX"; $resn_exch{"1HA  MGY"}=" HA1 MGY"; $resn_exch{"2HA  MGY"}=" HA2 MGY";
$resn_exch{"1HN  MGY"}=" HN1 MGY"; $resn_exch{"2HN  MGY"}=" HN2 MGY"; $resn_exch{"3HN  MGY"}=" HN3 MGY";
$resn_exch{"1HC1 MHA"}="HC11 MHA"; $resn_exch{"2HC1 MHA"}="HC12 MHA"; $resn_exch{"1HC3 MHA"}="HC31 MHA";
$resn_exch{"2HC3 MHA"}="HC32 MHA"; $resn_exch{"1HC5 MHA"}="HC51 MHA"; $resn_exch{"2HC5 MHA"}="HC52 MHA";
$resn_exch{"1HN2 MHA"}="HN21 MHA"; $resn_exch{"2HN2 MHA"}="HN22 MHA"; $resn_exch{"1HM3 MHB"}="HM31 MHB";
$resn_exch{"2HM3 MHB"}="HM32 MHB"; $resn_exch{"3HM3 MHB"}="HM33 MHB"; $resn_exch{"'HO4 MHB"}="HO4' MHB";
$resn_exch{"1H2  MHC"}=" H21 MHC"; $resn_exch{"2H2  MHC"}=" H22 MHC"; $resn_exch{"1H3  MHC"}=" H31 MHC";
$resn_exch{"2H3  MHC"}=" H32 MHC"; $resn_exch{"1H7  MHC"}=" H71 MHC"; $resn_exch{"2H7  MHC"}=" H72 MHC";
$resn_exch{"1H8  MHC"}=" H81 MHC"; $resn_exch{"2H8  MHC"}=" H82 MHC"; $resn_exch{"1H11 MHC"}="H111 MHC";
$resn_exch{"2H11 MHC"}="H112 MHC"; $resn_exch{"1HN2 MHF"}="HN21 MHF"; $resn_exch{"2HN2 MHF"}="HN22 MHF";
$resn_exch{"1H7  MHF"}=" H71 MHF"; $resn_exch{"2H7  MHF"}=" H72 MHF"; $resn_exch{"1H9  MHF"}=" H91 MHF";
$resn_exch{"2H9  MHF"}=" H92 MHF"; $resn_exch{"1H11 MHF"}="H111 MHF"; $resn_exch{"2H11 MHF"}="H112 MHF";
$resn_exch{"1HB  MHF"}=" HB1 MHF"; $resn_exch{"2HB  MHF"}=" HB2 MHF"; $resn_exch{"1HG  MHF"}=" HG1 MHF";
$resn_exch{"2HG  MHF"}=" HG2 MHF"; $resn_exch{"2HOE MHF"}="HOE2 MHF"; $resn_exch{" HN  MHL"}=" H   MHL";
$resn_exch{"1HB  MHL"}=" HB2 MHL"; $resn_exch{"2HB  MHL"}=" HB3 MHL"; $resn_exch{"1HD1 MHL"}="HD11 MHL";
$resn_exch{"2HD1 MHL"}="HD12 MHL"; $resn_exch{"3HD1 MHL"}="HD13 MHL"; $resn_exch{"1HD2 MHL"}="HD21 MHL";
$resn_exch{"2HD2 MHL"}="HD22 MHL"; $resn_exch{"3HD2 MHL"}="HD23 MHL"; $resn_exch{"1HN  MHL"}=" HN1 MHL";
$resn_exch{"2HN  MHL"}=" HN2 MHL"; $resn_exch{"3HN  MHL"}=" HN3 MHL"; $resn_exch{" HO3 MHL"}=" HD3 MHL";
$resn_exch{"2HAA MHM"}="HAA2 MHM"; $resn_exch{"2HMA MHM"}="HMA2 MHM"; $resn_exch{"3HMA MHM"}="HMA3 MHM";
$resn_exch{"2HBA MHM"}="HBA2 MHM"; $resn_exch{"AHO1 MHM"}="HO1A MHM"; $resn_exch{"2HMB MHM"}="HMB2 MHM";
$resn_exch{"3HMB MHM"}="HMB3 MHM"; $resn_exch{"2HBB MHM"}="HBB2 MHM"; $resn_exch{"2HMC MHM"}="HMC2 MHM";
$resn_exch{"3HMC MHM"}="HMC3 MHM"; $resn_exch{"2HBC MHM"}="HBC2 MHM"; $resn_exch{"2HMD MHM"}="HMD2 MHM";
$resn_exch{"3HMD MHM"}="HMD3 MHM"; $resn_exch{"2HAD MHM"}="HAD2 MHM"; $resn_exch{"2HBD MHM"}="HBD2 MHM";
$resn_exch{"DHO2 MHM"}="HO2D MHM"; $resn_exch{"1H8  MHN"}=" H81 MHN"; $resn_exch{"2H8  MHN"}=" H82 MHN";
$resn_exch{"1H7  MHN"}=" H71 MHN"; $resn_exch{"2H7  MHN"}=" H72 MHN"; $resn_exch{"1H6  MHN"}=" H61 MHN";
$resn_exch{"2H6  MHN"}=" H62 MHN"; $resn_exch{"1H5  MHN"}=" H51 MHN"; $resn_exch{"2H5  MHN"}=" H52 MHN";
$resn_exch{"1H4  MHN"}=" H41 MHN"; $resn_exch{"2H4  MHN"}=" H42 MHN"; $resn_exch{"1H1  MHN"}=" H11 MHN";
$resn_exch{"2H1  MHN"}=" H12 MHN"; $resn_exch{"3H1  MHN"}=" H13 MHN"; $resn_exch{"1H3  MHN"}=" H31 MHN";
$resn_exch{"2H3  MHN"}=" H32 MHN"; $resn_exch{"3H3  MHN"}=" H33 MHN"; $resn_exch{"1HN  MHO"}=" H   MHO";
$resn_exch{"2HN  MHO"}=" H2  MHO"; $resn_exch{"1HB  MHO"}=" HB2 MHO"; $resn_exch{"2HB  MHO"}=" HB3 MHO";
$resn_exch{"1HG  MHO"}=" HG2 MHO"; $resn_exch{"2HG  MHO"}=" HG3 MHO"; $resn_exch{"1HE  MHO"}=" HE1 MHO";
$resn_exch{"2HE  MHO"}=" HE2 MHO"; $resn_exch{"3HE  MHO"}=" HE3 MHO"; $resn_exch{" HN2 MHS"}=" H2  MHS";
$resn_exch{"1HB  MHS"}=" HB2 MHS"; $resn_exch{"2HB  MHS"}=" HB3 MHS"; $resn_exch{"1HM  MHS"}=" HM1 MHS";
$resn_exch{"2HM  MHS"}=" HM2 MHS"; $resn_exch{"3HM  MHS"}=" HM3 MHS"; $resn_exch{"1HN  MHZ"}=" HN1 MHZ";
$resn_exch{"2HN  MHZ"}=" HN2 MHZ"; $resn_exch{"1HNA MHZ"}="HNA1 MHZ"; $resn_exch{"1HB  MHZ"}=" HB1 MHZ";
$resn_exch{"2HB  MHZ"}=" HB2 MHZ"; $resn_exch{"1HCC MHZ"}="HCC1 MHZ"; $resn_exch{"2HCC MHZ"}="HCC2 MHZ";
$resn_exch{"1HG  MHZ"}=" HG1 MHZ"; $resn_exch{"2HG  MHZ"}=" HG2 MHZ"; $resn_exch{"1HE  MHZ"}=" HE1 MHZ";
$resn_exch{"2HE  MHZ"}=" HE2 MHZ"; $resn_exch{"3HE  MHZ"}=" HE3 MHZ"; $resn_exch{"1H5* MHZ"}="H5'1 MHZ";
$resn_exch{"2H5* MHZ"}="H5'2 MHZ"; $resn_exch{" H4* MHZ"}=" H4' MHZ"; $resn_exch{" H1* MHZ"}=" H1' MHZ";
$resn_exch{"1HN6 MHZ"}="HN61 MHZ"; $resn_exch{"2HN6 MHZ"}="HN62 MHZ"; $resn_exch{" H2* MHZ"}=" H2' MHZ";
$resn_exch{"*HO2 MHZ"}="HO2' MHZ"; $resn_exch{" H3* MHZ"}=" H3' MHZ"; $resn_exch{"*HO3 MHZ"}="HO3' MHZ";
$resn_exch{"1H12 MI2"}="H121 MI2"; $resn_exch{"2H12 MI2"}="H122 MI2"; $resn_exch{"3H12 MI2"}="H123 MI2";
$resn_exch{"1H19 MI2"}="H191 MI2"; $resn_exch{"2H19 MI2"}="H192 MI2"; $resn_exch{"3HOP MIA"}="HOP3 MIA";
$resn_exch{"2HOP MIA"}="HOP2 MIA"; $resn_exch{"1H5* MIA"}=" H5' MIA"; $resn_exch{"2H5* MIA"}="H5'' MIA";
$resn_exch{" H4* MIA"}=" H4' MIA"; $resn_exch{" H3* MIA"}=" H3' MIA"; $resn_exch{"H3T  MIA"}="HO3' MIA";
$resn_exch{" H2* MIA"}=" H2' MIA"; $resn_exch{"2HO* MIA"}="HO2' MIA"; $resn_exch{" H1* MIA"}=" H1' MIA";
$resn_exch{"1H11 MIA"}="H111 MIA"; $resn_exch{"2H11 MIA"}="H112 MIA"; $resn_exch{"3H11 MIA"}="H113 MIA";
$resn_exch{"1H12 MIA"}="H121 MIA"; $resn_exch{"2H12 MIA"}="H122 MIA"; $resn_exch{"1H13 MIA"}="H131 MIA";
$resn_exch{"2H13 MIA"}="H132 MIA"; $resn_exch{"1H15 MIA"}="H151 MIA"; $resn_exch{"2H15 MIA"}="H152 MIA";
$resn_exch{"3H15 MIA"}="H153 MIA"; $resn_exch{"1H16 MIA"}="H161 MIA"; $resn_exch{"2H16 MIA"}="H162 MIA";
$resn_exch{"3H16 MIA"}="H163 MIA"; $resn_exch{"1HM2 MIC"}="HM21 MIC"; $resn_exch{"2HM2 MIC"}="HM22 MIC";
$resn_exch{"3HM2 MIC"}="HM23 MIC"; $resn_exch{"1H4  MIC"}=" H41 MIC"; $resn_exch{"2H4  MIC"}=" H42 MIC";
$resn_exch{"1H2  MID"}=" H21 MID"; $resn_exch{"2H2  MID"}=" H22 MID"; $resn_exch{"1H3  MID"}="H31A MID";
$resn_exch{"2H3  MID"}="H32A MID"; $resn_exch{"1H4  MID"}=" H41 MID"; $resn_exch{"2H4  MID"}=" H42 MID";
$resn_exch{"1H5  MID"}=" H51 MID"; $resn_exch{"2H5  MID"}=" H52 MID"; $resn_exch{"1H6  MID"}=" H61 MID";
$resn_exch{"2H6  MID"}=" H62 MID"; $resn_exch{"1H11 MID"}="H111 MID"; $resn_exch{"2H11 MID"}="H112 MID";
$resn_exch{"1H19 MID"}="H191 MID"; $resn_exch{"2H19 MID"}="H192 MID"; $resn_exch{"1H37 MID"}="H371 MID";
$resn_exch{"2H37 MID"}="H372 MID"; $resn_exch{"1HBC MIK"}="HBC1 MIK"; $resn_exch{"2HBC MIK"}="HBC2 MIK";
$resn_exch{"1HGC MIK"}="HGC1 MIK"; $resn_exch{"2HGC MIK"}="HGC2 MIK"; $resn_exch{"1HDC MIK"}="HDC1 MIK";
$resn_exch{"2HDC MIK"}="HDC2 MIK"; $resn_exch{"1HEC MIK"}="HEC1 MIK"; $resn_exch{"2HEC MIK"}="HEC2 MIK";
$resn_exch{"1H2A MIL"}="H2A1 MIL"; $resn_exch{"2H2A MIL"}="H2A2 MIL"; $resn_exch{"3H2A MIL"}="H2A3 MIL";
$resn_exch{"1HIB MIM"}="HIB1 MIM"; $resn_exch{"2HIB MIM"}="HIB2 MIM"; $resn_exch{"3HIB MIM"}="HIB3 MIM";
$resn_exch{"1H9B MIM"}="H9B1 MIM"; $resn_exch{"2H9B MIM"}="H9B2 MIM"; $resn_exch{"1H8B MIM"}="H8B1 MIM";
$resn_exch{"2H8B MIM"}="H8B2 MIM"; $resn_exch{"1H7B MIM"}="H7B1 MIM"; $resn_exch{"2H7B MIM"}="H7B2 MIM";
$resn_exch{"1H6B MIM"}="H6B1 MIM"; $resn_exch{"2H6B MIM"}="H6B2 MIM"; $resn_exch{"1HAB MIM"}="HAB1 MIM";
$resn_exch{"2HAB MIM"}="HAB2 MIM"; $resn_exch{"1HBS MIM"}="HBS1 MIM"; $resn_exch{"2HBS MIM"}="HBS2 MIM";
$resn_exch{"SHOG MIM"}="HOGS MIM"; $resn_exch{"1HBK MIM"}="HBK1 MIM"; $resn_exch{"2HBK MIM"}="HBK2 MIM";
$resn_exch{"1HGK MIM"}="HGK1 MIM"; $resn_exch{"2HGK MIM"}="HGK2 MIM"; $resn_exch{"1HDK MIM"}="HDK1 MIM";
$resn_exch{"2HDK MIM"}="HDK2 MIM"; $resn_exch{"1HEK MIM"}="HEK1 MIM"; $resn_exch{"2HEK MIM"}="HEK2 MIM";
$resn_exch{"1HNZ MIM"}="HNZ1 MIM"; $resn_exch{"2HNZ MIM"}="HNZ2 MIM"; $resn_exch{"1HAY MIM"}="HAY1 MIM";
$resn_exch{"2HAY MIM"}="HAY2 MIM"; $resn_exch{"1HBY MIM"}="HBY1 MIM"; $resn_exch{"2HBY MIM"}="HBY2 MIM";
$resn_exch{"1HKY MIM"}="HKY1 MIM"; $resn_exch{"2HKY MIM"}="HKY2 MIM"; $resn_exch{"1HHY MIM"}="HHY1 MIM";
$resn_exch{"2HHY MIM"}="HHY2 MIM"; $resn_exch{"1HZY MIM"}="HZY1 MIM"; $resn_exch{"2HZY MIM"}="HZY2 MIM";
$resn_exch{"1HEY MIM"}="HEY1 MIM"; $resn_exch{"2HEY MIM"}="HEY2 MIM"; $resn_exch{"1HDY MIM"}="HDY1 MIM";
$resn_exch{"2HDY MIM"}="HDY2 MIM"; $resn_exch{"1HN1 MIN"}="HN11 MIN"; $resn_exch{"2HN1 MIN"}="HN12 MIN";
$resn_exch{"1H2  MIN"}="H21A MIN"; $resn_exch{"2H2  MIN"}="H22A MIN"; $resn_exch{"1H3  MIN"}=" H31 MIN";
$resn_exch{"2H3  MIN"}=" H32 MIN"; $resn_exch{"1H5  MIN"}=" H51 MIN"; $resn_exch{"2H5  MIN"}=" H52 MIN";
$resn_exch{"1H6  MIN"}=" H61 MIN"; $resn_exch{"2H6  MIN"}=" H62 MIN"; $resn_exch{"1H7  MIN"}=" H71 MIN";
$resn_exch{"2H7  MIN"}=" H72 MIN"; $resn_exch{"1H10 MIN"}="H101 MIN"; $resn_exch{"2H10 MIN"}="H102 MIN";
$resn_exch{"1H11 MIN"}="H111 MIN"; $resn_exch{"2H11 MIN"}="H112 MIN"; $resn_exch{"1H12 MIN"}="H121 MIN";
$resn_exch{"2H12 MIN"}="H122 MIN"; $resn_exch{"1H14 MIN"}="H141 MIN"; $resn_exch{"2H14 MIN"}="H142 MIN";
$resn_exch{"3H14 MIN"}="H143 MIN"; $resn_exch{"1H16 MIN"}="H161 MIN"; $resn_exch{"2H16 MIN"}="H162 MIN";
$resn_exch{"1H1B MIP"}="H1B1 MIP"; $resn_exch{"2H1B MIP"}="H1B2 MIP"; $resn_exch{"3H1B MIP"}="H1B3 MIP";
$resn_exch{"1H3B MIP"}="H3B1 MIP"; $resn_exch{"2H3B MIP"}="H3B2 MIP"; $resn_exch{"3H3B MIP"}="H3B3 MIP";
$resn_exch{"1HN  MIS"}=" H   MIS"; $resn_exch{"2HN  MIS"}=" H2  MIS"; $resn_exch{"1HB  MIS"}=" HB2 MIS";
$resn_exch{"2HB  MIS"}=" HB3 MIS"; $resn_exch{"1HOP MIS"}="HOP1 MIS"; $resn_exch{"1H2  MIS"}=" H21 MIS";
$resn_exch{"2H2  MIS"}=" H22 MIS"; $resn_exch{"3H2  MIS"}=" H23 MIS"; $resn_exch{"1H3  MIS"}=" H31 MIS";
$resn_exch{"2H3  MIS"}=" H32 MIS"; $resn_exch{"3H3  MIS"}=" H33 MIS"; $resn_exch{"1H6  MIT"}=" H61 MIT";
$resn_exch{"2H6  MIT"}=" H62 MIT"; $resn_exch{"1H8  MIT"}=" H81 MIT"; $resn_exch{"2H8  MIT"}=" H82 MIT";
$resn_exch{"1H16 MIT"}="H161 MIT"; $resn_exch{"2H16 MIT"}="H162 MIT"; $resn_exch{"1H17 MIT"}="H171 MIT";
$resn_exch{"2H17 MIT"}="H172 MIT"; $resn_exch{"1H18 MIT"}="H181 MIT"; $resn_exch{"2H18 MIT"}="H182 MIT";
$resn_exch{"9HN1 MIT"}="HN19 MIT"; $resn_exch{"1HN2 MIT"}="HN21 MIT"; $resn_exch{"2HN2 MIT"}="HN22 MIT";
$resn_exch{"1H23 MIT"}="H231 MIT"; $resn_exch{"2H23 MIT"}="H232 MIT"; $resn_exch{"3H23 MIT"}="H233 MIT";
$resn_exch{"1H28 MIT"}="H281 MIT"; $resn_exch{"2H28 MIT"}="H282 MIT"; $resn_exch{"1H30 MIT"}="H301 MIT";
$resn_exch{"2H30 MIT"}="H302 MIT"; $resn_exch{"1H31 MIT"}="H311 MIT"; $resn_exch{"2H31 MIT"}="H312 MIT";
$resn_exch{"1H35 MIT"}="H351 MIT"; $resn_exch{"2H35 MIT"}="H352 MIT"; $resn_exch{"3H35 MIT"}="H353 MIT";
$resn_exch{"1H27 MIU"}="H271 MIU"; $resn_exch{"2H27 MIU"}="H272 MIU"; $resn_exch{"1H28 MIU"}="H281 MIU";
$resn_exch{"2H28 MIU"}="H282 MIU"; $resn_exch{"1H30 MIU"}="H301 MIU"; $resn_exch{"2H30 MIU"}="H302 MIU";
$resn_exch{"1H31 MIU"}="H311 MIU"; $resn_exch{"2H31 MIU"}="H312 MIU"; $resn_exch{"1H32 MIU"}="H321 MIU";
$resn_exch{"2H32 MIU"}="H322 MIU"; $resn_exch{"1H16 MIU"}="H161 MIU"; $resn_exch{"2H16 MIU"}="H162 MIU";
$resn_exch{"1H17 MIU"}="H171 MIU"; $resn_exch{"2H17 MIU"}="H172 MIU"; $resn_exch{"1H18 MIU"}="H181 MIU";
$resn_exch{"2H18 MIU"}="H182 MIU"; $resn_exch{"1H21 MIU"}="H211 MIU"; $resn_exch{"2H21 MIU"}="H212 MIU";
$resn_exch{"1H22 MIU"}="H221 MIU"; $resn_exch{"2H22 MIU"}="H222 MIU"; $resn_exch{"1H33 MIU"}="H331 MIU";
$resn_exch{"2H33 MIU"}="H332 MIU"; $resn_exch{"1H8C MIU"}="H8C1 MIU"; $resn_exch{"2H8C MIU"}="H8C2 MIU";
$resn_exch{"1H9N MIU"}="H9N1 MIU"; $resn_exch{"2H9N MIU"}="H9N2 MIU"; $resn_exch{"1HAK MIX"}="HAK1 MIX";
$resn_exch{"2HAK MIX"}="HAK2 MIX"; $resn_exch{"1HAM MIX"}="HAM1 MIX"; $resn_exch{"2HAM MIX"}="HAM2 MIX";
$resn_exch{"1HAO MIX"}="HAO1 MIX"; $resn_exch{"2HAO MIX"}="HAO2 MIX"; $resn_exch{"1HAQ MIX"}="HAQ1 MIX";
$resn_exch{"2HAQ MIX"}="HAQ2 MIX"; $resn_exch{"1HAR MIX"}="HAR1 MIX"; $resn_exch{"2HAR MIX"}="HAR2 MIX";
$resn_exch{"1HAP MIX"}="HAP1 MIX"; $resn_exch{"2HAP MIX"}="HAP2 MIX"; $resn_exch{"1HAN MIX"}="HAN1 MIX";
$resn_exch{"2HAN MIX"}="HAN2 MIX"; $resn_exch{"1HAL MIX"}="HAL1 MIX"; $resn_exch{"2HAL MIX"}="HAL2 MIX";
$resn_exch{"1HN2 MIY"}="HN21 MIY"; $resn_exch{"2HN2 MIY"}="HN22 MIY"; $resn_exch{"1H20 MIY"}="H201 MIY";
$resn_exch{"2H20 MIY"}="H202 MIY"; $resn_exch{"3H20 MIY"}="H203 MIY"; $resn_exch{"1H19 MIY"}="H191 MIY";
$resn_exch{"2H19 MIY"}="H192 MIY"; $resn_exch{"3H19 MIY"}="H193 MIY"; $resn_exch{"1H6  MIY"}=" H61 MIY";
$resn_exch{"2H6  MIY"}=" H62 MIY"; $resn_exch{"1H8  MIY"}=" H81 MIY"; $resn_exch{"2H8  MIY"}=" H82 MIY";
$resn_exch{"1HN7 MIY"}="HN71 MIY"; $resn_exch{"2HN7 MIY"}="HN72 MIY"; $resn_exch{"3HN7 MIY"}="HN73 MIY";
$resn_exch{"1H71 MIY"}="H711 MIY"; $resn_exch{"2H71 MIY"}="H712 MIY"; $resn_exch{"3H71 MIY"}="H713 MIY";
$resn_exch{"1H1  MJI"}=" H11 MJI"; $resn_exch{"2H1  MJI"}=" H12 MJI"; $resn_exch{"1H2  MJI"}=" H21 MJI";
$resn_exch{"1H3  MJI"}=" H31 MJI"; $resn_exch{"2H3  MJI"}=" H32 MJI"; $resn_exch{"1H31 MJI"}="H311 MJI";
$resn_exch{"2H31 MJI"}="H312 MJI"; $resn_exch{"1HO2 MJI"}="HO21 MJI"; $resn_exch{"1H2P MJI"}="H2P1 MJI";
$resn_exch{"2H2P MJI"}="H2P2 MJI"; $resn_exch{"3H2P MJI"}="H2P3 MJI"; $resn_exch{"1H11 MJI"}="H111 MJI";
$resn_exch{"2H11 MJI"}="H112 MJI"; $resn_exch{"1H12 MJI"}="H121 MJI"; $resn_exch{"2H12 MJI"}="H122 MJI";
$resn_exch{"1H13 MJI"}="H131 MJI"; $resn_exch{"2H13 MJI"}="H132 MJI"; $resn_exch{"1H14 MJI"}="H141 MJI";
$resn_exch{"2H14 MJI"}="H142 MJI"; $resn_exch{"1H15 MJI"}="H151 MJI"; $resn_exch{"2H15 MJI"}="H152 MJI";
$resn_exch{"1H16 MJI"}="H161 MJI"; $resn_exch{"2H16 MJI"}="H162 MJI"; $resn_exch{"1H17 MJI"}="H171 MJI";
$resn_exch{"2H17 MJI"}="H172 MJI"; $resn_exch{"1H18 MJI"}="H181 MJI"; $resn_exch{"2H18 MJI"}="H182 MJI";
$resn_exch{"1H19 MJI"}="H191 MJI"; $resn_exch{"2H19 MJI"}="H192 MJI"; $resn_exch{"1H20 MJI"}="H201 MJI";
$resn_exch{"2H20 MJI"}="H202 MJI"; $resn_exch{"1H21 MJI"}="H211 MJI"; $resn_exch{"2H21 MJI"}="H212 MJI";
$resn_exch{"1H22 MJI"}="H221 MJI"; $resn_exch{"2H22 MJI"}="H222 MJI"; $resn_exch{"1H23 MJI"}="H231 MJI";
$resn_exch{"2H23 MJI"}="H232 MJI"; $resn_exch{"1H24 MJI"}="H241 MJI"; $resn_exch{"2H24 MJI"}="H242 MJI";
$resn_exch{"1H25 MJI"}="H251 MJI"; $resn_exch{"2H25 MJI"}="H252 MJI"; $resn_exch{"1H26 MJI"}="H261 MJI";
$resn_exch{"2H26 MJI"}="H262 MJI"; $resn_exch{"3H26 MJI"}="H263 MJI"; $resn_exch{"1H1  MK1"}="H11A MK1";
$resn_exch{"2H1  MK1"}=" H12 MK1"; $resn_exch{"1H5  MK1"}=" H51 MK1"; $resn_exch{"2H5  MK1"}=" H52 MK1";
$resn_exch{"3H5  MK1"}=" H53 MK1"; $resn_exch{"1H6  MK1"}=" H61 MK1"; $resn_exch{"2H6  MK1"}=" H62 MK1";
$resn_exch{"3H6  MK1"}=" H63 MK1"; $resn_exch{"1H7  MK1"}=" H71 MK1"; $resn_exch{"2H7  MK1"}=" H72 MK1";
$resn_exch{"3H7  MK1"}=" H73 MK1"; $resn_exch{"1H8  MK1"}=" H81 MK1"; $resn_exch{"2H8  MK1"}=" H82 MK1";
$resn_exch{"1H9  MK1"}=" H91 MK1"; $resn_exch{"2H9  MK1"}=" H92 MK1"; $resn_exch{"1H10 MK1"}="H101 MK1";
$resn_exch{"2H10 MK1"}="H102 MK1"; $resn_exch{"1H12 MK1"}="H121 MK1"; $resn_exch{"2H12 MK1"}="H122 MK1";
$resn_exch{"1H14 MK1"}="H141 MK1"; $resn_exch{"2H14 MK1"}="H142 MK1"; $resn_exch{"1H24 MK1"}="H241 MK1";
$resn_exch{"2H24 MK1"}="H242 MK1"; $resn_exch{"1H31 MK1"}="H311 MK1"; $resn_exch{"2H31 MK1"}="H312 MK1";
$resn_exch{"1H13 MKC"}="H131 MKC"; $resn_exch{"2H13 MKC"}="H132 MKC"; $resn_exch{"1H15 MKC"}="H151 MKC";
$resn_exch{"2H15 MKC"}="H152 MKC"; $resn_exch{"3H15 MKC"}="H153 MKC"; $resn_exch{"1H16 MKC"}="H161 MKC";
$resn_exch{"2H16 MKC"}="H162 MKC"; $resn_exch{"3H16 MKC"}="H163 MKC"; $resn_exch{"1H17 MKC"}="H171 MKC";
$resn_exch{"2H17 MKC"}="H172 MKC"; $resn_exch{"1H18 MKC"}="H181 MKC"; $resn_exch{"2H18 MKC"}="H182 MKC";
$resn_exch{"1H19 MKC"}="H191 MKC"; $resn_exch{"2H19 MKC"}="H192 MKC"; $resn_exch{"3H19 MKC"}="H193 MKC";
$resn_exch{"1HN  MKE"}=" HN1 MKE"; $resn_exch{"2HN  MKE"}=" HN2 MKE"; $resn_exch{"1HB  MKE"}=" HB1 MKE";
$resn_exch{"2HB  MKE"}=" HB2 MKE"; $resn_exch{"1HG  MKE"}=" HG1 MKE"; $resn_exch{"2HG  MKE"}=" HG2 MKE";
$resn_exch{"1HMK MKE"}="HMK1 MKE"; $resn_exch{"2HMK MKE"}="HMK2 MKE"; $resn_exch{"3HMK MKE"}="HMK3 MKE";
$resn_exch{"1H14 MKK"}="H141 MKK"; $resn_exch{"2H14 MKK"}="H142 MKK"; $resn_exch{"1H9  MKK"}=" H91 MKK";
$resn_exch{"2H9  MKK"}=" H92 MKK"; $resn_exch{"3H9  MKK"}=" H93 MKK"; $resn_exch{"1H10 MKK"}="H101 MKK";
$resn_exch{"2H10 MKK"}="H102 MKK"; $resn_exch{"3H10 MKK"}="H103 MKK"; $resn_exch{"1H7  MKK"}=" H71 MKK";
$resn_exch{"2H7  MKK"}=" H72 MKK"; $resn_exch{"1H8  MKK"}=" H81 MKK"; $resn_exch{"2H8  MKK"}=" H82 MKK";
$resn_exch{"1H11 MKK"}="H111 MKK"; $resn_exch{"2H11 MKK"}="H112 MKK"; $resn_exch{"1H12 MKK"}="H121 MKK";
$resn_exch{"2H12 MKK"}="H122 MKK"; $resn_exch{"1H15 MKK"}="H151 MKK"; $resn_exch{"2H15 MKK"}="H152 MKK";
$resn_exch{"1H16 MKK"}="H161 MKK"; $resn_exch{"2H16 MKK"}="H162 MKK"; $resn_exch{"1H18 MKK"}="H181 MKK";
$resn_exch{"2H18 MKK"}="H182 MKK"; $resn_exch{"1H26 MKK"}="H261 MKK"; $resn_exch{"2H26 MKK"}="H262 MKK";
$resn_exch{"3H26 MKK"}="H263 MKK"; $resn_exch{"1H9  MKR"}=" H91 MKR"; $resn_exch{"2H9  MKR"}=" H92 MKR";
$resn_exch{"3H9  MKR"}=" H93 MKR"; $resn_exch{"1H7  MKR"}=" H71 MKR"; $resn_exch{"2H7  MKR"}=" H72 MKR";
$resn_exch{"1H10 MKR"}="H101 MKR"; $resn_exch{"2H10 MKR"}="H102 MKR"; $resn_exch{"1H11 MKR"}="H111 MKR";
$resn_exch{"2H11 MKR"}="H112 MKR"; $resn_exch{"1H12 MKR"}="H121 MKR"; $resn_exch{"2H12 MKR"}="H122 MKR";
$resn_exch{"1H16 MKR"}="H161 MKR"; $resn_exch{"2H16 MKR"}="H162 MKR"; $resn_exch{"1H17 MKR"}="H171 MKR";
$resn_exch{"2H17 MKR"}="H172 MKR"; $resn_exch{"1H19 MKR"}="H191 MKR"; $resn_exch{"2H19 MKR"}="H192 MKR";
$resn_exch{"1H21 MKR"}="H211 MKR"; $resn_exch{"2H21 MKR"}="H212 MKR"; $resn_exch{"1H24 MKR"}="H241 MKR";
$resn_exch{"2H24 MKR"}="H242 MKR"; $resn_exch{"3H24 MKR"}="H243 MKR"; $resn_exch{"1H1  ML1"}=" H11 ML1";
$resn_exch{"2H1  ML1"}="H12A ML1"; $resn_exch{"1H3  ML1"}=" H31 ML1"; $resn_exch{"2H3  ML1"}=" H32 ML1";
$resn_exch{"3H3  ML1"}=" H33 ML1"; $resn_exch{"1H5  ML1"}=" H51 ML1"; $resn_exch{"2H5  ML1"}=" H52 ML1";
$resn_exch{"3H5  ML1"}=" H53 ML1"; $resn_exch{"1H6  ML1"}=" H61 ML1"; $resn_exch{"2H6  ML1"}=" H62 ML1";
$resn_exch{"1H11 ML1"}="H111 ML1"; $resn_exch{"2H11 ML1"}="H112 ML1"; $resn_exch{"1HC2 MLA"}="HC21 MLA";
$resn_exch{"2HC2 MLA"}="HC22 MLA"; $resn_exch{"1HN6 MLC"}="HN61 MLC"; $resn_exch{"2HN6 MLC"}="HN62 MLC";
$resn_exch{"1H5' MLC"}="H5'1 MLC"; $resn_exch{"2H5' MLC"}="H5'2 MLC"; $resn_exch{"1HPB MLC"}="HPB1 MLC";
$resn_exch{"2HPB MLC"}="HPB2 MLC"; $resn_exch{"1HP9 MLC"}="HP91 MLC"; $resn_exch{"2HP9 MLC"}="HP92 MLC";
$resn_exch{"3HP9 MLC"}="HP93 MLC"; $resn_exch{"1HP8 MLC"}="HP81 MLC"; $resn_exch{"2HP8 MLC"}="HP82 MLC";
$resn_exch{"3HP8 MLC"}="HP83 MLC"; $resn_exch{"1HP5 MLC"}="HP51 MLC"; $resn_exch{"2HP5 MLC"}="HP52 MLC";
$resn_exch{"1HP4 MLC"}="HP41 MLC"; $resn_exch{"2HP4 MLC"}="HP42 MLC"; $resn_exch{"1HP2 MLC"}="HP21 MLC";
$resn_exch{"2HP2 MLC"}="HP22 MLC"; $resn_exch{"1HP1 MLC"}="HP11 MLC"; $resn_exch{"2HP1 MLC"}="HP12 MLC";
$resn_exch{"1HM2 MLC"}="HM21 MLC"; $resn_exch{"2HM2 MLC"}="HM22 MLC"; $resn_exch{"1HAA MLD"}="HAA1 MLD";
$resn_exch{"2HAA MLD"}="HAA2 MLD"; $resn_exch{"3HAA MLD"}="HAA3 MLD"; $resn_exch{"1H6  MLD"}=" H61 MLD";
$resn_exch{"2H6  MLD"}=" H62 MLD"; $resn_exch{"1HA6 MLD"}="HA61 MLD"; $resn_exch{"2HA6 MLD"}="HA62 MLD";
$resn_exch{"1HAF MLD"}="HAF1 MLD"; $resn_exch{"2HAF MLD"}="HAF2 MLD"; $resn_exch{"3HAF MLD"}="HAF3 MLD";
$resn_exch{"1HAJ MLD"}="HAJ1 MLD"; $resn_exch{"2HAJ MLD"}="HAJ2 MLD"; $resn_exch{"3HAJ MLD"}="HAJ3 MLD";
$resn_exch{"1HB  MLD"}=" HB1 MLD"; $resn_exch{"2HB  MLD"}=" HB2 MLD"; $resn_exch{"3HB  MLD"}=" HB3 MLD";
$resn_exch{"1HBB MLD"}="HBB1 MLD"; $resn_exch{"2HBB MLD"}="HBB2 MLD"; $resn_exch{"1HBC MLD"}="HBC1 MLD";
$resn_exch{"2HBC MLD"}="HBC2 MLD"; $resn_exch{"1HBL MLD"}="HBL1 MLD"; $resn_exch{"2HBL MLD"}="HBL2 MLD";
$resn_exch{"3HBL MLD"}="HBL3 MLD"; $resn_exch{"1HBP MLD"}="HBP1 MLD"; $resn_exch{"2HBP MLD"}="HBP2 MLD";
$resn_exch{"1HBQ MLD"}="HBQ1 MLD"; $resn_exch{"2HBQ MLD"}="HBQ2 MLD"; $resn_exch{"1HBR MLD"}="HBR1 MLD";
$resn_exch{"2HBR MLD"}="HBR2 MLD"; $resn_exch{"1HBW MLD"}="HBW1 MLD"; $resn_exch{"2HBW MLD"}="HBW2 MLD";
$resn_exch{" HN  MLE"}=" H   MLE"; $resn_exch{"1HN  MLE"}=" HN1 MLE"; $resn_exch{"2HN  MLE"}=" HN2 MLE";
$resn_exch{"3HN  MLE"}=" HN3 MLE"; $resn_exch{"1HB  MLE"}=" HB2 MLE"; $resn_exch{"2HB  MLE"}=" HB3 MLE";
$resn_exch{"1HD1 MLE"}="HD11 MLE"; $resn_exch{"2HD1 MLE"}="HD12 MLE"; $resn_exch{"3HD1 MLE"}="HD13 MLE";
$resn_exch{"1HD2 MLE"}="HD21 MLE"; $resn_exch{"2HD2 MLE"}="HD22 MLE"; $resn_exch{"3HD2 MLE"}="HD23 MLE";
$resn_exch{"1H11 MLG"}="H111 MLG"; $resn_exch{"2H11 MLG"}="H112 MLG"; $resn_exch{"1H10 MLG"}="H101 MLG";
$resn_exch{"2H10 MLG"}="H102 MLG"; $resn_exch{"1H12 MLG"}="H121 MLG"; $resn_exch{"2H12 MLG"}="H122 MLG";
$resn_exch{"1H17 MLG"}="H171 MLG"; $resn_exch{"2H17 MLG"}="H172 MLG"; $resn_exch{"3H17 MLG"}="H173 MLG";
$resn_exch{"1H14 MLG"}="H141 MLG"; $resn_exch{"2H14 MLG"}="H142 MLG"; $resn_exch{"1H1  MLI"}=" H11 MLI";
$resn_exch{"2H1  MLI"}=" H12 MLI"; $resn_exch{"1H9C MLK"}="H9C1 MLK"; $resn_exch{"2H9C MLK"}="H9C2 MLK";
$resn_exch{"1H12 MLK"}="H121 MLK"; $resn_exch{"2H12 MLK"}="H122 MLK"; $resn_exch{"3H12 MLK"}="H123 MLK";
$resn_exch{"1H15 MLK"}="H151 MLK"; $resn_exch{"2H15 MLK"}="H152 MLK"; $resn_exch{"1H21 MLK"}="H211 MLK";
$resn_exch{"2H21 MLK"}="H212 MLK"; $resn_exch{"1H20 MLK"}="H201 MLK"; $resn_exch{"2H20 MLK"}="H202 MLK";
$resn_exch{"1H22 MLK"}="H221 MLK"; $resn_exch{"2H22 MLK"}="H222 MLK"; $resn_exch{"3H22 MLK"}="H223 MLK";
$resn_exch{"1H24 MLK"}="H241 MLK"; $resn_exch{"2H24 MLK"}="H242 MLK"; $resn_exch{"1H25 MLK"}="H251 MLK";
$resn_exch{"2H25 MLK"}="H252 MLK"; $resn_exch{"3H25 MLK"}="H253 MLK"; $resn_exch{"1H23 MLK"}="H231 MLK";
$resn_exch{"2H23 MLK"}="H232 MLK"; $resn_exch{"1H33 MLK"}="H331 MLK"; $resn_exch{"2H33 MLK"}="H332 MLK";
$resn_exch{"1H37 MLK"}="H371 MLK"; $resn_exch{"2H37 MLK"}="H372 MLK"; $resn_exch{"3H37 MLK"}="H373 MLK";
$resn_exch{"1H36 MLK"}="H361 MLK"; $resn_exch{"2H36 MLK"}="H362 MLK"; $resn_exch{"1H39 MLK"}="H391 MLK";
$resn_exch{"2H39 MLK"}="H392 MLK"; $resn_exch{"3H39 MLK"}="H393 MLK"; $resn_exch{"1H29 MLK"}="H291 MLK";
$resn_exch{"2H29 MLK"}="H292 MLK"; $resn_exch{"3H29 MLK"}="H293 MLK"; $resn_exch{"1HN  MLL"}=" H   MLL";
$resn_exch{"2HN  MLL"}=" H2  MLL"; $resn_exch{"1HB  MLL"}=" HB2 MLL"; $resn_exch{"2HB  MLL"}=" HB3 MLL";
$resn_exch{"1HD2 MLL"}="HD21 MLL"; $resn_exch{"2HD2 MLL"}="HD22 MLL"; $resn_exch{"3HD2 MLL"}="HD23 MLL";
$resn_exch{"1HD1 MLL"}="HD11 MLL"; $resn_exch{"2HD1 MLL"}="HD12 MLL"; $resn_exch{"3HD1 MLL"}="HD13 MLL";
$resn_exch{"1H10 MLL"}="H101 MLL"; $resn_exch{"2H10 MLL"}="H102 MLL"; $resn_exch{"3H10 MLL"}="H103 MLL";
$resn_exch{"1H2C MLM"}="H2C1 MLM"; $resn_exch{"2H2C MLM"}="H2C2 MLM"; $resn_exch{"1H6N MLM"}="H6N1 MLM";
$resn_exch{"2H6N MLM"}="H6N2 MLM"; $resn_exch{"3HOP MLN"}="HOP3 MLN"; $resn_exch{"4HOP MLN"}="HOP4 MLN";
$resn_exch{"1HC5 MLN"}="HC51 MLN"; $resn_exch{"1HC7 MLN"}="HC71 MLN"; $resn_exch{"2HC7 MLN"}="HC72 MLN";
$resn_exch{"1HC8 MLN"}="HC81 MLN"; $resn_exch{"2HC8 MLN"}="HC82 MLN"; $resn_exch{"1HC9 MLN"}="HC91 MLN";
$resn_exch{"2HC9 MLN"}="HC92 MLN"; $resn_exch{"1H10 MLN"}="H101 MLN"; $resn_exch{"2H10 MLN"}="H102 MLN";
$resn_exch{"3H10 MLN"}="H103 MLN"; $resn_exch{"1H14 MLN"}="H141 MLN"; $resn_exch{"2H14 MLN"}="H142 MLN";
$resn_exch{"1H16 MLN"}="H161 MLN"; $resn_exch{"2H16 MLN"}="H162 MLN"; $resn_exch{"3H16 MLN"}="H163 MLN";
$resn_exch{"1H17 MLN"}="H171 MLN"; $resn_exch{"2H17 MLN"}="H172 MLN"; $resn_exch{"3H17 MLN"}="H173 MLN";
$resn_exch{"1H1  MLP"}=" H11 MLP"; $resn_exch{"2H1  MLP"}=" H12 MLP"; $resn_exch{"1H2  MLP"}=" H21 MLP";
$resn_exch{"2H2  MLP"}=" H22 MLP"; $resn_exch{"1HN4 MLP"}="HN41 MLP"; $resn_exch{"2HN4 MLP"}="HN42 MLP";
$resn_exch{"1H6A MLR"}="H6A1 MLR"; $resn_exch{"2H6A MLR"}="H6A2 MLR"; $resn_exch{"2HOA MLR"}="HOA2 MLR";
$resn_exch{"3HOA MLR"}="HOA3 MLR"; $resn_exch{"6HOA MLR"}="HOA6 MLR"; $resn_exch{"1H6B MLR"}="H6B1 MLR";
$resn_exch{"2H6B MLR"}="H6B2 MLR"; $resn_exch{"2HOB MLR"}="HOB2 MLR"; $resn_exch{"3HOB MLR"}="HOB3 MLR";
$resn_exch{"6HOB MLR"}="HOB6 MLR"; $resn_exch{"1H6C MLR"}="H6C1 MLR"; $resn_exch{"2H6C MLR"}="H6C2 MLR";
$resn_exch{"2HOC MLR"}="HOC2 MLR"; $resn_exch{"3HOC MLR"}="HOC3 MLR"; $resn_exch{"4HOC MLR"}="HOC4 MLR";
$resn_exch{"6HOC MLR"}="HOC6 MLR"; $resn_exch{"1H3  MLT"}=" H31 MLT"; $resn_exch{"2H3  MLT"}=" H32 MLT";
$resn_exch{"1HN  MLY"}=" H   MLY"; $resn_exch{"2HN  MLY"}=" H2  MLY"; $resn_exch{"1HB  MLY"}=" HB2 MLY";
$resn_exch{"2HB  MLY"}=" HB3 MLY"; $resn_exch{"1HG  MLY"}=" HG2 MLY"; $resn_exch{"2HG  MLY"}=" HG3 MLY";
$resn_exch{"1HD  MLY"}=" HD2 MLY"; $resn_exch{"2HD  MLY"}=" HD3 MLY"; $resn_exch{"1HE  MLY"}=" HE2 MLY";
$resn_exch{"2HE  MLY"}=" HE3 MLY"; $resn_exch{"1HC1 MLY"}="HH11 MLY"; $resn_exch{"2HC1 MLY"}="HH12 MLY";
$resn_exch{"3HC1 MLY"}="HH13 MLY"; $resn_exch{"1HC2 MLY"}="HH21 MLY"; $resn_exch{"2HC2 MLY"}="HH22 MLY";
$resn_exch{"3HC2 MLY"}="HH23 MLY"; $resn_exch{"1HN  MLZ"}=" H   MLZ"; $resn_exch{"2HN  MLZ"}=" H2  MLZ";
$resn_exch{"1HB  MLZ"}=" HB2 MLZ"; $resn_exch{"2HB  MLZ"}=" HB3 MLZ"; $resn_exch{"1HG  MLZ"}=" HG2 MLZ";
$resn_exch{"2HG  MLZ"}=" HG3 MLZ"; $resn_exch{"1HD  MLZ"}=" HD2 MLZ"; $resn_exch{"2HD  MLZ"}=" HD3 MLZ";
$resn_exch{"1HE  MLZ"}=" HE2 MLZ"; $resn_exch{"2HE  MLZ"}=" HE3 MLZ"; $resn_exch{"1HCM MLZ"}="HCM1 MLZ";
$resn_exch{"2HCM MLZ"}="HCM2 MLZ"; $resn_exch{"3HCM MLZ"}="HCM3 MLZ"; $resn_exch{"1H1  MM1"}="H11A MM1";
$resn_exch{"2H1  MM1"}=" H12 MM1"; $resn_exch{"1H2  MM1"}=" H21 MM1"; $resn_exch{"2H2  MM1"}=" H22 MM1";
$resn_exch{"1H3  MM1"}=" H31 MM1"; $resn_exch{"2H3  MM1"}=" H32 MM1"; $resn_exch{"1H5  MM1"}=" H51 MM1";
$resn_exch{"2H5  MM1"}=" H52 MM1"; $resn_exch{"1H6  MM1"}=" H61 MM1"; $resn_exch{"2H6  MM1"}=" H62 MM1";
$resn_exch{"1H8  MM1"}=" H81 MM1"; $resn_exch{"2H8  MM1"}=" H82 MM1"; $resn_exch{"1H9  MM1"}=" H91 MM1";
$resn_exch{"2H9  MM1"}=" H92 MM1"; $resn_exch{"1H10 MM1"}="H101 MM1"; $resn_exch{"2H10 MM1"}="H102 MM1";
$resn_exch{"1H12 MM1"}="H121 MM1"; $resn_exch{"2H12 MM1"}="H122 MM1"; $resn_exch{"1H13 MM1"}="H131 MM1";
$resn_exch{"2H13 MM1"}="H132 MM1"; $resn_exch{"1H1  MM2"}="H11A MM2"; $resn_exch{"2H1  MM2"}=" H12 MM2";
$resn_exch{"1H2  MM2"}="H21A MM2"; $resn_exch{"2H2  MM2"}=" H22 MM2"; $resn_exch{"1H3  MM2"}=" H31 MM2";
$resn_exch{"2H3  MM2"}=" H32 MM2"; $resn_exch{"1H5  MM2"}=" H51 MM2"; $resn_exch{"2H5  MM2"}=" H52 MM2";
$resn_exch{"1H6  MM2"}=" H61 MM2"; $resn_exch{"2H6  MM2"}=" H62 MM2"; $resn_exch{"1H8  MM2"}=" H81 MM2";
$resn_exch{"2H8  MM2"}=" H82 MM2"; $resn_exch{"1H9  MM2"}=" H91 MM2"; $resn_exch{"2H9  MM2"}=" H92 MM2";
$resn_exch{"1H10 MM2"}="H101 MM2"; $resn_exch{"2H10 MM2"}="H102 MM2"; $resn_exch{"1H12 MM2"}="H121 MM2";
$resn_exch{"2H12 MM2"}="H122 MM2"; $resn_exch{"1H13 MM2"}="H131 MM2"; $resn_exch{"2H13 MM2"}="H132 MM2";
$resn_exch{"1H4  MM2"}=" H41 MM2"; $resn_exch{"2H4  MM2"}=" H42 MM2"; $resn_exch{"1H7  MM2"}=" H71 MM2";
$resn_exch{"2H7  MM2"}=" H72 MM2"; $resn_exch{"1H11 MM2"}="H111 MM2"; $resn_exch{"2H11 MM2"}="H112 MM2";
$resn_exch{"1H14 MM2"}="H141 MM2"; $resn_exch{"2H14 MM2"}="H142 MM2"; $resn_exch{"1H15 MM2"}="H151 MM2";
$resn_exch{"2H15 MM2"}="H152 MM2"; $resn_exch{"1H16 MM2"}="H161 MM2"; $resn_exch{"2H16 MM2"}="H162 MM2";
$resn_exch{"1H17 MM2"}="H171 MM2"; $resn_exch{"2H17 MM2"}="H172 MM2"; $resn_exch{"1H18 MM2"}="H181 MM2";
$resn_exch{"2H18 MM2"}="H182 MM2"; $resn_exch{"1H19 MM2"}="H191 MM2"; $resn_exch{"2H19 MM2"}="H192 MM2";
$resn_exch{"1H20 MM2"}="H201 MM2"; $resn_exch{"2H20 MM2"}="H202 MM2"; $resn_exch{"1H27 MM2"}="H271 MM2";
$resn_exch{"2H27 MM2"}="H272 MM2"; $resn_exch{"1H28 MM2"}="H281 MM2"; $resn_exch{"2H28 MM2"}="H282 MM2";
$resn_exch{"1H2  MM3"}=" H21 MM3"; $resn_exch{"1H3  MM3"}=" H31 MM3"; $resn_exch{"1H5  MM3"}=" H51 MM3";
$resn_exch{"1H6  MM3"}=" H61 MM3"; $resn_exch{"1H8  MM3"}=" H81 MM3"; $resn_exch{"2H8  MM3"}=" H82 MM3";
$resn_exch{"3H8  MM3"}=" H83 MM3"; $resn_exch{"1H13 MM3"}="H131 MM3"; $resn_exch{"1H16 MM3"}="H161 MM3";
$resn_exch{"2H16 MM3"}="H162 MM3"; $resn_exch{"1H17 MM3"}="H171 MM3"; $resn_exch{"2H17 MM3"}="H172 MM3";
$resn_exch{"1H18 MM3"}="H181 MM3"; $resn_exch{"2H18 MM3"}="H182 MM3"; $resn_exch{"1H19 MM3"}="H191 MM3";
$resn_exch{"2H19 MM3"}="H192 MM3"; $resn_exch{"3H19 MM3"}="H193 MM3"; $resn_exch{"1H20 MM3"}="H201 MM3";
$resn_exch{"2H20 MM3"}="H202 MM3"; $resn_exch{"3H20 MM3"}="H203 MM3"; $resn_exch{"1H17 MM5"}="H171 MM5";
$resn_exch{"2H17 MM5"}="H172 MM5"; $resn_exch{"1H18 MM5"}="H181 MM5"; $resn_exch{"2H18 MM5"}="H182 MM5";
$resn_exch{"1H19 MM5"}="H191 MM5"; $resn_exch{"2H19 MM5"}="H192 MM5"; $resn_exch{"1H20 MM5"}="H201 MM5";
$resn_exch{"2H20 MM5"}="H202 MM5"; $resn_exch{"1H21 MM5"}="H211 MM5"; $resn_exch{"2H21 MM5"}="H212 MM5";
$resn_exch{"1H22 MM5"}="H221 MM5"; $resn_exch{"2H22 MM5"}="H222 MM5"; $resn_exch{"1H23 MM5"}="H231 MM5";
$resn_exch{"2H23 MM5"}="H232 MM5"; $resn_exch{"1H24 MM5"}="H241 MM5"; $resn_exch{"2H24 MM5"}="H242 MM5";
$resn_exch{"1H25 MM5"}="H251 MM5"; $resn_exch{"2H25 MM5"}="H252 MM5"; $resn_exch{"1H26 MM5"}="H261 MM5";
$resn_exch{"2H26 MM5"}="H262 MM5"; $resn_exch{"1H27 MM5"}="H271 MM5"; $resn_exch{"2H27 MM5"}="H272 MM5";
$resn_exch{"1H28 MM5"}="H281 MM5"; $resn_exch{"2H28 MM5"}="H282 MM5"; $resn_exch{"1H5  MM5"}=" H51 MM5";
$resn_exch{"2H5  MM5"}=" H52 MM5"; $resn_exch{"1H6  MM5"}=" H61 MM5"; $resn_exch{"2H6  MM5"}=" H62 MM5";
$resn_exch{"1H8  MM5"}=" H81 MM5"; $resn_exch{"2H8  MM5"}=" H82 MM5"; $resn_exch{"1H9  MM5"}=" H91 MM5";
$resn_exch{"2H9  MM5"}=" H92 MM5"; $resn_exch{"1H10 MM5"}="H101 MM5"; $resn_exch{"2H10 MM5"}="H102 MM5";
$resn_exch{"1H12 MM5"}="H121 MM5"; $resn_exch{"2H12 MM5"}="H122 MM5"; $resn_exch{"1H13 MM5"}="H131 MM5";
$resn_exch{"2H13 MM5"}="H132 MM5"; $resn_exch{"1H1  MM5"}=" H11 MM5"; $resn_exch{"2H1  MM5"}=" H12 MM5";
$resn_exch{"1H2  MM5"}=" H21 MM5"; $resn_exch{"2H2  MM5"}=" H22 MM5"; $resn_exch{"1H3  MM5"}=" H31 MM5";
$resn_exch{"2H3  MM5"}=" H32 MM5"; $resn_exch{"1H13 MM6"}="H131 MM6"; $resn_exch{"2H13 MM6"}="H132 MM6";
$resn_exch{"1H1  MM6"}=" H11 MM6"; $resn_exch{"2H1  MM6"}=" H12 MM6"; $resn_exch{"1H12 MM6"}="H121 MM6";
$resn_exch{"2H12 MM6"}="H122 MM6"; $resn_exch{"1H10 MM6"}="H101 MM6"; $resn_exch{"2H10 MM6"}="H102 MM6";
$resn_exch{"1H9  MM6"}=" H91 MM6"; $resn_exch{"2H9  MM6"}=" H92 MM6"; $resn_exch{"1H8  MM6"}=" H81 MM6";
$resn_exch{"2H8  MM6"}=" H82 MM6"; $resn_exch{"1H6  MM6"}=" H61 MM6"; $resn_exch{"2H6  MM6"}=" H62 MM6";
$resn_exch{"1H5  MM6"}=" H51 MM6"; $resn_exch{"2H5  MM6"}=" H52 MM6"; $resn_exch{"1H3  MM6"}=" H31 MM6";
$resn_exch{"2H3  MM6"}=" H32 MM6"; $resn_exch{"1H2  MM6"}=" H21 MM6"; $resn_exch{"2H2  MM6"}=" H22 MM6";
$resn_exch{"1H6  MMA"}=" H61 MMA"; $resn_exch{"2H6  MMA"}=" H62 MMA"; $resn_exch{"1H7  MMA"}=" H71 MMA";
$resn_exch{"2H7  MMA"}=" H72 MMA"; $resn_exch{"3H7  MMA"}=" H73 MMA"; $resn_exch{"1H1  MMB"}="H11A MMB";
$resn_exch{"2H1  MMB"}="H12A MMB"; $resn_exch{"1H13 MMB"}="H131 MMB"; $resn_exch{"2H13 MMB"}="H132 MMB";
$resn_exch{"1H26 MMB"}="H261 MMB"; $resn_exch{"2H26 MMB"}="H262 MMB"; $resn_exch{"1H28 MMB"}="H281 MMB";
$resn_exch{"2H28 MMB"}="H282 MMB"; $resn_exch{"3H28 MMB"}="H283 MMB"; $resn_exch{"1H29 MMB"}="H291 MMB";
$resn_exch{"2H29 MMB"}="H292 MMB"; $resn_exch{"1H32 MMB"}="H321 MMB"; $resn_exch{"2H32 MMB"}="H322 MMB";
$resn_exch{"3H32 MMB"}="H323 MMB"; $resn_exch{"1H5  MMB"}=" H51 MMB"; $resn_exch{"2H5  MMB"}=" H52 MMB";
$resn_exch{"3H5  MMB"}=" H53 MMB"; $resn_exch{"1H   MMC"}=" H1  MMC"; $resn_exch{"2H   MMC"}=" H2  MMC";
$resn_exch{"3H   MMC"}=" H3  MMC"; $resn_exch{"1HB  MME"}=" HB2 MME"; $resn_exch{"2HB  MME"}=" HB3 MME";
$resn_exch{"1HG  MME"}=" HG2 MME"; $resn_exch{"2HG  MME"}=" HG3 MME"; $resn_exch{"1HE  MME"}=" HE1 MME";
$resn_exch{"2HE  MME"}=" HE2 MME"; $resn_exch{"3HE  MME"}=" HE3 MME"; $resn_exch{"1HM  MME"}=" HM1 MME";
$resn_exch{"2HM  MME"}=" HM2 MME"; $resn_exch{"3HM  MME"}=" HM3 MME"; $resn_exch{"1H13 MMI"}="H131 MMI";
$resn_exch{"2H13 MMI"}="H132 MMI"; $resn_exch{"3H13 MMI"}="H133 MMI"; $resn_exch{"1H14 MMI"}="H141 MMI";
$resn_exch{"2H14 MMI"}="H142 MMI"; $resn_exch{"3H14 MMI"}="H143 MMI"; $resn_exch{"1H17 MMI"}="H171 MMI";
$resn_exch{"2H17 MMI"}="H172 MMI"; $resn_exch{"1H22 MMI"}="H221 MMI"; $resn_exch{"2H22 MMI"}="H222 MMI";
$resn_exch{"1H28 MMI"}="H281 MMI"; $resn_exch{"2H28 MMI"}="H282 MMI"; $resn_exch{"1H27 MMI"}="H271 MMI";
$resn_exch{"2H27 MMI"}="H272 MMI"; $resn_exch{"1H26 MMI"}="H261 MMI"; $resn_exch{"2H26 MMI"}="H262 MMI";
$resn_exch{"1H25 MMI"}="H251 MMI"; $resn_exch{"2H25 MMI"}="H252 MMI"; $resn_exch{"1H24 MMI"}="H241 MMI";
$resn_exch{"2H24 MMI"}="H242 MMI"; $resn_exch{"1H33 MMI"}="H331 MMI"; $resn_exch{"2H33 MMI"}="H332 MMI";
$resn_exch{"1H35 MMI"}="H351 MMI"; $resn_exch{"2H35 MMI"}="H352 MMI"; $resn_exch{"3H35 MMI"}="H353 MMI";
$resn_exch{"1H36 MMI"}="H361 MMI"; $resn_exch{"2H36 MMI"}="H362 MMI"; $resn_exch{"3H36 MMI"}="H363 MMI";
$resn_exch{"1H37 MMI"}="H371 MMI"; $resn_exch{"2H37 MMI"}="H372 MMI"; $resn_exch{"1H39 MMI"}="H391 MMI";
$resn_exch{"2H39 MMI"}="H392 MMI"; $resn_exch{"3H39 MMI"}="H393 MMI"; $resn_exch{"1H44 MMI"}="H441 MMI";
$resn_exch{"2H44 MMI"}="H442 MMI"; $resn_exch{"3H44 MMI"}="H443 MMI"; $resn_exch{"1H43 MMI"}="H431 MMI";
$resn_exch{"2H43 MMI"}="H432 MMI"; $resn_exch{"3H43 MMI"}="H433 MMI"; $resn_exch{"1H51 MMI"}="H511 MMI";
$resn_exch{"2H51 MMI"}="H512 MMI"; $resn_exch{"1H2A MMM"}="H2A1 MMM"; $resn_exch{"2H2A MMM"}="H2A2 MMM";
$resn_exch{"3H2A MMM"}="H2A3 MMM"; $resn_exch{"1H5A MMM"}="H5A1 MMM"; $resn_exch{"2H5A MMM"}="H5A2 MMM";
$resn_exch{"1HAB MMM"}="HAB1 MMM"; $resn_exch{"2HAB MMM"}="HAB2 MMM"; $resn_exch{"3HAB MMM"}="HAB3 MMM";
$resn_exch{"1HBA MMM"}="HBA1 MMM"; $resn_exch{"2HBA MMM"}="HBA2 MMM"; $resn_exch{"1H8  MMN"}=" H81 MMN";
$resn_exch{"2H8  MMN"}=" H82 MMN"; $resn_exch{"3H8  MMN"}=" H83 MMN"; $resn_exch{"1H1  MMN"}=" H11 MMN";
$resn_exch{"2H1  MMN"}=" H12 MMN"; $resn_exch{"1H6  MMN"}=" H61 MMN"; $resn_exch{"2H6  MMN"}=" H62 MMN";
$resn_exch{"1HCB MMO"}="HCB1 MMO"; $resn_exch{"2HCB MMO"}="HCB2 MMO"; $resn_exch{"1HCG MMO"}="HCG1 MMO";
$resn_exch{"2HCG MMO"}="HCG2 MMO"; $resn_exch{"1HCD MMO"}="HCD1 MMO"; $resn_exch{"2HCD MMO"}="HCD2 MMO";
$resn_exch{"1HH2 MMO"}="HH21 MMO"; $resn_exch{"2HH2 MMO"}="HH22 MMO"; $resn_exch{"1HH1 MMO"}="HH11 MMO";
$resn_exch{"2HH1 MMO"}="HH12 MMO"; $resn_exch{"1HC  MMO"}=" HC1 MMO"; $resn_exch{"2HC  MMO"}=" HC2 MMO";
$resn_exch{"3HC  MMO"}=" HC3 MMO"; $resn_exch{"1H1  MMP"}=" H11 MMP"; $resn_exch{"2H1  MMP"}=" H12 MMP";
$resn_exch{"3H1  MMP"}=" H13 MMP"; $resn_exch{"1H16 MMP"}="H161 MMP"; $resn_exch{"2H16 MMP"}="H162 MMP";
$resn_exch{"3H16 MMP"}="H163 MMP"; $resn_exch{"1H17 MMP"}="H171 MMP"; $resn_exch{"2H17 MMP"}="H172 MMP";
$resn_exch{"1H18 MMP"}="H181 MMP"; $resn_exch{"2H18 MMP"}="H182 MMP"; $resn_exch{"3H18 MMP"}="H183 MMP";
$resn_exch{"1H26 MMP"}="H261 MMP"; $resn_exch{"2H26 MMP"}="H262 MMP"; $resn_exch{"3H26 MMP"}="H263 MMP";
$resn_exch{"1H27 MMP"}="H271 MMP"; $resn_exch{"2H27 MMP"}="H272 MMP"; $resn_exch{"1H28 MMP"}="H281 MMP";
$resn_exch{"2H28 MMP"}="H282 MMP"; $resn_exch{"3H28 MMP"}="H283 MMP"; $resn_exch{"1H36 MMP"}="H361 MMP";
$resn_exch{"2H36 MMP"}="H362 MMP"; $resn_exch{"3H36 MMP"}="H363 MMP"; $resn_exch{"1H37 MMP"}="H371 MMP";
$resn_exch{"2H37 MMP"}="H372 MMP"; $resn_exch{"1H38 MMP"}="H381 MMP"; $resn_exch{"2H38 MMP"}="H382 MMP";
$resn_exch{"1H46 MMP"}="H461 MMP"; $resn_exch{"2H46 MMP"}="H462 MMP"; $resn_exch{"3H46 MMP"}="H463 MMP";
$resn_exch{"1H47 MMP"}="H471 MMP"; $resn_exch{"2H47 MMP"}="H472 MMP"; $resn_exch{"1H48 MMP"}="H481 MMP";
$resn_exch{"2H48 MMP"}="H482 MMP"; $resn_exch{"1HC1 MMQ"}="HC11 MMQ"; $resn_exch{"2HC1 MMQ"}="HC12 MMQ";
$resn_exch{"1H4  MMR"}=" H41 MMR"; $resn_exch{"2H4  MMR"}=" H42 MMR"; $resn_exch{"1H6  MMR"}=" H61 MMR";
$resn_exch{"2H6  MMR"}=" H62 MMR"; $resn_exch{"1H7  MMR"}=" H71 MMR"; $resn_exch{"2H7  MMR"}=" H72 MMR";
$resn_exch{"3H7  MMR"}=" H73 MMR"; $resn_exch{"1H9  MMR"}=" H91 MMR"; $resn_exch{"2H9  MMR"}=" H92 MMR";
$resn_exch{"3H9  MMR"}=" H93 MMR"; $resn_exch{"1H12 MMR"}="H121 MMR"; $resn_exch{"2H12 MMR"}="H122 MMR";
$resn_exch{"3H12 MMR"}="H123 MMR"; $resn_exch{"1HC3 MMT"}="HC31 MMT"; $resn_exch{"2HC3 MMT"}="HC32 MMT";
$resn_exch{"3HC3 MMT"}="HC33 MMT"; $resn_exch{"1HNP MMT"}="HNP1 MMT"; $resn_exch{"2HNP MMT"}="HNP2 MMT";
$resn_exch{"3HNP MMT"}="HNP3 MMT"; $resn_exch{"1H5* MMT"}=" H5' MMT"; $resn_exch{"2H5* MMT"}="H5'' MMT";
$resn_exch{" H4* MMT"}=" H4' MMT"; $resn_exch{" H1* MMT"}=" H1' MMT"; $resn_exch{"1H2* MMT"}=" H2' MMT";
$resn_exch{"2H2* MMT"}="H2'' MMT"; $resn_exch{" H3* MMT"}=" H3' MMT"; $resn_exch{"H3T  MMT"}="HO3' MMT";
$resn_exch{"1H5M MMT"}=" H71 MMT"; $resn_exch{"2H5M MMT"}=" H72 MMT"; $resn_exch{"3H5M MMT"}=" H73 MMT";
$resn_exch{"1H4  MMZ"}=" H41 MMZ"; $resn_exch{"2H4  MMZ"}=" H42 MMZ"; $resn_exch{"3H4  MMZ"}=" H43 MMZ";
$resn_exch{"1H2  MN1"}=" H21 MN1"; $resn_exch{"2H2  MN1"}=" H22 MN1"; $resn_exch{"1H3  MN1"}=" H31 MN1";
$resn_exch{"2H3  MN1"}=" H32 MN1"; $resn_exch{"1H5  MN1"}=" H51 MN1"; $resn_exch{"2H5  MN1"}=" H52 MN1";
$resn_exch{"1H6  MN1"}=" H61 MN1"; $resn_exch{"2H6  MN1"}=" H62 MN1"; $resn_exch{"1H7  MN2"}=" H71 MN2";
$resn_exch{"2H7  MN2"}=" H72 MN2"; $resn_exch{"1H9  MN2"}=" H91 MN2"; $resn_exch{"2H9  MN2"}=" H92 MN2";
$resn_exch{"1H10 MN2"}="H101 MN2"; $resn_exch{"2H10 MN2"}="H102 MN2"; $resn_exch{"1H11 MN2"}="H111 MN2";
$resn_exch{"2H11 MN2"}="H112 MN2"; $resn_exch{"1HN2 MN2"}="HN21 MN2"; $resn_exch{"2HN2 MN2"}="HN22 MN2";
$resn_exch{"3HN2 MN2"}="HN23 MN2"; $resn_exch{"1HO1 MN5"}="HO11 MN5"; $resn_exch{"2HO1 MN5"}="HO12 MN5";
$resn_exch{"1HO2 MN5"}="HO21 MN5"; $resn_exch{"2HO2 MN5"}="HO22 MN5"; $resn_exch{"1HO3 MN5"}="HO31 MN5";
$resn_exch{"2HO3 MN5"}="HO32 MN5"; $resn_exch{"1HO4 MN5"}="HO41 MN5"; $resn_exch{"2HO4 MN5"}="HO42 MN5";
$resn_exch{"1HO5 MN5"}="HO51 MN5"; $resn_exch{"2HO5 MN5"}="HO52 MN5"; $resn_exch{"1HO1 MN6"}="HO11 MN6";
$resn_exch{"2HO1 MN6"}="HO12 MN6"; $resn_exch{"1HO2 MN6"}="HO21 MN6"; $resn_exch{"2HO2 MN6"}="HO22 MN6";
$resn_exch{"1HO3 MN6"}="HO31 MN6"; $resn_exch{"2HO3 MN6"}="HO32 MN6"; $resn_exch{"1HO4 MN6"}="HO41 MN6";
$resn_exch{"2HO4 MN6"}="HO42 MN6"; $resn_exch{"1HO5 MN6"}="HO51 MN6"; $resn_exch{"2HO5 MN6"}="HO52 MN6";
$resn_exch{"1HO6 MN6"}="HO61 MN6"; $resn_exch{"2HO6 MN6"}="HO62 MN6"; $resn_exch{"1H7  MN7"}=" H71 MN7";
$resn_exch{"2H7  MN7"}=" H72 MN7"; $resn_exch{"1H9  MN7"}=" H91 MN7"; $resn_exch{"2H9  MN7"}=" H92 MN7";
$resn_exch{"1H11 MN7"}="H111 MN7"; $resn_exch{"2H11 MN7"}="H112 MN7"; $resn_exch{"3H11 MN7"}="H113 MN7";
$resn_exch{"1H12 MN7"}="H121 MN7"; $resn_exch{"2H12 MN7"}="H122 MN7"; $resn_exch{"3H12 MN7"}="H123 MN7";
$resn_exch{"1H5  MN8"}=" H51 MN8"; $resn_exch{"2H5  MN8"}=" H52 MN8"; $resn_exch{"3H5  MN8"}=" H53 MN8";
$resn_exch{"1H6  MN8"}=" H61 MN8"; $resn_exch{"2H6  MN8"}=" H62 MN8"; $resn_exch{"3H6  MN8"}=" H63 MN8";
$resn_exch{"1H12 MN8"}="H121 MN8"; $resn_exch{"2H12 MN8"}="H122 MN8"; $resn_exch{"1H15 MN8"}="H151 MN8";
$resn_exch{"2H15 MN8"}="H152 MN8"; $resn_exch{"1H3  MNA"}=" H31 MNA"; $resn_exch{"2H3  MNA"}=" H32 MNA";
$resn_exch{"1H9  MNA"}=" H91 MNA"; $resn_exch{"2H9  MNA"}=" H92 MNA"; $resn_exch{"1H11 MNA"}="H111 MNA";
$resn_exch{"2H11 MNA"}="H112 MNA"; $resn_exch{"3H11 MNA"}="H113 MNA"; $resn_exch{"1H12 MNA"}="H121 MNA";
$resn_exch{"2H12 MNA"}="H122 MNA"; $resn_exch{"3H12 MNA"}="H123 MNA"; $resn_exch{"1HOB MNA"}="HOB1 MNA";
$resn_exch{"1H1  MNC"}=" H11 MNC"; $resn_exch{"2H1  MNC"}=" H12 MNC"; $resn_exch{"3H1  MNC"}=" H13 MNC";
$resn_exch{"1H8  MNG"}=" H81 MNG"; $resn_exch{"2H8  MNG"}=" H82 MNG"; $resn_exch{"1H10 MNG"}="H101 MNG";
$resn_exch{"2H10 MNG"}="H102 MNG"; $resn_exch{" H1* MNG"}=" H1' MNG"; $resn_exch{" H2* MNG"}=" H2' MNG";
$resn_exch{" H3* MNG"}=" H3' MNG"; $resn_exch{" H4* MNG"}=" H4' MNG"; $resn_exch{"1H5M MNG"}="H5M1 MNG";
$resn_exch{"2H5M MNG"}="H5M2 MNG"; $resn_exch{"3H5M MNG"}="H5M3 MNG"; $resn_exch{"1H1M MNG"}="H1M1 MNG";
$resn_exch{"2H1M MNG"}="H1M2 MNG"; $resn_exch{"3H1M MNG"}="H1M3 MNG"; $resn_exch{"1H2M MNG"}="H2M1 MNG";
$resn_exch{"2H2M MNG"}="H2M2 MNG"; $resn_exch{"3H2M MNG"}="H2M3 MNG"; $resn_exch{"1H7M MNG"}="H7M1 MNG";
$resn_exch{"2H7M MNG"}="H7M2 MNG"; $resn_exch{"3H7M MNG"}="H7M3 MNG"; $resn_exch{"1H9M MNG"}="H9M1 MNG";
$resn_exch{"2H9M MNG"}="H9M2 MNG"; $resn_exch{"3H9M MNG"}="H9M3 MNG"; $resn_exch{"*HO2 MNG"}="HO2' MNG";
$resn_exch{"*HO4 MNG"}="HO4' MNG"; $resn_exch{"1HMA MNH"}="HMA1 MNH"; $resn_exch{"2HMA MNH"}="HMA2 MNH";
$resn_exch{"3HMA MNH"}="HMA3 MNH"; $resn_exch{"1HAA MNH"}="HAA1 MNH"; $resn_exch{"2HAA MNH"}="HAA2 MNH";
$resn_exch{"1HBA MNH"}="HBA1 MNH"; $resn_exch{"2HBA MNH"}="HBA2 MNH"; $resn_exch{"1HMB MNH"}="HMB1 MNH";
$resn_exch{"2HMB MNH"}="HMB2 MNH"; $resn_exch{"3HMB MNH"}="HMB3 MNH"; $resn_exch{"1HBB MNH"}="HBB1 MNH";
$resn_exch{"2HBB MNH"}="HBB2 MNH"; $resn_exch{"1HMC MNH"}="HMC1 MNH"; $resn_exch{"2HMC MNH"}="HMC2 MNH";
$resn_exch{"3HMC MNH"}="HMC3 MNH"; $resn_exch{"1HBC MNH"}="HBC1 MNH"; $resn_exch{"2HBC MNH"}="HBC2 MNH";
$resn_exch{"1HMD MNH"}="HMD1 MNH"; $resn_exch{"2HMD MNH"}="HMD2 MNH"; $resn_exch{"3HMD MNH"}="HMD3 MNH";
$resn_exch{"1HAD MNH"}="HAD1 MNH"; $resn_exch{"2HAD MNH"}="HAD2 MNH"; $resn_exch{"1HBD MNH"}="HBD1 MNH";
$resn_exch{"2HBD MNH"}="HBD2 MNH"; $resn_exch{"1H8B MNI"}="H8B1 MNI"; $resn_exch{"2H8B MNI"}="H8B2 MNI";
$resn_exch{"3H8B MNI"}="H8B3 MNI"; $resn_exch{"1H9B MNI"}="H9B1 MNI"; $resn_exch{"2H9B MNI"}="H9B2 MNI";
$resn_exch{"3H9B MNI"}="H9B3 MNI"; $resn_exch{"1H1A MNI"}="H1A1 MNI"; $resn_exch{"2H1A MNI"}="H1A2 MNI";
$resn_exch{"1H6A MNI"}="H6A1 MNI"; $resn_exch{"2H6A MNI"}="H6A2 MNI"; $resn_exch{" HN  MNL"}=" H   MNL";
$resn_exch{"1HB  MNL"}=" HB2 MNL"; $resn_exch{"2HB  MNL"}=" HB3 MNL"; $resn_exch{"1HD  MNL"}=" HD2 MNL";
$resn_exch{"2HD  MNL"}=" HD3 MNL"; $resn_exch{"1HE  MNL"}=" HE1 MNL"; $resn_exch{"2HE  MNL"}=" HE2 MNL";
$resn_exch{"3HE  MNL"}=" HE3 MNL"; $resn_exch{"1HM1 MNL"}="HM11 MNL"; $resn_exch{"2HM1 MNL"}="HM12 MNL";
$resn_exch{"3HM1 MNL"}="HM13 MNL"; $resn_exch{"1HM4 MNL"}="HM41 MNL"; $resn_exch{"2HM4 MNL"}="HM42 MNL";
$resn_exch{"3HM4 MNL"}="HM43 MNL"; $resn_exch{"1HC9 MNM"}="HC91 MNM"; $resn_exch{"2HC9 MNM"}="HC92 MNM";
$resn_exch{"1H16 MNO"}="H161 MNO"; $resn_exch{"2H16 MNO"}="H162 MNO"; $resn_exch{"3H16 MNO"}="H163 MNO";
$resn_exch{"1H17 MNO"}="H171 MNO"; $resn_exch{"2H17 MNO"}="H172 MNO"; $resn_exch{"3H17 MNO"}="H173 MNO";
$resn_exch{"1H20 MNO"}="H201 MNO"; $resn_exch{"2H20 MNO"}="H202 MNO"; $resn_exch{"1H21 MNO"}="H211 MNO";
$resn_exch{"2H21 MNO"}="H212 MNO"; $resn_exch{"1H22 MNO"}="H221 MNO"; $resn_exch{"2H22 MNO"}="H222 MNO";
$resn_exch{"1H24 MNO"}="H241 MNO"; $resn_exch{"2H24 MNO"}="H242 MNO"; $resn_exch{"1H15 MNO"}="H151 MNO";
$resn_exch{"2H15 MNO"}="H152 MNO"; $resn_exch{"3H15 MNO"}="H153 MNO"; $resn_exch{"1H2  MNP"}=" H21 MNP";
$resn_exch{"2H2  MNP"}=" H22 MNP"; $resn_exch{"2HOB MNQ"}="HOB2 MNQ"; $resn_exch{"2HOA MNQ"}="HOA2 MNQ";
$resn_exch{"1HNA MNQ"}="HNA1 MNQ"; $resn_exch{"1HA2 MNQ"}="HA21 MNQ"; $resn_exch{"2HA2 MNQ"}="HA22 MNQ";
$resn_exch{"1HA1 MNQ"}="HA11 MNQ"; $resn_exch{"2HA1 MNQ"}="HA12 MNQ"; $resn_exch{"1HMA MNR"}="HMA1 MNR";
$resn_exch{"2HMA MNR"}="HMA2 MNR"; $resn_exch{"3HMA MNR"}="HMA3 MNR"; $resn_exch{"1HAA MNR"}="HAA1 MNR";
$resn_exch{"2HAA MNR"}="HAA2 MNR"; $resn_exch{"1HBA MNR"}="HBA1 MNR"; $resn_exch{"2HBA MNR"}="HBA2 MNR";
$resn_exch{"1HMB MNR"}="HMB1 MNR"; $resn_exch{"2HMB MNR"}="HMB2 MNR"; $resn_exch{"3HMB MNR"}="HMB3 MNR";
$resn_exch{"1HBB MNR"}="HBB1 MNR"; $resn_exch{"2HBB MNR"}="HBB2 MNR"; $resn_exch{"1HMC MNR"}="HMC1 MNR";
$resn_exch{"2HMC MNR"}="HMC2 MNR"; $resn_exch{"3HMC MNR"}="HMC3 MNR"; $resn_exch{"1HBC MNR"}="HBC1 MNR";
$resn_exch{"2HBC MNR"}="HBC2 MNR"; $resn_exch{"1HMD MNR"}="HMD1 MNR"; $resn_exch{"2HMD MNR"}="HMD2 MNR";
$resn_exch{"3HMD MNR"}="HMD3 MNR"; $resn_exch{"1HAD MNR"}="HAD1 MNR"; $resn_exch{"2HAD MNR"}="HAD2 MNR";
$resn_exch{"1HBD MNR"}="HBD1 MNR"; $resn_exch{"2HBD MNR"}="HBD2 MNR"; $resn_exch{"1HM1 MNS"}="HM11 MNS";
$resn_exch{"2HM1 MNS"}="HM12 MNS"; $resn_exch{"3HM1 MNS"}="HM13 MNS"; $resn_exch{"1HM2 MNS"}="HM21 MNS";
$resn_exch{"2HM2 MNS"}="HM22 MNS"; $resn_exch{"3HM2 MNS"}="HM23 MNS"; $resn_exch{"1HN3 MNS"}="HN31 MNS";
$resn_exch{"2HN3 MNS"}="HN32 MNS"; $resn_exch{"2HOB MNT"}="HOB2 MNT"; $resn_exch{"3HOB MNT"}="HOB3 MNT";
$resn_exch{"2HOA MNT"}="HOA2 MNT"; $resn_exch{"1H5* MNT"}="H5'1 MNT"; $resn_exch{"2H5* MNT"}="H5'2 MNT";
$resn_exch{" H4* MNT"}=" H4B MNT"; $resn_exch{" H3* MNT"}=" H3B MNT"; $resn_exch{"1H2* MNT"}="H2'1 MNT";
$resn_exch{"2H2* MNT"}="H2'2 MNT"; $resn_exch{" H1* MNT"}=" H1' MNT"; $resn_exch{"1HN6 MNT"}="HN61 MNT";
$resn_exch{"2HN6 MNT"}="HN62 MNT"; $resn_exch{"'HN2 MNT"}="HN2' MNT"; $resn_exch{"1HM' MNT"}="HM'1 MNT";
$resn_exch{"2HM' MNT"}="HM'2 MNT"; $resn_exch{"3HM' MNT"}="HM'3 MNT"; $resn_exch{"2HOP MNU"}="HOP2 MNU";
$resn_exch{"1H5* MNU"}=" H5' MNU"; $resn_exch{"2H5* MNU"}="H5'' MNU"; $resn_exch{" H4* MNU"}=" H4' MNU";
$resn_exch{" H1* MNU"}=" H1' MNU"; $resn_exch{"1H7  MNU"}=" H71 MNU"; $resn_exch{"2H7  MNU"}=" H72 MNU";
$resn_exch{"1H9  MNU"}=" H91 MNU"; $resn_exch{"2H9  MNU"}=" H92 MNU"; $resn_exch{"3H9  MNU"}=" H93 MNU";
$resn_exch{" H2* MNU"}=" H2' MNU"; $resn_exch{"2HO* MNU"}="HO2' MNU"; $resn_exch{" H3* MNU"}=" H3' MNU";
$resn_exch{"H3T  MNU"}="HO3' MNU"; $resn_exch{"3HOP MNU"}="HOP3 MNU"; $resn_exch{" HN  MNV"}=" H   MNV";
$resn_exch{"1HN  MNV"}=" HN1 MNV"; $resn_exch{"2HN  MNV"}=" HN2 MNV"; $resn_exch{"3HN  MNV"}=" HN3 MNV";
$resn_exch{"1HG1 MNV"}="HG11 MNV"; $resn_exch{"2HG1 MNV"}="HG12 MNV"; $resn_exch{"3HG1 MNV"}="HG13 MNV";
$resn_exch{"1HG2 MNV"}="HG21 MNV"; $resn_exch{"2HG2 MNV"}="HG22 MNV"; $resn_exch{"3HG2 MNV"}="HG23 MNV";
$resn_exch{"1HNT MNV"}="HNT1 MNV"; $resn_exch{"2HNT MNV"}="HNT2 MNV"; $resn_exch{"1H22 MNY"}="H221 MNY";
$resn_exch{"2H22 MNY"}="H222 MNY"; $resn_exch{"1H25 MNY"}="H251 MNY"; $resn_exch{"2H25 MNY"}="H252 MNY";
$resn_exch{"1HO1 MO1"}="HO11 MO1"; $resn_exch{"2HO1 MO1"}="HO12 MO1"; $resn_exch{"1HO1 MO2"}="HO11 MO2";
$resn_exch{"2HO1 MO2"}="HO12 MO2"; $resn_exch{"1HO2 MO2"}="HO21 MO2"; $resn_exch{"2HO2 MO2"}="HO22 MO2";
$resn_exch{"1HO1 MO3"}="HO11 MO3"; $resn_exch{"2HO1 MO3"}="HO12 MO3"; $resn_exch{"1HO2 MO3"}="HO21 MO3";
$resn_exch{"2HO2 MO3"}="HO22 MO3"; $resn_exch{"1HO3 MO3"}="HO31 MO3"; $resn_exch{"2HO3 MO3"}="HO32 MO3";
$resn_exch{"1HO1 MO4"}="HO11 MO4"; $resn_exch{"2HO1 MO4"}="HO12 MO4"; $resn_exch{"1HO2 MO4"}="HO21 MO4";
$resn_exch{"2HO2 MO4"}="HO22 MO4"; $resn_exch{"1HO3 MO4"}="HO31 MO4"; $resn_exch{"2HO3 MO4"}="HO32 MO4";
$resn_exch{"1HO4 MO4"}="HO41 MO4"; $resn_exch{"2HO4 MO4"}="HO42 MO4"; $resn_exch{"1HO1 MO5"}="HO11 MO5";
$resn_exch{"2HO1 MO5"}="HO12 MO5"; $resn_exch{"1HO2 MO5"}="HO21 MO5"; $resn_exch{"2HO2 MO5"}="HO22 MO5";
$resn_exch{"1HO3 MO5"}="HO31 MO5"; $resn_exch{"2HO3 MO5"}="HO32 MO5"; $resn_exch{"1HO4 MO5"}="HO41 MO5";
$resn_exch{"2HO4 MO5"}="HO42 MO5"; $resn_exch{"1HO5 MO5"}="HO51 MO5"; $resn_exch{"2HO5 MO5"}="HO52 MO5";
$resn_exch{"1HO1 MO6"}="HO11 MO6"; $resn_exch{"2HO1 MO6"}="HO12 MO6"; $resn_exch{"1HO2 MO6"}="HO21 MO6";
$resn_exch{"2HO2 MO6"}="HO22 MO6"; $resn_exch{"1HO3 MO6"}="HO31 MO6"; $resn_exch{"2HO3 MO6"}="HO32 MO6";
$resn_exch{"1HO4 MO6"}="HO41 MO6"; $resn_exch{"2HO4 MO6"}="HO42 MO6"; $resn_exch{"1HO5 MO6"}="HO51 MO6";
$resn_exch{"2HO5 MO6"}="HO52 MO6"; $resn_exch{"1HO6 MO6"}="HO61 MO6"; $resn_exch{"2HO6 MO6"}="HO62 MO6";
$resn_exch{"1H7  MO9"}=" H71 MO9"; $resn_exch{"2H7  MO9"}=" H72 MO9"; $resn_exch{"1H9  MO9"}=" H91 MO9";
$resn_exch{"2H9  MO9"}=" H92 MO9"; $resn_exch{"1H17 MO9"}="H171 MO9"; $resn_exch{"2H17 MO9"}="H172 MO9";
$resn_exch{"1H22 MO9"}="H221 MO9"; $resn_exch{"2H22 MO9"}="H222 MO9"; $resn_exch{"1H23 MO9"}="H231 MO9";
$resn_exch{"2H23 MO9"}="H232 MO9"; $resn_exch{"1H25 MO9"}="H251 MO9"; $resn_exch{"2H25 MO9"}="H252 MO9";
$resn_exch{"1H26 MO9"}="H261 MO9"; $resn_exch{"2H26 MO9"}="H262 MO9"; $resn_exch{"1H2  MOA"}=" H21 MOA";
$resn_exch{"1H4  MOA"}=" H41 MOA"; $resn_exch{"2H4  MOA"}=" H42 MOA"; $resn_exch{"1H5  MOA"}=" H51 MOA";
$resn_exch{"2H5  MOA"}=" H52 MOA"; $resn_exch{"1H7  MOA"}=" H71 MOA"; $resn_exch{"2H7  MOA"}=" H72 MOA";
$resn_exch{"3H7  MOA"}=" H73 MOA"; $resn_exch{"1H8  MOA"}=" H81 MOA"; $resn_exch{"2H8  MOA"}=" H82 MOA";
$resn_exch{"3H8  MOA"}=" H83 MOA"; $resn_exch{"1H9  MOA"}=" H91 MOA"; $resn_exch{"2H9  MOA"}=" H92 MOA";
$resn_exch{"3H9  MOA"}=" H93 MOA"; $resn_exch{"1H1O MOA"}="H1O1 MOA"; $resn_exch{"2H10 MOA"}="H102 MOA";
$resn_exch{"1H17 MOA"}="H171 MOA"; $resn_exch{"2H17 MOA"}="H172 MOA"; $resn_exch{"1HO4 MOA"}="HO41 MOA";
$resn_exch{"1HO5 MOA"}="HO51 MOA"; $resn_exch{"1H3  MOB"}=" H31 MOB"; $resn_exch{"2H3  MOB"}=" H32 MOB";
$resn_exch{"3H3  MOB"}=" H33 MOB"; $resn_exch{"1H5  MOB"}=" H51 MOB"; $resn_exch{"2H5  MOB"}=" H52 MOB";
$resn_exch{"3H5  MOB"}=" H53 MOB"; $resn_exch{"'HO4 MOB"}="HO4' MOB"; $resn_exch{"1H1  MOC"}=" H11 MOC";
$resn_exch{"2H1  MOC"}=" H12 MOC"; $resn_exch{"1HN2 MOC"}="HN21 MOC"; $resn_exch{"2HN2 MOC"}="HN22 MOC";
$resn_exch{"1H3  MOC"}=" H31 MOC"; $resn_exch{"2H3  MOC"}=" H32 MOC"; $resn_exch{"1HM6 MOC"}="HM61 MOC";
$resn_exch{"2HM6 MOC"}="HM62 MOC"; $resn_exch{"3HM6 MOC"}="HM63 MOC"; $resn_exch{"1HN7 MOC"}="HN71 MOC";
$resn_exch{"2HN7 MOC"}="HN72 MOC"; $resn_exch{"1H10 MOC"}="H101 MOC"; $resn_exch{"2H10 MOC"}="H102 MOC";
$resn_exch{"1H12 MOC"}="H121 MOC"; $resn_exch{"2H12 MOC"}="H122 MOC"; $resn_exch{"1HN2 MOD"}="HN21 MOD";
$resn_exch{"2HN2 MOD"}="HN22 MOD"; $resn_exch{"1HB  MOD"}=" HB1 MOD"; $resn_exch{"2HB  MOD"}=" HB2 MOD";
$resn_exch{"1HG  MOD"}=" HG1 MOD"; $resn_exch{"2HG  MOD"}=" HG2 MOD"; $resn_exch{"1HE  MOD"}=" HE1 MOD";
$resn_exch{"2HE  MOD"}=" HE2 MOD"; $resn_exch{"3HE  MOD"}=" HE3 MOD"; $resn_exch{"1H9  MOD"}=" H91 MOD";
$resn_exch{"2H9  MOD"}=" H92 MOD"; $resn_exch{"1HOS MOD"}="HOS1 MOD"; $resn_exch{"1H5* MOD"}="H5'1 MOD";
$resn_exch{"2H5* MOD"}="H5'2 MOD"; $resn_exch{" H4* MOD"}=" H4' MOD"; $resn_exch{" H3* MOD"}=" H3' MOD";
$resn_exch{" H2* MOD"}=" H2' MOD"; $resn_exch{" H1* MOD"}=" H1' MOD"; $resn_exch{"1HN6 MOD"}="HN61 MOD";
$resn_exch{"2HN6 MOD"}="HN62 MOD"; $resn_exch{"1HA* MOE"}="HA'1 MOE"; $resn_exch{"2HA* MOE"}="HA'2 MOE";
$resn_exch{"1HB* MOE"}="HB'1 MOE"; $resn_exch{"2HB* MOE"}="HB'2 MOE"; $resn_exch{"1HD* MOE"}="HD'1 MOE";
$resn_exch{"2HD* MOE"}="HD'2 MOE"; $resn_exch{"3HD* MOE"}="HD'3 MOE"; $resn_exch{"1H26 MOF"}="H261 MOF";
$resn_exch{"2H26 MOF"}="H262 MOF"; $resn_exch{"1H31 MOF"}="H311 MOF"; $resn_exch{"2H31 MOF"}="H312 MOF";
$resn_exch{"3H31 MOF"}="H313 MOF"; $resn_exch{"1H20 MOF"}="H201 MOF"; $resn_exch{"2H20 MOF"}="H202 MOF";
$resn_exch{"1H9  MOF"}=" H91 MOF"; $resn_exch{"2H9  MOF"}=" H92 MOF"; $resn_exch{"1H10 MOF"}="H101 MOF";
$resn_exch{"2H10 MOF"}="H102 MOF"; $resn_exch{"3H10 MOF"}="H103 MOF"; $resn_exch{"1H7  MOF"}=" H71 MOF";
$resn_exch{"2H7  MOF"}=" H72 MOF"; $resn_exch{"1H8  MOF"}=" H81 MOF"; $resn_exch{"2H8  MOF"}=" H82 MOF";
$resn_exch{"3H8  MOF"}=" H83 MOF"; $resn_exch{"1H12 MOF"}="H121 MOF"; $resn_exch{"2H12 MOF"}="H122 MOF";
$resn_exch{"1H10 MOG"}="H101 MOG"; $resn_exch{"2H10 MOG"}="H102 MOG"; $resn_exch{"3H10 MOG"}="H103 MOG";
$resn_exch{"1HN2 MOG"}="HN21 MOG"; $resn_exch{"2HN2 MOG"}="HN22 MOG"; $resn_exch{"1H   MOH"}=" H1  MOH";
$resn_exch{"2H   MOH"}=" H2  MOH"; $resn_exch{"3H   MOH"}=" H3  MOH"; $resn_exch{"1H2  MOI"}=" H21 MOI";
$resn_exch{"2H2  MOI"}=" H22 MOI"; $resn_exch{"1H7  MOI"}=" H71 MOI"; $resn_exch{"2H7  MOI"}=" H72 MOI";
$resn_exch{"1H8  MOI"}=" H81 MOI"; $resn_exch{"2H8  MOI"}=" H82 MOI"; $resn_exch{"1H9  MOI"}=" H91 MOI";
$resn_exch{"2H9  MOI"}=" H92 MOI"; $resn_exch{"3H9  MOI"}=" H93 MOI"; $resn_exch{"1HN1 MOL"}="HN11 MOL";
$resn_exch{"2HN1 MOL"}="HN12 MOL"; $resn_exch{"1H7  MOL"}=" H71 MOL"; $resn_exch{"2H7  MOL"}=" H72 MOL";
$resn_exch{"1H10 MOL"}="H101 MOL"; $resn_exch{"2H10 MOL"}="H102 MOL"; $resn_exch{"1H11 MOL"}="H111 MOL";
$resn_exch{"2H11 MOL"}="H112 MOL"; $resn_exch{"1H12 MOL"}="H121 MOL"; $resn_exch{"2H12 MOL"}="H122 MOL";
$resn_exch{"1H13 MOL"}="H131 MOL"; $resn_exch{"2H13 MOL"}="H132 MOL"; $resn_exch{"1H5  MON"}=" H51 MON";
$resn_exch{"2H5  MON"}=" H52 MON"; $resn_exch{"1H6  MON"}=" H61 MON"; $resn_exch{"2H6  MON"}=" H62 MON";
$resn_exch{"1H9  MON"}=" H91 MON"; $resn_exch{"2H9  MON"}=" H92 MON"; $resn_exch{"1H10 MON"}="H101 MON";
$resn_exch{"2H10 MON"}="H102 MON"; $resn_exch{"1H15 MON"}="H151 MON"; $resn_exch{"2H15 MON"}="H152 MON";
$resn_exch{"1H16 MON"}="H161 MON"; $resn_exch{"2H16 MON"}="H162 MON"; $resn_exch{"1H17 MON"}="H171 MON";
$resn_exch{"2H17 MON"}="H172 MON"; $resn_exch{"1H18 MON"}="H181 MON"; $resn_exch{"2H18 MON"}="H182 MON";
$resn_exch{"1H20 MON"}="H201 MON"; $resn_exch{"2H20 MON"}="H202 MON"; $resn_exch{"1H2  MOP"}=" H21 MOP";
$resn_exch{"2H2  MOP"}=" H22 MOP"; $resn_exch{"1H3  MOP"}=" H31 MOP"; $resn_exch{"2H3  MOP"}=" H32 MOP";
$resn_exch{"1H5  MOP"}=" H51 MOP"; $resn_exch{"2H5  MOP"}=" H52 MOP"; $resn_exch{"3H5  MOP"}=" H53 MOP";
$resn_exch{"1HM4 MOP"}="HM41 MOP"; $resn_exch{"2HM4 MOP"}="HM42 MOP"; $resn_exch{"3HM4 MOP"}="HM43 MOP";
$resn_exch{"1H3  MOQ"}=" H31 MOQ"; $resn_exch{"2H3  MOQ"}="H32A MOQ"; $resn_exch{"1H4  MOQ"}=" H41 MOQ";
$resn_exch{"2H4  MOQ"}=" H42 MOQ"; $resn_exch{"1H6  MOQ"}=" H61 MOQ"; $resn_exch{"2H6  MOQ"}=" H62 MOQ";
$resn_exch{"3H6  MOQ"}=" H63 MOQ"; $resn_exch{"1H9  MOQ"}=" H91 MOQ"; $resn_exch{"2H9  MOQ"}=" H92 MOQ";
$resn_exch{"1H10 MOQ"}="H101 MOQ"; $resn_exch{"2H10 MOQ"}="H102 MOQ"; $resn_exch{"3H10 MOQ"}="H103 MOQ";
$resn_exch{"1H15 MOQ"}="H151 MOQ"; $resn_exch{"2H15 MOQ"}="H152 MOQ"; $resn_exch{"3H15 MOQ"}="H153 MOQ";
$resn_exch{"1H19 MOQ"}="H191 MOQ"; $resn_exch{"2H19 MOQ"}="H192 MOQ"; $resn_exch{"3H19 MOQ"}="H193 MOQ";
$resn_exch{"1H20 MOQ"}="H201 MOQ"; $resn_exch{"2H20 MOQ"}="H202 MOQ"; $resn_exch{"3H20 MOQ"}="H203 MOQ";
$resn_exch{"1H23 MOQ"}="H231 MOQ"; $resn_exch{"2H23 MOQ"}="H232 MOQ"; $resn_exch{"3H23 MOQ"}="H233 MOQ";
$resn_exch{"1H25 MOQ"}="H251 MOQ"; $resn_exch{"2H25 MOQ"}="H252 MOQ"; $resn_exch{"1H26 MOQ"}="H261 MOQ";
$resn_exch{"2H26 MOQ"}="H262 MOQ"; $resn_exch{"1H28 MOQ"}="H281 MOQ"; $resn_exch{"2H28 MOQ"}="H282 MOQ";
$resn_exch{"3H28 MOQ"}="H283 MOQ"; $resn_exch{"1H29 MOQ"}="H291 MOQ"; $resn_exch{"2H29 MOQ"}="H292 MOQ";
$resn_exch{"1H31 MOQ"}="H311 MOQ"; $resn_exch{"2H31 MOQ"}="H312 MOQ"; $resn_exch{"3H31 MOQ"}="H313 MOQ";
$resn_exch{"1H33 MOQ"}="H331 MOQ"; $resn_exch{"2H33 MOQ"}="H332 MOQ"; $resn_exch{"3H33 MOQ"}="H333 MOQ";
$resn_exch{"1H34 MOQ"}="H341 MOQ"; $resn_exch{"2H34 MOQ"}="H342 MOQ"; $resn_exch{"1H2  MOR"}=" H21 MOR";
$resn_exch{"2H2  MOR"}=" H22 MOR"; $resn_exch{"1H3  MOR"}=" H31 MOR"; $resn_exch{"2H3  MOR"}=" H32 MOR";
$resn_exch{"1H5  MOR"}=" H51 MOR"; $resn_exch{"2H5  MOR"}=" H52 MOR"; $resn_exch{"1H6  MOR"}=" H61 MOR";
$resn_exch{"2H6  MOR"}=" H62 MOR"; $resn_exch{"1HN2 MOT"}="HN21 MOT"; $resn_exch{"2HN2 MOT"}="HN22 MOT";
$resn_exch{"1HN4 MOT"}="HN41 MOT"; $resn_exch{"2HN4 MOT"}="HN42 MOT"; $resn_exch{"1H9  MOT"}=" H91 MOT";
$resn_exch{"2H9  MOT"}=" H92 MOT"; $resn_exch{"1HM  MOT"}=" HM1 MOT"; $resn_exch{"2HM  MOT"}=" HM2 MOT";
$resn_exch{"3HM  MOT"}=" HM3 MOT"; $resn_exch{"1HB  MOT"}=" HB1 MOT"; $resn_exch{"2HB  MOT"}=" HB2 MOT";
$resn_exch{"1HG  MOT"}=" HG1 MOT"; $resn_exch{"2HG  MOT"}=" HG2 MOT"; $resn_exch{"2HOE MOT"}="HOE2 MOT";
$resn_exch{"1H44 MOU"}="H441 MOU"; $resn_exch{"2H44 MOU"}="H442 MOU"; $resn_exch{"1H43 MOU"}="H431 MOU";
$resn_exch{"2H43 MOU"}="H432 MOU"; $resn_exch{"1H42 MOU"}="H421 MOU"; $resn_exch{"2H42 MOU"}="H422 MOU";
$resn_exch{"1H41 MOU"}="H411 MOU"; $resn_exch{"2H41 MOU"}="H412 MOU"; $resn_exch{"1H40 MOU"}="H401 MOU";
$resn_exch{"2H40 MOU"}="H402 MOU"; $resn_exch{"1H39 MOU"}="H391 MOU"; $resn_exch{"2H39 MOU"}="H392 MOU";
$resn_exch{"1H38 MOU"}="H381 MOU"; $resn_exch{"2H38 MOU"}="H382 MOU"; $resn_exch{"1H26 MOU"}="H261 MOU";
$resn_exch{"2H26 MOU"}="H262 MOU"; $resn_exch{"1H13 MOU"}="H131 MOU"; $resn_exch{"2H13 MOU"}="H132 MOU";
$resn_exch{"1H19 MOU"}="H191 MOU"; $resn_exch{"2H19 MOU"}="H192 MOU"; $resn_exch{"3H19 MOU"}="H193 MOU";
$resn_exch{"1H17 MOU"}="H171 MOU"; $resn_exch{"2H17 MOU"}="H172 MOU"; $resn_exch{"1H18 MOU"}="H181 MOU";
$resn_exch{"2H18 MOU"}="H182 MOU"; $resn_exch{"1H7  MOU"}=" H71 MOU"; $resn_exch{"2H7  MOU"}=" H72 MOU";
$resn_exch{"1H21 MOU"}="H211 MOU"; $resn_exch{"2H21 MOU"}="H212 MOU"; $resn_exch{"3H21 MOU"}="H213 MOU";
$resn_exch{"1H22 MOU"}="H221 MOU"; $resn_exch{"2H22 MOU"}="H222 MOU"; $resn_exch{"3H22 MOU"}="H223 MOU";
$resn_exch{"1H34 MOU"}="H341 MOU"; $resn_exch{"2H34 MOU"}="H342 MOU"; $resn_exch{"3H34 MOU"}="H343 MOU";
$resn_exch{"1H36 MOU"}="H361 MOU"; $resn_exch{"2H36 MOU"}="H362 MOU"; $resn_exch{"3H36 MOU"}="H363 MOU";
$resn_exch{"1H62 MOX"}="H621 MOX"; $resn_exch{"2H62 MOX"}="H622 MOX"; $resn_exch{"3H62 MOX"}="H623 MOX";
$resn_exch{"1H5  MOX"}=" H51 MOX"; $resn_exch{"2H5  MOX"}=" H52 MOX"; $resn_exch{"1H9  MOX"}=" H91 MOX";
$resn_exch{"2H9  MOX"}=" H92 MOX"; $resn_exch{"3H9  MOX"}=" H93 MOX"; $resn_exch{"1H5  MOY"}=" H51 MOY";
$resn_exch{"2H5  MOY"}=" H52 MOY"; $resn_exch{"3H5  MOY"}=" H53 MOY"; $resn_exch{"1H6  MOY"}=" H61 MOY";
$resn_exch{"2H6  MOY"}=" H62 MOY"; $resn_exch{"3H6  MOY"}=" H63 MOY"; $resn_exch{"1H1  MP1"}=" H11 MP1";
$resn_exch{"2H1  MP1"}=" H12 MP1"; $resn_exch{"3H1  MP1"}=" H13 MP1"; $resn_exch{"1H16 MP1"}="H161 MP1";
$resn_exch{"2H16 MP1"}="H162 MP1"; $resn_exch{"3H16 MP1"}="H163 MP1"; $resn_exch{"1H17 MP1"}="H171 MP1";
$resn_exch{"2H17 MP1"}="H172 MP1"; $resn_exch{"1H18 MP1"}="H181 MP1"; $resn_exch{"2H18 MP1"}="H182 MP1";
$resn_exch{"3H18 MP1"}="H183 MP1"; $resn_exch{"1H26 MP1"}="H261 MP1"; $resn_exch{"2H26 MP1"}="H262 MP1";
$resn_exch{"3H26 MP1"}="H263 MP1"; $resn_exch{"1H27 MP1"}="H271 MP1"; $resn_exch{"2H27 MP1"}="H272 MP1";
$resn_exch{"1H28 MP1"}="H281 MP1"; $resn_exch{"2H28 MP1"}="H282 MP1"; $resn_exch{"3H28 MP1"}="H283 MP1";
$resn_exch{"1H36 MP1"}="H361 MP1"; $resn_exch{"2H36 MP1"}="H362 MP1"; $resn_exch{"3H36 MP1"}="H363 MP1";
$resn_exch{"1H37 MP1"}="H371 MP1"; $resn_exch{"2H37 MP1"}="H372 MP1"; $resn_exch{"1H38 MP1"}="H381 MP1";
$resn_exch{"2H38 MP1"}="H382 MP1"; $resn_exch{"1H46 MP1"}="H461 MP1"; $resn_exch{"2H46 MP1"}="H462 MP1";
$resn_exch{"3H46 MP1"}="H463 MP1"; $resn_exch{"1H47 MP1"}="H471 MP1"; $resn_exch{"2H47 MP1"}="H472 MP1";
$resn_exch{"1H48 MP1"}="H481 MP1"; $resn_exch{"2H48 MP1"}="H482 MP1"; $resn_exch{"1H17 MP2"}="H171 MP2";
$resn_exch{"2H17 MP2"}="H172 MP2"; $resn_exch{"1HA  MP2"}=" HA1 MP2"; $resn_exch{"2HA  MP2"}=" HA2 MP2";
$resn_exch{"1H23 MP2"}="H231 MP2"; $resn_exch{"2H23 MP2"}="H232 MP2"; $resn_exch{"1H3  MP3"}=" H31 MP3";
$resn_exch{"2H3  MP3"}=" H32 MP3"; $resn_exch{"2HOC MP3"}="HOC2 MP3"; $resn_exch{"3HOC MP3"}="HOC3 MP3";
$resn_exch{"1H3  MP5"}=" H31 MP5"; $resn_exch{"2H3  MP5"}=" H32 MP5"; $resn_exch{"1HN1 MP5"}="HN11 MP5";
$resn_exch{"2HN1 MP5"}="HN12 MP5"; $resn_exch{"1H5  MP5"}=" H51 MP5"; $resn_exch{"2H5  MP5"}=" H52 MP5";
$resn_exch{"3H5  MP5"}=" H53 MP5"; $resn_exch{"1H11 MP9"}="H111 MP9"; $resn_exch{"2H11 MP9"}="H112 MP9";
$resn_exch{"1H24 MP9"}="H241 MP9"; $resn_exch{"2H24 MP9"}="H242 MP9"; $resn_exch{"1H29 MP9"}="H291 MP9";
$resn_exch{"2H29 MP9"}="H292 MP9"; $resn_exch{"2H1  MPA"}="H12A MPA"; $resn_exch{"1H2  MPA"}=" H21 MPA";
$resn_exch{"2H2  MPA"}=" H22 MPA"; $resn_exch{"1H3  MPA"}=" H31 MPA"; $resn_exch{"2H3  MPA"}=" H32 MPA";
$resn_exch{"1H4  MPA"}=" H41 MPA"; $resn_exch{"2H4  MPA"}=" H42 MPA"; $resn_exch{"1H5  MPA"}=" H51 MPA";
$resn_exch{"2H5  MPA"}=" H52 MPA"; $resn_exch{"1H6  MPA"}=" H61 MPA"; $resn_exch{"2H6  MPA"}=" H62 MPA";
$resn_exch{"3H6  MPA"}=" H63 MPA"; $resn_exch{"1H8  MPA"}=" H81 MPA"; $resn_exch{"2H8  MPA"}=" H82 MPA";
$resn_exch{"1H10 MPA"}="H101 MPA"; $resn_exch{"2H10 MPA"}="H102 MPA"; $resn_exch{"1H11 MPA"}="H111 MPA";
$resn_exch{"2H11 MPA"}="H112 MPA"; $resn_exch{"1H13 MPA"}="H131 MPA"; $resn_exch{"2H13 MPA"}="H132 MPA";
$resn_exch{"3H13 MPA"}="H133 MPA"; $resn_exch{"1H15 MPA"}="H151 MPA"; $resn_exch{"2H15 MPA"}="H152 MPA";
$resn_exch{"3H15 MPA"}="H153 MPA"; $resn_exch{"1H16 MPA"}="H161 MPA"; $resn_exch{"2H16 MPA"}="H162 MPA";
$resn_exch{"3H16 MPA"}="H163 MPA"; $resn_exch{"1HM  MPB"}=" HM1 MPB"; $resn_exch{"2HM  MPB"}=" HM2 MPB";
$resn_exch{"3HM  MPB"}=" HM3 MPB"; $resn_exch{"2H1  MPC"}="H12A MPC"; $resn_exch{"1H2  MPC"}=" H21 MPC";
$resn_exch{"2H2  MPC"}=" H22 MPC"; $resn_exch{"1H3  MPC"}=" H31 MPC"; $resn_exch{"2H3  MPC"}=" H32 MPC";
$resn_exch{"1H4  MPC"}=" H41 MPC"; $resn_exch{"2H4  MPC"}=" H42 MPC"; $resn_exch{"1H5  MPC"}=" H51 MPC";
$resn_exch{"2H5  MPC"}=" H52 MPC"; $resn_exch{"1H6  MPC"}=" H61 MPC"; $resn_exch{"2H6  MPC"}=" H62 MPC";
$resn_exch{"3H6  MPC"}=" H63 MPC"; $resn_exch{"1H8  MPC"}=" H81 MPC"; $resn_exch{"2H8  MPC"}=" H82 MPC";
$resn_exch{"1H10 MPC"}="H101 MPC"; $resn_exch{"2H10 MPC"}="H102 MPC"; $resn_exch{"1H11 MPC"}="H111 MPC";
$resn_exch{"2H11 MPC"}="H112 MPC"; $resn_exch{"1H13 MPC"}="H131 MPC"; $resn_exch{"2H13 MPC"}="H132 MPC";
$resn_exch{"3H13 MPC"}="H133 MPC"; $resn_exch{"1H15 MPC"}="H151 MPC"; $resn_exch{"2H15 MPC"}="H152 MPC";
$resn_exch{"3H15 MPC"}="H153 MPC"; $resn_exch{"1H16 MPC"}="H161 MPC"; $resn_exch{"2H16 MPC"}="H162 MPC";
$resn_exch{"3H16 MPC"}="H163 MPC"; $resn_exch{"1H1  MPD"}=" H11 MPD"; $resn_exch{"2H1  MPD"}=" H12 MPD";
$resn_exch{"3H1  MPD"}=" H13 MPD"; $resn_exch{"1HM  MPD"}=" HM1 MPD"; $resn_exch{"2HM  MPD"}=" HM2 MPD";
$resn_exch{"3HM  MPD"}=" HM3 MPD"; $resn_exch{"1H3  MPD"}=" H31 MPD"; $resn_exch{"2H3  MPD"}=" H32 MPD";
$resn_exch{"1H5  MPD"}=" H51 MPD"; $resn_exch{"2H5  MPD"}=" H52 MPD"; $resn_exch{"3H5  MPD"}=" H53 MPD";
$resn_exch{"1H3  MPE"}=" H31 MPE"; $resn_exch{"2H3  MPE"}=" H32 MPE"; $resn_exch{"1H2  MPE"}=" H21 MPE";
$resn_exch{"2H2  MPE"}=" H22 MPE"; $resn_exch{"2H2  MPG"}=" H22 MPG"; $resn_exch{"1H2  MPG"}=" H21 MPG";
$resn_exch{"2H3  MPG"}=" H32 MPG"; $resn_exch{"1H3  MPG"}=" H31 MPG"; $resn_exch{"2H4  MPG"}=" H42 MPG";
$resn_exch{"1H4  MPG"}=" H41 MPG"; $resn_exch{"2H5  MPG"}=" H52 MPG"; $resn_exch{"1H5  MPG"}=" H51 MPG";
$resn_exch{"2H6  MPG"}=" H62 MPG"; $resn_exch{"1H6  MPG"}=" H61 MPG"; $resn_exch{"2H7  MPG"}=" H72 MPG";
$resn_exch{"1H7  MPG"}=" H71 MPG"; $resn_exch{"2H8  MPG"}=" H82 MPG"; $resn_exch{"1H8  MPG"}=" H81 MPG";
$resn_exch{"2H11 MPG"}="H112 MPG"; $resn_exch{"1H11 MPG"}="H111 MPG"; $resn_exch{"2H12 MPG"}="H122 MPG";
$resn_exch{"1H12 MPG"}="H121 MPG"; $resn_exch{"2H13 MPG"}="H132 MPG"; $resn_exch{"1H13 MPG"}="H131 MPG";
$resn_exch{"2H14 MPG"}="H142 MPG"; $resn_exch{"1H14 MPG"}="H141 MPG"; $resn_exch{"2H15 MPG"}="H152 MPG";
$resn_exch{"1H15 MPG"}="H151 MPG"; $resn_exch{"2H16 MPG"}="H162 MPG"; $resn_exch{"1H16 MPG"}="H161 MPG";
$resn_exch{"2H17 MPG"}="H172 MPG"; $resn_exch{"1H17 MPG"}="H171 MPG"; $resn_exch{"3H18 MPG"}="H183 MPG";
$resn_exch{"2H18 MPG"}="H182 MPG"; $resn_exch{"1H18 MPG"}="H181 MPG"; $resn_exch{"2H1  MPG"}="H12A MPG";
$resn_exch{"1H1  MPG"}=" H11 MPG"; $resn_exch{"2H21 MPG"}="H212 MPG"; $resn_exch{"1H21 MPG"}="H211 MPG";
$resn_exch{"1HB  MPH"}=" HB1 MPH"; $resn_exch{"2HB  MPH"}=" HB2 MPH"; $resn_exch{"1HG  MPH"}=" HG1 MPH";
$resn_exch{"2HG  MPH"}=" HG2 MPH"; $resn_exch{"1HE  MPH"}=" HE1 MPH"; $resn_exch{"2HE  MPH"}=" HE2 MPH";
$resn_exch{"3HE  MPH"}=" HE3 MPH"; $resn_exch{"1HB  MPJ"}=" HB1 MPJ"; $resn_exch{"2HB  MPJ"}=" HB2 MPJ";
$resn_exch{"1HG  MPJ"}=" HG1 MPJ"; $resn_exch{"2HG  MPJ"}=" HG2 MPJ"; $resn_exch{"1HE  MPJ"}=" HE1 MPJ";
$resn_exch{"2HE  MPJ"}=" HE2 MPJ"; $resn_exch{"3HE  MPJ"}=" HE3 MPJ"; $resn_exch{"2HOP MPL"}="HOP2 MPL";
$resn_exch{"3HOP MPL"}="HOP3 MPL"; $resn_exch{"1H5  MPL"}=" H51 MPL"; $resn_exch{"2H5  MPL"}=" H52 MPL";
$resn_exch{"1H1  MPL"}=" H11 MPL"; $resn_exch{"2H1  MPL"}=" H12 MPL"; $resn_exch{"3H1  MPL"}=" H13 MPL";
$resn_exch{"1H2  MPL"}=" H21 MPL"; $resn_exch{"2H2  MPL"}=" H22 MPL"; $resn_exch{"3H2  MPL"}=" H23 MPL";
$resn_exch{"CH4A MPM"}="H4AC MPM"; $resn_exch{"1H5A MPM"}="H5A1 MPM"; $resn_exch{"2H5A MPM"}="H5A2 MPM";
$resn_exch{"1H2A MPM"}="H2A1 MPM"; $resn_exch{"2H2A MPM"}="H2A2 MPM"; $resn_exch{"3H2A MPM"}="H2A3 MPM";
$resn_exch{"1HA2 MPM"}="HA21 MPM"; $resn_exch{"2HA2 MPM"}="HA22 MPM"; $resn_exch{"1HA1 MPM"}="HA11 MPM";
$resn_exch{"2HA1 MPM"}="HA12 MPM"; $resn_exch{"3HA1 MPM"}="HA13 MPM"; $resn_exch{"1HA3 MPM"}="HA31 MPM";
$resn_exch{"2HA3 MPM"}="HA32 MPM"; $resn_exch{"3HA3 MPM"}="HA33 MPM"; $resn_exch{"3HOP MPN"}="HOP3 MPN";
$resn_exch{"2HOP MPN"}="HOP2 MPN"; $resn_exch{"1H4' MPN"}="H4'1 MPN"; $resn_exch{"2H4' MPN"}="H4'2 MPN";
$resn_exch{"1H2  MPN"}=" H21 MPN"; $resn_exch{"2H2  MPN"}=" H22 MPN"; $resn_exch{"1H1  MPO"}=" H11 MPO";
$resn_exch{"2H1  MPO"}=" H12 MPO"; $resn_exch{"1H2  MPO"}=" H21 MPO"; $resn_exch{"2H2  MPO"}=" H22 MPO";
$resn_exch{"1H3  MPO"}=" H31 MPO"; $resn_exch{"2H3  MPO"}=" H32 MPO"; $resn_exch{"1H4  MPO"}=" H41 MPO";
$resn_exch{"2H4  MPO"}=" H42 MPO"; $resn_exch{"1H5  MPO"}=" H51 MPO"; $resn_exch{"2H5  MPO"}=" H52 MPO";
$resn_exch{"1H6  MPO"}=" H61 MPO"; $resn_exch{"2H6  MPO"}=" H62 MPO"; $resn_exch{"1H7  MPO"}=" H71 MPO";
$resn_exch{"2H7  MPO"}=" H72 MPO"; $resn_exch{"1HA  MPP"}=" HA1 MPP"; $resn_exch{"2HA  MPP"}=" HA2 MPP";
$resn_exch{"1HB  MPP"}=" HB1 MPP"; $resn_exch{"2HB  MPP"}=" HB2 MPP"; $resn_exch{"1HEA MPP"}="HEA1 MPP";
$resn_exch{"2HEA MPP"}="HEA2 MPP"; $resn_exch{"3HEA MPP"}="HEA3 MPP"; $resn_exch{"1HZA MPP"}="HZA1 MPP";
$resn_exch{"2HZA MPP"}="HZA2 MPP"; $resn_exch{"3HZA MPP"}="HZA3 MPP"; $resn_exch{"1HM  MPQ"}=" HM1 MPQ";
$resn_exch{"2HM  MPQ"}=" HM2 MPQ"; $resn_exch{"3HM  MPQ"}=" HM3 MPQ"; $resn_exch{"1HA  MPQ"}=" HA1 MPQ";
$resn_exch{"1HD1 MPQ"}="HD11 MPQ"; $resn_exch{"1HD2 MPQ"}="HD21 MPQ"; $resn_exch{"1HE1 MPQ"}="HE11 MPQ";
$resn_exch{"1HE2 MPQ"}="HE21 MPQ"; $resn_exch{"1HZ  MPQ"}=" HZ1 MPQ"; $resn_exch{"1H2  MPR"}=" H21 MPR";
$resn_exch{"2H2  MPR"}=" H22 MPR"; $resn_exch{"1H3  MPR"}=" H31 MPR"; $resn_exch{"2H3  MPR"}=" H32 MPR";
$resn_exch{"1HM  MPS"}=" HM1 MPS"; $resn_exch{"2HM  MPS"}=" HM2 MPS"; $resn_exch{"3HM  MPS"}=" HM3 MPS";
$resn_exch{"1HA  MPT"}=" HA1 MPT"; $resn_exch{"2HA  MPT"}=" HA2 MPT"; $resn_exch{"1HB  MPT"}=" HB1 MPT";
$resn_exch{"2HB  MPT"}=" HB2 MPT"; $resn_exch{"1H13 MPU"}="H131 MPU"; $resn_exch{"2H13 MPU"}="H132 MPU";
$resn_exch{"1H28 MPU"}="H281 MPU"; $resn_exch{"2H28 MPU"}="H282 MPU"; $resn_exch{"1H23 MPX"}="H231 MPX";
$resn_exch{"2H23 MPX"}="H232 MPX"; $resn_exch{"1H22 MPX"}="H221 MPX"; $resn_exch{"2H22 MPX"}="H222 MPX";
$resn_exch{"1H24 MPX"}="H241 MPX"; $resn_exch{"2H24 MPX"}="H242 MPX"; $resn_exch{"3H24 MPX"}="H243 MPX";
$resn_exch{"1H20 MPX"}="H201 MPX"; $resn_exch{"2H20 MPX"}="H202 MPX"; $resn_exch{"1H19 MPX"}="H191 MPX";
$resn_exch{"2H19 MPX"}="H192 MPX"; $resn_exch{"1H15 MPX"}="H151 MPX"; $resn_exch{"2H15 MPX"}="H152 MPX";
$resn_exch{"1H18 MPY"}="H181 MPY"; $resn_exch{"2H18 MPY"}="H182 MPY"; $resn_exch{"1H19 MPY"}="H191 MPY";
$resn_exch{"2H19 MPY"}="H192 MPY"; $resn_exch{"1H21 MPY"}="H211 MPY"; $resn_exch{"2H21 MPY"}="H212 MPY";
$resn_exch{"1H22 MPY"}="H221 MPY"; $resn_exch{"2H22 MPY"}="H222 MPY"; $resn_exch{"1H23 MPY"}="H231 MPY";
$resn_exch{"2H23 MPY"}="H232 MPY"; $resn_exch{"3H23 MPY"}="H233 MPY"; $resn_exch{"1H32 MPY"}="H321 MPY";
$resn_exch{"2H32 MPY"}="H322 MPY"; $resn_exch{"1H34 MPY"}="H341 MPY"; $resn_exch{"2H34 MPY"}="H342 MPY";
$resn_exch{"1H36 MPY"}="H361 MPY"; $resn_exch{"2H36 MPY"}="H362 MPY"; $resn_exch{"3H36 MPY"}="H363 MPY";
$resn_exch{"1H37 MPY"}="H371 MPY"; $resn_exch{"2H37 MPY"}="H372 MPY"; $resn_exch{"3H37 MPY"}="H373 MPY";
$resn_exch{"1H08 MPZ"}="H081 MPZ"; $resn_exch{"2H08 MPZ"}="H082 MPZ"; $resn_exch{"1H06 MPZ"}="H061 MPZ";
$resn_exch{"2H06 MPZ"}="H062 MPZ"; $resn_exch{"1H05 MPZ"}="H051 MPZ"; $resn_exch{"2H05 MPZ"}="H052 MPZ";
$resn_exch{"1H04 MPZ"}="H041 MPZ"; $resn_exch{"2H04 MPZ"}="H042 MPZ"; $resn_exch{"1H03 MPZ"}="H031 MPZ";
$resn_exch{"2H03 MPZ"}="H032 MPZ"; $resn_exch{"1H01 MPZ"}="H011 MPZ"; $resn_exch{"2H01 MPZ"}="H012 MPZ";
$resn_exch{"3H01 MPZ"}="H013 MPZ"; $resn_exch{"1H20 MQ0"}="H201 MQ0"; $resn_exch{"2H20 MQ0"}="H202 MQ0";
$resn_exch{"1HN9 MQ0"}="HN91 MQ0"; $resn_exch{"2HN9 MQ0"}="HN92 MQ0"; $resn_exch{"1H7  MQ1"}=" H71 MQ1";
$resn_exch{"2H7  MQ1"}=" H72 MQ1"; $resn_exch{"1H14 MQ1"}="H141 MQ1"; $resn_exch{"2H14 MQ1"}="H142 MQ1";
$resn_exch{"1H33 MQ1"}="H331 MQ1"; $resn_exch{"2H33 MQ1"}="H332 MQ1"; $resn_exch{"3H33 MQ1"}="H333 MQ1";
$resn_exch{"1H2M MQ7"}="H2M1 MQ7"; $resn_exch{"2H2M MQ7"}="H2M2 MQ7"; $resn_exch{"3H2M MQ7"}="H2M3 MQ7";
$resn_exch{"1H11 MQ7"}="H111 MQ7"; $resn_exch{"2H11 MQ7"}="H112 MQ7"; $resn_exch{"1H14 MQ7"}="H141 MQ7";
$resn_exch{"2H14 MQ7"}="H142 MQ7"; $resn_exch{"3H14 MQ7"}="H143 MQ7"; $resn_exch{"1H15 MQ7"}="H151 MQ7";
$resn_exch{"2H15 MQ7"}="H152 MQ7"; $resn_exch{"1H16 MQ7"}="H161 MQ7"; $resn_exch{"2H16 MQ7"}="H162 MQ7";
$resn_exch{"1H19 MQ7"}="H191 MQ7"; $resn_exch{"2H19 MQ7"}="H192 MQ7"; $resn_exch{"3H19 MQ7"}="H193 MQ7";
$resn_exch{"1H20 MQ7"}="H201 MQ7"; $resn_exch{"2H20 MQ7"}="H202 MQ7"; $resn_exch{"1H21 MQ7"}="H211 MQ7";
$resn_exch{"2H21 MQ7"}="H212 MQ7"; $resn_exch{"1H24 MQ7"}="H241 MQ7"; $resn_exch{"2H24 MQ7"}="H242 MQ7";
$resn_exch{"3H24 MQ7"}="H243 MQ7"; $resn_exch{"1H25 MQ7"}="H251 MQ7"; $resn_exch{"2H25 MQ7"}="H252 MQ7";
$resn_exch{"1H26 MQ7"}="H261 MQ7"; $resn_exch{"2H26 MQ7"}="H262 MQ7"; $resn_exch{"1H29 MQ7"}="H291 MQ7";
$resn_exch{"2H29 MQ7"}="H292 MQ7"; $resn_exch{"3H29 MQ7"}="H293 MQ7"; $resn_exch{"1H30 MQ7"}="H301 MQ7";
$resn_exch{"2H30 MQ7"}="H302 MQ7"; $resn_exch{"1H31 MQ7"}="H311 MQ7"; $resn_exch{"2H31 MQ7"}="H312 MQ7";
$resn_exch{"1H34 MQ7"}="H341 MQ7"; $resn_exch{"2H34 MQ7"}="H342 MQ7"; $resn_exch{"3H34 MQ7"}="H343 MQ7";
$resn_exch{"1H35 MQ7"}="H351 MQ7"; $resn_exch{"2H35 MQ7"}="H352 MQ7"; $resn_exch{"1H36 MQ7"}="H361 MQ7";
$resn_exch{"2H36 MQ7"}="H362 MQ7"; $resn_exch{"1H39 MQ7"}="H391 MQ7"; $resn_exch{"2H39 MQ7"}="H392 MQ7";
$resn_exch{"3H39 MQ7"}="H393 MQ7"; $resn_exch{"1H40 MQ7"}="H401 MQ7"; $resn_exch{"2H40 MQ7"}="H402 MQ7";
$resn_exch{"1H41 MQ7"}="H411 MQ7"; $resn_exch{"2H41 MQ7"}="H412 MQ7"; $resn_exch{"1H44 MQ7"}="H441 MQ7";
$resn_exch{"2H44 MQ7"}="H442 MQ7"; $resn_exch{"3H44 MQ7"}="H443 MQ7"; $resn_exch{"1H45 MQ7"}="H451 MQ7";
$resn_exch{"2H45 MQ7"}="H452 MQ7"; $resn_exch{"3H45 MQ7"}="H453 MQ7"; $resn_exch{"1H2M MQ8"}="H2M1 MQ8";
$resn_exch{"2H2M MQ8"}="H2M2 MQ8"; $resn_exch{"3H2M MQ8"}="H2M3 MQ8"; $resn_exch{"1H6  MQ8"}=" H61 MQ8";
$resn_exch{"1H7  MQ8"}=" H71 MQ8"; $resn_exch{"1H8  MQ8"}=" H81 MQ8"; $resn_exch{"1H9  MQ8"}=" H91 MQ8";
$resn_exch{"1H11 MQ8"}="H111 MQ8"; $resn_exch{"2H11 MQ8"}="H112 MQ8"; $resn_exch{"1H12 MQ8"}="H121 MQ8";
$resn_exch{"1H14 MQ8"}="H141 MQ8"; $resn_exch{"2H14 MQ8"}="H142 MQ8"; $resn_exch{"3H14 MQ8"}="H143 MQ8";
$resn_exch{"1H15 MQ8"}="H151 MQ8"; $resn_exch{"2H15 MQ8"}="H152 MQ8"; $resn_exch{"1H16 MQ8"}="H161 MQ8";
$resn_exch{"2H16 MQ8"}="H162 MQ8"; $resn_exch{"1H17 MQ8"}="H171 MQ8"; $resn_exch{"1H19 MQ8"}="H191 MQ8";
$resn_exch{"2H19 MQ8"}="H192 MQ8"; $resn_exch{"3H19 MQ8"}="H193 MQ8"; $resn_exch{"1H20 MQ8"}="H201 MQ8";
$resn_exch{"2H20 MQ8"}="H202 MQ8"; $resn_exch{"1H21 MQ8"}="H211 MQ8"; $resn_exch{"2H21 MQ8"}="H212 MQ8";
$resn_exch{"1H22 MQ8"}="H221 MQ8"; $resn_exch{"1H24 MQ8"}="H241 MQ8"; $resn_exch{"2H24 MQ8"}="H242 MQ8";
$resn_exch{"3H24 MQ8"}="H243 MQ8"; $resn_exch{"1H25 MQ8"}="H251 MQ8"; $resn_exch{"2H25 MQ8"}="H252 MQ8";
$resn_exch{"1H26 MQ8"}="H261 MQ8"; $resn_exch{"2H26 MQ8"}="H262 MQ8"; $resn_exch{"1H27 MQ8"}="H271 MQ8";
$resn_exch{"1H29 MQ8"}="H291 MQ8"; $resn_exch{"2H29 MQ8"}="H292 MQ8"; $resn_exch{"3H29 MQ8"}="H293 MQ8";
$resn_exch{"1H30 MQ8"}="H301 MQ8"; $resn_exch{"2H30 MQ8"}="H302 MQ8"; $resn_exch{"1H31 MQ8"}="H311 MQ8";
$resn_exch{"2H31 MQ8"}="H312 MQ8"; $resn_exch{"1H32 MQ8"}="H321 MQ8"; $resn_exch{"1H34 MQ8"}="H341 MQ8";
$resn_exch{"2H34 MQ8"}="H342 MQ8"; $resn_exch{"3H34 MQ8"}="H343 MQ8"; $resn_exch{"1H35 MQ8"}="H351 MQ8";
$resn_exch{"2H35 MQ8"}="H352 MQ8"; $resn_exch{"1H36 MQ8"}="H361 MQ8"; $resn_exch{"2H36 MQ8"}="H362 MQ8";
$resn_exch{"1H37 MQ8"}="H371 MQ8"; $resn_exch{"1H39 MQ8"}="H391 MQ8"; $resn_exch{"2H39 MQ8"}="H392 MQ8";
$resn_exch{"3H39 MQ8"}="H393 MQ8"; $resn_exch{"1H40 MQ8"}="H401 MQ8"; $resn_exch{"2H40 MQ8"}="H402 MQ8";
$resn_exch{"1H41 MQ8"}="H411 MQ8"; $resn_exch{"2H41 MQ8"}="H412 MQ8"; $resn_exch{"1H42 MQ8"}="H421 MQ8";
$resn_exch{"1H44 MQ8"}="H441 MQ8"; $resn_exch{"2H44 MQ8"}="H442 MQ8"; $resn_exch{"1H45 MQ8"}="H451 MQ8";
$resn_exch{"2H45 MQ8"}="H452 MQ8"; $resn_exch{"3H45 MQ8"}="H453 MQ8"; $resn_exch{"1H46 MQ8"}="H461 MQ8";
$resn_exch{"2H46 MQ8"}="H462 MQ8"; $resn_exch{"1H47 MQ8"}="H471 MQ8"; $resn_exch{"1H49 MQ8"}="H491 MQ8";
$resn_exch{"2H49 MQ8"}="H492 MQ8"; $resn_exch{"3H49 MQ8"}="H493 MQ8"; $resn_exch{"1H50 MQ8"}="H501 MQ8";
$resn_exch{"2H50 MQ8"}="H502 MQ8"; $resn_exch{"3H50 MQ8"}="H503 MQ8"; $resn_exch{"3H5M MQ9"}="H5M3 MQ9";
$resn_exch{"2H5M MQ9"}="H5M2 MQ9"; $resn_exch{"1H5M MQ9"}="H5M1 MQ9"; $resn_exch{"2H7  MQ9"}=" H72 MQ9";
$resn_exch{"1H7  MQ9"}=" H71 MQ9"; $resn_exch{"3H10 MQ9"}="H103 MQ9"; $resn_exch{"2H10 MQ9"}="H102 MQ9";
$resn_exch{"1H10 MQ9"}="H101 MQ9"; $resn_exch{"2H11 MQ9"}="H112 MQ9"; $resn_exch{"1H11 MQ9"}="H111 MQ9";
$resn_exch{"1H12 MQ9"}="H121 MQ9"; $resn_exch{"2H12 MQ9"}="H122 MQ9"; $resn_exch{"3H15 MQ9"}="H153 MQ9";
$resn_exch{"2H15 MQ9"}="H152 MQ9"; $resn_exch{"1H15 MQ9"}="H151 MQ9"; $resn_exch{"2H16 MQ9"}="H162 MQ9";
$resn_exch{"1H16 MQ9"}="H161 MQ9"; $resn_exch{"2H17 MQ9"}="H172 MQ9"; $resn_exch{"1H17 MQ9"}="H171 MQ9";
$resn_exch{"3H20 MQ9"}="H203 MQ9"; $resn_exch{"2H20 MQ9"}="H202 MQ9"; $resn_exch{"1H20 MQ9"}="H201 MQ9";
$resn_exch{"2H21 MQ9"}="H212 MQ9"; $resn_exch{"1H21 MQ9"}="H211 MQ9"; $resn_exch{"2H22 MQ9"}="H222 MQ9";
$resn_exch{"1H22 MQ9"}="H221 MQ9"; $resn_exch{"3H25 MQ9"}="H253 MQ9"; $resn_exch{"2H25 MQ9"}="H252 MQ9";
$resn_exch{"1H25 MQ9"}="H251 MQ9"; $resn_exch{"2H26 MQ9"}="H262 MQ9"; $resn_exch{"1H26 MQ9"}="H261 MQ9";
$resn_exch{"2H27 MQ9"}="H272 MQ9"; $resn_exch{"1H27 MQ9"}="H271 MQ9"; $resn_exch{"3H30 MQ9"}="H303 MQ9";
$resn_exch{"2H30 MQ9"}="H302 MQ9"; $resn_exch{"1H30 MQ9"}="H301 MQ9"; $resn_exch{"2H31 MQ9"}="H312 MQ9";
$resn_exch{"1H31 MQ9"}="H311 MQ9"; $resn_exch{"1H32 MQ9"}="H321 MQ9"; $resn_exch{"2H32 MQ9"}="H322 MQ9";
$resn_exch{"3H35 MQ9"}="H353 MQ9"; $resn_exch{"2H35 MQ9"}="H352 MQ9"; $resn_exch{"1H35 MQ9"}="H351 MQ9";
$resn_exch{"2H36 MQ9"}="H362 MQ9"; $resn_exch{"1H36 MQ9"}="H361 MQ9"; $resn_exch{"2H37 MQ9"}="H372 MQ9";
$resn_exch{"1H37 MQ9"}="H371 MQ9"; $resn_exch{"3H40 MQ9"}="H403 MQ9"; $resn_exch{"2H40 MQ9"}="H402 MQ9";
$resn_exch{"1H40 MQ9"}="H401 MQ9"; $resn_exch{"2H41 MQ9"}="H412 MQ9"; $resn_exch{"1H41 MQ9"}="H411 MQ9";
$resn_exch{"2H42 MQ9"}="H422 MQ9"; $resn_exch{"1H42 MQ9"}="H421 MQ9"; $resn_exch{"3H45 MQ9"}="H453 MQ9";
$resn_exch{"2H45 MQ9"}="H452 MQ9"; $resn_exch{"1H45 MQ9"}="H451 MQ9"; $resn_exch{"2H46 MQ9"}="H462 MQ9";
$resn_exch{"1H46 MQ9"}="H461 MQ9"; $resn_exch{"2H47 MQ9"}="H472 MQ9"; $resn_exch{"1H47 MQ9"}="H471 MQ9";
$resn_exch{"3H50 MQ9"}="H503 MQ9"; $resn_exch{"2H50 MQ9"}="H502 MQ9"; $resn_exch{"1H50 MQ9"}="H501 MQ9";
$resn_exch{"3H51 MQ9"}="H513 MQ9"; $resn_exch{"2H51 MQ9"}="H512 MQ9"; $resn_exch{"1H51 MQ9"}="H511 MQ9";
$resn_exch{"1H1C MQD"}="H1C1 MQD"; $resn_exch{"2H1C MQD"}="H1C2 MQD"; $resn_exch{"3H1C MQD"}="H1C3 MQD";
$resn_exch{"1HMC MQD"}="HMC1 MQD"; $resn_exch{"2HMC MQD"}="HMC2 MQD"; $resn_exch{"1H3C MQD"}="H3C1 MQD";
$resn_exch{"2H3C MQD"}="H3C2 MQD"; $resn_exch{"1H5C MQD"}="H5C1 MQD"; $resn_exch{"2H5C MQD"}="H5C2 MQD";
$resn_exch{"3H5C MQD"}="H5C3 MQD"; $resn_exch{"1H10 MQI"}="H101 MQI"; $resn_exch{"2H10 MQI"}="H102 MQI";
$resn_exch{"3H10 MQI"}="H103 MQI"; $resn_exch{"1H2  MQQ"}="H21A MQQ"; $resn_exch{"2H2  MQQ"}=" H22 MQQ";
$resn_exch{"1H3  MQQ"}="H31A MQQ"; $resn_exch{"2H3  MQQ"}=" H32 MQQ"; $resn_exch{"1H4  MQQ"}=" H41 MQQ";
$resn_exch{"2H4  MQQ"}=" H42 MQQ"; $resn_exch{"1H5  MQQ"}="H51A MQQ"; $resn_exch{"2H5  MQQ"}=" H52 MQQ";
$resn_exch{"1HB  MQQ"}=" HB1 MQQ"; $resn_exch{"2HB  MQQ"}=" HB2 MQQ"; $resn_exch{"1HG  MQQ"}=" HG1 MQQ";
$resn_exch{"2HG  MQQ"}=" HG2 MQQ"; $resn_exch{"1HM  MQQ"}=" HM1 MQQ"; $resn_exch{"2HM  MQQ"}=" HM2 MQQ";
$resn_exch{"1HB3 MQQ"}="HB31 MQQ"; $resn_exch{"2HB3 MQQ"}="HB32 MQQ"; $resn_exch{"1HB2 MQQ"}="HB21 MQQ";
$resn_exch{"2HB2 MQQ"}="HB22 MQQ"; $resn_exch{"1HT  MQQ"}=" HT1 MQQ"; $resn_exch{"2HT  MQQ"}=" HT2 MQQ";
$resn_exch{"1H2  MQU"}=" H21 MQU"; $resn_exch{"2H2  MQU"}=" H22 MQU"; $resn_exch{"1H14 MQU"}="H141 MQU";
$resn_exch{"2H14 MQU"}="H142 MQU"; $resn_exch{"3H14 MQU"}="H143 MQU"; $resn_exch{"1H5  MQU"}=" H51 MQU";
$resn_exch{"2H5  MQU"}=" H52 MQU"; $resn_exch{"1H18 MQU"}="H181 MQU"; $resn_exch{"2H18 MQU"}="H182 MQU";
$resn_exch{"3H18 MQU"}="H183 MQU"; $resn_exch{"1H25 MQU"}="H251 MQU"; $resn_exch{"2H25 MQU"}="H252 MQU";
$resn_exch{"1H26 MQU"}="H261 MQU"; $resn_exch{"2H26 MQU"}="H262 MQU"; $resn_exch{"1H10 MR0"}="H101 MR0";
$resn_exch{"2H10 MR0"}="H102 MR0"; $resn_exch{"1H13 MR0"}="H131 MR0"; $resn_exch{"2H13 MR0"}="H132 MR0";
$resn_exch{"1H17 MR0"}="H171 MR0"; $resn_exch{"2H17 MR0"}="H172 MR0"; $resn_exch{"1H9  MR0"}=" H91 MR0";
$resn_exch{"2H9  MR0"}=" H92 MR0"; $resn_exch{"1H11 MR0"}="H111 MR0"; $resn_exch{"2H11 MR0"}="H112 MR0";
$resn_exch{"1H12 MR0"}="H121 MR0"; $resn_exch{"2H12 MR0"}="H122 MR0"; $resn_exch{"3H12 MR0"}="H123 MR0";
$resn_exch{"1H14 MR0"}="H141 MR0"; $resn_exch{"2H14 MR0"}="H142 MR0"; $resn_exch{"3H14 MR0"}="H143 MR0";
$resn_exch{"1H18 MR0"}="H181 MR0"; $resn_exch{"2H18 MR0"}="H182 MR0"; $resn_exch{"1H19 MR0"}="H191 MR0";
$resn_exch{"2H19 MR0"}="H192 MR0"; $resn_exch{"1HO  MR0"}=" HO1 MR0"; $resn_exch{"2HO  MR0"}="HO2A MR0";
$resn_exch{"3HO  MR0"}=" HO3 MR0"; $resn_exch{"1H7A MR0"}="H7A1 MR0"; $resn_exch{"2H7A MR0"}="H7A2 MR0";
$resn_exch{"1H38 MR1"}="H381 MR1"; $resn_exch{"2H38 MR1"}="H382 MR1"; $resn_exch{"1H30 MR1"}="H301 MR1";
$resn_exch{"2H30 MR1"}="H302 MR1"; $resn_exch{"1HN  MR1"}=" HN1 MR1"; $resn_exch{"2HN  MR1"}=" HN2 MR1";
$resn_exch{"1HAA MR3"}="HAA1 MR3"; $resn_exch{"2HAA MR3"}="HAA2 MR3"; $resn_exch{"3HAA MR3"}="HAA3 MR3";
$resn_exch{"1HAA MR4"}="HAA1 MR4"; $resn_exch{"2HAA MR4"}="HAA2 MR4"; $resn_exch{"3HAA MR4"}="HAA3 MR4";
$resn_exch{"1HAB MR4"}="HAB1 MR4"; $resn_exch{"2HAB MR4"}="HAB2 MR4"; $resn_exch{"3HAB MR4"}="HAB3 MR4";
$resn_exch{"1HAA MR6"}="HAA1 MR6"; $resn_exch{"2HAA MR6"}="HAA2 MR6"; $resn_exch{"3HAA MR6"}="HAA3 MR6";
$resn_exch{"1HAB MR6"}="HAB1 MR6"; $resn_exch{"2HAB MR6"}="HAB2 MR6"; $resn_exch{"3HAB MR6"}="HAB3 MR6";
$resn_exch{"1H26 MR9"}="H261 MR9"; $resn_exch{"2H26 MR9"}="H262 MR9"; $resn_exch{"1H27 MR9"}="H271 MR9";
$resn_exch{"2H27 MR9"}="H272 MR9"; $resn_exch{"1H28 MR9"}="H281 MR9"; $resn_exch{"2H28 MR9"}="H282 MR9";
$resn_exch{"1H29 MR9"}="H291 MR9"; $resn_exch{"2H29 MR9"}="H292 MR9"; $resn_exch{"1H16 MR9"}="H161 MR9";
$resn_exch{"2H16 MR9"}="H162 MR9"; $resn_exch{"3H16 MR9"}="H163 MR9"; $resn_exch{"1H18 MR9"}="H181 MR9";
$resn_exch{"2H18 MR9"}="H182 MR9"; $resn_exch{"3H18 MR9"}="H183 MR9"; $resn_exch{"1H16 MRA"}="H161 MRA";
$resn_exch{"2H16 MRA"}="H162 MRA"; $resn_exch{"1H17 MRA"}="H171 MRA"; $resn_exch{"2H17 MRA"}="H172 MRA";
$resn_exch{"1H4  MRC"}=" H41 MRC"; $resn_exch{"2H4  MRC"}=" H42 MRC"; $resn_exch{"1H9  MRC"}=" H91 MRC";
$resn_exch{"2H9  MRC"}=" H92 MRC"; $resn_exch{"1H14 MRC"}="H141 MRC"; $resn_exch{"2H14 MRC"}="H142 MRC";
$resn_exch{"3H14 MRC"}="H143 MRC"; $resn_exch{"1H15 MRC"}="H151 MRC"; $resn_exch{"2H15 MRC"}="H152 MRC";
$resn_exch{"3H15 MRC"}="H153 MRC"; $resn_exch{"1H16 MRC"}="H161 MRC"; $resn_exch{"2H16 MRC"}="H162 MRC";
$resn_exch{"1HOP MRC"}="HOP1 MRC"; $resn_exch{"1H2* MRC"}="H2'1 MRC"; $resn_exch{"2H2* MRC"}="H2'2 MRC";
$resn_exch{"1H3* MRC"}="H3'1 MRC"; $resn_exch{"2H3* MRC"}="H3'2 MRC"; $resn_exch{"1H4* MRC"}="H4'1 MRC";
$resn_exch{"2H4* MRC"}="H4'2 MRC"; $resn_exch{"1H5* MRC"}="H5'1 MRC"; $resn_exch{"2H5* MRC"}="H5'2 MRC";
$resn_exch{"1H6* MRC"}="H6'1 MRC"; $resn_exch{"2H6* MRC"}="H6'2 MRC"; $resn_exch{"1H7* MRC"}="H7'1 MRC";
$resn_exch{"2H7* MRC"}="H7'2 MRC"; $resn_exch{"1H8* MRC"}="H8'1 MRC"; $resn_exch{"2H8* MRC"}="H8'2 MRC";
$resn_exch{"1H9* MRC"}="H9'1 MRC"; $resn_exch{"2H9* MRC"}="H9'2 MRC"; $resn_exch{"1H17 MRC"}="H171 MRC";
$resn_exch{"2H17 MRC"}="H172 MRC"; $resn_exch{"3H17 MRC"}="H173 MRC"; $resn_exch{"1H1C MRD"}="H1C1 MRD";
$resn_exch{"2H1C MRD"}="H1C2 MRD"; $resn_exch{"3H1C MRD"}="H1C3 MRD"; $resn_exch{"1HMC MRD"}="HMC1 MRD";
$resn_exch{"2HMC MRD"}="HMC2 MRD"; $resn_exch{"3HMC MRD"}="HMC3 MRD"; $resn_exch{"1H3C MRD"}="H3C1 MRD";
$resn_exch{"2H3C MRD"}="H3C2 MRD"; $resn_exch{"1H5C MRD"}="H5C1 MRD"; $resn_exch{"2H5C MRD"}="H5C2 MRD";
$resn_exch{"3H5C MRD"}="H5C3 MRD"; $resn_exch{"1HHX MRE"}="HHX1 MRE"; $resn_exch{"2HHX MRE"}="HHX2 MRE";
$resn_exch{"3HHX MRE"}="HHX3 MRE"; $resn_exch{"1HHZ MRE"}="HHZ1 MRE"; $resn_exch{"2HHZ MRE"}="HHZ2 MRE";
$resn_exch{"3HHZ MRE"}="HHZ3 MRE"; $resn_exch{"2HOP MRG"}="HOP2 MRG"; $resn_exch{"1H2  MRG"}=" H21 MRG";
$resn_exch{"1H21 MRG"}="H211 MRG"; $resn_exch{"2H21 MRG"}="H212 MRG"; $resn_exch{"1H22 MRG"}="H221 MRG";
$resn_exch{"2H22 MRG"}="H222 MRG"; $resn_exch{"1H23 MRG"}="H231 MRG"; $resn_exch{"2H23 MRG"}="H232 MRG";
$resn_exch{"1H2* MRG"}=" H2' MRG"; $resn_exch{"2H2* MRG"}="H2'' MRG"; $resn_exch{"1H5* MRG"}=" H5' MRG";
$resn_exch{"2H5* MRG"}="H5'' MRG"; $resn_exch{" H4* MRG"}=" H4' MRG"; $resn_exch{" H1* MRG"}=" H1' MRG";
$resn_exch{" H3* MRG"}=" H3' MRG"; $resn_exch{"H3T  MRG"}="HO3' MRG"; $resn_exch{"3HOP MRG"}="HOP3 MRG";
$resn_exch{"1H22 MRK"}="H221 MRK"; $resn_exch{"2H22 MRK"}="H222 MRK"; $resn_exch{"3H22 MRK"}="H223 MRK";
$resn_exch{"1H23 MRK"}="H231 MRK"; $resn_exch{"2H23 MRK"}="H232 MRK"; $resn_exch{"1HNB MRM"}="HNB1 MRM";
$resn_exch{"2HNB MRM"}="HNB2 MRM"; $resn_exch{"1HM2 MRM"}="HM21 MRM"; $resn_exch{"2HM2 MRM"}="HM22 MRM";
$resn_exch{"1HM5 MRM"}="HM51 MRM"; $resn_exch{"2HM5 MRM"}="HM52 MRM"; $resn_exch{"1HM3 MRM"}="HM31 MRM";
$resn_exch{"2HM3 MRM"}="HM32 MRM"; $resn_exch{"1HM4 MRM"}="HM41 MRM"; $resn_exch{"2HM4 MRM"}="HM42 MRM";
$resn_exch{"1H6  MRP"}=" H61 MRP"; $resn_exch{"2H6  MRP"}=" H62 MRP"; $resn_exch{"3H6  MRP"}=" H63 MRP";
$resn_exch{"1H7  MRP"}=" H71 MRP"; $resn_exch{"2H7  MRP"}=" H72 MRP"; $resn_exch{"3H7  MRP"}=" H73 MRP";
$resn_exch{" H3* MRR"}=" H3' MRR"; $resn_exch{" H2* MRR"}=" H2' MRR"; $resn_exch{" H1* MRR"}=" H1' MRR";
$resn_exch{"1HN6 MRR"}="HN61 MRR"; $resn_exch{"2HN6 MRR"}="HN62 MRR"; $resn_exch{" H4* MRR"}=" H4' MRR";
$resn_exch{"1H5* MRR"}="H5'1 MRR"; $resn_exch{"2H5* MRR"}="H5'2 MRR"; $resn_exch{"1H20 MRR"}="H201 MRR";
$resn_exch{"2H20 MRR"}="H202 MRR"; $resn_exch{"1H18 MRR"}="H181 MRR"; $resn_exch{"2H18 MRR"}="H182 MRR";
$resn_exch{"3H18 MRR"}="H183 MRR"; $resn_exch{"1H19 MRR"}="H191 MRR"; $resn_exch{"2H19 MRR"}="H192 MRR";
$resn_exch{"3H19 MRR"}="H193 MRR"; $resn_exch{"1H17 MRR"}="H171 MRR"; $resn_exch{"2H17 MRR"}="H172 MRR";
$resn_exch{"1H16 MRR"}="H161 MRR"; $resn_exch{"2H16 MRR"}="H162 MRR"; $resn_exch{"1H1  MRR"}=" H11 MRR";
$resn_exch{"2H1  MRR"}=" H12 MRR"; $resn_exch{"1H2  MRR"}=" H21 MRR"; $resn_exch{"2H2  MRR"}=" H22 MRR";
$resn_exch{"1H13 MRR"}="H131 MRR"; $resn_exch{"2H13 MRR"}="H132 MRR"; $resn_exch{"3H13 MRR"}="H133 MRR";
$resn_exch{"1H3  MRR"}=" H31 MRR"; $resn_exch{"2H3  MRR"}=" H32 MRR"; $resn_exch{"1H4  MRR"}=" H41 MRR";
$resn_exch{"2H4  MRR"}=" H42 MRR"; $resn_exch{"1H5  MRR"}=" H51 MRR"; $resn_exch{"2H5  MRR"}=" H52 MRR";
$resn_exch{"1H6  MRR"}=" H61 MRR"; $resn_exch{"2H6  MRR"}=" H62 MRR"; $resn_exch{"1H7  MRR"}=" H71 MRR";
$resn_exch{"2H7  MRR"}=" H72 MRR"; $resn_exch{"1H8  MRR"}=" H81 MRR"; $resn_exch{"2H8  MRR"}=" H82 MRR";
$resn_exch{"1H9  MRR"}=" H91 MRR"; $resn_exch{"2H9  MRR"}=" H92 MRR"; $resn_exch{"1H10 MRR"}="H101 MRR";
$resn_exch{"2H10 MRR"}="H102 MRR"; $resn_exch{"1H11 MRR"}="H111 MRR"; $resn_exch{"2H11 MRR"}="H112 MRR";
$resn_exch{"1H12 MRR"}="H121 MRR"; $resn_exch{"2H12 MRR"}="H122 MRR"; $resn_exch{"1H14 MRR"}="H141 MRR";
$resn_exch{"2H14 MRR"}="H142 MRR"; $resn_exch{"1H15 MRR"}="H151 MRR"; $resn_exch{"2H15 MRR"}="H152 MRR";
$resn_exch{"3H15 MRR"}="H153 MRR"; $resn_exch{" H3* MRS"}=" H3' MRS"; $resn_exch{" H2* MRS"}=" H2' MRS";
$resn_exch{" H1* MRS"}=" H1' MRS"; $resn_exch{"1HN6 MRS"}="HN61 MRS"; $resn_exch{"2HN6 MRS"}="HN62 MRS";
$resn_exch{" H4* MRS"}=" H4' MRS"; $resn_exch{"1H5* MRS"}="H5'1 MRS"; $resn_exch{"2H5* MRS"}="H5'2 MRS";
$resn_exch{"1H20 MRS"}="H201 MRS"; $resn_exch{"2H20 MRS"}="H202 MRS"; $resn_exch{"1H18 MRS"}="H181 MRS";
$resn_exch{"2H18 MRS"}="H182 MRS"; $resn_exch{"3H18 MRS"}="H183 MRS"; $resn_exch{"1H19 MRS"}="H191 MRS";
$resn_exch{"2H19 MRS"}="H192 MRS"; $resn_exch{"3H19 MRS"}="H193 MRS"; $resn_exch{"1H17 MRS"}="H171 MRS";
$resn_exch{"2H17 MRS"}="H172 MRS"; $resn_exch{"1H16 MRS"}="H161 MRS"; $resn_exch{"2H16 MRS"}="H162 MRS";
$resn_exch{"1H1  MRS"}=" H11 MRS"; $resn_exch{"2H1  MRS"}=" H12 MRS"; $resn_exch{"1H2  MRS"}=" H21 MRS";
$resn_exch{"2H2  MRS"}=" H22 MRS"; $resn_exch{"1H13 MRS"}="H131 MRS"; $resn_exch{"2H13 MRS"}="H132 MRS";
$resn_exch{"3H13 MRS"}="H133 MRS"; $resn_exch{"1H3  MRS"}=" H31 MRS"; $resn_exch{"2H3  MRS"}=" H32 MRS";
$resn_exch{"1H4  MRS"}=" H41 MRS"; $resn_exch{"2H4  MRS"}=" H42 MRS"; $resn_exch{"1H5  MRS"}=" H51 MRS";
$resn_exch{"2H5  MRS"}=" H52 MRS"; $resn_exch{"1H6  MRS"}=" H61 MRS"; $resn_exch{"2H6  MRS"}=" H62 MRS";
$resn_exch{"1H7  MRS"}=" H71 MRS"; $resn_exch{"2H7  MRS"}=" H72 MRS"; $resn_exch{"1H8  MRS"}=" H81 MRS";
$resn_exch{"2H8  MRS"}=" H82 MRS"; $resn_exch{"1H9  MRS"}=" H91 MRS"; $resn_exch{"2H9  MRS"}=" H92 MRS";
$resn_exch{"1H10 MRS"}="H101 MRS"; $resn_exch{"2H10 MRS"}="H102 MRS"; $resn_exch{"1H11 MRS"}="H111 MRS";
$resn_exch{"2H11 MRS"}="H112 MRS"; $resn_exch{"1H12 MRS"}="H121 MRS"; $resn_exch{"2H12 MRS"}="H122 MRS";
$resn_exch{"1H14 MRS"}="H141 MRS"; $resn_exch{"2H14 MRS"}="H142 MRS"; $resn_exch{"1H15 MRS"}="H151 MRS";
$resn_exch{"2H15 MRS"}="H152 MRS"; $resn_exch{"3H15 MRS"}="H153 MRS"; $resn_exch{"1H1  MRU"}=" H11 MRU";
$resn_exch{"2H1  MRU"}=" H12 MRU"; $resn_exch{"1H5  MRU"}=" H51 MRU"; $resn_exch{"2H5  MRU"}=" H52 MRU";
$resn_exch{"1H6  MRU"}=" H61 MRU"; $resn_exch{"2H6  MRU"}=" H62 MRU"; $resn_exch{"3H6  MRU"}=" H63 MRU";
$resn_exch{"1H20 MRX"}="H201 MRX"; $resn_exch{"2H20 MRX"}="H202 MRX"; $resn_exch{"1H15 MRX"}="H151 MRX";
$resn_exch{"2H15 MRX"}="H152 MRX"; $resn_exch{"1H14 MRX"}="H141 MRX"; $resn_exch{"2H14 MRX"}="H142 MRX";
$resn_exch{"1H13 MRX"}="H131 MRX"; $resn_exch{"2H13 MRX"}="H132 MRX"; $resn_exch{"1H12 MRX"}="H121 MRX";
$resn_exch{"2H12 MRX"}="H122 MRX"; $resn_exch{"1HAA MRY"}="HAA1 MRY"; $resn_exch{"2HAA MRY"}="HAA2 MRY";
$resn_exch{"1HAG MRY"}="HAG1 MRY"; $resn_exch{"2HAG MRY"}="HAG2 MRY"; $resn_exch{"1HG  MS1"}=" HG1 MS1";
$resn_exch{"2HG  MS1"}=" HG2 MS1"; $resn_exch{"1HB  MS1"}=" HB1 MS1"; $resn_exch{"2HB  MS1"}=" HB2 MS1";
$resn_exch{"1H5  MS1"}=" H51 MS1"; $resn_exch{"2H5  MS1"}=" H52 MS1"; $resn_exch{"1HN1 MS1"}="HN11 MS1";
$resn_exch{"2HN1 MS1"}="HN12 MS1"; $resn_exch{"1H9  MS1"}=" H91 MS1"; $resn_exch{"2H9  MS1"}=" H92 MS1";
$resn_exch{"1H2F MS1"}="H2F1 MS1"; $resn_exch{"2H2F MS1"}="H2F2 MS1"; $resn_exch{"1H6  MS2"}=" H61 MS2";
$resn_exch{"2H6  MS2"}=" H62 MS2"; $resn_exch{"3H6  MS2"}=" H63 MS2"; $resn_exch{"1H12 MS2"}="H121 MS2";
$resn_exch{"2H12 MS2"}="H122 MS2"; $resn_exch{"3H12 MS2"}="H123 MS2"; $resn_exch{"1H13 MS3"}="H131 MS3";
$resn_exch{"2H13 MS3"}="H132 MS3"; $resn_exch{"1H24 MS3"}="H241 MS3"; $resn_exch{"2H24 MS3"}="H242 MS3";
$resn_exch{"3H24 MS3"}="H243 MS3"; $resn_exch{"1H25 MS3"}="H251 MS3"; $resn_exch{"2H25 MS3"}="H252 MS3";
$resn_exch{"3H25 MS3"}="H253 MS3"; $resn_exch{"1H29 MS3"}="H291 MS3"; $resn_exch{"2H29 MS3"}="H292 MS3";
$resn_exch{"1H31 MS3"}="H311 MS3"; $resn_exch{"2H31 MS3"}="H312 MS3"; $resn_exch{"1H39 MS3"}="H391 MS3";
$resn_exch{"2H39 MS3"}="H392 MS3"; $resn_exch{"3H39 MS3"}="H393 MS3"; $resn_exch{"1H40 MS3"}="H401 MS3";
$resn_exch{"2H40 MS3"}="H402 MS3"; $resn_exch{"3H40 MS3"}="H403 MS3"; $resn_exch{"1H44 MS3"}="H441 MS3";
$resn_exch{"2H44 MS3"}="H442 MS3"; $resn_exch{" HN  MSA"}=" H   MSA"; $resn_exch{"1HN  MSA"}=" HN1 MSA";
$resn_exch{"2HN  MSA"}=" HN2 MSA"; $resn_exch{"3HN  MSA"}=" HN3 MSA"; $resn_exch{"1HG  MSA"}=" HG1 MSA";
$resn_exch{"2HG  MSA"}=" HG2 MSA"; $resn_exch{"3HG  MSA"}=" HG3 MSA"; $resn_exch{"1HH  MSB"}=" HH1 MSB";
$resn_exch{"2HH  MSB"}=" HH2 MSB"; $resn_exch{"3HH  MSB"}=" HH3 MSB"; $resn_exch{"1H01 MSC"}="H011 MSC";
$resn_exch{"1H02 MSC"}="H021 MSC"; $resn_exch{"1H03 MSC"}="H031 MSC"; $resn_exch{"1H05 MSC"}="H051 MSC";
$resn_exch{"2H05 MSC"}="H052 MSC"; $resn_exch{"1H07 MSC"}="H071 MSC"; $resn_exch{"1H08 MSC"}="H081 MSC";
$resn_exch{"1H09 MSC"}="H091 MSC"; $resn_exch{"1H10 MSC"}="H101 MSC"; $resn_exch{"1H11 MSC"}="H111 MSC";
$resn_exch{"1H14 MSC"}="H141 MSC"; $resn_exch{"1H15 MSC"}="H151 MSC"; $resn_exch{"1H17 MSC"}="H171 MSC";
$resn_exch{"1H18 MSC"}="H181 MSC"; $resn_exch{"1H19 MSC"}="H191 MSC"; $resn_exch{"1H20 MSC"}="H201 MSC";
$resn_exch{"1H22 MSC"}="H221 MSC"; $resn_exch{"2H22 MSC"}="H222 MSC"; $resn_exch{"1H23 MSC"}="H231 MSC";
$resn_exch{"1H24 MSC"}="H241 MSC"; $resn_exch{"1H31 MSC"}="H311 MSC"; $resn_exch{"1H32 MSC"}="H321 MSC";
$resn_exch{"1H33 MSC"}="H331 MSC"; $resn_exch{"1H35 MSC"}="H351 MSC"; $resn_exch{"2H35 MSC"}="H352 MSC";
$resn_exch{"1H37 MSC"}="H371 MSC"; $resn_exch{"1H38 MSC"}="H381 MSC"; $resn_exch{"1H39 MSC"}="H391 MSC";
$resn_exch{"1H40 MSC"}="H401 MSC"; $resn_exch{"1H41 MSC"}="H411 MSC"; $resn_exch{"1H44 MSC"}="H441 MSC";
$resn_exch{"1H45 MSC"}="H451 MSC"; $resn_exch{"2H45 MSC"}="H452 MSC"; $resn_exch{"1H48 MSC"}="H481 MSC";
$resn_exch{"1H49 MSC"}="H491 MSC"; $resn_exch{"1H50 MSC"}="H501 MSC"; $resn_exch{"NH11 MSD"}="H11N MSD";
$resn_exch{"CH12 MSD"}="H12C MSD"; $resn_exch{"1H13 MSD"}="H131 MSD"; $resn_exch{"2H13 MSD"}="H132 MSD";
$resn_exch{"CH14 MSD"}="H14C MSD"; $resn_exch{"CH18 MSD"}="H18C MSD"; $resn_exch{"CH19 MSD"}="H19C MSD";
$resn_exch{"OH22 MSD"}="H22O MSD"; $resn_exch{"1H25 MSD"}="H251 MSD"; $resn_exch{"2H25 MSD"}="H252 MSD";
$resn_exch{"3H25 MSD"}="H253 MSD"; $resn_exch{"1H44 MSD"}="H441 MSD"; $resn_exch{"2H44 MSD"}="H442 MSD";
$resn_exch{"3H44 MSD"}="H443 MSD"; $resn_exch{"1HN  MSE"}=" H   MSE"; $resn_exch{"2HN  MSE"}=" HN2 MSE";
$resn_exch{"1HB  MSE"}=" HB2 MSE"; $resn_exch{"2HB  MSE"}=" HB3 MSE"; $resn_exch{"1HG  MSE"}=" HG2 MSE";
$resn_exch{"2HG  MSE"}=" HG3 MSE"; $resn_exch{"1HE  MSE"}=" HE1 MSE"; $resn_exch{"2HE  MSE"}=" HE2 MSE";
$resn_exch{"3HE  MSE"}=" HE3 MSE"; $resn_exch{"1H2  MSF"}=" H21 MSF"; $resn_exch{"2H2  MSF"}=" H22 MSF";
$resn_exch{"1H4  MSF"}=" H41 MSF"; $resn_exch{"2H4  MSF"}=" H42 MSF"; $resn_exch{"3H4  MSF"}=" H43 MSF";
$resn_exch{" H5* MSG"}=" H5' MSG"; $resn_exch{"1H5* MSG"}="H5'1 MSG"; $resn_exch{"2H5* MSG"}="H5'2 MSG";
$resn_exch{" H4* MSG"}=" H4' MSG"; $resn_exch{" H1* MSG"}=" H1' MSG"; $resn_exch{"1H1  MSG"}=" H11 MSG";
$resn_exch{"2H1  MSG"}=" H12 MSG"; $resn_exch{"3H1  MSG"}=" H13 MSG"; $resn_exch{"1HN2 MSG"}="HN21 MSG";
$resn_exch{"2HN2 MSG"}="HN22 MSG"; $resn_exch{" H2* MSG"}=" H2' MSG"; $resn_exch{" H3* MSG"}=" H3' MSG";
$resn_exch{"1HC4 MSH"}="HC41 MSH"; $resn_exch{"2HC4 MSH"}="HC42 MSH"; $resn_exch{"3HC4 MSH"}="HC43 MSH";
$resn_exch{"1HC5 MSH"}="HC51 MSH"; $resn_exch{"2HC5 MSH"}="HC52 MSH"; $resn_exch{"3HC5 MSH"}="HC53 MSH";
$resn_exch{"1HC6 MSH"}="HC61 MSH"; $resn_exch{"2HC6 MSH"}="HC62 MSH"; $resn_exch{"1HC7 MSH"}="HC71 MSH";
$resn_exch{"1HC8 MSH"}="HC81 MSH"; $resn_exch{"2HC8 MSH"}="HC82 MSH"; $resn_exch{"1HC9 MSH"}="HC91 MSH";
$resn_exch{"2HC9 MSH"}="HC92 MSH"; $resn_exch{"1H10 MSH"}="H101 MSH"; $resn_exch{"2H10 MSH"}="H102 MSH";
$resn_exch{"3H10 MSH"}="H103 MSH"; $resn_exch{"1H10 MSI"}="H101 MSI"; $resn_exch{"2H10 MSI"}="H102 MSI";
$resn_exch{"1H11 MSI"}="H111 MSI"; $resn_exch{"2H11 MSI"}="H112 MSI"; $resn_exch{"1H12 MSI"}="H121 MSI";
$resn_exch{"2H12 MSI"}="H122 MSI"; $resn_exch{"1H13 MSI"}="H131 MSI"; $resn_exch{"2H13 MSI"}="H132 MSI";
$resn_exch{"1HC1 MSI"}="HC11 MSI"; $resn_exch{"2HC1 MSI"}="HC12 MSI"; $resn_exch{"3HC1 MSI"}="HC13 MSI";
$resn_exch{"1HN  MSL"}=" H   MSL"; $resn_exch{"2HN  MSL"}=" H2  MSL"; $resn_exch{"1HB  MSL"}=" HB2 MSL";
$resn_exch{"2HB  MSL"}=" HB3 MSL"; $resn_exch{"1HG  MSL"}=" HG2 MSL"; $resn_exch{"2HG  MSL"}=" HG3 MSL";
$resn_exch{" HNE MSL"}=" HE  MSL"; $resn_exch{"1HE  MSL"}=" HE1 MSL"; $resn_exch{"2HE  MSL"}=" HE2 MSL";
$resn_exch{"3HE  MSL"}=" HE3 MSL"; $resn_exch{"1HB1 MSM"}="HB11 MSM"; $resn_exch{"2HB1 MSM"}="HB12 MSM";
$resn_exch{"3HB1 MSM"}="HB13 MSM"; $resn_exch{"1HD1 MSM"}="HD11 MSM"; $resn_exch{"2HD1 MSM"}="HD12 MSM";
$resn_exch{"3HD1 MSM"}="HD13 MSM"; $resn_exch{"1HN5 MSN"}="HN51 MSN"; $resn_exch{"2HN5 MSN"}="HN52 MSN";
$resn_exch{"1H7  MSN"}=" H71 MSN"; $resn_exch{"2H7  MSN"}=" H72 MSN"; $resn_exch{"3H7  MSN"}=" H73 MSN";
$resn_exch{"1HN  MSO"}=" H   MSO"; $resn_exch{"2HN  MSO"}=" H2  MSO"; $resn_exch{"1HCB MSO"}=" HB2 MSO";
$resn_exch{"2HCB MSO"}=" HB3 MSO"; $resn_exch{"1HCG MSO"}=" HG2 MSO"; $resn_exch{"2HCG MSO"}=" HG3 MSO";
$resn_exch{"1HC  MSO"}=" HE1 MSO"; $resn_exch{"2HC  MSO"}=" HE2 MSO"; $resn_exch{"3HC  MSO"}=" HE3 MSO";
$resn_exch{"1HN2 MSP"}="HN21 MSP"; $resn_exch{"2HN2 MSP"}="HN22 MSP"; $resn_exch{"1HB  MSP"}=" HB1 MSP";
$resn_exch{"2HB  MSP"}=" HB2 MSP"; $resn_exch{"1HG  MSP"}=" HG1 MSP"; $resn_exch{"2HG  MSP"}=" HG2 MSP";
$resn_exch{"1HE  MSP"}=" HE1 MSP"; $resn_exch{"2HE  MSP"}=" HE2 MSP"; $resn_exch{"3HE  MSP"}=" HE3 MSP";
$resn_exch{"1H5* MSP"}="H5'1 MSP"; $resn_exch{"2H5* MSP"}="H5'2 MSP"; $resn_exch{" H4* MSP"}=" H4' MSP";
$resn_exch{" H3* MSP"}=" H3' MSP"; $resn_exch{" H2* MSP"}=" H2' MSP"; $resn_exch{" H1* MSP"}=" H1' MSP";
$resn_exch{"1HN6 MSP"}="HN61 MSP"; $resn_exch{"2HN6 MSP"}="HN62 MSP"; $resn_exch{"1H2  MSQ"}=" H21 MSQ";
$resn_exch{"1H7  MSQ"}=" H71 MSQ"; $resn_exch{"1H10 MSQ"}="H101 MSQ"; $resn_exch{"1H12 MSQ"}="H121 MSQ";
$resn_exch{"1H15 MSQ"}="H151 MSQ"; $resn_exch{"1H17 MSQ"}="H171 MSQ"; $resn_exch{"1H18 MSQ"}="H181 MSQ";
$resn_exch{"1H19 MSQ"}="H191 MSQ"; $resn_exch{"1H22 MSQ"}="H221 MSQ"; $resn_exch{"2H22 MSQ"}="H222 MSQ";
$resn_exch{"3H22 MSQ"}="H223 MSQ"; $resn_exch{"1H27 MSQ"}="H271 MSQ"; $resn_exch{"2H27 MSQ"}="H272 MSQ";
$resn_exch{"3H27 MSQ"}="H273 MSQ"; $resn_exch{"1H32 MSQ"}="H321 MSQ"; $resn_exch{"2H32 MSQ"}="H322 MSQ";
$resn_exch{"3H32 MSQ"}="H323 MSQ"; $resn_exch{"1H2N MSS"}="H2N1 MSS"; $resn_exch{"2H2N MSS"}="H2N2 MSS";
$resn_exch{"1H4' MSS"}="H4'1 MSS"; $resn_exch{"2H4' MSS"}="H4'2 MSS"; $resn_exch{"1HP  MSS"}=" HP1 MSS";
$resn_exch{"2HP  MSS"}=" HP2 MSS"; $resn_exch{"3H8  MST"}=" H83 MST"; $resn_exch{"2H8  MST"}=" H82 MST";
$resn_exch{"1H8  MST"}=" H81 MST"; $resn_exch{"3H11 MST"}="H113 MST"; $resn_exch{"2H11 MST"}="H112 MST";
$resn_exch{"1H11 MST"}="H111 MST"; $resn_exch{"3H12 MST"}="H123 MST"; $resn_exch{"2H12 MST"}="H122 MST";
$resn_exch{"1H12 MST"}="H121 MST"; $resn_exch{"3H13 MST"}="H133 MST"; $resn_exch{"2H13 MST"}="H132 MST";
$resn_exch{"1H13 MST"}="H131 MST"; $resn_exch{"2H15 MST"}="H152 MST"; $resn_exch{"1H15 MST"}="H151 MST";
$resn_exch{"3H16 MST"}="H163 MST"; $resn_exch{"2H16 MST"}="H162 MST"; $resn_exch{"1H16 MST"}="H161 MST";
$resn_exch{"1H2  MSU"}=" H21 MSU"; $resn_exch{"2H2  MSU"}=" H22 MSU"; $resn_exch{"1H3  MSU"}=" H31 MSU";
$resn_exch{"2H3  MSU"}=" H32 MSU"; $resn_exch{"1HT  MSU"}=" HT1 MSU"; $resn_exch{"2HT  MSU"}=" HT2 MSU";
$resn_exch{"3HT  MSU"}=" HT3 MSU"; $resn_exch{"1HA2 MT1"}="HA21 MT1"; $resn_exch{"2HA2 MT1"}="HA22 MT1";
$resn_exch{"1HA4 MT1"}="HA41 MT1"; $resn_exch{"2HA4 MT1"}="HA42 MT1"; $resn_exch{"1H9  MT1"}=" H91 MT1";
$resn_exch{"2H9  MT1"}=" H92 MT1"; $resn_exch{"1HM  MT1"}=" HM1 MT1"; $resn_exch{"2HM  MT1"}=" HM2 MT1";
$resn_exch{"3HM  MT1"}=" HM3 MT1"; $resn_exch{"1HB  MT1"}=" HB1 MT1"; $resn_exch{"2HB  MT1"}=" HB2 MT1";
$resn_exch{"1HG  MT1"}=" HG1 MT1"; $resn_exch{"2HG  MT1"}=" HG2 MT1"; $resn_exch{"1HCS MTA"}="HCS1 MTA";
$resn_exch{"2HCS MTA"}="HCS2 MTA"; $resn_exch{"3HCS MTA"}="HCS3 MTA"; $resn_exch{"1H5* MTA"}="H5'1 MTA";
$resn_exch{"2H5* MTA"}="H5'2 MTA"; $resn_exch{" H4* MTA"}=" H4' MTA"; $resn_exch{" H2* MTA"}=" H2' MTA";
$resn_exch{"HO2* MTA"}="HO2' MTA"; $resn_exch{" H3* MTA"}=" H3' MTA"; $resn_exch{" H1* MTA"}=" H1' MTA";
$resn_exch{"1H6  MTA"}=" H61 MTA"; $resn_exch{"2H6  MTA"}=" H62 MTA"; $resn_exch{"1H1  MTB"}=" H11 MTB";
$resn_exch{"2H1  MTB"}=" H12 MTB"; $resn_exch{"3H1  MTB"}=" H13 MTB"; $resn_exch{"1H2  MTB"}=" H21 MTB";
$resn_exch{"2H2  MTB"}=" H22 MTB"; $resn_exch{"3H2  MTB"}=" H23 MTB"; $resn_exch{"1H3  MTB"}=" H31 MTB";
$resn_exch{"2H3  MTB"}=" H32 MTB"; $resn_exch{"3H3  MTB"}=" H33 MTB"; $resn_exch{"'HO4 MTB"}="HO4' MTB";
$resn_exch{"1H1  MTC"}="H11A MTC"; $resn_exch{"2H1  MTC"}=" H12 MTC"; $resn_exch{"1H9  MTC"}=" H91 MTC";
$resn_exch{"2H9  MTC"}=" H92 MTC"; $resn_exch{"1H15 MTC"}="H151 MTC"; $resn_exch{"2H15 MTC"}="H152 MTC";
$resn_exch{"1H16 MTC"}="H161 MTC"; $resn_exch{"2H16 MTC"}="H162 MTC"; $resn_exch{"3H16 MTC"}="H163 MTC";
$resn_exch{"1H18 MTC"}="H181 MTC"; $resn_exch{"2H18 MTC"}="H182 MTC"; $resn_exch{"3H18 MTC"}="H183 MTC";
$resn_exch{"1H2  MTD"}=" H21 MTD"; $resn_exch{"2H2  MTD"}=" H22 MTD"; $resn_exch{"1H4  MTD"}=" H41 MTD";
$resn_exch{"2H4  MTD"}=" H42 MTD"; $resn_exch{"3H4  MTD"}=" H43 MTD"; $resn_exch{"1HN2 MTE"}="HN21 MTE";
$resn_exch{"2HN2 MTE"}="HN22 MTE"; $resn_exch{"1H4' MTE"}="H4'1 MTE"; $resn_exch{"2H4' MTE"}="H4'2 MTE";
$resn_exch{"2HOP MTE"}="HOP2 MTE"; $resn_exch{"3HOP MTE"}="HOP3 MTE"; $resn_exch{"1HM  MTF"}=" HM1 MTF";
$resn_exch{"2HM  MTF"}=" HM2 MTF"; $resn_exch{"3HM  MTF"}=" HM3 MTF"; $resn_exch{"1H2  MTG"}=" H21 MTG";
$resn_exch{"2H2  MTG"}=" H22 MTG"; $resn_exch{"1H4  MTG"}=" H41 MTG"; $resn_exch{"2H4  MTG"}=" H42 MTG";
$resn_exch{"3H4  MTG"}=" H43 MTG"; $resn_exch{"1HCS MTH"}="HCS1 MTH"; $resn_exch{"2HCS MTH"}="HCS2 MTH";
$resn_exch{"3HCS MTH"}="HCS3 MTH"; $resn_exch{"1H5* MTH"}="H5'1 MTH"; $resn_exch{"2H5* MTH"}="H5'2 MTH";
$resn_exch{"1H4* MTH"}="H4'1 MTH"; $resn_exch{"1H2* MTH"}="H2'1 MTH"; $resn_exch{"1HO2 MTH"}="HO21 MTH";
$resn_exch{"1H3* MTH"}="H3'1 MTH"; $resn_exch{"1HO3 MTH"}="HO31 MTH"; $resn_exch{"1H1* MTH"}="H1'1 MTH";
$resn_exch{"1HC8 MTH"}="HC81 MTH"; $resn_exch{"1HC7 MTH"}="HC71 MTH"; $resn_exch{"1HN6 MTH"}="HN61 MTH";
$resn_exch{"2HN6 MTH"}="HN62 MTH"; $resn_exch{"1HC2 MTH"}="HC21 MTH"; $resn_exch{"1HS  MTI"}=" HS1 MTI";
$resn_exch{"2HS  MTI"}=" HS2 MTI"; $resn_exch{"3HS  MTI"}=" HS3 MTI"; $resn_exch{"1H5* MTI"}="H5'1 MTI";
$resn_exch{"2H5* MTI"}="H5'2 MTI"; $resn_exch{" H4* MTI"}=" H4' MTI"; $resn_exch{"1H4* MTI"}="H4'1 MTI";
$resn_exch{"2H4* MTI"}="H4'2 MTI"; $resn_exch{" H2* MTI"}=" H2' MTI"; $resn_exch{" H3* MTI"}=" H3' MTI";
$resn_exch{" H1* MTI"}=" H1' MTI"; $resn_exch{"1HN  MTJ"}="HN1A MTJ"; $resn_exch{"2HN  MTJ"}=" HN2 MTJ";
$resn_exch{"3HN  MTJ"}=" HN3 MTJ"; $resn_exch{"1HB  MTJ"}=" HB1 MTJ"; $resn_exch{"2HB  MTJ"}=" HB2 MTJ";
$resn_exch{"1HM  MTJ"}=" HM1 MTJ"; $resn_exch{"2HM  MTJ"}=" HM2 MTJ"; $resn_exch{"3HM  MTJ"}=" HM3 MTJ";
$resn_exch{"1HAH MTK"}="HAH1 MTK"; $resn_exch{"2HAH MTK"}="HAH2 MTK"; $resn_exch{"1HAG MTK"}="HAG1 MTK";
$resn_exch{"2HAG MTK"}="HAG2 MTK"; $resn_exch{"1HAZ MTK"}="HAZ1 MTK"; $resn_exch{"2HAZ MTK"}="HAZ2 MTK";
$resn_exch{"1HAX MTK"}="HAX1 MTK"; $resn_exch{"2HAX MTK"}="HAX2 MTK"; $resn_exch{"1HAY MTK"}="HAY1 MTK";
$resn_exch{"2HAY MTK"}="HAY2 MTK"; $resn_exch{"1HBA MTK"}="HBA1 MTK"; $resn_exch{"2HBA MTK"}="HBA2 MTK";
$resn_exch{"1HAW MTK"}="HAW1 MTK"; $resn_exch{"2HAW MTK"}="HAW2 MTK"; $resn_exch{"1HAV MTK"}="HAV1 MTK";
$resn_exch{"2HAV MTK"}="HAV2 MTK"; $resn_exch{"1HAA MTK"}="HAA1 MTK"; $resn_exch{"2HAA MTK"}="HAA2 MTK";
$resn_exch{"3HAA MTK"}="HAA3 MTK"; $resn_exch{"1HAB MTK"}="HAB1 MTK"; $resn_exch{"2HAB MTK"}="HAB2 MTK";
$resn_exch{"3HAB MTK"}="HAB3 MTK"; $resn_exch{"1H1  MTL"}=" H11 MTL"; $resn_exch{"2H1  MTL"}=" H12 MTL";
$resn_exch{"1H6  MTL"}=" H61 MTL"; $resn_exch{"2H6  MTL"}=" H62 MTL"; $resn_exch{"1HCS MTM"}="HCS1 MTM";
$resn_exch{"2HCS MTM"}="HCS2 MTM"; $resn_exch{"3HCS MTM"}="HCS3 MTM"; $resn_exch{"1H5* MTM"}="H5'1 MTM";
$resn_exch{"2H5* MTM"}="H5'2 MTM"; $resn_exch{" H4* MTM"}=" H4' MTM"; $resn_exch{" H2* MTM"}=" H2' MTM";
$resn_exch{"2HO* MTM"}="HO'2 MTM"; $resn_exch{" H3* MTM"}=" H3' MTM"; $resn_exch{" H1* MTM"}=" H1' MTM";
$resn_exch{"1H6  MTM"}=" H61 MTM"; $resn_exch{"2H6  MTM"}=" H62 MTM"; $resn_exch{"1H4  MTN"}=" H41 MTN";
$resn_exch{"2H4  MTN"}=" H42 MTN"; $resn_exch{"1H6  MTN"}=" H61 MTN"; $resn_exch{"2H6  MTN"}=" H62 MTN";
$resn_exch{"3H6  MTN"}=" H63 MTN"; $resn_exch{"1H7  MTN"}=" H71 MTN"; $resn_exch{"2H7  MTN"}=" H72 MTN";
$resn_exch{"3H7  MTN"}=" H73 MTN"; $resn_exch{"1H8  MTN"}=" H81 MTN"; $resn_exch{"2H8  MTN"}=" H82 MTN";
$resn_exch{"3H8  MTN"}=" H83 MTN"; $resn_exch{"1H9  MTN"}=" H91 MTN"; $resn_exch{"2H9  MTN"}=" H92 MTN";
$resn_exch{"3H9  MTN"}=" H93 MTN"; $resn_exch{"1H   MTO"}=" H1  MTO"; $resn_exch{"2H   MTO"}=" H2  MTO";
$resn_exch{"1H5' MTP"}="H5'1 MTP"; $resn_exch{"2H5' MTP"}="H5'2 MTP"; $resn_exch{"1HS  MTP"}=" HS1 MTP";
$resn_exch{"2HS  MTP"}=" HS2 MTP"; $resn_exch{"3HS  MTP"}=" HS3 MTP"; $resn_exch{"1HN2 MTQ"}="HN21 MTQ";
$resn_exch{"2HN2 MTQ"}="HN22 MTQ"; $resn_exch{"1H4' MTQ"}="H4'1 MTQ"; $resn_exch{"2H4' MTQ"}="H4'2 MTQ";
$resn_exch{"2HOP MTQ"}="HOP2 MTQ"; $resn_exch{"3HOP MTQ"}="HOP3 MTQ"; $resn_exch{"2HOP MTR"}="HOP2 MTR";
$resn_exch{"1H5M MTR"}=" H71 MTR"; $resn_exch{"2H5M MTR"}=" H72 MTR"; $resn_exch{"3H5M MTR"}=" H73 MTR";
$resn_exch{"1H2* MTR"}=" H2' MTR"; $resn_exch{"2H2* MTR"}="H2'' MTR"; $resn_exch{"1H5* MTR"}=" H5' MTR";
$resn_exch{"2H5* MTR"}="H5'' MTR"; $resn_exch{" H4* MTR"}=" H4' MTR"; $resn_exch{" H1* MTR"}=" H1' MTR";
$resn_exch{" H3* MTR"}=" H3' MTR"; $resn_exch{"H3T  MTR"}="HO3' MTR"; $resn_exch{"3HOP MTR"}="HOP3 MTR";
$resn_exch{"1HN3 MTS"}="HN31 MTS"; $resn_exch{"2HN3 MTS"}="HN32 MTS"; $resn_exch{"1H15 MTS"}="H151 MTS";
$resn_exch{"2H15 MTS"}="H152 MTS"; $resn_exch{"3H15 MTS"}="H153 MTS"; $resn_exch{"1H18 MTS"}="H181 MTS";
$resn_exch{"2H18 MTS"}="H182 MTS"; $resn_exch{"3H18 MTS"}="H183 MTS"; $resn_exch{"1H14 MTT"}="H141 MTT";
$resn_exch{"1H13 MTT"}="H131 MTT"; $resn_exch{"1H12 MTT"}="H121 MTT"; $resn_exch{"1H11 MTT"}="H111 MTT";
$resn_exch{"1H15 MTT"}="H151 MTT"; $resn_exch{"1H16 MTT"}="H161 MTT"; $resn_exch{"2H16 MTT"}="H162 MTT";
$resn_exch{"1H24 MTT"}="H241 MTT"; $resn_exch{"1H23 MTT"}="H231 MTT"; $resn_exch{"1H22 MTT"}="H221 MTT";
$resn_exch{"1H21 MTT"}="H211 MTT"; $resn_exch{"1H25 MTT"}="H251 MTT"; $resn_exch{"1H26 MTT"}="H261 MTT";
$resn_exch{"2H26 MTT"}="H262 MTT"; $resn_exch{"1H34 MTT"}="H341 MTT"; $resn_exch{"1H33 MTT"}="H331 MTT";
$resn_exch{"1H32 MTT"}="H321 MTT"; $resn_exch{"1H31 MTT"}="H311 MTT"; $resn_exch{"1H35 MTT"}="H351 MTT";
$resn_exch{"1H36 MTT"}="H361 MTT"; $resn_exch{"2H36 MTT"}="H362 MTT"; $resn_exch{"1H44 MTT"}="H441 MTT";
$resn_exch{"1H43 MTT"}="H431 MTT"; $resn_exch{"1H42 MTT"}="H421 MTT"; $resn_exch{"1H41 MTT"}="H411 MTT";
$resn_exch{"1H45 MTT"}="H451 MTT"; $resn_exch{"1H46 MTT"}="H461 MTT"; $resn_exch{"2H46 MTT"}="H462 MTT";
$resn_exch{"1H5* MTU"}=" H5' MTU"; $resn_exch{"2H5* MTU"}="H5'' MTU"; $resn_exch{" H4* MTU"}=" H4' MTU";
$resn_exch{" H1* MTU"}=" H1' MTU"; $resn_exch{"1HN2 MTU"}="HN21 MTU"; $resn_exch{"2HN2 MTU"}="HN22 MTU";
$resn_exch{" H2* MTU"}=" H2' MTU"; $resn_exch{"HO2* MTU"}="HO2' MTU"; $resn_exch{" H3* MTU"}=" H3' MTU";
$resn_exch{"HO3* MTU"}="HO3' MTU"; $resn_exch{"1H2N MTV"}="H2N1 MTV"; $resn_exch{"2H2N MTV"}="H2N2 MTV";
$resn_exch{"1H3N MTV"}="H3N1 MTV"; $resn_exch{"1H5N MTV"}="H5N1 MTV"; $resn_exch{"1H6C MTV"}="H6C1 MTV";
$resn_exch{"1H3' MTV"}="H3'1 MTV"; $resn_exch{"1H4' MTV"}="H4'1 MTV"; $resn_exch{"2H4' MTV"}="H4'2 MTV";
$resn_exch{"1H2P MTV"}="H2P1 MTV"; $resn_exch{"1H3P MTV"}="H3P1 MTV"; $resn_exch{"1HM2 MTV"}="HM21 MTV";
$resn_exch{"1H2C MTW"}="H2C1 MTW"; $resn_exch{"2H2C MTW"}="H2C2 MTW"; $resn_exch{"1H8C MTW"}="H8C1 MTW";
$resn_exch{"2H8C MTW"}="H8C2 MTW"; $resn_exch{"1HN2 MTX"}="HN21 MTX"; $resn_exch{"2HN2 MTX"}="HN22 MTX";
$resn_exch{"1HN4 MTX"}="HN41 MTX"; $resn_exch{"2HN4 MTX"}="HN42 MTX"; $resn_exch{"1H9  MTX"}=" H91 MTX";
$resn_exch{"2H9  MTX"}=" H92 MTX"; $resn_exch{"1HM  MTX"}=" HM1 MTX"; $resn_exch{"2HM  MTX"}=" HM2 MTX";
$resn_exch{"3HM  MTX"}=" HM3 MTX"; $resn_exch{"1HB  MTX"}=" HB1 MTX"; $resn_exch{"2HB  MTX"}=" HB2 MTX";
$resn_exch{"1HG  MTX"}=" HG1 MTX"; $resn_exch{"2HG  MTX"}=" HG2 MTX"; $resn_exch{"2HOE MTX"}="HOE2 MTX";
$resn_exch{"1HN  MTY"}=" H   MTY"; $resn_exch{"2HN  MTY"}=" H2  MTY"; $resn_exch{"1HB  MTY"}=" HB2 MTY";
$resn_exch{"2HB  MTY"}=" HB3 MTY"; $resn_exch{" HOH MTY"}=" HH  MTY"; $resn_exch{"1H14 MTZ"}="H141 MTZ";
$resn_exch{"2H14 MTZ"}="H142 MTZ"; $resn_exch{"1H15 MTZ"}="H151 MTZ"; $resn_exch{"2H15 MTZ"}="H152 MTZ";
$resn_exch{"1H16 MTZ"}="H161 MTZ"; $resn_exch{"2H16 MTZ"}="H162 MTZ"; $resn_exch{"1H17 MTZ"}="H171 MTZ";
$resn_exch{"2H17 MTZ"}="H172 MTZ"; $resn_exch{"1H7C MTZ"}="H7C1 MTZ"; $resn_exch{"2H7C MTZ"}="H7C2 MTZ";
$resn_exch{"1H2C MTZ"}="H2C1 MTZ"; $resn_exch{"2H2C MTZ"}="H2C2 MTZ"; $resn_exch{"3H2C MTZ"}="H2C3 MTZ";
$resn_exch{"1HN  MTZ"}=" HN1 MTZ"; $resn_exch{"2HN  MTZ"}=" HN2 MTZ"; $resn_exch{"1H32 MU0"}="H321 MU0";
$resn_exch{"2H32 MU0"}="H322 MU0"; $resn_exch{"1H29 MU0"}="H291 MU0"; $resn_exch{"2H29 MU0"}="H292 MU0";
$resn_exch{"1H23 MU0"}="H231 MU0"; $resn_exch{"2H23 MU0"}="H232 MU0"; $resn_exch{"1H14 MU0"}="H141 MU0";
$resn_exch{"2H14 MU0"}="H142 MU0"; $resn_exch{"3H14 MU0"}="H143 MU0"; $resn_exch{"1H15 MU0"}="H151 MU0";
$resn_exch{"2H15 MU0"}="H152 MU0"; $resn_exch{"3H15 MU0"}="H153 MU0"; $resn_exch{"1H2  MU0"}="H21A MU0";
$resn_exch{"2H2  MU0"}=" H22 MU0"; $resn_exch{"1H39 MU0"}="H391 MU0"; $resn_exch{"2H39 MU0"}="H392 MU0";
$resn_exch{"1H1  MU0"}=" H11 MU0"; $resn_exch{"2H1  MU0"}="H12A MU0"; $resn_exch{"1H22 MU0"}="H221 MU0";
$resn_exch{"2H22 MU0"}="H222 MU0"; $resn_exch{"1H2  MU1"}="H21A MU1"; $resn_exch{"2H2  MU1"}=" H22 MU1";
$resn_exch{"3H2  MU1"}=" H23 MU1"; $resn_exch{"1H14 MU1"}="H141 MU1"; $resn_exch{"2H14 MU1"}="H142 MU1";
$resn_exch{"3H14 MU1"}="H143 MU1"; $resn_exch{"1H15 MU1"}="H151 MU1"; $resn_exch{"2H15 MU1"}="H152 MU1";
$resn_exch{"3H15 MU1"}="H153 MU1"; $resn_exch{"1H22 MU1"}="H221 MU1"; $resn_exch{"2H22 MU1"}="H222 MU1";
$resn_exch{"1H23 MU1"}="H231 MU1"; $resn_exch{"2H23 MU1"}="H232 MU1"; $resn_exch{"1H29 MU1"}="H291 MU1";
$resn_exch{"2H29 MU1"}="H292 MU1"; $resn_exch{"1H32 MU1"}="H321 MU1"; $resn_exch{"2H32 MU1"}="H322 MU1";
$resn_exch{"1H40 MU1"}="H401 MU1"; $resn_exch{"2H40 MU1"}="H402 MU1"; $resn_exch{"3H40 MU1"}="H403 MU1";
$resn_exch{"1H8C MU2"}="H8C1 MU2"; $resn_exch{"2H8C MU2"}="H8C2 MU2"; $resn_exch{"3H8C MU2"}="H8C3 MU2";
$resn_exch{"1H6C MU2"}="H6C1 MU2"; $resn_exch{"2H6C MU2"}="H6C2 MU2"; $resn_exch{"1H20 MU2"}="H201 MU2";
$resn_exch{"2H20 MU2"}="H202 MU2"; $resn_exch{"3H20 MU2"}="H203 MU2"; $resn_exch{"1H23 MU2"}="H231 MU2";
$resn_exch{"2H23 MU2"}="H232 MU2"; $resn_exch{"3H23 MU2"}="H233 MU2"; $resn_exch{"1H25 MU2"}="H251 MU2";
$resn_exch{"2H25 MU2"}="H252 MU2"; $resn_exch{"1H26 MU2"}="H261 MU2"; $resn_exch{"2H26 MU2"}="H262 MU2";
$resn_exch{"1HA  MU2"}=" HA1 MU2"; $resn_exch{"2HB  MU2"}=" HB2 MU2"; $resn_exch{"1H10 MUA"}="H101 MUA";
$resn_exch{"2H10 MUA"}="H102 MUA"; $resn_exch{"3H10 MUA"}="H103 MUA"; $resn_exch{"1H   MUB"}=" H1  MUB";
$resn_exch{"2H   MUB"}=" H2  MUB"; $resn_exch{"2HN  MUB"}=" HN2 MUB"; $resn_exch{"1H8  MUB"}=" H81 MUB";
$resn_exch{"2H8  MUB"}=" H82 MUB"; $resn_exch{"3H8  MUB"}=" H83 MUB"; $resn_exch{"3H   MUB"}=" H3  MUB";
$resn_exch{"9H   MUB"}=" H9  MUB"; $resn_exch{"1H11 MUB"}="H111 MUB"; $resn_exch{"2H11 MUB"}="H112 MUB";
$resn_exch{"3H11 MUB"}="H113 MUB"; $resn_exch{"4H   MUB"}=" H4A MUB"; $resn_exch{"5H   MUB"}=" H5  MUB";
$resn_exch{"1H6  MUB"}=" H61 MUB"; $resn_exch{"2H6  MUB"}=" H62 MUB"; $resn_exch{"6HO  MUB"}=" HO6 MUB";
$resn_exch{"1HM4 MUG"}="HM41 MUG"; $resn_exch{"2HM4 MUG"}="HM42 MUG"; $resn_exch{"3HM4 MUG"}="HM43 MUG";
$resn_exch{"'HO2 MUG"}="HO2' MUG"; $resn_exch{"'HO3 MUG"}="HO3' MUG"; $resn_exch{"'HO4 MUG"}="HO4' MUG";
$resn_exch{"1H6' MUG"}="H6'1 MUG"; $resn_exch{"2H6' MUG"}="H6'2 MUG"; $resn_exch{"'HO6 MUG"}="HO6' MUG";
$resn_exch{"1H15 MUH"}="H151 MUH"; $resn_exch{"2H15 MUH"}="H152 MUH"; $resn_exch{"3H15 MUH"}="H153 MUH";
$resn_exch{"1H47 MUI"}="H471 MUI"; $resn_exch{"2H47 MUI"}="H472 MUI"; $resn_exch{"3H47 MUI"}="H473 MUI";
$resn_exch{"1H25 MUI"}="H251 MUI"; $resn_exch{"2H25 MUI"}="H252 MUI"; $resn_exch{"1H32 MUI"}="H321 MUI";
$resn_exch{"2H32 MUI"}="H322 MUI"; $resn_exch{"1H16 MUI"}="H161 MUI"; $resn_exch{"2H16 MUI"}="H162 MUI";
$resn_exch{"1H12 MUI"}="H121 MUI"; $resn_exch{"2H12 MUI"}="H122 MUI"; $resn_exch{"1H15 MUI"}="H151 MUI";
$resn_exch{"2H15 MUI"}="H152 MUI"; $resn_exch{"3H15 MUI"}="H153 MUI"; $resn_exch{"1H14 MUI"}="H141 MUI";
$resn_exch{"2H14 MUI"}="H142 MUI"; $resn_exch{"3H14 MUI"}="H143 MUI"; $resn_exch{"1H18 MUI"}="H181 MUI";
$resn_exch{"2H18 MUI"}="H182 MUI"; $resn_exch{"3H18 MUI"}="H183 MUI"; $resn_exch{"1H1  MUL"}="H11A MUL";
$resn_exch{"2H1  MUL"}=" H12 MUL"; $resn_exch{"1H2  MUL"}=" H21 MUL"; $resn_exch{"2H2  MUL"}=" H22 MUL";
$resn_exch{"1H7  MUL"}=" H71 MUL"; $resn_exch{"2H7  MUL"}=" H72 MUL"; $resn_exch{"1H8  MUL"}=" H81 MUL";
$resn_exch{"2H8  MUL"}=" H82 MUL"; $resn_exch{"1H13 MUL"}="H131 MUL"; $resn_exch{"2H13 MUL"}="H132 MUL";
$resn_exch{"1H15 MUL"}="H151 MUL"; $resn_exch{"2H15 MUL"}="H152 MUL"; $resn_exch{"3H15 MUL"}="H153 MUL";
$resn_exch{"1H16 MUL"}="H161 MUL"; $resn_exch{"2H16 MUL"}="H162 MUL"; $resn_exch{"3H16 MUL"}="H163 MUL";
$resn_exch{"1H17 MUL"}="H171 MUL"; $resn_exch{"2H17 MUL"}="H172 MUL"; $resn_exch{"3H17 MUL"}="H173 MUL";
$resn_exch{"1H18 MUL"}="H181 MUL"; $resn_exch{"2H18 MUL"}="H182 MUL"; $resn_exch{"3H18 MUL"}="H183 MUL";
$resn_exch{"1H20 MUL"}="H201 MUL"; $resn_exch{"2H20 MUL"}="H202 MUL"; $resn_exch{"1H22 MUL"}="H221 MUL";
$resn_exch{"2H22 MUL"}="H222 MUL"; $resn_exch{"1H23 MUL"}="H231 MUL"; $resn_exch{"2H23 MUL"}="H232 MUL";
$resn_exch{"1H24 MUL"}="H241 MUL"; $resn_exch{"2H24 MUL"}="H242 MUL"; $resn_exch{"1H25 MUL"}="H251 MUL";
$resn_exch{"2H25 MUL"}="H252 MUL"; $resn_exch{"1H26 MUL"}="H261 MUL"; $resn_exch{"2H26 MUL"}="H262 MUL";
$resn_exch{"3H26 MUL"}="H263 MUL"; $resn_exch{"1H27 MUL"}="H271 MUL"; $resn_exch{"2H27 MUL"}="H272 MUL";
$resn_exch{"1H28 MUL"}="H281 MUL"; $resn_exch{"2H28 MUL"}="H282 MUL"; $resn_exch{"3H28 MUL"}="H283 MUL";
$resn_exch{"1H1  MUP"}=" H11 MUP"; $resn_exch{"2H1  MUP"}=" H12 MUP"; $resn_exch{"1H2  MUP"}=" H21 MUP";
$resn_exch{"2H2  MUP"}=" H22 MUP"; $resn_exch{"1H3  MUP"}=" H31 MUP"; $resn_exch{"2H3  MUP"}=" H32 MUP";
$resn_exch{"1H4  MUP"}=" H41 MUP"; $resn_exch{"2H4  MUP"}=" H42 MUP"; $resn_exch{"1H5  MUP"}=" H51 MUP";
$resn_exch{"2H5  MUP"}=" H52 MUP"; $resn_exch{"1H6  MUP"}=" H61 MUP"; $resn_exch{"2H6  MUP"}=" H62 MUP";
$resn_exch{"1H7  MUP"}=" H71 MUP"; $resn_exch{"2H7  MUP"}=" H72 MUP"; $resn_exch{"1H8  MUP"}=" H81 MUP";
$resn_exch{"2H8  MUP"}=" H82 MUP"; $resn_exch{"1H9  MUP"}=" H91 MUP"; $resn_exch{"2H9  MUP"}=" H92 MUP";
$resn_exch{"1H10 MUP"}="H101 MUP"; $resn_exch{"2H10 MUP"}="H102 MUP"; $resn_exch{"1H11 MUP"}="H111 MUP";
$resn_exch{"2H11 MUP"}="H112 MUP"; $resn_exch{"3H11 MUP"}="H113 MUP"; $resn_exch{"1HM  MUP"}=" HM1 MUP";
$resn_exch{"2HM  MUP"}=" HM2 MUP"; $resn_exch{"3HM  MUP"}=" HM3 MUP"; $resn_exch{"3HOP MUP"}="HOP3 MUP";
$resn_exch{"1HN2 MUR"}="HN21 MUR"; $resn_exch{"2HN2 MUR"}="HN22 MUR"; $resn_exch{"1H6  MUR"}=" H61 MUR";
$resn_exch{"2H6  MUR"}=" H62 MUR"; $resn_exch{"1H9  MUR"}=" H91 MUR"; $resn_exch{"2H9  MUR"}=" H92 MUR";
$resn_exch{"3H9  MUR"}=" H93 MUR"; $resn_exch{"1H16 MUS"}="H161 MUS"; $resn_exch{"2H16 MUS"}="H162 MUS";
$resn_exch{"1H18 MUS"}="H181 MUS"; $resn_exch{"2H18 MUS"}="H182 MUS"; $resn_exch{"3H18 MUS"}="H183 MUS";
$resn_exch{"1H10 MUS"}="H101 MUS"; $resn_exch{"2H10 MUS"}="H102 MUS"; $resn_exch{"1HM4 MUS"}="HM41 MUS";
$resn_exch{"2HM4 MUS"}="HM42 MUS"; $resn_exch{"3HM4 MUS"}="HM43 MUS"; $resn_exch{"1H18 MUT"}="H181 MUT";
$resn_exch{"2H18 MUT"}="H182 MUT"; $resn_exch{"3H18 MUT"}="H183 MUT"; $resn_exch{"1H12 MUT"}="H121 MUT";
$resn_exch{"2H12 MUT"}="H122 MUT"; $resn_exch{"1H14 MUT"}="H141 MUT"; $resn_exch{"2H14 MUT"}="H142 MUT";
$resn_exch{"3H14 MUT"}="H143 MUT"; $resn_exch{"1H15 MUT"}="H151 MUT"; $resn_exch{"2H15 MUT"}="H152 MUT";
$resn_exch{"3H15 MUT"}="H153 MUT"; $resn_exch{"1H16 MUT"}="H161 MUT"; $resn_exch{"2H16 MUT"}="H162 MUT";
$resn_exch{"1H32 MUT"}="H321 MUT"; $resn_exch{"2H32 MUT"}="H322 MUT"; $resn_exch{"1H25 MUT"}="H251 MUT";
$resn_exch{"2H25 MUT"}="H252 MUT"; $resn_exch{"1H47 MUT"}="H471 MUT"; $resn_exch{"2H47 MUT"}="H472 MUT";
$resn_exch{"3H47 MUT"}="H473 MUT"; $resn_exch{"1H18 MUU"}="H181 MUU"; $resn_exch{"2H18 MUU"}="H182 MUU";
$resn_exch{"3H18 MUU"}="H183 MUU"; $resn_exch{"1H12 MUU"}="H121 MUU"; $resn_exch{"2H12 MUU"}="H122 MUU";
$resn_exch{"1H15 MUU"}="H151 MUU"; $resn_exch{"2H15 MUU"}="H152 MUU"; $resn_exch{"1H14 MUU"}="H141 MUU";
$resn_exch{"2H14 MUU"}="H142 MUU"; $resn_exch{"1H16 MUU"}="H161 MUU"; $resn_exch{"2H16 MUU"}="H162 MUU";
$resn_exch{"1H32 MUU"}="H321 MUU"; $resn_exch{"2H32 MUU"}="H322 MUU"; $resn_exch{"1H23 MUU"}="H231 MUU";
$resn_exch{"2H23 MUU"}="H232 MUU"; $resn_exch{"1H24 MUU"}="H241 MUU"; $resn_exch{"2H24 MUU"}="H242 MUU";
$resn_exch{"1H28 MUU"}="H281 MUU"; $resn_exch{"2H28 MUU"}="H282 MUU"; $resn_exch{"3H28 MUU"}="H283 MUU";
$resn_exch{"1H28 MUV"}="H281 MUV"; $resn_exch{"2H28 MUV"}="H282 MUV"; $resn_exch{"3H28 MUV"}="H283 MUV";
$resn_exch{"1H24 MUV"}="H241 MUV"; $resn_exch{"2H24 MUV"}="H242 MUV"; $resn_exch{"1H23 MUV"}="H231 MUV";
$resn_exch{"2H23 MUV"}="H232 MUV"; $resn_exch{"1H32 MUV"}="H321 MUV"; $resn_exch{"2H32 MUV"}="H322 MUV";
$resn_exch{"1H16 MUV"}="H161 MUV"; $resn_exch{"2H16 MUV"}="H162 MUV"; $resn_exch{"1H12 MUV"}="H121 MUV";
$resn_exch{"2H12 MUV"}="H122 MUV"; $resn_exch{"1H14 MUV"}="H141 MUV"; $resn_exch{"2H14 MUV"}="H142 MUV";
$resn_exch{"1H15 MUV"}="H151 MUV"; $resn_exch{"2H15 MUV"}="H152 MUV"; $resn_exch{"1H15 MUW"}="H151 MUW";
$resn_exch{"2H15 MUW"}="H152 MUW"; $resn_exch{"1H45 MUW"}="H451 MUW"; $resn_exch{"2H45 MUW"}="H452 MUW";
$resn_exch{"1H11 MUW"}="H111 MUW"; $resn_exch{"2H11 MUW"}="H112 MUW"; $resn_exch{"1H13 MUW"}="H131 MUW";
$resn_exch{"2H13 MUW"}="H132 MUW"; $resn_exch{"3H13 MUW"}="H133 MUW"; $resn_exch{"1H14 MUW"}="H141 MUW";
$resn_exch{"2H14 MUW"}="H142 MUW"; $resn_exch{"3H14 MUW"}="H143 MUW"; $resn_exch{"1H22 MUW"}="H221 MUW";
$resn_exch{"2H22 MUW"}="H222 MUW"; $resn_exch{"1H17 MUW"}="H171 MUW"; $resn_exch{"2H17 MUW"}="H172 MUW";
$resn_exch{"1H18 MUW"}="H181 MUW"; $resn_exch{"2H18 MUW"}="H182 MUW"; $resn_exch{"1H19 MUW"}="H191 MUW";
$resn_exch{"2H19 MUW"}="H192 MUW"; $resn_exch{"1H20 MUW"}="H201 MUW"; $resn_exch{"2H20 MUW"}="H202 MUW";
$resn_exch{"1H44 MUW"}="H441 MUW"; $resn_exch{"2H44 MUW"}="H442 MUW"; $resn_exch{"3H44 MUW"}="H443 MUW";
$resn_exch{" HN  MVA"}=" H   MVA"; $resn_exch{"1HN  MVA"}=" HN1 MVA"; $resn_exch{"2HN  MVA"}=" HN2 MVA";
$resn_exch{"3HN  MVA"}=" HN3 MVA"; $resn_exch{"1HG1 MVA"}="HG11 MVA"; $resn_exch{"2HG1 MVA"}="HG12 MVA";
$resn_exch{"3HG1 MVA"}="HG13 MVA"; $resn_exch{"1HG2 MVA"}="HG21 MVA"; $resn_exch{"2HG2 MVA"}="HG22 MVA";
$resn_exch{"3HG2 MVA"}="HG23 MVA"; $resn_exch{"1H8  MVB"}=" H81 MVB"; $resn_exch{"2H8  MVB"}=" H82 MVB";
$resn_exch{"1H10 MVB"}="H101 MVB"; $resn_exch{"2H10 MVB"}="H102 MVB"; $resn_exch{"1H11 MVB"}="H111 MVB";
$resn_exch{"2H11 MVB"}="H112 MVB"; $resn_exch{"3H11 MVB"}="H113 MVB"; $resn_exch{"1H12 MVB"}="H121 MVB";
$resn_exch{"2H12 MVB"}="H122 MVB"; $resn_exch{"1H13 MVB"}="H131 MVB"; $resn_exch{"2H13 MVB"}="H132 MVB";
$resn_exch{"1H21 MVB"}="H211 MVB"; $resn_exch{"2H21 MVB"}="H212 MVB"; $resn_exch{"1H22 MVB"}="H221 MVB";
$resn_exch{"2H22 MVB"}="H222 MVB"; $resn_exch{"1H4  MVD"}=" H41 MVD"; $resn_exch{"2H4  MVD"}=" H42 MVD";
$resn_exch{"1H9  MVD"}=" H91 MVD"; $resn_exch{"2H9  MVD"}=" H92 MVD"; $resn_exch{"1H11 MVD"}="H111 MVD";
$resn_exch{"2H11 MVD"}="H112 MVD"; $resn_exch{"1H12 MVD"}="H121 MVD"; $resn_exch{"2H12 MVD"}="H122 MVD";
$resn_exch{"1H15 MVD"}="H151 MVD"; $resn_exch{"2H15 MVD"}="H152 MVD"; $resn_exch{"1H16 MVD"}="H161 MVD";
$resn_exch{"2H16 MVD"}="H162 MVD"; $resn_exch{"1H18 MVD"}="H181 MVD"; $resn_exch{"2H18 MVD"}="H182 MVD";
$resn_exch{"3H18 MVD"}="H183 MVD"; $resn_exch{"1H19 MVD"}="H191 MVD"; $resn_exch{"2H19 MVD"}="H192 MVD";
$resn_exch{"1H21 MVD"}="H211 MVD"; $resn_exch{"2H21 MVD"}="H212 MVD"; $resn_exch{"3H21 MVD"}="H213 MVD";
$resn_exch{"1H22 MVD"}="H221 MVD"; $resn_exch{"2H22 MVD"}="H222 MVD"; $resn_exch{"1H23 MVD"}="H231 MVD";
$resn_exch{"2H23 MVD"}="H232 MVD"; $resn_exch{"1H24 MVD"}="H241 MVD"; $resn_exch{"2H24 MVD"}="H242 MVD";
$resn_exch{"1H26 MVD"}="H261 MVD"; $resn_exch{"2H26 MVD"}="H262 MVD"; $resn_exch{"3H26 MVD"}="H263 MVD";
$resn_exch{"1H27 MVD"}="H271 MVD"; $resn_exch{"2H27 MVD"}="H272 MVD"; $resn_exch{"3H27 MVD"}="H273 MVD";
$resn_exch{"1H28 MVD"}="H281 MVD"; $resn_exch{"2H28 MVD"}="H282 MVD"; $resn_exch{"3H28 MVD"}="H283 MVD";
$resn_exch{"1H6C MVL"}="H6C1 MVL"; $resn_exch{"2H6C MVL"}="H6C2 MVL"; $resn_exch{"1HO1 MW1"}="HO11 MW1";
$resn_exch{"2HO1 MW1"}="HO12 MW1"; $resn_exch{"1HO1 MW2"}="HO11 MW2"; $resn_exch{"2HO1 MW2"}="HO12 MW2";
$resn_exch{"1HO2 MW2"}="HO21 MW2"; $resn_exch{"2HO2 MW2"}="HO22 MW2"; $resn_exch{"1HO1 MW3"}="HO11 MW3";
$resn_exch{"2HO1 MW3"}="HO12 MW3"; $resn_exch{"1HO2 MW3"}="HO21 MW3"; $resn_exch{"2HO2 MW3"}="HO22 MW3";
$resn_exch{"1HO3 MW3"}="HO31 MW3"; $resn_exch{"2HO3 MW3"}="HO32 MW3"; $resn_exch{"1H8  MX1"}=" H81 MX1";
$resn_exch{"2H8  MX1"}=" H82 MX1"; $resn_exch{"1H6  MX1"}=" H61 MX1"; $resn_exch{"2H6  MX1"}=" H62 MX1";
$resn_exch{"1H9  MX1"}=" H91 MX1"; $resn_exch{"2H9  MX1"}=" H92 MX1"; $resn_exch{"3H9  MX1"}=" H93 MX1";
$resn_exch{"1H20 MX3"}="H201 MX3"; $resn_exch{"2H20 MX3"}="H202 MX3"; $resn_exch{"1H7C MX3"}="H7C1 MX3";
$resn_exch{"2H7C MX3"}="H7C2 MX3"; $resn_exch{"1H9C MX3"}="H9C1 MX3"; $resn_exch{"2H9C MX3"}="H9C2 MX3";
$resn_exch{"1H10 MX3"}="H101 MX3"; $resn_exch{"2H10 MX3"}="H102 MX3"; $resn_exch{"1H12 MX3"}="H121 MX3";
$resn_exch{"2H12 MX3"}="H122 MX3"; $resn_exch{"1H14 MX3"}="H141 MX3"; $resn_exch{"2H14 MX3"}="H142 MX3";
$resn_exch{"3H14 MX3"}="H143 MX3"; $resn_exch{"1H2C MX4"}="H2C1 MX4"; $resn_exch{"2H2C MX4"}="H2C2 MX4";
$resn_exch{"1H3C MX4"}="H3C1 MX4"; $resn_exch{"2H3C MX4"}="H3C2 MX4"; $resn_exch{"1H10 MX4"}="H101 MX4";
$resn_exch{"2H10 MX4"}="H102 MX4"; $resn_exch{"1H9C MX4"}="H9C1 MX4"; $resn_exch{"2H9C MX4"}="H9C2 MX4";
$resn_exch{"1H16 MX4"}="H161 MX4"; $resn_exch{"2H16 MX4"}="H162 MX4"; $resn_exch{"1H2C MX5"}="H2C1 MX5";
$resn_exch{"2H2C MX5"}="H2C2 MX5"; $resn_exch{"1H20 MX5"}="H201 MX5"; $resn_exch{"2H20 MX5"}="H202 MX5";
$resn_exch{"1H9C MX5"}="H9C1 MX5"; $resn_exch{"2H9C MX5"}="H9C2 MX5"; $resn_exch{"1H10 MX5"}="H101 MX5";
$resn_exch{"2H10 MX5"}="H102 MX5"; $resn_exch{"1H12 MX5"}="H121 MX5"; $resn_exch{"2H12 MX5"}="H122 MX5";
$resn_exch{"1H14 MX5"}="H141 MX5"; $resn_exch{"2H14 MX5"}="H142 MX5"; $resn_exch{"1H19 MX5"}="H191 MX5";
$resn_exch{"2H19 MX5"}="H192 MX5"; $resn_exch{"1HN2 MXA"}="HN21 MXA"; $resn_exch{"2HN2 MXA"}="HN22 MXA";
$resn_exch{"1HN4 MXA"}="HN41 MXA"; $resn_exch{"2HN4 MXA"}="HN42 MXA"; $resn_exch{"1H5M MXA"}="H5M1 MXA";
$resn_exch{"2H5M MXA"}="H5M2 MXA"; $resn_exch{"3H5M MXA"}="H5M3 MXA"; $resn_exch{"1H9  MXA"}=" H91 MXA";
$resn_exch{"2H9  MXA"}=" H92 MXA"; $resn_exch{"1H21 MXA"}="H211 MXA"; $resn_exch{"2H21 MXA"}="H212 MXA";
$resn_exch{"3H21 MXA"}="H213 MXA"; $resn_exch{"1H51 MXA"}="H511 MXA"; $resn_exch{"2H51 MXA"}="H512 MXA";
$resn_exch{"3H51 MXA"}="H513 MXA"; $resn_exch{"1HAO MXD"}="HAO1 MXD"; $resn_exch{"2HAO MXD"}="HAO2 MXD";
$resn_exch{"1HAM MXD"}="HAM1 MXD"; $resn_exch{"2HAM MXD"}="HAM2 MXD"; $resn_exch{"1HAG MXD"}="HAG1 MXD";
$resn_exch{"2HAG MXD"}="HAG2 MXD"; $resn_exch{"1HAK MXD"}="HAK1 MXD"; $resn_exch{"2HAK MXD"}="HAK2 MXD";
$resn_exch{"1HAL MXD"}="HAL1 MXD"; $resn_exch{"2HAL MXD"}="HAL2 MXD"; $resn_exch{"1HAJ MXD"}="HAJ1 MXD";
$resn_exch{"2HAJ MXD"}="HAJ2 MXD"; $resn_exch{"1HAH MXD"}="HAH1 MXD"; $resn_exch{"2HAH MXD"}="HAH2 MXD";
$resn_exch{"1H1  MXE"}=" H11 MXE"; $resn_exch{"2H1  MXE"}=" H12 MXE"; $resn_exch{"1H2  MXE"}=" H21 MXE";
$resn_exch{"2H2  MXE"}=" H22 MXE"; $resn_exch{"1H3  MXE"}=" H31 MXE"; $resn_exch{"2H3  MXE"}=" H32 MXE";
$resn_exch{"3H3  MXE"}=" H33 MXE"; $resn_exch{"1H35 MXG"}="H351 MXG"; $resn_exch{"2H35 MXG"}="H352 MXG";
$resn_exch{"3H35 MXG"}="H353 MXG"; $resn_exch{"1H31 MXG"}="H311 MXG"; $resn_exch{"2H31 MXG"}="H312 MXG";
$resn_exch{"3H31 MXG"}="H313 MXG"; $resn_exch{"1H22 MXG"}="H221 MXG"; $resn_exch{"2H22 MXG"}="H222 MXG";
$resn_exch{"1H21 MXG"}="H211 MXG"; $resn_exch{"2H21 MXG"}="H212 MXG"; $resn_exch{"1H13 MXG"}="H131 MXG";
$resn_exch{"2H13 MXG"}="H132 MXG"; $resn_exch{"1H14 MXG"}="H141 MXG"; $resn_exch{"2H14 MXG"}="H142 MXG";
$resn_exch{"1H15 MXG"}="H151 MXG"; $resn_exch{"2H15 MXG"}="H152 MXG"; $resn_exch{"1H16 MXG"}="H161 MXG";
$resn_exch{"2H16 MXG"}="H162 MXG"; $resn_exch{"1H17 MXG"}="H171 MXG"; $resn_exch{"2H17 MXG"}="H172 MXG";
$resn_exch{"1HC3 MXG"}="HC31 MXG"; $resn_exch{"2HC3 MXG"}="HC32 MXG"; $resn_exch{"1HC2 MXG"}="HC21 MXG";
$resn_exch{"2HC2 MXG"}="HC22 MXG"; $resn_exch{"1H3  MXL"}=" H31 MXL"; $resn_exch{"1H6  MXL"}=" H61 MXL";
$resn_exch{"2H6  MXL"}=" H62 MXL"; $resn_exch{"1HO4 MXL"}="HO41 MXL"; $resn_exch{"1H8  MXL"}=" H81 MXL";
$resn_exch{"2H8  MXL"}=" H82 MXL"; $resn_exch{"1H9  MXL"}=" H91 MXL"; $resn_exch{"2H9  MXL"}=" H92 MXL";
$resn_exch{"3H9  MXL"}=" H93 MXL"; $resn_exch{"1HN2 MXL"}="HN21 MXL"; $resn_exch{"1H11 MXL"}="H111 MXL";
$resn_exch{"1HO7 MXL"}="HO71 MXL"; $resn_exch{"1H14 MXL"}="H141 MXL"; $resn_exch{"1H15 MXL"}="H151 MXL";
$resn_exch{"1H17 MXL"}="H171 MXL"; $resn_exch{"1H18 MXL"}="H181 MXL"; $resn_exch{"1HO9 MXL"}="HO91 MXL";
$resn_exch{"1H20 MXL"}="H201 MXL"; $resn_exch{"2H20 MXL"}="H202 MXL"; $resn_exch{"3H20 MXL"}="H203 MXL";
$resn_exch{"1H12 MXT"}="H121 MXT"; $resn_exch{"2H12 MXT"}="H122 MXT"; $resn_exch{"1H22 MXT"}="H221 MXT";
$resn_exch{"2H22 MXT"}="H222 MXT"; $resn_exch{"1H23 MXT"}="H231 MXT"; $resn_exch{"2H23 MXT"}="H232 MXT";
$resn_exch{"1H25 MXT"}="H251 MXT"; $resn_exch{"2H25 MXT"}="H252 MXT"; $resn_exch{"3H25 MXT"}="H253 MXT";
$resn_exch{"1H26 MXT"}="H261 MXT"; $resn_exch{"2H26 MXT"}="H262 MXT"; $resn_exch{"3H26 MXT"}="H263 MXT";
$resn_exch{"1H28 MXT"}="H281 MXT"; $resn_exch{"2H28 MXT"}="H282 MXT"; $resn_exch{"3H28 MXT"}="H283 MXT";
$resn_exch{"1H29 MXT"}="H291 MXT"; $resn_exch{"2H29 MXT"}="H292 MXT"; $resn_exch{"3H29 MXT"}="H293 MXT";
$resn_exch{"1H11 MXT"}="H111 MXT"; $resn_exch{"2H11 MXT"}="H112 MXT"; $resn_exch{"1HO2 MXT"}="HO21 MXT";
$resn_exch{"1HCM MXY"}="HCM1 MXY"; $resn_exch{"2HCM MXY"}="HCM2 MXY"; $resn_exch{"3HCM MXY"}="HCM3 MXY";
$resn_exch{"1H6  MXY"}=" H61 MXY"; $resn_exch{"2H6  MXY"}=" H62 MXY"; $resn_exch{"3H6  MXY"}=" H63 MXY";
$resn_exch{"1H1C MXZ"}="H1C1 MXZ"; $resn_exch{"1HMC MXZ"}="HMC1 MXZ"; $resn_exch{"2HMC MXZ"}="HMC2 MXZ";
$resn_exch{"3HMC MXZ"}="HMC3 MXZ"; $resn_exch{"1H6C MXZ"}="H6C1 MXZ"; $resn_exch{"2H6C MXZ"}="H6C2 MXZ";
$resn_exch{"3H6C MXZ"}="H6C3 MXZ"; $resn_exch{"1H4N MY1"}="H4N1 MY1"; $resn_exch{"2H4N MY1"}="H4N2 MY1";
$resn_exch{"1H5C MY1"}="H5C1 MY1"; $resn_exch{"2H5C MY1"}="H5C2 MY1"; $resn_exch{"1H9C MY1"}="H9C1 MY1";
$resn_exch{"2H9C MY1"}="H9C2 MY1"; $resn_exch{"1H15 MY1"}="H151 MY1"; $resn_exch{"2H15 MY1"}="H152 MY1";
$resn_exch{"1H16 MY1"}="H161 MY1"; $resn_exch{"2H16 MY1"}="H162 MY1"; $resn_exch{"1H2C MY2"}="H2C1 MY2";
$resn_exch{"2H2C MY2"}="H2C2 MY2"; $resn_exch{"1H10 MY2"}="H101 MY2"; $resn_exch{"2H10 MY2"}="H102 MY2";
$resn_exch{"1H9C MY2"}="H9C1 MY2"; $resn_exch{"2H9C MY2"}="H9C2 MY2"; $resn_exch{"1H16 MY2"}="H161 MY2";
$resn_exch{"2H16 MY2"}="H162 MY2"; $resn_exch{"1H20 MY3"}="H201 MY3"; $resn_exch{"2H20 MY3"}="H202 MY3";
$resn_exch{"1H7C MY3"}="H7C1 MY3"; $resn_exch{"2H7C MY3"}="H7C2 MY3"; $resn_exch{"1H10 MY3"}="H101 MY3";
$resn_exch{"2H10 MY3"}="H102 MY3"; $resn_exch{"1H12 MY3"}="H121 MY3"; $resn_exch{"2H12 MY3"}="H122 MY3";
$resn_exch{"1H20 MY4"}="H201 MY4"; $resn_exch{"2H20 MY4"}="H202 MY4"; $resn_exch{"1H10 MY4"}="H101 MY4";
$resn_exch{"2H10 MY4"}="H102 MY4"; $resn_exch{"1H12 MY4"}="H121 MY4"; $resn_exch{"2H12 MY4"}="H122 MY4";
$resn_exch{"1H14 MY4"}="H141 MY4"; $resn_exch{"2H14 MY4"}="H142 MY4"; $resn_exch{"1H16 MY4"}="H161 MY4";
$resn_exch{"2H16 MY4"}="H162 MY4"; $resn_exch{"1H2N MY5"}="H2N1 MY5"; $resn_exch{"2H2N MY5"}="H2N2 MY5";
$resn_exch{"1H3C MY5"}="H3C1 MY5"; $resn_exch{"2H3C MY5"}="H3C2 MY5"; $resn_exch{"1H9C MY5"}="H9C1 MY5";
$resn_exch{"2H9C MY5"}="H9C2 MY5"; $resn_exch{"1H15 MY5"}="H151 MY5"; $resn_exch{"2H15 MY5"}="H152 MY5";
$resn_exch{"3H15 MY5"}="H153 MY5"; $resn_exch{"1H19 MY5"}="H191 MY5"; $resn_exch{"2H19 MY5"}="H192 MY5";
$resn_exch{"1H15 MY9"}="H151 MY9"; $resn_exch{"2H15 MY9"}="H152 MY9"; $resn_exch{"3H15 MY9"}="H153 MY9";
$resn_exch{"1H16 MY9"}="H161 MY9"; $resn_exch{"2H16 MY9"}="H162 MY9"; $resn_exch{"3H16 MY9"}="H163 MY9";
$resn_exch{"1H18 MY9"}="H181 MY9"; $resn_exch{"2H18 MY9"}="H182 MY9"; $resn_exch{"3H18 MY9"}="H183 MY9";
$resn_exch{"1H20 MY9"}="H201 MY9"; $resn_exch{"2H20 MY9"}="H202 MY9"; $resn_exch{"1H31 MY9"}="H311 MY9";
$resn_exch{"2H31 MY9"}="H312 MY9"; $resn_exch{"3H31 MY9"}="H313 MY9"; $resn_exch{"1H32 MY9"}="H321 MY9";
$resn_exch{"2H32 MY9"}="H322 MY9"; $resn_exch{"3H32 MY9"}="H323 MY9"; $resn_exch{"1H33 MY9"}="H331 MY9";
$resn_exch{"2H33 MY9"}="H332 MY9"; $resn_exch{"3H33 MY9"}="H333 MY9"; $resn_exch{"1H6A MYA"}="H6A1 MYA";
$resn_exch{"2H6A MYA"}="H6A2 MYA"; $resn_exch{"CH2A MYA"}="H2AC MYA"; $resn_exch{"CH8A MYA"}="H8AC MYA";
$resn_exch{"1HFM MYA"}="HFM1 MYA"; $resn_exch{"2HFM MYA"}="HFM2 MYA"; $resn_exch{"3HFM MYA"}="HFM3 MYA";
$resn_exch{"1HBM MYA"}="HBM1 MYA"; $resn_exch{"2HBM MYA"}="HBM2 MYA"; $resn_exch{"1HCM MYA"}="HCM1 MYA";
$resn_exch{"2HCM MYA"}="HCM2 MYA"; $resn_exch{"1HDM MYA"}="HDM1 MYA"; $resn_exch{"2HDM MYA"}="HDM2 MYA";
$resn_exch{"1HEM MYA"}="HEM1 MYA"; $resn_exch{"2HEM MYA"}="HEM2 MYA"; $resn_exch{"1HAM MYA"}="HAM1 MYA";
$resn_exch{"2HAM MYA"}="HAM2 MYA"; $resn_exch{"1H9M MYA"}="H9M1 MYA"; $resn_exch{"2H9M MYA"}="H9M2 MYA";
$resn_exch{"1H8M MYA"}="H8M1 MYA"; $resn_exch{"2H8M MYA"}="H8M2 MYA"; $resn_exch{"1H7M MYA"}="H7M1 MYA";
$resn_exch{"2H7M MYA"}="H7M2 MYA"; $resn_exch{"1H3M MYA"}="H3M1 MYA"; $resn_exch{"2H3M MYA"}="H3M2 MYA";
$resn_exch{"1H4M MYA"}="H4M1 MYA"; $resn_exch{"2H4M MYA"}="H4M2 MYA"; $resn_exch{"1H5M MYA"}="H5M1 MYA";
$resn_exch{"2H5M MYA"}="H5M2 MYA"; $resn_exch{"1H6M MYA"}="H6M1 MYA"; $resn_exch{"2H6M MYA"}="H6M2 MYA";
$resn_exch{"OH7A MYA"}="H7AO MYA"; $resn_exch{"OH8A MYA"}="H8AO MYA"; $resn_exch{"1HN4 MYA"}="HN41 MYA";
$resn_exch{"1HC6 MYA"}="HC61 MYA"; $resn_exch{"2HC6 MYA"}="HC62 MYA"; $resn_exch{"1HC7 MYA"}="HC71 MYA";
$resn_exch{"2HC7 MYA"}="HC72 MYA"; $resn_exch{"1HN8 MYA"}="HN81 MYA"; $resn_exch{"1H13 MYA"}="H131 MYA";
$resn_exch{"2H13 MYA"}="H132 MYA"; $resn_exch{"3H13 MYA"}="H133 MYA"; $resn_exch{"1H14 MYA"}="H141 MYA";
$resn_exch{"2H14 MYA"}="H142 MYA"; $resn_exch{"3H14 MYA"}="H143 MYA"; $resn_exch{"1H12 MYA"}="H121 MYA";
$resn_exch{"2H12 MYA"}="H122 MYA"; $resn_exch{"OH2A MYA"}="H2AO MYA"; $resn_exch{"1H4X MYA"}="H4X1 MYA";
$resn_exch{"CH1X MYA"}="H1XC MYA"; $resn_exch{"CH2X MYA"}="H2XC MYA"; $resn_exch{"OH2X MYA"}="H2XO MYA";
$resn_exch{"1H3X MYA"}="H3X1 MYA"; $resn_exch{"1H5X MYA"}="H5X1 MYA"; $resn_exch{"2H5X MYA"}="H5X2 MYA";
$resn_exch{"OH4A MYA"}="H4AO MYA"; $resn_exch{"CH10 MYA"}="H10C MYA"; $resn_exch{"OH10 MYA"}="H10O MYA";
$resn_exch{"1HC3 MYA"}="HC31 MYA"; $resn_exch{"2HC3 MYA"}="HC32 MYA"; $resn_exch{"1HC2 MYA"}="HC21 MYA";
$resn_exch{"2HC2 MYA"}="HC22 MYA"; $resn_exch{"1H53 MYD"}="H531 MYD"; $resn_exch{"2H53 MYD"}="H532 MYD";
$resn_exch{"3H53 MYD"}="H533 MYD"; $resn_exch{"1H52 MYD"}="H521 MYD"; $resn_exch{"2H52 MYD"}="H522 MYD";
$resn_exch{"3H52 MYD"}="H523 MYD"; $resn_exch{"1H49 MYD"}="H491 MYD"; $resn_exch{"2H49 MYD"}="H492 MYD";
$resn_exch{"1H38 MYD"}="H381 MYD"; $resn_exch{"2H38 MYD"}="H382 MYD"; $resn_exch{"1H37 MYD"}="H371 MYD";
$resn_exch{"2H37 MYD"}="H372 MYD"; $resn_exch{"1H34 MYD"}="H341 MYD"; $resn_exch{"2H34 MYD"}="H342 MYD";
$resn_exch{"1H28 MYD"}="H281 MYD"; $resn_exch{"2H28 MYD"}="H282 MYD"; $resn_exch{"1H11 MYD"}="H111 MYD";
$resn_exch{"2H11 MYD"}="H112 MYD"; $resn_exch{"1H2  MYE"}="H21A MYE"; $resn_exch{"2H2  MYE"}=" H22 MYE";
$resn_exch{"1H3  MYE"}=" H31 MYE"; $resn_exch{"2H3  MYE"}=" H32 MYE"; $resn_exch{"1H4  MYE"}=" H41 MYE";
$resn_exch{"2H4  MYE"}=" H42 MYE"; $resn_exch{"1H10 MYE"}="H101 MYE"; $resn_exch{"2H10 MYE"}="H102 MYE";
$resn_exch{"1H12 MYE"}="H121 MYE"; $resn_exch{"2H12 MYE"}="H122 MYE"; $resn_exch{"3H12 MYE"}="H123 MYE";
$resn_exch{"1H13 MYE"}="H131 MYE"; $resn_exch{"2H13 MYE"}="H132 MYE"; $resn_exch{"3H13 MYE"}="H133 MYE";
$resn_exch{"1H18 MYE"}="H181 MYE"; $resn_exch{"2H18 MYE"}="H182 MYE"; $resn_exch{"1H1  MYE"}="H11A MYE";
$resn_exch{"2H1  MYE"}=" H12 MYE"; $resn_exch{"3H1  MYE"}=" H13 MYE"; $resn_exch{"1H12 MYG"}="H121 MYG";
$resn_exch{"2H12 MYG"}="H122 MYG"; $resn_exch{"1HN8 MYG"}="HN81 MYG"; $resn_exch{"2HN8 MYG"}="HN82 MYG";
$resn_exch{"0HO1 MYG"}="HO10 MYG"; $resn_exch{"2HO1 MYG"}="HO12 MYG"; $resn_exch{"1H23 MYP"}="H231 MYP";
$resn_exch{"1H21 MYP"}="H211 MYP"; $resn_exch{"2H21 MYP"}="H212 MYP"; $resn_exch{"1H22 MYP"}="H221 MYP";
$resn_exch{"2H22 MYP"}="H222 MYP"; $resn_exch{"1H25 MYP"}="H251 MYP"; $resn_exch{"2H25 MYP"}="H252 MYP";
$resn_exch{"1H24 MYP"}="H241 MYP"; $resn_exch{"2H24 MYP"}="H242 MYP"; $resn_exch{"1H29 MYP"}="H291 MYP";
$resn_exch{"2H29 MYP"}="H292 MYP"; $resn_exch{"1H28 MYP"}="H281 MYP"; $resn_exch{"2H28 MYP"}="H282 MYP";
$resn_exch{"1H26 MYP"}="H261 MYP"; $resn_exch{"2H26 MYP"}="H262 MYP"; $resn_exch{"1H27 MYP"}="H271 MYP";
$resn_exch{"2H27 MYP"}="H272 MYP"; $resn_exch{"12H1 MYP"}="H112 MYP"; $resn_exch{"1H13 MYP"}="H131 MYP";
$resn_exch{"2H13 MYP"}="H132 MYP"; $resn_exch{"1HC8 MYP"}="HC81 MYP"; $resn_exch{"2HC8 MYP"}="HC82 MYP";
$resn_exch{"1H10 MYP"}="H101 MYP"; $resn_exch{"2H10 MYP"}="H102 MYP"; $resn_exch{"3H10 MYP"}="H103 MYP";
$resn_exch{"1HC9 MYP"}="HC91 MYP"; $resn_exch{"2HC9 MYP"}="HC92 MYP"; $resn_exch{"1H3  MYQ"}=" H31 MYQ";
$resn_exch{"2H3  MYQ"}=" H32 MYQ"; $resn_exch{"1H2  MYR"}=" H21 MYR"; $resn_exch{"2H2  MYR"}=" H22 MYR";
$resn_exch{"1H3  MYR"}=" H31 MYR"; $resn_exch{"2H3  MYR"}=" H32 MYR"; $resn_exch{"1H4  MYR"}=" H41 MYR";
$resn_exch{"2H4  MYR"}=" H42 MYR"; $resn_exch{"1H5  MYR"}=" H51 MYR"; $resn_exch{"2H5  MYR"}=" H52 MYR";
$resn_exch{"1H6  MYR"}=" H61 MYR"; $resn_exch{"2H6  MYR"}=" H62 MYR"; $resn_exch{"1H7  MYR"}=" H71 MYR";
$resn_exch{"2H7  MYR"}=" H72 MYR"; $resn_exch{"1H8  MYR"}=" H81 MYR"; $resn_exch{"2H8  MYR"}=" H82 MYR";
$resn_exch{"1H9  MYR"}=" H91 MYR"; $resn_exch{"2H9  MYR"}=" H92 MYR"; $resn_exch{"1H10 MYR"}="H101 MYR";
$resn_exch{"2H10 MYR"}="H102 MYR"; $resn_exch{"1H11 MYR"}="H111 MYR"; $resn_exch{"2H11 MYR"}="H112 MYR";
$resn_exch{"1H12 MYR"}="H121 MYR"; $resn_exch{"2H12 MYR"}="H122 MYR"; $resn_exch{"1H13 MYR"}="H131 MYR";
$resn_exch{"2H13 MYR"}="H132 MYR"; $resn_exch{"1H14 MYR"}="H141 MYR"; $resn_exch{"2H14 MYR"}="H142 MYR";
$resn_exch{"3H14 MYR"}="H143 MYR"; $resn_exch{"1H1  MYS"}=" H11 MYS"; $resn_exch{"2H1  MYS"}=" H12 MYS";
$resn_exch{"3H1  MYS"}=" H13 MYS"; $resn_exch{"1H2  MYS"}=" H21 MYS"; $resn_exch{"2H2  MYS"}=" H22 MYS";
$resn_exch{"1H3  MYS"}=" H31 MYS"; $resn_exch{"2H3  MYS"}=" H32 MYS"; $resn_exch{"1H4  MYS"}=" H41 MYS";
$resn_exch{"2H4  MYS"}=" H42 MYS"; $resn_exch{"1H5  MYS"}=" H51 MYS"; $resn_exch{"2H5  MYS"}=" H52 MYS";
$resn_exch{"1H6  MYS"}=" H61 MYS"; $resn_exch{"2H6  MYS"}=" H62 MYS"; $resn_exch{"1H7  MYS"}=" H71 MYS";
$resn_exch{"2H7  MYS"}=" H72 MYS"; $resn_exch{"1H8  MYS"}=" H81 MYS"; $resn_exch{"2H8  MYS"}=" H82 MYS";
$resn_exch{"1H9  MYS"}=" H91 MYS"; $resn_exch{"2H9  MYS"}=" H92 MYS"; $resn_exch{"1H10 MYS"}="H101 MYS";
$resn_exch{"2H10 MYS"}="H102 MYS"; $resn_exch{"1H11 MYS"}="H111 MYS"; $resn_exch{"2H11 MYS"}="H112 MYS";
$resn_exch{"1H12 MYS"}="H121 MYS"; $resn_exch{"2H12 MYS"}="H122 MYS"; $resn_exch{"1H13 MYS"}="H131 MYS";
$resn_exch{"2H13 MYS"}="H132 MYS"; $resn_exch{"1H14 MYS"}="H141 MYS"; $resn_exch{"2H14 MYS"}="H142 MYS";
$resn_exch{"1H15 MYS"}="H151 MYS"; $resn_exch{"2H15 MYS"}="H152 MYS"; $resn_exch{"3H15 MYS"}="H153 MYS";
$resn_exch{"1H9  MYT"}=" H91 MYT"; $resn_exch{"2H9  MYT"}=" H92 MYT"; $resn_exch{"3H9  MYT"}=" H93 MYT";
$resn_exch{"1H11 MYT"}="H111 MYT"; $resn_exch{"2H11 MYT"}="H112 MYT"; $resn_exch{"3H11 MYT"}="H113 MYT";
$resn_exch{"1HN1 MYX"}="HN11 MYX"; $resn_exch{"2HN1 MYX"}="HN12 MYX"; $resn_exch{"1H4  MYX"}=" H41 MYX";
$resn_exch{"2H4  MYX"}=" H42 MYX"; $resn_exch{"3H4  MYX"}=" H43 MYX"; $resn_exch{"1H6  MYX"}=" H61 MYX";
$resn_exch{"2H6  MYX"}=" H62 MYX"; $resn_exch{"3H6  MYX"}=" H63 MYX"; $resn_exch{"1H8  MYX"}=" H81 MYX";
$resn_exch{"2H8  MYX"}=" H82 MYX"; $resn_exch{"3H8  MYX"}=" H83 MYX"; $resn_exch{"1H18 MYX"}="H181 MYX";
$resn_exch{"2H18 MYX"}="H182 MYX"; $resn_exch{"3H18 MYX"}="H183 MYX"; $resn_exch{"1H24 MYX"}="H241 MYX";
$resn_exch{"2H24 MYX"}="H242 MYX"; $resn_exch{"3H24 MYX"}="H243 MYX"; $resn_exch{"1H25 MYX"}="H251 MYX";
$resn_exch{"2H25 MYX"}="H252 MYX"; $resn_exch{"3H25 MYX"}="H253 MYX"; $resn_exch{"1H38 MYY"}="H381 MYY";
$resn_exch{"2H38 MYY"}="H382 MYY"; $resn_exch{"3H38 MYY"}="H383 MYY"; $resn_exch{"1H37 MYY"}="H371 MYY";
$resn_exch{"2H37 MYY"}="H372 MYY"; $resn_exch{"1H36 MYY"}="H361 MYY"; $resn_exch{"2H36 MYY"}="H362 MYY";
$resn_exch{"1H35 MYY"}="H351 MYY"; $resn_exch{"2H35 MYY"}="H352 MYY"; $resn_exch{"1H34 MYY"}="H341 MYY";
$resn_exch{"2H34 MYY"}="H342 MYY"; $resn_exch{"1H33 MYY"}="H331 MYY"; $resn_exch{"2H33 MYY"}="H332 MYY";
$resn_exch{"1H32 MYY"}="H321 MYY"; $resn_exch{"2H32 MYY"}="H322 MYY"; $resn_exch{"1H31 MYY"}="H311 MYY";
$resn_exch{"2H31 MYY"}="H312 MYY"; $resn_exch{"1H30 MYY"}="H301 MYY"; $resn_exch{"2H30 MYY"}="H302 MYY";
$resn_exch{"1H29 MYY"}="H291 MYY"; $resn_exch{"2H29 MYY"}="H292 MYY"; $resn_exch{"1H28 MYY"}="H281 MYY";
$resn_exch{"2H28 MYY"}="H282 MYY"; $resn_exch{"1H27 MYY"}="H271 MYY"; $resn_exch{"2H27 MYY"}="H272 MYY";
$resn_exch{"1H26 MYY"}="H261 MYY"; $resn_exch{"2H26 MYY"}="H262 MYY"; $resn_exch{"1H25 MYY"}="H251 MYY";
$resn_exch{"2H25 MYY"}="H252 MYY"; $resn_exch{"1H24 MYY"}="H241 MYY"; $resn_exch{"2H24 MYY"}="H242 MYY";
$resn_exch{"1H6C MYY"}="H6C1 MYY"; $resn_exch{"2H6C MYY"}="H6C2 MYY"; $resn_exch{"1H4C MYY"}="H4C1 MYY";
$resn_exch{"2H4C MYY"}="H4C2 MYY"; $resn_exch{"1H8C MYY"}="H8C1 MYY"; $resn_exch{"2H8C MYY"}="H8C2 MYY";
$resn_exch{"1H9C MYY"}="H9C1 MYY"; $resn_exch{"2H9C MYY"}="H9C2 MYY"; $resn_exch{"1H10 MYY"}="H101 MYY";
$resn_exch{"2H10 MYY"}="H102 MYY"; $resn_exch{"1H11 MYY"}="H111 MYY"; $resn_exch{"2H11 MYY"}="H112 MYY";
$resn_exch{"1H12 MYY"}="H121 MYY"; $resn_exch{"2H12 MYY"}="H122 MYY"; $resn_exch{"1H13 MYY"}="H131 MYY";
$resn_exch{"2H13 MYY"}="H132 MYY"; $resn_exch{"1H14 MYY"}="H141 MYY"; $resn_exch{"2H14 MYY"}="H142 MYY";
$resn_exch{"1H15 MYY"}="H151 MYY"; $resn_exch{"2H15 MYY"}="H152 MYY"; $resn_exch{"1H16 MYY"}="H161 MYY";
$resn_exch{"2H16 MYY"}="H162 MYY"; $resn_exch{"1H17 MYY"}="H171 MYY"; $resn_exch{"2H17 MYY"}="H172 MYY";
$resn_exch{"1H18 MYY"}="H181 MYY"; $resn_exch{"2H18 MYY"}="H182 MYY"; $resn_exch{"1H19 MYY"}="H191 MYY";
$resn_exch{"2H19 MYY"}="H192 MYY"; $resn_exch{"1H20 MYY"}="H201 MYY"; $resn_exch{"2H20 MYY"}="H202 MYY";
$resn_exch{"3H20 MYY"}="H203 MYY"; $resn_exch{"1H27 MZ1"}="H271 MZ1"; $resn_exch{"2H27 MZ1"}="H272 MZ1";
$resn_exch{"3H27 MZ1"}="H273 MZ1"; $resn_exch{"1H23 MZ1"}="H231 MZ1"; $resn_exch{"2H23 MZ1"}="H232 MZ1";
$resn_exch{"3H23 MZ1"}="H233 MZ1"; $resn_exch{"1H32 MZ1"}="H321 MZ1"; $resn_exch{"2H32 MZ1"}="H322 MZ1";
$resn_exch{"1H16 MZ1"}="H161 MZ1"; $resn_exch{"2H16 MZ1"}="H162 MZ1"; $resn_exch{"1H12 MZ1"}="H121 MZ1";
$resn_exch{"2H12 MZ1"}="H122 MZ1"; $resn_exch{"1H24 MZ2"}="H241 MZ2"; $resn_exch{"2H24 MZ2"}="H242 MZ2";
$resn_exch{"3H24 MZ2"}="H243 MZ2"; $resn_exch{"1H32 MZ2"}="H321 MZ2"; $resn_exch{"2H32 MZ2"}="H322 MZ2";
$resn_exch{"1H16 MZ2"}="H161 MZ2"; $resn_exch{"2H16 MZ2"}="H162 MZ2"; $resn_exch{"1H12 MZ2"}="H121 MZ2";
$resn_exch{"2H12 MZ2"}="H122 MZ2"; $resn_exch{"1H28 MZ3"}="H281 MZ3"; $resn_exch{"2H28 MZ3"}="H282 MZ3";
$resn_exch{"3H28 MZ3"}="H283 MZ3"; $resn_exch{"1H32 MZ3"}="H321 MZ3"; $resn_exch{"2H32 MZ3"}="H322 MZ3";
$resn_exch{"1H16 MZ3"}="H161 MZ3"; $resn_exch{"2H16 MZ3"}="H162 MZ3"; $resn_exch{"1H12 MZ3"}="H121 MZ3";
$resn_exch{"2H12 MZ3"}="H122 MZ3"; $resn_exch{"1H20 MZ3"}="H201 MZ3"; $resn_exch{"2H20 MZ3"}="H202 MZ3";
$resn_exch{"3H20 MZ3"}="H203 MZ3"; $resn_exch{"1H20 MZ4"}="H201 MZ4"; $resn_exch{"2H20 MZ4"}="H202 MZ4";
$resn_exch{"3H20 MZ4"}="H203 MZ4"; $resn_exch{"1H12 MZ4"}="H121 MZ4"; $resn_exch{"2H12 MZ4"}="H122 MZ4";
$resn_exch{"1H16 MZ4"}="H161 MZ4"; $resn_exch{"2H16 MZ4"}="H162 MZ4"; $resn_exch{"1H32 MZ4"}="H321 MZ4";
$resn_exch{"2H32 MZ4"}="H322 MZ4"; $resn_exch{"1H26 MZ5"}="H261 MZ5"; $resn_exch{"2H26 MZ5"}="H262 MZ5";
$resn_exch{"3H26 MZ5"}="H263 MZ5"; $resn_exch{"1H25 MZ5"}="H251 MZ5"; $resn_exch{"2H25 MZ5"}="H252 MZ5";
$resn_exch{"3H25 MZ5"}="H253 MZ5"; $resn_exch{"1H32 MZ5"}="H321 MZ5"; $resn_exch{"2H32 MZ5"}="H322 MZ5";
$resn_exch{"1H16 MZ5"}="H161 MZ5"; $resn_exch{"2H16 MZ5"}="H162 MZ5"; $resn_exch{"1H12 MZ5"}="H121 MZ5";
$resn_exch{"2H12 MZ5"}="H122 MZ5"; $resn_exch{"1H14 MZ5"}="H141 MZ5"; $resn_exch{"2H14 MZ5"}="H142 MZ5";
$resn_exch{"3H14 MZ5"}="H143 MZ5"; $resn_exch{"1H15 MZ5"}="H151 MZ5"; $resn_exch{"2H15 MZ5"}="H152 MZ5";
$resn_exch{"1H18 MZ5"}="H181 MZ5"; $resn_exch{"2H18 MZ5"}="H182 MZ5"; $resn_exch{"3H18 MZ5"}="H183 MZ5";
$resn_exch{"1H32 MZ6"}="H321 MZ6"; $resn_exch{"2H32 MZ6"}="H322 MZ6"; $resn_exch{"1H16 MZ6"}="H161 MZ6";
$resn_exch{"2H16 MZ6"}="H162 MZ6"; $resn_exch{"1H12 MZ6"}="H121 MZ6"; $resn_exch{"2H12 MZ6"}="H122 MZ6";
$resn_exch{"1H15 MZ6"}="H151 MZ6"; $resn_exch{"2H15 MZ6"}="H152 MZ6"; $resn_exch{"1H18 MZ6"}="H181 MZ6";
$resn_exch{"2H18 MZ6"}="H182 MZ6"; $resn_exch{"3H18 MZ6"}="H183 MZ6"; $resn_exch{"1H14 MZ6"}="H141 MZ6";
$resn_exch{"2H14 MZ6"}="H142 MZ6"; $resn_exch{"3H14 MZ6"}="H143 MZ6"; $resn_exch{"1H12 MZ7"}="H121 MZ7";
$resn_exch{"2H12 MZ7"}="H122 MZ7"; $resn_exch{"1H13 MZ7"}="H131 MZ7"; $resn_exch{"2H13 MZ7"}="H132 MZ7";
$resn_exch{"1H14 MZ7"}="H141 MZ7"; $resn_exch{"2H14 MZ7"}="H142 MZ7"; $resn_exch{"1H15 MZ7"}="H151 MZ7";
$resn_exch{"2H15 MZ7"}="H152 MZ7"; $resn_exch{"1H18 MZ7"}="H181 MZ7"; $resn_exch{"2H18 MZ7"}="H182 MZ7";
$resn_exch{"3H18 MZ7"}="H183 MZ7"; $resn_exch{"1H16 MZ7"}="H161 MZ7"; $resn_exch{"2H16 MZ7"}="H162 MZ7";
$resn_exch{"1H25 MZ7"}="H251 MZ7"; $resn_exch{"2H25 MZ7"}="H252 MZ7"; $resn_exch{"3H25 MZ7"}="H253 MZ7";
$resn_exch{"1H24 MZ7"}="H241 MZ7"; $resn_exch{"2H24 MZ7"}="H242 MZ7"; $resn_exch{"3H24 MZ7"}="H243 MZ7";
$resn_exch{"1H29 MZ7"}="H291 MZ7"; $resn_exch{"2H29 MZ7"}="H292 MZ7"; $resn_exch{"3H29 MZ7"}="H293 MZ7";
$resn_exch{"1H32 MZ7"}="H321 MZ7"; $resn_exch{"2H32 MZ7"}="H322 MZ7"; $resn_exch{"1H12 MZ8"}="H121 MZ8";
$resn_exch{"2H12 MZ8"}="H122 MZ8"; $resn_exch{"1H14 MZ8"}="H141 MZ8"; $resn_exch{"2H14 MZ8"}="H142 MZ8";
$resn_exch{"3H14 MZ8"}="H143 MZ8"; $resn_exch{"1H15 MZ8"}="H151 MZ8"; $resn_exch{"2H15 MZ8"}="H152 MZ8";
$resn_exch{"3H15 MZ8"}="H153 MZ8"; $resn_exch{"1H16 MZ8"}="H161 MZ8"; $resn_exch{"2H16 MZ8"}="H162 MZ8";
$resn_exch{"1H32 MZ8"}="H321 MZ8"; $resn_exch{"2H32 MZ8"}="H322 MZ8"; $resn_exch{"1H22 MZ9"}="H221 MZ9";
$resn_exch{"2H22 MZ9"}="H222 MZ9"; $resn_exch{"1H23 MZ9"}="H231 MZ9"; $resn_exch{"2H23 MZ9"}="H232 MZ9";
$resn_exch{"1H26 MZ9"}="H261 MZ9"; $resn_exch{"2H26 MZ9"}="H262 MZ9"; $resn_exch{"1H27 MZ9"}="H271 MZ9";
$resn_exch{"2H27 MZ9"}="H272 MZ9"; $resn_exch{"3H27 MZ9"}="H273 MZ9"; $resn_exch{"1H32 MZ9"}="H321 MZ9";
$resn_exch{"2H32 MZ9"}="H322 MZ9"; $resn_exch{"1H16 MZ9"}="H161 MZ9"; $resn_exch{"2H16 MZ9"}="H162 MZ9";
$resn_exch{"1H12 MZ9"}="H121 MZ9"; $resn_exch{"2H12 MZ9"}="H122 MZ9"; $resn_exch{"1H14 MZ9"}="H141 MZ9";
$resn_exch{"2H14 MZ9"}="H142 MZ9"; $resn_exch{"3H14 MZ9"}="H143 MZ9"; $resn_exch{"1H15 MZ9"}="H151 MZ9";
$resn_exch{"2H15 MZ9"}="H152 MZ9"; $resn_exch{"1H18 MZ9"}="H181 MZ9"; $resn_exch{"2H18 MZ9"}="H182 MZ9";
$resn_exch{"3H18 MZ9"}="H183 MZ9"; $resn_exch{"1H20 MZ9"}="H201 MZ9"; $resn_exch{"2H20 MZ9"}="H202 MZ9";
$resn_exch{"3H20 MZ9"}="H203 MZ9"; $resn_exch{"1HN1 MZM"}="HN11 MZM"; $resn_exch{"2HN1 MZM"}="HN12 MZM";
$resn_exch{"1H5  MZM"}=" H51 MZM"; $resn_exch{"2H5  MZM"}=" H52 MZM"; $resn_exch{"3H5  MZM"}=" H53 MZM";
$resn_exch{"1H4  MZM"}=" H41 MZM"; $resn_exch{"2H4  MZM"}=" H42 MZM"; $resn_exch{"3H4  MZM"}=" H43 MZM";
$resn_exch{"1H5* MZP"}="H5'1 MZP"; $resn_exch{"2H5* MZP"}="H5'2 MZP"; $resn_exch{"CH4* MZP"}="H4'C MZP";
$resn_exch{"CH3* MZP"}="H3'C MZP"; $resn_exch{"OH3* MZP"}="H3'O MZP"; $resn_exch{"CH2* MZP"}="H2'C MZP";
$resn_exch{"OH2* MZP"}="H2'O MZP"; $resn_exch{"CH1* MZP"}="H1'C MZP"; $resn_exch{"1HN6 MZP"}="HN61 MZP";
$resn_exch{"2HN6 MZP"}="HN62 MZP"; $resn_exch{"3HOP   N"}="HOP3   N"; $resn_exch{"2HOP   N"}="HOP2   N";
$resn_exch{"1H5*   N"}=" H5'   N"; $resn_exch{"2H5*   N"}="H5''   N"; $resn_exch{" H4*   N"}=" H4'   N";
$resn_exch{" H3*   N"}=" H3'   N"; $resn_exch{"H3T    N"}="HO3'   N"; $resn_exch{" H2*   N"}=" H2'   N";
$resn_exch{"2HO*   N"}="HO2'   N"; $resn_exch{" H1*   N"}=" H1'   N"; $resn_exch{"2H1*   N"}="H1'2   N";
$resn_exch{"1HN  N10"}="HN1A N10"; $resn_exch{"2HN  N10"}=" HN2 N10"; $resn_exch{"1HB  N10"}=" HB1 N10";
$resn_exch{"2HB  N10"}=" HB2 N10"; $resn_exch{"1HA1 N10"}="HA11 N10"; $resn_exch{"2HA1 N10"}="HA12 N10";
$resn_exch{"1HB2 N10"}="HB21 N10"; $resn_exch{"2HB2 N10"}="HB22 N10"; $resn_exch{"1HG3 N10"}="HG31 N10";
$resn_exch{"2HG3 N10"}="HG32 N10"; $resn_exch{"1HD4 N10"}="HD41 N10"; $resn_exch{"2HD4 N10"}="HD42 N10";
$resn_exch{"1HG5 N10"}="HG51 N10"; $resn_exch{"2HG5 N10"}="HG52 N10"; $resn_exch{"1HD6 N10"}="HD61 N10";
$resn_exch{"2HD6 N10"}="HD62 N10"; $resn_exch{"3HD6 N10"}="HD63 N10"; $resn_exch{"1H2  N12"}="H21A N12";
$resn_exch{"2H2  N12"}="H22A N12"; $resn_exch{"1H6  N12"}=" H61 N12"; $resn_exch{"2H6  N12"}=" H62 N12";
$resn_exch{"1H8  N12"}=" H81 N12"; $resn_exch{"2H8  N12"}=" H82 N12"; $resn_exch{"1H9  N12"}=" H91 N12";
$resn_exch{"2H9  N12"}=" H92 N12"; $resn_exch{"1H10 N12"}="H101 N12"; $resn_exch{"2H10 N12"}="H102 N12";
$resn_exch{"1H11 N12"}="H111 N12"; $resn_exch{"2H11 N12"}="H112 N12"; $resn_exch{"1H12 N12"}="H121 N12";
$resn_exch{"2H12 N12"}="H122 N12"; $resn_exch{"1H19 N12"}="H191 N12"; $resn_exch{"2H19 N12"}="H192 N12";
$resn_exch{"1H30 N12"}="H301 N12"; $resn_exch{"2H30 N12"}="H302 N12"; $resn_exch{"1H31 N12"}="H311 N12";
$resn_exch{"2H31 N12"}="H312 N12"; $resn_exch{"1H32 N12"}="H321 N12"; $resn_exch{"2H32 N12"}="H322 N12";
$resn_exch{"1H33 N12"}="H331 N12"; $resn_exch{"2H33 N12"}="H332 N12"; $resn_exch{"1H34 N12"}="H341 N12";
$resn_exch{"2H34 N12"}="H342 N12"; $resn_exch{"1H5D N1C"}="H5D1 N1C"; $resn_exch{"2H5D N1C"}="H5D2 N1C";
$resn_exch{" H1* N1C"}=" H1' N1C"; $resn_exch{" H2* N1C"}=" H2' N1C"; $resn_exch{"HO2* N1C"}="HO2' N1C";
$resn_exch{"HO3* N1C"}="HO3' N1C"; $resn_exch{"1H5* N1C"}="H5'1 N1C"; $resn_exch{"2H5* N1C"}="H5'2 N1C";
$resn_exch{"1HN7 N1H"}="HN71 N1H"; $resn_exch{"2HN7 N1H"}="HN72 N1H"; $resn_exch{"1H31 N1H"}="H311 N1H";
$resn_exch{"2H31 N1H"}="H312 N1H"; $resn_exch{"H2'' N1H"}=" H2B N1H"; $resn_exch{"H3'' N1H"}=" H3B N1H";
$resn_exch{"H4'' N1H"}=" H4B N1H"; $resn_exch{"H6'' N1H"}=" H6B N1H"; $resn_exch{"1HM2 N1T"}="HM21 N1T";
$resn_exch{"2HM2 N1T"}="HM22 N1T"; $resn_exch{"3HM2 N1T"}="HM23 N1T"; $resn_exch{"1HN4 N1T"}="HN41 N1T";
$resn_exch{"2HN4 N1T"}="HN42 N1T"; $resn_exch{"1H7' N1T"}="H7'1 N1T"; $resn_exch{"2H7' N1T"}="H7'2 N1T";
$resn_exch{"1HM4 N1T"}="HM41 N1T"; $resn_exch{"2HM4 N1T"}="HM42 N1T"; $resn_exch{"3HM4 N1T"}="HM43 N1T";
$resn_exch{"1H6  N1T"}=" H61 N1T"; $resn_exch{"2H6  N1T"}=" H62 N1T"; $resn_exch{"1H7  N1T"}=" H71 N1T";
$resn_exch{"2H7  N1T"}=" H72 N1T"; $resn_exch{"2HOA N1T"}="HOA2 N1T"; $resn_exch{"3HOB N1T"}="HOB3 N1T";
$resn_exch{"1H10 N20"}="H101 N20"; $resn_exch{"2H10 N20"}="H102 N20"; $resn_exch{"1H12 N20"}="H121 N20";
$resn_exch{"2H12 N20"}="H122 N20"; $resn_exch{"1H13 N20"}="H131 N20"; $resn_exch{"2H13 N20"}="H132 N20";
$resn_exch{"1H14 N20"}="H141 N20"; $resn_exch{"2H14 N20"}="H142 N20"; $resn_exch{"1H15 N20"}="H151 N20";
$resn_exch{"2H15 N20"}="H152 N20"; $resn_exch{"1H16 N20"}="H161 N20"; $resn_exch{"2H16 N20"}="H162 N20";
$resn_exch{"1H8  N25"}=" H81 N25"; $resn_exch{"2H8  N25"}=" H82 N25"; $resn_exch{"3H8  N25"}=" H83 N25";
$resn_exch{"1H15 N25"}="H151 N25"; $resn_exch{"2H15 N25"}="H152 N25"; $resn_exch{"1H17 N25"}="H171 N25";
$resn_exch{"2H17 N25"}="H172 N25"; $resn_exch{"1H18 N25"}="H181 N25"; $resn_exch{"2H18 N25"}="H182 N25";
$resn_exch{"3H18 N25"}="H183 N25"; $resn_exch{"1H19 N25"}="H191 N25"; $resn_exch{"2H19 N25"}="H192 N25";
$resn_exch{"3H19 N25"}="H193 N25"; $resn_exch{"1H34 N25"}="H341 N25"; $resn_exch{"2H34 N25"}="H342 N25";
$resn_exch{"3H34 N25"}="H343 N25"; $resn_exch{"1H1' N2B"}="H1'1 N2B"; $resn_exch{"2H1' N2B"}="H1'2 N2B";
$resn_exch{"1H2' N2B"}="H2'1 N2B"; $resn_exch{"2H2' N2B"}="H2'2 N2B"; $resn_exch{"3H2' N2B"}="H2'3 N2B";
$resn_exch{" HN  N2C"}=" H   N2C"; $resn_exch{"1HB  N2C"}=" HB2 N2C"; $resn_exch{"2HB  N2C"}=" HB3 N2C";
$resn_exch{"1HD  N2C"}=" HD1 N2C"; $resn_exch{"2HD  N2C"}=" HD2 N2C"; $resn_exch{"3HD  N2C"}=" HD3 N2C";
$resn_exch{"1HN  N2C"}=" HN1 N2C"; $resn_exch{"2HN  N2C"}=" HN2 N2C"; $resn_exch{"3HN  N2C"}=" HN3 N2C";
$resn_exch{"1HN  N2M"}=" HN1 N2M"; $resn_exch{"2HN  N2M"}=" HN2 N2M"; $resn_exch{"1H1C N2P"}="H1C1 N2P";
$resn_exch{"2H1C N2P"}="H1C2 N2P"; $resn_exch{"1H2C N2P"}="H2C1 N2P"; $resn_exch{"2H2C N2P"}="H2C2 N2P";
$resn_exch{"1H3C N2P"}="H3C1 N2P"; $resn_exch{"2H3C N2P"}="H3C2 N2P"; $resn_exch{"1H4C N2P"}="H4C1 N2P";
$resn_exch{"2H4C N2P"}="H4C2 N2P"; $resn_exch{"1HE2 N2P"}="HE21 N2P"; $resn_exch{"2HE2 N2P"}="HE22 N2P";
$resn_exch{"1H5C N2P"}="H5C1 N2P"; $resn_exch{"2H5C N2P"}="H5C2 N2P"; $resn_exch{"1H1N N2P"}="H1N1 N2P";
$resn_exch{"2H1N N2P"}="H1N2 N2P"; $resn_exch{"1H11 N2T"}="H111 N2T"; $resn_exch{"2H11 N2T"}="H112 N2T";
$resn_exch{"1H15 N2T"}="H151 N2T"; $resn_exch{"2H15 N2T"}="H152 N2T"; $resn_exch{"1H26 N2T"}="H261 N2T";
$resn_exch{"2H26 N2T"}="H262 N2T"; $resn_exch{"3H26 N2T"}="H263 N2T"; $resn_exch{"1H12 N2T"}="H121 N2T";
$resn_exch{"2H12 N2T"}="H122 N2T"; $resn_exch{"1H9  N2T"}=" H91 N2T"; $resn_exch{"2H9  N2T"}=" H92 N2T";
$resn_exch{"1H16 N2T"}="H161 N2T"; $resn_exch{"2H16 N2T"}="H162 N2T"; $resn_exch{"1H21 N30"}="H211 N30";
$resn_exch{"2H21 N30"}="H212 N30"; $resn_exch{"1H61 N30"}="H611 N30"; $resn_exch{"2H61 N30"}="H612 N30";
$resn_exch{"1H12 N30"}="H121 N30"; $resn_exch{"2H12 N30"}="H122 N30"; $resn_exch{"1H22 N30"}="H221 N30";
$resn_exch{"2H22 N30"}="H222 N30"; $resn_exch{"1H32 N30"}="H321 N30"; $resn_exch{"2H32 N30"}="H322 N30";
$resn_exch{"1H53 N30"}="H531 N30"; $resn_exch{"2H53 N30"}="H532 N30"; $resn_exch{"1H21 N33"}="H211 N33";
$resn_exch{"2H21 N33"}="H212 N33"; $resn_exch{"1H61 N33"}="H611 N33"; $resn_exch{"2H61 N33"}="H612 N33";
$resn_exch{"1H12 N33"}="H121 N33"; $resn_exch{"2H12 N33"}="H122 N33"; $resn_exch{"1H22 N33"}="H221 N33";
$resn_exch{"2H22 N33"}="H222 N33"; $resn_exch{"1H32 N33"}="H321 N33"; $resn_exch{"2H32 N33"}="H322 N33";
$resn_exch{"1H23 N33"}="H231 N33"; $resn_exch{"2H23 N33"}="H232 N33"; $resn_exch{"1H63 N33"}="H631 N33";
$resn_exch{"2H63 N33"}="H632 N33"; $resn_exch{"1H77 N33"}="H771 N33"; $resn_exch{"2H77 N33"}="H772 N33";
$resn_exch{"1H41 N3C"}="H411 N3C"; $resn_exch{"2H41 N3C"}="H412 N3C"; $resn_exch{"1H37 N3C"}="H371 N3C";
$resn_exch{"2H37 N3C"}="H372 N3C"; $resn_exch{"1H38 N3C"}="H381 N3C"; $resn_exch{"2H38 N3C"}="H382 N3C";
$resn_exch{"1H40 N3C"}="H401 N3C"; $resn_exch{"2H40 N3C"}="H402 N3C"; $resn_exch{"3H40 N3C"}="H403 N3C";
$resn_exch{"1H39 N3C"}="H391 N3C"; $resn_exch{"2H39 N3C"}="H392 N3C"; $resn_exch{"3H39 N3C"}="H393 N3C";
$resn_exch{"1H16 N3C"}="H161 N3C"; $resn_exch{"2H16 N3C"}="H162 N3C"; $resn_exch{"1H15 N3C"}="H151 N3C";
$resn_exch{"2H15 N3C"}="H152 N3C"; $resn_exch{"1H18 N3C"}="H181 N3C"; $resn_exch{"2H18 N3C"}="H182 N3C";
$resn_exch{"1H19 N3C"}="H191 N3C"; $resn_exch{"2H19 N3C"}="H192 N3C"; $resn_exch{"1H13 N3C"}="H131 N3C";
$resn_exch{"2H13 N3C"}="H132 N3C"; $resn_exch{"1H5' N3E"}="H5'1 N3E"; $resn_exch{"2H5' N3E"}="H5'2 N3E";
$resn_exch{"1H13 N3E"}="H131 N3E"; $resn_exch{"2H13 N3E"}="H132 N3E"; $resn_exch{"1H8  N3E"}=" H81 N3E";
$resn_exch{"2H8  N3E"}=" H82 N3E"; $resn_exch{"1H9  N3E"}=" H91 N3E"; $resn_exch{"2H9  N3E"}=" H92 N3E";
$resn_exch{"1H12 N3E"}="H121 N3E"; $resn_exch{"2H12 N3E"}="H122 N3E"; $resn_exch{"1HM2 N3T"}="HM21 N3T";
$resn_exch{"2HM2 N3T"}="HM22 N3T"; $resn_exch{"3HM2 N3T"}="HM23 N3T"; $resn_exch{"1HN4 N3T"}="HN41 N3T";
$resn_exch{"2HN4 N3T"}="HN42 N3T"; $resn_exch{"1H7' N3T"}="H7'1 N3T"; $resn_exch{"2H7' N3T"}="H7'2 N3T";
$resn_exch{"1HM4 N3T"}="HM41 N3T"; $resn_exch{"2HM4 N3T"}="HM42 N3T"; $resn_exch{"3HM4 N3T"}="HM43 N3T";
$resn_exch{"1H6  N3T"}=" H61 N3T"; $resn_exch{"2H6  N3T"}=" H62 N3T"; $resn_exch{"1H7  N3T"}=" H71 N3T";
$resn_exch{"2H7  N3T"}=" H72 N3T"; $resn_exch{"2HOA N3T"}="HOA2 N3T"; $resn_exch{"3HOB N3T"}="HOB3 N3T";
$resn_exch{"1H10 N41"}="H101 N41"; $resn_exch{"2H10 N41"}="H102 N41"; $resn_exch{"1H12 N41"}="H121 N41";
$resn_exch{"2H12 N41"}="H122 N41"; $resn_exch{"1H13 N41"}="H131 N41"; $resn_exch{"2H13 N41"}="H132 N41";
$resn_exch{"1H14 N41"}="H141 N41"; $resn_exch{"2H14 N41"}="H142 N41"; $resn_exch{"1H15 N41"}="H151 N41";
$resn_exch{"2H15 N41"}="H152 N41"; $resn_exch{"1H16 N41"}="H161 N41"; $resn_exch{"2H16 N41"}="H162 N41";
$resn_exch{"1H25 N41"}="H251 N41"; $resn_exch{"2H25 N41"}="H252 N41"; $resn_exch{"1H1' N4B"}="H1'1 N4B";
$resn_exch{"2H1' N4B"}="H1'2 N4B"; $resn_exch{"1H2' N4B"}="H2'1 N4B"; $resn_exch{"2H2' N4B"}="H2'2 N4B";
$resn_exch{"1H3' N4B"}="H3'1 N4B"; $resn_exch{"2H3' N4B"}="H3'2 N4B"; $resn_exch{"1H4' N4B"}="H4'1 N4B";
$resn_exch{"2H4' N4B"}="H4'2 N4B"; $resn_exch{"3H4' N4B"}="H4'3 N4B"; $resn_exch{"1H11 N4T"}="H111 N4T";
$resn_exch{"2H11 N4T"}="H112 N4T"; $resn_exch{"1H15 N4T"}="H151 N4T"; $resn_exch{"2H15 N4T"}="H152 N4T";
$resn_exch{"1HN1 N4T"}="HN11 N4T"; $resn_exch{"2HN1 N4T"}="HN12 N4T"; $resn_exch{"1H12 N4T"}="H121 N4T";
$resn_exch{"2H12 N4T"}="H122 N4T"; $resn_exch{"1H53 N5A"}="H531 N5A"; $resn_exch{"2H53 N5A"}="H532 N5A";
$resn_exch{"1H52 N5A"}="H521 N5A"; $resn_exch{"2H52 N5A"}="H522 N5A"; $resn_exch{"1H51 N5A"}="H511 N5A";
$resn_exch{"2H51 N5A"}="H512 N5A"; $resn_exch{"HN5* N5A"}="HN5' N5A"; $resn_exch{" H4* N5A"}=" H4' N5A";
$resn_exch{" H3* N5A"}=" H3' N5A"; $resn_exch{"HO3* N5A"}="HO3' N5A"; $resn_exch{" H2* N5A"}=" H2' N5A";
$resn_exch{"HO2* N5A"}="HO2' N5A"; $resn_exch{" H1* N5A"}=" H1' N5A"; $resn_exch{"1HN6 N5A"}="HN61 N5A";
$resn_exch{"2HN6 N5A"}="HN62 N5A"; $resn_exch{"1H5C N5B"}="H5C1 N5B"; $resn_exch{"2H5C N5B"}="H5C2 N5B";
$resn_exch{"1H6C N5B"}="H6C1 N5B"; $resn_exch{"2H6C N5B"}="H6C2 N5B"; $resn_exch{"3HOP N5I"}="HOP3 N5I";
$resn_exch{" H1* N5I"}=" H1' N5I"; $resn_exch{" H4* N5I"}=" H4' N5I"; $resn_exch{" H3* N5I"}=" H3' N5I";
$resn_exch{"1H2* N5I"}="H2'1 N5I"; $resn_exch{"2H2* N5I"}="H2'2 N5I"; $resn_exch{"1H5* N5I"}="H5'1 N5I";
$resn_exch{"2H5* N5I"}="H5'2 N5I"; $resn_exch{"2HOP N5I"}="HOP2 N5I"; $resn_exch{"1H52 N5O"}="H521 N5O";
$resn_exch{"2H52 N5O"}="H522 N5O"; $resn_exch{"1H51 N5O"}="H511 N5O"; $resn_exch{"2H51 N5O"}="H512 N5O";
$resn_exch{"HN5* N5O"}="HN5' N5O"; $resn_exch{" H4* N5O"}=" H4' N5O"; $resn_exch{" H3* N5O"}=" H3' N5O";
$resn_exch{"HO3* N5O"}="HO3' N5O"; $resn_exch{" H2* N5O"}=" H2' N5O"; $resn_exch{"HO2* N5O"}="HO2' N5O";
$resn_exch{" H1* N5O"}=" H1' N5O"; $resn_exch{"1HN6 N5O"}="HN61 N5O"; $resn_exch{"2HN6 N5O"}="HN62 N5O";
$resn_exch{" H1* N5P"}=" H1' N5P"; $resn_exch{" H4* N5P"}=" H4' N5P"; $resn_exch{" H3* N5P"}=" H3' N5P";
$resn_exch{"HO3* N5P"}="HO3' N5P"; $resn_exch{"1H2* N5P"}="H2'1 N5P"; $resn_exch{"2H2* N5P"}="H2'2 N5P";
$resn_exch{"1H5* N5P"}="H5'1 N5P"; $resn_exch{"2H5* N5P"}="H5'2 N5P"; $resn_exch{"3HOG N5P"}="HOG3 N5P";
$resn_exch{"2HOG N5P"}="HOG2 N5P"; $resn_exch{"1H11 N5T"}="H111 N5T"; $resn_exch{"2H11 N5T"}="H112 N5T";
$resn_exch{"1H31 N5T"}="H311 N5T"; $resn_exch{"2H31 N5T"}="H312 N5T"; $resn_exch{"3H31 N5T"}="H313 N5T";
$resn_exch{"1H35 N5T"}="H351 N5T"; $resn_exch{"2H35 N5T"}="H352 N5T"; $resn_exch{"3H35 N5T"}="H353 N5T";
$resn_exch{"1H2N N69"}="H2N1 N69"; $resn_exch{"2H2N N69"}="H2N2 N69"; $resn_exch{"1H19 N69"}="H191 N69";
$resn_exch{"2H19 N69"}="H192 N69"; $resn_exch{"3H19 N69"}="H193 N69"; $resn_exch{"1H18 N69"}="H181 N69";
$resn_exch{"2H18 N69"}="H182 N69"; $resn_exch{"3H18 N69"}="H183 N69"; $resn_exch{"1H10 N69"}="H101 N69";
$resn_exch{"2H10 N69"}="H102 N69"; $resn_exch{"1H12 N69"}="H121 N69"; $resn_exch{"2H12 N69"}="H122 N69";
$resn_exch{"1H13 N69"}="H131 N69"; $resn_exch{"2H13 N69"}="H132 N69"; $resn_exch{"1H14 N69"}="H141 N69";
$resn_exch{"2H14 N69"}="H142 N69"; $resn_exch{"1H15 N69"}="H151 N69"; $resn_exch{"2H15 N69"}="H152 N69";
$resn_exch{"1H16 N69"}="H161 N69"; $resn_exch{"2H16 N69"}="H162 N69"; $resn_exch{"2HOP N6G"}="HOP2 N6G";
$resn_exch{"1H5* N6G"}=" H5' N6G"; $resn_exch{"2H5* N6G"}="H5'' N6G"; $resn_exch{" H4* N6G"}=" H4' N6G";
$resn_exch{" H1* N6G"}=" H1' N6G"; $resn_exch{"1HN2 N6G"}="HN21 N6G"; $resn_exch{"2HN2 N6G"}="HN22 N6G";
$resn_exch{"1HN6 N6G"}="HN61 N6G"; $resn_exch{"2HN6 N6G"}="HN62 N6G"; $resn_exch{" H2* N6G"}=" H2' N6G";
$resn_exch{"2HO* N6G"}="HO2' N6G"; $resn_exch{" H3* N6G"}=" H3' N6G"; $resn_exch{"H3T  N6G"}="HO3' N6G";
$resn_exch{"3HOP N6G"}="HOP3 N6G"; $resn_exch{"1H10 N76"}="H101 N76"; $resn_exch{"2H10 N76"}="H102 N76";
$resn_exch{"1H12 N76"}="H121 N76"; $resn_exch{"2H12 N76"}="H122 N76"; $resn_exch{"1H13 N76"}="H131 N76";
$resn_exch{"2H13 N76"}="H132 N76"; $resn_exch{"1H14 N76"}="H141 N76"; $resn_exch{"2H14 N76"}="H142 N76";
$resn_exch{"1H15 N76"}="H151 N76"; $resn_exch{"2H15 N76"}="H152 N76"; $resn_exch{"1H16 N76"}="H161 N76";
$resn_exch{"2H16 N76"}="H162 N76"; $resn_exch{"1HN2 N76"}="HN21 N76"; $resn_exch{"1H26 N76"}="H261 N76";
$resn_exch{"2H26 N76"}="H262 N76"; $resn_exch{"1H8  N7P"}=" H81 N7P"; $resn_exch{"2H8  N7P"}=" H82 N7P";
$resn_exch{"3H8  N7P"}=" H83 N7P"; $resn_exch{"1H5  N7P"}=" H51 N7P"; $resn_exch{"2H5  N7P"}=" H52 N7P";
$resn_exch{"1H4  N7P"}=" H41 N7P"; $resn_exch{"2H4  N7P"}=" H42 N7P"; $resn_exch{"1H3  N7P"}=" H31 N7P";
$resn_exch{"2H3  N7P"}=" H32 N7P"; $resn_exch{"1H01 N8E"}="H011 N8E"; $resn_exch{"2H01 N8E"}="H012 N8E";
$resn_exch{"3H01 N8E"}="H013 N8E"; $resn_exch{"1H02 N8E"}="H021 N8E"; $resn_exch{"2H02 N8E"}="H022 N8E";
$resn_exch{"1H03 N8E"}="H031 N8E"; $resn_exch{"2H03 N8E"}="H032 N8E"; $resn_exch{"1H04 N8E"}="H041 N8E";
$resn_exch{"2H04 N8E"}="H042 N8E"; $resn_exch{"1H05 N8E"}="H051 N8E"; $resn_exch{"2H05 N8E"}="H052 N8E";
$resn_exch{"1H06 N8E"}="H061 N8E"; $resn_exch{"2H06 N8E"}="H062 N8E"; $resn_exch{"1H07 N8E"}="H071 N8E";
$resn_exch{"2H07 N8E"}="H072 N8E"; $resn_exch{"1H08 N8E"}="H081 N8E"; $resn_exch{"2H08 N8E"}="H082 N8E";
$resn_exch{"1H10 N8E"}="H101 N8E"; $resn_exch{"2H10 N8E"}="H102 N8E"; $resn_exch{"1H11 N8E"}="H111 N8E";
$resn_exch{"2H11 N8E"}="H112 N8E"; $resn_exch{"1H13 N8E"}="H131 N8E"; $resn_exch{"2H13 N8E"}="H132 N8E";
$resn_exch{"1H14 N8E"}="H141 N8E"; $resn_exch{"2H14 N8E"}="H142 N8E"; $resn_exch{"1H16 N8E"}="H161 N8E";
$resn_exch{"2H16 N8E"}="H162 N8E"; $resn_exch{"1H17 N8E"}="H171 N8E"; $resn_exch{"2H17 N8E"}="H172 N8E";
$resn_exch{"1H19 N8E"}="H191 N8E"; $resn_exch{"2H19 N8E"}="H192 N8E"; $resn_exch{"1H20 N8E"}="H201 N8E";
$resn_exch{"2H20 N8E"}="H202 N8E"; $resn_exch{"1H22 N8E"}="H221 N8E"; $resn_exch{"2H22 N8E"}="H222 N8E";
$resn_exch{"1H23 N8E"}="H231 N8E"; $resn_exch{"2H23 N8E"}="H232 N8E"; $resn_exch{"1HBD N8T"}="HBD1 N8T";
$resn_exch{"2HBD N8T"}="HBD2 N8T"; $resn_exch{"1HBE N8T"}="HBE1 N8T"; $resn_exch{"2HBE N8T"}="HBE2 N8T";
$resn_exch{"1HAQ N8T"}="HAQ1 N8T"; $resn_exch{"2HAQ N8T"}="HAQ2 N8T"; $resn_exch{"1HAP N8T"}="HAP1 N8T";
$resn_exch{"2HAP N8T"}="HAP2 N8T"; $resn_exch{"1HAB N8T"}="HAB1 N8T"; $resn_exch{"2HAB N8T"}="HAB2 N8T";
$resn_exch{"1HAC N8T"}="HAC1 N8T"; $resn_exch{"2HAC N8T"}="HAC2 N8T"; $resn_exch{"1HAD N8T"}="HAD1 N8T";
$resn_exch{"2HAD N8T"}="HAD2 N8T"; $resn_exch{"1HAE N8T"}="HAE1 N8T"; $resn_exch{"2HAE N8T"}="HAE2 N8T";
$resn_exch{"1HAF N8T"}="HAF1 N8T"; $resn_exch{"2HAF N8T"}="HAF2 N8T"; $resn_exch{"1HAG N8T"}="HAG1 N8T";
$resn_exch{"2HAG N8T"}="HAG2 N8T"; $resn_exch{"1HAH N8T"}="HAH1 N8T"; $resn_exch{"2HAH N8T"}="HAH2 N8T";
$resn_exch{"1HAI N8T"}="HAI1 N8T"; $resn_exch{"2HAI N8T"}="HAI2 N8T"; $resn_exch{"1HAW N8T"}="HAW1 N8T";
$resn_exch{"2HAW N8T"}="HAW2 N8T"; $resn_exch{"1HAX N8T"}="HAX1 N8T"; $resn_exch{"2HAX N8T"}="HAX2 N8T";
$resn_exch{"1HBL N8T"}="HBL1 N8T"; $resn_exch{"2HBL N8T"}="HBL2 N8T"; $resn_exch{"1HBK N8T"}="HBK1 N8T";
$resn_exch{"2HBK N8T"}="HBK2 N8T"; $resn_exch{"1H14 N9H"}="H141 N9H"; $resn_exch{"2H14 N9H"}="H142 N9H";
$resn_exch{"1H31 N9H"}="H311 N9H"; $resn_exch{"2H31 N9H"}="H312 N9H"; $resn_exch{"1H7  N9H"}=" H71 N9H";
$resn_exch{"2H7  N9H"}=" H72 N9H"; $resn_exch{"1H8  N9H"}=" H81 N9H"; $resn_exch{"2H8  N9H"}=" H82 N9H";
$resn_exch{"1H9  N9H"}=" H91 N9H"; $resn_exch{"2H9  N9H"}=" H92 N9H"; $resn_exch{"3H9  N9H"}=" H93 N9H";
$resn_exch{"1H11 N9H"}="H111 N9H"; $resn_exch{"2H11 N9H"}="H112 N9H"; $resn_exch{"1HN2 N9H"}="HN21 N9H";
$resn_exch{"2HN2 N9H"}="HN22 N9H"; $resn_exch{"1H1C NA1"}="H1C1 NA1"; $resn_exch{"1H6C NA1"}="H6C1 NA1";
$resn_exch{"2H6C NA1"}="H6C2 NA1"; $resn_exch{"1H8C NA1"}="H8C1 NA1"; $resn_exch{"2H8C NA1"}="H8C2 NA1";
$resn_exch{"3H8C NA1"}="H8C3 NA1"; $resn_exch{"1HO  NA1"}=" HO1 NA1"; $resn_exch{"1H9C NA1"}="H9C1 NA1";
$resn_exch{"2H9C NA1"}="H9C2 NA1"; $resn_exch{"3H9C NA1"}="H9C3 NA1"; $resn_exch{"1HO1 NA2"}="HO11 NA2";
$resn_exch{"2HO1 NA2"}="HO12 NA2"; $resn_exch{"1HO2 NA2"}="HO21 NA2"; $resn_exch{"2HO2 NA2"}="HO22 NA2";
$resn_exch{"1H2  NA3"}=" H21 NA3"; $resn_exch{"2H2  NA3"}=" H22 NA3"; $resn_exch{"1H2  NA4"}=" H21 NA4";
$resn_exch{"2H2  NA4"}=" H22 NA4"; $resn_exch{"1HO1 NA5"}="HO11 NA5"; $resn_exch{"2HO1 NA5"}="HO12 NA5";
$resn_exch{"1HO2 NA5"}="HO21 NA5"; $resn_exch{"2HO2 NA5"}="HO22 NA5"; $resn_exch{"1HO3 NA5"}="HO31 NA5";
$resn_exch{"2HO3 NA5"}="HO32 NA5"; $resn_exch{"1HO4 NA5"}="HO41 NA5"; $resn_exch{"2HO4 NA5"}="HO42 NA5";
$resn_exch{"1HO5 NA5"}="HO51 NA5"; $resn_exch{"2HO5 NA5"}="HO52 NA5"; $resn_exch{"1HO1 NA6"}="HO11 NA6";
$resn_exch{"2HO1 NA6"}="HO12 NA6"; $resn_exch{"1HO2 NA6"}="HO21 NA6"; $resn_exch{"2HO2 NA6"}="HO22 NA6";
$resn_exch{"1HO3 NA6"}="HO31 NA6"; $resn_exch{"2HO3 NA6"}="HO32 NA6"; $resn_exch{"1HO4 NA6"}="HO41 NA6";
$resn_exch{"2HO4 NA6"}="HO42 NA6"; $resn_exch{"1HO5 NA6"}="HO51 NA6"; $resn_exch{"2HO5 NA6"}="HO52 NA6";
$resn_exch{"1HO6 NA6"}="HO61 NA6"; $resn_exch{"2HO6 NA6"}="HO62 NA6"; $resn_exch{"1H5* NA7"}="H5'1 NA7";
$resn_exch{"2H5* NA7"}="H5'2 NA7"; $resn_exch{"HC4* NA7"}="HC4' NA7"; $resn_exch{"HC3* NA7"}="HC3' NA7";
$resn_exch{"HO3* NA7"}="HO3' NA7"; $resn_exch{"HC2* NA7"}="HC2' NA7"; $resn_exch{"HC1* NA7"}="HC1' NA7";
$resn_exch{"1HN6 NA7"}="HN61 NA7"; $resn_exch{"2HN6 NA7"}="HN62 NA7"; $resn_exch{"H51* NA7"}="H51' NA7";
$resn_exch{"H52* NA7"}="H52' NA7"; $resn_exch{"HN4* NA7"}="HN4' NA7"; $resn_exch{"HN3* NA7"}="HN3' NA7";
$resn_exch{"HN5* NA7"}="HN5' NA7"; $resn_exch{"HN6* NA7"}="HN6' NA7"; $resn_exch{"HN7* NA7"}="HN7' NA7";
$resn_exch{"1H1* NA7"}="H1'1 NA7"; $resn_exch{"2H1* NA7"}="H1'2 NA7"; $resn_exch{"1H7  NA9"}=" H71 NA9";
$resn_exch{"2H7  NA9"}=" H72 NA9"; $resn_exch{"1HN9 NA9"}="HN91 NA9"; $resn_exch{"2HN9 NA9"}="HN92 NA9";
$resn_exch{"1H15 NA9"}="H151 NA9"; $resn_exch{"2H15 NA9"}="H152 NA9"; $resn_exch{"3H15 NA9"}="H153 NA9";
$resn_exch{"1H16 NA9"}="H161 NA9"; $resn_exch{"2H16 NA9"}="H162 NA9"; $resn_exch{"3H16 NA9"}="H163 NA9";
$resn_exch{"1H22 NA9"}="H221 NA9"; $resn_exch{"2H22 NA9"}="H222 NA9"; $resn_exch{"1H26 NA9"}="H261 NA9";
$resn_exch{"2H26 NA9"}="H262 NA9"; $resn_exch{"1H28 NA9"}="H281 NA9"; $resn_exch{"2H28 NA9"}="H282 NA9";
$resn_exch{"1H29 NA9"}="H291 NA9"; $resn_exch{"2H29 NA9"}="H292 NA9"; $resn_exch{"1H30 NA9"}="H301 NA9";
$resn_exch{"2H30 NA9"}="H302 NA9"; $resn_exch{"1H31 NA9"}="H311 NA9"; $resn_exch{"2H31 NA9"}="H312 NA9";
$resn_exch{"1H32 NA9"}="H321 NA9"; $resn_exch{"2H32 NA9"}="H322 NA9"; $resn_exch{"1H6  NAA"}=" H61 NAA";
$resn_exch{"2H6  NAA"}=" H62 NAA"; $resn_exch{"1H8  NAA"}=" H81 NAA"; $resn_exch{"2H8  NAA"}=" H82 NAA";
$resn_exch{"3H8  NAA"}=" H83 NAA"; $resn_exch{"'HO4 NAB"}="HO4' NAB"; $resn_exch{"2HOA NAC"}="HOA2 NAC";
$resn_exch{"AH51 NAC"}="H51A NAC"; $resn_exch{"AH52 NAC"}="H52A NAC"; $resn_exch{"AH4* NAC"}=" H4B NAC";
$resn_exch{"AH3* NAC"}=" H3B NAC"; $resn_exch{"AHO3 NAC"}="HO3A NAC"; $resn_exch{"AH2* NAC"}=" H2B NAC";
$resn_exch{"AHO2 NAC"}="HO2A NAC"; $resn_exch{"AH1* NAC"}=" H1B NAC"; $resn_exch{" AH8 NAC"}=" H8A NAC";
$resn_exch{"AH61 NAC"}="H61A NAC"; $resn_exch{"AH62 NAC"}="H62A NAC"; $resn_exch{" AH2 NAC"}=" H2A NAC";
$resn_exch{"NH51 NAC"}="H51N NAC"; $resn_exch{"NH52 NAC"}="H52N NAC"; $resn_exch{"NH4* NAC"}=" H4D NAC";
$resn_exch{"NH3* NAC"}=" H3D NAC"; $resn_exch{"NHO3 NAC"}="HO3N NAC"; $resn_exch{"NH2* NAC"}=" H2D NAC";
$resn_exch{"NHO2 NAC"}="HO2N NAC"; $resn_exch{"NH1* NAC"}=" H1D NAC"; $resn_exch{" NH2 NAC"}=" H2N NAC";
$resn_exch{"NH81 NAC"}="H81N NAC"; $resn_exch{"NH82 NAC"}="H82N NAC"; $resn_exch{"NH83 NAC"}="H83N NAC";
$resn_exch{" NH4 NAC"}=" H4N NAC"; $resn_exch{" NH5 NAC"}=" H5N NAC"; $resn_exch{" NH6 NAC"}=" H6N NAC";
$resn_exch{"2HOA NAD"}="HOA2 NAD"; $resn_exch{"AH51 NAD"}="H51A NAD"; $resn_exch{"AH52 NAD"}="H52A NAD";
$resn_exch{"AH4* NAD"}=" H4B NAD"; $resn_exch{"AH3* NAD"}=" H3B NAD"; $resn_exch{"AHO3 NAD"}="HO3A NAD";
$resn_exch{"AH2* NAD"}=" H2B NAD"; $resn_exch{"AHO2 NAD"}="HO2A NAD"; $resn_exch{"AH1* NAD"}=" H1B NAD";
$resn_exch{" AH8 NAD"}=" H8A NAD"; $resn_exch{"AH61 NAD"}="H61A NAD"; $resn_exch{"AH62 NAD"}="H62A NAD";
$resn_exch{" AH2 NAD"}=" H2A NAD"; $resn_exch{"NH51 NAD"}="H51N NAD"; $resn_exch{"NH52 NAD"}="H52N NAD";
$resn_exch{"NH4* NAD"}=" H4D NAD"; $resn_exch{"NH3* NAD"}=" H3D NAD"; $resn_exch{"NHO3 NAD"}="HO3N NAD";
$resn_exch{"NH2* NAD"}=" H2D NAD"; $resn_exch{"NHO2 NAD"}="HO2N NAD"; $resn_exch{"NH1* NAD"}=" H1D NAD";
$resn_exch{" NH2 NAD"}=" H2N NAD"; $resn_exch{"NH71 NAD"}="H71N NAD"; $resn_exch{"NH72 NAD"}="H72N NAD";
$resn_exch{" NH4 NAD"}=" H4N NAD"; $resn_exch{" NH5 NAD"}=" H5N NAD"; $resn_exch{" NH6 NAD"}=" H6N NAD";
$resn_exch{"2HOA NAE"}="HOA2 NAE"; $resn_exch{"1H5' NAE"}="H5'1 NAE"; $resn_exch{"2H5' NAE"}="H5'2 NAE";
$resn_exch{"3HO' NAE"}="HO'3 NAE"; $resn_exch{"2HO' NAE"}="HO'2 NAE"; $resn_exch{"1H6A NAE"}="H6A1 NAE";
$resn_exch{"2H6A NAE"}="H6A2 NAE"; $resn_exch{"1H5* NAE"}="H5A1 NAE"; $resn_exch{"2H5* NAE"}="H5A2 NAE";
$resn_exch{" H4* NAE"}=" H4B NAE"; $resn_exch{" H3* NAE"}=" H3B NAE"; $resn_exch{"3HO* NAE"}="HOA3 NAE";
$resn_exch{" H2* NAE"}=" H2B NAE"; $resn_exch{"2HO* NAE"}="HOB2 NAE"; $resn_exch{" H1* NAE"}=" H1B NAE";
$resn_exch{"1H7N NAE"}="H7N1 NAE"; $resn_exch{"2H7N NAE"}="H7N2 NAE"; $resn_exch{"1H4P NAE"}="H4P1 NAE";
$resn_exch{"2H4P NAE"}="H4P2 NAE"; $resn_exch{"1H2P NAE"}="H2P1 NAE"; $resn_exch{"2H2P NAE"}="H2P2 NAE";
$resn_exch{"3H2P NAE"}="H2P3 NAE"; $resn_exch{"1HM1 NAF"}="HM11 NAF"; $resn_exch{"2HM1 NAF"}="HM12 NAF";
$resn_exch{"3HM1 NAF"}="HM13 NAF"; $resn_exch{"1HM2 NAF"}="HM21 NAF"; $resn_exch{"2HM2 NAF"}="HM22 NAF";
$resn_exch{"3HM2 NAF"}="HM23 NAF"; $resn_exch{"1HM3 NAF"}="HM31 NAF"; $resn_exch{"2HM3 NAF"}="HM32 NAF";
$resn_exch{"3HM3 NAF"}="HM33 NAF"; $resn_exch{"1H6  NAG"}=" H61 NAG"; $resn_exch{"2H6  NAG"}=" H62 NAG";
$resn_exch{"1H8  NAG"}=" H81 NAG"; $resn_exch{"2H8  NAG"}=" H82 NAG"; $resn_exch{"3H8  NAG"}=" H83 NAG";
$resn_exch{"2HOA NAH"}="HOA2 NAH"; $resn_exch{"AH51 NAH"}="H51A NAH"; $resn_exch{"AH52 NAH"}="H52A NAH";
$resn_exch{"AH4* NAH"}=" H4B NAH"; $resn_exch{"AH3* NAH"}=" H3B NAH"; $resn_exch{"AHO3 NAH"}="HO3A NAH";
$resn_exch{"AH2* NAH"}=" H2B NAH"; $resn_exch{"AHO2 NAH"}="HO2A NAH"; $resn_exch{"AH1* NAH"}=" H1B NAH";
$resn_exch{" AH8 NAH"}=" H8A NAH"; $resn_exch{"AH61 NAH"}="H61A NAH"; $resn_exch{"AH62 NAH"}="H62A NAH";
$resn_exch{" AH2 NAH"}=" H2A NAH"; $resn_exch{"NH51 NAH"}="H51N NAH"; $resn_exch{"NH52 NAH"}="H52N NAH";
$resn_exch{"NH4* NAH"}=" H4D NAH"; $resn_exch{"NH3* NAH"}=" H3D NAH"; $resn_exch{"NHO3 NAH"}="HO3N NAH";
$resn_exch{"NH2* NAH"}=" H2D NAH"; $resn_exch{"NHO2 NAH"}="HO2N NAH"; $resn_exch{"NH1* NAH"}=" H1D NAH";
$resn_exch{" NH2 NAH"}=" H2N NAH"; $resn_exch{"NH71 NAH"}="H71N NAH"; $resn_exch{"NH72 NAH"}="H72N NAH";
$resn_exch{" NH4 NAH"}=" H4N NAH"; $resn_exch{" NH5 NAH"}=" H5N NAH"; $resn_exch{" NH6 NAH"}=" H6N NAH";
$resn_exch{"2HOA NAI"}="HOA2 NAI"; $resn_exch{"AH51 NAI"}="H51A NAI"; $resn_exch{"AH52 NAI"}="H52A NAI";
$resn_exch{"AH4* NAI"}=" H4B NAI"; $resn_exch{"AH3* NAI"}=" H3B NAI"; $resn_exch{"AHO3 NAI"}="HO3A NAI";
$resn_exch{"AH2* NAI"}=" H2B NAI"; $resn_exch{"AHO2 NAI"}="HO2A NAI"; $resn_exch{"AH1* NAI"}=" H1B NAI";
$resn_exch{" AH8 NAI"}=" H8A NAI"; $resn_exch{"AH61 NAI"}="H61A NAI"; $resn_exch{"AH62 NAI"}="H62A NAI";
$resn_exch{" AH2 NAI"}=" H2A NAI"; $resn_exch{"1NHO NAI"}="HO1N NAI"; $resn_exch{"NH51 NAI"}="H51N NAI";
$resn_exch{"NH52 NAI"}="H52N NAI"; $resn_exch{"NH4* NAI"}=" H4D NAI"; $resn_exch{"NH3* NAI"}=" H3D NAI";
$resn_exch{"NHO3 NAI"}="HO3N NAI"; $resn_exch{"NH2* NAI"}=" H2D NAI"; $resn_exch{"NHO2 NAI"}="HO2N NAI";
$resn_exch{"NH1* NAI"}=" H1D NAI"; $resn_exch{" NH2 NAI"}=" H2N NAI"; $resn_exch{"NH71 NAI"}="H71N NAI";
$resn_exch{"NH72 NAI"}="H72N NAI"; $resn_exch{" NH4 NAI"}=" H4N NAI"; $resn_exch{"2NH4 NAI"}="H42N NAI";
$resn_exch{" NH5 NAI"}=" H5N NAI"; $resn_exch{" NH6 NAI"}=" H6N NAI"; $resn_exch{"AH51 NAJ"}="H51A NAJ";
$resn_exch{"AH52 NAJ"}="H52A NAJ"; $resn_exch{"AH4* NAJ"}=" H4B NAJ"; $resn_exch{"AH3* NAJ"}=" H3B NAJ";
$resn_exch{"AHO3 NAJ"}="HO3A NAJ"; $resn_exch{"AH2* NAJ"}=" H2B NAJ"; $resn_exch{"AHO2 NAJ"}="HO2A NAJ";
$resn_exch{"AH1* NAJ"}=" H1B NAJ"; $resn_exch{" AH8 NAJ"}=" H8A NAJ"; $resn_exch{"AH61 NAJ"}="H61A NAJ";
$resn_exch{"AH62 NAJ"}="H62A NAJ"; $resn_exch{" AH1 NAJ"}=" H1A NAJ"; $resn_exch{" AH2 NAJ"}=" H2A NAJ";
$resn_exch{"NH51 NAJ"}="H51N NAJ"; $resn_exch{"NH52 NAJ"}="H52N NAJ"; $resn_exch{"NH4* NAJ"}=" H4D NAJ";
$resn_exch{"NH3* NAJ"}=" H3D NAJ"; $resn_exch{"NHO3 NAJ"}="HO3N NAJ"; $resn_exch{"NH2* NAJ"}=" H2D NAJ";
$resn_exch{"NHO2 NAJ"}="HO2N NAJ"; $resn_exch{"NH1* NAJ"}=" H1D NAJ"; $resn_exch{" NH2 NAJ"}=" H2N NAJ";
$resn_exch{"NH71 NAJ"}="H71N NAJ"; $resn_exch{"NH72 NAJ"}="H72N NAJ"; $resn_exch{" NH4 NAJ"}=" H4N NAJ";
$resn_exch{" NH5 NAJ"}=" H5N NAJ"; $resn_exch{" NH6 NAJ"}=" H6N NAJ"; $resn_exch{"1HAE NAK"}="HAE1 NAK";
$resn_exch{"2HAE NAK"}="HAE2 NAK"; $resn_exch{"3HAE NAK"}="HAE3 NAK"; $resn_exch{"1HAB NAK"}="HAB1 NAK";
$resn_exch{"2HAB NAK"}="HAB2 NAK"; $resn_exch{"1H9  NAL"}=" H91 NAL"; $resn_exch{"2H9  NAL"}=" H92 NAL";
$resn_exch{"1HN  NAL"}=" HN1 NAL"; $resn_exch{"2HN  NAL"}=" HN2 NAL"; $resn_exch{"1HN  NAM"}=" H   NAM";
$resn_exch{"2HN  NAM"}=" H2  NAM"; $resn_exch{"1HB  NAM"}=" HB2 NAM"; $resn_exch{"2HB  NAM"}=" HB3 NAM";
$resn_exch{"1HNX NAM"}="HNX1 NAM"; $resn_exch{"2HNX NAM"}="HNX2 NAM"; $resn_exch{"1H3  NAN"}=" H31 NAN";
$resn_exch{"2H3  NAN"}=" H32 NAN"; $resn_exch{"1H9  NAN"}=" H91 NAN"; $resn_exch{"2H9  NAN"}=" H92 NAN";
$resn_exch{"1H11 NAN"}="H111 NAN"; $resn_exch{"2H11 NAN"}="H112 NAN"; $resn_exch{"3H11 NAN"}="H113 NAN";
$resn_exch{"1HOB NAN"}="HOB1 NAN"; $resn_exch{"1HO1 NAO"}="HO11 NAO"; $resn_exch{"2HO1 NAO"}="HO12 NAO";
$resn_exch{"2HOA NAP"}="HOA2 NAP"; $resn_exch{"AH51 NAP"}="H51A NAP"; $resn_exch{"AH52 NAP"}="H52A NAP";
$resn_exch{"AH4* NAP"}=" H4B NAP"; $resn_exch{"AH3* NAP"}=" H3B NAP"; $resn_exch{"AHO3 NAP"}="HO3A NAP";
$resn_exch{"AH2* NAP"}=" H2B NAP"; $resn_exch{"AH1* NAP"}=" H1B NAP"; $resn_exch{" AH8 NAP"}=" H8A NAP";
$resn_exch{"AH61 NAP"}="H61A NAP"; $resn_exch{"AH62 NAP"}="H62A NAP"; $resn_exch{" AH2 NAP"}=" H2A NAP";
$resn_exch{"NH51 NAP"}="H51N NAP"; $resn_exch{"NH52 NAP"}="H52N NAP"; $resn_exch{"NH4* NAP"}=" H4D NAP";
$resn_exch{"NH3* NAP"}=" H3D NAP"; $resn_exch{"NHO3 NAP"}="HO3N NAP"; $resn_exch{"NH2* NAP"}=" H2D NAP";
$resn_exch{"NHO2 NAP"}="HO2N NAP"; $resn_exch{"NH1* NAP"}=" H1D NAP"; $resn_exch{" NH2 NAP"}=" H2N NAP";
$resn_exch{"NH71 NAP"}="H71N NAP"; $resn_exch{"NH72 NAP"}="H72N NAP"; $resn_exch{" NH4 NAP"}=" H4N NAP";
$resn_exch{" NH5 NAP"}=" H5N NAP"; $resn_exch{" NH6 NAP"}=" H6N NAP"; $resn_exch{"2HOP NAP"}="HOP2 NAP";
$resn_exch{"3HOP NAP"}="HOP3 NAP"; $resn_exch{"2HOA NAQ"}="HOA2 NAQ"; $resn_exch{"1H5' NAQ"}="H5'1 NAQ";
$resn_exch{"2H5' NAQ"}="H5'2 NAQ"; $resn_exch{"3HO' NAQ"}="HO'3 NAQ"; $resn_exch{"2HO' NAQ"}="HO'2 NAQ";
$resn_exch{"1H6A NAQ"}="H6A1 NAQ"; $resn_exch{"2H6A NAQ"}="H6A2 NAQ"; $resn_exch{"1H5* NAQ"}="H5A1 NAQ";
$resn_exch{"2H5* NAQ"}="H5A2 NAQ"; $resn_exch{" H4* NAQ"}=" H4B NAQ"; $resn_exch{" H3* NAQ"}=" H3B NAQ";
$resn_exch{"3HO* NAQ"}="HOA3 NAQ"; $resn_exch{" H2* NAQ"}=" H2B NAQ"; $resn_exch{"2HO* NAQ"}="HOB2 NAQ";
$resn_exch{" H1* NAQ"}=" H1B NAQ"; $resn_exch{"1H7N NAQ"}="H7N1 NAQ"; $resn_exch{"2H7N NAQ"}="H7N2 NAQ";
$resn_exch{"1H5P NAQ"}="H5P1 NAQ"; $resn_exch{"2H5P NAQ"}="H5P2 NAQ"; $resn_exch{"3H5P NAQ"}="H5P3 NAQ";
$resn_exch{"1H2P NAQ"}="H2P1 NAQ"; $resn_exch{"2H2P NAQ"}="H2P2 NAQ"; $resn_exch{"1H1P NAQ"}="H1P1 NAQ";
$resn_exch{"2H1P NAQ"}="H1P2 NAQ"; $resn_exch{"3H1P NAQ"}="H1P3 NAQ"; $resn_exch{"1H1  NAR"}=" H11 NAR";
$resn_exch{"1H3  NAR"}=" H31 NAR"; $resn_exch{"1H8  NAR"}=" H81 NAR"; $resn_exch{"2H8  NAR"}=" H82 NAR";
$resn_exch{"1H11 NAR"}="H111 NAR"; $resn_exch{"1H12 NAR"}="H121 NAR"; $resn_exch{"1H14 NAR"}="H141 NAR";
$resn_exch{"1H15 NAR"}="H151 NAR"; $resn_exch{"1H4  NAR"}=" H41 NAR"; $resn_exch{"1H5  NAR"}=" H51 NAR";
$resn_exch{"3HOS NAS"}="HOS3 NAS"; $resn_exch{"1H16 NAT"}="H161 NAT"; $resn_exch{"2H16 NAT"}="H162 NAT";
$resn_exch{"1H17 NAT"}="H171 NAT"; $resn_exch{"2H17 NAT"}="H172 NAT"; $resn_exch{"3H17 NAT"}="H173 NAT";
$resn_exch{"1H18 NAT"}="H181 NAT"; $resn_exch{"2H18 NAT"}="H182 NAT"; $resn_exch{"3H18 NAT"}="H183 NAT";
$resn_exch{"1HC2 NAU"}="HC21 NAU"; $resn_exch{"2HC2 NAU"}="HC22 NAU"; $resn_exch{"1HC3 NAU"}="HC31 NAU";
$resn_exch{"2HC3 NAU"}="HC32 NAU"; $resn_exch{"1H11 NAU"}="H111 NAU"; $resn_exch{"2H11 NAU"}="H112 NAU";
$resn_exch{"3H11 NAU"}="H113 NAU"; $resn_exch{"1HC9 NAU"}="HC91 NAU"; $resn_exch{"2HC9 NAU"}="HC92 NAU";
$resn_exch{"1HC2 NAV"}="HC21 NAV"; $resn_exch{"2HC2 NAV"}="HC22 NAV"; $resn_exch{"1HC3 NAV"}="HC31 NAV";
$resn_exch{"2HC3 NAV"}="HC32 NAV"; $resn_exch{"1H11 NAV"}="H111 NAV"; $resn_exch{"2H11 NAV"}="H112 NAV";
$resn_exch{"3H11 NAV"}="H113 NAV"; $resn_exch{"1HC9 NAV"}="HC91 NAV"; $resn_exch{"2HC9 NAV"}="HC92 NAV";
$resn_exch{"1HO1 NAW"}="HO11 NAW"; $resn_exch{"2HO1 NAW"}="HO12 NAW"; $resn_exch{"1HO2 NAW"}="HO21 NAW";
$resn_exch{"2HO2 NAW"}="HO22 NAW"; $resn_exch{"1HO3 NAW"}="HO31 NAW"; $resn_exch{"2HO3 NAW"}="HO32 NAW";
$resn_exch{"2HOA NAX"}="HOA2 NAX"; $resn_exch{"AH51 NAX"}="H51A NAX"; $resn_exch{"AH52 NAX"}="H52A NAX";
$resn_exch{"AH4* NAX"}=" H4B NAX"; $resn_exch{"AH3* NAX"}=" H3B NAX"; $resn_exch{"AHO3 NAX"}="HO3A NAX";
$resn_exch{"AH2* NAX"}=" H2B NAX"; $resn_exch{"AHO2 NAX"}="HO2A NAX"; $resn_exch{"AH1* NAX"}=" H1B NAX";
$resn_exch{" AH8 NAX"}=" H8A NAX"; $resn_exch{"AH61 NAX"}="H61A NAX"; $resn_exch{"AH62 NAX"}="H62A NAX";
$resn_exch{" AH2 NAX"}=" H2A NAX"; $resn_exch{"2HNO NAX"}="HNO2 NAX"; $resn_exch{"1H5* NAX"}="H5'1 NAX";
$resn_exch{"2H5* NAX"}="H5'2 NAX"; $resn_exch{"NH4* NAX"}=" H4D NAX"; $resn_exch{"NH3* NAX"}=" H3D NAX";
$resn_exch{"NHO3 NAX"}="HO3N NAX"; $resn_exch{"NH2* NAX"}=" H2D NAX"; $resn_exch{"NHO2 NAX"}="HO2N NAX";
$resn_exch{"NH1* NAX"}=" H1D NAX"; $resn_exch{" NH2 NAX"}=" H2N NAX"; $resn_exch{"NH71 NAX"}="H71N NAX";
$resn_exch{"NH72 NAX"}="H72N NAX"; $resn_exch{"NH41 NAX"}="H41N NAX"; $resn_exch{"NH42 NAX"}="H42N NAX";
$resn_exch{"NH51 NAX"}="H51N NAX"; $resn_exch{"NH52 NAX"}="H52N NAX"; $resn_exch{" NH6 NAX"}=" H6N NAX";
$resn_exch{"NHO6 NAX"}="HO6N NAX"; $resn_exch{"1HC3 NAY"}="HC31 NAY"; $resn_exch{"2HC3 NAY"}="HC32 NAY";
$resn_exch{"1HC4 NAY"}="HC41 NAY"; $resn_exch{"2HC4 NAY"}="HC42 NAY"; $resn_exch{"1H11 NAY"}="H111 NAY";
$resn_exch{"2H11 NAY"}="H112 NAY"; $resn_exch{"3H11 NAY"}="H113 NAY"; $resn_exch{"1HC9 NAY"}="HC91 NAY";
$resn_exch{"2HC9 NAY"}="HC92 NAY"; $resn_exch{"1H6C NB1"}="H6C1 NB1"; $resn_exch{"2H6C NB1"}="H6C2 NB1";
$resn_exch{"1H8C NB1"}="H8C1 NB1"; $resn_exch{"2H8C NB1"}="H8C2 NB1"; $resn_exch{"1H9C NB1"}="H9C1 NB1";
$resn_exch{"2H9C NB1"}="H9C2 NB1"; $resn_exch{"1HMC NB1"}="HMC1 NB1"; $resn_exch{"2HMC NB1"}="HMC2 NB1";
$resn_exch{"3HMC NB1"}="HMC3 NB1"; $resn_exch{"4HO  NBA"}=" HO4 NBA"; $resn_exch{"4HC  NBA"}=" HC4 NBA";
$resn_exch{"1HC5 NBA"}="HC51 NBA"; $resn_exch{"2HC5 NBA"}="HC52 NBA"; $resn_exch{"1HC6 NBA"}="HC61 NBA";
$resn_exch{"2HC6 NBA"}="HC62 NBA"; $resn_exch{"7HN  NBA"}=" HN7 NBA"; $resn_exch{"1HC8 NBA"}="HC81 NBA";
$resn_exch{"2HC8 NBA"}="HC82 NBA"; $resn_exch{"1HC9 NBA"}="HC91 NBA"; $resn_exch{"2HC9 NBA"}="HC92 NBA";
$resn_exch{"1H10 NBA"}="H101 NBA"; $resn_exch{"2H10 NBA"}="H102 NBA"; $resn_exch{"1HN1 NBA"}="HN11 NBA";
$resn_exch{"1H12 NBA"}="H121 NBA"; $resn_exch{"2H12 NBA"}="H122 NBA"; $resn_exch{"1H13 NBA"}="H131 NBA";
$resn_exch{"2H13 NBA"}="H132 NBA"; $resn_exch{"1H14 NBA"}="H141 NBA"; $resn_exch{"2H14 NBA"}="H142 NBA";
$resn_exch{"1HN5 NBA"}="HN51 NBA"; $resn_exch{"1H16 NBA"}="H161 NBA"; $resn_exch{"2H16 NBA"}="H162 NBA";
$resn_exch{"3H16 NBA"}="H163 NBA"; $resn_exch{"4HOL NBA"}="HOL4 NBA"; $resn_exch{"1H11 NBB"}="H111 NBB";
$resn_exch{"2H11 NBB"}="H112 NBB"; $resn_exch{"1H12 NBB"}="H121 NBB"; $resn_exch{"2H12 NBB"}="H122 NBB";
$resn_exch{"1H13 NBB"}="H131 NBB"; $resn_exch{"2H13 NBB"}="H132 NBB"; $resn_exch{"1H14 NBB"}="H141 NBB";
$resn_exch{"2H14 NBB"}="H142 NBB"; $resn_exch{"3H14 NBB"}="H143 NBB"; $resn_exch{"1H9  NBC"}=" H91 NBC";
$resn_exch{"1H7  NBC"}=" H71 NBC"; $resn_exch{"1H8  NBC"}=" H81 NBC"; $resn_exch{"1H5  NBC"}=" H51 NBC";
$resn_exch{"1H6  NBC"}=" H61 NBC"; $resn_exch{"1H3  NBC"}="H31A NBC"; $resn_exch{"1H4  NBC"}="H41A NBC";
$resn_exch{"1H1  NBC"}="H11A NBC"; $resn_exch{"1H2  NBC"}="H21A NBC"; $resn_exch{"2HOA NBD"}="HOA2 NBD";
$resn_exch{"AH51 NBD"}="H51A NBD"; $resn_exch{"AH52 NBD"}="H52A NBD"; $resn_exch{"AH4* NBD"}=" H4B NBD";
$resn_exch{"AH3* NBD"}=" H3B NBD"; $resn_exch{"AHO3 NBD"}="HO3A NBD"; $resn_exch{"AH2* NBD"}=" H2B NBD";
$resn_exch{"AHO2 NBD"}="HO2A NBD"; $resn_exch{"AH1* NBD"}=" H1B NBD"; $resn_exch{" AH8 NBD"}=" H8A NBD";
$resn_exch{" AH6 NBD"}=" H6A NBD"; $resn_exch{" AH2 NBD"}=" H2A NBD"; $resn_exch{"NH51 NBD"}="H51N NBD";
$resn_exch{"NH52 NBD"}="H52N NBD"; $resn_exch{"NH4* NBD"}=" H4D NBD"; $resn_exch{"NH3* NBD"}=" H3D NBD";
$resn_exch{"NHO3 NBD"}="HO3N NBD"; $resn_exch{"NH2* NBD"}=" H2D NBD"; $resn_exch{"NHO2 NBD"}="HO2N NBD";
$resn_exch{"NH1* NBD"}=" H1D NBD"; $resn_exch{" NH2 NBD"}=" H2N NBD"; $resn_exch{"NH71 NBD"}="H71N NBD";
$resn_exch{"NH72 NBD"}="H72N NBD"; $resn_exch{" NH4 NBD"}=" H4N NBD"; $resn_exch{" NH5 NBD"}=" H5N NBD";
$resn_exch{" NH6 NBD"}=" H6N NBD"; $resn_exch{"1HB1 NBD"}="HB11 NBD"; $resn_exch{"2HB1 NBD"}="HB12 NBD";
$resn_exch{"1H25 NBF"}="H251 NBF"; $resn_exch{"2H25 NBF"}="H252 NBF"; $resn_exch{"3H25 NBF"}="H253 NBF";
$resn_exch{"1H24 NBF"}="H241 NBF"; $resn_exch{"2H24 NBF"}="H242 NBF"; $resn_exch{"1H7  NBF"}=" H71 NBF";
$resn_exch{"2H7  NBF"}=" H72 NBF"; $resn_exch{"1H6  NBG"}=" H61 NBG"; $resn_exch{"2H6  NBG"}=" H62 NBG";
$resn_exch{"1H8  NBG"}=" H81 NBG"; $resn_exch{"2H8  NBG"}=" H82 NBG"; $resn_exch{"3H8  NBG"}=" H83 NBG";
$resn_exch{"1H1  NBL"}=" H11 NBL"; $resn_exch{"2H1  NBL"}="H12A NBL"; $resn_exch{"3H1  NBL"}="H13A NBL";
$resn_exch{"1H3  NBL"}=" H31 NBL"; $resn_exch{"2H3  NBL"}=" H32 NBL"; $resn_exch{"3H3  NBL"}=" H33 NBL";
$resn_exch{"1H4  NBL"}=" H41 NBL"; $resn_exch{"2H4  NBL"}=" H42 NBL"; $resn_exch{"1H10 NBL"}="H101 NBL";
$resn_exch{"2H10 NBL"}="H102 NBL"; $resn_exch{"1H21 NBL"}="H211 NBL"; $resn_exch{"2H21 NBL"}="H212 NBL";
$resn_exch{"1H22 NBL"}="H221 NBL"; $resn_exch{"2H22 NBL"}="H222 NBL"; $resn_exch{"1H24 NBL"}="H241 NBL";
$resn_exch{"2H24 NBL"}="H242 NBL"; $resn_exch{"1H1  NBN"}=" H11 NBN"; $resn_exch{"2H1  NBN"}=" H12 NBN";
$resn_exch{"1H2  NBN"}=" H21 NBN"; $resn_exch{"2H2  NBN"}=" H22 NBN"; $resn_exch{"1H3  NBN"}=" H31 NBN";
$resn_exch{"2H3  NBN"}=" H32 NBN"; $resn_exch{"1H4  NBN"}=" H41 NBN"; $resn_exch{"2H4  NBN"}=" H42 NBN";
$resn_exch{"3H4  NBN"}=" H43 NBN"; $resn_exch{"2HOA NBP"}="HOA2 NBP"; $resn_exch{"AH51 NBP"}="H51A NBP";
$resn_exch{"AH52 NBP"}="H52A NBP"; $resn_exch{"AH4* NBP"}=" H4B NBP"; $resn_exch{"AH3* NBP"}=" H3B NBP";
$resn_exch{"AHO3 NBP"}="HO3A NBP"; $resn_exch{"AH2* NBP"}=" H2B NBP"; $resn_exch{"AH1* NBP"}=" H1B NBP";
$resn_exch{"AH61 NBP"}="H61A NBP"; $resn_exch{"AH62 NBP"}="H62A NBP"; $resn_exch{" AH2 NBP"}=" H2A NBP";
$resn_exch{"NH51 NBP"}="H51N NBP"; $resn_exch{"NH52 NBP"}="H52N NBP"; $resn_exch{"NH4* NBP"}=" H4D NBP";
$resn_exch{"NH3* NBP"}=" H3D NBP"; $resn_exch{"NHO3 NBP"}="HO3N NBP"; $resn_exch{"NH2* NBP"}=" H2D NBP";
$resn_exch{"NHO2 NBP"}="HO2N NBP"; $resn_exch{"NH1* NBP"}=" H1D NBP"; $resn_exch{" NH2 NBP"}=" H2N NBP";
$resn_exch{"NH71 NBP"}="H71N NBP"; $resn_exch{"NH72 NBP"}="H72N NBP"; $resn_exch{" NH4 NBP"}=" H4N NBP";
$resn_exch{" NH5 NBP"}=" H5N NBP"; $resn_exch{" NH6 NBP"}=" H6N NBP"; $resn_exch{"2HOP NBP"}="HOP2 NBP";
$resn_exch{"3HOP NBP"}="HOP3 NBP"; $resn_exch{" HN  NBQ"}=" H2  NBQ"; $resn_exch{" HB1 NBQ"}=" HB2 NBQ";
$resn_exch{" HB2 NBQ"}=" HB3 NBQ"; $resn_exch{"1HAE NBQ"}="HAE1 NBQ"; $resn_exch{"2HAE NBQ"}="HAE2 NBQ";
$resn_exch{"4HOP NBS"}="HOP4 NBS"; $resn_exch{"5HOP NBS"}="HOP5 NBS"; $resn_exch{"1HOP NBS"}="HOP1 NBS";
$resn_exch{"1H5* NBS"}="H5'1 NBS"; $resn_exch{"2H5* NBS"}="H5'2 NBS"; $resn_exch{" H4* NBS"}=" H4' NBS";
$resn_exch{" H1* NBS"}=" H1' NBS"; $resn_exch{"1HB1 NBS"}="HB11 NBS"; $resn_exch{"2HB1 NBS"}="HB12 NBS";
$resn_exch{" H2* NBS"}=" H2' NBS"; $resn_exch{"2HO* NBS"}="HO'2 NBS"; $resn_exch{" H3* NBS"}=" H3' NBS";
$resn_exch{"3HO* NBS"}="HO'3 NBS"; $resn_exch{"1H1C NBT"}="H1C1 NBT"; $resn_exch{"2H1C NBT"}="H1C2 NBT";
$resn_exch{"3H1C NBT"}="H1C3 NBT"; $resn_exch{"1H3C NBT"}="H3C1 NBT"; $resn_exch{"2H3C NBT"}="H3C2 NBT";
$resn_exch{"1H4C NBT"}="H4C1 NBT"; $resn_exch{"2H4C NBT"}="H4C2 NBT"; $resn_exch{"3H4C NBT"}="H4C3 NBT";
$resn_exch{"1H12 NBV"}="H121 NBV"; $resn_exch{"2H12 NBV"}="H122 NBV"; $resn_exch{"1H7C NBV"}="H7C1 NBV";
$resn_exch{"2H7C NBV"}="H7C2 NBV"; $resn_exch{"1H13 NBV"}="H131 NBV"; $resn_exch{"2H13 NBV"}="H132 NBV";
$resn_exch{"1H14 NBV"}="H141 NBV"; $resn_exch{"2H14 NBV"}="H142 NBV"; $resn_exch{"1H15 NBV"}="H151 NBV";
$resn_exch{"2H15 NBV"}="H152 NBV"; $resn_exch{"1H16 NBV"}="H161 NBV"; $resn_exch{"2H16 NBV"}="H162 NBV";
$resn_exch{"3H16 NBV"}="H163 NBV"; $resn_exch{"1H1   NC"}=" H11  NC"; $resn_exch{"2H1   NC"}=" H12  NC";
$resn_exch{"3H1   NC"}=" H13  NC"; $resn_exch{"H2   NC1"}="HC28 NC1"; $resn_exch{" H20 NC1"}=" H   NC1";
$resn_exch{" H21 NC1"}=" H2  NC1"; $resn_exch{" H17 NC1"}=" HB2 NC1"; $resn_exch{" H18 NC1"}=" HB3 NC1";
$resn_exch{"1H6C NC2"}="H6C1 NC2"; $resn_exch{"2H6C NC2"}="H6C2 NC2"; $resn_exch{"1H13 NC2"}="H131 NC2";
$resn_exch{"2H13 NC2"}="H132 NC2"; $resn_exch{"1H22 NC2"}="H221 NC2"; $resn_exch{"1H23 NC2"}="H231 NC2";
$resn_exch{"1HN  NC2"}=" HN1 NC2"; $resn_exch{"2HN  NC2"}=" HN2 NC2"; $resn_exch{"1HBC NC2"}="HBC1 NC2";
$resn_exch{"2HBC NC2"}="HBC2 NC2"; $resn_exch{"1H5  NC3"}=" H51 NC3"; $resn_exch{"2H5  NC3"}=" H52 NC3";
$resn_exch{"1H10 NC3"}="H101 NC3"; $resn_exch{"2H10 NC3"}="H102 NC3"; $resn_exch{"1H11 NC3"}="H111 NC3";
$resn_exch{"2H11 NC3"}="H112 NC3"; $resn_exch{"1H12 NC3"}="H121 NC3"; $resn_exch{"2H12 NC3"}="H122 NC3";
$resn_exch{"1H13 NC3"}="H131 NC3"; $resn_exch{"2H13 NC3"}="H132 NC3"; $resn_exch{"1H14 NC3"}="H141 NC3";
$resn_exch{"2H14 NC3"}="H142 NC3"; $resn_exch{"1H5  NC4"}=" H51 NC4"; $resn_exch{"2H5  NC4"}=" H52 NC4";
$resn_exch{"1H6  NC4"}=" H61 NC4"; $resn_exch{"2H6  NC4"}=" H62 NC4"; $resn_exch{"1H7  NC4"}=" H71 NC4";
$resn_exch{"2H7  NC4"}=" H72 NC4"; $resn_exch{"1H12 NC4"}="H121 NC4"; $resn_exch{"2H12 NC4"}="H122 NC4";
$resn_exch{"1H13 NC4"}="H131 NC4"; $resn_exch{"2H13 NC4"}="H132 NC4"; $resn_exch{"1H14 NC4"}="H141 NC4";
$resn_exch{"2H14 NC4"}="H142 NC4"; $resn_exch{"1H15 NC4"}="H151 NC4"; $resn_exch{"2H15 NC4"}="H152 NC4";
$resn_exch{"1H16 NC4"}="H161 NC4"; $resn_exch{"2H16 NC4"}="H162 NC4"; $resn_exch{"1H5  NC6"}=" H51 NC6";
$resn_exch{"2H5  NC6"}=" H52 NC6"; $resn_exch{"1H6  NC6"}=" H61 NC6"; $resn_exch{"2H6  NC6"}=" H62 NC6";
$resn_exch{"1H7  NC6"}=" H71 NC6"; $resn_exch{"2H7  NC6"}=" H72 NC6"; $resn_exch{"1H8  NC6"}=" H81 NC6";
$resn_exch{"2H8  NC6"}=" H82 NC6"; $resn_exch{"1H9  NC6"}=" H91 NC6"; $resn_exch{"2H9  NC6"}=" H92 NC6";
$resn_exch{"1H14 NC6"}="H141 NC6"; $resn_exch{"2H14 NC6"}="H142 NC6"; $resn_exch{"1H15 NC6"}="H151 NC6";
$resn_exch{"2H15 NC6"}="H152 NC6"; $resn_exch{"1H16 NC6"}="H161 NC6"; $resn_exch{"2H16 NC6"}="H162 NC6";
$resn_exch{"1H17 NC6"}="H171 NC6"; $resn_exch{"2H17 NC6"}="H172 NC6"; $resn_exch{"1H18 NC6"}="H181 NC6";
$resn_exch{"2H18 NC6"}="H182 NC6"; $resn_exch{"1H5  NC7"}=" H51 NC7"; $resn_exch{"2H5  NC7"}=" H52 NC7";
$resn_exch{"1H6  NC7"}=" H61 NC7"; $resn_exch{"2H6  NC7"}=" H62 NC7"; $resn_exch{"1H7  NC7"}=" H71 NC7";
$resn_exch{"2H7  NC7"}=" H72 NC7"; $resn_exch{"1H8  NC7"}=" H81 NC7"; $resn_exch{"2H8  NC7"}=" H82 NC7";
$resn_exch{"1H9  NC7"}=" H91 NC7"; $resn_exch{"2H9  NC7"}=" H92 NC7"; $resn_exch{"1H11 NC7"}="H111 NC7";
$resn_exch{"2H11 NC7"}="H112 NC7"; $resn_exch{"1H15 NC7"}="H151 NC7"; $resn_exch{"2H15 NC7"}="H152 NC7";
$resn_exch{"1H16 NC7"}="H161 NC7"; $resn_exch{"2H16 NC7"}="H162 NC7"; $resn_exch{"1H17 NC7"}="H171 NC7";
$resn_exch{"2H17 NC7"}="H172 NC7"; $resn_exch{"1H18 NC7"}="H181 NC7"; $resn_exch{"2H18 NC7"}="H182 NC7";
$resn_exch{"1H19 NC7"}="H191 NC7"; $resn_exch{"2H19 NC7"}="H192 NC7"; $resn_exch{"1HN7 NCA"}="HN71 NCA";
$resn_exch{"2HN7 NCA"}="HN72 NCA"; $resn_exch{" HO  NCB"}=" HXT NCB"; $resn_exch{"3HB  NCB"}=" HB3 NCB";
$resn_exch{"2HB  NCB"}=" HB2 NCB"; $resn_exch{"1HB  NCB"}=" HB1 NCB"; $resn_exch{"2H2  NCB"}=" H22 NCB";
$resn_exch{"1H2  NCB"}=" H21 NCB"; $resn_exch{"1H5' NCC"}="H5'1 NCC"; $resn_exch{"2H5' NCC"}="H5'2 NCC";
$resn_exch{"1HN4 NCC"}="HN41 NCC"; $resn_exch{"2HN4 NCC"}="HN42 NCC"; $resn_exch{"1H15 NCC"}="H151 NCC";
$resn_exch{"2H15 NCC"}="H152 NCC"; $resn_exch{"1H18 NCC"}="H181 NCC"; $resn_exch{"2H18 NCC"}="H182 NCC";
$resn_exch{"3H18 NCC"}="H183 NCC"; $resn_exch{"1H20 NCC"}="H201 NCC"; $resn_exch{"2H20 NCC"}="H202 NCC";
$resn_exch{"1H6  NCD"}=" H61 NCD"; $resn_exch{"1H61 NCD"}="H611 NCD"; $resn_exch{"1H1  NCD"}=" H11 NCD";
$resn_exch{"1H3  NCD"}=" H31 NCD"; $resn_exch{"2H3  NCD"}=" H32 NCD"; $resn_exch{"1H5  NCD"}=" H51 NCD";
$resn_exch{"2H5  NCD"}=" H52 NCD"; $resn_exch{"1HO5 NCD"}="HO51 NCD"; $resn_exch{"1H2C NCF"}="H2C1 NCF";
$resn_exch{"2H2C NCF"}="H2C2 NCF"; $resn_exch{"1H3C NCF"}="H3C1 NCF"; $resn_exch{"2H3C NCF"}="H3C2 NCF";
$resn_exch{"1H13 NCF"}="H131 NCF"; $resn_exch{"2H13 NCF"}="H132 NCF"; $resn_exch{"H8'' NCG"}=" H8B NCG";
$resn_exch{"H6'' NCG"}=" H6B NCG"; $resn_exch{"H3'' NCG"}=" H3B NCG"; $resn_exch{"H4'' NCG"}=" H4B NCG";
$resn_exch{"1H1  NCH"}=" H11 NCH"; $resn_exch{"2H1  NCH"}=" H12 NCH"; $resn_exch{"1H2  NCH"}=" H21 NCH";
$resn_exch{"2H2  NCH"}=" H22 NCH"; $resn_exch{"1H3  NCH"}=" H31 NCH"; $resn_exch{"2H3  NCH"}=" H32 NCH";
$resn_exch{"3H3  NCH"}=" H33 NCH"; $resn_exch{"1H5  NCH"}=" H51 NCH"; $resn_exch{"2H5  NCH"}=" H52 NCH";
$resn_exch{"3H5  NCH"}=" H53 NCH"; $resn_exch{"1H4  NCH"}=" H41 NCH"; $resn_exch{"2H4  NCH"}=" H42 NCH";
$resn_exch{"3H4  NCH"}=" H43 NCH"; $resn_exch{"1H3  NCM"}=" H31 NCM"; $resn_exch{"2H3  NCM"}=" H32 NCM";
$resn_exch{"1H5  NCM"}=" H51 NCM"; $resn_exch{"2H5  NCM"}=" H52 NCM"; $resn_exch{"1H6  NCM"}=" H61 NCM";
$resn_exch{"2H6  NCM"}=" H62 NCM"; $resn_exch{"1H7  NCM"}=" H71 NCM"; $resn_exch{"2H7  NCM"}=" H72 NCM";
$resn_exch{"1HOP NCN"}="HOP1 NCN"; $resn_exch{"1H5* NCN"}="H5'1 NCN"; $resn_exch{"2H5* NCN"}="H5'2 NCN";
$resn_exch{" H4* NCN"}=" H4' NCN"; $resn_exch{" H3* NCN"}=" H3' NCN"; $resn_exch{"3HO* NCN"}="HO'3 NCN";
$resn_exch{" H2* NCN"}=" H2' NCN"; $resn_exch{"2HO* NCN"}="HO'2 NCN"; $resn_exch{" H1* NCN"}=" H1' NCN";
$resn_exch{"1HN1 NCO"}="HN11 NCO"; $resn_exch{"2HN1 NCO"}="HN12 NCO"; $resn_exch{"3HN1 NCO"}="HN13 NCO";
$resn_exch{"1HN2 NCO"}="HN21 NCO"; $resn_exch{"2HN2 NCO"}="HN22 NCO"; $resn_exch{"3HN2 NCO"}="HN23 NCO";
$resn_exch{"1HN3 NCO"}="HN31 NCO"; $resn_exch{"2HN3 NCO"}="HN32 NCO"; $resn_exch{"3HN3 NCO"}="HN33 NCO";
$resn_exch{"1HN4 NCO"}="HN41 NCO"; $resn_exch{"2HN4 NCO"}="HN42 NCO"; $resn_exch{"3HN4 NCO"}="HN43 NCO";
$resn_exch{"1HN5 NCO"}="HN51 NCO"; $resn_exch{"2HN5 NCO"}="HN52 NCO"; $resn_exch{"3HN5 NCO"}="HN53 NCO";
$resn_exch{"1HN6 NCO"}="HN61 NCO"; $resn_exch{"2HN6 NCO"}="HN62 NCO"; $resn_exch{"3HN6 NCO"}="HN63 NCO";
$resn_exch{"1HN1 NCP"}="HN11 NCP"; $resn_exch{"2HN1 NCP"}="HN12 NCP"; $resn_exch{"1HN2 NCP"}="HN21 NCP";
$resn_exch{"2HN2 NCP"}="HN22 NCP"; $resn_exch{"1H7  NCR"}=" H71 NCR"; $resn_exch{"2H7  NCR"}=" H72 NCR";
$resn_exch{"3H7  NCR"}=" H73 NCR"; $resn_exch{"1H17 NCS"}="H171 NCS"; $resn_exch{"2H17 NCS"}="H172 NCS";
$resn_exch{"1H30 NCS"}="H301 NCS"; $resn_exch{"2H30 NCS"}="H302 NCS"; $resn_exch{"3H30 NCS"}="H303 NCS";
$resn_exch{"1H31 NCS"}="H311 NCS"; $resn_exch{"2H31 NCS"}="H312 NCS"; $resn_exch{"3H31 NCS"}="H313 NCS";
$resn_exch{" H1* NCS"}=" H1' NCS"; $resn_exch{" H2* NCS"}=" H2' NCS"; $resn_exch{"1HN2 NCS"}="HN21 NCS";
$resn_exch{"2HN2 NCS"}="HN22 NCS"; $resn_exch{"1HM2 NCS"}="HM21 NCS"; $resn_exch{"2HM2 NCS"}="HM22 NCS";
$resn_exch{"3HM2 NCS"}="HM23 NCS"; $resn_exch{" H3* NCS"}=" H3' NCS"; $resn_exch{"*HO3 NCS"}="HO3' NCS";
$resn_exch{" H4* NCS"}=" H4' NCS"; $resn_exch{"*HO4 NCS"}="HO4' NCS"; $resn_exch{" H5* NCS"}=" H5' NCS";
$resn_exch{"1HM5 NCS"}="HM51 NCS"; $resn_exch{"2HM5 NCS"}="HM52 NCS"; $resn_exch{"3HM5 NCS"}="HM53 NCS";
$resn_exch{"1HC7 NCT"}="HC71 NCT"; $resn_exch{"2HC7 NCT"}="HC72 NCT"; $resn_exch{"1HC8 NCT"}="HC81 NCT";
$resn_exch{"2HC8 NCT"}="HC82 NCT"; $resn_exch{"1HC9 NCT"}="HC91 NCT"; $resn_exch{"2HC9 NCT"}="HC92 NCT";
$resn_exch{"1H10 NCT"}="H101 NCT"; $resn_exch{"2H10 NCT"}="H102 NCT"; $resn_exch{"3H10 NCT"}="H103 NCT";
$resn_exch{"3HOP NCX"}="HOP3 NCX"; $resn_exch{"2HOP NCX"}="HOP2 NCX"; $resn_exch{"1H5* NCX"}=" H5' NCX";
$resn_exch{"2H5* NCX"}="H5'' NCX"; $resn_exch{" H4* NCX"}=" H4' NCX"; $resn_exch{" H1* NCX"}=" H1' NCX";
$resn_exch{"1HH7 NCX"}="HH71 NCX"; $resn_exch{"2HH7 NCX"}="HH72 NCX"; $resn_exch{" H3* NCX"}=" H3' NCX";
$resn_exch{"1H2* NCX"}=" H2' NCX"; $resn_exch{"2H2* NCX"}="H2'' NCX"; $resn_exch{"H3T  NCX"}="HO3' NCX";
$resn_exch{" HN  NCY"}=" H   NCY"; $resn_exch{"1HB  NCY"}=" HB2 NCY"; $resn_exch{"2HB  NCY"}=" HB3 NCY";
$resn_exch{"1HCN NCY"}="HCN1 NCY"; $resn_exch{"2HCN NCY"}="HCN2 NCY"; $resn_exch{"3HCN NCY"}="HCN3 NCY";
$resn_exch{"1H   NCZ"}=" H1  NCZ"; $resn_exch{"7H   NCZ"}=" H7  NCZ"; $resn_exch{"8H   NCZ"}=" H8  NCZ";
$resn_exch{"2H1  NCZ"}=" H12 NCZ"; $resn_exch{"3H1  NCZ"}=" H13 NCZ"; $resn_exch{"4H1  NCZ"}=" H14 NCZ";
$resn_exch{"5H1  NCZ"}=" H15 NCZ"; $resn_exch{"6H1  NCZ"}=" H16 NCZ"; $resn_exch{"6H   NCZ"}=" H6  NCZ";
$resn_exch{"5H   NCZ"}=" H5  NCZ"; $resn_exch{"2H   NCZ"}=" H2  NCZ"; $resn_exch{"3H   NCZ"}=" H3  NCZ";
$resn_exch{"1H17 NCZ"}="H171 NCZ"; $resn_exch{"2H17 NCZ"}="H172 NCZ"; $resn_exch{"3H17 NCZ"}="H173 NCZ";
$resn_exch{"1H18 NCZ"}="H181 NCZ"; $resn_exch{"2H18 NCZ"}="H182 NCZ"; $resn_exch{"3H18 NCZ"}="H183 NCZ";
$resn_exch{"2HOA NDA"}="HOA2 NDA"; $resn_exch{"AH51 NDA"}="H51A NDA"; $resn_exch{"AH52 NDA"}="H52A NDA";
$resn_exch{"AH4* NDA"}=" H4B NDA"; $resn_exch{"AH3* NDA"}=" H3B NDA"; $resn_exch{"AHO3 NDA"}="HO3A NDA";
$resn_exch{"AH2* NDA"}=" H2B NDA"; $resn_exch{"AHO2 NDA"}="HO2A NDA"; $resn_exch{"AH1* NDA"}=" H1B NDA";
$resn_exch{" AH8 NDA"}=" H8A NDA"; $resn_exch{"AH61 NDA"}="H61A NDA"; $resn_exch{"AH62 NDA"}="H62A NDA";
$resn_exch{" AH2 NDA"}=" H2A NDA"; $resn_exch{"NH51 NDA"}="H51N NDA"; $resn_exch{"NH52 NDA"}="H52N NDA";
$resn_exch{"NH4* NDA"}=" H4D NDA"; $resn_exch{"NH3* NDA"}=" H3D NDA"; $resn_exch{"NHO3 NDA"}="HO3N NDA";
$resn_exch{"NH2* NDA"}=" H2D NDA"; $resn_exch{"NHO2 NDA"}="HO2N NDA"; $resn_exch{"NH1* NDA"}=" H1D NDA";
$resn_exch{" NH2 NDA"}=" H2N NDA"; $resn_exch{"1HC7 NDA"}="HC71 NDA"; $resn_exch{"2HC7 NDA"}="HC72 NDA";
$resn_exch{"NH71 NDA"}="H71N NDA"; $resn_exch{"NH72 NDA"}="H72N NDA"; $resn_exch{" NH4 NDA"}=" H4N NDA";
$resn_exch{" NH5 NDA"}=" H5N NDA"; $resn_exch{" NH6 NDA"}=" H6N NDA"; $resn_exch{"2HOA NDC"}="HOA2 NDC";
$resn_exch{"1H5' NDC"}="H5'1 NDC"; $resn_exch{"2H5' NDC"}="H5'2 NDC"; $resn_exch{"3HO' NDC"}="HO'3 NDC";
$resn_exch{"2HO' NDC"}="HO'2 NDC"; $resn_exch{"1H6A NDC"}="H6A1 NDC"; $resn_exch{"2H6A NDC"}="H6A2 NDC";
$resn_exch{"1H5* NDC"}="H5A1 NDC"; $resn_exch{"2H5* NDC"}="H5A2 NDC"; $resn_exch{" H4* NDC"}=" H4B NDC";
$resn_exch{" H3* NDC"}=" H3B NDC"; $resn_exch{"3HO* NDC"}="HOA3 NDC"; $resn_exch{" H2* NDC"}=" H2B NDC";
$resn_exch{"2HO* NDC"}="HOB2 NDC"; $resn_exch{" H1* NDC"}=" H1B NDC"; $resn_exch{"1H7N NDC"}="H7N1 NDC";
$resn_exch{"2H7N NDC"}="H7N2 NDC"; $resn_exch{"1HC2 NDC"}="HC21 NDC"; $resn_exch{"2HC2 NDC"}="HC22 NDC";
$resn_exch{"1HC3 NDC"}="HC31 NDC"; $resn_exch{"2HC3 NDC"}="HC32 NDC"; $resn_exch{"1HC4 NDC"}="HC41 NDC";
$resn_exch{"2HC4 NDC"}="HC42 NDC"; $resn_exch{"1HC5 NDC"}="HC51 NDC"; $resn_exch{"2HC5 NDC"}="HC52 NDC";
$resn_exch{"NH71 NDE"}="H71N NDE"; $resn_exch{"NH72 NDE"}="H72N NDE"; $resn_exch{" NH5 NDE"}=" H5N NDE";
$resn_exch{" NH6 NDE"}=" H6N NDE"; $resn_exch{" NH2 NDE"}=" H2N NDE"; $resn_exch{"NH1* NDE"}=" H1D NDE";
$resn_exch{"NH2* NDE"}=" H2D NDE"; $resn_exch{"NHO2 NDE"}="HO2N NDE"; $resn_exch{"NH3* NDE"}=" H3D NDE";
$resn_exch{"NHO3 NDE"}="HO3N NDE"; $resn_exch{"NH4* NDE"}=" H4D NDE"; $resn_exch{"NH51 NDE"}="H51N NDE";
$resn_exch{"NH52 NDE"}="H52N NDE"; $resn_exch{"2HOA NDE"}="HOA2 NDE"; $resn_exch{"AH51 NDE"}="H51A NDE";
$resn_exch{"AH52 NDE"}="H52A NDE"; $resn_exch{"AH4* NDE"}=" H4B NDE"; $resn_exch{"AH1* NDE"}=" H1B NDE";
$resn_exch{"AH2* NDE"}=" H2B NDE"; $resn_exch{"AHO2 NDE"}="HO2A NDE"; $resn_exch{"AH3* NDE"}=" H3B NDE";
$resn_exch{"AHO3 NDE"}="HO3A NDE"; $resn_exch{" AH8 NDE"}=" H8A NDE"; $resn_exch{"AH61 NDE"}="H61A NDE";
$resn_exch{"AH62 NDE"}="H62A NDE"; $resn_exch{" AH2 NDE"}=" H2A NDE"; $resn_exch{"2HOP NDE"}="HOP2 NDE";
$resn_exch{"3HOP NDE"}="HOP3 NDE"; $resn_exch{"1H1  NDE"}=" H11 NDE"; $resn_exch{"2H1  NDE"}=" H12 NDE";
$resn_exch{"1H3  NDE"}=" H31 NDE"; $resn_exch{"1H31 NDE"}="H311 NDE"; $resn_exch{"1HB  NDF"}=" HB3 NDF";
$resn_exch{"2HB  NDF"}=" HB2 NDF"; $resn_exch{" HN  NDF"}=" H   NDF"; $resn_exch{"1H12 NDL"}="H121 NDL";
$resn_exch{"2H12 NDL"}="H122 NDL"; $resn_exch{"1H11 NDL"}="H111 NDL"; $resn_exch{"2H11 NDL"}="H112 NDL";
$resn_exch{"1H14 NDL"}="H141 NDL"; $resn_exch{"2H14 NDL"}="H142 NDL"; $resn_exch{"1H13 NDL"}="H131 NDL";
$resn_exch{"2H13 NDL"}="H132 NDL"; $resn_exch{"3H13 NDL"}="H133 NDL"; $resn_exch{"1H19 NDL"}="H191 NDL";
$resn_exch{"2H19 NDL"}="H192 NDL"; $resn_exch{"1H20 NDL"}="H201 NDL"; $resn_exch{"2H20 NDL"}="H202 NDL";
$resn_exch{"1H21 NDL"}="H211 NDL"; $resn_exch{"2H21 NDL"}="H212 NDL"; $resn_exch{"1H22 NDL"}="H221 NDL";
$resn_exch{"2H22 NDL"}="H222 NDL"; $resn_exch{"1H23 NDL"}="H231 NDL"; $resn_exch{"2H23 NDL"}="H232 NDL";
$resn_exch{"1H28 NDL"}="H281 NDL"; $resn_exch{"2H28 NDL"}="H282 NDL"; $resn_exch{"1H29 NDL"}="H291 NDL";
$resn_exch{"2H29 NDL"}="H292 NDL"; $resn_exch{"1H30 NDL"}="H301 NDL"; $resn_exch{"2H30 NDL"}="H302 NDL";
$resn_exch{"1H33 NDL"}="H331 NDL"; $resn_exch{"2H33 NDL"}="H332 NDL"; $resn_exch{"1H39 NDL"}="H391 NDL";
$resn_exch{"2H39 NDL"}="H392 NDL"; $resn_exch{"1H40 NDL"}="H401 NDL"; $resn_exch{"2H40 NDL"}="H402 NDL";
$resn_exch{"1H41 NDL"}="H411 NDL"; $resn_exch{"2H41 NDL"}="H412 NDL"; $resn_exch{"1H45 NDL"}="H451 NDL";
$resn_exch{"2H45 NDL"}="H452 NDL"; $resn_exch{"1HN9 NDM"}="HN91 NDM"; $resn_exch{"2HN9 NDM"}="HN92 NDM";
$resn_exch{"3HN9 NDM"}="HN93 NDM"; $resn_exch{"2H2  NDM"}=" H22 NDM"; $resn_exch{"1HN2 NDM"}="HN21 NDM";
$resn_exch{"2HN2 NDM"}="HN22 NDM"; $resn_exch{"1HN1 NDM"}="HN11 NDM"; $resn_exch{"6H6  NDM"}=" H66 NDM";
$resn_exch{"1HN7 NDM"}="HN71 NDM"; $resn_exch{"2HN7 NDM"}="HN72 NDM"; $resn_exch{"3HN7 NDM"}="HN73 NDM";
$resn_exch{"1HC8 NDM"}="HC81 NDM"; $resn_exch{"2HOA NDO"}="HOA2 NDO"; $resn_exch{"AH51 NDO"}="H51A NDO";
$resn_exch{"AH52 NDO"}="H52A NDO"; $resn_exch{"AH4* NDO"}=" H4B NDO"; $resn_exch{"AH3* NDO"}=" H3B NDO";
$resn_exch{"AHO3 NDO"}="HO3A NDO"; $resn_exch{"AH2* NDO"}=" H2B NDO"; $resn_exch{"AH1* NDO"}=" H1B NDO";
$resn_exch{" AH8 NDO"}=" H8A NDO"; $resn_exch{"AHO6 NDO"}="HO6A NDO"; $resn_exch{" AH2 NDO"}=" H2A NDO";
$resn_exch{"NH51 NDO"}="H51N NDO"; $resn_exch{"NH52 NDO"}="H52N NDO"; $resn_exch{"NH4* NDO"}=" H4D NDO";
$resn_exch{"NH3* NDO"}=" H3D NDO"; $resn_exch{"NHO3 NDO"}="HO3N NDO"; $resn_exch{"NH2* NDO"}=" H2D NDO";
$resn_exch{"NHO2 NDO"}="HO2N NDO"; $resn_exch{"NH1* NDO"}=" H1D NDO"; $resn_exch{" NH2 NDO"}=" H2N NDO";
$resn_exch{"NH71 NDO"}="H71N NDO"; $resn_exch{"NH72 NDO"}="H72N NDO"; $resn_exch{" NH4 NDO"}=" H4N NDO";
$resn_exch{" NH5 NDO"}=" H5N NDO"; $resn_exch{" NH6 NDO"}=" H6N NDO"; $resn_exch{"2HOP NDO"}="HOP2 NDO";
$resn_exch{"3HOP NDO"}="HOP3 NDO"; $resn_exch{"2HOA NDP"}="HOA2 NDP"; $resn_exch{"AH51 NDP"}="H51A NDP";
$resn_exch{"AH52 NDP"}="H52A NDP"; $resn_exch{"AH4* NDP"}=" H4B NDP"; $resn_exch{"AH3* NDP"}=" H3B NDP";
$resn_exch{"AHO3 NDP"}="HO3A NDP"; $resn_exch{"AH2* NDP"}=" H2B NDP"; $resn_exch{"AH1* NDP"}=" H1B NDP";
$resn_exch{" AH8 NDP"}=" H8A NDP"; $resn_exch{"AH61 NDP"}="H61A NDP"; $resn_exch{"AH62 NDP"}="H62A NDP";
$resn_exch{" AH2 NDP"}=" H2A NDP"; $resn_exch{"NH21 NDP"}="H21N NDP"; $resn_exch{"NH51 NDP"}="H51N NDP";
$resn_exch{"NH52 NDP"}="H52N NDP"; $resn_exch{"NH4* NDP"}=" H4D NDP"; $resn_exch{"NH3* NDP"}=" H3D NDP";
$resn_exch{"NHO3 NDP"}="HO3N NDP"; $resn_exch{"NH2* NDP"}=" H2D NDP"; $resn_exch{"NHO2 NDP"}="HO2N NDP";
$resn_exch{"NH1* NDP"}=" H1D NDP"; $resn_exch{" NH2 NDP"}=" H2N NDP"; $resn_exch{"NH71 NDP"}="H71N NDP";
$resn_exch{"NH72 NDP"}="H72N NDP"; $resn_exch{"NH41 NDP"}="H41N NDP"; $resn_exch{"NH42 NDP"}="H42N NDP";
$resn_exch{" NH5 NDP"}=" H5N NDP"; $resn_exch{" NH6 NDP"}=" H6N NDP"; $resn_exch{"2HOP NDP"}="HOP2 NDP";
$resn_exch{"3HOP NDP"}="HOP3 NDP"; $resn_exch{"1H20 NDR"}="H201 NDR"; $resn_exch{"2H20 NDR"}="H202 NDR";
$resn_exch{"1H15 NDR"}="H151 NDR"; $resn_exch{"2H15 NDR"}="H152 NDR"; $resn_exch{"1H18 NDR"}="H181 NDR";
$resn_exch{"2H18 NDR"}="H182 NDR"; $resn_exch{"1H9  NDR"}=" H91 NDR"; $resn_exch{"2H9  NDR"}=" H92 NDR";
$resn_exch{"1H13 NDR"}="H131 NDR"; $resn_exch{"2H13 NDR"}="H132 NDR"; $resn_exch{"1H6  NDR"}=" H61 NDR";
$resn_exch{"2H6  NDR"}=" H62 NDR"; $resn_exch{"1H7  NDR"}=" H71 NDR"; $resn_exch{"2H7  NDR"}=" H72 NDR";
$resn_exch{"3H7  NDR"}=" H73 NDR"; $resn_exch{"1H10 NDR"}="H101 NDR"; $resn_exch{"2H10 NDR"}="H102 NDR";
$resn_exch{"1H4  NDR"}=" H41 NDR"; $resn_exch{"2H4  NDR"}=" H42 NDR"; $resn_exch{"1H4  NDS"}=" H41 NDS";
$resn_exch{"2H4  NDS"}=" H42 NDS"; $resn_exch{"1H3  NDS"}=" H31 NDS"; $resn_exch{"2H3  NDS"}=" H32 NDS";
$resn_exch{"1H2  NDS"}=" H21 NDS"; $resn_exch{"2H2  NDS"}=" H22 NDS"; $resn_exch{"1H1  NDS"}=" H11 NDS";
$resn_exch{"2H1  NDS"}=" H12 NDS"; $resn_exch{"3H1  NDS"}=" H13 NDS"; $resn_exch{"1H7  NDS"}=" H71 NDS";
$resn_exch{"2H7  NDS"}=" H72 NDS"; $resn_exch{"3H7  NDS"}=" H73 NDS"; $resn_exch{"1H5  NDS"}=" H51 NDS";
$resn_exch{"2H5  NDS"}=" H52 NDS"; $resn_exch{"1H6  NDS"}=" H61 NDS"; $resn_exch{"2H6  NDS"}=" H62 NDS";
$resn_exch{"3H6  NDS"}=" H63 NDS"; $resn_exch{"1H21 NDT"}="H211 NDT"; $resn_exch{"2H21 NDT"}="H212 NDT";
$resn_exch{"3H21 NDT"}="H213 NDT"; $resn_exch{"1H5' NDU"}="H51' NDU"; $resn_exch{"2H5' NDU"}="H52' NDU";
$resn_exch{"1H2' NDU"}="H21' NDU"; $resn_exch{"2H2' NDU"}="H22' NDU"; $resn_exch{"1H61 NDU"}="H611 NDU";
$resn_exch{"2H61 NDU"}="H612 NDU"; $resn_exch{"1H15 NE8"}="H151 NE8"; $resn_exch{"2H15 NE8"}="H152 NE8";
$resn_exch{"1H14 NE8"}="H141 NE8"; $resn_exch{"2H14 NE8"}="H142 NE8"; $resn_exch{"1H23 NE8"}="H231 NE8";
$resn_exch{"2H23 NE8"}="H232 NE8"; $resn_exch{"1HN  NEA"}=" HN1 NEA"; $resn_exch{"2HN  NEA"}=" HN2 NEA";
$resn_exch{"1HB  NEA"}=" HB1 NEA"; $resn_exch{"2HB  NEA"}=" HB2 NEA"; $resn_exch{"1HG  NEA"}=" HG1 NEA";
$resn_exch{"2HG  NEA"}=" HG2 NEA"; $resn_exch{"1H5* NEA"}="H5'1 NEA"; $resn_exch{"2H5* NEA"}="H5'2 NEA";
$resn_exch{" H4* NEA"}=" H4' NEA"; $resn_exch{" H3* NEA"}=" H3' NEA"; $resn_exch{" H2* NEA"}=" H2' NEA";
$resn_exch{"2HO* NEA"}="HO'2 NEA"; $resn_exch{" H1* NEA"}=" H1' NEA"; $resn_exch{"1H6  NEA"}=" H61 NEA";
$resn_exch{"2H6  NEA"}=" H62 NEA"; $resn_exch{"1H1  NEB"}=" H11 NEB"; $resn_exch{"2H1  NEB"}=" H12 NEB";
$resn_exch{"3H1  NEB"}=" H13 NEB"; $resn_exch{"1H2  NEB"}=" H21 NEB"; $resn_exch{"2H2  NEB"}=" H22 NEB";
$resn_exch{"1H3  NEB"}=" H31 NEB"; $resn_exch{"2H3  NEB"}=" H32 NEB"; $resn_exch{"1H51 NEC"}="H511 NEC";
$resn_exch{"2H51 NEC"}="H512 NEC"; $resn_exch{"1H52 NEC"}="H521 NEC"; $resn_exch{"2H52 NEC"}="H522 NEC";
$resn_exch{"3H52 NEC"}="H523 NEC"; $resn_exch{" H4* NEC"}=" H4' NEC"; $resn_exch{" H3* NEC"}=" H3' NEC";
$resn_exch{"*HO3 NEC"}="HO3' NEC"; $resn_exch{" H2* NEC"}=" H2' NEC"; $resn_exch{"*HO2 NEC"}="HO2' NEC";
$resn_exch{" H1* NEC"}=" H1' NEC"; $resn_exch{"1HN6 NEC"}="HN61 NEC"; $resn_exch{"2HN6 NEC"}="HN62 NEC";
$resn_exch{"1H2  NED"}=" H21 NED"; $resn_exch{"2H2  NED"}=" H22 NED"; $resn_exch{"3H2  NED"}=" H23 NED";
$resn_exch{"1H6  NED"}=" H61 NED"; $resn_exch{"2H6  NED"}=" H62 NED"; $resn_exch{"1HN6 NED"}="HN61 NED";
$resn_exch{"2HN6 NED"}="HN62 NED"; $resn_exch{"3H6  NED"}=" H63 NED"; $resn_exch{"2HA  NEH"}=" HA2 NEH";
$resn_exch{"3HA  NEH"}=" HA3 NEH"; $resn_exch{"1HB  NEH"}=" HB1 NEH"; $resn_exch{"2HB  NEH"}=" HB2 NEH";
$resn_exch{"3HB  NEH"}=" HB3 NEH"; $resn_exch{"1H52 NEI"}="H521 NEI"; $resn_exch{"2H52 NEI"}="H522 NEI";
$resn_exch{"3H52 NEI"}="H523 NEI"; $resn_exch{"1H51 NEI"}="H511 NEI"; $resn_exch{"2H51 NEI"}="H512 NEI";
$resn_exch{"HN5* NEI"}="HN5' NEI"; $resn_exch{" H4* NEI"}=" H4' NEI"; $resn_exch{" H3* NEI"}=" H3' NEI";
$resn_exch{"HO3* NEI"}="HO3' NEI"; $resn_exch{" H2* NEI"}=" H2' NEI"; $resn_exch{"HO2* NEI"}="HO2' NEI";
$resn_exch{" H1* NEI"}=" H1' NEI"; $resn_exch{"1HN6 NEI"}="HN61 NEI"; $resn_exch{"2HN6 NEI"}="HN62 NEI";
$resn_exch{"1HB  NEM"}=" HB1 NEM"; $resn_exch{"2HB  NEM"}=" HB2 NEM"; $resn_exch{"1HM  NEM"}=" HM1 NEM";
$resn_exch{"2HM  NEM"}=" HM2 NEM"; $resn_exch{"3HM  NEM"}=" HM3 NEM"; $resn_exch{"1H1  NEN"}=" H11 NEN";
$resn_exch{"2H1  NEN"}=" H12 NEN"; $resn_exch{"1H4  NEN"}=" H41 NEN"; $resn_exch{"2H4  NEN"}=" H42 NEN";
$resn_exch{"1H5  NEN"}=" H51 NEN"; $resn_exch{"2H5  NEN"}=" H52 NEN"; $resn_exch{"1H6  NEN"}=" H61 NEN";
$resn_exch{"2H6  NEN"}=" H62 NEN"; $resn_exch{"3H6  NEN"}=" H63 NEN"; $resn_exch{"1H13 NEO"}="H131 NEO";
$resn_exch{"2H13 NEO"}="H132 NEO"; $resn_exch{"1H28 NEO"}="H281 NEO"; $resn_exch{"2H28 NEO"}="H282 NEO";
$resn_exch{" HN2 NEP"}=" H2  NEP"; $resn_exch{"1HB  NEP"}=" HB2 NEP"; $resn_exch{"2HB  NEP"}=" HB3 NEP";
$resn_exch{"1HOP NEP"}="HOP1 NEP"; $resn_exch{"2HOP NEP"}="HOP2 NEP"; $resn_exch{"1H2  NEQ"}=" H21 NEQ";
$resn_exch{"1H3  NEQ"}=" H31 NEQ"; $resn_exch{"1H5  NEQ"}=" H51 NEQ"; $resn_exch{"2H5  NEQ"}=" H52 NEQ";
$resn_exch{"1H6  NEQ"}=" H61 NEQ"; $resn_exch{"2H6  NEQ"}=" H62 NEQ"; $resn_exch{"3H6  NEQ"}=" H63 NEQ";
$resn_exch{"1H2  NER"}=" H21 NER"; $resn_exch{"2H2  NER"}=" H22 NER"; $resn_exch{"1H3  NER"}=" H31 NER";
$resn_exch{"2H3  NER"}=" H32 NER"; $resn_exch{"1H4  NER"}=" H41 NER"; $resn_exch{"2H4  NER"}=" H42 NER";
$resn_exch{"1H5  NER"}=" H51 NER"; $resn_exch{"2H5  NER"}=" H52 NER"; $resn_exch{"1H6  NER"}=" H61 NER";
$resn_exch{"2H6  NER"}=" H62 NER"; $resn_exch{"1H7  NER"}=" H71 NER"; $resn_exch{"2H7  NER"}=" H72 NER";
$resn_exch{"1H8  NER"}=" H81 NER"; $resn_exch{"2H8  NER"}=" H82 NER"; $resn_exch{"1H9  NER"}=" H91 NER";
$resn_exch{"2H9  NER"}=" H92 NER"; $resn_exch{"1H10 NER"}="H101 NER"; $resn_exch{"2H10 NER"}="H102 NER";
$resn_exch{"1H11 NER"}="H111 NER"; $resn_exch{"2H11 NER"}="H112 NER"; $resn_exch{"1H12 NER"}="H121 NER";
$resn_exch{"2H12 NER"}="H122 NER"; $resn_exch{"1H13 NER"}="H131 NER"; $resn_exch{"2H13 NER"}="H132 NER";
$resn_exch{"1H14 NER"}="H141 NER"; $resn_exch{"2H14 NER"}="H142 NER"; $resn_exch{"1H17 NER"}="H171 NER";
$resn_exch{"2H17 NER"}="H172 NER"; $resn_exch{"1H18 NER"}="H181 NER"; $resn_exch{"2H18 NER"}="H182 NER";
$resn_exch{"1H19 NER"}="H191 NER"; $resn_exch{"2H19 NER"}="H192 NER"; $resn_exch{"1H20 NER"}="H201 NER";
$resn_exch{"2H20 NER"}="H202 NER"; $resn_exch{"1H21 NER"}="H211 NER"; $resn_exch{"2H21 NER"}="H212 NER";
$resn_exch{"1H22 NER"}="H221 NER"; $resn_exch{"2H22 NER"}="H222 NER"; $resn_exch{"1H23 NER"}="H231 NER";
$resn_exch{"2H23 NER"}="H232 NER"; $resn_exch{"1H24 NER"}="H241 NER"; $resn_exch{"2H24 NER"}="H242 NER";
$resn_exch{"3H24 NER"}="H243 NER"; $resn_exch{"1HC7 NES"}="HC71 NES"; $resn_exch{"2HC7 NES"}="HC72 NES";
$resn_exch{"1HC8 NES"}="HC81 NES"; $resn_exch{"2HC8 NES"}="HC82 NES"; $resn_exch{"1HC2 NES"}="HC21 NES";
$resn_exch{"2HC2 NES"}="HC22 NES"; $resn_exch{"1HC4 NES"}="HC41 NES"; $resn_exch{"2HC4 NES"}="HC42 NES";
$resn_exch{"1HC6 NES"}="HC61 NES"; $resn_exch{"2HC6 NES"}="HC62 NES"; $resn_exch{"1H1  NET"}=" H11 NET";
$resn_exch{"2H1  NET"}=" H12 NET"; $resn_exch{"1H2  NET"}=" H21 NET"; $resn_exch{"2H2  NET"}=" H22 NET";
$resn_exch{"3H2  NET"}=" H23 NET"; $resn_exch{"1H3  NET"}=" H31 NET"; $resn_exch{"2H3  NET"}=" H32 NET";
$resn_exch{"1H4  NET"}=" H41 NET"; $resn_exch{"2H4  NET"}=" H42 NET"; $resn_exch{"3H4  NET"}=" H43 NET";
$resn_exch{"1H5  NET"}=" H51 NET"; $resn_exch{"2H5  NET"}=" H52 NET"; $resn_exch{"1H6  NET"}=" H61 NET";
$resn_exch{"2H6  NET"}=" H62 NET"; $resn_exch{"3H6  NET"}=" H63 NET"; $resn_exch{"1H7  NET"}=" H71 NET";
$resn_exch{"2H7  NET"}=" H72 NET"; $resn_exch{"1H8  NET"}=" H81 NET"; $resn_exch{"2H8  NET"}=" H82 NET";
$resn_exch{"3H8  NET"}=" H83 NET"; $resn_exch{"1H13 NEU"}="H131 NEU"; $resn_exch{"2H13 NEU"}="H132 NEU";
$resn_exch{"1H28 NEU"}="H281 NEU"; $resn_exch{"2H28 NEU"}="H282 NEU"; $resn_exch{"1H12 NEV"}="H121 NEV";
$resn_exch{"2H12 NEV"}="H122 NEV"; $resn_exch{"3H12 NEV"}="H123 NEV"; $resn_exch{"1H15 NEV"}="H151 NEV";
$resn_exch{"2H15 NEV"}="H152 NEV"; $resn_exch{"1H16 NEV"}="H161 NEV"; $resn_exch{"2H16 NEV"}="H162 NEV";
$resn_exch{"1H2  NEX"}=" H21 NEX"; $resn_exch{"2H2  NEX"}=" H22 NEX"; $resn_exch{"1H4  NEX"}=" H41 NEX";
$resn_exch{"2H4  NEX"}=" H42 NEX"; $resn_exch{"1H16 NEX"}="H161 NEX"; $resn_exch{"2H16 NEX"}="H162 NEX";
$resn_exch{"3H16 NEX"}="H163 NEX"; $resn_exch{"1H17 NEX"}="H171 NEX"; $resn_exch{"2H17 NEX"}="H172 NEX";
$resn_exch{"3H17 NEX"}="H173 NEX"; $resn_exch{"1H18 NEX"}="H181 NEX"; $resn_exch{"2H18 NEX"}="H182 NEX";
$resn_exch{"3H18 NEX"}="H183 NEX"; $resn_exch{"1H19 NEX"}="H191 NEX"; $resn_exch{"2H19 NEX"}="H192 NEX";
$resn_exch{"3H19 NEX"}="H193 NEX"; $resn_exch{"1H20 NEX"}="H201 NEX"; $resn_exch{"2H20 NEX"}="H202 NEX";
$resn_exch{"3H20 NEX"}="H203 NEX"; $resn_exch{"1H22 NEX"}="H221 NEX"; $resn_exch{"2H22 NEX"}="H222 NEX";
$resn_exch{"1H24 NEX"}="H241 NEX"; $resn_exch{"2H24 NEX"}="H242 NEX"; $resn_exch{"1H36 NEX"}="H361 NEX";
$resn_exch{"2H36 NEX"}="H362 NEX"; $resn_exch{"3H36 NEX"}="H363 NEX"; $resn_exch{"1H37 NEX"}="H371 NEX";
$resn_exch{"2H37 NEX"}="H372 NEX"; $resn_exch{"3H37 NEX"}="H373 NEX"; $resn_exch{"1H38 NEX"}="H381 NEX";
$resn_exch{"2H38 NEX"}="H382 NEX"; $resn_exch{"3H38 NEX"}="H383 NEX"; $resn_exch{"1H39 NEX"}="H391 NEX";
$resn_exch{"2H39 NEX"}="H392 NEX"; $resn_exch{"3H39 NEX"}="H393 NEX"; $resn_exch{"1H40 NEX"}="H401 NEX";
$resn_exch{"2H40 NEX"}="H402 NEX"; $resn_exch{"3H40 NEX"}="H403 NEX"; $resn_exch{"1H14 NEZ"}="H141 NEZ";
$resn_exch{"2H14 NEZ"}="H142 NEZ"; $resn_exch{"1H23 NEZ"}="H231 NEZ"; $resn_exch{"2H23 NEZ"}="H232 NEZ";
$resn_exch{"1H15 NEZ"}="H151 NEZ"; $resn_exch{"2H15 NEZ"}="H152 NEZ"; $resn_exch{"1H24 NEZ"}="H241 NEZ";
$resn_exch{"2H24 NEZ"}="H242 NEZ"; $resn_exch{"3H24 NEZ"}="H243 NEZ"; $resn_exch{"2H11  NF"}="H112  NF";
$resn_exch{"2H12  NF"}="H122  NF"; $resn_exch{"1H9   NF"}=" H91  NF"; $resn_exch{"1H8   NF"}=" H81  NF";
$resn_exch{"1H7   NF"}=" H71  NF"; $resn_exch{"1H6   NF"}=" H61  NF"; $resn_exch{"1H5   NF"}=" H51  NF";
$resn_exch{"1H4   NF"}=" H41  NF"; $resn_exch{"2H21  NF"}="H212  NF"; $resn_exch{"2H22  NF"}="H222  NF";
$resn_exch{"2H31  NF"}="H312  NF"; $resn_exch{"2H32  NF"}="H322  NF"; $resn_exch{"2H41  NF"}="H412  NF";
$resn_exch{"2H42  NF"}="H422  NF"; $resn_exch{"2H51  NF"}="H512  NF"; $resn_exch{"2H52  NF"}="H522  NF";
$resn_exch{"2H61  NF"}="H612  NF"; $resn_exch{"2H62  NF"}="H622  NF"; $resn_exch{"2H71  NF"}="H712  NF";
$resn_exch{"2H72  NF"}="H722  NF"; $resn_exch{"2H81  NF"}="H812  NF"; $resn_exch{"2H82  NF"}="H822  NF";
$resn_exch{"2H91  NF"}="H912  NF"; $resn_exch{"2H92  NF"}="H922  NF"; $resn_exch{"3H9   NF"}=" H93  NF";
$resn_exch{"3H8   NF"}=" H83  NF"; $resn_exch{"3H7   NF"}=" H73  NF"; $resn_exch{"3H6   NF"}=" H63  NF";
$resn_exch{"3H5   NF"}=" H53  NF"; $resn_exch{"3H4   NF"}=" H43  NF"; $resn_exch{"1HM  NF2"}=" HM1 NF2";
$resn_exch{"2HM  NF2"}=" HM2 NF2"; $resn_exch{"3HM  NF2"}=" HM3 NF2"; $resn_exch{" H1* NF2"}=" H1' NF2";
$resn_exch{" H2* NF2"}=" H2' NF2"; $resn_exch{"2HO* NF2"}="HO2' NF2"; $resn_exch{" H3* NF2"}=" H3' NF2";
$resn_exch{"H3T  NF2"}="HO3' NF2"; $resn_exch{" H4* NF2"}=" H4' NF2"; $resn_exch{"1H5* NF2"}=" H5' NF2";
$resn_exch{"2H5* NF2"}="H5'' NF2"; $resn_exch{"2HOP NF2"}="HOP2 NF2"; $resn_exch{"3HOP NF2"}="HOP3 NF2";
$resn_exch{" HN2 NFA"}=" H2  NFA"; $resn_exch{"1HB  NFA"}=" HB2 NFA"; $resn_exch{"2HB  NFA"}=" HB3 NFA";
$resn_exch{"1HXT NFA"}="HXT1 NFA"; $resn_exch{"2HXT NFA"}="HXT2 NFA"; $resn_exch{"1H3O NFG"}="H3O1 NFG";
$resn_exch{"1H4O NFG"}="H4O1 NFG"; $resn_exch{"2H6  NFG"}=" H62 NFG"; $resn_exch{"1H6  NFG"}=" H61 NFG";
$resn_exch{"1H19 NFN"}="H191 NFN"; $resn_exch{"2H19 NFN"}="H192 NFN"; $resn_exch{"3H19 NFN"}="H193 NFN";
$resn_exch{"1H20 NFN"}="H201 NFN"; $resn_exch{"2H20 NFN"}="H202 NFN"; $resn_exch{"3H20 NFN"}="H203 NFN";
$resn_exch{"1H17 NFN"}="H171 NFN"; $resn_exch{"2H17 NFN"}="H172 NFN"; $resn_exch{"1H18 NFN"}="H181 NFN";
$resn_exch{"2H18 NFN"}="H182 NFN"; $resn_exch{"3H18 NFN"}="H183 NFN"; $resn_exch{"1H1  NFP"}=" H11 NFP";
$resn_exch{"2H1  NFP"}=" H12 NFP"; $resn_exch{"3H1  NFP"}=" H13 NFP"; $resn_exch{"1H2  NFP"}=" H21 NFP";
$resn_exch{"2H2  NFP"}=" H22 NFP"; $resn_exch{"1HN  NFP"}=" HN1 NFP"; $resn_exch{"2HN  NFP"}=" HN2 NFP";
$resn_exch{"1H4  NFP"}=" H41 NFP"; $resn_exch{"2H4  NFP"}=" H42 NFP"; $resn_exch{"1H5  NFP"}=" H51 NFP";
$resn_exch{"2H5  NFP"}=" H52 NFP"; $resn_exch{"1H41 NFT"}="H411 NFT"; $resn_exch{"2H41 NFT"}="H412 NFT";
$resn_exch{"1H18 NFT"}="H181 NFT"; $resn_exch{"2H18 NFT"}="H182 NFT"; $resn_exch{"1H20 NFT"}="H201 NFT";
$resn_exch{"2H20 NFT"}="H202 NFT"; $resn_exch{"3H20 NFT"}="H203 NFT"; $resn_exch{"1H21 NFT"}="H211 NFT";
$resn_exch{"2H21 NFT"}="H212 NFT"; $resn_exch{"3H21 NFT"}="H213 NFT"; $resn_exch{"1H63 NFT"}="H631 NFT";
$resn_exch{"2H63 NFT"}="H632 NFT"; $resn_exch{"1H44 NFT"}="H441 NFT"; $resn_exch{"2H44 NFT"}="H442 NFT";
$resn_exch{"1H45 NFT"}="H451 NFT"; $resn_exch{"2H45 NFT"}="H452 NFT"; $resn_exch{"1HN4 NFZ"}="HN41 NFZ";
$resn_exch{"2HN4 NFZ"}="HN42 NFZ"; $resn_exch{"1H6  NG1"}=" H61 NG1"; $resn_exch{"2H6  NG1"}=" H62 NG1";
$resn_exch{"1H8  NG1"}=" H81 NG1"; $resn_exch{"2H8  NG1"}=" H82 NG1"; $resn_exch{"3H8  NG1"}=" H83 NG1";
$resn_exch{"1H6  NG6"}=" H61 NG6"; $resn_exch{"2H6  NG6"}=" H62 NG6"; $resn_exch{"1H8  NG6"}=" H81 NG6";
$resn_exch{"2H8  NG6"}=" H82 NG6"; $resn_exch{"3H8  NG6"}=" H83 NG6"; $resn_exch{"3HOS NG6"}="HOS3 NG6";
$resn_exch{"1H6  NGA"}=" H61 NGA"; $resn_exch{"2H6  NGA"}=" H62 NGA"; $resn_exch{"1H8  NGA"}=" H81 NGA";
$resn_exch{"2H8  NGA"}=" H82 NGA"; $resn_exch{"3H8  NGA"}=" H83 NGA"; $resn_exch{"1H2A NGD"}="H2A1 NGD";
$resn_exch{"2H2A NGD"}="H2A2 NGD"; $resn_exch{" H1* NGD"}=" H1' NGD"; $resn_exch{" H2* NGD"}=" H2' NGD";
$resn_exch{" H3* NGD"}=" H3' NGD"; $resn_exch{" H4* NGD"}=" H4' NGD"; $resn_exch{"1H5  NGD"}=" H51 NGD";
$resn_exch{"2H5  NGD"}=" H52 NGD"; $resn_exch{"1H5* NGD"}="H5'1 NGD"; $resn_exch{"2H5* NGD"}="H5'2 NGD";
$resn_exch{"HC4* NGD"}="HC4' NGD"; $resn_exch{"HC3* NGD"}="HC3' NGD"; $resn_exch{"HO3* NGD"}="HO3' NGD";
$resn_exch{"HC2* NGD"}="HC2' NGD"; $resn_exch{"HO2* NGD"}="HO2' NGD"; $resn_exch{"HC1* NGD"}="HC1' NGD";
$resn_exch{"1HN7 NGD"}="HN71 NGD"; $resn_exch{"2HN7 NGD"}="HN72 NGD"; $resn_exch{"1H7  NGH"}=" H71 NGH";
$resn_exch{"2H7  NGH"}=" H72 NGH"; $resn_exch{"3H7  NGH"}=" H73 NGH"; $resn_exch{"1H9  NGH"}=" H91 NGH";
$resn_exch{"2H9  NGH"}=" H92 NGH"; $resn_exch{"1H10 NGH"}="H101 NGH"; $resn_exch{"2H10 NGH"}="H102 NGH";
$resn_exch{"1H13 NGH"}="H131 NGH"; $resn_exch{"2H13 NGH"}="H132 NGH"; $resn_exch{"3H13 NGH"}="H133 NGH";
$resn_exch{"1H14 NGH"}="H141 NGH"; $resn_exch{"2H14 NGH"}="H142 NGH"; $resn_exch{"3H14 NGH"}="H143 NGH";
$resn_exch{"1H6C NGK"}="H6C1 NGK"; $resn_exch{"2H6C NGK"}="H6C2 NGK"; $resn_exch{"1H8C NGK"}="H8C1 NGK";
$resn_exch{"2H8C NGK"}="H8C2 NGK"; $resn_exch{"3H8C NGK"}="H8C3 NGK"; $resn_exch{"1H6  NGL"}=" H61 NGL";
$resn_exch{"2H6  NGL"}=" H62 NGL"; $resn_exch{"1H8  NGL"}=" H81 NGL"; $resn_exch{"2H8  NGL"}=" H82 NGL";
$resn_exch{"3H8  NGL"}=" H83 NGL"; $resn_exch{"3HOS NGL"}="HOS3 NGL"; $resn_exch{"1H8  NGM"}=" H81 NGM";
$resn_exch{"2H8  NGM"}=" H82 NGM"; $resn_exch{"1H13 NGM"}="H131 NGM"; $resn_exch{"2H13 NGM"}="H132 NGM";
$resn_exch{"3H13 NGM"}="H133 NGM"; $resn_exch{"1H15 NGM"}="H151 NGM"; $resn_exch{"2H15 NGM"}="H152 NGM";
$resn_exch{"3H15 NGM"}="H153 NGM"; $resn_exch{"1H22 NGM"}="H221 NGM"; $resn_exch{"2H22 NGM"}="H222 NGM";
$resn_exch{"3H22 NGM"}="H223 NGM"; $resn_exch{"1H23 NGM"}="H231 NGM"; $resn_exch{"2H23 NGM"}="H232 NGM";
$resn_exch{"3H23 NGM"}="H233 NGM"; $resn_exch{"1H24 NGM"}="H241 NGM"; $resn_exch{"2H24 NGM"}="H242 NGM";
$resn_exch{"3H24 NGM"}="H243 NGM"; $resn_exch{"1H25 NGM"}="H251 NGM"; $resn_exch{"2H25 NGM"}="H252 NGM";
$resn_exch{"3H25 NGM"}="H253 NGM"; $resn_exch{"1H26 NGM"}="H261 NGM"; $resn_exch{"2H26 NGM"}="H262 NGM";
$resn_exch{"3H26 NGM"}="H263 NGM"; $resn_exch{"1H27 NGM"}="H271 NGM"; $resn_exch{"2H27 NGM"}="H272 NGM";
$resn_exch{"3H27 NGM"}="H273 NGM"; $resn_exch{"1H28 NGM"}="H281 NGM"; $resn_exch{"2H28 NGM"}="H282 NGM";
$resn_exch{"3H28 NGM"}="H283 NGM"; $resn_exch{"1H29 NGM"}="H291 NGM"; $resn_exch{"2H29 NGM"}="H292 NGM";
$resn_exch{"3H29 NGM"}="H293 NGM"; $resn_exch{" H1* NGM"}=" H1' NGM"; $resn_exch{" H2* NGM"}=" H2' NGM";
$resn_exch{" H4* NGM"}=" H4' NGM"; $resn_exch{" H5* NGM"}=" H5' NGM"; $resn_exch{"1HC6 NGO"}="HC61 NGO";
$resn_exch{"2HC6 NGO"}="HC62 NGO"; $resn_exch{"1HC8 NGO"}="HC81 NGO"; $resn_exch{"2HC8 NGO"}="HC82 NGO";
$resn_exch{"3HC8 NGO"}="HC83 NGO"; $resn_exch{"1H8  NGP"}=" H81 NGP"; $resn_exch{"2H8  NGP"}=" H82 NGP";
$resn_exch{"1H13 NGP"}="H131 NGP"; $resn_exch{"2H13 NGP"}="H132 NGP"; $resn_exch{"3H13 NGP"}="H133 NGP";
$resn_exch{"1H15 NGP"}="H151 NGP"; $resn_exch{"2H15 NGP"}="H152 NGP"; $resn_exch{"3H15 NGP"}="H153 NGP";
$resn_exch{"19H1 NGP"}="H119 NGP"; $resn_exch{"21H2 NGP"}="H221 NGP"; $resn_exch{"1H22 NGP"}="H22A NGP";
$resn_exch{"2H22 NGP"}="H222 NGP"; $resn_exch{"3H22 NGP"}="H223 NGP"; $resn_exch{"1H23 NGP"}="H231 NGP";
$resn_exch{"2H23 NGP"}="H232 NGP"; $resn_exch{"3H23 NGP"}="H233 NGP"; $resn_exch{"1H24 NGP"}="H241 NGP";
$resn_exch{"2H24 NGP"}="H242 NGP"; $resn_exch{"3H24 NGP"}="H243 NGP"; $resn_exch{"1H25 NGP"}="H251 NGP";
$resn_exch{"2H25 NGP"}="H252 NGP"; $resn_exch{"3H25 NGP"}="H253 NGP"; $resn_exch{"1H26 NGP"}="H261 NGP";
$resn_exch{"2H26 NGP"}="H262 NGP"; $resn_exch{"3H26 NGP"}="H263 NGP"; $resn_exch{"1H27 NGP"}="H271 NGP";
$resn_exch{"2H27 NGP"}="H272 NGP"; $resn_exch{"3H27 NGP"}="H273 NGP"; $resn_exch{"1H28 NGP"}="H281 NGP";
$resn_exch{"2H28 NGP"}="H282 NGP"; $resn_exch{"3H28 NGP"}="H283 NGP"; $resn_exch{"1H29 NGP"}="H291 NGP";
$resn_exch{"2H29 NGP"}="H292 NGP"; $resn_exch{"3H29 NGP"}="H293 NGP"; $resn_exch{" H1* NGP"}=" H1' NGP";
$resn_exch{" H2* NGP"}=" H2' NGP"; $resn_exch{" H4* NGP"}=" H4' NGP"; $resn_exch{" H5* NGP"}=" H5' NGP";
$resn_exch{"1H6  NGS"}=" H61 NGS"; $resn_exch{"2H6  NGS"}=" H62 NGS"; $resn_exch{"1HH3 NGS"}="HH31 NGS";
$resn_exch{"2HH3 NGS"}="HH32 NGS"; $resn_exch{"3HH3 NGS"}="HH33 NGS"; $resn_exch{"1HC6 NGT"}="HC61 NGT";
$resn_exch{"2HC6 NGT"}="HC62 NGT"; $resn_exch{"1HC8 NGT"}="HC81 NGT"; $resn_exch{"2HC8 NGT"}="HC82 NGT";
$resn_exch{"3HC8 NGT"}="HC83 NGT"; $resn_exch{"1H8  NGU"}=" H81 NGU"; $resn_exch{"2H8  NGU"}=" H82 NGU";
$resn_exch{"1H13 NGU"}="H131 NGU"; $resn_exch{"2H13 NGU"}="H132 NGU"; $resn_exch{"3H13 NGU"}="H133 NGU";
$resn_exch{"1H15 NGU"}="H151 NGU"; $resn_exch{"2H15 NGU"}="H152 NGU"; $resn_exch{"3H15 NGU"}="H153 NGU";
$resn_exch{" H1* NGU"}=" H1' NGU"; $resn_exch{" H2* NGU"}=" H2' NGU"; $resn_exch{" H4* NGU"}=" H4' NGU";
$resn_exch{" H5* NGU"}=" H5' NGU"; $resn_exch{"1H25 NGU"}="H251 NGU"; $resn_exch{"2H25 NGU"}="H252 NGU";
$resn_exch{"3H25 NGU"}="H253 NGU"; $resn_exch{"1H26 NGU"}="H261 NGU"; $resn_exch{"2H26 NGU"}="H262 NGU";
$resn_exch{"3H26 NGU"}="H263 NGU"; $resn_exch{"1H27 NGU"}="H271 NGU"; $resn_exch{"2H27 NGU"}="H272 NGU";
$resn_exch{"3H27 NGU"}="H273 NGU"; $resn_exch{"1H28 NGU"}="H281 NGU"; $resn_exch{"2H28 NGU"}="H282 NGU";
$resn_exch{"3H28 NGU"}="H283 NGU"; $resn_exch{"1H29 NGU"}="H291 NGU"; $resn_exch{"2H29 NGU"}="H292 NGU";
$resn_exch{"3H29 NGU"}="H293 NGU"; $resn_exch{"1H22 NGU"}="H221 NGU"; $resn_exch{"2H22 NGU"}="H222 NGU";
$resn_exch{"3H22 NGU"}="H223 NGU"; $resn_exch{"1H23 NGU"}="H231 NGU"; $resn_exch{"2H23 NGU"}="H232 NGU";
$resn_exch{"1H24 NGU"}="H241 NGU"; $resn_exch{"2H24 NGU"}="H242 NGU"; $resn_exch{"3H24 NGU"}="H243 NGU";
$resn_exch{"1H15 NGV"}="H151 NGV"; $resn_exch{"2H15 NGV"}="H152 NGV"; $resn_exch{"3H15 NGV"}="H153 NGV";
$resn_exch{"1H13 NGV"}="H131 NGV"; $resn_exch{"2H13 NGV"}="H132 NGV"; $resn_exch{"3H13 NGV"}="H133 NGV";
$resn_exch{"1H8  NGV"}=" H81 NGV"; $resn_exch{"2H8  NGV"}=" H82 NGV"; $resn_exch{"1H6  NGZ"}=" H61 NGZ";
$resn_exch{"2H6  NGZ"}=" H62 NGZ"; $resn_exch{"1H8  NGZ"}=" H81 NGZ"; $resn_exch{"2H8  NGZ"}=" H82 NGZ";
$resn_exch{"3H8  NGZ"}=" H83 NGZ"; $resn_exch{"1HN  NH2"}=" HN1 NH2"; $resn_exch{"2HN  NH2"}=" HN2 NH2";
$resn_exch{"1HN  NH3"}=" HN1 NH3"; $resn_exch{"2HN  NH3"}=" HN2 NH3"; $resn_exch{"3HN  NH3"}=" HN3 NH3";
$resn_exch{"1HN  NH4"}=" HN1 NH4"; $resn_exch{"2HN  NH4"}=" HN2 NH4"; $resn_exch{"3HN  NH4"}=" HN3 NH4";
$resn_exch{"4HN  NH4"}=" HN4 NH4"; $resn_exch{"1H16 NHB"}="H161 NHB"; $resn_exch{"2H16 NHB"}="H162 NHB";
$resn_exch{"3H16 NHB"}="H163 NHB"; $resn_exch{"2HOA NHD"}="HOA2 NHD"; $resn_exch{"AH51 NHD"}="H51A NHD";
$resn_exch{"AH52 NHD"}="H52A NHD"; $resn_exch{"AH4* NHD"}=" H4B NHD"; $resn_exch{"AH3* NHD"}=" H3B NHD";
$resn_exch{"AHO3 NHD"}="HO3A NHD"; $resn_exch{"AH2* NHD"}=" H2B NHD"; $resn_exch{"AHO2 NHD"}="HO2A NHD";
$resn_exch{"AH1* NHD"}=" H1B NHD"; $resn_exch{" AH8 NHD"}=" H8A NHD"; $resn_exch{" AH6 NHD"}=" H6A NHD";
$resn_exch{" AH2 NHD"}=" H2A NHD"; $resn_exch{"NH51 NHD"}="H51N NHD"; $resn_exch{"NH52 NHD"}="H52N NHD";
$resn_exch{"NH4* NHD"}=" H4D NHD"; $resn_exch{"NH3* NHD"}=" H3D NHD"; $resn_exch{"NHO3 NHD"}="HO3N NHD";
$resn_exch{"NH2* NHD"}=" H2D NHD"; $resn_exch{"NHO2 NHD"}="HO2N NHD"; $resn_exch{"NH1* NHD"}=" H1D NHD";
$resn_exch{" NH2 NHD"}=" H2N NHD"; $resn_exch{"NH71 NHD"}="H71N NHD"; $resn_exch{"NH72 NHD"}="H72N NHD";
$resn_exch{" NH4 NHD"}=" H4N NHD"; $resn_exch{" NH5 NHD"}=" H5N NHD"; $resn_exch{" NH6 NHD"}=" H6N NHD";
$resn_exch{"1H3' NHE"}="H3'1 NHE"; $resn_exch{"2H3' NHE"}="H3'2 NHE"; $resn_exch{"1H2' NHE"}="H2'1 NHE";
$resn_exch{"2H2' NHE"}="H2'2 NHE"; $resn_exch{"1HC' NHE"}="HC'1 NHE"; $resn_exch{"1H6' NHE"}="H6'1 NHE";
$resn_exch{"2H6' NHE"}="H6'2 NHE"; $resn_exch{"1HC1 NHE"}="HC11 NHE"; $resn_exch{"2HC1 NHE"}="HC12 NHE";
$resn_exch{"1HC2 NHE"}="HC21 NHE"; $resn_exch{"2HC2 NHE"}="HC22 NHE"; $resn_exch{"3HO  NHE"}=" HO3 NHE";
$resn_exch{"1H5' NHE"}="H5'1 NHE"; $resn_exch{"2H5' NHE"}="H5'2 NHE"; $resn_exch{"1H4' NHE"}="H4'1 NHE";
$resn_exch{"2H4' NHE"}="H4'2 NHE"; $resn_exch{" H2  NHL"}=" HC2 NHL"; $resn_exch{"1H11 NHL"}="H111 NHL";
$resn_exch{"2H11 NHL"}="H112 NHL"; $resn_exch{"1HG  NHL"}=" HG  NHL"; $resn_exch{"1HB  NHL"}=" HB3 NHL";
$resn_exch{"2HB  NHL"}=" HB2 NHL"; $resn_exch{" NH2 NHL"}=" H2  NHL"; $resn_exch{"1HN6 NHM"}="HN61 NHM";
$resn_exch{"2HN6 NHM"}="HN62 NHM"; $resn_exch{"1HEM NHM"}="HEM1 NHM"; $resn_exch{"2HEM NHM"}="HEM2 NHM";
$resn_exch{"3HEM NHM"}="HEM3 NHM"; $resn_exch{"1HAM NHM"}="HAM1 NHM"; $resn_exch{"2HAM NHM"}="HAM2 NHM";
$resn_exch{"1HBM NHM"}="HBM1 NHM"; $resn_exch{"2HBM NHM"}="HBM2 NHM"; $resn_exch{"1HCM NHM"}="HCM1 NHM";
$resn_exch{"2HCM NHM"}="HCM2 NHM"; $resn_exch{"1HDM NHM"}="HDM1 NHM"; $resn_exch{"2HDM NHM"}="HDM2 NHM";
$resn_exch{"1H9M NHM"}="H9M1 NHM"; $resn_exch{"2H9M NHM"}="H9M2 NHM"; $resn_exch{"1H8M NHM"}="H8M1 NHM";
$resn_exch{"2H8M NHM"}="H8M2 NHM"; $resn_exch{"1H7M NHM"}="H7M1 NHM"; $resn_exch{"2H7M NHM"}="H7M2 NHM";
$resn_exch{"1H6M NHM"}="H6M1 NHM"; $resn_exch{"2H6M NHM"}="H6M2 NHM"; $resn_exch{"1H2M NHM"}="H2M1 NHM";
$resn_exch{"2H2M NHM"}="H2M2 NHM"; $resn_exch{"1H3M NHM"}="H3M1 NHM"; $resn_exch{"2H3M NHM"}="H3M2 NHM";
$resn_exch{"1H4M NHM"}="H4M1 NHM"; $resn_exch{"2H4M NHM"}="H4M2 NHM"; $resn_exch{"1H5M NHM"}="H5M1 NHM";
$resn_exch{"2H5M NHM"}="H5M2 NHM"; $resn_exch{"1HP  NHM"}=" HP1 NHM"; $resn_exch{"2HP  NHM"}=" HP2 NHM";
$resn_exch{"AHO7 NHM"}="HO7A NHM"; $resn_exch{"AHO8 NHM"}="HO8A NHM"; $resn_exch{"1H6  NHM"}=" H61 NHM";
$resn_exch{"2H6  NHM"}=" H62 NHM"; $resn_exch{"1H7  NHM"}=" H71 NHM"; $resn_exch{"2H7  NHM"}=" H72 NHM";
$resn_exch{"1H13 NHM"}="H131 NHM"; $resn_exch{"2H13 NHM"}="H132 NHM"; $resn_exch{"3H13 NHM"}="H133 NHM";
$resn_exch{"1H14 NHM"}="H141 NHM"; $resn_exch{"2H14 NHM"}="H142 NHM"; $resn_exch{"3H14 NHM"}="H143 NHM";
$resn_exch{"1H12 NHM"}="H121 NHM"; $resn_exch{"2H12 NHM"}="H122 NHM"; $resn_exch{"AHO1 NHM"}="HO1A NHM";
$resn_exch{"XHO2 NHM"}="HO2X NHM"; $resn_exch{"1H5X NHM"}="H5X1 NHM"; $resn_exch{"2H5X NHM"}="H5X2 NHM";
$resn_exch{"AHO5 NHM"}="HO5A NHM"; $resn_exch{"1H3  NHM"}=" H31 NHM"; $resn_exch{"2H3  NHM"}=" H32 NHM";
$resn_exch{"1H2  NHM"}=" H21 NHM"; $resn_exch{"2H2  NHM"}=" H22 NHM"; $resn_exch{"1H1  NHN"}=" H11 NHN";
$resn_exch{"2H1  NHN"}=" H12 NHN"; $resn_exch{"1H2  NHN"}=" H21 NHN"; $resn_exch{"2H2  NHN"}=" H22 NHN";
$resn_exch{"1H12 NHN"}="H121 NHN"; $resn_exch{"2H12 NHN"}="H122 NHN"; $resn_exch{"1H13 NHN"}="H131 NHN";
$resn_exch{"2H13 NHN"}="H132 NHN"; $resn_exch{"1H18 NHN"}="H181 NHN"; $resn_exch{"2H18 NHN"}="H182 NHN";
$resn_exch{"1H19 NHN"}="H191 NHN"; $resn_exch{"2H19 NHN"}="H192 NHN"; $resn_exch{"1H20 NHN"}="H201 NHN";
$resn_exch{"2H20 NHN"}="H202 NHN"; $resn_exch{"1H21 NHN"}="H211 NHN"; $resn_exch{"2H21 NHN"}="H212 NHN";
$resn_exch{"1H22 NHN"}="H221 NHN"; $resn_exch{"2H22 NHN"}="H222 NHN"; $resn_exch{"1H26 NHN"}="H261 NHN";
$resn_exch{"2H26 NHN"}="H262 NHN"; $resn_exch{"1H28 NHN"}="H281 NHN"; $resn_exch{"2H28 NHN"}="H282 NHN";
$resn_exch{"1H34 NHN"}="H341 NHN"; $resn_exch{"2H34 NHN"}="H342 NHN"; $resn_exch{"1H35 NHN"}="H351 NHN";
$resn_exch{"2H35 NHN"}="H352 NHN"; $resn_exch{"1H36 NHN"}="H361 NHN"; $resn_exch{"2H36 NHN"}="H362 NHN";
$resn_exch{"3H36 NHN"}="H363 NHN"; $resn_exch{"1H41 NHN"}="H411 NHN"; $resn_exch{"2H41 NHN"}="H412 NHN";
$resn_exch{"1H56 NHN"}="H561 NHN"; $resn_exch{"2H56 NHN"}="H562 NHN"; $resn_exch{"3H56 NHN"}="H563 NHN";
$resn_exch{"1H57 NHN"}="H571 NHN"; $resn_exch{"2H57 NHN"}="H572 NHN"; $resn_exch{"3H57 NHN"}="H573 NHN";
$resn_exch{"1H5' NHO"}="H5'1 NHO"; $resn_exch{"2H5' NHO"}="H5'2 NHO"; $resn_exch{"1H6A NHO"}="H6A1 NHO";
$resn_exch{"2H6A NHO"}="H6A2 NHO"; $resn_exch{"1H5* NHO"}="H5A1 NHO"; $resn_exch{"2H5* NHO"}="H5A2 NHO";
$resn_exch{" H4* NHO"}=" H4B NHO"; $resn_exch{" H3* NHO"}=" H3B NHO"; $resn_exch{" H2* NHO"}=" H2B NHO";
$resn_exch{" H1* NHO"}=" H1B NHO"; $resn_exch{"1H7N NHO"}="H7N1 NHO"; $resn_exch{"2H7N NHO"}="H7N2 NHO";
$resn_exch{"1H3R NHO"}="H3R1 NHO"; $resn_exch{"2H3R NHO"}="H3R2 NHO"; $resn_exch{"1H1R NHO"}="H1R1 NHO";
$resn_exch{"2H1R NHO"}="H1R2 NHO"; $resn_exch{"1HN1 NHP"}="HN11 NHP"; $resn_exch{"2HN1 NHP"}="HN12 NHP";
$resn_exch{"1H7  NHP"}=" H71 NHP"; $resn_exch{"2H7  NHP"}=" H72 NHP"; $resn_exch{"1H8  NHP"}=" H81 NHP";
$resn_exch{"2H8  NHP"}=" H82 NHP"; $resn_exch{"1H9  NHP"}=" H91 NHP"; $resn_exch{"2H9  NHP"}=" H92 NHP";
$resn_exch{"1H10 NHP"}="H101 NHP"; $resn_exch{"2H10 NHP"}="H102 NHP"; $resn_exch{"1H6A NHQ"}="H6A1 NHQ";
$resn_exch{"2H6A NHQ"}="H6A2 NHQ"; $resn_exch{"1HEM NHQ"}="HEM1 NHQ"; $resn_exch{"2HEM NHQ"}="HEM2 NHQ";
$resn_exch{"1HDM NHQ"}="HDM1 NHQ"; $resn_exch{"2HDM NHQ"}="HDM2 NHQ"; $resn_exch{"1HBM NHQ"}="HBM1 NHQ";
$resn_exch{"2HBM NHQ"}="HBM2 NHQ"; $resn_exch{"1HCM NHQ"}="HCM1 NHQ"; $resn_exch{"2HCM NHQ"}="HCM2 NHQ";
$resn_exch{"1H9M NHQ"}="H9M1 NHQ"; $resn_exch{"2H9M NHQ"}="H9M2 NHQ"; $resn_exch{"1H8M NHQ"}="H8M1 NHQ";
$resn_exch{"2H8M NHQ"}="H8M2 NHQ"; $resn_exch{"1H7M NHQ"}="H7M1 NHQ"; $resn_exch{"2H7M NHQ"}="H7M2 NHQ";
$resn_exch{"1H53 NHQ"}="H531 NHQ"; $resn_exch{"2H53 NHQ"}="H532 NHQ"; $resn_exch{"1H3M NHQ"}="H3M1 NHQ";
$resn_exch{"2H3M NHQ"}="H3M2 NHQ"; $resn_exch{"1H4M NHQ"}="H4M1 NHQ"; $resn_exch{"2H4M NHQ"}="H4M2 NHQ";
$resn_exch{"1H5M NHQ"}="H5M1 NHQ"; $resn_exch{"2H5M NHQ"}="H5M2 NHQ"; $resn_exch{"1HPC NHQ"}="HPC1 NHQ";
$resn_exch{"2HPC NHQ"}="HPC2 NHQ"; $resn_exch{"1H2C NHQ"}="H2C1 NHQ"; $resn_exch{"2H2C NHQ"}="H2C2 NHQ";
$resn_exch{"1H3C NHQ"}="H3C1 NHQ"; $resn_exch{"2H3C NHQ"}="H3C2 NHQ"; $resn_exch{"1H6C NHQ"}="H6C1 NHQ";
$resn_exch{"2H6C NHQ"}="H6C2 NHQ"; $resn_exch{"1H7C NHQ"}="H7C1 NHQ"; $resn_exch{"2H7C NHQ"}="H7C2 NHQ";
$resn_exch{"1H13 NHQ"}="H131 NHQ"; $resn_exch{"2H13 NHQ"}="H132 NHQ"; $resn_exch{"3H13 NHQ"}="H133 NHQ";
$resn_exch{"1H14 NHQ"}="H141 NHQ"; $resn_exch{"2H14 NHQ"}="H142 NHQ"; $resn_exch{"3H14 NHQ"}="H143 NHQ";
$resn_exch{"1H12 NHQ"}="H121 NHQ"; $resn_exch{"2H12 NHQ"}="H122 NHQ"; $resn_exch{"1H5X NHQ"}="H5X1 NHQ";
$resn_exch{"2H5X NHQ"}="H5X2 NHQ"; $resn_exch{"1HNA NHR"}="HNA1 NHR"; $resn_exch{"2HNA NHR"}="HNA2 NHR";
$resn_exch{" OH4 NHR"}=" H4O NHR"; $resn_exch{"1H9  NHR"}=" H91 NHR"; $resn_exch{"2H9  NHR"}=" H92 NHR";
$resn_exch{"1H10 NHR"}="H101 NHR"; $resn_exch{"1H16 NHR"}="H161 NHR"; $resn_exch{"1H12 NHR"}="H121 NHR";
$resn_exch{"1H13 NHR"}="H131 NHR"; $resn_exch{"1H15 NHR"}="H151 NHR"; $resn_exch{"1HB  NHR"}=" HB1 NHR";
$resn_exch{"2HB  NHR"}=" HB2 NHR"; $resn_exch{"1HG  NHR"}=" HG1 NHR"; $resn_exch{"2HG  NHR"}=" HG2 NHR";
$resn_exch{"1HOE NHR"}="HOE1 NHR"; $resn_exch{"1HOA NHR"}="HOA1 NHR"; $resn_exch{"1HN2 NHS"}="HN21 NHS";
$resn_exch{"2HN2 NHS"}="HN22 NHS"; $resn_exch{"1H9  NHS"}=" H91 NHS"; $resn_exch{"2H9  NHS"}=" H92 NHS";
$resn_exch{"1HB  NHS"}=" HB1 NHS"; $resn_exch{"2HB  NHS"}=" HB2 NHS"; $resn_exch{"1HG  NHS"}=" HG1 NHS";
$resn_exch{"2HG  NHS"}=" HG2 NHS"; $resn_exch{"2HOE NHS"}="HOE2 NHS"; $resn_exch{"1H6B NHV"}="H6B1 NHV";
$resn_exch{"2H6B NHV"}="H6B2 NHV"; $resn_exch{"1H7C NHV"}="H7C1 NHV"; $resn_exch{"2H7C NHV"}="H7C2 NHV";
$resn_exch{"1HN2 NHY"}="HN21 NHY"; $resn_exch{"2HN2 NHY"}="HN22 NHY"; $resn_exch{"1HO1 NI1"}="HO11 NI1";
$resn_exch{"2HO1 NI1"}="HO12 NI1"; $resn_exch{"1HO1 NI2"}="HO11 NI2"; $resn_exch{"2HO1 NI2"}="HO12 NI2";
$resn_exch{"3HO1 NI2"}="HO13 NI2"; $resn_exch{"4HO1 NI2"}="HO14 NI2"; $resn_exch{"1HO1 NI3"}="HO11 NI3";
$resn_exch{"2HO1 NI3"}="HO12 NI3"; $resn_exch{"1HO2 NI3"}="HO21 NI3"; $resn_exch{"2HO2 NI3"}="HO22 NI3";
$resn_exch{"1HO3 NI3"}="HO31 NI3"; $resn_exch{"2HO3 NI3"}="HO32 NI3"; $resn_exch{"1H2  NI9"}=" H21 NI9";
$resn_exch{"2H2  NI9"}=" H22 NI9"; $resn_exch{"1H5' NIA"}="H5'1 NIA"; $resn_exch{"2H5' NIA"}="H5'2 NIA";
$resn_exch{"1HN6 NIA"}="HN61 NIA"; $resn_exch{"2HN6 NIA"}="HN62 NIA"; $resn_exch{"1H4  NIC"}=" H41 NIC";
$resn_exch{"2H4  NIC"}=" H42 NIC"; $resn_exch{"1HB  NIG"}=" HB1 NIG"; $resn_exch{"2HB  NIG"}=" HB2 NIG";
$resn_exch{"1HG  NIG"}=" HG1 NIG"; $resn_exch{"2HG  NIG"}=" HG2 NIG"; $resn_exch{"1H1  NIK"}=" H11 NIK";
$resn_exch{"2H1  NIK"}=" H12 NIK"; $resn_exch{"1H2  NIK"}=" H21 NIK"; $resn_exch{"2H2  NIK"}=" H22 NIK";
$resn_exch{"1H3  NIK"}=" H31 NIK"; $resn_exch{"2H3  NIK"}=" H32 NIK"; $resn_exch{"1H4  NIK"}=" H41 NIK";
$resn_exch{"2H4  NIK"}=" H42 NIK"; $resn_exch{"1H5  NIK"}=" H51 NIK"; $resn_exch{"2H5  NIK"}=" H52 NIK";
$resn_exch{"1H1  NIM"}="H11A NIM"; $resn_exch{"2H1  NIM"}="H12A NIM"; $resn_exch{"3H1  NIM"}="H13A NIM";
$resn_exch{"1H7  NIP"}=" H71 NIP"; $resn_exch{"2H7  NIP"}=" H72 NIP"; $resn_exch{"1H10 NIP"}="H101 NIP";
$resn_exch{"2H10 NIP"}="H102 NIP"; $resn_exch{"1H11 NIP"}="H111 NIP"; $resn_exch{"2H11 NIP"}="H112 NIP";
$resn_exch{"1H12 NIP"}="H121 NIP"; $resn_exch{"2H12 NIP"}="H122 NIP"; $resn_exch{"1H13 NIP"}="H131 NIP";
$resn_exch{"2H13 NIP"}="H132 NIP"; $resn_exch{"1H14 NIP"}="H141 NIP"; $resn_exch{"2H14 NIP"}="H142 NIP";
$resn_exch{"1H1N NIR"}="H1N1 NIR"; $resn_exch{"2H1N NIR"}="H1N2 NIR"; $resn_exch{"1H4  NIR"}=" H41 NIR";
$resn_exch{"5H5  NIR"}=" H55 NIR"; $resn_exch{"1H8  NIR"}=" H81 NIR"; $resn_exch{" H1* NIR"}=" H1' NIR";
$resn_exch{" H2* NIR"}=" H2' NIR"; $resn_exch{" H3* NIR"}=" H3' NIR"; $resn_exch{" H4* NIR"}=" H4' NIR";
$resn_exch{"1H5* NIR"}="H5'1 NIR"; $resn_exch{"2H5* NIR"}="H5'2 NIR"; $resn_exch{"3H5* NIR"}="H5'3 NIR";
$resn_exch{"1H1  NIS"}=" H11 NIS"; $resn_exch{"2H1  NIS"}=" H12 NIS"; $resn_exch{"3H1  NIS"}=" H13 NIS";
$resn_exch{"1H4  NIS"}=" H41 NIS"; $resn_exch{"2H4  NIS"}=" H42 NIS"; $resn_exch{"3H4  NIS"}=" H43 NIS";
$resn_exch{"1HN1 NIT"}="HN11 NIT"; $resn_exch{"2HN1 NIT"}="HN12 NIT"; $resn_exch{"1HG2 NIU"}="HG21 NIU";
$resn_exch{"2HG2 NIU"}="HG22 NIU"; $resn_exch{"1HG3 NIU"}="HG31 NIU"; $resn_exch{"2HG3 NIU"}="HG32 NIU";
$resn_exch{"3HG3 NIU"}="HG33 NIU"; $resn_exch{"1HD1 NIU"}="HD11 NIU"; $resn_exch{"2HD1 NIU"}="HD12 NIU";
$resn_exch{"1HD2 NIU"}="HD21 NIU"; $resn_exch{"2HD2 NIU"}="HD22 NIU"; $resn_exch{"1HD3 NIU"}="HD31 NIU";
$resn_exch{"2HD3 NIU"}="HD32 NIU"; $resn_exch{"1HD4 NIU"}="HD41 NIU"; $resn_exch{"2HD4 NIU"}="HD42 NIU";
$resn_exch{"1HD5 NIU"}="HD51 NIU"; $resn_exch{"2HD5 NIU"}="HD52 NIU"; $resn_exch{"1H12 NIV"}="H121 NIV";
$resn_exch{"2H12 NIV"}="H122 NIV"; $resn_exch{"3H12 NIV"}="H123 NIV"; $resn_exch{"1H14 NIV"}="H141 NIV";
$resn_exch{"2H14 NIV"}="H142 NIV"; $resn_exch{"1H15 NIV"}="H151 NIV"; $resn_exch{"2H15 NIV"}="H152 NIV";
$resn_exch{"1HE1 NIX"}="HE11 NIX"; $resn_exch{"1HE2 NIX"}="HE21 NIX"; $resn_exch{"2HE1 NIX"}="HE12 NIX";
$resn_exch{"2HE2 NIX"}="HE22 NIX"; $resn_exch{"2HE3 NIX"}="HE32 NIX"; $resn_exch{"1H2L NIX"}="H2L1 NIX";
$resn_exch{" HN2 NIY"}=" H2  NIY"; $resn_exch{"1HB  NIY"}=" HB2 NIY"; $resn_exch{"2HB  NIY"}=" HB3 NIY";
$resn_exch{"1H4  NK1"}=" H41 NK1"; $resn_exch{"2H4  NK1"}=" H42 NK1"; $resn_exch{"1H3  NK1"}=" H31 NK1";
$resn_exch{"2H3  NK1"}=" H32 NK1"; $resn_exch{"1H2  NK1"}=" H21 NK1"; $resn_exch{"2H2  NK1"}=" H22 NK1";
$resn_exch{"1H9  NK1"}=" H91 NK1"; $resn_exch{"2H9  NK1"}=" H92 NK1"; $resn_exch{"1H5  NK2"}=" H51 NK2";
$resn_exch{"2H5  NK2"}=" H52 NK2"; $resn_exch{"1H4  NK2"}=" H41 NK2"; $resn_exch{"2H4  NK2"}=" H42 NK2";
$resn_exch{"1H3  NK2"}=" H31 NK2"; $resn_exch{"2H3  NK2"}=" H32 NK2"; $resn_exch{"1H2  NK2"}=" H21 NK2";
$resn_exch{"2H2  NK2"}=" H22 NK2"; $resn_exch{"1H9  NK2"}=" H91 NK2"; $resn_exch{"2H9  NK2"}=" H92 NK2";
$resn_exch{"1H3  NLA"}=" H31 NLA"; $resn_exch{"1H4  NLA"}=" H41 NLA"; $resn_exch{"1H5  NLA"}=" H51 NLA";
$resn_exch{"1H7  NLA"}=" H71 NLA"; $resn_exch{"1H8  NLA"}=" H81 NLA"; $resn_exch{"1H9  NLA"}=" H91 NLA";
$resn_exch{"1H10 NLA"}="H101 NLA"; $resn_exch{"1H11 NLA"}="H111 NLA"; $resn_exch{"2H11 NLA"}="H112 NLA";
$resn_exch{"1H8C NLC"}="H8C1 NLC"; $resn_exch{"2H8C NLC"}="H8C2 NLC"; $resn_exch{"3H8C NLC"}="H8C3 NLC";
$resn_exch{"1H6C NLC"}="H6C1 NLC"; $resn_exch{"2H6C NLC"}="H6C2 NLC"; $resn_exch{"1H16 NLC"}="H161 NLC";
$resn_exch{"2H16 NLC"}="H162 NLC"; $resn_exch{"1HN  NLE"}=" H   NLE"; $resn_exch{"2HN  NLE"}=" HN2 NLE";
$resn_exch{"1HB  NLE"}=" HB2 NLE"; $resn_exch{"2HB  NLE"}=" HB3 NLE"; $resn_exch{"1HG  NLE"}=" HG2 NLE";
$resn_exch{"2HG  NLE"}=" HG3 NLE"; $resn_exch{"1HD  NLE"}=" HD2 NLE"; $resn_exch{"2HD  NLE"}=" HD3 NLE";
$resn_exch{"1HE  NLE"}=" HE1 NLE"; $resn_exch{"2HE  NLE"}=" HE2 NLE"; $resn_exch{"3HE  NLE"}=" HE3 NLE";
$resn_exch{"1HBC NLG"}="HBC1 NLG"; $resn_exch{"2HBC NLG"}="HBC2 NLG"; $resn_exch{"1HGC NLG"}="HGC1 NLG";
$resn_exch{"2HGC NLG"}="HGC2 NLG"; $resn_exch{"1H8C NLG"}="H8C1 NLG"; $resn_exch{"2H8C NLG"}="H8C2 NLG";
$resn_exch{"3H8C NLG"}="H8C3 NLG"; $resn_exch{"1HN  NLN"}=" H   NLN"; $resn_exch{"2HN  NLN"}=" H2  NLN";
$resn_exch{"1HB  NLN"}=" HB2 NLN"; $resn_exch{"2HB  NLN"}=" HB3 NLN"; $resn_exch{"1HG  NLN"}=" HG2 NLN";
$resn_exch{"2HG  NLN"}=" HG3 NLN"; $resn_exch{"1HD  NLN"}=" HD2 NLN"; $resn_exch{"2HD  NLN"}=" HD3 NLN";
$resn_exch{"1HE  NLN"}=" HE1 NLN"; $resn_exch{"2HE  NLN"}=" HE2 NLN"; $resn_exch{"3HE  NLN"}=" HE3 NLN";
$resn_exch{"1HH2 NLN"}="HH21 NLN"; $resn_exch{"2HH2 NLN"}="HH22 NLN"; $resn_exch{"1HN  NLO"}=" H   NLO";
$resn_exch{"2HN  NLO"}=" H2  NLO"; $resn_exch{"1HB  NLO"}=" HB2 NLO"; $resn_exch{"2HB  NLO"}=" HB3 NLO";
$resn_exch{"1HG  NLO"}=" HG2 NLO"; $resn_exch{"2HG  NLO"}=" HG3 NLO"; $resn_exch{"1HD  NLO"}=" HD2 NLO";
$resn_exch{"2HD  NLO"}=" HD3 NLO"; $resn_exch{"1HE  NLO"}=" HE1 NLO"; $resn_exch{"2HE  NLO"}=" HE2 NLO";
$resn_exch{"3HE  NLO"}=" HE3 NLO"; $resn_exch{"1HN  NLP"}=" HN1 NLP"; $resn_exch{"2HN  NLP"}=" HN2 NLP";
$resn_exch{"1HB  NLP"}=" HB1 NLP"; $resn_exch{"2HB  NLP"}=" HB2 NLP"; $resn_exch{"1HG  NLP"}=" HG1 NLP";
$resn_exch{"2HG  NLP"}=" HG2 NLP"; $resn_exch{"1HD  NLP"}=" HD1 NLP"; $resn_exch{"2HD  NLP"}=" HD2 NLP";
$resn_exch{"1HE  NLP"}=" HE1 NLP"; $resn_exch{"2HE  NLP"}=" HE2 NLP"; $resn_exch{"3HE  NLP"}=" HE3 NLP";
$resn_exch{" HN  NLQ"}=" H   NLQ"; $resn_exch{"1H7  NLQ"}=" H71 NLQ"; $resn_exch{"2H7  NLQ"}=" H72 NLQ";
$resn_exch{"3H7  NLQ"}=" H73 NLQ"; $resn_exch{"1HB  NLQ"}=" HB2 NLQ"; $resn_exch{"2HB  NLQ"}=" HB3 NLQ";
$resn_exch{"1HG  NLQ"}=" HG2 NLQ"; $resn_exch{"2HG  NLQ"}=" HG3 NLQ"; $resn_exch{"1HE2 NLQ"}="HE21 NLQ";
$resn_exch{"2HE2 NLQ"}="HE22 NLQ"; $resn_exch{"1H2  NLT"}=" H21 NLT"; $resn_exch{"2H2  NLT"}=" H22 NLT";
$resn_exch{"1H3  NLT"}=" H31 NLT"; $resn_exch{"2H3  NLT"}=" H32 NLT"; $resn_exch{"1H4  NLT"}=" H41 NLT";
$resn_exch{"2H4  NLT"}=" H42 NLT"; $resn_exch{"1H5  NLT"}=" H51 NLT"; $resn_exch{"2H5  NLT"}=" H52 NLT";
$resn_exch{"1H6  NLT"}=" H61 NLT"; $resn_exch{"2H6  NLT"}=" H62 NLT"; $resn_exch{"1H7  NLT"}=" H71 NLT";
$resn_exch{"2H7  NLT"}=" H72 NLT"; $resn_exch{"1H8  NLT"}=" H81 NLT"; $resn_exch{"2H8  NLT"}=" H82 NLT";
$resn_exch{"1H9  NLT"}=" H91 NLT"; $resn_exch{"2H9  NLT"}=" H92 NLT"; $resn_exch{"1H10 NLT"}="H101 NLT";
$resn_exch{"2H10 NLT"}="H102 NLT"; $resn_exch{"1H11 NLT"}="H111 NLT"; $resn_exch{"2H11 NLT"}="H112 NLT";
$resn_exch{"1H12 NLT"}="H121 NLT"; $resn_exch{"2H12 NLT"}="H122 NLT"; $resn_exch{"3H12 NLT"}="H123 NLT";
$resn_exch{"1HB  NLT"}=" HB1 NLT"; $resn_exch{"2HB  NLT"}=" HB2 NLT"; $resn_exch{"1HO1 NLX"}="HO11 NLX";
$resn_exch{"1HO4 NLX"}="HO41 NLX"; $resn_exch{"1H1  NLX"}=" H11 NLX"; $resn_exch{"1H2  NLX"}=" H21 NLX";
$resn_exch{"1H5  NLX"}=" H51 NLX"; $resn_exch{"1H7  NLX"}=" H71 NLX"; $resn_exch{"2H7  NLX"}=" H72 NLX";
$resn_exch{"1H8  NLX"}=" H81 NLX"; $resn_exch{"2H8  NLX"}=" H82 NLX"; $resn_exch{"1H9  NLX"}=" H91 NLX";
$resn_exch{"1H10 NLX"}="H101 NLX"; $resn_exch{"2H10 NLX"}="H102 NLX"; $resn_exch{"1H15 NLX"}="H151 NLX";
$resn_exch{"2H15 NLX"}="H152 NLX"; $resn_exch{"1H16 NLX"}="H161 NLX"; $resn_exch{"2H16 NLX"}="H162 NLX";
$resn_exch{"1H17 NLX"}="H171 NLX"; $resn_exch{"2H17 NLX"}="H172 NLX"; $resn_exch{"1H18 NLX"}="H181 NLX";
$resn_exch{"1H19 NLX"}="H191 NLX"; $resn_exch{"2H19 NLX"}="H192 NLX"; $resn_exch{"1H20 NLX"}="H201 NLX";
$resn_exch{"2H20 NLX"}="H202 NLX"; $resn_exch{"3H20 NLX"}="H203 NLX"; $resn_exch{"1H08 NM1"}="H081 NM1";
$resn_exch{"2H08 NM1"}="H082 NM1"; $resn_exch{"1H23 NM1"}="H231 NM1"; $resn_exch{"2H23 NM1"}="H232 NM1";
$resn_exch{"1H32 NM1"}="H321 NM1"; $resn_exch{"2H32 NM1"}="H322 NM1"; $resn_exch{"1H41 NM1"}="H411 NM1";
$resn_exch{"2H41 NM1"}="H412 NM1"; $resn_exch{"1H54 NM1"}="H541 NM1"; $resn_exch{"2H54 NM1"}="H542 NM1";
$resn_exch{"3H54 NM1"}="H543 NM1"; $resn_exch{"1H4A NMA"}="H4A1 NMA"; $resn_exch{"2H4A NMA"}="H4A2 NMA";
$resn_exch{"2HOP NMA"}="HOP2 NMA"; $resn_exch{"3HOP NMA"}="HOP3 NMA"; $resn_exch{"1H5A NMA"}="H5A1 NMA";
$resn_exch{"2H5A NMA"}="H5A2 NMA"; $resn_exch{"1H2A NMA"}="H2A1 NMA"; $resn_exch{"2H2A NMA"}="H2A2 NMA";
$resn_exch{"3H2A NMA"}="H2A3 NMA"; $resn_exch{"1H9  NMA"}=" H91 NMA"; $resn_exch{"2H9  NMA"}=" H92 NMA";
$resn_exch{"3H9  NMA"}=" H93 NMA"; $resn_exch{"1H10 NMA"}="H101 NMA"; $resn_exch{"2H10 NMA"}="H102 NMA";
$resn_exch{"3H10 NMA"}="H103 NMA"; $resn_exch{"1H08 NMB"}="H081 NMB"; $resn_exch{"2H08 NMB"}="H082 NMB";
$resn_exch{"1H23 NMB"}="H231 NMB"; $resn_exch{"2H23 NMB"}="H232 NMB"; $resn_exch{"1H32 NMB"}="H321 NMB";
$resn_exch{"2H32 NMB"}="H322 NMB"; $resn_exch{"1H41 NMB"}="H411 NMB"; $resn_exch{"2H41 NMB"}="H412 NMB";
$resn_exch{"1HA  NMC"}=" HA1 NMC"; $resn_exch{"2HA  NMC"}=" HA2 NMC"; $resn_exch{"1HCN NMC"}="HCN1 NMC";
$resn_exch{"2HCN NMC"}="HCN2 NMC"; $resn_exch{"1HC2 NMC"}="HC21 NMC"; $resn_exch{"2HC2 NMC"}="HC22 NMC";
$resn_exch{"1HC3 NMC"}="HC31 NMC"; $resn_exch{"2HC3 NMC"}="HC32 NMC"; $resn_exch{"1H5* NMD"}="H5'1 NMD";
$resn_exch{"2H5* NMD"}="H5'2 NMD"; $resn_exch{" H4* NMD"}=" H4' NMD"; $resn_exch{" H3* NMD"}=" H3' NMD";
$resn_exch{"*HO3 NMD"}="HO3' NMD"; $resn_exch{" H2* NMD"}=" H2' NMD"; $resn_exch{"*HN2 NMD"}="HN2' NMD";
$resn_exch{" H1* NMD"}=" H1' NMD"; $resn_exch{"1H11 NMD"}="H111 NMD"; $resn_exch{"2H11 NMD"}="H112 NMD";
$resn_exch{"1H2M NMD"}="H2M1 NMD"; $resn_exch{"2H2M NMD"}="H2M2 NMD"; $resn_exch{"3H2M NMD"}="H2M3 NMD";
$resn_exch{"1H1M NMD"}="H1M1 NMD"; $resn_exch{"2H1M NMD"}="H1M2 NMD"; $resn_exch{"3H1M NMD"}="H1M3 NMD";
$resn_exch{"*HO5 NMD"}="HO5' NMD"; $resn_exch{"1HN  NME"}=" HN1 NME"; $resn_exch{"2HN  NME"}=" HN2 NME";
$resn_exch{"1H   NME"}=" H1  NME"; $resn_exch{"2H   NME"}=" H2  NME"; $resn_exch{"3H   NME"}=" H3  NME";
$resn_exch{"1HD  NMG"}=" HD1 NMG"; $resn_exch{"2HD  NMG"}=" HD2 NMG"; $resn_exch{"1HH1 NMG"}="HH11 NMG";
$resn_exch{"2HH1 NMG"}="HH12 NMG"; $resn_exch{"1HC7 NMH"}="HC71 NMH"; $resn_exch{"2HC7 NMH"}="HC72 NMH";
$resn_exch{"3HC7 NMH"}="HC73 NMH"; $resn_exch{"1HC6 NMH"}="HC61 NMH"; $resn_exch{"2HC6 NMH"}="HC62 NMH";
$resn_exch{"1HC5 NMH"}="HC51 NMH"; $resn_exch{"2HC5 NMH"}="HC52 NMH"; $resn_exch{"1HC4 NMH"}="HC41 NMH";
$resn_exch{"2HC4 NMH"}="HC42 NMH"; $resn_exch{"1HC3 NMH"}="HC31 NMH"; $resn_exch{"2HC3 NMH"}="HC32 NMH";
$resn_exch{"1HC1 NMH"}="HC11 NMH"; $resn_exch{"2HC1 NMH"}="HC12 NMH"; $resn_exch{"3HC1 NMH"}="HC13 NMH";
$resn_exch{" HN* NMH"}=" HN' NMH"; $resn_exch{" HC* NMH"}=" HC' NMH"; $resn_exch{"1H1  NML"}=" H11 NML";
$resn_exch{"2H1  NML"}=" H12 NML"; $resn_exch{"3H1  NML"}=" H13 NML"; $resn_exch{"1H3  NML"}=" H31 NML";
$resn_exch{"2H3  NML"}=" H32 NML"; $resn_exch{"3H3  NML"}=" H33 NML"; $resn_exch{"1HAA NMM"}="HAA1 NMM";
$resn_exch{"2HAA NMM"}="HAA2 NMM"; $resn_exch{"3HAA NMM"}="HAA3 NMM"; $resn_exch{" HNE NMM"}=" HE  NMM";
$resn_exch{"1HD  NMM"}=" HD3 NMM"; $resn_exch{"2HD  NMM"}=" HD2 NMM"; $resn_exch{"1HG  NMM"}=" HG3 NMM";
$resn_exch{"2HG  NMM"}=" HG2 NMM"; $resn_exch{"1HB  NMM"}=" HB3 NMM"; $resn_exch{"2HB  NMM"}=" HB2 NMM";
$resn_exch{"1HN  NMM"}=" H   NMM"; $resn_exch{"2HN  NMM"}=" H2  NMM"; $resn_exch{"OH1P NMN"}="H1PO NMN";
$resn_exch{"OH2P NMN"}="H2PO NMN"; $resn_exch{"1H5R NMN"}="H5R1 NMN"; $resn_exch{"2H5R NMN"}="H5R2 NMN";
$resn_exch{"CH4R NMN"}="H4RC NMN"; $resn_exch{"CH3R NMN"}="H3RC NMN"; $resn_exch{"OH3R NMN"}="H3RO NMN";
$resn_exch{"CH2R NMN"}="H2RC NMN"; $resn_exch{"OH2R NMN"}="H2RO NMN"; $resn_exch{"CH1R NMN"}="H1RC NMN";
$resn_exch{"1HN7 NMN"}="HN71 NMN"; $resn_exch{"2HN7 NMN"}="HN72 NMN"; $resn_exch{"1H26 NMP"}="H261 NMP";
$resn_exch{"2H26 NMP"}="H262 NMP"; $resn_exch{"1H25 NMP"}="H251 NMP"; $resn_exch{"2H25 NMP"}="H252 NMP";
$resn_exch{"1H24 NMP"}="H241 NMP"; $resn_exch{"2H24 NMP"}="H242 NMP"; $resn_exch{"1H22 NMP"}="H221 NMP";
$resn_exch{"2H22 NMP"}="H222 NMP"; $resn_exch{"3H22 NMP"}="H223 NMP"; $resn_exch{"1H15 NMP"}="H151 NMP";
$resn_exch{"2H15 NMP"}="H152 NMP"; $resn_exch{"1H3  NMP"}=" H31 NMP"; $resn_exch{"2H3  NMP"}=" H32 NMP";
$resn_exch{"1H9  NMP"}=" H91 NMP"; $resn_exch{"2H9  NMP"}=" H92 NMP"; $resn_exch{"1H10 NMP"}="H101 NMP";
$resn_exch{"2H10 NMP"}="H102 NMP"; $resn_exch{"1H11 NMP"}="H111 NMP"; $resn_exch{"2H11 NMP"}="H112 NMP";
$resn_exch{"1H12 NMP"}="H121 NMP"; $resn_exch{"2H12 NMP"}="H122 NMP"; $resn_exch{"1H13 NMP"}="H131 NMP";
$resn_exch{"2H13 NMP"}="H132 NMP"; $resn_exch{"2HOB NMQ"}="HOB2 NMQ"; $resn_exch{"2HOA NMQ"}="HOA2 NMQ";
$resn_exch{"1HA2 NMQ"}="HA21 NMQ"; $resn_exch{"2HA2 NMQ"}="HA22 NMQ"; $resn_exch{"1HA1 NMQ"}="HA11 NMQ";
$resn_exch{"2HA1 NMQ"}="HA12 NMQ"; $resn_exch{"1HA3 NMQ"}="HA31 NMQ"; $resn_exch{"2HA3 NMQ"}="HA32 NMQ";
$resn_exch{"3HA3 NMQ"}="HA33 NMQ"; $resn_exch{"2HOP NMS"}="HOP2 NMS"; $resn_exch{"1H5M NMS"}=" H71 NMS";
$resn_exch{"2H5M NMS"}=" H72 NMS"; $resn_exch{"3H5M NMS"}=" H73 NMS"; $resn_exch{" H2* NMS"}=" H2' NMS";
$resn_exch{"1H5* NMS"}=" H5' NMS"; $resn_exch{"2H5* NMS"}="H5'' NMS"; $resn_exch{" H4* NMS"}=" H4' NMS";
$resn_exch{" H1* NMS"}=" H1' NMS"; $resn_exch{" H3* NMS"}=" H3' NMS"; $resn_exch{"H3T  NMS"}="HO3' NMS";
$resn_exch{"1H7' NMS"}="H7'1 NMS"; $resn_exch{"2H7' NMS"}="H7'2 NMS"; $resn_exch{"1H10 NMS"}="H101 NMS";
$resn_exch{"2H10 NMS"}="H102 NMS"; $resn_exch{"3H10 NMS"}="H103 NMS"; $resn_exch{"3HOP NMS"}="HOP3 NMS";
$resn_exch{"3HOP NMT"}="HOP3 NMT"; $resn_exch{"2HOP NMT"}="HOP2 NMT"; $resn_exch{"1H5M NMT"}=" H71 NMT";
$resn_exch{"2H5M NMT"}=" H72 NMT"; $resn_exch{"3H5M NMT"}=" H73 NMT"; $resn_exch{" H2* NMT"}=" H2' NMT";
$resn_exch{"1H5* NMT"}=" H5' NMT"; $resn_exch{"2H5* NMT"}="H5'' NMT"; $resn_exch{" H4* NMT"}=" H4' NMT";
$resn_exch{" H1* NMT"}=" H1' NMT"; $resn_exch{" H3* NMT"}=" H3' NMT"; $resn_exch{"H3T  NMT"}="HO3' NMT";
$resn_exch{"1H9' NMT"}="H9'1 NMT"; $resn_exch{"2H9' NMT"}="H9'2 NMT"; $resn_exch{"3H9' NMT"}="H9'3 NMT";
$resn_exch{"1HN3 NMU"}="HN31 NMU"; $resn_exch{"2HN3 NMU"}="HN32 NMU"; $resn_exch{"1H5  NMU"}=" H51 NMU";
$resn_exch{"2H5  NMU"}=" H52 NMU"; $resn_exch{"3H5  NMU"}=" H53 NMU"; $resn_exch{" AH2 NMX"}=" H2A NMX";
$resn_exch{"AH61 NMX"}="H61A NMX"; $resn_exch{"AH62 NMX"}="H62A NMX"; $resn_exch{" AH8 NMX"}=" H8A NMX";
$resn_exch{"AH1* NMX"}=" H1B NMX"; $resn_exch{"AH2* NMX"}=" H2B NMX"; $resn_exch{"AHO2 NMX"}="HO2A NMX";
$resn_exch{"AH3* NMX"}=" H3B NMX"; $resn_exch{"8HOA NMX"}="HOA8 NMX"; $resn_exch{"9HOA NMX"}="HOA9 NMX";
$resn_exch{"AH4* NMX"}=" H4B NMX"; $resn_exch{"AH51 NMX"}="H51A NMX"; $resn_exch{"AH52 NMX"}="H52A NMX";
$resn_exch{"2HOA NMX"}="HOA2 NMX"; $resn_exch{"5HOA NMX"}="HOA5 NMX"; $resn_exch{"1H12 NMX"}="H121 NMX";
$resn_exch{"2H12 NMX"}="H122 NMX"; $resn_exch{"1H13 NMX"}="H131 NMX"; $resn_exch{"2H13 NMX"}="H132 NMX";
$resn_exch{"3H13 NMX"}="H133 NMX"; $resn_exch{"1H14 NMX"}="H141 NMX"; $resn_exch{"2H14 NMX"}="H142 NMX";
$resn_exch{"3H14 NMX"}="H143 NMX"; $resn_exch{"1H7  NMX"}=" H71 NMX"; $resn_exch{"2H7  NMX"}=" H72 NMX";
$resn_exch{"1H6  NMX"}=" H61 NMX"; $resn_exch{"2H6  NMX"}=" H62 NMX"; $resn_exch{"1H3  NMX"}=" H31 NMX";
$resn_exch{"2H3  NMX"}=" H32 NMX"; $resn_exch{"1H2  NMX"}=" H21 NMX"; $resn_exch{"2H2  NMX"}=" H22 NMX";
$resn_exch{"1HI1 NMX"}="HI11 NMX"; $resn_exch{"2HI1 NMX"}="HI12 NMX"; $resn_exch{"1HN2 NMY"}="HN21 NMY";
$resn_exch{"2HN2 NMY"}="HN22 NMY"; $resn_exch{"1H6  NMY"}=" H61 NMY"; $resn_exch{"2H6  NMY"}=" H62 NMY";
$resn_exch{"1H7  NMY"}=" H71 NMY"; $resn_exch{"2H7  NMY"}=" H72 NMY"; $resn_exch{"1H9  NMY"}=" H91 NMY";
$resn_exch{"2H9  NMY"}=" H92 NMY"; $resn_exch{"1H8  NMY"}=" H81 NMY"; $resn_exch{"2H8  NMY"}=" H82 NMY";
$resn_exch{"1H17 NMY"}="H171 NMY"; $resn_exch{"2H17 NMY"}="H172 NMY"; $resn_exch{"1H23 NMY"}="H231 NMY";
$resn_exch{"2H23 NMY"}="H232 NMY"; $resn_exch{"1HN6 NMY"}="HN61 NMY"; $resn_exch{"2HN6 NMY"}="HN62 NMY";
$resn_exch{"1HO4 NMY"}="HO41 NMY"; $resn_exch{"2HO2 NMY"}="HO22 NMY"; $resn_exch{"1HO2 NMY"}="HO21 NMY";
$resn_exch{"2HO1 NMY"}="HO12 NMY"; $resn_exch{"1H19 NMY"}="H191 NMY"; $resn_exch{"2H19 NMY"}="H192 NMY";
$resn_exch{"1HN3 NMY"}="HN31 NMY"; $resn_exch{"2HN3 NMY"}="HN32 NMY"; $resn_exch{"1H5  NN1"}=" H51 NN1";
$resn_exch{"2H5  NN1"}=" H52 NN1"; $resn_exch{"3H5  NN1"}=" H53 NN1"; $resn_exch{"1H17 NN1"}="H171 NN1";
$resn_exch{"2H17 NN1"}="H172 NN1"; $resn_exch{"1H21 NN1"}="H211 NN1"; $resn_exch{"2H21 NN1"}="H212 NN1";
$resn_exch{"1H24 NN1"}="H241 NN1"; $resn_exch{"2H24 NN1"}="H242 NN1"; $resn_exch{"1H1  NN1"}=" H11 NN1";
$resn_exch{"2H1  NN1"}=" H12 NN1"; $resn_exch{"3H1  NN1"}=" H13 NN1"; $resn_exch{"1H3  NN1"}=" H31 NN1";
$resn_exch{"2H3  NN1"}=" H32 NN1"; $resn_exch{"3H3  NN1"}=" H33 NN1"; $resn_exch{"1H19 NN1"}="H191 NN1";
$resn_exch{"2H19 NN1"}="H192 NN1"; $resn_exch{"1H23 NN1"}="H231 NN1"; $resn_exch{"2H23 NN1"}="H232 NN1";
$resn_exch{"1H24 NN2"}="H241 NN2"; $resn_exch{"2H24 NN2"}="H242 NN2"; $resn_exch{"1H25 NN2"}="H251 NN2";
$resn_exch{"2H25 NN2"}="H252 NN2"; $resn_exch{"1H30 NN2"}="H301 NN2"; $resn_exch{"2H30 NN2"}="H302 NN2";
$resn_exch{"1H29 NN2"}="H291 NN2"; $resn_exch{"2H29 NN2"}="H292 NN2"; $resn_exch{"1H5  NN3"}=" H51 NN3";
$resn_exch{"2H5  NN3"}=" H52 NN3"; $resn_exch{"1H6  NN3"}=" H61 NN3"; $resn_exch{"2H6  NN3"}=" H62 NN3";
$resn_exch{"1H7  NN3"}=" H71 NN3"; $resn_exch{"2H7  NN3"}=" H72 NN3"; $resn_exch{"3H7  NN3"}=" H73 NN3";
$resn_exch{"1H3  NN3"}=" H31 NN3"; $resn_exch{"2H3  NN3"}=" H32 NN3"; $resn_exch{"1H2  NN3"}="H21A NN3";
$resn_exch{"2H2  NN3"}="H22A NN3"; $resn_exch{"1H24 NN3"}="H241 NN3"; $resn_exch{"2H24 NN3"}="H242 NN3";
$resn_exch{"3H24 NN3"}="H243 NN3"; $resn_exch{"1H23 NN3"}="H231 NN3"; $resn_exch{"2H23 NN3"}="H232 NN3";
$resn_exch{"3H23 NN3"}="H233 NN3"; $resn_exch{"1H4  NN4"}=" H41 NN4"; $resn_exch{"2H4  NN4"}=" H42 NN4";
$resn_exch{"1H6  NN4"}=" H61 NN4"; $resn_exch{"2H6  NN4"}=" H62 NN4"; $resn_exch{"1H7  NN4"}=" H71 NN4";
$resn_exch{"2H7  NN4"}=" H72 NN4"; $resn_exch{"1H8  NN4"}=" H81 NN4"; $resn_exch{"2H8  NN4"}=" H82 NN4";
$resn_exch{"1H20 NN4"}="H201 NN4"; $resn_exch{"2H20 NN4"}="H202 NN4"; $resn_exch{"3H20 NN4"}="H203 NN4";
$resn_exch{"1H28 NN4"}="H281 NN4"; $resn_exch{"2H28 NN4"}="H282 NN4"; $resn_exch{"3H28 NN4"}="H283 NN4";
$resn_exch{"1H2  NN4"}=" H21 NN4"; $resn_exch{"2H2  NN4"}="H22A NN4"; $resn_exch{"1H13 NN4"}="H131 NN4";
$resn_exch{"2H13 NN4"}="H132 NN4"; $resn_exch{"1H19 NN4"}="H191 NN4"; $resn_exch{"2H19 NN4"}="H192 NN4";
$resn_exch{"3H19 NN4"}="H193 NN4"; $resn_exch{"1H10 NNB"}="H101 NNB"; $resn_exch{"2H10 NNB"}="H102 NNB";
$resn_exch{"1H9C NNB"}="H9C1 NNB"; $resn_exch{"2H9C NNB"}="H9C2 NNB"; $resn_exch{"1H10 NNC"}="H101 NNC";
$resn_exch{"2H10 NNC"}="H102 NNC"; $resn_exch{"1H9C NNC"}="H9C1 NNC"; $resn_exch{"2H9C NNC"}="H9C2 NNC";
$resn_exch{"1H6C NND"}="H6C1 NND"; $resn_exch{"2H6C NND"}="H6C2 NND"; $resn_exch{"1H1C NND"}="H1C1 NND";
$resn_exch{"2H1C NND"}="H1C2 NND"; $resn_exch{"1H8C NND"}="H8C1 NND"; $resn_exch{"2H8C NND"}="H8C2 NND";
$resn_exch{"1H7C NND"}="H7C1 NND"; $resn_exch{"2H7C NND"}="H7C2 NND"; $resn_exch{"1H9C NND"}="H9C1 NND";
$resn_exch{"2H9C NND"}="H9C2 NND"; $resn_exch{"1H10 NND"}="H101 NND"; $resn_exch{"2H10 NND"}="H102 NND";
$resn_exch{"1H14 NND"}="H141 NND"; $resn_exch{"2H14 NND"}="H142 NND"; $resn_exch{"1H13 NND"}="H131 NND";
$resn_exch{"2H13 NND"}="H132 NND"; $resn_exch{"1H12 NND"}="H121 NND"; $resn_exch{"2H12 NND"}="H122 NND";
$resn_exch{"1H11 NND"}="H111 NND"; $resn_exch{"2H11 NND"}="H112 NND"; $resn_exch{"1H16 NND"}="H161 NND";
$resn_exch{"2H16 NND"}="H162 NND"; $resn_exch{"3H16 NND"}="H163 NND"; $resn_exch{"1H6  NNG"}=" H61 NNG";
$resn_exch{"2H6  NNG"}=" H62 NNG"; $resn_exch{"1H8  NNG"}=" H81 NNG"; $resn_exch{"2H8  NNG"}=" H82 NNG";
$resn_exch{"3H8  NNG"}=" H83 NNG"; $resn_exch{" HN2 NNH"}=" H2  NNH"; $resn_exch{"1HB  NNH"}=" HB2 NNH";
$resn_exch{"2HB  NNH"}=" HB3 NNH"; $resn_exch{"1HG  NNH"}=" HG2 NNH"; $resn_exch{"2HG  NNH"}=" HG3 NNH";
$resn_exch{"1H10 NNI"}="H101 NNI"; $resn_exch{"2H10 NNI"}="H102 NNI"; $resn_exch{"1H9C NNI"}="H9C1 NNI";
$resn_exch{"2H9C NNI"}="H9C2 NNI"; $resn_exch{"1H5R NNR"}="H5R1 NNR"; $resn_exch{"2H5R NNR"}="H5R2 NNR";
$resn_exch{"1HN7 NNR"}="HN71 NNR"; $resn_exch{"2HN7 NNR"}="HN72 NNR"; $resn_exch{"1H1  NNS"}=" H11 NNS";
$resn_exch{"2H1  NNS"}=" H12 NNS"; $resn_exch{"3H1  NNS"}=" H13 NNS"; $resn_exch{"1HA  NOA"}=" HA1 NOA";
$resn_exch{"2HA  NOA"}=" HA2 NOA"; $resn_exch{"1H6* NOC"}="H6'1 NOC"; $resn_exch{"2H6* NOC"}="H6'2 NOC";
$resn_exch{"HO6* NOC"}="HO6' NOC"; $resn_exch{"1H5* NOC"}="H5'1 NOC"; $resn_exch{"2H5* NOC"}="H5'2 NOC";
$resn_exch{"HC4* NOC"}="HC4' NOC"; $resn_exch{"HC3* NOC"}="HC3' NOC"; $resn_exch{"HO3* NOC"}="HO3' NOC";
$resn_exch{"HC2* NOC"}="HC2' NOC"; $resn_exch{"HO2* NOC"}="HO2' NOC"; $resn_exch{"HC1* NOC"}="HC1' NOC";
$resn_exch{"1HN6 NOC"}="HN61 NOC"; $resn_exch{"2HN6 NOC"}="HN62 NOC"; $resn_exch{"1H11 NOD"}="H111 NOD";
$resn_exch{"2H11 NOD"}="H112 NOD"; $resn_exch{"2HO1 NOD"}="HO12 NOD"; $resn_exch{"1H14 NOD"}="H141 NOD";
$resn_exch{"2H14 NOD"}="H142 NOD"; $resn_exch{"4HO1 NOD"}="HO14 NOD"; $resn_exch{"1H15 NOD"}="H151 NOD";
$resn_exch{"2H15 NOD"}="H152 NOD"; $resn_exch{"7HO1 NOD"}="HO17 NOD"; $resn_exch{"1H21 NOD"}="H211 NOD";
$resn_exch{"2H21 NOD"}="H212 NOD"; $resn_exch{"3H21 NOD"}="H213 NOD"; $resn_exch{" H1* NOD"}=" H1' NOD";
$resn_exch{"1H2* NOD"}="H2'1 NOD"; $resn_exch{"2H2* NOD"}="H2'2 NOD"; $resn_exch{" H3* NOD"}=" H3' NOD";
$resn_exch{" H4* NOD"}=" H4' NOD"; $resn_exch{"*HO4 NOD"}="HO4' NOD"; $resn_exch{" H5* NOD"}=" H5' NOD";
$resn_exch{"1H6* NOD"}="H6'1 NOD"; $resn_exch{"2H6* NOD"}="H6'2 NOD"; $resn_exch{"3H6* NOD"}="H6'3 NOD";
$resn_exch{"HN3* NOD"}="HN3' NOD"; $resn_exch{"1H63 NOD"}="H631 NOD"; $resn_exch{"2H63 NOD"}="H632 NOD";
$resn_exch{"1H53 NOD"}="H531 NOD"; $resn_exch{"2H53 NOD"}="H532 NOD"; $resn_exch{"1HC1 NOE"}="HC11 NOE";
$resn_exch{"2HC1 NOE"}="HC12 NOE"; $resn_exch{"1HC2 NOE"}="HC21 NOE"; $resn_exch{"2HC2 NOE"}="HC22 NOE";
$resn_exch{"3HC2 NOE"}="HC23 NOE"; $resn_exch{"1HC1 NOG"}="HC11 NOG"; $resn_exch{"2HC1 NOG"}="HC12 NOG";
$resn_exch{"1HC2 NOG"}="HC21 NOG"; $resn_exch{"2HC2 NOG"}="HC22 NOG"; $resn_exch{"1HC6 NOG"}="HC61 NOG";
$resn_exch{"2HC6 NOG"}="HC62 NOG"; $resn_exch{"1HC7 NOG"}="HC71 NOG"; $resn_exch{"2HC7 NOG"}="HC72 NOG";
$resn_exch{"CH10 NOG"}="H10C NOG"; $resn_exch{"1H11 NOG"}="H111 NOG"; $resn_exch{"2H11 NOG"}="H112 NOG";
$resn_exch{"1H12 NOG"}="H121 NOG"; $resn_exch{"2H12 NOG"}="H122 NOG"; $resn_exch{"CH14 NOG"}="H14C NOG";
$resn_exch{"1H15 NOG"}="H151 NOG"; $resn_exch{"2H15 NOG"}="H152 NOG"; $resn_exch{"1H16 NOG"}="H161 NOG";
$resn_exch{"2H16 NOG"}="H162 NOG"; $resn_exch{"1H18 NOG"}="H181 NOG"; $resn_exch{"2H18 NOG"}="H182 NOG";
$resn_exch{"1H19 NOG"}="H191 NOG"; $resn_exch{"2H19 NOG"}="H192 NOG"; $resn_exch{"3H19 NOG"}="H193 NOG";
$resn_exch{"CH21 NOG"}="H21C NOG"; $resn_exch{"OH17 NOG"}="H17O NOG"; $resn_exch{"1H1  NOJ"}=" H11 NOJ";
$resn_exch{"2H1  NOJ"}=" H12 NOJ"; $resn_exch{"1H6  NOJ"}=" H61 NOJ"; $resn_exch{"2H6  NOJ"}=" H62 NOJ";
$resn_exch{"1H8C NOK"}="H8C1 NOK"; $resn_exch{"2H8C NOK"}="H8C2 NOK"; $resn_exch{"3H8C NOK"}="H8C3 NOK";
$resn_exch{"1H1C NOK"}="H1C1 NOK"; $resn_exch{"2H1C NOK"}="H1C2 NOK"; $resn_exch{"1H6C NOK"}="H6C1 NOK";
$resn_exch{"2H6C NOK"}="H6C2 NOK"; $resn_exch{"1H13 NOL"}="H131 NOL"; $resn_exch{"2H13 NOL"}="H132 NOL";
$resn_exch{"1H35 NOL"}="H351 NOL"; $resn_exch{"2H35 NOL"}="H352 NOL"; $resn_exch{"1H41 NOL"}="H411 NOL";
$resn_exch{"2H41 NOL"}="H412 NOL"; $resn_exch{"1H45 NOL"}="H451 NOL"; $resn_exch{"2H45 NOL"}="H452 NOL";
$resn_exch{"1H47 NOL"}="H471 NOL"; $resn_exch{"2H47 NOL"}="H472 NOL"; $resn_exch{"1H58 NOL"}="H581 NOL";
$resn_exch{"2H58 NOL"}="H582 NOL"; $resn_exch{"1H62 NOL"}="H621 NOL"; $resn_exch{"2H62 NOL"}="H622 NOL";
$resn_exch{"1H63 NOL"}="H631 NOL"; $resn_exch{"2H63 NOL"}="H632 NOL"; $resn_exch{"1H64 NOL"}="H641 NOL";
$resn_exch{"2H64 NOL"}="H642 NOL"; $resn_exch{"1H65 NOL"}="H651 NOL"; $resn_exch{"2H65 NOL"}="H652 NOL";
$resn_exch{"1H66 NOL"}="H661 NOL"; $resn_exch{"2H66 NOL"}="H662 NOL"; $resn_exch{"1H77 NOL"}="H771 NOL";
$resn_exch{"2H77 NOL"}="H772 NOL"; $resn_exch{"3H77 NOL"}="H773 NOL"; $resn_exch{"1H83 NOL"}="H831 NOL";
$resn_exch{"2H83 NOL"}="H832 NOL"; $resn_exch{"3H83 NOL"}="H833 NOL"; $resn_exch{"1H85 NOL"}="H851 NOL";
$resn_exch{"2H85 NOL"}="H852 NOL"; $resn_exch{"3H85 NOL"}="H853 NOL"; $resn_exch{"1H87 NOL"}="H871 NOL";
$resn_exch{"2H87 NOL"}="H872 NOL"; $resn_exch{"3H87 NOL"}="H873 NOL"; $resn_exch{"1H12 NOM"}="H121 NOM";
$resn_exch{"2H12 NOM"}="H122 NOM"; $resn_exch{"3H12 NOM"}="H123 NOM"; $resn_exch{"1H15 NOM"}="H151 NOM";
$resn_exch{"2H15 NOM"}="H152 NOM"; $resn_exch{"1H2  NON"}=" H21 NON"; $resn_exch{"2H2  NON"}=" H22 NON";
$resn_exch{"1H3  NON"}=" H31 NON"; $resn_exch{"2H3  NON"}=" H32 NON"; $resn_exch{"1H4  NON"}=" H41 NON";
$resn_exch{"2H4  NON"}=" H42 NON"; $resn_exch{"1H5  NON"}=" H51 NON"; $resn_exch{"2H5  NON"}=" H52 NON";
$resn_exch{"1H6  NON"}=" H61 NON"; $resn_exch{"2H6  NON"}=" H62 NON"; $resn_exch{"1H7  NON"}=" H71 NON";
$resn_exch{"2H7  NON"}=" H72 NON"; $resn_exch{"1H8  NON"}=" H81 NON"; $resn_exch{"2H8  NON"}=" H82 NON";
$resn_exch{"1H9  NON"}=" H91 NON"; $resn_exch{"2H9  NON"}=" H92 NON"; $resn_exch{"3H9  NON"}=" H93 NON";
$resn_exch{"1HM  NON"}=" HM1 NON"; $resn_exch{"2HM  NON"}=" HM2 NON"; $resn_exch{"3HM  NON"}=" HM3 NON";
$resn_exch{"1H2A NOP"}="H2A1 NOP"; $resn_exch{"2H2A NOP"}="H2A2 NOP"; $resn_exch{"3H2A NOP"}="H2A3 NOP";
$resn_exch{"1H5A NOP"}="H5A1 NOP"; $resn_exch{"2H5A NOP"}="H5A2 NOP"; $resn_exch{"2HOP NOP"}="HOP2 NOP";
$resn_exch{"3HOP NOP"}="HOP3 NOP"; $resn_exch{"1H11 NOQ"}="H111 NOQ"; $resn_exch{"2H11 NOQ"}="H112 NOQ";
$resn_exch{"1H16 NOQ"}="H161 NOQ"; $resn_exch{"2H16 NOQ"}="H162 NOQ"; $resn_exch{"1H17 NOQ"}="H171 NOQ";
$resn_exch{"2H17 NOQ"}="H172 NOQ"; $resn_exch{"1H15 NOQ"}="H151 NOQ"; $resn_exch{"2H15 NOQ"}="H152 NOQ";
$resn_exch{"1H14 NOQ"}="H141 NOQ"; $resn_exch{"2H14 NOQ"}="H142 NOQ"; $resn_exch{"1H30 NOQ"}="H301 NOQ";
$resn_exch{"2H30 NOQ"}="H302 NOQ"; $resn_exch{"1H44 NOQ"}="H441 NOQ"; $resn_exch{"2H44 NOQ"}="H442 NOQ";
$resn_exch{"3H44 NOQ"}="H443 NOQ"; $resn_exch{"1H20 NOQ"}="H201 NOQ"; $resn_exch{"2H20 NOQ"}="H202 NOQ";
$resn_exch{"1H21 NOQ"}="H211 NOQ"; $resn_exch{"2H21 NOQ"}="H212 NOQ"; $resn_exch{"1HN  NOR"}=" HN1 NOR";
$resn_exch{"2HN  NOR"}=" HN2 NOR"; $resn_exch{"1HB  NOR"}=" HB1 NOR"; $resn_exch{"2HB  NOR"}=" HB2 NOR";
$resn_exch{"1HD1 NOR"}="HD11 NOR"; $resn_exch{"2HD1 NOR"}="HD12 NOR"; $resn_exch{"1HD2 NOR"}="HD21 NOR";
$resn_exch{"2HD2 NOR"}="HD22 NOR"; $resn_exch{"1HE1 NOR"}="HE11 NOR"; $resn_exch{"2HE1 NOR"}="HE12 NOR";
$resn_exch{"1HE2 NOR"}="HE21 NOR"; $resn_exch{"2HE2 NOR"}="HE22 NOR"; $resn_exch{"1HZ  NOR"}=" HZ1 NOR";
$resn_exch{"2HZ  NOR"}=" HZ2 NOR"; $resn_exch{"1HM1 NOR"}="HM11 NOR"; $resn_exch{"2HM1 NOR"}="HM12 NOR";
$resn_exch{"3HM1 NOR"}="HM13 NOR"; $resn_exch{"1HM2 NOR"}="HM21 NOR"; $resn_exch{"2HM2 NOR"}="HM22 NOR";
$resn_exch{"3HM2 NOR"}="HM23 NOR"; $resn_exch{"1H5* NOS"}="H5'1 NOS"; $resn_exch{"2H5* NOS"}="H5'2 NOS";
$resn_exch{"*HO5 NOS"}="HO5' NOS"; $resn_exch{" H4* NOS"}=" H4' NOS"; $resn_exch{" H1* NOS"}=" H1' NOS";
$resn_exch{" H2* NOS"}=" H2' NOS"; $resn_exch{"*HO2 NOS"}="HO2' NOS"; $resn_exch{" H3* NOS"}=" H3' NOS";
$resn_exch{"*HO3 NOS"}="HO3' NOS"; $resn_exch{"1H1  NOV"}="H11A NOV"; $resn_exch{"2H1  NOV"}=" H12 NOV";
$resn_exch{"3H1  NOV"}=" H13 NOV"; $resn_exch{"1HN1 NOV"}="HN11 NOV"; $resn_exch{"2HN1 NOV"}="HN12 NOV";
$resn_exch{"1H23 NOV"}="H231 NOV"; $resn_exch{"2H23 NOV"}="H232 NOV"; $resn_exch{"3H23 NOV"}="H233 NOV";
$resn_exch{"1H26 NOV"}="H261 NOV"; $resn_exch{"2H26 NOV"}="H262 NOV"; $resn_exch{"3H26 NOV"}="H263 NOV";
$resn_exch{"1H2  NOV"}="H21A NOV"; $resn_exch{"2H2  NOV"}=" H22 NOV"; $resn_exch{"3H2  NOV"}=" H23 NOV";
$resn_exch{"1H20 NOV"}="H201 NOV"; $resn_exch{"2H20 NOV"}="H202 NOV"; $resn_exch{"1H24 NOV"}="H241 NOV";
$resn_exch{"2H24 NOV"}="H242 NOV"; $resn_exch{"3H24 NOV"}="H243 NOV"; $resn_exch{"1H25 NOV"}="H251 NOV";
$resn_exch{"2H25 NOV"}="H252 NOV"; $resn_exch{"3H25 NOV"}="H253 NOV"; $resn_exch{"1H2  NOX"}=" H21 NOX";
$resn_exch{"2H2  NOX"}=" H22 NOX"; $resn_exch{"1H3  NOX"}=" H31 NOX"; $resn_exch{"2H3  NOX"}=" H32 NOX";
$resn_exch{"1H4  NOX"}=" H41 NOX"; $resn_exch{"2H4  NOX"}=" H42 NOX"; $resn_exch{"1H5  NOX"}=" H51 NOX";
$resn_exch{"2H5  NOX"}=" H52 NOX"; $resn_exch{"1H6  NOX"}=" H61 NOX"; $resn_exch{"2H6  NOX"}=" H62 NOX";
$resn_exch{"1H8  NOX"}=" H81 NOX"; $resn_exch{"2H8  NOX"}=" H82 NOX"; $resn_exch{"1H9  NOX"}=" H91 NOX";
$resn_exch{"2H9  NOX"}=" H92 NOX"; $resn_exch{"1H19 NOX"}="H191 NOX"; $resn_exch{"2H19 NOX"}="H192 NOX";
$resn_exch{"1H20 NOX"}="H201 NOX"; $resn_exch{"2H20 NOX"}="H202 NOX"; $resn_exch{"1H21 NOX"}="H211 NOX";
$resn_exch{"2H21 NOX"}="H212 NOX"; $resn_exch{"1H7C NOY"}="H7C1 NOY"; $resn_exch{"2H7C NOY"}="H7C2 NOY";
$resn_exch{"1H6C NOY"}="H6C1 NOY"; $resn_exch{"2H6C NOY"}="H6C2 NOY"; $resn_exch{"3H6C NOY"}="H6C3 NOY";
$resn_exch{"2H1  NOY"}=" H12 NOY"; $resn_exch{"2H2  NOY"}=" H22 NOY"; $resn_exch{"1H7C NOZ"}="H7C1 NOZ";
$resn_exch{"2H7C NOZ"}="H7C2 NOZ"; $resn_exch{"1H6C NOZ"}="H6C1 NOZ"; $resn_exch{"2H6C NOZ"}="H6C2 NOZ";
$resn_exch{"'HO4  NP"}="HO4'  NP"; $resn_exch{"1H1   NP"}=" H11  NP"; $resn_exch{"2H1   NP"}=" H12  NP";
$resn_exch{"1H4   NP"}=" H41  NP"; $resn_exch{"2H4   NP"}=" H42  NP"; $resn_exch{"1H5   NP"}=" H51  NP";
$resn_exch{"2H5   NP"}=" H52  NP"; $resn_exch{"1H6   NP"}=" H61  NP"; $resn_exch{"2H6   NP"}=" H62  NP";
$resn_exch{"1H7   NP"}=" H71  NP"; $resn_exch{"2H7   NP"}=" H72  NP"; $resn_exch{"1H8   NP"}=" H81  NP";
$resn_exch{"2H8   NP"}=" H82  NP"; $resn_exch{"1HNZ NP1"}="HNZ1 NP1"; $resn_exch{"2HNZ NP1"}="HNZ2 NP1";
$resn_exch{"1HA  NP1"}=" HA1 NP1"; $resn_exch{"2HA  NP1"}=" HA2 NP1"; $resn_exch{"1HB  NP1"}=" HB1 NP1";
$resn_exch{"2HB  NP1"}=" HB2 NP1"; $resn_exch{"1HG  NP1"}=" HG1 NP1"; $resn_exch{"2HG  NP1"}=" HG2 NP1";
$resn_exch{"1HM  NP1"}=" HM1 NP1"; $resn_exch{"2HM  NP1"}=" HM2 NP1"; $resn_exch{"3HM  NP1"}=" HM3 NP1";
$resn_exch{"1HNZ NP2"}="HNZ1 NP2"; $resn_exch{"2HNZ NP2"}="HNZ2 NP2"; $resn_exch{"1HA  NP2"}=" HA1 NP2";
$resn_exch{"2HA  NP2"}=" HA2 NP2"; $resn_exch{"1HB  NP2"}=" HB1 NP2"; $resn_exch{"2HB  NP2"}=" HB2 NP2";
$resn_exch{"1HG  NP2"}=" HG1 NP2"; $resn_exch{"2HG  NP2"}=" HG2 NP2"; $resn_exch{"2HOP NP3"}="HOP2 NP3";
$resn_exch{"3HOP NP3"}="HOP3 NP3"; $resn_exch{"1H5* NP3"}="H5'1 NP3"; $resn_exch{"2H5* NP3"}="H5'2 NP3";
$resn_exch{" H4* NP3"}=" H4' NP3"; $resn_exch{" H1* NP3"}=" H1' NP3"; $resn_exch{"1H2* NP3"}="H2'1 NP3";
$resn_exch{"2H2* NP3"}="H2'2 NP3"; $resn_exch{" H3* NP3"}=" H3' NP3"; $resn_exch{"1H8C NP4"}="H8C1 NP4";
$resn_exch{"2H8C NP4"}="H8C2 NP4"; $resn_exch{"1H10 NP4"}="H101 NP4"; $resn_exch{"2H10 NP4"}="H102 NP4";
$resn_exch{"1H11 NP4"}="H111 NP4"; $resn_exch{"2H11 NP4"}="H112 NP4"; $resn_exch{"1H12 NP4"}="H121 NP4";
$resn_exch{"2H12 NP4"}="H122 NP4"; $resn_exch{"1H15 NP4"}="H151 NP4"; $resn_exch{"2H15 NP4"}="H152 NP4";
$resn_exch{"1H16 NP4"}="H161 NP4"; $resn_exch{"2H16 NP4"}="H162 NP4"; $resn_exch{"1H8C NP5"}="H8C1 NP5";
$resn_exch{"2H8C NP5"}="H8C2 NP5"; $resn_exch{"1H10 NP5"}="H101 NP5"; $resn_exch{"2H10 NP5"}="H102 NP5";
$resn_exch{"1H11 NP5"}="H111 NP5"; $resn_exch{"2H11 NP5"}="H112 NP5"; $resn_exch{"1H12 NP5"}="H121 NP5";
$resn_exch{"2H12 NP5"}="H122 NP5"; $resn_exch{"1H15 NP5"}="H151 NP5"; $resn_exch{"2H15 NP5"}="H152 NP5";
$resn_exch{"1H16 NP5"}="H161 NP5"; $resn_exch{"2H16 NP5"}="H162 NP5"; $resn_exch{"1H7  NPA"}=" H71 NPA";
$resn_exch{"2H7  NPA"}=" H72 NPA"; $resn_exch{"1HO1 NPB"}="HO11 NPB"; $resn_exch{"1HO2 NPB"}="HO21 NPB";
$resn_exch{"1HC2 NPB"}="HC21 NPB"; $resn_exch{"1HC4 NPB"}="HC41 NPB"; $resn_exch{"1HC5 NPB"}="HC51 NPB";
$resn_exch{"1HC6 NPB"}="HC61 NPB"; $resn_exch{"1H7  NPC"}=" H71 NPC"; $resn_exch{"2H7  NPC"}=" H72 NPC";
$resn_exch{"1H10 NPC"}="H101 NPC"; $resn_exch{"2H10 NPC"}="H102 NPC"; $resn_exch{"1H11 NPC"}="H111 NPC";
$resn_exch{"2H11 NPC"}="H112 NPC"; $resn_exch{"1H12 NPC"}="H121 NPC"; $resn_exch{"2H12 NPC"}="H122 NPC";
$resn_exch{"1H13 NPC"}="H131 NPC"; $resn_exch{"2H13 NPC"}="H132 NPC"; $resn_exch{"1H14 NPC"}="H141 NPC";
$resn_exch{"2H14 NPC"}="H142 NPC"; $resn_exch{"1HCN NPD"}="HCN1 NPD"; $resn_exch{"2HCN NPD"}="HCN2 NPD";
$resn_exch{"3HCN NPD"}="HCN3 NPD"; $resn_exch{"1HB  NPD"}=" HB1 NPD"; $resn_exch{"2HB  NPD"}=" HB2 NPD";
$resn_exch{"1HD1 NPD"}="HD11 NPD"; $resn_exch{"1HE1 NPD"}="HE11 NPD"; $resn_exch{"1HZ1 NPD"}="HZ11 NPD";
$resn_exch{"1HZ2 NPD"}="HZ21 NPD"; $resn_exch{"1HH2 NPD"}="HH21 NPD"; $resn_exch{"1HZ3 NPD"}="HZ31 NPD";
$resn_exch{"1HE3 NPD"}="HE31 NPD"; $resn_exch{"1H7  NPE"}=" H71 NPE"; $resn_exch{"2H7  NPE"}=" H72 NPE";
$resn_exch{"1H8  NPE"}=" H81 NPE"; $resn_exch{"2H8  NPE"}=" H82 NPE"; $resn_exch{"1H9  NPE"}=" H91 NPE";
$resn_exch{"2H9  NPE"}=" H92 NPE"; $resn_exch{"1H10 NPE"}="H101 NPE"; $resn_exch{"2H10 NPE"}="H102 NPE";
$resn_exch{"2HO' NPF"}="HO'2 NPF"; $resn_exch{"3HO' NPF"}="HO'3 NPF"; $resn_exch{"4HO' NPF"}="HO'4 NPF";
$resn_exch{"1H6' NPF"}="H6'1 NPF"; $resn_exch{"2H6' NPF"}="H6'2 NPF"; $resn_exch{"3H6' NPF"}="H6'3 NPF";
$resn_exch{"1H5  NPG"}=" H51 NPG"; $resn_exch{"2H5  NPG"}=" H52 NPG"; $resn_exch{"1H4  NPG"}=" H41 NPG";
$resn_exch{"2H4  NPG"}=" H42 NPG"; $resn_exch{"1HN  NPH"}=" H   NPH"; $resn_exch{"2HN  NPH"}=" H2  NPH";
$resn_exch{"1HB  NPH"}=" HB2 NPH"; $resn_exch{"2HB  NPH"}=" HB3 NPH"; $resn_exch{"1HD  NPH"}=" HD2 NPH";
$resn_exch{"2HD  NPH"}=" HD3 NPH"; $resn_exch{" HNZ NPH"}=" HZ  NPH"; $resn_exch{" H2  NPH"}=" HC2 NPH";
$resn_exch{"1H3  NPI"}=" H31 NPI"; $resn_exch{"2H3  NPI"}=" H32 NPI"; $resn_exch{"1H4  NPI"}=" H41 NPI";
$resn_exch{"2H4  NPI"}=" H42 NPI"; $resn_exch{"1H5  NPI"}=" H51 NPI"; $resn_exch{"2H5  NPI"}=" H52 NPI";
$resn_exch{"1H6  NPI"}=" H61 NPI"; $resn_exch{"2H6  NPI"}=" H62 NPI"; $resn_exch{"1HN2 NPI"}="HN21 NPI";
$resn_exch{"2HN2 NPI"}="HN22 NPI"; $resn_exch{"1H6A NPJ"}="H6A1 NPJ"; $resn_exch{"2H6A NPJ"}="H6A2 NPJ";
$resn_exch{"1H6B NPJ"}="H6B1 NPJ"; $resn_exch{"2H6B NPJ"}="H6B2 NPJ"; $resn_exch{"1H1  NPL"}=" H11 NPL";
$resn_exch{"2H1  NPL"}=" H12 NPL"; $resn_exch{"3H1  NPL"}=" H13 NPL"; $resn_exch{"1H2  NPL"}=" H21 NPL";
$resn_exch{"2H2  NPL"}=" H22 NPL"; $resn_exch{"3H2  NPL"}=" H23 NPL"; $resn_exch{"1H4  NPL"}=" H41 NPL";
$resn_exch{"2H4  NPL"}=" H42 NPL"; $resn_exch{"1HN4 NPL"}="HN41 NPL"; $resn_exch{"2HN4 NPL"}="HN42 NPL";
$resn_exch{"1H5  NPL"}=" H51 NPL"; $resn_exch{"2H5  NPL"}=" H52 NPL"; $resn_exch{"2HOP NPL"}="HOP2 NPL";
$resn_exch{"3HOP NPL"}="HOP3 NPL"; $resn_exch{"1H1M NPM"}="H1M1 NPM"; $resn_exch{"2H1M NPM"}="H1M2 NPM";
$resn_exch{"1H1  NPN"}=" H11 NPN"; $resn_exch{"2H1  NPN"}=" H12 NPN"; $resn_exch{"1H2  NPN"}=" H21 NPN";
$resn_exch{"2H2  NPN"}=" H22 NPN"; $resn_exch{"1H3  NPN"}=" H31 NPN"; $resn_exch{"2H3  NPN"}=" H32 NPN";
$resn_exch{"3H3  NPN"}=" H33 NPN"; $resn_exch{"1HM1 NPP"}="HM11 NPP"; $resn_exch{"2HM1 NPP"}="HM12 NPP";
$resn_exch{"3HM1 NPP"}="HM13 NPP"; $resn_exch{"1HM2 NPP"}="HM21 NPP"; $resn_exch{"2HM2 NPP"}="HM22 NPP";
$resn_exch{"3HM2 NPP"}="HM23 NPP"; $resn_exch{"1H3  NPP"}=" H31 NPP"; $resn_exch{"2H3  NPP"}=" H32 NPP";
$resn_exch{"1H5  NPP"}=" H51 NPP"; $resn_exch{"2H5  NPP"}=" H52 NPP"; $resn_exch{"1HM3 NPP"}="HM31 NPP";
$resn_exch{"2HM3 NPP"}="HM32 NPP"; $resn_exch{"3HM3 NPP"}="HM33 NPP"; $resn_exch{"1HM4 NPP"}="HM41 NPP";
$resn_exch{"2HM4 NPP"}="HM42 NPP"; $resn_exch{"3HM4 NPP"}="HM43 NPP"; $resn_exch{"1H13 NPP"}="H131 NPP";
$resn_exch{"2H13 NPP"}="H132 NPP"; $resn_exch{"1H14 NPP"}="H141 NPP"; $resn_exch{"2H14 NPP"}="H142 NPP";
$resn_exch{"1HN6 NPP"}="HN61 NPP"; $resn_exch{"2HN6 NPP"}="HN62 NPP"; $resn_exch{"1HN1 NPR"}="HN11 NPR";
$resn_exch{"2HN1 NPR"}="HN12 NPR"; $resn_exch{"1H9  NPR"}=" H91 NPR"; $resn_exch{"2H9  NPR"}=" H92 NPR";
$resn_exch{"1H17 NPR"}="H171 NPR"; $resn_exch{"2H17 NPR"}="H172 NPR"; $resn_exch{"1H10 NPS"}="H101 NPS";
$resn_exch{"2H10 NPS"}="H102 NPS"; $resn_exch{"3H10 NPS"}="H103 NPS"; $resn_exch{"1H12 NPS"}="H121 NPS";
$resn_exch{"2H12 NPS"}="H122 NPS"; $resn_exch{"3H12 NPS"}="H123 NPS"; $resn_exch{"1HN  NPT"}=" HN1 NPT";
$resn_exch{"2HN  NPT"}=" HN2 NPT"; $resn_exch{"1H1  NPT"}=" H11 NPT"; $resn_exch{"2H1  NPT"}=" H12 NPT";
$resn_exch{"1H3  NPT"}=" H31 NPT"; $resn_exch{"2H3  NPT"}=" H32 NPT"; $resn_exch{"3H3  NPT"}=" H33 NPT";
$resn_exch{"1H4  NPT"}=" H41 NPT"; $resn_exch{"2H4  NPT"}=" H42 NPT"; $resn_exch{"3H4  NPT"}=" H43 NPT";
$resn_exch{"1H5  NPT"}=" H51 NPT"; $resn_exch{"2H5  NPT"}=" H52 NPT"; $resn_exch{"3H5  NPT"}=" H53 NPT";
$resn_exch{"1H1  NPY"}=" H11 NPY"; $resn_exch{"1HAA NQG"}="HAA1 NQG"; $resn_exch{"2HAA NQG"}="HAA2 NQG";
$resn_exch{"3HAA NQG"}="HAA3 NQG"; $resn_exch{"1HAS NQG"}="HAS1 NQG"; $resn_exch{"2HAS NQG"}="HAS2 NQG";
$resn_exch{"1HBT NQG"}="HBT1 NQG"; $resn_exch{"2HBT NQG"}="HBT2 NQG"; $resn_exch{"1HB  NQG"}=" HB1 NQG";
$resn_exch{"2HB  NQG"}=" HB2 NQG"; $resn_exch{"1HAT NQG"}="HAT1 NQG"; $resn_exch{"2HAT NQG"}="HAT2 NQG";
$resn_exch{"1HAX NQG"}="HAX1 NQG"; $resn_exch{"2HAX NQG"}="HAX2 NQG"; $resn_exch{"1HAU NQG"}="HAU1 NQG";
$resn_exch{"2HAU NQG"}="HAU2 NQG"; $resn_exch{"1HAV NQG"}="HAV1 NQG"; $resn_exch{"2HAV NQG"}="HAV2 NQG";
$resn_exch{"1HAE NQG"}="HAE1 NQG"; $resn_exch{"2HAE NQG"}="HAE2 NQG"; $resn_exch{"1HAF NQG"}="HAF1 NQG";
$resn_exch{"2HAF NQG"}="HAF2 NQG"; $resn_exch{"1HAY NQG"}="HAY1 NQG"; $resn_exch{"2HAY NQG"}="HAY2 NQG";
$resn_exch{"1HAB NQI"}="HAB1 NQI"; $resn_exch{"2HAB NQI"}="HAB2 NQI"; $resn_exch{"3HAB NQI"}="HAB3 NQI";
$resn_exch{"1HAS NQI"}="HAS1 NQI"; $resn_exch{"2HAS NQI"}="HAS2 NQI"; $resn_exch{"1HBS NQI"}="HBS1 NQI";
$resn_exch{"2HBS NQI"}="HBS2 NQI"; $resn_exch{"1HB  NQI"}=" HB1 NQI"; $resn_exch{"2HB  NQI"}=" HB2 NQI";
$resn_exch{"1HAA NQI"}="HAA1 NQI"; $resn_exch{"2HAA NQI"}="HAA2 NQI"; $resn_exch{"1HAW NQI"}="HAW1 NQI";
$resn_exch{"2HAW NQI"}="HAW2 NQI"; $resn_exch{"1HAT NQI"}="HAT1 NQI"; $resn_exch{"2HAT NQI"}="HAT2 NQI";
$resn_exch{"1HAU NQI"}="HAU1 NQI"; $resn_exch{"2HAU NQI"}="HAU2 NQI"; $resn_exch{"1HAD NQI"}="HAD1 NQI";
$resn_exch{"2HAD NQI"}="HAD2 NQI"; $resn_exch{"1HAF NQI"}="HAF1 NQI"; $resn_exch{"2HAF NQI"}="HAF2 NQI";
$resn_exch{"1HAX NQI"}="HAX1 NQI"; $resn_exch{"2HAX NQI"}="HAX2 NQI"; $resn_exch{"1H24 NR2"}="H241 NR2";
$resn_exch{"2H24 NR2"}="H242 NR2"; $resn_exch{"1H21 NR2"}="H211 NR2"; $resn_exch{"2H21 NR2"}="H212 NR2";
$resn_exch{"3H21 NR2"}="H213 NR2"; $resn_exch{"1H1C NRB"}="H1C1 NRB"; $resn_exch{"2H1C NRB"}="H1C2 NRB";
$resn_exch{"3H1C NRB"}="H1C3 NRB"; $resn_exch{"1H2C NRB"}="H2C1 NRB"; $resn_exch{"2H2C NRB"}="H2C2 NRB";
$resn_exch{"1H6C NRB"}="H6C1 NRB"; $resn_exch{"2H6C NRB"}="H6C2 NRB"; $resn_exch{"3H6C NRB"}="H6C3 NRB";
$resn_exch{"1H9C NRB"}="H9C1 NRB"; $resn_exch{"2H9C NRB"}="H9C2 NRB"; $resn_exch{"3H9C NRB"}="H9C3 NRB";
$resn_exch{"1H13 NRB"}="H131 NRB"; $resn_exch{"2H13 NRB"}="H132 NRB"; $resn_exch{"3H13 NRB"}="H133 NRB";
$resn_exch{"1H14 NRB"}="H141 NRB"; $resn_exch{"2H14 NRB"}="H142 NRB"; $resn_exch{"1H16 NRB"}="H161 NRB";
$resn_exch{"2H16 NRB"}="H162 NRB"; $resn_exch{"3H16 NRB"}="H163 NRB"; $resn_exch{"1H17 NRB"}="H171 NRB";
$resn_exch{"2H17 NRB"}="H172 NRB"; $resn_exch{"3H17 NRB"}="H173 NRB"; $resn_exch{"1H20 NRB"}="H201 NRB";
$resn_exch{"2H20 NRB"}="H202 NRB"; $resn_exch{"3H20 NRB"}="H203 NRB"; $resn_exch{"1H21 NRB"}="H211 NRB";
$resn_exch{"2H21 NRB"}="H212 NRB"; $resn_exch{"1H23 NRB"}="H231 NRB"; $resn_exch{"2H23 NRB"}="H232 NRB";
$resn_exch{"3H23 NRB"}="H233 NRB"; $resn_exch{"1H28 NRB"}="H281 NRB"; $resn_exch{"2H28 NRB"}="H282 NRB";
$resn_exch{"3H28 NRB"}="H283 NRB"; $resn_exch{"1HN  NRG"}=" HN1 NRG"; $resn_exch{"2HN  NRG"}=" HN2 NRG";
$resn_exch{"1HCB NRG"}="HCB1 NRG"; $resn_exch{"2HCB NRG"}="HCB2 NRG"; $resn_exch{"1HCG NRG"}="HCG1 NRG";
$resn_exch{"2HCG NRG"}="HCG2 NRG"; $resn_exch{"1HCD NRG"}="HCD1 NRG"; $resn_exch{"2HCD NRG"}="HCD2 NRG";
$resn_exch{"2HOP NRI"}="HOP2 NRI"; $resn_exch{"3HOP NRI"}="HOP3 NRI"; $resn_exch{"1H2* NRI"}="H2'1 NRI";
$resn_exch{"2H2* NRI"}="H2'2 NRI"; $resn_exch{"1H5* NRI"}="H5'1 NRI"; $resn_exch{"2H5* NRI"}="H5'2 NRI";
$resn_exch{" H4* NRI"}=" H4' NRI"; $resn_exch{" H3* NRI"}=" H3' NRI"; $resn_exch{"1H6* NRI"}="H6'1 NRI";
$resn_exch{"2H6* NRI"}="H6'2 NRI"; $resn_exch{"1HN* NRI"}="HN'1 NRI"; $resn_exch{"3HO* NRI"}="HO'3 NRI";
$resn_exch{" HN2 NRQ"}=" H2  NRQ"; $resn_exch{"1HE  NRQ"}="HE1A NRQ"; $resn_exch{"2HE  NRQ"}="HE2A NRQ";
$resn_exch{"3HE  NRQ"}=" HE3 NRQ"; $resn_exch{"1HG1 NRQ"}="HG11 NRQ"; $resn_exch{"2HG1 NRQ"}="HG12 NRQ";
$resn_exch{"1HB1 NRQ"}="HB11 NRQ"; $resn_exch{"2HB1 NRQ"}="HB12 NRQ"; $resn_exch{"1HA3 NRQ"}="HA31 NRQ";
$resn_exch{"2HA3 NRQ"}="HA32 NRQ"; $resn_exch{" H4* NRT"}=" H4' NRT"; $resn_exch{"1H3* NRT"}="H3'1 NRT";
$resn_exch{"2H3* NRT"}="H3'2 NRT"; $resn_exch{"1H2* NRT"}="H2'1 NRT"; $resn_exch{"2H2* NRT"}="H2'2 NRT";
$resn_exch{"1H2N NRT"}="H2N1 NRT"; $resn_exch{"2H2N NRT"}="H2N2 NRT"; $resn_exch{"1H60 NRT"}="H601 NRT";
$resn_exch{"2H60 NRT"}="H602 NRT"; $resn_exch{"1H14 NRT"}="H141 NRT"; $resn_exch{"2H14 NRT"}="H142 NRT";
$resn_exch{"1HN1 NRU"}="HN11 NRU"; $resn_exch{"2HN1 NRU"}="HN12 NRU"; $resn_exch{"1HN4 NRU"}="HN41 NRU";
$resn_exch{"2HN4 NRU"}="HN42 NRU"; $resn_exch{"1HN5 NRU"}="HN51 NRU"; $resn_exch{"2HN5 NRU"}="HN52 NRU";
$resn_exch{"1HN2 NRU"}="HN21 NRU"; $resn_exch{"2HN2 NRU"}="HN22 NRU"; $resn_exch{"1HN3 NRU"}="HN31 NRU";
$resn_exch{"2HN3 NRU"}="HN32 NRU"; $resn_exch{"1HN6 NRU"}="HN61 NRU"; $resn_exch{"2HN6 NRU"}="HN62 NRU";
$resn_exch{"1HM1 NS1"}="HM11 NS1"; $resn_exch{"2HM1 NS1"}="HM12 NS1"; $resn_exch{"3HM1 NS1"}="HM13 NS1";
$resn_exch{"1HM2 NS1"}="HM21 NS1"; $resn_exch{"2HM2 NS1"}="HM22 NS1"; $resn_exch{"3HM2 NS1"}="HM23 NS1";
$resn_exch{"1H2  NS1"}=" H21 NS1"; $resn_exch{"2H2  NS1"}=" H22 NS1"; $resn_exch{"1H3  NS1"}=" H31 NS1";
$resn_exch{"2H3  NS1"}=" H32 NS1"; $resn_exch{"1H4  NS1"}=" H41 NS1"; $resn_exch{"2H4  NS1"}=" H42 NS1";
$resn_exch{"1H6  NS1"}=" H61 NS1"; $resn_exch{"2H6  NS1"}=" H62 NS1"; $resn_exch{"3H6  NS1"}=" H63 NS1";
$resn_exch{"1H8  NS1"}=" H81 NS1"; $resn_exch{"2H8  NS1"}=" H82 NS1"; $resn_exch{"1H9  NS1"}=" H91 NS1";
$resn_exch{"2H9  NS1"}=" H92 NS1"; $resn_exch{"1H11 NS1"}="H111 NS1"; $resn_exch{"2H11 NS1"}="H112 NS1";
$resn_exch{"3H11 NS1"}="H113 NS1"; $resn_exch{"1H16 NS1"}="H161 NS1"; $resn_exch{"2H16 NS1"}="H162 NS1";
$resn_exch{"3H16 NS1"}="H163 NS1"; $resn_exch{"1H22 NS1"}="H221 NS1"; $resn_exch{"2H22 NS1"}="H222 NS1";
$resn_exch{"3H22 NS1"}="H223 NS1"; $resn_exch{"1H27 NS1"}="H271 NS1"; $resn_exch{"2H27 NS1"}="H272 NS1";
$resn_exch{"3H27 NS1"}="H273 NS1"; $resn_exch{"1H32 NS1"}="H321 NS1"; $resn_exch{"2H32 NS1"}="H322 NS1";
$resn_exch{"3H32 NS1"}="H323 NS1"; $resn_exch{"1H33 NS1"}="H331 NS1"; $resn_exch{"2H33 NS1"}="H332 NS1";
$resn_exch{"1H34 NS1"}="H341 NS1"; $resn_exch{"2H34 NS1"}="H342 NS1"; $resn_exch{"1H35 NS1"}="H351 NS1";
$resn_exch{"2H35 NS1"}="H352 NS1"; $resn_exch{"1HM3 NS1"}="HM31 NS1"; $resn_exch{"2HM3 NS1"}="HM32 NS1";
$resn_exch{"3HM3 NS1"}="HM33 NS1"; $resn_exch{"1HM4 NS1"}="HM41 NS1"; $resn_exch{"2HM4 NS1"}="HM42 NS1";
$resn_exch{"3HM4 NS1"}="HM43 NS1"; $resn_exch{"1HM1 NS5"}="HM11 NS5"; $resn_exch{"2HM1 NS5"}="HM12 NS5";
$resn_exch{"3HM1 NS5"}="HM13 NS5"; $resn_exch{"1HM2 NS5"}="HM21 NS5"; $resn_exch{"2HM2 NS5"}="HM22 NS5";
$resn_exch{"3HM2 NS5"}="HM23 NS5"; $resn_exch{"1H2  NS5"}=" H21 NS5"; $resn_exch{"2H2  NS5"}=" H22 NS5";
$resn_exch{"1H3  NS5"}=" H31 NS5"; $resn_exch{"2H3  NS5"}=" H32 NS5"; $resn_exch{"1H4  NS5"}=" H41 NS5";
$resn_exch{"2H4  NS5"}=" H42 NS5"; $resn_exch{"1H6  NS5"}=" H61 NS5"; $resn_exch{"2H6  NS5"}=" H62 NS5";
$resn_exch{"3H6  NS5"}=" H63 NS5"; $resn_exch{"1H8  NS5"}=" H81 NS5"; $resn_exch{"2H8  NS5"}=" H82 NS5";
$resn_exch{"1H9  NS5"}=" H91 NS5"; $resn_exch{"2H9  NS5"}=" H92 NS5"; $resn_exch{"1H11 NS5"}="H111 NS5";
$resn_exch{"2H11 NS5"}="H112 NS5"; $resn_exch{"3H11 NS5"}="H113 NS5"; $resn_exch{"1H16 NS5"}="H161 NS5";
$resn_exch{"2H16 NS5"}="H162 NS5"; $resn_exch{"3H16 NS5"}="H163 NS5"; $resn_exch{"1H22 NS5"}="H221 NS5";
$resn_exch{"2H22 NS5"}="H222 NS5"; $resn_exch{"3H22 NS5"}="H223 NS5"; $resn_exch{"1H27 NS5"}="H271 NS5";
$resn_exch{"2H27 NS5"}="H272 NS5"; $resn_exch{"3H27 NS5"}="H273 NS5"; $resn_exch{"1H32 NS5"}="H321 NS5";
$resn_exch{"2H32 NS5"}="H322 NS5"; $resn_exch{"3H32 NS5"}="H323 NS5"; $resn_exch{"1H33 NS5"}="H331 NS5";
$resn_exch{"2H33 NS5"}="H332 NS5"; $resn_exch{"1H34 NS5"}="H341 NS5"; $resn_exch{"2H34 NS5"}="H342 NS5";
$resn_exch{"1H35 NS5"}="H351 NS5"; $resn_exch{"2H35 NS5"}="H352 NS5"; $resn_exch{"1HM3 NS5"}="HM31 NS5";
$resn_exch{"2HM3 NS5"}="HM32 NS5"; $resn_exch{"3HM3 NS5"}="HM33 NS5"; $resn_exch{"1HM4 NS5"}="HM41 NS5";
$resn_exch{"2HM4 NS5"}="HM42 NS5"; $resn_exch{"3HM4 NS5"}="HM43 NS5"; $resn_exch{"1H6  NSB"}=" H61 NSB";
$resn_exch{"2H6  NSB"}=" H62 NSB"; $resn_exch{"1H4  NSB"}=" H41 NSB"; $resn_exch{"2H4  NSB"}=" H42 NSB";
$resn_exch{"1H2  NSB"}=" H21 NSB"; $resn_exch{"2H2  NSB"}=" H22 NSB"; $resn_exch{"1H20 NSB"}="H201 NSB";
$resn_exch{"2H20 NSB"}="H202 NSB"; $resn_exch{"3H20 NSB"}="H203 NSB"; $resn_exch{"1H18 NSB"}="H181 NSB";
$resn_exch{"2H18 NSB"}="H182 NSB"; $resn_exch{"3H18 NSB"}="H183 NSB"; $resn_exch{"1H16 NSB"}="H161 NSB";
$resn_exch{"2H16 NSB"}="H162 NSB"; $resn_exch{"3H16 NSB"}="H163 NSB"; $resn_exch{"1HN1 NSC"}="HN11 NSC";
$resn_exch{"2HN1 NSC"}="HN12 NSC"; $resn_exch{"1H8  NSC"}=" H81 NSC"; $resn_exch{"2H8  NSC"}=" H82 NSC";
$resn_exch{"3H8  NSC"}=" H83 NSC"; $resn_exch{"1H25 NSC"}="H251 NSC"; $resn_exch{"2H25 NSC"}="H252 NSC";
$resn_exch{"1H20 NSC"}="H201 NSC"; $resn_exch{"2H20 NSC"}="H202 NSC"; $resn_exch{"3H20 NSC"}="H203 NSC";
$resn_exch{"1H12 NSI"}="H121 NSI"; $resn_exch{"2H12 NSI"}="H122 NSI"; $resn_exch{"1H28 NSI"}="H281 NSI";
$resn_exch{"2H28 NSI"}="H282 NSI"; $resn_exch{"3H28 NSI"}="H283 NSI"; $resn_exch{"1H2  NSK"}=" H21 NSK";
$resn_exch{"2H2  NSK"}=" H22 NSK"; $resn_exch{"1H3  NSK"}=" H31 NSK"; $resn_exch{"2H3  NSK"}=" H32 NSK";
$resn_exch{"1H7  NSK"}=" H71 NSK"; $resn_exch{"2H7  NSK"}=" H72 NSK"; $resn_exch{"1H8  NSK"}=" H81 NSK";
$resn_exch{"2H8  NSK"}=" H82 NSK"; $resn_exch{"1H9  NSK"}=" H91 NSK"; $resn_exch{"2H9  NSK"}=" H92 NSK";
$resn_exch{"1H10 NSK"}="H101 NSK"; $resn_exch{"2H10 NSK"}="H102 NSK"; $resn_exch{"1HN2 NSK"}="HN21 NSK";
$resn_exch{"2HN2 NSK"}="HN22 NSK"; $resn_exch{"1H1  NSM"}=" H11 NSM"; $resn_exch{"2H1  NSM"}=" H12 NSM";
$resn_exch{"3H1  NSM"}=" H13 NSM"; $resn_exch{"1H1  NSN"}=" H11 NSN"; $resn_exch{"2H1  NSN"}=" H12 NSN";
$resn_exch{"3H1  NSN"}=" H13 NSN"; $resn_exch{"1H5  NSN"}=" H51 NSN"; $resn_exch{"2H5  NSN"}=" H52 NSN";
$resn_exch{"1HN2 NSN"}="HN21 NSN"; $resn_exch{"2HN2 NSN"}="HN22 NSN"; $resn_exch{"1HN3 NSN"}="HN31 NSN";
$resn_exch{"2HN3 NSN"}="HN32 NSN"; $resn_exch{"1HN  NSP"}=" HN1 NSP"; $resn_exch{"2HN  NSP"}=" HN2 NSP";
$resn_exch{"1HCA NSP"}="HCA1 NSP"; $resn_exch{"2HCA NSP"}="HCA2 NSP"; $resn_exch{"1HQ2 NSP"}="HQ21 NSP";
$resn_exch{"2HQ2 NSP"}="HQ22 NSP"; $resn_exch{"1HS1 NSP"}="HS11 NSP"; $resn_exch{"2HS1 NSP"}="HS12 NSP";
$resn_exch{"3HS1 NSP"}="HS13 NSP"; $resn_exch{"1HN  NSS"}=" HN1 NSS"; $resn_exch{"2HN  NSS"}=" HN2 NSS";
$resn_exch{"3HN  NSS"}=" HN3 NSS"; $resn_exch{"1HB  NSS"}=" HB1 NSS"; $resn_exch{"2HB  NSS"}=" HB2 NSS";
$resn_exch{"1HD2 NSS"}="HD21 NSS"; $resn_exch{"2HD2 NSS"}="HD22 NSS"; $resn_exch{"1H5* NSS"}="H5'1 NSS";
$resn_exch{"2H5* NSS"}="H5'2 NSS"; $resn_exch{" H4* NSS"}=" H4' NSS"; $resn_exch{" H3* NSS"}=" H3' NSS";
$resn_exch{" H2* NSS"}=" H2' NSS"; $resn_exch{" H1* NSS"}=" H1' NSS"; $resn_exch{"1HN6 NSS"}="HN61 NSS";
$resn_exch{"2HN6 NSS"}="HN62 NSS"; $resn_exch{"1HN2  NT"}="HN21  NT"; $resn_exch{"2HN2  NT"}="HN22  NT";
$resn_exch{"1H2   NT"}=" H21  NT"; $resn_exch{"2H2   NT"}=" H22  NT"; $resn_exch{"1H8   NT"}=" H81  NT";
$resn_exch{"2H8   NT"}=" H82  NT"; $resn_exch{"3H8   NT"}=" H83  NT"; $resn_exch{"1H14  NT"}="H141  NT";
$resn_exch{"2H14  NT"}="H142  NT"; $resn_exch{"3H14  NT"}="H143  NT"; $resn_exch{"1H16  NT"}="H161  NT";
$resn_exch{"2H16  NT"}="H162  NT"; $resn_exch{"1H17  NT"}="H171  NT"; $resn_exch{"2H17  NT"}="H172  NT";
$resn_exch{"1HN0  NT"}="HN01  NT"; $resn_exch{"2HN0  NT"}="HN02  NT"; $resn_exch{"1H10 NT1"}="H101 NT1";
$resn_exch{"2H10 NT1"}="H102 NT1"; $resn_exch{"1H13 NT1"}="H131 NT1"; $resn_exch{"2H13 NT1"}="H132 NT1";
$resn_exch{"1H15 NT1"}="H151 NT1"; $resn_exch{"2H15 NT1"}="H152 NT1"; $resn_exch{"3H15 NT1"}="H153 NT1";
$resn_exch{"1H18 NT1"}="H181 NT1"; $resn_exch{"2H18 NT1"}="H182 NT1"; $resn_exch{"3H18 NT1"}="H183 NT1";
$resn_exch{"1H1B NT1"}="H1B1 NT1"; $resn_exch{"2H1B NT1"}="H1B2 NT1"; $resn_exch{"3H1B NT1"}="H1B3 NT1";
$resn_exch{"1H23 NT1"}="H231 NT1"; $resn_exch{"2H23 NT1"}="H232 NT1"; $resn_exch{"1H28 NT1"}="H281 NT1";
$resn_exch{"2H28 NT1"}="H282 NT1"; $resn_exch{"3H28 NT1"}="H283 NT1"; $resn_exch{"1H2B NT1"}="H2B1 NT1";
$resn_exch{"2H2B NT1"}="H2B2 NT1"; $resn_exch{"3H2B NT1"}="H2B3 NT1"; $resn_exch{"1H25 NT1"}="H251 NT1";
$resn_exch{"2H25 NT1"}="H252 NT1"; $resn_exch{"3H25 NT1"}="H253 NT1"; $resn_exch{"1H35 NT1"}="H351 NT1";
$resn_exch{"2H35 NT1"}="H352 NT1"; $resn_exch{"1H3D NT1"}="H3D1 NT1"; $resn_exch{"2H3D NT1"}="H3D2 NT1";
$resn_exch{"3H3D NT1"}="H3D3 NT1"; $resn_exch{"1H3G NT1"}="H3G1 NT1"; $resn_exch{"2H3G NT1"}="H3G2 NT1";
$resn_exch{"3H3G NT1"}="H3G3 NT1"; $resn_exch{"1H48 NT1"}="H481 NT1"; $resn_exch{"2H48 NT1"}="H482 NT1";
$resn_exch{"3H48 NT1"}="H483 NT1"; $resn_exch{"1H4B NT1"}="H4B1 NT1"; $resn_exch{"2H4B NT1"}="H4B2 NT1";
$resn_exch{"3H4B NT1"}="H4B3 NT1"; $resn_exch{"1H53 NT1"}="H531 NT1"; $resn_exch{"2H53 NT1"}="H532 NT1";
$resn_exch{"1H68 NT1"}="H681 NT1"; $resn_exch{"2H68 NT1"}="H682 NT1"; $resn_exch{"3H68 NT1"}="H683 NT1";
$resn_exch{"1H6B NT1"}="H6B1 NT1"; $resn_exch{"2H6B NT1"}="H6B2 NT1"; $resn_exch{"3H6B NT1"}="H6B3 NT1";
$resn_exch{"1H73 NT1"}="H731 NT1"; $resn_exch{"2H73 NT1"}="H732 NT1"; $resn_exch{"1H7B NT1"}="H7B1 NT1";
$resn_exch{"2H7B NT1"}="H7B2 NT1"; $resn_exch{"3H7B NT1"}="H7B3 NT1"; $resn_exch{"1H7I NT1"}="H7I1 NT1";
$resn_exch{"2H7I NT1"}="H7I2 NT1"; $resn_exch{"1H86 NT2"}="H861 NT2"; $resn_exch{"2H86 NT2"}="H862 NT2";
$resn_exch{"1H6  NTA"}=" H61 NTA"; $resn_exch{"2H6  NTA"}=" H62 NTA"; $resn_exch{"1H10 NTA"}="H101 NTA";
$resn_exch{"2H10 NTA"}="H102 NTA"; $resn_exch{"1H2  NTA"}=" H21 NTA"; $resn_exch{"2H2  NTA"}=" H22 NTA";
$resn_exch{"1HN  NTB"}=" HN1 NTB"; $resn_exch{"2HN  NTB"}=" HN2 NTB"; $resn_exch{"1H1  NTB"}=" H11 NTB";
$resn_exch{"2H1  NTB"}=" H12 NTB"; $resn_exch{"3H1  NTB"}=" H13 NTB"; $resn_exch{"1H2  NTB"}=" H21 NTB";
$resn_exch{"2H2  NTB"}=" H22 NTB"; $resn_exch{"3H2  NTB"}=" H23 NTB"; $resn_exch{"1H3  NTB"}=" H31 NTB";
$resn_exch{"2H3  NTB"}=" H32 NTB"; $resn_exch{"3H3  NTB"}=" H33 NTB"; $resn_exch{"1H2  NTC"}=" H21 NTC";
$resn_exch{"2H2  NTC"}=" H22 NTC"; $resn_exch{"1H4  NTC"}=" H41 NTC"; $resn_exch{"2H4  NTC"}=" H42 NTC";
$resn_exch{"1H2  NTD"}=" H21 NTD"; $resn_exch{"2H2  NTD"}=" H22 NTD"; $resn_exch{"1H3  NTD"}=" H31 NTD";
$resn_exch{"2H3  NTD"}=" H32 NTD"; $resn_exch{"1H4  NTD"}=" H41 NTD"; $resn_exch{"2H4  NTD"}=" H42 NTD";
$resn_exch{"1H6  NTF"}=" H61 NTF"; $resn_exch{"2H6  NTF"}=" H62 NTF"; $resn_exch{"1H11 NTH"}="H111 NTH";
$resn_exch{"2H11 NTH"}="H112 NTH"; $resn_exch{"1H12 NTH"}="H121 NTH"; $resn_exch{"2H12 NTH"}="H122 NTH";
$resn_exch{"1H15 NTH"}="H151 NTH"; $resn_exch{"2H15 NTH"}="H152 NTH"; $resn_exch{"1H16 NTH"}="H161 NTH";
$resn_exch{"2H16 NTH"}="H162 NTH"; $resn_exch{"1H18 NTH"}="H181 NTH"; $resn_exch{"2H18 NTH"}="H182 NTH";
$resn_exch{"3H18 NTH"}="H183 NTH"; $resn_exch{"1H22 NTH"}="H221 NTH"; $resn_exch{"2H22 NTH"}="H222 NTH";
$resn_exch{"1H21 NTH"}="H211 NTH"; $resn_exch{"2H21 NTH"}="H212 NTH"; $resn_exch{"1H2C NTJ"}="H2C1 NTJ";
$resn_exch{"2H2C NTJ"}="H2C2 NTJ"; $resn_exch{"3H2C NTJ"}="H2C3 NTJ"; $resn_exch{"1H1C NTJ"}="H1C1 NTJ";
$resn_exch{"2H1C NTJ"}="H1C2 NTJ"; $resn_exch{"3H1C NTJ"}="H1C3 NTJ"; $resn_exch{"1H3C NTJ"}="H3C1 NTJ";
$resn_exch{"2H3C NTJ"}="H3C2 NTJ"; $resn_exch{"1H4C NTJ"}="H4C1 NTJ"; $resn_exch{"2H4C NTJ"}="H4C2 NTJ";
$resn_exch{"3H4C NTJ"}="H4C3 NTJ"; $resn_exch{"1HZ  NTN"}=" HZ1 NTN"; $resn_exch{"2HZ  NTN"}=" HZ2 NTN";
$resn_exch{"1H15 NTO"}="H151 NTO"; $resn_exch{"2H15 NTO"}="H152 NTO"; $resn_exch{"1H33 NTO"}="H331 NTO";
$resn_exch{"2H33 NTO"}="H332 NTO"; $resn_exch{"1H53 NTO"}="H531 NTO"; $resn_exch{"2H53 NTO"}="H532 NTO";
$resn_exch{"1H5N NTO"}="H5N1 NTO"; $resn_exch{"2H5N NTO"}="H5N2 NTO"; $resn_exch{"3H5N NTO"}="H5N3 NTO";
$resn_exch{"1H11 NTP"}="H111 NTP"; $resn_exch{"2H11 NTP"}="H112 NTP"; $resn_exch{"3H11 NTP"}="H113 NTP";
$resn_exch{"1H15 NTP"}="H151 NTP"; $resn_exch{"2H15 NTP"}="H152 NTP"; $resn_exch{"1H1D NTP"}="H1D1 NTP";
$resn_exch{"2H1D NTP"}="H1D2 NTP"; $resn_exch{"3H1D NTP"}="H1D3 NTP"; $resn_exch{"1H28 NTP"}="H281 NTP";
$resn_exch{"2H28 NTP"}="H282 NTP"; $resn_exch{"3H28 NTP"}="H283 NTP"; $resn_exch{"1H2B NTP"}="H2B1 NTP";
$resn_exch{"2H2B NTP"}="H2B2 NTP"; $resn_exch{"3H2B NTP"}="H2B3 NTP"; $resn_exch{"1H33 NTP"}="H331 NTP";
$resn_exch{"2H33 NTP"}="H332 NTP"; $resn_exch{"DHO3 NTP"}="HO3D NTP"; $resn_exch{"IHO3 NTP"}="HO3I NTP";
$resn_exch{"1H48 NTP"}="H481 NTP"; $resn_exch{"2H48 NTP"}="H482 NTP"; $resn_exch{"3H48 NTP"}="H483 NTP";
$resn_exch{"1H53 NTP"}="H531 NTP"; $resn_exch{"2H53 NTP"}="H532 NTP"; $resn_exch{"DHO5 NTP"}="HO5D NTP";
$resn_exch{"IHO5 NTP"}="HO5I NTP"; $resn_exch{"1H5N NTP"}="H5N1 NTP"; $resn_exch{"2H5N NTP"}="H5N2 NTP";
$resn_exch{"3H5N NTP"}="H5N3 NTP"; $resn_exch{"1H2* NTU"}="H2'1 NTU"; $resn_exch{"2H2* NTU"}="H2'2 NTU";
$resn_exch{"3H2* NTU"}="H2'3 NTU"; $resn_exch{"1HN2 NTU"}="HN21 NTU"; $resn_exch{"2HN2 NTU"}="HN22 NTU";
$resn_exch{"1H10 NTU"}="H101 NTU"; $resn_exch{"2H10 NTU"}="H102 NTU"; $resn_exch{"1H11 NTU"}="H111 NTU";
$resn_exch{"2H11 NTU"}="H112 NTU"; $resn_exch{"1H12 NTU"}="H121 NTU"; $resn_exch{"2H12 NTU"}="H122 NTU";
$resn_exch{"1H13 NTU"}="H131 NTU"; $resn_exch{"2H13 NTU"}="H132 NTU"; $resn_exch{"1H14 NTU"}="H141 NTU";
$resn_exch{"2H14 NTU"}="H142 NTU"; $resn_exch{"1H4* NTU"}="H4'1 NTU"; $resn_exch{"2H4* NTU"}="H4'2 NTU";
$resn_exch{"3H4* NTU"}="H4'3 NTU"; $resn_exch{"1HN4 NTU"}="HN41 NTU"; $resn_exch{"2HN4 NTU"}="HN42 NTU";
$resn_exch{"1H6  NTZ"}=" H61 NTZ"; $resn_exch{"2H6  NTZ"}=" H62 NTZ"; $resn_exch{"1H9  NU1"}=" H91 NU1";
$resn_exch{"2H9  NU1"}=" H92 NU1"; $resn_exch{"3H9  NU1"}=" H93 NU1"; $resn_exch{"1H22 NU5"}="H221 NU5";
$resn_exch{"2H22 NU5"}="H222 NU5"; $resn_exch{"1H23 NU5"}="H231 NU5"; $resn_exch{"2H23 NU5"}="H232 NU5";
$resn_exch{"1H24 NU5"}="H241 NU5"; $resn_exch{"2H24 NU5"}="H242 NU5"; $resn_exch{"1H25 NU5"}="H251 NU5";
$resn_exch{"2H25 NU5"}="H252 NU5"; $resn_exch{"1H26 NU5"}="H261 NU5"; $resn_exch{"2H26 NU5"}="H262 NU5";
$resn_exch{"1H9  NU5"}=" H91 NU5"; $resn_exch{"2H9  NU5"}=" H92 NU5"; $resn_exch{"1H15 NU5"}="H151 NU5";
$resn_exch{"2H15 NU5"}="H152 NU5"; $resn_exch{"1H14 NU5"}="H141 NU5"; $resn_exch{"2H14 NU5"}="H142 NU5";
$resn_exch{"1H13 NU5"}="H131 NU5"; $resn_exch{"2H13 NU5"}="H132 NU5"; $resn_exch{"1H12 NU5"}="H121 NU5";
$resn_exch{"2H12 NU5"}="H122 NU5"; $resn_exch{"1H11 NU5"}="H111 NU5"; $resn_exch{"2H11 NU5"}="H112 NU5";
$resn_exch{"1HN3 NUP"}="HN31 NUP"; $resn_exch{"2HN3 NUP"}="HN32 NUP"; $resn_exch{" H1* NUP"}=" H1' NUP";
$resn_exch{" H2* NUP"}=" H2' NUP"; $resn_exch{"HO2* NUP"}="HO2' NUP"; $resn_exch{" H3* NUP"}=" H3' NUP";
$resn_exch{"HO3* NUP"}="HO3' NUP"; $resn_exch{" H4* NUP"}=" H4' NUP"; $resn_exch{"1H5* NUP"}="H5'1 NUP";
$resn_exch{"2H5* NUP"}="H5'2 NUP"; $resn_exch{"1HN  NVA"}=" H   NVA"; $resn_exch{"2HN  NVA"}=" H2  NVA";
$resn_exch{"1HB  NVA"}=" HB2 NVA"; $resn_exch{"2HB  NVA"}=" HB3 NVA"; $resn_exch{"1HG  NVA"}=" HG2 NVA";
$resn_exch{"2HG  NVA"}=" HG3 NVA"; $resn_exch{"1HD  NVA"}=" HD1 NVA"; $resn_exch{"2HD  NVA"}=" HD2 NVA";
$resn_exch{"3HD  NVA"}=" HD3 NVA"; $resn_exch{"1HAA NVG"}="HAA1 NVG"; $resn_exch{"2HAA NVG"}="HAA2 NVG";
$resn_exch{"1HAN NVG"}="HAN1 NVG"; $resn_exch{"2HAN NVG"}="HAN2 NVG"; $resn_exch{"1HAL NVG"}="HAL1 NVG";
$resn_exch{"2HAL NVG"}="HAL2 NVG"; $resn_exch{"1HAM NVG"}="HAM1 NVG"; $resn_exch{"2HAM NVG"}="HAM2 NVG";
$resn_exch{"1H7  NVI"}=" H71 NVI"; $resn_exch{"2H7  NVI"}=" H72 NVI"; $resn_exch{"1HCB NVP"}="HCB1 NVP";
$resn_exch{"2HCB NVP"}="HCB2 NVP"; $resn_exch{"1HCC NVP"}="HCC1 NVP"; $resn_exch{"2HCC NVP"}="HCC2 NVP";
$resn_exch{"1HCD NVP"}="HCD1 NVP"; $resn_exch{"2HCD NVP"}="HCD2 NVP"; $resn_exch{"3HCD NVP"}="HCD3 NVP";
$resn_exch{"2H1  NW1"}=" H12 NW1"; $resn_exch{"1H1  NW1"}=" H11 NW1"; $resn_exch{"2H11 NW1"}="H112 NW1";
$resn_exch{"1H11 NW1"}="H111 NW1"; $resn_exch{"2H10 NW1"}="H102 NW1"; $resn_exch{"1H10 NW1"}="H101 NW1";
$resn_exch{"2H9  NW1"}=" H92 NW1"; $resn_exch{"1H9  NW1"}=" H91 NW1"; $resn_exch{"2H8  NW1"}=" H82 NW1";
$resn_exch{"1H8  NW1"}=" H81 NW1"; $resn_exch{"2H7  NW1"}=" H72 NW1"; $resn_exch{"1H7  NW1"}=" H71 NW1";
$resn_exch{"3H9N NW1"}="H9N3 NW1"; $resn_exch{"2H9N NW1"}="H9N2 NW1"; $resn_exch{"2H2  NW1"}=" H22 NW1";
$resn_exch{"1H2  NW1"}=" H21 NW1"; $resn_exch{"1H2C NWA"}="H2C1 NWA"; $resn_exch{"2H2C NWA"}="H2C2 NWA";
$resn_exch{"1H3C NWA"}="H3C1 NWA"; $resn_exch{"2H3C NWA"}="H3C2 NWA"; $resn_exch{"1H6C NWA"}="H6C1 NWA";
$resn_exch{"2H6C NWA"}="H6C2 NWA"; $resn_exch{"3H6C NWA"}="H6C3 NWA"; $resn_exch{"1H8C NWA"}="H8C1 NWA";
$resn_exch{"2H8C NWA"}="H8C2 NWA"; $resn_exch{"3H8C NWA"}="H8C3 NWA"; $resn_exch{"1H9C NWA"}="H9C1 NWA";
$resn_exch{"2H9C NWA"}="H9C2 NWA"; $resn_exch{"3H9C NWA"}="H9C3 NWA"; $resn_exch{"1H10 NWA"}="H101 NWA";
$resn_exch{"2H10 NWA"}="H102 NWA"; $resn_exch{"3H10 NWA"}="H103 NWA"; $resn_exch{"1H4C NWA"}="H4C1 NWA";
$resn_exch{"2H4C NWA"}="H4C2 NWA"; $resn_exch{"1H2C NWB"}="H2C1 NWB"; $resn_exch{"2HA  NWB"}=" HA2 NWB";
$resn_exch{"1HA  NWB"}=" HA1 NWB"; $resn_exch{"2HB  NWB"}=" HB2 NWB"; $resn_exch{"1H6C NWB"}="H6C1 NWB";
$resn_exch{"2H6C NWB"}="H6C2 NWB"; $resn_exch{"3H6  NWB"}=" H63 NWB"; $resn_exch{"1H10 NWB"}="H101 NWB";
$resn_exch{"2H10 NWB"}="H102 NWB"; $resn_exch{"3H10 NWB"}="H103 NWB"; $resn_exch{"1H8C NWB"}="H8C1 NWB";
$resn_exch{"2H8C NWB"}="H8C2 NWB"; $resn_exch{"3H8C NWB"}="H8C3 NWB"; $resn_exch{"1H14 NWB"}="H141 NWB";
$resn_exch{"2H14 NWB"}="H142 NWB"; $resn_exch{"1H9C NWB"}="H9C1 NWB"; $resn_exch{"2H9C NWB"}="H9C2 NWB";
$resn_exch{"3H9C NWB"}="H9C3 NWB"; $resn_exch{"1HBC NXA"}="HBC1 NXA"; $resn_exch{"2HBC NXA"}="HBC2 NXA";
$resn_exch{"3HBC NXA"}="HBC3 NXA"; $resn_exch{"1HD  NXA"}=" HD1 NXA"; $resn_exch{"1H11 NXD"}="H111 NXD";
$resn_exch{"2H11 NXD"}="H112 NXD"; $resn_exch{"3H11 NXD"}="H113 NXD"; $resn_exch{"1H3  NXD"}=" H31 NXD";
$resn_exch{"2H3  NXD"}=" H32 NXD"; $resn_exch{"1HB  NXD"}=" HB1 NXD"; $resn_exch{"2HB  NXD"}=" HB2 NXD";
$resn_exch{"3HB  NXD"}=" HB3 NXD"; $resn_exch{"1H9  NXD"}=" H91 NXD"; $resn_exch{"2H9  NXD"}=" H92 NXD";
$resn_exch{"1HAK NXD"}="HAK1 NXD"; $resn_exch{"2HAK NXD"}="HAK2 NXD"; $resn_exch{"1H14 NXN"}="H141 NXN";
$resn_exch{"2H14 NXN"}="H142 NXN"; $resn_exch{"1H15 NXN"}="H151 NXN"; $resn_exch{"2H15 NXN"}="H152 NXN";
$resn_exch{"1H9  NXN"}=" H91 NXN"; $resn_exch{"2H9  NXN"}=" H92 NXN"; $resn_exch{"1H25 NYC"}="H251 NYC";
$resn_exch{"2H25 NYC"}="H252 NYC"; $resn_exch{"3H25 NYC"}="H253 NYC"; $resn_exch{"1H26 NYC"}="H261 NYC";
$resn_exch{"2H26 NYC"}="H262 NYC"; $resn_exch{"1H18 NYC"}="H181 NYC"; $resn_exch{"2H18 NYC"}="H182 NYC";
$resn_exch{"1HD1 NYG"}="HD11 NYG"; $resn_exch{"2HD1 NYG"}="HD12 NYG"; $resn_exch{"1HB1 NYG"}="HB11 NYG";
$resn_exch{"2HB1 NYG"}="HB12 NYG"; $resn_exch{"1HN  NYG"}=" HN1 NYG"; $resn_exch{"2HN  NYG"}=" HN2 NYG";
$resn_exch{"1HA3 NYG"}="HA31 NYG"; $resn_exch{"2HA3 NYG"}="HA32 NYG"; $resn_exch{"1H13 NYL"}="H131 NYL";
$resn_exch{"2H13 NYL"}="H132 NYL"; $resn_exch{"1H16 NYL"}="H161 NYL"; $resn_exch{"2H16 NYL"}="H162 NYL";
$resn_exch{"1HC6 NYM"}="HC61 NYM"; $resn_exch{"1HN3 NYM"}="HN31 NYM"; $resn_exch{"1HC7 NYM"}="HC71 NYM";
$resn_exch{"2HC7 NYM"}="HC72 NYM"; $resn_exch{"3HC7 NYM"}="HC73 NYM"; $resn_exch{"1H3* NYM"}="H3'1 NYM";
$resn_exch{"2H3* NYM"}="H3'2 NYM"; $resn_exch{"1H1* NYM"}="H1'1 NYM"; $resn_exch{"1H2* NYM"}="H2'1 NYM";
$resn_exch{"2H2* NYM"}="H2'2 NYM"; $resn_exch{"1HC3 NYM"}="HC31 NYM"; $resn_exch{"1H4* NYM"}="H4'1 NYM";
$resn_exch{"1H5* NYM"}="H5'1 NYM"; $resn_exch{"2H5* NYM"}="H5'2 NYM"; $resn_exch{"1HP3 NYM"}="HP31 NYM";
$resn_exch{"1HP2 NYM"}="HP21 NYM"; $resn_exch{"1H8N NYP"}="H8N1 NYP"; $resn_exch{"2H8N NYP"}="H8N2 NYP";
$resn_exch{"3H8N NYP"}="H8N3 NYP"; $resn_exch{"1H11 NYP"}="H111 NYP"; $resn_exch{"1HC7 NYP"}="HC71 NYP";
$resn_exch{"2HC7 NYP"}="HC72 NYP"; $resn_exch{"1H2  NYP"}=" H21 NYP"; $resn_exch{"1H3  NYP"}=" H31 NYP";
$resn_exch{"1H6  NYP"}=" H61 NYP"; $resn_exch{"1HAQ NZA"}="HAQ1 NZA"; $resn_exch{"2HAQ NZA"}="HAQ2 NZA";
$resn_exch{"1HN  NZH"}=" H   NZH"; $resn_exch{"2HN  NZH"}=" H2  NZH"; $resn_exch{"1HB  NZH"}=" HB2 NZH";
$resn_exch{"2HB  NZH"}=" HB3 NZH"; $resn_exch{" H   NZH"}=" HC  NZH"; $resn_exch{"1HA' NZQ"}="HA'1 NZQ";
$resn_exch{"2HA' NZQ"}="HA'2 NZQ"; $resn_exch{"1HAA NZQ"}="HAA1 NZQ"; $resn_exch{"2HAA NZQ"}="HAA2 NZQ";
$resn_exch{"1HN' NZQ"}="HN'1 NZQ"; $resn_exch{"2HN' NZQ"}="HN'2 NZQ"; $resn_exch{"1HNB NZQ"}="HNB1 NZQ";
$resn_exch{"2HNB NZQ"}="HNB2 NZQ"; $resn_exch{"1HNA NZQ"}="HNA1 NZQ"; $resn_exch{"2HNA NZQ"}="HNA2 NZQ";
$resn_exch{"1H4  O1C"}=" H41 O1C"; $resn_exch{"2H4  O1C"}=" H42 O1C"; $resn_exch{"1H12 O1C"}="H121 O1C";
$resn_exch{"2H12 O1C"}="H122 O1C"; $resn_exch{"1H11 O1C"}="H111 O1C"; $resn_exch{"2H11 O1C"}="H112 O1C";
$resn_exch{"1H9  O1C"}=" H91 O1C"; $resn_exch{"2H9  O1C"}=" H92 O1C"; $resn_exch{"1H15 O1C"}="H151 O1C";
$resn_exch{"2H15 O1C"}="H152 O1C"; $resn_exch{"1H16 O1C"}="H161 O1C"; $resn_exch{"2H16 O1C"}="H162 O1C";
$resn_exch{"1H19 O1C"}="H191 O1C"; $resn_exch{"2H19 O1C"}="H192 O1C"; $resn_exch{"1H22 O1C"}="H221 O1C";
$resn_exch{"2H22 O1C"}="H222 O1C"; $resn_exch{"1H18 O1C"}="H181 O1C"; $resn_exch{"2H18 O1C"}="H182 O1C";
$resn_exch{"3H18 O1C"}="H183 O1C"; $resn_exch{"1H21 O1C"}="H211 O1C"; $resn_exch{"2H21 O1C"}="H212 O1C";
$resn_exch{"3H21 O1C"}="H213 O1C"; $resn_exch{"1H23 O1C"}="H231 O1C"; $resn_exch{"2H23 O1C"}="H232 O1C";
$resn_exch{"1H24 O1C"}="H241 O1C"; $resn_exch{"2H24 O1C"}="H242 O1C"; $resn_exch{"1H27 O1C"}="H271 O1C";
$resn_exch{"2H27 O1C"}="H272 O1C"; $resn_exch{"3H27 O1C"}="H273 O1C"; $resn_exch{"1H26 O1C"}="H261 O1C";
$resn_exch{"2H26 O1C"}="H262 O1C"; $resn_exch{"3H26 O1C"}="H263 O1C"; $resn_exch{"1H28 O1C"}="H281 O1C";
$resn_exch{"2H28 O1C"}="H282 O1C"; $resn_exch{"1H29 O1C"}="H291 O1C"; $resn_exch{"2H29 O1C"}="H292 O1C";
$resn_exch{"1H30 O1C"}="H301 O1C"; $resn_exch{"2H30 O1C"}="H302 O1C"; $resn_exch{"2HOP O2C"}="HOP2 O2C";
$resn_exch{"1H5* O2C"}="H5'1 O2C"; $resn_exch{"2H5* O2C"}="H5'2 O2C"; $resn_exch{" H4* O2C"}=" H4' O2C";
$resn_exch{" H1* O2C"}=" H1' O2C"; $resn_exch{"1HN4 O2C"}="HN41 O2C"; $resn_exch{"2HN4 O2C"}="HN42 O2C";
$resn_exch{" H2* O2C"}=" H2' O2C"; $resn_exch{"2HO* O2C"}="HO'2 O2C"; $resn_exch{"1H3* O2C"}="H3'1 O2C";
$resn_exch{"2H3* O2C"}="H3'2 O2C"; $resn_exch{"3HOP O2C"}="HOP3 O2C"; $resn_exch{"1H7  O33"}=" H71 O33";
$resn_exch{"2H7  O33"}=" H72 O33"; $resn_exch{"1H14 O33"}="H141 O33"; $resn_exch{"2H14 O33"}="H142 O33";
$resn_exch{"1H18 O33"}="H181 O33"; $resn_exch{"2H18 O33"}="H182 O33"; $resn_exch{"3H18 O33"}="H183 O33";
$resn_exch{"1H19 O33"}="H191 O33"; $resn_exch{"2H19 O33"}="H192 O33"; $resn_exch{"3H19 O33"}="H193 O33";
$resn_exch{"1H20 O33"}="H201 O33"; $resn_exch{"2H20 O33"}="H202 O33"; $resn_exch{"1H21 O33"}="H211 O33";
$resn_exch{"2H21 O33"}="H212 O33"; $resn_exch{"1H22 O33"}="H221 O33"; $resn_exch{"2H22 O33"}="H222 O33";
$resn_exch{"3H22 O33"}="H223 O33"; $resn_exch{"1H35 O33"}="H351 O33"; $resn_exch{"2H35 O33"}="H352 O33";
$resn_exch{"1HO4 O4M"}="HO41 O4M"; $resn_exch{"2HO4 O4M"}="HO42 O4M"; $resn_exch{"1HO3 O4M"}="HO31 O4M";
$resn_exch{"2HO3 O4M"}="HO32 O4M"; $resn_exch{"1HO2 O4M"}="HO21 O4M"; $resn_exch{"2HO2 O4M"}="HO22 O4M";
$resn_exch{"1HO1 O4M"}="HO11 O4M"; $resn_exch{"2HO1 O4M"}="HO12 O4M"; $resn_exch{"1H2  OAA"}=" H21 OAA";
$resn_exch{"2H2  OAA"}=" H22 OAA"; $resn_exch{"1HB1 OAC"}="HB11 OAC"; $resn_exch{"2HB1 OAC"}="HB12 OAC";
$resn_exch{"1HB2 OAC"}="HB21 OAC"; $resn_exch{"2HB2 OAC"}="HB22 OAC"; $resn_exch{"3HB2 OAC"}="HB23 OAC";
$resn_exch{"2HOA OAD"}="HOA2 OAD"; $resn_exch{"1H5* OAD"}="H5'1 OAD"; $resn_exch{"2H5* OAD"}="H5'2 OAD";
$resn_exch{"4H*  OAD"}=" H'4 OAD"; $resn_exch{"3H*  OAD"}=" H'3 OAD"; $resn_exch{"3HO* OAD"}="HO'3 OAD";
$resn_exch{"2H*  OAD"}=" H'2 OAD"; $resn_exch{"2HO* OAD"}="HO'2 OAD"; $resn_exch{"1H*  OAD"}=" H'1 OAD";
$resn_exch{"1H6  OAD"}=" H61 OAD"; $resn_exch{"2H6  OAD"}=" H62 OAD"; $resn_exch{"2HOB OAD"}="HOB2 OAD";
$resn_exch{"1H5R OAD"}="H5R1 OAD"; $resn_exch{"2H5R OAD"}="H5R2 OAD"; $resn_exch{"4HR* OAD"}="HR'4 OAD";
$resn_exch{"3HR* OAD"}="HR'3 OAD"; $resn_exch{"3HOR OAD"}="HOR3 OAD"; $resn_exch{"2HR* OAD"}="HR'2 OAD";
$resn_exch{"1HR* OAD"}="HR'1 OAD"; $resn_exch{"1HOR OAD"}="HOR1 OAD"; $resn_exch{"1H7R OAD"}="H7R1 OAD";
$resn_exch{"2H7R OAD"}="H7R2 OAD"; $resn_exch{"3H7R OAD"}="H7R3 OAD"; $resn_exch{"1H15 OAL"}="H151 OAL";
$resn_exch{"2H15 OAL"}="H152 OAL"; $resn_exch{"1HAH OAN"}="HAH1 OAN"; $resn_exch{"2HAH OAN"}="HAH2 OAN";
$resn_exch{"3HAH OAN"}="HAH3 OAN"; $resn_exch{"1HAF OAN"}="HAF1 OAN"; $resn_exch{"2HAF OAN"}="HAF2 OAN";
$resn_exch{"1H1  OAP"}=" H11 OAP"; $resn_exch{"2H1  OAP"}="H12A OAP"; $resn_exch{"1H3  OAP"}=" H31 OAP";
$resn_exch{"2H3  OAP"}=" H32 OAP"; $resn_exch{"1H31 OAP"}="H311 OAP"; $resn_exch{"2H31 OAP"}="H312 OAP";
$resn_exch{"1H17 OAP"}="H171 OAP"; $resn_exch{"2H17 OAP"}="H172 OAP"; $resn_exch{"1H22 OAP"}="H221 OAP";
$resn_exch{"2H22 OAP"}="H222 OAP"; $resn_exch{"1H23 OAP"}="H231 OAP"; $resn_exch{"2H23 OAP"}="H232 OAP";
$resn_exch{"1H24 OAP"}="H241 OAP"; $resn_exch{"2H24 OAP"}="H242 OAP"; $resn_exch{"1H25 OAP"}="H251 OAP";
$resn_exch{"2H25 OAP"}="H252 OAP"; $resn_exch{"1H26 OAP"}="H261 OAP"; $resn_exch{"2H26 OAP"}="H262 OAP";
$resn_exch{"1H27 OAP"}="H271 OAP"; $resn_exch{"2H27 OAP"}="H272 OAP"; $resn_exch{"1HN1 OAR"}="HN11 OAR";
$resn_exch{"2HN1 OAR"}="HN12 OAR"; $resn_exch{"1HN4 OAR"}="HN41 OAR"; $resn_exch{"2HN4 OAR"}="HN42 OAR";
$resn_exch{"1H2  OAR"}=" H21 OAR"; $resn_exch{"2H2  OAR"}=" H22 OAR"; $resn_exch{"1H3  OAR"}=" H31 OAR";
$resn_exch{"2H3  OAR"}=" H32 OAR"; $resn_exch{"1H4  OAR"}=" H41 OAR"; $resn_exch{"2H4  OAR"}=" H42 OAR";
$resn_exch{"1H6  OAR"}=" H61 OAR"; $resn_exch{"2H6  OAR"}=" H62 OAR"; $resn_exch{" HN2 OAS"}=" H2  OAS";
$resn_exch{"1HB  OAS"}=" HB2 OAS"; $resn_exch{"2HB  OAS"}=" HB3 OAS"; $resn_exch{"1HC2 OAS"}="HC21 OAS";
$resn_exch{"2HC2 OAS"}="HC22 OAS"; $resn_exch{"3HC2 OAS"}="HC23 OAS"; $resn_exch{"1H4  OBE"}=" H41 OBE";
$resn_exch{"2H4  OBE"}=" H42 OBE"; $resn_exch{"1H6  OBE"}=" H61 OBE"; $resn_exch{"2H6  OBE"}=" H62 OBE";
$resn_exch{"1H7  OBE"}=" H71 OBE"; $resn_exch{"2H7  OBE"}=" H72 OBE"; $resn_exch{"1H16 OBE"}="H161 OBE";
$resn_exch{"2H16 OBE"}="H162 OBE"; $resn_exch{"3H16 OBE"}="H163 OBE"; $resn_exch{"1H8  OBI"}=" H81 OBI";
$resn_exch{"2H8  OBI"}=" H82 OBI"; $resn_exch{"1H7  OBI"}=" H71 OBI"; $resn_exch{"2H7  OBI"}=" H72 OBI";
$resn_exch{"1H2  OBN"}=" H21 OBN"; $resn_exch{"2H2  OBN"}="H22A OBN"; $resn_exch{"1H4  OBN"}=" H41 OBN";
$resn_exch{"2H4  OBN"}=" H42 OBN"; $resn_exch{"1H6  OBN"}=" H61 OBN"; $resn_exch{"2H6  OBN"}=" H62 OBN";
$resn_exch{"1H7  OBN"}=" H71 OBN"; $resn_exch{"2H7  OBN"}=" H72 OBN"; $resn_exch{"1H12 OBN"}="H121 OBN";
$resn_exch{"2H12 OBN"}="H122 OBN"; $resn_exch{"1H15 OBN"}="H151 OBN"; $resn_exch{"2H15 OBN"}="H152 OBN";
$resn_exch{"1H16 OBN"}="H161 OBN"; $resn_exch{"2H16 OBN"}="H162 OBN"; $resn_exch{"1H18 OBN"}="H181 OBN";
$resn_exch{"2H18 OBN"}="H182 OBN"; $resn_exch{"3H18 OBN"}="H183 OBN"; $resn_exch{"1H19 OBN"}="H191 OBN";
$resn_exch{"2H19 OBN"}="H192 OBN"; $resn_exch{"1H21 OBN"}="H211 OBN"; $resn_exch{"2H21 OBN"}="H212 OBN";
$resn_exch{"1H6' OBN"}="H6'1 OBN"; $resn_exch{"2H6' OBN"}="H6'2 OBN"; $resn_exch{"3H6' OBN"}="H6'3 OBN";
$resn_exch{"'HO2 OBN"}="HO2' OBN"; $resn_exch{"'HO3 OBN"}="HO3' OBN"; $resn_exch{"'HO4 OBN"}="HO4' OBN";
$resn_exch{"1H10 OBS"}="H101 OBS"; $resn_exch{"2H10 OBS"}="H102 OBS"; $resn_exch{"1H9C OBS"}="H9C1 OBS";
$resn_exch{"2H9C OBS"}="H9C2 OBS"; $resn_exch{"1H7C OBS"}="H7C1 OBS"; $resn_exch{"2H7C OBS"}="H7C2 OBS";
$resn_exch{"1H5C OBS"}="H5C1 OBS"; $resn_exch{"2H5C OBS"}="H5C2 OBS"; $resn_exch{"1H2C OBS"}="H2C1 OBS";
$resn_exch{"2H2C OBS"}="H2C2 OBS"; $resn_exch{"1HEC OBS"}="HEC1 OBS"; $resn_exch{"2HEC OBS"}="HEC2 OBS";
$resn_exch{"1HDC OBS"}="HDC1 OBS"; $resn_exch{"2HDC OBS"}="HDC2 OBS"; $resn_exch{"1HGC OBS"}="HGC1 OBS";
$resn_exch{"2HGC OBS"}="HGC2 OBS"; $resn_exch{"1HBC OBS"}="HBC1 OBS"; $resn_exch{"2HBC OBS"}="HBC2 OBS";
$resn_exch{"1HN  OBS"}=" HN1 OBS"; $resn_exch{"2HN  OBS"}=" HN2 OBS"; $resn_exch{"1HO1 OC1"}="HO11 OC1";
$resn_exch{"2HO1 OC1"}="HO12 OC1"; $resn_exch{"1HO1 OC2"}="HO11 OC2"; $resn_exch{"2HO1 OC2"}="HO12 OC2";
$resn_exch{"3HO1 OC2"}="HO13 OC2"; $resn_exch{"4HO1 OC2"}="HO14 OC2"; $resn_exch{"1HO1 OC3"}="HO11 OC3";
$resn_exch{"2HO1 OC3"}="HO12 OC3"; $resn_exch{"1HO2 OC3"}="HO21 OC3"; $resn_exch{"2HO2 OC3"}="HO22 OC3";
$resn_exch{"1HO3 OC3"}="HO31 OC3"; $resn_exch{"2HO3 OC3"}="HO32 OC3"; $resn_exch{"1HO1 OC4"}="HO11 OC4";
$resn_exch{"2HO1 OC4"}="HO12 OC4"; $resn_exch{"1HO2 OC4"}="HO21 OC4"; $resn_exch{"2HO2 OC4"}="HO22 OC4";
$resn_exch{"1HO3 OC4"}="HO31 OC4"; $resn_exch{"2HO3 OC4"}="HO32 OC4"; $resn_exch{"1HO4 OC4"}="HO41 OC4";
$resn_exch{"2HO4 OC4"}="HO42 OC4"; $resn_exch{"1HO1 OC5"}="HO11 OC5"; $resn_exch{"2HO1 OC5"}="HO12 OC5";
$resn_exch{"1HO2 OC5"}="HO21 OC5"; $resn_exch{"2HO2 OC5"}="HO22 OC5"; $resn_exch{"1HO3 OC5"}="HO31 OC5";
$resn_exch{"2HO3 OC5"}="HO32 OC5"; $resn_exch{"1HO4 OC5"}="HO41 OC5"; $resn_exch{"2HO4 OC5"}="HO42 OC5";
$resn_exch{"1HO5 OC5"}="HO51 OC5"; $resn_exch{"2HO5 OC5"}="HO52 OC5"; $resn_exch{"1HO1 OC6"}="HO11 OC6";
$resn_exch{"2HO1 OC6"}="HO12 OC6"; $resn_exch{"1HO2 OC6"}="HO21 OC6"; $resn_exch{"2HO2 OC6"}="HO22 OC6";
$resn_exch{"1HO3 OC6"}="HO31 OC6"; $resn_exch{"2HO3 OC6"}="HO32 OC6"; $resn_exch{"1HO4 OC6"}="HO41 OC6";
$resn_exch{"2HO4 OC6"}="HO42 OC6"; $resn_exch{"1HO5 OC6"}="HO51 OC6"; $resn_exch{"2HO5 OC6"}="HO52 OC6";
$resn_exch{"1HO6 OC6"}="HO61 OC6"; $resn_exch{"2HO6 OC6"}="HO62 OC6"; $resn_exch{"1HO1 OC7"}="HO11 OC7";
$resn_exch{"2HO1 OC7"}="HO12 OC7"; $resn_exch{"1HO2 OC7"}="HO21 OC7"; $resn_exch{"2HO2 OC7"}="HO22 OC7";
$resn_exch{"1HO3 OC7"}="HO31 OC7"; $resn_exch{"2HO3 OC7"}="HO32 OC7"; $resn_exch{"1HO4 OC7"}="HO41 OC7";
$resn_exch{"2HO4 OC7"}="HO42 OC7"; $resn_exch{"1HO5 OC7"}="HO51 OC7"; $resn_exch{"2HO5 OC7"}="HO52 OC7";
$resn_exch{"1HO6 OC7"}="HO61 OC7"; $resn_exch{"2HO6 OC7"}="HO62 OC7"; $resn_exch{"1HO7 OC7"}="HO71 OC7";
$resn_exch{"2HO7 OC7"}="HO72 OC7"; $resn_exch{"1HO1 OC8"}="HO11 OC8"; $resn_exch{"2HO1 OC8"}="HO12 OC8";
$resn_exch{"1HO2 OC8"}="HO21 OC8"; $resn_exch{"2HO2 OC8"}="HO22 OC8"; $resn_exch{"1HO3 OC8"}="HO31 OC8";
$resn_exch{"2HO3 OC8"}="HO32 OC8"; $resn_exch{"1HO4 OC8"}="HO41 OC8"; $resn_exch{"2HO4 OC8"}="HO42 OC8";
$resn_exch{"1HO5 OC8"}="HO51 OC8"; $resn_exch{"2HO5 OC8"}="HO52 OC8"; $resn_exch{"1HO6 OC8"}="HO61 OC8";
$resn_exch{"2HO6 OC8"}="HO62 OC8"; $resn_exch{"1HO7 OC8"}="HO71 OC8"; $resn_exch{"2HO7 OC8"}="HO72 OC8";
$resn_exch{"1HO8 OC8"}="HO81 OC8"; $resn_exch{"2HO8 OC8"}="HO82 OC8"; $resn_exch{"1HAA OC9"}="HAA1 OC9";
$resn_exch{"2HAA OC9"}="HAA2 OC9"; $resn_exch{"3HAA OC9"}="HAA3 OC9"; $resn_exch{"1HAC OC9"}="HAC1 OC9";
$resn_exch{"2HAC OC9"}="HAC2 OC9"; $resn_exch{"1HAE OC9"}="HAE1 OC9"; $resn_exch{"2HAE OC9"}="HAE2 OC9";
$resn_exch{"1HAG OC9"}="HAG1 OC9"; $resn_exch{"2HAG OC9"}="HAG2 OC9"; $resn_exch{"1HAI OC9"}="HAI1 OC9";
$resn_exch{"2HAI OC9"}="HAI2 OC9"; $resn_exch{"1HAH OC9"}="HAH1 OC9"; $resn_exch{"2HAH OC9"}="HAH2 OC9";
$resn_exch{"1HAF OC9"}="HAF1 OC9"; $resn_exch{"2HAF OC9"}="HAF2 OC9"; $resn_exch{"1HAD OC9"}="HAD1 OC9";
$resn_exch{"2HAD OC9"}="HAD2 OC9"; $resn_exch{"1H2  OCA"}=" H21 OCA"; $resn_exch{"2H2  OCA"}=" H22 OCA";
$resn_exch{"1H3  OCA"}=" H31 OCA"; $resn_exch{"2H3  OCA"}=" H32 OCA"; $resn_exch{"1H4  OCA"}=" H41 OCA";
$resn_exch{"2H4  OCA"}=" H42 OCA"; $resn_exch{"1H5  OCA"}=" H51 OCA"; $resn_exch{"2H5  OCA"}=" H52 OCA";
$resn_exch{"1H6  OCA"}=" H61 OCA"; $resn_exch{"2H6  OCA"}=" H62 OCA"; $resn_exch{"1H7  OCA"}=" H71 OCA";
$resn_exch{"2H7  OCA"}=" H72 OCA"; $resn_exch{"1H8  OCA"}=" H81 OCA"; $resn_exch{"2H8  OCA"}=" H82 OCA";
$resn_exch{"3H8  OCA"}=" H83 OCA"; $resn_exch{"1HAA OCB"}="HAA1 OCB"; $resn_exch{"2HAA OCB"}="HAA2 OCB";
$resn_exch{"3HAA OCB"}="HAA3 OCB"; $resn_exch{"1HAB OCB"}="HAB1 OCB"; $resn_exch{"2HAB OCB"}="HAB2 OCB";
$resn_exch{"1HAC OCB"}="HAC1 OCB"; $resn_exch{"2HAC OCB"}="HAC2 OCB"; $resn_exch{"1HAD OCB"}="HAD1 OCB";
$resn_exch{"2HAD OCB"}="HAD2 OCB"; $resn_exch{"1HAE OCB"}="HAE1 OCB"; $resn_exch{"2HAE OCB"}="HAE2 OCB";
$resn_exch{"1HAF OCB"}="HAF1 OCB"; $resn_exch{"2HAF OCB"}="HAF2 OCB"; $resn_exch{"1HAG OCB"}="HAG1 OCB";
$resn_exch{"2HAG OCB"}="HAG2 OCB"; $resn_exch{"1HAQ OCB"}="HAQ1 OCB"; $resn_exch{"2HAQ OCB"}="HAQ2 OCB";
$resn_exch{"1HAO OCB"}="HAO1 OCB"; $resn_exch{"2HAO OCB"}="HAO2 OCB"; $resn_exch{"1HAT OCB"}="HAT1 OCB";
$resn_exch{"2HAT OCB"}="HAT2 OCB"; $resn_exch{"3HAT OCB"}="HAT3 OCB"; $resn_exch{"1HAJ OCB"}="HAJ1 OCB";
$resn_exch{"2HAJ OCB"}="HAJ2 OCB"; $resn_exch{"3HAJ OCB"}="HAJ3 OCB"; $resn_exch{"1HAM OCB"}="HAM1 OCB";
$resn_exch{"2HAM OCB"}="HAM2 OCB"; $resn_exch{"3HAM OCB"}="HAM3 OCB"; $resn_exch{"1H4  OCC"}=" H41 OCC";
$resn_exch{"2H4  OCC"}=" H42 OCC"; $resn_exch{"1H12 OCC"}="H121 OCC"; $resn_exch{"2H12 OCC"}="H122 OCC";
$resn_exch{"1H11 OCC"}="H111 OCC"; $resn_exch{"2H11 OCC"}="H112 OCC"; $resn_exch{"1H9  OCC"}=" H91 OCC";
$resn_exch{"2H9  OCC"}=" H92 OCC"; $resn_exch{"1H15 OCC"}="H151 OCC"; $resn_exch{"2H15 OCC"}="H152 OCC";
$resn_exch{"1H16 OCC"}="H161 OCC"; $resn_exch{"2H16 OCC"}="H162 OCC"; $resn_exch{"1H19 OCC"}="H191 OCC";
$resn_exch{"2H19 OCC"}="H192 OCC"; $resn_exch{"1H22 OCC"}="H221 OCC"; $resn_exch{"2H22 OCC"}="H222 OCC";
$resn_exch{"1H18 OCC"}="H181 OCC"; $resn_exch{"2H18 OCC"}="H182 OCC"; $resn_exch{"3H18 OCC"}="H183 OCC";
$resn_exch{"1H21 OCC"}="H211 OCC"; $resn_exch{"2H21 OCC"}="H212 OCC"; $resn_exch{"3H21 OCC"}="H213 OCC";
$resn_exch{"1H23 OCC"}="H231 OCC"; $resn_exch{"2H23 OCC"}="H232 OCC"; $resn_exch{"1H24 OCC"}="H241 OCC";
$resn_exch{"2H24 OCC"}="H242 OCC"; $resn_exch{"1H27 OCC"}="H271 OCC"; $resn_exch{"2H27 OCC"}="H272 OCC";
$resn_exch{"3H27 OCC"}="H273 OCC"; $resn_exch{"1H26 OCC"}="H261 OCC"; $resn_exch{"2H26 OCC"}="H262 OCC";
$resn_exch{"3H26 OCC"}="H263 OCC"; $resn_exch{"1H29 OCC"}="H291 OCC"; $resn_exch{"2H29 OCC"}="H292 OCC";
$resn_exch{"1H30 OCC"}="H301 OCC"; $resn_exch{"2H30 OCC"}="H302 OCC"; $resn_exch{"1H31 OCC"}="H311 OCC";
$resn_exch{"2H31 OCC"}="H312 OCC"; $resn_exch{"1HO1 OCL"}="HO11 OCL"; $resn_exch{"2HO1 OCL"}="HO12 OCL";
$resn_exch{"1HO1 OCM"}="HO11 OCM"; $resn_exch{"2HO1 OCM"}="HO12 OCM"; $resn_exch{"1HO2 OCM"}="HO21 OCM";
$resn_exch{"2HO2 OCM"}="HO22 OCM"; $resn_exch{"1HO3 OCM"}="HO31 OCM"; $resn_exch{"2HO3 OCM"}="HO32 OCM";
$resn_exch{"1HO1 OCN"}="HO11 OCN"; $resn_exch{"2HO1 OCN"}="HO12 OCN"; $resn_exch{"3HO1 OCN"}="HO13 OCN";
$resn_exch{"4HO1 OCN"}="HO14 OCN"; $resn_exch{"1HO1 OCO"}="HO11 OCO"; $resn_exch{"2HO1 OCO"}="HO12 OCO";
$resn_exch{"1HO2 OCO"}="HO21 OCO"; $resn_exch{"2HO2 OCO"}="HO22 OCO"; $resn_exch{"1HO3 OCO"}="HO31 OCO";
$resn_exch{"2HO3 OCO"}="HO32 OCO"; $resn_exch{"1HO4 OCO"}="HO41 OCO"; $resn_exch{"2HO4 OCO"}="HO42 OCO";
$resn_exch{"1HO5 OCO"}="HO51 OCO"; $resn_exch{"2HO5 OCO"}="HO52 OCO"; $resn_exch{"1HO6 OCO"}="HO61 OCO";
$resn_exch{"2HO6 OCO"}="HO62 OCO"; $resn_exch{"1H1  OCP"}=" H11 OCP"; $resn_exch{"2H1  OCP"}=" H12 OCP";
$resn_exch{"1H3  OCP"}=" H31 OCP"; $resn_exch{"2H3  OCP"}=" H32 OCP"; $resn_exch{"1H4  OCP"}=" H41 OCP";
$resn_exch{"2H4  OCP"}=" H42 OCP"; $resn_exch{"1H5  OCP"}=" H51 OCP"; $resn_exch{"2H5  OCP"}=" H52 OCP";
$resn_exch{"1H6  OCP"}=" H61 OCP"; $resn_exch{"2H6  OCP"}=" H62 OCP"; $resn_exch{"1H7  OCP"}=" H71 OCP";
$resn_exch{"2H7  OCP"}=" H72 OCP"; $resn_exch{"1H8  OCP"}=" H81 OCP"; $resn_exch{"2H8  OCP"}=" H82 OCP";
$resn_exch{"1H9  OCP"}=" H91 OCP"; $resn_exch{"2H9  OCP"}=" H92 OCP"; $resn_exch{"1H10 OCP"}="H101 OCP";
$resn_exch{"2H10 OCP"}="H102 OCP"; $resn_exch{"1H11 OCP"}="H111 OCP"; $resn_exch{"2H11 OCP"}="H112 OCP";
$resn_exch{"3H11 OCP"}="H113 OCP"; $resn_exch{"1H13 OCP"}="H131 OCP"; $resn_exch{"2H13 OCP"}="H132 OCP";
$resn_exch{"1H14 OCP"}="H141 OCP"; $resn_exch{"2H14 OCP"}="H142 OCP"; $resn_exch{"1H15 OCP"}="H151 OCP";
$resn_exch{"2H15 OCP"}="H152 OCP"; $resn_exch{"1H16 OCP"}="H161 OCP"; $resn_exch{"2H16 OCP"}="H162 OCP";
$resn_exch{"1H17 OCP"}="H171 OCP"; $resn_exch{"2H17 OCP"}="H172 OCP"; $resn_exch{"1H18 OCP"}="H181 OCP";
$resn_exch{"2H18 OCP"}="H182 OCP"; $resn_exch{"1H19 OCP"}="H191 OCP"; $resn_exch{"2H19 OCP"}="H192 OCP";
$resn_exch{"1H20 OCP"}="H201 OCP"; $resn_exch{"2H20 OCP"}="H202 OCP"; $resn_exch{"3H20 OCP"}="H203 OCP";
$resn_exch{"1H23 OCP"}="H231 OCP"; $resn_exch{"2H23 OCP"}="H232 OCP"; $resn_exch{"1H24 OCP"}="H241 OCP";
$resn_exch{"2H24 OCP"}="H242 OCP"; $resn_exch{"1H25 OCP"}="H251 OCP"; $resn_exch{"2H25 OCP"}="H252 OCP";
$resn_exch{"1H26 OCP"}="H261 OCP"; $resn_exch{"2H26 OCP"}="H262 OCP"; $resn_exch{"1H27 OCP"}="H271 OCP";
$resn_exch{"2H27 OCP"}="H272 OCP"; $resn_exch{"1H28 OCP"}="H281 OCP"; $resn_exch{"2H28 OCP"}="H282 OCP";
$resn_exch{"1H29 OCP"}="H291 OCP"; $resn_exch{"2H29 OCP"}="H292 OCP"; $resn_exch{"1H30 OCP"}="H301 OCP";
$resn_exch{"2H30 OCP"}="H302 OCP"; $resn_exch{"3H30 OCP"}="H303 OCP"; $resn_exch{"1HMK OCQ"}="HMK1 OCQ";
$resn_exch{"2HMK OCQ"}="HMK2 OCQ"; $resn_exch{"1HN  OCQ"}=" HN1 OCQ"; $resn_exch{"2HN  OCQ"}=" HN2 OCQ";
$resn_exch{"1HB  OCQ"}=" HB1 OCQ"; $resn_exch{"2HB  OCQ"}=" HB2 OCQ"; $resn_exch{"1HAE OCQ"}="HAE1 OCQ";
$resn_exch{"2HAE OCQ"}="HAE2 OCQ"; $resn_exch{"1HD1 OCQ"}="HD11 OCQ"; $resn_exch{"2HD1 OCQ"}="HD12 OCQ";
$resn_exch{"1HN  OCS"}=" H   OCS"; $resn_exch{"2HN  OCS"}=" HN2 OCS"; $resn_exch{"1HB  OCS"}=" HB2 OCS";
$resn_exch{"2HB  OCS"}=" HB3 OCS"; $resn_exch{"1H1  OCT"}=" H11 OCT"; $resn_exch{"2H1  OCT"}=" H12 OCT";
$resn_exch{"3H1  OCT"}=" H13 OCT"; $resn_exch{"1H2  OCT"}=" H21 OCT"; $resn_exch{"2H2  OCT"}=" H22 OCT";
$resn_exch{"1H3  OCT"}=" H31 OCT"; $resn_exch{"2H3  OCT"}=" H32 OCT"; $resn_exch{"1H4  OCT"}=" H41 OCT";
$resn_exch{"2H4  OCT"}=" H42 OCT"; $resn_exch{"1H5  OCT"}=" H51 OCT"; $resn_exch{"2H5  OCT"}=" H52 OCT";
$resn_exch{"1H6  OCT"}=" H61 OCT"; $resn_exch{"2H6  OCT"}=" H62 OCT"; $resn_exch{"1H7  OCT"}=" H71 OCT";
$resn_exch{"2H7  OCT"}=" H72 OCT"; $resn_exch{"1H8  OCT"}=" H81 OCT"; $resn_exch{"2H8  OCT"}=" H82 OCT";
$resn_exch{"3H8  OCT"}=" H83 OCT"; $resn_exch{"2HC  OCV"}=" HC2 OCV"; $resn_exch{"1HC3 OCV"}="HC31 OCV";
$resn_exch{"2HC3 OCV"}="HC32 OCV"; $resn_exch{"1HC4 OCV"}="HC41 OCV"; $resn_exch{"2HC4 OCV"}="HC42 OCV";
$resn_exch{"1HC7 OCV"}="HC71 OCV"; $resn_exch{"2HC7 OCV"}="HC72 OCV"; $resn_exch{"1HN1 OCV"}="HN11 OCV";
$resn_exch{"1HC2 OCV"}="HC21 OCV"; $resn_exch{"1H14 OCV"}="H141 OCV"; $resn_exch{"2H14 OCV"}="H142 OCV";
$resn_exch{"1H16 OCV"}="H161 OCV"; $resn_exch{"2H16 OCV"}="H162 OCV"; $resn_exch{"17HS OCV"}="HS17 OCV";
$resn_exch{"1HO9 OCV"}="HO91 OCV"; $resn_exch{"3HC0 OCV"}="HC03 OCV"; $resn_exch{"3HC2 OCV"}="HC23 OCV";
$resn_exch{"1H33 OCV"}="H331 OCV"; $resn_exch{"2H33 OCV"}="H332 OCV"; $resn_exch{"3H33 OCV"}="H333 OCV";
$resn_exch{"1H37 OCV"}="H371 OCV"; $resn_exch{"2H37 OCV"}="H372 OCV"; $resn_exch{"3H37 OCV"}="H373 OCV";
$resn_exch{"4HO3 OCV"}="HO34 OCV"; $resn_exch{"1HN  OCY"}=" H   OCY"; $resn_exch{"2HN  OCY"}=" H2  OCY";
$resn_exch{"1HB  OCY"}=" HB2 OCY"; $resn_exch{"2HB  OCY"}=" HB3 OCY"; $resn_exch{"1HD  OCY"}=" HD2 OCY";
$resn_exch{"2HD  OCY"}=" HD3 OCY"; $resn_exch{"1HE  OCY"}=" HE2 OCY"; $resn_exch{"2HE  OCY"}=" HE3 OCY";
$resn_exch{" HOZ OCY"}=" HZ  OCY"; $resn_exch{"1HN  ODA"}=" H   ODA"; $resn_exch{"2HN  ODA"}=" H2  ODA";
$resn_exch{"1HA  ODA"}=" HA1 ODA"; $resn_exch{"1HB  ODA"}=" HB2 ODA"; $resn_exch{"2HB  ODA"}=" HB3 ODA";
$resn_exch{"1HG  ODA"}=" HG2 ODA"; $resn_exch{"2HG  ODA"}=" HG3 ODA"; $resn_exch{"1HA2 ODA"}=" HA2 ODA";
$resn_exch{"1HB2 ODA"}="HB23 ODA"; $resn_exch{"2HB2 ODA"}="HB22 ODA"; $resn_exch{"1HG2 ODA"}="HG23 ODA";
$resn_exch{"2HG2 ODA"}="HG22 ODA"; $resn_exch{"1HD2 ODA"}="HD22 ODA"; $resn_exch{"2HD2 ODA"}="HD23 ODA";
$resn_exch{"1H2  ODD"}=" H21 ODD"; $resn_exch{"2H2  ODD"}=" H22 ODD"; $resn_exch{"1H3  ODD"}=" H31 ODD";
$resn_exch{"2H3  ODD"}=" H32 ODD"; $resn_exch{"1H4  ODD"}=" H41 ODD"; $resn_exch{"2H4  ODD"}=" H42 ODD";
$resn_exch{"1H5  ODD"}=" H51 ODD"; $resn_exch{"2H5  ODD"}=" H52 ODD"; $resn_exch{"1H6  ODD"}=" H61 ODD";
$resn_exch{"2H6  ODD"}=" H62 ODD"; $resn_exch{"1H7  ODD"}=" H71 ODD"; $resn_exch{"2H7  ODD"}=" H72 ODD";
$resn_exch{"1H8  ODD"}=" H81 ODD"; $resn_exch{"2H8  ODD"}=" H82 ODD"; $resn_exch{"1H9  ODD"}=" H91 ODD";
$resn_exch{"2H9  ODD"}=" H92 ODD"; $resn_exch{"1H14 ODD"}="H141 ODD"; $resn_exch{"2H14 ODD"}="H142 ODD";
$resn_exch{"1H15 ODD"}="H151 ODD"; $resn_exch{"2H15 ODD"}="H152 ODD"; $resn_exch{"1H16 ODD"}="H161 ODD";
$resn_exch{"2H16 ODD"}="H162 ODD"; $resn_exch{"1H17 ODD"}="H171 ODD"; $resn_exch{"2H17 ODD"}="H172 ODD";
$resn_exch{"1H18 ODD"}="H181 ODD"; $resn_exch{"2H18 ODD"}="H182 ODD"; $resn_exch{"3H18 ODD"}="H183 ODD";
$resn_exch{"1HAJ ODE"}="HAJ1 ODE"; $resn_exch{"2HAJ ODE"}="HAJ2 ODE"; $resn_exch{"1HAU ODE"}="HAU1 ODE";
$resn_exch{"2HAU ODE"}="HAU2 ODE"; $resn_exch{"3HAU ODE"}="HAU3 ODE"; $resn_exch{"1HAS ODE"}="HAS1 ODE";
$resn_exch{"2HAS ODE"}="HAS2 ODE"; $resn_exch{"1HAG ODE"}="HAG1 ODE"; $resn_exch{"2HAG ODE"}="HAG2 ODE";
$resn_exch{"3HAG ODE"}="HAG3 ODE"; $resn_exch{"1HAC ODI"}="HAC1 ODI"; $resn_exch{"2HAC ODI"}="HAC2 ODI";
$resn_exch{"1HAE ODI"}="HAE1 ODI"; $resn_exch{"2HAE ODI"}="HAE2 ODI"; $resn_exch{"1HAG ODI"}="HAG1 ODI";
$resn_exch{"2HAG ODI"}="HAG2 ODI"; $resn_exch{"1HAI ODI"}="HAI1 ODI"; $resn_exch{"2HAI ODI"}="HAI2 ODI";
$resn_exch{"1HAJ ODI"}="HAJ1 ODI"; $resn_exch{"2HAJ ODI"}="HAJ2 ODI"; $resn_exch{"1HAH ODI"}="HAH1 ODI";
$resn_exch{"2HAH ODI"}="HAH2 ODI"; $resn_exch{"1HAF ODI"}="HAF1 ODI"; $resn_exch{"2HAF ODI"}="HAF2 ODI";
$resn_exch{"1HAD ODI"}="HAD1 ODI"; $resn_exch{"2HAD ODI"}="HAD2 ODI"; $resn_exch{"2HOP ODP"}="HOP2 ODP";
$resn_exch{"3HOP ODP"}="HOP3 ODP"; $resn_exch{"AH2* ODP"}=" H2B ODP"; $resn_exch{"AH3* ODP"}=" H3B ODP";
$resn_exch{"AHO3 ODP"}="HO3A ODP"; $resn_exch{"AH1* ODP"}=" H1B ODP"; $resn_exch{" AH8 ODP"}=" H8A ODP";
$resn_exch{" AH2 ODP"}=" H2A ODP"; $resn_exch{"AH62 ODP"}="H62A ODP"; $resn_exch{"AH61 ODP"}="H61A ODP";
$resn_exch{"AH4* ODP"}=" H4B ODP"; $resn_exch{"AH51 ODP"}="H51A ODP"; $resn_exch{"AH52 ODP"}="H52A ODP";
$resn_exch{"2HOA ODP"}="HOA2 ODP"; $resn_exch{"NH51 ODP"}="H51N ODP"; $resn_exch{"NH52 ODP"}="H52N ODP";
$resn_exch{"NH4* ODP"}=" H4D ODP"; $resn_exch{"NH1* ODP"}=" H1D ODP"; $resn_exch{"NH2* ODP"}=" H2D ODP";
$resn_exch{"NHO2 ODP"}="HO2N ODP"; $resn_exch{"NH3* ODP"}=" H3D ODP"; $resn_exch{"NHO3 ODP"}="HO3N ODP";
$resn_exch{" NH2 ODP"}=" H2N ODP"; $resn_exch{" NH3 ODP"}=" H3N ODP"; $resn_exch{"NH72 ODP"}="H72N ODP";
$resn_exch{"NH71 ODP"}="H71N ODP"; $resn_exch{" NH5 ODP"}=" H5N ODP"; $resn_exch{" NH6 ODP"}=" H6N ODP";
$resn_exch{"1H2  ODS"}=" H21 ODS"; $resn_exch{"2H2  ODS"}=" H22 ODS"; $resn_exch{"1H3  ODS"}=" H31 ODS";
$resn_exch{"2H3  ODS"}=" H32 ODS"; $resn_exch{"1HM  ODS"}=" HM1 ODS"; $resn_exch{"2HM  ODS"}=" HM2 ODS";
$resn_exch{"3HM  ODS"}=" HM3 ODS"; $resn_exch{"1H5  ODS"}=" H51 ODS"; $resn_exch{"2H5  ODS"}=" H52 ODS";
$resn_exch{"1H6  ODS"}=" H61 ODS"; $resn_exch{"2H6  ODS"}=" H62 ODS"; $resn_exch{"1H2  ODT"}=" H21 ODT";
$resn_exch{"2H2  ODT"}=" H22 ODT"; $resn_exch{"1H3  ODT"}=" H31 ODT"; $resn_exch{"2H3  ODT"}=" H32 ODT";
$resn_exch{"1H4  ODT"}=" H41 ODT"; $resn_exch{"2H4  ODT"}=" H42 ODT"; $resn_exch{"1H5  ODT"}=" H51 ODT";
$resn_exch{"2H5  ODT"}=" H52 ODT"; $resn_exch{"1H6  ODT"}=" H61 ODT"; $resn_exch{"2H6  ODT"}=" H62 ODT";
$resn_exch{"1H7  ODT"}=" H71 ODT"; $resn_exch{"2H7  ODT"}=" H72 ODT"; $resn_exch{"1H8  ODT"}=" H81 ODT";
$resn_exch{"2H8  ODT"}=" H82 ODT"; $resn_exch{"1H9  ODT"}=" H91 ODT"; $resn_exch{"2H9  ODT"}=" H92 ODT";
$resn_exch{"1H10 ODT"}="H101 ODT"; $resn_exch{"2H10 ODT"}="H102 ODT"; $resn_exch{"1H17 ODT"}="H171 ODT";
$resn_exch{"2H17 ODT"}="H172 ODT"; $resn_exch{"1H18 ODT"}="H181 ODT"; $resn_exch{"2H18 ODT"}="H182 ODT";
$resn_exch{"3H18 ODT"}="H183 ODT"; $resn_exch{"1H13 OEF"}="H131 OEF"; $resn_exch{"2H13 OEF"}="H132 OEF";
$resn_exch{"1H14 OEF"}="H141 OEF"; $resn_exch{"2H14 OEF"}="H142 OEF"; $resn_exch{"1H16 OEF"}="H161 OEF";
$resn_exch{"2H16 OEF"}="H162 OEF"; $resn_exch{"3H16 OEF"}="H163 OEF"; $resn_exch{"1H17 OEF"}="H171 OEF";
$resn_exch{"2H17 OEF"}="H172 OEF"; $resn_exch{"3H17 OEF"}="H173 OEF"; $resn_exch{"1HC1 OES"}="HC11 OES";
$resn_exch{"2HC1 OES"}="HC12 OES"; $resn_exch{"3HC1 OES"}="HC13 OES"; $resn_exch{"1HC2 OES"}="HC21 OES";
$resn_exch{"2HC2 OES"}="HC22 OES"; $resn_exch{"1HC3 OES"}="HC31 OES"; $resn_exch{"2HC3 OES"}="HC32 OES";
$resn_exch{"1HC4 OES"}="HC41 OES"; $resn_exch{"2HC4 OES"}="HC42 OES"; $resn_exch{"1HC5 OES"}="HC51 OES";
$resn_exch{"2HC5 OES"}="HC52 OES"; $resn_exch{"1HC6 OES"}="HC61 OES"; $resn_exch{"2HC6 OES"}="HC62 OES";
$resn_exch{"1HC7 OES"}="HC71 OES"; $resn_exch{"2HC7 OES"}="HC72 OES"; $resn_exch{"1HC8 OES"}="HC81 OES";
$resn_exch{"2HC8 OES"}="HC82 OES"; $resn_exch{"1H22 OES"}="H221 OES"; $resn_exch{"2H22 OES"}="H222 OES";
$resn_exch{"1H23 OES"}="H231 OES"; $resn_exch{"2H23 OES"}="H232 OES"; $resn_exch{"1HO3 OES"}="HO31 OES";
$resn_exch{"1H1  OET"}=" H11 OET"; $resn_exch{"2H1  OET"}=" H12 OET"; $resn_exch{"1H2  OET"}=" H21 OET";
$resn_exch{"2H2  OET"}=" H22 OET"; $resn_exch{"3H2  OET"}=" H23 OET"; $resn_exch{"1HO1 OF1"}="HO11 OF1";
$resn_exch{"2HO1 OF1"}="HO12 OF1"; $resn_exch{"1HO1 OF3"}="HO11 OF3"; $resn_exch{"2HO1 OF3"}="HO12 OF3";
$resn_exch{"1H17 OFF"}="H171 OFF"; $resn_exch{"1H16 OFF"}="H161 OFF"; $resn_exch{"1H13 OFF"}="H131 OFF";
$resn_exch{"2H13 OFF"}="H132 OFF"; $resn_exch{"1H8  OFF"}=" H81 OFF"; $resn_exch{"1H7  OFF"}=" H71 OFF";
$resn_exch{"1H4C OGA"}="H4C1 OGA"; $resn_exch{"2H4C OGA"}="H4C2 OGA"; $resn_exch{"1HNB OGG"}="HNB1 OGG";
$resn_exch{"2HNB OGG"}="HNB2 OGG"; $resn_exch{"1HD  OGG"}=" HD1 OGG"; $resn_exch{"2HD  OGG"}=" HD2 OGG";
$resn_exch{"1HF  OGG"}=" HF1 OGG"; $resn_exch{"2HF  OGG"}=" HF2 OGG"; $resn_exch{"1HG  OGG"}=" HG1 OGG";
$resn_exch{"2HG  OGG"}=" HG2 OGG"; $resn_exch{"1HI  OGG"}=" HI1 OGG"; $resn_exch{"2HI  OGG"}=" HI2 OGG";
$resn_exch{"1HNK OGG"}="HNK1 OGG"; $resn_exch{"2HNK OGG"}="HNK2 OGG"; $resn_exch{"1H15 OHA"}="H151 OHA";
$resn_exch{"2H15 OHA"}="H152 OHA"; $resn_exch{"1H14 OHA"}="H141 OHA"; $resn_exch{"2H14 OHA"}="H142 OHA";
$resn_exch{"1H13 OHA"}="H131 OHA"; $resn_exch{"2H13 OHA"}="H132 OHA"; $resn_exch{"1H12 OHA"}="H121 OHA";
$resn_exch{"2H12 OHA"}="H122 OHA"; $resn_exch{"1H11 OHA"}="H111 OHA"; $resn_exch{"2H11 OHA"}="H112 OHA";
$resn_exch{"1H10 OHA"}="H101 OHA"; $resn_exch{"2H10 OHA"}="H102 OHA"; $resn_exch{"1H9  OHA"}=" H91 OHA";
$resn_exch{"2H9  OHA"}=" H92 OHA"; $resn_exch{"1H8  OHA"}=" H81 OHA"; $resn_exch{"2H8  OHA"}=" H82 OHA";
$resn_exch{"1H6  OHA"}=" H61 OHA"; $resn_exch{"2H6  OHA"}=" H62 OHA"; $resn_exch{"1H5  OHA"}=" H51 OHA";
$resn_exch{"2H5  OHA"}=" H52 OHA"; $resn_exch{"1H4  OHA"}=" H41 OHA"; $resn_exch{"2H4  OHA"}=" H42 OHA";
$resn_exch{"1H3  OHA"}=" H31 OHA"; $resn_exch{"2H3  OHA"}=" H32 OHA"; $resn_exch{"1H2  OHA"}=" H21 OHA";
$resn_exch{"2H2  OHA"}=" H22 OHA"; $resn_exch{"1H1  OHA"}=" H11 OHA"; $resn_exch{"2H1  OHA"}=" H12 OHA";
$resn_exch{"3H1  OHA"}=" H13 OHA"; $resn_exch{"1H1  OHE"}=" H11 OHE"; $resn_exch{"2H1  OHE"}=" H12 OHE";
$resn_exch{"3H1  OHE"}=" H13 OHE"; $resn_exch{"1H2  OHE"}=" H21 OHE"; $resn_exch{"2H2  OHE"}=" H22 OHE";
$resn_exch{"1H20 OHH"}="H201 OHH"; $resn_exch{"2H20 OHH"}="H202 OHH"; $resn_exch{"3H20 OHH"}="H203 OHH";
$resn_exch{"1H24 OHH"}="H241 OHH"; $resn_exch{"2H24 OHH"}="H242 OHH"; $resn_exch{"1H23 OHH"}="H231 OHH";
$resn_exch{"2H23 OHH"}="H232 OHH"; $resn_exch{"1H22 OHH"}="H221 OHH"; $resn_exch{"2H22 OHH"}="H222 OHH";
$resn_exch{"1H25 OHH"}="H251 OHH"; $resn_exch{"2H25 OHH"}="H252 OHH"; $resn_exch{"1HB  OHI"}=" HB1 OHI";
$resn_exch{"2HB  OHI"}=" HB2 OHI"; $resn_exch{"1HN  OHI"}=" HN1 OHI"; $resn_exch{"2HN  OHI"}=" HN2 OHI";
$resn_exch{"1H21 OHM"}="H211 OHM"; $resn_exch{"2H21 OHM"}="H212 OHM"; $resn_exch{"3H21 OHM"}="H213 OHM";
$resn_exch{"1H20 OHM"}="H201 OHM"; $resn_exch{"2H20 OHM"}="H202 OHM"; $resn_exch{"1H19 OHM"}="H191 OHM";
$resn_exch{"2H19 OHM"}="H192 OHM"; $resn_exch{"1H18 OHM"}="H181 OHM"; $resn_exch{"2H18 OHM"}="H182 OHM";
$resn_exch{"1H17 OHM"}="H171 OHM"; $resn_exch{"2H17 OHM"}="H172 OHM"; $resn_exch{"1H16 OHM"}="H161 OHM";
$resn_exch{"2H16 OHM"}="H162 OHM"; $resn_exch{"1H15 OHM"}="H151 OHM"; $resn_exch{"2H15 OHM"}="H152 OHM";
$resn_exch{"1H14 OHM"}="H141 OHM"; $resn_exch{"2H14 OHM"}="H142 OHM"; $resn_exch{"1H13 OHM"}="H131 OHM";
$resn_exch{"2H13 OHM"}="H132 OHM"; $resn_exch{"1H10 OHM"}="H101 OHM"; $resn_exch{"2H10 OHM"}="H102 OHM";
$resn_exch{"1H5  OHM"}=" H51 OHM"; $resn_exch{"2H5  OHM"}=" H52 OHM"; $resn_exch{"1H4  OHM"}=" H41 OHM";
$resn_exch{"2H4  OHM"}=" H42 OHM"; $resn_exch{"1H21 OHN"}="H211 OHN"; $resn_exch{"2H21 OHN"}="H212 OHN";
$resn_exch{"3H21 OHN"}="H213 OHN"; $resn_exch{"1H20 OHN"}="H201 OHN"; $resn_exch{"2H20 OHN"}="H202 OHN";
$resn_exch{"1H19 OHN"}="H191 OHN"; $resn_exch{"2H19 OHN"}="H192 OHN"; $resn_exch{"1H18 OHN"}="H181 OHN";
$resn_exch{"2H18 OHN"}="H182 OHN"; $resn_exch{"1H17 OHN"}="H171 OHN"; $resn_exch{"2H17 OHN"}="H172 OHN";
$resn_exch{"1H16 OHN"}="H161 OHN"; $resn_exch{"2H16 OHN"}="H162 OHN"; $resn_exch{"1H15 OHN"}="H151 OHN";
$resn_exch{"2H15 OHN"}="H152 OHN"; $resn_exch{"1H14 OHN"}="H141 OHN"; $resn_exch{"2H14 OHN"}="H142 OHN";
$resn_exch{"1H13 OHN"}="H131 OHN"; $resn_exch{"2H13 OHN"}="H132 OHN"; $resn_exch{"1H10 OHN"}="H101 OHN";
$resn_exch{"2H10 OHN"}="H102 OHN"; $resn_exch{"1H5C OHN"}="H5C1 OHN"; $resn_exch{"2H5C OHN"}="H5C2 OHN";
$resn_exch{"1H4C OHN"}="H4C1 OHN"; $resn_exch{"2H4C OHN"}="H4C2 OHN"; $resn_exch{"1H2  OHO"}=" H21 OHO";
$resn_exch{"2H2  OHO"}=" H22 OHO"; $resn_exch{"1H3  OHO"}=" H31 OHO"; $resn_exch{"2H3  OHO"}=" H32 OHO";
$resn_exch{"1H4  OHO"}=" H41 OHO"; $resn_exch{"2H4  OHO"}=" H42 OHO"; $resn_exch{"1H5  OHO"}=" H51 OHO";
$resn_exch{"2H5  OHO"}=" H52 OHO"; $resn_exch{"1H6  OHO"}=" H61 OHO"; $resn_exch{"2H6  OHO"}=" H62 OHO";
$resn_exch{"1H7  OHP"}=" H71 OHP"; $resn_exch{"2H7  OHP"}=" H72 OHP"; $resn_exch{"1HA  OHS"}=" HA1 OHS";
$resn_exch{" HAC OHS"}=" H2  OHS"; $resn_exch{" HAA OHS"}=" H   OHS"; $resn_exch{"1HB  OHS"}=" HB2 OHS";
$resn_exch{"2HB  OHS"}=" HB3 OHS"; $resn_exch{" HO1 OHS"}=" HXT OHS"; $resn_exch{" HO2 OHS"}=" HZ  OHS";
$resn_exch{"1H10 OHT"}="H101 OHT"; $resn_exch{"2H10 OHT"}="H102 OHT"; $resn_exch{"3H10 OHT"}="H103 OHT";
$resn_exch{"1H9  OHT"}=" H91 OHT"; $resn_exch{"2H9  OHT"}=" H92 OHT"; $resn_exch{"1H23 OHT"}="H231 OHT";
$resn_exch{"2H23 OHT"}="H232 OHT"; $resn_exch{"1H24 OHT"}="H241 OHT"; $resn_exch{"2H24 OHT"}="H242 OHT";
$resn_exch{"1H25 OHT"}="H251 OHT"; $resn_exch{"2H25 OHT"}="H252 OHT"; $resn_exch{"3H25 OHT"}="H253 OHT";
$resn_exch{"1H26 OHT"}="H261 OHT"; $resn_exch{"2H26 OHT"}="H262 OHT"; $resn_exch{"3H26 OHT"}="H263 OHT";
$resn_exch{"1H7  OI1"}=" H71 OI1"; $resn_exch{"2H7  OI1"}=" H72 OI1"; $resn_exch{"1H14 OI1"}="H141 OI1";
$resn_exch{"2H14 OI1"}="H142 OI1"; $resn_exch{"1H15 OI1"}="H151 OI1"; $resn_exch{"2H15 OI1"}="H152 OI1";
$resn_exch{"1H22 OI1"}="H221 OI1"; $resn_exch{"2H22 OI1"}="H222 OI1"; $resn_exch{"1H23 OI1"}="H231 OI1";
$resn_exch{"2H23 OI1"}="H232 OI1"; $resn_exch{"1H24 OI1"}="H241 OI1"; $resn_exch{"2H24 OI1"}="H242 OI1";
$resn_exch{"3H24 OI1"}="H243 OI1"; $resn_exch{"1H25 OI1"}="H251 OI1"; $resn_exch{"2H25 OI1"}="H252 OI1";
$resn_exch{"1H32 OI1"}="H321 OI1"; $resn_exch{"2H32 OI1"}="H322 OI1"; $resn_exch{" HN  OIC"}=" H   OIC";
$resn_exch{"1HB  OIC"}=" HB2 OIC"; $resn_exch{"2HB  OIC"}=" HB3 OIC"; $resn_exch{"1H4  OIC"}=" H41 OIC";
$resn_exch{"2H4  OIC"}=" H42 OIC"; $resn_exch{"1H5  OIC"}=" H51 OIC"; $resn_exch{"2H5  OIC"}=" H52 OIC";
$resn_exch{"1H6  OIC"}=" H61 OIC"; $resn_exch{"2H6  OIC"}=" H62 OIC"; $resn_exch{"1H7  OIC"}=" H71 OIC";
$resn_exch{"2H7  OIC"}=" H72 OIC"; $resn_exch{"1H10 OIN"}="H101 OIN"; $resn_exch{"2H10 OIN"}="H102 OIN";
$resn_exch{"3H10 OIN"}="H103 OIN"; $resn_exch{"1H17 OIN"}="H171 OIN"; $resn_exch{"2H17 OIN"}="H172 OIN";
$resn_exch{"1H16 OIN"}="H161 OIN"; $resn_exch{"2H16 OIN"}="H162 OIN"; $resn_exch{"1H14 OIN"}="H141 OIN";
$resn_exch{"2H14 OIN"}="H142 OIN"; $resn_exch{"1H12 OIN"}="H121 OIN"; $resn_exch{"2H12 OIN"}="H122 OIN";
$resn_exch{"1H8  OIN"}=" H81 OIN"; $resn_exch{"2H8  OIN"}=" H82 OIN"; $resn_exch{"2HOP OIP"}="HOP2 OIP";
$resn_exch{"3HOP OIP"}="HOP3 OIP"; $resn_exch{"1H5* OIP"}=" H5' OIP"; $resn_exch{"2H5* OIP"}="H5'' OIP";
$resn_exch{" H4* OIP"}=" H4' OIP"; $resn_exch{" H3* OIP"}=" H3' OIP"; $resn_exch{"*HO3 OIP"}="HO3' OIP";
$resn_exch{"1H2* OIP"}=" H2' OIP"; $resn_exch{"2H2* OIP"}="H2'' OIP"; $resn_exch{" H1* OIP"}=" H1' OIP";
$resn_exch{"1H21 OIR"}="H211 OIR"; $resn_exch{"2H21 OIR"}="H212 OIR"; $resn_exch{"1H19 OIR"}="H191 OIR";
$resn_exch{"2H19 OIR"}="H192 OIR"; $resn_exch{"3H19 OIR"}="H193 OIR"; $resn_exch{"1H1  OIR"}=" H11 OIR";
$resn_exch{"2H1  OIR"}=" H12 OIR"; $resn_exch{"1H4  OIS"}=" H41 OIS"; $resn_exch{"2H4  OIS"}=" H42 OIS";
$resn_exch{"1H5  OIS"}=" H51 OIS"; $resn_exch{"2H5  OIS"}=" H52 OIS"; $resn_exch{"1H8  OIS"}=" H81 OIS";
$resn_exch{"2H8  OIS"}=" H82 OIS"; $resn_exch{"1H17 OIS"}="H171 OIS"; $resn_exch{"2H17 OIS"}="H172 OIS";
$resn_exch{"3H17 OIS"}="H173 OIS"; $resn_exch{"1H18 OIS"}="H181 OIS"; $resn_exch{"2H18 OIS"}="H182 OIS";
$resn_exch{"3H18 OIS"}="H183 OIS"; $resn_exch{"1H22 OIS"}="H221 OIS"; $resn_exch{"2H22 OIS"}="H222 OIS";
$resn_exch{"3H22 OIS"}="H223 OIS"; $resn_exch{"1H23 OIS"}="H231 OIS"; $resn_exch{"2H23 OIS"}="H232 OIS";
$resn_exch{"3H23 OIS"}="H233 OIS"; $resn_exch{"2H40 OIS"}="H402 OIS"; $resn_exch{"1H10 OIS"}="H101 OIS";
$resn_exch{"2H10 OIS"}="H102 OIS"; $resn_exch{"1H25 OIS"}="H251 OIS"; $resn_exch{"2H25 OIS"}="H252 OIS";
$resn_exch{"3H25 OIS"}="H253 OIS"; $resn_exch{"1H3  OKA"}=" H31 OKA"; $resn_exch{"2H3  OKA"}=" H32 OKA";
$resn_exch{"1H4  OKA"}=" H41 OKA"; $resn_exch{"1H5  OKA"}=" H51 OKA"; $resn_exch{"2H5  OKA"}=" H52 OKA";
$resn_exch{"1H6  OKA"}=" H61 OKA"; $resn_exch{"2H6  OKA"}=" H62 OKA"; $resn_exch{"1H7  OKA"}=" H71 OKA";
$resn_exch{"1H9  OKA"}=" H91 OKA"; $resn_exch{"1H11 OKA"}="H111 OKA"; $resn_exch{"2H11 OKA"}="H112 OKA";
$resn_exch{"1H12 OKA"}="H121 OKA"; $resn_exch{"1H13 OKA"}="H131 OKA"; $resn_exch{"1H14 OKA"}="H141 OKA";
$resn_exch{"1H15 OKA"}="H151 OKA"; $resn_exch{"1H16 OKA"}="H161 OKA"; $resn_exch{"1H17 OKA"}="H171 OKA";
$resn_exch{"2H17 OKA"}="H172 OKA"; $resn_exch{"1H18 OKA"}="H181 OKA"; $resn_exch{"2H18 OKA"}="H182 OKA";
$resn_exch{"1H20 OKA"}="H201 OKA"; $resn_exch{"2H20 OKA"}="H202 OKA"; $resn_exch{"1H21 OKA"}="H211 OKA";
$resn_exch{"2H21 OKA"}="H212 OKA"; $resn_exch{"1H22 OKA"}="H221 OKA"; $resn_exch{"1H23 OKA"}="H231 OKA";
$resn_exch{"1H24 OKA"}="H241 OKA"; $resn_exch{"1H26 OKA"}="H261 OKA"; $resn_exch{"1H27 OKA"}="H271 OKA";
$resn_exch{"1H28 OKA"}="H281 OKA"; $resn_exch{"2H28 OKA"}="H282 OKA"; $resn_exch{"1H29 OKA"}="H291 OKA";
$resn_exch{"1H30 OKA"}="H301 OKA"; $resn_exch{"1H31 OKA"}="H311 OKA"; $resn_exch{"1H32 OKA"}="H321 OKA";
$resn_exch{"2H32 OKA"}="H322 OKA"; $resn_exch{"1H33 OKA"}="H331 OKA"; $resn_exch{"2H33 OKA"}="H332 OKA";
$resn_exch{"1H35 OKA"}="H351 OKA"; $resn_exch{"2H35 OKA"}="H352 OKA"; $resn_exch{"1H36 OKA"}="H361 OKA";
$resn_exch{"2H36 OKA"}="H362 OKA"; $resn_exch{"1H37 OKA"}="H371 OKA"; $resn_exch{"2H37 OKA"}="H372 OKA";
$resn_exch{"1H38 OKA"}="H381 OKA"; $resn_exch{"2H38 OKA"}="H382 OKA"; $resn_exch{"1H39 OKA"}="H391 OKA";
$resn_exch{"2H39 OKA"}="H392 OKA"; $resn_exch{"3H39 OKA"}="H393 OKA"; $resn_exch{"1H40 OKA"}="H401 OKA";
$resn_exch{"2H40 OKA"}="H402 OKA"; $resn_exch{"3H40 OKA"}="H403 OKA"; $resn_exch{"1H41 OKA"}="H411 OKA";
$resn_exch{"2H41 OKA"}="H412 OKA"; $resn_exch{"1H42 OKA"}="H421 OKA"; $resn_exch{"2H42 OKA"}="H422 OKA";
$resn_exch{"3H42 OKA"}="H423 OKA"; $resn_exch{"1H43 OKA"}="H431 OKA"; $resn_exch{"2H43 OKA"}="H432 OKA";
$resn_exch{"3H43 OKA"}="H433 OKA"; $resn_exch{"1H44 OKA"}="H441 OKA"; $resn_exch{"2H44 OKA"}="H442 OKA";
$resn_exch{"3H44 OKA"}="H443 OKA"; $resn_exch{"1H2  OLA"}=" H21 OLA"; $resn_exch{"2H2  OLA"}=" H22 OLA";
$resn_exch{"1H3  OLA"}=" H31 OLA"; $resn_exch{"2H3  OLA"}=" H32 OLA"; $resn_exch{"1H4  OLA"}=" H41 OLA";
$resn_exch{"2H4  OLA"}=" H42 OLA"; $resn_exch{"1H5  OLA"}=" H51 OLA"; $resn_exch{"2H5  OLA"}=" H52 OLA";
$resn_exch{"1H6  OLA"}=" H61 OLA"; $resn_exch{"2H6  OLA"}=" H62 OLA"; $resn_exch{"1H7  OLA"}=" H71 OLA";
$resn_exch{"2H7  OLA"}=" H72 OLA"; $resn_exch{"1H8  OLA"}=" H81 OLA"; $resn_exch{"2H8  OLA"}=" H82 OLA";
$resn_exch{"1H11 OLA"}="H111 OLA"; $resn_exch{"2H11 OLA"}="H112 OLA"; $resn_exch{"1H12 OLA"}="H121 OLA";
$resn_exch{"2H12 OLA"}="H122 OLA"; $resn_exch{"1H13 OLA"}="H131 OLA"; $resn_exch{"2H13 OLA"}="H132 OLA";
$resn_exch{"1H14 OLA"}="H141 OLA"; $resn_exch{"2H14 OLA"}="H142 OLA"; $resn_exch{"1H15 OLA"}="H151 OLA";
$resn_exch{"2H15 OLA"}="H152 OLA"; $resn_exch{"1H16 OLA"}="H161 OLA"; $resn_exch{"2H16 OLA"}="H162 OLA";
$resn_exch{"1H17 OLA"}="H171 OLA"; $resn_exch{"2H17 OLA"}="H172 OLA"; $resn_exch{"1H18 OLA"}="H181 OLA";
$resn_exch{"2H18 OLA"}="H182 OLA"; $resn_exch{"3H18 OLA"}="H183 OLA"; $resn_exch{"1HB  OLE"}=" HB1 OLE";
$resn_exch{"2HB  OLE"}=" HB2 OLE"; $resn_exch{"1HD1 OLE"}="HD11 OLE"; $resn_exch{"2HD1 OLE"}="HD12 OLE";
$resn_exch{"3HD1 OLE"}="HD13 OLE"; $resn_exch{"1HD2 OLE"}="HD21 OLE"; $resn_exch{"2HD2 OLE"}="HD22 OLE";
$resn_exch{"3HD2 OLE"}="HD23 OLE"; $resn_exch{"1H2  OLI"}=" H21 OLI"; $resn_exch{"2H2  OLI"}=" H22 OLI";
$resn_exch{"1H3  OLI"}=" H31 OLI"; $resn_exch{"2H3  OLI"}=" H32 OLI"; $resn_exch{"1H4  OLI"}=" H41 OLI";
$resn_exch{"2H4  OLI"}=" H42 OLI"; $resn_exch{"1H5  OLI"}=" H51 OLI"; $resn_exch{"2H5  OLI"}=" H52 OLI";
$resn_exch{"1H6  OLI"}=" H61 OLI"; $resn_exch{"2H6  OLI"}=" H62 OLI"; $resn_exch{"1H7  OLI"}=" H71 OLI";
$resn_exch{"2H7  OLI"}=" H72 OLI"; $resn_exch{"1H8  OLI"}=" H81 OLI"; $resn_exch{"2H8  OLI"}=" H82 OLI";
$resn_exch{"1H11 OLI"}="H111 OLI"; $resn_exch{"2H11 OLI"}="H112 OLI"; $resn_exch{"1H12 OLI"}="H121 OLI";
$resn_exch{"2H12 OLI"}="H122 OLI"; $resn_exch{"1H13 OLI"}="H131 OLI"; $resn_exch{"2H13 OLI"}="H132 OLI";
$resn_exch{"1H14 OLI"}="H141 OLI"; $resn_exch{"2H14 OLI"}="H142 OLI"; $resn_exch{"1H15 OLI"}="H151 OLI";
$resn_exch{"2H15 OLI"}="H152 OLI"; $resn_exch{"1H16 OLI"}="H161 OLI"; $resn_exch{"2H16 OLI"}="H162 OLI";
$resn_exch{"1H17 OLI"}="H171 OLI"; $resn_exch{"2H17 OLI"}="H172 OLI"; $resn_exch{"1H18 OLI"}="H181 OLI";
$resn_exch{"2H18 OLI"}="H182 OLI"; $resn_exch{"3H18 OLI"}="H183 OLI"; $resn_exch{"1HB  OLN"}=" HB1 OLN";
$resn_exch{"2HB  OLN"}=" HB2 OLN"; $resn_exch{"1HG  OLN"}=" HG1 OLN"; $resn_exch{"2HG  OLN"}=" HG2 OLN";
$resn_exch{"1HD  OLN"}=" HD1 OLN"; $resn_exch{"2HD  OLN"}=" HD2 OLN"; $resn_exch{"1HH2 OLN"}="HH21 OLN";
$resn_exch{"2HH2 OLN"}="HH22 OLN"; $resn_exch{"1H2  OLN"}=" H21 OLN"; $resn_exch{"2H2  OLN"}=" H22 OLN";
$resn_exch{"3H2  OLN"}=" H23 OLN"; $resn_exch{"1H3  OLO"}=" H31 OLO"; $resn_exch{"2H3  OLO"}=" H32 OLO";
$resn_exch{"3H3  OLO"}=" H33 OLO"; $resn_exch{"1H4  OLO"}=" H41 OLO"; $resn_exch{"2H4  OLO"}=" H42 OLO";
$resn_exch{"1H5  OLO"}=" H51 OLO"; $resn_exch{"2H5  OLO"}=" H52 OLO"; $resn_exch{"1H6  OLO"}=" H61 OLO";
$resn_exch{"2H6  OLO"}=" H62 OLO"; $resn_exch{"1HAB OLP"}="HAB1 OLP"; $resn_exch{"2HAB OLP"}="HAB2 OLP";
$resn_exch{"3HAB OLP"}="HAB3 OLP"; $resn_exch{"1HAA OLP"}="HAA1 OLP"; $resn_exch{"2HAA OLP"}="HAA2 OLP";
$resn_exch{"3HAA OLP"}="HAA3 OLP"; $resn_exch{"1HAK OLP"}="HAK1 OLP"; $resn_exch{"2HAK OLP"}="HAK2 OLP";
$resn_exch{"1HAJ OLP"}="HAJ1 OLP"; $resn_exch{"2HAJ OLP"}="HAJ2 OLP"; $resn_exch{"1HN  OLT"}=" H   OLT";
$resn_exch{"2HN  OLT"}=" H2  OLT"; $resn_exch{" HCB OLT"}=" HB  OLT"; $resn_exch{"1HG2 OLT"}="HG21 OLT";
$resn_exch{"2HG2 OLT"}="HG22 OLT"; $resn_exch{"3HG2 OLT"}="HG23 OLT"; $resn_exch{"1HD1 OLT"}="HD11 OLT";
$resn_exch{"2HD1 OLT"}="HD12 OLT"; $resn_exch{"3HD1 OLT"}="HD13 OLT"; $resn_exch{"1H5* OLU"}="H5'1 OLU";
$resn_exch{"2H5* OLU"}="H5'2 OLU"; $resn_exch{"1HC  OMB"}=" HC1 OMB"; $resn_exch{"2HC  OMB"}=" HC2 OMB";
$resn_exch{"1HN4 OMC"}="HN41 OMC"; $resn_exch{"2HN4 OMC"}="HN42 OMC"; $resn_exch{" H1* OMC"}=" H1' OMC";
$resn_exch{" H2* OMC"}=" H2' OMC"; $resn_exch{"1HM2 OMC"}="HM21 OMC"; $resn_exch{"2HM2 OMC"}="HM22 OMC";
$resn_exch{"3HM2 OMC"}="HM23 OMC"; $resn_exch{" H3* OMC"}=" H3' OMC"; $resn_exch{" H4* OMC"}=" H4' OMC";
$resn_exch{"*HO3 OMC"}="HO3' OMC"; $resn_exch{"1H5* OMC"}=" H5' OMC"; $resn_exch{"2H5* OMC"}="H5'' OMC";
$resn_exch{"2HOP OMC"}="HOP2 OMC"; $resn_exch{"3HOP OMC"}="HOP3 OMC"; $resn_exch{"1H2  OMD"}=" H21 OMD";
$resn_exch{"2H2  OMD"}=" H22 OMD"; $resn_exch{"1H1  OME"}=" H11 OME"; $resn_exch{"2H1  OME"}=" H12 OME";
$resn_exch{"3H1  OME"}=" H13 OME"; $resn_exch{"2HOP OMG"}="HOP2 OMG"; $resn_exch{"3HOP OMG"}="HOP3 OMG";
$resn_exch{"1H5* OMG"}=" H5' OMG"; $resn_exch{"2H5* OMG"}="H5'' OMG"; $resn_exch{" H4* OMG"}=" H4' OMG";
$resn_exch{" H3* OMG"}=" H3' OMG"; $resn_exch{"*HO3 OMG"}="HO3' OMG"; $resn_exch{" H2* OMG"}=" H2' OMG";
$resn_exch{"1HM2 OMG"}="HM21 OMG"; $resn_exch{"2HM2 OMG"}="HM22 OMG"; $resn_exch{"3HM2 OMG"}="HM23 OMG";
$resn_exch{" H1* OMG"}=" H1' OMG"; $resn_exch{"1HN2 OMG"}="HN21 OMG"; $resn_exch{"2HN2 OMG"}="HN22 OMG";
$resn_exch{"1H1  OMN"}="H11A OMN"; $resn_exch{"2H1  OMN"}=" H12 OMN"; $resn_exch{"3H1  OMN"}=" H13 OMN";
$resn_exch{"2HOP OMP"}="HOP2 OMP"; $resn_exch{"3HOP OMP"}="HOP3 OMP"; $resn_exch{" H1* OMP"}=" H1' OMP";
$resn_exch{" H2* OMP"}=" H2' OMP"; $resn_exch{" H3* OMP"}=" H3' OMP"; $resn_exch{" H4* OMP"}=" H4' OMP";
$resn_exch{"*HO2 OMP"}="HO2' OMP"; $resn_exch{"*HO3 OMP"}="HO3' OMP"; $resn_exch{"1H5* OMP"}="H5'1 OMP";
$resn_exch{"2H5* OMP"}="H5'2 OMP"; $resn_exch{"1HN  OMT"}=" H   OMT"; $resn_exch{"2HN  OMT"}=" H2  OMT";
$resn_exch{"1HB  OMT"}=" HB2 OMT"; $resn_exch{"2HB  OMT"}=" HB3 OMT"; $resn_exch{"1HG  OMT"}=" HG2 OMT";
$resn_exch{"2HG  OMT"}=" HG3 OMT"; $resn_exch{"1HE  OMT"}=" HE1 OMT"; $resn_exch{"2HE  OMT"}=" HE2 OMT";
$resn_exch{"3HE  OMT"}=" HE3 OMT"; $resn_exch{" H1* OMU"}=" H1' OMU"; $resn_exch{" H2* OMU"}=" H2' OMU";
$resn_exch{"1HM2 OMU"}="HM21 OMU"; $resn_exch{"2HM2 OMU"}="HM22 OMU"; $resn_exch{"3HM2 OMU"}="HM23 OMU";
$resn_exch{" H3* OMU"}=" H3' OMU"; $resn_exch{" H4* OMU"}=" H4' OMU"; $resn_exch{"*HO3 OMU"}="HO3' OMU";
$resn_exch{"1H5* OMU"}=" H5' OMU"; $resn_exch{"2H5* OMU"}="H5'' OMU"; $resn_exch{"2HOP OMU"}="HOP2 OMU";
$resn_exch{"3HOP OMU"}="HOP3 OMU"; $resn_exch{"1HA7 ONA"}="HA71 ONA"; $resn_exch{"2HA7 ONA"}="HA72 ONA";
$resn_exch{"3HA7 ONA"}="HA73 ONA"; $resn_exch{" H3* ONA"}=" H3' ONA"; $resn_exch{" H2* ONA"}=" H2' ONA";
$resn_exch{" H4* ONA"}=" H4' ONA"; $resn_exch{"1H5* ONA"}="H5'1 ONA"; $resn_exch{"2H5* ONA"}="H5'2 ONA";
$resn_exch{" H1* ONA"}=" H1' ONA"; $resn_exch{"1HN6 ONA"}="HN61 ONA"; $resn_exch{"2HN6 ONA"}="HN62 ONA";
$resn_exch{"3HOP ONE"}="HOP3 ONE"; $resn_exch{"2HOP ONE"}="HOP2 ONE"; $resn_exch{"2H5* ONE"}="H5'' ONE";
$resn_exch{"1H5* ONE"}=" H5' ONE"; $resn_exch{" H4* ONE"}=" H4' ONE"; $resn_exch{" H3* ONE"}=" H3' ONE";
$resn_exch{" H2* ONE"}=" H2' ONE"; $resn_exch{"2HO  ONE"}=" HO2 ONE"; $resn_exch{"3HO  ONE"}=" HO3 ONE";
$resn_exch{" H1* ONE"}=" H1' ONE"; $resn_exch{"1HA7 ONM"}="HA71 ONM"; $resn_exch{"2HA7 ONM"}="HA72 ONM";
$resn_exch{"3HA7 ONM"}="HA73 ONM"; $resn_exch{" H3* ONM"}=" H3' ONM"; $resn_exch{" H2* ONM"}=" H2' ONM";
$resn_exch{" H4* ONM"}=" H4' ONM"; $resn_exch{"1H5* ONM"}="H5'1 ONM"; $resn_exch{"2H5* ONM"}="H5'2 ONM";
$resn_exch{" H1* ONM"}=" H1' ONM"; $resn_exch{"1HN2 ONM"}="HN21 ONM"; $resn_exch{"2HN2 ONM"}="HN22 ONM";
$resn_exch{"1H62 ONO"}="H621 ONO"; $resn_exch{"2H62 ONO"}="H622 ONO"; $resn_exch{"3H62 ONO"}="H623 ONO";
$resn_exch{"1H37 ONO"}="H371 ONO"; $resn_exch{"2H37 ONO"}="H372 ONO"; $resn_exch{"3H37 ONO"}="H373 ONO";
$resn_exch{"1H36 ONO"}="H361 ONO"; $resn_exch{"2H36 ONO"}="H362 ONO"; $resn_exch{"3H36 ONO"}="H363 ONO";
$resn_exch{"1H58 ONO"}="H581 ONO"; $resn_exch{"2H58 ONO"}="H582 ONO"; $resn_exch{"3H58 ONO"}="H583 ONO";
$resn_exch{"1H10 ONO"}="H101 ONO"; $resn_exch{"2H10 ONO"}="H102 ONO"; $resn_exch{"1HN9 ONO"}="HN91 ONO";
$resn_exch{"2HN9 ONO"}="HN92 ONO"; $resn_exch{"2HOB ONP"}="HOB2 ONP"; $resn_exch{"2HOA ONP"}="HOA2 ONP";
$resn_exch{"1HNA ONP"}="HNA1 ONP"; $resn_exch{"1HA2 ONP"}="HA21 ONP"; $resn_exch{"2HA2 ONP"}="HA22 ONP";
$resn_exch{"1HA1 ONP"}="HA11 ONP"; $resn_exch{"2HA1 ONP"}="HA12 ONP"; $resn_exch{"1H2C OOA"}="H2C1 OOA";
$resn_exch{"2H2C OOA"}="H2C2 OOA"; $resn_exch{"1H4C OOA"}="H4C1 OOA"; $resn_exch{"2H4C OOA"}="H4C2 OOA";
$resn_exch{"1H5C OOA"}="H5C1 OOA"; $resn_exch{"2H5C OOA"}="H5C2 OOA"; $resn_exch{"1H6C OOA"}="H6C1 OOA";
$resn_exch{"2H6C OOA"}="H6C2 OOA"; $resn_exch{"1H7C OOA"}="H7C1 OOA"; $resn_exch{"2H7C OOA"}="H7C2 OOA";
$resn_exch{"1H8C OOA"}="H8C1 OOA"; $resn_exch{"2H8C OOA"}="H8C2 OOA"; $resn_exch{"3H8C OOA"}="H8C3 OOA";
$resn_exch{"1H2  OPA"}=" H21 OPA"; $resn_exch{"2H2  OPA"}="H22A OPA"; $resn_exch{"1H5  OPA"}=" H51 OPA";
$resn_exch{"2H5  OPA"}=" H52 OPA"; $resn_exch{"1H6  OPA"}=" H61 OPA"; $resn_exch{"2H6  OPA"}=" H62 OPA";
$resn_exch{"1H21 OPB"}="H211 OPB"; $resn_exch{"2H21 OPB"}="H212 OPB"; $resn_exch{"3H21 OPB"}="H213 OPB";
$resn_exch{"1H20 OPB"}="H201 OPB"; $resn_exch{"2H20 OPB"}="H202 OPB"; $resn_exch{"1H19 OPB"}="H191 OPB";
$resn_exch{"2H19 OPB"}="H192 OPB"; $resn_exch{"1H18 OPB"}="H181 OPB"; $resn_exch{"2H18 OPB"}="H182 OPB";
$resn_exch{"1HBF OPC"}="HBF1 OPC"; $resn_exch{"2HBF OPC"}="HBF2 OPC"; $resn_exch{"3HBF OPC"}="HBF3 OPC";
$resn_exch{"1HBE OPC"}="HBE1 OPC"; $resn_exch{"2HBE OPC"}="HBE2 OPC"; $resn_exch{"1HBD OPC"}="HBD1 OPC";
$resn_exch{"2HBD OPC"}="HBD2 OPC"; $resn_exch{"1HBC OPC"}="HBC1 OPC"; $resn_exch{"2HBC OPC"}="HBC2 OPC";
$resn_exch{"1HBB OPC"}="HBB1 OPC"; $resn_exch{"2HBB OPC"}="HBB2 OPC"; $resn_exch{"1HBA OPC"}="HBA1 OPC";
$resn_exch{"2HBA OPC"}="HBA2 OPC"; $resn_exch{"1HAZ OPC"}="HAZ1 OPC"; $resn_exch{"2HAZ OPC"}="HAZ2 OPC";
$resn_exch{"1HAY OPC"}="HAY1 OPC"; $resn_exch{"2HAY OPC"}="HAY2 OPC"; $resn_exch{"1HAX OPC"}="HAX1 OPC";
$resn_exch{"2HAX OPC"}="HAX2 OPC"; $resn_exch{"1HAU OPC"}="HAU1 OPC"; $resn_exch{"2HAU OPC"}="HAU2 OPC";
$resn_exch{"1HAT OPC"}="HAT1 OPC"; $resn_exch{"2HAT OPC"}="HAT2 OPC"; $resn_exch{"1HAS OPC"}="HAS1 OPC";
$resn_exch{"2HAS OPC"}="HAS2 OPC"; $resn_exch{"1HAR OPC"}="HAR1 OPC"; $resn_exch{"2HAR OPC"}="HAR2 OPC";
$resn_exch{"1HAQ OPC"}="HAQ1 OPC"; $resn_exch{"2HAQ OPC"}="HAQ2 OPC"; $resn_exch{"1HAP OPC"}="HAP1 OPC";
$resn_exch{"2HAP OPC"}="HAP2 OPC"; $resn_exch{"1HAL OPC"}="HAL1 OPC"; $resn_exch{"2HAL OPC"}="HAL2 OPC";
$resn_exch{"1HAH OPC"}="HAH1 OPC"; $resn_exch{"2HAH OPC"}="HAH2 OPC"; $resn_exch{"1HAG OPC"}="HAG1 OPC";
$resn_exch{"2HAG OPC"}="HAG2 OPC"; $resn_exch{"1HAE OPC"}="HAE1 OPC"; $resn_exch{"2HAE OPC"}="HAE2 OPC";
$resn_exch{"3HAE OPC"}="HAE3 OPC"; $resn_exch{"1HBG OPC"}="HBG1 OPC"; $resn_exch{"2HBG OPC"}="HBG2 OPC";
$resn_exch{"3HBG OPC"}="HBG3 OPC"; $resn_exch{"1HAA OPC"}="HAA1 OPC"; $resn_exch{"2HAA OPC"}="HAA2 OPC";
$resn_exch{"3HAA OPC"}="HAA3 OPC"; $resn_exch{"1HAC OPC"}="HAC1 OPC"; $resn_exch{"2HAC OPC"}="HAC2 OPC";
$resn_exch{"3HAC OPC"}="HAC3 OPC"; $resn_exch{"1HBI OPC"}="HBI1 OPC"; $resn_exch{"2HBI OPC"}="HBI2 OPC";
$resn_exch{"1HBL OPC"}="HBL1 OPC"; $resn_exch{"2HBL OPC"}="HBL2 OPC"; $resn_exch{"1HBM OPC"}="HBM1 OPC";
$resn_exch{"2HBM OPC"}="HBM2 OPC"; $resn_exch{"1HBN OPC"}="HBN1 OPC"; $resn_exch{"2HBN OPC"}="HBN2 OPC";
$resn_exch{"1HBO OPC"}="HBO1 OPC"; $resn_exch{"2HBO OPC"}="HBO2 OPC"; $resn_exch{"1HBP OPC"}="HBP1 OPC";
$resn_exch{"2HBP OPC"}="HBP2 OPC"; $resn_exch{"1HBQ OPC"}="HBQ1 OPC"; $resn_exch{"2HBQ OPC"}="HBQ2 OPC";
$resn_exch{"1HBT OPC"}="HBT1 OPC"; $resn_exch{"2HBT OPC"}="HBT2 OPC"; $resn_exch{"1HBU OPC"}="HBU1 OPC";
$resn_exch{"2HBU OPC"}="HBU2 OPC"; $resn_exch{"1HBV OPC"}="HBV1 OPC"; $resn_exch{"2HBV OPC"}="HBV2 OPC";
$resn_exch{"1HBW OPC"}="HBW1 OPC"; $resn_exch{"2HBW OPC"}="HBW2 OPC"; $resn_exch{"1HBX OPC"}="HBX1 OPC";
$resn_exch{"2HBX OPC"}="HBX2 OPC"; $resn_exch{"1HBY OPC"}="HBY1 OPC"; $resn_exch{"2HBY OPC"}="HBY2 OPC";
$resn_exch{"1HBZ OPC"}="HBZ1 OPC"; $resn_exch{"2HBZ OPC"}="HBZ2 OPC"; $resn_exch{"1HCA OPC"}="HCA1 OPC";
$resn_exch{"2HCA OPC"}="HCA2 OPC"; $resn_exch{"1HCB OPC"}="HCB1 OPC"; $resn_exch{"2HCB OPC"}="HCB2 OPC";
$resn_exch{"3HCB OPC"}="HCB3 OPC"; $resn_exch{"1H8  OPD"}=" H81 OPD"; $resn_exch{"1H7  OPD"}=" H71 OPD";
$resn_exch{"1H5  OPD"}=" H51 OPD"; $resn_exch{"1H2  OPD"}=" H21 OPD"; $resn_exch{"1H3  OPD"}=" H31 OPD";
$resn_exch{"1H4  OPD"}=" H41 OPD"; $resn_exch{"1H11 OPD"}="H111 OPD"; $resn_exch{"2H11 OPD"}="H112 OPD";
$resn_exch{"1H12 OPD"}="H121 OPD"; $resn_exch{"2H12 OPD"}="H122 OPD"; $resn_exch{"1H13 OPD"}="H131 OPD";
$resn_exch{"2H13 OPD"}="H132 OPD"; $resn_exch{"1H14 OPD"}="H141 OPD"; $resn_exch{"2H14 OPD"}="H142 OPD";
$resn_exch{"1H15 OPD"}="H151 OPD"; $resn_exch{"2H15 OPD"}="H152 OPD"; $resn_exch{"1H16 OPD"}="H161 OPD";
$resn_exch{"2H16 OPD"}="H162 OPD"; $resn_exch{"1H17 OPD"}="H171 OPD"; $resn_exch{"2H17 OPD"}="H172 OPD";
$resn_exch{"1H20 OPD"}="H201 OPD"; $resn_exch{"1H6  OPD"}=" H61 OPD"; $resn_exch{"2H6  OPD"}=" H62 OPD";
$resn_exch{"1H9  OPD"}=" H91 OPD"; $resn_exch{"2H9  OPD"}=" H92 OPD"; $resn_exch{"1H10 OPD"}="H101 OPD";
$resn_exch{"2H10 OPD"}="H102 OPD"; $resn_exch{"3H10 OPD"}="H103 OPD"; $resn_exch{"1HA  OPE"}=" HA1 OPE";
$resn_exch{"2HA  OPE"}=" HA2 OPE"; $resn_exch{"1HB  OPE"}=" HB1 OPE"; $resn_exch{"2HB  OPE"}=" HB2 OPE";
$resn_exch{"1HN  OPE"}=" HN1 OPE"; $resn_exch{"2HN  OPE"}=" HN2 OPE"; $resn_exch{"1H1  OPG"}=" H11 OPG";
$resn_exch{"2H1  OPG"}=" H12 OPG"; $resn_exch{"1H7  OPG"}=" H71 OPG"; $resn_exch{"2H7  OPG"}=" H72 OPG";
$resn_exch{"1HB  OPH"}=" HB1 OPH"; $resn_exch{"2HB  OPH"}=" HB2 OPH"; $resn_exch{"1HS  OPH"}=" HS1 OPH";
$resn_exch{"2HS  OPH"}=" HS2 OPH"; $resn_exch{"3HS  OPH"}=" HS3 OPH"; $resn_exch{"1H3' OPM"}="H3'1 OPM";
$resn_exch{"2H3' OPM"}="H3'2 OPM"; $resn_exch{"1H4' OPM"}="H4'1 OPM"; $resn_exch{"2H4' OPM"}="H4'2 OPM";
$resn_exch{"1H5' OPM"}="H5'1 OPM"; $resn_exch{"2H5' OPM"}="H5'2 OPM"; $resn_exch{"3H5' OPM"}="H5'3 OPM";
$resn_exch{"1H1' OPM"}="H1'1 OPM"; $resn_exch{"2H1' OPM"}="H1'2 OPM"; $resn_exch{"1H2' OPM"}="H2'1 OPM";
$resn_exch{"2H2' OPM"}="H2'2 OPM"; $resn_exch{"1H6  OPM"}=" H61 OPM"; $resn_exch{"2H6  OPM"}=" H62 OPM";
$resn_exch{"1H1' OPN"}="H1'1 OPN"; $resn_exch{"2H1' OPN"}="H1'2 OPN"; $resn_exch{"3H1' OPN"}="H1'3 OPN";
$resn_exch{"1H2' OPN"}="H2'1 OPN"; $resn_exch{"2H2' OPN"}="H2'2 OPN"; $resn_exch{"1H3' OPN"}="H3'1 OPN";
$resn_exch{"2H3' OPN"}="H3'2 OPN"; $resn_exch{"1H5' OPN"}="H5'1 OPN"; $resn_exch{"2H5' OPN"}="H5'2 OPN";
$resn_exch{"1H8' OPN"}="H8'1 OPN"; $resn_exch{"2H8' OPN"}="H8'2 OPN"; $resn_exch{"1H5  OPP"}=" H51 OPP";
$resn_exch{"2H5  OPP"}=" H52 OPP"; $resn_exch{"1H5' OPP"}="H5'1 OPP"; $resn_exch{"2H5' OPP"}="H5'2 OPP";
$resn_exch{"1HN  OPR"}=" H   OPR"; $resn_exch{"2HN  OPR"}=" H2  OPR"; $resn_exch{"1HB  OPR"}=" HB2 OPR";
$resn_exch{"2HB  OPR"}=" HB3 OPR"; $resn_exch{"1HG  OPR"}=" HG2 OPR"; $resn_exch{"2HG  OPR"}=" HG3 OPR";
$resn_exch{"1HD  OPR"}=" HD2 OPR"; $resn_exch{"2HD  OPR"}=" HD3 OPR"; $resn_exch{"1HH2 OPR"}="HH21 OPR";
$resn_exch{"2HH2 OPR"}="HH22 OPR"; $resn_exch{"1H1  OPR"}=" H11 OPR"; $resn_exch{"2H1  OPR"}=" H12 OPR";
$resn_exch{"1H2  OPR"}=" H21 OPR"; $resn_exch{"2H2  OPR"}=" H22 OPR"; $resn_exch{"1HN  OPS"}=" HN1 OPS";
$resn_exch{"2HN  OPS"}=" HN2 OPS"; $resn_exch{"1HO  OPS"}=" HO1 OPS"; $resn_exch{"2HO  OPS"}=" HO2 OPS";
$resn_exch{"1HU  OPS"}=" HU1 OPS"; $resn_exch{"2HU  OPS"}=" HU2 OPS"; $resn_exch{"1HV  OPS"}=" HV1 OPS";
$resn_exch{"2HV  OPS"}=" HV2 OPS"; $resn_exch{"1H10 OQB"}="H101 OQB"; $resn_exch{"2H10 OQB"}="H102 OQB";
$resn_exch{"1H19 OQB"}="H191 OQB"; $resn_exch{"2H19 OQB"}="H192 OQB"; $resn_exch{"1H1* OR5"}="H1'1 OR5";
$resn_exch{"2H1* OR5"}="H1'2 OR5"; $resn_exch{"1HO* OR5"}="HO'1 OR5"; $resn_exch{" H2* OR5"}=" H2' OR5";
$resn_exch{"2HO* OR5"}="HO'2 OR5"; $resn_exch{" H3* OR5"}=" H3' OR5"; $resn_exch{"3HO* OR5"}="HO'3 OR5";
$resn_exch{" H4* OR5"}=" H4' OR5"; $resn_exch{"4HO* OR5"}="HO'4 OR5"; $resn_exch{"1H5* OR5"}="H5'1 OR5";
$resn_exch{"2H5* OR5"}="H5'2 OR5"; $resn_exch{"2HOP OR5"}="HOP2 OR5"; $resn_exch{"3HOP OR5"}="HOP3 OR5";
$resn_exch{"1HN  ORN"}=" H   ORN"; $resn_exch{"2HN  ORN"}=" HN2 ORN"; $resn_exch{"3HN  ORN"}=" HN3 ORN";
$resn_exch{"1HB  ORN"}=" HB2 ORN"; $resn_exch{"2HB  ORN"}=" HB3 ORN"; $resn_exch{"1HG  ORN"}=" HG2 ORN";
$resn_exch{"2HG  ORN"}=" HG3 ORN"; $resn_exch{"1HD  ORN"}=" HD2 ORN"; $resn_exch{"2HD  ORN"}=" HD3 ORN";
$resn_exch{"1HNE ORN"}=" HE1 ORN"; $resn_exch{"2HNE ORN"}=" HE2 ORN"; $resn_exch{"3HNE ORN"}=" HE3 ORN";
$resn_exch{"1H2  ORP"}=" H21 ORP"; $resn_exch{"2H2  ORP"}=" H22 ORP"; $resn_exch{"1H5  ORP"}=" H51 ORP";
$resn_exch{"2H5  ORP"}=" H52 ORP"; $resn_exch{"2HOP ORP"}="HOP2 ORP"; $resn_exch{"3HOP ORP"}="HOP3 ORP";
$resn_exch{"1HN  ORQ"}=" H   ORQ"; $resn_exch{"2HN  ORQ"}=" H2  ORQ"; $resn_exch{"1HB  ORQ"}=" HB2 ORQ";
$resn_exch{"2HB  ORQ"}=" HB3 ORQ"; $resn_exch{"1HG  ORQ"}=" HG2 ORQ"; $resn_exch{"2HG  ORQ"}=" HG3 ORQ";
$resn_exch{"1HD  ORQ"}=" HD2 ORQ"; $resn_exch{"2HD  ORQ"}=" HD3 ORQ"; $resn_exch{"1HNE ORQ"}=" HE  ORQ";
$resn_exch{"1H2  ORQ"}=" H21 ORQ"; $resn_exch{"2H2  ORQ"}=" H22 ORQ"; $resn_exch{"3H2  ORQ"}=" H23 ORQ";
$resn_exch{"HOXT ORQ"}=" HXT ORQ"; $resn_exch{"1H4A ORX"}="H4A1 ORX"; $resn_exch{"2H4A ORX"}="H4A2 ORX";
$resn_exch{"1H2A ORX"}="H2A1 ORX"; $resn_exch{"2H2A ORX"}="H2A2 ORX"; $resn_exch{"3H2A ORX"}="H2A3 ORX";
$resn_exch{"1H5A ORX"}="H5A1 ORX"; $resn_exch{"2H5A ORX"}="H5A2 ORX"; $resn_exch{"1HBA ORX"}="HBA1 ORX";
$resn_exch{"2HBA ORX"}="HBA2 ORX"; $resn_exch{"1HGA ORX"}="HGA1 ORX"; $resn_exch{"2HGA ORX"}="HGA2 ORX";
$resn_exch{"1HDA ORX"}="HDA1 ORX"; $resn_exch{"2HDA ORX"}="HDA2 ORX"; $resn_exch{"1HNE ORX"}="HNE1 ORX";
$resn_exch{"2HNE ORX"}="HNE2 ORX"; $resn_exch{"1H6  OSB"}=" H61 OSB"; $resn_exch{"1H5  OSB"}=" H51 OSB";
$resn_exch{"1H4  OSB"}=" H41 OSB"; $resn_exch{"1H3  OSB"}=" H31 OSB"; $resn_exch{"1H9  OSB"}=" H91 OSB";
$resn_exch{"2H9  OSB"}=" H92 OSB"; $resn_exch{"1H10 OSB"}="H101 OSB"; $resn_exch{"2H10 OSB"}="H102 OSB";
$resn_exch{"1H6  OSC"}=" H61 OSC"; $resn_exch{"2H6  OSC"}=" H62 OSC"; $resn_exch{"1H7  OSC"}=" H71 OSC";
$resn_exch{"2H7  OSC"}=" H72 OSC"; $resn_exch{"1H11 OSC"}="H111 OSC"; $resn_exch{"2H11 OSC"}="H112 OSC";
$resn_exch{"1H15 OSC"}="H151 OSC"; $resn_exch{"2H15 OSC"}="H152 OSC"; $resn_exch{"1H17 OSC"}="H171 OSC";
$resn_exch{"2H17 OSC"}="H172 OSC"; $resn_exch{"1H19 OSC"}="H191 OSC"; $resn_exch{"2H19 OSC"}="H192 OSC";
$resn_exch{"1H22 OSC"}="H221 OSC"; $resn_exch{"2H22 OSC"}="H222 OSC"; $resn_exch{"1H30 OSC"}="H301 OSC";
$resn_exch{"2H30 OSC"}="H302 OSC"; $resn_exch{"1H32 OSC"}="H321 OSC"; $resn_exch{"2H32 OSC"}="H322 OSC";
$resn_exch{"1H37 OSC"}="H371 OSC"; $resn_exch{"2H37 OSC"}="H372 OSC"; $resn_exch{"1H44 OSC"}="H441 OSC";
$resn_exch{"2H44 OSC"}="H442 OSC"; $resn_exch{" HN2 OSE"}=" H2  OSE"; $resn_exch{"1HB  OSE"}=" HB2 OSE";
$resn_exch{"2HB  OSE"}=" HB3 OSE"; $resn_exch{"1H   OSM"}=" H1  OSM"; $resn_exch{"2H   OSM"}=" H2  OSM";
$resn_exch{"1HN  OSM"}=" HN1 OSM"; $resn_exch{"2HN  OSM"}=" HN2 OSM"; $resn_exch{"1H7  OSP"}=" H71 OSP";
$resn_exch{"2H7  OSP"}=" H72 OSP"; $resn_exch{"1H8  OSP"}=" H81 OSP"; $resn_exch{"2H8  OSP"}=" H82 OSP";
$resn_exch{"1H9  OSP"}=" H91 OSP"; $resn_exch{"2H9  OSP"}=" H92 OSP"; $resn_exch{"1H10 OSP"}="H101 OSP";
$resn_exch{"2H10 OSP"}="H102 OSP"; $resn_exch{"1HN1 OSP"}="HN11 OSP"; $resn_exch{"2HN1 OSP"}="HN12 OSP";
$resn_exch{"1H2  OSS"}=" H21 OSS"; $resn_exch{"2H2  OSS"}=" H22 OSS"; $resn_exch{"1H3  OSS"}=" H31 OSS";
$resn_exch{"2H3  OSS"}=" H32 OSS"; $resn_exch{"1H4  OSS"}=" H41 OSS"; $resn_exch{"2H4  OSS"}=" H42 OSS";
$resn_exch{"1H5  OSS"}=" H51 OSS"; $resn_exch{"2H5  OSS"}=" H52 OSS"; $resn_exch{"1H7  OSS"}=" H71 OSS";
$resn_exch{"2H7  OSS"}=" H72 OSS"; $resn_exch{"1H8  OSS"}=" H81 OSS"; $resn_exch{"2H8  OSS"}=" H82 OSS";
$resn_exch{"1H10 OSS"}="H101 OSS"; $resn_exch{"2H10 OSS"}="H102 OSS"; $resn_exch{"1HN1 OSS"}="HN11 OSS";
$resn_exch{"2HN1 OSS"}="HN12 OSS"; $resn_exch{"1H14 OSS"}="H141 OSS"; $resn_exch{"2H14 OSS"}="H142 OSS";
$resn_exch{"1H15 OSS"}="H151 OSS"; $resn_exch{"2H15 OSS"}="H152 OSS"; $resn_exch{"1H5  OST"}=" H51 OST";
$resn_exch{"2H5  OST"}=" H52 OST"; $resn_exch{"3H5  OST"}=" H53 OST"; $resn_exch{"1H4  OST"}=" H41 OST";
$resn_exch{"2H4  OST"}=" H42 OST"; $resn_exch{"3H4  OST"}=" H43 OST"; $resn_exch{"1H6  OSU"}=" H61 OSU";
$resn_exch{"2H6  OSU"}=" H62 OSU"; $resn_exch{"1H1' OSU"}="H1'1 OSU"; $resn_exch{"2H1' OSU"}="H1'2 OSU";
$resn_exch{"1H6' OSU"}="H6'1 OSU"; $resn_exch{"2H6' OSU"}="H6'2 OSU"; $resn_exch{"1H2N OSU"}="H2N1 OSU";
$resn_exch{"2H2N OSU"}="H2N2 OSU"; $resn_exch{"1H3N OSU"}="H3N1 OSU"; $resn_exch{"2H3N OSU"}="H3N2 OSU";
$resn_exch{"1H4N OSU"}="H4N1 OSU"; $resn_exch{"2H4N OSU"}="H4N2 OSU"; $resn_exch{"1H5N OSU"}="H5N1 OSU";
$resn_exch{"2H5N OSU"}="H5N2 OSU"; $resn_exch{"1H6N OSU"}="H6N1 OSU"; $resn_exch{"2H6N OSU"}="H6N2 OSU";
$resn_exch{"1H7N OSU"}="H7N1 OSU"; $resn_exch{"2H7N OSU"}="H7N2 OSU"; $resn_exch{"1H8N OSU"}="H8N1 OSU";
$resn_exch{"2H8N OSU"}="H8N2 OSU"; $resn_exch{"3H8N OSU"}="H8N3 OSU"; $resn_exch{"1H2  OTA"}=" H21 OTA";
$resn_exch{"2H2  OTA"}="H22A OTA"; $resn_exch{"1H5  OTA"}=" H51 OTA"; $resn_exch{"2H5  OTA"}=" H52 OTA";
$resn_exch{"1H6  OTA"}=" H61 OTA"; $resn_exch{"2H6  OTA"}=" H62 OTA"; $resn_exch{"1H1  OTB"}=" H11 OTB";
$resn_exch{"2H1  OTB"}=" H12 OTB"; $resn_exch{"3H1  OTB"}=" H13 OTB"; $resn_exch{"1H2  OTB"}=" H21 OTB";
$resn_exch{"2H2  OTB"}=" H22 OTB"; $resn_exch{"3H2  OTB"}=" H23 OTB"; $resn_exch{"1H3  OTB"}=" H31 OTB";
$resn_exch{"2H3  OTB"}=" H32 OTB"; $resn_exch{"3H3  OTB"}=" H33 OTB"; $resn_exch{"1H1  OTE"}=" H11 OTE";
$resn_exch{"2H1  OTE"}=" H12 OTE"; $resn_exch{"3H1  OTE"}=" H13 OTE"; $resn_exch{"1H2  OTE"}="H21A OTE";
$resn_exch{"2H2  OTE"}=" H22 OTE"; $resn_exch{"1H3  OTE"}=" H31 OTE"; $resn_exch{"2H3  OTE"}=" H32 OTE";
$resn_exch{"1H4  OTE"}=" H41 OTE"; $resn_exch{"2H4  OTE"}=" H42 OTE"; $resn_exch{"1H5  OTE"}=" H51 OTE";
$resn_exch{"2H5  OTE"}=" H52 OTE"; $resn_exch{"1H6  OTE"}=" H61 OTE"; $resn_exch{"2H6  OTE"}=" H62 OTE";
$resn_exch{"1H7  OTE"}=" H71 OTE"; $resn_exch{"2H7  OTE"}=" H72 OTE"; $resn_exch{"1H8  OTE"}=" H81 OTE";
$resn_exch{"2H8  OTE"}=" H82 OTE"; $resn_exch{"1H10 OTE"}="H101 OTE"; $resn_exch{"2H10 OTE"}="H102 OTE";
$resn_exch{"1H11 OTE"}="H111 OTE"; $resn_exch{"2H11 OTE"}="H112 OTE"; $resn_exch{"1H13 OTE"}="H131 OTE";
$resn_exch{"2H13 OTE"}="H132 OTE"; $resn_exch{"1H14 OTE"}="H141 OTE"; $resn_exch{"2H14 OTE"}="H142 OTE";
$resn_exch{"1H16 OTE"}="H161 OTE"; $resn_exch{"2H16 OTE"}="H162 OTE"; $resn_exch{"1H17 OTE"}="H171 OTE";
$resn_exch{"2H17 OTE"}="H172 OTE"; $resn_exch{"1H19 OTE"}="H191 OTE"; $resn_exch{"2H19 OTE"}="H192 OTE";
$resn_exch{"1H20 OTE"}="H201 OTE"; $resn_exch{"2H20 OTE"}="H202 OTE"; $resn_exch{"1H6  OTG"}=" H61 OTG";
$resn_exch{"2H6  OTG"}=" H62 OTG"; $resn_exch{" H3* OTG"}=" H3' OTG"; $resn_exch{" H4* OTG"}=" H4' OTG";
$resn_exch{" H5* OTG"}=" H5' OTG"; $resn_exch{" H6* OTG"}=" H6' OTG"; $resn_exch{"1H7* OTG"}="H7'1 OTG";
$resn_exch{"2H7* OTG"}="H7'2 OTG"; $resn_exch{"3H7* OTG"}="H7'3 OTG"; $resn_exch{"1H40 OTP"}="H401 OTP";
$resn_exch{"2H40 OTP"}="H402 OTP"; $resn_exch{"1H38 OTP"}="H381 OTP"; $resn_exch{"2H38 OTP"}="H382 OTP";
$resn_exch{"3H38 OTP"}="H383 OTP"; $resn_exch{"1H36 OTP"}="H361 OTP"; $resn_exch{"2H36 OTP"}="H362 OTP";
$resn_exch{"1H35 OTP"}="H351 OTP"; $resn_exch{"2H35 OTP"}="H352 OTP"; $resn_exch{"1H33 OTP"}="H331 OTP";
$resn_exch{"2H33 OTP"}="H332 OTP"; $resn_exch{"3H33 OTP"}="H333 OTP"; $resn_exch{"1H31 OTP"}="H311 OTP";
$resn_exch{"2H31 OTP"}="H312 OTP"; $resn_exch{"1H30 OTP"}="H301 OTP"; $resn_exch{"2H30 OTP"}="H302 OTP";
$resn_exch{"1H28 OTP"}="H281 OTP"; $resn_exch{"2H28 OTP"}="H282 OTP"; $resn_exch{"3H28 OTP"}="H283 OTP";
$resn_exch{"1H26 OTP"}="H261 OTP"; $resn_exch{"2H26 OTP"}="H262 OTP"; $resn_exch{"1H25 OTP"}="H251 OTP";
$resn_exch{"2H25 OTP"}="H252 OTP"; $resn_exch{"1H23 OTP"}="H231 OTP"; $resn_exch{"2H23 OTP"}="H232 OTP";
$resn_exch{"3H23 OTP"}="H233 OTP"; $resn_exch{"1H21 OTP"}="H211 OTP"; $resn_exch{"2H21 OTP"}="H212 OTP";
$resn_exch{"1H20 OTP"}="H201 OTP"; $resn_exch{"2H20 OTP"}="H202 OTP"; $resn_exch{"1H18 OTP"}="H181 OTP";
$resn_exch{"2H18 OTP"}="H182 OTP"; $resn_exch{"3H18 OTP"}="H183 OTP"; $resn_exch{"1H16 OTP"}="H161 OTP";
$resn_exch{"2H16 OTP"}="H162 OTP"; $resn_exch{"1H15 OTP"}="H151 OTP"; $resn_exch{"2H15 OTP"}="H152 OTP";
$resn_exch{"1H13 OTP"}="H131 OTP"; $resn_exch{"2H13 OTP"}="H132 OTP"; $resn_exch{"3H13 OTP"}="H133 OTP";
$resn_exch{"1H11 OTP"}="H111 OTP"; $resn_exch{"2H11 OTP"}="H112 OTP"; $resn_exch{"1H10 OTP"}="H101 OTP";
$resn_exch{"2H10 OTP"}="H102 OTP"; $resn_exch{"1H8  OTP"}=" H81 OTP"; $resn_exch{"2H8  OTP"}=" H82 OTP";
$resn_exch{"3H8  OTP"}=" H83 OTP"; $resn_exch{"1H6  OTP"}=" H61 OTP"; $resn_exch{"2H6  OTP"}=" H62 OTP";
$resn_exch{"1H5  OTP"}=" H51 OTP"; $resn_exch{"2H5  OTP"}=" H52 OTP"; $resn_exch{"1H3  OTP"}=" H31 OTP";
$resn_exch{"2H3  OTP"}=" H32 OTP"; $resn_exch{"3H3  OTP"}=" H33 OTP"; $resn_exch{"1H1  OTP"}=" H11 OTP";
$resn_exch{"2H1  OTP"}=" H12 OTP"; $resn_exch{"3H1  OTP"}=" H13 OTP"; $resn_exch{"1HN8 OTR"}="HN81 OTR";
$resn_exch{"2HN8 OTR"}="HN82 OTR"; $resn_exch{"1H8  OTR"}=" H81 OTR"; $resn_exch{"2H8  OTR"}=" H82 OTR";
$resn_exch{"1H8  OTS"}=" H81 OTS"; $resn_exch{"2H8  OTS"}=" H82 OTS"; $resn_exch{"1HN8 OTS"}="HN81 OTS";
$resn_exch{"2HN8 OTS"}="HN82 OTS"; $resn_exch{" HN2 OTY"}=" H2  OTY"; $resn_exch{"1HB  OTY"}=" HB2 OTY";
$resn_exch{"2HB  OTY"}=" HB3 OTY"; $resn_exch{"1HE2 OTY"}="HE23 OTY"; $resn_exch{"2HE2 OTY"}="HE22 OTY";
$resn_exch{"1HN  OUT"}=" HN1 OUT"; $resn_exch{"2HN  OUT"}=" HN2 OUT"; $resn_exch{"1H5  OVA"}=" H51 OVA";
$resn_exch{"2H5  OVA"}=" H52 OVA"; $resn_exch{"1H6  OVA"}=" H61 OVA"; $resn_exch{"2H6  OVA"}=" H62 OVA";
$resn_exch{"1H11 OVA"}="H111 OVA"; $resn_exch{"2H11 OVA"}="H112 OVA"; $resn_exch{"3H11 OVA"}="H113 OVA";
$resn_exch{"1H23 OVA"}="H231 OVA"; $resn_exch{"2H23 OVA"}="H232 OVA"; $resn_exch{"1HA  OVA"}=" HA1 OVA";
$resn_exch{"2HA  OVA"}=" HA2 OVA"; $resn_exch{"3HA  OVA"}=" HA3 OVA"; $resn_exch{"1HB  OVA"}=" HB1 OVA";
$resn_exch{"2HB  OVA"}=" HB2 OVA"; $resn_exch{"3HB  OVA"}=" HB3 OVA"; $resn_exch{"1HC  OVA"}=" HC1 OVA";
$resn_exch{"2HC  OVA"}=" HC2 OVA"; $resn_exch{"3HC  OVA"}=" HC3 OVA"; $resn_exch{"1H1  OVA"}=" H11 OVA";
$resn_exch{"2H1  OVA"}="H12A OVA"; $resn_exch{"3H1  OVA"}=" H13 OVA"; $resn_exch{"1H6  OX2"}=" H61 OX2";
$resn_exch{"2H6  OX2"}=" H62 OX2"; $resn_exch{"1H9  OX2"}=" H91 OX2"; $resn_exch{"2H9  OX2"}=" H92 OX2";
$resn_exch{"3H9  OX2"}=" H93 OX2"; $resn_exch{"1H12 OX3"}="H121 OX3"; $resn_exch{"2H12 OX3"}="H122 OX3";
$resn_exch{"1H13 OX3"}="H131 OX3"; $resn_exch{"2H13 OX3"}="H132 OX3"; $resn_exch{"1H14 OX3"}="H141 OX3";
$resn_exch{"2H14 OX3"}="H142 OX3"; $resn_exch{"1H15 OX3"}="H151 OX3"; $resn_exch{"2H15 OX3"}="H152 OX3";
$resn_exch{"1H16 OX3"}="H161 OX3"; $resn_exch{"2H16 OX3"}="H162 OX3"; $resn_exch{"1H12 OX4"}="H121 OX4";
$resn_exch{"2H12 OX4"}="H122 OX4"; $resn_exch{"1H13 OX4"}="H131 OX4"; $resn_exch{"2H13 OX4"}="H132 OX4";
$resn_exch{"1H14 OX4"}="H141 OX4"; $resn_exch{"2H14 OX4"}="H142 OX4"; $resn_exch{"1H15 OX4"}="H151 OX4";
$resn_exch{"2H15 OX4"}="H152 OX4"; $resn_exch{"1H16 OX4"}="H161 OX4"; $resn_exch{"2H16 OX4"}="H162 OX4";
$resn_exch{"1H8  OX5"}=" H81 OX5"; $resn_exch{"2H8  OX5"}=" H82 OX5"; $resn_exch{"1H9  OX5"}=" H91 OX5";
$resn_exch{"2H9  OX5"}=" H92 OX5"; $resn_exch{"3H9  OX5"}=" H93 OX5"; $resn_exch{"1H11 OX5"}="H111 OX5";
$resn_exch{"2H11 OX5"}="H112 OX5"; $resn_exch{"3H11 OX5"}="H113 OX5"; $resn_exch{"1H13 OX5"}="H131 OX5";
$resn_exch{"2H13 OX5"}="H132 OX5"; $resn_exch{"3H13 OX5"}="H133 OX5"; $resn_exch{"1H14 OX6"}="H141 OX6";
$resn_exch{"2H14 OX6"}="H142 OX6"; $resn_exch{"3H14 OX6"}="H143 OX6"; $resn_exch{"1H13 OX6"}="H131 OX6";
$resn_exch{"2H13 OX6"}="H132 OX6"; $resn_exch{"3H13 OX6"}="H133 OX6"; $resn_exch{"1H11 OX6"}="H111 OX6";
$resn_exch{"2H11 OX6"}="H112 OX6"; $resn_exch{"3H11 OX6"}="H113 OX6"; $resn_exch{"1H17 OX6"}="H171 OX6";
$resn_exch{"2H17 OX6"}="H172 OX6"; $resn_exch{"3H17 OX6"}="H173 OX6"; $resn_exch{"1H16 OX6"}="H161 OX6";
$resn_exch{"2H16 OX6"}="H162 OX6"; $resn_exch{"3H16 OX6"}="H163 OX6"; $resn_exch{"1H14 OX7"}="H141 OX7";
$resn_exch{"2H14 OX7"}="H142 OX7"; $resn_exch{"3H14 OX7"}="H143 OX7"; $resn_exch{"1H13 OX7"}="H131 OX7";
$resn_exch{"2H13 OX7"}="H132 OX7"; $resn_exch{"3H13 OX7"}="H133 OX7"; $resn_exch{"1H11 OX7"}="H111 OX7";
$resn_exch{"2H11 OX7"}="H112 OX7"; $resn_exch{"3H11 OX7"}="H113 OX7"; $resn_exch{"1H17 OX7"}="H171 OX7";
$resn_exch{"2H17 OX7"}="H172 OX7"; $resn_exch{"3H17 OX7"}="H173 OX7"; $resn_exch{"1H16 OX7"}="H161 OX7";
$resn_exch{"2H16 OX7"}="H162 OX7"; $resn_exch{"3H16 OX7"}="H163 OX7"; $resn_exch{"1H1  OXA"}=" H11 OXA";
$resn_exch{"2H1  OXA"}=" H12 OXA"; $resn_exch{"1H2  OXA"}=" H21 OXA"; $resn_exch{"2H2  OXA"}=" H22 OXA";
$resn_exch{"3H2  OXA"}=" H23 OXA"; $resn_exch{"1H1' OXE"}="H1'1 OXE"; $resn_exch{"2H1' OXE"}="H1'2 OXE";
$resn_exch{"3H1' OXE"}="H1'3 OXE"; $resn_exch{"1H2' OXE"}="H2'1 OXE"; $resn_exch{"2H2' OXE"}="H2'2 OXE";
$resn_exch{"3H2' OXE"}="H2'3 OXE"; $resn_exch{"1H2  OXG"}=" H21 OXG"; $resn_exch{"2H2  OXG"}=" H22 OXG";
$resn_exch{"1H11 OXI"}="H111 OXI"; $resn_exch{"2H11 OXI"}="H112 OXI"; $resn_exch{"1H12 OXI"}="H121 OXI";
$resn_exch{"2H12 OXI"}="H122 OXI"; $resn_exch{"1H13 OXI"}="H131 OXI"; $resn_exch{"2H13 OXI"}="H132 OXI";
$resn_exch{"3H13 OXI"}="H133 OXI"; $resn_exch{"1H6N OXK"}="H6N1 OXK"; $resn_exch{"2H6N OXK"}="H6N2 OXK";
$resn_exch{"1H5' OXK"}="H5'1 OXK"; $resn_exch{"2H5' OXK"}="H5'2 OXK"; $resn_exch{"1HPB OXK"}="HPB1 OXK";
$resn_exch{"2HPB OXK"}="HPB2 OXK"; $resn_exch{"1HP9 OXK"}="HP91 OXK"; $resn_exch{"2HP9 OXK"}="HP92 OXK";
$resn_exch{"3HP9 OXK"}="HP93 OXK"; $resn_exch{"1HP8 OXK"}="HP81 OXK"; $resn_exch{"2HP8 OXK"}="HP82 OXK";
$resn_exch{"3HP8 OXK"}="HP83 OXK"; $resn_exch{"1HP5 OXK"}="HP51 OXK"; $resn_exch{"2HP5 OXK"}="HP52 OXK";
$resn_exch{"1HP4 OXK"}="HP41 OXK"; $resn_exch{"2HP4 OXK"}="HP42 OXK"; $resn_exch{"1HP2 OXK"}="HP21 OXK";
$resn_exch{"2HP2 OXK"}="HP22 OXK"; $resn_exch{"1HP1 OXK"}="HP11 OXK"; $resn_exch{"2HP1 OXK"}="HP12 OXK";
$resn_exch{"1HN  OXM"}=" HN1 OXM"; $resn_exch{"2HN  OXM"}=" HN2 OXM"; $resn_exch{"1H5  OXN"}=" H51 OXN";
$resn_exch{"2H5  OXN"}=" H52 OXN"; $resn_exch{"1H3  OXN"}=" H31 OXN"; $resn_exch{"2H3  OXN"}=" H32 OXN";
$resn_exch{"3H3  OXN"}=" H33 OXN"; $resn_exch{"1H4  OXN"}=" H41 OXN"; $resn_exch{"2H4  OXN"}=" H42 OXN";
$resn_exch{"3H4  OXN"}=" H43 OXN"; $resn_exch{"1H2  OXN"}=" H21 OXN"; $resn_exch{"2H2  OXN"}=" H22 OXN";
$resn_exch{"3H2  OXN"}=" H23 OXN"; $resn_exch{"1H8  OXN"}=" H81 OXN"; $resn_exch{"2H8  OXN"}=" H82 OXN";
$resn_exch{"3H8  OXN"}=" H83 OXN"; $resn_exch{"1H7  OXN"}=" H71 OXN"; $resn_exch{"2H7  OXN"}=" H72 OXN";
$resn_exch{"3H7  OXN"}=" H73 OXN"; $resn_exch{"1H16 OXN"}="H161 OXN"; $resn_exch{"2H16 OXN"}="H162 OXN";
$resn_exch{"1H17 OXN"}="H171 OXN"; $resn_exch{"2H17 OXN"}="H172 OXN"; $resn_exch{"1H19 OXN"}="H191 OXN";
$resn_exch{"2H19 OXN"}="H192 OXN"; $resn_exch{"1H20 OXN"}="H201 OXN"; $resn_exch{"2H20 OXN"}="H202 OXN";
$resn_exch{"1H22 OXN"}="H221 OXN"; $resn_exch{"2H22 OXN"}="H222 OXN"; $resn_exch{"1H23 OXN"}="H231 OXN";
$resn_exch{"2H23 OXN"}="H232 OXN"; $resn_exch{"1H25 OXN"}="H251 OXN"; $resn_exch{"2H25 OXN"}="H252 OXN";
$resn_exch{"1H26 OXN"}="H261 OXN"; $resn_exch{"2H26 OXN"}="H262 OXN"; $resn_exch{"1H28 OXN"}="H281 OXN";
$resn_exch{"2H28 OXN"}="H282 OXN"; $resn_exch{"1H29 OXN"}="H291 OXN"; $resn_exch{"2H29 OXN"}="H292 OXN";
$resn_exch{"1H31 OXN"}="H311 OXN"; $resn_exch{"2H31 OXN"}="H312 OXN"; $resn_exch{"1H32 OXN"}="H321 OXN";
$resn_exch{"2H32 OXN"}="H322 OXN"; $resn_exch{"1H34 OXN"}="H341 OXN"; $resn_exch{"2H34 OXN"}="H342 OXN";
$resn_exch{"1H35 OXN"}="H351 OXN"; $resn_exch{"2H35 OXN"}="H352 OXN"; $resn_exch{"1H37 OXN"}="H371 OXN";
$resn_exch{"2H37 OXN"}="H372 OXN"; $resn_exch{"1H38 OXN"}="H381 OXN"; $resn_exch{"2H38 OXN"}="H382 OXN";
$resn_exch{"1H40 OXN"}="H401 OXN"; $resn_exch{"2H40 OXN"}="H402 OXN"; $resn_exch{"1H41 OXN"}="H411 OXN";
$resn_exch{"2H41 OXN"}="H412 OXN"; $resn_exch{"1H43 OXN"}="H431 OXN"; $resn_exch{"2H43 OXN"}="H432 OXN";
$resn_exch{"1H44 OXN"}="H441 OXN"; $resn_exch{"2H44 OXN"}="H442 OXN"; $resn_exch{"1H5  OXP"}=" H51 OXP";
$resn_exch{"2H5  OXP"}=" H52 OXP"; $resn_exch{"3H5  OXP"}=" H53 OXP"; $resn_exch{"1H3  OXR"}=" H31 OXR";
$resn_exch{"2H3  OXR"}=" H32 OXR"; $resn_exch{"1H2  OXR"}=" H21 OXR"; $resn_exch{"2H2  OXR"}=" H22 OXR";
$resn_exch{"1H17 OXR"}="H171 OXR"; $resn_exch{"2H17 OXR"}="H172 OXR"; $resn_exch{"3H17 OXR"}="H173 OXR";
$resn_exch{"1H16 OXR"}="H161 OXR"; $resn_exch{"2H16 OXR"}="H162 OXR"; $resn_exch{"3H16 OXR"}="H163 OXR";
$resn_exch{"1H18 OXR"}="H181 OXR"; $resn_exch{"2H18 OXR"}="H182 OXR"; $resn_exch{"3H18 OXR"}="H183 OXR";
$resn_exch{"1H19 OXR"}="H191 OXR"; $resn_exch{"2H19 OXR"}="H192 OXR"; $resn_exch{"3H19 OXR"}="H193 OXR";
$resn_exch{"1H20 OXR"}="H201 OXR"; $resn_exch{"2H20 OXR"}="H202 OXR"; $resn_exch{"3H20 OXR"}="H203 OXR";
$resn_exch{"1H15 OXR"}="H151 OXR"; $resn_exch{"2H15 OXR"}="H152 OXR"; $resn_exch{"1H3  OXS"}=" H31 OXS";
$resn_exch{"2H3  OXS"}=" H32 OXS"; $resn_exch{" H3* OXT"}=" H3' OXT"; $resn_exch{" H2* OXT"}=" H2' OXT";
$resn_exch{" H4* OXT"}=" H4' OXT"; $resn_exch{" H1* OXT"}=" H1' OXT"; $resn_exch{"1H62 OXT"}="H621 OXT";
$resn_exch{"2H62 OXT"}="H622 OXT"; $resn_exch{"1H5* OXT"}="H5'1 OXT"; $resn_exch{"2H5* OXT"}="H5'2 OXT";
$resn_exch{"1H32 OXT"}="H321 OXT"; $resn_exch{"2H32 OXT"}="H322 OXT"; $resn_exch{"1H33 OXT"}="H331 OXT";
$resn_exch{"2H33 OXT"}="H332 OXT"; $resn_exch{"3H33 OXT"}="H333 OXT"; $resn_exch{"1H34 OXT"}="H341 OXT";
$resn_exch{"2H34 OXT"}="H342 OXT"; $resn_exch{"3H34 OXT"}="H343 OXT"; $resn_exch{"1H74 OXT"}="H741 OXT";
$resn_exch{"2H74 OXT"}="H742 OXT"; $resn_exch{"1H64 OXT"}="H641 OXT"; $resn_exch{"2H64 OXT"}="H642 OXT";
$resn_exch{"1H35 OXT"}="H351 OXT"; $resn_exch{"2H35 OXT"}="H352 OXT"; $resn_exch{"1H25 OXT"}="H251 OXT";
$resn_exch{"2H25 OXT"}="H252 OXT"; $resn_exch{"1H71 OXT"}="H711 OXT"; $resn_exch{"2H71 OXT"}="H712 OXT";
$resn_exch{"1H41 OXT"}="H411 OXT"; $resn_exch{"2H41 OXT"}="H412 OXT"; $resn_exch{"1HM2 OXT"}="HM21 OXT";
$resn_exch{"2HM2 OXT"}="HM22 OXT"; $resn_exch{"3HM2 OXT"}="HM23 OXT"; $resn_exch{"1HM4 OXT"}="HM41 OXT";
$resn_exch{"2HM4 OXT"}="HM42 OXT"; $resn_exch{"3HM4 OXT"}="HM43 OXT"; $resn_exch{"1H66 OXT"}="H661 OXT";
$resn_exch{"2H66 OXT"}="H662 OXT"; $resn_exch{"1H76 OXT"}="H761 OXT"; $resn_exch{"2H76 OXT"}="H762 OXT";
$resn_exch{"1HN  OXX"}=" H   OXX"; $resn_exch{"2HN  OXX"}=" H2  OXX"; $resn_exch{"1HB  OXX"}=" HB2 OXX";
$resn_exch{"2HB  OXX"}=" HB3 OXX"; $resn_exch{"1H1C OXZ"}="H1C1 OXZ"; $resn_exch{"2H1C OXZ"}="H1C2 OXZ";
$resn_exch{"1H5C OXZ"}="H5C1 OXZ"; $resn_exch{"2H5C OXZ"}="H5C2 OXZ"; $resn_exch{"1H14 OYA"}="H141 OYA";
$resn_exch{"2H14 OYA"}="H142 OYA"; $resn_exch{"3H14 OYA"}="H143 OYA"; $resn_exch{"1H5  OYA"}=" H51 OYA";
$resn_exch{"2H5  OYA"}=" H52 OYA"; $resn_exch{"1H6  OYA"}=" H61 OYA"; $resn_exch{"2H6  OYA"}=" H62 OYA";
$resn_exch{"1H11 OYA"}="H111 OYA"; $resn_exch{"2H11 OYA"}="H112 OYA"; $resn_exch{"1H7  OYA"}=" H71 OYA";
$resn_exch{"2H7  OYA"}=" H72 OYA"; $resn_exch{"1H8  OYA"}=" H81 OYA"; $resn_exch{"2H8  OYA"}=" H82 OYA";
$resn_exch{"1H9  OYA"}=" H91 OYA"; $resn_exch{"2H9  OYA"}=" H92 OYA"; $resn_exch{"H5'1   P"}=" H5'   P";
$resn_exch{"H5'2   P"}="H5''   P"; $resn_exch{"H2'1   P"}=" H2'   P"; $resn_exch{"H2'2   P"}="H2''   P";
$resn_exch{"1H11 P01"}="H111 P01"; $resn_exch{"2H11 P01"}="H112 P01"; $resn_exch{"3H11 P01"}="H113 P01";
$resn_exch{"1H12 P01"}="H121 P01"; $resn_exch{"2H12 P01"}="H122 P01"; $resn_exch{"3H12 P01"}="H123 P01";
$resn_exch{"1H26 P01"}="H261 P01"; $resn_exch{"2H26 P01"}="H262 P01"; $resn_exch{"3H26 P01"}="H263 P01";
$resn_exch{"1H27 P01"}="H271 P01"; $resn_exch{"2H27 P01"}="H272 P01"; $resn_exch{"3H27 P01"}="H273 P01";
$resn_exch{"1H23 P01"}="H231 P01"; $resn_exch{"2H23 P01"}="H232 P01"; $resn_exch{"1H3  P0E"}="H31A P0E";
$resn_exch{"2H3  P0E"}="H32A P0E"; $resn_exch{"1H4  P0E"}=" H41 P0E"; $resn_exch{"2H4  P0E"}=" H42 P0E";
$resn_exch{"1H15 P0E"}="H151 P0E"; $resn_exch{"2H15 P0E"}="H152 P0E"; $resn_exch{"1H16 P0E"}="H161 P0E";
$resn_exch{"2H16 P0E"}="H162 P0E"; $resn_exch{"1H17 P0E"}="H171 P0E"; $resn_exch{"2H17 P0E"}="H172 P0E";
$resn_exch{"1H18 P0E"}="H181 P0E"; $resn_exch{"2H18 P0E"}="H182 P0E"; $resn_exch{"1H19 P0E"}="H191 P0E";
$resn_exch{"2H19 P0E"}="H192 P0E"; $resn_exch{"1H20 P0E"}="H201 P0E"; $resn_exch{"2H20 P0E"}="H202 P0E";
$resn_exch{"1H21 P0E"}="H211 P0E"; $resn_exch{"2H21 P0E"}="H212 P0E"; $resn_exch{"1H24 P0E"}="H241 P0E";
$resn_exch{"2H24 P0E"}="H242 P0E"; $resn_exch{"1H25 P0E"}="H251 P0E"; $resn_exch{"2H25 P0E"}="H252 P0E";
$resn_exch{"1H26 P0E"}="H261 P0E"; $resn_exch{"2H26 P0E"}="H262 P0E"; $resn_exch{"1H27 P0E"}="H271 P0E";
$resn_exch{"2H27 P0E"}="H272 P0E"; $resn_exch{"1H28 P0E"}="H281 P0E"; $resn_exch{"2H28 P0E"}="H282 P0E";
$resn_exch{"1H29 P0E"}="H291 P0E"; $resn_exch{"2H29 P0E"}="H292 P0E"; $resn_exch{"1H30 P0E"}="H301 P0E";
$resn_exch{"2H30 P0E"}="H302 P0E"; $resn_exch{"1H33 P0E"}="H331 P0E"; $resn_exch{"2H33 P0E"}="H332 P0E";
$resn_exch{"1H34 P0E"}="H341 P0E"; $resn_exch{"2H34 P0E"}="H342 P0E"; $resn_exch{"1H35 P0E"}="H351 P0E";
$resn_exch{"2H35 P0E"}="H352 P0E"; $resn_exch{"1H36 P0E"}="H361 P0E"; $resn_exch{"2H36 P0E"}="H362 P0E";
$resn_exch{"1H37 P0E"}="H371 P0E"; $resn_exch{"2H37 P0E"}="H372 P0E"; $resn_exch{"1H38 P0E"}="H381 P0E";
$resn_exch{"2H38 P0E"}="H382 P0E"; $resn_exch{"1H39 P0E"}="H391 P0E"; $resn_exch{"2H39 P0E"}="H392 P0E";
$resn_exch{"1H40 P0E"}="H401 P0E"; $resn_exch{"2H40 P0E"}="H402 P0E"; $resn_exch{"1H41 P0E"}="H411 P0E";
$resn_exch{"2H41 P0E"}="H412 P0E"; $resn_exch{"3H41 P0E"}="H413 P0E"; $resn_exch{"1H42 P0E"}="H421 P0E";
$resn_exch{"2H42 P0E"}="H422 P0E"; $resn_exch{"1H43 P0E"}="H431 P0E"; $resn_exch{"2H43 P0E"}="H432 P0E";
$resn_exch{"1H44 P0E"}="H441 P0E"; $resn_exch{"2H44 P0E"}="H442 P0E"; $resn_exch{"1H45 P0E"}="H451 P0E";
$resn_exch{"2H45 P0E"}="H452 P0E"; $resn_exch{"1H46 P0E"}="H461 P0E"; $resn_exch{"2H46 P0E"}="H462 P0E";
$resn_exch{"3H46 P0E"}="H463 P0E"; $resn_exch{"1H47 P0E"}="H471 P0E"; $resn_exch{"2H47 P0E"}="H472 P0E";
$resn_exch{"1H48 P0E"}="H481 P0E"; $resn_exch{"2H48 P0E"}="H482 P0E"; $resn_exch{"1H2A P0P"}="H2A1 P0P";
$resn_exch{"2H2A P0P"}="H2A2 P0P"; $resn_exch{"3H2A P0P"}="H2A3 P0P"; $resn_exch{"1H5A P0P"}="H5A1 P0P";
$resn_exch{"2H5A P0P"}="H5A2 P0P"; $resn_exch{"1H02 P10"}="H021 P10"; $resn_exch{"2H02 P10"}="H022 P10";
$resn_exch{"1H15 P10"}="H151 P10"; $resn_exch{"2H15 P10"}="H152 P10"; $resn_exch{"1H16 P10"}="H161 P10";
$resn_exch{"2H16 P10"}="H162 P10"; $resn_exch{"1H27 P10"}="H271 P10"; $resn_exch{"2H27 P10"}="H272 P10";
$resn_exch{"1H38 P10"}="H381 P10"; $resn_exch{"2H38 P10"}="H382 P10"; $resn_exch{"1H3  P11"}=" H31 P11";
$resn_exch{"2H3  P11"}="H32A P11"; $resn_exch{"3H3  P11"}=" H33 P11"; $resn_exch{"1H11 P11"}="H111 P11";
$resn_exch{"2H11 P11"}="H112 P11"; $resn_exch{"3H11 P11"}="H113 P11"; $resn_exch{"1H13 P11"}="H131 P11";
$resn_exch{"2H13 P11"}="H132 P11"; $resn_exch{"1H14 P11"}="H141 P11"; $resn_exch{"2H14 P11"}="H142 P11";
$resn_exch{"1H17 P11"}="H171 P11"; $resn_exch{"2H17 P11"}="H172 P11"; $resn_exch{"3H17 P11"}="H173 P11";
$resn_exch{"1H21 P11"}="H211 P11"; $resn_exch{"2H21 P11"}="H212 P11"; $resn_exch{"1H22 P11"}="H221 P11";
$resn_exch{"2H22 P11"}="H222 P11"; $resn_exch{"NH10 P11"}="H10N P11"; $resn_exch{"1H25 P11"}="H251 P11";
$resn_exch{"2H25 P11"}="H252 P11"; $resn_exch{"3H25 P11"}="H253 P11"; $resn_exch{"NH11 P11"}="H11N P11";
$resn_exch{"1H33 P11"}="H331 P11"; $resn_exch{"2H33 P11"}="H332 P11"; $resn_exch{"3H33 P11"}="H333 P11";
$resn_exch{"NH14 P11"}="H14N P11"; $resn_exch{"1H35 P11"}="H351 P11"; $resn_exch{"2H35 P11"}="H352 P11";
$resn_exch{"1H36 P11"}="H361 P11"; $resn_exch{"2H36 P11"}="H362 P11"; $resn_exch{"NH16 P11"}="H16N P11";
$resn_exch{"1H37 P11"}="H371 P11"; $resn_exch{"2H37 P11"}="H372 P11"; $resn_exch{"1H38 P11"}="H381 P11";
$resn_exch{"2H38 P11"}="H382 P11"; $resn_exch{"1H39 P11"}="H391 P11"; $resn_exch{"2H39 P11"}="H392 P11";
$resn_exch{"NH17 P11"}="H17N P11"; $resn_exch{"1H40 P11"}="H401 P11"; $resn_exch{"2H40 P11"}="H402 P11";
$resn_exch{"3H40 P11"}="H403 P11"; $resn_exch{"1H41 P11"}="H411 P11"; $resn_exch{"2H41 P11"}="H412 P11";
$resn_exch{"3H41 P11"}="H413 P11"; $resn_exch{"1HB  P12"}=" HB1 P12"; $resn_exch{"2HB  P12"}=" HB2 P12";
$resn_exch{"1HG  P12"}=" HG1 P12"; $resn_exch{"2HG  P12"}=" HG2 P12"; $resn_exch{"1HD  P12"}=" HD1 P12";
$resn_exch{"2HD  P12"}=" HD2 P12"; $resn_exch{"1HE  P12"}=" HE1 P12"; $resn_exch{"2HE  P12"}=" HE2 P12";
$resn_exch{"3HE  P12"}=" HE3 P12"; $resn_exch{"1HG1 P12"}="HG11 P12"; $resn_exch{"2HG1 P12"}="HG12 P12";
$resn_exch{"1HD1 P12"}="HD11 P12"; $resn_exch{"2HD1 P12"}="HD12 P12"; $resn_exch{"1HD2 P12"}="HD21 P12";
$resn_exch{"2HD2 P12"}="HD22 P12"; $resn_exch{"1HG2 P12"}="HG21 P12"; $resn_exch{"2HG2 P12"}="HG22 P12";
$resn_exch{"1HA  P12"}=" HA1 P12"; $resn_exch{"2HA  P12"}=" HA2 P12"; $resn_exch{"1HH1 P12"}="HH11 P12";
$resn_exch{"1HH2 P12"}="HH21 P12"; $resn_exch{"2HH2 P12"}="HH22 P12"; $resn_exch{"1HB1 P12"}="HB11 P12";
$resn_exch{"2HB1 P12"}="HB12 P12"; $resn_exch{"3HB  P12"}=" HB3 P12"; $resn_exch{"1HN  P13"}="HN1A P13";
$resn_exch{"2HN  P13"}=" HN2 P13"; $resn_exch{"1HB  P13"}=" HB1 P13"; $resn_exch{"2HB  P13"}=" HB2 P13";
$resn_exch{"1HA  P13"}=" HA1 P13"; $resn_exch{"2HA  P13"}=" HA2 P13"; $resn_exch{"1HC  P13"}=" HC1 P13";
$resn_exch{"2HC  P13"}=" HC2 P13"; $resn_exch{"1HA1 P13"}="HA11 P13"; $resn_exch{"2HA1 P13"}="HA12 P13";
$resn_exch{"1HB1 P13"}="HB11 P13"; $resn_exch{"1HG  P13"}=" HG1 P13"; $resn_exch{"2HG  P13"}=" HG2 P13";
$resn_exch{"1HD  P13"}=" HD1 P13"; $resn_exch{"2HD  P13"}=" HD2 P13"; $resn_exch{"1HH1 P13"}="HH11 P13";
$resn_exch{"2HH1 P13"}="HH12 P13"; $resn_exch{"1HI1 P13"}="HI11 P13"; $resn_exch{"2HI1 P13"}="HI12 P13";
$resn_exch{"1HJ  P13"}=" HJ1 P13"; $resn_exch{"1HI2 P13"}="HI21 P13"; $resn_exch{"2HI2 P13"}="HI22 P13";
$resn_exch{"1HH2 P13"}="HH21 P13"; $resn_exch{"2HH2 P13"}="HH22 P13"; $resn_exch{"1HB  P14"}=" HB1 P14";
$resn_exch{"2HB  P14"}=" HB2 P14"; $resn_exch{"1HG  P14"}=" HG1 P14"; $resn_exch{"2HG  P14"}=" HG2 P14";
$resn_exch{"1HZ1 P14"}="HZ11 P14"; $resn_exch{"2HZ1 P14"}="HZ12 P14"; $resn_exch{"1HZ2 P14"}="HZ21 P14";
$resn_exch{"2HZ2 P14"}="HZ22 P14"; $resn_exch{"1HA  P14"}=" HA1 P14"; $resn_exch{"2HA  P14"}=" HA2 P14";
$resn_exch{"1HB1 P14"}="HB11 P14"; $resn_exch{"1HG1 P14"}="HG11 P14"; $resn_exch{"2HG1 P14"}="HG12 P14";
$resn_exch{"1HD  P14"}=" HD1 P14"; $resn_exch{"2HD  P14"}=" HD2 P14"; $resn_exch{"1HE  P14"}=" HE1 P14";
$resn_exch{"2HE  P14"}=" HE2 P14"; $resn_exch{"1HZ  P14"}=" HZ1 P14"; $resn_exch{"2HZ  P14"}=" HZ2 P14";
$resn_exch{"1HJ1 P14"}="HJ11 P14"; $resn_exch{"2HJ1 P14"}="HJ12 P14"; $resn_exch{"1HJ2 P14"}="HJ21 P14";
$resn_exch{"2HJ2 P14"}="HJ22 P14"; $resn_exch{"1H1  P14"}=" H11 P14"; $resn_exch{"2H1  P14"}=" H12 P14";
$resn_exch{"3H1  P14"}=" H13 P14"; $resn_exch{"1H13 P15"}="H131 P15"; $resn_exch{"2H13 P15"}="H132 P15";
$resn_exch{"3H13 P15"}="H133 P15"; $resn_exch{"1H12 P15"}="H121 P15"; $resn_exch{"2H12 P15"}="H122 P15";
$resn_exch{"1H11 P15"}="H111 P15"; $resn_exch{"2H11 P15"}="H112 P15"; $resn_exch{"1H10 P15"}="H101 P15";
$resn_exch{"2H10 P15"}="H102 P15"; $resn_exch{"1H9  P15"}=" H91 P15"; $resn_exch{"2H9  P15"}=" H92 P15";
$resn_exch{"1H8  P15"}=" H81 P15"; $resn_exch{"2H8  P15"}=" H82 P15"; $resn_exch{"1H7  P15"}=" H71 P15";
$resn_exch{"2H7  P15"}=" H72 P15"; $resn_exch{"1H6  P15"}=" H61 P15"; $resn_exch{"2H6  P15"}=" H62 P15";
$resn_exch{"1H5  P15"}=" H51 P15"; $resn_exch{"2H5  P15"}=" H52 P15"; $resn_exch{"1H4  P15"}=" H41 P15";
$resn_exch{"2H4  P15"}=" H42 P15"; $resn_exch{"1H3  P15"}=" H31 P15"; $resn_exch{"2H3  P15"}=" H32 P15";
$resn_exch{"1H2  P15"}=" H21 P15"; $resn_exch{"2H2  P15"}=" H22 P15"; $resn_exch{"1H1  P15"}=" H11 P15";
$resn_exch{"2H1  P15"}=" H12 P15"; $resn_exch{"1H2  P16"}=" H21 P16"; $resn_exch{"2H2  P16"}=" H22 P16";
$resn_exch{"1H20 P16"}="H201 P16"; $resn_exch{"2H20 P16"}="H202 P16"; $resn_exch{"3H20 P16"}="H203 P16";
$resn_exch{"1H1  P1A"}=" H11 P1A"; $resn_exch{"2H1  P1A"}=" H12 P1A"; $resn_exch{"1H4  P1A"}=" H41 P1A";
$resn_exch{"2H4  P1A"}=" H42 P1A"; $resn_exch{"1H19 P1A"}="H191 P1A"; $resn_exch{"2H19 P1A"}="H192 P1A";
$resn_exch{"3H19 P1A"}="H193 P1A"; $resn_exch{"1H11 P1A"}="H111 P1A"; $resn_exch{"2H11 P1A"}="H112 P1A";
$resn_exch{"1H12 P1A"}="H121 P1A"; $resn_exch{"2H12 P1A"}="H122 P1A"; $resn_exch{"1H18 P1A"}="H181 P1A";
$resn_exch{"2H18 P1A"}="H182 P1A"; $resn_exch{"3H18 P1A"}="H183 P1A"; $resn_exch{"1H15 P1A"}="H151 P1A";
$resn_exch{"2H15 P1A"}="H152 P1A"; $resn_exch{"1H16 P1A"}="H161 P1A"; $resn_exch{"2H16 P1A"}="H162 P1A";
$resn_exch{"1H21 P1A"}="H211 P1A"; $resn_exch{"2H21 P1A"}="H212 P1A"; $resn_exch{"3H21 P1A"}="H213 P1A";
$resn_exch{"1H23 P1A"}="H231 P1A"; $resn_exch{"2H23 P1A"}="H232 P1A"; $resn_exch{"1H24 P1A"}="H241 P1A";
$resn_exch{"2H24 P1A"}="H242 P1A"; $resn_exch{"1H26 P1A"}="H261 P1A"; $resn_exch{"2H26 P1A"}="H262 P1A";
$resn_exch{"3H26 P1A"}="H263 P1A"; $resn_exch{"1H27 P1A"}="H271 P1A"; $resn_exch{"2H27 P1A"}="H272 P1A";
$resn_exch{"3H27 P1A"}="H273 P1A"; $resn_exch{"1H3C P1C"}="H3C1 P1C"; $resn_exch{"2H3C P1C"}="H3C2 P1C";
$resn_exch{"1H4C P1C"}="H4C1 P1C"; $resn_exch{"2H4C P1C"}="H4C2 P1C"; $resn_exch{"1H7C P1C"}="H7C1 P1C";
$resn_exch{"2H7C P1C"}="H7C2 P1C"; $resn_exch{"1H14 P1C"}="H141 P1C"; $resn_exch{"2H14 P1C"}="H142 P1C";
$resn_exch{"1H33 P1C"}="H331 P1C"; $resn_exch{"2H33 P1C"}="H332 P1C"; $resn_exch{"3H33 P1C"}="H333 P1C";
$resn_exch{"1H37 P1C"}="H371 P1C"; $resn_exch{"2H37 P1C"}="H372 P1C"; $resn_exch{"1H42 P1C"}="H421 P1C";
$resn_exch{"1H1P P1D"}="H1P1 P1D"; $resn_exch{"2H1P P1D"}="H1P2 P1D"; $resn_exch{"1H5' P1D"}="H5'1 P1D";
$resn_exch{"2H5' P1D"}="H5'2 P1D"; $resn_exch{"1H1P P1G"}="H1P1 P1G"; $resn_exch{"2H1P P1G"}="H1P2 P1G";
$resn_exch{"1H5' P1G"}="H5'1 P1G"; $resn_exch{"2H5' P1G"}="H5'2 P1G"; $resn_exch{"1HN2 P1G"}="HN21 P1G";
$resn_exch{"2HN2 P1G"}="HN22 P1G"; $resn_exch{"1H1  P1H"}=" H11 P1H"; $resn_exch{"2H1  P1H"}="H12A P1H";
$resn_exch{"1HN3 P1H"}="HN31 P1H"; $resn_exch{"2HN3 P1H"}="HN32 P1H"; $resn_exch{"1H6  P1H"}=" H61 P1H";
$resn_exch{"1H11 P1H"}="H111 P1H"; $resn_exch{"2H11 P1H"}="H112 P1H"; $resn_exch{"1HN7 P1H"}="HN71 P1H";
$resn_exch{"2HN7 P1H"}="HN72 P1H"; $resn_exch{"1H16 P1H"}="H161 P1H"; $resn_exch{"1H23 P1H"}="H231 P1H";
$resn_exch{"1H26 P1H"}="H261 P1H"; $resn_exch{"1H29 P1H"}="H291 P1H"; $resn_exch{"2H29 P1H"}="H292 P1H";
$resn_exch{"1H28 P1H"}="H281 P1H"; $resn_exch{"2H28 P1H"}="H282 P1H"; $resn_exch{"1H30 P1H"}="H301 P1H";
$resn_exch{"2H30 P1H"}="H302 P1H"; $resn_exch{"3H30 P1H"}="H303 P1H"; $resn_exch{"1H3N P1L"}=" H   P1L";
$resn_exch{"2H3N P1L"}=" H2  P1L"; $resn_exch{"1H5C P1L"}=" HB2 P1L"; $resn_exch{"2H5C P1L"}=" HB3 P1L";
$resn_exch{"1H8C P1L"}="H8C1 P1L"; $resn_exch{"2H8C P1L"}="H8C2 P1L"; $resn_exch{"1H9C P1L"}="H9C1 P1L";
$resn_exch{"2H9C P1L"}="H9C2 P1L"; $resn_exch{"1H10 P1L"}="H101 P1L"; $resn_exch{"2H10 P1L"}="H102 P1L";
$resn_exch{"1H11 P1L"}="H111 P1L"; $resn_exch{"2H11 P1L"}="H112 P1L"; $resn_exch{"1H12 P1L"}="H121 P1L";
$resn_exch{"2H12 P1L"}="H122 P1L"; $resn_exch{"1H13 P1L"}="H131 P1L"; $resn_exch{"2H13 P1L"}="H132 P1L";
$resn_exch{"1H14 P1L"}="H141 P1L"; $resn_exch{"2H14 P1L"}="H142 P1L"; $resn_exch{"1H15 P1L"}="H151 P1L";
$resn_exch{"2H15 P1L"}="H152 P1L"; $resn_exch{"1H16 P1L"}="H161 P1L"; $resn_exch{"2H16 P1L"}="H162 P1L";
$resn_exch{"1H17 P1L"}="H171 P1L"; $resn_exch{"2H17 P1L"}="H172 P1L"; $resn_exch{"1H18 P1L"}="H181 P1L";
$resn_exch{"2H18 P1L"}="H182 P1L"; $resn_exch{"1H19 P1L"}="H191 P1L"; $resn_exch{"2H19 P1L"}="H192 P1L";
$resn_exch{"1H20 P1L"}="H201 P1L"; $resn_exch{"2H20 P1L"}="H202 P1L"; $resn_exch{"1H21 P1L"}="H211 P1L";
$resn_exch{"2H21 P1L"}="H212 P1L"; $resn_exch{"1H22 P1L"}="H221 P1L"; $resn_exch{"2H22 P1L"}="H222 P1L";
$resn_exch{"3H22 P1L"}="H223 P1L"; $resn_exch{" HO' P1L"}=" HXT P1L"; $resn_exch{"1H5* P1P"}=" H5' P1P";
$resn_exch{"2H5* P1P"}="H5'' P1P"; $resn_exch{"1H4* P1P"}="H4'1 P1P"; $resn_exch{"2H4* P1P"}="H4'2 P1P";
$resn_exch{"1H3* P1P"}="H3'1 P1P"; $resn_exch{"2H3* P1P"}="H3'2 P1P"; $resn_exch{"1H1  P1S"}=" H11 P1S";
$resn_exch{"2H1  P1S"}="H12A P1S"; $resn_exch{"1H16 P1S"}="H161 P1S"; $resn_exch{"2H16 P1S"}="H162 P1S";
$resn_exch{"1H17 P1S"}="H171 P1S"; $resn_exch{"2H17 P1S"}="H172 P1S"; $resn_exch{"1H5A P1T"}="H5A1 P1T";
$resn_exch{"2H5A P1T"}="H5A2 P1T"; $resn_exch{"1H2A P1T"}="H2A1 P1T"; $resn_exch{"2H2A P1T"}="H2A2 P1T";
$resn_exch{"3H2A P1T"}="H2A3 P1T"; $resn_exch{"1H4A P1T"}="H4A1 P1T"; $resn_exch{"2H4A P1T"}="H4A2 P1T";
$resn_exch{"1HB  P1T"}=" HB1 P1T"; $resn_exch{"2HB  P1T"}=" HB2 P1T"; $resn_exch{"1H21 P1Z"}="H211 P1Z";
$resn_exch{"2H21 P1Z"}="H212 P1Z"; $resn_exch{"3H21 P1Z"}="H213 P1Z"; $resn_exch{"1H20 P1Z"}="H201 P1Z";
$resn_exch{"2H20 P1Z"}="H202 P1Z"; $resn_exch{"1H19 P1Z"}="H191 P1Z"; $resn_exch{"2H19 P1Z"}="H192 P1Z";
$resn_exch{"1H18 P1Z"}="H181 P1Z"; $resn_exch{"2H18 P1Z"}="H182 P1Z"; $resn_exch{"1H1  P20"}="H11A P20";
$resn_exch{"2H1  P20"}=" H12 P20"; $resn_exch{"1H12 P20"}="H121 P20"; $resn_exch{"2H12 P20"}="H122 P20";
$resn_exch{"1H12 P21"}="H121 P21"; $resn_exch{"2H12 P21"}="H122 P21"; $resn_exch{"1H19 P21"}="H191 P21";
$resn_exch{"2H19 P21"}="H192 P21"; $resn_exch{"1H23 P21"}="H231 P21"; $resn_exch{"2H23 P21"}="H232 P21";
$resn_exch{"1H6C P21"}="H6C1 P21"; $resn_exch{"2H6C P21"}="H6C2 P21"; $resn_exch{"1H20 P21"}="H201 P21";
$resn_exch{"2H20 P21"}="H202 P21"; $resn_exch{"1H18 P21"}="H181 P21"; $resn_exch{"2H18 P21"}="H182 P21";
$resn_exch{"1H10 P21"}="H101 P21"; $resn_exch{"2H10 P21"}="H102 P21"; $resn_exch{"1H21 P21"}="H211 P21";
$resn_exch{"2H21 P21"}="H212 P21"; $resn_exch{"1H16 P21"}="H161 P21"; $resn_exch{"2H16 P21"}="H162 P21";
$resn_exch{"1H26 P21"}="H261 P21"; $resn_exch{"2H26 P21"}="H262 P21"; $resn_exch{"3H26 P21"}="H263 P21";
$resn_exch{"1H6  P22"}=" H61 P22"; $resn_exch{"2H6  P22"}=" H62 P22"; $resn_exch{"3H6  P22"}=" H63 P22";
$resn_exch{"1H7  P22"}=" H71 P22"; $resn_exch{"2H7  P22"}=" H72 P22"; $resn_exch{"1H5  P23"}=" H51 P23";
$resn_exch{"2H5  P23"}=" H52 P23"; $resn_exch{"3H5  P23"}=" H53 P23"; $resn_exch{"1H6  P23"}=" H61 P23";
$resn_exch{"2H6  P23"}=" H62 P23"; $resn_exch{"1H7  P23"}=" H71 P23"; $resn_exch{"2H7  P23"}=" H72 P23";
$resn_exch{"1HN1 P24"}="HN11 P24"; $resn_exch{"2HN1 P24"}="HN12 P24"; $resn_exch{"1H2  P24"}=" H21 P24";
$resn_exch{"2H2  P24"}=" H22 P24"; $resn_exch{"1H3  P24"}=" H31 P24"; $resn_exch{"2H3  P24"}=" H32 P24";
$resn_exch{"1H5  P24"}=" H51 P24"; $resn_exch{"2H5  P24"}=" H52 P24"; $resn_exch{"1H6  P24"}=" H61 P24";
$resn_exch{"2H6  P24"}=" H62 P24"; $resn_exch{"1H7  P24"}=" H71 P24"; $resn_exch{"2H7  P24"}=" H72 P24";
$resn_exch{"1H8  P24"}=" H81 P24"; $resn_exch{"2H8  P24"}=" H82 P24"; $resn_exch{"1HN9 P24"}="HN91 P24";
$resn_exch{"2HN9 P24"}="HN92 P24"; $resn_exch{"1H4  P25"}=" H41 P25"; $resn_exch{"2H4  P25"}=" H42 P25";
$resn_exch{"1HM4 P25"}="HM41 P25"; $resn_exch{"2HM4 P25"}="HM42 P25"; $resn_exch{"3HM4 P25"}="HM43 P25";
$resn_exch{"1H5  P25"}=" H51 P25"; $resn_exch{"2H5  P25"}=" H52 P25"; $resn_exch{"1H6  P25"}=" H61 P25";
$resn_exch{"2H6  P25"}=" H62 P25"; $resn_exch{"1H7  P25"}=" H71 P25"; $resn_exch{"2H7  P25"}=" H72 P25";
$resn_exch{"1H32 P27"}="H321 P27"; $resn_exch{"2H32 P27"}="H322 P27"; $resn_exch{"1H12 P27"}="H121 P27";
$resn_exch{"2H12 P27"}="H122 P27"; $resn_exch{"1H7  P28"}=" H71 P28"; $resn_exch{"2H7  P28"}=" H72 P28";
$resn_exch{"1H82 P28"}="H821 P28"; $resn_exch{"2H82 P28"}="H822 P28"; $resn_exch{"3H82 P28"}="H823 P28";
$resn_exch{"1HN2 P2G"}="HN21 P2G"; $resn_exch{"2HN2 P2G"}="HN22 P2G"; $resn_exch{"1H5' P2G"}="H5'1 P2G";
$resn_exch{"2H5' P2G"}="H5'2 P2G"; $resn_exch{"1H8  P2K"}=" H81 P2K"; $resn_exch{"2H8  P2K"}=" H82 P2K";
$resn_exch{"1H9  P2K"}=" H91 P2K"; $resn_exch{"2H9  P2K"}=" H92 P2K"; $resn_exch{"1H11 P2K"}="H111 P2K";
$resn_exch{"2H11 P2K"}="H112 P2K"; $resn_exch{"1H12 P2K"}="H121 P2K"; $resn_exch{"2H12 P2K"}="H122 P2K";
$resn_exch{"1H14 P2K"}="H141 P2K"; $resn_exch{"2H14 P2K"}="H142 P2K"; $resn_exch{"1H15 P2K"}="H151 P2K";
$resn_exch{"2H15 P2K"}="H152 P2K"; $resn_exch{"1H17 P2K"}="H171 P2K"; $resn_exch{"2H17 P2K"}="H172 P2K";
$resn_exch{"1H18 P2K"}="H181 P2K"; $resn_exch{"2H18 P2K"}="H182 P2K"; $resn_exch{"1H20 P2K"}="H201 P2K";
$resn_exch{"2H20 P2K"}="H202 P2K"; $resn_exch{"1H21 P2K"}="H211 P2K"; $resn_exch{"2H21 P2K"}="H212 P2K";
$resn_exch{"1H23 P2K"}="H231 P2K"; $resn_exch{"2H23 P2K"}="H232 P2K"; $resn_exch{"1H24 P2K"}="H241 P2K";
$resn_exch{"2H24 P2K"}="H242 P2K"; $resn_exch{"1H8C P2N"}="H8C1 P2N"; $resn_exch{"2H8C P2N"}="H8C2 P2N";
$resn_exch{"1H10 P2N"}="H101 P2N"; $resn_exch{"2H10 P2N"}="H102 P2N"; $resn_exch{"1H11 P2N"}="H111 P2N";
$resn_exch{"2H11 P2N"}="H112 P2N"; $resn_exch{"1H12 P2N"}="H121 P2N"; $resn_exch{"2H12 P2N"}="H122 P2N";
$resn_exch{"1H13 P2N"}="H131 P2N"; $resn_exch{"2H13 P2N"}="H132 P2N"; $resn_exch{"1H16 P2N"}="H161 P2N";
$resn_exch{"2H16 P2N"}="H162 P2N"; $resn_exch{"1H17 P2N"}="H171 P2N"; $resn_exch{"2H17 P2N"}="H172 P2N";
$resn_exch{"1H13 P2O"}="H131 P2O"; $resn_exch{"2H13 P2O"}="H132 P2O"; $resn_exch{"1H9  P2O"}=" H91 P2O";
$resn_exch{"2H9  P2O"}=" H92 P2O"; $resn_exch{"1H14 P2O"}="H141 P2O"; $resn_exch{"2H14 P2O"}="H142 P2O";
$resn_exch{"2HOP P2P"}="HOP2 P2P"; $resn_exch{"3HOP P2P"}="HOP3 P2P"; $resn_exch{"1H5* P2P"}="H5'1 P2P";
$resn_exch{"2H5* P2P"}="H5'2 P2P"; $resn_exch{" H4* P2P"}=" H4' P2P"; $resn_exch{" H3* P2P"}=" H3' P2P";
$resn_exch{"*HO3 P2P"}="HO3' P2P"; $resn_exch{" H2* P2P"}=" H2' P2P"; $resn_exch{"*HO2 P2P"}="HO2' P2P";
$resn_exch{" H1* P2P"}=" H1' P2P"; $resn_exch{"1HD  P2S"}=" HD1 P2S"; $resn_exch{"2HD  P2S"}=" HD2 P2S";
$resn_exch{"1HF  P2S"}=" HF1 P2S"; $resn_exch{"2HF  P2S"}=" HF2 P2S"; $resn_exch{"1HH  P2S"}=" HH1 P2S";
$resn_exch{"2HH  P2S"}=" HH2 P2S"; $resn_exch{"3HH  P2S"}=" HH3 P2S"; $resn_exch{"1HG  P2S"}=" HG1 P2S";
$resn_exch{"2HG  P2S"}=" HG2 P2S"; $resn_exch{"1HB  P2S"}=" HB1 P2S"; $resn_exch{"2HB  P2S"}=" HB2 P2S";
$resn_exch{"1HN  P2S"}=" HN1 P2S"; $resn_exch{"2HN  P2S"}=" HN2 P2S"; $resn_exch{"2HOP P2T"}="HOP2 P2T";
$resn_exch{"1H5* P2T"}=" H5' P2T"; $resn_exch{"2H5* P2T"}="H5'' P2T"; $resn_exch{" H4* P2T"}=" H4' P2T";
$resn_exch{" H1* P2T"}=" H1' P2T"; $resn_exch{"1H5M P2T"}=" H71 P2T"; $resn_exch{"2H5M P2T"}=" H72 P2T";
$resn_exch{"3H5M P2T"}=" H73 P2T"; $resn_exch{" H2* P2T"}=" H2' P2T"; $resn_exch{"1HB' P2T"}="HB'1 P2T";
$resn_exch{"2HB' P2T"}="HB'2 P2T"; $resn_exch{"1HC' P2T"}="HC'1 P2T"; $resn_exch{"2HC' P2T"}="HC'2 P2T";
$resn_exch{"1HD' P2T"}="HD'1 P2T"; $resn_exch{"2HD' P2T"}="HD'2 P2T"; $resn_exch{"3HD' P2T"}="HD'3 P2T";
$resn_exch{" H3* P2T"}=" H3' P2T"; $resn_exch{"H3T  P2T"}="HO3' P2T"; $resn_exch{"3HOP P2T"}="HOP3 P2T";
$resn_exch{" H1* P2U"}=" H1' P2U"; $resn_exch{"1H2* P2U"}=" H2' P2U"; $resn_exch{"2H2* P2U"}="H2'' P2U";
$resn_exch{" H3* P2U"}=" H3' P2U"; $resn_exch{" H4* P2U"}=" H4' P2U"; $resn_exch{"H3T  P2U"}="HO3' P2U";
$resn_exch{"1H5* P2U"}=" H5' P2U"; $resn_exch{"2H5* P2U"}="H5'' P2U"; $resn_exch{"2HOP P2U"}="HOP2 P2U";
$resn_exch{"3HOP P2U"}="HOP3 P2U"; $resn_exch{"1HNA P2Y"}="HNA1 P2Y"; $resn_exch{"2HNA P2Y"}="HNA2 P2Y";
$resn_exch{"1H   P2Y"}=" H1  P2Y"; $resn_exch{"2H   P2Y"}=" H2  P2Y"; $resn_exch{" HN  P2Y"}=" H   P2Y";
$resn_exch{"1HD  P2Y"}=" HD3 P2Y"; $resn_exch{"2HD  P2Y"}=" HD2 P2Y"; $resn_exch{"1HG  P2Y"}=" HG3 P2Y";
$resn_exch{"2HG  P2Y"}=" HG2 P2Y"; $resn_exch{"1HB  P2Y"}=" HB2 P2Y"; $resn_exch{"2HB  P2Y"}=" HB3 P2Y";
$resn_exch{"1H21 P33"}="H211 P33"; $resn_exch{"2H21 P33"}="H212 P33"; $resn_exch{"1H20 P33"}="H201 P33";
$resn_exch{"2H20 P33"}="H202 P33"; $resn_exch{"1H18 P33"}="H181 P33"; $resn_exch{"2H18 P33"}="H182 P33";
$resn_exch{"1H17 P33"}="H171 P33"; $resn_exch{"2H17 P33"}="H172 P33"; $resn_exch{"1H15 P33"}="H151 P33";
$resn_exch{"2H15 P33"}="H152 P33"; $resn_exch{"1H14 P33"}="H141 P33"; $resn_exch{"2H14 P33"}="H142 P33";
$resn_exch{"1H12 P33"}="H121 P33"; $resn_exch{"2H12 P33"}="H122 P33"; $resn_exch{"1H11 P33"}="H111 P33";
$resn_exch{"2H11 P33"}="H112 P33"; $resn_exch{"1H9  P33"}=" H91 P33"; $resn_exch{"2H9  P33"}=" H92 P33";
$resn_exch{"1H8  P33"}=" H81 P33"; $resn_exch{"2H8  P33"}=" H82 P33"; $resn_exch{"1H6  P33"}=" H61 P33";
$resn_exch{"2H6  P33"}=" H62 P33"; $resn_exch{"1H5  P33"}=" H51 P33"; $resn_exch{"2H5  P33"}=" H52 P33";
$resn_exch{"1H3  P33"}=" H31 P33"; $resn_exch{"2H3  P33"}=" H32 P33"; $resn_exch{"1H2  P33"}=" H21 P33";
$resn_exch{"2H2  P33"}="H22A P33"; $resn_exch{"1HAA P34"}="HAA1 P34"; $resn_exch{"2HAA P34"}="HAA2 P34";
$resn_exch{"3HAA P34"}="HAA3 P34"; $resn_exch{"1HAB P34"}="HAB1 P34"; $resn_exch{"2HAB P34"}="HAB2 P34";
$resn_exch{"3HAB P34"}="HAB3 P34"; $resn_exch{"1HA  P34"}=" HA1 P34"; $resn_exch{"2HA  P34"}=" HA2 P34";
$resn_exch{"1H44 P3A"}="H441 P3A"; $resn_exch{"2H44 P3A"}="H442 P3A"; $resn_exch{"1H45 P3A"}="H451 P3A";
$resn_exch{"2H45 P3A"}="H452 P3A"; $resn_exch{"1H46 P3A"}="H461 P3A"; $resn_exch{"2H46 P3A"}="H462 P3A";
$resn_exch{"1H47 P3A"}="H471 P3A"; $resn_exch{"2H47 P3A"}="H472 P3A"; $resn_exch{"1H48 P3A"}="H481 P3A";
$resn_exch{"2H48 P3A"}="H482 P3A"; $resn_exch{"1H49 P3A"}="H491 P3A"; $resn_exch{"2H49 P3A"}="H492 P3A";
$resn_exch{"1H50 P3A"}="H501 P3A"; $resn_exch{"2H50 P3A"}="H502 P3A"; $resn_exch{"1H51 P3A"}="H511 P3A";
$resn_exch{"2H51 P3A"}="H512 P3A"; $resn_exch{"1H52 P3A"}="H521 P3A"; $resn_exch{"2H52 P3A"}="H522 P3A";
$resn_exch{"1H53 P3A"}="H531 P3A"; $resn_exch{"2H53 P3A"}="H532 P3A"; $resn_exch{"1H54 P3A"}="H541 P3A";
$resn_exch{"2H54 P3A"}="H542 P3A"; $resn_exch{"1H55 P3A"}="H551 P3A"; $resn_exch{"2H55 P3A"}="H552 P3A";
$resn_exch{"1H56 P3A"}="H561 P3A"; $resn_exch{"2H56 P3A"}="H562 P3A"; $resn_exch{"1H57 P3A"}="H571 P3A";
$resn_exch{"2H57 P3A"}="H572 P3A"; $resn_exch{"1H58 P3A"}="H581 P3A"; $resn_exch{"2H58 P3A"}="H582 P3A";
$resn_exch{"3H58 P3A"}="H583 P3A"; $resn_exch{"1H40 P3A"}="H401 P3A"; $resn_exch{"2H40 P3A"}="H402 P3A";
$resn_exch{"1H39 P3A"}="H391 P3A"; $resn_exch{"2H39 P3A"}="H392 P3A"; $resn_exch{"1H14 P3A"}="H141 P3A";
$resn_exch{"2H14 P3A"}="H142 P3A"; $resn_exch{"1H8  P3A"}=" H81 P3A"; $resn_exch{"2H8  P3A"}=" H82 P3A";
$resn_exch{"1H1  P3A"}=" H11 P3A"; $resn_exch{"2H1  P3A"}=" H12 P3A"; $resn_exch{"1H4  P3A"}=" H41 P3A";
$resn_exch{"2H4  P3A"}=" H42 P3A"; $resn_exch{"1H22 P3A"}="H221 P3A"; $resn_exch{"2H22 P3A"}="H222 P3A";
$resn_exch{"1H23 P3A"}="H231 P3A"; $resn_exch{"2H23 P3A"}="H232 P3A"; $resn_exch{"1H24 P3A"}="H241 P3A";
$resn_exch{"2H24 P3A"}="H242 P3A"; $resn_exch{"1H25 P3A"}="H251 P3A"; $resn_exch{"2H25 P3A"}="H252 P3A";
$resn_exch{"1H26 P3A"}="H261 P3A"; $resn_exch{"2H26 P3A"}="H262 P3A"; $resn_exch{"1H27 P3A"}="H271 P3A";
$resn_exch{"2H27 P3A"}="H272 P3A"; $resn_exch{"1H28 P3A"}="H281 P3A"; $resn_exch{"2H28 P3A"}="H282 P3A";
$resn_exch{"1H31 P3A"}="H311 P3A"; $resn_exch{"2H31 P3A"}="H312 P3A"; $resn_exch{"1H32 P3A"}="H321 P3A";
$resn_exch{"2H32 P3A"}="H322 P3A"; $resn_exch{"1H33 P3A"}="H331 P3A"; $resn_exch{"2H33 P3A"}="H332 P3A";
$resn_exch{"1H34 P3A"}="H341 P3A"; $resn_exch{"2H34 P3A"}="H342 P3A"; $resn_exch{"1H35 P3A"}="H351 P3A";
$resn_exch{"2H35 P3A"}="H352 P3A"; $resn_exch{"1H36 P3A"}="H361 P3A"; $resn_exch{"2H36 P3A"}="H362 P3A";
$resn_exch{"3H36 P3A"}="H363 P3A"; $resn_exch{"1HAN P3D"}="HAN1 P3D"; $resn_exch{"2HAN P3D"}="HAN2 P3D";
$resn_exch{"1HAA P3D"}="HAA1 P3D"; $resn_exch{"2HAA P3D"}="HAA2 P3D"; $resn_exch{"3HAA P3D"}="HAA3 P3D";
$resn_exch{"1HAG P3D"}="HAG1 P3D"; $resn_exch{"1HAM P3D"}="HAM1 P3D"; $resn_exch{"2HAM P3D"}="HAM2 P3D";
$resn_exch{"1HAL P3D"}="HAL1 P3D"; $resn_exch{"2HAL P3D"}="HAL2 P3D"; $resn_exch{"1HAK P3D"}="HAK1 P3D";
$resn_exch{"2HAK P3D"}="HAK2 P3D"; $resn_exch{"1HAJ P3D"}="HAJ1 P3D"; $resn_exch{"2HAJ P3D"}="HAJ2 P3D";
$resn_exch{"1HAI P3D"}="HAI1 P3D"; $resn_exch{"2HAI P3D"}="HAI2 P3D"; $resn_exch{"1HAB P3D"}="HAB1 P3D";
$resn_exch{"2HAB P3D"}="HAB2 P3D"; $resn_exch{"1H5  P3F"}=" H51 P3F"; $resn_exch{"2H5  P3F"}=" H52 P3F";
$resn_exch{"1H1  P3F"}=" H11 P3F"; $resn_exch{"2H1  P3F"}="H12A P3F"; $resn_exch{"3H1  P3F"}="H13A P3F";
$resn_exch{"1H9  P3F"}=" H91 P3F"; $resn_exch{"2H9  P3F"}=" H92 P3F"; $resn_exch{"1H10 P3G"}="H101 P3G";
$resn_exch{"2H10 P3G"}="H102 P3G"; $resn_exch{"1H11 P3G"}="H111 P3G"; $resn_exch{"2H11 P3G"}="H112 P3G";
$resn_exch{"1H12 P3G"}="H121 P3G"; $resn_exch{"2H12 P3G"}="H122 P3G"; $resn_exch{"1H13 P3G"}="H131 P3G";
$resn_exch{"2H13 P3G"}="H132 P3G"; $resn_exch{"1H14 P3G"}="H141 P3G"; $resn_exch{"2H14 P3G"}="H142 P3G";
$resn_exch{"1H15 P3G"}="H151 P3G"; $resn_exch{"2H15 P3G"}="H152 P3G"; $resn_exch{"2H16 P3G"}="H162 P3G";
$resn_exch{"1H16 P3G"}="H161 P3G"; $resn_exch{"1HOP P3M"}="HOP1 P3M"; $resn_exch{"2HOP P3M"}="HOP2 P3M";
$resn_exch{"2HOA P3M"}="HOA2 P3M"; $resn_exch{"3HOA P3M"}="HOA3 P3M"; $resn_exch{"4HOA P3M"}="HOA4 P3M";
$resn_exch{"1H6A P3M"}="H6A1 P3M"; $resn_exch{"2H6A P3M"}="H6A2 P3M"; $resn_exch{"2HOB P3M"}="HOB2 P3M";
$resn_exch{"4HOB P3M"}="HOB4 P3M"; $resn_exch{"1H6B P3M"}="H6B1 P3M"; $resn_exch{"2H6B P3M"}="H6B2 P3M";
$resn_exch{"6HOB P3M"}="HOB6 P3M"; $resn_exch{"1HOC P3M"}="HOC1 P3M"; $resn_exch{"2HOC P3M"}="HOC2 P3M";
$resn_exch{"4HOC P3M"}="HOC4 P3M"; $resn_exch{"1H6C P3M"}="H6C1 P3M"; $resn_exch{"2H6C P3M"}="H6C2 P3M";
$resn_exch{"6HOC P3M"}="HOC6 P3M"; $resn_exch{"1HNP P3P"}="HNP1 P3P"; $resn_exch{"2HNP P3P"}="HNP2 P3P";
$resn_exch{"1HBP P3P"}="HBP1 P3P"; $resn_exch{"2HBP P3P"}="HBP2 P3P"; $resn_exch{"1HGP P3P"}="HGP1 P3P";
$resn_exch{"2HGP P3P"}="HGP2 P3P"; $resn_exch{"1HEP P3P"}="HEP1 P3P"; $resn_exch{"2HEP P3P"}="HEP2 P3P";
$resn_exch{"3HEP P3P"}="HEP3 P3P"; $resn_exch{"3HO  P3S"}=" HO3 P3S"; $resn_exch{"2HO  P3S"}=" HO2 P3S";
$resn_exch{"1HEC P3S"}="HEC1 P3S"; $resn_exch{"2HEC P3S"}="HEC2 P3S"; $resn_exch{"3HEC P3S"}="HEC3 P3S";
$resn_exch{"1HGC P3S"}="HGC1 P3S"; $resn_exch{"2HGC P3S"}="HGC2 P3S"; $resn_exch{"1HBC P3S"}="HBC1 P3S";
$resn_exch{"2HBC P3S"}="HBC2 P3S"; $resn_exch{"1HN  P3S"}=" HN1 P3S"; $resn_exch{"2HN  P3S"}=" HN2 P3S";
$resn_exch{"1HAM P3T"}="HAM1 P3T"; $resn_exch{"2HAM P3T"}="HAM2 P3T"; $resn_exch{"1HAA P3T"}="HAA1 P3T";
$resn_exch{"2HAA P3T"}="HAA2 P3T"; $resn_exch{"3HAA P3T"}="HAA3 P3T"; $resn_exch{"1HAG P3T"}="HAG1 P3T";
$resn_exch{"2HAG P3T"}="HAG2 P3T"; $resn_exch{"1HAL P3T"}="HAL1 P3T"; $resn_exch{"2HAL P3T"}="HAL2 P3T";
$resn_exch{"1HAK P3T"}="HAK1 P3T"; $resn_exch{"2HAK P3T"}="HAK2 P3T"; $resn_exch{"1HAJ P3T"}="HAJ1 P3T";
$resn_exch{"2HAJ P3T"}="HAJ2 P3T"; $resn_exch{"1HAI P3T"}="HAI1 P3T"; $resn_exch{"2HAI P3T"}="HAI2 P3T";
$resn_exch{"1HAB P3T"}="HAB1 P3T"; $resn_exch{"2HAB P3T"}="HAB2 P3T"; $resn_exch{"1H1  P3Y"}="H11A P3Y";
$resn_exch{"2H1  P3Y"}=" H12 P3Y"; $resn_exch{"3H1  P3Y"}=" H13 P3Y"; $resn_exch{"1H41 P3Y"}="H411 P3Y";
$resn_exch{"2H41 P3Y"}="H412 P3Y"; $resn_exch{"3H41 P3Y"}="H413 P3Y"; $resn_exch{"1H45 P3Y"}="H451 P3Y";
$resn_exch{"2H45 P3Y"}="H452 P3Y"; $resn_exch{"3H45 P3Y"}="H453 P3Y"; $resn_exch{"1H2  P4C"}="H21A P4C";
$resn_exch{"2H2  P4C"}=" H22 P4C"; $resn_exch{"1H3  P4C"}=" H31 P4C"; $resn_exch{"2H3  P4C"}=" H32 P4C";
$resn_exch{"1H5  P4C"}=" H51 P4C"; $resn_exch{"2H5  P4C"}=" H52 P4C"; $resn_exch{"1H6  P4C"}=" H61 P4C";
$resn_exch{"2H6  P4C"}=" H62 P4C"; $resn_exch{"1H8  P4C"}=" H81 P4C"; $resn_exch{"2H8  P4C"}=" H82 P4C";
$resn_exch{"1H9  P4C"}=" H91 P4C"; $resn_exch{"2H9  P4C"}=" H92 P4C"; $resn_exch{"1H11 P4C"}="H111 P4C";
$resn_exch{"2H11 P4C"}="H112 P4C"; $resn_exch{"1H12 P4C"}="H121 P4C"; $resn_exch{"2H12 P4C"}="H122 P4C";
$resn_exch{"1H14 P4C"}="H141 P4C"; $resn_exch{"2H14 P4C"}="H142 P4C"; $resn_exch{"1H15 P4C"}="H151 P4C";
$resn_exch{"2H15 P4C"}="H152 P4C"; $resn_exch{"1H17 P4C"}="H171 P4C"; $resn_exch{"2H17 P4C"}="H172 P4C";
$resn_exch{"1H18 P4C"}="H181 P4C"; $resn_exch{"2H18 P4C"}="H182 P4C"; $resn_exch{"1H20 P4C"}="H201 P4C";
$resn_exch{"2H20 P4C"}="H202 P4C"; $resn_exch{"1H8  P4G"}=" H81 P4G"; $resn_exch{"2H8  P4G"}=" H82 P4G";
$resn_exch{"3H8  P4G"}=" H83 P4G"; $resn_exch{"1H7  P4G"}=" H71 P4G"; $resn_exch{"2H7  P4G"}=" H72 P4G";
$resn_exch{"1H6  P4G"}=" H61 P4G"; $resn_exch{"2H6  P4G"}=" H62 P4G"; $resn_exch{"1H5  P4G"}=" H51 P4G";
$resn_exch{"2H5  P4G"}=" H52 P4G"; $resn_exch{"1H4  P4G"}=" H41 P4G"; $resn_exch{"2H4  P4G"}=" H42 P4G";
$resn_exch{"1H3  P4G"}=" H31 P4G"; $resn_exch{"2H3  P4G"}=" H32 P4G"; $resn_exch{"1H2  P4G"}=" H21 P4G";
$resn_exch{"2H2  P4G"}=" H22 P4G"; $resn_exch{"1H1  P4G"}=" H11 P4G"; $resn_exch{"2H1  P4G"}=" H12 P4G";
$resn_exch{"3H1  P4G"}=" H13 P4G"; $resn_exch{"1H8C P4O"}="H8C1 P4O"; $resn_exch{"2H8C P4O"}="H8C2 P4O";
$resn_exch{"1H9C P4O"}="H9C1 P4O"; $resn_exch{"2H9C P4O"}="H9C2 P4O"; $resn_exch{"1H6C P4T"}="H6C1 P4T";
$resn_exch{"2H6C P4T"}="H6C2 P4T"; $resn_exch{"3H6C P4T"}="H6C3 P4T"; $resn_exch{"1H2C P4T"}="H2C1 P4T";
$resn_exch{"2H2C P4T"}="H2C2 P4T"; $resn_exch{"3H2C P4T"}="H2C3 P4T"; $resn_exch{"1HB  P5A"}=" HB1 P5A";
$resn_exch{"2HB  P5A"}=" HB2 P5A"; $resn_exch{"1HG  P5A"}=" HG1 P5A"; $resn_exch{"2HG  P5A"}=" HG2 P5A";
$resn_exch{"1HD  P5A"}=" HD1 P5A"; $resn_exch{"2HD  P5A"}=" HD2 P5A"; $resn_exch{"1H1* P5A"}="H1'1 P5A";
$resn_exch{"2H1* P5A"}="H1'2 P5A"; $resn_exch{" H4* P5A"}=" H4' P5A"; $resn_exch{" H1* P5A"}=" H1' P5A";
$resn_exch{"1H6  P5A"}=" H61 P5A"; $resn_exch{"2H6  P5A"}=" H62 P5A"; $resn_exch{" H2* P5A"}=" H2' P5A";
$resn_exch{"2HO* P5A"}="HO'2 P5A"; $resn_exch{" H3* P5A"}=" H3' P5A"; $resn_exch{"1H18 P5B"}="H181 P5B";
$resn_exch{"2H18 P5B"}="H182 P5B"; $resn_exch{"1H19 P5B"}="H191 P5B"; $resn_exch{"2H19 P5B"}="H192 P5B";
$resn_exch{"1H21 P5B"}="H211 P5B"; $resn_exch{"2H21 P5B"}="H212 P5B"; $resn_exch{"1HN3 P5B"}="HN31 P5B";
$resn_exch{"2HN3 P5B"}="HN32 P5B"; $resn_exch{"1H1  P5B"}="H11A P5B"; $resn_exch{"2H1  P5B"}=" H12 P5B";
$resn_exch{"1H9  P5B"}=" H91 P5B"; $resn_exch{"2H9  P5B"}=" H92 P5B"; $resn_exch{"1H8  P5B"}=" H81 P5B";
$resn_exch{"2H8  P5B"}=" H82 P5B"; $resn_exch{"3H8  P5B"}=" H83 P5B"; $resn_exch{"1HN2 P5B"}="HN21 P5B";
$resn_exch{"2HN2 P5B"}="HN22 P5B"; $resn_exch{"'HO2 P5P"}="HO2' P5P"; $resn_exch{"'HO3 P5P"}="HO3' P5P";
$resn_exch{"1H5' P5P"}="H5'1 P5P"; $resn_exch{"2H5' P5P"}="H5'2 P5P"; $resn_exch{"2HOP P5P"}="HOP2 P5P";
$resn_exch{"3HOP P5P"}="HOP3 P5P"; $resn_exch{"1H17 P6C"}="H171 P6C"; $resn_exch{"2H17 P6C"}="H172 P6C";
$resn_exch{"1H6  P6C"}=" H61 P6C"; $resn_exch{"2H6  P6C"}=" H62 P6C"; $resn_exch{"3H6  P6C"}=" H63 P6C";
$resn_exch{"1H4  P6C"}=" H41 P6C"; $resn_exch{"2H4  P6C"}=" H42 P6C"; $resn_exch{"3H4  P6C"}=" H43 P6C";
$resn_exch{"1H2  P6G"}=" H21 P6G"; $resn_exch{"2H2  P6G"}=" H22 P6G"; $resn_exch{"1H3  P6G"}=" H31 P6G";
$resn_exch{"2H3  P6G"}=" H32 P6G"; $resn_exch{"1H5  P6G"}=" H51 P6G"; $resn_exch{"2H5  P6G"}=" H52 P6G";
$resn_exch{"1H6  P6G"}=" H61 P6G"; $resn_exch{"2H6  P6G"}=" H62 P6G"; $resn_exch{"1H8  P6G"}=" H81 P6G";
$resn_exch{"2H8  P6G"}=" H82 P6G"; $resn_exch{"1H9  P6G"}=" H91 P6G"; $resn_exch{"2H9  P6G"}=" H92 P6G";
$resn_exch{"1H11 P6G"}="H111 P6G"; $resn_exch{"2H11 P6G"}="H112 P6G"; $resn_exch{"1H12 P6G"}="H121 P6G";
$resn_exch{"2H12 P6G"}="H122 P6G"; $resn_exch{"1H14 P6G"}="H141 P6G"; $resn_exch{"2H14 P6G"}="H142 P6G";
$resn_exch{"1H15 P6G"}="H151 P6G"; $resn_exch{"2H15 P6G"}="H152 P6G"; $resn_exch{"1H17 P6G"}="H171 P6G";
$resn_exch{"2H17 P6G"}="H172 P6G"; $resn_exch{"1H18 P6G"}="H181 P6G"; $resn_exch{"2H18 P6G"}="H182 P6G";
$resn_exch{"1H1  P6L"}=" H11 P6L"; $resn_exch{"2H1  P6L"}=" H12 P6L"; $resn_exch{"1H6  P6L"}=" H61 P6L";
$resn_exch{"2H6  P6L"}=" H62 P6L"; $resn_exch{"1H7  P6L"}=" H71 P6L"; $resn_exch{"2H7  P6L"}=" H72 P6L";
$resn_exch{"1H18 P6L"}="H181 P6L"; $resn_exch{"2H18 P6L"}="H182 P6L"; $resn_exch{"1H19 P6L"}="H191 P6L";
$resn_exch{"2H19 P6L"}="H192 P6L"; $resn_exch{"1H20 P6L"}="H201 P6L"; $resn_exch{"2H20 P6L"}="H202 P6L";
$resn_exch{"1H21 P6L"}="H211 P6L"; $resn_exch{"2H21 P6L"}="H212 P6L"; $resn_exch{"1H24 P6L"}="H241 P6L";
$resn_exch{"2H24 P6L"}="H242 P6L"; $resn_exch{"1H25 P6L"}="H251 P6L"; $resn_exch{"2H25 P6L"}="H252 P6L";
$resn_exch{"1H26 P6L"}="H261 P6L"; $resn_exch{"2H26 P6L"}="H262 P6L"; $resn_exch{"1H27 P6L"}="H271 P6L";
$resn_exch{"2H27 P6L"}="H272 P6L"; $resn_exch{"1H28 P6L"}="H281 P6L"; $resn_exch{"2H28 P6L"}="H282 P6L";
$resn_exch{"1H29 P6L"}="H291 P6L"; $resn_exch{"2H29 P6L"}="H292 P6L"; $resn_exch{"1H30 P6L"}="H301 P6L";
$resn_exch{"2H30 P6L"}="H302 P6L"; $resn_exch{"1H31 P6L"}="H311 P6L"; $resn_exch{"2H31 P6L"}="H312 P6L";
$resn_exch{"1H32 P6L"}="H321 P6L"; $resn_exch{"2H32 P6L"}="H322 P6L"; $resn_exch{"1H35 P6L"}="H351 P6L";
$resn_exch{"2H35 P6L"}="H352 P6L"; $resn_exch{"1H36 P6L"}="H361 P6L"; $resn_exch{"2H36 P6L"}="H362 P6L";
$resn_exch{"1H37 P6L"}="H371 P6L"; $resn_exch{"2H37 P6L"}="H372 P6L"; $resn_exch{"1H38 P6L"}="H381 P6L";
$resn_exch{"2H38 P6L"}="H382 P6L"; $resn_exch{"1H39 P6L"}="H391 P6L"; $resn_exch{"2H39 P6L"}="H392 P6L";
$resn_exch{"1H40 P6L"}="H401 P6L"; $resn_exch{"2H40 P6L"}="H402 P6L"; $resn_exch{"1H41 P6L"}="H411 P6L";
$resn_exch{"2H41 P6L"}="H412 P6L"; $resn_exch{"1H42 P6L"}="H421 P6L"; $resn_exch{"2H42 P6L"}="H422 P6L";
$resn_exch{"1H43 P6L"}="H431 P6L"; $resn_exch{"2H43 P6L"}="H432 P6L"; $resn_exch{"1H44 P6L"}="H441 P6L";
$resn_exch{"2H44 P6L"}="H442 P6L"; $resn_exch{"3H44 P6L"}="H443 P6L"; $resn_exch{"1H45 P6L"}="H451 P6L";
$resn_exch{"2H45 P6L"}="H452 P6L"; $resn_exch{"1H46 P6L"}="H461 P6L"; $resn_exch{"2H46 P6L"}="H462 P6L";
$resn_exch{"1H47 P6L"}="H471 P6L"; $resn_exch{"2H47 P6L"}="H472 P6L"; $resn_exch{"1H48 P6L"}="H481 P6L";
$resn_exch{"2H48 P6L"}="H482 P6L"; $resn_exch{"1H49 P6L"}="H491 P6L"; $resn_exch{"2H49 P6L"}="H492 P6L";
$resn_exch{"3H49 P6L"}="H493 P6L"; $resn_exch{"1H50 P6L"}="H501 P6L"; $resn_exch{"2H50 P6L"}="H502 P6L";
$resn_exch{"1H32 P90"}="H321 P90"; $resn_exch{"2H32 P90"}="H322 P90"; $resn_exch{"1H12 P90"}="H121 P90";
$resn_exch{"2H12 P90"}="H122 P90"; $resn_exch{"1HN2 PA1"}="HN21 PA1"; $resn_exch{"2HN2 PA1"}="HN22 PA1";
$resn_exch{"1H6  PA1"}=" H61 PA1"; $resn_exch{"2H6  PA1"}=" H62 PA1"; $resn_exch{"1HN1 PA2"}="HN11 PA2";
$resn_exch{"2HN1 PA2"}="HN12 PA2"; $resn_exch{"1H4  PA2"}=" H41 PA2"; $resn_exch{"2H4  PA2"}=" H42 PA2";
$resn_exch{"1HN5 PA2"}="HN51 PA2"; $resn_exch{"2HN5 PA2"}="HN52 PA2"; $resn_exch{"1H6  PA2"}=" H61 PA2";
$resn_exch{"2H6  PA2"}=" H62 PA2"; $resn_exch{"1H1  PA3"}=" H11 PA3"; $resn_exch{"2H1  PA3"}=" H12 PA3";
$resn_exch{"1H3  PA3"}=" H31 PA3"; $resn_exch{"2H3  PA3"}=" H32 PA3"; $resn_exch{"1H5  PA3"}=" H51 PA3";
$resn_exch{"2H5  PA3"}=" H52 PA3"; $resn_exch{"1HN3 PA4"}="HN31 PA4"; $resn_exch{"2HN3 PA4"}="HN32 PA4";
$resn_exch{"1H6  PA4"}=" H61 PA4"; $resn_exch{"2H6  PA4"}=" H62 PA4"; $resn_exch{"1HN6 PA4"}="HN61 PA4";
$resn_exch{"2HN6 PA4"}="HN62 PA4"; $resn_exch{"2HOP PA5"}="HOP2 PA5"; $resn_exch{"3HOP PA5"}="HOP3 PA5";
$resn_exch{"1H32 PA6"}="H321 PA6"; $resn_exch{"2H32 PA6"}="H322 PA6"; $resn_exch{"1H33 PA6"}="H331 PA6";
$resn_exch{"2H33 PA6"}="H332 PA6"; $resn_exch{"1H34 PA6"}="H341 PA6"; $resn_exch{"2H34 PA6"}="H342 PA6";
$resn_exch{"1H35 PA6"}="H351 PA6"; $resn_exch{"2H35 PA6"}="H352 PA6"; $resn_exch{"3H35 PA6"}="H353 PA6";
$resn_exch{"1H3  PA6"}=" H31 PA6"; $resn_exch{"2H3  PA6"}=" H32 PA6"; $resn_exch{"1H1  PA6"}=" H11 PA6";
$resn_exch{"2H1  PA6"}=" H12 PA6"; $resn_exch{" H5* PA7"}=" H5' PA7"; $resn_exch{"1H51 PA7"}="H511 PA7";
$resn_exch{"2H51 PA7"}="H512 PA7"; $resn_exch{"1H52 PA7"}="H521 PA7"; $resn_exch{"2H52 PA7"}="H522 PA7";
$resn_exch{"1H53 PA7"}="H531 PA7"; $resn_exch{"2H53 PA7"}="H532 PA7"; $resn_exch{"3H53 PA7"}="H533 PA7";
$resn_exch{" H4* PA7"}=" H4' PA7"; $resn_exch{" H3* PA7"}=" H3' PA7"; $resn_exch{" H2* PA7"}=" H2' PA7";
$resn_exch{" H1* PA7"}=" H1' PA7"; $resn_exch{"1HN6 PA7"}="HN61 PA7"; $resn_exch{"2HN6 PA7"}="HN62 PA7";
$resn_exch{"1HM  PAA"}=" HM1 PAA"; $resn_exch{"2HM  PAA"}=" HM2 PAA"; $resn_exch{"3HM  PAA"}=" HM3 PAA";
$resn_exch{"1H2' PAA"}="H2'1 PAA"; $resn_exch{"2H2' PAA"}="H2'2 PAA"; $resn_exch{"'HO2 PAB"}="HO2' PAB";
$resn_exch{"1HN4 PAB"}="HN41 PAB"; $resn_exch{"2HN4 PAB"}="HN42 PAB"; $resn_exch{"1H2  PAC"}=" H21 PAC";
$resn_exch{"2H2  PAC"}=" H22 PAC"; $resn_exch{"2HOA PAD"}="HOA2 PAD"; $resn_exch{"AH51 PAD"}="H51A PAD";
$resn_exch{"AH52 PAD"}="H52A PAD"; $resn_exch{"AH4* PAD"}=" H4B PAD"; $resn_exch{"AH3* PAD"}=" H3B PAD";
$resn_exch{"AHO3 PAD"}="HO3A PAD"; $resn_exch{"AH2* PAD"}=" H2B PAD"; $resn_exch{"AHO2 PAD"}="HO2A PAD";
$resn_exch{"AH1* PAD"}=" H1B PAD"; $resn_exch{" AH8 PAD"}=" H8A PAD"; $resn_exch{"AH61 PAD"}="H61A PAD";
$resn_exch{"AH62 PAD"}="H62A PAD"; $resn_exch{" AH2 PAD"}=" H2A PAD"; $resn_exch{"NH51 PAD"}="H51N PAD";
$resn_exch{"NH52 PAD"}="H52N PAD"; $resn_exch{"NH4* PAD"}=" H4D PAD"; $resn_exch{"NH3* PAD"}=" H3D PAD";
$resn_exch{"NHO3 PAD"}="HO3N PAD"; $resn_exch{"NH2* PAD"}=" H2D PAD"; $resn_exch{"NHO2 PAD"}="HO2N PAD";
$resn_exch{"NH1* PAD"}=" H1D PAD"; $resn_exch{" NH2 PAD"}=" H2N PAD"; $resn_exch{"NH71 PAD"}="H71N PAD";
$resn_exch{"NH72 PAD"}="H72N PAD"; $resn_exch{" NH4 PAD"}=" H4N PAD"; $resn_exch{" NH5 PAD"}=" H5N PAD";
$resn_exch{" NH6 PAD"}=" H6N PAD"; $resn_exch{"1HOP PAE"}="HOP1 PAE"; $resn_exch{"2HOP PAE"}="HOP2 PAE";
$resn_exch{"1H1  PAE"}=" H11 PAE"; $resn_exch{"2H1  PAE"}=" H12 PAE"; $resn_exch{"1H2  PAF"}=" H21 PAF";
$resn_exch{"1HO3 PAF"}="HO31 PAF"; $resn_exch{"1H4  PAF"}=" H41 PAF"; $resn_exch{"2H4  PAF"}=" H42 PAF";
$resn_exch{"3H4  PAF"}=" H43 PAF"; $resn_exch{"1H5  PAF"}=" H51 PAF"; $resn_exch{"2H5  PAF"}=" H52 PAF";
$resn_exch{"3H5  PAF"}=" H53 PAF"; $resn_exch{"1H6  PAF"}=" H61 PAF"; $resn_exch{"2H6  PAF"}=" H62 PAF";
$resn_exch{"1HO4 PAF"}="HO41 PAF"; $resn_exch{"1H3  PAG"}=" H31 PAG"; $resn_exch{"2H3  PAG"}=" H32 PAG";
$resn_exch{"2HOP PAG"}="HOP2 PAG"; $resn_exch{"3HOP PAG"}="HOP3 PAG"; $resn_exch{"1H1  PAH"}=" H11 PAH";
$resn_exch{"2H1  PAH"}=" H12 PAH"; $resn_exch{"2HOP PAH"}="HOP2 PAH"; $resn_exch{"3HOP PAH"}="HOP3 PAH";
$resn_exch{"1HC6 PAI"}="HC61 PAI"; $resn_exch{"2HC6 PAI"}="HC62 PAI"; $resn_exch{"1HC1 PAI"}="HC11 PAI";
$resn_exch{"2HC1 PAI"}="HC12 PAI"; $resn_exch{"1HC7 PAI"}="HC71 PAI"; $resn_exch{"2HC7 PAI"}="HC72 PAI";
$resn_exch{"1HC9 PAI"}="HC91 PAI"; $resn_exch{"2HC9 PAI"}="HC92 PAI"; $resn_exch{"2HOP PAJ"}="HOP2 PAJ";
$resn_exch{"1H5* PAJ"}="H5'1 PAJ"; $resn_exch{"2H5* PAJ"}="H5'2 PAJ"; $resn_exch{" H4* PAJ"}=" H4' PAJ";
$resn_exch{" H3* PAJ"}=" H3' PAJ"; $resn_exch{" H2* PAJ"}=" H2' PAJ"; $resn_exch{"2HO* PAJ"}="HO'2 PAJ";
$resn_exch{" H1* PAJ"}=" H1' PAJ"; $resn_exch{"1H6  PAJ"}=" H61 PAJ"; $resn_exch{"2H6  PAJ"}=" H62 PAJ";
$resn_exch{"1H14 PAJ"}="H141 PAJ"; $resn_exch{"2H14 PAJ"}="H142 PAJ"; $resn_exch{"3H14 PAJ"}="H143 PAJ";
$resn_exch{"1H15 PAJ"}="H151 PAJ"; $resn_exch{"2H15 PAJ"}="H152 PAJ"; $resn_exch{"3H15 PAJ"}="H153 PAJ";
$resn_exch{"1H16 PAJ"}="H161 PAJ"; $resn_exch{"2H16 PAJ"}="H162 PAJ"; $resn_exch{"2HOP PAL"}="HOP2 PAL";
$resn_exch{"3HOP PAL"}="HOP3 PAL"; $resn_exch{"1H1P PAL"}="H1P1 PAL"; $resn_exch{"2H1P PAL"}="H1P2 PAL";
$resn_exch{"1H3  PAL"}=" H31 PAL"; $resn_exch{"2H3  PAL"}=" H32 PAL"; $resn_exch{"1H2  PAM"}=" H21 PAM";
$resn_exch{"2H2  PAM"}=" H22 PAM"; $resn_exch{"1H3  PAM"}=" H31 PAM"; $resn_exch{"2H3  PAM"}=" H32 PAM";
$resn_exch{"1H4  PAM"}=" H41 PAM"; $resn_exch{"2H4  PAM"}=" H42 PAM"; $resn_exch{"1H5  PAM"}=" H51 PAM";
$resn_exch{"2H5  PAM"}=" H52 PAM"; $resn_exch{"1H6  PAM"}=" H61 PAM"; $resn_exch{"2H6  PAM"}=" H62 PAM";
$resn_exch{"1H7  PAM"}=" H71 PAM"; $resn_exch{"2H7  PAM"}=" H72 PAM"; $resn_exch{"1H8  PAM"}=" H81 PAM";
$resn_exch{"2H8  PAM"}=" H82 PAM"; $resn_exch{"1H11 PAM"}="H111 PAM"; $resn_exch{"2H11 PAM"}="H112 PAM";
$resn_exch{"1H12 PAM"}="H121 PAM"; $resn_exch{"2H12 PAM"}="H122 PAM"; $resn_exch{"1H13 PAM"}="H131 PAM";
$resn_exch{"2H13 PAM"}="H132 PAM"; $resn_exch{"1H14 PAM"}="H141 PAM"; $resn_exch{"2H14 PAM"}="H142 PAM";
$resn_exch{"1H15 PAM"}="H151 PAM"; $resn_exch{"2H15 PAM"}="H152 PAM"; $resn_exch{"1H16 PAM"}="H161 PAM";
$resn_exch{"2H16 PAM"}="H162 PAM"; $resn_exch{"3H16 PAM"}="H163 PAM"; $resn_exch{"1H5  PAN"}=" H51 PAN";
$resn_exch{"2H5  PAN"}=" H52 PAN"; $resn_exch{"2HOP PAN"}="HOP2 PAN"; $resn_exch{"3HOP PAN"}="HOP3 PAN";
$resn_exch{"1HN  PAO"}=" HN1 PAO"; $resn_exch{"2HN  PAO"}=" HN2 PAO"; $resn_exch{"1HB  PAO"}=" HB1 PAO";
$resn_exch{"2HB  PAO"}=" HB2 PAO"; $resn_exch{"1HG  PAO"}=" HG1 PAO"; $resn_exch{"2HG  PAO"}=" HG2 PAO";
$resn_exch{"1HD  PAO"}=" HD1 PAO"; $resn_exch{"2HD  PAO"}=" HD2 PAO"; $resn_exch{"1HP  PAO"}=" HP1 PAO";
$resn_exch{"2HP  PAO"}=" HP2 PAO"; $resn_exch{"2HOP PAO"}="HOP2 PAO"; $resn_exch{"3HOP PAO"}="HOP3 PAO";
$resn_exch{"2HOB PAP"}="HOB2 PAP"; $resn_exch{"3HOB PAP"}="HOB3 PAP"; $resn_exch{"2HOA PAP"}="HOA2 PAP";
$resn_exch{"1H5* PAP"}="H5'1 PAP"; $resn_exch{"2H5* PAP"}="H5'2 PAP"; $resn_exch{" H4* PAP"}=" H4' PAP";
$resn_exch{" H3* PAP"}=" H3' PAP"; $resn_exch{" H2* PAP"}=" H2' PAP"; $resn_exch{"*HO2 PAP"}="HO2' PAP";
$resn_exch{" H1* PAP"}=" H1' PAP"; $resn_exch{"1HN6 PAP"}="HN61 PAP"; $resn_exch{"2HN6 PAP"}="HN62 PAP";
$resn_exch{"1HN  PAQ"}=" H   PAQ"; $resn_exch{"2HN  PAQ"}=" H2  PAQ"; $resn_exch{"1HB  PAQ"}=" HB2 PAQ";
$resn_exch{"2HB  PAQ"}=" HB3 PAQ"; $resn_exch{" HOH PAQ"}=" HH  PAQ"; $resn_exch{" H2  PAQ"}=" HC2 PAQ";
$resn_exch{"1HN2 PAR"}="HN21 PAR"; $resn_exch{"2HN2 PAR"}="HN22 PAR"; $resn_exch{"1HO3 PAR"}="HO31 PAR";
$resn_exch{"1HO4 PAR"}="HO41 PAR"; $resn_exch{"1H61 PAR"}="H611 PAR"; $resn_exch{"2H61 PAR"}="H612 PAR";
$resn_exch{"1HO6 PAR"}="HO61 PAR"; $resn_exch{"1H12 PAR"}="H121 PAR"; $resn_exch{"2H12 PAR"}="H122 PAR";
$resn_exch{"1H22 PAR"}="H221 PAR"; $resn_exch{"2H22 PAR"}="H222 PAR"; $resn_exch{"1H32 PAR"}="H321 PAR";
$resn_exch{"2H32 PAR"}="H322 PAR"; $resn_exch{"2HO6 PAR"}="HO62 PAR"; $resn_exch{"3HO2 PAR"}="HO23 PAR";
$resn_exch{"1H53 PAR"}="H531 PAR"; $resn_exch{"2H53 PAR"}="H532 PAR"; $resn_exch{"3HO5 PAR"}="HO53 PAR";
$resn_exch{"1H24 PAR"}="H241 PAR"; $resn_exch{"2H24 PAR"}="H242 PAR"; $resn_exch{"4HO3 PAR"}="HO34 PAR";
$resn_exch{"4HO4 PAR"}="HO44 PAR"; $resn_exch{"1H64 PAR"}="H641 PAR"; $resn_exch{"2H64 PAR"}="H642 PAR";
$resn_exch{"1HN6 PAR"}="HN61 PAR"; $resn_exch{"2HN6 PAR"}="HN62 PAR"; $resn_exch{"1HB  PAS"}=" HB1 PAS";
$resn_exch{"2HB  PAS"}=" HB2 PAS"; $resn_exch{"1HO  PAS"}=" HO1 PAS"; $resn_exch{"2HO  PAS"}=" HO2 PAS";
$resn_exch{" HN2 PAT"}=" H2  PAT"; $resn_exch{"1HB  PAT"}=" HB2 PAT"; $resn_exch{"2HB  PAT"}=" HB3 PAT";
$resn_exch{"2HOP PAT"}="HOP2 PAT"; $resn_exch{"3HOP PAT"}="HOP3 PAT"; $resn_exch{"1H2  PAU"}=" H21 PAU";
$resn_exch{"2H2  PAU"}=" H22 PAU"; $resn_exch{"1H3  PAU"}=" H31 PAU"; $resn_exch{"2H3  PAU"}=" H32 PAU";
$resn_exch{"3H3  PAU"}=" H33 PAU"; $resn_exch{"1H4  PAU"}=" H41 PAU"; $resn_exch{"2H4  PAU"}=" H42 PAU";
$resn_exch{"3H4  PAU"}=" H43 PAU"; $resn_exch{" H5* PAU"}=" H5' PAU"; $resn_exch{"1H8  PAU"}=" H81 PAU";
$resn_exch{"2H8  PAU"}=" H82 PAU"; $resn_exch{"1H9  PAU"}=" H91 PAU"; $resn_exch{"2H9  PAU"}=" H92 PAU";
$resn_exch{"1H11 PAV"}="H111 PAV"; $resn_exch{"2H11 PAV"}="H112 PAV"; $resn_exch{"3H11 PAV"}="H113 PAV";
$resn_exch{"1H6  PAV"}=" H61 PAV"; $resn_exch{"2H6  PAV"}=" H62 PAV"; $resn_exch{"1HN1 PAW"}="HN11 PAW";
$resn_exch{"2HN1 PAW"}="HN12 PAW"; $resn_exch{"1H2  PAW"}=" H21 PAW"; $resn_exch{"2H2  PAW"}=" H22 PAW";
$resn_exch{"1H3  PAW"}=" H31 PAW"; $resn_exch{"2H3  PAW"}=" H32 PAW"; $resn_exch{"1H5  PAW"}=" H51 PAW";
$resn_exch{"2H5  PAW"}=" H52 PAW"; $resn_exch{"1H6  PAW"}=" H61 PAW"; $resn_exch{"2H6  PAW"}=" H62 PAW";
$resn_exch{"1H8  PAW"}=" H81 PAW"; $resn_exch{"2H8  PAW"}=" H82 PAW"; $resn_exch{"1H9  PAW"}=" H91 PAW";
$resn_exch{"2H9  PAW"}=" H92 PAW"; $resn_exch{"1H11 PAW"}="H111 PAW"; $resn_exch{"2H11 PAW"}="H112 PAW";
$resn_exch{"1H12 PAW"}="H121 PAW"; $resn_exch{"2H12 PAW"}="H122 PAW"; $resn_exch{"1H13 PAW"}="H131 PAW";
$resn_exch{"2H13 PAW"}="H132 PAW"; $resn_exch{"1H6A PAX"}="H6A1 PAX"; $resn_exch{"2H6A PAX"}="H6A2 PAX";
$resn_exch{"1H5* PAX"}="H5'1 PAX"; $resn_exch{"2H5* PAX"}="H5'2 PAX"; $resn_exch{" H4* PAX"}=" H4' PAX";
$resn_exch{" H1* PAX"}=" H1' PAX"; $resn_exch{" H2* PAX"}=" H2' PAX"; $resn_exch{" H3* PAX"}=" H3' PAX";
$resn_exch{"1H5  PAX"}=" H51 PAX"; $resn_exch{"2H5  PAX"}=" H52 PAX"; $resn_exch{"1H2  PAX"}=" H21 PAX";
$resn_exch{"2H2  PAX"}=" H22 PAX"; $resn_exch{"1HM5 PAX"}="HM51 PAX"; $resn_exch{"2HM5 PAX"}="HM52 PAX";
$resn_exch{"3HM5 PAX"}="HM53 PAX"; $resn_exch{"1H7  PB1"}=" H71 PB1"; $resn_exch{"2H7  PB1"}=" H72 PB1";
$resn_exch{"1H6  PB1"}=" H61 PB1"; $resn_exch{"2H6  PB1"}=" H62 PB1"; $resn_exch{"1H2  PB1"}=" H21 PB1";
$resn_exch{"2H2  PB1"}=" H22 PB1"; $resn_exch{"1HN1 PB1"}="HN11 PB1"; $resn_exch{"2HN1 PB1"}="HN12 PB1";
$resn_exch{"1H9  PB1"}=" H91 PB1"; $resn_exch{"2H9  PB1"}=" H92 PB1"; $resn_exch{"1H5  PB1"}=" H51 PB1";
$resn_exch{"2H5  PB1"}=" H52 PB1"; $resn_exch{"1H11 PB1"}="H111 PB1"; $resn_exch{"2H11 PB1"}="H112 PB1";
$resn_exch{"1H12 PB1"}="H121 PB1"; $resn_exch{"2H12 PB1"}="H122 PB1"; $resn_exch{"1H7  PB3"}=" H71 PB3";
$resn_exch{"2H7  PB3"}=" H72 PB3"; $resn_exch{"3H7  PB3"}=" H73 PB3"; $resn_exch{"1H8  PB3"}=" H81 PB3";
$resn_exch{"2H8  PB3"}=" H82 PB3"; $resn_exch{"3H8  PB3"}=" H83 PB3"; $resn_exch{"1H11 PB3"}="H111 PB3";
$resn_exch{"2H11 PB3"}="H112 PB3"; $resn_exch{"1H17 PB3"}="H171 PB3"; $resn_exch{"2H17 PB3"}="H172 PB3";
$resn_exch{"1H7  PB4"}=" H71 PB4"; $resn_exch{"2H7  PB4"}=" H72 PB4"; $resn_exch{"3H7  PB4"}=" H73 PB4";
$resn_exch{"1H8  PB4"}=" H81 PB4"; $resn_exch{"2H8  PB4"}=" H82 PB4"; $resn_exch{"3H8  PB4"}=" H83 PB4";
$resn_exch{"1H10 PB4"}="H101 PB4"; $resn_exch{"2H10 PB4"}="H102 PB4"; $resn_exch{"1H13 PB4"}="H131 PB4";
$resn_exch{"2H13 PB4"}="H132 PB4"; $resn_exch{"1H11 PB5"}="H111 PB5"; $resn_exch{"2H11 PB5"}="H112 PB5";
$resn_exch{"1H17 PB5"}="H171 PB5"; $resn_exch{"2H17 PB5"}="H172 PB5"; $resn_exch{"1HA  PBA"}=" HA1 PBA";
$resn_exch{"2HA  PBA"}=" HA2 PBA"; $resn_exch{"1HB  PBA"}=" HB1 PBA"; $resn_exch{"2HB  PBA"}=" HB2 PBA";
$resn_exch{"1HN  PBB"}=" H   PBB"; $resn_exch{"2HN  PBB"}=" H2  PBB"; $resn_exch{"1HB  PBB"}=" HB2 PBB";
$resn_exch{"2HB  PBB"}=" HB3 PBB"; $resn_exch{" H2  PBB"}=" HC2 PBB"; $resn_exch{"1H7  PBB"}=" H71 PBB";
$resn_exch{"2H7  PBB"}=" H72 PBB"; $resn_exch{"1HD  PBE"}=" HD1 PBE"; $resn_exch{"2HD  PBE"}=" HD2 PBE";
$resn_exch{"1HB  PBE"}=" HB1 PBE"; $resn_exch{"2HB  PBE"}=" HB2 PBE"; $resn_exch{"1HG  PBE"}=" HG1 PBE";
$resn_exch{"2HG  PBE"}=" HG2 PBE"; $resn_exch{"1HE  PBE"}=" HE1 PBE"; $resn_exch{"2HE  PBE"}=" HE2 PBE";
$resn_exch{"3HE  PBE"}=" HE3 PBE"; $resn_exch{"1HF  PBE"}=" HF1 PBE"; $resn_exch{"2HF  PBE"}=" HF2 PBE";
$resn_exch{"3HF  PBE"}=" HF3 PBE"; $resn_exch{"1HN  PBF"}=" H   PBF"; $resn_exch{"2HN  PBF"}=" H2  PBF";
$resn_exch{"1HB  PBF"}=" HB2 PBF"; $resn_exch{"2HB  PBF"}=" HB3 PBF"; $resn_exch{"2HHA PBG"}="HHA2 PBG";
$resn_exch{"1HHA PBG"}="HHA1 PBG"; $resn_exch{"1H4A PBG"}="H4A1 PBG"; $resn_exch{"2H5A PBG"}="H5A2 PBG";
$resn_exch{"1H5A PBG"}="H5A1 PBG"; $resn_exch{"OH2A PBG"}="H2AO PBG"; $resn_exch{"2H7A PBG"}="H7A2 PBG";
$resn_exch{"1H7A PBG"}="H7A1 PBG"; $resn_exch{"2H8A PBG"}="H8A2 PBG"; $resn_exch{"1H8A PBG"}="H8A1 PBG";
$resn_exch{"OH4A PBG"}="H4AO PBG"; $resn_exch{"2H1  PBG"}=" H12 PBG"; $resn_exch{"1H1  PBG"}=" H11 PBG";
$resn_exch{"1HN1 PBI"}="HN11 PBI"; $resn_exch{"2HN1 PBI"}="HN12 PBI"; $resn_exch{"1HA1 PBI"}="HA11 PBI";
$resn_exch{"2HA1 PBI"}="HA12 PBI"; $resn_exch{"1HA3 PBI"}="HA31 PBI"; $resn_exch{"2HA3 PBI"}="HA32 PBI";
$resn_exch{"1HB2 PBI"}="HB21 PBI"; $resn_exch{"2HB2 PBI"}="HB22 PBI"; $resn_exch{"1H1  PBM"}=" H11 PBM";
$resn_exch{"2H1  PBM"}=" H12 PBM"; $resn_exch{"3H1  PBM"}=" H13 PBM"; $resn_exch{"1H2  PBM"}=" H21 PBM";
$resn_exch{"2H2  PBM"}=" H22 PBM"; $resn_exch{"3H2  PBM"}=" H23 PBM"; $resn_exch{"1H3  PBM"}=" H31 PBM";
$resn_exch{"2H3  PBM"}=" H32 PBM"; $resn_exch{"3H3  PBM"}=" H33 PBM"; $resn_exch{"1H1  PBN"}=" H11 PBN";
$resn_exch{"2H1  PBN"}=" H12 PBN"; $resn_exch{"1H2  PBN"}=" H21 PBN"; $resn_exch{"2H2  PBN"}=" H22 PBN";
$resn_exch{"1H3  PBN"}=" H31 PBN"; $resn_exch{"2H3  PBN"}=" H32 PBN"; $resn_exch{"1H4  PBN"}=" H41 PBN";
$resn_exch{"2H4  PBN"}=" H42 PBN"; $resn_exch{"1HN  PBN"}=" HN1 PBN"; $resn_exch{"2HN  PBN"}=" HN2 PBN";
$resn_exch{"3H10 PBO"}="H103 PBO"; $resn_exch{"1H10 PBO"}="H101 PBO"; $resn_exch{"2H10 PBO"}="H102 PBO";
$resn_exch{"1H11 PBO"}="H111 PBO"; $resn_exch{"2H11 PBO"}="H112 PBO"; $resn_exch{"1H12 PBO"}="H121 PBO";
$resn_exch{"2H12 PBO"}="H122 PBO"; $resn_exch{"1HH  PBP"}=" HH1 PBP"; $resn_exch{"2HH  PBP"}=" HH2 PBP";
$resn_exch{"3HH  PBP"}=" HH3 PBP"; $resn_exch{"1H17 PBS"}="H171 PBS"; $resn_exch{"2H17 PBS"}="H172 PBS";
$resn_exch{"3H17 PBS"}="H173 PBS"; $resn_exch{"1H16 PBS"}="H161 PBS"; $resn_exch{"2H16 PBS"}="H162 PBS";
$resn_exch{"1H15 PBS"}="H151 PBS"; $resn_exch{"2H15 PBS"}="H152 PBS"; $resn_exch{"1H14 PBS"}="H141 PBS";
$resn_exch{"2H14 PBS"}="H142 PBS"; $resn_exch{"1H13 PBS"}="H131 PBS"; $resn_exch{"2H13 PBS"}="H132 PBS";
$resn_exch{"1H12 PBS"}="H121 PBS"; $resn_exch{"2H12 PBS"}="H122 PBS"; $resn_exch{"1H11 PBS"}="H111 PBS";
$resn_exch{"2H11 PBS"}="H112 PBS"; $resn_exch{"1H10 PBS"}="H101 PBS"; $resn_exch{"2H10 PBS"}="H102 PBS";
$resn_exch{"1H9  PBS"}=" H91 PBS"; $resn_exch{"2H9  PBS"}=" H92 PBS"; $resn_exch{"1H8  PBS"}=" H81 PBS";
$resn_exch{"2H8  PBS"}=" H82 PBS"; $resn_exch{"1H7  PBS"}=" H71 PBS"; $resn_exch{"2H7  PBS"}=" H72 PBS";
$resn_exch{"1H6  PBS"}=" H61 PBS"; $resn_exch{"2H6  PBS"}=" H62 PBS"; $resn_exch{"1H5  PBS"}=" H51 PBS";
$resn_exch{"2H5  PBS"}=" H52 PBS"; $resn_exch{"1H4  PBS"}=" H41 PBS"; $resn_exch{"2H4  PBS"}=" H42 PBS";
$resn_exch{"1H19 PBS"}="H191 PBS"; $resn_exch{"2H19 PBS"}="H192 PBS"; $resn_exch{"1H20 PBS"}="H201 PBS";
$resn_exch{"2H20 PBS"}="H202 PBS"; $resn_exch{"1H21 PBS"}="H211 PBS"; $resn_exch{"2H21 PBS"}="H212 PBS";
$resn_exch{"1H22 PBS"}="H221 PBS"; $resn_exch{"2H22 PBS"}="H222 PBS"; $resn_exch{"1H23 PBS"}="H231 PBS";
$resn_exch{"2H23 PBS"}="H232 PBS"; $resn_exch{"1H24 PBS"}="H241 PBS"; $resn_exch{"2H24 PBS"}="H242 PBS";
$resn_exch{"1H25 PBS"}="H251 PBS"; $resn_exch{"2H25 PBS"}="H252 PBS"; $resn_exch{"3H25 PBS"}="H253 PBS";
$resn_exch{"1H   PBS"}=" H1A PBS"; $resn_exch{"2H   PBS"}=" H2A PBS"; $resn_exch{"1HG5 PBS"}="HG51 PBS";
$resn_exch{"2HG5 PBS"}="HG52 PBS"; $resn_exch{"1H5* PBT"}="H5'1 PBT"; $resn_exch{"2H5* PBT"}="H5'2 PBT";
$resn_exch{" H4* PBT"}=" H4' PBT"; $resn_exch{" H3* PBT"}=" H3' PBT"; $resn_exch{"1H2* PBT"}="H2'1 PBT";
$resn_exch{"2H2* PBT"}="H2'2 PBT"; $resn_exch{" H1* PBT"}=" H1' PBT"; $resn_exch{"1H5M PBT"}="H5M1 PBT";
$resn_exch{"2H5M PBT"}="H5M2 PBT"; $resn_exch{"3H5M PBT"}="H5M3 PBT"; $resn_exch{"1H6C PBT"}="H6C1 PBT";
$resn_exch{"2H6C PBT"}="H6C2 PBT"; $resn_exch{"1H3' PBU"}="H3'1 PBU"; $resn_exch{"2H3' PBU"}="H3'2 PBU";
$resn_exch{"1H1' PBU"}="H1'1 PBU"; $resn_exch{"2H1' PBU"}="H1'2 PBU"; $resn_exch{"2H0  PBU"}=" H02 PBU";
$resn_exch{"3H0  PBU"}=" H03 PBU"; $resn_exch{"6H0  PBU"}=" H06 PBU"; $resn_exch{"1H8  PBU"}=" H81 PBU";
$resn_exch{"2H8  PBU"}=" H82 PBU"; $resn_exch{"1H9  PBU"}=" H91 PBU"; $resn_exch{"2H9  PBU"}=" H92 PBU";
$resn_exch{"1H12 PBU"}="H121 PBU"; $resn_exch{"2H12 PBU"}="H122 PBU"; $resn_exch{"1H13 PBU"}="H131 PBU";
$resn_exch{"2H13 PBU"}="H132 PBU"; $resn_exch{"1H14 PBU"}="H141 PBU"; $resn_exch{"2H14 PBU"}="H142 PBU";
$resn_exch{"3H14 PBU"}="H143 PBU"; $resn_exch{"1HN2 PBZ"}="HN21 PBZ"; $resn_exch{"2HN2 PBZ"}="HN22 PBZ";
$resn_exch{"1HN3 PBZ"}="HN31 PBZ"; $resn_exch{"2HN3 PBZ"}="HN32 PBZ"; $resn_exch{"1HN1 PBZ"}="HN11 PBZ";
$resn_exch{"2HN1 PBZ"}="HN12 PBZ"; $resn_exch{"1H1   PC"}=" H11  PC"; $resn_exch{"2H1   PC"}=" H12  PC";
$resn_exch{"1H2   PC"}=" H21  PC"; $resn_exch{"2H2   PC"}=" H22  PC"; $resn_exch{"1H3   PC"}=" H31  PC";
$resn_exch{"2H3   PC"}=" H32  PC"; $resn_exch{"3H3   PC"}=" H33  PC"; $resn_exch{"1H4   PC"}=" H41  PC";
$resn_exch{"2H4   PC"}=" H42  PC"; $resn_exch{"3H4   PC"}=" H43  PC"; $resn_exch{"1H5   PC"}=" H51  PC";
$resn_exch{"2H5   PC"}=" H52  PC"; $resn_exch{"3H5   PC"}=" H53  PC"; $resn_exch{"1HL7 PC0"}="HL71 PC0";
$resn_exch{"2HL7 PC0"}="HL72 PC0"; $resn_exch{"3HL7 PC0"}="HL73 PC0"; $resn_exch{"1H11 PC0"}="H111 PC0";
$resn_exch{"2H11 PC0"}="H112 PC0"; $resn_exch{"1H12 PC0"}="H121 PC0"; $resn_exch{"2H12 PC0"}="H122 PC0";
$resn_exch{"1HA  PC0"}=" HA1 PC0"; $resn_exch{"2HA  PC0"}=" HA2 PC0"; $resn_exch{"1HB  PC0"}=" HB1 PC0";
$resn_exch{"2HB  PC0"}=" HB2 PC0"; $resn_exch{"3HB  PC0"}=" HB3 PC0"; $resn_exch{"1H11 PC1"}="H111 PC1";
$resn_exch{"2H11 PC1"}="H112 PC1"; $resn_exch{"1H12 PC1"}="H121 PC1"; $resn_exch{"2H12 PC1"}="H122 PC1";
$resn_exch{"1H13 PC1"}="H131 PC1"; $resn_exch{"2H13 PC1"}="H132 PC1"; $resn_exch{"3H13 PC1"}="H133 PC1";
$resn_exch{"1H14 PC1"}="H141 PC1"; $resn_exch{"2H14 PC1"}="H142 PC1"; $resn_exch{"3H14 PC1"}="H143 PC1";
$resn_exch{"1H15 PC1"}="H151 PC1"; $resn_exch{"2H15 PC1"}="H152 PC1"; $resn_exch{"3H15 PC1"}="H153 PC1";
$resn_exch{"1H1  PC1"}=" H11 PC1"; $resn_exch{"2H1  PC1"}=" H12 PC1"; $resn_exch{"1H22 PC1"}="H221 PC1";
$resn_exch{"2H22 PC1"}="H222 PC1"; $resn_exch{"1H23 PC1"}="H231 PC1"; $resn_exch{"2H23 PC1"}="H232 PC1";
$resn_exch{"1H24 PC1"}="H241 PC1"; $resn_exch{"2H24 PC1"}="H242 PC1"; $resn_exch{"1H25 PC1"}="H251 PC1";
$resn_exch{"2H25 PC1"}="H252 PC1"; $resn_exch{"1H26 PC1"}="H261 PC1"; $resn_exch{"2H26 PC1"}="H262 PC1";
$resn_exch{"1H27 PC1"}="H271 PC1"; $resn_exch{"2H27 PC1"}="H272 PC1"; $resn_exch{"1H28 PC1"}="H281 PC1";
$resn_exch{"2H28 PC1"}="H282 PC1"; $resn_exch{"1H29 PC1"}="H291 PC1"; $resn_exch{"2H29 PC1"}="H292 PC1";
$resn_exch{"1H2A PC1"}="H2A1 PC1"; $resn_exch{"2H2A PC1"}="H2A2 PC1"; $resn_exch{"1H2B PC1"}="H2B1 PC1";
$resn_exch{"2H2B PC1"}="H2B2 PC1"; $resn_exch{"1H2C PC1"}="H2C1 PC1"; $resn_exch{"2H2C PC1"}="H2C2 PC1";
$resn_exch{"1H2D PC1"}="H2D1 PC1"; $resn_exch{"2H2D PC1"}="H2D2 PC1"; $resn_exch{"1H2E PC1"}="H2E1 PC1";
$resn_exch{"2H2E PC1"}="H2E2 PC1"; $resn_exch{"1H2F PC1"}="H2F1 PC1"; $resn_exch{"2H2F PC1"}="H2F2 PC1";
$resn_exch{"1H2G PC1"}="H2G1 PC1"; $resn_exch{"2H2G PC1"}="H2G2 PC1"; $resn_exch{"1H2H PC1"}="H2H1 PC1";
$resn_exch{"2H2H PC1"}="H2H2 PC1"; $resn_exch{"1H2I PC1"}="H2I1 PC1"; $resn_exch{"2H2I PC1"}="H2I2 PC1";
$resn_exch{"3H2I PC1"}="H2I3 PC1"; $resn_exch{"1H3  PC1"}=" H31 PC1"; $resn_exch{"2H3  PC1"}=" H32 PC1";
$resn_exch{"1H32 PC1"}="H321 PC1"; $resn_exch{"2H32 PC1"}="H322 PC1"; $resn_exch{"1H33 PC1"}="H331 PC1";
$resn_exch{"2H33 PC1"}="H332 PC1"; $resn_exch{"1H34 PC1"}="H341 PC1"; $resn_exch{"2H34 PC1"}="H342 PC1";
$resn_exch{"1H35 PC1"}="H351 PC1"; $resn_exch{"2H35 PC1"}="H352 PC1"; $resn_exch{"1H36 PC1"}="H361 PC1";
$resn_exch{"2H36 PC1"}="H362 PC1"; $resn_exch{"1H37 PC1"}="H371 PC1"; $resn_exch{"2H37 PC1"}="H372 PC1";
$resn_exch{"1H38 PC1"}="H381 PC1"; $resn_exch{"2H38 PC1"}="H382 PC1"; $resn_exch{"1H39 PC1"}="H391 PC1";
$resn_exch{"2H39 PC1"}="H392 PC1"; $resn_exch{"1H3A PC1"}="H3A1 PC1"; $resn_exch{"2H3A PC1"}="H3A2 PC1";
$resn_exch{"1H3B PC1"}="H3B1 PC1"; $resn_exch{"2H3B PC1"}="H3B2 PC1"; $resn_exch{"1H3C PC1"}="H3C1 PC1";
$resn_exch{"2H3C PC1"}="H3C2 PC1"; $resn_exch{"1H3D PC1"}="H3D1 PC1"; $resn_exch{"2H3D PC1"}="H3D2 PC1";
$resn_exch{"1H3E PC1"}="H3E1 PC1"; $resn_exch{"2H3E PC1"}="H3E2 PC1"; $resn_exch{"1H3F PC1"}="H3F1 PC1";
$resn_exch{"2H3F PC1"}="H3F2 PC1"; $resn_exch{"1H3G PC1"}="H3G1 PC1"; $resn_exch{"2H3G PC1"}="H3G2 PC1";
$resn_exch{"1H3H PC1"}="H3H1 PC1"; $resn_exch{"2H3H PC1"}="H3H2 PC1"; $resn_exch{"1H3I PC1"}="H3I1 PC1";
$resn_exch{"2H3I PC1"}="H3I2 PC1"; $resn_exch{"3H3I PC1"}="H3I3 PC1"; $resn_exch{"2H1  PC2"}=" H12 PC2";
$resn_exch{"1H1  PC2"}=" H11 PC2"; $resn_exch{"2H3  PC2"}=" H32 PC2"; $resn_exch{"1H3  PC2"}=" H31 PC2";
$resn_exch{"2H4  PC2"}=" H42 PC2"; $resn_exch{"1H4  PC2"}=" H41 PC2"; $resn_exch{"2H5  PC2"}=" H52 PC2";
$resn_exch{"1H5  PC2"}=" H51 PC2"; $resn_exch{"3H6  PC2"}=" H63 PC2"; $resn_exch{"2H6  PC2"}=" H62 PC2";
$resn_exch{"1H6  PC2"}=" H61 PC2"; $resn_exch{"3H7  PC2"}=" H73 PC2"; $resn_exch{"2H7  PC2"}=" H72 PC2";
$resn_exch{"1H7  PC2"}=" H71 PC2"; $resn_exch{"3H8  PC2"}=" H83 PC2"; $resn_exch{"2H8  PC2"}=" H82 PC2";
$resn_exch{"1H8  PC2"}=" H81 PC2"; $resn_exch{"2H12 PC2"}="H122 PC2"; $resn_exch{"1H12 PC2"}="H121 PC2";
$resn_exch{"2H13 PC2"}="H132 PC2"; $resn_exch{"1H13 PC2"}="H131 PC2"; $resn_exch{"2H14 PC2"}="H142 PC2";
$resn_exch{"1H14 PC2"}="H141 PC2"; $resn_exch{"2H15 PC2"}="H152 PC2"; $resn_exch{"1H15 PC2"}="H151 PC2";
$resn_exch{"2H16 PC2"}="H162 PC2"; $resn_exch{"1H16 PC2"}="H161 PC2"; $resn_exch{"2H17 PC2"}="H172 PC2";
$resn_exch{"1H17 PC2"}="H171 PC2"; $resn_exch{"2H18 PC2"}="H182 PC2"; $resn_exch{"1H18 PC2"}="H181 PC2";
$resn_exch{"2H19 PC2"}="H192 PC2"; $resn_exch{"1H19 PC2"}="H191 PC2"; $resn_exch{"2H20 PC2"}="H202 PC2";
$resn_exch{"1H20 PC2"}="H201 PC2"; $resn_exch{"2H21 PC2"}="H212 PC2"; $resn_exch{"1H21 PC2"}="H211 PC2";
$resn_exch{"2H22 PC2"}="H222 PC2"; $resn_exch{"1H22 PC2"}="H221 PC2"; $resn_exch{"2H23 PC2"}="H232 PC2";
$resn_exch{"1H23 PC2"}="H231 PC2"; $resn_exch{"2H24 PC2"}="H242 PC2"; $resn_exch{"1H24 PC2"}="H241 PC2";
$resn_exch{"2H25 PC2"}="H252 PC2"; $resn_exch{"1H25 PC2"}="H251 PC2"; $resn_exch{"2H26 PC2"}="H262 PC2";
$resn_exch{"1H26 PC2"}="H261 PC2"; $resn_exch{"2H27 PC2"}="H272 PC2"; $resn_exch{"1H27 PC2"}="H271 PC2";
$resn_exch{"3H28 PC2"}="H283 PC2"; $resn_exch{"2H28 PC2"}="H282 PC2"; $resn_exch{"1H28 PC2"}="H281 PC2";
$resn_exch{"2H32 PC2"}="H322 PC2"; $resn_exch{"1H32 PC2"}="H321 PC2"; $resn_exch{"2H33 PC2"}="H332 PC2";
$resn_exch{"1H33 PC2"}="H331 PC2"; $resn_exch{"2H34 PC2"}="H342 PC2"; $resn_exch{"1H34 PC2"}="H341 PC2";
$resn_exch{"2H35 PC2"}="H352 PC2"; $resn_exch{"1H35 PC2"}="H351 PC2"; $resn_exch{"2H36 PC2"}="H362 PC2";
$resn_exch{"1H36 PC2"}="H361 PC2"; $resn_exch{"2H37 PC2"}="H372 PC2"; $resn_exch{"1H37 PC2"}="H371 PC2";
$resn_exch{"2H38 PC2"}="H382 PC2"; $resn_exch{"1H38 PC2"}="H381 PC2"; $resn_exch{"2H39 PC2"}="H392 PC2";
$resn_exch{"1H39 PC2"}="H391 PC2"; $resn_exch{"2H40 PC2"}="H402 PC2"; $resn_exch{"1H40 PC2"}="H401 PC2";
$resn_exch{"2H41 PC2"}="H412 PC2"; $resn_exch{"1H41 PC2"}="H411 PC2"; $resn_exch{"2H42 PC2"}="H422 PC2";
$resn_exch{"1H42 PC2"}="H421 PC2"; $resn_exch{"2H43 PC2"}="H432 PC2"; $resn_exch{"1H43 PC2"}="H431 PC2";
$resn_exch{"2H44 PC2"}="H442 PC2"; $resn_exch{"1H44 PC2"}="H441 PC2"; $resn_exch{"2H45 PC2"}="H452 PC2";
$resn_exch{"1H45 PC2"}="H451 PC2"; $resn_exch{"2H46 PC2"}="H462 PC2"; $resn_exch{"1H46 PC2"}="H461 PC2";
$resn_exch{"2H47 PC2"}="H472 PC2"; $resn_exch{"1H47 PC2"}="H471 PC2"; $resn_exch{"3H48 PC2"}="H483 PC2";
$resn_exch{"2H48 PC2"}="H482 PC2"; $resn_exch{"1H48 PC2"}="H481 PC2"; $resn_exch{"2HOP PC2"}="HOP2 PC2";
$resn_exch{"1H8  PC3"}=" H81 PC3"; $resn_exch{"2H8  PC3"}=" H82 PC3"; $resn_exch{"3H8  PC3"}=" H83 PC3";
$resn_exch{"1H9  PC3"}=" H91 PC3"; $resn_exch{"2H9  PC3"}=" H92 PC3"; $resn_exch{"3H9  PC3"}=" H93 PC3";
$resn_exch{"1H10 PC3"}="H101 PC3"; $resn_exch{"2H10 PC3"}="H102 PC3"; $resn_exch{"3H10 PC3"}="H103 PC3";
$resn_exch{"1H11 PC3"}="H111 PC3"; $resn_exch{"2H11 PC3"}="H112 PC3"; $resn_exch{"3H11 PC3"}="H113 PC3";
$resn_exch{"1H12 PC3"}="H121 PC3"; $resn_exch{"2H12 PC3"}="H122 PC3"; $resn_exch{"1H22 PC3"}="H221 PC3";
$resn_exch{"2H22 PC3"}="H222 PC3"; $resn_exch{"1H14 PC3"}="H141 PC3"; $resn_exch{"2H14 PC3"}="H142 PC3";
$resn_exch{"1H15 PC3"}="H151 PC3"; $resn_exch{"2H15 PC3"}="H152 PC3"; $resn_exch{"1H16 PC3"}="H161 PC3";
$resn_exch{"2H16 PC3"}="H162 PC3"; $resn_exch{"1H17 PC3"}="H171 PC3"; $resn_exch{"2H17 PC3"}="H172 PC3";
$resn_exch{"1H18 PC3"}="H181 PC3"; $resn_exch{"2H18 PC3"}="H182 PC3"; $resn_exch{"2H19 PC3"}="H192 PC3";
$resn_exch{"1H19 PC3"}="H191 PC3"; $resn_exch{"2HO2 PC3"}="HO22 PC3"; $resn_exch{"2HO3 PC3"}="HO32 PC3";
$resn_exch{"1H14 PC5"}="H141 PC5"; $resn_exch{"2H14 PC5"}="H142 PC5"; $resn_exch{"3H14 PC5"}="H143 PC5";
$resn_exch{"1H13 PC5"}="H131 PC5"; $resn_exch{"2H13 PC5"}="H132 PC5"; $resn_exch{"1H12 PC5"}="H121 PC5";
$resn_exch{"2H12 PC5"}="H122 PC5"; $resn_exch{"1H11 PC5"}="H111 PC5"; $resn_exch{"2H11 PC5"}="H112 PC5";
$resn_exch{"1H9  PC5"}=" H91 PC5"; $resn_exch{"2H9  PC5"}=" H92 PC5"; $resn_exch{"1H16 PC5"}="H161 PC5";
$resn_exch{"2H16 PC5"}="H162 PC5"; $resn_exch{"1H17 PC5"}="H171 PC5"; $resn_exch{"2H17 PC5"}="H172 PC5";
$resn_exch{"1H18 PC5"}="H181 PC5"; $resn_exch{"2H18 PC5"}="H182 PC5"; $resn_exch{"1H19 PC5"}="H191 PC5";
$resn_exch{"2H19 PC5"}="H192 PC5"; $resn_exch{"3H19 PC5"}="H193 PC5"; $resn_exch{"1H7  PC5"}=" H71 PC5";
$resn_exch{"2H7  PC5"}=" H72 PC5"; $resn_exch{"1H5  PC5"}=" H51 PC5"; $resn_exch{"2H5  PC5"}=" H52 PC5";
$resn_exch{"1H4  PC5"}=" H41 PC5"; $resn_exch{"2H4  PC5"}=" H42 PC5"; $resn_exch{"1H1  PC5"}=" H11 PC5";
$resn_exch{"2H1  PC5"}=" H12 PC5"; $resn_exch{"3H1  PC5"}=" H13 PC5"; $resn_exch{"1H2  PC5"}=" H21 PC5";
$resn_exch{"2H2  PC5"}=" H22 PC5"; $resn_exch{"3H2  PC5"}=" H23 PC5"; $resn_exch{"1H3  PC5"}=" H31 PC5";
$resn_exch{"2H3  PC5"}=" H32 PC5"; $resn_exch{"3H3  PC5"}=" H33 PC5"; $resn_exch{"1HE1 PC6"}="HE11 PC6";
$resn_exch{"2HE1 PC6"}="HE12 PC6"; $resn_exch{"1HE2 PC6"}="HE21 PC6"; $resn_exch{"2HE2 PC6"}="HE22 PC6";
$resn_exch{"1HM3 PC6"}="HM31 PC6"; $resn_exch{"2HM3 PC6"}="HM32 PC6"; $resn_exch{"3HM3 PC6"}="HM33 PC6";
$resn_exch{"1HM2 PC6"}="HM21 PC6"; $resn_exch{"2HM2 PC6"}="HM22 PC6"; $resn_exch{"3HM2 PC6"}="HM23 PC6";
$resn_exch{"1HM1 PC6"}="HM11 PC6"; $resn_exch{"2HM1 PC6"}="HM12 PC6"; $resn_exch{"3HM1 PC6"}="HM13 PC6";
$resn_exch{"1H39 PC6"}="H391 PC6"; $resn_exch{"2H39 PC6"}="H392 PC6"; $resn_exch{"1H40 PC6"}="H401 PC6";
$resn_exch{"2H40 PC6"}="H402 PC6"; $resn_exch{"1H27 PC6"}="H271 PC6"; $resn_exch{"2H27 PC6"}="H272 PC6";
$resn_exch{"1H28 PC6"}="H281 PC6"; $resn_exch{"2H28 PC6"}="H282 PC6"; $resn_exch{"1H29 PC6"}="H291 PC6";
$resn_exch{"2H29 PC6"}="H292 PC6"; $resn_exch{"1H30 PC6"}="H301 PC6"; $resn_exch{"2H30 PC6"}="H302 PC6";
$resn_exch{"1H31 PC6"}="H311 PC6"; $resn_exch{"2H31 PC6"}="H312 PC6"; $resn_exch{"1H32 PC6"}="H321 PC6";
$resn_exch{"2H32 PC6"}="H322 PC6"; $resn_exch{"1H33 PC6"}="H331 PC6"; $resn_exch{"2H33 PC6"}="H332 PC6";
$resn_exch{"1H34 PC6"}="H341 PC6"; $resn_exch{"2H34 PC6"}="H342 PC6"; $resn_exch{"1H35 PC6"}="H351 PC6";
$resn_exch{"2H35 PC6"}="H352 PC6"; $resn_exch{"1H36 PC6"}="H361 PC6"; $resn_exch{"2H36 PC6"}="H362 PC6";
$resn_exch{"1H37 PC6"}="H371 PC6"; $resn_exch{"2H37 PC6"}="H372 PC6"; $resn_exch{"3H37 PC6"}="H373 PC6";
$resn_exch{"1H2  PC6"}=" H21 PC6"; $resn_exch{"2H2  PC6"}=" H22 PC6"; $resn_exch{"1H3  PC6"}=" H31 PC6";
$resn_exch{"2H3  PC6"}=" H32 PC6"; $resn_exch{"1H4  PC6"}=" H41 PC6"; $resn_exch{"2H4  PC6"}=" H42 PC6";
$resn_exch{"1H5  PC6"}=" H51 PC6"; $resn_exch{"2H5  PC6"}=" H52 PC6"; $resn_exch{"1H6  PC6"}=" H61 PC6";
$resn_exch{"2H6  PC6"}=" H62 PC6"; $resn_exch{"1H7  PC6"}=" H71 PC6"; $resn_exch{"2H7  PC6"}=" H72 PC6";
$resn_exch{"1H8  PC6"}=" H81 PC6"; $resn_exch{"2H8  PC6"}=" H82 PC6"; $resn_exch{"1H9  PC6"}=" H91 PC6";
$resn_exch{"2H9  PC6"}=" H92 PC6"; $resn_exch{"1H10 PC6"}="H101 PC6"; $resn_exch{"2H10 PC6"}="H102 PC6";
$resn_exch{"1H11 PC6"}="H111 PC6"; $resn_exch{"2H11 PC6"}="H112 PC6"; $resn_exch{"1H12 PC6"}="H121 PC6";
$resn_exch{"2H12 PC6"}="H122 PC6"; $resn_exch{"1H13 PC6"}="H131 PC6"; $resn_exch{"2H13 PC6"}="H132 PC6";
$resn_exch{"1H14 PC6"}="H141 PC6"; $resn_exch{"2H14 PC6"}="H142 PC6"; $resn_exch{"1H15 PC6"}="H151 PC6";
$resn_exch{"2H15 PC6"}="H152 PC6"; $resn_exch{"1H16 PC6"}="H161 PC6"; $resn_exch{"2H16 PC6"}="H162 PC6";
$resn_exch{"1H17 PC6"}="H171 PC6"; $resn_exch{"2H17 PC6"}="H172 PC6"; $resn_exch{"1H18 PC6"}="H181 PC6";
$resn_exch{"2H18 PC6"}="H182 PC6"; $resn_exch{"1H19 PC6"}="H191 PC6"; $resn_exch{"2H19 PC6"}="H192 PC6";
$resn_exch{"1H20 PC6"}="H201 PC6"; $resn_exch{"2H20 PC6"}="H202 PC6"; $resn_exch{"1H21 PC6"}="H211 PC6";
$resn_exch{"2H21 PC6"}="H212 PC6"; $resn_exch{"1H22 PC6"}="H221 PC6"; $resn_exch{"2H22 PC6"}="H222 PC6";
$resn_exch{"1H23 PC6"}="H231 PC6"; $resn_exch{"2H23 PC6"}="H232 PC6"; $resn_exch{"1H24 PC6"}="H241 PC6";
$resn_exch{"2H24 PC6"}="H242 PC6"; $resn_exch{"1H25 PC6"}="H251 PC6"; $resn_exch{"2H25 PC6"}="H252 PC6";
$resn_exch{"3H25 PC6"}="H253 PC6"; $resn_exch{"1H48 PC7"}="H481 PC7"; $resn_exch{"2H48 PC7"}="H482 PC7";
$resn_exch{"3H48 PC7"}="H483 PC7"; $resn_exch{"1H47 PC7"}="H471 PC7"; $resn_exch{"2H47 PC7"}="H472 PC7";
$resn_exch{"1H46 PC7"}="H461 PC7"; $resn_exch{"2H46 PC7"}="H462 PC7"; $resn_exch{"1H45 PC7"}="H451 PC7";
$resn_exch{"2H45 PC7"}="H452 PC7"; $resn_exch{"1H44 PC7"}="H441 PC7"; $resn_exch{"2H44 PC7"}="H442 PC7";
$resn_exch{"1H43 PC7"}="H431 PC7"; $resn_exch{"2H43 PC7"}="H432 PC7"; $resn_exch{"1H42 PC7"}="H421 PC7";
$resn_exch{"2H42 PC7"}="H422 PC7"; $resn_exch{"1H41 PC7"}="H411 PC7"; $resn_exch{"2H41 PC7"}="H412 PC7";
$resn_exch{"1H40 PC7"}="H401 PC7"; $resn_exch{"2H40 PC7"}="H402 PC7"; $resn_exch{"1H39 PC7"}="H391 PC7";
$resn_exch{"2H39 PC7"}="H392 PC7"; $resn_exch{"1H38 PC7"}="H381 PC7"; $resn_exch{"2H38 PC7"}="H382 PC7";
$resn_exch{"1H37 PC7"}="H371 PC7"; $resn_exch{"2H37 PC7"}="H372 PC7"; $resn_exch{"1H36 PC7"}="H361 PC7";
$resn_exch{"2H36 PC7"}="H362 PC7"; $resn_exch{"1H35 PC7"}="H351 PC7"; $resn_exch{"2H35 PC7"}="H352 PC7";
$resn_exch{"1H34 PC7"}="H341 PC7"; $resn_exch{"2H34 PC7"}="H342 PC7"; $resn_exch{"1H33 PC7"}="H331 PC7";
$resn_exch{"2H33 PC7"}="H332 PC7"; $resn_exch{"1H32 PC7"}="H321 PC7"; $resn_exch{"2H32 PC7"}="H322 PC7";
$resn_exch{"1H1  PC7"}=" H11 PC7"; $resn_exch{"2H1  PC7"}=" H12 PC7"; $resn_exch{"1H4  PC7"}=" H41 PC7";
$resn_exch{"2H4  PC7"}=" H42 PC7"; $resn_exch{"1H5  PC7"}=" H51 PC7"; $resn_exch{"2H5  PC7"}=" H52 PC7";
$resn_exch{"1H7  PC7"}=" H71 PC7"; $resn_exch{"2H7  PC7"}=" H72 PC7"; $resn_exch{"3H7  PC7"}=" H73 PC7";
$resn_exch{"1H8  PC7"}=" H81 PC7"; $resn_exch{"2H8  PC7"}=" H82 PC7"; $resn_exch{"3H8  PC7"}=" H83 PC7";
$resn_exch{"1H6  PC7"}=" H61 PC7"; $resn_exch{"2H6  PC7"}=" H62 PC7"; $resn_exch{"3H6  PC7"}=" H63 PC7";
$resn_exch{"1H3  PC7"}=" H31 PC7"; $resn_exch{"2H3  PC7"}=" H32 PC7"; $resn_exch{"1H12 PC7"}="H121 PC7";
$resn_exch{"2H12 PC7"}="H122 PC7"; $resn_exch{"1H13 PC7"}="H131 PC7"; $resn_exch{"2H13 PC7"}="H132 PC7";
$resn_exch{"1H14 PC7"}="H141 PC7"; $resn_exch{"2H14 PC7"}="H142 PC7"; $resn_exch{"1H15 PC7"}="H151 PC7";
$resn_exch{"2H15 PC7"}="H152 PC7"; $resn_exch{"1H16 PC7"}="H161 PC7"; $resn_exch{"2H16 PC7"}="H162 PC7";
$resn_exch{"1H17 PC7"}="H171 PC7"; $resn_exch{"2H17 PC7"}="H172 PC7"; $resn_exch{"1H18 PC7"}="H181 PC7";
$resn_exch{"2H18 PC7"}="H182 PC7"; $resn_exch{"1H19 PC7"}="H191 PC7"; $resn_exch{"2H19 PC7"}="H192 PC7";
$resn_exch{"1H20 PC7"}="H201 PC7"; $resn_exch{"2H20 PC7"}="H202 PC7"; $resn_exch{"1H21 PC7"}="H211 PC7";
$resn_exch{"2H21 PC7"}="H212 PC7"; $resn_exch{"1H22 PC7"}="H221 PC7"; $resn_exch{"2H22 PC7"}="H222 PC7";
$resn_exch{"1H23 PC7"}="H231 PC7"; $resn_exch{"2H23 PC7"}="H232 PC7"; $resn_exch{"1H24 PC7"}="H241 PC7";
$resn_exch{"2H24 PC7"}="H242 PC7"; $resn_exch{"1H25 PC7"}="H251 PC7"; $resn_exch{"2H25 PC7"}="H252 PC7";
$resn_exch{"1H26 PC7"}="H261 PC7"; $resn_exch{"2H26 PC7"}="H262 PC7"; $resn_exch{"3H26 PC7"}="H263 PC7";
$resn_exch{"1H48 PC9"}="H481 PC9"; $resn_exch{"2H48 PC9"}="H482 PC9"; $resn_exch{"3H48 PC9"}="H483 PC9";
$resn_exch{"1H47 PC9"}="H471 PC9"; $resn_exch{"2H47 PC9"}="H472 PC9"; $resn_exch{"1H46 PC9"}="H461 PC9";
$resn_exch{"2H46 PC9"}="H462 PC9"; $resn_exch{"1H45 PC9"}="H451 PC9"; $resn_exch{"2H45 PC9"}="H452 PC9";
$resn_exch{"1H44 PC9"}="H441 PC9"; $resn_exch{"2H44 PC9"}="H442 PC9"; $resn_exch{"1H43 PC9"}="H431 PC9";
$resn_exch{"2H43 PC9"}="H432 PC9"; $resn_exch{"1H42 PC9"}="H421 PC9"; $resn_exch{"2H42 PC9"}="H422 PC9";
$resn_exch{"1H41 PC9"}="H411 PC9"; $resn_exch{"2H41 PC9"}="H412 PC9"; $resn_exch{"1H40 PC9"}="H401 PC9";
$resn_exch{"2H40 PC9"}="H402 PC9"; $resn_exch{"1H39 PC9"}="H391 PC9"; $resn_exch{"2H39 PC9"}="H392 PC9";
$resn_exch{"1H38 PC9"}="H381 PC9"; $resn_exch{"2H38 PC9"}="H382 PC9"; $resn_exch{"1H35 PC9"}="H351 PC9";
$resn_exch{"2H35 PC9"}="H352 PC9"; $resn_exch{"1H34 PC9"}="H341 PC9"; $resn_exch{"2H34 PC9"}="H342 PC9";
$resn_exch{"1H33 PC9"}="H331 PC9"; $resn_exch{"2H33 PC9"}="H332 PC9"; $resn_exch{"1H32 PC9"}="H321 PC9";
$resn_exch{"2H32 PC9"}="H322 PC9"; $resn_exch{"1H1  PC9"}=" H11 PC9"; $resn_exch{"2H1  PC9"}=" H12 PC9";
$resn_exch{"1H4  PC9"}=" H41 PC9"; $resn_exch{"2H4  PC9"}=" H42 PC9"; $resn_exch{"1H5  PC9"}=" H51 PC9";
$resn_exch{"2H5  PC9"}=" H52 PC9"; $resn_exch{"1H7  PC9"}=" H71 PC9"; $resn_exch{"2H7  PC9"}=" H72 PC9";
$resn_exch{"3H7  PC9"}=" H73 PC9"; $resn_exch{"1H8  PC9"}=" H81 PC9"; $resn_exch{"2H8  PC9"}=" H82 PC9";
$resn_exch{"3H8  PC9"}=" H83 PC9"; $resn_exch{"1H6  PC9"}=" H61 PC9"; $resn_exch{"2H6  PC9"}=" H62 PC9";
$resn_exch{"3H6  PC9"}=" H63 PC9"; $resn_exch{"1H3  PC9"}=" H31 PC9"; $resn_exch{"2H3  PC9"}=" H32 PC9";
$resn_exch{"1H12 PC9"}="H121 PC9"; $resn_exch{"2H12 PC9"}="H122 PC9"; $resn_exch{"1H13 PC9"}="H131 PC9";
$resn_exch{"2H13 PC9"}="H132 PC9"; $resn_exch{"1H14 PC9"}="H141 PC9"; $resn_exch{"2H14 PC9"}="H142 PC9";
$resn_exch{"1H15 PC9"}="H151 PC9"; $resn_exch{"2H15 PC9"}="H152 PC9"; $resn_exch{"1H16 PC9"}="H161 PC9";
$resn_exch{"2H16 PC9"}="H162 PC9"; $resn_exch{"1H17 PC9"}="H171 PC9"; $resn_exch{"2H17 PC9"}="H172 PC9";
$resn_exch{"1H18 PC9"}="H181 PC9"; $resn_exch{"2H18 PC9"}="H182 PC9"; $resn_exch{"1H19 PC9"}="H191 PC9";
$resn_exch{"2H19 PC9"}="H192 PC9"; $resn_exch{"1H20 PC9"}="H201 PC9"; $resn_exch{"2H20 PC9"}="H202 PC9";
$resn_exch{"1H21 PC9"}="H211 PC9"; $resn_exch{"2H21 PC9"}="H212 PC9"; $resn_exch{"1H22 PC9"}="H221 PC9";
$resn_exch{"2H22 PC9"}="H222 PC9"; $resn_exch{"1H23 PC9"}="H231 PC9"; $resn_exch{"2H23 PC9"}="H232 PC9";
$resn_exch{"1H24 PC9"}="H241 PC9"; $resn_exch{"2H24 PC9"}="H242 PC9"; $resn_exch{"1H25 PC9"}="H251 PC9";
$resn_exch{"2H25 PC9"}="H252 PC9"; $resn_exch{"1H26 PC9"}="H261 PC9"; $resn_exch{"2H26 PC9"}="H262 PC9";
$resn_exch{"3H26 PC9"}="H263 PC9"; $resn_exch{" HN  PCA"}=" H   PCA"; $resn_exch{"1HB  PCA"}=" HB2 PCA";
$resn_exch{"2HB  PCA"}=" HB3 PCA"; $resn_exch{"1HG  PCA"}=" HG2 PCA"; $resn_exch{"2HG  PCA"}=" HG3 PCA";
$resn_exch{"1HM  PCB"}=" HM1 PCB"; $resn_exch{"2HM  PCB"}=" HM2 PCB"; $resn_exch{"3HM  PCB"}=" HM3 PCB";
$resn_exch{"1HM' PCB"}="HM'1 PCB"; $resn_exch{"2HM' PCB"}="HM'2 PCB"; $resn_exch{"3HM' PCB"}="HM'3 PCB";
$resn_exch{"1HB  PCC"}=" HB2 PCC"; $resn_exch{"2HB  PCC"}=" HB3 PCC"; $resn_exch{"1HG  PCC"}=" HG2 PCC";
$resn_exch{"2HG  PCC"}=" HG3 PCC"; $resn_exch{"2H4  PCD"}=" H42 PCD"; $resn_exch{"1H4  PCD"}=" H41 PCD";
$resn_exch{" H1* PCD"}=" H1' PCD"; $resn_exch{" H2* PCD"}=" H2' PCD"; $resn_exch{"1H2O PCD"}="H2O1 PCD";
$resn_exch{" H3* PCD"}=" H3' PCD"; $resn_exch{" H4* PCD"}=" H4D PCD"; $resn_exch{"1H3O PCD"}="H3O1 PCD";
$resn_exch{"2H5* PCD"}="H5'2 PCD"; $resn_exch{"1H5* PCD"}="H5'1 PCD"; $resn_exch{"2HOA PCD"}="HOA2 PCD";
$resn_exch{"2HOB PCD"}="HOB2 PCD"; $resn_exch{"2H10 PCD"}="H102 PCD"; $resn_exch{"1H10 PCD"}="H101 PCD";
$resn_exch{"2H2' PCD"}="H2'2 PCD"; $resn_exch{"1H2' PCD"}="H2'1 PCD"; $resn_exch{"1HR1 PCD"}="HR11 PCD";
$resn_exch{"2HR1 PCD"}="HR12 PCD"; $resn_exch{"1HN  PCE"}=" H   PCE"; $resn_exch{"2HN  PCE"}=" H2  PCE";
$resn_exch{"1HB  PCE"}=" HB2 PCE"; $resn_exch{"2HB  PCE"}=" HB3 PCE"; $resn_exch{"1HD1 PCE"}="HD12 PCE";
$resn_exch{"2HD1 PCE"}="HD13 PCE"; $resn_exch{"1HD2 PCE"}="HD23 PCE"; $resn_exch{"2HD2 PCE"}="HD22 PCE";
$resn_exch{"1HE1 PCE"}="HE12 PCE"; $resn_exch{"2HE1 PCE"}="HE13 PCE"; $resn_exch{"1HE2 PCE"}="HE23 PCE";
$resn_exch{"2HE2 PCE"}="HE22 PCE"; $resn_exch{"1HZ  PCE"}=" HZ2 PCE"; $resn_exch{"2HZ  PCE"}=" HZ3 PCE";
$resn_exch{"1HM  PCE"}=" HM1 PCE"; $resn_exch{"2HM  PCE"}=" HM2 PCE"; $resn_exch{"1HB2 PCE"}="HB21 PCE";
$resn_exch{"2HB2 PCE"}="HB22 PCE"; $resn_exch{"1HD3 PCE"}="HD31 PCE"; $resn_exch{"1H11 PCF"}="H111 PCF";
$resn_exch{"2H11 PCF"}="H112 PCF"; $resn_exch{"1H12 PCF"}="H121 PCF"; $resn_exch{"2H12 PCF"}="H122 PCF";
$resn_exch{"1H13 PCF"}="H131 PCF"; $resn_exch{"2H13 PCF"}="H132 PCF"; $resn_exch{"3H13 PCF"}="H133 PCF";
$resn_exch{"1H14 PCF"}="H141 PCF"; $resn_exch{"2H14 PCF"}="H142 PCF"; $resn_exch{"3H14 PCF"}="H143 PCF";
$resn_exch{"1H15 PCF"}="H151 PCF"; $resn_exch{"2H15 PCF"}="H152 PCF"; $resn_exch{"3H15 PCF"}="H153 PCF";
$resn_exch{"1H1  PCF"}=" H11 PCF"; $resn_exch{"2H1  PCF"}=" H12 PCF"; $resn_exch{"1H2  PCF"}=" H21 PCF";
$resn_exch{"1H3  PCF"}=" H31 PCF"; $resn_exch{"2H3  PCF"}=" H32 PCF"; $resn_exch{"1H32 PCF"}="H321 PCF";
$resn_exch{"2H32 PCF"}="H322 PCF"; $resn_exch{"1H33 PCF"}="H331 PCF"; $resn_exch{"2H33 PCF"}="H332 PCF";
$resn_exch{"1H34 PCF"}="H341 PCF"; $resn_exch{"2H34 PCF"}="H342 PCF"; $resn_exch{"1H35 PCF"}="H351 PCF";
$resn_exch{"2H35 PCF"}="H352 PCF"; $resn_exch{"1H36 PCF"}="H361 PCF"; $resn_exch{"2H36 PCF"}="H362 PCF";
$resn_exch{"1H37 PCF"}="H371 PCF"; $resn_exch{"2H37 PCF"}="H372 PCF"; $resn_exch{"1H38 PCF"}="H381 PCF";
$resn_exch{"2H38 PCF"}="H382 PCF"; $resn_exch{"1H39 PCF"}="H391 PCF"; $resn_exch{"2H39 PCF"}="H392 PCF";
$resn_exch{"1H40 PCF"}="H401 PCF"; $resn_exch{"2H40 PCF"}="H402 PCF"; $resn_exch{"1H41 PCF"}="H411 PCF";
$resn_exch{"2H41 PCF"}="H412 PCF"; $resn_exch{"1H42 PCF"}="H421 PCF"; $resn_exch{"2H42 PCF"}="H422 PCF";
$resn_exch{"1H43 PCF"}="H431 PCF"; $resn_exch{"2H43 PCF"}="H432 PCF"; $resn_exch{"1H22 PCF"}="H221 PCF";
$resn_exch{"2H22 PCF"}="H222 PCF"; $resn_exch{"1H23 PCF"}="H231 PCF"; $resn_exch{"2H23 PCF"}="H232 PCF";
$resn_exch{"1H24 PCF"}="H241 PCF"; $resn_exch{"2H24 PCF"}="H242 PCF"; $resn_exch{"1H25 PCF"}="H251 PCF";
$resn_exch{"2H25 PCF"}="H252 PCF"; $resn_exch{"1H26 PCF"}="H261 PCF"; $resn_exch{"2H26 PCF"}="H262 PCF";
$resn_exch{"1H27 PCF"}="H271 PCF"; $resn_exch{"2H27 PCF"}="H272 PCF"; $resn_exch{"1H28 PCF"}="H281 PCF";
$resn_exch{"2H28 PCF"}="H282 PCF"; $resn_exch{"1H29 PCF"}="H291 PCF"; $resn_exch{"2H29 PCF"}="H292 PCF";
$resn_exch{"1H30 PCF"}="H301 PCF"; $resn_exch{"2H30 PCF"}="H302 PCF"; $resn_exch{"1H47 PCF"}="H471 PCF";
$resn_exch{"2H47 PCF"}="H472 PCF"; $resn_exch{"1H48 PCF"}="H481 PCF"; $resn_exch{"2H48 PCF"}="H482 PCF";
$resn_exch{"1H49 PCF"}="H491 PCF"; $resn_exch{"2H49 PCF"}="H492 PCF"; $resn_exch{"1H50 PCF"}="H501 PCF";
$resn_exch{"2H50 PCF"}="H502 PCF"; $resn_exch{"1H51 PCF"}="H511 PCF"; $resn_exch{"2H51 PCF"}="H512 PCF";
$resn_exch{"1H52 PCF"}="H521 PCF"; $resn_exch{"2H52 PCF"}="H522 PCF"; $resn_exch{"3H52 PCF"}="H523 PCF";
$resn_exch{"1H44 PCF"}="H441 PCF"; $resn_exch{"2H44 PCF"}="H442 PCF"; $resn_exch{"1H45 PCF"}="H451 PCF";
$resn_exch{"2H45 PCF"}="H452 PCF"; $resn_exch{"1H46 PCF"}="H461 PCF"; $resn_exch{"2H46 PCF"}="H462 PCF";
$resn_exch{"3H46 PCF"}="H463 PCF"; $resn_exch{"AHO1 PCG"}="HO1A PCG"; $resn_exch{"1H5* PCG"}="H5'1 PCG";
$resn_exch{"2H5* PCG"}="H5'2 PCG"; $resn_exch{" H4* PCG"}=" H4' PCG"; $resn_exch{" H3* PCG"}=" H3' PCG";
$resn_exch{" H2* PCG"}=" H2' PCG"; $resn_exch{"*HO2 PCG"}="HO2' PCG"; $resn_exch{" H1* PCG"}=" H1' PCG";
$resn_exch{"1HN2 PCG"}="HN21 PCG"; $resn_exch{"2HN2 PCG"}="HN22 PCG"; $resn_exch{"1HN  PCH"}=" HN1 PCH";
$resn_exch{"2HN  PCH"}=" HN2 PCH"; $resn_exch{"1HB  PCH"}=" HB1 PCH"; $resn_exch{"2HB  PCH"}=" HB2 PCH";
$resn_exch{"1HD1 PCH"}="HD11 PCH"; $resn_exch{"2HD1 PCH"}="HD12 PCH"; $resn_exch{"1HD2 PCH"}="HD21 PCH";
$resn_exch{"2HD2 PCH"}="HD22 PCH"; $resn_exch{"1HE1 PCH"}="HE11 PCH"; $resn_exch{"2HE1 PCH"}="HE12 PCH";
$resn_exch{"1HE2 PCH"}="HE21 PCH"; $resn_exch{"2HE2 PCH"}="HE22 PCH"; $resn_exch{"1HCZ PCH"}="HCZ1 PCH";
$resn_exch{"2HCZ PCH"}="HCZ2 PCH"; $resn_exch{"1HM  PCH"}=" HM1 PCH"; $resn_exch{"2HM  PCH"}=" HM2 PCH";
$resn_exch{"1H51 PCJ"}="H511 PCJ"; $resn_exch{"2H51 PCJ"}="H512 PCJ"; $resn_exch{"3H51 PCJ"}="H513 PCJ";
$resn_exch{"1H52 PCJ"}="H521 PCJ"; $resn_exch{"2H52 PCJ"}="H522 PCJ"; $resn_exch{"1H53 PCJ"}="H531 PCJ";
$resn_exch{"2H53 PCJ"}="H532 PCJ"; $resn_exch{"1H54 PCJ"}="H541 PCJ"; $resn_exch{"2H54 PCJ"}="H542 PCJ";
$resn_exch{"1H55 PCJ"}="H551 PCJ"; $resn_exch{"2H55 PCJ"}="H552 PCJ"; $resn_exch{"1H56 PCJ"}="H561 PCJ";
$resn_exch{"2H56 PCJ"}="H562 PCJ"; $resn_exch{"1H57 PCJ"}="H571 PCJ"; $resn_exch{"2H57 PCJ"}="H572 PCJ";
$resn_exch{"1H58 PCJ"}="H581 PCJ"; $resn_exch{"2H58 PCJ"}="H582 PCJ"; $resn_exch{"1H59 PCJ"}="H591 PCJ";
$resn_exch{"2H59 PCJ"}="H592 PCJ"; $resn_exch{"1H60 PCJ"}="H601 PCJ"; $resn_exch{"2H60 PCJ"}="H602 PCJ";
$resn_exch{"1H61 PCJ"}="H611 PCJ"; $resn_exch{"2H61 PCJ"}="H612 PCJ"; $resn_exch{"1H62 PCJ"}="H621 PCJ";
$resn_exch{"2H62 PCJ"}="H622 PCJ"; $resn_exch{"1H63 PCJ"}="H631 PCJ"; $resn_exch{"2H63 PCJ"}="H632 PCJ";
$resn_exch{"1H64 PCJ"}="H641 PCJ"; $resn_exch{"2H64 PCJ"}="H642 PCJ"; $resn_exch{"1H65 PCJ"}="H651 PCJ";
$resn_exch{"2H65 PCJ"}="H652 PCJ"; $resn_exch{"1H   PCJ"}=" H1  PCJ"; $resn_exch{"2H   PCJ"}=" H2A PCJ";
$resn_exch{"1HB  PCJ"}=" HB1 PCJ"; $resn_exch{"2HB  PCJ"}=" HB2 PCJ"; $resn_exch{"1H1  PCJ"}=" H11 PCJ";
$resn_exch{"2H1  PCJ"}=" H12 PCJ"; $resn_exch{"1H3  PCJ"}=" H31 PCJ"; $resn_exch{"2H3  PCJ"}=" H32 PCJ";
$resn_exch{"1H25 PCJ"}="H251 PCJ"; $resn_exch{"2H25 PCJ"}="H252 PCJ"; $resn_exch{"1H24 PCJ"}="H241 PCJ";
$resn_exch{"2H24 PCJ"}="H242 PCJ"; $resn_exch{"1H23 PCJ"}="H231 PCJ"; $resn_exch{"2H23 PCJ"}="H232 PCJ";
$resn_exch{"1H22 PCJ"}="H221 PCJ"; $resn_exch{"2H22 PCJ"}="H222 PCJ"; $resn_exch{"1H21 PCJ"}="H211 PCJ";
$resn_exch{"2H21 PCJ"}="H212 PCJ"; $resn_exch{"1H20 PCJ"}="H201 PCJ"; $resn_exch{"2H20 PCJ"}="H202 PCJ";
$resn_exch{"1H19 PCJ"}="H191 PCJ"; $resn_exch{"2H19 PCJ"}="H192 PCJ"; $resn_exch{"1H18 PCJ"}="H181 PCJ";
$resn_exch{"2H18 PCJ"}="H182 PCJ"; $resn_exch{"1H17 PCJ"}="H171 PCJ"; $resn_exch{"2H17 PCJ"}="H172 PCJ";
$resn_exch{"1H16 PCJ"}="H161 PCJ"; $resn_exch{"2H16 PCJ"}="H162 PCJ"; $resn_exch{"1H15 PCJ"}="H151 PCJ";
$resn_exch{"2H15 PCJ"}="H152 PCJ"; $resn_exch{"1H14 PCJ"}="H141 PCJ"; $resn_exch{"2H14 PCJ"}="H142 PCJ";
$resn_exch{"1H13 PCJ"}="H131 PCJ"; $resn_exch{"2H13 PCJ"}="H132 PCJ"; $resn_exch{"1H12 PCJ"}="H121 PCJ";
$resn_exch{"2H12 PCJ"}="H122 PCJ"; $resn_exch{"1H11 PCJ"}="H111 PCJ"; $resn_exch{"2H11 PCJ"}="H112 PCJ";
$resn_exch{"3H11 PCJ"}="H113 PCJ"; $resn_exch{"1H45 PCJ"}="H451 PCJ"; $resn_exch{"2H45 PCJ"}="H452 PCJ";
$resn_exch{"1H44 PCJ"}="H441 PCJ"; $resn_exch{"2H44 PCJ"}="H442 PCJ"; $resn_exch{"1H43 PCJ"}="H431 PCJ";
$resn_exch{"2H43 PCJ"}="H432 PCJ"; $resn_exch{"1H42 PCJ"}="H421 PCJ"; $resn_exch{"2H42 PCJ"}="H422 PCJ";
$resn_exch{"1H41 PCJ"}="H411 PCJ"; $resn_exch{"2H41 PCJ"}="H412 PCJ"; $resn_exch{"1H40 PCJ"}="H401 PCJ";
$resn_exch{"2H40 PCJ"}="H402 PCJ"; $resn_exch{"1H39 PCJ"}="H391 PCJ"; $resn_exch{"2H39 PCJ"}="H392 PCJ";
$resn_exch{"1H38 PCJ"}="H381 PCJ"; $resn_exch{"2H38 PCJ"}="H382 PCJ"; $resn_exch{"1H37 PCJ"}="H371 PCJ";
$resn_exch{"2H37 PCJ"}="H372 PCJ"; $resn_exch{"1H36 PCJ"}="H361 PCJ"; $resn_exch{"2H36 PCJ"}="H362 PCJ";
$resn_exch{"1H35 PCJ"}="H351 PCJ"; $resn_exch{"2H35 PCJ"}="H352 PCJ"; $resn_exch{"1H34 PCJ"}="H341 PCJ";
$resn_exch{"2H34 PCJ"}="H342 PCJ"; $resn_exch{"1H33 PCJ"}="H331 PCJ"; $resn_exch{"2H33 PCJ"}="H332 PCJ";
$resn_exch{"1H32 PCJ"}="H321 PCJ"; $resn_exch{"2H32 PCJ"}="H322 PCJ"; $resn_exch{"1H31 PCJ"}="H311 PCJ";
$resn_exch{"2H31 PCJ"}="H312 PCJ"; $resn_exch{"3H31 PCJ"}="H313 PCJ"; $resn_exch{"1H48 PCK"}="H481 PCK";
$resn_exch{"2H48 PCK"}="H482 PCK"; $resn_exch{"3H48 PCK"}="H483 PCK"; $resn_exch{"1H47 PCK"}="H471 PCK";
$resn_exch{"2H47 PCK"}="H472 PCK"; $resn_exch{"1H46 PCK"}="H461 PCK"; $resn_exch{"2H46 PCK"}="H462 PCK";
$resn_exch{"1H45 PCK"}="H451 PCK"; $resn_exch{"2H45 PCK"}="H452 PCK"; $resn_exch{"1H44 PCK"}="H441 PCK";
$resn_exch{"2H44 PCK"}="H442 PCK"; $resn_exch{"1H43 PCK"}="H431 PCK"; $resn_exch{"2H43 PCK"}="H432 PCK";
$resn_exch{"1H42 PCK"}="H421 PCK"; $resn_exch{"2H42 PCK"}="H422 PCK"; $resn_exch{"1H41 PCK"}="H411 PCK";
$resn_exch{"2H41 PCK"}="H412 PCK"; $resn_exch{"1H38 PCK"}="H381 PCK"; $resn_exch{"2H38 PCK"}="H382 PCK";
$resn_exch{"1H37 PCK"}="H371 PCK"; $resn_exch{"2H37 PCK"}="H372 PCK"; $resn_exch{"1H36 PCK"}="H361 PCK";
$resn_exch{"2H36 PCK"}="H362 PCK"; $resn_exch{"1H35 PCK"}="H351 PCK"; $resn_exch{"2H35 PCK"}="H352 PCK";
$resn_exch{"1H34 PCK"}="H341 PCK"; $resn_exch{"2H34 PCK"}="H342 PCK"; $resn_exch{"1H33 PCK"}="H331 PCK";
$resn_exch{"2H33 PCK"}="H332 PCK"; $resn_exch{"1H32 PCK"}="H321 PCK"; $resn_exch{"2H32 PCK"}="H322 PCK";
$resn_exch{"1H1  PCK"}=" H11 PCK"; $resn_exch{"2H1  PCK"}=" H12 PCK"; $resn_exch{"1H4  PCK"}=" H41 PCK";
$resn_exch{"2H4  PCK"}=" H42 PCK"; $resn_exch{"1H5  PCK"}=" H51 PCK"; $resn_exch{"2H5  PCK"}=" H52 PCK";
$resn_exch{"1H7  PCK"}=" H71 PCK"; $resn_exch{"2H7  PCK"}=" H72 PCK"; $resn_exch{"3H7  PCK"}=" H73 PCK";
$resn_exch{"1H8  PCK"}=" H81 PCK"; $resn_exch{"2H8  PCK"}=" H82 PCK"; $resn_exch{"3H8  PCK"}=" H83 PCK";
$resn_exch{"1H6  PCK"}=" H61 PCK"; $resn_exch{"2H6  PCK"}=" H62 PCK"; $resn_exch{"3H6  PCK"}=" H63 PCK";
$resn_exch{"1H3  PCK"}=" H31 PCK"; $resn_exch{"2H3  PCK"}=" H32 PCK"; $resn_exch{"1H12 PCK"}="H121 PCK";
$resn_exch{"2H12 PCK"}="H122 PCK"; $resn_exch{"1H13 PCK"}="H131 PCK"; $resn_exch{"2H13 PCK"}="H132 PCK";
$resn_exch{"1H14 PCK"}="H141 PCK"; $resn_exch{"2H14 PCK"}="H142 PCK"; $resn_exch{"1H15 PCK"}="H151 PCK";
$resn_exch{"2H15 PCK"}="H152 PCK"; $resn_exch{"1H16 PCK"}="H161 PCK"; $resn_exch{"2H16 PCK"}="H162 PCK";
$resn_exch{"1H17 PCK"}="H171 PCK"; $resn_exch{"2H17 PCK"}="H172 PCK"; $resn_exch{"1H18 PCK"}="H181 PCK";
$resn_exch{"2H18 PCK"}="H182 PCK"; $resn_exch{"1H21 PCK"}="H211 PCK"; $resn_exch{"2H21 PCK"}="H212 PCK";
$resn_exch{"1H22 PCK"}="H221 PCK"; $resn_exch{"2H22 PCK"}="H222 PCK"; $resn_exch{"1H23 PCK"}="H231 PCK";
$resn_exch{"2H23 PCK"}="H232 PCK"; $resn_exch{"1H24 PCK"}="H241 PCK"; $resn_exch{"2H24 PCK"}="H242 PCK";
$resn_exch{"1H25 PCK"}="H251 PCK"; $resn_exch{"2H25 PCK"}="H252 PCK"; $resn_exch{"1H26 PCK"}="H261 PCK";
$resn_exch{"2H26 PCK"}="H262 PCK"; $resn_exch{"1H27 PCK"}="H271 PCK"; $resn_exch{"2H27 PCK"}="H272 PCK";
$resn_exch{"1H28 PCK"}="H281 PCK"; $resn_exch{"2H28 PCK"}="H282 PCK"; $resn_exch{"3H28 PCK"}="H283 PCK";
$resn_exch{"1H7  PCM"}=" H71 PCM"; $resn_exch{"2H7  PCM"}=" H72 PCM"; $resn_exch{"1H12 PCM"}="H121 PCM";
$resn_exch{"2H12 PCM"}="H122 PCM"; $resn_exch{"1H14 PCM"}="H141 PCM"; $resn_exch{"2H14 PCM"}="H142 PCM";
$resn_exch{"3H14 PCM"}="H143 PCM"; $resn_exch{"1H15 PCM"}="H151 PCM"; $resn_exch{"2H15 PCM"}="H152 PCM";
$resn_exch{"3H15 PCM"}="H153 PCM"; $resn_exch{"1H21 PCM"}="H211 PCM"; $resn_exch{"2H21 PCM"}="H212 PCM";
$resn_exch{"1H30 PCM"}="H301 PCM"; $resn_exch{"2H30 PCM"}="H302 PCM"; $resn_exch{"1H35 PCM"}="H351 PCM";
$resn_exch{"2H35 PCM"}="H352 PCM"; $resn_exch{"1H37 PCM"}="H371 PCM"; $resn_exch{"2H37 PCM"}="H372 PCM";
$resn_exch{"3H37 PCM"}="H373 PCM"; $resn_exch{"1H38 PCM"}="H381 PCM"; $resn_exch{"2H38 PCM"}="H382 PCM";
$resn_exch{"3H38 PCM"}="H383 PCM"; $resn_exch{"1H42 PCM"}="H421 PCM"; $resn_exch{"2H42 PCM"}="H422 PCM";
$resn_exch{"1H51 PCN"}="H511 PCN"; $resn_exch{"2H51 PCN"}="H512 PCN"; $resn_exch{"3H51 PCN"}="H513 PCN";
$resn_exch{"1H52 PCN"}="H521 PCN"; $resn_exch{"2H52 PCN"}="H522 PCN"; $resn_exch{"3H52 PCN"}="H523 PCN";
$resn_exch{"OH63 PCN"}="H63O PCN"; $resn_exch{"1H13 PCN"}="H131 PCN"; $resn_exch{"2H13 PCN"}="H132 PCN";
$resn_exch{"3H13 PCN"}="H133 PCN"; $resn_exch{"1H19 PCN"}="H191 PCN"; $resn_exch{"2H19 PCN"}="H192 PCN";
$resn_exch{"1H16 PCO"}="H161 PCO"; $resn_exch{"2H16 PCO"}="H162 PCO"; $resn_exch{"1H15 PCO"}="H151 PCO";
$resn_exch{"2H15 PCO"}="H152 PCO"; $resn_exch{"1HC6 PCO"}="HC61 PCO"; $resn_exch{"2HC6 PCO"}="HC62 PCO";
$resn_exch{"1HC8 PCO"}="HC81 PCO"; $resn_exch{"2HC8 PCO"}="HC82 PCO"; $resn_exch{"3HC8 PCO"}="HC83 PCO";
$resn_exch{"1HC9 PCO"}="HC91 PCO"; $resn_exch{"2HC9 PCO"}="HC92 PCO"; $resn_exch{"3HC9 PCO"}="HC93 PCO";
$resn_exch{"1H5  PCP"}=" H51 PCP"; $resn_exch{"2H5  PCP"}=" H52 PCP"; $resn_exch{"1HP  PCP"}=" HP1 PCP";
$resn_exch{"2HP  PCP"}=" HP2 PCP"; $resn_exch{"2HOP PCP"}="HOP2 PCP"; $resn_exch{"3HOP PCP"}="HOP3 PCP";
$resn_exch{"2HOA PCP"}="HOA2 PCP"; $resn_exch{"2HOB PCP"}="HOB2 PCP"; $resn_exch{"3HOB PCP"}="HOB3 PCP";
$resn_exch{"1H2  PCQ"}=" H21 PCQ"; $resn_exch{"1H6  PCQ"}=" H61 PCQ"; $resn_exch{"1H2' PCQ"}="H2'1 PCQ";
$resn_exch{"1H6' PCQ"}="H6'1 PCQ"; $resn_exch{"1HB  PCR"}=" HB1 PCR"; $resn_exch{"2HB  PCR"}=" HB2 PCR";
$resn_exch{"3HB  PCR"}=" HB3 PCR"; $resn_exch{"1HD1 PCR"}="HD11 PCR"; $resn_exch{"1HE1 PCR"}="HE11 PCR";
$resn_exch{"1HD2 PCR"}="HD21 PCR"; $resn_exch{"1HE2 PCR"}="HE21 PCR"; $resn_exch{"1HO  PCR"}=" HO1 PCR";
$resn_exch{" HN2 PCS"}=" H2  PCS"; $resn_exch{"1HB  PCS"}=" HB2 PCS"; $resn_exch{"2HB  PCS"}=" HB3 PCS";
$resn_exch{"1HT  PCS"}=" HT1 PCS"; $resn_exch{"2HT  PCS"}=" HT2 PCS"; $resn_exch{"2HOP PCT"}="HOP2 PCT";
$resn_exch{"3HOP PCT"}="HOP3 PCT"; $resn_exch{"1H1P PCT"}="H1P1 PCT"; $resn_exch{"2H1P PCT"}="H1P2 PCT";
$resn_exch{"1HN1 PCT"}="HN11 PCT"; $resn_exch{"2HN1 PCT"}="HN12 PCT"; $resn_exch{"1HMA PCU"}="HMA1 PCU";
$resn_exch{"2HMA PCU"}="HMA2 PCU"; $resn_exch{"3HMA PCU"}="HMA3 PCU"; $resn_exch{"1HMB PCU"}="HMB1 PCU";
$resn_exch{"2HMB PCU"}="HMB2 PCU"; $resn_exch{"3HMB PCU"}="HMB3 PCU"; $resn_exch{"1HMC PCU"}="HMC1 PCU";
$resn_exch{"2HMC PCU"}="HMC2 PCU"; $resn_exch{"3HMC PCU"}="HMC3 PCU"; $resn_exch{"1HMD PCU"}="HMD1 PCU";
$resn_exch{"2HMD PCU"}="HMD2 PCU"; $resn_exch{"3HMD PCU"}="HMD3 PCU"; $resn_exch{"1HC9 PCV"}="HC91 PCV";
$resn_exch{"2HC9 PCV"}="HC92 PCV"; $resn_exch{"1HC7 PCV"}="HC71 PCV"; $resn_exch{"2HC7 PCV"}="HC72 PCV";
$resn_exch{"2H2  PCV"}=" H22 PCV"; $resn_exch{"1HC4 PCV"}="HC41 PCV"; $resn_exch{"2HC4 PCV"}="HC42 PCV";
$resn_exch{"1HC5 PCV"}="HC51 PCV"; $resn_exch{"2HC5 PCV"}="HC52 PCV"; $resn_exch{"1HN2 PCV"}="HN21 PCV";
$resn_exch{"2HN2 PCV"}="HN22 PCV"; $resn_exch{"1H1  PCW"}=" H11 PCW"; $resn_exch{"2H1  PCW"}=" H12 PCW";
$resn_exch{"1H3  PCW"}=" H31 PCW"; $resn_exch{"2H3  PCW"}=" H32 PCW"; $resn_exch{"1H4  PCW"}=" H41 PCW";
$resn_exch{"2H4  PCW"}=" H42 PCW"; $resn_exch{"1H5  PCW"}=" H51 PCW"; $resn_exch{"2H5  PCW"}=" H52 PCW";
$resn_exch{"1H6  PCW"}=" H61 PCW"; $resn_exch{"2H6  PCW"}=" H62 PCW"; $resn_exch{"3H6  PCW"}=" H63 PCW";
$resn_exch{"1H7  PCW"}=" H71 PCW"; $resn_exch{"2H7  PCW"}=" H72 PCW"; $resn_exch{"3H7  PCW"}=" H73 PCW";
$resn_exch{"1H8  PCW"}=" H81 PCW"; $resn_exch{"2H8  PCW"}=" H82 PCW"; $resn_exch{"3H8  PCW"}=" H83 PCW";
$resn_exch{"1H12 PCW"}="H121 PCW"; $resn_exch{"2H12 PCW"}="H122 PCW"; $resn_exch{"1H13 PCW"}="H131 PCW";
$resn_exch{"2H13 PCW"}="H132 PCW"; $resn_exch{"1H14 PCW"}="H141 PCW"; $resn_exch{"2H14 PCW"}="H142 PCW";
$resn_exch{"1H15 PCW"}="H151 PCW"; $resn_exch{"2H15 PCW"}="H152 PCW"; $resn_exch{"1H16 PCW"}="H161 PCW";
$resn_exch{"2H16 PCW"}="H162 PCW"; $resn_exch{"1H17 PCW"}="H171 PCW"; $resn_exch{"2H17 PCW"}="H172 PCW";
$resn_exch{"1H18 PCW"}="H181 PCW"; $resn_exch{"2H18 PCW"}="H182 PCW"; $resn_exch{"1H21 PCW"}="H211 PCW";
$resn_exch{"2H21 PCW"}="H212 PCW"; $resn_exch{"1H22 PCW"}="H221 PCW"; $resn_exch{"2H22 PCW"}="H222 PCW";
$resn_exch{"1H23 PCW"}="H231 PCW"; $resn_exch{"2H23 PCW"}="H232 PCW"; $resn_exch{"1H24 PCW"}="H241 PCW";
$resn_exch{"2H24 PCW"}="H242 PCW"; $resn_exch{"1H25 PCW"}="H251 PCW"; $resn_exch{"2H25 PCW"}="H252 PCW";
$resn_exch{"1H26 PCW"}="H261 PCW"; $resn_exch{"2H26 PCW"}="H262 PCW"; $resn_exch{"1H27 PCW"}="H271 PCW";
$resn_exch{"2H27 PCW"}="H272 PCW"; $resn_exch{"1H28 PCW"}="H281 PCW"; $resn_exch{"2H28 PCW"}="H282 PCW";
$resn_exch{"3H28 PCW"}="H283 PCW"; $resn_exch{"1H32 PCW"}="H321 PCW"; $resn_exch{"2H32 PCW"}="H322 PCW";
$resn_exch{"1H33 PCW"}="H331 PCW"; $resn_exch{"2H33 PCW"}="H332 PCW"; $resn_exch{"1H34 PCW"}="H341 PCW";
$resn_exch{"2H34 PCW"}="H342 PCW"; $resn_exch{"1H35 PCW"}="H351 PCW"; $resn_exch{"2H35 PCW"}="H352 PCW";
$resn_exch{"1H36 PCW"}="H361 PCW"; $resn_exch{"2H36 PCW"}="H362 PCW"; $resn_exch{"1H37 PCW"}="H371 PCW";
$resn_exch{"2H37 PCW"}="H372 PCW"; $resn_exch{"1H38 PCW"}="H381 PCW"; $resn_exch{"2H38 PCW"}="H382 PCW";
$resn_exch{"1H41 PCW"}="H411 PCW"; $resn_exch{"2H41 PCW"}="H412 PCW"; $resn_exch{"1H42 PCW"}="H421 PCW";
$resn_exch{"2H42 PCW"}="H422 PCW"; $resn_exch{"1H43 PCW"}="H431 PCW"; $resn_exch{"2H43 PCW"}="H432 PCW";
$resn_exch{"1H44 PCW"}="H441 PCW"; $resn_exch{"2H44 PCW"}="H442 PCW"; $resn_exch{"1H45 PCW"}="H451 PCW";
$resn_exch{"2H45 PCW"}="H452 PCW"; $resn_exch{"1H46 PCW"}="H461 PCW"; $resn_exch{"2H46 PCW"}="H462 PCW";
$resn_exch{"1H47 PCW"}="H471 PCW"; $resn_exch{"2H47 PCW"}="H472 PCW"; $resn_exch{"1H48 PCW"}="H481 PCW";
$resn_exch{"2H48 PCW"}="H482 PCW"; $resn_exch{"3H48 PCW"}="H483 PCW"; $resn_exch{"1H2C PCX"}="H2C1 PCX";
$resn_exch{"2H2C PCX"}="H2C2 PCX"; $resn_exch{"1H4C PCX"}="H4C1 PCX"; $resn_exch{"2H4C PCX"}="H4C2 PCX";
$resn_exch{"1H5C PCX"}="H5C1 PCX"; $resn_exch{"2H5C PCX"}="H5C2 PCX"; $resn_exch{"1H3N PCX"}="H3N1 PCX";
$resn_exch{"2H3N PCX"}="H3N2 PCX"; $resn_exch{"1H9C PCX"}="H9C1 PCX"; $resn_exch{"2H9C PCX"}="H9C2 PCX";
$resn_exch{"1H7C PCX"}="H7C1 PCX"; $resn_exch{"2H7C PCX"}="H7C2 PCX"; $resn_exch{"1H9  PCY"}=" H91 PCY";
$resn_exch{"2H9  PCY"}=" H92 PCY"; $resn_exch{"3H9  PCY"}=" H93 PCY"; $resn_exch{"1H10 PCY"}="H101 PCY";
$resn_exch{"2H10 PCY"}="H102 PCY"; $resn_exch{"3H10 PCY"}="H103 PCY"; $resn_exch{"1H11 PCY"}="H111 PCY";
$resn_exch{"2H11 PCY"}="H112 PCY"; $resn_exch{"3H11 PCY"}="H113 PCY"; $resn_exch{"1H13 PCY"}="H131 PCY";
$resn_exch{"2H13 PCY"}="H132 PCY"; $resn_exch{"3H13 PCY"}="H133 PCY"; $resn_exch{"1H16 PCY"}="H161 PCY";
$resn_exch{"2H16 PCY"}="H162 PCY"; $resn_exch{"1H18 PCY"}="H181 PCY"; $resn_exch{"2H18 PCY"}="H182 PCY";
$resn_exch{"1H34 PCY"}="H341 PCY"; $resn_exch{"2H34 PCY"}="H342 PCY"; $resn_exch{"3H34 PCY"}="H343 PCY";
$resn_exch{"1H38 PCY"}="H381 PCY"; $resn_exch{"2H38 PCY"}="H382 PCY"; $resn_exch{"3H38 PCY"}="H383 PCY";
$resn_exch{"1H51 PCZ"}="H511 PCZ"; $resn_exch{"2H51 PCZ"}="H512 PCZ"; $resn_exch{"1H52 PCZ"}="H521 PCZ";
$resn_exch{"2H52 PCZ"}="H522 PCZ"; $resn_exch{"1H13 PCZ"}="H131 PCZ"; $resn_exch{"2H13 PCZ"}="H132 PCZ";
$resn_exch{"3H13 PCZ"}="H133 PCZ"; $resn_exch{"1H19 PCZ"}="H191 PCZ"; $resn_exch{"2H19 PCZ"}="H192 PCZ";
$resn_exch{"1H1  PD1"}=" H11 PD1"; $resn_exch{"2H1  PD1"}=" H12 PD1"; $resn_exch{"3H1  PD1"}="H13A PD1";
$resn_exch{"1H2  PD1"}="H21A PD1"; $resn_exch{"2H2  PD1"}=" H22 PD1"; $resn_exch{"1H3  PD1"}=" H31 PD1";
$resn_exch{"2H3  PD1"}=" H32 PD1"; $resn_exch{"3H3  PD1"}=" H33 PD1"; $resn_exch{"1H4  PD1"}=" H41 PD1";
$resn_exch{"2H4  PD1"}=" H42 PD1"; $resn_exch{"1H5  PD1"}=" H51 PD1"; $resn_exch{"2H5  PD1"}=" H52 PD1";
$resn_exch{"1H6  PD1"}=" H61 PD1"; $resn_exch{"2H6  PD1"}=" H62 PD1"; $resn_exch{"1H7  PD1"}=" H71 PD1";
$resn_exch{"2H7  PD1"}=" H72 PD1"; $resn_exch{"1H8  PD1"}=" H81 PD1"; $resn_exch{"2H8  PD1"}=" H82 PD1";
$resn_exch{"1H22 PD1"}="H221 PD1"; $resn_exch{"2H22 PD1"}="H222 PD1"; $resn_exch{"3H22 PD1"}="H223 PD1";
$resn_exch{"1H23 PD1"}="H231 PD1"; $resn_exch{"2H23 PD1"}="H232 PD1"; $resn_exch{"3H23 PD1"}="H233 PD1";
$resn_exch{"1H26 PD1"}="H261 PD1"; $resn_exch{"2H26 PD1"}="H262 PD1"; $resn_exch{"3H26 PD1"}="H263 PD1";
$resn_exch{"1H27 PD1"}="H271 PD1"; $resn_exch{"2H27 PD1"}="H272 PD1"; $resn_exch{"3H27 PD1"}="H273 PD1";
$resn_exch{"1H28 PD1"}="H281 PD1"; $resn_exch{"2H28 PD1"}="H282 PD1"; $resn_exch{"3H28 PD1"}="H283 PD1";
$resn_exch{"H1\"  PD7"}="H1'' PD7"; $resn_exch{"H1\"A PD7"}="H1AA PD7"; $resn_exch{"H2\"  PD7"}="H2'' PD7";
$resn_exch{"H2\"A PD7"}="H2AA PD7"; $resn_exch{"H3\"  PD7"}="H3'' PD7"; $resn_exch{"H3\"A PD7"}="H3AA PD7";
$resn_exch{"H4\"  PD7"}="H4'' PD7"; $resn_exch{"H4\"A PD7"}="H4AA PD7"; $resn_exch{"H5\"  PD7"}="H5'' PD7";
$resn_exch{"H5\"A PD7"}="H5AA PD7"; $resn_exch{"H6\"  PD7"}="H6'' PD7"; $resn_exch{"H6\"A PD7"}="H6AA PD7";
$resn_exch{"H6\"B PD7"}="H6AB PD7"; $resn_exch{"1H5N PD8"}="H5N1 PD8"; $resn_exch{"2H5N PD8"}="H5N2 PD8";
$resn_exch{"1H11 PD8"}="H111 PD8"; $resn_exch{"2H11 PD8"}="H112 PD8"; $resn_exch{"1H2A PDA"}="H2A1 PDA";
$resn_exch{"2H2A PDA"}="H2A2 PDA"; $resn_exch{"3H2A PDA"}="H2A3 PDA"; $resn_exch{"1H4A PDA"}="H4A1 PDA";
$resn_exch{"2H4A PDA"}="H4A2 PDA"; $resn_exch{"1H5A PDA"}="H5A1 PDA"; $resn_exch{"2H5A PDA"}="H5A2 PDA";
$resn_exch{"1HB  PDA"}=" HB1 PDA"; $resn_exch{"2HB  PDA"}=" HB2 PDA"; $resn_exch{"3HB  PDA"}=" HB3 PDA";
$resn_exch{" HN  PDD"}=" H   PDD"; $resn_exch{"1H2A PDD"}="H2A1 PDD"; $resn_exch{"2H2A PDD"}="H2A2 PDD";
$resn_exch{"3H2A PDD"}="H2A3 PDD"; $resn_exch{"1H4A PDD"}="H4A1 PDD"; $resn_exch{"2H4A PDD"}="H4A2 PDD";
$resn_exch{"1H5A PDD"}="H5A1 PDD"; $resn_exch{"2H5A PDD"}="H5A2 PDD"; $resn_exch{"2HOP PDD"}="HOP2 PDD";
$resn_exch{"3HOP PDD"}="HOP3 PDD"; $resn_exch{"1HB  PDD"}=" HB1 PDD"; $resn_exch{"2HB  PDD"}=" HB2 PDD";
$resn_exch{"3HB  PDD"}=" HB3 PDD"; $resn_exch{"1HC8 PDE"}="HC81 PDE"; $resn_exch{"2HC8 PDE"}="HC82 PDE";
$resn_exch{"1HC9 PDE"}="HC91 PDE"; $resn_exch{"2HC9 PDE"}="HC92 PDE"; $resn_exch{"1H10 PDE"}="H101 PDE";
$resn_exch{"2H10 PDE"}="H102 PDE"; $resn_exch{"1H1D PDE"}="H1D1 PDE"; $resn_exch{"2H1D PDE"}="H1D2 PDE";
$resn_exch{"3H1D PDE"}="H1D3 PDE"; $resn_exch{"1H2A PDG"}="H2A1 PDG"; $resn_exch{"2H2A PDG"}="H2A2 PDG";
$resn_exch{"3H2A PDG"}="H2A3 PDG"; $resn_exch{"1H4A PDG"}="H4A1 PDG"; $resn_exch{"2H4A PDG"}="H4A2 PDG";
$resn_exch{"1H5A PDG"}="H5A1 PDG"; $resn_exch{"2H5A PDG"}="H5A2 PDG"; $resn_exch{"1HBA PDG"}="HBA1 PDG";
$resn_exch{"2HBA PDG"}="HBA2 PDG"; $resn_exch{"1HGA PDG"}="HGA1 PDG"; $resn_exch{"2HGA PDG"}="HGA2 PDG";
$resn_exch{"1HB  PDH"}=" HB1 PDH"; $resn_exch{"2HB  PDH"}=" HB2 PDH"; $resn_exch{"3HOP PDI"}="HOP3 PDI";
$resn_exch{"2HOP PDI"}="HOP2 PDI"; $resn_exch{"1H11 PDJ"}="H111 PDJ"; $resn_exch{"2H11 PDJ"}="H112 PDJ";
$resn_exch{"3H11 PDJ"}="H113 PDJ"; $resn_exch{"1H12 PDJ"}="H121 PDJ"; $resn_exch{"2H12 PDJ"}="H122 PDJ";
$resn_exch{"1H13 PDJ"}="H131 PDJ"; $resn_exch{"2H13 PDJ"}="H132 PDJ"; $resn_exch{"1H14 PDJ"}="H141 PDJ";
$resn_exch{"2H14 PDJ"}="H142 PDJ"; $resn_exch{"1H15 PDJ"}="H151 PDJ"; $resn_exch{"2H15 PDJ"}="H152 PDJ";
$resn_exch{"1H16 PDJ"}="H161 PDJ"; $resn_exch{"2H16 PDJ"}="H162 PDJ"; $resn_exch{"1H17 PDJ"}="H171 PDJ";
$resn_exch{"2H17 PDJ"}="H172 PDJ"; $resn_exch{"1H18 PDJ"}="H181 PDJ"; $resn_exch{"2H18 PDJ"}="H182 PDJ";
$resn_exch{"1H19 PDJ"}="H191 PDJ"; $resn_exch{"2H19 PDJ"}="H192 PDJ"; $resn_exch{"1H20 PDJ"}="H201 PDJ";
$resn_exch{"2H20 PDJ"}="H202 PDJ"; $resn_exch{"1H21 PDJ"}="H211 PDJ"; $resn_exch{"2H21 PDJ"}="H212 PDJ";
$resn_exch{"1H22 PDJ"}="H221 PDJ"; $resn_exch{"2H22 PDJ"}="H222 PDJ"; $resn_exch{"1H23 PDJ"}="H231 PDJ";
$resn_exch{"2H23 PDJ"}="H232 PDJ"; $resn_exch{"1H24 PDJ"}="H241 PDJ"; $resn_exch{"2H24 PDJ"}="H242 PDJ";
$resn_exch{"1H25 PDJ"}="H251 PDJ"; $resn_exch{"2H25 PDJ"}="H252 PDJ"; $resn_exch{"1H3  PDJ"}=" H31 PDJ";
$resn_exch{"2H3  PDJ"}=" H32 PDJ"; $resn_exch{"1H1  PDJ"}=" H11 PDJ"; $resn_exch{"2H1  PDJ"}=" H12 PDJ";
$resn_exch{"1HB  PDJ"}=" HB1 PDJ"; $resn_exch{"2HB  PDJ"}=" HB2 PDJ"; $resn_exch{"1HN  PDJ"}=" HN1 PDJ";
$resn_exch{"2HN  PDJ"}=" HN2 PDJ"; $resn_exch{"1H   PDJ"}=" H1  PDJ"; $resn_exch{"2H   PDJ"}=" H2A PDJ";
$resn_exch{"1H45 PDJ"}="H451 PDJ"; $resn_exch{"2H45 PDJ"}="H452 PDJ"; $resn_exch{"1H44 PDJ"}="H441 PDJ";
$resn_exch{"2H44 PDJ"}="H442 PDJ"; $resn_exch{"1H43 PDJ"}="H431 PDJ"; $resn_exch{"2H43 PDJ"}="H432 PDJ";
$resn_exch{"1H42 PDJ"}="H421 PDJ"; $resn_exch{"2H42 PDJ"}="H422 PDJ"; $resn_exch{"1H41 PDJ"}="H411 PDJ";
$resn_exch{"2H41 PDJ"}="H412 PDJ"; $resn_exch{"1H40 PDJ"}="H401 PDJ"; $resn_exch{"2H40 PDJ"}="H402 PDJ";
$resn_exch{"1H39 PDJ"}="H391 PDJ"; $resn_exch{"2H39 PDJ"}="H392 PDJ"; $resn_exch{"1H38 PDJ"}="H381 PDJ";
$resn_exch{"2H38 PDJ"}="H382 PDJ"; $resn_exch{"1H37 PDJ"}="H371 PDJ"; $resn_exch{"2H37 PDJ"}="H372 PDJ";
$resn_exch{"1H36 PDJ"}="H361 PDJ"; $resn_exch{"2H36 PDJ"}="H362 PDJ"; $resn_exch{"1H35 PDJ"}="H351 PDJ";
$resn_exch{"2H35 PDJ"}="H352 PDJ"; $resn_exch{"1H34 PDJ"}="H341 PDJ"; $resn_exch{"2H34 PDJ"}="H342 PDJ";
$resn_exch{"1H33 PDJ"}="H331 PDJ"; $resn_exch{"2H33 PDJ"}="H332 PDJ"; $resn_exch{"1H32 PDJ"}="H321 PDJ";
$resn_exch{"2H32 PDJ"}="H322 PDJ"; $resn_exch{"1H31 PDJ"}="H311 PDJ"; $resn_exch{"2H31 PDJ"}="H312 PDJ";
$resn_exch{"3H31 PDJ"}="H313 PDJ"; $resn_exch{" HN  PDL"}=" H   PDL"; $resn_exch{"1H2A PDL"}="H2A1 PDL";
$resn_exch{"2H2A PDL"}="H2A2 PDL"; $resn_exch{"3H2A PDL"}="H2A3 PDL"; $resn_exch{"1H4A PDL"}="H4A1 PDL";
$resn_exch{"2H4A PDL"}="H4A2 PDL"; $resn_exch{"1H5A PDL"}="H5A1 PDL"; $resn_exch{"2H5A PDL"}="H5A2 PDL";
$resn_exch{"2HOP PDL"}="HOP2 PDL"; $resn_exch{"3HOP PDL"}="HOP3 PDL"; $resn_exch{"1HB  PDL"}=" HB1 PDL";
$resn_exch{"2HB  PDL"}=" HB2 PDL"; $resn_exch{"3HB  PDL"}=" HB3 PDL"; $resn_exch{"1HP8 PDM"}="HP81 PDM";
$resn_exch{"2HP8 PDM"}="HP82 PDM"; $resn_exch{"1HP9 PDM"}="HP91 PDM"; $resn_exch{"2HP9 PDM"}="HP92 PDM";
$resn_exch{"NHP2 PDM"}="HP2N PDM"; $resn_exch{"OHP3 PDM"}="HP3O PDM"; $resn_exch{"OHP4 PDM"}="HP4O PDM";
$resn_exch{"1H12 PDM"}="H121 PDM"; $resn_exch{"2H12 PDM"}="H122 PDM"; $resn_exch{"1H13 PDM"}="H131 PDM";
$resn_exch{"2H13 PDM"}="H132 PDM"; $resn_exch{"1HC  PDN"}=" HC1 PDN"; $resn_exch{"2HC  PDN"}=" HC2 PDN";
$resn_exch{"4HC  PDN"}=" HC4 PDN"; $resn_exch{"1HC6 PDN"}="HC61 PDN"; $resn_exch{"2HC6 PDN"}="HC62 PDN";
$resn_exch{"1HC7 PDN"}="HC71 PDN"; $resn_exch{"2HC7 PDN"}="HC72 PDN"; $resn_exch{"8HC  PDN"}=" HC8 PDN";
$resn_exch{"9HC  PDN"}=" HC9 PDN"; $resn_exch{"1H12 PDN"}="H121 PDN"; $resn_exch{"2H12 PDN"}="H122 PDN";
$resn_exch{"1HC4 PDN"}="HC41 PDN"; $resn_exch{"1H15 PDN"}="H151 PDN"; $resn_exch{"2H15 PDN"}="H152 PDN";
$resn_exch{"1H16 PDN"}="H161 PDN"; $resn_exch{"2H16 PDN"}="H162 PDN"; $resn_exch{"1H18 PDN"}="H181 PDN";
$resn_exch{"2H18 PDN"}="H182 PDN"; $resn_exch{"3H18 PDN"}="H183 PDN"; $resn_exch{"1H19 PDN"}="H191 PDN";
$resn_exch{"2H19 PDN"}="H192 PDN"; $resn_exch{"3H19 PDN"}="H193 PDN"; $resn_exch{"2HC1 PDN"}="HC12 PDN";
$resn_exch{"3HO  PDN"}=" HO3 PDN"; $resn_exch{"4HO  PDN"}=" HO4 PDN"; $resn_exch{"5HO  PDN"}=" HO5 PDN";
$resn_exch{"1H1  PDO"}=" H11 PDO"; $resn_exch{"2H1  PDO"}=" H12 PDO"; $resn_exch{"1H2  PDO"}=" H21 PDO";
$resn_exch{"2H2  PDO"}=" H22 PDO"; $resn_exch{"1H3  PDO"}=" H31 PDO"; $resn_exch{"2H3  PDO"}=" H32 PDO";
$resn_exch{"1H2A PDP"}="H2A1 PDP"; $resn_exch{"2H2A PDP"}="H2A2 PDP"; $resn_exch{"3H2A PDP"}="H2A3 PDP";
$resn_exch{"1H5A PDP"}="H5A1 PDP"; $resn_exch{"2H5A PDP"}="H5A2 PDP"; $resn_exch{"2HOA PDP"}="HOA2 PDP";
$resn_exch{"2HOB PDP"}="HOB2 PDP"; $resn_exch{"3HOB PDP"}="HOB3 PDP"; $resn_exch{"1H26 PDS"}="H261 PDS";
$resn_exch{"2H26 PDS"}="H262 PDS"; $resn_exch{"3H26 PDS"}="H263 PDS"; $resn_exch{"1H10 PDS"}="H101 PDS";
$resn_exch{"2H10 PDS"}="H102 PDS"; $resn_exch{"3H10 PDS"}="H103 PDS"; $resn_exch{"1H17 PDS"}="H171 PDS";
$resn_exch{"2H17 PDS"}="H172 PDS"; $resn_exch{"1H22 PDS"}="H221 PDS"; $resn_exch{"2H22 PDS"}="H222 PDS";
$resn_exch{"1H27 PDS"}="H271 PDS"; $resn_exch{"2H27 PDS"}="H272 PDS"; $resn_exch{"1H32 PDS"}="H321 PDS";
$resn_exch{"2H32 PDS"}="H322 PDS"; $resn_exch{"3H32 PDS"}="H323 PDS"; $resn_exch{"1H33 PDS"}="H331 PDS";
$resn_exch{"2H33 PDS"}="H332 PDS"; $resn_exch{"3H33 PDS"}="H333 PDS"; $resn_exch{"1H2  PDT"}=" H21 PDT";
$resn_exch{"2H2  PDT"}=" H22 PDT"; $resn_exch{"1H1  PDT"}=" H11 PDT"; $resn_exch{"2H1  PDT"}=" H12 PDT";
$resn_exch{"1H3  PDT"}=" H31 PDT"; $resn_exch{"2H3  PDT"}=" H32 PDT"; $resn_exch{"3HOP PDU"}="HOP3 PDU";
$resn_exch{"2HOP PDU"}="HOP2 PDU"; $resn_exch{"1H5* PDU"}=" H5' PDU"; $resn_exch{"2H5* PDU"}="H5'' PDU";
$resn_exch{" H4* PDU"}=" H4' PDU"; $resn_exch{" H3* PDU"}=" H3' PDU"; $resn_exch{"H3T  PDU"}="HO3' PDU";
$resn_exch{"2H2* PDU"}="H2'' PDU"; $resn_exch{"1H2* PDU"}=" H2' PDU"; $resn_exch{" H1* PDU"}=" H1' PDU";
$resn_exch{"1H5A PDU"}="H5A1 PDU"; $resn_exch{"2H5A PDU"}="H5A2 PDU"; $resn_exch{"1H5B PDU"}="H5B1 PDU";
$resn_exch{"2H5B PDU"}="H5B2 PDU"; $resn_exch{"1H5M PDU"}=" H71 PDU"; $resn_exch{"2H5M PDU"}=" H72 PDU";
$resn_exch{"3H5M PDU"}=" H73 PDU"; $resn_exch{"1H6C PDX"}="H6C1 PDX"; $resn_exch{"2H6C PDX"}="H6C2 PDX";
$resn_exch{"1HN6 PE0"}="HN61 PE0"; $resn_exch{"2HN6 PE0"}="HN62 PE0"; $resn_exch{"1H2A PE1"}="H2A1 PE1";
$resn_exch{"2H2A PE1"}="H2A2 PE1"; $resn_exch{"3H2A PE1"}="H2A3 PE1"; $resn_exch{"1H4A PE1"}="H4A1 PE1";
$resn_exch{"2H4A PE1"}="H4A2 PE1"; $resn_exch{"1H5A PE1"}="H5A1 PE1"; $resn_exch{"2H5A PE1"}="H5A2 PE1";
$resn_exch{"1HBC PE1"}="HBC1 PE1"; $resn_exch{"2HBC PE1"}="HBC2 PE1"; $resn_exch{"1HGC PE1"}="HGC1 PE1";
$resn_exch{"2HGC PE1"}="HGC2 PE1"; $resn_exch{"1HDC PE1"}="HDC1 PE1"; $resn_exch{"2HDC PE1"}="HDC2 PE1";
$resn_exch{"1HEC PE1"}="HEC1 PE1"; $resn_exch{"2HEC PE1"}="HEC2 PE1"; $resn_exch{"1HZN PE1"}="HZN1 PE1";
$resn_exch{"2HZN PE1"}="HZN2 PE1"; $resn_exch{"1H4* PE2"}="H4'1 PE2"; $resn_exch{"2H4* PE2"}="H4'2 PE2";
$resn_exch{"*HO4 PE2"}="HO4' PE2"; $resn_exch{"1H3* PE2"}="H3'1 PE2"; $resn_exch{"2H3* PE2"}="H3'2 PE2";
$resn_exch{"*HO3 PE2"}="HO3' PE2"; $resn_exch{" H2* PE2"}=" H2' PE2"; $resn_exch{"1H12 PE2"}="H121 PE2";
$resn_exch{"2H12 PE2"}="H122 PE2"; $resn_exch{"1H1* PE2"}="H1'1 PE2"; $resn_exch{"2H1* PE2"}="H1'2 PE2";
$resn_exch{"1HN2 PE2"}="HN21 PE2"; $resn_exch{"2HN2 PE2"}="HN22 PE2"; $resn_exch{"1H42 PE3"}="H421 PE3";
$resn_exch{"2H42 PE3"}="H422 PE3"; $resn_exch{"1H41 PE3"}="H411 PE3"; $resn_exch{"2H41 PE3"}="H412 PE3";
$resn_exch{"1H39 PE3"}="H391 PE3"; $resn_exch{"2H39 PE3"}="H392 PE3"; $resn_exch{"1H38 PE3"}="H381 PE3";
$resn_exch{"2H38 PE3"}="H382 PE3"; $resn_exch{"1H36 PE3"}="H361 PE3"; $resn_exch{"2H36 PE3"}="H362 PE3";
$resn_exch{"1H35 PE3"}="H351 PE3"; $resn_exch{"2H35 PE3"}="H352 PE3"; $resn_exch{"1H33 PE3"}="H331 PE3";
$resn_exch{"2H33 PE3"}="H332 PE3"; $resn_exch{"1H32 PE3"}="H321 PE3"; $resn_exch{"2H32 PE3"}="H322 PE3";
$resn_exch{"1H30 PE3"}="H301 PE3"; $resn_exch{"2H30 PE3"}="H302 PE3"; $resn_exch{"1H29 PE3"}="H291 PE3";
$resn_exch{"2H29 PE3"}="H292 PE3"; $resn_exch{"1H27 PE3"}="H271 PE3"; $resn_exch{"2H27 PE3"}="H272 PE3";
$resn_exch{"1H26 PE3"}="H261 PE3"; $resn_exch{"2H26 PE3"}="H262 PE3"; $resn_exch{"1H24 PE3"}="H241 PE3";
$resn_exch{"2H24 PE3"}="H242 PE3"; $resn_exch{"1H23 PE3"}="H231 PE3"; $resn_exch{"2H23 PE3"}="H232 PE3";
$resn_exch{"1H21 PE3"}="H211 PE3"; $resn_exch{"2H21 PE3"}="H212 PE3"; $resn_exch{"1H20 PE3"}="H201 PE3";
$resn_exch{"2H20 PE3"}="H202 PE3"; $resn_exch{"1H18 PE3"}="H181 PE3"; $resn_exch{"2H18 PE3"}="H182 PE3";
$resn_exch{"1H17 PE3"}="H171 PE3"; $resn_exch{"2H17 PE3"}="H172 PE3"; $resn_exch{"1H15 PE3"}="H151 PE3";
$resn_exch{"2H15 PE3"}="H152 PE3"; $resn_exch{"1H14 PE3"}="H141 PE3"; $resn_exch{"2H14 PE3"}="H142 PE3";
$resn_exch{"1H12 PE3"}="H121 PE3"; $resn_exch{"2H12 PE3"}="H122 PE3"; $resn_exch{"1H11 PE3"}="H111 PE3";
$resn_exch{"2H11 PE3"}="H112 PE3"; $resn_exch{"1H9  PE3"}=" H91 PE3"; $resn_exch{"2H9  PE3"}=" H92 PE3";
$resn_exch{"1H8  PE3"}=" H81 PE3"; $resn_exch{"2H8  PE3"}=" H82 PE3"; $resn_exch{"1H6  PE3"}=" H61 PE3";
$resn_exch{"2H6  PE3"}=" H62 PE3"; $resn_exch{"1H5  PE3"}=" H51 PE3"; $resn_exch{"2H5  PE3"}=" H52 PE3";
$resn_exch{"1H3  PE3"}=" H31 PE3"; $resn_exch{"2H3  PE3"}=" H32 PE3"; $resn_exch{"1H2  PE3"}=" H21 PE3";
$resn_exch{"2H2  PE3"}=" H22 PE3"; $resn_exch{"1H1  PE4"}=" H11 PE4"; $resn_exch{"2H1  PE4"}=" H12 PE4";
$resn_exch{"1H2  PE4"}=" H21 PE4"; $resn_exch{"2H2  PE4"}=" H22 PE4"; $resn_exch{"1H3  PE4"}=" H31 PE4";
$resn_exch{"2H3  PE4"}=" H32 PE4"; $resn_exch{"1H4  PE4"}=" H41 PE4"; $resn_exch{"2H4  PE4"}=" H42 PE4";
$resn_exch{"1H5  PE4"}=" H51 PE4"; $resn_exch{"2H5  PE4"}=" H52 PE4"; $resn_exch{"1H6  PE4"}=" H61 PE4";
$resn_exch{"2H6  PE4"}=" H62 PE4"; $resn_exch{"1H7  PE4"}=" H71 PE4"; $resn_exch{"2H7  PE4"}=" H72 PE4";
$resn_exch{"1H8  PE4"}=" H81 PE4"; $resn_exch{"2H8  PE4"}=" H82 PE4"; $resn_exch{"1H9  PE4"}=" H91 PE4";
$resn_exch{"2H9  PE4"}=" H92 PE4"; $resn_exch{"1H10 PE4"}="H101 PE4"; $resn_exch{"2H10 PE4"}="H102 PE4";
$resn_exch{"1H11 PE4"}="H111 PE4"; $resn_exch{"2H11 PE4"}="H112 PE4"; $resn_exch{"1H12 PE4"}="H121 PE4";
$resn_exch{"2H12 PE4"}="H122 PE4"; $resn_exch{"1H13 PE4"}="H131 PE4"; $resn_exch{"2H13 PE4"}="H132 PE4";
$resn_exch{"1H14 PE4"}="H141 PE4"; $resn_exch{"2H14 PE4"}="H142 PE4"; $resn_exch{"1H15 PE4"}="H151 PE4";
$resn_exch{"2H15 PE4"}="H152 PE4"; $resn_exch{"1H16 PE4"}="H161 PE4"; $resn_exch{"2H16 PE4"}="H162 PE4";
$resn_exch{"3H16 PE4"}="H163 PE4"; $resn_exch{"1H48 PE5"}="H481 PE5"; $resn_exch{"2H48 PE5"}="H482 PE5";
$resn_exch{"3H48 PE5"}="H483 PE5"; $resn_exch{"1H50 PE5"}="H501 PE5"; $resn_exch{"2H50 PE5"}="H502 PE5";
$resn_exch{"1H1  PE5"}=" H11 PE5"; $resn_exch{"2H1  PE5"}=" H12 PE5"; $resn_exch{"1H2  PE5"}=" H21 PE5";
$resn_exch{"2H2  PE5"}=" H22 PE5"; $resn_exch{"1H3  PE5"}=" H31 PE5"; $resn_exch{"2H3  PE5"}=" H32 PE5";
$resn_exch{"1H4  PE5"}=" H41 PE5"; $resn_exch{"2H4  PE5"}=" H42 PE5"; $resn_exch{"1H5  PE5"}=" H51 PE5";
$resn_exch{"2H5  PE5"}="H52A PE5"; $resn_exch{"1H6  PE5"}=" H61 PE5"; $resn_exch{"2H6  PE5"}=" H62 PE5";
$resn_exch{"1H7  PE5"}=" H71 PE5"; $resn_exch{"2H7  PE5"}=" H72 PE5"; $resn_exch{"1H8  PE5"}=" H81 PE5";
$resn_exch{"2H8  PE5"}=" H82 PE5"; $resn_exch{"1H9  PE5"}=" H91 PE5"; $resn_exch{"2H9  PE5"}=" H92 PE5";
$resn_exch{"1H10 PE5"}="H101 PE5"; $resn_exch{"2H10 PE5"}="H102 PE5"; $resn_exch{"1H11 PE5"}="H111 PE5";
$resn_exch{"2H11 PE5"}="H112 PE5"; $resn_exch{"1H12 PE5"}="H121 PE5"; $resn_exch{"2H12 PE5"}="H122 PE5";
$resn_exch{"1H13 PE5"}="H131 PE5"; $resn_exch{"2H13 PE5"}="H132 PE5"; $resn_exch{"1H14 PE5"}="H141 PE5";
$resn_exch{"2H14 PE5"}="H142 PE5"; $resn_exch{"1H15 PE5"}="H151 PE5"; $resn_exch{"2H15 PE5"}="H152 PE5";
$resn_exch{"1H16 PE5"}="H161 PE5"; $resn_exch{"2H16 PE5"}="H162 PE5"; $resn_exch{"1H3  PE6"}=" H31 PE6";
$resn_exch{"2H3  PE6"}=" H32 PE6"; $resn_exch{"2HOP PE6"}="HOP2 PE6"; $resn_exch{"1H4  PE6"}=" H41 PE6";
$resn_exch{"2H4  PE6"}=" H42 PE6"; $resn_exch{"1H6  PE6"}=" H61 PE6"; $resn_exch{"2H6  PE6"}=" H62 PE6";
$resn_exch{"1H7  PE6"}=" H71 PE6"; $resn_exch{"2H7  PE6"}=" H72 PE6"; $resn_exch{"1H9  PE6"}=" H91 PE6";
$resn_exch{"2H9  PE6"}=" H92 PE6"; $resn_exch{"1H10 PE6"}="H101 PE6"; $resn_exch{"2H10 PE6"}="H102 PE6";
$resn_exch{"1H19 PE6"}="H191 PE6"; $resn_exch{"2H19 PE6"}="H192 PE6"; $resn_exch{"1H18 PE6"}="H181 PE6";
$resn_exch{"2H18 PE6"}="H182 PE6"; $resn_exch{"1H16 PE6"}="H161 PE6"; $resn_exch{"2H16 PE6"}="H162 PE6";
$resn_exch{"1H15 PE6"}="H151 PE6"; $resn_exch{"2H15 PE6"}="H152 PE6"; $resn_exch{"1H13 PE6"}="H131 PE6";
$resn_exch{"2H13 PE6"}="H132 PE6"; $resn_exch{"1H12 PE6"}="H121 PE6"; $resn_exch{"2H12 PE6"}="H122 PE6";
$resn_exch{"3HOP PE6"}="HOP3 PE6"; $resn_exch{"1H2  PE7"}=" H21 PE7"; $resn_exch{"2H2  PE7"}=" H22 PE7";
$resn_exch{"1H3  PE7"}=" H31 PE7"; $resn_exch{"2H3  PE7"}=" H32 PE7"; $resn_exch{"1H5  PE7"}=" H51 PE7";
$resn_exch{"2H5  PE7"}=" H52 PE7"; $resn_exch{"1H6  PE7"}=" H61 PE7"; $resn_exch{"2H6  PE7"}=" H62 PE7";
$resn_exch{"1H8  PE7"}=" H81 PE7"; $resn_exch{"2H8  PE7"}=" H82 PE7"; $resn_exch{"1H9  PE7"}=" H91 PE7";
$resn_exch{"2H9  PE7"}=" H92 PE7"; $resn_exch{"1H11 PE7"}="H111 PE7"; $resn_exch{"2H11 PE7"}="H112 PE7";
$resn_exch{"1H12 PE7"}="H121 PE7"; $resn_exch{"2H12 PE7"}="H122 PE7"; $resn_exch{"1H14 PE7"}="H141 PE7";
$resn_exch{"2H14 PE7"}="H142 PE7"; $resn_exch{"1H15 PE7"}="H151 PE7"; $resn_exch{"2H15 PE7"}="H152 PE7";
$resn_exch{"1H17 PE7"}="H171 PE7"; $resn_exch{"2H17 PE7"}="H172 PE7"; $resn_exch{"1H18 PE7"}="H181 PE7";
$resn_exch{"2H18 PE7"}="H182 PE7"; $resn_exch{"1H20 PE7"}="H201 PE7"; $resn_exch{"2H20 PE7"}="H202 PE7";
$resn_exch{"1H21 PE7"}="H211 PE7"; $resn_exch{"2H21 PE7"}="H212 PE7"; $resn_exch{"2HO  PE7"}=" HO2 PE7";
$resn_exch{"1H2  PE8"}=" H21 PE8"; $resn_exch{"2H2  PE8"}=" H22 PE8"; $resn_exch{"1H3  PE8"}=" H31 PE8";
$resn_exch{"2H3  PE8"}=" H32 PE8"; $resn_exch{"1H5  PE8"}=" H51 PE8"; $resn_exch{"2H5  PE8"}=" H52 PE8";
$resn_exch{"1H6  PE8"}=" H61 PE8"; $resn_exch{"2H6  PE8"}=" H62 PE8"; $resn_exch{"1H8  PE8"}=" H81 PE8";
$resn_exch{"2H8  PE8"}=" H82 PE8"; $resn_exch{"1H9  PE8"}=" H91 PE8"; $resn_exch{"2H9  PE8"}=" H92 PE8";
$resn_exch{"1H11 PE8"}="H111 PE8"; $resn_exch{"2H11 PE8"}="H112 PE8"; $resn_exch{"1H12 PE8"}="H121 PE8";
$resn_exch{"2H12 PE8"}="H122 PE8"; $resn_exch{"1H14 PE8"}="H141 PE8"; $resn_exch{"2H14 PE8"}="H142 PE8";
$resn_exch{"1H15 PE8"}="H151 PE8"; $resn_exch{"2H15 PE8"}="H152 PE8"; $resn_exch{"1H17 PE8"}="H171 PE8";
$resn_exch{"2H17 PE8"}="H172 PE8"; $resn_exch{"1H18 PE8"}="H181 PE8"; $resn_exch{"2H18 PE8"}="H182 PE8";
$resn_exch{"1H20 PE8"}="H201 PE8"; $resn_exch{"2H20 PE8"}="H202 PE8"; $resn_exch{"1H21 PE8"}="H211 PE8";
$resn_exch{"2H21 PE8"}="H212 PE8"; $resn_exch{"1H23 PE8"}="H231 PE8"; $resn_exch{"2H23 PE8"}="H232 PE8";
$resn_exch{"1H24 PE8"}="H241 PE8"; $resn_exch{"2H24 PE8"}="H242 PE8"; $resn_exch{"1HAA PE9"}="HAA1 PE9";
$resn_exch{"2HAA PE9"}="HAA2 PE9"; $resn_exch{"3HAA PE9"}="HAA3 PE9"; $resn_exch{"1HAC PE9"}="HAC1 PE9";
$resn_exch{"2HAC PE9"}="HAC2 PE9"; $resn_exch{"1HAE PE9"}="HAE1 PE9"; $resn_exch{"2HAE PE9"}="HAE2 PE9";
$resn_exch{"1HAF PE9"}="HAF1 PE9"; $resn_exch{"2HAF PE9"}="HAF2 PE9"; $resn_exch{"1HAD PE9"}="HAD1 PE9";
$resn_exch{"2HAD PE9"}="HAD2 PE9"; $resn_exch{"1H2  PEA"}=" H21 PEA"; $resn_exch{"2H2  PEA"}=" H22 PEA";
$resn_exch{"1H1  PEA"}=" H11 PEA"; $resn_exch{"2H1  PEA"}=" H12 PEA"; $resn_exch{"1HN  PEA"}=" HN1 PEA";
$resn_exch{"2HN  PEA"}=" HN2 PEA"; $resn_exch{"3HN  PEA"}=" HN3 PEA"; $resn_exch{"1HHC PEB"}="HHC1 PEB";
$resn_exch{"2HHC PEB"}="HHC2 PEB"; $resn_exch{"1HMC PEB"}="HMC1 PEB"; $resn_exch{"2HMC PEB"}="HMC2 PEB";
$resn_exch{"3HMC PEB"}="HMC3 PEB"; $resn_exch{"1HAC PEB"}="HAC1 PEB"; $resn_exch{"2HAC PEB"}="HAC2 PEB";
$resn_exch{"1HBC PEB"}="HBC1 PEB"; $resn_exch{"2HBC PEB"}="HBC2 PEB"; $resn_exch{"2HOC PEB"}="HOC2 PEB";
$resn_exch{"1H1D PEB"}="H1D1 PEB"; $resn_exch{"1HMD PEB"}="HMD1 PEB"; $resn_exch{"2HMD PEB"}="HMD2 PEB";
$resn_exch{"3HMD PEB"}="HMD3 PEB"; $resn_exch{"1HAD PEB"}="HAD1 PEB"; $resn_exch{"1HBD PEB"}="HBD1 PEB";
$resn_exch{"2HBD PEB"}="HBD2 PEB"; $resn_exch{"1H2A PEB"}="H2A1 PEB"; $resn_exch{"1H3A PEB"}="H3A1 PEB";
$resn_exch{"1HMA PEB"}="HMA1 PEB"; $resn_exch{"2HMA PEB"}="HMA2 PEB"; $resn_exch{"3HMA PEB"}="HMA3 PEB";
$resn_exch{"1HBA PEB"}="HBA1 PEB"; $resn_exch{"2HBA PEB"}="HBA2 PEB"; $resn_exch{"3HBA PEB"}="HBA3 PEB";
$resn_exch{"1HHA PEB"}="HHA1 PEB"; $resn_exch{"1HAA PEB"}="HAA1 PEB"; $resn_exch{"2HAA PEB"}="HAA2 PEB";
$resn_exch{"1HHB PEB"}="HHB1 PEB"; $resn_exch{"1HMB PEB"}="HMB1 PEB"; $resn_exch{"2HMB PEB"}="HMB2 PEB";
$resn_exch{"3HMB PEB"}="HMB3 PEB"; $resn_exch{"1HAB PEB"}="HAB1 PEB"; $resn_exch{"2HAB PEB"}="HAB2 PEB";
$resn_exch{"1HBB PEB"}="HBB1 PEB"; $resn_exch{"2HBB PEB"}="HBB2 PEB"; $resn_exch{"2HOB PEB"}="HOB2 PEB";
$resn_exch{"1HN  PEC"}=" H   PEC"; $resn_exch{"2HN  PEC"}=" H2  PEC"; $resn_exch{"1HB  PEC"}=" HB2 PEC";
$resn_exch{"2HB  PEC"}=" HB3 PEC"; $resn_exch{"1H1  PEC"}=" H11 PEC"; $resn_exch{"2H1  PEC"}=" H12 PEC";
$resn_exch{"1H2  PEC"}=" H21 PEC"; $resn_exch{"2H2  PEC"}=" H22 PEC"; $resn_exch{"1H3  PEC"}=" H31 PEC";
$resn_exch{"2H3  PEC"}=" H32 PEC"; $resn_exch{"1H4  PEC"}=" H41 PEC"; $resn_exch{"2H4  PEC"}=" H42 PEC";
$resn_exch{"1H5  PEC"}=" H51 PEC"; $resn_exch{"2H5  PEC"}=" H52 PEC"; $resn_exch{"3H5  PEC"}=" H53 PEC";
$resn_exch{"2HOP PED"}="HOP2 PED"; $resn_exch{"3HOP PED"}="HOP3 PED"; $resn_exch{"1H2* PED"}="H2'1 PED";
$resn_exch{"2H2* PED"}="H2'2 PED"; $resn_exch{"1H5* PED"}="H5'1 PED"; $resn_exch{"2H5* PED"}="H5'2 PED";
$resn_exch{"1H4* PED"}="H4'1 PED"; $resn_exch{"4HO* PED"}="HO'4 PED"; $resn_exch{" H3* PED"}=" H3' PED";
$resn_exch{"1H1* PED"}="H1'1 PED"; $resn_exch{"2H1* PED"}="H1'2 PED"; $resn_exch{"3H1* PED"}="H1'3 PED";
$resn_exch{"1H1  PEE"}=" H11 PEE"; $resn_exch{"2H1  PEE"}=" H12 PEE"; $resn_exch{"1HN  PEE"}=" HN1 PEE";
$resn_exch{"2HN  PEE"}=" HN2 PEE"; $resn_exch{"3HN  PEE"}=" HN3 PEE"; $resn_exch{"1H3  PEE"}=" H31 PEE";
$resn_exch{"2H3  PEE"}=" H32 PEE"; $resn_exch{"1H4  PEE"}=" H41 PEE"; $resn_exch{"2H4  PEE"}=" H42 PEE";
$resn_exch{"1H5  PEE"}=" H51 PEE"; $resn_exch{"2H5  PEE"}=" H52 PEE"; $resn_exch{"1H11 PEE"}="H111 PEE";
$resn_exch{"2H11 PEE"}="H112 PEE"; $resn_exch{"1H12 PEE"}="H121 PEE"; $resn_exch{"2H12 PEE"}="H122 PEE";
$resn_exch{"1H13 PEE"}="H131 PEE"; $resn_exch{"2H13 PEE"}="H132 PEE"; $resn_exch{"1H14 PEE"}="H141 PEE";
$resn_exch{"2H14 PEE"}="H142 PEE"; $resn_exch{"1H15 PEE"}="H151 PEE"; $resn_exch{"2H15 PEE"}="H152 PEE";
$resn_exch{"1H16 PEE"}="H161 PEE"; $resn_exch{"2H16 PEE"}="H162 PEE"; $resn_exch{"1H17 PEE"}="H171 PEE";
$resn_exch{"2H17 PEE"}="H172 PEE"; $resn_exch{"1H18 PEE"}="H181 PEE"; $resn_exch{"2H18 PEE"}="H182 PEE";
$resn_exch{"1H19 PEE"}="H191 PEE"; $resn_exch{"2H19 PEE"}="H192 PEE"; $resn_exch{"1H20 PEE"}="H201 PEE";
$resn_exch{"2H20 PEE"}="H202 PEE"; $resn_exch{"1H21 PEE"}="H211 PEE"; $resn_exch{"2H21 PEE"}="H212 PEE";
$resn_exch{"1H22 PEE"}="H221 PEE"; $resn_exch{"2H22 PEE"}="H222 PEE"; $resn_exch{"1H23 PEE"}="H231 PEE";
$resn_exch{"2H23 PEE"}="H232 PEE"; $resn_exch{"1H24 PEE"}="H241 PEE"; $resn_exch{"2H24 PEE"}="H242 PEE";
$resn_exch{"1H25 PEE"}="H251 PEE"; $resn_exch{"2H25 PEE"}="H252 PEE"; $resn_exch{"1H26 PEE"}="H261 PEE";
$resn_exch{"2H26 PEE"}="H262 PEE"; $resn_exch{"1H27 PEE"}="H271 PEE"; $resn_exch{"2H27 PEE"}="H272 PEE";
$resn_exch{"3H27 PEE"}="H273 PEE"; $resn_exch{"1H31 PEE"}="H311 PEE"; $resn_exch{"2H31 PEE"}="H312 PEE";
$resn_exch{"1H32 PEE"}="H321 PEE"; $resn_exch{"2H32 PEE"}="H322 PEE"; $resn_exch{"1H33 PEE"}="H331 PEE";
$resn_exch{"2H33 PEE"}="H332 PEE"; $resn_exch{"1H34 PEE"}="H341 PEE"; $resn_exch{"2H34 PEE"}="H342 PEE";
$resn_exch{"1H35 PEE"}="H351 PEE"; $resn_exch{"2H35 PEE"}="H352 PEE"; $resn_exch{"1H36 PEE"}="H361 PEE";
$resn_exch{"2H36 PEE"}="H362 PEE"; $resn_exch{"1H37 PEE"}="H371 PEE"; $resn_exch{"2H37 PEE"}="H372 PEE";
$resn_exch{"1H38 PEE"}="H381 PEE"; $resn_exch{"2H38 PEE"}="H382 PEE"; $resn_exch{"1H39 PEE"}="H391 PEE";
$resn_exch{"2H39 PEE"}="H392 PEE"; $resn_exch{"1H40 PEE"}="H401 PEE"; $resn_exch{"2H40 PEE"}="H402 PEE";
$resn_exch{"1H41 PEE"}="H411 PEE"; $resn_exch{"2H41 PEE"}="H412 PEE"; $resn_exch{"1H42 PEE"}="H421 PEE";
$resn_exch{"2H42 PEE"}="H422 PEE"; $resn_exch{"1H43 PEE"}="H431 PEE"; $resn_exch{"2H43 PEE"}="H432 PEE";
$resn_exch{"1H44 PEE"}="H441 PEE"; $resn_exch{"2H44 PEE"}="H442 PEE"; $resn_exch{"1H45 PEE"}="H451 PEE";
$resn_exch{"2H45 PEE"}="H452 PEE"; $resn_exch{"1H46 PEE"}="H461 PEE"; $resn_exch{"2H46 PEE"}="H462 PEE";
$resn_exch{"1H47 PEE"}="H471 PEE"; $resn_exch{"2H47 PEE"}="H472 PEE"; $resn_exch{"3H47 PEE"}="H473 PEE";
$resn_exch{"1H2  PEF"}=" H21 PEF"; $resn_exch{"1H1  PEF"}=" H11 PEF"; $resn_exch{"2H1  PEF"}=" H12 PEF";
$resn_exch{"1HN  PEF"}=" HN1 PEF"; $resn_exch{"2HN  PEF"}=" HN2 PEF"; $resn_exch{"1H3  PEF"}=" H31 PEF";
$resn_exch{"2H3  PEF"}=" H32 PEF"; $resn_exch{"1H4  PEF"}=" H41 PEF"; $resn_exch{"2H4  PEF"}=" H42 PEF";
$resn_exch{"1H5  PEF"}=" H51 PEF"; $resn_exch{"2H5  PEF"}=" H52 PEF"; $resn_exch{"2H11 PEF"}="H112 PEF";
$resn_exch{"1H11 PEF"}="H111 PEF"; $resn_exch{"2H12 PEF"}="H122 PEF"; $resn_exch{"1H12 PEF"}="H121 PEF";
$resn_exch{"2H13 PEF"}="H132 PEF"; $resn_exch{"1H13 PEF"}="H131 PEF"; $resn_exch{"2H14 PEF"}="H142 PEF";
$resn_exch{"1H14 PEF"}="H141 PEF"; $resn_exch{"2H15 PEF"}="H152 PEF"; $resn_exch{"1H15 PEF"}="H151 PEF";
$resn_exch{"2H16 PEF"}="H162 PEF"; $resn_exch{"1H16 PEF"}="H161 PEF"; $resn_exch{"2H17 PEF"}="H172 PEF";
$resn_exch{"1H17 PEF"}="H171 PEF"; $resn_exch{"2H18 PEF"}="H182 PEF"; $resn_exch{"1H18 PEF"}="H181 PEF";
$resn_exch{"2H19 PEF"}="H192 PEF"; $resn_exch{"1H19 PEF"}="H191 PEF"; $resn_exch{"2H20 PEF"}="H202 PEF";
$resn_exch{"1H20 PEF"}="H201 PEF"; $resn_exch{"2H21 PEF"}="H212 PEF"; $resn_exch{"1H21 PEF"}="H211 PEF";
$resn_exch{"2H22 PEF"}="H222 PEF"; $resn_exch{"1H22 PEF"}="H221 PEF"; $resn_exch{"2H23 PEF"}="H232 PEF";
$resn_exch{"1H23 PEF"}="H231 PEF"; $resn_exch{"2H24 PEF"}="H242 PEF"; $resn_exch{"1H24 PEF"}="H241 PEF";
$resn_exch{"2H25 PEF"}="H252 PEF"; $resn_exch{"1H25 PEF"}="H251 PEF"; $resn_exch{"3H25 PEF"}="H253 PEF";
$resn_exch{"2H31 PEF"}="H312 PEF"; $resn_exch{"1H31 PEF"}="H311 PEF"; $resn_exch{"2H32 PEF"}="H322 PEF";
$resn_exch{"1H32 PEF"}="H321 PEF"; $resn_exch{"2H33 PEF"}="H332 PEF"; $resn_exch{"1H33 PEF"}="H331 PEF";
$resn_exch{"2H34 PEF"}="H342 PEF"; $resn_exch{"1H34 PEF"}="H341 PEF"; $resn_exch{"2H35 PEF"}="H352 PEF";
$resn_exch{"1H35 PEF"}="H351 PEF"; $resn_exch{"2H36 PEF"}="H362 PEF"; $resn_exch{"1H36 PEF"}="H361 PEF";
$resn_exch{"2H37 PEF"}="H372 PEF"; $resn_exch{"1H37 PEF"}="H371 PEF"; $resn_exch{"2H38 PEF"}="H382 PEF";
$resn_exch{"1H38 PEF"}="H381 PEF"; $resn_exch{"2H39 PEF"}="H392 PEF"; $resn_exch{"1H39 PEF"}="H391 PEF";
$resn_exch{"2H40 PEF"}="H402 PEF"; $resn_exch{"1H40 PEF"}="H401 PEF"; $resn_exch{"2H41 PEF"}="H412 PEF";
$resn_exch{"1H41 PEF"}="H411 PEF"; $resn_exch{"2H42 PEF"}="H422 PEF"; $resn_exch{"1H42 PEF"}="H421 PEF";
$resn_exch{"2H43 PEF"}="H432 PEF"; $resn_exch{"1H43 PEF"}="H431 PEF"; $resn_exch{"2H44 PEF"}="H442 PEF";
$resn_exch{"1H44 PEF"}="H441 PEF"; $resn_exch{"2H45 PEF"}="H452 PEF"; $resn_exch{"1H45 PEF"}="H451 PEF";
$resn_exch{"3H45 PEF"}="H453 PEF"; $resn_exch{"2HOP PEF"}="HOP2 PEF"; $resn_exch{"1H1  PEG"}=" H11 PEG";
$resn_exch{"2H1  PEG"}=" H12 PEG"; $resn_exch{"1H2  PEG"}=" H21 PEG"; $resn_exch{"2H2  PEG"}=" H22 PEG";
$resn_exch{"1H3  PEG"}=" H31 PEG"; $resn_exch{"2H3  PEG"}=" H32 PEG"; $resn_exch{"1H4  PEG"}=" H41 PEG";
$resn_exch{"2H4  PEG"}=" H42 PEG"; $resn_exch{"1HN  PEH"}=" HN1 PEH"; $resn_exch{"2HN  PEH"}=" HN2 PEH";
$resn_exch{"1H11 PEH"}="H111 PEH"; $resn_exch{"2H11 PEH"}="H112 PEH"; $resn_exch{"1H12 PEH"}="H121 PEH";
$resn_exch{"2H12 PEH"}="H122 PEH"; $resn_exch{"1H1  PEH"}=" H11 PEH"; $resn_exch{"2H1  PEH"}=" H12 PEH";
$resn_exch{"1H3  PEH"}=" H31 PEH"; $resn_exch{"2H3  PEH"}=" H32 PEH"; $resn_exch{"1H32 PEH"}="H321 PEH";
$resn_exch{"2H32 PEH"}="H322 PEH"; $resn_exch{"1H33 PEH"}="H331 PEH"; $resn_exch{"2H33 PEH"}="H332 PEH";
$resn_exch{"1H34 PEH"}="H341 PEH"; $resn_exch{"2H34 PEH"}="H342 PEH"; $resn_exch{"1H35 PEH"}="H351 PEH";
$resn_exch{"2H35 PEH"}="H352 PEH"; $resn_exch{"1H36 PEH"}="H361 PEH"; $resn_exch{"2H36 PEH"}="H362 PEH";
$resn_exch{"1H37 PEH"}="H371 PEH"; $resn_exch{"2H37 PEH"}="H372 PEH"; $resn_exch{"1H38 PEH"}="H381 PEH";
$resn_exch{"2H38 PEH"}="H382 PEH"; $resn_exch{"1H39 PEH"}="H391 PEH"; $resn_exch{"2H39 PEH"}="H392 PEH";
$resn_exch{"1H3A PEH"}="H3A1 PEH"; $resn_exch{"2H3A PEH"}="H3A2 PEH"; $resn_exch{"1H3B PEH"}="H3B1 PEH";
$resn_exch{"2H3B PEH"}="H3B2 PEH"; $resn_exch{"1H3C PEH"}="H3C1 PEH"; $resn_exch{"2H3C PEH"}="H3C2 PEH";
$resn_exch{"1H3D PEH"}="H3D1 PEH"; $resn_exch{"2H3D PEH"}="H3D2 PEH"; $resn_exch{"1H3E PEH"}="H3E1 PEH";
$resn_exch{"2H3E PEH"}="H3E2 PEH"; $resn_exch{"1H3F PEH"}="H3F1 PEH"; $resn_exch{"2H3F PEH"}="H3F2 PEH";
$resn_exch{"1H3G PEH"}="H3G1 PEH"; $resn_exch{"2H3G PEH"}="H3G2 PEH"; $resn_exch{"1H3H PEH"}="H3H1 PEH";
$resn_exch{"2H3H PEH"}="H3H2 PEH"; $resn_exch{"1H3I PEH"}="H3I1 PEH"; $resn_exch{"2H3I PEH"}="H3I2 PEH";
$resn_exch{"3H3I PEH"}="H3I3 PEH"; $resn_exch{"1H22 PEH"}="H221 PEH"; $resn_exch{"2H22 PEH"}="H222 PEH";
$resn_exch{"1H23 PEH"}="H231 PEH"; $resn_exch{"2H23 PEH"}="H232 PEH"; $resn_exch{"1H24 PEH"}="H241 PEH";
$resn_exch{"2H24 PEH"}="H242 PEH"; $resn_exch{"1H25 PEH"}="H251 PEH"; $resn_exch{"2H25 PEH"}="H252 PEH";
$resn_exch{"1H26 PEH"}="H261 PEH"; $resn_exch{"2H26 PEH"}="H262 PEH"; $resn_exch{"1H27 PEH"}="H271 PEH";
$resn_exch{"2H27 PEH"}="H272 PEH"; $resn_exch{"1H28 PEH"}="H281 PEH"; $resn_exch{"2H28 PEH"}="H282 PEH";
$resn_exch{"1H29 PEH"}="H291 PEH"; $resn_exch{"2H29 PEH"}="H292 PEH"; $resn_exch{"1H2A PEH"}="H2A1 PEH";
$resn_exch{"2H2A PEH"}="H2A2 PEH"; $resn_exch{"1H2B PEH"}="H2B1 PEH"; $resn_exch{"2H2B PEH"}="H2B2 PEH";
$resn_exch{"1H2C PEH"}="H2C1 PEH"; $resn_exch{"2H2C PEH"}="H2C2 PEH"; $resn_exch{"1H2D PEH"}="H2D1 PEH";
$resn_exch{"2H2D PEH"}="H2D2 PEH"; $resn_exch{"1H2E PEH"}="H2E1 PEH"; $resn_exch{"2H2E PEH"}="H2E2 PEH";
$resn_exch{"1H2F PEH"}="H2F1 PEH"; $resn_exch{"2H2F PEH"}="H2F2 PEH"; $resn_exch{"1H2G PEH"}="H2G1 PEH";
$resn_exch{"2H2G PEH"}="H2G2 PEH"; $resn_exch{"1H2H PEH"}="H2H1 PEH"; $resn_exch{"2H2H PEH"}="H2H2 PEH";
$resn_exch{"1H2I PEH"}="H2I1 PEH"; $resn_exch{"2H2I PEH"}="H2I2 PEH"; $resn_exch{"3H2I PEH"}="H2I3 PEH";
$resn_exch{"1H1  PEI"}=" H11 PEI"; $resn_exch{"2H1  PEI"}=" H12 PEI"; $resn_exch{"3H1  PEI"}=" H13 PEI";
$resn_exch{"1H2  PEI"}=" H21 PEI"; $resn_exch{"2H2  PEI"}=" H22 PEI"; $resn_exch{"1H3  PEI"}=" H31 PEI";
$resn_exch{"2H3  PEI"}=" H32 PEI"; $resn_exch{"1H4  PEI"}=" H41 PEI"; $resn_exch{"2H4  PEI"}=" H42 PEI";
$resn_exch{"1HN  PEK"}=" HN1 PEK"; $resn_exch{"2HN  PEK"}=" HN2 PEK"; $resn_exch{"1H01 PEK"}="H011 PEK";
$resn_exch{"2H01 PEK"}="H012 PEK"; $resn_exch{"1H03 PEK"}="H031 PEK"; $resn_exch{"2H03 PEK"}="H032 PEK";
$resn_exch{"1H04 PEK"}="H041 PEK"; $resn_exch{"2H04 PEK"}="H042 PEK"; $resn_exch{"1H05 PEK"}="H051 PEK";
$resn_exch{"2H05 PEK"}="H052 PEK"; $resn_exch{"1H2  PEK"}=" H21 PEK"; $resn_exch{"2H2  PEK"}=" H22 PEK";
$resn_exch{"1H3  PEK"}=" H31 PEK"; $resn_exch{"2H3  PEK"}=" H32 PEK"; $resn_exch{"1H4  PEK"}=" H41 PEK";
$resn_exch{"2H4  PEK"}=" H42 PEK"; $resn_exch{"1H7  PEK"}=" H71 PEK"; $resn_exch{"2H7  PEK"}=" H72 PEK";
$resn_exch{"1H10 PEK"}="H101 PEK"; $resn_exch{"2H10 PEK"}="H102 PEK"; $resn_exch{"1H13 PEK"}="H131 PEK";
$resn_exch{"2H13 PEK"}="H132 PEK"; $resn_exch{"1H16 PEK"}="H161 PEK"; $resn_exch{"2H16 PEK"}="H162 PEK";
$resn_exch{"1H17 PEK"}="H171 PEK"; $resn_exch{"2H17 PEK"}="H172 PEK"; $resn_exch{"1H18 PEK"}="H181 PEK";
$resn_exch{"2H18 PEK"}="H182 PEK"; $resn_exch{"1H19 PEK"}="H191 PEK"; $resn_exch{"2H19 PEK"}="H192 PEK";
$resn_exch{"1H20 PEK"}="H201 PEK"; $resn_exch{"2H20 PEK"}="H202 PEK"; $resn_exch{"3H20 PEK"}="H203 PEK";
$resn_exch{"1H22 PEK"}="H221 PEK"; $resn_exch{"2H22 PEK"}="H222 PEK"; $resn_exch{"1H23 PEK"}="H231 PEK";
$resn_exch{"2H23 PEK"}="H232 PEK"; $resn_exch{"1H24 PEK"}="H241 PEK"; $resn_exch{"2H24 PEK"}="H242 PEK";
$resn_exch{"1H25 PEK"}="H251 PEK"; $resn_exch{"2H25 PEK"}="H252 PEK"; $resn_exch{"1H26 PEK"}="H261 PEK";
$resn_exch{"2H26 PEK"}="H262 PEK"; $resn_exch{"1H27 PEK"}="H271 PEK"; $resn_exch{"2H27 PEK"}="H272 PEK";
$resn_exch{"1H28 PEK"}="H281 PEK"; $resn_exch{"2H28 PEK"}="H282 PEK"; $resn_exch{"1H29 PEK"}="H291 PEK";
$resn_exch{"2H29 PEK"}="H292 PEK"; $resn_exch{"1H30 PEK"}="H301 PEK"; $resn_exch{"2H30 PEK"}="H302 PEK";
$resn_exch{"1H31 PEK"}="H311 PEK"; $resn_exch{"2H31 PEK"}="H312 PEK"; $resn_exch{"1H32 PEK"}="H321 PEK";
$resn_exch{"2H32 PEK"}="H322 PEK"; $resn_exch{"1H33 PEK"}="H331 PEK"; $resn_exch{"2H33 PEK"}="H332 PEK";
$resn_exch{"1H34 PEK"}="H341 PEK"; $resn_exch{"2H34 PEK"}="H342 PEK"; $resn_exch{"1H35 PEK"}="H351 PEK";
$resn_exch{"2H35 PEK"}="H352 PEK"; $resn_exch{"1H36 PEK"}="H361 PEK"; $resn_exch{"2H36 PEK"}="H362 PEK";
$resn_exch{"1H37 PEK"}="H371 PEK"; $resn_exch{"2H37 PEK"}="H372 PEK"; $resn_exch{"1H38 PEK"}="H381 PEK";
$resn_exch{"2H38 PEK"}="H382 PEK"; $resn_exch{"3H38 PEK"}="H383 PEK"; $resn_exch{"1H2  PEM"}=" H21 PEM";
$resn_exch{"1H3  PEM"}=" H31 PEM"; $resn_exch{"1H5  PEM"}=" H51 PEM"; $resn_exch{"1HC8 PEM"}="HC81 PEM";
$resn_exch{"2HC8 PEM"}="HC82 PEM"; $resn_exch{"1HC9 PEM"}="HC91 PEM"; $resn_exch{"2HC9 PEM"}="HC92 PEM";
$resn_exch{"1H17 PEM"}="H171 PEM"; $resn_exch{"2H17 PEM"}="H172 PEM"; $resn_exch{"3H17 PEM"}="H173 PEM";
$resn_exch{"1H18 PEM"}="H181 PEM"; $resn_exch{"2H18 PEM"}="H182 PEM"; $resn_exch{"3H18 PEM"}="H183 PEM";
$resn_exch{"1H3  PEP"}=" H31 PEP"; $resn_exch{"2H3  PEP"}=" H32 PEP"; $resn_exch{"2HOP PEP"}="HOP2 PEP";
$resn_exch{"3HOP PEP"}="HOP3 PEP"; $resn_exch{"1H2  PEQ"}=" H21 PEQ"; $resn_exch{"1H3  PEQ"}=" H31 PEQ";
$resn_exch{"2H3  PEQ"}=" H32 PEQ"; $resn_exch{"3H3  PEQ"}=" H33 PEQ"; $resn_exch{"2HOP PEQ"}="HOP2 PEQ";
$resn_exch{"3HOP PEQ"}="HOP3 PEQ"; $resn_exch{"1HA  PET"}=" HA1 PET"; $resn_exch{"2HA  PET"}=" HA2 PET";
$resn_exch{"1HB  PET"}=" HB1 PET"; $resn_exch{"2HB  PET"}=" HB2 PET"; $resn_exch{"1HA' PET"}="HA'1 PET";
$resn_exch{"2HA' PET"}="HA'2 PET"; $resn_exch{"1HB' PET"}="HB'1 PET"; $resn_exch{"2HB' PET"}="HB'2 PET";
$resn_exch{"1HN2 PET"}="HN21 PET"; $resn_exch{"2HN2 PET"}="HN22 PET"; $resn_exch{"'HN1 PET"}="HN1' PET";
$resn_exch{"1HN' PET"}="HN'1 PET"; $resn_exch{"2HN' PET"}="HN'2 PET"; $resn_exch{"1HAA PEU"}="HAA1 PEU";
$resn_exch{"2HAA PEU"}="HAA2 PEU"; $resn_exch{"3HAA PEU"}="HAA3 PEU"; $resn_exch{"1HAC PEU"}="HAC1 PEU";
$resn_exch{"2HAC PEU"}="HAC2 PEU"; $resn_exch{"1HAD PEU"}="HAD1 PEU"; $resn_exch{"2HAD PEU"}="HAD2 PEU";
$resn_exch{"1HAF PEU"}="HAF1 PEU"; $resn_exch{"2HAF PEU"}="HAF2 PEU"; $resn_exch{"1HAG PEU"}="HAG1 PEU";
$resn_exch{"2HAG PEU"}="HAG2 PEU"; $resn_exch{"1HAI PEU"}="HAI1 PEU"; $resn_exch{"2HAI PEU"}="HAI2 PEU";
$resn_exch{"1HAJ PEU"}="HAJ1 PEU"; $resn_exch{"2HAJ PEU"}="HAJ2 PEU"; $resn_exch{"1HAL PEU"}="HAL1 PEU";
$resn_exch{"2HAL PEU"}="HAL2 PEU"; $resn_exch{"1HAM PEU"}="HAM1 PEU"; $resn_exch{"2HAM PEU"}="HAM2 PEU";
$resn_exch{"1HAO PEU"}="HAO1 PEU"; $resn_exch{"2HAO PEU"}="HAO2 PEU"; $resn_exch{"1HAP PEU"}="HAP1 PEU";
$resn_exch{"2HAP PEU"}="HAP2 PEU"; $resn_exch{"1HAR PEU"}="HAR1 PEU"; $resn_exch{"2HAR PEU"}="HAR2 PEU";
$resn_exch{"1HAS PEU"}="HAS1 PEU"; $resn_exch{"2HAS PEU"}="HAS2 PEU"; $resn_exch{"1HAU PEU"}="HAU1 PEU";
$resn_exch{"2HAU PEU"}="HAU2 PEU"; $resn_exch{"1HAV PEU"}="HAV1 PEU"; $resn_exch{"2HAV PEU"}="HAV2 PEU";
$resn_exch{"1HAX PEU"}="HAX1 PEU"; $resn_exch{"2HAX PEU"}="HAX2 PEU"; $resn_exch{"1HAY PEU"}="HAY1 PEU";
$resn_exch{"2HAY PEU"}="HAY2 PEU"; $resn_exch{"1HBA PEU"}="HBA1 PEU"; $resn_exch{"2HBA PEU"}="HBA2 PEU";
$resn_exch{"1HBB PEU"}="HBB1 PEU"; $resn_exch{"2HBB PEU"}="HBB2 PEU"; $resn_exch{"1HCG PEU"}="HCG1 PEU";
$resn_exch{"2HCG PEU"}="HCG2 PEU"; $resn_exch{"1HCF PEU"}="HCF1 PEU"; $resn_exch{"2HCF PEU"}="HCF2 PEU";
$resn_exch{"1HCD PEU"}="HCD1 PEU"; $resn_exch{"2HCD PEU"}="HCD2 PEU"; $resn_exch{"1HCC PEU"}="HCC1 PEU";
$resn_exch{"2HCC PEU"}="HCC2 PEU"; $resn_exch{"1HCA PEU"}="HCA1 PEU"; $resn_exch{"2HCA PEU"}="HCA2 PEU";
$resn_exch{"1HBZ PEU"}="HBZ1 PEU"; $resn_exch{"2HBZ PEU"}="HBZ2 PEU"; $resn_exch{"1HBX PEU"}="HBX1 PEU";
$resn_exch{"2HBX PEU"}="HBX2 PEU"; $resn_exch{"1HBW PEU"}="HBW1 PEU"; $resn_exch{"2HBW PEU"}="HBW2 PEU";
$resn_exch{"1HBU PEU"}="HBU1 PEU"; $resn_exch{"2HBU PEU"}="HBU2 PEU"; $resn_exch{"1HBT PEU"}="HBT1 PEU";
$resn_exch{"2HBT PEU"}="HBT2 PEU"; $resn_exch{"1HBR PEU"}="HBR1 PEU"; $resn_exch{"2HBR PEU"}="HBR2 PEU";
$resn_exch{"1HBQ PEU"}="HBQ1 PEU"; $resn_exch{"2HBQ PEU"}="HBQ2 PEU"; $resn_exch{"1HBO PEU"}="HBO1 PEU";
$resn_exch{"2HBO PEU"}="HBO2 PEU"; $resn_exch{"1HBN PEU"}="HBN1 PEU"; $resn_exch{"2HBN PEU"}="HBN2 PEU";
$resn_exch{"1HBL PEU"}="HBL1 PEU"; $resn_exch{"2HBL PEU"}="HBL2 PEU"; $resn_exch{"1HBK PEU"}="HBK1 PEU";
$resn_exch{"2HBK PEU"}="HBK2 PEU"; $resn_exch{"1HBI PEU"}="HBI1 PEU"; $resn_exch{"2HBI PEU"}="HBI2 PEU";
$resn_exch{"1HBH PEU"}="HBH1 PEU"; $resn_exch{"2HBH PEU"}="HBH2 PEU"; $resn_exch{"1HBF PEU"}="HBF1 PEU";
$resn_exch{"2HBF PEU"}="HBF2 PEU"; $resn_exch{"1HBE PEU"}="HBE1 PEU"; $resn_exch{"2HBE PEU"}="HBE2 PEU";
$resn_exch{"1HCH PEU"}="HCH1 PEU"; $resn_exch{"2HCH PEU"}="HCH2 PEU"; $resn_exch{"1HCI PEU"}="HCI1 PEU";
$resn_exch{"2HCI PEU"}="HCI2 PEU"; $resn_exch{"1HCK PEU"}="HCK1 PEU"; $resn_exch{"2HCK PEU"}="HCK2 PEU";
$resn_exch{"1HCL PEU"}="HCL1 PEU"; $resn_exch{"2HCL PEU"}="HCL2 PEU"; $resn_exch{"1HCN PEU"}="HCN1 PEU";
$resn_exch{"2HCN PEU"}="HCN2 PEU"; $resn_exch{"1HCO PEU"}="HCO1 PEU"; $resn_exch{"2HCO PEU"}="HCO2 PEU";
$resn_exch{"1HCQ PEU"}="HCQ1 PEU"; $resn_exch{"2HCQ PEU"}="HCQ2 PEU"; $resn_exch{"1HCR PEU"}="HCR1 PEU";
$resn_exch{"2HCR PEU"}="HCR2 PEU"; $resn_exch{"1HCT PEU"}="HCT1 PEU"; $resn_exch{"2HCT PEU"}="HCT2 PEU";
$resn_exch{"1HCU PEU"}="HCU1 PEU"; $resn_exch{"2HCU PEU"}="HCU2 PEU"; $resn_exch{"1HCW PEU"}="HCW1 PEU";
$resn_exch{"2HCW PEU"}="HCW2 PEU"; $resn_exch{"1HCX PEU"}="HCX1 PEU"; $resn_exch{"2HCX PEU"}="HCX2 PEU";
$resn_exch{"1HCZ PEU"}="HCZ1 PEU"; $resn_exch{"2HCZ PEU"}="HCZ2 PEU"; $resn_exch{"1HDA PEU"}="HDA1 PEU";
$resn_exch{"2HDA PEU"}="HDA2 PEU"; $resn_exch{"1HDC PEU"}="HDC1 PEU"; $resn_exch{"2HDC PEU"}="HDC2 PEU";
$resn_exch{"1HDD PEU"}="HDD1 PEU"; $resn_exch{"2HDD PEU"}="HDD2 PEU"; $resn_exch{"1H48 PEV"}="H481 PEV";
$resn_exch{"2H48 PEV"}="H482 PEV"; $resn_exch{"3H48 PEV"}="H483 PEV"; $resn_exch{"1H47 PEV"}="H471 PEV";
$resn_exch{"2H47 PEV"}="H472 PEV"; $resn_exch{"1H46 PEV"}="H461 PEV"; $resn_exch{"2H46 PEV"}="H462 PEV";
$resn_exch{"1H45 PEV"}="H451 PEV"; $resn_exch{"2H45 PEV"}="H452 PEV"; $resn_exch{"1H44 PEV"}="H441 PEV";
$resn_exch{"2H44 PEV"}="H442 PEV"; $resn_exch{"1H43 PEV"}="H431 PEV"; $resn_exch{"2H43 PEV"}="H432 PEV";
$resn_exch{"1H42 PEV"}="H421 PEV"; $resn_exch{"2H42 PEV"}="H422 PEV"; $resn_exch{"1H41 PEV"}="H411 PEV";
$resn_exch{"2H41 PEV"}="H412 PEV"; $resn_exch{"1H40 PEV"}="H401 PEV"; $resn_exch{"2H40 PEV"}="H402 PEV";
$resn_exch{"1H39 PEV"}="H391 PEV"; $resn_exch{"2H39 PEV"}="H392 PEV"; $resn_exch{"1H38 PEV"}="H381 PEV";
$resn_exch{"2H38 PEV"}="H382 PEV"; $resn_exch{"1H37 PEV"}="H371 PEV"; $resn_exch{"2H37 PEV"}="H372 PEV";
$resn_exch{"1H36 PEV"}="H361 PEV"; $resn_exch{"2H36 PEV"}="H362 PEV"; $resn_exch{"1H35 PEV"}="H351 PEV";
$resn_exch{"2H35 PEV"}="H352 PEV"; $resn_exch{"1H34 PEV"}="H341 PEV"; $resn_exch{"2H34 PEV"}="H342 PEV";
$resn_exch{"1H33 PEV"}="H331 PEV"; $resn_exch{"2H33 PEV"}="H332 PEV"; $resn_exch{"1H32 PEV"}="H321 PEV";
$resn_exch{"2H32 PEV"}="H322 PEV"; $resn_exch{"1H1  PEV"}=" H11 PEV"; $resn_exch{"2H1  PEV"}=" H12 PEV";
$resn_exch{"1H4  PEV"}=" H41 PEV"; $resn_exch{"2H4  PEV"}=" H42 PEV"; $resn_exch{"1H5  PEV"}=" H51 PEV";
$resn_exch{"2H5  PEV"}=" H52 PEV"; $resn_exch{"1HN6 PEV"}="HN61 PEV"; $resn_exch{"2HN6 PEV"}="HN62 PEV";
$resn_exch{"1H3  PEV"}=" H31 PEV"; $resn_exch{"2H3  PEV"}=" H32 PEV"; $resn_exch{"1H12 PEV"}="H121 PEV";
$resn_exch{"2H12 PEV"}="H122 PEV"; $resn_exch{"1H13 PEV"}="H131 PEV"; $resn_exch{"2H13 PEV"}="H132 PEV";
$resn_exch{"1H14 PEV"}="H141 PEV"; $resn_exch{"2H14 PEV"}="H142 PEV"; $resn_exch{"1H15 PEV"}="H151 PEV";
$resn_exch{"2H15 PEV"}="H152 PEV"; $resn_exch{"1H16 PEV"}="H161 PEV"; $resn_exch{"2H16 PEV"}="H162 PEV";
$resn_exch{"1H17 PEV"}="H171 PEV"; $resn_exch{"2H17 PEV"}="H172 PEV"; $resn_exch{"1H18 PEV"}="H181 PEV";
$resn_exch{"2H18 PEV"}="H182 PEV"; $resn_exch{"1H19 PEV"}="H191 PEV"; $resn_exch{"2H19 PEV"}="H192 PEV";
$resn_exch{"1H20 PEV"}="H201 PEV"; $resn_exch{"2H20 PEV"}="H202 PEV"; $resn_exch{"1H21 PEV"}="H211 PEV";
$resn_exch{"2H21 PEV"}="H212 PEV"; $resn_exch{"1H22 PEV"}="H221 PEV"; $resn_exch{"2H22 PEV"}="H222 PEV";
$resn_exch{"1H23 PEV"}="H231 PEV"; $resn_exch{"2H23 PEV"}="H232 PEV"; $resn_exch{"1H24 PEV"}="H241 PEV";
$resn_exch{"2H24 PEV"}="H242 PEV"; $resn_exch{"1H25 PEV"}="H251 PEV"; $resn_exch{"2H25 PEV"}="H252 PEV";
$resn_exch{"1H26 PEV"}="H261 PEV"; $resn_exch{"2H26 PEV"}="H262 PEV"; $resn_exch{"3H26 PEV"}="H263 PEV";
$resn_exch{"1H48 PEW"}="H481 PEW"; $resn_exch{"2H48 PEW"}="H482 PEW"; $resn_exch{"3H48 PEW"}="H483 PEW";
$resn_exch{"1H47 PEW"}="H471 PEW"; $resn_exch{"2H47 PEW"}="H472 PEW"; $resn_exch{"1H46 PEW"}="H461 PEW";
$resn_exch{"2H46 PEW"}="H462 PEW"; $resn_exch{"1H45 PEW"}="H451 PEW"; $resn_exch{"2H45 PEW"}="H452 PEW";
$resn_exch{"1H44 PEW"}="H441 PEW"; $resn_exch{"2H44 PEW"}="H442 PEW"; $resn_exch{"1H43 PEW"}="H431 PEW";
$resn_exch{"2H43 PEW"}="H432 PEW"; $resn_exch{"1H42 PEW"}="H421 PEW"; $resn_exch{"2H42 PEW"}="H422 PEW";
$resn_exch{"1H41 PEW"}="H411 PEW"; $resn_exch{"2H41 PEW"}="H412 PEW"; $resn_exch{"1H38 PEW"}="H381 PEW";
$resn_exch{"2H38 PEW"}="H382 PEW"; $resn_exch{"1H37 PEW"}="H371 PEW"; $resn_exch{"2H37 PEW"}="H372 PEW";
$resn_exch{"1H36 PEW"}="H361 PEW"; $resn_exch{"2H36 PEW"}="H362 PEW"; $resn_exch{"1H35 PEW"}="H351 PEW";
$resn_exch{"2H35 PEW"}="H352 PEW"; $resn_exch{"1H34 PEW"}="H341 PEW"; $resn_exch{"2H34 PEW"}="H342 PEW";
$resn_exch{"1H33 PEW"}="H331 PEW"; $resn_exch{"2H33 PEW"}="H332 PEW"; $resn_exch{"1H32 PEW"}="H321 PEW";
$resn_exch{"2H32 PEW"}="H322 PEW"; $resn_exch{"1H1  PEW"}=" H11 PEW"; $resn_exch{"2H1  PEW"}=" H12 PEW";
$resn_exch{"1H4  PEW"}=" H41 PEW"; $resn_exch{"2H4  PEW"}=" H42 PEW"; $resn_exch{"1H5  PEW"}=" H51 PEW";
$resn_exch{"2H5  PEW"}=" H52 PEW"; $resn_exch{"1HN6 PEW"}="HN61 PEW"; $resn_exch{"2HN6 PEW"}="HN62 PEW";
$resn_exch{"1H3  PEW"}=" H31 PEW"; $resn_exch{"2H3  PEW"}=" H32 PEW"; $resn_exch{"1H12 PEW"}="H121 PEW";
$resn_exch{"2H12 PEW"}="H122 PEW"; $resn_exch{"1H13 PEW"}="H131 PEW"; $resn_exch{"2H13 PEW"}="H132 PEW";
$resn_exch{"1H14 PEW"}="H141 PEW"; $resn_exch{"2H14 PEW"}="H142 PEW"; $resn_exch{"1H15 PEW"}="H151 PEW";
$resn_exch{"2H15 PEW"}="H152 PEW"; $resn_exch{"1H16 PEW"}="H161 PEW"; $resn_exch{"2H16 PEW"}="H162 PEW";
$resn_exch{"1H17 PEW"}="H171 PEW"; $resn_exch{"2H17 PEW"}="H172 PEW"; $resn_exch{"1H18 PEW"}="H181 PEW";
$resn_exch{"2H18 PEW"}="H182 PEW"; $resn_exch{"1H19 PEW"}="H191 PEW"; $resn_exch{"2H19 PEW"}="H192 PEW";
$resn_exch{"1H20 PEW"}="H201 PEW"; $resn_exch{"2H20 PEW"}="H202 PEW"; $resn_exch{"1H21 PEW"}="H211 PEW";
$resn_exch{"2H21 PEW"}="H212 PEW"; $resn_exch{"1H22 PEW"}="H221 PEW"; $resn_exch{"2H22 PEW"}="H222 PEW";
$resn_exch{"1H23 PEW"}="H231 PEW"; $resn_exch{"2H23 PEW"}="H232 PEW"; $resn_exch{"1H24 PEW"}="H241 PEW";
$resn_exch{"2H24 PEW"}="H242 PEW"; $resn_exch{"1H25 PEW"}="H251 PEW"; $resn_exch{"2H25 PEW"}="H252 PEW";
$resn_exch{"1H26 PEW"}="H261 PEW"; $resn_exch{"2H26 PEW"}="H262 PEW"; $resn_exch{"3H26 PEW"}="H263 PEW";
$resn_exch{"1H3  PEZ"}=" H31 PEZ"; $resn_exch{"2H3  PEZ"}=" H32 PEZ"; $resn_exch{"1H4  PEZ"}=" H41 PEZ";
$resn_exch{"2H4  PEZ"}=" H42 PEZ"; $resn_exch{"3H4  PEZ"}=" H43 PEZ"; $resn_exch{"1H15  PF"}="H151  PF";
$resn_exch{"2H15  PF"}="H152  PF"; $resn_exch{"1H16  PF"}="H161  PF"; $resn_exch{"2H16  PF"}="H162  PF";
$resn_exch{"1HAG PF1"}="HAG1 PF1"; $resn_exch{"2HAG PF1"}="HAG2 PF1"; $resn_exch{"1HAH PF1"}="HAH1 PF1";
$resn_exch{"2HAH PF1"}="HAH2 PF1"; $resn_exch{"1HAI PF1"}="HAI1 PF1"; $resn_exch{"2HAI PF1"}="HAI2 PF1";
$resn_exch{"1HAA PF1"}="HAA1 PF1"; $resn_exch{"2HAA PF1"}="HAA2 PF1"; $resn_exch{"3HAA PF1"}="HAA3 PF1";
$resn_exch{"1HAB PF1"}="HAB1 PF1"; $resn_exch{"2HAB PF1"}="HAB2 PF1"; $resn_exch{"1H18 PF3"}="H181 PF3";
$resn_exch{"2H18 PF3"}="H182 PF3"; $resn_exch{"1HN  PF5"}=" H   PF5"; $resn_exch{"2HN  PF5"}=" H2  PF5";
$resn_exch{"1HB  PF5"}=" HB2 PF5"; $resn_exch{"2HB  PF5"}=" HB3 PF5"; $resn_exch{"HOXT PF5"}=" HXT PF5";
$resn_exch{"CH17 PFA"}="H17C PFA"; $resn_exch{"1HC7 PFA"}="HC71 PFA"; $resn_exch{"2HC7 PFA"}="HC72 PFA";
$resn_exch{"3HC7 PFA"}="HC73 PFA"; $resn_exch{"1H19 PFA"}="H191 PFA"; $resn_exch{"2H19 PFA"}="H192 PFA";
$resn_exch{"3H19 PFA"}="H193 PFA"; $resn_exch{"CH31 PFA"}="H31C PFA"; $resn_exch{"CH22 PFA"}="H22C PFA";
$resn_exch{"CH25 PFA"}="H25C PFA"; $resn_exch{"1HC8 PFA"}="HC81 PFA"; $resn_exch{"2HC8 PFA"}="HC82 PFA";
$resn_exch{"3HC8 PFA"}="HC83 PFA"; $resn_exch{"1HC9 PFA"}="HC91 PFA"; $resn_exch{"2HC9 PFA"}="HC92 PFA";
$resn_exch{"3HC9 PFA"}="HC93 PFA"; $resn_exch{"CH29 PFA"}="H29C PFA"; $resn_exch{"OH27 PFA"}="H27O PFA";
$resn_exch{"1H7  PFB"}=" H71 PFB"; $resn_exch{"2H7  PFB"}=" H72 PFB"; $resn_exch{"2H14 PFC"}="H142 PFC";
$resn_exch{"1H14 PFC"}="H141 PFC"; $resn_exch{"2H17 PFC"}="H172 PFC"; $resn_exch{"1H17 PFC"}="H171 PFC";
$resn_exch{"2H18 PFC"}="H182 PFC"; $resn_exch{"1H18 PFC"}="H181 PFC"; $resn_exch{"2H21 PFC"}="H212 PFC";
$resn_exch{"1H21 PFC"}="H211 PFC"; $resn_exch{"2H24 PFC"}="H242 PFC"; $resn_exch{"1H24 PFC"}="H241 PFC";
$resn_exch{"2H25 PFC"}="H252 PFC"; $resn_exch{"1H25 PFC"}="H251 PFC"; $resn_exch{"2H27 PFC"}="H272 PFC";
$resn_exch{"1H27 PFC"}="H271 PFC"; $resn_exch{"2H29 PFC"}="H292 PFC"; $resn_exch{"1H29 PFC"}="H291 PFC";
$resn_exch{"2H30 PFC"}="H302 PFC"; $resn_exch{"1H30 PFC"}="H301 PFC"; $resn_exch{"2H32 PFC"}="H322 PFC";
$resn_exch{"1H32 PFC"}="H321 PFC"; $resn_exch{"2H34 PFC"}="H342 PFC"; $resn_exch{"1H34 PFC"}="H341 PFC";
$resn_exch{"3H35 PFC"}="H353 PFC"; $resn_exch{"2H35 PFC"}="H352 PFC"; $resn_exch{"1H35 PFC"}="H351 PFC";
$resn_exch{"3H36 PFC"}="H363 PFC"; $resn_exch{"2H36 PFC"}="H362 PFC"; $resn_exch{"1H36 PFC"}="H361 PFC";
$resn_exch{"3H37 PFC"}="H373 PFC"; $resn_exch{"2H37 PFC"}="H372 PFC"; $resn_exch{"1H37 PFC"}="H371 PFC";
$resn_exch{"2H38 PFC"}="H382 PFC"; $resn_exch{"1H38 PFC"}="H381 PFC"; $resn_exch{"1H5  PFD"}=" H51 PFD";
$resn_exch{"2H5  PFD"}=" H52 PFD"; $resn_exch{"1H11 PFD"}="H111 PFD"; $resn_exch{"2H11 PFD"}="H112 PFD";
$resn_exch{"1H19 PFD"}="H191 PFD"; $resn_exch{"2H19 PFD"}="H192 PFD"; $resn_exch{"1H21 PFD"}="H211 PFD";
$resn_exch{"2H21 PFD"}="H212 PFD"; $resn_exch{"3H21 PFD"}="H213 PFD"; $resn_exch{"1H47 PFE"}="H471 PFE";
$resn_exch{"2H47 PFE"}="H472 PFE"; $resn_exch{"1H43 PFE"}="H431 PFE"; $resn_exch{"1H35 PFE"}="H351 PFE";
$resn_exch{"1H33 PFE"}="H331 PFE"; $resn_exch{"1H37 PFE"}="H371 PFE"; $resn_exch{"1H34 PFE"}="H341 PFE";
$resn_exch{"1H3  PFE"}=" H31 PFE"; $resn_exch{"1H7  PFE"}=" H71 PFE"; $resn_exch{"1H9  PFE"}=" H91 PFE";
$resn_exch{"1H20 PFE"}="H201 PFE"; $resn_exch{"2H20 PFE"}="H202 PFE"; $resn_exch{"1H21 PFE"}="H211 PFE";
$resn_exch{"2H21 PFE"}="H212 PFE"; $resn_exch{"3H21 PFE"}="H213 PFE"; $resn_exch{"1H18 PFE"}="H181 PFE";
$resn_exch{"1H19 PFE"}="H191 PFE"; $resn_exch{"2H19 PFE"}="H192 PFE"; $resn_exch{"1H26 PFE"}="H261 PFE";
$resn_exch{"2H26 PFE"}="H262 PFE"; $resn_exch{"3H26 PFE"}="H263 PFE"; $resn_exch{"1HN  PFF"}=" H   PFF";
$resn_exch{"2HN  PFF"}=" H2  PFF"; $resn_exch{"1HBC PFF"}=" HB2 PFF"; $resn_exch{"2HBC PFF"}=" HB3 PFF";
$resn_exch{"1HN2 PFG"}="HN21 PFG"; $resn_exch{"2HN2 PFG"}="HN22 PFG"; $resn_exch{"1H9  PFG"}=" H91 PFG";
$resn_exch{"2H9  PFG"}=" H92 PFG"; $resn_exch{"1HP1 PFG"}="HP11 PFG"; $resn_exch{"2HP1 PFG"}="HP12 PFG";
$resn_exch{"1HNP PFG"}="HNP1 PFG"; $resn_exch{"1HB1 PFG"}="HB11 PFG"; $resn_exch{"2HB1 PFG"}="HB12 PFG";
$resn_exch{"1HG1 PFG"}="HG11 PFG"; $resn_exch{"2HG1 PFG"}="HG12 PFG"; $resn_exch{"1HO2 PFG"}="HO21 PFG";
$resn_exch{"2HNP PFG"}="HNP2 PFG"; $resn_exch{"1HB2 PFG"}="HB21 PFG"; $resn_exch{"2HB2 PFG"}="HB22 PFG";
$resn_exch{"1HG2 PFG"}="HG21 PFG"; $resn_exch{"2HG2 PFG"}="HG22 PFG"; $resn_exch{"2HO2 PFG"}="HO22 PFG";
$resn_exch{"3HNP PFG"}="HNP3 PFG"; $resn_exch{"1HB3 PFG"}="HB31 PFG"; $resn_exch{"2HB3 PFG"}="HB32 PFG";
$resn_exch{"1HG3 PFG"}="HG31 PFG"; $resn_exch{"2HG3 PFG"}="HG32 PFG"; $resn_exch{"3HO2 PFG"}="HO23 PFG";
$resn_exch{"4HNP PFG"}="HNP4 PFG"; $resn_exch{"1HB4 PFG"}="HB41 PFG"; $resn_exch{"2HB4 PFG"}="HB42 PFG";
$resn_exch{"1HG4 PFG"}="HG41 PFG"; $resn_exch{"2HG4 PFG"}="HG42 PFG"; $resn_exch{"4HO2 PFG"}="HO24 PFG";
$resn_exch{"1H18 PFH"}="H181 PFH"; $resn_exch{"2H18 PFH"}="H182 PFH"; $resn_exch{"1H16 PFH"}="H161 PFH";
$resn_exch{"2H16 PFH"}="H162 PFH"; $resn_exch{"1H19 PFH"}="H191 PFH"; $resn_exch{"2H19 PFH"}="H192 PFH";
$resn_exch{"1H14 PFH"}="H141 PFH"; $resn_exch{"2H14 PFH"}="H142 PFH"; $resn_exch{"3H14 PFH"}="H143 PFH";
$resn_exch{"1H12 PFH"}="H121 PFH"; $resn_exch{"2H12 PFH"}="H122 PFH"; $resn_exch{"3H12 PFH"}="H123 PFH";
$resn_exch{"1H2  PFI"}="H21A PFI"; $resn_exch{"2H2  PFI"}=" H22 PFI"; $resn_exch{"3H2  PFI"}=" H23 PFI";
$resn_exch{"1H1  PFI"}=" H11 PFI"; $resn_exch{"2H1  PFI"}=" H12 PFI"; $resn_exch{"3H1  PFI"}=" H13 PFI";
$resn_exch{"1H12 PFI"}="H121 PFI"; $resn_exch{"2H12 PFI"}="H122 PFI"; $resn_exch{"1H11 PFI"}="H111 PFI";
$resn_exch{"2H11 PFI"}="H112 PFI"; $resn_exch{"1H6  PFI"}=" H61 PFI"; $resn_exch{"2H6  PFI"}=" H62 PFI";
$resn_exch{"1H13 PFI"}="H131 PFI"; $resn_exch{"2H13 PFI"}="H132 PFI"; $resn_exch{"1H16 PFI"}="H161 PFI";
$resn_exch{"2H16 PFI"}="H162 PFI"; $resn_exch{"1H15 PFI"}="H151 PFI"; $resn_exch{"2H15 PFI"}="H152 PFI";
$resn_exch{"1H14 PFI"}="H141 PFI"; $resn_exch{"2H14 PFI"}="H142 PFI"; $resn_exch{"1H18 PFJ"}="H181 PFJ";
$resn_exch{"2H18 PFJ"}="H182 PFJ"; $resn_exch{"1H16 PFJ"}="H161 PFJ"; $resn_exch{"2H16 PFJ"}="H162 PFJ";
$resn_exch{"1H19 PFJ"}="H191 PFJ"; $resn_exch{"2H19 PFJ"}="H192 PFJ"; $resn_exch{"1H14 PFJ"}="H141 PFJ";
$resn_exch{"2H14 PFJ"}="H142 PFJ"; $resn_exch{"3H14 PFJ"}="H143 PFJ"; $resn_exch{"1H12 PFJ"}="H121 PFJ";
$resn_exch{"2H12 PFJ"}="H122 PFJ"; $resn_exch{"3H12 PFJ"}="H123 PFJ"; $resn_exch{"1H8  PFL"}=" H81 PFL";
$resn_exch{"2H8  PFL"}=" H82 PFL"; $resn_exch{"3H8  PFL"}=" H83 PFL"; $resn_exch{"1H9  PFL"}=" H91 PFL";
$resn_exch{"2H9  PFL"}=" H92 PFL"; $resn_exch{"3H9  PFL"}=" H93 PFL"; $resn_exch{"1H11 PFL"}="H111 PFL";
$resn_exch{"2H11 PFL"}="H112 PFL"; $resn_exch{"3H11 PFL"}="H113 PFL"; $resn_exch{"1H12 PFL"}="H121 PFL";
$resn_exch{"2H12 PFL"}="H122 PFL"; $resn_exch{"3H12 PFL"}="H123 PFL"; $resn_exch{"1HN  PFM"}=" HN1 PFM";
$resn_exch{"2HN  PFM"}=" HN2 PFM"; $resn_exch{"1HB  PFM"}=" HB1 PFM"; $resn_exch{"2HB  PFM"}=" HB2 PFM";
$resn_exch{"1H2A PFM"}="H2A1 PFM"; $resn_exch{"2H2A PFM"}="H2A2 PFM"; $resn_exch{"3H2A PFM"}="H2A3 PFM";
$resn_exch{"1H5A PFM"}="H5A1 PFM"; $resn_exch{"2H5A PFM"}="H5A2 PFM"; $resn_exch{"2HOP PFM"}="HOP2 PFM";
$resn_exch{"3HOP PFM"}="HOP3 PFM"; $resn_exch{"1HAB PFP"}="HAB1 PFP"; $resn_exch{"2HAB PFP"}="HAB2 PFP";
$resn_exch{"3HAB PFP"}="HAB3 PFP"; $resn_exch{"1HAN PFP"}="HAN1 PFP"; $resn_exch{"2HAN PFP"}="HAN2 PFP";
$resn_exch{"1HAM PFP"}="HAM1 PFP"; $resn_exch{"2HAM PFP"}="HAM2 PFP"; $resn_exch{"1HAA PFP"}="HAA1 PFP";
$resn_exch{"2HAA PFP"}="HAA2 PFP"; $resn_exch{"3HAA PFP"}="HAA3 PFP"; $resn_exch{"1H23 PFQ"}="H231 PFQ";
$resn_exch{"2H23 PFQ"}="H232 PFQ"; $resn_exch{"1H8C PFQ"}="H8C1 PFQ"; $resn_exch{"2H8C PFQ"}="H8C2 PFQ";
$resn_exch{"1H1  PFS"}=" H11 PFS"; $resn_exch{"2H1  PFS"}=" H12 PFS"; $resn_exch{"1HH3 PFS"}="HH31 PFS";
$resn_exch{"2HH3 PFS"}="HH32 PFS"; $resn_exch{"3HH3 PFS"}="HH33 PFS"; $resn_exch{"1H3  PFS"}=" H31 PFS";
$resn_exch{"2H3  PFS"}=" H32 PFS"; $resn_exch{"1H11 PFS"}="H111 PFS"; $resn_exch{"2H11 PFS"}="H112 PFS";
$resn_exch{"1H12 PFS"}="H121 PFS"; $resn_exch{"2H12 PFS"}="H122 PFS"; $resn_exch{"1H31 PFS"}="H311 PFS";
$resn_exch{"2H31 PFS"}="H312 PFS"; $resn_exch{"1H32 PFS"}="H321 PFS"; $resn_exch{"2H32 PFS"}="H322 PFS";
$resn_exch{"1H1N PFS"}="H1N1 PFS"; $resn_exch{"2H1N PFS"}="H1N2 PFS"; $resn_exch{"3H1N PFS"}="H1N3 PFS";
$resn_exch{"1H2N PFS"}="H2N1 PFS"; $resn_exch{"2H2N PFS"}="H2N2 PFS"; $resn_exch{"3H2N PFS"}="H2N3 PFS";
$resn_exch{"1H3N PFS"}="H3N1 PFS"; $resn_exch{"2H3N PFS"}="H3N2 PFS"; $resn_exch{"3H3N PFS"}="H3N3 PFS";
$resn_exch{"1H13 PFS"}="H131 PFS"; $resn_exch{"2H13 PFS"}="H132 PFS"; $resn_exch{"1H14 PFS"}="H141 PFS";
$resn_exch{"2H14 PFS"}="H142 PFS"; $resn_exch{"1H15 PFS"}="H151 PFS"; $resn_exch{"2H15 PFS"}="H152 PFS";
$resn_exch{"1H16 PFS"}="H161 PFS"; $resn_exch{"2H16 PFS"}="H162 PFS"; $resn_exch{"1H17 PFS"}="H171 PFS";
$resn_exch{"2H17 PFS"}="H172 PFS"; $resn_exch{"1H18 PFS"}="H181 PFS"; $resn_exch{"2H18 PFS"}="H182 PFS";
$resn_exch{"1H19 PFS"}="H191 PFS"; $resn_exch{"2H19 PFS"}="H192 PFS"; $resn_exch{"1H20 PFS"}="H201 PFS";
$resn_exch{"2H20 PFS"}="H202 PFS"; $resn_exch{"1H21 PFS"}="H211 PFS"; $resn_exch{"2H21 PFS"}="H212 PFS";
$resn_exch{"1H22 PFS"}="H221 PFS"; $resn_exch{"2H22 PFS"}="H222 PFS"; $resn_exch{"1H23 PFS"}="H231 PFS";
$resn_exch{"2H23 PFS"}="H232 PFS"; $resn_exch{"1H24 PFS"}="H241 PFS"; $resn_exch{"2H24 PFS"}="H242 PFS";
$resn_exch{"1H25 PFS"}="H251 PFS"; $resn_exch{"2H25 PFS"}="H252 PFS"; $resn_exch{"1H26 PFS"}="H261 PFS";
$resn_exch{"2H26 PFS"}="H262 PFS"; $resn_exch{"3H26 PFS"}="H263 PFS"; $resn_exch{"1H4  PFZ"}=" H41 PFZ";
$resn_exch{"2H4  PFZ"}=" H42 PFZ"; $resn_exch{"1H5  PFZ"}=" H51 PFZ"; $resn_exch{"2H5  PFZ"}=" H52 PFZ";
$resn_exch{"1H6  PFZ"}=" H61 PFZ"; $resn_exch{"2H6  PFZ"}=" H62 PFZ"; $resn_exch{"1H7  PFZ"}=" H71 PFZ";
$resn_exch{"2H7  PFZ"}=" H72 PFZ"; $resn_exch{"1H8  PFZ"}=" H81 PFZ"; $resn_exch{"2H8  PFZ"}=" H82 PFZ";
$resn_exch{"1H9  PFZ"}=" H91 PFZ"; $resn_exch{"2H9  PFZ"}=" H92 PFZ"; $resn_exch{"1H10 PFZ"}="H101 PFZ";
$resn_exch{"2H10 PFZ"}="H102 PFZ"; $resn_exch{"3H10 PFZ"}="H103 PFZ"; $resn_exch{"1H5  PG0"}=" H51 PG0";
$resn_exch{"2H5  PG0"}=" H52 PG0"; $resn_exch{"3H5  PG0"}=" H53 PG0"; $resn_exch{"1H4  PG0"}=" H41 PG0";
$resn_exch{"2H4  PG0"}=" H42 PG0"; $resn_exch{"1H3  PG0"}=" H31 PG0"; $resn_exch{"2H3  PG0"}=" H32 PG0";
$resn_exch{"1H2  PG0"}=" H21 PG0"; $resn_exch{"2H2  PG0"}=" H22 PG0"; $resn_exch{"1H1  PG0"}=" H11 PG0";
$resn_exch{"2H1  PG0"}=" H12 PG0"; $resn_exch{"H2   PG1"}="HC22 PG1"; $resn_exch{" H23 PG1"}=" H   PG1";
$resn_exch{" H24 PG1"}=" H2  PG1"; $resn_exch{" H20 PG1"}=" HB2 PG1"; $resn_exch{" H21 PG1"}=" HB3 PG1";
$resn_exch{"1H2  PG2"}=" H21 PG2"; $resn_exch{"2H2  PG2"}=" H22 PG2"; $resn_exch{"1H3  PG2"}=" H31 PG2";
$resn_exch{"2H3  PG2"}=" H32 PG2"; $resn_exch{"1H4  PG2"}=" H41 PG2"; $resn_exch{"2H4  PG2"}=" H42 PG2";
$resn_exch{"1H7  PG2"}=" H71 PG2"; $resn_exch{"2H7  PG2"}=" H72 PG2"; $resn_exch{"1H10 PG2"}="H101 PG2";
$resn_exch{"2H10 PG2"}="H102 PG2"; $resn_exch{"1H16 PG2"}="H161 PG2"; $resn_exch{"2H16 PG2"}="H162 PG2";
$resn_exch{"1H17 PG2"}="H171 PG2"; $resn_exch{"2H17 PG2"}="H172 PG2"; $resn_exch{"1H18 PG2"}="H181 PG2";
$resn_exch{"2H18 PG2"}="H182 PG2"; $resn_exch{"1H19 PG2"}="H191 PG2"; $resn_exch{"2H19 PG2"}="H192 PG2";
$resn_exch{"1H20 PG2"}="H201 PG2"; $resn_exch{"2H20 PG2"}="H202 PG2"; $resn_exch{"3H20 PG2"}="H203 PG2";
$resn_exch{"1HB  PG3"}=" HB1 PG3"; $resn_exch{"2HB  PG3"}=" HB2 PG3"; $resn_exch{"1HG  PG3"}=" HG1 PG3";
$resn_exch{"2HG  PG3"}=" HG2 PG3"; $resn_exch{"1HD  PG3"}=" HD1 PG3"; $resn_exch{"2HD  PG3"}=" HD2 PG3";
$resn_exch{"1HH1 PG3"}="HH11 PG3"; $resn_exch{"2HH1 PG3"}="HH12 PG3"; $resn_exch{"1HH2 PG3"}="HH21 PG3";
$resn_exch{"2HH2 PG3"}="HH22 PG3"; $resn_exch{"1H1  PG5"}=" H11 PG5"; $resn_exch{"2H1  PG5"}=" H12 PG5";
$resn_exch{"3H1  PG5"}=" H13 PG5"; $resn_exch{"1H2  PG5"}=" H21 PG5"; $resn_exch{"2H2  PG5"}=" H22 PG5";
$resn_exch{"1H3  PG5"}=" H31 PG5"; $resn_exch{"2H3  PG5"}=" H32 PG5"; $resn_exch{"1H4  PG5"}=" H41 PG5";
$resn_exch{"2H4  PG5"}=" H42 PG5"; $resn_exch{"1H5  PG5"}=" H51 PG5"; $resn_exch{"2H5  PG5"}=" H52 PG5";
$resn_exch{"1H6  PG5"}=" H61 PG5"; $resn_exch{"2H6  PG5"}=" H62 PG5"; $resn_exch{"1H7  PG5"}=" H71 PG5";
$resn_exch{"2H7  PG5"}=" H72 PG5"; $resn_exch{"1H8  PG5"}=" H81 PG5"; $resn_exch{"2H8  PG5"}=" H82 PG5";
$resn_exch{"3H8  PG5"}=" H83 PG5"; $resn_exch{"1H1  PG6"}=" H11 PG6"; $resn_exch{"2H1  PG6"}=" H12 PG6";
$resn_exch{"3H1  PG6"}=" H13 PG6"; $resn_exch{"1H2  PG6"}=" H21 PG6"; $resn_exch{"2H2  PG6"}=" H22 PG6";
$resn_exch{"1H3  PG6"}=" H31 PG6"; $resn_exch{"2H3  PG6"}=" H32 PG6"; $resn_exch{"1H4  PG6"}=" H41 PG6";
$resn_exch{"2H4  PG6"}=" H42 PG6"; $resn_exch{"1H5  PG6"}=" H51 PG6"; $resn_exch{"2H5  PG6"}=" H52 PG6";
$resn_exch{"1H6  PG6"}=" H61 PG6"; $resn_exch{"2H6  PG6"}=" H62 PG6"; $resn_exch{"1H7  PG6"}=" H71 PG6";
$resn_exch{"2H7  PG6"}=" H72 PG6"; $resn_exch{"1H8  PG6"}=" H81 PG6"; $resn_exch{"2H8  PG6"}=" H82 PG6";
$resn_exch{"1H9  PG6"}=" H91 PG6"; $resn_exch{"2H9  PG6"}=" H92 PG6"; $resn_exch{"1H10 PG6"}="H101 PG6";
$resn_exch{"2H10 PG6"}="H102 PG6"; $resn_exch{"1H11 PG6"}="H111 PG6"; $resn_exch{"2H11 PG6"}="H112 PG6";
$resn_exch{"1H12 PG6"}="H121 PG6"; $resn_exch{"2H12 PG6"}="H122 PG6"; $resn_exch{"3H12 PG6"}="H123 PG6";
$resn_exch{"2HOP PG7"}="HOP2 PG7"; $resn_exch{"H3T  PG7"}="HO3' PG7"; $resn_exch{" H1* PG7"}=" H1' PG7";
$resn_exch{"1H2* PG7"}=" H2' PG7"; $resn_exch{"2H2* PG7"}="H2'' PG7"; $resn_exch{" H3* PG7"}=" H3' PG7";
$resn_exch{" H4* PG7"}=" H4' PG7"; $resn_exch{"1H5* PG7"}=" H5' PG7"; $resn_exch{"2H5* PG7"}="H5'' PG7";
$resn_exch{"1H2  PG7"}=" H21 PG7"; $resn_exch{"3HOP PG7"}="HOP3 PG7"; $resn_exch{"1HN  PG9"}=" H   PG9";
$resn_exch{"2HN  PG9"}=" H2  PG9"; $resn_exch{"3HOP PGA"}="HOP3 PGA"; $resn_exch{"4HOP PGA"}="HOP4 PGA";
$resn_exch{"1H2  PGA"}=" H21 PGA"; $resn_exch{"2H2  PGA"}=" H22 PGA"; $resn_exch{"1H2  PGC"}=" H21 PGC";
$resn_exch{"2H2  PGC"}=" H22 PGC"; $resn_exch{"3HOP PGC"}="HOP3 PGC"; $resn_exch{"4HOP PGC"}="HOP4 PGC";
$resn_exch{"2HOB PGD"}="HOB2 PGD"; $resn_exch{"2HOA PGD"}="HOA2 PGD"; $resn_exch{"1H5' PGD"}="H5'1 PGD";
$resn_exch{"2H5' PGD"}="H5'2 PGD"; $resn_exch{"'HO3 PGD"}="HO3' PGD"; $resn_exch{"'HO2 PGD"}="HO2' PGD";
$resn_exch{"1HN2 PGD"}="HN21 PGD"; $resn_exch{"2HN2 PGD"}="HN22 PGD"; $resn_exch{"1H10 PGD"}="H101 PGD";
$resn_exch{"2H10 PGD"}="H102 PGD"; $resn_exch{"1H19 PGD"}="H191 PGD"; $resn_exch{"2H19 PGD"}="H192 PGD";
$resn_exch{"2H1  PGE"}=" H12 PGE"; $resn_exch{"2H2  PGE"}=" H22 PGE"; $resn_exch{"2H3  PGE"}=" H32 PGE";
$resn_exch{"2H4  PGE"}=" H42 PGE"; $resn_exch{"2H6  PGE"}=" H62 PGE"; $resn_exch{"2H5  PGE"}=" H52 PGE";
$resn_exch{"1H7  PGF"}=" H71 PGF"; $resn_exch{"2H7  PGF"}=" H72 PGF"; $resn_exch{"3H7  PGF"}=" H73 PGF";
$resn_exch{"1H8  PGF"}=" H81 PGF"; $resn_exch{"2H8  PGF"}=" H82 PGF"; $resn_exch{"1H9  PGF"}=" H91 PGF";
$resn_exch{"2H9  PGF"}=" H92 PGF"; $resn_exch{"1H10 PGF"}="H101 PGF"; $resn_exch{"2H10 PGF"}="H102 PGF";
$resn_exch{"1H11 PGF"}="H111 PGF"; $resn_exch{"2H11 PGF"}="H112 PGF"; $resn_exch{"1H12 PGF"}="H121 PGF";
$resn_exch{"2H12 PGF"}="H122 PGF"; $resn_exch{"1H13 PGF"}="H131 PGF"; $resn_exch{"2H13 PGF"}="H132 PGF";
$resn_exch{"1H14 PGF"}="H141 PGF"; $resn_exch{"2H14 PGF"}="H142 PGF"; $resn_exch{"1H15 PGF"}="H151 PGF";
$resn_exch{"2H15 PGF"}="H152 PGF"; $resn_exch{"3H15 PGF"}="H153 PGF"; $resn_exch{"1H2  PGG"}=" H21 PGG";
$resn_exch{"1H3  PGG"}=" H31 PGG"; $resn_exch{"1H5  PGG"}=" H51 PGG"; $resn_exch{"1H6  PGG"}=" H61 PGG";
$resn_exch{"1HO2 PGG"}="HO21 PGG"; $resn_exch{"1H8  PGG"}=" H81 PGG"; $resn_exch{"2H8  PGG"}=" H82 PGG";
$resn_exch{"1H9  PGG"}=" H91 PGG"; $resn_exch{"2H9  PGG"}=" H92 PGG"; $resn_exch{"1H10 PGG"}="H101 PGG";
$resn_exch{"2H10 PGG"}="H102 PGG"; $resn_exch{"1HN2 PGG"}="HN21 PGG"; $resn_exch{"1H12 PGG"}="H121 PGG";
$resn_exch{"2H12 PGG"}="H122 PGG"; $resn_exch{"1HO7 PGG"}="HO71 PGG"; $resn_exch{"1H2  PGH"}=" H21 PGH";
$resn_exch{"2H2  PGH"}=" H22 PGH"; $resn_exch{"3HOP PGH"}="HOP3 PGH"; $resn_exch{"4HOP PGH"}="HOP4 PGH";
$resn_exch{"1H6C PGI"}="H6C1 PGI"; $resn_exch{"2H6C PGI"}="H6C2 PGI"; $resn_exch{"1H9C PGI"}="H9C1 PGI";
$resn_exch{"2H9C PGI"}="H9C2 PGI"; $resn_exch{"1H10 PGI"}="H101 PGI"; $resn_exch{"2H10 PGI"}="H102 PGI";
$resn_exch{"1H6  PGJ"}=" H61 PGJ"; $resn_exch{"2H6  PGJ"}=" H62 PGJ"; $resn_exch{"1H10 PGJ"}="H101 PGJ";
$resn_exch{"2H10 PGJ"}="H102 PGJ"; $resn_exch{"1H11 PGJ"}="H111 PGJ"; $resn_exch{"2H11 PGJ"}="H112 PGJ";
$resn_exch{"3H11 PGJ"}="H113 PGJ"; $resn_exch{"1H48 PGK"}="H481 PGK"; $resn_exch{"2H48 PGK"}="H482 PGK";
$resn_exch{"3H48 PGK"}="H483 PGK"; $resn_exch{"1H47 PGK"}="H471 PGK"; $resn_exch{"2H47 PGK"}="H472 PGK";
$resn_exch{"1H46 PGK"}="H461 PGK"; $resn_exch{"2H46 PGK"}="H462 PGK"; $resn_exch{"1H45 PGK"}="H451 PGK";
$resn_exch{"2H45 PGK"}="H452 PGK"; $resn_exch{"1H44 PGK"}="H441 PGK"; $resn_exch{"2H44 PGK"}="H442 PGK";
$resn_exch{"1H43 PGK"}="H431 PGK"; $resn_exch{"2H43 PGK"}="H432 PGK"; $resn_exch{"1H42 PGK"}="H421 PGK";
$resn_exch{"2H42 PGK"}="H422 PGK"; $resn_exch{"1H41 PGK"}="H411 PGK"; $resn_exch{"2H41 PGK"}="H412 PGK";
$resn_exch{"1H38 PGK"}="H381 PGK"; $resn_exch{"2H38 PGK"}="H382 PGK"; $resn_exch{"1H37 PGK"}="H371 PGK";
$resn_exch{"2H37 PGK"}="H372 PGK"; $resn_exch{"1H36 PGK"}="H361 PGK"; $resn_exch{"2H36 PGK"}="H362 PGK";
$resn_exch{"1H35 PGK"}="H351 PGK"; $resn_exch{"2H35 PGK"}="H352 PGK"; $resn_exch{"1H34 PGK"}="H341 PGK";
$resn_exch{"2H34 PGK"}="H342 PGK"; $resn_exch{"1H33 PGK"}="H331 PGK"; $resn_exch{"2H33 PGK"}="H332 PGK";
$resn_exch{"1H32 PGK"}="H321 PGK"; $resn_exch{"2H32 PGK"}="H322 PGK"; $resn_exch{"1H1  PGK"}=" H11 PGK";
$resn_exch{"2H1  PGK"}=" H12 PGK"; $resn_exch{"1H4  PGK"}=" H41 PGK"; $resn_exch{"2H4  PGK"}=" H42 PGK";
$resn_exch{"1H6  PGK"}=" H61 PGK"; $resn_exch{"2H6  PGK"}=" H62 PGK"; $resn_exch{"1H3  PGK"}=" H31 PGK";
$resn_exch{"2H3  PGK"}=" H32 PGK"; $resn_exch{"1H12 PGK"}="H121 PGK"; $resn_exch{"2H12 PGK"}="H122 PGK";
$resn_exch{"1H13 PGK"}="H131 PGK"; $resn_exch{"2H13 PGK"}="H132 PGK"; $resn_exch{"1H14 PGK"}="H141 PGK";
$resn_exch{"2H14 PGK"}="H142 PGK"; $resn_exch{"1H15 PGK"}="H151 PGK"; $resn_exch{"2H15 PGK"}="H152 PGK";
$resn_exch{"1H16 PGK"}="H161 PGK"; $resn_exch{"2H16 PGK"}="H162 PGK"; $resn_exch{"1H17 PGK"}="H171 PGK";
$resn_exch{"2H17 PGK"}="H172 PGK"; $resn_exch{"1H18 PGK"}="H181 PGK"; $resn_exch{"2H18 PGK"}="H182 PGK";
$resn_exch{"1H19 PGK"}="H191 PGK"; $resn_exch{"2H19 PGK"}="H192 PGK"; $resn_exch{"1H20 PGK"}="H201 PGK";
$resn_exch{"2H20 PGK"}="H202 PGK"; $resn_exch{"1H21 PGK"}="H211 PGK"; $resn_exch{"2H21 PGK"}="H212 PGK";
$resn_exch{"1H22 PGK"}="H221 PGK"; $resn_exch{"2H22 PGK"}="H222 PGK"; $resn_exch{"1H23 PGK"}="H231 PGK";
$resn_exch{"2H23 PGK"}="H232 PGK"; $resn_exch{"1H24 PGK"}="H241 PGK"; $resn_exch{"2H24 PGK"}="H242 PGK";
$resn_exch{"1H25 PGK"}="H251 PGK"; $resn_exch{"2H25 PGK"}="H252 PGK"; $resn_exch{"1H26 PGK"}="H261 PGK";
$resn_exch{"2H26 PGK"}="H262 PGK"; $resn_exch{"3H26 PGK"}="H263 PGK"; $resn_exch{"1HN  PGL"}=" HN1 PGL";
$resn_exch{"2HN  PGL"}=" HN2 PGL"; $resn_exch{"1H   PGL"}=" H1  PGL"; $resn_exch{"2H   PGL"}=" H2  PGL";
$resn_exch{"1HC1 PGM"}="HC11 PGM"; $resn_exch{"2HC1 PGM"}="HC12 PGM"; $resn_exch{"1HC3 PGM"}="HC31 PGM";
$resn_exch{"2HC3 PGM"}="HC32 PGM"; $resn_exch{"1HC7 PGM"}="HC71 PGM"; $resn_exch{"2HC7 PGM"}="HC72 PGM";
$resn_exch{"1HC9 PGM"}="HC91 PGM"; $resn_exch{"2HC9 PGM"}="HC92 PGM"; $resn_exch{"1HCB PGM"}="HCB1 PGM";
$resn_exch{"2HCB PGM"}="HCB2 PGM"; $resn_exch{"1HCC PGM"}="HCC1 PGM"; $resn_exch{"2HCC PGM"}="HCC2 PGM";
$resn_exch{"1HCD PGM"}="HCD1 PGM"; $resn_exch{"2HCD PGM"}="HCD2 PGM"; $resn_exch{"1HCE PGM"}="HCE1 PGM";
$resn_exch{"2HCE PGM"}="HCE2 PGM"; $resn_exch{"1HCF PGM"}="HCF1 PGM"; $resn_exch{"2HCF PGM"}="HCF2 PGM";
$resn_exch{"1HCG PGM"}="HCG1 PGM"; $resn_exch{"2HCG PGM"}="HCG2 PGM"; $resn_exch{"1HCH PGM"}="HCH1 PGM";
$resn_exch{"2HCH PGM"}="HCH2 PGM"; $resn_exch{"1HCI PGM"}="HCI1 PGM"; $resn_exch{"2HCI PGM"}="HCI2 PGM";
$resn_exch{"1HCJ PGM"}="HCJ1 PGM"; $resn_exch{"2HCJ PGM"}="HCJ2 PGM"; $resn_exch{"1HCK PGM"}="HCK1 PGM";
$resn_exch{"2HCK PGM"}="HCK2 PGM"; $resn_exch{"1HCL PGM"}="HCL1 PGM"; $resn_exch{"2HCL PGM"}="HCL2 PGM";
$resn_exch{"1HCM PGM"}="HCM1 PGM"; $resn_exch{"2HCM PGM"}="HCM2 PGM"; $resn_exch{"1HCN PGM"}="HCN1 PGM";
$resn_exch{"2HCN PGM"}="HCN2 PGM"; $resn_exch{"1HCO PGM"}="HCO1 PGM"; $resn_exch{"2HCO PGM"}="HCO2 PGM";
$resn_exch{"1HCP PGM"}="HCP1 PGM"; $resn_exch{"2HCP PGM"}="HCP2 PGM"; $resn_exch{"3HCP PGM"}="HCP3 PGM";
$resn_exch{"5HOP PGN"}="HOP5 PGN"; $resn_exch{"6HOP PGN"}="HOP6 PGN"; $resn_exch{"2HOP PGN"}="HOP2 PGN";
$resn_exch{"3HOP PGN"}="HOP3 PGN"; $resn_exch{"1H5* PGN"}=" H5' PGN"; $resn_exch{"2H5* PGN"}="H5'' PGN";
$resn_exch{" H4* PGN"}=" H4' PGN"; $resn_exch{" H3* PGN"}=" H3' PGN"; $resn_exch{"1H2* PGN"}=" H2' PGN";
$resn_exch{"2H2* PGN"}="H2'' PGN"; $resn_exch{" H1* PGN"}=" H1' PGN"; $resn_exch{"1HN2 PGN"}="HN21 PGN";
$resn_exch{"2HN2 PGN"}="HN22 PGN"; $resn_exch{"5HOP PGP"}="HOP5 PGP"; $resn_exch{"6HOP PGP"}="HOP6 PGP";
$resn_exch{"2HOP PGP"}="HOP2 PGP"; $resn_exch{"3HOP PGP"}="HOP3 PGP"; $resn_exch{"1H5* PGP"}=" H5' PGP";
$resn_exch{"2H5* PGP"}="H5'' PGP"; $resn_exch{" H4* PGP"}=" H4' PGP"; $resn_exch{" H3* PGP"}=" H3' PGP";
$resn_exch{" H2* PGP"}=" H2' PGP"; $resn_exch{"*HO2 PGP"}="HO2' PGP"; $resn_exch{" H1* PGP"}=" H1' PGP";
$resn_exch{"1HN2 PGP"}="HN21 PGP"; $resn_exch{"2HN2 PGP"}="HN22 PGP"; $resn_exch{"1H1  PGQ"}=" H11 PGQ";
$resn_exch{"2H1  PGQ"}=" H12 PGQ"; $resn_exch{"1H3  PGQ"}=" H31 PGQ"; $resn_exch{"2H3  PGQ"}=" H32 PGQ";
$resn_exch{"3H3  PGQ"}=" H33 PGQ"; $resn_exch{"1H1  PGR"}=" H11 PGR"; $resn_exch{"2H1  PGR"}=" H12 PGR";
$resn_exch{"1H3  PGR"}=" H31 PGR"; $resn_exch{"2H3  PGR"}=" H32 PGR"; $resn_exch{"3H3  PGR"}=" H33 PGR";
$resn_exch{"2HOA PGS"}="HOA2 PGS"; $resn_exch{"3HOA PGS"}="HOA3 PGS"; $resn_exch{"1H5* PGS"}="H5'1 PGS";
$resn_exch{"2H5* PGS"}="H5'2 PGS"; $resn_exch{" H4* PGS"}=" H4' PGS"; $resn_exch{" H3* PGS"}=" H3' PGS";
$resn_exch{"*HO3 PGS"}="HO3' PGS"; $resn_exch{" H2* PGS"}=" H2' PGS"; $resn_exch{"*HO2 PGS"}="HO2' PGS";
$resn_exch{" H1* PGS"}=" H1' PGS"; $resn_exch{"2HOP PGS"}="HOP2 PGS"; $resn_exch{"3HOP PGS"}="HOP3 PGS";
$resn_exch{"1H48 PGT"}="H481 PGT"; $resn_exch{"2H48 PGT"}="H482 PGT"; $resn_exch{"3H48 PGT"}="H483 PGT";
$resn_exch{"1H47 PGT"}="H471 PGT"; $resn_exch{"2H47 PGT"}="H472 PGT"; $resn_exch{"1H46 PGT"}="H461 PGT";
$resn_exch{"2H46 PGT"}="H462 PGT"; $resn_exch{"1H45 PGT"}="H451 PGT"; $resn_exch{"2H45 PGT"}="H452 PGT";
$resn_exch{"1H44 PGT"}="H441 PGT"; $resn_exch{"2H44 PGT"}="H442 PGT"; $resn_exch{"1H43 PGT"}="H431 PGT";
$resn_exch{"2H43 PGT"}="H432 PGT"; $resn_exch{"1H42 PGT"}="H421 PGT"; $resn_exch{"2H42 PGT"}="H422 PGT";
$resn_exch{"1H41 PGT"}="H411 PGT"; $resn_exch{"2H41 PGT"}="H412 PGT"; $resn_exch{"1H40 PGT"}="H401 PGT";
$resn_exch{"2H40 PGT"}="H402 PGT"; $resn_exch{"1H39 PGT"}="H391 PGT"; $resn_exch{"2H39 PGT"}="H392 PGT";
$resn_exch{"1H38 PGT"}="H381 PGT"; $resn_exch{"2H38 PGT"}="H382 PGT"; $resn_exch{"1H37 PGT"}="H371 PGT";
$resn_exch{"2H37 PGT"}="H372 PGT"; $resn_exch{"1H36 PGT"}="H361 PGT"; $resn_exch{"2H36 PGT"}="H362 PGT";
$resn_exch{"1H35 PGT"}="H351 PGT"; $resn_exch{"2H35 PGT"}="H352 PGT"; $resn_exch{"1H34 PGT"}="H341 PGT";
$resn_exch{"2H34 PGT"}="H342 PGT"; $resn_exch{"1H33 PGT"}="H331 PGT"; $resn_exch{"2H33 PGT"}="H332 PGT";
$resn_exch{"1H32 PGT"}="H321 PGT"; $resn_exch{"2H32 PGT"}="H322 PGT"; $resn_exch{"1H1  PGT"}=" H11 PGT";
$resn_exch{"2H1  PGT"}=" H12 PGT"; $resn_exch{"1H4  PGT"}=" H41 PGT"; $resn_exch{"2H4  PGT"}=" H42 PGT";
$resn_exch{"1H6  PGT"}=" H61 PGT"; $resn_exch{"2H6  PGT"}=" H62 PGT"; $resn_exch{"1H3  PGT"}=" H31 PGT";
$resn_exch{"2H3  PGT"}=" H32 PGT"; $resn_exch{"1H12 PGT"}="H121 PGT"; $resn_exch{"2H12 PGT"}="H122 PGT";
$resn_exch{"1H13 PGT"}="H131 PGT"; $resn_exch{"2H13 PGT"}="H132 PGT"; $resn_exch{"1H14 PGT"}="H141 PGT";
$resn_exch{"2H14 PGT"}="H142 PGT"; $resn_exch{"1H15 PGT"}="H151 PGT"; $resn_exch{"2H15 PGT"}="H152 PGT";
$resn_exch{"1H16 PGT"}="H161 PGT"; $resn_exch{"2H16 PGT"}="H162 PGT"; $resn_exch{"1H17 PGT"}="H171 PGT";
$resn_exch{"2H17 PGT"}="H172 PGT"; $resn_exch{"1H18 PGT"}="H181 PGT"; $resn_exch{"2H18 PGT"}="H182 PGT";
$resn_exch{"1H19 PGT"}="H191 PGT"; $resn_exch{"2H19 PGT"}="H192 PGT"; $resn_exch{"1H20 PGT"}="H201 PGT";
$resn_exch{"2H20 PGT"}="H202 PGT"; $resn_exch{"1H21 PGT"}="H211 PGT"; $resn_exch{"2H21 PGT"}="H212 PGT";
$resn_exch{"1H22 PGT"}="H221 PGT"; $resn_exch{"2H22 PGT"}="H222 PGT"; $resn_exch{"1H23 PGT"}="H231 PGT";
$resn_exch{"2H23 PGT"}="H232 PGT"; $resn_exch{"1H24 PGT"}="H241 PGT"; $resn_exch{"2H24 PGT"}="H242 PGT";
$resn_exch{"1H25 PGT"}="H251 PGT"; $resn_exch{"2H25 PGT"}="H252 PGT"; $resn_exch{"1H26 PGT"}="H261 PGT";
$resn_exch{"2H26 PGT"}="H262 PGT"; $resn_exch{"3H26 PGT"}="H263 PGT"; $resn_exch{"1H2A PGU"}="H2A1 PGU";
$resn_exch{"2H2A PGU"}="H2A2 PGU"; $resn_exch{"3H2A PGU"}="H2A3 PGU"; $resn_exch{"1H4A PGU"}="H4A1 PGU";
$resn_exch{"2H4A PGU"}="H4A2 PGU"; $resn_exch{"1H5A PGU"}="H5A1 PGU"; $resn_exch{"2H5A PGU"}="H5A2 PGU";
$resn_exch{"1HB  PGU"}=" HB1 PGU"; $resn_exch{"2HB  PGU"}=" HB2 PGU"; $resn_exch{"1HG  PGU"}=" HG1 PGU";
$resn_exch{"2HG  PGU"}=" HG2 PGU"; $resn_exch{"1H01 PGV"}="H011 PGV"; $resn_exch{"2H01 PGV"}="H012 PGV";
$resn_exch{"1H03 PGV"}="H031 PGV"; $resn_exch{"2H03 PGV"}="H032 PGV"; $resn_exch{"1H04 PGV"}="H041 PGV";
$resn_exch{"2H04 PGV"}="H042 PGV"; $resn_exch{"1H06 PGV"}="H061 PGV"; $resn_exch{"2H06 PGV"}="H062 PGV";
$resn_exch{"1H2  PGV"}=" H21 PGV"; $resn_exch{"2H2  PGV"}=" H22 PGV"; $resn_exch{"1H3  PGV"}=" H31 PGV";
$resn_exch{"2H3  PGV"}=" H32 PGV"; $resn_exch{"1H4  PGV"}=" H41 PGV"; $resn_exch{"2H4  PGV"}=" H42 PGV";
$resn_exch{"1H5  PGV"}=" H51 PGV"; $resn_exch{"2H5  PGV"}=" H52 PGV"; $resn_exch{"1H6  PGV"}=" H61 PGV";
$resn_exch{"2H6  PGV"}=" H62 PGV"; $resn_exch{"1H7  PGV"}=" H71 PGV"; $resn_exch{"2H7  PGV"}=" H72 PGV";
$resn_exch{"1H8  PGV"}=" H81 PGV"; $resn_exch{"2H8  PGV"}=" H82 PGV"; $resn_exch{"1H9  PGV"}=" H91 PGV";
$resn_exch{"2H9  PGV"}=" H92 PGV"; $resn_exch{"1H10 PGV"}="H101 PGV"; $resn_exch{"2H10 PGV"}="H102 PGV";
$resn_exch{"1H13 PGV"}="H131 PGV"; $resn_exch{"2H13 PGV"}="H132 PGV"; $resn_exch{"1H14 PGV"}="H141 PGV";
$resn_exch{"2H14 PGV"}="H142 PGV"; $resn_exch{"1H15 PGV"}="H151 PGV"; $resn_exch{"2H15 PGV"}="H152 PGV";
$resn_exch{"1H16 PGV"}="H161 PGV"; $resn_exch{"2H16 PGV"}="H162 PGV"; $resn_exch{"1H17 PGV"}="H171 PGV";
$resn_exch{"2H17 PGV"}="H172 PGV"; $resn_exch{"1H18 PGV"}="H181 PGV"; $resn_exch{"2H18 PGV"}="H182 PGV";
$resn_exch{"3H18 PGV"}="H183 PGV"; $resn_exch{"1H20 PGV"}="H201 PGV"; $resn_exch{"2H20 PGV"}="H202 PGV";
$resn_exch{"1H21 PGV"}="H211 PGV"; $resn_exch{"2H21 PGV"}="H212 PGV"; $resn_exch{"1H22 PGV"}="H221 PGV";
$resn_exch{"2H22 PGV"}="H222 PGV"; $resn_exch{"1H23 PGV"}="H231 PGV"; $resn_exch{"2H23 PGV"}="H232 PGV";
$resn_exch{"1H24 PGV"}="H241 PGV"; $resn_exch{"2H24 PGV"}="H242 PGV"; $resn_exch{"1H25 PGV"}="H251 PGV";
$resn_exch{"2H25 PGV"}="H252 PGV"; $resn_exch{"1H26 PGV"}="H261 PGV"; $resn_exch{"2H26 PGV"}="H262 PGV";
$resn_exch{"1H27 PGV"}="H271 PGV"; $resn_exch{"2H27 PGV"}="H272 PGV"; $resn_exch{"1H28 PGV"}="H281 PGV";
$resn_exch{"2H28 PGV"}="H282 PGV"; $resn_exch{"1H29 PGV"}="H291 PGV"; $resn_exch{"2H29 PGV"}="H292 PGV";
$resn_exch{"1H30 PGV"}="H301 PGV"; $resn_exch{"2H30 PGV"}="H302 PGV"; $resn_exch{"1H31 PGV"}="H311 PGV";
$resn_exch{"2H31 PGV"}="H312 PGV"; $resn_exch{"1H32 PGV"}="H321 PGV"; $resn_exch{"2H32 PGV"}="H322 PGV";
$resn_exch{"1H33 PGV"}="H331 PGV"; $resn_exch{"2H33 PGV"}="H332 PGV"; $resn_exch{"1H34 PGV"}="H341 PGV";
$resn_exch{"2H34 PGV"}="H342 PGV"; $resn_exch{"3H34 PGV"}="H343 PGV"; $resn_exch{" H1  PGW"}=" HAD PGW";
$resn_exch{"H2   PGW"}="HADA PGW"; $resn_exch{"H3   PGW"}="HOAE PGW"; $resn_exch{"H4   PGW"}="HOAF PGW";
$resn_exch{" H5  PGW"}=" H01 PGW"; $resn_exch{"H6   PGW"}="H01A PGW"; $resn_exch{" H7  PGW"}=" H02 PGW";
$resn_exch{" H8  PGW"}=" H2  PGW"; $resn_exch{" H9  PGW"}=" H2A PGW"; $resn_exch{" H10 PGW"}=" H03 PGW";
$resn_exch{"H11  PGW"}="H03A PGW"; $resn_exch{" H12 PGW"}=" H3  PGW"; $resn_exch{" H13 PGW"}=" H3A PGW";
$resn_exch{" H14 PGW"}=" H04 PGW"; $resn_exch{"H15  PGW"}="H04A PGW"; $resn_exch{" H16 PGW"}=" H4  PGW";
$resn_exch{" H17 PGW"}=" H4A PGW"; $resn_exch{" H18 PGW"}=" H05 PGW"; $resn_exch{" H19 PGW"}=" H5  PGW";
$resn_exch{" H20 PGW"}=" H5A PGW"; $resn_exch{" H21 PGW"}=" H06 PGW"; $resn_exch{"H22  PGW"}="H06A PGW";
$resn_exch{" H23 PGW"}=" H6  PGW"; $resn_exch{" H24 PGW"}=" H6A PGW"; $resn_exch{" H25 PGW"}=" H07 PGW";
$resn_exch{"H26  PGW"}="H07A PGW"; $resn_exch{" H27 PGW"}=" H7  PGW"; $resn_exch{" H28 PGW"}=" H7A PGW";
$resn_exch{" H29 PGW"}=" H08 PGW"; $resn_exch{"H30  PGW"}="H08A PGW"; $resn_exch{" H31 PGW"}=" H8  PGW";
$resn_exch{" H32 PGW"}=" H8A PGW"; $resn_exch{" H33 PGW"}=" H09 PGW"; $resn_exch{"H34  PGW"}="H09A PGW";
$resn_exch{" H35 PGW"}=" H11 PGW"; $resn_exch{"H36  PGW"}="H11A PGW"; $resn_exch{" H37 PGW"}=" H12 PGW";
$resn_exch{"H38  PGW"}="H12A PGW"; $resn_exch{" H39 PGW"}=" H13 PGW"; $resn_exch{"H40  PGW"}="H13A PGW";
$resn_exch{"H41  PGW"}="HO13 PGW"; $resn_exch{" H42 PGW"}=" H14 PGW"; $resn_exch{"H43  PGW"}="H14A PGW";
$resn_exch{"H44  PGW"}="H14B PGW"; $resn_exch{" H45 PGW"}=" H15 PGW"; $resn_exch{"H46  PGW"}="H15A PGW";
$resn_exch{" H47 PGW"}=" H16 PGW"; $resn_exch{"H48  PGW"}="H16A PGW"; $resn_exch{" H49 PGW"}=" H17 PGW";
$resn_exch{"H50  PGW"}="H17A PGW"; $resn_exch{" H51 PGW"}=" H18 PGW"; $resn_exch{"H52  PGW"}="H18A PGW";
$resn_exch{" H53 PGW"}=" H20 PGW"; $resn_exch{"H54  PGW"}="H20A PGW"; $resn_exch{" H55 PGW"}=" H21 PGW";
$resn_exch{"H56  PGW"}="H21A PGW"; $resn_exch{" H57 PGW"}=" H22 PGW"; $resn_exch{"H58  PGW"}="H22A PGW";
$resn_exch{" H59 PGW"}=" H23 PGW"; $resn_exch{"H60  PGW"}="H23A PGW"; $resn_exch{" H61 PGW"}=" H24 PGW";
$resn_exch{"H62  PGW"}="H24A PGW"; $resn_exch{" H63 PGW"}=" H25 PGW"; $resn_exch{"H64  PGW"}="H25A PGW";
$resn_exch{" H65 PGW"}=" H26 PGW"; $resn_exch{"H66  PGW"}="H26A PGW"; $resn_exch{" H67 PGW"}=" H27 PGW";
$resn_exch{"H68  PGW"}="H27A PGW"; $resn_exch{" H69 PGW"}=" H28 PGW"; $resn_exch{"H70  PGW"}="H28A PGW";
$resn_exch{" H71 PGW"}=" H29 PGW"; $resn_exch{"H72  PGW"}="H29A PGW"; $resn_exch{"H73  PGW"}="H29B PGW";
$resn_exch{" H74 PGW"}=" H30 PGW"; $resn_exch{"H75  PGW"}="H30A PGW"; $resn_exch{" H76 PGW"}=" H9  PGW";
$resn_exch{" H77 PGW"}=" H10 PGW"; $resn_exch{"1H2  PGX"}=" H21 PGX"; $resn_exch{"2H2  PGX"}=" H22 PGX";
$resn_exch{"1H3  PGX"}=" H31 PGX"; $resn_exch{"2H3  PGX"}=" H32 PGX"; $resn_exch{"1H4  PGX"}=" H41 PGX";
$resn_exch{"2H4  PGX"}=" H42 PGX"; $resn_exch{"1H5  PGX"}=" H51 PGX"; $resn_exch{"1H6  PGX"}=" H61 PGX";
$resn_exch{"1H7  PGX"}=" H71 PGX"; $resn_exch{"2H7  PGX"}=" H72 PGX"; $resn_exch{"1H8  PGX"}=" H81 PGX";
$resn_exch{"1H9  PGX"}=" H91 PGX"; $resn_exch{"1H10 PGX"}="H101 PGX"; $resn_exch{"2H10 PGX"}="H102 PGX";
$resn_exch{"1H11 PGX"}="H111 PGX"; $resn_exch{"1H12 PGX"}="H121 PGX"; $resn_exch{"1H13 PGX"}="H131 PGX";
$resn_exch{"1H14 PGX"}="H141 PGX"; $resn_exch{"1H15 PGX"}="H151 PGX"; $resn_exch{"1H16 PGX"}="H161 PGX";
$resn_exch{"2H16 PGX"}="H162 PGX"; $resn_exch{"1H17 PGX"}="H171 PGX"; $resn_exch{"2H17 PGX"}="H172 PGX";
$resn_exch{"1H18 PGX"}="H181 PGX"; $resn_exch{"2H18 PGX"}="H182 PGX"; $resn_exch{"1H19 PGX"}="H191 PGX";
$resn_exch{"2H19 PGX"}="H192 PGX"; $resn_exch{"1H20 PGX"}="H201 PGX"; $resn_exch{"2H20 PGX"}="H202 PGX";
$resn_exch{"3H20 PGX"}="H203 PGX"; $resn_exch{"1HO6 PGX"}="HO61 PGX"; $resn_exch{"1HN  PGY"}=" H   PGY";
$resn_exch{"2HN  PGY"}=" H2  PGY"; $resn_exch{"1H1  PH1"}=" H11 PH1"; $resn_exch{"2H1  PH1"}=" H12 PH1";
$resn_exch{"1H2  PH1"}=" H21 PH1"; $resn_exch{"2H2  PH1"}=" H22 PH1"; $resn_exch{"1H4  PH1"}=" H41 PH1";
$resn_exch{"2H4  PH1"}=" H42 PH1"; $resn_exch{"3H4  PH1"}="H43A PH1"; $resn_exch{"1H5  PH1"}=" H51 PH1";
$resn_exch{"2H5  PH1"}="H52A PH1"; $resn_exch{"1H6  PH1"}=" H61 PH1"; $resn_exch{"2H6  PH1"}=" H62 PH1";
$resn_exch{"1H7  PH1"}=" H71 PH1"; $resn_exch{"2H7  PH1"}=" H72 PH1"; $resn_exch{"1H9  PH1"}=" H91 PH1";
$resn_exch{"2H9  PH1"}=" H92 PH1"; $resn_exch{"3H9  PH1"}=" H93 PH1"; $resn_exch{"1H10 PH1"}="H101 PH1";
$resn_exch{"2H10 PH1"}="H102 PH1"; $resn_exch{"1H11 PH1"}="H111 PH1"; $resn_exch{"2H11 PH1"}="H112 PH1";
$resn_exch{"1H12 PH1"}="H121 PH1"; $resn_exch{"2H12 PH1"}="H122 PH1"; $resn_exch{"1H14 PH1"}="H141 PH1";
$resn_exch{"2H14 PH1"}="H142 PH1"; $resn_exch{"3H14 PH1"}="H143 PH1"; $resn_exch{"1H15 PH1"}="H151 PH1";
$resn_exch{"2H15 PH1"}="H152 PH1"; $resn_exch{"1H16 PH1"}="H161 PH1"; $resn_exch{"2H16 PH1"}="H162 PH1";
$resn_exch{"1H17 PH1"}="H171 PH1"; $resn_exch{"2H17 PH1"}="H172 PH1"; $resn_exch{"1H19 PH1"}="H191 PH1";
$resn_exch{"2H19 PH1"}="H192 PH1"; $resn_exch{"3H19 PH1"}="H193 PH1"; $resn_exch{"1H20 PH1"}="H201 PH1";
$resn_exch{"2H20 PH1"}="H202 PH1"; $resn_exch{"3H20 PH1"}="H203 PH1"; $resn_exch{"1H31 PH1"}="H311 PH1";
$resn_exch{"2H31 PH1"}="H312 PH1"; $resn_exch{"1H32 PH1"}="H321 PH1"; $resn_exch{"2H32 PH1"}="H322 PH1";
$resn_exch{"1H34 PH1"}="H341 PH1"; $resn_exch{"2H34 PH1"}="H342 PH1"; $resn_exch{"3H34 PH1"}="H343 PH1";
$resn_exch{"1H35 PH1"}="H351 PH1"; $resn_exch{"2H35 PH1"}="H352 PH1"; $resn_exch{"1H36 PH1"}="H361 PH1";
$resn_exch{"2H36 PH1"}="H362 PH1"; $resn_exch{"1H37 PH1"}="H371 PH1"; $resn_exch{"2H37 PH1"}="H372 PH1";
$resn_exch{"1H39 PH1"}="H391 PH1"; $resn_exch{"2H39 PH1"}="H392 PH1"; $resn_exch{"3H39 PH1"}="H393 PH1";
$resn_exch{"1H40 PH1"}="H401 PH1"; $resn_exch{"2H40 PH1"}="H402 PH1"; $resn_exch{"1H41 PH1"}="H411 PH1";
$resn_exch{"2H41 PH1"}="H412 PH1"; $resn_exch{"1H42 PH1"}="H421 PH1"; $resn_exch{"2H42 PH1"}="H422 PH1";
$resn_exch{"1H44 PH1"}="H441 PH1"; $resn_exch{"2H44 PH1"}="H442 PH1"; $resn_exch{"3H44 PH1"}="H443 PH1";
$resn_exch{"1H45 PH1"}="H451 PH1"; $resn_exch{"2H45 PH1"}="H452 PH1"; $resn_exch{"1H46 PH1"}="H461 PH1";
$resn_exch{"2H46 PH1"}="H462 PH1"; $resn_exch{"1H47 PH1"}="H471 PH1"; $resn_exch{"2H47 PH1"}="H472 PH1";
$resn_exch{"1H49 PH1"}="H491 PH1"; $resn_exch{"2H49 PH1"}="H492 PH1"; $resn_exch{"3H49 PH1"}="H493 PH1";
$resn_exch{"1H50 PH1"}="H501 PH1"; $resn_exch{"2H50 PH1"}="H502 PH1"; $resn_exch{"3H50 PH1"}="H503 PH1";
$resn_exch{"1H51 PH1"}="H511 PH1"; $resn_exch{"2H51 PH1"}="H512 PH1"; $resn_exch{"1H53 PH1"}="H531 PH1";
$resn_exch{"2H53 PH1"}="H532 PH1"; $resn_exch{"3H53 PH1"}="H533 PH1"; $resn_exch{"1H3  PH2"}=" H31 PH2";
$resn_exch{"2H3  PH2"}=" H32 PH2"; $resn_exch{"1HN6 PH2"}="HN61 PH2"; $resn_exch{"2HN6 PH2"}="HN62 PH2";
$resn_exch{"1H11 PH2"}="H111 PH2"; $resn_exch{"2H11 PH2"}="H112 PH2"; $resn_exch{"1H3C PH5"}="H3C1 PH5";
$resn_exch{"2H3C PH5"}="H3C2 PH5"; $resn_exch{"1H11 PH5"}="H111 PH5"; $resn_exch{"2H11 PH5"}="H112 PH5";
$resn_exch{"1H12 PH5"}="H121 PH5"; $resn_exch{"2H12 PH5"}="H122 PH5"; $resn_exch{"3H12 PH5"}="H123 PH5";
$resn_exch{"1H9C PH5"}="H9C1 PH5"; $resn_exch{"2H9C PH5"}="H9C2 PH5"; $resn_exch{"1HBC PH5"}="HBC1 PH5";
$resn_exch{"2HBC PH5"}="HBC2 PH5"; $resn_exch{"1H1  PH9"}=" H11 PH9"; $resn_exch{"2H1  PH9"}=" H12 PH9";
$resn_exch{"1HC4 PH9"}="HC41 PH9"; $resn_exch{"2HC4 PH9"}="HC42 PH9"; $resn_exch{"1HC2 PH9"}="HC21 PH9";
$resn_exch{"2HC2 PH9"}="HC22 PH9"; $resn_exch{"1HC6 PH9"}="HC61 PH9"; $resn_exch{"2HC6 PH9"}="HC62 PH9";
$resn_exch{"1H10 PH9"}="H101 PH9"; $resn_exch{"2H10 PH9"}="H102 PH9"; $resn_exch{"1HC9 PH9"}="HC91 PH9";
$resn_exch{"2HC9 PH9"}="HC92 PH9"; $resn_exch{"1HC7 PH9"}="HC71 PH9"; $resn_exch{"2HC7 PH9"}="HC72 PH9";
$resn_exch{"1HN  PHA"}=" H   PHA"; $resn_exch{"2HN  PHA"}=" H2  PHA"; $resn_exch{" H   PHA"}=" HC  PHA";
$resn_exch{"1HB  PHA"}=" HB2 PHA"; $resn_exch{"2HB  PHA"}=" HB3 PHA"; $resn_exch{"'HO2 PHB"}="HO2' PHB";
$resn_exch{"1HA  PHC"}=" HA1 PHC"; $resn_exch{"2HA  PHC"}=" HA2 PHC"; $resn_exch{"3HA  PHC"}=" HA3 PHC";
$resn_exch{"1HG2 PHC"}="HG21 PHC"; $resn_exch{"2HG2 PHC"}="HG22 PHC"; $resn_exch{"3HG2 PHC"}="HG23 PHC";
$resn_exch{"2H   PHD"}=" H2  PHD"; $resn_exch{"1H   PHD"}=" H   PHD"; $resn_exch{"2HB  PHD"}=" HB2 PHD";
$resn_exch{"1HB  PHD"}=" HB3 PHD"; $resn_exch{" HO  PHD"}=" HXT PHD"; $resn_exch{" HN2 PHE"}=" H2  PHE";
$resn_exch{"1HB  PHE"}=" HB2 PHE"; $resn_exch{"2HB  PHE"}=" HB3 PHE"; $resn_exch{"1HA1 PHF"}="HA11 PHF";
$resn_exch{"1HB1 PHF"}="HB11 PHF"; $resn_exch{"1HB2 PHF"}="HB21 PHF"; $resn_exch{"1HB3 PHF"}="HB31 PHF";
$resn_exch{"1HC1 PHF"}="HC11 PHF"; $resn_exch{"1HC2 PHF"}="HC21 PHF"; $resn_exch{"1HC3 PHF"}="HC31 PHF";
$resn_exch{"1HD1 PHF"}="HD11 PHF"; $resn_exch{"1HE1 PHF"}="HE11 PHF"; $resn_exch{"1HE2 PHF"}="HE21 PHF";
$resn_exch{"1HE3 PHF"}="HE31 PHF"; $resn_exch{"1H3  PHH"}=" H31 PHH"; $resn_exch{"2H3  PHH"}=" H32 PHH";
$resn_exch{"1HN  PHI"}=" H   PHI"; $resn_exch{"2HN  PHI"}=" H2  PHI"; $resn_exch{"1HB  PHI"}=" HB2 PHI";
$resn_exch{"2HB  PHI"}=" HB3 PHI"; $resn_exch{"1H2  PHJ"}=" H21 PHJ"; $resn_exch{"2H2  PHJ"}=" H22 PHJ";
$resn_exch{"1HN  PHL"}=" H   PHL"; $resn_exch{"2HN  PHL"}=" H2  PHL"; $resn_exch{" H1  PHL"}=" HC1 PHL";
$resn_exch{" H2  PHL"}=" HC2 PHL"; $resn_exch{"1HB  PHL"}=" HB2 PHL"; $resn_exch{"2HB  PHL"}=" HB3 PHL";
$resn_exch{"1HM  PHM"}=" HM1 PHM"; $resn_exch{"2HM  PHM"}=" HM2 PHM"; $resn_exch{"3HM  PHM"}=" HM3 PHM";
$resn_exch{"1HN  PHM"}=" H   PHM"; $resn_exch{"2HN  PHM"}=" H2  PHM"; $resn_exch{"1HB  PHM"}=" HB2 PHM";
$resn_exch{"2HB  PHM"}=" HB3 PHM"; $resn_exch{"1HMA PHO"}="HMA1 PHO"; $resn_exch{"2HMA PHO"}="HMA2 PHO";
$resn_exch{"3HMA PHO"}="HMA3 PHO"; $resn_exch{"1HAA PHO"}="HAA1 PHO"; $resn_exch{"2HAA PHO"}="HAA2 PHO";
$resn_exch{"1HBA PHO"}="HBA1 PHO"; $resn_exch{"2HBA PHO"}="HBA2 PHO"; $resn_exch{"1HMB PHO"}="HMB1 PHO";
$resn_exch{"2HMB PHO"}="HMB2 PHO"; $resn_exch{"3HMB PHO"}="HMB3 PHO"; $resn_exch{"1HBB PHO"}="HBB1 PHO";
$resn_exch{"2HBB PHO"}="HBB2 PHO"; $resn_exch{"1HMC PHO"}="HMC1 PHO"; $resn_exch{"2HMC PHO"}="HMC2 PHO";
$resn_exch{"3HMC PHO"}="HMC3 PHO"; $resn_exch{"1HAC PHO"}="HAC1 PHO"; $resn_exch{"2HAC PHO"}="HAC2 PHO";
$resn_exch{"1HBC PHO"}="HBC1 PHO"; $resn_exch{"2HBC PHO"}="HBC2 PHO"; $resn_exch{"3HBC PHO"}="HBC3 PHO";
$resn_exch{"1HMD PHO"}="HMD1 PHO"; $resn_exch{"2HMD PHO"}="HMD2 PHO"; $resn_exch{"3HMD PHO"}="HMD3 PHO";
$resn_exch{"1HED PHO"}="HED1 PHO"; $resn_exch{"2HED PHO"}="HED2 PHO"; $resn_exch{"3HED PHO"}="HED3 PHO";
$resn_exch{"1H1  PHO"}=" H11 PHO"; $resn_exch{"2H1  PHO"}=" H12 PHO"; $resn_exch{"1H4  PHO"}=" H41 PHO";
$resn_exch{"2H4  PHO"}=" H42 PHO"; $resn_exch{"3H4  PHO"}=" H43 PHO"; $resn_exch{"1H5  PHO"}=" H51 PHO";
$resn_exch{"2H5  PHO"}=" H52 PHO"; $resn_exch{"1H6  PHO"}=" H61 PHO"; $resn_exch{"2H6  PHO"}=" H62 PHO";
$resn_exch{"1H7  PHO"}=" H71 PHO"; $resn_exch{"2H7  PHO"}=" H72 PHO"; $resn_exch{"1H9  PHO"}=" H91 PHO";
$resn_exch{"2H9  PHO"}=" H92 PHO"; $resn_exch{"3H9  PHO"}=" H93 PHO"; $resn_exch{"1H10 PHO"}="H101 PHO";
$resn_exch{"2H10 PHO"}="H102 PHO"; $resn_exch{"1H11 PHO"}="H111 PHO"; $resn_exch{"2H11 PHO"}="H112 PHO";
$resn_exch{"1H12 PHO"}="H121 PHO"; $resn_exch{"2H12 PHO"}="H122 PHO"; $resn_exch{"1H14 PHO"}="H141 PHO";
$resn_exch{"2H14 PHO"}="H142 PHO"; $resn_exch{"3H14 PHO"}="H143 PHO"; $resn_exch{"1H15 PHO"}="H151 PHO";
$resn_exch{"2H15 PHO"}="H152 PHO"; $resn_exch{"1H16 PHO"}="H161 PHO"; $resn_exch{"2H16 PHO"}="H162 PHO";
$resn_exch{"1H17 PHO"}="H171 PHO"; $resn_exch{"2H17 PHO"}="H172 PHO"; $resn_exch{"1H19 PHO"}="H191 PHO";
$resn_exch{"2H19 PHO"}="H192 PHO"; $resn_exch{"3H19 PHO"}="H193 PHO"; $resn_exch{"1H20 PHO"}="H201 PHO";
$resn_exch{"2H20 PHO"}="H202 PHO"; $resn_exch{"3H20 PHO"}="H203 PHO"; $resn_exch{"1HB1 PHP"}="HB11 PHP";
$resn_exch{"2HB1 PHP"}="HB12 PHP"; $resn_exch{"1HD1 PHP"}="HD11 PHP"; $resn_exch{"2HD1 PHP"}="HD12 PHP";
$resn_exch{"1HE1 PHP"}="HE11 PHP"; $resn_exch{"2HE1 PHP"}="HE12 PHP"; $resn_exch{"1HG2 PHP"}="HG21 PHP";
$resn_exch{"2HG2 PHP"}="HG22 PHP"; $resn_exch{"3HG2 PHP"}="HG23 PHP"; $resn_exch{"4HG2 PHP"}="HG24 PHP";
$resn_exch{"5HG2 PHP"}="HG25 PHP"; $resn_exch{"6HG2 PHP"}="HG26 PHP"; $resn_exch{"1HB3 PHP"}="HB31 PHP";
$resn_exch{"2HB3 PHP"}="HB32 PHP"; $resn_exch{"1HD3 PHP"}="HD31 PHP"; $resn_exch{"2HD3 PHP"}="HD32 PHP";
$resn_exch{"1HE3 PHP"}="HE31 PHP"; $resn_exch{"2HE3 PHP"}="HE32 PHP"; $resn_exch{"'HO3 PHP"}="HO3' PHP";
$resn_exch{"1HB4 PHP"}="HB41 PHP"; $resn_exch{"2HB4 PHP"}="HB42 PHP"; $resn_exch{"1HD4 PHP"}="HD41 PHP";
$resn_exch{"2HD4 PHP"}="HD42 PHP"; $resn_exch{"1HE4 PHP"}="HE41 PHP"; $resn_exch{"2HE4 PHP"}="HE42 PHP";
$resn_exch{"1HG5 PHP"}="HG51 PHP"; $resn_exch{"2HG5 PHP"}="HG52 PHP"; $resn_exch{"3HG5 PHP"}="HG53 PHP";
$resn_exch{"4HG5 PHP"}="HG54 PHP"; $resn_exch{"5HG5 PHP"}="HG55 PHP"; $resn_exch{"6HG5 PHP"}="HG56 PHP";
$resn_exch{"1HB6 PHP"}="HB61 PHP"; $resn_exch{"2HB6 PHP"}="HB62 PHP"; $resn_exch{"1HD6 PHP"}="HD61 PHP";
$resn_exch{"1HE6 PHP"}="HE61 PHP"; $resn_exch{"2HE6 PHP"}="HE62 PHP"; $resn_exch{"2HD6 PHP"}="HD62 PHP";
$resn_exch{"1H1  PHQ"}=" H11 PHQ"; $resn_exch{"1H2  PHQ"}=" H21 PHQ"; $resn_exch{"2H2  PHQ"}=" H22 PHQ";
$resn_exch{"1H4  PHQ"}=" H41 PHQ"; $resn_exch{"1H5  PHQ"}=" H51 PHQ"; $resn_exch{"1H6  PHQ"}=" H61 PHQ";
$resn_exch{"1H7  PHQ"}=" H71 PHQ"; $resn_exch{"1H8  PHQ"}=" H81 PHQ"; $resn_exch{"1H20 PHR"}="H201 PHR";
$resn_exch{"2H20 PHR"}="H202 PHR"; $resn_exch{"1H19 PHR"}="H191 PHR"; $resn_exch{"2H19 PHR"}="H192 PHR";
$resn_exch{"1H18 PHR"}="H181 PHR"; $resn_exch{"2H18 PHR"}="H182 PHR"; $resn_exch{"1H17 PHR"}="H171 PHR";
$resn_exch{"2H17 PHR"}="H172 PHR"; $resn_exch{"1H16 PHR"}="H161 PHR"; $resn_exch{"2H16 PHR"}="H162 PHR";
$resn_exch{"1H15 PHR"}="H151 PHR"; $resn_exch{"2H15 PHR"}="H152 PHR"; $resn_exch{"1H10 PHR"}="H101 PHR";
$resn_exch{"2H10 PHR"}="H102 PHR"; $resn_exch{"1H14 PHR"}="H141 PHR"; $resn_exch{"2H14 PHR"}="H142 PHR";
$resn_exch{"PHO2 PHS"}="HO2P PHS"; $resn_exch{"PHO3 PHS"}="HO3P PHS"; $resn_exch{"1HOA PHW"}="HOA1 PHW";
$resn_exch{"1HC1 PHW"}="HC11 PHW"; $resn_exch{"2HC1 PHW"}="HC12 PHW"; $resn_exch{"3HC1 PHW"}="HC13 PHW";
$resn_exch{"1HC4 PHW"}="HC41 PHW"; $resn_exch{"2HC4 PHW"}="HC42 PHW"; $resn_exch{"1H16 PHW"}="H161 PHW";
$resn_exch{"2H16 PHW"}="H162 PHW"; $resn_exch{"1HOC PHW"}="HOC1 PHW"; $resn_exch{"1H61 PHW"}="H611 PHW";
$resn_exch{"2H61 PHW"}="H612 PHW"; $resn_exch{"3H61 PHW"}="H613 PHW"; $resn_exch{"1H8A PHW"}="H8A1 PHW";
$resn_exch{"2H8A PHW"}="H8A2 PHW"; $resn_exch{"1H8B PHW"}="H8B1 PHW"; $resn_exch{"2H8B PHW"}="H8B2 PHW";
$resn_exch{"1H8C PHW"}="H8C1 PHW"; $resn_exch{"2H8C PHW"}="H8C2 PHW"; $resn_exch{"1HN3 PHW"}="HN31 PHW";
$resn_exch{"1HAH PHX"}="HAH1 PHX"; $resn_exch{"2HAH PHX"}="HAH2 PHX"; $resn_exch{"1H2  PHY"}=" H21 PHY";
$resn_exch{"2H2  PHY"}=" H22 PHY"; $resn_exch{"3H2  PHY"}=" H23 PHY"; $resn_exch{"1H1' PHY"}="H1'1 PHY";
$resn_exch{"2H1' PHY"}="H1'2 PHY"; $resn_exch{"1H3' PHY"}="H3'1 PHY"; $resn_exch{"2H3' PHY"}="H3'2 PHY";
$resn_exch{"3H3' PHY"}="H3'3 PHY"; $resn_exch{"1HN1 PHY"}="HN11 PHY"; $resn_exch{"2HN1 PHY"}="HN12 PHY";
$resn_exch{"PHO4 PHY"}="HO4P PHY"; $resn_exch{"PHO5 PHY"}="HO5P PHY"; $resn_exch{"1HN1 PHZ"}="HN11 PHZ";
$resn_exch{"2HN1 PHZ"}="HN12 PHZ"; $resn_exch{"1H1  PI1"}=" H11 PI1"; $resn_exch{"2H1  PI1"}=" H12 PI1";
$resn_exch{"1H4  PI1"}=" H41 PI1"; $resn_exch{"2H4  PI1"}="H42A PI1"; $resn_exch{"1H9  PI1"}=" H91 PI1";
$resn_exch{"2H9  PI1"}=" H92 PI1"; $resn_exch{"1H12 PI1"}="H121 PI1"; $resn_exch{"2H12 PI1"}="H122 PI1";
$resn_exch{"1H15 PI1"}="H151 PI1"; $resn_exch{"2H15 PI1"}="H152 PI1"; $resn_exch{"1H23 PI1"}="H231 PI1";
$resn_exch{"2H23 PI1"}="H232 PI1"; $resn_exch{"1H26 PI1"}="H261 PI1"; $resn_exch{"2H26 PI1"}="H262 PI1";
$resn_exch{"1H31 PI1"}="H311 PI1"; $resn_exch{"2H31 PI1"}="H312 PI1"; $resn_exch{"1H32 PI1"}="H321 PI1";
$resn_exch{"2H32 PI1"}="H322 PI1"; $resn_exch{"1H33 PI1"}="H331 PI1"; $resn_exch{"2H33 PI1"}="H332 PI1";
$resn_exch{"1H39 PI1"}="H391 PI1"; $resn_exch{"2H39 PI1"}="H392 PI1"; $resn_exch{"1H40 PI1"}="H401 PI1";
$resn_exch{"2H40 PI1"}="H402 PI1"; $resn_exch{"3H40 PI1"}="H403 PI1"; $resn_exch{"1H41 PI1"}="H411 PI1";
$resn_exch{"2H41 PI1"}="H412 PI1"; $resn_exch{"3H41 PI1"}="H413 PI1"; $resn_exch{"1H47 PI1"}="H471 PI1";
$resn_exch{"2H47 PI1"}="H472 PI1"; $resn_exch{"3H47 PI1"}="H473 PI1"; $resn_exch{"1H48 PI1"}="H481 PI1";
$resn_exch{"2H48 PI1"}="H482 PI1"; $resn_exch{"3H48 PI1"}="H483 PI1"; $resn_exch{"1H49 PI1"}="H491 PI1";
$resn_exch{"2H49 PI1"}="H492 PI1"; $resn_exch{"1H1  PI2"}=" H11 PI2"; $resn_exch{"2H1  PI2"}=" H12 PI2";
$resn_exch{"1H4  PI2"}=" H41 PI2"; $resn_exch{"2H4  PI2"}=" H42 PI2"; $resn_exch{"1H9  PI2"}=" H91 PI2";
$resn_exch{"2H9  PI2"}=" H92 PI2"; $resn_exch{"1H12 PI2"}="H121 PI2"; $resn_exch{"2H12 PI2"}="H122 PI2";
$resn_exch{"1H15 PI2"}="H151 PI2"; $resn_exch{"2H15 PI2"}="H152 PI2"; $resn_exch{"1H23 PI2"}="H231 PI2";
$resn_exch{"2H23 PI2"}="H232 PI2"; $resn_exch{"1H26 PI2"}="H261 PI2"; $resn_exch{"2H26 PI2"}="H262 PI2";
$resn_exch{"1H28 PI2"}="H281 PI2"; $resn_exch{"2H28 PI2"}="H282 PI2"; $resn_exch{"1H29 PI2"}="H291 PI2";
$resn_exch{"2H29 PI2"}="H292 PI2"; $resn_exch{"1H31 PI2"}="H311 PI2"; $resn_exch{"2H31 PI2"}="H312 PI2";
$resn_exch{"3H31 PI2"}="H313 PI2"; $resn_exch{"1H32 PI2"}="H321 PI2"; $resn_exch{"2H32 PI2"}="H322 PI2";
$resn_exch{"3H32 PI2"}="H323 PI2"; $resn_exch{"1H1  PI3"}=" H11 PI3"; $resn_exch{"2H1  PI3"}="H12A PI3";
$resn_exch{"1H4  PI3"}=" H41 PI3"; $resn_exch{"2H4  PI3"}=" H42 PI3"; $resn_exch{"1H10 PI3"}="H101 PI3";
$resn_exch{"2H10 PI3"}="H102 PI3"; $resn_exch{"3H10 PI3"}="H103 PI3"; $resn_exch{"1H11 PI3"}="H111 PI3";
$resn_exch{"2H11 PI3"}="H112 PI3"; $resn_exch{"3H11 PI3"}="H113 PI3"; $resn_exch{"1H14 PI3"}="H141 PI3";
$resn_exch{"2H14 PI3"}="H142 PI3"; $resn_exch{"1H22 PI3"}="H221 PI3"; $resn_exch{"2H22 PI3"}="H222 PI3";
$resn_exch{"1H25 PI3"}="H251 PI3"; $resn_exch{"2H25 PI3"}="H252 PI3"; $resn_exch{"1H27 PI3"}="H271 PI3";
$resn_exch{"2H27 PI3"}="H272 PI3"; $resn_exch{"1H28 PI3"}="H281 PI3"; $resn_exch{"2H28 PI3"}="H282 PI3";
$resn_exch{"1H30 PI3"}="H301 PI3"; $resn_exch{"2H30 PI3"}="H302 PI3"; $resn_exch{"3H30 PI3"}="H303 PI3";
$resn_exch{"1H31 PI3"}="H311 PI3"; $resn_exch{"2H31 PI3"}="H312 PI3"; $resn_exch{"3H31 PI3"}="H313 PI3";
$resn_exch{"1H1  PI4"}=" H11 PI4"; $resn_exch{"2H1  PI4"}=" H12 PI4"; $resn_exch{"1H4  PI4"}=" H41 PI4";
$resn_exch{"2H4  PI4"}=" H42 PI4"; $resn_exch{"1H9  PI4"}=" H91 PI4"; $resn_exch{"2H9  PI4"}=" H92 PI4";
$resn_exch{"1H12 PI4"}="H121 PI4"; $resn_exch{"2H12 PI4"}="H122 PI4"; $resn_exch{"1H15 PI4"}="H151 PI4";
$resn_exch{"2H15 PI4"}="H152 PI4"; $resn_exch{"1H23 PI4"}="H231 PI4"; $resn_exch{"2H23 PI4"}="H232 PI4";
$resn_exch{"1H26 PI4"}="H261 PI4"; $resn_exch{"2H26 PI4"}="H262 PI4"; $resn_exch{"1H29 PI4"}="H291 PI4";
$resn_exch{"2H29 PI4"}="H292 PI4"; $resn_exch{"1H30 PI4"}="H301 PI4"; $resn_exch{"2H30 PI4"}="H302 PI4";
$resn_exch{"1H31 PI4"}="H311 PI4"; $resn_exch{"2H31 PI4"}="H312 PI4"; $resn_exch{"1H32 PI4"}="H321 PI4";
$resn_exch{"2H32 PI4"}="H322 PI4"; $resn_exch{"1H36 PI4"}="H361 PI4"; $resn_exch{"2H36 PI4"}="H362 PI4";
$resn_exch{"3H36 PI4"}="H363 PI4"; $resn_exch{"1H37 PI4"}="H371 PI4"; $resn_exch{"2H37 PI4"}="H372 PI4";
$resn_exch{"3H37 PI4"}="H373 PI4"; $resn_exch{"1H38 PI4"}="H381 PI4"; $resn_exch{"2H38 PI4"}="H382 PI4";
$resn_exch{"3H38 PI4"}="H383 PI4"; $resn_exch{"1H1  PI5"}=" H11 PI5"; $resn_exch{"2H1  PI5"}=" H12 PI5";
$resn_exch{"1H2  PI5"}=" H21 PI5"; $resn_exch{"2H2  PI5"}="H22A PI5"; $resn_exch{"1H5  PI5"}=" H51 PI5";
$resn_exch{"2H5  PI5"}=" H52 PI5"; $resn_exch{"1H9  PI5"}=" H91 PI5"; $resn_exch{"2H9  PI5"}=" H92 PI5";
$resn_exch{"3H9  PI5"}=" H93 PI5"; $resn_exch{"1H10 PI5"}="H101 PI5"; $resn_exch{"2H10 PI5"}="H102 PI5";
$resn_exch{"3H10 PI5"}="H103 PI5"; $resn_exch{"1H15 PI5"}="H151 PI5"; $resn_exch{"2H15 PI5"}="H152 PI5";
$resn_exch{"1H23 PI5"}="H231 PI5"; $resn_exch{"2H23 PI5"}="H232 PI5"; $resn_exch{"1H28 PI5"}="H281 PI5";
$resn_exch{"2H28 PI5"}="H282 PI5"; $resn_exch{"1H36 PI5"}="H361 PI5"; $resn_exch{"2H36 PI5"}="H362 PI5";
$resn_exch{"1H44 PI5"}="H441 PI5"; $resn_exch{"2H44 PI5"}="H442 PI5"; $resn_exch{"1H45 PI5"}="H451 PI5";
$resn_exch{"2H45 PI5"}="H452 PI5"; $resn_exch{"3H45 PI5"}="H453 PI5"; $resn_exch{"1H46 PI5"}="H461 PI5";
$resn_exch{"2H46 PI5"}="H462 PI5"; $resn_exch{"3H46 PI5"}="H463 PI5"; $resn_exch{"1H48 PI5"}="H481 PI5";
$resn_exch{"2H48 PI5"}="H482 PI5"; $resn_exch{"1H49 PI5"}="H491 PI5"; $resn_exch{"2H49 PI5"}="H492 PI5";
$resn_exch{"1H3  PI6"}=" H31 PI6"; $resn_exch{"2H3  PI6"}=" H32 PI6"; $resn_exch{"3H3  PI6"}="H33A PI6";
$resn_exch{"1H4  PI6"}="H41A PI6"; $resn_exch{"2H4  PI6"}=" H42 PI6"; $resn_exch{"3H4  PI6"}=" H43 PI6";
$resn_exch{"1H5  PI6"}=" H51 PI6"; $resn_exch{"2H5  PI6"}=" H52 PI6"; $resn_exch{"3H5  PI6"}=" H53 PI6";
$resn_exch{"1H10 PI6"}="H101 PI6"; $resn_exch{"2H10 PI6"}="H102 PI6"; $resn_exch{"1H19 PI6"}="H191 PI6";
$resn_exch{"2H19 PI6"}="H192 PI6"; $resn_exch{"1H22 PI6"}="H221 PI6"; $resn_exch{"2H22 PI6"}="H222 PI6";
$resn_exch{"1H30 PI6"}="H301 PI6"; $resn_exch{"2H30 PI6"}="H302 PI6"; $resn_exch{"1H38 PI6"}="H381 PI6";
$resn_exch{"2H38 PI6"}="H382 PI6"; $resn_exch{"1H39 PI6"}="H391 PI6"; $resn_exch{"2H39 PI6"}="H392 PI6";
$resn_exch{"3H39 PI6"}="H393 PI6"; $resn_exch{"1H40 PI6"}="H401 PI6"; $resn_exch{"2H40 PI6"}="H402 PI6";
$resn_exch{"3H40 PI6"}="H403 PI6"; $resn_exch{"1H42 PI6"}="H421 PI6"; $resn_exch{"2H42 PI6"}="H422 PI6";
$resn_exch{"1H43 PI6"}="H431 PI6"; $resn_exch{"2H43 PI6"}="H432 PI6"; $resn_exch{"1H1  PI7"}=" H11 PI7";
$resn_exch{"2H1  PI7"}="H12A PI7"; $resn_exch{"3H1  PI7"}="H13A PI7"; $resn_exch{"1H2  PI7"}="H21A PI7";
$resn_exch{"2H2  PI7"}=" H22 PI7"; $resn_exch{"1H10 PI7"}="H101 PI7"; $resn_exch{"2H10 PI7"}="H102 PI7";
$resn_exch{"3H10 PI7"}="H103 PI7"; $resn_exch{"1H11 PI7"}="H111 PI7"; $resn_exch{"2H11 PI7"}="H112 PI7";
$resn_exch{"3H11 PI7"}="H113 PI7"; $resn_exch{"1H14 PI7"}="H141 PI7"; $resn_exch{"2H14 PI7"}="H142 PI7";
$resn_exch{"1H22 PI7"}="H221 PI7"; $resn_exch{"2H22 PI7"}="H222 PI7"; $resn_exch{"1H27 PI7"}="H271 PI7";
$resn_exch{"2H27 PI7"}="H272 PI7"; $resn_exch{"1H35 PI7"}="H351 PI7"; $resn_exch{"2H35 PI7"}="H352 PI7";
$resn_exch{"1H43 PI7"}="H431 PI7"; $resn_exch{"2H43 PI7"}="H432 PI7"; $resn_exch{"1H44 PI7"}="H441 PI7";
$resn_exch{"2H44 PI7"}="H442 PI7"; $resn_exch{"3H44 PI7"}="H443 PI7"; $resn_exch{"1H45 PI7"}="H451 PI7";
$resn_exch{"2H45 PI7"}="H452 PI7"; $resn_exch{"3H45 PI7"}="H453 PI7"; $resn_exch{"1H47 PI7"}="H471 PI7";
$resn_exch{"2H47 PI7"}="H472 PI7"; $resn_exch{"1H48 PI7"}="H481 PI7"; $resn_exch{"2H48 PI7"}="H482 PI7";
$resn_exch{"1HN1 PI8"}="HN11 PI8"; $resn_exch{"1H2  PI8"}=" H21 PI8"; $resn_exch{"1H1  PI8"}=" H11 PI8";
$resn_exch{"1HO1 PI8"}="HO11 PI8"; $resn_exch{"1H9  PI8"}=" H91 PI8"; $resn_exch{"2H9  PI8"}=" H92 PI8";
$resn_exch{"1H4  PI8"}=" H41 PI8"; $resn_exch{"1H7  PI8"}=" H71 PI8"; $resn_exch{"1H5  PI8"}=" H51 PI8";
$resn_exch{"1H6  PI8"}=" H61 PI8"; $resn_exch{"1H11 PI8"}="H111 PI8"; $resn_exch{"1H12 PI8"}="H121 PI8";
$resn_exch{"2H12 PI8"}="H122 PI8"; $resn_exch{"1H18 PI8"}="H181 PI8"; $resn_exch{"1H14 PI8"}="H141 PI8";
$resn_exch{"1H17 PI8"}="H171 PI8"; $resn_exch{"1H15 PI8"}="H151 PI8"; $resn_exch{"1H16 PI8"}="H161 PI8";
$resn_exch{"1H19 PI8"}="H191 PI8"; $resn_exch{"2H19 PI8"}="H192 PI8"; $resn_exch{"1HO3 PI8"}="HO31 PI8";
$resn_exch{"1H20 PI8"}="H201 PI8"; $resn_exch{"1H21 PI8"}="H211 PI8"; $resn_exch{"2H21 PI8"}="H212 PI8";
$resn_exch{"1HN2 PI8"}="HN21 PI8"; $resn_exch{"1H22 PI8"}="H221 PI8"; $resn_exch{"1H23 PI8"}="H231 PI8";
$resn_exch{"2H23 PI8"}="H232 PI8"; $resn_exch{"1H25 PI8"}="H251 PI8"; $resn_exch{"1H26 PI8"}="H261 PI8";
$resn_exch{"1H29 PI8"}="H291 PI8"; $resn_exch{"1H28 PI8"}="H281 PI8"; $resn_exch{"1HN3 PI8"}="HN31 PI8";
$resn_exch{"1H31 PI8"}="H311 PI8"; $resn_exch{"1H32 PI8"}="H321 PI8"; $resn_exch{"1H33 PI8"}="H331 PI8";
$resn_exch{"2H33 PI8"}="H332 PI8"; $resn_exch{"3H33 PI8"}="H333 PI8"; $resn_exch{"1H34 PI8"}="H341 PI8";
$resn_exch{"2H34 PI8"}="H342 PI8"; $resn_exch{"1H35 PI8"}="H351 PI8"; $resn_exch{"2H35 PI8"}="H352 PI8";
$resn_exch{"3H35 PI8"}="H353 PI8"; $resn_exch{"1NH4 PI8"}="H41N PI8"; $resn_exch{"1H37 PI8"}="H371 PI8";
$resn_exch{"2H37 PI8"}="H372 PI8"; $resn_exch{"1H38 PI8"}="H381 PI8"; $resn_exch{"2H38 PI8"}="H382 PI8";
$resn_exch{"1H39 PI8"}="H391 PI8"; $resn_exch{"2H39 PI8"}="H392 PI8"; $resn_exch{"1H40 PI8"}="H401 PI8";
$resn_exch{"2H40 PI8"}="H402 PI8"; $resn_exch{"1H41 PI8"}="H411 PI8"; $resn_exch{"2H41 PI8"}="H412 PI8";
$resn_exch{"1H4  PI9"}=" H41 PI9"; $resn_exch{"2H4  PI9"}=" H42 PI9"; $resn_exch{"1H5  PI9"}=" H51 PI9";
$resn_exch{"2H5  PI9"}=" H52 PI9"; $resn_exch{"1H3  PI9"}=" H31 PI9"; $resn_exch{"2H3  PI9"}=" H32 PI9";
$resn_exch{"1H2  PI9"}=" H21 PI9"; $resn_exch{"2H2  PI9"}=" H22 PI9"; $resn_exch{"1HN1 PI9"}="HN11 PI9";
$resn_exch{"1H7  PI9"}=" H71 PI9"; $resn_exch{"1H8  PI9"}=" H81 PI9"; $resn_exch{"1H10 PI9"}="H101 PI9";
$resn_exch{"2H10 PI9"}="H102 PI9"; $resn_exch{"3H10 PI9"}="H103 PI9"; $resn_exch{"1H9  PI9"}=" H91 PI9";
$resn_exch{"2H9  PI9"}=" H92 PI9"; $resn_exch{"3H9  PI9"}=" H93 PI9"; $resn_exch{"1HN2 PI9"}="HN21 PI9";
$resn_exch{"1H12 PI9"}="H121 PI9"; $resn_exch{"1H13 PI9"}="H131 PI9"; $resn_exch{"2H13 PI9"}="H132 PI9";
$resn_exch{"1H15 PI9"}="H151 PI9"; $resn_exch{"1H16 PI9"}="H161 PI9"; $resn_exch{"1H19 PI9"}="H191 PI9";
$resn_exch{"1H18 PI9"}="H181 PI9"; $resn_exch{"1H1  PI9"}=" H11 PI9"; $resn_exch{"2H1  PI9"}=" H12 PI9";
$resn_exch{"1H21 PI9"}="H211 PI9"; $resn_exch{"2H21 PI9"}="H212 PI9"; $resn_exch{"1H22 PI9"}="H221 PI9";
$resn_exch{"2H22 PI9"}="H222 PI9"; $resn_exch{"1H23 PI9"}="H231 PI9"; $resn_exch{"2H23 PI9"}="H232 PI9";
$resn_exch{"1H24 PI9"}="H241 PI9"; $resn_exch{"1H26 PI9"}="H261 PI9"; $resn_exch{"2H26 PI9"}="H262 PI9";
$resn_exch{"3H26 PI9"}="H263 PI9"; $resn_exch{"1H25 PI9"}="H251 PI9"; $resn_exch{"2H25 PI9"}="H252 PI9";
$resn_exch{"3H25 PI9"}="H253 PI9"; $resn_exch{"1H32 PI9"}="H321 PI9"; $resn_exch{"1H28 PI9"}="H281 PI9";
$resn_exch{"1H31 PI9"}="H311 PI9"; $resn_exch{"1H29 PI9"}="H291 PI9"; $resn_exch{"1HN4 PI9"}="HN41 PI9";
$resn_exch{"2HN4 PI9"}="HN42 PI9"; $resn_exch{"1HN1 PIA"}="HN11 PIA"; $resn_exch{"2HN1 PIA"}="HN12 PIA";
$resn_exch{"1HB1 PIA"}="HB11 PIA"; $resn_exch{"2HB1 PIA"}="HB12 PIA"; $resn_exch{"3HB1 PIA"}="HB13 PIA";
$resn_exch{"1HA3 PIA"}="HA31 PIA"; $resn_exch{"2HA3 PIA"}="HA32 PIA"; $resn_exch{"1HC  PIB"}=" HC1 PIB";
$resn_exch{"2HC  PIB"}=" HC2 PIB"; $resn_exch{"3HC  PIB"}=" HC3 PIB"; $resn_exch{"4HC  PIB"}=" HC4 PIB";
$resn_exch{"5HC  PIB"}=" HC5 PIB"; $resn_exch{"6HC  PIB"}=" HC6 PIB"; $resn_exch{"2HO  PIB"}=" HO2 PIB";
$resn_exch{"4HO  PIB"}=" HO4 PIB"; $resn_exch{"5HO  PIB"}=" HO5 PIB"; $resn_exch{"6HO  PIB"}=" HO6 PIB";
$resn_exch{"11HO PIB"}="HO11 PIB"; $resn_exch{"31HO PIB"}="HO31 PIB"; $resn_exch{"33HO PIB"}="HO33 PIB";
$resn_exch{"1HC7 PIB"}="HC71 PIB"; $resn_exch{"2HC7 PIB"}="HC72 PIB"; $resn_exch{"8HC  PIB"}=" HC8 PIB";
$resn_exch{"1HC9 PIB"}="HC91 PIB"; $resn_exch{"2HC9 PIB"}="HC92 PIB"; $resn_exch{"1H11 PIB"}="H111 PIB";
$resn_exch{"2H11 PIB"}="H112 PIB"; $resn_exch{"1H12 PIB"}="H121 PIB"; $resn_exch{"2H12 PIB"}="H122 PIB";
$resn_exch{"1H13 PIB"}="H131 PIB"; $resn_exch{"2H13 PIB"}="H132 PIB"; $resn_exch{"3H13 PIB"}="H133 PIB";
$resn_exch{"1H15 PIB"}="H151 PIB"; $resn_exch{"2H15 PIB"}="H152 PIB"; $resn_exch{"1H16 PIB"}="H161 PIB";
$resn_exch{"2H16 PIB"}="H162 PIB"; $resn_exch{"1H17 PIB"}="H171 PIB"; $resn_exch{"2H17 PIB"}="H172 PIB";
$resn_exch{"3H17 PIB"}="H173 PIB"; $resn_exch{"1H2  PID"}=" H21 PID"; $resn_exch{"2H2  PID"}="H22A PID";
$resn_exch{"1H4  PID"}=" H41 PID"; $resn_exch{"2H4  PID"}=" H42 PID"; $resn_exch{"1H26 PID"}="H261 PID";
$resn_exch{"2H26 PID"}="H262 PID"; $resn_exch{"1H28 PID"}="H281 PID"; $resn_exch{"2H28 PID"}="H282 PID";
$resn_exch{"1H31 PID"}="H311 PID"; $resn_exch{"2H31 PID"}="H312 PID"; $resn_exch{"3H31 PID"}="H313 PID";
$resn_exch{"1HM1 PID"}="HM11 PID"; $resn_exch{"2HM1 PID"}="HM12 PID"; $resn_exch{"3HM1 PID"}="HM13 PID";
$resn_exch{"1HM2 PID"}="HM21 PID"; $resn_exch{"2HM2 PID"}="HM22 PID"; $resn_exch{"3HM2 PID"}="HM23 PID";
$resn_exch{"1HM3 PID"}="HM31 PID"; $resn_exch{"2HM3 PID"}="HM32 PID"; $resn_exch{"3HM3 PID"}="HM33 PID";
$resn_exch{"1HM4 PID"}="HM41 PID"; $resn_exch{"2HM4 PID"}="HM42 PID"; $resn_exch{"3HM4 PID"}="HM43 PID";
$resn_exch{"1HM5 PID"}="HM51 PID"; $resn_exch{"2HM5 PID"}="HM52 PID"; $resn_exch{"3HM5 PID"}="HM53 PID";
$resn_exch{"1HM6 PID"}="HM61 PID"; $resn_exch{"2HM6 PID"}="HM62 PID"; $resn_exch{"3HM6 PID"}="HM63 PID";
$resn_exch{"1HM7 PID"}="HM71 PID"; $resn_exch{"2HM7 PID"}="HM72 PID"; $resn_exch{"3HM7 PID"}="HM73 PID";
$resn_exch{"1HM8 PID"}="HM81 PID"; $resn_exch{"2HM8 PID"}="HM82 PID"; $resn_exch{"3HM8 PID"}="HM83 PID";
$resn_exch{"1H1  PIE"}=" H11 PIE"; $resn_exch{"2H1  PIE"}=" H12 PIE"; $resn_exch{"1H2  PIE"}=" H21 PIE";
$resn_exch{"1H3  PIE"}=" H31 PIE"; $resn_exch{"2H3  PIE"}=" H32 PIE"; $resn_exch{"1H32 PIE"}="H321 PIE";
$resn_exch{"2H32 PIE"}="H322 PIE"; $resn_exch{"1H33 PIE"}="H331 PIE"; $resn_exch{"2H33 PIE"}="H332 PIE";
$resn_exch{"1H34 PIE"}="H341 PIE"; $resn_exch{"2H34 PIE"}="H342 PIE"; $resn_exch{"1H35 PIE"}="H351 PIE";
$resn_exch{"2H35 PIE"}="H352 PIE"; $resn_exch{"1H36 PIE"}="H361 PIE"; $resn_exch{"2H36 PIE"}="H362 PIE";
$resn_exch{"1H37 PIE"}="H371 PIE"; $resn_exch{"2H37 PIE"}="H372 PIE"; $resn_exch{"1H38 PIE"}="H381 PIE";
$resn_exch{"2H38 PIE"}="H382 PIE"; $resn_exch{"1H39 PIE"}="H391 PIE"; $resn_exch{"2H39 PIE"}="H392 PIE";
$resn_exch{"1H40 PIE"}="H401 PIE"; $resn_exch{"2H40 PIE"}="H402 PIE"; $resn_exch{"1H41 PIE"}="H411 PIE";
$resn_exch{"2H41 PIE"}="H412 PIE"; $resn_exch{"1H42 PIE"}="H421 PIE"; $resn_exch{"2H42 PIE"}="H422 PIE";
$resn_exch{"1H22 PIE"}="H221 PIE"; $resn_exch{"2H22 PIE"}="H222 PIE"; $resn_exch{"1H23 PIE"}="H231 PIE";
$resn_exch{"2H23 PIE"}="H232 PIE"; $resn_exch{"1H24 PIE"}="H241 PIE"; $resn_exch{"2H24 PIE"}="H242 PIE";
$resn_exch{"1H25 PIE"}="H251 PIE"; $resn_exch{"2H25 PIE"}="H252 PIE"; $resn_exch{"1H26 PIE"}="H261 PIE";
$resn_exch{"2H26 PIE"}="H262 PIE"; $resn_exch{"1H27 PIE"}="H271 PIE"; $resn_exch{"2H27 PIE"}="H272 PIE";
$resn_exch{"1H28 PIE"}="H281 PIE"; $resn_exch{"2H28 PIE"}="H282 PIE"; $resn_exch{"1H29 PIE"}="H291 PIE";
$resn_exch{"1H47 PIE"}="H471 PIE"; $resn_exch{"1H48 PIE"}="H481 PIE"; $resn_exch{"2H48 PIE"}="H482 PIE";
$resn_exch{"1H49 PIE"}="H491 PIE"; $resn_exch{"2H49 PIE"}="H492 PIE"; $resn_exch{"1H50 PIE"}="H501 PIE";
$resn_exch{"2H50 PIE"}="H502 PIE"; $resn_exch{"1H51 PIE"}="H511 PIE"; $resn_exch{"2H51 PIE"}="H512 PIE";
$resn_exch{"1H1' PIE"}="H1'1 PIE"; $resn_exch{"1H2' PIE"}="H2'1 PIE"; $resn_exch{"1H3' PIE"}="H3'1 PIE";
$resn_exch{"1H4' PIE"}="H4'1 PIE"; $resn_exch{"1H5' PIE"}="H5'1 PIE"; $resn_exch{"1H6' PIE"}="H6'1 PIE";
$resn_exch{"1HO1 PIE"}="HO11 PIE"; $resn_exch{"1HO2 PIE"}="HO21 PIE"; $resn_exch{"1HO3 PIE"}="HO31 PIE";
$resn_exch{"1HO4 PIE"}="HO41 PIE"; $resn_exch{"1HO6 PIE"}="HO61 PIE"; $resn_exch{"1H52 PIE"}="H521 PIE";
$resn_exch{"2H52 PIE"}="H522 PIE"; $resn_exch{"1H53 PIE"}="H531 PIE"; $resn_exch{"2H53 PIE"}="H532 PIE";
$resn_exch{"1H54 PIE"}="H541 PIE"; $resn_exch{"2H54 PIE"}="H542 PIE"; $resn_exch{"1H55 PIE"}="H551 PIE";
$resn_exch{"2H55 PIE"}="H552 PIE"; $resn_exch{"3H55 PIE"}="H553 PIE"; $resn_exch{"1H43 PIE"}="H431 PIE";
$resn_exch{"2H43 PIE"}="H432 PIE"; $resn_exch{"1H44 PIE"}="H441 PIE"; $resn_exch{"2H44 PIE"}="H442 PIE";
$resn_exch{"1H45 PIE"}="H451 PIE"; $resn_exch{"2H45 PIE"}="H452 PIE"; $resn_exch{"1H46 PIE"}="H461 PIE";
$resn_exch{"2H46 PIE"}="H462 PIE"; $resn_exch{"3H45 PIE"}="H453 PIE"; $resn_exch{"1H1  PIG"}=" H11 PIG";
$resn_exch{"2H1  PIG"}=" H12 PIG"; $resn_exch{"1H2  PIG"}=" H21 PIG"; $resn_exch{"2H2  PIG"}=" H22 PIG";
$resn_exch{"1H3  PIG"}=" H31 PIG"; $resn_exch{"2H3  PIG"}=" H32 PIG"; $resn_exch{"1H4  PIG"}=" H41 PIG";
$resn_exch{"2H4  PIG"}=" H42 PIG"; $resn_exch{"1H5  PIG"}=" H51 PIG"; $resn_exch{"2H5  PIG"}=" H52 PIG";
$resn_exch{"1H6  PIG"}=" H61 PIG"; $resn_exch{"2H6  PIG"}=" H62 PIG"; $resn_exch{"1H21 PII"}="H211 PII";
$resn_exch{"2H21 PII"}="H212 PII"; $resn_exch{"1H25 PII"}="H251 PII"; $resn_exch{"2H25 PII"}="H252 PII";
$resn_exch{"1HR1 PII"}="HR11 PII"; $resn_exch{"2HR1 PII"}="HR12 PII"; $resn_exch{"1HR2 PII"}="HR21 PII";
$resn_exch{"2HR2 PII"}="HR22 PII"; $resn_exch{"1HL1 PII"}="HL11 PII"; $resn_exch{"2HL1 PII"}="HL12 PII";
$resn_exch{"1HL2 PII"}="HL21 PII"; $resn_exch{"2HL2 PII"}="HL22 PII"; $resn_exch{"1HR3 PII"}="HR31 PII";
$resn_exch{"2HR3 PII"}="HR32 PII"; $resn_exch{"1HR4 PII"}="HR41 PII"; $resn_exch{"2HR4 PII"}="HR42 PII";
$resn_exch{"1HR5 PII"}="HR51 PII"; $resn_exch{"2HR5 PII"}="HR52 PII"; $resn_exch{"1HR6 PII"}="HR61 PII";
$resn_exch{"2HR6 PII"}="HR62 PII"; $resn_exch{"1HR7 PII"}="HR71 PII"; $resn_exch{"2HR7 PII"}="HR72 PII";
$resn_exch{"1HR8 PII"}="HR81 PII"; $resn_exch{"2HR8 PII"}="HR82 PII"; $resn_exch{"1HR9 PII"}="HR91 PII";
$resn_exch{"2HR9 PII"}="HR92 PII"; $resn_exch{"1HG  PII"}=" HG1 PII"; $resn_exch{"1HH  PII"}=" HH1 PII";
$resn_exch{"1HL3 PII"}="HL31 PII"; $resn_exch{"2HL3 PII"}="HL32 PII"; $resn_exch{"1HL4 PII"}="HL41 PII";
$resn_exch{"2HL4 PII"}="HL42 PII"; $resn_exch{"1HL5 PII"}="HL51 PII"; $resn_exch{"2HL5 PII"}="HL52 PII";
$resn_exch{"1HL6 PII"}="HL61 PII"; $resn_exch{"2HL6 PII"}="HL62 PII"; $resn_exch{"1HL7 PII"}="HL71 PII";
$resn_exch{"2HL7 PII"}="HL72 PII"; $resn_exch{"1HL8 PII"}="HL81 PII"; $resn_exch{"2HL8 PII"}="HL82 PII";
$resn_exch{"1HL9 PII"}="HL91 PII"; $resn_exch{"2HL9 PII"}="HL92 PII"; $resn_exch{"1H11 PIL"}="H111 PIL";
$resn_exch{"2H11 PIL"}="H112 PIL"; $resn_exch{"3H11 PIL"}="H113 PIL"; $resn_exch{"1H5  PIL"}=" H51 PIL";
$resn_exch{"2H5  PIL"}=" H52 PIL"; $resn_exch{"1H6  PIL"}=" H61 PIL"; $resn_exch{"2H6  PIL"}=" H62 PIL";
$resn_exch{"1H7  PIL"}=" H71 PIL"; $resn_exch{"2H7  PIL"}=" H72 PIL"; $resn_exch{"1H8  PIL"}=" H81 PIL";
$resn_exch{"2H8  PIL"}=" H82 PIL"; $resn_exch{"1H1  PIN"}=" H11 PIN"; $resn_exch{"2H1  PIN"}=" H12 PIN";
$resn_exch{"1H2  PIN"}=" H21 PIN"; $resn_exch{"2H2  PIN"}=" H22 PIN"; $resn_exch{"1H3  PIN"}=" H31 PIN";
$resn_exch{"2H3  PIN"}=" H32 PIN"; $resn_exch{"1H4  PIN"}=" H41 PIN"; $resn_exch{"2H4  PIN"}=" H42 PIN";
$resn_exch{"1H1' PIN"}="H1'1 PIN"; $resn_exch{"2H1' PIN"}="H1'2 PIN"; $resn_exch{"1H2' PIN"}="H2'1 PIN";
$resn_exch{"2H2' PIN"}="H2'2 PIN"; $resn_exch{"1H3' PIN"}="H3'1 PIN"; $resn_exch{"2H3' PIN"}="H3'2 PIN";
$resn_exch{"1H4' PIN"}="H4'1 PIN"; $resn_exch{"2H4' PIN"}="H4'2 PIN"; $resn_exch{"'HO3 PIN"}="HO3' PIN";
$resn_exch{"2HP1 PIO"}="HP12 PIO"; $resn_exch{"2HP4 PIO"}="HP42 PIO"; $resn_exch{"3HP5 PIO"}="HP53 PIO";
$resn_exch{"2HP5 PIO"}="HP52 PIO"; $resn_exch{"1HC1 PIO"}="HC11 PIO"; $resn_exch{"2HC1 PIO"}="HC12 PIO";
$resn_exch{"1HC3 PIO"}="HC31 PIO"; $resn_exch{"2HC3 PIO"}="HC32 PIO"; $resn_exch{"1H2A PIO"}="H2A1 PIO";
$resn_exch{"2H2A PIO"}="H2A2 PIO"; $resn_exch{"1H3A PIO"}="H3A1 PIO"; $resn_exch{"2H3A PIO"}="H3A2 PIO";
$resn_exch{"1H4A PIO"}="H4A1 PIO"; $resn_exch{"2H4A PIO"}="H4A2 PIO"; $resn_exch{"1H5A PIO"}="H5A1 PIO";
$resn_exch{"2H5A PIO"}="H5A2 PIO"; $resn_exch{"1H6A PIO"}="H6A1 PIO"; $resn_exch{"2H6A PIO"}="H6A2 PIO";
$resn_exch{"1H7A PIO"}="H7A1 PIO"; $resn_exch{"2H7A PIO"}="H7A2 PIO"; $resn_exch{"1H8A PIO"}="H8A1 PIO";
$resn_exch{"2H8A PIO"}="H8A2 PIO"; $resn_exch{"3H8A PIO"}="H8A3 PIO"; $resn_exch{"1H2B PIO"}="H2B1 PIO";
$resn_exch{"2H2B PIO"}="H2B2 PIO"; $resn_exch{"1H3B PIO"}="H3B1 PIO"; $resn_exch{"2H3B PIO"}="H3B2 PIO";
$resn_exch{"1H4B PIO"}="H4B1 PIO"; $resn_exch{"2H4B PIO"}="H4B2 PIO"; $resn_exch{"1H5B PIO"}="H5B1 PIO";
$resn_exch{"2H5B PIO"}="H5B2 PIO"; $resn_exch{"1H6B PIO"}="H6B1 PIO"; $resn_exch{"2H6B PIO"}="H6B2 PIO";
$resn_exch{"1H7B PIO"}="H7B1 PIO"; $resn_exch{"2H7B PIO"}="H7B2 PIO"; $resn_exch{"1H8B PIO"}="H8B1 PIO";
$resn_exch{"2H8B PIO"}="H8B2 PIO"; $resn_exch{"3H8B PIO"}="H8B3 PIO"; $resn_exch{"1H2  PIP"}=" H21 PIP";
$resn_exch{"2H2  PIP"}=" H22 PIP"; $resn_exch{"1H3  PIP"}=" H31 PIP"; $resn_exch{"2H3  PIP"}=" H32 PIP";
$resn_exch{"1H4  PIP"}=" H41 PIP"; $resn_exch{"2H4  PIP"}=" H42 PIP"; $resn_exch{"1H5  PIP"}=" H51 PIP";
$resn_exch{"2H5  PIP"}=" H52 PIP"; $resn_exch{"1H6  PIP"}=" H61 PIP"; $resn_exch{"2H6  PIP"}=" H62 PIP";
$resn_exch{"1HM  PIQ"}=" HM1 PIQ"; $resn_exch{"2HM  PIQ"}=" HM2 PIQ"; $resn_exch{"3HM  PIQ"}=" HM3 PIQ";
$resn_exch{"1HN5 PIR"}="HN51 PIR"; $resn_exch{"2HN5 PIR"}="HN52 PIR"; $resn_exch{"'HO2 PIR"}="HO2' PIR";
$resn_exch{"'HO3 PIR"}="HO3' PIR"; $resn_exch{"'HN4 PIR"}="HN4' PIR"; $resn_exch{"1H5' PIR"}="H5'1 PIR";
$resn_exch{"2H5' PIR"}="H5'2 PIR"; $resn_exch{"'HO5 PIR"}="HO5' PIR"; $resn_exch{"1HN1 PIU"}="HN11 PIU";
$resn_exch{"2HN1 PIU"}="HN12 PIU"; $resn_exch{"1H15 PIU"}="H151 PIU"; $resn_exch{"2H15 PIU"}="H152 PIU";
$resn_exch{"1H16 PIU"}="H161 PIU"; $resn_exch{"2H16 PIU"}="H162 PIU"; $resn_exch{"1H12 PIU"}="H121 PIU";
$resn_exch{"2H12 PIU"}="H122 PIU"; $resn_exch{"3H12 PIU"}="H123 PIU"; $resn_exch{"1H13 PIU"}="H131 PIU";
$resn_exch{"2H13 PIU"}="H132 PIU"; $resn_exch{"3H13 PIU"}="H133 PIU"; $resn_exch{"1H14 PIU"}="H141 PIU";
$resn_exch{"2H14 PIU"}="H142 PIU"; $resn_exch{"3H14 PIU"}="H143 PIU"; $resn_exch{"1H1  PIV"}=" H11 PIV";
$resn_exch{"2H1  PIV"}=" H12 PIV"; $resn_exch{"3H1  PIV"}=" H13 PIV"; $resn_exch{"1H2  PIV"}=" H21 PIV";
$resn_exch{"2H2  PIV"}=" H22 PIV"; $resn_exch{"3H2  PIV"}=" H23 PIV"; $resn_exch{"1H3  PIV"}=" H31 PIV";
$resn_exch{"2H3  PIV"}=" H32 PIV"; $resn_exch{"3H3  PIV"}=" H33 PIV"; $resn_exch{"2H2  PKF"}="H22A PKF";
$resn_exch{"1H2  PKF"}=" H21 PKF"; $resn_exch{"2H9  PKF"}=" H92 PKF"; $resn_exch{"1H9  PKF"}=" H91 PKF";
$resn_exch{"2H11 PKF"}="H112 PKF"; $resn_exch{"1H11 PKF"}="H111 PKF"; $resn_exch{"2H12 PKF"}="H122 PKF";
$resn_exch{"1H12 PKF"}="H121 PKF"; $resn_exch{"2H14 PKF"}="H142 PKF"; $resn_exch{"1H14 PKF"}="H141 PKF";
$resn_exch{"2H16 PKF"}="H162 PKF"; $resn_exch{"1H16 PKF"}="H161 PKF"; $resn_exch{"2H17 PKF"}="H172 PKF";
$resn_exch{"1H17 PKF"}="H171 PKF"; $resn_exch{"2H19 PKF"}="H192 PKF"; $resn_exch{"1H19 PKF"}="H191 PKF";
$resn_exch{"3H3  PKF"}=" H33 PKF"; $resn_exch{"2H3  PKF"}=" H32 PKF"; $resn_exch{"1H3  PKF"}="H31A PKF";
$resn_exch{"1H4N PKF"}="H4N1 PKF"; $resn_exch{"2H26 PKF"}="H262 PKF"; $resn_exch{"1H26 PKF"}="H261 PKF";
$resn_exch{"3H29 PKF"}="H293 PKF"; $resn_exch{"2H29 PKF"}="H292 PKF"; $resn_exch{"1H29 PKF"}="H291 PKF";
$resn_exch{"3H32 PKF"}="H323 PKF"; $resn_exch{"2H32 PKF"}="H322 PKF"; $resn_exch{"1H32 PKF"}="H321 PKF";
$resn_exch{"2H33 PKF"}="H332 PKF"; $resn_exch{"1H33 PKF"}="H331 PKF"; $resn_exch{"2H35 PKF"}="H352 PKF";
$resn_exch{"1H35 PKF"}="H351 PKF"; $resn_exch{"2H36 PKF"}="H362 PKF"; $resn_exch{"1H36 PKF"}="H361 PKF";
$resn_exch{"1HN2 PL0"}="HN21 PL0"; $resn_exch{"2HN2 PL0"}="HN22 PL0"; $resn_exch{"1H2A PL2"}="H2A1 PL2";
$resn_exch{"2H2A PL2"}="H2A2 PL2"; $resn_exch{"3H2A PL2"}="H2A3 PL2"; $resn_exch{"1H4A PL2"}="H4A1 PL2";
$resn_exch{"2H4A PL2"}="H4A2 PL2"; $resn_exch{"1HA  PL2"}=" HA1 PL2"; $resn_exch{"2HA  PL2"}=" HA2 PL2";
$resn_exch{"1HG  PL2"}=" HG1 PL2"; $resn_exch{"2HG  PL2"}=" HG2 PL2"; $resn_exch{"1H5M PL2"}="H5M1 PL2";
$resn_exch{"2H5M PL2"}="H5M2 PL2"; $resn_exch{"1HB  PL2"}=" HB1 PL2"; $resn_exch{"2HB  PL2"}=" HB2 PL2";
$resn_exch{"1HD  PL2"}=" HD1 PL2"; $resn_exch{"2HD  PL2"}=" HD2 PL2"; $resn_exch{"1HH1 PL2"}="HH11 PL2";
$resn_exch{"2HH1 PL2"}="HH12 PL2"; $resn_exch{"1H1C PL3"}="H1C1 PL3"; $resn_exch{"2H1C PL3"}="H1C2 PL3";
$resn_exch{"1H2C PL3"}="H2C1 PL3"; $resn_exch{"2H2C PL3"}="H2C2 PL3"; $resn_exch{"1H3C PL3"}="H3C1 PL3";
$resn_exch{"2H3C PL3"}="H3C2 PL3"; $resn_exch{"1H4C PL3"}="H4C1 PL3"; $resn_exch{"2H4C PL3"}="H4C2 PL3";
$resn_exch{"1H5C PL3"}="H5C1 PL3"; $resn_exch{"2H5C PL3"}="H5C2 PL3"; $resn_exch{"1H6C PL3"}="H6C1 PL3";
$resn_exch{"2H6C PL3"}="H6C2 PL3"; $resn_exch{"1H7C PL3"}="H7C1 PL3"; $resn_exch{"2H7C PL3"}="H7C2 PL3";
$resn_exch{"1H8C PL3"}="H8C1 PL3"; $resn_exch{"2H8C PL3"}="H8C2 PL3"; $resn_exch{"1H9C PL3"}="H9C1 PL3";
$resn_exch{"2H9C PL3"}="H9C2 PL3"; $resn_exch{"1HAC PL3"}="HAC1 PL3"; $resn_exch{"2HAC PL3"}="HAC2 PL3";
$resn_exch{"1HBC PL3"}="HBC1 PL3"; $resn_exch{"2HBC PL3"}="HBC2 PL3"; $resn_exch{"1HCC PL3"}="HCC1 PL3";
$resn_exch{"2HCC PL3"}="HCC2 PL3"; $resn_exch{"1HDC PL3"}="HDC1 PL3"; $resn_exch{"2HDC PL3"}="HDC2 PL3";
$resn_exch{"1HEC PL3"}="HEC1 PL3"; $resn_exch{"2HEC PL3"}="HEC2 PL3"; $resn_exch{"1HFC PL3"}="HFC1 PL3";
$resn_exch{"2HFC PL3"}="HFC2 PL3"; $resn_exch{"1HGC PL3"}="HGC1 PL3"; $resn_exch{"2HGC PL3"}="HGC2 PL3";
$resn_exch{"3HGC PL3"}="HGC3 PL3"; $resn_exch{"1H10 PL7"}="H101 PL7"; $resn_exch{"2H10 PL7"}="H102 PL7";
$resn_exch{"1H11 PL7"}="H111 PL7"; $resn_exch{"2H11 PL7"}="H112 PL7"; $resn_exch{"1H5C PL7"}="H5C1 PL7";
$resn_exch{"2H5C PL7"}="H5C2 PL7"; $resn_exch{"1H7C PL7"}="H7C1 PL7"; $resn_exch{"2H7C PL7"}="H7C2 PL7";
$resn_exch{"1HC7 PL9"}="HC71 PL9"; $resn_exch{"2HC7 PL9"}="HC72 PL9"; $resn_exch{"1H10 PL9"}="H101 PL9";
$resn_exch{"2H10 PL9"}="H102 PL9"; $resn_exch{"3H10 PL9"}="H103 PL9"; $resn_exch{"1H11 PL9"}="H111 PL9";
$resn_exch{"2H11 PL9"}="H112 PL9"; $resn_exch{"1H12 PL9"}="H121 PL9"; $resn_exch{"2H12 PL9"}="H122 PL9";
$resn_exch{"1H15 PL9"}="H151 PL9"; $resn_exch{"2H15 PL9"}="H152 PL9"; $resn_exch{"3H15 PL9"}="H153 PL9";
$resn_exch{"1H16 PL9"}="H161 PL9"; $resn_exch{"2H16 PL9"}="H162 PL9"; $resn_exch{"1H17 PL9"}="H171 PL9";
$resn_exch{"2H17 PL9"}="H172 PL9"; $resn_exch{"1H20 PL9"}="H201 PL9"; $resn_exch{"2H20 PL9"}="H202 PL9";
$resn_exch{"3H20 PL9"}="H203 PL9"; $resn_exch{"1H21 PL9"}="H211 PL9"; $resn_exch{"2H21 PL9"}="H212 PL9";
$resn_exch{"1H22 PL9"}="H221 PL9"; $resn_exch{"2H22 PL9"}="H222 PL9"; $resn_exch{"1H25 PL9"}="H251 PL9";
$resn_exch{"2H25 PL9"}="H252 PL9"; $resn_exch{"3H25 PL9"}="H253 PL9"; $resn_exch{"1H26 PL9"}="H261 PL9";
$resn_exch{"2H26 PL9"}="H262 PL9"; $resn_exch{"1H27 PL9"}="H271 PL9"; $resn_exch{"2H27 PL9"}="H272 PL9";
$resn_exch{"1H30 PL9"}="H301 PL9"; $resn_exch{"2H30 PL9"}="H302 PL9"; $resn_exch{"3H30 PL9"}="H303 PL9";
$resn_exch{"1H31 PL9"}="H311 PL9"; $resn_exch{"2H31 PL9"}="H312 PL9"; $resn_exch{"1H32 PL9"}="H321 PL9";
$resn_exch{"2H32 PL9"}="H322 PL9"; $resn_exch{"1H35 PL9"}="H351 PL9"; $resn_exch{"2H35 PL9"}="H352 PL9";
$resn_exch{"3H35 PL9"}="H353 PL9"; $resn_exch{"1H36 PL9"}="H361 PL9"; $resn_exch{"2H36 PL9"}="H362 PL9";
$resn_exch{"1H37 PL9"}="H371 PL9"; $resn_exch{"2H37 PL9"}="H372 PL9"; $resn_exch{"1H40 PL9"}="H401 PL9";
$resn_exch{"2H40 PL9"}="H402 PL9"; $resn_exch{"3H40 PL9"}="H403 PL9"; $resn_exch{"1H41 PL9"}="H411 PL9";
$resn_exch{"2H41 PL9"}="H412 PL9"; $resn_exch{"1H42 PL9"}="H421 PL9"; $resn_exch{"2H42 PL9"}="H422 PL9";
$resn_exch{"1H45 PL9"}="H451 PL9"; $resn_exch{"2H45 PL9"}="H452 PL9"; $resn_exch{"3H45 PL9"}="H453 PL9";
$resn_exch{"1H46 PL9"}="H461 PL9"; $resn_exch{"2H46 PL9"}="H462 PL9"; $resn_exch{"1H47 PL9"}="H471 PL9";
$resn_exch{"2H47 PL9"}="H472 PL9"; $resn_exch{"1H50 PL9"}="H501 PL9"; $resn_exch{"2H50 PL9"}="H502 PL9";
$resn_exch{"3H50 PL9"}="H503 PL9"; $resn_exch{"1H51 PL9"}="H511 PL9"; $resn_exch{"2H51 PL9"}="H512 PL9";
$resn_exch{"3H51 PL9"}="H513 PL9"; $resn_exch{"1H52 PL9"}="H521 PL9"; $resn_exch{"2H52 PL9"}="H522 PL9";
$resn_exch{"3H52 PL9"}="H523 PL9"; $resn_exch{"1H53 PL9"}="H531 PL9"; $resn_exch{"2H53 PL9"}="H532 PL9";
$resn_exch{"3H53 PL9"}="H533 PL9"; $resn_exch{"1H2A PLA"}="H2A1 PLA"; $resn_exch{"2H2A PLA"}="H2A2 PLA";
$resn_exch{"3H2A PLA"}="H2A3 PLA"; $resn_exch{"1H4A PLA"}="H4A1 PLA"; $resn_exch{"2H4A PLA"}="H4A2 PLA";
$resn_exch{"1H5A PLA"}="H5A1 PLA"; $resn_exch{"2H5A PLA"}="H5A2 PLA"; $resn_exch{"2HOP PLA"}="HOP2 PLA";
$resn_exch{"3HOP PLA"}="HOP3 PLA"; $resn_exch{"1HB1 PLA"}="HB11 PLA"; $resn_exch{"2HB1 PLA"}="HB12 PLA";
$resn_exch{"1HB2 PLA"}="HB21 PLA"; $resn_exch{"2HB2 PLA"}="HB22 PLA"; $resn_exch{"3HB2 PLA"}="HB23 PLA";
$resn_exch{"1H1  PLC"}=" H11 PLC"; $resn_exch{"2H1  PLC"}=" H12 PLC"; $resn_exch{"1H3  PLC"}=" H31 PLC";
$resn_exch{"2H3  PLC"}=" H32 PLC"; $resn_exch{"1H4  PLC"}=" H41 PLC"; $resn_exch{"2H4  PLC"}=" H42 PLC";
$resn_exch{"1H5  PLC"}=" H51 PLC"; $resn_exch{"2H5  PLC"}=" H52 PLC"; $resn_exch{"1H6  PLC"}=" H61 PLC";
$resn_exch{"2H6  PLC"}=" H62 PLC"; $resn_exch{"3H6  PLC"}=" H63 PLC"; $resn_exch{"1H7  PLC"}=" H71 PLC";
$resn_exch{"2H7  PLC"}=" H72 PLC"; $resn_exch{"3H7  PLC"}=" H73 PLC"; $resn_exch{"1H8  PLC"}=" H81 PLC";
$resn_exch{"2H8  PLC"}=" H82 PLC"; $resn_exch{"3H8  PLC"}=" H83 PLC"; $resn_exch{"1H1' PLC"}="H1'1 PLC";
$resn_exch{"2H1' PLC"}="H1'2 PLC"; $resn_exch{"1H2' PLC"}="H2'1 PLC"; $resn_exch{"2H2' PLC"}="H2'2 PLC";
$resn_exch{"1H3' PLC"}="H3'1 PLC"; $resn_exch{"2H3' PLC"}="H3'2 PLC"; $resn_exch{"1H4' PLC"}="H4'1 PLC";
$resn_exch{"2H4' PLC"}="H4'2 PLC"; $resn_exch{"1H5' PLC"}="H5'1 PLC"; $resn_exch{"2H5' PLC"}="H5'2 PLC";
$resn_exch{"1H6' PLC"}="H6'1 PLC"; $resn_exch{"2H6' PLC"}="H6'2 PLC"; $resn_exch{"1H7' PLC"}="H7'1 PLC";
$resn_exch{"2H7' PLC"}="H7'2 PLC"; $resn_exch{"1H8' PLC"}="H8'1 PLC"; $resn_exch{"2H8' PLC"}="H8'2 PLC";
$resn_exch{"1H9' PLC"}="H9'1 PLC"; $resn_exch{"2H9' PLC"}="H9'2 PLC"; $resn_exch{"1HT' PLC"}="HT'1 PLC";
$resn_exch{"2HT' PLC"}="HT'2 PLC"; $resn_exch{"1HE' PLC"}="HE'1 PLC"; $resn_exch{"2HE' PLC"}="HE'2 PLC";
$resn_exch{"3HE' PLC"}="HE'3 PLC"; $resn_exch{"1H1\" PLC"}="H1A1 PLC"; $resn_exch{"2H1\" PLC"}="H1A2 PLC";
$resn_exch{"1H2\" PLC"}="H2A1 PLC"; $resn_exch{"2H2\" PLC"}="H2A2 PLC"; $resn_exch{"1H3\" PLC"}="H3A1 PLC";
$resn_exch{"2H3\" PLC"}="H3A2 PLC"; $resn_exch{"1H4\" PLC"}="H4A1 PLC"; $resn_exch{"2H4\" PLC"}="H4A2 PLC";
$resn_exch{"1H5\" PLC"}="H5A1 PLC"; $resn_exch{"2H5\" PLC"}="H5A2 PLC"; $resn_exch{"1H6\" PLC"}="H6A1 PLC";
$resn_exch{"2H6\" PLC"}="H6A2 PLC"; $resn_exch{"1H7\" PLC"}="H7A1 PLC"; $resn_exch{"2H7\" PLC"}="H7A2 PLC";
$resn_exch{"1H8\" PLC"}="H8A1 PLC"; $resn_exch{"2H8\" PLC"}="H8A2 PLC"; $resn_exch{"1H9\" PLC"}="H9A1 PLC";
$resn_exch{"2H9\" PLC"}="H9A2 PLC"; $resn_exch{"1HT\" PLC"}="HTA1 PLC"; $resn_exch{"2HT\" PLC"}="HTA2 PLC";
$resn_exch{"1HE\" PLC"}="HEA1 PLC"; $resn_exch{"2HE\" PLC"}="HEA2 PLC"; $resn_exch{"3HE\" PLC"}="HEA3 PLC";
$resn_exch{"2HOP PLC"}="HOP2 PLC"; $resn_exch{"1H1  PLD"}=" H11 PLD"; $resn_exch{"2H1  PLD"}=" H12 PLD";
$resn_exch{"1H3  PLD"}=" H31 PLD"; $resn_exch{"2H3  PLD"}=" H32 PLD"; $resn_exch{"1H4  PLD"}=" H41 PLD";
$resn_exch{"2H4  PLD"}=" H42 PLD"; $resn_exch{"1H5  PLD"}=" H51 PLD"; $resn_exch{"2H5  PLD"}=" H52 PLD";
$resn_exch{"1H6  PLD"}=" H61 PLD"; $resn_exch{"2H6  PLD"}=" H62 PLD"; $resn_exch{"3H6  PLD"}=" H63 PLD";
$resn_exch{"1H7  PLD"}=" H71 PLD"; $resn_exch{"2H7  PLD"}=" H72 PLD"; $resn_exch{"3H7  PLD"}=" H73 PLD";
$resn_exch{"1H8  PLD"}=" H81 PLD"; $resn_exch{"2H8  PLD"}=" H82 PLD"; $resn_exch{"3H8  PLD"}=" H83 PLD";
$resn_exch{"1H1' PLD"}="H1'1 PLD"; $resn_exch{"2H1' PLD"}="H1'2 PLD"; $resn_exch{"1H2' PLD"}="H2'1 PLD";
$resn_exch{"2H2' PLD"}="H2'2 PLD"; $resn_exch{"1H3' PLD"}="H3'1 PLD"; $resn_exch{"2H3' PLD"}="H3'2 PLD";
$resn_exch{"1H4' PLD"}="H4'1 PLD"; $resn_exch{"2H4' PLD"}="H4'2 PLD"; $resn_exch{"1H5' PLD"}="H5'1 PLD";
$resn_exch{"2H5' PLD"}="H5'2 PLD"; $resn_exch{"1H6' PLD"}="H6'1 PLD"; $resn_exch{"2H6' PLD"}="H6'2 PLD";
$resn_exch{"1H7' PLD"}="H7'1 PLD"; $resn_exch{"2H7' PLD"}="H7'2 PLD"; $resn_exch{"1H8' PLD"}="H8'1 PLD";
$resn_exch{"2H8' PLD"}="H8'2 PLD"; $resn_exch{"1H9' PLD"}="H9'1 PLD"; $resn_exch{"2H9' PLD"}="H9'2 PLD";
$resn_exch{"1HT' PLD"}="HT'1 PLD"; $resn_exch{"2HT' PLD"}="HT'2 PLD"; $resn_exch{"1HE' PLD"}="HE'1 PLD";
$resn_exch{"2HE' PLD"}="HE'2 PLD"; $resn_exch{"1H1\" PLD"}="H1A1 PLD"; $resn_exch{"2H1\" PLD"}="H1A2 PLD";
$resn_exch{"1H2\" PLD"}="H2A1 PLD"; $resn_exch{"2H2\" PLD"}="H2A2 PLD"; $resn_exch{"1H3\" PLD"}="H3A1 PLD";
$resn_exch{"2H3\" PLD"}="H3A2 PLD"; $resn_exch{"1H4\" PLD"}="H4A1 PLD"; $resn_exch{"2H4\" PLD"}="H4A2 PLD";
$resn_exch{"1H5\" PLD"}="H5A1 PLD"; $resn_exch{"2H5\" PLD"}="H5A2 PLD"; $resn_exch{"1H6\" PLD"}="H6A1 PLD";
$resn_exch{"2H6\" PLD"}="H6A2 PLD"; $resn_exch{"1H7\" PLD"}="H7A1 PLD"; $resn_exch{"2H7\" PLD"}="H7A2 PLD";
$resn_exch{"1H8\" PLD"}="H8A1 PLD"; $resn_exch{"2H8\" PLD"}="H8A2 PLD"; $resn_exch{"1H9\" PLD"}="H9A1 PLD";
$resn_exch{"2H9\" PLD"}="H9A2 PLD"; $resn_exch{"1HT\" PLD"}="HTA1 PLD"; $resn_exch{"2HT\" PLD"}="HTA2 PLD";
$resn_exch{"1HE\" PLD"}="HEA1 PLD"; $resn_exch{"2HE\" PLD"}="HEA2 PLD"; $resn_exch{"2HOP PLD"}="HOP2 PLD";
$resn_exch{"1HA' PLD"}="HA'1 PLD"; $resn_exch{"2HA' PLD"}="HA'2 PLD"; $resn_exch{"1HB' PLD"}="HB'1 PLD";
$resn_exch{"2HB' PLD"}="HB'2 PLD"; $resn_exch{"1HC' PLD"}="HC'1 PLD"; $resn_exch{"2HC' PLD"}="HC'2 PLD";
$resn_exch{"1HD' PLD"}="HD'1 PLD"; $resn_exch{"2HD' PLD"}="HD'2 PLD"; $resn_exch{"1HF' PLD"}="HF'1 PLD";
$resn_exch{"2HF' PLD"}="HF'2 PLD"; $resn_exch{"1HG' PLD"}="HG'1 PLD"; $resn_exch{"2HG' PLD"}="HG'2 PLD";
$resn_exch{"1HH' PLD"}="HH'1 PLD"; $resn_exch{"2HH' PLD"}="HH'2 PLD"; $resn_exch{"1HI' PLD"}="HI'1 PLD";
$resn_exch{"2HI' PLD"}="HI'2 PLD"; $resn_exch{"1HJ' PLD"}="HJ'1 PLD"; $resn_exch{"2HJ' PLD"}="HJ'2 PLD";
$resn_exch{"3HJ' PLD"}="HJ'3 PLD"; $resn_exch{"1HA\" PLD"}="HAA1 PLD"; $resn_exch{"2HA\" PLD"}="HAA2 PLD";
$resn_exch{"1HB\" PLD"}="HBA1 PLD"; $resn_exch{"2HB\" PLD"}="HBA2 PLD"; $resn_exch{"1HC\" PLD"}="HCA1 PLD";
$resn_exch{"2HC\" PLD"}="HCA2 PLD"; $resn_exch{"1HD\" PLD"}="HDA1 PLD"; $resn_exch{"2HD\" PLD"}="HDA2 PLD";
$resn_exch{"1HF\" PLD"}="HFA1 PLD"; $resn_exch{"2HF\" PLD"}="HFA2 PLD"; $resn_exch{"1HG\" PLD"}="HGA1 PLD";
$resn_exch{"2HG\" PLD"}="HGA2 PLD"; $resn_exch{"1HH\" PLD"}="HHA1 PLD"; $resn_exch{"2HH\" PLD"}="HHA2 PLD";
$resn_exch{"1HI\" PLD"}="HIA1 PLD"; $resn_exch{"2HI\" PLD"}="HIA2 PLD"; $resn_exch{"1HJ\" PLD"}="HJA1 PLD";
$resn_exch{"2HJ\" PLD"}="HJA2 PLD"; $resn_exch{"3HJ\" PLD"}="HJA3 PLD"; $resn_exch{"1HB  PLE"}=" HB1 PLE";
$resn_exch{"2HB  PLE"}=" HB2 PLE"; $resn_exch{"1HD1 PLE"}="HD11 PLE"; $resn_exch{"2HD1 PLE"}="HD12 PLE";
$resn_exch{"3HD1 PLE"}="HD13 PLE"; $resn_exch{"1HD2 PLE"}="HD21 PLE"; $resn_exch{"2HD2 PLE"}="HD22 PLE";
$resn_exch{"3HD2 PLE"}="HD23 PLE"; $resn_exch{"1HN  PLE"}=" HN1 PLE"; $resn_exch{"2HN  PLE"}=" HN2 PLE";
$resn_exch{"1H2A PLG"}="H2A1 PLG"; $resn_exch{"2H2A PLG"}="H2A2 PLG"; $resn_exch{"3H2A PLG"}="H2A3 PLG";
$resn_exch{"1H4A PLG"}="H4A1 PLG"; $resn_exch{"2H4A PLG"}="H4A2 PLG"; $resn_exch{"1H5A PLG"}="H5A1 PLG";
$resn_exch{"2H5A PLG"}="H5A2 PLG"; $resn_exch{"2HOP PLG"}="HOP2 PLG"; $resn_exch{"3HOP PLG"}="HOP3 PLG";
$resn_exch{"1H2  PLH"}=" H21 PLH"; $resn_exch{"2H2  PLH"}=" H22 PLH"; $resn_exch{"1H4  PLH"}=" H41 PLH";
$resn_exch{"2H4  PLH"}=" H42 PLH"; $resn_exch{"1H6  PLH"}=" H61 PLH"; $resn_exch{"2H6  PLH"}=" H62 PLH";
$resn_exch{"3H6  PLH"}=" H63 PLH"; $resn_exch{"1H7  PLH"}=" H71 PLH"; $resn_exch{"2H7  PLH"}=" H72 PLH";
$resn_exch{"3H7  PLH"}=" H73 PLH"; $resn_exch{"1H11 PLH"}="H111 PLH"; $resn_exch{"2H11 PLH"}="H112 PLH";
$resn_exch{"3H11 PLH"}="H113 PLH"; $resn_exch{"1H12 PLH"}="H121 PLH"; $resn_exch{"2H12 PLH"}="H122 PLH";
$resn_exch{"1H2  PLM"}=" H21 PLM"; $resn_exch{"2H2  PLM"}=" H22 PLM"; $resn_exch{"1H3  PLM"}=" H31 PLM";
$resn_exch{"2H3  PLM"}=" H32 PLM"; $resn_exch{"1H4  PLM"}=" H41 PLM"; $resn_exch{"2H4  PLM"}=" H42 PLM";
$resn_exch{"1H5  PLM"}=" H51 PLM"; $resn_exch{"2H5  PLM"}=" H52 PLM"; $resn_exch{"1H6  PLM"}=" H61 PLM";
$resn_exch{"2H6  PLM"}=" H62 PLM"; $resn_exch{"1H7  PLM"}=" H71 PLM"; $resn_exch{"2H7  PLM"}=" H72 PLM";
$resn_exch{"1H8  PLM"}=" H81 PLM"; $resn_exch{"2H8  PLM"}=" H82 PLM"; $resn_exch{"1H9  PLM"}=" H91 PLM";
$resn_exch{"2H9  PLM"}=" H92 PLM"; $resn_exch{"1HA  PLM"}=" HA1 PLM"; $resn_exch{"2HA  PLM"}=" HA2 PLM";
$resn_exch{"1HB  PLM"}=" HB1 PLM"; $resn_exch{"2HB  PLM"}=" HB2 PLM"; $resn_exch{"1HC  PLM"}=" HC1 PLM";
$resn_exch{"2HC  PLM"}=" HC2 PLM"; $resn_exch{"1HD  PLM"}=" HD1 PLM"; $resn_exch{"2HD  PLM"}=" HD2 PLM";
$resn_exch{"1HE  PLM"}=" HE1 PLM"; $resn_exch{"2HE  PLM"}=" HE2 PLM"; $resn_exch{"1HF  PLM"}=" HF1 PLM";
$resn_exch{"2HF  PLM"}=" HF2 PLM"; $resn_exch{"1HG  PLM"}=" HG1 PLM"; $resn_exch{"2HG  PLM"}=" HG2 PLM";
$resn_exch{"3HG  PLM"}=" HG3 PLM"; $resn_exch{"1H1  PLO"}=" H11 PLO"; $resn_exch{"2H1  PLO"}=" H12 PLO";
$resn_exch{"1H2  PLO"}=" H21 PLO"; $resn_exch{"2H2  PLO"}=" H22 PLO"; $resn_exch{"1H4  PLO"}=" H41 PLO";
$resn_exch{"2H4  PLO"}=" H42 PLO"; $resn_exch{"1H7  PLO"}=" H71 PLO"; $resn_exch{"2H7  PLO"}=" H72 PLO";
$resn_exch{"1H11 PLO"}="H111 PLO"; $resn_exch{"2H11 PLO"}="H112 PLO"; $resn_exch{"1H12 PLO"}="H121 PLO";
$resn_exch{"2H12 PLO"}="H122 PLO"; $resn_exch{"1H15 PLO"}="H151 PLO"; $resn_exch{"2H15 PLO"}="H152 PLO";
$resn_exch{"1H16 PLO"}="H161 PLO"; $resn_exch{"2H16 PLO"}="H162 PLO"; $resn_exch{"1H18 PLO"}="H181 PLO";
$resn_exch{"2H18 PLO"}="H182 PLO"; $resn_exch{"3H18 PLO"}="H183 PLO"; $resn_exch{"1H19 PLO"}="H191 PLO";
$resn_exch{"2H19 PLO"}="H192 PLO"; $resn_exch{"3H19 PLO"}="H193 PLO"; $resn_exch{"1H21 PLO"}="H211 PLO";
$resn_exch{"2H21 PLO"}="H212 PLO"; $resn_exch{"3H21 PLO"}="H213 PLO"; $resn_exch{"1H2A PLP"}="H2A1 PLP";
$resn_exch{"2H2A PLP"}="H2A2 PLP"; $resn_exch{"3H2A PLP"}="H2A3 PLP"; $resn_exch{"1H5A PLP"}="H5A1 PLP";
$resn_exch{"2H5A PLP"}="H5A2 PLP"; $resn_exch{"2HOP PLP"}="HOP2 PLP"; $resn_exch{"3HOP PLP"}="HOP3 PLP";
$resn_exch{"1H2A PLR"}="H2A1 PLR"; $resn_exch{"2H2A PLR"}="H2A2 PLR"; $resn_exch{"3H2A PLR"}="H2A3 PLR";
$resn_exch{"1H4A PLR"}="H4A1 PLR"; $resn_exch{"2H4A PLR"}="H4A2 PLR"; $resn_exch{"3H4A PLR"}="H4A3 PLR";
$resn_exch{"1H5A PLR"}="H5A1 PLR"; $resn_exch{"2H5A PLR"}="H5A2 PLR"; $resn_exch{"1HB  PLS"}=" HB1 PLS";
$resn_exch{"2HB  PLS"}=" HB2 PLS"; $resn_exch{"1H2A PLS"}="H2A1 PLS"; $resn_exch{"2H2A PLS"}="H2A2 PLS";
$resn_exch{"3H2A PLS"}="H2A3 PLS"; $resn_exch{"1H4A PLS"}="H4A1 PLS"; $resn_exch{"2H4A PLS"}="H4A2 PLS";
$resn_exch{"1H5A PLS"}="H5A1 PLS"; $resn_exch{"2H5A PLS"}="H5A2 PLS"; $resn_exch{"2HOP PLS"}="HOP2 PLS";
$resn_exch{"3HOP PLS"}="HOP3 PLS"; $resn_exch{"1H2A PLT"}="H2A1 PLT"; $resn_exch{"2H2A PLT"}="H2A2 PLT";
$resn_exch{"3H2A PLT"}="H2A3 PLT"; $resn_exch{"1H5A PLT"}="H5A1 PLT"; $resn_exch{"2H5A PLT"}="H5A2 PLT";
$resn_exch{"2HOP PLT"}="HOP2 PLT"; $resn_exch{"3HOP PLT"}="HOP3 PLT"; $resn_exch{"1HB  PLT"}=" HB1 PLT";
$resn_exch{"2HB  PLT"}=" HB2 PLT"; $resn_exch{"1HB  PLU"}=" HB1 PLU"; $resn_exch{"2HB  PLU"}=" HB2 PLU";
$resn_exch{"1HD1 PLU"}="HD11 PLU"; $resn_exch{"2HD1 PLU"}="HD12 PLU"; $resn_exch{"3HD1 PLU"}="HD13 PLU";
$resn_exch{"1HD2 PLU"}="HD21 PLU"; $resn_exch{"2HD2 PLU"}="HD22 PLU"; $resn_exch{"3HD2 PLU"}="HD23 PLU";
$resn_exch{"1HN  PLU"}=" HN1 PLU"; $resn_exch{"2HN  PLU"}=" HN2 PLU"; $resn_exch{"1HB  PLV"}=" HB1 PLV";
$resn_exch{"2HB  PLV"}=" HB2 PLV"; $resn_exch{"1H2A PLV"}="H2A1 PLV"; $resn_exch{"2H2A PLV"}="H2A2 PLV";
$resn_exch{"3H2A PLV"}="H2A3 PLV"; $resn_exch{"1H4A PLV"}="H4A1 PLV"; $resn_exch{"2H4A PLV"}="H4A2 PLV";
$resn_exch{"1H5A PLV"}="H5A1 PLV"; $resn_exch{"2H5A PLV"}="H5A2 PLV"; $resn_exch{"1H7  PLV"}=" H71 PLV";
$resn_exch{"2H7  PLV"}=" H72 PLV"; $resn_exch{"3H7  PLV"}=" H73 PLV"; $resn_exch{"1H23 PLX"}="H231 PLX";
$resn_exch{"2H23 PLX"}="H232 PLX"; $resn_exch{"3H23 PLX"}="H233 PLX"; $resn_exch{"1H22 PLX"}="H221 PLX";
$resn_exch{"2H22 PLX"}="H222 PLX"; $resn_exch{"1H21 PLX"}="H211 PLX"; $resn_exch{"2H21 PLX"}="H212 PLX";
$resn_exch{"1H20 PLX"}="H201 PLX"; $resn_exch{"2H20 PLX"}="H202 PLX"; $resn_exch{"1H19 PLX"}="H191 PLX";
$resn_exch{"2H19 PLX"}="H192 PLX"; $resn_exch{"1H18 PLX"}="H181 PLX"; $resn_exch{"2H18 PLX"}="H182 PLX";
$resn_exch{"1H17 PLX"}="H171 PLX"; $resn_exch{"2H17 PLX"}="H172 PLX"; $resn_exch{"1H16 PLX"}="H161 PLX";
$resn_exch{"2H16 PLX"}="H162 PLX"; $resn_exch{"1H15 PLX"}="H151 PLX"; $resn_exch{"2H15 PLX"}="H152 PLX";
$resn_exch{"1H14 PLX"}="H141 PLX"; $resn_exch{"2H14 PLX"}="H142 PLX"; $resn_exch{"1H13 PLX"}="H131 PLX";
$resn_exch{"2H13 PLX"}="H132 PLX"; $resn_exch{"1H12 PLX"}="H121 PLX"; $resn_exch{"2H12 PLX"}="H122 PLX";
$resn_exch{"1H11 PLX"}="H111 PLX"; $resn_exch{"2H11 PLX"}="H112 PLX"; $resn_exch{"1H10 PLX"}="H101 PLX";
$resn_exch{"2H10 PLX"}="H102 PLX"; $resn_exch{"1H9  PLX"}=" H91 PLX"; $resn_exch{"2H9  PLX"}=" H92 PLX";
$resn_exch{"1H8  PLX"}=" H81 PLX"; $resn_exch{"2H8  PLX"}=" H82 PLX"; $resn_exch{"1H7  PLX"}=" H71 PLX";
$resn_exch{"2H7  PLX"}=" H72 PLX"; $resn_exch{"1H3  PLX"}=" H31 PLX"; $resn_exch{"2H3  PLX"}=" H32 PLX";
$resn_exch{"1H2  PLX"}=" H21 PLX"; $resn_exch{"2H2  PLX"}=" H22 PLX"; $resn_exch{"1H1  PLX"}=" H11 PLX";
$resn_exch{"2H1  PLX"}=" H12 PLX"; $resn_exch{"1H1C PLX"}="H1C1 PLX"; $resn_exch{"2H1C PLX"}="H1C2 PLX";
$resn_exch{"3H1C PLX"}="H1C3 PLX"; $resn_exch{"1H1B PLX"}="H1B1 PLX"; $resn_exch{"2H1B PLX"}="H1B2 PLX";
$resn_exch{"3H1B PLX"}="H1B3 PLX"; $resn_exch{"1H1A PLX"}="H1A1 PLX"; $resn_exch{"2H1A PLX"}="H1A2 PLX";
$resn_exch{"3H1A PLX"}="H1A3 PLX"; $resn_exch{"1H5  PLX"}=" H51 PLX"; $resn_exch{"2H5  PLX"}=" H52 PLX";
$resn_exch{"1H25 PLX"}="H251 PLX"; $resn_exch{"2H25 PLX"}="H252 PLX"; $resn_exch{"1H26 PLX"}="H261 PLX";
$resn_exch{"2H26 PLX"}="H262 PLX"; $resn_exch{"1H27 PLX"}="H271 PLX"; $resn_exch{"2H27 PLX"}="H272 PLX";
$resn_exch{"1H28 PLX"}="H281 PLX"; $resn_exch{"2H28 PLX"}="H282 PLX"; $resn_exch{"1H29 PLX"}="H291 PLX";
$resn_exch{"2H29 PLX"}="H292 PLX"; $resn_exch{"1H30 PLX"}="H301 PLX"; $resn_exch{"2H30 PLX"}="H302 PLX";
$resn_exch{"1H31 PLX"}="H311 PLX"; $resn_exch{"2H31 PLX"}="H312 PLX"; $resn_exch{"1H32 PLX"}="H321 PLX";
$resn_exch{"2H32 PLX"}="H322 PLX"; $resn_exch{"1H33 PLX"}="H331 PLX"; $resn_exch{"2H33 PLX"}="H332 PLX";
$resn_exch{"1H34 PLX"}="H341 PLX"; $resn_exch{"2H34 PLX"}="H342 PLX"; $resn_exch{"1H35 PLX"}="H351 PLX";
$resn_exch{"2H35 PLX"}="H352 PLX"; $resn_exch{"1H36 PLX"}="H361 PLX"; $resn_exch{"2H36 PLX"}="H362 PLX";
$resn_exch{"1H37 PLX"}="H371 PLX"; $resn_exch{"2H37 PLX"}="H372 PLX"; $resn_exch{"1H38 PLX"}="H381 PLX";
$resn_exch{"2H38 PLX"}="H382 PLX"; $resn_exch{"1H39 PLX"}="H391 PLX"; $resn_exch{"2H39 PLX"}="H392 PLX";
$resn_exch{"3H39 PLX"}="H393 PLX"; $resn_exch{"1H1  PLY"}=" H11 PLY"; $resn_exch{"1H2  PLY"}=" H21 PLY";
$resn_exch{"2H2  PLY"}=" H22 PLY"; $resn_exch{"1H3  PLY"}=" H31 PLY"; $resn_exch{"2H3  PLY"}=" H32 PLY";
$resn_exch{"1H4  PLY"}=" H41 PLY"; $resn_exch{"2H4  PLY"}=" H42 PLY"; $resn_exch{"1H5  PLY"}=" H51 PLY";
$resn_exch{"2H5  PLY"}=" H52 PLY"; $resn_exch{"1H6  PLY"}=" H61 PLY"; $resn_exch{"2H6  PLY"}=" H62 PLY";
$resn_exch{"1H7  PLY"}=" H71 PLY"; $resn_exch{"2H7  PLY"}=" H72 PLY"; $resn_exch{"1H8  PLY"}=" H81 PLY";
$resn_exch{"2H8  PLY"}=" H82 PLY"; $resn_exch{"1H9  PLY"}=" H91 PLY"; $resn_exch{"2H9  PLY"}=" H92 PLY";
$resn_exch{"1HA  PLY"}=" HA1 PLY"; $resn_exch{"2HA  PLY"}=" HA2 PLY"; $resn_exch{"1HB  PLY"}=" HB1 PLY";
$resn_exch{"2HB  PLY"}=" HB2 PLY"; $resn_exch{"1HC  PLY"}=" HC1 PLY"; $resn_exch{"2HC  PLY"}=" HC2 PLY";
$resn_exch{"1HD  PLY"}=" HD1 PLY"; $resn_exch{"2HD  PLY"}=" HD2 PLY"; $resn_exch{"1HE  PLY"}=" HE1 PLY";
$resn_exch{"2HE  PLY"}=" HE2 PLY"; $resn_exch{"1HF  PLY"}=" HF1 PLY"; $resn_exch{"2HF  PLY"}=" HF2 PLY";
$resn_exch{"1HG  PLY"}=" HG1 PLY"; $resn_exch{"2HG  PLY"}=" HG2 PLY"; $resn_exch{"3HG  PLY"}=" HG3 PLY";
$resn_exch{"1H2A PLZ"}="H2A1 PLZ"; $resn_exch{"2H2A PLZ"}="H2A2 PLZ"; $resn_exch{"3H2A PLZ"}="H2A3 PLZ";
$resn_exch{"1H4A PLZ"}="H4A1 PLZ"; $resn_exch{"2H4A PLZ"}="H4A2 PLZ"; $resn_exch{"1H5  PLZ"}=" H51 PLZ";
$resn_exch{"2H5  PLZ"}=" H52 PLZ"; $resn_exch{"1H6  PLZ"}=" H61 PLZ"; $resn_exch{"2H6  PLZ"}=" H62 PLZ";
$resn_exch{"1H8  PLZ"}=" H81 PLZ"; $resn_exch{"2H8  PLZ"}=" H82 PLZ"; $resn_exch{"1H5M PLZ"}="H5M1 PLZ";
$resn_exch{"2H5M PLZ"}="H5M2 PLZ"; $resn_exch{"1H35 PM1"}="H351 PM1"; $resn_exch{"2H35 PM1"}="H352 PM1";
$resn_exch{"1H7C PM2"}="H7C1 PM2"; $resn_exch{"2H7C PM2"}="H7C2 PM2"; $resn_exch{"1H8N PM2"}="H8N1 PM2";
$resn_exch{"2H8N PM2"}="H8N2 PM2"; $resn_exch{"1H9C PM2"}="H9C1 PM2"; $resn_exch{"2H9C PM2"}="H9C2 PM2";
$resn_exch{"1H10 PM2"}="H101 PM2"; $resn_exch{"2H10 PM2"}="H102 PM2"; $resn_exch{"1H15 PM2"}="H151 PM2";
$resn_exch{"2H15 PM2"}="H152 PM2"; $resn_exch{"1H16 PM2"}="H161 PM2"; $resn_exch{"2H16 PM2"}="H162 PM2";
$resn_exch{"1H22 PM2"}="H221 PM2"; $resn_exch{"2H22 PM2"}="H222 PM2"; $resn_exch{"1H24 PM2"}="H241 PM2";
$resn_exch{"2H24 PM2"}="H242 PM2"; $resn_exch{"1HN  PM3"}=" H   PM3"; $resn_exch{"2HN  PM3"}=" H2  PM3";
$resn_exch{"1HB  PM3"}=" HB2 PM3"; $resn_exch{"2HB  PM3"}=" HB3 PM3"; $resn_exch{"1HH4 PM3"}="HH42 PM3";
$resn_exch{"2HH4 PM3"}="HH43 PM3"; $resn_exch{"1H28 PM4"}="H281 PM4"; $resn_exch{"2H28 PM4"}="H282 PM4";
$resn_exch{"1H30 PM4"}="H301 PM4"; $resn_exch{"2H30 PM4"}="H302 PM4"; $resn_exch{"3H30 PM4"}="H303 PM4";
$resn_exch{"1H31 PM4"}="H311 PM4"; $resn_exch{"2H31 PM4"}="H312 PM4"; $resn_exch{"3H31 PM4"}="H313 PM4";
$resn_exch{"1H37 PM4"}="H371 PM4"; $resn_exch{"2H37 PM4"}="H372 PM4"; $resn_exch{"1H38 PM4"}="H381 PM4";
$resn_exch{"2H38 PM4"}="H382 PM4"; $resn_exch{"1H42 PM4"}="H421 PM4"; $resn_exch{"2H42 PM4"}="H422 PM4";
$resn_exch{"1H43 PM4"}="H431 PM4"; $resn_exch{"2H43 PM4"}="H432 PM4"; $resn_exch{"1H2  PM4"}=" H21 PM4";
$resn_exch{"2H2  PM4"}=" H22 PM4"; $resn_exch{"1H3  PM4"}=" H31 PM4"; $resn_exch{"2H3  PM4"}="H32A PM4";
$resn_exch{"1H4  PM4"}="H41A PM4"; $resn_exch{"2H4  PM4"}=" H42 PM4"; $resn_exch{"1H5  PM4"}=" H51 PM4";
$resn_exch{"2H5  PM4"}=" H52 PM4"; $resn_exch{"1H6  PM4"}=" H61 PM4"; $resn_exch{"2H6  PM4"}=" H62 PM4";
$resn_exch{"3H6  PM4"}=" H63 PM4"; $resn_exch{"1H28 PM5"}="H281 PM5"; $resn_exch{"2H28 PM5"}="H282 PM5";
$resn_exch{"1H30 PM5"}="H301 PM5"; $resn_exch{"2H30 PM5"}="H302 PM5"; $resn_exch{"3H30 PM5"}="H303 PM5";
$resn_exch{"1H31 PM5"}="H311 PM5"; $resn_exch{"2H31 PM5"}="H312 PM5"; $resn_exch{"3H31 PM5"}="H313 PM5";
$resn_exch{"1H37 PM5"}="H371 PM5"; $resn_exch{"2H37 PM5"}="H372 PM5"; $resn_exch{"1H38 PM5"}="H381 PM5";
$resn_exch{"2H38 PM5"}="H382 PM5"; $resn_exch{"1H42 PM5"}="H421 PM5"; $resn_exch{"2H42 PM5"}="H422 PM5";
$resn_exch{"1H43 PM5"}="H431 PM5"; $resn_exch{"2H43 PM5"}="H432 PM5"; $resn_exch{"1H2  PM5"}=" H21 PM5";
$resn_exch{"2H2  PM5"}=" H22 PM5"; $resn_exch{"1H3  PM5"}=" H31 PM5"; $resn_exch{"2H3  PM5"}="H32A PM5";
$resn_exch{"1H4  PM5"}="H41A PM5"; $resn_exch{"2H4  PM5"}=" H42 PM5"; $resn_exch{"1H5  PM5"}=" H51 PM5";
$resn_exch{"2H5  PM5"}=" H52 PM5"; $resn_exch{"1H6  PM5"}=" H61 PM5"; $resn_exch{"2H6  PM5"}=" H62 PM5";
$resn_exch{"1H7  PM5"}=" H71 PM5"; $resn_exch{"2H7  PM5"}=" H72 PM5"; $resn_exch{"3H7  PM5"}=" H73 PM5";
$resn_exch{"1H28 PM8"}="H281 PM8"; $resn_exch{"2H28 PM8"}="H282 PM8"; $resn_exch{"1H30 PM8"}="H301 PM8";
$resn_exch{"2H30 PM8"}="H302 PM8"; $resn_exch{"3H30 PM8"}="H303 PM8"; $resn_exch{"1H31 PM8"}="H311 PM8";
$resn_exch{"2H31 PM8"}="H312 PM8"; $resn_exch{"3H31 PM8"}="H313 PM8"; $resn_exch{"1H37 PM8"}="H371 PM8";
$resn_exch{"2H37 PM8"}="H372 PM8"; $resn_exch{"1H38 PM8"}="H381 PM8"; $resn_exch{"2H38 PM8"}="H382 PM8";
$resn_exch{"1H42 PM8"}="H421 PM8"; $resn_exch{"2H42 PM8"}="H422 PM8"; $resn_exch{"1H43 PM8"}="H431 PM8";
$resn_exch{"2H43 PM8"}="H432 PM8"; $resn_exch{"1H2  PM8"}=" H21 PM8"; $resn_exch{"2H2  PM8"}=" H22 PM8";
$resn_exch{"1H3  PM8"}=" H31 PM8"; $resn_exch{"2H3  PM8"}="H32A PM8"; $resn_exch{"1H4  PM8"}="H41A PM8";
$resn_exch{"2H4  PM8"}=" H42 PM8"; $resn_exch{"1H5  PM8"}=" H51 PM8"; $resn_exch{"2H5  PM8"}=" H52 PM8";
$resn_exch{"1H6  PM8"}=" H61 PM8"; $resn_exch{"2H6  PM8"}=" H62 PM8"; $resn_exch{"1H7  PM8"}=" H71 PM8";
$resn_exch{"2H7  PM8"}=" H72 PM8"; $resn_exch{"1H8  PM8"}=" H81 PM8"; $resn_exch{"2H8  PM8"}=" H82 PM8";
$resn_exch{"1H9  PM8"}=" H91 PM8"; $resn_exch{"2H9  PM8"}=" H92 PM8"; $resn_exch{"1H10 PM8"}="H101 PM8";
$resn_exch{"2H10 PM8"}="H102 PM8"; $resn_exch{"3H10 PM8"}="H103 PM8"; $resn_exch{"1HCA PMC"}="HCA1 PMC";
$resn_exch{"2HCA PMC"}="HCA2 PMC"; $resn_exch{"OHC2 PMC"}="HC2O PMC"; $resn_exch{"1H7* PMD"}="H7'1 PMD";
$resn_exch{"1H9* PMD"}="H9'1 PMD"; $resn_exch{"2H9* PMD"}="H9'2 PMD"; $resn_exch{"1H1' PMD"}="H1'1 PMD";
$resn_exch{"1H2' PMD"}="H2'1 PMD"; $resn_exch{"1H3' PMD"}="H3'1 PMD"; $resn_exch{"1H4' PMD"}="H4'1 PMD";
$resn_exch{"1H6' PMD"}="H6'1 PMD"; $resn_exch{"1H6  PMD"}=" H61 PMD"; $resn_exch{"1H8' PMD"}="H8'1 PMD";
$resn_exch{"1H9' PMD"}=" H9B PMD"; $resn_exch{"'H10 PMD"}="H10' PMD"; $resn_exch{"'H11 PMD"}="H11' PMD";
$resn_exch{"1HN2 PMD"}="HN21 PMD"; $resn_exch{"2HN2 PMD"}="HN22 PMD"; $resn_exch{"1HN4 PMD"}="HN41 PMD";
$resn_exch{"2HN4 PMD"}="HN42 PMD"; $resn_exch{"1HN1 PME"}="HN11 PME"; $resn_exch{"2HN1 PME"}="HN12 PME";
$resn_exch{"1H3  PME"}=" H31 PME"; $resn_exch{"2H3  PME"}=" H32 PME"; $resn_exch{"1H11 PME"}="H111 PME";
$resn_exch{"2H11 PME"}="H112 PME"; $resn_exch{"1H21 PME"}="H211 PME"; $resn_exch{"2H21 PME"}="H212 PME";
$resn_exch{"3H21 PME"}="H213 PME"; $resn_exch{"1H2A PMG"}="H2A1 PMG"; $resn_exch{"2H2A PMG"}="H2A2 PMG";
$resn_exch{"3H2A PMG"}="H2A3 PMG"; $resn_exch{"1H4A PMG"}="H4A1 PMG"; $resn_exch{"2H4A PMG"}="H4A2 PMG";
$resn_exch{"1H5A PMG"}="H5A1 PMG"; $resn_exch{"2H5A PMG"}="H5A2 PMG"; $resn_exch{"1HB2 PMG"}="HB21 PMG";
$resn_exch{"2HB2 PMG"}="HB22 PMG"; $resn_exch{"3HB2 PMG"}="HB23 PMG"; $resn_exch{"1HB1 PMG"}="HB11 PMG";
$resn_exch{"2HB1 PMG"}="HB12 PMG"; $resn_exch{"1HG  PMG"}=" HG1 PMG"; $resn_exch{"2HG  PMG"}=" HG2 PMG";
$resn_exch{"1H2A PMH"}="H2A1 PMH"; $resn_exch{"2H2A PMH"}="H2A2 PMH"; $resn_exch{"3H2A PMH"}="H2A3 PMH";
$resn_exch{"1H4A PMH"}="H4A1 PMH"; $resn_exch{"2H4A PMH"}="H4A2 PMH"; $resn_exch{"1H5A PMH"}="H5A1 PMH";
$resn_exch{"2H5A PMH"}="H5A2 PMH"; $resn_exch{"1H9' PMI"}="H9'1 PMI"; $resn_exch{"2H9' PMI"}="H9'2 PMI";
$resn_exch{"1H11 PMI"}="H111 PMI"; $resn_exch{"2H11 PMI"}="H112 PMI"; $resn_exch{"1H7' PMI"}="H7'1 PMI";
$resn_exch{"2H7' PMI"}="H7'2 PMI"; $resn_exch{"1HC2 PML"}="HC21 PML"; $resn_exch{"2HC2 PML"}="HC22 PML";
$resn_exch{"1HC3 PML"}="HC31 PML"; $resn_exch{"2HC3 PML"}="HC32 PML"; $resn_exch{"1HC4 PML"}="HC41 PML";
$resn_exch{"2HC4 PML"}="HC42 PML"; $resn_exch{"1HC5 PML"}="HC51 PML"; $resn_exch{"2HC5 PML"}="HC52 PML";
$resn_exch{"1HC6 PML"}="HC61 PML"; $resn_exch{"2HC6 PML"}="HC62 PML"; $resn_exch{"1HN2 PMM"}="HN21 PMM";
$resn_exch{"2HN2 PMM"}="HN22 PMM"; $resn_exch{"1H9  PMM"}=" H91 PMM"; $resn_exch{"2H9  PMM"}=" H92 PMM";
$resn_exch{"2HOP PMM"}="HOP2 PMM"; $resn_exch{"3HOP PMM"}="HOP3 PMM"; $resn_exch{"1H2  PMN"}=" H21 PMN";
$resn_exch{"2H2  PMN"}=" H22 PMN"; $resn_exch{"1H3  PMN"}=" H31 PMN"; $resn_exch{"2H3  PMN"}=" H32 PMN";
$resn_exch{"1H11 PMN"}="H111 PMN"; $resn_exch{"2H11 PMN"}="H112 PMN"; $resn_exch{"1H13 PMN"}="H131 PMN";
$resn_exch{"2H13 PMN"}="H132 PMN"; $resn_exch{"1H14 PMN"}="H141 PMN"; $resn_exch{"2H14 PMN"}="H142 PMN";
$resn_exch{"1H17 PMN"}="H171 PMN"; $resn_exch{"2H17 PMN"}="H172 PMN"; $resn_exch{"3H17 PMN"}="H173 PMN";
$resn_exch{"1H18 PMN"}="H181 PMN"; $resn_exch{"2H18 PMN"}="H182 PMN"; $resn_exch{"3H18 PMN"}="H183 PMN";
$resn_exch{"1H2  PMO"}=" H21 PMO"; $resn_exch{"1H4  PMO"}=" H41 PMO"; $resn_exch{"1H6  PMO"}=" H61 PMO";
$resn_exch{"1H7  PMO"}=" H71 PMO"; $resn_exch{"1H11 PMO"}="H111 PMO"; $resn_exch{"2H11 PMO"}="H112 PMO";
$resn_exch{"3H11 PMO"}="H113 PMO"; $resn_exch{"2HOP PMO"}="HOP2 PMO"; $resn_exch{"3HOP PMO"}="HOP3 PMO";
$resn_exch{"1H5* PMO"}="H5'1 PMO"; $resn_exch{"2H5* PMO"}="H5'2 PMO"; $resn_exch{" H4* PMO"}=" H4' PMO";
$resn_exch{" H3* PMO"}=" H3' PMO"; $resn_exch{"*HO3 PMO"}="HO3' PMO"; $resn_exch{" H2* PMO"}=" H2' PMO";
$resn_exch{"*HO2 PMO"}="HO2' PMO"; $resn_exch{" H1* PMO"}=" H1' PMO"; $resn_exch{"1H2A PMP"}="H2A1 PMP";
$resn_exch{"2H2A PMP"}="H2A2 PMP"; $resn_exch{"3H2A PMP"}="H2A3 PMP"; $resn_exch{"1H4A PMP"}="H4A1 PMP";
$resn_exch{"2H4A PMP"}="H4A2 PMP"; $resn_exch{"1HNA PMP"}="HNA1 PMP"; $resn_exch{"2HNA PMP"}="HNA2 PMP";
$resn_exch{"1H5A PMP"}="H5A1 PMP"; $resn_exch{"2H5A PMP"}="H5A2 PMP"; $resn_exch{"2HOP PMP"}="HOP2 PMP";
$resn_exch{"3HOP PMP"}="HOP3 PMP"; $resn_exch{"1H   PMS"}=" H1  PMS"; $resn_exch{"2H   PMS"}=" H2A PMS";
$resn_exch{"2HOS PMS"}="HOS2 PMS"; $resn_exch{"1H7' PMT"}="H7'1 PMT"; $resn_exch{"2H7' PMT"}="H7'2 PMT";
$resn_exch{"1H9' PMT"}="H9'1 PMT"; $resn_exch{"2H9' PMT"}="H9'2 PMT"; $resn_exch{"3H9' PMT"}="H9'3 PMT";
$resn_exch{"1H8' PMT"}="H8'1 PMT"; $resn_exch{"2H8' PMT"}="H8'2 PMT"; $resn_exch{"3H8' PMT"}="H8'3 PMT";
$resn_exch{"1H3' PMT"}="H3'1 PMT"; $resn_exch{"2H3' PMT"}="H3'2 PMT"; $resn_exch{"1H2' PMT"}="H2'1 PMT";
$resn_exch{"2H2' PMT"}="H2'2 PMT"; $resn_exch{"1H5* PMT"}="H5'1 PMT"; $resn_exch{"2H5* PMT"}="H5'' PMT";
$resn_exch{" H4* PMT"}=" H4' PMT"; $resn_exch{" H3* PMT"}=" H3B PMT"; $resn_exch{" H2* PMT"}=" H2' PMT";
$resn_exch{" H1* PMT"}=" H1' PMT"; $resn_exch{"1HN4 PMT"}="HN41 PMT"; $resn_exch{"2HN4 PMT"}="HN42 PMT";
$resn_exch{"1H5  PMV"}=" H51 PMV"; $resn_exch{"2H5  PMV"}=" H52 PMV"; $resn_exch{"1H4  PMV"}=" H41 PMV";
$resn_exch{"2H4  PMV"}=" H42 PMV"; $resn_exch{"1H3A PMV"}="H3A1 PMV"; $resn_exch{"2H3A PMV"}="H3A2 PMV";
$resn_exch{"3H3A PMV"}="H3A3 PMV"; $resn_exch{"1H2  PMV"}=" H21 PMV"; $resn_exch{"2H2  PMV"}=" H22 PMV";
$resn_exch{"1HA1 PMZ"}="HA11 PMZ"; $resn_exch{"2HA1 PMZ"}="HA12 PMZ"; $resn_exch{"1HB1 PMZ"}="HB11 PMZ";
$resn_exch{"2HB1 PMZ"}="HB12 PMZ"; $resn_exch{"1HE1 PMZ"}="HE11 PMZ"; $resn_exch{"2HE1 PMZ"}="HE12 PMZ";
$resn_exch{"3HE1 PMZ"}="HE13 PMZ"; $resn_exch{"1HE2 PMZ"}="HE21 PMZ"; $resn_exch{"2HE2 PMZ"}="HE22 PMZ";
$resn_exch{"3HE2 PMZ"}="HE23 PMZ"; $resn_exch{"1HB2 PMZ"}="HB21 PMZ"; $resn_exch{"2HB2 PMZ"}="HB22 PMZ";
$resn_exch{"3HB2 PMZ"}="HB23 PMZ"; $resn_exch{"1H1C PN1"}="H1C1 PN1"; $resn_exch{"2H1C PN1"}="H1C2 PN1";
$resn_exch{"3H1C PN1"}="H1C3 PN1"; $resn_exch{"1H16 PN1"}="H161 PN1"; $resn_exch{"2H16 PN1"}="H162 PN1";
$resn_exch{"3H16 PN1"}="H163 PN1"; $resn_exch{"1H2N PN1"}="H2N1 PN1"; $resn_exch{"2H2N PN1"}="H2N2 PN1";
$resn_exch{"1HN  PN2"}=" HN1 PN2"; $resn_exch{"2HN  PN2"}=" HN2 PN2"; $resn_exch{"1HCB PN2"}="HCB1 PN2";
$resn_exch{"2HCB PN2"}="HCB2 PN2"; $resn_exch{"6HOP PN2"}="HOP6 PN2"; $resn_exch{"1H12 PN2"}="H121 PN2";
$resn_exch{"2H12 PN2"}="H122 PN2"; $resn_exch{"1H13 PN2"}="H131 PN2"; $resn_exch{"2H13 PN2"}="H132 PN2";
$resn_exch{"3H13 PN2"}="H133 PN2"; $resn_exch{"1H14 PN2"}="H141 PN2"; $resn_exch{"2H14 PN2"}="H142 PN2";
$resn_exch{"3H14 PN2"}="H143 PN2"; $resn_exch{"10H  PN2"}=" H10 PN2"; $resn_exch{"10OH PN2"}="H10O PN2";
$resn_exch{"8HN  PN2"}=" HN8 PN2"; $resn_exch{"1H7  PN2"}=" H71 PN2"; $resn_exch{"2H7  PN2"}=" H72 PN2";
$resn_exch{"1H42 PN2"}="H421 PN2"; $resn_exch{"2H42 PN2"}="H422 PN2"; $resn_exch{"4HN  PN2"}=" HN4 PN2";
$resn_exch{"1H3  PN2"}=" H31 PN2"; $resn_exch{"2H   PN2"}=" H2  PN2"; $resn_exch{"1H47 PN2"}="H471 PN2";
$resn_exch{"2H47 PN2"}="H472 PN2"; $resn_exch{"1H6  PNA"}=" H61 PNA"; $resn_exch{"2H6  PNA"}=" H62 PNA";
$resn_exch{"1H7  PNB"}=" H71 PNB"; $resn_exch{"2H7  PNB"}=" H72 PNB"; $resn_exch{"1H8  PNB"}=" H81 PNB";
$resn_exch{"2H8  PNB"}=" H82 PNB"; $resn_exch{"1H9  PNB"}=" H91 PNB"; $resn_exch{"2H9  PNB"}=" H92 PNB";
$resn_exch{"1H10 PNB"}="H101 PNB"; $resn_exch{"2H10 PNB"}="H102 PNB"; $resn_exch{"1H12 PNB"}="H121 PNB";
$resn_exch{"2H12 PNB"}="H122 PNB"; $resn_exch{"1H7  PNC"}=" H71 PNC"; $resn_exch{"2H7  PNC"}=" H72 PNC";
$resn_exch{"1H8  PNC"}=" H81 PNC"; $resn_exch{"2H8  PNC"}=" H82 PNC"; $resn_exch{"1H9  PNC"}=" H91 PNC";
$resn_exch{"2H9  PNC"}=" H92 PNC"; $resn_exch{"1H10 PNC"}="H101 PNC"; $resn_exch{"2H10 PNC"}="H102 PNC";
$resn_exch{"1H12 PNC"}="H121 PNC"; $resn_exch{"2H12 PNC"}="H122 PNC"; $resn_exch{"1HN1 PND"}="HN11 PND";
$resn_exch{"2HN1 PND"}="HN12 PND"; $resn_exch{"1HC8 PNE"}="HC81 PNE"; $resn_exch{"2HC8 PNE"}="HC82 PNE";
$resn_exch{"1HC9 PNE"}="HC91 PNE"; $resn_exch{"2HC9 PNE"}="HC92 PNE"; $resn_exch{"1H10 PNE"}="H101 PNE";
$resn_exch{"2H10 PNE"}="H102 PNE"; $resn_exch{"1H1L PNE"}="H1L1 PNE"; $resn_exch{"2H1L PNE"}="H1L2 PNE";
$resn_exch{"3H1L PNE"}="H1L3 PNE"; $resn_exch{"1H2  PNF"}=" H21 PNF"; $resn_exch{"1H3  PNF"}=" H31 PNF";
$resn_exch{"1H5  PNF"}=" H51 PNF"; $resn_exch{"1H6  PNF"}=" H61 PNF"; $resn_exch{"1HO2 PNF"}="HO21 PNF";
$resn_exch{"1H8  PNF"}=" H81 PNF"; $resn_exch{"2H8  PNF"}=" H82 PNF"; $resn_exch{"1H9  PNF"}=" H91 PNF";
$resn_exch{"2H9  PNF"}=" H92 PNF"; $resn_exch{"1H10 PNF"}="H101 PNF"; $resn_exch{"2H10 PNF"}="H102 PNF";
$resn_exch{"1HN2 PNF"}="HN21 PNF"; $resn_exch{"1H12 PNF"}="H121 PNF"; $resn_exch{"2H12 PNF"}="H122 PNF";
$resn_exch{"1H13 PNF"}="H131 PNF"; $resn_exch{"2H13 PNF"}="H132 PNF"; $resn_exch{"1H14 PNF"}="H141 PNF";
$resn_exch{"2H14 PNF"}="H142 PNF"; $resn_exch{"1H15 PNF"}="H151 PNF"; $resn_exch{"2H15 PNF"}="H152 PNF";
$resn_exch{"1H16 PNF"}="H161 PNF"; $resn_exch{"2H16 PNF"}="H162 PNF"; $resn_exch{"1HO8 PNF"}="HO81 PNF";
$resn_exch{"1H6  PNG"}=" H61 PNG"; $resn_exch{"2H6  PNG"}=" H62 PNG"; $resn_exch{"1HMA PNI"}="HMA1 PNI";
$resn_exch{"2HMA PNI"}="HMA2 PNI"; $resn_exch{"3HMA PNI"}="HMA3 PNI"; $resn_exch{"1HMB PNI"}="HMB1 PNI";
$resn_exch{"2HMB PNI"}="HMB2 PNI"; $resn_exch{"3HMB PNI"}="HMB3 PNI"; $resn_exch{"1HMC PNI"}="HMC1 PNI";
$resn_exch{"2HMC PNI"}="HMC2 PNI"; $resn_exch{"3HMC PNI"}="HMC3 PNI"; $resn_exch{"1HMD PNI"}="HMD1 PNI";
$resn_exch{"2HMD PNI"}="HMD2 PNI"; $resn_exch{"3HMD PNI"}="HMD3 PNI"; $resn_exch{"1H2  PNL"}=" H21 PNL";
$resn_exch{"2H2  PNL"}=" H22 PNL"; $resn_exch{"1H3  PNL"}=" H31 PNL"; $resn_exch{"2H3  PNL"}=" H32 PNL";
$resn_exch{"3H3  PNL"}=" H33 PNL"; $resn_exch{"1H10 PNM"}="H101 PNM"; $resn_exch{"2H10 PNM"}="H102 PNM";
$resn_exch{"3H10 PNM"}="H103 PNM"; $resn_exch{"1HC9 PNM"}="HC91 PNM"; $resn_exch{"2HC9 PNM"}="HC92 PNM";
$resn_exch{"3HC9 PNM"}="HC93 PNM"; $resn_exch{"1H17 PNM"}="H171 PNM"; $resn_exch{"2H17 PNM"}="H172 PNM";
$resn_exch{"1H10 PNN"}="H101 PNN"; $resn_exch{"2H10 PNN"}="H102 PNN"; $resn_exch{"3H10 PNN"}="H103 PNN";
$resn_exch{"1HC9 PNN"}="HC91 PNN"; $resn_exch{"2HC9 PNN"}="HC92 PNN"; $resn_exch{"3HC9 PNN"}="HC93 PNN";
$resn_exch{"1H17 PNN"}="H171 PNN"; $resn_exch{"2H17 PNN"}="H172 PNN"; $resn_exch{"1HN  PNO"}=" HN1 PNO";
$resn_exch{"2HN  PNO"}=" HN2 PNO"; $resn_exch{"1H   PNO"}=" H1  PNO"; $resn_exch{"2H   PNO"}=" H2  PNO";
$resn_exch{"3H   PNO"}=" H3  PNO"; $resn_exch{"1HM  PNP"}=" HM1 PNP"; $resn_exch{"2HM  PNP"}=" HM2 PNP";
$resn_exch{"3HM  PNP"}=" HM3 PNP"; $resn_exch{"3HOP PNP"}="HOP3 PNP"; $resn_exch{"2HOB PNQ"}="HOB2 PNQ";
$resn_exch{"2HOA PNQ"}="HOA2 PNQ"; $resn_exch{"1HNA PNQ"}="HNA1 PNQ"; $resn_exch{"1HA2 PNQ"}="HA21 PNQ";
$resn_exch{"2HA2 PNQ"}="HA22 PNQ"; $resn_exch{"1HA1 PNQ"}="HA11 PNQ"; $resn_exch{"2HA1 PNQ"}="HA12 PNQ";
$resn_exch{"1HOP PNS"}="HOP1 PNS"; $resn_exch{"2HOP PNS"}="HOP2 PNS"; $resn_exch{"2H28 PNS"}="H282 PNS";
$resn_exch{"1H28 PNS"}="H281 PNS"; $resn_exch{"3H30 PNS"}="H303 PNS"; $resn_exch{"2H30 PNS"}="H302 PNS";
$resn_exch{"1H30 PNS"}="H301 PNS"; $resn_exch{"3H31 PNS"}="H313 PNS"; $resn_exch{"2H31 PNS"}="H312 PNS";
$resn_exch{"1H31 PNS"}="H311 PNS"; $resn_exch{"2H37 PNS"}="H372 PNS"; $resn_exch{"1H37 PNS"}="H371 PNS";
$resn_exch{"2H38 PNS"}="H382 PNS"; $resn_exch{"1H38 PNS"}="H381 PNS"; $resn_exch{"2H42 PNS"}="H422 PNS";
$resn_exch{"1H42 PNS"}="H421 PNS"; $resn_exch{"1H43 PNS"}="H431 PNS"; $resn_exch{"2H43 PNS"}="H432 PNS";
$resn_exch{"1H7  PNT"}=" H71 PNT"; $resn_exch{"2H7  PNT"}=" H72 PNT"; $resn_exch{"1H8  PNT"}=" H81 PNT";
$resn_exch{"2H8  PNT"}=" H82 PNT"; $resn_exch{"1H10 PNT"}="H101 PNT"; $resn_exch{"2H10 PNT"}="H102 PNT";
$resn_exch{"1H7' PNT"}="H7'1 PNT"; $resn_exch{"2H7' PNT"}="H7'2 PNT"; $resn_exch{"1H8' PNT"}="H8'1 PNT";
$resn_exch{"2H8' PNT"}="H8'2 PNT"; $resn_exch{"1HN  PNT"}=" HN1 PNT"; $resn_exch{"2HN  PNT"}=" HN2 PNT";
$resn_exch{"3HN  PNT"}=" HN3 PNT"; $resn_exch{"1HN' PNT"}="HN'1 PNT"; $resn_exch{"2HN' PNT"}="HN'2 PNT";
$resn_exch{"3HN' PNT"}="HN'3 PNT"; $resn_exch{"CH11 PNU"}="H11C PNU"; $resn_exch{"CH13 PNU"}="H13C PNU";
$resn_exch{"CH15 PNU"}="H15C PNU"; $resn_exch{"CH16 PNU"}="H16C PNU"; $resn_exch{"1H18 PNU"}="H181 PNU";
$resn_exch{"2H18 PNU"}="H182 PNU"; $resn_exch{"1H21 PNU"}="H211 PNU"; $resn_exch{"2H21 PNU"}="H212 PNU";
$resn_exch{"3H21 PNU"}="H213 PNU"; $resn_exch{"1H2  PNV"}="H21A PNV"; $resn_exch{"2H2  PNV"}="H22A PNV";
$resn_exch{"1H12 PNV"}="H121 PNV"; $resn_exch{"2H12 PNV"}="H122 PNV"; $resn_exch{"3H12 PNV"}="H123 PNV";
$resn_exch{"1H15 PNV"}="H151 PNV"; $resn_exch{"2H15 PNV"}="H152 PNV"; $resn_exch{"3H15 PNV"}="H153 PNV";
$resn_exch{"1HAA PNX"}="HAA1 PNX"; $resn_exch{"2HAA PNX"}="HAA2 PNX"; $resn_exch{"3HAA PNX"}="HAA3 PNX";
$resn_exch{"1HAJ PNX"}="HAJ1 PNX"; $resn_exch{"2HAJ PNX"}="HAJ2 PNX"; $resn_exch{"1HAH PNX"}="HAH1 PNX";
$resn_exch{"2HAH PNX"}="HAH2 PNX"; $resn_exch{"1HAI PNX"}="HAI1 PNX"; $resn_exch{"2HAI PNX"}="HAI2 PNX";
$resn_exch{"1HAK PNX"}="HAK1 PNX"; $resn_exch{"2HAK PNX"}="HAK2 PNX"; $resn_exch{"1HAB PNX"}="HAB1 PNX";
$resn_exch{"2HAB PNX"}="HAB2 PNX"; $resn_exch{"3HAB PNX"}="HAB3 PNX"; $resn_exch{"1HAC PNX"}="HAC1 PNX";
$resn_exch{"2HAC PNX"}="HAC2 PNX"; $resn_exch{"3HAC PNX"}="HAC3 PNX"; $resn_exch{"1H2N PNZ"}="H2N1 PNZ";
$resn_exch{"2H2N PNZ"}="H2N2 PNZ"; $resn_exch{"1H3C PNZ"}="H3C1 PNZ"; $resn_exch{"2H3C PNZ"}="H3C2 PNZ";
$resn_exch{"1HC  PO0"}=" HC1 PO0"; $resn_exch{"2HC  PO0"}=" HC2 PO0"; $resn_exch{"1HB  PO0"}=" HB1 PO0";
$resn_exch{"2HB  PO0"}=" HB2 PO0"; $resn_exch{"1HN1 PO1"}="HN11 PO1"; $resn_exch{"2HN1 PO1"}="HN12 PO1";
$resn_exch{"1H7  PO1"}=" H71 PO1"; $resn_exch{"2H7  PO1"}=" H72 PO1"; $resn_exch{"3H7  PO1"}=" H73 PO1";
$resn_exch{"1H10 PO1"}="H101 PO1"; $resn_exch{"2H10 PO1"}="H102 PO1"; $resn_exch{"1H11 PO1"}="H111 PO1";
$resn_exch{"2H11 PO1"}="H112 PO1"; $resn_exch{"1H12 PO1"}="H121 PO1"; $resn_exch{"2H12 PO1"}="H122 PO1";
$resn_exch{"1H13 PO1"}="H131 PO1"; $resn_exch{"2H13 PO1"}="H132 PO1"; $resn_exch{"1H16 PO1"}="H161 PO1";
$resn_exch{"2H16 PO1"}="H162 PO1"; $resn_exch{"3H16 PO1"}="H163 PO1"; $resn_exch{"1H18 PO1"}="H181 PO1";
$resn_exch{"2H18 PO1"}="H182 PO1"; $resn_exch{"1H19 PO1"}="H191 PO1"; $resn_exch{"2H19 PO1"}="H192 PO1";
$resn_exch{"1HN2 PO1"}="HN21 PO1"; $resn_exch{"2HN2 PO1"}="HN22 PO1"; $resn_exch{"1H1  POA"}=" H11 POA";
$resn_exch{"2H1  POA"}=" H12 POA"; $resn_exch{"1H3  POB"}=" H31 POB"; $resn_exch{"2H3  POB"}=" H32 POB";
$resn_exch{"1H4  POB"}=" H41 POB"; $resn_exch{"2H4  POB"}=" H42 POB"; $resn_exch{"3H4  POB"}=" H43 POB";
$resn_exch{"5HOP POB"}="HOP5 POB"; $resn_exch{"6HOP POB"}="HOP6 POB"; $resn_exch{"1HN1 POB"}="HN11 POB";
$resn_exch{"2HN1 POB"}="HN12 POB"; $resn_exch{"1H2' POB"}="H2'1 POB"; $resn_exch{"2H2' POB"}="H2'2 POB";
$resn_exch{"3H2' POB"}="H2'3 POB"; $resn_exch{"1H4  POC"}=" H41 POC"; $resn_exch{"2H4  POC"}=" H42 POC";
$resn_exch{"1H5  POC"}=" H51 POC"; $resn_exch{"2H5  POC"}=" H52 POC"; $resn_exch{"1H6  POC"}=" H61 POC";
$resn_exch{"2H6  POC"}=" H62 POC"; $resn_exch{"3H6  POC"}=" H63 POC"; $resn_exch{"1H7  POC"}=" H71 POC";
$resn_exch{"2H7  POC"}=" H72 POC"; $resn_exch{"3H7  POC"}=" H73 POC"; $resn_exch{"1H8  POC"}=" H81 POC";
$resn_exch{"2H8  POC"}=" H82 POC"; $resn_exch{"3H8  POC"}=" H83 POC"; $resn_exch{"1H12 POD"}="H121 POD";
$resn_exch{"2H12 POD"}="H122 POD"; $resn_exch{"1H1  POD"}="H11A POD"; $resn_exch{"2H1  POD"}=" H12 POD";
$resn_exch{"1H20 POD"}="H201 POD"; $resn_exch{"2H20 POD"}="H202 POD"; $resn_exch{"3H20 POD"}="H203 POD";
$resn_exch{"1H21 POD"}="H211 POD"; $resn_exch{"2H21 POD"}="H212 POD"; $resn_exch{"3H21 POD"}="H213 POD";
$resn_exch{"1H22 POD"}="H221 POD"; $resn_exch{"2H22 POD"}="H222 POD"; $resn_exch{"3H22 POD"}="H223 POD";
$resn_exch{"1H1  POG"}=" H11 POG"; $resn_exch{"2H1  POG"}="H12A POG"; $resn_exch{"3H1  POG"}=" H13 POG";
$resn_exch{"1H3  POG"}=" H31 POG"; $resn_exch{"2H3  POG"}=" H32 POG"; $resn_exch{"1H4  POG"}=" H41 POG";
$resn_exch{"2H4  POG"}=" H42 POG"; $resn_exch{"1H7  POG"}=" H71 POG"; $resn_exch{"2H7  POG"}=" H72 POG";
$resn_exch{"1H9  POG"}=" H91 POG"; $resn_exch{"2H9  POG"}=" H92 POG"; $resn_exch{"1H11 POG"}="H111 POG";
$resn_exch{"2H11 POG"}="H112 POG"; $resn_exch{"1H13 POG"}="H131 POG"; $resn_exch{"2H13 POG"}="H132 POG";
$resn_exch{"1H15 POG"}="H151 POG"; $resn_exch{"2H15 POG"}="H152 POG"; $resn_exch{"1H16 POG"}="H161 POG";
$resn_exch{"2H16 POG"}="H162 POG"; $resn_exch{"3H16 POG"}="H163 POG"; $resn_exch{"1H17 POG"}="H171 POG";
$resn_exch{"2H17 POG"}="H172 POG"; $resn_exch{"3H17 POG"}="H173 POG"; $resn_exch{"1H18 POG"}="H181 POG";
$resn_exch{"2H18 POG"}="H182 POG"; $resn_exch{"3H18 POG"}="H183 POG"; $resn_exch{"1H19 POG"}="H191 POG";
$resn_exch{"2H19 POG"}="H192 POG"; $resn_exch{"3H19 POG"}="H193 POG"; $resn_exch{"1H20 POG"}="H201 POG";
$resn_exch{"2H20 POG"}="H202 POG"; $resn_exch{"3H20 POG"}="H203 POG"; $resn_exch{"1H21 POG"}="H211 POG";
$resn_exch{"2H21 POG"}="H212 POG"; $resn_exch{"3H21 POG"}="H213 POG"; $resn_exch{"1H5  POH"}=" H51 POH";
$resn_exch{"2H11 POH"}="H112 POH"; $resn_exch{"1H1  POH"}=" H11 POH"; $resn_exch{"1H3  POH"}=" H31 POH";
$resn_exch{"1H4  POH"}=" H41 POH"; $resn_exch{"2H19 POH"}="H192 POH"; $resn_exch{"1H81 POH"}="H811 POH";
$resn_exch{"1H82 POH"}="H821 POH"; $resn_exch{"1H83 POH"}="H831 POH"; $resn_exch{"1H2  POH"}=" H21 POH";
$resn_exch{"1H6  POH"}=" H61 POH"; $resn_exch{"1H7  POH"}=" H71 POH"; $resn_exch{"3H7  POH"}=" H73 POH";
$resn_exch{"2H5  POH"}=" H52 POH"; $resn_exch{"1H21 POH"}="H211 POH"; $resn_exch{"2H1  POH"}=" H12 POH";
$resn_exch{"2H27 POH"}="H272 POH"; $resn_exch{"2H3  POH"}=" H32 POH"; $resn_exch{"1H28 POH"}="H281 POH";
$resn_exch{"2H4  POH"}=" H42 POH"; $resn_exch{"1H29 POH"}="H291 POH"; $resn_exch{"2H81 POH"}="H812 POH";
$resn_exch{"2H82 POH"}="H822 POH"; $resn_exch{"2H83 POH"}="H832 POH"; $resn_exch{"2H2  POH"}=" H22 POH";
$resn_exch{"2H6  POH"}=" H62 POH"; $resn_exch{"3H5  POH"}=" H53 POH"; $resn_exch{"1H31 POH"}="H311 POH";
$resn_exch{"3H1  POH"}=" H13 POH"; $resn_exch{"3H37 POH"}="H373 POH"; $resn_exch{"3H3  POH"}=" H33 POH";
$resn_exch{"1H38 POH"}="H381 POH"; $resn_exch{"3H4  POH"}=" H43 POH"; $resn_exch{"1H39 POH"}="H391 POH";
$resn_exch{"3H81 POH"}="H813 POH"; $resn_exch{"3H82 POH"}="H823 POH"; $resn_exch{"3H83 POH"}="H833 POH";
$resn_exch{"3H2  POH"}=" H23 POH"; $resn_exch{"3H6  POH"}=" H63 POH"; $resn_exch{"4H5  POH"}=" H54 POH";
$resn_exch{"1H41 POH"}="H411 POH"; $resn_exch{"4H1  POH"}=" H14 POH"; $resn_exch{"4H47 POH"}="H474 POH";
$resn_exch{"4H3  POH"}=" H34 POH"; $resn_exch{"1H48 POH"}="H481 POH"; $resn_exch{"4H4  POH"}=" H44 POH";
$resn_exch{"1H49 POH"}="H491 POH"; $resn_exch{"4H81 POH"}="H814 POH"; $resn_exch{"4H82 POH"}="H824 POH";
$resn_exch{"4H83 POH"}="H834 POH"; $resn_exch{"4H2  POH"}=" H24 POH"; $resn_exch{"4H6  POH"}=" H64 POH";
$resn_exch{"1HM  POI"}=" HM1 POI"; $resn_exch{"2HM  POI"}=" HM2 POI"; $resn_exch{"3HM  POI"}=" HM3 POI";
$resn_exch{"1HB  POI"}=" HB1 POI"; $resn_exch{"2HB  POI"}=" HB2 POI"; $resn_exch{"1HG  POI"}=" HG1 POI";
$resn_exch{"2HG  POI"}=" HG2 POI"; $resn_exch{"1HD  POI"}=" HD1 POI"; $resn_exch{"2HD  POI"}=" HD2 POI";
$resn_exch{"1H4A POI"}="H4A1 POI"; $resn_exch{"2H4A POI"}="H4A2 POI"; $resn_exch{"1H2A POI"}="H2A1 POI";
$resn_exch{"2H2A POI"}="H2A2 POI"; $resn_exch{"3H2A POI"}="H2A3 POI"; $resn_exch{"1H5A POI"}="H5A1 POI";
$resn_exch{"2H5A POI"}="H5A2 POI"; $resn_exch{"1H1  POL"}=" H11 POL"; $resn_exch{"2H1  POL"}=" H12 POL";
$resn_exch{"1H2  POL"}=" H21 POL"; $resn_exch{"2H2  POL"}=" H22 POL"; $resn_exch{"1H3  POL"}=" H31 POL";
$resn_exch{"2H3  POL"}=" H32 POL"; $resn_exch{"3H3  POL"}=" H33 POL"; $resn_exch{" HN  POM"}=" H   POM";
$resn_exch{"1HB  POM"}=" HB2 POM"; $resn_exch{"2HB  POM"}=" HB3 POM"; $resn_exch{"1HD1 POM"}="HD11 POM";
$resn_exch{"2HD1 POM"}="HD12 POM"; $resn_exch{"3HD1 POM"}="HD13 POM"; $resn_exch{"1H28 POO"}="H281 POO";
$resn_exch{"2H28 POO"}="H282 POO"; $resn_exch{"1H29 POO"}="H291 POO"; $resn_exch{"2H29 POO"}="H292 POO";
$resn_exch{"1H30 POO"}="H301 POO"; $resn_exch{"2H30 POO"}="H302 POO"; $resn_exch{"1H31 POO"}="H311 POO";
$resn_exch{"2H31 POO"}="H312 POO"; $resn_exch{"3H31 POO"}="H313 POO"; $resn_exch{"1H27 POO"}="H271 POO";
$resn_exch{"2H27 POO"}="H272 POO"; $resn_exch{"1H25 POO"}="H251 POO"; $resn_exch{"2H25 POO"}="H252 POO";
$resn_exch{"1H24 POO"}="H241 POO"; $resn_exch{"2H24 POO"}="H242 POO"; $resn_exch{"3H24 POO"}="H243 POO";
$resn_exch{"1H22 POO"}="H221 POO"; $resn_exch{"2H22 POO"}="H222 POO"; $resn_exch{"1H17 POO"}="H171 POO";
$resn_exch{"2H17 POO"}="H172 POO"; $resn_exch{"1H20 POO"}="H201 POO"; $resn_exch{"2H20 POO"}="H202 POO";
$resn_exch{"1H21 POO"}="H211 POO"; $resn_exch{"2H21 POO"}="H212 POO"; $resn_exch{"1H19 POO"}="H191 POO";
$resn_exch{"2H19 POO"}="H192 POO"; $resn_exch{"1H18 POO"}="H181 POO"; $resn_exch{"2H18 POO"}="H182 POO";
$resn_exch{"1H6  POQ"}=" H61 POQ"; $resn_exch{"2H6  POQ"}=" H62 POQ"; $resn_exch{"1H15 POQ"}="H151 POQ";
$resn_exch{"2H15 POQ"}="H152 POQ"; $resn_exch{"1H16 POQ"}="H161 POQ"; $resn_exch{"2H16 POQ"}="H162 POQ";
$resn_exch{"3H16 POQ"}="H163 POQ"; $resn_exch{"1H6B POQ"}="H6B1 POQ"; $resn_exch{"2H6B POQ"}="H6B2 POQ";
$resn_exch{"1H25 POQ"}="H251 POQ"; $resn_exch{"2H25 POQ"}="H252 POQ"; $resn_exch{"1H26 POQ"}="H261 POQ";
$resn_exch{"2H26 POQ"}="H262 POQ"; $resn_exch{"3H26 POQ"}="H263 POQ"; $resn_exch{"1H18 POS"}="H181 POS";
$resn_exch{"2H18 POS"}="H182 POS"; $resn_exch{"1H21 POS"}="H211 POS"; $resn_exch{"2H21 POS"}="H212 POS";
$resn_exch{"1H26 POS"}="H261 POS"; $resn_exch{"2H26 POS"}="H262 POS"; $resn_exch{"1H28 POS"}="H281 POS";
$resn_exch{"2H28 POS"}="H282 POS"; $resn_exch{"3H28 POS"}="H283 POS"; $resn_exch{"1H29 POS"}="H291 POS";
$resn_exch{"2H29 POS"}="H292 POS"; $resn_exch{"3H29 POS"}="H293 POS"; $resn_exch{"1H4  POT"}=" H41 POT";
$resn_exch{"2H4  POT"}=" H42 POT"; $resn_exch{"3H4  POT"}=" H43 POT"; $resn_exch{"1H3  POT"}=" H31 POT";
$resn_exch{"2H3  POT"}=" H32 POT"; $resn_exch{"1H5  POT"}=" H51 POT"; $resn_exch{"2H5  POT"}=" H52 POT";
$resn_exch{"1H1  POT"}="H11A POT"; $resn_exch{"2H1  POT"}=" H12 POT"; $resn_exch{"3H1  POT"}=" H13 POT";
$resn_exch{"1H10 PP1"}="H101 PP1"; $resn_exch{"2H10 PP1"}="H102 PP1"; $resn_exch{"1H24 PP1"}="H241 PP1";
$resn_exch{"2H24 PP1"}="H242 PP1"; $resn_exch{"3H24 PP1"}="H243 PP1"; $resn_exch{"1H29 PP1"}="H291 PP1";
$resn_exch{"2H29 PP1"}="H292 PP1"; $resn_exch{"3H29 PP1"}="H293 PP1"; $resn_exch{"1H33 PP1"}="H331 PP1";
$resn_exch{"2H33 PP1"}="H332 PP1"; $resn_exch{"3H33 PP1"}="H333 PP1"; $resn_exch{"1H37 PP1"}="H371 PP1";
$resn_exch{"2H37 PP1"}="H372 PP1"; $resn_exch{"3H37 PP1"}="H373 PP1"; $resn_exch{"1H2A PP3"}="H2A1 PP3";
$resn_exch{"2H2A PP3"}="H2A2 PP3"; $resn_exch{"3H2A PP3"}="H2A3 PP3"; $resn_exch{"1H4A PP3"}="H4A1 PP3";
$resn_exch{"2H4A PP3"}="H4A2 PP3"; $resn_exch{"1H5A PP3"}="H5A1 PP3"; $resn_exch{"2H5A PP3"}="H5A2 PP3";
$resn_exch{"2HOP PP3"}="HOP2 PP3"; $resn_exch{"3HOP PP3"}="HOP3 PP3"; $resn_exch{"1HB  PP3"}=" HB1 PP3";
$resn_exch{"2HB  PP3"}=" HB2 PP3"; $resn_exch{"3HB  PP3"}=" HB3 PP3"; $resn_exch{"1H13 PP4"}="H131 PP4";
$resn_exch{"2H13 PP4"}="H132 PP4"; $resn_exch{"1HV1 PP4"}="HV11 PP4"; $resn_exch{"2HV1 PP4"}="HV12 PP4";
$resn_exch{"3HV1 PP4"}="HV13 PP4"; $resn_exch{"1HV2 PP4"}="HV21 PP4"; $resn_exch{"2HV2 PP4"}="HV22 PP4";
$resn_exch{"3HV2 PP4"}="HV23 PP4"; $resn_exch{"1HB  PP4"}=" HB1 PP4"; $resn_exch{"2HB  PP4"}=" HB2 PP4";
$resn_exch{"1HS  PP4"}=" HS1 PP4"; $resn_exch{"2HS  PP4"}=" HS2 PP4"; $resn_exch{"3HS  PP4"}=" HS3 PP4";
$resn_exch{"1H11 PP5"}="H111 PP5"; $resn_exch{"2H11 PP5"}="H112 PP5"; $resn_exch{"1H14 PP5"}="H141 PP5";
$resn_exch{"2H14 PP5"}="H142 PP5"; $resn_exch{"1HV1 PP5"}="HV11 PP5"; $resn_exch{"2HV1 PP5"}="HV12 PP5";
$resn_exch{"3HV1 PP5"}="HV13 PP5"; $resn_exch{"1HV2 PP5"}="HV21 PP5"; $resn_exch{"2HV2 PP5"}="HV22 PP5";
$resn_exch{"3HV2 PP5"}="HV23 PP5"; $resn_exch{"1HB  PP5"}=" HB1 PP5"; $resn_exch{"2HB  PP5"}=" HB2 PP5";
$resn_exch{"1HS  PP5"}=" HS1 PP5"; $resn_exch{"2HS  PP5"}=" HS2 PP5"; $resn_exch{"3HS  PP5"}=" HS3 PP5";
$resn_exch{"1H11 PP6"}="H111 PP6"; $resn_exch{"2H11 PP6"}="H112 PP6"; $resn_exch{"1H13 PP6"}="H131 PP6";
$resn_exch{"2H13 PP6"}="H132 PP6"; $resn_exch{"1HV1 PP6"}="HV11 PP6"; $resn_exch{"2HV1 PP6"}="HV12 PP6";
$resn_exch{"3HV1 PP6"}="HV13 PP6"; $resn_exch{"1HV2 PP6"}="HV21 PP6"; $resn_exch{"2HV2 PP6"}="HV22 PP6";
$resn_exch{"3HV2 PP6"}="HV23 PP6"; $resn_exch{"1HB  PP6"}=" HB1 PP6"; $resn_exch{"2HB  PP6"}=" HB2 PP6";
$resn_exch{"1HS  PP6"}=" HS1 PP6"; $resn_exch{"2HS  PP6"}=" HS2 PP6"; $resn_exch{"3HS  PP6"}=" HS3 PP6";
$resn_exch{"1HV1 PP7"}="HV11 PP7"; $resn_exch{"2HV1 PP7"}="HV12 PP7"; $resn_exch{"1HV3 PP7"}="HV31 PP7";
$resn_exch{"2HV3 PP7"}="HV32 PP7"; $resn_exch{"3HV3 PP7"}="HV33 PP7"; $resn_exch{"1HV8 PP7"}="HV81 PP7";
$resn_exch{"2HV8 PP7"}="HV82 PP7"; $resn_exch{"3HV8 PP7"}="HV83 PP7"; $resn_exch{"1HV4 PP7"}="HV41 PP7";
$resn_exch{"3HV4 PP7"}="HV43 PP7"; $resn_exch{"2HV4 PP7"}="HV42 PP7"; $resn_exch{"1HV9 PP7"}="HV91 PP7";
$resn_exch{"2HV9 PP7"}="HV92 PP7"; $resn_exch{"3HV9 PP7"}="HV93 PP7"; $resn_exch{"1HC2 PP7"}="HC21 PP7";
$resn_exch{"2HC2 PP7"}="HC22 PP7"; $resn_exch{"1HCB PP7"}="HCB1 PP7"; $resn_exch{"2HCB PP7"}="HCB2 PP7";
$resn_exch{"1HBL PP7"}="HBL1 PP7"; $resn_exch{"2HBL PP7"}="HBL2 PP7"; $resn_exch{"1HL1 PP7"}="HL11 PP7";
$resn_exch{"3HL1 PP7"}="HL13 PP7"; $resn_exch{"2HL1 PP7"}="HL12 PP7"; $resn_exch{"1HL2 PP7"}="HL21 PP7";
$resn_exch{"3HL2 PP7"}="HL23 PP7"; $resn_exch{"2HL2 PP7"}="HL22 PP7"; $resn_exch{"1HC6 PP7"}="HC61 PP7";
$resn_exch{"2HC6 PP7"}="HC62 PP7"; $resn_exch{"2HNV PP8"}="HNV2 PP8"; $resn_exch{"1HV1 PP8"}="HV11 PP8";
$resn_exch{"2HV1 PP8"}="HV12 PP8"; $resn_exch{"1HV3 PP8"}="HV31 PP8"; $resn_exch{"2HV3 PP8"}="HV32 PP8";
$resn_exch{"3HV3 PP8"}="HV33 PP8"; $resn_exch{"1HV8 PP8"}="HV81 PP8"; $resn_exch{"2HV8 PP8"}="HV82 PP8";
$resn_exch{"3HV8 PP8"}="HV83 PP8"; $resn_exch{"1HV4 PP8"}="HV41 PP8"; $resn_exch{"2HV4 PP8"}="HV42 PP8";
$resn_exch{"3HV4 PP8"}="HV43 PP8"; $resn_exch{"1HV9 PP8"}="HV91 PP8"; $resn_exch{"2HV9 PP8"}="HV92 PP8";
$resn_exch{"3HV9 PP8"}="HV93 PP8"; $resn_exch{"1H2  PP8"}=" H21 PP8"; $resn_exch{"2H2  PP8"}=" H22 PP8";
$resn_exch{"1HN2 PP8"}="HN21 PP8"; $resn_exch{"2HN2 PP8"}="HN22 PP8"; $resn_exch{"1HBL PP8"}="HBL1 PP8";
$resn_exch{"2HBL PP8"}="HBL2 PP8"; $resn_exch{"1HL1 PP8"}="HL11 PP8"; $resn_exch{"2HL1 PP8"}="HL12 PP8";
$resn_exch{"3HL1 PP8"}="HL13 PP8"; $resn_exch{"1HL2 PP8"}="HL21 PP8"; $resn_exch{"2HL2 PP8"}="HL22 PP8";
$resn_exch{"3HL2 PP8"}="HL23 PP8"; $resn_exch{"1H6  PP8"}=" H61 PP8"; $resn_exch{"2H6  PP8"}=" H62 PP8";
$resn_exch{"1H7  PP8"}=" H71 PP8"; $resn_exch{"2H7  PP8"}=" H72 PP8"; $resn_exch{"3H7  PP8"}=" H73 PP8";
$resn_exch{"1HMA PP9"}="HMA1 PP9"; $resn_exch{"2HMA PP9"}="HMA2 PP9"; $resn_exch{"3HMA PP9"}="HMA3 PP9";
$resn_exch{"1HAA PP9"}="HAA1 PP9"; $resn_exch{"2HAA PP9"}="HAA2 PP9"; $resn_exch{"1HBA PP9"}="HBA1 PP9";
$resn_exch{"2HBA PP9"}="HBA2 PP9"; $resn_exch{"1HMB PP9"}="HMB1 PP9"; $resn_exch{"2HMB PP9"}="HMB2 PP9";
$resn_exch{"3HMB PP9"}="HMB3 PP9"; $resn_exch{"1HBB PP9"}="HBB1 PP9"; $resn_exch{"2HBB PP9"}="HBB2 PP9";
$resn_exch{"1HMC PP9"}="HMC1 PP9"; $resn_exch{"2HMC PP9"}="HMC2 PP9"; $resn_exch{"3HMC PP9"}="HMC3 PP9";
$resn_exch{"1HBC PP9"}="HBC1 PP9"; $resn_exch{"2HBC PP9"}="HBC2 PP9"; $resn_exch{"1HMD PP9"}="HMD1 PP9";
$resn_exch{"2HMD PP9"}="HMD2 PP9"; $resn_exch{"3HMD PP9"}="HMD3 PP9"; $resn_exch{"1HAD PP9"}="HAD1 PP9";
$resn_exch{"2HAD PP9"}="HAD2 PP9"; $resn_exch{"1HBD PP9"}="HBD1 PP9"; $resn_exch{"2HBD PP9"}="HBD2 PP9";
$resn_exch{"1H3  PPB"}=" H31 PPB"; $resn_exch{"2H3  PPB"}=" H32 PPB"; $resn_exch{"1H9  PPB"}=" H91 PPB";
$resn_exch{"2H9  PPB"}=" H92 PPB"; $resn_exch{"1H10 PPB"}="H101 PPB"; $resn_exch{"2H10 PPB"}="H102 PPB";
$resn_exch{"1H11 PPB"}="H111 PPB"; $resn_exch{"2H11 PPB"}="H112 PPB"; $resn_exch{"1H13 PPB"}="H131 PPB";
$resn_exch{"2H13 PPB"}="H132 PPB"; $resn_exch{"1HN6 PPB"}="HN61 PPB"; $resn_exch{"2HN6 PPB"}="HN62 PPB";
$resn_exch{"1H5  PPC"}=" H51 PPC"; $resn_exch{"2H5  PPC"}=" H52 PPC"; $resn_exch{"PHO1 PPC"}="HO1P PPC";
$resn_exch{"PHO2 PPC"}="HO2P PPC"; $resn_exch{"AHO1 PPC"}="HO1A PPC"; $resn_exch{"BHO1 PPC"}="HO1B PPC";
$resn_exch{"BHO3 PPC"}="HO3B PPC"; $resn_exch{"1H3A PPC"}="H3A1 PPC"; $resn_exch{"2H3A PPC"}="H3A2 PPC";
$resn_exch{"1H2A PPD"}="H2A1 PPD"; $resn_exch{"2H2A PPD"}="H2A2 PPD"; $resn_exch{"3H2A PPD"}="H2A3 PPD";
$resn_exch{"1H4A PPD"}="H4A1 PPD"; $resn_exch{"2H4A PPD"}="H4A2 PPD"; $resn_exch{"1H5A PPD"}="H5A1 PPD";
$resn_exch{"2H5A PPD"}="H5A2 PPD"; $resn_exch{"2HOP PPD"}="HOP2 PPD"; $resn_exch{"3HOP PPD"}="HOP3 PPD";
$resn_exch{"1HB  PPD"}=" HB1 PPD"; $resn_exch{"2HB  PPD"}=" HB2 PPD"; $resn_exch{"1H2A PPE"}="H2A1 PPE";
$resn_exch{"2H2A PPE"}="H2A2 PPE"; $resn_exch{"3H2A PPE"}="H2A3 PPE"; $resn_exch{"1H4A PPE"}="H4A1 PPE";
$resn_exch{"2H4A PPE"}="H4A2 PPE"; $resn_exch{"1H5A PPE"}="H5A1 PPE"; $resn_exch{"2H5A PPE"}="H5A2 PPE";
$resn_exch{"2HOP PPE"}="HOP2 PPE"; $resn_exch{"3HOP PPE"}="HOP3 PPE"; $resn_exch{"1HBA PPE"}="HBA1 PPE";
$resn_exch{"2HBA PPE"}="HBA2 PPE"; $resn_exch{"1HGA PPE"}="HGA1 PPE"; $resn_exch{"2HGA PPE"}="HGA2 PPE";
$resn_exch{"2HOE PPE"}="HOE2 PPE"; $resn_exch{"1H2A PPG"}="H2A1 PPG"; $resn_exch{"2H2A PPG"}="H2A2 PPG";
$resn_exch{"3H2A PPG"}="H2A3 PPG"; $resn_exch{"1H4A PPG"}="H4A1 PPG"; $resn_exch{"2H4A PPG"}="H4A2 PPG";
$resn_exch{"1H5A PPG"}="H5A1 PPG"; $resn_exch{"2H5A PPG"}="H5A2 PPG"; $resn_exch{"2HOP PPG"}="HOP2 PPG";
$resn_exch{"3HOP PPG"}="HOP3 PPG"; $resn_exch{"3HOB PPG"}="HOB3 PPG"; $resn_exch{"1HEI PPG"}="HEI1 PPG";
$resn_exch{"2HEI PPG"}="HEI2 PPG"; $resn_exch{"1HFI PPG"}="HFI1 PPG"; $resn_exch{"2HFI PPG"}="HFI2 PPG";
$resn_exch{"1HNI PPG"}="HNI1 PPG"; $resn_exch{"2HNI PPG"}="HNI2 PPG"; $resn_exch{"1HN  PPH"}=" HN1 PPH";
$resn_exch{"2HN  PPH"}=" HN2 PPH"; $resn_exch{"2HOP PPH"}="HOP2 PPH"; $resn_exch{"1HB  PPH"}=" HB1 PPH";
$resn_exch{"2HB  PPH"}=" HB2 PPH"; $resn_exch{"1H2  PPI"}=" H21 PPI"; $resn_exch{"2H2  PPI"}=" H22 PPI";
$resn_exch{"1H3  PPI"}=" H31 PPI"; $resn_exch{"2H3  PPI"}=" H32 PPI"; $resn_exch{"3H3  PPI"}=" H33 PPI";
$resn_exch{"1H2A PPJ"}="H2A1 PPJ"; $resn_exch{"2H2A PPJ"}="H2A2 PPJ"; $resn_exch{"3H2A PPJ"}="H2A3 PPJ";
$resn_exch{"1H5A PPJ"}="H5A1 PPJ"; $resn_exch{"2H5A PPJ"}="H5A2 PPJ"; $resn_exch{"1HB  PPJ"}=" HB1 PPJ";
$resn_exch{"1HC  PPJ"}=" HC1 PPJ"; $resn_exch{"2HC  PPJ"}=" HC2 PPJ"; $resn_exch{"3HC  PPJ"}=" HC3 PPJ";
$resn_exch{"1H3  PPL"}=" H31 PPL"; $resn_exch{"2H3  PPL"}=" H32 PPL"; $resn_exch{"1H4  PPL"}=" H41 PPL";
$resn_exch{"2H4  PPL"}=" H42 PPL"; $resn_exch{"1H5  PPL"}=" H51 PPL"; $resn_exch{"2H5  PPL"}=" H52 PPL";
$resn_exch{"1H6  PPL"}=" H61 PPL"; $resn_exch{"2H6  PPL"}=" H62 PPL"; $resn_exch{"1H2' PPL"}="H2'1 PPL";
$resn_exch{"2H2' PPL"}="H2'2 PPL"; $resn_exch{"3H2' PPL"}="H2'3 PPL"; $resn_exch{"1H3' PPL"}="H3'1 PPL";
$resn_exch{"2H3' PPL"}="H3'2 PPL"; $resn_exch{"3H3' PPL"}="H3'3 PPL"; $resn_exch{"1H4' PPL"}="H4'1 PPL";
$resn_exch{"2H4' PPL"}="H4'2 PPL"; $resn_exch{"3H4' PPL"}="H4'3 PPL"; $resn_exch{"1HM  PPM"}=" HM1 PPM";
$resn_exch{"2HM  PPM"}=" HM2 PPM"; $resn_exch{"3HM  PPM"}=" HM3 PPM"; $resn_exch{"2HOP PPM"}="HOP2 PPM";
$resn_exch{"3HOP PPM"}="HOP3 PPM"; $resn_exch{"1HN  PPN"}=" H   PPN"; $resn_exch{"2HN  PPN"}=" H2  PPN";
$resn_exch{"1HB  PPN"}=" HB2 PPN"; $resn_exch{"2HB  PPN"}=" HB3 PPN"; $resn_exch{"1HC9 PPO"}="HC91 PPO";
$resn_exch{"2HC9 PPO"}="HC92 PPO"; $resn_exch{"1H1  PPP"}=" H11 PPP"; $resn_exch{"2H1  PPP"}=" H12 PPP";
$resn_exch{"1H3  PPP"}=" H31 PPP"; $resn_exch{"2H3  PPP"}=" H32 PPP"; $resn_exch{"1HNP PPQ"}="HNP1 PPQ";
$resn_exch{"2HNP PPQ"}="HNP2 PPQ"; $resn_exch{"1HBP PPQ"}="HBP1 PPQ"; $resn_exch{"2HBP PPQ"}="HBP2 PPQ";
$resn_exch{"1HGP PPQ"}="HGP1 PPQ"; $resn_exch{"2HGP PPQ"}="HGP2 PPQ"; $resn_exch{"1HEP PPQ"}="HEP1 PPQ";
$resn_exch{"2HEP PPQ"}="HEP2 PPQ"; $resn_exch{"3HEP PPQ"}="HEP3 PPQ"; $resn_exch{"OH2' PPR"}="H2'O PPR";
$resn_exch{"1H3  PPR"}=" H31 PPR"; $resn_exch{"2H3  PPR"}=" H32 PPR"; $resn_exch{"PHO5 PPS"}="HO5P PPS";
$resn_exch{"1H5* PPS"}="H5'1 PPS"; $resn_exch{"2H5* PPS"}="H5'2 PPS"; $resn_exch{" H4* PPS"}=" H4' PPS";
$resn_exch{" H1* PPS"}=" H1' PPS"; $resn_exch{"1HN6 PPS"}="HN61 PPS"; $resn_exch{"2HN6 PPS"}="HN62 PPS";
$resn_exch{" H2* PPS"}=" H2' PPS"; $resn_exch{"*HO2 PPS"}="HO2' PPS"; $resn_exch{" H3* PPS"}=" H3' PPS";
$resn_exch{"PHO1 PPS"}="HO1P PPS"; $resn_exch{"PHO2 PPS"}="HO2P PPS"; $resn_exch{"1HA  PPT"}=" HA1 PPT";
$resn_exch{"2HA  PPT"}=" HA2 PPT"; $resn_exch{"1HB  PPT"}=" HB1 PPT"; $resn_exch{"2HB  PPT"}=" HB2 PPT";
$resn_exch{"1HH  PPT"}=" HH1 PPT"; $resn_exch{"2HH  PPT"}=" HH2 PPT"; $resn_exch{"3HH  PPT"}=" HH3 PPT";
$resn_exch{"2HOP PPU"}="HOP2 PPU"; $resn_exch{"3HOP PPU"}="HOP3 PPU"; $resn_exch{"1HN  PPU"}=" HN1 PPU";
$resn_exch{"2HN  PPU"}=" HN2 PPU"; $resn_exch{"1HB  PPU"}=" HB1 PPU"; $resn_exch{"2HB  PPU"}=" HB2 PPU";
$resn_exch{"1HM  PPU"}=" HM1 PPU"; $resn_exch{"2HM  PPU"}=" HM2 PPU"; $resn_exch{"3HM  PPU"}=" HM3 PPU";
$resn_exch{"1H5* PPU"}=" H5' PPU"; $resn_exch{"2H5* PPU"}="H5'' PPU"; $resn_exch{" H4* PPU"}=" H4' PPU";
$resn_exch{" H3* PPU"}=" H3' PPU"; $resn_exch{"3HN* PPU"}="HN'3 PPU"; $resn_exch{" H2* PPU"}=" H2' PPU";
$resn_exch{"*HO2 PPU"}="HO2' PPU"; $resn_exch{" H1* PPU"}=" H1' PPU"; $resn_exch{"1H9  PPU"}=" H91 PPU";
$resn_exch{"2H9  PPU"}=" H92 PPU"; $resn_exch{"3H9  PPU"}=" H93 PPU"; $resn_exch{"1H10 PPU"}="H101 PPU";
$resn_exch{"2H10 PPU"}="H102 PPU"; $resn_exch{"3H10 PPU"}="H103 PPU"; $resn_exch{"2HOP PPW"}="HOP2 PPW";
$resn_exch{"1HN2 PPW"}="HN21 PPW"; $resn_exch{"2HN2 PPW"}="HN22 PPW"; $resn_exch{"1H2* PPW"}=" H2' PPW";
$resn_exch{"2H2* PPW"}="H2'' PPW"; $resn_exch{"1H5* PPW"}=" H5' PPW"; $resn_exch{"2H5* PPW"}="H5'' PPW";
$resn_exch{" H4* PPW"}=" H4' PPW"; $resn_exch{" H1* PPW"}=" H1' PPW"; $resn_exch{" H3* PPW"}=" H3' PPW";
$resn_exch{"H3T  PPW"}="HO3' PPW"; $resn_exch{"3HOP PPW"}="HOP3 PPW"; $resn_exch{"1HN1 PPX"}="HN11 PPX";
$resn_exch{"2HN1 PPX"}="HN12 PPX"; $resn_exch{"1H3  PPX"}=" H31 PPX"; $resn_exch{"2H3  PPX"}=" H32 PPX";
$resn_exch{"1H13 PPX"}="H131 PPX"; $resn_exch{"2H13 PPX"}="H132 PPX"; $resn_exch{"1H15 PPX"}="H151 PPX";
$resn_exch{"2H15 PPX"}="H152 PPX"; $resn_exch{"1H16 PPX"}="H161 PPX"; $resn_exch{"2H16 PPX"}="H162 PPX";
$resn_exch{"1H20 PPX"}="H201 PPX"; $resn_exch{"2H20 PPX"}="H202 PPX"; $resn_exch{"1H30 PPX"}="H301 PPX";
$resn_exch{"2H30 PPX"}="H302 PPX"; $resn_exch{"1H3  PPY"}=" H31 PPY"; $resn_exch{"2H3  PPY"}=" H32 PPY";
$resn_exch{"1H6  PPZ"}=" H61 PPZ"; $resn_exch{"2H6  PPZ"}=" H62 PPZ"; $resn_exch{"1H7  PPZ"}=" H71 PPZ";
$resn_exch{"2H7  PPZ"}=" H72 PPZ"; $resn_exch{"1HN2 PQ0"}="HN21 PQ0"; $resn_exch{"2HN2 PQ0"}="HN22 PQ0";
$resn_exch{"3HOP PQ1"}="HOP3 PQ1"; $resn_exch{"2HOP PQ1"}="HOP2 PQ1"; $resn_exch{"1H5* PQ1"}=" H5' PQ1";
$resn_exch{"2H5* PQ1"}="H5'' PQ1"; $resn_exch{" H4* PQ1"}=" H4' PQ1"; $resn_exch{" H1* PQ1"}=" H1' PQ1";
$resn_exch{"1HN2 PQ1"}="HN21 PQ1"; $resn_exch{"2HN2 PQ1"}="HN22 PQ1"; $resn_exch{"1H10 PQ1"}="H101 PQ1";
$resn_exch{"2H10 PQ1"}="H102 PQ1"; $resn_exch{"1H11 PQ1"}="H111 PQ1"; $resn_exch{"2H11 PQ1"}="H112 PQ1";
$resn_exch{" H2* PQ1"}=" H2' PQ1"; $resn_exch{" H3* PQ1"}=" H3' PQ1"; $resn_exch{"1H6  PQ9"}=" H61 PQ9";
$resn_exch{"2H6  PQ9"}=" H62 PQ9"; $resn_exch{"3H6  PQ9"}=" H63 PQ9"; $resn_exch{"1H9  PQ9"}=" H91 PQ9";
$resn_exch{"2H9  PQ9"}=" H92 PQ9"; $resn_exch{"3H9  PQ9"}=" H93 PQ9"; $resn_exch{"1H11 PQ9"}="H111 PQ9";
$resn_exch{"2H11 PQ9"}="H112 PQ9"; $resn_exch{"1H14 PQ9"}="H141 PQ9"; $resn_exch{"2H14 PQ9"}="H142 PQ9";
$resn_exch{"3H14 PQ9"}="H143 PQ9"; $resn_exch{"1H15 PQ9"}="H151 PQ9"; $resn_exch{"2H15 PQ9"}="H152 PQ9";
$resn_exch{"1H16 PQ9"}="H161 PQ9"; $resn_exch{"2H16 PQ9"}="H162 PQ9"; $resn_exch{"1H19 PQ9"}="H191 PQ9";
$resn_exch{"2H19 PQ9"}="H192 PQ9"; $resn_exch{"3H19 PQ9"}="H193 PQ9"; $resn_exch{"1H20 PQ9"}="H201 PQ9";
$resn_exch{"2H20 PQ9"}="H202 PQ9"; $resn_exch{"1H21 PQ9"}="H211 PQ9"; $resn_exch{"2H21 PQ9"}="H212 PQ9";
$resn_exch{"1H24 PQ9"}="H241 PQ9"; $resn_exch{"2H24 PQ9"}="H242 PQ9"; $resn_exch{"3H24 PQ9"}="H243 PQ9";
$resn_exch{"1H25 PQ9"}="H251 PQ9"; $resn_exch{"2H25 PQ9"}="H252 PQ9"; $resn_exch{"1H26 PQ9"}="H261 PQ9";
$resn_exch{"2H26 PQ9"}="H262 PQ9"; $resn_exch{"1H29 PQ9"}="H291 PQ9"; $resn_exch{"2H29 PQ9"}="H292 PQ9";
$resn_exch{"3H29 PQ9"}="H293 PQ9"; $resn_exch{"1H30 PQ9"}="H301 PQ9"; $resn_exch{"2H30 PQ9"}="H302 PQ9";
$resn_exch{"1H31 PQ9"}="H311 PQ9"; $resn_exch{"2H31 PQ9"}="H312 PQ9"; $resn_exch{"1H34 PQ9"}="H341 PQ9";
$resn_exch{"2H34 PQ9"}="H342 PQ9"; $resn_exch{"3H34 PQ9"}="H343 PQ9"; $resn_exch{"1H35 PQ9"}="H351 PQ9";
$resn_exch{"2H35 PQ9"}="H352 PQ9"; $resn_exch{"1H36 PQ9"}="H361 PQ9"; $resn_exch{"2H36 PQ9"}="H362 PQ9";
$resn_exch{"1H39 PQ9"}="H391 PQ9"; $resn_exch{"2H39 PQ9"}="H392 PQ9"; $resn_exch{"3H39 PQ9"}="H393 PQ9";
$resn_exch{"1H40 PQ9"}="H401 PQ9"; $resn_exch{"2H40 PQ9"}="H402 PQ9"; $resn_exch{"1H41 PQ9"}="H411 PQ9";
$resn_exch{"2H41 PQ9"}="H412 PQ9"; $resn_exch{"1H44 PQ9"}="H441 PQ9"; $resn_exch{"2H44 PQ9"}="H442 PQ9";
$resn_exch{"3H44 PQ9"}="H443 PQ9"; $resn_exch{"1H45 PQ9"}="H451 PQ9"; $resn_exch{"2H45 PQ9"}="H452 PQ9";
$resn_exch{"3H45 PQ9"}="H453 PQ9"; $resn_exch{"1HN1 PQA"}="HN11 PQA"; $resn_exch{"2HN1 PQA"}="HN12 PQA";
$resn_exch{"1H23 PQA"}="H231 PQA"; $resn_exch{"2H23 PQA"}="H232 PQA"; $resn_exch{"1H24 PQA"}="H241 PQA";
$resn_exch{"2H24 PQA"}="H242 PQA"; $resn_exch{"1H26 PQA"}="H261 PQA"; $resn_exch{"2H26 PQA"}="H262 PQA";
$resn_exch{"1H27 PQA"}="H271 PQA"; $resn_exch{"2H27 PQA"}="H272 PQA"; $resn_exch{"1HN1 PQB"}="HN11 PQB";
$resn_exch{"2HN1 PQB"}="HN12 PQB"; $resn_exch{"1H22 PQB"}="H221 PQB"; $resn_exch{"2H22 PQB"}="H222 PQB";
$resn_exch{"1H26 PQB"}="H261 PQB"; $resn_exch{"2H26 PQB"}="H262 PQB"; $resn_exch{"1H68 PQE"}="H681 PQE";
$resn_exch{"2H68 PQE"}="H682 PQE"; $resn_exch{"1H52 PQE"}="H521 PQE"; $resn_exch{"2H52 PQE"}="H522 PQE";
$resn_exch{"1H55 PQE"}="H551 PQE"; $resn_exch{"2H55 PQE"}="H552 PQE"; $resn_exch{"1H58 PQE"}="H581 PQE";
$resn_exch{"2H58 PQE"}="H582 PQE"; $resn_exch{"1H61 PQE"}="H611 PQE"; $resn_exch{"2H61 PQE"}="H612 PQE";
$resn_exch{"1H62 PQE"}="H621 PQE"; $resn_exch{"2H62 PQE"}="H622 PQE"; $resn_exch{"1H63 PQE"}="H631 PQE";
$resn_exch{"2H63 PQE"}="H632 PQE"; $resn_exch{"1H66 PQE"}="H661 PQE"; $resn_exch{"2H66 PQE"}="H662 PQE";
$resn_exch{"1H67 PQE"}="H671 PQE"; $resn_exch{"2H67 PQE"}="H672 PQE"; $resn_exch{"1H65 PQE"}="H651 PQE";
$resn_exch{"2H65 PQE"}="H652 PQE"; $resn_exch{"1H64 PQE"}="H641 PQE"; $resn_exch{"2H64 PQE"}="H642 PQE";
$resn_exch{"1H60 PQE"}="H601 PQE"; $resn_exch{"2H60 PQE"}="H602 PQE"; $resn_exch{"1H59 PQE"}="H591 PQE";
$resn_exch{"2H59 PQE"}="H592 PQE"; $resn_exch{"1H57 PQE"}="H571 PQE"; $resn_exch{"2H57 PQE"}="H572 PQE";
$resn_exch{"1H56 PQE"}="H561 PQE"; $resn_exch{"2H56 PQE"}="H562 PQE"; $resn_exch{"1H54 PQE"}="H541 PQE";
$resn_exch{"2H54 PQE"}="H542 PQE"; $resn_exch{"1H53 PQE"}="H531 PQE"; $resn_exch{"2H53 PQE"}="H532 PQE";
$resn_exch{"1H51 PQE"}="H511 PQE"; $resn_exch{"2H51 PQE"}="H512 PQE"; $resn_exch{"1H50 PQE"}="H501 PQE";
$resn_exch{"2H50 PQE"}="H502 PQE"; $resn_exch{"1H49 PQE"}="H491 PQE"; $resn_exch{"2H49 PQE"}="H492 PQE";
$resn_exch{"1H48 PQE"}="H481 PQE"; $resn_exch{"2H48 PQE"}="H482 PQE"; $resn_exch{"1H47 PQE"}="H471 PQE";
$resn_exch{"2H47 PQE"}="H472 PQE"; $resn_exch{"1H46 PQE"}="H461 PQE"; $resn_exch{"2H46 PQE"}="H462 PQE";
$resn_exch{"1H45 PQE"}="H451 PQE"; $resn_exch{"2H45 PQE"}="H452 PQE"; $resn_exch{"1H44 PQE"}="H441 PQE";
$resn_exch{"2H44 PQE"}="H442 PQE"; $resn_exch{"1H43 PQE"}="H431 PQE"; $resn_exch{"2H43 PQE"}="H432 PQE";
$resn_exch{"1H42 PQE"}="H421 PQE"; $resn_exch{"2H42 PQE"}="H422 PQE"; $resn_exch{"1H41 PQE"}="H411 PQE";
$resn_exch{"2H41 PQE"}="H412 PQE"; $resn_exch{"1H40 PQE"}="H401 PQE"; $resn_exch{"2H40 PQE"}="H402 PQE";
$resn_exch{"3H40 PQE"}="H403 PQE"; $resn_exch{"1H2M PQN"}="H2M1 PQN"; $resn_exch{"2H2M PQN"}="H2M2 PQN";
$resn_exch{"3H2M PQN"}="H2M3 PQN"; $resn_exch{"1H11 PQN"}="H111 PQN"; $resn_exch{"2H11 PQN"}="H112 PQN";
$resn_exch{"1H14 PQN"}="H141 PQN"; $resn_exch{"2H14 PQN"}="H142 PQN"; $resn_exch{"3H14 PQN"}="H143 PQN";
$resn_exch{"1H15 PQN"}="H151 PQN"; $resn_exch{"2H15 PQN"}="H152 PQN"; $resn_exch{"1H16 PQN"}="H161 PQN";
$resn_exch{"2H16 PQN"}="H162 PQN"; $resn_exch{"1H17 PQN"}="H171 PQN"; $resn_exch{"2H17 PQN"}="H172 PQN";
$resn_exch{"1H19 PQN"}="H191 PQN"; $resn_exch{"2H19 PQN"}="H192 PQN"; $resn_exch{"3H19 PQN"}="H193 PQN";
$resn_exch{"1H20 PQN"}="H201 PQN"; $resn_exch{"2H20 PQN"}="H202 PQN"; $resn_exch{"1H21 PQN"}="H211 PQN";
$resn_exch{"2H21 PQN"}="H212 PQN"; $resn_exch{"1H22 PQN"}="H221 PQN"; $resn_exch{"2H22 PQN"}="H222 PQN";
$resn_exch{"1H24 PQN"}="H241 PQN"; $resn_exch{"2H24 PQN"}="H242 PQN"; $resn_exch{"3H24 PQN"}="H243 PQN";
$resn_exch{"1H25 PQN"}="H251 PQN"; $resn_exch{"2H25 PQN"}="H252 PQN"; $resn_exch{"1H26 PQN"}="H261 PQN";
$resn_exch{"2H26 PQN"}="H262 PQN"; $resn_exch{"1H27 PQN"}="H271 PQN"; $resn_exch{"2H27 PQN"}="H272 PQN";
$resn_exch{"1H29 PQN"}="H291 PQN"; $resn_exch{"2H29 PQN"}="H292 PQN"; $resn_exch{"3H29 PQN"}="H293 PQN";
$resn_exch{"1H30 PQN"}="H301 PQN"; $resn_exch{"2H30 PQN"}="H302 PQN"; $resn_exch{"3H30 PQN"}="H303 PQN";
$resn_exch{"2HOB PQQ"}="HOB2 PQQ"; $resn_exch{"7HOB PQQ"}="HOB7 PQQ"; $resn_exch{"9HOB PQQ"}="HOB9 PQQ";
$resn_exch{" HN2 PR0"}=" H2  PR0"; $resn_exch{"1HB  PR0"}=" HB2 PR0"; $resn_exch{"2HB  PR0"}=" HB3 PR0";
$resn_exch{"1H1  PR1"}=" H11 PR1"; $resn_exch{"1H2  PR1"}=" H21 PR1"; $resn_exch{"1H5  PR1"}=" H51 PR1";
$resn_exch{"1H11 PR1"}="H111 PR1"; $resn_exch{"1H12 PR1"}="H121 PR1"; $resn_exch{"2H12 PR1"}="H122 PR1";
$resn_exch{"1H13 PR1"}="H131 PR1"; $resn_exch{"2H13 PR1"}="H132 PR1"; $resn_exch{"1H44 PR1"}="H441 PR1";
$resn_exch{"1H17 PR1"}="H171 PR1"; $resn_exch{"1H18 PR1"}="H181 PR1"; $resn_exch{"1H19 PR1"}="H191 PR1";
$resn_exch{"1H25 PR1"}="H251 PR1"; $resn_exch{"1H30 PR1"}="H301 PR1"; $resn_exch{"1H31 PR1"}="H311 PR1";
$resn_exch{"2H31 PR1"}="H312 PR1"; $resn_exch{"1H32 PR1"}="H321 PR1"; $resn_exch{"2H32 PR1"}="H322 PR1";
$resn_exch{"1H37 PR1"}="H371 PR1"; $resn_exch{"1H17 PR2"}="H171 PR2"; $resn_exch{"1H18 PR2"}="H181 PR2";
$resn_exch{"1H19 PR2"}="H191 PR2"; $resn_exch{"1H25 PR2"}="H251 PR2"; $resn_exch{"1H30 PR2"}="H301 PR2";
$resn_exch{"1H31 PR2"}="H311 PR2"; $resn_exch{"2H31 PR2"}="H312 PR2"; $resn_exch{"1H32 PR2"}="H321 PR2";
$resn_exch{"2H32 PR2"}="H322 PR2"; $resn_exch{"1H37 PR2"}="H371 PR2"; $resn_exch{"1H1  PR2"}=" H11 PR2";
$resn_exch{"2H1  PR2"}=" H12 PR2"; $resn_exch{"1H3  PR2"}=" H31 PR2"; $resn_exch{"1H4  PR2"}=" H41 PR2";
$resn_exch{"1H6  PR2"}=" H61 PR2"; $resn_exch{"1H27 PR2"}="H271 PR2"; $resn_exch{"1HN  PR3"}=" H   PR3";
$resn_exch{"2HN  PR3"}=" H2  PR3"; $resn_exch{" H   PR3"}=" HC  PR3"; $resn_exch{"1HB  PR3"}=" HB2 PR3";
$resn_exch{"2HB  PR3"}=" HB3 PR3"; $resn_exch{"1HE  PR3"}=" HE2 PR3"; $resn_exch{"2HE  PR3"}=" HE3 PR3";
$resn_exch{"1HZ  PR3"}=" HZ2 PR3"; $resn_exch{"2HZ  PR3"}=" HZ3 PR3"; $resn_exch{"1HH  PR3"}=" HH1 PR3";
$resn_exch{"2HH  PR3"}=" HH2 PR3"; $resn_exch{"3HH  PR3"}=" HH3 PR3"; $resn_exch{"2HOP PR5"}="HOP2 PR5";
$resn_exch{"1H5* PR5"}=" H5' PR5"; $resn_exch{"2H5* PR5"}="H5'' PR5"; $resn_exch{" H4* PR5"}=" H4' PR5";
$resn_exch{" H1* PR5"}=" H1' PR5"; $resn_exch{" H3* PR5"}=" H3' PR5"; $resn_exch{"H3T  PR5"}="HO3' PR5";
$resn_exch{" H2* PR5"}=" H2' PR5"; $resn_exch{"2HO* PR5"}="HO2' PR5"; $resn_exch{"3HOP PR5"}="HOP3 PR5";
$resn_exch{"1H1  PRA"}=" H11 PRA"; $resn_exch{"2H1  PRA"}=" H12 PRA"; $resn_exch{"1H2  PRA"}=" H21 PRA";
$resn_exch{"2H2  PRA"}=" H22 PRA"; $resn_exch{"1H3  PRA"}=" H31 PRA"; $resn_exch{"2H3  PRA"}=" H32 PRA";
$resn_exch{"1HN  PRA"}=" HN1 PRA"; $resn_exch{"2HN  PRA"}=" HN2 PRA"; $resn_exch{"3HN  PRA"}=" HN3 PRA";
$resn_exch{"1H5  PRB"}=" H51 PRB"; $resn_exch{"2H5  PRB"}=" H52 PRB"; $resn_exch{"2HO1 PRB"}="HO12 PRB";
$resn_exch{"1H16 PRB"}="H161 PRB"; $resn_exch{"2H16 PRB"}="H162 PRB"; $resn_exch{"3H16 PRB"}="H163 PRB";
$resn_exch{"1H17 PRB"}="H171 PRB"; $resn_exch{"2H17 PRB"}="H172 PRB"; $resn_exch{"3H17 PRB"}="H173 PRB";
$resn_exch{"1H18 PRB"}="H181 PRB"; $resn_exch{"2H18 PRB"}="H182 PRB"; $resn_exch{"3H18 PRB"}="H183 PRB";
$resn_exch{"1H19 PRB"}="H191 PRB"; $resn_exch{"2H19 PRB"}="H192 PRB"; $resn_exch{"3H19 PRB"}="H193 PRB";
$resn_exch{"1H20 PRB"}="H201 PRB"; $resn_exch{"2H20 PRB"}="H202 PRB"; $resn_exch{"0HO2 PRB"}="HO20 PRB";
$resn_exch{"1HA2 PRB"}="HA21 PRB"; $resn_exch{"2HA2 PRB"}="HA22 PRB"; $resn_exch{"3HA2 PRB"}="HA23 PRB";
$resn_exch{"CH11 PRC"}="H11C PRC"; $resn_exch{"CH12 PRC"}="H12C PRC"; $resn_exch{"NH13 PRC"}="H13N PRC";
$resn_exch{"CH15 PRC"}="H15C PRC"; $resn_exch{"CH17 PRC"}="H17C PRC"; $resn_exch{"CH18 PRC"}="H18C PRC";
$resn_exch{"1H20 PRC"}="H201 PRC"; $resn_exch{"2H20 PRC"}="H202 PRC"; $resn_exch{"3H20 PRC"}="H203 PRC";
$resn_exch{"NH21 PRC"}="H21N PRC"; $resn_exch{"CH25 PRC"}="H25C PRC"; $resn_exch{"CH26 PRC"}="H26C PRC";
$resn_exch{"CH27 PRC"}="H27C PRC"; $resn_exch{"CH29 PRC"}="H29C PRC"; $resn_exch{" H7* PRD"}=" H7' PRD";
$resn_exch{"1HC6 PRD"}="HC61 PRD"; $resn_exch{"2HC6 PRD"}="HC62 PRD"; $resn_exch{"3HC6 PRD"}="HC63 PRD";
$resn_exch{" H5* PRD"}=" H5' PRD"; $resn_exch{"1HN4 PRD"}="HN41 PRD"; $resn_exch{"2HN4 PRD"}="HN42 PRD";
$resn_exch{"1HN2 PRD"}="HN21 PRD"; $resn_exch{"2HN2 PRD"}="HN22 PRD"; $resn_exch{"1HC7 PRD"}="HC71 PRD";
$resn_exch{"2HC7 PRD"}="HC72 PRD"; $resn_exch{"1HC2 PRD"}="HC21 PRD"; $resn_exch{"2HC2 PRD"}="HC22 PRD";
$resn_exch{"3HC2 PRD"}="HC23 PRD"; $resn_exch{"1HC5 PRD"}="HC51 PRD"; $resn_exch{"2HC5 PRD"}="HC52 PRD";
$resn_exch{"3HC5 PRD"}="HC53 PRD"; $resn_exch{"2HO7 PRE"}="HO72 PRE"; $resn_exch{"1H8  PRE"}=" H81 PRE";
$resn_exch{"2H8  PRE"}=" H82 PRE"; $resn_exch{"'HO2 PRE"}="HO2' PRE"; $resn_exch{"1H9  PRF"}=" H91 PRF";
$resn_exch{"1H10 PRF"}="H101 PRF"; $resn_exch{"2H10 PRF"}="H102 PRF"; $resn_exch{"1H11 PRF"}="H111 PRF";
$resn_exch{"2H11 PRF"}="H112 PRF"; $resn_exch{"1H8  PRF"}=" H81 PRF"; $resn_exch{"1HN9 PRF"}="HN91 PRF";
$resn_exch{"1HN2 PRF"}="HN21 PRF"; $resn_exch{"2HN2 PRF"}="HN22 PRF"; $resn_exch{" H1* PRH"}=" H1' PRH";
$resn_exch{" H2* PRH"}=" H2' PRH"; $resn_exch{"*HO2 PRH"}="HO2' PRH"; $resn_exch{" H3* PRH"}=" H3' PRH";
$resn_exch{"*HO3 PRH"}="HO3' PRH"; $resn_exch{" H4* PRH"}=" H4' PRH"; $resn_exch{"1H5* PRH"}="H5'1 PRH";
$resn_exch{"2H5* PRH"}="H5'2 PRH"; $resn_exch{"*HO5 PRH"}="HO5' PRH"; $resn_exch{"1HCB PRI"}="HCB1 PRI";
$resn_exch{"2HCB PRI"}="HCB2 PRI"; $resn_exch{"1HCG PRI"}="HCG1 PRI"; $resn_exch{"2HCG PRI"}="HCG2 PRI";
$resn_exch{"THO' PRI"}="HO'T PRI"; $resn_exch{"1HCD PRI"}="HCD1 PRI"; $resn_exch{"2HCD PRI"}="HCD2 PRI";
$resn_exch{" AHC PRI"}=" HCA PRI"; $resn_exch{"1HNF PRL"}="HNF1 PRL"; $resn_exch{"2HNF PRL"}="HNF2 PRL";
$resn_exch{"1HNG PRL"}="HNG1 PRL"; $resn_exch{"2HNG PRL"}="HNG2 PRL"; $resn_exch{"1H36 PRM"}="H361 PRM";
$resn_exch{"2H36 PRM"}="H362 PRM"; $resn_exch{"1H37 PRM"}="H371 PRM"; $resn_exch{"2H37 PRM"}="H372 PRM";
$resn_exch{"1H41 PRM"}="H411 PRM"; $resn_exch{"2H41 PRM"}="H412 PRM"; $resn_exch{"1H42 PRM"}="H421 PRM";
$resn_exch{"2H42 PRM"}="H422 PRM"; $resn_exch{"1H43 PRM"}="H431 PRM"; $resn_exch{"2H43 PRM"}="H432 PRM";
$resn_exch{"1H44 PRM"}="H441 PRM"; $resn_exch{"2H44 PRM"}="H442 PRM"; $resn_exch{"1H45 PRM"}="H451 PRM";
$resn_exch{"2H45 PRM"}="H452 PRM"; $resn_exch{"3H45 PRM"}="H453 PRM"; $resn_exch{"1H46 PRM"}="H461 PRM";
$resn_exch{"2H46 PRM"}="H462 PRM"; $resn_exch{"3H46 PRM"}="H463 PRM"; $resn_exch{"1H47 PRM"}="H471 PRM";
$resn_exch{"2H47 PRM"}="H472 PRM"; $resn_exch{"1H48 PRM"}="H481 PRM"; $resn_exch{"2H48 PRM"}="H482 PRM";
$resn_exch{"3H48 PRM"}="H483 PRM"; $resn_exch{" H1* PRN"}=" H1' PRN"; $resn_exch{"1H2* PRN"}=" H2' PRN";
$resn_exch{"2H2* PRN"}="H2'' PRN"; $resn_exch{" H3* PRN"}=" H3' PRN"; $resn_exch{"HO3* PRN"}="HO3' PRN";
$resn_exch{" H4* PRN"}=" H4' PRN"; $resn_exch{"1H5* PRN"}=" H5' PRN"; $resn_exch{"2H5* PRN"}="H5'' PRN";
$resn_exch{"2HOP PRN"}="HOP2 PRN"; $resn_exch{"3HOP PRN"}="HOP3 PRN"; $resn_exch{" HT1 PRO"}=" H   PRO";
$resn_exch{"1HB  PRO"}=" HB2 PRO"; $resn_exch{"2HB  PRO"}=" HB3 PRO"; $resn_exch{"1HG  PRO"}=" HG2 PRO";
$resn_exch{"2HG  PRO"}=" HG3 PRO"; $resn_exch{"1HD  PRO"}=" HD2 PRO"; $resn_exch{"2HD  PRO"}=" HD3 PRO";
$resn_exch{"1H5  PRP"}=" H51 PRP"; $resn_exch{"2H5  PRP"}=" H52 PRP"; $resn_exch{"2HOP PRP"}="HOP2 PRP";
$resn_exch{"3HOP PRP"}="HOP3 PRP"; $resn_exch{"2HOA PRP"}="HOA2 PRP"; $resn_exch{"2HOB PRP"}="HOB2 PRP";
$resn_exch{"3HOB PRP"}="HOB3 PRP"; $resn_exch{"1HAJ PRQ"}="HAJ1 PRQ"; $resn_exch{"2HAJ PRQ"}="HAJ2 PRQ";
$resn_exch{"1HAA PRQ"}="HAA1 PRQ"; $resn_exch{"2HAA PRQ"}="HAA2 PRQ"; $resn_exch{" HN2 PRR"}=" H2  PRR";
$resn_exch{" H2  PRR"}=" HC2 PRR"; $resn_exch{"1H5  PRR"}=" H51 PRR"; $resn_exch{"2H5  PRR"}=" H52 PRR";
$resn_exch{"1H10 PRR"}="H101 PRR"; $resn_exch{"2H10 PRR"}="H102 PRR"; $resn_exch{"3H10 PRR"}="H103 PRR";
$resn_exch{" HN  PRS"}=" H   PRS"; $resn_exch{"1HD  PRS"}=" HD3 PRS"; $resn_exch{"2HD  PRS"}=" HD2 PRS";
$resn_exch{"1HB  PRS"}=" HB2 PRS"; $resn_exch{"2HB  PRS"}=" HB3 PRS"; $resn_exch{"1H5* PRT"}="H5'1 PRT";
$resn_exch{"2H5* PRT"}="H5'2 PRT"; $resn_exch{" H4* PRT"}=" H4' PRT"; $resn_exch{" H3* PRT"}=" H3' PRT";
$resn_exch{" H2* PRT"}=" H2' PRT"; $resn_exch{" H1* PRT"}=" H1' PRT"; $resn_exch{"1H15 PRT"}="H151 PRT";
$resn_exch{"2H15 PRT"}="H152 PRT"; $resn_exch{" H2* PRX"}=" H2' PRX"; $resn_exch{" H1* PRX"}=" H1' PRX";
$resn_exch{" H4* PRX"}=" H4' PRX"; $resn_exch{"1H5* PRX"}="H5'1 PRX"; $resn_exch{"2H5* PRX"}="H5'2 PRX";
$resn_exch{"1H1P PRX"}="H1P1 PRX"; $resn_exch{"2H1P PRX"}="H1P2 PRX"; $resn_exch{"1H2P PRX"}="H2P1 PRX";
$resn_exch{"2H2P PRX"}="H2P2 PRX"; $resn_exch{"1H3P PRX"}="H3P1 PRX"; $resn_exch{"2H3P PRX"}="H3P2 PRX";
$resn_exch{"3H3P PRX"}="H3P3 PRX"; $resn_exch{" H3* PRX"}=" H3' PRX"; $resn_exch{"1HN6 PRX"}="HN61 PRX";
$resn_exch{"2HN6 PRX"}="HN62 PRX"; $resn_exch{"1HC4 PRY"}="HC41 PRY"; $resn_exch{"2HC4 PRY"}="HC42 PRY";
$resn_exch{"1HN6 PRY"}="HN61 PRY"; $resn_exch{"2HN6 PRY"}="HN62 PRY"; $resn_exch{"1HC9 PRY"}="HC91 PRY";
$resn_exch{"2HC9 PRY"}="HC92 PRY"; $resn_exch{"1H10 PRY"}="H101 PRY"; $resn_exch{"2H10 PRY"}="H102 PRY";
$resn_exch{"3H10 PRY"}="H103 PRY"; $resn_exch{"2H21 PRZ"}="H212 PRZ"; $resn_exch{"1H21 PRZ"}="H211 PRZ";
$resn_exch{"3H23 PRZ"}="H233 PRZ"; $resn_exch{"2H23 PRZ"}="H232 PRZ"; $resn_exch{"1H23 PRZ"}="H231 PRZ";
$resn_exch{"3H24 PRZ"}="H243 PRZ"; $resn_exch{"2H24 PRZ"}="H242 PRZ"; $resn_exch{"1H24 PRZ"}="H241 PRZ";
$resn_exch{"3H31 PRZ"}="H313 PRZ"; $resn_exch{"2H31 PRZ"}="H312 PRZ"; $resn_exch{"1H31 PRZ"}="H311 PRZ";
$resn_exch{"1HB  PS0"}=" HB1 PS0"; $resn_exch{"2HB  PS0"}=" HB2 PS0"; $resn_exch{"1H48 PS2"}="H481 PS2";
$resn_exch{"2H48 PS2"}="H482 PS2"; $resn_exch{"3H48 PS2"}="H483 PS2"; $resn_exch{"1H47 PS2"}="H471 PS2";
$resn_exch{"2H47 PS2"}="H472 PS2"; $resn_exch{"1H46 PS2"}="H461 PS2"; $resn_exch{"2H46 PS2"}="H462 PS2";
$resn_exch{"1H45 PS2"}="H451 PS2"; $resn_exch{"2H45 PS2"}="H452 PS2"; $resn_exch{"1H44 PS2"}="H441 PS2";
$resn_exch{"2H44 PS2"}="H442 PS2"; $resn_exch{"1H43 PS2"}="H431 PS2"; $resn_exch{"2H43 PS2"}="H432 PS2";
$resn_exch{"1H42 PS2"}="H421 PS2"; $resn_exch{"2H42 PS2"}="H422 PS2"; $resn_exch{"1H41 PS2"}="H411 PS2";
$resn_exch{"2H41 PS2"}="H412 PS2"; $resn_exch{"1H38 PS2"}="H381 PS2"; $resn_exch{"2H38 PS2"}="H382 PS2";
$resn_exch{"1H37 PS2"}="H371 PS2"; $resn_exch{"2H37 PS2"}="H372 PS2"; $resn_exch{"1H36 PS2"}="H361 PS2";
$resn_exch{"2H36 PS2"}="H362 PS2"; $resn_exch{"1H35 PS2"}="H351 PS2"; $resn_exch{"2H35 PS2"}="H352 PS2";
$resn_exch{"1H34 PS2"}="H341 PS2"; $resn_exch{"2H34 PS2"}="H342 PS2"; $resn_exch{"1H33 PS2"}="H331 PS2";
$resn_exch{"2H33 PS2"}="H332 PS2"; $resn_exch{"1H32 PS2"}="H321 PS2"; $resn_exch{"2H32 PS2"}="H322 PS2";
$resn_exch{"1H1  PS2"}=" H11 PS2"; $resn_exch{"2H1  PS2"}=" H12 PS2"; $resn_exch{"1H4  PS2"}=" H41 PS2";
$resn_exch{"2H4  PS2"}=" H42 PS2"; $resn_exch{"1HN5 PS2"}="HN51 PS2"; $resn_exch{"2HN5 PS2"}="HN52 PS2";
$resn_exch{"1H3  PS2"}=" H31 PS2"; $resn_exch{"2H3  PS2"}=" H32 PS2"; $resn_exch{"1H12 PS2"}="H121 PS2";
$resn_exch{"2H12 PS2"}="H122 PS2"; $resn_exch{"1H13 PS2"}="H131 PS2"; $resn_exch{"2H13 PS2"}="H132 PS2";
$resn_exch{"1H14 PS2"}="H141 PS2"; $resn_exch{"2H14 PS2"}="H142 PS2"; $resn_exch{"1H15 PS2"}="H151 PS2";
$resn_exch{"2H15 PS2"}="H152 PS2"; $resn_exch{"1H16 PS2"}="H161 PS2"; $resn_exch{"2H16 PS2"}="H162 PS2";
$resn_exch{"1H17 PS2"}="H171 PS2"; $resn_exch{"2H17 PS2"}="H172 PS2"; $resn_exch{"1H18 PS2"}="H181 PS2";
$resn_exch{"2H18 PS2"}="H182 PS2"; $resn_exch{"1H19 PS2"}="H191 PS2"; $resn_exch{"2H19 PS2"}="H192 PS2";
$resn_exch{"1H20 PS2"}="H201 PS2"; $resn_exch{"2H20 PS2"}="H202 PS2"; $resn_exch{"1H21 PS2"}="H211 PS2";
$resn_exch{"2H21 PS2"}="H212 PS2"; $resn_exch{"1H22 PS2"}="H221 PS2"; $resn_exch{"2H22 PS2"}="H222 PS2";
$resn_exch{"1H23 PS2"}="H231 PS2"; $resn_exch{"2H23 PS2"}="H232 PS2"; $resn_exch{"1H24 PS2"}="H241 PS2";
$resn_exch{"2H24 PS2"}="H242 PS2"; $resn_exch{"1H25 PS2"}="H251 PS2"; $resn_exch{"2H25 PS2"}="H252 PS2";
$resn_exch{"1H26 PS2"}="H261 PS2"; $resn_exch{"2H26 PS2"}="H262 PS2"; $resn_exch{"3H26 PS2"}="H263 PS2";
$resn_exch{"1HN  PSA"}=" H   PSA"; $resn_exch{"2HN  PSA"}=" H2  PSA"; $resn_exch{"1HB  PSA"}=" HB2 PSA";
$resn_exch{"2HB  PSA"}=" HB3 PSA"; $resn_exch{"1HM  PSA"}=" HM1 PSA"; $resn_exch{"2HM  PSA"}=" HM2 PSA";
$resn_exch{"1HN1 PSB"}="HN11 PSB"; $resn_exch{"2HN1 PSB"}="HN12 PSB"; $resn_exch{"1H21 PSB"}="H211 PSB";
$resn_exch{"2H21 PSB"}="H212 PSB"; $resn_exch{"1H35 PSB"}="H351 PSB"; $resn_exch{"2H35 PSB"}="H352 PSB";
$resn_exch{"1H01 PSC"}="H011 PSC"; $resn_exch{"2H01 PSC"}="H012 PSC"; $resn_exch{"1H03 PSC"}="H031 PSC";
$resn_exch{"2H03 PSC"}="H032 PSC"; $resn_exch{"1H04 PSC"}="H041 PSC"; $resn_exch{"2H04 PSC"}="H042 PSC";
$resn_exch{"1H05 PSC"}="H051 PSC"; $resn_exch{"2H05 PSC"}="H052 PSC"; $resn_exch{"1H06 PSC"}="H061 PSC";
$resn_exch{"2H06 PSC"}="H062 PSC"; $resn_exch{"3H06 PSC"}="H063 PSC"; $resn_exch{"1H07 PSC"}="H071 PSC";
$resn_exch{"2H07 PSC"}="H072 PSC"; $resn_exch{"3H07 PSC"}="H073 PSC"; $resn_exch{"1H08 PSC"}="H081 PSC";
$resn_exch{"2H08 PSC"}="H082 PSC"; $resn_exch{"3H08 PSC"}="H083 PSC"; $resn_exch{"1H2  PSC"}=" H21 PSC";
$resn_exch{"2H2  PSC"}=" H22 PSC"; $resn_exch{"1H3  PSC"}=" H31 PSC"; $resn_exch{"2H3  PSC"}=" H32 PSC";
$resn_exch{"1H4  PSC"}=" H41 PSC"; $resn_exch{"2H4  PSC"}=" H42 PSC"; $resn_exch{"1H5  PSC"}=" H51 PSC";
$resn_exch{"2H5  PSC"}=" H52 PSC"; $resn_exch{"1H6  PSC"}=" H61 PSC"; $resn_exch{"2H6  PSC"}=" H62 PSC";
$resn_exch{"1H7  PSC"}=" H71 PSC"; $resn_exch{"2H7  PSC"}=" H72 PSC"; $resn_exch{"1H8  PSC"}=" H81 PSC";
$resn_exch{"2H8  PSC"}=" H82 PSC"; $resn_exch{"1H11 PSC"}="H111 PSC"; $resn_exch{"2H11 PSC"}="H112 PSC";
$resn_exch{"1H14 PSC"}="H141 PSC"; $resn_exch{"2H14 PSC"}="H142 PSC"; $resn_exch{"1H15 PSC"}="H151 PSC";
$resn_exch{"2H15 PSC"}="H152 PSC"; $resn_exch{"1H16 PSC"}="H161 PSC"; $resn_exch{"2H16 PSC"}="H162 PSC";
$resn_exch{"1H17 PSC"}="H171 PSC"; $resn_exch{"2H17 PSC"}="H172 PSC"; $resn_exch{"1H18 PSC"}="H181 PSC";
$resn_exch{"2H18 PSC"}="H182 PSC"; $resn_exch{"3H18 PSC"}="H183 PSC"; $resn_exch{"1H20 PSC"}="H201 PSC";
$resn_exch{"2H20 PSC"}="H202 PSC"; $resn_exch{"1H21 PSC"}="H211 PSC"; $resn_exch{"2H21 PSC"}="H212 PSC";
$resn_exch{"1H22 PSC"}="H221 PSC"; $resn_exch{"2H22 PSC"}="H222 PSC"; $resn_exch{"1H23 PSC"}="H231 PSC";
$resn_exch{"2H23 PSC"}="H232 PSC"; $resn_exch{"1H24 PSC"}="H241 PSC"; $resn_exch{"2H24 PSC"}="H242 PSC";
$resn_exch{"1H25 PSC"}="H251 PSC"; $resn_exch{"2H25 PSC"}="H252 PSC"; $resn_exch{"1H26 PSC"}="H261 PSC";
$resn_exch{"2H26 PSC"}="H262 PSC"; $resn_exch{"1H27 PSC"}="H271 PSC"; $resn_exch{"2H27 PSC"}="H272 PSC";
$resn_exch{"1H28 PSC"}="H281 PSC"; $resn_exch{"2H28 PSC"}="H282 PSC"; $resn_exch{"1H29 PSC"}="H291 PSC";
$resn_exch{"2H29 PSC"}="H292 PSC"; $resn_exch{"1H30 PSC"}="H301 PSC"; $resn_exch{"2H30 PSC"}="H302 PSC";
$resn_exch{"1H31 PSC"}="H311 PSC"; $resn_exch{"2H31 PSC"}="H312 PSC"; $resn_exch{"1H32 PSC"}="H321 PSC";
$resn_exch{"2H32 PSC"}="H322 PSC"; $resn_exch{"1H33 PSC"}="H331 PSC"; $resn_exch{"2H33 PSC"}="H332 PSC";
$resn_exch{"1H34 PSC"}="H341 PSC"; $resn_exch{"2H34 PSC"}="H342 PSC"; $resn_exch{"3H34 PSC"}="H343 PSC";
$resn_exch{"1HCB PSD"}="HCB1 PSD"; $resn_exch{"2HCB PSD"}="HCB2 PSD"; $resn_exch{"1HCG PSD"}="HCG1 PSD";
$resn_exch{"2HCG PSD"}="HCG2 PSD"; $resn_exch{"1HCD PSD"}="HCD1 PSD"; $resn_exch{"2HCD PSD"}="HCD2 PSD";
$resn_exch{" AHC PSD"}=" HCA PSD"; $resn_exch{"1HN0 PSD"}="HN01 PSD"; $resn_exch{"2HOP PSD"}="HOP2 PSD";
$resn_exch{"1H5' PSD"}="H5'1 PSD"; $resn_exch{"2H5' PSD"}="H5'2 PSD"; $resn_exch{"4HC' PSD"}="HC'4 PSD";
$resn_exch{"1HC' PSD"}="HC'1 PSD"; $resn_exch{"2HC  PSD"}=" HC2 PSD"; $resn_exch{"1HN6 PSD"}="HN61 PSD";
$resn_exch{"2HN6 PSD"}="HN62 PSD"; $resn_exch{"8HC  PSD"}=" HC8 PSD"; $resn_exch{"2HC' PSD"}="HC'2 PSD";
$resn_exch{"2HO' PSD"}="HO'2 PSD"; $resn_exch{"3HC' PSD"}="HC'3 PSD"; $resn_exch{"3HO' PSD"}="HO'3 PSD";
$resn_exch{"1HN2 PSE"}="HN21 PSE"; $resn_exch{"2HN2 PSE"}="HN22 PSE"; $resn_exch{"1H1  PSE"}=" H11 PSE";
$resn_exch{"2H1  PSE"}=" H12 PSE"; $resn_exch{"1H3  PSE"}=" H31 PSE"; $resn_exch{"2H3  PSE"}=" H32 PSE";
$resn_exch{"1H2  PSF"}=" H21 PSF"; $resn_exch{"2H2  PSF"}=" H22 PSF"; $resn_exch{"1H3  PSF"}=" H31 PSF";
$resn_exch{"1H4  PSF"}=" H41 PSF"; $resn_exch{"2H4  PSF"}=" H42 PSF"; $resn_exch{"1H6  PSF"}=" H61 PSF";
$resn_exch{"2H6  PSF"}=" H62 PSF"; $resn_exch{"1H7  PSF"}=" H71 PSF"; $resn_exch{"2H7  PSF"}=" H72 PSF";
$resn_exch{"1H13 PSF"}="H131 PSF"; $resn_exch{"2H13 PSF"}="H132 PSF"; $resn_exch{"1H14 PSF"}="H141 PSF";
$resn_exch{"2H14 PSF"}="H142 PSF"; $resn_exch{"1H15 PSF"}="H151 PSF"; $resn_exch{"2H15 PSF"}="H152 PSF";
$resn_exch{"1H   PSF"}=" H1  PSF"; $resn_exch{"2H   PSF"}=" H2  PSF"; $resn_exch{"1HB  PSF"}=" HB1 PSF";
$resn_exch{"2HB  PSF"}=" HB2 PSF"; $resn_exch{"1H8  PSF"}=" H81 PSF"; $resn_exch{"2H8  PSF"}=" H82 PSF";
$resn_exch{"1H9  PSF"}=" H91 PSF"; $resn_exch{"2H9  PSF"}=" H92 PSF"; $resn_exch{"1H10 PSF"}="H101 PSF";
$resn_exch{"2H10 PSF"}="H102 PSF"; $resn_exch{"3H10 PSF"}="H103 PSF"; $resn_exch{"1H16 PSF"}="H161 PSF";
$resn_exch{"2H16 PSF"}="H162 PSF"; $resn_exch{"1H17 PSF"}="H171 PSF"; $resn_exch{"2H17 PSF"}="H172 PSF";
$resn_exch{"3H17 PSF"}="H173 PSF"; $resn_exch{"2H6  PSG"}=" H62 PSG"; $resn_exch{"1H6  PSG"}=" H61 PSG";
$resn_exch{"1H2O PSG"}="H2O1 PSG"; $resn_exch{"1H3O PSG"}="H3O1 PSG"; $resn_exch{"1H4O PSG"}="H4O1 PSG";
$resn_exch{"1H6O PSG"}="H6O1 PSG"; $resn_exch{"1HN  PSH"}=" HN1 PSH"; $resn_exch{"2HN  PSH"}=" HN2 PSH";
$resn_exch{"1HB  PSH"}=" HB1 PSH"; $resn_exch{"2HB  PSH"}=" HB2 PSH"; $resn_exch{"1HB1 PSI"}="HB11 PSI";
$resn_exch{"2HB1 PSI"}="HB12 PSI"; $resn_exch{"3HB1 PSI"}="HB13 PSI"; $resn_exch{"1HN4 PSI"}="HN41 PSI";
$resn_exch{"2HN4 PSI"}="HN42 PSI"; $resn_exch{"1HB3 PSI"}="HB31 PSI"; $resn_exch{"2HB3 PSI"}="HB32 PSI";
$resn_exch{"3HB3 PSI"}="HB33 PSI"; $resn_exch{"1HB2 PSI"}="HB21 PSI"; $resn_exch{"2HB2 PSI"}="HB22 PSI";
$resn_exch{"1H44 PSI"}="H441 PSI"; $resn_exch{"2H44 PSI"}="H442 PSI"; $resn_exch{"1HA4 PSI"}="HA41 PSI";
$resn_exch{"2HA4 PSI"}="HA42 PSI"; $resn_exch{"1HG6 PSI"}="HG61 PSI"; $resn_exch{"2HG6 PSI"}="HG62 PSI";
$resn_exch{"3HG6 PSI"}="HG63 PSI"; $resn_exch{"1HG5 PSI"}="HG51 PSI"; $resn_exch{"2HG5 PSI"}="HG52 PSI";
$resn_exch{"3HG5 PSI"}="HG53 PSI"; $resn_exch{"1HG8 PSI"}="HG81 PSI"; $resn_exch{"2HG8 PSI"}="HG82 PSI";
$resn_exch{"3HG8 PSI"}="HG83 PSI"; $resn_exch{"1HG7 PSI"}="HG71 PSI"; $resn_exch{"2HG7 PSI"}="HG72 PSI";
$resn_exch{"3HG7 PSI"}="HG73 PSI"; $resn_exch{"1H76 PSI"}="H761 PSI"; $resn_exch{"2H76 PSI"}="H762 PSI";
$resn_exch{"3H76 PSI"}="H763 PSI"; $resn_exch{"1HAA PSK"}="HAA1 PSK"; $resn_exch{"2HAA PSK"}="HAA2 PSK";
$resn_exch{"3HAA PSK"}="HAA3 PSK"; $resn_exch{"1HAJ PSK"}="HAJ1 PSK"; $resn_exch{"2HAJ PSK"}="HAJ2 PSK";
$resn_exch{"1HAB PSK"}="HAB1 PSK"; $resn_exch{"2HAB PSK"}="HAB2 PSK"; $resn_exch{"3HAB PSK"}="HAB3 PSK";
$resn_exch{"1HAL PSK"}="HAL1 PSK"; $resn_exch{"2HAL PSK"}="HAL2 PSK"; $resn_exch{"1H19 PSM"}="H191 PSM";
$resn_exch{"2H19 PSM"}="H192 PSM"; $resn_exch{"1H20 PSM"}="H201 PSM"; $resn_exch{"2H20 PSM"}="H202 PSM";
$resn_exch{"1H32 PSM"}="H321 PSM"; $resn_exch{"2H32 PSM"}="H322 PSM"; $resn_exch{"1H3  PSM"}="H31A PSM";
$resn_exch{"2H3  PSM"}=" H32 PSM"; $resn_exch{"3H3  PSM"}=" H33 PSM"; $resn_exch{"1H9  PSM"}=" H91 PSM";
$resn_exch{"2H9  PSM"}=" H92 PSM"; $resn_exch{"1H8  PSM"}=" H81 PSM"; $resn_exch{"2H8  PSM"}=" H82 PSM";
$resn_exch{"1HN2 PSM"}="HN21 PSM"; $resn_exch{"2HN2 PSM"}="HN22 PSM"; $resn_exch{"1H6  PSM"}=" H61 PSM";
$resn_exch{"2H6  PSM"}=" H62 PSM"; $resn_exch{"3H6  PSM"}=" H63 PSM"; $resn_exch{"1H7  PSM"}=" H71 PSM";
$resn_exch{"2H7  PSM"}=" H72 PSM"; $resn_exch{"3H7  PSM"}=" H73 PSM"; $resn_exch{"1H12 PSN"}="H121 PSN";
$resn_exch{"2H12 PSN"}="H122 PSN"; $resn_exch{"1H12 PSO"}="H121 PSO"; $resn_exch{"2H12 PSO"}="H122 PSO";
$resn_exch{"3H12 PSO"}="H123 PSO"; $resn_exch{"1H13 PSO"}="H131 PSO"; $resn_exch{"2H13 PSO"}="H132 PSO";
$resn_exch{"1H15 PSO"}="H151 PSO"; $resn_exch{"2H15 PSO"}="H152 PSO"; $resn_exch{"3H15 PSO"}="H153 PSO";
$resn_exch{"1H16 PSO"}="H161 PSO"; $resn_exch{"2H16 PSO"}="H162 PSO"; $resn_exch{"3H16 PSO"}="H163 PSO";
$resn_exch{"AH10 PSP"}="H10A PSP"; $resn_exch{"1HO  PSQ"}=" HO1 PSQ"; $resn_exch{"1HN  PSQ"}=" HN1 PSQ";
$resn_exch{"2HN  PSQ"}=" HN2 PSQ"; $resn_exch{"1HN1 PSQ"}="HN11 PSQ"; $resn_exch{"1H1  PSQ"}=" H11 PSQ";
$resn_exch{"1H2  PSQ"}=" H21 PSQ"; $resn_exch{"2H2  PSQ"}=" H22 PSQ"; $resn_exch{"1H3  PSQ"}=" H31 PSQ";
$resn_exch{"2H3  PSQ"}=" H32 PSQ"; $resn_exch{"1H4  PSQ"}=" H41 PSQ"; $resn_exch{"2H4  PSQ"}=" H42 PSQ";
$resn_exch{"1HN2 PSQ"}="HN21 PSQ"; $resn_exch{"1HN3 PSQ"}="HN31 PSQ"; $resn_exch{"2HN3 PSQ"}="HN32 PSQ";
$resn_exch{"1H23 PSR"}="H231 PSR"; $resn_exch{"1H26 PSR"}="H261 PSR"; $resn_exch{"1H28 PSR"}="H281 PSR";
$resn_exch{"2H28 PSR"}="H282 PSR"; $resn_exch{"1H30 PSR"}="H301 PSR"; $resn_exch{"2H30 PSR"}="H302 PSR";
$resn_exch{"3H30 PSR"}="H303 PSR"; $resn_exch{"1H31 PSR"}="H311 PSR"; $resn_exch{"2H31 PSR"}="H312 PSR";
$resn_exch{"3H31 PSR"}="H313 PSR"; $resn_exch{"1H32 PSR"}="H321 PSR"; $resn_exch{"1H33 PSR"}="H331 PSR";
$resn_exch{"1H36 PSR"}="H361 PSR"; $resn_exch{"1H37 PSR"}="H371 PSR"; $resn_exch{"2H37 PSR"}="H372 PSR";
$resn_exch{"1H38 PSR"}="H381 PSR"; $resn_exch{"2H38 PSR"}="H382 PSR"; $resn_exch{"1H41 PSR"}="H411 PSR";
$resn_exch{"1H42 PSR"}="H421 PSR"; $resn_exch{"2H42 PSR"}="H422 PSR"; $resn_exch{"1H43 PSR"}="H431 PSR";
$resn_exch{"2H43 PSR"}="H432 PSR"; $resn_exch{"1HC2 PSR"}="HC21 PSR"; $resn_exch{"2HC2 PSR"}="HC22 PSR";
$resn_exch{"1HC3 PSR"}="HC31 PSR"; $resn_exch{"2HC3 PSR"}="HC32 PSR"; $resn_exch{"1HC4 PSR"}="HC41 PSR";
$resn_exch{"2HC4 PSR"}="HC42 PSR"; $resn_exch{"3HC4 PSR"}="HC43 PSR"; $resn_exch{"1H1  PSS"}=" H11 PSS";
$resn_exch{"2H1  PSS"}=" H12 PSS"; $resn_exch{"1H2  PSS"}=" H21 PSS"; $resn_exch{"2H2  PSS"}=" H22 PSS";
$resn_exch{"3H2  PSS"}=" H23 PSS"; $resn_exch{"1H'  PSS"}=" H'1 PSS"; $resn_exch{"2H'  PSS"}=" H'2 PSS";
$resn_exch{"1HOP PST"}="HOP1 PST"; $resn_exch{"2HOP PST"}="HOP2 PST"; $resn_exch{"1H5* PST"}=" H5' PST";
$resn_exch{"2H5* PST"}="H5'' PST"; $resn_exch{" H4* PST"}=" H4' PST"; $resn_exch{" H3* PST"}=" H3' PST";
$resn_exch{"1H2* PST"}=" H2' PST"; $resn_exch{"2H2* PST"}="H2'' PST"; $resn_exch{" H1* PST"}=" H1' PST";
$resn_exch{"1H5M PST"}=" H71 PST"; $resn_exch{"2H5M PST"}=" H72 PST"; $resn_exch{"3H5M PST"}=" H73 PST";
$resn_exch{" H1* PSU"}=" H1' PSU"; $resn_exch{" H2* PSU"}=" H2' PSU"; $resn_exch{"*HO2 PSU"}="HO2' PSU";
$resn_exch{" H3* PSU"}=" H3' PSU"; $resn_exch{" H4* PSU"}=" H4' PSU"; $resn_exch{"*HO3 PSU"}="HO3' PSU";
$resn_exch{"1H5* PSU"}=" H5' PSU"; $resn_exch{"2H5* PSU"}="H5'' PSU"; $resn_exch{"2HOP PSU"}="HOP2 PSU";
$resn_exch{"3HOP PSU"}="HOP3 PSU"; $resn_exch{"1HAA PSX"}="HAA1 PSX"; $resn_exch{"2HAA PSX"}="HAA2 PSX";
$resn_exch{"3HAA PSX"}="HAA3 PSX"; $resn_exch{"1H6  PSX"}=" H61 PSX"; $resn_exch{"2H6  PSX"}=" H62 PSX";
$resn_exch{"1HAB PSX"}="HAB1 PSX"; $resn_exch{"2HAB PSX"}="HAB2 PSX"; $resn_exch{"3HAB PSX"}="HAB3 PSX";
$resn_exch{"1H2  PSX"}=" H21 PSX"; $resn_exch{"2H2  PSX"}=" H22 PSX"; $resn_exch{"1H4A PSZ"}="H4A1 PSZ";
$resn_exch{"2H4A PSZ"}="H4A2 PSZ"; $resn_exch{"1H2A PSZ"}="H2A1 PSZ"; $resn_exch{"2H2A PSZ"}="H2A2 PSZ";
$resn_exch{"3H2A PSZ"}="H2A3 PSZ"; $resn_exch{"1H5A PSZ"}="H5A1 PSZ"; $resn_exch{"2H5A PSZ"}="H5A2 PSZ";
$resn_exch{"1H11 PT1"}="H111 PT1"; $resn_exch{"2H11 PT1"}="H112 PT1"; $resn_exch{"1H13 PT1"}="H131 PT1";
$resn_exch{"2H13 PT1"}="H132 PT1"; $resn_exch{"1H16 PT2"}="H161 PT2"; $resn_exch{"2H16 PT2"}="H162 PT2";
$resn_exch{"1H17 PT2"}="H171 PT2"; $resn_exch{"2H17 PT2"}="H172 PT2"; $resn_exch{"1H2  PT3"}=" H21 PT3";
$resn_exch{"1H3  PT3"}=" H31 PT3"; $resn_exch{"1NH4 PT3"}="H41N PT3"; $resn_exch{"1H41 PT3"}="H411 PT3";
$resn_exch{"2H41 PT3"}="H412 PT3"; $resn_exch{"1H42 PT3"}="H421 PT3"; $resn_exch{"2H42 PT3"}="H422 PT3";
$resn_exch{"1H43 PT3"}="H431 PT3"; $resn_exch{"2H43 PT3"}="H432 PT3"; $resn_exch{"3H43 PT3"}="H433 PT3";
$resn_exch{"1HN  PTA"}=" H   PTA"; $resn_exch{"2HN  PTA"}=" H2  PTA"; $resn_exch{"1HB  PTA"}=" HB2 PTA";
$resn_exch{"2HB  PTA"}=" HB3 PTA"; $resn_exch{"1HD1 PTA"}="HD11 PTA"; $resn_exch{"2HD1 PTA"}="HD12 PTA";
$resn_exch{"3HD1 PTA"}="HD13 PTA"; $resn_exch{"1HD2 PTA"}="HD21 PTA"; $resn_exch{"2HD2 PTA"}="HD22 PTA";
$resn_exch{"3HD2 PTA"}="HD23 PTA"; $resn_exch{"2HOP PTA"}="HOP2 PTA"; $resn_exch{"1HM  PTA"}=" HM1 PTA";
$resn_exch{"2HM  PTA"}=" HM2 PTA"; $resn_exch{"1HB  PTB"}=" HB1 PTB"; $resn_exch{"2HB  PTB"}=" HB2 PTB";
$resn_exch{"1HG  PTB"}=" HG1 PTB"; $resn_exch{"2HG  PTB"}=" HG2 PTB"; $resn_exch{"1HD  PTB"}=" HD1 PTB";
$resn_exch{"2HD  PTB"}=" HD2 PTB"; $resn_exch{"1H7  PTB"}=" H71 PTB"; $resn_exch{"2H7  PTB"}=" H72 PTB";
$resn_exch{"3H7  PTB"}=" H73 PTB"; $resn_exch{"1H8  PTB"}=" H81 PTB"; $resn_exch{"2H8  PTB"}=" H82 PTB";
$resn_exch{"3H8  PTB"}=" H83 PTB"; $resn_exch{"1H9  PTB"}=" H91 PTB"; $resn_exch{"2H9  PTB"}=" H92 PTB";
$resn_exch{"3H9  PTB"}=" H93 PTB"; $resn_exch{"1H2  PTC"}=" H21 PTC"; $resn_exch{"2H2  PTC"}=" H22 PTC";
$resn_exch{"3H2  PTC"}=" H23 PTC"; $resn_exch{"1HC2 PTD"}="HC21 PTD"; $resn_exch{"2HC2 PTD"}="HC22 PTD";
$resn_exch{"1HC3 PTD"}="HC31 PTD"; $resn_exch{"2HC3 PTD"}="HC32 PTD"; $resn_exch{"1HC4 PTD"}="HC41 PTD";
$resn_exch{"2HC4 PTD"}="HC42 PTD"; $resn_exch{"1H1  PTE"}=" H11 PTE"; $resn_exch{"2H1  PTE"}=" H12 PTE";
$resn_exch{"1H10 PTE"}="H101 PTE"; $resn_exch{"2H10 PTE"}="H102 PTE"; $resn_exch{"1H21 PTE"}="H211 PTE";
$resn_exch{"2H21 PTE"}="H212 PTE"; $resn_exch{"1H30 PTE"}="H301 PTE"; $resn_exch{"2H30 PTE"}="H302 PTE";
$resn_exch{"2HOP PTE"}="HOP2 PTE"; $resn_exch{"6HOP PTE"}="HOP6 PTE"; $resn_exch{"1HO1 PTE"}="HO11 PTE";
$resn_exch{"2HO1 PTE"}="HO12 PTE"; $resn_exch{"1HO2 PTE"}="HO21 PTE"; $resn_exch{"2HO2 PTE"}="HO22 PTE";
$resn_exch{"1H1  PTF"}=" H11 PTF"; $resn_exch{"2H1  PTF"}=" H12 PTF"; $resn_exch{"3H1  PTF"}=" H13 PTF";
$resn_exch{"1H3  PTF"}=" H31 PTF"; $resn_exch{"2H3  PTF"}=" H32 PTF"; $resn_exch{"1HN  PTH"}=" H   PTH";
$resn_exch{"2HN  PTH"}=" H2  PTH"; $resn_exch{"1HB  PTH"}=" HB2 PTH"; $resn_exch{"2HB  PTH"}=" HB3 PTH";
$resn_exch{"1HF  PTH"}=" HF1 PTH"; $resn_exch{"2HF  PTH"}=" HF2 PTH"; $resn_exch{"PHO2 PTH"}="HO2P PTH";
$resn_exch{"PHO3 PTH"}="HO3P PTH"; $resn_exch{"1H3C PTI"}="H3C1 PTI"; $resn_exch{"2H3C PTI"}="H3C2 PTI";
$resn_exch{"1H6C PTI"}="H6C1 PTI"; $resn_exch{"2H6C PTI"}="H6C2 PTI"; $resn_exch{"1H9C PTI"}="H9C1 PTI";
$resn_exch{"2H9C PTI"}="H9C2 PTI"; $resn_exch{"1H12 PTI"}="H121 PTI"; $resn_exch{"2H12 PTI"}="H122 PTI";
$resn_exch{"1H15 PTI"}="H151 PTI"; $resn_exch{"2H15 PTI"}="H152 PTI"; $resn_exch{"1H18 PTI"}="H181 PTI";
$resn_exch{"2H18 PTI"}="H182 PTI"; $resn_exch{"1H21 PTI"}="H211 PTI"; $resn_exch{"2H21 PTI"}="H212 PTI";
$resn_exch{"1H38 PTI"}="H381 PTI"; $resn_exch{"2H38 PTI"}="H382 PTI"; $resn_exch{"1H41 PTI"}="H411 PTI";
$resn_exch{"2H41 PTI"}="H412 PTI"; $resn_exch{"1H2  PTL"}=" H21 PTL"; $resn_exch{"2H2  PTL"}=" H22 PTL";
$resn_exch{"1H3  PTL"}=" H31 PTL"; $resn_exch{"2H3  PTL"}=" H32 PTL"; $resn_exch{"1H4  PTL"}=" H41 PTL";
$resn_exch{"2H4  PTL"}=" H42 PTL"; $resn_exch{"1H5  PTL"}=" H51 PTL"; $resn_exch{"2H5  PTL"}=" H52 PTL";
$resn_exch{"3H5  PTL"}=" H53 PTL"; $resn_exch{" HN2 PTM"}=" H2  PTM"; $resn_exch{"1HC3 PTM"}="HC31 PTM";
$resn_exch{"2HC3 PTM"}="HC32 PTM"; $resn_exch{"3HC3 PTM"}="HC33 PTM"; $resn_exch{"1HB  PTM"}=" HB2 PTM";
$resn_exch{"2HB  PTM"}=" HB3 PTM"; $resn_exch{"1H1  PTN"}=" H11 PTN"; $resn_exch{"2H1  PTN"}=" H12 PTN";
$resn_exch{"3H1  PTN"}=" H13 PTN"; $resn_exch{"1H2  PTN"}=" H21 PTN"; $resn_exch{"2H2  PTN"}=" H22 PTN";
$resn_exch{"3H2  PTN"}=" H23 PTN"; $resn_exch{"1H3  PTN"}=" H31 PTN"; $resn_exch{"2H3  PTN"}=" H32 PTN";
$resn_exch{"3H3  PTN"}=" H33 PTN"; $resn_exch{"1H2  PTO"}=" H21 PTO"; $resn_exch{"2H2  PTO"}=" H22 PTO";
$resn_exch{"1H4  PTO"}=" H41 PTO"; $resn_exch{"2H4  PTO"}=" H42 PTO"; $resn_exch{"1H6  PTO"}=" H61 PTO";
$resn_exch{"2H6  PTO"}=" H62 PTO"; $resn_exch{"1H7  PTO"}=" H71 PTO"; $resn_exch{"2H7  PTO"}=" H72 PTO";
$resn_exch{"1H9  PTO"}=" H91 PTO"; $resn_exch{"2H9  PTO"}=" H92 PTO"; $resn_exch{"3H9  PTO"}=" H93 PTO";
$resn_exch{"1H5M PTP"}="H5M1 PTP"; $resn_exch{"2H5M PTP"}="H5M2 PTP"; $resn_exch{"3H5M PTP"}="H5M3 PTP";
$resn_exch{" H1* PTP"}=" H1' PTP"; $resn_exch{"1H2* PTP"}="H2'1 PTP"; $resn_exch{"2H2* PTP"}="H2'2 PTP";
$resn_exch{" H3* PTP"}=" H3' PTP"; $resn_exch{" H4* PTP"}=" H4' PTP"; $resn_exch{"1H5* PTP"}="H5'1 PTP";
$resn_exch{"2H5* PTP"}="H5'2 PTP"; $resn_exch{"1HN  PTR"}=" H   PTR"; $resn_exch{"2HN  PTR"}=" HN2 PTR";
$resn_exch{"1HB  PTR"}=" HB2 PTR"; $resn_exch{"2HB  PTR"}=" HB3 PTR"; $resn_exch{"PHO2 PTR"}="HO2P PTR";
$resn_exch{"PHO3 PTR"}="HO3P PTR"; $resn_exch{"1HN3 PTS"}="HN31 PTS"; $resn_exch{"2HN3 PTS"}="HN32 PTS";
$resn_exch{"1HN4 PTS"}="HN41 PTS"; $resn_exch{"2HN4 PTS"}="HN42 PTS"; $resn_exch{"1H15 PTS"}="H151 PTS";
$resn_exch{"2H15 PTS"}="H152 PTS"; $resn_exch{"3H15 PTS"}="H153 PTS"; $resn_exch{"1H10 PTT"}="H101 PTT";
$resn_exch{"2H10 PTT"}="H102 PTT"; $resn_exch{"1H30 PTT"}="H301 PTT"; $resn_exch{"2H30 PTT"}="H302 PTT";
$resn_exch{"1HA1 PTT"}="HA11 PTT"; $resn_exch{"2HA1 PTT"}="HA12 PTT"; $resn_exch{"3HA1 PTT"}="HA13 PTT";
$resn_exch{"3HOP PTT"}="HOP3 PTT"; $resn_exch{"7HOP PTT"}="HOP7 PTT"; $resn_exch{"1HN1 PTU"}="HN11 PTU";
$resn_exch{"2HN1 PTU"}="HN12 PTU"; $resn_exch{"1H2  PTU"}=" H21 PTU"; $resn_exch{"1H3  PTU"}=" H31 PTU";
$resn_exch{"1H4  PTU"}=" H41 PTU"; $resn_exch{"1H5  PTU"}=" H51 PTU"; $resn_exch{"1H6  PTU"}=" H61 PTU";
$resn_exch{"1H1' PTU"}="H1'1 PTU"; $resn_exch{"2H1' PTU"}="H1'2 PTU"; $resn_exch{"1H2' PTU"}="H2'1 PTU";
$resn_exch{"2H2' PTU"}="H2'2 PTU"; $resn_exch{"3H2' PTU"}="H2'3 PTU"; $resn_exch{"1H7  PTX"}=" H71 PTX";
$resn_exch{"2H7  PTX"}=" H72 PTX"; $resn_exch{"1H18 PTX"}="H181 PTX"; $resn_exch{"2H18 PTX"}="H182 PTX";
$resn_exch{"3H18 PTX"}="H183 PTX"; $resn_exch{"1H1  PTX"}=" H11 PTX"; $resn_exch{"2H1  PTX"}=" H12 PTX";
$resn_exch{"1HC1 PTY"}="HC11 PTY"; $resn_exch{"2HC1 PTY"}="HC12 PTY"; $resn_exch{"1HC2 PTY"}="HC21 PTY";
$resn_exch{"2HC2 PTY"}="HC22 PTY"; $resn_exch{"1HC3 PTY"}="HC31 PTY"; $resn_exch{"2HC3 PTY"}="HC32 PTY";
$resn_exch{"1HC5 PTY"}="HC51 PTY"; $resn_exch{"2HC5 PTY"}="HC52 PTY"; $resn_exch{"1H11 PTY"}="H111 PTY";
$resn_exch{"2H11 PTY"}="H112 PTY"; $resn_exch{"1H12 PTY"}="H121 PTY"; $resn_exch{"2H12 PTY"}="H122 PTY";
$resn_exch{"1H13 PTY"}="H131 PTY"; $resn_exch{"2H13 PTY"}="H132 PTY"; $resn_exch{"1H14 PTY"}="H141 PTY";
$resn_exch{"2H14 PTY"}="H142 PTY"; $resn_exch{"1H15 PTY"}="H151 PTY"; $resn_exch{"2H15 PTY"}="H152 PTY";
$resn_exch{"1H16 PTY"}="H161 PTY"; $resn_exch{"2H16 PTY"}="H162 PTY"; $resn_exch{"1H17 PTY"}="H171 PTY";
$resn_exch{"2H17 PTY"}="H172 PTY"; $resn_exch{"1H18 PTY"}="H181 PTY"; $resn_exch{"2H18 PTY"}="H182 PTY";
$resn_exch{"1H19 PTY"}="H191 PTY"; $resn_exch{"2H19 PTY"}="H192 PTY"; $resn_exch{"1H20 PTY"}="H201 PTY";
$resn_exch{"2H20 PTY"}="H202 PTY"; $resn_exch{"1H21 PTY"}="H211 PTY"; $resn_exch{"2H21 PTY"}="H212 PTY";
$resn_exch{"1H22 PTY"}="H221 PTY"; $resn_exch{"2H22 PTY"}="H222 PTY"; $resn_exch{"1H23 PTY"}="H231 PTY";
$resn_exch{"2H23 PTY"}="H232 PTY"; $resn_exch{"1H24 PTY"}="H241 PTY"; $resn_exch{"2H24 PTY"}="H242 PTY";
$resn_exch{"1H25 PTY"}="H251 PTY"; $resn_exch{"2H25 PTY"}="H252 PTY"; $resn_exch{"1H26 PTY"}="H261 PTY";
$resn_exch{"2H26 PTY"}="H262 PTY"; $resn_exch{"1H27 PTY"}="H271 PTY"; $resn_exch{"2H27 PTY"}="H272 PTY";
$resn_exch{"1H28 PTY"}="H281 PTY"; $resn_exch{"2H28 PTY"}="H282 PTY"; $resn_exch{"1H29 PTY"}="H291 PTY";
$resn_exch{"2H29 PTY"}="H292 PTY"; $resn_exch{"3H29 PTY"}="H293 PTY"; $resn_exch{"1H31 PTY"}="H311 PTY";
$resn_exch{"2H31 PTY"}="H312 PTY"; $resn_exch{"1H32 PTY"}="H321 PTY"; $resn_exch{"2H32 PTY"}="H322 PTY";
$resn_exch{"1H33 PTY"}="H331 PTY"; $resn_exch{"2H33 PTY"}="H332 PTY"; $resn_exch{"1H34 PTY"}="H341 PTY";
$resn_exch{"2H34 PTY"}="H342 PTY"; $resn_exch{"1H35 PTY"}="H351 PTY"; $resn_exch{"2H35 PTY"}="H352 PTY";
$resn_exch{"1H36 PTY"}="H361 PTY"; $resn_exch{"2H36 PTY"}="H362 PTY"; $resn_exch{"1H37 PTY"}="H371 PTY";
$resn_exch{"2H37 PTY"}="H372 PTY"; $resn_exch{"1H38 PTY"}="H381 PTY"; $resn_exch{"2H38 PTY"}="H382 PTY";
$resn_exch{"1H39 PTY"}="H391 PTY"; $resn_exch{"2H39 PTY"}="H392 PTY"; $resn_exch{"1H40 PTY"}="H401 PTY";
$resn_exch{"2H40 PTY"}="H402 PTY"; $resn_exch{"1H41 PTY"}="H411 PTY"; $resn_exch{"2H41 PTY"}="H412 PTY";
$resn_exch{"1H42 PTY"}="H421 PTY"; $resn_exch{"2H42 PTY"}="H422 PTY"; $resn_exch{"1H43 PTY"}="H431 PTY";
$resn_exch{"2H43 PTY"}="H432 PTY"; $resn_exch{"1H44 PTY"}="H441 PTY"; $resn_exch{"2H44 PTY"}="H442 PTY";
$resn_exch{"3H44 PTY"}="H443 PTY"; $resn_exch{"OH12 PTY"}="H12O PTY"; $resn_exch{"1HN1 PTY"}="HN11 PTY";
$resn_exch{"2HN1 PTY"}="HN12 PTY"; $resn_exch{"2HOP  PU"}="HOP2  PU"; $resn_exch{"3HOP  PU"}="HOP3  PU";
$resn_exch{"1HN   PU"}=" HN1  PU"; $resn_exch{"1HB   PU"}=" HB1  PU"; $resn_exch{"2HB   PU"}=" HB2  PU";
$resn_exch{"1HM   PU"}=" HM1  PU"; $resn_exch{"2HM   PU"}=" HM2  PU"; $resn_exch{"3HM   PU"}=" HM3  PU";
$resn_exch{"H5T   PU"}="HO5'  PU"; $resn_exch{"1H5*  PU"}=" H5'  PU"; $resn_exch{"2H5*  PU"}="H5''  PU";
$resn_exch{" H4*  PU"}=" H4'  PU"; $resn_exch{" H3*  PU"}=" H3'  PU"; $resn_exch{"3HN*  PU"}="HN'3  PU";
$resn_exch{" H2*  PU"}=" H2'  PU"; $resn_exch{"*HO2  PU"}="HO2'  PU"; $resn_exch{" H1*  PU"}=" H1'  PU";
$resn_exch{"1H9   PU"}=" H91  PU"; $resn_exch{"2H9   PU"}=" H92  PU"; $resn_exch{"3H9   PU"}=" H93  PU";
$resn_exch{"1H10  PU"}="H101  PU"; $resn_exch{"2H10  PU"}="H102  PU"; $resn_exch{"3H10  PU"}="H103  PU";
$resn_exch{"1H19 PU0"}="H191 PU0"; $resn_exch{"2H19 PU0"}="H192 PU0"; $resn_exch{"3H19 PU0"}="H193 PU0";
$resn_exch{"1H8C PU0"}="H8C1 PU0"; $resn_exch{"2H8C PU0"}="H8C2 PU0"; $resn_exch{"3H8C PU0"}="H8C3 PU0";
$resn_exch{"1H9C PU0"}="H9C1 PU0"; $resn_exch{"2H9C PU0"}="H9C2 PU0"; $resn_exch{"1H5N PU0"}="H5N1 PU0";
$resn_exch{"2H5N PU0"}="H5N2 PU0"; $resn_exch{"1H15 PU0"}="H151 PU0"; $resn_exch{"2H15 PU0"}="H152 PU0";
$resn_exch{"1H16 PU0"}="H161 PU0"; $resn_exch{"2H16 PU0"}="H162 PU0"; $resn_exch{"1H17 PU0"}="H171 PU0";
$resn_exch{"2H17 PU0"}="H172 PU0"; $resn_exch{"1H18 PU0"}="H181 PU0"; $resn_exch{"2H18 PU0"}="H182 PU0";
$resn_exch{"3H18 PU0"}="H183 PU0"; $resn_exch{"1H20 PU1"}="H201 PU1"; $resn_exch{"2H20 PU1"}="H202 PU1";
$resn_exch{"3H20 PU1"}="H203 PU1"; $resn_exch{"1H19 PU1"}="H191 PU1"; $resn_exch{"2H19 PU1"}="H192 PU1";
$resn_exch{"3H19 PU1"}="H193 PU1"; $resn_exch{"1H18 PU1"}="H181 PU1"; $resn_exch{"2H18 PU1"}="H182 PU1";
$resn_exch{"3H18 PU1"}="H183 PU1"; $resn_exch{"1H6C PU1"}="H6C1 PU1"; $resn_exch{"2H6C PU1"}="H6C2 PU1";
$resn_exch{"1H5N PU1"}="H5N1 PU1"; $resn_exch{"2H5N PU1"}="H5N2 PU1"; $resn_exch{"1H7C PU1"}="H7C1 PU1";
$resn_exch{"2H7C PU1"}="H7C2 PU1"; $resn_exch{"1H8C PU1"}="H8C1 PU1"; $resn_exch{"2H8C PU1"}="H8C2 PU1";
$resn_exch{"1H9C PU1"}="H9C1 PU1"; $resn_exch{"2H9C PU1"}="H9C2 PU1"; $resn_exch{"1H18 PU2"}="H181 PU2";
$resn_exch{"2H18 PU2"}="H182 PU2"; $resn_exch{"3H18 PU2"}="H183 PU2"; $resn_exch{"1H9C PU2"}="H9C1 PU2";
$resn_exch{"2H9C PU2"}="H9C2 PU2"; $resn_exch{"1H5N PU2"}="H5N1 PU2"; $resn_exch{"2H5N PU2"}="H5N2 PU2";
$resn_exch{"1H7C PU2"}="H7C1 PU2"; $resn_exch{"2H7C PU2"}="H7C2 PU2"; $resn_exch{"3H7C PU2"}="H7C3 PU2";
$resn_exch{"1H19 PU3"}="H191 PU3"; $resn_exch{"2H19 PU3"}="H192 PU3"; $resn_exch{"3H19 PU3"}="H193 PU3";
$resn_exch{"1H7C PU3"}="H7C1 PU3"; $resn_exch{"2H7C PU3"}="H7C2 PU3"; $resn_exch{"3H7C PU3"}="H7C3 PU3";
$resn_exch{"1H8C PU3"}="H8C1 PU3"; $resn_exch{"2H8C PU3"}="H8C2 PU3"; $resn_exch{"3H8C PU3"}="H8C3 PU3";
$resn_exch{"1H9C PU3"}="H9C1 PU3"; $resn_exch{"2H9C PU3"}="H9C2 PU3"; $resn_exch{"1H5N PU3"}="H5N1 PU3";
$resn_exch{"2H5N PU3"}="H5N2 PU3"; $resn_exch{"1H15 PU3"}="H151 PU3"; $resn_exch{"2H15 PU3"}="H152 PU3";
$resn_exch{"1H16 PU3"}="H161 PU3"; $resn_exch{"2H16 PU3"}="H162 PU3"; $resn_exch{"1H17 PU3"}="H171 PU3";
$resn_exch{"2H17 PU3"}="H172 PU3"; $resn_exch{"1H18 PU3"}="H181 PU3"; $resn_exch{"2H18 PU3"}="H182 PU3";
$resn_exch{"3H18 PU3"}="H183 PU3"; $resn_exch{"1H18 PU4"}="H181 PU4"; $resn_exch{"2H18 PU4"}="H182 PU4";
$resn_exch{"3H18 PU4"}="H183 PU4"; $resn_exch{"1H17 PU4"}="H171 PU4"; $resn_exch{"2H17 PU4"}="H172 PU4";
$resn_exch{"1H16 PU4"}="H161 PU4"; $resn_exch{"2H16 PU4"}="H162 PU4"; $resn_exch{"1H15 PU4"}="H151 PU4";
$resn_exch{"2H15 PU4"}="H152 PU4"; $resn_exch{"1H5N PU4"}="H5N1 PU4"; $resn_exch{"2H5N PU4"}="H5N2 PU4";
$resn_exch{"1H9C PU4"}="H9C1 PU4"; $resn_exch{"2H9C PU4"}="H9C2 PU4"; $resn_exch{"1H7C PU4"}="H7C1 PU4";
$resn_exch{"2H7C PU4"}="H7C2 PU4"; $resn_exch{"3H7C PU4"}="H7C3 PU4"; $resn_exch{"1H18 PU5"}="H181 PU5";
$resn_exch{"2H18 PU5"}="H182 PU5"; $resn_exch{"3H18 PU5"}="H183 PU5"; $resn_exch{"1H17 PU5"}="H171 PU5";
$resn_exch{"2H17 PU5"}="H172 PU5"; $resn_exch{"1H16 PU5"}="H161 PU5"; $resn_exch{"2H16 PU5"}="H162 PU5";
$resn_exch{"1H15 PU5"}="H151 PU5"; $resn_exch{"2H15 PU5"}="H152 PU5"; $resn_exch{"1H9C PU5"}="H9C1 PU5";
$resn_exch{"2H9C PU5"}="H9C2 PU5"; $resn_exch{"1H5N PU5"}="H5N1 PU5"; $resn_exch{"2H5N PU5"}="H5N2 PU5";
$resn_exch{"1H7C PU5"}="H7C1 PU5"; $resn_exch{"2H7C PU5"}="H7C2 PU5"; $resn_exch{"3H7C PU5"}="H7C3 PU5";
$resn_exch{"1H18 PU6"}="H181 PU6"; $resn_exch{"2H18 PU6"}="H182 PU6"; $resn_exch{"3H18 PU6"}="H183 PU6";
$resn_exch{"1H17 PU6"}="H171 PU6"; $resn_exch{"2H17 PU6"}="H172 PU6"; $resn_exch{"1H16 PU6"}="H161 PU6";
$resn_exch{"2H16 PU6"}="H162 PU6"; $resn_exch{"1H15 PU6"}="H151 PU6"; $resn_exch{"2H15 PU6"}="H152 PU6";
$resn_exch{"1H5N PU6"}="H5N1 PU6"; $resn_exch{"2H5N PU6"}="H5N2 PU6"; $resn_exch{"1H9C PU6"}="H9C1 PU6";
$resn_exch{"2H9C PU6"}="H9C2 PU6"; $resn_exch{"1H7C PU6"}="H7C1 PU6"; $resn_exch{"2H7C PU6"}="H7C2 PU6";
$resn_exch{"1H19 PU7"}="H191 PU7"; $resn_exch{"2H19 PU7"}="H192 PU7"; $resn_exch{"3H19 PU7"}="H193 PU7";
$resn_exch{"1H8C PU7"}="H8C1 PU7"; $resn_exch{"2H8C PU7"}="H8C2 PU7"; $resn_exch{"3H8C PU7"}="H8C3 PU7";
$resn_exch{"1H9C PU7"}="H9C1 PU7"; $resn_exch{"2H9C PU7"}="H9C2 PU7"; $resn_exch{"1H5N PU7"}="H5N1 PU7";
$resn_exch{"2H5N PU7"}="H5N2 PU7"; $resn_exch{"1H15 PU7"}="H151 PU7"; $resn_exch{"2H15 PU7"}="H152 PU7";
$resn_exch{"1H16 PU7"}="H161 PU7"; $resn_exch{"2H16 PU7"}="H162 PU7"; $resn_exch{"1H17 PU7"}="H171 PU7";
$resn_exch{"2H17 PU7"}="H172 PU7"; $resn_exch{"1H18 PU7"}="H181 PU7"; $resn_exch{"2H18 PU7"}="H182 PU7";
$resn_exch{"3H18 PU7"}="H183 PU7"; $resn_exch{"1H19 PU8"}="H191 PU8"; $resn_exch{"2H19 PU8"}="H192 PU8";
$resn_exch{"3H19 PU8"}="H193 PU8"; $resn_exch{"1H7C PU8"}="H7C1 PU8"; $resn_exch{"2H7C PU8"}="H7C2 PU8";
$resn_exch{"3H7C PU8"}="H7C3 PU8"; $resn_exch{"1H8C PU8"}="H8C1 PU8"; $resn_exch{"2H8C PU8"}="H8C2 PU8";
$resn_exch{"3H8C PU8"}="H8C3 PU8"; $resn_exch{"1H9C PU8"}="H9C1 PU8"; $resn_exch{"2H9C PU8"}="H9C2 PU8";
$resn_exch{"1H5N PU8"}="H5N1 PU8"; $resn_exch{"2H5N PU8"}="H5N2 PU8"; $resn_exch{"1H15 PU8"}="H151 PU8";
$resn_exch{"2H15 PU8"}="H152 PU8"; $resn_exch{"1H16 PU8"}="H161 PU8"; $resn_exch{"2H16 PU8"}="H162 PU8";
$resn_exch{"1H17 PU8"}="H171 PU8"; $resn_exch{"2H17 PU8"}="H172 PU8"; $resn_exch{"1H18 PU8"}="H181 PU8";
$resn_exch{"2H18 PU8"}="H182 PU8"; $resn_exch{"3H18 PU8"}="H183 PU8"; $resn_exch{"1H20 PU9"}="H201 PU9";
$resn_exch{"2H20 PU9"}="H202 PU9"; $resn_exch{"3H20 PU9"}="H203 PU9"; $resn_exch{"1H19 PU9"}="H191 PU9";
$resn_exch{"2H19 PU9"}="H192 PU9"; $resn_exch{"3H19 PU9"}="H193 PU9"; $resn_exch{"1H18 PU9"}="H181 PU9";
$resn_exch{"2H18 PU9"}="H182 PU9"; $resn_exch{"3H18 PU9"}="H183 PU9"; $resn_exch{"1H6C PU9"}="H6C1 PU9";
$resn_exch{"2H6C PU9"}="H6C2 PU9"; $resn_exch{"1H5N PU9"}="H5N1 PU9"; $resn_exch{"2H5N PU9"}="H5N2 PU9";
$resn_exch{"1H7C PU9"}="H7C1 PU9"; $resn_exch{"2H7C PU9"}="H7C2 PU9"; $resn_exch{"1H8C PU9"}="H8C1 PU9";
$resn_exch{"2H8C PU9"}="H8C2 PU9"; $resn_exch{"1H9C PU9"}="H9C1 PU9"; $resn_exch{"2H9C PU9"}="H9C2 PU9";
$resn_exch{"1H6A PUA"}="H6A1 PUA"; $resn_exch{"2H6A PUA"}="H6A2 PUA"; $resn_exch{"1H5* PUA"}="H5'1 PUA";
$resn_exch{"2H5* PUA"}="H5'2 PUA"; $resn_exch{" H4* PUA"}=" H4' PUA"; $resn_exch{" H1* PUA"}=" H1' PUA";
$resn_exch{" H2* PUA"}=" H2' PUA"; $resn_exch{"HO2* PUA"}="HO2' PUA"; $resn_exch{" H3* PUA"}=" H3' PUA";
$resn_exch{"1H5  PUA"}=" H51 PUA"; $resn_exch{"2H5  PUA"}=" H52 PUA"; $resn_exch{"1H2  PUA"}=" H21 PUA";
$resn_exch{"2H2  PUA"}=" H22 PUA"; $resn_exch{"1HHC PUB"}="HHC1 PUB"; $resn_exch{"2HHC PUB"}="HHC2 PUB";
$resn_exch{"1HMC PUB"}="HMC1 PUB"; $resn_exch{"2HMC PUB"}="HMC2 PUB"; $resn_exch{"3HMC PUB"}="HMC3 PUB";
$resn_exch{"1HAC PUB"}="HAC1 PUB"; $resn_exch{"2HAC PUB"}="HAC2 PUB"; $resn_exch{"1HBC PUB"}="HBC1 PUB";
$resn_exch{"2HBC PUB"}="HBC2 PUB"; $resn_exch{"2HOC PUB"}="HOC2 PUB"; $resn_exch{"1H1D PUB"}="H1D1 PUB";
$resn_exch{"1HMD PUB"}="HMD1 PUB"; $resn_exch{"2HMD PUB"}="HMD2 PUB"; $resn_exch{"3HMD PUB"}="HMD3 PUB";
$resn_exch{"1HBD PUB"}="HBD1 PUB"; $resn_exch{"2HBD PUB"}="HBD2 PUB"; $resn_exch{"3HBD PUB"}="HBD3 PUB";
$resn_exch{"1H4A PUB"}="H4A1 PUB"; $resn_exch{"1HMA PUB"}="HMA1 PUB"; $resn_exch{"2HMA PUB"}="HMA2 PUB";
$resn_exch{"3HMA PUB"}="HMA3 PUB"; $resn_exch{"1HBA PUB"}="HBA1 PUB"; $resn_exch{"2HBA PUB"}="HBA2 PUB";
$resn_exch{"3HBA PUB"}="HBA3 PUB"; $resn_exch{"1HHA PUB"}="HHA1 PUB"; $resn_exch{"2HHA PUB"}="HHA2 PUB";
$resn_exch{"1HAA PUB"}="HAA1 PUB"; $resn_exch{"2HAA PUB"}="HAA2 PUB"; $resn_exch{"1HAD PUB"}="HAD1 PUB";
$resn_exch{"2HAD PUB"}="HAD2 PUB"; $resn_exch{"1HHB PUB"}="HHB1 PUB"; $resn_exch{"1HMB PUB"}="HMB1 PUB";
$resn_exch{"2HMB PUB"}="HMB2 PUB"; $resn_exch{"3HMB PUB"}="HMB3 PUB"; $resn_exch{"1HAB PUB"}="HAB1 PUB";
$resn_exch{"2HAB PUB"}="HAB2 PUB"; $resn_exch{"1HBB PUB"}="HBB1 PUB"; $resn_exch{"2HBB PUB"}="HBB2 PUB";
$resn_exch{"2HOB PUB"}="HOB2 PUB"; $resn_exch{"1H10 PUG"}="H101 PUG"; $resn_exch{"2H10 PUG"}="H102 PUG";
$resn_exch{"1H14 PUG"}="H141 PUG"; $resn_exch{"2H14 PUG"}="H142 PUG"; $resn_exch{"1H20 PUL"}="H201 PUL";
$resn_exch{"2H20 PUL"}="H202 PUL"; $resn_exch{"1H43 PUL"}="H431 PUL"; $resn_exch{"2H43 PUL"}="H432 PUL";
$resn_exch{"3H43 PUL"}="H433 PUL"; $resn_exch{"1H42 PUL"}="H421 PUL"; $resn_exch{"2H42 PUL"}="H422 PUL";
$resn_exch{"3H42 PUL"}="H423 PUL"; $resn_exch{"1H41 PUL"}="H411 PUL"; $resn_exch{"2H41 PUL"}="H412 PUL";
$resn_exch{"3H41 PUL"}="H413 PUL"; $resn_exch{"1H2C PUL"}="H2C1 PUL"; $resn_exch{"2H2C PUL"}="H2C2 PUL";
$resn_exch{"1H44 PUL"}="H441 PUL"; $resn_exch{"2H44 PUL"}="H442 PUL"; $resn_exch{"3H44 PUL"}="H443 PUL";
$resn_exch{"1H45 PUL"}="H451 PUL"; $resn_exch{"2H45 PUL"}="H452 PUL"; $resn_exch{"3H45 PUL"}="H453 PUL";
$resn_exch{"1H46 PUL"}="H461 PUL"; $resn_exch{"2H46 PUL"}="H462 PUL"; $resn_exch{"3H46 PUL"}="H463 PUL";
$resn_exch{"1H47 PUL"}="H471 PUL"; $resn_exch{"2H47 PUL"}="H472 PUL"; $resn_exch{"3H47 PUL"}="H473 PUL";
$resn_exch{"1H40 PUL"}="H401 PUL"; $resn_exch{"2H40 PUL"}="H402 PUL"; $resn_exch{"3H40 PUL"}="H403 PUL";
$resn_exch{"1H34 PUL"}="H341 PUL"; $resn_exch{"2H34 PUL"}="H342 PUL"; $resn_exch{"3H34 PUL"}="H343 PUL";
$resn_exch{" H1* PUR"}=" H1' PUR"; $resn_exch{" H2* PUR"}=" H2' PUR"; $resn_exch{"*HO2 PUR"}="HO2' PUR";
$resn_exch{" H3* PUR"}=" H3' PUR"; $resn_exch{"*HO3 PUR"}="HO3' PUR"; $resn_exch{" H4* PUR"}=" H4' PUR";
$resn_exch{"1H5* PUR"}="H5'1 PUR"; $resn_exch{"2H5* PUR"}="H5'2 PUR"; $resn_exch{"*HO5 PUR"}="HO5' PUR";
$resn_exch{"1HN2 PUT"}="HN21 PUT"; $resn_exch{"2HN2 PUT"}="HN22 PUT"; $resn_exch{"1H4  PUT"}=" H41 PUT";
$resn_exch{"2H4  PUT"}=" H42 PUT"; $resn_exch{"1H3  PUT"}=" H31 PUT"; $resn_exch{"2H3  PUT"}=" H32 PUT";
$resn_exch{"1H2  PUT"}=" H21 PUT"; $resn_exch{"2H2  PUT"}=" H22 PUT"; $resn_exch{"1H1  PUT"}=" H11 PUT";
$resn_exch{"2H1  PUT"}=" H12 PUT"; $resn_exch{"1HN1 PUT"}="HN11 PUT"; $resn_exch{"2HN1 PUT"}="HN12 PUT";
$resn_exch{"1H18 PUX"}="H181 PUX"; $resn_exch{"2H18 PUX"}="H182 PUX"; $resn_exch{"3H18 PUX"}="H183 PUX";
$resn_exch{"1H17 PUX"}="H171 PUX"; $resn_exch{"2H17 PUX"}="H172 PUX"; $resn_exch{"1H16 PUX"}="H161 PUX";
$resn_exch{"2H16 PUX"}="H162 PUX"; $resn_exch{"1H15 PUX"}="H151 PUX"; $resn_exch{"2H15 PUX"}="H152 PUX";
$resn_exch{"1H5N PUX"}="H5N1 PUX"; $resn_exch{"2H5N PUX"}="H5N2 PUX"; $resn_exch{"1H9C PUX"}="H9C1 PUX";
$resn_exch{"2H9C PUX"}="H9C2 PUX"; $resn_exch{"1H7C PUX"}="H7C1 PUX"; $resn_exch{"2H7C PUX"}="H7C2 PUX";
$resn_exch{"1HN  PUY"}=" HN1 PUY"; $resn_exch{"2HN  PUY"}=" HN2 PUY"; $resn_exch{"1HB  PUY"}=" HB1 PUY";
$resn_exch{"2HB  PUY"}=" HB2 PUY"; $resn_exch{"1HMZ PUY"}="HMZ1 PUY"; $resn_exch{"2HMZ PUY"}="HMZ2 PUY";
$resn_exch{"3HMZ PUY"}="HMZ3 PUY"; $resn_exch{" H5* PUY"}=" H5' PUY"; $resn_exch{"1H5* PUY"}="H5'1 PUY";
$resn_exch{"2H5* PUY"}="H5'' PUY"; $resn_exch{" H4* PUY"}=" H4' PUY"; $resn_exch{" H1* PUY"}=" H1' PUY";
$resn_exch{"1H9  PUY"}=" H91 PUY"; $resn_exch{"2H9  PUY"}=" H92 PUY"; $resn_exch{"3H9  PUY"}=" H93 PUY";
$resn_exch{"1H10 PUY"}="H101 PUY"; $resn_exch{"2H10 PUY"}="H102 PUY"; $resn_exch{"3H10 PUY"}="H103 PUY";
$resn_exch{" H2* PUY"}=" H2' PUY"; $resn_exch{" H3* PUY"}=" H3' PUY"; $resn_exch{"1H17 PUZ"}="H171 PUZ";
$resn_exch{"2H17 PUZ"}="H172 PUZ"; $resn_exch{"1H16 PUZ"}="H161 PUZ"; $resn_exch{"2H16 PUZ"}="H162 PUZ";
$resn_exch{"1H15 PUZ"}="H151 PUZ"; $resn_exch{"2H15 PUZ"}="H152 PUZ"; $resn_exch{"1H5N PUZ"}="H5N1 PUZ";
$resn_exch{"2H5N PUZ"}="H5N2 PUZ"; $resn_exch{"1H9C PUZ"}="H9C1 PUZ"; $resn_exch{"2H9C PUZ"}="H9C2 PUZ";
$resn_exch{"1H7C PUZ"}="H7C1 PUZ"; $resn_exch{"2H7C PUZ"}="H7C2 PUZ"; $resn_exch{"3H7C PUZ"}="H7C3 PUZ";
$resn_exch{"1H8C PUZ"}="H8C1 PUZ"; $resn_exch{"2H8C PUZ"}="H8C2 PUZ"; $resn_exch{"3H8C PUZ"}="H8C3 PUZ";
$resn_exch{"1H3  PVA"}=" H31 PVA"; $resn_exch{"2H3  PVA"}=" H32 PVA"; $resn_exch{"3H3  PVA"}=" H33 PVA";
$resn_exch{"1HM  PVA"}=" HM1 PVA"; $resn_exch{"2HM  PVA"}=" HM2 PVA"; $resn_exch{"3HM  PVA"}=" HM3 PVA";
$resn_exch{"1HN  PVA"}=" HN1 PVA"; $resn_exch{"2HN  PVA"}=" HN2 PVA"; $resn_exch{"2HOP PVA"}="HOP2 PVA";
$resn_exch{"3HOP PVA"}="HOP3 PVA"; $resn_exch{"1H16 PVB"}="H161 PVB"; $resn_exch{"2H16 PVB"}="H162 PVB";
$resn_exch{"3H16 PVB"}="H163 PVB"; $resn_exch{"1H14 PVB"}="H141 PVB"; $resn_exch{"2H14 PVB"}="H142 PVB";
$resn_exch{"3H14 PVB"}="H143 PVB"; $resn_exch{"1H15 PVB"}="H151 PVB"; $resn_exch{"2H15 PVB"}="H152 PVB";
$resn_exch{"1H11 PVB"}="H111 PVB"; $resn_exch{"2H11 PVB"}="H112 PVB"; $resn_exch{"3H11 PVB"}="H113 PVB";
$resn_exch{"1H10 PVB"}="H101 PVB"; $resn_exch{"2H10 PVB"}="H102 PVB"; $resn_exch{"3H10 PVB"}="H103 PVB";
$resn_exch{"1HP2 PVC"}="HP21 PVC"; $resn_exch{"2HP2 PVC"}="HP22 PVC"; $resn_exch{"1HP3 PVC"}="HP31 PVC";
$resn_exch{"2HP3 PVC"}="HP32 PVC"; $resn_exch{"1HP4 PVC"}="HP41 PVC"; $resn_exch{"2HP4 PVC"}="HP42 PVC";
$resn_exch{"1HP5 PVC"}="HP51 PVC"; $resn_exch{"2HP5 PVC"}="HP52 PVC"; $resn_exch{"1HP6 PVC"}="HP61 PVC";
$resn_exch{"2HP6 PVC"}="HP62 PVC"; $resn_exch{"1HP7 PVC"}="HP71 PVC"; $resn_exch{"2HP7 PVC"}="HP72 PVC";
$resn_exch{"1HP8 PVC"}="HP81 PVC"; $resn_exch{"2HP8 PVC"}="HP82 PVC"; $resn_exch{"1HP9 PVC"}="HP91 PVC";
$resn_exch{"2HP9 PVC"}="HP92 PVC"; $resn_exch{"1HPA PVC"}="HPA1 PVC"; $resn_exch{"2HPA PVC"}="HPA2 PVC";
$resn_exch{"1HPB PVC"}="HPB1 PVC"; $resn_exch{"2HPB PVC"}="HPB2 PVC"; $resn_exch{"1HPC PVC"}="HPC1 PVC";
$resn_exch{"2HPC PVC"}="HPC2 PVC"; $resn_exch{"1HPD PVC"}="HPD1 PVC"; $resn_exch{"2HPD PVC"}="HPD2 PVC";
$resn_exch{"1HC6 PVC"}="HC61 PVC"; $resn_exch{"2HC6 PVC"}="HC62 PVC"; $resn_exch{"1HPE PVC"}="HPE1 PVC";
$resn_exch{"2HPE PVC"}="HPE2 PVC"; $resn_exch{"1HPF PVC"}="HPF1 PVC"; $resn_exch{"2HPF PVC"}="HPF2 PVC";
$resn_exch{"1HPG PVC"}="HPG1 PVC"; $resn_exch{"2HPG PVC"}="HPG2 PVC"; $resn_exch{"3HPG PVC"}="HPG3 PVC";
$resn_exch{"1H12 PVD"}="H121 PVD"; $resn_exch{"2H12 PVD"}="H122 PVD"; $resn_exch{"1H13 PVD"}="H131 PVD";
$resn_exch{"2H13 PVD"}="H132 PVD"; $resn_exch{"1H20 PVD"}="H201 PVD"; $resn_exch{"2H20 PVD"}="H202 PVD";
$resn_exch{"1H21 PVD"}="H211 PVD"; $resn_exch{"2H21 PVD"}="H212 PVD"; $resn_exch{"1H29 PVD"}="H291 PVD";
$resn_exch{"2H29 PVD"}="H292 PVD"; $resn_exch{"1H35 PVD"}="H351 PVD"; $resn_exch{"2H35 PVD"}="H352 PVD";
$resn_exch{"1H36 PVD"}="H361 PVD"; $resn_exch{"2H36 PVD"}="H362 PVD"; $resn_exch{"1H37 PVD"}="H371 PVD";
$resn_exch{"2H37 PVD"}="H372 PVD"; $resn_exch{"1H40 PVD"}="H401 PVD"; $resn_exch{"2H40 PVD"}="H402 PVD";
$resn_exch{"1H46 PVD"}="H461 PVD"; $resn_exch{"2H46 PVD"}="H462 PVD"; $resn_exch{"1H53 PVD"}="H531 PVD";
$resn_exch{"2H53 PVD"}="H532 PVD"; $resn_exch{"1H54 PVD"}="H541 PVD"; $resn_exch{"2H54 PVD"}="H542 PVD";
$resn_exch{"1H55 PVD"}="H551 PVD"; $resn_exch{"2H55 PVD"}="H552 PVD"; $resn_exch{"1H65 PVD"}="H651 PVD";
$resn_exch{"2H65 PVD"}="H652 PVD"; $resn_exch{"1H66 PVD"}="H661 PVD"; $resn_exch{"2H66 PVD"}="H662 PVD";
$resn_exch{"1H67 PVD"}="H671 PVD"; $resn_exch{"2H67 PVD"}="H672 PVD"; $resn_exch{"1H68 PVD"}="H681 PVD";
$resn_exch{"2H68 PVD"}="H682 PVD"; $resn_exch{"1H74 PVD"}="H741 PVD"; $resn_exch{"2H74 PVD"}="H742 PVD";
$resn_exch{"1H75 PVD"}="H751 PVD"; $resn_exch{"2H75 PVD"}="H752 PVD"; $resn_exch{"1H76 PVD"}="H761 PVD";
$resn_exch{"2H76 PVD"}="H762 PVD"; $resn_exch{"1H87 PVD"}="H871 PVD"; $resn_exch{"2H87 PVD"}="H872 PVD";
$resn_exch{"3H87 PVD"}="H873 PVD"; $resn_exch{"1H94 PVD"}="H941 PVD"; $resn_exch{"2H94 PVD"}="H942 PVD";
$resn_exch{"3H94 PVD"}="H943 PVD"; $resn_exch{" HN2 PVH"}=" H2  PVH"; $resn_exch{"1HB  PVH"}=" HB2 PVH";
$resn_exch{"2HB  PVH"}=" HB3 PVH"; $resn_exch{"1HM  PVH"}=" HM1 PVH"; $resn_exch{"2HM  PVH"}=" HM2 PVH";
$resn_exch{"3HM  PVH"}=" HM3 PVH"; $resn_exch{"3H3  PVL"}=" H33 PVL"; $resn_exch{"1H1  PVL"}=" H11 PVL";
$resn_exch{"2H1  PVL"}=" H12 PVL"; $resn_exch{"3H1  PVL"}=" H13 PVL"; $resn_exch{"1H32 PVN"}="H321 PVN";
$resn_exch{"2H32 PVN"}="H322 PVN"; $resn_exch{"1H21 PVN"}="H211 PVN"; $resn_exch{"2H21 PVN"}="H212 PVN";
$resn_exch{"3H21 PVN"}="H213 PVN"; $resn_exch{"1H5C PVN"}="H5C1 PVN"; $resn_exch{"2H5C PVN"}="H5C2 PVN";
$resn_exch{"1H41 PVN"}="H411 PVN"; $resn_exch{"2H41 PVN"}="H412 PVN"; $resn_exch{"3H41 PVN"}="H413 PVN";
$resn_exch{"1H10 PVN"}="H101 PVN"; $resn_exch{"1H51 PVN"}="H511 PVN"; $resn_exch{"2H51 PVN"}="H512 PVN";
$resn_exch{"1H52 PVN"}="H521 PVN"; $resn_exch{"2H52 PVN"}="H522 PVN"; $resn_exch{"1H61 PVN"}="H611 PVN";
$resn_exch{"2H61 PVN"}="H612 PVN"; $resn_exch{"1H62 PVN"}="H621 PVN"; $resn_exch{"2H62 PVN"}="H622 PVN";
$resn_exch{"1H15 PVN"}="H151 PVN"; $resn_exch{"1H71 PVN"}="H711 PVN"; $resn_exch{"2H71 PVN"}="H712 PVN";
$resn_exch{"3H71 PVN"}="H713 PVN"; $resn_exch{"1H81 PVN"}="H811 PVN"; $resn_exch{"2H81 PVN"}="H812 PVN";
$resn_exch{"3H81 PVN"}="H813 PVN"; $resn_exch{"1H91 PVN"}="H911 PVN"; $resn_exch{"2H91 PVN"}="H912 PVN";
$resn_exch{"1H92 PVN"}="H921 PVN"; $resn_exch{"2H92 PVN"}="H922 PVN"; $resn_exch{"3H92 PVN"}="H923 PVN";
$resn_exch{"3H32 PVN"}="H323 PVN"; $resn_exch{"1H15 PX1"}="H151 PX1"; $resn_exch{"2H15 PX1"}="H152 PX1";
$resn_exch{"3H15 PX1"}="H153 PX1"; $resn_exch{"1H16 PX1"}="H161 PX1"; $resn_exch{"2H16 PX1"}="H162 PX1";
$resn_exch{"3H16 PX1"}="H163 PX1"; $resn_exch{"1H7N PX1"}="H7N1 PX1"; $resn_exch{"2H7N PX1"}="H7N2 PX1";
$resn_exch{"1HN2 PXA"}="HN21 PXA"; $resn_exch{"2HN2 PXA"}="HN22 PXA"; $resn_exch{" H*  PXA"}=" H'  PXA";
$resn_exch{"1H15 PXA"}="H151 PXA"; $resn_exch{"2H15 PXA"}="H152 PXA"; $resn_exch{"3H15 PXA"}="H153 PXA";
$resn_exch{"1H16 PXA"}="H161 PXA"; $resn_exch{"2H16 PXA"}="H162 PXA"; $resn_exch{"3H16 PXA"}="H163 PXA";
$resn_exch{"1HN2 PXF"}="HN21 PXF"; $resn_exch{"2HN2 PXF"}="HN22 PXF"; $resn_exch{" H*  PXF"}=" H'  PXF";
$resn_exch{"1H15 PXF"}="H151 PXF"; $resn_exch{"2H15 PXF"}="H152 PXF"; $resn_exch{"3H15 PXF"}="H153 PXF";
$resn_exch{"1H16 PXF"}="H161 PXF"; $resn_exch{"2H16 PXF"}="H162 PXF"; $resn_exch{"3H16 PXF"}="H163 PXF";
$resn_exch{"1H2A PXG"}="H2A1 PXG"; $resn_exch{"2H2A PXG"}="H2A2 PXG"; $resn_exch{"3H2A PXG"}="H2A3 PXG";
$resn_exch{"1H5A PXG"}="H5A1 PXG"; $resn_exch{"2H5A PXG"}="H5A2 PXG"; $resn_exch{"1HOP PXG"}="HOP1 PXG";
$resn_exch{"2HOP PXG"}="HOP2 PXG"; $resn_exch{"1H4A PXG"}="H4A1 PXG"; $resn_exch{"2H4A PXG"}="H4A2 PXG";
$resn_exch{"1H1C PXI"}="H1C1 PXI"; $resn_exch{"2H1C PXI"}="H1C2 PXI"; $resn_exch{"3H1C PXI"}="H1C3 PXI";
$resn_exch{"1H2C PXI"}="H2C1 PXI"; $resn_exch{"2H2C PXI"}="H2C2 PXI"; $resn_exch{"1H6C PXI"}="H6C1 PXI";
$resn_exch{"2H6C PXI"}="H6C2 PXI"; $resn_exch{"3H6C PXI"}="H6C3 PXI"; $resn_exch{"1H10 PXI"}="H101 PXI";
$resn_exch{"2H10 PXI"}="H102 PXI"; $resn_exch{"3H10 PXI"}="H103 PXI"; $resn_exch{"1H11 PXI"}="H111 PXI";
$resn_exch{"2H11 PXI"}="H112 PXI"; $resn_exch{"1H13 PXI"}="H131 PXI"; $resn_exch{"2H13 PXI"}="H132 PXI";
$resn_exch{"3H13 PXI"}="H133 PXI"; $resn_exch{"1H14 PXI"}="H141 PXI"; $resn_exch{"2H14 PXI"}="H142 PXI";
$resn_exch{"3H14 PXI"}="H143 PXI"; $resn_exch{"1H17 PXI"}="H171 PXI"; $resn_exch{"2H17 PXI"}="H172 PXI";
$resn_exch{"3H17 PXI"}="H173 PXI"; $resn_exch{"1H18 PXI"}="H181 PXI"; $resn_exch{"2H18 PXI"}="H182 PXI";
$resn_exch{"1H20 PXI"}="H201 PXI"; $resn_exch{"2H20 PXI"}="H202 PXI"; $resn_exch{"3H20 PXI"}="H203 PXI";
$resn_exch{"1H25 PXI"}="H251 PXI"; $resn_exch{"2H25 PXI"}="H252 PXI"; $resn_exch{"3H25 PXI"}="H253 PXI";
$resn_exch{"1H5A PXL"}="H5A1 PXL"; $resn_exch{"2H5A PXL"}="H5A2 PXL"; $resn_exch{"1H2A PXL"}="H2A1 PXL";
$resn_exch{"2H2A PXL"}="H2A2 PXL"; $resn_exch{"3H2A PXL"}="H2A3 PXL"; $resn_exch{"1H2A PXM"}="H2A1 PXM";
$resn_exch{"2H2A PXM"}="H2A2 PXM"; $resn_exch{"3H2A PXM"}="H2A3 PXM"; $resn_exch{"1H5A PXM"}="H5A1 PXM";
$resn_exch{"2H5A PXM"}="H5A2 PXM"; $resn_exch{"1H4A PXM"}="H4A1 PXM"; $resn_exch{"2H4A PXM"}="H4A2 PXM";
$resn_exch{"1HN4 PXM"}="HN41 PXM"; $resn_exch{"2HN4 PXM"}="HN42 PXM"; $resn_exch{"1H2A PXP"}="H2A1 PXP";
$resn_exch{"2H2A PXP"}="H2A2 PXP"; $resn_exch{"3H2A PXP"}="H2A3 PXP"; $resn_exch{"1H4A PXP"}="H4A1 PXP";
$resn_exch{"2H4A PXP"}="H4A2 PXP"; $resn_exch{"4HOA PXP"}="HOA4 PXP"; $resn_exch{"1H5A PXP"}="H5A1 PXP";
$resn_exch{"2H5A PXP"}="H5A2 PXP"; $resn_exch{"2HOP PXP"}="HOP2 PXP"; $resn_exch{"3HOP PXP"}="HOP3 PXP";
$resn_exch{"1H13 PXT"}="H131 PXT"; $resn_exch{"2H13 PXT"}="H132 PXT"; $resn_exch{"1H17 PXT"}="H171 PXT";
$resn_exch{"2H17 PXT"}="H172 PXT"; $resn_exch{"1H42 PXT"}="H421 PXT"; $resn_exch{"2H42 PXT"}="H422 PXT";
$resn_exch{"3H42 PXT"}="H423 PXT"; $resn_exch{"1H9  PXT"}=" H91 PXT"; $resn_exch{"2H9  PXT"}=" H92 PXT";
$resn_exch{"1H8  PXT"}=" H81 PXT"; $resn_exch{"2H8  PXT"}=" H82 PXT"; $resn_exch{"1H6  PXT"}=" H61 PXT";
$resn_exch{"2H6  PXT"}=" H62 PXT"; $resn_exch{"1H5  PXT"}=" H51 PXT"; $resn_exch{"2H5  PXT"}=" H52 PXT";
$resn_exch{"1H4  PXT"}=" H41 PXT"; $resn_exch{"2H4  PXT"}=" H42 PXT"; $resn_exch{"1H41 PXT"}="H411 PXT";
$resn_exch{"2H41 PXT"}="H412 PXT"; $resn_exch{"3H41 PXT"}="H413 PXT"; $resn_exch{"1H43 PXT"}="H431 PXT";
$resn_exch{"2H43 PXT"}="H432 PXT"; $resn_exch{"3H43 PXT"}="H433 PXT"; $resn_exch{"1H19 PXT"}="H191 PXT";
$resn_exch{"2H19 PXT"}="H192 PXT"; $resn_exch{"1H20 PXT"}="H201 PXT"; $resn_exch{"2H20 PXT"}="H202 PXT";
$resn_exch{"1H44 PXT"}="H441 PXT"; $resn_exch{"2H44 PXT"}="H442 PXT"; $resn_exch{"3H44 PXT"}="H443 PXT";
$resn_exch{"1H24 PXT"}="H241 PXT"; $resn_exch{"2H24 PXT"}="H242 PXT"; $resn_exch{"1H23 PXT"}="H231 PXT";
$resn_exch{"2H23 PXT"}="H232 PXT"; $resn_exch{"1H26 PXT"}="H261 PXT"; $resn_exch{"2H26 PXT"}="H262 PXT";
$resn_exch{"1H45 PXT"}="H451 PXT"; $resn_exch{"2H45 PXT"}="H452 PXT"; $resn_exch{"3H45 PXT"}="H453 PXT";
$resn_exch{"1H46 PXT"}="H461 PXT"; $resn_exch{"2H46 PXT"}="H462 PXT"; $resn_exch{"3H46 PXT"}="H463 PXT";
$resn_exch{"1H34 PXT"}="H341 PXT"; $resn_exch{"2H34 PXT"}="H342 PXT"; $resn_exch{"1H40 PXT"}="H401 PXT";
$resn_exch{"2H40 PXT"}="H402 PXT"; $resn_exch{"1H39 PXT"}="H391 PXT"; $resn_exch{"2H39 PXT"}="H392 PXT";
$resn_exch{"1H47 PXT"}="H471 PXT"; $resn_exch{"2H47 PXT"}="H472 PXT"; $resn_exch{"3H47 PXT"}="H473 PXT";
$resn_exch{"1H1' PXY"}="H1'1 PXY"; $resn_exch{"2H1' PXY"}="H1'2 PXY"; $resn_exch{"3H1' PXY"}="H1'3 PXY";
$resn_exch{"1H4' PXY"}="H4'1 PXY"; $resn_exch{"2H4' PXY"}="H4'2 PXY"; $resn_exch{"3H4' PXY"}="H4'3 PXY";
$resn_exch{"1HN2 PXZ"}="HN21 PXZ"; $resn_exch{"2HN2 PXZ"}="HN22 PXZ"; $resn_exch{" H*  PXZ"}=" H'  PXZ";
$resn_exch{"1H15 PXZ"}="H151 PXZ"; $resn_exch{"2H15 PXZ"}="H152 PXZ"; $resn_exch{"3H15 PXZ"}="H153 PXZ";
$resn_exch{"1H16 PXZ"}="H161 PXZ"; $resn_exch{"2H16 PXZ"}="H162 PXZ"; $resn_exch{"3H16 PXZ"}="H163 PXZ";
$resn_exch{"1H   PY2"}=" H1  PY2"; $resn_exch{"2H   PY2"}=" H2A PY2"; $resn_exch{"1H18 PY3"}="H181 PY3";
$resn_exch{"2H18 PY3"}="H182 PY3"; $resn_exch{"1H19 PY3"}="H191 PY3"; $resn_exch{"2H19 PY3"}="H192 PY3";
$resn_exch{"1H22 PY3"}="H221 PY3"; $resn_exch{"2H22 PY3"}="H222 PY3"; $resn_exch{"1H33 PY3"}="H331 PY3";
$resn_exch{"2H33 PY3"}="H332 PY3"; $resn_exch{"3H33 PY3"}="H333 PY3"; $resn_exch{"1H34 PY3"}="H341 PY3";
$resn_exch{"2H34 PY3"}="H342 PY3"; $resn_exch{"3H34 PY3"}="H343 PY3"; $resn_exch{"1H40 PY3"}="H401 PY3";
$resn_exch{"2H40 PY3"}="H402 PY3"; $resn_exch{"1H44 PY3"}="H441 PY3"; $resn_exch{"2H44 PY3"}="H442 PY3";
$resn_exch{"1H45 PY3"}="H451 PY3"; $resn_exch{"2H45 PY3"}="H452 PY3"; $resn_exch{"3H45 PY3"}="H453 PY3";
$resn_exch{"1H47 PY3"}="H471 PY3"; $resn_exch{"2H47 PY3"}="H472 PY3"; $resn_exch{"3H47 PY3"}="H473 PY3";
$resn_exch{"1H2A PY4"}="H2A1 PY4"; $resn_exch{"2H2A PY4"}="H2A2 PY4"; $resn_exch{"3H2A PY4"}="H2A3 PY4";
$resn_exch{"1H4A PY4"}="H4A1 PY4"; $resn_exch{"2H4A PY4"}="H4A2 PY4"; $resn_exch{"1H5A PY4"}="H5A1 PY4";
$resn_exch{"2H5A PY4"}="H5A2 PY4"; $resn_exch{"2HOP PY4"}="HOP2 PY4"; $resn_exch{"3HOP PY4"}="HOP3 PY4";
$resn_exch{"1HB  PY4"}=" HB1 PY4"; $resn_exch{"2HB  PY4"}=" HB2 PY4"; $resn_exch{"1H2A PY5"}="H2A1 PY5";
$resn_exch{"2H2A PY5"}="H2A2 PY5"; $resn_exch{"3H2A PY5"}="H2A3 PY5"; $resn_exch{"1H4A PY5"}="H4A1 PY5";
$resn_exch{"2H4A PY5"}="H4A2 PY5"; $resn_exch{"1H5A PY5"}="H5A1 PY5"; $resn_exch{"2H5A PY5"}="H5A2 PY5";
$resn_exch{"2HOP PY5"}="HOP2 PY5"; $resn_exch{"3HOP PY5"}="HOP3 PY5"; $resn_exch{"1HB  PY5"}=" HB1 PY5";
$resn_exch{"2HB  PY5"}=" HB2 PY5"; $resn_exch{"1H2A PY6"}="H2A1 PY6"; $resn_exch{"2H2A PY6"}="H2A2 PY6";
$resn_exch{"3H2A PY6"}="H2A3 PY6"; $resn_exch{"1H4A PY6"}="H4A1 PY6"; $resn_exch{"2H4A PY6"}="H4A2 PY6";
$resn_exch{"1H5A PY6"}="H5A1 PY6"; $resn_exch{"2H5A PY6"}="H5A2 PY6"; $resn_exch{"2HOP PY6"}="HOP2 PY6";
$resn_exch{"3HOP PY6"}="HOP3 PY6"; $resn_exch{"1HB  PY6"}=" HB1 PY6"; $resn_exch{"2HB  PY6"}=" HB2 PY6";
$resn_exch{"1H7  PY7"}=" H71 PY7"; $resn_exch{"2H7  PY7"}=" H72 PY7"; $resn_exch{"1H8C PY8"}="H8C1 PY8";
$resn_exch{"2H8C PY8"}="H8C2 PY8"; $resn_exch{"1H9C PY8"}="H9C1 PY8"; $resn_exch{"2H9C PY8"}="H9C2 PY8";
$resn_exch{"1H10 PY8"}="H101 PY8"; $resn_exch{"2H10 PY8"}="H102 PY8"; $resn_exch{" H2  PYA"}=" HC2 PYA";
$resn_exch{"1HB  PYA"}=" HB3 PYA"; $resn_exch{"2HB  PYA"}=" HB2 PYA"; $resn_exch{"1HN  PYA"}=" H   PYA";
$resn_exch{"2HN  PYA"}=" H2  PYA"; $resn_exch{"1HD  PYB"}=" HD1 PYB"; $resn_exch{"2HD  PYB"}=" HD2 PYB";
$resn_exch{"3HD  PYB"}=" HD3 PYB"; $resn_exch{"1H3  PYC"}=" H31 PYC"; $resn_exch{"1H4  PYC"}=" H41 PYC";
$resn_exch{"1H6  PYC"}=" H61 PYC"; $resn_exch{"1HM2 PYD"}="HM21 PYD"; $resn_exch{"2HM2 PYD"}="HM22 PYD";
$resn_exch{"3HM2 PYD"}="HM23 PYD"; $resn_exch{"1HN4 PYD"}="HN41 PYD"; $resn_exch{"2HN4 PYD"}="HN42 PYD";
$resn_exch{"1HC7 PYD"}="HC71 PYD"; $resn_exch{"2HC7 PYD"}="HC72 PYD"; $resn_exch{"3HC7 PYD"}="HC73 PYD";
$resn_exch{"1H1  PYE"}=" H11 PYE"; $resn_exch{"2H1  PYE"}=" H12 PYE"; $resn_exch{"1H2  PYE"}=" H21 PYE";
$resn_exch{"2H2  PYE"}=" H22 PYE"; $resn_exch{"1H3  PYE"}=" H31 PYE"; $resn_exch{"2H3  PYE"}=" H32 PYE";
$resn_exch{"1H4  PYE"}=" H41 PYE"; $resn_exch{"2H4  PYE"}=" H42 PYE"; $resn_exch{"1H5  PYE"}=" H51 PYE";
$resn_exch{"2H5  PYE"}=" H52 PYE"; $resn_exch{"1H7  PYF"}=" H71 PYF"; $resn_exch{"2H7  PYF"}=" H72 PYF";
$resn_exch{"1HB  PYH"}=" HB1 PYH"; $resn_exch{"2HB  PYH"}=" HB2 PYH"; $resn_exch{"1HG  PYH"}=" HG1 PYH";
$resn_exch{"2HG  PYH"}=" HG2 PYH"; $resn_exch{"1HD  PYH"}=" HD1 PYH"; $resn_exch{"2HD  PYH"}=" HD2 PYH";
$resn_exch{"1HE  PYH"}=" HE1 PYH"; $resn_exch{"2HE  PYH"}=" HE2 PYH"; $resn_exch{"1HAJ PYH"}="HAJ1 PYH";
$resn_exch{"2HAJ PYH"}="HAJ2 PYH"; $resn_exch{"1HAA PYH"}="HAA1 PYH"; $resn_exch{"2HAA PYH"}="HAA2 PYH";
$resn_exch{"3HAA PYH"}="HAA3 PYH"; $resn_exch{"1HM2 PYI"}="HM21 PYI"; $resn_exch{"2HM2 PYI"}="HM22 PYI";
$resn_exch{"3HM2 PYI"}="HM23 PYI"; $resn_exch{"1H7A PYI"}="H7A1 PYI"; $resn_exch{"2H7A PYI"}="H7A2 PYI";
$resn_exch{"1HM6 PYI"}="HM61 PYI"; $resn_exch{"2HM6 PYI"}="HM62 PYI"; $resn_exch{"1HM7 PYI"}="HM71 PYI";
$resn_exch{"2HM7 PYI"}="HM72 PYI"; $resn_exch{"1HM4 PYI"}="HM41 PYI"; $resn_exch{"2HM4 PYI"}="HM42 PYI";
$resn_exch{"3HM4 PYI"}="HM43 PYI"; $resn_exch{"1H4A PYI"}="H4A1 PYI"; $resn_exch{"2H4A PYI"}="H4A2 PYI";
$resn_exch{"1HCB PYL"}="HCB1 PYL"; $resn_exch{"2HCB PYL"}="HCB2 PYL"; $resn_exch{"1HCX PYL"}="HCX1 PYL";
$resn_exch{"2HCX PYL"}="HCX2 PYL"; $resn_exch{"3HCX PYL"}="HCX3 PYL"; $resn_exch{"1HCD PYL"}="HCD1 PYL";
$resn_exch{"2HCD PYL"}="HCD2 PYL"; $resn_exch{"1HCE PYL"}="HCE1 PYL"; $resn_exch{"2HCE PYL"}="HCE2 PYL";
$resn_exch{"1HCZ PYL"}="HCZ1 PYL"; $resn_exch{"2HN1 PYM"}="HN12 PYM"; $resn_exch{"1HN1 PYM"}="HN11 PYM";
$resn_exch{"3H3  PYM"}=" H33 PYM"; $resn_exch{"2H3  PYM"}=" H32 PYM"; $resn_exch{"1H3  PYM"}=" H31 PYM";
$resn_exch{"1H2  PYN"}=" H21 PYN"; $resn_exch{"1H5  PYN"}=" H51 PYN"; $resn_exch{"1H7  PYN"}=" H71 PYN";
$resn_exch{"1H8  PYN"}=" H81 PYN"; $resn_exch{"1H9  PYN"}=" H91 PYN"; $resn_exch{"1H10 PYN"}="H101 PYN";
$resn_exch{"1H12 PYN"}="H121 PYN"; $resn_exch{"1HN  PYN"}=" HN1 PYN"; $resn_exch{"1H15 PYN"}="H151 PYN";
$resn_exch{"2H15 PYN"}="H152 PYN"; $resn_exch{"1H16 PYN"}="H161 PYN"; $resn_exch{"2H16 PYN"}="H162 PYN";
$resn_exch{"1HN1 PYN"}="HN11 PYN"; $resn_exch{"1H18 PYN"}="H181 PYN"; $resn_exch{"2H18 PYN"}="H182 PYN";
$resn_exch{"1H19 PYN"}="H191 PYN"; $resn_exch{"2H19 PYN"}="H192 PYN"; $resn_exch{"1H21 PYN"}="H211 PYN";
$resn_exch{"2H21 PYN"}="H212 PYN"; $resn_exch{"1H23 PYN"}="H231 PYN"; $resn_exch{"2H23 PYN"}="H232 PYN";
$resn_exch{"1H24 PYN"}="H241 PYN"; $resn_exch{"2H24 PYN"}="H242 PYN"; $resn_exch{"1H25 PYN"}="H251 PYN";
$resn_exch{"2H25 PYN"}="H252 PYN"; $resn_exch{"1H26 PYN"}="H261 PYN"; $resn_exch{"2H26 PYN"}="H262 PYN";
$resn_exch{"1H28 PYN"}="H281 PYN"; $resn_exch{"2H28 PYN"}="H282 PYN"; $resn_exch{"1H29 PYN"}="H291 PYN";
$resn_exch{"2H29 PYN"}="H292 PYN"; $resn_exch{"1H30 PYN"}="H301 PYN"; $resn_exch{"2H30 PYN"}="H302 PYN";
$resn_exch{"3HOP PYO"}="HOP3 PYO"; $resn_exch{"1HOP PYO"}="HOP1 PYO"; $resn_exch{"2H5* PYO"}="H5'' PYO";
$resn_exch{"1H5* PYO"}=" H5' PYO"; $resn_exch{" H4* PYO"}=" H4' PYO"; $resn_exch{" H3* PYO"}=" H3' PYO";
$resn_exch{" H1* PYO"}=" H1' PYO"; $resn_exch{"3HO  PYO"}=" HO3 PYO"; $resn_exch{" H2* PYO"}=" H2' PYO";
$resn_exch{"2HO  PYO"}=" HO2 PYO"; $resn_exch{"2HOP PYP"}="HOP2 PYP"; $resn_exch{"1H5* PYP"}="H5'1 PYP";
$resn_exch{"2H5* PYP"}="H5'2 PYP"; $resn_exch{" H4* PYP"}=" H4' PYP"; $resn_exch{" H1* PYP"}=" H1' PYP";
$resn_exch{"2H2* PYP"}="H2'2 PYP"; $resn_exch{"1H2* PYP"}="H2'1 PYP"; $resn_exch{" H3* PYP"}=" H3' PYP";
$resn_exch{"3HOP PYP"}="HOP3 PYP"; $resn_exch{"1H1  PYQ"}=" H11 PYQ"; $resn_exch{"1H6  PYQ"}=" H61 PYQ";
$resn_exch{"2H6  PYQ"}=" H62 PYQ"; $resn_exch{"1H7  PYQ"}=" H71 PYQ"; $resn_exch{"2H7  PYQ"}=" H72 PYQ";
$resn_exch{"1H11 PYQ"}="H111 PYQ"; $resn_exch{"2H11 PYQ"}="H112 PYQ"; $resn_exch{"1H12 PYQ"}="H121 PYQ";
$resn_exch{"2H12 PYQ"}="H122 PYQ"; $resn_exch{"1H13 PYQ"}="H131 PYQ"; $resn_exch{"1H14 PYQ"}="H141 PYQ";
$resn_exch{"1H3  PYR"}=" H31 PYR"; $resn_exch{"2H3  PYR"}=" H32 PYR"; $resn_exch{"3H3  PYR"}=" H33 PYR";
$resn_exch{"1HN  PYT"}=" HN1 PYT"; $resn_exch{"2HN  PYT"}=" HN2 PYT"; $resn_exch{"1HC  PYT"}=" HC1 PYT";
$resn_exch{"2HC  PYT"}=" HC2 PYT"; $resn_exch{"1HT  PYT"}=" HT1 PYT"; $resn_exch{"2HT  PYT"}=" HT2 PYT";
$resn_exch{"1HN  PYX"}=" H   PYX"; $resn_exch{"2HN  PYX"}=" H2  PYX"; $resn_exch{"1HB  PYX"}=" HB2 PYX";
$resn_exch{"2HB  PYX"}=" HB3 PYX"; $resn_exch{"1HE  PYX"}=" HE2 PYX"; $resn_exch{"2HE  PYX"}=" HE3 PYX";
$resn_exch{" H2  PYX"}=" HC2 PYX"; $resn_exch{"1HM6 PYX"}="HM61 PYX"; $resn_exch{"2HM6 PYX"}="HM62 PYX";
$resn_exch{"1HN6 PYX"}="HN61 PYX"; $resn_exch{"2HN6 PYX"}="HN62 PYX"; $resn_exch{"1HM4 PYX"}="HM41 PYX";
$resn_exch{"2HM4 PYX"}="HM42 PYX"; $resn_exch{"3HM4 PYX"}="HM43 PYX"; $resn_exch{"2HOP PYY"}="HOP2 PYY";
$resn_exch{"1H5* PYY"}="H5'1 PYY"; $resn_exch{"2H5* PYY"}="H5'2 PYY"; $resn_exch{" H4* PYY"}=" H4' PYY";
$resn_exch{" H1* PYY"}=" H1' PYY"; $resn_exch{" H2* PYY"}=" H2' PYY"; $resn_exch{"2HO* PYY"}="HO'2 PYY";
$resn_exch{" H3* PYY"}=" H3' PYY"; $resn_exch{"3HOP PYY"}="HOP3 PYY"; $resn_exch{"1H1C PZ1"}="H1C1 PZ1";
$resn_exch{"2H1C PZ1"}="H1C2 PZ1"; $resn_exch{"3H1C PZ1"}="H1C3 PZ1"; $resn_exch{"1H4C PZ1"}="H4C1 PZ1";
$resn_exch{"2H4C PZ1"}="H4C2 PZ1"; $resn_exch{"1H21 PZ1"}="H211 PZ1"; $resn_exch{"2H21 PZ1"}="H212 PZ1";
$resn_exch{"1H22 PZ1"}="H221 PZ1"; $resn_exch{"2H22 PZ1"}="H222 PZ1"; $resn_exch{"1H24 PZ1"}="H241 PZ1";
$resn_exch{"2H24 PZ1"}="H242 PZ1"; $resn_exch{"1H31 PZ1"}="H311 PZ1"; $resn_exch{"2H31 PZ1"}="H312 PZ1";
$resn_exch{"1H32 PZ1"}="H321 PZ1"; $resn_exch{"2H32 PZ1"}="H322 PZ1"; $resn_exch{"1H33 PZ1"}="H331 PZ1";
$resn_exch{"2H33 PZ1"}="H332 PZ1"; $resn_exch{"1H34 PZ1"}="H341 PZ1"; $resn_exch{"2H34 PZ1"}="H342 PZ1";
$resn_exch{"12H1 PZ1"}="H112 PZ1"; $resn_exch{"1H18 PZ1"}="H181 PZ1"; $resn_exch{"2H18 PZ1"}="H182 PZ1";
$resn_exch{"1H19 PZ1"}="H191 PZ1"; $resn_exch{"2H19 PZ1"}="H192 PZ1"; $resn_exch{"1HN1 PZM"}="HN11 PZM";
$resn_exch{"2HN1 PZM"}="HN12 PZM"; $resn_exch{"1HB  PZM"}=" HB1 PZM"; $resn_exch{"2HB  PZM"}=" HB2 PZM";
$resn_exch{"1HF1 PZM"}="HF11 PZM"; $resn_exch{"2HF1 PZM"}="HF12 PZM"; $resn_exch{"3HF1 PZM"}="HF13 PZM";
$resn_exch{"1H1  PZN"}=" H11 PZN"; $resn_exch{"2H1  PZN"}=" H12 PZN"; $resn_exch{"3H1  PZN"}=" H13 PZN";
$resn_exch{"1H2  PZN"}=" H21 PZN"; $resn_exch{"2H2  PZN"}=" H22 PZN"; $resn_exch{"1H3  PZN"}=" H31 PZN";
$resn_exch{"2H3  PZN"}=" H32 PZN"; $resn_exch{"1H4  PZN"}=" H41 PZN"; $resn_exch{"2H4  PZN"}=" H42 PZN";
$resn_exch{"1H5  PZN"}=" H51 PZN"; $resn_exch{"2H5  PZN"}=" H52 PZN"; $resn_exch{"1H6  PZN"}=" H61 PZN";
$resn_exch{"2H6  PZN"}=" H62 PZN"; $resn_exch{"1H7  PZN"}=" H71 PZN"; $resn_exch{"2H7  PZN"}=" H72 PZN";
$resn_exch{"3H7  PZN"}=" H73 PZN"; $resn_exch{"1H8  PZN"}=" H81 PZN"; $resn_exch{"2H8  PZN"}=" H82 PZN";
$resn_exch{"1H11 PZN"}="H111 PZN"; $resn_exch{"2H11 PZN"}="H112 PZN"; $resn_exch{"1H15 PZN"}="H151 PZN";
$resn_exch{"2H15 PZN"}="H152 PZN"; $resn_exch{"1H16 PZN"}="H161 PZN"; $resn_exch{"2H16 PZN"}="H162 PZN";
$resn_exch{"3H16 PZN"}="H163 PZN"; $resn_exch{"1H23 PZN"}="H231 PZN"; $resn_exch{"2H23 PZN"}="H232 PZN";
$resn_exch{"3H23 PZN"}="H233 PZN"; $resn_exch{"1H21 PZN"}="H211 PZN"; $resn_exch{"2H21 PZN"}="H212 PZN";
$resn_exch{"3H21 PZN"}="H213 PZN"; $resn_exch{"1H22 PZN"}="H221 PZN"; $resn_exch{"2H22 PZN"}="H222 PZN";
$resn_exch{"3H22 PZN"}="H223 PZN"; $resn_exch{"1H25 PZN"}="H251 PZN"; $resn_exch{"2H25 PZN"}="H252 PZN";
$resn_exch{"1H28 PZN"}="H281 PZN"; $resn_exch{"2H28 PZN"}="H282 PZN"; $resn_exch{"3H28 PZN"}="H283 PZN";
$resn_exch{"1H2  PZQ"}=" H21 PZQ"; $resn_exch{"2H2  PZQ"}=" H22 PZQ"; $resn_exch{"1H3  PZQ"}=" H31 PZQ";
$resn_exch{"2H3  PZQ"}=" H32 PZQ"; $resn_exch{"1H5  PZQ"}=" H51 PZQ"; $resn_exch{"2H5  PZQ"}=" H52 PZQ";
$resn_exch{"1H6  PZQ"}=" H61 PZQ"; $resn_exch{"2H6  PZQ"}=" H62 PZQ"; $resn_exch{"1H9  PZQ"}=" H91 PZQ";
$resn_exch{"2H9  PZQ"}=" H92 PZQ"; $resn_exch{"1H10 PZQ"}="H101 PZQ"; $resn_exch{"2H10 PZQ"}="H102 PZQ";
$resn_exch{"1H13 PZQ"}="H131 PZQ"; $resn_exch{"2H13 PZQ"}="H132 PZQ"; $resn_exch{"1H15 PZQ"}="H151 PZQ";
$resn_exch{"2H15 PZQ"}="H152 PZQ"; $resn_exch{"1H16 PZQ"}="H161 PZQ"; $resn_exch{"2H16 PZQ"}="H162 PZQ";
$resn_exch{"1H10 PZZ"}="H101 PZZ"; $resn_exch{"2H10 PZZ"}="H102 PZZ"; $resn_exch{"1H12 PZZ"}="H121 PZZ";
$resn_exch{"2H12 PZZ"}="H122 PZZ"; $resn_exch{"3H12 PZZ"}="H123 PZZ"; $resn_exch{"1H13 PZZ"}="H131 PZZ";
$resn_exch{"2H13 PZZ"}="H132 PZZ"; $resn_exch{"3H13 PZZ"}="H133 PZZ"; $resn_exch{"1H14 PZZ"}="H141 PZZ";
$resn_exch{"2H14 PZZ"}="H142 PZZ"; $resn_exch{"1H15 PZZ"}="H151 PZZ"; $resn_exch{"2H15 PZZ"}="H152 PZZ";
$resn_exch{"1H16 PZZ"}="H161 PZZ"; $resn_exch{"2H16 PZZ"}="H162 PZZ"; $resn_exch{"1H7  Q2Y"}=" H71 Q2Y";
$resn_exch{"2H7  Q2Y"}=" H72 Q2Y"; $resn_exch{"1H9  Q2Y"}=" H91 Q2Y"; $resn_exch{"2H9  Q2Y"}=" H92 Q2Y";
$resn_exch{"1H17 Q2Y"}="H171 Q2Y"; $resn_exch{"2H17 Q2Y"}="H172 Q2Y"; $resn_exch{"1H1  Q2Y"}="H11A Q2Y";
$resn_exch{"2H1  Q2Y"}=" H12 Q2Y"; $resn_exch{"3H1  Q2Y"}=" H13 Q2Y"; $resn_exch{"1H50 Q50"}="H501 Q50";
$resn_exch{"2H50 Q50"}="H502 Q50"; $resn_exch{"1H41 Q50"}="H411 Q50"; $resn_exch{"2H41 Q50"}="H412 Q50";
$resn_exch{"1H32 Q50"}="H321 Q50"; $resn_exch{"2H32 Q50"}="H322 Q50"; $resn_exch{"1H33 Q50"}="H331 Q50";
$resn_exch{"2H33 Q50"}="H332 Q50"; $resn_exch{"1H36 Q50"}="H361 Q50"; $resn_exch{"2H36 Q50"}="H362 Q50";
$resn_exch{"1H21 Q50"}="H211 Q50"; $resn_exch{"2H21 Q50"}="H212 Q50"; $resn_exch{"1H19 Q50"}="H191 Q50";
$resn_exch{"2H19 Q50"}="H192 Q50"; $resn_exch{"1H4  Q50"}=" H41 Q50"; $resn_exch{"2H4  Q50"}=" H42 Q50";
$resn_exch{"3H4  Q50"}="H43A Q50"; $resn_exch{"1H3  Q50"}="H31A Q50"; $resn_exch{"2H3  Q50"}=" H32 Q50";
$resn_exch{"3H3  Q50"}=" H33 Q50"; $resn_exch{"1H2  Q50"}=" H21 Q50"; $resn_exch{"2H2  Q50"}=" H22 Q50";
$resn_exch{"3H2  Q50"}="H23A Q50"; $resn_exch{"1H10 Q50"}="H101 Q50"; $resn_exch{"2H10 Q50"}="H102 Q50";
$resn_exch{"1H4  Q72"}="H41A Q72"; $resn_exch{"2H4  Q72"}=" H42 Q72"; $resn_exch{"3H4  Q72"}=" H43 Q72";
$resn_exch{"1H15 Q72"}="H151 Q72"; $resn_exch{"2H15 Q72"}="H152 Q72"; $resn_exch{"3H15 Q72"}="H153 Q72";
$resn_exch{"1H14 Q72"}="H141 Q72"; $resn_exch{"2H14 Q72"}="H142 Q72"; $resn_exch{"3H14 Q72"}="H143 Q72";
$resn_exch{"1H23 Q72"}="H231 Q72"; $resn_exch{"2H23 Q72"}="H232 Q72"; $resn_exch{"1H32 Q72"}="H321 Q72";
$resn_exch{"2H32 Q72"}="H322 Q72"; $resn_exch{"1H38 Q72"}="H381 Q72"; $resn_exch{"2H38 Q72"}="H382 Q72";
$resn_exch{"1H37 Q72"}="H371 Q72"; $resn_exch{"2H37 Q72"}="H372 Q72"; $resn_exch{"1H46 Q72"}="H461 Q72";
$resn_exch{"2H46 Q72"}="H462 Q72"; $resn_exch{"1H45 Q72"}="H451 Q72"; $resn_exch{"2H45 Q72"}="H452 Q72";
$resn_exch{"1H49 Q72"}="H491 Q72"; $resn_exch{"2H49 Q72"}="H492 Q72"; $resn_exch{"1H43 Q72"}="H431 Q72";
$resn_exch{"2H43 Q72"}="H432 Q72"; $resn_exch{"1H42 Q72"}="H421 Q72"; $resn_exch{"2H42 Q72"}="H422 Q72";
$resn_exch{"1H35 Q72"}="H351 Q72"; $resn_exch{"2H35 Q72"}="H352 Q72"; $resn_exch{"1H34 Q72"}="H341 Q72";
$resn_exch{"2H34 Q72"}="H342 Q72"; $resn_exch{"1H12 Q72"}="H121 Q72"; $resn_exch{"2H12 Q72"}="H122 Q72";
$resn_exch{"3H12 Q72"}="H123 Q72"; $resn_exch{"1H11 Q72"}="H111 Q72"; $resn_exch{"2H11 Q72"}="H112 Q72";
$resn_exch{"3H11 Q72"}="H113 Q72"; $resn_exch{"1H1  Q72"}=" H11 Q72"; $resn_exch{"2H1  Q72"}=" H12 Q72";
$resn_exch{"3H1  Q72"}="H13A Q72"; $resn_exch{"1H2  Q82"}="H21A Q82"; $resn_exch{"2H2  Q82"}="H22A Q82";
$resn_exch{"1H7  Q82"}="H71A Q82"; $resn_exch{"2H7  Q82"}="H72A Q82"; $resn_exch{"1H31 Q82"}="H311 Q82";
$resn_exch{"2H31 Q82"}="H312 Q82"; $resn_exch{"1H61 Q82"}="H611 Q82"; $resn_exch{"2H61 Q82"}="H612 Q82";
$resn_exch{"1H76 Q82"}="H761 Q82"; $resn_exch{"2H76 Q82"}="H762 Q82"; $resn_exch{"1H26 Q82"}="H261 Q82";
$resn_exch{"2H26 Q82"}="H262 Q82"; $resn_exch{"1H15 QFI"}="H151 QFI"; $resn_exch{"2H15 QFI"}="H152 QFI";
$resn_exch{"1H16 QFI"}="H161 QFI"; $resn_exch{"2H16 QFI"}="H162 QFI"; $resn_exch{"1H24 QFI"}="H241 QFI";
$resn_exch{"2H24 QFI"}="H242 QFI"; $resn_exch{"3H24 QFI"}="H243 QFI"; $resn_exch{"1H25 QFI"}="H251 QFI";
$resn_exch{"2H25 QFI"}="H252 QFI"; $resn_exch{"3H25 QFI"}="H253 QFI"; $resn_exch{"1H26 QFI"}="H261 QFI";
$resn_exch{"2H26 QFI"}="H262 QFI"; $resn_exch{"3H26 QFI"}="H263 QFI"; $resn_exch{"1H8  QFI"}=" H81 QFI";
$resn_exch{"2H8  QFI"}=" H82 QFI"; $resn_exch{"1H41 QFI"}="H411 QFI"; $resn_exch{"2H41 QFI"}="H412 QFI";
$resn_exch{"1H3  QFI"}=" H31 QFI"; $resn_exch{"2H3  QFI"}=" H32 QFI"; $resn_exch{"1H2  QFI"}=" H21 QFI";
$resn_exch{"2H2  QFI"}="H22A QFI"; $resn_exch{"1H1  QFI"}="H11A QFI"; $resn_exch{"2H1  QFI"}="H12A QFI";
$resn_exch{"1H9C QGA"}="H9C1 QGA"; $resn_exch{"2H9C QGA"}="H9C2 QGA"; $resn_exch{"1H12 QGA"}="H121 QGA";
$resn_exch{"2H12 QGA"}="H122 QGA"; $resn_exch{"1H1C QGA"}="H1C1 QGA"; $resn_exch{"2H1C QGA"}="H1C2 QGA";
$resn_exch{"1H15 QGA"}="H151 QGA"; $resn_exch{"2H15 QGA"}="H152 QGA"; $resn_exch{"1H18 QGA"}="H181 QGA";
$resn_exch{"2H18 QGA"}="H182 QGA"; $resn_exch{"1H24 QGA"}="H241 QGA"; $resn_exch{"2H24 QGA"}="H242 QGA";
$resn_exch{"1H27 QGA"}="H271 QGA"; $resn_exch{"2H27 QGA"}="H272 QGA"; $resn_exch{"1H35 QGA"}="H351 QGA";
$resn_exch{"2H35 QGA"}="H352 QGA"; $resn_exch{"1H36 QGA"}="H361 QGA"; $resn_exch{"2H36 QGA"}="H362 QGA";
$resn_exch{"1H60 QGA"}="H601 QGA"; $resn_exch{"2H60 QGA"}="H602 QGA"; $resn_exch{"1H43 QGA"}="H431 QGA";
$resn_exch{"2H43 QGA"}="H432 QGA"; $resn_exch{"1H1N QGG"}="H1N1 QGG"; $resn_exch{"2H1N QGG"}="H1N2 QGG";
$resn_exch{"1H27 QGG"}="H271 QGG"; $resn_exch{"2H27 QGG"}="H272 QGG"; $resn_exch{"1H19 QGG"}="H191 QGG";
$resn_exch{"2H19 QGG"}="H192 QGG"; $resn_exch{"1H20 QGG"}="H201 QGG"; $resn_exch{"2H20 QGG"}="H202 QGG";
$resn_exch{"1H21 QGG"}="H211 QGG"; $resn_exch{"2H21 QGG"}="H212 QGG"; $resn_exch{"1H29 QIN"}="H291 QIN";
$resn_exch{"2H29 QIN"}="H292 QIN"; $resn_exch{"3H29 QIN"}="H293 QIN"; $resn_exch{"1H56 QIN"}="H561 QIN";
$resn_exch{"2H56 QIN"}="H562 QIN"; $resn_exch{"1H22 QIN"}="H221 QIN"; $resn_exch{"2H22 QIN"}="H222 QIN";
$resn_exch{"3H22 QIN"}="H223 QIN"; $resn_exch{"1H8  QIN"}=" H81 QIN"; $resn_exch{"2H8  QIN"}=" H82 QIN";
$resn_exch{"1HN1 QIN"}="HN11 QIN"; $resn_exch{"2HN1 QIN"}="HN12 QIN"; $resn_exch{"1H10 QIN"}="H101 QIN";
$resn_exch{"2H10 QIN"}="H102 QIN"; $resn_exch{"1H11 QIN"}="H111 QIN"; $resn_exch{"2H11 QIN"}="H112 QIN";
$resn_exch{"1H13 QIN"}="H131 QIN"; $resn_exch{"2H13 QIN"}="H132 QIN"; $resn_exch{"3H13 QIN"}="H133 QIN";
$resn_exch{"1H5  QIN"}=" H51 QIN"; $resn_exch{"2H5  QIN"}=" H52 QIN"; $resn_exch{"1H1  QIN"}=" H11 QIN";
$resn_exch{"2H1  QIN"}=" H12 QIN"; $resn_exch{"1H4  QIN"}=" H41 QIN"; $resn_exch{"2H4  QIN"}=" H42 QIN";
$resn_exch{"3H4  QIN"}=" H43 QIN"; $resn_exch{"1H67 QKH"}="H671 QKH"; $resn_exch{"2H67 QKH"}="H672 QKH";
$resn_exch{"1H81 QKH"}="H811 QKH"; $resn_exch{"2H81 QKH"}="H812 QKH"; $resn_exch{"3H81 QKH"}="H813 QKH";
$resn_exch{"1H82 QKH"}="H821 QKH"; $resn_exch{"2H82 QKH"}="H822 QKH"; $resn_exch{"3H82 QKH"}="H823 QKH";
$resn_exch{"1H83 QKH"}="H831 QKH"; $resn_exch{"2H83 QKH"}="H832 QKH"; $resn_exch{"3H83 QKH"}="H833 QKH";
$resn_exch{"1H66 QKH"}="H661 QKH"; $resn_exch{"2H66 QKH"}="H662 QKH"; $resn_exch{"1H79 QKH"}="H791 QKH";
$resn_exch{"2H79 QKH"}="H792 QKH"; $resn_exch{"3H79 QKH"}="H793 QKH"; $resn_exch{"1H65 QKH"}="H651 QKH";
$resn_exch{"2H65 QKH"}="H652 QKH"; $resn_exch{"1H78 QKH"}="H781 QKH"; $resn_exch{"2H78 QKH"}="H782 QKH";
$resn_exch{"3H78 QKH"}="H783 QKH"; $resn_exch{"1H80 QKH"}="H801 QKH"; $resn_exch{"2H80 QKH"}="H802 QKH";
$resn_exch{"3H80 QKH"}="H803 QKH"; $resn_exch{"1H64 QKH"}="H641 QKH"; $resn_exch{"2H64 QKH"}="H642 QKH";
$resn_exch{"1H63 QKH"}="H631 QKH"; $resn_exch{"2H63 QKH"}="H632 QKH"; $resn_exch{"1H62 QKH"}="H621 QKH";
$resn_exch{"2H62 QKH"}="H622 QKH"; $resn_exch{"1H61 QKH"}="H611 QKH"; $resn_exch{"2H61 QKH"}="H612 QKH";
$resn_exch{"1H84 QKH"}="H841 QKH"; $resn_exch{"2H84 QKH"}="H842 QKH"; $resn_exch{"3H84 QKH"}="H843 QKH";
$resn_exch{"1HD1 QLG"}="HD11 QLG"; $resn_exch{"2HD1 QLG"}="HD12 QLG"; $resn_exch{"3HD1 QLG"}="HD13 QLG";
$resn_exch{"1HD2 QLG"}="HD21 QLG"; $resn_exch{"2HD2 QLG"}="HD22 QLG"; $resn_exch{"3HD2 QLG"}="HD23 QLG";
$resn_exch{"1HB1 QLG"}="HB11 QLG"; $resn_exch{"2HB1 QLG"}="HB12 QLG"; $resn_exch{"1HG1 QLG"}="HG11 QLG";
$resn_exch{"2HG1 QLG"}="HG12 QLG"; $resn_exch{"1HE1 QLG"}="HE11 QLG"; $resn_exch{"2HE1 QLG"}="HE12 QLG";
$resn_exch{"1HA3 QLG"}="HA31 QLG"; $resn_exch{"2HA3 QLG"}="HA32 QLG"; $resn_exch{"1HB  QLP"}=" HB1 QLP";
$resn_exch{"2HB  QLP"}=" HB2 QLP"; $resn_exch{"1HG  QLP"}=" HG1 QLP"; $resn_exch{"2HG  QLP"}=" HG2 QLP";
$resn_exch{"1HE2 QLP"}="HE21 QLP"; $resn_exch{"2HE2 QLP"}="HE22 QLP"; $resn_exch{"1H4A QLP"}="H4A1 QLP";
$resn_exch{"2H4A QLP"}="H4A2 QLP"; $resn_exch{"1H5A QLP"}="H5A1 QLP"; $resn_exch{"2H5A QLP"}="H5A2 QLP";
$resn_exch{"1H2A QLP"}="H2A1 QLP"; $resn_exch{"2H2A QLP"}="H2A2 QLP"; $resn_exch{"3H2A QLP"}="H2A3 QLP";
$resn_exch{"1H7  QMS"}=" H71 QMS"; $resn_exch{"2H7  QMS"}=" H72 QMS"; $resn_exch{"3H7  QMS"}=" H73 QMS";
$resn_exch{"1HN1 QN1"}="HN11 QN1"; $resn_exch{"2HN1 QN1"}="HN12 QN1"; $resn_exch{"1H33 QN1"}="H331 QN1";
$resn_exch{"2H33 QN1"}="H332 QN1"; $resn_exch{"1H25 QN1"}="H251 QN1"; $resn_exch{"2H25 QN1"}="H252 QN1";
$resn_exch{"1H21 QN1"}="H211 QN1"; $resn_exch{"2H21 QN1"}="H212 QN1"; $resn_exch{"1H7  QN1"}=" H71 QN1";
$resn_exch{"2H7  QN1"}=" H72 QN1"; $resn_exch{"1HN2 QN1"}="HN21 QN1"; $resn_exch{"2HN2 QN1"}="HN2A QN1";
$resn_exch{"1H7  QN2"}=" H71 QN2"; $resn_exch{"2H7  QN2"}=" H72 QN2"; $resn_exch{"1H33 QN2"}="H331 QN2";
$resn_exch{"2H33 QN2"}="H332 QN2"; $resn_exch{"1H25 QN2"}="H251 QN2"; $resn_exch{"2H25 QN2"}="H252 QN2";
$resn_exch{"1H21 QN2"}="H211 QN2"; $resn_exch{"2H21 QN2"}="H212 QN2"; $resn_exch{"1H29 QNO"}="H291 QNO";
$resn_exch{"2H29 QNO"}="H292 QNO"; $resn_exch{"3H29 QNO"}="H293 QNO"; $resn_exch{"1H28 QNO"}="H281 QNO";
$resn_exch{"2H28 QNO"}="H282 QNO"; $resn_exch{"1H27 QNO"}="H271 QNO"; $resn_exch{"2H27 QNO"}="H272 QNO";
$resn_exch{"1H26 QNO"}="H261 QNO"; $resn_exch{"2H26 QNO"}="H262 QNO"; $resn_exch{"1H25 QNO"}="H251 QNO";
$resn_exch{"2H25 QNO"}="H252 QNO"; $resn_exch{"1H24 QNO"}="H241 QNO"; $resn_exch{"2H24 QNO"}="H242 QNO";
$resn_exch{"1H23 QNO"}="H231 QNO"; $resn_exch{"2H23 QNO"}="H232 QNO"; $resn_exch{"1H22 QNO"}="H221 QNO";
$resn_exch{"2H22 QNO"}="H222 QNO"; $resn_exch{"1H21 QNO"}="H211 QNO"; $resn_exch{"2H21 QNO"}="H212 QNO";
$resn_exch{"1HAW QPP"}="HAW1 QPP"; $resn_exch{"2HAW QPP"}="HAW2 QPP"; $resn_exch{"3HAW QPP"}="HAW3 QPP";
$resn_exch{"1H16 QQ2"}="H161 QQ2"; $resn_exch{"2H16 QQ2"}="H162 QQ2"; $resn_exch{"1H27 QQ2"}="H271 QQ2";
$resn_exch{"2H27 QQ2"}="H272 QQ2"; $resn_exch{"1H29 QQ2"}="H291 QQ2"; $resn_exch{"2H29 QQ2"}="H292 QQ2";
$resn_exch{"1H31 QQ2"}="H311 QQ2"; $resn_exch{"2H31 QQ2"}="H312 QQ2"; $resn_exch{"3H31 QQ2"}="H313 QQ2";
$resn_exch{"1H32 QQ2"}="H321 QQ2"; $resn_exch{"2H32 QQ2"}="H322 QQ2"; $resn_exch{"3H32 QQ2"}="H323 QQ2";
$resn_exch{"1H33 QQ2"}="H331 QQ2"; $resn_exch{"2H33 QQ2"}="H332 QQ2"; $resn_exch{"3H33 QQ2"}="H333 QQ2";
$resn_exch{"1HN  QSI"}=" HN1 QSI"; $resn_exch{"2HN  QSI"}=" HN2 QSI"; $resn_exch{"1HB  QSI"}=" HB1 QSI";
$resn_exch{"2HB  QSI"}=" HB2 QSI"; $resn_exch{"1HG  QSI"}=" HG1 QSI"; $resn_exch{"2HG  QSI"}=" HG2 QSI";
$resn_exch{"1HN2 QSI"}="HN21 QSI"; $resn_exch{"2HN2 QSI"}="HN22 QSI"; $resn_exch{"1H5' QSI"}="H5'1 QSI";
$resn_exch{"2H5' QSI"}="H5'2 QSI"; $resn_exch{"1HN6 QSI"}="HN61 QSI"; $resn_exch{"2HN6 QSI"}="HN62 QSI";
$resn_exch{"1HC7 QUA"}="HC71 QUA"; $resn_exch{"2HC7 QUA"}="HC72 QUA"; $resn_exch{"1H14 QUA"}="H141 QUA";
$resn_exch{"2H14 QUA"}="H142 QUA"; $resn_exch{"3H14 QUA"}="H143 QUA"; $resn_exch{"1H20 QUM"}="H201 QUM";
$resn_exch{"2H20 QUM"}="H202 QUM"; $resn_exch{"3H20 QUM"}="H203 QUM"; $resn_exch{"1H21 QUM"}="H211 QUM";
$resn_exch{"2H21 QUM"}="H212 QUM"; $resn_exch{"1H22 QUM"}="H221 QUM"; $resn_exch{"2H22 QUM"}="H222 QUM";
$resn_exch{"1H23 QUM"}="H231 QUM"; $resn_exch{"2H23 QUM"}="H232 QUM"; $resn_exch{"1H27 QUM"}="H271 QUM";
$resn_exch{"2H27 QUM"}="H272 QUM"; $resn_exch{"1H25 QUM"}="H251 QUM"; $resn_exch{"2H25 QUM"}="H252 QUM";
$resn_exch{"1H28 QUM"}="H281 QUM"; $resn_exch{"2H28 QUM"}="H282 QUM"; $resn_exch{"1H26 QUM"}="H261 QUM";
$resn_exch{"2H26 QUM"}="H262 QUM"; $resn_exch{"1H17 QUM"}="H171 QUM"; $resn_exch{"2H17 QUM"}="H172 QUM";
$resn_exch{"3H17 QUM"}="H173 QUM"; $resn_exch{"1H2  QUN"}=" H21 QUN"; $resn_exch{"1H3  QUN"}=" H31 QUN";
$resn_exch{"1H6  QUN"}=" H61 QUN"; $resn_exch{"1H10 QUN"}="H101 QUN"; $resn_exch{"1H12 QUN"}="H121 QUN";
$resn_exch{"1H13 QUN"}="H131 QUN"; $resn_exch{"1H14 QUN"}="H141 QUN"; $resn_exch{"2H14 QUN"}="H142 QUN";
$resn_exch{"3H14 QUN"}="H143 QUN"; $resn_exch{"1HN2 QUN"}="HN21 QUN"; $resn_exch{"1H15 QUN"}="H151 QUN";
$resn_exch{"1H16 QUN"}="H161 QUN"; $resn_exch{"2H16 QUN"}="H162 QUN"; $resn_exch{"3H16 QUN"}="H163 QUN";
$resn_exch{"1H17 QUN"}="H171 QUN"; $resn_exch{"2H17 QUN"}="H172 QUN"; $resn_exch{"1H18 QUN"}="H181 QUN";
$resn_exch{"2H18 QUN"}="H182 QUN"; $resn_exch{"1H19 QUN"}="H191 QUN"; $resn_exch{"2H19 QUN"}="H192 QUN";
$resn_exch{"1H20 QUN"}="H201 QUN"; $resn_exch{"2H20 QUN"}="H202 QUN"; $resn_exch{"1H21 QUN"}="H211 QUN";
$resn_exch{"2H21 QUN"}="H212 QUN"; $resn_exch{"1H22 QUN"}="H221 QUN"; $resn_exch{"2H22 QUN"}="H222 QUN";
$resn_exch{"3H22 QUN"}="H223 QUN"; $resn_exch{"1H23 QUN"}="H231 QUN"; $resn_exch{"2H23 QUN"}="H232 QUN";
$resn_exch{"3H23 QUN"}="H233 QUN"; $resn_exch{"2HOP QUO"}="HOP2 QUO"; $resn_exch{"3HOP QUO"}="HOP3 QUO";
$resn_exch{"1H5* QUO"}=" H5' QUO"; $resn_exch{"2H5* QUO"}="H5'' QUO"; $resn_exch{" H4* QUO"}=" H4' QUO";
$resn_exch{" H1* QUO"}=" H1' QUO"; $resn_exch{"1HN2 QUO"}="HN21 QUO"; $resn_exch{"2HN2 QUO"}="HN22 QUO";
$resn_exch{"1H10 QUO"}="H101 QUO"; $resn_exch{"2H10 QUO"}="H102 QUO"; $resn_exch{"1H15 QUO"}="H151 QUO";
$resn_exch{"2H15 QUO"}="H152 QUO"; $resn_exch{"1H16 QUO"}="H161 QUO"; $resn_exch{"2H16 QUO"}="H162 QUO";
$resn_exch{" H2* QUO"}=" H2' QUO"; $resn_exch{"HO2* QUO"}="HO2' QUO"; $resn_exch{" H3* QUO"}=" H3' QUO";
$resn_exch{"HO3* QUO"}="HO3' QUO"; $resn_exch{"1H03 QUS"}="H031 QUS"; $resn_exch{"2H03 QUS"}="H032 QUS";
$resn_exch{"1HP3 QUS"}="HP31 QUS"; $resn_exch{"2HP3 QUS"}="HP32 QUS"; $resn_exch{"1H6Q QV4"}="H6Q1 QV4";
$resn_exch{"2H6Q QV4"}="H6Q2 QV4"; $resn_exch{"1H6P QV4"}="H6P1 QV4"; $resn_exch{"2H6P QV4"}="H6P2 QV4";
$resn_exch{"1H6B QV4"}="H6B1 QV4"; $resn_exch{"2H6B QV4"}="H6B2 QV4"; $resn_exch{"1H6S QV4"}="H6S1 QV4";
$resn_exch{"2H6S QV4"}="H6S2 QV4"; $resn_exch{"3H6S QV4"}="H6S3 QV4"; $resn_exch{"1H7T QV4"}="H7T1 QV4";
$resn_exch{"2H7T QV4"}="H7T2 QV4"; $resn_exch{"2HOP   R"}="HOP2   R"; $resn_exch{"3HOP   R"}="HOP3   R";
$resn_exch{"1H5*   R"}=" H5'   R"; $resn_exch{"2H5*   R"}="H5''   R"; $resn_exch{" H4*   R"}=" H4'   R";
$resn_exch{" H3*   R"}=" H3'   R"; $resn_exch{"*HO3   R"}="HO3'   R"; $resn_exch{"1H2*   R"}=" H2'   R";
$resn_exch{"2H2*   R"}="H2''   R"; $resn_exch{" H1*   R"}=" H1'   R"; $resn_exch{"1HB    R"}=" HB1   R";
$resn_exch{"2HB    R"}=" HB2   R"; $resn_exch{"1H20 R01"}="H201 R01"; $resn_exch{"2H20 R01"}="H202 R01";
$resn_exch{"1H27 R01"}="H271 R01"; $resn_exch{"2H27 R01"}="H272 R01"; $resn_exch{"3H27 R01"}="H273 R01";
$resn_exch{"1H25 R01"}="H251 R01"; $resn_exch{"2H25 R01"}="H252 R01"; $resn_exch{"1H29 R01"}="H291 R01";
$resn_exch{"2H29 R01"}="H292 R01"; $resn_exch{"1H15 R02"}="H151 R02"; $resn_exch{"2H15 R02"}="H152 R02";
$resn_exch{"1H17 R02"}="H171 R02"; $resn_exch{"2H17 R02"}="H172 R02"; $resn_exch{"1H19 R02"}="H191 R02";
$resn_exch{"2H19 R02"}="H192 R02"; $resn_exch{"1H27 R02"}="H271 R02"; $resn_exch{"2H27 R02"}="H272 R02";
$resn_exch{"1H25 R02"}="H251 R02"; $resn_exch{"2H25 R02"}="H252 R02"; $resn_exch{"1H29 R02"}="H291 R02";
$resn_exch{"2H29 R02"}="H292 R02"; $resn_exch{"1H3F R02"}="H3F1 R02"; $resn_exch{"2H3F R02"}="H3F2 R02";
$resn_exch{"1H3A R03"}="H3A1 R03"; $resn_exch{"2H3A R03"}="H3A2 R03"; $resn_exch{"1H1A R03"}="H1A1 R03";
$resn_exch{"2H1A R03"}="H1A2 R03"; $resn_exch{"1H1B R03"}="H1B1 R03"; $resn_exch{"2H1B R03"}="H1B2 R03";
$resn_exch{"1H2B R03"}="H2B1 R03"; $resn_exch{"2H2B R03"}="H2B2 R03"; $resn_exch{"1H3B R03"}="H3B1 R03";
$resn_exch{"2H3B R03"}="H3B2 R03"; $resn_exch{"1H4B R03"}="H4B1 R03"; $resn_exch{"2H4B R03"}="H4B2 R03";
$resn_exch{"1H5B R03"}="H5B1 R03"; $resn_exch{"2H5B R03"}="H5B2 R03"; $resn_exch{"1H6B R03"}="H6B1 R03";
$resn_exch{"2H6B R03"}="H6B2 R03"; $resn_exch{"1H1E R03"}="H1E1 R03"; $resn_exch{"2H1E R03"}="H1E2 R03";
$resn_exch{"3H1E R03"}="H1E3 R03"; $resn_exch{"1H1F R03"}="H1F1 R03"; $resn_exch{"2H1F R03"}="H1F2 R03";
$resn_exch{"3H1F R03"}="H1F3 R03"; $resn_exch{"1H3A R04"}="H3A1 R04"; $resn_exch{"2H3A R04"}="H3A2 R04";
$resn_exch{"1H1A R04"}="H1A1 R04"; $resn_exch{"2H1A R04"}="H1A2 R04"; $resn_exch{"1H1B R04"}="H1B1 R04";
$resn_exch{"2H1B R04"}="H1B2 R04"; $resn_exch{"1H4B R04"}="H4B1 R04"; $resn_exch{"2H4B R04"}="H4B2 R04";
$resn_exch{"1H1E R04"}="H1E1 R04"; $resn_exch{"2H1E R04"}="H1E2 R04"; $resn_exch{"3H1E R04"}="H1E3 R04";
$resn_exch{"1H1F R04"}="H1F1 R04"; $resn_exch{"2H1F R04"}="H1F2 R04"; $resn_exch{"3H1F R04"}="H1F3 R04";
$resn_exch{"1HN2 R11"}="HN21 R11"; $resn_exch{"2HN2 R11"}="HN22 R11"; $resn_exch{"CH10 R11"}="H10C R11";
$resn_exch{"CH11 R11"}="H11C R11"; $resn_exch{"CH13 R11"}="H13C R11"; $resn_exch{"1H14 R11"}="H141 R11";
$resn_exch{"2H14 R11"}="H142 R11"; $resn_exch{"3H14 R11"}="H143 R11"; $resn_exch{"1HC9 R11"}="HC91 R11";
$resn_exch{"2HC9 R11"}="HC92 R11"; $resn_exch{"CH19 R11"}="H19C R11"; $resn_exch{"CH20 R11"}="H20C R11";
$resn_exch{"CH21 R11"}="H21C R11"; $resn_exch{"CH22 R11"}="H22C R11"; $resn_exch{"1H23 R11"}="H231 R11";
$resn_exch{"2H23 R11"}="H232 R11"; $resn_exch{"3H23 R11"}="H233 R11"; $resn_exch{"1H14 R12"}="H141 R12";
$resn_exch{"2H14 R12"}="H142 R12"; $resn_exch{"1H18 R12"}="H181 R12"; $resn_exch{"2H18 R12"}="H182 R12";
$resn_exch{"3H18 R12"}="H183 R12"; $resn_exch{"1H19 R12"}="H191 R12"; $resn_exch{"2H19 R12"}="H192 R12";
$resn_exch{"3H19 R12"}="H193 R12"; $resn_exch{"1H20 R12"}="H201 R12"; $resn_exch{"2H20 R12"}="H202 R12";
$resn_exch{"3H20 R12"}="H203 R12"; $resn_exch{"1H23 R12"}="H231 R12"; $resn_exch{"2H23 R12"}="H232 R12";
$resn_exch{"3H23 R12"}="H233 R12"; $resn_exch{"1H2  R13"}=" H21 R13"; $resn_exch{"2H2  R13"}=" H22 R13";
$resn_exch{"1H3  R13"}=" H31 R13"; $resn_exch{"2H3  R13"}=" H32 R13"; $resn_exch{"1H18 R13"}="H181 R13";
$resn_exch{"2H18 R13"}="H182 R13"; $resn_exch{"3H18 R13"}="H183 R13"; $resn_exch{"1H19 R13"}="H191 R13";
$resn_exch{"2H19 R13"}="H192 R13"; $resn_exch{"3H19 R13"}="H193 R13"; $resn_exch{"1H20 R13"}="H201 R13";
$resn_exch{"2H20 R13"}="H202 R13"; $resn_exch{"3H20 R13"}="H203 R13"; $resn_exch{"1H21 R13"}="H211 R13";
$resn_exch{"2H21 R13"}="H212 R13"; $resn_exch{"3H21 R13"}="H213 R13"; $resn_exch{"1H22 R13"}="H221 R13";
$resn_exch{"2H22 R13"}="H222 R13"; $resn_exch{"3H22 R13"}="H223 R13"; $resn_exch{"1H23 R13"}="H231 R13";
$resn_exch{"2H23 R13"}="H232 R13"; $resn_exch{"3H23 R13"}="H233 R13"; $resn_exch{"1H27 R16"}="H271 R16";
$resn_exch{"2H27 R16"}="H272 R16"; $resn_exch{"3H27 R16"}="H273 R16"; $resn_exch{"1H28 R16"}="H281 R16";
$resn_exch{"2H28 R16"}="H282 R16"; $resn_exch{"1H29 R16"}="H291 R16"; $resn_exch{"2H29 R16"}="H292 R16";
$resn_exch{"1H30 R16"}="H301 R16"; $resn_exch{"2H30 R16"}="H302 R16"; $resn_exch{"1H31 R16"}="H311 R16";
$resn_exch{"2H31 R16"}="H312 R16"; $resn_exch{"1H32 R16"}="H321 R16"; $resn_exch{"2H32 R16"}="H322 R16";
$resn_exch{"1H33 R16"}="H331 R16"; $resn_exch{"2H33 R16"}="H332 R16"; $resn_exch{"1H34 R16"}="H341 R16";
$resn_exch{"2H34 R16"}="H342 R16"; $resn_exch{"1H35 R16"}="H351 R16"; $resn_exch{"2H35 R16"}="H352 R16";
$resn_exch{"1H36 R16"}="H361 R16"; $resn_exch{"2H36 R16"}="H362 R16"; $resn_exch{"1H37 R16"}="H371 R16";
$resn_exch{"2H37 R16"}="H372 R16"; $resn_exch{"1H38 R16"}="H381 R16"; $resn_exch{"2H38 R16"}="H382 R16";
$resn_exch{"1H39 R16"}="H391 R16"; $resn_exch{"2H39 R16"}="H392 R16"; $resn_exch{"1H40 R16"}="H401 R16";
$resn_exch{"2H40 R16"}="H402 R16"; $resn_exch{"1H41 R16"}="H411 R16"; $resn_exch{"2H41 R16"}="H412 R16";
$resn_exch{"1H42 R16"}="H421 R16"; $resn_exch{"2H42 R16"}="H422 R16"; $resn_exch{"3H42 R16"}="H423 R16";
$resn_exch{"1H3A R17"}="H3A1 R17"; $resn_exch{"2H3A R17"}="H3A2 R17"; $resn_exch{"1H1A R17"}="H1A1 R17";
$resn_exch{"2H1A R17"}="H1A2 R17"; $resn_exch{"1H1B R17"}="H1B1 R17"; $resn_exch{"2H1B R17"}="H1B2 R17";
$resn_exch{"1H4B R17"}="H4B1 R17"; $resn_exch{"2H4B R17"}="H4B2 R17"; $resn_exch{"1H1E R17"}="H1E1 R17";
$resn_exch{"2H1E R17"}="H1E2 R17"; $resn_exch{"3H1E R17"}="H1E3 R17"; $resn_exch{"2H1  R18"}="H12A R18";
$resn_exch{"1H1  R18"}="H11A R18"; $resn_exch{"2H2  R18"}=" H22 R18"; $resn_exch{"1H2  R18"}=" H21 R18";
$resn_exch{"2H6  R18"}=" H62 R18"; $resn_exch{"1H6  R18"}=" H61 R18"; $resn_exch{"2H7  R18"}=" H72 R18";
$resn_exch{"1H7  R18"}=" H71 R18"; $resn_exch{"2H15 R18"}="H152 R18"; $resn_exch{"1H15 R18"}="H151 R18";
$resn_exch{"2H16 R18"}="H162 R18"; $resn_exch{"1H16 R18"}="H161 R18"; $resn_exch{"3H18 R18"}="H183 R18";
$resn_exch{"2H18 R18"}="H182 R18"; $resn_exch{"1H18 R18"}="H181 R18"; $resn_exch{"3H27 R18"}="H273 R18";
$resn_exch{"2H27 R18"}="H272 R18"; $resn_exch{"1H27 R18"}="H271 R18"; $resn_exch{"1H1A R19"}="H1A1 R19";
$resn_exch{"2H1A R19"}="H1A2 R19"; $resn_exch{"1H1E R19"}="H1E1 R19"; $resn_exch{"2H1E R19"}="H1E2 R19";
$resn_exch{"3H1E R19"}="H1E3 R19"; $resn_exch{"1H1B R19"}="H1B1 R19"; $resn_exch{"2H1B R19"}="H1B2 R19";
$resn_exch{"1H2B R19"}="H2B1 R19"; $resn_exch{"2H2B R19"}="H2B2 R19"; $resn_exch{"1H3B R19"}="H3B1 R19";
$resn_exch{"2H3B R19"}="H3B2 R19"; $resn_exch{"1H4B R19"}="H4B1 R19"; $resn_exch{"2H4B R19"}="H4B2 R19";
$resn_exch{"1H5B R19"}="H5B1 R19"; $resn_exch{"2H5B R19"}="H5B2 R19"; $resn_exch{"1H6B R19"}="H6B1 R19";
$resn_exch{"2H6B R19"}="H6B2 R19"; $resn_exch{"1H3A R19"}="H3A1 R19"; $resn_exch{"2H3A R19"}="H3A2 R19";
$resn_exch{"1HN  R1A"}=" H   R1A"; $resn_exch{"2HN  R1A"}=" H2  R1A"; $resn_exch{"1HB  R1A"}=" HB2 R1A";
$resn_exch{"2HB  R1A"}=" HB3 R1A"; $resn_exch{"1HE  R1A"}=" HE2 R1A"; $resn_exch{"2HE  R1A"}=" HE3 R1A";
$resn_exch{"1H9  R1A"}=" H91 R1A"; $resn_exch{"2H9  R1A"}=" H92 R1A"; $resn_exch{"3H9  R1A"}=" H93 R1A";
$resn_exch{"1H8  R1A"}=" H81 R1A"; $resn_exch{"2H8  R1A"}=" H82 R1A"; $resn_exch{"3H8  R1A"}=" H83 R1A";
$resn_exch{"1H7  R1A"}=" H71 R1A"; $resn_exch{"2H7  R1A"}=" H72 R1A"; $resn_exch{"3H7  R1A"}=" H73 R1A";
$resn_exch{"1H6  R1A"}=" H61 R1A"; $resn_exch{"2H6  R1A"}=" H62 R1A"; $resn_exch{"3H6  R1A"}=" H63 R1A";
$resn_exch{"1H8  R1B"}=" H81 R1B"; $resn_exch{"2H8  R1B"}=" H82 R1B"; $resn_exch{"3H8  R1B"}=" H83 R1B";
$resn_exch{"1H9  R1B"}=" H91 R1B"; $resn_exch{"2H9  R1B"}=" H92 R1B"; $resn_exch{"3H9  R1B"}=" H93 R1B";
$resn_exch{"1H7  R1B"}=" H71 R1B"; $resn_exch{"2H7  R1B"}=" H72 R1B"; $resn_exch{"3H7  R1B"}=" H73 R1B";
$resn_exch{"1H6  R1B"}=" H61 R1B"; $resn_exch{"2H6  R1B"}=" H62 R1B"; $resn_exch{"3H6  R1B"}=" H63 R1B";
$resn_exch{"1H10 R1B"}="H101 R1B"; $resn_exch{"2H10 R1B"}="H102 R1B"; $resn_exch{"3H10 R1B"}="H103 R1B";
$resn_exch{"1HE  R1B"}=" HE3 R1B"; $resn_exch{"2HE  R1B"}=" HE2 R1B"; $resn_exch{"1HB  R1B"}=" HB3 R1B";
$resn_exch{"2HB  R1B"}=" HB2 R1B"; $resn_exch{"1HN  R1B"}=" H   R1B"; $resn_exch{"2HN  R1B"}=" H2  R1B";
$resn_exch{"1HN  R1F"}=" H   R1F"; $resn_exch{"2HN  R1F"}=" H2  R1F"; $resn_exch{"1HB  R1F"}=" HB2 R1F";
$resn_exch{"2HB  R1F"}=" HB3 R1F"; $resn_exch{"1HE  R1F"}=" HE2 R1F"; $resn_exch{"2HE  R1F"}=" HE3 R1F";
$resn_exch{"1H9  R1F"}=" H91 R1F"; $resn_exch{"2H9  R1F"}=" H92 R1F"; $resn_exch{"3H9  R1F"}=" H93 R1F";
$resn_exch{"1H8  R1F"}=" H81 R1F"; $resn_exch{"2H8  R1F"}=" H82 R1F"; $resn_exch{"3H8  R1F"}=" H83 R1F";
$resn_exch{"1H6  R1F"}=" H61 R1F"; $resn_exch{"2H6  R1F"}=" H62 R1F"; $resn_exch{"3H6  R1F"}=" H63 R1F";
$resn_exch{"1H7  R1F"}=" H71 R1F"; $resn_exch{"2H7  R1F"}=" H72 R1F"; $resn_exch{"3H7  R1F"}=" H73 R1F";
$resn_exch{"1H5' R1P"}="H5'1 R1P"; $resn_exch{"2H5' R1P"}="H5'2 R1P"; $resn_exch{"'HO5 R1P"}="HO5' R1P";
$resn_exch{"'HO2 R1P"}="HO2' R1P"; $resn_exch{"'HO3 R1P"}="HO3' R1P"; $resn_exch{"2HOP R1P"}="HOP2 R1P";
$resn_exch{"3HOP R1P"}="HOP3 R1P"; $resn_exch{"1H6  R1T"}=" H61 R1T"; $resn_exch{"2H6  R1T"}=" H62 R1T";
$resn_exch{"1H7  R1T"}=" H71 R1T"; $resn_exch{"2H7  R1T"}=" H72 R1T"; $resn_exch{"1HM4 R1T"}="HM41 R1T";
$resn_exch{"2HM4 R1T"}="HM42 R1T"; $resn_exch{"3HM4 R1T"}="HM43 R1T"; $resn_exch{"1H7' R1T"}="H7'1 R1T";
$resn_exch{"2H7' R1T"}="H7'2 R1T"; $resn_exch{"1HM2 R1T"}="HM21 R1T"; $resn_exch{"2HM2 R1T"}="HM22 R1T";
$resn_exch{"3HM2 R1T"}="HM23 R1T"; $resn_exch{"1H4' R1T"}="H4'1 R1T"; $resn_exch{"2H4' R1T"}="H4'2 R1T";
$resn_exch{"1HP2 R1T"}="HP21 R1T"; $resn_exch{"2HP2 R1T"}="HP22 R1T"; $resn_exch{"3HP2 R1T"}="HP23 R1T";
$resn_exch{"1H12 R20"}="H121 R20"; $resn_exch{"2H12 R20"}="H122 R20"; $resn_exch{"3H12 R20"}="H123 R20";
$resn_exch{" H2* R22"}=" H2' R22"; $resn_exch{" H4* R22"}=" H4' R22"; $resn_exch{" H6* R22"}=" H6' R22";
$resn_exch{"1H8* R22"}="H8'1 R22"; $resn_exch{"2H8* R22"}="H8'2 R22"; $resn_exch{"3H8* R22"}="H8'3 R22";
$resn_exch{"1H9* R22"}="H9'1 R22"; $resn_exch{"2H9* R22"}="H9'2 R22"; $resn_exch{"3H9* R22"}="H9'3 R22";
$resn_exch{"1H7  R22"}=" H71 R22"; $resn_exch{"2H7  R22"}=" H72 R22"; $resn_exch{"3H7  R22"}=" H73 R22";
$resn_exch{"1H10 R22"}="H101 R22"; $resn_exch{"2H10 R22"}="H102 R22"; $resn_exch{"1H12 R22"}="H121 R22";
$resn_exch{"2H12 R22"}="H122 R22"; $resn_exch{"1H4E R23"}="H4E1 R23"; $resn_exch{"2H4E R23"}="H4E2 R23";
$resn_exch{"1H5E R23"}="H5E1 R23"; $resn_exch{"2H5E R23"}="H5E2 R23"; $resn_exch{"1H6E R23"}="H6E1 R23";
$resn_exch{"2H6E R23"}="H6E2 R23"; $resn_exch{"1H2E R23"}="H2E1 R23"; $resn_exch{"2H2E R23"}="H2E2 R23";
$resn_exch{"1H3E R23"}="H3E1 R23"; $resn_exch{"2H3E R23"}="H3E2 R23"; $resn_exch{"1H1D R23"}="H1D1 R23";
$resn_exch{"2H1D R23"}="H1D2 R23"; $resn_exch{"1H3B R23"}="H3B1 R23"; $resn_exch{"2H3B R23"}="H3B2 R23";
$resn_exch{"1H2B R23"}="H2B1 R23"; $resn_exch{"2H2B R23"}="H2B2 R23"; $resn_exch{"1H6B R23"}="H6B1 R23";
$resn_exch{"2H6B R23"}="H6B2 R23"; $resn_exch{"1H5B R23"}="H5B1 R23"; $resn_exch{"2H5B R23"}="H5B2 R23";
$resn_exch{"1H1A R23"}="H1A1 R23"; $resn_exch{"2H1A R23"}="H1A2 R23"; $resn_exch{"3H1A R23"}="H1A3 R23";
$resn_exch{"1H10 R36"}="H101 R36"; $resn_exch{"1H13 R36"}="H131 R36"; $resn_exch{"1H14 R36"}="H141 R36";
$resn_exch{"1H3  R36"}=" H31 R36"; $resn_exch{"2H3  R36"}=" H32 R36"; $resn_exch{"3H3  R36"}=" H33 R36";
$resn_exch{"1H2  R36"}=" H21 R36"; $resn_exch{"2H2  R36"}=" H22 R36"; $resn_exch{"3H2  R36"}=" H23 R36";
$resn_exch{"1H26 R36"}="H261 R36"; $resn_exch{"1H27 R36"}="H271 R36"; $resn_exch{"1H28 R36"}="H281 R36";
$resn_exch{"1H1  R36"}=" H11 R36"; $resn_exch{"2H1  R36"}=" H12 R36"; $resn_exch{"1H30 R36"}="H301 R36";
$resn_exch{"1H34 R36"}="H341 R36"; $resn_exch{"2H34 R36"}="H342 R36"; $resn_exch{"1H16 R36"}="H161 R36";
$resn_exch{"1H15 R36"}="H151 R36"; $resn_exch{"OH14 R36"}="H14O R36"; $resn_exch{"1H12 R36"}="H121 R36";
$resn_exch{"2H12 R36"}="H122 R36"; $resn_exch{"1H4  R36"}=" H41 R36"; $resn_exch{"1H5  R36"}=" H51 R36";
$resn_exch{"2H5  R36"}=" H52 R36"; $resn_exch{"1H6  R36"}=" H61 R36"; $resn_exch{"1H11 R36"}="H111 R36";
$resn_exch{"2H11 R36"}="H112 R36"; $resn_exch{"3H11 R36"}="H113 R36"; $resn_exch{"1H7  R36"}=" H71 R36";
$resn_exch{"2H7  R36"}=" H72 R36"; $resn_exch{"3H7  R36"}=" H73 R36"; $resn_exch{"1HN3 R36"}="HN31 R36";
$resn_exch{"1H17 R36"}="H171 R36"; $resn_exch{"1H18 R36"}="H181 R36"; $resn_exch{"2H18 R36"}="H182 R36";
$resn_exch{"1H20 R36"}="H201 R36"; $resn_exch{"1H21 R36"}="H211 R36"; $resn_exch{"1H22 R36"}="H221 R36";
$resn_exch{"1H23 R36"}="H231 R36"; $resn_exch{"1H24 R36"}="H241 R36"; $resn_exch{"1H10 R37"}="H101 R37";
$resn_exch{"1H13 R37"}="H131 R37"; $resn_exch{"1H14 R37"}="H141 R37"; $resn_exch{"1H3  R37"}=" H31 R37";
$resn_exch{"2H3  R37"}=" H32 R37"; $resn_exch{"3H3  R37"}=" H33 R37"; $resn_exch{"1H2  R37"}=" H21 R37";
$resn_exch{"2H2  R37"}=" H22 R37"; $resn_exch{"3H2  R37"}=" H23 R37"; $resn_exch{"1H26 R37"}="H261 R37";
$resn_exch{"1H27 R37"}="H271 R37"; $resn_exch{"1H29 R37"}="H291 R37"; $resn_exch{"1H1  R37"}=" H11 R37";
$resn_exch{"2H1  R37"}=" H12 R37"; $resn_exch{"1H30 R37"}="H301 R37"; $resn_exch{"1H34 R37"}="H341 R37";
$resn_exch{"2H34 R37"}="H342 R37"; $resn_exch{"1H16 R37"}="H161 R37"; $resn_exch{"1H15 R37"}="H151 R37";
$resn_exch{"OH14 R37"}="H14O R37"; $resn_exch{"1H12 R37"}="H121 R37"; $resn_exch{"2H12 R37"}="H122 R37";
$resn_exch{"1H4  R37"}=" H41 R37"; $resn_exch{"1H5  R37"}=" H51 R37"; $resn_exch{"2H5  R37"}=" H52 R37";
$resn_exch{"1H6  R37"}=" H61 R37"; $resn_exch{"1H11 R37"}="H111 R37"; $resn_exch{"2H11 R37"}="H112 R37";
$resn_exch{"3H11 R37"}="H113 R37"; $resn_exch{"1H7  R37"}=" H71 R37"; $resn_exch{"2H7  R37"}=" H72 R37";
$resn_exch{"3H7  R37"}=" H73 R37"; $resn_exch{"1HN3 R37"}="HN31 R37"; $resn_exch{"1H17 R37"}="H171 R37";
$resn_exch{"1H18 R37"}="H181 R37"; $resn_exch{"2H18 R37"}="H182 R37"; $resn_exch{"1H20 R37"}="H201 R37";
$resn_exch{"1H21 R37"}="H211 R37"; $resn_exch{"1H22 R37"}="H221 R37"; $resn_exch{"1H23 R37"}="H231 R37";
$resn_exch{"1H24 R37"}="H241 R37"; $resn_exch{"1H3A R46"}="H3A1 R46"; $resn_exch{"2H3A R46"}="H3A2 R46";
$resn_exch{"1H1A R46"}="H1A1 R46"; $resn_exch{"2H1A R46"}="H1A2 R46"; $resn_exch{"1H1B R46"}="H1B1 R46";
$resn_exch{"2H1B R46"}="H1B2 R46"; $resn_exch{"1H2B R46"}="H2B1 R46"; $resn_exch{"2H2B R46"}="H2B2 R46";
$resn_exch{"1H3B R46"}="H3B1 R46"; $resn_exch{"2H3B R46"}="H3B2 R46"; $resn_exch{"1H4B R46"}="H4B1 R46";
$resn_exch{"2H4B R46"}="H4B2 R46"; $resn_exch{"1H5B R46"}="H5B1 R46"; $resn_exch{"2H5B R46"}="H5B2 R46";
$resn_exch{"1H6B R46"}="H6B1 R46"; $resn_exch{"2H6B R46"}="H6B2 R46"; $resn_exch{"1H1F R46"}="H1F1 R46";
$resn_exch{"2H1F R46"}="H1F2 R46"; $resn_exch{"3H1F R46"}="H1F3 R46"; $resn_exch{"3HM2 R4A"}="HM23 R4A";
$resn_exch{"2HM2 R4A"}="HM22 R4A"; $resn_exch{"1HM2 R4A"}="HM21 R4A"; $resn_exch{"3HM1 R4A"}="HM13 R4A";
$resn_exch{"2HM1 R4A"}="HM12 R4A"; $resn_exch{"1HM1 R4A"}="HM11 R4A"; $resn_exch{"1HL4 R4A"}="HL41 R4A";
$resn_exch{"2HL4 R4A"}="HL42 R4A"; $resn_exch{"1HL3 R4A"}="HL31 R4A"; $resn_exch{"2HL3 R4A"}="HL32 R4A";
$resn_exch{"1HL2 R4A"}="HL21 R4A"; $resn_exch{"2HL2 R4A"}="HL22 R4A"; $resn_exch{"1HL1 R4A"}="HL11 R4A";
$resn_exch{"2HL1 R4A"}="HL12 R4A"; $resn_exch{"1HAN R4A"}="HAN1 R4A"; $resn_exch{"1HCK R4A"}="HCK1 R4A";
$resn_exch{"1HCJ R4A"}="HCJ1 R4A"; $resn_exch{"1HCE R4A"}="HCE1 R4A"; $resn_exch{"1HCD R4A"}="HCD1 R4A";
$resn_exch{"1HCC R4A"}="HCC1 R4A"; $resn_exch{"1HCB R4A"}="HCB1 R4A"; $resn_exch{"1HBK R4A"}="HBK1 R4A";
$resn_exch{"1HBJ R4A"}="HBJ1 R4A"; $resn_exch{"1HBI R4A"}="HBI1 R4A"; $resn_exch{"1HBH R4A"}="HBH1 R4A";
$resn_exch{"1HBE R4A"}="HBE1 R4A"; $resn_exch{"1HBD R4A"}="HBD1 R4A"; $resn_exch{"1HBC R4A"}="HBC1 R4A";
$resn_exch{"1HBB R4A"}="HBB1 R4A"; $resn_exch{"1H32 R55"}="H321 R55"; $resn_exch{"2H32 R55"}="H322 R55";
$resn_exch{"1H34 R55"}="H341 R55"; $resn_exch{"2H34 R55"}="H342 R55"; $resn_exch{"1H35 R55"}="H351 R55";
$resn_exch{"2H35 R55"}="H352 R55"; $resn_exch{"1H36 R55"}="H361 R55"; $resn_exch{"2H36 R55"}="H362 R55";
$resn_exch{"1H92 R55"}="H921 R55"; $resn_exch{"2H92 R55"}="H922 R55"; $resn_exch{"1H93 R55"}="H931 R55";
$resn_exch{"2H93 R55"}="H932 R55"; $resn_exch{"1H97 R55"}="H971 R55"; $resn_exch{"2H97 R55"}="H972 R55";
$resn_exch{"3H97 R55"}="H973 R55"; $resn_exch{"1H98 R55"}="H981 R55"; $resn_exch{"2H98 R55"}="H982 R55";
$resn_exch{"3H98 R55"}="H983 R55"; $resn_exch{"1H95 R55"}="H951 R55"; $resn_exch{"2H95 R55"}="H952 R55";
$resn_exch{"1H96 R55"}="H961 R55"; $resn_exch{"2H96 R55"}="H962 R55"; $resn_exch{"1H12 R56"}="H121 R56";
$resn_exch{"2H12 R56"}="H122 R56"; $resn_exch{"1H27 R56"}="H271 R56"; $resn_exch{"2H27 R56"}="H272 R56";
$resn_exch{"1H30 R56"}="H301 R56"; $resn_exch{"2H30 R56"}="H302 R56"; $resn_exch{"1H34 R56"}="H341 R56";
$resn_exch{"2H34 R56"}="H342 R56"; $resn_exch{"1H43 R56"}="H431 R56"; $resn_exch{"2H43 R56"}="H432 R56";
$resn_exch{"1H46 R56"}="H461 R56"; $resn_exch{"2H46 R56"}="H462 R56"; $resn_exch{"1H49 R56"}="H491 R56";
$resn_exch{"2H49 R56"}="H492 R56"; $resn_exch{"1HN5 R56"}="HN51 R56"; $resn_exch{"2HN5 R56"}="HN52 R56";
$resn_exch{"3HM2 R5A"}="HM23 R5A"; $resn_exch{"2HM2 R5A"}="HM22 R5A"; $resn_exch{"1HM2 R5A"}="HM21 R5A";
$resn_exch{"3HM1 R5A"}="HM13 R5A"; $resn_exch{"2HM1 R5A"}="HM12 R5A"; $resn_exch{"1HM1 R5A"}="HM11 R5A";
$resn_exch{"1HL5 R5A"}="HL51 R5A"; $resn_exch{"2HL5 R5A"}="HL52 R5A"; $resn_exch{"1HL4 R5A"}="HL41 R5A";
$resn_exch{"2HL4 R5A"}="HL42 R5A"; $resn_exch{"1HL3 R5A"}="HL31 R5A"; $resn_exch{"2HL3 R5A"}="HL32 R5A";
$resn_exch{"1HL2 R5A"}="HL21 R5A"; $resn_exch{"2HL2 R5A"}="HL22 R5A"; $resn_exch{"1HL1 R5A"}="HL11 R5A";
$resn_exch{"2HL1 R5A"}="HL12 R5A"; $resn_exch{"1HCK R5A"}="HCK1 R5A"; $resn_exch{"1HCJ R5A"}="HCJ1 R5A";
$resn_exch{"1HCE R5A"}="HCE1 R5A"; $resn_exch{"1HCD R5A"}="HCD1 R5A"; $resn_exch{"1HCC R5A"}="HCC1 R5A";
$resn_exch{"1HCB R5A"}="HCB1 R5A"; $resn_exch{"1HBK R5A"}="HBK1 R5A"; $resn_exch{"1HBJ R5A"}="HBJ1 R5A";
$resn_exch{"1HBI R5A"}="HBI1 R5A"; $resn_exch{"1HBH R5A"}="HBH1 R5A"; $resn_exch{"1HBE R5A"}="HBE1 R5A";
$resn_exch{"1HBD R5A"}="HBD1 R5A"; $resn_exch{"1HBC R5A"}="HBC1 R5A"; $resn_exch{"1HBB R5A"}="HBB1 R5A";
$resn_exch{"3HM2 R5B"}="HM23 R5B"; $resn_exch{"2HM2 R5B"}="HM22 R5B"; $resn_exch{"1HM2 R5B"}="HM21 R5B";
$resn_exch{"3HM1 R5B"}="HM13 R5B"; $resn_exch{"2HM1 R5B"}="HM12 R5B"; $resn_exch{"1HM1 R5B"}="HM11 R5B";
$resn_exch{"1HL5 R5B"}="HL51 R5B"; $resn_exch{"2HL5 R5B"}="HL52 R5B"; $resn_exch{"1HL4 R5B"}="HL41 R5B";
$resn_exch{"2HL4 R5B"}="HL42 R5B"; $resn_exch{"1HL3 R5B"}="HL31 R5B"; $resn_exch{"2HL3 R5B"}="HL32 R5B";
$resn_exch{"1HL2 R5B"}="HL21 R5B"; $resn_exch{"2HL2 R5B"}="HL22 R5B"; $resn_exch{"1HL1 R5B"}="HL11 R5B";
$resn_exch{"2HL1 R5B"}="HL12 R5B"; $resn_exch{"1HCK R5B"}="HCK1 R5B"; $resn_exch{"1HCJ R5B"}="HCJ1 R5B";
$resn_exch{"1HCE R5B"}="HCE1 R5B"; $resn_exch{"1HCD R5B"}="HCD1 R5B"; $resn_exch{"1HCC R5B"}="HCC1 R5B";
$resn_exch{"1HCB R5B"}="HCB1 R5B"; $resn_exch{"1HBK R5B"}="HBK1 R5B"; $resn_exch{"1HBJ R5B"}="HBJ1 R5B";
$resn_exch{"1HBI R5B"}="HBI1 R5B"; $resn_exch{"1HBH R5B"}="HBH1 R5B"; $resn_exch{"1HBE R5B"}="HBE1 R5B";
$resn_exch{"1HBD R5B"}="HBD1 R5B"; $resn_exch{"1HBC R5B"}="HBC1 R5B"; $resn_exch{"1HBB R5B"}="HBB1 R5B";
$resn_exch{"2HOP R5P"}="HOP2 R5P"; $resn_exch{"3HOP R5P"}="HOP3 R5P"; $resn_exch{"1H30 R64"}="H301 R64";
$resn_exch{"2H30 R64"}="H302 R64"; $resn_exch{"3H30 R64"}="H303 R64"; $resn_exch{"1H11 R64"}="H111 R64";
$resn_exch{"2H11 R64"}="H112 R64"; $resn_exch{"3H11 R64"}="H113 R64"; $resn_exch{"1H13 R64"}="H131 R64";
$resn_exch{"2H13 R64"}="H132 R64"; $resn_exch{"1H14 R64"}="H141 R64"; $resn_exch{"2H14 R64"}="H142 R64";
$resn_exch{"1H15 R64"}="H151 R64"; $resn_exch{"2H15 R64"}="H152 R64"; $resn_exch{"1H17 R64"}="H171 R64";
$resn_exch{"2H17 R64"}="H172 R64"; $resn_exch{"1H32 R68"}="H321 R68"; $resn_exch{"2H32 R68"}="H322 R68";
$resn_exch{"1H34 R68"}="H341 R68"; $resn_exch{"2H34 R68"}="H342 R68"; $resn_exch{"1H35 R68"}="H351 R68";
$resn_exch{"2H35 R68"}="H352 R68"; $resn_exch{"1H36 R68"}="H361 R68"; $resn_exch{"2H36 R68"}="H362 R68";
$resn_exch{"1H97 R68"}="H971 R68"; $resn_exch{"2H97 R68"}="H972 R68"; $resn_exch{"3H97 R68"}="H973 R68";
$resn_exch{"1H98 R68"}="H981 R68"; $resn_exch{"2H98 R68"}="H982 R68"; $resn_exch{"3H98 R68"}="H983 R68";
$resn_exch{"1H93 R68"}="H931 R68"; $resn_exch{"2H93 R68"}="H932 R68"; $resn_exch{"1H94 R68"}="H941 R68";
$resn_exch{"2H94 R68"}="H942 R68"; $resn_exch{"1H95 R68"}="H951 R68"; $resn_exch{"2H95 R68"}="H952 R68";
$resn_exch{"1H32 R69"}="H321 R69"; $resn_exch{"2H32 R69"}="H322 R69"; $resn_exch{"1H34 R69"}="H341 R69";
$resn_exch{"2H34 R69"}="H342 R69"; $resn_exch{"1H35 R69"}="H351 R69"; $resn_exch{"2H35 R69"}="H352 R69";
$resn_exch{"1H36 R69"}="H361 R69"; $resn_exch{"2H36 R69"}="H362 R69"; $resn_exch{"1H98 R69"}="H981 R69";
$resn_exch{"2H98 R69"}="H982 R69"; $resn_exch{"3H98 R69"}="H983 R69"; $resn_exch{"1H99 R69"}="H991 R69";
$resn_exch{"2H99 R69"}="H992 R69"; $resn_exch{"3H99 R69"}="H993 R69"; $resn_exch{"3HM2 R6A"}="HM23 R6A";
$resn_exch{"2HM2 R6A"}="HM22 R6A"; $resn_exch{"1HM2 R6A"}="HM21 R6A"; $resn_exch{"3HM1 R6A"}="HM13 R6A";
$resn_exch{"2HM1 R6A"}="HM12 R6A"; $resn_exch{"1HM1 R6A"}="HM11 R6A"; $resn_exch{"1HL6 R6A"}="HL61 R6A";
$resn_exch{"2HL6 R6A"}="HL62 R6A"; $resn_exch{"1HL5 R6A"}="HL51 R6A"; $resn_exch{"2HL5 R6A"}="HL52 R6A";
$resn_exch{"1HL4 R6A"}="HL41 R6A"; $resn_exch{"2HL4 R6A"}="HL42 R6A"; $resn_exch{"1HL3 R6A"}="HL31 R6A";
$resn_exch{"2HL3 R6A"}="HL32 R6A"; $resn_exch{"1HCK R6A"}="HCK1 R6A"; $resn_exch{"1HCJ R6A"}="HCJ1 R6A";
$resn_exch{"1HCE R6A"}="HCE1 R6A"; $resn_exch{"1HCD R6A"}="HCD1 R6A"; $resn_exch{"1HCC R6A"}="HCC1 R6A";
$resn_exch{"1HCB R6A"}="HCB1 R6A"; $resn_exch{"1HBK R6A"}="HBK1 R6A"; $resn_exch{"1HL2 R6A"}="HL21 R6A";
$resn_exch{"2HL2 R6A"}="HL22 R6A"; $resn_exch{"1HBJ R6A"}="HBJ1 R6A"; $resn_exch{"1HBI R6A"}="HBI1 R6A";
$resn_exch{"1HL1 R6A"}="HL11 R6A"; $resn_exch{"2HL1 R6A"}="HL12 R6A"; $resn_exch{"1HBH R6A"}="HBH1 R6A";
$resn_exch{"1HBE R6A"}="HBE1 R6A"; $resn_exch{"1HBD R6A"}="HBD1 R6A"; $resn_exch{"1HBC R6A"}="HBC1 R6A";
$resn_exch{"1HBB R6A"}="HBB1 R6A"; $resn_exch{"1HAQ R6C"}="HAQ1 R6C"; $resn_exch{"2HAQ R6C"}="HAQ2 R6C";
$resn_exch{"1HAK R6C"}="HAK1 R6C"; $resn_exch{"2HAK R6C"}="HAK2 R6C"; $resn_exch{"1HAI R6C"}="HAI1 R6C";
$resn_exch{"2HAI R6C"}="HAI2 R6C"; $resn_exch{"3HAI R6C"}="HAI3 R6C"; $resn_exch{"1HAY R6C"}="HAY1 R6C";
$resn_exch{"2HAY R6C"}="HAY2 R6C"; $resn_exch{"3HAY R6C"}="HAY3 R6C"; $resn_exch{"1HBA R6C"}="HBA1 R6C";
$resn_exch{"2HBA R6C"}="HBA2 R6C"; $resn_exch{"3HBA R6C"}="HBA3 R6C"; $resn_exch{"1HAD R6C"}="HAD1 R6C";
$resn_exch{"2HAD R6C"}="HAD2 R6C"; $resn_exch{"1HC  R6G"}=" HC1 R6G"; $resn_exch{"2HC  R6G"}=" HC2 R6G";
$resn_exch{"1H28 R6G"}="H281 R6G"; $resn_exch{"2H28 R6G"}="H282 R6G"; $resn_exch{"1H31 R6G"}="H311 R6G";
$resn_exch{"2H31 R6G"}="H312 R6G"; $resn_exch{"1H29 R6G"}="H291 R6G"; $resn_exch{"2H29 R6G"}="H292 R6G";
$resn_exch{"1H21 R6G"}="H211 R6G"; $resn_exch{"2H21 R6G"}="H212 R6G"; $resn_exch{"3H21 R6G"}="H213 R6G";
$resn_exch{"1H20 R6G"}="H201 R6G"; $resn_exch{"2H20 R6G"}="H202 R6G"; $resn_exch{"3H20 R6G"}="H203 R6G";
$resn_exch{"1H24 R6G"}="H241 R6G"; $resn_exch{"2H24 R6G"}="H242 R6G"; $resn_exch{"1H25 R6G"}="H251 R6G";
$resn_exch{"2H25 R6G"}="H252 R6G"; $resn_exch{"1H26 R6G"}="H261 R6G"; $resn_exch{"2H26 R6G"}="H262 R6G";
$resn_exch{"1H27 R6G"}="H271 R6G"; $resn_exch{"2H27 R6G"}="H272 R6G"; $resn_exch{"1H30 R6G"}="H301 R6G";
$resn_exch{"2H30 R6G"}="H302 R6G"; $resn_exch{"3H30 R6G"}="H303 R6G"; $resn_exch{"1H32 R6G"}="H321 R6G";
$resn_exch{"2H32 R6G"}="H322 R6G"; $resn_exch{"3H32 R6G"}="H323 R6G"; $resn_exch{"1HO  R6G"}=" HO1 R6G";
$resn_exch{"1H15 R71"}="H151 R71"; $resn_exch{"2H15 R71"}="H152 R71"; $resn_exch{"1H16 R71"}="H161 R71";
$resn_exch{"2H16 R71"}="H162 R71"; $resn_exch{"1H17 R71"}="H171 R71"; $resn_exch{"2H17 R71"}="H172 R71";
$resn_exch{"1H18 R71"}="H181 R71"; $resn_exch{"2H18 R71"}="H182 R71"; $resn_exch{"1H19 R71"}="H191 R71";
$resn_exch{"2H19 R71"}="H192 R71"; $resn_exch{"1H20 R71"}="H201 R71"; $resn_exch{"2H20 R71"}="H202 R71";
$resn_exch{"1H27 R71"}="H271 R71"; $resn_exch{"2H27 R71"}="H272 R71"; $resn_exch{"3H27 R71"}="H273 R71";
$resn_exch{"1H25 R71"}="H251 R71"; $resn_exch{"2H25 R71"}="H252 R71"; $resn_exch{"1H29 R71"}="H291 R71";
$resn_exch{"2H29 R71"}="H292 R71"; $resn_exch{"1H3A R79"}="H3A1 R79"; $resn_exch{"2H3A R79"}="H3A2 R79";
$resn_exch{"1H1A R79"}="H1A1 R79"; $resn_exch{"2H1A R79"}="H1A2 R79"; $resn_exch{"1H1B R79"}="H1B1 R79";
$resn_exch{"2H1B R79"}="H1B2 R79"; $resn_exch{"1H2B R79"}="H2B1 R79"; $resn_exch{"2H2B R79"}="H2B2 R79";
$resn_exch{"1H3B R79"}="H3B1 R79"; $resn_exch{"2H3B R79"}="H3B2 R79"; $resn_exch{"1H4B R79"}="H4B1 R79";
$resn_exch{"2H4B R79"}="H4B2 R79"; $resn_exch{"1H5B R79"}="H5B1 R79"; $resn_exch{"2H5B R79"}="H5B2 R79";
$resn_exch{"1H6B R79"}="H6B1 R79"; $resn_exch{"2H6B R79"}="H6B2 R79"; $resn_exch{"1H1F R79"}="H1F1 R79";
$resn_exch{"2H1F R79"}="H1F2 R79"; $resn_exch{"3H1F R79"}="H1F3 R79"; $resn_exch{"1HN  R7A"}=" H   R7A";
$resn_exch{"2HN  R7A"}=" H2  R7A"; $resn_exch{"1HB  R7A"}=" HB2 R7A"; $resn_exch{"2HB  R7A"}=" HB3 R7A";
$resn_exch{"1HE  R7A"}=" HE2 R7A"; $resn_exch{"2HE  R7A"}=" HE3 R7A"; $resn_exch{"1H9  R7A"}=" H91 R7A";
$resn_exch{"2H9  R7A"}=" H92 R7A"; $resn_exch{"3H9  R7A"}=" H93 R7A"; $resn_exch{"1H8  R7A"}=" H81 R7A";
$resn_exch{"2H8  R7A"}=" H82 R7A"; $resn_exch{"3H8  R7A"}=" H83 R7A"; $resn_exch{"1H6  R7A"}=" H61 R7A";
$resn_exch{"2H6  R7A"}=" H62 R7A"; $resn_exch{"3H6  R7A"}=" H63 R7A"; $resn_exch{"1H7  R7A"}=" H71 R7A";
$resn_exch{"2H7  R7A"}=" H72 R7A"; $resn_exch{"3H7  R7A"}=" H73 R7A"; $resn_exch{"3HM2 R7U"}="HM23 R7U";
$resn_exch{"2HM2 R7U"}="HM22 R7U"; $resn_exch{"1HM2 R7U"}="HM21 R7U"; $resn_exch{"3HM1 R7U"}="HM13 R7U";
$resn_exch{"2HM1 R7U"}="HM12 R7U"; $resn_exch{"1HM1 R7U"}="HM11 R7U"; $resn_exch{"1HL7 R7U"}="HL71 R7U";
$resn_exch{"2HL7 R7U"}="HL72 R7U"; $resn_exch{"1HL6 R7U"}="HL61 R7U"; $resn_exch{"2HL6 R7U"}="HL62 R7U";
$resn_exch{"1HL5 R7U"}="HL51 R7U"; $resn_exch{"2HL5 R7U"}="HL52 R7U"; $resn_exch{"1HL4 R7U"}="HL41 R7U";
$resn_exch{"2HL4 R7U"}="HL42 R7U"; $resn_exch{"1HCK R7U"}="HCK1 R7U"; $resn_exch{"1HCJ R7U"}="HCJ1 R7U";
$resn_exch{"1HCE R7U"}="HCE1 R7U"; $resn_exch{"1HCD R7U"}="HCD1 R7U"; $resn_exch{"1HCC R7U"}="HCC1 R7U";
$resn_exch{"1HCB R7U"}="HCB1 R7U"; $resn_exch{"1HBK R7U"}="HBK1 R7U"; $resn_exch{"1HL3 R7U"}="HL31 R7U";
$resn_exch{"2HL3 R7U"}="HL32 R7U"; $resn_exch{"1HBJ R7U"}="HBJ1 R7U"; $resn_exch{"1HBI R7U"}="HBI1 R7U";
$resn_exch{"1HL1 R7U"}="HL11 R7U"; $resn_exch{"2HL1 R7U"}="HL12 R7U"; $resn_exch{"1HL2 R7U"}="HL21 R7U";
$resn_exch{"2HL2 R7U"}="HL22 R7U"; $resn_exch{"1HBH R7U"}="HBH1 R7U"; $resn_exch{"1HBE R7U"}="HBE1 R7U";
$resn_exch{"1HBD R7U"}="HBD1 R7U"; $resn_exch{"1HBC R7U"}="HBC1 R7U"; $resn_exch{"1HBB R7U"}="HBB1 R7U";
$resn_exch{"1H48 R86"}="H481 R86"; $resn_exch{"2H48 R86"}="H482 R86"; $resn_exch{"3H48 R86"}="H483 R86";
$resn_exch{"1H38 R86"}="H381 R86"; $resn_exch{"2H38 R86"}="H382 R86"; $resn_exch{"1H37 R86"}="H371 R86";
$resn_exch{"2H37 R86"}="H372 R86"; $resn_exch{"1H32 R86"}="H321 R86"; $resn_exch{"2H32 R86"}="H322 R86";
$resn_exch{"1H26 R86"}="H261 R86"; $resn_exch{"2H26 R86"}="H262 R86"; $resn_exch{"1H31 R86"}="H311 R86";
$resn_exch{"2H31 R86"}="H312 R86"; $resn_exch{"3H31 R86"}="H313 R86"; $resn_exch{"1H30 R86"}="H301 R86";
$resn_exch{"2H30 R86"}="H302 R86"; $resn_exch{"3H30 R86"}="H303 R86"; $resn_exch{"1H23 R86"}="H231 R86";
$resn_exch{"2H23 R86"}="H232 R86"; $resn_exch{"1H22 R86"}="H221 R86"; $resn_exch{"2H22 R86"}="H222 R86";
$resn_exch{"1H17 R86"}="H171 R86"; $resn_exch{"2H17 R86"}="H172 R86"; $resn_exch{"3H17 R86"}="H173 R86";
$resn_exch{"1H15 R88"}="H151 R88"; $resn_exch{"2H15 R88"}="H152 R88"; $resn_exch{"1H20 R88"}="H201 R88";
$resn_exch{"2H20 R88"}="H202 R88"; $resn_exch{"1H27 R88"}="H271 R88"; $resn_exch{"2H27 R88"}="H272 R88";
$resn_exch{"3H27 R88"}="H273 R88"; $resn_exch{"1H26 R88"}="H261 R88"; $resn_exch{"2H26 R88"}="H262 R88";
$resn_exch{"1H3E R88"}="H3E1 R88"; $resn_exch{"2H3E R88"}="H3E2 R88"; $resn_exch{"1H92 R94"}="H921 R94";
$resn_exch{"2H92 R94"}="H922 R94"; $resn_exch{"1H93 R94"}="H931 R94"; $resn_exch{"2H93 R94"}="H932 R94";
$resn_exch{"1H94 R94"}="H941 R94"; $resn_exch{"2H94 R94"}="H942 R94"; $resn_exch{"1H95 R94"}="H951 R94";
$resn_exch{"2H95 R94"}="H952 R94"; $resn_exch{"1H96 R94"}="H961 R94"; $resn_exch{"2H96 R94"}="H962 R94";
$resn_exch{"1H36 R94"}="H361 R94"; $resn_exch{"2H36 R94"}="H362 R94"; $resn_exch{"1H35 R94"}="H351 R94";
$resn_exch{"2H35 R94"}="H352 R94"; $resn_exch{"1H34 R94"}="H341 R94"; $resn_exch{"2H34 R94"}="H342 R94";
$resn_exch{"1H32 R94"}="H321 R94"; $resn_exch{"2H32 R94"}="H322 R94"; $resn_exch{"1H32 R96"}="H321 R96";
$resn_exch{"2H32 R96"}="H322 R96"; $resn_exch{"1H34 R96"}="H341 R96"; $resn_exch{"2H34 R96"}="H342 R96";
$resn_exch{"1H35 R96"}="H351 R96"; $resn_exch{"2H35 R96"}="H352 R96"; $resn_exch{"1H36 R96"}="H361 R96";
$resn_exch{"2H36 R96"}="H362 R96"; $resn_exch{"1H92 R96"}="H921 R96"; $resn_exch{"2H92 R96"}="H922 R96";
$resn_exch{"1H93 R96"}="H931 R96"; $resn_exch{"2H93 R96"}="H932 R96"; $resn_exch{"1H94 R96"}="H941 R96";
$resn_exch{"2H94 R96"}="H942 R96"; $resn_exch{"1H97 R96"}="H971 R96"; $resn_exch{"2H97 R96"}="H972 R96";
$resn_exch{"3H97 R96"}="H973 R96"; $resn_exch{"1H98 R96"}="H981 R96"; $resn_exch{"2H98 R96"}="H982 R96";
$resn_exch{"3H98 R96"}="H983 R96"; $resn_exch{"1H96 R96"}="H961 R96"; $resn_exch{"2H96 R96"}="H962 R96";
$resn_exch{"1H2  R99"}=" H21 R99"; $resn_exch{"2H2  R99"}=" H22 R99"; $resn_exch{"1H3  R99"}=" H31 R99";
$resn_exch{"1H10 R99"}="H101 R99"; $resn_exch{"1H12 R99"}="H121 R99"; $resn_exch{"2H12 R99"}="H122 R99";
$resn_exch{"1H13 R99"}="H131 R99"; $resn_exch{"2H13 R99"}="H132 R99"; $resn_exch{"1H15 R99"}="H151 R99";
$resn_exch{"1H16 R99"}="H161 R99"; $resn_exch{"1H17 R99"}="H171 R99"; $resn_exch{"1H18 R99"}="H181 R99";
$resn_exch{"1H19 R99"}="H191 R99"; $resn_exch{"1H21 R99"}="H211 R99"; $resn_exch{"2H21 R99"}="H212 R99";
$resn_exch{"1H22 R99"}="H221 R99"; $resn_exch{"2H22 R99"}="H222 R99"; $resn_exch{"1H23 R99"}="H231 R99";
$resn_exch{"1H25 R99"}="H251 R99"; $resn_exch{"2H25 R99"}="H252 R99"; $resn_exch{"3H25 R99"}="H253 R99";
$resn_exch{"1H24 R99"}="H241 R99"; $resn_exch{"2H24 R99"}="H242 R99"; $resn_exch{"3H24 R99"}="H243 R99";
$resn_exch{"3HM2 R9A"}="HM23 R9A"; $resn_exch{"2HM2 R9A"}="HM22 R9A"; $resn_exch{"1HM2 R9A"}="HM21 R9A";
$resn_exch{"3HM1 R9A"}="HM13 R9A"; $resn_exch{"2HM1 R9A"}="HM12 R9A"; $resn_exch{"1HM1 R9A"}="HM11 R9A";
$resn_exch{"1HL9 R9A"}="HL91 R9A"; $resn_exch{"2HL9 R9A"}="HL92 R9A"; $resn_exch{"1HL8 R9A"}="HL81 R9A";
$resn_exch{"2HL8 R9A"}="HL82 R9A"; $resn_exch{"1HL7 R9A"}="HL71 R9A"; $resn_exch{"2HL7 R9A"}="HL72 R9A";
$resn_exch{"1HL6 R9A"}="HL61 R9A"; $resn_exch{"2HL6 R9A"}="HL62 R9A"; $resn_exch{"1HCK R9A"}="HCK1 R9A";
$resn_exch{"1HCJ R9A"}="HCJ1 R9A"; $resn_exch{"1HCE R9A"}="HCE1 R9A"; $resn_exch{"1HCD R9A"}="HCD1 R9A";
$resn_exch{"1HCC R9A"}="HCC1 R9A"; $resn_exch{"1HCB R9A"}="HCB1 R9A"; $resn_exch{"1HBK R9A"}="HBK1 R9A";
$resn_exch{"1HL5 R9A"}="HL51 R9A"; $resn_exch{"2HL5 R9A"}="HL52 R9A"; $resn_exch{"1HBJ R9A"}="HBJ1 R9A";
$resn_exch{"1HBI R9A"}="HBI1 R9A"; $resn_exch{"1HL3 R9A"}="HL31 R9A"; $resn_exch{"2HL3 R9A"}="HL32 R9A";
$resn_exch{"1HL2 R9A"}="HL21 R9A"; $resn_exch{"2HL2 R9A"}="HL22 R9A"; $resn_exch{"1HL4 R9A"}="HL41 R9A";
$resn_exch{"2HL4 R9A"}="HL42 R9A"; $resn_exch{"1HL1 R9A"}="HL11 R9A"; $resn_exch{"2HL1 R9A"}="HL12 R9A";
$resn_exch{"1HBH R9A"}="HBH1 R9A"; $resn_exch{"1HBE R9A"}="HBE1 R9A"; $resn_exch{"1HBD R9A"}="HBD1 R9A";
$resn_exch{"1HBC R9A"}="HBC1 R9A"; $resn_exch{"1HBB R9A"}="HBB1 R9A"; $resn_exch{"1H14 RA2"}="H141 RA2";
$resn_exch{"2H14 RA2"}="H142 RA2"; $resn_exch{"1H15 RA2"}="H151 RA2"; $resn_exch{"2H15 RA2"}="H152 RA2";
$resn_exch{"3H15 RA2"}="H153 RA2"; $resn_exch{"1H21 RA2"}="H211 RA2"; $resn_exch{"2H21 RA2"}="H212 RA2";
$resn_exch{"1H23 RA2"}="H231 RA2"; $resn_exch{"2H23 RA2"}="H232 RA2"; $resn_exch{"3H23 RA2"}="H233 RA2";
$resn_exch{"1H30 RA2"}="H301 RA2"; $resn_exch{"2H30 RA2"}="H302 RA2"; $resn_exch{"1H31 RA2"}="H311 RA2";
$resn_exch{"2H31 RA2"}="H312 RA2"; $resn_exch{"1H38 RA2"}="H381 RA2"; $resn_exch{"2H28 RA2"}="H282 RA2";
$resn_exch{"1H42 RA2"}="H421 RA2"; $resn_exch{"2H42 RA2"}="H422 RA2"; $resn_exch{"1H22 RA4"}="H221 RA4";
$resn_exch{"2H22 RA4"}="H222 RA4"; $resn_exch{"1H20 RA4"}="H201 RA4"; $resn_exch{"2H20 RA4"}="H202 RA4";
$resn_exch{"3H20 RA4"}="H203 RA4"; $resn_exch{"1H19 RA4"}="H191 RA4"; $resn_exch{"2H19 RA4"}="H192 RA4";
$resn_exch{"3H19 RA4"}="H193 RA4"; $resn_exch{"1H15 RA4"}="H151 RA4"; $resn_exch{"2H15 RA4"}="H152 RA4";
$resn_exch{"1H14 RA4"}="H141 RA4"; $resn_exch{"2H14 RA4"}="H142 RA4"; $resn_exch{"1H13 RA4"}="H131 RA4";
$resn_exch{"2H13 RA4"}="H132 RA4"; $resn_exch{"1H8  RA4"}=" H81 RA4"; $resn_exch{"2H8  RA4"}=" H82 RA4";
$resn_exch{"1H7  RA4"}=" H71 RA4"; $resn_exch{"2H7  RA4"}=" H72 RA4"; $resn_exch{"1H6  RA4"}=" H61 RA4";
$resn_exch{"2H6  RA4"}=" H62 RA4"; $resn_exch{"1H5  RA4"}=" H51 RA4"; $resn_exch{"2H5  RA4"}=" H52 RA4";
$resn_exch{"1HN3 RA4"}="HN31 RA4"; $resn_exch{"2HN3 RA4"}="HN32 RA4"; $resn_exch{"1HN3 RA8"}="HN31 RA8";
$resn_exch{"2HN3 RA8"}="HN32 RA8"; $resn_exch{"1H6  RA8"}=" H61 RA8"; $resn_exch{"2H6  RA8"}=" H62 RA8";
$resn_exch{"1H7  RA8"}=" H71 RA8"; $resn_exch{"2H7  RA8"}=" H72 RA8"; $resn_exch{"1H8  RA8"}=" H81 RA8";
$resn_exch{"2H8  RA8"}=" H82 RA8"; $resn_exch{"1H9  RA8"}=" H91 RA8"; $resn_exch{"2H9  RA8"}=" H92 RA8";
$resn_exch{"1H27 RA8"}="H271 RA8"; $resn_exch{"2H27 RA8"}="H272 RA8"; $resn_exch{"1H20 RA8"}="H201 RA8";
$resn_exch{"2H20 RA8"}="H202 RA8"; $resn_exch{"1H23 RA8"}="H231 RA8"; $resn_exch{"2H23 RA8"}="H232 RA8";
$resn_exch{"3H23 RA8"}="H233 RA8"; $resn_exch{"1H22 RA8"}="H221 RA8"; $resn_exch{"2H22 RA8"}="H222 RA8";
$resn_exch{"3H22 RA8"}="H223 RA8"; $resn_exch{"1H17 RA8"}="H171 RA8"; $resn_exch{"2H17 RA8"}="H172 RA8";
$resn_exch{"1H13 RA8"}="H131 RA8"; $resn_exch{"2H13 RA8"}="H132 RA8"; $resn_exch{"1H14 RA8"}="H141 RA8";
$resn_exch{"2H14 RA8"}="H142 RA8"; $resn_exch{"1HC6 RAA"}="HC61 RAA"; $resn_exch{"2HC6 RAA"}="HC62 RAA";
$resn_exch{"3HC6 RAA"}="HC63 RAA"; $resn_exch{"1H5' RAB"}="H5'1 RAB"; $resn_exch{"2H5' RAB"}="H5'2 RAB";
$resn_exch{"1HN6 RAB"}="HN61 RAB"; $resn_exch{"2HN6 RAB"}="HN62 RAB"; $resn_exch{"1HC8 RAC"}="HC81 RAC";
$resn_exch{"2HC8 RAC"}="HC82 RAC"; $resn_exch{"3HC8 RAC"}="HC83 RAC"; $resn_exch{"1HC9 RAC"}="HC91 RAC";
$resn_exch{"2HC9 RAC"}="HC92 RAC"; $resn_exch{"3HC9 RAC"}="HC93 RAC"; $resn_exch{"1H11 RAC"}="H111 RAC";
$resn_exch{"2H11 RAC"}="H112 RAC"; $resn_exch{"CH13 RAC"}="H13C RAC"; $resn_exch{"CH14 RAC"}="H14C RAC";
$resn_exch{"CH16 RAC"}="H16C RAC"; $resn_exch{"CH17 RAC"}="H17C RAC"; $resn_exch{"1H3  RAD"}=" H31 RAD";
$resn_exch{"2H3  RAD"}=" H32 RAD"; $resn_exch{"1H4  RAD"}=" H41 RAD"; $resn_exch{"2H4  RAD"}=" H42 RAD";
$resn_exch{"1H5  RAD"}=" H51 RAD"; $resn_exch{"2H5  RAD"}=" H52 RAD"; $resn_exch{"1H6  RAD"}=" H61 RAD";
$resn_exch{"2H6  RAD"}=" H62 RAD"; $resn_exch{"1H11 RAD"}="H111 RAD"; $resn_exch{"2H11 RAD"}="H112 RAD";
$resn_exch{"1H12 RAD"}="H121 RAD"; $resn_exch{"2H12 RAD"}="H122 RAD"; $resn_exch{"1H14 RAD"}="H141 RAD";
$resn_exch{"2H14 RAD"}="H142 RAD"; $resn_exch{"1H23 RAD"}="H231 RAD"; $resn_exch{"2H23 RAD"}="H232 RAD";
$resn_exch{"1H32 RAD"}="H321 RAD"; $resn_exch{"2H32 RAD"}="H322 RAD"; $resn_exch{"1H35 RAD"}="H351 RAD";
$resn_exch{"2H35 RAD"}="H352 RAD"; $resn_exch{"1H37 RAD"}="H371 RAD"; $resn_exch{"2H37 RAD"}="H372 RAD";
$resn_exch{"1H40 RAD"}="H401 RAD"; $resn_exch{"2H40 RAD"}="H402 RAD"; $resn_exch{"1H41 RAD"}="H411 RAD";
$resn_exch{"2H41 RAD"}="H412 RAD"; $resn_exch{"1H42 RAD"}="H421 RAD"; $resn_exch{"2H42 RAD"}="H422 RAD";
$resn_exch{"3H42 RAD"}="H423 RAD"; $resn_exch{"1H43 RAD"}="H431 RAD"; $resn_exch{"2H43 RAD"}="H432 RAD";
$resn_exch{"3H43 RAD"}="H433 RAD"; $resn_exch{"1H44 RAD"}="H441 RAD"; $resn_exch{"2H44 RAD"}="H442 RAD";
$resn_exch{"3H44 RAD"}="H443 RAD"; $resn_exch{"1H45 RAD"}="H451 RAD"; $resn_exch{"2H45 RAD"}="H452 RAD";
$resn_exch{"3H45 RAD"}="H453 RAD"; $resn_exch{"1H46 RAD"}="H461 RAD"; $resn_exch{"2H46 RAD"}="H462 RAD";
$resn_exch{"3H46 RAD"}="H463 RAD"; $resn_exch{"1H47 RAD"}="H471 RAD"; $resn_exch{"2H47 RAD"}="H472 RAD";
$resn_exch{"3H47 RAD"}="H473 RAD"; $resn_exch{"1H48 RAD"}="H481 RAD"; $resn_exch{"2H48 RAD"}="H482 RAD";
$resn_exch{"3H48 RAD"}="H483 RAD"; $resn_exch{"1H49 RAD"}="H491 RAD"; $resn_exch{"2H49 RAD"}="H492 RAD";
$resn_exch{"1H50 RAD"}="H501 RAD"; $resn_exch{"2H50 RAD"}="H502 RAD"; $resn_exch{"3H50 RAD"}="H503 RAD";
$resn_exch{"1H51 RAD"}="H511 RAD"; $resn_exch{"2H51 RAD"}="H512 RAD"; $resn_exch{"3H51 RAD"}="H513 RAD";
$resn_exch{"1H52 RAD"}="H521 RAD"; $resn_exch{"2H52 RAD"}="H522 RAD"; $resn_exch{"3H52 RAD"}="H523 RAD";
$resn_exch{"1HC2 RAE"}="HC21 RAE"; $resn_exch{"2HC2 RAE"}="HC22 RAE"; $resn_exch{"1HC6 RAE"}="HC61 RAE";
$resn_exch{"2HC6 RAE"}="HC62 RAE"; $resn_exch{"3HC6 RAE"}="HC63 RAE"; $resn_exch{"1H26 RAG"}="H261 RAG";
$resn_exch{"2H26 RAG"}="H262 RAG"; $resn_exch{"3H26 RAG"}="H263 RAG"; $resn_exch{"1H20 RAG"}="H201 RAG";
$resn_exch{"2H20 RAG"}="H202 RAG"; $resn_exch{"1H19 RAG"}="H191 RAG"; $resn_exch{"2H19 RAG"}="H192 RAG";
$resn_exch{"1H18 RAG"}="H181 RAG"; $resn_exch{"2H18 RAG"}="H182 RAG"; $resn_exch{"1H31 RAG"}="H311 RAG";
$resn_exch{"2H31 RAG"}="H312 RAG"; $resn_exch{"3H31 RAG"}="H313 RAG"; $resn_exch{"1H12 RAG"}="H121 RAG";
$resn_exch{"2H12 RAG"}="H122 RAG"; $resn_exch{"3H12 RAG"}="H123 RAG"; $resn_exch{"1H56 RAG"}="H561 RAG";
$resn_exch{"2H56 RAG"}="H562 RAG"; $resn_exch{"1H48 RAG"}="H481 RAG"; $resn_exch{"2H48 RAG"}="H482 RAG";
$resn_exch{"1H39 RAG"}="H391 RAG"; $resn_exch{"2H39 RAG"}="H392 RAG"; $resn_exch{"1H14 RAI"}="H141 RAI";
$resn_exch{"2H14 RAI"}="H142 RAI"; $resn_exch{"1H17 RAI"}="H171 RAI"; $resn_exch{"2H17 RAI"}="H172 RAI";
$resn_exch{"1H18 RAI"}="H181 RAI"; $resn_exch{"2H18 RAI"}="H182 RAI"; $resn_exch{"1H24 RAI"}="H241 RAI";
$resn_exch{"2H24 RAI"}="H242 RAI"; $resn_exch{"1H28 RAI"}="H281 RAI"; $resn_exch{"2H28 RAI"}="H282 RAI";
$resn_exch{"1H28 RAJ"}="H281 RAJ"; $resn_exch{"2H28 RAJ"}="H282 RAJ"; $resn_exch{"3H28 RAJ"}="H283 RAJ";
$resn_exch{"1H29 RAJ"}="H291 RAJ"; $resn_exch{"2H29 RAJ"}="H292 RAJ"; $resn_exch{"3H29 RAJ"}="H293 RAJ";
$resn_exch{"1H27 RAJ"}="H271 RAJ"; $resn_exch{"2H27 RAJ"}="H272 RAJ"; $resn_exch{"1H26 RAJ"}="H261 RAJ";
$resn_exch{"2H26 RAJ"}="H262 RAJ"; $resn_exch{"1H25 RAJ"}="H251 RAJ"; $resn_exch{"2H25 RAJ"}="H252 RAJ";
$resn_exch{"1H16 RAJ"}="H161 RAJ"; $resn_exch{"2H16 RAJ"}="H162 RAJ"; $resn_exch{"3H16 RAJ"}="H163 RAJ";
$resn_exch{"1H24 RAL"}="H241 RAL"; $resn_exch{"2H24 RAL"}="H242 RAL"; $resn_exch{"1H25 RAL"}="H251 RAL";
$resn_exch{"2H25 RAL"}="H252 RAL"; $resn_exch{"1H27 RAL"}="H271 RAL"; $resn_exch{"2H27 RAL"}="H272 RAL";
$resn_exch{"1H28 RAL"}="H281 RAL"; $resn_exch{"2H28 RAL"}="H282 RAL"; $resn_exch{"1H29 RAL"}="H291 RAL";
$resn_exch{"2H29 RAL"}="H292 RAL"; $resn_exch{"1H30 RAL"}="H301 RAL"; $resn_exch{"2H30 RAL"}="H302 RAL";
$resn_exch{"1H31 RAL"}="H311 RAL"; $resn_exch{"2H31 RAL"}="H312 RAL"; $resn_exch{"1H6  RAM"}=" H61 RAM";
$resn_exch{"2H6  RAM"}=" H62 RAM"; $resn_exch{"3H6  RAM"}=" H63 RAM"; $resn_exch{"1H8  RAN"}=" H81 RAN";
$resn_exch{"2H8  RAN"}=" H82 RAN"; $resn_exch{"3H8  RAN"}=" H83 RAN"; $resn_exch{"1H6  RAN"}=" H61 RAN";
$resn_exch{"2H6  RAN"}=" H62 RAN"; $resn_exch{"1HC7 RAO"}="HC71 RAO"; $resn_exch{"2HC7 RAO"}="HC72 RAO";
$resn_exch{"3HC7 RAO"}="HC73 RAO"; $resn_exch{"1HC6 RAO"}="HC61 RAO"; $resn_exch{"2HC6 RAO"}="HC62 RAO";
$resn_exch{"3HC6 RAO"}="HC63 RAO"; $resn_exch{"1H3  RAP"}="H31A RAP"; $resn_exch{"2H3  RAP"}=" H32 RAP";
$resn_exch{"1H4  RAP"}=" H41 RAP"; $resn_exch{"2H4  RAP"}=" H42 RAP"; $resn_exch{"1H5  RAP"}=" H51 RAP";
$resn_exch{"2H5  RAP"}=" H52 RAP"; $resn_exch{"1H6  RAP"}=" H61 RAP"; $resn_exch{"2H6  RAP"}=" H62 RAP";
$resn_exch{"1H12 RAP"}="H121 RAP"; $resn_exch{"2H12 RAP"}="H122 RAP"; $resn_exch{"1H13 RAP"}="H131 RAP";
$resn_exch{"2H13 RAP"}="H132 RAP"; $resn_exch{"1H15 RAP"}="H151 RAP"; $resn_exch{"2H15 RAP"}="H152 RAP";
$resn_exch{"1H24 RAP"}="H241 RAP"; $resn_exch{"2H24 RAP"}="H242 RAP"; $resn_exch{"1H33 RAP"}="H331 RAP";
$resn_exch{"2H33 RAP"}="H332 RAP"; $resn_exch{"1H36 RAP"}="H361 RAP"; $resn_exch{"2H36 RAP"}="H362 RAP";
$resn_exch{"1H38 RAP"}="H381 RAP"; $resn_exch{"2H38 RAP"}="H382 RAP"; $resn_exch{"1H41 RAP"}="H411 RAP";
$resn_exch{"2H41 RAP"}="H412 RAP"; $resn_exch{"1H42 RAP"}="H421 RAP"; $resn_exch{"2H42 RAP"}="H422 RAP";
$resn_exch{"1H43 RAP"}="H431 RAP"; $resn_exch{"2H43 RAP"}="H432 RAP"; $resn_exch{"3H43 RAP"}="H433 RAP";
$resn_exch{"1H44 RAP"}="H441 RAP"; $resn_exch{"2H44 RAP"}="H442 RAP"; $resn_exch{"3H44 RAP"}="H443 RAP";
$resn_exch{"1H45 RAP"}="H451 RAP"; $resn_exch{"2H45 RAP"}="H452 RAP"; $resn_exch{"3H45 RAP"}="H453 RAP";
$resn_exch{"1H46 RAP"}="H461 RAP"; $resn_exch{"2H46 RAP"}="H462 RAP"; $resn_exch{"3H46 RAP"}="H463 RAP";
$resn_exch{"1H47 RAP"}="H471 RAP"; $resn_exch{"2H47 RAP"}="H472 RAP"; $resn_exch{"3H47 RAP"}="H473 RAP";
$resn_exch{"1H48 RAP"}="H481 RAP"; $resn_exch{"2H48 RAP"}="H482 RAP"; $resn_exch{"3H48 RAP"}="H483 RAP";
$resn_exch{"1H49 RAP"}="H491 RAP"; $resn_exch{"2H49 RAP"}="H492 RAP"; $resn_exch{"3H49 RAP"}="H493 RAP";
$resn_exch{"1H50 RAP"}="H501 RAP"; $resn_exch{"2H50 RAP"}="H502 RAP"; $resn_exch{"3H50 RAP"}="H503 RAP";
$resn_exch{"1H51 RAP"}="H511 RAP"; $resn_exch{"2H51 RAP"}="H512 RAP"; $resn_exch{"3H51 RAP"}="H513 RAP";
$resn_exch{"1H52 RAP"}="H521 RAP"; $resn_exch{"2H52 RAP"}="H522 RAP"; $resn_exch{"3H52 RAP"}="H523 RAP";
$resn_exch{"1H11 RAS"}="H111 RAS"; $resn_exch{"2H11 RAS"}="H112 RAS"; $resn_exch{"1H1  RAS"}=" H11 RAS";
$resn_exch{"2H1  RAS"}=" H12 RAS"; $resn_exch{"1H2  RAS"}=" H21 RAS"; $resn_exch{"2H2  RAS"}=" H22 RAS";
$resn_exch{"1H02 RAZ"}="H021 RAZ"; $resn_exch{"2H02 RAZ"}="H022 RAZ"; $resn_exch{"1H03 RAZ"}="H031 RAZ";
$resn_exch{"2H03 RAZ"}="H032 RAZ"; $resn_exch{"1H07 RAZ"}="H071 RAZ"; $resn_exch{"2H07 RAZ"}="H072 RAZ";
$resn_exch{"1H10 RAZ"}="H101 RAZ"; $resn_exch{"2H10 RAZ"}="H102 RAZ"; $resn_exch{"1H11 RAZ"}="H111 RAZ";
$resn_exch{"2H11 RAZ"}="H112 RAZ"; $resn_exch{"1H17 RAZ"}="H171 RAZ"; $resn_exch{"2H17 RAZ"}="H172 RAZ";
$resn_exch{"3H17 RAZ"}="H173 RAZ"; $resn_exch{"1H18 RAZ"}="H181 RAZ"; $resn_exch{"2H18 RAZ"}="H182 RAZ";
$resn_exch{"3H18 RAZ"}="H183 RAZ"; $resn_exch{"1H19 RAZ"}="H191 RAZ"; $resn_exch{"2H19 RAZ"}="H192 RAZ";
$resn_exch{"3H19 RAZ"}="H193 RAZ"; $resn_exch{"1H32 RAZ"}="H321 RAZ"; $resn_exch{"2H32 RAZ"}="H322 RAZ";
$resn_exch{"3H32 RAZ"}="H323 RAZ"; $resn_exch{"1H20 RB1"}="H201 RB1"; $resn_exch{"2H20 RB1"}="H202 RB1";
$resn_exch{"3H20 RB1"}="H203 RB1"; $resn_exch{"1H21 RB1"}="H211 RB1"; $resn_exch{"2H21 RB1"}="H212 RB1";
$resn_exch{"3H21 RB1"}="H213 RB1"; $resn_exch{"1H22 RB1"}="H221 RB1"; $resn_exch{"2H22 RB1"}="H222 RB1";
$resn_exch{"3H22 RB1"}="H223 RB1"; $resn_exch{"1H10 RB1"}="H101 RB1"; $resn_exch{"2H10 RB1"}="H102 RB1";
$resn_exch{"1H11 RB1"}="H111 RB1"; $resn_exch{"2H11 RB1"}="H112 RB1"; $resn_exch{"1H11 RB2"}="H111 RB2";
$resn_exch{"2H11 RB2"}="H112 RB2"; $resn_exch{"3H11 RB2"}="H113 RB2"; $resn_exch{"1H4  RB2"}=" H41 RB2";
$resn_exch{"2H4  RB2"}=" H42 RB2"; $resn_exch{"1H2  RB2"}=" H21 RB2"; $resn_exch{"2H2  RB2"}=" H22 RB2";
$resn_exch{"1H1  RB2"}=" H11 RB2"; $resn_exch{"2H1  RB2"}="H12A RB2"; $resn_exch{"1H2A RB3"}="H2A1 RB3";
$resn_exch{"2H2A RB3"}="H2A2 RB3"; $resn_exch{"1H4A RB3"}="H4A1 RB3"; $resn_exch{"2H4A RB3"}="H4A2 RB3";
$resn_exch{"3H4A RB3"}="H4A3 RB3"; $resn_exch{"1H2B RB3"}="H2B1 RB3"; $resn_exch{"2H2B RB3"}="H2B2 RB3";
$resn_exch{"1H4B RB3"}="H4B1 RB3"; $resn_exch{"2H4B RB3"}="H4B2 RB3"; $resn_exch{"3H4B RB3"}="H4B3 RB3";
$resn_exch{"1H2C RB3"}="H2C1 RB3"; $resn_exch{"2H2C RB3"}="H2C2 RB3"; $resn_exch{"1H4C RB3"}="H4C1 RB3";
$resn_exch{"2H4C RB3"}="H4C2 RB3"; $resn_exch{"3H4C RB3"}="H4C3 RB3"; $resn_exch{"1H1  RB3"}=" H11 RB3";
$resn_exch{"2H1  RB3"}=" H12 RB3"; $resn_exch{"3H1  RB3"}=" H13 RB3"; $resn_exch{"1H11 RBC"}="H111 RBC";
$resn_exch{"2H11 RBC"}="H112 RBC"; $resn_exch{"1H12 RBC"}="H121 RBC"; $resn_exch{"2H12 RBC"}="H122 RBC";
$resn_exch{"1H7  RBC"}=" H71 RBC"; $resn_exch{"2H7  RBC"}=" H72 RBC"; $resn_exch{"1H21 RBC"}="H211 RBC";
$resn_exch{"2H21 RBC"}="H212 RBC"; $resn_exch{"1H41 RBC"}="H411 RBC"; $resn_exch{"2H41 RBC"}="H412 RBC";
$resn_exch{"3H41 RBC"}="H413 RBC"; $resn_exch{"1H1D RBC"}="H1D1 RBC"; $resn_exch{"2H1D RBC"}="H1D2 RBC";
$resn_exch{"1H4  RBE"}=" H41 RBE"; $resn_exch{"2H4  RBE"}=" H42 RBE"; $resn_exch{"1H6  RBE"}=" H61 RBE";
$resn_exch{"2H6  RBE"}=" H62 RBE"; $resn_exch{"1H7  RBE"}=" H71 RBE"; $resn_exch{"2H7  RBE"}=" H72 RBE";
$resn_exch{"1HN1 RBE"}="HN11 RBE"; $resn_exch{"2HN1 RBE"}="HN12 RBE"; $resn_exch{"1HC7 RBF"}="HC71 RBF";
$resn_exch{"2HC7 RBF"}="HC72 RBF"; $resn_exch{"3HC7 RBF"}="HC73 RBF"; $resn_exch{"1HC8 RBF"}="HC81 RBF";
$resn_exch{"2HC8 RBF"}="HC82 RBF"; $resn_exch{"3HC8 RBF"}="HC83 RBF"; $resn_exch{"1HC1 RBF"}="HC11 RBF";
$resn_exch{"2HC1 RBF"}="HC12 RBF"; $resn_exch{"HC2* RBF"}="HC2' RBF"; $resn_exch{"HO2* RBF"}="HO2' RBF";
$resn_exch{"HC3* RBF"}="HC3' RBF"; $resn_exch{"HO3* RBF"}="HO3' RBF"; $resn_exch{"HC4* RBF"}="HC4' RBF";
$resn_exch{"HO4* RBF"}="HO4' RBF"; $resn_exch{"1HC5 RBF"}="HC51 RBF"; $resn_exch{"2HC5 RBF"}="HC52 RBF";
$resn_exch{"HO5* RBF"}="HO5' RBF"; $resn_exch{"1H12 RBP"}="H121 RBP"; $resn_exch{"2H12 RBP"}="H122 RBP";
$resn_exch{"1H11 RBP"}="H111 RBP"; $resn_exch{"2H11 RBP"}="H112 RBP"; $resn_exch{"1H15 RBP"}="H151 RBP";
$resn_exch{"2H15 RBP"}="H152 RBP"; $resn_exch{"1H14 RBP"}="H141 RBP"; $resn_exch{"2H14 RBP"}="H142 RBP";
$resn_exch{"1H11 RBS"}="H111 RBS"; $resn_exch{"1H51 RBS"}="H511 RBS"; $resn_exch{"2H51 RBS"}="H512 RBS";
$resn_exch{"1H13 RBT"}="H131 RBT"; $resn_exch{"2H13 RBT"}="H132 RBT"; $resn_exch{"3H13 RBT"}="H133 RBT";
$resn_exch{"1H14 RBT"}="H141 RBT"; $resn_exch{"2H14 RBT"}="H142 RBT"; $resn_exch{"3H14 RBT"}="H143 RBT";
$resn_exch{"1H30 RBT"}="H301 RBT"; $resn_exch{"2H30 RBT"}="H302 RBT"; $resn_exch{"3H30 RBT"}="H303 RBT";
$resn_exch{"1H31 RBT"}="H311 RBT"; $resn_exch{"2H31 RBT"}="H312 RBT"; $resn_exch{"3H31 RBT"}="H313 RBT";
$resn_exch{"1H32 RBT"}="H321 RBT"; $resn_exch{"2H32 RBT"}="H322 RBT"; $resn_exch{"3H32 RBT"}="H323 RBT";
$resn_exch{"1H33 RBT"}="H331 RBT"; $resn_exch{"2H33 RBT"}="H332 RBT"; $resn_exch{"3H33 RBT"}="H333 RBT";
$resn_exch{"1H34 RBT"}="H341 RBT"; $resn_exch{"2H34 RBT"}="H342 RBT"; $resn_exch{"3H34 RBT"}="H343 RBT";
$resn_exch{"1H36 RBT"}="H361 RBT"; $resn_exch{"2H36 RBT"}="H362 RBT"; $resn_exch{"3H36 RBT"}="H363 RBT";
$resn_exch{"1H37 RBT"}="H371 RBT"; $resn_exch{"2H37 RBT"}="H372 RBT"; $resn_exch{"3H37 RBT"}="H373 RBT";
$resn_exch{"1H39 RBT"}="H391 RBT"; $resn_exch{"2H39 RBT"}="H392 RBT"; $resn_exch{"1H40 RBT"}="H401 RBT";
$resn_exch{"2H40 RBT"}="H402 RBT"; $resn_exch{"1H41 RBT"}="H411 RBT"; $resn_exch{"2H41 RBT"}="H412 RBT";
$resn_exch{"1H42 RBT"}="H421 RBT"; $resn_exch{"2H42 RBT"}="H422 RBT"; $resn_exch{"1H43 RBT"}="H431 RBT";
$resn_exch{"2H43 RBT"}="H432 RBT"; $resn_exch{"1H45 RBT"}="H451 RBT"; $resn_exch{"2H45 RBT"}="H452 RBT";
$resn_exch{"3H45 RBT"}="H453 RBT"; $resn_exch{"1H46 RBT"}="H461 RBT"; $resn_exch{"2H46 RBT"}="H462 RBT";
$resn_exch{"3H46 RBT"}="H463 RBT"; $resn_exch{"1H5* RBY"}="H5'1 RBY"; $resn_exch{"2H5* RBY"}="H5'2 RBY";
$resn_exch{" H4* RBY"}=" H4' RBY"; $resn_exch{" H3* RBY"}=" H3' RBY"; $resn_exch{"HO3* RBY"}="HO3' RBY";
$resn_exch{" H2* RBY"}=" H2' RBY"; $resn_exch{"HO2* RBY"}="HO2' RBY"; $resn_exch{" H1* RBY"}=" H1' RBY";
$resn_exch{"1HN6 RBY"}="HN61 RBY"; $resn_exch{"2HN6 RBY"}="HN62 RBY"; $resn_exch{"1HX  RBY"}=" HX1 RBY";
$resn_exch{"2HX  RBY"}=" HX2 RBY"; $resn_exch{"1HR5 RBY"}="HR51 RBY"; $resn_exch{"2HR5 RBY"}="HR52 RBY";
$resn_exch{"1H8  RBZ"}=" H81 RBZ"; $resn_exch{"2H8  RBZ"}=" H82 RBZ"; $resn_exch{"3H8  RBZ"}=" H83 RBZ";
$resn_exch{"1H9  RBZ"}=" H91 RBZ"; $resn_exch{"2H9  RBZ"}=" H92 RBZ"; $resn_exch{"3H9  RBZ"}=" H93 RBZ";
$resn_exch{"2HOP RBZ"}="HOP2 RBZ"; $resn_exch{"3HOP RBZ"}="HOP3 RBZ"; $resn_exch{"1H5* RBZ"}="H5'1 RBZ";
$resn_exch{"2H5* RBZ"}="H5'2 RBZ"; $resn_exch{" H4* RBZ"}=" H4' RBZ"; $resn_exch{" H3* RBZ"}=" H3' RBZ";
$resn_exch{"3HO* RBZ"}="HO'3 RBZ"; $resn_exch{" H2* RBZ"}=" H2' RBZ"; $resn_exch{"2HO* RBZ"}="HO'2 RBZ";
$resn_exch{" H1* RBZ"}=" H1' RBZ"; $resn_exch{"1H6  RC1"}=" H61 RC1"; $resn_exch{"2H6  RC1"}=" H62 RC1";
$resn_exch{"1H9  RC1"}=" H91 RC1"; $resn_exch{"2H9  RC1"}=" H92 RC1"; $resn_exch{"3H9  RC1"}=" H93 RC1";
$resn_exch{"1HB2 RC7"}="HB21 RC7"; $resn_exch{"2HB2 RC7"}="HB22 RC7"; $resn_exch{"1HA3 RC7"}="HA31 RC7";
$resn_exch{"2HA3 RC7"}="HA32 RC7"; $resn_exch{"1HA1 RC7"}="HA11 RC7"; $resn_exch{"2HA1 RC7"}="HA12 RC7";
$resn_exch{"1HB1 RC7"}="HB11 RC7"; $resn_exch{"2HB1 RC7"}="HB12 RC7"; $resn_exch{"3H4  RCA"}=" H43 RCA";
$resn_exch{"2H4  RCA"}=" H42 RCA"; $resn_exch{"1H4  RCA"}=" H41 RCA"; $resn_exch{"2H12 RCA"}="H122 RCA";
$resn_exch{"1H12 RCA"}="H121 RCA"; $resn_exch{"2H6  RCA"}=" H62 RCA"; $resn_exch{"1H6  RCA"}=" H61 RCA";
$resn_exch{"3H8  RCA"}=" H83 RCA"; $resn_exch{"2H8  RCA"}=" H82 RCA"; $resn_exch{"1H8  RCA"}=" H81 RCA";
$resn_exch{"1H2  RCL"}=" H21 RCL"; $resn_exch{"2H2  RCL"}=" H22 RCL"; $resn_exch{"1H3  RCL"}=" H31 RCL";
$resn_exch{"2H3  RCL"}=" H32 RCL"; $resn_exch{"1H4  RCL"}=" H41 RCL"; $resn_exch{"2H4  RCL"}=" H42 RCL";
$resn_exch{"1H5  RCL"}=" H51 RCL"; $resn_exch{"2H5  RCL"}=" H52 RCL"; $resn_exch{"1H6  RCL"}=" H61 RCL";
$resn_exch{"2H6  RCL"}=" H62 RCL"; $resn_exch{"1H7  RCL"}=" H71 RCL"; $resn_exch{"2H7  RCL"}=" H72 RCL";
$resn_exch{"1H8  RCL"}=" H81 RCL"; $resn_exch{"2H8  RCL"}=" H82 RCL"; $resn_exch{"1H11 RCL"}="H111 RCL";
$resn_exch{"2H11 RCL"}="H112 RCL"; $resn_exch{"1H13 RCL"}="H131 RCL"; $resn_exch{"2H13 RCL"}="H132 RCL";
$resn_exch{"1H14 RCL"}="H141 RCL"; $resn_exch{"2H14 RCL"}="H142 RCL"; $resn_exch{"1H15 RCL"}="H151 RCL";
$resn_exch{"2H15 RCL"}="H152 RCL"; $resn_exch{"1H16 RCL"}="H161 RCL"; $resn_exch{"2H16 RCL"}="H162 RCL";
$resn_exch{"1H17 RCL"}="H171 RCL"; $resn_exch{"2H17 RCL"}="H172 RCL"; $resn_exch{"1H18 RCL"}="H181 RCL";
$resn_exch{"2H18 RCL"}="H182 RCL"; $resn_exch{"3H18 RCL"}="H183 RCL"; $resn_exch{"1H29 RCP"}="H291 RCP";
$resn_exch{"2H29 RCP"}="H292 RCP"; $resn_exch{"1H30 RCP"}="H301 RCP"; $resn_exch{"2H30 RCP"}="H302 RCP";
$resn_exch{"1H28 RCP"}="H281 RCP"; $resn_exch{"2H28 RCP"}="H282 RCP"; $resn_exch{"1H27 RCP"}="H271 RCP";
$resn_exch{"2H27 RCP"}="H272 RCP"; $resn_exch{"1H24 RCP"}="H241 RCP"; $resn_exch{"2H24 RCP"}="H242 RCP";
$resn_exch{"1H23 RCP"}="H231 RCP"; $resn_exch{"2H23 RCP"}="H232 RCP"; $resn_exch{"1H22 RCP"}="H221 RCP";
$resn_exch{"2H22 RCP"}="H222 RCP"; $resn_exch{"1H21 RCP"}="H211 RCP"; $resn_exch{"2H21 RCP"}="H212 RCP";
$resn_exch{"1H19 RCP"}="H191 RCP"; $resn_exch{"2H19 RCP"}="H192 RCP"; $resn_exch{"1H17 RCP"}="H171 RCP";
$resn_exch{"2H17 RCP"}="H172 RCP"; $resn_exch{"1H18 RCP"}="H181 RCP"; $resn_exch{"2H18 RCP"}="H182 RCP";
$resn_exch{"1H16 RCP"}="H161 RCP"; $resn_exch{"2H16 RCP"}="H162 RCP"; $resn_exch{"1H1C RCY"}="H1C1 RCY";
$resn_exch{"2H1C RCY"}="H1C2 RCY"; $resn_exch{"3H1C RCY"}="H1C3 RCY"; $resn_exch{"1H1V RCY"}="H1V1 RCY";
$resn_exch{"2H1V RCY"}="H1V2 RCY"; $resn_exch{"3H1V RCY"}="H1V3 RCY"; $resn_exch{"1H1Y RCY"}="H1Y1 RCY";
$resn_exch{"2H1Y RCY"}="H1Y2 RCY"; $resn_exch{"3H1Y RCY"}="H1Y3 RCY"; $resn_exch{"1H1Z RCY"}="H1Z1 RCY";
$resn_exch{"2H1Z RCY"}="H1Z2 RCY"; $resn_exch{"3H1Z RCY"}="H1Z3 RCY"; $resn_exch{"1H1M RCY"}="H1M1 RCY";
$resn_exch{"2H1M RCY"}="H1M2 RCY"; $resn_exch{"1H1L RCY"}="H1L1 RCY"; $resn_exch{"2H1L RCY"}="H1L2 RCY";
$resn_exch{"1HB  RCY"}=" HB1 RCY"; $resn_exch{"2HB  RCY"}=" HB2 RCY"; $resn_exch{"1HN  RCY"}=" HN1 RCY";
$resn_exch{"2HN  RCY"}=" HN2 RCY"; $resn_exch{"1H7  RDA"}=" H71 RDA"; $resn_exch{"2H7  RDA"}=" H72 RDA";
$resn_exch{"3H7  RDA"}=" H73 RDA"; $resn_exch{"1H9  RDA"}=" H91 RDA"; $resn_exch{"2H9  RDA"}=" H92 RDA";
$resn_exch{"1H10 RDA"}="H101 RDA"; $resn_exch{"2H10 RDA"}="H102 RDA"; $resn_exch{"1H16 RDA"}="H161 RDA";
$resn_exch{"2H16 RDA"}="H162 RDA"; $resn_exch{"3H16 RDA"}="H163 RDA"; $resn_exch{"1H8  RDC"}=" H81 RDC";
$resn_exch{"2H8  RDC"}=" H82 RDC"; $resn_exch{"1H16 RDC"}="H161 RDC"; $resn_exch{"2H16 RDC"}="H162 RDC";
$resn_exch{"1H18 RDC"}="H181 RDC"; $resn_exch{"2H18 RDC"}="H182 RDC"; $resn_exch{"3H18 RDC"}="H183 RDC";
$resn_exch{"1HAC RDE"}="HAC1 RDE"; $resn_exch{"2HAC RDE"}="HAC2 RDE"; $resn_exch{"3HAC RDE"}="HAC3 RDE";
$resn_exch{"1HAA RDE"}="HAA1 RDE"; $resn_exch{"2HAA RDE"}="HAA2 RDE"; $resn_exch{"3HAA RDE"}="HAA3 RDE";
$resn_exch{"1HAJ RDE"}="HAJ1 RDE"; $resn_exch{"1HAB RDE"}="HAB1 RDE"; $resn_exch{"2HAB RDE"}="HAB2 RDE";
$resn_exch{"3HAB RDE"}="HAB3 RDE"; $resn_exch{"1HAM RDE"}="HAM1 RDE"; $resn_exch{"2HAM RDE"}="HAM2 RDE";
$resn_exch{"1HAL RDE"}="HAL1 RDE"; $resn_exch{"2HAL RDE"}="HAL2 RDE"; $resn_exch{"1HAK RDE"}="HAK1 RDE";
$resn_exch{"1HAI RDE"}="HAI1 RDE"; $resn_exch{"1HD1 RDF"}="HD11 RDF"; $resn_exch{"2HD1 RDF"}="HD12 RDF";
$resn_exch{"3HD1 RDF"}="HD13 RDF"; $resn_exch{"1HD2 RDF"}="HD21 RDF"; $resn_exch{"2HD2 RDF"}="HD22 RDF";
$resn_exch{"3HD2 RDF"}="HD23 RDF"; $resn_exch{"1HB0 RDF"}="HB01 RDF"; $resn_exch{"2HB0 RDF"}="HB02 RDF";
$resn_exch{"1H4  RDI"}=" H41 RDI"; $resn_exch{"2H4  RDI"}=" H42 RDI"; $resn_exch{"1H8  RDI"}=" H81 RDI";
$resn_exch{"2H8  RDI"}=" H82 RDI"; $resn_exch{"1H10 RDI"}="H101 RDI"; $resn_exch{"2H10 RDI"}="H102 RDI";
$resn_exch{"1H11 RDI"}="H111 RDI"; $resn_exch{"2H11 RDI"}="H112 RDI"; $resn_exch{"1H12 RDI"}="H121 RDI";
$resn_exch{"2H12 RDI"}="H122 RDI"; $resn_exch{"1H13 RDI"}="H131 RDI"; $resn_exch{"2H13 RDI"}="H132 RDI";
$resn_exch{"1H16 RDI"}="H161 RDI"; $resn_exch{"2H16 RDI"}="H162 RDI"; $resn_exch{"1H18 RDI"}="H181 RDI";
$resn_exch{"2H18 RDI"}="H182 RDI"; $resn_exch{"3H18 RDI"}="H183 RDI"; $resn_exch{"OH15 RDL"}="H15O RDL";
$resn_exch{"1H15 RDL"}="H151 RDL"; $resn_exch{"2H15 RDL"}="H152 RDL"; $resn_exch{"1H14 RDL"}="H141 RDL";
$resn_exch{"OH14 RDL"}="H14O RDL"; $resn_exch{"1H13 RDL"}="H131 RDL"; $resn_exch{"OH13 RDL"}="H13O RDL";
$resn_exch{"1H12 RDL"}="H121 RDL"; $resn_exch{"OH12 RDL"}="H12O RDL"; $resn_exch{"1H11 RDL"}="H111 RDL";
$resn_exch{"2H11 RDL"}="H112 RDL"; $resn_exch{"2HOP RDP"}="HOP2 RDP"; $resn_exch{"3HOP RDP"}="HOP3 RDP";
$resn_exch{" H1* RDP"}=" H1' RDP"; $resn_exch{" H2* RDP"}=" H2' RDP"; $resn_exch{"*HO2 RDP"}="HO2' RDP";
$resn_exch{" H3* RDP"}=" H3' RDP"; $resn_exch{"*HO3 RDP"}="HO3' RDP"; $resn_exch{" H4* RDP"}=" H4' RDP";
$resn_exch{"1H5* RDP"}="H5'1 RDP"; $resn_exch{"2H5* RDP"}="H5'2 RDP"; $resn_exch{"*HO5 RDP"}="HO5' RDP";
$resn_exch{"1H10 RDR"}="H101 RDR"; $resn_exch{"2H10 RDR"}="H102 RDR"; $resn_exch{"1H6  RDR"}=" H61 RDR";
$resn_exch{"2H6  RDR"}=" H62 RDR"; $resn_exch{"1H4  RDR"}=" H41 RDR"; $resn_exch{"2H4  RDR"}=" H42 RDR";
$resn_exch{"1H17 RDR"}="H171 RDR"; $resn_exch{"2H17 RDR"}="H172 RDR"; $resn_exch{"1H19 RDR"}="H191 RDR";
$resn_exch{"2H19 RDR"}="H192 RDR"; $resn_exch{"1H13 RDR"}="H131 RDR"; $resn_exch{"2H13 RDR"}="H132 RDR";
$resn_exch{"1H2  RE9"}=" H21 RE9"; $resn_exch{"2H2  RE9"}=" H22 RE9"; $resn_exch{"1H3  RE9"}=" H31 RE9";
$resn_exch{"2H3  RE9"}=" H32 RE9"; $resn_exch{"1H4  RE9"}=" H41 RE9"; $resn_exch{"2H4  RE9"}=" H42 RE9";
$resn_exch{"1H8  RE9"}=" H81 RE9"; $resn_exch{"2H8  RE9"}=" H82 RE9"; $resn_exch{"1H10 RE9"}="H101 RE9";
$resn_exch{"2H10 RE9"}="H102 RE9"; $resn_exch{"1H16 RE9"}="H161 RE9"; $resn_exch{"2H16 RE9"}="H162 RE9";
$resn_exch{"3H16 RE9"}="H163 RE9"; $resn_exch{"1H17 RE9"}="H171 RE9"; $resn_exch{"2H17 RE9"}="H172 RE9";
$resn_exch{"3H17 RE9"}="H173 RE9"; $resn_exch{"1H18 RE9"}="H181 RE9"; $resn_exch{"2H18 RE9"}="H182 RE9";
$resn_exch{"1H19 RE9"}="H191 RE9"; $resn_exch{"2H19 RE9"}="H192 RE9"; $resn_exch{"3H19 RE9"}="H193 RE9";
$resn_exch{"1H20 RE9"}="H201 RE9"; $resn_exch{"2H20 RE9"}="H202 RE9"; $resn_exch{"3H20 RE9"}="H203 RE9";
$resn_exch{"1H2  REA"}=" H21 REA"; $resn_exch{"2H2  REA"}=" H22 REA"; $resn_exch{"1H3  REA"}=" H31 REA";
$resn_exch{"2H3  REA"}=" H32 REA"; $resn_exch{"1H4  REA"}=" H41 REA"; $resn_exch{"2H4  REA"}=" H42 REA";
$resn_exch{"1H16 REA"}="H161 REA"; $resn_exch{"2H16 REA"}="H162 REA"; $resn_exch{"3H16 REA"}="H163 REA";
$resn_exch{"1H17 REA"}="H171 REA"; $resn_exch{"2H17 REA"}="H172 REA"; $resn_exch{"3H17 REA"}="H173 REA";
$resn_exch{"1H18 REA"}="H181 REA"; $resn_exch{"2H18 REA"}="H182 REA"; $resn_exch{"3H18 REA"}="H183 REA";
$resn_exch{"1H19 REA"}="H191 REA"; $resn_exch{"2H19 REA"}="H192 REA"; $resn_exch{"3H19 REA"}="H193 REA";
$resn_exch{"1H20 REA"}="H201 REA"; $resn_exch{"2H20 REA"}="H202 REA"; $resn_exch{"3H20 REA"}="H203 REA";
$resn_exch{"2H2  RED"}=" H22 RED"; $resn_exch{"1H2  RED"}=" H21 RED"; $resn_exch{"2H3  RED"}=" H32 RED";
$resn_exch{"1H3  RED"}=" H31 RED"; $resn_exch{"2H4  RED"}=" H42 RED"; $resn_exch{"1H4  RED"}=" H41 RED";
$resn_exch{"2H5  RED"}=" H52 RED"; $resn_exch{"1H5  RED"}=" H51 RED"; $resn_exch{"2H7  RED"}=" H72 RED";
$resn_exch{"1H7  RED"}=" H71 RED"; $resn_exch{"2H8  RED"}=" H82 RED"; $resn_exch{"1H8  RED"}=" H81 RED";
$resn_exch{"1H7  REP"}=" H71 REP"; $resn_exch{"1H8  REP"}=" H81 REP"; $resn_exch{"1H9  REP"}=" H91 REP";
$resn_exch{"1H14 REP"}="H141 REP"; $resn_exch{"1H15 REP"}="H151 REP"; $resn_exch{"1H16 REP"}="H161 REP";
$resn_exch{"1H17 REP"}="H171 REP"; $resn_exch{"1H18 REP"}="H181 REP"; $resn_exch{"1H1  REQ"}=" H11 REQ";
$resn_exch{"2H1  REQ"}=" H12 REQ"; $resn_exch{"1H2  REQ"}=" H21 REQ"; $resn_exch{"2H2  REQ"}=" H22 REQ";
$resn_exch{"1H3  REQ"}=" H31 REQ"; $resn_exch{"2H3  REQ"}=" H32 REQ"; $resn_exch{"1H19 REQ"}="H191 REQ";
$resn_exch{"2H19 REQ"}="H192 REQ"; $resn_exch{"3H19 REQ"}="H193 REQ"; $resn_exch{"1H20 REQ"}="H201 REQ";
$resn_exch{"2H20 REQ"}="H202 REQ"; $resn_exch{"3H20 REQ"}="H203 REQ"; $resn_exch{"1H4C RES"}="H4C1 RES";
$resn_exch{"2H4C RES"}="H4C2 RES"; $resn_exch{"1H2  RET"}=" H21 RET"; $resn_exch{"2H2  RET"}=" H22 RET";
$resn_exch{"1H3  RET"}=" H31 RET"; $resn_exch{"2H3  RET"}=" H32 RET"; $resn_exch{"1H4  RET"}=" H41 RET";
$resn_exch{"2H4  RET"}=" H42 RET"; $resn_exch{"1H16 RET"}="H161 RET"; $resn_exch{"2H16 RET"}="H162 RET";
$resn_exch{"3H16 RET"}="H163 RET"; $resn_exch{"1H17 RET"}="H171 RET"; $resn_exch{"2H17 RET"}="H172 RET";
$resn_exch{"3H17 RET"}="H173 RET"; $resn_exch{"1H18 RET"}="H181 RET"; $resn_exch{"2H18 RET"}="H182 RET";
$resn_exch{"3H18 RET"}="H183 RET"; $resn_exch{"1H19 RET"}="H191 RET"; $resn_exch{"2H19 RET"}="H192 RET";
$resn_exch{"3H19 RET"}="H193 RET"; $resn_exch{"1H20 RET"}="H201 RET"; $resn_exch{"2H20 RET"}="H202 RET";
$resn_exch{"3H20 RET"}="H203 RET"; $resn_exch{"1H13 REW"}="H131 REW"; $resn_exch{"2H13 REW"}="H132 REW";
$resn_exch{"1H22 REW"}="H221 REW"; $resn_exch{"2H22 REW"}="H222 REW"; $resn_exch{"1H35 REW"}="H351 REW";
$resn_exch{"2H35 REW"}="H352 REW"; $resn_exch{"3H35 REW"}="H353 REW"; $resn_exch{"1H14 REW"}="H141 REW";
$resn_exch{"2H14 REW"}="H142 REW"; $resn_exch{"1H1  REW"}="H11A REW"; $resn_exch{"2H1  REW"}="H12A REW";
$resn_exch{"3H1  REW"}=" H13 REW"; $resn_exch{"1HC1 REX"}="HC11 REX"; $resn_exch{"2HC1 REX"}="HC12 REX";
$resn_exch{"1HC2 REX"}="HC21 REX"; $resn_exch{"2HC2 REX"}="HC22 REX"; $resn_exch{"1HC3 REX"}="HC31 REX";
$resn_exch{"2HC3 REX"}="HC32 REX"; $resn_exch{"1HC4 REX"}="HC41 REX"; $resn_exch{"2HC4 REX"}="HC42 REX";
$resn_exch{"1HC8 REX"}="HC81 REX"; $resn_exch{"2HC8 REX"}="HC82 REX"; $resn_exch{"3HC8 REX"}="HC83 REX";
$resn_exch{"1H11 REX"}="H111 REX"; $resn_exch{"2H11 REX"}="H112 REX"; $resn_exch{"3H11 REX"}="H113 REX";
$resn_exch{"1H15 REX"}="H151 REX"; $resn_exch{"2H15 REX"}="H152 REX"; $resn_exch{"1HN4 REX"}="HN41 REX";
$resn_exch{"2HN4 REX"}="HN42 REX"; $resn_exch{"3HN4 REX"}="HN43 REX"; $resn_exch{"1HC1 REY"}="HC11 REY";
$resn_exch{"2HC1 REY"}="HC12 REY"; $resn_exch{"1HC2 REY"}="HC21 REY"; $resn_exch{"2HC2 REY"}="HC22 REY";
$resn_exch{"1HC3 REY"}="HC31 REY"; $resn_exch{"2HC3 REY"}="HC32 REY"; $resn_exch{"1HC4 REY"}="HC41 REY";
$resn_exch{"2HC4 REY"}="HC42 REY"; $resn_exch{"1HC8 REY"}="HC81 REY"; $resn_exch{"2HC8 REY"}="HC82 REY";
$resn_exch{"3HC8 REY"}="HC83 REY"; $resn_exch{"1H12 REY"}="H121 REY"; $resn_exch{"2H12 REY"}="H122 REY";
$resn_exch{"1HN4 REY"}="HN41 REY"; $resn_exch{"2HN4 REY"}="HN42 REY"; $resn_exch{"3HN4 REY"}="HN43 REY";
$resn_exch{"1HA1 REZ"}="HA11 REZ"; $resn_exch{"2HA1 REZ"}="HA12 REZ"; $resn_exch{"1HB1 REZ"}="HB11 REZ";
$resn_exch{"2HB1 REZ"}="HB12 REZ"; $resn_exch{"1HG1 REZ"}="HG11 REZ"; $resn_exch{"2HG1 REZ"}="HG12 REZ";
$resn_exch{"1HD1 REZ"}="HD11 REZ"; $resn_exch{"2HD1 REZ"}="HD12 REZ"; $resn_exch{"1HZ1 REZ"}="HZ11 REZ";
$resn_exch{"2HZ1 REZ"}="HZ12 REZ"; $resn_exch{"1HBC REZ"}="HBC1 REZ"; $resn_exch{"2HBC REZ"}="HBC2 REZ";
$resn_exch{"3HBC REZ"}="HBC3 REZ"; $resn_exch{"1H3  RFA"}=" H31 RFA"; $resn_exch{"2H3  RFA"}=" H32 RFA";
$resn_exch{"1H4  RFA"}=" H41 RFA"; $resn_exch{"2H4  RFA"}=" H42 RFA"; $resn_exch{"1H5  RFA"}=" H51 RFA";
$resn_exch{"1H6  RFA"}=" H61 RFA"; $resn_exch{"1H9  RFA"}=" H91 RFA"; $resn_exch{"2H9  RFA"}=" H92 RFA";
$resn_exch{"1H10 RFA"}="H101 RFA"; $resn_exch{"2H10 RFA"}="H102 RFA"; $resn_exch{"1H11 RFA"}="H111 RFA";
$resn_exch{"1H12 RFA"}="H121 RFA"; $resn_exch{"1H15 RFA"}="H151 RFA"; $resn_exch{"2H15 RFA"}="H152 RFA";
$resn_exch{"1H16 RFA"}="H161 RFA"; $resn_exch{"2H16 RFA"}="H162 RFA"; $resn_exch{"1H18 RFA"}="H181 RFA";
$resn_exch{"19H1 RFA"}="H119 RFA"; $resn_exch{"20H2 RFA"}="H220 RFA"; $resn_exch{"1H21 RFA"}="H211 RFA";
$resn_exch{"2H21 RFA"}="H212 RFA"; $resn_exch{"22H2 RFA"}="H222 RFA"; $resn_exch{"1H23 RFA"}="H231 RFA";
$resn_exch{"2H23 RFA"}="H232 RFA"; $resn_exch{"1H24 RFA"}="H241 RFA"; $resn_exch{"2H24 RFA"}="H242 RFA";
$resn_exch{"1H27 RFA"}="H271 RFA"; $resn_exch{"2H27 RFA"}="H272 RFA"; $resn_exch{"1H28 RFA"}="H281 RFA";
$resn_exch{"2H28 RFA"}="H282 RFA"; $resn_exch{"1H29 RFA"}="H291 RFA"; $resn_exch{"2H29 RFA"}="H292 RFA";
$resn_exch{"1H30 RFA"}="H301 RFA"; $resn_exch{"2H30 RFA"}="H302 RFA"; $resn_exch{"1H33 RFA"}="H331 RFA";
$resn_exch{"2H33 RFA"}="H332 RFA"; $resn_exch{"1H34 RFA"}="H341 RFA"; $resn_exch{"2H34 RFA"}="H342 RFA";
$resn_exch{"1H35 RFA"}="H351 RFA"; $resn_exch{"2H35 RFA"}="H352 RFA"; $resn_exch{"1H36 RFA"}="H361 RFA";
$resn_exch{"2H36 RFA"}="H362 RFA"; $resn_exch{"1H38 RFA"}="H381 RFA"; $resn_exch{"2H38 RFA"}="H382 RFA";
$resn_exch{"3H38 RFA"}="H383 RFA"; $resn_exch{"1H39 RFA"}="H391 RFA"; $resn_exch{"2H39 RFA"}="H392 RFA";
$resn_exch{"40H4 RFA"}="H440 RFA"; $resn_exch{"41H4 RFA"}="H441 RFA"; $resn_exch{"42H4 RFA"}="H442 RFA";
$resn_exch{"43H4 RFA"}="H443 RFA"; $resn_exch{"44H4 RFA"}="H444 RFA"; $resn_exch{"45H4 RFA"}="H445 RFA";
$resn_exch{"50H5 RFA"}="H550 RFA"; $resn_exch{"51H5 RFA"}="H551 RFA"; $resn_exch{"52H5 RFA"}="H552 RFA";
$resn_exch{"53H5 RFA"}="H553 RFA"; $resn_exch{"54H5 RFA"}="H554 RFA"; $resn_exch{"55H5 RFA"}="H555 RFA";
$resn_exch{"1H60 RFA"}="H601 RFA"; $resn_exch{"1H61 RFA"}="H611 RFA"; $resn_exch{"1H62 RFA"}="H621 RFA";
$resn_exch{"2H62 RFA"}="H622 RFA"; $resn_exch{"1H63 RFA"}="H631 RFA"; $resn_exch{"1H64 RFA"}="H641 RFA";
$resn_exch{"2H64 RFA"}="H642 RFA"; $resn_exch{"1H65 RFA"}="H651 RFA"; $resn_exch{"1H66 RFA"}="H661 RFA";
$resn_exch{"1H67 RFA"}="H671 RFA"; $resn_exch{"2H67 RFA"}="H672 RFA"; $resn_exch{"1H68 RFA"}="H681 RFA";
$resn_exch{"2H68 RFA"}="H682 RFA"; $resn_exch{"1H69 RFA"}="H691 RFA"; $resn_exch{"1H70 RFA"}="H701 RFA";
$resn_exch{"2H70 RFA"}="H702 RFA"; $resn_exch{"1H3  RFB"}=" H31 RFB"; $resn_exch{"2H3  RFB"}=" H32 RFB";
$resn_exch{"1H4  RFB"}=" H41 RFB"; $resn_exch{"2H4  RFB"}=" H42 RFB"; $resn_exch{"1H5  RFB"}=" H51 RFB";
$resn_exch{"1H6  RFB"}=" H61 RFB"; $resn_exch{"1H9  RFB"}=" H91 RFB"; $resn_exch{"2H9  RFB"}=" H92 RFB";
$resn_exch{"1H10 RFB"}="H101 RFB"; $resn_exch{"2H10 RFB"}="H102 RFB"; $resn_exch{"1H11 RFB"}="H111 RFB";
$resn_exch{"1H12 RFB"}="H121 RFB"; $resn_exch{"1H15 RFB"}="H151 RFB"; $resn_exch{"2H15 RFB"}="H152 RFB";
$resn_exch{"1H16 RFB"}="H161 RFB"; $resn_exch{"2H16 RFB"}="H162 RFB"; $resn_exch{"1H18 RFB"}="H181 RFB";
$resn_exch{"19H1 RFB"}="H119 RFB"; $resn_exch{"20H2 RFB"}="H220 RFB"; $resn_exch{"1H21 RFB"}="H211 RFB";
$resn_exch{"2H21 RFB"}="H212 RFB"; $resn_exch{"22H2 RFB"}="H222 RFB"; $resn_exch{"1H23 RFB"}="H231 RFB";
$resn_exch{"2H23 RFB"}="H232 RFB"; $resn_exch{"1H24 RFB"}="H241 RFB"; $resn_exch{"2H24 RFB"}="H242 RFB";
$resn_exch{"1H27 RFB"}="H271 RFB"; $resn_exch{"2H27 RFB"}="H272 RFB"; $resn_exch{"1H28 RFB"}="H281 RFB";
$resn_exch{"2H28 RFB"}="H282 RFB"; $resn_exch{"1H29 RFB"}="H291 RFB"; $resn_exch{"2H29 RFB"}="H292 RFB";
$resn_exch{"1H30 RFB"}="H301 RFB"; $resn_exch{"2H30 RFB"}="H302 RFB"; $resn_exch{"1H33 RFB"}="H331 RFB";
$resn_exch{"2H33 RFB"}="H332 RFB"; $resn_exch{"1H34 RFB"}="H341 RFB"; $resn_exch{"2H34 RFB"}="H342 RFB";
$resn_exch{"1H35 RFB"}="H351 RFB"; $resn_exch{"2H35 RFB"}="H352 RFB"; $resn_exch{"1H36 RFB"}="H361 RFB";
$resn_exch{"2H36 RFB"}="H362 RFB"; $resn_exch{"1H38 RFB"}="H381 RFB"; $resn_exch{"2H38 RFB"}="H382 RFB";
$resn_exch{"3H38 RFB"}="H383 RFB"; $resn_exch{"1H39 RFB"}="H391 RFB"; $resn_exch{"2H39 RFB"}="H392 RFB";
$resn_exch{"40H4 RFB"}="H440 RFB"; $resn_exch{"41H4 RFB"}="H441 RFB"; $resn_exch{"42H4 RFB"}="H442 RFB";
$resn_exch{"43H4 RFB"}="H443 RFB"; $resn_exch{"44H4 RFB"}="H444 RFB"; $resn_exch{"45H4 RFB"}="H445 RFB";
$resn_exch{"50H5 RFB"}="H550 RFB"; $resn_exch{"51H5 RFB"}="H551 RFB"; $resn_exch{"52H5 RFB"}="H552 RFB";
$resn_exch{"53H5 RFB"}="H553 RFB"; $resn_exch{"54H5 RFB"}="H554 RFB"; $resn_exch{"55H5 RFB"}="H555 RFB";
$resn_exch{"1H60 RFB"}="H601 RFB"; $resn_exch{"1H61 RFB"}="H611 RFB"; $resn_exch{"1H62 RFB"}="H621 RFB";
$resn_exch{"2H62 RFB"}="H622 RFB"; $resn_exch{"1H63 RFB"}="H631 RFB"; $resn_exch{"1H64 RFB"}="H641 RFB";
$resn_exch{"2H64 RFB"}="H642 RFB"; $resn_exch{"1H65 RFB"}="H651 RFB"; $resn_exch{"1H66 RFB"}="H661 RFB";
$resn_exch{"1H67 RFB"}="H671 RFB"; $resn_exch{"2H67 RFB"}="H672 RFB"; $resn_exch{"1H68 RFB"}="H681 RFB";
$resn_exch{"2H68 RFB"}="H682 RFB"; $resn_exch{"1H69 RFB"}="H691 RFB"; $resn_exch{"1H70 RFB"}="H701 RFB";
$resn_exch{"2H70 RFB"}="H702 RFB"; $resn_exch{" H3* RFC"}=" H3' RFC"; $resn_exch{" H2* RFC"}=" H2' RFC";
$resn_exch{" H1* RFC"}=" H1' RFC"; $resn_exch{"1HN6 RFC"}="HN61 RFC"; $resn_exch{"2HN6 RFC"}="HN62 RFC";
$resn_exch{" H4* RFC"}=" H4' RFC"; $resn_exch{"1H5* RFC"}="H5'1 RFC"; $resn_exch{"2H5* RFC"}="H5'2 RFC";
$resn_exch{"1H4  RFC"}=" H41 RFC"; $resn_exch{"2H4  RFC"}=" H42 RFC"; $resn_exch{"1H1  RFC"}="H11X RFC";
$resn_exch{"2H1  RFC"}=" H12 RFC"; $resn_exch{"3H1  RFC"}=" H13 RFC"; $resn_exch{"1H14 RFC"}="H141 RFC";
$resn_exch{"2H14 RFC"}="H142 RFC"; $resn_exch{"3H14 RFC"}="H143 RFC"; $resn_exch{"1H7  RFC"}=" H71 RFC";
$resn_exch{"2H7  RFC"}=" H72 RFC"; $resn_exch{"1H6  RFC"}=" H61 RFC"; $resn_exch{"2H6  RFC"}=" H62 RFC";
$resn_exch{"1H3  RFC"}=" H31 RFC"; $resn_exch{"2H3  RFC"}=" H32 RFC"; $resn_exch{"1H2  RFC"}=" H21 RFC";
$resn_exch{"2H2  RFC"}=" H22 RFC"; $resn_exch{"1H13 RFC"}="H131 RFC"; $resn_exch{"2H13 RFC"}="H132 RFC";
$resn_exch{"3H13 RFC"}="H133 RFC"; $resn_exch{"1H9  RFC"}=" H91 RFC"; $resn_exch{"2H9  RFC"}=" H92 RFC";
$resn_exch{"1H12 RFC"}="H121 RFC"; $resn_exch{"2H12 RFC"}="H122 RFC"; $resn_exch{"3H12 RFC"}="H123 RFC";
$resn_exch{"1H11 RFC"}="H111 RFC"; $resn_exch{"2H11 RFC"}="H112 RFC"; $resn_exch{"3H11 RFC"}="H113 RFC";
$resn_exch{"1H5' RFL"}="H5'1 RFL"; $resn_exch{"2H5' RFL"}="H5'2 RFL"; $resn_exch{"1HC2 RFL"}="HC21 RFL";
$resn_exch{"1HN6 RFL"}="HN61 RFL"; $resn_exch{"2HN6 RFL"}="HN62 RFL"; $resn_exch{"1HC8 RFL"}="HC81 RFL";
$resn_exch{"1H5R RFL"}="H5R1 RFL"; $resn_exch{"2H5R RFL"}="H5R2 RFL"; $resn_exch{"1H1R RFL"}="H1R1 RFL";
$resn_exch{"2H1R RFL"}="H1R2 RFL"; $resn_exch{"1HM1 RFL"}="HM11 RFL"; $resn_exch{"2HM1 RFL"}="HM12 RFL";
$resn_exch{"3HM1 RFL"}="HM13 RFL"; $resn_exch{"1HM2 RFL"}="HM21 RFL"; $resn_exch{"2HM2 RFL"}="HM22 RFL";
$resn_exch{"3HM2 RFL"}="HM23 RFL"; $resn_exch{"1H7M RFL"}="H7M1 RFL"; $resn_exch{"2H7M RFL"}="H7M2 RFL";
$resn_exch{"3H7M RFL"}="H7M3 RFL"; $resn_exch{"1H13 RFP"}="H131 RFP"; $resn_exch{"2H13 RFP"}="H132 RFP";
$resn_exch{"3H13 RFP"}="H133 RFP"; $resn_exch{"1H14 RFP"}="H141 RFP"; $resn_exch{"2H14 RFP"}="H142 RFP";
$resn_exch{"3H14 RFP"}="H143 RFP"; $resn_exch{"CH17 RFP"}="H17C RFP"; $resn_exch{"CH18 RFP"}="H18C RFP";
$resn_exch{"CH19 RFP"}="H19C RFP"; $resn_exch{"CH20 RFP"}="H20C RFP"; $resn_exch{"1H21 RFP"}="H211 RFP";
$resn_exch{"CH22 RFP"}="H22C RFP"; $resn_exch{"CH23 RFP"}="H23C RFP"; $resn_exch{"CH24 RFP"}="H24C RFP";
$resn_exch{"CH25 RFP"}="H25C RFP"; $resn_exch{"CH26 RFP"}="H26C RFP"; $resn_exch{"CH27 RFP"}="H27C RFP";
$resn_exch{"CH28 RFP"}="H28C RFP"; $resn_exch{"CH29 RFP"}="H29C RFP"; $resn_exch{"1H30 RFP"}="H301 RFP";
$resn_exch{"2H30 RFP"}="H302 RFP"; $resn_exch{"3H30 RFP"}="H303 RFP"; $resn_exch{"1H31 RFP"}="H311 RFP";
$resn_exch{"2H31 RFP"}="H312 RFP"; $resn_exch{"3H31 RFP"}="H313 RFP"; $resn_exch{"1H32 RFP"}="H321 RFP";
$resn_exch{"2H32 RFP"}="H322 RFP"; $resn_exch{"3H32 RFP"}="H323 RFP"; $resn_exch{"1H33 RFP"}="H331 RFP";
$resn_exch{"2H33 RFP"}="H332 RFP"; $resn_exch{"3H33 RFP"}="H333 RFP"; $resn_exch{"1H34 RFP"}="H341 RFP";
$resn_exch{"2H34 RFP"}="H342 RFP"; $resn_exch{"3H34 RFP"}="H343 RFP"; $resn_exch{"1H36 RFP"}="H361 RFP";
$resn_exch{"2H36 RFP"}="H362 RFP"; $resn_exch{"3H36 RFP"}="H363 RFP"; $resn_exch{"1H37 RFP"}="H371 RFP";
$resn_exch{"2H37 RFP"}="H372 RFP"; $resn_exch{"3H37 RFP"}="H373 RFP"; $resn_exch{"1H38 RFP"}="H381 RFP";
$resn_exch{"2H38 RFP"}="H382 RFP"; $resn_exch{"3H38 RFP"}="H383 RFP"; $resn_exch{"1H39 RFP"}="H391 RFP";
$resn_exch{"2H39 RFP"}="H392 RFP"; $resn_exch{"1H40 RFP"}="H401 RFP"; $resn_exch{"2H40 RFP"}="H402 RFP";
$resn_exch{"1H41 RFP"}="H411 RFP"; $resn_exch{"2H41 RFP"}="H412 RFP"; $resn_exch{"1H42 RFP"}="H421 RFP";
$resn_exch{"2H42 RFP"}="H422 RFP"; $resn_exch{"OH10 RFP"}="H10O RFP"; $resn_exch{"OH12 RFP"}="H12O RFP";
$resn_exch{"1H6' RG1"}="H6'1 RG1"; $resn_exch{"2H6' RG1"}="H6'2 RG1"; $resn_exch{"'HO2 RG1"}="HO2' RG1";
$resn_exch{"'HO3 RG1"}="HO3' RG1"; $resn_exch{"'HO4 RG1"}="HO4' RG1"; $resn_exch{"'HO6 RG1"}="HO6' RG1";
$resn_exch{"1HM1 RG1"}="HM11 RG1"; $resn_exch{"2HM1 RG1"}="HM12 RG1"; $resn_exch{"3HM1 RG1"}="HM13 RG1";
$resn_exch{"1HM2 RG1"}="HM21 RG1"; $resn_exch{"2HM2 RG1"}="HM22 RG1"; $resn_exch{"3HM2 RG1"}="HM23 RG1";
$resn_exch{"1H2  RG1"}="H21A RG1"; $resn_exch{"2H2  RG1"}=" H22 RG1"; $resn_exch{"1H3  RG1"}=" H31 RG1";
$resn_exch{"2H3  RG1"}=" H32 RG1"; $resn_exch{"1H4  RG1"}=" H41 RG1"; $resn_exch{"2H4  RG1"}=" H42 RG1";
$resn_exch{"1HM3 RG1"}="HM31 RG1"; $resn_exch{"2HM3 RG1"}="HM32 RG1"; $resn_exch{"3HM3 RG1"}="HM33 RG1";
$resn_exch{"1HM4 RG1"}="HM41 RG1"; $resn_exch{"2HM4 RG1"}="HM42 RG1"; $resn_exch{"3HM4 RG1"}="HM43 RG1";
$resn_exch{"1HM5 RG1"}="HM51 RG1"; $resn_exch{"2HM5 RG1"}="HM52 RG1"; $resn_exch{"3HM5 RG1"}="HM53 RG1";
$resn_exch{"1HM6 RG1"}="HM61 RG1"; $resn_exch{"2HM6 RG1"}="HM62 RG1"; $resn_exch{"3HM6 RG1"}="HM63 RG1";
$resn_exch{"1HM7 RG1"}="HM71 RG1"; $resn_exch{"2HM7 RG1"}="HM72 RG1"; $resn_exch{"3HM7 RG1"}="HM73 RG1";
$resn_exch{"1HM8 RG1"}="HM81 RG1"; $resn_exch{"2HM8 RG1"}="HM82 RG1"; $resn_exch{"3HM8 RG1"}="HM83 RG1";
$resn_exch{"1HM9 RG1"}="HM91 RG1"; $resn_exch{"2HM9 RG1"}="HM92 RG1"; $resn_exch{"3HM9 RG1"}="HM93 RG1";
$resn_exch{"1HM0 RG1"}="HM01 RG1"; $resn_exch{"2HM0 RG1"}="HM02 RG1"; $resn_exch{"3HM0 RG1"}="HM03 RG1";
$resn_exch{"1H38 RGA"}="H381 RGA"; $resn_exch{"2H38 RGA"}="H382 RGA"; $resn_exch{"3H38 RGA"}="H383 RGA";
$resn_exch{"1H6  RGA"}=" H61 RGA"; $resn_exch{"2H6  RGA"}=" H62 RGA"; $resn_exch{"1H48 RGA"}="H481 RGA";
$resn_exch{"2H48 RGA"}="H482 RGA"; $resn_exch{"3H48 RGA"}="H483 RGA"; $resn_exch{"1H8  RGA"}=" H81 RGA";
$resn_exch{"2H8  RGA"}=" H82 RGA"; $resn_exch{"1H10 RGA"}="H101 RGA"; $resn_exch{"2H10 RGA"}="H102 RGA";
$resn_exch{"1H41 RGA"}="H411 RGA"; $resn_exch{"2H41 RGA"}="H412 RGA"; $resn_exch{"3H41 RGA"}="H413 RGA";
$resn_exch{"1H49 RGA"}="H491 RGA"; $resn_exch{"2H49 RGA"}="H492 RGA"; $resn_exch{"3H49 RGA"}="H493 RGA";
$resn_exch{"1H14 RGA"}="H141 RGA"; $resn_exch{"2H14 RGA"}="H142 RGA"; $resn_exch{"1H50 RGA"}="H501 RGA";
$resn_exch{"2H50 RGA"}="H502 RGA"; $resn_exch{"3H50 RGA"}="H503 RGA"; $resn_exch{"1H42 RGA"}="H421 RGA";
$resn_exch{"2H42 RGA"}="H422 RGA"; $resn_exch{"3H42 RGA"}="H423 RGA"; $resn_exch{"1H51 RGA"}="H511 RGA";
$resn_exch{"2H51 RGA"}="H512 RGA"; $resn_exch{"3H51 RGA"}="H513 RGA"; $resn_exch{"1H20 RGA"}="H201 RGA";
$resn_exch{"2H20 RGA"}="H202 RGA"; $resn_exch{"1H52 RGA"}="H521 RGA"; $resn_exch{"2H52 RGA"}="H522 RGA";
$resn_exch{"3H52 RGA"}="H523 RGA"; $resn_exch{"1H43 RGA"}="H431 RGA"; $resn_exch{"2H43 RGA"}="H432 RGA";
$resn_exch{"3H43 RGA"}="H433 RGA"; $resn_exch{"1H44 RGA"}="H441 RGA"; $resn_exch{"2H44 RGA"}="H442 RGA";
$resn_exch{"3H44 RGA"}="H443 RGA"; $resn_exch{"1H53 RGA"}="H531 RGA"; $resn_exch{"2H53 RGA"}="H532 RGA";
$resn_exch{"3H53 RGA"}="H533 RGA"; $resn_exch{"1H45 RGA"}="H451 RGA"; $resn_exch{"2H45 RGA"}="H452 RGA";
$resn_exch{"3H45 RGA"}="H453 RGA"; $resn_exch{"1H29 RGA"}="H291 RGA"; $resn_exch{"2H29 RGA"}="H292 RGA";
$resn_exch{"1H30 RGA"}="H301 RGA"; $resn_exch{"2H30 RGA"}="H302 RGA"; $resn_exch{"1H46 RGA"}="H461 RGA";
$resn_exch{"2H46 RGA"}="H462 RGA"; $resn_exch{"3H46 RGA"}="H463 RGA"; $resn_exch{"1H54 RGA"}="H541 RGA";
$resn_exch{"2H54 RGA"}="H542 RGA"; $resn_exch{"3H54 RGA"}="H543 RGA"; $resn_exch{"1H34 RGA"}="H341 RGA";
$resn_exch{"2H34 RGA"}="H342 RGA"; $resn_exch{"1H47 RGA"}="H471 RGA"; $resn_exch{"2H47 RGA"}="H472 RGA";
$resn_exch{"3H47 RGA"}="H473 RGA"; $resn_exch{"1H38 RGC"}="H381 RGC"; $resn_exch{"2H38 RGC"}="H382 RGC";
$resn_exch{"3H38 RGC"}="H383 RGC"; $resn_exch{"1H6  RGC"}=" H61 RGC"; $resn_exch{"2H6  RGC"}=" H62 RGC";
$resn_exch{"1H48 RGC"}="H481 RGC"; $resn_exch{"2H48 RGC"}="H482 RGC"; $resn_exch{"3H48 RGC"}="H483 RGC";
$resn_exch{"1H8  RGC"}=" H81 RGC"; $resn_exch{"2H8  RGC"}=" H82 RGC"; $resn_exch{"1H10 RGC"}="H101 RGC";
$resn_exch{"2H10 RGC"}="H102 RGC"; $resn_exch{"1H41 RGC"}="H411 RGC"; $resn_exch{"2H41 RGC"}="H412 RGC";
$resn_exch{"3H41 RGC"}="H413 RGC"; $resn_exch{"1H49 RGC"}="H491 RGC"; $resn_exch{"2H49 RGC"}="H492 RGC";
$resn_exch{"3H49 RGC"}="H493 RGC"; $resn_exch{"1H14 RGC"}="H141 RGC"; $resn_exch{"2H14 RGC"}="H142 RGC";
$resn_exch{"1H50 RGC"}="H501 RGC"; $resn_exch{"2H50 RGC"}="H502 RGC"; $resn_exch{"3H50 RGC"}="H503 RGC";
$resn_exch{"1H42 RGC"}="H421 RGC"; $resn_exch{"2H42 RGC"}="H422 RGC"; $resn_exch{"3H42 RGC"}="H423 RGC";
$resn_exch{"1H51 RGC"}="H511 RGC"; $resn_exch{"2H51 RGC"}="H512 RGC"; $resn_exch{"3H51 RGC"}="H513 RGC";
$resn_exch{"1H20 RGC"}="H201 RGC"; $resn_exch{"2H20 RGC"}="H202 RGC"; $resn_exch{"1H52 RGC"}="H521 RGC";
$resn_exch{"2H52 RGC"}="H522 RGC"; $resn_exch{"3H52 RGC"}="H523 RGC"; $resn_exch{"1H43 RGC"}="H431 RGC";
$resn_exch{"2H43 RGC"}="H432 RGC"; $resn_exch{"3H43 RGC"}="H433 RGC"; $resn_exch{"1H44 RGC"}="H441 RGC";
$resn_exch{"2H44 RGC"}="H442 RGC"; $resn_exch{"3H44 RGC"}="H443 RGC"; $resn_exch{"1H53 RGC"}="H531 RGC";
$resn_exch{"2H53 RGC"}="H532 RGC"; $resn_exch{"3H53 RGC"}="H533 RGC"; $resn_exch{"1H45 RGC"}="H451 RGC";
$resn_exch{"2H45 RGC"}="H452 RGC"; $resn_exch{"3H45 RGC"}="H453 RGC"; $resn_exch{"1H29 RGC"}="H291 RGC";
$resn_exch{"2H29 RGC"}="H292 RGC"; $resn_exch{"1H30 RGC"}="H301 RGC"; $resn_exch{"2H30 RGC"}="H302 RGC";
$resn_exch{"1H46 RGC"}="H461 RGC"; $resn_exch{"2H46 RGC"}="H462 RGC"; $resn_exch{"3H46 RGC"}="H463 RGC";
$resn_exch{"1H54 RGC"}="H541 RGC"; $resn_exch{"2H54 RGC"}="H542 RGC"; $resn_exch{"3H54 RGC"}="H543 RGC";
$resn_exch{"1H34 RGC"}="H341 RGC"; $resn_exch{"2H34 RGC"}="H342 RGC"; $resn_exch{"1H35 RGC"}="H351 RGC";
$resn_exch{"2H35 RGC"}="H352 RGC"; $resn_exch{"1H1C RGI"}="H1C1 RGI"; $resn_exch{"2HA  RGI"}=" HA2 RGI";
$resn_exch{"3HB  RGI"}=" HB3 RGI"; $resn_exch{"1HG  RGP"}=" HG1 RGP"; $resn_exch{"2HG  RGP"}=" HG2 RGP";
$resn_exch{"1HB  RGP"}=" HB1 RGP"; $resn_exch{"2HB  RGP"}=" HB2 RGP"; $resn_exch{"1HN  RGP"}=" HN1 RGP";
$resn_exch{"2HN  RGP"}=" HN2 RGP"; $resn_exch{"1HOB RGS"}="HOB1 RGS"; $resn_exch{"1HN2 RGS"}="HN21 RGS";
$resn_exch{"2HN2 RGS"}="HN22 RGS"; $resn_exch{"1H3  RGS"}=" H31 RGS"; $resn_exch{"2H3  RGS"}=" H32 RGS";
$resn_exch{"1H4  RGS"}=" H41 RGS"; $resn_exch{"2H4  RGS"}=" H42 RGS"; $resn_exch{"1H6  RGS"}=" H61 RGS";
$resn_exch{"2H6  RGS"}=" H62 RGS"; $resn_exch{"7HOB RGS"}="HOB7 RGS"; $resn_exch{"1HB  RGS"}=" HB1 RGS";
$resn_exch{"2HB  RGS"}=" HB2 RGS"; $resn_exch{"1HB* RGS"}="HB'1 RGS"; $resn_exch{"2HB* RGS"}="HB'2 RGS";
$resn_exch{" HA* RGS"}=" HA' RGS"; $resn_exch{" HN* RGS"}=" HN' RGS"; $resn_exch{"1H6* RGS"}="H6'1 RGS";
$resn_exch{"2H6* RGS"}="H6'2 RGS"; $resn_exch{"*HO7 RGS"}="HO7' RGS"; $resn_exch{"*HN4 RGS"}="HN4' RGS";
$resn_exch{"1H4* RGS"}="H4'1 RGS"; $resn_exch{"2H4* RGS"}="H4'2 RGS"; $resn_exch{"1H3* RGS"}="H3'1 RGS";
$resn_exch{"2H3* RGS"}="H3'2 RGS"; $resn_exch{" H2* RGS"}=" H2' RGS"; $resn_exch{"1H2* RGS"}="H2'1 RGS";
$resn_exch{"2H2* RGS"}="H2'2 RGS"; $resn_exch{"*HO1 RGS"}="HO1' RGS"; $resn_exch{"1HC7 RH1"}="HC71 RH1";
$resn_exch{"2HC7 RH1"}="HC72 RH1"; $resn_exch{"1HC8 RH1"}="HC81 RH1"; $resn_exch{"2HC8 RH1"}="HC82 RH1";
$resn_exch{"1HC9 RH1"}="HC91 RH1"; $resn_exch{"2HC9 RH1"}="HC92 RH1"; $resn_exch{"1H10 RH1"}="H101 RH1";
$resn_exch{"2H10 RH1"}="H102 RH1"; $resn_exch{"1H11 RH1"}="H111 RH1"; $resn_exch{"2H11 RH1"}="H112 RH1";
$resn_exch{"3HO  RH1"}=" HO3 RH1"; $resn_exch{"1H12 RH1"}="H121 RH1"; $resn_exch{"2H12 RH1"}="H122 RH1";
$resn_exch{"3H12 RH1"}="H123 RH1"; $resn_exch{"1H6  RHA"}=" H61 RHA"; $resn_exch{"2H6  RHA"}=" H62 RHA";
$resn_exch{"3H6  RHA"}=" H63 RHA"; $resn_exch{"2HOP RHA"}="HOP2 RHA"; $resn_exch{"3HOP RHA"}="HOP3 RHA";
$resn_exch{"1HC1 RHC"}="HC11 RHC"; $resn_exch{"1HO1 RHC"}="HO11 RHC"; $resn_exch{"1HC2 RHC"}="HC21 RHC";
$resn_exch{"1HO2 RHC"}="HO21 RHC"; $resn_exch{"1HC3 RHC"}="HC31 RHC"; $resn_exch{"1HO3 RHC"}="HO31 RHC";
$resn_exch{"1HC4 RHC"}="HC41 RHC"; $resn_exch{"1HCE RHC"}="HCE1 RHC"; $resn_exch{"2HCE RHC"}="HCE2 RHC";
$resn_exch{"1HN  RHC"}=" HN1 RHC"; $resn_exch{"2HN  RHC"}=" HN2 RHC"; $resn_exch{"1HCA RHC"}="HCA1 RHC";
$resn_exch{"1HCB RHC"}="HCB1 RHC"; $resn_exch{"2HCB RHC"}="HCB2 RHC"; $resn_exch{"1HCG RHC"}="HCG1 RHC";
$resn_exch{"2HCG RHC"}="HCG2 RHC"; $resn_exch{"1HOX RHC"}="HOX1 RHC"; $resn_exch{"1HN1 RHD"}="HN11 RHD";
$resn_exch{"2HN1 RHD"}="HN12 RHD"; $resn_exch{"3HN1 RHD"}="HN13 RHD"; $resn_exch{"1HN2 RHD"}="HN21 RHD";
$resn_exch{"2HN2 RHD"}="HN22 RHD"; $resn_exch{"3HN2 RHD"}="HN23 RHD"; $resn_exch{"1HN3 RHD"}="HN31 RHD";
$resn_exch{"2HN3 RHD"}="HN32 RHD"; $resn_exch{"3HN3 RHD"}="HN33 RHD"; $resn_exch{"1HN4 RHD"}="HN41 RHD";
$resn_exch{"2HN4 RHD"}="HN42 RHD"; $resn_exch{"3HN4 RHD"}="HN43 RHD"; $resn_exch{"1HN5 RHD"}="HN51 RHD";
$resn_exch{"2HN5 RHD"}="HN52 RHD"; $resn_exch{"3HN5 RHD"}="HN53 RHD"; $resn_exch{"1HN6 RHD"}="HN61 RHD";
$resn_exch{"2HN6 RHD"}="HN62 RHD"; $resn_exch{"3HN6 RHD"}="HN63 RHD"; $resn_exch{"1H2  RHM"}=" H21 RHM";
$resn_exch{"2H2  RHM"}=" H22 RHM"; $resn_exch{"3H2  RHM"}=" H23 RHM"; $resn_exch{"1H11 RHM"}="H111 RHM";
$resn_exch{"2H11 RHM"}="H112 RHM"; $resn_exch{"3H11 RHM"}="H113 RHM"; $resn_exch{"1H9  RHM"}=" H91 RHM";
$resn_exch{"2H9  RHM"}=" H92 RHM"; $resn_exch{"1H4  RHM"}=" H41 RHM"; $resn_exch{"2H4  RHM"}=" H42 RHM";
$resn_exch{"1H6  RHM"}=" H61 RHM"; $resn_exch{"2H6  RHM"}=" H62 RHM"; $resn_exch{"1H7  RHM"}=" H71 RHM";
$resn_exch{"2H7  RHM"}=" H72 RHM"; $resn_exch{"1HC6 RHO"}="HC61 RHO"; $resn_exch{"2HC6 RHO"}="HC62 RHO";
$resn_exch{"3HC6 RHO"}="HC63 RHO"; $resn_exch{"1HC7 RHO"}="HC71 RHO"; $resn_exch{"2HC7 RHO"}="HC72 RHO";
$resn_exch{"3HC7 RHO"}="HC73 RHO"; $resn_exch{"CH10 RHO"}="H10C RHO"; $resn_exch{"CH12 RHO"}="H12C RHO";
$resn_exch{"1H13 RHO"}="H131 RHO"; $resn_exch{"2H13 RHO"}="H132 RHO"; $resn_exch{"1H15 RHO"}="H151 RHO";
$resn_exch{"2H15 RHO"}="H152 RHO"; $resn_exch{"3H15 RHO"}="H153 RHO"; $resn_exch{"1H16 RHO"}="H161 RHO";
$resn_exch{"2H16 RHO"}="H162 RHO"; $resn_exch{"3H16 RHO"}="H163 RHO"; $resn_exch{"CH19 RHO"}="H19C RHO";
$resn_exch{"CH21 RHO"}="H21C RHO"; $resn_exch{"CH22 RHO"}="H22C RHO"; $resn_exch{"CH25 RHO"}="H25C RHO";
$resn_exch{"CH26 RHO"}="H26C RHO"; $resn_exch{"1H11 RHP"}="H111 RHP"; $resn_exch{"2H11 RHP"}="H112 RHP";
$resn_exch{"1H1  RHP"}=" H11 RHP"; $resn_exch{"2H1  RHP"}=" H12 RHP"; $resn_exch{"1H2  RHP"}=" H21 RHP";
$resn_exch{"2H2  RHP"}=" H22 RHP"; $resn_exch{"1H3  RHQ"}=" H31 RHQ"; $resn_exch{"1H6  RHQ"}=" H61 RHQ";
$resn_exch{"1H10 RHQ"}="H101 RHQ"; $resn_exch{"1H13 RHQ"}="H131 RHQ"; $resn_exch{"1HN1 RHQ"}="HN11 RHQ";
$resn_exch{"1H15 RHQ"}="H151 RHQ"; $resn_exch{"1H16 RHQ"}="H161 RHQ"; $resn_exch{"1H17 RHQ"}="H171 RHQ";
$resn_exch{"1H18 RHQ"}="H181 RHQ"; $resn_exch{"1H20 RHQ"}="H201 RHQ"; $resn_exch{"2H20 RHQ"}="H202 RHQ";
$resn_exch{"3H20 RHQ"}="H203 RHQ"; $resn_exch{"1H21 RHQ"}="H211 RHQ"; $resn_exch{"2H21 RHQ"}="H212 RHQ";
$resn_exch{"3H21 RHQ"}="H213 RHQ"; $resn_exch{"1H22 RHQ"}="H221 RHQ"; $resn_exch{"2H22 RHQ"}="H222 RHQ";
$resn_exch{"1H23 RHQ"}="H231 RHQ"; $resn_exch{"2H23 RHQ"}="H232 RHQ"; $resn_exch{"3H23 RHQ"}="H233 RHQ";
$resn_exch{"1HN2 RHQ"}="HN21 RHQ"; $resn_exch{"1H24 RHQ"}="H241 RHQ"; $resn_exch{"2H24 RHQ"}="H242 RHQ";
$resn_exch{"1H25 RHQ"}="H251 RHQ"; $resn_exch{"2H25 RHQ"}="H252 RHQ"; $resn_exch{"3H25 RHQ"}="H253 RHQ";
$resn_exch{"1H28 RHQ"}="H281 RHQ"; $resn_exch{"2H28 RHQ"}="H282 RHQ"; $resn_exch{"1H29 RHQ"}="H291 RHQ";
$resn_exch{"2H29 RHQ"}="H292 RHQ"; $resn_exch{"3H29 RHQ"}="H293 RHQ"; $resn_exch{"1HN' RHS"}="HN'1 RHS";
$resn_exch{"2HN' RHS"}="HN'2 RHS"; $resn_exch{"1HT2 RHS"}="HT21 RHS"; $resn_exch{"2HT2 RHS"}="HT22 RHS";
$resn_exch{"3HT2 RHS"}="HT23 RHS"; $resn_exch{"1HT3 RHS"}="HT31 RHS"; $resn_exch{"2HT3 RHS"}="HT32 RHS";
$resn_exch{"3HT3 RHS"}="HT33 RHS"; $resn_exch{"1HT4 RHS"}="HT41 RHS"; $resn_exch{"2HT4 RHS"}="HT42 RHS";
$resn_exch{"3HT4 RHS"}="HT43 RHS"; $resn_exch{"1H1' RHS"}="H1'1 RHS"; $resn_exch{"2H1' RHS"}="H1'2 RHS";
$resn_exch{"1H4  RHS"}=" H41 RHS"; $resn_exch{"2H4  RHS"}=" H42 RHS"; $resn_exch{"1H5  RHS"}=" H51 RHS";
$resn_exch{"2H5  RHS"}=" H52 RHS"; $resn_exch{"3H5  RHS"}=" H53 RHS"; $resn_exch{"1H2  RHS"}=" H21 RHS";
$resn_exch{"2H2  RHS"}=" H22 RHS"; $resn_exch{"1H1  RHS"}=" H11 RHS"; $resn_exch{"2H1  RHS"}=" H12 RHS";
$resn_exch{"3H1  RHS"}=" H13 RHS"; $resn_exch{"1H12 RHX"}="H121 RHX"; $resn_exch{"2H12 RHX"}="H122 RHX";
$resn_exch{"1H8  RHX"}=" H81 RHX"; $resn_exch{"2H8  RHX"}=" H82 RHX"; $resn_exch{"2HOP RIA"}="HOP2 RIA";
$resn_exch{"3HOP RIA"}="HOP3 RIA"; $resn_exch{"1H5* RIA"}=" H5' RIA"; $resn_exch{"2H5* RIA"}="H5'' RIA";
$resn_exch{" H4* RIA"}=" H4A RIA"; $resn_exch{" H3* RIA"}=" H3A RIA"; $resn_exch{"*HO3 RIA"}="HO3A RIA";
$resn_exch{" H2* RIA"}=" H2A RIA"; $resn_exch{" H1* RIA"}=" H1A RIA"; $resn_exch{"1HN6 RIA"}="HN61 RIA";
$resn_exch{"2HN6 RIA"}="HN62 RIA"; $resn_exch{"'HO2 RIA"}="HO2' RIA"; $resn_exch{"'HO3 RIA"}="HO3' RIA";
$resn_exch{"1H5' RIA"}="H5'1 RIA"; $resn_exch{"2H5' RIA"}="H5'2 RIA"; $resn_exch{"2HO' RIA"}="HO'2 RIA";
$resn_exch{"3HO' RIA"}="HO'3 RIA"; $resn_exch{"*HO5 RIB"}="HO5' RIB"; $resn_exch{"1H5* RIB"}="H5'1 RIB";
$resn_exch{"2H5* RIB"}="H5'2 RIB"; $resn_exch{" H4* RIB"}=" H4' RIB"; $resn_exch{" H3* RIB"}=" H3' RIB";
$resn_exch{"*HO3 RIB"}="HO3' RIB"; $resn_exch{" H2* RIB"}=" H2' RIB"; $resn_exch{"*HO2 RIB"}="HO2' RIB";
$resn_exch{" H1* RIB"}=" H1' RIB"; $resn_exch{"*HO1 RIB"}="HO1' RIB"; $resn_exch{"1H2  RIC"}=" H21 RIC";
$resn_exch{"1H4  RIC"}=" H41 RIC"; $resn_exch{"1H7  RIC"}=" H71 RIC"; $resn_exch{"1H8  RIC"}=" H81 RIC";
$resn_exch{"2H8  RIC"}=" H82 RIC"; $resn_exch{"3H8  RIC"}=" H83 RIC"; $resn_exch{"1H9  RIC"}=" H91 RIC";
$resn_exch{"2H9  RIC"}=" H92 RIC"; $resn_exch{"3H9  RIC"}=" H93 RIC"; $resn_exch{"1HC5 RIC"}="HC51 RIC";
$resn_exch{"2HC5 RIC"}="HC52 RIC"; $resn_exch{"1H1Q RIC"}="H1Q1 RIC"; $resn_exch{"1HC5 RIF"}="HC51 RIF";
$resn_exch{"2HC5 RIF"}="HC52 RIF"; $resn_exch{"3HC5 RIF"}="HC53 RIF"; $resn_exch{"1HC6 RIF"}="HC61 RIF";
$resn_exch{"2HC6 RIF"}="HC62 RIF"; $resn_exch{"3HC6 RIF"}="HC63 RIF"; $resn_exch{"1H24 RIF"}="H241 RIF";
$resn_exch{"2H24 RIF"}="H242 RIF"; $resn_exch{"3H24 RIF"}="H243 RIF"; $resn_exch{"1H26 RIF"}="H261 RIF";
$resn_exch{"2H26 RIF"}="H262 RIF"; $resn_exch{"1H27 RIF"}="H271 RIF"; $resn_exch{"2H27 RIF"}="H272 RIF";
$resn_exch{"1H28 RIF"}="H281 RIF"; $resn_exch{"2H28 RIF"}="H282 RIF"; $resn_exch{"1H29 RIF"}="H291 RIF";
$resn_exch{"2H29 RIF"}="H292 RIF"; $resn_exch{"1H31 RIF"}="H311 RIF"; $resn_exch{"2H31 RIF"}="H312 RIF";
$resn_exch{"3H31 RIF"}="H313 RIF"; $resn_exch{"1H32 RIF"}="H321 RIF"; $resn_exch{"2H32 RIF"}="H322 RIF";
$resn_exch{"3H32 RIF"}="H323 RIF"; $resn_exch{"1H33 RIF"}="H331 RIF"; $resn_exch{"2H33 RIF"}="H332 RIF";
$resn_exch{"3H33 RIF"}="H333 RIF"; $resn_exch{"1H34 RIF"}="H341 RIF"; $resn_exch{"2H34 RIF"}="H342 RIF";
$resn_exch{"3H34 RIF"}="H343 RIF"; $resn_exch{"1H36 RIF"}="H361 RIF"; $resn_exch{"2H36 RIF"}="H362 RIF";
$resn_exch{"1H37 RIF"}="H371 RIF"; $resn_exch{"2H37 RIF"}="H372 RIF"; $resn_exch{"3H37 RIF"}="H373 RIF";
$resn_exch{"OH10 RIF"}="H10O RIF"; $resn_exch{"OH17 RIF"}="H17O RIF"; $resn_exch{"1H45 RIF"}="H451 RIF";
$resn_exch{"2H45 RIF"}="H452 RIF"; $resn_exch{"1H50 RIF"}="H501 RIF"; $resn_exch{"2H50 RIF"}="H502 RIF";
$resn_exch{"3H50 RIF"}="H503 RIF"; $resn_exch{"1H55 RIF"}="H551 RIF"; $resn_exch{"1H1B RIG"}="H1B1 RIG";
$resn_exch{"2H1B RIG"}="H1B2 RIG"; $resn_exch{"3H1B RIG"}="H1B3 RIG"; $resn_exch{"1HDC RIG"}="HDC1 RIG";
$resn_exch{"2HDC RIG"}="HDC2 RIG"; $resn_exch{"1HGC RIG"}="HGC1 RIG"; $resn_exch{"2HGC RIG"}="HGC2 RIG";
$resn_exch{"1HBC RIG"}="HBC1 RIG"; $resn_exch{"2HBC RIG"}="HBC2 RIG"; $resn_exch{"1HAR RIG"}="HAR1 RIG";
$resn_exch{"2HAR RIG"}="HAR2 RIG"; $resn_exch{"1HAQ RIG"}="HAQ1 RIG"; $resn_exch{"2HAQ RIG"}="HAQ2 RIG";
$resn_exch{"1HAP RIG"}="HAP1 RIG"; $resn_exch{"2HAP RIG"}="HAP2 RIG"; $resn_exch{"1HAW RIG"}="HAW1 RIG";
$resn_exch{"2HAW RIG"}="HAW2 RIG"; $resn_exch{"1HBD RIG"}="HBD1 RIG"; $resn_exch{"2HBD RIG"}="HBD2 RIG";
$resn_exch{"1HBQ RIG"}="HBQ1 RIG"; $resn_exch{"2HBQ RIG"}="HBQ2 RIG"; $resn_exch{"1HBR RIG"}="HBR1 RIG";
$resn_exch{"2HBR RIG"}="HBR2 RIG"; $resn_exch{"1HBN RIG"}="HBN1 RIG"; $resn_exch{"2HBN RIG"}="HBN2 RIG";
$resn_exch{"1H1  RIL"}=" H11 RIL"; $resn_exch{"2H1  RIL"}=" H12 RIL"; $resn_exch{"1H5  RIL"}=" H51 RIL";
$resn_exch{"2H5  RIL"}=" H52 RIL"; $resn_exch{"1H7  RIL"}=" H71 RIL"; $resn_exch{"2H7  RIL"}=" H72 RIL";
$resn_exch{"1H10 RIL"}="H101 RIL"; $resn_exch{"2H10 RIL"}="H102 RIL"; $resn_exch{"1H13 RIL"}="H131 RIL";
$resn_exch{"2H13 RIL"}="H132 RIL"; $resn_exch{"1H23 RIL"}="H231 RIL"; $resn_exch{"2H23 RIL"}="H232 RIL";
$resn_exch{"1H27 RIL"}="H271 RIL"; $resn_exch{"2H27 RIL"}="H272 RIL"; $resn_exch{"1H34 RIL"}="H341 RIL";
$resn_exch{"2H34 RIL"}="H342 RIL"; $resn_exch{"3H34 RIL"}="H343 RIL"; $resn_exch{"1H38 RIL"}="H381 RIL";
$resn_exch{"2H38 RIL"}="H382 RIL"; $resn_exch{"3H38 RIL"}="H383 RIL"; $resn_exch{"1HA  RIM"}=" HA1 RIM";
$resn_exch{"2HA  RIM"}=" HA2 RIM"; $resn_exch{"3HA  RIM"}=" HA3 RIM"; $resn_exch{"1HNC RIM"}="HNC1 RIM";
$resn_exch{"2HNC RIM"}="HNC2 RIM"; $resn_exch{"1HE1 RIM"}="HE11 RIM"; $resn_exch{"2HE1 RIM"}="HE12 RIM";
$resn_exch{"1HE2 RIM"}="HE21 RIM"; $resn_exch{"2HE2 RIM"}="HE22 RIM"; $resn_exch{"1HE3 RIM"}="HE31 RIM";
$resn_exch{"2HE3 RIM"}="HE32 RIM"; $resn_exch{"1HG1 RIM"}="HG11 RIM"; $resn_exch{"2HG1 RIM"}="HG12 RIM";
$resn_exch{"1HG2 RIM"}="HG21 RIM"; $resn_exch{"2HG2 RIM"}="HG22 RIM"; $resn_exch{"1HG3 RIM"}="HG31 RIM";
$resn_exch{"2HG3 RIM"}="HG32 RIM"; $resn_exch{"1HB  RIN"}=" HB1 RIN"; $resn_exch{"2HB  RIN"}=" HB2 RIN";
$resn_exch{"1HG  RIN"}=" HG1 RIN"; $resn_exch{"2HG  RIN"}=" HG2 RIN"; $resn_exch{"1HD  RIN"}=" HD1 RIN";
$resn_exch{"2HD  RIN"}=" HD2 RIN"; $resn_exch{"1HE  RIN"}=" HE1 RIN"; $resn_exch{"2HE  RIN"}=" HE2 RIN";
$resn_exch{"1HZ  RIN"}=" HZ1 RIN"; $resn_exch{"2HZ  RIN"}=" HZ2 RIN"; $resn_exch{"1H5  RIN"}=" H51 RIN";
$resn_exch{"2H5  RIN"}=" H52 RIN"; $resn_exch{"1H4  RIN"}=" H41 RIN"; $resn_exch{"2H4  RIN"}=" H42 RIN";
$resn_exch{"1H3  RIN"}=" H31 RIN"; $resn_exch{"2H3  RIN"}=" H32 RIN"; $resn_exch{"1H2  RIN"}=" H21 RIN";
$resn_exch{"2H2  RIN"}=" H22 RIN"; $resn_exch{"1H1  RIN"}=" H11 RIN"; $resn_exch{"2H1  RIN"}=" H12 RIN";
$resn_exch{"1H5  RIP"}=" H51 RIP"; $resn_exch{"2H5  RIP"}=" H52 RIP"; $resn_exch{"1HC7 RIS"}="HC71 RIS";
$resn_exch{"2HC7 RIS"}="HC72 RIS"; $resn_exch{"1H6  RIT"}=" H61 RIT"; $resn_exch{"2H6  RIT"}="H62A RIT";
$resn_exch{"1H14 RIT"}="H141 RIT"; $resn_exch{"2H14 RIT"}="H142 RIT"; $resn_exch{"1H26 RIT"}="H261 RIT";
$resn_exch{"2H26 RIT"}="H262 RIT"; $resn_exch{"1H44 RIT"}="H441 RIT"; $resn_exch{"2H44 RIT"}="H442 RIT";
$resn_exch{"1H64 RIT"}="H641 RIT"; $resn_exch{"2H64 RIT"}="H642 RIT"; $resn_exch{"3H64 RIT"}="H643 RIT";
$resn_exch{"1H68 RIT"}="H681 RIT"; $resn_exch{"2H68 RIT"}="H682 RIT"; $resn_exch{"3H68 RIT"}="H683 RIT";
$resn_exch{"1H75 RIT"}="H751 RIT"; $resn_exch{"2H75 RIT"}="H752 RIT"; $resn_exch{"1H86 RIT"}="H861 RIT";
$resn_exch{"2H86 RIT"}="H862 RIT"; $resn_exch{"3H86 RIT"}="H863 RIT"; $resn_exch{"1H90 RIT"}="H901 RIT";
$resn_exch{"2H90 RIT"}="H902 RIT"; $resn_exch{"3H90 RIT"}="H903 RIT"; $resn_exch{"1H95 RIT"}="H951 RIT";
$resn_exch{"2H95 RIT"}="H952 RIT"; $resn_exch{"3H95 RIT"}="H953 RIT"; $resn_exch{"1H2  RL2"}=" H21 RL2";
$resn_exch{"2H2  RL2"}=" H22 RL2"; $resn_exch{"1H3  RL2"}=" H31 RL2"; $resn_exch{"1H10 RL2"}="H101 RL2";
$resn_exch{"1H12 RL2"}="H121 RL2"; $resn_exch{"2H12 RL2"}="H122 RL2"; $resn_exch{"1H13 RL2"}="H131 RL2";
$resn_exch{"2H13 RL2"}="H132 RL2"; $resn_exch{"1H15 RL2"}="H151 RL2"; $resn_exch{"1H16 RL2"}="H161 RL2";
$resn_exch{"1H17 RL2"}="H171 RL2"; $resn_exch{"1H18 RL2"}="H181 RL2"; $resn_exch{"1H19 RL2"}="H191 RL2";
$resn_exch{"1H21 RL2"}="H211 RL2"; $resn_exch{"2H21 RL2"}="H212 RL2"; $resn_exch{"1H22 RL2"}="H221 RL2";
$resn_exch{"2H22 RL2"}="H222 RL2"; $resn_exch{"1H23 RL2"}="H231 RL2"; $resn_exch{"1H25 RL2"}="H251 RL2";
$resn_exch{"2H25 RL2"}="H252 RL2"; $resn_exch{"3H25 RL2"}="H253 RL2"; $resn_exch{"1H24 RL2"}="H241 RL2";
$resn_exch{"2H24 RL2"}="H242 RL2"; $resn_exch{"3H24 RL2"}="H243 RL2"; $resn_exch{"OH19 RLP"}="H19O RLP";
$resn_exch{"1H17 RLP"}="H171 RLP"; $resn_exch{"2H17 RLP"}="H172 RLP"; $resn_exch{"1H16 RLP"}="H161 RLP";
$resn_exch{"2H16 RLP"}="H162 RLP"; $resn_exch{"1H11 RLP"}="H111 RLP"; $resn_exch{"2H11 RLP"}="H112 RLP";
$resn_exch{"1H12 RLP"}="H121 RLP"; $resn_exch{"OH12 RLP"}="H12O RLP"; $resn_exch{"1H13 RLP"}="H131 RLP";
$resn_exch{"OH13 RLP"}="H13O RLP"; $resn_exch{"1H14 RLP"}="H141 RLP"; $resn_exch{"OH14 RLP"}="H14O RLP";
$resn_exch{"1H15 RLP"}="H151 RLP"; $resn_exch{"2H15 RLP"}="H152 RLP"; $resn_exch{"OH15 RLP"}="H15O RLP";
$resn_exch{"1H10 RM1"}="H101 RM1"; $resn_exch{"2H10 RM1"}="H102 RM1"; $resn_exch{"3H10 RM1"}="H103 RM1";
$resn_exch{"1H2C RM1"}="H2C1 RM1"; $resn_exch{"2H2C RM1"}="H2C2 RM1"; $resn_exch{"1H1C RM1"}="H1C1 RM1";
$resn_exch{"2H1C RM1"}="H1C2 RM1"; $resn_exch{"1H13 RM2"}="H131 RM2"; $resn_exch{"2H13 RM2"}="H132 RM2";
$resn_exch{"1H1C RM2"}="H1C1 RM2"; $resn_exch{"2H1C RM2"}="H1C2 RM2"; $resn_exch{"1H2C RM2"}="H2C1 RM2";
$resn_exch{"2H2C RM2"}="H2C2 RM2"; $resn_exch{"1H11 RMA"}="H111 RMA"; $resn_exch{"2H11 RMA"}="H112 RMA";
$resn_exch{"1H10 RMA"}="H101 RMA"; $resn_exch{"2H10 RMA"}="H102 RMA"; $resn_exch{"3H10 RMA"}="H103 RMA";
$resn_exch{"1H1  RMA"}=" H11 RMA"; $resn_exch{"2H1  RMA"}=" H12 RMA"; $resn_exch{"1H2  RMA"}=" H21 RMA";
$resn_exch{"2H2  RMA"}=" H22 RMA"; $resn_exch{"1H2  RMB"}=" H21 RMB"; $resn_exch{"1H4  RMB"}=" H41 RMB";
$resn_exch{"1H6  RMB"}=" H61 RMB"; $resn_exch{"1H7  RMB"}=" H71 RMB"; $resn_exch{"1H8  RMB"}=" H81 RMB";
$resn_exch{"2H8  RMB"}=" H82 RMB"; $resn_exch{"3H8  RMB"}=" H83 RMB"; $resn_exch{"2HOP RMB"}="HOP2 RMB";
$resn_exch{"3HOP RMB"}="HOP3 RMB"; $resn_exch{"1H5* RMB"}="H5'1 RMB"; $resn_exch{"2H5* RMB"}="H5'2 RMB";
$resn_exch{" H4* RMB"}=" H4' RMB"; $resn_exch{" H3* RMB"}=" H3' RMB"; $resn_exch{"*HO3 RMB"}="HO3' RMB";
$resn_exch{" H2* RMB"}=" H2' RMB"; $resn_exch{"*HO2 RMB"}="HO2' RMB"; $resn_exch{" H1* RMB"}=" H1' RMB";
$resn_exch{"1HAQ RMC"}="HAQ1 RMC"; $resn_exch{"2HAQ RMC"}="HAQ2 RMC"; $resn_exch{"1HAK RMC"}="HAK1 RMC";
$resn_exch{"2HAK RMC"}="HAK2 RMC"; $resn_exch{"1HAI RMC"}="HAI1 RMC"; $resn_exch{"2HAI RMC"}="HAI2 RMC";
$resn_exch{"3HAI RMC"}="HAI3 RMC"; $resn_exch{"1HAY RMC"}="HAY1 RMC"; $resn_exch{"2HAY RMC"}="HAY2 RMC";
$resn_exch{"3HAY RMC"}="HAY3 RMC"; $resn_exch{"1HBA RMC"}="HBA1 RMC"; $resn_exch{"2HBA RMC"}="HBA2 RMC";
$resn_exch{"3HBA RMC"}="HBA3 RMC"; $resn_exch{"1HAD RMC"}="HAD1 RMC"; $resn_exch{"2HAD RMC"}="HAD2 RMC";
$resn_exch{"1HBB RMC"}="HBB1 RMC"; $resn_exch{"2HBB RMC"}="HBB2 RMC"; $resn_exch{"3HBB RMC"}="HBB3 RMC";
$resn_exch{"1HMP RMP"}="HMP1 RMP"; $resn_exch{"2HMP RMP"}="HMP2 RMP"; $resn_exch{"3HMP RMP"}="HMP3 RMP";
$resn_exch{"1H5* RMP"}=" H5' RMP"; $resn_exch{"2H5* RMP"}="H5'' RMP"; $resn_exch{" H4* RMP"}=" H4' RMP";
$resn_exch{" H1* RMP"}=" H1' RMP"; $resn_exch{"1H6  RMP"}=" H61 RMP"; $resn_exch{"2H6  RMP"}=" H62 RMP";
$resn_exch{" H3* RMP"}=" H3' RMP"; $resn_exch{"1H2* RMP"}=" H2' RMP"; $resn_exch{"2H2* RMP"}="H2'' RMP";
$resn_exch{"H3T  RMP"}="HO3' RMP"; $resn_exch{"3HOP RMP"}="HOP3 RMP"; $resn_exch{"1H13 RNA"}="H131 RNA";
$resn_exch{"2H13 RNA"}="H132 RNA"; $resn_exch{"3H13 RNA"}="H133 RNA"; $resn_exch{"1H4  RNA"}=" H41 RNA";
$resn_exch{"2H4  RNA"}=" H42 RNA"; $resn_exch{"3H4  RNA"}=" H43 RNA"; $resn_exch{"1HA  RNG"}=" HA1 RNG";
$resn_exch{"2HA  RNG"}=" HA2 RNG"; $resn_exch{"1HG  RNG"}=" HG1 RNG"; $resn_exch{"2HG  RNG"}=" HG2 RNG";
$resn_exch{"1HF  RNG"}=" HF1 RNG"; $resn_exch{"2HF  RNG"}=" HF2 RNG"; $resn_exch{"1HE  RNG"}=" HE1 RNG";
$resn_exch{"2HE  RNG"}=" HE2 RNG"; $resn_exch{"1HD  RNG"}=" HD1 RNG"; $resn_exch{"2HD  RNG"}=" HD2 RNG";
$resn_exch{"1HC  RNG"}=" HC1 RNG"; $resn_exch{"2HC  RNG"}=" HC2 RNG"; $resn_exch{"1HB  RNG"}=" HB1 RNG";
$resn_exch{"2HB  RNG"}=" HB2 RNG"; $resn_exch{"1H8  RNO"}=" H81 RNO"; $resn_exch{"2H8  RNO"}=" H82 RNO";
$resn_exch{"1H11 RNP"}="H111 RNP"; $resn_exch{"2H11 RNP"}="H112 RNP"; $resn_exch{"1H15 RNP"}="H151 RNP";
$resn_exch{"2H15 RNP"}="H152 RNP"; $resn_exch{"3H15 RNP"}="H153 RNP"; $resn_exch{"1H16 RNP"}="H161 RNP";
$resn_exch{"2H16 RNP"}="H162 RNP"; $resn_exch{"3H16 RNP"}="H163 RNP"; $resn_exch{"1H6  RNS"}=" H61 RNS";
$resn_exch{"2H6  RNS"}=" H62 RNS"; $resn_exch{"3H6  RNS"}=" H63 RNS"; $resn_exch{"1H6  RNT"}=" H61 RNT";
$resn_exch{"2H6  RNT"}=" H62 RNT"; $resn_exch{"3H6  RNT"}=" H63 RNT"; $resn_exch{"1H7  RO0"}=" H71 RO0";
$resn_exch{"2H7  RO0"}=" H72 RO0"; $resn_exch{"1H11 RO0"}="H111 RO0"; $resn_exch{"2H11 RO0"}="H112 RO0";
$resn_exch{"1H19 RO0"}="H191 RO0"; $resn_exch{"2H19 RO0"}="H192 RO0"; $resn_exch{"3H19 RO0"}="H193 RO0";
$resn_exch{"1H20 RO0"}="H201 RO0"; $resn_exch{"2H20 RO0"}="H202 RO0"; $resn_exch{"1H21 RO0"}="H211 RO0";
$resn_exch{"2H21 RO0"}="H212 RO0"; $resn_exch{"1H22 RO0"}="H221 RO0"; $resn_exch{"2H22 RO0"}="H222 RO0";
$resn_exch{"3H22 RO0"}="H223 RO0"; $resn_exch{"1H25 RO0"}="H251 RO0"; $resn_exch{"2H25 RO0"}="H252 RO0";
$resn_exch{"1H26 RO0"}="H261 RO0"; $resn_exch{"2H26 RO0"}="H262 RO0"; $resn_exch{"1H31 RO0"}="H311 RO0";
$resn_exch{"2H31 RO0"}="H312 RO0"; $resn_exch{"3H31 RO0"}="H313 RO0"; $resn_exch{"1H33 RO0"}="H331 RO0";
$resn_exch{"2H33 RO0"}="H332 RO0"; $resn_exch{"3H33 RO0"}="H333 RO0"; $resn_exch{"1H40 RO0"}="H401 RO0";
$resn_exch{"2H40 RO0"}="H402 RO0"; $resn_exch{"3H40 RO0"}="H403 RO0"; $resn_exch{"1H37 RO1"}="H371 RO1";
$resn_exch{"2H37 RO1"}="H372 RO1"; $resn_exch{"3H37 RO1"}="H373 RO1"; $resn_exch{"1H33 RO1"}="H331 RO1";
$resn_exch{"2H33 RO1"}="H332 RO1"; $resn_exch{"3H33 RO1"}="H333 RO1"; $resn_exch{"1H34 RO1"}="H341 RO1";
$resn_exch{"2H34 RO1"}="H342 RO1"; $resn_exch{"3H34 RO1"}="H343 RO1"; $resn_exch{"1H36 RO1"}="H361 RO1";
$resn_exch{"2H36 RO1"}="H362 RO1"; $resn_exch{"1H19 RO1"}="H191 RO1"; $resn_exch{"2H19 RO1"}="H192 RO1";
$resn_exch{"1H8  RO1"}=" H81 RO1"; $resn_exch{"2H8  RO1"}=" H82 RO1"; $resn_exch{"1H23 RO1"}="H231 RO1";
$resn_exch{"2H23 RO1"}="H232 RO1"; $resn_exch{"3H23 RO1"}="H233 RO1"; $resn_exch{"1H22 RO1"}="H221 RO1";
$resn_exch{"2H22 RO1"}="H222 RO1"; $resn_exch{"3H22 RO1"}="H223 RO1"; $resn_exch{"1H21 RO1"}="H211 RO1";
$resn_exch{"2H21 RO1"}="H212 RO1"; $resn_exch{"3H21 RO1"}="H213 RO1"; $resn_exch{"1H9  RO1"}=" H91 RO1";
$resn_exch{"2H9  RO1"}=" H92 RO1"; $resn_exch{"1H14 RO1"}="H141 RO1"; $resn_exch{"2H14 RO1"}="H142 RO1";
$resn_exch{"1H7  RO1"}=" H71 RO1"; $resn_exch{"2H7  RO1"}=" H72 RO1"; $resn_exch{"1H16 RO1"}="H161 RO1";
$resn_exch{"2H16 RO1"}="H162 RO1"; $resn_exch{"1H20 RO1"}="H201 RO1"; $resn_exch{"2H20 RO1"}="H202 RO1";
$resn_exch{"1H18 RO1"}="H181 RO1"; $resn_exch{"2H18 RO1"}="H182 RO1"; $resn_exch{"1H21 RO2"}="H211 RO2";
$resn_exch{"2H21 RO2"}="H212 RO2"; $resn_exch{"1H22 RO2"}="H221 RO2"; $resn_exch{"2H22 RO2"}="H222 RO2";
$resn_exch{"1H23 RO2"}="H231 RO2"; $resn_exch{"2H23 RO2"}="H232 RO2"; $resn_exch{"1H24 RO2"}="H241 RO2";
$resn_exch{"2H24 RO2"}="H242 RO2"; $resn_exch{"3H25 RO2"}="H253 RO2"; $resn_exch{"2H25 RO2"}="H252 RO2";
$resn_exch{"1H25 RO2"}="H251 RO2"; $resn_exch{"1H9  RO4"}=" H91 RO4"; $resn_exch{"2H9  RO4"}=" H92 RO4";
$resn_exch{"1H31 RO4"}="H311 RO4"; $resn_exch{"2H31 RO4"}="H312 RO4"; $resn_exch{"1H15 RO4"}="H151 RO4";
$resn_exch{"2H15 RO4"}="H152 RO4"; $resn_exch{"1H19 RO4"}="H191 RO4"; $resn_exch{"2H19 RO4"}="H192 RO4";
$resn_exch{"1H32 RO4"}="H321 RO4"; $resn_exch{"2H32 RO4"}="H322 RO4"; $resn_exch{"3H32 RO4"}="H323 RO4";
$resn_exch{"1H17 RO4"}="H171 RO4"; $resn_exch{"2H17 RO4"}="H172 RO4"; $resn_exch{"3H17 RO4"}="H173 RO4";
$resn_exch{"1H18 RO4"}="H181 RO4"; $resn_exch{"2H18 RO4"}="H182 RO4"; $resn_exch{"3H18 RO4"}="H183 RO4";
$resn_exch{"1H21 RO4"}="H211 RO4"; $resn_exch{"2H21 RO4"}="H212 RO4"; $resn_exch{"3H21 RO4"}="H213 RO4";
$resn_exch{"1H22 RO4"}="H221 RO4"; $resn_exch{"2H22 RO4"}="H222 RO4"; $resn_exch{"3H22 RO4"}="H223 RO4";
$resn_exch{"1H27 RO4"}="H271 RO4"; $resn_exch{"2H27 RO4"}="H272 RO4"; $resn_exch{"3H27 RO4"}="H273 RO4";
$resn_exch{"1H1' ROB"}="H1'1 ROB"; $resn_exch{"2H1' ROB"}="H1'2 ROB"; $resn_exch{"'HO2 ROB"}="HO2' ROB";
$resn_exch{"'HO3 ROB"}="HO3' ROB"; $resn_exch{"1H5' ROB"}="H5'1 ROB"; $resn_exch{"2H5' ROB"}="H5'2 ROB";
$resn_exch{"2HOP ROB"}="HOP2 ROB"; $resn_exch{"3HOP ROB"}="HOP3 ROB"; $resn_exch{"1H3  ROC"}=" H31 ROC";
$resn_exch{"1H4  ROC"}=" H41 ROC"; $resn_exch{"1H6  ROC"}=" H61 ROC"; $resn_exch{"1H7  ROC"}=" H71 ROC";
$resn_exch{"1H8  ROC"}=" H81 ROC"; $resn_exch{"1H9  ROC"}=" H91 ROC"; $resn_exch{"1HN2 ROC"}="HN21 ROC";
$resn_exch{"1H11 ROC"}="H111 ROC"; $resn_exch{"1H13 ROC"}="H131 ROC"; $resn_exch{"2H13 ROC"}="H132 ROC";
$resn_exch{"1HN3 ROC"}="HN31 ROC"; $resn_exch{"2HN3 ROC"}="HN32 ROC"; $resn_exch{"1H15 ROC"}="H151 ROC";
$resn_exch{"1HO4 ROC"}="HO41 ROC"; $resn_exch{"1H16 ROC"}="H161 ROC"; $resn_exch{"1HN4 ROC"}="HN41 ROC";
$resn_exch{"1H17 ROC"}="H171 ROC"; $resn_exch{"2H17 ROC"}="H172 ROC"; $resn_exch{"1H19 ROC"}="H191 ROC";
$resn_exch{"1H20 ROC"}="H201 ROC"; $resn_exch{"1H21 ROC"}="H211 ROC"; $resn_exch{"1H22 ROC"}="H221 ROC";
$resn_exch{"1H23 ROC"}="H231 ROC"; $resn_exch{"1H24 ROC"}="H241 ROC"; $resn_exch{"2H24 ROC"}="H242 ROC";
$resn_exch{"1H25 ROC"}="H251 ROC"; $resn_exch{"1H27 ROC"}="H271 ROC"; $resn_exch{"2H27 ROC"}="H272 ROC";
$resn_exch{"1H28 ROC"}="H281 ROC"; $resn_exch{"1H29 ROC"}="H291 ROC"; $resn_exch{"2H29 ROC"}="H292 ROC";
$resn_exch{"1H30 ROC"}="H301 ROC"; $resn_exch{"2H30 ROC"}="H302 ROC"; $resn_exch{"1H31 ROC"}="H311 ROC";
$resn_exch{"2H31 ROC"}="H312 ROC"; $resn_exch{"1H32 ROC"}="H321 ROC"; $resn_exch{"2H32 ROC"}="H322 ROC";
$resn_exch{"1H33 ROC"}="H331 ROC"; $resn_exch{"1H34 ROC"}="H341 ROC"; $resn_exch{"2H34 ROC"}="H342 ROC";
$resn_exch{"1HN6 ROC"}="HN61 ROC"; $resn_exch{"1H36 ROC"}="H361 ROC"; $resn_exch{"2H36 ROC"}="H362 ROC";
$resn_exch{"3H36 ROC"}="H363 ROC"; $resn_exch{"1H37 ROC"}="H371 ROC"; $resn_exch{"2H37 ROC"}="H372 ROC";
$resn_exch{"3H37 ROC"}="H373 ROC"; $resn_exch{"1H38 ROC"}="H381 ROC"; $resn_exch{"2H38 ROC"}="H382 ROC";
$resn_exch{"3H38 ROC"}="H383 ROC"; $resn_exch{"1H23 ROF"}="H231 ROF"; $resn_exch{"2H23 ROF"}="H232 ROF";
$resn_exch{"1H22 ROF"}="H221 ROF"; $resn_exch{"2H22 ROF"}="H222 ROF"; $resn_exch{"1H20 ROF"}="H201 ROF";
$resn_exch{"2H20 ROF"}="H202 ROF"; $resn_exch{"1HN2 ROI"}="HN21 ROI"; $resn_exch{"2HN2 ROI"}="HN22 ROI";
$resn_exch{"1H16 ROI"}="H161 ROI"; $resn_exch{"2H16 ROI"}="H162 ROI"; $resn_exch{"3H16 ROI"}="H163 ROI";
$resn_exch{"1H15 ROI"}="H151 ROI"; $resn_exch{"2H15 ROI"}="H152 ROI"; $resn_exch{"3H15 ROI"}="H153 ROI";
$resn_exch{"1H6  ROL"}=" H61 ROL"; $resn_exch{"1H9  ROL"}=" H91 ROL"; $resn_exch{"1H10 ROL"}="H101 ROL";
$resn_exch{"1H3  ROL"}=" H31 ROL"; $resn_exch{"1H2  ROL"}=" H21 ROL"; $resn_exch{"2H2  ROL"}=" H22 ROL";
$resn_exch{"1H4  ROL"}=" H41 ROL"; $resn_exch{"2H4  ROL"}=" H42 ROL"; $resn_exch{"1H11 ROL"}="H111 ROL";
$resn_exch{"1H15 ROL"}="H151 ROL"; $resn_exch{"2H15 ROL"}="H152 ROL"; $resn_exch{"1H14 ROL"}="H141 ROL";
$resn_exch{"2H14 ROL"}="H142 ROL"; $resn_exch{"1H13 ROL"}="H131 ROL"; $resn_exch{"2H13 ROL"}="H132 ROL";
$resn_exch{"1H12 ROL"}="H121 ROL"; $resn_exch{"2H12 ROL"}="H122 ROL"; $resn_exch{"1H16 ROL"}="H161 ROL";
$resn_exch{"2H16 ROL"}="H162 ROL"; $resn_exch{"3H16 ROL"}="H163 ROL"; $resn_exch{"2H2' ROM"}="H2'2 ROM";
$resn_exch{"2H3' ROM"}="H3'2 ROM"; $resn_exch{"1H6B ROM"}="H6B1 ROM"; $resn_exch{"3H6B ROM"}="H6B3 ROM";
$resn_exch{"2H6B ROM"}="H6B2 ROM"; $resn_exch{"1H6A ROM"}="H6A1 ROM"; $resn_exch{"3H6A ROM"}="H6A3 ROM";
$resn_exch{"2H6A ROM"}="H6A2 ROM"; $resn_exch{"1H4B ROM"}="H4B1 ROM"; $resn_exch{"3H4B ROM"}="H4B3 ROM";
$resn_exch{"2H4B ROM"}="H4B2 ROM"; $resn_exch{"1H4A ROM"}="H4A1 ROM"; $resn_exch{"3H4A ROM"}="H4A3 ROM";
$resn_exch{"2H4A ROM"}="H4A2 ROM"; $resn_exch{"1H3A ROM"}="H3A1 ROM"; $resn_exch{"2H3A ROM"}="H3A2 ROM";
$resn_exch{"1H7A ROM"}="H7A1 ROM"; $resn_exch{"3H7A ROM"}="H7A3 ROM"; $resn_exch{"2H7A ROM"}="H7A2 ROM";
$resn_exch{"2H14 ROM"}="H142 ROM"; $resn_exch{"1H14 ROM"}="H141 ROM"; $resn_exch{"2H17 ROM"}="H172 ROM";
$resn_exch{"1H17 ROM"}="H171 ROM"; $resn_exch{"3H18 ROM"}="H183 ROM"; $resn_exch{"2H18 ROM"}="H182 ROM";
$resn_exch{"4H18 ROM"}="H184 ROM"; $resn_exch{"2H23 ROM"}="H232 ROM"; $resn_exch{"1HN  RON"}=" HN1 RON";
$resn_exch{"2HN  RON"}=" HN2 RON"; $resn_exch{"1HB  RON"}=" HB1 RON"; $resn_exch{"2HB  RON"}=" HB2 RON";
$resn_exch{"1HG  RON"}=" HG1 RON"; $resn_exch{"2HG  RON"}=" HG2 RON"; $resn_exch{"1HD  RON"}=" HD1 RON";
$resn_exch{"2HD  RON"}=" HD2 RON"; $resn_exch{"3HD  RON"}=" HD3 RON"; $resn_exch{"1HCA ROP"}="HCA1 ROP";
$resn_exch{"2HCA ROP"}="HCA2 ROP"; $resn_exch{"3HCA ROP"}="HCA3 ROP"; $resn_exch{"1HCB ROP"}="HCB1 ROP";
$resn_exch{"2HCB ROP"}="HCB2 ROP"; $resn_exch{"1HND ROP"}="HND1 ROP"; $resn_exch{"2HND ROP"}="HND2 ROP";
$resn_exch{"1H20 ROS"}="H201 ROS"; $resn_exch{"2H20 ROS"}="H202 ROS"; $resn_exch{"3H20 ROS"}="H203 ROS";
$resn_exch{"1H21 ROS"}="H211 ROS"; $resn_exch{"2H21 ROS"}="H212 ROS"; $resn_exch{"3H21 ROS"}="H213 ROS";
$resn_exch{"1H22 ROS"}="H221 ROS"; $resn_exch{"2H22 ROS"}="H222 ROS"; $resn_exch{"3H22 ROS"}="H223 ROS";
$resn_exch{"1H23 ROS"}="H231 ROS"; $resn_exch{"2H23 ROS"}="H232 ROS"; $resn_exch{"3H23 ROS"}="H233 ROS";
$resn_exch{"1H7  ROX"}=" H71 ROX"; $resn_exch{"2H7  ROX"}=" H72 ROX"; $resn_exch{"1H34 ROX"}="H341 ROX";
$resn_exch{"2H34 ROX"}="H342 ROX"; $resn_exch{"3H34 ROX"}="H343 ROX"; $resn_exch{"1H33 ROX"}="H331 ROX";
$resn_exch{"2H33 ROX"}="H332 ROX"; $resn_exch{"3H33 ROX"}="H333 ROX"; $resn_exch{"1H35 ROX"}="H351 ROX";
$resn_exch{"2H35 ROX"}="H352 ROX"; $resn_exch{"3H35 ROX"}="H353 ROX"; $resn_exch{"1H36 ROX"}="H361 ROX";
$resn_exch{"2H36 ROX"}="H362 ROX"; $resn_exch{"1H30 ROX"}="H301 ROX"; $resn_exch{"2H30 ROX"}="H302 ROX";
$resn_exch{"3H30 ROX"}="H303 ROX"; $resn_exch{"1H32 ROX"}="H321 ROX"; $resn_exch{"2H32 ROX"}="H322 ROX";
$resn_exch{"3H32 ROX"}="H323 ROX"; $resn_exch{"1H25 ROX"}="H251 ROX"; $resn_exch{"2H25 ROX"}="H252 ROX";
$resn_exch{"1H27 ROX"}="H271 ROX"; $resn_exch{"2H27 ROX"}="H272 ROX"; $resn_exch{"3H27 ROX"}="H273 ROX";
$resn_exch{"1H28 ROX"}="H281 ROX"; $resn_exch{"2H28 ROX"}="H282 ROX"; $resn_exch{"3H28 ROX"}="H283 ROX";
$resn_exch{"1H15 ROX"}="H151 ROX"; $resn_exch{"2H15 ROX"}="H152 ROX"; $resn_exch{"1H20 ROX"}="H201 ROX";
$resn_exch{"2H20 ROX"}="H202 ROX"; $resn_exch{"3H20 ROX"}="H203 ROX"; $resn_exch{"1H29 ROX"}="H291 ROX";
$resn_exch{"2H29 ROX"}="H292 ROX"; $resn_exch{"3H29 ROX"}="H293 ROX"; $resn_exch{"1H21 ROX"}="H211 ROX";
$resn_exch{"2H21 ROX"}="H212 ROX"; $resn_exch{"3H21 ROX"}="H213 ROX"; $resn_exch{"1H37 ROX"}="H371 ROX";
$resn_exch{"2H37 ROX"}="H372 ROX"; $resn_exch{"3H37 ROX"}="H373 ROX"; $resn_exch{"1H31 ROX"}="H311 ROX";
$resn_exch{"2H31 ROX"}="H312 ROX"; $resn_exch{"3H31 ROX"}="H313 ROX"; $resn_exch{"1H19 ROX"}="H191 ROX";
$resn_exch{"2H19 ROX"}="H192 ROX"; $resn_exch{"3H19 ROX"}="H193 ROX"; $resn_exch{"1H38 ROX"}="H381 ROX";
$resn_exch{"2H38 ROX"}="H382 ROX"; $resn_exch{"1H39 ROX"}="H391 ROX"; $resn_exch{"2H39 ROX"}="H392 ROX";
$resn_exch{"1H40 ROX"}="H401 ROX"; $resn_exch{"2H40 ROX"}="H402 ROX"; $resn_exch{"1H41 ROX"}="H411 ROX";
$resn_exch{"2H41 ROX"}="H412 ROX"; $resn_exch{"3H41 ROX"}="H413 ROX"; $resn_exch{"2HOP RP1"}="HOP2 RP1";
$resn_exch{"1H5* RP1"}="H5'1 RP1"; $resn_exch{"2H5* RP1"}="H5'2 RP1"; $resn_exch{" H4* RP1"}=" H4' RP1";
$resn_exch{" H3* RP1"}=" H3' RP1"; $resn_exch{" H2* RP1"}=" H2' RP1"; $resn_exch{"*HO2 RP1"}="HO2' RP1";
$resn_exch{" H1* RP1"}=" H1' RP1"; $resn_exch{"1HN6 RP1"}="HN61 RP1"; $resn_exch{"2HN6 RP1"}="HN62 RP1";
$resn_exch{"1H12 RP4"}="H121 RP4"; $resn_exch{"2H12 RP4"}="H122 RP4"; $resn_exch{"1H5' RP5"}="H5'1 RP5";
$resn_exch{"2H5' RP5"}="H5'2 RP5"; $resn_exch{"1H6' RPA"}="H6'1 RPA"; $resn_exch{"2H6' RPA"}="H6'2 RPA";
$resn_exch{"'HO2 RPA"}="HO2' RPA"; $resn_exch{"'HO3 RPA"}="HO3' RPA"; $resn_exch{"'HO4 RPA"}="HO4' RPA";
$resn_exch{"'HO6 RPA"}="HO6' RPA"; $resn_exch{"1HM1 RPA"}="HM11 RPA"; $resn_exch{"2HM1 RPA"}="HM12 RPA";
$resn_exch{"3HM1 RPA"}="HM13 RPA"; $resn_exch{"1HM2 RPA"}="HM21 RPA"; $resn_exch{"2HM2 RPA"}="HM22 RPA";
$resn_exch{"3HM2 RPA"}="HM23 RPA"; $resn_exch{"1H2  RPA"}="H21A RPA"; $resn_exch{"2H2  RPA"}=" H22 RPA";
$resn_exch{"1H3  RPA"}=" H31 RPA"; $resn_exch{"2H3  RPA"}=" H32 RPA"; $resn_exch{"1H4  RPA"}=" H41 RPA";
$resn_exch{"2H4  RPA"}=" H42 RPA"; $resn_exch{"1HM3 RPA"}="HM31 RPA"; $resn_exch{"2HM3 RPA"}="HM32 RPA";
$resn_exch{"3HM3 RPA"}="HM33 RPA"; $resn_exch{"1HM4 RPA"}="HM41 RPA"; $resn_exch{"2HM4 RPA"}="HM42 RPA";
$resn_exch{"3HM4 RPA"}="HM43 RPA"; $resn_exch{"1HM5 RPA"}="HM51 RPA"; $resn_exch{"2HM5 RPA"}="HM52 RPA";
$resn_exch{"3HM5 RPA"}="HM53 RPA"; $resn_exch{"2HM6 RPA"}="HM62 RPA"; $resn_exch{"3HM6 RPA"}="HM63 RPA";
$resn_exch{"1HM7 RPA"}="HM71 RPA"; $resn_exch{"2HM7 RPA"}="HM72 RPA"; $resn_exch{"3HM7 RPA"}="HM73 RPA";
$resn_exch{"1HM8 RPA"}="HM81 RPA"; $resn_exch{"2HM8 RPA"}="HM82 RPA"; $resn_exch{"3HM8 RPA"}="HM83 RPA";
$resn_exch{"1HM9 RPA"}="HM91 RPA"; $resn_exch{"2HM9 RPA"}="HM92 RPA"; $resn_exch{"3HM9 RPA"}="HM93 RPA";
$resn_exch{"1HM0 RPA"}="HM01 RPA"; $resn_exch{"2HM0 RPA"}="HM02 RPA"; $resn_exch{"3HM0 RPA"}="HM03 RPA";
$resn_exch{"1HM6 RPA"}="HM61 RPA"; $resn_exch{"1H5* RPD"}="H5'1 RPD"; $resn_exch{"2H5* RPD"}="H5'2 RPD";
$resn_exch{"HO3* RPD"}="HO3' RPD"; $resn_exch{"HO2* RPD"}="HO2' RPD"; $resn_exch{"1H5  RPD"}=" H51 RPD";
$resn_exch{"2H5  RPD"}=" H52 RPD"; $resn_exch{"1H6  RPD"}=" H61 RPD"; $resn_exch{"2H6  RPD"}=" H62 RPD";
$resn_exch{"1H7  RPD"}=" H71 RPD"; $resn_exch{"2H7  RPD"}=" H72 RPD"; $resn_exch{"1H8  RPD"}=" H81 RPD";
$resn_exch{"1H11 RPD"}="H111 RPD"; $resn_exch{"2H11 RPD"}="H112 RPD"; $resn_exch{"1H1C RPF"}="H1C1 RPF";
$resn_exch{"2H1C RPF"}="H1C2 RPF"; $resn_exch{"3H1C RPF"}="H1C3 RPF"; $resn_exch{"1H4C RPF"}="H4C1 RPF";
$resn_exch{"2H4C RPF"}="H4C2 RPF"; $resn_exch{"1H21 RPF"}="H211 RPF"; $resn_exch{"2H21 RPF"}="H212 RPF";
$resn_exch{"1H22 RPF"}="H221 RPF"; $resn_exch{"2H22 RPF"}="H222 RPF"; $resn_exch{"1H24 RPF"}="H241 RPF";
$resn_exch{"2H24 RPF"}="H242 RPF"; $resn_exch{"1H31 RPF"}="H311 RPF"; $resn_exch{"2H31 RPF"}="H312 RPF";
$resn_exch{"1H32 RPF"}="H321 RPF"; $resn_exch{"2H32 RPF"}="H322 RPF"; $resn_exch{"1H33 RPF"}="H331 RPF";
$resn_exch{"2H33 RPF"}="H332 RPF"; $resn_exch{"1H34 RPF"}="H341 RPF"; $resn_exch{"2H34 RPF"}="H342 RPF";
$resn_exch{"1H35 RPF"}="H351 RPF"; $resn_exch{"2H35 RPF"}="H352 RPF"; $resn_exch{"1H36 RPF"}="H361 RPF";
$resn_exch{"2H36 RPF"}="H362 RPF"; $resn_exch{"1H9  RPH"}=" H91 RPH"; $resn_exch{"2H9  RPH"}=" H92 RPH";
$resn_exch{"1H11 RPH"}="H111 RPH"; $resn_exch{"2H11 RPH"}="H112 RPH"; $resn_exch{"1H12 RPH"}="H121 RPH";
$resn_exch{"2H12 RPH"}="H122 RPH"; $resn_exch{"1H14 RPH"}="H141 RPH"; $resn_exch{"2H14 RPH"}="H142 RPH";
$resn_exch{"1H5* RPL"}="H5'1 RPL"; $resn_exch{"2H5* RPL"}="H5'2 RPL"; $resn_exch{"HO3* RPL"}="HO3' RPL";
$resn_exch{"HO2* RPL"}="HO2' RPL"; $resn_exch{"1H5  RPL"}=" H51 RPL"; $resn_exch{"2H5  RPL"}=" H52 RPL";
$resn_exch{"1H6  RPL"}=" H61 RPL"; $resn_exch{"2H6  RPL"}=" H62 RPL"; $resn_exch{"1H7  RPL"}=" H71 RPL";
$resn_exch{"2H7  RPL"}=" H72 RPL"; $resn_exch{"1H8  RPL"}=" H81 RPL"; $resn_exch{"1H11 RPL"}="H111 RPL";
$resn_exch{"2H11 RPL"}="H112 RPL"; $resn_exch{"1HC1 RPN"}="HC11 RPN"; $resn_exch{"2HC1 RPN"}="HC12 RPN";
$resn_exch{"1HN6 RPP"}="HN61 RPP"; $resn_exch{"2HN6 RPP"}="HN62 RPP"; $resn_exch{" H1* RPP"}=" H1' RPP";
$resn_exch{" H2* RPP"}=" H2' RPP"; $resn_exch{" H3* RPP"}=" H3' RPP"; $resn_exch{" H4* RPP"}=" H4' RPP";
$resn_exch{"2HO* RPP"}="HO'2 RPP"; $resn_exch{"3HO* RPP"}="HO'3 RPP"; $resn_exch{"1H5* RPP"}="H5'1 RPP";
$resn_exch{"2H5* RPP"}="H5'2 RPP"; $resn_exch{"5HO* RPP"}="HO'5 RPP"; $resn_exch{"1H1  RPR"}=" H11 RPR";
$resn_exch{"1H3  RPR"}=" H31 RPR"; $resn_exch{"1H4  RPR"}=" H41 RPR"; $resn_exch{"1H5  RPR"}=" H51 RPR";
$resn_exch{"1H7  RPR"}=" H71 RPR"; $resn_exch{"1H9  RPR"}=" H91 RPR"; $resn_exch{"1H10 RPR"}="H101 RPR";
$resn_exch{"1H12 RPR"}="H121 RPR"; $resn_exch{"1H14 RPR"}="H141 RPR"; $resn_exch{"1H15 RPR"}="H151 RPR";
$resn_exch{"1H16 RPR"}="H161 RPR"; $resn_exch{"2H16 RPR"}="H162 RPR"; $resn_exch{"3H16 RPR"}="H163 RPR";
$resn_exch{"1H17 RPR"}="H171 RPR"; $resn_exch{"1H20 RPR"}="H201 RPR"; $resn_exch{"2H20 RPR"}="H202 RPR";
$resn_exch{"1H21 RPR"}="H211 RPR"; $resn_exch{"2H21 RPR"}="H212 RPR"; $resn_exch{"1H22 RPR"}="H221 RPR";
$resn_exch{"2H22 RPR"}="H222 RPR"; $resn_exch{"1H23 RPR"}="H231 RPR"; $resn_exch{"1H25 RPR"}="H251 RPR";
$resn_exch{"1H26 RPR"}="H261 RPR"; $resn_exch{"1H27 RPR"}="H271 RPR"; $resn_exch{"1H30 RPR"}="H301 RPR";
$resn_exch{"2H30 RPR"}="H302 RPR"; $resn_exch{"1H31 RPR"}="H311 RPR"; $resn_exch{"1H34 RPR"}="H341 RPR";
$resn_exch{"2H34 RPR"}="H342 RPR"; $resn_exch{"3H34 RPR"}="H343 RPR"; $resn_exch{"1H13 RPT"}="H131 RPT";
$resn_exch{"2H13 RPT"}="H132 RPT"; $resn_exch{"3H13 RPT"}="H133 RPT"; $resn_exch{"1H14 RPT"}="H141 RPT";
$resn_exch{"2H14 RPT"}="H142 RPT"; $resn_exch{"3H14 RPT"}="H143 RPT"; $resn_exch{"1H30 RPT"}="H301 RPT";
$resn_exch{"2H30 RPT"}="H302 RPT"; $resn_exch{"3H30 RPT"}="H303 RPT"; $resn_exch{"1H31 RPT"}="H311 RPT";
$resn_exch{"2H31 RPT"}="H312 RPT"; $resn_exch{"3H31 RPT"}="H313 RPT"; $resn_exch{"1H32 RPT"}="H321 RPT";
$resn_exch{"2H32 RPT"}="H322 RPT"; $resn_exch{"3H32 RPT"}="H323 RPT"; $resn_exch{"1H33 RPT"}="H331 RPT";
$resn_exch{"2H33 RPT"}="H332 RPT"; $resn_exch{"3H33 RPT"}="H333 RPT"; $resn_exch{"1H34 RPT"}="H341 RPT";
$resn_exch{"2H34 RPT"}="H342 RPT"; $resn_exch{"3H34 RPT"}="H343 RPT"; $resn_exch{"1H36 RPT"}="H361 RPT";
$resn_exch{"2H36 RPT"}="H362 RPT"; $resn_exch{"3H36 RPT"}="H363 RPT"; $resn_exch{"1H37 RPT"}="H371 RPT";
$resn_exch{"2H37 RPT"}="H372 RPT"; $resn_exch{"3H37 RPT"}="H373 RPT"; $resn_exch{"1H39 RPT"}="H391 RPT";
$resn_exch{"2H39 RPT"}="H392 RPT"; $resn_exch{"1H40 RPT"}="H401 RPT"; $resn_exch{"2H40 RPT"}="H402 RPT";
$resn_exch{"1H41 RPT"}="H411 RPT"; $resn_exch{"2H41 RPT"}="H412 RPT"; $resn_exch{"1H42 RPT"}="H421 RPT";
$resn_exch{"2H42 RPT"}="H422 RPT"; $resn_exch{"1H44 RPT"}="H441 RPT"; $resn_exch{"2H44 RPT"}="H442 RPT";
$resn_exch{"1H45 RPT"}="H451 RPT"; $resn_exch{"2H45 RPT"}="H452 RPT"; $resn_exch{"1H46 RPT"}="H461 RPT";
$resn_exch{"2H46 RPT"}="H462 RPT"; $resn_exch{"1H47 RPT"}="H471 RPT"; $resn_exch{"2H47 RPT"}="H472 RPT";
$resn_exch{"1H3  RPX"}="H31A RPX"; $resn_exch{"2H3  RPX"}=" H32 RPX"; $resn_exch{"1H4  RPX"}="H41A RPX";
$resn_exch{"2H4  RPX"}="H42A RPX"; $resn_exch{"1H5  RPX"}=" H51 RPX"; $resn_exch{"2H5  RPX"}=" H52 RPX";
$resn_exch{"1H6  RPX"}=" H61 RPX"; $resn_exch{"2H6  RPX"}=" H62 RPX"; $resn_exch{"1H12 RPX"}="H121 RPX";
$resn_exch{"2H12 RPX"}="H122 RPX"; $resn_exch{"1H13 RPX"}="H131 RPX"; $resn_exch{"2H13 RPX"}="H132 RPX";
$resn_exch{"1H15 RPX"}="H151 RPX"; $resn_exch{"2H15 RPX"}="H152 RPX"; $resn_exch{"1H24 RPX"}="H241 RPX";
$resn_exch{"2H24 RPX"}="H242 RPX"; $resn_exch{"1H30 RPX"}="H301 RPX"; $resn_exch{"2H30 RPX"}="H302 RPX";
$resn_exch{"1H33 RPX"}="H331 RPX"; $resn_exch{"2H33 RPX"}="H332 RPX"; $resn_exch{"1H36 RPX"}="H361 RPX";
$resn_exch{"2H36 RPX"}="H362 RPX"; $resn_exch{"1H43 RPX"}="H431 RPX"; $resn_exch{"2H43 RPX"}="H432 RPX";
$resn_exch{"3H43 RPX"}="H433 RPX"; $resn_exch{"1H44 RPX"}="H441 RPX"; $resn_exch{"2H44 RPX"}="H442 RPX";
$resn_exch{"3H44 RPX"}="H443 RPX"; $resn_exch{"1H45 RPX"}="H451 RPX"; $resn_exch{"2H45 RPX"}="H452 RPX";
$resn_exch{"3H45 RPX"}="H453 RPX"; $resn_exch{"1H46 RPX"}="H461 RPX"; $resn_exch{"2H46 RPX"}="H462 RPX";
$resn_exch{"3H46 RPX"}="H463 RPX"; $resn_exch{"1H47 RPX"}="H471 RPX"; $resn_exch{"2H47 RPX"}="H472 RPX";
$resn_exch{"3H47 RPX"}="H473 RPX"; $resn_exch{"1H48 RPX"}="H481 RPX"; $resn_exch{"2H48 RPX"}="H482 RPX";
$resn_exch{"3H48 RPX"}="H483 RPX"; $resn_exch{"1H49 RPX"}="H491 RPX"; $resn_exch{"2H49 RPX"}="H492 RPX";
$resn_exch{"3H49 RPX"}="H493 RPX"; $resn_exch{"1H50 RPX"}="H501 RPX"; $resn_exch{"2H50 RPX"}="H502 RPX";
$resn_exch{"1H51 RPX"}="H511 RPX"; $resn_exch{"2H51 RPX"}="H512 RPX"; $resn_exch{"3H51 RPX"}="H513 RPX";
$resn_exch{"1H52 RPX"}="H521 RPX"; $resn_exch{"2H52 RPX"}="H522 RPX"; $resn_exch{"3H52 RPX"}="H523 RPX";
$resn_exch{"1H53 RPX"}="H531 RPX"; $resn_exch{"2H53 RPX"}="H532 RPX"; $resn_exch{"3H53 RPX"}="H533 RPX";
$resn_exch{"1H5  RQ3"}=" H51 RQ3"; $resn_exch{"2H5  RQ3"}=" H52 RQ3"; $resn_exch{"3H5  RQ3"}=" H53 RQ3";
$resn_exch{"1H6  RQ3"}=" H61 RQ3"; $resn_exch{"2H6  RQ3"}=" H62 RQ3"; $resn_exch{"3H6  RQ3"}=" H63 RQ3";
$resn_exch{"1H9  RQ3"}=" H91 RQ3"; $resn_exch{"1H10 RQ3"}="H101 RQ3"; $resn_exch{"1H12 RQ3"}="H121 RQ3";
$resn_exch{"1H13 RQ3"}="H131 RQ3"; $resn_exch{"1H14 RQ3"}="H141 RQ3"; $resn_exch{"2H14 RQ3"}="H142 RQ3";
$resn_exch{"1H17 RQ3"}="H171 RQ3"; $resn_exch{"1H19 RQ3"}="H191 RQ3"; $resn_exch{"1H21 RQ3"}="H211 RQ3";
$resn_exch{"1H22 RQ3"}="H221 RQ3"; $resn_exch{"1H24 RQ3"}="H241 RQ3"; $resn_exch{"2H24 RQ3"}="H242 RQ3";
$resn_exch{"3H24 RQ3"}="H243 RQ3"; $resn_exch{"1H25 RQ3"}="H251 RQ3"; $resn_exch{"2H25 RQ3"}="H252 RQ3";
$resn_exch{"3H25 RQ3"}="H253 RQ3"; $resn_exch{"1HL1 RR1"}="HL11 RR1"; $resn_exch{"2HL1 RR1"}="HL12 RR1";
$resn_exch{"1HL2 RR1"}="HL21 RR1"; $resn_exch{"2HL2 RR1"}="HL22 RR1"; $resn_exch{"1HAQ RRC"}="HAQ1 RRC";
$resn_exch{"2HAQ RRC"}="HAQ2 RRC"; $resn_exch{"1HAK RRC"}="HAK1 RRC"; $resn_exch{"2HAK RRC"}="HAK2 RRC";
$resn_exch{"1HAI RRC"}="HAI1 RRC"; $resn_exch{"2HAI RRC"}="HAI2 RRC"; $resn_exch{"3HAI RRC"}="HAI3 RRC";
$resn_exch{"1HAY RRC"}="HAY1 RRC"; $resn_exch{"2HAY RRC"}="HAY2 RRC"; $resn_exch{"3HAY RRC"}="HAY3 RRC";
$resn_exch{"1HBA RRC"}="HBA1 RRC"; $resn_exch{"2HBA RRC"}="HBA2 RRC"; $resn_exch{"3HBA RRC"}="HBA3 RRC";
$resn_exch{"1HAD RRC"}="HAD1 RRC"; $resn_exch{"2HAD RRC"}="HAD2 RRC"; $resn_exch{"1H30 RRP"}="H301 RRP";
$resn_exch{"2H30 RRP"}="H302 RRP"; $resn_exch{"1H21 RRP"}="H211 RRP"; $resn_exch{"2H21 RRP"}="H212 RRP";
$resn_exch{"1H4  RRP"}=" H41 RRP"; $resn_exch{"2H4  RRP"}=" H42 RRP"; $resn_exch{"1H7  RRP"}=" H71 RRP";
$resn_exch{"2H7  RRP"}=" H72 RRP"; $resn_exch{"1H2  RRP"}=" H21 RRP"; $resn_exch{"2H2  RRP"}="H22A RRP";
$resn_exch{"1H13 RRR"}="H131 RRR"; $resn_exch{"2H13 RRR"}="H132 RRR"; $resn_exch{"1H18 RRR"}="H181 RRR";
$resn_exch{"2H18 RRR"}="H182 RRR"; $resn_exch{"1H20 RRR"}="H201 RRR"; $resn_exch{"2H20 RRR"}="H202 RRR";
$resn_exch{"1H21 RRR"}="H211 RRR"; $resn_exch{"2H21 RRR"}="H212 RRR"; $resn_exch{"1H2  RRS"}=" H21 RRS";
$resn_exch{"2H2  RRS"}="H22A RRS"; $resn_exch{"1H7  RRS"}=" H71 RRS"; $resn_exch{"2H7  RRS"}=" H72 RRS";
$resn_exch{"1H8  RRS"}=" H81 RRS"; $resn_exch{"2H8  RRS"}=" H82 RRS"; $resn_exch{"1H9  RRS"}=" H91 RRS";
$resn_exch{"2H9  RRS"}=" H92 RRS"; $resn_exch{"1H10 RRS"}="H101 RRS"; $resn_exch{"2H10 RRS"}="H102 RRS";
$resn_exch{"1H11 RRS"}="H111 RRS"; $resn_exch{"2H11 RRS"}="H112 RRS"; $resn_exch{"1H12 RRS"}="H121 RRS";
$resn_exch{"2H12 RRS"}="H122 RRS"; $resn_exch{"1H13 RRS"}="H131 RRS"; $resn_exch{"2H13 RRS"}="H132 RRS";
$resn_exch{"1H15 RRS"}="H151 RRS"; $resn_exch{"2H15 RRS"}="H152 RRS"; $resn_exch{"3H15 RRS"}="H153 RRS";
$resn_exch{"1H16 RRS"}="H161 RRS"; $resn_exch{"2H16 RRS"}="H162 RRS"; $resn_exch{"3H16 RRS"}="H163 RRS";
$resn_exch{"1H17 RRS"}="H171 RRS"; $resn_exch{"2H17 RRS"}="H172 RRS"; $resn_exch{"1HB2 RRT"}="HB21 RRT";
$resn_exch{"2HB2 RRT"}="HB22 RRT"; $resn_exch{"1HB1 RRT"}="HB11 RRT"; $resn_exch{"2HB1 RRT"}="HB12 RRT";
$resn_exch{"1HA2 RRT"}="HA21 RRT"; $resn_exch{"2HA2 RRT"}="HA22 RRT"; $resn_exch{"1H25 RS1"}="H251 RS1";
$resn_exch{"2H25 RS1"}="H252 RS1"; $resn_exch{"1H29 RS1"}="H291 RS1"; $resn_exch{"2H29 RS1"}="H292 RS1";
$resn_exch{"1H30 RS1"}="H301 RS1"; $resn_exch{"2H30 RS1"}="H302 RS1"; $resn_exch{"1H32 RS1"}="H321 RS1";
$resn_exch{"2H32 RS1"}="H322 RS1"; $resn_exch{"1H33 RS1"}="H331 RS1"; $resn_exch{"2H33 RS1"}="H332 RS1";
$resn_exch{"1H27 RS2"}="H271 RS2"; $resn_exch{"2H27 RS2"}="H272 RS2"; $resn_exch{"1H36 RS2"}="H361 RS2";
$resn_exch{"2H36 RS2"}="H362 RS2"; $resn_exch{"1H37 RS2"}="H371 RS2"; $resn_exch{"2H37 RS2"}="H372 RS2";
$resn_exch{"1H39 RS2"}="H391 RS2"; $resn_exch{"2H39 RS2"}="H392 RS2"; $resn_exch{"1H40 RS2"}="H401 RS2";
$resn_exch{"2H40 RS2"}="H402 RS2"; $resn_exch{"1H2  RS7"}=" H21 RS7"; $resn_exch{"1H3  RS7"}=" H31 RS7";
$resn_exch{"1H12 RS7"}="H121 RS7"; $resn_exch{"2H12 RS7"}="H122 RS7"; $resn_exch{"1H13 RS7"}="H131 RS7";
$resn_exch{"2H13 RS7"}="H132 RS7"; $resn_exch{"1H14 RS7"}="H141 RS7"; $resn_exch{"2H14 RS7"}="H142 RS7";
$resn_exch{"1H15 RS7"}="H151 RS7"; $resn_exch{"2H15 RS7"}="H152 RS7"; $resn_exch{"1H16 RS7"}="H161 RS7";
$resn_exch{"2H16 RS7"}="H162 RS7"; $resn_exch{"1H17 RS7"}="H171 RS7"; $resn_exch{"2H17 RS7"}="H172 RS7";
$resn_exch{"3H17 RS7"}="H173 RS7"; $resn_exch{"1H11 RSA"}="H111 RSA"; $resn_exch{"2H11 RSA"}="H112 RSA";
$resn_exch{"1H1  RSA"}=" H11 RSA"; $resn_exch{"2H1  RSA"}=" H12 RSA"; $resn_exch{"1H2  RSA"}=" H21 RSA";
$resn_exch{"2H2  RSA"}=" H22 RSA"; $resn_exch{"2H4  RSD"}=" H42 RSD"; $resn_exch{"2H33 RSD"}="H332 RSD";
$resn_exch{"2H47 RSD"}="H472 RSD"; $resn_exch{"2H39 RSD"}="H392 RSD"; $resn_exch{"1H3  RSD"}="H31A RSD";
$resn_exch{"2H3  RSD"}="H32A RSD"; $resn_exch{"3H3  RSD"}=" H33 RSD"; $resn_exch{"1H4  RSD"}=" H41 RSD";
$resn_exch{"1H6  RSD"}=" H61 RSD"; $resn_exch{"2H6  RSD"}=" H62 RSD"; $resn_exch{"3H6  RSD"}=" H63 RSD";
$resn_exch{"1H8  RSD"}=" H81 RSD"; $resn_exch{"2H8  RSD"}=" H82 RSD"; $resn_exch{"3H8  RSD"}=" H83 RSD";
$resn_exch{"2H11 RSD"}="H112 RSD"; $resn_exch{"1H11 RSD"}="H111 RSD"; $resn_exch{"1H13 RSD"}="H131 RSD";
$resn_exch{"2H13 RSD"}="H132 RSD"; $resn_exch{"3H13 RSD"}="H133 RSD"; $resn_exch{"1H15 RSD"}="H151 RSD";
$resn_exch{"2H15 RSD"}="H152 RSD"; $resn_exch{"3H15 RSD"}="H153 RSD"; $resn_exch{"19H1 RSD"}="H119 RSD";
$resn_exch{"21H2 RSD"}="H221 RSD"; $resn_exch{"28H2 RSD"}="H228 RSD"; $resn_exch{"1H30 RSD"}="H301 RSD";
$resn_exch{"1H33 RSD"}="H331 RSD"; $resn_exch{"1H34 RSD"}="H341 RSD"; $resn_exch{"2H34 RSD"}="H342 RSD";
$resn_exch{"3H34 RSD"}="H343 RSD"; $resn_exch{"1H37 RSD"}="H371 RSD"; $resn_exch{"2H37 RSD"}="H372 RSD";
$resn_exch{"1H38 RSD"}="H381 RSD"; $resn_exch{"2H38 RSD"}="H382 RSD"; $resn_exch{"3H38 RSD"}="H383 RSD";
$resn_exch{"1H39 RSD"}="H391 RSD"; $resn_exch{"1H41 RSD"}="H411 RSD"; $resn_exch{"2H41 RSD"}="H412 RSD";
$resn_exch{"3H41 RSD"}="H413 RSD"; $resn_exch{"1H42 RSD"}="H421 RSD"; $resn_exch{"2H42 RSD"}="H422 RSD";
$resn_exch{"3H42 RSD"}="H423 RSD"; $resn_exch{"1H43 RSD"}="H431 RSD"; $resn_exch{"2H43 RSD"}="H432 RSD";
$resn_exch{"3H43 RSD"}="H433 RSD"; $resn_exch{"1H46 RSD"}="H461 RSD"; $resn_exch{"2H46 RSD"}="H462 RSD";
$resn_exch{"3H46 RSD"}="H463 RSD"; $resn_exch{"1H47 RSD"}="H471 RSD"; $resn_exch{"1H49 RSD"}="H491 RSD";
$resn_exch{"2H49 RSD"}="H492 RSD"; $resn_exch{"3H49 RSD"}="H493 RSD"; $resn_exch{"1H51 RSD"}="H511 RSD";
$resn_exch{"2H51 RSD"}="H512 RSD"; $resn_exch{"3H51 RSD"}="H513 RSD"; $resn_exch{"1H2C RSH"}="H2C1 RSH";
$resn_exch{"1H3C RSH"}="H3C1 RSH"; $resn_exch{"2H3C RSH"}="H3C2 RSH"; $resn_exch{"1H6C RSH"}="H6C1 RSH";
$resn_exch{"2H6C RSH"}="H6C2 RSH"; $resn_exch{"1H13 RSO"}="H131 RSO"; $resn_exch{"2H13 RSO"}="H132 RSO";
$resn_exch{"1H2  RSS"}=" H21 RSS"; $resn_exch{"2H2  RSS"}="H22A RSS"; $resn_exch{"1H7  RSS"}=" H71 RSS";
$resn_exch{"2H7  RSS"}=" H72 RSS"; $resn_exch{"1H8  RSS"}=" H81 RSS"; $resn_exch{"2H8  RSS"}=" H82 RSS";
$resn_exch{"1H9  RSS"}=" H91 RSS"; $resn_exch{"2H9  RSS"}=" H92 RSS"; $resn_exch{"1H10 RSS"}="H101 RSS";
$resn_exch{"2H10 RSS"}="H102 RSS"; $resn_exch{"1H11 RSS"}="H111 RSS"; $resn_exch{"2H11 RSS"}="H112 RSS";
$resn_exch{"1H12 RSS"}="H121 RSS"; $resn_exch{"2H12 RSS"}="H122 RSS"; $resn_exch{"1H13 RSS"}="H131 RSS";
$resn_exch{"2H13 RSS"}="H132 RSS"; $resn_exch{"1H15 RSS"}="H151 RSS"; $resn_exch{"2H15 RSS"}="H152 RSS";
$resn_exch{"3H15 RSS"}="H153 RSS"; $resn_exch{"1H16 RSS"}="H161 RSS"; $resn_exch{"2H16 RSS"}="H162 RSS";
$resn_exch{"3H16 RSS"}="H163 RSS"; $resn_exch{"1H17 RSS"}="H171 RSS"; $resn_exch{"2H17 RSS"}="H172 RSS";
$resn_exch{"3HOP  RT"}="HOP3  RT"; $resn_exch{"2HOP  RT"}="HOP2  RT"; $resn_exch{"1H5*  RT"}=" H5'  RT";
$resn_exch{"2H5*  RT"}="H5''  RT"; $resn_exch{" H4*  RT"}=" H4'  RT"; $resn_exch{" H3*  RT"}=" H3'  RT";
$resn_exch{"H3T   RT"}="HO3'  RT"; $resn_exch{" H2*  RT"}=" H2'  RT"; $resn_exch{"2HO*  RT"}="HO2'  RT";
$resn_exch{" H1*  RT"}=" H1'  RT"; $resn_exch{"1H5M  RT"}=" H71  RT"; $resn_exch{"2H5M  RT"}=" H72  RT";
$resn_exch{"3H5M  RT"}=" H73  RT"; $resn_exch{"1H7  RTA"}=" H71 RTA"; $resn_exch{"2H7  RTA"}=" H72 RTA";
$resn_exch{"1H8  RTA"}=" H81 RTA"; $resn_exch{"2H8  RTA"}=" H82 RTA"; $resn_exch{"1H9  RTA"}=" H91 RTA";
$resn_exch{"1H10 RTA"}="H101 RTA"; $resn_exch{"11H1 RTA"}="H111 RTA"; $resn_exch{"12H1 RTA"}="H112 RTA";
$resn_exch{"1H13 RTA"}="H131 RTA"; $resn_exch{"2H13 RTA"}="H132 RTA"; $resn_exch{"1H14 RTA"}="H141 RTA";
$resn_exch{"2H14 RTA"}="H142 RTA"; $resn_exch{"1H15 RTA"}="H151 RTA"; $resn_exch{"2H15 RTA"}="H152 RTA";
$resn_exch{"16H1 RTA"}="H116 RTA"; $resn_exch{"1H17 RTA"}="H171 RTA"; $resn_exch{"2H17 RTA"}="H172 RTA";
$resn_exch{"1H18 RTA"}="H181 RTA"; $resn_exch{"2H18 RTA"}="H182 RTA"; $resn_exch{"1H19 RTA"}="H191 RTA";
$resn_exch{"2H19 RTA"}="H192 RTA"; $resn_exch{"1H20 RTA"}="H201 RTA"; $resn_exch{"2H20 RTA"}="H202 RTA";
$resn_exch{"21H2 RTA"}="H221 RTA"; $resn_exch{"22H2 RTA"}="H222 RTA"; $resn_exch{"1H23 RTA"}="H231 RTA";
$resn_exch{"2H23 RTA"}="H232 RTA"; $resn_exch{"1H24 RTA"}="H241 RTA"; $resn_exch{"2H24 RTA"}="H242 RTA";
$resn_exch{"1H25 RTA"}="H251 RTA"; $resn_exch{"2H25 RTA"}="H252 RTA"; $resn_exch{"1H26 RTA"}="H261 RTA";
$resn_exch{"2H26 RTA"}="H262 RTA"; $resn_exch{"27H2 RTA"}="H227 RTA"; $resn_exch{"1H28 RTA"}="H281 RTA";
$resn_exch{"2H28 RTA"}="H282 RTA"; $resn_exch{"1H29 RTA"}="H291 RTA"; $resn_exch{"2H29 RTA"}="H292 RTA";
$resn_exch{"1H30 RTA"}="H301 RTA"; $resn_exch{"2H30 RTA"}="H302 RTA"; $resn_exch{"1H31 RTA"}="H311 RTA";
$resn_exch{"2H31 RTA"}="H312 RTA"; $resn_exch{"1H7  RTB"}=" H71 RTB"; $resn_exch{"1H8  RTB"}=" H81 RTB";
$resn_exch{"1H9  RTB"}=" H91 RTB"; $resn_exch{"1H13 RTB"}="H131 RTB"; $resn_exch{"1H14 RTB"}="H141 RTB";
$resn_exch{"1H15 RTB"}="H151 RTB"; $resn_exch{"1H17 RTB"}="H171 RTB"; $resn_exch{"1H18 RTB"}="H181 RTB";
$resn_exch{"1H19 RTB"}="H191 RTB"; $resn_exch{"1H23 RTB"}="H231 RTB"; $resn_exch{"1H24 RTB"}="H241 RTB";
$resn_exch{"1H25 RTB"}="H251 RTB"; $resn_exch{"1H26 RTB"}="H261 RTB"; $resn_exch{"1H28 RTB"}="H281 RTB";
$resn_exch{"1H29 RTB"}="H291 RTB"; $resn_exch{"1H30 RTB"}="H301 RTB"; $resn_exch{"1H31 RTB"}="H311 RTB";
$resn_exch{"1H32 RTB"}="H321 RTB"; $resn_exch{"1H33 RTB"}="H331 RTB"; $resn_exch{"1H2  RTL"}=" H21 RTL";
$resn_exch{"2H2  RTL"}=" H22 RTL"; $resn_exch{"1H3  RTL"}=" H31 RTL"; $resn_exch{"2H3  RTL"}=" H32 RTL";
$resn_exch{"1H4  RTL"}=" H41 RTL"; $resn_exch{"2H4  RTL"}=" H42 RTL"; $resn_exch{"1H15 RTL"}="H151 RTL";
$resn_exch{"2H15 RTL"}="H152 RTL"; $resn_exch{"1H16 RTL"}="H161 RTL"; $resn_exch{"2H16 RTL"}="H162 RTL";
$resn_exch{"3H16 RTL"}="H163 RTL"; $resn_exch{"1H17 RTL"}="H171 RTL"; $resn_exch{"2H17 RTL"}="H172 RTL";
$resn_exch{"3H17 RTL"}="H173 RTL"; $resn_exch{"1H18 RTL"}="H181 RTL"; $resn_exch{"2H18 RTL"}="H182 RTL";
$resn_exch{"3H18 RTL"}="H183 RTL"; $resn_exch{"1H19 RTL"}="H191 RTL"; $resn_exch{"2H19 RTL"}="H192 RTL";
$resn_exch{"3H19 RTL"}="H193 RTL"; $resn_exch{"1H20 RTL"}="H201 RTL"; $resn_exch{"2H20 RTL"}="H202 RTL";
$resn_exch{"3H20 RTL"}="H203 RTL"; $resn_exch{"2HOG RTP"}="HOG2 RTP"; $resn_exch{"3HOG RTP"}="HOG3 RTP";
$resn_exch{"2HOB RTP"}="HOB2 RTP"; $resn_exch{"2HOA RTP"}="HOA2 RTP"; $resn_exch{"1H5* RTP"}="H5'1 RTP";
$resn_exch{"2H5* RTP"}="H5'2 RTP"; $resn_exch{" H4* RTP"}=" H4' RTP"; $resn_exch{" H1* RTP"}=" H1' RTP";
$resn_exch{"1HN3 RTP"}="HN31 RTP"; $resn_exch{"2HN3 RTP"}="HN32 RTP"; $resn_exch{" H2* RTP"}=" H2' RTP";
$resn_exch{"*HO2 RTP"}="HO2' RTP"; $resn_exch{" H3* RTP"}=" H3' RTP"; $resn_exch{"*HO3 RTP"}="HO3' RTP";
$resn_exch{"1H30 RTR"}="H301 RTR"; $resn_exch{"2H30 RTR"}="H302 RTR"; $resn_exch{"1H21 RTR"}="H211 RTR";
$resn_exch{"2H21 RTR"}="H212 RTR"; $resn_exch{"1H12 RTR"}="H121 RTR"; $resn_exch{"2H12 RTR"}="H122 RTR";
$resn_exch{"1H13 RTR"}="H131 RTR"; $resn_exch{"2H13 RTR"}="H132 RTR"; $resn_exch{"1H17 RTR"}="H171 RTR";
$resn_exch{"2H17 RTR"}="H172 RTR"; $resn_exch{"1H30 RU6"}="H301 RU6"; $resn_exch{"2H30 RU6"}="H302 RU6";
$resn_exch{"3H30 RU6"}="H303 RU6"; $resn_exch{"1H31 RU6"}="H311 RU6"; $resn_exch{"2H31 RU6"}="H312 RU6";
$resn_exch{"3H31 RU6"}="H313 RU6"; $resn_exch{"1H25 RU6"}="H251 RU6"; $resn_exch{"2H25 RU6"}="H252 RU6";
$resn_exch{"1H27 RU6"}="H271 RU6"; $resn_exch{"2H27 RU6"}="H272 RU6"; $resn_exch{"3H27 RU6"}="H273 RU6";
$resn_exch{"1H28 RU6"}="H281 RU6"; $resn_exch{"2H28 RU6"}="H282 RU6"; $resn_exch{"3H28 RU6"}="H283 RU6";
$resn_exch{"1H29 RU6"}="H291 RU6"; $resn_exch{"2H29 RU6"}="H292 RU6"; $resn_exch{"3H29 RU6"}="H293 RU6";
$resn_exch{"1H32 RU6"}="H321 RU6"; $resn_exch{"2H32 RU6"}="H322 RU6"; $resn_exch{"3H32 RU6"}="H323 RU6";
$resn_exch{"1H7  RU6"}=" H71 RU6"; $resn_exch{"2H7  RU6"}=" H72 RU6"; $resn_exch{"1H33 RU6"}="H331 RU6";
$resn_exch{"2H33 RU6"}="H332 RU6"; $resn_exch{"3H33 RU6"}="H333 RU6"; $resn_exch{"1H35 RU6"}="H351 RU6";
$resn_exch{"2H35 RU6"}="H352 RU6"; $resn_exch{"3H35 RU6"}="H353 RU6"; $resn_exch{"1H38 RU6"}="H381 RU6";
$resn_exch{"2H38 RU6"}="H382 RU6"; $resn_exch{"3H38 RU6"}="H383 RU6"; $resn_exch{"1H40 RU6"}="H401 RU6";
$resn_exch{"2H40 RU6"}="H402 RU6"; $resn_exch{"1H41 RU6"}="H411 RU6"; $resn_exch{"2H41 RU6"}="H412 RU6";
$resn_exch{"1H42 RU6"}="H421 RU6"; $resn_exch{"2H42 RU6"}="H422 RU6"; $resn_exch{"1H43 RU6"}="H431 RU6";
$resn_exch{"2H43 RU6"}="H432 RU6"; $resn_exch{"1H34 RU6"}="H341 RU6"; $resn_exch{"2H34 RU6"}="H342 RU6";
$resn_exch{"3H34 RU6"}="H343 RU6"; $resn_exch{"1H36 RU6"}="H361 RU6"; $resn_exch{"2H36 RU6"}="H362 RU6";
$resn_exch{"1H37 RU6"}="H371 RU6"; $resn_exch{"2H37 RU6"}="H372 RU6"; $resn_exch{"3H37 RU6"}="H373 RU6";
$resn_exch{"1H15 RU6"}="H151 RU6"; $resn_exch{"2H15 RU6"}="H152 RU6"; $resn_exch{"1H20 RU6"}="H201 RU6";
$resn_exch{"2H20 RU6"}="H202 RU6"; $resn_exch{"3H20 RU6"}="H203 RU6"; $resn_exch{"1H19 RU6"}="H191 RU6";
$resn_exch{"2H19 RU6"}="H192 RU6"; $resn_exch{"3H19 RU6"}="H193 RU6"; $resn_exch{"1H21 RU6"}="H211 RU6";
$resn_exch{"2H21 RU6"}="H212 RU6"; $resn_exch{"3H21 RU6"}="H213 RU6"; $resn_exch{"1H10 RU7"}="H101 RU7";
$resn_exch{"2H10 RU7"}="H102 RU7"; $resn_exch{"3H10 RU7"}="H103 RU7"; $resn_exch{"1H9  RU7"}=" H91 RU7";
$resn_exch{"2H9  RU7"}=" H92 RU7"; $resn_exch{"3H9  RU7"}=" H93 RU7"; $resn_exch{"1H7  RU7"}=" H71 RU7";
$resn_exch{"2H7  RU7"}=" H72 RU7"; $resn_exch{"3H7  RU7"}=" H73 RU7"; $resn_exch{"1H38 RUA"}="H381 RUA";
$resn_exch{"2H38 RUA"}="H382 RUA"; $resn_exch{"3H38 RUA"}="H383 RUA"; $resn_exch{"1H18 RUA"}="H181 RUA";
$resn_exch{"19H1 RUA"}="H119 RUA"; $resn_exch{"1H15 RUA"}="H151 RUA"; $resn_exch{"1H24 RUA"}="H241 RUA";
$resn_exch{"1H23 RUA"}="H231 RUA"; $resn_exch{"20H2 RUA"}="H220 RUA"; $resn_exch{"1H21 RUA"}="H211 RUA";
$resn_exch{"22H2 RUA"}="H222 RUA"; $resn_exch{"1H39 RUA"}="H391 RUA"; $resn_exch{"2H39 RUA"}="H392 RUA";
$resn_exch{"3H39 RUA"}="H393 RUA"; $resn_exch{"1H3  RUA"}=" H31 RUA"; $resn_exch{"1H4  RUA"}=" H41 RUA";
$resn_exch{"1H5  RUA"}=" H51 RUA"; $resn_exch{"7H7  RUA"}=" H77 RUA"; $resn_exch{"1H6  RUA"}=" H61 RUA";
$resn_exch{"8H8  RUA"}=" H88 RUA"; $resn_exch{"1H9  RUA"}=" H91 RUA"; $resn_exch{"1H10 RUA"}="H101 RUA";
$resn_exch{"1H11 RUA"}="H111 RUA"; $resn_exch{"1H12 RUA"}="H121 RUA"; $resn_exch{"1H27 RUA"}="H271 RUA";
$resn_exch{"1H28 RUA"}="H281 RUA"; $resn_exch{"1H29 RUA"}="H291 RUA"; $resn_exch{"31H3 RUA"}="H331 RUA";
$resn_exch{"1H30 RUA"}="H301 RUA"; $resn_exch{"32H3 RUA"}="H332 RUA"; $resn_exch{"1H33 RUA"}="H33A RUA";
$resn_exch{"1H34 RUA"}="H341 RUA"; $resn_exch{"1H35 RUA"}="H351 RUA"; $resn_exch{"1H36 RUA"}="H361 RUA";
$resn_exch{"1H1  RUB"}=" H11 RUB"; $resn_exch{"2H1  RUB"}=" H12 RUB"; $resn_exch{"1H5  RUB"}=" H51 RUB";
$resn_exch{"2H5  RUB"}=" H52 RUB"; $resn_exch{"2HOP RUB"}="HOP2 RUB"; $resn_exch{"3HOP RUB"}="HOP3 RUB";
$resn_exch{"5HOP RUB"}="HOP5 RUB"; $resn_exch{"6HOP RUB"}="HOP6 RUB"; $resn_exch{" H2\" RUN"}=" H2' RUN";
$resn_exch{"1H3\" RUN"}="H3'1 RUN"; $resn_exch{"2H3\" RUN"}="H3'2 RUN"; $resn_exch{"3H3\" RUN"}="H3'3 RUN";
$resn_exch{"1H4\" RUN"}="H4'1 RUN"; $resn_exch{"2H4\" RUN"}="H4'2 RUN"; $resn_exch{"3H4\" RUN"}="H4'3 RUN";
$resn_exch{"1H2  RUN"}=" H21 RUN"; $resn_exch{"2H2  RUN"}=" H22 RUN"; $resn_exch{"1H11 RUN"}="H111 RUN";
$resn_exch{"2H11 RUN"}="H112 RUN"; $resn_exch{"3H11 RUN"}="H113 RUN"; $resn_exch{"1H12 RUN"}="H121 RUN";
$resn_exch{"2H12 RUN"}="H122 RUN"; $resn_exch{"3H12 RUN"}="H123 RUN"; $resn_exch{"1HA  RUN"}=" HA1 RUN";
$resn_exch{"2HA  RUN"}=" HA2 RUN"; $resn_exch{"1HB  RUN"}=" HB1 RUN"; $resn_exch{"2HB  RUN"}=" HB2 RUN";
$resn_exch{"1HC  RUN"}=" HC1 RUN"; $resn_exch{"2HC  RUN"}=" HC2 RUN"; $resn_exch{"1H6  RUP"}=" H61 RUP";
$resn_exch{"2H6  RUP"}=" H62 RUP"; $resn_exch{"1H28 RUP"}="H281 RUP"; $resn_exch{"2H28 RUP"}="H282 RUP";
$resn_exch{"1H29 RUP"}="H291 RUP"; $resn_exch{"2H29 RUP"}="H292 RUP"; $resn_exch{"1H15 RUP"}="H151 RUP";
$resn_exch{"2H15 RUP"}="H152 RUP"; $resn_exch{"1H25 RUP"}="H251 RUP"; $resn_exch{"2H25 RUP"}="H252 RUP";
$resn_exch{"1H8  RUP"}=" H81 RUP"; $resn_exch{"2H8  RUP"}=" H82 RUP"; $resn_exch{"1H11 RUP"}="H111 RUP";
$resn_exch{"2H11 RUP"}="H112 RUP"; $resn_exch{"1H5  RUP"}=" H51 RUP"; $resn_exch{"2H5  RUP"}=" H52 RUP";
$resn_exch{"1H3  RUP"}=" H31 RUP"; $resn_exch{"2H3  RUP"}="H32A RUP"; $resn_exch{"1H2  RUP"}="H21A RUP";
$resn_exch{"2H2  RUP"}="H22A RUP"; $resn_exch{"1H21 RUT"}="H211 RUT"; $resn_exch{"2H21 RUT"}="H212 RUT";
$resn_exch{"1H27 RUT"}="H271 RUT"; $resn_exch{"2H27 RUT"}="H272 RUT"; $resn_exch{"3H27 RUT"}="H273 RUT";
$resn_exch{"1HOP RVP"}="HOP1 RVP"; $resn_exch{"2HOP RVP"}="HOP2 RVP"; $resn_exch{"1H5* RVP"}="H5'1 RVP";
$resn_exch{"2H5* RVP"}="H5'2 RVP"; $resn_exch{" H4* RVP"}=" H4' RVP"; $resn_exch{" H3* RVP"}=" H3' RVP";
$resn_exch{"3HO* RVP"}="HO'3 RVP"; $resn_exch{" H2* RVP"}=" H2' RVP"; $resn_exch{"2HO* RVP"}="HO'2 RVP";
$resn_exch{" H1* RVP"}=" H1' RVP"; $resn_exch{"1HN  RVP"}=" HN1 RVP"; $resn_exch{"2HN  RVP"}=" HN2 RVP";
$resn_exch{"1H3C RWF"}="H3C1 RWF"; $resn_exch{"2H3C RWF"}="H3C2 RWF"; $resn_exch{"1H5C RWF"}="H5C1 RWF";
$resn_exch{"2H5C RWF"}="H5C2 RWF"; $resn_exch{"3H5C RWF"}="H5C3 RWF"; $resn_exch{"1H51 RWJ"}="H511 RWJ";
$resn_exch{"1H52 RWJ"}="H521 RWJ"; $resn_exch{"2H52 RWJ"}="H522 RWJ"; $resn_exch{"1H53 RWJ"}="H531 RWJ";
$resn_exch{"2H53 RWJ"}="H532 RWJ"; $resn_exch{"1H54 RWJ"}="H541 RWJ"; $resn_exch{"2H54 RWJ"}="H542 RWJ";
$resn_exch{"1H55 RWJ"}="H551 RWJ"; $resn_exch{"2H55 RWJ"}="H552 RWJ"; $resn_exch{"1NH  RWJ"}=" H1N RWJ";
$resn_exch{"1HA  RWJ"}=" HA1 RWJ"; $resn_exch{"1HB  RWJ"}=" HB1 RWJ"; $resn_exch{"2HB  RWJ"}=" HB2 RWJ";
$resn_exch{"1HG  RWJ"}=" HG1 RWJ"; $resn_exch{"2HG  RWJ"}=" HG2 RWJ"; $resn_exch{"1HD  RWJ"}=" HD1 RWJ";
$resn_exch{"2HD  RWJ"}=" HD2 RWJ"; $resn_exch{"1HE  RWJ"}=" HE1 RWJ"; $resn_exch{"1HN1 RWJ"}="HN11 RWJ";
$resn_exch{"1HN2 RWJ"}="HN21 RWJ"; $resn_exch{"2HN2 RWJ"}="HN22 RWJ"; $resn_exch{"1H63 RWJ"}="H631 RWJ";
$resn_exch{"1H64 RWJ"}="H641 RWJ"; $resn_exch{"1H65 RWJ"}="H651 RWJ"; $resn_exch{"1H66 RWJ"}="H661 RWJ";
$resn_exch{"1H25 RX3"}="H251 RX3"; $resn_exch{"2H25 RX3"}="H252 RX3"; $resn_exch{"1H20 RX3"}="H201 RX3";
$resn_exch{"2H20 RX3"}="H202 RX3"; $resn_exch{"1H21 RX3"}="H211 RX3"; $resn_exch{"2H21 RX3"}="H212 RX3";
$resn_exch{"1H22 RX3"}="H221 RX3"; $resn_exch{"2H22 RX3"}="H222 RX3"; $resn_exch{"1H10 RX3"}="H101 RX3";
$resn_exch{"2H10 RX3"}="H102 RX3"; $resn_exch{"1H2  RX3"}=" H21 RX3"; $resn_exch{"2H2  RX3"}=" H22 RX3";
$resn_exch{"1HN1 RXP"}="HN11 RXP"; $resn_exch{"2HN2 RXP"}="HN22 RXP"; $resn_exch{"1H2  RXP"}=" H21 RXP";
$resn_exch{"2H2  RXP"}="H22A RXP"; $resn_exch{"1H13 RXP"}="H131 RXP"; $resn_exch{"2H13 RXP"}="H132 RXP";
$resn_exch{"1H38 RXP"}="H381 RXP"; $resn_exch{"2H38 RXP"}="H382 RXP"; $resn_exch{"1H39 RXP"}="H391 RXP";
$resn_exch{"2H39 RXP"}="H392 RXP"; $resn_exch{"1H21 RXP"}="H211 RXP"; $resn_exch{"2H21 RXP"}="H212 RXP";
$resn_exch{"1H23 RXP"}="H231 RXP"; $resn_exch{"2H23 RXP"}="H232 RXP"; $resn_exch{"1H31 RXP"}="H311 RXP";
$resn_exch{"2H31 RXP"}="H312 RXP"; $resn_exch{"1H71 RYU"}="H711 RYU"; $resn_exch{"2H71 RYU"}="H712 RYU";
$resn_exch{"3H71 RYU"}="H713 RYU"; $resn_exch{"1H21 RYU"}="H211 RYU"; $resn_exch{"2H21 RYU"}="H212 RYU";
$resn_exch{"1H22 RYU"}="H221 RYU"; $resn_exch{"2H22 RYU"}="H222 RYU"; $resn_exch{"1H24 RYU"}="H241 RYU";
$resn_exch{"2H24 RYU"}="H242 RYU"; $resn_exch{"1H25 RYU"}="H251 RYU"; $resn_exch{"2H25 RYU"}="H252 RYU";
$resn_exch{"1H11 S01"}="H111 S01"; $resn_exch{"2H11 S01"}="H112 S01"; $resn_exch{"1H14 S01"}="H141 S01";
$resn_exch{"2H14 S01"}="H142 S01"; $resn_exch{"1H23 S01"}="H231 S01"; $resn_exch{"2H23 S01"}="H232 S01";
$resn_exch{"3H23 S01"}="H233 S01"; $resn_exch{"1H31 S01"}="H311 S01"; $resn_exch{"2H31 S01"}="H312 S01";
$resn_exch{"1H34 S01"}="H341 S01"; $resn_exch{"2H34 S01"}="H342 S01"; $resn_exch{"1H38 S01"}="H381 S01";
$resn_exch{"2H38 S01"}="H382 S01"; $resn_exch{"1H41 S01"}="H411 S01"; $resn_exch{"2H41 S01"}="H412 S01";
$resn_exch{"1H46 S01"}="H461 S01"; $resn_exch{"2H46 S01"}="H462 S01"; $resn_exch{"2HOP S02"}="HOP2 S02";
$resn_exch{"3HOP S02"}="HOP3 S02"; $resn_exch{"1H5* S02"}="H5'1 S02"; $resn_exch{"2H5* S02"}="H5'2 S02";
$resn_exch{"1HA  S02"}=" HA1 S02"; $resn_exch{"2HA  S02"}=" HA2 S02"; $resn_exch{"1H6A S02"}="H6A1 S02";
$resn_exch{"2H6A S02"}="H6A2 S02"; $resn_exch{"1H2A S02"}="H2A1 S02"; $resn_exch{"2H2A S02"}="H2A2 S02";
$resn_exch{"1H3A S02"}="H3A1 S02"; $resn_exch{"2H3A S02"}="H3A2 S02"; $resn_exch{"1H5A S02"}="H5A1 S02";
$resn_exch{"2H5A S02"}="H5A2 S02"; $resn_exch{"1H6B S02"}="H6B1 S02"; $resn_exch{"2H6B S02"}="H6B2 S02";
$resn_exch{"1H2B S02"}="H2B1 S02"; $resn_exch{"2H2B S02"}="H2B2 S02"; $resn_exch{"1H3B S02"}="H3B1 S02";
$resn_exch{"2H3B S02"}="H3B2 S02"; $resn_exch{"1H5B S02"}="H5B1 S02"; $resn_exch{"2H5B S02"}="H5B2 S02";
$resn_exch{"1HD* S02"}="HD'1 S02"; $resn_exch{"2HD* S02"}="HD'2 S02"; $resn_exch{"1H3* S02"}="H3'1 S02";
$resn_exch{"2H3* S02"}="H3'2 S02"; $resn_exch{"HO3* S02"}="HO3' S02"; $resn_exch{"1H33 S03"}="H331 S03";
$resn_exch{"2H33 S03"}="H332 S03"; $resn_exch{"1H17 S03"}="H171 S03"; $resn_exch{"2H17 S03"}="H172 S03";
$resn_exch{"3H17 S03"}="H173 S03"; $resn_exch{"1H24 S03"}="H241 S03"; $resn_exch{"2H24 S03"}="H242 S03";
$resn_exch{"1H27 S03"}="H271 S03"; $resn_exch{"2H27 S03"}="H272 S03"; $resn_exch{"1H28 S03"}="H281 S03";
$resn_exch{"2H28 S03"}="H282 S03"; $resn_exch{"1H31 S03"}="H311 S03"; $resn_exch{"2H31 S03"}="H312 S03";
$resn_exch{"1H30 S03"}="H301 S03"; $resn_exch{"2H30 S03"}="H302 S03"; $resn_exch{"1H2  S0H"}=" H21 S0H";
$resn_exch{"2H2  S0H"}=" H22 S0H"; $resn_exch{"1H1  S0H"}=" H11 S0H"; $resn_exch{"2H1  S0H"}=" H12 S0H";
$resn_exch{"3H1  S0H"}="H13A S0H"; $resn_exch{"1H12 S10"}="H121 S10"; $resn_exch{"2H12 S10"}="H122 S10";
$resn_exch{"1H13 S10"}="H131 S10"; $resn_exch{"2H13 S10"}="H132 S10"; $resn_exch{"3H13 S10"}="H133 S10";
$resn_exch{"1H8C S10"}="H8C1 S10"; $resn_exch{"2H8C S10"}="H8C2 S10"; $resn_exch{"1H7C S10"}="H7C1 S10";
$resn_exch{"2H7C S10"}="H7C2 S10"; $resn_exch{"1H6C S10"}="H6C1 S10"; $resn_exch{"2H6C S10"}="H6C2 S10";
$resn_exch{"1H2C S10"}="H2C1 S10"; $resn_exch{"2H2C S10"}="H2C2 S10"; $resn_exch{"1H1C S10"}="H1C1 S10";
$resn_exch{"2H1C S10"}="H1C2 S10"; $resn_exch{"3H1C S10"}="H1C3 S10"; $resn_exch{"1H13 S11"}="H131 S11";
$resn_exch{"2H13 S11"}="H132 S11"; $resn_exch{"3H13 S11"}="H133 S11"; $resn_exch{"1H14 S11"}="H141 S11";
$resn_exch{"2H14 S11"}="H142 S11"; $resn_exch{"3H14 S11"}="H143 S11"; $resn_exch{"1H15 S11"}="H151 S11";
$resn_exch{"2H15 S11"}="H152 S11"; $resn_exch{"1HA  S11"}=" HA1 S11"; $resn_exch{"2HA  S11"}=" HA2 S11";
$resn_exch{"1H24 S11"}="H241 S11"; $resn_exch{"2H24 S11"}="H242 S11"; $resn_exch{"1H30 S14"}="H301 S14";
$resn_exch{"2H30 S14"}="H302 S14"; $resn_exch{"1H35 S14"}="H351 S14"; $resn_exch{"2H35 S14"}="H352 S14";
$resn_exch{"3H35 S14"}="H353 S14"; $resn_exch{"1H39 S14"}="H391 S14"; $resn_exch{"2H39 S14"}="H392 S14";
$resn_exch{"3H39 S14"}="H393 S14"; $resn_exch{"1H43 S14"}="H431 S14"; $resn_exch{"2H43 S14"}="H432 S14";
$resn_exch{"1H45 S14"}="H451 S14"; $resn_exch{"2H45 S14"}="H452 S14"; $resn_exch{"1H46 S14"}="H461 S14";
$resn_exch{"2H46 S14"}="H462 S14"; $resn_exch{"1HBC S17"}="HBC1 S17"; $resn_exch{"2HBC S17"}="HBC2 S17";
$resn_exch{"1HAA S17"}="HAA1 S17"; $resn_exch{"2HAA S17"}="HAA2 S17"; $resn_exch{"3HAA S17"}="HAA3 S17";
$resn_exch{"1HAP S17"}="HAP1 S17"; $resn_exch{"2HAP S17"}="HAP2 S17"; $resn_exch{"1HAN S17"}="HAN1 S17";
$resn_exch{"2HAN S17"}="HAN2 S17"; $resn_exch{"1HAL S17"}="HAL1 S17"; $resn_exch{"2HAL S17"}="HAL2 S17";
$resn_exch{"1HAK S17"}="HAK1 S17"; $resn_exch{"2HAK S17"}="HAK2 S17"; $resn_exch{"1HAM S17"}="HAM1 S17";
$resn_exch{"2HAM S17"}="HAM2 S17"; $resn_exch{"1HAO S17"}="HAO1 S17"; $resn_exch{"2HAO S17"}="HAO2 S17";
$resn_exch{"1H13 S1A"}="H131 S1A"; $resn_exch{"2H13 S1A"}="H132 S1A"; $resn_exch{"1H14 S1A"}="H141 S1A";
$resn_exch{"2H14 S1A"}="H142 S1A"; $resn_exch{"1H15 S1A"}="H151 S1A"; $resn_exch{"2H15 S1A"}="H152 S1A";
$resn_exch{"1H16 S1A"}="H161 S1A"; $resn_exch{"2H16 S1A"}="H162 S1A"; $resn_exch{"1H19 S1A"}="H191 S1A";
$resn_exch{"2H19 S1A"}="H192 S1A"; $resn_exch{"3H19 S1A"}="H193 S1A"; $resn_exch{"1H20 S1A"}="H201 S1A";
$resn_exch{"2H20 S1A"}="H202 S1A"; $resn_exch{"3H20 S1A"}="H203 S1A"; $resn_exch{"1H21 S1A"}="H211 S1A";
$resn_exch{"2H21 S1A"}="H212 S1A"; $resn_exch{"3H21 S1A"}="H213 S1A"; $resn_exch{"1H22 S1A"}="H221 S1A";
$resn_exch{"2H22 S1A"}="H222 S1A"; $resn_exch{"3H22 S1A"}="H223 S1A"; $resn_exch{"1H23 S1A"}="H231 S1A";
$resn_exch{"2H23 S1A"}="H232 S1A"; $resn_exch{"3H23 S1A"}="H233 S1A"; $resn_exch{"1H24 S1A"}="H241 S1A";
$resn_exch{"2H24 S1A"}="H242 S1A"; $resn_exch{"3H24 S1A"}="H243 S1A"; $resn_exch{" HN2 S1H"}=" H2  S1H";
$resn_exch{"1HB  S1H"}=" HB2 S1H"; $resn_exch{"2HB  S1H"}=" HB3 S1H"; $resn_exch{"1H1  S1H"}=" H11 S1H";
$resn_exch{"2H1  S1H"}=" H12 S1H"; $resn_exch{"1H2  S1H"}=" H21 S1H"; $resn_exch{"2H2  S1H"}=" H22 S1H";
$resn_exch{"1H3  S1H"}=" H31 S1H"; $resn_exch{"2H3  S1H"}=" H32 S1H"; $resn_exch{"1H4  S1H"}=" H41 S1H";
$resn_exch{"2H4  S1H"}=" H42 S1H"; $resn_exch{"1H5  S1H"}=" H51 S1H"; $resn_exch{"2H5  S1H"}=" H52 S1H";
$resn_exch{"1H6  S1H"}=" H61 S1H"; $resn_exch{"2H6  S1H"}=" H62 S1H"; $resn_exch{"1H7  S1H"}=" H71 S1H";
$resn_exch{"2H7  S1H"}=" H72 S1H"; $resn_exch{"1H8  S1H"}=" H81 S1H"; $resn_exch{"2H8  S1H"}=" H82 S1H";
$resn_exch{"1H9  S1H"}=" H91 S1H"; $resn_exch{"2H9  S1H"}=" H92 S1H"; $resn_exch{"1H10 S1H"}="H101 S1H";
$resn_exch{"2H10 S1H"}="H102 S1H"; $resn_exch{"1H11 S1H"}="H111 S1H"; $resn_exch{"2H11 S1H"}="H112 S1H";
$resn_exch{"1H12 S1H"}="H121 S1H"; $resn_exch{"2H12 S1H"}="H122 S1H"; $resn_exch{"1H13 S1H"}="H131 S1H";
$resn_exch{"2H13 S1H"}="H132 S1H"; $resn_exch{"1H14 S1H"}="H141 S1H"; $resn_exch{"2H14 S1H"}="H142 S1H";
$resn_exch{"1H15 S1H"}="H151 S1H"; $resn_exch{"2H15 S1H"}="H152 S1H"; $resn_exch{"1H16 S1H"}="H161 S1H";
$resn_exch{"2H16 S1H"}="H162 S1H"; $resn_exch{"3H16 S1H"}="H163 S1H"; $resn_exch{"1H39 S1S"}="H391 S1S";
$resn_exch{"2H39 S1S"}="H392 S1S"; $resn_exch{"1H12 S1S"}="H121 S1S"; $resn_exch{"2H12 S1S"}="H122 S1S";
$resn_exch{"1H32 S1S"}="H321 S1S"; $resn_exch{"2H32 S1S"}="H322 S1S"; $resn_exch{"1H33 S1S"}="H331 S1S";
$resn_exch{"2H33 S1S"}="H332 S1S"; $resn_exch{"1H34 S1S"}="H341 S1S"; $resn_exch{"2H34 S1S"}="H342 S1S";
$resn_exch{"1H35 S1S"}="H351 S1S"; $resn_exch{"2H35 S1S"}="H352 S1S"; $resn_exch{"1H36 S1S"}="H361 S1S";
$resn_exch{"2H36 S1S"}="H362 S1S"; $resn_exch{"1H28 S1S"}="H281 S1S"; $resn_exch{"2H28 S1S"}="H282 S1S";
$resn_exch{"1HN1 S1S"}="HN11 S1S"; $resn_exch{"2HN1 S1S"}="HN12 S1S"; $resn_exch{"1H25 S1S"}="H251 S1S";
$resn_exch{"2H25 S1S"}="H252 S1S"; $resn_exch{"1H10 S1S"}="H101 S1S"; $resn_exch{"2H10 S1S"}="H102 S1S";
$resn_exch{"1H6  S1T"}=" H61 S1T"; $resn_exch{"2H6  S1T"}=" H62 S1T"; $resn_exch{"1H7  S1T"}=" H71 S1T";
$resn_exch{"2H7  S1T"}=" H72 S1T"; $resn_exch{"1HM4 S1T"}="HM41 S1T"; $resn_exch{"2HM4 S1T"}="HM42 S1T";
$resn_exch{"3HM4 S1T"}="HM43 S1T"; $resn_exch{"1H7' S1T"}="H7'1 S1T"; $resn_exch{"2H7' S1T"}="H7'2 S1T";
$resn_exch{"1HM2 S1T"}="HM21 S1T"; $resn_exch{"2HM2 S1T"}="HM22 S1T"; $resn_exch{"3HM2 S1T"}="HM23 S1T";
$resn_exch{"1H4' S1T"}="H4'1 S1T"; $resn_exch{"2H4' S1T"}="H4'2 S1T"; $resn_exch{"1HP2 S1T"}="HP21 S1T";
$resn_exch{"2HP2 S1T"}="HP22 S1T"; $resn_exch{"3HP2 S1T"}="HP23 S1T"; $resn_exch{"1H9  S27"}=" H91 S27";
$resn_exch{"2H9  S27"}=" H92 S27"; $resn_exch{"3H9  S27"}=" H93 S27"; $resn_exch{"1H11 S27"}="H111 S27";
$resn_exch{"2H11 S27"}="H112 S27"; $resn_exch{"1H7  S27"}=" H71 S27"; $resn_exch{"2H7  S27"}=" H72 S27";
$resn_exch{"1H10 S27"}="H101 S27"; $resn_exch{"2H10 S27"}="H102 S27"; $resn_exch{"3H10 S27"}="H103 S27";
$resn_exch{"1H23 S27"}="H231 S27"; $resn_exch{"2H23 S27"}="H232 S27"; $resn_exch{"3H23 S27"}="H233 S27";
$resn_exch{"1H31 S2A"}="H311 S2A"; $resn_exch{"2H31 S2A"}="H312 S2A"; $resn_exch{"3H31 S2A"}="H313 S2A";
$resn_exch{"1HN  S2A"}=" HN1 S2A"; $resn_exch{"2HN  S2A"}=" HN2 S2A"; $resn_exch{"3HN  S2A"}=" HN3 S2A";
$resn_exch{"1HM  S2A"}=" HM1 S2A"; $resn_exch{"2HM  S2A"}=" HM2 S2A"; $resn_exch{"3HM  S2A"}=" HM3 S2A";
$resn_exch{"1HB  S2C"}=" HB1 S2C"; $resn_exch{"2HB  S2C"}=" HB2 S2C"; $resn_exch{"1HD  S2C"}=" HD1 S2C";
$resn_exch{"2HD  S2C"}=" HD2 S2C"; $resn_exch{"1HE  S2C"}=" HE1 S2C"; $resn_exch{"2HE  S2C"}=" HE2 S2C";
$resn_exch{"1H3C S2D"}="H3C1 S2D"; $resn_exch{"2H3C S2D"}="H3C2 S2D"; $resn_exch{"3H3C S2D"}="H3C3 S2D";
$resn_exch{"1H5M S2M"}=" H71 S2M"; $resn_exch{"2H5M S2M"}=" H72 S2M"; $resn_exch{"3H5M S2M"}=" H73 S2M";
$resn_exch{"1H5* S2M"}=" H5' S2M"; $resn_exch{"2H5* S2M"}="H5'' S2M"; $resn_exch{"2HOP S2M"}="HOP2 S2M";
$resn_exch{" H4* S2M"}=" H4' S2M"; $resn_exch{" H1* S2M"}=" H1' S2M"; $resn_exch{" H2* S2M"}=" H2' S2M";
$resn_exch{" H3* S2M"}=" H3' S2M"; $resn_exch{"H3T  S2M"}="HO3' S2M"; $resn_exch{"1HA* S2M"}="HA'1 S2M";
$resn_exch{"2HA* S2M"}="HA'2 S2M"; $resn_exch{"1HB* S2M"}="HB'1 S2M"; $resn_exch{"2HB* S2M"}="HB'2 S2M";
$resn_exch{"1HD* S2M"}="HD'1 S2M"; $resn_exch{"2HD* S2M"}="HD'2 S2M"; $resn_exch{"3HD* S2M"}="HD'3 S2M";
$resn_exch{"3HOP S2M"}="HOP3 S2M"; $resn_exch{"1H6  S3P"}=" H61 S3P"; $resn_exch{"2H6  S3P"}=" H62 S3P";
$resn_exch{"1HAB S48"}="HAB1 S48"; $resn_exch{"2HAB S48"}="HAB2 S48"; $resn_exch{"3HAB S48"}="HAB3 S48";
$resn_exch{"1HA  S48"}=" HA1 S48"; $resn_exch{"2HA  S48"}=" HA2 S48"; $resn_exch{"1HAA S48"}="HAA1 S48";
$resn_exch{"2HAA S48"}="HAA2 S48"; $resn_exch{"3HAA S48"}="HAA3 S48"; $resn_exch{"1HAV S48"}="HAV1 S48";
$resn_exch{"2HAV S48"}="HAV2 S48"; $resn_exch{"1HAS S48"}="HAS1 S48"; $resn_exch{"2HAS S48"}="HAS2 S48";
$resn_exch{"1HAU S48"}="HAU1 S48"; $resn_exch{"2HAU S48"}="HAU2 S48"; $resn_exch{"1HAC S48"}="HAC1 S48";
$resn_exch{"2HAC S48"}="HAC2 S48"; $resn_exch{"3HAC S48"}="HAC3 S48"; $resn_exch{" H1* S4C"}=" H1' S4C";
$resn_exch{"1H4  S4C"}=" H41 S4C"; $resn_exch{"2H4  S4C"}=" H42 S4C"; $resn_exch{" H2* S4C"}=" H2' S4C";
$resn_exch{" H3* S4C"}=" H3' S4C"; $resn_exch{" H4* S4C"}=" H4' S4C"; $resn_exch{"1H5* S4C"}=" H5' S4C";
$resn_exch{"2H5* S4C"}="H5'' S4C"; $resn_exch{"1HA  S4M"}=" HA1 S4M"; $resn_exch{"2HA  S4M"}=" HA2 S4M";
$resn_exch{"1HN  S4M"}=" HN1 S4M"; $resn_exch{"2HN  S4M"}=" HN2 S4M"; $resn_exch{"1HB  S4M"}=" HB1 S4M";
$resn_exch{"2HB  S4M"}=" HB2 S4M"; $resn_exch{"1HG  S4M"}=" HG1 S4M"; $resn_exch{"2HG  S4M"}=" HG2 S4M";
$resn_exch{"1HE  S4M"}=" HE1 S4M"; $resn_exch{"2HE  S4M"}=" HE2 S4M"; $resn_exch{"3HE  S4M"}=" HE3 S4M";
$resn_exch{"1H5* S4M"}="H5'1 S4M"; $resn_exch{"2H5* S4M"}="H5'2 S4M"; $resn_exch{" H4* S4M"}=" H4' S4M";
$resn_exch{" H1* S4M"}=" H1' S4M"; $resn_exch{" H2* S4M"}=" H2' S4M"; $resn_exch{"HO2* S4M"}="HO2' S4M";
$resn_exch{" H3* S4M"}=" H3' S4M"; $resn_exch{"HO3* S4M"}="HO3' S4M"; $resn_exch{"1HN6 S4M"}="HN61 S4M";
$resn_exch{"2HN6 S4M"}="HN62 S4M"; $resn_exch{"1HOP S4U"}="HOP1 S4U"; $resn_exch{"2HOP S4U"}="HOP2 S4U";
$resn_exch{"1H5' S4U"}="H5'1 S4U"; $resn_exch{"2H5' S4U"}="H5'2 S4U"; $resn_exch{"'HO2 S4U"}="HO2' S4U";
$resn_exch{"'HO3 S4U"}="HO3' S4U"; $resn_exch{"1H23 S55"}="H231 S55"; $resn_exch{"2H23 S55"}="H232 S55";
$resn_exch{"3H23 S55"}="H233 S55"; $resn_exch{"1H33 S55"}="H331 S55"; $resn_exch{"2H33 S55"}="H332 S55";
$resn_exch{"1H25 S55"}="H251 S55"; $resn_exch{"2H25 S55"}="H252 S55"; $resn_exch{"1H26 S55"}="H261 S55";
$resn_exch{"2H26 S55"}="H262 S55"; $resn_exch{"3H26 S55"}="H263 S55"; $resn_exch{"1H14 S55"}="H141 S55";
$resn_exch{"2H14 S55"}="H142 S55"; $resn_exch{"1H11 S55"}="H111 S55"; $resn_exch{"2H11 S55"}="H112 S55";
$resn_exch{"1H10 S55"}="H101 S55"; $resn_exch{"2H10 S55"}="H102 S55"; $resn_exch{"1H1  S57"}=" H11 S57";
$resn_exch{"2H1  S57"}=" H12 S57"; $resn_exch{"3H1  S57"}=" H13 S57"; $resn_exch{"1H6B S57"}="H6B1 S57";
$resn_exch{"2H6B S57"}="H6B2 S57"; $resn_exch{"1H5B S57"}="H5B1 S57"; $resn_exch{"2H5B S57"}="H5B2 S57";
$resn_exch{"1H4B S57"}="H4B1 S57"; $resn_exch{"2H4B S57"}="H4B2 S57"; $resn_exch{"1H3B S57"}="H3B1 S57";
$resn_exch{"2H3B S57"}="H3B2 S57"; $resn_exch{"1H2B S57"}="H2B1 S57"; $resn_exch{"2H2B S57"}="H2B2 S57";
$resn_exch{"1H1B S57"}="H1B1 S57"; $resn_exch{"2H1B S57"}="H1B2 S57"; $resn_exch{"1HN3 S58"}="HN31 S58";
$resn_exch{"2HN3 S58"}="HN32 S58"; $resn_exch{"1H1  S5H"}=" H11 S5H"; $resn_exch{"2H1  S5H"}=" H12 S5H";
$resn_exch{"1H2  S5H"}=" H21 S5H"; $resn_exch{"2H2  S5H"}=" H22 S5H"; $resn_exch{"1H4  S5H"}=" H41 S5H";
$resn_exch{"2H4  S5H"}=" H42 S5H"; $resn_exch{"1H6  S5H"}=" H61 S5H"; $resn_exch{"2H6  S5H"}=" H62 S5H";
$resn_exch{"1H7  S5H"}=" H71 S5H"; $resn_exch{"2H7  S5H"}=" H72 S5H"; $resn_exch{"1H11 S5H"}="H111 S5H";
$resn_exch{"2H11 S5H"}="H112 S5H"; $resn_exch{"1H12 S5H"}="H121 S5H"; $resn_exch{"2H12 S5H"}="H122 S5H";
$resn_exch{"1H15 S5H"}="H151 S5H"; $resn_exch{"2H15 S5H"}="H152 S5H"; $resn_exch{"1H16 S5H"}="H161 S5H";
$resn_exch{"2H16 S5H"}="H162 S5H"; $resn_exch{"1H18 S5H"}="H181 S5H"; $resn_exch{"2H18 S5H"}="H182 S5H";
$resn_exch{"3H18 S5H"}="H183 S5H"; $resn_exch{"1H19 S5H"}="H191 S5H"; $resn_exch{"2H19 S5H"}="H192 S5H";
$resn_exch{"3H19 S5H"}="H193 S5H"; $resn_exch{"1H21 S5H"}="H211 S5H"; $resn_exch{"2H21 S5H"}="H212 S5H";
$resn_exch{"3H21 S5H"}="H213 S5H"; $resn_exch{"1HH2 S5H"}="HH21 S5H"; $resn_exch{"2HH2 S5H"}="HH22 S5H";
$resn_exch{"1HH3 S5H"}="HH31 S5H"; $resn_exch{"2HH3 S5H"}="HH32 S5H"; $resn_exch{"5HOH S5H"}="HOH5 S5H";
$resn_exch{"2HOP S6G"}="HOP2 S6G"; $resn_exch{"3HOP S6G"}="HOP3 S6G"; $resn_exch{"1H5* S6G"}=" H5' S6G";
$resn_exch{"2H5* S6G"}="H5'' S6G"; $resn_exch{" H4* S6G"}=" H4' S6G"; $resn_exch{" H3* S6G"}=" H3' S6G";
$resn_exch{"*HO3 S6G"}="HO3' S6G"; $resn_exch{"1H2* S6G"}=" H2' S6G"; $resn_exch{"2H2* S6G"}="H2'' S6G";
$resn_exch{" H1* S6G"}=" H1' S6G"; $resn_exch{"1HN2 S6G"}="HN21 S6G"; $resn_exch{"2HN2 S6G"}="HN22 S6G";
$resn_exch{"1H1  S6P"}=" H11 S6P"; $resn_exch{"2H1  S6P"}=" H12 S6P"; $resn_exch{"1H6  S6P"}=" H61 S6P";
$resn_exch{"2H6  S6P"}=" H62 S6P"; $resn_exch{"1H1  S70"}=" H11 S70"; $resn_exch{"2H1  S70"}=" H12 S70";
$resn_exch{"1H4  S70"}=" H41 S70"; $resn_exch{"2H4  S70"}=" H42 S70"; $resn_exch{"1HN1 S79"}="HN11 S79";
$resn_exch{"2HN1 S79"}="HN12 S79"; $resn_exch{"1H17 S79"}="H171 S79"; $resn_exch{"2H17 S79"}="H172 S79";
$resn_exch{"1H9  S80"}=" H91 S80"; $resn_exch{"2H9  S80"}=" H92 S80"; $resn_exch{"1H8  S80"}=" H81 S80";
$resn_exch{"2H8  S80"}=" H82 S80"; $resn_exch{"1H7  S80"}=" H71 S80"; $resn_exch{"2H7  S80"}=" H72 S80";
$resn_exch{"1H6  S80"}=" H61 S80"; $resn_exch{"2H6  S80"}=" H62 S80"; $resn_exch{"1H11 S80"}="H111 S80";
$resn_exch{"2H11 S80"}="H112 S80"; $resn_exch{"1H12 S80"}="H121 S80"; $resn_exch{"2H12 S80"}="H122 S80";
$resn_exch{"1H13 S80"}="H131 S80"; $resn_exch{"2H13 S80"}="H132 S80"; $resn_exch{"1H14 S80"}="H141 S80";
$resn_exch{"2H14 S80"}="H142 S80"; $resn_exch{"1H17 S80"}="H171 S80"; $resn_exch{"2H17 S80"}="H172 S80";
$resn_exch{"3H17 S80"}="H173 S80"; $resn_exch{"1H16 S80"}="H161 S80"; $resn_exch{"2H16 S80"}="H162 S80";
$resn_exch{"3H16 S80"}="H163 S80"; $resn_exch{"1H18 S80"}="H181 S80"; $resn_exch{"2H18 S80"}="H182 S80";
$resn_exch{"1H19 S80"}="H191 S80"; $resn_exch{"2H19 S80"}="H192 S80"; $resn_exch{"1H20 S80"}="H201 S80";
$resn_exch{"2H20 S80"}="H202 S80"; $resn_exch{"3H20 S80"}="H203 S80"; $resn_exch{"1H12 S9L"}="H121 S9L";
$resn_exch{"2H12 S9L"}="H122 S9L"; $resn_exch{"1H22 S9L"}="H221 S9L"; $resn_exch{"2H22 S9L"}="H222 S9L";
$resn_exch{"1H13 S9L"}="H131 S9L"; $resn_exch{"2H13 S9L"}="H132 S9L"; $resn_exch{"1H23 S9L"}="H231 S9L";
$resn_exch{"2H23 S9L"}="H232 S9L"; $resn_exch{"1H14 S9L"}="H141 S9L"; $resn_exch{"2H14 S9L"}="H142 S9L";
$resn_exch{"1H24 S9L"}="H241 S9L"; $resn_exch{"2H24 S9L"}="H242 S9L"; $resn_exch{"HO3* S9L"}="HO3' S9L";
$resn_exch{"1H1  SA1"}="H11A SA1"; $resn_exch{"2H1  SA1"}="H12A SA1"; $resn_exch{"1H4  SA1"}=" H41 SA1";
$resn_exch{"2H4  SA1"}=" H42 SA1"; $resn_exch{"3H4  SA1"}=" H43 SA1"; $resn_exch{"1H14 SA1"}="H141 SA1";
$resn_exch{"2H14 SA1"}="H142 SA1"; $resn_exch{"1H15 SA1"}="H151 SA1"; $resn_exch{"2H15 SA1"}="H152 SA1";
$resn_exch{"1H16 SA1"}="H161 SA1"; $resn_exch{"2H16 SA1"}="H162 SA1"; $resn_exch{"1H20 SA1"}="H201 SA1";
$resn_exch{"2H20 SA1"}="H202 SA1"; $resn_exch{"1H19 SA2"}="H191 SA2"; $resn_exch{"2H19 SA2"}="H192 SA2";
$resn_exch{"1H18 SA2"}="H181 SA2"; $resn_exch{"2H18 SA2"}="H182 SA2"; $resn_exch{"1H17 SA2"}="H171 SA2";
$resn_exch{"2H17 SA2"}="H172 SA2"; $resn_exch{"1H14 SA2"}="H141 SA2"; $resn_exch{"2H14 SA2"}="H142 SA2";
$resn_exch{"1H21 SA2"}="H211 SA2"; $resn_exch{"2H21 SA2"}="H212 SA2"; $resn_exch{"3H21 SA2"}="H213 SA2";
$resn_exch{"2H5  SA2"}=" H52 SA2"; $resn_exch{"2H6  SA2"}=" H62 SA2"; $resn_exch{"1H36 SA3"}="H361 SA3";
$resn_exch{"2H36 SA3"}="H362 SA3"; $resn_exch{"1HN  SA8"}=" HN1 SA8"; $resn_exch{"2HN  SA8"}=" HN2 SA8";
$resn_exch{"1HB  SA8"}=" HB1 SA8"; $resn_exch{"2HB  SA8"}=" HB2 SA8"; $resn_exch{"1HG  SA8"}=" HG1 SA8";
$resn_exch{"2HG  SA8"}=" HG2 SA8"; $resn_exch{"1HE  SA8"}=" HE1 SA8"; $resn_exch{"2HE  SA8"}=" HE2 SA8";
$resn_exch{"3HE  SA8"}=" HE3 SA8"; $resn_exch{"1H5* SA8"}="H5'1 SA8"; $resn_exch{"2H5* SA8"}="H5'2 SA8";
$resn_exch{" H4* SA8"}=" H4' SA8"; $resn_exch{" H3* SA8"}=" H3' SA8"; $resn_exch{" H2* SA8"}=" H2' SA8";
$resn_exch{" H1* SA8"}=" H1' SA8"; $resn_exch{"1HN6 SA8"}="HN61 SA8"; $resn_exch{"2HN6 SA8"}="HN62 SA8";
$resn_exch{"1H9  SAA"}=" H91 SAA"; $resn_exch{"2H9  SAA"}=" H92 SAA"; $resn_exch{"3H9  SAA"}=" H93 SAA";
$resn_exch{"1H1' SAB"}="H1'1 SAB"; $resn_exch{"2H1' SAB"}="H1'2 SAB"; $resn_exch{"1H2' SAB"}="H2'1 SAB";
$resn_exch{"2H2' SAB"}="H2'2 SAB"; $resn_exch{"1H3' SAB"}="H3'1 SAB"; $resn_exch{"2H3' SAB"}="H3'2 SAB";
$resn_exch{"1H4' SAB"}="H4'1 SAB"; $resn_exch{"2H4' SAB"}="H4'2 SAB"; $resn_exch{"1HN4 SAB"}="HN41 SAB";
$resn_exch{"2HN4 SAB"}="HN42 SAB"; $resn_exch{"1HN3 SAB"}="HN31 SAB"; $resn_exch{"2HN3 SAB"}="HN32 SAB";
$resn_exch{"1H2A SAC"}="H2A1 SAC"; $resn_exch{"2H2A SAC"}="H2A2 SAC"; $resn_exch{"3H2A SAC"}="H2A3 SAC";
$resn_exch{" HN  SAC"}=" H   SAC"; $resn_exch{"1HB  SAC"}=" HB2 SAC"; $resn_exch{"2HB  SAC"}=" HB3 SAC";
$resn_exch{" HOG SAC"}=" HG  SAC"; $resn_exch{"2HOA SAD"}="HOA2 SAD"; $resn_exch{"AH51 SAD"}="H51A SAD";
$resn_exch{"AH52 SAD"}="H52A SAD"; $resn_exch{"AH4* SAD"}=" H4B SAD"; $resn_exch{"AH3* SAD"}=" H3B SAD";
$resn_exch{"AHO3 SAD"}="HO3A SAD"; $resn_exch{"AH2* SAD"}=" H2B SAD"; $resn_exch{"AHO2 SAD"}="HO2A SAD";
$resn_exch{"AH1* SAD"}=" H1B SAD"; $resn_exch{" AH8 SAD"}=" H8A SAD"; $resn_exch{"AH61 SAD"}="H61A SAD";
$resn_exch{"AH62 SAD"}="H62A SAD"; $resn_exch{" AH2 SAD"}=" H2A SAD"; $resn_exch{"1H3  SAD"}=" H31 SAD";
$resn_exch{"2H3  SAD"}=" H32 SAD"; $resn_exch{"2HON SAD"}="HON2 SAD"; $resn_exch{"NH51 SAD"}="H51N SAD";
$resn_exch{"NH52 SAD"}="H52N SAD"; $resn_exch{"NH4* SAD"}=" H4D SAD"; $resn_exch{"NH3* SAD"}=" H3D SAD";
$resn_exch{"NHO3 SAD"}="HO3N SAD"; $resn_exch{"NH2* SAD"}=" H2D SAD"; $resn_exch{"NHO2 SAD"}="HO2N SAD";
$resn_exch{"NH1* SAD"}=" H1D SAD"; $resn_exch{"NH61 SAD"}="H61N SAD"; $resn_exch{"NH62 SAD"}="H62N SAD";
$resn_exch{"2HOA SAE"}="HOA2 SAE"; $resn_exch{"AH51 SAE"}="H51A SAE"; $resn_exch{"AH52 SAE"}="H52A SAE";
$resn_exch{"AH4* SAE"}=" H4B SAE"; $resn_exch{"AH3* SAE"}=" H3B SAE"; $resn_exch{"AHO3 SAE"}="HO3A SAE";
$resn_exch{"AH2* SAE"}=" H2B SAE"; $resn_exch{"AHO2 SAE"}="HO2A SAE"; $resn_exch{"AH1* SAE"}=" H1B SAE";
$resn_exch{" AH8 SAE"}=" H8A SAE"; $resn_exch{"AH61 SAE"}="H61A SAE"; $resn_exch{"AH62 SAE"}="H62A SAE";
$resn_exch{" AH2 SAE"}=" H2A SAE"; $resn_exch{"2HON SAE"}="HON2 SAE"; $resn_exch{"NH51 SAE"}="H51N SAE";
$resn_exch{"NH52 SAE"}="H52N SAE"; $resn_exch{"NH4* SAE"}=" H4D SAE"; $resn_exch{"NH3* SAE"}=" H3D SAE";
$resn_exch{"NHO3 SAE"}="HO3N SAE"; $resn_exch{"NH2* SAE"}=" H2D SAE"; $resn_exch{"NHO2 SAE"}="HO2N SAE";
$resn_exch{"NH1* SAE"}=" H1D SAE"; $resn_exch{"NH61 SAE"}="H61N SAE"; $resn_exch{"NH62 SAE"}="H62N SAE";
$resn_exch{"1H7C SAF"}="H7C1 SAF"; $resn_exch{"2H7C SAF"}="H7C2 SAF"; $resn_exch{"3H7C SAF"}="H7C3 SAF";
$resn_exch{"1H8C SAF"}="H8C1 SAF"; $resn_exch{"2H8C SAF"}="H8C2 SAF"; $resn_exch{"3H8C SAF"}="H8C3 SAF";
$resn_exch{"1H10 SAF"}="H101 SAF"; $resn_exch{"2H10 SAF"}="H102 SAF"; $resn_exch{"3H10 SAF"}="H103 SAF";
$resn_exch{"1H7C SAG"}="H7C1 SAG"; $resn_exch{"2H7C SAG"}="H7C2 SAG"; $resn_exch{"1H18 SAG"}="H181 SAG";
$resn_exch{"2H18 SAG"}="H182 SAG"; $resn_exch{"3H18 SAG"}="H183 SAG"; $resn_exch{"1H21 SAG"}="H211 SAG";
$resn_exch{"2H21 SAG"}="H212 SAG"; $resn_exch{"1HN  SAH"}=" HN1 SAH"; $resn_exch{"2HN  SAH"}=" HN2 SAH";
$resn_exch{"1HB  SAH"}=" HB1 SAH"; $resn_exch{"2HB  SAH"}=" HB2 SAH"; $resn_exch{"1HG  SAH"}=" HG1 SAH";
$resn_exch{"2HG  SAH"}=" HG2 SAH"; $resn_exch{"1H5* SAH"}="H5'1 SAH"; $resn_exch{"2H5* SAH"}="H5'2 SAH";
$resn_exch{" H4* SAH"}=" H4' SAH"; $resn_exch{" H3* SAH"}=" H3' SAH"; $resn_exch{"*HO3 SAH"}="HO3' SAH";
$resn_exch{" H2* SAH"}=" H2' SAH"; $resn_exch{"*HO2 SAH"}="HO2' SAH"; $resn_exch{" H1* SAH"}=" H1' SAH";
$resn_exch{"1HN6 SAH"}="HN61 SAH"; $resn_exch{"2HN6 SAH"}="HN62 SAH"; $resn_exch{"1HN  SAI"}=" HN1 SAI";
$resn_exch{"2HN  SAI"}=" HN2 SAI"; $resn_exch{"1HB  SAI"}=" HB1 SAI"; $resn_exch{"2HB  SAI"}=" HB2 SAI";
$resn_exch{"1HG  SAI"}=" HG1 SAI"; $resn_exch{"2HG  SAI"}=" HG2 SAI"; $resn_exch{"1H5* SAI"}="H5'1 SAI";
$resn_exch{"2H5* SAI"}="H5'2 SAI"; $resn_exch{" H4* SAI"}=" H4' SAI"; $resn_exch{" H3* SAI"}=" H3' SAI";
$resn_exch{"*HO3 SAI"}="HO3' SAI"; $resn_exch{" H2* SAI"}=" H2' SAI"; $resn_exch{"*HO2 SAI"}="HO2' SAI";
$resn_exch{" H1* SAI"}=" H1' SAI"; $resn_exch{"1HN6 SAI"}="HN61 SAI"; $resn_exch{"2HN6 SAI"}="HN62 SAI";
$resn_exch{"'HO2 SAL"}="HO2' SAL"; $resn_exch{"1HN  SAM"}=" HN1 SAM"; $resn_exch{"2HN  SAM"}=" HN2 SAM";
$resn_exch{"1HB  SAM"}=" HB1 SAM"; $resn_exch{"2HB  SAM"}=" HB2 SAM"; $resn_exch{"1HG  SAM"}=" HG1 SAM";
$resn_exch{"2HG  SAM"}=" HG2 SAM"; $resn_exch{"1HE  SAM"}=" HE1 SAM"; $resn_exch{"2HE  SAM"}=" HE2 SAM";
$resn_exch{"3HE  SAM"}=" HE3 SAM"; $resn_exch{"1H5* SAM"}="H5'1 SAM"; $resn_exch{"2H5* SAM"}="H5'2 SAM";
$resn_exch{" H4* SAM"}=" H4' SAM"; $resn_exch{" H3* SAM"}=" H3' SAM"; $resn_exch{"*HO3 SAM"}="HO3' SAM";
$resn_exch{" H2* SAM"}=" H2' SAM"; $resn_exch{"*HO2 SAM"}="HO2' SAM"; $resn_exch{" H1* SAM"}=" H1' SAM";
$resn_exch{"1HN6 SAM"}="HN61 SAM"; $resn_exch{"2HN6 SAM"}="HN62 SAM"; $resn_exch{"1HN1 SAN"}="HN11 SAN";
$resn_exch{"2HN1 SAN"}="HN12 SAN"; $resn_exch{"1HN2 SAN"}="HN21 SAN"; $resn_exch{"2HN2 SAN"}="HN22 SAN";
$resn_exch{"2HOG SAP"}="HOG2 SAP"; $resn_exch{"3HOG SAP"}="HOG3 SAP"; $resn_exch{"2HOB SAP"}="HOB2 SAP";
$resn_exch{"2HOA SAP"}="HOA2 SAP"; $resn_exch{"1H5* SAP"}="H5'1 SAP"; $resn_exch{"2H5* SAP"}="H5'2 SAP";
$resn_exch{" H4* SAP"}=" H4' SAP"; $resn_exch{" H3* SAP"}=" H3' SAP"; $resn_exch{"*HO3 SAP"}="HO3' SAP";
$resn_exch{" H2* SAP"}=" H2' SAP"; $resn_exch{"*HO2 SAP"}="HO2' SAP"; $resn_exch{" H1* SAP"}=" H1' SAP";
$resn_exch{"1HN6 SAP"}="HN61 SAP"; $resn_exch{"2HN6 SAP"}="HN62 SAP"; $resn_exch{" HN  SAR"}=" H   SAR";
$resn_exch{"1HA  SAR"}=" HA2 SAR"; $resn_exch{"2HA  SAR"}=" HA3 SAR"; $resn_exch{"1HN  SAR"}=" HN1 SAR";
$resn_exch{"2HN  SAR"}=" HN2 SAR"; $resn_exch{"3HN  SAR"}=" HN3 SAR"; $resn_exch{"1H2  SAT"}=" H21 SAT";
$resn_exch{"2H2  SAT"}=" H22 SAT"; $resn_exch{"1H02 SAZ"}="H021 SAZ"; $resn_exch{"2H02 SAZ"}="H022 SAZ";
$resn_exch{"1H03 SAZ"}="H031 SAZ"; $resn_exch{"2H03 SAZ"}="H032 SAZ"; $resn_exch{"1H07 SAZ"}="H071 SAZ";
$resn_exch{"2H07 SAZ"}="H072 SAZ"; $resn_exch{"1H09 SAZ"}="H091 SAZ"; $resn_exch{"2H09 SAZ"}="H092 SAZ";
$resn_exch{"1H10 SAZ"}="H101 SAZ"; $resn_exch{"2H10 SAZ"}="H102 SAZ"; $resn_exch{"1H16 SAZ"}="H161 SAZ";
$resn_exch{"2H16 SAZ"}="H162 SAZ"; $resn_exch{"3H16 SAZ"}="H163 SAZ"; $resn_exch{"1H17 SAZ"}="H171 SAZ";
$resn_exch{"2H17 SAZ"}="H172 SAZ"; $resn_exch{"3H17 SAZ"}="H173 SAZ"; $resn_exch{"1H26 SAZ"}="H261 SAZ";
$resn_exch{"2H26 SAZ"}="H262 SAZ"; $resn_exch{"3H26 SAZ"}="H263 SAZ"; $resn_exch{"1H36 SAZ"}="H361 SAZ";
$resn_exch{"2H36 SAZ"}="H362 SAZ"; $resn_exch{"3H36 SAZ"}="H363 SAZ"; $resn_exch{"1H5  SB1"}=" H51 SB1";
$resn_exch{"2H5  SB1"}=" H52 SB1"; $resn_exch{"1H8  SB1"}=" H81 SB1"; $resn_exch{"2H8  SB1"}=" H82 SB1";
$resn_exch{"1H9  SB1"}=" H91 SB1"; $resn_exch{"2H9  SB1"}=" H92 SB1"; $resn_exch{"1H10 SB1"}="H101 SB1";
$resn_exch{"2H10 SB1"}="H102 SB1"; $resn_exch{"1H11 SB1"}="H111 SB1"; $resn_exch{"2H11 SB1"}="H112 SB1";
$resn_exch{"1H12 SB1"}="H121 SB1"; $resn_exch{"2H12 SB1"}="H122 SB1"; $resn_exch{"1H13 SB1"}="H131 SB1";
$resn_exch{"2H13 SB1"}="H132 SB1"; $resn_exch{"1H14 SB1"}="H141 SB1"; $resn_exch{"2H14 SB1"}="H142 SB1";
$resn_exch{"1H15 SB1"}="H151 SB1"; $resn_exch{"2H15 SB1"}="H152 SB1"; $resn_exch{"1H16 SB1"}="H161 SB1";
$resn_exch{"2H16 SB1"}="H162 SB1"; $resn_exch{"1H17 SB1"}="H171 SB1"; $resn_exch{"2H17 SB1"}="H172 SB1";
$resn_exch{"1H18 SB1"}="H181 SB1"; $resn_exch{"2H18 SB1"}="H182 SB1"; $resn_exch{"1H22 SB1"}="H221 SB1";
$resn_exch{"2H22 SB1"}="H222 SB1"; $resn_exch{"1H23 SB1"}="H231 SB1"; $resn_exch{"2H23 SB1"}="H232 SB1";
$resn_exch{"1H24 SB1"}="H241 SB1"; $resn_exch{"2H24 SB1"}="H242 SB1"; $resn_exch{"1H25 SB1"}="H251 SB1";
$resn_exch{"2H25 SB1"}="H252 SB1"; $resn_exch{"1H26 SB1"}="H261 SB1"; $resn_exch{"2H26 SB1"}="H262 SB1";
$resn_exch{"3H26 SB1"}="H263 SB1"; $resn_exch{"1H27 SB1"}="H271 SB1"; $resn_exch{"2H27 SB1"}="H272 SB1";
$resn_exch{"3H27 SB1"}="H273 SB1"; $resn_exch{"1H1  SB2"}=" H11 SB2"; $resn_exch{"2H1  SB2"}=" H12 SB2";
$resn_exch{"3H1  SB2"}=" H13 SB2"; $resn_exch{"1H3  SB3"}=" H31 SB3"; $resn_exch{"2H3  SB3"}=" H32 SB3";
$resn_exch{"1H4  SB3"}=" H41 SB3"; $resn_exch{"2H4  SB3"}=" H42 SB3"; $resn_exch{"1H5  SB3"}=" H51 SB3";
$resn_exch{"2H5  SB3"}=" H52 SB3"; $resn_exch{"1H6  SB3"}=" H61 SB3"; $resn_exch{"2H6  SB3"}=" H62 SB3";
$resn_exch{"1H11 SB3"}="H111 SB3"; $resn_exch{"2H11 SB3"}="H112 SB3"; $resn_exch{"1H12 SB3"}="H121 SB3";
$resn_exch{"2H12 SB3"}="H122 SB3"; $resn_exch{"3H12 SB3"}="H123 SB3"; $resn_exch{"1H13 SB3"}="H131 SB3";
$resn_exch{"2H13 SB3"}="H132 SB3"; $resn_exch{"3H13 SB3"}="H133 SB3"; $resn_exch{"1H14 SB3"}="H141 SB3";
$resn_exch{"2H14 SB3"}="H142 SB3"; $resn_exch{"3H14 SB3"}="H143 SB3"; $resn_exch{"1H16 SB3"}="H161 SB3";
$resn_exch{"2H16 SB3"}="H162 SB3"; $resn_exch{"1H17 SB3"}="H171 SB3"; $resn_exch{"2H17 SB3"}="H172 SB3";
$resn_exch{"1H1  SB4"}=" H11 SB4"; $resn_exch{"2H1  SB4"}=" H12 SB4"; $resn_exch{"1H2  SB4"}=" H21 SB4";
$resn_exch{"2H2  SB4"}=" H22 SB4"; $resn_exch{"1H4  SB4"}=" H41 SB4"; $resn_exch{"2H4  SB4"}=" H42 SB4";
$resn_exch{"1H5  SB4"}=" H51 SB4"; $resn_exch{"2H5  SB4"}=" H52 SB4"; $resn_exch{"1HN7 SB4"}="HN71 SB4";
$resn_exch{"2HN7 SB4"}="HN72 SB4"; $resn_exch{"1HN7 SB5"}="HN71 SB5"; $resn_exch{"2HN7 SB5"}="HN72 SB5";
$resn_exch{"1HCD SB5"}="HCD1 SB5"; $resn_exch{"2HCD SB5"}="HCD2 SB5"; $resn_exch{"1HC2 SB5"}="HC21 SB5";
$resn_exch{"2HC2 SB5"}="HC22 SB5"; $resn_exch{"1HC3 SB5"}="HC31 SB5"; $resn_exch{"2HC3 SB5"}="HC32 SB5";
$resn_exch{"1HCD SB6"}="HCD1 SB6"; $resn_exch{"2HCD SB6"}="HCD2 SB6"; $resn_exch{"1HE2 SB6"}="HE21 SB6";
$resn_exch{"2HE2 SB6"}="HE22 SB6"; $resn_exch{"1HE3 SB6"}="HE31 SB6"; $resn_exch{"2HE3 SB6"}="HE32 SB6";
$resn_exch{"1H7  SB7"}=" H71 SB7"; $resn_exch{"2H7  SB7"}=" H72 SB7"; $resn_exch{"1H8  SB7"}=" H81 SB7";
$resn_exch{"2H8  SB7"}=" H82 SB7"; $resn_exch{"1H11 SB7"}="H111 SB7"; $resn_exch{"2H11 SB7"}="H112 SB7";
$resn_exch{"1H24 SB7"}="H241 SB7"; $resn_exch{"2H24 SB7"}="H242 SB7"; $resn_exch{"1H25 SB8"}="H251 SB8";
$resn_exch{"2H25 SB8"}="H252 SB8"; $resn_exch{"1H26 SB8"}="H261 SB8"; $resn_exch{"2H26 SB8"}="H262 SB8";
$resn_exch{"1H7  SB9"}=" H71 SB9"; $resn_exch{"2H7  SB9"}=" H72 SB9"; $resn_exch{"3H7  SB9"}=" H73 SB9";
$resn_exch{"1H11 SB9"}="H111 SB9"; $resn_exch{"2H11 SB9"}="H112 SB9"; $resn_exch{"1H12 SB9"}="H121 SB9";
$resn_exch{"2H12 SB9"}="H122 SB9"; $resn_exch{"1H15 SB9"}="H151 SB9"; $resn_exch{"2H15 SB9"}="H152 SB9";
$resn_exch{"1H7  SBA"}=" H71 SBA"; $resn_exch{"2H7  SBA"}=" H72 SBA"; $resn_exch{"1H23 SBA"}="H231 SBA";
$resn_exch{"2H23 SBA"}="H232 SBA"; $resn_exch{"1H25 SBA"}="H251 SBA"; $resn_exch{"2H25 SBA"}="H252 SBA";
$resn_exch{"3H25 SBA"}="H253 SBA"; $resn_exch{"1H26 SBA"}="H261 SBA"; $resn_exch{"2H26 SBA"}="H262 SBA";
$resn_exch{"3H26 SBA"}="H263 SBA"; $resn_exch{"1H19 SBA"}="H191 SBA"; $resn_exch{"2H19 SBA"}="H192 SBA";
$resn_exch{"1H21 SBA"}="H211 SBA"; $resn_exch{"2H21 SBA"}="H212 SBA"; $resn_exch{"3H21 SBA"}="H213 SBA";
$resn_exch{"1H22 SBA"}="H221 SBA"; $resn_exch{"2H22 SBA"}="H222 SBA"; $resn_exch{"3H22 SBA"}="H223 SBA";
$resn_exch{"1H16 SBA"}="H161 SBA"; $resn_exch{"2H16 SBA"}="H162 SBA"; $resn_exch{"1H18 SBA"}="H181 SBA";
$resn_exch{"2H18 SBA"}="H182 SBA"; $resn_exch{"3H18 SBA"}="H183 SBA"; $resn_exch{"1H10 SBB"}="H101 SBB";
$resn_exch{"2H10 SBB"}="H102 SBB"; $resn_exch{"1H12 SBB"}="H121 SBB"; $resn_exch{"2H12 SBB"}="H122 SBB";
$resn_exch{"1H13 SBB"}="H131 SBB"; $resn_exch{"2H13 SBB"}="H132 SBB"; $resn_exch{"3H13 SBB"}="H133 SBB";
$resn_exch{"1H15 SBB"}="H151 SBB"; $resn_exch{"2H15 SBB"}="H152 SBB"; $resn_exch{"1H20 SBC"}="H201 SBC";
$resn_exch{"2H20 SBC"}="H202 SBC"; $resn_exch{"1H25 SBC"}="H251 SBC"; $resn_exch{"2H25 SBC"}="H252 SBC";
$resn_exch{"1HN  SBD"}=" H   SBD"; $resn_exch{"2HN  SBD"}=" H2  SBD"; $resn_exch{"1HB  SBD"}=" HB2 SBD";
$resn_exch{"2HB  SBD"}=" HB3 SBD"; $resn_exch{"1H11 SBD"}="H111 SBD"; $resn_exch{"2H11 SBD"}="H112 SBD";
$resn_exch{" H2  SBD"}=" HC2 SBD"; $resn_exch{"1H16 SBD"}="H161 SBD"; $resn_exch{"2H16 SBD"}="H162 SBD";
$resn_exch{"3H16 SBD"}="H163 SBD"; $resn_exch{"2HZ1 SBE"}="HZ12 SBE"; $resn_exch{"3HZ1 SBE"}="HZ13 SBE";
$resn_exch{"2HZ2 SBE"}="HZ22 SBE"; $resn_exch{"3HZ2 SBE"}="HZ23 SBE"; $resn_exch{"1HN  SBG"}=" HN1 SBG";
$resn_exch{"2HN  SBG"}=" HN2 SBG"; $resn_exch{"1HBC SBG"}="HBC1 SBG"; $resn_exch{"2HBC SBG"}="HBC2 SBG";
$resn_exch{"1H1C SBG"}="H1C1 SBG"; $resn_exch{"2H1C SBG"}="H1C2 SBG"; $resn_exch{"3H1C SBG"}="H1C3 SBG";
$resn_exch{"1H1  SBH"}=" H11 SBH"; $resn_exch{"2H1  SBH"}=" H12 SBH"; $resn_exch{"1H2  SBH"}=" H21 SBH";
$resn_exch{"2H2  SBH"}=" H22 SBH"; $resn_exch{"1H4  SBH"}=" H41 SBH"; $resn_exch{"2H4  SBH"}=" H42 SBH";
$resn_exch{"3H4  SBH"}=" H43 SBH"; $resn_exch{"1H5  SBH"}=" H51 SBH"; $resn_exch{"2H5  SBH"}=" H52 SBH";
$resn_exch{"1H5  SBI"}=" H51 SBI"; $resn_exch{"2H5  SBI"}=" H52 SBI"; $resn_exch{"1H6  SBI"}=" H61 SBI";
$resn_exch{"2H6  SBI"}=" H62 SBI"; $resn_exch{"1HN  SBL"}=" H   SBL"; $resn_exch{"2HN  SBL"}=" H2  SBL";
$resn_exch{"1HB  SBL"}=" HB2 SBL"; $resn_exch{"2HB  SBL"}=" HB3 SBL"; $resn_exch{"1H11 SBL"}="H111 SBL";
$resn_exch{"2H11 SBL"}="H112 SBL"; $resn_exch{" H2  SBL"}=" HC2 SBL"; $resn_exch{"1H16 SBL"}="H161 SBL";
$resn_exch{"2H16 SBL"}="H162 SBL"; $resn_exch{"3H16 SBL"}="H163 SBL"; $resn_exch{"1HN1 SBN"}="HN11 SBN";
$resn_exch{"2HN1 SBN"}="HN12 SBN"; $resn_exch{"1H14 SBN"}="H141 SBN"; $resn_exch{"2H14 SBN"}="H142 SBN";
$resn_exch{"3H14 SBN"}="H143 SBN"; $resn_exch{"1H15 SBN"}="H151 SBN"; $resn_exch{"2H15 SBN"}="H152 SBN";
$resn_exch{"3H15 SBN"}="H153 SBN"; $resn_exch{"1H1  SBO"}=" H11 SBO"; $resn_exch{"1H2  SBO"}=" H21 SBO";
$resn_exch{"1H3  SBO"}=" H31 SBO"; $resn_exch{"2HZ2 SBP"}="HZ22 SBP"; $resn_exch{"3HZ2 SBP"}="HZ23 SBP";
$resn_exch{"2HZ3 SBP"}="HZ32 SBP"; $resn_exch{"3HZ3 SBP"}="HZ33 SBP"; $resn_exch{"2HZ1 SBP"}="HZ12 SBP";
$resn_exch{"3HZ1 SBP"}="HZ13 SBP"; $resn_exch{"2HH  SBP"}=" HH2 SBP"; $resn_exch{"3HH  SBP"}=" HH3 SBP";
$resn_exch{"2HQ  SBP"}=" HQ2 SBP"; $resn_exch{"3HQ  SBP"}=" HQ3 SBP"; $resn_exch{"1HL1 SBP"}="HL11 SBP";
$resn_exch{"2HL1 SBP"}="HL12 SBP"; $resn_exch{"1HL2 SBP"}="HL21 SBP"; $resn_exch{"2HL2 SBP"}="HL22 SBP";
$resn_exch{"1HC1 SBR"}="HC11 SBR"; $resn_exch{"2HC1 SBR"}="HC12 SBR"; $resn_exch{"1HC3 SBR"}="HC31 SBR";
$resn_exch{"2HC3 SBR"}="HC32 SBR"; $resn_exch{"1HN6 SBR"}="HN61 SBR"; $resn_exch{"2HN6 SBR"}="HN62 SBR";
$resn_exch{"1H26 SBR"}="H261 SBR"; $resn_exch{"2H26 SBR"}="H262 SBR"; $resn_exch{"3H26 SBR"}="H263 SBR";
$resn_exch{"1HC1 SBS"}="HC11 SBS"; $resn_exch{"2HC1 SBS"}="HC12 SBS"; $resn_exch{"1HC3 SBS"}="HC31 SBS";
$resn_exch{"2HC3 SBS"}="HC32 SBS"; $resn_exch{"1HN6 SBS"}="HN61 SBS"; $resn_exch{"2HN6 SBS"}="HN62 SBS";
$resn_exch{"1H26 SBS"}="H261 SBS"; $resn_exch{"2H26 SBS"}="H262 SBS"; $resn_exch{"3H26 SBS"}="H263 SBS";
$resn_exch{"1H1  SBT"}=" H11 SBT"; $resn_exch{"2H1  SBT"}=" H12 SBT"; $resn_exch{"3H1  SBT"}=" H13 SBT";
$resn_exch{"1H3  SBT"}=" H31 SBT"; $resn_exch{"2H3  SBT"}=" H32 SBT"; $resn_exch{"1H4  SBT"}=" H41 SBT";
$resn_exch{"2H4  SBT"}=" H42 SBT"; $resn_exch{"3H4  SBT"}=" H43 SBT"; $resn_exch{"1H1  SBU"}=" H11 SBU";
$resn_exch{"2H1  SBU"}=" H12 SBU"; $resn_exch{"3H1  SBU"}=" H13 SBU"; $resn_exch{"1H2  SBU"}=" H21 SBU";
$resn_exch{"1H3  SBU"}=" H31 SBU"; $resn_exch{"2H3  SBU"}=" H32 SBU"; $resn_exch{"1H4  SBU"}=" H41 SBU";
$resn_exch{"2H4  SBU"}=" H42 SBU"; $resn_exch{"3H4  SBU"}=" H43 SBU"; $resn_exch{"1H3  SBX"}=" H31 SBX";
$resn_exch{"2H3  SBX"}=" H32 SBX"; $resn_exch{"1H4  SBX"}=" H41 SBX"; $resn_exch{"2H4  SBX"}=" H42 SBX";
$resn_exch{"1H5  SBX"}=" H51 SBX"; $resn_exch{"2H5  SBX"}=" H52 SBX"; $resn_exch{"1H6  SBX"}=" H61 SBX";
$resn_exch{"2H6  SBX"}=" H62 SBX"; $resn_exch{"1H11 SBX"}="H111 SBX"; $resn_exch{"2H11 SBX"}="H112 SBX";
$resn_exch{"1H12 SBX"}="H121 SBX"; $resn_exch{"2H12 SBX"}="H122 SBX"; $resn_exch{"3H12 SBX"}="H123 SBX";
$resn_exch{"1H13 SBX"}="H131 SBX"; $resn_exch{"2H13 SBX"}="H132 SBX"; $resn_exch{"3H13 SBX"}="H133 SBX";
$resn_exch{"1H14 SBX"}="H141 SBX"; $resn_exch{"2H14 SBX"}="H142 SBX"; $resn_exch{"3H14 SBX"}="H143 SBX";
$resn_exch{"1H16 SBX"}="H161 SBX"; $resn_exch{"2H16 SBX"}="H162 SBX"; $resn_exch{"1H17 SBX"}="H171 SBX";
$resn_exch{"2H17 SBX"}="H172 SBX"; $resn_exch{"1H25 SBX"}="H251 SBX"; $resn_exch{"2H25 SBX"}="H252 SBX";
$resn_exch{"1H26 SBX"}="H261 SBX"; $resn_exch{"2H26 SBX"}="H262 SBX"; $resn_exch{"1H27 SBX"}="H271 SBX";
$resn_exch{"2H27 SBX"}="H272 SBX"; $resn_exch{"1H28 SBX"}="H281 SBX"; $resn_exch{"2H28 SBX"}="H282 SBX";
$resn_exch{"1H29 SBX"}="H291 SBX"; $resn_exch{"2H29 SBX"}="H292 SBX"; $resn_exch{"2HZ2 SBZ"}="HZ22 SBZ";
$resn_exch{"3HZ2 SBZ"}="HZ23 SBZ"; $resn_exch{"2HZ3 SBZ"}="HZ32 SBZ"; $resn_exch{"3HZ3 SBZ"}="HZ33 SBZ";
$resn_exch{"2HZ1 SBZ"}="HZ12 SBZ"; $resn_exch{"3HZ1 SBZ"}="HZ13 SBZ"; $resn_exch{"2HM1 SBZ"}="HM12 SBZ";
$resn_exch{"3HM1 SBZ"}="HM13 SBZ"; $resn_exch{"2HM2 SBZ"}="HM22 SBZ"; $resn_exch{"3HM2 SBZ"}="HM23 SBZ";
$resn_exch{"1HN4  SC"}="HN41  SC"; $resn_exch{"2HN4  SC"}="HN42  SC"; $resn_exch{" H1*  SC"}=" H1'  SC";
$resn_exch{"1H2*  SC"}=" H2'  SC"; $resn_exch{"2H2*  SC"}="H2''  SC"; $resn_exch{" H3*  SC"}=" H3'  SC";
$resn_exch{" H4*  SC"}=" H4'  SC"; $resn_exch{"*HO3  SC"}="HO3'  SC"; $resn_exch{"1H5*  SC"}=" H5'  SC";
$resn_exch{"2H5*  SC"}="H5''  SC"; $resn_exch{"1HOP  SC"}="HOP1  SC"; $resn_exch{"3HOP  SC"}="HOP3  SC";
$resn_exch{"1H6  SC1"}=" H61 SC1"; $resn_exch{"2H6  SC1"}=" H62 SC1"; $resn_exch{"1H9  SC1"}=" H91 SC1";
$resn_exch{"2H9  SC1"}=" H92 SC1"; $resn_exch{"3H9  SC1"}=" H93 SC1"; $resn_exch{"1HBC SC2"}="HBC1 SC2";
$resn_exch{"2HBC SC2"}="HBC2 SC2"; $resn_exch{"1HMC SC2"}="HMC1 SC2"; $resn_exch{"2HMC SC2"}="HMC2 SC2";
$resn_exch{"3HMC SC2"}="HMC3 SC2"; $resn_exch{"1H11 SC3"}="H111 SC3"; $resn_exch{"2H11 SC3"}="H112 SC3";
$resn_exch{"1H12 SC3"}="H121 SC3"; $resn_exch{"2H12 SC3"}="H122 SC3"; $resn_exch{"1H13 SC3"}="H131 SC3";
$resn_exch{"2H13 SC3"}="H132 SC3"; $resn_exch{"1H14 SC3"}="H141 SC3"; $resn_exch{"2H14 SC3"}="H142 SC3";
$resn_exch{"1H15 SC3"}="H151 SC3"; $resn_exch{"2H15 SC3"}="H152 SC3"; $resn_exch{"1H16 SC3"}="H161 SC3";
$resn_exch{"2H16 SC3"}="H162 SC3"; $resn_exch{"1H24 SC3"}="H241 SC3"; $resn_exch{"2H24 SC3"}="H242 SC3";
$resn_exch{"3H24 SC3"}="H243 SC3"; $resn_exch{"1H26 SC3"}="H261 SC3"; $resn_exch{"2H26 SC3"}="H262 SC3";
$resn_exch{"3H26 SC3"}="H263 SC3"; $resn_exch{"1H7  SC4"}=" H71 SC4"; $resn_exch{"2H7  SC4"}=" H72 SC4";
$resn_exch{"1H21 SC4"}="H211 SC4"; $resn_exch{"2H21 SC4"}="H212 SC4"; $resn_exch{"3H21 SC4"}="H213 SC4";
$resn_exch{"1H14 SC4"}="H141 SC4"; $resn_exch{"2H14 SC4"}="H142 SC4"; $resn_exch{"1H7  SC5"}=" H71 SC5";
$resn_exch{"2H7  SC5"}=" H72 SC5"; $resn_exch{"3H7  SC5"}=" H73 SC5"; $resn_exch{"1H9  SC5"}=" H91 SC5";
$resn_exch{"2H9  SC5"}=" H92 SC5"; $resn_exch{"1H12 SC5"}="H121 SC5"; $resn_exch{"2H12 SC5"}="H122 SC5";
$resn_exch{"1HN4 SC5"}="HN41 SC5"; $resn_exch{"2HN4 SC5"}="HN42 SC5"; $resn_exch{"1H8  SC6"}=" H81 SC6";
$resn_exch{"2H8  SC6"}=" H82 SC6"; $resn_exch{"1H9  SC6"}=" H91 SC6"; $resn_exch{"2H9  SC6"}=" H92 SC6";
$resn_exch{"1H10 SC6"}="H101 SC6"; $resn_exch{"2H10 SC6"}="H102 SC6"; $resn_exch{"1H12 SC6"}="H121 SC6";
$resn_exch{"2H12 SC6"}="H122 SC6"; $resn_exch{"1H30 SC6"}="H301 SC6"; $resn_exch{"2H30 SC6"}="H302 SC6";
$resn_exch{"3H30 SC6"}="H303 SC6"; $resn_exch{"1H17 SC6"}="H171 SC6"; $resn_exch{"2H17 SC6"}="H172 SC6";
$resn_exch{"1H18 SC6"}="H181 SC6"; $resn_exch{"2H18 SC6"}="H182 SC6"; $resn_exch{"1H20 SC6"}="H201 SC6";
$resn_exch{"2H20 SC6"}="H202 SC6"; $resn_exch{"1H37 SC6"}="H371 SC6"; $resn_exch{"2H37 SC6"}="H372 SC6";
$resn_exch{"3H37 SC6"}="H373 SC6"; $resn_exch{"1H8  SC7"}=" H81 SC7"; $resn_exch{"2H8  SC7"}=" H82 SC7";
$resn_exch{"1H9  SC7"}=" H91 SC7"; $resn_exch{"2H9  SC7"}=" H92 SC7"; $resn_exch{"1H10 SC7"}="H101 SC7";
$resn_exch{"2H10 SC7"}="H102 SC7"; $resn_exch{"1H11 SC7"}="H111 SC7"; $resn_exch{"2H11 SC7"}="H112 SC7";
$resn_exch{"3H11 SC7"}="H113 SC7"; $resn_exch{"1H12 SC7"}="H121 SC7"; $resn_exch{"2H12 SC7"}="H122 SC7";
$resn_exch{"1H13 SC7"}="H131 SC7"; $resn_exch{"2H13 SC7"}="H132 SC7"; $resn_exch{"3H13 SC7"}="H133 SC7";
$resn_exch{"1H14 SC7"}="H141 SC7"; $resn_exch{"2H14 SC7"}="H142 SC7"; $resn_exch{"3H14 SC7"}="H143 SC7";
$resn_exch{"1H18 SC7"}="H181 SC7"; $resn_exch{"2H18 SC7"}="H182 SC7"; $resn_exch{"1H26 SC7"}="H261 SC7";
$resn_exch{"2H26 SC7"}="H262 SC7"; $resn_exch{"1H28 SC7"}="H281 SC7"; $resn_exch{"2H28 SC7"}="H282 SC7";
$resn_exch{"1H29 SC7"}="H291 SC7"; $resn_exch{"2H29 SC7"}="H292 SC7"; $resn_exch{"1H30 SC7"}="H301 SC7";
$resn_exch{"2H30 SC7"}="H302 SC7"; $resn_exch{"1H31 SC7"}="H311 SC7"; $resn_exch{"2H31 SC7"}="H312 SC7";
$resn_exch{"3H31 SC7"}="H313 SC7"; $resn_exch{"1H12 SC8"}="H121 SC8"; $resn_exch{"2H12 SC8"}="H122 SC8";
$resn_exch{"1H11 SC9"}="H111 SC9"; $resn_exch{"2H11 SC9"}="H112 SC9"; $resn_exch{"1HN6 SCA"}="HN61 SCA";
$resn_exch{"2HN6 SCA"}="HN62 SCA"; $resn_exch{"'HO2 SCA"}="HO2' SCA"; $resn_exch{"1H5' SCA"}="H5'1 SCA";
$resn_exch{"2H5' SCA"}="H5'2 SCA"; $resn_exch{"1HPB SCA"}="HPB1 SCA"; $resn_exch{"2HPB SCA"}="HPB2 SCA";
$resn_exch{"1H9  SCA"}=" H91 SCA"; $resn_exch{"2H9  SCA"}=" H92 SCA"; $resn_exch{"3H9  SCA"}=" H93 SCA";
$resn_exch{"1H8  SCA"}=" H81 SCA"; $resn_exch{"2H8  SCA"}=" H82 SCA"; $resn_exch{"3H8  SCA"}=" H83 SCA";
$resn_exch{"1H5  SCA"}=" H51 SCA"; $resn_exch{"2H5  SCA"}=" H52 SCA"; $resn_exch{"1H4  SCA"}=" H41 SCA";
$resn_exch{"2H4  SCA"}=" H42 SCA"; $resn_exch{"1H2  SCA"}=" H21 SCA"; $resn_exch{"2H2  SCA"}=" H22 SCA";
$resn_exch{"1H1  SCA"}=" H11 SCA"; $resn_exch{"2H1  SCA"}=" H12 SCA"; $resn_exch{"1HS2 SCA"}="HS21 SCA";
$resn_exch{"2HS2 SCA"}="HS22 SCA"; $resn_exch{"1HS3 SCA"}="HS31 SCA"; $resn_exch{"2HS3 SCA"}="HS32 SCA";
$resn_exch{"1H1  SCB"}=" H11 SCB"; $resn_exch{"2H1  SCB"}="H12A SCB"; $resn_exch{"3H1  SCB"}="H13A SCB";
$resn_exch{"1H2  SCB"}="H21A SCB"; $resn_exch{"2H2  SCB"}="H22A SCB"; $resn_exch{"1H3  SCB"}=" H31 SCB";
$resn_exch{"2H3  SCB"}=" H32 SCB"; $resn_exch{"1H1  SCC"}=" H11 SCC"; $resn_exch{"2H1  SCC"}=" H12 SCC";
$resn_exch{"1H2  SCC"}=" H21 SCC"; $resn_exch{"2H2  SCC"}=" H22 SCC"; $resn_exch{"3H2  SCC"}=" H23 SCC";
$resn_exch{"1HN6 SCD"}="HN61 SCD"; $resn_exch{"2HN6 SCD"}="HN62 SCD"; $resn_exch{"'HO2 SCD"}="HO2' SCD";
$resn_exch{"1H5' SCD"}="H5'1 SCD"; $resn_exch{"2H5' SCD"}="H5'2 SCD"; $resn_exch{"1HPB SCD"}="HPB1 SCD";
$resn_exch{"2HPB SCD"}="HPB2 SCD"; $resn_exch{"1HP9 SCD"}="HP91 SCD"; $resn_exch{"2HP9 SCD"}="HP92 SCD";
$resn_exch{"3HP9 SCD"}="HP93 SCD"; $resn_exch{"1HP8 SCD"}="HP81 SCD"; $resn_exch{"2HP8 SCD"}="HP82 SCD";
$resn_exch{"3HP8 SCD"}="HP83 SCD"; $resn_exch{"3HOP SCD"}="HOP3 SCD"; $resn_exch{"2HNP SCD"}="HNP2 SCD";
$resn_exch{"1HP5 SCD"}="HP51 SCD"; $resn_exch{"2HP5 SCD"}="HP52 SCD"; $resn_exch{"1HP4 SCD"}="HP41 SCD";
$resn_exch{"2HP4 SCD"}="HP42 SCD"; $resn_exch{"1HNP SCD"}="HNP1 SCD"; $resn_exch{"1HP2 SCD"}="HP21 SCD";
$resn_exch{"2HP2 SCD"}="HP22 SCD"; $resn_exch{"1HP1 SCD"}="HP11 SCD"; $resn_exch{"2HP1 SCD"}="HP12 SCD";
$resn_exch{"1HPS SCD"}="HPS1 SCD"; $resn_exch{"2HPS SCD"}="HPS2 SCD"; $resn_exch{"1HS2 SCD"}="HS21 SCD";
$resn_exch{"2HS2 SCD"}="HS22 SCD"; $resn_exch{"1HS3 SCD"}="HS31 SCD"; $resn_exch{"2HS3 SCD"}="HS32 SCD";
$resn_exch{"5HOS SCD"}="HOS5 SCD"; $resn_exch{"1H11 SCE"}="H111 SCE"; $resn_exch{"2H11 SCE"}="H112 SCE";
$resn_exch{"1H18 SCE"}="H181 SCE"; $resn_exch{"2H18 SCE"}="H182 SCE"; $resn_exch{"3H18 SCE"}="H183 SCE";
$resn_exch{"1H12 SCF"}="H121 SCF"; $resn_exch{"2H12 SCF"}="H122 SCF"; $resn_exch{"1H9  SCG"}=" H91 SCG";
$resn_exch{"2H9  SCG"}=" H92 SCG"; $resn_exch{"1H8  SCG"}=" H81 SCG"; $resn_exch{"2H8  SCG"}=" H82 SCG";
$resn_exch{"1H15 SCG"}="H151 SCG"; $resn_exch{"2H15 SCG"}="H152 SCG"; $resn_exch{"3H15 SCG"}="H153 SCG";
$resn_exch{"1H23 SCG"}="H231 SCG"; $resn_exch{"2H23 SCG"}="H232 SCG"; $resn_exch{" HN  SCH"}=" H2  SCH";
$resn_exch{"1HB  SCH"}=" HB2 SCH"; $resn_exch{"2HB  SCH"}=" HB3 SCH"; $resn_exch{"1HE  SCH"}=" HE1 SCH";
$resn_exch{"2HE  SCH"}=" HE2 SCH"; $resn_exch{"3HE  SCH"}=" HE3 SCH"; $resn_exch{"1HN  SCI"}=" HN1 SCI";
$resn_exch{"2HN  SCI"}=" HN2 SCI"; $resn_exch{"1HB  SCI"}=" HB1 SCI"; $resn_exch{"2HB  SCI"}=" HB2 SCI";
$resn_exch{"1HG  SCI"}=" HG1 SCI"; $resn_exch{"2HG  SCI"}=" HG2 SCI"; $resn_exch{"1HD  SCI"}=" HD1 SCI";
$resn_exch{"2HD  SCI"}=" HD2 SCI"; $resn_exch{"1HN1 SCI"}="HN11 SCI"; $resn_exch{"2HN1 SCI"}="HN12 SCI";
$resn_exch{"1H12 SCJ"}="H121 SCJ"; $resn_exch{"2H12 SCJ"}="H122 SCJ"; $resn_exch{"1H15 SCK"}="H151 SCK";
$resn_exch{"2H15 SCK"}="H152 SCK"; $resn_exch{"1H16 SCK"}="H161 SCK"; $resn_exch{"2H16 SCK"}="H162 SCK";
$resn_exch{"1H20 SCK"}="H201 SCK"; $resn_exch{"2H20 SCK"}="H202 SCK"; $resn_exch{"3H20 SCK"}="H203 SCK";
$resn_exch{"1H19 SCK"}="H191 SCK"; $resn_exch{"2H19 SCK"}="H192 SCK"; $resn_exch{"3H19 SCK"}="H193 SCK";
$resn_exch{"1H18 SCK"}="H181 SCK"; $resn_exch{"2H18 SCK"}="H182 SCK"; $resn_exch{"3H18 SCK"}="H183 SCK";
$resn_exch{"1H11 SCK"}="H111 SCK"; $resn_exch{"2H11 SCK"}="H112 SCK"; $resn_exch{"1H6  SCK"}=" H61 SCK";
$resn_exch{"2H6  SCK"}=" H62 SCK"; $resn_exch{"1H3  SCK"}=" H31 SCK"; $resn_exch{"2H3  SCK"}=" H32 SCK";
$resn_exch{"1H2  SCK"}=" H21 SCK"; $resn_exch{"2H2  SCK"}=" H22 SCK"; $resn_exch{"1H8  SCK"}=" H81 SCK";
$resn_exch{"2H8  SCK"}=" H82 SCK"; $resn_exch{"3H8  SCK"}=" H83 SCK"; $resn_exch{"1H9  SCK"}=" H91 SCK";
$resn_exch{"2H9  SCK"}=" H92 SCK"; $resn_exch{"3H9  SCK"}=" H93 SCK"; $resn_exch{"1H10 SCK"}="H101 SCK";
$resn_exch{"2H10 SCK"}="H102 SCK"; $resn_exch{"3H10 SCK"}="H103 SCK"; $resn_exch{"1H9  SCL"}=" H91 SCL";
$resn_exch{"2H9  SCL"}=" H92 SCL"; $resn_exch{"3H9  SCL"}=" H93 SCL"; $resn_exch{"1H2M SCM"}="H2M1 SCM";
$resn_exch{"2H2M SCM"}="H2M2 SCM"; $resn_exch{"3H2M SCM"}="H2M3 SCM"; $resn_exch{"1H3  SCM"}=" H31 SCM";
$resn_exch{"2H3  SCM"}=" H32 SCM"; $resn_exch{"1H8M SCM"}="H8M1 SCM"; $resn_exch{"2H8M SCM"}="H8M2 SCM";
$resn_exch{"3H8M SCM"}="H8M3 SCM"; $resn_exch{"1H1M SCM"}="H1M1 SCM"; $resn_exch{"2H1M SCM"}="H1M2 SCM";
$resn_exch{"3H1M SCM"}="H1M3 SCM"; $resn_exch{"1HN6 SCO"}="HN61 SCO"; $resn_exch{"2HN6 SCO"}="HN62 SCO";
$resn_exch{"*HC2 SCO"}="HC2' SCO"; $resn_exch{"1HC5 SCO"}="HC51 SCO"; $resn_exch{"2HC5 SCO"}="HC52 SCO";
$resn_exch{"1H11 SCO"}="H111 SCO"; $resn_exch{"2H11 SCO"}="H112 SCO"; $resn_exch{"1H13 SCO"}="H131 SCO";
$resn_exch{"2H13 SCO"}="H132 SCO"; $resn_exch{"3H13 SCO"}="H133 SCO"; $resn_exch{"1H14 SCO"}="H141 SCO";
$resn_exch{"2H14 SCO"}="H142 SCO"; $resn_exch{"3H14 SCO"}="H143 SCO"; $resn_exch{"1H27 SCO"}="H271 SCO";
$resn_exch{"2H27 SCO"}="H272 SCO"; $resn_exch{"1H26 SCO"}="H261 SCO"; $resn_exch{"2H26 SCO"}="H262 SCO";
$resn_exch{"1H33 SCO"}="H331 SCO"; $resn_exch{"2H33 SCO"}="H332 SCO"; $resn_exch{"1H32 SCO"}="H321 SCO";
$resn_exch{"2H32 SCO"}="H322 SCO"; $resn_exch{"1H42 SCO"}="H421 SCO"; $resn_exch{"2H42 SCO"}="H422 SCO";
$resn_exch{"1H43 SCO"}="H431 SCO"; $resn_exch{"2H43 SCO"}="H432 SCO"; $resn_exch{"1H2A SCP"}="H2A1 SCP";
$resn_exch{"2H2A SCP"}="H2A2 SCP"; $resn_exch{"3H2A SCP"}="H2A3 SCP"; $resn_exch{"1H4A SCP"}="H4A1 SCP";
$resn_exch{"2H4A SCP"}="H4A2 SCP"; $resn_exch{"1H5A SCP"}="H5A1 SCP"; $resn_exch{"2H5A SCP"}="H5A2 SCP";
$resn_exch{"2HOP SCP"}="HOP2 SCP"; $resn_exch{"3HOP SCP"}="HOP3 SCP"; $resn_exch{"1HB  SCP"}=" HB1 SCP";
$resn_exch{"2HB  SCP"}=" HB2 SCP"; $resn_exch{"1H12 SCQ"}="H121 SCQ"; $resn_exch{"2H12 SCQ"}="H122 SCQ";
$resn_exch{"1H6  SCR"}=" H61 SCR"; $resn_exch{"2H6  SCR"}=" H62 SCR"; $resn_exch{"1H11 SCR"}="H111 SCR";
$resn_exch{"2H11 SCR"}="H112 SCR"; $resn_exch{"1H16 SCR"}="H161 SCR"; $resn_exch{"2H16 SCR"}="H162 SCR";
$resn_exch{"1HN  SCS"}=" H   SCS"; $resn_exch{"2HN  SCS"}=" H2  SCS"; $resn_exch{" H   SCS"}=" HC  SCS";
$resn_exch{"1HB  SCS"}=" HB2 SCS"; $resn_exch{"2HB  SCS"}=" HB3 SCS"; $resn_exch{"1HE  SCS"}=" HE1 SCS";
$resn_exch{"2HE  SCS"}=" HE2 SCS"; $resn_exch{"3HE  SCS"}=" HE3 SCS"; $resn_exch{"1HZ  SCS"}=" HZ2 SCS";
$resn_exch{"2HZ  SCS"}=" HZ3 SCS"; $resn_exch{"1H14 SCT"}="H141 SCT"; $resn_exch{"2H14 SCT"}="H142 SCT";
$resn_exch{"1H12 SCT"}="H121 SCT"; $resn_exch{"2H12 SCT"}="H122 SCT"; $resn_exch{"1H15 SCT"}="H151 SCT";
$resn_exch{"2H15 SCT"}="H152 SCT"; $resn_exch{"1H3C SCT"}="H3C1 SCT"; $resn_exch{"2H3C SCT"}="H3C2 SCT";
$resn_exch{"3H3C SCT"}="H3C3 SCT"; $resn_exch{"1H11 SCU"}="H111 SCU"; $resn_exch{"2H11 SCU"}="H112 SCU";
$resn_exch{"1H6  SCU"}=" H61 SCU"; $resn_exch{"2H6  SCU"}=" H62 SCU"; $resn_exch{"1H15 SCU"}="H151 SCU";
$resn_exch{"2H15 SCU"}="H152 SCU"; $resn_exch{"1H16 SCU"}="H161 SCU"; $resn_exch{"2H16 SCU"}="H162 SCU";
$resn_exch{"1H20 SCU"}="H201 SCU"; $resn_exch{"2H20 SCU"}="H202 SCU"; $resn_exch{"3H20 SCU"}="H203 SCU";
$resn_exch{"1H19 SCU"}="H191 SCU"; $resn_exch{"2H19 SCU"}="H192 SCU"; $resn_exch{"3H19 SCU"}="H193 SCU";
$resn_exch{"1H18 SCU"}="H181 SCU"; $resn_exch{"2H18 SCU"}="H182 SCU"; $resn_exch{"3H18 SCU"}="H183 SCU";
$resn_exch{"2HC  SCV"}=" HC2 SCV"; $resn_exch{"1HC3 SCV"}="HC31 SCV"; $resn_exch{"2HC3 SCV"}="HC32 SCV";
$resn_exch{"1HC4 SCV"}="HC41 SCV"; $resn_exch{"2HC4 SCV"}="HC42 SCV"; $resn_exch{"7HC  SCV"}=" HC7 SCV";
$resn_exch{"1HC0 SCV"}="HC01 SCV"; $resn_exch{"1HC2 SCV"}="HC21 SCV"; $resn_exch{"1H13 SCV"}="H131 SCV";
$resn_exch{"2H13 SCV"}="H132 SCV"; $resn_exch{"3H13 SCV"}="H133 SCV"; $resn_exch{"1HC5 SCV"}="HC51 SCV";
$resn_exch{"2HC5 SCV"}="HC52 SCV"; $resn_exch{"1HN1 SCV"}="HN11 SCV"; $resn_exch{"2HN1 SCV"}="HN12 SCV";
$resn_exch{"2HO  SCV"}=" HO2 SCV"; $resn_exch{"1H14 SCV"}="H141 SCV"; $resn_exch{"2H14 SCV"}="H142 SCV";
$resn_exch{"3H14 SCV"}="H143 SCV"; $resn_exch{"6HO  SCV"}=" HO6 SCV"; $resn_exch{"2HN  SCV"}=" HN2 SCV";
$resn_exch{"1H12 SCW"}="H121 SCW"; $resn_exch{"2H12 SCW"}="H122 SCW"; $resn_exch{"1H12 SCX"}="H121 SCX";
$resn_exch{"2H12 SCX"}="H122 SCX"; $resn_exch{"1H24 SCX"}="H241 SCX"; $resn_exch{"2H24 SCX"}="H242 SCX";
$resn_exch{"1H25 SCX"}="H251 SCX"; $resn_exch{"2H25 SCX"}="H252 SCX"; $resn_exch{"1H26 SCX"}="H261 SCX";
$resn_exch{"2H26 SCX"}="H262 SCX"; $resn_exch{"3H26 SCX"}="H263 SCX"; $resn_exch{" HN2 SCY"}=" H2  SCY";
$resn_exch{"1HB  SCY"}=" HB2 SCY"; $resn_exch{"2HB  SCY"}=" HB3 SCY"; $resn_exch{"1HE  SCY"}=" HE1 SCY";
$resn_exch{"2HE  SCY"}=" HE2 SCY"; $resn_exch{"3HE  SCY"}=" HE3 SCY"; $resn_exch{"1H12 SCZ"}="H121 SCZ";
$resn_exch{"2H12 SCZ"}="H122 SCZ"; $resn_exch{"1H25 SCZ"}="H251 SCZ"; $resn_exch{"2H25 SCZ"}="H252 SCZ";
$resn_exch{"1H26 SCZ"}="H261 SCZ"; $resn_exch{"2H26 SCZ"}="H262 SCZ"; $resn_exch{"1HBE SD2"}="HBE1 SD2";
$resn_exch{"2HBE SD2"}="HBE2 SD2"; $resn_exch{"3HBE SD2"}="HBE3 SD2"; $resn_exch{"1HBC SD2"}="HBC1 SD2";
$resn_exch{"2HBC SD2"}="HBC2 SD2"; $resn_exch{"1HBB SD2"}="HBB1 SD2"; $resn_exch{"2HBB SD2"}="HBB2 SD2";
$resn_exch{"1HBF SD2"}="HBF1 SD2"; $resn_exch{"2HBF SD2"}="HBF2 SD2"; $resn_exch{"1HAU SD2"}="HAU1 SD2";
$resn_exch{"2HAU SD2"}="HAU2 SD2"; $resn_exch{"1HAV SD2"}="HAV1 SD2"; $resn_exch{"2HAV SD2"}="HAV2 SD2";
$resn_exch{"1HAW SD2"}="HAW1 SD2"; $resn_exch{"2HAW SD2"}="HAW2 SD2"; $resn_exch{"1HAD SD2"}="HAD1 SD2";
$resn_exch{"2HAD SD2"}="HAD2 SD2"; $resn_exch{"1HAI SD2"}="HAI1 SD2"; $resn_exch{"2HAI SD2"}="HAI2 SD2";
$resn_exch{"1H2  SD8"}=" H21 SD8"; $resn_exch{"2H2  SD8"}=" H22 SD8"; $resn_exch{"1H9  SD8"}=" H91 SD8";
$resn_exch{"2H9  SD8"}=" H92 SD8"; $resn_exch{"1H10 SD8"}="H101 SD8"; $resn_exch{"2H10 SD8"}="H102 SD8";
$resn_exch{"1H11 SD8"}="H111 SD8"; $resn_exch{"2H11 SD8"}="H112 SD8"; $resn_exch{"1H12 SD8"}="H121 SD8";
$resn_exch{"2H12 SD8"}="H122 SD8"; $resn_exch{"1H17 SD8"}="H171 SD8"; $resn_exch{"2H17 SD8"}="H172 SD8";
$resn_exch{"1H18 SD8"}="H181 SD8"; $resn_exch{"2H18 SD8"}="H182 SD8"; $resn_exch{"3H18 SD8"}="H183 SD8";
$resn_exch{"1H13 SDA"}="H131 SDA"; $resn_exch{"2H13 SDA"}="H132 SDA"; $resn_exch{"1H10 SDA"}="H101 SDA";
$resn_exch{"2H10 SDA"}="H102 SDA"; $resn_exch{"1HP2 SDC"}="HP21 SDC"; $resn_exch{"2HP2 SDC"}="HP22 SDC";
$resn_exch{"1H06 SDC"}="H061 SDC"; $resn_exch{"2H06 SDC"}="H062 SDC"; $resn_exch{"1H08 SDC"}="H081 SDC";
$resn_exch{"2H08 SDC"}="H082 SDC"; $resn_exch{"1H09 SDC"}="H091 SDC"; $resn_exch{"2H09 SDC"}="H092 SDC";
$resn_exch{"1H12 SDC"}="H121 SDC"; $resn_exch{"2H12 SDC"}="H122 SDC"; $resn_exch{"1H7  SDK"}=" H71 SDK";
$resn_exch{"2H7  SDK"}=" H72 SDK"; $resn_exch{"1H12 SDK"}="H121 SDK"; $resn_exch{"2H12 SDK"}="H122 SDK";
$resn_exch{"1H14 SDK"}="H141 SDK"; $resn_exch{"2H14 SDK"}="H142 SDK"; $resn_exch{"3H14 SDK"}="H143 SDK";
$resn_exch{"1H15 SDK"}="H151 SDK"; $resn_exch{"2H15 SDK"}="H152 SDK"; $resn_exch{"3H15 SDK"}="H153 SDK";
$resn_exch{"1H19 SDK"}="H191 SDK"; $resn_exch{"2H19 SDK"}="H192 SDK"; $resn_exch{"1H29 SDK"}="H291 SDK";
$resn_exch{"2H29 SDK"}="H292 SDK"; $resn_exch{"1H34 SDK"}="H341 SDK"; $resn_exch{"2H34 SDK"}="H342 SDK";
$resn_exch{"1H36 SDK"}="H361 SDK"; $resn_exch{"2H36 SDK"}="H362 SDK"; $resn_exch{"3H36 SDK"}="H363 SDK";
$resn_exch{"1H37 SDK"}="H371 SDK"; $resn_exch{"2H37 SDK"}="H372 SDK"; $resn_exch{"3H37 SDK"}="H373 SDK";
$resn_exch{"1H41 SDK"}="H411 SDK"; $resn_exch{"2H41 SDK"}="H412 SDK"; $resn_exch{" HN2 SDP"}=" H2  SDP";
$resn_exch{"1HB  SDP"}=" HB2 SDP"; $resn_exch{"2HB  SDP"}=" HB3 SDP"; $resn_exch{"1H31 SDP"}="H311 SDP";
$resn_exch{"2H31 SDP"}="H312 SDP"; $resn_exch{"1H32 SDP"}="H321 SDP"; $resn_exch{"2H32 SDP"}="H322 SDP";
$resn_exch{"3H32 SDP"}="H323 SDP"; $resn_exch{"1H41 SDP"}="H411 SDP"; $resn_exch{"2H41 SDP"}="H412 SDP";
$resn_exch{"1H42 SDP"}="H421 SDP"; $resn_exch{"2H42 SDP"}="H422 SDP"; $resn_exch{"3H42 SDP"}="H423 SDP";
$resn_exch{"1H1C SDS"}="H1C1 SDS"; $resn_exch{"2H1C SDS"}="H1C2 SDS"; $resn_exch{"1H2C SDS"}="H2C1 SDS";
$resn_exch{"2H2C SDS"}="H2C2 SDS"; $resn_exch{"3H2C SDS"}="H2C3 SDS"; $resn_exch{"1H3C SDS"}="H3C1 SDS";
$resn_exch{"2H3C SDS"}="H3C2 SDS"; $resn_exch{"1H4C SDS"}="H4C1 SDS"; $resn_exch{"2H4C SDS"}="H4C2 SDS";
$resn_exch{"1H5C SDS"}="H5C1 SDS"; $resn_exch{"2H5C SDS"}="H5C2 SDS"; $resn_exch{"1H6C SDS"}="H6C1 SDS";
$resn_exch{"2H6C SDS"}="H6C2 SDS"; $resn_exch{"1H7C SDS"}="H7C1 SDS"; $resn_exch{"2H7C SDS"}="H7C2 SDS";
$resn_exch{"1H8C SDS"}="H8C1 SDS"; $resn_exch{"2H8C SDS"}="H8C2 SDS"; $resn_exch{"1H9C SDS"}="H9C1 SDS";
$resn_exch{"2H9C SDS"}="H9C2 SDS"; $resn_exch{"1H10 SDS"}="H101 SDS"; $resn_exch{"2H10 SDS"}="H102 SDS";
$resn_exch{"1H11 SDS"}="H111 SDS"; $resn_exch{"2H11 SDS"}="H112 SDS"; $resn_exch{"1H12 SDS"}="H121 SDS";
$resn_exch{"2H12 SDS"}="H122 SDS"; $resn_exch{"1H7  SDZ"}=" H71 SDZ"; $resn_exch{"2H7  SDZ"}=" H72 SDZ";
$resn_exch{"1H8  SDZ"}=" H81 SDZ"; $resn_exch{"2H8  SDZ"}=" H82 SDZ"; $resn_exch{"1H9  SDZ"}=" H91 SDZ";
$resn_exch{"2H9  SDZ"}=" H92 SDZ"; $resn_exch{"1H10 SDZ"}="H101 SDZ"; $resn_exch{"2H10 SDZ"}="H102 SDZ";
$resn_exch{"1H11 SDZ"}="H111 SDZ"; $resn_exch{"2H11 SDZ"}="H112 SDZ"; $resn_exch{"1H13 SDZ"}="H131 SDZ";
$resn_exch{"2H13 SDZ"}="H132 SDZ"; $resn_exch{"1H21 SDZ"}="H211 SDZ"; $resn_exch{"2H21 SDZ"}="H212 SDZ";
$resn_exch{"1H22 SDZ"}="H221 SDZ"; $resn_exch{"2H22 SDZ"}="H222 SDZ"; $resn_exch{"1H23 SDZ"}="H231 SDZ";
$resn_exch{"2H23 SDZ"}="H232 SDZ"; $resn_exch{"1H24 SDZ"}="H241 SDZ"; $resn_exch{"2H24 SDZ"}="H242 SDZ";
$resn_exch{"1H25 SDZ"}="H251 SDZ"; $resn_exch{"2H25 SDZ"}="H252 SDZ"; $resn_exch{"1HN  SEA"}=" HN1 SEA";
$resn_exch{"2HN  SEA"}=" HN2 SEA"; $resn_exch{"1HA  SEA"}=" HA1 SEA"; $resn_exch{"2HA  SEA"}=" HA2 SEA";
$resn_exch{"1HB  SEA"}=" HB1 SEA"; $resn_exch{"2HB  SEA"}=" HB2 SEA"; $resn_exch{" HN2 SEB"}=" H2  SEB";
$resn_exch{"1HE  SEB"}=" HE3 SEB"; $resn_exch{"2HE  SEB"}=" HE2 SEB"; $resn_exch{"1HB  SEB"}=" HB3 SEB";
$resn_exch{"2HB  SEB"}=" HB2 SEB"; $resn_exch{"1HN  SEC"}=" H   SEC"; $resn_exch{"2HN  SEC"}=" H2  SEC";
$resn_exch{"1HB  SEC"}=" HB2 SEC"; $resn_exch{"2HB  SEC"}=" HB3 SEC"; $resn_exch{"1HN  SEG"}=" H   SEG";
$resn_exch{"2HN  SEG"}=" H2  SEG"; $resn_exch{" HOD SEG"}=" HD  SEG"; $resn_exch{"1HB  SEG"}=" HB3 SEG";
$resn_exch{"2HB  SEG"}=" HB2 SEG"; $resn_exch{"1H1C SEH"}="H1C1 SEH"; $resn_exch{"2H1C SEH"}="H1C2 SEH";
$resn_exch{"1H2C SEH"}="H2C1 SEH"; $resn_exch{"2H2C SEH"}="H2C2 SEH"; $resn_exch{"3H2C SEH"}="H2C3 SEH";
$resn_exch{"1HBC SEH"}="HBC1 SEH"; $resn_exch{"2HBC SEH"}="HBC2 SEH"; $resn_exch{"1H2  SEI"}=" H21 SEI";
$resn_exch{"2H2  SEI"}=" H22 SEI"; $resn_exch{"1H3  SEI"}=" H31 SEI"; $resn_exch{"2H3  SEI"}=" H32 SEI";
$resn_exch{"1H20 SEI"}="H201 SEI"; $resn_exch{"1H21 SEI"}="H211 SEI"; $resn_exch{"1H30 SEI"}="H301 SEI";
$resn_exch{"1H31 SEI"}="H311 SEI"; $resn_exch{"2H31 SEI"}="H312 SEI"; $resn_exch{"3H31 SEI"}="H313 SEI";
$resn_exch{"1H35 SEI"}="H351 SEI"; $resn_exch{"2H35 SEI"}="H352 SEI"; $resn_exch{"3H35 SEI"}="H353 SEI";
$resn_exch{"1H5  SEI"}=" H51 SEI"; $resn_exch{"2H5  SEI"}=" H52 SEI"; $resn_exch{"1H6  SEI"}=" H61 SEI";
$resn_exch{"2H6  SEI"}=" H62 SEI"; $resn_exch{"1H17 SEI"}="H171 SEI"; $resn_exch{"2H18 SEI"}="H182 SEI";
$resn_exch{"1H28 SEI"}="H281 SEI"; $resn_exch{"1H29 SEI"}="H291 SEI"; $resn_exch{"1H44 SEI"}="H441 SEI";
$resn_exch{"1H45 SEI"}="H451 SEI"; $resn_exch{"2H45 SEI"}="H452 SEI"; $resn_exch{"1H46 SEI"}="H461 SEI";
$resn_exch{"2H46 SEI"}="H462 SEI"; $resn_exch{"1H47 SEI"}="H471 SEI"; $resn_exch{"2H47 SEI"}="H472 SEI";
$resn_exch{"1H53 SEI"}="H531 SEI"; $resn_exch{"1H59 SEI"}="H591 SEI"; $resn_exch{"1H61 SEI"}="H611 SEI";
$resn_exch{"1H62 SEI"}="H621 SEI"; $resn_exch{"2H62 SEI"}="H622 SEI"; $resn_exch{"3H62 SEI"}="H623 SEI";
$resn_exch{"1H66 SEI"}="H661 SEI"; $resn_exch{"2H66 SEI"}="H662 SEI"; $resn_exch{"3H66 SEI"}="H663 SEI";
$resn_exch{"1HN1 SEL"}="HN11 SEL"; $resn_exch{"2HN1 SEL"}="HN12 SEL"; $resn_exch{"1HA  SEL"}=" HA1 SEL";
$resn_exch{"1HB  SEL"}=" HB1 SEL"; $resn_exch{"2HB  SEL"}=" HB2 SEL"; $resn_exch{"1HB2 SEL"}="HB21 SEL";
$resn_exch{"2HB2 SEL"}="HB22 SEL"; $resn_exch{"1HO2 SEL"}="HO21 SEL"; $resn_exch{"1H1  SEM"}=" H11 SEM";
$resn_exch{"2H1  SEM"}=" H12 SEM"; $resn_exch{"3H1  SEM"}=" H13 SEM"; $resn_exch{"1HN3 SEM"}="HN31 SEM";
$resn_exch{"2HN3 SEM"}="HN32 SEM"; $resn_exch{"1H4  SEM"}=" H41 SEM"; $resn_exch{"2H4  SEM"}=" H42 SEM";
$resn_exch{"1H'  SEM"}=" H'1 SEM"; $resn_exch{"2H'  SEM"}=" H'2 SEM"; $resn_exch{"1HO  SEO"}=" HO1 SEO";
$resn_exch{"1HB1 SEO"}="HB11 SEO"; $resn_exch{"2HB1 SEO"}="HB12 SEO"; $resn_exch{"1HB  SEO"}=" HB1 SEO";
$resn_exch{"2HB  SEO"}=" HB2 SEO"; $resn_exch{"2HN  SEP"}=" H2  SEP"; $resn_exch{"1HB  SEP"}=" HB2 SEP";
$resn_exch{"2HB  SEP"}=" HB3 SEP"; $resn_exch{"2HOP SEP"}="HOP2 SEP"; $resn_exch{"3HOP SEP"}="HOP3 SEP";
$resn_exch{" HN2 SER"}=" H2  SER"; $resn_exch{"1HB  SER"}=" HB2 SER"; $resn_exch{"2HB  SER"}=" HB3 SER";
$resn_exch{"1H1  SES"}=" H11 SES"; $resn_exch{"2H1  SES"}=" H12 SES"; $resn_exch{"3H1  SES"}=" H13 SES";
$resn_exch{"1H2  SES"}=" H21 SES"; $resn_exch{"2H2  SES"}=" H22 SES"; $resn_exch{"1H3  SES"}=" H31 SES";
$resn_exch{"2H3  SES"}=" H32 SES"; $resn_exch{"1H4  SES"}=" H41 SES"; $resn_exch{"2H4  SES"}=" H42 SES";
$resn_exch{"1H11 SES"}="H111 SES"; $resn_exch{"2H11 SES"}="H112 SES"; $resn_exch{"3H11 SES"}="H113 SES";
$resn_exch{"1H12 SES"}="H121 SES"; $resn_exch{"2H12 SES"}="H122 SES"; $resn_exch{"1H13 SES"}="H131 SES";
$resn_exch{"2H13 SES"}="H132 SES"; $resn_exch{"1HN2 SES"}="HN21 SES"; $resn_exch{"2HN2 SES"}="HN22 SES";
$resn_exch{"1HN  SET"}=" H   SET"; $resn_exch{"2HN  SET"}=" H2  SET"; $resn_exch{"1HB  SET"}=" HB2 SET";
$resn_exch{"2HB  SET"}=" HB3 SET"; $resn_exch{" HOG SET"}=" HG  SET"; $resn_exch{"1HNT SET"}="HNT1 SET";
$resn_exch{"2HNT SET"}="HNT2 SET"; $resn_exch{"1H1  SEU"}=" H11 SEU"; $resn_exch{"2H1  SEU"}=" H12 SEU";
$resn_exch{"3H1  SEU"}=" H13 SEU"; $resn_exch{"1H2  SEU"}=" H21 SEU"; $resn_exch{"2H2  SEU"}=" H22 SEU";
$resn_exch{"1HN6 SEU"}="HN61 SEU"; $resn_exch{"2HN6 SEU"}="HN62 SEU"; $resn_exch{"1HAQ SF1"}="HAQ1 SF1";
$resn_exch{"2HAQ SF1"}="HAQ2 SF1"; $resn_exch{"1HAA SF2"}="HAA1 SF2"; $resn_exch{"2HAA SF2"}="HAA2 SF2";
$resn_exch{"3HAA SF2"}="HAA3 SF2"; $resn_exch{"1HAQ SF2"}="HAQ1 SF2"; $resn_exch{"2HAQ SF2"}="HAQ2 SF2";
$resn_exch{"1H43 SFA"}="H431 SFA"; $resn_exch{"2H43 SFA"}="H432 SFA"; $resn_exch{"1H44 SFA"}="H441 SFA";
$resn_exch{"2H44 SFA"}="H442 SFA"; $resn_exch{"3H44 SFA"}="H443 SFA"; $resn_exch{"1H39 SFA"}="H391 SFA";
$resn_exch{"2H39 SFA"}="H392 SFA"; $resn_exch{"1H45 SFA"}="H451 SFA"; $resn_exch{"2H45 SFA"}="H452 SFA";
$resn_exch{"3H45 SFA"}="H453 SFA"; $resn_exch{"1H46 SFA"}="H461 SFA"; $resn_exch{"2H46 SFA"}="H462 SFA";
$resn_exch{"3H46 SFA"}="H463 SFA"; $resn_exch{"1H47 SFA"}="H471 SFA"; $resn_exch{"2H47 SFA"}="H472 SFA";
$resn_exch{"3H47 SFA"}="H473 SFA"; $resn_exch{"1H32 SFA"}="H321 SFA"; $resn_exch{"2H32 SFA"}="H322 SFA";
$resn_exch{"1H48 SFA"}="H481 SFA"; $resn_exch{"2H48 SFA"}="H482 SFA"; $resn_exch{"3H48 SFA"}="H483 SFA";
$resn_exch{"1H29 SFA"}="H291 SFA"; $resn_exch{"2H29 SFA"}="H292 SFA"; $resn_exch{"1H28 SFA"}="H281 SFA";
$resn_exch{"2H28 SFA"}="H282 SFA"; $resn_exch{"1H49 SFA"}="H491 SFA"; $resn_exch{"2H49 SFA"}="H492 SFA";
$resn_exch{"3H49 SFA"}="H493 SFA"; $resn_exch{"1H22 SFA"}="H221 SFA"; $resn_exch{"2H22 SFA"}="H222 SFA";
$resn_exch{"1H50 SFA"}="H501 SFA"; $resn_exch{"2H50 SFA"}="H502 SFA"; $resn_exch{"3H50 SFA"}="H503 SFA";
$resn_exch{"1H51 SFA"}="H511 SFA"; $resn_exch{"2H51 SFA"}="H512 SFA"; $resn_exch{"1H52 SFA"}="H521 SFA";
$resn_exch{"2H52 SFA"}="H522 SFA"; $resn_exch{"1H54 SFA"}="H541 SFA"; $resn_exch{"2H54 SFA"}="H542 SFA";
$resn_exch{"3H54 SFA"}="H543 SFA"; $resn_exch{"1H56 SFA"}="H561 SFA"; $resn_exch{"2H56 SFA"}="H562 SFA";
$resn_exch{"3H56 SFA"}="H563 SFA"; $resn_exch{"1H57 SFA"}="H571 SFA"; $resn_exch{"2H57 SFA"}="H572 SFA";
$resn_exch{"3H57 SFA"}="H573 SFA"; $resn_exch{"1H58 SFA"}="H581 SFA"; $resn_exch{"2H58 SFA"}="H582 SFA";
$resn_exch{"1H5  SFA"}=" H51 SFA"; $resn_exch{"2H5  SFA"}=" H52 SFA"; $resn_exch{"1H4  SFA"}=" H41 SFA";
$resn_exch{"2H4  SFA"}="H42A SFA"; $resn_exch{"1H3  SFA"}="H31A SFA"; $resn_exch{"2H3  SFA"}=" H32 SFA";
$resn_exch{" H3* SFC"}=" H3' SFC"; $resn_exch{" H2* SFC"}=" H2' SFC"; $resn_exch{" H1* SFC"}=" H1' SFC";
$resn_exch{"1HN6 SFC"}="HN61 SFC"; $resn_exch{"2HN6 SFC"}="HN62 SFC"; $resn_exch{" H4* SFC"}=" H4' SFC";
$resn_exch{"1H5* SFC"}="H5'1 SFC"; $resn_exch{"2H5* SFC"}="H5'2 SFC"; $resn_exch{"1H4  SFC"}=" H41 SFC";
$resn_exch{"2H4  SFC"}=" H42 SFC"; $resn_exch{"1H1  SFC"}="H11X SFC"; $resn_exch{"2H1  SFC"}=" H12 SFC";
$resn_exch{"3H1  SFC"}=" H13 SFC"; $resn_exch{"1H14 SFC"}="H141 SFC"; $resn_exch{"2H14 SFC"}="H142 SFC";
$resn_exch{"3H14 SFC"}="H143 SFC"; $resn_exch{"1H7  SFC"}=" H71 SFC"; $resn_exch{"2H7  SFC"}=" H72 SFC";
$resn_exch{"1H6  SFC"}=" H61 SFC"; $resn_exch{"2H6  SFC"}=" H62 SFC"; $resn_exch{"1H3  SFC"}=" H31 SFC";
$resn_exch{"2H3  SFC"}=" H32 SFC"; $resn_exch{"1H2  SFC"}=" H21 SFC"; $resn_exch{"2H2  SFC"}=" H22 SFC";
$resn_exch{"1H13 SFC"}="H131 SFC"; $resn_exch{"2H13 SFC"}="H132 SFC"; $resn_exch{"3H13 SFC"}="H133 SFC";
$resn_exch{"1H9  SFC"}=" H91 SFC"; $resn_exch{"2H9  SFC"}=" H92 SFC"; $resn_exch{"1H12 SFC"}="H121 SFC";
$resn_exch{"2H12 SFC"}="H122 SFC"; $resn_exch{"3H12 SFC"}="H123 SFC"; $resn_exch{"1H11 SFC"}="H111 SFC";
$resn_exch{"2H11 SFC"}="H112 SFC"; $resn_exch{"3H11 SFC"}="H113 SFC"; $resn_exch{"1H3F SFD"}="H3F1 SFD";
$resn_exch{"2H3F SFD"}="H3F2 SFD"; $resn_exch{"3H3F SFD"}="H3F3 SFD"; $resn_exch{"1HAF SFD"}="HAF1 SFD";
$resn_exch{"2HAF SFD"}="HAF2 SFD"; $resn_exch{"3HAF SFD"}="HAF3 SFD"; $resn_exch{"1H1R SFD"}="H1R1 SFD";
$resn_exch{"2H1R SFD"}="H1R2 SFD"; $resn_exch{"1H5R SFD"}="H5R1 SFD"; $resn_exch{"2H5R SFD"}="H5R2 SFD";
$resn_exch{"1H0R SFD"}="H0R1 SFD"; $resn_exch{"2H0R SFD"}="H0R2 SFD"; $resn_exch{"1HN9 SFD"}="HN91 SFD";
$resn_exch{"2HN9 SFD"}="HN92 SFD"; $resn_exch{"1HN  SFG"}=" HN1 SFG"; $resn_exch{"2HN  SFG"}=" HN2 SFG";
$resn_exch{"1HB  SFG"}=" HB1 SFG"; $resn_exch{"2HB  SFG"}=" HB2 SFG"; $resn_exch{"1HG  SFG"}=" HG1 SFG";
$resn_exch{"2HG  SFG"}=" HG2 SFG"; $resn_exch{"1HNE SFG"}="HNE1 SFG"; $resn_exch{"2HNE SFG"}="HNE2 SFG";
$resn_exch{"1H5* SFG"}="H5'1 SFG"; $resn_exch{"2H5* SFG"}="H5'2 SFG"; $resn_exch{" H4* SFG"}=" H4' SFG";
$resn_exch{" H3* SFG"}=" H3' SFG"; $resn_exch{"*HO3 SFG"}="HO3' SFG"; $resn_exch{" H2* SFG"}=" H2' SFG";
$resn_exch{"*HO2 SFG"}="HO2' SFG"; $resn_exch{" H1* SFG"}=" H1' SFG"; $resn_exch{"1HN6 SFG"}="HN61 SFG";
$resn_exch{"2HN6 SFG"}="HN62 SFG"; $resn_exch{"1H54 SFM"}="H541 SFM"; $resn_exch{"2H54 SFM"}="H542 SFM";
$resn_exch{"3H54 SFM"}="H543 SFM"; $resn_exch{"1H52 SFM"}="H521 SFM"; $resn_exch{"2H52 SFM"}="H522 SFM";
$resn_exch{"1H51 SFM"}="H511 SFM"; $resn_exch{"2H51 SFM"}="H512 SFM"; $resn_exch{"1H56 SFM"}="H561 SFM";
$resn_exch{"2H56 SFM"}="H562 SFM"; $resn_exch{"3H56 SFM"}="H563 SFM"; $resn_exch{"1H57 SFM"}="H571 SFM";
$resn_exch{"2H57 SFM"}="H572 SFM"; $resn_exch{"3H57 SFM"}="H573 SFM"; $resn_exch{"1H58 SFM"}="H581 SFM";
$resn_exch{"2H58 SFM"}="H582 SFM"; $resn_exch{"1H5  SFM"}=" H51 SFM"; $resn_exch{"2H5  SFM"}=" H52 SFM";
$resn_exch{"1H4  SFM"}=" H41 SFM"; $resn_exch{"2H4  SFM"}=" H42 SFM"; $resn_exch{"1H3  SFM"}=" H31 SFM";
$resn_exch{"2H3  SFM"}=" H32 SFM"; $resn_exch{"1H22 SFM"}="H221 SFM"; $resn_exch{"2H22 SFM"}="H222 SFM";
$resn_exch{"1H50 SFM"}="H501 SFM"; $resn_exch{"2H50 SFM"}="H502 SFM"; $resn_exch{"3H50 SFM"}="H503 SFM";
$resn_exch{"1H49 SFM"}="H491 SFM"; $resn_exch{"2H49 SFM"}="H492 SFM"; $resn_exch{"3H49 SFM"}="H493 SFM";
$resn_exch{"1H26 SFM"}="H261 SFM"; $resn_exch{"2H26 SFM"}="H262 SFM"; $resn_exch{"1H2' SFT"}="H2'1 SFT";
$resn_exch{"2H2' SFT"}="H2'2 SFT"; $resn_exch{"3H2' SFT"}="H2'3 SFT"; $resn_exch{"1H2A SFT"}="H2A1 SFT";
$resn_exch{"2H2A SFT"}="H2A2 SFT"; $resn_exch{"1H2B SFT"}="H2B1 SFT"; $resn_exch{"2H2B SFT"}="H2B2 SFT";
$resn_exch{"1H2C SFT"}="H2C1 SFT"; $resn_exch{"2H2C SFT"}="H2C2 SFT"; $resn_exch{"1H2D SFT"}="H2D1 SFT";
$resn_exch{"2H2D SFT"}="H2D2 SFT"; $resn_exch{"1H1' SFT"}="H1'1 SFT"; $resn_exch{"2H1' SFT"}="H1'2 SFT";
$resn_exch{"1H1A SFT"}="H1A1 SFT"; $resn_exch{"2H1A SFT"}="H1A2 SFT"; $resn_exch{"1H1B SFT"}="H1B1 SFT";
$resn_exch{"2H1B SFT"}="H1B2 SFT"; $resn_exch{"1H1C SFT"}="H1C1 SFT"; $resn_exch{"2H1C SFT"}="H1C2 SFT";
$resn_exch{"1H1D SFT"}="H1D1 SFT"; $resn_exch{"2H1D SFT"}="H1D2 SFT"; $resn_exch{"1H1E SFT"}="H1E1 SFT";
$resn_exch{"2H1E SFT"}="H1E2 SFT"; $resn_exch{"1H1F SFT"}="H1F1 SFT"; $resn_exch{"2H1F SFT"}="H1F2 SFT";
$resn_exch{"1H1G SFT"}="H1G1 SFT"; $resn_exch{"2H1G SFT"}="H1G2 SFT"; $resn_exch{"1H1H SFT"}="H1H1 SFT";
$resn_exch{"2H1H SFT"}="H1H2 SFT"; $resn_exch{"1H1I SFT"}="H1I1 SFT"; $resn_exch{"2H1I SFT"}="H1I2 SFT";
$resn_exch{"1H9' SFT"}="H9'1 SFT"; $resn_exch{"2H9' SFT"}="H9'2 SFT"; $resn_exch{"1H8' SFT"}="H8'1 SFT";
$resn_exch{"2H8' SFT"}="H8'2 SFT"; $resn_exch{"1H7' SFT"}="H7'1 SFT"; $resn_exch{"2H7' SFT"}="H7'2 SFT";
$resn_exch{"1H6' SFT"}="H6'1 SFT"; $resn_exch{"2H6' SFT"}="H6'2 SFT"; $resn_exch{"1H5' SFT"}="H5'1 SFT";
$resn_exch{"2H5' SFT"}="H5'2 SFT"; $resn_exch{"1H4' SFT"}="H4'1 SFT"; $resn_exch{"2H4' SFT"}="H4'2 SFT";
$resn_exch{"1H3' SFT"}="H3'1 SFT"; $resn_exch{"2H3' SFT"}="H3'2 SFT"; $resn_exch{"1HA  SFT"}=" HA1 SFT";
$resn_exch{"2HB  SFT"}=" HB2 SFT"; $resn_exch{" H1* SFT"}=" H1' SFT"; $resn_exch{" H2* SFT"}=" H2B SFT";
$resn_exch{" H5* SFT"}=" H5' SFT"; $resn_exch{"1H6* SFT"}="H6A1 SFT"; $resn_exch{"2H6* SFT"}="H6A2 SFT";
$resn_exch{" H6* SFT"}=" H6' SFT"; $resn_exch{" H4* SFT"}=" H4' SFT"; $resn_exch{" H3* SFT"}=" H3' SFT";
$resn_exch{"1H6C SFT"}="H6C1 SFT"; $resn_exch{"2H6C SFT"}="H6C2 SFT"; $resn_exch{"1H7C SFT"}="H7C1 SFT";
$resn_exch{"2H7C SFT"}="H7C2 SFT"; $resn_exch{"1H8C SFT"}="H8C1 SFT"; $resn_exch{"2H8C SFT"}="H8C2 SFT";
$resn_exch{"1H9C SFT"}="H9C1 SFT"; $resn_exch{"2H9C SFT"}="H9C2 SFT"; $resn_exch{"1H10 SFT"}="H101 SFT";
$resn_exch{"2H10 SFT"}="H102 SFT"; $resn_exch{"1H11 SFT"}="H111 SFT"; $resn_exch{"2H11 SFT"}="H112 SFT";
$resn_exch{"1H12 SFT"}="H121 SFT"; $resn_exch{"2H12 SFT"}="H122 SFT"; $resn_exch{"1H13 SFT"}="H131 SFT";
$resn_exch{"2H13 SFT"}="H132 SFT"; $resn_exch{"1H14 SFT"}="H141 SFT"; $resn_exch{"2H14 SFT"}="H142 SFT";
$resn_exch{"1H15 SFT"}="H151 SFT"; $resn_exch{"2H15 SFT"}="H152 SFT"; $resn_exch{"1H16 SFT"}="H161 SFT";
$resn_exch{"2H16 SFT"}="H162 SFT"; $resn_exch{"1H17 SFT"}="H171 SFT"; $resn_exch{"2H17 SFT"}="H172 SFT";
$resn_exch{"1H18 SFT"}="H181 SFT"; $resn_exch{"2H18 SFT"}="H182 SFT"; $resn_exch{"3H18 SFT"}="H183 SFT";
$resn_exch{"1H3  SG1"}=" H31 SG1"; $resn_exch{"1H4  SG1"}=" H41 SG1"; $resn_exch{"1H9  SG1"}=" H91 SG1";
$resn_exch{"2H9  SG1"}=" H92 SG1"; $resn_exch{"1H10 SG1"}="H101 SG1"; $resn_exch{"2H10 SG1"}="H102 SG1";
$resn_exch{"1H11 SG1"}="H111 SG1"; $resn_exch{"2H11 SG1"}="H112 SG1"; $resn_exch{"1H16 SG1"}="H161 SG1";
$resn_exch{"1H17 SG1"}="H171 SG1"; $resn_exch{"2H17 SG1"}="H172 SG1"; $resn_exch{"1H3  SG2"}=" H31 SG2";
$resn_exch{"1H4  SG2"}=" H41 SG2"; $resn_exch{"1H6  SG2"}=" H61 SG2"; $resn_exch{"2H6  SG2"}=" H62 SG2";
$resn_exch{"3H6  SG2"}=" H63 SG2"; $resn_exch{"1H11 SG2"}="H111 SG2"; $resn_exch{"1H12 SG2"}="H121 SG2";
$resn_exch{"2H12 SG2"}="H122 SG2"; $resn_exch{"2H5C SG3"}="H5C2 SG3"; $resn_exch{"2H6C SG3"}="H6C2 SG3";
$resn_exch{"1H12 SG3"}="H121 SG3"; $resn_exch{"2H12 SG3"}="H122 SG3"; $resn_exch{"3H12 SG3"}="H123 SG3";
$resn_exch{"1H9C SG3"}="H9C1 SG3"; $resn_exch{"2H9C SG3"}="H9C2 SG3"; $resn_exch{"1H6  SGA"}=" H61 SGA";
$resn_exch{"2H6  SGA"}=" H62 SGA"; $resn_exch{"3HOS SGA"}="HOS3 SGA"; $resn_exch{"1HN  SGB"}=" HN1 SGB";
$resn_exch{"2HN  SGB"}=" HN2 SGB"; $resn_exch{"1HBC SGB"}="HBC1 SGB"; $resn_exch{"2HBC SGB"}="HBC2 SGB";
$resn_exch{"1H1C SGB"}="H1C1 SGB"; $resn_exch{"2H1C SGB"}="H1C2 SGB"; $resn_exch{"3H1C SGB"}="H1C3 SGB";
$resn_exch{"1H4C SGB"}="H4C1 SGB"; $resn_exch{"2H4C SGB"}="H4C2 SGB"; $resn_exch{"3H4C SGB"}="H4C3 SGB";
$resn_exch{"1H3C SGB"}="H3C1 SGB"; $resn_exch{"2H3C SGB"}="H3C2 SGB"; $resn_exch{"3H3C SGB"}="H3C3 SGB";
$resn_exch{"2H6  SGC"}=" H62 SGC"; $resn_exch{"1H6  SGC"}=" H61 SGC"; $resn_exch{"1HN  SGI"}=" HN1 SGI";
$resn_exch{"2HN  SGI"}=" HN2 SGI"; $resn_exch{"1HA  SGI"}=" HA1 SGI"; $resn_exch{"2HA  SGI"}=" HA2 SGI";
$resn_exch{"1HB  SGI"}=" HB1 SGI"; $resn_exch{"2HB  SGI"}=" HB2 SGI"; $resn_exch{"1H1  SGL"}=" H11 SGL";
$resn_exch{"2H1  SGL"}=" H12 SGL"; $resn_exch{"1H3  SGL"}=" H31 SGL"; $resn_exch{"2H3  SGL"}=" H32 SGL";
$resn_exch{"1H1  SGM"}=" H11 SGM"; $resn_exch{"2H1  SGM"}=" H12 SGM"; $resn_exch{"1H3  SGM"}=" H31 SGM";
$resn_exch{"2H3  SGM"}=" H32 SGM"; $resn_exch{"1H6  SGN"}=" H61 SGN"; $resn_exch{"2H6  SGN"}=" H62 SGN";
$resn_exch{"3HOS SGN"}="HOS3 SGN"; $resn_exch{"6HOS SGN"}="HOS6 SGN"; $resn_exch{"1HOP SGP"}="HOP1 SGP";
$resn_exch{"*HO5 SGP"}="HO5' SGP"; $resn_exch{"1H5* SGP"}="H5'1 SGP"; $resn_exch{"2H5* SGP"}="H5'2 SGP";
$resn_exch{" H4* SGP"}=" H4' SGP"; $resn_exch{" H3* SGP"}=" H3' SGP"; $resn_exch{" H2* SGP"}=" H2' SGP";
$resn_exch{" H1* SGP"}=" H1' SGP"; $resn_exch{"1HN2 SGP"}="HN21 SGP"; $resn_exch{"2HN2 SGP"}="HN22 SGP";
$resn_exch{"1HBC SGR"}="HBC1 SGR"; $resn_exch{"2HBC SGR"}="HBC2 SGR"; $resn_exch{"1H4N SGR"}="H4N1 SGR";
$resn_exch{"2H4N SGR"}="H4N2 SGR"; $resn_exch{"1H3C SGR"}="H3C1 SGR"; $resn_exch{"2H3C SGR"}="H3C2 SGR";
$resn_exch{"3H3C SGR"}="H3C3 SGR"; $resn_exch{"1HA  SGX"}=" HA1 SGX"; $resn_exch{"2HB  SGX"}=" HB2 SGX";
$resn_exch{"1H3N SGX"}="H3N1 SGX"; $resn_exch{"2H3N SGX"}="H3N2 SGX"; $resn_exch{"1H1C SGX"}="H1C1 SGX";
$resn_exch{"2H1C SGX"}="H1C2 SGX"; $resn_exch{"3H1C SGX"}="H1C3 SGX"; $resn_exch{"1H1C SH1"}="H1C1 SH1";
$resn_exch{"2H1C SH1"}="H1C2 SH1"; $resn_exch{"1H2C SH1"}="H2C1 SH1"; $resn_exch{"2H2C SH1"}="H2C2 SH1";
$resn_exch{"1H10 SH1"}="H101 SH1"; $resn_exch{"2H10 SH1"}="H102 SH1"; $resn_exch{"1H13 SH1"}="H131 SH1";
$resn_exch{"2H13 SH1"}="H132 SH1"; $resn_exch{"3H13 SH1"}="H133 SH1"; $resn_exch{"1H16 SH1"}="H161 SH1";
$resn_exch{"2H16 SH1"}="H162 SH1"; $resn_exch{"1H17 SH1"}="H171 SH1"; $resn_exch{"2H17 SH1"}="H172 SH1";
$resn_exch{"1H18 SH1"}="H181 SH1"; $resn_exch{"2H18 SH1"}="H182 SH1"; $resn_exch{"1H5N SH1"}="H5N1 SH1";
$resn_exch{"2H5N SH1"}="H5N2 SH1"; $resn_exch{"1H6N SH1"}="H6N1 SH1"; $resn_exch{"2H6N SH1"}="H6N2 SH1";
$resn_exch{"1H20 SH1"}="H201 SH1"; $resn_exch{"2H20 SH1"}="H202 SH1"; $resn_exch{"1H8  SH4"}=" H81 SH4";
$resn_exch{"2H8  SH4"}=" H82 SH4"; $resn_exch{"3H8  SH4"}=" H83 SH4"; $resn_exch{"1H9  SH4"}=" H91 SH4";
$resn_exch{"2H9  SH4"}=" H92 SH4"; $resn_exch{"1H15 SH4"}="H151 SH4"; $resn_exch{"2H15 SH4"}="H152 SH4";
$resn_exch{"3H15 SH4"}="H153 SH4"; $resn_exch{"1HN  SHC"}=" H   SHC"; $resn_exch{"2HN  SHC"}=" H2  SHC";
$resn_exch{"1HB  SHC"}=" HB2 SHC"; $resn_exch{"2HB  SHC"}=" HB3 SHC"; $resn_exch{"1H1  SHC"}=" H11 SHC";
$resn_exch{"2H1  SHC"}=" H12 SHC"; $resn_exch{"1H2  SHC"}=" H21 SHC"; $resn_exch{"2H2  SHC"}=" H22 SHC";
$resn_exch{"1H3  SHC"}=" H31 SHC"; $resn_exch{"2H3  SHC"}=" H32 SHC"; $resn_exch{"1H4  SHC"}=" H41 SHC";
$resn_exch{"2H4  SHC"}=" H42 SHC"; $resn_exch{"1H5  SHC"}=" H51 SHC"; $resn_exch{"2H5  SHC"}=" H52 SHC";
$resn_exch{"1H6  SHC"}=" H61 SHC"; $resn_exch{"2H6  SHC"}=" H62 SHC"; $resn_exch{"3H6  SHC"}=" H63 SHC";
$resn_exch{"1H5  SHF"}=" H51 SHF"; $resn_exch{"2H5  SHF"}=" H52 SHF"; $resn_exch{"3H5  SHF"}=" H53 SHF";
$resn_exch{"1H3  SHF"}=" H31 SHF"; $resn_exch{"2H3  SHF"}=" H32 SHF"; $resn_exch{"1H2  SHF"}=" H21 SHF";
$resn_exch{"2H2  SHF"}=" H22 SHF"; $resn_exch{"1H2  SHH"}=" H21 SHH"; $resn_exch{"2H2  SHH"}=" H22 SHH";
$resn_exch{"1H3  SHH"}=" H31 SHH"; $resn_exch{"2H3  SHH"}=" H32 SHH"; $resn_exch{"1H4  SHH"}=" H41 SHH";
$resn_exch{"2H4  SHH"}=" H42 SHH"; $resn_exch{"1H5  SHH"}=" H51 SHH"; $resn_exch{"2H5  SHH"}=" H52 SHH";
$resn_exch{"1H6  SHH"}=" H61 SHH"; $resn_exch{"2H6  SHH"}=" H62 SHH"; $resn_exch{"1H7  SHH"}=" H71 SHH";
$resn_exch{"2H7  SHH"}=" H72 SHH"; $resn_exch{"1H10 SHH"}="H101 SHH"; $resn_exch{"1H11 SHH"}="H111 SHH";
$resn_exch{"1H12 SHH"}="H121 SHH"; $resn_exch{"1H13 SHH"}="H131 SHH"; $resn_exch{"1H14 SHH"}="H141 SHH";
$resn_exch{"1H41 SHI"}="H411 SHI"; $resn_exch{"2H41 SHI"}="H412 SHI"; $resn_exch{"1HN1 SHI"}="HN11 SHI";
$resn_exch{"2HN1 SHI"}="HN12 SHI"; $resn_exch{"1H1  SHM"}="H11A SHM"; $resn_exch{"1H2  SHM"}=" H21 SHM";
$resn_exch{"2H1  SHM"}="H12A SHM"; $resn_exch{"2H2  SHM"}=" H22 SHM"; $resn_exch{"3H1  SHM"}=" H13 SHM";
$resn_exch{"3H2  SHM"}=" H23 SHM"; $resn_exch{"4H1  SHM"}=" H14 SHM"; $resn_exch{"4H2  SHM"}=" H24 SHM";
$resn_exch{"5H1  SHM"}=" H15 SHM"; $resn_exch{"5H2  SHM"}=" H25 SHM"; $resn_exch{"1HAC SHO"}="HAC1 SHO";
$resn_exch{"2HAC SHO"}="HAC2 SHO"; $resn_exch{"1HBC SHO"}="HBC1 SHO"; $resn_exch{"2HBC SHO"}="HBC2 SHO";
$resn_exch{"1HGC SHO"}="HGC1 SHO"; $resn_exch{"2HGC SHO"}="HGC2 SHO"; $resn_exch{"1HDC SHO"}="HDC1 SHO";
$resn_exch{"2HDC SHO"}="HDC2 SHO"; $resn_exch{"1HN  SHP"}=" H   SHP"; $resn_exch{"2HN  SHP"}=" H2  SHP";
$resn_exch{" H2* SHP"}=" H2B SHP"; $resn_exch{" H3* SHP"}=" H3B SHP"; $resn_exch{" H5* SHP"}=" H5B SHP";
$resn_exch{" H6* SHP"}=" H6' SHP"; $resn_exch{" H2  SHP"}=" HC2 SHP"; $resn_exch{"1H6  SHP"}=" H61 SHP";
$resn_exch{"2H6  SHP"}=" H62 SHP"; $resn_exch{"1H6' SHP"}="H6'1 SHP"; $resn_exch{"2H6' SHP"}="H6'2 SHP";
$resn_exch{"'HO2 SHP"}="HO2' SHP"; $resn_exch{"'HO3 SHP"}="HO3' SHP"; $resn_exch{"'HO4 SHP"}="HO4' SHP";
$resn_exch{"'HO6 SHP"}="HO6' SHP"; $resn_exch{"2H1  SHR"}=" H12 SHR"; $resn_exch{"1H1  SHR"}=" H11 SHR";
$resn_exch{"2H9  SHR"}=" H92 SHR"; $resn_exch{"1H9  SHR"}=" H91 SHR"; $resn_exch{"2H8  SHR"}=" H82 SHR";
$resn_exch{"1H8  SHR"}=" H81 SHR"; $resn_exch{"2H7  SHR"}=" H72 SHR"; $resn_exch{"1H7  SHR"}=" H71 SHR";
$resn_exch{"2H6  SHR"}=" H62 SHR"; $resn_exch{"1H6  SHR"}=" H61 SHR"; $resn_exch{"2H3  SHR"}=" H32 SHR";
$resn_exch{"1H3  SHR"}=" H31 SHR"; $resn_exch{"2H2  SHR"}=" H22 SHR"; $resn_exch{"1H2  SHR"}=" H21 SHR";
$resn_exch{"SHO2 SHT"}="HO2S SHT"; $resn_exch{"1HCD SHT"}="HCD1 SHT"; $resn_exch{"2HCD SHT"}="HCD2 SHT";
$resn_exch{"1HCC SHT"}="HCC1 SHT"; $resn_exch{"2HCC SHT"}="HCC2 SHT"; $resn_exch{"1HK6 SHT"}="HK61 SHT";
$resn_exch{"2HK6 SHT"}="HK62 SHT"; $resn_exch{"1HK5 SHT"}="HK51 SHT"; $resn_exch{"2HK5 SHT"}="HK52 SHT";
$resn_exch{"1HK4 SHT"}="HK41 SHT"; $resn_exch{"2HK4 SHT"}="HK42 SHT"; $resn_exch{"1HK3 SHT"}="HK31 SHT";
$resn_exch{"2HK3 SHT"}="HK32 SHT"; $resn_exch{"KHC2 SHT"}="HC2K SHT"; $resn_exch{"KHC1 SHT"}="HC1K SHT";
$resn_exch{" AHC SHT"}=" HCA SHT"; $resn_exch{" BHC SHT"}=" HCB SHT"; $resn_exch{"1HG2 SHT"}="HG21 SHT";
$resn_exch{"2HG2 SHT"}="HG22 SHT"; $resn_exch{"3HG2 SHT"}="HG23 SHT"; $resn_exch{"2HOP SHT"}="HOP2 SHT";
$resn_exch{"8HOP SHT"}="HOP8 SHT"; $resn_exch{"1HC5 SHU"}="HC51 SHU"; $resn_exch{"2HC5 SHU"}="HC52 SHU";
$resn_exch{"1HC  SHU"}=" HC1 SHU"; $resn_exch{"2HC  SHU"}=" HC2 SHU"; $resn_exch{"3HC  SHU"}=" HC3 SHU";
$resn_exch{"1HC3 SHU"}="HC31 SHU"; $resn_exch{"2HC3 SHU"}="HC32 SHU"; $resn_exch{"1HC2 SHU"}="HC21 SHU";
$resn_exch{"2HC2 SHU"}="HC22 SHU"; $resn_exch{"1H2  SHV"}=" H21 SHV"; $resn_exch{"2H2  SHV"}=" H22 SHV";
$resn_exch{"1H3  SHV"}=" H31 SHV"; $resn_exch{"2H3  SHV"}=" H32 SHV"; $resn_exch{"1H4  SHV"}=" H41 SHV";
$resn_exch{"2H4  SHV"}=" H42 SHV"; $resn_exch{"1H5  SHV"}=" H51 SHV"; $resn_exch{"2H5  SHV"}=" H52 SHV";
$resn_exch{"1H6  SHV"}=" H61 SHV"; $resn_exch{"2H6  SHV"}=" H62 SHV"; $resn_exch{"1H7  SHV"}=" H71 SHV";
$resn_exch{"2H7  SHV"}=" H72 SHV"; $resn_exch{"3H7  SHV"}=" H73 SHV"; $resn_exch{"1H2N SHY"}="H2N1 SHY";
$resn_exch{"2H2N SHY"}="H2N2 SHY"; $resn_exch{"1H12 SHY"}="H121 SHY"; $resn_exch{"2H12 SHY"}="H122 SHY";
$resn_exch{"1H14 SHY"}="H141 SHY"; $resn_exch{"2H14 SHY"}="H142 SHY"; $resn_exch{"1H23 SHY"}="H231 SHY";
$resn_exch{"2H23 SHY"}="H232 SHY"; $resn_exch{"1H28 SHY"}="H281 SHY"; $resn_exch{"2H28 SHY"}="H282 SHY";
$resn_exch{"1H3  SI2"}=" H31 SI2"; $resn_exch{"2H3  SI2"}=" H32 SI2"; $resn_exch{"1H9  SI2"}=" H91 SI2";
$resn_exch{"2H9  SI2"}=" H92 SI2"; $resn_exch{"1H11 SI2"}="H111 SI2"; $resn_exch{"2H11 SI2"}="H112 SI2";
$resn_exch{"3H11 SI2"}="H113 SI2"; $resn_exch{"1HOB SI2"}="HOB1 SI2"; $resn_exch{"1H3  SIA"}=" H31 SIA";
$resn_exch{"2H3  SIA"}=" H32 SIA"; $resn_exch{"1H9  SIA"}=" H91 SIA"; $resn_exch{"2H9  SIA"}=" H92 SIA";
$resn_exch{"1H11 SIA"}="H111 SIA"; $resn_exch{"2H11 SIA"}="H112 SIA"; $resn_exch{"3H11 SIA"}="H113 SIA";
$resn_exch{"1HOB SIA"}="HOB1 SIA"; $resn_exch{"1HN  SIB"}=" H   SIB"; $resn_exch{"2HN  SIB"}=" H2  SIB";
$resn_exch{"1HB  SIB"}=" HB2 SIB"; $resn_exch{"2HB  SIB"}=" HB3 SIB"; $resn_exch{"1HG  SIB"}=" HG2 SIB";
$resn_exch{"2HG  SIB"}=" HG3 SIB"; $resn_exch{"1H5* SIB"}="H5'1 SIB"; $resn_exch{"2H5* SIB"}="H5'2 SIB";
$resn_exch{" H4* SIB"}=" H4' SIB"; $resn_exch{" H3* SIB"}=" H3' SIB"; $resn_exch{"HO3* SIB"}="HO3' SIB";
$resn_exch{" H2* SIB"}=" H2' SIB"; $resn_exch{"HO2* SIB"}="HO2' SIB"; $resn_exch{" H1* SIB"}=" H1' SIB";
$resn_exch{" H2  SIB"}=" HC2 SIB"; $resn_exch{"1HS0 SIE"}="HS01 SIE"; $resn_exch{"2HS0 SIE"}="HS02 SIE";
$resn_exch{"1HP1 SIE"}="HP11 SIE"; $resn_exch{"2HP1 SIE"}="HP12 SIE"; $resn_exch{"1HP2 SIE"}="HP21 SIE";
$resn_exch{"2HP2 SIE"}="HP22 SIE"; $resn_exch{"1HP3 SIE"}="HP31 SIE"; $resn_exch{"2HP3 SIE"}="HP32 SIE";
$resn_exch{"1HP4 SIE"}="HP41 SIE"; $resn_exch{"2HP4 SIE"}="HP42 SIE"; $resn_exch{"1HP5 SIE"}="HP51 SIE";
$resn_exch{"2HP5 SIE"}="HP52 SIE"; $resn_exch{"1HP6 SIE"}="HP61 SIE"; $resn_exch{"2HP6 SIE"}="HP62 SIE";
$resn_exch{"1HP7 SIE"}="HP71 SIE"; $resn_exch{"2HP7 SIE"}="HP72 SIE"; $resn_exch{"1HP8 SIE"}="HP81 SIE";
$resn_exch{"2HP8 SIE"}="HP82 SIE"; $resn_exch{"1HP9 SIE"}="HP91 SIE"; $resn_exch{"2HP9 SIE"}="HP92 SIE";
$resn_exch{"1HP0 SIE"}="HP01 SIE"; $resn_exch{"2HP0 SIE"}="HP02 SIE"; $resn_exch{"3HP0 SIE"}="HP03 SIE";
$resn_exch{"1H20 SIE"}="H201 SIE"; $resn_exch{"2H20 SIE"}="H202 SIE"; $resn_exch{"1H2  SIF"}=" H21 SIF";
$resn_exch{"2H2  SIF"}=" H22 SIF"; $resn_exch{"1H5  SIF"}=" H51 SIF"; $resn_exch{"2H5  SIF"}=" H52 SIF";
$resn_exch{"3H5  SIF"}=" H53 SIF"; $resn_exch{"1H6  SIF"}=" H61 SIF"; $resn_exch{"2H6  SIF"}=" H62 SIF";
$resn_exch{"3H6  SIF"}=" H63 SIF"; $resn_exch{"1H7  SIF"}=" H71 SIF"; $resn_exch{"2H7  SIF"}=" H72 SIF";
$resn_exch{"3H7  SIF"}=" H73 SIF"; $resn_exch{"1H16 SIG"}="H161 SIG"; $resn_exch{"2H16 SIG"}="H162 SIG";
$resn_exch{"3H16 SIG"}="H163 SIG"; $resn_exch{"1H20 SIG"}="H201 SIG"; $resn_exch{"2H20 SIG"}="H202 SIG";
$resn_exch{"1H21 SIG"}="H211 SIG"; $resn_exch{"2H21 SIG"}="H212 SIG"; $resn_exch{"1H26 SIG"}="H261 SIG";
$resn_exch{"2H26 SIG"}="H262 SIG"; $resn_exch{"3H26 SIG"}="H263 SIG"; $resn_exch{"1H27 SIG"}="H271 SIG";
$resn_exch{"2H27 SIG"}="H272 SIG"; $resn_exch{"3H27 SIG"}="H273 SIG"; $resn_exch{"1H28 SIG"}="H281 SIG";
$resn_exch{"2H28 SIG"}="H282 SIG"; $resn_exch{"3H28 SIG"}="H283 SIG"; $resn_exch{"1H33 SIG"}="H331 SIG";
$resn_exch{"2H33 SIG"}="H332 SIG"; $resn_exch{"3H33 SIG"}="H333 SIG"; $resn_exch{"1H34 SIG"}="H341 SIG";
$resn_exch{"2H34 SIG"}="H342 SIG"; $resn_exch{"3H34 SIG"}="H343 SIG"; $resn_exch{"1H35 SIG"}="H351 SIG";
$resn_exch{"2H35 SIG"}="H352 SIG"; $resn_exch{"3H35 SIG"}="H353 SIG"; $resn_exch{"1H38 SIG"}="H381 SIG";
$resn_exch{"2H38 SIG"}="H382 SIG"; $resn_exch{"3H38 SIG"}="H383 SIG"; $resn_exch{"1H39 SIG"}="H391 SIG";
$resn_exch{"2H39 SIG"}="H392 SIG"; $resn_exch{"3H39 SIG"}="H393 SIG"; $resn_exch{"1H1  SIH"}="H11A SIH";
$resn_exch{"2H1  SIH"}=" H12 SIH"; $resn_exch{"1H2  SIH"}=" H21 SIH"; $resn_exch{"2H2  SIH"}=" H22 SIH";
$resn_exch{"1H4  SIH"}=" H41 SIH"; $resn_exch{"1H6  SIH"}=" H61 SIH"; $resn_exch{"2H6  SIH"}=" H62 SIH";
$resn_exch{"1H7  SIH"}=" H71 SIH"; $resn_exch{"2H7  SIH"}=" H72 SIH"; $resn_exch{"1H12 SIH"}="H121 SIH";
$resn_exch{"2H12 SIH"}="H122 SIH"; $resn_exch{"1H15 SIH"}="H151 SIH"; $resn_exch{"2H15 SIH"}="H152 SIH";
$resn_exch{"1H16 SIH"}="H161 SIH"; $resn_exch{"2H16 SIH"}="H162 SIH"; $resn_exch{"1H18 SIH"}="H181 SIH";
$resn_exch{"2H18 SIH"}="H182 SIH"; $resn_exch{"3H18 SIH"}="H183 SIH"; $resn_exch{"1H19 SIH"}="H191 SIH";
$resn_exch{"2H19 SIH"}="H192 SIH"; $resn_exch{"3H19 SIH"}="H193 SIH"; $resn_exch{"1H21 SIH"}="H211 SIH";
$resn_exch{"2H21 SIH"}="H212 SIH"; $resn_exch{"3H21 SIH"}="H213 SIH"; $resn_exch{"1HH2 SIH"}="HH21 SIH";
$resn_exch{"2HH2 SIH"}="HH22 SIH"; $resn_exch{"1HH3 SIH"}="HH31 SIH"; $resn_exch{"2HH3 SIH"}="HH32 SIH";
$resn_exch{"5HOH SIH"}="HOH5 SIH"; $resn_exch{"1H1  SIL"}=" H11 SIL"; $resn_exch{"2H1  SIL"}=" H12 SIL";
$resn_exch{"1H5  SIL"}=" H51 SIL"; $resn_exch{"2H5  SIL"}=" H52 SIL"; $resn_exch{"1H7  SIL"}=" H71 SIL";
$resn_exch{"2H7  SIL"}=" H72 SIL"; $resn_exch{"1H10 SIL"}="H101 SIL"; $resn_exch{"2H10 SIL"}="H102 SIL";
$resn_exch{"1H13 SIL"}="H131 SIL"; $resn_exch{"2H13 SIL"}="H132 SIL"; $resn_exch{"1H23 SIL"}="H231 SIL";
$resn_exch{"2H23 SIL"}="H232 SIL"; $resn_exch{"1H27 SIL"}="H271 SIL"; $resn_exch{"2H27 SIL"}="H272 SIL";
$resn_exch{"1H34 SIL"}="H341 SIL"; $resn_exch{"2H34 SIL"}="H342 SIL"; $resn_exch{"3H34 SIL"}="H343 SIL";
$resn_exch{"1H38 SIL"}="H381 SIL"; $resn_exch{"2H38 SIL"}="H382 SIL"; $resn_exch{"3H38 SIL"}="H383 SIL";
$resn_exch{"1H2  SIM"}=" H21 SIM"; $resn_exch{"2H2  SIM"}=" H22 SIM"; $resn_exch{"1H4  SIM"}=" H41 SIM";
$resn_exch{"2H4  SIM"}=" H42 SIM"; $resn_exch{"1H6  SIM"}=" H61 SIM"; $resn_exch{"2H6  SIM"}=" H62 SIM";
$resn_exch{"1H7  SIM"}=" H71 SIM"; $resn_exch{"2H7  SIM"}=" H72 SIM"; $resn_exch{"1H15 SIM"}="H151 SIM";
$resn_exch{"2H15 SIM"}="H152 SIM"; $resn_exch{"1H9A SIM"}="H9A1 SIM"; $resn_exch{"2H9A SIM"}="H9A2 SIM";
$resn_exch{"3H9A SIM"}="H9A3 SIM"; $resn_exch{"1H20 SIM"}="H201 SIM"; $resn_exch{"2H20 SIM"}="H202 SIM";
$resn_exch{"1H21 SIM"}="H211 SIM"; $resn_exch{"2H21 SIM"}="H212 SIM"; $resn_exch{"3H21 SIM"}="H213 SIM";
$resn_exch{"1H22 SIM"}="H221 SIM"; $resn_exch{"2H22 SIM"}="H222 SIM"; $resn_exch{"3H22 SIM"}="H223 SIM";
$resn_exch{"1H23 SIM"}="H231 SIM"; $resn_exch{"2H23 SIM"}="H232 SIM"; $resn_exch{"3H23 SIM"}="H233 SIM";
$resn_exch{"1H24 SIM"}="H241 SIM"; $resn_exch{"2H24 SIM"}="H242 SIM"; $resn_exch{"3H24 SIM"}="H243 SIM";
$resn_exch{"1H2  SIN"}=" H21 SIN"; $resn_exch{"2H2  SIN"}=" H22 SIN"; $resn_exch{"1H3  SIN"}=" H31 SIN";
$resn_exch{"2H3  SIN"}=" H32 SIN"; $resn_exch{"1H12 SK2"}="H121 SK2"; $resn_exch{"2H12 SK2"}="H122 SK2";
$resn_exch{"1H6  SK2"}=" H61 SK2"; $resn_exch{"2H6  SK2"}=" H62 SK2"; $resn_exch{"3H6  SK2"}=" H63 SK2";
$resn_exch{"1H3  SK2"}=" H31 SK2"; $resn_exch{"2H3  SK2"}=" H32 SK2"; $resn_exch{"1H8  SK2"}=" H81 SK2";
$resn_exch{"2H8  SK2"}=" H82 SK2"; $resn_exch{"1H6  SK3"}=" H61 SK3"; $resn_exch{"2H6  SK3"}=" H62 SK3";
$resn_exch{"1H11 SKA"}="H111 SKA"; $resn_exch{"2H11 SKA"}="H112 SKA"; $resn_exch{"1H9  SKA"}=" H91 SKA";
$resn_exch{"2H9  SKA"}=" H92 SKA"; $resn_exch{"1H8  SKA"}=" H81 SKA"; $resn_exch{"2H8  SKA"}=" H82 SKA";
$resn_exch{"1H3  SKD"}=" H31 SKD"; $resn_exch{"2H3  SKD"}=" H32 SKD"; $resn_exch{"1H9  SKD"}=" H91 SKD";
$resn_exch{"2H9  SKD"}=" H92 SKD"; $resn_exch{"1H11 SKD"}="H111 SKD"; $resn_exch{"2H11 SKD"}="H112 SKD";
$resn_exch{"3H11 SKD"}="H113 SKD"; $resn_exch{"1HOB SKD"}="HOB1 SKD"; $resn_exch{"1HC1 SKF"}="HC11 SKF";
$resn_exch{"2HC1 SKF"}="HC12 SKF"; $resn_exch{"1HC2 SKF"}="HC21 SKF"; $resn_exch{"2HC2 SKF"}="HC22 SKF";
$resn_exch{"1HC3 SKF"}="HC31 SKF"; $resn_exch{"2HC3 SKF"}="HC32 SKF"; $resn_exch{"1H5  SKM"}=" H51 SKM";
$resn_exch{"2H5  SKM"}=" H52 SKM"; $resn_exch{"1H6  SKP"}=" H61 SKP"; $resn_exch{"2H6  SKP"}=" H62 SKP";
$resn_exch{"1H9  SKP"}=" H91 SKP"; $resn_exch{"2H9  SKP"}=" H92 SKP"; $resn_exch{"3H9  SKP"}=" H93 SKP";
$resn_exch{"1H3C SL1"}="H3C1 SL1"; $resn_exch{"2H3C SL1"}="H3C2 SL1"; $resn_exch{"1H20 SL1"}="H201 SL1";
$resn_exch{"2H20 SL1"}="H202 SL1"; $resn_exch{"1H21 SL1"}="H211 SL1"; $resn_exch{"2H21 SL1"}="H212 SL1";
$resn_exch{"1H22 SL1"}="H221 SL1"; $resn_exch{"2H22 SL1"}="H222 SL1"; $resn_exch{"1H6N SL1"}="H6N1 SL1";
$resn_exch{"2H6N SL1"}="H6N2 SL1"; $resn_exch{"1H24 SL1"}="H241 SL1"; $resn_exch{"2H24 SL1"}="H242 SL1";
$resn_exch{"1H3  SLB"}=" H31 SLB"; $resn_exch{"2H3  SLB"}=" H32 SLB"; $resn_exch{"1H9  SLB"}=" H91 SLB";
$resn_exch{"2H9  SLB"}=" H92 SLB"; $resn_exch{"1H11 SLB"}="H111 SLB"; $resn_exch{"2H11 SLB"}="H112 SLB";
$resn_exch{"3H11 SLB"}="H113 SLB"; $resn_exch{"1HOB SLB"}="HOB1 SLB"; $resn_exch{"1HH  SLE"}=" HH1 SLE";
$resn_exch{"2HH  SLE"}=" HH2 SLE"; $resn_exch{"1H3  SLE"}=" H31 SLE"; $resn_exch{"2H3  SLE"}=" H32 SLE";
$resn_exch{"1HM  SLE"}=" HM1 SLE"; $resn_exch{"2HM  SLE"}=" HM2 SLE"; $resn_exch{"3HM  SLE"}=" HM3 SLE";
$resn_exch{"1H5  SLE"}=" H51 SLE"; $resn_exch{"2H5  SLE"}=" H52 SLE"; $resn_exch{"3H5  SLE"}=" H53 SLE";
$resn_exch{"1H35 SLF"}="H351 SLF"; $resn_exch{"2H35 SLF"}="H352 SLF"; $resn_exch{"3H35 SLF"}="H353 SLF";
$resn_exch{"1H34 SLF"}="H341 SLF"; $resn_exch{"2H34 SLF"}="H342 SLF"; $resn_exch{"1H33 SLF"}="H331 SLF";
$resn_exch{"2H33 SLF"}="H332 SLF"; $resn_exch{"1H32 SLF"}="H321 SLF"; $resn_exch{"2H32 SLF"}="H322 SLF";
$resn_exch{"1H31 SLF"}="H311 SLF"; $resn_exch{"2H31 SLF"}="H312 SLF"; $resn_exch{"1H30 SLF"}="H301 SLF";
$resn_exch{"2H30 SLF"}="H302 SLF"; $resn_exch{"1H29 SLF"}="H291 SLF"; $resn_exch{"2H29 SLF"}="H292 SLF";
$resn_exch{"1H28 SLF"}="H281 SLF"; $resn_exch{"2H28 SLF"}="H282 SLF"; $resn_exch{"1H27 SLF"}="H271 SLF";
$resn_exch{"2H27 SLF"}="H272 SLF"; $resn_exch{"1H26 SLF"}="H261 SLF"; $resn_exch{"2H26 SLF"}="H262 SLF";
$resn_exch{"1H25 SLF"}="H251 SLF"; $resn_exch{"2H25 SLF"}="H252 SLF"; $resn_exch{"1H24 SLF"}="H241 SLF";
$resn_exch{"2H24 SLF"}="H242 SLF"; $resn_exch{"1H23 SLF"}="H231 SLF"; $resn_exch{"2H23 SLF"}="H232 SLF";
$resn_exch{"1H22 SLF"}="H221 SLF"; $resn_exch{"2H22 SLF"}="H222 SLF"; $resn_exch{"1H21 SLF"}="H211 SLF";
$resn_exch{"2H21 SLF"}="H212 SLF"; $resn_exch{"1H20 SLF"}="H201 SLF"; $resn_exch{"2H20 SLF"}="H202 SLF";
$resn_exch{"1H19 SLF"}="H191 SLF"; $resn_exch{"2H19 SLF"}="H192 SLF"; $resn_exch{"1H   SLF"}=" H1A SLF";
$resn_exch{"2H   SLF"}=" H2A SLF"; $resn_exch{"1H41 SLF"}="H411 SLF"; $resn_exch{"2H41 SLF"}="H412 SLF";
$resn_exch{"1H5  SLF"}=" H51 SLF"; $resn_exch{"2H5  SLF"}=" H52 SLF"; $resn_exch{"1H6  SLF"}=" H61 SLF";
$resn_exch{"2H6  SLF"}=" H62 SLF"; $resn_exch{"1H7  SLF"}=" H71 SLF"; $resn_exch{"2H7  SLF"}=" H72 SLF";
$resn_exch{"1H8  SLF"}=" H81 SLF"; $resn_exch{"2H8  SLF"}=" H82 SLF"; $resn_exch{"1H9  SLF"}=" H91 SLF";
$resn_exch{"2H9  SLF"}=" H92 SLF"; $resn_exch{"1H10 SLF"}="H101 SLF"; $resn_exch{"2H10 SLF"}="H102 SLF";
$resn_exch{"1H11 SLF"}="H111 SLF"; $resn_exch{"2H11 SLF"}="H112 SLF"; $resn_exch{"1H12 SLF"}="H121 SLF";
$resn_exch{"2H12 SLF"}="H122 SLF"; $resn_exch{"1H13 SLF"}="H131 SLF"; $resn_exch{"2H13 SLF"}="H132 SLF";
$resn_exch{"1H14 SLF"}="H141 SLF"; $resn_exch{"2H14 SLF"}="H142 SLF"; $resn_exch{"1H15 SLF"}="H151 SLF";
$resn_exch{"2H15 SLF"}="H152 SLF"; $resn_exch{"1H16 SLF"}="H161 SLF"; $resn_exch{"2H16 SLF"}="H162 SLF";
$resn_exch{"1H17 SLF"}="H171 SLF"; $resn_exch{"2H17 SLF"}="H172 SLF"; $resn_exch{"3H17 SLF"}="H173 SLF";
$resn_exch{"1H18 SLT"}="H181 SLT"; $resn_exch{"2H18 SLT"}="H182 SLT"; $resn_exch{"3H18 SLT"}="H183 SLT";
$resn_exch{"1H16 SLT"}="H161 SLT"; $resn_exch{"2H16 SLT"}="H162 SLT"; $resn_exch{"1H10 SLT"}="H101 SLT";
$resn_exch{"2H10 SLT"}="H102 SLT"; $resn_exch{"1H6  SLT"}=" H61 SLT"; $resn_exch{"2H6  SLT"}=" H62 SLT";
$resn_exch{"1H6A SLT"}="H6A1 SLT"; $resn_exch{"2H6A SLT"}="H6A2 SLT"; $resn_exch{"1H21 SLU"}="H211 SLU";
$resn_exch{"2H21 SLU"}="H212 SLU"; $resn_exch{"1H38 SLU"}="H381 SLU"; $resn_exch{"2H38 SLU"}="H382 SLU";
$resn_exch{"1HN  SLZ"}=" H   SLZ"; $resn_exch{"2HN  SLZ"}=" H2  SLZ"; $resn_exch{"1HB  SLZ"}=" HB2 SLZ";
$resn_exch{"2HB  SLZ"}=" HB3 SLZ"; $resn_exch{"1HD  SLZ"}=" HD2 SLZ"; $resn_exch{"2HD  SLZ"}=" HD3 SLZ";
$resn_exch{"1HE  SLZ"}=" HE2 SLZ"; $resn_exch{"2HE  SLZ"}=" HE3 SLZ"; $resn_exch{"1HZ  SLZ"}=" HZ1 SLZ";
$resn_exch{"2HZ  SLZ"}=" HZ2 SLZ"; $resn_exch{"1H6N SM1"}="H6N1 SM1"; $resn_exch{"2H6N SM1"}="H6N2 SM1";
$resn_exch{"1H11 SM1"}="H111 SM1"; $resn_exch{"2H11 SM1"}="H112 SM1"; $resn_exch{"1H22 SM1"}="H221 SM1";
$resn_exch{"2H22 SM1"}="H222 SM1"; $resn_exch{"3H22 SM1"}="H223 SM1"; $resn_exch{"1H6C SM1"}="H6C1 SM1";
$resn_exch{"2H6C SM1"}="H6C2 SM1"; $resn_exch{"1H1C SM1"}="H1C1 SM1"; $resn_exch{"2H1C SM1"}="H1C2 SM1";
$resn_exch{"1HAG SM2"}="HAG1 SM2"; $resn_exch{"2HAG SM2"}="HAG2 SM2"; $resn_exch{"1HAG SM3"}="HAG1 SM3";
$resn_exch{"2HAG SM3"}="HAG2 SM3"; $resn_exch{"1HAG SM4"}="HAG1 SM4"; $resn_exch{"2HAG SM4"}="HAG2 SM4";
$resn_exch{"1H3M SMA"}="H3M1 SMA"; $resn_exch{"2H3M SMA"}="H3M2 SMA"; $resn_exch{"3H3M SMA"}="H3M3 SMA";
$resn_exch{"1H5M SMA"}="H5M1 SMA"; $resn_exch{"2H5M SMA"}="H5M2 SMA"; $resn_exch{"3H5M SMA"}="H5M3 SMA";
$resn_exch{"1H7M SMA"}="H7M1 SMA"; $resn_exch{"2H7M SMA"}="H7M2 SMA"; $resn_exch{"3H7M SMA"}="H7M3 SMA";
$resn_exch{"1H9  SMA"}=" H91 SMA"; $resn_exch{"2H9  SMA"}=" H92 SMA"; $resn_exch{"1H10 SMA"}="H101 SMA";
$resn_exch{"2H10 SMA"}="H102 SMA"; $resn_exch{"1H21 SMA"}="H211 SMA"; $resn_exch{"2H21 SMA"}="H212 SMA";
$resn_exch{"3H21 SMA"}="H213 SMA"; $resn_exch{"1H22 SMA"}="H221 SMA"; $resn_exch{"2H22 SMA"}="H222 SMA";
$resn_exch{"3H22 SMA"}="H223 SMA"; $resn_exch{"1H23 SMA"}="H231 SMA"; $resn_exch{"2H23 SMA"}="H232 SMA";
$resn_exch{"3H23 SMA"}="H233 SMA"; $resn_exch{"1H24 SMA"}="H241 SMA"; $resn_exch{"2H24 SMA"}="H242 SMA";
$resn_exch{"3H24 SMA"}="H243 SMA"; $resn_exch{"1H25 SMA"}="H251 SMA"; $resn_exch{"2H25 SMA"}="H252 SMA";
$resn_exch{"3H25 SMA"}="H253 SMA"; $resn_exch{"1H26 SMA"}="H261 SMA"; $resn_exch{"2H26 SMA"}="H262 SMA";
$resn_exch{"3H26 SMA"}="H263 SMA"; $resn_exch{"1H1  SMB"}=" H11 SMB"; $resn_exch{"2H1  SMB"}=" H12 SMB";
$resn_exch{"3H1  SMB"}=" H13 SMB"; $resn_exch{"1H2  SMB"}=" H21 SMB"; $resn_exch{"2H2  SMB"}=" H22 SMB";
$resn_exch{"1H4  SMB"}=" H41 SMB"; $resn_exch{"2H4  SMB"}=" H42 SMB"; $resn_exch{"3H4  SMB"}=" H43 SMB";
$resn_exch{"1HN  SMC"}=" H   SMC"; $resn_exch{"2HN  SMC"}=" H2  SMC"; $resn_exch{"1HB  SMC"}=" HB2 SMC";
$resn_exch{"2HB  SMC"}=" HB3 SMC"; $resn_exch{"1HCS SMC"}="HCS1 SMC"; $resn_exch{"2HCS SMC"}="HCS2 SMC";
$resn_exch{"3HCS SMC"}="HCS3 SMC"; $resn_exch{"1H16 SMD"}="H161 SMD"; $resn_exch{"2H16 SMD"}="H162 SMD";
$resn_exch{"1H27 SMD"}="H271 SMD"; $resn_exch{"2H27 SMD"}="H272 SMD"; $resn_exch{"3H27 SMD"}="H273 SMD";
$resn_exch{"1H26 SMD"}="H261 SMD"; $resn_exch{"2H26 SMD"}="H262 SMD"; $resn_exch{"1HN  SME"}=" H   SME";
$resn_exch{"2HN  SME"}=" H2  SME"; $resn_exch{"1HB  SME"}=" HB2 SME"; $resn_exch{"2HB  SME"}=" HB3 SME";
$resn_exch{"1HG  SME"}=" HG2 SME"; $resn_exch{"2HG  SME"}=" HG3 SME"; $resn_exch{"1HE  SME"}=" HE1 SME";
$resn_exch{"2HE  SME"}=" HE2 SME"; $resn_exch{"3HE  SME"}=" HE3 SME"; $resn_exch{" HN2 SMF"}=" H2  SMF";
$resn_exch{"1HB  SMF"}=" HB2 SMF"; $resn_exch{"2HB  SMF"}=" HB3 SMF"; $resn_exch{"1HH  SMF"}=" HH2 SMF";
$resn_exch{"2HH  SMF"}=" HH3 SMF"; $resn_exch{"1H5  SMG"}=" H51 SMG"; $resn_exch{"2H5  SMG"}=" H52 SMG";
$resn_exch{"1H4  SMG"}=" H41 SMG"; $resn_exch{"2H4  SMG"}=" H42 SMG"; $resn_exch{"1HB  SMG"}=" HB1 SMG";
$resn_exch{"2HB  SMG"}=" HB2 SMG"; $resn_exch{"1HG  SMG"}=" HG1 SMG"; $resn_exch{"2HG  SMG"}=" HG2 SMG";
$resn_exch{"1HE  SMG"}=" HE1 SMG"; $resn_exch{"2HE  SMG"}=" HE2 SMG"; $resn_exch{"3HE  SMG"}=" HE3 SMG";
$resn_exch{"1H32 SML"}="H321 SML"; $resn_exch{"2H32 SML"}="H322 SML"; $resn_exch{"3H32 SML"}="H323 SML";
$resn_exch{"1H37 SML"}="H371 SML"; $resn_exch{"1H38 SML"}="H381 SML"; $resn_exch{"1H39 SML"}="H391 SML";
$resn_exch{"1H40 SML"}="H401 SML"; $resn_exch{"1H41 SML"}="H411 SML"; $resn_exch{"1HN  SMM"}=" HN1 SMM";
$resn_exch{"2HN  SMM"}=" HN2 SMM"; $resn_exch{"1HA  SMM"}=" HA1 SMM"; $resn_exch{"1HB  SMM"}=" HB1 SMM";
$resn_exch{"2HB  SMM"}=" HB2 SMM"; $resn_exch{"1HG  SMM"}=" HG1 SMM"; $resn_exch{"2HG  SMM"}=" HG2 SMM";
$resn_exch{"1HD  SMM"}=" HD1 SMM"; $resn_exch{"1HE  SMM"}=" HE1 SMM"; $resn_exch{"2HE  SMM"}=" HE2 SMM";
$resn_exch{"3HE  SMM"}=" HE3 SMM"; $resn_exch{"1H5* SMM"}="H5'1 SMM"; $resn_exch{"2H5* SMM"}="H5'2 SMM";
$resn_exch{" H4* SMM"}=" H4' SMM"; $resn_exch{" H1* SMM"}=" H1' SMM"; $resn_exch{"1HN6 SMM"}="HN61 SMM";
$resn_exch{"2HN6 SMM"}="HN62 SMM"; $resn_exch{" H2* SMM"}=" H2' SMM"; $resn_exch{"*HO2 SMM"}="HO2' SMM";
$resn_exch{" H3* SMM"}=" H3' SMM"; $resn_exch{"*HO3 SMM"}="HO3' SMM"; $resn_exch{"1HXT SMM"}="HXT1 SMM";
$resn_exch{"2HXT SMM"}="HXT2 SMM"; $resn_exch{"3HXT SMM"}="HXT3 SMM"; $resn_exch{"1HMP SMP"}="HMP1 SMP";
$resn_exch{"2HMP SMP"}="HMP2 SMP"; $resn_exch{"3HMP SMP"}="HMP3 SMP"; $resn_exch{"1H5* SMP"}=" H5' SMP";
$resn_exch{"2H5* SMP"}="H5'' SMP"; $resn_exch{" H4* SMP"}=" H4' SMP"; $resn_exch{" H1* SMP"}=" H1' SMP";
$resn_exch{"1H6  SMP"}=" H61 SMP"; $resn_exch{"2H6  SMP"}=" H62 SMP"; $resn_exch{" H3* SMP"}=" H3' SMP";
$resn_exch{"1H2* SMP"}=" H2' SMP"; $resn_exch{"2H2* SMP"}="H2'' SMP"; $resn_exch{"H3T  SMP"}="HO3' SMP";
$resn_exch{"3HOP SMP"}="HOP3 SMP"; $resn_exch{"1HN1 SMS"}="HN11 SMS"; $resn_exch{"2HN1 SMS"}="HN12 SMS";
$resn_exch{"1HC1 SMS"}="HC11 SMS"; $resn_exch{"2HC1 SMS"}="HC12 SMS"; $resn_exch{"1HC6 SMS"}="HC61 SMS";
$resn_exch{"2HC6 SMS"}="HC62 SMS"; $resn_exch{"1HC9 SMS"}="HC91 SMS"; $resn_exch{"2HC9 SMS"}="HC92 SMS";
$resn_exch{"3HC9 SMS"}="HC93 SMS"; $resn_exch{"1HC8 SMS"}="HC81 SMS"; $resn_exch{"2HC8 SMS"}="HC82 SMS";
$resn_exch{"3HC8 SMS"}="HC83 SMS"; $resn_exch{"2H5' SMT"}="H5'2 SMT"; $resn_exch{"1H5A SMT"}="H5A1 SMT";
$resn_exch{"2H5A SMT"}="H5A2 SMT"; $resn_exch{"1HA  SMT"}=" HA1 SMT"; $resn_exch{"2HA  SMT"}=" HA2 SMT";
$resn_exch{"1HB  SMT"}=" HB1 SMT"; $resn_exch{"2HB  SMT"}=" HB2 SMT"; $resn_exch{"1HD  SMT"}=" HD1 SMT";
$resn_exch{"2HD  SMT"}=" HD2 SMT"; $resn_exch{"3HD  SMT"}=" HD3 SMT"; $resn_exch{"1HB  SN0"}=" HB1 SN0";
$resn_exch{"2HB  SN0"}=" HB2 SN0"; $resn_exch{"1HG  SN0"}=" HG1 SN0"; $resn_exch{"2HG  SN0"}=" HG2 SN0";
$resn_exch{"1HD  SN0"}="HD1A SN0"; $resn_exch{"2HD  SN0"}=" HD2 SN0"; $resn_exch{"3HD  SN0"}=" HD3 SN0";
$resn_exch{"1H2  SN0"}=" H21 SN0"; $resn_exch{"2H2  SN0"}=" H22 SN0"; $resn_exch{"1H3  SN0"}=" H31 SN0";
$resn_exch{"2H3  SN0"}=" H32 SN0"; $resn_exch{"1HC7 SN2"}="HC71 SN2"; $resn_exch{"2HC7 SN2"}="HC72 SN2";
$resn_exch{"1HC8 SN2"}="HC81 SN2"; $resn_exch{"2HC8 SN2"}="HC82 SN2"; $resn_exch{"1HC9 SN2"}="HC91 SN2";
$resn_exch{"2HC9 SN2"}="HC92 SN2"; $resn_exch{"1H10 SN2"}="H101 SN2"; $resn_exch{"2H10 SN2"}="H102 SN2";
$resn_exch{"1HN4 SN2"}="HN41 SN2"; $resn_exch{"2HN4 SN2"}="HN42 SN2"; $resn_exch{"1H2  SN3"}=" H21 SN3";
$resn_exch{"2H2  SN3"}=" H22 SN3"; $resn_exch{"1H5  SN3"}=" H51 SN3"; $resn_exch{"2H5  SN3"}=" H52 SN3";
$resn_exch{"1H10 SN3"}="H101 SN3"; $resn_exch{"2H10 SN3"}="H102 SN3"; $resn_exch{"1H13 SN3"}="H131 SN3";
$resn_exch{"2H13 SN3"}="H132 SN3"; $resn_exch{"1H14 SN3"}="H141 SN3"; $resn_exch{"2H14 SN3"}="H142 SN3";
$resn_exch{"1H16 SN3"}="H161 SN3"; $resn_exch{"2H16 SN3"}="H162 SN3"; $resn_exch{"1H18 SN3"}="H181 SN3";
$resn_exch{"2H18 SN3"}="H182 SN3"; $resn_exch{"2H19 SN3"}="H192 SN3"; $resn_exch{"1H22 SN3"}="H221 SN3";
$resn_exch{"2H22 SN3"}="H222 SN3"; $resn_exch{"1H29 SN3"}="H291 SN3"; $resn_exch{"2H29 SN3"}="H292 SN3";
$resn_exch{"3H29 SN3"}="H293 SN3"; $resn_exch{"1H30 SN3"}="H301 SN3"; $resn_exch{"2H30 SN3"}="H302 SN3";
$resn_exch{"3H30 SN3"}="H303 SN3"; $resn_exch{"1H36 SN3"}="H361 SN3"; $resn_exch{"2H36 SN3"}="H362 SN3";
$resn_exch{"3H36 SN3"}="H363 SN3"; $resn_exch{"1H37 SN3"}="H371 SN3"; $resn_exch{"2H37 SN3"}="H372 SN3";
$resn_exch{"1H34 SN6"}="H341 SN6"; $resn_exch{"2H34 SN6"}="H342 SN6"; $resn_exch{"3H34 SN6"}="H343 SN6";
$resn_exch{"1H35 SN6"}="H351 SN6"; $resn_exch{"2H35 SN6"}="H352 SN6"; $resn_exch{"3H35 SN6"}="H353 SN6";
$resn_exch{"1H34 SN7"}="H341 SN7"; $resn_exch{"2H34 SN7"}="H342 SN7"; $resn_exch{"3H34 SN7"}="H343 SN7";
$resn_exch{"1H35 SN7"}="H351 SN7"; $resn_exch{"2H35 SN7"}="H352 SN7"; $resn_exch{"3H35 SN7"}="H353 SN7";
$resn_exch{"1H36 SN7"}="H361 SN7"; $resn_exch{"2H36 SN7"}="H362 SN7"; $resn_exch{"1H37 SN7"}="H371 SN7";
$resn_exch{"2H37 SN7"}="H372 SN7"; $resn_exch{"1HAA SN8"}="HAA1 SN8"; $resn_exch{"2HAA SN8"}="HAA2 SN8";
$resn_exch{"3HAA SN8"}="HAA3 SN8"; $resn_exch{"1HAC SN8"}="HAC1 SN8"; $resn_exch{"2HAC SN8"}="HAC2 SN8";
$resn_exch{"3HAC SN8"}="HAC3 SN8"; $resn_exch{"1HAT SN8"}="HAT1 SN8"; $resn_exch{"1HAJ SN8"}="HAJ1 SN8";
$resn_exch{"1HAL SN8"}="HAL1 SN8"; $resn_exch{"1HAI SN8"}="HAI1 SN8"; $resn_exch{"1HAK SN8"}="HAK1 SN8";
$resn_exch{"1HAF SN8"}="HAF1 SN8"; $resn_exch{"1HAH SN8"}="HAH1 SN8"; $resn_exch{"1HAE SN8"}="HAE1 SN8";
$resn_exch{"1HAG SN8"}="HAG1 SN8"; $resn_exch{"1HAN SN8"}="HAN1 SN8"; $resn_exch{"1HAR SN8"}="HAR1 SN8";
$resn_exch{"1HAB SN8"}="HAB1 SN8"; $resn_exch{"2HAB SN8"}="HAB2 SN8"; $resn_exch{"3HAB SN8"}="HAB3 SN8";
$resn_exch{"1HAS SN8"}="HAS1 SN8"; $resn_exch{"1HAO SN8"}="HAO1 SN8"; $resn_exch{"1HAA SN9"}="HAA1 SN9";
$resn_exch{"2HAA SN9"}="HAA2 SN9"; $resn_exch{"3HAA SN9"}="HAA3 SN9"; $resn_exch{"1HAC SN9"}="HAC1 SN9";
$resn_exch{"2HAC SN9"}="HAC2 SN9"; $resn_exch{"3HAC SN9"}="HAC3 SN9"; $resn_exch{"1HAU SN9"}="HAU1 SN9";
$resn_exch{"1HAL SN9"}="HAL1 SN9"; $resn_exch{"1HAN SN9"}="HAN1 SN9"; $resn_exch{"1HAK SN9"}="HAK1 SN9";
$resn_exch{"1HAM SN9"}="HAM1 SN9"; $resn_exch{"1HAH SN9"}="HAH1 SN9"; $resn_exch{"1HAJ SN9"}="HAJ1 SN9";
$resn_exch{"1HAG SN9"}="HAG1 SN9"; $resn_exch{"1HAI SN9"}="HAI1 SN9"; $resn_exch{"1HAO SN9"}="HAO1 SN9";
$resn_exch{"1HAS SN9"}="HAS1 SN9"; $resn_exch{"1HAB SN9"}="HAB1 SN9"; $resn_exch{"2HAB SN9"}="HAB2 SN9";
$resn_exch{"3HAB SN9"}="HAB3 SN9"; $resn_exch{"1HAT SN9"}="HAT1 SN9"; $resn_exch{"1HAP SN9"}="HAP1 SN9";
$resn_exch{"1H14 SNA"}="H141 SNA"; $resn_exch{"2H14 SNA"}="H142 SNA"; $resn_exch{"1H20 SNA"}="H201 SNA";
$resn_exch{"2H20 SNA"}="H202 SNA"; $resn_exch{"1H21 SNA"}="H211 SNA"; $resn_exch{"2H21 SNA"}="H212 SNA";
$resn_exch{"1H22 SNA"}="H221 SNA"; $resn_exch{"2H22 SNA"}="H222 SNA"; $resn_exch{"1H23 SNA"}="H231 SNA";
$resn_exch{"2H23 SNA"}="H232 SNA"; $resn_exch{"1H24 SNA"}="H241 SNA"; $resn_exch{"2H24 SNA"}="H242 SNA";
$resn_exch{"1H27 SNA"}="H271 SNA"; $resn_exch{"2H27 SNA"}="H272 SNA"; $resn_exch{"1H46 SNA"}="H461 SNA";
$resn_exch{"2H46 SNA"}="H462 SNA"; $resn_exch{"1H53 SNA"}="H531 SNA"; $resn_exch{"2H53 SNA"}="H532 SNA";
$resn_exch{"1H57 SNA"}="H571 SNA"; $resn_exch{"2H57 SNA"}="H572 SNA"; $resn_exch{"1HN  SNC"}=" H   SNC";
$resn_exch{"2HN  SNC"}=" H2  SNC"; $resn_exch{"1HB  SNC"}=" HB2 SNC"; $resn_exch{"2HB  SNC"}=" HB3 SNC";
$resn_exch{"2HOA SND"}="HOA2 SND"; $resn_exch{"AH51 SND"}="H51A SND"; $resn_exch{"AH52 SND"}="H52A SND";
$resn_exch{"AH4* SND"}=" H4B SND"; $resn_exch{"AH3* SND"}=" H3B SND"; $resn_exch{"AHO3 SND"}="HO3A SND";
$resn_exch{"AH2* SND"}=" H2B SND"; $resn_exch{"AHO2 SND"}="HO2A SND"; $resn_exch{"AH1* SND"}=" H1B SND";
$resn_exch{" AH8 SND"}=" H8A SND"; $resn_exch{"AH61 SND"}="H61A SND"; $resn_exch{"AH62 SND"}="H62A SND";
$resn_exch{" AH2 SND"}=" H2A SND"; $resn_exch{"NH51 SND"}="H51N SND"; $resn_exch{"NH52 SND"}="H52N SND";
$resn_exch{"NH4* SND"}=" H4D SND"; $resn_exch{"NH3* SND"}=" H3D SND"; $resn_exch{"NHO3 SND"}="HO3N SND";
$resn_exch{"NH2* SND"}=" H2D SND"; $resn_exch{"NHO2 SND"}="HO2N SND"; $resn_exch{"NH1* SND"}=" H1D SND";
$resn_exch{" NH2 SND"}=" H2N SND"; $resn_exch{"NH71 SND"}="H71N SND"; $resn_exch{"NH72 SND"}="H72N SND";
$resn_exch{" NH4 SND"}=" H4N SND"; $resn_exch{" NH5 SND"}=" H5N SND"; $resn_exch{" NH6 SND"}=" H6N SND";
$resn_exch{"1H6C SNG"}="H6C1 SNG"; $resn_exch{"2H6C SNG"}="H6C2 SNG"; $resn_exch{"1H8C SNG"}="H8C1 SNG";
$resn_exch{"2H8C SNG"}="H8C2 SNG"; $resn_exch{"3H8C SNG"}="H8C3 SNG"; $resn_exch{"1HMC SNG"}="HMC1 SNG";
$resn_exch{"2HMC SNG"}="HMC2 SNG"; $resn_exch{"3HMC SNG"}="HMC3 SNG"; $resn_exch{"1H20 SNH"}="H201 SNH";
$resn_exch{"2H20 SNH"}="H202 SNH"; $resn_exch{"3H20 SNH"}="H203 SNH"; $resn_exch{"1H21 SNH"}="H211 SNH";
$resn_exch{"2H21 SNH"}="H212 SNH"; $resn_exch{"3H21 SNH"}="H213 SNH"; $resn_exch{"1H28 SNH"}="H281 SNH";
$resn_exch{"2H28 SNH"}="H282 SNH"; $resn_exch{"3H28 SNH"}="H283 SNH"; $resn_exch{" H5* SNI"}=" H5' SNI";
$resn_exch{"1H5* SNI"}="H5'1 SNI"; $resn_exch{"2H5* SNI"}="H5'2 SNI"; $resn_exch{" H4* SNI"}=" H4' SNI";
$resn_exch{" H1* SNI"}=" H1' SNI"; $resn_exch{" H2* SNI"}=" H2' SNI"; $resn_exch{" H3* SNI"}=" H3' SNI";
$resn_exch{"1H8  SNL"}=" H81 SNL"; $resn_exch{"2H8  SNL"}=" H82 SNL"; $resn_exch{"1H4  SNL"}=" H41 SNL";
$resn_exch{"2H4  SNL"}=" H42 SNL"; $resn_exch{"1H1  SNL"}=" H11 SNL"; $resn_exch{"2H1  SNL"}=" H12 SNL";
$resn_exch{"1H3  SNL"}=" H31 SNL"; $resn_exch{"2H3  SNL"}=" H32 SNL"; $resn_exch{"1H7  SNL"}=" H71 SNL";
$resn_exch{"2H7  SNL"}=" H72 SNL"; $resn_exch{"1H10 SNL"}="H101 SNL"; $resn_exch{"2H10 SNL"}="H102 SNL";
$resn_exch{"3H10 SNL"}="H103 SNL"; $resn_exch{"1H11 SNL"}="H111 SNL"; $resn_exch{"2H11 SNL"}="H112 SNL";
$resn_exch{"3H11 SNL"}="H113 SNL"; $resn_exch{"1H12 SNL"}="H121 SNL"; $resn_exch{"2H12 SNL"}="H122 SNL";
$resn_exch{"1H13 SNL"}="H131 SNL"; $resn_exch{"2H13 SNL"}="H132 SNL"; $resn_exch{"1H14 SNL"}="H141 SNL";
$resn_exch{"2H14 SNL"}="H142 SNL"; $resn_exch{"1H17 SNL"}="H171 SNL"; $resn_exch{"2H17 SNL"}="H172 SNL";
$resn_exch{"3H17 SNL"}="H173 SNL"; $resn_exch{"1H21 SNL"}="H211 SNL"; $resn_exch{"2H21 SNL"}="H212 SNL";
$resn_exch{"1HN3 SNN"}="HN31 SNN"; $resn_exch{"2HN3 SNN"}="HN32 SNN"; $resn_exch{"1H4  SNN"}=" H41 SNN";
$resn_exch{"2H4  SNN"}=" H42 SNN"; $resn_exch{"2H2  SNO"}=" H22 SNO"; $resn_exch{"3H3  SNO"}=" H33 SNO";
$resn_exch{"5H5  SNO"}=" H55 SNO"; $resn_exch{"6H6  SNO"}=" H66 SNO"; $resn_exch{"7H7  SNO"}=" H77 SNO";
$resn_exch{"1H8  SNO"}=" H81 SNO"; $resn_exch{"2H8  SNO"}=" H82 SNO"; $resn_exch{"2H11 SNP"}="H112 SNP";
$resn_exch{"1H11 SNP"}="H111 SNP"; $resn_exch{"1H2O SNP"}="H2O1 SNP"; $resn_exch{"2H13 SNP"}="H132 SNP";
$resn_exch{"1H13 SNP"}="H131 SNP"; $resn_exch{"3H15 SNP"}="H153 SNP"; $resn_exch{"2H15 SNP"}="H152 SNP";
$resn_exch{"1H15 SNP"}="H151 SNP"; $resn_exch{"3H16 SNP"}="H163 SNP"; $resn_exch{"2H16 SNP"}="H162 SNP";
$resn_exch{"1H16 SNP"}="H161 SNP"; $resn_exch{"1H1  SNR"}=" H11 SNR"; $resn_exch{"1H2  SNR"}=" H21 SNR";
$resn_exch{"2H1  SNR"}=" H12 SNR"; $resn_exch{"2H2  SNR"}=" H22 SNR"; $resn_exch{"3H1  SNR"}=" H13 SNR";
$resn_exch{"3H2  SNR"}=" H23 SNR"; $resn_exch{"1H12 SO1"}="H121 SO1"; $resn_exch{"2H12 SO1"}="H122 SO1";
$resn_exch{"1H10 SO1"}="H101 SO1"; $resn_exch{"2H10 SO1"}="H102 SO1"; $resn_exch{"1H18 SO1"}="H181 SO1";
$resn_exch{"2H18 SO1"}="H182 SO1"; $resn_exch{"1H24 SO1"}="H241 SO1"; $resn_exch{"2H24 SO1"}="H242 SO1";
$resn_exch{"1H25 SO1"}="H251 SO1"; $resn_exch{"2H25 SO1"}="H252 SO1"; $resn_exch{"3H25 SO1"}="H253 SO1";
$resn_exch{"1H7  SO1"}=" H71 SO1"; $resn_exch{"2H7  SO1"}=" H72 SO1"; $resn_exch{"1H8  SO1"}=" H81 SO1";
$resn_exch{"2H8  SO1"}=" H82 SO1"; $resn_exch{"1H61 SO1"}="H611 SO1"; $resn_exch{"2H61 SO1"}="H612 SO1";
$resn_exch{"3H61 SO1"}="H613 SO1"; $resn_exch{"1H65 SO1"}="H651 SO1"; $resn_exch{"2H65 SO1"}="H652 SO1";
$resn_exch{"3H65 SO1"}="H653 SO1"; $resn_exch{"1H20 SO1"}="H201 SO1"; $resn_exch{"2H20 SO1"}="H202 SO1";
$resn_exch{"3H20 SO1"}="H203 SO1"; $resn_exch{"1H21 SO1"}="H211 SO1"; $resn_exch{"2H21 SO1"}="H212 SO1";
$resn_exch{"3H21 SO1"}="H213 SO1"; $resn_exch{"1HNB SOA"}="HNB1 SOA"; $resn_exch{"2HNB SOA"}="HNB2 SOA";
$resn_exch{"1H12 SOA"}="H121 SOA"; $resn_exch{"2H12 SOA"}="H122 SOA"; $resn_exch{"1HN  SOC"}=" H   SOC";
$resn_exch{"2HN  SOC"}=" H2  SOC"; $resn_exch{"1HB  SOC"}=" HB2 SOC"; $resn_exch{"2HB  SOC"}=" HB3 SOC";
$resn_exch{"HSE1 SOC"}=" HE  SOC"; $resn_exch{"1H10 SOD"}="H101 SOD"; $resn_exch{"2H10 SOD"}="H102 SOD";
$resn_exch{"1H18 SOD"}="H181 SOD"; $resn_exch{"2H18 SOD"}="H182 SOD"; $resn_exch{"1H25 SOD"}="H251 SOD";
$resn_exch{"2H25 SOD"}="H252 SOD"; $resn_exch{"3H25 SOD"}="H253 SOD"; $resn_exch{"1H7  SOD"}=" H71 SOD";
$resn_exch{"2H7  SOD"}=" H72 SOD"; $resn_exch{"1H8  SOD"}=" H81 SOD"; $resn_exch{"2H8  SOD"}=" H82 SOD";
$resn_exch{"1H61 SOD"}="H611 SOD"; $resn_exch{"2H61 SOD"}="H612 SOD"; $resn_exch{"3H61 SOD"}="H613 SOD";
$resn_exch{"1H65 SOD"}="H651 SOD"; $resn_exch{"2H65 SOD"}="H652 SOD"; $resn_exch{"1H70 SOD"}="H701 SOD";
$resn_exch{"2H70 SOD"}="H702 SOD"; $resn_exch{"1H53 SOD"}="H531 SOD"; $resn_exch{"2H53 SOD"}="H532 SOD";
$resn_exch{"1H20 SOD"}="H201 SOD"; $resn_exch{"2H20 SOD"}="H202 SOD"; $resn_exch{"3H20 SOD"}="H203 SOD";
$resn_exch{"1H21 SOD"}="H211 SOD"; $resn_exch{"2H21 SOD"}="H212 SOD"; $resn_exch{"3H21 SOD"}="H213 SOD";
$resn_exch{"1H6  SOG"}=" H61 SOG"; $resn_exch{"2H6  SOG"}=" H62 SOG"; $resn_exch{"1H1' SOG"}="H1'1 SOG";
$resn_exch{"2H1' SOG"}="H1'2 SOG"; $resn_exch{"1H2' SOG"}="H2'1 SOG"; $resn_exch{"2H2' SOG"}="H2'2 SOG";
$resn_exch{"1H3' SOG"}="H3'1 SOG"; $resn_exch{"2H3' SOG"}="H3'2 SOG"; $resn_exch{"1H4' SOG"}="H4'1 SOG";
$resn_exch{"2H4' SOG"}="H4'2 SOG"; $resn_exch{"1H5' SOG"}="H5'1 SOG"; $resn_exch{"2H5' SOG"}="H5'2 SOG";
$resn_exch{"1H6' SOG"}="H6'1 SOG"; $resn_exch{"2H6' SOG"}="H6'2 SOG"; $resn_exch{"1H7' SOG"}="H7'1 SOG";
$resn_exch{"2H7' SOG"}="H7'2 SOG"; $resn_exch{"1H8' SOG"}="H8'1 SOG"; $resn_exch{"2H8' SOG"}="H8'2 SOG";
$resn_exch{"3H8' SOG"}="H8'3 SOG"; $resn_exch{"1H1  SOM"}=" H11 SOM"; $resn_exch{"2H1  SOM"}=" H12 SOM";
$resn_exch{"3H1  SOM"}=" H13 SOM"; $resn_exch{"1HB  SON"}=" HB1 SON"; $resn_exch{"2HB  SON"}=" HB2 SON";
$resn_exch{"1H5' SON"}="H5'1 SON"; $resn_exch{"2H5' SON"}="H5'2 SON"; $resn_exch{"1HN6 SON"}="HN61 SON";
$resn_exch{"2HN6 SON"}="HN62 SON"; $resn_exch{"1HN6 SOP"}="HN61 SOP"; $resn_exch{"2HN6 SOP"}="HN62 SOP";
$resn_exch{" H1* SOP"}=" H1' SOP"; $resn_exch{" H2* SOP"}=" H2' SOP"; $resn_exch{"HO2* SOP"}="HO2' SOP";
$resn_exch{" H3* SOP"}=" H3' SOP"; $resn_exch{" H4* SOP"}=" H4' SOP"; $resn_exch{"1H5* SOP"}="H5'1 SOP";
$resn_exch{"2H5* SOP"}="H5'2 SOP"; $resn_exch{"1HP9 SOP"}="HP91 SOP"; $resn_exch{"2HP9 SOP"}="HP92 SOP";
$resn_exch{"1HPB SOP"}="HPB1 SOP"; $resn_exch{"2HPB SOP"}="HPB2 SOP"; $resn_exch{"3HPB SOP"}="HPB3 SOP";
$resn_exch{"1HPA SOP"}="HPA1 SOP"; $resn_exch{"2HPA SOP"}="HPA2 SOP"; $resn_exch{"3HPA SOP"}="HPA3 SOP";
$resn_exch{"1HP5 SOP"}="HP51 SOP"; $resn_exch{"2HP5 SOP"}="HP52 SOP"; $resn_exch{"1HP4 SOP"}="HP41 SOP";
$resn_exch{"2HP4 SOP"}="HP42 SOP"; $resn_exch{"1HP2 SOP"}="HP21 SOP"; $resn_exch{"2HP2 SOP"}="HP22 SOP";
$resn_exch{"1HP1 SOP"}="HP11 SOP"; $resn_exch{"2HP1 SOP"}="HP12 SOP"; $resn_exch{"1HA1 SOP"}="HA11 SOP";
$resn_exch{"2HA1 SOP"}="HA12 SOP"; $resn_exch{"1HA3 SOP"}="HA31 SOP"; $resn_exch{"2HA3 SOP"}="HA32 SOP";
$resn_exch{"3HA3 SOP"}="HA33 SOP"; $resn_exch{"1H1  SOR"}=" H11 SOR"; $resn_exch{"2H1  SOR"}=" H12 SOR";
$resn_exch{"1H6  SOR"}=" H61 SOR"; $resn_exch{"2H6  SOR"}=" H62 SOR"; $resn_exch{"1H2  SOT"}=" H21 SOT";
$resn_exch{"2H2  SOT"}=" H22 SOT"; $resn_exch{"1H3  SOT"}=" H31 SOT"; $resn_exch{"2H3  SOT"}=" H32 SOT";
$resn_exch{"1H5  SOT"}=" H51 SOT"; $resn_exch{"2H5  SOT"}=" H52 SOT"; $resn_exch{"1H6  SOT"}=" H61 SOT";
$resn_exch{"2H6  SOT"}=" H62 SOT"; $resn_exch{"1H10 SOX"}="H101 SOX"; $resn_exch{"2H10 SOX"}="H102 SOX";
$resn_exch{"3H10 SOX"}="H103 SOX"; $resn_exch{"1H9C SOX"}="H9C1 SOX"; $resn_exch{"2H9C SOX"}="H9C2 SOX";
$resn_exch{"3H9C SOX"}="H9C3 SOX"; $resn_exch{"1H17 SOX"}="H171 SOX"; $resn_exch{"2H17 SOX"}="H172 SOX";
$resn_exch{"1HN  SOY"}=" HN1 SOY"; $resn_exch{"2HN  SOY"}=" HN2 SOY"; $resn_exch{"1HBC SOY"}="HBC1 SOY";
$resn_exch{"2HBC SOY"}="HBC2 SOY"; $resn_exch{"1H7C SOY"}="H7C1 SOY"; $resn_exch{"2H7C SOY"}="H7C2 SOY";
$resn_exch{"3H7C SOY"}="H7C3 SOY"; $resn_exch{"1H8C SOY"}="H8C1 SOY"; $resn_exch{"2H8C SOY"}="H8C2 SOY";
$resn_exch{"3H8C SOY"}="H8C3 SOY"; $resn_exch{"1H19 SOY"}="H191 SOY"; $resn_exch{"2H19 SOY"}="H192 SOY";
$resn_exch{"3H19 SOY"}="H193 SOY"; $resn_exch{"2HOP SP1"}="HOP2 SP1"; $resn_exch{"1H5* SP1"}="H5'1 SP1";
$resn_exch{"2H5* SP1"}="H5'2 SP1"; $resn_exch{" H4* SP1"}=" H4' SP1"; $resn_exch{" H3* SP1"}=" H3' SP1";
$resn_exch{" H2* SP1"}=" H2' SP1"; $resn_exch{"*HO2 SP1"}="HO2' SP1"; $resn_exch{" H1* SP1"}=" H1' SP1";
$resn_exch{"1HN6 SP1"}="HN61 SP1"; $resn_exch{"2HN6 SP1"}="HN62 SP1"; $resn_exch{"1HMA SP2"}="HMA1 SP2";
$resn_exch{"2HMA SP2"}="HMA2 SP2"; $resn_exch{"3HMA SP2"}="HMA3 SP2"; $resn_exch{"1HM1 SP2"}="HM11 SP2";
$resn_exch{"2HM1 SP2"}="HM12 SP2"; $resn_exch{"3HM1 SP2"}="HM13 SP2"; $resn_exch{"1HM2 SP2"}="HM21 SP2";
$resn_exch{"2HM2 SP2"}="HM22 SP2"; $resn_exch{"3HM2 SP2"}="HM23 SP2"; $resn_exch{"1H2  SP2"}="H21A SP2";
$resn_exch{"2H2  SP2"}=" H22 SP2"; $resn_exch{"1H3  SP2"}=" H31 SP2"; $resn_exch{"2H3  SP2"}=" H32 SP2";
$resn_exch{"1H4  SP2"}=" H41 SP2"; $resn_exch{"2H4  SP2"}=" H42 SP2"; $resn_exch{"1HM3 SP2"}="HM31 SP2";
$resn_exch{"2HM3 SP2"}="HM32 SP2"; $resn_exch{"3HM3 SP2"}="HM33 SP2"; $resn_exch{"1HM4 SP2"}="HM41 SP2";
$resn_exch{"2HM4 SP2"}="HM42 SP2"; $resn_exch{"3HM4 SP2"}="HM43 SP2"; $resn_exch{"1HM5 SP2"}="HM51 SP2";
$resn_exch{"2HM5 SP2"}="HM52 SP2"; $resn_exch{"3HM5 SP2"}="HM53 SP2"; $resn_exch{"1HM6 SP2"}="HM61 SP2";
$resn_exch{"2HM6 SP2"}="HM62 SP2"; $resn_exch{"3HM6 SP2"}="HM63 SP2"; $resn_exch{"1HM7 SP2"}="HM71 SP2";
$resn_exch{"2HM7 SP2"}="HM72 SP2"; $resn_exch{"3HM7 SP2"}="HM73 SP2"; $resn_exch{"1H23 SP2"}="H231 SP2";
$resn_exch{"2H23 SP2"}="H232 SP2"; $resn_exch{"1H24 SP2"}="H241 SP2"; $resn_exch{"2H24 SP2"}="H242 SP2";
$resn_exch{"1HM8 SP2"}="HM81 SP2"; $resn_exch{"2HM8 SP2"}="HM82 SP2"; $resn_exch{"3HM8 SP2"}="HM83 SP2";
$resn_exch{"1H27 SP2"}="H271 SP2"; $resn_exch{"2H27 SP2"}="H272 SP2"; $resn_exch{"1H28 SP2"}="H281 SP2";
$resn_exch{"2H28 SP2"}="H282 SP2"; $resn_exch{"1HMB SP2"}="HMB1 SP2"; $resn_exch{"2HMB SP2"}="HMB2 SP2";
$resn_exch{"3HMB SP2"}="HMB3 SP2"; $resn_exch{"1HM9 SP2"}="HM91 SP2"; $resn_exch{"2HM9 SP2"}="HM92 SP2";
$resn_exch{"3HM9 SP2"}="HM93 SP2"; $resn_exch{"1H4  SP6"}=" H41 SP6"; $resn_exch{"2H4  SP6"}=" H42 SP6";
$resn_exch{"1H6  SP6"}=" H61 SP6"; $resn_exch{"1H11 SP6"}="H111 SP6"; $resn_exch{"2H11 SP6"}="H112 SP6";
$resn_exch{"1H6  SPA"}=" H61 SPA"; $resn_exch{"2H6  SPA"}=" H62 SPA"; $resn_exch{"CH10 SPB"}="H10C SPB";
$resn_exch{"CH11 SPB"}="H11C SPB"; $resn_exch{"CH13 SPB"}="H13C SPB"; $resn_exch{"CH14 SPB"}="H14C SPB";
$resn_exch{"NH15 SPB"}="H15N SPB"; $resn_exch{"1H17 SPB"}="H171 SPB"; $resn_exch{"2H17 SPB"}="H172 SPB";
$resn_exch{"1H18 SPB"}="H181 SPB"; $resn_exch{"2H18 SPB"}="H182 SPB"; $resn_exch{"1H19 SPB"}="H191 SPB";
$resn_exch{"2H19 SPB"}="H192 SPB"; $resn_exch{"OH23 SPB"}="H23O SPB"; $resn_exch{"1HC2 SPC"}="HC21 SPC";
$resn_exch{"1HC3 SPC"}="HC31 SPC"; $resn_exch{"1HC5 SPC"}="HC51 SPC"; $resn_exch{"1HC6 SPC"}="HC61 SPC";
$resn_exch{"1HC8 SPC"}="HC81 SPC"; $resn_exch{"2HC8 SPC"}="HC82 SPC"; $resn_exch{"3HC8 SPC"}="HC83 SPC";
$resn_exch{"1H15 SPC"}="H151 SPC"; $resn_exch{"1H16 SPC"}="H161 SPC"; $resn_exch{"2H16 SPC"}="H162 SPC";
$resn_exch{"1H17 SPC"}="H171 SPC"; $resn_exch{"1H18 SPC"}="H181 SPC"; $resn_exch{"2H18 SPC"}="H182 SPC";
$resn_exch{"1H23 SPC"}="H231 SPC"; $resn_exch{"1H25 SPC"}="H251 SPC"; $resn_exch{"2H25 SPC"}="H252 SPC";
$resn_exch{"3H25 SPC"}="H253 SPC"; $resn_exch{"1HN1 SPD"}="HN11 SPD"; $resn_exch{"2HN1 SPD"}="HN12 SPD";
$resn_exch{"1H2  SPD"}=" H21 SPD"; $resn_exch{"2H2  SPD"}=" H22 SPD"; $resn_exch{"1H3  SPD"}=" H31 SPD";
$resn_exch{"2H3  SPD"}=" H32 SPD"; $resn_exch{"1H4  SPD"}=" H41 SPD"; $resn_exch{"2H4  SPD"}=" H42 SPD";
$resn_exch{"1H5  SPD"}=" H51 SPD"; $resn_exch{"2H5  SPD"}=" H52 SPD"; $resn_exch{"1H7  SPD"}=" H71 SPD";
$resn_exch{"2H7  SPD"}=" H72 SPD"; $resn_exch{"1H8  SPD"}=" H81 SPD"; $resn_exch{"2H8  SPD"}=" H82 SPD";
$resn_exch{"1H9  SPD"}=" H91 SPD"; $resn_exch{"2H9  SPD"}=" H92 SPD"; $resn_exch{"1H10 SPD"}="H101 SPD";
$resn_exch{"2H10 SPD"}="H102 SPD"; $resn_exch{"1HN1 SPE"}="HN11 SPE"; $resn_exch{"2HN1 SPE"}="HN12 SPE";
$resn_exch{"1H2  SPE"}=" H21 SPE"; $resn_exch{"2H2  SPE"}=" H22 SPE"; $resn_exch{"1H3  SPE"}=" H31 SPE";
$resn_exch{"2H3  SPE"}=" H32 SPE"; $resn_exch{"1H4  SPE"}=" H41 SPE"; $resn_exch{"2H4  SPE"}=" H42 SPE";
$resn_exch{"1H6  SPE"}=" H61 SPE"; $resn_exch{"2H6  SPE"}=" H62 SPE"; $resn_exch{"1H7  SPE"}=" H71 SPE";
$resn_exch{"2H7  SPE"}=" H72 SPE"; $resn_exch{"1H8  SPE"}=" H81 SPE"; $resn_exch{"2H8  SPE"}=" H82 SPE";
$resn_exch{"1H10 SPE"}="H101 SPE"; $resn_exch{"2H10 SPE"}="H102 SPE"; $resn_exch{"1H11 SPE"}="H111 SPE";
$resn_exch{"2H11 SPE"}="H112 SPE"; $resn_exch{"1H12 SPE"}="H121 SPE"; $resn_exch{"2H12 SPE"}="H122 SPE";
$resn_exch{"1HN3 SPE"}="HN31 SPE"; $resn_exch{"2HN3 SPE"}="HN32 SPE"; $resn_exch{"1HN6 SPF"}="HN61 SPF";
$resn_exch{"2HN6 SPF"}="HN62 SPF"; $resn_exch{" H1* SPF"}=" H1' SPF"; $resn_exch{" H2* SPF"}=" H2' SPF";
$resn_exch{" H3* SPF"}=" H3' SPF"; $resn_exch{" H4* SPF"}=" H4' SPF"; $resn_exch{"1H5* SPF"}="H5'1 SPF";
$resn_exch{"2H5* SPF"}="H5'2 SPF"; $resn_exch{"1H1  SPF"}=" H11 SPF"; $resn_exch{"2H1  SPF"}="H12X SPF";
$resn_exch{"3H1  SPF"}=" H13 SPF"; $resn_exch{"1H14 SPF"}="H141 SPF"; $resn_exch{"2H14 SPF"}="H142 SPF";
$resn_exch{"3H14 SPF"}="H143 SPF"; $resn_exch{"1H7  SPF"}=" H71 SPF"; $resn_exch{"2H7  SPF"}=" H72 SPF";
$resn_exch{"1H6  SPF"}=" H61 SPF"; $resn_exch{"2H6  SPF"}=" H62 SPF"; $resn_exch{"1H3  SPF"}=" H31 SPF";
$resn_exch{"2H3  SPF"}=" H32 SPF"; $resn_exch{"1H2  SPF"}=" H21 SPF"; $resn_exch{"2H2  SPF"}=" H22 SPF";
$resn_exch{"1H11 SPF"}="H111 SPF"; $resn_exch{"2H11 SPF"}="H112 SPF"; $resn_exch{"3H11 SPF"}="H113 SPF";
$resn_exch{"1H13 SPF"}="H131 SPF"; $resn_exch{"2H13 SPF"}="H132 SPF"; $resn_exch{"3H13 SPF"}="H133 SPF";
$resn_exch{"2HOP SPG"}="HOP2 SPG"; $resn_exch{"3HOP SPG"}="HOP3 SPG"; $resn_exch{"2HOA SPG"}="HOA2 SPG";
$resn_exch{"3HOA SPG"}="HOA3 SPG"; $resn_exch{"1H5* SPG"}="H5'1 SPG"; $resn_exch{"2H5* SPG"}="H5'2 SPG";
$resn_exch{" H4* SPG"}=" H4' SPG"; $resn_exch{" H3* SPG"}=" H3' SPG"; $resn_exch{"*HO3 SPG"}="HO3' SPG";
$resn_exch{" H2* SPG"}=" H2' SPG"; $resn_exch{"*HO2 SPG"}="HO2' SPG"; $resn_exch{" H1* SPG"}=" H1' SPG";
$resn_exch{"1H1  SPH"}=" H11 SPH"; $resn_exch{"2H1  SPH"}=" H12 SPH"; $resn_exch{"1HN2 SPH"}="HN21 SPH";
$resn_exch{"2HN2 SPH"}="HN22 SPH"; $resn_exch{"1H6  SPH"}=" H61 SPH"; $resn_exch{"2H6  SPH"}=" H62 SPH";
$resn_exch{"1H7  SPH"}=" H71 SPH"; $resn_exch{"2H7  SPH"}=" H72 SPH"; $resn_exch{"1H8  SPH"}=" H81 SPH";
$resn_exch{"2H8  SPH"}=" H82 SPH"; $resn_exch{"1H9  SPH"}=" H91 SPH"; $resn_exch{"2H9  SPH"}=" H92 SPH";
$resn_exch{"1H10 SPH"}="H101 SPH"; $resn_exch{"2H10 SPH"}="H102 SPH"; $resn_exch{"1H11 SPH"}="H111 SPH";
$resn_exch{"2H11 SPH"}="H112 SPH"; $resn_exch{"1H12 SPH"}="H121 SPH"; $resn_exch{"2H12 SPH"}="H122 SPH";
$resn_exch{"1H13 SPH"}="H131 SPH"; $resn_exch{"2H13 SPH"}="H132 SPH"; $resn_exch{"1H14 SPH"}="H141 SPH";
$resn_exch{"2H14 SPH"}="H142 SPH"; $resn_exch{"1H15 SPH"}="H151 SPH"; $resn_exch{"2H15 SPH"}="H152 SPH";
$resn_exch{"1H16 SPH"}="H161 SPH"; $resn_exch{"2H16 SPH"}="H162 SPH"; $resn_exch{"1H17 SPH"}="H171 SPH";
$resn_exch{"2H17 SPH"}="H172 SPH"; $resn_exch{"1H18 SPH"}="H181 SPH"; $resn_exch{"2H18 SPH"}="H182 SPH";
$resn_exch{"3H18 SPH"}="H183 SPH"; $resn_exch{"1HC2 SPI"}="HC21 SPI"; $resn_exch{"1HC3 SPI"}="HC31 SPI";
$resn_exch{"1HC5 SPI"}="HC51 SPI"; $resn_exch{"1HC6 SPI"}="HC61 SPI"; $resn_exch{"1HC7 SPI"}="HC71 SPI";
$resn_exch{"2HC7 SPI"}="HC72 SPI"; $resn_exch{"3HC7 SPI"}="HC73 SPI"; $resn_exch{"1HC8 SPI"}="HC81 SPI";
$resn_exch{"1HC9 SPI"}="HC91 SPI"; $resn_exch{"2HC9 SPI"}="HC92 SPI"; $resn_exch{"1H10 SPI"}="H101 SPI";
$resn_exch{"2H10 SPI"}="H102 SPI"; $resn_exch{"1H11 SPI"}="H111 SPI"; $resn_exch{"2H11 SPI"}="H112 SPI";
$resn_exch{"1H13 SPI"}="H131 SPI"; $resn_exch{"2H13 SPI"}="H132 SPI"; $resn_exch{"1H15 SPI"}="H151 SPI";
$resn_exch{"1H16 SPI"}="H161 SPI"; $resn_exch{"1H17 SPI"}="H171 SPI"; $resn_exch{"1H18 SPI"}="H181 SPI";
$resn_exch{"1H19 SPI"}="H191 SPI"; $resn_exch{"1H10 SPK"}="H101 SPK"; $resn_exch{"2H10 SPK"}="H102 SPK";
$resn_exch{"1H11 SPK"}="H111 SPK"; $resn_exch{"2H11 SPK"}="H112 SPK"; $resn_exch{"1H12 SPK"}="H121 SPK";
$resn_exch{"2H12 SPK"}="H122 SPK"; $resn_exch{"1H13 SPK"}="H131 SPK"; $resn_exch{"2H13 SPK"}="H132 SPK";
$resn_exch{"1H14 SPK"}="H141 SPK"; $resn_exch{"2H14 SPK"}="H142 SPK"; $resn_exch{"3H14 SPK"}="H143 SPK";
$resn_exch{"1H1  SPL"}=" H11 SPL"; $resn_exch{"2H1  SPL"}=" H12 SPL"; $resn_exch{"1H6  SPL"}=" H61 SPL";
$resn_exch{"2H6  SPL"}=" H62 SPL"; $resn_exch{"1H7  SPL"}=" H71 SPL"; $resn_exch{"2H7  SPL"}=" H72 SPL";
$resn_exch{"1H8  SPL"}=" H81 SPL"; $resn_exch{"2H8  SPL"}=" H82 SPL"; $resn_exch{"1H9  SPL"}=" H91 SPL";
$resn_exch{"2H9  SPL"}=" H92 SPL"; $resn_exch{"1H10 SPL"}="H101 SPL"; $resn_exch{"2H10 SPL"}="H102 SPL";
$resn_exch{"1H11 SPL"}="H111 SPL"; $resn_exch{"2H11 SPL"}="H112 SPL"; $resn_exch{"1H12 SPL"}="H121 SPL";
$resn_exch{"2H12 SPL"}="H122 SPL"; $resn_exch{"1H13 SPL"}="H131 SPL"; $resn_exch{"2H13 SPL"}="H132 SPL";
$resn_exch{"1H14 SPL"}="H141 SPL"; $resn_exch{"2H14 SPL"}="H142 SPL"; $resn_exch{"1H15 SPL"}="H151 SPL";
$resn_exch{"2H15 SPL"}="H152 SPL"; $resn_exch{"1H16 SPL"}="H161 SPL"; $resn_exch{"2H16 SPL"}="H162 SPL";
$resn_exch{"1H17 SPL"}="H171 SPL"; $resn_exch{"2H17 SPL"}="H172 SPL"; $resn_exch{"1H18 SPL"}="H181 SPL";
$resn_exch{"2H18 SPL"}="H182 SPL"; $resn_exch{"3H18 SPL"}="H183 SPL"; $resn_exch{"1H20 SPL"}="H201 SPL";
$resn_exch{"2H20 SPL"}="H202 SPL"; $resn_exch{"1H21 SPL"}="H211 SPL"; $resn_exch{"2H21 SPL"}="H212 SPL";
$resn_exch{"1H22 SPL"}="H221 SPL"; $resn_exch{"2H22 SPL"}="H222 SPL"; $resn_exch{"1H23 SPL"}="H231 SPL";
$resn_exch{"2H23 SPL"}="H232 SPL"; $resn_exch{"1H24 SPL"}="H241 SPL"; $resn_exch{"2H24 SPL"}="H242 SPL";
$resn_exch{"1H25 SPL"}="H251 SPL"; $resn_exch{"2H25 SPL"}="H252 SPL"; $resn_exch{"1H26 SPL"}="H261 SPL";
$resn_exch{"2H26 SPL"}="H262 SPL"; $resn_exch{"3H26 SPL"}="H263 SPL"; $resn_exch{"1HN1 SPM"}="HN11 SPM";
$resn_exch{"2HN1 SPM"}="HN12 SPM"; $resn_exch{"1H2  SPM"}=" H21 SPM"; $resn_exch{"2H2  SPM"}=" H22 SPM";
$resn_exch{"1H3  SPM"}=" H31 SPM"; $resn_exch{"2H3  SPM"}=" H32 SPM"; $resn_exch{"1H4  SPM"}=" H41 SPM";
$resn_exch{"2H4  SPM"}=" H42 SPM"; $resn_exch{"1H6  SPM"}=" H61 SPM"; $resn_exch{"2H6  SPM"}=" H62 SPM";
$resn_exch{"1H7  SPM"}=" H71 SPM"; $resn_exch{"2H7  SPM"}=" H72 SPM"; $resn_exch{"1H8  SPM"}=" H81 SPM";
$resn_exch{"2H8  SPM"}=" H82 SPM"; $resn_exch{"1H9  SPM"}=" H91 SPM"; $resn_exch{"2H9  SPM"}=" H92 SPM";
$resn_exch{"1H11 SPM"}="H111 SPM"; $resn_exch{"2H11 SPM"}="H112 SPM"; $resn_exch{"1H12 SPM"}="H121 SPM";
$resn_exch{"2H12 SPM"}="H122 SPM"; $resn_exch{"1H13 SPM"}="H131 SPM"; $resn_exch{"2H13 SPM"}="H132 SPM";
$resn_exch{"1HN4 SPM"}="HN41 SPM"; $resn_exch{"2HN4 SPM"}="HN42 SPM"; $resn_exch{"1HMA SPN"}="HMA1 SPN";
$resn_exch{"2HMA SPN"}="HMA2 SPN"; $resn_exch{"3HMA SPN"}="HMA3 SPN"; $resn_exch{"1HM1 SPN"}="HM11 SPN";
$resn_exch{"2HM1 SPN"}="HM12 SPN"; $resn_exch{"3HM1 SPN"}="HM13 SPN"; $resn_exch{"1HM2 SPN"}="HM21 SPN";
$resn_exch{"2HM2 SPN"}="HM22 SPN"; $resn_exch{"3HM2 SPN"}="HM23 SPN"; $resn_exch{"1H3  SPN"}=" H31 SPN";
$resn_exch{"2H3  SPN"}=" H32 SPN"; $resn_exch{"1HM3 SPN"}="HM31 SPN"; $resn_exch{"2HM3 SPN"}="HM32 SPN";
$resn_exch{"3HM3 SPN"}="HM33 SPN"; $resn_exch{"1H6  SPN"}=" H61 SPN"; $resn_exch{"2H6  SPN"}=" H62 SPN";
$resn_exch{"1H7  SPN"}=" H71 SPN"; $resn_exch{"2H7  SPN"}=" H72 SPN"; $resn_exch{"1HM4 SPN"}="HM41 SPN";
$resn_exch{"2HM4 SPN"}="HM42 SPN"; $resn_exch{"3HM4 SPN"}="HM43 SPN"; $resn_exch{"1H10 SPN"}="H101 SPN";
$resn_exch{"2H10 SPN"}="H102 SPN"; $resn_exch{"1H11 SPN"}="H111 SPN"; $resn_exch{"2H11 SPN"}="H112 SPN";
$resn_exch{"1HM5 SPN"}="HM51 SPN"; $resn_exch{"2HM5 SPN"}="HM52 SPN"; $resn_exch{"3HM5 SPN"}="HM53 SPN";
$resn_exch{"1H14 SPN"}="H141 SPN"; $resn_exch{"2H14 SPN"}="H142 SPN"; $resn_exch{"1H15 SPN"}="H151 SPN";
$resn_exch{"2H15 SPN"}="H152 SPN"; $resn_exch{"1H16 SPN"}="H161 SPN"; $resn_exch{"2H16 SPN"}="H162 SPN";
$resn_exch{"1H17 SPN"}="H171 SPN"; $resn_exch{"2H17 SPN"}="H172 SPN"; $resn_exch{"1HM6 SPN"}="HM61 SPN";
$resn_exch{"2HM6 SPN"}="HM62 SPN"; $resn_exch{"3HM6 SPN"}="HM63 SPN"; $resn_exch{"1H20 SPN"}="H201 SPN";
$resn_exch{"2H20 SPN"}="H202 SPN"; $resn_exch{"1H21 SPN"}="H211 SPN"; $resn_exch{"2H21 SPN"}="H212 SPN";
$resn_exch{"1HM7 SPN"}="HM71 SPN"; $resn_exch{"2HM7 SPN"}="HM72 SPN"; $resn_exch{"3HM7 SPN"}="HM73 SPN";
$resn_exch{"1H23 SPN"}="H231 SPN"; $resn_exch{"2H23 SPN"}="H232 SPN"; $resn_exch{"1H24 SPN"}="H241 SPN";
$resn_exch{"2H24 SPN"}="H242 SPN"; $resn_exch{"1HM8 SPN"}="HM81 SPN"; $resn_exch{"2HM8 SPN"}="HM82 SPN";
$resn_exch{"3HM8 SPN"}="HM83 SPN"; $resn_exch{"1H27 SPN"}="H271 SPN"; $resn_exch{"2H27 SPN"}="H272 SPN";
$resn_exch{"1H28 SPN"}="H281 SPN"; $resn_exch{"2H28 SPN"}="H282 SPN"; $resn_exch{"1HM9 SPN"}="HM91 SPN";
$resn_exch{"2HM9 SPN"}="HM92 SPN"; $resn_exch{"3HM9 SPN"}="HM93 SPN"; $resn_exch{"1HMB SPN"}="HMB1 SPN";
$resn_exch{"2HMB SPN"}="HMB2 SPN"; $resn_exch{"3HMB SPN"}="HMB3 SPN"; $resn_exch{"1HM1 SPO"}="HM11 SPO";
$resn_exch{"2HM1 SPO"}="HM12 SPO"; $resn_exch{"3HM1 SPO"}="HM13 SPO"; $resn_exch{"1H2  SPO"}="H21A SPO";
$resn_exch{"2H2  SPO"}="H22A SPO"; $resn_exch{"3H2  SPO"}=" H23 SPO"; $resn_exch{"1H3  SPO"}=" H31 SPO";
$resn_exch{"2H3  SPO"}="H32A SPO"; $resn_exch{"3H3  SPO"}=" H33 SPO"; $resn_exch{"1H4  SPO"}=" H41 SPO";
$resn_exch{"2H4  SPO"}=" H42 SPO"; $resn_exch{"1H8  SPO"}=" H81 SPO"; $resn_exch{"2H8  SPO"}=" H82 SPO";
$resn_exch{"3H8  SPO"}=" H83 SPO"; $resn_exch{"1H13 SPO"}="H131 SPO"; $resn_exch{"2H13 SPO"}="H132 SPO";
$resn_exch{"3H13 SPO"}="H133 SPO"; $resn_exch{"1H18 SPO"}="H181 SPO"; $resn_exch{"2H18 SPO"}="H182 SPO";
$resn_exch{"3H18 SPO"}="H183 SPO"; $resn_exch{"1H24 SPO"}="H241 SPO"; $resn_exch{"2H24 SPO"}="H242 SPO";
$resn_exch{"3H24 SPO"}="H243 SPO"; $resn_exch{"1H29 SPO"}="H291 SPO"; $resn_exch{"2H29 SPO"}="H292 SPO";
$resn_exch{"3H29 SPO"}="H293 SPO"; $resn_exch{"1H30 SPO"}="H301 SPO"; $resn_exch{"2H30 SPO"}="H302 SPO";
$resn_exch{"1H31 SPO"}="H311 SPO"; $resn_exch{"2H31 SPO"}="H312 SPO"; $resn_exch{"1H34 SPO"}="H341 SPO";
$resn_exch{"2H34 SPO"}="H342 SPO"; $resn_exch{"3H34 SPO"}="H343 SPO"; $resn_exch{"1H35 SPO"}="H351 SPO";
$resn_exch{"2H35 SPO"}="H352 SPO"; $resn_exch{"1H36 SPO"}="H361 SPO"; $resn_exch{"2H36 SPO"}="H362 SPO";
$resn_exch{"1H39 SPO"}="H391 SPO"; $resn_exch{"2H39 SPO"}="H392 SPO"; $resn_exch{"3H39 SPO"}="H393 SPO";
$resn_exch{"1H40 SPO"}="H401 SPO"; $resn_exch{"2H40 SPO"}="H402 SPO"; $resn_exch{"3H40 SPO"}="H403 SPO";
$resn_exch{"1H8  SPP"}=" H81 SPP"; $resn_exch{"2H8  SPP"}=" H82 SPP"; $resn_exch{"1H9  SPP"}=" H91 SPP";
$resn_exch{"2H9  SPP"}=" H92 SPP"; $resn_exch{"1H11 SPP"}="H111 SPP"; $resn_exch{"2H11 SPP"}="H112 SPP";
$resn_exch{"1H12 SPP"}="H121 SPP"; $resn_exch{"2H12 SPP"}="H122 SPP"; $resn_exch{"1HB  SPP"}=" HB1 SPP";
$resn_exch{"2HB  SPP"}=" HB2 SPP"; $resn_exch{"3HB  SPP"}=" HB3 SPP"; $resn_exch{"1HC  SPP"}=" HC1 SPP";
$resn_exch{"2HC  SPP"}=" HC2 SPP"; $resn_exch{"3HC  SPP"}=" HC3 SPP"; $resn_exch{"1HE  SPP"}=" HE1 SPP";
$resn_exch{"2HE  SPP"}=" HE2 SPP"; $resn_exch{"3HE  SPP"}=" HE3 SPP"; $resn_exch{"1H10 SPQ"}="H101 SPQ";
$resn_exch{"2H10 SPQ"}="H102 SPQ"; $resn_exch{"1H6  SPQ"}=" H61 SPQ"; $resn_exch{"2H6  SPQ"}=" H62 SPQ";
$resn_exch{"1H18 SPR"}="H181 SPR"; $resn_exch{"2H18 SPR"}="H182 SPR"; $resn_exch{"1H2  SPR"}=" H21 SPR";
$resn_exch{"2H2  SPR"}=" H22 SPR"; $resn_exch{"1H7  SPR"}=" H71 SPR"; $resn_exch{"2H7  SPR"}=" H72 SPR";
$resn_exch{"1H14 SPR"}="H141 SPR"; $resn_exch{"2H14 SPR"}="H142 SPR"; $resn_exch{"1H16 SPR"}="H161 SPR";
$resn_exch{"2H16 SPR"}="H162 SPR"; $resn_exch{"3H16 SPR"}="H163 SPR"; $resn_exch{"1H17 SPR"}="H171 SPR";
$resn_exch{"2H17 SPR"}="H172 SPR"; $resn_exch{"3H17 SPR"}="H173 SPR"; $resn_exch{"1H20 SPR"}="H201 SPR";
$resn_exch{"2H20 SPR"}="H202 SPR"; $resn_exch{"3H20 SPR"}="H203 SPR"; $resn_exch{"1H6A SPR"}="H6A1 SPR";
$resn_exch{"2H6A SPR"}="H6A2 SPR"; $resn_exch{"3H6A SPR"}="H6A3 SPR"; $resn_exch{"1H7A SPR"}="H7A1 SPR";
$resn_exch{"2H7A SPR"}="H7A2 SPR"; $resn_exch{"3H7A SPR"}="H7A3 SPR"; $resn_exch{"1H8A SPR"}="H8A1 SPR";
$resn_exch{"2H8A SPR"}="H8A2 SPR"; $resn_exch{"3H8A SPR"}="H8A3 SPR"; $resn_exch{"1H2B SPR"}="H2B1 SPR";
$resn_exch{"2H2B SPR"}="H2B2 SPR"; $resn_exch{"1H6B SPR"}="H6B1 SPR"; $resn_exch{"2H6B SPR"}="H6B2 SPR";
$resn_exch{"3H6B SPR"}="H6B3 SPR"; $resn_exch{"1H7B SPR"}="H7B1 SPR"; $resn_exch{"2H7B SPR"}="H7B2 SPR";
$resn_exch{"3H7B SPR"}="H7B3 SPR"; $resn_exch{"1H1C SPR"}="H1C1 SPR"; $resn_exch{"1H2C SPR"}="H2C1 SPR";
$resn_exch{"2H2C SPR"}="H2C2 SPR"; $resn_exch{"1H3C SPR"}="H3C1 SPR"; $resn_exch{"2H3C SPR"}="H3C2 SPR";
$resn_exch{"1H6C SPR"}="H6C1 SPR"; $resn_exch{"2H6C SPR"}="H6C2 SPR"; $resn_exch{"3H6C SPR"}="H6C3 SPR";
$resn_exch{"1H7C SPR"}="H7C1 SPR"; $resn_exch{"2H7C SPR"}="H7C2 SPR"; $resn_exch{"3H7C SPR"}="H7C3 SPR";
$resn_exch{"1H8C SPR"}="H8C1 SPR"; $resn_exch{"2H8C SPR"}="H8C2 SPR"; $resn_exch{"3H8C SPR"}="H8C3 SPR";
$resn_exch{"1H7  SPS"}=" H71 SPS"; $resn_exch{"2H7  SPS"}=" H72 SPS"; $resn_exch{"3H7  SPS"}=" H73 SPS";
$resn_exch{"1H8  SPS"}=" H81 SPS"; $resn_exch{"1H9  SPS"}=" H91 SPS"; $resn_exch{"1H12 SPS"}="H121 SPS";
$resn_exch{"1H13 SPS"}="H131 SPS"; $resn_exch{"2H13 SPS"}="H132 SPS"; $resn_exch{"1H14 SPS"}="H141 SPS";
$resn_exch{"2H14 SPS"}="H142 SPS"; $resn_exch{"1H16 SPS"}="H161 SPS"; $resn_exch{"2H16 SPS"}="H162 SPS";
$resn_exch{"1H18 SPS"}="H181 SPS"; $resn_exch{"2H18 SPS"}="H182 SPS"; $resn_exch{"3H18 SPS"}="H183 SPS";
$resn_exch{"1H5M SPT"}=" H71 SPT"; $resn_exch{"2H5M SPT"}=" H72 SPT"; $resn_exch{"3H5M SPT"}=" H73 SPT";
$resn_exch{"1H2* SPT"}=" H2' SPT"; $resn_exch{"2H2* SPT"}="H2'' SPT"; $resn_exch{"1H5* SPT"}=" H5' SPT";
$resn_exch{"2H5* SPT"}="H5'' SPT"; $resn_exch{" H4* SPT"}=" H4' SPT"; $resn_exch{" H1* SPT"}=" H1' SPT";
$resn_exch{" H3* SPT"}=" H3' SPT"; $resn_exch{"H3T  SPT"}="HO3' SPT"; $resn_exch{"3HOP SPT"}="HOP3 SPT";
$resn_exch{"2HOP SPT"}="HOP2 SPT"; $resn_exch{"OH2' SPV"}="H2'O SPV"; $resn_exch{"1H3  SPV"}=" H31 SPV";
$resn_exch{"2H3  SPV"}=" H32 SPV"; $resn_exch{"OH2S SPV"}="H2SO SPV"; $resn_exch{"1D10 SPW"}=" DXT SPW";
$resn_exch{"2D10 SPW"}=" DXU SPW"; $resn_exch{"1D14 SPW"}=" DXX SPW"; $resn_exch{"2D14 SPW"}=" DXY SPW";
$resn_exch{"3D14 SPW"}=" DXZ SPW"; $resn_exch{"1H8  SPX"}=" H81 SPX"; $resn_exch{"2H8  SPX"}=" H82 SPX";
$resn_exch{"1H48 SPX"}="H481 SPX"; $resn_exch{"2H48 SPX"}="H482 SPX"; $resn_exch{"3H48 SPX"}="H483 SPX";
$resn_exch{"1H6  SPX"}=" H61 SPX"; $resn_exch{"2H6  SPX"}=" H62 SPX"; $resn_exch{"1H38 SPX"}="H381 SPX";
$resn_exch{"2H38 SPX"}="H382 SPX"; $resn_exch{"3H38 SPX"}="H383 SPX"; $resn_exch{"1H41 SPX"}="H411 SPX";
$resn_exch{"2H41 SPX"}="H412 SPX"; $resn_exch{"3H41 SPX"}="H413 SPX"; $resn_exch{"1H49 SPX"}="H491 SPX";
$resn_exch{"2H49 SPX"}="H492 SPX"; $resn_exch{"3H49 SPX"}="H493 SPX"; $resn_exch{"1H14 SPX"}="H141 SPX";
$resn_exch{"2H14 SPX"}="H142 SPX"; $resn_exch{"1H50 SPX"}="H501 SPX"; $resn_exch{"2H50 SPX"}="H502 SPX";
$resn_exch{"3H50 SPX"}="H503 SPX"; $resn_exch{"1H42 SPX"}="H421 SPX"; $resn_exch{"2H42 SPX"}="H422 SPX";
$resn_exch{"3H42 SPX"}="H423 SPX"; $resn_exch{"1H20 SPX"}="H201 SPX"; $resn_exch{"2H20 SPX"}="H202 SPX";
$resn_exch{"1H51 SPX"}="H511 SPX"; $resn_exch{"2H51 SPX"}="H512 SPX"; $resn_exch{"3H51 SPX"}="H513 SPX";
$resn_exch{"1H43 SPX"}="H431 SPX"; $resn_exch{"2H43 SPX"}="H432 SPX"; $resn_exch{"3H43 SPX"}="H433 SPX";
$resn_exch{"1H44 SPX"}="H441 SPX"; $resn_exch{"2H44 SPX"}="H442 SPX"; $resn_exch{"3H44 SPX"}="H443 SPX";
$resn_exch{"1H52 SPX"}="H521 SPX"; $resn_exch{"2H52 SPX"}="H522 SPX"; $resn_exch{"3H52 SPX"}="H523 SPX";
$resn_exch{"1H45 SPX"}="H451 SPX"; $resn_exch{"2H45 SPX"}="H452 SPX"; $resn_exch{"3H45 SPX"}="H453 SPX";
$resn_exch{"1H29 SPX"}="H291 SPX"; $resn_exch{"2H29 SPX"}="H292 SPX"; $resn_exch{"1H30 SPX"}="H301 SPX";
$resn_exch{"2H30 SPX"}="H302 SPX"; $resn_exch{"1H46 SPX"}="H461 SPX"; $resn_exch{"2H46 SPX"}="H462 SPX";
$resn_exch{"3H46 SPX"}="H463 SPX"; $resn_exch{"1H53 SPX"}="H531 SPX"; $resn_exch{"2H53 SPX"}="H532 SPX";
$resn_exch{"3H53 SPX"}="H533 SPX"; $resn_exch{"1H34 SPX"}="H341 SPX"; $resn_exch{"2H34 SPX"}="H342 SPX";
$resn_exch{"1H47 SPX"}="H471 SPX"; $resn_exch{"2H47 SPX"}="H472 SPX"; $resn_exch{"3H47 SPX"}="H473 SPX";
$resn_exch{"1H1C SQA"}="H1C1 SQA"; $resn_exch{"2H1C SQA"}="H1C2 SQA"; $resn_exch{"3H1C SQA"}="H1C3 SQA";
$resn_exch{"1H3C SQA"}="H3C1 SQA"; $resn_exch{"2H3C SQA"}="H3C2 SQA"; $resn_exch{"1H4C SQA"}="H4C1 SQA";
$resn_exch{"2H4C SQA"}="H4C2 SQA"; $resn_exch{"1H5C SQA"}="H5C1 SQA"; $resn_exch{"2H5C SQA"}="H5C2 SQA";
$resn_exch{"1H8C SQA"}="H8C1 SQA"; $resn_exch{"2H8C SQA"}="H8C2 SQA"; $resn_exch{"1H9C SQA"}="H9C1 SQA";
$resn_exch{"2H9C SQA"}="H9C2 SQA"; $resn_exch{"1H12 SQA"}="H121 SQA"; $resn_exch{"2H12 SQA"}="H122 SQA";
$resn_exch{"1H13 SQA"}="H131 SQA"; $resn_exch{"2H13 SQA"}="H132 SQA"; $resn_exch{"1H16 SQA"}="H161 SQA";
$resn_exch{"2H16 SQA"}="H162 SQA"; $resn_exch{"1H17 SQA"}="H171 SQA"; $resn_exch{"2H17 SQA"}="H172 SQA";
$resn_exch{"1H20 SQA"}="H201 SQA"; $resn_exch{"2H20 SQA"}="H202 SQA"; $resn_exch{"1H21 SQA"}="H211 SQA";
$resn_exch{"2H21 SQA"}="H212 SQA"; $resn_exch{"1H24 SQA"}="H241 SQA"; $resn_exch{"2H24 SQA"}="H242 SQA";
$resn_exch{"3H24 SQA"}="H243 SQA"; $resn_exch{"1H25 SQA"}="H251 SQA"; $resn_exch{"2H25 SQA"}="H252 SQA";
$resn_exch{"3H25 SQA"}="H253 SQA"; $resn_exch{"1H26 SQA"}="H261 SQA"; $resn_exch{"2H26 SQA"}="H262 SQA";
$resn_exch{"3H26 SQA"}="H263 SQA"; $resn_exch{"1H27 SQA"}="H271 SQA"; $resn_exch{"2H27 SQA"}="H272 SQA";
$resn_exch{"3H27 SQA"}="H273 SQA"; $resn_exch{"1H28 SQA"}="H281 SQA"; $resn_exch{"2H28 SQA"}="H282 SQA";
$resn_exch{"3H28 SQA"}="H283 SQA"; $resn_exch{"1H29 SQA"}="H291 SQA"; $resn_exch{"2H29 SQA"}="H292 SQA";
$resn_exch{"3H29 SQA"}="H293 SQA"; $resn_exch{"1H30 SQA"}="H301 SQA"; $resn_exch{"2H30 SQA"}="H302 SQA";
$resn_exch{"3H30 SQA"}="H303 SQA"; $resn_exch{"1H44 SQD"}="H441 SQD"; $resn_exch{"2H44 SQD"}="H442 SQD";
$resn_exch{"1H46 SQD"}="H461 SQD"; $resn_exch{"2H46 SQD"}="H462 SQD"; $resn_exch{"1H8  SQD"}=" H81 SQD";
$resn_exch{"2H8  SQD"}=" H82 SQD"; $resn_exch{"1H9  SQD"}=" H91 SQD"; $resn_exch{"2H9  SQD"}=" H92 SQD";
$resn_exch{"1H10 SQD"}="H101 SQD"; $resn_exch{"2H10 SQD"}="H102 SQD"; $resn_exch{"1H11 SQD"}="H111 SQD";
$resn_exch{"2H11 SQD"}="H112 SQD"; $resn_exch{"1H12 SQD"}="H121 SQD"; $resn_exch{"2H12 SQD"}="H122 SQD";
$resn_exch{"1H13 SQD"}="H131 SQD"; $resn_exch{"2H13 SQD"}="H132 SQD"; $resn_exch{"1H14 SQD"}="H141 SQD";
$resn_exch{"2H14 SQD"}="H142 SQD"; $resn_exch{"1H15 SQD"}="H151 SQD"; $resn_exch{"2H15 SQD"}="H152 SQD";
$resn_exch{"1H16 SQD"}="H161 SQD"; $resn_exch{"2H16 SQD"}="H162 SQD"; $resn_exch{"1H17 SQD"}="H171 SQD";
$resn_exch{"2H17 SQD"}="H172 SQD"; $resn_exch{"1H18 SQD"}="H181 SQD"; $resn_exch{"2H18 SQD"}="H182 SQD";
$resn_exch{"1H19 SQD"}="H191 SQD"; $resn_exch{"2H19 SQD"}="H192 SQD"; $resn_exch{"1H20 SQD"}="H201 SQD";
$resn_exch{"2H20 SQD"}="H202 SQD"; $resn_exch{"1H21 SQD"}="H211 SQD"; $resn_exch{"2H21 SQD"}="H212 SQD";
$resn_exch{"1H22 SQD"}="H221 SQD"; $resn_exch{"2H22 SQD"}="H222 SQD"; $resn_exch{"3H22 SQD"}="H223 SQD";
$resn_exch{"1H24 SQD"}="H241 SQD"; $resn_exch{"2H24 SQD"}="H242 SQD"; $resn_exch{"1H25 SQD"}="H251 SQD";
$resn_exch{"2H25 SQD"}="H252 SQD"; $resn_exch{"1H26 SQD"}="H261 SQD"; $resn_exch{"2H26 SQD"}="H262 SQD";
$resn_exch{"1H27 SQD"}="H271 SQD"; $resn_exch{"2H27 SQD"}="H272 SQD"; $resn_exch{"1H28 SQD"}="H281 SQD";
$resn_exch{"2H28 SQD"}="H282 SQD"; $resn_exch{"1H29 SQD"}="H291 SQD"; $resn_exch{"2H29 SQD"}="H292 SQD";
$resn_exch{"1H30 SQD"}="H301 SQD"; $resn_exch{"2H30 SQD"}="H302 SQD"; $resn_exch{"1H31 SQD"}="H311 SQD";
$resn_exch{"2H31 SQD"}="H312 SQD"; $resn_exch{"1H32 SQD"}="H321 SQD"; $resn_exch{"2H32 SQD"}="H322 SQD";
$resn_exch{"1H33 SQD"}="H331 SQD"; $resn_exch{"2H33 SQD"}="H332 SQD"; $resn_exch{"1H34 SQD"}="H341 SQD";
$resn_exch{"2H34 SQD"}="H342 SQD"; $resn_exch{"1H35 SQD"}="H351 SQD"; $resn_exch{"2H35 SQD"}="H352 SQD";
$resn_exch{"1H36 SQD"}="H361 SQD"; $resn_exch{"2H36 SQD"}="H362 SQD"; $resn_exch{"1H37 SQD"}="H371 SQD";
$resn_exch{"2H37 SQD"}="H372 SQD"; $resn_exch{"1H38 SQD"}="H381 SQD"; $resn_exch{"2H38 SQD"}="H382 SQD";
$resn_exch{"3H38 SQD"}="H383 SQD"; $resn_exch{"1H6  SQD"}=" H61 SQD"; $resn_exch{"2H6  SQD"}=" H62 SQD";
$resn_exch{"1H1  SQU"}=" H11 SQU"; $resn_exch{"2H1  SQU"}="H12A SQU"; $resn_exch{"3H1  SQU"}=" H13 SQU";
$resn_exch{"1H3  SQU"}=" H31 SQU"; $resn_exch{"2H3  SQU"}=" H32 SQU"; $resn_exch{"3H3  SQU"}=" H33 SQU";
$resn_exch{"1H4  SQU"}=" H41 SQU"; $resn_exch{"2H4  SQU"}=" H42 SQU"; $resn_exch{"1H5  SQU"}=" H51 SQU";
$resn_exch{"2H5  SQU"}=" H52 SQU"; $resn_exch{"1H6  SQU"}=" H61 SQU"; $resn_exch{"2H6  SQU"}=" H62 SQU";
$resn_exch{"1H7  SQU"}=" H71 SQU"; $resn_exch{"2H7  SQU"}=" H72 SQU"; $resn_exch{"1H9  SQU"}=" H91 SQU";
$resn_exch{"2H9  SQU"}=" H92 SQU"; $resn_exch{"1H10 SQU"}="H101 SQU"; $resn_exch{"2H10 SQU"}="H102 SQU";
$resn_exch{"1H11 SQU"}="H111 SQU"; $resn_exch{"2H11 SQU"}="H112 SQU"; $resn_exch{"1H13 SQU"}="H131 SQU";
$resn_exch{"2H13 SQU"}="H132 SQU"; $resn_exch{"3H13 SQU"}="H133 SQU"; $resn_exch{"1H14 SQU"}="H141 SQU";
$resn_exch{"2H14 SQU"}="H142 SQU"; $resn_exch{"1H15 SQU"}="H151 SQU"; $resn_exch{"2H15 SQU"}="H152 SQU";
$resn_exch{"1H16 SQU"}="H161 SQU"; $resn_exch{"2H16 SQU"}="H162 SQU"; $resn_exch{"1H17 SQU"}="H171 SQU";
$resn_exch{"2H17 SQU"}="H172 SQU"; $resn_exch{"1H18 SQU"}="H181 SQU"; $resn_exch{"2H18 SQU"}="H182 SQU";
$resn_exch{"1H20 SQU"}="H201 SQU"; $resn_exch{"2H20 SQU"}="H202 SQU"; $resn_exch{"1H21 SQU"}="H211 SQU";
$resn_exch{"2H21 SQU"}="H212 SQU"; $resn_exch{"1H22 SQU"}="H221 SQU"; $resn_exch{"2H22 SQU"}="H222 SQU";
$resn_exch{"1H23 SQU"}="H231 SQU"; $resn_exch{"2H23 SQU"}="H232 SQU"; $resn_exch{"1H25 SQU"}="H251 SQU";
$resn_exch{"2H25 SQU"}="H252 SQU"; $resn_exch{"1H26 SQU"}="H261 SQU"; $resn_exch{"2H26 SQU"}="H262 SQU";
$resn_exch{"1H27 SQU"}="H271 SQU"; $resn_exch{"2H27 SQU"}="H272 SQU"; $resn_exch{"1H29 SQU"}="H291 SQU";
$resn_exch{"2H29 SQU"}="H292 SQU"; $resn_exch{"3H29 SQU"}="H293 SQU"; $resn_exch{"1H30 SQU"}="H301 SQU";
$resn_exch{"2H30 SQU"}="H302 SQU"; $resn_exch{"3H30 SQU"}="H303 SQU"; $resn_exch{"1HS  SR1"}=" HS1 SR1";
$resn_exch{"2HS  SR1"}=" HS2 SR1"; $resn_exch{"3HS  SR1"}=" HS3 SR1"; $resn_exch{"1H5' SR1"}="H5'1 SR1";
$resn_exch{"2H5' SR1"}="H5'2 SR1"; $resn_exch{"1HO' SR1"}="HO'1 SR1"; $resn_exch{"2HO' SR1"}="HO'2 SR1";
$resn_exch{"3HO' SR1"}="HO'3 SR1"; $resn_exch{"1H11 SR2"}="H111 SR2"; $resn_exch{"1H51 SR2"}="H511 SR2";
$resn_exch{"1H61 SR2"}="H611 SR2"; $resn_exch{"2H61 SR2"}="H612 SR2"; $resn_exch{"1H65 SR2"}="H651 SR2";
$resn_exch{"2H65 SR2"}="H652 SR2"; $resn_exch{"3H65 SR2"}="H653 SR2"; $resn_exch{"1H67 SR2"}="H671 SR2";
$resn_exch{"2H67 SR2"}="H672 SR2"; $resn_exch{"3H67 SR2"}="H673 SR2"; $resn_exch{"1HOP SRA"}="HOP1 SRA";
$resn_exch{"3HOP SRA"}="HOP3 SRA"; $resn_exch{"1H5* SRA"}=" H5' SRA"; $resn_exch{"2H5* SRA"}="H5'' SRA";
$resn_exch{" H4* SRA"}=" H4' SRA"; $resn_exch{" H3* SRA"}=" H3' SRA"; $resn_exch{"*HO3 SRA"}="HO3' SRA";
$resn_exch{"1H2* SRA"}=" H2' SRA"; $resn_exch{" H1* SRA"}=" H1' SRA"; $resn_exch{"1HN6 SRA"}="HN61 SRA";
$resn_exch{"2HN6 SRA"}="HN62 SRA"; $resn_exch{"*HO2 SRA"}="HO2' SRA"; $resn_exch{"1H7' SRB"}="H7'1 SRB";
$resn_exch{"2H7' SRB"}="H7'2 SRB"; $resn_exch{"1H9' SRB"}="H9'1 SRB"; $resn_exch{"2H9' SRB"}="H9'2 SRB";
$resn_exch{"3H9' SRB"}="H9'3 SRB"; $resn_exch{"1HOB SRB"}="HOB1 SRB"; $resn_exch{"2HOB SRB"}="HOB2 SRB";
$resn_exch{"3HOB SRB"}="HOB3 SRB"; $resn_exch{"1H7' SRD"}="H7'1 SRD"; $resn_exch{"2H7' SRD"}="H7'2 SRD";
$resn_exch{"1H9' SRD"}="H9'1 SRD"; $resn_exch{"2H9' SRD"}="H9'2 SRD"; $resn_exch{"3H9' SRD"}="H9'3 SRD";
$resn_exch{"1HOB SRD"}="HOB1 SRD"; $resn_exch{"2HOB SRD"}="HOB2 SRD"; $resn_exch{"3HOB SRD"}="HOB3 SRD";
$resn_exch{"1HB  SRI"}=" HB1 SRI"; $resn_exch{"2HB  SRI"}=" HB2 SRI"; $resn_exch{"1H14 SRL"}="H141 SRL";
$resn_exch{"2H14 SRL"}="H142 SRL"; $resn_exch{"3H14 SRL"}="H143 SRL"; $resn_exch{"1H15 SRL"}="H151 SRL";
$resn_exch{"2H15 SRL"}="H152 SRL"; $resn_exch{"3H15 SRL"}="H153 SRL"; $resn_exch{"1H16 SRL"}="H161 SRL";
$resn_exch{"2H16 SRL"}="H162 SRL"; $resn_exch{"3H16 SRL"}="H163 SRL"; $resn_exch{"1H17 SRL"}="H171 SRL";
$resn_exch{"2H17 SRL"}="H172 SRL"; $resn_exch{"3H17 SRL"}="H173 SRL"; $resn_exch{"1H18 SRL"}="H181 SRL";
$resn_exch{"2H18 SRL"}="H182 SRL"; $resn_exch{"3H18 SRL"}="H183 SRL"; $resn_exch{"1H19 SRL"}="H191 SRL";
$resn_exch{"2H19 SRL"}="H192 SRL"; $resn_exch{"3H19 SRL"}="H193 SRL"; $resn_exch{"1H22 SRL"}="H221 SRL";
$resn_exch{"2H22 SRL"}="H222 SRL"; $resn_exch{"1H23 SRL"}="H231 SRL"; $resn_exch{"2H23 SRL"}="H232 SRL";
$resn_exch{"3H23 SRL"}="H233 SRL"; $resn_exch{"1H25 SRL"}="H251 SRL"; $resn_exch{"2H25 SRL"}="H252 SRL";
$resn_exch{"1H26 SRL"}="H261 SRL"; $resn_exch{"2H26 SRL"}="H262 SRL"; $resn_exch{"3H26 SRL"}="H263 SRL";
$resn_exch{"1H29 SRL"}="H291 SRL"; $resn_exch{"2H29 SRL"}="H292 SRL"; $resn_exch{"1H30 SRL"}="H301 SRL";
$resn_exch{"2H30 SRL"}="H302 SRL"; $resn_exch{"3H30 SRL"}="H303 SRL"; $resn_exch{"1H32 SRL"}="H321 SRL";
$resn_exch{"2H32 SRL"}="H322 SRL"; $resn_exch{"1H33 SRL"}="H331 SRL"; $resn_exch{"2H33 SRL"}="H332 SRL";
$resn_exch{"3H33 SRL"}="H333 SRL"; $resn_exch{"1HMA SRM"}="HMA1 SRM"; $resn_exch{"2HMA SRM"}="HMA2 SRM";
$resn_exch{"3HMA SRM"}="HMA3 SRM"; $resn_exch{"1HDA SRM"}="HDA1 SRM"; $resn_exch{"2HDA SRM"}="HDA2 SRM";
$resn_exch{"1HAA SRM"}="HAA1 SRM"; $resn_exch{"2HAA SRM"}="HAA2 SRM"; $resn_exch{"1HBA SRM"}="HBA1 SRM";
$resn_exch{"2HBA SRM"}="HBA2 SRM"; $resn_exch{"1HMB SRM"}="HMB1 SRM"; $resn_exch{"2HMB SRM"}="HMB2 SRM";
$resn_exch{"3HMB SRM"}="HMB3 SRM"; $resn_exch{"1HDB SRM"}="HDB1 SRM"; $resn_exch{"2HDB SRM"}="HDB2 SRM";
$resn_exch{"1HAB SRM"}="HAB1 SRM"; $resn_exch{"2HAB SRM"}="HAB2 SRM"; $resn_exch{"1HBB SRM"}="HBB1 SRM";
$resn_exch{"2HBB SRM"}="HBB2 SRM"; $resn_exch{"1HCD SRM"}="HCD1 SRM"; $resn_exch{"2HCD SRM"}="HCD2 SRM";
$resn_exch{"1HAC SRM"}="HAC1 SRM"; $resn_exch{"2HAC SRM"}="HAC2 SRM"; $resn_exch{"1HBC SRM"}="HBC1 SRM";
$resn_exch{"2HBC SRM"}="HBC2 SRM"; $resn_exch{"1HAD SRM"}="HAD1 SRM"; $resn_exch{"2HAD SRM"}="HAD2 SRM";
$resn_exch{"1HBD SRM"}="HBD1 SRM"; $resn_exch{"2HBD SRM"}="HBD2 SRM"; $resn_exch{"1HDD SRM"}="HDD1 SRM";
$resn_exch{"2HDD SRM"}="HDD2 SRM"; $resn_exch{"1H44 SRN"}="H441 SRN"; $resn_exch{"2H44 SRN"}="H442 SRN";
$resn_exch{"1H43 SRN"}="H431 SRN"; $resn_exch{"2H43 SRN"}="H432 SRN"; $resn_exch{"1H42 SRN"}="H421 SRN";
$resn_exch{"2H42 SRN"}="H422 SRN"; $resn_exch{"1H41 SRN"}="H411 SRN"; $resn_exch{"2H41 SRN"}="H412 SRN";
$resn_exch{"1H40 SRN"}="H401 SRN"; $resn_exch{"2H40 SRN"}="H402 SRN"; $resn_exch{"3H40 SRN"}="H403 SRN";
$resn_exch{"1H37 SRN"}="H371 SRN"; $resn_exch{"2H37 SRN"}="H372 SRN"; $resn_exch{"3H37 SRN"}="H373 SRN";
$resn_exch{"1H30 SRN"}="H301 SRN"; $resn_exch{"2H30 SRN"}="H302 SRN"; $resn_exch{"1H27 SRN"}="H271 SRN";
$resn_exch{"2H27 SRN"}="H272 SRN"; $resn_exch{"1H26 SRN"}="H261 SRN"; $resn_exch{"2H26 SRN"}="H262 SRN";
$resn_exch{"1H20 SRN"}="H201 SRN"; $resn_exch{"2H20 SRN"}="H202 SRN"; $resn_exch{"1H18 SRN"}="H181 SRN";
$resn_exch{"2H18 SRN"}="H182 SRN"; $resn_exch{"3H18 SRN"}="H183 SRN"; $resn_exch{"1H15 SRN"}="H151 SRN";
$resn_exch{"2H15 SRN"}="H152 SRN"; $resn_exch{"1H34 SRN"}="H341 SRN"; $resn_exch{"2H34 SRN"}="H342 SRN";
$resn_exch{"1H11 SRN"}="H111 SRN"; $resn_exch{"2H11 SRN"}="H112 SRN"; $resn_exch{"3H11 SRN"}="H113 SRN";
$resn_exch{"1HB  SRO"}=" HB1 SRO"; $resn_exch{"2HB  SRO"}=" HB2 SRO"; $resn_exch{"1HA  SRO"}=" HA1 SRO";
$resn_exch{"2HA  SRO"}=" HA2 SRO"; $resn_exch{"1HNZ SRO"}="HNZ1 SRO"; $resn_exch{"2HNZ SRO"}="HNZ2 SRO";
$resn_exch{"1H5* SRP"}="H5'1 SRP"; $resn_exch{"2H5* SRP"}="H5'2 SRP"; $resn_exch{" H4* SRP"}=" H4' SRP";
$resn_exch{" H3* SRP"}=" H3' SRP"; $resn_exch{" H2* SRP"}=" H2' SRP"; $resn_exch{" H1* SRP"}=" H1' SRP";
$resn_exch{"1HN6 SRP"}="HN61 SRP"; $resn_exch{"2HN6 SRP"}="HN62 SRP"; $resn_exch{"1HN  SRP"}=" HN1 SRP";
$resn_exch{"2HN  SRP"}=" HN2 SRP"; $resn_exch{"1HB  SRP"}=" HB1 SRP"; $resn_exch{"2HB  SRP"}=" HB2 SRP";
$resn_exch{"1H2  SRS"}=" H21 SRS"; $resn_exch{"2H2  SRS"}="H22A SRS"; $resn_exch{"1H7  SRS"}=" H71 SRS";
$resn_exch{"2H7  SRS"}=" H72 SRS"; $resn_exch{"1H8  SRS"}=" H81 SRS"; $resn_exch{"2H8  SRS"}=" H82 SRS";
$resn_exch{"1H9  SRS"}=" H91 SRS"; $resn_exch{"2H9  SRS"}=" H92 SRS"; $resn_exch{"1H10 SRS"}="H101 SRS";
$resn_exch{"2H10 SRS"}="H102 SRS"; $resn_exch{"1H11 SRS"}="H111 SRS"; $resn_exch{"2H11 SRS"}="H112 SRS";
$resn_exch{"1H12 SRS"}="H121 SRS"; $resn_exch{"2H12 SRS"}="H122 SRS"; $resn_exch{"1H13 SRS"}="H131 SRS";
$resn_exch{"2H13 SRS"}="H132 SRS"; $resn_exch{"1H15 SRS"}="H151 SRS"; $resn_exch{"2H15 SRS"}="H152 SRS";
$resn_exch{"3H15 SRS"}="H153 SRS"; $resn_exch{"1H16 SRS"}="H161 SRS"; $resn_exch{"2H16 SRS"}="H162 SRS";
$resn_exch{"3H16 SRS"}="H163 SRS"; $resn_exch{"1H17 SRS"}="H171 SRS"; $resn_exch{"2H17 SRS"}="H172 SRS";
$resn_exch{"1H26 SRS"}="H261 SRS"; $resn_exch{"2H26 SRS"}="H262 SRS"; $resn_exch{"3H26 SRS"}="H263 SRS";
$resn_exch{"1HN1 SRY"}="HN11 SRY"; $resn_exch{"1HB1 SRY"}="HB11 SRY"; $resn_exch{"2HB1 SRY"}="HB12 SRY";
$resn_exch{"1HC1 SRY"}="HC11 SRY"; $resn_exch{"1HO2 SRY"}="HO21 SRY"; $resn_exch{"1HN3 SRY"}="HN31 SRY";
$resn_exch{"1HE1 SRY"}="HE11 SRY"; $resn_exch{"2HE1 SRY"}="HE12 SRY"; $resn_exch{"1HF1 SRY"}="HF11 SRY";
$resn_exch{"1HO5 SRY"}="HO51 SRY"; $resn_exch{"1HO6 SRY"}="HO61 SRY"; $resn_exch{"2HO3 SRY"}="HO32 SRY";
$resn_exch{"1HG2 SRY"}="HG21 SRY"; $resn_exch{"1HH2 SRY"}="HH21 SRY"; $resn_exch{"2HH2 SRY"}="HH22 SRY";
$resn_exch{"3HH2 SRY"}="HH23 SRY"; $resn_exch{"3HN2 SRY"}="HN23 SRY"; $resn_exch{"1HI3 SRY"}="HI31 SRY";
$resn_exch{"2HI3 SRY"}="HI32 SRY"; $resn_exch{"3HI3 SRY"}="HI33 SRY"; $resn_exch{"3HO3 SRY"}="HO33 SRY";
$resn_exch{"3HO4 SRY"}="HO43 SRY"; $resn_exch{"1H63 SRY"}="H631 SRY"; $resn_exch{"2H63 SRY"}="H632 SRY";
$resn_exch{"3HO6 SRY"}="HO63 SRY"; $resn_exch{"1H8  SS1"}=" H81 SS1"; $resn_exch{"2H8  SS1"}=" H82 SS1";
$resn_exch{"3H8  SS1"}=" H83 SS1"; $resn_exch{"1H8  SS2"}=" H81 SS2"; $resn_exch{"2H8  SS2"}=" H82 SS2";
$resn_exch{"3H8  SS2"}=" H83 SS2"; $resn_exch{"1H9C SS3"}="H9C1 SS3"; $resn_exch{"2H9C SS3"}="H9C2 SS3";
$resn_exch{"3H9C SS3"}="H9C3 SS3"; $resn_exch{"1H24 SS3"}="H241 SS3"; $resn_exch{"2H24 SS3"}="H242 SS3";
$resn_exch{"1H25 SS3"}="H251 SS3"; $resn_exch{"2H25 SS3"}="H252 SS3"; $resn_exch{"1H4N SS3"}="H4N1 SS3";
$resn_exch{"2H4N SS3"}="H4N2 SS3"; $resn_exch{"1H9C SS4"}="H9C1 SS4"; $resn_exch{"2H9C SS4"}="H9C2 SS4";
$resn_exch{"3H9C SS4"}="H9C3 SS4"; $resn_exch{"1H25 SS4"}="H251 SS4"; $resn_exch{"2H25 SS4"}="H252 SS4";
$resn_exch{"1H24 SS4"}="H241 SS4"; $resn_exch{"2H24 SS4"}="H242 SS4"; $resn_exch{"1H4N SS4"}="H4N1 SS4";
$resn_exch{"2H4N SS4"}="H4N2 SS4"; $resn_exch{"1H7C SS5"}="H7C1 SS5"; $resn_exch{"2H7C SS5"}="H7C2 SS5";
$resn_exch{"1H29 SS5"}="H291 SS5"; $resn_exch{"2H29 SS5"}="H292 SS5"; $resn_exch{"1H10 SS5"}="H101 SS5";
$resn_exch{"2H10 SS5"}="H102 SS5"; $resn_exch{"1H12 SS5"}="H121 SS5"; $resn_exch{"2H12 SS5"}="H122 SS5";
$resn_exch{"1H28 SS5"}="H281 SS5"; $resn_exch{"2H28 SS5"}="H282 SS5"; $resn_exch{"3H28 SS5"}="H283 SS5";
$resn_exch{"1H31 SS5"}="H311 SS5"; $resn_exch{"2H31 SS5"}="H312 SS5"; $resn_exch{"1H35 SS5"}="H351 SS5";
$resn_exch{"2H35 SS5"}="H352 SS5"; $resn_exch{"1H32 SS5"}="H321 SS5"; $resn_exch{"2H32 SS5"}="H322 SS5";
$resn_exch{"1H33 SS5"}="H331 SS5"; $resn_exch{"2H33 SS5"}="H332 SS5"; $resn_exch{"1H34 SS5"}="H341 SS5";
$resn_exch{"2H34 SS5"}="H342 SS5"; $resn_exch{"1HN1 SSA"}="HN11 SSA"; $resn_exch{"2HN1 SSA"}="HN12 SSA";
$resn_exch{"1HB  SSA"}=" HB1 SSA"; $resn_exch{"2HB  SSA"}=" HB2 SSA"; $resn_exch{"1H5* SSA"}="H5'1 SSA";
$resn_exch{"2H5* SSA"}="H5'2 SSA"; $resn_exch{" H4* SSA"}=" H4' SSA"; $resn_exch{" H3* SSA"}=" H3' SSA";
$resn_exch{"*HO3 SSA"}="HO3' SSA"; $resn_exch{" H2* SSA"}=" H2' SSA"; $resn_exch{"*HO2 SSA"}="HO2' SSA";
$resn_exch{" H1* SSA"}=" H1' SSA"; $resn_exch{"1HN6 SSA"}="HN61 SSA"; $resn_exch{"2HN6 SSA"}="HN62 SSA";
$resn_exch{"1H2  SSB"}=" H21 SSB"; $resn_exch{"2H2  SSB"}=" H22 SSB"; $resn_exch{"1H4  SSB"}=" H41 SSB";
$resn_exch{"2H4  SSB"}=" H42 SSB"; $resn_exch{"1H5  SSB"}=" H51 SSB"; $resn_exch{"2H5  SSB"}=" H52 SSB";
$resn_exch{"1H7  SSB"}=" H71 SSB"; $resn_exch{"2H7  SSB"}=" H72 SSB"; $resn_exch{"1H8  SSB"}=" H81 SSB";
$resn_exch{"2H8  SSB"}=" H82 SSB"; $resn_exch{"1H9  SSB"}=" H91 SSB"; $resn_exch{"2H9  SSB"}=" H92 SSB";
$resn_exch{"1H10 SSB"}="H101 SSB"; $resn_exch{"2H10 SSB"}="H102 SSB"; $resn_exch{"3H10 SSB"}="H103 SSB";
$resn_exch{"1HSS SSC"}="HSS1 SSC"; $resn_exch{"2HSS SSC"}="HSS2 SSC"; $resn_exch{"1H3S SSC"}="H3S1 SSC";
$resn_exch{"2H3S SSC"}="H3S2 SSC"; $resn_exch{"1H5S SSC"}="H5S1 SSC"; $resn_exch{"2H5S SSC"}="H5S2 SSC";
$resn_exch{"1H3  SSD"}=" H31 SSD"; $resn_exch{"2H3  SSD"}=" H32 SSD"; $resn_exch{"1H6  SSD"}=" H61 SSD";
$resn_exch{"2H6  SSD"}=" H62 SSD"; $resn_exch{"1H7  SSD"}=" H71 SSD"; $resn_exch{"2H7  SSD"}=" H72 SSD";
$resn_exch{"1H10 SSD"}="H101 SSD"; $resn_exch{"2H10 SSD"}="H102 SSD"; $resn_exch{"1H3  SSE"}=" H31 SSE";
$resn_exch{"2H3  SSE"}=" H32 SSE"; $resn_exch{"1H6  SSE"}=" H61 SSE"; $resn_exch{"2H6  SSE"}=" H62 SSE";
$resn_exch{"1H7  SSE"}=" H71 SSE"; $resn_exch{"2H7  SSE"}=" H72 SSE"; $resn_exch{"1H10 SSE"}="H101 SSE";
$resn_exch{"2H10 SSE"}="H102 SSE"; $resn_exch{"2H6  SSG"}=" H62 SSG"; $resn_exch{"1H6  SSG"}=" H61 SSG";
$resn_exch{"1H2C SSH"}="H2C1 SSH"; $resn_exch{"1H3C SSH"}="H3C1 SSH"; $resn_exch{"2H3C SSH"}="H3C2 SSH";
$resn_exch{"1H6C SSH"}="H6C1 SSH"; $resn_exch{"2H6C SSH"}="H6C2 SSH"; $resn_exch{"1H3  SSO"}=" H31 SSO";
$resn_exch{"2H3  SSO"}=" H32 SSO"; $resn_exch{"1H6  SSO"}=" H61 SSO"; $resn_exch{"2H6  SSO"}=" H62 SSO";
$resn_exch{"1H7  SSO"}=" H71 SSO"; $resn_exch{"2H7  SSO"}=" H72 SSO"; $resn_exch{"1H10 SSO"}="H101 SSO";
$resn_exch{"2H10 SSO"}="H102 SSO"; $resn_exch{"1HA1 SSP"}="HA11 SSP"; $resn_exch{"2HA1 SSP"}="HA12 SSP";
$resn_exch{"3HA1 SSP"}="HA13 SSP"; $resn_exch{"1HB1 SSP"}="HB11 SSP"; $resn_exch{"2HB1 SSP"}="HB12 SSP";
$resn_exch{"1HG1 SSP"}="HG11 SSP"; $resn_exch{"2HG1 SSP"}="HG12 SSP"; $resn_exch{"1HA2 SSP"}="HA21 SSP";
$resn_exch{"2HA2 SSP"}="HA22 SSP"; $resn_exch{"3HA2 SSP"}="HA23 SSP"; $resn_exch{"1HB2 SSP"}="HB21 SSP";
$resn_exch{"2HB2 SSP"}="HB22 SSP"; $resn_exch{"1HG2 SSP"}="HG21 SSP"; $resn_exch{"2HG2 SSP"}="HG22 SSP";
$resn_exch{"1H5' SSS"}="H5'1 SSS"; $resn_exch{"2H5' SSS"}="H5'2 SSS"; $resn_exch{"1H3N SSS"}="H3N1 SSS";
$resn_exch{"2H3N SSS"}="H3N2 SSS"; $resn_exch{"1H63 SSS"}="H631 SSS"; $resn_exch{"2H63 SSS"}="H632 SSS";
$resn_exch{"2HO* SSU"}="HO2' SSU"; $resn_exch{" H2* SSU"}=" H2' SSU"; $resn_exch{" H1* SSU"}=" H1' SSU";
$resn_exch{" H4* SSU"}=" H4' SSU"; $resn_exch{"1H5* SSU"}=" H5' SSU"; $resn_exch{"2H5* SSU"}="H5'' SSU";
$resn_exch{" H3* SSU"}=" H3' SSU"; $resn_exch{"H3T  SSU"}="HO3' SSU"; $resn_exch{"6H   SSU"}=" H6  SSU";
$resn_exch{"5H   SSU"}=" H5  SSU"; $resn_exch{"3H   SSU"}=" H3  SSU"; $resn_exch{"'HO2 ST1"}="HO2' ST1";
$resn_exch{"1HM4 ST1"}="HM41 ST1"; $resn_exch{"2HM4 ST1"}="HM42 ST1"; $resn_exch{"3HM4 ST1"}="HM43 ST1";
$resn_exch{"'HO2 ST2"}="HO2' ST2"; $resn_exch{"1HN3 ST2"}="HN31 ST2"; $resn_exch{"2HN3 ST2"}="HN32 ST2";
$resn_exch{"1HM4 ST2"}="HM41 ST2"; $resn_exch{"2HM4 ST2"}="HM42 ST2"; $resn_exch{"3HM4 ST2"}="HM43 ST2";
$resn_exch{"'HO2 ST3"}="HO2' ST3"; $resn_exch{"1HN3 ST3"}="HN31 ST3"; $resn_exch{"2HN3 ST3"}="HN32 ST3";
$resn_exch{"1HM4 ST3"}="HM41 ST3"; $resn_exch{"2HM4 ST3"}="HM42 ST3"; $resn_exch{"3HM4 ST3"}="HM43 ST3";
$resn_exch{"'HO2 ST4"}="HO2' ST4"; $resn_exch{"1HN1 ST4"}="HN11 ST4"; $resn_exch{"2HN1 ST4"}="HN12 ST4";
$resn_exch{"1HN2 ST4"}="HN21 ST4"; $resn_exch{"2HN2 ST4"}="HN22 ST4"; $resn_exch{"1HM4 ST4"}="HM41 ST4";
$resn_exch{"2HM4 ST4"}="HM42 ST4"; $resn_exch{"3HM4 ST4"}="HM43 ST4"; $resn_exch{"'HO2 ST5"}="HO2' ST5";
$resn_exch{"1HM3 ST5"}="HM31 ST5"; $resn_exch{"2HM3 ST5"}="HM32 ST5"; $resn_exch{"1HM4 ST5"}="HM41 ST5";
$resn_exch{"2HM4 ST5"}="HM42 ST5"; $resn_exch{"3HM4 ST5"}="HM43 ST5"; $resn_exch{"'HO2 ST6"}="HO2' ST6";
$resn_exch{"1HM3 ST6"}="HM31 ST6"; $resn_exch{"2HM3 ST6"}="HM32 ST6"; $resn_exch{"1HN3 ST6"}="HN31 ST6";
$resn_exch{"2HN3 ST6"}="HN32 ST6"; $resn_exch{"3HN3 ST6"}="HN33 ST6"; $resn_exch{"1HM4 ST6"}="HM41 ST6";
$resn_exch{"2HM4 ST6"}="HM42 ST6"; $resn_exch{"3HM4 ST6"}="HM43 ST6"; $resn_exch{"1H10 ST8"}="H101 ST8";
$resn_exch{"2H10 ST8"}="H102 ST8"; $resn_exch{"1H12 ST8"}="H121 ST8"; $resn_exch{"2H12 ST8"}="H122 ST8";
$resn_exch{"1H13 ST8"}="H131 ST8"; $resn_exch{"2H13 ST8"}="H132 ST8"; $resn_exch{"1H14 ST8"}="H141 ST8";
$resn_exch{"2H14 ST8"}="H142 ST8"; $resn_exch{"1H15 ST8"}="H151 ST8"; $resn_exch{"2H15 ST8"}="H152 ST8";
$resn_exch{"1H16 ST8"}="H161 ST8"; $resn_exch{"2H16 ST8"}="H162 ST8"; $resn_exch{"1H25 ST8"}="H251 ST8";
$resn_exch{"2H25 ST8"}="H252 ST8"; $resn_exch{"1H9N ST8"}="H9N1 ST8"; $resn_exch{"2H9N ST8"}="H9N2 ST8";
$resn_exch{"1HN  STA"}=" HN1 STA"; $resn_exch{"2HN  STA"}=" HN2 STA"; $resn_exch{"1HB  STA"}=" HB1 STA";
$resn_exch{"2HB  STA"}=" HB2 STA"; $resn_exch{"1HD1 STA"}="HD11 STA"; $resn_exch{"2HD1 STA"}="HD12 STA";
$resn_exch{"3HD1 STA"}="HD13 STA"; $resn_exch{"1HD2 STA"}="HD21 STA"; $resn_exch{"2HD2 STA"}="HD22 STA";
$resn_exch{"3HD2 STA"}="HD23 STA"; $resn_exch{"1HM  STA"}=" HM1 STA"; $resn_exch{"2HM  STA"}=" HM2 STA";
$resn_exch{"1H1' STB"}="H1'1 STB"; $resn_exch{"2H1' STB"}="H1'2 STB"; $resn_exch{"1H2' STB"}="H2'1 STB";
$resn_exch{"2H2' STB"}="H2'2 STB"; $resn_exch{"1H3' STB"}="H3'1 STB"; $resn_exch{"2H3' STB"}="H3'2 STB";
$resn_exch{"1H4' STB"}="H4'1 STB"; $resn_exch{"2H4' STB"}="H4'2 STB"; $resn_exch{"1H6' STB"}="H6'1 STB";
$resn_exch{"2H6' STB"}="H6'2 STB"; $resn_exch{"1HN3 STB"}="HN31 STB"; $resn_exch{"2HN3 STB"}="HN32 STB";
$resn_exch{"1HN1 STC"}="HN11 STC"; $resn_exch{"1H3  STC"}=" H31 STC"; $resn_exch{"1H4  STC"}=" H41 STC";
$resn_exch{"1H5  STC"}=" H51 STC"; $resn_exch{"1H6  STC"}=" H61 STC"; $resn_exch{"1H19 STC"}="H191 STC";
$resn_exch{"1H22 STC"}="H221 STC"; $resn_exch{"1H23 STC"}="H231 STC"; $resn_exch{"1H11 STD"}="H111 STD";
$resn_exch{"2H11 STD"}="H112 STD"; $resn_exch{"3H11 STD"}="H113 STD"; $resn_exch{"1H12 STD"}="H121 STD";
$resn_exch{"2H12 STD"}="H122 STD"; $resn_exch{"1H14 STD"}="H141 STD"; $resn_exch{"2H14 STD"}="H142 STD";
$resn_exch{"3H14 STD"}="H143 STD"; $resn_exch{"1H15 STD"}="H151 STD"; $resn_exch{"2H15 STD"}="H152 STD";
$resn_exch{"1H18 STD"}="H181 STD"; $resn_exch{"2H18 STD"}="H182 STD"; $resn_exch{"3H18 STD"}="H183 STD";
$resn_exch{"1H23 STD"}="H231 STD"; $resn_exch{"2H23 STD"}="H232 STD"; $resn_exch{"3H23 STD"}="H233 STD";
$resn_exch{"1H24 STD"}="H241 STD"; $resn_exch{"2H24 STD"}="H242 STD"; $resn_exch{"3H24 STD"}="H243 STD";
$resn_exch{"1H27 STD"}="H271 STD"; $resn_exch{"2H27 STD"}="H272 STD"; $resn_exch{"3H27 STD"}="H273 STD";
$resn_exch{"1H29 STD"}="H291 STD"; $resn_exch{"2H29 STD"}="H292 STD"; $resn_exch{"3H29 STD"}="H293 STD";
$resn_exch{"1H31 STD"}="H311 STD"; $resn_exch{"2H31 STD"}="H312 STD"; $resn_exch{"1H2  STE"}=" H21 STE";
$resn_exch{"2H2  STE"}=" H22 STE"; $resn_exch{"1H3  STE"}=" H31 STE"; $resn_exch{"2H3  STE"}=" H32 STE";
$resn_exch{"1H4  STE"}=" H41 STE"; $resn_exch{"2H4  STE"}=" H42 STE"; $resn_exch{"1H5  STE"}=" H51 STE";
$resn_exch{"2H5  STE"}=" H52 STE"; $resn_exch{"1H6  STE"}=" H61 STE"; $resn_exch{"2H6  STE"}=" H62 STE";
$resn_exch{"1H7  STE"}=" H71 STE"; $resn_exch{"2H7  STE"}=" H72 STE"; $resn_exch{"1H8  STE"}=" H81 STE";
$resn_exch{"2H8  STE"}=" H82 STE"; $resn_exch{"1H9  STE"}=" H91 STE"; $resn_exch{"2H9  STE"}=" H92 STE";
$resn_exch{"1H10 STE"}="H101 STE"; $resn_exch{"2H10 STE"}="H102 STE"; $resn_exch{"1H11 STE"}="H111 STE";
$resn_exch{"2H11 STE"}="H112 STE"; $resn_exch{"1H12 STE"}="H121 STE"; $resn_exch{"2H12 STE"}="H122 STE";
$resn_exch{"1H13 STE"}="H131 STE"; $resn_exch{"2H13 STE"}="H132 STE"; $resn_exch{"1H14 STE"}="H141 STE";
$resn_exch{"2H14 STE"}="H142 STE"; $resn_exch{"1H15 STE"}="H151 STE"; $resn_exch{"2H15 STE"}="H152 STE";
$resn_exch{"1H16 STE"}="H161 STE"; $resn_exch{"2H16 STE"}="H162 STE"; $resn_exch{"1H17 STE"}="H171 STE";
$resn_exch{"2H17 STE"}="H172 STE"; $resn_exch{"1H18 STE"}="H181 STE"; $resn_exch{"2H18 STE"}="H182 STE";
$resn_exch{"3H18 STE"}="H183 STE"; $resn_exch{"7H   STF"}=" H7  STF"; $resn_exch{"9H1  STF"}=" H19 STF";
$resn_exch{"6H   STF"}=" H6  STF"; $resn_exch{"5H   STF"}=" H5  STF"; $resn_exch{"4H   STF"}=" H4  STF";
$resn_exch{"3H   STF"}=" H3  STF"; $resn_exch{"2H   STF"}=" H2  STF"; $resn_exch{"1H   STF"}=" H1  STF";
$resn_exch{"1H6  STG"}=" H61 STG"; $resn_exch{"2H6  STG"}=" H62 STG"; $resn_exch{"1H7  STG"}=" H71 STG";
$resn_exch{"2H7  STG"}=" H72 STG"; $resn_exch{"1H11 STG"}="H111 STG"; $resn_exch{"2H11 STG"}="H112 STG";
$resn_exch{"1H12 STG"}="H121 STG"; $resn_exch{"2H12 STG"}="H122 STG"; $resn_exch{"1H15 STG"}="H151 STG";
$resn_exch{"2H15 STG"}="H152 STG"; $resn_exch{"1H18 STG"}="H181 STG"; $resn_exch{"2H18 STG"}="H182 STG";
$resn_exch{"3H18 STG"}="H183 STG"; $resn_exch{"1HN7 STH"}="HN71 STH"; $resn_exch{"2HN7 STH"}="HN72 STH";
$resn_exch{"1H1  STI"}=" H11 STI"; $resn_exch{"1H6  STI"}=" H61 STI"; $resn_exch{"1H4  STI"}=" H41 STI";
$resn_exch{"1H2  STI"}=" H21 STI"; $resn_exch{"1H12 STI"}="H121 STI"; $resn_exch{"1H11 STI"}="H111 STI";
$resn_exch{"1H13 STI"}="H131 STI"; $resn_exch{"1H18 STI"}="H181 STI"; $resn_exch{"1H17 STI"}="H171 STI";
$resn_exch{"1H15 STI"}="H151 STI"; $resn_exch{"1H21 STI"}="H211 STI"; $resn_exch{"1H25 STI"}="H251 STI";
$resn_exch{"1H26 STI"}="H261 STI"; $resn_exch{"1H28 STI"}="H281 STI"; $resn_exch{"1H29 STI"}="H291 STI";
$resn_exch{"1H46 STI"}="H461 STI"; $resn_exch{"2H46 STI"}="H462 STI"; $resn_exch{"1H53 STI"}="H531 STI";
$resn_exch{"2H53 STI"}="H532 STI"; $resn_exch{"1H52 STI"}="H521 STI"; $resn_exch{"2H52 STI"}="H522 STI";
$resn_exch{"1H54 STI"}="H541 STI"; $resn_exch{"2H54 STI"}="H542 STI"; $resn_exch{"3H54 STI"}="H543 STI";
$resn_exch{"1H50 STI"}="H501 STI"; $resn_exch{"2H50 STI"}="H502 STI"; $resn_exch{"1H49 STI"}="H491 STI";
$resn_exch{"2H49 STI"}="H492 STI"; $resn_exch{"1H20 STI"}="H201 STI"; $resn_exch{"2H20 STI"}="H202 STI";
$resn_exch{"3H20 STI"}="H203 STI"; $resn_exch{"1HD1 STN"}="HD11 STN"; $resn_exch{"2HD1 STN"}="HD12 STN";
$resn_exch{"3HD1 STN"}="HD13 STN"; $resn_exch{"1HCN STN"}="HCN1 STN"; $resn_exch{"2HCN STN"}="HCN2 STN";
$resn_exch{"1HCB STN"}="HCB1 STN"; $resn_exch{"2HCB STN"}="HCB2 STN"; $resn_exch{"1HD2 STN"}="HD21 STN";
$resn_exch{"2HD2 STN"}="HD22 STN"; $resn_exch{"3HD2 STN"}="HD23 STN"; $resn_exch{"1H20 STO"}="H201 STO";
$resn_exch{"2H20 STO"}="H202 STO"; $resn_exch{"1H27 STO"}="H271 STO"; $resn_exch{"2H27 STO"}="H272 STO";
$resn_exch{"1H32 STO"}="H321 STO"; $resn_exch{"2H32 STO"}="H322 STO"; $resn_exch{"3H32 STO"}="H323 STO";
$resn_exch{"1H34 STO"}="H341 STO"; $resn_exch{"2H34 STO"}="H342 STO"; $resn_exch{"3H34 STO"}="H343 STO";
$resn_exch{"1H35 STO"}="H351 STO"; $resn_exch{"2H35 STO"}="H352 STO"; $resn_exch{"3H35 STO"}="H353 STO";
$resn_exch{"2HOG STP"}="HOG2 STP"; $resn_exch{"3HOG STP"}="HOG3 STP"; $resn_exch{"2HOB STP"}="HOB2 STP";
$resn_exch{"2HOA STP"}="HOA2 STP"; $resn_exch{"1H5* STP"}="H5'1 STP"; $resn_exch{"2H5* STP"}="H5'2 STP";
$resn_exch{" H4* STP"}=" H4' STP"; $resn_exch{" H3* STP"}=" H3' STP"; $resn_exch{"*HO3 STP"}="HO3' STP";
$resn_exch{"1H2* STP"}="H2'1 STP"; $resn_exch{"2H2* STP"}="H2'2 STP"; $resn_exch{" H1* STP"}=" H1' STP";
$resn_exch{"1HN6 STP"}="HN61 STP"; $resn_exch{"2HN6 STP"}="HN62 STP"; $resn_exch{"1H1  STR"}=" H11 STR";
$resn_exch{"2H1  STR"}=" H12 STR"; $resn_exch{"1H2  STR"}=" H21 STR"; $resn_exch{"2H2  STR"}=" H22 STR";
$resn_exch{"1H6  STR"}=" H61 STR"; $resn_exch{"2H6  STR"}=" H62 STR"; $resn_exch{"1H7  STR"}=" H71 STR";
$resn_exch{"2H7  STR"}=" H72 STR"; $resn_exch{"1H11 STR"}="H111 STR"; $resn_exch{"2H11 STR"}="H112 STR";
$resn_exch{"1H12 STR"}="H121 STR"; $resn_exch{"2H12 STR"}="H122 STR"; $resn_exch{"1H15 STR"}="H151 STR";
$resn_exch{"2H15 STR"}="H152 STR"; $resn_exch{"1H16 STR"}="H161 STR"; $resn_exch{"2H16 STR"}="H162 STR";
$resn_exch{"1H18 STR"}="H181 STR"; $resn_exch{"2H18 STR"}="H182 STR"; $resn_exch{"3H18 STR"}="H183 STR";
$resn_exch{"1H19 STR"}="H191 STR"; $resn_exch{"2H19 STR"}="H192 STR"; $resn_exch{"3H19 STR"}="H193 STR";
$resn_exch{"1H21 STR"}="H211 STR"; $resn_exch{"2H21 STR"}="H212 STR"; $resn_exch{"3H21 STR"}="H213 STR";
$resn_exch{"1H2  STS"}=" H21 STS"; $resn_exch{"2H2  STS"}=" H22 STS"; $resn_exch{"1H3  STS"}=" H31 STS";
$resn_exch{"2H3  STS"}=" H32 STS"; $resn_exch{"1H24 STU"}="H241 STU"; $resn_exch{"2H24 STU"}="H242 STU";
$resn_exch{"1H26 STU"}="H261 STU"; $resn_exch{"2H26 STU"}="H262 STU"; $resn_exch{"3H26 STU"}="H263 STU";
$resn_exch{"1H9  STU"}=" H91 STU"; $resn_exch{"2H9  STU"}=" H92 STU"; $resn_exch{"1H27 STU"}="H271 STU";
$resn_exch{"2H27 STU"}="H272 STU"; $resn_exch{"3H27 STU"}="H273 STU"; $resn_exch{"1H28 STU"}="H281 STU";
$resn_exch{"2H28 STU"}="H282 STU"; $resn_exch{"3H28 STU"}="H283 STU"; $resn_exch{"1HA  STX"}=" HA1 STX";
$resn_exch{"2HA  STX"}=" HA2 STX"; $resn_exch{"1HD  STX"}=" HD1 STX"; $resn_exch{"2HD  STX"}=" HD2 STX";
$resn_exch{"3HD  STX"}=" HD3 STX"; $resn_exch{"1H7  STX"}=" H71 STX"; $resn_exch{"2H7  STX"}=" H72 STX";
$resn_exch{"3H7  STX"}=" H73 STX"; $resn_exch{"1H8  STX"}=" H81 STX"; $resn_exch{"2H8  STX"}=" H82 STX";
$resn_exch{"3H8  STX"}=" H83 STX"; $resn_exch{"1H9  STX"}=" H91 STX"; $resn_exch{"2H9  STX"}=" H92 STX";
$resn_exch{"3H9  STX"}=" H93 STX"; $resn_exch{"1HB  STY"}=" HB1 STY"; $resn_exch{"2HB  STY"}=" HB2 STY";
$resn_exch{"1H10 SU1"}="H101 SU1"; $resn_exch{"'HN1 SU1"}="HN1' SU1"; $resn_exch{"1H5' SU1"}="H5'1 SU1";
$resn_exch{"1H11 SU1"}="H111 SU1"; $resn_exch{"2H11 SU1"}="H112 SU1"; $resn_exch{"3H11 SU1"}="H113 SU1";
$resn_exch{"1H12 SU1"}="H121 SU1"; $resn_exch{"2H12 SU1"}="H122 SU1"; $resn_exch{"1H13 SU1"}="H131 SU1";
$resn_exch{"2H13 SU1"}="H132 SU1"; $resn_exch{"1H10 SU2"}="H101 SU2"; $resn_exch{"2H10 SU2"}="H102 SU2";
$resn_exch{"1H2* SU2"}="H2'1 SU2"; $resn_exch{"2H2* SU2"}="H2'2 SU2"; $resn_exch{"1H3* SU2"}="H3'1 SU2";
$resn_exch{"2H3* SU2"}="H3'2 SU2"; $resn_exch{"1H5* SU2"}="H5'1 SU2"; $resn_exch{"2H5* SU2"}="H5'2 SU2";
$resn_exch{"1H6* SU2"}="H6'1 SU2"; $resn_exch{"2H6* SU2"}="H6'2 SU2"; $resn_exch{"1H12 SU9"}="H121 SU9";
$resn_exch{"2H12 SU9"}="H122 SU9"; $resn_exch{"3H12 SU9"}="H123 SU9"; $resn_exch{"1HN1 SUA"}="HN11 SUA";
$resn_exch{"2HN1 SUA"}="HN12 SUA"; $resn_exch{"1H1  SUA"}=" H11 SUA"; $resn_exch{"1H2  SUA"}=" H21 SUA";
$resn_exch{"1H4  SUA"}=" H41 SUA"; $resn_exch{"1H5  SUA"}=" H51 SUA"; $resn_exch{"1HN2 SUA"}="HN21 SUA";
$resn_exch{"1H8  SUA"}=" H81 SUA"; $resn_exch{"2H8  SUA"}=" H82 SUA"; $resn_exch{"1H9  SUA"}=" H91 SUA";
$resn_exch{"2H9  SUA"}=" H92 SUA"; $resn_exch{"1H11 SUA"}="H111 SUA"; $resn_exch{"1H12 SUA"}="H121 SUA";
$resn_exch{"1H13 SUA"}="H131 SUA"; $resn_exch{"1H3  SUB"}=" H31 SUB"; $resn_exch{"2H3  SUB"}=" H32 SUB";
$resn_exch{"1H4  SUB"}=" H41 SUB"; $resn_exch{"2H4  SUB"}=" H42 SUB"; $resn_exch{"1H12 SUB"}="H121 SUB";
$resn_exch{"2H12 SUB"}="H122 SUB"; $resn_exch{"3H12 SUB"}="H123 SUB"; $resn_exch{"1H13 SUB"}="H131 SUB";
$resn_exch{"2H13 SUB"}="H132 SUB"; $resn_exch{"1H14 SUB"}="H141 SUB"; $resn_exch{"2H14 SUB"}="H142 SUB";
$resn_exch{"1H15 SUB"}="H151 SUB"; $resn_exch{"2H15 SUB"}="H152 SUB"; $resn_exch{"1H23 SUB"}="H231 SUB";
$resn_exch{"2H23 SUB"}="H232 SUB"; $resn_exch{"3H23 SUB"}="H233 SUB"; $resn_exch{"1H6  SUC"}=" H61 SUC";
$resn_exch{"2H6  SUC"}=" H62 SUC"; $resn_exch{"1H1' SUC"}="H1'1 SUC"; $resn_exch{"2H1' SUC"}="H1'2 SUC";
$resn_exch{"1H6' SUC"}="H6'1 SUC"; $resn_exch{"2H6' SUC"}="H6'2 SUC"; $resn_exch{"'HO1 SUC"}="HO1' SUC";
$resn_exch{"'HO3 SUC"}="HO3' SUC"; $resn_exch{"'HO4 SUC"}="HO4' SUC"; $resn_exch{"'HO6 SUC"}="HO6' SUC";
$resn_exch{"1H1  SUD"}=" H11 SUD"; $resn_exch{"2H1  SUD"}=" H12 SUD"; $resn_exch{"1HN4 SUD"}="HN41 SUD";
$resn_exch{"2HN4 SUD"}="HN42 SUD"; $resn_exch{" H1* SUD"}=" H1' SUD"; $resn_exch{" H2* SUD"}=" H2' SUD";
$resn_exch{" H3* SUD"}=" H3' SUD"; $resn_exch{" H4* SUD"}=" H4' SUD"; $resn_exch{"1H5* SUD"}="H5'1 SUD";
$resn_exch{"2H5* SUD"}="H5'2 SUD"; $resn_exch{"1H29 SUD"}="H291 SUD"; $resn_exch{"2H29 SUD"}="H292 SUD";
$resn_exch{"1H32 SUD"}="H321 SUD"; $resn_exch{"2H32 SUD"}="H322 SUD"; $resn_exch{"3H32 SUD"}="H323 SUD";
$resn_exch{"1H2  SUF"}=" H21 SUF"; $resn_exch{"2H2  SUF"}=" H22 SUF"; $resn_exch{"1H3  SUF"}=" H31 SUF";
$resn_exch{"2H3  SUF"}=" H32 SUF"; $resn_exch{"3H3  SUF"}=" H33 SUF"; $resn_exch{"1H4  SUF"}=" H41 SUF";
$resn_exch{"2H4  SUF"}=" H42 SUF"; $resn_exch{"3H4  SUF"}=" H43 SUF"; $resn_exch{"1HB  SUG"}=" HB1 SUG";
$resn_exch{"2HB  SUG"}=" HB2 SUG"; $resn_exch{"1HG  SUG"}=" HG1 SUG"; $resn_exch{"2HG  SUG"}=" HG2 SUG";
$resn_exch{"1HD  SUG"}=" HD1 SUG"; $resn_exch{"2HD  SUG"}="HD2A SUG"; $resn_exch{"1HH2 SUG"}="HH21 SUG";
$resn_exch{"2HH2 SUG"}="HH22 SUG"; $resn_exch{"1HW  SUG"}=" HW1 SUG"; $resn_exch{"2HW  SUG"}=" HW2 SUG";
$resn_exch{"1HX  SUG"}=" HX1 SUG"; $resn_exch{"2HX  SUG"}=" HX2 SUG"; $resn_exch{"1HN  SUJ"}=" HN1 SUJ";
$resn_exch{"2HN  SUJ"}=" HN2 SUJ"; $resn_exch{"1HBC SUJ"}="HBC1 SUJ"; $resn_exch{"2HBC SUJ"}="HBC2 SUJ";
$resn_exch{"1HGC SUJ"}="HGC1 SUJ"; $resn_exch{"2HGC SUJ"}="HGC2 SUJ"; $resn_exch{"1H4C SUJ"}="H4C1 SUJ";
$resn_exch{"2H4C SUJ"}="H4C2 SUJ"; $resn_exch{"3H4C SUJ"}="H4C3 SUJ"; $resn_exch{"1H6  SUM"}=" H61 SUM";
$resn_exch{"2H6  SUM"}=" H62 SUM"; $resn_exch{"1H11 SUM"}="H111 SUM"; $resn_exch{"2H11 SUM"}="H112 SUM";
$resn_exch{"1H22 SUM"}="H221 SUM"; $resn_exch{"2H22 SUM"}="H222 SUM"; $resn_exch{"1H23 SUM"}="H231 SUM";
$resn_exch{"2H23 SUM"}="H232 SUM"; $resn_exch{"1H24 SUM"}="H241 SUM"; $resn_exch{"2H24 SUM"}="H242 SUM";
$resn_exch{"1H25 SUM"}="H251 SUM"; $resn_exch{"2H25 SUM"}="H252 SUM"; $resn_exch{"1H26 SUM"}="H261 SUM";
$resn_exch{"2H26 SUM"}="H262 SUM"; $resn_exch{"1H27 SUM"}="H271 SUM"; $resn_exch{"2H27 SUM"}="H272 SUM";
$resn_exch{"1H28 SUM"}="H281 SUM"; $resn_exch{"2H28 SUM"}="H282 SUM"; $resn_exch{"3H28 SUM"}="H283 SUM";
$resn_exch{"1HCM SUM"}="HCM1 SUM"; $resn_exch{"2HCM SUM"}="HCM2 SUM"; $resn_exch{"1HN  SUN"}=" HN1 SUN";
$resn_exch{"2HN  SUN"}=" HN2 SUN"; $resn_exch{"1HBC SUN"}="HBC1 SUN"; $resn_exch{"2HBC SUN"}="HBC2 SUN";
$resn_exch{"1H2C SUN"}="H2C1 SUN"; $resn_exch{"2H2C SUN"}="H2C2 SUN"; $resn_exch{"3H2C SUN"}="H2C3 SUN";
$resn_exch{"1H1C SUN"}="H1C1 SUN"; $resn_exch{"2H1C SUN"}="H1C2 SUN"; $resn_exch{"3H1C SUN"}="H1C3 SUN";
$resn_exch{"1H3C SUN"}="H3C1 SUN"; $resn_exch{"2H3C SUN"}="H3C2 SUN"; $resn_exch{"1H4C SUN"}="H4C1 SUN";
$resn_exch{"2H4C SUN"}="H4C2 SUN"; $resn_exch{"3H4C SUN"}="H4C3 SUN"; $resn_exch{"1HB  SUO"}=" HB1 SUO";
$resn_exch{"2HB  SUO"}=" HB2 SUO"; $resn_exch{"1HG  SUO"}=" HG1 SUO"; $resn_exch{"2HG  SUO"}=" HG2 SUO";
$resn_exch{"1HD  SUO"}="HD1A SUO"; $resn_exch{"2HD  SUO"}=" HD2 SUO"; $resn_exch{"1HNE SUO"}="HNE1 SUO";
$resn_exch{"2HNE SUO"}="HNE2 SUO"; $resn_exch{"1HW  SUO"}=" HW1 SUO"; $resn_exch{"2HW  SUO"}=" HW2 SUO";
$resn_exch{"1HX  SUO"}=" HX1 SUO"; $resn_exch{"2HX  SUO"}=" HX2 SUO"; $resn_exch{"1H6  SUP"}=" H61 SUP";
$resn_exch{"2H6  SUP"}=" H62 SUP"; $resn_exch{"1H1' SUP"}="H1'1 SUP"; $resn_exch{"2H1' SUP"}="H1'2 SUP";
$resn_exch{"1H6' SUP"}="H6'1 SUP"; $resn_exch{"2H6' SUP"}="H6'2 SUP"; $resn_exch{"1HOP SUR"}="HOP1 SUR";
$resn_exch{"3HOP SUR"}="HOP3 SUR"; $resn_exch{"1H5' SUR"}="H5'1 SUR"; $resn_exch{"2H5' SUR"}="H5'2 SUR";
$resn_exch{"1HO2 SUW"}="HO21 SUW"; $resn_exch{"2HO2 SUW"}="HO22 SUW"; $resn_exch{"3HO2 SUW"}="HO23 SUW";
$resn_exch{"1HX3 SUW"}="HX31 SUW"; $resn_exch{"2HX3 SUW"}="HX32 SUW"; $resn_exch{"1H11 SUW"}="H111 SUW";
$resn_exch{"2H11 SUW"}="H112 SUW"; $resn_exch{"3H11 SUW"}="H113 SUW"; $resn_exch{"1HX9 SUW"}="HX91 SUW";
$resn_exch{"2HX9 SUW"}="HX92 SUW"; $resn_exch{" HN2 SVA"}=" H2  SVA"; $resn_exch{"1HB  SVA"}=" HB2 SVA";
$resn_exch{"2HB  SVA"}=" HB3 SVA"; $resn_exch{"1H27 SVR"}="H271 SVR"; $resn_exch{"2H27 SVR"}="H272 SVR";
$resn_exch{"3H27 SVR"}="H273 SVR"; $resn_exch{"1H59 SVR"}="H591 SVR"; $resn_exch{"2H59 SVR"}="H592 SVR";
$resn_exch{"3H59 SVR"}="H593 SVR"; $resn_exch{"1HBC SVV"}="HBC1 SVV"; $resn_exch{"2HA  SVV"}=" HA2 SVV";
$resn_exch{"1H4N SVV"}="H4N1 SVV"; $resn_exch{"2H4N SVV"}="H4N2 SVV"; $resn_exch{"1HBC SVW"}="HBC1 SVW";
$resn_exch{"2HBC SVW"}="HBC2 SVW"; $resn_exch{"1H2N SVW"}="H2N1 SVW"; $resn_exch{"2H2N SVW"}="H2N2 SVW";
$resn_exch{"1HN  SVX"}=" HN1 SVX"; $resn_exch{"2HN  SVX"}=" HN2 SVX"; $resn_exch{"1HBC SVX"}="HBC1 SVX";
$resn_exch{"2HBC SVX"}="HBC2 SVX"; $resn_exch{"1H4C SVX"}="H4C1 SVX"; $resn_exch{"2H4C SVX"}="H4C2 SVX";
$resn_exch{"3H4C SVX"}="H4C3 SVX"; $resn_exch{"1H2C SVX"}="H2C1 SVX"; $resn_exch{"2H2C SVX"}="H2C2 SVX";
$resn_exch{"1H1C SVX"}="H1C1 SVX"; $resn_exch{"2H1C SVX"}="H1C2 SVX"; $resn_exch{"3H1C SVX"}="H1C3 SVX";
$resn_exch{"1HN  SVY"}=" HN1 SVY"; $resn_exch{"2HN  SVY"}=" HN2 SVY"; $resn_exch{"1HBC SVY"}="HBC1 SVY";
$resn_exch{"2HBC SVY"}="HBC2 SVY"; $resn_exch{"1H2C SVY"}="H2C1 SVY"; $resn_exch{"2H2C SVY"}="H2C2 SVY";
$resn_exch{"3H2C SVY"}="H2C3 SVY"; $resn_exch{"1H3C SVY"}="H3C1 SVY"; $resn_exch{"2H3C SVY"}="H3C2 SVY";
$resn_exch{"3H3C SVY"}="H3C3 SVY"; $resn_exch{"1H2' SVY"}="H2'1 SVY"; $resn_exch{"2H2' SVY"}="H2'2 SVY";
$resn_exch{"3H2' SVY"}="H2'3 SVY"; $resn_exch{"1H3' SVY"}="H3'1 SVY"; $resn_exch{"2H3' SVY"}="H3'2 SVY";
$resn_exch{"3H3' SVY"}="H3'3 SVY"; $resn_exch{"1HN  SVZ"}=" HN1 SVZ"; $resn_exch{"2HN  SVZ"}=" HN2 SVZ";
$resn_exch{"1HBC SVZ"}="HBC1 SVZ"; $resn_exch{"2HBC SVZ"}="HBC2 SVZ"; $resn_exch{"1H7C SVZ"}="H7C1 SVZ";
$resn_exch{"2H7C SVZ"}="H7C2 SVZ"; $resn_exch{"3H7C SVZ"}="H7C3 SVZ"; $resn_exch{"1H6C SVZ"}="H6C1 SVZ";
$resn_exch{"2H6C SVZ"}="H6C2 SVZ"; $resn_exch{"3H6C SVZ"}="H6C3 SVZ"; $resn_exch{"1HC5 SWA"}="HC51 SWA";
$resn_exch{"2HC5 SWA"}="HC52 SWA"; $resn_exch{"1HC6 SWA"}="HC61 SWA"; $resn_exch{"2HC6 SWA"}="HC62 SWA";
$resn_exch{"1HC2 SWA"}="HC21 SWA"; $resn_exch{"2HC2 SWA"}="HC22 SWA"; $resn_exch{"1HC9 SWA"}="HC91 SWA";
$resn_exch{"2HC9 SWA"}="HC92 SWA"; $resn_exch{"OH13 SWA"}="H13O SWA"; $resn_exch{"OH11 SWA"}="H11O SWA";
$resn_exch{"1H1C SWF"}="H1C1 SWF"; $resn_exch{"2H1C SWF"}="H1C2 SWF"; $resn_exch{"3H1C SWF"}="H1C3 SWF";
$resn_exch{"1H4C SWF"}="H4C1 SWF"; $resn_exch{"2H4C SWF"}="H4C2 SWF"; $resn_exch{"1H25 SWI"}="H251 SWI";
$resn_exch{"2H25 SWI"}="H252 SWI"; $resn_exch{"1H26 SWI"}="H261 SWI"; $resn_exch{"2H26 SWI"}="H262 SWI";
$resn_exch{"1H32 SWI"}="H321 SWI"; $resn_exch{"2H32 SWI"}="H322 SWI"; $resn_exch{"3H32 SWI"}="H323 SWI";
$resn_exch{"1H30 SWI"}="H301 SWI"; $resn_exch{"2H30 SWI"}="H302 SWI"; $resn_exch{"1H39 SWI"}="H391 SWI";
$resn_exch{"2H39 SWI"}="H392 SWI"; $resn_exch{"3H39 SWI"}="H393 SWI"; $resn_exch{"1H28 SWI"}="H281 SWI";
$resn_exch{"2H28 SWI"}="H282 SWI"; $resn_exch{"1H37 SWI"}="H371 SWI"; $resn_exch{"2H37 SWI"}="H372 SWI";
$resn_exch{"3H37 SWI"}="H373 SWI"; $resn_exch{"1H36 SWI"}="H361 SWI"; $resn_exch{"2H36 SWI"}="H362 SWI";
$resn_exch{"3H36 SWI"}="H363 SWI"; $resn_exch{"1H35 SWI"}="H351 SWI"; $resn_exch{"2H35 SWI"}="H352 SWI";
$resn_exch{"3H35 SWI"}="H353 SWI"; $resn_exch{"1H18 SWI"}="H181 SWI"; $resn_exch{"2H18 SWI"}="H182 SWI";
$resn_exch{"1H34 SWI"}="H341 SWI"; $resn_exch{"2H34 SWI"}="H342 SWI"; $resn_exch{"3H34 SWI"}="H343 SWI";
$resn_exch{"1H38 SWI"}="H381 SWI"; $resn_exch{"2H38 SWI"}="H382 SWI"; $resn_exch{"3H38 SWI"}="H383 SWI";
$resn_exch{"1H14 SWI"}="H141 SWI"; $resn_exch{"2H14 SWI"}="H142 SWI"; $resn_exch{"1H12 SWI"}="H121 SWI";
$resn_exch{"2H12 SWI"}="H122 SWI"; $resn_exch{"1H8  SWI"}=" H81 SWI"; $resn_exch{"2H8  SWI"}=" H82 SWI";
$resn_exch{"1H6  SWI"}="H61A SWI"; $resn_exch{"2H6  SWI"}="H62A SWI"; $resn_exch{"1H33 SWI"}="H331 SWI";
$resn_exch{"2H33 SWI"}="H332 SWI"; $resn_exch{"3H33 SWI"}="H333 SWI"; $resn_exch{"1H72 SWI"}="H721 SWI";
$resn_exch{"2H72 SWI"}="H722 SWI"; $resn_exch{"3H72 SWI"}="H723 SWI"; $resn_exch{"1H45 SWI"}="H451 SWI";
$resn_exch{"2H45 SWI"}="H452 SWI"; $resn_exch{"2HO1 SWI"}="HO12 SWI"; $resn_exch{"1H47 SWI"}="H471 SWI";
$resn_exch{"2H47 SWI"}="H472 SWI"; $resn_exch{"1H51 SWI"}="H511 SWI"; $resn_exch{"2H51 SWI"}="H512 SWI";
$resn_exch{"1H53 SWI"}="H531 SWI"; $resn_exch{"2H53 SWI"}="H532 SWI"; $resn_exch{"1H77 SWI"}="H771 SWI";
$resn_exch{"2H77 SWI"}="H772 SWI"; $resn_exch{"3H77 SWI"}="H773 SWI"; $resn_exch{"1H73 SWI"}="H731 SWI";
$resn_exch{"2H73 SWI"}="H732 SWI"; $resn_exch{"3H73 SWI"}="H733 SWI"; $resn_exch{"5HO1 SWI"}="HO15 SWI";
$resn_exch{"1H57 SWI"}="H571 SWI"; $resn_exch{"2H57 SWI"}="H572 SWI"; $resn_exch{"6HO1 SWI"}="HO16 SWI";
$resn_exch{"1H74 SWI"}="H741 SWI"; $resn_exch{"2H74 SWI"}="H742 SWI"; $resn_exch{"3H74 SWI"}="H743 SWI";
$resn_exch{"1H75 SWI"}="H751 SWI"; $resn_exch{"2H75 SWI"}="H752 SWI"; $resn_exch{"3H75 SWI"}="H753 SWI";
$resn_exch{"8HO1 SWI"}="HO18 SWI"; $resn_exch{"1H76 SWI"}="H761 SWI"; $resn_exch{"2H76 SWI"}="H762 SWI";
$resn_exch{"3H76 SWI"}="H763 SWI"; $resn_exch{"1H64 SWI"}="H641 SWI"; $resn_exch{"2H64 SWI"}="H642 SWI";
$resn_exch{"1H65 SWI"}="H651 SWI"; $resn_exch{"2H65 SWI"}="H652 SWI"; $resn_exch{"1H71 SWI"}="H711 SWI";
$resn_exch{"2H71 SWI"}="H712 SWI"; $resn_exch{"3H71 SWI"}="H713 SWI"; $resn_exch{"1H69 SWI"}="H691 SWI";
$resn_exch{"2H69 SWI"}="H692 SWI"; $resn_exch{"1H67 SWI"}="H671 SWI"; $resn_exch{"2H67 SWI"}="H672 SWI";
$resn_exch{"1H78 SWI"}="H781 SWI"; $resn_exch{"2H78 SWI"}="H782 SWI"; $resn_exch{"3H78 SWI"}="H783 SWI";
$resn_exch{"1HN  SXE"}=" HN1 SXE"; $resn_exch{"2HN  SXE"}=" HN2 SXE"; $resn_exch{"1HBC SXE"}="HBC1 SXE";
$resn_exch{"2HBC SXE"}="HBC2 SXE"; $resn_exch{"1H3C SXE"}="H3C1 SXE"; $resn_exch{"2H3C SXE"}="H3C2 SXE";
$resn_exch{"3H3C SXE"}="H3C3 SXE"; $resn_exch{"1H2C SXE"}="H2C1 SXE"; $resn_exch{"2H2C SXE"}="H2C2 SXE";
$resn_exch{"3H2C SXE"}="H2C3 SXE"; $resn_exch{"1H4C SXE"}="H4C1 SXE"; $resn_exch{"2H4C SXE"}="H4C2 SXE";
$resn_exch{"1H5C SXE"}="H5C1 SXE"; $resn_exch{"2H5C SXE"}="H5C2 SXE"; $resn_exch{"3H5C SXE"}="H5C3 SXE";
$resn_exch{"1H15 SXX"}="H151 SXX"; $resn_exch{"2H15 SXX"}="H152 SXX"; $resn_exch{"3H15 SXX"}="H153 SXX";
$resn_exch{"1H14 SXX"}="H141 SXX"; $resn_exch{"2H14 SXX"}="H142 SXX"; $resn_exch{"3H14 SXX"}="H143 SXX";
$resn_exch{"1HN1 SY1"}="HN11 SY1"; $resn_exch{"2HN1 SY1"}="HN12 SY1"; $resn_exch{"1H4  SY1"}=" H41 SY1";
$resn_exch{"2H4  SY1"}=" H42 SY1"; $resn_exch{"1H5  SY1"}=" H51 SY1"; $resn_exch{"2H5  SY1"}=" H52 SY1";
$resn_exch{"1H7  SY1"}=" H71 SY1"; $resn_exch{"2H7  SY1"}=" H72 SY1"; $resn_exch{"1H19 SY1"}="H191 SY1";
$resn_exch{"2H19 SY1"}="H192 SY1"; $resn_exch{"1H3  SY1"}=" H31 SY1"; $resn_exch{"2H3  SY1"}=" H32 SY1";
$resn_exch{"1H3  SYB"}="H31A SYB"; $resn_exch{"2H3  SYB"}="H32A SYB"; $resn_exch{"1H4  SYB"}=" H41 SYB";
$resn_exch{"2H4  SYB"}="H42A SYB"; $resn_exch{"3H4  SYB"}="H43A SYB"; $resn_exch{"1H7  SYB"}=" H71 SYB";
$resn_exch{"2H7  SYB"}=" H72 SYB"; $resn_exch{"1H8  SYB"}=" H81 SYB"; $resn_exch{"2H8  SYB"}=" H82 SYB";
$resn_exch{"1H9  SYB"}=" H91 SYB"; $resn_exch{"2H9  SYB"}=" H92 SYB"; $resn_exch{"1H12 SYB"}="H121 SYB";
$resn_exch{"2H12 SYB"}="H122 SYB"; $resn_exch{"3H12 SYB"}="H123 SYB"; $resn_exch{"1H13 SYB"}="H131 SYB";
$resn_exch{"2H13 SYB"}="H132 SYB"; $resn_exch{"1H20 SYB"}="H201 SYB"; $resn_exch{"2H20 SYB"}="H202 SYB";
$resn_exch{"3H20 SYB"}="H203 SYB"; $resn_exch{"1H21 SYB"}="H211 SYB"; $resn_exch{"2H21 SYB"}="H212 SYB";
$resn_exch{"3H21 SYB"}="H213 SYB"; $resn_exch{"1H24 SYB"}="H241 SYB"; $resn_exch{"2H24 SYB"}="H242 SYB";
$resn_exch{"1H27 SYB"}="H271 SYB"; $resn_exch{"2H27 SYB"}="H272 SYB"; $resn_exch{"1H39 SYB"}="H391 SYB";
$resn_exch{"2H39 SYB"}="H392 SYB"; $resn_exch{"3H39 SYB"}="H393 SYB"; $resn_exch{"1H81 SYB"}="H811 SYB";
$resn_exch{"2H81 SYB"}="H812 SYB"; $resn_exch{"1H91 SYB"}="H911 SYB"; $resn_exch{"2H91 SYB"}="H912 SYB";
$resn_exch{"1H92 SYB"}="H921 SYB"; $resn_exch{"2H92 SYB"}="H922 SYB"; $resn_exch{"1H94 SYB"}="H941 SYB";
$resn_exch{"2H94 SYB"}="H942 SYB"; $resn_exch{"1H95 SYB"}="H951 SYB"; $resn_exch{"2H95 SYB"}="H952 SYB";
$resn_exch{"1H96 SYB"}="H961 SYB"; $resn_exch{"2H96 SYB"}="H962 SYB"; $resn_exch{"1HB  SYM"}=" HB1 SYM";
$resn_exch{"2HB  SYM"}=" HB2 SYM"; $resn_exch{"1HG2 SYM"}="HG21 SYM"; $resn_exch{"2HG2 SYM"}="HG22 SYM";
$resn_exch{"3HG2 SYM"}="HG23 SYM"; $resn_exch{"1HN  SYM"}=" HN1 SYM"; $resn_exch{"2HN  SYM"}=" HN2 SYM";
$resn_exch{"1H10 SYR"}="H101 SYR"; $resn_exch{"2H10 SYR"}="H102 SYR"; $resn_exch{"3H10 SYR"}="H103 SYR";
$resn_exch{"1H7C SYR"}="H7C1 SYR"; $resn_exch{"2H7C SYR"}="H7C2 SYR"; $resn_exch{"3H7C SYR"}="H7C3 SYR";
$resn_exch{"1H9C SYR"}="H9C1 SYR"; $resn_exch{"2H9C SYR"}="H9C2 SYR"; $resn_exch{"3H9C SYR"}="H9C3 SYR";
$resn_exch{"3HOP   T"}="HOP3   T"; $resn_exch{"2HOP   T"}="HOP2   T"; $resn_exch{"1H5*   T"}=" H5'   T";
$resn_exch{"2H5*   T"}="H5''   T"; $resn_exch{" H4*   T"}=" H4'   T"; $resn_exch{" H3*   T"}=" H3'   T";
$resn_exch{"H3T    T"}="HO3'   T"; $resn_exch{"1H2*   T"}=" H2'   T"; $resn_exch{"2H2*   T"}="H2''   T";
$resn_exch{" H1*   T"}=" H1'   T"; $resn_exch{"1H5M   T"}=" H71   T"; $resn_exch{"2H5M   T"}=" H72   T";
$resn_exch{"3H5M   T"}=" H73   T"; $resn_exch{"1HC6 T10"}="HC61 T10"; $resn_exch{"2HC6 T10"}="HC62 T10";
$resn_exch{"1H10 T10"}="H101 T10"; $resn_exch{"2H10 T10"}="H102 T10"; $resn_exch{"1H21 T10"}="H211 T10";
$resn_exch{"2H21 T10"}="H212 T10"; $resn_exch{"1H32 T10"}="H321 T10"; $resn_exch{"2H32 T10"}="H322 T10";
$resn_exch{"1H7  T11"}=" H71 T11"; $resn_exch{"2H7  T11"}=" H72 T11"; $resn_exch{"1H35 T12"}="H351 T12";
$resn_exch{"2H35 T12"}="H352 T12"; $resn_exch{"3H35 T12"}="H353 T12"; $resn_exch{"1H31 T12"}="H311 T12";
$resn_exch{"2H31 T12"}="H312 T12"; $resn_exch{"3H31 T12"}="H313 T12"; $resn_exch{"1H27 T12"}="H271 T12";
$resn_exch{"2H27 T12"}="H272 T12"; $resn_exch{"3H27 T12"}="H273 T12"; $resn_exch{"1H39 T12"}="H391 T12";
$resn_exch{"2H39 T12"}="H392 T12"; $resn_exch{"1H40 T12"}="H401 T12"; $resn_exch{"2H40 T12"}="H402 T12";
$resn_exch{"1H43 T12"}="H431 T12"; $resn_exch{"2H43 T12"}="H432 T12"; $resn_exch{"1H2  T16"}=" H21 T16";
$resn_exch{"2H2  T16"}="H22A T16"; $resn_exch{"1H3  T16"}=" H31 T16"; $resn_exch{"2H3  T16"}=" H32 T16";
$resn_exch{"AHO1 T16"}="HO1A T16"; $resn_exch{"BHO1 T16"}="HO1B T16"; $resn_exch{"1H7A T16"}="H7A1 T16";
$resn_exch{"2H7A T16"}="H7A2 T16"; $resn_exch{"1H7B T16"}="H7B1 T16"; $resn_exch{"2H7B T16"}="H7B2 T16";
$resn_exch{"1H7C T16"}="H7C1 T16"; $resn_exch{"2H7C T16"}="H7C2 T16"; $resn_exch{"1H11 T16"}="H111 T16";
$resn_exch{"2H11 T16"}="H112 T16"; $resn_exch{"1H14 T16"}="H141 T16"; $resn_exch{"2H14 T16"}="H142 T16";
$resn_exch{"1H15 T16"}="H151 T16"; $resn_exch{"2H15 T16"}="H152 T16"; $resn_exch{"1H16 T16"}="H161 T16";
$resn_exch{"2H16 T16"}="H162 T16"; $resn_exch{"1H1  T19"}="H11A T19"; $resn_exch{"2H1  T19"}=" H12 T19";
$resn_exch{"3H1  T19"}=" H13 T19"; $resn_exch{"1H2  T19"}=" H21 T19"; $resn_exch{"2H2  T19"}="H22A T19";
$resn_exch{"3H2  T19"}="H23A T19"; $resn_exch{"1H15 T19"}="H151 T19"; $resn_exch{"2H15 T19"}="H152 T19";
$resn_exch{"1H1  T1A"}=" H11 T1A"; $resn_exch{"2H1  T1A"}=" H12 T1A"; $resn_exch{"1H2  T1A"}=" H21 T1A";
$resn_exch{"2H2  T1A"}=" H22 T1A"; $resn_exch{"3H2  T1A"}=" H23 T1A"; $resn_exch{"1H3  T1A"}=" H31 T1A";
$resn_exch{"2H3  T1A"}=" H32 T1A"; $resn_exch{"1H4  T1A"}=" H41 T1A"; $resn_exch{"2H4  T1A"}=" H42 T1A";
$resn_exch{"3H4  T1A"}=" H43 T1A"; $resn_exch{"1H5  T1A"}=" H51 T1A"; $resn_exch{"2H5  T1A"}=" H52 T1A";
$resn_exch{"1H6  T1A"}=" H61 T1A"; $resn_exch{"2H6  T1A"}=" H62 T1A"; $resn_exch{"3H6  T1A"}=" H63 T1A";
$resn_exch{"1H7  T1A"}=" H71 T1A"; $resn_exch{"2H7  T1A"}=" H72 T1A"; $resn_exch{"1H8  T1A"}=" H81 T1A";
$resn_exch{"2H8  T1A"}=" H82 T1A"; $resn_exch{"3H8  T1A"}=" H83 T1A"; $resn_exch{"1H1  T1D"}="H11A T1D";
$resn_exch{"1H8C T1D"}="H8C1 T1D"; $resn_exch{"2H8C T1D"}="H8C2 T1D"; $resn_exch{"3H8C T1D"}="H8C3 T1D";
$resn_exch{"1H17 T1P"}="H171 T1P"; $resn_exch{"2H17 T1P"}="H172 T1P"; $resn_exch{"1H16 T1P"}="H161 T1P";
$resn_exch{"2H16 T1P"}="H162 T1P"; $resn_exch{"1H15 T1P"}="H151 T1P"; $resn_exch{"2H15 T1P"}="H152 T1P";
$resn_exch{"1H10 T1P"}="H101 T1P"; $resn_exch{"2H10 T1P"}="H102 T1P"; $resn_exch{"1H14 T1P"}="H141 T1P";
$resn_exch{"2H14 T1P"}="H142 T1P"; $resn_exch{"2HOP T23"}="HOP2 T23"; $resn_exch{"3HOP T23"}="HOP3 T23";
$resn_exch{"1H5A T23"}="H5A1 T23"; $resn_exch{"2H5A T23"}="H5A2 T23"; $resn_exch{"3H5A T23"}="H5A3 T23";
$resn_exch{" H2* T23"}=" H2' T23"; $resn_exch{"1H2M T23"}="H2M1 T23"; $resn_exch{"2H2M T23"}="H2M2 T23";
$resn_exch{"3H2M T23"}="H2M3 T23"; $resn_exch{"1H5* T23"}=" H5' T23"; $resn_exch{"2H5* T23"}="H5'' T23";
$resn_exch{" H4* T23"}=" H4' T23"; $resn_exch{" H1* T23"}=" H1' T23"; $resn_exch{" H3* T23"}=" H3' T23";
$resn_exch{"1H3M T23"}="H3M1 T23"; $resn_exch{"2H3M T23"}="H3M2 T23"; $resn_exch{"3H3M T23"}="H3M3 T23";
$resn_exch{"1HN1 T29"}="HN11 T29"; $resn_exch{"2HN1 T29"}="HN12 T29"; $resn_exch{"1H3  T29"}=" H31 T29";
$resn_exch{"2H3  T29"}=" H32 T29"; $resn_exch{"1H4  T29"}=" H41 T29"; $resn_exch{"2H4  T29"}=" H42 T29";
$resn_exch{"1H5  T29"}=" H51 T29"; $resn_exch{"2H5  T29"}=" H52 T29"; $resn_exch{"AHO1 T29"}="HO1A T29";
$resn_exch{"BHO1 T29"}="HO1B T29"; $resn_exch{"1H9A T29"}="H9A1 T29"; $resn_exch{"2H9A T29"}="H9A2 T29";
$resn_exch{"1H9B T29"}="H9B1 T29"; $resn_exch{"2H9B T29"}="H9B2 T29"; $resn_exch{"1H9C T29"}="H9C1 T29";
$resn_exch{"2H9C T29"}="H9C2 T29"; $resn_exch{"1H13 T29"}="H131 T29"; $resn_exch{"2H13 T29"}="H132 T29";
$resn_exch{"1H16 T29"}="H161 T29"; $resn_exch{"2H16 T29"}="H162 T29"; $resn_exch{"1H17 T29"}="H171 T29";
$resn_exch{"2H17 T29"}="H172 T29"; $resn_exch{"3H17 T29"}="H173 T29"; $resn_exch{"1H11 T2D"}="H111 T2D";
$resn_exch{"2H11 T2D"}="H112 T2D"; $resn_exch{"1H1  T2M"}=" H11 T2M"; $resn_exch{"2H1  T2M"}=" H12 T2M";
$resn_exch{"3H1  T2M"}=" H13 T2M"; $resn_exch{"1H8  T2M"}=" H81 T2M"; $resn_exch{"2H8  T2M"}=" H82 T2M";
$resn_exch{"3H8  T2M"}=" H83 T2M"; $resn_exch{"1H9  T2M"}=" H91 T2M"; $resn_exch{"2H9  T2M"}=" H92 T2M";
$resn_exch{"3H9  T2M"}=" H93 T2M"; $resn_exch{"1H10 T2M"}="H101 T2M"; $resn_exch{"2H10 T2M"}="H102 T2M";
$resn_exch{"3H10 T2M"}="H103 T2M"; $resn_exch{"1H17 T2P"}="H171 T2P"; $resn_exch{"2H17 T2P"}="H172 T2P";
$resn_exch{"1H16 T2P"}="H161 T2P"; $resn_exch{"2H16 T2P"}="H162 T2P"; $resn_exch{"1H15 T2P"}="H151 T2P";
$resn_exch{"2H15 T2P"}="H152 T2P"; $resn_exch{"1H10 T2P"}="H101 T2P"; $resn_exch{"2H10 T2P"}="H102 T2P";
$resn_exch{"1H14 T2P"}="H141 T2P"; $resn_exch{"2H14 T2P"}="H142 T2P"; $resn_exch{"3HOP T2S"}="HOP3 T2S";
$resn_exch{"2HOP T2S"}="HOP2 T2S"; $resn_exch{"1H5* T2S"}=" H5' T2S"; $resn_exch{"2H5* T2S"}="H5'' T2S";
$resn_exch{" H4* T2S"}=" H4' T2S"; $resn_exch{" H3* T2S"}=" H3' T2S"; $resn_exch{"H3T  T2S"}="HO3' T2S";
$resn_exch{"1H2* T2S"}=" H2' T2S"; $resn_exch{"2H2* T2S"}="H2'' T2S"; $resn_exch{" H1* T2S"}=" H1' T2S";
$resn_exch{"1H5M T2S"}=" H71 T2S"; $resn_exch{"2H5M T2S"}=" H72 T2S"; $resn_exch{"3H5M T2S"}=" H73 T2S";
$resn_exch{"2HOP T2T"}="HOP2 T2T"; $resn_exch{"1H5* T2T"}=" H5' T2T"; $resn_exch{"2H5* T2T"}="H5'' T2T";
$resn_exch{"1HAB T2T"}="HAB1 T2T"; $resn_exch{"2HAB T2T"}="HAB2 T2T"; $resn_exch{"3HAB T2T"}="HAB3 T2T";
$resn_exch{"1H2X T2T"}="H2X1 T2T"; $resn_exch{"2H2X T2T"}="H2X2 T2T"; $resn_exch{"1HH2 T2T"}="HH21 T2T";
$resn_exch{"2HH2 T2T"}="HH22 T2T"; $resn_exch{"1H2P T2T"}="H2P1 T2T"; $resn_exch{"2H2P T2T"}="H2P2 T2T";
$resn_exch{"3H2P T2T"}="H2P3 T2T"; $resn_exch{"1H5X T2T"}="H5X1 T2T"; $resn_exch{"2H5X T2T"}="H5X2 T2T";
$resn_exch{" H4* T2T"}=" H4' T2T"; $resn_exch{" H1* T2T"}=" H1' T2T"; $resn_exch{"1H5A T2T"}="H5A1 T2T";
$resn_exch{"2H5A T2T"}="H5A2 T2T"; $resn_exch{"3H5A T2T"}="H5A3 T2T"; $resn_exch{" H2* T2T"}=" H2' T2T";
$resn_exch{"1HH3 T2T"}="HH31 T2T"; $resn_exch{"2HH3 T2T"}="HH32 T2T"; $resn_exch{"3HH3 T2T"}="HH33 T2T";
$resn_exch{" H3* T2T"}=" H3' T2T"; $resn_exch{"H3T  T2T"}="HO3' T2T"; $resn_exch{"3HOP T2T"}="HOP3 T2T";
$resn_exch{"1C13  T3"}="H131  T3"; $resn_exch{"2C13  T3"}="H132  T3"; $resn_exch{"1HN1  T3"}="HN11  T3";
$resn_exch{"2HN1  T3"}="HN12  T3"; $resn_exch{"1H5* T31"}="H5'1 T31"; $resn_exch{"2H5* T31"}="H5'2 T31";
$resn_exch{" H4* T31"}=" H4' T31"; $resn_exch{" H3* T31"}=" H3' T31"; $resn_exch{" H2* T31"}=" H2' T31";
$resn_exch{" H1* T31"}=" H1' T31"; $resn_exch{"1H5* T32"}="H5'1 T32"; $resn_exch{"2H5* T32"}="H5'2 T32";
$resn_exch{" H4* T32"}=" H4' T32"; $resn_exch{" H3* T32"}=" H3' T32"; $resn_exch{"1H2* T32"}="H2'1 T32";
$resn_exch{"2H2* T32"}="H2'2 T32"; $resn_exch{" H1* T32"}=" H1' T32"; $resn_exch{"1H5M T32"}="H5M1 T32";
$resn_exch{"2H5M T32"}="H5M2 T32"; $resn_exch{"3H5M T32"}="H5M3 T32"; $resn_exch{" H*  T32"}=" H'  T32";
$resn_exch{"1HM* T32"}="HM'1 T32"; $resn_exch{"2HM* T32"}="HM'2 T32"; $resn_exch{"3HM* T32"}="HM'3 T32";
$resn_exch{"1H7  T33"}=" H71 T33"; $resn_exch{"2H7  T33"}=" H72 T33"; $resn_exch{"1HN8 T33"}="HN81 T33";
$resn_exch{"2HN8 T33"}="HN82 T33"; $resn_exch{"4HO' T33"}="HO'4 T33"; $resn_exch{"1H5* T36"}="H5'1 T36";
$resn_exch{"2H5* T36"}="H5'2 T36"; $resn_exch{" H4* T36"}=" H4' T36"; $resn_exch{" H3* T36"}=" H3' T36";
$resn_exch{"1H2* T36"}="H2'1 T36"; $resn_exch{"2H2* T36"}="H2'2 T36"; $resn_exch{" H1* T36"}=" H1' T36";
$resn_exch{"1H5M T36"}="H5M1 T36"; $resn_exch{"2H5M T36"}="H5M2 T36"; $resn_exch{"3H5M T36"}="H5M3 T36";
$resn_exch{"1H5* T37"}="H5'1 T37"; $resn_exch{"2H5* T37"}="H5'2 T37"; $resn_exch{" H4* T37"}=" H4' T37";
$resn_exch{" H3* T37"}=" H3' T37"; $resn_exch{"1HN3 T37"}="HN31 T37"; $resn_exch{"2HN3 T37"}="HN32 T37";
$resn_exch{"1H2* T37"}="H2'1 T37"; $resn_exch{"2H2* T37"}="H2'2 T37"; $resn_exch{" H1* T37"}=" H1' T37";
$resn_exch{"1H5A T37"}="H5A1 T37"; $resn_exch{"2H5A T37"}="H5A2 T37"; $resn_exch{"3H5A T37"}="H5A3 T37";
$resn_exch{"1H5* T38"}="H5'1 T38"; $resn_exch{"2H5* T38"}="H5'2 T38"; $resn_exch{" H4* T38"}=" H4' T38";
$resn_exch{" H3* T38"}=" H3' T38"; $resn_exch{" H2* T38"}=" H2' T38"; $resn_exch{" H1* T38"}=" H1' T38";
$resn_exch{"1H5M T38"}="H5M1 T38"; $resn_exch{"2H5M T38"}="H5M2 T38"; $resn_exch{"3H5M T38"}="H5M3 T38";
$resn_exch{"1HA* T38"}="HA'1 T38"; $resn_exch{"2HA* T38"}="HA'2 T38"; $resn_exch{"1HB* T38"}="HB'1 T38";
$resn_exch{"2HB* T38"}="HB'2 T38"; $resn_exch{"1HD* T38"}="HD'1 T38"; $resn_exch{"2HD* T38"}="HD'2 T38";
$resn_exch{"3HD* T38"}="HD'3 T38"; $resn_exch{"1H5* T39"}="H5'1 T39"; $resn_exch{"2H5* T39"}="H5'2 T39";
$resn_exch{" H4* T39"}=" H4' T39"; $resn_exch{" H3* T39"}=" H3' T39"; $resn_exch{" H2* T39"}=" H2' T39";
$resn_exch{" H1* T39"}=" H1' T39"; $resn_exch{"1H5M T39"}="H5M1 T39"; $resn_exch{"2H5M T39"}="H5M2 T39";
$resn_exch{"3H5M T39"}="H5M3 T39"; $resn_exch{"1HA* T39"}="HA'1 T39"; $resn_exch{"2HA* T39"}="HA'2 T39";
$resn_exch{"1HD* T39"}="HD'1 T39"; $resn_exch{"2HD* T39"}="HD'2 T39"; $resn_exch{"3HD* T39"}="HD'3 T39";
$resn_exch{"1HB* T39"}="HB'1 T39"; $resn_exch{"2HB* T39"}="HB'2 T39"; $resn_exch{"1H1  T3A"}=" H11 T3A";
$resn_exch{"2H1  T3A"}=" H12 T3A"; $resn_exch{"1H2  T3A"}=" H21 T3A"; $resn_exch{"2H2  T3A"}=" H22 T3A";
$resn_exch{"1H3  T3A"}=" H31 T3A"; $resn_exch{"2H3  T3A"}=" H32 T3A"; $resn_exch{"1H5  T3A"}=" H51 T3A";
$resn_exch{"2H5  T3A"}=" H52 T3A"; $resn_exch{"1H6  T3A"}=" H61 T3A"; $resn_exch{"2H6  T3A"}=" H62 T3A";
$resn_exch{"1H7  T3A"}=" H71 T3A"; $resn_exch{"2H7  T3A"}=" H72 T3A"; $resn_exch{"1HAN T3O"}="HAN1 T3O";
$resn_exch{"2HAN T3O"}="HAN2 T3O"; $resn_exch{"1HAL T3O"}="HAL1 T3O"; $resn_exch{"2HAL T3O"}="HAL2 T3O";
$resn_exch{"3HAL T3O"}="HAL3 T3O"; $resn_exch{"1HAM T3O"}="HAM1 T3O"; $resn_exch{"2HAM T3O"}="HAM2 T3O";
$resn_exch{"1HAF T3O"}="HAF1 T3O"; $resn_exch{"2HAF T3O"}="HAF2 T3O"; $resn_exch{"3HAF T3O"}="HAF3 T3O";
$resn_exch{"1HAQ T3O"}="HAQ1 T3O"; $resn_exch{"2HAQ T3O"}="HAQ2 T3O"; $resn_exch{"3HAQ T3O"}="HAQ3 T3O";
$resn_exch{"2HOP T3P"}="HOP2 T3P"; $resn_exch{"3HOP T3P"}="HOP3 T3P"; $resn_exch{" H5* T3P"}=" H5' T3P";
$resn_exch{"1H5* T3P"}="H5'1 T3P"; $resn_exch{"2H5* T3P"}="H5'' T3P"; $resn_exch{" H4* T3P"}=" H4' T3P";
$resn_exch{" H3* T3P"}=" H3' T3P"; $resn_exch{"1H2* T3P"}=" H2' T3P"; $resn_exch{"2H2* T3P"}="H2'' T3P";
$resn_exch{" H1* T3P"}=" H1' T3P"; $resn_exch{"1H5  T3P"}=" H51 T3P"; $resn_exch{"2H5  T3P"}=" H52 T3P";
$resn_exch{"3H5  T3P"}=" H53 T3P"; $resn_exch{"1H5M T41"}="H5M1 T41"; $resn_exch{"2H5M T41"}="H5M2 T41";
$resn_exch{"3H5M T41"}="H5M3 T41"; $resn_exch{" H1* T41"}=" H1' T41"; $resn_exch{"1H2* T41"}="H2'1 T41";
$resn_exch{" H3* T41"}=" H3' T41"; $resn_exch{" H4* T41"}=" H4' T41"; $resn_exch{"1H5* T41"}="H5'1 T41";
$resn_exch{"2H5* T41"}="H5'2 T41"; $resn_exch{"1H1  T41"}=" H11 T41"; $resn_exch{"2H1  T41"}=" H12 T41";
$resn_exch{"3H1  T41"}=" H13 T41"; $resn_exch{"1H1  T42"}="H11A T42"; $resn_exch{"2H1  T42"}="H12A T42";
$resn_exch{"3H1  T42"}=" H13 T42"; $resn_exch{"1H2  T42"}=" H21 T42"; $resn_exch{"2H2  T42"}="H22A T42";
$resn_exch{"1H3  T42"}=" H31 T42"; $resn_exch{"2H3  T42"}="H32A T42"; $resn_exch{"1H4  T42"}=" H41 T42";
$resn_exch{"2H4  T42"}=" H42 T42"; $resn_exch{"1H42 T42"}="H421 T42"; $resn_exch{"2H42 T42"}="H422 T42";
$resn_exch{"1H43 T42"}="H431 T42"; $resn_exch{"2H43 T42"}="H432 T42"; $resn_exch{"1H45 T42"}="H451 T42";
$resn_exch{"2H45 T42"}="H452 T42"; $resn_exch{"1H46 T42"}="H461 T42"; $resn_exch{"2H46 T42"}="H462 T42";
$resn_exch{"1H7  T44"}=" H71 T44"; $resn_exch{"2H7  T44"}=" H72 T44"; $resn_exch{"1HN8 T44"}="HN81 T44";
$resn_exch{"2HN8 T44"}="HN82 T44"; $resn_exch{"'HO4 T44"}="HO4' T44"; $resn_exch{"1H4C T45"}="H4C1 T45";
$resn_exch{"2H4C T45"}="H4C2 T45"; $resn_exch{"3H4C T45"}="H4C3 T45"; $resn_exch{"1H5* T48"}="H5'1 T48";
$resn_exch{"2H5* T48"}="H5'2 T48"; $resn_exch{" H4* T48"}=" H4' T48"; $resn_exch{" H3* T48"}=" H3' T48";
$resn_exch{"1H2* T48"}="H2'1 T48"; $resn_exch{"2H2* T48"}="H2'2 T48"; $resn_exch{" H1* T48"}=" H1' T48";
$resn_exch{"1H5M T48"}="H5M1 T48"; $resn_exch{"2H5M T48"}="H5M2 T48"; $resn_exch{"3H5M T48"}="H5M3 T48";
$resn_exch{"1H5* T49"}="H5'1 T49"; $resn_exch{"2H5* T49"}="H5'2 T49"; $resn_exch{" H4* T49"}=" H4' T49";
$resn_exch{" H3* T49"}=" H3' T49"; $resn_exch{"1H2* T49"}="H2'1 T49"; $resn_exch{"2H2* T49"}="H2'2 T49";
$resn_exch{" H1* T49"}=" H1' T49"; $resn_exch{"1H5M T49"}="H5M1 T49"; $resn_exch{"2H5M T49"}="H5M2 T49";
$resn_exch{"3H5M T49"}="H5M3 T49"; $resn_exch{"1H2  T4A"}=" H21 T4A"; $resn_exch{"1H6  T4A"}=" H61 T4A";
$resn_exch{"1H7  T4A"}=" H71 T4A"; $resn_exch{"2H7  T4A"}=" H72 T4A"; $resn_exch{"1H6' T4A"}="H6'1 T4A";
$resn_exch{"1H2' T4A"}="H2'1 T4A"; $resn_exch{"1HO9 T4A"}="HO91 T4A"; $resn_exch{"OH4' T4A"}="H4'O T4A";
$resn_exch{"1H9  T4B"}=" H91 T4B"; $resn_exch{"2H9  T4B"}=" H92 T4B"; $resn_exch{"1H21 T4B"}="H211 T4B";
$resn_exch{"2H21 T4B"}="H212 T4B"; $resn_exch{"1H15 T4B"}="H151 T4B"; $resn_exch{"2H15 T4B"}="H152 T4B";
$resn_exch{"3H15 T4B"}="H153 T4B"; $resn_exch{"1H2A T4K"}="H2A1 T4K"; $resn_exch{"2H2A T4K"}="H2A2 T4K";
$resn_exch{"3H2A T4K"}="H2A3 T4K"; $resn_exch{"1H4A T4K"}="H4A1 T4K"; $resn_exch{"1H5B T4K"}="H5B1 T4K";
$resn_exch{"2H5B T4K"}="H5B2 T4K"; $resn_exch{"1H6G T4K"}="H6G1 T4K"; $resn_exch{"2H6G T4K"}="H6G2 T4K";
$resn_exch{"3H6G T4K"}="H6G3 T4K"; $resn_exch{"1H5  T4K"}=" H51 T4K"; $resn_exch{"2H5  T4K"}=" H52 T4K";
$resn_exch{"1H2  T4K"}=" H21 T4K"; $resn_exch{"2H2  T4K"}=" H22 T4K"; $resn_exch{"1H5A T4K"}="H5A1 T4K";
$resn_exch{"2H5A T4K"}="H5A2 T4K"; $resn_exch{"3H5A T4K"}="H5A3 T4K"; $resn_exch{"1H17 T4P"}="H171 T4P";
$resn_exch{"2H17 T4P"}="H172 T4P"; $resn_exch{"1H16 T4P"}="H161 T4P"; $resn_exch{"2H16 T4P"}="H162 T4P";
$resn_exch{"1H15 T4P"}="H151 T4P"; $resn_exch{"2H15 T4P"}="H152 T4P"; $resn_exch{"1H10 T4P"}="H101 T4P";
$resn_exch{"2H10 T4P"}="H102 T4P"; $resn_exch{"1H14 T4P"}="H141 T4P"; $resn_exch{"2H14 T4P"}="H142 T4P";
$resn_exch{"3HOP T4S"}="HOP3 T4S"; $resn_exch{"2HOP T4S"}="HOP2 T4S"; $resn_exch{"1H5A T4S"}="H5A1 T4S";
$resn_exch{"2H5A T4S"}="H5A2 T4S"; $resn_exch{"3H5A T4S"}="H5A3 T4S"; $resn_exch{"1H2* T4S"}=" H2' T4S";
$resn_exch{"2H2* T4S"}="H2'' T4S"; $resn_exch{"1H5* T4S"}=" H5' T4S"; $resn_exch{"2H5* T4S"}="H5'' T4S";
$resn_exch{" H4* T4S"}=" H4' T4S"; $resn_exch{" H1* T4S"}=" H1' T4S"; $resn_exch{" H3* T4S"}=" H3' T4S";
$resn_exch{"H3T  T4S"}="HO3' T4S"; $resn_exch{"1H4A T50"}="H4A1 T50"; $resn_exch{"2H4A T50"}="H4A2 T50";
$resn_exch{"3H4A T50"}="H4A3 T50"; $resn_exch{"1H10 T50"}="H101 T50"; $resn_exch{"2H10 T50"}="H102 T50";
$resn_exch{"1HB7 T5A"}="HB71 T5A"; $resn_exch{"2HB7 T5A"}="HB72 T5A"; $resn_exch{"3HB7 T5A"}="HB73 T5A";
$resn_exch{"1HB5 T5A"}="HB51 T5A"; $resn_exch{"2HB5 T5A"}="HB52 T5A"; $resn_exch{"1HB2 T5A"}="HB21 T5A";
$resn_exch{"2HB2 T5A"}="HB22 T5A"; $resn_exch{"3HB* T5A"}="HB'3 T5A"; $resn_exch{"AHO1 T5A"}="HO1A T5A";
$resn_exch{"BHO2 T5A"}="HO2B T5A"; $resn_exch{"CHO2 T5A"}="HO2C T5A"; $resn_exch{"DHO1 T5A"}="HO1D T5A";
$resn_exch{"EHO1 T5A"}="HO1E T5A"; $resn_exch{"1HA5 T5A"}="HA51 T5A"; $resn_exch{"2HA5 T5A"}="HA52 T5A";
$resn_exch{"1HA6 T5A"}="HA61 T5A"; $resn_exch{"2HA6 T5A"}="HA62 T5A"; $resn_exch{"1H13 T5C"}="H131 T5C";
$resn_exch{"2H13 T5C"}="H132 T5C"; $resn_exch{"1H12 T5C"}="H121 T5C"; $resn_exch{"2H12 T5C"}="H122 T5C";
$resn_exch{"1H11 T5C"}="H111 T5C"; $resn_exch{"2H11 T5C"}="H112 T5C"; $resn_exch{"3H11 T5C"}="H113 T5C";
$resn_exch{"1H10 T5C"}="H101 T5C"; $resn_exch{"2H10 T5C"}="H102 T5C"; $resn_exch{"3H10 T5C"}="H103 T5C";
$resn_exch{"1H34 T5C"}="H341 T5C"; $resn_exch{"2H34 T5C"}="H342 T5C"; $resn_exch{"3H34 T5C"}="H343 T5C";
$resn_exch{"1H19 T5C"}="H191 T5C"; $resn_exch{"1H20 T5C"}="H201 T5C"; $resn_exch{"1H31 T5C"}="H311 T5C";
$resn_exch{"2H31 T5C"}="H312 T5C"; $resn_exch{"3H31 T5C"}="H313 T5C"; $resn_exch{"1H30 T5C"}="H301 T5C";
$resn_exch{"2H30 T5C"}="H302 T5C"; $resn_exch{"3H30 T5C"}="H303 T5C"; $resn_exch{"1H32 T5C"}="H321 T5C";
$resn_exch{"2H32 T5C"}="H322 T5C"; $resn_exch{"1H33 T5C"}="H331 T5C"; $resn_exch{"2H33 T5C"}="H332 T5C";
$resn_exch{"1H17 T5P"}="H171 T5P"; $resn_exch{"2H17 T5P"}="H172 T5P"; $resn_exch{"1H16 T5P"}="H161 T5P";
$resn_exch{"2H16 T5P"}="H162 T5P"; $resn_exch{"1H15 T5P"}="H151 T5P"; $resn_exch{"2H15 T5P"}="H152 T5P";
$resn_exch{"1H10 T5P"}="H101 T5P"; $resn_exch{"2H10 T5P"}="H102 T5P"; $resn_exch{"1H14 T5P"}="H141 T5P";
$resn_exch{"2H14 T5P"}="H142 T5P"; $resn_exch{"1H7  T5X"}=" H71 T5X"; $resn_exch{"2H7  T5X"}=" H72 T5X";
$resn_exch{"1H6  T5X"}=" H61 T5X"; $resn_exch{"2H6  T5X"}=" H62 T5X"; $resn_exch{"1HX5 T5X"}="HX51 T5X";
$resn_exch{"2HX5 T5X"}="HX52 T5X"; $resn_exch{"1HX1 T5X"}="HX11 T5X"; $resn_exch{"2HX1 T5X"}="HX12 T5X";
$resn_exch{"1HM4 T5X"}="HM41 T5X"; $resn_exch{"2HM4 T5X"}="HM42 T5X"; $resn_exch{"3HM4 T5X"}="HM43 T5X";
$resn_exch{"1H7, T5X"}="H7,1 T5X"; $resn_exch{"2H7, T5X"}="H7,2 T5X"; $resn_exch{"1H4, T5X"}="H4,1 T5X";
$resn_exch{"2H4, T5X"}="H4,2 T5X"; $resn_exch{"1HM2 T5X"}="HM21 T5X"; $resn_exch{"2HM2 T5X"}="HM22 T5X";
$resn_exch{"3HM2 T5X"}="HM23 T5X"; $resn_exch{"1H8' T66"}="H8'1 T66"; $resn_exch{"2H8' T66"}="H8'2 T66";
$resn_exch{"1H5' T66"}="H5'1 T66"; $resn_exch{"1H3' T66"}="H3'1 T66"; $resn_exch{"2H3' T66"}="H3'2 T66";
$resn_exch{"1H2' T66"}="H2'1 T66"; $resn_exch{"2H2' T66"}="H2'2 T66"; $resn_exch{"1H1' T66"}="H1'1 T66";
$resn_exch{"2H1' T66"}="H1'2 T66"; $resn_exch{"1HM5 T66"}="HM51 T66"; $resn_exch{"2HM5 T66"}="HM52 T66";
$resn_exch{"3HM5 T66"}="HM53 T66"; $resn_exch{"1HB  T66"}=" HB1 T66"; $resn_exch{"2HB  T66"}=" HB2 T66";
$resn_exch{"1HG  T66"}=" HG1 T66"; $resn_exch{"2HG  T66"}=" HG2 T66"; $resn_exch{"1HD  T66"}=" HD1 T66";
$resn_exch{"2HD  T66"}=" HD2 T66"; $resn_exch{"1HE  T66"}=" HE1 T66"; $resn_exch{"2HE  T66"}=" HE2 T66";
$resn_exch{"1HZ  T66"}=" HZ1 T66"; $resn_exch{"2HZ  T66"}=" HZ2 T66"; $resn_exch{"2HOP T6A"}="HOP2 T6A";
$resn_exch{"3HOP T6A"}="HOP3 T6A"; $resn_exch{"1H5* T6A"}=" H5' T6A"; $resn_exch{"2H5* T6A"}="H5'' T6A";
$resn_exch{" H4* T6A"}=" H4' T6A"; $resn_exch{" H3* T6A"}=" H3' T6A"; $resn_exch{"H3T  T6A"}="HO3' T6A";
$resn_exch{" H2* T6A"}=" H2' T6A"; $resn_exch{"2HO* T6A"}="HO2' T6A"; $resn_exch{" H1* T6A"}=" H1' T6A";
$resn_exch{"1H15 T6A"}="H151 T6A"; $resn_exch{"2H15 T6A"}="H152 T6A"; $resn_exch{"3H15 T6A"}="H153 T6A";
$resn_exch{"1H7  T6F"}=" H71 T6F"; $resn_exch{"2H7  T6F"}=" H72 T6F"; $resn_exch{"1H6  T6F"}=" H61 T6F";
$resn_exch{"2H6  T6F"}=" H62 T6F"; $resn_exch{"1HF1 T6F"}="HF11 T6F"; $resn_exch{"2HF1 T6F"}="HF12 T6F";
$resn_exch{"1HF6 T6F"}="HF61 T6F"; $resn_exch{"2HF6 T6F"}="HF62 T6F"; $resn_exch{"1HM4 T6F"}="HM41 T6F";
$resn_exch{"2HM4 T6F"}="HM42 T6F"; $resn_exch{"3HM4 T6F"}="HM43 T6F"; $resn_exch{"1H7, T6F"}="H7,1 T6F";
$resn_exch{"2H7, T6F"}="H7,2 T6F"; $resn_exch{"1H4, T6F"}="H4,1 T6F"; $resn_exch{"2H4, T6F"}="H4,2 T6F";
$resn_exch{"1HM2 T6F"}="HM21 T6F"; $resn_exch{"2HM2 T6F"}="HM22 T6F"; $resn_exch{"3HM2 T6F"}="HM23 T6F";
$resn_exch{"1HC6 T6P"}="HC61 T6P"; $resn_exch{"2HC6 T6P"}="HC62 T6P"; $resn_exch{"1H6P T6P"}="H6P1 T6P";
$resn_exch{"2H6P T6P"}="H6P2 T6P"; $resn_exch{"7HOP T6P"}="HOP7 T6P"; $resn_exch{"8HOP T6P"}="HOP8 T6P";
$resn_exch{"1H7  T76"}=" H71 T76"; $resn_exch{"2H7  T76"}=" H72 T76"; $resn_exch{"1H18 T76"}="H181 T76";
$resn_exch{"2H18 T76"}="H182 T76"; $resn_exch{"1H19 T76"}="H191 T76"; $resn_exch{"2H19 T76"}="H192 T76";
$resn_exch{"1H20 T76"}="H201 T76"; $resn_exch{"2H20 T76"}="H202 T76"; $resn_exch{"1H34 T76"}="H341 T76";
$resn_exch{"2H34 T76"}="H342 T76"; $resn_exch{"1H35 T76"}="H351 T76"; $resn_exch{"2H35 T76"}="H352 T76";
$resn_exch{"1H43 T76"}="H431 T76"; $resn_exch{"2H43 T76"}="H432 T76"; $resn_exch{"1H47 T76"}="H471 T76";
$resn_exch{"2H47 T76"}="H472 T76"; $resn_exch{"1H48 T76"}="H481 T76"; $resn_exch{"2H48 T76"}="H482 T76";
$resn_exch{"1H49 T76"}="H491 T76"; $resn_exch{"2H49 T76"}="H492 T76"; $resn_exch{"1H50 T76"}="H501 T76";
$resn_exch{"2H50 T76"}="H502 T76"; $resn_exch{"1H51 T76"}="H511 T76"; $resn_exch{"2H51 T76"}="H512 T76";
$resn_exch{"1H1  T80"}=" H11 T80"; $resn_exch{"2H1  T80"}=" H12 T80"; $resn_exch{"3H1  T80"}=" H13 T80";
$resn_exch{"1H3  T80"}=" H31 T80"; $resn_exch{"2H3  T80"}=" H32 T80"; $resn_exch{"1H4  T80"}=" H41 T80";
$resn_exch{"2H4  T80"}=" H42 T80"; $resn_exch{"1H6  T80"}=" H61 T80"; $resn_exch{"2H6  T80"}=" H62 T80";
$resn_exch{"1H7  T80"}=" H71 T80"; $resn_exch{"2H7  T80"}=" H72 T80"; $resn_exch{"1H9  T80"}=" H91 T80";
$resn_exch{"2H9  T80"}=" H92 T80"; $resn_exch{"1H10 T80"}="H101 T80"; $resn_exch{"2H10 T80"}="H102 T80";
$resn_exch{"1H12 T80"}="H121 T80"; $resn_exch{"2H12 T80"}="H122 T80"; $resn_exch{"1H13 T80"}="H131 T80";
$resn_exch{"2H13 T80"}="H132 T80"; $resn_exch{"1H15 T80"}="H151 T80"; $resn_exch{"2H15 T80"}="H152 T80";
$resn_exch{"1H16 T80"}="H161 T80"; $resn_exch{"2H16 T80"}="H162 T80"; $resn_exch{"1H20 T80"}="H201 T80";
$resn_exch{"2H20 T80"}="H202 T80"; $resn_exch{"1H21 T80"}="H211 T80"; $resn_exch{"2H21 T80"}="H212 T80";
$resn_exch{"1H22 T80"}="H221 T80"; $resn_exch{"2H22 T80"}="H222 T80"; $resn_exch{"1H23 T80"}="H231 T80";
$resn_exch{"2H23 T80"}="H232 T80"; $resn_exch{"1H24 T80"}="H241 T80"; $resn_exch{"2H24 T80"}="H242 T80";
$resn_exch{"1H25 T80"}="H251 T80"; $resn_exch{"2H25 T80"}="H252 T80"; $resn_exch{"1H26 T80"}="H261 T80";
$resn_exch{"2H26 T80"}="H262 T80"; $resn_exch{"1H27 T80"}="H271 T80"; $resn_exch{"2H27 T80"}="H272 T80";
$resn_exch{"1H28 T80"}="H281 T80"; $resn_exch{"2H28 T80"}="H282 T80"; $resn_exch{"1H29 T80"}="H291 T80";
$resn_exch{"2H29 T80"}="H292 T80"; $resn_exch{"1H30 T80"}="H301 T80"; $resn_exch{"2H30 T80"}="H302 T80";
$resn_exch{"1H31 T80"}="H311 T80"; $resn_exch{"2H31 T80"}="H312 T80"; $resn_exch{"1H32 T80"}="H321 T80";
$resn_exch{"2H32 T80"}="H322 T80"; $resn_exch{"1H33 T80"}="H331 T80"; $resn_exch{"2H33 T80"}="H332 T80";
$resn_exch{"1H34 T80"}="H341 T80"; $resn_exch{"2H34 T80"}="H342 T80"; $resn_exch{"1H35 T80"}="H351 T80";
$resn_exch{"2H35 T80"}="H352 T80"; $resn_exch{"1H36 T80"}="H361 T80"; $resn_exch{"2H36 T80"}="H362 T80";
$resn_exch{"3H36 T80"}="H363 T80"; $resn_exch{"1HN2 T87"}="HN21 T87"; $resn_exch{"2HN2 T87"}="HN22 T87";
$resn_exch{"CH10 T87"}="H10C T87"; $resn_exch{"CH11 T87"}="H11C T87"; $resn_exch{"CH13 T87"}="H13C T87";
$resn_exch{"1H14 T87"}="H141 T87"; $resn_exch{"2H14 T87"}="H142 T87"; $resn_exch{"3H14 T87"}="H143 T87";
$resn_exch{"1HC9 T87"}="HC91 T87"; $resn_exch{"2HC9 T87"}="HC92 T87"; $resn_exch{"1H15 T87"}="H151 T87";
$resn_exch{"2H15 T87"}="H152 T87"; $resn_exch{"CH19 T87"}="H19C T87"; $resn_exch{"CH20 T87"}="H20C T87";
$resn_exch{"CH21 T87"}="H21C T87"; $resn_exch{"1H23 T87"}="H231 T87"; $resn_exch{"2H23 T87"}="H232 T87";
$resn_exch{"1H25 T87"}="H251 T87"; $resn_exch{"2H25 T87"}="H252 T87"; $resn_exch{"3H25 T87"}="H253 T87";
$resn_exch{"1H26 T87"}="H261 T87"; $resn_exch{"2H26 T87"}="H262 T87"; $resn_exch{"1H1  T98"}="H11A T98";
$resn_exch{"2H1  T98"}="H12A T98"; $resn_exch{"3H1  T98"}=" H13 T98"; $resn_exch{"1H14 T98"}="H141 T98";
$resn_exch{"2H14 T98"}="H142 T98"; $resn_exch{"3H14 T98"}="H143 T98"; $resn_exch{"1H16 T98"}="H161 T98";
$resn_exch{"2H16 T98"}="H162 T98"; $resn_exch{"3H16 T98"}="H163 T98"; $resn_exch{"1H01 TA1"}="H011 TA1";
$resn_exch{"1H02 TA1"}="H021 TA1"; $resn_exch{"1H05 TA1"}="H051 TA1"; $resn_exch{"1H06 TA1"}="H061 TA1";
$resn_exch{"1HC7 TA1"}="HC71 TA1"; $resn_exch{"1H08 TA1"}="H081 TA1"; $resn_exch{"1H09 TA1"}="H091 TA1";
$resn_exch{"1H10 TA1"}="H101 TA1"; $resn_exch{"1H13 TA1"}="H131 TA1"; $resn_exch{"2H13 TA1"}="H132 TA1";
$resn_exch{"3H13 TA1"}="H133 TA1"; $resn_exch{"1H14 TA1"}="H141 TA1"; $resn_exch{"2H14 TA1"}="H142 TA1";
$resn_exch{"1H15 TA1"}="H151 TA1"; $resn_exch{"1H16 TA1"}="H161 TA1"; $resn_exch{"2H16 TA1"}="H162 TA1";
$resn_exch{"1H17 TA1"}="H171 TA1"; $resn_exch{"1H07 TA1"}="H071 TA1"; $resn_exch{"1H19 TA1"}="H191 TA1";
$resn_exch{"2H19 TA1"}="H192 TA1"; $resn_exch{"3H19 TA1"}="H193 TA1"; $resn_exch{"1H21 TA1"}="H211 TA1";
$resn_exch{"1H23 TA1"}="H231 TA1"; $resn_exch{"2H23 TA1"}="H232 TA1"; $resn_exch{"3H23 TA1"}="H233 TA1";
$resn_exch{"1H26 TA1"}="H261 TA1"; $resn_exch{"1H28 TA1"}="H281 TA1"; $resn_exch{"1HO1 TA1"}="HO11 TA1";
$resn_exch{"1H29 TA1"}="H291 TA1"; $resn_exch{"1HN0 TA1"}="HN01 TA1"; $resn_exch{"1H32 TA1"}="H321 TA1";
$resn_exch{"1H33 TA1"}="H331 TA1"; $resn_exch{"1H34 TA1"}="H341 TA1"; $resn_exch{"1H35 TA1"}="H351 TA1";
$resn_exch{"1H36 TA1"}="H361 TA1"; $resn_exch{"1H38 TA1"}="H381 TA1"; $resn_exch{"1H39 TA1"}="H391 TA1";
$resn_exch{"1H40 TA1"}="H401 TA1"; $resn_exch{"1H41 TA1"}="H411 TA1"; $resn_exch{"1H42 TA1"}="H421 TA1";
$resn_exch{"1H43 TA1"}="H431 TA1"; $resn_exch{"2H43 TA1"}="H432 TA1"; $resn_exch{"1H44 TA1"}="H441 TA1";
$resn_exch{"2H44 TA1"}="H442 TA1"; $resn_exch{"3H44 TA1"}="H443 TA1"; $resn_exch{"1H46 TA1"}="H461 TA1";
$resn_exch{"2H46 TA1"}="H462 TA1"; $resn_exch{"3H46 TA1"}="H463 TA1"; $resn_exch{"1H47 TA1"}="H471 TA1";
$resn_exch{"2H47 TA1"}="H472 TA1"; $resn_exch{"3H47 TA1"}="H473 TA1"; $resn_exch{"1H   TA2"}=" H1  TA2";
$resn_exch{"2H   TA2"}=" H2  TA2"; $resn_exch{"1HN  TA2"}=" HN1 TA2"; $resn_exch{"2HN  TA2"}=" HN2 TA2";
$resn_exch{"1H5* TA3"}=" H5' TA3"; $resn_exch{"2H5* TA3"}="H5'' TA3"; $resn_exch{" H4* TA3"}=" H4' TA3";
$resn_exch{" H3* TA3"}=" H3' TA3"; $resn_exch{"HO3* TA3"}="HO3' TA3"; $resn_exch{"1H2* TA3"}=" H2' TA3";
$resn_exch{"2H2* TA3"}="H2'' TA3"; $resn_exch{" H1* TA3"}=" H1' TA3"; $resn_exch{"1H4  TA3"}=" H41 TA3";
$resn_exch{"2H4  TA3"}=" H42 TA3"; $resn_exch{"1H5M TA3"}=" H71 TA3"; $resn_exch{"2H5M TA3"}=" H72 TA3";
$resn_exch{"3H5M TA3"}=" H73 TA3"; $resn_exch{"1HB  TA4"}=" HB2 TA4"; $resn_exch{"2HB  TA4"}=" HB3 TA4";
$resn_exch{"1HD2 TA4"}="HD21 TA4"; $resn_exch{"2HD2 TA4"}="HD22 TA4"; $resn_exch{"3HD2 TA4"}="HD23 TA4";
$resn_exch{"1HD1 TA4"}="HD11 TA4"; $resn_exch{"2HD1 TA4"}="HD12 TA4"; $resn_exch{"3HD1 TA4"}="HD13 TA4";
$resn_exch{"1HT6 TA4"}="HT61 TA4"; $resn_exch{"2HT6 TA4"}="HT62 TA4"; $resn_exch{" HN2 TA4"}=" H2  TA4";
$resn_exch{"1HBC TA5"}="HBC1 TA5"; $resn_exch{"2HBC TA5"}="HBC2 TA5"; $resn_exch{"1H1C TA6"}="H1C1 TA6";
$resn_exch{"2H1C TA6"}="H1C2 TA6"; $resn_exch{"1H6C TA6"}="H6C1 TA6"; $resn_exch{"2H6C TA6"}="H6C2 TA6";
$resn_exch{"1HC1 TAA"}="HC11 TAA"; $resn_exch{"2HC1 TAA"}="HC12 TAA"; $resn_exch{"1H12 TAA"}="H121 TAA";
$resn_exch{"2H12 TAA"}="H122 TAA"; $resn_exch{"1H3' TAB"}="H3'1 TAB"; $resn_exch{"1H4' TAB"}="H4'1 TAB";
$resn_exch{"1H6' TAB"}="H6'1 TAB"; $resn_exch{"1H11 TAB"}="H111 TAB"; $resn_exch{"2H11 TAB"}="H112 TAB";
$resn_exch{"1HN2 TAB"}="HN21 TAB"; $resn_exch{"2HN2 TAB"}="HN22 TAB"; $resn_exch{"1HN1 TAB"}="HN11 TAB";
$resn_exch{"2HN1 TAB"}="HN12 TAB"; $resn_exch{"1H19 TAB"}="H191 TAB"; $resn_exch{"2H19 TAB"}="H192 TAB";
$resn_exch{"1H20 TAB"}="H201 TAB"; $resn_exch{"2H20 TAB"}="H202 TAB"; $resn_exch{"3H20 TAB"}="H203 TAB";
$resn_exch{"1H21 TAB"}="H211 TAB"; $resn_exch{"2H21 TAB"}="H212 TAB"; $resn_exch{"1H22 TAB"}="H221 TAB";
$resn_exch{"2H22 TAB"}="H222 TAB"; $resn_exch{"1H25 TAB"}="H251 TAB"; $resn_exch{"2H25 TAB"}="H252 TAB";
$resn_exch{"3H25 TAB"}="H253 TAB"; $resn_exch{"1H28 TAB"}="H281 TAB"; $resn_exch{"1H29 TAB"}="H291 TAB";
$resn_exch{"1H30 TAB"}="H301 TAB"; $resn_exch{"1H31 TAB"}="H311 TAB"; $resn_exch{"1H32 TAB"}="H321 TAB";
$resn_exch{"1HN2 TAC"}="HN21 TAC"; $resn_exch{"2HN2 TAC"}="HN22 TAC"; $resn_exch{"1H42 TAC"}="H421 TAC";
$resn_exch{"2H42 TAC"}="H422 TAC"; $resn_exch{"3H42 TAC"}="H423 TAC"; $resn_exch{"1H43 TAC"}="H431 TAC";
$resn_exch{"2H43 TAC"}="H432 TAC"; $resn_exch{"3H43 TAC"}="H433 TAC"; $resn_exch{"1H5  TAC"}="H51A TAC";
$resn_exch{"2H5  TAC"}=" H52 TAC"; $resn_exch{"1H62 TAC"}="H621 TAC"; $resn_exch{"2H62 TAC"}="H622 TAC";
$resn_exch{"3H62 TAC"}="H623 TAC"; $resn_exch{"2HOA TAD"}="HOA2 TAD"; $resn_exch{"AH51 TAD"}="H51A TAD";
$resn_exch{"AH52 TAD"}="H52A TAD"; $resn_exch{"AH4* TAD"}=" H4B TAD"; $resn_exch{"AH3* TAD"}=" H3B TAD";
$resn_exch{"AHO3 TAD"}="HO3A TAD"; $resn_exch{"AH2* TAD"}=" H2B TAD"; $resn_exch{"AHO2 TAD"}="HO2A TAD";
$resn_exch{"AH1* TAD"}=" H1B TAD"; $resn_exch{" AH8 TAD"}=" H8A TAD"; $resn_exch{"AH61 TAD"}="H61A TAD";
$resn_exch{"AH62 TAD"}="H62A TAD"; $resn_exch{" AH2 TAD"}=" H2A TAD"; $resn_exch{"1H3  TAD"}=" H31 TAD";
$resn_exch{"2H3  TAD"}=" H32 TAD"; $resn_exch{"2HON TAD"}="HON2 TAD"; $resn_exch{"NH51 TAD"}="H51N TAD";
$resn_exch{"NH52 TAD"}="H52N TAD"; $resn_exch{"NH4* TAD"}=" H4D TAD"; $resn_exch{"NH3* TAD"}=" H3D TAD";
$resn_exch{"NHO3 TAD"}="HO3N TAD"; $resn_exch{"NH2* TAD"}=" H2D TAD"; $resn_exch{"NHO2 TAD"}="HO2N TAD";
$resn_exch{"NH1* TAD"}=" H1D TAD"; $resn_exch{" NH5 TAD"}=" H5N TAD"; $resn_exch{"NH61 TAD"}="H61N TAD";
$resn_exch{"NH62 TAD"}="H62N TAD"; $resn_exch{"1HOP TAF"}="HOP1 TAF"; $resn_exch{"2HOP TAF"}="HOP2 TAF";
$resn_exch{"1H5M TAF"}=" H71 TAF"; $resn_exch{"2H5M TAF"}=" H72 TAF"; $resn_exch{"3H5M TAF"}=" H73 TAF";
$resn_exch{" H2* TAF"}=" H2' TAF"; $resn_exch{"1H5* TAF"}=" H5' TAF"; $resn_exch{"2H5* TAF"}="H5'' TAF";
$resn_exch{" H4* TAF"}=" H4' TAF"; $resn_exch{" H1* TAF"}=" H1' TAF"; $resn_exch{" H3* TAF"}=" H3' TAF";
$resn_exch{"3HO* TAF"}="HO'3 TAF"; $resn_exch{"1H1  TAG"}=" H11 TAG"; $resn_exch{"2H1  TAG"}=" H12 TAG";
$resn_exch{"1H6  TAG"}=" H61 TAG"; $resn_exch{"2H6  TAG"}=" H62 TAG"; $resn_exch{"1H6' TAL"}="H6'1 TAL";
$resn_exch{"2H6' TAL"}="H6'2 TAL"; $resn_exch{"3H6' TAL"}="H6'3 TAL"; $resn_exch{"1H7  TAL"}=" H71 TAL";
$resn_exch{"2H7  TAL"}=" H72 TAL"; $resn_exch{"3H7  TAL"}=" H73 TAL"; $resn_exch{"1H1  TAM"}=" H11 TAM";
$resn_exch{"2H1  TAM"}=" H12 TAM"; $resn_exch{"1H2  TAM"}=" H21 TAM"; $resn_exch{"2H2  TAM"}=" H22 TAM";
$resn_exch{"1H3  TAM"}=" H31 TAM"; $resn_exch{"2H3  TAM"}=" H32 TAM"; $resn_exch{"1H4  TAM"}=" H41 TAM";
$resn_exch{"2H4  TAM"}=" H42 TAM"; $resn_exch{"1H5  TAM"}=" H51 TAM"; $resn_exch{"2H5  TAM"}=" H52 TAM";
$resn_exch{"1H6  TAM"}=" H61 TAM"; $resn_exch{"2H6  TAM"}=" H62 TAM"; $resn_exch{"1HN  TAM"}=" HN1 TAM";
$resn_exch{"2HN  TAM"}=" HN2 TAM"; $resn_exch{"1H53 TAO"}="H531 TAO"; $resn_exch{"2H53 TAO"}="H532 TAO";
$resn_exch{"3H53 TAO"}="H533 TAO"; $resn_exch{"1H42 TAO"}="H421 TAO"; $resn_exch{"2H42 TAO"}="H422 TAO";
$resn_exch{"1H52 TAO"}="H521 TAO"; $resn_exch{"2H52 TAO"}="H522 TAO"; $resn_exch{"3H52 TAO"}="H523 TAO";
$resn_exch{"1H56 TAO"}="H561 TAO"; $resn_exch{"2H56 TAO"}="H562 TAO"; $resn_exch{"3H56 TAO"}="H563 TAO";
$resn_exch{"1H22 TAO"}="H221 TAO"; $resn_exch{"2H22 TAO"}="H222 TAO"; $resn_exch{"3H22 TAO"}="H223 TAO";
$resn_exch{"1H16 TAO"}="H161 TAO"; $resn_exch{"2H16 TAO"}="H162 TAO"; $resn_exch{"3H16 TAO"}="H163 TAO";
$resn_exch{"1H20 TAO"}="H201 TAO"; $resn_exch{"2H20 TAO"}="H202 TAO"; $resn_exch{"3H20 TAO"}="H203 TAO";
$resn_exch{"1H5  TAO"}="H51A TAO"; $resn_exch{"2H5  TAO"}=" H52 TAO"; $resn_exch{"1H7  TAO"}=" H71 TAO";
$resn_exch{"2H7  TAO"}=" H72 TAO"; $resn_exch{"1H13 TAO"}="H131 TAO"; $resn_exch{"2H13 TAO"}="H132 TAO";
$resn_exch{"3H13 TAO"}="H133 TAO"; $resn_exch{"1H41 TAO"}="H411 TAO"; $resn_exch{"2H41 TAO"}="H412 TAO";
$resn_exch{"3H41 TAO"}="H413 TAO"; $resn_exch{"1H40 TAO"}="H401 TAO"; $resn_exch{"2H40 TAO"}="H402 TAO";
$resn_exch{"1H46 TAO"}="H461 TAO"; $resn_exch{"2H46 TAO"}="H462 TAO"; $resn_exch{"3H46 TAO"}="H463 TAO";
$resn_exch{"1H47 TAO"}="H471 TAO"; $resn_exch{"2H47 TAO"}="H472 TAO"; $resn_exch{"3H47 TAO"}="H473 TAO";
$resn_exch{"1H44 TAO"}="H441 TAO"; $resn_exch{"2H44 TAO"}="H442 TAO"; $resn_exch{"3H44 TAO"}="H443 TAO";
$resn_exch{"1H25 TAO"}="H251 TAO"; $resn_exch{"2H25 TAO"}="H252 TAO"; $resn_exch{"3H25 TAO"}="H253 TAO";
$resn_exch{"1H8  TAO"}=" H81 TAO"; $resn_exch{"2H8  TAO"}=" H82 TAO"; $resn_exch{"3H8  TAO"}=" H83 TAO";
$resn_exch{"1H37 TAO"}="H371 TAO"; $resn_exch{"2H37 TAO"}="H372 TAO"; $resn_exch{"3H37 TAO"}="H373 TAO";
$resn_exch{"2HOA TAP"}="HOA2 TAP"; $resn_exch{"AH51 TAP"}="H51A TAP"; $resn_exch{"AH52 TAP"}="H52A TAP";
$resn_exch{"AH4* TAP"}=" H4B TAP"; $resn_exch{"AH3* TAP"}=" H3B TAP"; $resn_exch{"AHO3 TAP"}="HO3A TAP";
$resn_exch{"AH2* TAP"}=" H2B TAP"; $resn_exch{"AH1* TAP"}=" H1B TAP"; $resn_exch{" AH8 TAP"}=" H8A TAP";
$resn_exch{"AH61 TAP"}="H61A TAP"; $resn_exch{"AH62 TAP"}="H62A TAP"; $resn_exch{" AH2 TAP"}=" H2A TAP";
$resn_exch{"NH51 TAP"}="H51N TAP"; $resn_exch{"NH52 TAP"}="H52N TAP"; $resn_exch{"NH4* TAP"}=" H4D TAP";
$resn_exch{"NH3* TAP"}=" H3D TAP"; $resn_exch{"NHO3 TAP"}="HO3N TAP"; $resn_exch{"NH2* TAP"}=" H2D TAP";
$resn_exch{"NHO2 TAP"}="HO2N TAP"; $resn_exch{"NH1* TAP"}=" H1D TAP"; $resn_exch{" NH2 TAP"}=" H2N TAP";
$resn_exch{"NH71 TAP"}="H71N TAP"; $resn_exch{"NH72 TAP"}="H72N TAP"; $resn_exch{" NH4 TAP"}=" H4N TAP";
$resn_exch{" NH5 TAP"}=" H5N TAP"; $resn_exch{" NH6 TAP"}=" H6N TAP"; $resn_exch{"2HOP TAP"}="HOP2 TAP";
$resn_exch{"3HOP TAP"}="HOP3 TAP"; $resn_exch{"1H4N TAQ"}="H4N1 TAQ"; $resn_exch{"2H4N TAQ"}="H4N2 TAQ";
$resn_exch{"1H2N TAQ"}="H2N1 TAQ"; $resn_exch{"2H2N TAQ"}="H2N2 TAQ"; $resn_exch{"1HAA TAQ"}="HAA1 TAQ";
$resn_exch{"2HAA TAQ"}="HAA2 TAQ"; $resn_exch{"1H1  TAS"}=" H11 TAS"; $resn_exch{"1H2  TAS"}=" H21 TAS";
$resn_exch{"1H3  TAS"}=" H31 TAS"; $resn_exch{"1H5* TAT"}="H5'1 TAT"; $resn_exch{"2H5* TAT"}="H5'2 TAT";
$resn_exch{" H4* TAT"}=" H4' TAT"; $resn_exch{" H3* TAT"}=" H3' TAT"; $resn_exch{" H2* TAT"}=" H2' TAT";
$resn_exch{" H1* TAT"}=" H1' TAT"; $resn_exch{"1H6N TAT"}="H6N1 TAT"; $resn_exch{"2H6N TAT"}="H6N2 TAT";
$resn_exch{"1H1C TAU"}="H1C1 TAU"; $resn_exch{"2H1C TAU"}="H1C2 TAU"; $resn_exch{"1H2C TAU"}="H2C1 TAU";
$resn_exch{"2H2C TAU"}="H2C2 TAU"; $resn_exch{"1H1N TAU"}="H1N1 TAU"; $resn_exch{"2H1N TAU"}="H1N2 TAU";
$resn_exch{"1H56 TAV"}="H561 TAV"; $resn_exch{"2H56 TAV"}="H562 TAV"; $resn_exch{"3H56 TAV"}="H563 TAV";
$resn_exch{"1H67 TAV"}="H671 TAV"; $resn_exch{"2H67 TAV"}="H672 TAV"; $resn_exch{"1HA  TAX"}=" HA1 TAX";
$resn_exch{"2HA  TAX"}=" HA2 TAX"; $resn_exch{"1HB  TAX"}=" HB1 TAX"; $resn_exch{"2HB  TAX"}=" HB2 TAX";
$resn_exch{"3HB  TAX"}=" HB3 TAX"; $resn_exch{"1HA1 TAX"}="HA11 TAX"; $resn_exch{"2HA1 TAX"}="HA12 TAX";
$resn_exch{"1HB1 TAX"}="HB11 TAX"; $resn_exch{"2HB1 TAX"}="HB12 TAX"; $resn_exch{"1H21 TAX"}="H211 TAX";
$resn_exch{"2H21 TAX"}="H212 TAX"; $resn_exch{"3H21 TAX"}="H213 TAX"; $resn_exch{"1H22 TAX"}="H221 TAX";
$resn_exch{"2H22 TAX"}="H222 TAX"; $resn_exch{"3H22 TAX"}="H223 TAX"; $resn_exch{"1HC5 TAZ"}="HC51 TAZ";
$resn_exch{"1HC6 TAZ"}="HC61 TAZ"; $resn_exch{"2HC6 TAZ"}="HC62 TAZ"; $resn_exch{"1H20 TAZ"}="H201 TAZ";
$resn_exch{"2H20 TAZ"}="H202 TAZ"; $resn_exch{"3H20 TAZ"}="H203 TAZ"; $resn_exch{"1H14 TAZ"}="H141 TAZ";
$resn_exch{"2H14 TAZ"}="H142 TAZ"; $resn_exch{"1H19 TAZ"}="H191 TAZ"; $resn_exch{"1H18 TAZ"}="H181 TAZ";
$resn_exch{"1H4  TB1"}=" H41 TB1"; $resn_exch{"2H4  TB1"}=" H42 TB1"; $resn_exch{"1HN  TB1"}=" HN1 TB1";
$resn_exch{"2HN  TB1"}=" HN2 TB1"; $resn_exch{"1H4  TB9"}=" H41 TB9"; $resn_exch{"2H4  TB9"}=" H42 TB9";
$resn_exch{"1H7  TB9"}=" H71 TB9"; $resn_exch{"2H7  TB9"}=" H72 TB9"; $resn_exch{"1H11 TB9"}="H111 TB9";
$resn_exch{"2H11 TB9"}="H112 TB9"; $resn_exch{"3H11 TB9"}="H113 TB9"; $resn_exch{"1H12 TB9"}="H121 TB9";
$resn_exch{"2H12 TB9"}="H122 TB9"; $resn_exch{"1H15 TB9"}="H151 TB9"; $resn_exch{"2H15 TB9"}="H152 TB9";
$resn_exch{"3H15 TB9"}="H153 TB9"; $resn_exch{"1H16 TB9"}="H161 TB9"; $resn_exch{"2H16 TB9"}="H162 TB9";
$resn_exch{"3H16 TB9"}="H163 TB9"; $resn_exch{"1H11 TBA"}="H111 TBA"; $resn_exch{"2H11 TBA"}="H112 TBA";
$resn_exch{"1H12 TBA"}="H121 TBA"; $resn_exch{"2H12 TBA"}="H122 TBA"; $resn_exch{"1H21 TBA"}="H211 TBA";
$resn_exch{"2H21 TBA"}="H212 TBA"; $resn_exch{"1H22 TBA"}="H221 TBA"; $resn_exch{"2H22 TBA"}="H222 TBA";
$resn_exch{"1H31 TBA"}="H311 TBA"; $resn_exch{"2H31 TBA"}="H312 TBA"; $resn_exch{"1H32 TBA"}="H321 TBA";
$resn_exch{"2H32 TBA"}="H322 TBA"; $resn_exch{"1H41 TBA"}="H411 TBA"; $resn_exch{"2H41 TBA"}="H412 TBA";
$resn_exch{"1H42 TBA"}="H421 TBA"; $resn_exch{"2H42 TBA"}="H422 TBA"; $resn_exch{"1H13 TBA"}="H131 TBA";
$resn_exch{"2H13 TBA"}="H132 TBA"; $resn_exch{"1H14 TBA"}="H141 TBA"; $resn_exch{"2H14 TBA"}="H142 TBA";
$resn_exch{"3H14 TBA"}="H143 TBA"; $resn_exch{"1H23 TBA"}="H231 TBA"; $resn_exch{"2H23 TBA"}="H232 TBA";
$resn_exch{"1H24 TBA"}="H241 TBA"; $resn_exch{"2H24 TBA"}="H242 TBA"; $resn_exch{"3H24 TBA"}="H243 TBA";
$resn_exch{"1H33 TBA"}="H331 TBA"; $resn_exch{"2H33 TBA"}="H332 TBA"; $resn_exch{"1H34 TBA"}="H341 TBA";
$resn_exch{"2H34 TBA"}="H342 TBA"; $resn_exch{"3H34 TBA"}="H343 TBA"; $resn_exch{"1H43 TBA"}="H431 TBA";
$resn_exch{"2H43 TBA"}="H432 TBA"; $resn_exch{"1H44 TBA"}="H441 TBA"; $resn_exch{"2H44 TBA"}="H442 TBA";
$resn_exch{"3H44 TBA"}="H443 TBA"; $resn_exch{"1H10 TBC"}="H101 TBC"; $resn_exch{"2H10 TBC"}="H102 TBC";
$resn_exch{"2H7  TBC"}=" H72 TBC"; $resn_exch{"1H7  TBC"}=" H71 TBC"; $resn_exch{"1H8  TBC"}=" H81 TBC";
$resn_exch{"2H8  TBC"}=" H82 TBC"; $resn_exch{"OH1B TBD"}="H1BO TBD"; $resn_exch{"OH2B TBD"}="H2BO TBD";
$resn_exch{"OH2A TBD"}="H2AO TBD"; $resn_exch{"1H5* TBD"}="H5'1 TBD"; $resn_exch{"2H5* TBD"}="H5'2 TBD";
$resn_exch{"CH4* TBD"}="H4'C TBD"; $resn_exch{"CH1* TBD"}="H1'C TBD"; $resn_exch{"1HCA TBD"}="HCA1 TBD";
$resn_exch{"2HCA TBD"}="HCA2 TBD"; $resn_exch{"3HCA TBD"}="HCA3 TBD"; $resn_exch{"1H2* TBD"}="H2'1 TBD";
$resn_exch{"2H2* TBD"}="H2'2 TBD"; $resn_exch{"CH3* TBD"}="H3'C TBD"; $resn_exch{"OH3* TBD"}="H3'O TBD";
$resn_exch{"1H20 TBE"}="H201 TBE"; $resn_exch{"2H20 TBE"}="H202 TBE"; $resn_exch{"3H20 TBE"}="H203 TBE";
$resn_exch{"1H14 TBE"}="H141 TBE"; $resn_exch{"2H14 TBE"}="H142 TBE"; $resn_exch{"1H1  TBF"}=" H11 TBF";
$resn_exch{"2H1  TBF"}=" H12 TBF"; $resn_exch{"3H1  TBF"}=" H13 TBF"; $resn_exch{"1H2  TBF"}=" H21 TBF";
$resn_exch{"2H2  TBF"}=" H22 TBF"; $resn_exch{"3H2  TBF"}=" H23 TBF"; $resn_exch{"1H3  TBF"}=" H31 TBF";
$resn_exch{"2H3  TBF"}=" H32 TBF"; $resn_exch{"3H3  TBF"}=" H33 TBF"; $resn_exch{"1HN  TBG"}=" H   TBG";
$resn_exch{"2HN  TBG"}=" H2  TBG"; $resn_exch{"1H1  TBG"}="HG11 TBG"; $resn_exch{"2H1  TBG"}="HG12 TBG";
$resn_exch{"3H1  TBG"}="HG13 TBG"; $resn_exch{"1H2  TBG"}="HG21 TBG"; $resn_exch{"2H2  TBG"}="HG22 TBG";
$resn_exch{"3H2  TBG"}="HG23 TBG"; $resn_exch{"1H3  TBG"}="HG31 TBG"; $resn_exch{"2H3  TBG"}="HG32 TBG";
$resn_exch{"3H3  TBG"}="HG33 TBG"; $resn_exch{"1H42 TBH"}="H421 TBH"; $resn_exch{"1H5  TBH"}=" H51 TBH";
$resn_exch{"1H23 TBH"}="H231 TBH"; $resn_exch{"2H23 TBH"}="H232 TBH"; $resn_exch{"1H26 TBH"}="H261 TBH";
$resn_exch{"1H2  TBH"}=" H21 TBH"; $resn_exch{"1H3  TBH"}=" H31 TBH"; $resn_exch{"1H7  TBH"}=" H71 TBH";
$resn_exch{"2H7  TBH"}=" H72 TBH"; $resn_exch{"1H8  TBH"}=" H81 TBH"; $resn_exch{"1H11 TBH"}="H111 TBH";
$resn_exch{"1H13 TBH"}="H131 TBH"; $resn_exch{"1H15 TBH"}="H151 TBH"; $resn_exch{"2H15 TBH"}="H152 TBH";
$resn_exch{"1H16 TBH"}="H161 TBH"; $resn_exch{"2H16 TBH"}="H162 TBH"; $resn_exch{"1H17 TBH"}="H171 TBH";
$resn_exch{"2H17 TBH"}="H172 TBH"; $resn_exch{"1H18 TBH"}="H181 TBH"; $resn_exch{"2H18 TBH"}="H182 TBH";
$resn_exch{"1H19 TBH"}="H191 TBH"; $resn_exch{"2H19 TBH"}="H192 TBH"; $resn_exch{"3H19 TBH"}="H193 TBH";
$resn_exch{"1H14 TBH"}="H141 TBH"; $resn_exch{"1H10 TBH"}="H101 TBH"; $resn_exch{"1H12 TBH"}="H121 TBH";
$resn_exch{"1H20 TBH"}="H201 TBH"; $resn_exch{"1H21 TBH"}="H211 TBH"; $resn_exch{"1H22 TBH"}="H221 TBH";
$resn_exch{"1H35 TBH"}="H351 TBH"; $resn_exch{"1H36 TBH"}="H361 TBH"; $resn_exch{"1H40 TBH"}="H401 TBH";
$resn_exch{"2H40 TBH"}="H402 TBH"; $resn_exch{"3H40 TBH"}="H403 TBH"; $resn_exch{"1H39 TBH"}="H391 TBH";
$resn_exch{"2H39 TBH"}="H392 TBH"; $resn_exch{"3H39 TBH"}="H393 TBH"; $resn_exch{"1H41 TBH"}="H411 TBH";
$resn_exch{"2H41 TBH"}="H412 TBH"; $resn_exch{"3H41 TBH"}="H413 TBH"; $resn_exch{"1H28 TBH"}="H281 TBH";
$resn_exch{"2H28 TBH"}="H282 TBH"; $resn_exch{"1H30 TBH"}="H301 TBH"; $resn_exch{"1H31 TBH"}="H311 TBH";
$resn_exch{"1H32 TBH"}="H321 TBH"; $resn_exch{"1H33 TBH"}="H331 TBH"; $resn_exch{"1H34 TBH"}="H341 TBH";
$resn_exch{"1H14 TBI"}="H141 TBI"; $resn_exch{"2H14 TBI"}="H142 TBI"; $resn_exch{"1H20 TBI"}="H201 TBI";
$resn_exch{"2H20 TBI"}="H202 TBI"; $resn_exch{"3H20 TBI"}="H203 TBI"; $resn_exch{"7H7  TBI"}=" H77 TBI";
$resn_exch{"1HCT TBM"}="HCT1 TBM"; $resn_exch{"2HCT TBM"}="HCT2 TBM"; $resn_exch{"3HCT TBM"}="HCT3 TBM";
$resn_exch{"1HCN TBM"}="HCN1 TBM"; $resn_exch{"2HCN TBM"}="HCN2 TBM"; $resn_exch{"3HCN TBM"}="HCN3 TBM";
$resn_exch{"1HD1 TBM"}="HD11 TBM"; $resn_exch{"2HD1 TBM"}="HD12 TBM"; $resn_exch{"3HD1 TBM"}="HD13 TBM";
$resn_exch{"1HD2 TBM"}="HD22 TBM"; $resn_exch{"2HD2 TBM"}="HD23 TBM"; $resn_exch{"1HH  TBM"}=" HH1 TBM";
$resn_exch{"2HH  TBM"}=" HH2 TBM"; $resn_exch{"3HH  TBM"}=" HH3 TBM"; $resn_exch{"1H5' TBN"}="H5'1 TBN";
$resn_exch{"2H5' TBN"}="H5'2 TBN"; $resn_exch{"1HN6 TBN"}="HN61 TBN"; $resn_exch{"2HN6 TBN"}="HN62 TBN";
$resn_exch{"1H4  TBO"}=" H41 TBO"; $resn_exch{"2H4  TBO"}=" H42 TBO"; $resn_exch{"1H7  TBO"}=" H71 TBO";
$resn_exch{"2H7  TBO"}=" H72 TBO"; $resn_exch{"1H11 TBO"}="H111 TBO"; $resn_exch{"2H11 TBO"}="H112 TBO";
$resn_exch{"3H11 TBO"}="H113 TBO"; $resn_exch{"1H12 TBO"}="H121 TBO"; $resn_exch{"2H12 TBO"}="H122 TBO";
$resn_exch{"1H15 TBO"}="H151 TBO"; $resn_exch{"2H15 TBO"}="H152 TBO"; $resn_exch{"3H15 TBO"}="H153 TBO";
$resn_exch{"1H16 TBO"}="H161 TBO"; $resn_exch{"2H16 TBO"}="H162 TBO"; $resn_exch{"3H16 TBO"}="H163 TBO";
$resn_exch{"1H4C TBT"}="H4C1 TBT"; $resn_exch{"2H4C TBT"}="H4C2 TBT"; $resn_exch{"1H1  TBU"}=" H11 TBU";
$resn_exch{"2H1  TBU"}=" H12 TBU"; $resn_exch{"3H1  TBU"}=" H13 TBU"; $resn_exch{"1H2  TBU"}=" H21 TBU";
$resn_exch{"2H2  TBU"}=" H22 TBU"; $resn_exch{"3H2  TBU"}=" H23 TBU"; $resn_exch{"1H3  TBU"}=" H31 TBU";
$resn_exch{"2H3  TBU"}=" H32 TBU"; $resn_exch{"3H3  TBU"}=" H33 TBU"; $resn_exch{"1HN1 TBZ"}="HN11 TBZ";
$resn_exch{"2HN1 TBZ"}="HN12 TBZ"; $resn_exch{"3HN1 TBZ"}="HN13 TBZ"; $resn_exch{"1H2  TBZ"}=" H21 TBZ";
$resn_exch{"2H2  TBZ"}=" H22 TBZ"; $resn_exch{"1H3  TBZ"}="H31A TBZ"; $resn_exch{"2H3  TBZ"}=" H32 TBZ";
$resn_exch{"1H4  TBZ"}=" H41 TBZ"; $resn_exch{"2H4  TBZ"}=" H42 TBZ"; $resn_exch{"1H32 TBZ"}="H321 TBZ";
$resn_exch{"2H32 TBZ"}="H322 TBZ"; $resn_exch{"3H32 TBZ"}="H323 TBZ"; $resn_exch{"1H5* TC1"}=" H5' TC1";
$resn_exch{"2H5* TC1"}="H5'' TC1"; $resn_exch{" H4* TC1"}=" H4' TC1"; $resn_exch{" H3* TC1"}=" H3' TC1";
$resn_exch{"1H2* TC1"}=" H2' TC1"; $resn_exch{"2H2* TC1"}="H2'' TC1"; $resn_exch{" H1* TC1"}=" H1' TC1";
$resn_exch{"4H1  TC1"}=" H14 TC1"; $resn_exch{"5H   TC1"}=" H5  TC1"; $resn_exch{"7H   TC1"}=" H7  TC1";
$resn_exch{"8H   TC1"}=" H8  TC1"; $resn_exch{"9H   TC1"}=" H9  TC1"; $resn_exch{"0H1  TC1"}=" H10 TC1";
$resn_exch{"H3T  TC1"}="HO3' TC1"; $resn_exch{"3HOP TC1"}="HOP3 TC1"; $resn_exch{"1H1  TC4"}=" H11 TC4";
$resn_exch{"2H1  TC4"}=" H12 TC4"; $resn_exch{"1H2  TC4"}=" H21 TC4"; $resn_exch{"2H2  TC4"}=" H22 TC4";
$resn_exch{"1H3  TC4"}=" H31 TC4"; $resn_exch{"2H3  TC4"}=" H32 TC4"; $resn_exch{"1H4  TC4"}=" H41 TC4";
$resn_exch{"2H4  TC4"}=" H42 TC4"; $resn_exch{"3H4  TC4"}=" H43 TC4"; $resn_exch{"1H5  TC4"}=" H51 TC4";
$resn_exch{"2H5  TC4"}=" H52 TC4"; $resn_exch{"1H8  TC4"}=" H81 TC4"; $resn_exch{"2H8  TC4"}=" H82 TC4";
$resn_exch{"1H9  TC4"}=" H91 TC4"; $resn_exch{"2H9  TC4"}=" H92 TC4"; $resn_exch{"1H10 TC4"}="H101 TC4";
$resn_exch{"2H10 TC4"}="H102 TC4"; $resn_exch{"1H11 TC4"}="H111 TC4"; $resn_exch{"2H11 TC4"}="H112 TC4";
$resn_exch{"3H11 TC4"}="H113 TC4"; $resn_exch{"1H12 TC4"}="H121 TC4"; $resn_exch{"2H12 TC4"}="H122 TC4";
$resn_exch{"1H14 TC4"}="H141 TC4"; $resn_exch{"2H14 TC4"}="H142 TC4"; $resn_exch{"1H15 TC4"}="H151 TC4";
$resn_exch{"2H15 TC4"}="H152 TC4"; $resn_exch{"1H16 TC4"}="H161 TC4"; $resn_exch{"2H16 TC4"}="H162 TC4";
$resn_exch{"1H17 TC4"}="H171 TC4"; $resn_exch{"2H17 TC4"}="H172 TC4"; $resn_exch{"3H17 TC4"}="H173 TC4";
$resn_exch{"1H6  TCB"}=" H61 TCB"; $resn_exch{"2H6  TCB"}=" H62 TCB"; $resn_exch{"1H6' TCB"}="H6'1 TCB";
$resn_exch{"2H6' TCB"}="H6'2 TCB"; $resn_exch{"'HO1 TCB"}="HO1' TCB"; $resn_exch{"'HO2 TCB"}="HO2' TCB";
$resn_exch{"'HO3 TCB"}="HO3' TCB"; $resn_exch{"'HO6 TCB"}="HO6' TCB"; $resn_exch{"CH12 TCC"}="H12C TCC";
$resn_exch{"CH10 TCC"}="H10C TCC"; $resn_exch{"1H1  TCH"}=" H11 TCH"; $resn_exch{"2H1  TCH"}="H12A TCH";
$resn_exch{"1H2  TCH"}=" H21 TCH"; $resn_exch{"2H2  TCH"}=" H22 TCH"; $resn_exch{"1H4  TCH"}=" H41 TCH";
$resn_exch{"2H4  TCH"}=" H42 TCH"; $resn_exch{"1H6  TCH"}=" H61 TCH"; $resn_exch{"2H6  TCH"}=" H62 TCH";
$resn_exch{"1H11 TCH"}="H111 TCH"; $resn_exch{"2H11 TCH"}="H112 TCH"; $resn_exch{"1H15 TCH"}="H151 TCH";
$resn_exch{"2H15 TCH"}="H152 TCH"; $resn_exch{"1H16 TCH"}="H161 TCH"; $resn_exch{"2H16 TCH"}="H162 TCH";
$resn_exch{"1H18 TCH"}="H181 TCH"; $resn_exch{"2H18 TCH"}="H182 TCH"; $resn_exch{"3H18 TCH"}="H183 TCH";
$resn_exch{"1H19 TCH"}="H191 TCH"; $resn_exch{"2H19 TCH"}="H192 TCH"; $resn_exch{"3H19 TCH"}="H193 TCH";
$resn_exch{"1H21 TCH"}="H211 TCH"; $resn_exch{"2H21 TCH"}="H212 TCH"; $resn_exch{"3H21 TCH"}="H213 TCH";
$resn_exch{"1H22 TCH"}="H221 TCH"; $resn_exch{"2H22 TCH"}="H222 TCH"; $resn_exch{"1H23 TCH"}="H231 TCH";
$resn_exch{"2H23 TCH"}="H232 TCH"; $resn_exch{"1H25 TCH"}="H251 TCH"; $resn_exch{"2H25 TCH"}="H252 TCH";
$resn_exch{"1H26 TCH"}="H261 TCH"; $resn_exch{"2H26 TCH"}="H262 TCH"; $resn_exch{"1H1  TCK"}=" H11 TCK";
$resn_exch{"2H1  TCK"}=" H12 TCK"; $resn_exch{"3H1  TCK"}=" H13 TCK"; $resn_exch{"1H8  TCK"}=" H81 TCK";
$resn_exch{"2H8  TCK"}=" H82 TCK"; $resn_exch{"3H8  TCK"}=" H83 TCK"; $resn_exch{"1HB  TCK"}=" HB1 TCK";
$resn_exch{"2HB  TCK"}=" HB2 TCK"; $resn_exch{"1HG  TCK"}=" HG1 TCK"; $resn_exch{"2HG  TCK"}=" HG2 TCK";
$resn_exch{"1HD  TCK"}=" HD1 TCK"; $resn_exch{"2HD  TCK"}=" HD2 TCK"; $resn_exch{"1HE  TCK"}=" HE1 TCK";
$resn_exch{"2HE  TCK"}=" HE2 TCK"; $resn_exch{"1HNZ TCK"}="HNZ1 TCK"; $resn_exch{"2HNZ TCK"}="HNZ2 TCK";
$resn_exch{"1H1  TCL"}=" H11 TCL"; $resn_exch{"1H4  TCL"}=" H41 TCL"; $resn_exch{"1H3  TCL"}=" H31 TCL";
$resn_exch{"1H10 TCL"}="H101 TCL"; $resn_exch{"1H12 TCL"}="H121 TCL"; $resn_exch{"1H13 TCL"}="H131 TCL";
$resn_exch{"1H17 TCL"}="H171 TCL"; $resn_exch{"1H3  TCM"}=" H31 TCM"; $resn_exch{"2H3  TCM"}=" H32 TCM";
$resn_exch{"1H5  TCM"}=" H51 TCM"; $resn_exch{"2H5  TCM"}=" H52 TCM"; $resn_exch{"1H6  TCM"}=" H61 TCM";
$resn_exch{"2H6  TCM"}=" H62 TCM"; $resn_exch{"1H8  TCM"}=" H81 TCM"; $resn_exch{"2H8  TCM"}=" H82 TCM";
$resn_exch{"3H8  TCM"}=" H83 TCM"; $resn_exch{"1H9  TCM"}=" H91 TCM"; $resn_exch{"2H9  TCM"}=" H92 TCM";
$resn_exch{"3H9  TCM"}=" H93 TCM"; $resn_exch{"1H10 TCM"}="H101 TCM"; $resn_exch{"2H10 TCM"}="H102 TCM";
$resn_exch{"3H10 TCM"}="H103 TCM"; $resn_exch{"1HG2 TCO"}="HG21 TCO"; $resn_exch{"2HG2 TCO"}="HG22 TCO";
$resn_exch{"1HD2 TCO"}="HD21 TCO"; $resn_exch{"2HD2 TCO"}="HD22 TCO"; $resn_exch{"1HE  TCO"}=" HE1 TCO";
$resn_exch{"2HE  TCO"}=" HE2 TCO"; $resn_exch{"1HD1 TCO"}="HD11 TCO"; $resn_exch{"2HD1 TCO"}="HD12 TCO";
$resn_exch{"1HG1 TCO"}="HG11 TCO"; $resn_exch{"2HG1 TCO"}="HG12 TCO"; $resn_exch{"1H4  TCO"}=" H41 TCO";
$resn_exch{"2H4  TCO"}=" H42 TCO"; $resn_exch{"3H4  TCO"}=" H43 TCO"; $resn_exch{"1H6  TCO"}=" H61 TCO";
$resn_exch{"2H6  TCO"}=" H62 TCO"; $resn_exch{"3H6  TCO"}=" H63 TCO"; $resn_exch{"1H5  TCO"}=" H51 TCO";
$resn_exch{"2H5  TCO"}=" H52 TCO"; $resn_exch{"3H5  TCO"}=" H53 TCO"; $resn_exch{"1HP  TCP"}=" HP1 TCP";
$resn_exch{"2HP  TCP"}=" HP2 TCP"; $resn_exch{"3HP  TCP"}=" HP3 TCP"; $resn_exch{"1H5* TCP"}=" H5' TCP";
$resn_exch{"2H5* TCP"}="H5'' TCP"; $resn_exch{" H4* TCP"}=" H4' TCP"; $resn_exch{" H3* TCP"}=" H3' TCP";
$resn_exch{"*HO3 TCP"}="HO3' TCP"; $resn_exch{"1H2* TCP"}=" H2' TCP"; $resn_exch{"2H2* TCP"}="H2'' TCP";
$resn_exch{" H1* TCP"}=" H1' TCP"; $resn_exch{"1H5M TCP"}=" H71 TCP"; $resn_exch{"2H5M TCP"}=" H72 TCP";
$resn_exch{"3H5M TCP"}=" H73 TCP"; $resn_exch{"1HT2 TCQ"}="HT21 TCQ"; $resn_exch{"2HT2 TCQ"}="HT22 TCQ";
$resn_exch{"1HB  TCR"}=" HB1 TCR"; $resn_exch{"2HB  TCR"}=" HB2 TCR"; $resn_exch{"1H9  TCR"}=" H91 TCR";
$resn_exch{"2H9  TCR"}=" H92 TCR"; $resn_exch{"CH16 TCT"}="H16C TCT"; $resn_exch{"CH15 TCT"}="H15C TCT";
$resn_exch{"CH13 TCT"}="H13C TCT"; $resn_exch{"CH10 TCT"}="H10C TCT"; $resn_exch{"2H22 TDA"}="H222 TDA";
$resn_exch{"1H22 TDA"}="H221 TDA"; $resn_exch{"2H23 TDA"}="H232 TDA"; $resn_exch{"1H23 TDA"}="H231 TDA";
$resn_exch{"2H24 TDA"}="H242 TDA"; $resn_exch{"1H24 TDA"}="H241 TDA"; $resn_exch{"2H25 TDA"}="H252 TDA";
$resn_exch{"1H25 TDA"}="H251 TDA"; $resn_exch{"2H26 TDA"}="H262 TDA"; $resn_exch{"1H26 TDA"}="H261 TDA";
$resn_exch{"2H27 TDA"}="H272 TDA"; $resn_exch{"1H27 TDA"}="H271 TDA"; $resn_exch{"2H28 TDA"}="H282 TDA";
$resn_exch{"1H28 TDA"}="H281 TDA"; $resn_exch{"2H29 TDA"}="H292 TDA"; $resn_exch{"1H29 TDA"}="H291 TDA";
$resn_exch{"2H30 TDA"}="H302 TDA"; $resn_exch{"1H30 TDA"}="H301 TDA"; $resn_exch{"2H31 TDA"}="H312 TDA";
$resn_exch{"1H31 TDA"}="H311 TDA"; $resn_exch{"2H32 TDA"}="H322 TDA"; $resn_exch{"1H32 TDA"}="H321 TDA";
$resn_exch{"3H33 TDA"}="H333 TDA"; $resn_exch{"2H33 TDA"}="H332 TDA"; $resn_exch{"1H33 TDA"}="H331 TDA";
$resn_exch{"1H8  TDB"}=" H81 TDB"; $resn_exch{"2H8  TDB"}=" H82 TDB"; $resn_exch{"3H8  TDB"}=" H83 TDB";
$resn_exch{"1H15 TDB"}="H151 TDB"; $resn_exch{"2H15 TDB"}="H152 TDB"; $resn_exch{"1H16 TDB"}="H161 TDB";
$resn_exch{"2H16 TDB"}="H162 TDB"; $resn_exch{"1H17 TDB"}="H171 TDB"; $resn_exch{"2H17 TDB"}="H172 TDB";
$resn_exch{"3H17 TDB"}="H173 TDB"; $resn_exch{"1H62 TDC"}="H621 TDC"; $resn_exch{"2H62 TDC"}="H622 TDC";
$resn_exch{"3H62 TDC"}="H623 TDC"; $resn_exch{"1H5C TDC"}="H5C1 TDC"; $resn_exch{"2H5C TDC"}="H5C2 TDC";
$resn_exch{"1H43 TDC"}="H431 TDC"; $resn_exch{"2H43 TDC"}="H432 TDC"; $resn_exch{"3H43 TDC"}="H433 TDC";
$resn_exch{"1H42 TDC"}="H421 TDC"; $resn_exch{"2H42 TDC"}="H422 TDC"; $resn_exch{"3H42 TDC"}="H423 TDC";
$resn_exch{"1H21 TDC"}="H211 TDC"; $resn_exch{"2H21 TDC"}="H212 TDC"; $resn_exch{"1H4  TDE"}=" H41 TDE";
$resn_exch{"2H4  TDE"}=" H42 TDE"; $resn_exch{"1H5  TDE"}=" H51 TDE"; $resn_exch{"2H5  TDE"}=" H52 TDE";
$resn_exch{"1H7  TDE"}=" H71 TDE"; $resn_exch{"2H7  TDE"}=" H72 TDE"; $resn_exch{"1H25 TDE"}="H251 TDE";
$resn_exch{"2H25 TDE"}="H252 TDE"; $resn_exch{"3H25 TDE"}="H253 TDE"; $resn_exch{"1H21 TDE"}="H211 TDE";
$resn_exch{"2H21 TDE"}="H212 TDE"; $resn_exch{"3H21 TDE"}="H213 TDE"; $resn_exch{"1H20 TDE"}="H201 TDE";
$resn_exch{"2H20 TDE"}="H202 TDE"; $resn_exch{"3H20 TDE"}="H203 TDE"; $resn_exch{"1H19 TDE"}="H191 TDE";
$resn_exch{"2H19 TDE"}="H192 TDE"; $resn_exch{"3H19 TDE"}="H193 TDE"; $resn_exch{"1H6  TDG"}=" H61 TDG";
$resn_exch{"2H6  TDG"}=" H62 TDG"; $resn_exch{"'HO2 TDG"}="HO2' TDG"; $resn_exch{"'HO3 TDG"}="HO3' TDG";
$resn_exch{"'HO4 TDG"}="HO4' TDG"; $resn_exch{"1H6' TDG"}="H6'1 TDG"; $resn_exch{"2H6' TDG"}="H6'2 TDG";
$resn_exch{"'HO6 TDG"}="HO6' TDG"; $resn_exch{"1HS5 TDI"}="HS51 TDI"; $resn_exch{"2HS5 TDI"}="HS52 TDI";
$resn_exch{"3HS5 TDI"}="HS53 TDI"; $resn_exch{"1H5* TDI"}="H5'1 TDI"; $resn_exch{"2H5* TDI"}="H5'2 TDI";
$resn_exch{" H4* TDI"}=" H4' TDI"; $resn_exch{"1H1* TDI"}="H1'1 TDI"; $resn_exch{"2H1* TDI"}="H1'2 TDI";
$resn_exch{"1H10 TDI"}="H101 TDI"; $resn_exch{"2H10 TDI"}="H102 TDI"; $resn_exch{"1HN6 TDI"}="HN61 TDI";
$resn_exch{"2HN6 TDI"}="HN62 TDI"; $resn_exch{"1H2* TDI"}="H2'1 TDI"; $resn_exch{"2H2* TDI"}="H2'2 TDI";
$resn_exch{" H3* TDI"}=" H3' TDI"; $resn_exch{"1H7  TDK"}=" H71 TDK"; $resn_exch{"2H7  TDK"}=" H72 TDK";
$resn_exch{"1H6  TDK"}=" H61 TDK"; $resn_exch{"2H6  TDK"}=" H62 TDK"; $resn_exch{"1HM4 TDK"}="HM41 TDK";
$resn_exch{"2HM4 TDK"}="HM42 TDK"; $resn_exch{"3HM4 TDK"}="HM43 TDK"; $resn_exch{"1HMB TDK"}="HMB1 TDK";
$resn_exch{"2HMB TDK"}="HMB2 TDK"; $resn_exch{"3HMB TDK"}="HMB3 TDK"; $resn_exch{"1HMC TDK"}="HMC1 TDK";
$resn_exch{"2HMC TDK"}="HMC2 TDK"; $resn_exch{"3HMC TDK"}="HMC3 TDK"; $resn_exch{"1H7* TDK"}="H7'1 TDK";
$resn_exch{"2H7* TDK"}="H7'2 TDK"; $resn_exch{" H6* TDK"}=" H6' TDK"; $resn_exch{"1HM2 TDK"}="HM21 TDK";
$resn_exch{"2HM2 TDK"}="HM22 TDK"; $resn_exch{"3HM2 TDK"}="HM23 TDK"; $resn_exch{"1H4* TDK"}="H4'1 TDK";
$resn_exch{"2H4* TDK"}="H4'2 TDK"; $resn_exch{"1H7  TDL"}=" H71 TDL"; $resn_exch{"2H7  TDL"}=" H72 TDL";
$resn_exch{"1H6  TDL"}=" H61 TDL"; $resn_exch{"2H6  TDL"}=" H62 TDL"; $resn_exch{"1HM4 TDL"}="HM41 TDL";
$resn_exch{"2HM4 TDL"}="HM42 TDL"; $resn_exch{"3HM4 TDL"}="HM43 TDL"; $resn_exch{"1HLB TDL"}="HLB1 TDL";
$resn_exch{"2HLB TDL"}="HLB2 TDL"; $resn_exch{"3HLB TDL"}="HLB3 TDL"; $resn_exch{"1H7* TDL"}="H7'1 TDL";
$resn_exch{"2H7* TDL"}="H7'2 TDL"; $resn_exch{" H6* TDL"}=" H6' TDL"; $resn_exch{"1HM2 TDL"}="HM21 TDL";
$resn_exch{"2HM2 TDL"}="HM22 TDL"; $resn_exch{"3HM2 TDL"}="HM23 TDL"; $resn_exch{"1H4* TDL"}="H4'1 TDL";
$resn_exch{"2H4* TDL"}="H4'2 TDL"; $resn_exch{"1H7  TDM"}=" H71 TDM"; $resn_exch{"2H7  TDM"}=" H72 TDM";
$resn_exch{"1H6  TDM"}=" H61 TDM"; $resn_exch{"2H6  TDM"}=" H62 TDM"; $resn_exch{"1H29 TDM"}="H291 TDM";
$resn_exch{"2H29 TDM"}="H292 TDM"; $resn_exch{"3H29 TDM"}="H293 TDM"; $resn_exch{"1HM4 TDM"}="HM41 TDM";
$resn_exch{"2HM4 TDM"}="HM42 TDM"; $resn_exch{"3HM4 TDM"}="HM43 TDM"; $resn_exch{"1H7* TDM"}="H7'1 TDM";
$resn_exch{"2H7* TDM"}="H7'2 TDM"; $resn_exch{"1H4* TDM"}="H4'1 TDM"; $resn_exch{"2H4* TDM"}="H4'2 TDM";
$resn_exch{"1HM2 TDM"}="HM21 TDM"; $resn_exch{"2HM2 TDM"}="HM22 TDM"; $resn_exch{"3HM2 TDM"}="HM23 TDM";
$resn_exch{" H6* TDM"}=" H6' TDM"; $resn_exch{"OH3P TDO"}="H3PO TDO"; $resn_exch{"1H6C TDO"}="H6C1 TDO";
$resn_exch{"2H6C TDO"}="H6C2 TDO"; $resn_exch{"3H6C TDO"}="H6C3 TDO"; $resn_exch{"OH2P TDO"}="H2PO TDO";
$resn_exch{"1H5' TDO"}="H5'1 TDO"; $resn_exch{"2H5' TDO"}="H5'2 TDO"; $resn_exch{"1H2' TDO"}="H2'1 TDO";
$resn_exch{"2H2' TDO"}="H2'2 TDO"; $resn_exch{"1H5A TDO"}="H5A1 TDO"; $resn_exch{"2H5A TDO"}="H5A2 TDO";
$resn_exch{"3H5A TDO"}="H5A3 TDO"; $resn_exch{"1H2A TDP"}="H2A1 TDP"; $resn_exch{"2H2A TDP"}="H2A2 TDP";
$resn_exch{"3H2A TDP"}="H2A3 TDP"; $resn_exch{"1H4* TDP"}="H4'1 TDP"; $resn_exch{"2H4* TDP"}="H4'2 TDP";
$resn_exch{" H6* TDP"}=" H6' TDP"; $resn_exch{"1H35 TDP"}="H351 TDP"; $resn_exch{"2H35 TDP"}="H352 TDP";
$resn_exch{"1H4A TDP"}="H4A1 TDP"; $resn_exch{"2H4A TDP"}="H4A2 TDP"; $resn_exch{"3H4A TDP"}="H4A3 TDP";
$resn_exch{"1H5A TDP"}="H5A1 TDP"; $resn_exch{"2H5A TDP"}="H5A2 TDP"; $resn_exch{"1H5B TDP"}="H5B1 TDP";
$resn_exch{"2H5B TDP"}="H5B2 TDP"; $resn_exch{"1H5M TDR"}="H5M1 TDR"; $resn_exch{"2H5M TDR"}="H5M2 TDR";
$resn_exch{"3H5M TDR"}="H5M3 TDR"; $resn_exch{"1HBC TDS"}="HBC1 TDS"; $resn_exch{"2HBC TDS"}="HBC2 TDS";
$resn_exch{"3HBC TDS"}="HBC3 TDS"; $resn_exch{"1HBB TDS"}="HBB1 TDS"; $resn_exch{"2HBB TDS"}="HBB2 TDS";
$resn_exch{"1HBA TDS"}="HBA1 TDS"; $resn_exch{"2HBA TDS"}="HBA2 TDS"; $resn_exch{"1HAZ TDS"}="HAZ1 TDS";
$resn_exch{"2HAZ TDS"}="HAZ2 TDS"; $resn_exch{"1HAY TDS"}="HAY1 TDS"; $resn_exch{"2HAY TDS"}="HAY2 TDS";
$resn_exch{"1HAX TDS"}="HAX1 TDS"; $resn_exch{"2HAX TDS"}="HAX2 TDS"; $resn_exch{"1HAW TDS"}="HAW1 TDS";
$resn_exch{"2HAW TDS"}="HAW2 TDS"; $resn_exch{"1HAV TDS"}="HAV1 TDS"; $resn_exch{"2HAV TDS"}="HAV2 TDS";
$resn_exch{"1HAU TDS"}="HAU1 TDS"; $resn_exch{"2HAU TDS"}="HAU2 TDS"; $resn_exch{"1HAT TDS"}="HAT1 TDS";
$resn_exch{"2HAT TDS"}="HAT2 TDS"; $resn_exch{"1HAS TDS"}="HAS1 TDS"; $resn_exch{"2HAS TDS"}="HAS2 TDS";
$resn_exch{"1HAR TDS"}="HAR1 TDS"; $resn_exch{"2HAR TDS"}="HAR2 TDS"; $resn_exch{"1HAQ TDS"}="HAQ1 TDS";
$resn_exch{"2HAQ TDS"}="HAQ2 TDS"; $resn_exch{"1HAI TDS"}="HAI1 TDS"; $resn_exch{"2HAI TDS"}="HAI2 TDS";
$resn_exch{"3HAI TDS"}="HAI3 TDS"; $resn_exch{"1HAJ TDS"}="HAJ1 TDS"; $resn_exch{"2HAJ TDS"}="HAJ2 TDS";
$resn_exch{"3HAJ TDS"}="HAJ3 TDS"; $resn_exch{"1HAA TDS"}="HAA1 TDS"; $resn_exch{"2HAA TDS"}="HAA2 TDS";
$resn_exch{"3HAA TDS"}="HAA3 TDS"; $resn_exch{"1H5M TDT"}="H5M1 TDT"; $resn_exch{"2H5M TDT"}="H5M2 TDT";
$resn_exch{"1H7N TDT"}="H7N1 TDT"; $resn_exch{"2H7N TDT"}="H7N2 TDT"; $resn_exch{"1H5B TDT"}="H5B1 TDT";
$resn_exch{"2H5B TDT"}="H5B2 TDT"; $resn_exch{"1H6A TDT"}="H6A1 TDT"; $resn_exch{"2H6A TDT"}="H6A2 TDT";
$resn_exch{"1H2  TDT"}=" H21 TDT"; $resn_exch{"2H2  TDT"}=" H22 TDT"; $resn_exch{"1H3  TDT"}=" H31 TDT";
$resn_exch{"2H3  TDT"}=" H32 TDT"; $resn_exch{"1H4  TDT"}=" H41 TDT"; $resn_exch{"2H4  TDT"}=" H42 TDT";
$resn_exch{"1H5  TDT"}=" H51 TDT"; $resn_exch{"2H5  TDT"}=" H52 TDT"; $resn_exch{"1H6  TDT"}=" H61 TDT";
$resn_exch{"2H6  TDT"}=" H62 TDT"; $resn_exch{"1H7  TDT"}=" H71 TDT"; $resn_exch{"2H7  TDT"}=" H72 TDT";
$resn_exch{"1H5M TDX"}="H5M1 TDX"; $resn_exch{"2H5M TDX"}="H5M2 TDX"; $resn_exch{"3H5M TDX"}="H5M3 TDX";
$resn_exch{" H1* TDX"}=" H1B TDX"; $resn_exch{"1H2* TDX"}="H2'1 TDX"; $resn_exch{"2H2* TDX"}="H2'2 TDX";
$resn_exch{" H3* TDX"}=" H3B TDX"; $resn_exch{"*HO3 TDX"}="HO3' TDX"; $resn_exch{" H4* TDX"}=" H4B TDX";
$resn_exch{"1H5* TDX"}="H5'1 TDX"; $resn_exch{"2H5* TDX"}="H5'2 TDX"; $resn_exch{"1H5' TDX"}=" H5B TDX";
$resn_exch{"2H5' TDX"}=" H5D TDX"; $resn_exch{"'HO2 TDX"}="HO2' TDX"; $resn_exch{"'HO3 TDX"}="HO3A TDX";
$resn_exch{"'HO4 TDX"}="HO4' TDX"; $resn_exch{"1HAM TDZ"}="HAM1 TDZ"; $resn_exch{"2HAM TDZ"}="HAM2 TDZ";
$resn_exch{"1HAL TDZ"}="HAL1 TDZ"; $resn_exch{"2HAL TDZ"}="HAL2 TDZ"; $resn_exch{"1HAC TDZ"}="HAC1 TDZ";
$resn_exch{"2HAC TDZ"}="HAC2 TDZ"; $resn_exch{"3HAC TDZ"}="HAC3 TDZ"; $resn_exch{"1HAA TDZ"}="HAA1 TDZ";
$resn_exch{"2HAA TDZ"}="HAA2 TDZ"; $resn_exch{"3HAA TDZ"}="HAA3 TDZ"; $resn_exch{"1HAB TDZ"}="HAB1 TDZ";
$resn_exch{"2HAB TDZ"}="HAB2 TDZ"; $resn_exch{"3HAB TDZ"}="HAB3 TDZ"; $resn_exch{"1HAD TDZ"}="HAD1 TDZ";
$resn_exch{"2HAD TDZ"}="HAD2 TDZ"; $resn_exch{"3HAD TDZ"}="HAD3 TDZ"; $resn_exch{"1HAN TDZ"}="HAN1 TDZ";
$resn_exch{"2HAN TDZ"}="HAN2 TDZ"; $resn_exch{"1HAO TDZ"}="HAO1 TDZ"; $resn_exch{"2HAO TDZ"}="HAO2 TDZ";
$resn_exch{"1H11 TEA"}="H111 TEA"; $resn_exch{"2H11 TEA"}="H112 TEA"; $resn_exch{"1H12 TEA"}="H121 TEA";
$resn_exch{"2H12 TEA"}="H122 TEA"; $resn_exch{"3H12 TEA"}="H123 TEA"; $resn_exch{"1H21 TEA"}="H211 TEA";
$resn_exch{"2H21 TEA"}="H212 TEA"; $resn_exch{"1H22 TEA"}="H221 TEA"; $resn_exch{"2H22 TEA"}="H222 TEA";
$resn_exch{"3H22 TEA"}="H223 TEA"; $resn_exch{"1H31 TEA"}="H311 TEA"; $resn_exch{"2H31 TEA"}="H312 TEA";
$resn_exch{"1H32 TEA"}="H321 TEA"; $resn_exch{"2H32 TEA"}="H322 TEA"; $resn_exch{"3H32 TEA"}="H323 TEA";
$resn_exch{"1HN  TEE"}=" HN1 TEE"; $resn_exch{"2HN  TEE"}=" HN2 TEE"; $resn_exch{"1H15 TEI"}="H151 TEI";
$resn_exch{"2H15 TEI"}="H152 TEI"; $resn_exch{"1H17 TEI"}="H171 TEI"; $resn_exch{"2H17 TEI"}="H172 TEI";
$resn_exch{"3H17 TEI"}="H173 TEI"; $resn_exch{"1H18 TEI"}="H181 TEI"; $resn_exch{"2H18 TEI"}="H182 TEI";
$resn_exch{"3H18 TEI"}="H183 TEI"; $resn_exch{"1H19 TEI"}="H191 TEI"; $resn_exch{"2H19 TEI"}="H192 TEI";
$resn_exch{"3H19 TEI"}="H193 TEI"; $resn_exch{"1H1  TEL"}=" H11 TEL"; $resn_exch{"2H1  TEL"}=" H12 TEL";
$resn_exch{"3H1  TEL"}=" H13 TEL"; $resn_exch{"1H8  TEL"}=" H81 TEL"; $resn_exch{"2H8  TEL"}=" H82 TEL";
$resn_exch{"1H11 TEL"}="H111 TEL"; $resn_exch{"2H11 TEL"}="H112 TEL"; $resn_exch{"1H12 TEL"}="H121 TEL";
$resn_exch{"2H12 TEL"}="H122 TEL"; $resn_exch{"3H12 TEL"}="H123 TEL"; $resn_exch{"1H14 TEL"}="H141 TEL";
$resn_exch{"2H14 TEL"}="H142 TEL"; $resn_exch{"3H14 TEL"}="H143 TEL"; $resn_exch{"1H17 TEL"}="H171 TEL";
$resn_exch{"2H17 TEL"}="H172 TEL"; $resn_exch{"1H22 TEL"}="H221 TEL"; $resn_exch{"2H22 TEL"}="H222 TEL";
$resn_exch{"1H23 TEL"}="H231 TEL"; $resn_exch{"2H23 TEL"}="H232 TEL"; $resn_exch{"3H23 TEL"}="H233 TEL";
$resn_exch{"1H24 TEL"}="H241 TEL"; $resn_exch{"2H24 TEL"}="H242 TEL"; $resn_exch{"1H25 TEL"}="H251 TEL";
$resn_exch{"2H25 TEL"}="H252 TEL"; $resn_exch{"3H25 TEL"}="H253 TEL"; $resn_exch{"1H27 TEL"}="H271 TEL";
$resn_exch{"2H27 TEL"}="H272 TEL"; $resn_exch{"1H33 TEL"}="H331 TEL"; $resn_exch{"2H33 TEL"}="H332 TEL";
$resn_exch{"3H33 TEL"}="H333 TEL"; $resn_exch{"1H35 TEL"}="H351 TEL"; $resn_exch{"2H35 TEL"}="H352 TEL";
$resn_exch{"3H35 TEL"}="H353 TEL"; $resn_exch{"1H38 TEL"}="H381 TEL"; $resn_exch{"2H38 TEL"}="H382 TEL";
$resn_exch{"3H38 TEL"}="H383 TEL"; $resn_exch{"1H54 TEL"}="H541 TEL"; $resn_exch{"2H54 TEL"}="H542 TEL";
$resn_exch{"1H55 TEL"}="H551 TEL"; $resn_exch{"2H55 TEL"}="H552 TEL"; $resn_exch{"3H55 TEL"}="H553 TEL";
$resn_exch{"1H57 TEL"}="H571 TEL"; $resn_exch{"2H57 TEL"}="H572 TEL"; $resn_exch{"3H57 TEL"}="H573 TEL";
$resn_exch{"1H58 TEL"}="H581 TEL"; $resn_exch{"2H58 TEL"}="H582 TEL"; $resn_exch{"3H58 TEL"}="H583 TEL";
$resn_exch{"1H3  TEM"}=" H31 TEM"; $resn_exch{"2H3  TEM"}=" H32 TEM"; $resn_exch{"1H9  TEM"}=" H91 TEM";
$resn_exch{"2H9  TEM"}=" H92 TEM"; $resn_exch{"4HOB TEO"}="HOB4 TEO"; $resn_exch{"1H1  TEP"}=" H11 TEP";
$resn_exch{"2H1  TEP"}=" H12 TEP"; $resn_exch{"3H1  TEP"}=" H13 TEP"; $resn_exch{"1H3  TEP"}=" H31 TEP";
$resn_exch{"2H3  TEP"}=" H32 TEP"; $resn_exch{"3H3  TEP"}=" H33 TEP"; $resn_exch{"1HN1 TER"}="HN11 TER";
$resn_exch{"2HN1 TER"}="HN12 TER"; $resn_exch{"1H2  TER"}=" H21 TER"; $resn_exch{"2H2  TER"}=" H22 TER";
$resn_exch{"1H3  TER"}=" H31 TER"; $resn_exch{"2H3  TER"}=" H32 TER"; $resn_exch{"1H4  TER"}=" H41 TER";
$resn_exch{"2H4  TER"}=" H42 TER"; $resn_exch{"1H6  TER"}=" H61 TER"; $resn_exch{"2H6  TER"}=" H62 TER";
$resn_exch{"1H7  TER"}=" H71 TER"; $resn_exch{"2H7  TER"}=" H72 TER"; $resn_exch{"1H8  TER"}=" H81 TER";
$resn_exch{"2H8  TER"}=" H82 TER"; $resn_exch{"1H10 TER"}="H101 TER"; $resn_exch{"2H10 TER"}="H102 TER";
$resn_exch{"1H11 TER"}="H111 TER"; $resn_exch{"2H11 TER"}="H112 TER"; $resn_exch{"1H12 TER"}="H121 TER";
$resn_exch{"2H12 TER"}="H122 TER"; $resn_exch{"1H13 TER"}="H131 TER"; $resn_exch{"2H13 TER"}="H132 TER";
$resn_exch{"1HN4 TER"}="HN41 TER"; $resn_exch{"2HN4 TER"}="HN42 TER"; $resn_exch{"1H1  TES"}=" H11 TES";
$resn_exch{"2H1  TES"}=" H12 TES"; $resn_exch{"1H2  TES"}=" H21 TES"; $resn_exch{"2H2  TES"}=" H22 TES";
$resn_exch{"1H6  TES"}=" H61 TES"; $resn_exch{"2H6  TES"}=" H62 TES"; $resn_exch{"1H7  TES"}=" H71 TES";
$resn_exch{"2H7  TES"}=" H72 TES"; $resn_exch{"1H11 TES"}="H111 TES"; $resn_exch{"2H11 TES"}="H112 TES";
$resn_exch{"1H12 TES"}="H121 TES"; $resn_exch{"2H12 TES"}="H122 TES"; $resn_exch{"1H15 TES"}="H151 TES";
$resn_exch{"2H15 TES"}="H152 TES"; $resn_exch{"1H16 TES"}="H161 TES"; $resn_exch{"2H16 TES"}="H162 TES";
$resn_exch{"1H18 TES"}="H181 TES"; $resn_exch{"2H18 TES"}="H182 TES"; $resn_exch{"3H18 TES"}="H183 TES";
$resn_exch{"1H19 TES"}="H191 TES"; $resn_exch{"2H19 TES"}="H192 TES"; $resn_exch{"3H19 TES"}="H193 TES";
$resn_exch{"1H12 TET"}="H121 TET"; $resn_exch{"2H12 TET"}="H122 TET"; $resn_exch{"1H17 TET"}="H171 TET";
$resn_exch{"2H17 TET"}="H172 TET"; $resn_exch{"1H8  TET"}=" H81 TET"; $resn_exch{"2H8  TET"}=" H82 TET";
$resn_exch{"3H8  TET"}=" H83 TET"; $resn_exch{"1H19 TET"}="H191 TET"; $resn_exch{"2H19 TET"}="H192 TET";
$resn_exch{"1H3C TF1"}="H3C1 TF1"; $resn_exch{"2H3C TF1"}="H3C2 TF1"; $resn_exch{"1H5C TF1"}="H5C1 TF1";
$resn_exch{"2H5C TF1"}="H5C2 TF1"; $resn_exch{"1H6C TF1"}="H6C1 TF1"; $resn_exch{"2H6C TF1"}="H6C2 TF1";
$resn_exch{"3H6C TF1"}="H6C3 TF1"; $resn_exch{"1H9C TF1"}="H9C1 TF1"; $resn_exch{"2H9C TF1"}="H9C2 TF1";
$resn_exch{"3H9C TF1"}="H9C3 TF1"; $resn_exch{"1H10 TF1"}="H101 TF1"; $resn_exch{"2H10 TF1"}="H102 TF1";
$resn_exch{"3H10 TF1"}="H103 TF1"; $resn_exch{"1H7C TF2"}="H7C1 TF2"; $resn_exch{"2H7C TF2"}="H7C2 TF2";
$resn_exch{"1H9C TF2"}="H9C1 TF2"; $resn_exch{"2H9C TF2"}="H9C2 TF2"; $resn_exch{"3H9C TF2"}="H9C3 TF2";
$resn_exch{"1H12 TF2"}="H121 TF2"; $resn_exch{"2H12 TF2"}="H122 TF2"; $resn_exch{"3H12 TF2"}="H123 TF2";
$resn_exch{"1H19 TF2"}="H191 TF2"; $resn_exch{"2H19 TF2"}="H192 TF2"; $resn_exch{"3H19 TF2"}="H193 TF2";
$resn_exch{"1H6C TF3"}="H6C1 TF3"; $resn_exch{"2H6C TF3"}="H6C2 TF3"; $resn_exch{"1H9C TF3"}="H9C1 TF3";
$resn_exch{"2H9C TF3"}="H9C2 TF3"; $resn_exch{"1H16 TF3"}="H161 TF3"; $resn_exch{"2H16 TF3"}="H162 TF3";
$resn_exch{"1H18 TF3"}="H181 TF3"; $resn_exch{"2H18 TF3"}="H182 TF3"; $resn_exch{"1H19 TF3"}="H191 TF3";
$resn_exch{"2H19 TF3"}="H192 TF3"; $resn_exch{"3H19 TF3"}="H193 TF3"; $resn_exch{"1H20 TF3"}="H201 TF3";
$resn_exch{"2H20 TF3"}="H202 TF3"; $resn_exch{"3H20 TF3"}="H203 TF3"; $resn_exch{"1H2N TF3"}="H2N1 TF3";
$resn_exch{"2H2N TF3"}="H2N2 TF3"; $resn_exch{"1H17 TF5"}="H171 TF5"; $resn_exch{"2H17 TF5"}="H172 TF5";
$resn_exch{"1H16 TF5"}="H161 TF5"; $resn_exch{"2H16 TF5"}="H162 TF5"; $resn_exch{"1HB  TFB"}=" HB1 TFB";
$resn_exch{"2HB  TFB"}=" HB2 TFB"; $resn_exch{"1HG  TFB"}=" HG1 TFB"; $resn_exch{"2HG  TFB"}=" HG2 TFB";
$resn_exch{"1HD  TFB"}=" HD1 TFB"; $resn_exch{"2HD  TFB"}=" HD2 TFB"; $resn_exch{"1H1  TFC"}=" H11 TFC";
$resn_exch{"2H1  TFC"}=" H12 TFC"; $resn_exch{"3H1  TFC"}=" H13 TFC"; $resn_exch{"1H2  TFC"}=" H21 TFC";
$resn_exch{"2H2  TFC"}=" H22 TFC"; $resn_exch{"1H3  TFC"}=" H31 TFC"; $resn_exch{"2H3  TFC"}=" H32 TFC";
$resn_exch{"1H4  TFC"}=" H41 TFC"; $resn_exch{"2H4  TFC"}=" H42 TFC"; $resn_exch{"1H5  TFC"}=" H51 TFC";
$resn_exch{"2H5  TFC"}=" H52 TFC"; $resn_exch{"1H6  TFC"}=" H61 TFC"; $resn_exch{"2H6  TFC"}=" H62 TFC";
$resn_exch{"1H7  TFC"}=" H71 TFC"; $resn_exch{"2H7  TFC"}=" H72 TFC"; $resn_exch{"1H8  TFC"}=" H81 TFC";
$resn_exch{"2H8  TFC"}=" H82 TFC"; $resn_exch{"1H9  TFC"}=" H91 TFC"; $resn_exch{"2H9  TFC"}=" H92 TFC";
$resn_exch{"2HOP TFE"}="HOP2 TFE"; $resn_exch{"1H5* TFE"}=" H5' TFE"; $resn_exch{"2H5* TFE"}="H5'' TFE";
$resn_exch{" H4* TFE"}=" H4' TFE"; $resn_exch{" H1* TFE"}=" H1' TFE"; $resn_exch{"1H5M TFE"}=" H71 TFE";
$resn_exch{"2H5M TFE"}=" H72 TFE"; $resn_exch{"3H5M TFE"}=" H73 TFE"; $resn_exch{" H2* TFE"}=" H2' TFE";
$resn_exch{"1HB' TFE"}="HB'1 TFE"; $resn_exch{"2HB' TFE"}="HB'2 TFE"; $resn_exch{" H3* TFE"}=" H3' TFE";
$resn_exch{"H3T  TFE"}="HO3' TFE"; $resn_exch{"3HOP TFE"}="HOP3 TFE"; $resn_exch{"1H5  TFI"}=" H51 TFI";
$resn_exch{"2H5  TFI"}=" H52 TFI"; $resn_exch{"3H5  TFI"}=" H53 TFI"; $resn_exch{"1H6  TFI"}=" H61 TFI";
$resn_exch{"2H6  TFI"}=" H62 TFI"; $resn_exch{"3H6  TFI"}=" H63 TFI"; $resn_exch{"1H8  TFI"}=" H81 TFI";
$resn_exch{"2H8  TFI"}=" H82 TFI"; $resn_exch{"1H19 TFI"}="H191 TFI"; $resn_exch{"2H19 TFI"}="H192 TFI";
$resn_exch{"3H19 TFI"}="H193 TFI"; $resn_exch{"1HN5 TFI"}="HN51 TFI"; $resn_exch{"2HN5 TFI"}="HN52 TFI";
$resn_exch{"1H5  TFK"}=" H51 TFK"; $resn_exch{"2H5  TFK"}=" H52 TFK"; $resn_exch{"3H5  TFK"}=" H53 TFK";
$resn_exch{"1H6  TFK"}=" H61 TFK"; $resn_exch{"2H6  TFK"}=" H62 TFK"; $resn_exch{"3H6  TFK"}=" H63 TFK";
$resn_exch{"1H8  TFK"}=" H81 TFK"; $resn_exch{"2H8  TFK"}=" H82 TFK"; $resn_exch{"1H20 TFK"}="H201 TFK";
$resn_exch{"2H20 TFK"}="H202 TFK"; $resn_exch{"3H20 TFK"}="H203 TFK"; $resn_exch{"1H15 TFL"}="H151 TFL";
$resn_exch{"2H15 TFL"}="H152 TFL"; $resn_exch{"3H15 TFL"}="H153 TFL"; $resn_exch{"1H16 TFL"}="H161 TFL";
$resn_exch{"2H16 TFL"}="H162 TFL"; $resn_exch{"3H16 TFL"}="H163 TFL"; $resn_exch{"1H17 TFL"}="H171 TFL";
$resn_exch{"2H17 TFL"}="H172 TFL"; $resn_exch{"3H17 TFL"}="H173 TFL"; $resn_exch{"1HN1 TFM"}="HN11 TFM";
$resn_exch{"2HN1 TFM"}="HN12 TFM"; $resn_exch{"1H2  TFM"}=" H21 TFM"; $resn_exch{"1H3  TFM"}=" H31 TFM";
$resn_exch{"1H5  TFM"}=" H51 TFM"; $resn_exch{"1H6  TFM"}=" H61 TFM"; $resn_exch{"1H1' TFM"}="H1'1 TFM";
$resn_exch{"2H1' TFM"}="H1'2 TFM"; $resn_exch{"1H2' TFM"}="H2'1 TFM"; $resn_exch{"2H2' TFM"}="H2'2 TFM";
$resn_exch{"3H2' TFM"}="H2'3 TFM"; $resn_exch{"2HOP TFO"}="HOP2 TFO"; $resn_exch{"1H9' TFO"}="H9'1 TFO";
$resn_exch{"2H9' TFO"}="H9'2 TFO"; $resn_exch{"1H8' TFO"}="H8'1 TFO"; $resn_exch{"2H8' TFO"}="H8'2 TFO";
$resn_exch{"3H8' TFO"}="H8'3 TFO"; $resn_exch{"1H6' TFO"}="H6'1 TFO"; $resn_exch{"2H6' TFO"}="H6'2 TFO";
$resn_exch{"1HN6 TFO"}="HN61 TFO"; $resn_exch{"2HN6 TFO"}="HN62 TFO"; $resn_exch{"3HOP TFO"}="HOP3 TFO";
$resn_exch{"1H13 TFP"}="H131 TFP"; $resn_exch{"2H13 TFP"}="H132 TFP"; $resn_exch{"1H14 TFP"}="H141 TFP";
$resn_exch{"2H14 TFP"}="H142 TFP"; $resn_exch{"1H15 TFP"}="H151 TFP"; $resn_exch{"2H15 TFP"}="H152 TFP";
$resn_exch{"1H16 TFP"}="H161 TFP"; $resn_exch{"2H16 TFP"}="H162 TFP"; $resn_exch{"1H17 TFP"}="H171 TFP";
$resn_exch{"2H17 TFP"}="H172 TFP"; $resn_exch{"1H18 TFP"}="H181 TFP"; $resn_exch{"2H18 TFP"}="H182 TFP";
$resn_exch{"1H19 TFP"}="H191 TFP"; $resn_exch{"2H19 TFP"}="H192 TFP"; $resn_exch{"1H20 TFP"}="H201 TFP";
$resn_exch{"2H20 TFP"}="H202 TFP"; $resn_exch{"3H20 TFP"}="H203 TFP"; $resn_exch{"1H13 TFQ"}="H131 TFQ";
$resn_exch{"2H13 TFQ"}="H132 TFQ"; $resn_exch{"1H7  TFQ"}=" H71 TFQ"; $resn_exch{"2H7  TFQ"}=" H72 TFQ";
$resn_exch{"1HN9 TFQ"}="HN91 TFQ"; $resn_exch{"2HN9 TFQ"}="HN92 TFQ"; $resn_exch{"2HOP TFT"}="HOP2 TFT";
$resn_exch{"1H5M TFT"}=" H71 TFT"; $resn_exch{"2H5M TFT"}=" H72 TFT"; $resn_exch{"3H5M TFT"}=" H73 TFT";
$resn_exch{"CH2T TFT"}="H2TC TFT"; $resn_exch{"1H4T TFT"}="H4T1 TFT"; $resn_exch{"2H4T TFT"}="H4T2 TFT";
$resn_exch{"CH1T TFT"}="H1TC TFT"; $resn_exch{"CH3T TFT"}="H3TC TFT"; $resn_exch{"3HOP TFT"}="HOP3 TFT";
$resn_exch{"1H34 TG1"}="H341 TG1"; $resn_exch{"2H34 TG1"}="H342 TG1"; $resn_exch{"3H34 TG1"}="H343 TG1";
$resn_exch{"1HC9 TG1"}="HC91 TG1"; $resn_exch{"2HC9 TG1"}="HC92 TG1"; $resn_exch{"1H14 TG1"}="H141 TG1";
$resn_exch{"2H14 TG1"}="H142 TG1"; $resn_exch{"1H15 TG1"}="H151 TG1"; $resn_exch{"2H15 TG1"}="H152 TG1";
$resn_exch{"1H16 TG1"}="H161 TG1"; $resn_exch{"2H16 TG1"}="H162 TG1"; $resn_exch{"1H17 TG1"}="H171 TG1";
$resn_exch{"2H17 TG1"}="H172 TG1"; $resn_exch{"1H18 TG1"}="H181 TG1"; $resn_exch{"2H18 TG1"}="H182 TG1";
$resn_exch{"1H19 TG1"}="H191 TG1"; $resn_exch{"2H19 TG1"}="H192 TG1"; $resn_exch{"1H20 TG1"}="H201 TG1";
$resn_exch{"2H20 TG1"}="H202 TG1"; $resn_exch{"3H20 TG1"}="H203 TG1"; $resn_exch{"1H23 TG1"}="H231 TG1";
$resn_exch{"2H23 TG1"}="H232 TG1"; $resn_exch{"3H23 TG1"}="H233 TG1"; $resn_exch{"1H25 TG1"}="H251 TG1";
$resn_exch{"2H25 TG1"}="H252 TG1"; $resn_exch{"3H25 TG1"}="H253 TG1"; $resn_exch{"1H26 TG1"}="H261 TG1";
$resn_exch{"2H26 TG1"}="H262 TG1"; $resn_exch{"3H26 TG1"}="H263 TG1"; $resn_exch{"1H31 TG1"}="H311 TG1";
$resn_exch{"2H31 TG1"}="H312 TG1"; $resn_exch{"3H31 TG1"}="H313 TG1"; $resn_exch{"1H33 TG1"}="H331 TG1";
$resn_exch{"2H33 TG1"}="H332 TG1"; $resn_exch{"3H33 TG1"}="H333 TG1"; $resn_exch{"1H28 TG1"}="H281 TG1";
$resn_exch{"2H28 TG1"}="H282 TG1"; $resn_exch{"1H29 TG1"}="H291 TG1"; $resn_exch{"2H29 TG1"}="H292 TG1";
$resn_exch{"1H30 TG1"}="H301 TG1"; $resn_exch{"2H30 TG1"}="H302 TG1"; $resn_exch{"3H30 TG1"}="H303 TG1";
$resn_exch{"1H19 TGC"}="H191 TGC"; $resn_exch{"2H19 TGC"}="H192 TGC"; $resn_exch{"1HN6 TGC"}="HN61 TGC";
$resn_exch{"2HN6 TGC"}="HN62 TGC"; $resn_exch{"1H46 TGC"}="H461 TGC"; $resn_exch{"2H46 TGC"}="H462 TGC";
$resn_exch{"1H53 TGC"}="H531 TGC"; $resn_exch{"2H53 TGC"}="H532 TGC"; $resn_exch{"3H53 TGC"}="H533 TGC";
$resn_exch{"1H57 TGC"}="H571 TGC"; $resn_exch{"2H57 TGC"}="H572 TGC"; $resn_exch{"3H57 TGC"}="H573 TGC";
$resn_exch{"1H62 TGC"}="H621 TGC"; $resn_exch{"2H62 TGC"}="H622 TGC"; $resn_exch{"1H61 TGC"}="H611 TGC";
$resn_exch{"2H61 TGC"}="H612 TGC"; $resn_exch{"1H74 TGC"}="H741 TGC"; $resn_exch{"2H74 TGC"}="H742 TGC";
$resn_exch{"1H73 TGC"}="H731 TGC"; $resn_exch{"2H73 TGC"}="H732 TGC"; $resn_exch{"1H2' TGC"}="H2'1 TGC";
$resn_exch{"2H2' TGC"}="H2'2 TGC"; $resn_exch{"1H3' TGC"}="H3'1 TGC"; $resn_exch{"2H3' TGC"}="H3'2 TGC";
$resn_exch{"1HA2 TGF"}="HA21 TGF"; $resn_exch{"2HA2 TGF"}="HA22 TGF"; $resn_exch{"1H7  TGF"}=" H71 TGF";
$resn_exch{"2H7  TGF"}=" H72 TGF"; $resn_exch{"1H9  TGF"}=" H91 TGF"; $resn_exch{"2H9  TGF"}=" H92 TGF";
$resn_exch{"1HB1 TGF"}="HB11 TGF"; $resn_exch{"2HB1 TGF"}="HB12 TGF"; $resn_exch{"1HG1 TGF"}="HG11 TGF";
$resn_exch{"2HG1 TGF"}="HG12 TGF"; $resn_exch{"1HB2 TGF"}="HB21 TGF"; $resn_exch{"2HB2 TGF"}="HB22 TGF";
$resn_exch{"1HG2 TGF"}="HG21 TGF"; $resn_exch{"2HG2 TGF"}="HG22 TGF"; $resn_exch{"1HB3 TGF"}="HB31 TGF";
$resn_exch{"2HB3 TGF"}="HB32 TGF"; $resn_exch{"1HG3 TGF"}="HG31 TGF"; $resn_exch{"2HG3 TGF"}="HG32 TGF";
$resn_exch{"EH23 TGF"}="H23E TGF"; $resn_exch{"1HN1 TGG"}="HN11 TGG"; $resn_exch{"2HN1 TGG"}="HN12 TGG";
$resn_exch{"1HB1 TGG"}="HB11 TGG"; $resn_exch{"2HB1 TGG"}="HB12 TGG"; $resn_exch{"1HG1 TGG"}="HG11 TGG";
$resn_exch{"2HG1 TGG"}="HG12 TGG"; $resn_exch{"1HA3 TGG"}="HA31 TGG"; $resn_exch{"2HA3 TGG"}="HA32 TGG";
$resn_exch{"1HB2 TGG"}="HB21 TGG"; $resn_exch{"2HB2 TGG"}="HB22 TGG"; $resn_exch{"1HF2 TGG"}="HF21 TGG";
$resn_exch{"2HF2 TGG"}="HF22 TGG"; $resn_exch{"1HA2 TGL"}="HA21 TGL"; $resn_exch{"2HA2 TGL"}="HA22 TGL";
$resn_exch{"1HA3 TGL"}="HA31 TGL"; $resn_exch{"2HA3 TGL"}="HA32 TGL"; $resn_exch{"1HA4 TGL"}="HA41 TGL";
$resn_exch{"2HA4 TGL"}="HA42 TGL"; $resn_exch{"1HA5 TGL"}="HA51 TGL"; $resn_exch{"2HA5 TGL"}="HA52 TGL";
$resn_exch{"1HA6 TGL"}="HA61 TGL"; $resn_exch{"2HA6 TGL"}="HA62 TGL"; $resn_exch{"1HA7 TGL"}="HA71 TGL";
$resn_exch{"2HA7 TGL"}="HA72 TGL"; $resn_exch{"1HA8 TGL"}="HA81 TGL"; $resn_exch{"2HA8 TGL"}="HA82 TGL";
$resn_exch{"1HA9 TGL"}="HA91 TGL"; $resn_exch{"2HA9 TGL"}="HA92 TGL"; $resn_exch{"1HB2 TGL"}="HB21 TGL";
$resn_exch{"2HB2 TGL"}="HB22 TGL"; $resn_exch{"1HB3 TGL"}="HB31 TGL"; $resn_exch{"2HB3 TGL"}="HB32 TGL";
$resn_exch{"1HB4 TGL"}="HB41 TGL"; $resn_exch{"2HB4 TGL"}="HB42 TGL"; $resn_exch{"1HB5 TGL"}="HB51 TGL";
$resn_exch{"2HB5 TGL"}="HB52 TGL"; $resn_exch{"1HB6 TGL"}="HB61 TGL"; $resn_exch{"2HB6 TGL"}="HB62 TGL";
$resn_exch{"1HB7 TGL"}="HB71 TGL"; $resn_exch{"2HB7 TGL"}="HB72 TGL"; $resn_exch{"1HB8 TGL"}="HB81 TGL";
$resn_exch{"2HB8 TGL"}="HB82 TGL"; $resn_exch{"1HB9 TGL"}="HB91 TGL"; $resn_exch{"2HB9 TGL"}="HB92 TGL";
$resn_exch{"1H10 TGL"}="H101 TGL"; $resn_exch{"2H10 TGL"}="H102 TGL"; $resn_exch{"1H11 TGL"}="H111 TGL";
$resn_exch{"2H11 TGL"}="H112 TGL"; $resn_exch{"1H12 TGL"}="H121 TGL"; $resn_exch{"2H12 TGL"}="H122 TGL";
$resn_exch{"1H13 TGL"}="H131 TGL"; $resn_exch{"2H13 TGL"}="H132 TGL"; $resn_exch{"1H14 TGL"}="H141 TGL";
$resn_exch{"2H14 TGL"}="H142 TGL"; $resn_exch{"1HG1 TGL"}="HG11 TGL"; $resn_exch{"2HG1 TGL"}="HG12 TGL";
$resn_exch{"1HG3 TGL"}="HG31 TGL"; $resn_exch{"2HG3 TGL"}="HG32 TGL"; $resn_exch{"1HC2 TGL"}="HC21 TGL";
$resn_exch{"2HC2 TGL"}="HC22 TGL"; $resn_exch{"1HC3 TGL"}="HC31 TGL"; $resn_exch{"2HC3 TGL"}="HC32 TGL";
$resn_exch{"1HC4 TGL"}="HC41 TGL"; $resn_exch{"2HC4 TGL"}="HC42 TGL"; $resn_exch{"1HC5 TGL"}="HC51 TGL";
$resn_exch{"2HC5 TGL"}="HC52 TGL"; $resn_exch{"1HC6 TGL"}="HC61 TGL"; $resn_exch{"2HC6 TGL"}="HC62 TGL";
$resn_exch{"1HC7 TGL"}="HC71 TGL"; $resn_exch{"2HC7 TGL"}="HC72 TGL"; $resn_exch{"1HC8 TGL"}="HC81 TGL";
$resn_exch{"2HC8 TGL"}="HC82 TGL"; $resn_exch{"1HC9 TGL"}="HC91 TGL"; $resn_exch{"2HC9 TGL"}="HC92 TGL";
$resn_exch{"1H15 TGL"}="H151 TGL"; $resn_exch{"2H15 TGL"}="H152 TGL"; $resn_exch{"1H16 TGL"}="H161 TGL";
$resn_exch{"2H16 TGL"}="H162 TGL"; $resn_exch{"1H17 TGL"}="H171 TGL"; $resn_exch{"2H17 TGL"}="H172 TGL";
$resn_exch{"1H18 TGL"}="H181 TGL"; $resn_exch{"2H18 TGL"}="H182 TGL"; $resn_exch{"1H19 TGL"}="H191 TGL";
$resn_exch{"2H19 TGL"}="H192 TGL"; $resn_exch{"1H20 TGL"}="H201 TGL"; $resn_exch{"2H20 TGL"}="H202 TGL";
$resn_exch{"1H21 TGL"}="H211 TGL"; $resn_exch{"2H21 TGL"}="H212 TGL"; $resn_exch{"1H22 TGL"}="H221 TGL";
$resn_exch{"2H22 TGL"}="H222 TGL"; $resn_exch{"1H23 TGL"}="H231 TGL"; $resn_exch{"2H23 TGL"}="H232 TGL";
$resn_exch{"1H24 TGL"}="H241 TGL"; $resn_exch{"2H24 TGL"}="H242 TGL"; $resn_exch{"1H25 TGL"}="H251 TGL";
$resn_exch{"2H25 TGL"}="H252 TGL"; $resn_exch{"1H26 TGL"}="H261 TGL"; $resn_exch{"2H26 TGL"}="H262 TGL";
$resn_exch{"1H27 TGL"}="H271 TGL"; $resn_exch{"2H27 TGL"}="H272 TGL"; $resn_exch{"1H28 TGL"}="H281 TGL";
$resn_exch{"2H28 TGL"}="H282 TGL"; $resn_exch{"3H28 TGL"}="H283 TGL"; $resn_exch{"1H29 TGL"}="H291 TGL";
$resn_exch{"2H29 TGL"}="H292 TGL"; $resn_exch{"1H30 TGL"}="H301 TGL"; $resn_exch{"2H30 TGL"}="H302 TGL";
$resn_exch{"1H31 TGL"}="H311 TGL"; $resn_exch{"2H31 TGL"}="H312 TGL"; $resn_exch{"1H32 TGL"}="H321 TGL";
$resn_exch{"2H32 TGL"}="H322 TGL"; $resn_exch{"3H32 TGL"}="H323 TGL"; $resn_exch{"1H33 TGL"}="H331 TGL";
$resn_exch{"2H33 TGL"}="H332 TGL"; $resn_exch{"1H34 TGL"}="H341 TGL"; $resn_exch{"2H34 TGL"}="H342 TGL";
$resn_exch{"1H35 TGL"}="H351 TGL"; $resn_exch{"2H35 TGL"}="H352 TGL"; $resn_exch{"1H36 TGL"}="H361 TGL";
$resn_exch{"2H36 TGL"}="H362 TGL"; $resn_exch{"3H36 TGL"}="H363 TGL"; $resn_exch{"1H14 TGN"}="H141 TGN";
$resn_exch{"2H14 TGN"}="H142 TGN"; $resn_exch{"3H14 TGN"}="H143 TGN"; $resn_exch{"1H18 TGN"}="H181 TGN";
$resn_exch{"2H18 TGN"}="H182 TGN"; $resn_exch{"1H21 TGN"}="H211 TGN"; $resn_exch{"2H21 TGN"}="H212 TGN";
$resn_exch{"1H26 TGN"}="H261 TGN"; $resn_exch{"2H26 TGN"}="H262 TGN"; $resn_exch{"1H1  TGN"}="H11A TGN";
$resn_exch{"2H1  TGN"}=" H12 TGN"; $resn_exch{"3H1  TGN"}="H13A TGN"; $resn_exch{"1H2  TGP"}=" H21 TGP";
$resn_exch{"2H2  TGP"}=" H22 TGP"; $resn_exch{"1H2* TGP"}=" H2' TGP"; $resn_exch{"2H2* TGP"}="H2'' TGP";
$resn_exch{"1H5* TGP"}=" H5' TGP"; $resn_exch{"2H5* TGP"}="H5'' TGP"; $resn_exch{" H4* TGP"}=" H4' TGP";
$resn_exch{" H1* TGP"}=" H1' TGP"; $resn_exch{" H3* TGP"}=" H3' TGP"; $resn_exch{"H3T  TGP"}="HO3' TGP";
$resn_exch{"3HOP TGP"}="HOP3 TGP"; $resn_exch{"2HOP TGP"}="HOP2 TGP"; $resn_exch{"1H6  TGT"}=" H61 TGT";
$resn_exch{"2H6  TGT"}=" H62 TGT"; $resn_exch{"1H11 TGT"}="H111 TGT"; $resn_exch{"2H11 TGT"}="H112 TGT";
$resn_exch{"3H11 TGT"}="H113 TGT"; $resn_exch{"1HN1 TGT"}="HN11 TGT"; $resn_exch{"2HN1 TGT"}="HN12 TGT";
$resn_exch{"1HN2 TGT"}="HN21 TGT"; $resn_exch{"2HN2 TGT"}="HN22 TGT"; $resn_exch{"1H6  TH1"}=" H61 TH1";
$resn_exch{"2H6  TH1"}=" H62 TH1"; $resn_exch{"1H14 TH1"}="H141 TH1"; $resn_exch{"2H14 TH1"}="H142 TH1";
$resn_exch{"3H14 TH1"}="H143 TH1"; $resn_exch{"1H1C TH2"}="H1C1 TH2"; $resn_exch{"2H1C TH2"}="H1C2 TH2";
$resn_exch{"1H2C TH2"}="H2C1 TH2"; $resn_exch{"2H2C TH2"}="H2C2 TH2"; $resn_exch{"1H6C TH2"}="H6C1 TH2";
$resn_exch{"2H6C TH2"}="H6C2 TH2"; $resn_exch{"1H7C TH2"}="H7C1 TH2"; $resn_exch{"2H7C TH2"}="H7C2 TH2";
$resn_exch{"1H11 TH2"}="H111 TH2"; $resn_exch{"2H11 TH2"}="H112 TH2"; $resn_exch{"1H12 TH2"}="H121 TH2";
$resn_exch{"2H12 TH2"}="H122 TH2"; $resn_exch{"1H15 TH2"}="H151 TH2"; $resn_exch{"2H15 TH2"}="H152 TH2";
$resn_exch{"1H16 TH2"}="H161 TH2"; $resn_exch{"2H16 TH2"}="H162 TH2"; $resn_exch{"1H18 TH2"}="H181 TH2";
$resn_exch{"2H18 TH2"}="H182 TH2"; $resn_exch{"3H18 TH2"}="H183 TH2"; $resn_exch{"1H21 TH2"}="H211 TH2";
$resn_exch{"2H21 TH2"}="H212 TH2"; $resn_exch{"1H22 TH2"}="H221 TH2"; $resn_exch{"2H22 TH2"}="H222 TH2";
$resn_exch{"1H19 TH2"}="H191 TH2"; $resn_exch{"2H19 TH2"}="H192 TH2"; $resn_exch{"3H19 TH2"}="H193 TH2";
$resn_exch{"1H1C TH3"}="H1C1 TH3"; $resn_exch{"2H1C TH3"}="H1C2 TH3"; $resn_exch{"1H4C TH3"}="H4C1 TH3";
$resn_exch{"2H4C TH3"}="H4C2 TH3"; $resn_exch{"1H5C TH3"}="H5C1 TH3"; $resn_exch{"2H5C TH3"}="H5C2 TH3";
$resn_exch{"1H10 TH3"}="H101 TH3"; $resn_exch{"2H10 TH3"}="H102 TH3"; $resn_exch{"1H11 TH3"}="H111 TH3";
$resn_exch{"2H11 TH3"}="H112 TH3"; $resn_exch{"1H12 TH3"}="H12A TH3"; $resn_exch{"2H12 TH3"}="H12B TH3";
$resn_exch{"1H13 TH3"}="H13A TH3"; $resn_exch{"2H13 TH3"}="H13B TH3"; $resn_exch{"1H14 TH3"}="H14A TH3";
$resn_exch{"2H14 TH3"}="H14B TH3"; $resn_exch{"1H15 TH3"}="H151 TH3"; $resn_exch{"2H15 TH3"}="H152 TH3";
$resn_exch{"1H16 TH3"}="H161 TH3"; $resn_exch{"2H16 TH3"}="H162 TH3"; $resn_exch{"1H17 TH3"}="H171 TH3";
$resn_exch{"2H17 TH3"}="H172 TH3"; $resn_exch{"1H18 TH3"}="H181 TH3"; $resn_exch{"2H18 TH3"}="H182 TH3";
$resn_exch{"1H19 TH3"}="H191 TH3"; $resn_exch{"2H19 TH3"}="H192 TH3"; $resn_exch{"1H20 TH3"}="H201 TH3";
$resn_exch{"2H20 TH3"}="H202 TH3"; $resn_exch{"1H21 TH3"}="H211 TH3"; $resn_exch{"2H21 TH3"}="H212 TH3";
$resn_exch{"1H11 THA"}="H111 THA"; $resn_exch{"2H11 THA"}="H112 THA"; $resn_exch{"1H12 THA"}="H121 THA";
$resn_exch{"2H12 THA"}="H122 THA"; $resn_exch{"1H13 THA"}="H131 THA"; $resn_exch{"2H13 THA"}="H132 THA";
$resn_exch{"1H14 THA"}="H141 THA"; $resn_exch{"2H14 THA"}="H142 THA"; $resn_exch{"1HN5 THA"}="HN51 THA";
$resn_exch{"2HN5 THA"}="HN52 THA"; $resn_exch{"1HN2 THB"}="HN21 THB"; $resn_exch{"2HN2 THB"}="HN22 THB";
$resn_exch{"1H7  THB"}=" H71 THB"; $resn_exch{"2H7  THB"}=" H72 THB"; $resn_exch{"1H11 THB"}="H111 THB";
$resn_exch{"2H11 THB"}="H112 THB"; $resn_exch{"3H11 THB"}="H113 THB"; $resn_exch{" HN  THC"}=" H   THC";
$resn_exch{"1HM  THC"}=" HM1 THC"; $resn_exch{"2HM  THC"}=" HM2 THC"; $resn_exch{"3HM  THC"}=" HM3 THC";
$resn_exch{"1HG2 THC"}="HG21 THC"; $resn_exch{"2HG2 THC"}="HG22 THC"; $resn_exch{"3HG2 THC"}="HG23 THC";
$resn_exch{"1HM2 THD"}="HM21 THD"; $resn_exch{"2HM2 THD"}="HM22 THD"; $resn_exch{"3HM2 THD"}="HM23 THD";
$resn_exch{"1H4' THD"}="H4'1 THD"; $resn_exch{"2H4' THD"}="H4'2 THD"; $resn_exch{"1H7' THD"}="H7'1 THD";
$resn_exch{"2H7' THD"}="H7'2 THD"; $resn_exch{"1HM4 THD"}="HM41 THD"; $resn_exch{"2HM4 THD"}="HM42 THD";
$resn_exch{"3HM4 THD"}="HM43 THD"; $resn_exch{"1H6C THD"}="H6C1 THD"; $resn_exch{"2H6C THD"}="H6C2 THD";
$resn_exch{"1H7C THD"}="H7C1 THD"; $resn_exch{"2H7C THD"}="H7C2 THD"; $resn_exch{"1H9C THD"}="H9C1 THD";
$resn_exch{"2H9C THD"}="H9C2 THD"; $resn_exch{"1H1  THE"}=" H11 THE"; $resn_exch{"2H1  THE"}=" H12 THE";
$resn_exch{"1HN2 THF"}="HN21 THF"; $resn_exch{"2HN2 THF"}="HN22 THF"; $resn_exch{"1H9  THF"}=" H91 THF";
$resn_exch{"2H9  THF"}=" H92 THF"; $resn_exch{"1HB  THF"}=" HB1 THF"; $resn_exch{"2HB  THF"}=" HB2 THF";
$resn_exch{"1HG  THF"}=" HG1 THF"; $resn_exch{"2HG  THF"}=" HG2 THF"; $resn_exch{"2HOE THF"}="HOE2 THF";
$resn_exch{"1HCP THF"}="HCP1 THF"; $resn_exch{"2HCP THF"}="HCP2 THF"; $resn_exch{"1HC7 THG"}="HC71 THG";
$resn_exch{"2HC7 THG"}="HC72 THG"; $resn_exch{"1HC9 THG"}="HC91 THG"; $resn_exch{"2HC9 THG"}="HC92 THG";
$resn_exch{"1HC6 THG"}="HC61 THG"; $resn_exch{"1HCB THG"}="HCB1 THG"; $resn_exch{"2HCB THG"}="HCB2 THG";
$resn_exch{"1HCG THG"}="HCG1 THG"; $resn_exch{"2HCG THG"}="HCG2 THG"; $resn_exch{"1HN2 THG"}="HN21 THG";
$resn_exch{"2HN2 THG"}="HN22 THG"; $resn_exch{"1HG  THH"}=" HG1 THH"; $resn_exch{"2HG  THH"}=" HG2 THH";
$resn_exch{"1HB  THH"}=" HB1 THH"; $resn_exch{"2HB  THH"}=" HB2 THH"; $resn_exch{"1H9  THH"}=" H91 THH";
$resn_exch{"2H9  THH"}=" H92 THH"; $resn_exch{"1H7  THH"}=" H71 THH"; $resn_exch{"2H7  THH"}=" H72 THH";
$resn_exch{"1H11 THH"}="H111 THH"; $resn_exch{"2H11 THH"}="H112 THH"; $resn_exch{"3H11 THH"}="H113 THH";
$resn_exch{"1HA2 THH"}="HA21 THH"; $resn_exch{"2HA2 THH"}="HA22 THH"; $resn_exch{"1H2  THI"}=" H21 THI";
$resn_exch{"2H2  THI"}=" H22 THI"; $resn_exch{"1H3  THI"}=" H31 THI"; $resn_exch{"2H3  THI"}=" H32 THI";
$resn_exch{"1H2  THK"}="H21A THK"; $resn_exch{"2H2  THK"}="H22A THK"; $resn_exch{"1H3  THK"}=" H31 THK";
$resn_exch{"2H3  THK"}=" H32 THK"; $resn_exch{"1H5  THK"}=" H51 THK"; $resn_exch{"2H5  THK"}=" H52 THK";
$resn_exch{"1H6  THK"}=" H61 THK"; $resn_exch{"2H6  THK"}=" H62 THK"; $resn_exch{"1H7  THK"}=" H71 THK";
$resn_exch{"2H7  THK"}=" H72 THK"; $resn_exch{"1H8  THK"}=" H81 THK"; $resn_exch{"2H8  THK"}=" H82 THK";
$resn_exch{"1H9  THK"}=" H91 THK"; $resn_exch{"2H9  THK"}=" H92 THK"; $resn_exch{"1H52 THK"}="H521 THK";
$resn_exch{"2H52 THK"}="H522 THK"; $resn_exch{"1H55 THK"}="H551 THK"; $resn_exch{"2H55 THK"}="H552 THK";
$resn_exch{"1H7  THL"}=" H71 THL"; $resn_exch{"2H7  THL"}=" H72 THL"; $resn_exch{"1H9  THL"}=" H91 THL";
$resn_exch{"2H9  THL"}=" H92 THL"; $resn_exch{"1HA2 THL"}="HA21 THL"; $resn_exch{"2HA2 THL"}="HA22 THL";
$resn_exch{"1HB  THL"}=" HB1 THL"; $resn_exch{"2HB  THL"}=" HB2 THL"; $resn_exch{"1HG  THL"}=" HG1 THL";
$resn_exch{"2HG  THL"}=" HG2 THL"; $resn_exch{"*HO5 THM"}="HO5' THM"; $resn_exch{"1H5* THM"}="H5'1 THM";
$resn_exch{"2H5* THM"}="H5'2 THM"; $resn_exch{" H4* THM"}=" H4' THM"; $resn_exch{" H3* THM"}=" H3' THM";
$resn_exch{"*HO3 THM"}="HO3' THM"; $resn_exch{"1H2* THM"}="H2'1 THM"; $resn_exch{"2H2* THM"}="H2'2 THM";
$resn_exch{" H1* THM"}=" H1' THM"; $resn_exch{"1HM5 THM"}="HM51 THM"; $resn_exch{"2HM5 THM"}="HM52 THM";
$resn_exch{"3HM5 THM"}="HM53 THM"; $resn_exch{"1H2  THN"}=" H21 THN"; $resn_exch{"2H2  THN"}=" H22 THN";
$resn_exch{"1H3' THN"}="H3'1 THN"; $resn_exch{"2H3' THN"}="H3'2 THN"; $resn_exch{"1H13 THN"}="H131 THN";
$resn_exch{"2H13 THN"}="H132 THN"; $resn_exch{"1HN  THO"}=" H   THO"; $resn_exch{"2HN  THO"}=" H2  THO";
$resn_exch{"1HG2 THO"}="HG21 THO"; $resn_exch{"2HG2 THO"}="HG22 THO"; $resn_exch{"3HG2 THO"}="HG23 THO";
$resn_exch{"1H   THO"}=" HC1 THO"; $resn_exch{"2H   THO"}=" HC2 THO"; $resn_exch{"5HOP THP"}="HOP5 THP";
$resn_exch{"6HOP THP"}="HOP6 THP"; $resn_exch{"2HOP THP"}="HOP2 THP"; $resn_exch{"3HOP THP"}="HOP3 THP";
$resn_exch{"1H5* THP"}="H5'1 THP"; $resn_exch{"2H5* THP"}="H5'2 THP"; $resn_exch{" H4* THP"}=" H4' THP";
$resn_exch{" H3* THP"}=" H3' THP"; $resn_exch{"1H2* THP"}="H2'1 THP"; $resn_exch{"2H2* THP"}="H2'2 THP";
$resn_exch{" H1* THP"}=" H1' THP"; $resn_exch{"1H5  THP"}=" H51 THP"; $resn_exch{"2H5  THP"}=" H52 THP";
$resn_exch{"3H5  THP"}=" H53 THP"; $resn_exch{"1H1  THQ"}=" H11 THQ"; $resn_exch{"1HM3 THQ"}="HM31 THQ";
$resn_exch{"2HM3 THQ"}="HM32 THQ"; $resn_exch{"3HM3 THQ"}="HM33 THQ"; $resn_exch{"1HC4 THQ"}="HC41 THQ";
$resn_exch{"2HC4 THQ"}="HC42 THQ"; $resn_exch{"1HC5 THQ"}="HC51 THQ"; $resn_exch{"2HC5 THQ"}="HC52 THQ";
$resn_exch{"1H9  THQ"}=" H91 THQ"; $resn_exch{"1H10 THQ"}="H101 THQ"; $resn_exch{" HN2 THR"}=" H2  THR";
$resn_exch{"1HG2 THR"}="HG21 THR"; $resn_exch{"2HG2 THR"}="HG22 THR"; $resn_exch{"3HG2 THR"}="HG23 THR";
$resn_exch{"2HSP THS"}="HSP2 THS"; $resn_exch{"3HSP THS"}="HSP3 THS"; $resn_exch{"1H5* THS"}="H5'1 THS";
$resn_exch{"2H5* THS"}="H5'2 THS"; $resn_exch{" H4* THS"}=" H4' THS"; $resn_exch{" H3* THS"}=" H3' THS";
$resn_exch{"*HO3 THS"}="HO3' THS"; $resn_exch{"1H2* THS"}="H2'1 THS"; $resn_exch{"2H2* THS"}="H2'2 THS";
$resn_exch{" H1* THS"}=" H1' THS"; $resn_exch{"1HM5 THS"}="HM51 THS"; $resn_exch{"2HM5 THS"}="HM52 THS";
$resn_exch{"3HM5 THS"}="HM53 THS"; $resn_exch{"1HC1 THT"}="HC11 THT"; $resn_exch{"2HC1 THT"}="HC12 THT";
$resn_exch{"3HC1 THT"}="HC13 THT"; $resn_exch{"1HN3 THT"}="HN31 THT"; $resn_exch{"1HC4 THT"}="HC41 THT";
$resn_exch{"2HC4 THT"}="HC42 THT"; $resn_exch{"1HC5 THT"}="HC51 THT"; $resn_exch{"2HC5 THT"}="HC52 THT";
$resn_exch{"1HC8 THT"}="HC81 THT"; $resn_exch{"2HC8 THT"}="HC82 THT"; $resn_exch{"1HC9 THT"}="HC91 THT";
$resn_exch{"2HC9 THT"}="HC92 THT"; $resn_exch{"1H10 THT"}="H101 THT"; $resn_exch{"2H10 THT"}="H102 THT";
$resn_exch{"1H11 THT"}="H111 THT"; $resn_exch{"2H11 THT"}="H112 THT"; $resn_exch{"1H12 THT"}="H121 THT";
$resn_exch{"2H12 THT"}="H122 THT"; $resn_exch{"1H13 THT"}="H131 THT"; $resn_exch{"2H13 THT"}="H132 THT";
$resn_exch{"1H14 THT"}="H141 THT"; $resn_exch{"2H14 THT"}="H142 THT"; $resn_exch{"1H15 THT"}="H151 THT";
$resn_exch{"2H15 THT"}="H152 THT"; $resn_exch{"1H16 THT"}="H161 THT"; $resn_exch{"2H16 THT"}="H162 THT";
$resn_exch{"1H17 THT"}="H171 THT"; $resn_exch{"2H17 THT"}="H172 THT"; $resn_exch{"1H18 THT"}="H181 THT";
$resn_exch{"2H18 THT"}="H182 THT"; $resn_exch{"1H19 THT"}="H191 THT"; $resn_exch{"2H19 THT"}="H192 THT";
$resn_exch{"1H20 THT"}="H201 THT"; $resn_exch{"2H20 THT"}="H202 THT"; $resn_exch{"1H21 THT"}="H211 THT";
$resn_exch{"2H21 THT"}="H212 THT"; $resn_exch{"1H22 THT"}="H221 THT"; $resn_exch{"2H22 THT"}="H222 THT";
$resn_exch{"3H22 THT"}="H223 THT"; $resn_exch{"1H5  THU"}=" H51 THU"; $resn_exch{"2H5  THU"}=" H52 THU";
$resn_exch{"1H6  THU"}=" H61 THU"; $resn_exch{"2H6  THU"}=" H62 THU"; $resn_exch{" H1* THU"}=" H1' THU";
$resn_exch{"1H2* THU"}="H2'1 THU"; $resn_exch{"2H2* THU"}="H2'2 THU"; $resn_exch{" H3* THU"}=" H3' THU";
$resn_exch{"*HO3 THU"}="HO3' THU"; $resn_exch{" H4* THU"}=" H4' THU"; $resn_exch{"1H5* THU"}="H5'1 THU";
$resn_exch{"2H5* THU"}="H5'2 THU"; $resn_exch{"*HO5 THU"}="HO5' THU"; $resn_exch{"1H5B THV"}="H5B1 THV";
$resn_exch{"2H5B THV"}="H5B2 THV"; $resn_exch{"1H5A THV"}="H5A1 THV"; $resn_exch{"2H5A THV"}="H5A2 THV";
$resn_exch{"1H4A THV"}="H4A1 THV"; $resn_exch{"2H4A THV"}="H4A2 THV"; $resn_exch{"3H4A THV"}="H4A3 THV";
$resn_exch{"1H10 THV"}="H101 THV"; $resn_exch{"2H10 THV"}="H102 THV"; $resn_exch{"3H10 THV"}="H103 THV";
$resn_exch{"1H11 THV"}="H111 THV"; $resn_exch{"2H11 THV"}="H112 THV"; $resn_exch{"3H11 THV"}="H113 THV";
$resn_exch{"1H35 THV"}="H351 THV"; $resn_exch{"2H35 THV"}="H352 THV"; $resn_exch{" H6* THV"}=" H6' THV";
$resn_exch{"1H2A THV"}="H2A1 THV"; $resn_exch{"2H2A THV"}="H2A2 THV"; $resn_exch{"3H2A THV"}="H2A3 THV";
$resn_exch{"1H4* THV"}="H4'1 THV"; $resn_exch{"2H4* THV"}="H4'2 THV"; $resn_exch{"1H5A THW"}="H5A1 THW";
$resn_exch{"2H5A THW"}="H5A2 THW"; $resn_exch{"1H5B THW"}="H5B1 THW"; $resn_exch{"2H5B THW"}="H5B2 THW";
$resn_exch{"1H4A THW"}="H4A1 THW"; $resn_exch{"2H4A THW"}="H4A2 THW"; $resn_exch{"3H4A THW"}="H4A3 THW";
$resn_exch{"1H35 THW"}="H351 THW"; $resn_exch{"2H35 THW"}="H352 THW"; $resn_exch{" H6* THW"}=" H6' THW";
$resn_exch{"1H2A THW"}="H2A1 THW"; $resn_exch{"2H2A THW"}="H2A2 THW"; $resn_exch{"3H2A THW"}="H2A3 THW";
$resn_exch{"1H4* THW"}="H4'1 THW"; $resn_exch{"2H4* THW"}="H4'2 THW"; $resn_exch{" H1* THX"}=" H1' THX";
$resn_exch{"1H10 THX"}="H101 THX"; $resn_exch{"2H10 THX"}="H102 THX"; $resn_exch{"1H11 THX"}="H111 THX";
$resn_exch{"2H11 THX"}="H112 THX"; $resn_exch{"1H12 THX"}="H121 THX"; $resn_exch{"2H12 THX"}="H122 THX";
$resn_exch{"1H2* THX"}="H2'1 THX"; $resn_exch{"2H2* THX"}="H2'2 THX"; $resn_exch{"1H22 THX"}="H221 THX";
$resn_exch{"2H22 THX"}="H222 THX"; $resn_exch{"1H23 THX"}="H231 THX"; $resn_exch{"2H23 THX"}="H232 THX";
$resn_exch{" H3* THX"}=" H3' THX"; $resn_exch{"1H33 THX"}="H331 THX"; $resn_exch{"2H33 THX"}="H332 THX";
$resn_exch{"1H34 THX"}="H341 THX"; $resn_exch{"2H34 THX"}="H342 THX"; $resn_exch{" H4* THX"}=" H4' THX";
$resn_exch{"1H44 THX"}="H441 THX"; $resn_exch{"2H44 THX"}="H442 THX"; $resn_exch{"1H45 THX"}="H451 THX";
$resn_exch{"2H45 THX"}="H452 THX"; $resn_exch{"1H5* THX"}="H5'1 THX"; $resn_exch{"2H5* THX"}="H5'2 THX";
$resn_exch{"1H5M THX"}="H5M1 THX"; $resn_exch{"2H5M THX"}="H5M2 THX"; $resn_exch{"3H5M THX"}="H5M3 THX";
$resn_exch{"1H7  THX"}=" H71 THX"; $resn_exch{"2H7  THX"}=" H72 THX"; $resn_exch{"1H8  THX"}=" H81 THX";
$resn_exch{"2H8  THX"}=" H82 THX"; $resn_exch{"1H9  THX"}=" H91 THX"; $resn_exch{"2H9  THX"}=" H92 THX";
$resn_exch{"1H15 THX"}="H151 THX"; $resn_exch{"2H15 THX"}="H152 THX"; $resn_exch{"2HOP THX"}="HOP2 THX";
$resn_exch{"3HO* THX"}="HO'3 THX"; $resn_exch{"1H5B THY"}="H5B1 THY"; $resn_exch{"2H5B THY"}="H5B2 THY";
$resn_exch{"1H5A THY"}="H5A1 THY"; $resn_exch{"2H5A THY"}="H5A2 THY"; $resn_exch{"1H4A THY"}="H4A1 THY";
$resn_exch{"2H4A THY"}="H4A2 THY"; $resn_exch{"3H4A THY"}="H4A3 THY"; $resn_exch{"1H11 THY"}="H111 THY";
$resn_exch{"2H11 THY"}="H112 THY"; $resn_exch{"3H11 THY"}="H113 THY"; $resn_exch{"1H10 THY"}="H101 THY";
$resn_exch{"2H10 THY"}="H102 THY"; $resn_exch{"1H12 THY"}="H121 THY"; $resn_exch{"2H12 THY"}="H122 THY";
$resn_exch{"3H12 THY"}="H123 THY"; $resn_exch{"1H35 THY"}="H351 THY"; $resn_exch{"2H35 THY"}="H352 THY";
$resn_exch{" H6* THY"}=" H6' THY"; $resn_exch{"1H2A THY"}="H2A1 THY"; $resn_exch{"2H2A THY"}="H2A2 THY";
$resn_exch{"3H2A THY"}="H2A3 THY"; $resn_exch{"1H4* THY"}="H4'1 THY"; $resn_exch{"2H4* THY"}="H4'2 THY";
$resn_exch{"1HB1 TI1"}="HB11 TI1"; $resn_exch{"2HB1 TI1"}="HB12 TI1"; $resn_exch{"1HG1 TI1"}="HG11 TI1";
$resn_exch{"2HG1 TI1"}="HG12 TI1"; $resn_exch{"1HD1 TI1"}="HD11 TI1"; $resn_exch{"2HD1 TI1"}="HD12 TI1";
$resn_exch{"1HE1 TI1"}="HE11 TI1"; $resn_exch{"2HE1 TI1"}="HE12 TI1"; $resn_exch{"1HF1 TI1"}="HF11 TI1";
$resn_exch{"2HF1 TI1"}="HF12 TI1"; $resn_exch{"3HF1 TI1"}="HF13 TI1"; $resn_exch{"1HB2 TI1"}="HB21 TI1";
$resn_exch{"2HB2 TI1"}="HB22 TI1"; $resn_exch{"1HB3 TI1"}="HB31 TI1"; $resn_exch{"2HB3 TI1"}="HB32 TI1";
$resn_exch{"3HB3 TI1"}="HB33 TI1"; $resn_exch{"1HB1 TI2"}="HB11 TI2"; $resn_exch{"2HB1 TI2"}="HB12 TI2";
$resn_exch{"1HB2 TI2"}="HB21 TI2"; $resn_exch{"2HB2 TI2"}="HB22 TI2"; $resn_exch{"1HB3 TI2"}="HB31 TI2";
$resn_exch{"2HB3 TI2"}="HB32 TI2"; $resn_exch{"1HB1 TI3"}="HB11 TI3"; $resn_exch{"2HB1 TI3"}="HB12 TI3";
$resn_exch{"1HA2 TI3"}="HA21 TI3"; $resn_exch{"2HA2 TI3"}="HA22 TI3"; $resn_exch{"1HB3 TI3"}="HB31 TI3";
$resn_exch{"2HB3 TI3"}="HB32 TI3"; $resn_exch{"1HG3 TI3"}="HG31 TI3"; $resn_exch{"2HG3 TI3"}="HG32 TI3";
$resn_exch{"1HN6 TIA"}="HN61 TIA"; $resn_exch{"2HN6 TIA"}="HN62 TIA"; $resn_exch{" H1* TIA"}=" H1' TIA";
$resn_exch{" H2* TIA"}=" H2' TIA"; $resn_exch{"*HO2 TIA"}="HO2' TIA"; $resn_exch{" H3* TIA"}=" H3' TIA";
$resn_exch{"*HO3 TIA"}="HO3' TIA"; $resn_exch{" H4* TIA"}=" H4' TIA"; $resn_exch{"1H5* TIA"}="H5'1 TIA";
$resn_exch{"2H5* TIA"}="H5'2 TIA"; $resn_exch{"3H5* TIA"}="H5'3 TIA"; $resn_exch{"1H2  TIB"}=" H21 TIB";
$resn_exch{"2H2  TIB"}=" H22 TIB"; $resn_exch{"1H5  TIB"}=" H51 TIB"; $resn_exch{"2H5  TIB"}=" H52 TIB";
$resn_exch{"1H14 TIB"}="H141 TIB"; $resn_exch{"2H14 TIB"}="H142 TIB"; $resn_exch{"3H14 TIB"}="H143 TIB";
$resn_exch{"1H15 TIB"}="H151 TIB"; $resn_exch{"2H15 TIB"}="H152 TIB"; $resn_exch{"1H18 TIB"}="H181 TIB";
$resn_exch{"2H18 TIB"}="H182 TIB"; $resn_exch{"3H18 TIB"}="H183 TIB"; $resn_exch{"1H19 TIB"}="H191 TIB";
$resn_exch{"2H19 TIB"}="H192 TIB"; $resn_exch{"3H19 TIB"}="H193 TIB"; $resn_exch{"1HN  TIH"}=" H   TIH";
$resn_exch{"2HN  TIH"}=" H2  TIH"; $resn_exch{"1HB  TIH"}=" HB2 TIH"; $resn_exch{"2HB  TIH"}=" HB3 TIH";
$resn_exch{"1HC1 TIN"}="HC11 TIN"; $resn_exch{"2HC1 TIN"}="HC12 TIN"; $resn_exch{"1HC2 TIN"}="HC21 TIN";
$resn_exch{"2HC2 TIN"}="HC22 TIN"; $resn_exch{"1HC4 TIN"}="HC41 TIN"; $resn_exch{"2HC4 TIN"}="HC42 TIN";
$resn_exch{"1HC5 TIN"}="HC51 TIN"; $resn_exch{"2HC5 TIN"}="HC52 TIN"; $resn_exch{"1HC6 TIN"}="HC61 TIN";
$resn_exch{"2HC6 TIN"}="HC62 TIN"; $resn_exch{"1H17 TIN"}="H171 TIN"; $resn_exch{"2H17 TIN"}="H172 TIN";
$resn_exch{"1H19 TIN"}="H191 TIN"; $resn_exch{"2H19 TIN"}="H192 TIN"; $resn_exch{"1H25 TIN"}="H251 TIN";
$resn_exch{"2H25 TIN"}="H252 TIN"; $resn_exch{"1H26 TIN"}="H261 TIN"; $resn_exch{"2H26 TIN"}="H262 TIN";
$resn_exch{"1H29 TIN"}="H291 TIN"; $resn_exch{"2H29 TIN"}="H292 TIN"; $resn_exch{"1H32 TIN"}="H321 TIN";
$resn_exch{"2H32 TIN"}="H322 TIN"; $resn_exch{"3H32 TIN"}="H323 TIN"; $resn_exch{"1H38 TIN"}="H381 TIN";
$resn_exch{"2H38 TIN"}="H382 TIN"; $resn_exch{"1H74 TIN"}="H741 TIN"; $resn_exch{"2H74 TIN"}="H742 TIN";
$resn_exch{"1H76 TIN"}="H761 TIN"; $resn_exch{"2H76 TIN"}="H762 TIN"; $resn_exch{"1H82 TIN"}="H821 TIN";
$resn_exch{"2H82 TIN"}="H822 TIN"; $resn_exch{"3H82 TIN"}="H823 TIN"; $resn_exch{"1HC7 TIN"}="HC71 TIN";
$resn_exch{"2HC7 TIN"}="HC72 TIN"; $resn_exch{"3HC7 TIN"}="HC73 TIN"; $resn_exch{"1HB1 TIO"}="HB11 TIO";
$resn_exch{"2HB1 TIO"}="HB12 TIO"; $resn_exch{"1HA1 TIO"}="HA11 TIO"; $resn_exch{"1HB2 TIO"}="HB21 TIO";
$resn_exch{"2HB2 TIO"}="HB22 TIO"; $resn_exch{"1HA2 TIO"}="HA21 TIO"; $resn_exch{"2HA2 TIO"}="HA22 TIO";
$resn_exch{"1HOH TIO"}="HOH1 TIO"; $resn_exch{"1H21 TIT"}="H211 TIT"; $resn_exch{"2H21 TIT"}="H212 TIT";
$resn_exch{"1H20 TIT"}="H201 TIT"; $resn_exch{"2H20 TIT"}="H202 TIT"; $resn_exch{"1H06 TIT"}="H061 TIT";
$resn_exch{"2H06 TIT"}="H062 TIT"; $resn_exch{"1H01 TIT"}="H011 TIT"; $resn_exch{"2H01 TIT"}="H012 TIT";
$resn_exch{"1H29 TIT"}="H291 TIT"; $resn_exch{"2H29 TIT"}="H292 TIT"; $resn_exch{"1H31 TIT"}="H311 TIT";
$resn_exch{"2H31 TIT"}="H312 TIT"; $resn_exch{"3H31 TIT"}="H313 TIT"; $resn_exch{"1H32 TIT"}="H321 TIT";
$resn_exch{"2H32 TIT"}="H322 TIT"; $resn_exch{"3H32 TIT"}="H323 TIT"; $resn_exch{"1H28 TIT"}="H281 TIT";
$resn_exch{"2H28 TIT"}="H282 TIT"; $resn_exch{"3H28 TIT"}="H283 TIT"; $resn_exch{"1H18 TIT"}="H181 TIT";
$resn_exch{"2H18 TIT"}="H182 TIT"; $resn_exch{"3H18 TIT"}="H183 TIT"; $resn_exch{"1H19 TIT"}="H191 TIT";
$resn_exch{"2H19 TIT"}="H192 TIT"; $resn_exch{"3H19 TIT"}="H193 TIT"; $resn_exch{"HO2* TIZ"}="HO2' TIZ";
$resn_exch{" H2* TIZ"}=" H2' TIZ"; $resn_exch{" H3* TIZ"}=" H3' TIZ"; $resn_exch{"HO3* TIZ"}="HO3' TIZ";
$resn_exch{" H4* TIZ"}=" H4' TIZ"; $resn_exch{"1H5* TIZ"}="H5'1 TIZ"; $resn_exch{"2H5* TIZ"}="H5'2 TIZ";
$resn_exch{"HO5* TIZ"}="HO5' TIZ"; $resn_exch{" H1* TIZ"}=" H1' TIZ"; $resn_exch{"1H1A TIZ"}="H1A1 TIZ";
$resn_exch{"2H1A TIZ"}="H1A2 TIZ"; $resn_exch{"1HXT TK4"}="HXT1 TK4"; $resn_exch{"1H22 TK4"}="H221 TK4";
$resn_exch{"2H22 TK4"}="H222 TK4"; $resn_exch{"3H22 TK4"}="H223 TK4"; $resn_exch{"2HXT TK4"}="HXT2 TK4";
$resn_exch{"1H   TKL"}=" H1  TKL"; $resn_exch{"2H   TKL"}=" H2  TKL"; $resn_exch{"1HB  TKL"}=" HB1 TKL";
$resn_exch{"2HB  TKL"}=" HB2 TKL"; $resn_exch{"1HD1 TKL"}="HD11 TKL"; $resn_exch{"2HD1 TKL"}="HD12 TKL";
$resn_exch{"3HD1 TKL"}="HD13 TKL"; $resn_exch{"1HD2 TKL"}="HD21 TKL"; $resn_exch{"2HD2 TKL"}="HD22 TKL";
$resn_exch{"3HD2 TKL"}="HD23 TKL"; $resn_exch{"1HN  TKL"}=" HN1 TKL"; $resn_exch{"2HN  TKL"}=" HN2 TKL";
$resn_exch{"1HI  TKL"}=" HI1 TKL"; $resn_exch{"2HI  TKL"}=" HI2 TKL"; $resn_exch{"1HJ  TKL"}=" HJ1 TKL";
$resn_exch{"2HJ  TKL"}=" HJ2 TKL"; $resn_exch{"1HK  TKL"}=" HK1 TKL"; $resn_exch{"2HK  TKL"}=" HK2 TKL";
$resn_exch{"1HNZ TKL"}="HNZ1 TKL"; $resn_exch{"2HNZ TKL"}="HNZ2 TKL"; $resn_exch{"1HN2 TL1"}="HN21 TL1";
$resn_exch{"2HN2 TL1"}="HN22 TL1"; $resn_exch{"1H8  TL1"}=" H81 TL1"; $resn_exch{"2H8  TL1"}=" H82 TL1";
$resn_exch{"1HN4 TL1"}="HN41 TL1"; $resn_exch{"2HN4 TL1"}="HN42 TL1"; $resn_exch{"1H18 TL1"}="H181 TL1";
$resn_exch{"2H18 TL1"}="H182 TL1"; $resn_exch{"1H10 TL2"}="H101 TL2"; $resn_exch{"2H10 TL2"}="H102 TL2";
$resn_exch{"1H17 TL2"}="H171 TL2"; $resn_exch{"2H17 TL2"}="H172 TL2"; $resn_exch{"1H23 TL2"}="H231 TL2";
$resn_exch{"2H23 TL2"}="H232 TL2"; $resn_exch{"1H27 TL2"}="H271 TL2"; $resn_exch{"2H27 TL2"}="H272 TL2";
$resn_exch{"1HN2 TL3"}="HN21 TL3"; $resn_exch{"2HN2 TL3"}="HN22 TL3"; $resn_exch{"1H7  TL3"}=" H71 TL3";
$resn_exch{"2H7  TL3"}=" H72 TL3"; $resn_exch{"1HN4 TL3"}="HN41 TL3"; $resn_exch{"2HN4 TL3"}="HN42 TL3";
$resn_exch{"1H17 TL3"}="H171 TL3"; $resn_exch{"2H17 TL3"}="H172 TL3"; $resn_exch{"1H10 TL4"}="H101 TL4";
$resn_exch{"2H10 TL4"}="H102 TL4"; $resn_exch{"1H22 TL4"}="H221 TL4"; $resn_exch{"2H22 TL4"}="H222 TL4";
$resn_exch{"1H24 TL4"}="H241 TL4"; $resn_exch{"2H24 TL4"}="H242 TL4"; $resn_exch{"1HN3 TL4"}="HN31 TL4";
$resn_exch{"2HN3 TL4"}="HN32 TL4"; $resn_exch{"1H9  TL5"}=" H91 TL5"; $resn_exch{"2H9  TL5"}=" H92 TL5";
$resn_exch{"3H9  TL5"}=" H93 TL5"; $resn_exch{"1H10 TL5"}="H101 TL5"; $resn_exch{"2H10 TL5"}="H102 TL5";
$resn_exch{"3H10 TL5"}="H103 TL5"; $resn_exch{"1H11 TL5"}="H111 TL5"; $resn_exch{"2H11 TL5"}="H112 TL5";
$resn_exch{"3H11 TL5"}="H113 TL5"; $resn_exch{"1H12 TL5"}="H121 TL5"; $resn_exch{"2H12 TL5"}="H122 TL5";
$resn_exch{"3H12 TL5"}="H123 TL5"; $resn_exch{"1H10 TL6"}="H101 TL6"; $resn_exch{"2H10 TL6"}="H102 TL6";
$resn_exch{"3H10 TL6"}="H103 TL6"; $resn_exch{"1H9  TL6"}=" H91 TL6"; $resn_exch{"2H9  TL6"}=" H92 TL6";
$resn_exch{"3H9  TL6"}=" H93 TL6"; $resn_exch{"1H8  TL6"}=" H81 TL6"; $resn_exch{"2H8  TL6"}=" H82 TL6";
$resn_exch{"1H5* TLB"}="H5'1 TLB"; $resn_exch{"2H5* TLB"}="H5'2 TLB"; $resn_exch{" H3* TLB"}=" H3' TLB";
$resn_exch{" H2* TLB"}=" H2' TLB"; $resn_exch{" H1* TLB"}=" H1' TLB"; $resn_exch{"1H5M TLB"}="H5M1 TLB";
$resn_exch{"2H5M TLB"}="H5M2 TLB"; $resn_exch{"1H9* TLB"}="H9'1 TLB"; $resn_exch{"2H9* TLB"}="H9'2 TLB";
$resn_exch{"2HOP TLC"}="HOP2 TLC"; $resn_exch{"3HOP TLC"}="HOP3 TLC"; $resn_exch{"1H5* TLC"}=" H5' TLC";
$resn_exch{"2H5* TLC"}="H5'' TLC"; $resn_exch{" H4* TLC"}=" H4' TLC"; $resn_exch{" H1* TLC"}=" H1' TLC";
$resn_exch{"1H5M TLC"}=" H71 TLC"; $resn_exch{"2H5M TLC"}=" H72 TLC"; $resn_exch{"3H5M TLC"}=" H73 TLC";
$resn_exch{" H2* TLC"}=" H2' TLC"; $resn_exch{"1H3L TLC"}="H3L1 TLC"; $resn_exch{"2H3L TLC"}="H3L2 TLC";
$resn_exch{"1H2L TLC"}="H2L1 TLC"; $resn_exch{"2H2L TLC"}="H2L2 TLC"; $resn_exch{"H3T  TLC"}="HO3' TLC";
$resn_exch{"1H13 TLD"}="H131 TLD"; $resn_exch{"2H13 TLD"}="H132 TLD"; $resn_exch{"3H13 TLD"}="H133 TLD";
$resn_exch{"1H5  TLM"}=" H51 TLM"; $resn_exch{"1H7  TLM"}=" H71 TLM"; $resn_exch{"1H8  TLM"}=" H81 TLM";
$resn_exch{"2H8  TLM"}=" H82 TLM"; $resn_exch{"1H9  TLM"}=" H91 TLM"; $resn_exch{"2H9  TLM"}=" H92 TLM";
$resn_exch{"3H9  TLM"}=" H93 TLM"; $resn_exch{"1H10 TLM"}="H101 TLM"; $resn_exch{"2H10 TLM"}="H102 TLM";
$resn_exch{"3H10 TLM"}="H103 TLM"; $resn_exch{"1H11 TLM"}="H111 TLM"; $resn_exch{"2H11 TLM"}="H112 TLM";
$resn_exch{"3H11 TLM"}="H113 TLM"; $resn_exch{"2HOP TLN"}="HOP2 TLN"; $resn_exch{"3HOP TLN"}="HOP3 TLN";
$resn_exch{"1H5* TLN"}=" H5' TLN"; $resn_exch{"2H5* TLN"}="H5'' TLN"; $resn_exch{" H1* TLN"}=" H1' TLN";
$resn_exch{"1H5M TLN"}=" H71 TLN"; $resn_exch{"2H5M TLN"}=" H72 TLN"; $resn_exch{"3H5M TLN"}=" H73 TLN";
$resn_exch{" H3* TLN"}=" H3' TLN"; $resn_exch{"1H2* TLN"}=" H2' TLN"; $resn_exch{"H3T  TLN"}="HO3' TLN";
$resn_exch{"1H6* TLN"}="H6'1 TLN"; $resn_exch{"2H6* TLN"}="H6'2 TLN"; $resn_exch{"1H2  TLX"}=" H21 TLX";
$resn_exch{"2H2  TLX"}=" H22 TLX"; $resn_exch{"1H3  TLX"}=" H31 TLX"; $resn_exch{"2H3  TLX"}=" H32 TLX";
$resn_exch{"1H4  TLX"}=" H41 TLX"; $resn_exch{"2H4  TLX"}=" H42 TLX"; $resn_exch{"1H6  TLX"}=" H61 TLX";
$resn_exch{"2H6  TLX"}=" H62 TLX"; $resn_exch{"1H7  TLX"}=" H71 TLX"; $resn_exch{"2H7  TLX"}=" H72 TLX";
$resn_exch{"1H8  TLX"}=" H81 TLX"; $resn_exch{"2H8  TLX"}=" H82 TLX"; $resn_exch{"1H1' TLX"}="H1'1 TLX";
$resn_exch{"2H1' TLX"}="H1'2 TLX"; $resn_exch{"1H3' TLX"}="H3'1 TLX"; $resn_exch{"2H3' TLX"}="H3'2 TLX";
$resn_exch{"1H5' TLX"}="H5'1 TLX"; $resn_exch{"2H5' TLX"}="H5'2 TLX"; $resn_exch{"3H5' TLX"}="H5'3 TLX";
$resn_exch{"1HM' TLX"}="HM'1 TLX"; $resn_exch{"2HM' TLX"}="HM'2 TLX"; $resn_exch{"3HM' TLX"}="HM'3 TLX";
$resn_exch{"1HN' TLX"}="HN'1 TLX"; $resn_exch{"2HN' TLX"}="HN'2 TLX"; $resn_exch{"1H5' TM2"}="H5'1 TM2";
$resn_exch{"2H5' TM2"}="H5'2 TM2"; $resn_exch{"1H7C TM2"}="H7C1 TM2"; $resn_exch{"2H7C TM2"}="H7C2 TM2";
$resn_exch{"1H9C TM2"}="H9C1 TM2"; $resn_exch{"2H9C TM2"}="H9C2 TM2"; $resn_exch{"1H11 TM2"}="H111 TM2";
$resn_exch{"2H11 TM2"}="H112 TM2"; $resn_exch{"1H61 TM5"}="H611 TM5"; $resn_exch{"2H61 TM5"}="H612 TM5";
$resn_exch{"1H62 TM5"}="H621 TM5"; $resn_exch{"2H62 TM5"}="H622 TM5"; $resn_exch{"1H63 TM5"}="H631 TM5";
$resn_exch{"2H63 TM5"}="H632 TM5"; $resn_exch{"1H64 TM5"}="H641 TM5"; $resn_exch{"2H64 TM5"}="H642 TM5";
$resn_exch{"1H65 TM5"}="H651 TM5"; $resn_exch{"2H65 TM5"}="H652 TM5"; $resn_exch{"1H62 TM6"}="H621 TM6";
$resn_exch{"2H62 TM6"}="H622 TM6"; $resn_exch{"1H63 TM6"}="H631 TM6"; $resn_exch{"2H63 TM6"}="H632 TM6";
$resn_exch{"1H64 TM6"}="H641 TM6"; $resn_exch{"2H64 TM6"}="H642 TM6"; $resn_exch{"1H65 TM6"}="H651 TM6";
$resn_exch{"2H65 TM6"}="H652 TM6"; $resn_exch{"1H66 TM6"}="H661 TM6"; $resn_exch{"2H66 TM6"}="H662 TM6";
$resn_exch{"1H67 TM6"}="H671 TM6"; $resn_exch{"2H67 TM6"}="H672 TM6"; $resn_exch{"1H1  TMA"}=" H11 TMA";
$resn_exch{"2H1  TMA"}=" H12 TMA"; $resn_exch{"3H1  TMA"}=" H13 TMA"; $resn_exch{"1H2  TMA"}=" H21 TMA";
$resn_exch{"2H2  TMA"}=" H22 TMA"; $resn_exch{"3H2  TMA"}=" H23 TMA"; $resn_exch{"1H3  TMA"}=" H31 TMA";
$resn_exch{"2H3  TMA"}=" H32 TMA"; $resn_exch{"3H3  TMA"}=" H33 TMA"; $resn_exch{"1H4  TMA"}=" H41 TMA";
$resn_exch{"2H4  TMA"}=" H42 TMA"; $resn_exch{"3H4  TMA"}=" H43 TMA"; $resn_exch{"1HCT TMB"}="HCT1 TMB";
$resn_exch{"2HCT TMB"}="HCT2 TMB"; $resn_exch{"3HCT TMB"}="HCT3 TMB"; $resn_exch{"1HCN TMB"}="HCN1 TMB";
$resn_exch{"2HCN TMB"}="HCN2 TMB"; $resn_exch{"3HCN TMB"}="HCN3 TMB"; $resn_exch{"1HD1 TMB"}="HD11 TMB";
$resn_exch{"2HD1 TMB"}="HD12 TMB"; $resn_exch{"3HD1 TMB"}="HD13 TMB"; $resn_exch{"1HD2 TMB"}="HD21 TMB";
$resn_exch{"2HD2 TMB"}="HD22 TMB"; $resn_exch{"1HH  TMB"}=" HH1 TMB"; $resn_exch{"2HH  TMB"}=" HH2 TMB";
$resn_exch{"3HH  TMB"}=" HH3 TMB"; $resn_exch{"2H14 TMC"}="H142 TMC"; $resn_exch{"1H14 TMC"}="H141 TMC";
$resn_exch{"2H15 TMC"}="H152 TMC"; $resn_exch{"1H15 TMC"}="H151 TMC"; $resn_exch{"2H12 TMC"}="H122 TMC";
$resn_exch{"1H12 TMC"}="H121 TMC"; $resn_exch{"3H3  TMC"}=" H33 TMC"; $resn_exch{"2H3  TMC"}=" H32 TMC";
$resn_exch{"1H3  TMC"}=" H31 TMC"; $resn_exch{" HN  TMD"}=" H   TMD"; $resn_exch{"1HN  TMD"}=" HN1 TMD";
$resn_exch{"2HN  TMD"}=" HN2 TMD"; $resn_exch{"3HN  TMD"}=" HN3 TMD"; $resn_exch{"1HD1 TMD"}="HD13 TMD";
$resn_exch{"2HD1 TMD"}="HD12 TMD"; $resn_exch{"1HD2 TMD"}="HD21 TMD"; $resn_exch{"2HD2 TMD"}="HD22 TMD";
$resn_exch{"3HD2 TMD"}="HD23 TMD"; $resn_exch{"1HE  TMD"}=" HE2 TMD"; $resn_exch{"2HE  TMD"}=" HE3 TMD";
$resn_exch{"1HZ  TMD"}=" HZ2 TMD"; $resn_exch{"2HZ  TMD"}=" HZ3 TMD"; $resn_exch{"1HH  TMD"}=" HH1 TMD";
$resn_exch{"2HH  TMD"}=" HH2 TMD"; $resn_exch{"3HH  TMD"}=" HH3 TMD"; $resn_exch{" HO2 TMD"}=" HG2 TMD";
$resn_exch{"1H1  TME"}=" H11 TME"; $resn_exch{"2H1  TME"}=" H12 TME"; $resn_exch{"3H1  TME"}=" H13 TME";
$resn_exch{"1H2  TME"}=" H21 TME"; $resn_exch{"2H2  TME"}=" H22 TME"; $resn_exch{"1H3  TME"}=" H31 TME";
$resn_exch{"2H3  TME"}=" H32 TME"; $resn_exch{"3H3  TME"}=" H33 TME"; $resn_exch{"1HN2 TMF"}="HN21 TMF";
$resn_exch{"2HN2 TMF"}="HN22 TMF"; $resn_exch{"1H9  TMF"}=" H91 TMF"; $resn_exch{"2H9  TMF"}=" H92 TMF";
$resn_exch{"1HB  TMF"}=" HB1 TMF"; $resn_exch{"2HB  TMF"}=" HB2 TMF"; $resn_exch{"1HG  TMF"}=" HG1 TMF";
$resn_exch{"2HG  TMF"}=" HG2 TMF"; $resn_exch{"2HOE TMF"}="HOE2 TMF"; $resn_exch{"1HCP TMF"}="HCP1 TMF";
$resn_exch{"2HCP TMF"}="HCP2 TMF"; $resn_exch{"1H6  TMH"}=" H61 TMH"; $resn_exch{"2H6  TMH"}=" H62 TMH";
$resn_exch{"1H4  TMH"}=" H41 TMH"; $resn_exch{"2H4  TMH"}=" H42 TMH"; $resn_exch{"1H8  TMH"}=" H81 TMH";
$resn_exch{"2H8  TMH"}=" H82 TMH"; $resn_exch{"3H8  TMH"}=" H83 TMH"; $resn_exch{"1H9  TMH"}=" H91 TMH";
$resn_exch{"2H9  TMH"}=" H92 TMH"; $resn_exch{"3H9  TMH"}=" H93 TMH"; $resn_exch{"1H10 TMH"}="H101 TMH";
$resn_exch{"2H10 TMH"}="H102 TMH"; $resn_exch{"3H10 TMH"}="H103 TMH"; $resn_exch{"1H1  TMJ"}="H11A TMJ";
$resn_exch{"2H1  TMJ"}="H12A TMJ"; $resn_exch{"1H2  TMJ"}=" H21 TMJ"; $resn_exch{"2H2  TMJ"}=" H22 TMJ";
$resn_exch{"1H3  TMJ"}=" H31 TMJ"; $resn_exch{"2H3  TMJ"}=" H32 TMJ"; $resn_exch{"1HM1 TML"}="HM11 TML";
$resn_exch{"2HM1 TML"}="HM12 TML"; $resn_exch{"3HM1 TML"}="HM13 TML"; $resn_exch{"1HM2 TML"}="HM21 TML";
$resn_exch{"2HM2 TML"}="HM22 TML"; $resn_exch{"3HM2 TML"}="HM23 TML"; $resn_exch{"1HM3 TML"}="HM31 TML";
$resn_exch{"2HM3 TML"}="HM32 TML"; $resn_exch{"3HM3 TML"}="HM33 TML"; $resn_exch{"1H1  TMN"}=" H11 TMN";
$resn_exch{"2H1  TMN"}=" H12 TMN"; $resn_exch{"1H2  TMN"}=" H21 TMN"; $resn_exch{"2H2  TMN"}=" H22 TMN";
$resn_exch{"1H3  TMN"}=" H31 TMN"; $resn_exch{"2H3  TMN"}=" H32 TMN"; $resn_exch{"1HN  TMN"}=" HN1 TMN";
$resn_exch{"2HN  TMN"}=" HN2 TMN"; $resn_exch{"2HOP TMP"}="HOP2 TMP"; $resn_exch{"3HOP TMP"}="HOP3 TMP";
$resn_exch{"1H5* TMP"}="H5'1 TMP"; $resn_exch{"2H5* TMP"}="H5'2 TMP"; $resn_exch{" H4* TMP"}=" H4' TMP";
$resn_exch{" H3* TMP"}=" H3' TMP"; $resn_exch{"*HO3 TMP"}="HO3' TMP"; $resn_exch{"1H2* TMP"}="H2'1 TMP";
$resn_exch{"2H2* TMP"}="H2'2 TMP"; $resn_exch{" H1* TMP"}=" H1' TMP"; $resn_exch{"1H5  TMP"}=" H51 TMP";
$resn_exch{"2H5  TMP"}=" H52 TMP"; $resn_exch{"3H5  TMP"}=" H53 TMP"; $resn_exch{"1H9  TMQ"}=" H91 TMQ";
$resn_exch{"2H9  TMQ"}=" H92 TMQ"; $resn_exch{"1H17 TMQ"}="H171 TMQ"; $resn_exch{"2H17 TMQ"}="H172 TMQ";
$resn_exch{"3H17 TMQ"}="H173 TMQ"; $resn_exch{"1H21 TMQ"}="H211 TMQ"; $resn_exch{"2H21 TMQ"}="H212 TMQ";
$resn_exch{"3H21 TMQ"}="H213 TMQ"; $resn_exch{"1H22 TMQ"}="H221 TMQ"; $resn_exch{"2H22 TMQ"}="H222 TMQ";
$resn_exch{"3H22 TMQ"}="H223 TMQ"; $resn_exch{"1H23 TMQ"}="H231 TMQ"; $resn_exch{"2H23 TMQ"}="H232 TMQ";
$resn_exch{"3H23 TMQ"}="H233 TMQ"; $resn_exch{"1HN2 TMQ"}="HN21 TMQ"; $resn_exch{"2HN2 TMQ"}="HN22 TMQ";
$resn_exch{"1HN5 TMQ"}="HN51 TMQ"; $resn_exch{"2HN5 TMQ"}="HN52 TMQ"; $resn_exch{"1H2  TMR"}=" H21 TMR";
$resn_exch{"2H2  TMR"}=" H22 TMR"; $resn_exch{"1H6  TMR"}=" H61 TMR"; $resn_exch{"2H6  TMR"}=" H62 TMR";
$resn_exch{"3H6  TMR"}=" H63 TMR"; $resn_exch{"1H7  TMR"}=" H71 TMR"; $resn_exch{"2H7  TMR"}=" H72 TMR";
$resn_exch{"3H7  TMR"}=" H73 TMR"; $resn_exch{"1H6  TMT"}=" H61 TMT"; $resn_exch{"2H6  TMT"}=" H62 TMT";
$resn_exch{"3H6  TMT"}=" H63 TMT"; $resn_exch{"1H7  TMT"}=" H71 TMT"; $resn_exch{"2H7  TMT"}=" H72 TMT";
$resn_exch{"3H7  TMT"}=" H73 TMT"; $resn_exch{"1H8  TMT"}=" H81 TMT"; $resn_exch{"2H8  TMT"}=" H82 TMT";
$resn_exch{"3H8  TMT"}=" H83 TMT"; $resn_exch{"1H13 TMU"}="H131 TMU"; $resn_exch{"2H13 TMU"}="H132 TMU";
$resn_exch{"1H21 TMU"}="H211 TMU"; $resn_exch{"2H21 TMU"}="H212 TMU"; $resn_exch{"3H21 TMU"}="H213 TMU";
$resn_exch{"1H6  TMZ"}=" H61 TMZ"; $resn_exch{"2H6  TMZ"}=" H62 TMZ"; $resn_exch{"3H6  TMZ"}=" H63 TMZ";
$resn_exch{"1H7  TMZ"}=" H71 TMZ"; $resn_exch{"2H7  TMZ"}=" H72 TMZ"; $resn_exch{"3H7  TMZ"}=" H73 TMZ";
$resn_exch{"1H8  TMZ"}=" H81 TMZ"; $resn_exch{"2H8  TMZ"}=" H82 TMZ"; $resn_exch{"3H8  TMZ"}=" H83 TMZ";
$resn_exch{"1H12 TN3"}="H121 TN3"; $resn_exch{"2H12 TN3"}="H122 TN3"; $resn_exch{"3H12 TN3"}="H123 TN3";
$resn_exch{"1H5  TN4"}=" H51 TN4"; $resn_exch{"2H5  TN4"}=" H52 TN4"; $resn_exch{"1H6  TN4"}=" H61 TN4";
$resn_exch{"2H6  TN4"}=" H62 TN4"; $resn_exch{"1H11 TN4"}="H111 TN4"; $resn_exch{"2H11 TN4"}="H112 TN4";
$resn_exch{"1H31 TN4"}="H311 TN4"; $resn_exch{"2H31 TN4"}="H312 TN4"; $resn_exch{"3H31 TN4"}="H313 TN4";
$resn_exch{"1H23 TN4"}="H231 TN4"; $resn_exch{"2H23 TN4"}="H232 TN4"; $resn_exch{"1H2B TN4"}="H2B1 TN4";
$resn_exch{"2H2B TN4"}="H2B2 TN4"; $resn_exch{"3H2B TN4"}="H2B3 TN4"; $resn_exch{"1H2C TN4"}="H2C1 TN4";
$resn_exch{"2H2C TN4"}="H2C2 TN4"; $resn_exch{"3H2C TN4"}="H2C3 TN4"; $resn_exch{"1H2A TN4"}="H2A1 TN4";
$resn_exch{"2H2A TN4"}="H2A2 TN4"; $resn_exch{"3H2A TN4"}="H2A3 TN4"; $resn_exch{"1H44 TN4"}="H441 TN4";
$resn_exch{"2H44 TN4"}="H442 TN4"; $resn_exch{"1HN1 TN5"}="HN11 TN5"; $resn_exch{"2HN1 TN5"}="HN12 TN5";
$resn_exch{"1HN  TNB"}=" H   TNB"; $resn_exch{"2HN  TNB"}=" H2  TNB"; $resn_exch{"1HB  TNB"}=" HB2 TNB";
$resn_exch{"2HB  TNB"}=" HB3 TNB"; $resn_exch{"1H11 TNC"}="H111 TNC"; $resn_exch{"2H11 TNC"}="H112 TNC";
$resn_exch{"1HN2 TNC"}="HN21 TNC"; $resn_exch{"2HN2 TNC"}="HN22 TNC"; $resn_exch{"1H20 TNC"}="H201 TNC";
$resn_exch{"2H20 TNC"}="H202 TNC"; $resn_exch{"3H20 TNC"}="H203 TNC"; $resn_exch{"1H21 TNC"}="H211 TNC";
$resn_exch{"2H21 TNC"}="H212 TNC"; $resn_exch{"3H21 TNC"}="H213 TNC"; $resn_exch{"1H5* TND"}="H5'1 TND";
$resn_exch{"2H5* TND"}="H5'2 TND"; $resn_exch{" H4* TND"}=" H4' TND"; $resn_exch{" H3* TND"}=" H3' TND";
$resn_exch{"*HO3 TND"}="HO3' TND"; $resn_exch{" H2* TND"}=" H2' TND"; $resn_exch{"*HN2 TND"}="HN2' TND";
$resn_exch{" H1* TND"}=" H1' TND"; $resn_exch{"1H2  TND"}=" H21 TND"; $resn_exch{"2H2  TND"}=" H22 TND";
$resn_exch{"1H3  TND"}=" H31 TND"; $resn_exch{"2H3  TND"}=" H32 TND"; $resn_exch{"1H4  TND"}=" H41 TND";
$resn_exch{"2H4  TND"}=" H42 TND"; $resn_exch{"1H2M TND"}="H2M1 TND"; $resn_exch{"2H2M TND"}="H2M2 TND";
$resn_exch{"3H2M TND"}="H2M3 TND"; $resn_exch{"1H1M TND"}="H1M1 TND"; $resn_exch{"2H1M TND"}="H1M2 TND";
$resn_exch{"3H1M TND"}="H1M3 TND"; $resn_exch{"*HO5 TND"}="HO5' TND"; $resn_exch{"1H2  TNE"}=" H21 TNE";
$resn_exch{"2H2  TNE"}=" H22 TNE"; $resn_exch{"1H4  TNE"}=" H41 TNE"; $resn_exch{"2H4  TNE"}=" H42 TNE";
$resn_exch{"1H6  TNE"}=" H61 TNE"; $resn_exch{"2H6  TNE"}=" H62 TNE"; $resn_exch{"1H7  TNE"}=" H71 TNE";
$resn_exch{"2H7  TNE"}=" H72 TNE"; $resn_exch{"1H9  TNE"}=" H91 TNE"; $resn_exch{"2H9  TNE"}=" H92 TNE";
$resn_exch{"3H9  TNE"}=" H93 TNE"; $resn_exch{"1H13 TNK"}="H131 TNK"; $resn_exch{"2H13 TNK"}="H132 TNK";
$resn_exch{"1H15 TNK"}="H151 TNK"; $resn_exch{"2H15 TNK"}="H152 TNK"; $resn_exch{"3H15 TNK"}="H153 TNK";
$resn_exch{"1H16 TNK"}="H161 TNK"; $resn_exch{"2H16 TNK"}="H162 TNK"; $resn_exch{"3H16 TNK"}="H163 TNK";
$resn_exch{"1H17 TNK"}="H171 TNK"; $resn_exch{"2H17 TNK"}="H172 TNK"; $resn_exch{"1H18 TNK"}="H181 TNK";
$resn_exch{"2H18 TNK"}="H182 TNK"; $resn_exch{"1H7C TNL"}="H7C1 TNL"; $resn_exch{"2H7C TNL"}="H7C2 TNL";
$resn_exch{"3H7C TNL"}="H7C3 TNL"; $resn_exch{"1H15 TNP"}="H151 TNP"; $resn_exch{"2H15 TNP"}="H152 TNP";
$resn_exch{"3H15 TNP"}="H153 TNP"; $resn_exch{"1H16 TNP"}="H161 TNP"; $resn_exch{"2H16 TNP"}="H162 TNP";
$resn_exch{"3H16 TNP"}="H163 TNP"; $resn_exch{"1H17 TNP"}="H171 TNP"; $resn_exch{"2H17 TNP"}="H172 TNP";
$resn_exch{"3H17 TNP"}="H173 TNP"; $resn_exch{"1H18 TNP"}="H181 TNP"; $resn_exch{"2H18 TNP"}="H182 TNP";
$resn_exch{"3H18 TNP"}="H183 TNP"; $resn_exch{"1H19 TNP"}="H191 TNP"; $resn_exch{"2H19 TNP"}="H192 TNP";
$resn_exch{"3H19 TNP"}="H193 TNP"; $resn_exch{"1HN  TNR"}=" H   TNR"; $resn_exch{"2HN  TNR"}=" H2  TNR";
$resn_exch{"1HCB TNR"}=" HB2 TNR"; $resn_exch{"2HCB TNR"}=" HB3 TNR"; $resn_exch{"1HC8 TNR"}="HC81 TNR";
$resn_exch{"2HC8 TNR"}="HC82 TNR"; $resn_exch{"3HC8 TNR"}="HC83 TNR"; $resn_exch{"1HC6 TNR"}="HC61 TNR";
$resn_exch{"2HC6 TNR"}="HC62 TNR"; $resn_exch{"1H19 TNS"}="H191 TNS"; $resn_exch{"2H19 TNS"}="H192 TNS";
$resn_exch{"1H18 TNS"}="H181 TNS"; $resn_exch{"2H18 TNS"}="H182 TNS"; $resn_exch{"1H17 TNS"}="H171 TNS";
$resn_exch{"2H17 TNS"}="H172 TNS"; $resn_exch{"1H16 TNS"}="H161 TNS"; $resn_exch{"2H16 TNS"}="H162 TNS";
$resn_exch{"1H23 TNS"}="H231 TNS"; $resn_exch{"2H23 TNS"}="H232 TNS"; $resn_exch{"3H23 TNS"}="H233 TNS";
$resn_exch{"1HN  TNT"}=" HN1 TNT"; $resn_exch{"2HN  TNT"}=" HN2 TNT"; $resn_exch{"3HN  TNT"}=" HN3 TNT";
$resn_exch{"1HN' TNT"}="HN'1 TNT"; $resn_exch{"2HN' TNT"}="HN'2 TNT"; $resn_exch{"3HN' TNT"}="HN'3 TNT";
$resn_exch{"1HA  TNT"}=" HA1 TNT"; $resn_exch{"2HA  TNT"}=" HA2 TNT"; $resn_exch{"1HB  TNT"}=" HB1 TNT";
$resn_exch{"2HB  TNT"}=" HB2 TNT"; $resn_exch{"1HA' TNT"}="HA'1 TNT"; $resn_exch{"2HA' TNT"}="HA'2 TNT";
$resn_exch{"1H9' TNV"}="H9'1 TNV"; $resn_exch{"2H9' TNV"}="H9'2 TNV"; $resn_exch{"1H8' TNV"}="H8'1 TNV";
$resn_exch{"2H8' TNV"}="H8'2 TNV"; $resn_exch{"3H8' TNV"}="H8'3 TNV"; $resn_exch{"1H6' TNV"}="H6'1 TNV";
$resn_exch{"2H6' TNV"}="H6'2 TNV"; $resn_exch{"1HN6 TNV"}="HN61 TNV"; $resn_exch{"2HN6 TNV"}="HN62 TNV";
$resn_exch{"1H6  TOA"}=" H61 TOA"; $resn_exch{"2H6  TOA"}=" H62 TOA"; $resn_exch{"1HN3 TOA"}="HN31 TOA";
$resn_exch{"2HN3 TOA"}="HN32 TOA"; $resn_exch{"3HN3 TOA"}="HN33 TOA"; $resn_exch{"1H2  TOB"}=" H21 TOB";
$resn_exch{"2H2  TOB"}=" H22 TOB"; $resn_exch{"1H4  TOB"}=" H41 TOB"; $resn_exch{"2H4  TOB"}=" H42 TOB";
$resn_exch{"1HN1 TOB"}="HN11 TOB"; $resn_exch{"2HN1 TOB"}="HN12 TOB"; $resn_exch{"3HN1 TOB"}="HN13 TOB";
$resn_exch{"1HN3 TOB"}="HN31 TOB"; $resn_exch{"2HN3 TOB"}="HN32 TOB"; $resn_exch{"3HN3 TOB"}="HN33 TOB";
$resn_exch{"1H3  TOC"}=" H31 TOC"; $resn_exch{"2H3  TOC"}=" H32 TOC"; $resn_exch{"1H6  TOC"}=" H61 TOC";
$resn_exch{"2H6  TOC"}=" H62 TOC"; $resn_exch{"1HN2 TOC"}="HN21 TOC"; $resn_exch{"2HN2 TOC"}="HN22 TOC";
$resn_exch{"3HN2 TOC"}="HN23 TOC"; $resn_exch{"1HN6 TOC"}="HN61 TOC"; $resn_exch{"2HN6 TOC"}="HN62 TOC";
$resn_exch{"3HN6 TOC"}="HN63 TOC"; $resn_exch{"1HA* TOE"}="HA'1 TOE"; $resn_exch{"2HA* TOE"}="HA'2 TOE";
$resn_exch{"1HB* TOE"}="HB'1 TOE"; $resn_exch{"2HB* TOE"}="HB'2 TOE"; $resn_exch{"1HD* TOE"}="HD'1 TOE";
$resn_exch{"2HD* TOE"}="HD'2 TOE"; $resn_exch{"1HE* TOE"}="HE'1 TOE"; $resn_exch{"2HE* TOE"}="HE'2 TOE";
$resn_exch{"1HG* TOE"}="HG'1 TOE"; $resn_exch{"2HG* TOE"}="HG'2 TOE"; $resn_exch{"1HH* TOE"}="HH'1 TOE";
$resn_exch{"2HH* TOE"}="HH'2 TOE"; $resn_exch{"1HK* TOE"}="HK'1 TOE"; $resn_exch{"2HK* TOE"}="HK'2 TOE";
$resn_exch{"3HK* TOE"}="HK'3 TOE"; $resn_exch{"1H8  TOH"}=" H81 TOH"; $resn_exch{"1H7  TOH"}=" H71 TOH";
$resn_exch{"1H4  TOL"}=" H41 TOL"; $resn_exch{"2H4  TOL"}=" H42 TOL"; $resn_exch{"3H4  TOL"}=" H43 TOL";
$resn_exch{"1H14 TOL"}="H141 TOL"; $resn_exch{"2H14 TOL"}="H142 TOL"; $resn_exch{"3H14 TOL"}="H143 TOL";
$resn_exch{"1H15 TOL"}="H151 TOL"; $resn_exch{"2H15 TOL"}="H152 TOL"; $resn_exch{"1H11 TOM"}="H111 TOM";
$resn_exch{"2H11 TOM"}="H112 TOM"; $resn_exch{"3H11 TOM"}="H113 TOM"; $resn_exch{"1H1  TON"}=" H11 TON";
$resn_exch{"2H1  TON"}=" H12 TON"; $resn_exch{"1H2  TON"}=" H21 TON"; $resn_exch{"2H2  TON"}=" H22 TON";
$resn_exch{"1H3  TON"}=" H31 TON"; $resn_exch{"2H3  TON"}=" H32 TON"; $resn_exch{"1H4  TON"}=" H41 TON";
$resn_exch{"2H4  TON"}=" H42 TON"; $resn_exch{"2H16 TON"}="H162 TON"; $resn_exch{"1H8  TON"}=" H81 TON";
$resn_exch{"2H8  TON"}=" H82 TON"; $resn_exch{"3H8  TON"}=" H83 TON"; $resn_exch{"1H7  TON"}=" H71 TON";
$resn_exch{"2H7  TON"}=" H72 TON"; $resn_exch{"3H7  TON"}=" H73 TON"; $resn_exch{"1H6  TON"}=" H61 TON";
$resn_exch{"2H6  TON"}=" H62 TON"; $resn_exch{"1H12 TON"}="H121 TON"; $resn_exch{"2H12 TON"}="H122 TON";
$resn_exch{"3H12 TON"}="H123 TON"; $resn_exch{"1H11 TON"}="H111 TON"; $resn_exch{"2H11 TON"}="H112 TON";
$resn_exch{"3H11 TON"}="H113 TON"; $resn_exch{"1H10 TON"}="H101 TON"; $resn_exch{"2H10 TON"}="H102 TON";
$resn_exch{"3H10 TON"}="H103 TON"; $resn_exch{"1HN4 TOP"}="HN41 TOP"; $resn_exch{"2HN4 TOP"}="HN42 TOP";
$resn_exch{"1HN7 TOP"}="HN71 TOP"; $resn_exch{"2HN7 TOP"}="HN72 TOP"; $resn_exch{"1H9  TOP"}=" H91 TOP";
$resn_exch{"2H9  TOP"}=" H92 TOP"; $resn_exch{"1H14 TOP"}="H141 TOP"; $resn_exch{"2H14 TOP"}="H142 TOP";
$resn_exch{"3H14 TOP"}="H143 TOP"; $resn_exch{"1H17 TOP"}="H171 TOP"; $resn_exch{"2H17 TOP"}="H172 TOP";
$resn_exch{"3H17 TOP"}="H173 TOP"; $resn_exch{"1H20 TOP"}="H201 TOP"; $resn_exch{"2H20 TOP"}="H202 TOP";
$resn_exch{"3H20 TOP"}="H203 TOP"; $resn_exch{"2HOS TOS"}="HOS2 TOS"; $resn_exch{"1H   TOS"}=" H1  TOS";
$resn_exch{"2H   TOS"}=" H2A TOS"; $resn_exch{"3H   TOS"}=" H3A TOS"; $resn_exch{"1H6  TOT"}=" H61 TOT";
$resn_exch{"2H6  TOT"}=" H62 TOT"; $resn_exch{"3H6  TOT"}=" H63 TOT"; $resn_exch{"1H22 TOT"}="H221 TOT";
$resn_exch{"2H22 TOT"}="H222 TOT"; $resn_exch{"1H23 TOT"}="H231 TOT"; $resn_exch{"2H23 TOT"}="H232 TOT";
$resn_exch{"1H24 TOT"}="H241 TOT"; $resn_exch{"2H24 TOT"}="H242 TOT"; $resn_exch{"1H26 TOT"}="H261 TOT";
$resn_exch{"2H26 TOT"}="H262 TOT"; $resn_exch{"1H27 TOT"}="H271 TOT"; $resn_exch{"2H27 TOT"}="H272 TOT";
$resn_exch{"1H28 TOT"}="H281 TOT"; $resn_exch{"2H28 TOT"}="H282 TOT"; $resn_exch{"3H28 TOT"}="H283 TOT";
$resn_exch{"1H29 TOT"}="H291 TOT"; $resn_exch{"2H29 TOT"}="H292 TOT"; $resn_exch{"3H29 TOT"}="H293 TOT";
$resn_exch{"1H36 TOT"}="H361 TOT"; $resn_exch{"2H36 TOT"}="H362 TOT"; $resn_exch{"3H36 TOT"}="H363 TOT";
$resn_exch{"1H52 TOT"}="H521 TOT"; $resn_exch{"2H52 TOT"}="H522 TOT"; $resn_exch{"1H53 TOT"}="H531 TOT";
$resn_exch{"2H53 TOT"}="H532 TOT"; $resn_exch{"1H54 TOT"}="H541 TOT"; $resn_exch{"2H54 TOT"}="H542 TOT";
$resn_exch{"1H56 TOT"}="H561 TOT"; $resn_exch{"2H56 TOT"}="H562 TOT"; $resn_exch{"1H58 TOT"}="H581 TOT";
$resn_exch{"2H58 TOT"}="H582 TOT"; $resn_exch{"3H58 TOT"}="H583 TOT"; $resn_exch{"1H59 TOT"}="H591 TOT";
$resn_exch{"2H59 TOT"}="H592 TOT"; $resn_exch{"3H59 TOT"}="H593 TOT"; $resn_exch{"1HN1 TOU"}="HN11 TOU";
$resn_exch{"2HN1 TOU"}="HN12 TOU"; $resn_exch{"1HN2 TOU"}="HN21 TOU"; $resn_exch{"2HN2 TOU"}="HN22 TOU";
$resn_exch{"1HN  TOX"}=" H   TOX"; $resn_exch{"2HN  TOX"}=" H2  TOX"; $resn_exch{"1H   TOX"}=" HC1 TOX";
$resn_exch{"2H   TOX"}=" HC2 TOX"; $resn_exch{"1HB  TOX"}=" HB2 TOX"; $resn_exch{"2HB  TOX"}=" HB3 TOX";
$resn_exch{"1HN2 TOY"}="HN21 TOY"; $resn_exch{"2HN2 TOY"}="HN22 TOY"; $resn_exch{"1H31 TOY"}="H311 TOY";
$resn_exch{"2H31 TOY"}="H312 TOY"; $resn_exch{"OH41 TOY"}="H41O TOY"; $resn_exch{"1H61 TOY"}="H611 TOY";
$resn_exch{"2H61 TOY"}="H612 TOY"; $resn_exch{"1HN6 TOY"}="HN61 TOY"; $resn_exch{"2HN6 TOY"}="HN62 TOY";
$resn_exch{"1HN1 TOY"}="HN11 TOY"; $resn_exch{"2HN1 TOY"}="HN12 TOY"; $resn_exch{"1H22 TOY"}="H221 TOY";
$resn_exch{"2H22 TOY"}="H222 TOY"; $resn_exch{"1HN  TOY"}=" HN1 TOY"; $resn_exch{"2HN  TOY"}=" HN2 TOY";
$resn_exch{"OH52 TOY"}="H52O TOY"; $resn_exch{"OH23 TOY"}="H23O TOY"; $resn_exch{"1HN3 TOY"}="HN31 TOY";
$resn_exch{"2HN3 TOY"}="HN32 TOY"; $resn_exch{"OH43 TOY"}="H43O TOY"; $resn_exch{"1H63 TOY"}="H631 TOY";
$resn_exch{"2H63 TOY"}="H632 TOY"; $resn_exch{"OH63 TOY"}="H63O TOY"; $resn_exch{"1H8' TP1"}="H8'1 TP1";
$resn_exch{"2H8' TP1"}="H8'2 TP1"; $resn_exch{"1H5' TP1"}="H5'1 TP1"; $resn_exch{"2H5' TP1"}="H5'2 TP1";
$resn_exch{"1H3' TP1"}="H3'1 TP1"; $resn_exch{"2H3' TP1"}="H3'2 TP1"; $resn_exch{"1H2' TP1"}="H2'1 TP1";
$resn_exch{"2H2' TP1"}="H2'2 TP1"; $resn_exch{"1H1' TP1"}="H1'1 TP1"; $resn_exch{"2H1' TP1"}="H1'2 TP1";
$resn_exch{"3H1' TP1"}="H1'3 TP1"; $resn_exch{"1HM5 TP1"}="HM51 TP1"; $resn_exch{"2HM5 TP1"}="HM52 TP1";
$resn_exch{"3HM5 TP1"}="HM53 TP1"; $resn_exch{"1HC2 TP2"}="HC21 TP2"; $resn_exch{"2HC2 TP2"}="HC22 TP2";
$resn_exch{"1HC3 TP2"}="HC31 TP2"; $resn_exch{"2HC3 TP2"}="HC32 TP2"; $resn_exch{"1HN4 TP2"}="HN41 TP2";
$resn_exch{"1HC6 TP2"}="HC61 TP2"; $resn_exch{"1H10 TP2"}="H101 TP2"; $resn_exch{"1H11 TP2"}="H111 TP2";
$resn_exch{"1H13 TP2"}="H131 TP2"; $resn_exch{"1H14 TP2"}="H141 TP2"; $resn_exch{"1H15 TP2"}="H151 TP2";
$resn_exch{"2H15 TP2"}="H152 TP2"; $resn_exch{"3H15 TP2"}="H153 TP2"; $resn_exch{"1H18 TP2"}="H181 TP2";
$resn_exch{"2H18 TP2"}="H182 TP2"; $resn_exch{"1H19 TP2"}="H191 TP2"; $resn_exch{"2H19 TP2"}="H192 TP2";
$resn_exch{"1H20 TP2"}="H201 TP2"; $resn_exch{"2H20 TP2"}="H202 TP2"; $resn_exch{"1HC1 TP3"}="HC11 TP3";
$resn_exch{"2HC1 TP3"}="HC12 TP3"; $resn_exch{"1HC2 TP3"}="HC21 TP3"; $resn_exch{"2HC2 TP3"}="HC22 TP3";
$resn_exch{"1HC4 TP3"}="HC41 TP3"; $resn_exch{"1HC5 TP3"}="HC51 TP3"; $resn_exch{"2HC5 TP3"}="HC52 TP3";
$resn_exch{"1HO8 TP3"}="HO81 TP3"; $resn_exch{"1H13 TP3"}="H131 TP3"; $resn_exch{"1H14 TP3"}="H141 TP3";
$resn_exch{"1H16 TP3"}="H161 TP3"; $resn_exch{"1H17 TP3"}="H171 TP3"; $resn_exch{"1H19 TP3"}="H191 TP3";
$resn_exch{"1H20 TP3"}="H201 TP3"; $resn_exch{"1H21 TP3"}="H211 TP3"; $resn_exch{"2H21 TP3"}="H212 TP3";
$resn_exch{"1H22 TP3"}="H221 TP3"; $resn_exch{"2H22 TP3"}="H222 TP3"; $resn_exch{"1H27 TP3"}="H271 TP3";
$resn_exch{"1H28 TP3"}="H281 TP3"; $resn_exch{"1HC1 TP4"}="HC11 TP4"; $resn_exch{"2HC1 TP4"}="HC12 TP4";
$resn_exch{"1HC2 TP4"}="HC21 TP4"; $resn_exch{"2HC2 TP4"}="HC22 TP4"; $resn_exch{"1HC4 TP4"}="HC41 TP4";
$resn_exch{"1HC5 TP4"}="HC51 TP4"; $resn_exch{"2HC5 TP4"}="HC52 TP4"; $resn_exch{"1HN8 TP4"}="HN81 TP4";
$resn_exch{"1H13 TP4"}="H131 TP4"; $resn_exch{"1H14 TP4"}="H141 TP4"; $resn_exch{"1H16 TP4"}="H161 TP4";
$resn_exch{"1H17 TP4"}="H171 TP4"; $resn_exch{"1H19 TP4"}="H191 TP4"; $resn_exch{"1H20 TP4"}="H201 TP4";
$resn_exch{"1H21 TP4"}="H211 TP4"; $resn_exch{"2H21 TP4"}="H212 TP4"; $resn_exch{"1H22 TP4"}="H221 TP4";
$resn_exch{"2H22 TP4"}="H222 TP4"; $resn_exch{"1H27 TP4"}="H271 TP4"; $resn_exch{"1H28 TP4"}="H281 TP4";
$resn_exch{"1H30 TP4"}="H301 TP4"; $resn_exch{"2H30 TP4"}="H302 TP4"; $resn_exch{"1H31 TP4"}="H311 TP4";
$resn_exch{"2H31 TP4"}="H312 TP4"; $resn_exch{"1H33 TP4"}="H331 TP4"; $resn_exch{"1HN2 TP5"}="HN21 TP5";
$resn_exch{"2HN2 TP5"}="HN22 TP5"; $resn_exch{"1H19 TP6"}="H191 TP6"; $resn_exch{"2H19 TP6"}="H192 TP6";
$resn_exch{"1H18 TP6"}="H181 TP6"; $resn_exch{"2H18 TP6"}="H182 TP6"; $resn_exch{"1H17 TP6"}="H171 TP6";
$resn_exch{"2H17 TP6"}="H172 TP6"; $resn_exch{"1H16 TP6"}="H161 TP6"; $resn_exch{"2H16 TP6"}="H162 TP6";
$resn_exch{"1H15 TP6"}="H151 TP6"; $resn_exch{"2H15 TP6"}="H152 TP6"; $resn_exch{"1H10 TP6"}="H101 TP6";
$resn_exch{"2H10 TP6"}="H102 TP6"; $resn_exch{"1H14 TP6"}="H141 TP6"; $resn_exch{"2H14 TP6"}="H142 TP6";
$resn_exch{"2HOP TP7"}="HOP2 TP7"; $resn_exch{"3HOP TP7"}="HOP3 TP7"; $resn_exch{"1HG  TP7"}=" HG1 TP7";
$resn_exch{"2HG  TP7"}=" HG2 TP7"; $resn_exch{"3HG  TP7"}=" HG3 TP7"; $resn_exch{"1H2  TP7"}=" H21 TP7";
$resn_exch{"2H2  TP7"}=" H22 TP7"; $resn_exch{"1H3  TP7"}=" H31 TP7"; $resn_exch{"2H3  TP7"}=" H32 TP7";
$resn_exch{"1H4  TP7"}=" H41 TP7"; $resn_exch{"2H4  TP7"}=" H42 TP7"; $resn_exch{"1H5  TP7"}=" H51 TP7";
$resn_exch{"2H5  TP7"}=" H52 TP7"; $resn_exch{"1H6  TP7"}=" H61 TP7"; $resn_exch{"2H6  TP7"}=" H62 TP7";
$resn_exch{"1H7  TP7"}=" H71 TP7"; $resn_exch{"2H7  TP7"}=" H72 TP7"; $resn_exch{"1H10 TP8"}="H101 TP8";
$resn_exch{"2H10 TP8"}="H102 TP8"; $resn_exch{"1H6  TP8"}=" H61 TP8"; $resn_exch{"2H6  TP8"}=" H62 TP8";
$resn_exch{"1H7  TP8"}=" H71 TP8"; $resn_exch{"2H7  TP8"}=" H72 TP8"; $resn_exch{"1HM4 TP8"}="HM41 TP8";
$resn_exch{"2HM4 TP8"}="HM42 TP8"; $resn_exch{"3HM4 TP8"}="HM43 TP8"; $resn_exch{"1H7, TP8"}="H7,1 TP8";
$resn_exch{"2H7, TP8"}="H7,2 TP8"; $resn_exch{"1HM2 TP8"}="HM21 TP8"; $resn_exch{"2HM2 TP8"}="HM22 TP8";
$resn_exch{"3HM2 TP8"}="HM23 TP8"; $resn_exch{"1H7  TP9"}=" H71 TP9"; $resn_exch{"2H7  TP9"}=" H72 TP9";
$resn_exch{"1H6  TP9"}=" H61 TP9"; $resn_exch{"2H6  TP9"}=" H62 TP9"; $resn_exch{"1H7' TP9"}="H7'1 TP9";
$resn_exch{"2H7' TP9"}="H7'2 TP9"; $resn_exch{"1H4' TP9"}="H4'1 TP9"; $resn_exch{"2H4' TP9"}="H4'2 TP9";
$resn_exch{"1HM2 TP9"}="HM21 TP9"; $resn_exch{"2HM2 TP9"}="HM22 TP9"; $resn_exch{"3HM2 TP9"}="HM23 TP9";
$resn_exch{"1HM4 TP9"}="HM41 TP9"; $resn_exch{"2HM4 TP9"}="HM42 TP9"; $resn_exch{"3HM4 TP9"}="HM43 TP9";
$resn_exch{"1H3  TPA"}=" H31 TPA"; $resn_exch{"2H3  TPA"}=" H32 TPA"; $resn_exch{"1HN  TPA"}=" HN1 TPA";
$resn_exch{"2HN  TPA"}=" HN2 TPA"; $resn_exch{"3HN  TPA"}=" HN3 TPA"; $resn_exch{"1H21 TPB"}="H211 TPB";
$resn_exch{"2H21 TPB"}="H212 TPB"; $resn_exch{"3H21 TPB"}="H213 TPB"; $resn_exch{"1H61 TPB"}="H611 TPB";
$resn_exch{"2H61 TPB"}="H612 TPB"; $resn_exch{"3H61 TPB"}="H613 TPB"; $resn_exch{"1H41 TPB"}="H411 TPB";
$resn_exch{"2H41 TPB"}="H412 TPB"; $resn_exch{"3H41 TPB"}="H413 TPB"; $resn_exch{"1HN4 TPC"}="HN41 TPC";
$resn_exch{"2HN4 TPC"}="HN42 TPC"; $resn_exch{"1H2' TPC"}="H2'1 TPC"; $resn_exch{"2H2' TPC"}="H2'2 TPC";
$resn_exch{"1H5' TPC"}="H5'1 TPC"; $resn_exch{"2H5' TPC"}="H5'2 TPC"; $resn_exch{"1H9  TPD"}=" H91 TPD";
$resn_exch{"2H9  TPD"}=" H92 TPD"; $resn_exch{"1HNL TPD"}="HNL1 TPD"; $resn_exch{"2HNL TPD"}="HNL2 TPD";
$resn_exch{"2HPA TPE"}="HPA2 TPE"; $resn_exch{"2HPB TPE"}="HPB2 TPE"; $resn_exch{"2H5* TPE"}="H5'2 TPE";
$resn_exch{"1H5* TPE"}="H5'1 TPE"; $resn_exch{" H4* TPE"}=" H4' TPE"; $resn_exch{" H3* TPE"}=" H3' TPE";
$resn_exch{"3HHR TPE"}="HHR3 TPE"; $resn_exch{"2HHR TPE"}="HHR2 TPE"; $resn_exch{"1HHR TPE"}="HHR1 TPE";
$resn_exch{"2H2* TPE"}="H2'2 TPE"; $resn_exch{"1H2* TPE"}="H2'1 TPE"; $resn_exch{" H1* TPE"}=" H1' TPE";
$resn_exch{"3H5M TPE"}="H5M3 TPE"; $resn_exch{"2H5M TPE"}="H5M2 TPE"; $resn_exch{"1H5M TPE"}="H5M1 TPE";
$resn_exch{"1HC2 TPF"}="HC21 TPF"; $resn_exch{"2HC2 TPF"}="HC22 TPF"; $resn_exch{"1HC5 TPF"}="HC51 TPF";
$resn_exch{"2HC5 TPF"}="HC52 TPF"; $resn_exch{"1HBZ TPG"}="HBZ1 TPG"; $resn_exch{"2HBZ TPG"}="HBZ2 TPG";
$resn_exch{"3HBZ TPG"}="HBZ3 TPG"; $resn_exch{"1HBW TPG"}="HBW1 TPG"; $resn_exch{"2HBW TPG"}="HBW2 TPG";
$resn_exch{"3HBW TPG"}="HBW3 TPG"; $resn_exch{"1HBC TPG"}="HBC1 TPG"; $resn_exch{"2HBC TPG"}="HBC2 TPG";
$resn_exch{"3HBC TPG"}="HBC3 TPG"; $resn_exch{"1HBI TPG"}="HBI1 TPG"; $resn_exch{"2HBI TPG"}="HBI2 TPG";
$resn_exch{"1HBT TPG"}="HBT1 TPG"; $resn_exch{"2HBT TPG"}="HBT2 TPG"; $resn_exch{"1H5* TPG"}=" H5' TPG";
$resn_exch{"2H5* TPG"}="H5'' TPG"; $resn_exch{" H4* TPG"}=" H4' TPG"; $resn_exch{" H3* TPG"}=" H3' TPG";
$resn_exch{" H2* TPG"}=" H2' TPG"; $resn_exch{" H1* TPG"}=" H1' TPG"; $resn_exch{"1HN2 TPG"}="HN21 TPG";
$resn_exch{"2HN2 TPG"}="HN22 TPG"; $resn_exch{"1NH  TPH"}=" H   TPH"; $resn_exch{"2NH  TPH"}=" H2  TPH";
$resn_exch{"1HB  TPH"}=" HB2 TPH"; $resn_exch{"2HB  TPH"}=" HB3 TPH"; $resn_exch{"1HO1 TPI"}="HO11 TPI";
$resn_exch{"1HO2 TPI"}="HO21 TPI"; $resn_exch{"1H2  TPI"}=" H21 TPI"; $resn_exch{"1H4  TPI"}=" H41 TPI";
$resn_exch{"1H5  TPI"}=" H51 TPI"; $resn_exch{"1H6  TPI"}=" H61 TPI"; $resn_exch{"1H8  TPI"}=" H81 TPI";
$resn_exch{"1H9  TPI"}=" H91 TPI"; $resn_exch{"1HN1 TPI"}="HN11 TPI"; $resn_exch{"1H13 TPI"}="H131 TPI";
$resn_exch{"1HN2 TPI"}="HN21 TPI"; $resn_exch{"2HN2 TPI"}="HN22 TPI"; $resn_exch{"1H15 TPI"}="H151 TPI";
$resn_exch{"2H15 TPI"}="H152 TPI"; $resn_exch{"1H16 TPI"}="H161 TPI"; $resn_exch{"2H16 TPI"}="H162 TPI";
$resn_exch{"1HO7 TPI"}="HO71 TPI"; $resn_exch{" HN2 TPL"}=" H2  TPL"; $resn_exch{"1HB  TPL"}=" HB1 TPL";
$resn_exch{"2HB  TPL"}=" HB2 TPL"; $resn_exch{"1HC  TPL"}=" HC1 TPL"; $resn_exch{"2HC  TPL"}=" HC2 TPL";
$resn_exch{"1HC2 TPM"}="HC21 TPM"; $resn_exch{"1HC3 TPM"}="HC31 TPM"; $resn_exch{"1HC5 TPM"}="HC51 TPM";
$resn_exch{"1HC6 TPM"}="HC61 TPM"; $resn_exch{"1HC7 TPM"}="HC71 TPM"; $resn_exch{"2HC7 TPM"}="HC72 TPM";
$resn_exch{"1HN1 TPM"}="HN11 TPM"; $resn_exch{"2HN1 TPM"}="HN12 TPM"; $resn_exch{"1HN2 TPM"}="HN21 TPM";
$resn_exch{"1HN3 TPM"}="HN31 TPM"; $resn_exch{"1HC9 TPM"}="HC91 TPM"; $resn_exch{"2HC9 TPM"}="HC92 TPM";
$resn_exch{"1H10 TPM"}="H101 TPM"; $resn_exch{"2H10 TPM"}="H102 TPM"; $resn_exch{"1H11 TPM"}="H111 TPM";
$resn_exch{"2H11 TPM"}="H112 TPM"; $resn_exch{"1H12 TPM"}="H121 TPM"; $resn_exch{"2H12 TPM"}="H122 TPM";
$resn_exch{"1H8' TPN"}="H8'1 TPN"; $resn_exch{"2H8' TPN"}="H8'2 TPN"; $resn_exch{"1H5' TPN"}="H5'1 TPN";
$resn_exch{"2H5' TPN"}="H5'2 TPN"; $resn_exch{"1H3' TPN"}="H3'1 TPN"; $resn_exch{"2H3' TPN"}="H3'2 TPN";
$resn_exch{"1H2' TPN"}="H2'1 TPN"; $resn_exch{"2H2' TPN"}="H2'2 TPN"; $resn_exch{"1H1' TPN"}="H1'1 TPN";
$resn_exch{"2H1' TPN"}="H1'2 TPN"; $resn_exch{"3H1' TPN"}="H1'3 TPN"; $resn_exch{"1HM5 TPN"}="HM51 TPN";
$resn_exch{"2HM5 TPN"}="HM52 TPN"; $resn_exch{"3HM5 TPN"}="HM53 TPN"; $resn_exch{"2HN  TPO"}=" H2  TPO";
$resn_exch{"1HG2 TPO"}="HG21 TPO"; $resn_exch{"2HG2 TPO"}="HG22 TPO"; $resn_exch{"3HG2 TPO"}="HG23 TPO";
$resn_exch{"2HOP TPO"}="HOP2 TPO"; $resn_exch{"3HOP TPO"}="HOP3 TPO"; $resn_exch{"2HN  TPQ"}=" H2  TPQ";
$resn_exch{"1HB  TPQ"}=" HB2 TPQ"; $resn_exch{"2HB  TPQ"}=" HB3 TPQ"; $resn_exch{"1HC7 TPR"}="HC71 TPR";
$resn_exch{"2HC7 TPR"}="HC72 TPR"; $resn_exch{"3HC7 TPR"}="HC73 TPR"; $resn_exch{"1HCD TPR"}="HCD1 TPR";
$resn_exch{"2HCD TPR"}="HCD2 TPR"; $resn_exch{"1HCG TPR"}="HCG1 TPR"; $resn_exch{"2HCG TPR"}="HCG2 TPR";
$resn_exch{"1HCB TPR"}="HCB1 TPR"; $resn_exch{"2HCB TPR"}="HCB2 TPR"; $resn_exch{"1H5  TPV"}=" H51 TPV";
$resn_exch{"2H5  TPV"}=" H52 TPV"; $resn_exch{"1HO8 TPV"}="HO81 TPV"; $resn_exch{"1H9  TPV"}=" H91 TPV";
$resn_exch{"2H9  TPV"}=" H92 TPV"; $resn_exch{"1H10 TPV"}="H101 TPV"; $resn_exch{"2H10 TPV"}="H102 TPV";
$resn_exch{"1H11 TPV"}="H111 TPV"; $resn_exch{"2H11 TPV"}="H112 TPV"; $resn_exch{"3H11 TPV"}="H113 TPV";
$resn_exch{"1H12 TPV"}="H121 TPV"; $resn_exch{"2H12 TPV"}="H122 TPV"; $resn_exch{"1H13 TPV"}="H131 TPV";
$resn_exch{"2H13 TPV"}="H132 TPV"; $resn_exch{"1H15 TPV"}="H151 TPV"; $resn_exch{"1H16 TPV"}="H161 TPV";
$resn_exch{"1H17 TPV"}="H171 TPV"; $resn_exch{"1H18 TPV"}="H181 TPV"; $resn_exch{"1H19 TPV"}="H191 TPV";
$resn_exch{"1H20 TPV"}="H201 TPV"; $resn_exch{"1H21 TPV"}="H211 TPV"; $resn_exch{"2H21 TPV"}="H212 TPV";
$resn_exch{"1H22 TPV"}="H221 TPV"; $resn_exch{"2H22 TPV"}="H222 TPV"; $resn_exch{"3H22 TPV"}="H223 TPV";
$resn_exch{"1H24 TPV"}="H241 TPV"; $resn_exch{"1H25 TPV"}="H251 TPV"; $resn_exch{"1H26 TPV"}="H261 TPV";
$resn_exch{"1H28 TPV"}="H281 TPV"; $resn_exch{"1H29 TPV"}="H291 TPV"; $resn_exch{"1H35 TPV"}="H351 TPV";
$resn_exch{"1H37 TPV"}="H371 TPV"; $resn_exch{"1H38 TPV"}="H381 TPV"; $resn_exch{"1H2A TPW"}="H2A1 TPW";
$resn_exch{"2H2A TPW"}="H2A2 TPW"; $resn_exch{"3H2A TPW"}="H2A3 TPW"; $resn_exch{"1H35 TPW"}="H351 TPW";
$resn_exch{"2H35 TPW"}="H352 TPW"; $resn_exch{"1H5A TPW"}="H5A1 TPW"; $resn_exch{"2H5A TPW"}="H5A2 TPW";
$resn_exch{"1H4A TPW"}="H4A1 TPW"; $resn_exch{"2H4A TPW"}="H4A2 TPW"; $resn_exch{"3H4A TPW"}="H4A3 TPW";
$resn_exch{"1H5B TPW"}="H5B1 TPW"; $resn_exch{"2H5B TPW"}="H5B2 TPW"; $resn_exch{" H6* TPW"}=" H6' TPW";
$resn_exch{"1H4* TPW"}="H4'1 TPW"; $resn_exch{"2H4* TPW"}="H4'2 TPW"; $resn_exch{"2HD  TPX"}=" HD2 TPX";
$resn_exch{"1HD  TPX"}=" HD1 TPX"; $resn_exch{"3HE  TPX"}=" HE3 TPX"; $resn_exch{"2HE  TPX"}=" HE2 TPX";
$resn_exch{"1HE  TPX"}=" HE1 TPX"; $resn_exch{"2HK  TPX"}=" HK2 TPX"; $resn_exch{"1HK  TPX"}=" HK1 TPX";
$resn_exch{"1H_N TPX"}=" HN1 TPX"; $resn_exch{"3HJ  TPX"}=" HJ3 TPX"; $resn_exch{"2HJ  TPX"}=" HJ2 TPX";
$resn_exch{"1HJ  TPX"}=" HJ1 TPX"; $resn_exch{"2HD  TPY"}=" HD2 TPY"; $resn_exch{"1HD  TPY"}=" HD1 TPY";
$resn_exch{"3HE  TPY"}=" HE3 TPY"; $resn_exch{"2HE  TPY"}=" HE2 TPY"; $resn_exch{"1HE  TPY"}=" HE1 TPY";
$resn_exch{"1H_N TPY"}=" HN1 TPY"; $resn_exch{"2HK  TPY"}=" HK2 TPY"; $resn_exch{"1HK  TPY"}=" HK1 TPY";
$resn_exch{"1HN7 TQ3"}="HN71 TQ3"; $resn_exch{"2HN7 TQ3"}="HN72 TQ3"; $resn_exch{"1H14 TQ3"}="H141 TQ3";
$resn_exch{"2H14 TQ3"}="H142 TQ3"; $resn_exch{"1HN7 TQ4"}="HN71 TQ4"; $resn_exch{"2HN7 TQ4"}="HN72 TQ4";
$resn_exch{"1H14 TQ4"}="H141 TQ4"; $resn_exch{"2H14 TQ4"}="H142 TQ4"; $resn_exch{"1H32 TQ4"}="H321 TQ4";
$resn_exch{"2H32 TQ4"}="H322 TQ4"; $resn_exch{"3H32 TQ4"}="H323 TQ4"; $resn_exch{"1HN7 TQ5"}="HN71 TQ5";
$resn_exch{"2HN7 TQ5"}="HN72 TQ5"; $resn_exch{"1H14 TQ5"}="H141 TQ5"; $resn_exch{"2H14 TQ5"}="H142 TQ5";
$resn_exch{"1H33 TQ5"}="H331 TQ5"; $resn_exch{"2H33 TQ5"}="H332 TQ5"; $resn_exch{"3H33 TQ5"}="H333 TQ5";
$resn_exch{"1H37 TQ5"}="H371 TQ5"; $resn_exch{"2H37 TQ5"}="H372 TQ5"; $resn_exch{"3H37 TQ5"}="H373 TQ5";
$resn_exch{"1H41 TQ5"}="H411 TQ5"; $resn_exch{"2H41 TQ5"}="H412 TQ5"; $resn_exch{"3H41 TQ5"}="H413 TQ5";
$resn_exch{"1H11 TQ6"}="H111 TQ6"; $resn_exch{"2H11 TQ6"}="H112 TQ6"; $resn_exch{"1H12 TQ6"}="H121 TQ6";
$resn_exch{"2H12 TQ6"}="H122 TQ6"; $resn_exch{"1H32 TQ6"}="H321 TQ6"; $resn_exch{"2H32 TQ6"}="H322 TQ6";
$resn_exch{"1H33 TQ6"}="H331 TQ6"; $resn_exch{"2H33 TQ6"}="H332 TQ6"; $resn_exch{"1H35 TQ6"}="H351 TQ6";
$resn_exch{"2H35 TQ6"}="H352 TQ6"; $resn_exch{"1H36 TQ6"}="H361 TQ6"; $resn_exch{"2H36 TQ6"}="H362 TQ6";
$resn_exch{" H2* TQD"}=" H2' TQD"; $resn_exch{" H4* TQD"}=" H4D TQD"; $resn_exch{"1H5* TQD"}="H5'1 TQD";
$resn_exch{"2H5* TQD"}="H5'2 TQD"; $resn_exch{" H6* TQD"}=" H6' TQD"; $resn_exch{"1H7* TQD"}="H7'1 TQD";
$resn_exch{"2H7* TQD"}="H7'2 TQD"; $resn_exch{"1H8  TQD"}=" H81 TQD"; $resn_exch{"2H8  TQD"}=" H82 TQD";
$resn_exch{"1H8* TQD"}="H8'1 TQD"; $resn_exch{"2H8* TQD"}="H8'2 TQD"; $resn_exch{"1H2' TQD"}="H2'1 TQD";
$resn_exch{"2H2' TQD"}="H2'2 TQD"; $resn_exch{"1H3B TQD"}="H3B1 TQD"; $resn_exch{"2H3B TQD"}="H3B2 TQD";
$resn_exch{"3H3B TQD"}="H3B3 TQD"; $resn_exch{"1H4B TQD"}="H4B1 TQD"; $resn_exch{"2H4B TQD"}="H4B2 TQD";
$resn_exch{"3H4B TQD"}="H4B3 TQD"; $resn_exch{"1H5B TQD"}="H5B1 TQD"; $resn_exch{"2H5B TQD"}="H5B2 TQD";
$resn_exch{"3H5B TQD"}="H5B3 TQD"; $resn_exch{"1H6' TQD"}="H6'1 TQD"; $resn_exch{"2H6' TQD"}="H6'2 TQD";
$resn_exch{" H1* TQD"}=" H1D TQD"; $resn_exch{"1H2* TQD"}="H2A1 TQD"; $resn_exch{"2H2* TQD"}="H2A2 TQD";
$resn_exch{" H3* TQD"}=" H3D TQD"; $resn_exch{"1H4* TQD"}="H4'1 TQD"; $resn_exch{"2H4* TQD"}="H4'2 TQD";
$resn_exch{"1H9* TQD"}="H9'1 TQD"; $resn_exch{"2H9* TQD"}="H9'2 TQD"; $resn_exch{"3H9* TQD"}="H9'3 TQD";
$resn_exch{"1HN  TQQ"}=" H   TQQ"; $resn_exch{"2HN  TQQ"}=" H2  TQQ"; $resn_exch{"1HB  TQQ"}=" HB2 TQQ";
$resn_exch{"2HB  TQQ"}=" HB3 TQQ"; $resn_exch{" H4* TQT"}=" H4B TQT"; $resn_exch{"1H7' TQT"}="H7'1 TQT";
$resn_exch{"2H7' TQT"}="H7'2 TQT"; $resn_exch{" H8* TQT"}=" H8' TQT"; $resn_exch{"1H9' TQT"}="H9'1 TQT";
$resn_exch{"2H9' TQT"}="H9'2 TQT"; $resn_exch{"1H*0 TQT"}="H'01 TQT"; $resn_exch{"2H*0 TQT"}="H'02 TQT";
$resn_exch{" H*1 TQT"}=" H'1 TQT"; $resn_exch{"1H*2 TQT"}="H'21 TQT"; $resn_exch{"2H*2 TQT"}="H'22 TQT";
$resn_exch{"1H*3 TQT"}="H'31 TQT"; $resn_exch{"2H*3 TQT"}="H'32 TQT"; $resn_exch{"1H*4 TQT"}="H'41 TQT";
$resn_exch{"2H*4 TQT"}="H'42 TQT"; $resn_exch{"1H*5 TQT"}="H'51 TQT"; $resn_exch{"2H*5 TQT"}="H'52 TQT";
$resn_exch{" H*6 TQT"}=" H'6 TQT"; $resn_exch{"1H*7 TQT"}="H'71 TQT"; $resn_exch{"2H*7 TQT"}="H'72 TQT";
$resn_exch{"1H2' TQT"}="H2'1 TQT"; $resn_exch{"2H2' TQT"}="H2'2 TQT"; $resn_exch{"1H4' TQT"}="H4'1 TQT";
$resn_exch{"2H4' TQT"}="H4'2 TQT"; $resn_exch{"1H5' TQT"}="H5'1 TQT"; $resn_exch{"2H5' TQT"}="H5'2 TQT";
$resn_exch{"1H8' TQT"}="H8'1 TQT"; $resn_exch{"2H8' TQT"}="H8'2 TQT"; $resn_exch{"1HG  TRA"}=" HG1 TRA";
$resn_exch{"2HG  TRA"}=" HG2 TRA"; $resn_exch{"1H4  TRB"}=" H41 TRB"; $resn_exch{"2H4  TRB"}=" H42 TRB";
$resn_exch{"1H7  TRB"}=" H71 TRB"; $resn_exch{"2H7  TRB"}=" H72 TRB"; $resn_exch{"1H11 TRB"}="H111 TRB";
$resn_exch{"2H11 TRB"}="H112 TRB"; $resn_exch{"3H11 TRB"}="H113 TRB"; $resn_exch{"1H12 TRB"}="H121 TRB";
$resn_exch{"2H12 TRB"}="H122 TRB"; $resn_exch{"1H15 TRB"}="H151 TRB"; $resn_exch{"2H15 TRB"}="H152 TRB";
$resn_exch{"3H15 TRB"}="H153 TRB"; $resn_exch{"1H16 TRB"}="H161 TRB"; $resn_exch{"2H16 TRB"}="H162 TRB";
$resn_exch{"3H16 TRB"}="H163 TRB"; $resn_exch{"1H2  TRC"}=" H21 TRC"; $resn_exch{"2H2  TRC"}=" H22 TRC";
$resn_exch{"1H4  TRC"}=" H41 TRC"; $resn_exch{"2H4  TRC"}=" H42 TRC"; $resn_exch{"1H1  TRD"}=" H11 TRD";
$resn_exch{"2H1  TRD"}=" H12 TRD"; $resn_exch{"3H1  TRD"}=" H13 TRD"; $resn_exch{"1H2  TRD"}=" H21 TRD";
$resn_exch{"2H2  TRD"}=" H22 TRD"; $resn_exch{"1H3  TRD"}=" H31 TRD"; $resn_exch{"2H3  TRD"}=" H32 TRD";
$resn_exch{"1H4  TRD"}=" H41 TRD"; $resn_exch{"2H4  TRD"}=" H42 TRD"; $resn_exch{"1H5  TRD"}=" H51 TRD";
$resn_exch{"2H5  TRD"}=" H52 TRD"; $resn_exch{"1H6  TRD"}=" H61 TRD"; $resn_exch{"2H6  TRD"}=" H62 TRD";
$resn_exch{"1H7  TRD"}=" H71 TRD"; $resn_exch{"2H7  TRD"}=" H72 TRD"; $resn_exch{"1H8  TRD"}=" H81 TRD";
$resn_exch{"2H8  TRD"}=" H82 TRD"; $resn_exch{"1H9  TRD"}=" H91 TRD"; $resn_exch{"2H9  TRD"}=" H92 TRD";
$resn_exch{"1H10 TRD"}="H101 TRD"; $resn_exch{"2H10 TRD"}="H102 TRD"; $resn_exch{"1H11 TRD"}="H111 TRD";
$resn_exch{"2H11 TRD"}="H112 TRD"; $resn_exch{"1H12 TRD"}="H121 TRD"; $resn_exch{"2H12 TRD"}="H122 TRD";
$resn_exch{"1H13 TRD"}="H131 TRD"; $resn_exch{"2H13 TRD"}="H132 TRD"; $resn_exch{"3H13 TRD"}="H133 TRD";
$resn_exch{"1HC6 TRE"}="HC61 TRE"; $resn_exch{"2HC6 TRE"}="HC62 TRE"; $resn_exch{"1H6P TRE"}="H6P1 TRE";
$resn_exch{"2H6P TRE"}="H6P2 TRE"; $resn_exch{"1HN  TRF"}=" H   TRF"; $resn_exch{"2HN  TRF"}=" H2  TRF";
$resn_exch{"1HB  TRF"}=" HB2 TRF"; $resn_exch{"2HB  TRF"}=" HB3 TRF"; $resn_exch{"1HB  TRG"}=" HB1 TRG";
$resn_exch{"2HB  TRG"}=" HB2 TRG"; $resn_exch{"1HG  TRG"}=" HG1 TRG"; $resn_exch{"2HG  TRG"}=" HG2 TRG";
$resn_exch{"1HD  TRG"}=" HD1 TRG"; $resn_exch{"2HD  TRG"}=" HD2 TRG"; $resn_exch{"1HE  TRG"}=" HE1 TRG";
$resn_exch{"2HE  TRG"}=" HE2 TRG"; $resn_exch{"1HH1 TRG"}="HH11 TRG"; $resn_exch{"2HH1 TRG"}="HH12 TRG";
$resn_exch{"3HH1 TRG"}="HH13 TRG"; $resn_exch{"1HH2 TRG"}="HH21 TRG"; $resn_exch{"2HH2 TRG"}="HH22 TRG";
$resn_exch{"3HH2 TRG"}="HH23 TRG"; $resn_exch{"OH4P TRH"}="H4PO TRH"; $resn_exch{"1HC6 TRH"}="HC61 TRH";
$resn_exch{"2HC6 TRH"}="HC62 TRH"; $resn_exch{"3HC6 TRH"}="HC63 TRH"; $resn_exch{"OH1P TRH"}="H1PO TRH";
$resn_exch{"1H5' TRH"}="H5'1 TRH"; $resn_exch{"2H5' TRH"}="H5'2 TRH"; $resn_exch{"CH4' TRH"}="H4'C TRH";
$resn_exch{"CH3' TRH"}="H3'C TRH"; $resn_exch{"OH3' TRH"}="H3'O TRH"; $resn_exch{"1H2' TRH"}="H2'1 TRH";
$resn_exch{"2H2' TRH"}="H2'2 TRH"; $resn_exch{"CH1' TRH"}="H1'C TRH"; $resn_exch{"NH31 TRH"}="H31N TRH";
$resn_exch{"1HCA TRH"}="HCA1 TRH"; $resn_exch{"2HCA TRH"}="HCA2 TRH"; $resn_exch{"3HCA TRH"}="HCA3 TRH";
$resn_exch{"CH61 TRH"}="H61C TRH"; $resn_exch{"1HN  TRJ"}=" HN1 TRJ"; $resn_exch{"2HN  TRJ"}=" HN2 TRJ";
$resn_exch{"1HA  TRJ"}=" HA1 TRJ"; $resn_exch{"2HA  TRJ"}=" HA2 TRJ"; $resn_exch{"1HD1 TRJ"}="HD11 TRJ";
$resn_exch{"1HD2 TRJ"}="HD21 TRJ"; $resn_exch{"1HE2 TRJ"}="HE21 TRJ"; $resn_exch{"1HZ  TRJ"}=" HZ1 TRJ";
$resn_exch{"1HB  TRJ"}=" HB1 TRJ"; $resn_exch{"2HB  TRJ"}=" HB2 TRJ"; $resn_exch{"1HN2 TRJ"}="HN21 TRJ";
$resn_exch{"2HN2 TRJ"}="HN22 TRJ"; $resn_exch{"1HC3 TRL"}="HC31 TRL"; $resn_exch{"2HC3 TRL"}="HC32 TRL";
$resn_exch{"1HC4 TRL"}="HC41 TRL"; $resn_exch{"2HC4 TRL"}="HC42 TRL"; $resn_exch{"3HC4 TRL"}="HC43 TRL";
$resn_exch{"1HC9 TRL"}="HC91 TRL"; $resn_exch{"2HC9 TRL"}="HC92 TRL"; $resn_exch{"1H12 TRL"}="H121 TRL";
$resn_exch{"2H12 TRL"}="H122 TRL"; $resn_exch{"1HC1 TRL"}="HC11 TRL"; $resn_exch{"2HC1 TRL"}="HC12 TRL";
$resn_exch{"3HC1 TRL"}="HC13 TRL"; $resn_exch{"1H13 TRL"}="H131 TRL"; $resn_exch{"2H13 TRL"}="H132 TRL";
$resn_exch{"3H13 TRL"}="H133 TRL"; $resn_exch{"1HC7 TRL"}="HC71 TRL"; $resn_exch{"2HC7 TRL"}="HC72 TRL";
$resn_exch{"3HC7 TRL"}="HC73 TRL"; $resn_exch{"1H21 TRL"}="H211 TRL"; $resn_exch{"2H21 TRL"}="H212 TRL";
$resn_exch{"3H21 TRL"}="H213 TRL"; $resn_exch{"1H26 TRL"}="H261 TRL"; $resn_exch{"2H26 TRL"}="H262 TRL";
$resn_exch{"3H26 TRL"}="H263 TRL"; $resn_exch{"1H25 TRL"}="H251 TRL"; $resn_exch{"2H25 TRL"}="H252 TRL";
$resn_exch{"3H25 TRL"}="H253 TRL"; $resn_exch{"1HE1 TRM"}="HE11 TRM"; $resn_exch{"1HZ2 TRM"}="HZ21 TRM";
$resn_exch{"1HH2 TRM"}="HH21 TRM"; $resn_exch{"1HZ3 TRM"}="HZ31 TRM"; $resn_exch{"1HE3 TRM"}="HE31 TRM";
$resn_exch{"1HN  TRN"}=" H   TRN"; $resn_exch{"2HN  TRN"}=" H2  TRN"; $resn_exch{"1HB  TRN"}=" HB2 TRN";
$resn_exch{"2HB  TRN"}=" HB3 TRN"; $resn_exch{" HNE TRN"}=" HE1 TRN"; $resn_exch{" HN2 TRO"}=" H2  TRO";
$resn_exch{"1HB  TRO"}=" HB2 TRO"; $resn_exch{"2HB  TRO"}=" HB3 TRO"; $resn_exch{" HOD TRO"}=" HD1 TRO";
$resn_exch{" HN2 TRP"}=" H2  TRP"; $resn_exch{"1HB  TRP"}=" HB2 TRP"; $resn_exch{"2HB  TRP"}=" HB3 TRP";
$resn_exch{"1HN  TRQ"}=" H   TRQ"; $resn_exch{"2HN  TRQ"}=" H2  TRQ"; $resn_exch{"1HB  TRQ"}=" HB2 TRQ";
$resn_exch{"2HB  TRQ"}=" HB3 TRQ"; $resn_exch{" HNE TRQ"}=" HE1 TRQ"; $resn_exch{"1H2  TRR"}=" H21 TRR";
$resn_exch{"2H2  TRR"}=" H22 TRR"; $resn_exch{"1H4  TRR"}=" H41 TRR"; $resn_exch{"2H4  TRR"}=" H42 TRR";
$resn_exch{"1H7  TRR"}=" H71 TRR"; $resn_exch{"2H7  TRR"}=" H72 TRR"; $resn_exch{"1H17 TRR"}="H171 TRR";
$resn_exch{"2H17 TRR"}="H172 TRR"; $resn_exch{"3H17 TRR"}="H173 TRR"; $resn_exch{"1H18 TRR"}="H181 TRR";
$resn_exch{"2H18 TRR"}="H182 TRR"; $resn_exch{"3H18 TRR"}="H183 TRR"; $resn_exch{"1H19 TRR"}="H191 TRR";
$resn_exch{"2H19 TRR"}="H192 TRR"; $resn_exch{"3H19 TRR"}="H193 TRR"; $resn_exch{"1H1  TRS"}=" H11 TRS";
$resn_exch{"2H1  TRS"}=" H12 TRS"; $resn_exch{"1H2  TRS"}=" H21 TRS"; $resn_exch{"2H2  TRS"}=" H22 TRS";
$resn_exch{"1H3  TRS"}=" H31 TRS"; $resn_exch{"2H3  TRS"}=" H32 TRS"; $resn_exch{"1HN  TRS"}=" HN1 TRS";
$resn_exch{"2HN  TRS"}=" HN2 TRS"; $resn_exch{"3HN  TRS"}=" HN3 TRS"; $resn_exch{"1H25 TRT"}="H251 TRT";
$resn_exch{"2H25 TRT"}="H252 TRT"; $resn_exch{"3H25 TRT"}="H253 TRT"; $resn_exch{"1H23 TRT"}="H231 TRT";
$resn_exch{"2H23 TRT"}="H232 TRT"; $resn_exch{"1H22 TRT"}="H221 TRT"; $resn_exch{"2H22 TRT"}="H222 TRT";
$resn_exch{"1H20 TRT"}="H201 TRT"; $resn_exch{"2H20 TRT"}="H202 TRT"; $resn_exch{"1H19 TRT"}="H191 TRT";
$resn_exch{"2H19 TRT"}="H192 TRT"; $resn_exch{"1H17 TRT"}="H171 TRT"; $resn_exch{"2H17 TRT"}="H172 TRT";
$resn_exch{"1H16 TRT"}="H161 TRT"; $resn_exch{"2H16 TRT"}="H162 TRT"; $resn_exch{"1H8C TRT"}="H8C1 TRT";
$resn_exch{"2H8C TRT"}="H8C2 TRT"; $resn_exch{"3H8C TRT"}="H8C3 TRT"; $resn_exch{"1H7C TRT"}="H7C1 TRT";
$resn_exch{"2H7C TRT"}="H7C2 TRT"; $resn_exch{"3H7C TRT"}="H7C3 TRT"; $resn_exch{"1H5C TRT"}="H5C1 TRT";
$resn_exch{"2H5C TRT"}="H5C2 TRT"; $resn_exch{"1H2C TRT"}="H2C1 TRT"; $resn_exch{"2H2C TRT"}="H2C2 TRT";
$resn_exch{"3H2C TRT"}="H2C3 TRT"; $resn_exch{"1H4C TRT"}="H4C1 TRT"; $resn_exch{"2H4C TRT"}="H4C2 TRT";
$resn_exch{"3H4C TRT"}="H4C3 TRT"; $resn_exch{"1H3C TRT"}="H3C1 TRT"; $resn_exch{"2H3C TRT"}="H3C2 TRT";
$resn_exch{"3H3C TRT"}="H3C3 TRT"; $resn_exch{"1H15 TRU"}="H151 TRU"; $resn_exch{"2H15 TRU"}="H152 TRU";
$resn_exch{" HN2 TRW"}=" H2  TRW"; $resn_exch{"1HB  TRW"}=" HB2 TRW"; $resn_exch{"2HB  TRW"}=" HB3 TRW";
$resn_exch{" HN2 TRX"}=" H2  TRX"; $resn_exch{"1HB  TRX"}=" HB2 TRX"; $resn_exch{"2HB  TRX"}=" HB3 TRX";
$resn_exch{"1H12 TRZ"}="H121 TRZ"; $resn_exch{"2H12 TRZ"}="H122 TRZ"; $resn_exch{"3H12 TRZ"}="H123 TRZ";
$resn_exch{"1H14 TRZ"}="H141 TRZ"; $resn_exch{"2H14 TRZ"}="H142 TRZ"; $resn_exch{"3H14 TRZ"}="H143 TRZ";
$resn_exch{"1H15 TRZ"}="H151 TRZ"; $resn_exch{"2H15 TRZ"}="H152 TRZ"; $resn_exch{"3H15 TRZ"}="H153 TRZ";
$resn_exch{"1H16 TRZ"}="H161 TRZ"; $resn_exch{"2H16 TRZ"}="H162 TRZ"; $resn_exch{"3H16 TRZ"}="H163 TRZ";
$resn_exch{"1H17 TRZ"}="H171 TRZ"; $resn_exch{"2H17 TRZ"}="H172 TRZ"; $resn_exch{"3H17 TRZ"}="H173 TRZ";
$resn_exch{"1H7  TRZ"}=" H71 TRZ"; $resn_exch{"2H7  TRZ"}=" H72 TRZ"; $resn_exch{"3H7  TRZ"}=" H73 TRZ";
$resn_exch{"1H8  TRZ"}=" H81 TRZ"; $resn_exch{"2H8  TRZ"}=" H82 TRZ"; $resn_exch{"1H18 TRZ"}="H181 TRZ";
$resn_exch{"2H18 TRZ"}="H182 TRZ"; $resn_exch{"1HOP  TS"}="HOP1  TS"; $resn_exch{"3HOP  TS"}="HOP3  TS";
$resn_exch{"1H5*  TS"}=" H5'  TS"; $resn_exch{"2H5*  TS"}="H5''  TS"; $resn_exch{" H4*  TS"}=" H4'  TS";
$resn_exch{" H3*  TS"}=" H3'  TS"; $resn_exch{"*HO3  TS"}="HO3'  TS"; $resn_exch{"1H2*  TS"}=" H2'  TS";
$resn_exch{"2H2*  TS"}="H2''  TS"; $resn_exch{" H1*  TS"}=" H1'  TS"; $resn_exch{"1HM5  TS"}="HM51  TS";
$resn_exch{"2HM5  TS"}="HM52  TS"; $resn_exch{"3HM5  TS"}="HM53  TS"; $resn_exch{"1H10 TS0"}="H101 TS0";
$resn_exch{"2H10 TS0"}="H102 TS0"; $resn_exch{"1H14 TS0"}="H141 TS0"; $resn_exch{"2H14 TS0"}="H142 TS0";
$resn_exch{"1H15 TS0"}="H151 TS0"; $resn_exch{"2H15 TS0"}="H152 TS0"; $resn_exch{"1H16 TS0"}="H161 TS0";
$resn_exch{"2H16 TS0"}="H162 TS0"; $resn_exch{"1H17 TS0"}="H171 TS0"; $resn_exch{"2H17 TS0"}="H172 TS0";
$resn_exch{"1H18 TS0"}="H181 TS0"; $resn_exch{"2H18 TS0"}="H182 TS0"; $resn_exch{"1H10 TS1"}="H101 TS1";
$resn_exch{"2H10 TS1"}="H102 TS1"; $resn_exch{"1H14 TS1"}="H141 TS1"; $resn_exch{"2H14 TS1"}="H142 TS1";
$resn_exch{"1H15 TS1"}="H151 TS1"; $resn_exch{"2H15 TS1"}="H152 TS1"; $resn_exch{"1H16 TS1"}="H161 TS1";
$resn_exch{"2H16 TS1"}="H162 TS1"; $resn_exch{"1H17 TS1"}="H171 TS1"; $resn_exch{"2H17 TS1"}="H172 TS1";
$resn_exch{"1H18 TS1"}="H181 TS1"; $resn_exch{"2H18 TS1"}="H182 TS1"; $resn_exch{"1HN1 TS2"}="HN11 TS2";
$resn_exch{"2HN1 TS2"}="HN12 TS2"; $resn_exch{"1HB1 TS2"}="HB11 TS2"; $resn_exch{"2HB1 TS2"}="HB12 TS2";
$resn_exch{"1HG1 TS2"}="HG11 TS2"; $resn_exch{"2HG1 TS2"}="HG12 TS2"; $resn_exch{"1HB2 TS2"}="HB21 TS2";
$resn_exch{"2HB2 TS2"}="HB22 TS2"; $resn_exch{"1HA3 TS2"}="HA31 TS2"; $resn_exch{"2HA3 TS2"}="HA32 TS2";
$resn_exch{"1HA4 TS2"}="HA41 TS2"; $resn_exch{"2HA4 TS2"}="HA42 TS2"; $resn_exch{"1HB5 TS2"}="HB51 TS2";
$resn_exch{"2HB5 TS2"}="HB52 TS2"; $resn_exch{"1HN6 TS2"}="HN61 TS2"; $resn_exch{"2HN6 TS2"}="HN62 TS2";
$resn_exch{"1HB6 TS2"}="HB61 TS2"; $resn_exch{"2HB6 TS2"}="HB62 TS2"; $resn_exch{"1HG6 TS2"}="HG61 TS2";
$resn_exch{"2HG6 TS2"}="HG62 TS2"; $resn_exch{"'HN1 TS2"}="HN1' TS2"; $resn_exch{"1H1' TS2"}="H1'1 TS2";
$resn_exch{"2H1' TS2"}="H1'2 TS2"; $resn_exch{"1H2' TS2"}="H2'1 TS2"; $resn_exch{"2H2' TS2"}="H2'2 TS2";
$resn_exch{"'HN2 TS2"}="HN2' TS2"; $resn_exch{"1H3' TS2"}="H3'1 TS2"; $resn_exch{"2H3' TS2"}="H3'2 TS2";
$resn_exch{"1H4' TS2"}="H4'1 TS2"; $resn_exch{"2H4' TS2"}="H4'2 TS2"; $resn_exch{"1H5' TS2"}="H5'1 TS2";
$resn_exch{"2H5' TS2"}="H5'2 TS2"; $resn_exch{"1H6' TS2"}="H6'1 TS2"; $resn_exch{"2H6' TS2"}="H6'2 TS2";
$resn_exch{"1H7' TS2"}="H7'1 TS2"; $resn_exch{"2H7' TS2"}="H7'2 TS2"; $resn_exch{"'HN3 TS2"}="HN3' TS2";
$resn_exch{"1HN1 TS3"}="HN11 TS3"; $resn_exch{"2HN1 TS3"}="HN12 TS3"; $resn_exch{"1HB1 TS3"}="HB11 TS3";
$resn_exch{"2HB1 TS3"}="HB12 TS3"; $resn_exch{"1HG1 TS3"}="HG11 TS3"; $resn_exch{"2HG1 TS3"}="HG12 TS3";
$resn_exch{"1HB2 TS3"}="HB21 TS3"; $resn_exch{"2HB2 TS3"}="HB22 TS3"; $resn_exch{"1HA3 TS3"}="HA31 TS3";
$resn_exch{"2HA3 TS3"}="HA32 TS3"; $resn_exch{"1HA4 TS3"}="HA41 TS3"; $resn_exch{"2HA4 TS3"}="HA42 TS3";
$resn_exch{"1HB  TS3"}=" HB1 TS3"; $resn_exch{"2HB  TS3"}=" HB2 TS3"; $resn_exch{"1HN6 TS3"}="HN61 TS3";
$resn_exch{"2HN6 TS3"}="HN62 TS3"; $resn_exch{"1HB6 TS3"}="HB61 TS3"; $resn_exch{"2HB6 TS3"}="HB62 TS3";
$resn_exch{"1HG6 TS3"}="HG61 TS3"; $resn_exch{"2HG6 TS3"}="HG62 TS3"; $resn_exch{"'HN1 TS3"}="HN1' TS3";
$resn_exch{"1H1' TS3"}="H1'1 TS3"; $resn_exch{"2H1' TS3"}="H1'2 TS3"; $resn_exch{"1H2' TS3"}="H2'1 TS3";
$resn_exch{"2H2' TS3"}="H2'2 TS3"; $resn_exch{"'HN2 TS3"}="HN2' TS3"; $resn_exch{"1H3' TS3"}="H3'1 TS3";
$resn_exch{"2H3' TS3"}="H3'2 TS3"; $resn_exch{"1H4' TS3"}="H4'1 TS3"; $resn_exch{"2H4' TS3"}="H4'2 TS3";
$resn_exch{"1H5' TS3"}="H5'1 TS3"; $resn_exch{"2H5' TS3"}="H5'2 TS3"; $resn_exch{"1H6' TS3"}="H6'1 TS3";
$resn_exch{"2H6' TS3"}="H6'2 TS3"; $resn_exch{"1H7' TS3"}="H7'1 TS3"; $resn_exch{"2H7' TS3"}="H7'2 TS3";
$resn_exch{"'HN3 TS3"}="HN3' TS3"; $resn_exch{"1HN1 TS4"}="HN11 TS4"; $resn_exch{"2HN1 TS4"}="HN12 TS4";
$resn_exch{"1HB1 TS4"}="HB11 TS4"; $resn_exch{"2HB1 TS4"}="HB12 TS4"; $resn_exch{"1HG1 TS4"}="HG11 TS4";
$resn_exch{"2HG1 TS4"}="HG12 TS4"; $resn_exch{"1HB2 TS4"}="HB21 TS4"; $resn_exch{"2HB2 TS4"}="HB22 TS4";
$resn_exch{"1HA3 TS4"}="HA31 TS4"; $resn_exch{"2HA3 TS4"}="HA32 TS4"; $resn_exch{"1HA4 TS4"}="HA41 TS4";
$resn_exch{"2HA4 TS4"}="HA42 TS4"; $resn_exch{"1HB5 TS4"}="HB51 TS4"; $resn_exch{"2HB5 TS4"}="HB52 TS4";
$resn_exch{"1HN6 TS4"}="HN61 TS4"; $resn_exch{"2HN6 TS4"}="HN62 TS4"; $resn_exch{"1HB6 TS4"}="HB61 TS4";
$resn_exch{"2HB6 TS4"}="HB62 TS4"; $resn_exch{"1HG6 TS4"}="HG61 TS4"; $resn_exch{"2HG6 TS4"}="HG62 TS4";
$resn_exch{"'HN1 TS4"}="HN1' TS4"; $resn_exch{"1H1' TS4"}="H1'1 TS4"; $resn_exch{"2H1' TS4"}="H1'2 TS4";
$resn_exch{"1H2' TS4"}="H2'1 TS4"; $resn_exch{"2H2' TS4"}="H2'2 TS4"; $resn_exch{"'HN2 TS4"}="HN2' TS4";
$resn_exch{"1H3' TS4"}="H3'1 TS4"; $resn_exch{"2H3' TS4"}="H3'2 TS4"; $resn_exch{"1H4' TS4"}="H4'1 TS4";
$resn_exch{"2H4' TS4"}="H4'2 TS4"; $resn_exch{"1H5' TS4"}="H5'1 TS4"; $resn_exch{"2H5' TS4"}="H5'2 TS4";
$resn_exch{"1H6' TS4"}="H6'1 TS4"; $resn_exch{"2H6' TS4"}="H6'2 TS4"; $resn_exch{"1H7' TS4"}="H7'1 TS4";
$resn_exch{"2H7' TS4"}="H7'2 TS4"; $resn_exch{"1HN3 TS4"}="HN31 TS4"; $resn_exch{"2HN3 TS4"}="HN32 TS4";
$resn_exch{"1H1T TS4"}="H1T1 TS4"; $resn_exch{"2H1T TS4"}="H1T2 TS4"; $resn_exch{"1H2T TS4"}="H2T1 TS4";
$resn_exch{"2H2T TS4"}="H2T2 TS4"; $resn_exch{"1H3T TS4"}="H3T1 TS4"; $resn_exch{"2H3T TS4"}="H3T2 TS4";
$resn_exch{"1H4T TS4"}="H4T1 TS4"; $resn_exch{"2H4T TS4"}="H4T2 TS4"; $resn_exch{"1H5T TS4"}="H5T1 TS4";
$resn_exch{"2H5T TS4"}="H5T2 TS4"; $resn_exch{"1H6T TS4"}="H6T1 TS4"; $resn_exch{"2H6T TS4"}="H6T2 TS4";
$resn_exch{"1H7T TS4"}="H7T1 TS4"; $resn_exch{"2H7T TS4"}="H7T2 TS4"; $resn_exch{"1HNT TS4"}="HNT1 TS4";
$resn_exch{"2HNT TS4"}="HNT2 TS4"; $resn_exch{"1HN1 TS5"}="HN11 TS5"; $resn_exch{"2HN1 TS5"}="HN12 TS5";
$resn_exch{"1HB1 TS5"}="HB11 TS5"; $resn_exch{"2HB1 TS5"}="HB12 TS5"; $resn_exch{"1HG1 TS5"}="HG11 TS5";
$resn_exch{"2HG1 TS5"}="HG12 TS5"; $resn_exch{"1HB2 TS5"}="HB21 TS5"; $resn_exch{"2HB2 TS5"}="HB22 TS5";
$resn_exch{"1HA3 TS5"}="HA31 TS5"; $resn_exch{"2HA3 TS5"}="HA32 TS5"; $resn_exch{"'HN1 TS5"}="HN1' TS5";
$resn_exch{"1H1' TS5"}="H1'1 TS5"; $resn_exch{"2H1' TS5"}="H1'2 TS5"; $resn_exch{"1H2' TS5"}="H2'1 TS5";
$resn_exch{"2H2' TS5"}="H2'2 TS5"; $resn_exch{"'HN2 TS5"}="HN2' TS5"; $resn_exch{"1H3' TS5"}="H3'1 TS5";
$resn_exch{"2H3' TS5"}="H3'2 TS5"; $resn_exch{"1H4' TS5"}="H4'1 TS5"; $resn_exch{"2H4' TS5"}="H4'2 TS5";
$resn_exch{"1H5' TS5"}="H5'1 TS5"; $resn_exch{"2H5' TS5"}="H5'2 TS5"; $resn_exch{"1H6' TS5"}="H6'1 TS5";
$resn_exch{"2H6' TS5"}="H6'2 TS5"; $resn_exch{"1H7' TS5"}="H7'1 TS5"; $resn_exch{"2H7' TS5"}="H7'2 TS5";
$resn_exch{"1HN3 TS5"}="HN31 TS5"; $resn_exch{"2HN3 TS5"}="HN32 TS5"; $resn_exch{"1H6  TSA"}=" H61 TSA";
$resn_exch{"2H6  TSA"}=" H62 TSA"; $resn_exch{"1H9  TSA"}=" H91 TSA"; $resn_exch{"2H9  TSA"}=" H92 TSA";
$resn_exch{"1HG2 TSB"}="HG21 TSB"; $resn_exch{"2HG2 TSB"}="HG22 TSB"; $resn_exch{"3HG2 TSB"}="HG23 TSB";
$resn_exch{"1HN1 TSB"}="HN11 TSB"; $resn_exch{"2HN1 TSB"}="HN12 TSB"; $resn_exch{"1H5* TSB"}="H5'1 TSB";
$resn_exch{"2H5* TSB"}="H5'2 TSB"; $resn_exch{" H4* TSB"}=" H4' TSB"; $resn_exch{" H3* TSB"}=" H3' TSB";
$resn_exch{"*HO3 TSB"}="HO3' TSB"; $resn_exch{" H2* TSB"}=" H2' TSB"; $resn_exch{"*HO2 TSB"}="HO2' TSB";
$resn_exch{" H1* TSB"}=" H1' TSB"; $resn_exch{"1HN6 TSB"}="HN61 TSB"; $resn_exch{"2HN6 TSB"}="HN62 TSB";
$resn_exch{"1HN  TSC"}=" HN1 TSC"; $resn_exch{"2HN  TSC"}=" HN2 TSC"; $resn_exch{"1HB  TSC"}=" HB1 TSC";
$resn_exch{"2HB  TSC"}=" HB2 TSC"; $resn_exch{"1D3  TSD"}=" D31 TSD"; $resn_exch{"2D3  TSD"}=" D32 TSD";
$resn_exch{"1D2  TSD"}=" D21 TSD"; $resn_exch{"2D2  TSD"}=" D22 TSD"; $resn_exch{"1H4  TSD"}=" H41 TSD";
$resn_exch{"2H4  TSD"}=" H42 TSD"; $resn_exch{"3H4  TSD"}=" H43 TSD"; $resn_exch{"1H5  TSD"}=" H51 TSD";
$resn_exch{"2H5  TSD"}=" H52 TSD"; $resn_exch{"1H2P TSE"}="H2P1 TSE"; $resn_exch{"2H2P TSE"}="H2P2 TSE";
$resn_exch{"3HOP TSE"}="HOP3 TSE"; $resn_exch{"2HOP TSE"}="HOP2 TSE"; $resn_exch{"1H18 TSF"}="H181 TSF";
$resn_exch{"2H18 TSF"}="H182 TSF"; $resn_exch{"1H17 TSF"}="H171 TSF"; $resn_exch{"2H17 TSF"}="H172 TSF";
$resn_exch{"1H16 TSF"}="H161 TSF"; $resn_exch{"2H16 TSF"}="H162 TSF"; $resn_exch{"1H15 TSF"}="H151 TSF";
$resn_exch{"2H15 TSF"}="H152 TSF"; $resn_exch{"1H10 TSF"}="H101 TSF"; $resn_exch{"2H10 TSF"}="H102 TSF";
$resn_exch{"1H14 TSF"}="H141 TSF"; $resn_exch{"2H14 TSF"}="H142 TSF"; $resn_exch{"1HB  TSH"}=" HB1 TSH";
$resn_exch{"2HB  TSH"}=" HB2 TSH"; $resn_exch{"1HN1 TSI"}="HN11 TSI"; $resn_exch{"2HN1 TSI"}="HN12 TSI";
$resn_exch{"1HC  TSI"}="HC1A TSI"; $resn_exch{"2HC  TSI"}="HC2A TSI"; $resn_exch{"3HC  TSI"}=" HC3 TSI";
$resn_exch{"1H_A TSI"}=" H81 TSI"; $resn_exch{"2H_B TSI"}=" H82 TSI"; $resn_exch{"3H_C TSI"}=" H83 TSI";
$resn_exch{"1H14 TSL"}="H141 TSL"; $resn_exch{"2H14 TSL"}="H142 TSL"; $resn_exch{"3H14 TSL"}="H143 TSL";
$resn_exch{"1H20 TSL"}="H201 TSL"; $resn_exch{"2H20 TSL"}="H202 TSL"; $resn_exch{"3H20 TSL"}="H203 TSL";
$resn_exch{"1H1  TSM"}=" H11 TSM"; $resn_exch{"2H1  TSM"}=" H12 TSM"; $resn_exch{"3H1  TSM"}=" H13 TSM";
$resn_exch{"1H2  TSM"}=" H21 TSM"; $resn_exch{"2H2  TSM"}=" H22 TSM"; $resn_exch{"3H2  TSM"}=" H23 TSM";
$resn_exch{"1H3  TSM"}=" H31 TSM"; $resn_exch{"2H3  TSM"}=" H32 TSM"; $resn_exch{"3H3  TSM"}=" H33 TSM";
$resn_exch{"1H2  TSN"}=" H21 TSN"; $resn_exch{"1H3  TSN"}=" H31 TSN"; $resn_exch{"1H5  TSN"}=" H51 TSN";
$resn_exch{"1H6  TSN"}=" H61 TSN"; $resn_exch{"1H11 TSN"}="H111 TSN"; $resn_exch{"1H12 TSN"}="H121 TSN";
$resn_exch{"1H14 TSN"}="H141 TSN"; $resn_exch{"2H14 TSN"}="H142 TSN"; $resn_exch{"3H14 TSN"}="H143 TSN";
$resn_exch{"1H15 TSN"}="H151 TSN"; $resn_exch{"2H15 TSN"}="H152 TSN"; $resn_exch{"3H15 TSN"}="H153 TSN";
$resn_exch{"1H17 TSN"}="H171 TSN"; $resn_exch{"2H17 TSN"}="H172 TSN"; $resn_exch{"3H17 TSN"}="H173 TSN";
$resn_exch{"1H16 TSN"}="H161 TSN"; $resn_exch{"2H16 TSN"}="H162 TSN"; $resn_exch{"3H16 TSN"}="H163 TSN";
$resn_exch{"2HOP TSP"}="HOP2 TSP"; $resn_exch{"3HOP TSP"}="HOP3 TSP"; $resn_exch{"1H5* TSP"}=" H5' TSP";
$resn_exch{"2H5* TSP"}="H5'' TSP"; $resn_exch{" H4* TSP"}=" H4' TSP"; $resn_exch{" H3* TSP"}=" H3' TSP";
$resn_exch{"*HS3 TSP"}="HS3' TSP"; $resn_exch{"1H2* TSP"}=" H2' TSP"; $resn_exch{"2H2* TSP"}="H2'' TSP";
$resn_exch{" H1* TSP"}=" H1' TSP"; $resn_exch{"1H5M TSP"}=" H71 TSP"; $resn_exch{"2H5M TSP"}=" H72 TSP";
$resn_exch{"3H5M TSP"}=" H73 TSP"; $resn_exch{"1HN  TSR"}=" HN1 TSR"; $resn_exch{"2HN  TSR"}=" HN2 TSR";
$resn_exch{"1HB  TSR"}=" HB1 TSR"; $resn_exch{"2HB  TSR"}=" HB2 TSR"; $resn_exch{"1HN1 TSS"}="HN11 TSS";
$resn_exch{"2HN1 TSS"}="HN12 TSS"; $resn_exch{"1HA  TSS"}=" HA1 TSS"; $resn_exch{"2HA  TSS"}=" HA2 TSS";
$resn_exch{"1HB  TSS"}=" HB1 TSS"; $resn_exch{"2HB  TSS"}=" HB2 TSS"; $resn_exch{"1H1  TST"}=" H11 TST";
$resn_exch{"2H1  TST"}="H12A TST"; $resn_exch{"1H2  TST"}=" H21 TST"; $resn_exch{"2H2  TST"}=" H22 TST";
$resn_exch{"1H4  TST"}=" H41 TST"; $resn_exch{"2H4  TST"}=" H42 TST"; $resn_exch{"1H7  TST"}=" H71 TST";
$resn_exch{"2H7  TST"}=" H72 TST"; $resn_exch{"1H10 TST"}="H101 TST"; $resn_exch{"2H10 TST"}="H102 TST";
$resn_exch{"3H10 TST"}="H103 TST"; $resn_exch{"1H17 TST"}="H171 TST"; $resn_exch{"2H17 TST"}="H172 TST";
$resn_exch{"3H17 TST"}="H173 TST"; $resn_exch{"1H18 TST"}="H181 TST"; $resn_exch{"2H18 TST"}="H182 TST";
$resn_exch{"3H18 TST"}="H183 TST"; $resn_exch{"1H7  TSU"}=" H71 TSU"; $resn_exch{"2H7  TSU"}=" H72 TSU";
$resn_exch{"3H7  TSU"}=" H73 TSU"; $resn_exch{"1H31 TSX"}="H311 TSX"; $resn_exch{"2H31 TSX"}="H312 TSX";
$resn_exch{"1H33 TSX"}="H331 TSX"; $resn_exch{"2H33 TSX"}="H332 TSX"; $resn_exch{"1H41 TSX"}="H411 TSX";
$resn_exch{"2H41 TSX"}="H412 TSX"; $resn_exch{"1H42 TSX"}="H421 TSX"; $resn_exch{"2H42 TSX"}="H422 TSX";
$resn_exch{"1H43 TSX"}="H431 TSX"; $resn_exch{"2H43 TSX"}="H432 TSX"; $resn_exch{"1H44 TSX"}="H441 TSX";
$resn_exch{"2H44 TSX"}="H442 TSX"; $resn_exch{"3H44 TSX"}="H443 TSX"; $resn_exch{"1H9  TSX"}=" H91 TSX";
$resn_exch{"1H14 TSX"}="H141 TSX"; $resn_exch{"2H14 TSX"}="H142 TSX"; $resn_exch{"1HN  TSX"}=" HN1 TSX";
$resn_exch{"1H4  TSX"}=" H41 TSX"; $resn_exch{"2H4  TSX"}=" H42 TSX"; $resn_exch{"3H4  TSX"}=" H43 TSX";
$resn_exch{"1HN1 TSZ"}="HN11 TSZ"; $resn_exch{"2HN1 TSZ"}="HN12 TSZ"; $resn_exch{"1HN3 TSZ"}="HN31 TSZ";
$resn_exch{"2HN3 TSZ"}="HN32 TSZ"; $resn_exch{"1H5*  TT"}="H5'1  TT"; $resn_exch{"2H5*  TT"}="H5'2  TT";
$resn_exch{" H4*  TT"}=" H4'  TT"; $resn_exch{" H3*  TT"}=" H3'  TT"; $resn_exch{" H1*  TT"}=" H1'  TT";
$resn_exch{"1H2*  TT"}="H2'1  TT"; $resn_exch{"2H2*  TT"}="H2'2  TT"; $resn_exch{"1H7C  TT"}="H7C1  TT";
$resn_exch{"2H7C  TT"}="H7C2  TT"; $resn_exch{"1H5A  TT"}="H5A1  TT"; $resn_exch{"2H5A  TT"}="H5A2  TT";
$resn_exch{"3H5A  TT"}="H5A3  TT"; $resn_exch{"1H5R  TT"}="H5R1  TT"; $resn_exch{"2H5R  TT"}="H5R2  TT";
$resn_exch{"1H2R  TT"}="H2R1  TT"; $resn_exch{"2H2R  TT"}="H2R2  TT"; $resn_exch{"1H5M  TT"}="H5M1  TT";
$resn_exch{"2H5M  TT"}="H5M2  TT"; $resn_exch{"3H5M  TT"}="H5M3  TT"; $resn_exch{"1H   TTB"}=" H1  TTB";
$resn_exch{"2H   TTB"}=" H2  TTB"; $resn_exch{"3H   TTB"}=" H3A TTB"; $resn_exch{"1H18 TTB"}="H181 TTB";
$resn_exch{"2H18 TTB"}="H182 TTB"; $resn_exch{"3H18 TTB"}="H183 TTB"; $resn_exch{"1H19 TTB"}="H191 TTB";
$resn_exch{"2H19 TTB"}="H192 TTB"; $resn_exch{"3H19 TTB"}="H193 TTB"; $resn_exch{"1H20 TTB"}="H201 TTB";
$resn_exch{"2H20 TTB"}="H202 TTB"; $resn_exch{"1H21 TTB"}="H211 TTB"; $resn_exch{"2H21 TTB"}="H212 TTB";
$resn_exch{"1H23 TTB"}="H231 TTB"; $resn_exch{"2H23 TTB"}="H232 TTB"; $resn_exch{"3H23 TTB"}="H233 TTB";
$resn_exch{"1H24 TTB"}="H241 TTB"; $resn_exch{"2H24 TTB"}="H242 TTB"; $resn_exch{"3H24 TTB"}="H243 TTB";
$resn_exch{"1H25 TTC"}="H251 TTC"; $resn_exch{"2H25 TTC"}="H252 TTC"; $resn_exch{"1H31 TTC"}="H311 TTC";
$resn_exch{"2H31 TTC"}="H312 TTC"; $resn_exch{"3H31 TTC"}="H313 TTC"; $resn_exch{"1H24 TTC"}="H241 TTC";
$resn_exch{"1H17 TTC"}="H171 TTC"; $resn_exch{"1H19 TTC"}="H191 TTC"; $resn_exch{"2H19 TTC"}="H192 TTC";
$resn_exch{"1H11 TTC"}="H111 TTC"; $resn_exch{"2H11 TTC"}="H112 TTC"; $resn_exch{"1H7  TTC"}=" H71 TTC";
$resn_exch{"1H4  TTC"}=" H41 TTC"; $resn_exch{"1H3  TTC"}=" H31 TTC"; $resn_exch{"1H27 TTC"}="H271 TTC";
$resn_exch{"2H27 TTC"}="H272 TTC"; $resn_exch{"1H29 TTC"}="H291 TTC"; $resn_exch{"2H29 TTC"}="H292 TTC";
$resn_exch{"3H29 TTC"}="H293 TTC"; $resn_exch{"1H30 TTC"}="H301 TTC"; $resn_exch{"2H30 TTC"}="H302 TTC";
$resn_exch{"3H30 TTC"}="H303 TTC"; $resn_exch{"1H26 TTC"}="H261 TTC"; $resn_exch{"2HOP TTD"}="HOP2 TTD";
$resn_exch{"3HOP TTD"}="HOP3 TTD"; $resn_exch{"1H5* TTD"}=" H5' TTD"; $resn_exch{"2H5* TTD"}="H5'' TTD";
$resn_exch{"1H2* TTD"}=" H2' TTD"; $resn_exch{"2H2* TTD"}="H2'' TTD"; $resn_exch{" H1* TTD"}=" H1' TTD";
$resn_exch{"1H5A TTD"}="H5A1 TTD"; $resn_exch{"2H5A TTD"}="H5A2 TTD"; $resn_exch{"3H5A TTD"}="H5A3 TTD";
$resn_exch{"1H5R TTD"}="H5R1 TTD"; $resn_exch{"2H5R TTD"}="H5R2 TTD"; $resn_exch{"1H2R TTD"}="H2R1 TTD";
$resn_exch{"2H2R TTD"}="H2R2 TTD"; $resn_exch{"1H5M TTD"}=" H71 TTD"; $resn_exch{"2H5M TTD"}=" H72 TTD";
$resn_exch{"3H5M TTD"}=" H73 TTD"; $resn_exch{" H4* TTD"}=" H4' TTD"; $resn_exch{"H3T  TTD"}="HO3' TTD";
$resn_exch{"1H5M TTE"}="H5M1 TTE"; $resn_exch{"2H5M TTE"}="H5M2 TTE"; $resn_exch{"3H5M TTE"}="H5M3 TTE";
$resn_exch{" H1* TTE"}=" H1' TTE"; $resn_exch{"1H2* TTE"}="H2'1 TTE"; $resn_exch{"2H2* TTE"}="H2'2 TTE";
$resn_exch{" H3* TTE"}=" H3' TTE"; $resn_exch{"3HO* TTE"}="HO'3 TTE"; $resn_exch{" H4* TTE"}=" H4' TTE";
$resn_exch{"1H5* TTE"}="H5'1 TTE"; $resn_exch{"2H5* TTE"}="H5'2 TTE"; $resn_exch{"2HOA TTE"}="HOA2 TTE";
$resn_exch{"2HOB TTE"}="HOB2 TTE"; $resn_exch{"1HCC TTE"}="HCC1 TTE"; $resn_exch{"2HCC TTE"}="HCC2 TTE";
$resn_exch{"2HOG TTE"}="HOG2 TTE"; $resn_exch{"3HOG TTE"}="HOG3 TTE"; $resn_exch{"1H6  TTF"}=" H61 TTF";
$resn_exch{"2H6  TTF"}=" H62 TTF"; $resn_exch{"1H25 TTG"}="H251 TTG"; $resn_exch{"2H25 TTG"}="H252 TTG";
$resn_exch{"1H31 TTG"}="H311 TTG"; $resn_exch{"2H31 TTG"}="H312 TTG"; $resn_exch{"3H31 TTG"}="H313 TTG";
$resn_exch{"1H24 TTG"}="H241 TTG"; $resn_exch{"1H17 TTG"}="H171 TTG"; $resn_exch{"OH22 TTG"}="H22O TTG";
$resn_exch{"1H19 TTG"}="H191 TTG"; $resn_exch{"2H19 TTG"}="H192 TTG"; $resn_exch{"1H11 TTG"}="H111 TTG";
$resn_exch{"2H11 TTG"}="H112 TTG"; $resn_exch{"1H7  TTG"}=" H71 TTG"; $resn_exch{"1H4  TTG"}=" H41 TTG";
$resn_exch{"1H3  TTG"}=" H31 TTG"; $resn_exch{"1H27 TTG"}="H271 TTG"; $resn_exch{"2H27 TTG"}="H272 TTG";
$resn_exch{"1H29 TTG"}="H291 TTG"; $resn_exch{"2H29 TTG"}="H292 TTG"; $resn_exch{"3H29 TTG"}="H293 TTG";
$resn_exch{"1H30 TTG"}="H301 TTG"; $resn_exch{"2H30 TTG"}="H302 TTG"; $resn_exch{"3H30 TTG"}="H303 TTG";
$resn_exch{"1H26 TTG"}="H261 TTG"; $resn_exch{"OH19 TTG"}="H19O TTG"; $resn_exch{"1H20 TTH"}="H201 TTH";
$resn_exch{"2H20 TTH"}="H202 TTH"; $resn_exch{"3H20 TTH"}="H203 TTH"; $resn_exch{"1H19 TTH"}="H191 TTH";
$resn_exch{"2H19 TTH"}="H192 TTH"; $resn_exch{"3H19 TTH"}="H193 TTH"; $resn_exch{"1H16 TTH"}="H161 TTH";
$resn_exch{"2H16 TTH"}="H162 TTH"; $resn_exch{"1H14 TTH"}="H141 TTH"; $resn_exch{"2H14 TTH"}="H142 TTH";
$resn_exch{"3H14 TTH"}="H143 TTH"; $resn_exch{"1H15 TTH"}="H151 TTH"; $resn_exch{"2H15 TTH"}="H152 TTH";
$resn_exch{"1H11 TTH"}="H111 TTH"; $resn_exch{"2H11 TTH"}="H112 TTH"; $resn_exch{"1HC9 TTH"}="HC91 TTH";
$resn_exch{"2HC9 TTH"}="HC92 TTH"; $resn_exch{"1H10 TTH"}="H101 TTH"; $resn_exch{"2H10 TTH"}="H102 TTH";
$resn_exch{"3H10 TTH"}="H103 TTH"; $resn_exch{"1HC6 TTH"}="HC61 TTH"; $resn_exch{"2HC6 TTH"}="HC62 TTH";
$resn_exch{"1HC5 TTH"}="HC51 TTH"; $resn_exch{"2HC5 TTH"}="HC52 TTH"; $resn_exch{"1HC4 TTH"}="HC41 TTH";
$resn_exch{"2HC4 TTH"}="HC42 TTH"; $resn_exch{"3HC4 TTH"}="HC43 TTH"; $resn_exch{"1HC1 TTH"}="HC11 TTH";
$resn_exch{"2HC1 TTH"}="HC12 TTH"; $resn_exch{"3HC1 TTH"}="HC13 TTH"; $resn_exch{"1HAG TTL"}="HAG1 TTL";
$resn_exch{"2HAG TTL"}="HAG2 TTL"; $resn_exch{"1HAH TTL"}="HAH1 TTL"; $resn_exch{"2HAH TTL"}="HAH2 TTL";
$resn_exch{"1HAA TTL"}="HAA1 TTL"; $resn_exch{"2HAA TTL"}="HAA2 TTL"; $resn_exch{"2HOP TTM"}="HOP2 TTM";
$resn_exch{"1H5* TTM"}=" H5' TTM"; $resn_exch{"2H5* TTM"}="H5'' TTM"; $resn_exch{" H4* TTM"}=" H4' TTM";
$resn_exch{" H1* TTM"}=" H1' TTM"; $resn_exch{"1H5M TTM"}=" H71 TTM"; $resn_exch{"2H5M TTM"}=" H72 TTM";
$resn_exch{"3H5M TTM"}=" H73 TTM"; $resn_exch{"1H2* TTM"}=" H2' TTM"; $resn_exch{"2H2* TTM"}="H2'' TTM";
$resn_exch{" H3* TTM"}=" H3' TTM"; $resn_exch{"H3T  TTM"}="HO3' TTM"; $resn_exch{"3HOP TTM"}="HOP3 TTM";
$resn_exch{"2HOA TTP"}="HOA2 TTP"; $resn_exch{"2HOB TTP"}="HOB2 TTP"; $resn_exch{"2HOG TTP"}="HOG2 TTP";
$resn_exch{"3HOG TTP"}="HOG3 TTP"; $resn_exch{"1H5* TTP"}="H5'1 TTP"; $resn_exch{"2H5* TTP"}="H5'2 TTP";
$resn_exch{" H4* TTP"}=" H4' TTP"; $resn_exch{" H3* TTP"}=" H3' TTP"; $resn_exch{"*HO3 TTP"}="HO3' TTP";
$resn_exch{"1H2* TTP"}="H2'1 TTP"; $resn_exch{"2H2* TTP"}="H2'2 TTP"; $resn_exch{" H1* TTP"}=" H1' TTP";
$resn_exch{"1HM5 TTP"}="HM51 TTP"; $resn_exch{"2HM5 TTP"}="HM52 TTP"; $resn_exch{"3HM5 TTP"}="HM53 TTP";
$resn_exch{"1HN  TTQ"}=" H   TTQ"; $resn_exch{"2HN  TTQ"}=" H2  TTQ"; $resn_exch{"1HB  TTQ"}=" HB2 TTQ";
$resn_exch{"2HB  TTQ"}=" HB3 TTQ"; $resn_exch{"HNE1 TTQ"}=" HE1 TTQ"; $resn_exch{"1HN2 TTQ"}="HN21 TTQ";
$resn_exch{"2HN2 TTQ"}="HN22 TTQ"; $resn_exch{"HOXT TTQ"}=" HXT TTQ"; $resn_exch{" HN2 TTS"}=" H2  TTS";
$resn_exch{"1HB  TTS"}=" HB2 TTS"; $resn_exch{"2HB  TTS"}=" HB3 TTS"; $resn_exch{" H3* TTS"}=" H3' TTS";
$resn_exch{" H2* TTS"}=" H2' TTS"; $resn_exch{" H6* TTS"}=" H6' TTS"; $resn_exch{" H5* TTS"}=" H5' TTS";
$resn_exch{"1H2  TTS"}=" H21 TTS"; $resn_exch{"2H2  TTS"}=" H22 TTS"; $resn_exch{"1H1  TTS"}=" H11 TTS";
$resn_exch{"2H1  TTS"}=" H12 TTS"; $resn_exch{"1H7  TTX"}=" H71 TTX"; $resn_exch{"2H7  TTX"}=" H72 TTX";
$resn_exch{"3H7  TTX"}=" H73 TTX"; $resn_exch{"1H16 TTX"}="H161 TTX"; $resn_exch{"1H22 TTX"}="H221 TTX";
$resn_exch{"1H21 TTX"}="H211 TTX"; $resn_exch{"1H20 TTX"}="H201 TTX"; $resn_exch{"1H19 TTX"}="H191 TTX";
$resn_exch{"1H18 TTX"}="H181 TTX"; $resn_exch{"1H5  TTX"}=" H51 TTX"; $resn_exch{"1H4  TTX"}=" H41 TTX";
$resn_exch{"2H4  TTX"}=" H42 TTX"; $resn_exch{"1H2  TTX"}=" H21 TTX"; $resn_exch{"1H1  TTX"}=" H11 TTX";
$resn_exch{"2H1  TTX"}=" H12 TTX"; $resn_exch{"3H1  TTX"}=" H13 TTX"; $resn_exch{"1H3  TTX"}=" H31 TTX";
$resn_exch{"2H3  TTX"}=" H32 TTX"; $resn_exch{"3H3  TTX"}=" H33 TTX"; $resn_exch{"1HN2 TTX"}="HN21 TTX";
$resn_exch{"1H9  TTX"}=" H91 TTX"; $resn_exch{"1H10 TTX"}="H101 TTX"; $resn_exch{"2H10 TTX"}="H102 TTX";
$resn_exch{"3H10 TTX"}="H103 TTX"; $resn_exch{"1H11 TTX"}="H111 TTX"; $resn_exch{"2H11 TTX"}="H112 TTX";
$resn_exch{"3H11 TTX"}="H113 TTX"; $resn_exch{"1H13 TTX"}="H131 TTX"; $resn_exch{"2H13 TTX"}="H132 TTX";
$resn_exch{"1HN4 TTX"}="HN41 TTX"; $resn_exch{"1H7C TTZ"}="H7C1 TTZ"; $resn_exch{"2H7C TTZ"}="H7C2 TTZ";
$resn_exch{"1H6' TTZ"}="H6'1 TTZ"; $resn_exch{"2H6' TTZ"}="H6'2 TTZ"; $resn_exch{"1H1  TUD"}=" H11 TUD";
$resn_exch{"2H1  TUD"}=" H12 TUD"; $resn_exch{"1H2  TUD"}=" H21 TUD"; $resn_exch{"2H2  TUD"}=" H22 TUD";
$resn_exch{"1H4  TUD"}=" H41 TUD"; $resn_exch{"2H4  TUD"}=" H42 TUD"; $resn_exch{"1H6  TUD"}=" H61 TUD";
$resn_exch{"2H6  TUD"}=" H62 TUD"; $resn_exch{"1H11 TUD"}="H111 TUD"; $resn_exch{"2H11 TUD"}="H112 TUD";
$resn_exch{"1H12 TUD"}="H121 TUD"; $resn_exch{"2H12 TUD"}="H122 TUD"; $resn_exch{"1H15 TUD"}="H151 TUD";
$resn_exch{"2H15 TUD"}="H152 TUD"; $resn_exch{"1H16 TUD"}="H161 TUD"; $resn_exch{"2H16 TUD"}="H162 TUD";
$resn_exch{"1H18 TUD"}="H181 TUD"; $resn_exch{"2H18 TUD"}="H182 TUD"; $resn_exch{"3H18 TUD"}="H183 TUD";
$resn_exch{"1H19 TUD"}="H191 TUD"; $resn_exch{"2H19 TUD"}="H192 TUD"; $resn_exch{"3H19 TUD"}="H193 TUD";
$resn_exch{"1H21 TUD"}="H211 TUD"; $resn_exch{"2H21 TUD"}="H212 TUD"; $resn_exch{"3H21 TUD"}="H213 TUD";
$resn_exch{"1H22 TUD"}="H221 TUD"; $resn_exch{"2H22 TUD"}="H222 TUD"; $resn_exch{"1H23 TUD"}="H231 TUD";
$resn_exch{"2H23 TUD"}="H232 TUD"; $resn_exch{"1H25 TUD"}="H251 TUD"; $resn_exch{"2H25 TUD"}="H252 TUD";
$resn_exch{"1H26 TUD"}="H261 TUD"; $resn_exch{"2H26 TUD"}="H262 TUD"; $resn_exch{"1H8  TUI"}=" H81 TUI";
$resn_exch{"2H8  TUI"}=" H82 TUI"; $resn_exch{"1H17 TUI"}="H171 TUI"; $resn_exch{"2H17 TUI"}="H172 TUI";
$resn_exch{" H1  TUO"}=" HAE TUO"; $resn_exch{" H2  TUO"}=" HAA TUO"; $resn_exch{" H3  TUO"}=" HAB TUO";
$resn_exch{" H4  TUO"}=" HAC TUO"; $resn_exch{" H5  TUO"}=" HAG TUO"; $resn_exch{" H6  TUO"}=" HAL TUO";
$resn_exch{"H7   TUO"}="HNAI TUO"; $resn_exch{"H8   TUO"}="HNAA TUO"; $resn_exch{" H9  TUO"}=" HAT TUO";
$resn_exch{" H10 TUO"}=" HAU TUO"; $resn_exch{"H11  TUO"}="HNAR TUO"; $resn_exch{"H12  TUO"}="HNAW TUO";
$resn_exch{"1H1G TUX"}="H1G1 TUX"; $resn_exch{"2H1G TUX"}="H1G2 TUX"; $resn_exch{"2H2A TUX"}="H2A2 TUX";
$resn_exch{"3H2A TUX"}="H2A3 TUX"; $resn_exch{"3H3A TUX"}="H3A3 TUX"; $resn_exch{"2H3A TUX"}="H3A2 TUX";
$resn_exch{"2H4A TUX"}="H4A2 TUX"; $resn_exch{"3H4A TUX"}="H4A3 TUX"; $resn_exch{"3H5A TUX"}="H5A3 TUX";
$resn_exch{"2H5A TUX"}="H5A2 TUX"; $resn_exch{"2H6A TUX"}="H6A2 TUX"; $resn_exch{"3H6A TUX"}="H6A3 TUX";
$resn_exch{"3H7A TUX"}="H7A3 TUX"; $resn_exch{"2H7A TUX"}="H7A2 TUX"; $resn_exch{"2H8A TUX"}="H8A2 TUX";
$resn_exch{"3H8A TUX"}="H8A3 TUX"; $resn_exch{"3H9A TUX"}="H9A3 TUX"; $resn_exch{"2H9A TUX"}="H9A2 TUX";
$resn_exch{"2HAA TUX"}="HAA2 TUX"; $resn_exch{"3HAA TUX"}="HAA3 TUX"; $resn_exch{"3HBA TUX"}="HBA3 TUX";
$resn_exch{"2HBA TUX"}="HBA2 TUX"; $resn_exch{"2HCA TUX"}="HCA2 TUX"; $resn_exch{"3HCA TUX"}="HCA3 TUX";
$resn_exch{"3HDA TUX"}="HDA3 TUX"; $resn_exch{"2HDA TUX"}="HDA2 TUX"; $resn_exch{"1HEA TUX"}="HEA1 TUX";
$resn_exch{"2HEA TUX"}="HEA2 TUX"; $resn_exch{"3HEA TUX"}="HEA3 TUX"; $resn_exch{"2HZH TUX"}="HZH2 TUX";
$resn_exch{"3HZH TUX"}="HZH3 TUX"; $resn_exch{"1HNH TUX"}="HNH1 TUX"; $resn_exch{"1HOH TUX"}="HOH1 TUX";
$resn_exch{"4HOG TUX"}="HOG4 TUX"; $resn_exch{"2H6G TUX"}="H6G2 TUX"; $resn_exch{"3H6G TUX"}="H6G3 TUX";
$resn_exch{"6HOG TUX"}="HOG6 TUX"; $resn_exch{"1HAJ TWN"}="HAJ1 TWN"; $resn_exch{"2HAJ TWN"}="HAJ2 TWN";
$resn_exch{"1HAM TWN"}="HAM1 TWN"; $resn_exch{"2HAM TWN"}="HAM2 TWN"; $resn_exch{"1HAO TWN"}="HAO1 TWN";
$resn_exch{"2HAO TWN"}="HAO2 TWN"; $resn_exch{"3HAO TWN"}="HAO3 TWN"; $resn_exch{"1HAD TWN"}="HAD1 TWN";
$resn_exch{"2HAD TWN"}="HAD2 TWN"; $resn_exch{"1HAN TWN"}="HAN1 TWN"; $resn_exch{"2HAN TWN"}="HAN2 TWN";
$resn_exch{"3HAN TWN"}="HAN3 TWN"; $resn_exch{"1HAK TWN"}="HAK1 TWN"; $resn_exch{"2HAK TWN"}="HAK2 TWN";
$resn_exch{"3HAK TWN"}="HAK3 TWN"; $resn_exch{"1HAL TWN"}="HAL1 TWN"; $resn_exch{"2HAL TWN"}="HAL2 TWN";
$resn_exch{"1HAP TWN"}="HAP1 TWN"; $resn_exch{"2HAP TWN"}="HAP2 TWN"; $resn_exch{"1HAW TWN"}="HAW1 TWN";
$resn_exch{"2HAW TWN"}="HAW2 TWN"; $resn_exch{"1HAS TWN"}="HAS1 TWN"; $resn_exch{"2HAS TWN"}="HAS2 TWN";
$resn_exch{"1HAX TWN"}="HAX1 TWN"; $resn_exch{"2HAX TWN"}="HAX2 TWN"; $resn_exch{"1HAT TWN"}="HAT1 TWN";
$resn_exch{"2HAT TWN"}="HAT2 TWN"; $resn_exch{"1HAU TWN"}="HAU1 TWN"; $resn_exch{"2HAU TWN"}="HAU2 TWN";
$resn_exch{"1HAV TWN"}="HAV1 TWN"; $resn_exch{"2HAV TWN"}="HAV2 TWN"; $resn_exch{"1HAR TWN"}="HAR1 TWN";
$resn_exch{"2HAR TWN"}="HAR2 TWN"; $resn_exch{"1HAQ TWN"}="HAQ1 TWN"; $resn_exch{"2HAQ TWN"}="HAQ2 TWN";
$resn_exch{"1HAY TWN"}="HAY1 TWN"; $resn_exch{"2HAY TWN"}="HAY2 TWN"; $resn_exch{"3HAY TWN"}="HAY3 TWN";
$resn_exch{"1H1C TWT"}="H1C1 TWT"; $resn_exch{"2H1C TWT"}="H1C2 TWT"; $resn_exch{"3H1C TWT"}="H1C3 TWT";
$resn_exch{"1H2C TWT"}="H2C1 TWT"; $resn_exch{"2H2C TWT"}="H2C2 TWT"; $resn_exch{"1H3C TWT"}="H3C1 TWT";
$resn_exch{"2H3C TWT"}="H3C2 TWT"; $resn_exch{"1H4C TWT"}="H4C1 TWT"; $resn_exch{"2H4C TWT"}="H4C2 TWT";
$resn_exch{"1H5C TWT"}="H5C1 TWT"; $resn_exch{"2H5C TWT"}="H5C2 TWT"; $resn_exch{"1H6C TWT"}="H6C1 TWT";
$resn_exch{"2H6C TWT"}="H6C2 TWT"; $resn_exch{"1H7C TWT"}="H7C1 TWT"; $resn_exch{"2H7C TWT"}="H7C2 TWT";
$resn_exch{"1H8C TWT"}="H8C1 TWT"; $resn_exch{"2H8C TWT"}="H8C2 TWT"; $resn_exch{"1H9C TWT"}="H9C1 TWT";
$resn_exch{"2H9C TWT"}="H9C2 TWT"; $resn_exch{"1H10 TWT"}="H101 TWT"; $resn_exch{"2H10 TWT"}="H102 TWT";
$resn_exch{"1H11 TWT"}="H111 TWT"; $resn_exch{"2H11 TWT"}="H112 TWT"; $resn_exch{"1H12 TWT"}="H121 TWT";
$resn_exch{"2H12 TWT"}="H122 TWT"; $resn_exch{"1H13 TWT"}="H131 TWT"; $resn_exch{"2H13 TWT"}="H132 TWT";
$resn_exch{"1H14 TWT"}="H141 TWT"; $resn_exch{"2H14 TWT"}="H142 TWT"; $resn_exch{"1H15 TWT"}="H151 TWT";
$resn_exch{"2H15 TWT"}="H152 TWT"; $resn_exch{"1H16 TWT"}="H161 TWT"; $resn_exch{"2H16 TWT"}="H162 TWT";
$resn_exch{"1H17 TWT"}="H171 TWT"; $resn_exch{"2H17 TWT"}="H172 TWT"; $resn_exch{"1H18 TWT"}="H181 TWT";
$resn_exch{"2H18 TWT"}="H182 TWT"; $resn_exch{"1H19 TWT"}="H191 TWT"; $resn_exch{"2H19 TWT"}="H192 TWT";
$resn_exch{"1H20 TWT"}="H201 TWT"; $resn_exch{"2H20 TWT"}="H202 TWT"; $resn_exch{"1H21 TWT"}="H211 TWT";
$resn_exch{"2H21 TWT"}="H212 TWT"; $resn_exch{"1H22 TWT"}="H221 TWT"; $resn_exch{"2H22 TWT"}="H222 TWT";
$resn_exch{"3H22 TWT"}="H223 TWT"; $resn_exch{"1H4  TX4"}=" H41 TX4"; $resn_exch{"2H4  TX4"}=" H42 TX4";
$resn_exch{"1H2  TX5"}=" H21 TX5"; $resn_exch{"2H2  TX5"}=" H22 TX5"; $resn_exch{"1H4  TX5"}=" H41 TX5";
$resn_exch{"2H4  TX5"}=" H42 TX5"; $resn_exch{"1H9  TX5"}=" H91 TX5"; $resn_exch{"2H9  TX5"}=" H92 TX5";
$resn_exch{"1H11 TX5"}="H111 TX5"; $resn_exch{"2H11 TX5"}="H112 TX5"; $resn_exch{"1H12 TX5"}="H121 TX5";
$resn_exch{"2H12 TX5"}="H122 TX5"; $resn_exch{"1H15 TX5"}="H151 TX5"; $resn_exch{"2H15 TX5"}="H152 TX5";
$resn_exch{"1H16 TX5"}="H161 TX5"; $resn_exch{"2H16 TX5"}="H162 TX5"; $resn_exch{"1H18 TX5"}="H181 TX5";
$resn_exch{"2H18 TX5"}="H182 TX5"; $resn_exch{"3H18 TX5"}="H183 TX5"; $resn_exch{"1H19 TX5"}="H191 TX5";
$resn_exch{"2H19 TX5"}="H192 TX5"; $resn_exch{"1H21 TX5"}="H211 TX5"; $resn_exch{"2H21 TX5"}="H212 TX5";
$resn_exch{"3H21 TX5"}="H213 TX5"; $resn_exch{"1H22 TX5"}="H221 TX5"; $resn_exch{"2H22 TX5"}="H222 TX5";
$resn_exch{"1H27 TX5"}="H271 TX5"; $resn_exch{"2H27 TX5"}="H272 TX5"; $resn_exch{"3H27 TX5"}="H273 TX5";
$resn_exch{"1H26 TX5"}="H261 TX5"; $resn_exch{"2H26 TX5"}="H262 TX5"; $resn_exch{"3H26 TX5"}="H263 TX5";
$resn_exch{"1H7N TXD"}="H7N1 TXD"; $resn_exch{"2H7N TXD"}="H7N2 TXD"; $resn_exch{"1H2N TXD"}="H2N1 TXD";
$resn_exch{"2H2N TXD"}="H2N2 TXD"; $resn_exch{"1H6N TXD"}="H6N1 TXD"; $resn_exch{"2H6N TXD"}="H6N2 TXD";
$resn_exch{"H1*  TXD"}="H1'A TXD"; $resn_exch{"H2*  TXD"}="H2'A TXD"; $resn_exch{"HO2* TXD"}="HO2A TXD";
$resn_exch{"H3*  TXD"}="H3'A TXD"; $resn_exch{"HO3* TXD"}="HO3A TXD"; $resn_exch{"H4*  TXD"}="H4'A TXD";
$resn_exch{"1H5* TXD"}="H5'1 TXD"; $resn_exch{"2H5* TXD"}="H5'2 TXD"; $resn_exch{"1H5' TXD"}="H5'A TXD";
$resn_exch{"2H5' TXD"}="H5'B TXD"; $resn_exch{"1H6A TXD"}="H6A1 TXD"; $resn_exch{"2H6A TXD"}="H6A2 TXD";
$resn_exch{"1H13 TXF"}="H131 TXF"; $resn_exch{"2H13 TXF"}="H132 TXF"; $resn_exch{"1H14 TXF"}="H141 TXF";
$resn_exch{"2H14 TXF"}="H142 TXF"; $resn_exch{"1H15 TXF"}="H151 TXF"; $resn_exch{"2H15 TXF"}="H152 TXF";
$resn_exch{"1H3  TXF"}=" H31 TXF"; $resn_exch{"2H3  TXF"}=" H32 TXF"; $resn_exch{"1H2  TXF"}="H21A TXF";
$resn_exch{"2H2  TXF"}="H22A TXF"; $resn_exch{"1H31 TXF"}="H311 TXF"; $resn_exch{"2H31 TXF"}="H312 TXF";
$resn_exch{"3H31 TXF"}="H313 TXF"; $resn_exch{"1H30 TXF"}="H301 TXF"; $resn_exch{"2H30 TXF"}="H302 TXF";
$resn_exch{"3H30 TXF"}="H303 TXF"; $resn_exch{"1H6  TXL"}=" H61 TXL"; $resn_exch{"2H6  TXL"}=" H62 TXL";
$resn_exch{"1H14 TXL"}="H141 TXL"; $resn_exch{"2H14 TXL"}="H142 TXL"; $resn_exch{"1H16 TXL"}="H161 TXL";
$resn_exch{"2H16 TXL"}="H162 TXL"; $resn_exch{"3H16 TXL"}="H163 TXL"; $resn_exch{"1H17 TXL"}="H171 TXL";
$resn_exch{"2H17 TXL"}="H172 TXL"; $resn_exch{"3H17 TXL"}="H173 TXL"; $resn_exch{"1H18 TXL"}="H181 TXL";
$resn_exch{"2H18 TXL"}="H182 TXL"; $resn_exch{"3H18 TXL"}="H183 TXL"; $resn_exch{"1H19 TXL"}="H191 TXL";
$resn_exch{"2H19 TXL"}="H192 TXL"; $resn_exch{"3H19 TXL"}="H193 TXL"; $resn_exch{"1H20 TXL"}="H201 TXL";
$resn_exch{"2H20 TXL"}="H202 TXL"; $resn_exch{"1H32 TXL"}="H321 TXL"; $resn_exch{"2H32 TXL"}="H322 TXL";
$resn_exch{"3H32 TXL"}="H323 TXL"; $resn_exch{"1H33 TXL"}="H331 TXL"; $resn_exch{"2H33 TXL"}="H332 TXL";
$resn_exch{"3H33 TXL"}="H333 TXL"; $resn_exch{"1H34 TXL"}="H341 TXL"; $resn_exch{"2H34 TXL"}="H342 TXL";
$resn_exch{"3H34 TXL"}="H343 TXL"; $resn_exch{"1H43 TXL"}="H431 TXL"; $resn_exch{"2H43 TXL"}="H432 TXL";
$resn_exch{"3H43 TXL"}="H433 TXL"; $resn_exch{"1H5* TXP"}="H5'1 TXP"; $resn_exch{"2H5* TXP"}="H5'2 TXP";
$resn_exch{"H4*  TXP"}="H4'A TXP"; $resn_exch{"H3*  TXP"}="H3'A TXP"; $resn_exch{"HO3* TXP"}="HO3A TXP";
$resn_exch{"H2*  TXP"}="H2'A TXP"; $resn_exch{"HO2* TXP"}="HO2' TXP"; $resn_exch{"H1*  TXP"}="H1'A TXP";
$resn_exch{"1H6N TXP"}="H6N1 TXP"; $resn_exch{"2H6N TXP"}="H6N2 TXP"; $resn_exch{"1H7N TXP"}="H7N1 TXP";
$resn_exch{"2H7N TXP"}="H7N2 TXP"; $resn_exch{"1H2N TXP"}="H2N1 TXP"; $resn_exch{"2H2N TXP"}="H2N2 TXP";
$resn_exch{"1H5' TXP"}="H5'A TXP"; $resn_exch{"2H5' TXP"}="H5'B TXP"; $resn_exch{"1H6A TXP"}="H6A1 TXP";
$resn_exch{"2H6A TXP"}="H6A2 TXP"; $resn_exch{"1HN  TY2"}=" HN1 TY2"; $resn_exch{"2HA  TY2"}=" HA2 TY2";
$resn_exch{"1HA  TY2"}=" HA1 TY2"; $resn_exch{"2HB  TY2"}=" HB2 TY2"; $resn_exch{"1HE2 TY2"}="HE21 TY2";
$resn_exch{"2HE2 TY2"}="HE22 TY2"; $resn_exch{"1HN  TY3"}=" HN1 TY3"; $resn_exch{"2HN  TY3"}=" HN2 TY3";
$resn_exch{"1HBC TY3"}="HBC1 TY3"; $resn_exch{"2HBC TY3"}="HBC2 TY3"; $resn_exch{"1H5* TYA"}="H5'1 TYA";
$resn_exch{"2H5* TYA"}="H5'2 TYA"; $resn_exch{" H4* TYA"}=" H4' TYA"; $resn_exch{" H3* TYA"}=" H3' TYA";
$resn_exch{"3HO* TYA"}="HO'3 TYA"; $resn_exch{" H2* TYA"}=" H2' TYA"; $resn_exch{"2HO* TYA"}="HO'2 TYA";
$resn_exch{" H1* TYA"}=" H1' TYA"; $resn_exch{"1H6  TYA"}=" H61 TYA"; $resn_exch{"2H6  TYA"}=" H62 TYA";
$resn_exch{"1HN  TYA"}=" HN1 TYA"; $resn_exch{"2HN  TYA"}=" HN2 TYA"; $resn_exch{"1H   TYA"}=" H1  TYA";
$resn_exch{"2H   TYA"}=" H2A TYA"; $resn_exch{"1HB  TYA"}=" HB1 TYA"; $resn_exch{"2HB  TYA"}=" HB2 TYA";
$resn_exch{" HN2 TYB"}=" H2  TYB"; $resn_exch{"1HB  TYB"}=" HB2 TYB"; $resn_exch{"2HB  TYB"}=" HB3 TYB";
$resn_exch{"1HT2 TYC"}="HT21 TYC"; $resn_exch{"2HT2 TYC"}="HT22 TYC"; $resn_exch{"2HOA TYD"}="HOA2 TYD";
$resn_exch{"2HOB TYD"}="HOB2 TYD"; $resn_exch{"3HOB TYD"}="HOB3 TYD"; $resn_exch{"1H5* TYD"}="H5'1 TYD";
$resn_exch{"2H5* TYD"}="H5'2 TYD"; $resn_exch{" H4* TYD"}=" H4' TYD"; $resn_exch{" H3* TYD"}=" H3' TYD";
$resn_exch{"*HO3 TYD"}="HO3' TYD"; $resn_exch{"1H2* TYD"}="H2'1 TYD"; $resn_exch{"2H2* TYD"}="H2'2 TYD";
$resn_exch{" H1* TYD"}=" H1' TYD"; $resn_exch{"1H5  TYD"}=" H51 TYD"; $resn_exch{"2H5  TYD"}=" H52 TYD";
$resn_exch{"3H5  TYD"}=" H53 TYD"; $resn_exch{"1HN  TYI"}=" H   TYI"; $resn_exch{"2HN  TYI"}=" H2  TYI";
$resn_exch{"1HB  TYI"}=" HB2 TYI"; $resn_exch{"2HB  TYI"}=" HB3 TYI"; $resn_exch{" HOH TYI"}=" HH  TYI";
$resn_exch{"1H19 TYK"}="H191 TYK"; $resn_exch{"2H19 TYK"}="H192 TYK"; $resn_exch{"1H2  TYK"}=" H21 TYK";
$resn_exch{"2H2  TYK"}=" H22 TYK"; $resn_exch{"1H7  TYK"}=" H71 TYK"; $resn_exch{"2H7  TYK"}=" H72 TYK";
$resn_exch{"1H16 TYK"}="H161 TYK"; $resn_exch{"2H16 TYK"}="H162 TYK"; $resn_exch{"1H17 TYK"}="H171 TYK";
$resn_exch{"2H17 TYK"}="H172 TYK"; $resn_exch{"3H17 TYK"}="H173 TYK"; $resn_exch{"1H18 TYK"}="H181 TYK";
$resn_exch{"2H18 TYK"}="H182 TYK"; $resn_exch{"3H18 TYK"}="H183 TYK"; $resn_exch{"1H21 TYK"}="H211 TYK";
$resn_exch{"2H21 TYK"}="H212 TYK"; $resn_exch{"3H21 TYK"}="H213 TYK"; $resn_exch{"1H22 TYK"}="H221 TYK";
$resn_exch{"2H22 TYK"}="H222 TYK"; $resn_exch{"3H22 TYK"}="H223 TYK"; $resn_exch{"1H23 TYK"}="H231 TYK";
$resn_exch{"2H23 TYK"}="H232 TYK"; $resn_exch{"1H6A TYK"}="H6A1 TYK"; $resn_exch{"2H6A TYK"}="H6A2 TYK";
$resn_exch{"3H6A TYK"}="H6A3 TYK"; $resn_exch{"1H7A TYK"}="H7A1 TYK"; $resn_exch{"2H7A TYK"}="H7A2 TYK";
$resn_exch{"3H7A TYK"}="H7A3 TYK"; $resn_exch{"1H8A TYK"}="H8A1 TYK"; $resn_exch{"2H8A TYK"}="H8A2 TYK";
$resn_exch{"3H8A TYK"}="H8A3 TYK"; $resn_exch{"1H2B TYK"}="H2B1 TYK"; $resn_exch{"2H2B TYK"}="H2B2 TYK";
$resn_exch{"1H6B TYK"}="H6B1 TYK"; $resn_exch{"2H6B TYK"}="H6B2 TYK"; $resn_exch{"3H6B TYK"}="H6B3 TYK";
$resn_exch{"1H7B TYK"}="H7B1 TYK"; $resn_exch{"2H7B TYK"}="H7B2 TYK"; $resn_exch{"3H7B TYK"}="H7B3 TYK";
$resn_exch{"1H6C TYK"}="H6C1 TYK"; $resn_exch{"2H6C TYK"}="H6C2 TYK"; $resn_exch{"3H6C TYK"}="H6C3 TYK";
$resn_exch{"1H7C TYK"}="H7C1 TYK"; $resn_exch{"2H7C TYK"}="H7C2 TYK"; $resn_exch{"3H7C TYK"}="H7C3 TYK";
$resn_exch{"1H8C TYK"}="H8C1 TYK"; $resn_exch{"2H8C TYK"}="H8C2 TYK"; $resn_exch{"3H8C TYK"}="H8C3 TYK";
$resn_exch{"1HM  TYL"}=" HM1 TYL"; $resn_exch{"2HM  TYL"}=" HM2 TYL"; $resn_exch{"3HM  TYL"}=" HM3 TYL";
$resn_exch{"OH1P TYM"}="H1PO TYM"; $resn_exch{"1H5' TYM"}="H5'1 TYM"; $resn_exch{"2H5' TYM"}="H5'2 TYM";
$resn_exch{"CH4' TYM"}="H4'C TYM"; $resn_exch{"CH1' TYM"}="H1'C TYM"; $resn_exch{"1HN6 TYM"}="HN61 TYM";
$resn_exch{"2HN6 TYM"}="HN62 TYM"; $resn_exch{"CH2' TYM"}="H2'C TYM"; $resn_exch{"OH2' TYM"}="H2'O TYM";
$resn_exch{"CH3' TYM"}="H3'C TYM"; $resn_exch{"OH3' TYM"}="H3'O TYM"; $resn_exch{"1HH3 TYM"}="HH31 TYM";
$resn_exch{"2HH3 TYM"}="HH32 TYM"; $resn_exch{"1HCB TYM"}="HCB1 TYM"; $resn_exch{"2HCB TYM"}="HCB2 TYM";
$resn_exch{"CHE3 TYM"}="HE3C TYM"; $resn_exch{"CHD1 TYM"}="HD1C TYM"; $resn_exch{"NHE1 TYM"}="HE1N TYM";
$resn_exch{"CHZ2 TYM"}="HZ2C TYM"; $resn_exch{"CHZ3 TYM"}="HZ3C TYM"; $resn_exch{"CHH2 TYM"}="HH2C TYM";
$resn_exch{"1HN  TYN"}=" H   TYN"; $resn_exch{"2HN  TYN"}=" H2  TYN"; $resn_exch{"1HB  TYN"}=" HB2 TYN";
$resn_exch{"2HB  TYN"}=" HB3 TYN"; $resn_exch{"1HN3 TYN"}="HN31 TYN"; $resn_exch{"2HN3 TYN"}="HN32 TYN";
$resn_exch{"1HN  TYO"}=" HN1 TYO"; $resn_exch{"2HN  TYO"}=" HN2 TYO"; $resn_exch{"1HB  TYO"}=" HB1 TYO";
$resn_exch{"2HB  TYO"}=" HB2 TYO"; $resn_exch{"1HD2 TYO"}="HD21 TYO"; $resn_exch{"1HBC TYP"}="HBC1 TYP";
$resn_exch{"2HBC TYP"}="HBC2 TYP"; $resn_exch{"1HBP TYP"}="HBP1 TYP"; $resn_exch{"2HBP TYP"}="HBP2 TYP";
$resn_exch{"1HGP TYP"}="HGP1 TYP"; $resn_exch{"2HGP TYP"}="HGP2 TYP"; $resn_exch{"1HDP TYP"}="HDP1 TYP";
$resn_exch{"2HDP TYP"}="HDP2 TYP"; $resn_exch{" HN2 TYQ"}=" H2  TYQ"; $resn_exch{"1HB  TYQ"}=" HB2 TYQ";
$resn_exch{"2HB  TYQ"}=" HB3 TYQ"; $resn_exch{" HOZ TYQ"}=" HZ  TYQ"; $resn_exch{" HN2 TYR"}=" H2  TYR";
$resn_exch{"1HB  TYR"}=" HB2 TYR"; $resn_exch{"2HB  TYR"}=" HB3 TYR"; $resn_exch{"1HN  TYS"}=" H   TYS";
$resn_exch{"2HN  TYS"}=" HN2 TYS"; $resn_exch{"1HB  TYS"}=" HB2 TYS"; $resn_exch{"2HB  TYS"}=" HB3 TYS";
$resn_exch{" H5* TYU"}=" H5' TYU"; $resn_exch{"1H5* TYU"}="H5'1 TYU"; $resn_exch{"2H5* TYU"}="H5'' TYU";
$resn_exch{" H4* TYU"}=" H4' TYU"; $resn_exch{" H1* TYU"}=" H1' TYU"; $resn_exch{"1H6  TYU"}=" H61 TYU";
$resn_exch{"2H6  TYU"}=" H62 TYU"; $resn_exch{"1H5  TYU"}=" H51 TYU"; $resn_exch{"2H5  TYU"}=" H52 TYU";
$resn_exch{" H2* TYU"}=" H2' TYU"; $resn_exch{" H3* TYU"}=" H3' TYU"; $resn_exch{"1H3  TYV"}=" H31 TYV";
$resn_exch{"2H3  TYV"}=" H32 TYV"; $resn_exch{"1H6  TYV"}=" H61 TYV"; $resn_exch{"2H6  TYV"}=" H62 TYV";
$resn_exch{"3H6  TYV"}=" H63 TYV"; $resn_exch{"1HN  TYX"}="HN1A TYX"; $resn_exch{"2HN  TYX"}=" HN2 TYX";
$resn_exch{"1HB  TYX"}=" HB1 TYX"; $resn_exch{"2HB  TYX"}=" HB2 TYX"; $resn_exch{"1H8  TYX"}=" H81 TYX";
$resn_exch{"2H8  TYX"}=" H82 TYX"; $resn_exch{" HN2 TYY"}=" H2  TYY"; $resn_exch{"1HB  TYY"}=" HB2 TYY";
$resn_exch{"2HB  TYY"}=" HB3 TYY"; $resn_exch{"1H9C TYZ"}="H9C1 TYZ"; $resn_exch{"2H9C TYZ"}="H9C2 TYZ";
$resn_exch{"3H9C TYZ"}="H9C3 TYZ"; $resn_exch{"1HN3 TZ4"}="HN31 TZ4"; $resn_exch{"2HN3 TZ4"}="HN32 TZ4";
$resn_exch{"1HN1 TZ4"}="HN11 TZ4"; $resn_exch{"2HN1 TZ4"}="HN12 TZ4"; $resn_exch{"1H20 TZ4"}="H201 TZ4";
$resn_exch{"2H20 TZ4"}="H202 TZ4"; $resn_exch{"1H21 TZ4"}="H211 TZ4"; $resn_exch{"2H21 TZ4"}="H212 TZ4";
$resn_exch{"1H22 TZ4"}="H221 TZ4"; $resn_exch{"2H22 TZ4"}="H222 TZ4"; $resn_exch{"1H23 TZ4"}="H231 TZ4";
$resn_exch{"2H23 TZ4"}="H232 TZ4"; $resn_exch{"1H24 TZ4"}="H241 TZ4"; $resn_exch{"2H24 TZ4"}="H242 TZ4";
$resn_exch{"1H25 TZ4"}="H251 TZ4"; $resn_exch{"2H25 TZ4"}="H252 TZ4"; $resn_exch{"1H28 TZ4"}="H281 TZ4";
$resn_exch{"2H28 TZ4"}="H282 TZ4"; $resn_exch{"1H29 TZ4"}="H291 TZ4"; $resn_exch{"2H29 TZ4"}="H292 TZ4";
$resn_exch{"1H38 TZ4"}="H381 TZ4"; $resn_exch{"2H38 TZ4"}="H382 TZ4"; $resn_exch{"1H40 TZ4"}="H401 TZ4";
$resn_exch{"2H40 TZ4"}="H402 TZ4"; $resn_exch{"1H41 TZ4"}="H411 TZ4"; $resn_exch{"2H41 TZ4"}="H412 TZ4";
$resn_exch{"1H42 TZ4"}="H421 TZ4"; $resn_exch{"2H42 TZ4"}="H422 TZ4"; $resn_exch{"1HN2 TZ5"}="HN21 TZ5";
$resn_exch{"2HN2 TZ5"}="HN22 TZ5"; $resn_exch{"1HN1 TZ5"}="HN11 TZ5"; $resn_exch{"2HN1 TZ5"}="HN12 TZ5";
$resn_exch{"1H20 TZ5"}="H201 TZ5"; $resn_exch{"2H20 TZ5"}="H202 TZ5"; $resn_exch{"1H21 TZ5"}="H211 TZ5";
$resn_exch{"2H21 TZ5"}="H212 TZ5"; $resn_exch{"1H22 TZ5"}="H221 TZ5"; $resn_exch{"2H22 TZ5"}="H222 TZ5";
$resn_exch{"1H23 TZ5"}="H231 TZ5"; $resn_exch{"2H23 TZ5"}="H232 TZ5"; $resn_exch{"1H24 TZ5"}="H241 TZ5";
$resn_exch{"2H24 TZ5"}="H242 TZ5"; $resn_exch{"1H25 TZ5"}="H251 TZ5"; $resn_exch{"2H25 TZ5"}="H252 TZ5";
$resn_exch{"1H28 TZ5"}="H281 TZ5"; $resn_exch{"2H28 TZ5"}="H282 TZ5"; $resn_exch{"1H29 TZ5"}="H291 TZ5";
$resn_exch{"2H29 TZ5"}="H292 TZ5"; $resn_exch{"1H38 TZ5"}="H381 TZ5"; $resn_exch{"2H38 TZ5"}="H382 TZ5";
$resn_exch{"1H40 TZ5"}="H401 TZ5"; $resn_exch{"2H40 TZ5"}="H402 TZ5"; $resn_exch{"1H41 TZ5"}="H411 TZ5";
$resn_exch{"2H41 TZ5"}="H412 TZ5"; $resn_exch{"1H42 TZ5"}="H421 TZ5"; $resn_exch{"2H42 TZ5"}="H422 TZ5";
$resn_exch{"1HNH TZA"}="HNH1 TZA"; $resn_exch{"2HNH TZA"}="HNH2 TZA"; $resn_exch{"2HE2 TZA"}="HE22 TZA";
$resn_exch{"2HD2 TZA"}="HD22 TZA"; $resn_exch{"1HE1 TZA"}="HE11 TZA"; $resn_exch{"1HD1 TZA"}="HD11 TZA";
$resn_exch{"1HCX TZA"}="HCX1 TZA"; $resn_exch{"2HCX TZA"}="HCX2 TZA"; $resn_exch{"1HCB TZA"}="HCB1 TZA";
$resn_exch{"2HCB TZA"}="HCB2 TZA"; $resn_exch{"1HN  TZA"}=" HN1 TZA"; $resn_exch{"2HN  TZA"}=" HN2 TZA";
$resn_exch{"1HC  TZB"}=" HC1 TZB"; $resn_exch{"2HC  TZB"}=" HC2 TZB"; $resn_exch{"3HC  TZB"}="HC3A TZB";
$resn_exch{"1HN  TZB"}=" HN1 TZB"; $resn_exch{"2HN  TZB"}=" HN2 TZB"; $resn_exch{"1HC4 TZB"}="HC41 TZB";
$resn_exch{"2HC4 TZB"}="HC42 TZB"; $resn_exch{"1HN7 TZC"}="HN71 TZC"; $resn_exch{"2HN7 TZC"}="HN72 TZC";
$resn_exch{"1HN8 TZC"}="HN81 TZC"; $resn_exch{"2HN8 TZC"}="HN82 TZC"; $resn_exch{"1H2A TZD"}="H2A1 TZD";
$resn_exch{"2H2A TZD"}="H2A2 TZD"; $resn_exch{"3H2A TZD"}="H2A3 TZD"; $resn_exch{"1H4* TZD"}="H4'1 TZD";
$resn_exch{"2H4* TZD"}="H4'2 TZD"; $resn_exch{" H6* TZD"}=" H6' TZD"; $resn_exch{"1H35 TZD"}="H351 TZD";
$resn_exch{"2H35 TZD"}="H352 TZD"; $resn_exch{"1H4A TZD"}="H4A1 TZD"; $resn_exch{"2H4A TZD"}="H4A2 TZD";
$resn_exch{"3H4A TZD"}="H4A3 TZD"; $resn_exch{"1H5A TZD"}="H5A1 TZD"; $resn_exch{"2H5A TZD"}="H5A2 TZD";
$resn_exch{"1H5B TZD"}="H5B1 TZD"; $resn_exch{"2H5B TZD"}="H5B2 TZD"; $resn_exch{"H11* TZE"}="H11' TZE";
$resn_exch{"H12* TZE"}="H12' TZE"; $resn_exch{"H21* TZE"}="H21' TZE"; $resn_exch{"H22* TZE"}="H22' TZE";
$resn_exch{"1H7  TZL"}=" H71 TZL"; $resn_exch{"2H7  TZL"}=" H72 TZL"; $resn_exch{"1H8  TZL"}=" H81 TZL";
$resn_exch{"2H8  TZL"}=" H82 TZL"; $resn_exch{"3H8  TZL"}=" H83 TZL"; $resn_exch{"1H9  TZL"}=" H91 TZL";
$resn_exch{"2H9  TZL"}=" H92 TZL"; $resn_exch{"3H9  TZL"}=" H93 TZL"; $resn_exch{"1HC2 TZO"}="HC21 TZO";
$resn_exch{"1HC4 TZO"}="HC41 TZO"; $resn_exch{"1H2  TZP"}=" H21 TZP"; $resn_exch{"1HM4 TZP"}="HM41 TZP";
$resn_exch{"2HM4 TZP"}="HM42 TZP"; $resn_exch{"3HM4 TZP"}="HM43 TZP"; $resn_exch{"1H6  TZP"}=" H61 TZP";
$resn_exch{"2H6  TZP"}=" H62 TZP"; $resn_exch{"1H7  TZP"}=" H71 TZP"; $resn_exch{"2H7  TZP"}=" H72 TZP";
$resn_exch{"1HO1 TZP"}="HO11 TZP"; $resn_exch{"1HO3 TZP"}="HO31 TZP"; $resn_exch{"1H24 TZY"}="H241 TZY";
$resn_exch{"2H24 TZY"}="H242 TZY"; $resn_exch{"3H24 TZY"}="H243 TZY"; $resn_exch{"1H23 TZY"}="H231 TZY";
$resn_exch{"2H23 TZY"}="H232 TZY"; $resn_exch{"3H23 TZY"}="H233 TZY"; $resn_exch{"1H   TZZ"}=" H1  TZZ";
$resn_exch{"2H   TZZ"}=" H2  TZZ"; $resn_exch{"3H   TZZ"}=" H3  TZZ"; $resn_exch{"1H1  TZZ"}=" H11 TZZ";
$resn_exch{"2H1  TZZ"}=" H12 TZZ"; $resn_exch{"3H1  TZZ"}=" H13 TZZ"; $resn_exch{"1H2  TZZ"}=" H21 TZZ";
$resn_exch{"2H2  TZZ"}=" H22 TZZ"; $resn_exch{"3H2  TZZ"}=" H23 TZZ"; $resn_exch{"3HOP   U"}="HOP3   U";
$resn_exch{"2HOP   U"}="HOP2   U"; $resn_exch{"1H5*   U"}=" H5'   U"; $resn_exch{"2H5*   U"}="H5''   U";
$resn_exch{" H4*   U"}=" H4'   U"; $resn_exch{" H3*   U"}=" H3'   U"; $resn_exch{"H3T    U"}="HO3'   U";
$resn_exch{" H2*   U"}=" H2'   U"; $resn_exch{"2HO*   U"}="HO2'   U"; $resn_exch{" H1*   U"}=" H1'   U";
$resn_exch{"1HG3 U01"}="HG31 U01"; $resn_exch{"2HG3 U01"}="HG32 U01"; $resn_exch{"3HG3 U01"}="HG33 U01";
$resn_exch{"1HG4 U01"}="HG41 U01"; $resn_exch{"2HG4 U01"}="HG42 U01"; $resn_exch{"3HG4 U01"}="HG43 U01";
$resn_exch{"1HZ8 U02"}="HZ81 U02"; $resn_exch{"2HZ8 U02"}="HZ82 U02"; $resn_exch{"3HZ8 U02"}="HZ83 U02";
$resn_exch{"1HZ7 U02"}="HZ71 U02"; $resn_exch{"2HZ7 U02"}="HZ72 U02"; $resn_exch{"3HZ7 U02"}="HZ73 U02";
$resn_exch{"1HZ6 U02"}="HZ61 U02"; $resn_exch{"2HZ6 U02"}="HZ62 U02"; $resn_exch{"3HZ6 U02"}="HZ63 U02";
$resn_exch{"3HNZ U02"}="HNZ3 U02"; $resn_exch{"1HZ3 U02"}="HZ31 U02"; $resn_exch{"2HZ3 U02"}="HZ32 U02";
$resn_exch{"1HZ2 U02"}="HZ21 U02"; $resn_exch{"2HZ2 U02"}="HZ22 U02"; $resn_exch{"1HNZ U02"}="HNZ1 U02";
$resn_exch{"1HD1 U02"}="HD11 U02"; $resn_exch{"2HD1 U02"}="HD12 U02"; $resn_exch{"1HD2 U02"}="HD21 U02";
$resn_exch{"2HD2 U02"}="HD22 U02"; $resn_exch{"3HD2 U02"}="HD23 U02"; $resn_exch{"1HE  U02"}=" HE1 U02";
$resn_exch{"2HE  U02"}="HE2A U02"; $resn_exch{"1HG2 U03"}="HG21 U03"; $resn_exch{"2HG2 U03"}="HG22 U03";
$resn_exch{"1HG3 U03"}="HG31 U03"; $resn_exch{"2HG3 U03"}="HG32 U03"; $resn_exch{"3HG3 U03"}="HG33 U03";
$resn_exch{"1HE3 U03"}="HE31 U03"; $resn_exch{"2HE3 U03"}="HE32 U03"; $resn_exch{"3HE3 U03"}="HE33 U03";
$resn_exch{"1HB7 U04"}="HB71 U04"; $resn_exch{"2HB7 U04"}="HB72 U04"; $resn_exch{"1HB8 U04"}="HB81 U04";
$resn_exch{"2HB8 U04"}="HB82 U04"; $resn_exch{"3HB8 U04"}="HB83 U04"; $resn_exch{"1HG2 U04"}="HG21 U04";
$resn_exch{"2HG2 U04"}="HG22 U04"; $resn_exch{"1HG5 U04"}="HG51 U04"; $resn_exch{"2HG5 U04"}="HG52 U04";
$resn_exch{"3HG5 U04"}="HG53 U04"; $resn_exch{"1HG6 U04"}="HG61 U04"; $resn_exch{"2HG6 U04"}="HG62 U04";
$resn_exch{"3HG6 U04"}="HG63 U04"; $resn_exch{"1HG7 U04"}="HG71 U04"; $resn_exch{"2HG7 U04"}="HG72 U04";
$resn_exch{"3HG7 U04"}="HG73 U04"; $resn_exch{"1H12 U05"}="H121 U05"; $resn_exch{"2H12 U05"}="H122 U05";
$resn_exch{"3H12 U05"}="H123 U05"; $resn_exch{"1H14 U05"}="H141 U05"; $resn_exch{"2H14 U05"}="H142 U05";
$resn_exch{"1H15 U05"}="H151 U05"; $resn_exch{"2H15 U05"}="H152 U05"; $resn_exch{"3H15 U05"}="H153 U05";
$resn_exch{"1H1  U0E"}="H11A U0E"; $resn_exch{"2H1  U0E"}=" H12 U0E"; $resn_exch{"3H1  U0E"}="H13A U0E";
$resn_exch{"1H4  U0E"}=" H41 U0E"; $resn_exch{"2H4  U0E"}=" H42 U0E"; $resn_exch{"1H6  U0E"}=" H61 U0E";
$resn_exch{"2H6  U0E"}=" H62 U0E"; $resn_exch{"1H7  U0E"}=" H71 U0E"; $resn_exch{"2H7  U0E"}=" H72 U0E";
$resn_exch{"1H8  U0E"}=" H81 U0E"; $resn_exch{"2H8  U0E"}=" H82 U0E"; $resn_exch{"1H9  U0E"}=" H91 U0E";
$resn_exch{"2H9  U0E"}=" H92 U0E"; $resn_exch{"1H10 U0E"}="H101 U0E"; $resn_exch{"2H10 U0E"}="H102 U0E";
$resn_exch{"1H12 U0E"}="H121 U0E"; $resn_exch{"2H12 U0E"}="H122 U0E"; $resn_exch{"1H15 U0E"}="H151 U0E";
$resn_exch{"2H15 U0E"}="H152 U0E"; $resn_exch{"3H15 U0E"}="H153 U0E"; $resn_exch{"1H16 U0E"}="H161 U0E";
$resn_exch{"2H16 U0E"}="H162 U0E"; $resn_exch{"3H16 U0E"}="H163 U0E"; $resn_exch{"1H19 U0E"}="H191 U0E";
$resn_exch{"2H19 U0E"}="H192 U0E"; $resn_exch{"1H20 U0E"}="H201 U0E"; $resn_exch{"2H20 U0E"}="H202 U0E";
$resn_exch{"1H25 U0E"}="H251 U0E"; $resn_exch{"2H25 U0E"}="H252 U0E"; $resn_exch{"1H26 U0E"}="H261 U0E";
$resn_exch{"2H26 U0E"}="H262 U0E"; $resn_exch{"1H27 U0E"}="H271 U0E"; $resn_exch{"2H27 U0E"}="H272 U0E";
$resn_exch{"1HN3 U0E"}="HN31 U0E"; $resn_exch{"2HN3 U0E"}="HN32 U0E"; $resn_exch{"1HN5 U0E"}="HN51 U0E";
$resn_exch{"2HN5 U0E"}="HN52 U0E"; $resn_exch{"1HN8 U0E"}="HN81 U0E"; $resn_exch{"2HN8 U0E"}="HN82 U0E";
$resn_exch{"1H1M U10"}="H1M1 U10"; $resn_exch{"2H1M U10"}="H1M2 U10"; $resn_exch{"3H1M U10"}="H1M3 U10";
$resn_exch{"1H3M U10"}="H3M1 U10"; $resn_exch{"2H3M U10"}="H3M2 U10"; $resn_exch{"3H3M U10"}="H3M3 U10";
$resn_exch{"1H4M U10"}="H4M1 U10"; $resn_exch{"2H4M U10"}="H4M2 U10"; $resn_exch{"3H4M U10"}="H4M3 U10";
$resn_exch{"1H7  U10"}=" H71 U10"; $resn_exch{"2H7  U10"}=" H72 U10"; $resn_exch{"1H10 U10"}="H101 U10";
$resn_exch{"2H10 U10"}="H102 U10"; $resn_exch{"3H10 U10"}="H103 U10"; $resn_exch{"1H11 U10"}="H111 U10";
$resn_exch{"2H11 U10"}="H112 U10"; $resn_exch{"1H12 U10"}="H121 U10"; $resn_exch{"2H12 U10"}="H122 U10";
$resn_exch{"1H15 U10"}="H151 U10"; $resn_exch{"2H15 U10"}="H152 U10"; $resn_exch{"3H15 U10"}="H153 U10";
$resn_exch{"1H16 U10"}="H161 U10"; $resn_exch{"2H16 U10"}="H162 U10"; $resn_exch{"1H17 U10"}="H171 U10";
$resn_exch{"2H17 U10"}="H172 U10"; $resn_exch{"1H20 U10"}="H201 U10"; $resn_exch{"2H20 U10"}="H202 U10";
$resn_exch{"3H20 U10"}="H203 U10"; $resn_exch{"1H21 U10"}="H211 U10"; $resn_exch{"2H21 U10"}="H212 U10";
$resn_exch{"1H22 U10"}="H221 U10"; $resn_exch{"2H22 U10"}="H222 U10"; $resn_exch{"1H25 U10"}="H251 U10";
$resn_exch{"2H25 U10"}="H252 U10"; $resn_exch{"3H25 U10"}="H253 U10"; $resn_exch{"1H26 U10"}="H261 U10";
$resn_exch{"2H26 U10"}="H262 U10"; $resn_exch{"1H27 U10"}="H271 U10"; $resn_exch{"2H27 U10"}="H272 U10";
$resn_exch{"1H30 U10"}="H301 U10"; $resn_exch{"2H30 U10"}="H302 U10"; $resn_exch{"3H30 U10"}="H303 U10";
$resn_exch{"1H31 U10"}="H311 U10"; $resn_exch{"2H31 U10"}="H312 U10"; $resn_exch{"1H32 U10"}="H321 U10";
$resn_exch{"2H32 U10"}="H322 U10"; $resn_exch{"1H35 U10"}="H351 U10"; $resn_exch{"2H35 U10"}="H352 U10";
$resn_exch{"3H35 U10"}="H353 U10"; $resn_exch{"1H36 U10"}="H361 U10"; $resn_exch{"2H36 U10"}="H362 U10";
$resn_exch{"1H37 U10"}="H371 U10"; $resn_exch{"2H37 U10"}="H372 U10"; $resn_exch{"1H40 U10"}="H401 U10";
$resn_exch{"2H40 U10"}="H402 U10"; $resn_exch{"3H40 U10"}="H403 U10"; $resn_exch{"1H41 U10"}="H411 U10";
$resn_exch{"2H41 U10"}="H412 U10"; $resn_exch{"1H42 U10"}="H421 U10"; $resn_exch{"2H42 U10"}="H422 U10";
$resn_exch{"1H45 U10"}="H451 U10"; $resn_exch{"2H45 U10"}="H452 U10"; $resn_exch{"3H45 U10"}="H453 U10";
$resn_exch{"1H46 U10"}="H461 U10"; $resn_exch{"2H46 U10"}="H462 U10"; $resn_exch{"1H47 U10"}="H471 U10";
$resn_exch{"2H47 U10"}="H472 U10"; $resn_exch{"1H50 U10"}="H501 U10"; $resn_exch{"2H50 U10"}="H502 U10";
$resn_exch{"3H50 U10"}="H503 U10"; $resn_exch{"1H51 U10"}="H511 U10"; $resn_exch{"2H51 U10"}="H512 U10";
$resn_exch{"1H52 U10"}="H521 U10"; $resn_exch{"2H52 U10"}="H522 U10"; $resn_exch{"1H55 U10"}="H551 U10";
$resn_exch{"2H55 U10"}="H552 U10"; $resn_exch{"3H55 U10"}="H553 U10"; $resn_exch{"1H56 U10"}="H561 U10";
$resn_exch{"2H56 U10"}="H562 U10"; $resn_exch{"3H56 U10"}="H563 U10"; $resn_exch{"1H24 U11"}="H241 U11";
$resn_exch{"2H24 U11"}="H242 U11"; $resn_exch{"1H37 U11"}="H371 U11"; $resn_exch{"2H37 U11"}="H372 U11";
$resn_exch{"3H37 U11"}="H373 U11"; $resn_exch{"1H43 U11"}="H431 U11"; $resn_exch{"2H43 U11"}="H432 U11";
$resn_exch{"1H62 U11"}="H621 U11"; $resn_exch{"2H62 U11"}="H622 U11"; $resn_exch{"3H62 U11"}="H623 U11";
$resn_exch{"2HN5 U12"}="HN52 U12"; $resn_exch{"1HN2 U12"}="HN21 U12"; $resn_exch{"2HN2 U12"}="HN22 U12";
$resn_exch{"1H16 U13"}="H161 U13"; $resn_exch{"2H16 U13"}="H162 U13"; $resn_exch{"2H18 U13"}="H182 U13";
$resn_exch{"1HN4 U14"}="HN41 U14"; $resn_exch{"2HN4 U14"}="HN42 U14"; $resn_exch{"2HN1 U14"}="HN12 U14";
$resn_exch{"1H44 U15"}="H441 U15"; $resn_exch{"2H44 U15"}="H442 U15"; $resn_exch{"3H44 U15"}="H443 U15";
$resn_exch{"1H39 U15"}="H391 U15"; $resn_exch{"2H39 U15"}="H392 U15"; $resn_exch{"1H53 U15"}="H531 U15";
$resn_exch{"2H53 U15"}="H532 U15"; $resn_exch{"3H53 U15"}="H533 U15"; $resn_exch{"1H51 U15"}="H511 U15";
$resn_exch{"2H51 U15"}="H512 U15"; $resn_exch{"3H51 U15"}="H513 U15"; $resn_exch{"1H28 U15"}="H281 U15";
$resn_exch{"2H28 U15"}="H282 U15"; $resn_exch{"3H28 U15"}="H283 U15"; $resn_exch{"1H15 U15"}="H151 U15";
$resn_exch{"2H15 U15"}="H152 U15"; $resn_exch{"1H9  U15"}=" H91 U15"; $resn_exch{"2H9  U15"}=" H92 U15";
$resn_exch{"3H9  U15"}=" H93 U15"; $resn_exch{"1H14 U16"}="H141 U16"; $resn_exch{"2H14 U16"}="H142 U16";
$resn_exch{"3H14 U16"}="H143 U16"; $resn_exch{"1H15 U16"}="H151 U16"; $resn_exch{"2H15 U16"}="H152 U16";
$resn_exch{"3H15 U16"}="H153 U16"; $resn_exch{"1H24 U16"}="H241 U16"; $resn_exch{"2H24 U16"}="H242 U16";
$resn_exch{"1H37 U16"}="H371 U16"; $resn_exch{"2H37 U16"}="H372 U16"; $resn_exch{"3H37 U16"}="H373 U16";
$resn_exch{"1H45 U16"}="H451 U16"; $resn_exch{"2H45 U16"}="H452 U16"; $resn_exch{"1H52 U16"}="H521 U16";
$resn_exch{"2H52 U16"}="H522 U16"; $resn_exch{"3H52 U16"}="H523 U16"; $resn_exch{"1H53 U16"}="H531 U16";
$resn_exch{"2H53 U16"}="H532 U16"; $resn_exch{"3H53 U16"}="H533 U16"; $resn_exch{"1H62 U16"}="H621 U16";
$resn_exch{"2H62 U16"}="H622 U16"; $resn_exch{"3H62 U16"}="H623 U16"; $resn_exch{"1H3  U17"}=" H31 U17";
$resn_exch{"2H3  U17"}=" H32 U17"; $resn_exch{"1H6  U17"}=" H61 U17"; $resn_exch{"2H6  U17"}=" H62 U17";
$resn_exch{"1H2  U17"}=" H21 U17"; $resn_exch{"2H2  U17"}=" H22 U17"; $resn_exch{"1H5  U17"}=" H51 U17";
$resn_exch{"2H5  U17"}=" H52 U17"; $resn_exch{"3H5  U17"}=" H53 U17"; $resn_exch{"1HN1 U17"}="HN11 U17";
$resn_exch{"2HN1 U17"}="HN12 U17"; $resn_exch{"1H13 U17"}="H131 U17"; $resn_exch{"2H13 U17"}="H132 U17";
$resn_exch{"1H15 U17"}="H151 U17"; $resn_exch{"2H15 U17"}="H152 U17"; $resn_exch{"1H18 U17"}="H181 U17";
$resn_exch{"2H18 U17"}="H182 U17"; $resn_exch{"3H18 U17"}="H183 U17"; $resn_exch{"1H19 U17"}="H191 U17";
$resn_exch{"2H19 U17"}="H192 U17"; $resn_exch{"3H19 U17"}="H193 U17"; $resn_exch{"1H20 U17"}="H201 U17";
$resn_exch{"2H20 U17"}="H202 U17"; $resn_exch{"3H20 U17"}="H203 U17"; $resn_exch{"1HA2 U18"}="HA21 U18";
$resn_exch{"2HA2 U18"}="HA22 U18"; $resn_exch{"3HA2 U18"}="HA23 U18"; $resn_exch{"1H11 U18"}="H111 U18";
$resn_exch{"2H11 U18"}="H112 U18"; $resn_exch{"1H19 U18"}="H191 U18"; $resn_exch{"2H19 U18"}="H192 U18";
$resn_exch{"1HB  U18"}=" HB1 U18"; $resn_exch{"2HB  U18"}=" HB2 U18"; $resn_exch{"1HG  U18"}=" HG1 U18";
$resn_exch{"2HG  U18"}=" HG2 U18"; $resn_exch{"1HN5 U19"}="HN51 U19"; $resn_exch{"2HN5 U19"}="HN52 U19";
$resn_exch{"2HN4 U19"}="HN42 U19"; $resn_exch{"1H5  U1K"}=" H51 U1K"; $resn_exch{"2H5  U1K"}=" H52 U1K";
$resn_exch{"1H13 U1K"}="H131 U1K"; $resn_exch{"2H13 U1K"}="H132 U1K"; $resn_exch{"3H13 U1K"}="H133 U1K";
$resn_exch{"1H14 U1K"}="H141 U1K"; $resn_exch{"2H14 U1K"}="H142 U1K"; $resn_exch{"3H14 U1K"}="H143 U1K";
$resn_exch{"1H15 U1K"}="H151 U1K"; $resn_exch{"2H15 U1K"}="H152 U1K"; $resn_exch{"3H15 U1K"}="H153 U1K";
$resn_exch{"1H16 U1K"}="H161 U1K"; $resn_exch{"2H16 U1K"}="H162 U1K"; $resn_exch{"3H16 U1K"}="H163 U1K";
$resn_exch{"1H7  U1N"}=" H71 U1N"; $resn_exch{"2H7  U1N"}=" H72 U1N"; $resn_exch{"1H8  U1N"}=" H81 U1N";
$resn_exch{"2H8  U1N"}=" H82 U1N"; $resn_exch{"1H11 U1N"}="H111 U1N"; $resn_exch{"2H11 U1N"}="H112 U1N";
$resn_exch{"3H11 U1N"}="H113 U1N"; $resn_exch{"1H13 U1N"}="H131 U1N"; $resn_exch{"2H13 U1N"}="H132 U1N";
$resn_exch{"1H24 U1N"}="H241 U1N"; $resn_exch{"2H24 U1N"}="H242 U1N"; $resn_exch{"1H6  U20"}=" H61 U20";
$resn_exch{"2H6  U20"}=" H62 U20"; $resn_exch{"1HCB U20"}="HCB1 U20"; $resn_exch{"2HCB U20"}="HCB2 U20";
$resn_exch{"3HCB U20"}="HCB3 U20"; $resn_exch{"1HCA U20"}="HCA1 U20"; $resn_exch{"2HCA U20"}="HCA2 U20";
$resn_exch{"1HBZ U20"}="HBZ1 U20"; $resn_exch{"2HBZ U20"}="HBZ2 U20"; $resn_exch{"1HBY U20"}="HBY1 U20";
$resn_exch{"2HBY U20"}="HBY2 U20"; $resn_exch{"1HBX U20"}="HBX1 U20"; $resn_exch{"2HBX U20"}="HBX2 U20";
$resn_exch{"1HBW U20"}="HBW1 U20"; $resn_exch{"2HBW U20"}="HBW2 U20"; $resn_exch{"1HBV U20"}="HBV1 U20";
$resn_exch{"2HBV U20"}="HBV2 U20"; $resn_exch{"1HBU U20"}="HBU1 U20"; $resn_exch{"2HBU U20"}="HBU2 U20";
$resn_exch{"1HBS U20"}="HBS1 U20"; $resn_exch{"2HBS U20"}="HBS2 U20"; $resn_exch{"1HBR U20"}="HBR1 U20";
$resn_exch{"2HBR U20"}="HBR2 U20"; $resn_exch{"1HBL U20"}="HBL1 U20"; $resn_exch{"2HBL U20"}="HBL2 U20";
$resn_exch{"1HBI U20"}="HBI1 U20"; $resn_exch{"2HBI U20"}="HBI2 U20"; $resn_exch{"1HAQ U20"}="HAQ1 U20";
$resn_exch{"2HAQ U20"}="HAQ2 U20"; $resn_exch{"3HAQ U20"}="HAQ3 U20"; $resn_exch{"1H5' U20"}="H5'1 U20";
$resn_exch{"2H5' U20"}="H5'2 U20"; $resn_exch{"1H6  U21"}=" H61 U21"; $resn_exch{"2H6  U21"}=" H62 U21";
$resn_exch{"1HBX U21"}="HBX1 U21"; $resn_exch{"2HBX U21"}="HBX2 U21"; $resn_exch{"3HBX U21"}="HBX3 U21";
$resn_exch{"1HBW U21"}="HBW1 U21"; $resn_exch{"2HBW U21"}="HBW2 U21"; $resn_exch{"1HBV U21"}="HBV1 U21";
$resn_exch{"2HBV U21"}="HBV2 U21"; $resn_exch{"1HBU U21"}="HBU1 U21"; $resn_exch{"2HBU U21"}="HBU2 U21";
$resn_exch{"1HBS U21"}="HBS1 U21"; $resn_exch{"2HBS U21"}="HBS2 U21"; $resn_exch{"1HBR U21"}="HBR1 U21";
$resn_exch{"2HBR U21"}="HBR2 U21"; $resn_exch{"1HBL U21"}="HBL1 U21"; $resn_exch{"2HBL U21"}="HBL2 U21";
$resn_exch{"1HBI U21"}="HBI1 U21"; $resn_exch{"2HBI U21"}="HBI2 U21"; $resn_exch{"1HAQ U21"}="HAQ1 U21";
$resn_exch{"2HAQ U21"}="HAQ2 U21"; $resn_exch{"3HAQ U21"}="HAQ3 U21"; $resn_exch{"1H5' U21"}="H5'1 U21";
$resn_exch{"2H5' U21"}="H5'2 U21"; $resn_exch{"3HOP U25"}="HOP3 U25"; $resn_exch{"2HOP U25"}="HOP2 U25";
$resn_exch{"1H5* U25"}=" H5' U25"; $resn_exch{"2H5* U25"}="H5'' U25"; $resn_exch{" H4* U25"}=" H4' U25";
$resn_exch{" H3* U25"}=" H3' U25"; $resn_exch{"3HO* U25"}="HO'3 U25"; $resn_exch{" H2* U25"}=" H2' U25";
$resn_exch{"2HO* U25"}="HO2' U25"; $resn_exch{" H1* U25"}=" H1' U25"; $resn_exch{"1H5' U2F"}="H5'1 U2F";
$resn_exch{"2H5' U2F"}="H5'2 U2F"; $resn_exch{"1H6C U2F"}="H6C1 U2F"; $resn_exch{"2H6C U2F"}="H6C2 U2F";
$resn_exch{"1H2G U2G"}="H2G1 U2G"; $resn_exch{"2H2G U2G"}="H2G2 U2G"; $resn_exch{"*H1G U2G"}="H1G' U2G";
$resn_exch{"*H2G U2G"}="H2G' U2G"; $resn_exch{"2HOG U2G"}="HOG2 U2G"; $resn_exch{"*H3G U2G"}="H3G' U2G";
$resn_exch{"3HOG U2G"}="HOG3 U2G"; $resn_exch{"*H4G U2G"}="H4G' U2G"; $resn_exch{"1H5G U2G"}="H5G1 U2G";
$resn_exch{"2H5G U2G"}="H5G2 U2G"; $resn_exch{"2HOP U2G"}="HOP2 U2G"; $resn_exch{"5HOU U2G"}="HOU5 U2G";
$resn_exch{"1H5U U2G"}="H5U1 U2G"; $resn_exch{"2H5U U2G"}="H5U2 U2G"; $resn_exch{"*H4U U2G"}="H4U' U2G";
$resn_exch{"*H3U U2G"}="H3U' U2G"; $resn_exch{"3HOU U2G"}="HOU3 U2G"; $resn_exch{"*H2U U2G"}="H2U' U2G";
$resn_exch{"*H1U U2G"}="H1U' U2G"; $resn_exch{"3HOP U2L"}="HOP3 U2L"; $resn_exch{"2HOP U2L"}="HOP2 U2L";
$resn_exch{"1H5* U2L"}=" H5' U2L"; $resn_exch{"2H5* U2L"}="H5'' U2L"; $resn_exch{" H4* U2L"}=" H4' U2L";
$resn_exch{" H3* U2L"}=" H3' U2L"; $resn_exch{" H2* U2L"}=" H2' U2L"; $resn_exch{"2HO* U2L"}="HO2' U2L";
$resn_exch{" H1* U2L"}=" H1' U2L"; $resn_exch{"1HA* U2L"}="HA'1 U2L"; $resn_exch{"2HA* U2L"}="HA'2 U2L";
$resn_exch{"1HB* U2L"}="HB'1 U2L"; $resn_exch{"2HB* U2L"}="HB'2 U2L"; $resn_exch{"1HD* U2L"}="HD'1 U2L";
$resn_exch{"2HD* U2L"}="HD'2 U2L"; $resn_exch{"3HD* U2L"}="HD'3 U2L"; $resn_exch{"1H7  U2L"}=" H71 U2L";
$resn_exch{"2H7  U2L"}=" H72 U2L"; $resn_exch{"3H7  U2L"}=" H73 U2L"; $resn_exch{"1H5* U2N"}=" H5' U2N";
$resn_exch{"2H5* U2N"}="H5'' U2N"; $resn_exch{" H4* U2N"}=" H4' U2N"; $resn_exch{" H1* U2N"}=" H1' U2N";
$resn_exch{" H2* U2N"}=" H2' U2N"; $resn_exch{"1H2* U2N"}="H2'1 U2N"; $resn_exch{"2H2* U2N"}="H2'' U2N";
$resn_exch{" H3* U2N"}=" H3' U2N"; $resn_exch{"H3T  U2N"}="HO3' U2N"; $resn_exch{"1H5' U2P"}="H5'1 U2P";
$resn_exch{"2H5' U2P"}="H5'2 U2P"; $resn_exch{"1H5* U31"}="H5'1 U31"; $resn_exch{"2H5* U31"}="H5'2 U31";
$resn_exch{" H4* U31"}=" H4' U31"; $resn_exch{" H3* U31"}=" H3' U31"; $resn_exch{" H2* U31"}=" H2' U31";
$resn_exch{" H1* U31"}=" H1' U31"; $resn_exch{"1HA* U31"}="HA'1 U31"; $resn_exch{"2HA* U31"}="HA'2 U31";
$resn_exch{"1HB* U31"}="HB'1 U31"; $resn_exch{"2HB* U31"}="HB'2 U31"; $resn_exch{"1HCC U31"}="HCC1 U31";
$resn_exch{"2HCC U31"}="HCC2 U31"; $resn_exch{"1HD* U31"}="HD'1 U31"; $resn_exch{"2HD* U31"}="HD'2 U31";
$resn_exch{"1H5C U32"}="H5C1 U32"; $resn_exch{"2H5C U32"}="H5C2 U32"; $resn_exch{"3H5C U32"}="H5C3 U32";
$resn_exch{"1H6C U32"}="H6C1 U32"; $resn_exch{"2H6C U32"}="H6C2 U32"; $resn_exch{"3H6C U32"}="H6C3 U32";
$resn_exch{"1H11 U32"}="H111 U32"; $resn_exch{"2H11 U32"}="H112 U32"; $resn_exch{" H1* U33"}=" H1' U33";
$resn_exch{"1H2* U33"}="H2'1 U33"; $resn_exch{"2H2* U33"}="H2'2 U33"; $resn_exch{" H3* U33"}=" H3' U33";
$resn_exch{" H4* U33"}=" H4' U33"; $resn_exch{"1H5* U33"}="H5'1 U33"; $resn_exch{"2H5* U33"}="H5'2 U33";
$resn_exch{" H5* U33"}=" H5' U33"; $resn_exch{"1H5* U34"}="H5'1 U34"; $resn_exch{"2H5* U34"}="H5'2 U34";
$resn_exch{" H4* U34"}=" H4' U34"; $resn_exch{" H3* U34"}=" H3' U34"; $resn_exch{" H2* U34"}=" H2' U34";
$resn_exch{" H1* U34"}=" H1' U34"; $resn_exch{"1H5* U36"}="H5'1 U36"; $resn_exch{"2H5* U36"}="H5'2 U36";
$resn_exch{" H4* U36"}=" H4' U36"; $resn_exch{" H3* U36"}=" H3' U36"; $resn_exch{" H2* U36"}=" H2' U36";
$resn_exch{" H1* U36"}=" H1' U36"; $resn_exch{"1HA* U36"}="HA'1 U36"; $resn_exch{"2HA* U36"}="HA'2 U36";
$resn_exch{"1HB* U36"}="HB'1 U36"; $resn_exch{"2HB* U36"}="HB'2 U36"; $resn_exch{"1HD* U36"}="HD'1 U36";
$resn_exch{"2HD* U36"}="HD'2 U36"; $resn_exch{"3HD* U36"}="HD'3 U36"; $resn_exch{"1H5* U37"}="H5'1 U37";
$resn_exch{"2H5* U37"}="H5'2 U37"; $resn_exch{" H4* U37"}=" H4' U37"; $resn_exch{" H3* U37"}=" H3' U37";
$resn_exch{" H2* U37"}=" H2' U37"; $resn_exch{" H1* U37"}=" H1' U37"; $resn_exch{" H2* U3H"}=" H2' U3H";
$resn_exch{" H1* U3H"}=" H1' U3H"; $resn_exch{"*H2* U3H"}="H2'' U3H"; $resn_exch{" H3* U3H"}=" H3' U3H";
$resn_exch{"2H3* U3H"}="H3'2 U3H"; $resn_exch{" H4* U3H"}=" H4' U3H"; $resn_exch{"1H5* U3H"}="H5'1 U3H";
$resn_exch{"2H5* U3H"}="H5'2 U3H"; $resn_exch{"1HOP U3P"}="HOP1 U3P"; $resn_exch{"3HOP U3P"}="HOP3 U3P";
$resn_exch{"*HO5 U3P"}="HO5' U3P"; $resn_exch{"1H5* U3P"}="H5'1 U3P"; $resn_exch{"2H5* U3P"}="H5'2 U3P";
$resn_exch{" H4* U3P"}=" H4' U3P"; $resn_exch{" H3* U3P"}=" H3' U3P"; $resn_exch{" H2* U3P"}=" H2' U3P";
$resn_exch{"*HO2 U3P"}="HO2' U3P"; $resn_exch{" H1* U3P"}=" H1' U3P"; $resn_exch{"1H4  U49"}=" H41 U49";
$resn_exch{"1H6  U49"}=" H61 U49"; $resn_exch{"1H7  U49"}=" H71 U49"; $resn_exch{"1H8  U49"}=" H81 U49";
$resn_exch{"1H10 U49"}="H101 U49"; $resn_exch{"2H10 U49"}="H102 U49"; $resn_exch{"1H12 U49"}="H121 U49";
$resn_exch{"1H13 U49"}="H131 U49"; $resn_exch{"1H15 U49"}="H151 U49"; $resn_exch{"2H15 U49"}="H152 U49";
$resn_exch{"1H17 U49"}="H171 U49"; $resn_exch{"1H21 U49"}="H211 U49"; $resn_exch{"1H22 U49"}="H221 U49";
$resn_exch{"2H22 U49"}="H222 U49"; $resn_exch{"1H23 U49"}="H231 U49"; $resn_exch{"1H27 U49"}="H271 U49";
$resn_exch{"1H29 U49"}="H291 U49"; $resn_exch{"2H29 U49"}="H292 U49"; $resn_exch{"1H31 U49"}="H311 U49";
$resn_exch{"1H32 U49"}="H321 U49"; $resn_exch{"1H54 U49"}="H541 U49"; $resn_exch{"1HAR U51"}="HAR1 U51";
$resn_exch{"2HAR U51"}="HAR2 U51"; $resn_exch{"1HAQ U51"}="HAQ1 U51"; $resn_exch{"2HAQ U51"}="HAQ2 U51";
$resn_exch{"1HAP U51"}="HAP1 U51"; $resn_exch{"2HAP U51"}="HAP2 U51"; $resn_exch{"1HAO U51"}="HAO1 U51";
$resn_exch{"2HAO U51"}="HAO2 U51"; $resn_exch{"1HAB U51"}="HAB1 U51"; $resn_exch{"2HAB U51"}="HAB2 U51";
$resn_exch{"1HAI U51"}="HAI1 U51"; $resn_exch{"2HAI U51"}="HAI2 U51"; $resn_exch{"1HAJ U51"}="HAJ1 U51";
$resn_exch{"2HAJ U51"}="HAJ2 U51"; $resn_exch{"1HAK U51"}="HAK1 U51"; $resn_exch{"2HAK U51"}="HAK2 U51";
$resn_exch{"1HAL U51"}="HAL1 U51"; $resn_exch{"2HAL U51"}="HAL2 U51"; $resn_exch{"1HAM U51"}="HAM1 U51";
$resn_exch{"2HAM U51"}="HAM2 U51"; $resn_exch{"1HAU U51"}="HAU1 U51"; $resn_exch{"2HAU U51"}="HAU2 U51";
$resn_exch{"3HAU U51"}="HAU3 U51"; $resn_exch{"1H17 U55"}="H171 U55"; $resn_exch{"2H17 U55"}="H172 U55";
$resn_exch{"1H13 U55"}="H131 U55"; $resn_exch{"1H16 U55"}="H161 U55"; $resn_exch{"1H18 U55"}="H181 U55";
$resn_exch{"1H06 U55"}="H061 U55"; $resn_exch{"1H05 U55"}="H051 U55"; $resn_exch{"1H02 U55"}="H021 U55";
$resn_exch{"1H03 U55"}="H031 U55"; $resn_exch{"1H10 U55"}="H101 U55"; $resn_exch{"2H10 U55"}="H102 U55";
$resn_exch{" H1* U5P"}=" H1' U5P"; $resn_exch{" H2* U5P"}=" H2' U5P"; $resn_exch{"*HO2 U5P"}="HO2' U5P";
$resn_exch{" H3* U5P"}=" H3' U5P"; $resn_exch{" H4* U5P"}=" H4' U5P"; $resn_exch{"*HO3 U5P"}="HO3' U5P";
$resn_exch{"1H5* U5P"}="H5'1 U5P"; $resn_exch{"2H5* U5P"}="H5'2 U5P"; $resn_exch{"2HOP U5P"}="HOP2 U5P";
$resn_exch{"3HOP U5P"}="HOP3 U5P"; $resn_exch{"1H3  U66"}=" H31 U66"; $resn_exch{"1H4  U66"}=" H41 U66";
$resn_exch{"1H5  U66"}=" H51 U66"; $resn_exch{"1H6  U66"}=" H61 U66"; $resn_exch{"1H9  U66"}=" H91 U66";
$resn_exch{"2H9  U66"}=" H92 U66"; $resn_exch{"1H10 U66"}="H101 U66"; $resn_exch{"1H11 U66"}="H111 U66";
$resn_exch{"1H12 U66"}="H121 U66"; $resn_exch{"1H14 U66"}="H141 U66"; $resn_exch{"2H14 U66"}="H142 U66";
$resn_exch{"1H15 U66"}="H151 U66"; $resn_exch{"1H16 U66"}="H161 U66"; $resn_exch{"17H1 U66"}="H117 U66";
$resn_exch{"1H18 U66"}="H181 U66"; $resn_exch{"2H18 U66"}="H182 U66"; $resn_exch{"1H19 U66"}="H191 U66";
$resn_exch{"2H19 U66"}="H192 U66"; $resn_exch{"1H20 U66"}="H201 U66"; $resn_exch{"2H20 U66"}="H202 U66";
$resn_exch{"1H23 U66"}="H231 U66"; $resn_exch{"2H23 U66"}="H232 U66"; $resn_exch{"1H24 U66"}="H241 U66";
$resn_exch{"1H26 U66"}="H261 U66"; $resn_exch{"2H26 U66"}="H262 U66"; $resn_exch{"1H29 U66"}="H291 U66";
$resn_exch{"1H30 U66"}="H301 U66"; $resn_exch{"1H8C U73"}="H8C1 U73"; $resn_exch{"2H8C U73"}="H8C2 U73";
$resn_exch{"3H8C U73"}="H8C3 U73"; $resn_exch{"1H7C U73"}="H7C1 U73"; $resn_exch{"2H7C U73"}="H7C2 U73";
$resn_exch{"3H7C U73"}="H7C3 U73"; $resn_exch{"1H12 U73"}="H121 U73"; $resn_exch{"2H12 U73"}="H122 U73";
$resn_exch{"1HB  U89"}=" HB1 U89"; $resn_exch{"2HB  U89"}=" HB2 U89"; $resn_exch{"1HG  U89"}=" HG1 U89";
$resn_exch{"2HG  U89"}=" HG2 U89"; $resn_exch{"1H1  U89"}=" H11 U89"; $resn_exch{"2H1  U89"}=" H12 U89";
$resn_exch{"1H2  U89"}=" H21 U89"; $resn_exch{"2H2  U89"}=" H22 U89"; $resn_exch{"1H3  U89"}=" H31 U89";
$resn_exch{"2H3  U89"}=" H32 U89"; $resn_exch{"1H4  U89"}=" H41 U89"; $resn_exch{"2H4  U89"}=" H42 U89";
$resn_exch{"1H7  U89"}=" H71 U89"; $resn_exch{"2H7  U89"}=" H72 U89"; $resn_exch{"1H8  U89"}=" H81 U89";
$resn_exch{"2H8  U89"}=" H82 U89"; $resn_exch{"1H11 U89"}="H111 U89"; $resn_exch{"2H11 U89"}="H112 U89";
$resn_exch{"1H12 U89"}="H121 U89"; $resn_exch{"2H12 U89"}="H122 U89"; $resn_exch{"1H13 U89"}="H131 U89";
$resn_exch{"2H13 U89"}="H132 U89"; $resn_exch{"1HN8 U89"}="HN81 U89"; $resn_exch{"2HN8 U89"}="HN82 U89";
$resn_exch{"1HN1 U89"}="HN11 U89"; $resn_exch{"2HN1 U89"}="HN12 U89"; $resn_exch{"2HOP U89"}="HOP2 U89";
$resn_exch{"3HOP U89"}="HOP3 U89"; $resn_exch{"1HC  U8U"}=" HC1 U8U"; $resn_exch{"2HC  U8U"}=" HC2 U8U";
$resn_exch{"1HN  U8U"}=" HN1 U8U"; $resn_exch{"1HA  U8U"}=" HA1 U8U"; $resn_exch{"2HA  U8U"}=" HA2 U8U";
$resn_exch{"3HA  U8U"}=" HA3 U8U"; $resn_exch{" H1* U8U"}=" H1' U8U"; $resn_exch{"2HO* U8U"}="HO2' U8U";
$resn_exch{" H2* U8U"}=" H2' U8U"; $resn_exch{"H3T  U8U"}="HO3' U8U"; $resn_exch{" H3* U8U"}=" H3' U8U";
$resn_exch{" H4* U8U"}=" H4' U8U"; $resn_exch{"1H5* U8U"}=" H5' U8U"; $resn_exch{"2H5* U8U"}="H5'' U8U";
$resn_exch{"2HOP U8U"}="HOP2 U8U"; $resn_exch{"3HOP U8U"}="HOP3 U8U"; $resn_exch{"1H55 UA1"}="H551 UA1";
$resn_exch{"2H55 UA1"}="H552 UA1"; $resn_exch{"3H55 UA1"}="H553 UA1"; $resn_exch{"1H54 UA1"}="H541 UA1";
$resn_exch{"2H54 UA1"}="H542 UA1"; $resn_exch{"3H54 UA1"}="H543 UA1"; $resn_exch{"1H53 UA1"}="H531 UA1";
$resn_exch{"2H53 UA1"}="H532 UA1"; $resn_exch{"3H53 UA1"}="H533 UA1"; $resn_exch{"1H34 UA1"}="H341 UA1";
$resn_exch{"2H34 UA1"}="H342 UA1"; $resn_exch{"1H23 UA1"}="H231 UA1"; $resn_exch{"2H23 UA1"}="H232 UA1";
$resn_exch{"3H23 UA1"}="H233 UA1"; $resn_exch{"1H66 UA1"}="H661 UA1"; $resn_exch{"2H66 UA1"}="H662 UA1";
$resn_exch{"HO5* UA2"}="HO5' UA2"; $resn_exch{"1H5* UA2"}="H5'1 UA2"; $resn_exch{"2H5* UA2"}="H5'2 UA2";
$resn_exch{" H4* UA2"}=" H4' UA2"; $resn_exch{"HN4* UA2"}="HN4' UA2"; $resn_exch{" H3* UA2"}=" H3' UA2";
$resn_exch{"HO3* UA2"}="HO3' UA2"; $resn_exch{" H2* UA2"}=" H2' UA2"; $resn_exch{"HO2* UA2"}="HO2' UA2";
$resn_exch{" H1* UA2"}=" H1' UA2"; $resn_exch{"1HN6 UA2"}="HN61 UA2"; $resn_exch{"2HN6 UA2"}="HN62 UA2";
$resn_exch{" H5* UA3"}=" H5' UA3"; $resn_exch{"1H5  UA3"}=" H51 UA3"; $resn_exch{"2H5  UA3"}=" H52 UA3";
$resn_exch{" H3* UA3"}=" H3' UA3"; $resn_exch{" H2* UA3"}=" H2' UA3"; $resn_exch{" H1* UA3"}=" H1' UA3";
$resn_exch{"1H10 UA4"}="H101 UA4"; $resn_exch{"2H10 UA4"}="H102 UA4"; $resn_exch{"3H10 UA4"}="H103 UA4";
$resn_exch{"1H22 UA4"}="H221 UA4"; $resn_exch{"2H22 UA4"}="H222 UA4"; $resn_exch{"1H24 UA4"}="H241 UA4";
$resn_exch{"2H24 UA4"}="H242 UA4"; $resn_exch{"1H26 UA4"}="H261 UA4"; $resn_exch{"2H26 UA4"}="H262 UA4";
$resn_exch{"3H26 UA4"}="H263 UA4"; $resn_exch{"1H35 UA4"}="H351 UA4"; $resn_exch{"2H35 UA4"}="H352 UA4";
$resn_exch{"3H35 UA4"}="H353 UA4"; $resn_exch{"1H23 UA4"}="H231 UA4"; $resn_exch{"2H23 UA4"}="H232 UA4";
$resn_exch{"1H9  UA4"}=" H91 UA4"; $resn_exch{"2H9  UA4"}=" H92 UA4"; $resn_exch{"1HN7 UA4"}="HN71 UA4";
$resn_exch{"2HN7 UA4"}="HN72 UA4"; $resn_exch{"1HN8 UA4"}="HN81 UA4"; $resn_exch{"2HN8 UA4"}="HN82 UA4";
$resn_exch{"1H7  UA5"}=" H71 UA5"; $resn_exch{"2H7  UA5"}=" H72 UA5"; $resn_exch{"1H8  UA5"}=" H81 UA5";
$resn_exch{"2H8  UA5"}=" H82 UA5"; $resn_exch{"3H8  UA5"}=" H83 UA5"; $resn_exch{"1H7C UAA"}="H7C1 UAA";
$resn_exch{"2H7C UAA"}="H7C2 UAA"; $resn_exch{" H1* UAG"}=" H1B UAG"; $resn_exch{" H2* UAG"}=" H2B UAG";
$resn_exch{"2HO* UAG"}="HO'2 UAG"; $resn_exch{" H3* UAG"}=" H3B UAG"; $resn_exch{"3HO* UAG"}="HO'3 UAG";
$resn_exch{" H4* UAG"}=" H4B UAG"; $resn_exch{"1H5* UAG"}="H5'1 UAG"; $resn_exch{"2H5* UAG"}="H5'2 UAG";
$resn_exch{"2HOA UAG"}="HOA2 UAG"; $resn_exch{"2HOB UAG"}="HOB2 UAG"; $resn_exch{"'HN2 UAG"}="HN2' UAG";
$resn_exch{"1H8' UAG"}="H8'1 UAG"; $resn_exch{"2H8' UAG"}="H8'2 UAG"; $resn_exch{"3H8' UAG"}="H8'3 UAG";
$resn_exch{"'HO4 UAG"}="HO4' UAG"; $resn_exch{"1H6' UAG"}="H6'1 UAG"; $resn_exch{"2H6' UAG"}="H6'2 UAG";
$resn_exch{"'HO6 UAG"}="HO6' UAG"; $resn_exch{"1H20 UAG"}="H201 UAG"; $resn_exch{"2H20 UAG"}="H202 UAG";
$resn_exch{"3H20 UAG"}="H203 UAG"; $resn_exch{"1H23 UAG"}="H231 UAG"; $resn_exch{"2H23 UAG"}="H232 UAG";
$resn_exch{"3H23 UAG"}="H233 UAG"; $resn_exch{"1H25 UAG"}="H251 UAG"; $resn_exch{"2H25 UAG"}="H252 UAG";
$resn_exch{"1H26 UAG"}="H261 UAG"; $resn_exch{"2H26 UAG"}="H262 UAG"; $resn_exch{"1H1  UAP"}=" H11 UAP";
$resn_exch{"2H1  UAP"}=" H12 UAP"; $resn_exch{"3HOS UAP"}="HOS3 UAP"; $resn_exch{"3HOP UAR"}="HOP3 UAR";
$resn_exch{"2HOP UAR"}="HOP2 UAR"; $resn_exch{"1H5* UAR"}=" H5' UAR"; $resn_exch{"2H5* UAR"}="H5'' UAR";
$resn_exch{" H4* UAR"}=" H4' UAR"; $resn_exch{" H3* UAR"}=" H3' UAR"; $resn_exch{"H3T  UAR"}="HO3' UAR";
$resn_exch{" H2* UAR"}=" H2' UAR"; $resn_exch{"2HO* UAR"}="HO2' UAR"; $resn_exch{" H1* UAR"}=" H1' UAR";
$resn_exch{"1HN  UBA"}=" HN1 UBA"; $resn_exch{"2HN  UBA"}=" HN2 UBA"; $resn_exch{"1HB  UBA"}=" HB1 UBA";
$resn_exch{"2HB  UBA"}=" HB2 UBA"; $resn_exch{"1H6  UBA"}=" H61 UBA"; $resn_exch{"2H6  UBA"}=" H62 UBA";
$resn_exch{"3H6  UBA"}=" H63 UBA"; $resn_exch{"1H17 UBA"}="H171 UBA"; $resn_exch{"2H17 UBA"}="H172 UBA";
$resn_exch{"1H5* UBB"}="H5'1 UBB"; $resn_exch{"2H5* UBB"}="H5'2 UBB"; $resn_exch{" H4* UBB"}=" H4' UBB";
$resn_exch{" H1* UBB"}=" H1' UBB"; $resn_exch{"1H2* UBB"}="H2'1 UBB"; $resn_exch{"2H2* UBB"}="H2'2 UBB";
$resn_exch{"1H3* UBB"}="H3'1 UBB"; $resn_exch{"2H3* UBB"}="H3'2 UBB"; $resn_exch{"1H17 UBC"}="H171 UBC";
$resn_exch{"2H17 UBC"}="H172 UBC"; $resn_exch{"1HB  UBC"}=" HB1 UBC"; $resn_exch{"2HB  UBC"}=" HB2 UBC";
$resn_exch{"1HN  UBC"}=" HN1 UBC"; $resn_exch{"2HN  UBC"}=" HN2 UBC"; $resn_exch{"1HA  UC1"}=" HA1 UC1";
$resn_exch{"2HA  UC1"}=" HA2 UC1"; $resn_exch{"3HA  UC1"}=" HA3 UC1"; $resn_exch{"1HC  UC1"}=" HC1 UC1";
$resn_exch{"2HC  UC1"}=" HC2 UC1"; $resn_exch{"1HF  UC1"}=" HF1 UC1"; $resn_exch{"2HF  UC1"}=" HF2 UC1";
$resn_exch{"3HF  UC1"}=" HF3 UC1"; $resn_exch{"1HG  UC1"}=" HG1 UC1"; $resn_exch{"2HG  UC1"}=" HG2 UC1";
$resn_exch{"3HG  UC1"}=" HG3 UC1"; $resn_exch{"1HA  UC2"}=" HA1 UC2"; $resn_exch{"2HA  UC2"}=" HA2 UC2";
$resn_exch{"3HA  UC2"}=" HA3 UC2"; $resn_exch{"1HF  UC2"}=" HF1 UC2"; $resn_exch{"2HF  UC2"}=" HF2 UC2";
$resn_exch{"3HF  UC2"}=" HF3 UC2"; $resn_exch{"1HG  UC2"}=" HG1 UC2"; $resn_exch{"2HG  UC2"}=" HG2 UC2";
$resn_exch{"3HG  UC2"}=" HG3 UC2"; $resn_exch{"1HH  UC2"}=" HH1 UC2"; $resn_exch{"2HH  UC2"}=" HH2 UC2";
$resn_exch{"3HH  UC2"}=" HH3 UC2"; $resn_exch{"1HD  UC3"}=" HD1 UC3"; $resn_exch{"2HD  UC3"}=" HD2 UC3";
$resn_exch{"3HD  UC3"}=" HD3 UC3"; $resn_exch{"1HE  UC3"}=" HE1 UC3"; $resn_exch{"2HE  UC3"}=" HE2 UC3";
$resn_exch{"3HE  UC3"}=" HE3 UC3"; $resn_exch{"1HH  UC3"}=" HH1 UC3"; $resn_exch{"2HH  UC3"}=" HH2 UC3";
$resn_exch{"3HH  UC3"}=" HH3 UC3"; $resn_exch{"1HI  UC3"}=" HI1 UC3"; $resn_exch{"2HI  UC3"}=" HI2 UC3";
$resn_exch{"3HI  UC3"}=" HI3 UC3"; $resn_exch{"1H9  UC4"}=" H91 UC4"; $resn_exch{"2H9  UC4"}=" H92 UC4";
$resn_exch{"3H9  UC4"}=" H93 UC4"; $resn_exch{"1H10 UC4"}="H101 UC4"; $resn_exch{"2H10 UC4"}="H102 UC4";
$resn_exch{"3H10 UC4"}="H103 UC4"; $resn_exch{"1HD  UC4"}=" HD1 UC4"; $resn_exch{"2HD  UC4"}=" HD2 UC4";
$resn_exch{"1HE  UC4"}=" HE1 UC4"; $resn_exch{"2HE  UC4"}=" HE2 UC4"; $resn_exch{"1HF  UC4"}=" HF1 UC4";
$resn_exch{"2HF  UC4"}=" HF2 UC4"; $resn_exch{"3HF  UC4"}=" HF3 UC4"; $resn_exch{" H1* UC5"}=" H1' UC5";
$resn_exch{"1H2* UC5"}="H2'1 UC5"; $resn_exch{"2H2* UC5"}="H2'2 UC5"; $resn_exch{" H3* UC5"}=" H3' UC5";
$resn_exch{"HO3* UC5"}="HO3' UC5"; $resn_exch{" H4* UC5"}=" H4' UC5"; $resn_exch{"1H5* UC5"}="H5'1 UC5";
$resn_exch{"2H5* UC5"}="H5'2 UC5"; $resn_exch{"1H3A UC5"}="H3A1 UC5"; $resn_exch{"2H3A UC5"}="H3A2 UC5";
$resn_exch{"1H29 UCB"}="H291 UCB"; $resn_exch{"2H29 UCB"}="H292 UCB"; $resn_exch{"3H29 UCB"}="H293 UCB";
$resn_exch{"1HAA UCD"}="HAA1 UCD"; $resn_exch{"2HAA UCD"}="HAA2 UCD"; $resn_exch{"3HAA UCD"}="HAA3 UCD";
$resn_exch{"1H6  UCD"}=" H61 UCD"; $resn_exch{"2H6  UCD"}=" H62 UCD"; $resn_exch{"1H5* UCL"}="H5'1 UCL";
$resn_exch{"2H5* UCL"}="H5'2 UCL"; $resn_exch{" H4* UCL"}=" H4' UCL"; $resn_exch{" H3* UCL"}=" H3' UCL";
$resn_exch{"HO3* UCL"}="HO3' UCL"; $resn_exch{"1H2* UCL"}="H2'1 UCL"; $resn_exch{"2H2* UCL"}="H2'2 UCL";
$resn_exch{" H1* UCL"}=" H1' UCL"; $resn_exch{"1H03 UCM"}="H031 UCM"; $resn_exch{"2H03 UCM"}="H032 UCM";
$resn_exch{"1H04 UCM"}="H041 UCM"; $resn_exch{"2H04 UCM"}="H042 UCM"; $resn_exch{"1H24 UCN"}="H241 UCN";
$resn_exch{"2H24 UCN"}="H242 UCN"; $resn_exch{"1H28 UCN"}="H281 UCN"; $resn_exch{"2H28 UCN"}="H282 UCN";
$resn_exch{"3H28 UCN"}="H283 UCN"; $resn_exch{"1H27 UCN"}="H271 UCN"; $resn_exch{"2H27 UCN"}="H272 UCN";
$resn_exch{"3H27 UCN"}="H273 UCN"; $resn_exch{"1H26 UCN"}="H261 UCN"; $resn_exch{"2H26 UCN"}="H262 UCN";
$resn_exch{"3H26 UCN"}="H263 UCN"; $resn_exch{" H1* UCP"}=" H1' UCP"; $resn_exch{"1H2* UCP"}="H2'1 UCP";
$resn_exch{"2H2* UCP"}="H2'2 UCP"; $resn_exch{" H3* UCP"}=" H3' UCP"; $resn_exch{"3HO* UCP"}="HO'3 UCP";
$resn_exch{" H4* UCP"}=" H4' UCP"; $resn_exch{"1H5* UCP"}="H5'1 UCP"; $resn_exch{"2H5* UCP"}="H5'2 UCP";
$resn_exch{"2HOA UCP"}="HOA2 UCP"; $resn_exch{"2HOB UCP"}="HOB2 UCP"; $resn_exch{"1HCC UCP"}="HCC1 UCP";
$resn_exch{"2HCC UCP"}="HCC2 UCP"; $resn_exch{"2HOG UCP"}="HOG2 UCP"; $resn_exch{"3HOG UCP"}="HOG3 UCP";
$resn_exch{"1H6' UD1"}="H6'1 UD1"; $resn_exch{"2H6' UD1"}="H6'2 UD1"; $resn_exch{"1H8' UD1"}="H8'1 UD1";
$resn_exch{"2H8' UD1"}="H8'2 UD1"; $resn_exch{"3H8' UD1"}="H8'3 UD1"; $resn_exch{"'HN2 UD1"}="HN2' UD1";
$resn_exch{"'HO3 UD1"}="HO3' UD1"; $resn_exch{"'HO4 UD1"}="HO4' UD1"; $resn_exch{"'HO6 UD1"}="HO6' UD1";
$resn_exch{" H1* UD1"}=" H1B UD1"; $resn_exch{" H2* UD1"}=" H2B UD1"; $resn_exch{"*HO2 UD1"}="HO2' UD1";
$resn_exch{" H3* UD1"}=" H3B UD1"; $resn_exch{" H4* UD1"}=" H4B UD1"; $resn_exch{"*HO3 UD1"}="HO3A UD1";
$resn_exch{"1H5* UD1"}="H5'1 UD1"; $resn_exch{"2H5* UD1"}="H5'2 UD1"; $resn_exch{"2HOA UD1"}="HOA2 UD1";
$resn_exch{"2HOB UD1"}="HOB2 UD1"; $resn_exch{"1H6' UD2"}="H6'1 UD2"; $resn_exch{"2H6' UD2"}="H6'2 UD2";
$resn_exch{"1H8' UD2"}="H8'1 UD2"; $resn_exch{"2H8' UD2"}="H8'2 UD2"; $resn_exch{"3H8' UD2"}="H8'3 UD2";
$resn_exch{" H1* UD2"}=" H1B UD2"; $resn_exch{" H2* UD2"}=" H2B UD2"; $resn_exch{"HO2* UD2"}="HO2' UD2";
$resn_exch{" H3* UD2"}=" H3B UD2"; $resn_exch{" H4* UD2"}=" H4B UD2"; $resn_exch{"HO3* UD2"}="HO3A UD2";
$resn_exch{"1H5* UD2"}="H5'1 UD2"; $resn_exch{"2H5* UD2"}="H5'2 UD2"; $resn_exch{"HO1A UD2"}="HOA2 UD2";
$resn_exch{"HO1B UD2"}="HOB2 UD2"; $resn_exch{"1H5* UD5"}=" H5' UD5"; $resn_exch{"2H5* UD5"}="H5'' UD5";
$resn_exch{" H4* UD5"}=" H4' UD5"; $resn_exch{" H3* UD5"}=" H3' UD5"; $resn_exch{" H2* UD5"}=" H2' UD5";
$resn_exch{" H1* UD5"}=" H1' UD5"; $resn_exch{"1H12 UDA"}="H121 UDA"; $resn_exch{"2H12 UDA"}="H122 UDA";
$resn_exch{"1H17 UDA"}="H171 UDA"; $resn_exch{"2H17 UDA"}="H172 UDA"; $resn_exch{"1H8  UDA"}=" H81 UDA";
$resn_exch{"2H8  UDA"}=" H82 UDA"; $resn_exch{"3H8  UDA"}=" H83 UDA"; $resn_exch{"1H19 UDA"}="H191 UDA";
$resn_exch{"2H19 UDA"}="H192 UDA"; $resn_exch{"3H19 UDA"}="H193 UDA"; $resn_exch{" H1* UDH"}=" H1' UDH";
$resn_exch{" H2* UDH"}=" H2' UDH"; $resn_exch{"*HO2 UDH"}="HO2' UDH"; $resn_exch{" H3* UDH"}=" H3' UDH";
$resn_exch{" H4* UDH"}=" H4' UDH"; $resn_exch{"*HO3 UDH"}="HO3' UDH"; $resn_exch{"1H5* UDH"}="H5'1 UDH";
$resn_exch{"2H5* UDH"}="H5'2 UDH"; $resn_exch{"2HOA UDH"}="HOA2 UDH"; $resn_exch{"3HOB UDH"}="HOB3 UDH";
$resn_exch{"1H1' UDH"}="H1'1 UDH"; $resn_exch{"2H1' UDH"}="H1'2 UDH"; $resn_exch{"1H2' UDH"}="H2'1 UDH";
$resn_exch{"2H2' UDH"}="H2'2 UDH"; $resn_exch{"1H3' UDH"}="H3'1 UDH"; $resn_exch{"2H3' UDH"}="H3'2 UDH";
$resn_exch{"1H4' UDH"}="H4'1 UDH"; $resn_exch{"2H4' UDH"}="H4'2 UDH"; $resn_exch{"1H5' UDH"}=" H5B UDH";
$resn_exch{"2H5' UDH"}=" H5D UDH"; $resn_exch{"1H6' UDH"}="H6'1 UDH"; $resn_exch{"2H6' UDH"}="H6'2 UDH";
$resn_exch{"1HN' UDH"}="HN'1 UDH"; $resn_exch{"2HN' UDH"}="HN'2 UDH"; $resn_exch{"1H6' UDM"}="H6'1 UDM";
$resn_exch{"2H6' UDM"}="H6'2 UDM"; $resn_exch{"1H8' UDM"}="H8'1 UDM"; $resn_exch{"2H8' UDM"}="H8'2 UDM";
$resn_exch{"3H8' UDM"}="H8'3 UDM"; $resn_exch{"1HB  UDM"}=" HB1 UDM"; $resn_exch{"2HB  UDM"}=" HB2 UDM";
$resn_exch{" H1* UDM"}=" H1D UDM"; $resn_exch{" H2* UDM"}=" H2D UDM"; $resn_exch{" H3* UDM"}=" H3D UDM";
$resn_exch{" H4* UDM"}=" H4D UDM"; $resn_exch{"1H5* UDM"}="H5'1 UDM"; $resn_exch{"2H5* UDM"}="H5'2 UDM";
$resn_exch{" H1* UDP"}=" H1' UDP"; $resn_exch{" H2* UDP"}=" H2' UDP"; $resn_exch{"*HO2 UDP"}="HO2' UDP";
$resn_exch{" H3* UDP"}=" H3' UDP"; $resn_exch{" H4* UDP"}=" H4' UDP"; $resn_exch{"*HO3 UDP"}="HO3' UDP";
$resn_exch{"1H5* UDP"}="H5'1 UDP"; $resn_exch{"2H5* UDP"}="H5'2 UDP"; $resn_exch{"2HOA UDP"}="HOA2 UDP";
$resn_exch{"2HOB UDP"}="HOB2 UDP"; $resn_exch{"3HOB UDP"}="HOB3 UDP"; $resn_exch{"1H11 UDT"}="H111 UDT";
$resn_exch{"2H11 UDT"}="H112 UDT"; $resn_exch{"3H11 UDT"}="H113 UDT"; $resn_exch{"1H10 UDT"}="H101 UDT";
$resn_exch{"2H10 UDT"}="H102 UDT"; $resn_exch{"1H9  UDT"}=" H91 UDT"; $resn_exch{"2H9  UDT"}=" H92 UDT";
$resn_exch{"1H8  UDT"}=" H81 UDT"; $resn_exch{"2H8  UDT"}=" H82 UDT"; $resn_exch{"1H7  UDT"}=" H71 UDT";
$resn_exch{"2H7  UDT"}=" H72 UDT"; $resn_exch{"1H6  UDT"}=" H61 UDT"; $resn_exch{"2H6  UDT"}=" H62 UDT";
$resn_exch{"1H5  UDT"}=" H51 UDT"; $resn_exch{"2H5  UDT"}=" H52 UDT"; $resn_exch{"1H4  UDT"}=" H41 UDT";
$resn_exch{"2H4  UDT"}=" H42 UDT"; $resn_exch{"1H3  UDT"}=" H31 UDT"; $resn_exch{"2H3  UDT"}=" H32 UDT";
$resn_exch{"1H2  UDT"}=" H21 UDT"; $resn_exch{"2H2  UDT"}=" H22 UDT"; $resn_exch{"1H2' UDX"}="H2'1 UDX";
$resn_exch{"2HO' UDX"}="HO'2 UDX"; $resn_exch{"1H3' UDX"}="H3'1 UDX"; $resn_exch{"3HO' UDX"}="HO'3 UDX";
$resn_exch{"1H4' UDX"}="H4'1 UDX"; $resn_exch{"4HO' UDX"}="HO'4 UDX"; $resn_exch{"1H5' UDX"}="H5'1 UDX";
$resn_exch{"2H5' UDX"}="H5'2 UDX"; $resn_exch{"2HOB UDX"}="HOB2 UDX"; $resn_exch{"2HOA UDX"}="HOA2 UDX";
$resn_exch{"1H5* UDX"}="H5A1 UDX"; $resn_exch{"2H5* UDX"}="H5A2 UDX"; $resn_exch{"1H4* UDX"}="H4A1 UDX";
$resn_exch{"1H3* UDX"}="H3A1 UDX"; $resn_exch{"3HO* UDX"}="HOA3 UDX"; $resn_exch{"1H2* UDX"}="H2A1 UDX";
$resn_exch{"2HO* UDX"}="HOC2 UDX"; $resn_exch{"1H1* UDX"}="H1'1 UDX"; $resn_exch{"1H6  UDX"}=" H61 UDX";
$resn_exch{"1H5  UDX"}=" H51 UDX"; $resn_exch{" H1* UFG"}=" H1D UFG"; $resn_exch{" H2* UFG"}=" H2D UFG";
$resn_exch{"*HO2 UFG"}="HO2' UFG"; $resn_exch{" H3* UFG"}=" H3D UFG"; $resn_exch{" H4* UFG"}=" H4D UFG";
$resn_exch{"*HO3 UFG"}="HO3' UFG"; $resn_exch{"1H5* UFG"}="H5'1 UFG"; $resn_exch{"2H5* UFG"}="H5'2 UFG";
$resn_exch{"2HOA UFG"}="HOA2 UFG"; $resn_exch{"2HOB UFG"}="HOB2 UFG"; $resn_exch{"1H6' UFG"}="H6'1 UFG";
$resn_exch{"2H6' UFG"}="H6'2 UFG"; $resn_exch{"'HO2 UFG"}="HO2A UFG"; $resn_exch{"'HO3 UFG"}="HO3A UFG";
$resn_exch{"'HO6 UFG"}="HO6' UFG"; $resn_exch{" H1* UFM"}=" H1D UFM"; $resn_exch{" H2* UFM"}=" H2D UFM";
$resn_exch{"*HO2 UFM"}="HO2' UFM"; $resn_exch{" H3* UFM"}=" H3D UFM"; $resn_exch{" H4* UFM"}=" H4D UFM";
$resn_exch{"*HO3 UFM"}="HO3' UFM"; $resn_exch{"1H5* UFM"}="H5'1 UFM"; $resn_exch{"2H5* UFM"}="H5'2 UFM";
$resn_exch{"2HOA UFM"}="HOA2 UFM"; $resn_exch{"2HOB UFM"}="HOB2 UFM"; $resn_exch{"1H6' UFM"}="H6'1 UFM";
$resn_exch{"2H6' UFM"}="H6'2 UFM"; $resn_exch{"'HO2 UFM"}="HO2A UFM"; $resn_exch{"'HO3 UFM"}="HO3A UFM";
$resn_exch{"'HO4 UFM"}="HO4' UFM"; $resn_exch{"'HO6 UFM"}="HO6' UFM"; $resn_exch{" H1* UFP"}=" H1' UFP";
$resn_exch{"1H2* UFP"}="H2'1 UFP"; $resn_exch{"2H2* UFP"}="H2'2 UFP"; $resn_exch{" H3* UFP"}=" H3' UFP";
$resn_exch{" H4* UFP"}=" H4' UFP"; $resn_exch{"*HO3 UFP"}="HO3' UFP"; $resn_exch{"1H5* UFP"}="H5'1 UFP";
$resn_exch{"2H5* UFP"}="H5'2 UFP"; $resn_exch{"2HOP UFP"}="HOP2 UFP"; $resn_exch{"3HOP UFP"}="HOP3 UFP";
$resn_exch{"1H5* UFR"}="H5'1 UFR"; $resn_exch{"2H5* UFR"}="H5'2 UFR"; $resn_exch{" H4* UFR"}=" H4' UFR";
$resn_exch{" H3* UFR"}=" H3' UFR"; $resn_exch{"HO3* UFR"}="HO3' UFR"; $resn_exch{"1H2* UFR"}="H2'1 UFR";
$resn_exch{"2H2* UFR"}="H2'2 UFR"; $resn_exch{" H1* UFR"}=" H1' UFR"; $resn_exch{"1H1' UGA"}="H1'1 UGA";
$resn_exch{"1H2' UGA"}="H2'1 UGA"; $resn_exch{"2HO' UGA"}="HO'2 UGA"; $resn_exch{"1H3' UGA"}="H3'1 UGA";
$resn_exch{"3HO' UGA"}="HO'3 UGA"; $resn_exch{"1H4' UGA"}="H4'1 UGA"; $resn_exch{"4HO' UGA"}="HO'4 UGA";
$resn_exch{"1H5' UGA"}="H5'1 UGA"; $resn_exch{"6HO' UGA"}="HO'6 UGA"; $resn_exch{"2HOB UGA"}="HOB2 UGA";
$resn_exch{"2HOA UGA"}="HOA2 UGA"; $resn_exch{"1H5* UGA"}="H5A1 UGA"; $resn_exch{"2H5* UGA"}="H5'2 UGA";
$resn_exch{"1H4* UGA"}="H4A1 UGA"; $resn_exch{"1H3* UGA"}="H3A1 UGA"; $resn_exch{"3HO* UGA"}="HOA3 UGA";
$resn_exch{"1H2* UGA"}="H2A1 UGA"; $resn_exch{"2HO* UGA"}="HOC2 UGA"; $resn_exch{"1H1* UGA"}="H1A1 UGA";
$resn_exch{"1H6  UGA"}=" H61 UGA"; $resn_exch{"1H5  UGA"}=" H51 UGA"; $resn_exch{"1H8  UHD"}=" H81 UHD";
$resn_exch{"2H8  UHD"}=" H82 UHD"; $resn_exch{"1H9  UHD"}=" H91 UHD"; $resn_exch{"2H9  UHD"}=" H92 UHD";
$resn_exch{"1H10 UHD"}="H101 UHD"; $resn_exch{"2H10 UHD"}="H102 UHD"; $resn_exch{"1H11 UHD"}="H111 UHD";
$resn_exch{"2H11 UHD"}="H112 UHD"; $resn_exch{"1H12 UHD"}="H121 UHD"; $resn_exch{"2H12 UHD"}="H122 UHD";
$resn_exch{"1H13 UHD"}="H131 UHD"; $resn_exch{"2H13 UHD"}="H132 UHD"; $resn_exch{"1H14 UHD"}="H141 UHD";
$resn_exch{"2H14 UHD"}="H142 UHD"; $resn_exch{"1H15 UHD"}="H151 UHD"; $resn_exch{"2H15 UHD"}="H152 UHD";
$resn_exch{"1H16 UHD"}="H161 UHD"; $resn_exch{"2H16 UHD"}="H162 UHD"; $resn_exch{"1H17 UHD"}="H171 UHD";
$resn_exch{"2H17 UHD"}="H172 UHD"; $resn_exch{"1H18 UHD"}="H181 UHD"; $resn_exch{"2H18 UHD"}="H182 UHD";
$resn_exch{"3H18 UHD"}="H183 UHD"; $resn_exch{"1HN1 UI1"}="HN11 UI1"; $resn_exch{"2HN1 UI1"}="HN12 UI1";
$resn_exch{"1H2  UI1"}="H21A UI1"; $resn_exch{"2H2  UI1"}=" H22 UI1"; $resn_exch{"1H29 UI1"}="H291 UI1";
$resn_exch{"2H29 UI1"}="H292 UI1"; $resn_exch{"1H30 UI2"}="H301 UI2"; $resn_exch{"2H30 UI2"}="H302 UI2";
$resn_exch{"1H18 UI3"}="H181 UI3"; $resn_exch{"2H18 UI3"}="H182 UI3"; $resn_exch{"1H20 UI3"}="H201 UI3";
$resn_exch{"2H20 UI3"}="H202 UI3"; $resn_exch{"3H20 UI3"}="H203 UI3"; $resn_exch{"1H37 UI3"}="H371 UI3";
$resn_exch{"2H37 UI3"}="H372 UI3"; $resn_exch{"3H37 UI3"}="H373 UI3"; $resn_exch{"1HN1 UIB"}="HN11 UIB";
$resn_exch{"2HN1 UIB"}="HN12 UIB"; $resn_exch{"1H13 UIB"}="H131 UIB"; $resn_exch{"2H13 UIB"}="H132 UIB";
$resn_exch{"3H13 UIB"}="H133 UIB"; $resn_exch{"1H14 UIB"}="H141 UIB"; $resn_exch{"2H14 UIB"}="H142 UIB";
$resn_exch{"3H14 UIB"}="H143 UIB"; $resn_exch{"1H12 UIB"}="H121 UIB"; $resn_exch{"2H12 UIB"}="H122 UIB";
$resn_exch{"3H12 UIB"}="H123 UIB"; $resn_exch{"1H23 UIB"}="H231 UIB"; $resn_exch{"2H23 UIB"}="H232 UIB";
$resn_exch{"1H28 UIB"}="H281 UIB"; $resn_exch{"2H28 UIB"}="H282 UIB"; $resn_exch{"1H18 UIB"}="H181 UIB";
$resn_exch{"2H18 UIB"}="H182 UIB"; $resn_exch{"1H41 UIB"}="H411 UIB"; $resn_exch{"2H41 UIB"}="H412 UIB";
$resn_exch{"3H41 UIB"}="H413 UIB"; $resn_exch{"1H12 UIC"}="H121 UIC"; $resn_exch{"2H12 UIC"}="H122 UIC";
$resn_exch{"1H14 UIC"}="H141 UIC"; $resn_exch{"2H14 UIC"}="H142 UIC"; $resn_exch{"1H16 UIC"}="H161 UIC";
$resn_exch{"2H16 UIC"}="H162 UIC"; $resn_exch{"1H2  UIC"}=" H21 UIC"; $resn_exch{"2H2  UIC"}="H22A UIC";
$resn_exch{"1H23 UIC"}="H231 UIC"; $resn_exch{"2H23 UIC"}="H232 UIC"; $resn_exch{"1H32 UIC"}="H321 UIC";
$resn_exch{"2H32 UIC"}="H322 UIC"; $resn_exch{"1H33 UIC"}="H331 UIC"; $resn_exch{"2H33 UIC"}="H332 UIC";
$resn_exch{"1H35 UIC"}="H351 UIC"; $resn_exch{"2H35 UIC"}="H352 UIC"; $resn_exch{"3H35 UIC"}="H353 UIC";
$resn_exch{"1H36 UIC"}="H361 UIC"; $resn_exch{"2H36 UIC"}="H362 UIC"; $resn_exch{"3H36 UIC"}="H363 UIC";
$resn_exch{"1H3  UIC"}="H31A UIC"; $resn_exch{"2H3  UIC"}=" H32 UIC"; $resn_exch{"1HG2 UIN"}="HG21 UIN";
$resn_exch{"2HG2 UIN"}="HG22 UIN"; $resn_exch{"1HG3 UIN"}="HG31 UIN"; $resn_exch{"2HG3 UIN"}="HG32 UIN";
$resn_exch{"3HG3 UIN"}="HG33 UIN"; $resn_exch{"1HD1 UIN"}="HD11 UIN"; $resn_exch{"2HD1 UIN"}="HD12 UIN";
$resn_exch{"1HD2 UIN"}="HD21 UIN"; $resn_exch{"2HD2 UIN"}="HD22 UIN"; $resn_exch{"1HD3 UIN"}="HD31 UIN";
$resn_exch{"2HD3 UIN"}="HD32 UIN"; $resn_exch{"1HD4 UIN"}="HD41 UIN"; $resn_exch{"2HD4 UIN"}="HD42 UIN";
$resn_exch{"1HD5 UIN"}="HD51 UIN"; $resn_exch{"2HD5 UIN"}="HD52 UIN"; $resn_exch{"1HD6 UIN"}="HD61 UIN";
$resn_exch{"2HD6 UIN"}="HD62 UIN"; $resn_exch{"1HN2 UIP"}="HN21 UIP"; $resn_exch{"2HN2 UIP"}="HN22 UIP";
$resn_exch{"1H12 UIP"}="H121 UIP"; $resn_exch{"2H12 UIP"}="H122 UIP"; $resn_exch{"1H13 UIP"}="H131 UIP";
$resn_exch{"2H13 UIP"}="H132 UIP"; $resn_exch{"1H14 UIP"}="H141 UIP"; $resn_exch{"2H14 UIP"}="H142 UIP";
$resn_exch{"1H23 UIP"}="H231 UIP"; $resn_exch{"2H23 UIP"}="H232 UIP"; $resn_exch{"1H28 UIP"}="H281 UIP";
$resn_exch{"2H28 UIP"}="H282 UIP"; $resn_exch{"1H18 UIP"}="H181 UIP"; $resn_exch{"2H18 UIP"}="H182 UIP";
$resn_exch{"1H42 UIP"}="H421 UIP"; $resn_exch{"2H42 UIP"}="H422 UIP"; $resn_exch{"3H42 UIP"}="H423 UIP";
$resn_exch{"1HN2 UIQ"}="HN21 UIQ"; $resn_exch{"2HN2 UIQ"}="HN22 UIQ"; $resn_exch{"1H12 UIQ"}="H121 UIQ";
$resn_exch{"2H12 UIQ"}="H122 UIQ"; $resn_exch{"1H13 UIQ"}="H131 UIQ"; $resn_exch{"2H13 UIQ"}="H132 UIQ";
$resn_exch{"1H14 UIQ"}="H141 UIQ"; $resn_exch{"2H14 UIQ"}="H142 UIQ"; $resn_exch{"1H23 UIQ"}="H231 UIQ";
$resn_exch{"2H23 UIQ"}="H232 UIQ"; $resn_exch{"1H28 UIQ"}="H281 UIQ"; $resn_exch{"2H28 UIQ"}="H282 UIQ";
$resn_exch{"1H41 UIQ"}="H411 UIQ"; $resn_exch{"2H41 UIQ"}="H412 UIQ"; $resn_exch{"3H41 UIQ"}="H413 UIQ";
$resn_exch{"1H42 UIQ"}="H421 UIQ"; $resn_exch{"2H42 UIQ"}="H422 UIQ"; $resn_exch{"3H42 UIQ"}="H423 UIQ";
$resn_exch{"1HN1 UIR"}="HN11 UIR"; $resn_exch{"2HN1 UIR"}="HN12 UIR"; $resn_exch{"1H12 UIR"}="H121 UIR";
$resn_exch{"2H12 UIR"}="H122 UIR"; $resn_exch{"1H13 UIR"}="H131 UIR"; $resn_exch{"2H13 UIR"}="H132 UIR";
$resn_exch{"1H14 UIR"}="H141 UIR"; $resn_exch{"2H14 UIR"}="H142 UIR"; $resn_exch{"1H42 UIR"}="H421 UIR";
$resn_exch{"2H42 UIR"}="H422 UIR"; $resn_exch{"1H41 UIR"}="H411 UIR"; $resn_exch{"2H41 UIR"}="H412 UIR";
$resn_exch{"1H23 UIR"}="H231 UIR"; $resn_exch{"2H23 UIR"}="H232 UIR"; $resn_exch{"1H28 UIR"}="H281 UIR";
$resn_exch{"2H28 UIR"}="H282 UIR"; $resn_exch{"1HN2 UIZ"}="HN21 UIZ"; $resn_exch{"2HN2 UIZ"}="HN22 UIZ";
$resn_exch{"1H23 UIZ"}="H231 UIZ"; $resn_exch{"2H23 UIZ"}="H232 UIZ"; $resn_exch{"1H28 UIZ"}="H281 UIZ";
$resn_exch{"2H28 UIZ"}="H282 UIZ"; $resn_exch{"1H14 UIZ"}="H141 UIZ"; $resn_exch{"2H14 UIZ"}="H142 UIZ";
$resn_exch{"1H13 UIZ"}="H131 UIZ"; $resn_exch{"2H13 UIZ"}="H132 UIZ"; $resn_exch{"1H12 UIZ"}="H121 UIZ";
$resn_exch{"2H12 UIZ"}="H122 UIZ"; $resn_exch{"1H1  UIZ"}="H11A UIZ"; $resn_exch{"2H1  UIZ"}=" H12 UIZ";
$resn_exch{"1H   UKC"}=" H1  UKC"; $resn_exch{"2H   UKC"}=" H2A UKC"; $resn_exch{"3H   UKC"}=" H3A UKC";
$resn_exch{"1H6  UKC"}=" H61 UKC"; $resn_exch{"2H6  UKC"}=" H62 UKC"; $resn_exch{"3H6  UKC"}=" H63 UKC";
$resn_exch{"1H15 UKC"}="H151 UKC"; $resn_exch{"2H15 UKC"}="H152 UKC"; $resn_exch{"1H16 UKC"}="H161 UKC";
$resn_exch{"2H16 UKC"}="H162 UKC"; $resn_exch{"3H16 UKC"}="H163 UKC"; $resn_exch{"1HN  UKP"}=" HN1 UKP";
$resn_exch{"1HA  UKP"}=" HA1 UKP"; $resn_exch{"1HCB UKP"}="HCB1 UKP"; $resn_exch{"2HCB UKP"}="HCB2 UKP";
$resn_exch{"1HD1 UKP"}="HD11 UKP"; $resn_exch{"1HZ  UKP"}=" HZ1 UKP"; $resn_exch{"1HE2 UKP"}="HE21 UKP";
$resn_exch{"1HD2 UKP"}="HD21 UKP"; $resn_exch{"1HX1 UKP"}="HX11 UKP"; $resn_exch{"2HX1 UKP"}="HX12 UKP";
$resn_exch{"1HX2 UKP"}="HX21 UKP"; $resn_exch{"1H8  UKP"}=" H81 UKP"; $resn_exch{"1H10 UKP"}="H101 UKP";
$resn_exch{"1H12 UKP"}="H121 UKP"; $resn_exch{"1H13 UKP"}="H131 UKP"; $resn_exch{"2H13 UKP"}="H132 UKP";
$resn_exch{"3H13 UKP"}="H133 UKP"; $resn_exch{"1H14 UKP"}="H141 UKP"; $resn_exch{"2H14 UKP"}="H142 UKP";
$resn_exch{"3H14 UKP"}="H143 UKP"; $resn_exch{"1H15 UKP"}="H151 UKP"; $resn_exch{"1H16 UKP"}="H161 UKP";
$resn_exch{"2H16 UKP"}="H162 UKP"; $resn_exch{"3H16 UKP"}="H163 UKP"; $resn_exch{"1H17 UKP"}="H171 UKP";
$resn_exch{"2H17 UKP"}="H172 UKP"; $resn_exch{"3H17 UKP"}="H173 UKP"; $resn_exch{"1H18 UKP"}="H181 UKP";
$resn_exch{"1H19 UKP"}="H191 UKP"; $resn_exch{"2H19 UKP"}="H192 UKP"; $resn_exch{"3H19 UKP"}="H193 UKP";
$resn_exch{"1H20 UKP"}="H201 UKP"; $resn_exch{"2H20 UKP"}="H202 UKP"; $resn_exch{"3H20 UKP"}="H203 UKP";
$resn_exch{"1H1  UKP"}=" H11 UKP"; $resn_exch{"2H1  UKP"}=" H12 UKP"; $resn_exch{"1H2  UKP"}=" H21 UKP";
$resn_exch{"2H2  UKP"}=" H22 UKP"; $resn_exch{"1H4  UKP"}=" H41 UKP"; $resn_exch{"2H4  UKP"}=" H42 UKP";
$resn_exch{"1H5  UKP"}=" H51 UKP"; $resn_exch{"2H5  UKP"}=" H52 UKP"; $resn_exch{"1H22 UKP"}="H221 UKP";
$resn_exch{"2H22 UKP"}="H222 UKP"; $resn_exch{"1H23 UKP"}="H231 UKP"; $resn_exch{"2H23 UKP"}="H232 UKP";
$resn_exch{"1H24 UKP"}="H241 UKP"; $resn_exch{"2H24 UKP"}="H242 UKP"; $resn_exch{"1H6  ULA"}=" H61 ULA";
$resn_exch{"2H6  ULA"}=" H62 ULA"; $resn_exch{"1H5  ULA"}=" H51 ULA"; $resn_exch{"2H5  ULA"}=" H52 ULA";
$resn_exch{"1H4  ULA"}=" H41 ULA"; $resn_exch{"2H4  ULA"}=" H42 ULA"; $resn_exch{"1H2  ULA"}=" H21 ULA";
$resn_exch{"2H2  ULA"}="H22A ULA"; $resn_exch{"1H8  ULA"}=" H81 ULA"; $resn_exch{"2H8  ULA"}=" H82 ULA";
$resn_exch{"1H36 ULA"}="H361 ULA"; $resn_exch{"2H36 ULA"}="H362 ULA"; $resn_exch{"3H36 ULA"}="H363 ULA";
$resn_exch{"1H21 ULA"}="H211 ULA"; $resn_exch{"2H21 ULA"}="H212 ULA"; $resn_exch{"1H46 ULA"}="H461 ULA";
$resn_exch{"2H46 ULA"}="H462 ULA"; $resn_exch{"3H46 ULA"}="H463 ULA"; $resn_exch{"1H37 ULA"}="H371 ULA";
$resn_exch{"2H37 ULA"}="H372 ULA"; $resn_exch{"3H37 ULA"}="H373 ULA"; $resn_exch{"1H25 ULA"}="H251 ULA";
$resn_exch{"2H25 ULA"}="H252 ULA"; $resn_exch{"1H45 ULA"}="H451 ULA"; $resn_exch{"2H45 ULA"}="H452 ULA";
$resn_exch{"3H45 ULA"}="H453 ULA"; $resn_exch{"1H38 ULA"}="H381 ULA"; $resn_exch{"2H38 ULA"}="H382 ULA";
$resn_exch{"3H38 ULA"}="H383 ULA"; $resn_exch{"1H28 ULA"}="H281 ULA"; $resn_exch{"2H28 ULA"}="H282 ULA";
$resn_exch{"1H29 ULA"}="H291 ULA"; $resn_exch{"2H29 ULA"}="H292 ULA"; $resn_exch{"1H39 ULA"}="H391 ULA";
$resn_exch{"2H39 ULA"}="H392 ULA"; $resn_exch{"3H39 ULA"}="H393 ULA"; $resn_exch{"1H40 ULA"}="H401 ULA";
$resn_exch{"2H40 ULA"}="H402 ULA"; $resn_exch{"3H40 ULA"}="H403 ULA"; $resn_exch{"1H42 ULA"}="H421 ULA";
$resn_exch{"2H42 ULA"}="H422 ULA"; $resn_exch{"3H42 ULA"}="H423 ULA"; $resn_exch{"1H41 ULA"}="H411 ULA";
$resn_exch{"2H41 ULA"}="H412 ULA"; $resn_exch{"1HO2 ULA"}="HO21 ULA"; $resn_exch{"1H44 ULA"}="H441 ULA";
$resn_exch{"2H44 ULA"}="H442 ULA"; $resn_exch{"3H44 ULA"}="H443 ULA"; $resn_exch{" H1* UM3"}=" H1' UM3";
$resn_exch{"1H2* UM3"}="H2'1 UM3"; $resn_exch{"2H2* UM3"}="H2'2 UM3"; $resn_exch{" H3* UM3"}=" H3' UM3";
$resn_exch{" H4* UM3"}=" H4' UM3"; $resn_exch{"1H5* UM3"}="H5'1 UM3"; $resn_exch{"2H5* UM3"}="H5'2 UM3";
$resn_exch{"*HO5 UM3"}="HO5' UM3"; $resn_exch{"2HOP UM3"}="HOP2 UM3"; $resn_exch{"3HOP UM3"}="HOP3 UM3";
$resn_exch{" H1* UMA"}=" H1B UMA"; $resn_exch{" H2* UMA"}=" H2B UMA"; $resn_exch{"*HO2 UMA"}="HO2' UMA";
$resn_exch{" H3* UMA"}=" H3B UMA"; $resn_exch{" H4* UMA"}=" H4B UMA"; $resn_exch{"*HO3 UMA"}="HO3' UMA";
$resn_exch{"1H5* UMA"}="H5'1 UMA"; $resn_exch{"2H5* UMA"}="H5'2 UMA"; $resn_exch{"2HOA UMA"}="HOA2 UMA";
$resn_exch{"2HOB UMA"}="HOB2 UMA"; $resn_exch{"1H8' UMA"}="H8'1 UMA"; $resn_exch{"2H8' UMA"}="H8'2 UMA";
$resn_exch{"3H8' UMA"}="H8'3 UMA"; $resn_exch{"1H6' UMA"}="H6'1 UMA"; $resn_exch{"2H6' UMA"}="H6'2 UMA";
$resn_exch{"1H20 UMA"}="H201 UMA"; $resn_exch{"2H20 UMA"}="H202 UMA"; $resn_exch{"3H20 UMA"}="H203 UMA";
$resn_exch{"1H23 UMA"}="H231 UMA"; $resn_exch{"2H23 UMA"}="H232 UMA"; $resn_exch{"3H23 UMA"}="H233 UMA";
$resn_exch{" H5* UMF"}=" H5' UMF"; $resn_exch{"1H5* UMF"}="H5'1 UMF"; $resn_exch{"2H5* UMF"}="H5'2 UMF";
$resn_exch{" H4* UMF"}=" H4' UMF"; $resn_exch{" H3* UMF"}=" H3' UMF"; $resn_exch{" H2* UMF"}=" H2' UMF";
$resn_exch{" H1* UMF"}=" H1' UMF"; $resn_exch{"1H10 UMG"}="H101 UMG"; $resn_exch{"2H10 UMG"}="H102 UMG";
$resn_exch{"3H10 UMG"}="H103 UMG"; $resn_exch{"1H16 UMG"}="H161 UMG"; $resn_exch{"2H16 UMG"}="H162 UMG";
$resn_exch{"1H18 UMG"}="H181 UMG"; $resn_exch{"2H18 UMG"}="H182 UMG"; $resn_exch{"3H18 UMG"}="H183 UMG";
$resn_exch{"1H24 UMG"}="H241 UMG"; $resn_exch{"2H24 UMG"}="H242 UMG"; $resn_exch{"1H26 UMG"}="H261 UMG";
$resn_exch{"2H26 UMG"}="H262 UMG"; $resn_exch{"3H26 UMG"}="H263 UMG"; $resn_exch{"1H32 UMG"}="H321 UMG";
$resn_exch{"2H32 UMG"}="H322 UMG"; $resn_exch{"1H34 UMG"}="H341 UMG"; $resn_exch{"2H34 UMG"}="H342 UMG";
$resn_exch{"3H34 UMG"}="H343 UMG"; $resn_exch{" H1* UMP"}=" H1' UMP"; $resn_exch{"1H2* UMP"}=" H2' UMP";
$resn_exch{"2H2* UMP"}="H2'' UMP"; $resn_exch{" H3* UMP"}=" H3' UMP"; $resn_exch{" H4* UMP"}=" H4' UMP";
$resn_exch{"*HO3 UMP"}="HO3' UMP"; $resn_exch{"1H5* UMP"}=" H5' UMP"; $resn_exch{"2H5* UMP"}="H5'' UMP";
$resn_exch{"2HOP UMP"}="HOP2 UMP"; $resn_exch{"3HOP UMP"}="HOP3 UMP"; $resn_exch{"1H2  UMQ"}=" H21 UMQ";
$resn_exch{"1H4  UMQ"}=" H41 UMQ"; $resn_exch{"1H5  UMQ"}=" H51 UMQ"; $resn_exch{"1H6  UMQ"}=" H61 UMQ";
$resn_exch{"2H6  UMQ"}=" H62 UMQ"; $resn_exch{"1HO2 UMQ"}="HO21 UMQ"; $resn_exch{"1HO4 UMQ"}="HO41 UMQ";
$resn_exch{"1H1  UMQ"}=" H11 UMQ"; $resn_exch{"1H3  UMQ"}=" H31 UMQ"; $resn_exch{"1HO3 UMQ"}="HO31 UMQ";
$resn_exch{"1HO6 UMQ"}="HO61 UMQ"; $resn_exch{"1H1' UMQ"}="H1'1 UMQ"; $resn_exch{"1H2' UMQ"}="H2'1 UMQ";
$resn_exch{"1H3' UMQ"}="H3'1 UMQ"; $resn_exch{"1H4' UMQ"}="H4'1 UMQ"; $resn_exch{"1H5' UMQ"}="H5'1 UMQ";
$resn_exch{"1H6' UMQ"}="H6'1 UMQ"; $resn_exch{"2H6' UMQ"}="H6'2 UMQ"; $resn_exch{"'HO3 UMQ"}="HO3' UMQ";
$resn_exch{"'HO6 UMQ"}="HO6' UMQ"; $resn_exch{"'HO2 UMQ"}="HO2' UMQ"; $resn_exch{"1HA  UMQ"}=" HA1 UMQ";
$resn_exch{"2HA  UMQ"}=" HA2 UMQ"; $resn_exch{"1HB  UMQ"}=" HB1 UMQ"; $resn_exch{"2HB  UMQ"}=" HB2 UMQ";
$resn_exch{"1HC  UMQ"}=" HC1 UMQ"; $resn_exch{"2HC  UMQ"}=" HC2 UMQ"; $resn_exch{"1HD  UMQ"}=" HD1 UMQ";
$resn_exch{"2HD  UMQ"}=" HD2 UMQ"; $resn_exch{"1HF  UMQ"}=" HF1 UMQ"; $resn_exch{"2HF  UMQ"}=" HF2 UMQ";
$resn_exch{"1HG  UMQ"}=" HG1 UMQ"; $resn_exch{"2HG  UMQ"}=" HG2 UMQ"; $resn_exch{"1HH  UMQ"}=" HH1 UMQ";
$resn_exch{"2HH  UMQ"}=" HH2 UMQ"; $resn_exch{"1HI  UMQ"}=" HI1 UMQ"; $resn_exch{"2HI  UMQ"}=" HI2 UMQ";
$resn_exch{"1HJ  UMQ"}=" HJ1 UMQ"; $resn_exch{"2HJ  UMQ"}=" HJ2 UMQ"; $resn_exch{"1HK  UMQ"}=" HK1 UMQ";
$resn_exch{"2HK  UMQ"}=" HK2 UMQ"; $resn_exch{"1HL  UMQ"}=" HL1 UMQ"; $resn_exch{"2HL  UMQ"}=" HL2 UMQ";
$resn_exch{"3HL  UMQ"}=" HL3 UMQ"; $resn_exch{"3HOP UMS"}="HOP3 UMS"; $resn_exch{"2HOP UMS"}="HOP2 UMS";
$resn_exch{"2H5' UMS"}="H5'2 UMS"; $resn_exch{"H3T  UMS"}="HO3' UMS"; $resn_exch{"2HA' UMS"}="HA'2 UMS";
$resn_exch{"3HA' UMS"}="HA'3 UMS"; $resn_exch{"1H1C UN1"}="H1C1 UN1"; $resn_exch{"2H1C UN1"}="H1C2 UN1";
$resn_exch{"1H5C UN1"}="H5C1 UN1"; $resn_exch{"2H5C UN1"}="H5C2 UN1"; $resn_exch{"1H6C UN1"}="H6C1 UN1";
$resn_exch{"2H6C UN1"}="H6C2 UN1"; $resn_exch{"1H4N UN1"}="H4N1 UN1"; $resn_exch{"2H4N UN1"}="H4N2 UN1";
$resn_exch{"1HN  UN2"}=" HN1 UN2"; $resn_exch{"2HN  UN2"}=" HN2 UN2"; $resn_exch{"1HBC UN2"}="HBC1 UN2";
$resn_exch{"2HBC UN2"}="HBC2 UN2"; $resn_exch{"1HGC UN2"}="HGC1 UN2"; $resn_exch{"1H4  UN3"}=" H41 UN3";
$resn_exch{"2H4  UN3"}=" H42 UN3"; $resn_exch{"3H4  UN3"}=" H43 UN3"; $resn_exch{" H4* UN4"}=" H4' UN4";
$resn_exch{"1H3* UN4"}="H3'1 UN4"; $resn_exch{"2H3* UN4"}="H3'2 UN4"; $resn_exch{"1H2* UN4"}="H2'1 UN4";
$resn_exch{"2H2* UN4"}="H2'2 UN4"; $resn_exch{"1H2N UN4"}="H2N1 UN4"; $resn_exch{"2H2N UN4"}="H2N2 UN4";
$resn_exch{"1H60 UN4"}="H601 UN4"; $resn_exch{"2H60 UN4"}="H602 UN4"; $resn_exch{"1H20 UN5"}="H201 UN5";
$resn_exch{"2H20 UN5"}="H202 UN5"; $resn_exch{"3H20 UN5"}="H203 UN5"; $resn_exch{"1H11 UN5"}="H111 UN5";
$resn_exch{"2H11 UN5"}="H112 UN5"; $resn_exch{"1H7  UN5"}=" H71 UN5"; $resn_exch{"2H7  UN5"}=" H72 UN5";
$resn_exch{"1H12 UN5"}="H121 UN5"; $resn_exch{"2H12 UN5"}="H122 UN5"; $resn_exch{"1H13 UN5"}="H131 UN5";
$resn_exch{"2H13 UN5"}="H132 UN5"; $resn_exch{"1H7  UN6"}=" H71 UN6"; $resn_exch{"2H7  UN6"}=" H72 UN6";
$resn_exch{"1H9  UN6"}=" H91 UN6"; $resn_exch{"2H9  UN6"}=" H92 UN6"; $resn_exch{"1H10 UN6"}="H101 UN6";
$resn_exch{"2H10 UN6"}="H102 UN6"; $resn_exch{"1H7  UN7"}=" H71 UN7"; $resn_exch{"2H7  UN7"}=" H72 UN7";
$resn_exch{"1H19 UN7"}="H191 UN7"; $resn_exch{"2H19 UN7"}="H192 UN7"; $resn_exch{"1H28 UN7"}="H281 UN7";
$resn_exch{"2H28 UN7"}="H282 UN7"; $resn_exch{"3H28 UN7"}="H283 UN7"; $resn_exch{"1H22 UN7"}="H221 UN7";
$resn_exch{"2H22 UN7"}="H222 UN7"; $resn_exch{"1H25 UN7"}="H251 UN7"; $resn_exch{"2H25 UN7"}="H252 UN7";
$resn_exch{"1H4  UN8"}=" H41 UN8"; $resn_exch{"2H4  UN8"}=" H42 UN8"; $resn_exch{"3H4  UN8"}=" H43 UN8";
$resn_exch{"1H16 UN8"}="H161 UN8"; $resn_exch{"2H16 UN8"}="H162 UN8"; $resn_exch{"1H59 UN8"}="H591 UN8";
$resn_exch{"2H59 UN8"}="H592 UN8"; $resn_exch{"1H10 UN8"}="H101 UN8"; $resn_exch{"2H10 UN8"}="H102 UN8";
$resn_exch{"1H11 UN8"}="H111 UN8"; $resn_exch{"2H11 UN8"}="H112 UN8"; $resn_exch{"1H12 UN8"}="H121 UN8";
$resn_exch{"2H12 UN8"}="H122 UN8"; $resn_exch{"3H12 UN8"}="H123 UN8"; $resn_exch{"1H17 UN8"}="H171 UN8";
$resn_exch{"2H17 UN8"}="H172 UN8"; $resn_exch{"1H15 UN9"}="H151 UN9"; $resn_exch{"2H15 UN9"}="H152 UN9";
$resn_exch{"3H11 UNA"}="H113 UNA"; $resn_exch{"2H11 UNA"}="H112 UNA"; $resn_exch{"1H11 UNA"}="H111 UNA";
$resn_exch{"2H10 UNA"}="H102 UNA"; $resn_exch{"1H10 UNA"}="H101 UNA"; $resn_exch{"2H9  UNA"}=" H92 UNA";
$resn_exch{"1H9  UNA"}=" H91 UNA"; $resn_exch{"2H8  UNA"}=" H82 UNA"; $resn_exch{"1H8  UNA"}=" H81 UNA";
$resn_exch{"2H7  UNA"}=" H72 UNA"; $resn_exch{"1H7  UNA"}=" H71 UNA"; $resn_exch{"2H6  UNA"}=" H62 UNA";
$resn_exch{"1H6  UNA"}=" H61 UNA"; $resn_exch{"2H5  UNA"}=" H52 UNA"; $resn_exch{"1H5  UNA"}=" H51 UNA";
$resn_exch{"2H4  UNA"}=" H42 UNA"; $resn_exch{"1H4  UNA"}=" H41 UNA"; $resn_exch{"2H3  UNA"}=" H32 UNA";
$resn_exch{"1H3  UNA"}=" H31 UNA"; $resn_exch{"2H2  UNA"}=" H22 UNA"; $resn_exch{"1H2  UNA"}=" H21 UNA";
$resn_exch{"1H22 UNB"}="H221 UNB"; $resn_exch{"2H22 UNB"}="H222 UNB"; $resn_exch{"1H23 UNB"}="H231 UNB";
$resn_exch{"2H23 UNB"}="H232 UNB"; $resn_exch{"3H23 UNB"}="H233 UNB"; $resn_exch{"1H27 UNB"}="H271 UNB";
$resn_exch{"2H27 UNB"}="H272 UNB"; $resn_exch{"1H35 UNB"}="H351 UNB"; $resn_exch{"2H35 UNB"}="H352 UNB";
$resn_exch{"1HN5 UNC"}="HN51 UNC"; $resn_exch{"2HN5 UNC"}="HN52 UNC"; $resn_exch{"1H1  UND"}=" H11 UND";
$resn_exch{"2H1  UND"}=" H12 UND"; $resn_exch{"3H1  UND"}=" H13 UND"; $resn_exch{"1H2  UND"}=" H21 UND";
$resn_exch{"2H2  UND"}=" H22 UND"; $resn_exch{"1H3  UND"}=" H31 UND"; $resn_exch{"2H3  UND"}=" H32 UND";
$resn_exch{"1H4  UND"}=" H41 UND"; $resn_exch{"2H4  UND"}=" H42 UND"; $resn_exch{"1H5  UND"}=" H51 UND";
$resn_exch{"2H5  UND"}=" H52 UND"; $resn_exch{"1H6  UND"}=" H61 UND"; $resn_exch{"2H6  UND"}=" H62 UND";
$resn_exch{"1H7  UND"}=" H71 UND"; $resn_exch{"2H7  UND"}=" H72 UND"; $resn_exch{"1H8  UND"}=" H81 UND";
$resn_exch{"2H8  UND"}=" H82 UND"; $resn_exch{"1H9  UND"}=" H91 UND"; $resn_exch{"2H9  UND"}=" H92 UND";
$resn_exch{"1H10 UND"}="H101 UND"; $resn_exch{"2H10 UND"}="H102 UND"; $resn_exch{"1H11 UND"}="H111 UND";
$resn_exch{"2H11 UND"}="H112 UND"; $resn_exch{"3H11 UND"}="H113 UND"; $resn_exch{"1H4C UNG"}="H4C1 UNG";
$resn_exch{"1H4  UNH"}=" H41 UNH"; $resn_exch{"2H4  UNH"}="H42A UNH"; $resn_exch{"1H6  UNH"}=" H61 UNH";
$resn_exch{"2H6  UNH"}=" H62 UNH"; $resn_exch{"3H6  UNH"}=" H63 UNH"; $resn_exch{"1H7  UNH"}=" H71 UNH";
$resn_exch{"2H7  UNH"}=" H72 UNH"; $resn_exch{"3H7  UNH"}=" H73 UNH"; $resn_exch{"1H11 UNH"}="H111 UNH";
$resn_exch{"2H11 UNH"}="H112 UNH"; $resn_exch{"1H12 UNH"}="H121 UNH"; $resn_exch{"2H12 UNH"}="H122 UNH";
$resn_exch{"1H13 UNH"}="H131 UNH"; $resn_exch{"2H13 UNH"}="H132 UNH"; $resn_exch{"1H14 UNH"}="H141 UNH";
$resn_exch{"2H14 UNH"}="H142 UNH"; $resn_exch{"1H15 UNH"}="H151 UNH"; $resn_exch{"2H15 UNH"}="H152 UNH";
$resn_exch{"1H20 UNH"}="H201 UNH"; $resn_exch{"2H20 UNH"}="H202 UNH"; $resn_exch{"1H22 UNH"}="H221 UNH";
$resn_exch{"2H22 UNH"}="H222 UNH"; $resn_exch{"3H22 UNH"}="H223 UNH"; $resn_exch{"1H23 UNH"}="H231 UNH";
$resn_exch{"2H23 UNH"}="H232 UNH"; $resn_exch{"3H23 UNH"}="H233 UNH"; $resn_exch{"1H28 UNH"}="H281 UNH";
$resn_exch{"2H28 UNH"}="H282 UNH"; $resn_exch{"1H29 UNH"}="H291 UNH"; $resn_exch{"2H29 UNH"}="H292 UNH";
$resn_exch{"1H30 UNH"}="H301 UNH"; $resn_exch{"2H30 UNH"}="H302 UNH"; $resn_exch{"3H30 UNH"}="H303 UNH";
$resn_exch{"1H35 UNH"}="H351 UNH"; $resn_exch{"2H35 UNH"}="H352 UNH"; $resn_exch{"1H49 UNH"}="H491 UNH";
$resn_exch{"2H49 UNH"}="H492 UNH"; $resn_exch{"1HG3 UNI"}="HG31 UNI"; $resn_exch{"2HG3 UNI"}="HG32 UNI";
$resn_exch{"1HG4 UNI"}="HG41 UNI"; $resn_exch{"2HG4 UNI"}="HG42 UNI"; $resn_exch{"1HD1 UNI"}="HD11 UNI";
$resn_exch{"2HD1 UNI"}="HD12 UNI"; $resn_exch{"1HD2 UNI"}="HD21 UNI"; $resn_exch{"2HD2 UNI"}="HD22 UNI";
$resn_exch{"1HD3 UNI"}="HD31 UNI"; $resn_exch{"2HD3 UNI"}="HD32 UNI"; $resn_exch{"1HD4 UNI"}="HD41 UNI";
$resn_exch{"2HD4 UNI"}="HD42 UNI"; $resn_exch{"1HD5 UNI"}="HD51 UNI"; $resn_exch{"2HD5 UNI"}="HD52 UNI";
$resn_exch{"1HD6 UNI"}="HD61 UNI"; $resn_exch{"2HD6 UNI"}="HD62 UNI"; $resn_exch{" HN  UNK"}=" H   UNK";
$resn_exch{"2HN  UNK"}=" H2  UNK"; $resn_exch{"1HB  UNK"}=" HB1 UNK"; $resn_exch{"2HB  UNK"}=" HB2 UNK";
$resn_exch{"1HG  UNK"}=" HG1 UNK"; $resn_exch{"2HG  UNK"}=" HG2 UNK"; $resn_exch{"3HG  UNK"}=" HG3 UNK";
$resn_exch{"1H31 UNN"}="H311 UNN"; $resn_exch{"2H31 UNN"}="H312 UNN"; $resn_exch{"3H31 UNN"}="H313 UNN";
$resn_exch{" H1* UP1"}=" H1' UP1"; $resn_exch{" H2* UP1"}=" H2' UP1"; $resn_exch{" H3* UP1"}=" H3' UP1";
$resn_exch{" H4* UP1"}=" H4' UP1"; $resn_exch{"1H5* UP1"}="H5'1 UP1"; $resn_exch{"2H5* UP1"}="H5'2 UP1";
$resn_exch{"1H6  UP1"}=" H61 UP1"; $resn_exch{"2H6  UP1"}=" H62 UP1"; $resn_exch{" AH8 UP5"}=" H8A UP5";
$resn_exch{"AH61 UP5"}="H61A UP5"; $resn_exch{"AH62 UP5"}="H62A UP5"; $resn_exch{" AH2 UP5"}=" H2A UP5";
$resn_exch{"AH51 UP5"}="H51A UP5"; $resn_exch{"AH52 UP5"}="H52A UP5"; $resn_exch{"AH4* UP5"}=" H4F UP5";
$resn_exch{"AH3* UP5"}=" H3F UP5"; $resn_exch{"AHO3 UP5"}="HO3A UP5"; $resn_exch{"AH2* UP5"}=" H2F UP5";
$resn_exch{"AHO2 UP5"}="HO2A UP5"; $resn_exch{"AH1* UP5"}=" H1F UP5"; $resn_exch{"2HOA UP5"}="HOA2 UP5";
$resn_exch{"2HOB UP5"}="HOB2 UP5"; $resn_exch{"2HOG UP5"}="HOG2 UP5"; $resn_exch{"2HOD UP5"}="HOD2 UP5";
$resn_exch{"2HOE UP5"}="HOE2 UP5"; $resn_exch{"UH51 UP5"}="H51U UP5"; $resn_exch{"UH52 UP5"}="H52U UP5";
$resn_exch{"UH4* UP5"}=" H4J UP5"; $resn_exch{"UH3* UP5"}=" H3J UP5"; $resn_exch{"*HO3 UP5"}="HO3' UP5";
$resn_exch{"UH2* UP5"}=" H2J UP5"; $resn_exch{"*HO2 UP5"}="HO2' UP5"; $resn_exch{"UH1* UP5"}=" H1J UP5";
$resn_exch{" UH3 UP5"}=" H3U UP5"; $resn_exch{" UH5 UP5"}=" H5U UP5"; $resn_exch{" UH6 UP5"}=" H6U UP5";
$resn_exch{" H1* UP6"}=" H1' UP6"; $resn_exch{" H2* UP6"}=" H2' UP6"; $resn_exch{" H3* UP6"}=" H3' UP6";
$resn_exch{" H4* UP6"}=" H4' UP6"; $resn_exch{"*HO2 UP6"}="HO2' UP6"; $resn_exch{"*HO3 UP6"}="HO3' UP6";
$resn_exch{"1H5* UP6"}="H5'1 UP6"; $resn_exch{"2H5* UP6"}="H5'2 UP6"; $resn_exch{"2HOP UP6"}="HOP2 UP6";
$resn_exch{"3HOP UP6"}="HOP3 UP6"; $resn_exch{"1H5U UPA"}="H5U1 UPA"; $resn_exch{"2H5U UPA"}="H5U2 UPA";
$resn_exch{"5HOU UPA"}="HOU5 UPA"; $resn_exch{"*H4U UPA"}="H4U' UPA"; $resn_exch{"*H3U UPA"}="H3U' UPA";
$resn_exch{"3HOU UPA"}="HOU3 UPA"; $resn_exch{"*H2U UPA"}="H2U' UPA"; $resn_exch{"*H1U UPA"}="H1U' UPA";
$resn_exch{"2HOP UPA"}="HOP2 UPA"; $resn_exch{"1H5A UPA"}="H5A1 UPA"; $resn_exch{"2H5A UPA"}="H5A2 UPA";
$resn_exch{"*H4A UPA"}="H4A' UPA"; $resn_exch{"*H3A UPA"}="H3A' UPA"; $resn_exch{"3HOA UPA"}="HOA3 UPA";
$resn_exch{"*H2A UPA"}="H2A' UPA"; $resn_exch{"2HOA UPA"}="HOA2 UPA"; $resn_exch{"*H1A UPA"}="H1A' UPA";
$resn_exch{"1H6A UPA"}="H6A1 UPA"; $resn_exch{"2H6A UPA"}="H6A2 UPA"; $resn_exch{" H1* UPF"}=" H1D UPF";
$resn_exch{" H2* UPF"}=" H2D UPF"; $resn_exch{"*HO2 UPF"}="HO2' UPF"; $resn_exch{" H3* UPF"}=" H3D UPF";
$resn_exch{" H4* UPF"}=" H4D UPF"; $resn_exch{"*HO3 UPF"}="HO3' UPF"; $resn_exch{"1H5* UPF"}="H5'1 UPF";
$resn_exch{"2H5* UPF"}="H5'2 UPF"; $resn_exch{"2HOA UPF"}="HOA2 UPF"; $resn_exch{"2HOB UPF"}="HOB2 UPF";
$resn_exch{"1H6' UPF"}="H6'1 UPF"; $resn_exch{"2H6' UPF"}="H6'2 UPF"; $resn_exch{"'HO3 UPF"}="HO3A UPF";
$resn_exch{"'HO4 UPF"}="HO4' UPF"; $resn_exch{"'HO6 UPF"}="HO6' UPF"; $resn_exch{" H1* UPG"}=" H1C UPG";
$resn_exch{" H2* UPG"}=" H2C UPG"; $resn_exch{"*HO2 UPG"}="HO2C UPG"; $resn_exch{" H3* UPG"}=" H3C UPG";
$resn_exch{" H4* UPG"}=" H4C UPG"; $resn_exch{"*HO3 UPG"}="HO3C UPG"; $resn_exch{"1H5* UPG"}="H5C1 UPG";
$resn_exch{"2H5* UPG"}="H5C2 UPG"; $resn_exch{"2HOA UPG"}="HOA2 UPG"; $resn_exch{"2HOB UPG"}="HOB2 UPG";
$resn_exch{"1H6' UPG"}="H6'1 UPG"; $resn_exch{"2H6' UPG"}="H6'2 UPG"; $resn_exch{"'HO2 UPG"}="HO2' UPG";
$resn_exch{"'HO3 UPG"}="HO3' UPG"; $resn_exch{"'HO4 UPG"}="HO4' UPG"; $resn_exch{"'HO6 UPG"}="HO6' UPG";
$resn_exch{"1H1  UPL"}=" H11 UPL"; $resn_exch{"2H1  UPL"}=" H12 UPL"; $resn_exch{"3H1  UPL"}=" H13 UPL";
$resn_exch{"1H2  UPL"}=" H21 UPL"; $resn_exch{"2H2  UPL"}=" H22 UPL"; $resn_exch{"1H3  UPL"}=" H31 UPL";
$resn_exch{"2H3  UPL"}=" H32 UPL"; $resn_exch{"1H4  UPL"}=" H41 UPL"; $resn_exch{"2H4  UPL"}=" H42 UPL";
$resn_exch{"1H5  UPL"}=" H51 UPL"; $resn_exch{"2H5  UPL"}=" H52 UPL"; $resn_exch{"1H6  UPL"}=" H61 UPL";
$resn_exch{"2H6  UPL"}=" H62 UPL"; $resn_exch{"1H7  UPL"}=" H71 UPL"; $resn_exch{"2H7  UPL"}=" H72 UPL";
$resn_exch{"1H8  UPL"}=" H81 UPL"; $resn_exch{"2H8  UPL"}=" H82 UPL"; $resn_exch{"1H9  UPL"}=" H91 UPL";
$resn_exch{"2H9  UPL"}=" H92 UPL"; $resn_exch{"1H10 UPL"}="H101 UPL"; $resn_exch{"2H10 UPL"}="H102 UPL";
$resn_exch{"1H11 UPL"}="H111 UPL"; $resn_exch{"2H11 UPL"}="H112 UPL"; $resn_exch{"1H12 UPL"}="H121 UPL";
$resn_exch{"1H13 UPL"}="H131 UPL"; $resn_exch{"2H13 UPL"}="H132 UPL"; $resn_exch{"1H14 UPL"}="H141 UPL";
$resn_exch{"2H14 UPL"}="H142 UPL"; $resn_exch{"1H15 UPL"}="H151 UPL"; $resn_exch{"2H15 UPL"}="H152 UPL";
$resn_exch{"1H16 UPL"}="H161 UPL"; $resn_exch{"2H16 UPL"}="H162 UPL"; $resn_exch{"1H17 UPL"}="H171 UPL";
$resn_exch{"2H17 UPL"}="H172 UPL"; $resn_exch{"1H18 UPL"}="H181 UPL"; $resn_exch{"2H18 UPL"}="H182 UPL";
$resn_exch{"1H19 UPL"}="H191 UPL"; $resn_exch{"2H19 UPL"}="H192 UPL"; $resn_exch{"1H20 UPL"}="H201 UPL";
$resn_exch{"2H20 UPL"}="H202 UPL"; $resn_exch{"1H21 UPL"}="H211 UPL"; $resn_exch{"2H21 UPL"}="H212 UPL";
$resn_exch{"1H22 UPL"}="H221 UPL"; $resn_exch{"2H22 UPL"}="H222 UPL"; $resn_exch{"1H23 UPL"}="H231 UPL";
$resn_exch{"2H23 UPL"}="H232 UPL"; $resn_exch{"1H24 UPL"}="H241 UPL"; $resn_exch{"2H24 UPL"}="H242 UPL";
$resn_exch{"1H25 UPL"}="H251 UPL"; $resn_exch{"2H25 UPL"}="H252 UPL"; $resn_exch{"3H25 UPL"}="H253 UPL";
$resn_exch{"1H26 UPL"}="H261 UPL"; $resn_exch{"2H26 UPL"}="H262 UPL"; $resn_exch{"1H27 UPL"}="H271 UPL";
$resn_exch{"2H27 UPL"}="H272 UPL"; $resn_exch{"1H28 UPL"}="H281 UPL"; $resn_exch{"2H28 UPL"}="H282 UPL";
$resn_exch{"1H29 UPL"}="H291 UPL"; $resn_exch{"2H29 UPL"}="H292 UPL"; $resn_exch{"1H30 UPL"}="H301 UPL";
$resn_exch{"2H30 UPL"}="H302 UPL"; $resn_exch{"1H31 UPL"}="H311 UPL"; $resn_exch{"2H31 UPL"}="H312 UPL";
$resn_exch{"1H32 UPL"}="H321 UPL"; $resn_exch{"2H32 UPL"}="H322 UPL"; $resn_exch{"1H33 UPL"}="H331 UPL";
$resn_exch{"2H33 UPL"}="H332 UPL"; $resn_exch{"1H34 UPL"}="H341 UPL"; $resn_exch{"2H34 UPL"}="H342 UPL";
$resn_exch{"3H34 UPL"}="H343 UPL"; $resn_exch{"2HOA UPM"}="HOA2 UPM"; $resn_exch{"2HOB UPM"}="HOB2 UPM";
$resn_exch{" H1* UPM"}=" H1' UPM"; $resn_exch{" H2* UPM"}=" H2' UPM"; $resn_exch{"2HO* UPM"}="HO'2 UPM";
$resn_exch{" H3* UPM"}=" H3' UPM"; $resn_exch{"3HO* UPM"}="HO'3 UPM"; $resn_exch{" H4* UPM"}=" H4' UPM";
$resn_exch{"1H5* UPM"}="H5'1 UPM"; $resn_exch{"2H5* UPM"}="H5'2 UPM"; $resn_exch{"1H20 UPM"}="H201 UPM";
$resn_exch{"2H20 UPM"}="H202 UPM"; $resn_exch{"3H20 UPM"}="H203 UPM"; $resn_exch{"2HND UPM"}="HND2 UPM";
$resn_exch{"1H8D UPM"}="H8D1 UPM"; $resn_exch{"2H8D UPM"}="H8D2 UPM"; $resn_exch{"4HOD UPM"}="HOD4 UPM";
$resn_exch{"1H6D UPM"}="H6D1 UPM"; $resn_exch{"2H6D UPM"}="H6D2 UPM"; $resn_exch{"6HOD UPM"}="HOD6 UPM";
$resn_exch{" H1* UPP"}=" H1' UPP"; $resn_exch{" H2* UPP"}=" H2B UPP"; $resn_exch{"*HO2 UPP"}="HO2' UPP";
$resn_exch{" H3* UPP"}=" H3B UPP"; $resn_exch{" H4* UPP"}=" H4B UPP"; $resn_exch{"*HO3 UPP"}="HO3' UPP";
$resn_exch{"1H5* UPP"}="H5'1 UPP"; $resn_exch{"2H5* UPP"}="H5'2 UPP"; $resn_exch{"2HOA UPP"}="HOA2 UPP";
$resn_exch{"2HOB UPP"}="HOB2 UPP"; $resn_exch{"1H5* UPR"}="H5'1 UPR"; $resn_exch{"2H5* UPR"}="H5'2 UPR";
$resn_exch{" H4* UPR"}=" H4' UPR"; $resn_exch{" H3* UPR"}=" H3' UPR"; $resn_exch{"1H2* UPR"}="H2'1 UPR";
$resn_exch{"2H2* UPR"}="H2'2 UPR"; $resn_exch{" H1* UPR"}=" H1' UPR"; $resn_exch{"H1*A UPU"}="H1'A UPU";
$resn_exch{"CH2A UPU"}="H2AC UPU"; $resn_exch{"OH2A UPU"}="H2AO UPU"; $resn_exch{"CH3A UPU"}="H3AC UPU";
$resn_exch{"OH3A UPU"}="H3AO UPU"; $resn_exch{"H4*A UPU"}="H4'A UPU"; $resn_exch{"1H*A UPU"}="H'A1 UPU";
$resn_exch{"2H*A UPU"}="H'A2 UPU"; $resn_exch{"CH3B UPU"}="H3BC UPU"; $resn_exch{"H4*B UPU"}="H4'B UPU";
$resn_exch{"1H*B UPU"}="H'B1 UPU"; $resn_exch{"2H*B UPU"}="H'B2 UPU"; $resn_exch{"H5*B UPU"}="H5'B UPU";
$resn_exch{"CH2B UPU"}="H2BC UPU"; $resn_exch{"OH2B UPU"}="H2BO UPU"; $resn_exch{"H1*B UPU"}="H1'B UPU";
$resn_exch{"1HM5  UQ"}="HM51  UQ"; $resn_exch{"2HM5  UQ"}="HM52  UQ"; $resn_exch{"3HM5  UQ"}="HM53  UQ";
$resn_exch{"1HM3  UQ"}="HM31  UQ"; $resn_exch{"2HM3  UQ"}="HM32  UQ"; $resn_exch{"3HM3  UQ"}="HM33  UQ";
$resn_exch{"1HM2  UQ"}="HM21  UQ"; $resn_exch{"2HM2  UQ"}="HM22  UQ"; $resn_exch{"3HM2  UQ"}="HM23  UQ";
$resn_exch{"1H7   UQ"}=" H71  UQ"; $resn_exch{"2H7   UQ"}=" H72  UQ"; $resn_exch{"1H10  UQ"}="H101  UQ";
$resn_exch{"2H10  UQ"}="H102  UQ"; $resn_exch{"3H10  UQ"}="H103  UQ"; $resn_exch{"1H11  UQ"}="H111  UQ";
$resn_exch{"2H11  UQ"}="H112  UQ"; $resn_exch{"1H12  UQ"}="H121  UQ"; $resn_exch{"2H12  UQ"}="H122  UQ";
$resn_exch{"1H15  UQ"}="H151  UQ"; $resn_exch{"2H15  UQ"}="H152  UQ"; $resn_exch{"3H15  UQ"}="H153  UQ";
$resn_exch{"1H16  UQ"}="H161  UQ"; $resn_exch{"2H16  UQ"}="H162  UQ"; $resn_exch{"1H17  UQ"}="H171  UQ";
$resn_exch{"2H17  UQ"}="H172  UQ"; $resn_exch{"1H20  UQ"}="H201  UQ"; $resn_exch{"2H20  UQ"}="H202  UQ";
$resn_exch{"3H20  UQ"}="H203  UQ"; $resn_exch{"1H21  UQ"}="H211  UQ"; $resn_exch{"2H21  UQ"}="H212  UQ";
$resn_exch{"1H22  UQ"}="H221  UQ"; $resn_exch{"2H22  UQ"}="H222  UQ"; $resn_exch{"1H25  UQ"}="H251  UQ";
$resn_exch{"2H25  UQ"}="H252  UQ"; $resn_exch{"3H25  UQ"}="H253  UQ"; $resn_exch{"1H26  UQ"}="H261  UQ";
$resn_exch{"2H26  UQ"}="H262  UQ"; $resn_exch{"1H27  UQ"}="H271  UQ"; $resn_exch{"2H27  UQ"}="H272  UQ";
$resn_exch{"1H30  UQ"}="H301  UQ"; $resn_exch{"2H30  UQ"}="H302  UQ"; $resn_exch{"3H30  UQ"}="H303  UQ";
$resn_exch{"1H31  UQ"}="H311  UQ"; $resn_exch{"2H31  UQ"}="H312  UQ"; $resn_exch{"1H32  UQ"}="H321  UQ";
$resn_exch{"2H32  UQ"}="H322  UQ"; $resn_exch{"1H35  UQ"}="H351  UQ"; $resn_exch{"2H35  UQ"}="H352  UQ";
$resn_exch{"3H35  UQ"}="H353  UQ"; $resn_exch{"1H36  UQ"}="H361  UQ"; $resn_exch{"2H36  UQ"}="H362  UQ";
$resn_exch{"1H37  UQ"}="H371  UQ"; $resn_exch{"2H37  UQ"}="H372  UQ"; $resn_exch{"1H40  UQ"}="H401  UQ";
$resn_exch{"2H40  UQ"}="H402  UQ"; $resn_exch{"3H40  UQ"}="H403  UQ"; $resn_exch{"1H41  UQ"}="H411  UQ";
$resn_exch{"2H41  UQ"}="H412  UQ"; $resn_exch{"1H42  UQ"}="H421  UQ"; $resn_exch{"2H42  UQ"}="H422  UQ";
$resn_exch{"1H45  UQ"}="H451  UQ"; $resn_exch{"2H45  UQ"}="H452  UQ"; $resn_exch{"3H45  UQ"}="H453  UQ";
$resn_exch{"1H46  UQ"}="H461  UQ"; $resn_exch{"2H46  UQ"}="H462  UQ"; $resn_exch{"1H47  UQ"}="H471  UQ";
$resn_exch{"2H47  UQ"}="H472  UQ"; $resn_exch{"1H50  UQ"}="H501  UQ"; $resn_exch{"2H50  UQ"}="H502  UQ";
$resn_exch{"3H50  UQ"}="H503  UQ"; $resn_exch{"1H51  UQ"}="H511  UQ"; $resn_exch{"2H51  UQ"}="H512  UQ";
$resn_exch{"1H52  UQ"}="H521  UQ"; $resn_exch{"2H52  UQ"}="H522  UQ"; $resn_exch{"1H55  UQ"}="H551  UQ";
$resn_exch{"2H55  UQ"}="H552  UQ"; $resn_exch{"3H55  UQ"}="H553  UQ"; $resn_exch{"1H56  UQ"}="H561  UQ";
$resn_exch{"2H56  UQ"}="H562  UQ"; $resn_exch{"3H56  UQ"}="H563  UQ"; $resn_exch{"1HM2 UQ1"}="HM21 UQ1";
$resn_exch{"2HM2 UQ1"}="HM22 UQ1"; $resn_exch{"3HM2 UQ1"}="HM23 UQ1"; $resn_exch{"1HM3 UQ1"}="HM31 UQ1";
$resn_exch{"2HM3 UQ1"}="HM32 UQ1"; $resn_exch{"3HM3 UQ1"}="HM33 UQ1"; $resn_exch{"1HM5 UQ1"}="HM51 UQ1";
$resn_exch{"2HM5 UQ1"}="HM52 UQ1"; $resn_exch{"3HM5 UQ1"}="HM53 UQ1"; $resn_exch{"1H7  UQ1"}=" H71 UQ1";
$resn_exch{"2H7  UQ1"}=" H72 UQ1"; $resn_exch{"1H10 UQ1"}="H101 UQ1"; $resn_exch{"2H10 UQ1"}="H102 UQ1";
$resn_exch{"3H10 UQ1"}="H103 UQ1"; $resn_exch{"1H11 UQ1"}="H111 UQ1"; $resn_exch{"2H11 UQ1"}="H112 UQ1";
$resn_exch{"3H11 UQ1"}="H113 UQ1"; $resn_exch{"1H2M UQ2"}="H2M1 UQ2"; $resn_exch{"2H2M UQ2"}="H2M2 UQ2";
$resn_exch{"3H2M UQ2"}="H2M3 UQ2"; $resn_exch{"1H3M UQ2"}="H3M1 UQ2"; $resn_exch{"2H3M UQ2"}="H3M2 UQ2";
$resn_exch{"3H3M UQ2"}="H3M3 UQ2"; $resn_exch{"1H5M UQ2"}="H5M1 UQ2"; $resn_exch{"2H5M UQ2"}="H5M2 UQ2";
$resn_exch{"3H5M UQ2"}="H5M3 UQ2"; $resn_exch{"1H7  UQ2"}=" H71 UQ2"; $resn_exch{"2H7  UQ2"}=" H72 UQ2";
$resn_exch{"1H10 UQ2"}="H101 UQ2"; $resn_exch{"2H10 UQ2"}="H102 UQ2"; $resn_exch{"3H10 UQ2"}="H103 UQ2";
$resn_exch{"1H11 UQ2"}="H111 UQ2"; $resn_exch{"2H11 UQ2"}="H112 UQ2"; $resn_exch{"1H12 UQ2"}="H121 UQ2";
$resn_exch{"2H12 UQ2"}="H122 UQ2"; $resn_exch{"1H15 UQ2"}="H151 UQ2"; $resn_exch{"2H15 UQ2"}="H152 UQ2";
$resn_exch{"3H15 UQ2"}="H153 UQ2"; $resn_exch{"1H16 UQ2"}="H161 UQ2"; $resn_exch{"2H16 UQ2"}="H162 UQ2";
$resn_exch{"3H16 UQ2"}="H163 UQ2"; $resn_exch{"1H1M UQ5"}="H1M1 UQ5"; $resn_exch{"2H1M UQ5"}="H1M2 UQ5";
$resn_exch{"3H1M UQ5"}="H1M3 UQ5"; $resn_exch{"1H3M UQ5"}="H3M1 UQ5"; $resn_exch{"2H3M UQ5"}="H3M2 UQ5";
$resn_exch{"3H3M UQ5"}="H3M3 UQ5"; $resn_exch{"1H4M UQ5"}="H4M1 UQ5"; $resn_exch{"2H4M UQ5"}="H4M2 UQ5";
$resn_exch{"3H4M UQ5"}="H4M3 UQ5"; $resn_exch{"1H7  UQ5"}=" H71 UQ5"; $resn_exch{"2H7  UQ5"}=" H72 UQ5";
$resn_exch{"1H10 UQ5"}="H101 UQ5"; $resn_exch{"2H10 UQ5"}="H102 UQ5"; $resn_exch{"3H10 UQ5"}="H103 UQ5";
$resn_exch{"1H11 UQ5"}="H111 UQ5"; $resn_exch{"2H11 UQ5"}="H112 UQ5"; $resn_exch{"1H12 UQ5"}="H121 UQ5";
$resn_exch{"2H12 UQ5"}="H122 UQ5"; $resn_exch{"1H15 UQ5"}="H151 UQ5"; $resn_exch{"2H15 UQ5"}="H152 UQ5";
$resn_exch{"3H15 UQ5"}="H153 UQ5"; $resn_exch{"1H16 UQ5"}="H161 UQ5"; $resn_exch{"2H16 UQ5"}="H162 UQ5";
$resn_exch{"1H17 UQ5"}="H171 UQ5"; $resn_exch{"2H17 UQ5"}="H172 UQ5"; $resn_exch{"1H20 UQ5"}="H201 UQ5";
$resn_exch{"2H20 UQ5"}="H202 UQ5"; $resn_exch{"3H20 UQ5"}="H203 UQ5"; $resn_exch{"1H21 UQ5"}="H211 UQ5";
$resn_exch{"2H21 UQ5"}="H212 UQ5"; $resn_exch{"1H22 UQ5"}="H221 UQ5"; $resn_exch{"2H22 UQ5"}="H222 UQ5";
$resn_exch{"1H25 UQ5"}="H251 UQ5"; $resn_exch{"2H25 UQ5"}="H252 UQ5"; $resn_exch{"3H25 UQ5"}="H253 UQ5";
$resn_exch{"1H26 UQ5"}="H261 UQ5"; $resn_exch{"2H26 UQ5"}="H262 UQ5"; $resn_exch{"1H27 UQ5"}="H271 UQ5";
$resn_exch{"2H27 UQ5"}="H272 UQ5"; $resn_exch{"1H30 UQ5"}="H301 UQ5"; $resn_exch{"2H30 UQ5"}="H302 UQ5";
$resn_exch{"3H30 UQ5"}="H303 UQ5"; $resn_exch{"1H31 UQ5"}="H311 UQ5"; $resn_exch{"2H31 UQ5"}="H312 UQ5";
$resn_exch{"3H31 UQ5"}="H313 UQ5"; $resn_exch{"1H1M UQ6"}="H1M1 UQ6"; $resn_exch{"2H1M UQ6"}="H1M2 UQ6";
$resn_exch{"3H1M UQ6"}="H1M3 UQ6"; $resn_exch{"1H3M UQ6"}="H3M1 UQ6"; $resn_exch{"2H3M UQ6"}="H3M2 UQ6";
$resn_exch{"3H3M UQ6"}="H3M3 UQ6"; $resn_exch{"1H4M UQ6"}="H4M1 UQ6"; $resn_exch{"2H4M UQ6"}="H4M2 UQ6";
$resn_exch{"3H4M UQ6"}="H4M3 UQ6"; $resn_exch{"1H7  UQ6"}=" H71 UQ6"; $resn_exch{"2H7  UQ6"}=" H72 UQ6";
$resn_exch{"1H10 UQ6"}="H101 UQ6"; $resn_exch{"2H10 UQ6"}="H102 UQ6"; $resn_exch{"3H10 UQ6"}="H103 UQ6";
$resn_exch{"1H11 UQ6"}="H111 UQ6"; $resn_exch{"2H11 UQ6"}="H112 UQ6"; $resn_exch{"1H12 UQ6"}="H121 UQ6";
$resn_exch{"2H12 UQ6"}="H122 UQ6"; $resn_exch{"1H15 UQ6"}="H151 UQ6"; $resn_exch{"2H15 UQ6"}="H152 UQ6";
$resn_exch{"3H15 UQ6"}="H153 UQ6"; $resn_exch{"1H16 UQ6"}="H161 UQ6"; $resn_exch{"2H16 UQ6"}="H162 UQ6";
$resn_exch{"1H17 UQ6"}="H171 UQ6"; $resn_exch{"2H17 UQ6"}="H172 UQ6"; $resn_exch{"1H20 UQ6"}="H201 UQ6";
$resn_exch{"2H20 UQ6"}="H202 UQ6"; $resn_exch{"3H20 UQ6"}="H203 UQ6"; $resn_exch{"1H21 UQ6"}="H211 UQ6";
$resn_exch{"2H21 UQ6"}="H212 UQ6"; $resn_exch{"1H22 UQ6"}="H221 UQ6"; $resn_exch{"2H22 UQ6"}="H222 UQ6";
$resn_exch{"1H25 UQ6"}="H251 UQ6"; $resn_exch{"2H25 UQ6"}="H252 UQ6"; $resn_exch{"3H25 UQ6"}="H253 UQ6";
$resn_exch{"1H26 UQ6"}="H261 UQ6"; $resn_exch{"2H26 UQ6"}="H262 UQ6"; $resn_exch{"1H27 UQ6"}="H271 UQ6";
$resn_exch{"2H27 UQ6"}="H272 UQ6"; $resn_exch{"1H30 UQ6"}="H301 UQ6"; $resn_exch{"2H30 UQ6"}="H302 UQ6";
$resn_exch{"3H30 UQ6"}="H303 UQ6"; $resn_exch{"1H31 UQ6"}="H311 UQ6"; $resn_exch{"2H31 UQ6"}="H312 UQ6";
$resn_exch{"1H32 UQ6"}="H321 UQ6"; $resn_exch{"2H32 UQ6"}="H322 UQ6"; $resn_exch{"1H35 UQ6"}="H351 UQ6";
$resn_exch{"2H35 UQ6"}="H352 UQ6"; $resn_exch{"3H35 UQ6"}="H353 UQ6"; $resn_exch{"1H36 UQ6"}="H361 UQ6";
$resn_exch{"2H36 UQ6"}="H362 UQ6"; $resn_exch{"3H36 UQ6"}="H363 UQ6"; $resn_exch{"1HM2 UQ7"}="HM21 UQ7";
$resn_exch{"2HM2 UQ7"}="HM22 UQ7"; $resn_exch{"3HM2 UQ7"}="HM23 UQ7"; $resn_exch{"1HM3 UQ7"}="HM31 UQ7";
$resn_exch{"2HM3 UQ7"}="HM32 UQ7"; $resn_exch{"3HM3 UQ7"}="HM33 UQ7"; $resn_exch{"1HM5 UQ7"}="HM51 UQ7";
$resn_exch{"2HM5 UQ7"}="HM52 UQ7"; $resn_exch{"3HM5 UQ7"}="HM53 UQ7"; $resn_exch{"1H7  UQ7"}=" H71 UQ7";
$resn_exch{"2H7  UQ7"}=" H72 UQ7"; $resn_exch{"1H10 UQ7"}="H101 UQ7"; $resn_exch{"2H10 UQ7"}="H102 UQ7";
$resn_exch{"3H10 UQ7"}="H103 UQ7"; $resn_exch{"1H11 UQ7"}="H111 UQ7"; $resn_exch{"2H11 UQ7"}="H112 UQ7";
$resn_exch{"1H12 UQ7"}="H121 UQ7"; $resn_exch{"2H12 UQ7"}="H122 UQ7"; $resn_exch{"1H15 UQ7"}="H151 UQ7";
$resn_exch{"2H15 UQ7"}="H152 UQ7"; $resn_exch{"3H15 UQ7"}="H153 UQ7"; $resn_exch{"1H16 UQ7"}="H161 UQ7";
$resn_exch{"2H16 UQ7"}="H162 UQ7"; $resn_exch{"1H17 UQ7"}="H171 UQ7"; $resn_exch{"2H17 UQ7"}="H172 UQ7";
$resn_exch{"1H20 UQ7"}="H201 UQ7"; $resn_exch{"2H20 UQ7"}="H202 UQ7"; $resn_exch{"3H20 UQ7"}="H203 UQ7";
$resn_exch{"1H21 UQ7"}="H211 UQ7"; $resn_exch{"2H21 UQ7"}="H212 UQ7"; $resn_exch{"1H22 UQ7"}="H221 UQ7";
$resn_exch{"2H22 UQ7"}="H222 UQ7"; $resn_exch{"1H25 UQ7"}="H251 UQ7"; $resn_exch{"2H25 UQ7"}="H252 UQ7";
$resn_exch{"3H25 UQ7"}="H253 UQ7"; $resn_exch{"1H26 UQ7"}="H261 UQ7"; $resn_exch{"2H26 UQ7"}="H262 UQ7";
$resn_exch{"1H27 UQ7"}="H271 UQ7"; $resn_exch{"2H27 UQ7"}="H272 UQ7"; $resn_exch{"1H30 UQ7"}="H301 UQ7";
$resn_exch{"2H30 UQ7"}="H302 UQ7"; $resn_exch{"3H30 UQ7"}="H303 UQ7"; $resn_exch{"1H31 UQ7"}="H311 UQ7";
$resn_exch{"2H31 UQ7"}="H312 UQ7"; $resn_exch{"1H32 UQ7"}="H321 UQ7"; $resn_exch{"2H32 UQ7"}="H322 UQ7";
$resn_exch{"1H35 UQ7"}="H351 UQ7"; $resn_exch{"2H35 UQ7"}="H352 UQ7"; $resn_exch{"3H35 UQ7"}="H353 UQ7";
$resn_exch{"1H36 UQ7"}="H361 UQ7"; $resn_exch{"2H36 UQ7"}="H362 UQ7"; $resn_exch{"1H37 UQ7"}="H371 UQ7";
$resn_exch{"2H37 UQ7"}="H372 UQ7"; $resn_exch{"1H40 UQ7"}="H401 UQ7"; $resn_exch{"2H40 UQ7"}="H402 UQ7";
$resn_exch{"3H40 UQ7"}="H403 UQ7"; $resn_exch{"1H41 UQ7"}="H411 UQ7"; $resn_exch{"2H41 UQ7"}="H412 UQ7";
$resn_exch{"3H41 UQ7"}="H413 UQ7"; $resn_exch{"1H17 UR2"}="H171 UR2"; $resn_exch{"2H17 UR2"}="H172 UR2";
$resn_exch{"1H18 UR2"}="H181 UR2"; $resn_exch{"2H18 UR2"}="H182 UR2"; $resn_exch{"1H25 UR2"}="H251 UR2";
$resn_exch{"2H25 UR2"}="H252 UR2"; $resn_exch{"3H25 UR2"}="H253 UR2"; $resn_exch{"1H26 UR2"}="H261 UR2";
$resn_exch{"2H26 UR2"}="H262 UR2"; $resn_exch{"1H27 UR2"}="H271 UR2"; $resn_exch{"2H27 UR2"}="H272 UR2";
$resn_exch{"1H28 UR2"}="H281 UR2"; $resn_exch{"2H28 UR2"}="H282 UR2"; $resn_exch{"1H30 UR2"}="H301 UR2";
$resn_exch{"2H30 UR2"}="H302 UR2"; $resn_exch{"1H31 UR2"}="H311 UR2"; $resn_exch{"2H31 UR2"}="H312 UR2";
$resn_exch{"1H32 UR2"}="H321 UR2"; $resn_exch{"2H32 UR2"}="H322 UR2"; $resn_exch{"1H33 UR2"}="H331 UR2";
$resn_exch{"2H33 UR2"}="H332 UR2"; $resn_exch{"1H34 UR2"}="H341 UR2"; $resn_exch{"2H34 UR2"}="H342 UR2";
$resn_exch{"1H7  UR2"}=" H71 UR2"; $resn_exch{"2H7  UR2"}=" H72 UR2"; $resn_exch{"1H9  UR2"}=" H91 UR2";
$resn_exch{"2H9  UR2"}=" H92 UR2"; $resn_exch{"2HOP UR2"}="HOP2 UR2"; $resn_exch{"3HOP UR2"}="HOP3 UR2";
$resn_exch{"2HOP UR3"}="HOP2 UR3"; $resn_exch{"1H5* UR3"}=" H5' UR3"; $resn_exch{"2H5* UR3"}="H5'' UR3";
$resn_exch{" H4* UR3"}=" H4' UR3"; $resn_exch{" H1* UR3"}=" H1' UR3"; $resn_exch{"1H3U UR3"}="H3U1 UR3";
$resn_exch{"2H3U UR3"}="H3U2 UR3"; $resn_exch{"3H3U UR3"}="H3U3 UR3"; $resn_exch{" H2* UR3"}=" H2' UR3";
$resn_exch{"2HO* UR3"}="HO2' UR3"; $resn_exch{" H3* UR3"}=" H3' UR3"; $resn_exch{"H3T  UR3"}="HO3' UR3";
$resn_exch{"3HOP UR3"}="HOP3 UR3"; $resn_exch{"*HO5 URD"}="HO5' URD"; $resn_exch{"1H5* URD"}=" H5' URD";
$resn_exch{"2H5* URD"}="H5'' URD"; $resn_exch{" H4* URD"}=" H4' URD"; $resn_exch{" H1* URD"}=" H1' URD";
$resn_exch{"1H3  URD"}=" H31 URD"; $resn_exch{"2H3  URD"}=" H32 URD"; $resn_exch{" H2* URD"}=" H2' URD";
$resn_exch{"*HO2 URD"}="HO2' URD"; $resn_exch{" H3* URD"}=" H3' URD"; $resn_exch{"*HO3 URD"}="HO3' URD";
$resn_exch{"1HN1 URE"}="HN11 URE"; $resn_exch{"2HN1 URE"}="HN12 URE"; $resn_exch{"1HN2 URE"}="HN21 URE";
$resn_exch{"2HN2 URE"}="HN22 URE"; $resn_exch{" H1* URI"}=" H1' URI"; $resn_exch{" H2* URI"}=" H2' URI";
$resn_exch{" H3* URI"}=" H3' URI"; $resn_exch{" H4* URI"}=" H4' URI"; $resn_exch{"*HO2 URI"}="HO2' URI";
$resn_exch{"*HO3 URI"}="HO3' URI"; $resn_exch{"1H5* URI"}="H5'1 URI"; $resn_exch{"2H5* URI"}="H5'2 URI";
$resn_exch{"*HO5 URI"}="HO5' URI"; $resn_exch{"1HN5 URN"}="HN51 URN"; $resn_exch{"2HN5 URN"}="HN52 URN";
$resn_exch{"1HN6 URN"}="HN61 URN"; $resn_exch{"2HN6 URN"}="HN62 URN"; $resn_exch{"1HN3 URP"}="HN31 URP";
$resn_exch{"2HN3 URP"}="HN32 URP"; $resn_exch{"1H6  URP"}=" H61 URP"; $resn_exch{"2H6  URP"}=" H62 URP";
$resn_exch{"1H5  URP"}=" H51 URP"; $resn_exch{"2H5  URP"}=" H52 URP"; $resn_exch{"1H5* US1"}="H5'1 US1";
$resn_exch{"2H5* US1"}="H5'2 US1"; $resn_exch{" H4* US1"}=" H4' US1"; $resn_exch{" H3* US1"}=" H3' US1";
$resn_exch{"1H2* US1"}="H2'1 US1"; $resn_exch{"2H2* US1"}="H2'2 US1"; $resn_exch{" H1* US1"}=" H1' US1";
$resn_exch{" H1* USQ"}=" H1D USQ"; $resn_exch{" H2* USQ"}=" H2D USQ"; $resn_exch{"*HO2 USQ"}="HO2' USQ";
$resn_exch{" H3* USQ"}=" H3D USQ"; $resn_exch{" H4* USQ"}=" H4D USQ"; $resn_exch{"*HO3 USQ"}="HO3' USQ";
$resn_exch{"1H5* USQ"}="H5'1 USQ"; $resn_exch{"2H5* USQ"}="H5'2 USQ"; $resn_exch{"2HOA USQ"}="HOA2 USQ";
$resn_exch{"2HOB USQ"}="HOB2 USQ"; $resn_exch{"1H6' USQ"}="H6'1 USQ"; $resn_exch{"2H6' USQ"}="H6'2 USQ";
$resn_exch{"'HO2 USQ"}="HO2A USQ"; $resn_exch{"'HO3 USQ"}="HO3A USQ"; $resn_exch{"'HO4 USQ"}="HO4' USQ";
$resn_exch{"1H5* UTP"}="H5'1 UTP"; $resn_exch{"2H5* UTP"}="H5'2 UTP"; $resn_exch{" H4* UTP"}=" H4' UTP";
$resn_exch{" H1* UTP"}=" H1' UTP"; $resn_exch{" H2* UTP"}=" H2' UTP"; $resn_exch{" H3* UTP"}=" H3' UTP";
$resn_exch{" H1* UVC"}=" H1' UVC"; $resn_exch{" H2* UVC"}=" H2' UVC"; $resn_exch{" H3* UVC"}=" H3' UVC";
$resn_exch{" H4* UVC"}=" H4' UVC"; $resn_exch{"1H5* UVC"}="H5'1 UVC"; $resn_exch{"2H5* UVC"}="H5'2 UVC";
$resn_exch{"*HO5 UVC"}="HO5' UVC"; $resn_exch{"1HOV UVC"}="HOV1 UVC"; $resn_exch{"3HOV UVC"}="HOV3 UVC";
$resn_exch{"1H1M UVW"}="H1M1 UVW"; $resn_exch{"2H1M UVW"}="H1M2 UVW"; $resn_exch{"3H1M UVW"}="H1M3 UVW";
$resn_exch{"2HOP UVW"}="HOP2 UVW"; $resn_exch{"3HOP UVW"}="HOP3 UVW"; $resn_exch{"1H1C UZ9"}="H1C1 UZ9";
$resn_exch{"2H1C UZ9"}="H1C2 UZ9"; $resn_exch{"1H2C UZ9"}="H2C1 UZ9"; $resn_exch{"2H2C UZ9"}="H2C2 UZ9";
$resn_exch{"1H4C UZ9"}="H4C1 UZ9"; $resn_exch{"2H4C UZ9"}="H4C2 UZ9"; $resn_exch{"1H6C UZ9"}="H6C1 UZ9";
$resn_exch{"2H6C UZ9"}="H6C2 UZ9"; $resn_exch{"1HN  UZ9"}=" HN1 UZ9"; $resn_exch{"2HN  UZ9"}=" HN2 UZ9";
$resn_exch{"1HBC UZ9"}="HBC1 UZ9"; $resn_exch{"2HBC UZ9"}="HBC2 UZ9"; $resn_exch{"1HGC UZ9"}="HGC1 UZ9";
$resn_exch{"2HGC UZ9"}="HGC2 UZ9"; $resn_exch{"1HDC UZ9"}="HDC1 UZ9"; $resn_exch{"2HDC UZ9"}="HDC2 UZ9";
$resn_exch{"1HEC UZ9"}="HEC1 UZ9"; $resn_exch{"2HEC UZ9"}="HEC2 UZ9"; $resn_exch{"1H23 UZ9"}="H231 UZ9";
$resn_exch{"2H23 UZ9"}="H232 UZ9"; $resn_exch{"1H22 UZ9"}="H221 UZ9"; $resn_exch{"2H22 UZ9"}="H222 UZ9";
$resn_exch{"1H21 UZ9"}="H211 UZ9"; $resn_exch{"2H21 UZ9"}="H212 UZ9"; $resn_exch{"3H21 UZ9"}="H213 UZ9";
$resn_exch{"1H16 UZ9"}="H161 UZ9"; $resn_exch{"2H16 UZ9"}="H162 UZ9"; $resn_exch{"1H12 UZ9"}="H121 UZ9";
$resn_exch{"2H12 UZ9"}="H122 UZ9"; $resn_exch{"1H18 UZ9"}="H181 UZ9"; $resn_exch{"2H18 UZ9"}="H182 UZ9";
$resn_exch{"3H18 UZ9"}="H183 UZ9"; $resn_exch{"1H15 UZ9"}="H151 UZ9"; $resn_exch{"2H15 UZ9"}="H152 UZ9";
$resn_exch{"1H7C UZ9"}="H7C1 UZ9"; $resn_exch{"2H7C UZ9"}="H7C2 UZ9"; $resn_exch{"1H11 UZ9"}="H111 UZ9";
$resn_exch{"2H11 UZ9"}="H112 UZ9"; $resn_exch{"1H19 UZ9"}="H191 UZ9"; $resn_exch{"2H19 UZ9"}="H192 UZ9";
$resn_exch{"3H19 UZ9"}="H193 UZ9"; $resn_exch{"1HAA V12"}="HAA1 V12"; $resn_exch{"2HAA V12"}="HAA2 V12";
$resn_exch{"1HAN V12"}="HAN1 V12"; $resn_exch{"2HAN V12"}="HAN2 V12"; $resn_exch{"1HAL V12"}="HAL1 V12";
$resn_exch{"2HAL V12"}="HAL2 V12"; $resn_exch{"1H14 V15"}="H141 V15"; $resn_exch{"2H14 V15"}="H142 V15";
$resn_exch{"1H15 V15"}="H151 V15"; $resn_exch{"2H15 V15"}="H152 V15"; $resn_exch{"1H16 V15"}="H161 V15";
$resn_exch{"2H16 V15"}="H162 V15"; $resn_exch{"1HAA V25"}="HAA1 V25"; $resn_exch{"2HAA V25"}="HAA2 V25";
$resn_exch{"3HAA V25"}="HAA3 V25"; $resn_exch{"1HAL V25"}="HAL1 V25"; $resn_exch{"2HAL V25"}="HAL2 V25";
$resn_exch{"1HAB V25"}="HAB1 V25"; $resn_exch{"2HAB V25"}="HAB2 V25"; $resn_exch{"3HAB V25"}="HAB3 V25";
$resn_exch{"1HAD V25"}="HAD1 V25"; $resn_exch{"2HAD V25"}="HAD2 V25"; $resn_exch{"1H7  V35"}=" H71 V35";
$resn_exch{"2H7  V35"}=" H72 V35"; $resn_exch{"1H9  V35"}=" H91 V35"; $resn_exch{"2H9  V35"}=" H92 V35";
$resn_exch{"3H9  V35"}=" H93 V35"; $resn_exch{"1HOB V35"}="HOB1 V35"; $resn_exch{"2HOB V35"}="HOB2 V35";
$resn_exch{"3HOB V35"}="HOB3 V35"; $resn_exch{"1H7  V36"}=" H71 V36"; $resn_exch{"2H7  V36"}=" H72 V36";
$resn_exch{"1H9  V36"}=" H91 V36"; $resn_exch{"2H9  V36"}=" H92 V36"; $resn_exch{"3H9  V36"}=" H93 V36";
$resn_exch{"1HOB V36"}="HOB1 V36"; $resn_exch{"2HOB V36"}="HOB2 V36"; $resn_exch{"3HOB V36"}="HOB3 V36";
$resn_exch{"1HBI V5X"}="HBI1 V5X"; $resn_exch{"2HBI V5X"}="HBI2 V5X"; $resn_exch{"3HBI V5X"}="HBI3 V5X";
$resn_exch{"1HBP V5X"}="HBP1 V5X"; $resn_exch{"2HBP V5X"}="HBP2 V5X"; $resn_exch{"3HBP V5X"}="HBP3 V5X";
$resn_exch{"1HBH V5X"}="HBH1 V5X"; $resn_exch{"2HBH V5X"}="HBH2 V5X"; $resn_exch{"1HBC V5X"}="HBC1 V5X";
$resn_exch{"2HBC V5X"}="HBC2 V5X"; $resn_exch{"1HAV V5X"}="HAV1 V5X"; $resn_exch{"2HAV V5X"}="HAV2 V5X";
$resn_exch{"1HAW V5X"}="HAW1 V5X"; $resn_exch{"2HAW V5X"}="HAW2 V5X"; $resn_exch{"1HAX V5X"}="HAX1 V5X";
$resn_exch{"2HAX V5X"}="HAX2 V5X"; $resn_exch{"1HAY V5X"}="HAY1 V5X"; $resn_exch{"2HAY V5X"}="HAY2 V5X";
$resn_exch{"1HAQ V5X"}="HAQ1 V5X"; $resn_exch{"2HAQ V5X"}="HAQ2 V5X"; $resn_exch{"1HAP V5X"}="HAP1 V5X";
$resn_exch{"2HAP V5X"}="HAP2 V5X"; $resn_exch{"1H22  VA"}="H221  VA"; $resn_exch{"2H22  VA"}="H222  VA";
$resn_exch{"3H22  VA"}="H223  VA"; $resn_exch{"1H1   VA"}="H11A  VA"; $resn_exch{"2H1   VA"}="H12A  VA";
$resn_exch{"1H2   VA"}=" H21  VA"; $resn_exch{"2H2   VA"}=" H22  VA"; $resn_exch{"1H3   VA"}=" H31  VA";
$resn_exch{"2H3   VA"}=" H32  VA"; $resn_exch{"1H4   VA"}=" H41  VA"; $resn_exch{"2H4   VA"}=" H42  VA";
$resn_exch{"1H5   VA"}=" H51  VA"; $resn_exch{"2H5   VA"}=" H52  VA"; $resn_exch{"1H6   VA"}=" H61  VA";
$resn_exch{"2H6   VA"}=" H62  VA"; $resn_exch{"1H7   VA"}=" H71  VA"; $resn_exch{"2H7   VA"}=" H72  VA";
$resn_exch{"1H8   VA"}=" H81  VA"; $resn_exch{"2H8   VA"}=" H82  VA"; $resn_exch{"1H9   VA"}=" H91  VA";
$resn_exch{"2H9   VA"}=" H92  VA"; $resn_exch{"1H10  VA"}="H101  VA"; $resn_exch{"2H10  VA"}="H102  VA";
$resn_exch{"1H13  VA"}="H131  VA"; $resn_exch{"2H13  VA"}="H132  VA"; $resn_exch{"1H14  VA"}="H141  VA";
$resn_exch{"2H14  VA"}="H142  VA"; $resn_exch{"1H15  VA"}="H151  VA"; $resn_exch{"2H15  VA"}="H152  VA";
$resn_exch{"1H16  VA"}="H161  VA"; $resn_exch{"2H16  VA"}="H162  VA"; $resn_exch{"1H17  VA"}="H171  VA";
$resn_exch{"2H17  VA"}="H172  VA"; $resn_exch{"1H18  VA"}="H181  VA"; $resn_exch{"2H18  VA"}="H182  VA";
$resn_exch{"3H18  VA"}="H183  VA"; $resn_exch{"1H12 VA1"}="H121 VA1"; $resn_exch{"2H12 VA1"}="H122 VA1";
$resn_exch{"1HG1 VAA"}="HG11 VAA"; $resn_exch{"2HG1 VAA"}="HG12 VAA"; $resn_exch{"3HG1 VAA"}="HG13 VAA";
$resn_exch{"1HG2 VAA"}="HG21 VAA"; $resn_exch{"2HG2 VAA"}="HG22 VAA"; $resn_exch{"3HG2 VAA"}="HG23 VAA";
$resn_exch{"3HNS VAA"}="HNS3 VAA"; $resn_exch{"5HN* VAA"}="HN'5 VAA"; $resn_exch{"1H5* VAA"}="H5'1 VAA";
$resn_exch{"2H5* VAA"}="H5'2 VAA"; $resn_exch{" H4* VAA"}=" H4' VAA"; $resn_exch{" H1* VAA"}=" H1' VAA";
$resn_exch{"1H6  VAA"}=" H61 VAA"; $resn_exch{"2H6  VAA"}=" H62 VAA"; $resn_exch{" H2* VAA"}=" H2' VAA";
$resn_exch{"2HO* VAA"}="HO'2 VAA"; $resn_exch{" H3* VAA"}=" H3' VAA"; $resn_exch{"3HT* VAA"}="HT'3 VAA";
$resn_exch{"1H3  VAC"}=" H31 VAC"; $resn_exch{"2H3  VAC"}=" H32 VAC"; $resn_exch{"1H4  VAC"}=" H41 VAC";
$resn_exch{"2H4  VAC"}=" H42 VAC"; $resn_exch{"1H12 VAC"}="H121 VAC"; $resn_exch{"2H12 VAC"}="H122 VAC";
$resn_exch{"9HN1 VAC"}="HN19 VAC"; $resn_exch{"2HO2 VAC"}="HO22 VAC"; $resn_exch{"1H26 VAC"}="H261 VAC";
$resn_exch{"2H26 VAC"}="H262 VAC"; $resn_exch{"1H31 VAC"}="H311 VAC"; $resn_exch{"2H31 VAC"}="H312 VAC";
$resn_exch{"1H68 VAC"}="H681 VAC"; $resn_exch{"2H68 VAC"}="H682 VAC"; $resn_exch{"0HO7 VAC"}="HO70 VAC";
$resn_exch{"6HN8 VAC"}="HN86 VAC"; $resn_exch{"1HG1 VAD"}="HG11 VAD"; $resn_exch{"2HG1 VAD"}="HG12 VAD";
$resn_exch{"3HG1 VAD"}="HG13 VAD"; $resn_exch{"1HG2 VAD"}="HG21 VAD"; $resn_exch{"2HG2 VAD"}="HG22 VAD";
$resn_exch{"3HG2 VAD"}="HG23 VAD"; $resn_exch{"1HN  VAF"}=" HN1 VAF"; $resn_exch{"2HN  VAF"}=" HN2 VAF";
$resn_exch{"1HG1 VAF"}="HG11 VAF"; $resn_exch{"2HG1 VAF"}="HG12 VAF"; $resn_exch{"3HG1 VAF"}="HG13 VAF";
$resn_exch{"1HG2 VAF"}="HG21 VAF"; $resn_exch{"2HG2 VAF"}="HG22 VAF"; $resn_exch{"3HG2 VAF"}="HG23 VAF";
$resn_exch{"1HJ  VAF"}=" HJ1 VAF"; $resn_exch{"2HJ  VAF"}=" HJ2 VAF"; $resn_exch{"3HJ  VAF"}=" HJ3 VAF";
$resn_exch{"1H1  VAG"}="H11A VAG"; $resn_exch{"2H1  VAG"}=" H12 VAG"; $resn_exch{"3H1  VAG"}=" H13 VAG";
$resn_exch{"1HN2 VAG"}="HN21 VAG"; $resn_exch{"2HN2 VAG"}="HN22 VAG"; $resn_exch{"1H6  VAG"}="H61A VAG";
$resn_exch{"2H6  VAG"}="H62A VAG"; $resn_exch{"1H8  VAG"}=" H81 VAG"; $resn_exch{"2H8  VAG"}=" H82 VAG";
$resn_exch{"3H8  VAG"}=" H83 VAG"; $resn_exch{"1H9  VAG"}=" H91 VAG"; $resn_exch{"2H9  VAG"}=" H92 VAG";
$resn_exch{"3H9  VAG"}=" H93 VAG"; $resn_exch{"1H28 VAG"}="H281 VAG"; $resn_exch{"2H28 VAG"}="H282 VAG";
$resn_exch{"1HN1 VAG"}="HN11 VAG"; $resn_exch{"2HN1 VAG"}="HN12 VAG"; $resn_exch{"3HN4 VAG"}="HN43 VAG";
$resn_exch{" HN2 VAL"}=" H2  VAL"; $resn_exch{"1HG1 VAL"}="HG11 VAL"; $resn_exch{"2HG1 VAL"}="HG12 VAL";
$resn_exch{"3HG1 VAL"}="HG13 VAL"; $resn_exch{"1HG2 VAL"}="HG21 VAL"; $resn_exch{"2HG2 VAL"}="HG22 VAL";
$resn_exch{"3HG2 VAL"}="HG23 VAL"; $resn_exch{"1H12 VAM"}="H121 VAM"; $resn_exch{"2H12 VAM"}="H122 VAM";
$resn_exch{"1HMC VAM"}="HMC1 VAM"; $resn_exch{"2HMC VAM"}="HMC2 VAM"; $resn_exch{"3HMC VAM"}="HMC3 VAM";
$resn_exch{"1H2C VAM"}="H2C1 VAM"; $resn_exch{"2H2C VAM"}="H2C2 VAM"; $resn_exch{"1H1  VAN"}="H11A VAN";
$resn_exch{"2H1  VAN"}=" H12 VAN"; $resn_exch{"3H1  VAN"}=" H13 VAN"; $resn_exch{"1HN2 VAN"}="HN21 VAN";
$resn_exch{"2HN2 VAN"}="HN22 VAN"; $resn_exch{"1H6  VAN"}="H61A VAN"; $resn_exch{"2H6  VAN"}="H62A VAN";
$resn_exch{"1H8  VAN"}=" H81 VAN"; $resn_exch{"2H8  VAN"}=" H82 VAN"; $resn_exch{"3H8  VAN"}=" H83 VAN";
$resn_exch{"1H9  VAN"}=" H91 VAN"; $resn_exch{"2H9  VAN"}=" H92 VAN"; $resn_exch{"3H9  VAN"}=" H93 VAN";
$resn_exch{"1H28 VAN"}="H281 VAN"; $resn_exch{"2H28 VAN"}="H282 VAN"; $resn_exch{"1HN1 VAN"}="HN11 VAN";
$resn_exch{"2HN1 VAN"}="HN12 VAN"; $resn_exch{"3HN4 VAN"}="HN43 VAN"; $resn_exch{"'HO3 VAN"}="HO3' VAN";
$resn_exch{"'HO4 VAN"}="HO4' VAN"; $resn_exch{"1H6' VAN"}="H6'1 VAN"; $resn_exch{"2H6' VAN"}="H6'2 VAN";
$resn_exch{"'HO6 VAN"}="HO6' VAN"; $resn_exch{" H1\" VAN"}=" H1B VAN"; $resn_exch{"1H2\" VAN"}="H2'1 VAN";
$resn_exch{"2H2\" VAN"}="H2'2 VAN"; $resn_exch{"1HN3 VAN"}="HN31 VAN"; $resn_exch{"2HN3 VAN"}="HN32 VAN";
$resn_exch{"3HN3 VAN"}="HN33 VAN"; $resn_exch{"1H3M VAN"}="H3M1 VAN"; $resn_exch{"2H3M VAN"}="H3M2 VAN";
$resn_exch{"3H3M VAN"}="H3M3 VAN"; $resn_exch{" H4\" VAN"}=" H4B VAN"; $resn_exch{"\"HO4 VAN"}="HO4A VAN";
$resn_exch{" H5\" VAN"}=" H5B VAN"; $resn_exch{"1H5M VAN"}="H5M1 VAN"; $resn_exch{"2H5M VAN"}="H5M2 VAN";
$resn_exch{"3H5M VAN"}="H5M3 VAN"; $resn_exch{"1H1  VAS"}=" H11 VAS"; $resn_exch{"2H1  VAS"}=" H12 VAS";
$resn_exch{"3H1  VAS"}=" H13 VAS"; $resn_exch{"1H2' VAS"}="H2'1 VAS"; $resn_exch{"2H2' VAS"}="H2'2 VAS";
$resn_exch{"3H2' VAS"}="H2'3 VAS"; $resn_exch{"1H3' VAS"}="H3'1 VAS"; $resn_exch{"2H3' VAS"}="H3'2 VAS";
$resn_exch{"3H3' VAS"}="H3'3 VAS"; $resn_exch{"1H1  VAX"}="H11A VAX"; $resn_exch{"2H1  VAX"}=" H12 VAX";
$resn_exch{"3H1  VAX"}=" H13 VAX"; $resn_exch{"1HN2 VAX"}="HN21 VAX"; $resn_exch{"2HN2 VAX"}="HN22 VAX";
$resn_exch{"1H6  VAX"}="H61A VAX"; $resn_exch{"2H6  VAX"}="H62A VAX"; $resn_exch{"1H8  VAX"}="H81A VAX";
$resn_exch{"2H8  VAX"}=" H82 VAX"; $resn_exch{"3H8  VAX"}="H83A VAX"; $resn_exch{"1H9  VAX"}=" H91 VAX";
$resn_exch{"2H9  VAX"}=" H92 VAX"; $resn_exch{"3H9  VAX"}=" H93 VAX"; $resn_exch{"1H28 VAX"}="H281 VAX";
$resn_exch{"2H28 VAX"}="H282 VAX"; $resn_exch{"1HN1 VAX"}="HN11 VAX"; $resn_exch{"2HN1 VAX"}="HN12 VAX";
$resn_exch{"1H84 VAX"}="H841 VAX"; $resn_exch{"2H84 VAX"}="H842 VAX"; $resn_exch{"3H84 VAX"}="H843 VAX";
$resn_exch{"1HN8 VAX"}="HN81 VAX"; $resn_exch{"2HN8 VAX"}="HN82 VAX"; $resn_exch{"3HN8 VAX"}="HN83 VAX";
$resn_exch{"1H89 VAX"}="H891 VAX"; $resn_exch{"2H89 VAX"}="H892 VAX"; $resn_exch{"3H89 VAX"}="H893 VAX";
$resn_exch{"1H90 VAX"}="H901 VAX"; $resn_exch{"2H90 VAX"}="H902 VAX"; $resn_exch{"'HO3 VAX"}="HO3' VAX";
$resn_exch{"'HO4 VAX"}="HO4' VAX"; $resn_exch{"1H6' VAX"}="H6'1 VAX"; $resn_exch{"2H6' VAX"}="H6'2 VAX";
$resn_exch{"'HO6 VAX"}="HO6' VAX"; $resn_exch{" H1\" VAX"}=" H1B VAX"; $resn_exch{"1H2\" VAX"}="H2'1 VAX";
$resn_exch{"2H2\" VAX"}="H2'2 VAX"; $resn_exch{"1HN3 VAX"}="HN31 VAX"; $resn_exch{"2HN3 VAX"}="HN32 VAX";
$resn_exch{"3HN3 VAX"}="HN33 VAX"; $resn_exch{"1H3M VAX"}="H3M1 VAX"; $resn_exch{"2H3M VAX"}="H3M2 VAX";
$resn_exch{"3H3M VAX"}="H3M3 VAX"; $resn_exch{" H4\" VAX"}=" H4B VAX"; $resn_exch{"\"HO4 VAX"}="HO4A VAX";
$resn_exch{" H5\" VAX"}=" H5B VAX"; $resn_exch{"1H5M VAX"}="H5M1 VAX"; $resn_exch{"2H5M VAX"}="H5M2 VAX";
$resn_exch{"3H5M VAX"}="H5M3 VAX"; $resn_exch{"1H8C VBZ"}="H8C1 VBZ"; $resn_exch{"2H8C VBZ"}="H8C2 VBZ";
$resn_exch{"1H6C VBZ"}="H6C1 VBZ"; $resn_exch{"2H6C VBZ"}="H6C2 VBZ"; $resn_exch{"1H7C VBZ"}="H7C1 VBZ";
$resn_exch{"2H7C VBZ"}="H7C2 VBZ"; $resn_exch{"1H9C VBZ"}="H9C1 VBZ"; $resn_exch{"2H9C VBZ"}="H9C2 VBZ";
$resn_exch{"1H2  VCA"}=" H21 VCA"; $resn_exch{"2H2  VCA"}=" H22 VCA"; $resn_exch{"1H3  VCA"}=" H31 VCA";
$resn_exch{"2H3  VCA"}=" H32 VCA"; $resn_exch{"1H4  VCA"}=" H41 VCA"; $resn_exch{"2H4  VCA"}=" H42 VCA";
$resn_exch{"1H5  VCA"}=" H51 VCA"; $resn_exch{"2H5  VCA"}=" H52 VCA"; $resn_exch{"1H6  VCA"}=" H61 VCA";
$resn_exch{"2H6  VCA"}=" H62 VCA"; $resn_exch{"1H7  VCA"}=" H71 VCA"; $resn_exch{"2H7  VCA"}=" H72 VCA";
$resn_exch{"1H8  VCA"}=" H81 VCA"; $resn_exch{"2H8  VCA"}=" H82 VCA"; $resn_exch{"1H9  VCA"}=" H91 VCA";
$resn_exch{"2H9  VCA"}=" H92 VCA"; $resn_exch{"1H10 VCA"}="H101 VCA"; $resn_exch{"2H10 VCA"}="H102 VCA";
$resn_exch{"1H13 VCA"}="H131 VCA"; $resn_exch{"2H13 VCA"}="H132 VCA"; $resn_exch{"1H14 VCA"}="H141 VCA";
$resn_exch{"2H14 VCA"}="H142 VCA"; $resn_exch{"1H15 VCA"}="H151 VCA"; $resn_exch{"2H15 VCA"}="H152 VCA";
$resn_exch{"1H16 VCA"}="H161 VCA"; $resn_exch{"2H16 VCA"}="H162 VCA"; $resn_exch{"1H17 VCA"}="H171 VCA";
$resn_exch{"2H17 VCA"}="H172 VCA"; $resn_exch{"1H18 VCA"}="H181 VCA"; $resn_exch{"2H18 VCA"}="H182 VCA";
$resn_exch{"3H18 VCA"}="H183 VCA"; $resn_exch{"1H28 VD1"}="H281 VD1"; $resn_exch{"2H28 VD1"}="H282 VD1";
$resn_exch{"1H4  VD1"}=" H41 VD1"; $resn_exch{"2H4  VD1"}=" H42 VD1"; $resn_exch{"1H10 VD1"}="H101 VD1";
$resn_exch{"2H10 VD1"}="H102 VD1"; $resn_exch{"1H9  VD1"}=" H91 VD1"; $resn_exch{"2H9  VD1"}=" H92 VD1";
$resn_exch{"1H11 VD1"}="H111 VD1"; $resn_exch{"2H11 VD1"}="H112 VD1"; $resn_exch{"1H12 VD1"}="H121 VD1";
$resn_exch{"2H12 VD1"}="H122 VD1"; $resn_exch{"1H18 VD1"}="H181 VD1"; $resn_exch{"2H18 VD1"}="H182 VD1";
$resn_exch{"3H18 VD1"}="H183 VD1"; $resn_exch{"1H15 VD1"}="H151 VD1"; $resn_exch{"2H15 VD1"}="H152 VD1";
$resn_exch{"1H16 VD1"}="H161 VD1"; $resn_exch{"2H16 VD1"}="H162 VD1"; $resn_exch{"1H21 VD1"}="H211 VD1";
$resn_exch{"2H21 VD1"}="H212 VD1"; $resn_exch{"3H21 VD1"}="H213 VD1"; $resn_exch{"1H22 VD1"}="H221 VD1";
$resn_exch{"2H22 VD1"}="H222 VD1"; $resn_exch{"1H23 VD1"}="H231 VD1"; $resn_exch{"2H23 VD1"}="H232 VD1";
$resn_exch{"3H23 VD1"}="H233 VD1"; $resn_exch{"1H27 VD2"}="H271 VD2"; $resn_exch{"2H27 VD2"}="H272 VD2";
$resn_exch{"3H27 VD2"}="H273 VD2"; $resn_exch{"1H26 VD2"}="H261 VD2"; $resn_exch{"2H26 VD2"}="H262 VD2";
$resn_exch{"3H26 VD2"}="H263 VD2"; $resn_exch{"1H24 VD2"}="H241 VD2"; $resn_exch{"2H24 VD2"}="H242 VD2";
$resn_exch{"1H23 VD2"}="H231 VD2"; $resn_exch{"2H23 VD2"}="H232 VD2"; $resn_exch{"1H22 VD2"}="H221 VD2";
$resn_exch{"2H22 VD2"}="H222 VD2"; $resn_exch{"1H21 VD2"}="H211 VD2"; $resn_exch{"2H21 VD2"}="H212 VD2";
$resn_exch{"3H21 VD2"}="H213 VD2"; $resn_exch{"1H16 VD2"}="H161 VD2"; $resn_exch{"2H16 VD2"}="H162 VD2";
$resn_exch{"1H18 VD2"}="H181 VD2"; $resn_exch{"2H18 VD2"}="H182 VD2"; $resn_exch{"3H18 VD2"}="H183 VD2";
$resn_exch{"1H15 VD2"}="H151 VD2"; $resn_exch{"2H15 VD2"}="H152 VD2"; $resn_exch{"1H12 VD2"}="H121 VD2";
$resn_exch{"2H12 VD2"}="H122 VD2"; $resn_exch{"1H11 VD2"}="H111 VD2"; $resn_exch{"2H11 VD2"}="H112 VD2";
$resn_exch{"1H9  VD2"}=" H91 VD2"; $resn_exch{"2H9  VD2"}=" H92 VD2"; $resn_exch{"1H10 VD2"}="H101 VD2";
$resn_exch{"2H10 VD2"}="H102 VD2"; $resn_exch{"1H4  VD2"}=" H41 VD2"; $resn_exch{"2H4  VD2"}=" H42 VD2";
$resn_exch{"1H28 VD2"}="H281 VD2"; $resn_exch{"2H28 VD2"}="H282 VD2"; $resn_exch{"3H28 VD2"}="H283 VD2";
$resn_exch{"1H1  VD3"}=" H11 VD3"; $resn_exch{"2H1  VD3"}=" H12 VD3"; $resn_exch{"1H2  VD3"}=" H21 VD3";
$resn_exch{"2H2  VD3"}=" H22 VD3"; $resn_exch{"1H4  VD3"}=" H41 VD3"; $resn_exch{"2H4  VD3"}=" H42 VD3";
$resn_exch{"1H10 VD3"}="H101 VD3"; $resn_exch{"2H10 VD3"}="H102 VD3"; $resn_exch{"1H11 VD3"}="H111 VD3";
$resn_exch{"2H11 VD3"}="H112 VD3"; $resn_exch{"1H12 VD3"}="H121 VD3"; $resn_exch{"2H12 VD3"}="H122 VD3";
$resn_exch{"1H15 VD3"}="H151 VD3"; $resn_exch{"2H15 VD3"}="H152 VD3"; $resn_exch{"1H16 VD3"}="H161 VD3";
$resn_exch{"2H16 VD3"}="H162 VD3"; $resn_exch{"1H18 VD3"}="H181 VD3"; $resn_exch{"2H18 VD3"}="H182 VD3";
$resn_exch{"3H18 VD3"}="H183 VD3"; $resn_exch{"1H19 VD3"}="H191 VD3"; $resn_exch{"2H19 VD3"}="H192 VD3";
$resn_exch{"1H21 VD3"}="H211 VD3"; $resn_exch{"2H21 VD3"}="H212 VD3"; $resn_exch{"3H21 VD3"}="H213 VD3";
$resn_exch{"1H22 VD3"}="H221 VD3"; $resn_exch{"2H22 VD3"}="H222 VD3"; $resn_exch{"1H23 VD3"}="H231 VD3";
$resn_exch{"2H23 VD3"}="H232 VD3"; $resn_exch{"1H24 VD3"}="H241 VD3"; $resn_exch{"2H24 VD3"}="H242 VD3";
$resn_exch{"1H26 VD3"}="H261 VD3"; $resn_exch{"2H26 VD3"}="H262 VD3"; $resn_exch{"3H26 VD3"}="H263 VD3";
$resn_exch{"1H27 VD3"}="H271 VD3"; $resn_exch{"2H27 VD3"}="H272 VD3"; $resn_exch{"3H27 VD3"}="H273 VD3";
$resn_exch{"1H26 VD4"}="H261 VD4"; $resn_exch{"2H26 VD4"}="H262 VD4"; $resn_exch{"3H26 VD4"}="H263 VD4";
$resn_exch{"1H27 VD4"}="H271 VD4"; $resn_exch{"2H27 VD4"}="H272 VD4"; $resn_exch{"3H27 VD4"}="H273 VD4";
$resn_exch{"1H24 VD4"}="H241 VD4"; $resn_exch{"2H24 VD4"}="H242 VD4"; $resn_exch{"1H23 VD4"}="H231 VD4";
$resn_exch{"2H23 VD4"}="H232 VD4"; $resn_exch{"1H22 VD4"}="H221 VD4"; $resn_exch{"2H22 VD4"}="H222 VD4";
$resn_exch{"1H21 VD4"}="H211 VD4"; $resn_exch{"2H21 VD4"}="H212 VD4"; $resn_exch{"3H21 VD4"}="H213 VD4";
$resn_exch{"1H18 VD4"}="H181 VD4"; $resn_exch{"2H18 VD4"}="H182 VD4"; $resn_exch{"3H18 VD4"}="H183 VD4";
$resn_exch{"1H12 VD4"}="H121 VD4"; $resn_exch{"2H12 VD4"}="H122 VD4"; $resn_exch{"1H11 VD4"}="H111 VD4";
$resn_exch{"2H11 VD4"}="H112 VD4"; $resn_exch{"1H9  VD4"}=" H91 VD4"; $resn_exch{"2H9  VD4"}=" H92 VD4";
$resn_exch{"1H16 VD4"}="H161 VD4"; $resn_exch{"2H16 VD4"}="H162 VD4"; $resn_exch{"1H15 VD4"}="H151 VD4";
$resn_exch{"2H15 VD4"}="H152 VD4"; $resn_exch{"1H10 VD4"}="H101 VD4"; $resn_exch{"2H10 VD4"}="H102 VD4";
$resn_exch{"1H4  VD4"}=" H41 VD4"; $resn_exch{"2H4  VD4"}=" H42 VD4"; $resn_exch{"1H28 VD4"}="H281 VD4";
$resn_exch{"2H28 VD4"}="H282 VD4"; $resn_exch{"1H28 VD5"}="H281 VD5"; $resn_exch{"2H28 VD5"}="H282 VD5";
$resn_exch{"1H10 VD5"}="H101 VD5"; $resn_exch{"2H10 VD5"}="H102 VD5"; $resn_exch{"1H4  VD5"}=" H41 VD5";
$resn_exch{"2H4  VD5"}=" H42 VD5"; $resn_exch{"1H9  VD5"}=" H91 VD5"; $resn_exch{"2H9  VD5"}=" H92 VD5";
$resn_exch{"1H11 VD5"}="H111 VD5"; $resn_exch{"2H11 VD5"}="H112 VD5"; $resn_exch{"1H12 VD5"}="H121 VD5";
$resn_exch{"2H12 VD5"}="H122 VD5"; $resn_exch{"1H18 VD5"}="H181 VD5"; $resn_exch{"2H18 VD5"}="H182 VD5";
$resn_exch{"3H18 VD5"}="H183 VD5"; $resn_exch{"1H15 VD5"}="H151 VD5"; $resn_exch{"2H15 VD5"}="H152 VD5";
$resn_exch{"1H16 VD5"}="H161 VD5"; $resn_exch{"2H16 VD5"}="H162 VD5"; $resn_exch{"1H21 VD5"}="H211 VD5";
$resn_exch{"2H21 VD5"}="H212 VD5"; $resn_exch{"3H21 VD5"}="H213 VD5"; $resn_exch{"1H22 VD5"}="H221 VD5";
$resn_exch{"2H22 VD5"}="H222 VD5"; $resn_exch{"1H23 VD5"}="H231 VD5"; $resn_exch{"2H23 VD5"}="H232 VD5";
$resn_exch{"1H24 VD5"}="H241 VD5"; $resn_exch{"2H24 VD5"}="H242 VD5"; $resn_exch{"1H27 VD5"}="H271 VD5";
$resn_exch{"2H27 VD5"}="H272 VD5"; $resn_exch{"3H27 VD5"}="H273 VD5"; $resn_exch{"1H26 VD5"}="H261 VD5";
$resn_exch{"2H26 VD5"}="H262 VD5"; $resn_exch{"3H26 VD5"}="H263 VD5"; $resn_exch{"1HN  VDL"}=" HN1 VDL";
$resn_exch{"2HN  VDL"}=" HN2 VDL"; $resn_exch{"1HG1 VDL"}="HG11 VDL"; $resn_exch{"2HG1 VDL"}="HG12 VDL";
$resn_exch{"3HG1 VDL"}="HG13 VDL"; $resn_exch{"1HG2 VDL"}="HG21 VDL"; $resn_exch{"2HG2 VDL"}="HG22 VDL";
$resn_exch{"1H7C VDM"}="H7C1 VDM"; $resn_exch{"2H7C VDM"}="H7C2 VDM"; $resn_exch{" H1* VDM"}=" H1' VDM";
$resn_exch{" H2* VDM"}=" H2' VDM"; $resn_exch{"1H6* VDM"}="H6'1 VDM"; $resn_exch{"2H6* VDM"}="H6'2 VDM";
$resn_exch{" H3* VDM"}=" H3' VDM"; $resn_exch{" H5* VDM"}=" H5' VDM"; $resn_exch{"1H7* VDM"}="H7'1 VDM";
$resn_exch{"2H7* VDM"}="H7'2 VDM"; $resn_exch{" H4* VDM"}=" H4' VDM"; $resn_exch{" H7* VDM"}=" H7' VDM";
$resn_exch{"1H1  VDN"}=" H11 VDN"; $resn_exch{"2H1  VDN"}=" H12 VDN"; $resn_exch{"3H1  VDN"}=" H13 VDN";
$resn_exch{"1H2  VDN"}=" H21 VDN"; $resn_exch{"2H2  VDN"}=" H22 VDN"; $resn_exch{"1H13 VDN"}="H131 VDN";
$resn_exch{"2H13 VDN"}="H132 VDN"; $resn_exch{"3H13 VDN"}="H133 VDN"; $resn_exch{"1H15 VDN"}="H151 VDN";
$resn_exch{"2H15 VDN"}="H152 VDN"; $resn_exch{"1H16 VDN"}="H161 VDN"; $resn_exch{"2H16 VDN"}="H162 VDN";
$resn_exch{"1H18 VDN"}="H181 VDN"; $resn_exch{"2H18 VDN"}="H182 VDN"; $resn_exch{"1H19 VDN"}="H191 VDN";
$resn_exch{"2H19 VDN"}="H192 VDN"; $resn_exch{"1H20 VDN"}="H201 VDN"; $resn_exch{"2H20 VDN"}="H202 VDN";
$resn_exch{"1H31 VDN"}="H311 VDN"; $resn_exch{"2H31 VDN"}="H312 VDN"; $resn_exch{"3H31 VDN"}="H313 VDN";
$resn_exch{"1H32 VDN"}="H321 VDN"; $resn_exch{"2H32 VDN"}="H322 VDN"; $resn_exch{"1H33 VDN"}="H331 VDN";
$resn_exch{"2H33 VDN"}="H332 VDN"; $resn_exch{"1H34 VDN"}="H341 VDN"; $resn_exch{"2H34 VDN"}="H342 VDN";
$resn_exch{"3H34 VDN"}="H343 VDN"; $resn_exch{"1H1  VDX"}=" H11 VDX"; $resn_exch{"1H2  VDX"}=" H21 VDX";
$resn_exch{"2H2  VDX"}=" H22 VDX"; $resn_exch{"1H3  VDX"}=" H31 VDX"; $resn_exch{"1H4  VDX"}=" H41 VDX";
$resn_exch{"2H4  VDX"}=" H42 VDX"; $resn_exch{"1H6  VDX"}=" H61 VDX"; $resn_exch{"1H7  VDX"}=" H71 VDX";
$resn_exch{"1H9  VDX"}=" H91 VDX"; $resn_exch{"2H9  VDX"}=" H92 VDX"; $resn_exch{"1H11 VDX"}="H111 VDX";
$resn_exch{"2H11 VDX"}="H112 VDX"; $resn_exch{"1H12 VDX"}="H121 VDX"; $resn_exch{"2H12 VDX"}="H122 VDX";
$resn_exch{"1H14 VDX"}="H141 VDX"; $resn_exch{"1H15 VDX"}="H151 VDX"; $resn_exch{"2H15 VDX"}="H152 VDX";
$resn_exch{"1H16 VDX"}="H161 VDX"; $resn_exch{"2H16 VDX"}="H162 VDX"; $resn_exch{"1H17 VDX"}="H171 VDX";
$resn_exch{"1H18 VDX"}="H181 VDX"; $resn_exch{"2H18 VDX"}="H182 VDX"; $resn_exch{"3H18 VDX"}="H183 VDX";
$resn_exch{"1H19 VDX"}="H191 VDX"; $resn_exch{"2H19 VDX"}="H192 VDX"; $resn_exch{"1H20 VDX"}="H201 VDX";
$resn_exch{"1H21 VDX"}="H211 VDX"; $resn_exch{"2H21 VDX"}="H212 VDX"; $resn_exch{"3H21 VDX"}="H213 VDX";
$resn_exch{"1H22 VDX"}="H221 VDX"; $resn_exch{"2H22 VDX"}="H222 VDX"; $resn_exch{"1H23 VDX"}="H231 VDX";
$resn_exch{"2H23 VDX"}="H232 VDX"; $resn_exch{"1H24 VDX"}="H241 VDX"; $resn_exch{"2H24 VDX"}="H242 VDX";
$resn_exch{"1H26 VDX"}="H261 VDX"; $resn_exch{"2H26 VDX"}="H262 VDX"; $resn_exch{"3H26 VDX"}="H263 VDX";
$resn_exch{"1H27 VDX"}="H271 VDX"; $resn_exch{"2H27 VDX"}="H272 VDX"; $resn_exch{"3H27 VDX"}="H273 VDX";
$resn_exch{"1H1  VDY"}=" H11 VDY"; $resn_exch{"2H1  VDY"}=" H12 VDY"; $resn_exch{"1H2  VDY"}=" H21 VDY";
$resn_exch{"2H2  VDY"}=" H22 VDY"; $resn_exch{"1H3  VDY"}=" H31 VDY"; $resn_exch{"1H4  VDY"}=" H41 VDY";
$resn_exch{"2H4  VDY"}=" H42 VDY"; $resn_exch{"1H6  VDY"}=" H61 VDY"; $resn_exch{"1H7  VDY"}=" H71 VDY";
$resn_exch{"1H9  VDY"}=" H91 VDY"; $resn_exch{"2H9  VDY"}=" H92 VDY"; $resn_exch{"1H11 VDY"}="H111 VDY";
$resn_exch{"2H11 VDY"}="H112 VDY"; $resn_exch{"1H12 VDY"}="H121 VDY"; $resn_exch{"2H12 VDY"}="H122 VDY";
$resn_exch{"1H14 VDY"}="H141 VDY"; $resn_exch{"1H15 VDY"}="H151 VDY"; $resn_exch{"2H15 VDY"}="H152 VDY";
$resn_exch{"1H16 VDY"}="H161 VDY"; $resn_exch{"2H16 VDY"}="H162 VDY"; $resn_exch{"1H17 VDY"}="H171 VDY";
$resn_exch{"1H18 VDY"}="H181 VDY"; $resn_exch{"2H18 VDY"}="H182 VDY"; $resn_exch{"3H18 VDY"}="H183 VDY";
$resn_exch{"1H19 VDY"}="H191 VDY"; $resn_exch{"2H19 VDY"}="H192 VDY"; $resn_exch{"1H20 VDY"}="H201 VDY";
$resn_exch{"1H21 VDY"}="H211 VDY"; $resn_exch{"2H21 VDY"}="H212 VDY"; $resn_exch{"3H21 VDY"}="H213 VDY";
$resn_exch{"1H22 VDY"}="H221 VDY"; $resn_exch{"2H22 VDY"}="H222 VDY"; $resn_exch{"1H23 VDY"}="H231 VDY";
$resn_exch{"2H23 VDY"}="H232 VDY"; $resn_exch{"1H24 VDY"}="H241 VDY"; $resn_exch{"2H24 VDY"}="H242 VDY";
$resn_exch{"1H26 VDY"}="H261 VDY"; $resn_exch{"2H26 VDY"}="H262 VDY"; $resn_exch{"3H26 VDY"}="H263 VDY";
$resn_exch{"1H27 VDY"}="H271 VDY"; $resn_exch{"2H27 VDY"}="H272 VDY"; $resn_exch{"3H27 VDY"}="H273 VDY";
$resn_exch{"1H28 VDZ"}="H281 VDZ"; $resn_exch{"2H28 VDZ"}="H282 VDZ"; $resn_exch{"1H4  VDZ"}=" H41 VDZ";
$resn_exch{"2H4  VDZ"}=" H42 VDZ"; $resn_exch{"1H10 VDZ"}="H101 VDZ"; $resn_exch{"2H10 VDZ"}="H102 VDZ";
$resn_exch{"1H9  VDZ"}=" H91 VDZ"; $resn_exch{"2H9  VDZ"}=" H92 VDZ"; $resn_exch{"1H11 VDZ"}="H111 VDZ";
$resn_exch{"2H11 VDZ"}="H112 VDZ"; $resn_exch{"1H12 VDZ"}="H121 VDZ"; $resn_exch{"2H12 VDZ"}="H122 VDZ";
$resn_exch{"1H18 VDZ"}="H181 VDZ"; $resn_exch{"2H18 VDZ"}="H182 VDZ"; $resn_exch{"3H18 VDZ"}="H183 VDZ";
$resn_exch{"1H15 VDZ"}="H151 VDZ"; $resn_exch{"2H15 VDZ"}="H152 VDZ"; $resn_exch{"1H16 VDZ"}="H161 VDZ";
$resn_exch{"2H16 VDZ"}="H162 VDZ"; $resn_exch{"1H21 VDZ"}="H211 VDZ"; $resn_exch{"2H21 VDZ"}="H212 VDZ";
$resn_exch{"3H21 VDZ"}="H213 VDZ"; $resn_exch{"1H22 VDZ"}="H221 VDZ"; $resn_exch{"2H22 VDZ"}="H222 VDZ";
$resn_exch{"1H23 VDZ"}="H231 VDZ"; $resn_exch{"2H23 VDZ"}="H232 VDZ"; $resn_exch{"1H24 VDZ"}="H241 VDZ";
$resn_exch{"2H24 VDZ"}="H242 VDZ"; $resn_exch{"1H26 VDZ"}="H261 VDZ"; $resn_exch{"2H26 VDZ"}="H262 VDZ";
$resn_exch{"3H26 VDZ"}="H263 VDZ"; $resn_exch{"1H27 VDZ"}="H271 VDZ"; $resn_exch{"2H27 VDZ"}="H272 VDZ";
$resn_exch{"3H27 VDZ"}="H273 VDZ"; $resn_exch{"1HMB VER"}="HMB1 VER"; $resn_exch{"2HMB VER"}="HMB2 VER";
$resn_exch{"3HMB VER"}="HMB3 VER"; $resn_exch{"1HBB VER"}="HBB1 VER"; $resn_exch{"2HBB VER"}="HBB2 VER";
$resn_exch{"1HMC VER"}="HMC1 VER"; $resn_exch{"2HMC VER"}="HMC2 VER"; $resn_exch{"3HMC VER"}="HMC3 VER";
$resn_exch{"1HBC VER"}="HBC1 VER"; $resn_exch{"2HBC VER"}="HBC2 VER"; $resn_exch{"1HMD VER"}="HMD1 VER";
$resn_exch{"2HMD VER"}="HMD2 VER"; $resn_exch{"3HMD VER"}="HMD3 VER"; $resn_exch{"1HAD VER"}="HAD1 VER";
$resn_exch{"2HAD VER"}="HAD2 VER"; $resn_exch{"1HBD VER"}="HBD1 VER"; $resn_exch{"2HBD VER"}="HBD2 VER";
$resn_exch{"1H1  VER"}=" H11 VER"; $resn_exch{"2H1  VER"}=" H12 VER"; $resn_exch{"3H1  VER"}=" H13 VER";
$resn_exch{"1H5C VG0"}="H5C1 VG0"; $resn_exch{"2H5C VG0"}="H5C2 VG0"; $resn_exch{"1H6C VG0"}="H6C1 VG0";
$resn_exch{"2H6C VG0"}="H6C2 VG0"; $resn_exch{"1H39 VG0"}="H391 VG0"; $resn_exch{"2H39 VG0"}="H392 VG0";
$resn_exch{"1H40 VG0"}="H401 VG0"; $resn_exch{"2H40 VG0"}="H402 VG0"; $resn_exch{"3H40 VG0"}="H403 VG0";
$resn_exch{"1H47 VG0"}="H471 VG0"; $resn_exch{"2H47 VG0"}="H472 VG0"; $resn_exch{"1H48 VG0"}="H481 VG0";
$resn_exch{"2H48 VG0"}="H482 VG0"; $resn_exch{"1H49 VG0"}="H491 VG0"; $resn_exch{"2H49 VG0"}="H492 VG0";
$resn_exch{"1H61 VG0"}="H611 VG0"; $resn_exch{"2H61 VG0"}="H612 VG0"; $resn_exch{"3H61 VG0"}="H613 VG0";
$resn_exch{"1H65 VG0"}="H651 VG0"; $resn_exch{"2H65 VG0"}="H652 VG0"; $resn_exch{"3H65 VG0"}="H653 VG0";
$resn_exch{"1H69 VG0"}="H691 VG0"; $resn_exch{"2H69 VG0"}="H692 VG0"; $resn_exch{"1H70 VG0"}="H701 VG0";
$resn_exch{"2H70 VG0"}="H702 VG0"; $resn_exch{"1H73 VG0"}="H731 VG0"; $resn_exch{"2H73 VG0"}="H732 VG0";
$resn_exch{"1H79 VG0"}="H791 VG0"; $resn_exch{"2H79 VG0"}="H792 VG0"; $resn_exch{"3H79 VG0"}="H793 VG0";
$resn_exch{"1H83 VG0"}="H831 VG0"; $resn_exch{"2H83 VG0"}="H832 VG0"; $resn_exch{"3H83 VG0"}="H833 VG0";
$resn_exch{"1HOP VG1"}="HOP1 VG1"; $resn_exch{"2HOP VG1"}="HOP2 VG1"; $resn_exch{"1H2C VG2"}="H2C1 VG2";
$resn_exch{"2H2C VG2"}="H2C2 VG2"; $resn_exch{"1H6C VG2"}="H6C1 VG2"; $resn_exch{"2H6C VG2"}="H6C2 VG2";
$resn_exch{"1H3C VG2"}="H3C1 VG2"; $resn_exch{"2H3C VG2"}="H3C2 VG2"; $resn_exch{"1H4C VG2"}="H4C1 VG2";
$resn_exch{"2H4C VG2"}="H4C2 VG2"; $resn_exch{"1H5C VG2"}="H5C1 VG2"; $resn_exch{"2H5C VG2"}="H5C2 VG2";
$resn_exch{"1H19 VG2"}="H191 VG2"; $resn_exch{"2H19 VG2"}="H192 VG2"; $resn_exch{"3H19 VG2"}="H193 VG2";
$resn_exch{"1H22 VG2"}="H221 VG2"; $resn_exch{"2H22 VG2"}="H222 VG2"; $resn_exch{"1H23 VG2"}="H231 VG2";
$resn_exch{"2H23 VG2"}="H232 VG2"; $resn_exch{"1H24 VG2"}="H241 VG2"; $resn_exch{"2H24 VG2"}="H242 VG2";
$resn_exch{"1H29 VG2"}="H291 VG2"; $resn_exch{"2H29 VG2"}="H292 VG2"; $resn_exch{"3H29 VG2"}="H293 VG2";
$resn_exch{"1H5C VG3"}="H5C1 VG3"; $resn_exch{"2H5C VG3"}="H5C2 VG3"; $resn_exch{"1H21 VG3"}="H211 VG3";
$resn_exch{"2H21 VG3"}="H212 VG3"; $resn_exch{"1H24 VG3"}="H241 VG3"; $resn_exch{"2H24 VG3"}="H242 VG3";
$resn_exch{"3H24 VG3"}="H243 VG3"; $resn_exch{"1H60 VG3"}="H601 VG3"; $resn_exch{"2H60 VG3"}="H602 VG3";
$resn_exch{"1H43 VG3"}="H431 VG3"; $resn_exch{"2H43 VG3"}="H432 VG3"; $resn_exch{"1H47 VG3"}="H471 VG3";
$resn_exch{"2H47 VG3"}="H472 VG3"; $resn_exch{"1H44 VG3"}="H441 VG3"; $resn_exch{"2H44 VG3"}="H442 VG3";
$resn_exch{"1H45 VG3"}="H451 VG3"; $resn_exch{"2H45 VG3"}="H452 VG3"; $resn_exch{"1H46 VG3"}="H461 VG3";
$resn_exch{"2H46 VG3"}="H462 VG3"; $resn_exch{"1H61 VG3"}="H611 VG3"; $resn_exch{"2H61 VG3"}="H612 VG3";
$resn_exch{"1H64 VG3"}="H641 VG3"; $resn_exch{"2H64 VG3"}="H642 VG3"; $resn_exch{"1H67 VG3"}="H671 VG3";
$resn_exch{"2H67 VG3"}="H672 VG3"; $resn_exch{"1H70 VG3"}="H701 VG3"; $resn_exch{"2H70 VG3"}="H702 VG3";
$resn_exch{"3H70 VG3"}="H703 VG3"; $resn_exch{"1H11 VG4"}="H111 VG4"; $resn_exch{"2H11 VG4"}="H112 VG4";
$resn_exch{"1H15 VG4"}="H151 VG4"; $resn_exch{"2H15 VG4"}="H152 VG4"; $resn_exch{"1H22 VG4"}="H221 VG4";
$resn_exch{"2H22 VG4"}="H222 VG4"; $resn_exch{"3H22 VG4"}="H223 VG4"; $resn_exch{"1H23 VG4"}="H231 VG4";
$resn_exch{"2H23 VG4"}="H232 VG4"; $resn_exch{"1H27 VG4"}="H271 VG4"; $resn_exch{"2H27 VG4"}="H272 VG4";
$resn_exch{"1H24 VG4"}="H241 VG4"; $resn_exch{"2H24 VG4"}="H242 VG4"; $resn_exch{"1H25 VG4"}="H251 VG4";
$resn_exch{"2H25 VG4"}="H252 VG4"; $resn_exch{"1H26 VG4"}="H261 VG4"; $resn_exch{"2H26 VG4"}="H262 VG4";
$resn_exch{"1H67 VG4"}="H671 VG4"; $resn_exch{"2H67 VG4"}="H672 VG4"; $resn_exch{"1H68 VG4"}="H681 VG4";
$resn_exch{"2H68 VG4"}="H682 VG4"; $resn_exch{"1H71 VG4"}="H711 VG4"; $resn_exch{"2H71 VG4"}="H712 VG4";
$resn_exch{"3H71 VG4"}="H713 VG4"; $resn_exch{"1H80 VG4"}="H801 VG4"; $resn_exch{"2H80 VG4"}="H802 VG4";
$resn_exch{"1H11 VG5"}="H111 VG5"; $resn_exch{"2H11 VG5"}="H112 VG5"; $resn_exch{"1H15 VG5"}="H151 VG5";
$resn_exch{"2H15 VG5"}="H152 VG5"; $resn_exch{"1H22 VG5"}="H221 VG5"; $resn_exch{"2H22 VG5"}="H222 VG5";
$resn_exch{"3H22 VG5"}="H223 VG5"; $resn_exch{"1H23 VG5"}="H231 VG5"; $resn_exch{"2H23 VG5"}="H232 VG5";
$resn_exch{"1H27 VG5"}="H271 VG5"; $resn_exch{"2H27 VG5"}="H272 VG5"; $resn_exch{"1H24 VG5"}="H241 VG5";
$resn_exch{"2H24 VG5"}="H242 VG5"; $resn_exch{"1H25 VG5"}="H251 VG5"; $resn_exch{"2H25 VG5"}="H252 VG5";
$resn_exch{"1H26 VG5"}="H261 VG5"; $resn_exch{"2H26 VG5"}="H262 VG5"; $resn_exch{"1H67 VG5"}="H671 VG5";
$resn_exch{"2H67 VG5"}="H672 VG5"; $resn_exch{"1H68 VG5"}="H681 VG5"; $resn_exch{"2H68 VG5"}="H682 VG5";
$resn_exch{"3H68 VG5"}="H683 VG5"; $resn_exch{"1H80 VG5"}="H801 VG5"; $resn_exch{"2H80 VG5"}="H802 VG5";
$resn_exch{"1H11 VG6"}="H111 VG6"; $resn_exch{"2H11 VG6"}="H112 VG6"; $resn_exch{"1H15 VG6"}="H151 VG6";
$resn_exch{"2H15 VG6"}="H152 VG6"; $resn_exch{"1H59 VG6"}="H591 VG6"; $resn_exch{"2H59 VG6"}="H592 VG6";
$resn_exch{"1H39 VG6"}="H391 VG6"; $resn_exch{"2H39 VG6"}="H392 VG6"; $resn_exch{"1H41 VG6"}="H411 VG6";
$resn_exch{"2H41 VG6"}="H412 VG6"; $resn_exch{"3H41 VG6"}="H413 VG6"; $resn_exch{"1H51 VG6"}="H511 VG6";
$resn_exch{"2H51 VG6"}="H512 VG6"; $resn_exch{"1H49 VG6"}="H491 VG6"; $resn_exch{"2H49 VG6"}="H492 VG6";
$resn_exch{"1H50 VG6"}="H501 VG6"; $resn_exch{"2H50 VG6"}="H502 VG6"; $resn_exch{"1H11 VG7"}="H111 VG7";
$resn_exch{"2H11 VG7"}="H112 VG7"; $resn_exch{"1H15 VG7"}="H151 VG7"; $resn_exch{"2H15 VG7"}="H152 VG7";
$resn_exch{"1H39 VG7"}="H391 VG7"; $resn_exch{"2H39 VG7"}="H392 VG7"; $resn_exch{"1H41 VG7"}="H411 VG7";
$resn_exch{"2H41 VG7"}="H412 VG7"; $resn_exch{"3H41 VG7"}="H413 VG7"; $resn_exch{"1H51 VG7"}="H511 VG7";
$resn_exch{"2H51 VG7"}="H512 VG7"; $resn_exch{"1H49 VG7"}="H491 VG7"; $resn_exch{"2H49 VG7"}="H492 VG7";
$resn_exch{"1H50 VG7"}="H501 VG7"; $resn_exch{"2H50 VG7"}="H502 VG7"; $resn_exch{"1H61 VG7"}="H611 VG7";
$resn_exch{"2H61 VG7"}="H612 VG7"; $resn_exch{"1H65 VG7"}="H651 VG7"; $resn_exch{"2H65 VG7"}="H652 VG7";
$resn_exch{"1H62 VG7"}="H621 VG7"; $resn_exch{"2H62 VG7"}="H622 VG7"; $resn_exch{"1H63 VG7"}="H631 VG7";
$resn_exch{"2H63 VG7"}="H632 VG7"; $resn_exch{"1H64 VG7"}="H641 VG7"; $resn_exch{"2H64 VG7"}="H642 VG7";
$resn_exch{"1H34 VIA"}="H341 VIA"; $resn_exch{"2H34 VIA"}="H342 VIA"; $resn_exch{"3H34 VIA"}="H343 VIA";
$resn_exch{"1H33 VIA"}="H331 VIA"; $resn_exch{"2H33 VIA"}="H332 VIA"; $resn_exch{"1H32 VIA"}="H321 VIA";
$resn_exch{"2H32 VIA"}="H322 VIA"; $resn_exch{"1H31 VIA"}="H311 VIA"; $resn_exch{"2H31 VIA"}="H312 VIA";
$resn_exch{"3H31 VIA"}="H313 VIA"; $resn_exch{"1H2  VIA"}=" H21 VIA"; $resn_exch{"2H2  VIA"}="H22A VIA";
$resn_exch{"1H1  VIA"}=" H11 VIA"; $resn_exch{"2H1  VIA"}=" H12 VIA"; $resn_exch{"3H1  VIA"}=" H13 VIA";
$resn_exch{"1H19 VIA"}="H191 VIA"; $resn_exch{"2H19 VIA"}="H192 VIA"; $resn_exch{"1H18 VIA"}="H181 VIA";
$resn_exch{"2H18 VIA"}="H182 VIA"; $resn_exch{"1H20 VIA"}="H201 VIA"; $resn_exch{"2H20 VIA"}="H202 VIA";
$resn_exch{"3H20 VIA"}="H203 VIA"; $resn_exch{"1H16 VIA"}="H161 VIA"; $resn_exch{"2H16 VIA"}="H162 VIA";
$resn_exch{"1H15 VIA"}="H151 VIA"; $resn_exch{"2H15 VIA"}="H152 VIA"; $resn_exch{"1HM2 VIB"}="HM21 VIB";
$resn_exch{"2HM2 VIB"}="HM22 VIB"; $resn_exch{"3HM2 VIB"}="HM23 VIB"; $resn_exch{"1H4A VIB"}="H4A1 VIB";
$resn_exch{"2H4A VIB"}="H4A2 VIB"; $resn_exch{"1HM4 VIB"}="HM41 VIB"; $resn_exch{"2HM4 VIB"}="HM42 VIB";
$resn_exch{"3HM4 VIB"}="HM43 VIB"; $resn_exch{"1HC7 VIB"}="HC71 VIB"; $resn_exch{"2HC7 VIB"}="HC72 VIB";
$resn_exch{"1HC6 VIB"}="HC61 VIB"; $resn_exch{"2HC6 VIB"}="HC62 VIB"; $resn_exch{"1H7A VIB"}="H7A1 VIB";
$resn_exch{"2H7A VIB"}="H7A2 VIB"; $resn_exch{"CH6A VIB"}="H6AC VIB"; $resn_exch{"1H1C VIG"}="H1C1 VIG";
$resn_exch{"2H1C VIG"}="H1C2 VIG"; $resn_exch{"1H2C VIG"}="H2C1 VIG"; $resn_exch{"2H2C VIG"}="H2C2 VIG";
$resn_exch{"1H4C VIG"}="H4C1 VIG"; $resn_exch{"2H4C VIG"}="H4C2 VIG"; $resn_exch{"1H5C VIG"}="H5C1 VIG";
$resn_exch{"2H5C VIG"}="H5C2 VIG"; $resn_exch{"3H5C VIG"}="H5C3 VIG"; $resn_exch{"1H3N VIG"}="H3N1 VIG";
$resn_exch{"2H3N VIG"}="H3N2 VIG"; $resn_exch{"1HB  VIO"}=" HB1 VIO"; $resn_exch{"2HB  VIO"}=" HB2 VIO";
$resn_exch{"1HG  VIO"}=" HG1 VIO"; $resn_exch{"2HG  VIO"}=" HG2 VIO"; $resn_exch{"1HD  VIO"}=" HD1 VIO";
$resn_exch{"2HD  VIO"}=" HD2 VIO"; $resn_exch{"1HH1 VIO"}="HH11 VIO"; $resn_exch{"2HH1 VIO"}="HH12 VIO";
$resn_exch{"1HH2 VIO"}="HH21 VIO"; $resn_exch{"2HH2 VIO"}="HH22 VIO"; $resn_exch{"1H1  VIO"}=" H11 VIO";
$resn_exch{"2H1  VIO"}=" H12 VIO"; $resn_exch{"1H2  VIO"}=" H21 VIO"; $resn_exch{"2H2  VIO"}=" H22 VIO";
$resn_exch{"3H2  VIO"}=" H23 VIO"; $resn_exch{"1H13 VIR"}="H131 VIR"; $resn_exch{"2H13 VIR"}="H132 VIR";
$resn_exch{"1H16 VIR"}="H161 VIR"; $resn_exch{"2H16 VIR"}="H162 VIR"; $resn_exch{"1H21 VIR"}="H211 VIR";
$resn_exch{"2H21 VIR"}="H212 VIR"; $resn_exch{"3H21 VIR"}="H213 VIR"; $resn_exch{"1H24 VIR"}="H241 VIR";
$resn_exch{"2H24 VIR"}="H242 VIR"; $resn_exch{"1HC3 VIR"}="HC31 VIR"; $resn_exch{"2HC3 VIR"}="HC32 VIR";
$resn_exch{"1H31 VIR"}="H311 VIR"; $resn_exch{"2H31 VIR"}="H312 VIR"; $resn_exch{"3H31 VIR"}="H313 VIR";
$resn_exch{"1H34 VIR"}="H341 VIR"; $resn_exch{"2H34 VIR"}="H342 VIR"; $resn_exch{"3H34 VIR"}="H343 VIR";
$resn_exch{"1H35 VIR"}="H351 VIR"; $resn_exch{"2H35 VIR"}="H352 VIR"; $resn_exch{"3H35 VIR"}="H353 VIR";
$resn_exch{"1HC4 VIR"}="HC41 VIR"; $resn_exch{"2HC4 VIR"}="HC42 VIR"; $resn_exch{"'H18 VIR"}="H18' VIR";
$resn_exch{"1HC1 VIT"}="HC11 VIT"; $resn_exch{"2HC1 VIT"}="HC12 VIT"; $resn_exch{"3HC1 VIT"}="HC13 VIT";
$resn_exch{"1HC8 VIT"}="HC81 VIT"; $resn_exch{"2HC8 VIT"}="HC82 VIT"; $resn_exch{"3HC8 VIT"}="HC83 VIT";
$resn_exch{"1H10 VIT"}="H101 VIT"; $resn_exch{"2H10 VIT"}="H102 VIT"; $resn_exch{"1H11 VIT"}="H111 VIT";
$resn_exch{"2H11 VIT"}="H112 VIT"; $resn_exch{"1H12 VIT"}="H121 VIT"; $resn_exch{"2H12 VIT"}="H122 VIT";
$resn_exch{"3H12 VIT"}="H123 VIT"; $resn_exch{"1H13 VIT"}="H131 VIT"; $resn_exch{"2H13 VIT"}="H132 VIT";
$resn_exch{"3H13 VIT"}="H133 VIT"; $resn_exch{"1HO2 VIT"}="HO21 VIT"; $resn_exch{"1H14 VIT"}="H141 VIT";
$resn_exch{"2H14 VIT"}="H142 VIT"; $resn_exch{"3H14 VIT"}="H143 VIT"; $resn_exch{"1H15 VIT"}="H151 VIT";
$resn_exch{"2H15 VIT"}="H152 VIT"; $resn_exch{"1H16 VIT"}="H161 VIT"; $resn_exch{"2H16 VIT"}="H162 VIT";
$resn_exch{"1H17 VIT"}="H171 VIT"; $resn_exch{"2H17 VIT"}="H172 VIT"; $resn_exch{"1H19 VIT"}="H191 VIT";
$resn_exch{"2H19 VIT"}="H192 VIT"; $resn_exch{"3H19 VIT"}="H193 VIT"; $resn_exch{"1H20 VIT"}="H201 VIT";
$resn_exch{"2H20 VIT"}="H202 VIT"; $resn_exch{"1H21 VIT"}="H211 VIT"; $resn_exch{"2H21 VIT"}="H212 VIT";
$resn_exch{"1H22 VIT"}="H221 VIT"; $resn_exch{"2H22 VIT"}="H222 VIT"; $resn_exch{"1H24 VIT"}="H241 VIT";
$resn_exch{"2H24 VIT"}="H242 VIT"; $resn_exch{"1H25 VIT"}="H251 VIT"; $resn_exch{"2H25 VIT"}="H252 VIT";
$resn_exch{"1H26 VIT"}="H261 VIT"; $resn_exch{"2H26 VIT"}="H262 VIT"; $resn_exch{"1H28 VIT"}="H281 VIT";
$resn_exch{"2H28 VIT"}="H282 VIT"; $resn_exch{"3H28 VIT"}="H283 VIT"; $resn_exch{"1H29 VIT"}="H291 VIT";
$resn_exch{"2H29 VIT"}="H292 VIT"; $resn_exch{"3H29 VIT"}="H293 VIT"; $resn_exch{"1HO2 VIV"}="HO21 VIV";
$resn_exch{"1HC1 VIV"}="HC11 VIV"; $resn_exch{"2HC1 VIV"}="HC12 VIV"; $resn_exch{"3HC1 VIV"}="HC13 VIV";
$resn_exch{"1HC8 VIV"}="HC81 VIV"; $resn_exch{"2HC8 VIV"}="HC82 VIV"; $resn_exch{"3HC8 VIV"}="HC83 VIV";
$resn_exch{"1H10 VIV"}="H101 VIV"; $resn_exch{"2H10 VIV"}="H102 VIV"; $resn_exch{"1H11 VIV"}="H111 VIV";
$resn_exch{"2H11 VIV"}="H112 VIV"; $resn_exch{"1H12 VIV"}="H121 VIV"; $resn_exch{"2H12 VIV"}="H122 VIV";
$resn_exch{"3H12 VIV"}="H123 VIV"; $resn_exch{"1H13 VIV"}="H131 VIV"; $resn_exch{"2H13 VIV"}="H132 VIV";
$resn_exch{"3H13 VIV"}="H133 VIV"; $resn_exch{"1H14 VIV"}="H141 VIV"; $resn_exch{"2H14 VIV"}="H142 VIV";
$resn_exch{"3H14 VIV"}="H143 VIV"; $resn_exch{"1H15 VIV"}="H151 VIV"; $resn_exch{"2H15 VIV"}="H152 VIV";
$resn_exch{"1H16 VIV"}="H161 VIV"; $resn_exch{"2H16 VIV"}="H162 VIV"; $resn_exch{"1H17 VIV"}="H171 VIV";
$resn_exch{"2H17 VIV"}="H172 VIV"; $resn_exch{"1H19 VIV"}="H191 VIV"; $resn_exch{"2H19 VIV"}="H192 VIV";
$resn_exch{"3H19 VIV"}="H193 VIV"; $resn_exch{"1H20 VIV"}="H201 VIV"; $resn_exch{"2H20 VIV"}="H202 VIV";
$resn_exch{"1H21 VIV"}="H211 VIV"; $resn_exch{"2H21 VIV"}="H212 VIV"; $resn_exch{"1H22 VIV"}="H221 VIV";
$resn_exch{"2H22 VIV"}="H222 VIV"; $resn_exch{"1H24 VIV"}="H241 VIV"; $resn_exch{"2H24 VIV"}="H242 VIV";
$resn_exch{"1H25 VIV"}="H251 VIV"; $resn_exch{"2H25 VIV"}="H252 VIV"; $resn_exch{"1H26 VIV"}="H261 VIV";
$resn_exch{"2H26 VIV"}="H262 VIV"; $resn_exch{"1H28 VIV"}="H281 VIV"; $resn_exch{"2H28 VIV"}="H282 VIV";
$resn_exch{"3H28 VIV"}="H283 VIV"; $resn_exch{"1H29 VIV"}="H291 VIV"; $resn_exch{"2H29 VIV"}="H292 VIV";
$resn_exch{"3H29 VIV"}="H293 VIV"; $resn_exch{"1H2K VK3"}="H2K1 VK3"; $resn_exch{"1H6K VK3"}="H6K1 VK3";
$resn_exch{"1H7K VK3"}="H7K1 VK3"; $resn_exch{"1H8K VK3"}="H8K1 VK3"; $resn_exch{"1H9K VK3"}="H9K1 VK3";
$resn_exch{"1H11 VK3"}="H111 VK3"; $resn_exch{"2H11 VK3"}="H112 VK3"; $resn_exch{"3H11 VK3"}="H113 VK3";
$resn_exch{"1H71 VLB"}="H711 VLB"; $resn_exch{"2H71 VLB"}="H712 VLB"; $resn_exch{"3H71 VLB"}="H713 VLB";
$resn_exch{"1H70 VLB"}="H701 VLB"; $resn_exch{"2H70 VLB"}="H702 VLB"; $resn_exch{"1H55 VLB"}="H551 VLB";
$resn_exch{"2H55 VLB"}="H552 VLB"; $resn_exch{"1H53 VLB"}="H531 VLB"; $resn_exch{"2H53 VLB"}="H532 VLB";
$resn_exch{"1H51 VLB"}="H511 VLB"; $resn_exch{"2H51 VLB"}="H512 VLB"; $resn_exch{"1H69 VLB"}="H691 VLB";
$resn_exch{"2H69 VLB"}="H692 VLB"; $resn_exch{"1H57 VLB"}="H571 VLB"; $resn_exch{"2H57 VLB"}="H572 VLB";
$resn_exch{"1H58 VLB"}="H581 VLB"; $resn_exch{"2H58 VLB"}="H582 VLB"; $resn_exch{"1H76 VLB"}="H761 VLB";
$resn_exch{"2H76 VLB"}="H762 VLB"; $resn_exch{"3H76 VLB"}="H763 VLB"; $resn_exch{"1H11 VLB"}="H111 VLB";
$resn_exch{"2H11 VLB"}="H112 VLB"; $resn_exch{"1H10 VLB"}="H101 VLB"; $resn_exch{"2H10 VLB"}="H102 VLB";
$resn_exch{"1H8  VLB"}=" H81 VLB"; $resn_exch{"2H8  VLB"}=" H82 VLB"; $resn_exch{"1H20 VLB"}="H201 VLB";
$resn_exch{"2H20 VLB"}="H202 VLB"; $resn_exch{"1H21 VLB"}="H211 VLB"; $resn_exch{"2H21 VLB"}="H212 VLB";
$resn_exch{"3H21 VLB"}="H213 VLB"; $resn_exch{"1H30 VLB"}="H301 VLB"; $resn_exch{"2H30 VLB"}="H302 VLB";
$resn_exch{"3H30 VLB"}="H303 VLB"; $resn_exch{"1H26 VLB"}="H261 VLB"; $resn_exch{"2H26 VLB"}="H262 VLB";
$resn_exch{"3H26 VLB"}="H263 VLB"; $resn_exch{"1H22 VLB"}="H221 VLB"; $resn_exch{"2H22 VLB"}="H222 VLB";
$resn_exch{"3H22 VLB"}="H223 VLB"; $resn_exch{"1H33 VLB"}="H331 VLB"; $resn_exch{"2H33 VLB"}="H332 VLB";
$resn_exch{"3H33 VLB"}="H333 VLB"; $resn_exch{"1HN  VLL"}=" HN1 VLL"; $resn_exch{"2HN  VLL"}=" HN2 VLL";
$resn_exch{"1HG1 VLL"}="HG11 VLL"; $resn_exch{"2HG1 VLL"}="HG12 VLL"; $resn_exch{"3HG1 VLL"}="HG13 VLL";
$resn_exch{"1HG2 VLL"}="HG21 VLL"; $resn_exch{"2HG2 VLL"}="HG22 VLL"; $resn_exch{"1HN  VLM"}=" H   VLM";
$resn_exch{"2HN  VLM"}=" H2  VLM"; $resn_exch{"1HG1 VLM"}="HG11 VLM"; $resn_exch{"2HG1 VLM"}="HG12 VLM";
$resn_exch{"3HG1 VLM"}="HG13 VLM"; $resn_exch{"1HG2 VLM"}="HG21 VLM"; $resn_exch{"2HG2 VLM"}="HG22 VLM";
$resn_exch{"3HG2 VLM"}="HG23 VLM"; $resn_exch{"1HNT VLM"}="HNT1 VLM"; $resn_exch{"2HNT VLM"}="HNT2 VLM";
$resn_exch{"1HN  VME"}=" HN1 VME"; $resn_exch{"2HN  VME"}=" HN2 VME"; $resn_exch{"1HG1 VME"}="HG11 VME";
$resn_exch{"2HG1 VME"}="HG12 VME"; $resn_exch{"3HG1 VME"}="HG13 VME"; $resn_exch{"1HG2 VME"}="HG21 VME";
$resn_exch{"2HG2 VME"}="HG22 VME"; $resn_exch{"3HG2 VME"}="HG23 VME"; $resn_exch{"1HMC VME"}="HMC1 VME";
$resn_exch{"2HMC VME"}="HMC2 VME"; $resn_exch{"3HMC VME"}="HMC3 VME"; $resn_exch{"1HG2 VMS"}="HG21 VMS";
$resn_exch{"2HG2 VMS"}="HG22 VMS"; $resn_exch{"3HG2 VMS"}="HG23 VMS"; $resn_exch{"1HG1 VMS"}="HG11 VMS";
$resn_exch{"2HG1 VMS"}="HG12 VMS"; $resn_exch{"3HG1 VMS"}="HG13 VMS"; $resn_exch{"1HN  VMS"}=" H   VMS";
$resn_exch{"2HN  VMS"}=" H2  VMS"; $resn_exch{"1H5* VMS"}="H5'1 VMS"; $resn_exch{"2H5* VMS"}="H5'2 VMS";
$resn_exch{" H4* VMS"}=" H4' VMS"; $resn_exch{" H3* VMS"}=" H3' VMS"; $resn_exch{"H2   VMS"}="HO3' VMS";
$resn_exch{" H2* VMS"}=" H2' VMS"; $resn_exch{" H1* VMS"}=" H1' VMS"; $resn_exch{"1HN6 VMS"}="HN61 VMS";
$resn_exch{"2HN6 VMS"}="HN62 VMS"; $resn_exch{"1HV  VNL"}=" HV1 VNL"; $resn_exch{"2HV  VNL"}=" HV2 VNL";
$resn_exch{"3HV  VNL"}=" HV3 VNL"; $resn_exch{"1HN  VOL"}=" HN1 VOL"; $resn_exch{"2HN  VOL"}=" HN2 VOL";
$resn_exch{"1H5  VPR"}=" H51 VPR"; $resn_exch{"2H5  VPR"}=" H52 VPR"; $resn_exch{"3H5  VPR"}=" H53 VPR";
$resn_exch{"1H4  VPR"}=" H41 VPR"; $resn_exch{"2H4  VPR"}=" H42 VPR"; $resn_exch{"1H3  VPR"}=" H31 VPR";
$resn_exch{"2H3  VPR"}=" H32 VPR"; $resn_exch{"1H3A VPR"}="H3A1 VPR"; $resn_exch{"2H3A VPR"}="H3A2 VPR";
$resn_exch{"1H4A VPR"}="H4A1 VPR"; $resn_exch{"2H4A VPR"}="H4A2 VPR"; $resn_exch{"1H5A VPR"}="H5A1 VPR";
$resn_exch{"2H5A VPR"}="H5A2 VPR"; $resn_exch{"3H5A VPR"}="H5A3 VPR"; $resn_exch{"1HN  VPR"}=" HN1 VPR";
$resn_exch{"2HN  VPR"}=" HN2 VPR"; $resn_exch{"1H6  VR1"}=" H61 VR1"; $resn_exch{"2H6  VR1"}=" H62 VR1";
$resn_exch{"1H7  VR1"}=" H71 VR1"; $resn_exch{"2H7  VR1"}=" H72 VR1"; $resn_exch{"3H7  VR1"}=" H73 VR1";
$resn_exch{"1H8  VR1"}=" H81 VR1"; $resn_exch{"1H3  VRS"}="H31A VRS"; $resn_exch{"2H3  VRS"}="H32A VRS";
$resn_exch{"1H4  VRS"}=" H41 VRS"; $resn_exch{"2H4  VRS"}="H42A VRS"; $resn_exch{"3H4  VRS"}="H43A VRS";
$resn_exch{"1H7  VRS"}=" H71 VRS"; $resn_exch{"2H7  VRS"}=" H72 VRS"; $resn_exch{"1H8  VRS"}=" H81 VRS";
$resn_exch{"2H8  VRS"}=" H82 VRS"; $resn_exch{"1H9  VRS"}=" H91 VRS"; $resn_exch{"2H9  VRS"}=" H92 VRS";
$resn_exch{"1H12 VRS"}="H121 VRS"; $resn_exch{"2H12 VRS"}="H122 VRS"; $resn_exch{"3H12 VRS"}="H123 VRS";
$resn_exch{"1H13 VRS"}="H131 VRS"; $resn_exch{"2H13 VRS"}="H132 VRS"; $resn_exch{"1H24 VRS"}="H241 VRS";
$resn_exch{"2H24 VRS"}="H242 VRS"; $resn_exch{"1H25 VRS"}="H251 VRS"; $resn_exch{"2H25 VRS"}="H252 VRS";
$resn_exch{"1H27 VRS"}="H271 VRS"; $resn_exch{"2H27 VRS"}="H272 VRS"; $resn_exch{"1H39 VRS"}="H391 VRS";
$resn_exch{"2H39 VRS"}="H392 VRS"; $resn_exch{"3H39 VRS"}="H393 VRS"; $resn_exch{"1H26 VRX"}="H261 VRX";
$resn_exch{"2H26 VRX"}="H262 VRX"; $resn_exch{"3H26 VRX"}="H263 VRX"; $resn_exch{"1H25 VRX"}="H251 VRX";
$resn_exch{"2H25 VRX"}="H252 VRX"; $resn_exch{"1H3  VS1"}=" H31 VS1"; $resn_exch{"1H12 VS1"}="H121 VS1";
$resn_exch{"2H12 VS1"}="H122 VS1"; $resn_exch{"1H15 VS1"}="H151 VS1"; $resn_exch{"1H16 VS1"}="H161 VS1";
$resn_exch{"1H17 VS1"}="H171 VS1"; $resn_exch{"1H18 VS1"}="H181 VS1"; $resn_exch{"1H19 VS1"}="H191 VS1";
$resn_exch{"1H21 VS1"}="H211 VS1"; $resn_exch{"1H23 VS1"}="H231 VS1"; $resn_exch{"2H23 VS1"}="H232 VS1";
$resn_exch{"1H24 VS1"}="H241 VS1"; $resn_exch{"2H24 VS1"}="H242 VS1"; $resn_exch{"1H30 VS1"}="H301 VS1";
$resn_exch{"1H31 VS1"}="H311 VS1"; $resn_exch{"1H32 VS1"}="H321 VS1"; $resn_exch{"1H33 VS1"}="H331 VS1";
$resn_exch{"1H34 VS1"}="H341 VS1"; $resn_exch{"1H25 VS1"}="H251 VS1"; $resn_exch{"2H25 VS1"}="H252 VS1";
$resn_exch{"1H36 VS1"}="H361 VS1"; $resn_exch{"2H37 VS1"}="H372 VS1"; $resn_exch{"1H38 VS1"}="H381 VS1";
$resn_exch{"1H39 VS1"}="H391 VS1"; $resn_exch{"1H40 VS1"}="H401 VS1"; $resn_exch{"1H5  VS1"}=" H51 VS1";
$resn_exch{"2H5  VS1"}=" H52 VS1"; $resn_exch{"1H6  VS1"}=" H61 VS1"; $resn_exch{"2H6  VS1"}=" H62 VS1";
$resn_exch{"1H8  VS1"}=" H81 VS1"; $resn_exch{"2H8  VS1"}=" H82 VS1"; $resn_exch{"1H9  VS1"}=" H91 VS1";
$resn_exch{"2H9  VS1"}=" H92 VS1"; $resn_exch{"1H22 VS1"}="H221 VS1"; $resn_exch{"2H22 VS1"}="H222 VS1";
$resn_exch{"1H2  VS2"}=" H21 VS2"; $resn_exch{"1H3  VS2"}=" H31 VS2"; $resn_exch{"1H4  VS2"}=" H41 VS2";
$resn_exch{"1H5  VS2"}=" H51 VS2"; $resn_exch{"1H6  VS2"}=" H61 VS2"; $resn_exch{"1H7  VS2"}=" H71 VS2";
$resn_exch{"2H7  VS2"}=" H72 VS2"; $resn_exch{"1HN1 VS2"}="HN11 VS2"; $resn_exch{"1H9  VS2"}=" H91 VS2";
$resn_exch{"1H10 VS2"}="H101 VS2"; $resn_exch{"2H10 VS2"}="H102 VS2"; $resn_exch{"1H12 VS2"}="H121 VS2";
$resn_exch{"1H13 VS2"}="H131 VS2"; $resn_exch{"1H14 VS2"}="H141 VS2"; $resn_exch{"1H15 VS2"}="H151 VS2";
$resn_exch{"1H16 VS2"}="H161 VS2"; $resn_exch{"1HN2 VS2"}="HN21 VS2"; $resn_exch{"1H18 VS2"}="H181 VS2";
$resn_exch{"1H19 VS2"}="H191 VS2"; $resn_exch{"2H19 VS2"}="H192 VS2"; $resn_exch{"1H20 VS2"}="H201 VS2";
$resn_exch{"2H20 VS2"}="H202 VS2"; $resn_exch{"1H22 VS2"}="H221 VS2"; $resn_exch{"1H23 VS2"}="H231 VS2";
$resn_exch{"1H24 VS2"}="H241 VS2"; $resn_exch{"1H25 VS2"}="H251 VS2"; $resn_exch{"1H26 VS2"}="H261 VS2";
$resn_exch{"1H28 VS2"}="H281 VS2"; $resn_exch{"2H28 VS2"}="H282 VS2"; $resn_exch{"1H29 VS2"}="H291 VS2";
$resn_exch{"2H29 VS2"}="H292 VS2"; $resn_exch{"1H31 VS2"}="H311 VS2"; $resn_exch{"1H32 VS2"}="H321 VS2";
$resn_exch{"1H33 VS2"}="H331 VS2"; $resn_exch{"1H34 VS2"}="H341 VS2"; $resn_exch{"1H35 VS2"}="H351 VS2";
$resn_exch{"1H27 VS2"}="H271 VS2"; $resn_exch{"2H27 VS2"}="H272 VS2"; $resn_exch{"1H2  VS3"}=" H21 VS3";
$resn_exch{"1H3  VS3"}=" H31 VS3"; $resn_exch{"1H4  VS3"}=" H41 VS3"; $resn_exch{"1H5  VS3"}=" H51 VS3";
$resn_exch{"1H6  VS3"}=" H61 VS3"; $resn_exch{"1H7  VS3"}=" H71 VS3"; $resn_exch{"2H7  VS3"}=" H72 VS3";
$resn_exch{"1HN1 VS3"}="HN11 VS3"; $resn_exch{"1H9  VS3"}=" H91 VS3"; $resn_exch{"1H10 VS3"}="H101 VS3";
$resn_exch{"2H10 VS3"}="H102 VS3"; $resn_exch{"1H12 VS3"}="H121 VS3"; $resn_exch{"1H13 VS3"}="H131 VS3";
$resn_exch{"1H14 VS3"}="H141 VS3"; $resn_exch{"1H15 VS3"}="H151 VS3"; $resn_exch{"1H16 VS3"}="H161 VS3";
$resn_exch{"1HN2 VS3"}="HN21 VS3"; $resn_exch{"1H18 VS3"}="H181 VS3"; $resn_exch{"1H19 VS3"}="H191 VS3";
$resn_exch{"2H19 VS3"}="H192 VS3"; $resn_exch{"1H20 VS3"}="H201 VS3"; $resn_exch{"2H20 VS3"}="H202 VS3";
$resn_exch{"1H22 VS3"}="H221 VS3"; $resn_exch{"1H23 VS3"}="H231 VS3"; $resn_exch{"1H24 VS3"}="H241 VS3";
$resn_exch{"1H25 VS3"}="H251 VS3"; $resn_exch{"1H26 VS3"}="H261 VS3"; $resn_exch{"1H28 VS3"}="H281 VS3";
$resn_exch{"2H28 VS3"}="H282 VS3"; $resn_exch{"1H31 VS3"}="H311 VS3"; $resn_exch{"1H32 VS3"}="H321 VS3";
$resn_exch{"1H34 VS3"}="H341 VS3"; $resn_exch{"1H35 VS3"}="H351 VS3"; $resn_exch{"1H27 VS3"}="H271 VS3";
$resn_exch{"2H27 VS3"}="H272 VS3"; $resn_exch{"1H39 VS4"}="H391 VS4"; $resn_exch{"2H39 VS4"}="H392 VS4";
$resn_exch{"1H40 VS4"}="H401 VS4"; $resn_exch{"2H40 VS4"}="H402 VS4"; $resn_exch{"1H33 VS4"}="H331 VS4";
$resn_exch{"2H33 VS4"}="H332 VS4"; $resn_exch{"3H33 VS4"}="H333 VS4"; $resn_exch{"1H49 VS4"}="H491 VS4";
$resn_exch{"2H49 VS4"}="H492 VS4"; $resn_exch{"1H48 VS4"}="H481 VS4"; $resn_exch{"2H48 VS4"}="H482 VS4";
$resn_exch{"1HN1 VS4"}="HN11 VS4"; $resn_exch{"1H9  VS4"}=" H91 VS4"; $resn_exch{"1H10 VS4"}="H101 VS4";
$resn_exch{"2H10 VS4"}="H102 VS4"; $resn_exch{"1H12 VS4"}="H121 VS4"; $resn_exch{"1H13 VS4"}="H131 VS4";
$resn_exch{"1H14 VS4"}="H141 VS4"; $resn_exch{"1H15 VS4"}="H151 VS4"; $resn_exch{"1H16 VS4"}="H161 VS4";
$resn_exch{"1HN2 VS4"}="HN21 VS4"; $resn_exch{"1H18 VS4"}="H181 VS4"; $resn_exch{"1H19 VS4"}="H191 VS4";
$resn_exch{"2H19 VS4"}="H192 VS4"; $resn_exch{"1H20 VS4"}="H201 VS4"; $resn_exch{"2H20 VS4"}="H202 VS4";
$resn_exch{"1H22 VS4"}="H221 VS4"; $resn_exch{"1H23 VS4"}="H231 VS4"; $resn_exch{"1H24 VS4"}="H241 VS4";
$resn_exch{"1H25 VS4"}="H251 VS4"; $resn_exch{"1H26 VS4"}="H261 VS4"; $resn_exch{"1H28 VS4"}="H281 VS4";
$resn_exch{"2H28 VS4"}="H282 VS4"; $resn_exch{"1HN5 VS4"}="HN51 VS4"; $resn_exch{"1H34 VS4"}="H341 VS4";
$resn_exch{"2H34 VS4"}="H342 VS4"; $resn_exch{"1H42 VS4"}="H421 VS4"; $resn_exch{"1H43 VS4"}="H431 VS4";
$resn_exch{"1H44 VS4"}="H441 VS4"; $resn_exch{"1H45 VS4"}="H451 VS4"; $resn_exch{"1H46 VS4"}="H461 VS4";
$resn_exch{"1H27 VS4"}="H271 VS4"; $resn_exch{"2H27 VS4"}="H272 VS4"; $resn_exch{"1H1  VSC"}=" H11 VSC";
$resn_exch{"2H1  VSC"}=" H12 VSC"; $resn_exch{"1H9  VSC"}=" H91 VSC"; $resn_exch{"1H32 VSC"}="H321 VSC";
$resn_exch{"2H32 VSC"}="H322 VSC"; $resn_exch{"1H33 VSC"}="H331 VSC"; $resn_exch{"2H33 VSC"}="H332 VSC";
$resn_exch{"1H13 VSC"}="H131 VSC"; $resn_exch{"1H25 VSC"}="H251 VSC"; $resn_exch{"2H25 VSC"}="H252 VSC";
$resn_exch{"1H27 VSC"}="H271 VSC"; $resn_exch{"1H28 VSC"}="H281 VSC"; $resn_exch{"1H29 VSC"}="H291 VSC";
$resn_exch{"1H30 VSC"}="H301 VSC"; $resn_exch{"1H31 VSC"}="H311 VSC"; $resn_exch{"1H14 VSC"}="H141 VSC";
$resn_exch{"2H14 VSC"}="H142 VSC"; $resn_exch{"1H16 VSC"}="H161 VSC"; $resn_exch{"1H17 VSC"}="H171 VSC";
$resn_exch{"1H18 VSC"}="H181 VSC"; $resn_exch{"1H19 VSC"}="H191 VSC"; $resn_exch{"1H20 VSC"}="H201 VSC";
$resn_exch{"1H2  VSC"}="H21A VSC"; $resn_exch{"2H2  VSC"}=" H22 VSC"; $resn_exch{"1H4  VSC"}=" H41 VSC";
$resn_exch{"1H5  VSC"}=" H51 VSC"; $resn_exch{"1H6  VSC"}=" H61 VSC"; $resn_exch{"1H7  VSC"}=" H71 VSC";
$resn_exch{"1H8  VSC"}=" H81 VSC"; $resn_exch{"1H37 VSC"}="H371 VSC"; $resn_exch{"1H38 VSC"}="H381 VSC";
$resn_exch{"1H39 VSC"}="H391 VSC"; $resn_exch{"1H40 VSC"}="H401 VSC"; $resn_exch{"1H41 VSC"}="H411 VSC";
$resn_exch{"1H4  VSO"}=" H41 VSO"; $resn_exch{"2H4  VSO"}=" H42 VSO"; $resn_exch{"1H14 VTQ"}="H141 VTQ";
$resn_exch{"2H14 VTQ"}="H142 VTQ"; $resn_exch{"3H14 VTQ"}="H143 VTQ"; $resn_exch{"1H11 VTQ"}="H111 VTQ";
$resn_exch{"2H11 VTQ"}="H112 VTQ"; $resn_exch{"1H10 VTQ"}="H101 VTQ"; $resn_exch{"2H10 VTQ"}="H102 VTQ";
$resn_exch{"1H8C VTQ"}="H8C1 VTQ"; $resn_exch{"2H8C VTQ"}="H8C2 VTQ"; $resn_exch{"3H8C VTQ"}="H8C3 VTQ";
$resn_exch{"1H12 VTQ"}="H121 VTQ"; $resn_exch{"2H12 VTQ"}="H122 VTQ"; $resn_exch{"3H12 VTQ"}="H123 VTQ";
$resn_exch{"1H13 VTQ"}="H131 VTQ"; $resn_exch{"2H13 VTQ"}="H132 VTQ"; $resn_exch{"3H13 VTQ"}="H133 VTQ";
$resn_exch{"1H15 VTQ"}="H151 VTQ"; $resn_exch{"2H15 VTQ"}="H152 VTQ"; $resn_exch{"1H16 VTQ"}="H161 VTQ";
$resn_exch{"2H16 VTQ"}="H162 VTQ"; $resn_exch{"1H17 VTQ"}="H171 VTQ"; $resn_exch{"2H17 VTQ"}="H172 VTQ";
$resn_exch{"1H19 VTQ"}="H191 VTQ"; $resn_exch{"2H19 VTQ"}="H192 VTQ"; $resn_exch{"3H19 VTQ"}="H193 VTQ";
$resn_exch{"1H20 VTQ"}="H201 VTQ"; $resn_exch{"2H20 VTQ"}="H202 VTQ"; $resn_exch{"1H21 VTQ"}="H211 VTQ";
$resn_exch{"2H21 VTQ"}="H212 VTQ"; $resn_exch{"1H22 VTQ"}="H221 VTQ"; $resn_exch{"2H22 VTQ"}="H222 VTQ";
$resn_exch{"1H1C VTQ"}="H1C1 VTQ"; $resn_exch{"2H1C VTQ"}="H1C2 VTQ"; $resn_exch{"3H1C VTQ"}="H1C3 VTQ";
$resn_exch{"1H24 VTQ"}="H241 VTQ"; $resn_exch{"2H24 VTQ"}="H242 VTQ"; $resn_exch{"1H25 VTQ"}="H251 VTQ";
$resn_exch{"2H25 VTQ"}="H252 VTQ"; $resn_exch{"1H26 VTQ"}="H261 VTQ"; $resn_exch{"2H26 VTQ"}="H262 VTQ";
$resn_exch{"1H28 VTQ"}="H281 VTQ"; $resn_exch{"2H28 VTQ"}="H282 VTQ"; $resn_exch{"3H28 VTQ"}="H283 VTQ";
$resn_exch{"1H29 VTQ"}="H291 VTQ"; $resn_exch{"2H29 VTQ"}="H292 VTQ"; $resn_exch{"3H29 VTQ"}="H293 VTQ";
$resn_exch{"2H    VX"}=" H2   VX"; $resn_exch{"3H    VX"}=" H3   VX"; $resn_exch{"1H2   VX"}=" H21  VX";
$resn_exch{"2H2   VX"}=" H22  VX"; $resn_exch{"1H3   VX"}=" H31  VX"; $resn_exch{"2H3   VX"}=" H32  VX";
$resn_exch{"3H3   VX"}=" H33  VX"; $resn_exch{"1H2  VX6"}=" H21 VX6"; $resn_exch{"2H2  VX6"}=" H22 VX6";
$resn_exch{"1H3  VX6"}=" H31 VX6"; $resn_exch{"2H3  VX6"}=" H32 VX6"; $resn_exch{"1H5  VX6"}=" H51 VX6";
$resn_exch{"2H5  VX6"}=" H52 VX6"; $resn_exch{"1H6  VX6"}=" H61 VX6"; $resn_exch{"2H6  VX6"}=" H62 VX6";
$resn_exch{"1H7  VX6"}=" H71 VX6"; $resn_exch{"2H7  VX6"}=" H72 VX6"; $resn_exch{"3H7  VX6"}=" H73 VX6";
$resn_exch{"1H21 VX6"}="H211 VX6"; $resn_exch{"2H21 VX6"}="H212 VX6"; $resn_exch{"3H21 VX6"}="H213 VX6";
$resn_exch{"1H34 VX6"}="H341 VX6"; $resn_exch{"2H34 VX6"}="H342 VX6"; $resn_exch{"1H35 VX6"}="H351 VX6";
$resn_exch{"2H35 VX6"}="H352 VX6"; $resn_exch{"1H1  VXA"}=" H11 VXA"; $resn_exch{"2H1  VXA"}=" H12 VXA";
$resn_exch{"3H1  VXA"}=" H13 VXA"; $resn_exch{"1H6  VXR"}=" H61 VXR"; $resn_exch{"2H6  VXR"}=" H62 VXR";
$resn_exch{"1H7  VXR"}=" H71 VXR"; $resn_exch{"2H7  VXR"}=" H72 VXR"; $resn_exch{"3H7  VXR"}=" H73 VXR";
$resn_exch{"1HVC VXX"}="HVC1 VXX"; $resn_exch{"2HVC VXX"}="HVC2 VXX"; $resn_exch{"3HVC VXX"}="HVC3 VXX";
$resn_exch{"1H9C VXX"}="H9C1 VXX"; $resn_exch{"2H9C VXX"}="H9C2 VXX"; $resn_exch{"3H9C VXX"}="H9C3 VXX";
$resn_exch{"1H11 VZZ"}="H111 VZZ"; $resn_exch{"2H11 VZZ"}="H112 VZZ"; $resn_exch{"1H10 VZZ"}="H101 VZZ";
$resn_exch{"2H10 VZZ"}="H102 VZZ"; $resn_exch{"1H9  VZZ"}=" H91 VZZ"; $resn_exch{"2H9  VZZ"}=" H92 VZZ";
$resn_exch{"1H8  VZZ"}=" H81 VZZ"; $resn_exch{"2H8  VZZ"}=" H82 VZZ"; $resn_exch{"1H7  VZZ"}=" H71 VZZ";
$resn_exch{"2H7  VZZ"}=" H72 VZZ"; $resn_exch{"1H6  VZZ"}=" H61 VZZ"; $resn_exch{"2H6  VZZ"}=" H62 VZZ";
$resn_exch{"1H5  VZZ"}=" H51 VZZ"; $resn_exch{"2H5  VZZ"}=" H52 VZZ"; $resn_exch{"1H4  VZZ"}=" H41 VZZ";
$resn_exch{"2H4  VZZ"}=" H42 VZZ"; $resn_exch{"1H3  VZZ"}=" H31 VZZ"; $resn_exch{"2H3  VZZ"}=" H32 VZZ";
$resn_exch{"1H2  VZZ"}=" H21 VZZ"; $resn_exch{"2H2  VZZ"}=" H22 VZZ"; $resn_exch{"1HM3 W01"}="HM31 W01";
$resn_exch{"2HM3 W01"}="HM32 W01"; $resn_exch{"3HM3 W01"}="HM33 W01"; $resn_exch{"1H1C W01"}="H1C1 W01";
$resn_exch{"2H1C W01"}="H1C2 W01"; $resn_exch{"1H2C W01"}="H2C1 W01"; $resn_exch{"2H2C W01"}="H2C2 W01";
$resn_exch{"1H3C W01"}="H3C1 W01"; $resn_exch{"2H3C W01"}="H3C2 W01"; $resn_exch{"1HM2 W01"}="HM21 W01";
$resn_exch{"2HM2 W01"}="HM22 W01"; $resn_exch{"3HM2 W01"}="HM23 W01"; $resn_exch{"1HM6 W01"}="HM61 W01";
$resn_exch{"2HM6 W01"}="HM62 W01"; $resn_exch{"3HM6 W01"}="HM63 W01"; $resn_exch{"1HM4 W01"}="HM41 W01";
$resn_exch{"2HM4 W01"}="HM42 W01"; $resn_exch{"3HM4 W01"}="HM43 W01"; $resn_exch{"3HM3 W02"}="HM33 W02";
$resn_exch{"2HM3 W02"}="HM32 W02"; $resn_exch{"1HM3 W02"}="HM31 W02"; $resn_exch{"2H1C W02"}="H1C2 W02";
$resn_exch{"1H1C W02"}="H1C1 W02"; $resn_exch{"2H2C W02"}="H2C2 W02"; $resn_exch{"1H2C W02"}="H2C1 W02";
$resn_exch{"2H3C W02"}="H3C2 W02"; $resn_exch{"1H3C W02"}="H3C1 W02"; $resn_exch{"3HM2 W02"}="HM23 W02";
$resn_exch{"2HM2 W02"}="HM22 W02"; $resn_exch{"1HM2 W02"}="HM21 W02"; $resn_exch{"3HM6 W02"}="HM63 W02";
$resn_exch{"2HM6 W02"}="HM62 W02"; $resn_exch{"1HM6 W02"}="HM61 W02"; $resn_exch{"3HM4 W02"}="HM43 W02";
$resn_exch{"2HM4 W02"}="HM42 W02"; $resn_exch{"1HM4 W02"}="HM41 W02"; $resn_exch{"3HM3 W03"}="HM33 W03";
$resn_exch{"2HM3 W03"}="HM32 W03"; $resn_exch{"1HM3 W03"}="HM31 W03"; $resn_exch{"2H1C W03"}="H1C2 W03";
$resn_exch{"1H1C W03"}="H1C1 W03"; $resn_exch{"2H2C W03"}="H2C2 W03"; $resn_exch{"1H2C W03"}="H2C1 W03";
$resn_exch{"2H3C W03"}="H3C2 W03"; $resn_exch{"1H3C W03"}="H3C1 W03"; $resn_exch{"3HM2 W03"}="HM23 W03";
$resn_exch{"2HM2 W03"}="HM22 W03"; $resn_exch{"1HM2 W03"}="HM21 W03"; $resn_exch{"3HM6 W03"}="HM63 W03";
$resn_exch{"2HM6 W03"}="HM62 W03"; $resn_exch{"1HM6 W03"}="HM61 W03"; $resn_exch{"3HM4 W03"}="HM43 W03";
$resn_exch{"2HM4 W03"}="HM42 W03"; $resn_exch{"1HM4 W03"}="HM41 W03"; $resn_exch{"1H3C W05"}="H3C1 W05";
$resn_exch{"2H3C W05"}="H3C2 W05"; $resn_exch{"1H4C W05"}="H4C1 W05"; $resn_exch{"2H4C W05"}="H4C2 W05";
$resn_exch{"1H7C W05"}="H7C1 W05"; $resn_exch{"2H7C W05"}="H7C2 W05"; $resn_exch{"1H14 W05"}="H141 W05";
$resn_exch{"2H14 W05"}="H142 W05"; $resn_exch{"1H16 W05"}="H161 W05"; $resn_exch{"2H16 W05"}="H162 W05";
$resn_exch{"1H32 W05"}="H321 W05"; $resn_exch{"2H32 W05"}="H322 W05"; $resn_exch{"3H32 W05"}="H323 W05";
$resn_exch{"1HM3 W11"}="HM31 W11"; $resn_exch{"2HM3 W11"}="HM32 W11"; $resn_exch{"3HM3 W11"}="HM33 W11";
$resn_exch{"1H4  W11"}=" H41 W11"; $resn_exch{"1H1C W11"}="H1C1 W11"; $resn_exch{"2H1C W11"}="H1C2 W11";
$resn_exch{"1H2C W11"}="H2C1 W11"; $resn_exch{"2H2C W11"}="H2C2 W11"; $resn_exch{"1H3C W11"}="H3C1 W11";
$resn_exch{"2H3C W11"}="H3C2 W11"; $resn_exch{"1HM2 W11"}="HM21 W11"; $resn_exch{"2HM2 W11"}="HM22 W11";
$resn_exch{"3HM2 W11"}="HM23 W11"; $resn_exch{"1HM6 W11"}="HM61 W11"; $resn_exch{"2HM6 W11"}="HM62 W11";
$resn_exch{"3HM6 W11"}="HM63 W11"; $resn_exch{"1H25 W2X"}="H251 W2X"; $resn_exch{"2H25 W2X"}="H252 W2X";
$resn_exch{"1H7C W2X"}="H7C1 W2X"; $resn_exch{"2H7C W2X"}="H7C2 W2X"; $resn_exch{"1H4C W2X"}="H4C1 W2X";
$resn_exch{"2H4C W2X"}="H4C2 W2X"; $resn_exch{"1H3C W2X"}="H3C1 W2X"; $resn_exch{"2H3C W2X"}="H3C2 W2X";
$resn_exch{"1H14 W2X"}="H141 W2X"; $resn_exch{"2H14 W2X"}="H142 W2X"; $resn_exch{"1H37 W2X"}="H371 W2X";
$resn_exch{"2H37 W2X"}="H372 W2X"; $resn_exch{"1H33 W2X"}="H331 W2X"; $resn_exch{"2H33 W2X"}="H332 W2X";
$resn_exch{"3H33 W2X"}="H333 W2X"; $resn_exch{"1H31 W33"}="H311 W33"; $resn_exch{"2H31 W33"}="H312 W33";
$resn_exch{"3H31 W33"}="H313 W33"; $resn_exch{"1H1C W33"}="H1C1 W33"; $resn_exch{"2H1C W33"}="H1C2 W33";
$resn_exch{"1H2C W33"}="H2C1 W33"; $resn_exch{"2H2C W33"}="H2C2 W33"; $resn_exch{"1H3C W33"}="H3C1 W33";
$resn_exch{"2H3C W33"}="H3C2 W33"; $resn_exch{"1H4C W33"}="H4C1 W33"; $resn_exch{"2H4C W33"}="H4C2 W33";
$resn_exch{"1H5C W33"}="H5C1 W33"; $resn_exch{"2H5C W33"}="H5C2 W33"; $resn_exch{"1H4A W33"}="H4A1 W33";
$resn_exch{"2H4A W33"}="H4A2 W33"; $resn_exch{"1H5A W33"}="H5A1 W33"; $resn_exch{"2H5A W33"}="H5A2 W33";
$resn_exch{"1H31 W35"}="H311 W35"; $resn_exch{"2H31 W35"}="H312 W35"; $resn_exch{"3H31 W35"}="H313 W35";
$resn_exch{"1H1C W35"}="H1C1 W35"; $resn_exch{"2H1C W35"}="H1C2 W35"; $resn_exch{"1H2C W35"}="H2C1 W35";
$resn_exch{"2H2C W35"}="H2C2 W35"; $resn_exch{"1H3C W35"}="H3C1 W35"; $resn_exch{"2H3C W35"}="H3C2 W35";
$resn_exch{"1H4C W35"}="H4C1 W35"; $resn_exch{"2H4C W35"}="H4C2 W35"; $resn_exch{"1H5C W35"}="H5C1 W35";
$resn_exch{"2H5C W35"}="H5C2 W35"; $resn_exch{"1H4A W35"}="H4A1 W35"; $resn_exch{"2H4A W35"}="H4A2 W35";
$resn_exch{"1H5A W35"}="H5A1 W35"; $resn_exch{"2H5A W35"}="H5A2 W35"; $resn_exch{"1H3A W37"}="H3A1 W37";
$resn_exch{"2H3A W37"}="H3A2 W37"; $resn_exch{"1H1A W37"}="H1A1 W37"; $resn_exch{"2H1A W37"}="H1A2 W37";
$resn_exch{"1H1B W37"}="H1B1 W37"; $resn_exch{"2H1B W37"}="H1B2 W37"; $resn_exch{"1H2B W37"}="H2B1 W37";
$resn_exch{"2H2B W37"}="H2B2 W37"; $resn_exch{"1H3B W37"}="H3B1 W37"; $resn_exch{"2H3B W37"}="H3B2 W37";
$resn_exch{"1H4B W37"}="H4B1 W37"; $resn_exch{"2H4B W37"}="H4B2 W37"; $resn_exch{"1H5B W37"}="H5B1 W37";
$resn_exch{"2H5B W37"}="H5B2 W37"; $resn_exch{"1H6B W37"}="H6B1 W37"; $resn_exch{"2H6B W37"}="H6B2 W37";
$resn_exch{"1H1F W37"}="H1F1 W37"; $resn_exch{"2H1F W37"}="H1F2 W37"; $resn_exch{"3H1F W37"}="H1F3 W37";
$resn_exch{"1HOD W42"}="HOD1 W42"; $resn_exch{"1H2D W42"}="H2D1 W42"; $resn_exch{"2H2D W42"}="H2D2 W42";
$resn_exch{"1H3D W42"}="H3D1 W42"; $resn_exch{"2H3D W42"}="H3D2 W42"; $resn_exch{"1H5D W42"}="H5D1 W42";
$resn_exch{"2H5D W42"}="H5D2 W42"; $resn_exch{"1H31 W42"}="H311 W42"; $resn_exch{"2H31 W42"}="H312 W42";
$resn_exch{"1H1C W42"}="H1C1 W42"; $resn_exch{"2H1C W42"}="H1C2 W42"; $resn_exch{"1H2C W42"}="H2C1 W42";
$resn_exch{"2H2C W42"}="H2C2 W42"; $resn_exch{"1H3C W42"}="H3C1 W42"; $resn_exch{"2H3C W42"}="H3C2 W42";
$resn_exch{"1H4C W42"}="H4C1 W42"; $resn_exch{"2H4C W42"}="H4C2 W42"; $resn_exch{"1H5C W42"}="H5C1 W42";
$resn_exch{"2H5C W42"}="H5C2 W42"; $resn_exch{"1H4A W42"}="H4A1 W42"; $resn_exch{"2H4A W42"}="H4A2 W42";
$resn_exch{"1H5A W42"}="H5A1 W42"; $resn_exch{"2H5A W42"}="H5A2 W42"; $resn_exch{"1H31 W43"}="H311 W43";
$resn_exch{"2H31 W43"}="H312 W43"; $resn_exch{"3H31 W43"}="H313 W43"; $resn_exch{"1H1C W43"}="H1C1 W43";
$resn_exch{"2H1C W43"}="H1C2 W43"; $resn_exch{"1H2C W43"}="H2C1 W43"; $resn_exch{"2H2C W43"}="H2C2 W43";
$resn_exch{"1H3C W43"}="H3C1 W43"; $resn_exch{"2H3C W43"}="H3C2 W43"; $resn_exch{"1H4C W43"}="H4C1 W43";
$resn_exch{"2H4C W43"}="H4C2 W43"; $resn_exch{"1H5C W43"}="H5C1 W43"; $resn_exch{"2H5C W43"}="H5C2 W43";
$resn_exch{"1H6C W43"}="H6C1 W43"; $resn_exch{"2H6C W43"}="H6C2 W43"; $resn_exch{"1H7C W43"}="H7C1 W43";
$resn_exch{"2H7C W43"}="H7C2 W43"; $resn_exch{"1HM1 W43"}="HM11 W43"; $resn_exch{"2HM1 W43"}="HM12 W43";
$resn_exch{"3HM1 W43"}="HM13 W43"; $resn_exch{"1H5A W43"}="H5A1 W43"; $resn_exch{"2H5A W43"}="H5A2 W43";
$resn_exch{"1H31 W54"}="H311 W54"; $resn_exch{"2H31 W54"}="H312 W54"; $resn_exch{"3H31 W54"}="H313 W54";
$resn_exch{"1H1C W54"}="H1C1 W54"; $resn_exch{"2H1C W54"}="H1C2 W54"; $resn_exch{"1H2C W54"}="H2C1 W54";
$resn_exch{"2H2C W54"}="H2C2 W54"; $resn_exch{"1H3C W54"}="H3C1 W54"; $resn_exch{"2H3C W54"}="H3C2 W54";
$resn_exch{"1H4C W54"}="H4C1 W54"; $resn_exch{"2H4C W54"}="H4C2 W54"; $resn_exch{"1H5C W54"}="H5C1 W54";
$resn_exch{"2H5C W54"}="H5C2 W54"; $resn_exch{"1H4A W54"}="H4A1 W54"; $resn_exch{"2H4A W54"}="H4A2 W54";
$resn_exch{"1H5A W54"}="H5A1 W54"; $resn_exch{"2H5A W54"}="H5A2 W54"; $resn_exch{"1H31 W56"}="H311 W56";
$resn_exch{"2H31 W56"}="H312 W56"; $resn_exch{"3H31 W56"}="H313 W56"; $resn_exch{"1H1C W56"}="H1C1 W56";
$resn_exch{"2H1C W56"}="H1C2 W56"; $resn_exch{"1H2C W56"}="H2C1 W56"; $resn_exch{"2H2C W56"}="H2C2 W56";
$resn_exch{"1H3C W56"}="H3C1 W56"; $resn_exch{"2H3C W56"}="H3C2 W56"; $resn_exch{"1H4C W56"}="H4C1 W56";
$resn_exch{"2H4C W56"}="H4C2 W56"; $resn_exch{"1H5C W56"}="H5C1 W56"; $resn_exch{"2H5C W56"}="H5C2 W56";
$resn_exch{"1HM1 W56"}="HM11 W56"; $resn_exch{"2HM1 W56"}="HM12 W56"; $resn_exch{"3HM1 W56"}="HM13 W56";
$resn_exch{"1H5A W56"}="H5A1 W56"; $resn_exch{"2H5A W56"}="H5A2 W56"; $resn_exch{"1H31 W59"}="H311 W59";
$resn_exch{"2H31 W59"}="H312 W59"; $resn_exch{"3H31 W59"}="H313 W59"; $resn_exch{"1H1C W59"}="H1C1 W59";
$resn_exch{"2H1C W59"}="H1C2 W59"; $resn_exch{"1H2C W59"}="H2C1 W59"; $resn_exch{"2H2C W59"}="H2C2 W59";
$resn_exch{"1H3C W59"}="H3C1 W59"; $resn_exch{"2H3C W59"}="H3C2 W59"; $resn_exch{"1H4C W59"}="H4C1 W59";
$resn_exch{"2H4C W59"}="H4C2 W59"; $resn_exch{"1H5C W59"}="H5C1 W59"; $resn_exch{"2H5C W59"}="H5C2 W59";
$resn_exch{"1H6C W59"}="H6C1 W59"; $resn_exch{"2H6C W59"}="H6C2 W59"; $resn_exch{"1H7C W59"}="H7C1 W59";
$resn_exch{"2H7C W59"}="H7C2 W59"; $resn_exch{"1HM1 W59"}="HM11 W59"; $resn_exch{"2HM1 W59"}="HM12 W59";
$resn_exch{"1HM2 W59"}="HM21 W59"; $resn_exch{"2HM2 W59"}="HM22 W59"; $resn_exch{"3HM2 W59"}="HM23 W59";
$resn_exch{"1H5A W59"}="H5A1 W59"; $resn_exch{"2H5A W59"}="H5A2 W59"; $resn_exch{"1H31 W71"}="H311 W71";
$resn_exch{"2H31 W71"}="H312 W71"; $resn_exch{"3H31 W71"}="H313 W71"; $resn_exch{"1H1C W71"}="H1C1 W71";
$resn_exch{"2H1C W71"}="H1C2 W71"; $resn_exch{"1H2C W71"}="H2C1 W71"; $resn_exch{"2H2C W71"}="H2C2 W71";
$resn_exch{"1H3C W71"}="H3C1 W71"; $resn_exch{"2H3C W71"}="H3C2 W71"; $resn_exch{"1H4C W71"}="H4C1 W71";
$resn_exch{"2H4C W71"}="H4C2 W71"; $resn_exch{"1H5C W71"}="H5C1 W71"; $resn_exch{"2H5C W71"}="H5C2 W71";
$resn_exch{"1H6C W71"}="H6C1 W71"; $resn_exch{"2H6C W71"}="H6C2 W71"; $resn_exch{"1H7C W71"}="H7C1 W71";
$resn_exch{"2H7C W71"}="H7C2 W71"; $resn_exch{"1H4A W71"}="H4A1 W71"; $resn_exch{"2H4A W71"}="H4A2 W71";
$resn_exch{"1H5A W71"}="H5A1 W71"; $resn_exch{"2H5A W71"}="H5A2 W71"; $resn_exch{"1H3  W72"}=" H31 W72";
$resn_exch{"2H3  W72"}=" H32 W72"; $resn_exch{"1H6  W72"}=" H61 W72"; $resn_exch{"2H6  W72"}=" H62 W72";
$resn_exch{"1H7  W72"}=" H71 W72"; $resn_exch{"2H7  W72"}=" H72 W72"; $resn_exch{"1H31 W84"}="H311 W84";
$resn_exch{"2H31 W84"}="H312 W84"; $resn_exch{"3H31 W84"}="H313 W84"; $resn_exch{"1H1C W84"}="H1C1 W84";
$resn_exch{"2H1C W84"}="H1C2 W84"; $resn_exch{"1H2C W84"}="H2C1 W84"; $resn_exch{"2H2C W84"}="H2C2 W84";
$resn_exch{"1H3C W84"}="H3C1 W84"; $resn_exch{"2H3C W84"}="H3C2 W84"; $resn_exch{"1H4C W84"}="H4C1 W84";
$resn_exch{"2H4C W84"}="H4C2 W84"; $resn_exch{"1H5C W84"}="H5C1 W84"; $resn_exch{"2H5C W84"}="H5C2 W84";
$resn_exch{"1H6C W84"}="H6C1 W84"; $resn_exch{"2H6C W84"}="H6C2 W84"; $resn_exch{"1H7C W84"}="H7C1 W84";
$resn_exch{"2H7C W84"}="H7C2 W84"; $resn_exch{"1HM1 W84"}="HM11 W84"; $resn_exch{"2HM1 W84"}="HM12 W84";
$resn_exch{"3HM1 W84"}="HM13 W84"; $resn_exch{"1H5A W84"}="H5A1 W84"; $resn_exch{"2H5A W84"}="H5A2 W84";
$resn_exch{"1H31 W8R"}="H311 W8R"; $resn_exch{"2H31 W8R"}="H312 W8R"; $resn_exch{"3H31 W8R"}="H313 W8R";
$resn_exch{"1H1C W8R"}="H1C1 W8R"; $resn_exch{"2H1C W8R"}="H1C2 W8R"; $resn_exch{"1H2C W8R"}="H2C1 W8R";
$resn_exch{"2H2C W8R"}="H2C2 W8R"; $resn_exch{"1H3C W8R"}="H3C1 W8R"; $resn_exch{"2H3C W8R"}="H3C2 W8R";
$resn_exch{"1H4C W8R"}="H4C1 W8R"; $resn_exch{"2H4C W8R"}="H4C2 W8R"; $resn_exch{"1H5C W8R"}="H5C1 W8R";
$resn_exch{"2H5C W8R"}="H5C2 W8R"; $resn_exch{"1H6C W8R"}="H6C1 W8R"; $resn_exch{"2H6C W8R"}="H6C2 W8R";
$resn_exch{"1H7C W8R"}="H7C1 W8R"; $resn_exch{"2H7C W8R"}="H7C2 W8R"; $resn_exch{"1HM1 W8R"}="HM11 W8R";
$resn_exch{"2HM1 W8R"}="HM12 W8R"; $resn_exch{"3HM1 W8R"}="HM13 W8R"; $resn_exch{"1H5A W8R"}="H5A1 W8R";
$resn_exch{"2H5A W8R"}="H5A2 W8R"; $resn_exch{"1H31 W91"}="H311 W91"; $resn_exch{"2H31 W91"}="H312 W91";
$resn_exch{"3H31 W91"}="H313 W91"; $resn_exch{"1H1C W91"}="H1C1 W91"; $resn_exch{"2H1C W91"}="H1C2 W91";
$resn_exch{"1H2C W91"}="H2C1 W91"; $resn_exch{"2H2C W91"}="H2C2 W91"; $resn_exch{"1H3C W91"}="H3C1 W91";
$resn_exch{"2H3C W91"}="H3C2 W91"; $resn_exch{"1H4A W91"}="H4A1 W91"; $resn_exch{"2H4A W91"}="H4A2 W91";
$resn_exch{"1H5A W91"}="H5A1 W91"; $resn_exch{"2H5A W91"}="H5A2 W91"; $resn_exch{"1H12 WAC"}="H121 WAC";
$resn_exch{"2H12 WAC"}="H122 WAC"; $resn_exch{"1H16 WAC"}="H161 WAC"; $resn_exch{"2H16 WAC"}="H162 WAC";
$resn_exch{"1H17 WAC"}="H171 WAC"; $resn_exch{"2H17 WAC"}="H172 WAC"; $resn_exch{"1H18 WAC"}="H181 WAC";
$resn_exch{"2H18 WAC"}="H182 WAC"; $resn_exch{"1H19 WAC"}="H191 WAC"; $resn_exch{"2H19 WAC"}="H192 WAC";
$resn_exch{"1H29 WAC"}="H291 WAC"; $resn_exch{"2H29 WAC"}="H292 WAC"; $resn_exch{"1H30 WAC"}="H301 WAC";
$resn_exch{"2H30 WAC"}="H302 WAC"; $resn_exch{"1H33 WAC"}="H331 WAC"; $resn_exch{"2H33 WAC"}="H332 WAC";
$resn_exch{"1H41 WAC"}="H411 WAC"; $resn_exch{"2H41 WAC"}="H412 WAC"; $resn_exch{"3H41 WAC"}="H413 WAC";
$resn_exch{"1H42 WAC"}="H421 WAC"; $resn_exch{"2H42 WAC"}="H422 WAC"; $resn_exch{"3H42 WAC"}="H423 WAC";
$resn_exch{"1H20 WAI"}="H201 WAI"; $resn_exch{"2H20 WAI"}="H202 WAI"; $resn_exch{"3H20 WAI"}="H203 WAI";
$resn_exch{"1H35 WAI"}="H351 WAI"; $resn_exch{"2H35 WAI"}="H352 WAI"; $resn_exch{"1H36 WAI"}="H361 WAI";
$resn_exch{"2H36 WAI"}="H362 WAI"; $resn_exch{"1H38 WAI"}="H381 WAI"; $resn_exch{"2H38 WAI"}="H382 WAI";
$resn_exch{"1H39 WAI"}="H391 WAI"; $resn_exch{"2H39 WAI"}="H392 WAI"; $resn_exch{"1H48 WAI"}="H481 WAI";
$resn_exch{"2H48 WAI"}="H482 WAI"; $resn_exch{"1H49 WAI"}="H491 WAI"; $resn_exch{"2H49 WAI"}="H492 WAI";
$resn_exch{"1H52 WAI"}="H521 WAI"; $resn_exch{"2H52 WAI"}="H522 WAI"; $resn_exch{"1H18 WBT"}="H181 WBT";
$resn_exch{"2H18 WBT"}="H182 WBT"; $resn_exch{"1H19 WBT"}="H191 WBT"; $resn_exch{"2H19 WBT"}="H192 WBT";
$resn_exch{"1H29 WBT"}="H291 WBT"; $resn_exch{"2H29 WBT"}="H292 WBT"; $resn_exch{"1H30 WBT"}="H301 WBT";
$resn_exch{"2H30 WBT"}="H302 WBT"; $resn_exch{"1H32 WBT"}="H321 WBT"; $resn_exch{"2H32 WBT"}="H322 WBT";
$resn_exch{"1H33 WBT"}="H331 WBT"; $resn_exch{"2H33 WBT"}="H332 WBT"; $resn_exch{"1H01 WBU"}="H011 WBU";
$resn_exch{"2H01 WBU"}="H012 WBU"; $resn_exch{"1HMC WIA"}="HMC1 WIA"; $resn_exch{"2HMC WIA"}="HMC2 WIA";
$resn_exch{"3HMC WIA"}="HMC3 WIA"; $resn_exch{"1H16 WR1"}="H161 WR1"; $resn_exch{"2H16 WR1"}="H162 WR1";
$resn_exch{"1H8  WR1"}=" H81 WR1"; $resn_exch{"2H8  WR1"}=" H82 WR1"; $resn_exch{"1H5  WR1"}=" H51 WR1";
$resn_exch{"2H5  WR1"}=" H52 WR1"; $resn_exch{"3H5  WR1"}=" H53 WR1"; $resn_exch{"1H2  WR1"}="H21A WR1";
$resn_exch{"2H2  WR1"}="H22A WR1"; $resn_exch{"1H1  WR1"}="H11A WR1"; $resn_exch{"2H1  WR1"}="H12A WR1";
$resn_exch{"1H8  WRA"}=" H81 WRA"; $resn_exch{"2H8  WRA"}=" H82 WRA"; $resn_exch{"1H9  WRA"}=" H91 WRA";
$resn_exch{"2H9  WRA"}=" H92 WRA"; $resn_exch{"1H10 WRA"}="H101 WRA"; $resn_exch{"2H10 WRA"}="H102 WRA";
$resn_exch{"1H1  WRA"}=" H11 WRA"; $resn_exch{"2H1  WRA"}=" H12 WRA"; $resn_exch{"3H1  WRA"}=" H13 WRA";
$resn_exch{"1H2  WRA"}=" H21 WRA"; $resn_exch{"2H2  WRA"}=" H22 WRA"; $resn_exch{"3H2  WRA"}=" H23 WRA";
$resn_exch{"1HH1 WRA"}="HH11 WRA"; $resn_exch{"2HH1 WRA"}="HH12 WRA"; $resn_exch{"1HH2 WRA"}="HH21 WRA";
$resn_exch{"2HH2 WRA"}="HH22 WRA"; $resn_exch{"1H8  WRB"}=" H81 WRB"; $resn_exch{"2H8  WRB"}=" H82 WRB";
$resn_exch{"1H9  WRB"}=" H91 WRB"; $resn_exch{"2H9  WRB"}=" H92 WRB"; $resn_exch{"1H10 WRB"}="H101 WRB";
$resn_exch{"2H10 WRB"}="H102 WRB"; $resn_exch{"1H13 WRB"}="H131 WRB"; $resn_exch{"1H14 WRB"}="H141 WRB";
$resn_exch{"1H15 WRB"}="H151 WRB"; $resn_exch{"1H16 WRB"}="H161 WRB"; $resn_exch{"1HM1 WRB"}="HM11 WRB";
$resn_exch{"2HM1 WRB"}="HM12 WRB"; $resn_exch{"3HM1 WRB"}="HM13 WRB"; $resn_exch{"1HM2 WRB"}="HM21 WRB";
$resn_exch{"2HM2 WRB"}="HM22 WRB"; $resn_exch{"3HM2 WRB"}="HM23 WRB"; $resn_exch{"1HN1 WRB"}="HN11 WRB";
$resn_exch{"2HN1 WRB"}="HN12 WRB"; $resn_exch{"1HN2 WRB"}="HN21 WRB"; $resn_exch{"2HN2 WRB"}="HN22 WRB";
$resn_exch{"2HC  WRR"}=" HC2 WRR"; $resn_exch{"1HC3 WRR"}="HC31 WRR"; $resn_exch{"2HC3 WRR"}="HC32 WRR";
$resn_exch{"4HC  WRR"}=" HC4 WRR"; $resn_exch{"1HC  WRR"}=" HC1 WRR"; $resn_exch{"2HA  WRR"}=" HA2 WRR";
$resn_exch{"3HC  WRR"}=" HC3 WRR"; $resn_exch{"7HC  WRR"}=" HC7 WRR"; $resn_exch{"8HC  WRR"}=" HC8 WRR";
$resn_exch{"9HC  WRR"}=" HC9 WRR"; $resn_exch{"1HC0 WRR"}="HC01 WRR"; $resn_exch{"1HC2 WRR"}="HC21 WRR";
$resn_exch{"1HC5 WRR"}="HC51 WRR"; $resn_exch{"1HC6 WRR"}="HC61 WRR"; $resn_exch{"1HC7 WRR"}="HC71 WRR";
$resn_exch{"1HC8 WRR"}="HC81 WRR"; $resn_exch{"3HO  WRR"}=" HO3 WRR"; $resn_exch{"4HO  WRR"}=" HO4 WRR";
$resn_exch{"2HC  WRS"}=" HC2 WRS"; $resn_exch{"1HC3 WRS"}="HC31 WRS"; $resn_exch{"2HC3 WRS"}="HC32 WRS";
$resn_exch{"4HC  WRS"}=" HC4 WRS"; $resn_exch{"1HC  WRS"}=" HC1 WRS"; $resn_exch{"2HA  WRS"}=" HA2 WRS";
$resn_exch{"3HC  WRS"}=" HC3 WRS"; $resn_exch{"7HC  WRS"}=" HC7 WRS"; $resn_exch{"8HC  WRS"}=" HC8 WRS";
$resn_exch{"9HC  WRS"}=" HC9 WRS"; $resn_exch{"1HC0 WRS"}="HC01 WRS"; $resn_exch{"1HC2 WRS"}="HC21 WRS";
$resn_exch{"1HC5 WRS"}="HC51 WRS"; $resn_exch{"1HC6 WRS"}="HC61 WRS"; $resn_exch{"1HC7 WRS"}="HC71 WRS";
$resn_exch{"1HC8 WRS"}="HC81 WRS"; $resn_exch{"3HO  WRS"}=" HO3 WRS"; $resn_exch{"4HO  WRS"}=" HO4 WRS";
$resn_exch{"1HB  WSA"}=" HB1 WSA"; $resn_exch{"2HB  WSA"}=" HB2 WSA"; $resn_exch{"1HH3 WSA"}="HH31 WSA";
$resn_exch{"2HH3 WSA"}="HH32 WSA"; $resn_exch{"1H5' WSA"}="H5'1 WSA"; $resn_exch{"2H5' WSA"}="H5'2 WSA";
$resn_exch{"1HN6 WSA"}="HN61 WSA"; $resn_exch{"2HN6 WSA"}="HN62 WSA"; $resn_exch{"1H15 WST"}="H151 WST";
$resn_exch{"2H15 WST"}="H152 WST"; $resn_exch{"1H16 WST"}="H161 WST"; $resn_exch{"2H16 WST"}="H162 WST";
$resn_exch{"1H17 WST"}="H171 WST"; $resn_exch{"2H17 WST"}="H172 WST"; $resn_exch{"1H11 WW7"}="H111 WW7";
$resn_exch{"2H11 WW7"}="H112 WW7"; $resn_exch{"1H12 WW7"}="H121 WW7"; $resn_exch{"2H12 WW7"}="H122 WW7";
$resn_exch{"1H14 WW7"}="H141 WW7"; $resn_exch{"2H14 WW7"}="H142 WW7"; $resn_exch{"1H13 WW7"}="H131 WW7";
$resn_exch{"2H13 WW7"}="H132 WW7"; $resn_exch{"1H15 WW7"}="H151 WW7"; $resn_exch{"2H15 WW7"}="H152 WW7";
$resn_exch{"1H16 WW7"}="H161 WW7"; $resn_exch{"2H16 WW7"}="H162 WW7"; $resn_exch{"1HN2 WW7"}="HN21 WW7";
$resn_exch{"2HN2 WW7"}="HN22 WW7"; $resn_exch{"1H5B WWF"}="H5B1 WWF"; $resn_exch{"2H5B WWF"}="H5B2 WWF";
$resn_exch{"1H5A WWF"}="H5A1 WWF"; $resn_exch{"2H5A WWF"}="H5A2 WWF"; $resn_exch{"1H4A WWF"}="H4A1 WWF";
$resn_exch{"2H4A WWF"}="H4A2 WWF"; $resn_exch{"3H4A WWF"}="H4A3 WWF"; $resn_exch{"1H9C WWF"}="H9C1 WWF";
$resn_exch{"2H9C WWF"}="H9C2 WWF"; $resn_exch{"1H11 WWF"}="H111 WWF"; $resn_exch{"2H11 WWF"}="H112 WWF";
$resn_exch{"3H11 WWF"}="H113 WWF"; $resn_exch{"1H12 WWF"}="H121 WWF"; $resn_exch{"2H12 WWF"}="H122 WWF";
$resn_exch{"3H12 WWF"}="H123 WWF"; $resn_exch{"1H35 WWF"}="H351 WWF"; $resn_exch{"2H35 WWF"}="H352 WWF";
$resn_exch{" H6* WWF"}=" H6' WWF"; $resn_exch{"1H2A WWF"}="H2A1 WWF"; $resn_exch{"2H2A WWF"}="H2A2 WWF";
$resn_exch{"3H2A WWF"}="H2A3 WWF"; $resn_exch{"1H4* WWF"}="H4'1 WWF"; $resn_exch{"2H4* WWF"}="H4'2 WWF";
$resn_exch{"1H2  WY2"}=" H21 WY2"; $resn_exch{"2H2  WY2"}=" H22 WY2"; $resn_exch{"1H69 WY2"}="H691 WY2";
$resn_exch{"2H69 WY2"}="H692 WY2"; $resn_exch{"1H70 WY2"}="H701 WY2"; $resn_exch{"2H70 WY2"}="H702 WY2";
$resn_exch{"1H71 WY2"}="H711 WY2"; $resn_exch{"2H71 WY2"}="H712 WY2"; $resn_exch{"1HC2 WY4"}="HC21 WY4";
$resn_exch{"2HC2 WY4"}="HC22 WY4"; $resn_exch{"1H67 WY4"}="H671 WY4"; $resn_exch{"2H67 WY4"}="H672 WY4";
$resn_exch{"1H69 WY4"}="H691 WY4"; $resn_exch{"2H69 WY4"}="H692 WY4"; $resn_exch{"1H70 WY4"}="H701 WY4";
$resn_exch{"2H70 WY4"}="H702 WY4"; $resn_exch{"2HOP   X"}="HOP2   X"; $resn_exch{"3HOP   X"}="HOP3   X";
$resn_exch{"1H5*   X"}=" H5'   X"; $resn_exch{"2H5*   X"}="H5''   X"; $resn_exch{" H4*   X"}=" H4'   X";
$resn_exch{" H3*   X"}=" H3'   X"; $resn_exch{"*HO3   X"}="HO3'   X"; $resn_exch{"1H2*   X"}=" H2'   X";
$resn_exch{"2H2*   X"}="H2''   X"; $resn_exch{" H1*   X"}=" H1'   X"; $resn_exch{"1H2    X"}=" H21   X";
$resn_exch{"2H2    X"}=" H22   X"; $resn_exch{"1H2A   X"}="H2A1   X"; $resn_exch{"2H2A   X"}="H2A2   X";
$resn_exch{"1H3    X"}=" H31   X"; $resn_exch{"2H3    X"}=" H32   X"; $resn_exch{"1HM    X"}=" HM1   X";
$resn_exch{"2HM    X"}=" HM2   X"; $resn_exch{"3HM    X"}=" HM3   X"; $resn_exch{"1H5A X04"}="H5A1 X04";
$resn_exch{"2H5A X04"}="H5A2 X04"; $resn_exch{"1HN2 X04"}="HN21 X04"; $resn_exch{"2HN2 X04"}="HN22 X04";
$resn_exch{"1H2A X04"}="H2A1 X04"; $resn_exch{"2H2A X04"}="H2A2 X04"; $resn_exch{"3H2A X04"}="H2A3 X04";
$resn_exch{"1H5  X1P"}=" H51 X1P"; $resn_exch{"2H5  X1P"}=" H52 X1P"; $resn_exch{"1H5  X2F"}=" H51 X2F";
$resn_exch{"2H5  X2F"}=" H52 X2F"; $resn_exch{"3HOP X4A"}="HOP3 X4A"; $resn_exch{"2HOP X4A"}="HOP2 X4A";
$resn_exch{"1H5* X4A"}=" H5' X4A"; $resn_exch{"2H5* X4A"}="H5'' X4A"; $resn_exch{" H1* X4A"}=" H1' X4A";
$resn_exch{" H3* X4A"}=" H3' X4A"; $resn_exch{"1H2* X4A"}=" H2' X4A"; $resn_exch{"2H2* X4A"}="H2'' X4A";
$resn_exch{"H3T  X4A"}="HO3' X4A"; $resn_exch{"1HC4 X7O"}="HC41 X7O"; $resn_exch{"2HC4 X7O"}="HC42 X7O";
$resn_exch{"1HC6 X7O"}="HC61 X7O"; $resn_exch{"2HC6 X7O"}="HC62 X7O"; $resn_exch{"3HC6 X7O"}="HC63 X7O";
$resn_exch{"1HN2 X7O"}="HN21 X7O"; $resn_exch{"2HN2 X7O"}="HN22 X7O"; $resn_exch{"1HN7 XAA"}="HN71 XAA";
$resn_exch{"2HN7 XAA"}="HN72 XAA"; $resn_exch{"1HC  XAA"}=" HC1 XAA"; $resn_exch{"2HC  XAA"}=" HC2 XAA";
$resn_exch{"3HC  XAA"}=" HC3 XAA"; $resn_exch{"3HOP XAD"}="HOP3 XAD"; $resn_exch{"2HOP XAD"}="HOP2 XAD";
$resn_exch{"1H6' XAD"}="H6'1 XAD"; $resn_exch{"2H6' XAD"}="H6'2 XAD"; $resn_exch{"1H3' XAD"}="H3'1 XAD";
$resn_exch{"2H3' XAD"}="H3'2 XAD"; $resn_exch{"1H2' XAD"}="H2'1 XAD"; $resn_exch{"2H2' XAD"}="H2'2 XAD";
$resn_exch{"1HN6 XAD"}="HN61 XAD"; $resn_exch{"2HN6 XAD"}="HN62 XAD"; $resn_exch{"1H5' XAE"}="H5'1 XAE";
$resn_exch{"2H5' XAE"}="H5'2 XAE"; $resn_exch{"1H2' XAE"}="H2'1 XAE"; $resn_exch{"2H2' XAE"}="H2'2 XAE";
$resn_exch{"1HN6 XAE"}="HN61 XAE"; $resn_exch{"2HN6 XAE"}="HN62 XAE"; $resn_exch{"H3T  XAE"}="HO3' XAE";
$resn_exch{"2HOP XAL"}="HOP2 XAL"; $resn_exch{"1H5* XAL"}=" H5' XAL"; $resn_exch{"2H5* XAL"}="H5'' XAL";
$resn_exch{" H4* XAL"}=" H4' XAL"; $resn_exch{" H6* XAL"}=" H6' XAL"; $resn_exch{" H7* XAL"}=" H7' XAL";
$resn_exch{" H3* XAL"}=" H3' XAL"; $resn_exch{"H3T  XAL"}="HO3' XAL"; $resn_exch{"1H2* XAL"}=" H2' XAL";
$resn_exch{"2H2* XAL"}="H2'' XAL"; $resn_exch{" H1* XAL"}=" H1' XAL"; $resn_exch{"1HN6 XAL"}="HN61 XAL";
$resn_exch{"2HN6 XAL"}="HN62 XAL"; $resn_exch{"3HOP XAL"}="HOP3 XAL"; $resn_exch{"1HN  XAO"}=" HN1 XAO";
$resn_exch{"2HN  XAO"}=" HN2 XAO"; $resn_exch{"1H4  XAO"}=" H41 XAO"; $resn_exch{"2H4  XAO"}=" H42 XAO";
$resn_exch{"1H6  XAO"}=" H61 XAO"; $resn_exch{"2H6  XAO"}=" H62 XAO"; $resn_exch{"3H6  XAO"}=" H63 XAO";
$resn_exch{"1H7  XAO"}=" H71 XAO"; $resn_exch{"2H7  XAO"}=" H72 XAO"; $resn_exch{"3H7  XAO"}=" H73 XAO";
$resn_exch{"1H7' XAO"}="H7'1 XAO"; $resn_exch{"2H7' XAO"}="H7'2 XAO"; $resn_exch{"1H2' XAO"}="H2'1 XAO";
$resn_exch{"2H2' XAO"}="H2'2 XAO"; $resn_exch{"1H6' XAO"}="H6'1 XAO"; $resn_exch{"2H6' XAO"}="H6'2 XAO";
$resn_exch{"1H3' XAO"}="H3'1 XAO"; $resn_exch{"2H3' XAO"}="H3'2 XAO"; $resn_exch{"1H5' XAO"}="H5'1 XAO";
$resn_exch{"2H5' XAO"}="H5'2 XAO"; $resn_exch{"1H4' XAO"}="H4'1 XAO"; $resn_exch{"2H4' XAO"}="H4'2 XAO";
$resn_exch{"1HN1 XAP"}="HN11 XAP"; $resn_exch{"2HN1 XAP"}="HN12 XAP"; $resn_exch{"1H1  XAP"}=" H11 XAP";
$resn_exch{"2H1  XAP"}=" H12 XAP"; $resn_exch{"1H2  XAP"}=" H21 XAP"; $resn_exch{"2H2  XAP"}=" H22 XAP";
$resn_exch{"1H3  XAP"}=" H31 XAP"; $resn_exch{"2H3  XAP"}=" H32 XAP"; $resn_exch{"1HN2 XAP"}="HN21 XAP";
$resn_exch{"2HN2 XAP"}="HN22 XAP"; $resn_exch{"3HOP XAR"}="HOP3 XAR"; $resn_exch{"2HOP XAR"}="HOP2 XAR";
$resn_exch{"1H5* XAR"}="H5'1 XAR"; $resn_exch{"2H5* XAR"}="H5'2 XAR"; $resn_exch{" H4* XAR"}=" H4' XAR";
$resn_exch{" H3* XAR"}=" H3' XAR"; $resn_exch{"1H2* XAR"}="H2'1 XAR"; $resn_exch{"2H2* XAR"}="H2'2 XAR";
$resn_exch{" H1* XAR"}=" H1' XAR"; $resn_exch{" H7* XAR"}=" H7' XAR"; $resn_exch{" H6* XAR"}=" H6' XAR";
$resn_exch{"1HN6 XAR"}="HN61 XAR"; $resn_exch{"2HN6 XAR"}="HN62 XAR"; $resn_exch{"1H2  XAT"}=" H21 XAT";
$resn_exch{"2H2  XAT"}=" H22 XAT"; $resn_exch{"1H4  XAT"}=" H41 XAT"; $resn_exch{"2H4  XAT"}=" H42 XAT";
$resn_exch{"1H16 XAT"}="H161 XAT"; $resn_exch{"2H16 XAT"}="H162 XAT"; $resn_exch{"3H16 XAT"}="H163 XAT";
$resn_exch{"1H17 XAT"}="H171 XAT"; $resn_exch{"2H17 XAT"}="H172 XAT"; $resn_exch{"3H17 XAT"}="H173 XAT";
$resn_exch{"1H18 XAT"}="H181 XAT"; $resn_exch{"2H18 XAT"}="H182 XAT"; $resn_exch{"3H18 XAT"}="H183 XAT";
$resn_exch{"1H19 XAT"}="H191 XAT"; $resn_exch{"2H19 XAT"}="H192 XAT"; $resn_exch{"3H19 XAT"}="H193 XAT";
$resn_exch{"1H20 XAT"}="H201 XAT"; $resn_exch{"2H20 XAT"}="H202 XAT"; $resn_exch{"3H20 XAT"}="H203 XAT";
$resn_exch{"1H22 XAT"}="H221 XAT"; $resn_exch{"2H22 XAT"}="H222 XAT"; $resn_exch{"1H24 XAT"}="H241 XAT";
$resn_exch{"2H24 XAT"}="H242 XAT"; $resn_exch{"1H36 XAT"}="H361 XAT"; $resn_exch{"2H36 XAT"}="H362 XAT";
$resn_exch{"3H36 XAT"}="H363 XAT"; $resn_exch{"1H37 XAT"}="H371 XAT"; $resn_exch{"2H37 XAT"}="H372 XAT";
$resn_exch{"3H37 XAT"}="H373 XAT"; $resn_exch{"1H38 XAT"}="H381 XAT"; $resn_exch{"2H38 XAT"}="H382 XAT";
$resn_exch{"3H38 XAT"}="H383 XAT"; $resn_exch{"1H39 XAT"}="H391 XAT"; $resn_exch{"2H39 XAT"}="H392 XAT";
$resn_exch{"3H39 XAT"}="H393 XAT"; $resn_exch{"1H40 XAT"}="H401 XAT"; $resn_exch{"2H40 XAT"}="H402 XAT";
$resn_exch{"3H40 XAT"}="H403 XAT"; $resn_exch{"1HC3 XBB"}="HC31 XBB"; $resn_exch{"2HC3 XBB"}="HC32 XBB";
$resn_exch{"1HC6 XBB"}="HC61 XBB"; $resn_exch{"2HC6 XBB"}="HC62 XBB"; $resn_exch{"1HC4 XBB"}="HC41 XBB";
$resn_exch{"2HC4 XBB"}="HC42 XBB"; $resn_exch{"1HN7 XBB"}="HN71 XBB"; $resn_exch{"2HN7 XBB"}="HN72 XBB";
$resn_exch{"1H1  XBP"}=" H11 XBP"; $resn_exch{"2H1  XBP"}=" H12 XBP"; $resn_exch{"1H5  XBP"}=" H51 XBP";
$resn_exch{"2H5  XBP"}=" H52 XBP"; $resn_exch{"2HOP XBP"}="HOP2 XBP"; $resn_exch{"3HOP XBP"}="HOP3 XBP";
$resn_exch{"5HOP XBP"}="HOP5 XBP"; $resn_exch{"6HOP XBP"}="HOP6 XBP"; $resn_exch{"1H5C XC2"}="H5C1 XC2";
$resn_exch{"2H5C XC2"}="H5C2 XC2"; $resn_exch{"1H24 XC2"}="H241 XC2"; $resn_exch{"2H24 XC2"}="H242 XC2";
$resn_exch{"1H26 XC2"}="H261 XC2"; $resn_exch{"2H26 XC2"}="H262 XC2"; $resn_exch{"1H1C XC2"}="H1C1 XC2";
$resn_exch{"2H1C XC2"}="H1C2 XC2"; $resn_exch{"1H4C XC2"}="H4C1 XC2"; $resn_exch{"2H4C XC2"}="H4C2 XC2";
$resn_exch{"1H2C XC2"}="H2C1 XC2"; $resn_exch{"2H2C XC2"}="H2C2 XC2"; $resn_exch{"1H34 XC2"}="H341 XC2";
$resn_exch{"2H34 XC2"}="H342 XC2"; $resn_exch{"1H31 XC2"}="H311 XC2"; $resn_exch{"2H31 XC2"}="H312 XC2";
$resn_exch{"1H30 XC2"}="H301 XC2"; $resn_exch{"2H30 XC2"}="H302 XC2"; $resn_exch{"2HOP XCL"}="HOP2 XCL";
$resn_exch{"1H5* XCL"}=" H5' XCL"; $resn_exch{"2H5* XCL"}="H5'' XCL"; $resn_exch{" H4* XCL"}=" H4' XCL";
$resn_exch{" H6* XCL"}=" H6' XCL"; $resn_exch{" H7* XCL"}=" H7' XCL"; $resn_exch{" H3* XCL"}=" H3' XCL";
$resn_exch{"H3T  XCL"}="HO3' XCL"; $resn_exch{"1H2* XCL"}=" H2' XCL"; $resn_exch{"2H2* XCL"}="H2'' XCL";
$resn_exch{" H1* XCL"}=" H1' XCL"; $resn_exch{"1HN4 XCL"}="HN41 XCL"; $resn_exch{"2HN4 XCL"}="HN42 XCL";
$resn_exch{"3HOP XCL"}="HOP3 XCL"; $resn_exch{"1HN4 XCS"}="HN41 XCS"; $resn_exch{"2HN4 XCS"}="HN42 XCS";
$resn_exch{"1H5' XCS"}="H5'1 XCS"; $resn_exch{"2H5' XCS"}="H5'2 XCS"; $resn_exch{"1H7A XCS"}="H7A1 XCS";
$resn_exch{"2H7A XCS"}="H7A2 XCS"; $resn_exch{"3H7A XCS"}="H7A3 XCS"; $resn_exch{"1H2' XCS"}="H2'1 XCS";
$resn_exch{"2H2' XCS"}="H2'2 XCS"; $resn_exch{"H3T  XCS"}="HO3' XCS"; $resn_exch{"3HOP XCT"}="HOP3 XCT";
$resn_exch{"2HOP XCT"}="HOP2 XCT"; $resn_exch{"1H6' XCT"}="H6'1 XCT"; $resn_exch{"2H6' XCT"}="H6'2 XCT";
$resn_exch{"1H3' XCT"}="H3'1 XCT"; $resn_exch{"2H3' XCT"}="H3'2 XCT"; $resn_exch{"1H2' XCT"}="H2'1 XCT";
$resn_exch{"2H2' XCT"}="H2'2 XCT"; $resn_exch{"1HN4 XCT"}="HN41 XCT"; $resn_exch{"2HN4 XCT"}="HN42 XCT";
$resn_exch{"3HOP XCY"}="HOP3 XCY"; $resn_exch{"2HOP XCY"}="HOP2 XCY"; $resn_exch{"1HNZ XCY"}="HNZ1 XCY";
$resn_exch{"2HNZ XCY"}="HNZ2 XCY"; $resn_exch{"1HB  XCY"}=" HB1 XCY"; $resn_exch{"2HB  XCY"}=" HB2 XCY";
$resn_exch{"1HH  XCY"}=" HH1 XCY"; $resn_exch{"2HH  XCY"}=" HH2 XCY"; $resn_exch{"1H2* XCY"}=" H2' XCY";
$resn_exch{"2H2* XCY"}="H2'' XCY"; $resn_exch{"1H5* XCY"}=" H5' XCY"; $resn_exch{"2H5* XCY"}="H5'' XCY";
$resn_exch{" H4* XCY"}=" H4' XCY"; $resn_exch{" H1* XCY"}=" H1' XCY"; $resn_exch{" H3* XCY"}=" H3' XCY";
$resn_exch{"H3T  XCY"}="HO3' XCY"; $resn_exch{"1H4C XDL"}="H4C1 XDL"; $resn_exch{"2H4C XDL"}="H4C2 XDL";
$resn_exch{"1H5C XDL"}="H5C1 XDL"; $resn_exch{"2H5C XDL"}="H5C2 XDL"; $resn_exch{"1H6C XDL"}="H6C1 XDL";
$resn_exch{"2H6C XDL"}="H6C2 XDL"; $resn_exch{"1H1  XDN"}=" H11 XDN"; $resn_exch{"2H1  XDN"}=" H12 XDN";
$resn_exch{"1H5  XDN"}=" H51 XDN"; $resn_exch{"2H5  XDN"}=" H52 XDN"; $resn_exch{"1H1  XDP"}=" H11 XDP";
$resn_exch{"2H1  XDP"}=" H12 XDP"; $resn_exch{"1H5  XDP"}=" H51 XDP"; $resn_exch{"2H5  XDP"}=" H52 XDP";
$resn_exch{"1HO2 XDP"}="HO21 XDP"; $resn_exch{"2HO2 XDP"}="HO22 XDP"; $resn_exch{"2HOP XDP"}="HOP2 XDP";
$resn_exch{"3HOP XDP"}="HOP3 XDP"; $resn_exch{"5HOP XDP"}="HOP5 XDP"; $resn_exch{"6HOP XDP"}="HOP6 XDP";
$resn_exch{"1H1  XE4"}="H11A XE4"; $resn_exch{"2H1  XE4"}=" H12 XE4"; $resn_exch{"1H3  XE4"}=" H31 XE4";
$resn_exch{"2H3  XE4"}=" H32 XE4"; $resn_exch{"1H10 XE4"}="H101 XE4"; $resn_exch{"2H10 XE4"}="H102 XE4";
$resn_exch{"1H13 XE4"}="H131 XE4"; $resn_exch{"2H13 XE4"}="H132 XE4"; $resn_exch{"1H14 XE4"}="H141 XE4";
$resn_exch{"2H14 XE4"}="H142 XE4"; $resn_exch{"1H15 XE4"}="H151 XE4"; $resn_exch{"2H15 XE4"}="H152 XE4";
$resn_exch{"1H16 XE4"}="H161 XE4"; $resn_exch{"2H16 XE4"}="H162 XE4"; $resn_exch{"1H17 XE4"}="H171 XE4";
$resn_exch{"2H17 XE4"}="H172 XE4"; $resn_exch{"1H24 XE4"}="H241 XE4"; $resn_exch{"2H24 XE4"}="H242 XE4";
$resn_exch{"3H24 XE4"}="H243 XE4"; $resn_exch{"1H25 XE4"}="H251 XE4"; $resn_exch{"2H25 XE4"}="H252 XE4";
$resn_exch{"3H25 XE4"}="H253 XE4"; $resn_exch{"1H26 XE4"}="H261 XE4"; $resn_exch{"2H26 XE4"}="H262 XE4";
$resn_exch{"3H26 XE4"}="H263 XE4"; $resn_exch{"1H1C XED"}="H1C1 XED"; $resn_exch{"2H1C XED"}="H1C2 XED";
$resn_exch{"1H2C XED"}="H2C1 XED"; $resn_exch{"2H2C XED"}="H2C2 XED"; $resn_exch{"1H5C XED"}="H5C1 XED";
$resn_exch{"2H5C XED"}="H5C2 XED"; $resn_exch{"1H6C XED"}="H6C1 XED"; $resn_exch{"2H6C XED"}="H6C2 XED";
$resn_exch{"1H7C XED"}="H7C1 XED"; $resn_exch{"2H7C XED"}="H7C2 XED"; $resn_exch{"3H7C XED"}="H7C3 XED";
$resn_exch{"1H19 XED"}="H191 XED"; $resn_exch{"2H19 XED"}="H192 XED"; $resn_exch{"1H20 XED"}="H201 XED";
$resn_exch{"2H20 XED"}="H202 XED"; $resn_exch{"3H20 XED"}="H203 XED"; $resn_exch{"1H5' XGA"}="H5'1 XGA";
$resn_exch{"2H5' XGA"}="H5'2 XGA"; $resn_exch{"1H2' XGA"}="H2'1 XGA"; $resn_exch{"2H2' XGA"}="H2'2 XGA";
$resn_exch{"1HN2 XGA"}="HN21 XGA"; $resn_exch{"2HN2 XGA"}="HN22 XGA"; $resn_exch{"H3T  XGA"}="HO3' XGA";
$resn_exch{"2HOP XGL"}="HOP2 XGL"; $resn_exch{"1H5* XGL"}=" H5' XGL"; $resn_exch{"2H5* XGL"}="H5'' XGL";
$resn_exch{" H4* XGL"}=" H4' XGL"; $resn_exch{" H6* XGL"}=" H6' XGL"; $resn_exch{" H7* XGL"}=" H7' XGL";
$resn_exch{" H3* XGL"}=" H3' XGL"; $resn_exch{"H3T  XGL"}="HO3' XGL"; $resn_exch{"1H2* XGL"}=" H2' XGL";
$resn_exch{"2H2* XGL"}="H2'' XGL"; $resn_exch{" H1* XGL"}=" H1' XGL"; $resn_exch{"1HN2 XGL"}="HN21 XGL";
$resn_exch{"2HN2 XGL"}="HN22 XGL"; $resn_exch{"3HOP XGL"}="HOP3 XGL"; $resn_exch{"3HOP XGU"}="HOP3 XGU";
$resn_exch{"2HOP XGU"}="HOP2 XGU"; $resn_exch{"1H6' XGU"}="H6'1 XGU"; $resn_exch{"2H6' XGU"}="H6'2 XGU";
$resn_exch{"1H3' XGU"}="H3'1 XGU"; $resn_exch{"2H3' XGU"}="H3'2 XGU"; $resn_exch{"1H2' XGU"}="H2'1 XGU";
$resn_exch{"2H2' XGU"}="H2'2 XGU"; $resn_exch{"1HN2 XGU"}="HN21 XGU"; $resn_exch{"2HN2 XGU"}="HN22 XGU";
$resn_exch{"1H2  XIF"}=" H21 XIF"; $resn_exch{"2H2  XIF"}=" H22 XIF"; $resn_exch{"1H5  XIF"}=" H51 XIF";
$resn_exch{"2H5  XIF"}=" H52 XIF"; $resn_exch{"1H6  XIF"}=" H61 XIF"; $resn_exch{"2H6  XIF"}=" H62 XIF";
$resn_exch{"1H10 XIH"}="H101 XIH"; $resn_exch{"2H10 XIH"}="H102 XIH"; $resn_exch{"1H15 XIH"}="H151 XIH";
$resn_exch{"2H15 XIH"}="H152 XIH"; $resn_exch{"1H17 XIH"}="H171 XIH"; $resn_exch{"2H17 XIH"}="H172 XIH";
$resn_exch{"1H   XIH"}=" H1  XIH"; $resn_exch{"2H   XIH"}=" H2  XIH"; $resn_exch{"3H   XIH"}=" H3  XIH";
$resn_exch{"1H11 XIH"}="H111 XIH"; $resn_exch{"2H11 XIH"}="H112 XIH"; $resn_exch{"1H12 XIH"}="H121 XIH";
$resn_exch{"2H12 XIH"}="H122 XIH"; $resn_exch{"1H14 XIH"}="H141 XIH"; $resn_exch{"2H14 XIH"}="H142 XIH";
$resn_exch{"1H27 XIH"}="H271 XIH"; $resn_exch{"2H27 XIH"}="H272 XIH"; $resn_exch{"3H27 XIH"}="H273 XIH";
$resn_exch{"1H5  XIL"}=" H51 XIL"; $resn_exch{"2H5  XIL"}=" H52 XIL"; $resn_exch{"1H5' XIL"}="H5'1 XIL";
$resn_exch{"2H5' XIL"}="H5'2 XIL"; $resn_exch{"1H6' XIL"}="H6'1 XIL"; $resn_exch{"2H6' XIL"}="H6'2 XIL";
$resn_exch{"1H5  XIM"}=" H51 XIM"; $resn_exch{"2H5  XIM"}=" H52 XIM"; $resn_exch{"1H2  XK2"}="H21A XK2";
$resn_exch{"2H2  XK2"}="H22A XK2"; $resn_exch{"1H7  XK2"}="H71A XK2"; $resn_exch{"2H7  XK2"}=" H72 XK2";
$resn_exch{"1H31 XK2"}="H311 XK2"; $resn_exch{"2H31 XK2"}="H312 XK2"; $resn_exch{"1H61 XK2"}="H611 XK2";
$resn_exch{"2H61 XK2"}="H612 XK2"; $resn_exch{"1HA  XL1"}=" HA1 XL1"; $resn_exch{"2HA  XL1"}=" HA2 XL1";
$resn_exch{"3HA  XL1"}=" HA3 XL1"; $resn_exch{"1HB  XL1"}=" HB1 XL1"; $resn_exch{"2HB  XL1"}=" HB2 XL1";
$resn_exch{"1H26 XLC"}="H261 XLC"; $resn_exch{"2H26 XLC"}="H262 XLC"; $resn_exch{"1H29 XLC"}="H291 XLC";
$resn_exch{"2H29 XLC"}="H292 XLC"; $resn_exch{"1H30 XLC"}="H301 XLC"; $resn_exch{"2H30 XLC"}="H302 XLC";
$resn_exch{"1H33 XLC"}="H331 XLC"; $resn_exch{"2H33 XLC"}="H332 XLC"; $resn_exch{"1H34 XLC"}="H341 XLC";
$resn_exch{"2H34 XLC"}="H342 XLC"; $resn_exch{"1H35 XLC"}="H351 XLC"; $resn_exch{"2H35 XLC"}="H352 XLC";
$resn_exch{"3H35 XLC"}="H353 XLC"; $resn_exch{"1H14 XLD"}="H141 XLD"; $resn_exch{"1H16 XLD"}="H161 XLD";
$resn_exch{"1H18 XLD"}="H181 XLD"; $resn_exch{"2H18 XLD"}="H182 XLD"; $resn_exch{"3H18 XLD"}="H183 XLD";
$resn_exch{"1H19 XLD"}="H191 XLD"; $resn_exch{"1H24 XLD"}="H241 XLD"; $resn_exch{"1H28 XLD"}="H281 XLD";
$resn_exch{"2H28 XLD"}="H282 XLD"; $resn_exch{"1H31 XLD"}="H311 XLD"; $resn_exch{"2H31 XLD"}="H312 XLD";
$resn_exch{"3H31 XLD"}="H313 XLD"; $resn_exch{"1H34 XLD"}="H341 XLD"; $resn_exch{"2H34 XLD"}="H342 XLD";
$resn_exch{"1H35 XLD"}="H351 XLD"; $resn_exch{"2H35 XLD"}="H352 XLD"; $resn_exch{"1H5C XLM"}="H5C1 XLM";
$resn_exch{"2H5C XLM"}="H5C2 XLM"; $resn_exch{"1H6' XLM"}="H6'1 XLM"; $resn_exch{"2H6' XLM"}="H6'2 XLM";
$resn_exch{"1H7' XLM"}="H7'1 XLM"; $resn_exch{"2H7' XLM"}="H7'2 XLM"; $resn_exch{"3H7' XLM"}="H7'3 XLM";
$resn_exch{"1H5  XLS"}=" H51 XLS"; $resn_exch{"2H5  XLS"}=" H52 XLS"; $resn_exch{"1H17 XMA"}="H171 XMA";
$resn_exch{"2H17 XMA"}="H172 XMA"; $resn_exch{"1H20 XMA"}="H201 XMA"; $resn_exch{"2H20 XMA"}="H202 XMA";
$resn_exch{"1H22 XMA"}="H221 XMA"; $resn_exch{"2H22 XMA"}="H222 XMA"; $resn_exch{"3H22 XMA"}="H223 XMA";
$resn_exch{"1H23 XMA"}="H231 XMA"; $resn_exch{"2H23 XMA"}="H232 XMA"; $resn_exch{"1H25 XMA"}="H251 XMA";
$resn_exch{"2H25 XMA"}="H252 XMA"; $resn_exch{"1H30 XMA"}="H301 XMA"; $resn_exch{"2H30 XMA"}="H302 XMA";
$resn_exch{"1H32 XMA"}="H321 XMA"; $resn_exch{"2H32 XMA"}="H322 XMA"; $resn_exch{"1H35 XMA"}="H351 XMA";
$resn_exch{"2H35 XMA"}="H352 XMA"; $resn_exch{"1H38 XMA"}="H381 XMA"; $resn_exch{"2H38 XMA"}="H382 XMA";
$resn_exch{"1H4  XMB"}=" H41 XMB"; $resn_exch{"2H4  XMB"}=" H42 XMB"; $resn_exch{"1H5  XMB"}=" H51 XMB";
$resn_exch{"2H5  XMB"}=" H52 XMB"; $resn_exch{"1H9  XMB"}=" H91 XMB"; $resn_exch{"2H9  XMB"}=" H92 XMB";
$resn_exch{"1H10 XMB"}="H101 XMB"; $resn_exch{"2H10 XMB"}="H102 XMB"; $resn_exch{"1H26 XMB"}="H261 XMB";
$resn_exch{"2H26 XMB"}="H262 XMB"; $resn_exch{"1H29 XMB"}="H291 XMB"; $resn_exch{"2H29 XMB"}="H292 XMB";
$resn_exch{"1H34 XMB"}="H341 XMB"; $resn_exch{"2H34 XMB"}="H342 XMB"; $resn_exch{"1H36 XMB"}="H361 XMB";
$resn_exch{"2H36 XMB"}="H362 XMB"; $resn_exch{"1H38 XMB"}="H381 XMB"; $resn_exch{"2H38 XMB"}="H382 XMB";
$resn_exch{"1H41 XMB"}="H411 XMB"; $resn_exch{"2H41 XMB"}="H412 XMB"; $resn_exch{"1H16 XMC"}="H161 XMC";
$resn_exch{"2H16 XMC"}="H162 XMC"; $resn_exch{"1H22 XMC"}="H221 XMC"; $resn_exch{"2H22 XMC"}="H222 XMC";
$resn_exch{"1H23 XMC"}="H231 XMC"; $resn_exch{"2H23 XMC"}="H232 XMC"; $resn_exch{"1H25 XMC"}="H251 XMC";
$resn_exch{"2H25 XMC"}="H252 XMC"; $resn_exch{"1H26 XMC"}="H261 XMC"; $resn_exch{"2H26 XMC"}="H262 XMC";
$resn_exch{"1H27 XMC"}="H271 XMC"; $resn_exch{"2H27 XMC"}="H272 XMC"; $resn_exch{"1H30 XMC"}="H301 XMC";
$resn_exch{"2H30 XMC"}="H302 XMC"; $resn_exch{"1H35 XMC"}="H351 XMC"; $resn_exch{"2H35 XMC"}="H352 XMC";
$resn_exch{"1H37 XMC"}="H371 XMC"; $resn_exch{"2H37 XMC"}="H372 XMC"; $resn_exch{"1H39 XMC"}="H391 XMC";
$resn_exch{"2H39 XMC"}="H392 XMC"; $resn_exch{"1H42 XMC"}="H421 XMC"; $resn_exch{"2H42 XMC"}="H422 XMC";
$resn_exch{"1H17 XMD"}="H171 XMD"; $resn_exch{"2H17 XMD"}="H172 XMD"; $resn_exch{"1H18 XMD"}="H181 XMD";
$resn_exch{"2H18 XMD"}="H182 XMD"; $resn_exch{"3H18 XMD"}="H183 XMD"; $resn_exch{"1H20 XMD"}="H201 XMD";
$resn_exch{"2H20 XMD"}="H202 XMD"; $resn_exch{"1H24 XMD"}="H241 XMD"; $resn_exch{"2H24 XMD"}="H242 XMD";
$resn_exch{"1H27 XMD"}="H271 XMD"; $resn_exch{"2H27 XMD"}="H272 XMD"; $resn_exch{"1H32 XMD"}="H321 XMD";
$resn_exch{"2H32 XMD"}="H322 XMD"; $resn_exch{"1H34 XMD"}="H341 XMD"; $resn_exch{"2H34 XMD"}="H342 XMD";
$resn_exch{"1H36 XMD"}="H361 XMD"; $resn_exch{"2H36 XMD"}="H362 XMD"; $resn_exch{"1H39 XMD"}="H391 XMD";
$resn_exch{"2H39 XMD"}="H392 XMD"; $resn_exch{"1H16 XME"}="H161 XME"; $resn_exch{"2H16 XME"}="H162 XME";
$resn_exch{"1H18 XME"}="H181 XME"; $resn_exch{"2H18 XME"}="H182 XME"; $resn_exch{"1H19 XME"}="H191 XME";
$resn_exch{"2H19 XME"}="H192 XME"; $resn_exch{"1H22 XME"}="H221 XME"; $resn_exch{"2H22 XME"}="H222 XME";
$resn_exch{"1H27 XME"}="H271 XME"; $resn_exch{"2H27 XME"}="H272 XME"; $resn_exch{"1H29 XME"}="H291 XME";
$resn_exch{"2H29 XME"}="H292 XME"; $resn_exch{"1H34 XME"}="H341 XME"; $resn_exch{"2H34 XME"}="H342 XME";
$resn_exch{"1H37 XME"}="H371 XME"; $resn_exch{"2H37 XME"}="H372 XME"; $resn_exch{"1H14 XMF"}="H141 XMF";
$resn_exch{"2H14 XMF"}="H142 XMF"; $resn_exch{"1H15 XMF"}="H151 XMF"; $resn_exch{"2H15 XMF"}="H152 XMF";
$resn_exch{"1H19 XMF"}="H191 XMF"; $resn_exch{"2H19 XMF"}="H192 XMF"; $resn_exch{"1H22 XMF"}="H221 XMF";
$resn_exch{"2H22 XMF"}="H222 XMF"; $resn_exch{"1H27 XMF"}="H271 XMF"; $resn_exch{"2H27 XMF"}="H272 XMF";
$resn_exch{"1H29 XMF"}="H291 XMF"; $resn_exch{"2H29 XMF"}="H292 XMF"; $resn_exch{"1H31 XMF"}="H311 XMF";
$resn_exch{"2H31 XMF"}="H312 XMF"; $resn_exch{"1H33 XMF"}="H331 XMF"; $resn_exch{"2H33 XMF"}="H332 XMF";
$resn_exch{"1H36 XMF"}="H361 XMF"; $resn_exch{"2H36 XMF"}="H362 XMF"; $resn_exch{"1H1  XMG"}="H11A XMG";
$resn_exch{"2H1  XMG"}=" H12 XMG"; $resn_exch{"1H4  XMG"}=" H41 XMG"; $resn_exch{"2H4  XMG"}=" H42 XMG";
$resn_exch{"1H5  XMG"}=" H51 XMG"; $resn_exch{"2H5  XMG"}=" H52 XMG"; $resn_exch{"1H8  XMG"}=" H81 XMG";
$resn_exch{"2H8  XMG"}=" H82 XMG"; $resn_exch{"1H29 XMG"}="H291 XMG"; $resn_exch{"2H29 XMG"}="H292 XMG";
$resn_exch{"1H31 XMG"}="H311 XMG"; $resn_exch{"2H31 XMG"}="H312 XMG"; $resn_exch{"1H33 XMG"}="H331 XMG";
$resn_exch{"2H33 XMG"}="H332 XMG"; $resn_exch{"1H36 XMG"}="H361 XMG"; $resn_exch{"2H36 XMG"}="H362 XMG";
$resn_exch{"1H1  XMH"}=" H11 XMH"; $resn_exch{"2H1  XMH"}="H12A XMH"; $resn_exch{"1H6  XMH"}=" H61 XMH";
$resn_exch{"2H6  XMH"}=" H62 XMH"; $resn_exch{"1H8  XMH"}=" H81 XMH"; $resn_exch{"2H8  XMH"}=" H82 XMH";
$resn_exch{"1H10 XMH"}="H101 XMH"; $resn_exch{"2H10 XMH"}="H102 XMH"; $resn_exch{"1H13 XMH"}="H131 XMH";
$resn_exch{"2H13 XMH"}="H132 XMH"; $resn_exch{"1H17 XMH"}="H171 XMH"; $resn_exch{"2H17 XMH"}="H172 XMH";
$resn_exch{"1H19 XMH"}="H191 XMH"; $resn_exch{"2H19 XMH"}="H192 XMH"; $resn_exch{"1H20 XMH"}="H201 XMH";
$resn_exch{"2H20 XMH"}="H202 XMH"; $resn_exch{"1H36 XMH"}="H361 XMH"; $resn_exch{"2H36 XMH"}="H362 XMH";
$resn_exch{"1H37 XMH"}="H371 XMH"; $resn_exch{"2H37 XMH"}="H372 XMH"; $resn_exch{"3H37 XMH"}="H373 XMH";
$resn_exch{"1H5  XMI"}=" H51 XMI"; $resn_exch{"2H5  XMI"}=" H52 XMI"; $resn_exch{"1H7  XMI"}=" H71 XMI";
$resn_exch{"2H7  XMI"}=" H72 XMI"; $resn_exch{"1H8  XMI"}=" H81 XMI"; $resn_exch{"2H8  XMI"}=" H82 XMI";
$resn_exch{"1H11 XMI"}="H111 XMI"; $resn_exch{"2H11 XMI"}="H112 XMI"; $resn_exch{"1H16 XMI"}="H161 XMI";
$resn_exch{"2H16 XMI"}="H162 XMI"; $resn_exch{"1H18 XMI"}="H181 XMI"; $resn_exch{"2H18 XMI"}="H182 XMI";
$resn_exch{"1H20 XMI"}="H201 XMI"; $resn_exch{"2H20 XMI"}="H202 XMI"; $resn_exch{"1H23 XMI"}="H231 XMI";
$resn_exch{"2H23 XMI"}="H232 XMI"; $resn_exch{"1H10 XMJ"}="H101 XMJ"; $resn_exch{"2H10 XMJ"}="H102 XMJ";
$resn_exch{"1H13 XMJ"}="H131 XMJ"; $resn_exch{"2H13 XMJ"}="H132 XMJ"; $resn_exch{"1H14 XMJ"}="H141 XMJ";
$resn_exch{"2H14 XMJ"}="H142 XMJ"; $resn_exch{"1H20 XMJ"}="H201 XMJ"; $resn_exch{"2H20 XMJ"}="H202 XMJ";
$resn_exch{"1H25 XMJ"}="H251 XMJ"; $resn_exch{"2H25 XMJ"}="H252 XMJ"; $resn_exch{"1H27 XMJ"}="H271 XMJ";
$resn_exch{"2H27 XMJ"}="H272 XMJ"; $resn_exch{"1H29 XMJ"}="H291 XMJ"; $resn_exch{"2H29 XMJ"}="H292 XMJ";
$resn_exch{"1H32 XMJ"}="H321 XMJ"; $resn_exch{"2H32 XMJ"}="H322 XMJ"; $resn_exch{"1H1  XMK"}=" H11 XMK";
$resn_exch{"2H1  XMK"}="H12A XMK"; $resn_exch{"1H6  XMK"}=" H61 XMK"; $resn_exch{"2H6  XMK"}=" H62 XMK";
$resn_exch{"1H8  XMK"}=" H81 XMK"; $resn_exch{"2H8  XMK"}=" H82 XMK"; $resn_exch{"1H10 XMK"}="H101 XMK";
$resn_exch{"2H10 XMK"}="H102 XMK"; $resn_exch{"1H13 XMK"}="H131 XMK"; $resn_exch{"2H13 XMK"}="H132 XMK";
$resn_exch{"1H21 XMK"}="H211 XMK"; $resn_exch{"2H21 XMK"}="H212 XMK"; $resn_exch{"1H22 XMK"}="H221 XMK";
$resn_exch{"2H22 XMK"}="H222 XMK"; $resn_exch{"1H25 XMK"}="H251 XMK"; $resn_exch{"2H25 XMK"}="H252 XMK";
$resn_exch{"1H26 XMK"}="H261 XMK"; $resn_exch{"2H26 XMK"}="H262 XMK"; $resn_exch{"1H6  XMM"}=" H61 XMM";
$resn_exch{"2H6  XMM"}=" H62 XMM"; $resn_exch{"1H1  XN1"}="H11A XN1"; $resn_exch{"2H1  XN1"}=" H12 XN1";
$resn_exch{"1H5  XN1"}=" H51 XN1"; $resn_exch{"2H5  XN1"}=" H52 XN1"; $resn_exch{"3H5  XN1"}=" H53 XN1";
$resn_exch{"1H6  XN1"}=" H61 XN1"; $resn_exch{"2H6  XN1"}=" H62 XN1"; $resn_exch{"3H6  XN1"}=" H63 XN1";
$resn_exch{"1H7  XN1"}=" H71 XN1"; $resn_exch{"2H7  XN1"}=" H72 XN1"; $resn_exch{"3H7  XN1"}=" H73 XN1";
$resn_exch{"1H8  XN1"}=" H81 XN1"; $resn_exch{"2H8  XN1"}=" H82 XN1"; $resn_exch{"1H9  XN1"}=" H91 XN1";
$resn_exch{"2H9  XN1"}=" H92 XN1"; $resn_exch{"1H10 XN1"}="H101 XN1"; $resn_exch{"2H10 XN1"}="H102 XN1";
$resn_exch{"1H12 XN1"}="H121 XN1"; $resn_exch{"2H12 XN1"}="H122 XN1"; $resn_exch{"1H14 XN1"}="H141 XN1";
$resn_exch{"1H24 XN1"}="H241 XN1"; $resn_exch{"2H24 XN1"}="H242 XN1"; $resn_exch{"1H31 XN1"}="H311 XN1";
$resn_exch{"2H31 XN1"}="H312 XN1"; $resn_exch{"1H37 XN1"}="H371 XN1"; $resn_exch{"2H37 XN1"}="H372 XN1";
$resn_exch{"3H37 XN1"}="H373 XN1"; $resn_exch{"1H1  XN2"}="H11A XN2"; $resn_exch{"2H1  XN2"}=" H12 XN2";
$resn_exch{"1H5  XN2"}=" H51 XN2"; $resn_exch{"2H5  XN2"}=" H52 XN2"; $resn_exch{"3H5  XN2"}=" H53 XN2";
$resn_exch{"1H6  XN2"}=" H61 XN2"; $resn_exch{"2H6  XN2"}=" H62 XN2"; $resn_exch{"3H6  XN2"}=" H63 XN2";
$resn_exch{"1H7  XN2"}=" H71 XN2"; $resn_exch{"2H7  XN2"}=" H72 XN2"; $resn_exch{"3H7  XN2"}=" H73 XN2";
$resn_exch{"1H8  XN2"}=" H81 XN2"; $resn_exch{"2H8  XN2"}=" H82 XN2"; $resn_exch{"1H9  XN2"}=" H91 XN2";
$resn_exch{"2H9  XN2"}=" H92 XN2"; $resn_exch{"1H10 XN2"}="H101 XN2"; $resn_exch{"2H10 XN2"}="H102 XN2";
$resn_exch{"1H12 XN2"}="H121 XN2"; $resn_exch{"2H12 XN2"}="H122 XN2"; $resn_exch{"1H14 XN2"}="H141 XN2";
$resn_exch{"1H24 XN2"}="H241 XN2"; $resn_exch{"2H24 XN2"}="H242 XN2"; $resn_exch{"1H31 XN2"}="H311 XN2";
$resn_exch{"2H31 XN2"}="H312 XN2"; $resn_exch{"1H38 XN2"}="H381 XN2"; $resn_exch{"2H38 XN2"}="H382 XN2";
$resn_exch{"1H39 XN2"}="H391 XN2"; $resn_exch{"2H39 XN2"}="H392 XN2"; $resn_exch{"3H39 XN2"}="H393 XN2";
$resn_exch{"1H1  XN3"}="H11A XN3"; $resn_exch{"2H1  XN3"}=" H12 XN3"; $resn_exch{"1H5  XN3"}=" H51 XN3";
$resn_exch{"2H5  XN3"}=" H52 XN3"; $resn_exch{"3H5  XN3"}=" H53 XN3"; $resn_exch{"1H6  XN3"}=" H61 XN3";
$resn_exch{"2H6  XN3"}=" H62 XN3"; $resn_exch{"3H6  XN3"}=" H63 XN3"; $resn_exch{"1H7  XN3"}=" H71 XN3";
$resn_exch{"2H7  XN3"}=" H72 XN3"; $resn_exch{"3H7  XN3"}=" H73 XN3"; $resn_exch{"1H8  XN3"}=" H81 XN3";
$resn_exch{"2H8  XN3"}=" H82 XN3"; $resn_exch{"1H9  XN3"}=" H91 XN3"; $resn_exch{"2H9  XN3"}=" H92 XN3";
$resn_exch{"1H10 XN3"}="H101 XN3"; $resn_exch{"2H10 XN3"}="H102 XN3"; $resn_exch{"1H12 XN3"}="H121 XN3";
$resn_exch{"2H12 XN3"}="H122 XN3"; $resn_exch{"1H14 XN3"}="H141 XN3"; $resn_exch{"2H14 XN3"}="H142 XN3";
$resn_exch{"1H24 XN3"}="H241 XN3"; $resn_exch{"2H24 XN3"}="H242 XN3"; $resn_exch{"1H31 XN3"}="H311 XN3";
$resn_exch{"2H31 XN3"}="H312 XN3"; $resn_exch{"1H38 XN3"}="H381 XN3"; $resn_exch{"2H38 XN3"}="H382 XN3";
$resn_exch{"1H2  XPE"}=" H21 XPE"; $resn_exch{"2H2  XPE"}=" H22 XPE"; $resn_exch{"1H3  XPE"}="H31A XPE";
$resn_exch{"2H3  XPE"}=" H32 XPE"; $resn_exch{"1H5  XPE"}=" H51 XPE"; $resn_exch{"2H5  XPE"}=" H52 XPE";
$resn_exch{"1H6  XPE"}=" H61 XPE"; $resn_exch{"2H6  XPE"}=" H62 XPE"; $resn_exch{"1H8  XPE"}=" H81 XPE";
$resn_exch{"2H8  XPE"}=" H82 XPE"; $resn_exch{"1H9  XPE"}=" H91 XPE"; $resn_exch{"2H9  XPE"}=" H92 XPE";
$resn_exch{"1H11 XPE"}="H111 XPE"; $resn_exch{"2H11 XPE"}="H112 XPE"; $resn_exch{"1H12 XPE"}="H121 XPE";
$resn_exch{"2H12 XPE"}="H122 XPE"; $resn_exch{"1H14 XPE"}="H141 XPE"; $resn_exch{"2H14 XPE"}="H142 XPE";
$resn_exch{"1H15 XPE"}="H151 XPE"; $resn_exch{"2H15 XPE"}="H152 XPE"; $resn_exch{"1H17 XPE"}="H171 XPE";
$resn_exch{"2H17 XPE"}="H172 XPE"; $resn_exch{"1H18 XPE"}="H181 XPE"; $resn_exch{"2H18 XPE"}="H182 XPE";
$resn_exch{"1H20 XPE"}="H201 XPE"; $resn_exch{"2H20 XPE"}="H202 XPE"; $resn_exch{"1H21 XPE"}="H211 XPE";
$resn_exch{"2H21 XPE"}="H212 XPE"; $resn_exch{"1H23 XPE"}="H231 XPE"; $resn_exch{"2H23 XPE"}="H232 XPE";
$resn_exch{"1H24 XPE"}="H241 XPE"; $resn_exch{"2H24 XPE"}="H242 XPE"; $resn_exch{"1H26 XPE"}="H261 XPE";
$resn_exch{"2H26 XPE"}="H262 XPE"; $resn_exch{"1H27 XPE"}="H271 XPE"; $resn_exch{"2H27 XPE"}="H272 XPE";
$resn_exch{"1H29 XPE"}="H291 XPE"; $resn_exch{"2H29 XPE"}="H292 XPE"; $resn_exch{"1H30 XPE"}="H301 XPE";
$resn_exch{"2H30 XPE"}="H302 XPE"; $resn_exch{"1H9  XPP"}=" H91 XPP"; $resn_exch{"2H9  XPP"}=" H92 XPP";
$resn_exch{"3H9  XPP"}=" H93 XPP"; $resn_exch{"1H15 XPP"}="H151 XPP"; $resn_exch{"2H15 XPP"}="H152 XPP";
$resn_exch{"3H15 XPP"}="H153 XPP"; $resn_exch{"1H21 XPP"}="H211 XPP"; $resn_exch{"2H21 XPP"}="H212 XPP";
$resn_exch{"3H21 XPP"}="H213 XPP"; $resn_exch{"1H22 XPP"}="H221 XPP"; $resn_exch{"2H22 XPP"}="H222 XPP";
$resn_exch{"3H22 XPP"}="H223 XPP"; $resn_exch{"1H20 XPP"}="H201 XPP"; $resn_exch{"2H20 XPP"}="H202 XPP";
$resn_exch{"1H18 XPP"}="H181 XPP"; $resn_exch{"2H18 XPP"}="H182 XPP"; $resn_exch{"1H19 XPP"}="H191 XPP";
$resn_exch{"2H19 XPP"}="H192 XPP"; $resn_exch{"1H3  XPP"}=" H31 XPP"; $resn_exch{"2H3  XPP"}="H32A XPP";
$resn_exch{"3H3  XPP"}=" H33 XPP"; $resn_exch{"1H28 XPP"}="H281 XPP"; $resn_exch{"2H28 XPP"}="H282 XPP";
$resn_exch{"3H28 XPP"}="H283 XPP"; $resn_exch{"1H30 XPP"}="H301 XPP"; $resn_exch{"2H30 XPP"}="H302 XPP";
$resn_exch{"1H29 XPP"}="H291 XPP"; $resn_exch{"2H29 XPP"}="H292 XPP"; $resn_exch{"1H46 XPX"}="H461 XPX";
$resn_exch{"2H46 XPX"}="H462 XPX"; $resn_exch{"3H46 XPX"}="H463 XPX"; $resn_exch{"1H45 XPX"}="H451 XPX";
$resn_exch{"2H45 XPX"}="H452 XPX"; $resn_exch{"1H44 XPX"}="H441 XPX"; $resn_exch{"2H44 XPX"}="H442 XPX";
$resn_exch{"1H43 XPX"}="H431 XPX"; $resn_exch{"2H43 XPX"}="H432 XPX"; $resn_exch{"1H42 XPX"}="H421 XPX";
$resn_exch{"2H42 XPX"}="H422 XPX"; $resn_exch{"1H41 XPX"}="H411 XPX"; $resn_exch{"2H41 XPX"}="H412 XPX";
$resn_exch{"1H40 XPX"}="H401 XPX"; $resn_exch{"2H40 XPX"}="H402 XPX"; $resn_exch{"1H39 XPX"}="H391 XPX";
$resn_exch{"2H39 XPX"}="H392 XPX"; $resn_exch{"1H38 XPX"}="H381 XPX"; $resn_exch{"2H38 XPX"}="H382 XPX";
$resn_exch{"1H37 XPX"}="H371 XPX"; $resn_exch{"2H37 XPX"}="H372 XPX"; $resn_exch{"1H36 XPX"}="H361 XPX";
$resn_exch{"2H36 XPX"}="H362 XPX"; $resn_exch{"1H35 XPX"}="H351 XPX"; $resn_exch{"2H35 XPX"}="H352 XPX";
$resn_exch{"1H34 XPX"}="H341 XPX"; $resn_exch{"2H34 XPX"}="H342 XPX"; $resn_exch{"1H33 XPX"}="H331 XPX";
$resn_exch{"2H33 XPX"}="H332 XPX"; $resn_exch{"1H32 XPX"}="H321 XPX"; $resn_exch{"2H32 XPX"}="H322 XPX";
$resn_exch{"1H30 XPX"}="H301 XPX"; $resn_exch{"2H30 XPX"}="H302 XPX"; $resn_exch{"1H15 XPX"}="H151 XPX";
$resn_exch{"2H15 XPX"}="H152 XPX"; $resn_exch{"1H16 XPX"}="H161 XPX"; $resn_exch{"2H16 XPX"}="H162 XPX";
$resn_exch{"1H17 XPX"}="H171 XPX"; $resn_exch{"2H17 XPX"}="H172 XPX"; $resn_exch{"1H18 XPX"}="H181 XPX";
$resn_exch{"2H18 XPX"}="H182 XPX"; $resn_exch{"1H19 XPX"}="H191 XPX"; $resn_exch{"2H19 XPX"}="H192 XPX";
$resn_exch{"1H20 XPX"}="H201 XPX"; $resn_exch{"2H20 XPX"}="H202 XPX"; $resn_exch{"1H21 XPX"}="H211 XPX";
$resn_exch{"2H21 XPX"}="H212 XPX"; $resn_exch{"1H22 XPX"}="H221 XPX"; $resn_exch{"2H22 XPX"}="H222 XPX";
$resn_exch{"1H23 XPX"}="H231 XPX"; $resn_exch{"2H23 XPX"}="H232 XPX"; $resn_exch{"1H24 XPX"}="H241 XPX";
$resn_exch{"2H24 XPX"}="H242 XPX"; $resn_exch{"1H25 XPX"}="H251 XPX"; $resn_exch{"2H25 XPX"}="H252 XPX";
$resn_exch{"1H26 XPX"}="H261 XPX"; $resn_exch{"2H26 XPX"}="H262 XPX"; $resn_exch{"1H27 XPX"}="H271 XPX";
$resn_exch{"2H27 XPX"}="H272 XPX"; $resn_exch{"1H28 XPX"}="H281 XPX"; $resn_exch{"2H28 XPX"}="H282 XPX";
$resn_exch{"1H29 XPX"}="H291 XPX"; $resn_exch{"2H29 XPX"}="H292 XPX"; $resn_exch{"3H29 XPX"}="H293 XPX";
$resn_exch{"1H12 XPX"}="H121 XPX"; $resn_exch{"2H12 XPX"}="H122 XPX"; $resn_exch{"1H11 XPX"}="H111 XPX";
$resn_exch{"2H11 XPX"}="H112 XPX"; $resn_exch{"1H5  XPX"}="H51A XPX"; $resn_exch{"2H5  XPX"}="H52A XPX";
$resn_exch{"1H6  XS2"}=" H61 XS2"; $resn_exch{"2H6  XS2"}=" H62 XS2"; $resn_exch{"3H6  XS2"}=" H63 XS2";
$resn_exch{"1H5  XS2"}=" H51 XS2"; $resn_exch{"2H5  XS2"}=" H52 XS2"; $resn_exch{"1H11 XS2"}="H111 XS2";
$resn_exch{"2H11 XS2"}="H112 XS2"; $resn_exch{"1H17 XS2"}="H171 XS2"; $resn_exch{"2H17 XS2"}="H172 XS2";
$resn_exch{"1H23 XS2"}="H231 XS2"; $resn_exch{"2H23 XS2"}="H232 XS2"; $resn_exch{"1H29 XS2"}="H291 XS2";
$resn_exch{"2H29 XS2"}="H292 XS2"; $resn_exch{"1H6  XTG"}=" H61 XTG"; $resn_exch{"2H6  XTG"}=" H62 XTG";
$resn_exch{"1H11 XTG"}="H111 XTG"; $resn_exch{"2H11 XTG"}="H112 XTG"; $resn_exch{"3HOP XTH"}="HOP3 XTH";
$resn_exch{"2HOP XTH"}="HOP2 XTH"; $resn_exch{"1H6' XTH"}="H6'1 XTH"; $resn_exch{"2H6' XTH"}="H6'2 XTH";
$resn_exch{"1H3' XTH"}="H3'1 XTH"; $resn_exch{"2H3' XTH"}="H3'2 XTH"; $resn_exch{"1H2' XTH"}="H2'1 XTH";
$resn_exch{"2H2' XTH"}="H2'2 XTH"; $resn_exch{"1H5M XTH"}=" H71 XTH"; $resn_exch{"2H5M XTH"}=" H72 XTH";
$resn_exch{"3H5M XTH"}=" H73 XTH"; $resn_exch{"2HOP XTL"}="HOP2 XTL"; $resn_exch{"1H5* XTL"}=" H5' XTL";
$resn_exch{"2H5* XTL"}="H5'' XTL"; $resn_exch{" H4* XTL"}=" H4' XTL"; $resn_exch{" H6* XTL"}=" H6' XTL";
$resn_exch{" H7* XTL"}=" H7' XTL"; $resn_exch{" H3* XTL"}=" H3' XTL"; $resn_exch{"H3T  XTL"}="HO3' XTL";
$resn_exch{"1H2* XTL"}=" H2' XTL"; $resn_exch{"2H2* XTL"}="H2'' XTL"; $resn_exch{" H1* XTL"}=" H1' XTL";
$resn_exch{"1H5M XTL"}=" H71 XTL"; $resn_exch{"2H5M XTL"}=" H72 XTL"; $resn_exch{"3H5M XTL"}=" H73 XTL";
$resn_exch{"3HOP XTL"}="HOP3 XTL"; $resn_exch{" H1* XTS"}=" H1' XTS"; $resn_exch{" H4* XTS"}=" H4' XTS";
$resn_exch{"1H5* XTS"}="H5'1 XTS"; $resn_exch{"2H5* XTS"}="H5'2 XTS"; $resn_exch{"HO5* XTS"}="HO5' XTS";
$resn_exch{" H3* XTS"}=" H3' XTS"; $resn_exch{"HO3* XTS"}="HO3' XTS"; $resn_exch{" H2* XTS"}=" H2' XTS";
$resn_exch{"HO2* XTS"}="HO2' XTS"; $resn_exch{"1H5' XTY"}="H5'1 XTY"; $resn_exch{"2H5' XTY"}="H5'2 XTY";
$resn_exch{"1H2' XTY"}="H2'1 XTY"; $resn_exch{"2H2' XTY"}="H2'2 XTY"; $resn_exch{"1H7A XTY"}="H7A1 XTY";
$resn_exch{"2H7A XTY"}="H7A2 XTY"; $resn_exch{"3H7A XTY"}="H7A3 XTY"; $resn_exch{"H3T  XTY"}="HO3' XTY";
$resn_exch{"1HN2 XUG"}="HN21 XUG"; $resn_exch{"2HN2 XUG"}="HN22 XUG"; $resn_exch{" H1* XUG"}=" H1' XUG";
$resn_exch{" H2* XUG"}=" H2' XUG"; $resn_exch{"1HA* XUG"}="HA'1 XUG"; $resn_exch{"2HA* XUG"}="HA'2 XUG";
$resn_exch{"3HA* XUG"}="HA'3 XUG"; $resn_exch{" H3* XUG"}=" H3' XUG"; $resn_exch{"H3T  XUG"}="HO3' XUG";
$resn_exch{" H4* XUG"}=" H4' XUG"; $resn_exch{"1H5* XUG"}=" H5' XUG"; $resn_exch{"2H5* XUG"}="H5'' XUG";
$resn_exch{"2HOP XUG"}="HOP2 XUG"; $resn_exch{"3HOP XUG"}="HOP3 XUG"; $resn_exch{"1H1  XUL"}=" H11 XUL";
$resn_exch{"2H1  XUL"}=" H12 XUL"; $resn_exch{"1H5  XUL"}=" H51 XUL"; $resn_exch{"2H5  XUL"}=" H52 XUL";
$resn_exch{"1HO4 XV6"}="HO41 XV6"; $resn_exch{"1HO5 XV6"}="HO51 XV6"; $resn_exch{"1H2  XV6"}=" H21 XV6";
$resn_exch{"2H2  XV6"}=" H22 XV6"; $resn_exch{"1H3  XV6"}=" H31 XV6"; $resn_exch{"1H4  XV6"}=" H41 XV6";
$resn_exch{"1H5  XV6"}=" H51 XV6"; $resn_exch{"1H6  XV6"}=" H61 XV6"; $resn_exch{"1H7  XV6"}=" H71 XV6";
$resn_exch{"2H7  XV6"}=" H72 XV6"; $resn_exch{"1H31 XV6"}="H311 XV6"; $resn_exch{"2H31 XV6"}="H312 XV6";
$resn_exch{"1H33 XV6"}="H331 XV6"; $resn_exch{"1H34 XV6"}="H341 XV6"; $resn_exch{"1H35 XV6"}="H351 XV6";
$resn_exch{"1H36 XV6"}="H361 XV6"; $resn_exch{"1H37 XV6"}="H371 XV6"; $resn_exch{"1H61 XV6"}="H611 XV6";
$resn_exch{"2H61 XV6"}="H612 XV6"; $resn_exch{"1H63 XV6"}="H631 XV6"; $resn_exch{"1H64 XV6"}="H641 XV6";
$resn_exch{"1H65 XV6"}="H651 XV6"; $resn_exch{"1H66 XV6"}="H661 XV6"; $resn_exch{"1H67 XV6"}="H671 XV6";
$resn_exch{"1H21 XV6"}="H211 XV6"; $resn_exch{"1H23 XV6"}="H231 XV6"; $resn_exch{"1H24 XV6"}="H241 XV6";
$resn_exch{"1H25 XV6"}="H251 XV6"; $resn_exch{"1H28 XV6"}="H281 XV6"; $resn_exch{"1H29 XV6"}="H291 XV6";
$resn_exch{"1H71 XV6"}="H711 XV6"; $resn_exch{"1H73 XV6"}="H731 XV6"; $resn_exch{"1H74 XV6"}="H741 XV6";
$resn_exch{"1H75 XV6"}="H751 XV6"; $resn_exch{"1H78 XV6"}="H781 XV6"; $resn_exch{"1H79 XV6"}="H791 XV6";
$resn_exch{"1HN  XX1"}=" H   XX1"; $resn_exch{"2HN  XX1"}=" H2  XX1"; $resn_exch{"1HB  XX1"}=" HB2 XX1";
$resn_exch{"2HB  XX1"}=" HB3 XX1"; $resn_exch{"1HG  XX1"}=" HG2 XX1"; $resn_exch{"2HG  XX1"}=" HG3 XX1";
$resn_exch{"1HD  XX1"}=" HD2 XX1"; $resn_exch{"2HD  XX1"}=" HD3 XX1"; $resn_exch{"1HE  XX1"}=" HE2 XX1";
$resn_exch{"2HE  XX1"}=" HE3 XX1"; $resn_exch{" HNZ XX1"}=" HZ  XX1"; $resn_exch{" H2  XX1"}=" HC2 XX1";
$resn_exch{"1H3  XX3"}=" H31 XX3"; $resn_exch{"1H01 XX4"}="H011 XX4"; $resn_exch{"2H01 XX4"}="H012 XX4";
$resn_exch{"3H01 XX4"}="H013 XX4"; $resn_exch{"1H05 XX4"}="H051 XX4"; $resn_exch{"2H05 XX4"}="H052 XX4";
$resn_exch{"1H06 XX4"}="H061 XX4"; $resn_exch{"2H06 XX4"}="H062 XX4"; $resn_exch{"1H08 XX4"}="H081 XX4";
$resn_exch{"2H08 XX4"}="H082 XX4"; $resn_exch{"1H25 XX4"}="H251 XX4"; $resn_exch{"2H25 XX4"}="H252 XX4";
$resn_exch{"1H27 XX4"}="H271 XX4"; $resn_exch{"2H27 XX4"}="H272 XX4"; $resn_exch{"1H28 XX4"}="H281 XX4";
$resn_exch{"2H28 XX4"}="H282 XX4"; $resn_exch{"1H29 XX4"}="H291 XX4"; $resn_exch{"2H29 XX4"}="H292 XX4";
$resn_exch{"1H30 XX4"}="H301 XX4"; $resn_exch{"2H30 XX4"}="H302 XX4"; $resn_exch{"1H31 XX4"}="H311 XX4";
$resn_exch{"2H31 XX4"}="H312 XX4"; $resn_exch{"1HC5 XX5"}="HC51 XX5"; $resn_exch{"2HC5 XX5"}="HC52 XX5";
$resn_exch{"1HC7 XX5"}="HC71 XX5"; $resn_exch{"2HC7 XX5"}="HC72 XX5"; $resn_exch{"3HC7 XX5"}="HC73 XX5";
$resn_exch{"1HC8 XX5"}="HC81 XX5"; $resn_exch{"2HC8 XX5"}="HC82 XX5"; $resn_exch{"3HC8 XX5"}="HC83 XX5";
$resn_exch{"1H14 XX5"}="H141 XX5"; $resn_exch{"2H14 XX5"}="H142 XX5"; $resn_exch{"1H20 XX5"}="H201 XX5";
$resn_exch{"2H20 XX5"}="H202 XX5"; $resn_exch{"1HN1 XXX"}="HN11 XXX"; $resn_exch{"2HN1 XXX"}="HN12 XXX";
$resn_exch{"1H7  XXX"}=" H71 XXX"; $resn_exch{"2H7  XXX"}=" H72 XXX"; $resn_exch{"1HN2 XXX"}="HN21 XXX";
$resn_exch{"2HN2 XXX"}="HN22 XXX"; $resn_exch{"1H12 XXX"}="H121 XXX"; $resn_exch{"2H12 XXX"}="H122 XXX";
$resn_exch{"1HN3 XXX"}="HN31 XXX"; $resn_exch{"2HN3 XXX"}="HN32 XXX"; $resn_exch{"1HN4 XXX"}="HN41 XXX";
$resn_exch{"2HN4 XXX"}="HN42 XXX"; $resn_exch{"1HN1 XXY"}="HN11 XXY"; $resn_exch{"2HN1 XXY"}="HN12 XXY";
$resn_exch{"1HG1 XXY"}="HG11 XXY"; $resn_exch{"2HG1 XXY"}="HG12 XXY"; $resn_exch{"3HG1 XXY"}="HG13 XXY";
$resn_exch{"1HA3 XXY"}="HA31 XXY"; $resn_exch{"2HA3 XXY"}="HA32 XXY"; $resn_exch{"1HB2 XXY"}="HB21 XXY";
$resn_exch{"2HB2 XXY"}="HB22 XXY"; $resn_exch{"1H19 XY2"}="H191 XY2"; $resn_exch{"2H19 XY2"}="H192 XY2";
$resn_exch{"3H19 XY2"}="H193 XY2"; $resn_exch{"1H16 XY2"}="H161 XY2"; $resn_exch{"2H16 XY2"}="H162 XY2";
$resn_exch{"3H16 XY2"}="H163 XY2"; $resn_exch{"1H14 XY2"}="H141 XY2"; $resn_exch{"2H14 XY2"}="H142 XY2";
$resn_exch{"1H13 XY2"}="H131 XY2"; $resn_exch{"2H13 XY2"}="H132 XY2"; $resn_exch{"1H12 XY2"}="H121 XY2";
$resn_exch{"2H12 XY2"}="H122 XY2"; $resn_exch{"3H12 XY2"}="H123 XY2"; $resn_exch{"1H5' XYA"}="H5'1 XYA";
$resn_exch{"2H5' XYA"}="H5'2 XYA"; $resn_exch{"1HN6 XYA"}="HN61 XYA"; $resn_exch{"2HN6 XYA"}="HN62 XYA";
$resn_exch{"1H8  XYD"}=" H81 XYD"; $resn_exch{"2H8  XYD"}=" H82 XYD"; $resn_exch{"3H8  XYD"}=" H83 XYD";
$resn_exch{"1H9  XYD"}=" H91 XYD"; $resn_exch{"2H9  XYD"}=" H92 XYD"; $resn_exch{"3H9  XYD"}=" H93 XYD";
$resn_exch{"1HA3 XYG"}="HA31 XYG"; $resn_exch{"2HA3 XYG"}="HA32 XYG"; $resn_exch{"2HN  XYG"}=" HN2 XYG";
$resn_exch{"1HB1 XYG"}="HB11 XYG"; $resn_exch{"2HB1 XYG"}="HB12 XYG"; $resn_exch{"3HB1 XYG"}="HB13 XYG";
$resn_exch{"1H2  XYH"}=" H21 XYH"; $resn_exch{"1H3  XYH"}=" H31 XYH"; $resn_exch{"1H4  XYH"}=" H41 XYH";
$resn_exch{"1H1  XYL"}=" H11 XYL"; $resn_exch{"2H1  XYL"}=" H12 XYL"; $resn_exch{"1H5  XYL"}=" H51 XYL";
$resn_exch{"2H5  XYL"}=" H52 XYL"; $resn_exch{"1H5B XYP"}="H5B1 XYP"; $resn_exch{"2H5B XYP"}="H5B2 XYP";
$resn_exch{"1HH3 XYQ"}="HH31 XYQ"; $resn_exch{"2HH3 XYQ"}="HH32 XYQ"; $resn_exch{"3HH3 XYQ"}="HH33 XYQ";
$resn_exch{"1H5N XYQ"}="H5N1 XYQ"; $resn_exch{"2H5N XYQ"}="H5N2 XYQ"; $resn_exch{"1H5A XYQ"}="H5A1 XYQ";
$resn_exch{"2H5A XYQ"}="H5A2 XYQ"; $resn_exch{"1HN6 XYQ"}="HN61 XYQ"; $resn_exch{"2HN6 XYQ"}="HN62 XYQ";
$resn_exch{"1H5  XYS"}=" H51 XYS"; $resn_exch{"2H5  XYS"}=" H52 XYS"; $resn_exch{"1H5  XYZ"}=" H51 XYZ";
$resn_exch{"2H5  XYZ"}=" H52 XYZ"; $resn_exch{"2HOP   Y"}="HOP2   Y"; $resn_exch{"3HOP   Y"}="HOP3   Y";
$resn_exch{"1H5*   Y"}=" H5'   Y"; $resn_exch{"2H5*   Y"}="H5''   Y"; $resn_exch{" H4*   Y"}=" H4'   Y";
$resn_exch{" H3*   Y"}=" H3'   Y"; $resn_exch{"*HO3   Y"}="HO3'   Y"; $resn_exch{"1H2*   Y"}=" H2'   Y";
$resn_exch{"2H2*   Y"}="H2''   Y"; $resn_exch{" H1*   Y"}=" H1'   Y"; $resn_exch{"1HB    Y"}=" HB1   Y";
$resn_exch{"2HB    Y"}=" HB2   Y"; $resn_exch{"1H02 Y11"}="H021 Y11"; $resn_exch{"2H02 Y11"}="H022 Y11";
$resn_exch{"1H03 Y11"}="H031 Y11"; $resn_exch{"2H03 Y11"}="H032 Y11"; $resn_exch{"1H04 Y11"}="H041 Y11";
$resn_exch{"2H04 Y11"}="H042 Y11"; $resn_exch{"1H05 Y11"}="H051 Y11"; $resn_exch{"2H05 Y11"}="H052 Y11";
$resn_exch{"1H11 Y11"}="H111 Y11"; $resn_exch{"2H11 Y11"}="H112 Y11"; $resn_exch{"1H13 Y11"}="H131 Y11";
$resn_exch{"2H13 Y11"}="H132 Y11"; $resn_exch{"1H14 Y11"}="H141 Y11"; $resn_exch{"2H14 Y11"}="H142 Y11";
$resn_exch{"1H15 Y11"}="H151 Y11"; $resn_exch{"2H15 Y11"}="H152 Y11"; $resn_exch{"1H16 Y11"}="H161 Y11";
$resn_exch{"2H16 Y11"}="H162 Y11"; $resn_exch{"1H17 Y11"}="H171 Y11"; $resn_exch{"2H17 Y11"}="H172 Y11";
$resn_exch{"1H21 Y11"}="H211 Y11"; $resn_exch{"2H21 Y11"}="H212 Y11"; $resn_exch{"1H22 Y11"}="H221 Y11";
$resn_exch{"2H22 Y11"}="H222 Y11"; $resn_exch{"1H23 Y11"}="H231 Y11"; $resn_exch{"2H23 Y11"}="H232 Y11";
$resn_exch{"1H24 Y11"}="H241 Y11"; $resn_exch{"2H24 Y11"}="H242 Y11"; $resn_exch{"1H26 Y11"}="H261 Y11";
$resn_exch{"2H26 Y11"}="H262 Y11"; $resn_exch{"3H26 Y11"}="H263 Y11"; $resn_exch{"1H7  Y12"}=" H71 Y12";
$resn_exch{"2H7  Y12"}=" H72 Y12"; $resn_exch{"1H8  Y12"}=" H81 Y12"; $resn_exch{"2H8  Y12"}=" H82 Y12";
$resn_exch{"1H25 Y12"}="H251 Y12"; $resn_exch{"2H25 Y12"}="H252 Y12"; $resn_exch{"3H25 Y12"}="H253 Y12";
$resn_exch{"1H21 Y12"}="H211 Y12"; $resn_exch{"2H21 Y12"}="H212 Y12"; $resn_exch{"3H21 Y12"}="H213 Y12";
$resn_exch{"1H7  Y13"}=" H71 Y13"; $resn_exch{"2H7  Y13"}=" H72 Y13"; $resn_exch{"1H8  Y13"}=" H81 Y13";
$resn_exch{"2H8  Y13"}=" H82 Y13"; $resn_exch{"1H02 Y14"}="H021 Y14"; $resn_exch{"2H02 Y14"}="H022 Y14";
$resn_exch{"1H03 Y14"}="H031 Y14"; $resn_exch{"2H03 Y14"}="H032 Y14"; $resn_exch{"1H04 Y14"}="H041 Y14";
$resn_exch{"2H04 Y14"}="H042 Y14"; $resn_exch{"1H05 Y14"}="H051 Y14"; $resn_exch{"2H05 Y14"}="H052 Y14";
$resn_exch{"1H11 Y14"}="H111 Y14"; $resn_exch{"2H11 Y14"}="H112 Y14"; $resn_exch{"1H13 Y14"}="H131 Y14";
$resn_exch{"2H13 Y14"}="H132 Y14"; $resn_exch{"1H14 Y14"}="H141 Y14"; $resn_exch{"2H14 Y14"}="H142 Y14";
$resn_exch{"1H15 Y14"}="H151 Y14"; $resn_exch{"2H15 Y14"}="H152 Y14"; $resn_exch{"1H16 Y14"}="H161 Y14";
$resn_exch{"2H16 Y14"}="H162 Y14"; $resn_exch{"1H17 Y14"}="H171 Y14"; $resn_exch{"2H17 Y14"}="H172 Y14";
$resn_exch{"1H20 Y14"}="H201 Y14"; $resn_exch{"2H20 Y14"}="H202 Y14"; $resn_exch{"1H21 Y14"}="H211 Y14";
$resn_exch{"2H21 Y14"}="H212 Y14"; $resn_exch{"1H22 Y14"}="H221 Y14"; $resn_exch{"2H22 Y14"}="H222 Y14";
$resn_exch{"1H25 Y14"}="H251 Y14"; $resn_exch{"2H25 Y14"}="H252 Y14"; $resn_exch{"1H02 Y15"}="H021 Y15";
$resn_exch{"2H02 Y15"}="H022 Y15"; $resn_exch{"1H03 Y15"}="H031 Y15"; $resn_exch{"2H03 Y15"}="H032 Y15";
$resn_exch{"1H04 Y15"}="H041 Y15"; $resn_exch{"2H04 Y15"}="H042 Y15"; $resn_exch{"1H05 Y15"}="H051 Y15";
$resn_exch{"2H05 Y15"}="H052 Y15"; $resn_exch{"1H11 Y15"}="H111 Y15"; $resn_exch{"2H11 Y15"}="H112 Y15";
$resn_exch{"1H13 Y15"}="H131 Y15"; $resn_exch{"2H13 Y15"}="H132 Y15"; $resn_exch{"1H14 Y15"}="H141 Y15";
$resn_exch{"2H14 Y15"}="H142 Y15"; $resn_exch{"1H15 Y15"}="H151 Y15"; $resn_exch{"2H15 Y15"}="H152 Y15";
$resn_exch{"1H16 Y15"}="H161 Y15"; $resn_exch{"2H16 Y15"}="H162 Y15"; $resn_exch{"1H17 Y15"}="H171 Y15";
$resn_exch{"2H17 Y15"}="H172 Y15"; $resn_exch{"1H20 Y15"}="H201 Y15"; $resn_exch{"2H20 Y15"}="H202 Y15";
$resn_exch{"1H21 Y15"}="H211 Y15"; $resn_exch{"2H21 Y15"}="H212 Y15"; $resn_exch{"3H21 Y15"}="H213 Y15";
$resn_exch{"1H24 Y15"}="H241 Y15"; $resn_exch{"2H24 Y15"}="H242 Y15"; $resn_exch{"1H15 Y19"}="H151 Y19";
$resn_exch{"2H15 Y19"}="H152 Y19"; $resn_exch{"1H16 Y19"}="H161 Y19"; $resn_exch{"2H16 Y19"}="H162 Y19";
$resn_exch{"3H16 Y19"}="H163 Y19"; $resn_exch{"1H8C Y19"}="H8C1 Y19"; $resn_exch{"2H8C Y19"}="H8C2 Y19";
$resn_exch{"1H12 Y19"}="H121 Y19"; $resn_exch{"2H12 Y19"}="H122 Y19"; $resn_exch{"1H36 Y27"}="H361 Y27";
$resn_exch{"2H36 Y27"}="H362 Y27"; $resn_exch{"1H35 Y27"}="H351 Y27"; $resn_exch{"2H35 Y27"}="H352 Y27";
$resn_exch{"1H33 Y27"}="H331 Y27"; $resn_exch{"2H33 Y27"}="H332 Y27"; $resn_exch{"1H32 Y27"}="H321 Y27";
$resn_exch{"2H32 Y27"}="H322 Y27"; $resn_exch{"1H42 Y27"}="H421 Y27"; $resn_exch{"2H42 Y27"}="H422 Y27";
$resn_exch{"3H42 Y27"}="H423 Y27"; $resn_exch{"1H43 Y27"}="H431 Y27"; $resn_exch{"2H43 Y27"}="H432 Y27";
$resn_exch{"1H18  Y3"}="H181  Y3"; $resn_exch{"2H18  Y3"}="H182  Y3"; $resn_exch{"3H18  Y3"}="H183  Y3";
$resn_exch{"1H12 YBH"}="H121 YBH"; $resn_exch{"2H12 YBH"}="H122 YBH"; $resn_exch{"1H11 YBH"}="H111 YBH";
$resn_exch{"2H11 YBH"}="H112 YBH"; $resn_exch{"1H25 YBH"}="H251 YBH"; $resn_exch{"2H25 YBH"}="H252 YBH";
$resn_exch{"1H13 YBH"}="H131 YBH"; $resn_exch{"2H13 YBH"}="H132 YBH"; $resn_exch{"1H14 YBH"}="H141 YBH";
$resn_exch{"2H14 YBH"}="H142 YBH"; $resn_exch{"1H2  YBT"}=" H21 YBT"; $resn_exch{"2H2  YBT"}=" H22 YBT";
$resn_exch{"1H3  YBT"}=" H31 YBT"; $resn_exch{"2H3  YBT"}=" H32 YBT"; $resn_exch{"1H4  YBT"}=" H41 YBT";
$resn_exch{"2H4  YBT"}=" H42 YBT"; $resn_exch{"1H5  YBT"}=" H51 YBT"; $resn_exch{"2H5  YBT"}=" H52 YBT";
$resn_exch{"1H6  YBT"}=" H61 YBT"; $resn_exch{"2H6  YBT"}=" H62 YBT"; $resn_exch{"1H7  YBT"}=" H71 YBT";
$resn_exch{"2H7  YBT"}=" H72 YBT"; $resn_exch{"1H8  YBT"}=" H81 YBT"; $resn_exch{"2H8  YBT"}=" H82 YBT";
$resn_exch{" HN2 YCM"}=" H2  YCM"; $resn_exch{"1HB  YCM"}=" HB2 YCM"; $resn_exch{"2HB  YCM"}=" HB3 YCM";
$resn_exch{"1HD  YCM"}=" HD2 YCM"; $resn_exch{"2HD  YCM"}=" HD3 YCM"; $resn_exch{"1HZ  YCM"}="HZ21 YCM";
$resn_exch{"2HZ  YCM"}="HZ22 YCM"; $resn_exch{"1HN6 YE1"}="HN61 YE1"; $resn_exch{"2HN6 YE1"}="HN62 YE1";
$resn_exch{" H1* YE1"}=" H1' YE1"; $resn_exch{" H2* YE1"}=" H2' YE1"; $resn_exch{"HO2* YE1"}="HO2' YE1";
$resn_exch{" H3* YE1"}=" H3' YE1"; $resn_exch{" H4* YE1"}=" H4' YE1"; $resn_exch{"1H5* YE1"}="H5'1 YE1";
$resn_exch{"2H5* YE1"}="H5'2 YE1"; $resn_exch{"1H12 YE1"}="H121 YE1"; $resn_exch{"2H12 YE1"}="H122 YE1";
$resn_exch{"1H14 YE1"}="H141 YE1"; $resn_exch{"2H14 YE1"}="H142 YE1"; $resn_exch{"3H14 YE1"}="H143 YE1";
$resn_exch{"1H13 YE1"}="H131 YE1"; $resn_exch{"2H13 YE1"}="H132 YE1"; $resn_exch{"3H13 YE1"}="H133 YE1";
$resn_exch{"1HC7 YE1"}="HC71 YE1"; $resn_exch{"2HC7 YE1"}="HC72 YE1"; $resn_exch{"1HC6 YE1"}="HC61 YE1";
$resn_exch{"2HC6 YE1"}="HC62 YE1"; $resn_exch{"1HC3 YE1"}="HC31 YE1"; $resn_exch{"2HC3 YE1"}="HC32 YE1";
$resn_exch{"1HC2 YE1"}="HC21 YE1"; $resn_exch{"2HC2 YE1"}="HC22 YE1"; $resn_exch{"1HAC YE1"}="HAC1 YE1";
$resn_exch{"2HAC YE1"}="HAC2 YE1"; $resn_exch{"1HAP YE7"}="HAP1 YE7"; $resn_exch{"2HAP YE7"}="HAP2 YE7";
$resn_exch{"1H1  YF1"}=" H11 YF1"; $resn_exch{"2H1  YF1"}=" H12 YF1"; $resn_exch{"3H1  YF1"}=" H13 YF1";
$resn_exch{"1H5  YF1"}=" H51 YF1"; $resn_exch{"2H5  YF1"}=" H52 YF1"; $resn_exch{"3H5  YF1"}=" H53 YF1";
$resn_exch{"1HN2 YF1"}="HN21 YF1"; $resn_exch{"2HN2 YF1"}="HN22 YF1"; $resn_exch{"1HM2 YF3"}="HM21 YF3";
$resn_exch{"2HM2 YF3"}="HM22 YF3"; $resn_exch{"3HM2 YF3"}="HM23 YF3"; $resn_exch{"1H4' YF3"}="H4'1 YF3";
$resn_exch{"2H4' YF3"}="H4'2 YF3"; $resn_exch{"1H7' YF3"}="H7'1 YF3"; $resn_exch{"2H7' YF3"}="H7'2 YF3";
$resn_exch{"1HM4 YF3"}="HM41 YF3"; $resn_exch{"2HM4 YF3"}="HM42 YF3"; $resn_exch{"3HM4 YF3"}="HM43 YF3";
$resn_exch{"1H2  YF3"}=" H21 YF3"; $resn_exch{"2H2  YF3"}=" H22 YF3"; $resn_exch{"1HM2 YF4"}="HM21 YF4";
$resn_exch{"2HM2 YF4"}="HM22 YF4"; $resn_exch{"3HM2 YF4"}="HM23 YF4"; $resn_exch{"1H4' YF4"}="H4'1 YF4";
$resn_exch{"2H4' YF4"}="H4'2 YF4"; $resn_exch{"1H7' YF4"}="H7'1 YF4"; $resn_exch{"2H7' YF4"}="H7'2 YF4";
$resn_exch{"1HM4 YF4"}="HM41 YF4"; $resn_exch{"2HM4 YF4"}="HM42 YF4"; $resn_exch{"3HM4 YF4"}="HM43 YF4";
$resn_exch{"1H4  YF4"}=" H41 YF4"; $resn_exch{"2H4  YF4"}=" H42 YF4"; $resn_exch{"1H2  YF4"}=" H21 YF4";
$resn_exch{"2H2  YF4"}=" H22 YF4"; $resn_exch{"3H2  YF4"}=" H23 YF4"; $resn_exch{"1H3   YG"}=" H31  YG";
$resn_exch{"2H3   YG"}=" H32  YG"; $resn_exch{"3H3   YG"}=" H33  YG"; $resn_exch{"1H10  YG"}="H101  YG";
$resn_exch{"2H10  YG"}="H102  YG"; $resn_exch{"3H10  YG"}="H103  YG"; $resn_exch{"1H13  YG"}="H131  YG";
$resn_exch{"2H13  YG"}="H132  YG"; $resn_exch{"1H14  YG"}="H141  YG"; $resn_exch{"2H14  YG"}="H142  YG";
$resn_exch{"1H19  YG"}="H191  YG"; $resn_exch{"2H19  YG"}="H192  YG"; $resn_exch{"3H19  YG"}="H193  YG";
$resn_exch{"1H24  YG"}="H241  YG"; $resn_exch{"2H24  YG"}="H242  YG"; $resn_exch{"3H24  YG"}="H243  YG";
$resn_exch{" H1*  YG"}=" H1'  YG"; $resn_exch{" H2*  YG"}=" H2'  YG"; $resn_exch{"*HO2  YG"}="HO2'  YG";
$resn_exch{" H3*  YG"}=" H3'  YG"; $resn_exch{"*HO3  YG"}="HO3'  YG"; $resn_exch{" H4*  YG"}=" H4'  YG";
$resn_exch{"1H5*  YG"}=" H5'  YG"; $resn_exch{"2H5*  YG"}="H5''  YG"; $resn_exch{"2HOP  YG"}="HOP2  YG";
$resn_exch{"3HOP  YG"}="HOP3  YG"; $resn_exch{"1H6C YLL"}="H6C1 YLL"; $resn_exch{"2H6C YLL"}="H6C2 YLL";
$resn_exch{"1HAA YLY"}="HAA1 YLY"; $resn_exch{"2HAA YLY"}="HAA2 YLY"; $resn_exch{"3HAA YLY"}="HAA3 YLY";
$resn_exch{"1HAP YLY"}="HAP1 YLY"; $resn_exch{"2HAP YLY"}="HAP2 YLY"; $resn_exch{"1HAJ YLY"}="HAJ1 YLY";
$resn_exch{"2HAJ YLY"}="HAJ2 YLY"; $resn_exch{"1HAO YLY"}="HAO1 YLY"; $resn_exch{"2HAO YLY"}="HAO2 YLY";
$resn_exch{"1HAM YLY"}="HAM1 YLY"; $resn_exch{"2HAM YLY"}="HAM2 YLY"; $resn_exch{"1HAN YLY"}="HAN1 YLY";
$resn_exch{"2HAN YLY"}="HAN2 YLY"; $resn_exch{"1HB  YLY"}=" HB1 YLY"; $resn_exch{"2HB  YLY"}=" HB2 YLY";
$resn_exch{"1HN  YLY"}=" HN1 YLY"; $resn_exch{"2HN  YLY"}=" HN2 YLY"; $resn_exch{"1H5* YLY"}="H5'1 YLY";
$resn_exch{"2H5* YLY"}="H5'2 YLY"; $resn_exch{" H4* YLY"}=" H4' YLY"; $resn_exch{" H3* YLY"}=" H3' YLY";
$resn_exch{"HO3* YLY"}="HO3' YLY"; $resn_exch{" H2* YLY"}=" H2' YLY"; $resn_exch{"HO2* YLY"}="HO2' YLY";
$resn_exch{" H1* YLY"}=" H1' YLY"; $resn_exch{"1HN6 YLY"}="HN61 YLY"; $resn_exch{"2HN6 YLY"}="HN62 YLY";
$resn_exch{"1H1  YMA"}=" H11 YMA"; $resn_exch{"2H1  YMA"}=" H12 YMA"; $resn_exch{"1H10 YMA"}="H101 YMA";
$resn_exch{"2H10 YMA"}="H102 YMA"; $resn_exch{"1H11 YMA"}="H111 YMA"; $resn_exch{"2H11 YMA"}="H112 YMA";
$resn_exch{"1H14 YMA"}="H141 YMA"; $resn_exch{"2H14 YMA"}="H142 YMA"; $resn_exch{"1H16 YMA"}="H161 YMA";
$resn_exch{"2H16 YMA"}="H162 YMA"; $resn_exch{"1H18 YMA"}="H181 YMA"; $resn_exch{"2H18 YMA"}="H182 YMA";
$resn_exch{"1H12 YMA"}="H121 YMA"; $resn_exch{"2H12 YMA"}="H122 YMA"; $resn_exch{"1H3  YMA"}=" H31 YMA";
$resn_exch{"2H3  YMA"}=" H32 YMA"; $resn_exch{"1H4  YMA"}=" H41 YMA"; $resn_exch{"2H4  YMA"}=" H42 YMA";
$resn_exch{"1H5  YMA"}=" H51 YMA"; $resn_exch{"2H5  YMA"}=" H52 YMA"; $resn_exch{"1H7  YMA"}=" H71 YMA";
$resn_exch{"2H7  YMA"}=" H72 YMA"; $resn_exch{"1H9  YMA"}=" H91 YMA"; $resn_exch{"2H9  YMA"}=" H92 YMA";
$resn_exch{"1H17 YMA"}="H171 YMA"; $resn_exch{"2H17 YMA"}="H172 YMA"; $resn_exch{"1H7  YML"}=" H71 YML";
$resn_exch{"2H7  YML"}=" H72 YML"; $resn_exch{"1H2  YML"}=" H21 YML"; $resn_exch{"2H2  YML"}=" H22 YML";
$resn_exch{"1H9  YML"}=" H91 YML"; $resn_exch{"2H9  YML"}=" H92 YML"; $resn_exch{"1HAC YML"}="HAC1 YML";
$resn_exch{"2HAC YML"}="HAC2 YML"; $resn_exch{"3HAC YML"}="HAC3 YML"; $resn_exch{"1HAB YML"}="HAB1 YML";
$resn_exch{"2HAB YML"}="HAB2 YML"; $resn_exch{"3HAB YML"}="HAB3 YML"; $resn_exch{"1HAA YML"}="HAA1 YML";
$resn_exch{"2HAA YML"}="HAA2 YML"; $resn_exch{"3HAA YML"}="HAA3 YML"; $resn_exch{"1HN  YMP"}=" HN1 YMP";
$resn_exch{"2HN  YMP"}=" HN2 YMP"; $resn_exch{"1HB  YMP"}=" HB1 YMP"; $resn_exch{"2HB  YMP"}=" HB2 YMP";
$resn_exch{"1H5* YMP"}="H5'1 YMP"; $resn_exch{"2H5* YMP"}="H5'2 YMP"; $resn_exch{" H4* YMP"}=" H4' YMP";
$resn_exch{" H1* YMP"}=" H1' YMP"; $resn_exch{" H2* YMP"}=" H2' YMP"; $resn_exch{" H3* YMP"}=" H3' YMP";
$resn_exch{"1HN6 YMP"}="HN61 YMP"; $resn_exch{"2HN6 YMP"}="HN62 YMP"; $resn_exch{"1HN  YOF"}=" H   YOF";
$resn_exch{"2HN  YOF"}=" H2  YOF"; $resn_exch{"1HB  YOF"}=" HB2 YOF"; $resn_exch{"2HB  YOF"}=" HB3 YOF";
$resn_exch{" HO  YOF"}=" HH  YOF"; $resn_exch{"1HBA YOK"}="HBA1 YOK"; $resn_exch{"2HBA YOK"}="HBA2 YOK";
$resn_exch{"1HAA YOK"}="HAA1 YOK"; $resn_exch{"2HAA YOK"}="HAA2 YOK"; $resn_exch{"1H22 YPA"}="H221 YPA";
$resn_exch{"2H22 YPA"}="H222 YPA"; $resn_exch{"1H23 YPA"}="H231 YPA"; $resn_exch{"2H23 YPA"}="H232 YPA";
$resn_exch{"1H39 YPA"}="H391 YPA"; $resn_exch{"2H39 YPA"}="H392 YPA"; $resn_exch{"1H49 YPA"}="H491 YPA";
$resn_exch{"2H49 YPA"}="H492 YPA"; $resn_exch{"1H50 YPA"}="H501 YPA"; $resn_exch{"2H50 YPA"}="H502 YPA";
$resn_exch{"3H50 YPA"}="H503 YPA"; $resn_exch{"3HOP YRR"}="HOP3 YRR"; $resn_exch{"1HC5 YRR"}="HC51 YRR";
$resn_exch{"2HC5 YRR"}="HC52 YRR"; $resn_exch{"H3T  YRR"}="HO3' YRR"; $resn_exch{"1HC2 YRR"}="HC21 YRR";
$resn_exch{"2HC2 YRR"}="HC22 YRR"; $resn_exch{"1HC1 YRR"}="HC11 YRR"; $resn_exch{"2HC1 YRR"}="HC12 YRR";
$resn_exch{"1HB  YSA"}=" HB1 YSA"; $resn_exch{"2HB  YSA"}=" HB2 YSA"; $resn_exch{"1HN  YSA"}=" HN1 YSA";
$resn_exch{"2HN  YSA"}=" HN2 YSA"; $resn_exch{"1H5* YSA"}="H5'1 YSA"; $resn_exch{"2H5* YSA"}="H5'2 YSA";
$resn_exch{" H4* YSA"}=" H4' YSA"; $resn_exch{" H3* YSA"}=" H3' YSA"; $resn_exch{" H2* YSA"}=" H2' YSA";
$resn_exch{" H1* YSA"}=" H1' YSA"; $resn_exch{"1HN6 YSA"}="HN61 YSA"; $resn_exch{"2HN6 YSA"}="HN62 YSA";
$resn_exch{"1H15 YSH"}="H151 YSH"; $resn_exch{"2H15 YSH"}="H152 YSH"; $resn_exch{"1H17 YSH"}="H171 YSH";
$resn_exch{"2H17 YSH"}="H172 YSH"; $resn_exch{"3H17 YSH"}="H173 YSH"; $resn_exch{"1H18 YSH"}="H181 YSH";
$resn_exch{"2H18 YSH"}="H182 YSH"; $resn_exch{"3H18 YSH"}="H183 YSH"; $resn_exch{"1H19 YSH"}="H191 YSH";
$resn_exch{"2H19 YSH"}="H192 YSH"; $resn_exch{"3H19 YSH"}="H193 YSH"; $resn_exch{"2HOP YYG"}="HOP2 YYG";
$resn_exch{"3HOP YYG"}="HOP3 YYG"; $resn_exch{"1H5* YYG"}=" H5' YYG"; $resn_exch{"2H5* YYG"}="H5'' YYG";
$resn_exch{" H4* YYG"}=" H4' YYG"; $resn_exch{" H3* YYG"}=" H3' YYG"; $resn_exch{"H3T  YYG"}="HO3' YYG";
$resn_exch{" H2* YYG"}=" H2' YYG"; $resn_exch{"2HO* YYG"}="HO2' YYG"; $resn_exch{" H1* YYG"}=" H1' YYG";
$resn_exch{"1H3  YYG"}=" H31 YYG"; $resn_exch{"2H3  YYG"}=" H32 YYG"; $resn_exch{"3H3  YYG"}=" H33 YYG";
$resn_exch{"1H10 YYG"}="H101 YYG"; $resn_exch{"2H10 YYG"}="H102 YYG"; $resn_exch{"3H10 YYG"}="H103 YYG";
$resn_exch{"1H13 YYG"}="H131 YYG"; $resn_exch{"2H13 YYG"}="H132 YYG"; $resn_exch{"1H14 YYG"}="H141 YYG";
$resn_exch{"2H14 YYG"}="H142 YYG"; $resn_exch{"1H19 YYG"}="H191 YYG"; $resn_exch{"2H19 YYG"}="H192 YYG";
$resn_exch{"3H19 YYG"}="H193 YYG"; $resn_exch{"1H24 YYG"}="H241 YYG"; $resn_exch{"2H24 YYG"}="H242 YYG";
$resn_exch{"3H24 YYG"}="H243 YYG"; $resn_exch{"1H5* YYY"}="H5'1 YYY"; $resn_exch{"2H5* YYY"}="H5'2 YYY";
$resn_exch{" H4* YYY"}=" H4' YYY"; $resn_exch{" H3* YYY"}=" H3' YYY"; $resn_exch{" H1* YYY"}=" H1' YYY";
$resn_exch{"1H2* YYY"}="H2'1 YYY"; $resn_exch{"2H2* YYY"}="H2'2 YYY"; $resn_exch{"1H4N YYY"}="H4N1 YYY";
$resn_exch{"2H4N YYY"}="H4N2 YYY"; $resn_exch{"1H3  YZ9"}=" H31 YZ9"; $resn_exch{"2H3  YZ9"}=" H32 YZ9";
$resn_exch{"3H3  YZ9"}=" H33 YZ9"; $resn_exch{"1H11 YZ9"}="H111 YZ9"; $resn_exch{"2H11 YZ9"}="H112 YZ9";
$resn_exch{"2HOP   Z"}="HOP2   Z"; $resn_exch{"1H5*   Z"}=" H5'   Z"; $resn_exch{"2H5*   Z"}="H5''   Z";
$resn_exch{" H4*   Z"}=" H4'   Z"; $resn_exch{" H3*   Z"}=" H3'   Z"; $resn_exch{"H3T    Z"}="HO3'   Z";
$resn_exch{"1H2*   Z"}=" H2'   Z"; $resn_exch{"2H2*   Z"}="H2''   Z"; $resn_exch{" H1*   Z"}=" H1'   Z";
$resn_exch{"3HOP   Z"}="HOP3   Z"; $resn_exch{"1H11 Z34"}="H111 Z34"; $resn_exch{"1H12 Z34"}="H121 Z34";
$resn_exch{"2H12 Z34"}="H122 Z34"; $resn_exch{"1H30 Z34"}="H301 Z34"; $resn_exch{"2H30 Z34"}="H302 Z34";
$resn_exch{"3H30 Z34"}="H303 Z34"; $resn_exch{"1H28 Z34"}="H281 Z34"; $resn_exch{"2H28 Z34"}="H282 Z34";
$resn_exch{"1H29 Z34"}="H291 Z34"; $resn_exch{"2H29 Z34"}="H292 Z34"; $resn_exch{"1H32 Z34"}="H321 Z34";
$resn_exch{"2H32 Z34"}="H322 Z34"; $resn_exch{"1H36 Z34"}="H361 Z34"; $resn_exch{"2H36 Z34"}="H362 Z34";
$resn_exch{"3H36 Z34"}="H363 Z34"; $resn_exch{"1H7T Z5A"}="H7T1 Z5A"; $resn_exch{"2H7T Z5A"}="H7T2 Z5A";
$resn_exch{"3H7T Z5A"}="H7T3 Z5A"; $resn_exch{"1H5' Z5A"}="H5'1 Z5A"; $resn_exch{"2H5' Z5A"}="H5'2 Z5A";
$resn_exch{"1H2' Z5A"}="H2'1 Z5A"; $resn_exch{"2H2' Z5A"}="H2'2 Z5A"; $resn_exch{"1H5* Z5A"}="H5A1 Z5A";
$resn_exch{"2H5* Z5A"}="H5A2 Z5A"; $resn_exch{" H4* Z5A"}=" H4B Z5A"; $resn_exch{" H3* Z5A"}=" H3B Z5A";
$resn_exch{"3HO* Z5A"}="HO'3 Z5A"; $resn_exch{" H2* Z5A"}=" H2' Z5A"; $resn_exch{"2HO* Z5A"}="HO'2 Z5A";
$resn_exch{" H1* Z5A"}=" H1B Z5A"; $resn_exch{"1H6A Z5A"}="H6A1 Z5A"; $resn_exch{"2H6A Z5A"}="H6A2 Z5A";
$resn_exch{"1HN  ZAB"}=" HN1 ZAB"; $resn_exch{"2HN  ZAB"}=" HN2 ZAB"; $resn_exch{"2HA  ZAB"}=" HA2 ZAB";
$resn_exch{"3HA  ZAB"}=" HA3 ZAB"; $resn_exch{"2HM  ZAB"}=" HM2 ZAB"; $resn_exch{"3HM  ZAB"}=" HM3 ZAB";
$resn_exch{"1HOB ZAF"}="HOB1 ZAF"; $resn_exch{"1H2A ZAF"}="H2A1 ZAF"; $resn_exch{"2H2A ZAF"}="H2A2 ZAF";
$resn_exch{"3HOB ZAF"}="HOB3 ZAF"; $resn_exch{"1H4A ZAF"}="H4A1 ZAF"; $resn_exch{"2H4A ZAF"}="H4A2 ZAF";
$resn_exch{"3H4A ZAF"}="H4A3 ZAF"; $resn_exch{"1H7A ZAF"}="H7A1 ZAF"; $resn_exch{"2H7A ZAF"}="H7A2 ZAF";
$resn_exch{"3H7A ZAF"}="H7A3 ZAF"; $resn_exch{"1H11 ZAF"}="H111 ZAF"; $resn_exch{"2H11 ZAF"}="H112 ZAF";
$resn_exch{"*HP2 ZAF"}="HP2' ZAF"; $resn_exch{"*HP3 ZAF"}="HP3' ZAF"; $resn_exch{"*HP4 ZAF"}="HP4' ZAF";
$resn_exch{"*HP5 ZAF"}="HP5' ZAF"; $resn_exch{"*HP6 ZAF"}="HP6' ZAF"; $resn_exch{"1H4C ZAH"}="H4C1 ZAH";
$resn_exch{"2H4C ZAH"}="H4C2 ZAH"; $resn_exch{"1H5C ZAH"}="H5C1 ZAH"; $resn_exch{"2H5C ZAH"}="H5C2 ZAH";
$resn_exch{"1H6C ZAH"}="H6C1 ZAH"; $resn_exch{"2H6C ZAH"}="H6C2 ZAH"; $resn_exch{"1H11 ZAH"}="H111 ZAH";
$resn_exch{"2H11 ZAH"}="H112 ZAH"; $resn_exch{"3H11 ZAH"}="H113 ZAH"; $resn_exch{"1H18 ZAH"}="H181 ZAH";
$resn_exch{"2H18 ZAH"}="H182 ZAH"; $resn_exch{"1H13 ZAM"}="H131 ZAM"; $resn_exch{"2H13 ZAM"}="H132 ZAM";
$resn_exch{"1H30 ZAM"}="H301 ZAM"; $resn_exch{"2H30 ZAM"}="H302 ZAM"; $resn_exch{"1H33 ZAM"}="H331 ZAM";
$resn_exch{"2H33 ZAM"}="H332 ZAM"; $resn_exch{"3H33 ZAM"}="H333 ZAM"; $resn_exch{"1H10 ZAM"}="H101 ZAM";
$resn_exch{"2H10 ZAM"}="H102 ZAM"; $resn_exch{"1H17 ZAM"}="H171 ZAM"; $resn_exch{"2H17 ZAM"}="H172 ZAM";
$resn_exch{"3H17 ZAM"}="H173 ZAM"; $resn_exch{"1H23 ZAM"}="H231 ZAM"; $resn_exch{"2H23 ZAM"}="H232 ZAM";
$resn_exch{"3H23 ZAM"}="H233 ZAM"; $resn_exch{"1H43 ZAM"}="H431 ZAM"; $resn_exch{"2H43 ZAM"}="H432 ZAM";
$resn_exch{"3H43 ZAM"}="H433 ZAM"; $resn_exch{"1H10 ZAP"}="H101 ZAP"; $resn_exch{"2H10 ZAP"}="H102 ZAP";
$resn_exch{"1HN2 ZAP"}="HN21 ZAP"; $resn_exch{"2HN2 ZAP"}="HN22 ZAP"; $resn_exch{"2HOP ZAP"}="HOP2 ZAP";
$resn_exch{"3HOP ZAP"}="HOP3 ZAP"; $resn_exch{"1H19 ZAR"}="H191 ZAR"; $resn_exch{"2H19 ZAR"}="H192 ZAR";
$resn_exch{"3H19 ZAR"}="H193 ZAR"; $resn_exch{"1HAK ZD6"}="HAK1 ZD6"; $resn_exch{"2HAK ZD6"}="HAK2 ZD6";
$resn_exch{"1HAM ZD6"}="HAM1 ZD6"; $resn_exch{"2HAM ZD6"}="HAM2 ZD6"; $resn_exch{"1HAB ZD6"}="HAB1 ZD6";
$resn_exch{"2HAB ZD6"}="HAB2 ZD6"; $resn_exch{"3HAB ZD6"}="HAB3 ZD6"; $resn_exch{"1HAN ZD6"}="HAN1 ZD6";
$resn_exch{"2HAN ZD6"}="HAN2 ZD6"; $resn_exch{"1HAL ZD6"}="HAL1 ZD6"; $resn_exch{"2HAL ZD6"}="HAL2 ZD6";
$resn_exch{"1HAO ZD6"}="HAO1 ZD6"; $resn_exch{"2HAO ZD6"}="HAO2 ZD6"; $resn_exch{"1HAA ZD6"}="HAA1 ZD6";
$resn_exch{"2HAA ZD6"}="HAA2 ZD6"; $resn_exch{"3HAA ZD6"}="HAA3 ZD6"; $resn_exch{"1HAB ZDP"}="HAB1 ZDP";
$resn_exch{"2HAB ZDP"}="HAB2 ZDP"; $resn_exch{"1HAK ZDP"}="HAK1 ZDP"; $resn_exch{"2HAK ZDP"}="HAK2 ZDP";
$resn_exch{"1HAJ ZDP"}="HAJ1 ZDP"; $resn_exch{"2HAJ ZDP"}="HAJ2 ZDP"; $resn_exch{"1HAM ZDP"}="HAM1 ZDP";
$resn_exch{"2HAM ZDP"}="HAM2 ZDP"; $resn_exch{"1HAC ZDP"}="HAC1 ZDP"; $resn_exch{"2HAC ZDP"}="HAC2 ZDP";
$resn_exch{"1HAN ZDP"}="HAN1 ZDP"; $resn_exch{"2HAN ZDP"}="HAN2 ZDP"; $resn_exch{"3HAN ZDP"}="HAN3 ZDP";
$resn_exch{"1HAN ZDR"}="HAN1 ZDR"; $resn_exch{"2HAN ZDR"}="HAN2 ZDR"; $resn_exch{"3HAN ZDR"}="HAN3 ZDR";
$resn_exch{"1HAC ZDR"}="HAC1 ZDR"; $resn_exch{"2HAC ZDR"}="HAC2 ZDR"; $resn_exch{"1HAM ZDR"}="HAM1 ZDR";
$resn_exch{"2HAM ZDR"}="HAM2 ZDR"; $resn_exch{"1HAJ ZDR"}="HAJ1 ZDR"; $resn_exch{"2HAJ ZDR"}="HAJ2 ZDR";
$resn_exch{"1HAK ZDR"}="HAK1 ZDR"; $resn_exch{"2HAK ZDR"}="HAK2 ZDR"; $resn_exch{"1HAB ZDR"}="HAB1 ZDR";
$resn_exch{"2HAB ZDR"}="HAB2 ZDR"; $resn_exch{"2HOP ZDU"}="HOP2 ZDU"; $resn_exch{"1H5A ZDU"}="H5A1 ZDU";
$resn_exch{"2H5A ZDU"}="H5A2 ZDU"; $resn_exch{"1H5B ZDU"}="H5B1 ZDU"; $resn_exch{"2H5B ZDU"}="H5B2 ZDU";
$resn_exch{"1H5G ZDU"}="H5G1 ZDU"; $resn_exch{"2H5G ZDU"}="H5G2 ZDU"; $resn_exch{"1H5D ZDU"}="H5D1 ZDU";
$resn_exch{"2H5D ZDU"}="H5D2 ZDU"; $resn_exch{"1H2* ZDU"}=" H2' ZDU"; $resn_exch{"2H2* ZDU"}="H2'' ZDU";
$resn_exch{"1H5* ZDU"}=" H5' ZDU"; $resn_exch{"2H5* ZDU"}="H5'' ZDU"; $resn_exch{" H4* ZDU"}=" H4' ZDU";
$resn_exch{" H1* ZDU"}=" H1' ZDU"; $resn_exch{" H3* ZDU"}=" H3' ZDU"; $resn_exch{"H3T  ZDU"}="HO3' ZDU";
$resn_exch{"3HOP ZDU"}="HOP3 ZDU"; $resn_exch{"1H14 ZEA"}="H141 ZEA"; $resn_exch{"2H14 ZEA"}="H142 ZEA";
$resn_exch{"1H15 ZEA"}="H151 ZEA"; $resn_exch{"2H15 ZEA"}="H152 ZEA"; $resn_exch{"3H15 ZEA"}="H153 ZEA";
$resn_exch{"1H11 ZEA"}="H111 ZEA"; $resn_exch{"2H11 ZEA"}="H112 ZEA"; $resn_exch{"1H5' ZEB"}="H5'1 ZEB";
$resn_exch{"2H5' ZEB"}="H5'2 ZEB"; $resn_exch{"1H14 ZEC"}="H141 ZEC"; $resn_exch{"2H14 ZEC"}="H142 ZEC";
$resn_exch{"1HZ9 ZED"}="HZ91 ZED"; $resn_exch{"2HZ9 ZED"}="HZ92 ZED"; $resn_exch{"1H11 ZED"}="H111 ZED";
$resn_exch{"2H11 ZED"}="H112 ZED"; $resn_exch{"1H12 ZED"}="H121 ZED"; $resn_exch{"2H12 ZED"}="H122 ZED";
$resn_exch{"3H12 ZED"}="H123 ZED"; $resn_exch{"1H15 ZED"}="H151 ZED"; $resn_exch{"2H15 ZED"}="H152 ZED";
$resn_exch{"1HAA ZEM"}="HAA1 ZEM"; $resn_exch{"2HAA ZEM"}="HAA2 ZEM"; $resn_exch{"1HMA ZEM"}="HMA1 ZEM";
$resn_exch{"2HMA ZEM"}="HMA2 ZEM"; $resn_exch{"3HMA ZEM"}="HMA3 ZEM"; $resn_exch{"1HBA ZEM"}="HBA1 ZEM";
$resn_exch{"2HBA ZEM"}="HBA2 ZEM"; $resn_exch{"1HMB ZEM"}="HMB1 ZEM"; $resn_exch{"2HMB ZEM"}="HMB2 ZEM";
$resn_exch{"3HMB ZEM"}="HMB3 ZEM"; $resn_exch{"1HBB ZEM"}="HBB1 ZEM"; $resn_exch{"2HBB ZEM"}="HBB2 ZEM";
$resn_exch{"1HMC ZEM"}="HMC1 ZEM"; $resn_exch{"2HMC ZEM"}="HMC2 ZEM"; $resn_exch{"3HMC ZEM"}="HMC3 ZEM";
$resn_exch{"1HBC ZEM"}="HBC1 ZEM"; $resn_exch{"2HBC ZEM"}="HBC2 ZEM"; $resn_exch{"1HMD ZEM"}="HMD1 ZEM";
$resn_exch{"2HMD ZEM"}="HMD2 ZEM"; $resn_exch{"3HMD ZEM"}="HMD3 ZEM"; $resn_exch{"1HAD ZEM"}="HAD1 ZEM";
$resn_exch{"2HAD ZEM"}="HAD2 ZEM"; $resn_exch{"1HBD ZEM"}="HBD1 ZEM"; $resn_exch{"2HBD ZEM"}="HBD2 ZEM";
$resn_exch{"2H8  ZEN"}=" H82 ZEN"; $resn_exch{"1H8  ZEN"}=" H81 ZEN"; $resn_exch{"2H9  ZEN"}=" H92 ZEN";
$resn_exch{"1H9  ZEN"}=" H91 ZEN"; $resn_exch{"2H11 ZEN"}="H112 ZEN"; $resn_exch{"1H11 ZEN"}="H111 ZEN";
$resn_exch{"2H12 ZEN"}="H122 ZEN"; $resn_exch{"1H12 ZEN"}="H121 ZEN"; $resn_exch{"2H15 ZEN"}="H152 ZEN";
$resn_exch{"1H15 ZEN"}="H151 ZEN"; $resn_exch{"2H16 ZEN"}="H162 ZEN"; $resn_exch{"1H16 ZEN"}="H161 ZEN";
$resn_exch{"2H18 ZEN"}="H182 ZEN"; $resn_exch{"1H18 ZEN"}="H181 ZEN"; $resn_exch{"2H19 ZEN"}="H192 ZEN";
$resn_exch{"1H19 ZEN"}="H191 ZEN"; $resn_exch{"1H14 ZES"}="H141 ZES"; $resn_exch{"2H14 ZES"}="H142 ZES";
$resn_exch{"1H27 ZES"}="H271 ZES"; $resn_exch{"2H27 ZES"}="H272 ZES"; $resn_exch{"1H17 ZEX"}="H171 ZEX";
$resn_exch{"2H17 ZEX"}="H172 ZEX"; $resn_exch{"3H17 ZEX"}="H173 ZEX"; $resn_exch{"1H4  ZEX"}=" H41 ZEX";
$resn_exch{"2H4  ZEX"}=" H42 ZEX"; $resn_exch{"1H2  ZEX"}=" H21 ZEX"; $resn_exch{"2H2  ZEX"}=" H22 ZEX";
$resn_exch{"1H18 ZEX"}="H181 ZEX"; $resn_exch{"2H18 ZEX"}="H182 ZEX"; $resn_exch{"3H18 ZEX"}="H183 ZEX";
$resn_exch{"1H16 ZEX"}="H161 ZEX"; $resn_exch{"2H16 ZEX"}="H162 ZEX"; $resn_exch{"3H16 ZEX"}="H163 ZEX";
$resn_exch{"1H19 ZEX"}="H191 ZEX"; $resn_exch{"2H19 ZEX"}="H192 ZEX"; $resn_exch{"3H19 ZEX"}="H193 ZEX";
$resn_exch{"1H20 ZEX"}="H201 ZEX"; $resn_exch{"2H20 ZEX"}="H202 ZEX"; $resn_exch{"3H20 ZEX"}="H203 ZEX";
$resn_exch{"1H40 ZEX"}="H401 ZEX"; $resn_exch{"2H40 ZEX"}="H402 ZEX"; $resn_exch{"3H40 ZEX"}="H403 ZEX";
$resn_exch{"1H39 ZEX"}="H391 ZEX"; $resn_exch{"2H39 ZEX"}="H392 ZEX"; $resn_exch{"3H39 ZEX"}="H393 ZEX";
$resn_exch{"1H36 ZEX"}="H361 ZEX"; $resn_exch{"2H36 ZEX"}="H362 ZEX"; $resn_exch{"3H36 ZEX"}="H363 ZEX";
$resn_exch{"1H37 ZEX"}="H371 ZEX"; $resn_exch{"2H37 ZEX"}="H372 ZEX"; $resn_exch{"3H37 ZEX"}="H373 ZEX";
$resn_exch{"1H22 ZEX"}="H221 ZEX"; $resn_exch{"2H22 ZEX"}="H222 ZEX"; $resn_exch{"1H38 ZEX"}="H381 ZEX";
$resn_exch{"2H38 ZEX"}="H382 ZEX"; $resn_exch{"3H38 ZEX"}="H383 ZEX"; $resn_exch{"1H2  ZFB"}=" H21 ZFB";
$resn_exch{"2H2  ZFB"}=" H22 ZFB"; $resn_exch{"1H11 ZFB"}="H111 ZFB"; $resn_exch{"2H11 ZFB"}="H112 ZFB";
$resn_exch{"1H12 ZFB"}="H121 ZFB"; $resn_exch{"2H12 ZFB"}="H122 ZFB"; $resn_exch{"1HN2 ZHH"}="HN21 ZHH";
$resn_exch{"2HN2 ZHH"}="HN22 ZHH"; $resn_exch{"1HA  ZHH"}=" HA1 ZHH"; $resn_exch{"2HA  ZHH"}=" HA2 ZHH";
$resn_exch{"1HB  ZHH"}=" HB1 ZHH"; $resn_exch{"2HB  ZHH"}=" HB2 ZHH"; $resn_exch{"1HF1 ZHH"}="HF11 ZHH";
$resn_exch{"2HF1 ZHH"}="HF12 ZHH"; $resn_exch{"3HF1 ZHH"}="HF13 ZHH"; $resn_exch{"1HN2 ZHZ"}="HN21 ZHZ";
$resn_exch{"2HN2 ZHZ"}="HN22 ZHZ"; $resn_exch{"1HB  ZHZ"}=" HB1 ZHZ"; $resn_exch{"2HB  ZHZ"}=" HB2 ZHZ";
$resn_exch{"1HF1 ZHZ"}="HF11 ZHZ"; $resn_exch{"2HF1 ZHZ"}="HF12 ZHZ"; $resn_exch{"3HF1 ZHZ"}="HF13 ZHZ";
$resn_exch{"2HOA ZID"}="HOA2 ZID"; $resn_exch{"AH51 ZID"}="H51A ZID"; $resn_exch{"AH52 ZID"}="H52A ZID";
$resn_exch{"AH4* ZID"}=" H4B ZID"; $resn_exch{"AH3* ZID"}=" H3B ZID"; $resn_exch{"AHO3 ZID"}="HO3A ZID";
$resn_exch{"AH2* ZID"}=" H2B ZID"; $resn_exch{"AHO2 ZID"}="HO2A ZID"; $resn_exch{"AH1* ZID"}=" H1B ZID";
$resn_exch{" AH8 ZID"}=" H8A ZID"; $resn_exch{"AH61 ZID"}="H61A ZID"; $resn_exch{"AH62 ZID"}="H62A ZID";
$resn_exch{" AH2 ZID"}=" H2A ZID"; $resn_exch{"NH51 ZID"}="H51N ZID"; $resn_exch{"NH52 ZID"}="H52N ZID";
$resn_exch{"NH4* ZID"}=" H4D ZID"; $resn_exch{"NH3* ZID"}=" H3D ZID"; $resn_exch{"NHO3 ZID"}="HO3N ZID";
$resn_exch{"NH2* ZID"}=" H2D ZID"; $resn_exch{"NHO2 ZID"}="HO2N ZID"; $resn_exch{"NH1* ZID"}=" H1D ZID";
$resn_exch{" NH2 ZID"}=" H2N ZID"; $resn_exch{"NH71 ZID"}="H71N ZID"; $resn_exch{"NH72 ZID"}="H72N ZID";
$resn_exch{" NH5 ZID"}=" H5N ZID"; $resn_exch{" NH6 ZID"}=" H6N ZID"; $resn_exch{" ZH2 ZID"}=" H2Z ZID";
$resn_exch{" ZH3 ZID"}=" H3Z ZID"; $resn_exch{" ZH5 ZID"}=" H5Z ZID"; $resn_exch{" ZH6 ZID"}=" H6Z ZID";
$resn_exch{"1H7C ZIO"}="H7C1 ZIO"; $resn_exch{"2H7C ZIO"}="H7C2 ZIO"; $resn_exch{"1H34 ZIO"}="H341 ZIO";
$resn_exch{"2H34 ZIO"}="H342 ZIO"; $resn_exch{"3H34 ZIO"}="H343 ZIO"; $resn_exch{"1H33 ZIO"}="H331 ZIO";
$resn_exch{"2H33 ZIO"}="H332 ZIO"; $resn_exch{"1H35 ZIO"}="H351 ZIO"; $resn_exch{"2H35 ZIO"}="H352 ZIO";
$resn_exch{"3H35 ZIO"}="H353 ZIO"; $resn_exch{"1H36 ZIO"}="H361 ZIO"; $resn_exch{"2H36 ZIO"}="H362 ZIO";
$resn_exch{"3H36 ZIO"}="H363 ZIO"; $resn_exch{"1H30 ZIO"}="H301 ZIO"; $resn_exch{"2H30 ZIO"}="H302 ZIO";
$resn_exch{"3H30 ZIO"}="H303 ZIO"; $resn_exch{"1H32 ZIO"}="H321 ZIO"; $resn_exch{"2H32 ZIO"}="H322 ZIO";
$resn_exch{"3H32 ZIO"}="H323 ZIO"; $resn_exch{"1H25 ZIO"}="H251 ZIO"; $resn_exch{"2H25 ZIO"}="H252 ZIO";
$resn_exch{"1H27 ZIO"}="H271 ZIO"; $resn_exch{"2H27 ZIO"}="H272 ZIO"; $resn_exch{"3H27 ZIO"}="H273 ZIO";
$resn_exch{"1H28 ZIO"}="H281 ZIO"; $resn_exch{"2H28 ZIO"}="H282 ZIO"; $resn_exch{"3H28 ZIO"}="H283 ZIO";
$resn_exch{"1H15 ZIO"}="H151 ZIO"; $resn_exch{"2H15 ZIO"}="H152 ZIO"; $resn_exch{"1H20 ZIO"}="H201 ZIO";
$resn_exch{"2H20 ZIO"}="H202 ZIO"; $resn_exch{"3H20 ZIO"}="H203 ZIO"; $resn_exch{"1H29 ZIO"}="H291 ZIO";
$resn_exch{"2H29 ZIO"}="H292 ZIO"; $resn_exch{"3H29 ZIO"}="H293 ZIO"; $resn_exch{"1H21 ZIO"}="H211 ZIO";
$resn_exch{"2H21 ZIO"}="H212 ZIO"; $resn_exch{"3H21 ZIO"}="H213 ZIO"; $resn_exch{"1H31 ZIO"}="H311 ZIO";
$resn_exch{"2H31 ZIO"}="H312 ZIO"; $resn_exch{"3H31 ZIO"}="H313 ZIO"; $resn_exch{"1H14 ZIP"}="H141 ZIP";
$resn_exch{"2H14 ZIP"}="H142 ZIP"; $resn_exch{"3H14 ZIP"}="H143 ZIP"; $resn_exch{"1H15 ZIP"}="H151 ZIP";
$resn_exch{"2H15 ZIP"}="H152 ZIP"; $resn_exch{"3H15 ZIP"}="H153 ZIP"; $resn_exch{"1H11 ZIP"}="H111 ZIP";
$resn_exch{"2H11 ZIP"}="H112 ZIP"; $resn_exch{"1H7  ZIT"}=" H71 ZIT"; $resn_exch{"2H7  ZIT"}=" H72 ZIT";
$resn_exch{"1H9  ZIT"}=" H91 ZIT"; $resn_exch{"2H9  ZIT"}=" H92 ZIT"; $resn_exch{"1H15 ZIT"}="H151 ZIT";
$resn_exch{"2H15 ZIT"}="H152 ZIT"; $resn_exch{"1H16 ZIT"}="H161 ZIT"; $resn_exch{"2H16 ZIT"}="H162 ZIT";
$resn_exch{"3H16 ZIT"}="H163 ZIT"; $resn_exch{"1H17 ZIT"}="H171 ZIT"; $resn_exch{"2H17 ZIT"}="H172 ZIT";
$resn_exch{"3H17 ZIT"}="H173 ZIT"; $resn_exch{"1H18 ZIT"}="H181 ZIT"; $resn_exch{"2H18 ZIT"}="H182 ZIT";
$resn_exch{"3H18 ZIT"}="H183 ZIT"; $resn_exch{"1H19 ZIT"}="H191 ZIT"; $resn_exch{"2H19 ZIT"}="H192 ZIT";
$resn_exch{"3H19 ZIT"}="H193 ZIT"; $resn_exch{"1H20 ZIT"}="H201 ZIT"; $resn_exch{"2H20 ZIT"}="H202 ZIT";
$resn_exch{"3H20 ZIT"}="H203 ZIT"; $resn_exch{"1H21 ZIT"}="H211 ZIT"; $resn_exch{"2H21 ZIT"}="H212 ZIT";
$resn_exch{"3H21 ZIT"}="H213 ZIT"; $resn_exch{"1H22 ZIT"}="H221 ZIT"; $resn_exch{"2H22 ZIT"}="H222 ZIT";
$resn_exch{"3H22 ZIT"}="H223 ZIT"; $resn_exch{"1H23 ZIT"}="H231 ZIT"; $resn_exch{"2H23 ZIT"}="H232 ZIT";
$resn_exch{"3H23 ZIT"}="H233 ZIT"; $resn_exch{"1H1A ZIT"}="H1A1 ZIT"; $resn_exch{"1H2A ZIT"}="H2A1 ZIT";
$resn_exch{"1H3A ZIT"}="H3A1 ZIT"; $resn_exch{"1H4A ZIT"}="H4A1 ZIT"; $resn_exch{"2H4A ZIT"}="H4A2 ZIT";
$resn_exch{"1H5A ZIT"}="H5A1 ZIT"; $resn_exch{"1C6A ZIT"}="H6A1 ZIT"; $resn_exch{"2C6A ZIT"}="H6A2 ZIT";
$resn_exch{"3C6A ZIT"}="H6A3 ZIT"; $resn_exch{"1H7A ZIT"}="H7A1 ZIT"; $resn_exch{"2H7A ZIT"}="H7A2 ZIT";
$resn_exch{"3H7A ZIT"}="H7A3 ZIT"; $resn_exch{"1H8A ZIT"}="H8A1 ZIT"; $resn_exch{"2H8A ZIT"}="H8A2 ZIT";
$resn_exch{"3H8A ZIT"}="H8A3 ZIT"; $resn_exch{"1H1B ZIT"}="H1B1 ZIT"; $resn_exch{"1H2B ZIT"}="H2B1 ZIT";
$resn_exch{"2H2B ZIT"}="H2B2 ZIT"; $resn_exch{"1H4B ZIT"}="H4B1 ZIT"; $resn_exch{"1H5B ZIT"}="H5B1 ZIT";
$resn_exch{"1C6B ZIT"}="H6B1 ZIT"; $resn_exch{"2C6B ZIT"}="H6B2 ZIT"; $resn_exch{"3C6B ZIT"}="H6B3 ZIT";
$resn_exch{"1H7B ZIT"}="H7B1 ZIT"; $resn_exch{"2H7B ZIT"}="H7B2 ZIT"; $resn_exch{"3H7B ZIT"}="H7B3 ZIT";
$resn_exch{"1H8B ZIT"}="H8B1 ZIT"; $resn_exch{"2H8B ZIT"}="H8B2 ZIT"; $resn_exch{"3H8B ZIT"}="H8B3 ZIT";
$resn_exch{"1H2C ZK5"}="H2C1 ZK5"; $resn_exch{"2H2C ZK5"}="H2C2 ZK5"; $resn_exch{"1H6C ZK5"}="H6C1 ZK5";
$resn_exch{"2H6C ZK5"}="H6C2 ZK5"; $resn_exch{"1H7C ZK5"}="H7C1 ZK5"; $resn_exch{"2H7C ZK5"}="H7C2 ZK5";
$resn_exch{"1H1C ZK5"}="H1C1 ZK5"; $resn_exch{"2H1C ZK5"}="H1C2 ZK5"; $resn_exch{"1H12 ZK5"}="H121 ZK5";
$resn_exch{"2H12 ZK5"}="H122 ZK5"; $resn_exch{"1H15 ZK5"}="H151 ZK5"; $resn_exch{"2H15 ZK5"}="H152 ZK5";
$resn_exch{"1H16 ZK5"}="H161 ZK5"; $resn_exch{"2H16 ZK5"}="H162 ZK5"; $resn_exch{"1H18 ZK5"}="H181 ZK5";
$resn_exch{"2H18 ZK5"}="H182 ZK5"; $resn_exch{"3H18 ZK5"}="H183 ZK5"; $resn_exch{"1H19 ZK5"}="H191 ZK5";
$resn_exch{"2H19 ZK5"}="H192 ZK5"; $resn_exch{"3H19 ZK5"}="H193 ZK5"; $resn_exch{"1H21 ZK5"}="H211 ZK5";
$resn_exch{"2H21 ZK5"}="H212 ZK5"; $resn_exch{"1H11 ZMR"}="H111 ZMR"; $resn_exch{"2H11 ZMR"}="H112 ZMR";
$resn_exch{"3H11 ZMR"}="H113 ZMR"; $resn_exch{"1H9  ZMR"}=" H91 ZMR"; $resn_exch{"2H9  ZMR"}=" H92 ZMR";
$resn_exch{"1HN1 ZMR"}="HN11 ZMR"; $resn_exch{"2HN1 ZMR"}="HN12 ZMR"; $resn_exch{"1HN2 ZMR"}="HN21 ZMR";
$resn_exch{"2HN2 ZMR"}="HN22 ZMR"; $resn_exch{"1HO1 ZN3"}="HO11 ZN3"; $resn_exch{"2HO1 ZN3"}="HO12 ZN3";
$resn_exch{"1HMA ZNH"}="HMA1 ZNH"; $resn_exch{"2HMA ZNH"}="HMA2 ZNH"; $resn_exch{"3HMA ZNH"}="HMA3 ZNH";
$resn_exch{"1HAA ZNH"}="HAA1 ZNH"; $resn_exch{"2HAA ZNH"}="HAA2 ZNH"; $resn_exch{"1HBA ZNH"}="HBA1 ZNH";
$resn_exch{"2HBA ZNH"}="HBA2 ZNH"; $resn_exch{"1HMB ZNH"}="HMB1 ZNH"; $resn_exch{"2HMB ZNH"}="HMB2 ZNH";
$resn_exch{"3HMB ZNH"}="HMB3 ZNH"; $resn_exch{"1HBB ZNH"}="HBB1 ZNH"; $resn_exch{"2HBB ZNH"}="HBB2 ZNH";
$resn_exch{"1HMC ZNH"}="HMC1 ZNH"; $resn_exch{"2HMC ZNH"}="HMC2 ZNH"; $resn_exch{"3HMC ZNH"}="HMC3 ZNH";
$resn_exch{"1HBC ZNH"}="HBC1 ZNH"; $resn_exch{"2HBC ZNH"}="HBC2 ZNH"; $resn_exch{"1HMD ZNH"}="HMD1 ZNH";
$resn_exch{"2HMD ZNH"}="HMD2 ZNH"; $resn_exch{"3HMD ZNH"}="HMD3 ZNH"; $resn_exch{"1HAD ZNH"}="HAD1 ZNH";
$resn_exch{"2HAD ZNH"}="HAD2 ZNH"; $resn_exch{"1HBD ZNH"}="HBD1 ZNH"; $resn_exch{"2HBD ZNH"}="HBD2 ZNH";
$resn_exch{"1HO1 ZNO"}="HO11 ZNO"; $resn_exch{"2HO1 ZNO"}="HO12 ZNO"; $resn_exch{"3HO1 ZNO"}="HO13 ZNO";
$resn_exch{"4HO1 ZNO"}="HO14 ZNO"; $resn_exch{"1HO1 ZO3"}="HO11 ZO3"; $resn_exch{"2HO1 ZO3"}="HO12 ZO3";
$resn_exch{"1HO2 ZO3"}="HO21 ZO3"; $resn_exch{"2HO2 ZO3"}="HO22 ZO3"; $resn_exch{"1HO3 ZO3"}="HO31 ZO3";
$resn_exch{"2HO3 ZO3"}="HO32 ZO3"; $resn_exch{"1H7  ZOL"}=" H71 ZOL"; $resn_exch{"2H7  ZOL"}=" H72 ZOL";
$resn_exch{"1H28 ZPG"}="H281 ZPG"; $resn_exch{"2H28 ZPG"}="H282 ZPG"; $resn_exch{"3H28 ZPG"}="H283 ZPG";
$resn_exch{"1H24 ZPG"}="H241 ZPG"; $resn_exch{"2H24 ZPG"}="H242 ZPG"; $resn_exch{"1H26 ZPG"}="H261 ZPG";
$resn_exch{"2H26 ZPG"}="H262 ZPG"; $resn_exch{"3H26 ZPG"}="H263 ZPG"; $resn_exch{"1H21 ZPG"}="H211 ZPG";
$resn_exch{"2H21 ZPG"}="H212 ZPG"; $resn_exch{"1H25 ZPG"}="H251 ZPG"; $resn_exch{"2H25 ZPG"}="H252 ZPG";
$resn_exch{"3H25 ZPG"}="H253 ZPG"; $resn_exch{"1H18 ZPG"}="H181 ZPG"; $resn_exch{"2H18 ZPG"}="H182 ZPG";
$resn_exch{"1H16 ZPG"}="H161 ZPG"; $resn_exch{"2H16 ZPG"}="H162 ZPG"; $resn_exch{"3H16 ZPG"}="H163 ZPG";
$resn_exch{"1H14 ZPG"}="H141 ZPG"; $resn_exch{"2H14 ZPG"}="H142 ZPG"; $resn_exch{"1H12 ZPG"}="H121 ZPG";
$resn_exch{"2H12 ZPG"}="H122 ZPG"; $resn_exch{"3H12 ZPG"}="H123 ZPG"; $resn_exch{"1H10 ZPG"}="H101 ZPG";
$resn_exch{"2H10 ZPG"}="H102 ZPG"; $resn_exch{"1H8  ZPG"}=" H81 ZPG"; $resn_exch{"2H8  ZPG"}=" H82 ZPG";
$resn_exch{"3H8  ZPG"}=" H83 ZPG"; $resn_exch{"1H6  ZPG"}=" H61 ZPG"; $resn_exch{"2H6  ZPG"}=" H62 ZPG";
$resn_exch{"1H4  ZPG"}=" H41 ZPG"; $resn_exch{"2H4  ZPG"}=" H42 ZPG"; $resn_exch{"1H3  ZPG"}=" H31 ZPG";
$resn_exch{"2H3  ZPG"}=" H32 ZPG"; $resn_exch{"3H3  ZPG"}=" H33 ZPG"; $resn_exch{"1H14 ZPQ"}="H141 ZPQ";
$resn_exch{"2H14 ZPQ"}="H142 ZPQ"; $resn_exch{"3H14 ZPQ"}="H143 ZPQ"; $resn_exch{"1H15 ZPQ"}="H151 ZPQ";
$resn_exch{"2H15 ZPQ"}="H152 ZPQ"; $resn_exch{"1H17 ZPQ"}="H171 ZPQ"; $resn_exch{"2H17 ZPQ"}="H172 ZPQ";
$resn_exch{"3H17 ZPQ"}="H173 ZPQ"; $resn_exch{"1H22 ZPQ"}="H221 ZPQ"; $resn_exch{"2H22 ZPQ"}="H222 ZPQ";
$resn_exch{"1H23 ZPQ"}="H231 ZPQ"; $resn_exch{"2H23 ZPQ"}="H232 ZPQ"; $resn_exch{"3H23 ZPQ"}="H233 ZPQ";
$resn_exch{"1H33 ZPQ"}="H331 ZPQ"; $resn_exch{"2H33 ZPQ"}="H332 ZPQ"; $resn_exch{"1H35 ZPQ"}="H351 ZPQ";
$resn_exch{"2H35 ZPQ"}="H352 ZPQ"; $resn_exch{"3H35 ZPQ"}="H353 ZPQ"; $resn_exch{"1H36 ZPQ"}="H361 ZPQ";
$resn_exch{"2H36 ZPQ"}="H362 ZPQ"; $resn_exch{"3H36 ZPQ"}="H363 ZPQ"; $resn_exch{"1H37 ZPQ"}="H371 ZPQ";
$resn_exch{"2H37 ZPQ"}="H372 ZPQ"; $resn_exch{"1H39 ZPQ"}="H391 ZPQ"; $resn_exch{"2H39 ZPQ"}="H392 ZPQ";
$resn_exch{"3H39 ZPQ"}="H393 ZPQ"; $resn_exch{"1H43 ZPQ"}="H431 ZPQ"; $resn_exch{"2H43 ZPQ"}="H432 ZPQ";
$resn_exch{"3H43 ZPQ"}="H433 ZPQ"; $resn_exch{"1H44 ZPQ"}="H441 ZPQ"; $resn_exch{"2H44 ZPQ"}="H442 ZPQ";
$resn_exch{"3H44 ZPQ"}="H443 ZPQ"; $resn_exch{"1H51 ZPQ"}="H511 ZPQ"; $resn_exch{"2H51 ZPQ"}="H512 ZPQ";
$resn_exch{"1H53 ZPQ"}="H531 ZPQ"; $resn_exch{"2H53 ZPQ"}="H532 ZPQ"; $resn_exch{"3H53 ZPQ"}="H533 ZPQ";
$resn_exch{"1H54 ZPQ"}="H541 ZPQ"; $resn_exch{"2H54 ZPQ"}="H542 ZPQ"; $resn_exch{"3H54 ZPQ"}="H543 ZPQ";
$resn_exch{"1H1  ZPR"}=" H11 ZPR"; $resn_exch{"1H3  ZPR"}=" H31 ZPR"; $resn_exch{"1H4  ZPR"}=" H41 ZPR";
$resn_exch{"2H4  ZPR"}=" H42 ZPR"; $resn_exch{"1H5  ZPR"}=" H51 ZPR"; $resn_exch{"2H5  ZPR"}=" H52 ZPR";
$resn_exch{"1H6  ZPR"}=" H61 ZPR"; $resn_exch{"2H6  ZPR"}=" H62 ZPR"; $resn_exch{"1H10 ZPR"}="H101 ZPR";
$resn_exch{"1H11 ZPR"}="H111 ZPR"; $resn_exch{"2H11 ZPR"}="H112 ZPR"; $resn_exch{"1H12 ZPR"}="H121 ZPR";
$resn_exch{"2H12 ZPR"}="H122 ZPR"; $resn_exch{"1H13 ZPR"}="H131 ZPR"; $resn_exch{"2H13 ZPR"}="H132 ZPR";
$resn_exch{"1H18 ZPR"}="H181 ZPR"; $resn_exch{"2H18 ZPR"}="H182 ZPR"; $resn_exch{"1H20 ZPR"}="H201 ZPR";
$resn_exch{"1H21 ZPR"}="H211 ZPR"; $resn_exch{"1H22 ZPR"}="H221 ZPR"; $resn_exch{"1H23 ZPR"}="H231 ZPR";
$resn_exch{"1H24 ZPR"}="H241 ZPR"; $resn_exch{"1H2  ZRA"}=" H21 ZRA"; $resn_exch{"2H2  ZRA"}=" H22 ZRA";
$resn_exch{"1HB1 ZRA"}="HB11 ZRA"; $resn_exch{"2HB1 ZRA"}="HB12 ZRA"; $resn_exch{"1HG  ZRA"}=" HG1 ZRA";
$resn_exch{"2HG  ZRA"}=" HG2 ZRA"; $resn_exch{"1HD  ZRA"}=" HD1 ZRA"; $resn_exch{"2HD  ZRA"}=" HD2 ZRA";
$resn_exch{"1HB2 ZRA"}="HB21 ZRA"; $resn_exch{"2HB2 ZRA"}="HB22 ZRA"; $resn_exch{"3HB2 ZRA"}="HB23 ZRA";
$resn_exch{"1HM  ZRA"}=" HM1 ZRA"; $resn_exch{"2HM  ZRA"}=" HM2 ZRA"; $resn_exch{"3HM  ZRA"}=" HM3 ZRA";
$resn_exch{"1H9  ZST"}=" H91 ZST"; $resn_exch{"2H9  ZST"}=" H92 ZST"; $resn_exch{"1H17 ZST"}="H171 ZST";
$resn_exch{"2H17 ZST"}="H172 ZST"; $resn_exch{"1H2  ZYA"}=" H21 ZYA"; $resn_exch{"2H2  ZYA"}=" H22 ZYA";
$resn_exch{"1HB1 ZYA"}="HB11 ZYA"; $resn_exch{"2HB1 ZYA"}="HB12 ZYA"; $resn_exch{"1HB2 ZYA"}="HB21 ZYA";
$resn_exch{"2HB2 ZYA"}="HB22 ZYA"; $resn_exch{"3HB2 ZYA"}="HB23 ZYA"; $resn_exch{"1HM  ZYA"}=" HM1 ZYA";
$resn_exch{"2HM  ZYA"}=" HM2 ZYA"; $resn_exch{"3HM  ZYA"}=" HM3 ZYA"; $resn_exch{"1HN1 ZYX"}="HN11 ZYX";
$resn_exch{"2HN1 ZYX"}="HN12 ZYX"; $resn_exch{"1H7  ZYX"}=" H71 ZYX"; $resn_exch{"2H7  ZYX"}=" H72 ZYX";
$resn_exch{"1H8  ZYX"}=" H81 ZYX"; $resn_exch{"2H8  ZYX"}=" H82 ZYX"; $resn_exch{"1HN2 ZYX"}="HN21 ZYX";
$resn_exch{"2HN2 ZYX"}="HN22 ZYX"; $resn_exch{"1H6C ZZ1"}="H6C1 ZZ1"; $resn_exch{"2H6C ZZ1"}="H6C2 ZZ1";
$resn_exch{"3H6C ZZ1"}="H6C3 ZZ1"; $resn_exch{"1H7C ZZZ"}="H7C1 ZZZ"; $resn_exch{"2H7C ZZZ"}="H7C2 ZZZ";
$resn_exch{"1H2N ZZZ"}="H2N1 ZZZ"; $resn_exch{"2H2N ZZZ"}="H2N2 ZZZ"; 
}

sub build_new2old{

%resn_exch=();

$resn_exch{" C7   DT"}=" C5A  DT"; $resn_exch{"H2''   A"}="2H2*   A"; $resn_exch{"H2''   C"}="2H2*   C";
$resn_exch{"H2''   T"}="2H2*   T"; $resn_exch{"H2''   G"}="2H2*   G"; $resn_exch{"H2''   I"}="2H2*   I";
$resn_exch{" H2'   A"}="1H2*   A"; $resn_exch{" H2'   C"}="1H2*   C"; $resn_exch{" H2'   U"}="1H2*   U";
$resn_exch{" H2'   G"}="1H2*   G"; $resn_exch{" H2'   I"}="1H2*   I"; $resn_exch{"HO5'   A"}="5HO*   A";
$resn_exch{"HO5'   C"}="5HO*   C"; $resn_exch{"HO5'   U"}="5HO*   U"; $resn_exch{"HO5'   G"}="5HO*   G";
$resn_exch{"HO5'   I"}="5HO*   I"; $resn_exch{"HO5'  DT"}="5HO*  DT"; $resn_exch{"HO5'  DA"}="5HO*  DA";
$resn_exch{"HO5'  DC"}="5HO*  DC"; $resn_exch{"HO5'  DG"}="5HO*  DG"; $resn_exch{"HO5'  DI"}="5HO*  DI";
$resn_exch{"HO5'  DU"}="5HO*  DU"; $resn_exch{"HO3'   A"}="3HO*   A"; $resn_exch{"HO3'   C"}="3HO*   C";
$resn_exch{"HO3'   U"}="3HO*   U"; $resn_exch{"HO3'   G"}="3HO*   G"; $resn_exch{"HO3'   I"}="3HO*   I";
$resn_exch{"HO3'  DT"}="3HO*  DT"; $resn_exch{"HO3'  DA"}="3HO*  DA"; $resn_exch{"HO3'  DC"}="3HO*  DC";
$resn_exch{"HO3'  DG"}="3HO*  DG"; $resn_exch{"HO3'  DI"}="3HO*  DI"; $resn_exch{"HO3'  DU"}="3HO*  DU";
$resn_exch{" H1  GLY"}="1H   GLY"; $resn_exch{" H2  GLY"}="2H   GLY"; $resn_exch{" H3  GLY"}="3H   GLY";
$resn_exch{" H1  ALA"}="1H   ALA"; $resn_exch{" H2  ALA"}="2H   ALA"; $resn_exch{" H3  ALA"}="3H   ALA";
$resn_exch{" H1  VAL"}="1H   VAL"; $resn_exch{" H2  VAL"}="2H   VAL"; $resn_exch{" H3  VAL"}="3H   VAL";
$resn_exch{" H1  PHE"}="1H   PHE"; $resn_exch{" H2  PHE"}="2H   PHE"; $resn_exch{" H3  PHE"}="3H   PHE";
$resn_exch{" H1  PRO"}="1H   PRO"; $resn_exch{" H2  PRO"}="2H   PRO"; $resn_exch{" H3  PRO"}="3H   PRO";
$resn_exch{" H1  MET"}="1H   MET"; $resn_exch{" H2  MET"}="2H   MET"; $resn_exch{" H3  MET"}="3H   MET";
$resn_exch{" H1  ILE"}="1H   ILE"; $resn_exch{" H2  ILE"}="2H   ILE"; $resn_exch{" H3  ILE"}="3H   ILE";
$resn_exch{" H1  LEU"}="1H   LEU"; $resn_exch{" H2  LEU"}="2H   LEU"; $resn_exch{" H3  LEU"}="3H   LEU";
$resn_exch{" H1  ASP"}="1H   ASP"; $resn_exch{" H2  ASP"}="2H   ASP"; $resn_exch{" H3  ASP"}="3H   ASP";
$resn_exch{" H1  GLU"}="1H   GLU"; $resn_exch{" H2  GLU"}="2H   GLU"; $resn_exch{" H3  GLU"}="3H   GLU";
$resn_exch{" H1  LYS"}="1H   LYS"; $resn_exch{" H2  LYS"}="2H   LYS"; $resn_exch{" H3  LYS"}="3H   LYS";
$resn_exch{" H1  ARG"}="1H   ARG"; $resn_exch{" H2  ARG"}="2H   ARG"; $resn_exch{" H3  ARG"}="3H   ARG";
$resn_exch{" H1  SER"}="1H   SER"; $resn_exch{" H2  SER"}="2H   SER"; $resn_exch{" H3  SER"}="3H   SER";
$resn_exch{" H1  THR"}="1H   THR"; $resn_exch{" H2  THR"}="2H   THR"; $resn_exch{" H3  THR"}="3H   THR";
$resn_exch{" H1  HIS"}="1H   HIS"; $resn_exch{" H2  HIS"}="2H   HIS"; $resn_exch{" H3  HIS"}="3H   HIS";
$resn_exch{" H1  CYS"}="1H   CYS"; $resn_exch{" H2  CYS"}="2H   CYS"; $resn_exch{" H3  CYS"}="3H   CYS";
$resn_exch{" H1  ASN"}="1H   ASN"; $resn_exch{" H2  ASN"}="2H   ASN"; $resn_exch{" H3  ASN"}="3H   ASN";
$resn_exch{" H1  GLN"}="1H   GLN"; $resn_exch{" H2  GLN"}="2H   GLN"; $resn_exch{" H3  GLN"}="3H   GLN";
$resn_exch{" H1  TRP"}="1H   TRP"; $resn_exch{" H2  TRP"}="2H   TRP"; $resn_exch{" H3  TRP"}="3H   TRP";
$resn_exch{" H1  ASX"}="1H   ASX"; $resn_exch{" H2  ASX"}="2H   ASX"; $resn_exch{" H3  ASX"}="3H   ASX";
$resn_exch{" H1  GLX"}="1H   GLX"; $resn_exch{" H2  GLX"}="2H   GLX"; $resn_exch{" H3  GLX"}="3H   GLX";
$resn_exch{" H1  ABU"}="1H   ABU"; $resn_exch{" H2  ABU"}="2H   ABU"; $resn_exch{" H3  ABU"}="3H   ABU";
$resn_exch{" H1  AIB"}="1H   AIB"; $resn_exch{" H2  AIB"}="2H   AIB"; $resn_exch{" H3  AIB"}="3H   AIB";
$resn_exch{" H1  ABU"}="1H   ABU"; $resn_exch{" H2  ABU"}="2H   ABU"; $resn_exch{" H3  ABU"}="3H   ABU";
$resn_exch{" H1  MSE"}="1H   MSE"; $resn_exch{" H2  MSE"}="2H   MSE"; $resn_exch{" H3  MSE"}="3H   MSE";
$resn_exch{" H1  PCA"}="1H   PCA"; $resn_exch{" H2  PCA"}="2H   PCA"; $resn_exch{" H3  PCA"}="3H   PCA";
$resn_exch{" HN1 0AO"}="1HN  0AO"; $resn_exch{" OD1 0AZ"}=" OD  0AZ"; $resn_exch{" O5' 101"}=" O5* 101";
$resn_exch{" C5' 101"}=" C5* 101"; $resn_exch{" C4' 101"}=" C4* 101"; $resn_exch{" O4' 101"}=" O4* 101";
$resn_exch{" C3' 101"}=" C3* 101"; $resn_exch{" O3' 101"}=" O3* 101"; $resn_exch{" C2' 101"}=" C2* 101";
$resn_exch{" C1' 101"}=" C1* 101"; $resn_exch{" O5' 102"}=" O5* 102"; $resn_exch{" C5' 102"}=" C5* 102";
$resn_exch{" C4' 102"}=" C4* 102"; $resn_exch{" O4' 102"}=" O4* 102"; $resn_exch{" C3' 102"}=" C3* 102";
$resn_exch{" O3' 102"}=" O3* 102"; $resn_exch{" C2' 102"}=" C2* 102"; $resn_exch{" O2' 102"}=" O2* 102";
$resn_exch{" C1' 102"}=" C1* 102"; $resn_exch{" C5' 103"}=" C5* 103"; $resn_exch{" C4' 103"}=" C4* 103";
$resn_exch{" O4' 103"}=" O4* 103"; $resn_exch{" C3' 103"}=" C3* 103"; $resn_exch{" O3' 103"}=" O3* 103";
$resn_exch{" C2' 103"}=" C2* 103"; $resn_exch{" C1' 103"}=" C1* 103"; $resn_exch{" O5' 10C"}=" O5* 10C";
$resn_exch{" C5' 10C"}=" C5* 10C"; $resn_exch{" C4' 10C"}=" C4* 10C"; $resn_exch{" O4' 10C"}=" O4* 10C";
$resn_exch{" C1' 10C"}=" C1* 10C"; $resn_exch{" C3' 10C"}=" C3* 10C"; $resn_exch{" C2' 10C"}=" C2* 10C";
$resn_exch{" O2' 10C"}=" O2* 10C"; $resn_exch{" O3' 10C"}=" O3* 10C"; $resn_exch{" C6' 10C"}=" C6* 10C";
$resn_exch{" O5' 112"}=" O5* 112"; $resn_exch{" C5' 112"}=" C5* 112"; $resn_exch{" C4' 112"}=" C4* 112";
$resn_exch{" O4' 112"}=" O4* 112"; $resn_exch{" C3' 112"}=" C3* 112"; $resn_exch{" O3' 112"}=" O3* 112";
$resn_exch{" C2' 112"}=" C2* 112"; $resn_exch{" O2' 112"}=" O2* 112"; $resn_exch{" C1' 112"}=" C1* 112";
$resn_exch{" OP1 125"}=" O1P 125"; $resn_exch{" OP2 125"}=" O2P 125"; $resn_exch{" OP3 125"}=" O3P 125";
$resn_exch{" O5' 125"}=" O5* 125"; $resn_exch{" C5' 125"}=" C5* 125"; $resn_exch{" C4' 125"}=" C4* 125";
$resn_exch{" O4' 125"}=" O4* 125"; $resn_exch{" C3' 125"}=" C3* 125"; $resn_exch{" O3' 125"}=" O3* 125";
$resn_exch{" C2' 125"}=" C2* 125"; $resn_exch{" O2' 125"}=" O2* 125"; $resn_exch{" C1' 125"}=" C1* 125";
$resn_exch{" CA' 125"}=" CA* 125"; $resn_exch{" CB' 125"}=" CB* 125"; $resn_exch{" FC' 125"}=" FC* 125";
$resn_exch{" OP1 126"}=" O1P 126"; $resn_exch{" OP2 126"}=" O2P 126"; $resn_exch{" OP3 126"}=" O3P 126";
$resn_exch{" O5' 126"}=" O5* 126"; $resn_exch{" C5' 126"}=" C5* 126"; $resn_exch{" C4' 126"}=" C4* 126";
$resn_exch{" O4' 126"}=" O4* 126"; $resn_exch{" C3' 126"}=" C3* 126"; $resn_exch{" O3' 126"}=" O3* 126";
$resn_exch{" C2' 126"}=" C2* 126"; $resn_exch{" O2' 126"}=" O2* 126"; $resn_exch{" C1' 126"}=" C1* 126";
$resn_exch{" CA' 126"}=" CA* 126"; $resn_exch{" CB' 126"}=" CB* 126"; $resn_exch{" OC' 126"}=" OC* 126";
$resn_exch{" CD' 126"}=" CD* 126"; $resn_exch{" CE' 126"}=" CE* 126"; $resn_exch{" OF' 126"}=" OF* 126";
$resn_exch{" CG' 126"}=" CG* 126"; $resn_exch{" CH' 126"}=" CH* 126"; $resn_exch{" OI' 126"}=" OI* 126";
$resn_exch{" CK' 126"}=" CK* 126"; $resn_exch{" OP1 127"}=" O1P 127"; $resn_exch{" OP2 127"}=" O2P 127";
$resn_exch{" OP3 127"}=" O3P 127"; $resn_exch{" O5' 127"}=" O5* 127"; $resn_exch{" C5' 127"}=" C5* 127";
$resn_exch{" C4' 127"}=" C4* 127"; $resn_exch{" O4' 127"}=" O4* 127"; $resn_exch{" C3' 127"}=" C3* 127";
$resn_exch{" O3' 127"}=" O3* 127"; $resn_exch{" C2' 127"}=" C2* 127"; $resn_exch{" O2' 127"}=" O2* 127";
$resn_exch{" C1' 127"}=" C1* 127"; $resn_exch{" CA' 127"}=" CA* 127"; $resn_exch{" CB' 127"}=" CB* 127";
$resn_exch{" OC' 127"}=" OC* 127"; $resn_exch{" ND' 127"}=" ND* 127"; $resn_exch{" O5' 128"}=" O5* 128";
$resn_exch{" C5' 128"}=" C5* 128"; $resn_exch{" C4' 128"}=" C4* 128"; $resn_exch{" O4' 128"}=" O4* 128";
$resn_exch{" C3' 128"}=" C3* 128"; $resn_exch{" O3' 128"}=" O3* 128"; $resn_exch{" C2' 128"}=" C2* 128";
$resn_exch{" O2' 128"}=" O2* 128"; $resn_exch{" C1' 128"}=" C1* 128"; $resn_exch{" OP3 12A"}=" O3P 12A";
$resn_exch{" OP1 12A"}=" O1P 12A"; $resn_exch{" OP2 12A"}=" O2P 12A"; $resn_exch{" O5' 12A"}=" O5* 12A";
$resn_exch{" C5' 12A"}=" C5* 12A"; $resn_exch{" C4' 12A"}=" C4* 12A"; $resn_exch{" O4' 12A"}=" O4* 12A";
$resn_exch{" C3' 12A"}=" C3* 12A"; $resn_exch{" O3' 12A"}=" O3* 12A"; $resn_exch{" C2' 12A"}=" C2* 12A";
$resn_exch{" O2' 12A"}=" O2* 12A"; $resn_exch{" C1' 12A"}=" C1* 12A"; $resn_exch{" C1B 130"}=" C1* 130";
$resn_exch{" C2B 130"}=" C2* 130"; $resn_exch{" C3B 130"}=" C3* 130"; $resn_exch{" C4B 130"}=" C4* 130";
$resn_exch{" C5B 130"}=" C5* 130"; $resn_exch{" C6B 130"}=" C6* 130"; $resn_exch{" C1B 132"}=" C1* 132";
$resn_exch{" C2B 132"}=" C2* 132"; $resn_exch{" C3B 132"}=" C3* 132"; $resn_exch{" C4B 132"}=" C4* 132";
$resn_exch{" C5B 132"}=" C5* 132"; $resn_exch{" C6B 132"}=" C6* 132"; $resn_exch{" C1B 134"}=" C1* 134";
$resn_exch{" C2B 134"}=" C2* 134"; $resn_exch{" C3B 134"}=" C3* 134"; $resn_exch{" C4B 134"}=" C4* 134";
$resn_exch{" C5B 134"}=" C5* 134"; $resn_exch{" C6B 134"}=" C6* 134"; $resn_exch{" O5D 139"}="O5*A 139";
$resn_exch{" C5D 139"}="C5*A 139"; $resn_exch{" C4D 139"}="C4*A 139"; $resn_exch{" O4D 139"}="O4*A 139";
$resn_exch{" C1D 139"}="C1*A 139"; $resn_exch{" C2D 139"}="C2*A 139"; $resn_exch{" O2D 139"}="O2*A 139";
$resn_exch{" C3D 139"}="C3*A 139"; $resn_exch{" O3D 139"}="O3*A 139"; $resn_exch{" O5B 139"}="O5*U 139";
$resn_exch{" C5B 139"}="C5*U 139"; $resn_exch{" C4B 139"}="C4*U 139"; $resn_exch{" O4B 139"}="O4*U 139";
$resn_exch{" C1B 139"}="C1*U 139"; $resn_exch{" C2B 139"}="C2*U 139"; $resn_exch{" C3B 139"}="C3*U 139";
$resn_exch{" O3B 139"}="O3*U 139"; $resn_exch{" CDB 176"}=" CD* 176"; $resn_exch{" CGB 176"}=" CG* 176";
$resn_exch{" CDB 177"}=" CD* 177"; $resn_exch{" CGB 177"}=" CG* 177"; $resn_exch{" O5' 191"}=" O5* 191";
$resn_exch{" C5' 191"}=" C5* 191"; $resn_exch{" C4' 191"}=" C4* 191"; $resn_exch{" O4' 191"}=" O4* 191";
$resn_exch{" C3' 191"}=" C3* 191"; $resn_exch{" O3' 191"}=" O3* 191"; $resn_exch{" C2' 191"}=" C2* 191";
$resn_exch{" O2' 191"}=" O2* 191"; $resn_exch{" C1' 191"}=" C1* 191"; $resn_exch{" C9' 1AA"}=" C9* 1AA";
$resn_exch{" C8' 1AA"}=" C8* 1AA"; $resn_exch{" C7' 1AA"}=" C7* 1AA"; $resn_exch{" O2' 1AA"}=" O2* 1AA";
$resn_exch{" O4' 1AA"}=" O4* 1AA"; $resn_exch{" O3' 1AA"}=" O3* 1AA"; $resn_exch{" O1' 1AA"}=" O1* 1AA";
$resn_exch{" C1' 1AA"}=" C1* 1AA"; $resn_exch{" C2' 1AA"}=" C2* 1AA"; $resn_exch{" C3' 1AA"}=" C3* 1AA";
$resn_exch{" C4' 1AA"}=" C4* 1AA"; $resn_exch{" C5' 1AA"}=" C5* 1AA"; $resn_exch{" C6' 1AA"}=" C6* 1AA";
$resn_exch{" OP1 1AP"}=" O1P 1AP"; $resn_exch{" OP2 1AP"}=" O2P 1AP"; $resn_exch{" OP3 1AP"}=" O3P 1AP";
$resn_exch{" O5' 1DA"}=" O5* 1DA"; $resn_exch{" C5' 1DA"}=" C5* 1DA"; $resn_exch{" C4' 1DA"}=" C4* 1DA";
$resn_exch{" O4' 1DA"}=" O4* 1DA"; $resn_exch{" C3' 1DA"}=" C3* 1DA"; $resn_exch{" O3' 1DA"}=" O3* 1DA";
$resn_exch{" C2' 1DA"}=" C2* 1DA"; $resn_exch{" O2' 1DA"}=" O2* 1DA"; $resn_exch{" C1' 1DA"}=" C1* 1DA";
$resn_exch{" C1' 1DG"}=" C1* 1DG"; $resn_exch{" O4' 1DG"}=" O4* 1DG"; $resn_exch{" C2' 1DG"}=" C2* 1DG";
$resn_exch{" O2' 1DG"}=" O2* 1DG"; $resn_exch{" C3' 1DG"}=" C3* 1DG"; $resn_exch{" O3' 1DG"}=" O3* 1DG";
$resn_exch{" C4' 1DG"}=" C4* 1DG"; $resn_exch{" C5' 1DG"}=" C5* 1DG"; $resn_exch{" O5' 1DG"}=" O5* 1DG";
$resn_exch{" CA' 1IQ"}="C10' 1IQ"; $resn_exch{" CB' 1IQ"}="C11' 1IQ"; $resn_exch{" OC' 1IQ"}="O12' 1IQ";
$resn_exch{" OD' 1IQ"}="O13' 1IQ"; $resn_exch{" OP1 1MA"}=" O1P 1MA"; $resn_exch{" OP2 1MA"}=" O2P 1MA";
$resn_exch{" OP3 1MA"}=" O3P 1MA"; $resn_exch{" O5' 1MA"}=" O5* 1MA"; $resn_exch{" C5' 1MA"}=" C5* 1MA";
$resn_exch{" C4' 1MA"}=" C4* 1MA"; $resn_exch{" O4' 1MA"}=" O4* 1MA"; $resn_exch{" C3' 1MA"}=" C3* 1MA";
$resn_exch{" O3' 1MA"}=" O3* 1MA"; $resn_exch{" C2' 1MA"}=" C2* 1MA"; $resn_exch{" O2' 1MA"}=" O2* 1MA";
$resn_exch{" C1' 1MA"}=" C1* 1MA"; $resn_exch{" OP1 1MG"}=" O1P 1MG"; $resn_exch{" OP2 1MG"}=" O2P 1MG";
$resn_exch{" OP3 1MG"}=" O3P 1MG"; $resn_exch{" O5' 1MG"}=" O5* 1MG"; $resn_exch{" C5' 1MG"}=" C5* 1MG";
$resn_exch{" C4' 1MG"}=" C4* 1MG"; $resn_exch{" O4' 1MG"}=" O4* 1MG"; $resn_exch{" C3' 1MG"}=" C3* 1MG";
$resn_exch{" O3' 1MG"}=" O3* 1MG"; $resn_exch{" C2' 1MG"}=" C2* 1MG"; $resn_exch{" O2' 1MG"}=" O2* 1MG";
$resn_exch{" C1' 1MG"}=" C1* 1MG"; $resn_exch{" O'1 1PR"}="1O0' 1PR"; $resn_exch{" C1B 1PR"}=" C1\" 1PR";
$resn_exch{" C2B 1PR"}=" C2\" 1PR"; $resn_exch{" C3B 1PR"}=" C3\" 1PR"; $resn_exch{" C4B 1PR"}=" C4\" 1PR";
$resn_exch{" C5B 1PR"}=" C5\" 1PR"; $resn_exch{" O2B 1PR"}=" O2\" 1PR"; $resn_exch{" O3B 1PR"}=" O3\" 1PR";
$resn_exch{" O4B 1PR"}=" O4\" 1PR"; $resn_exch{" O6B 1PR"}=" O6\" 1PR"; $resn_exch{" P7B 1PR"}=" P7\" 1PR";
$resn_exch{" O'L 1PR"}="1O2\" 1PR"; $resn_exch{" O8B 1PR"}=" O8\" 1PR"; $resn_exch{" O9B 1PR"}=" O9\" 1PR";
$resn_exch{" O5' 1RB"}=" O5* 1RB"; $resn_exch{" C5' 1RB"}=" C5* 1RB"; $resn_exch{" C4' 1RB"}=" C4* 1RB";
$resn_exch{" O4' 1RB"}=" O4* 1RB"; $resn_exch{" C3' 1RB"}=" C3* 1RB"; $resn_exch{" O3' 1RB"}=" O3* 1RB";
$resn_exch{" C2' 1RB"}=" C2* 1RB"; $resn_exch{" O2' 1RB"}=" O2* 1RB"; $resn_exch{" C1' 1RB"}=" C1* 1RB";
$resn_exch{" O5' 1SC"}=" O5* 1SC"; $resn_exch{" C5' 1SC"}=" C5* 1SC"; $resn_exch{" C4' 1SC"}=" C4* 1SC";
$resn_exch{" O4' 1SC"}=" O4* 1SC"; $resn_exch{" C3' 1SC"}=" C3* 1SC"; $resn_exch{" O3' 1SC"}=" O3* 1SC";
$resn_exch{" C2' 1SC"}=" C2* 1SC"; $resn_exch{" O2' 1SC"}=" O2* 1SC"; $resn_exch{" C1' 1SC"}=" C1* 1SC";
$resn_exch{" N1' 1TP"}=" N1* 1TP"; $resn_exch{" C2' 1TP"}=" C2* 1TP"; $resn_exch{" N3' 1TP"}=" N3* 1TP";
$resn_exch{" C4' 1TP"}=" C4* 1TP"; $resn_exch{" N4' 1TP"}=" N4* 1TP"; $resn_exch{" C5' 1TP"}=" C5* 1TP";
$resn_exch{" C6' 1TP"}=" C6* 1TP"; $resn_exch{" C7' 1TP"}=" C7* 1TP"; $resn_exch{" N1' 223"}=" N1* 223";
$resn_exch{" C6' 223"}=" C6* 223"; $resn_exch{" C4' 223"}=" C4* 223"; $resn_exch{" C5' 223"}=" C5* 223";
$resn_exch{" O5' 223"}=" O5* 223"; $resn_exch{" C3' 223"}=" C3* 223"; $resn_exch{" O3' 223"}=" O3* 223";
$resn_exch{" C2' 223"}=" C2* 223"; $resn_exch{" CB' 232"}="C11' 232"; $resn_exch{" CC' 232"}="C12' 232";
$resn_exch{" CD' 232"}="C13' 232"; $resn_exch{" CE' 232"}="C14' 232"; $resn_exch{" CF' 232"}="C15' 232";
$resn_exch{" CG' 232"}="C16' 232"; $resn_exch{" CH' 232"}="C17' 232"; $resn_exch{" CJ' 232"}="C19' 232";
$resn_exch{" CA' 232"}="C10' 232"; $resn_exch{" OA' 232"}="O10' 232"; $resn_exch{" CI' 232"}="C18' 232";
$resn_exch{" CK' 232"}="C20' 232"; $resn_exch{" NL' 232"}="N21' 232"; $resn_exch{" CM' 232"}="C22' 232";
$resn_exch{" CN' 232"}="C23' 232"; $resn_exch{" CO' 232"}="C24' 232"; $resn_exch{" CP' 232"}="C25' 232";
$resn_exch{" CQ' 232"}="C26' 232"; $resn_exch{" CR' 232"}="C27' 232"; $resn_exch{"CL1  23D"}="CLAR 23D";
$resn_exch{" OXT 23F"}=" O   23F"; $resn_exch{" O   23F"}=" OXT 23F"; $resn_exch{" O5' 23T"}=" O5* 23T";
$resn_exch{" C5' 23T"}=" C5* 23T"; $resn_exch{" C4' 23T"}=" C4* 23T"; $resn_exch{" O4' 23T"}=" O4* 23T";
$resn_exch{" C1' 23T"}=" C1* 23T"; $resn_exch{" C2' 23T"}=" C2* 23T"; $resn_exch{" C3' 23T"}=" C3* 23T";
$resn_exch{" O5' 25A"}=" O5* 25A"; $resn_exch{" C5' 25A"}=" C5* 25A"; $resn_exch{" C4' 25A"}=" C4* 25A";
$resn_exch{" O4' 25A"}=" O4* 25A"; $resn_exch{" C1' 25A"}=" C1* 25A"; $resn_exch{" C2' 25A"}=" C2* 25A";
$resn_exch{" O2' 25A"}=" O2* 25A"; $resn_exch{" C3' 25A"}=" C3* 25A"; $resn_exch{" O3' 25A"}=" O3* 25A";
$resn_exch{" OBP 25A"}="O11P 25A"; $resn_exch{" OCP 25A"}="O12P 25A"; $resn_exch{" OF' 25A"}="O15* 25A";
$resn_exch{" CF' 25A"}="C15* 25A"; $resn_exch{" CE' 25A"}="C14* 25A"; $resn_exch{" OE' 25A"}="O14* 25A";
$resn_exch{" CB' 25A"}="C11* 25A"; $resn_exch{" CC' 25A"}="C12* 25A"; $resn_exch{" OC' 25A"}="O12* 25A";
$resn_exch{" CD' 25A"}="C13* 25A"; $resn_exch{" OD' 25A"}="O13* 25A"; $resn_exch{" OLP 25A"}="O21P 25A";
$resn_exch{" OMP 25A"}="O22P 25A"; $resn_exch{" OP' 25A"}="O25* 25A"; $resn_exch{" CP' 25A"}="C25* 25A";
$resn_exch{" CO' 25A"}="C24* 25A"; $resn_exch{" OO' 25A"}="O24* 25A"; $resn_exch{" CL' 25A"}="C21* 25A";
$resn_exch{" CM' 25A"}="C22* 25A"; $resn_exch{" OM' 25A"}="O22* 25A"; $resn_exch{" CN' 25A"}="C23* 25A";
$resn_exch{" ON' 25A"}="O23* 25A"; $resn_exch{" C1' 26A"}=" C1* 26A"; $resn_exch{" O4' 26A"}=" O4* 26A";
$resn_exch{" C4' 26A"}=" C4* 26A"; $resn_exch{" C5' 26A"}=" C5* 26A"; $resn_exch{" O5' 26A"}=" O5* 26A";
$resn_exch{" C2' 26A"}=" C2* 26A"; $resn_exch{" O2' 26A"}=" O2* 26A"; $resn_exch{" C3' 26A"}=" C3* 26A";
$resn_exch{" O3' 26A"}=" O3* 26A"; $resn_exch{" C9' 2AA"}=" C9* 2AA"; $resn_exch{" C8' 2AA"}=" C8* 2AA";
$resn_exch{" C7' 2AA"}=" C7* 2AA"; $resn_exch{" C6' 2AA"}=" C6* 2AA"; $resn_exch{" C5' 2AA"}=" C5* 2AA";
$resn_exch{" C4' 2AA"}=" C4* 2AA"; $resn_exch{" C3' 2AA"}=" C3* 2AA"; $resn_exch{" C2' 2AA"}=" C2* 2AA";
$resn_exch{" O3' 2AA"}=" O3* 2AA"; $resn_exch{" O4' 2AA"}=" O4* 2AA"; $resn_exch{" C1' 2AA"}=" C1* 2AA";
$resn_exch{" O2' 2AA"}=" O2* 2AA"; $resn_exch{" O1' 2AA"}=" O1* 2AA"; $resn_exch{" O5' 2AD"}=" O5* 2AD";
$resn_exch{" C5' 2AD"}=" C5* 2AD"; $resn_exch{" C4' 2AD"}=" C4* 2AD"; $resn_exch{" O4' 2AD"}=" O4* 2AD";
$resn_exch{" C3' 2AD"}=" C3* 2AD"; $resn_exch{" O3' 2AD"}=" O3* 2AD"; $resn_exch{" C2' 2AD"}=" C2* 2AD";
$resn_exch{" C1' 2AD"}=" C1* 2AD"; $resn_exch{" C5' 2AM"}=" C5* 2AM"; $resn_exch{" O5' 2AM"}=" O5* 2AM";
$resn_exch{" C4' 2AM"}=" C4* 2AM"; $resn_exch{" O4' 2AM"}=" O4* 2AM"; $resn_exch{" C3' 2AM"}=" C3* 2AM";
$resn_exch{" O3' 2AM"}=" O3* 2AM"; $resn_exch{" C2' 2AM"}=" C2* 2AM"; $resn_exch{" O2' 2AM"}=" O2* 2AM";
$resn_exch{" C1' 2AM"}=" C1* 2AM"; $resn_exch{" OP1 2AR"}=" O1P 2AR"; $resn_exch{" OP2 2AR"}=" O2P 2AR";
$resn_exch{" OP3 2AR"}=" O3P 2AR"; $resn_exch{" O5' 2AR"}=" O5* 2AR"; $resn_exch{" C5' 2AR"}=" C5* 2AR";
$resn_exch{" C4' 2AR"}=" C4* 2AR"; $resn_exch{" CX' 2AR"}=" CX* 2AR"; $resn_exch{" C1' 2AR"}=" C1* 2AR";
$resn_exch{" C2' 2AR"}=" C2* 2AR"; $resn_exch{" C3' 2AR"}=" C3* 2AR"; $resn_exch{" O3' 2AR"}=" O3* 2AR";
$resn_exch{" OP1 2AT"}=" O1P 2AT"; $resn_exch{" OP2 2AT"}=" O2P 2AT"; $resn_exch{" O5' 2AT"}=" O5* 2AT";
$resn_exch{" C5' 2AT"}=" C5* 2AT"; $resn_exch{" C4' 2AT"}=" C4* 2AT"; $resn_exch{" O4' 2AT"}=" O4* 2AT";
$resn_exch{" C1' 2AT"}=" C1* 2AT"; $resn_exch{" C2' 2AT"}=" C2* 2AT"; $resn_exch{" O2' 2AT"}=" O2* 2AT";
$resn_exch{" C3' 2AT"}=" C3* 2AT"; $resn_exch{" O3' 2AT"}=" O3* 2AT"; $resn_exch{" OP3 2AT"}=" O3P 2AT";
$resn_exch{" OP3 2AU"}=" O3P 2AU"; $resn_exch{" OP1 2AU"}=" O1P 2AU"; $resn_exch{" O5' 2AU"}=" O5* 2AU";
$resn_exch{" C5' 2AU"}=" C5* 2AU"; $resn_exch{" C4' 2AU"}=" C4* 2AU"; $resn_exch{" O4' 2AU"}=" O4* 2AU";
$resn_exch{" C3' 2AU"}=" C3* 2AU"; $resn_exch{" O3' 2AU"}=" O3* 2AU"; $resn_exch{" C2' 2AU"}=" C2* 2AU";
$resn_exch{" C1' 2AU"}=" C1* 2AU"; $resn_exch{" N2' 2AU"}=" N2* 2AU"; $resn_exch{"H1P1 2AU"}="1H1P 2AU";
$resn_exch{" O5' 2BA"}=" O5* 2BA"; $resn_exch{" C5' 2BA"}=" C5* 2BA"; $resn_exch{" C4' 2BA"}=" C4* 2BA";
$resn_exch{" O4' 2BA"}=" O4* 2BA"; $resn_exch{" C3' 2BA"}=" C3* 2BA"; $resn_exch{" O3' 2BA"}=" O3* 2BA";
$resn_exch{" C2' 2BA"}=" C2* 2BA"; $resn_exch{" O2' 2BA"}=" O2* 2BA"; $resn_exch{" C1' 2BA"}=" C1* 2BA";
$resn_exch{" P1  2BA"}="1P   2BA"; $resn_exch{"O1P1 2BA"}="1O1P 2BA"; $resn_exch{"O2P1 2BA"}="1O2P 2BA";
$resn_exch{"O5'1 2BA"}="1O5* 2BA"; $resn_exch{"C5'1 2BA"}="1C5* 2BA"; $resn_exch{"C4'1 2BA"}="1C4* 2BA";
$resn_exch{"O4'1 2BA"}="1O4* 2BA"; $resn_exch{"C3'1 2BA"}="1C3* 2BA"; $resn_exch{"O3'1 2BA"}="1O3* 2BA";
$resn_exch{"C2'1 2BA"}="1C2* 2BA"; $resn_exch{"O2'1 2BA"}="1O2* 2BA"; $resn_exch{"C1'1 2BA"}="1C1* 2BA";
$resn_exch{" N91 2BA"}="1N9  2BA"; $resn_exch{" C81 2BA"}="1C8  2BA"; $resn_exch{" N71 2BA"}="1N7  2BA";
$resn_exch{" C51 2BA"}="1C5  2BA"; $resn_exch{" C61 2BA"}="1C6  2BA"; $resn_exch{" N61 2BA"}="1N6  2BA";
$resn_exch{" N11 2BA"}="1N1  2BA"; $resn_exch{" C21 2BA"}="1C2  2BA"; $resn_exch{" N31 2BA"}="1N3  2BA";
$resn_exch{" C41 2BA"}="1C4  2BA"; $resn_exch{" OP1 2BD"}=" O1P 2BD"; $resn_exch{" OP2 2BD"}=" O2P 2BD";
$resn_exch{" OP3 2BD"}=" O3P 2BD"; $resn_exch{" O5' 2BD"}=" O5* 2BD"; $resn_exch{" C5' 2BD"}=" C5* 2BD";
$resn_exch{" C4' 2BD"}=" C4* 2BD"; $resn_exch{" O4' 2BD"}=" O4* 2BD"; $resn_exch{" C1' 2BD"}=" C1* 2BD";
$resn_exch{" C2' 2BD"}=" C2* 2BD"; $resn_exch{" C3' 2BD"}=" C3* 2BD"; $resn_exch{" O3' 2BD"}=" O3* 2BD";
$resn_exch{" OP3 2BT"}=" O3P 2BT"; $resn_exch{" OP2 2BT"}=" O2P 2BT"; $resn_exch{" O5' 2BT"}=" O5* 2BT";
$resn_exch{" C5' 2BT"}=" C5* 2BT"; $resn_exch{" C4' 2BT"}=" C4* 2BT"; $resn_exch{" O4' 2BT"}=" O4* 2BT";
$resn_exch{" C1' 2BT"}=" C1* 2BT"; $resn_exch{" C2' 2BT"}=" C2* 2BT"; $resn_exch{" O2' 2BT"}=" O2* 2BT";
$resn_exch{" CA' 2BT"}=" CA* 2BT"; $resn_exch{" CB' 2BT"}=" CB* 2BT"; $resn_exch{" CC' 2BT"}=" CC* 2BT";
$resn_exch{" C3' 2BT"}=" C3* 2BT"; $resn_exch{" O3' 2BT"}=" O3* 2BT"; $resn_exch{" CD' 2BT"}=" CD* 2BT";
$resn_exch{" OP1 2BT"}=" O1P 2BT"; $resn_exch{" OP1 2BU"}=" O1P 2BU"; $resn_exch{" OP2 2BU"}=" O2P 2BU";
$resn_exch{" OP3 2BU"}=" O3P 2BU"; $resn_exch{" O5' 2BU"}=" O5* 2BU"; $resn_exch{" C5' 2BU"}=" C5* 2BU";
$resn_exch{" C4' 2BU"}=" C4* 2BU"; $resn_exch{" O4' 2BU"}=" O4* 2BU"; $resn_exch{" C1' 2BU"}=" C1* 2BU";
$resn_exch{" C2' 2BU"}=" C2* 2BU"; $resn_exch{" C3' 2BU"}=" C3* 2BU"; $resn_exch{" O3' 2BU"}=" O3* 2BU";
$resn_exch{" OP1 2DA"}=" O1P 2DA"; $resn_exch{" OP2 2DA"}=" O2P 2DA"; $resn_exch{" OP3 2DA"}=" O3P 2DA";
$resn_exch{" O5' 2DA"}=" O5* 2DA"; $resn_exch{" C5' 2DA"}=" C5* 2DA"; $resn_exch{" C4' 2DA"}=" C4* 2DA";
$resn_exch{" O4' 2DA"}=" O4* 2DA"; $resn_exch{" C3' 2DA"}=" C3* 2DA"; $resn_exch{" C2' 2DA"}=" C2* 2DA";
$resn_exch{" C1' 2DA"}=" C1* 2DA"; $resn_exch{" O5' 2DF"}=" O5* 2DF"; $resn_exch{" C5' 2DF"}=" C5* 2DF";
$resn_exch{" C4' 2DF"}=" C4* 2DF"; $resn_exch{" O4' 2DF"}=" O4* 2DF"; $resn_exch{" C1' 2DF"}=" C1* 2DF";
$resn_exch{" C3' 2DF"}=" C3* 2DF"; $resn_exch{" C2' 2DF"}=" C2* 2DF"; $resn_exch{" O3' 2DF"}=" O3* 2DF";
$resn_exch{" C5' 2DI"}=" C5* 2DI"; $resn_exch{" O5' 2DI"}=" O5* 2DI"; $resn_exch{" C4' 2DI"}=" C4* 2DI";
$resn_exch{" O4' 2DI"}=" O4* 2DI"; $resn_exch{" C1' 2DI"}=" C1* 2DI"; $resn_exch{" C2' 2DI"}=" C2* 2DI";
$resn_exch{" C3' 2DI"}=" C3* 2DI"; $resn_exch{" OP1 2DM"}=" O1P 2DM"; $resn_exch{" OP2 2DM"}=" O2P 2DM";
$resn_exch{" O3' 2DM"}=" O3* 2DM"; $resn_exch{" OP3 2DM"}=" O3P 2DM"; $resn_exch{" OP1 2DT"}=" O1P 2DT";
$resn_exch{" OP2 2DT"}=" O2P 2DT"; $resn_exch{" OP3 2DT"}=" O3P 2DT"; $resn_exch{" O5' 2DT"}=" O5* 2DT";
$resn_exch{" C2' 2DT"}=" C2* 2DT"; $resn_exch{" C5' 2DT"}=" C5* 2DT"; $resn_exch{" C4' 2DT"}=" C4* 2DT";
$resn_exch{" O4' 2DT"}=" O4* 2DT"; $resn_exch{" C1' 2DT"}=" C1* 2DT"; $resn_exch{" C3' 2DT"}=" C3* 2DT";
$resn_exch{" OP3 2EG"}=" O3P 2EG"; $resn_exch{" OP1 2EG"}=" O1P 2EG"; $resn_exch{" OP2 2EG"}=" O2P 2EG";
$resn_exch{" O5' 2EG"}=" O5* 2EG"; $resn_exch{" C5' 2EG"}=" C5* 2EG"; $resn_exch{" C4' 2EG"}=" C4* 2EG";
$resn_exch{" O4' 2EG"}=" O4* 2EG"; $resn_exch{" C3' 2EG"}=" C3* 2EG"; $resn_exch{" O3' 2EG"}=" O3* 2EG";
$resn_exch{" C2' 2EG"}=" C2* 2EG"; $resn_exch{" C1' 2EG"}=" C1* 2EG"; $resn_exch{" NA  2FH"}=" N_A 2FH";
$resn_exch{" NB  2FH"}=" N_B 2FH"; $resn_exch{" NC  2FH"}=" N_C 2FH"; $resn_exch{" ND  2FH"}=" N_D 2FH";
$resn_exch{" O5' 2GP"}=" O5* 2GP"; $resn_exch{" C5' 2GP"}=" C5* 2GP"; $resn_exch{" C4' 2GP"}=" C4* 2GP";
$resn_exch{" O4' 2GP"}=" O4* 2GP"; $resn_exch{" C3' 2GP"}=" C3* 2GP"; $resn_exch{" O3' 2GP"}=" O3* 2GP";
$resn_exch{" C2' 2GP"}=" C2* 2GP"; $resn_exch{" O2' 2GP"}=" O2* 2GP"; $resn_exch{" C1' 2GP"}=" C1* 2GP";
$resn_exch{" OP1 2GT"}=" O1P 2GT"; $resn_exch{" OP2 2GT"}=" O2P 2GT"; $resn_exch{" O5' 2GT"}=" O5* 2GT";
$resn_exch{" C5' 2GT"}=" C5* 2GT"; $resn_exch{" C4' 2GT"}=" C4* 2GT"; $resn_exch{" O4' 2GT"}=" O4* 2GT";
$resn_exch{" C1' 2GT"}=" C1* 2GT"; $resn_exch{" C2' 2GT"}=" C2* 2GT"; $resn_exch{" O2' 2GT"}=" O2* 2GT";
$resn_exch{" C3' 2GT"}=" C3* 2GT"; $resn_exch{" O3' 2GT"}=" O3* 2GT"; $resn_exch{" OP3 2GT"}=" O3P 2GT";
$resn_exch{" OP1 2MA"}=" O1P 2MA"; $resn_exch{" OP2 2MA"}=" O2P 2MA"; $resn_exch{" OP3 2MA"}=" O3P 2MA";
$resn_exch{" O5' 2MA"}=" O5* 2MA"; $resn_exch{" C5' 2MA"}=" C5* 2MA"; $resn_exch{" C4' 2MA"}=" C4* 2MA";
$resn_exch{" O4' 2MA"}=" O4* 2MA"; $resn_exch{" C3' 2MA"}=" C3* 2MA"; $resn_exch{" O3' 2MA"}=" O3* 2MA";
$resn_exch{" C2' 2MA"}=" C2* 2MA"; $resn_exch{" O2' 2MA"}=" O2* 2MA"; $resn_exch{" C1' 2MA"}=" C1* 2MA";
$resn_exch{" N1A 2MC"}=" AN1 2MC"; $resn_exch{" C2A 2MC"}=" AC2 2MC"; $resn_exch{" N3A 2MC"}=" AN3 2MC";
$resn_exch{" C4A 2MC"}=" AC4 2MC"; $resn_exch{" C5A 2MC"}=" AC5 2MC"; $resn_exch{" C6A 2MC"}=" AC6 2MC";
$resn_exch{" N6A 2MC"}=" AN6 2MC"; $resn_exch{" N7A 2MC"}=" AN7 2MC"; $resn_exch{" C8A 2MC"}=" AC8 2MC";
$resn_exch{" N9A 2MC"}=" AN9 2MC"; $resn_exch{" C1B 2MC"}="AC1* 2MC"; $resn_exch{" C2B 2MC"}="AC2* 2MC";
$resn_exch{" O2B 2MC"}="AO2* 2MC"; $resn_exch{" C3B 2MC"}="AC3* 2MC"; $resn_exch{" O3B 2MC"}="AO3* 2MC";
$resn_exch{" P3B 2MC"}="AP3* 2MC"; $resn_exch{" O7A 2MC"}=" AO7 2MC"; $resn_exch{" O8A 2MC"}=" AO8 2MC";
$resn_exch{" O9A 2MC"}=" AO9 2MC"; $resn_exch{" C4B 2MC"}="AC4* 2MC"; $resn_exch{" O4B 2MC"}="AO4* 2MC";
$resn_exch{" C5B 2MC"}="AC5* 2MC"; $resn_exch{" O5B 2MC"}="AO5* 2MC"; $resn_exch{" P1A 2MC"}=" AP1 2MC";
$resn_exch{" O1A 2MC"}=" AO1 2MC"; $resn_exch{" O2A 2MC"}=" AO2 2MC"; $resn_exch{" O3A 2MC"}=" AO3 2MC";
$resn_exch{" P2A 2MC"}=" AP2 2MC"; $resn_exch{" O4A 2MC"}=" AO4 2MC"; $resn_exch{" O5A 2MC"}=" AO5 2MC";
$resn_exch{" O6A 2MC"}=" AO6 2MC"; $resn_exch{" CBP 2MC"}="PC11 2MC"; $resn_exch{" CCP 2MC"}="PC12 2MC";
$resn_exch{" CDP 2MC"}="PC13 2MC"; $resn_exch{" CEP 2MC"}="PC14 2MC"; $resn_exch{" CAP 2MC"}="PC10 2MC";
$resn_exch{" OAP 2MC"}="PO10 2MC"; $resn_exch{" C9P 2MC"}=" PC9 2MC"; $resn_exch{" O9P 2MC"}=" PO9 2MC";
$resn_exch{" N8P 2MC"}=" PN8 2MC"; $resn_exch{" C7P 2MC"}=" PC7 2MC"; $resn_exch{" C6P 2MC"}=" PC6 2MC";
$resn_exch{" C5P 2MC"}=" PC5 2MC"; $resn_exch{" O5P 2MC"}=" PO5 2MC"; $resn_exch{" N4P 2MC"}=" PN4 2MC";
$resn_exch{" C3P 2MC"}=" PC3 2MC"; $resn_exch{" C2P 2MC"}=" PC2 2MC"; $resn_exch{" S1P 2MC"}=" PS1 2MC";
$resn_exch{" O5' 2MD"}=" O5* 2MD"; $resn_exch{" C5' 2MD"}=" C5* 2MD"; $resn_exch{" C4' 2MD"}=" C4* 2MD";
$resn_exch{" O4' 2MD"}=" O4* 2MD"; $resn_exch{" C1' 2MD"}=" C1* 2MD"; $resn_exch{" C2' 2MD"}=" C2* 2MD";
$resn_exch{" O2' 2MD"}=" O2* 2MD"; $resn_exch{" C3' 2MD"}=" C3* 2MD"; $resn_exch{" O3' 2MD"}=" O3* 2MD";
$resn_exch{" CA' 2ME"}=" CA* 2ME"; $resn_exch{" CB' 2ME"}=" CB* 2ME"; $resn_exch{" OC' 2ME"}=" OC* 2ME";
$resn_exch{" CD' 2ME"}=" CD* 2ME"; $resn_exch{" OP1 2MG"}=" O1P 2MG"; $resn_exch{" OP2 2MG"}=" O2P 2MG";
$resn_exch{" OP3 2MG"}=" O3P 2MG"; $resn_exch{" O5' 2MG"}=" O5* 2MG"; $resn_exch{" C5' 2MG"}=" C5* 2MG";
$resn_exch{" C4' 2MG"}=" C4* 2MG"; $resn_exch{" O4' 2MG"}=" O4* 2MG"; $resn_exch{" C3' 2MG"}=" C3* 2MG";
$resn_exch{" O3' 2MG"}=" O3* 2MG"; $resn_exch{" C2' 2MG"}=" C2* 2MG"; $resn_exch{" O2' 2MG"}=" O2* 2MG";
$resn_exch{" C1' 2MG"}=" C1* 2MG"; $resn_exch{" C1' 2MU"}=" C1* 2MU"; $resn_exch{" C2' 2MU"}=" C2* 2MU";
$resn_exch{" O2' 2MU"}=" O2* 2MU"; $resn_exch{" C3' 2MU"}=" C3* 2MU"; $resn_exch{" C4' 2MU"}=" C4* 2MU";
$resn_exch{" O3' 2MU"}=" O3* 2MU"; $resn_exch{" O4' 2MU"}=" O4* 2MU"; $resn_exch{" C5' 2MU"}=" C5* 2MU";
$resn_exch{" O5' 2MU"}=" O5* 2MU"; $resn_exch{" C6' 2MU"}=" C6* 2MU"; $resn_exch{" OP1 2MU"}=" O1P 2MU";
$resn_exch{" OP2 2MU"}=" O2P 2MU"; $resn_exch{" OP3 2MU"}=" O3P 2MU"; $resn_exch{" C1' 2ND"}=" C1* 2ND";
$resn_exch{" O4' 2ND"}=" O4* 2ND"; $resn_exch{" C2' 2ND"}=" C2* 2ND"; $resn_exch{" C3' 2ND"}=" C3* 2ND";
$resn_exch{" O3' 2ND"}=" O3* 2ND"; $resn_exch{" C4' 2ND"}=" C4* 2ND"; $resn_exch{" C5' 2ND"}=" C5* 2ND";
$resn_exch{" O5' 2ND"}=" O5* 2ND"; $resn_exch{" OP3 2NT"}=" O3P 2NT"; $resn_exch{" OP2 2NT"}=" O2P 2NT";
$resn_exch{" O5' 2NT"}=" O5* 2NT"; $resn_exch{" C5' 2NT"}=" C5* 2NT"; $resn_exch{" C4' 2NT"}=" C4* 2NT";
$resn_exch{" O4' 2NT"}=" O4* 2NT"; $resn_exch{" C1' 2NT"}=" C1* 2NT"; $resn_exch{" C2' 2NT"}=" C2* 2NT";
$resn_exch{" O2' 2NT"}=" O2* 2NT"; $resn_exch{" C3' 2NT"}=" C3* 2NT"; $resn_exch{" O3' 2NT"}=" O3* 2NT";
$resn_exch{" OP1 2NT"}=" O1P 2NT"; $resn_exch{" OP1 2OT"}=" O1P 2OT"; $resn_exch{" OP2 2OT"}=" O2P 2OT";
$resn_exch{" O5' 2OT"}=" O5* 2OT"; $resn_exch{" C5' 2OT"}=" C5* 2OT"; $resn_exch{" C4' 2OT"}=" C4* 2OT";
$resn_exch{" O4' 2OT"}=" O4* 2OT"; $resn_exch{" C1' 2OT"}=" C1* 2OT"; $resn_exch{" C2' 2OT"}=" C2* 2OT";
$resn_exch{" O2' 2OT"}=" O2* 2OT"; $resn_exch{" C3' 2OT"}=" C3* 2OT"; $resn_exch{" O3' 2OT"}=" O3* 2OT";
$resn_exch{" OP3 2OT"}=" O3P 2OT"; $resn_exch{" OP3 2PR"}=" O3P 2PR"; $resn_exch{" OP1 2PR"}=" O1P 2PR";
$resn_exch{" OP2 2PR"}=" O2P 2PR"; $resn_exch{" O5' 2PR"}=" O5* 2PR"; $resn_exch{" C5' 2PR"}=" C5* 2PR";
$resn_exch{" C4' 2PR"}=" C4* 2PR"; $resn_exch{" O4' 2PR"}=" O4* 2PR"; $resn_exch{" C3' 2PR"}=" C3* 2PR";
$resn_exch{" O3' 2PR"}=" O3* 2PR"; $resn_exch{" C2' 2PR"}=" C2* 2PR"; $resn_exch{" C1' 2PR"}=" C1* 2PR";
$resn_exch{" OP3 2ST"}=" O3P 2ST"; $resn_exch{" OP1 2ST"}=" O1P 2ST"; $resn_exch{" OP2 2ST"}=" O2P 2ST";
$resn_exch{" O5' 2ST"}=" O5* 2ST"; $resn_exch{" C2' 2ST"}=" C2* 2ST"; $resn_exch{" C5' 2ST"}=" C5* 2ST";
$resn_exch{" C4' 2ST"}=" C4* 2ST"; $resn_exch{" O4' 2ST"}=" O4* 2ST"; $resn_exch{" C1' 2ST"}=" C1* 2ST";
$resn_exch{" C3' 2ST"}=" C3* 2ST"; $resn_exch{" CA' 2ST"}=" CA* 2ST"; $resn_exch{" O3' 2ST"}=" O3* 2ST";
$resn_exch{" N1' 2TP"}=" N1* 2TP"; $resn_exch{" C2' 2TP"}=" C2* 2TP"; $resn_exch{" N3' 2TP"}=" N3* 2TP";
$resn_exch{" C4' 2TP"}=" C4* 2TP"; $resn_exch{" N4' 2TP"}=" N4* 2TP"; $resn_exch{" C5' 2TP"}=" C5* 2TP";
$resn_exch{" C6' 2TP"}=" C6* 2TP"; $resn_exch{" C2' 2VA"}=" C2* 2VA"; $resn_exch{" C3' 2VA"}=" C3* 2VA";
$resn_exch{" O3' 2VA"}=" O3* 2VA"; $resn_exch{" C4' 2VA"}=" C4* 2VA"; $resn_exch{" C5' 2VA"}=" C5* 2VA";
$resn_exch{" O5' 2VA"}=" O5* 2VA"; $resn_exch{" O4' 2VA"}=" O4* 2VA"; $resn_exch{" C1' 2VA"}=" C1* 2VA";
$resn_exch{" CVX 312"}="C31' 312"; $resn_exch{" CWX 312"}="C32' 312"; $resn_exch{" OXX 312"}="O33' 312";
$resn_exch{" OYX 312"}="O34' 312"; $resn_exch{" C6X 312"}="C42' 312"; $resn_exch{" C7X 312"}="C43' 312";
$resn_exch{" O8X 312"}="O44' 312"; $resn_exch{" O9X 312"}="O45' 312"; $resn_exch{" C1B 312"}="C1'' 312";
$resn_exch{" C2B 312"}="C2'' 312"; $resn_exch{" C3B 312"}="C3'' 312"; $resn_exch{" C4B 312"}="C4'' 312";
$resn_exch{" C5B 312"}="C5'' 312"; $resn_exch{" C6B 312"}="C6'' 312"; $resn_exch{" O2B 312"}="O2'' 312";
$resn_exch{" CV' 334"}="C31' 334"; $resn_exch{"H2'1 34B"}="1H2' 34B"; $resn_exch{"H2'2 34B"}="2H2' 34B";
$resn_exch{"H4'1 34B"}="1H4' 34B"; $resn_exch{"H4'2 34B"}="2H4' 34B"; $resn_exch{" C1' 357"}=" C1* 357";
$resn_exch{" C7' 357"}=" C7* 357"; $resn_exch{" N1' 357"}=" N1\" 357"; $resn_exch{" C6' 357"}=" C6\" 357";
$resn_exch{" C2' 357"}=" C2\" 357"; $resn_exch{" C3' 357"}=" C3\" 357"; $resn_exch{" O4' 357"}=" O4\" 357";
$resn_exch{" C5' 357"}=" C5\" 357"; $resn_exch{" C6B 357"}=" C6* 357"; $resn_exch{" C5B 357"}=" C5* 357";
$resn_exch{" C4' 357"}=" C4* 357"; $resn_exch{" C3B 357"}=" C3* 357"; $resn_exch{" C8' 357"}=" C8* 357";
$resn_exch{" C2B 357"}=" C2* 357"; $resn_exch{" O5' 35G"}=" O5* 35G"; $resn_exch{" C5' 35G"}=" C5* 35G";
$resn_exch{" C4' 35G"}=" C4* 35G"; $resn_exch{" O4' 35G"}=" O4* 35G"; $resn_exch{" C3' 35G"}=" C3* 35G";
$resn_exch{" O3' 35G"}=" O3* 35G"; $resn_exch{" C2' 35G"}=" C2* 35G"; $resn_exch{" O2' 35G"}=" O2* 35G";
$resn_exch{" C1' 35G"}=" C1* 35G"; $resn_exch{" PA  3AA"}=" AP  3AA"; $resn_exch{" O1A 3AA"}=" AO1 3AA";
$resn_exch{" O2A 3AA"}=" AO2 3AA"; $resn_exch{" O5B 3AA"}="AO5* 3AA"; $resn_exch{" C5B 3AA"}="AC5* 3AA";
$resn_exch{" C4B 3AA"}="AC4* 3AA"; $resn_exch{" O4B 3AA"}="AO4* 3AA"; $resn_exch{" C1B 3AA"}="AC1* 3AA";
$resn_exch{" N9A 3AA"}=" AN9 3AA"; $resn_exch{" C4A 3AA"}=" AC4 3AA"; $resn_exch{" N3A 3AA"}=" AN3 3AA";
$resn_exch{" C2A 3AA"}=" AC2 3AA"; $resn_exch{" N1A 3AA"}=" AN1 3AA"; $resn_exch{" C6A 3AA"}=" AC6 3AA";
$resn_exch{" N6A 3AA"}=" AN6 3AA"; $resn_exch{" C5A 3AA"}=" AC5 3AA"; $resn_exch{" N7A 3AA"}=" AN7 3AA";
$resn_exch{" C8A 3AA"}=" AC8 3AA"; $resn_exch{" C2B 3AA"}="AC2* 3AA"; $resn_exch{" O2B 3AA"}="AO2* 3AA";
$resn_exch{" P2B 3AA"}="AP2* 3AA"; $resn_exch{" O1X 3AA"}="AOP1 3AA"; $resn_exch{" O2X 3AA"}="AOP2 3AA";
$resn_exch{" O3X 3AA"}="AOP3 3AA"; $resn_exch{" C3B 3AA"}="AC3* 3AA"; $resn_exch{" O3B 3AA"}="AO3* 3AA";
$resn_exch{" PN  3AA"}=" NP  3AA"; $resn_exch{" O1N 3AA"}=" NO1 3AA"; $resn_exch{" O2N 3AA"}=" NO2 3AA";
$resn_exch{" O5D 3AA"}="NO5* 3AA"; $resn_exch{" C5D 3AA"}="NC5* 3AA"; $resn_exch{" C4D 3AA"}="NC4* 3AA";
$resn_exch{" O4D 3AA"}="NO4* 3AA"; $resn_exch{" C3D 3AA"}="NC3* 3AA"; $resn_exch{" O3D 3AA"}="NO3* 3AA";
$resn_exch{" C2D 3AA"}="NC2* 3AA"; $resn_exch{" O2D 3AA"}="NO2* 3AA"; $resn_exch{" C1D 3AA"}="NC1* 3AA";
$resn_exch{" N1N 3AA"}=" NN1 3AA"; $resn_exch{" C2N 3AA"}=" NC2 3AA"; $resn_exch{" C4N 3AA"}=" NC4 3AA";
$resn_exch{" C5N 3AA"}=" NC5 3AA"; $resn_exch{" C6N 3AA"}=" NC6 3AA"; $resn_exch{" C3N 3AA"}=" NC3 3AA";
$resn_exch{" N7N 3AA"}=" NN7 3AA"; $resn_exch{" O5' 3AD"}=" O5* 3AD"; $resn_exch{" C5' 3AD"}=" C5* 3AD";
$resn_exch{" C4' 3AD"}=" C4* 3AD"; $resn_exch{" O4' 3AD"}=" O4* 3AD"; $resn_exch{" C3' 3AD"}=" C3* 3AD";
$resn_exch{" C2' 3AD"}=" C2* 3AD"; $resn_exch{" O2' 3AD"}=" O2* 3AD"; $resn_exch{" C1' 3AD"}=" C1* 3AD";
$resn_exch{" O2' 3AM"}=" O2* 3AM"; $resn_exch{" C2' 3AM"}=" C2* 3AM"; $resn_exch{" C3' 3AM"}=" C3* 3AM";
$resn_exch{" O3' 3AM"}=" O3* 3AM"; $resn_exch{" C1' 3AM"}=" C1* 3AM"; $resn_exch{" O4' 3AM"}=" O4* 3AM";
$resn_exch{" C4' 3AM"}=" C4* 3AM"; $resn_exch{" C5' 3AM"}=" C5* 3AM"; $resn_exch{" O5' 3AM"}=" O5* 3AM";
$resn_exch{" O5' 3AN"}=" O5* 3AN"; $resn_exch{" C5' 3AN"}=" C5* 3AN"; $resn_exch{" C4' 3AN"}=" C4* 3AN";
$resn_exch{" O4' 3AN"}=" O4* 3AN"; $resn_exch{" C3' 3AN"}=" C3* 3AN"; $resn_exch{" N3' 3AN"}=" N3* 3AN";
$resn_exch{" C2' 3AN"}=" C2* 3AN"; $resn_exch{" O2' 3AN"}=" O2* 3AN"; $resn_exch{" C1' 3AN"}=" C1* 3AN";
$resn_exch{" O5' 3AT"}=" O5* 3AT"; $resn_exch{" C5' 3AT"}=" C5* 3AT"; $resn_exch{" C4' 3AT"}=" C4* 3AT";
$resn_exch{" O4' 3AT"}=" O4* 3AT"; $resn_exch{" C3' 3AT"}=" C3* 3AT"; $resn_exch{" C2' 3AT"}=" C2* 3AT";
$resn_exch{" O2' 3AT"}=" O2* 3AT"; $resn_exch{" C1' 3AT"}=" C1* 3AT"; $resn_exch{" O5' 3DA"}=" O5* 3DA";
$resn_exch{" C5' 3DA"}=" C5* 3DA"; $resn_exch{" C4' 3DA"}=" C4* 3DA"; $resn_exch{" O4' 3DA"}=" O4* 3DA";
$resn_exch{" C1' 3DA"}=" C1* 3DA"; $resn_exch{" C2' 3DA"}=" C2* 3DA"; $resn_exch{" O2' 3DA"}=" O2* 3DA";
$resn_exch{" C3' 3DA"}=" C3* 3DA"; $resn_exch{" O5' 3DR"}=" O5* 3DR"; $resn_exch{" OP1 3DR"}=" O1P 3DR";
$resn_exch{" OP2 3DR"}=" O2P 3DR"; $resn_exch{" OP3 3DR"}=" O3P 3DR"; $resn_exch{" C2' 3DR"}=" C2* 3DR";
$resn_exch{" C5' 3DR"}=" C5* 3DR"; $resn_exch{" C4' 3DR"}=" C4* 3DR"; $resn_exch{" O4' 3DR"}=" O4* 3DR";
$resn_exch{" C1' 3DR"}=" C1* 3DR"; $resn_exch{" C3' 3DR"}=" C3* 3DR"; $resn_exch{" O3' 3DR"}=" O3* 3DR";
$resn_exch{" O5' 3GP"}=" O5* 3GP"; $resn_exch{" C5' 3GP"}=" C5* 3GP"; $resn_exch{" C4' 3GP"}=" C4* 3GP";
$resn_exch{" O4' 3GP"}=" O4* 3GP"; $resn_exch{" C3' 3GP"}=" C3* 3GP"; $resn_exch{" O3' 3GP"}=" O3* 3GP";
$resn_exch{" C2' 3GP"}=" C2* 3GP"; $resn_exch{" O2' 3GP"}=" O2* 3GP"; $resn_exch{" C1' 3GP"}=" C1* 3GP";
$resn_exch{" N1A 3HC"}=" AN1 3HC"; $resn_exch{" C2A 3HC"}=" AC2 3HC"; $resn_exch{" N3A 3HC"}=" AN3 3HC";
$resn_exch{" C4A 3HC"}=" AC4 3HC"; $resn_exch{" C5A 3HC"}=" AC5 3HC"; $resn_exch{" C6A 3HC"}=" AC6 3HC";
$resn_exch{" N6A 3HC"}=" AN6 3HC"; $resn_exch{" N7A 3HC"}=" AN7 3HC"; $resn_exch{" C8A 3HC"}=" AC8 3HC";
$resn_exch{" N9A 3HC"}=" AN9 3HC"; $resn_exch{" C1B 3HC"}="AC1* 3HC"; $resn_exch{" C2B 3HC"}="AC2* 3HC";
$resn_exch{" O2B 3HC"}="AO2* 3HC"; $resn_exch{" C3B 3HC"}="AC3* 3HC"; $resn_exch{" O3B 3HC"}="AO3* 3HC";
$resn_exch{" P3B 3HC"}="AP3* 3HC"; $resn_exch{" O7A 3HC"}=" AO7 3HC"; $resn_exch{" O8A 3HC"}=" AO8 3HC";
$resn_exch{" O9A 3HC"}=" AO9 3HC"; $resn_exch{" C4B 3HC"}="AC4* 3HC"; $resn_exch{" O4B 3HC"}="AO4* 3HC";
$resn_exch{" C5B 3HC"}="AC5* 3HC"; $resn_exch{" O5B 3HC"}="AO5* 3HC"; $resn_exch{" P1A 3HC"}=" AP1 3HC";
$resn_exch{" O1A 3HC"}=" AO1 3HC"; $resn_exch{" O2A 3HC"}=" AO2 3HC"; $resn_exch{" O3A 3HC"}=" AO3 3HC";
$resn_exch{" P2A 3HC"}=" AP2 3HC"; $resn_exch{" O4A 3HC"}=" AO4 3HC"; $resn_exch{" O5A 3HC"}=" AO5 3HC";
$resn_exch{" O6A 3HC"}=" AO6 3HC"; $resn_exch{" CBP 3HC"}="PC11 3HC"; $resn_exch{" CCP 3HC"}="PC12 3HC";
$resn_exch{" CDP 3HC"}="PC13 3HC"; $resn_exch{" CEP 3HC"}="PC14 3HC"; $resn_exch{" CAP 3HC"}="PC10 3HC";
$resn_exch{" OAP 3HC"}="PO10 3HC"; $resn_exch{" C9P 3HC"}=" PC9 3HC"; $resn_exch{" O9P 3HC"}=" PO9 3HC";
$resn_exch{" N8P 3HC"}=" PN8 3HC"; $resn_exch{" C7P 3HC"}=" PC7 3HC"; $resn_exch{" C6P 3HC"}=" PC6 3HC";
$resn_exch{" C5P 3HC"}=" PC5 3HC"; $resn_exch{" O5P 3HC"}=" PO5 3HC"; $resn_exch{" N4P 3HC"}=" PN4 3HC";
$resn_exch{" C3P 3HC"}=" PC3 3HC"; $resn_exch{" C2P 3HC"}=" PC2 3HC"; $resn_exch{" S1P 3HC"}=" PS1 3HC";
$resn_exch{" OP1 3ME"}=" O1P 3ME"; $resn_exch{" OP2 3ME"}=" O2P 3ME"; $resn_exch{" O5' 3PO"}=" O5* 3PO";
$resn_exch{" C1B 44D"}=" C1* 44D"; $resn_exch{" C2B 44D"}=" C2* 44D"; $resn_exch{" C3B 44D"}=" C3* 44D";
$resn_exch{" C4B 44D"}=" C4* 44D"; $resn_exch{" C5B 44D"}=" C5* 44D"; $resn_exch{" C6B 44D"}=" C6* 44D";
$resn_exch{" O3' 44D"}=" O3* 44D"; $resn_exch{" O4B 44D"}=" O4* 44D"; $resn_exch{" O5B 44D"}=" O5* 44D";
$resn_exch{" OP3 4AC"}=" O3P 4AC"; $resn_exch{" OP1 4AC"}=" O1P 4AC"; $resn_exch{" OP2 4AC"}=" O2P 4AC";
$resn_exch{" O5' 4AC"}=" O5* 4AC"; $resn_exch{" C5' 4AC"}=" C5* 4AC"; $resn_exch{" C4' 4AC"}=" C4* 4AC";
$resn_exch{" O4' 4AC"}=" O4* 4AC"; $resn_exch{" C3' 4AC"}=" C3* 4AC"; $resn_exch{" O3' 4AC"}=" O3* 4AC";
$resn_exch{" C2' 4AC"}=" C2* 4AC"; $resn_exch{" O2' 4AC"}=" O2* 4AC"; $resn_exch{" C1' 4AC"}=" C1* 4AC";
$resn_exch{" O5' 4AD"}=" O5* 4AD"; $resn_exch{" C5' 4AD"}=" C5* 4AD"; $resn_exch{" C4' 4AD"}=" C4* 4AD";
$resn_exch{" C3' 4AD"}=" C3* 4AD"; $resn_exch{" O3' 4AD"}=" O3* 4AD"; $resn_exch{" C2' 4AD"}=" C2* 4AD";
$resn_exch{" O2' 4AD"}=" O2* 4AD"; $resn_exch{" C1' 4AD"}=" C1* 4AD"; $resn_exch{" O4' 4AD"}=" O4* 4AD";
$resn_exch{" OXT 4BF"}=" OT1 4BF"; $resn_exch{" N1A 4CA"}=" AN1 4CA"; $resn_exch{" C2A 4CA"}=" AC2 4CA";
$resn_exch{" N3A 4CA"}=" AN3 4CA"; $resn_exch{" C4A 4CA"}=" AC4 4CA"; $resn_exch{" C5A 4CA"}=" AC5 4CA";
$resn_exch{" C6A 4CA"}=" AC6 4CA"; $resn_exch{" N6A 4CA"}=" AN6 4CA"; $resn_exch{" N7A 4CA"}=" AN7 4CA";
$resn_exch{" C8A 4CA"}=" AC8 4CA"; $resn_exch{" N9A 4CA"}=" AN9 4CA"; $resn_exch{" C1D 4CA"}="AC1* 4CA";
$resn_exch{" C2D 4CA"}="AC2* 4CA"; $resn_exch{" O2D 4CA"}="AO2* 4CA"; $resn_exch{" C3D 4CA"}="AC3* 4CA";
$resn_exch{" O3D 4CA"}="AO3* 4CA"; $resn_exch{" P3D 4CA"}="AP3* 4CA"; $resn_exch{" O7A 4CA"}=" AO7 4CA";
$resn_exch{" O8A 4CA"}=" AO8 4CA"; $resn_exch{" O9A 4CA"}=" AO9 4CA"; $resn_exch{" C4D 4CA"}="AC4* 4CA";
$resn_exch{" O4D 4CA"}="AO4* 4CA"; $resn_exch{" C5D 4CA"}="AC5* 4CA"; $resn_exch{" O5D 4CA"}="AO5* 4CA";
$resn_exch{" P1A 4CA"}=" AP1 4CA"; $resn_exch{" O1A 4CA"}=" AO1 4CA"; $resn_exch{" O2A 4CA"}=" AO2 4CA";
$resn_exch{" O3A 4CA"}=" AO3 4CA"; $resn_exch{" P2A 4CA"}=" AP2 4CA"; $resn_exch{" O4A 4CA"}=" AO4 4CA";
$resn_exch{" O5A 4CA"}=" AO5 4CA"; $resn_exch{" O6A 4CA"}=" AO6 4CA"; $resn_exch{" CBP 4CA"}="PC11 4CA";
$resn_exch{" CCP 4CA"}="PC12 4CA"; $resn_exch{" CDP 4CA"}="PC13 4CA"; $resn_exch{" CEP 4CA"}="PC14 4CA";
$resn_exch{" CAP 4CA"}="PC10 4CA"; $resn_exch{" OAP 4CA"}="PO10 4CA"; $resn_exch{" C9P 4CA"}=" PC9 4CA";
$resn_exch{" O9P 4CA"}=" PO9 4CA"; $resn_exch{" N8P 4CA"}=" PN8 4CA"; $resn_exch{" C7P 4CA"}=" PC7 4CA";
$resn_exch{" C6P 4CA"}=" PC6 4CA"; $resn_exch{" C5P 4CA"}=" PC5 4CA"; $resn_exch{" O5P 4CA"}=" PO5 4CA";
$resn_exch{" N4P 4CA"}=" PN4 4CA"; $resn_exch{" C3P 4CA"}=" PC3 4CA"; $resn_exch{" C2P 4CA"}=" PC2 4CA";
$resn_exch{" S1P 4CA"}=" PS1 4CA"; $resn_exch{" C1B 4CA"}=" BC1 4CA"; $resn_exch{" C2B 4CA"}=" BC2 4CA";
$resn_exch{" C3B 4CA"}=" BC3 4CA"; $resn_exch{" C4B 4CA"}=" BC4 4CA"; $resn_exch{" C5B 4CA"}=" BC5 4CA";
$resn_exch{" O2B 4CA"}=" BO2 4CA"; $resn_exch{" C6B 4CA"}=" BC6 4CA"; $resn_exch{" C7B 4CA"}=" BC7 4CA";
$resn_exch{" N1A 4CO"}=" AN1 4CO"; $resn_exch{" C2A 4CO"}=" AC2 4CO"; $resn_exch{" N3A 4CO"}=" AN3 4CO";
$resn_exch{" C4A 4CO"}=" AC4 4CO"; $resn_exch{" C5A 4CO"}=" AC5 4CO"; $resn_exch{" C6A 4CO"}=" AC6 4CO";
$resn_exch{" N6A 4CO"}=" AN6 4CO"; $resn_exch{" N7A 4CO"}=" AN7 4CO"; $resn_exch{" C8A 4CO"}=" AC8 4CO";
$resn_exch{" N9A 4CO"}=" AN9 4CO"; $resn_exch{" C1D 4CO"}="AC1* 4CO"; $resn_exch{" C2D 4CO"}="AC2* 4CO";
$resn_exch{" O2D 4CO"}="AO2* 4CO"; $resn_exch{" C3D 4CO"}="AC3* 4CO"; $resn_exch{" O3D 4CO"}="AO3* 4CO";
$resn_exch{" P3D 4CO"}="AP3* 4CO"; $resn_exch{" O7A 4CO"}=" AO7 4CO"; $resn_exch{" O8A 4CO"}=" AO8 4CO";
$resn_exch{" O9A 4CO"}=" AO9 4CO"; $resn_exch{" C4D 4CO"}="AC4* 4CO"; $resn_exch{" O4D 4CO"}="AO4* 4CO";
$resn_exch{" C5D 4CO"}="AC5* 4CO"; $resn_exch{" O5D 4CO"}="AO5* 4CO"; $resn_exch{" P1A 4CO"}=" AP1 4CO";
$resn_exch{" O1A 4CO"}=" AO1 4CO"; $resn_exch{" O2A 4CO"}=" AO2 4CO"; $resn_exch{" O3A 4CO"}=" AO3 4CO";
$resn_exch{" P2A 4CO"}=" AP2 4CO"; $resn_exch{" O4A 4CO"}=" AO4 4CO"; $resn_exch{" O5A 4CO"}=" AO5 4CO";
$resn_exch{" O6A 4CO"}=" AO6 4CO"; $resn_exch{" CBP 4CO"}="PC11 4CO"; $resn_exch{" CCP 4CO"}="PC12 4CO";
$resn_exch{" CDP 4CO"}="PC13 4CO"; $resn_exch{" CEP 4CO"}="PC14 4CO"; $resn_exch{" CAP 4CO"}="PC10 4CO";
$resn_exch{" OAP 4CO"}="PO10 4CO"; $resn_exch{" C9P 4CO"}=" PC9 4CO"; $resn_exch{" O9P 4CO"}=" PO9 4CO";
$resn_exch{" N8P 4CO"}=" PN8 4CO"; $resn_exch{" C7P 4CO"}=" PC7 4CO"; $resn_exch{" C6P 4CO"}=" PC6 4CO";
$resn_exch{" C5P 4CO"}=" PC5 4CO"; $resn_exch{" O5P 4CO"}=" PO5 4CO"; $resn_exch{" N4P 4CO"}=" PN4 4CO";
$resn_exch{" C3P 4CO"}=" PC3 4CO"; $resn_exch{" C2P 4CO"}=" PC2 4CO"; $resn_exch{" S1P 4CO"}=" PS1 4CO";
$resn_exch{" O1B 4CO"}=" BO1 4CO"; $resn_exch{" C1B 4CO"}=" BC1 4CO"; $resn_exch{" C2B 4CO"}=" BC2 4CO";
$resn_exch{" C3B 4CO"}=" BC3 4CO"; $resn_exch{" C4B 4CO"}=" BC4 4CO"; $resn_exch{" C5B 4CO"}=" BC5 4CO";
$resn_exch{" O2B 4CO"}=" BO2 4CO"; $resn_exch{" C6B 4CO"}=" BC6 4CO"; $resn_exch{" C7B 4CO"}=" BC7 4CO";
$resn_exch{" CB  4CO"}=" BC  4CO"; $resn_exch{" OXT 4IN"}=" O1  4IN"; $resn_exch{" OPF 4IP"}="1OP5 4IP";
$resn_exch{" OPG 4IP"}="2OP5 4IP"; $resn_exch{" OPH 4IP"}="3OP5 4IP"; $resn_exch{" O5' 4MF"}=" O5* 4MF";
$resn_exch{" C5' 4MF"}=" C5* 4MF"; $resn_exch{" C4' 4MF"}=" C4* 4MF"; $resn_exch{" C3' 4MF"}=" C3* 4MF";
$resn_exch{" O3' 4MF"}=" O3* 4MF"; $resn_exch{" C2' 4MF"}=" C2* 4MF"; $resn_exch{" C1' 4MF"}=" C1* 4MF";
$resn_exch{" O4' 4MF"}=" O4* 4MF"; $resn_exch{" OP1 4OC"}=" O1P 4OC"; $resn_exch{" OP2 4OC"}=" O2P 4OC";
$resn_exch{" O5' 4OC"}=" O5* 4OC"; $resn_exch{" C5' 4OC"}=" C5* 4OC"; $resn_exch{" C4' 4OC"}=" C4* 4OC";
$resn_exch{" O4' 4OC"}=" O4* 4OC"; $resn_exch{" C3' 4OC"}=" C3* 4OC"; $resn_exch{" C2' 4OC"}=" C2* 4OC";
$resn_exch{" O2' 4OC"}=" O2* 4OC"; $resn_exch{" C1' 4OC"}=" C1* 4OC"; $resn_exch{" O3' 4OC"}=" O3* 4OC";
$resn_exch{" OP3 4OC"}=" O3P 4OC"; $resn_exch{" OP1 4PC"}=" O1P 4PC"; $resn_exch{" OP2 4PC"}=" O2P 4PC";
$resn_exch{" O5' 4PC"}=" O5* 4PC"; $resn_exch{" C5' 4PC"}=" C5* 4PC"; $resn_exch{" C4' 4PC"}=" C4* 4PC";
$resn_exch{" O4' 4PC"}=" O4* 4PC"; $resn_exch{" C3' 4PC"}=" C3* 4PC"; $resn_exch{" O3' 4PC"}=" O3* 4PC";
$resn_exch{" C2' 4PC"}=" C2* 4PC"; $resn_exch{" C1' 4PC"}=" C1* 4PC"; $resn_exch{" OP3 4PC"}=" O3P 4PC";
$resn_exch{" OP1 4PD"}=" O1P 4PD"; $resn_exch{" OP2 4PD"}=" O2P 4PD"; $resn_exch{" O5' 4PD"}=" O5* 4PD";
$resn_exch{" C5' 4PD"}=" C5* 4PD"; $resn_exch{" C4' 4PD"}=" C4* 4PD"; $resn_exch{" O4' 4PD"}=" O4* 4PD";
$resn_exch{" C3' 4PD"}=" C3* 4PD"; $resn_exch{" O3' 4PD"}=" O3* 4PD"; $resn_exch{" C2' 4PD"}=" C2* 4PD";
$resn_exch{" C1' 4PD"}=" C1* 4PD"; $resn_exch{" OP3 4PD"}=" O3P 4PD"; $resn_exch{" OP1 4PE"}=" O1P 4PE";
$resn_exch{" OP2 4PE"}=" O2P 4PE"; $resn_exch{" O5' 4PE"}=" O5* 4PE"; $resn_exch{" C5' 4PE"}=" C5* 4PE";
$resn_exch{" C4' 4PE"}=" C4* 4PE"; $resn_exch{" O4' 4PE"}=" O4* 4PE"; $resn_exch{" C3' 4PE"}=" C3* 4PE";
$resn_exch{" O3' 4PE"}=" O3* 4PE"; $resn_exch{" C2' 4PE"}=" C2* 4PE"; $resn_exch{" C1' 4PE"}=" C1* 4PE";
$resn_exch{" OP3 4PE"}=" O3P 4PE"; $resn_exch{" OP1 4SC"}=" O1P 4SC"; $resn_exch{" OP2 4SC"}=" O2P 4SC";
$resn_exch{" O5' 4SC"}=" O5* 4SC"; $resn_exch{" C5' 4SC"}=" C5* 4SC"; $resn_exch{" C4' 4SC"}=" C4* 4SC";
$resn_exch{" S4' 4SC"}=" S4* 4SC"; $resn_exch{" C3' 4SC"}=" C3* 4SC"; $resn_exch{" O3' 4SC"}=" O3* 4SC";
$resn_exch{" C2' 4SC"}=" C2* 4SC"; $resn_exch{" C1' 4SC"}=" C1* 4SC"; $resn_exch{" C1' 4SU"}=" C1* 4SU";
$resn_exch{" C2' 4SU"}=" C2* 4SU"; $resn_exch{" O2' 4SU"}=" O2* 4SU"; $resn_exch{" C3' 4SU"}=" C3* 4SU";
$resn_exch{" C4' 4SU"}=" C4* 4SU"; $resn_exch{" O3' 4SU"}=" O3* 4SU"; $resn_exch{" O4' 4SU"}=" O4* 4SU";
$resn_exch{" C5' 4SU"}=" C5* 4SU"; $resn_exch{" O5' 4SU"}=" O5* 4SU"; $resn_exch{" OP1 4SU"}=" O1P 4SU";
$resn_exch{" OP2 4SU"}=" O2P 4SU"; $resn_exch{" OP3 4SU"}=" O3P 4SU"; $resn_exch{" C1B 4TA"}=" C1* 4TA";
$resn_exch{" C2B 4TA"}=" C2* 4TA"; $resn_exch{" C3B 4TA"}=" C3* 4TA"; $resn_exch{" O3B 4TA"}=" O3* 4TA";
$resn_exch{" C4B 4TA"}=" C4* 4TA"; $resn_exch{" C5B 4TA"}=" C5* 4TA"; $resn_exch{" O5' 4TA"}=" O5* 4TA";
$resn_exch{" O4' 4TA"}=" O4* 4TA"; $resn_exch{" O5' 4TC"}=" O5* 4TC"; $resn_exch{" C5' 4TC"}=" C5* 4TC";
$resn_exch{" C4' 4TC"}=" C4* 4TC"; $resn_exch{" C3' 4TC"}=" C3* 4TC"; $resn_exch{" O3' 4TC"}=" O3* 4TC";
$resn_exch{" C2' 4TC"}=" C2* 4TC"; $resn_exch{" O2' 4TC"}=" O2* 4TC"; $resn_exch{" C1' 4TC"}=" C1* 4TC";
$resn_exch{" O4' 4TC"}=" O4* 4TC"; $resn_exch{" C1' 523"}=" C1* 523"; $resn_exch{" C2' 523"}=" C2* 523";
$resn_exch{" C3' 523"}=" C3* 523"; $resn_exch{" C4' 523"}=" C4* 523"; $resn_exch{" O4' 523"}=" O4* 523";
$resn_exch{" O3' 523"}=" O3* 523"; $resn_exch{" C5' 523"}=" C5* 523"; $resn_exch{" O5' 523"}=" O5* 523";
$resn_exch{" OP3 5AA"}=" O3P 5AA"; $resn_exch{" OP2 5AA"}=" O2P 5AA"; $resn_exch{" O5' 5AA"}=" O5* 5AA";
$resn_exch{" C5' 5AA"}=" C5* 5AA"; $resn_exch{" C4' 5AA"}=" C4* 5AA"; $resn_exch{" O4' 5AA"}=" O4* 5AA";
$resn_exch{" C1' 5AA"}=" C1* 5AA"; $resn_exch{" C2' 5AA"}=" C2* 5AA"; $resn_exch{" O2' 5AA"}=" O2* 5AA";
$resn_exch{" C3' 5AA"}=" C3* 5AA"; $resn_exch{" N3' 5AA"}=" N3* 5AA"; $resn_exch{" OP1 5AA"}=" O1P 5AA";
$resn_exch{" O5' 5AS"}=" O5* 5AS"; $resn_exch{" C5' 5AS"}=" C5* 5AS"; $resn_exch{" C4' 5AS"}=" C4* 5AS";
$resn_exch{" O4' 5AS"}=" O4* 5AS"; $resn_exch{" C3' 5AS"}=" C3* 5AS"; $resn_exch{" O3' 5AS"}=" O3* 5AS";
$resn_exch{" C2' 5AS"}=" C2* 5AS"; $resn_exch{" O2' 5AS"}=" O2* 5AS"; $resn_exch{" C1' 5AS"}=" C1* 5AS";
$resn_exch{" N5' 5AT"}=" N5* 5AT"; $resn_exch{" C2' 5AT"}=" C2* 5AT"; $resn_exch{" C5' 5AT"}=" C5* 5AT";
$resn_exch{" C4' 5AT"}=" C4* 5AT"; $resn_exch{" O4' 5AT"}=" O4* 5AT"; $resn_exch{" C1' 5AT"}=" C1* 5AT";
$resn_exch{" C3' 5AT"}=" C3* 5AT"; $resn_exch{" O3' 5AT"}=" O3* 5AT"; $resn_exch{" OP1 5BU"}=" O1P 5BU";
$resn_exch{" OP2 5BU"}=" O2P 5BU"; $resn_exch{" OP3 5BU"}=" O3P 5BU"; $resn_exch{" O5' 5BU"}=" O5* 5BU";
$resn_exch{" C5' 5BU"}=" C5* 5BU"; $resn_exch{" C4' 5BU"}=" C4* 5BU"; $resn_exch{" O4' 5BU"}=" O4* 5BU";
$resn_exch{" C3' 5BU"}=" C3* 5BU"; $resn_exch{" O3' 5BU"}=" O3* 5BU"; $resn_exch{" C2' 5BU"}=" C2* 5BU";
$resn_exch{" O2' 5BU"}=" O2* 5BU"; $resn_exch{" C1' 5BU"}=" C1* 5BU"; $resn_exch{" O5' 5CA"}=" O5* 5CA";
$resn_exch{" C5' 5CA"}=" C5* 5CA"; $resn_exch{" C4' 5CA"}=" C4* 5CA"; $resn_exch{" O4' 5CA"}=" O4* 5CA";
$resn_exch{" C3' 5CA"}=" C3* 5CA"; $resn_exch{" O3' 5CA"}=" O3* 5CA"; $resn_exch{" C2' 5CA"}=" C2* 5CA";
$resn_exch{" O2' 5CA"}=" O2* 5CA"; $resn_exch{" C1' 5CA"}=" C1* 5CA"; $resn_exch{" O2' 5CD"}=" O2* 5CD";
$resn_exch{" C2' 5CD"}=" C2* 5CD"; $resn_exch{" C3' 5CD"}=" C3* 5CD"; $resn_exch{" O3' 5CD"}=" O3* 5CD";
$resn_exch{" C4' 5CD"}=" C4* 5CD"; $resn_exch{" C5' 5CD"}=" C5* 5CD"; $resn_exch{" O4' 5CD"}=" O4* 5CD";
$resn_exch{" C1' 5CD"}=" C1* 5CD"; $resn_exch{"CL5' 5CG"}="CL5* 5CG"; $resn_exch{" C5' 5CG"}=" C5* 5CG";
$resn_exch{" C4' 5CG"}=" C4* 5CG"; $resn_exch{" O4' 5CG"}=" O4* 5CG"; $resn_exch{" C3' 5CG"}=" C3* 5CG";
$resn_exch{" O3' 5CG"}=" O3* 5CG"; $resn_exch{" C2' 5CG"}=" C2* 5CG"; $resn_exch{" O2' 5CG"}=" O2* 5CG";
$resn_exch{" C1' 5CG"}=" C1* 5CG"; $resn_exch{" C1' 5CM"}=" C1* 5CM"; $resn_exch{" C2' 5CM"}=" C2* 5CM";
$resn_exch{" C3' 5CM"}=" C3* 5CM"; $resn_exch{" C4' 5CM"}=" C4* 5CM"; $resn_exch{" O4' 5CM"}=" O4* 5CM";
$resn_exch{" O3' 5CM"}=" O3* 5CM"; $resn_exch{" C5' 5CM"}=" C5* 5CM"; $resn_exch{" O5' 5CM"}=" O5* 5CM";
$resn_exch{" OP1 5CM"}=" O1P 5CM"; $resn_exch{" OP2 5CM"}=" O2P 5CM"; $resn_exch{" OP3 5CM"}=" O3P 5CM";
$resn_exch{" C2' 5F1"}=" C2* 5F1"; $resn_exch{" C3' 5F1"}=" C3* 5F1"; $resn_exch{" O3' 5F1"}=" O3* 5F1";
$resn_exch{" C4' 5F1"}=" C4* 5F1"; $resn_exch{" C5' 5F1"}=" C5* 5F1"; $resn_exch{" O4' 5F1"}=" O4* 5F1";
$resn_exch{" C1' 5F1"}=" C1* 5F1"; $resn_exch{" O5' 5FA"}=" O5* 5FA"; $resn_exch{" C5' 5FA"}=" C5* 5FA";
$resn_exch{" C4' 5FA"}=" C4* 5FA"; $resn_exch{" O4' 5FA"}=" O4* 5FA"; $resn_exch{" C3' 5FA"}=" C3* 5FA";
$resn_exch{" O3' 5FA"}=" O3* 5FA"; $resn_exch{" C2' 5FA"}=" C2* 5FA"; $resn_exch{" O2' 5FA"}=" O2* 5FA";
$resn_exch{" C1' 5FA"}=" C1* 5FA"; $resn_exch{" OP2 5FC"}=" O2P 5FC"; $resn_exch{" OP1 5FC"}=" O1P 5FC";
$resn_exch{" O5' 5FC"}=" O5* 5FC"; $resn_exch{" C2' 5FC"}=" C2* 5FC"; $resn_exch{" C5' 5FC"}=" C5* 5FC";
$resn_exch{" C4' 5FC"}=" C4* 5FC"; $resn_exch{" O4' 5FC"}=" O4* 5FC"; $resn_exch{" C1' 5FC"}=" C1* 5FC";
$resn_exch{" C3' 5FC"}=" C3* 5FC"; $resn_exch{" O3' 5FC"}=" O3* 5FC"; $resn_exch{" OP3 5FC"}=" O3P 5FC";
$resn_exch{" O2' 5FD"}=" O2* 5FD"; $resn_exch{" C2' 5FD"}=" C2* 5FD"; $resn_exch{" C3' 5FD"}=" C3* 5FD";
$resn_exch{" O3' 5FD"}=" O3* 5FD"; $resn_exch{" C4' 5FD"}=" C4* 5FD"; $resn_exch{" C5' 5FD"}=" C5* 5FD";
$resn_exch{" O4' 5FD"}=" O4* 5FD"; $resn_exch{" C1' 5FD"}=" C1* 5FD"; $resn_exch{" C1' 5FU"}=" C1* 5FU";
$resn_exch{" O4' 5FU"}=" O4* 5FU"; $resn_exch{" C2' 5FU"}=" C2* 5FU"; $resn_exch{" O2' 5FU"}=" O2* 5FU";
$resn_exch{" C3' 5FU"}=" C3* 5FU"; $resn_exch{" O3' 5FU"}=" O3* 5FU"; $resn_exch{" C4' 5FU"}=" C4* 5FU";
$resn_exch{" C5' 5FU"}=" C5* 5FU"; $resn_exch{" O5' 5FU"}=" O5* 5FU"; $resn_exch{" O5' 5GP"}=" O5* 5GP";
$resn_exch{" C5' 5GP"}=" C5* 5GP"; $resn_exch{" C4' 5GP"}=" C4* 5GP"; $resn_exch{" O4' 5GP"}=" O4* 5GP";
$resn_exch{" C3' 5GP"}=" C3* 5GP"; $resn_exch{" O3' 5GP"}=" O3* 5GP"; $resn_exch{" C2' 5GP"}=" C2* 5GP";
$resn_exch{" O2' 5GP"}=" O2* 5GP"; $resn_exch{" C1' 5GP"}=" C1* 5GP"; $resn_exch{" O5' 5HT"}=" O5* 5HT";
$resn_exch{" C5' 5HT"}=" C5* 5HT"; $resn_exch{" C4' 5HT"}=" C4* 5HT"; $resn_exch{" O4' 5HT"}=" O4* 5HT";
$resn_exch{" C3' 5HT"}=" C3* 5HT"; $resn_exch{" O3' 5HT"}=" O3* 5HT"; $resn_exch{" C2' 5HT"}=" C2* 5HT";
$resn_exch{" C1' 5HT"}=" C1* 5HT"; $resn_exch{" OP1 5HU"}=" O1P 5HU"; $resn_exch{" OP2 5HU"}=" O2P 5HU";
$resn_exch{" OP3 5HU"}=" O3P 5HU"; $resn_exch{" O5' 5HU"}=" O5* 5HU"; $resn_exch{" C5' 5HU"}=" C5* 5HU";
$resn_exch{" C4' 5HU"}=" C4* 5HU"; $resn_exch{" O4' 5HU"}=" O4* 5HU"; $resn_exch{" C3' 5HU"}=" C3* 5HU";
$resn_exch{" O3' 5HU"}=" O3* 5HU"; $resn_exch{" C2' 5HU"}=" C2* 5HU"; $resn_exch{" C1' 5HU"}=" C1* 5HU";
$resn_exch{" O2' 5IC"}=" O2* 5IC"; $resn_exch{" C2' 5IC"}=" C2* 5IC"; $resn_exch{" C3' 5IC"}=" C3* 5IC";
$resn_exch{" O3' 5IC"}=" O3* 5IC"; $resn_exch{" C4' 5IC"}=" C4* 5IC"; $resn_exch{" O4' 5IC"}=" O4* 5IC";
$resn_exch{" C1' 5IC"}=" C1* 5IC"; $resn_exch{" C5' 5IC"}=" C5* 5IC"; $resn_exch{" O5' 5IC"}=" O5* 5IC";
$resn_exch{" OP1 5IC"}=" O1P 5IC"; $resn_exch{" OP3 5IC"}=" O3P 5IC"; $resn_exch{" OP2 5IC"}=" O2P 5IC";
$resn_exch{" OP1 5IT"}=" O1P 5IT"; $resn_exch{" OP2 5IT"}=" O2P 5IT"; $resn_exch{" OP3 5IT"}=" O3P 5IT";
$resn_exch{" O5' 5IT"}=" O5* 5IT"; $resn_exch{" C5' 5IT"}=" C5* 5IT"; $resn_exch{" C4' 5IT"}=" C4* 5IT";
$resn_exch{" O4' 5IT"}=" O4* 5IT"; $resn_exch{" C3' 5IT"}=" C3* 5IT"; $resn_exch{" O3' 5IT"}=" O3* 5IT";
$resn_exch{" C2' 5IT"}=" C2* 5IT"; $resn_exch{" C1' 5IT"}=" C1* 5IT"; $resn_exch{" C1' 5IU"}=" C1* 5IU";
$resn_exch{" C2' 5IU"}=" C2* 5IU"; $resn_exch{" C3' 5IU"}=" C3* 5IU"; $resn_exch{" C4' 5IU"}=" C4* 5IU";
$resn_exch{" O3' 5IU"}=" O3* 5IU"; $resn_exch{" O4' 5IU"}=" O4* 5IU"; $resn_exch{" C5' 5IU"}=" C5* 5IU";
$resn_exch{" O5' 5IU"}=" O5* 5IU"; $resn_exch{" OP1 5IU"}=" O1P 5IU"; $resn_exch{" OP2 5IU"}=" O2P 5IU";
$resn_exch{" OP3 5IU"}=" O3P 5IU"; $resn_exch{" OP1 5MC"}=" O1P 5MC"; $resn_exch{" OP2 5MC"}=" O2P 5MC";
$resn_exch{" OP3 5MC"}=" O3P 5MC"; $resn_exch{" O5' 5MC"}=" O5* 5MC"; $resn_exch{" C5' 5MC"}=" C5* 5MC";
$resn_exch{" C4' 5MC"}=" C4* 5MC"; $resn_exch{" O4' 5MC"}=" O4* 5MC"; $resn_exch{" C3' 5MC"}=" C3* 5MC";
$resn_exch{" O3' 5MC"}=" O3* 5MC"; $resn_exch{" C2' 5MC"}=" C2* 5MC"; $resn_exch{" O2' 5MC"}=" O2* 5MC";
$resn_exch{" C1' 5MC"}=" C1* 5MC"; $resn_exch{" C1' 5MU"}=" C1* 5MU"; $resn_exch{" C2' 5MU"}=" C2* 5MU";
$resn_exch{" O2' 5MU"}=" O2* 5MU"; $resn_exch{" C3' 5MU"}=" C3* 5MU"; $resn_exch{" C4' 5MU"}=" C4* 5MU";
$resn_exch{" O3' 5MU"}=" O3* 5MU"; $resn_exch{" O4' 5MU"}=" O4* 5MU"; $resn_exch{" C5' 5MU"}=" C5* 5MU";
$resn_exch{" O5' 5MU"}=" O5* 5MU"; $resn_exch{" OP1 5MU"}=" O1P 5MU"; $resn_exch{" OP2 5MU"}=" O2P 5MU";
$resn_exch{" OP3 5MU"}=" O3P 5MU"; $resn_exch{" OP1 5NC"}=" O1P 5NC"; $resn_exch{" OP2 5NC"}=" O2P 5NC";
$resn_exch{" OP3 5NC"}=" O3P 5NC"; $resn_exch{" O5' 5NC"}=" O5* 5NC"; $resn_exch{" C5' 5NC"}=" C5* 5NC";
$resn_exch{" C4' 5NC"}=" C4* 5NC"; $resn_exch{" O4' 5NC"}=" O4* 5NC"; $resn_exch{" C3' 5NC"}=" C3* 5NC";
$resn_exch{" O3' 5NC"}=" O3* 5NC"; $resn_exch{" C2' 5NC"}=" C2* 5NC"; $resn_exch{" C1' 5NC"}=" C1* 5NC";
$resn_exch{" OP3 5PC"}=" O3P 5PC"; $resn_exch{" OP1 5PC"}=" O1P 5PC"; $resn_exch{" OP2 5PC"}=" O2P 5PC";
$resn_exch{" C5' 5PC"}=" C5* 5PC"; $resn_exch{" O5' 5PC"}=" O5* 5PC"; $resn_exch{" C4' 5PC"}=" C4* 5PC";
$resn_exch{" O4' 5PC"}=" O4* 5PC"; $resn_exch{" C3' 5PC"}=" C3* 5PC"; $resn_exch{" O3' 5PC"}=" O3* 5PC";
$resn_exch{" C2' 5PC"}=" C2* 5PC"; $resn_exch{" C1' 5PC"}=" C1* 5PC"; $resn_exch{" OP2 5PY"}=" O2P 5PY";
$resn_exch{" OP1 5PY"}=" O1P 5PY"; $resn_exch{" O5' 5PY"}=" O5* 5PY"; $resn_exch{" C5' 5PY"}=" C5* 5PY";
$resn_exch{" C4' 5PY"}=" C4* 5PY"; $resn_exch{" O4' 5PY"}=" O4* 5PY"; $resn_exch{" C1' 5PY"}=" C1* 5PY";
$resn_exch{" C2' 5PY"}=" C2* 5PY"; $resn_exch{" C3' 5PY"}=" C3* 5PY"; $resn_exch{" O3' 5PY"}=" O3* 5PY";
$resn_exch{" OP3 5PY"}=" O3P 5PY"; $resn_exch{" C1' 5UD"}=" C1* 5UD"; $resn_exch{" C2' 5UD"}=" C2* 5UD";
$resn_exch{" O2' 5UD"}=" O2* 5UD"; $resn_exch{" C3' 5UD"}=" C3* 5UD"; $resn_exch{" O3' 5UD"}=" O3* 5UD";
$resn_exch{" C4' 5UD"}=" C4* 5UD"; $resn_exch{" O4' 5UD"}=" O4* 5UD"; $resn_exch{" C5' 5UD"}=" C5* 5UD";
$resn_exch{" O5' 5UD"}=" O5* 5UD"; $resn_exch{" CVX 607"}="C31' 607"; $resn_exch{" CWX 607"}="C32' 607";
$resn_exch{" OXX 607"}="O33' 607"; $resn_exch{" OYX 607"}="O34' 607"; $resn_exch{" C6X 607"}="C42' 607";
$resn_exch{" C7X 607"}="C43' 607"; $resn_exch{" O8X 607"}="O44' 607"; $resn_exch{" O9X 607"}="O45' 607";
$resn_exch{" C1B 607"}="C1'' 607"; $resn_exch{" C2B 607"}="C2'' 607"; $resn_exch{" C3B 607"}="C3'' 607";
$resn_exch{" C4B 607"}="C4'' 607"; $resn_exch{" C5B 607"}="C5'' 607"; $resn_exch{" C6B 607"}="C6'' 607";
$resn_exch{" N2B 607"}="N2'' 607"; $resn_exch{" OP1 64T"}=" O1P 64T"; $resn_exch{" OP2 64T"}=" O2P 64T";
$resn_exch{" O5' 64T"}=" O5* 64T"; $resn_exch{" C5' 64T"}=" C5* 64T"; $resn_exch{" C4' 64T"}=" C4* 64T";
$resn_exch{" O4' 64T"}=" O4* 64T"; $resn_exch{" C3' 64T"}=" C3* 64T"; $resn_exch{" O3' 64T"}=" O3* 64T";
$resn_exch{" C2' 64T"}=" C2* 64T"; $resn_exch{" C1' 64T"}=" C1* 64T"; $resn_exch{" OP3 64T"}=" O3P 64T";
$resn_exch{" CV' 653"}="C31' 653"; $resn_exch{" CW' 653"}="C32' 653"; $resn_exch{" NX' 653"}="N33' 653";
$resn_exch{" C1B 653"}="C1'' 653"; $resn_exch{" C2B 653"}="C2'' 653"; $resn_exch{" C3B 653"}="C3'' 653";
$resn_exch{" C4B 653"}="C4'' 653"; $resn_exch{" C5B 653"}="C5'' 653"; $resn_exch{" C6B 653"}="C6'' 653";
$resn_exch{" C1B 655"}="C1'' 655"; $resn_exch{" C2B 655"}="C2'' 655"; $resn_exch{" C3B 655"}="C3'' 655";
$resn_exch{" C4B 655"}="C4'' 655"; $resn_exch{" C5B 655"}="C5'' 655"; $resn_exch{" CV' 678"}="C31' 678";
$resn_exch{" CW' 678"}="C32' 678"; $resn_exch{" OX' 678"}="O33' 678"; $resn_exch{" OY' 678"}="O34' 678";
$resn_exch{" CVX 693"}="C31' 693"; $resn_exch{" CWX 693"}="C32' 693"; $resn_exch{" OXX 693"}="O33' 693";
$resn_exch{" OYX 693"}="O34' 693"; $resn_exch{" C6X 693"}="C42' 693"; $resn_exch{" C7X 693"}="C43' 693";
$resn_exch{" O8X 693"}="O44' 693"; $resn_exch{" O9X 693"}="O45' 693"; $resn_exch{" C1B 693"}="C1'' 693";
$resn_exch{" C2B 693"}="C2'' 693"; $resn_exch{" C3B 693"}="C3'' 693"; $resn_exch{" C4B 693"}="C4'' 693";
$resn_exch{" C5B 693"}="C5'' 693"; $resn_exch{" C6B 693"}="C6'' 693"; $resn_exch{" F2B 693"}="F2'' 693";
$resn_exch{" C1B 696"}="C1'' 696"; $resn_exch{" C2B 696"}="C2'' 696"; $resn_exch{" C3B 696"}="C3'' 696";
$resn_exch{" C4B 696"}="C4'' 696"; $resn_exch{" C5B 696"}="C5'' 696"; $resn_exch{" C6B 696"}="C6'' 696";
$resn_exch{" OP2 6CT"}=" O2P 6CT"; $resn_exch{" OP1 6CT"}=" O1P 6CT"; $resn_exch{" O5' 6CT"}=" O5* 6CT";
$resn_exch{" C5' 6CT"}=" C5* 6CT"; $resn_exch{" C4' 6CT"}=" C4* 6CT"; $resn_exch{" C3' 6CT"}=" C3* 6CT";
$resn_exch{" O3' 6CT"}=" O3* 6CT"; $resn_exch{" C2' 6CT"}=" C2* 6CT"; $resn_exch{" C1' 6CT"}=" C1* 6CT";
$resn_exch{" C'  6CT"}=" C*  6CT"; $resn_exch{" CM' 6CT"}=" CM* 6CT"; $resn_exch{"HO1P 6CT"}="PHO1 6CT";
$resn_exch{" C1' 6FA"}=" C1* 6FA"; $resn_exch{" C2' 6FA"}=" C2* 6FA"; $resn_exch{" O2' 6FA"}=" O2* 6FA";
$resn_exch{" C3' 6FA"}=" C3* 6FA"; $resn_exch{" O3' 6FA"}=" O3* 6FA"; $resn_exch{" C4' 6FA"}=" C4* 6FA";
$resn_exch{" O4' 6FA"}=" O4* 6FA"; $resn_exch{" C5' 6FA"}=" C5* 6FA"; $resn_exch{" O5' 6FA"}=" O5* 6FA";
$resn_exch{" PA  6FA"}=" AP  6FA"; $resn_exch{" O1A 6FA"}=" AO1 6FA"; $resn_exch{" O2A 6FA"}=" AO2 6FA";
$resn_exch{" O5B 6FA"}="AO5* 6FA"; $resn_exch{" C5B 6FA"}="AC5* 6FA"; $resn_exch{" C4B 6FA"}="AC4* 6FA";
$resn_exch{" O4B 6FA"}="AO4* 6FA"; $resn_exch{" C3B 6FA"}="AC3* 6FA"; $resn_exch{" O3B 6FA"}="AO3* 6FA";
$resn_exch{" C2B 6FA"}="AC2* 6FA"; $resn_exch{" O2B 6FA"}="AO2* 6FA"; $resn_exch{" C1B 6FA"}="AC1* 6FA";
$resn_exch{" N9A 6FA"}=" AN9 6FA"; $resn_exch{" C8A 6FA"}=" AC8 6FA"; $resn_exch{" N7A 6FA"}=" AN7 6FA";
$resn_exch{" C5X 6FA"}=" AC5 6FA"; $resn_exch{" C6A 6FA"}=" AC6 6FA"; $resn_exch{" N6A 6FA"}=" AN6 6FA";
$resn_exch{" N1A 6FA"}=" AN1 6FA"; $resn_exch{" C2A 6FA"}=" AC2 6FA"; $resn_exch{" N3A 6FA"}=" AN3 6FA";
$resn_exch{" C4X 6FA"}=" AC4 6FA"; $resn_exch{" OP1 6HA"}=" O1P 6HA"; $resn_exch{" OP2 6HA"}=" O2P 6HA";
$resn_exch{" OP3 6HA"}=" O3P 6HA"; $resn_exch{" O5' 6HA"}=" O5* 6HA"; $resn_exch{" C5' 6HA"}=" C5* 6HA";
$resn_exch{" C4' 6HA"}=" C4* 6HA"; $resn_exch{" O4' 6HA"}=" O4* 6HA"; $resn_exch{" C3' 6HA"}=" C3* 6HA";
$resn_exch{" O3' 6HA"}=" O3* 6HA"; $resn_exch{" C2' 6HA"}=" C2* 6HA"; $resn_exch{" C1' 6HA"}=" C1* 6HA";
$resn_exch{" C6' 6HA"}=" C6* 6HA"; $resn_exch{" OP1 6HC"}=" O1P 6HC"; $resn_exch{" OP2 6HC"}=" O2P 6HC";
$resn_exch{" OP3 6HC"}=" O3P 6HC"; $resn_exch{" O5' 6HC"}=" O5* 6HC"; $resn_exch{" C5' 6HC"}=" C5* 6HC";
$resn_exch{" C4' 6HC"}=" C4* 6HC"; $resn_exch{" O4' 6HC"}=" O4* 6HC"; $resn_exch{" C3' 6HC"}=" C3* 6HC";
$resn_exch{" O3' 6HC"}=" O3* 6HC"; $resn_exch{" C2' 6HC"}=" C2* 6HC"; $resn_exch{" C1' 6HC"}=" C1* 6HC";
$resn_exch{" C6' 6HC"}=" C6* 6HC"; $resn_exch{" OP1 6HG"}=" O1P 6HG"; $resn_exch{" OP2 6HG"}=" O2P 6HG";
$resn_exch{" OP3 6HG"}=" O3P 6HG"; $resn_exch{" O5' 6HG"}=" O5* 6HG"; $resn_exch{" C5' 6HG"}=" C5* 6HG";
$resn_exch{" C4' 6HG"}=" C4* 6HG"; $resn_exch{" O4' 6HG"}=" O4* 6HG"; $resn_exch{" C3' 6HG"}=" C3* 6HG";
$resn_exch{" O3' 6HG"}=" O3* 6HG"; $resn_exch{" C2' 6HG"}=" C2* 6HG"; $resn_exch{" C1' 6HG"}=" C1* 6HG";
$resn_exch{" C6' 6HG"}=" C6* 6HG"; $resn_exch{" OP1 6HT"}=" O1P 6HT"; $resn_exch{" OP2 6HT"}=" O2P 6HT";
$resn_exch{" OP3 6HT"}=" O3P 6HT"; $resn_exch{" O5' 6HT"}=" O5* 6HT"; $resn_exch{" C5' 6HT"}=" C5* 6HT";
$resn_exch{" C4' 6HT"}=" C4* 6HT"; $resn_exch{" O4' 6HT"}=" O4* 6HT"; $resn_exch{" C3' 6HT"}=" C3* 6HT";
$resn_exch{" O3' 6HT"}=" O3* 6HT"; $resn_exch{" C2' 6HT"}=" C2* 6HT"; $resn_exch{" C1' 6HT"}=" C1* 6HT";
$resn_exch{" C6' 6HT"}=" C6* 6HT"; $resn_exch{" OP3 6IA"}=" O3P 6IA"; $resn_exch{" OP1 6IA"}=" O1P 6IA";
$resn_exch{" OP2 6IA"}=" O2P 6IA"; $resn_exch{" O5' 6IA"}=" O5* 6IA"; $resn_exch{" C5' 6IA"}=" C5* 6IA";
$resn_exch{" C4' 6IA"}=" C4* 6IA"; $resn_exch{" O4' 6IA"}=" O4* 6IA"; $resn_exch{" C3' 6IA"}=" C3* 6IA";
$resn_exch{" O3' 6IA"}=" O3* 6IA"; $resn_exch{" C2' 6IA"}=" C2* 6IA"; $resn_exch{" O2' 6IA"}=" O2* 6IA";
$resn_exch{" C1' 6IA"}=" C1* 6IA"; $resn_exch{" OP3 6MA"}=" O3P 6MA"; $resn_exch{" OP1 6MA"}=" O1P 6MA";
$resn_exch{" OP2 6MA"}=" O2P 6MA"; $resn_exch{" O5' 6MA"}=" O5* 6MA"; $resn_exch{" C5' 6MA"}=" C5* 6MA";
$resn_exch{" C4' 6MA"}=" C4* 6MA"; $resn_exch{" O4' 6MA"}=" O4* 6MA"; $resn_exch{" C3' 6MA"}=" C3* 6MA";
$resn_exch{" O3' 6MA"}=" O3* 6MA"; $resn_exch{" C2' 6MA"}=" C2* 6MA"; $resn_exch{" C1' 6MA"}=" C1* 6MA";
$resn_exch{" OP3 6MC"}=" O3P 6MC"; $resn_exch{" OP1 6MC"}=" O1P 6MC"; $resn_exch{" OP2 6MC"}=" O2P 6MC";
$resn_exch{" O5' 6MC"}=" O5* 6MC"; $resn_exch{" C5' 6MC"}=" C5* 6MC"; $resn_exch{" C4' 6MC"}=" C4* 6MC";
$resn_exch{" O4' 6MC"}=" O4* 6MC"; $resn_exch{" C3' 6MC"}=" C3* 6MC"; $resn_exch{" O3' 6MC"}=" O3* 6MC";
$resn_exch{" C2' 6MC"}=" C2* 6MC"; $resn_exch{" C1' 6MC"}=" C1* 6MC"; $resn_exch{" O5' 6MI"}=" O5* 6MI";
$resn_exch{" C2' 6MI"}=" C2* 6MI"; $resn_exch{" C5' 6MI"}=" C5* 6MI"; $resn_exch{" C4' 6MI"}=" C4* 6MI";
$resn_exch{" O4' 6MI"}=" O4* 6MI"; $resn_exch{" C1' 6MI"}=" C1* 6MI"; $resn_exch{" C3' 6MI"}=" C3* 6MI";
$resn_exch{" O3' 6MI"}=" O3* 6MI"; $resn_exch{" OP3 6MT"}=" O3P 6MT"; $resn_exch{" OP1 6MT"}=" O1P 6MT";
$resn_exch{" OP2 6MT"}=" O2P 6MT"; $resn_exch{" O5' 6MT"}=" O5* 6MT"; $resn_exch{" C5' 6MT"}=" C5* 6MT";
$resn_exch{" C4' 6MT"}=" C4* 6MT"; $resn_exch{" O4' 6MT"}=" O4* 6MT"; $resn_exch{" C3' 6MT"}=" C3* 6MT";
$resn_exch{" O3' 6MT"}=" O3* 6MT"; $resn_exch{" C2' 6MT"}=" C2* 6MT"; $resn_exch{" C1' 6MT"}=" C1* 6MT";
$resn_exch{" O5' 6MZ"}=" O5* 6MZ"; $resn_exch{" C5' 6MZ"}=" C5* 6MZ"; $resn_exch{" C4' 6MZ"}=" C4* 6MZ";
$resn_exch{" O4' 6MZ"}=" O4* 6MZ"; $resn_exch{" C1' 6MZ"}=" C1* 6MZ"; $resn_exch{" C2' 6MZ"}=" C2* 6MZ";
$resn_exch{" O2' 6MZ"}=" O2* 6MZ"; $resn_exch{" C3' 6MZ"}=" C3* 6MZ"; $resn_exch{" O3' 6MZ"}=" O3* 6MZ";
$resn_exch{" C1B 6NH"}="C1'' 6NH"; $resn_exch{" C2B 6NH"}="C2'' 6NH"; $resn_exch{" C3B 6NH"}="C3'' 6NH";
$resn_exch{" C4B 6NH"}="C4'' 6NH"; $resn_exch{" C5B 6NH"}="C5'' 6NH"; $resn_exch{" C6B 6NH"}="C6'' 6NH";
$resn_exch{" OP1 6OG"}=" O1P 6OG"; $resn_exch{" OP2 6OG"}=" O2P 6OG"; $resn_exch{" O5' 6OG"}=" O5* 6OG";
$resn_exch{" C2' 6OG"}=" C2* 6OG"; $resn_exch{" C5' 6OG"}=" C5* 6OG"; $resn_exch{" C4' 6OG"}=" C4* 6OG";
$resn_exch{" O4' 6OG"}=" O4* 6OG"; $resn_exch{" C1' 6OG"}=" C1* 6OG"; $resn_exch{" C3' 6OG"}=" C3* 6OG";
$resn_exch{" O3' 6OG"}=" O3* 6OG"; $resn_exch{" OP3 6OG"}=" O3P 6OG"; $resn_exch{" C1' 70U"}=" C1* 70U";
$resn_exch{" O2' 70U"}=" O2* 70U"; $resn_exch{" C2' 70U"}=" C2* 70U"; $resn_exch{" O3' 70U"}=" O3* 70U";
$resn_exch{" C3' 70U"}=" C3* 70U"; $resn_exch{" C4' 70U"}=" C4* 70U"; $resn_exch{" O4' 70U"}=" O4* 70U";
$resn_exch{" C5' 70U"}=" C5* 70U"; $resn_exch{" O5' 70U"}=" O5* 70U"; $resn_exch{" OP1 70U"}=" O1P 70U";
$resn_exch{" OP2 70U"}=" O2P 70U"; $resn_exch{" OP3 70U"}=" O3P 70U"; $resn_exch{" C1B 762"}="C1'' 762";
$resn_exch{" C2B 762"}="C2'' 762"; $resn_exch{" C3B 762"}="C3'' 762"; $resn_exch{" C4B 762"}="C4'' 762";
$resn_exch{" C5B 762"}="C5'' 762"; $resn_exch{" C6B 762"}="C6'' 762"; $resn_exch{" C1B 780"}="C1'' 780";
$resn_exch{" C2B 780"}="C2'' 780"; $resn_exch{" C3B 780"}="C3'' 780"; $resn_exch{" C4B 780"}="C4'' 780";
$resn_exch{" C5B 780"}="C5'' 780"; $resn_exch{" C6B 780"}="C6'' 780"; $resn_exch{" C3X 783"}="CM3' 783";
$resn_exch{" C1B 783"}="C1'' 783"; $resn_exch{" C2B 783"}="C2'' 783"; $resn_exch{" C3B 783"}="C3'' 783";
$resn_exch{" C4B 783"}="C4'' 783"; $resn_exch{" C5B 783"}="C5'' 783"; $resn_exch{" C6B 783"}="C6'' 783";
$resn_exch{" C3X 785"}="CF3' 785"; $resn_exch{" FV' 785"}="F31' 785"; $resn_exch{" FW' 785"}="F32' 785";
$resn_exch{" FX' 785"}="F33' 785"; $resn_exch{" OP1 7DA"}=" O1P 7DA"; $resn_exch{" OP2 7DA"}=" O2P 7DA";
$resn_exch{" O5' 7DA"}=" O5* 7DA"; $resn_exch{" C2' 7DA"}=" C2* 7DA"; $resn_exch{" C5' 7DA"}=" C5* 7DA";
$resn_exch{" C4' 7DA"}=" C4* 7DA"; $resn_exch{" O4' 7DA"}=" O4* 7DA"; $resn_exch{" C1' 7DA"}=" C1* 7DA";
$resn_exch{" C3' 7DA"}=" C3* 7DA"; $resn_exch{" O3' 7DA"}=" O3* 7DA"; $resn_exch{" OP3 7DA"}=" O3P 7DA";
$resn_exch{" OP1 7GU"}=" O1P 7GU"; $resn_exch{" OP2 7GU"}=" O2P 7GU"; $resn_exch{" O5' 7GU"}=" O5* 7GU";
$resn_exch{" C2' 7GU"}=" C2* 7GU"; $resn_exch{" C5' 7GU"}=" C5* 7GU"; $resn_exch{" C4' 7GU"}=" C4* 7GU";
$resn_exch{" O4' 7GU"}=" O4* 7GU"; $resn_exch{" C1' 7GU"}=" C1* 7GU"; $resn_exch{" C3' 7GU"}=" C3* 7GU";
$resn_exch{" O3' 7GU"}=" O3* 7GU"; $resn_exch{" OP3 7GU"}=" O3P 7GU"; $resn_exch{" OP1 7MG"}=" O1P 7MG";
$resn_exch{" OP2 7MG"}=" O2P 7MG"; $resn_exch{" OP3 7MG"}=" O3P 7MG"; $resn_exch{" O5' 7MG"}=" O5* 7MG";
$resn_exch{" C5' 7MG"}=" C5* 7MG"; $resn_exch{" C4' 7MG"}=" C4* 7MG"; $resn_exch{" O4' 7MG"}=" O4* 7MG";
$resn_exch{" C3' 7MG"}=" C3* 7MG"; $resn_exch{" O3' 7MG"}=" O3* 7MG"; $resn_exch{" C2' 7MG"}=" C2* 7MG";
$resn_exch{" O2' 7MG"}=" O2* 7MG"; $resn_exch{" C1' 7MG"}=" C1* 7MG"; $resn_exch{" C1B 7NH"}="C1'' 7NH";
$resn_exch{" C2B 7NH"}="C2'' 7NH"; $resn_exch{" C3B 7NH"}="C3'' 7NH"; $resn_exch{" C4B 7NH"}="C4'' 7NH";
$resn_exch{" C5B 7NH"}="C5'' 7NH"; $resn_exch{" C6B 7NH"}="C6'' 7NH"; $resn_exch{" O5' 7RA"}=" O5* 7RA";
$resn_exch{" C5' 7RA"}=" C5* 7RA"; $resn_exch{" C4' 7RA"}=" C4* 7RA"; $resn_exch{" O4' 7RA"}=" O4* 7RA";
$resn_exch{" C3' 7RA"}=" C3* 7RA"; $resn_exch{" O3' 7RA"}=" O3* 7RA"; $resn_exch{" C2' 7RA"}=" C2* 7RA";
$resn_exch{" O2' 7RA"}=" O2* 7RA"; $resn_exch{" C1' 7RA"}=" C1* 7RA"; $resn_exch{" O5' 7RP"}=" O5* 7RP";
$resn_exch{" C5' 7RP"}=" C5* 7RP"; $resn_exch{" C4' 7RP"}=" C4* 7RP"; $resn_exch{" O4' 7RP"}=" O4* 7RP";
$resn_exch{" C3' 7RP"}=" C3* 7RP"; $resn_exch{" O3' 7RP"}=" O3* 7RP"; $resn_exch{" C2' 7RP"}=" C2* 7RP";
$resn_exch{" O2' 7RP"}=" O2* 7RP"; $resn_exch{" C1' 7RP"}=" C1* 7RP"; $resn_exch{" CEA 81A"}="C14A 81A";
$resn_exch{" CEB 81A"}="C14B 81A"; $resn_exch{" CVX 847"}="C31' 847"; $resn_exch{" CWX 847"}="C32' 847";
$resn_exch{" OXX 847"}="O33' 847"; $resn_exch{" OYX 847"}="O34' 847"; $resn_exch{" C6X 847"}="C42' 847";
$resn_exch{" C7X 847"}="C43' 847"; $resn_exch{" O8X 847"}="O44' 847"; $resn_exch{" O9X 847"}="O45' 847";
$resn_exch{" O5' 8BR"}=" O5* 8BR"; $resn_exch{" C5' 8BR"}=" C5* 8BR"; $resn_exch{" C4' 8BR"}=" C4* 8BR";
$resn_exch{" O4' 8BR"}=" O4* 8BR"; $resn_exch{" C3' 8BR"}=" C3* 8BR"; $resn_exch{" O3' 8BR"}=" O3* 8BR";
$resn_exch{" C2' 8BR"}=" C2* 8BR"; $resn_exch{" O2' 8BR"}=" O2* 8BR"; $resn_exch{" C1' 8BR"}=" C1* 8BR";
$resn_exch{" C5' 8DA"}=" C5* 8DA"; $resn_exch{" C4' 8DA"}=" C4* 8DA"; $resn_exch{" C3' 8DA"}=" C3* 8DA";
$resn_exch{" O3' 8DA"}=" O3* 8DA"; $resn_exch{" O4' 8DA"}=" O4* 8DA"; $resn_exch{" C1' 8DA"}=" C1* 8DA";
$resn_exch{" C2' 8DA"}=" C2* 8DA"; $resn_exch{" O5' 8DG"}=" O5* 8DG"; $resn_exch{" C5' 8DG"}=" C5* 8DG";
$resn_exch{" C4' 8DG"}=" C4* 8DG"; $resn_exch{" O4' 8DG"}=" O4* 8DG"; $resn_exch{" C3' 8DG"}=" C3* 8DG";
$resn_exch{" O3' 8DG"}=" O3* 8DG"; $resn_exch{" C2' 8DG"}=" C2* 8DG"; $resn_exch{" C1' 8DG"}=" C1* 8DG";
$resn_exch{" OP3 8FG"}=" O3P 8FG"; $resn_exch{" OP1 8FG"}=" O1P 8FG"; $resn_exch{" OP2 8FG"}=" O2P 8FG";
$resn_exch{" O5' 8FG"}=" O5* 8FG"; $resn_exch{" C2' 8FG"}=" C2* 8FG"; $resn_exch{" C5' 8FG"}=" C5* 8FG";
$resn_exch{" C4' 8FG"}=" C4* 8FG"; $resn_exch{" O4' 8FG"}=" O4* 8FG"; $resn_exch{" C1' 8FG"}=" C1* 8FG";
$resn_exch{" C3' 8FG"}=" C3* 8FG"; $resn_exch{" O3' 8FG"}=" O3* 8FG"; $resn_exch{" O5' 8GT"}=" O5* 8GT";
$resn_exch{" C5' 8GT"}=" C5* 8GT"; $resn_exch{" C4' 8GT"}=" C4* 8GT"; $resn_exch{" O4' 8GT"}=" O4* 8GT";
$resn_exch{" C3' 8GT"}=" C3* 8GT"; $resn_exch{" O3' 8GT"}=" O3* 8GT"; $resn_exch{" C2' 8GT"}=" C2* 8GT";
$resn_exch{" O2' 8GT"}=" O2* 8GT"; $resn_exch{" C1' 8GT"}=" C1* 8GT"; $resn_exch{" C1' 8HG"}=" C1* 8HG";
$resn_exch{" O4' 8HG"}=" O4* 8HG"; $resn_exch{" C2' 8HG"}=" C2* 8HG"; $resn_exch{" C3' 8HG"}=" C3* 8HG";
$resn_exch{" O3' 8HG"}=" O3* 8HG"; $resn_exch{" C4' 8HG"}=" C4* 8HG"; $resn_exch{" C5' 8HG"}=" C5* 8HG";
$resn_exch{" O5' 8HG"}=" O5* 8HG"; $resn_exch{" O5B 8ID"}=" O5* 8ID"; $resn_exch{" C5B 8ID"}=" C5* 8ID";
$resn_exch{" C4B 8ID"}=" C4* 8ID"; $resn_exch{" O4B 8ID"}=" O4* 8ID"; $resn_exch{" C1B 8ID"}=" C1* 8ID";
$resn_exch{" C2B 8ID"}=" C2* 8ID"; $resn_exch{" O2B 8ID"}=" O2* 8ID"; $resn_exch{" C3B 8ID"}=" C3* 8ID";
$resn_exch{" O3B 8ID"}=" O3* 8ID"; $resn_exch{" OP3 8MG"}=" O3P 8MG"; $resn_exch{" OP2 8MG"}=" O2P 8MG";
$resn_exch{" O5' 8MG"}=" O5* 8MG"; $resn_exch{" C2' 8MG"}=" C2* 8MG"; $resn_exch{" C5' 8MG"}=" C5* 8MG";
$resn_exch{" C4' 8MG"}=" C4* 8MG"; $resn_exch{" O4' 8MG"}=" O4* 8MG"; $resn_exch{" C1' 8MG"}=" C1* 8MG";
$resn_exch{" C3' 8MG"}=" C3* 8MG"; $resn_exch{" O3' 8MG"}=" O3* 8MG"; $resn_exch{" OP1 8MG"}=" O1P 8MG";
$resn_exch{" OP3 8OG"}=" O3P 8OG"; $resn_exch{" OP1 8OG"}=" O1P 8OG"; $resn_exch{" OP2 8OG"}=" O2P 8OG";
$resn_exch{" O5' 8OG"}=" O5* 8OG"; $resn_exch{" C5' 8OG"}=" C5* 8OG"; $resn_exch{" C4' 8OG"}=" C4* 8OG";
$resn_exch{" O4' 8OG"}=" O4* 8OG"; $resn_exch{" C3' 8OG"}=" C3* 8OG"; $resn_exch{" O3' 8OG"}=" O3* 8OG";
$resn_exch{" C2' 8OG"}=" C2* 8OG"; $resn_exch{" C1' 8OG"}=" C1* 8OG"; $resn_exch{" CV' 907"}="C31' 907";
$resn_exch{" C5' 93A"}=" C5* 93A"; $resn_exch{" O5' 93A"}=" O5* 93A"; $resn_exch{" C1B 991"}="C1'' 991";
$resn_exch{" C2B 991"}="C2'' 991"; $resn_exch{" C3B 991"}="C3'' 991"; $resn_exch{" C4B 991"}="C4'' 991";
$resn_exch{" C5B 991"}="C5'' 991"; $resn_exch{" OP3   A"}=" O3P   A"; $resn_exch{" OP1   A"}=" O1P   A";
$resn_exch{" OP2   A"}=" O2P   A"; $resn_exch{" O5'   A"}=" O5*   A"; $resn_exch{" C5'   A"}=" C5*   A";
$resn_exch{" C4'   A"}=" C4*   A"; $resn_exch{" O4'   A"}=" O4*   A"; $resn_exch{" C3'   A"}=" C3*   A";
$resn_exch{" O3'   A"}=" O3*   A"; $resn_exch{" C2'   A"}=" C2*   A"; $resn_exch{" O2'   A"}=" O2*   A";
$resn_exch{" C1'   A"}=" C1*   A"; $resn_exch{" O5' A12"}=" O5* A12"; $resn_exch{" C5' A12"}=" C5* A12";
$resn_exch{" C4' A12"}=" C4* A12"; $resn_exch{" O4' A12"}=" O4* A12"; $resn_exch{" C3' A12"}=" C3* A12";
$resn_exch{" O3' A12"}=" O3* A12"; $resn_exch{" C2' A12"}=" C2* A12"; $resn_exch{" O2' A12"}=" O2* A12";
$resn_exch{" C1' A12"}=" C1* A12"; $resn_exch{" O5' A1P"}=" O5* A1P"; $resn_exch{" C2' A1P"}=" C2* A1P";
$resn_exch{" C5' A1P"}=" C5* A1P"; $resn_exch{" C4' A1P"}=" C4* A1P"; $resn_exch{" O4' A1P"}=" O4* A1P";
$resn_exch{" C1' A1P"}=" C1* A1P"; $resn_exch{" C3' A1P"}=" C3* A1P"; $resn_exch{" O3' A1P"}=" O3* A1P";
$resn_exch{" C1' A1R"}=" C1* A1R"; $resn_exch{" C2' A1R"}=" C2* A1R"; $resn_exch{" O2' A1R"}=" O2* A1R";
$resn_exch{" C3' A1R"}=" C3* A1R"; $resn_exch{" O3' A1R"}=" O3* A1R"; $resn_exch{" O4' A1R"}=" O4* A1R";
$resn_exch{" C4' A1R"}=" C4* A1R"; $resn_exch{" C5' A1R"}=" C5* A1R"; $resn_exch{" O5' A1R"}=" O5* A1R";
$resn_exch{" P2D A22"}="P2*A A22"; $resn_exch{" O11 A22"}="O1PA A22"; $resn_exch{" O12 A22"}="O2PA A22";
$resn_exch{" O13 A22"}="O3PA A22"; $resn_exch{" O14 A22"}="O5PA A22"; $resn_exch{" O15 A22"}="O6PA A22";
$resn_exch{" O5D A22"}="O5*A A22"; $resn_exch{" C5D A22"}="C5*A A22"; $resn_exch{" C4D A22"}="C4*A A22";
$resn_exch{" O4D A22"}="O4*A A22"; $resn_exch{" C3D A22"}="C3*A A22"; $resn_exch{" O3D A22"}="O3*A A22";
$resn_exch{" C2D A22"}="C2*A A22"; $resn_exch{" O2D A22"}="O2*A A22"; $resn_exch{" C1D A22"}="C1*A A22";
$resn_exch{" O16 A22"}="O1PB A22"; $resn_exch{" O17 A22"}="O2PB A22"; $resn_exch{" O18 A22"}="O3PB A22";
$resn_exch{" O5E A22"}="O5*B A22"; $resn_exch{" C5E A22"}="C5*B A22"; $resn_exch{" C4E A22"}="C4*B A22";
$resn_exch{" O4E A22"}="O4*B A22"; $resn_exch{" C3E A22"}="C3*B A22"; $resn_exch{" O3E A22"}="O3*B A22";
$resn_exch{" C2E A22"}="C2*B A22"; $resn_exch{" O2E A22"}="O2*B A22"; $resn_exch{" C1E A22"}="C1*B A22";
$resn_exch{" OP1 A23"}=" O1P A23"; $resn_exch{" OP2 A23"}=" O2P A23"; $resn_exch{" OP3 A23"}=" O3P A23";
$resn_exch{" O5' A23"}=" O5* A23"; $resn_exch{" C5' A23"}=" C5* A23"; $resn_exch{" C4' A23"}=" C4* A23";
$resn_exch{" O4' A23"}=" O4* A23"; $resn_exch{" C3' A23"}=" C3* A23"; $resn_exch{" O3' A23"}=" O3* A23";
$resn_exch{" C2' A23"}=" C2* A23"; $resn_exch{" O2' A23"}=" O2* A23"; $resn_exch{" C1' A23"}=" C1* A23";
$resn_exch{" O1B A24"}=" O1* A24"; $resn_exch{" C2B A24"}=" C2* A24"; $resn_exch{" C3B A24"}=" C3* A24";
$resn_exch{" N4' A24"}=" N4* A24"; $resn_exch{" C5B A24"}=" C5* A24"; $resn_exch{" C6B A24"}=" C6* A24";
$resn_exch{" C7B A24"}=" C7* A24"; $resn_exch{" C8' A24"}=" C8* A24"; $resn_exch{" O5D A2D"}="O5*A A2D";
$resn_exch{" C5D A2D"}="C5*A A2D"; $resn_exch{" C4D A2D"}="C4*A A2D"; $resn_exch{" O4D A2D"}="O4*A A2D";
$resn_exch{" C3D A2D"}="C3*A A2D"; $resn_exch{" O3D A2D"}="O3*A A2D"; $resn_exch{" C2D A2D"}="C2*A A2D";
$resn_exch{" O2D A2D"}="O2*A A2D"; $resn_exch{" C1D A2D"}="C1*A A2D"; $resn_exch{" O5E A2D"}="O5*B A2D";
$resn_exch{" C5E A2D"}="C5*B A2D"; $resn_exch{" C4E A2D"}="C4*B A2D"; $resn_exch{" O4E A2D"}="O4*B A2D";
$resn_exch{" C3E A2D"}="C3*B A2D"; $resn_exch{" O3E A2D"}="O3*B A2D"; $resn_exch{" C2E A2D"}="C2*B A2D";
$resn_exch{" O2E A2D"}="O2*B A2D"; $resn_exch{" C1E A2D"}="C1*B A2D"; $resn_exch{" O2' A2L"}=" O2* A2L";
$resn_exch{" C2' A2L"}=" C2* A2L"; $resn_exch{" C1' A2L"}=" C1* A2L"; $resn_exch{" O4' A2L"}=" O4* A2L";
$resn_exch{" C4' A2L"}=" C4* A2L"; $resn_exch{" C5' A2L"}=" C5* A2L"; $resn_exch{" O5' A2L"}=" O5* A2L";
$resn_exch{" OP1 A2L"}=" O1P A2L"; $resn_exch{" OP3 A2L"}=" O3P A2L"; $resn_exch{" OP2 A2L"}=" O2P A2L";
$resn_exch{" C3' A2L"}=" C3* A2L"; $resn_exch{" O3' A2L"}=" O3* A2L"; $resn_exch{" CA' A2L"}=" CA* A2L";
$resn_exch{" CB' A2L"}=" CB* A2L"; $resn_exch{" OC' A2L"}=" OC* A2L"; $resn_exch{" CD' A2L"}=" CD* A2L";
$resn_exch{" OP3 A2M"}=" O3P A2M"; $resn_exch{" OP1 A2M"}=" O1P A2M"; $resn_exch{" OP2 A2M"}=" O2P A2M";
$resn_exch{" O5' A2M"}=" O5* A2M"; $resn_exch{" C5' A2M"}=" C5* A2M"; $resn_exch{" C4' A2M"}=" C4* A2M";
$resn_exch{" O4' A2M"}=" O4* A2M"; $resn_exch{" C3' A2M"}=" C3* A2M"; $resn_exch{" O3' A2M"}=" O3* A2M";
$resn_exch{" C2' A2M"}=" C2* A2M"; $resn_exch{" O2' A2M"}=" O2* A2M"; $resn_exch{" C1' A2M"}=" C1* A2M";
$resn_exch{" CM' A2M"}=" CM* A2M"; $resn_exch{" O5' A2P"}=" O5* A2P"; $resn_exch{" C5' A2P"}=" C5* A2P";
$resn_exch{" C4' A2P"}=" C4* A2P"; $resn_exch{" O4' A2P"}=" O4* A2P"; $resn_exch{" C3' A2P"}=" C3* A2P";
$resn_exch{" O3' A2P"}=" O3* A2P"; $resn_exch{" C2' A2P"}=" C2* A2P"; $resn_exch{" O2' A2P"}=" O2* A2P";
$resn_exch{" C1' A2P"}=" C1* A2P"; $resn_exch{" P2' A2R"}=" P2* A2R"; $resn_exch{" O2' A2R"}=" O2* A2R";
$resn_exch{" C2' A2R"}=" C2* A2R"; $resn_exch{" C1' A2R"}=" C1* A2R"; $resn_exch{" O4' A2R"}=" O4* A2R";
$resn_exch{" C3' A2R"}=" C3* A2R"; $resn_exch{" O3' A2R"}=" O3* A2R"; $resn_exch{" C4' A2R"}=" C4* A2R";
$resn_exch{" C5' A2R"}=" C5* A2R"; $resn_exch{" O5' A2R"}=" O5* A2R"; $resn_exch{"O15' A2R"}="O15* A2R";
$resn_exch{"C15' A2R"}="C15* A2R"; $resn_exch{"C14' A2R"}="C14* A2R"; $resn_exch{"C13' A2R"}="C13* A2R";
$resn_exch{"O13' A2R"}="O13* A2R"; $resn_exch{"C12' A2R"}="C12* A2R"; $resn_exch{"O12' A2R"}="O12* A2R";
$resn_exch{"C11' A2R"}="C11* A2R"; $resn_exch{"O14' A2R"}="O14* A2R"; $resn_exch{"O11' A2R"}="O11* A2R";
$resn_exch{" O1B A32"}=" O1* A32"; $resn_exch{" C2B A32"}=" C2* A32"; $resn_exch{" C3B A32"}=" C3* A32";
$resn_exch{" N4' A32"}=" N4* A32"; $resn_exch{" C5B A32"}=" C5* A32"; $resn_exch{" C6B A32"}=" C6* A32";
$resn_exch{" C7B A32"}=" C7* A32"; $resn_exch{" C8' A32"}=" C8* A32"; $resn_exch{" C9' A32"}=" C9* A32";
$resn_exch{" O5' A34"}=" O5* A34"; $resn_exch{" C2' A34"}=" C2* A34"; $resn_exch{" C5' A34"}=" C5* A34";
$resn_exch{" C4' A34"}=" C4* A34"; $resn_exch{" O4' A34"}=" O4* A34"; $resn_exch{" C1' A34"}=" C1* A34";
$resn_exch{" C3' A34"}=" C3* A34"; $resn_exch{" O3' A34"}=" O3* A34"; $resn_exch{" O5' A35"}=" O5* A35";
$resn_exch{" C5' A35"}=" C5* A35"; $resn_exch{" C4' A35"}=" C4* A35"; $resn_exch{" O4' A35"}=" O4* A35";
$resn_exch{" C1' A35"}=" C1* A35"; $resn_exch{" C2' A35"}=" C2* A35"; $resn_exch{" C3' A35"}=" C3* A35";
$resn_exch{" O3' A35"}=" O3* A35"; $resn_exch{" O5' A38"}=" O5* A38"; $resn_exch{" C5' A38"}=" C5* A38";
$resn_exch{" C4' A38"}=" C4* A38"; $resn_exch{" O4' A38"}=" O4* A38"; $resn_exch{" C3' A38"}=" C3* A38";
$resn_exch{" O3' A38"}=" O3* A38"; $resn_exch{" C2' A38"}=" C2* A38"; $resn_exch{" C1' A38"}=" C1* A38";
$resn_exch{" O5' A39"}=" O5* A39"; $resn_exch{" C5' A39"}=" C5* A39"; $resn_exch{" C4' A39"}=" C4* A39";
$resn_exch{" O4' A39"}=" O4* A39"; $resn_exch{" C3' A39"}=" C3* A39"; $resn_exch{" O3' A39"}=" O3* A39";
$resn_exch{" C2' A39"}=" C2* A39"; $resn_exch{" O2' A39"}=" O2* A39"; $resn_exch{" C1' A39"}=" C1* A39";
$resn_exch{" OP1 A3A"}=" O1P A3A"; $resn_exch{" OP2 A3A"}=" O2P A3A"; $resn_exch{" O6' A3A"}=" O6* A3A";
$resn_exch{" C5' A3A"}=" C5* A3A"; $resn_exch{" C4' A3A"}=" C4* A3A"; $resn_exch{" O5' A3A"}=" O5* A3A";
$resn_exch{" C3' A3A"}=" C3* A3A"; $resn_exch{" O3' A3A"}=" O3* A3A"; $resn_exch{" C2' A3A"}=" C2* A3A";
$resn_exch{" C1' A3A"}=" C1* A3A"; $resn_exch{" PA  A3D"}=" AP  A3D"; $resn_exch{" O1A A3D"}=" AO1 A3D";
$resn_exch{" O2A A3D"}=" AO2 A3D"; $resn_exch{" O5B A3D"}="AO5* A3D"; $resn_exch{" C5B A3D"}="AC5* A3D";
$resn_exch{" C4B A3D"}="AC4* A3D"; $resn_exch{" O4B A3D"}="AO4* A3D"; $resn_exch{" C3B A3D"}="AC3* A3D";
$resn_exch{" O3B A3D"}="AO3* A3D"; $resn_exch{" C2B A3D"}="AC2* A3D"; $resn_exch{" O2B A3D"}="AO2* A3D";
$resn_exch{" C1B A3D"}="AC1* A3D"; $resn_exch{" N9A A3D"}=" AN9 A3D"; $resn_exch{" C8A A3D"}=" AC8 A3D";
$resn_exch{" N7A A3D"}=" AN7 A3D"; $resn_exch{" C5A A3D"}=" AC5 A3D"; $resn_exch{" C6A A3D"}=" AC6 A3D";
$resn_exch{" N6A A3D"}=" AN6 A3D"; $resn_exch{" N1A A3D"}=" AN1 A3D"; $resn_exch{" C2A A3D"}=" AC2 A3D";
$resn_exch{" N3A A3D"}=" AN3 A3D"; $resn_exch{" C4A A3D"}=" AC4 A3D"; $resn_exch{" PN  A3D"}=" NP  A3D";
$resn_exch{" O1N A3D"}=" NO1 A3D"; $resn_exch{" O2N A3D"}=" NO2 A3D"; $resn_exch{" O5D A3D"}="NO5* A3D";
$resn_exch{" C5D A3D"}="NC5* A3D"; $resn_exch{" C4D A3D"}="NC4* A3D"; $resn_exch{" O4D A3D"}="NO4* A3D";
$resn_exch{" C3D A3D"}="NC3* A3D"; $resn_exch{" O3D A3D"}="NO3* A3D"; $resn_exch{" C2D A3D"}="NC2* A3D";
$resn_exch{" O2D A3D"}="NO2* A3D"; $resn_exch{" C1D A3D"}="NC1* A3D"; $resn_exch{" N1N A3D"}=" NN1 A3D";
$resn_exch{" C2N A3D"}=" NC2 A3D"; $resn_exch{" C3N A3D"}=" NC3 A3D"; $resn_exch{" C7N A3D"}=" NC7 A3D";
$resn_exch{" O7N A3D"}=" NO7 A3D"; $resn_exch{" C8N A3D"}=" NC8 A3D"; $resn_exch{" C4N A3D"}=" NC4 A3D";
$resn_exch{" C5N A3D"}=" NC5 A3D"; $resn_exch{" C6N A3D"}=" NC6 A3D"; $resn_exch{" O5' A3P"}=" O5* A3P";
$resn_exch{" C5' A3P"}=" C5* A3P"; $resn_exch{" C4' A3P"}=" C4* A3P"; $resn_exch{" O4' A3P"}=" O4* A3P";
$resn_exch{" C3' A3P"}=" C3* A3P"; $resn_exch{" O3' A3P"}=" O3* A3P"; $resn_exch{" C2' A3P"}=" C2* A3P";
$resn_exch{" O2' A3P"}=" O2* A3P"; $resn_exch{" C1' A3P"}=" C1* A3P"; $resn_exch{" O5' A40"}=" O5* A40";
$resn_exch{" C5' A40"}=" C5* A40"; $resn_exch{" C4' A40"}=" C4* A40"; $resn_exch{" O4' A40"}=" O4* A40";
$resn_exch{" C1' A40"}=" C1* A40"; $resn_exch{" C2' A40"}=" C2* A40"; $resn_exch{" C3' A40"}=" C3* A40";
$resn_exch{" O3' A40"}=" O3* A40"; $resn_exch{" O5' A43"}=" O5* A43"; $resn_exch{" C5' A43"}=" C5* A43";
$resn_exch{" C4' A43"}=" C4* A43"; $resn_exch{" O4' A43"}=" O4* A43"; $resn_exch{" C3' A43"}=" C3* A43";
$resn_exch{" C2' A43"}=" C2* A43"; $resn_exch{" C1' A43"}=" C1* A43"; $resn_exch{" O5' A44"}=" O5* A44";
$resn_exch{" C5' A44"}=" C5* A44"; $resn_exch{" C4' A44"}=" C4* A44"; $resn_exch{" O4' A44"}=" O4* A44";
$resn_exch{" C3' A44"}=" C3* A44"; $resn_exch{" O3' A44"}=" O3* A44"; $resn_exch{" C2' A44"}=" C2* A44";
$resn_exch{" O2' A44"}=" O2* A44"; $resn_exch{" C1' A44"}=" C1* A44"; $resn_exch{" CA' A44"}=" CA* A44";
$resn_exch{" OC' A44"}=" OC* A44"; $resn_exch{" O5' A47"}=" O5* A47"; $resn_exch{" C5' A47"}=" C5* A47";
$resn_exch{" C4' A47"}=" C4* A47"; $resn_exch{" O4' A47"}=" O4* A47"; $resn_exch{" C3' A47"}=" C3* A47";
$resn_exch{" O3' A47"}=" O3* A47"; $resn_exch{" C2' A47"}=" C2* A47"; $resn_exch{" C1' A47"}=" C1* A47";
$resn_exch{" O5' A4P"}=" O5* A4P"; $resn_exch{" C5' A4P"}=" C5* A4P"; $resn_exch{" C4' A4P"}=" C4* A4P";
$resn_exch{" O4' A4P"}=" O4* A4P"; $resn_exch{" C3' A4P"}=" C3* A4P"; $resn_exch{" O3' A4P"}=" O3* A4P";
$resn_exch{" C2' A4P"}=" C2* A4P"; $resn_exch{" O2' A4P"}=" O2* A4P"; $resn_exch{" O5' A5A"}=" O5* A5A";
$resn_exch{" C5' A5A"}=" C5* A5A"; $resn_exch{" C4' A5A"}=" C4* A5A"; $resn_exch{" O4' A5A"}=" O4* A5A";
$resn_exch{" C3' A5A"}=" C3* A5A"; $resn_exch{" O3' A5A"}=" O3* A5A"; $resn_exch{" C2' A5A"}=" C2* A5A";
$resn_exch{" O2' A5A"}=" O2* A5A"; $resn_exch{" C1' A5A"}=" C1* A5A"; $resn_exch{" OP1 A5M"}=" O1P A5M";
$resn_exch{" OP2 A5M"}=" O2P A5M"; $resn_exch{" O5' A5M"}=" O5* A5M"; $resn_exch{" C5' A5M"}=" C5* A5M";
$resn_exch{" C4' A5M"}=" C4* A5M"; $resn_exch{" O4' A5M"}=" O4* A5M"; $resn_exch{" C1' A5M"}=" C1* A5M";
$resn_exch{" C2' A5M"}=" C2* A5M"; $resn_exch{" N2' A5M"}=" N2* A5M"; $resn_exch{" C3' A5M"}=" C3* A5M";
$resn_exch{" O3' A5M"}=" O3* A5M"; $resn_exch{" OP3 A5M"}=" O3P A5M"; $resn_exch{" C4' AAD"}=" C4* AAD";
$resn_exch{" O4' AAD"}=" O4* AAD"; $resn_exch{" C3' AAD"}=" C3* AAD"; $resn_exch{" O3' AAD"}=" O3* AAD";
$resn_exch{" C2' AAD"}=" C2* AAD"; $resn_exch{" O2' AAD"}=" O2* AAD"; $resn_exch{" C1' AAD"}=" C1* AAD";
$resn_exch{" C5' AAD"}=" C5* AAD"; $resn_exch{" O6' AAM"}=" O6* AAM"; $resn_exch{" C5' AAM"}=" C5* AAM";
$resn_exch{" C4' AAM"}=" C4* AAM"; $resn_exch{" O5' AAM"}=" O5* AAM"; $resn_exch{" C3' AAM"}=" C3* AAM";
$resn_exch{" O3' AAM"}=" O3* AAM"; $resn_exch{" C2' AAM"}=" C2* AAM"; $resn_exch{" O2' AAM"}=" O2* AAM";
$resn_exch{" C1' AAM"}=" C1* AAM"; $resn_exch{" C5' AAT"}=" C5* AAT"; $resn_exch{" C4' AAT"}=" C4* AAT";
$resn_exch{" O4' AAT"}=" O4* AAT"; $resn_exch{" C3' AAT"}=" C3* AAT"; $resn_exch{" O3' AAT"}=" O3* AAT";
$resn_exch{" C2' AAT"}=" C2* AAT"; $resn_exch{" O2' AAT"}=" O2* AAT"; $resn_exch{" C1' AAT"}=" C1* AAT";
$resn_exch{" O1X ABF"}="O1P' ABF"; $resn_exch{" O2X ABF"}="O2P' ABF"; $resn_exch{" O3X ABF"}="O3P' ABF";
$resn_exch{" O5' ABM"}=" O5* ABM"; $resn_exch{" C5' ABM"}=" C5* ABM"; $resn_exch{" C4' ABM"}=" C4* ABM";
$resn_exch{" O4' ABM"}=" O4* ABM"; $resn_exch{" C3' ABM"}=" C3* ABM"; $resn_exch{" O3' ABM"}=" O3* ABM";
$resn_exch{" C2' ABM"}=" C2* ABM"; $resn_exch{" O2' ABM"}=" O2* ABM"; $resn_exch{" C1' ABM"}=" C1* ABM";
$resn_exch{" O5' ABP"}=" O5* ABP"; $resn_exch{" C5' ABP"}=" C5* ABP"; $resn_exch{" C4' ABP"}=" C4* ABP";
$resn_exch{" O4' ABP"}=" O4* ABP"; $resn_exch{" C3' ABP"}=" C3* ABP"; $resn_exch{" O3' ABP"}=" O3* ABP";
$resn_exch{" C2' ABP"}=" C2* ABP"; $resn_exch{" O2' ABP"}=" O2* ABP"; $resn_exch{" C1' ABP"}=" C1* ABP";
$resn_exch{" OP1 ABR"}=" O1P ABR"; $resn_exch{" OP2 ABR"}=" O2P ABR"; $resn_exch{" O5' ABR"}=" O5* ABR";
$resn_exch{" C5' ABR"}=" C5* ABR"; $resn_exch{" C4' ABR"}=" C4* ABR"; $resn_exch{" O4' ABR"}=" O4* ABR";
$resn_exch{" C1' ABR"}=" C1* ABR"; $resn_exch{" C2' ABR"}=" C2* ABR"; $resn_exch{" C3' ABR"}=" C3* ABR";
$resn_exch{" O3' ABR"}=" O3* ABR"; $resn_exch{" OP3 ABR"}=" O3P ABR"; $resn_exch{" OP1 ABS"}=" O1P ABS";
$resn_exch{" OP2 ABS"}=" O2P ABS"; $resn_exch{" O5' ABS"}=" O5* ABS"; $resn_exch{" C5' ABS"}=" C5* ABS";
$resn_exch{" C4' ABS"}=" C4* ABS"; $resn_exch{" O4' ABS"}=" O4* ABS"; $resn_exch{" C1' ABS"}=" C1* ABS";
$resn_exch{" C2' ABS"}=" C2* ABS"; $resn_exch{" C3' ABS"}=" C3* ABS"; $resn_exch{" O3' ABS"}=" O3* ABS";
$resn_exch{" OP3 ABS"}=" O3P ABS"; $resn_exch{" O5' ABT"}=" O5* ABT"; $resn_exch{" C5' ABT"}=" C5* ABT";
$resn_exch{" C4' ABT"}=" C4* ABT"; $resn_exch{" O4' ABT"}=" O4* ABT"; $resn_exch{" C1' ABT"}=" C1* ABT";
$resn_exch{" C2' ABT"}=" C2* ABT"; $resn_exch{" C3' ABT"}=" C3* ABT"; $resn_exch{" N3' ABT"}=" N3* ABT";
$resn_exch{" N4' ABT"}=" N4* ABT"; $resn_exch{" N5' ABT"}=" N5* ABT"; $resn_exch{" C3' AC2"}=" C3* AC2";
$resn_exch{" O3' AC2"}=" O3* AC2"; $resn_exch{" C2' AC2"}=" C2* AC2"; $resn_exch{" O1' AC2"}=" O1* AC2";
$resn_exch{" C1' AC2"}=" C1* AC2"; $resn_exch{" O5' ACK"}=" O5* ACK"; $resn_exch{" C5' ACK"}=" C5* ACK";
$resn_exch{" C4' ACK"}=" C4* ACK"; $resn_exch{" O4' ACK"}=" O4* ACK"; $resn_exch{" C3' ACK"}=" C3* ACK";
$resn_exch{" O3' ACK"}=" O3* ACK"; $resn_exch{" C1' ACK"}=" C1* ACK"; $resn_exch{" C2' ACK"}=" C2* ACK";
$resn_exch{" N1A ACO"}=" AN1 ACO"; $resn_exch{" C2A ACO"}=" AC2 ACO"; $resn_exch{" N3A ACO"}=" AN3 ACO";
$resn_exch{" C4A ACO"}=" AC4 ACO"; $resn_exch{" C5A ACO"}=" AC5 ACO"; $resn_exch{" C6A ACO"}=" AC6 ACO";
$resn_exch{" N6A ACO"}=" AN6 ACO"; $resn_exch{" N7A ACO"}=" AN7 ACO"; $resn_exch{" C8A ACO"}=" AC8 ACO";
$resn_exch{" N9A ACO"}=" AN9 ACO"; $resn_exch{" C1B ACO"}="AC1* ACO"; $resn_exch{" C2B ACO"}="AC2* ACO";
$resn_exch{" O2B ACO"}="AO2* ACO"; $resn_exch{" C3B ACO"}="AC3* ACO"; $resn_exch{" O3B ACO"}="AO3* ACO";
$resn_exch{" P3B ACO"}="AP3* ACO"; $resn_exch{" O7A ACO"}=" AO7 ACO"; $resn_exch{" O8A ACO"}=" AO8 ACO";
$resn_exch{" O9A ACO"}=" AO9 ACO"; $resn_exch{" C4B ACO"}="AC4* ACO"; $resn_exch{" O4B ACO"}="AO4* ACO";
$resn_exch{" C5B ACO"}="AC5* ACO"; $resn_exch{" O5B ACO"}="AO5* ACO"; $resn_exch{" P1A ACO"}=" AP1 ACO";
$resn_exch{" O1A ACO"}=" AO1 ACO"; $resn_exch{" O2A ACO"}=" AO2 ACO"; $resn_exch{" O3A ACO"}=" AO3 ACO";
$resn_exch{" P2A ACO"}=" AP2 ACO"; $resn_exch{" O4A ACO"}=" AO4 ACO"; $resn_exch{" O5A ACO"}=" AO5 ACO";
$resn_exch{" O6A ACO"}=" AO6 ACO"; $resn_exch{" CBP ACO"}="PC11 ACO"; $resn_exch{" CCP ACO"}="PC12 ACO";
$resn_exch{" CDP ACO"}="PC13 ACO"; $resn_exch{" CEP ACO"}="PC14 ACO"; $resn_exch{" CAP ACO"}="PC10 ACO";
$resn_exch{" OAP ACO"}="PO10 ACO"; $resn_exch{" C9P ACO"}=" PC9 ACO"; $resn_exch{" O9P ACO"}=" PO9 ACO";
$resn_exch{" N8P ACO"}=" PN8 ACO"; $resn_exch{" C7P ACO"}=" PC7 ACO"; $resn_exch{" C6P ACO"}=" PC6 ACO";
$resn_exch{" C5P ACO"}=" PC5 ACO"; $resn_exch{" O5P ACO"}=" PO5 ACO"; $resn_exch{" N4P ACO"}=" PN4 ACO";
$resn_exch{" C3P ACO"}=" PC3 ACO"; $resn_exch{" C2P ACO"}=" PC2 ACO"; $resn_exch{" S1P ACO"}=" PS1 ACO";
$resn_exch{" O5' ACP"}=" O5* ACP"; $resn_exch{" C5' ACP"}=" C5* ACP"; $resn_exch{" C4' ACP"}=" C4* ACP";
$resn_exch{" O4' ACP"}=" O4* ACP"; $resn_exch{" C3' ACP"}=" C3* ACP"; $resn_exch{" O3' ACP"}=" O3* ACP";
$resn_exch{" C2' ACP"}=" C2* ACP"; $resn_exch{" O2' ACP"}=" O2* ACP"; $resn_exch{" C1' ACP"}=" C1* ACP";
$resn_exch{" O5' ACQ"}=" O5* ACQ"; $resn_exch{" C5' ACQ"}=" C5* ACQ"; $resn_exch{" C4' ACQ"}=" C4* ACQ";
$resn_exch{" O4' ACQ"}=" O4* ACQ"; $resn_exch{" C3' ACQ"}=" C3* ACQ"; $resn_exch{" O3' ACQ"}=" O3* ACQ";
$resn_exch{" C2' ACQ"}=" C2* ACQ"; $resn_exch{" O2' ACQ"}=" O2* ACQ"; $resn_exch{" C1' ACQ"}=" C1* ACQ";
$resn_exch{" OP1 AD2"}=" O1P AD2"; $resn_exch{" OP2 AD2"}=" O2P AD2"; $resn_exch{" OP3 AD2"}=" O3P AD2";
$resn_exch{" O5' AD3"}=" O5* AD3"; $resn_exch{" C5' AD3"}=" C5* AD3"; $resn_exch{" C4' AD3"}=" C4* AD3";
$resn_exch{" O4' AD3"}=" O4* AD3"; $resn_exch{" C3' AD3"}=" C3* AD3"; $resn_exch{" O3' AD3"}=" O3* AD3";
$resn_exch{" C2' AD3"}=" C2* AD3"; $resn_exch{" O2' AD3"}=" O2* AD3"; $resn_exch{" C1' AD3"}=" C1* AD3";
$resn_exch{" O5' ADI"}=" O5* ADI"; $resn_exch{" C5' ADI"}=" C5* ADI"; $resn_exch{" C4' ADI"}=" C4* ADI";
$resn_exch{" O4' ADI"}=" O4* ADI"; $resn_exch{" C3' ADI"}=" C3* ADI"; $resn_exch{" C2' ADI"}=" C2* ADI";
$resn_exch{" C1' ADI"}=" C1* ADI"; $resn_exch{" O5' ADN"}=" O5* ADN"; $resn_exch{" C5' ADN"}=" C5* ADN";
$resn_exch{" C4' ADN"}=" C4* ADN"; $resn_exch{" O4' ADN"}=" O4* ADN"; $resn_exch{" C3' ADN"}=" C3* ADN";
$resn_exch{" O3' ADN"}=" O3* ADN"; $resn_exch{" C2' ADN"}=" C2* ADN"; $resn_exch{" O2' ADN"}=" O2* ADN";
$resn_exch{" C1' ADN"}=" C1* ADN"; $resn_exch{" O5' ADP"}=" O5* ADP"; $resn_exch{" C5' ADP"}=" C5* ADP";
$resn_exch{" C4' ADP"}=" C4* ADP"; $resn_exch{" O4' ADP"}=" O4* ADP"; $resn_exch{" C3' ADP"}=" C3* ADP";
$resn_exch{" O3' ADP"}=" O3* ADP"; $resn_exch{" C2' ADP"}=" C2* ADP"; $resn_exch{" O2' ADP"}=" O2* ADP";
$resn_exch{" C1' ADP"}=" C1* ADP"; $resn_exch{" O5D ADQ"}=" O5* ADQ"; $resn_exch{" C5D ADQ"}=" C5* ADQ";
$resn_exch{" C4D ADQ"}=" C4* ADQ"; $resn_exch{" O4D ADQ"}=" O4* ADQ"; $resn_exch{" C3D ADQ"}=" C3* ADQ";
$resn_exch{" O3D ADQ"}=" O3* ADQ"; $resn_exch{" C2D ADQ"}=" C2* ADQ"; $resn_exch{" O2D ADQ"}=" O2* ADQ";
$resn_exch{" C1D ADQ"}=" C1* ADQ"; $resn_exch{" O5' ADS"}=" O5* ADS"; $resn_exch{" C5' ADS"}=" C5* ADS";
$resn_exch{" C4' ADS"}=" C4* ADS"; $resn_exch{" O4' ADS"}=" O4* ADS"; $resn_exch{" C3' ADS"}=" C3* ADS";
$resn_exch{" O3' ADS"}=" O3* ADS"; $resn_exch{" C2' ADS"}=" C2* ADS"; $resn_exch{" O2' ADS"}=" O2* ADS";
$resn_exch{" C1' ADS"}=" C1* ADS"; $resn_exch{" C1' ADT"}=" C1* ADT"; $resn_exch{" O4' ADT"}=" O4* ADT";
$resn_exch{" C4' ADT"}=" C4* ADT"; $resn_exch{" C5' ADT"}=" C5* ADT"; $resn_exch{" O5' ADT"}=" O5* ADT";
$resn_exch{" C3' ADT"}=" C3* ADT"; $resn_exch{" N3' ADT"}=" N3* ADT"; $resn_exch{" C8' ADT"}=" C8* ADT";
$resn_exch{" O8' ADT"}=" O8* ADT"; $resn_exch{" C9' ADT"}=" C9* ADT"; $resn_exch{" C2' ADT"}=" C2* ADT";
$resn_exch{" C1' ADU"}=" C1* ADU"; $resn_exch{" O4' ADU"}=" O4* ADU"; $resn_exch{" C4' ADU"}=" C4* ADU";
$resn_exch{" C5' ADU"}=" C5* ADU"; $resn_exch{" O5' ADU"}=" O5* ADU"; $resn_exch{" C3' ADU"}=" C3* ADU";
$resn_exch{" N3' ADU"}=" N3* ADU"; $resn_exch{" C8' ADU"}=" C8* ADU"; $resn_exch{" O8' ADU"}=" O8* ADU";
$resn_exch{" C9' ADU"}=" C9* ADU"; $resn_exch{" C2' ADU"}=" C2* ADU"; $resn_exch{" O2' ADU"}=" O2* ADU";
$resn_exch{" O5' ADV"}=" O5* ADV"; $resn_exch{" C5' ADV"}=" C5* ADV"; $resn_exch{" C4' ADV"}=" C4* ADV";
$resn_exch{" O4' ADV"}=" O4* ADV"; $resn_exch{" C3' ADV"}=" C3* ADV"; $resn_exch{" O3' ADV"}=" O3* ADV";
$resn_exch{" C2' ADV"}=" C2* ADV"; $resn_exch{" O2' ADV"}=" O2* ADV"; $resn_exch{" C1' ADV"}=" C1* ADV";
$resn_exch{" O5' ADW"}=" O5* ADW"; $resn_exch{" C5' ADW"}=" C5* ADW"; $resn_exch{" C4' ADW"}=" C4* ADW";
$resn_exch{" O4' ADW"}=" O4* ADW"; $resn_exch{" C3' ADW"}=" C3* ADW"; $resn_exch{" O3' ADW"}=" O3* ADW";
$resn_exch{" C2' ADW"}=" C2* ADW"; $resn_exch{" O2' ADW"}=" O2* ADW"; $resn_exch{" C1' ADW"}=" C1* ADW";
$resn_exch{" O5' ADX"}=" O5* ADX"; $resn_exch{" C5' ADX"}=" C5* ADX"; $resn_exch{" C4' ADX"}=" C4* ADX";
$resn_exch{" O4' ADX"}=" O4* ADX"; $resn_exch{" C3' ADX"}=" C3* ADX"; $resn_exch{" O3' ADX"}=" O3* ADX";
$resn_exch{" C2' ADX"}=" C2* ADX"; $resn_exch{" O2' ADX"}=" O2* ADX"; $resn_exch{" C1' ADX"}=" C1* ADX";
$resn_exch{" O5' ADY"}=" O5* ADY"; $resn_exch{" C5' ADY"}=" C5* ADY"; $resn_exch{" C4' ADY"}=" C4* ADY";
$resn_exch{" O4' ADY"}=" O4* ADY"; $resn_exch{" C3' ADY"}=" C3* ADY"; $resn_exch{" O3' ADY"}=" O3* ADY";
$resn_exch{" C2' ADY"}=" C2* ADY"; $resn_exch{" O2' ADY"}=" O2* ADY"; $resn_exch{" C1' ADY"}=" C1* ADY";
$resn_exch{" C1' ADZ"}=" C1* ADZ"; $resn_exch{" OP1 AET"}=" O1P AET"; $resn_exch{" OP2 AET"}=" O2P AET";
$resn_exch{" OP3 AET"}=" O3P AET"; $resn_exch{" O5' AET"}=" O5* AET"; $resn_exch{" C5' AET"}=" C5* AET";
$resn_exch{" C4' AET"}=" C4* AET"; $resn_exch{" O4' AET"}=" O4* AET"; $resn_exch{" C3' AET"}=" C3* AET";
$resn_exch{" O3' AET"}=" O3* AET"; $resn_exch{" C2' AET"}=" C2* AET"; $resn_exch{" O2' AET"}=" O2* AET";
$resn_exch{" C1' AET"}=" C1* AET"; $resn_exch{" ODA AET"}="O13A AET"; $resn_exch{" ODB AET"}="O13B AET";
$resn_exch{" OP3 AFG"}=" O3P AFG"; $resn_exch{" OP1 AFG"}=" O1P AFG"; $resn_exch{" OP2 AFG"}=" O2P AFG";
$resn_exch{" O5' AFG"}=" O5* AFG"; $resn_exch{" C2' AFG"}=" C2* AFG"; $resn_exch{" C5' AFG"}=" C5* AFG";
$resn_exch{" C4' AFG"}=" C4* AFG"; $resn_exch{" O4' AFG"}=" O4* AFG"; $resn_exch{" C1' AFG"}=" C1* AFG";
$resn_exch{" C3' AFG"}=" C3* AFG"; $resn_exch{" O3' AFG"}=" O3* AFG"; $resn_exch{" C1D AFH"}=" C1* AFH";
$resn_exch{" C2D AFH"}=" C2* AFH"; $resn_exch{" O2' AFH"}=" O2* AFH"; $resn_exch{" C3D AFH"}=" C3* AFH";
$resn_exch{" C4D AFH"}=" C4* AFH"; $resn_exch{" O4D AFH"}=" O4* AFH"; $resn_exch{" O3D AFH"}=" O3* AFH";
$resn_exch{" C5D AFH"}=" C5* AFH"; $resn_exch{" O5D AFH"}=" O5* AFH"; $resn_exch{" CDB AGF"}="1CD1 AGF";
$resn_exch{" CDL AGF"}="2CD1 AGF"; $resn_exch{" CEB AGF"}="1CE1 AGF"; $resn_exch{" CEL AGF"}="2CE1 AGF";
$resn_exch{" CDE AGF"}="1CD4 AGF"; $resn_exch{" CDO AGF"}="2CD4 AGF"; $resn_exch{" CEE AGF"}="1CE4 AGF";
$resn_exch{" CEO AGF"}="2CE4 AGF"; $resn_exch{" O5' AGN"}=" O5* AGN"; $resn_exch{" C5B AGN"}=" C5* AGN";
$resn_exch{" C4B AGN"}=" C4* AGN"; $resn_exch{" O4' AGN"}=" O4* AGN"; $resn_exch{" C3B AGN"}=" C3* AGN";
$resn_exch{" O3' AGN"}=" O3* AGN"; $resn_exch{" C2B AGN"}=" C2* AGN"; $resn_exch{" C1B AGN"}=" C1* AGN";
$resn_exch{" O5' AGS"}=" O5* AGS"; $resn_exch{" C5' AGS"}=" C5* AGS"; $resn_exch{" C4' AGS"}=" C4* AGS";
$resn_exch{" O4' AGS"}=" O4* AGS"; $resn_exch{" C3' AGS"}=" C3* AGS"; $resn_exch{" O3' AGS"}=" O3* AGS";
$resn_exch{" C2' AGS"}=" C2* AGS"; $resn_exch{" O2' AGS"}=" O2* AGS"; $resn_exch{" C1' AGS"}=" C1* AGS";
$resn_exch{" O5' AHR"}=" O5* AHR"; $resn_exch{" C5' AHR"}=" C5* AHR"; $resn_exch{" C4' AHR"}=" C4* AHR";
$resn_exch{" O4' AHR"}=" O4* AHR"; $resn_exch{" C3' AHR"}=" C3* AHR"; $resn_exch{" O3' AHR"}=" O3* AHR";
$resn_exch{" C2' AHR"}=" C2* AHR"; $resn_exch{" O2' AHR"}=" O2* AHR"; $resn_exch{" C1' AHR"}=" C1* AHR";
$resn_exch{" O1' AHR"}=" O1* AHR"; $resn_exch{" C4' AHU"}=" C4* AHU"; $resn_exch{" C7' AHU"}=" C7* AHU";
$resn_exch{" O7' AHU"}=" O7* AHU"; $resn_exch{" O5' AHU"}=" O5* AHU"; $resn_exch{" C6' AHU"}=" C6* AHU";
$resn_exch{" C1' AHU"}=" C1* AHU"; $resn_exch{" C2' AHU"}=" C2* AHU"; $resn_exch{" C3' AHU"}=" C3* AHU";
$resn_exch{" O3' AHU"}=" O3* AHU"; $resn_exch{" O5' AHX"}=" O5* AHX"; $resn_exch{" C5' AHX"}=" C5* AHX";
$resn_exch{" C4' AHX"}=" C4* AHX"; $resn_exch{" O4' AHX"}=" O4* AHX"; $resn_exch{" C3' AHX"}=" C3* AHX";
$resn_exch{" O3' AHX"}=" O3* AHX"; $resn_exch{" C2' AHX"}=" C2* AHX"; $resn_exch{" O2' AHX"}=" O2* AHX";
$resn_exch{" C1' AHX"}=" C1* AHX"; $resn_exch{" O2' AHZ"}=" O2* AHZ"; $resn_exch{" C2' AHZ"}=" C2* AHZ";
$resn_exch{" C3' AHZ"}=" C3* AHZ"; $resn_exch{" O3' AHZ"}=" O3* AHZ"; $resn_exch{" C1' AHZ"}=" C1* AHZ";
$resn_exch{" O4' AHZ"}=" O4* AHZ"; $resn_exch{" C4' AHZ"}=" C4* AHZ"; $resn_exch{" C5' AHZ"}=" C5* AHZ";
$resn_exch{" O5' AHZ"}=" O5* AHZ"; $resn_exch{" C1B AI1"}=" C1* AI1"; $resn_exch{" C2B AI1"}=" C2* AI1";
$resn_exch{" C3B AI1"}=" C3* AI1"; $resn_exch{" C4B AI1"}=" C4* AI1"; $resn_exch{" C5B AI1"}=" C5* AI1";
$resn_exch{" C6B AI1"}=" C6* AI1"; $resn_exch{" C7B AI1"}=" C7* AI1"; $resn_exch{" O   AIB"}=" O1  AIB";
$resn_exch{" OXT AIB"}=" O2  AIB"; $resn_exch{" O2' AIS"}=" O2* AIS"; $resn_exch{" C2' AIS"}=" C2* AIS";
$resn_exch{" C3' AIS"}=" C3* AIS"; $resn_exch{" O3' AIS"}=" O3* AIS"; $resn_exch{" C4' AIS"}=" C4* AIS";
$resn_exch{" C5' AIS"}=" C5* AIS"; $resn_exch{" O5' AIS"}=" O5* AIS"; $resn_exch{" O4' AIS"}=" O4* AIS";
$resn_exch{" C1' AIS"}=" C1* AIS"; $resn_exch{"CL28 AIZ"}="8CL2 AIZ"; $resn_exch{" O4' AKA"}=" O4* AKA";
$resn_exch{" C4' AKA"}=" C4* AKA"; $resn_exch{" C5' AKA"}=" C5* AKA"; $resn_exch{" C6' AKA"}=" C6* AKA";
$resn_exch{" O5' AKA"}=" O5* AKA"; $resn_exch{" C3' AKA"}=" C3* AKA"; $resn_exch{" N3' AKA"}=" N3* AKA";
$resn_exch{" C8' AKA"}=" C8* AKA"; $resn_exch{" C7' AKA"}=" C7* AKA"; $resn_exch{" C2' AKA"}=" C2* AKA";
$resn_exch{" C1' AKA"}=" C1* AKA"; $resn_exch{" C1' AKT"}=" C1* AKT"; $resn_exch{" C2' AKT"}=" C2* AKT";
$resn_exch{" C3' AKT"}=" C3* AKT"; $resn_exch{" N3' AKT"}=" N3* AKT"; $resn_exch{" C8' AKT"}=" C8* AKT";
$resn_exch{" C7' AKT"}=" C7* AKT"; $resn_exch{" C4' AKT"}=" C4* AKT"; $resn_exch{" O4' AKT"}=" O4* AKT";
$resn_exch{" C5' AKT"}=" C5* AKT"; $resn_exch{" O5' AKT"}=" O5* AKT"; $resn_exch{" C6' AKT"}=" C6* AKT";
$resn_exch{" C1' ALL"}=" C1* ALL"; $resn_exch{" C2' ALL"}=" C2* ALL"; $resn_exch{" C3' ALL"}=" C3* ALL";
$resn_exch{" C4' ALL"}=" C4* ALL"; $resn_exch{" C5' ALL"}=" C5* ALL"; $resn_exch{" C6' ALL"}=" C6* ALL";
$resn_exch{" O1' ALL"}=" O1* ALL"; $resn_exch{" O2' ALL"}=" O2* ALL"; $resn_exch{" O3' ALL"}=" O3* ALL";
$resn_exch{" O4' ALL"}=" O4* ALL"; $resn_exch{" O5' ALL"}=" O5* ALL"; $resn_exch{" O6' ALL"}=" O6* ALL";
$resn_exch{" CX5 ALW"}="3C11 ALW"; $resn_exch{" O5' AMD"}=" O5* AMD"; $resn_exch{" C5' AMD"}=" C5* AMD";
$resn_exch{" C4' AMD"}=" C4* AMD"; $resn_exch{" O4' AMD"}=" O4* AMD"; $resn_exch{" C1' AMD"}=" C1* AMD";
$resn_exch{" C2' AMD"}=" C2* AMD"; $resn_exch{" C3' AMD"}=" C3* AMD"; $resn_exch{" O3' AMD"}=" O3* AMD";
$resn_exch{" O5' AMO"}=" O5* AMO"; $resn_exch{" C5' AMO"}=" C5* AMO"; $resn_exch{" C4' AMO"}=" C4* AMO";
$resn_exch{" O4' AMO"}=" O4* AMO"; $resn_exch{" C1' AMO"}=" C1* AMO"; $resn_exch{" C2' AMO"}=" C2* AMO";
$resn_exch{" C3' AMO"}=" C3* AMO"; $resn_exch{" O3' AMO"}=" O3* AMO"; $resn_exch{" O2' AMO"}=" O2* AMO";
$resn_exch{" O5' AMP"}=" O5* AMP"; $resn_exch{" C5' AMP"}=" C5* AMP"; $resn_exch{" C4' AMP"}=" C4* AMP";
$resn_exch{" O4' AMP"}=" O4* AMP"; $resn_exch{" C3' AMP"}=" C3* AMP"; $resn_exch{" O3' AMP"}=" O3* AMP";
$resn_exch{" C2' AMP"}=" C2* AMP"; $resn_exch{" O2' AMP"}=" O2* AMP"; $resn_exch{" C1' AMP"}=" C1* AMP";
$resn_exch{" O5' AMW"}=" O5* AMW"; $resn_exch{" C5' AMW"}=" C5* AMW"; $resn_exch{" C4' AMW"}=" C4* AMW";
$resn_exch{" O4' AMW"}=" O4* AMW"; $resn_exch{" C3' AMW"}=" C3* AMW"; $resn_exch{" O3' AMW"}=" O3* AMW";
$resn_exch{" C2' AMW"}=" C2* AMW"; $resn_exch{" O2' AMW"}=" O2* AMW"; $resn_exch{" C1' AMW"}=" C1* AMW";
$resn_exch{" N1A AMX"}=" AN1 AMX"; $resn_exch{" C2A AMX"}=" AC2 AMX"; $resn_exch{" N3A AMX"}=" AN3 AMX";
$resn_exch{" C4A AMX"}=" AC4 AMX"; $resn_exch{" C5A AMX"}=" AC5 AMX"; $resn_exch{" C6A AMX"}=" AC6 AMX";
$resn_exch{" N6A AMX"}=" AN6 AMX"; $resn_exch{" N7A AMX"}=" AN7 AMX"; $resn_exch{" C8A AMX"}=" AC8 AMX";
$resn_exch{" N9A AMX"}=" AN9 AMX"; $resn_exch{" C1B AMX"}="AC1* AMX"; $resn_exch{" C2B AMX"}="AC2* AMX";
$resn_exch{" O2B AMX"}="AO2* AMX"; $resn_exch{" C3B AMX"}="AC3* AMX"; $resn_exch{" O3B AMX"}="AO3* AMX";
$resn_exch{" P3B AMX"}="AP3* AMX"; $resn_exch{" O7A AMX"}=" AO7 AMX"; $resn_exch{" O8A AMX"}=" AO8 AMX";
$resn_exch{" O9A AMX"}=" AO9 AMX"; $resn_exch{" C4B AMX"}="AC4* AMX"; $resn_exch{" O4B AMX"}="AO4* AMX";
$resn_exch{" C5B AMX"}="AC5* AMX"; $resn_exch{" O5B AMX"}="AO5* AMX"; $resn_exch{" P1A AMX"}=" AP1 AMX";
$resn_exch{" O1A AMX"}=" AO1 AMX"; $resn_exch{" O2A AMX"}=" AO2 AMX"; $resn_exch{" O3A AMX"}=" AO3 AMX";
$resn_exch{" P2A AMX"}=" AP2 AMX"; $resn_exch{" O4A AMX"}=" AO4 AMX"; $resn_exch{" O5A AMX"}=" AO5 AMX";
$resn_exch{" O6A AMX"}=" AO6 AMX"; $resn_exch{" CBP AMX"}="PC11 AMX"; $resn_exch{" CCP AMX"}="PC12 AMX";
$resn_exch{" CDP AMX"}="PC13 AMX"; $resn_exch{" CEP AMX"}="PC14 AMX"; $resn_exch{" CAP AMX"}="PC10 AMX";
$resn_exch{" OAP AMX"}="PO10 AMX"; $resn_exch{" C9P AMX"}=" PC9 AMX"; $resn_exch{" O9P AMX"}=" PO9 AMX";
$resn_exch{" N8P AMX"}=" PN8 AMX"; $resn_exch{" C7P AMX"}=" PC7 AMX"; $resn_exch{" C6P AMX"}=" PC6 AMX";
$resn_exch{" C5P AMX"}=" PC5 AMX"; $resn_exch{" O5P AMX"}=" PO5 AMX"; $resn_exch{" N4P AMX"}=" PN4 AMX";
$resn_exch{" C3P AMX"}=" PC3 AMX"; $resn_exch{" C2P AMX"}=" PC2 AMX"; $resn_exch{" O5' AN2"}=" O5* AN2";
$resn_exch{" C5' AN2"}=" C5* AN2"; $resn_exch{" C4' AN2"}=" C4* AN2"; $resn_exch{" O4' AN2"}=" O4* AN2";
$resn_exch{" C3' AN2"}=" C3* AN2"; $resn_exch{" O3' AN2"}=" O3* AN2"; $resn_exch{" C2' AN2"}=" C2* AN2";
$resn_exch{" O2' AN2"}=" O2* AN2"; $resn_exch{" C1' AN2"}=" C1* AN2"; $resn_exch{" O5' ANP"}=" O5* ANP";
$resn_exch{" C5' ANP"}=" C5* ANP"; $resn_exch{" C4' ANP"}=" C4* ANP"; $resn_exch{" O4' ANP"}=" O4* ANP";
$resn_exch{" C3' ANP"}=" C3* ANP"; $resn_exch{" O3' ANP"}=" O3* ANP"; $resn_exch{" C2' ANP"}=" C2* ANP";
$resn_exch{" O2' ANP"}=" O2* ANP"; $resn_exch{" C1' ANP"}=" C1* ANP"; $resn_exch{" O3' ANU"}=" O3* ANU";
$resn_exch{" C3' ANU"}=" C3* ANU"; $resn_exch{" C2' ANU"}=" C2* ANU"; $resn_exch{" C1' ANU"}=" C1* ANU";
$resn_exch{" O4' ANU"}=" O4* ANU"; $resn_exch{" C4' ANU"}=" C4* ANU"; $resn_exch{" C5' ANU"}=" C5* ANU";
$resn_exch{" O5' ANU"}=" O5* ANU"; $resn_exch{" O5' ANZ"}=" O5* ANZ"; $resn_exch{" C5' ANZ"}=" C5* ANZ";
$resn_exch{" C4' ANZ"}=" C4* ANZ"; $resn_exch{" O4' ANZ"}=" O4* ANZ"; $resn_exch{" C3' ANZ"}=" C3* ANZ";
$resn_exch{" O3' ANZ"}=" O3* ANZ"; $resn_exch{" C2' ANZ"}=" C2* ANZ"; $resn_exch{" O2' ANZ"}=" O2* ANZ";
$resn_exch{" C1' ANZ"}=" C1* ANZ"; $resn_exch{" O7N AP0"}=" NO7 AP0"; $resn_exch{" C7N AP0"}=" NC7 AP0";
$resn_exch{" C8N AP0"}=" NC8 AP0"; $resn_exch{" C3N AP0"}=" NC3 AP0"; $resn_exch{" C4N AP0"}=" NC4 AP0";
$resn_exch{" C5N AP0"}=" NC5 AP0"; $resn_exch{" C6N AP0"}=" NC6 AP0"; $resn_exch{" C2N AP0"}=" NC2 AP0";
$resn_exch{" N1N AP0"}=" NN1 AP0"; $resn_exch{" C1D AP0"}="NC1* AP0"; $resn_exch{" C2D AP0"}="NC2* AP0";
$resn_exch{" O2D AP0"}="NO2* AP0"; $resn_exch{" C3D AP0"}="NC3* AP0"; $resn_exch{" O3D AP0"}="NO3* AP0";
$resn_exch{" O4D AP0"}="NO4* AP0"; $resn_exch{" C4D AP0"}="NC4* AP0"; $resn_exch{" C5D AP0"}="NC5* AP0";
$resn_exch{" O5D AP0"}="NO5* AP0"; $resn_exch{" PN  AP0"}=" NP  AP0"; $resn_exch{" O1N AP0"}=" NO1 AP0";
$resn_exch{" O2N AP0"}=" NO2 AP0"; $resn_exch{" PA  AP0"}=" AP  AP0"; $resn_exch{" O1A AP0"}=" AO1 AP0";
$resn_exch{" O2A AP0"}=" AO2 AP0"; $resn_exch{" O5B AP0"}="AO5* AP0"; $resn_exch{" C5B AP0"}="AC5* AP0";
$resn_exch{" C4B AP0"}="AC4* AP0"; $resn_exch{" O4B AP0"}="AO4* AP0"; $resn_exch{" C1B AP0"}="AC1* AP0";
$resn_exch{" C2B AP0"}="AC2* AP0"; $resn_exch{" O2B AP0"}="AO2* AP0"; $resn_exch{" C3B AP0"}="AC3* AP0";
$resn_exch{" O3B AP0"}="AO3* AP0"; $resn_exch{" N9A AP0"}=" AN9 AP0"; $resn_exch{" C8A AP0"}=" AC8 AP0";
$resn_exch{" N7A AP0"}=" AN7 AP0"; $resn_exch{" C5A AP0"}=" AC5 AP0"; $resn_exch{" C6A AP0"}=" AC6 AP0";
$resn_exch{" N6A AP0"}=" AN6 AP0"; $resn_exch{" C4A AP0"}=" AC4 AP0"; $resn_exch{" N3A AP0"}=" AN3 AP0";
$resn_exch{" C2A AP0"}=" AC2 AP0"; $resn_exch{" N1A AP0"}=" AN1 AP0"; $resn_exch{" O5' AP2"}=" O5* AP2";
$resn_exch{" C5' AP2"}=" C5* AP2"; $resn_exch{" C4' AP2"}=" C4* AP2"; $resn_exch{" O4' AP2"}=" O4* AP2";
$resn_exch{" C3' AP2"}=" C3* AP2"; $resn_exch{" O3' AP2"}=" O3* AP2"; $resn_exch{" C2' AP2"}=" C2* AP2";
$resn_exch{" O2' AP2"}=" O2* AP2"; $resn_exch{" C1' AP2"}=" C1* AP2"; $resn_exch{" O5F AP5"}="AO5* AP5";
$resn_exch{" C5F AP5"}="AC5* AP5"; $resn_exch{" C4F AP5"}="AC4* AP5"; $resn_exch{" O4F AP5"}="AO4* AP5";
$resn_exch{" C3F AP5"}="AC3* AP5"; $resn_exch{" O3F AP5"}="AO3* AP5"; $resn_exch{" C2F AP5"}="AC2* AP5";
$resn_exch{" O2F AP5"}="AO2* AP5"; $resn_exch{" C1F AP5"}="AC1* AP5"; $resn_exch{" N9A AP5"}=" AN9 AP5";
$resn_exch{" C8A AP5"}=" AC8 AP5"; $resn_exch{" N7A AP5"}=" AN7 AP5"; $resn_exch{" C5A AP5"}=" AC5 AP5";
$resn_exch{" C6A AP5"}=" AC6 AP5"; $resn_exch{" N6A AP5"}=" AN6 AP5"; $resn_exch{" N1A AP5"}=" AN1 AP5";
$resn_exch{" C2A AP5"}=" AC2 AP5"; $resn_exch{" N3A AP5"}=" AN3 AP5"; $resn_exch{" C4A AP5"}=" AC4 AP5";
$resn_exch{" O5J AP5"}="BO5* AP5"; $resn_exch{" C5J AP5"}="BC5* AP5"; $resn_exch{" C4J AP5"}="BC4* AP5";
$resn_exch{" O4J AP5"}="BO4* AP5"; $resn_exch{" C3J AP5"}="BC3* AP5"; $resn_exch{" O3J AP5"}="BO3* AP5";
$resn_exch{" C2J AP5"}="BC2* AP5"; $resn_exch{" O2J AP5"}="BO2* AP5"; $resn_exch{" C1J AP5"}="BC1* AP5";
$resn_exch{" N9B AP5"}=" BN9 AP5"; $resn_exch{" C8B AP5"}=" BC8 AP5"; $resn_exch{" N7B AP5"}=" BN7 AP5";
$resn_exch{" C5B AP5"}=" BC5 AP5"; $resn_exch{" C6B AP5"}=" BC6 AP5"; $resn_exch{" N6B AP5"}=" BN6 AP5";
$resn_exch{" N1B AP5"}=" BN1 AP5"; $resn_exch{" C2B AP5"}=" BC2 AP5"; $resn_exch{" N3B AP5"}=" BN3 AP5";
$resn_exch{" C4B AP5"}=" BC4 AP5"; $resn_exch{" C1' AP7"}=" C1* AP7"; $resn_exch{" C2' AP7"}=" C2* AP7";
$resn_exch{" O2' AP7"}=" O2* AP7"; $resn_exch{" C3' AP7"}=" C3* AP7"; $resn_exch{" O3' AP7"}=" O3* AP7";
$resn_exch{" C4' AP7"}=" C4* AP7"; $resn_exch{" O4' AP7"}=" O4* AP7"; $resn_exch{" C5' AP7"}=" C5* AP7";
$resn_exch{" O5' AP7"}=" O5* AP7"; $resn_exch{" OP1 AP7"}=" O1P AP7"; $resn_exch{" OP2 AP7"}=" O2P AP7";
$resn_exch{" OP3 AP7"}=" O3P AP7"; $resn_exch{" O5' APC"}=" O5* APC"; $resn_exch{" C5' APC"}=" C5* APC";
$resn_exch{" C4' APC"}=" C4* APC"; $resn_exch{" O4' APC"}=" O4* APC"; $resn_exch{" C3' APC"}=" C3* APC";
$resn_exch{" O3' APC"}=" O3* APC"; $resn_exch{" C2' APC"}=" C2* APC"; $resn_exch{" O2' APC"}=" O2* APC";
$resn_exch{" C1' APC"}=" C1* APC"; $resn_exch{" O5' APK"}=" O5* APK"; $resn_exch{" C5' APK"}=" C5* APK";
$resn_exch{" C4' APK"}=" C4* APK"; $resn_exch{" O4' APK"}=" O4* APK"; $resn_exch{" C3' APK"}=" C3* APK";
$resn_exch{" O3' APK"}=" O3* APK"; $resn_exch{" C2' APK"}=" C2* APK"; $resn_exch{" O2' APK"}=" O2* APK";
$resn_exch{" C1' APK"}=" C1* APK"; $resn_exch{" C1' APR"}=" C1* APR"; $resn_exch{" C2' APR"}=" C2* APR";
$resn_exch{" O2' APR"}=" O2* APR"; $resn_exch{" C3' APR"}=" C3* APR"; $resn_exch{" O3' APR"}=" O3* APR";
$resn_exch{" O4' APR"}=" O4* APR"; $resn_exch{" C4' APR"}=" C4* APR"; $resn_exch{" C5' APR"}=" C5* APR";
$resn_exch{" O5' APR"}=" O5* APR"; $resn_exch{" O5D APR"}="RO5* APR"; $resn_exch{" C5D APR"}="RC5* APR";
$resn_exch{" O4D APR"}="RO4* APR"; $resn_exch{" O1D APR"}="RO1* APR"; $resn_exch{" C1D APR"}="RC1* APR";
$resn_exch{" O2D APR"}="RO2* APR"; $resn_exch{" C2D APR"}="RC2* APR"; $resn_exch{" O3D APR"}="RO3* APR";
$resn_exch{" C3D APR"}="RC3* APR"; $resn_exch{" C4D APR"}="RC4* APR"; $resn_exch{" PA  APU"}=" AP  APU";
$resn_exch{" O1A APU"}=" AO1 APU"; $resn_exch{" O2A APU"}=" AO2 APU"; $resn_exch{" O5B APU"}="AO5* APU";
$resn_exch{" C5B APU"}="AC5* APU"; $resn_exch{" C4B APU"}="AC4* APU"; $resn_exch{" O4B APU"}="AO4* APU";
$resn_exch{" C3B APU"}="AC3* APU"; $resn_exch{" O3B APU"}="AO3* APU"; $resn_exch{" C2B APU"}="AC2* APU";
$resn_exch{" O2B APU"}="AO2* APU"; $resn_exch{" C1B APU"}="AC1* APU"; $resn_exch{" N9A APU"}=" AN9 APU";
$resn_exch{" C8A APU"}=" AC8 APU"; $resn_exch{" N7A APU"}=" AN7 APU"; $resn_exch{" C5A APU"}=" AC5 APU";
$resn_exch{" C6A APU"}=" AC6 APU"; $resn_exch{" N6A APU"}=" AN6 APU"; $resn_exch{" N1A APU"}=" AN1 APU";
$resn_exch{" C2A APU"}=" AC2 APU"; $resn_exch{" N3A APU"}=" AN3 APU"; $resn_exch{" C4A APU"}=" AC4 APU";
$resn_exch{" N1U APU"}=" UN1 APU"; $resn_exch{" C2U APU"}=" UC2 APU"; $resn_exch{" O2U APU"}=" UO2 APU";
$resn_exch{" N3U APU"}=" UN3 APU"; $resn_exch{" C4U APU"}=" UC4 APU"; $resn_exch{" O4U APU"}=" UO4 APU";
$resn_exch{" C5U APU"}=" UC5 APU"; $resn_exch{" C6U APU"}=" UC6 APU"; $resn_exch{" C1D APU"}="UC1* APU";
$resn_exch{" C2D APU"}="UC2* APU"; $resn_exch{" O2D APU"}="UO2* APU"; $resn_exch{" C3D APU"}="UC3* APU";
$resn_exch{" O3D APU"}="UO3* APU"; $resn_exch{" C4D APU"}="UC4* APU"; $resn_exch{" O4D APU"}="UO4* APU";
$resn_exch{" O5D APU"}="UO5* APU"; $resn_exch{" C5D APU"}="UC5* APU"; $resn_exch{" PU  APU"}=" UP  APU";
$resn_exch{" O1X APU"}="UO1P APU"; $resn_exch{" O2X APU"}="UO2P APU"; $resn_exch{" O3X APU"}="UO3P APU";
$resn_exch{" O5' APW"}=" O5* APW"; $resn_exch{" C5' APW"}=" C5* APW"; $resn_exch{" C4' APW"}=" C4* APW";
$resn_exch{" C3' APW"}=" C3* APW"; $resn_exch{" O3' APW"}=" O3* APW"; $resn_exch{" C2' APW"}=" C2* APW";
$resn_exch{" O2' APW"}=" O2* APW"; $resn_exch{" C1' APW"}=" C1* APW"; $resn_exch{" O4' APW"}=" O4* APW";
$resn_exch{" C1' AQP"}=" C1* AQP"; $resn_exch{" O4' AQP"}=" O4* AQP"; $resn_exch{" C4' AQP"}=" C4* AQP";
$resn_exch{" C5' AQP"}=" C5* AQP"; $resn_exch{" O5' AQP"}=" O5* AQP"; $resn_exch{" C3' AQP"}=" C3* AQP";
$resn_exch{" O3' AQP"}=" O3* AQP"; $resn_exch{" C2' AQP"}=" C2* AQP"; $resn_exch{" O2' AQP"}=" O2* AQP";
$resn_exch{" OXT AR4"}=" O   AR4"; $resn_exch{" O5' AR4"}=" O5* AR4"; $resn_exch{" C5' AR4"}=" C5* AR4";
$resn_exch{" C4' AR4"}=" C4* AR4"; $resn_exch{" O4' AR4"}=" O4* AR4"; $resn_exch{" C1' AR4"}=" C1* AR4";
$resn_exch{" C2' AR4"}=" C2* AR4"; $resn_exch{" F2' AR4"}=" F2* AR4"; $resn_exch{" C3' AR4"}=" C3* AR4";
$resn_exch{" O3' AR4"}=" O3* AR4"; $resn_exch{" O   AR4"}=" O1  AR4"; $resn_exch{" O2' AR4"}=" O2* AR4";
$resn_exch{" OP1  AS"}=" O1P  AS"; $resn_exch{" OP3  AS"}=" O3P  AS"; $resn_exch{" O5'  AS"}=" O5*  AS";
$resn_exch{" C5'  AS"}=" C5*  AS"; $resn_exch{" C4'  AS"}=" C4*  AS"; $resn_exch{" O4'  AS"}=" O4*  AS";
$resn_exch{" C3'  AS"}=" C3*  AS"; $resn_exch{" O3'  AS"}=" O3*  AS"; $resn_exch{" C2'  AS"}=" C2*  AS";
$resn_exch{" C1'  AS"}=" C1*  AS"; $resn_exch{" OP1 ASU"}=" O1P ASU"; $resn_exch{" OP2 ASU"}=" O2P ASU";
$resn_exch{" OP3 ASU"}=" O3P ASU"; $resn_exch{" O5' ASU"}=" O5* ASU"; $resn_exch{" S4' ASU"}=" S4* ASU";
$resn_exch{" O1' ASU"}=" O1* ASU"; $resn_exch{" C2' ASU"}=" C2* ASU"; $resn_exch{" C5' ASU"}=" C5* ASU";
$resn_exch{" C4' ASU"}=" C4* ASU"; $resn_exch{" C1' ASU"}=" C1* ASU"; $resn_exch{" C3' ASU"}=" C3* ASU";
$resn_exch{" O3' ASU"}=" O3* ASU"; $resn_exch{" XD1 ASX"}=" AD1 ASX"; $resn_exch{" XD2 ASX"}=" AD2 ASX";
$resn_exch{" O5' AT4"}=" O5* AT4"; $resn_exch{" C5' AT4"}=" C5* AT4"; $resn_exch{" C4' AT4"}=" C4* AT4";
$resn_exch{" O4' AT4"}=" O4* AT4"; $resn_exch{" C3' AT4"}=" C3* AT4"; $resn_exch{" O3' AT4"}=" O3* AT4";
$resn_exch{" C2' AT4"}=" C2* AT4"; $resn_exch{" C1' AT4"}=" C1* AT4"; $resn_exch{" O2' AT4"}=" O2* AT4";
$resn_exch{" ND  ATA"}=" N\"  ATA"; $resn_exch{" CAD ATA"}=" CA\" ATA"; $resn_exch{" CD  ATA"}=" C\"  ATA";
$resn_exch{" OD  ATA"}=" O\"  ATA"; $resn_exch{" OT' ATA"}=" OT\" ATA"; $resn_exch{" OP1 ATD"}=" O1P ATD";
$resn_exch{" OP2 ATD"}=" O2P ATD"; $resn_exch{" OP3 ATD"}=" O3P ATD"; $resn_exch{" O5' ATD"}=" O5* ATD";
$resn_exch{" C5' ATD"}=" C5* ATD"; $resn_exch{" C4' ATD"}=" C4* ATD"; $resn_exch{" O4' ATD"}=" O4* ATD";
$resn_exch{" C3' ATD"}=" C3* ATD"; $resn_exch{" O3' ATD"}=" O3* ATD"; $resn_exch{" C2' ATD"}=" C2* ATD";
$resn_exch{" C1' ATD"}=" C1* ATD"; $resn_exch{" O5' ATF"}=" O5* ATF"; $resn_exch{" C5' ATF"}=" C5* ATF";
$resn_exch{" C4' ATF"}=" C4* ATF"; $resn_exch{" O4' ATF"}=" O4* ATF"; $resn_exch{" C3' ATF"}=" C3* ATF";
$resn_exch{" O3' ATF"}=" O3* ATF"; $resn_exch{" C2' ATF"}=" C2* ATF"; $resn_exch{" O2' ATF"}=" O2* ATF";
$resn_exch{" C1' ATF"}=" C1* ATF"; $resn_exch{" O5' ATG"}=" O5* ATG"; $resn_exch{" C5' ATG"}=" C5* ATG";
$resn_exch{" C4' ATG"}=" C4* ATG"; $resn_exch{" O4' ATG"}=" O4* ATG"; $resn_exch{" C3' ATG"}=" C3* ATG";
$resn_exch{" O3' ATG"}=" O3* ATG"; $resn_exch{" C2' ATG"}=" C2* ATG"; $resn_exch{" O2' ATG"}=" O2* ATG";
$resn_exch{" C1' ATG"}=" C1* ATG"; $resn_exch{" OP1 ATL"}=" O1P ATL"; $resn_exch{" OP2 ATL"}=" O2P ATL";
$resn_exch{" O5' ATL"}=" O5* ATL"; $resn_exch{" C5' ATL"}=" C5* ATL"; $resn_exch{" C4' ATL"}=" C4* ATL";
$resn_exch{" O4' ATL"}=" O4* ATL"; $resn_exch{" C3' ATL"}=" C3* ATL"; $resn_exch{" O3' ATL"}=" O3* ATL";
$resn_exch{" C2' ATL"}=" C2* ATL"; $resn_exch{" O2' ATL"}=" O2* ATL"; $resn_exch{" C1' ATL"}=" C1* ATL";
$resn_exch{" C6' ATL"}=" C6* ATL"; $resn_exch{" OP1 ATM"}=" O1P ATM"; $resn_exch{" OP2 ATM"}=" O2P ATM";
$resn_exch{" OP3 ATM"}=" O3P ATM"; $resn_exch{" O5' ATM"}=" O5* ATM"; $resn_exch{" C5' ATM"}=" C5* ATM";
$resn_exch{" C4' ATM"}=" C4* ATM"; $resn_exch{" O4' ATM"}=" O4* ATM"; $resn_exch{" C3' ATM"}=" C3* ATM";
$resn_exch{" N3' ATM"}=" N3* ATM"; $resn_exch{" N4' ATM"}=" N4* ATM"; $resn_exch{" N5' ATM"}=" N5* ATM";
$resn_exch{" C2' ATM"}=" C2* ATM"; $resn_exch{" C1' ATM"}=" C1* ATM"; $resn_exch{" O5' ATP"}=" O5* ATP";
$resn_exch{" C5' ATP"}=" C5* ATP"; $resn_exch{" C4' ATP"}=" C4* ATP"; $resn_exch{" O4' ATP"}=" O4* ATP";
$resn_exch{" C3' ATP"}=" C3* ATP"; $resn_exch{" O3' ATP"}=" O3* ATP"; $resn_exch{" C2' ATP"}=" C2* ATP";
$resn_exch{" O2' ATP"}=" O2* ATP"; $resn_exch{" C1' ATP"}=" C1* ATP"; $resn_exch{" O5' ATR"}=" O5* ATR";
$resn_exch{" C5' ATR"}=" C5* ATR"; $resn_exch{" C4' ATR"}=" C4* ATR"; $resn_exch{" O4' ATR"}=" O4* ATR";
$resn_exch{" C3' ATR"}=" C3* ATR"; $resn_exch{" O3' ATR"}=" O3* ATR"; $resn_exch{" C2' ATR"}=" C2* ATR";
$resn_exch{" O2' ATR"}=" O2* ATR"; $resn_exch{" P2' ATR"}=" P2* ATR"; $resn_exch{" C1' ATR"}=" C1* ATR";
$resn_exch{" O5' ATY"}=" O5* ATY"; $resn_exch{" C5' ATY"}=" C5* ATY"; $resn_exch{" C4' ATY"}=" C4* ATY";
$resn_exch{" O4' ATY"}=" O4* ATY"; $resn_exch{" C3' ATY"}=" C3* ATY"; $resn_exch{" O3' ATY"}=" O3* ATY";
$resn_exch{" C2' ATY"}=" C2* ATY"; $resn_exch{" C1' ATY"}=" C1* ATY"; $resn_exch{" O5' AV2"}=" O5* AV2";
$resn_exch{" C5' AV2"}=" C5* AV2"; $resn_exch{" C4' AV2"}=" C4* AV2"; $resn_exch{" O4' AV2"}=" O4* AV2";
$resn_exch{" C3' AV2"}=" C3* AV2"; $resn_exch{" O3' AV2"}=" O3* AV2"; $resn_exch{" C2' AV2"}=" C2* AV2";
$resn_exch{" O2' AV2"}=" O2* AV2"; $resn_exch{" C1' AV2"}=" C1* AV2"; $resn_exch{" OP3 AVC"}=" O3P AVC";
$resn_exch{" OP1 AVC"}=" O1P AVC"; $resn_exch{" OP2 AVC"}=" O2P AVC"; $resn_exch{" O5' AVC"}=" O5* AVC";
$resn_exch{" C5' AVC"}=" C5* AVC"; $resn_exch{" C4' AVC"}=" C4* AVC"; $resn_exch{" O4' AVC"}=" O4* AVC";
$resn_exch{" C3' AVC"}=" C3* AVC"; $resn_exch{" O3' AVC"}=" O3* AVC"; $resn_exch{" C2' AVC"}=" C2* AVC";
$resn_exch{" O2' AVC"}=" O2* AVC"; $resn_exch{" C1' AVC"}=" C1* AVC"; $resn_exch{" O5' AZD"}=" O5* AZD";
$resn_exch{" C5' AZD"}=" C5* AZD"; $resn_exch{" C4' AZD"}=" C4* AZD"; $resn_exch{" O4' AZD"}=" O4* AZD";
$resn_exch{" C3' AZD"}=" C3* AZD"; $resn_exch{" N3' AZD"}=" N3* AZD"; $resn_exch{" N4' AZD"}=" N4* AZD";
$resn_exch{" N5' AZD"}=" N5* AZD"; $resn_exch{" C2' AZD"}=" C2* AZD"; $resn_exch{" C1' AZD"}=" C1* AZD";
$resn_exch{" O5' AZT"}=" O5* AZT"; $resn_exch{" C5' AZT"}=" C5* AZT"; $resn_exch{" C4' AZT"}=" C4* AZT";
$resn_exch{" O4' AZT"}=" O4* AZT"; $resn_exch{" C3' AZT"}=" C3* AZT"; $resn_exch{" N3' AZT"}=" N3* AZT";
$resn_exch{" N4' AZT"}=" N4* AZT"; $resn_exch{" N5' AZT"}=" N5* AZT"; $resn_exch{" C2' AZT"}=" C2* AZT";
$resn_exch{" C1' AZT"}=" C1* AZT"; $resn_exch{" OP3 B1P"}=" O3P B1P"; $resn_exch{" OP1 B1P"}=" O1P B1P";
$resn_exch{" OP2 B1P"}=" O2P B1P"; $resn_exch{" O5' B1P"}=" O5* B1P"; $resn_exch{" C5' B1P"}=" C5* B1P";
$resn_exch{" C4' B1P"}=" C4* B1P"; $resn_exch{" O4' B1P"}=" O4* B1P"; $resn_exch{" C1' B1P"}=" C1* B1P";
$resn_exch{" O1' B1P"}=" O1* B1P"; $resn_exch{" C2' B1P"}=" C2* B1P"; $resn_exch{" C3' B1P"}=" C3* B1P";
$resn_exch{" O3' B1P"}=" O3* B1P"; $resn_exch{" O5E B4P"}="AO5* B4P"; $resn_exch{" C5E B4P"}="AC5* B4P";
$resn_exch{" C4E B4P"}="AC4* B4P"; $resn_exch{" O4E B4P"}="AO4* B4P"; $resn_exch{" C3E B4P"}="AC3* B4P";
$resn_exch{" O3E B4P"}="AO3* B4P"; $resn_exch{" C2E B4P"}="AC2* B4P"; $resn_exch{" O2E B4P"}="AO2* B4P";
$resn_exch{" C1E B4P"}="AC1* B4P"; $resn_exch{" N9A B4P"}=" AN9 B4P"; $resn_exch{" C8A B4P"}=" AC8 B4P";
$resn_exch{" N7A B4P"}=" AN7 B4P"; $resn_exch{" C5A B4P"}=" AC5 B4P"; $resn_exch{" C6A B4P"}=" AC6 B4P";
$resn_exch{" N6A B4P"}=" AN6 B4P"; $resn_exch{" N1A B4P"}=" AN1 B4P"; $resn_exch{" C2A B4P"}=" AC2 B4P";
$resn_exch{" N3A B4P"}=" AN3 B4P"; $resn_exch{" C4A B4P"}=" AC4 B4P"; $resn_exch{" O5F B4P"}="BO5* B4P";
$resn_exch{" C5F B4P"}="BC5* B4P"; $resn_exch{" C4F B4P"}="BC4* B4P"; $resn_exch{" O4F B4P"}="BO4* B4P";
$resn_exch{" C3F B4P"}="BC3* B4P"; $resn_exch{" O3F B4P"}="BO3* B4P"; $resn_exch{" C2F B4P"}="BC2* B4P";
$resn_exch{" O2F B4P"}="BO2* B4P"; $resn_exch{" C1F B4P"}="BC1* B4P"; $resn_exch{" N9B B4P"}=" BN9 B4P";
$resn_exch{" C8B B4P"}=" BC8 B4P"; $resn_exch{" N7B B4P"}=" BN7 B4P"; $resn_exch{" C5B B4P"}=" BC5 B4P";
$resn_exch{" C6B B4P"}=" BC6 B4P"; $resn_exch{" N6B B4P"}=" BN6 B4P"; $resn_exch{" N1B B4P"}=" BN1 B4P";
$resn_exch{" C2B B4P"}=" BC2 B4P"; $resn_exch{" N3B B4P"}=" BN3 B4P"; $resn_exch{" C4B B4P"}=" BC4 B4P";
$resn_exch{" C1A B9A"}=" AC1 B9A"; $resn_exch{" C2A B9A"}=" AC2 B9A"; $resn_exch{" C3A B9A"}=" AC3 B9A";
$resn_exch{" C4A B9A"}=" AC4 B9A"; $resn_exch{" C5A B9A"}=" AC5 B9A"; $resn_exch{" C6A B9A"}=" AC6 B9A";
$resn_exch{" C7A B9A"}=" AC7 B9A"; $resn_exch{" C8A B9A"}=" AC8 B9A"; $resn_exch{" C9A B9A"}=" AC9 B9A";
$resn_exch{" N9A B9A"}=" AN9 B9A"; $resn_exch{" NAA B9A"}="AN10 B9A"; $resn_exch{" CBA B9A"}="AC11 B9A";
$resn_exch{" CCA B9A"}="AC12 B9A"; $resn_exch{" CDA B9A"}="AC13 B9A"; $resn_exch{" CEA B9A"}="AC14 B9A";
$resn_exch{" C1B B9A"}=" BC1 B9A"; $resn_exch{" C2B B9A"}=" BC2 B9A"; $resn_exch{" C3B B9A"}=" BC3 B9A";
$resn_exch{" C4B B9A"}=" BC4 B9A"; $resn_exch{" C5B B9A"}=" BC5 B9A"; $resn_exch{" C6B B9A"}=" BC6 B9A";
$resn_exch{" C7B B9A"}=" BC7 B9A"; $resn_exch{" C8B B9A"}=" BC8 B9A"; $resn_exch{" C9B B9A"}=" BC9 B9A";
$resn_exch{" NAB B9A"}="BN10 B9A"; $resn_exch{" CBB B9A"}="BC11 B9A"; $resn_exch{" CCB B9A"}="BC12 B9A";
$resn_exch{" CDB B9A"}="BC13 B9A"; $resn_exch{" CEB B9A"}="BC14 B9A"; $resn_exch{" N9B B9A"}=" BN9 B9A";
$resn_exch{" OAC BA1"}="2O1A BA1"; $resn_exch{" C1B BA1"}=" C1\" BA1"; $resn_exch{" OBC BA1"}="2O1B BA1";
$resn_exch{" C2B BA1"}=" C2\" BA1"; $resn_exch{" C3B BA1"}=" C3\" BA1"; $resn_exch{" C4B BA1"}=" C4\" BA1";
$resn_exch{" O4' BA1"}=" O4\" BA1"; $resn_exch{" C5B BA1"}=" C5\" BA1"; $resn_exch{" C6B BA1"}=" C6\" BA1";
$resn_exch{" O6' BA1"}=" O6\" BA1"; $resn_exch{" C7B BA1"}=" C7\" BA1"; $resn_exch{" C8' BA1"}=" C8\" BA1";
$resn_exch{" O8' BA1"}=" O8\" BA1"; $resn_exch{" C9' BA1"}=" C9\" BA1"; $resn_exch{" CXU BA1"}="2C10 BA1";
$resn_exch{" OXU BA1"}="2O10 BA1"; $resn_exch{" CX4 BA1"}="2C11 BA1"; $resn_exch{" CXE BA1"}="2C12 BA1";
$resn_exch{" CXO BA1"}="2C13 BA1"; $resn_exch{" CXY BA1"}="2C14 BA1"; $resn_exch{" CX8 BA1"}="2C15 BA1";
$resn_exch{" OX8 BA1"}="2O15 BA1"; $resn_exch{" OXI BA1"}="2O16 BA1"; $resn_exch{" C1' BA2"}=" C1* BA2";
$resn_exch{" C2' BA2"}=" C2* BA2"; $resn_exch{" O2' BA2"}=" O2* BA2"; $resn_exch{" C3' BA2"}=" C3* BA2";
$resn_exch{" O3' BA2"}=" O3* BA2"; $resn_exch{" C4' BA2"}=" C4* BA2"; $resn_exch{" O4' BA2"}=" O4* BA2";
$resn_exch{" C5' BA2"}=" C5* BA2"; $resn_exch{" CAA  BC"}="C10A  BC"; $resn_exch{" CEA  BC"}="C14A  BC";
$resn_exch{" CEB  BC"}="C14B  BC"; $resn_exch{" CEC  BC"}="C14C  BC"; $resn_exch{" N1A BCA"}=" AN1 BCA";
$resn_exch{" C2A BCA"}=" AC2 BCA"; $resn_exch{" N3A BCA"}=" AN3 BCA"; $resn_exch{" C4A BCA"}=" AC4 BCA";
$resn_exch{" C5A BCA"}=" AC5 BCA"; $resn_exch{" C6A BCA"}=" AC6 BCA"; $resn_exch{" N6A BCA"}=" AN6 BCA";
$resn_exch{" N7A BCA"}=" AN7 BCA"; $resn_exch{" C8A BCA"}=" AC8 BCA"; $resn_exch{" N9A BCA"}=" AN9 BCA";
$resn_exch{" C1D BCA"}="AC1* BCA"; $resn_exch{" C2D BCA"}="AC2* BCA"; $resn_exch{" O2D BCA"}="AO2* BCA";
$resn_exch{" C3D BCA"}="AC3* BCA"; $resn_exch{" O3D BCA"}="AO3* BCA"; $resn_exch{" P3D BCA"}="AP3* BCA";
$resn_exch{" O7A BCA"}=" AO7 BCA"; $resn_exch{" O8A BCA"}=" AO8 BCA"; $resn_exch{" O9A BCA"}=" AO9 BCA";
$resn_exch{" C4D BCA"}="AC4* BCA"; $resn_exch{" O4D BCA"}="AO4* BCA"; $resn_exch{" C5D BCA"}="AC5* BCA";
$resn_exch{" O5D BCA"}="AO5* BCA"; $resn_exch{" P1A BCA"}=" AP1 BCA"; $resn_exch{" O1A BCA"}=" AO1 BCA";
$resn_exch{" O2A BCA"}=" AO2 BCA"; $resn_exch{" O3A BCA"}=" AO3 BCA"; $resn_exch{" P2A BCA"}=" AP2 BCA";
$resn_exch{" O4A BCA"}=" AO4 BCA"; $resn_exch{" O5A BCA"}=" AO5 BCA"; $resn_exch{" O6A BCA"}=" AO6 BCA";
$resn_exch{" CBP BCA"}="PC11 BCA"; $resn_exch{" CCP BCA"}="PC12 BCA"; $resn_exch{" CDP BCA"}="PC13 BCA";
$resn_exch{" CEP BCA"}="PC14 BCA"; $resn_exch{" CAP BCA"}="PC10 BCA"; $resn_exch{" OAP BCA"}="PO10 BCA";
$resn_exch{" C9P BCA"}=" PC9 BCA"; $resn_exch{" O9P BCA"}=" PO9 BCA"; $resn_exch{" N8P BCA"}=" PN8 BCA";
$resn_exch{" C7P BCA"}=" PC7 BCA"; $resn_exch{" C6P BCA"}=" PC6 BCA"; $resn_exch{" C5P BCA"}=" PC5 BCA";
$resn_exch{" O5P BCA"}=" PO5 BCA"; $resn_exch{" N4P BCA"}=" PN4 BCA"; $resn_exch{" C3P BCA"}=" PC3 BCA";
$resn_exch{" C2P BCA"}=" PC2 BCA"; $resn_exch{" S1P BCA"}=" PS1 BCA"; $resn_exch{" C1B BCA"}=" BC1 BCA";
$resn_exch{" O1B BCA"}=" BO1 BCA"; $resn_exch{" C2B BCA"}=" BC2 BCA"; $resn_exch{" C3B BCA"}=" BC3 BCA";
$resn_exch{" C4B BCA"}=" BC4 BCA"; $resn_exch{" C5B BCA"}=" BC5 BCA"; $resn_exch{" O2B BCA"}=" BO2 BCA";
$resn_exch{" C6B BCA"}=" BC6 BCA"; $resn_exch{" C7B BCA"}=" BC7 BCA"; $resn_exch{" NA  BCB"}=" N_A BCB";
$resn_exch{" NB  BCB"}=" N_B BCB"; $resn_exch{" NC  BCB"}=" N_C BCB"; $resn_exch{" ND  BCB"}=" N_D BCB";
$resn_exch{" NA  BCL"}=" N_A BCL"; $resn_exch{" NB  BCL"}=" N_B BCL"; $resn_exch{" NC  BCL"}=" N_C BCL";
$resn_exch{" ND  BCL"}=" N_D BCL"; $resn_exch{" C1' BDA"}=" C1* BDA"; $resn_exch{" C2' BDA"}=" C2* BDA";
$resn_exch{" C3' BDA"}=" C3* BDA"; $resn_exch{" N3' BDA"}=" N3* BDA"; $resn_exch{" C4' BDA"}=" C4* BDA";
$resn_exch{" O4' BDA"}=" O4* BDA"; $resn_exch{" C5' BDA"}=" C5* BDA"; $resn_exch{" O5' BDA"}=" O5* BDA";
$resn_exch{" C6' BDA"}=" C6* BDA"; $resn_exch{" C7' BDA"}=" C7* BDA"; $resn_exch{" C8' BDA"}=" C8* BDA";
$resn_exch{" C1B BDA"}="'C1  BDA"; $resn_exch{" C2B BDA"}="'C2  BDA"; $resn_exch{" C3B BDA"}="'C3  BDA";
$resn_exch{" C4B BDA"}="'C4  BDA"; $resn_exch{" O4B BDA"}="'O4  BDA"; $resn_exch{" C5B BDA"}="'C5  BDA";
$resn_exch{" C6B BDA"}="'C6  BDA"; $resn_exch{" O6' BDA"}="'O6  BDA"; $resn_exch{" C7B BDA"}="'C7  BDA";
$resn_exch{" C8B BDA"}="'C8  BDA"; $resn_exch{" O8' BDA"}="'O8  BDA"; $resn_exch{" C9' BDA"}="'C9  BDA";
$resn_exch{" CA' BDA"}="'C10 BDA"; $resn_exch{" OA' BDA"}="'O10 BDA"; $resn_exch{" CB' BDA"}="'C11 BDA";
$resn_exch{" CC' BDA"}="'C12 BDA"; $resn_exch{" OC' BDA"}="'O12 BDA"; $resn_exch{" CD' BDA"}="'C13 BDA";
$resn_exch{" OD' BDA"}="'O13 BDA"; $resn_exch{" CE' BDA"}="'C14 BDA"; $resn_exch{" CF' BDA"}="'C15 BDA";
$resn_exch{" CG' BDA"}="'C16 BDA"; $resn_exch{" CH' BDA"}="'C17 BDA"; $resn_exch{" OH' BDA"}="'O17 BDA";
$resn_exch{" CI' BDA"}="'C18 BDA"; $resn_exch{" CJ' BDA"}="'C19 BDA"; $resn_exch{" OJ' BDA"}="'O19 BDA";
$resn_exch{" CK' BDA"}="'C20 BDA"; $resn_exch{" CL' BDA"}="'C21 BDA"; $resn_exch{" C1D BDA"}="'C1* BDA";
$resn_exch{" C2D BDA"}="'C2* BDA"; $resn_exch{" C3D BDA"}="'C3* BDA"; $resn_exch{" N3D BDA"}="'N3* BDA";
$resn_exch{" C4D BDA"}="'C4* BDA"; $resn_exch{" O4D BDA"}="'O4* BDA"; $resn_exch{" C5D BDA"}="'C5* BDA";
$resn_exch{" O5D BDA"}="'O5* BDA"; $resn_exch{" C6D BDA"}="'C6* BDA"; $resn_exch{" C7D BDA"}="'C7* BDA";
$resn_exch{" C8D BDA"}="'C8* BDA"; $resn_exch{" C1A BDA"}="'C9A BDA"; $resn_exch{" C1C BDA"}="'C9B BDA";
$resn_exch{" CF4 BDC"}="'CE4 BDC"; $resn_exch{" CF3 BDC"}="'CE3 BDC"; $resn_exch{" CF2 BDC"}="'CE2 BDC";
$resn_exch{" CF1 BDC"}="'CE1 BDC"; $resn_exch{" N9' BDC"}="'N9  BDC"; $resn_exch{" C5' BDC"}="'C5  BDC";
$resn_exch{" C6' BDC"}="'C6  BDC"; $resn_exch{" C7' BDC"}="'C7  BDC"; $resn_exch{" C8' BDC"}="'C8  BDC";
$resn_exch{" C9' BDC"}="'C9  BDC"; $resn_exch{" C1' BDC"}="'C1  BDC"; $resn_exch{" C2' BDC"}="'C2  BDC";
$resn_exch{" C3' BDC"}="'C3  BDC"; $resn_exch{" C4' BDC"}="'C4  BDC"; $resn_exch{" CE' BDC"}="'C14 BDC";
$resn_exch{" CD' BDC"}="'C13 BDC"; $resn_exch{" CB' BDC"}="'C11 BDC"; $resn_exch{" CC' BDC"}="'C12 BDC";
$resn_exch{" NA' BDC"}="'N10 BDC"; $resn_exch{" CB1 BDC"}="'CD1 BDC"; $resn_exch{" O1X BDC"}="'OD1 BDC";
$resn_exch{" NB1 BDC"}="'ND1 BDC"; $resn_exch{" CB2 BDC"}="'CD2 BDC"; $resn_exch{" CB3 BDC"}="'CD3 BDC";
$resn_exch{" NB2 BDC"}="'ND2 BDC"; $resn_exch{" CB7 BDC"}="'CD7 BDC"; $resn_exch{" CB8 BDC"}="'CD8 BDC";
$resn_exch{" OXV BDM"}="1O21 BDM"; $resn_exch{" OXW BDM"}="2O21 BDM"; $resn_exch{" OX5 BDM"}="1O22 BDM";
$resn_exch{" OX6 BDM"}="2O22 BDM"; $resn_exch{" OP1 BGM"}=" O1P BGM"; $resn_exch{" OP2 BGM"}=" O2P BGM";
$resn_exch{" O5' BGM"}=" O5* BGM"; $resn_exch{" C5' BGM"}=" C5* BGM"; $resn_exch{" C4' BGM"}=" C4* BGM";
$resn_exch{" O4' BGM"}=" O4* BGM"; $resn_exch{" C1' BGM"}=" C1* BGM"; $resn_exch{" C2' BGM"}=" C2* BGM";
$resn_exch{" C3' BGM"}=" C3* BGM"; $resn_exch{" O3' BGM"}=" O3* BGM"; $resn_exch{" OP3 BGM"}=" O3P BGM";
$resn_exch{" C2' BGS"}=" C2* BGS"; $resn_exch{" O2' BGS"}=" O2* BGS"; $resn_exch{" O1' BGS"}=" O1* BGS";
$resn_exch{" O5' BIS"}=" O5* BIS"; $resn_exch{" C5' BIS"}=" C5* BIS"; $resn_exch{" C4' BIS"}=" C4* BIS";
$resn_exch{" O4' BIS"}=" O4* BIS"; $resn_exch{" C3' BIS"}=" C3* BIS"; $resn_exch{" O3' BIS"}=" O3* BIS";
$resn_exch{" C2' BIS"}=" C2* BIS"; $resn_exch{" O2' BIS"}=" O2* BIS"; $resn_exch{" C1' BIS"}=" C1* BIS";
$resn_exch{" N1' BIZ"}=" *N1 BIZ"; $resn_exch{" C2' BIZ"}=" *C2 BIZ"; $resn_exch{" C3' BIZ"}=" *C3 BIZ";
$resn_exch{" C4' BIZ"}=" *C4 BIZ"; $resn_exch{" C5' BIZ"}=" *C5 BIZ"; $resn_exch{" C6' BIZ"}=" *C6 BIZ";
$resn_exch{" C7' BIZ"}=" *C7 BIZ"; $resn_exch{" N8' BIZ"}=" *N8 BIZ"; $resn_exch{" C9' BIZ"}=" *C9 BIZ";
$resn_exch{" CA' BIZ"}="*C10 BIZ"; $resn_exch{" CB' BIZ"}="*C11 BIZ"; $resn_exch{" OC' BIZ"}="*O12 BIZ";
$resn_exch{" ND' BIZ"}="*N13 BIZ"; $resn_exch{" CE' BIZ"}="*C14 BIZ"; $resn_exch{" CF' BIZ"}="*C15 BIZ";
$resn_exch{" CG' BIZ"}="*C16 BIZ"; $resn_exch{" CH' BIZ"}="*C17 BIZ"; $resn_exch{" CI' BIZ"}="*C18 BIZ";
$resn_exch{" CJ' BIZ"}="*C19 BIZ"; $resn_exch{" CK' BIZ"}="*C20 BIZ"; $resn_exch{" OL' BIZ"}="*O21 BIZ";
$resn_exch{" CM' BIZ"}="*C22 BIZ"; $resn_exch{" CN' BIZ"}="*C23 BIZ"; $resn_exch{" CO' BIZ"}="*C24 BIZ";
$resn_exch{" CP' BIZ"}="*C25 BIZ"; $resn_exch{" CQ' BIZ"}="*C26 BIZ"; $resn_exch{" NR' BIZ"}="*N27 BIZ";
$resn_exch{" NA  BLA"}=" N_A BLA"; $resn_exch{" NB  BLA"}=" N_B BLA"; $resn_exch{" OB  BLA"}=" O_B BLA";
$resn_exch{" NC  BLA"}=" N_C BLA"; $resn_exch{" OC  BLA"}=" O_C BLA"; $resn_exch{" ND  BLA"}=" N_D BLA";
$resn_exch{" C6B BLI"}=" C6* BLI"; $resn_exch{" C1B BLI"}=" C1* BLI"; $resn_exch{" C2B BLI"}=" C2* BLI";
$resn_exch{" C3B BLI"}=" C3* BLI"; $resn_exch{" C4B BLI"}=" C4* BLI"; $resn_exch{" C5B BLI"}=" C5* BLI";
$resn_exch{" C1' BLS"}=" C1* BLS"; $resn_exch{" C2' BLS"}=" C2* BLS"; $resn_exch{" C3' BLS"}=" C3* BLS";
$resn_exch{" C4' BLS"}=" C4* BLS"; $resn_exch{" C5' BLS"}=" C5* BLS"; $resn_exch{" O5' BLS"}=" O5* BLS";
$resn_exch{" C6' BLS"}=" C6* BLS"; $resn_exch{" NA  BLV"}=" N_A BLV"; $resn_exch{" OA  BLV"}=" O_A BLV";
$resn_exch{" NB  BLV"}=" N_B BLV"; $resn_exch{" NC  BLV"}=" N_C BLV"; $resn_exch{" ND  BLV"}=" N_D BLV";
$resn_exch{" OD  BLV"}=" O_D BLV"; $resn_exch{" OP1 BMP"}=" O1P BMP"; $resn_exch{" OP2 BMP"}=" O2P BMP";
$resn_exch{" OP3 BMP"}=" O3P BMP"; $resn_exch{" O5' BMP"}=" O5* BMP"; $resn_exch{" C5' BMP"}=" C5* BMP";
$resn_exch{" C4' BMP"}=" C4* BMP"; $resn_exch{" O4' BMP"}=" O4* BMP"; $resn_exch{" C3' BMP"}=" C3* BMP";
$resn_exch{" O3' BMP"}=" O3* BMP"; $resn_exch{" C2' BMP"}=" C2* BMP"; $resn_exch{" O2' BMP"}=" O2* BMP";
$resn_exch{" C1' BMP"}=" C1* BMP"; $resn_exch{" O5' BMQ"}=" O5* BMQ"; $resn_exch{" C5' BMQ"}=" C5* BMQ";
$resn_exch{" C4' BMQ"}=" C4* BMQ"; $resn_exch{" O4' BMQ"}=" O4* BMQ"; $resn_exch{" C3' BMQ"}=" C3* BMQ";
$resn_exch{" O3' BMQ"}=" O3* BMQ"; $resn_exch{" C2' BMQ"}=" C2* BMQ"; $resn_exch{" O2' BMQ"}=" O2* BMQ";
$resn_exch{" C1' BMQ"}=" C1* BMQ"; $resn_exch{" O1' BMY"}=" O1\" BMY"; $resn_exch{" C2B BMY"}=" C2\" BMY";
$resn_exch{" C3B BMY"}=" C3\" BMY"; $resn_exch{" C4B BMY"}=" C4\" BMY"; $resn_exch{" C5B BMY"}=" C5\" BMY";
$resn_exch{" C6' BMY"}=" C6\" BMY"; $resn_exch{" C7B BMY"}=" C7\" BMY"; $resn_exch{" O8' BMY"}=" O8\" BMY";
$resn_exch{" O9B BMY"}=" O9\" BMY"; $resn_exch{" C11 BNA"}="AC4A BNA"; $resn_exch{" C12 BNA"}="AC8A BNA";
$resn_exch{" C13 BNA"}="AC4B BNA"; $resn_exch{" C14 BNA"}="AC8B BNA"; $resn_exch{" C1' BNR"}=" C1* BNR";
$resn_exch{" C2' BNR"}=" C2* BNR"; $resn_exch{" C3' BNR"}=" C3* BNR"; $resn_exch{" O3' BNR"}=" O3* BNR";
$resn_exch{" C4' BNR"}=" C4* BNR"; $resn_exch{" N4' BNR"}=" N4* BNR"; $resn_exch{" C5' BNR"}=" C5* BNR";
$resn_exch{" O5' BNR"}=" O5* BNR"; $resn_exch{" C6' BNR"}=" C6* BNR"; $resn_exch{" C7' BNR"}=" C7* BNR";
$resn_exch{" C8' BNR"}=" C8* BNR"; $resn_exch{" C1B BNR"}="'C1  BNR"; $resn_exch{" C2B BNR"}="'C2  BNR";
$resn_exch{" C3B BNR"}="'C3  BNR"; $resn_exch{" C4B BNR"}="'C4  BNR"; $resn_exch{" O4' BNR"}="'O4  BNR";
$resn_exch{" C5B BNR"}="'C5  BNR"; $resn_exch{" C6B BNR"}="'C6  BNR"; $resn_exch{" O6' BNR"}="'O6  BNR";
$resn_exch{" C7B BNR"}="'C7  BNR"; $resn_exch{" C8B BNR"}="'C8  BNR"; $resn_exch{" O8' BNR"}="'O8  BNR";
$resn_exch{" C9' BNR"}="'C9  BNR"; $resn_exch{" CA' BNR"}="'C10 BNR"; $resn_exch{" OA' BNR"}="'O10 BNR";
$resn_exch{" CB' BNR"}="'C11 BNR"; $resn_exch{" CC' BNR"}="'C12 BNR"; $resn_exch{" OC' BNR"}="'O12 BNR";
$resn_exch{" CD' BNR"}="'C13 BNR"; $resn_exch{" OD' BNR"}="'O13 BNR"; $resn_exch{" CE' BNR"}="'C14 BNR";
$resn_exch{" CF' BNR"}="'C15 BNR"; $resn_exch{" CG' BNR"}="'C16 BNR"; $resn_exch{" CH' BNR"}="'C17 BNR";
$resn_exch{" OH' BNR"}="'O17 BNR"; $resn_exch{" CI' BNR"}="'C18 BNR"; $resn_exch{" CJ' BNR"}="'C19 BNR";
$resn_exch{" OJ' BNR"}="'O19 BNR"; $resn_exch{" CK' BNR"}="'C20 BNR"; $resn_exch{" CL' BNR"}="'C21 BNR";
$resn_exch{" C1D BNR"}="'C1* BNR"; $resn_exch{" C2D BNR"}="'C2* BNR"; $resn_exch{" C3D BNR"}="'C3* BNR";
$resn_exch{" O3D BNR"}="'O3* BNR"; $resn_exch{" C4D BNR"}="'C4* BNR"; $resn_exch{" N4D BNR"}="'N4* BNR";
$resn_exch{" C5D BNR"}="'C5* BNR"; $resn_exch{" O5D BNR"}="'O5* BNR"; $resn_exch{" C6D BNR"}="'C6* BNR";
$resn_exch{" C7D BNR"}="'C7* BNR"; $resn_exch{" C8D BNR"}="'C8* BNR"; $resn_exch{" C1A BNR"}="'C9A BNR";
$resn_exch{" C1C BNR"}="'C9B BNR"; $resn_exch{" OP1 BOE"}=" O1P BOE"; $resn_exch{" OP2 BOE"}=" O2P BOE";
$resn_exch{" O5' BOE"}=" O5* BOE"; $resn_exch{" C5' BOE"}=" C5* BOE"; $resn_exch{" C4' BOE"}=" C4* BOE";
$resn_exch{" O4' BOE"}=" O4* BOE"; $resn_exch{" C1' BOE"}=" C1* BOE"; $resn_exch{" C2' BOE"}=" C2* BOE";
$resn_exch{" O2' BOE"}=" O2* BOE"; $resn_exch{" C3' BOE"}=" C3* BOE"; $resn_exch{" O3' BOE"}=" O3* BOE";
$resn_exch{" OP3 BOE"}=" O3P BOE"; $resn_exch{" NA  BPB"}=" N_A BPB"; $resn_exch{" NB  BPB"}=" N_B BPB";
$resn_exch{" NC  BPB"}=" N_C BPB"; $resn_exch{" ND  BPB"}=" N_D BPB"; $resn_exch{" C3' BPG"}=" C3* BPG";
$resn_exch{" O3' BPG"}=" O3* BPG"; $resn_exch{" C2' BPG"}=" C2* BPG"; $resn_exch{" C1' BPG"}=" C1* BPG";
$resn_exch{" NA  BPH"}=" N_A BPH"; $resn_exch{" NB  BPH"}=" N_B BPH"; $resn_exch{" NC  BPH"}=" N_C BPH";
$resn_exch{" ND  BPH"}=" N_D BPH"; $resn_exch{" CXA BPS"}="CX10 BPS"; $resn_exch{" CXB BPS"}="CX11 BPS";
$resn_exch{" CXC BPS"}="CX12 BPS"; $resn_exch{" CKA BPY"}="CK10 BPY"; $resn_exch{" CKB BPY"}="CK11 BPY";
$resn_exch{" CKC BPY"}="CK12 BPY"; $resn_exch{" C1' BRU"}=" C1* BRU"; $resn_exch{" C2' BRU"}=" C2* BRU";
$resn_exch{" C3' BRU"}=" C3* BRU"; $resn_exch{" C4' BRU"}=" C4* BRU"; $resn_exch{" O3' BRU"}=" O3* BRU";
$resn_exch{" O4' BRU"}=" O4* BRU"; $resn_exch{" C5' BRU"}=" C5* BRU"; $resn_exch{" O5' BRU"}=" O5* BRU";
$resn_exch{" OP1 BRU"}=" O1P BRU"; $resn_exch{" OP2 BRU"}=" O2P BRU"; $resn_exch{" OP3 BRU"}=" O3P BRU";
$resn_exch{" C1B BSP"}=" C1* BSP"; $resn_exch{" C2B BSP"}=" C2* BSP"; $resn_exch{" C3B BSP"}=" C3* BSP";
$resn_exch{" C4B BSP"}=" C4* BSP"; $resn_exch{" C5B BSP"}=" C5* BSP"; $resn_exch{" C6B BSP"}=" C6* BSP";
$resn_exch{" O4B BSP"}=" O4* BSP"; $resn_exch{" S1B BSP"}=" S1* BSP"; $resn_exch{" CBB BT5"}="C11B BT5";
$resn_exch{" OBB BT5"}="O11B BT5"; $resn_exch{" OCB BT5"}="O12B BT5"; $resn_exch{" CAB BT5"}="C10B BT5";
$resn_exch{" OP1 BT5"}=" O1P BT5"; $resn_exch{" OP2 BT5"}=" O2P BT5"; $resn_exch{" O5' BT5"}=" O5* BT5";
$resn_exch{" C5' BT5"}=" C5* BT5"; $resn_exch{" C4' BT5"}=" C4* BT5"; $resn_exch{" O4' BT5"}=" O4* BT5";
$resn_exch{" C3' BT5"}=" C3* BT5"; $resn_exch{" O3' BT5"}=" O3* BT5"; $resn_exch{" C2' BT5"}=" C2* BT5";
$resn_exch{" O2' BT5"}=" O2* BT5"; $resn_exch{" C1' BT5"}=" C1* BT5"; $resn_exch{" O5' BTD"}=" O5* BTD";
$resn_exch{" C5' BTD"}=" C5* BTD"; $resn_exch{" C4' BTD"}=" C4* BTD"; $resn_exch{" O4' BTD"}=" O4* BTD";
$resn_exch{" C1' BTD"}=" C1* BTD"; $resn_exch{" C2' BTD"}=" C2* BTD"; $resn_exch{" C3' BTD"}=" C3* BTD";
$resn_exch{" O3' BTD"}=" O3* BTD"; $resn_exch{" C'  BTP"}=" C*  BTP"; $resn_exch{" S'  BTP"}=" S*  BTP";
$resn_exch{" O5' BTX"}=" O5* BTX"; $resn_exch{" C5' BTX"}=" C5* BTX"; $resn_exch{" C4' BTX"}=" C4* BTX";
$resn_exch{" O4' BTX"}=" O4* BTX"; $resn_exch{" C3' BTX"}=" C3* BTX"; $resn_exch{" O3' BTX"}=" O3* BTX";
$resn_exch{" C2' BTX"}=" C2* BTX"; $resn_exch{" O2' BTX"}=" O2* BTX"; $resn_exch{" C1' BTX"}=" C1* BTX";
$resn_exch{" CBB BTX"}="C11B BTX"; $resn_exch{" OBB BTX"}="O11B BTX"; $resn_exch{" CAB BTX"}="C10B BTX";
$resn_exch{" O5' BVD"}=" O5* BVD"; $resn_exch{" C5' BVD"}=" C5* BVD"; $resn_exch{" C4' BVD"}=" C4* BVD";
$resn_exch{" O4' BVD"}=" O4* BVD"; $resn_exch{" C1' BVD"}=" C1* BVD"; $resn_exch{" C2' BVD"}=" C2* BVD";
$resn_exch{" C3' BVD"}=" C3* BVD"; $resn_exch{" O3' BVD"}=" O3* BVD"; $resn_exch{" O3' BVP"}=" O3* BVP";
$resn_exch{" C3' BVP"}=" C3* BVP"; $resn_exch{" C2' BVP"}=" C2* BVP"; $resn_exch{" C1' BVP"}=" C1* BVP";
$resn_exch{" O4' BVP"}=" O4* BVP"; $resn_exch{" C4' BVP"}=" C4* BVP"; $resn_exch{" C5' BVP"}=" C5* BVP";
$resn_exch{" O5' BVP"}=" O5* BVP"; $resn_exch{" O5' BZG"}=" O5* BZG"; $resn_exch{" CT' BZG"}=" CT* BZG";
$resn_exch{" OS' BZG"}=" OS* BZG"; $resn_exch{" CP' BZG"}=" CP* BZG"; $resn_exch{" C5' BZG"}=" C5* BZG";
$resn_exch{" C4' BZG"}=" C4* BZG"; $resn_exch{" C3' BZG"}=" C3* BZG"; $resn_exch{" O3' BZG"}=" O3* BZG";
$resn_exch{" OP3   C"}=" O3P   C"; $resn_exch{" OP1   C"}=" O1P   C"; $resn_exch{" OP2   C"}=" O2P   C";
$resn_exch{" O5'   C"}=" O5*   C"; $resn_exch{" C5'   C"}=" C5*   C"; $resn_exch{" C4'   C"}=" C4*   C";
$resn_exch{" O4'   C"}=" O4*   C"; $resn_exch{" C3'   C"}=" C3*   C"; $resn_exch{" O3'   C"}=" O3*   C";
$resn_exch{" C2'   C"}=" C2*   C"; $resn_exch{" O2'   C"}=" O2*   C"; $resn_exch{" C1'   C"}=" C1*   C";
$resn_exch{" CVX C1B"}="C31' C1B"; $resn_exch{" CWX C1B"}="C32' C1B"; $resn_exch{" OXX C1B"}="O33' C1B";
$resn_exch{" OYX C1B"}="O34' C1B"; $resn_exch{" C6X C1B"}="C42' C1B"; $resn_exch{" C7X C1B"}="C43' C1B";
$resn_exch{" O8X C1B"}="O44' C1B"; $resn_exch{" O9X C1B"}="O45' C1B"; $resn_exch{" C1B C1B"}="C1'' C1B";
$resn_exch{" C2B C1B"}="C2'' C1B"; $resn_exch{" C3B C1B"}="C3'' C1B"; $resn_exch{" C4B C1B"}="C4'' C1B";
$resn_exch{" C5B C1B"}="C5'' C1B"; $resn_exch{" C6B C1B"}="C6'' C1B"; $resn_exch{" O2B C1B"}="O2'' C1B";
$resn_exch{" OP3 C25"}=" O3P C25"; $resn_exch{" OP1 C25"}=" O1P C25"; $resn_exch{" OP2 C25"}=" O2P C25";
$resn_exch{" O5' C25"}=" O5* C25"; $resn_exch{" C5' C25"}=" C5* C25"; $resn_exch{" C4' C25"}=" C4* C25";
$resn_exch{" O4' C25"}=" O4* C25"; $resn_exch{" C3' C25"}=" C3* C25"; $resn_exch{" O3' C25"}=" O3* C25";
$resn_exch{" C2' C25"}=" C2* C25"; $resn_exch{" O2' C25"}=" O2* C25"; $resn_exch{" C1' C25"}=" C1* C25";
$resn_exch{" C1' C2G"}=" C1* C2G"; $resn_exch{" C2' C2G"}=" C2* C2G"; $resn_exch{" O2' C2G"}=" O2* C2G";
$resn_exch{" C3' C2G"}=" C3* C2G"; $resn_exch{" C4' C2G"}=" C4* C2G"; $resn_exch{" O4' C2G"}=" O4* C2G";
$resn_exch{" O3' C2G"}=" O3* C2G"; $resn_exch{" C5' C2G"}=" C5* C2G"; $resn_exch{" O5' C2G"}=" O5* C2G";
$resn_exch{" OP3 C2L"}=" O3P C2L"; $resn_exch{" OP1 C2L"}=" O1P C2L"; $resn_exch{" OP2 C2L"}=" O2P C2L";
$resn_exch{" O5' C2L"}=" O5* C2L"; $resn_exch{" C5' C2L"}=" C5* C2L"; $resn_exch{" C4' C2L"}=" C4* C2L";
$resn_exch{" O4' C2L"}=" O4* C2L"; $resn_exch{" C3' C2L"}=" C3* C2L"; $resn_exch{" O3' C2L"}=" O3* C2L";
$resn_exch{" C2' C2L"}=" C2* C2L"; $resn_exch{" O2' C2L"}=" O2* C2L"; $resn_exch{" C1' C2L"}=" C1* C2L";
$resn_exch{" CA' C2L"}=" CA* C2L"; $resn_exch{" CB' C2L"}=" CB* C2L"; $resn_exch{" OC' C2L"}=" OC* C2L";
$resn_exch{" CD' C2L"}=" CD* C2L"; $resn_exch{" C1' C2P"}=" C1* C2P"; $resn_exch{" C2' C2P"}=" C2* C2P";
$resn_exch{" O2' C2P"}=" O2* C2P"; $resn_exch{" C3' C2P"}=" C3* C2P"; $resn_exch{" C4' C2P"}=" C4* C2P";
$resn_exch{" O4' C2P"}=" O4* C2P"; $resn_exch{" O3' C2P"}=" O3* C2P"; $resn_exch{" C5' C2P"}=" C5* C2P";
$resn_exch{" O5' C2P"}=" O5* C2P"; $resn_exch{" O5' C2R"}=" O5* C2R"; $resn_exch{" C2' C2R"}=" C2* C2R";
$resn_exch{" O2' C2R"}=" O2* C2R"; $resn_exch{" C3' C2R"}=" C3* C2R"; $resn_exch{" O3' C2R"}=" O3* C2R";
$resn_exch{" C5' C2R"}=" C5* C2R"; $resn_exch{" C4' C2R"}=" C4* C2R"; $resn_exch{" O4' C2R"}=" O4* C2R";
$resn_exch{" C1' C2R"}=" C1* C2R"; $resn_exch{" OP3 C2S"}=" O3P C2S"; $resn_exch{" O5' C2S"}=" O5* C2S";
$resn_exch{" C5' C2S"}=" C5* C2S"; $resn_exch{" C4' C2S"}=" C4* C2S"; $resn_exch{" O4' C2S"}=" O4* C2S";
$resn_exch{" C3' C2S"}=" C3* C2S"; $resn_exch{" O3' C2S"}=" O3* C2S"; $resn_exch{" C2' C2S"}=" C2* C2S";
$resn_exch{" C1' C2S"}=" C1* C2S"; $resn_exch{" O5' C31"}=" O5* C31"; $resn_exch{" C5' C31"}=" C5* C31";
$resn_exch{" C4' C31"}=" C4* C31"; $resn_exch{" O4' C31"}=" O4* C31"; $resn_exch{" C3' C31"}=" C3* C31";
$resn_exch{" O3' C31"}=" O3* C31"; $resn_exch{" C2' C31"}=" C2* C31"; $resn_exch{" O2' C31"}=" O2* C31";
$resn_exch{" C1' C31"}=" C1* C31"; $resn_exch{" CA' C31"}=" CA* C31"; $resn_exch{" CB' C31"}=" CB* C31";
$resn_exch{" ND' C31"}=" ND* C31"; $resn_exch{" O5' C32"}=" O5* C32"; $resn_exch{" C5' C32"}=" C5* C32";
$resn_exch{" C4' C32"}=" C4* C32"; $resn_exch{" O4' C32"}=" O4* C32"; $resn_exch{" C3' C32"}=" C3* C32";
$resn_exch{" O3' C32"}=" O3* C32"; $resn_exch{" C2' C32"}=" C2* C32"; $resn_exch{" C1' C32"}=" C1* C32";
$resn_exch{" OP1 C34"}=" O1P C34"; $resn_exch{" OP2 C34"}=" O2P C34"; $resn_exch{" OP3 C34"}=" O3P C34";
$resn_exch{" O5' C34"}=" O5* C34"; $resn_exch{" C5' C34"}=" C5* C34"; $resn_exch{" C4' C34"}=" C4* C34";
$resn_exch{" O4' C34"}=" O4* C34"; $resn_exch{" C1' C34"}=" C1* C34"; $resn_exch{" C2' C34"}=" C2* C34";
$resn_exch{" C3' C34"}=" C3* C34"; $resn_exch{" O3' C34"}=" O3* C34"; $resn_exch{" O5' C36"}=" O5* C36";
$resn_exch{" C5' C36"}=" C5* C36"; $resn_exch{" C4' C36"}=" C4* C36"; $resn_exch{" O4' C36"}=" O4* C36";
$resn_exch{" C3' C36"}=" C3* C36"; $resn_exch{" O3' C36"}=" O3* C36"; $resn_exch{" C2' C36"}=" C2* C36";
$resn_exch{" C1' C36"}=" C1* C36"; $resn_exch{" O5' C37"}=" O5* C37"; $resn_exch{" C5' C37"}=" C5* C37";
$resn_exch{" C4' C37"}=" C4* C37"; $resn_exch{" O4' C37"}=" O4* C37"; $resn_exch{" C3' C37"}=" C3* C37";
$resn_exch{" O3' C37"}=" O3* C37"; $resn_exch{" C2' C37"}=" C2* C37"; $resn_exch{" C1' C37"}=" C1* C37";
$resn_exch{" O5' C38"}=" O5* C38"; $resn_exch{" C5' C38"}=" C5* C38"; $resn_exch{" C4' C38"}=" C4* C38";
$resn_exch{" O4' C38"}=" O4* C38"; $resn_exch{" C3' C38"}=" C3* C38"; $resn_exch{" O3' C38"}=" O3* C38";
$resn_exch{" C2' C38"}=" C2* C38"; $resn_exch{" C1' C38"}=" C1* C38"; $resn_exch{" O5' C3P"}=" O5* C3P";
$resn_exch{" C5' C3P"}=" C5* C3P"; $resn_exch{" C4' C3P"}=" C4* C3P"; $resn_exch{" O4' C3P"}=" O4* C3P";
$resn_exch{" C3' C3P"}=" C3* C3P"; $resn_exch{" O3' C3P"}=" O3* C3P"; $resn_exch{" C2' C3P"}=" C2* C3P";
$resn_exch{" O2' C3P"}=" O2* C3P"; $resn_exch{" C1' C3P"}=" C1* C3P"; $resn_exch{" C1' C42"}=" C1* C42";
$resn_exch{" C2' C42"}=" C2* C42"; $resn_exch{" C3' C42"}=" C3* C42"; $resn_exch{" C4' C42"}=" C4* C42";
$resn_exch{" O4' C42"}=" O4* C42"; $resn_exch{" C5' C42"}=" C5* C42"; $resn_exch{" O5' C42"}=" O5* C42";
$resn_exch{" O5' C43"}=" O5* C43"; $resn_exch{" C5' C43"}=" C5* C43"; $resn_exch{" C4' C43"}=" C4* C43";
$resn_exch{" O4' C43"}=" O4* C43"; $resn_exch{" C3' C43"}=" C3* C43"; $resn_exch{" O3' C43"}=" O3* C43";
$resn_exch{" C2' C43"}=" C2* C43"; $resn_exch{" O2' C43"}=" O2* C43"; $resn_exch{" C1' C43"}=" C1* C43";
$resn_exch{" CA' C43"}=" CA* C43"; $resn_exch{" CB' C43"}=" CB* C43"; $resn_exch{" CD' C43"}=" CD* C43";
$resn_exch{" OC' C43"}=" OC* C43"; $resn_exch{" C1' C45"}=" C1* C45"; $resn_exch{" C2' C45"}=" C2* C45";
$resn_exch{" C3' C45"}=" C3* C45"; $resn_exch{" C4' C45"}=" C4* C45"; $resn_exch{" O4' C45"}=" O4* C45";
$resn_exch{" O3' C45"}=" O3* C45"; $resn_exch{" C5' C45"}=" C5* C45"; $resn_exch{" O5' C45"}=" O5* C45";
$resn_exch{" O5' C46"}=" O5* C46"; $resn_exch{" C5' C46"}=" C5* C46"; $resn_exch{" C4' C46"}=" C4* C46";
$resn_exch{" O4' C46"}=" O4* C46"; $resn_exch{" C3' C46"}=" C3* C46"; $resn_exch{" O3' C46"}=" O3* C46";
$resn_exch{" C2' C46"}=" C2* C46"; $resn_exch{" C1' C46"}=" C1* C46"; $resn_exch{" O5' C49"}=" O5* C49";
$resn_exch{" C5' C49"}=" C5* C49"; $resn_exch{" C4' C49"}=" C4* C49"; $resn_exch{" O4' C49"}=" O4* C49";
$resn_exch{" C3' C49"}=" C3* C49"; $resn_exch{" O3' C49"}=" O3* C49"; $resn_exch{" C2' C49"}=" C2* C49";
$resn_exch{" C1' C49"}=" C1* C49"; $resn_exch{" C1D C5G"}=" C1* C5G"; $resn_exch{" C2D C5G"}=" C2* C5G";
$resn_exch{" O2D C5G"}=" O2* C5G"; $resn_exch{" C3D C5G"}=" C3* C5G"; $resn_exch{" C4D C5G"}=" C4* C5G";
$resn_exch{" O4D C5G"}=" O4* C5G"; $resn_exch{" O3D C5G"}=" O3* C5G"; $resn_exch{" C5D C5G"}=" C5* C5G";
$resn_exch{" O5D C5G"}=" O5* C5G"; $resn_exch{" O5' C5P"}=" O5* C5P"; $resn_exch{" C5' C5P"}=" C5* C5P";
$resn_exch{" C4' C5P"}=" C4* C5P"; $resn_exch{" O4' C5P"}=" O4* C5P"; $resn_exch{" C3' C5P"}=" C3* C5P";
$resn_exch{" O3' C5P"}=" O3* C5P"; $resn_exch{" C2' C5P"}=" C2* C5P"; $resn_exch{" O2' C5P"}=" O2* C5P";
$resn_exch{" C1' C5P"}=" C1* C5P"; $resn_exch{" C11 CA3"}="0CT1 CA3"; $resn_exch{" C11 CA5"}="0CT1 CA5";
$resn_exch{" N1A CAA"}=" AN1 CAA"; $resn_exch{" C2A CAA"}=" AC2 CAA"; $resn_exch{" N3A CAA"}=" AN3 CAA";
$resn_exch{" C4A CAA"}=" AC4 CAA"; $resn_exch{" C5A CAA"}=" AC5 CAA"; $resn_exch{" C6A CAA"}=" AC6 CAA";
$resn_exch{" N6A CAA"}=" AN6 CAA"; $resn_exch{" N7A CAA"}=" AN7 CAA"; $resn_exch{" C8A CAA"}=" AC8 CAA";
$resn_exch{" N9A CAA"}=" AN9 CAA"; $resn_exch{" C1B CAA"}="AC1* CAA"; $resn_exch{" C2B CAA"}="AC2* CAA";
$resn_exch{" O2B CAA"}="AO2* CAA"; $resn_exch{" C3B CAA"}="AC3* CAA"; $resn_exch{" O3B CAA"}="AO3* CAA";
$resn_exch{" P3B CAA"}="AP3* CAA"; $resn_exch{" O7A CAA"}=" AO7 CAA"; $resn_exch{" O8A CAA"}=" AO8 CAA";
$resn_exch{" O9A CAA"}=" AO9 CAA"; $resn_exch{" C4B CAA"}="AC4* CAA"; $resn_exch{" O4B CAA"}="AO4* CAA";
$resn_exch{" C5B CAA"}="AC5* CAA"; $resn_exch{" O5B CAA"}="AO5* CAA"; $resn_exch{" P1A CAA"}=" AP1 CAA";
$resn_exch{" O1A CAA"}=" AO1 CAA"; $resn_exch{" O2A CAA"}=" AO2 CAA"; $resn_exch{" O3A CAA"}=" AO3 CAA";
$resn_exch{" P2A CAA"}=" AP2 CAA"; $resn_exch{" O4A CAA"}=" AO4 CAA"; $resn_exch{" O5A CAA"}=" AO5 CAA";
$resn_exch{" O6A CAA"}=" AO6 CAA"; $resn_exch{" CBP CAA"}="PC11 CAA"; $resn_exch{" CCP CAA"}="PC12 CAA";
$resn_exch{" CDP CAA"}="PC13 CAA"; $resn_exch{" CEP CAA"}="PC14 CAA"; $resn_exch{" CAP CAA"}="PC10 CAA";
$resn_exch{" OAP CAA"}="PO10 CAA"; $resn_exch{" C9P CAA"}=" PC9 CAA"; $resn_exch{" O9P CAA"}=" PO9 CAA";
$resn_exch{" N8P CAA"}=" PN8 CAA"; $resn_exch{" C7P CAA"}=" PC7 CAA"; $resn_exch{" C6P CAA"}=" PC6 CAA";
$resn_exch{" C5P CAA"}=" PC5 CAA"; $resn_exch{" O5P CAA"}=" PO5 CAA"; $resn_exch{" N4P CAA"}=" PN4 CAA";
$resn_exch{" C3P CAA"}=" PC3 CAA"; $resn_exch{" C2P CAA"}=" PC2 CAA"; $resn_exch{" S1P CAA"}=" PS1 CAA";
$resn_exch{" O5' CAG"}=" O5* CAG"; $resn_exch{" C5B CAG"}=" C5* CAG"; $resn_exch{" C4B CAG"}=" C4* CAG";
$resn_exch{" O4' CAG"}=" O4* CAG"; $resn_exch{" C3B CAG"}=" C3* CAG"; $resn_exch{" O3' CAG"}=" O3* CAG";
$resn_exch{" C2B CAG"}=" C2* CAG"; $resn_exch{" O2' CAG"}=" O2* CAG"; $resn_exch{" C1B CAG"}=" C1* CAG";
$resn_exch{" O'L CAG"}="1O2' CAG"; $resn_exch{" O'M CAG"}="2O2' CAG"; $resn_exch{" N1A CAO"}=" AN1 CAO";
$resn_exch{" C2A CAO"}=" AC2 CAO"; $resn_exch{" N3A CAO"}=" AN3 CAO"; $resn_exch{" C4A CAO"}=" AC4 CAO";
$resn_exch{" C5A CAO"}=" AC5 CAO"; $resn_exch{" C6A CAO"}=" AC6 CAO"; $resn_exch{" N6A CAO"}=" AN6 CAO";
$resn_exch{" N7A CAO"}=" AN7 CAO"; $resn_exch{" C8A CAO"}=" AC8 CAO"; $resn_exch{" N9A CAO"}=" AN9 CAO";
$resn_exch{" C1B CAO"}="AC1* CAO"; $resn_exch{" C2B CAO"}="AC2* CAO"; $resn_exch{" O2B CAO"}="AO2* CAO";
$resn_exch{" C3B CAO"}="AC3* CAO"; $resn_exch{" O3B CAO"}="AO3* CAO"; $resn_exch{" P3B CAO"}="AP3* CAO";
$resn_exch{" O7A CAO"}=" AO7 CAO"; $resn_exch{" O8A CAO"}=" AO8 CAO"; $resn_exch{" O9A CAO"}=" AO9 CAO";
$resn_exch{" C4B CAO"}="AC4* CAO"; $resn_exch{" O4B CAO"}="AO4* CAO"; $resn_exch{" C5B CAO"}="AC5* CAO";
$resn_exch{" O5B CAO"}="AO5* CAO"; $resn_exch{" P1A CAO"}=" AP1 CAO"; $resn_exch{" O1A CAO"}=" AO1 CAO";
$resn_exch{" O2A CAO"}=" AO2 CAO"; $resn_exch{" O3A CAO"}=" AO3 CAO"; $resn_exch{" P2A CAO"}=" AP2 CAO";
$resn_exch{" O4A CAO"}=" AO4 CAO"; $resn_exch{" O5A CAO"}=" AO5 CAO"; $resn_exch{" O6A CAO"}=" AO6 CAO";
$resn_exch{" CBP CAO"}="PC11 CAO"; $resn_exch{" CCP CAO"}="PC12 CAO"; $resn_exch{" CDP CAO"}="PC13 CAO";
$resn_exch{" CEP CAO"}="PC14 CAO"; $resn_exch{" CAP CAO"}="PC10 CAO"; $resn_exch{" OAP CAO"}="PO10 CAO";
$resn_exch{" C9P CAO"}=" PC9 CAO"; $resn_exch{" O9P CAO"}=" PO9 CAO"; $resn_exch{" N8P CAO"}=" PN8 CAO";
$resn_exch{" C7P CAO"}=" PC7 CAO"; $resn_exch{" C6P CAO"}=" PC6 CAO"; $resn_exch{" C5P CAO"}=" PC5 CAO";
$resn_exch{" O5P CAO"}=" PO5 CAO"; $resn_exch{" N4P CAO"}=" PN4 CAO"; $resn_exch{" C3P CAO"}=" PC3 CAO";
$resn_exch{" C2P CAO"}=" PC2 CAO"; $resn_exch{" S1P CAO"}=" PS1 CAO"; $resn_exch{" O1P CAO"}=" PO1 CAO";
$resn_exch{" OP1 CAR"}=" O1P CAR"; $resn_exch{" OP2 CAR"}=" O2P CAR"; $resn_exch{" OP3 CAR"}=" O3P CAR";
$resn_exch{" O5' CAR"}=" O5* CAR"; $resn_exch{" C5' CAR"}=" C5* CAR"; $resn_exch{" C4' CAR"}=" C4* CAR";
$resn_exch{" O4' CAR"}=" O4* CAR"; $resn_exch{" C3' CAR"}=" C3* CAR"; $resn_exch{" O3' CAR"}=" O3* CAR";
$resn_exch{" C2' CAR"}=" C2* CAR"; $resn_exch{" O2' CAR"}=" O2* CAR"; $resn_exch{" C1' CAR"}=" C1* CAR";
$resn_exch{" CA' CAV"}=" CA* CAV"; $resn_exch{" CB' CAV"}=" CB* CAV"; $resn_exch{" O3' CB2"}=" O3* CB2";
$resn_exch{" C3' CB2"}=" C3* CB2"; $resn_exch{" C2' CB2"}=" C2* CB2"; $resn_exch{" O2' CB2"}=" O2* CB2";
$resn_exch{" C1' CB2"}=" C1* CB2"; $resn_exch{" O4' CB2"}=" O4* CB2"; $resn_exch{" C4' CB2"}=" C4* CB2";
$resn_exch{" C5' CB2"}=" C5* CB2"; $resn_exch{" O5' CB2"}=" O5* CB2"; $resn_exch{" OP1 CB2"}=" O1P CB2";
$resn_exch{" OP2 CB2"}=" O2P CB2"; $resn_exch{" OP3 CB2"}=" O3P CB2"; $resn_exch{" OP1 CBR"}=" O1P CBR";
$resn_exch{" OP2 CBR"}=" O2P CBR"; $resn_exch{" O5' CBR"}=" O5* CBR"; $resn_exch{" C2' CBR"}=" C2* CBR";
$resn_exch{" C5' CBR"}=" C5* CBR"; $resn_exch{" C4' CBR"}=" C4* CBR"; $resn_exch{" O4' CBR"}=" O4* CBR";
$resn_exch{" C1' CBR"}=" C1* CBR"; $resn_exch{" C3' CBR"}=" C3* CBR"; $resn_exch{" O3' CBR"}=" O3* CBR";
$resn_exch{" OP3 CBR"}=" O3P CBR"; $resn_exch{" C1A CBS"}=" AC1 CBS"; $resn_exch{" C2A CBS"}=" AC2 CBS";
$resn_exch{" C3A CBS"}=" AC3 CBS"; $resn_exch{" C4A CBS"}=" AC4 CBS"; $resn_exch{" C5A CBS"}=" AC5 CBS";
$resn_exch{" C6A CBS"}=" AC6 CBS"; $resn_exch{" C7A CBS"}=" AC7 CBS"; $resn_exch{" C8A CBS"}=" AC8 CBS";
$resn_exch{" N2A CBS"}=" AN2 CBS"; $resn_exch{" O1A CBS"}=" AO1 CBS"; $resn_exch{" O3A CBS"}=" AO3 CBS";
$resn_exch{" O4A CBS"}=" AO4 CBS"; $resn_exch{" O5A CBS"}=" AO5 CBS"; $resn_exch{" O6A CBS"}=" AO6 CBS";
$resn_exch{" O7A CBS"}=" AO7 CBS"; $resn_exch{" C1B CBS"}=" BC1 CBS"; $resn_exch{" C2B CBS"}=" BC2 CBS";
$resn_exch{" C3B CBS"}=" BC3 CBS"; $resn_exch{" C4B CBS"}=" BC4 CBS"; $resn_exch{" C5B CBS"}=" BC5 CBS";
$resn_exch{" C6B CBS"}=" BC6 CBS"; $resn_exch{" C7B CBS"}=" BC7 CBS"; $resn_exch{" C8B CBS"}=" BC8 CBS";
$resn_exch{" N2B CBS"}=" BN2 CBS"; $resn_exch{" O1B CBS"}=" BO1 CBS"; $resn_exch{" O3B CBS"}=" BO3 CBS";
$resn_exch{" O5B CBS"}=" BO5 CBS"; $resn_exch{" O6B CBS"}=" BO6 CBS"; $resn_exch{" O7B CBS"}=" BO7 CBS";
$resn_exch{" O5' CBV"}=" O5* CBV"; $resn_exch{" C5' CBV"}=" C5* CBV"; $resn_exch{" C4' CBV"}=" C4* CBV";
$resn_exch{" O4' CBV"}=" O4* CBV"; $resn_exch{" C3' CBV"}=" C3* CBV"; $resn_exch{" O3' CBV"}=" O3* CBV";
$resn_exch{" C2' CBV"}=" C2* CBV"; $resn_exch{" O2' CBV"}=" O2* CBV"; $resn_exch{" C1' CBV"}=" C1* CBV";
$resn_exch{" OP1 CCC"}=" O1P CCC"; $resn_exch{" OP2 CCC"}=" O2P CCC"; $resn_exch{" OP3 CCC"}=" O3P CCC";
$resn_exch{" O5' CCC"}=" O5* CCC"; $resn_exch{" C5' CCC"}=" C5* CCC"; $resn_exch{" C4' CCC"}=" C4* CCC";
$resn_exch{" O4' CCC"}=" O4* CCC"; $resn_exch{" C3' CCC"}=" C3* CCC"; $resn_exch{" O3' CCC"}=" O3* CCC";
$resn_exch{" C2' CCC"}=" C2* CCC"; $resn_exch{" O2' CCC"}=" O2* CCC"; $resn_exch{" C1' CCC"}=" C1* CCC";
$resn_exch{" NA  CCH"}=" N_A CCH"; $resn_exch{" NB  CCH"}=" N_B CCH"; $resn_exch{" NC  CCH"}=" N_C CCH";
$resn_exch{" ND  CCH"}=" N_D CCH"; $resn_exch{" S1P CCQ"}=" PS1 CCQ"; $resn_exch{" C2P CCQ"}=" PC2 CCQ";
$resn_exch{" C3P CCQ"}=" PC3 CCQ"; $resn_exch{" N4P CCQ"}=" PN4 CCQ"; $resn_exch{" C5P CCQ"}=" PC5 CCQ";
$resn_exch{" O5P CCQ"}=" PO5 CCQ"; $resn_exch{" C6P CCQ"}=" PC6 CCQ"; $resn_exch{" C7P CCQ"}=" PC7 CCQ";
$resn_exch{" N8P CCQ"}=" PN8 CCQ"; $resn_exch{" C9P CCQ"}=" PC9 CCQ"; $resn_exch{" O9P CCQ"}=" PO9 CCQ";
$resn_exch{" CAP CCQ"}="PC10 CCQ"; $resn_exch{" OAP CCQ"}="PO10 CCQ"; $resn_exch{" CBP CCQ"}="PC11 CCQ";
$resn_exch{" CDP CCQ"}="PC13 CCQ"; $resn_exch{" CEP CCQ"}="PC14 CCQ"; $resn_exch{" CCP CCQ"}="PC12 CCQ";
$resn_exch{" O6A CCQ"}=" AO6 CCQ"; $resn_exch{" P2A CCQ"}=" AP2 CCQ"; $resn_exch{" O4A CCQ"}=" AO4 CCQ";
$resn_exch{" O5A CCQ"}=" AO5 CCQ"; $resn_exch{" O3A CCQ"}=" AO3 CCQ"; $resn_exch{" P1A CCQ"}=" AP1 CCQ";
$resn_exch{" O1A CCQ"}=" AO1 CCQ"; $resn_exch{" O2A CCQ"}=" AO2 CCQ"; $resn_exch{" O5D CCQ"}="AO5* CCQ";
$resn_exch{" C5D CCQ"}="AC5* CCQ"; $resn_exch{" C4D CCQ"}="AC4* CCQ"; $resn_exch{" C3D CCQ"}="AC3* CCQ";
$resn_exch{" O3D CCQ"}="AO3* CCQ"; $resn_exch{" P3D CCQ"}="AP3* CCQ"; $resn_exch{" O7A CCQ"}=" AO7 CCQ";
$resn_exch{" O8A CCQ"}=" AO8 CCQ"; $resn_exch{" O9A CCQ"}=" AO9 CCQ"; $resn_exch{" C2D CCQ"}="AC2* CCQ";
$resn_exch{" O2D CCQ"}="AO2* CCQ"; $resn_exch{" C1D CCQ"}="AC1* CCQ"; $resn_exch{" O4D CCQ"}="AO4* CCQ";
$resn_exch{" N9A CCQ"}=" AN9 CCQ"; $resn_exch{" C4A CCQ"}=" AC4 CCQ"; $resn_exch{" C5X CCQ"}=" AC5 CCQ";
$resn_exch{" N7A CCQ"}=" AN7 CCQ"; $resn_exch{" C8A CCQ"}=" AC8 CCQ"; $resn_exch{" C6A CCQ"}=" AC6 CCQ";
$resn_exch{" N6A CCQ"}=" AN6 CCQ"; $resn_exch{" N1A CCQ"}=" AN1 CCQ"; $resn_exch{" C2A CCQ"}=" AC2 CCQ";
$resn_exch{" N3A CCQ"}=" AN3 CCQ"; $resn_exch{" C1' CDC"}=" C1* CDC"; $resn_exch{" C2' CDC"}=" C2* CDC";
$resn_exch{" O2' CDC"}=" O2* CDC"; $resn_exch{" C3' CDC"}=" C3* CDC"; $resn_exch{" C4' CDC"}=" C4* CDC";
$resn_exch{" O4' CDC"}=" O4* CDC"; $resn_exch{" O3' CDC"}=" O3* CDC"; $resn_exch{" C5' CDC"}=" C5* CDC";
$resn_exch{" O5' CDC"}=" O5* CDC"; $resn_exch{" O5' CDF"}=" O5* CDF"; $resn_exch{" C5' CDF"}=" C5* CDF";
$resn_exch{" C4' CDF"}=" C4* CDF"; $resn_exch{" O4' CDF"}=" O4* CDF"; $resn_exch{" C3' CDF"}=" C3* CDF";
$resn_exch{" O3' CDF"}=" O3* CDF"; $resn_exch{" C2' CDF"}=" C2* CDF"; $resn_exch{" O2' CDF"}=" O2* CDF";
$resn_exch{" C1' CDF"}=" C1* CDF"; $resn_exch{" C1' CDM"}=" C1* CDM"; $resn_exch{" C2' CDM"}=" C2* CDM";
$resn_exch{" O2' CDM"}=" O2* CDM"; $resn_exch{" C3' CDM"}=" C3* CDM"; $resn_exch{" C4' CDM"}=" C4* CDM";
$resn_exch{" O4' CDM"}=" O4* CDM"; $resn_exch{" O3' CDM"}=" O3* CDM"; $resn_exch{" C5' CDM"}=" C5* CDM";
$resn_exch{" O5' CDM"}=" O5* CDM"; $resn_exch{" C1' CDP"}=" C1* CDP"; $resn_exch{" C2' CDP"}=" C2* CDP";
$resn_exch{" O2' CDP"}=" O2* CDP"; $resn_exch{" C3' CDP"}=" C3* CDP"; $resn_exch{" C4' CDP"}=" C4* CDP";
$resn_exch{" O4' CDP"}=" O4* CDP"; $resn_exch{" O3' CDP"}=" O3* CDP"; $resn_exch{" C5' CDP"}=" C5* CDP";
$resn_exch{" O5' CDP"}=" O5* CDP"; $resn_exch{" O5' CDY"}=" O5* CDY"; $resn_exch{" C5' CDY"}=" C5* CDY";
$resn_exch{" C4' CDY"}=" C4* CDY"; $resn_exch{" O4' CDY"}=" O4* CDY"; $resn_exch{" C3' CDY"}=" C3* CDY";
$resn_exch{" C2' CDY"}=" C2* CDY"; $resn_exch{" C1' CDY"}=" C1* CDY"; $resn_exch{" O5' CFL"}=" O5* CFL";
$resn_exch{" C5' CFL"}=" C5* CFL"; $resn_exch{" C4' CFL"}=" C4* CFL"; $resn_exch{" O4' CFL"}=" O4* CFL";
$resn_exch{" C3' CFL"}=" C3* CFL"; $resn_exch{" O3' CFL"}=" O3* CFL"; $resn_exch{" C2' CFL"}=" C2* CFL";
$resn_exch{" C1' CFL"}=" C1* CFL"; $resn_exch{" CA' CG2"}=" CA* CG2"; $resn_exch{" CB' CG2"}=" CB* CG2";
$resn_exch{" OB' CG2"}=" OB* CG2"; $resn_exch{" CC' CG2"}=" CC* CG2"; $resn_exch{" OC' CG2"}=" OC* CG2";
$resn_exch{" CD' CG2"}=" CD* CG2"; $resn_exch{" OD' CG2"}=" OD* CG2"; $resn_exch{" CE' CG2"}=" CE* CG2";
$resn_exch{" OE' CG2"}=" OE* CG2"; $resn_exch{" O5' CG2"}=" O5* CG2"; $resn_exch{" C5' CG2"}=" C5* CG2";
$resn_exch{" C4' CG2"}=" C4* CG2"; $resn_exch{" O4' CG2"}=" O4* CG2"; $resn_exch{" C3' CG2"}=" C3* CG2";
$resn_exch{" O3' CG2"}=" O3* CG2"; $resn_exch{" C2' CG2"}=" C2* CG2"; $resn_exch{" O2' CG2"}=" O2* CG2";
$resn_exch{" C1' CG2"}=" C1* CG2"; $resn_exch{" O5B CGP"}="CO5* CGP"; $resn_exch{" C5X CGP"}="CC5* CGP";
$resn_exch{" C4X CGP"}="CC4* CGP"; $resn_exch{" O4B CGP"}="CO4* CGP"; $resn_exch{" C3X CGP"}="CC3* CGP";
$resn_exch{" O3B CGP"}="CO3* CGP"; $resn_exch{" C2X CGP"}="CC2* CGP"; $resn_exch{" C1X CGP"}="CC1* CGP";
$resn_exch{" N1C CGP"}=" CN1 CGP"; $resn_exch{" N3C CGP"}=" CN3 CGP"; $resn_exch{" O2C CGP"}=" CO2 CGP";
$resn_exch{" N4C CGP"}=" CN4 CGP"; $resn_exch{" O5D CGP"}="GO5* CGP"; $resn_exch{" C5D CGP"}="GC5* CGP";
$resn_exch{" C4D CGP"}="GC4* CGP"; $resn_exch{" O4D CGP"}="GO4* CGP"; $resn_exch{" C3D CGP"}="GC3* CGP";
$resn_exch{" O3D CGP"}="GO3* CGP"; $resn_exch{" C2D CGP"}="GC2* CGP"; $resn_exch{" C1D CGP"}="GC1* CGP";
$resn_exch{" N9G CGP"}=" GN9 CGP"; $resn_exch{" C8G CGP"}=" GC8 CGP"; $resn_exch{" N7G CGP"}=" GN7 CGP";
$resn_exch{" C5G CGP"}=" GC5 CGP"; $resn_exch{" C6G CGP"}=" GC6 CGP"; $resn_exch{" O6G CGP"}=" GO6 CGP";
$resn_exch{" N1G CGP"}=" GN1 CGP"; $resn_exch{" C2G CGP"}=" GC2 CGP"; $resn_exch{" N2G CGP"}=" GN2 CGP";
$resn_exch{" N3G CGP"}=" GN3 CGP"; $resn_exch{" C4G CGP"}=" GC4 CGP"; $resn_exch{"O5'R CGR"}="RO5* CGR";
$resn_exch{"C5'R CGR"}="RC5* CGR"; $resn_exch{"C4'R CGR"}="RC4* CGR"; $resn_exch{"O4'R CGR"}="RO4* CGR";
$resn_exch{"C1'R CGR"}="RC1* CGR"; $resn_exch{"C2'R CGR"}="RC2* CGR"; $resn_exch{"O2'R CGR"}="RO2* CGR";
$resn_exch{"C3'R CGR"}="RC3* CGR"; $resn_exch{"O3'R CGR"}="RO3* CGR"; $resn_exch{" C1' CGR"}=" C1* CGR";
$resn_exch{" O4' CGR"}=" O4* CGR"; $resn_exch{" C2' CGR"}=" C2* CGR"; $resn_exch{" O2' CGR"}=" O2* CGR";
$resn_exch{" C3' CGR"}=" C3* CGR"; $resn_exch{" O3' CGR"}=" O3* CGR"; $resn_exch{" C4' CGR"}=" C4* CGR";
$resn_exch{" C5' CGR"}=" C5* CGR"; $resn_exch{" O5' CGR"}=" O5* CGR"; $resn_exch{"OE11 CGU"}="OE1  CGU";
$resn_exch{"OE12 CGU"}="OE2  CGU"; $resn_exch{"OE21 CGU"}="OE3  CGU"; $resn_exch{"OE22 CGU"}="OE4  CGU";
$resn_exch{" OP3  CH"}=" O3P  CH"; $resn_exch{" OP1  CH"}=" O1P  CH"; $resn_exch{" OP2  CH"}=" O2P  CH";
$resn_exch{" O5'  CH"}=" O5*  CH"; $resn_exch{" C5'  CH"}=" C5*  CH"; $resn_exch{" C4'  CH"}=" C4*  CH";
$resn_exch{" O4'  CH"}=" O4*  CH"; $resn_exch{" C3'  CH"}=" C3*  CH"; $resn_exch{" O3'  CH"}=" O3*  CH";
$resn_exch{" C2'  CH"}=" C2*  CH"; $resn_exch{" O2'  CH"}=" O2*  CH"; $resn_exch{" C1'  CH"}=" C1*  CH";
$resn_exch{" C1' CH1"}=" C1* CH1"; $resn_exch{" C2' CH1"}=" C2* CH1"; $resn_exch{" O2' CH1"}=" O2* CH1";
$resn_exch{" C3' CH1"}=" C3* CH1"; $resn_exch{" C4' CH1"}=" C4* CH1"; $resn_exch{" O4' CH1"}=" O4* CH1";
$resn_exch{" C5' CH1"}=" C5* CH1"; $resn_exch{" O5' CH1"}=" O5* CH1"; $resn_exch{" NA  CHL"}=" N_A CHL";
$resn_exch{" NB  CHL"}=" N_B CHL"; $resn_exch{" NC  CHL"}=" N_C CHL"; $resn_exch{" ND  CHL"}=" N_D CHL";
$resn_exch{" N1A CIC"}=" AN1 CIC"; $resn_exch{" C2A CIC"}=" AC2 CIC"; $resn_exch{" N3A CIC"}=" AN3 CIC";
$resn_exch{" C4A CIC"}=" AC4 CIC"; $resn_exch{" C5A CIC"}=" AC5 CIC"; $resn_exch{" C6A CIC"}=" AC6 CIC";
$resn_exch{" N6A CIC"}=" AN6 CIC"; $resn_exch{" N7A CIC"}=" AN7 CIC"; $resn_exch{" C8A CIC"}=" AC8 CIC";
$resn_exch{" N9A CIC"}=" AN9 CIC"; $resn_exch{" C1B CIC"}="AC1* CIC"; $resn_exch{" C2B CIC"}="AC2* CIC";
$resn_exch{" O2B CIC"}="AO2* CIC"; $resn_exch{" C3B CIC"}="AC3* CIC"; $resn_exch{" O3B CIC"}="AO3* CIC";
$resn_exch{" P3B CIC"}="AP3* CIC"; $resn_exch{" O7A CIC"}=" AO7 CIC"; $resn_exch{" O8A CIC"}=" AO8 CIC";
$resn_exch{" O9A CIC"}=" AO9 CIC"; $resn_exch{" C4B CIC"}="AC4* CIC"; $resn_exch{" O4B CIC"}="AO4* CIC";
$resn_exch{" C5B CIC"}="AC5* CIC"; $resn_exch{" O5B CIC"}="AO5* CIC"; $resn_exch{" P1A CIC"}=" AP1 CIC";
$resn_exch{" O1A CIC"}=" AO1 CIC"; $resn_exch{" O2A CIC"}=" AO2 CIC"; $resn_exch{" O3A CIC"}=" AO3 CIC";
$resn_exch{" P2A CIC"}=" AP2 CIC"; $resn_exch{" O4A CIC"}=" AO4 CIC"; $resn_exch{" O5A CIC"}=" AO5 CIC";
$resn_exch{" O6A CIC"}=" AO6 CIC"; $resn_exch{" CBP CIC"}="PC11 CIC"; $resn_exch{" CCP CIC"}="PC12 CIC";
$resn_exch{" CDP CIC"}="PC13 CIC"; $resn_exch{" CEP CIC"}="PC14 CIC"; $resn_exch{" CAP CIC"}="PC10 CIC";
$resn_exch{" OAP CIC"}="PO10 CIC"; $resn_exch{" C9P CIC"}=" PC9 CIC"; $resn_exch{" O9P CIC"}=" PO9 CIC";
$resn_exch{" N8P CIC"}=" PN8 CIC"; $resn_exch{" C7P CIC"}=" PC7 CIC"; $resn_exch{" C6P CIC"}=" PC6 CIC";
$resn_exch{" C5P CIC"}=" PC5 CIC"; $resn_exch{" O5P CIC"}=" PO5 CIC"; $resn_exch{" N4P CIC"}=" PN4 CIC";
$resn_exch{" C3P CIC"}=" PC3 CIC"; $resn_exch{" C2P CIC"}=" PC2 CIC"; $resn_exch{" S1P CIC"}=" PS1 CIC";
$resn_exch{" C1P CIC"}=" PC1 CIC"; $resn_exch{" OBA CIE"}="O11A CIE"; $resn_exch{" OBB CIE"}="O11B CIE";
$resn_exch{" NA  CL1"}=" N_A CL1"; $resn_exch{" NB  CL1"}=" N_B CL1"; $resn_exch{" NC  CL1"}=" N_C CL1";
$resn_exch{" ND  CL1"}=" N_D CL1"; $resn_exch{" NA  CL2"}=" N_A CL2"; $resn_exch{" NB  CL2"}=" N_B CL2";
$resn_exch{" NC  CL2"}=" N_C CL2"; $resn_exch{" ND  CL2"}=" N_D CL2"; $resn_exch{" NA  CLA"}=" N_A CLA";
$resn_exch{" NB  CLA"}=" N_B CLA"; $resn_exch{" NC  CLA"}=" N_C CLA"; $resn_exch{" ND  CLA"}=" N_D CLA";
$resn_exch{"CL1  CLM"}="CL1A CLM"; $resn_exch{"CL2  CLM"}="CL1B CLM"; $resn_exch{" NA  CLN"}=" N_A CLN";
$resn_exch{" NB  CLN"}=" N_B CLN"; $resn_exch{" NC  CLN"}=" N_C CLN"; $resn_exch{" ND  CLN"}=" N_D CLN";
$resn_exch{" O5' CM0"}=" O5* CM0"; $resn_exch{" C3' CM0"}=" C3* CM0"; $resn_exch{" C5' CM0"}=" C5* CM0";
$resn_exch{" C4' CM0"}=" C4* CM0"; $resn_exch{" O4' CM0"}=" O4* CM0"; $resn_exch{" C2' CM0"}=" C2* CM0";
$resn_exch{" C1' CM0"}=" C1* CM0"; $resn_exch{" O2' CM0"}=" O2* CM0"; $resn_exch{" O3' CM0"}=" O3* CM0";
$resn_exch{" N1A CMC"}=" AN1 CMC"; $resn_exch{" C2A CMC"}=" AC2 CMC"; $resn_exch{" N3A CMC"}=" AN3 CMC";
$resn_exch{" C4A CMC"}=" AC4 CMC"; $resn_exch{" C5A CMC"}=" AC5 CMC"; $resn_exch{" C6A CMC"}=" AC6 CMC";
$resn_exch{" N6A CMC"}=" AN6 CMC"; $resn_exch{" N7A CMC"}=" AN7 CMC"; $resn_exch{" C8A CMC"}=" AC8 CMC";
$resn_exch{" N9A CMC"}=" AN9 CMC"; $resn_exch{" C1B CMC"}="AC1* CMC"; $resn_exch{" C2B CMC"}="AC2* CMC";
$resn_exch{" O2B CMC"}="AO2* CMC"; $resn_exch{" C3B CMC"}="AC3* CMC"; $resn_exch{" O3B CMC"}="AO3* CMC";
$resn_exch{" P3B CMC"}="AP3* CMC"; $resn_exch{" O7A CMC"}=" AO7 CMC"; $resn_exch{" O8A CMC"}=" AO8 CMC";
$resn_exch{" O9A CMC"}=" AO9 CMC"; $resn_exch{" C4B CMC"}="AC4* CMC"; $resn_exch{" O4B CMC"}="AO4* CMC";
$resn_exch{" C5B CMC"}="AC5* CMC"; $resn_exch{" O5B CMC"}="AO5* CMC"; $resn_exch{" P1A CMC"}=" AP1 CMC";
$resn_exch{" O1A CMC"}=" AO1 CMC"; $resn_exch{" O2A CMC"}=" AO2 CMC"; $resn_exch{" O3A CMC"}=" AO3 CMC";
$resn_exch{" P2A CMC"}=" AP2 CMC"; $resn_exch{" O4A CMC"}=" AO4 CMC"; $resn_exch{" O5A CMC"}=" AO5 CMC";
$resn_exch{" O6A CMC"}=" AO6 CMC"; $resn_exch{" CBP CMC"}="PC11 CMC"; $resn_exch{" CCP CMC"}="PC12 CMC";
$resn_exch{" CDP CMC"}="PC13 CMC"; $resn_exch{" CEP CMC"}="PC14 CMC"; $resn_exch{" CAP CMC"}="PC10 CMC";
$resn_exch{" OAP CMC"}="PO10 CMC"; $resn_exch{" C9P CMC"}=" PC9 CMC"; $resn_exch{" O9P CMC"}=" PO9 CMC";
$resn_exch{" N8P CMC"}=" PN8 CMC"; $resn_exch{" C7P CMC"}=" PC7 CMC"; $resn_exch{" C6P CMC"}=" PC6 CMC";
$resn_exch{" C5P CMC"}=" PC5 CMC"; $resn_exch{" O5P CMC"}=" PO5 CMC"; $resn_exch{" N4P CMC"}=" PN4 CMC";
$resn_exch{" C3P CMC"}=" PC3 CMC"; $resn_exch{" C2P CMC"}=" PC2 CMC"; $resn_exch{" S1P CMC"}=" PS1 CMC";
$resn_exch{" C1' CMD"}=" C1* CMD"; $resn_exch{" C2' CMD"}=" C2* CMD"; $resn_exch{" C3' CMD"}=" C3* CMD";
$resn_exch{" C4' CMD"}=" C4* CMD"; $resn_exch{" O4' CMD"}=" O4* CMD"; $resn_exch{" C5' CMD"}=" C5* CMD";
$resn_exch{" O5' CMD"}=" O5* CMD"; $resn_exch{" C6' CMD"}=" C6* CMD"; $resn_exch{" N3' CMD"}=" N3* CMD";
$resn_exch{" C1' CMK"}=" C1* CMK"; $resn_exch{" C2' CMK"}=" C2* CMK"; $resn_exch{" O2' CMK"}=" O2* CMK";
$resn_exch{" C3' CMK"}=" C3* CMK"; $resn_exch{" C4' CMK"}=" C4* CMK"; $resn_exch{" O4' CMK"}=" O4* CMK";
$resn_exch{" O3' CMK"}=" O3* CMK"; $resn_exch{" C5' CMK"}=" C5* CMK"; $resn_exch{" O5' CMK"}=" O5* CMK";
$resn_exch{" O5' CMP"}=" O5* CMP"; $resn_exch{" C5' CMP"}=" C5* CMP"; $resn_exch{" C4' CMP"}=" C4* CMP";
$resn_exch{" O4' CMP"}=" O4* CMP"; $resn_exch{" C3' CMP"}=" C3* CMP"; $resn_exch{" O3' CMP"}=" O3* CMP";
$resn_exch{" C2' CMP"}=" C2* CMP"; $resn_exch{" O2' CMP"}=" O2* CMP"; $resn_exch{" C1' CMP"}=" C1* CMP";
$resn_exch{" OP1 CMR"}=" O1P CMR"; $resn_exch{" O5' CMR"}=" O5* CMR"; $resn_exch{" C5' CMR"}=" C5* CMR";
$resn_exch{" C4' CMR"}=" C4* CMR"; $resn_exch{" O4' CMR"}=" O4* CMR"; $resn_exch{" C1' CMR"}=" C1* CMR";
$resn_exch{" C3' CMR"}=" C3* CMR"; $resn_exch{" C2' CMR"}=" C2* CMR"; $resn_exch{" O3' CMR"}=" O3* CMR";
$resn_exch{" OP3 CMR"}=" O3P CMR"; $resn_exch{" N1A CMX"}=" AN1 CMX"; $resn_exch{" C2A CMX"}=" AC2 CMX";
$resn_exch{" N3A CMX"}=" AN3 CMX"; $resn_exch{" C4A CMX"}=" AC4 CMX"; $resn_exch{" C5A CMX"}=" AC5 CMX";
$resn_exch{" C6A CMX"}=" AC6 CMX"; $resn_exch{" N6A CMX"}=" AN6 CMX"; $resn_exch{" N7A CMX"}=" AN7 CMX";
$resn_exch{" C8A CMX"}=" AC8 CMX"; $resn_exch{" N9A CMX"}=" AN9 CMX"; $resn_exch{" C1B CMX"}="AC1* CMX";
$resn_exch{" C2B CMX"}="AC2* CMX"; $resn_exch{" O2B CMX"}="AO2* CMX"; $resn_exch{" C3B CMX"}="AC3* CMX";
$resn_exch{" O3B CMX"}="AO3* CMX"; $resn_exch{" P3B CMX"}="AP3* CMX"; $resn_exch{" O7A CMX"}=" AO7 CMX";
$resn_exch{" O8A CMX"}=" AO8 CMX"; $resn_exch{" O9A CMX"}=" AO9 CMX"; $resn_exch{" C4B CMX"}="AC4* CMX";
$resn_exch{" O4B CMX"}="AO4* CMX"; $resn_exch{" C5B CMX"}="AC5* CMX"; $resn_exch{" O5B CMX"}="AO5* CMX";
$resn_exch{" P1A CMX"}=" AP1 CMX"; $resn_exch{" O1A CMX"}=" AO1 CMX"; $resn_exch{" O2A CMX"}=" AO2 CMX";
$resn_exch{" O3A CMX"}=" AO3 CMX"; $resn_exch{" P2A CMX"}=" AP2 CMX"; $resn_exch{" O4A CMX"}=" AO4 CMX";
$resn_exch{" O5A CMX"}=" AO5 CMX"; $resn_exch{" O6A CMX"}=" AO6 CMX"; $resn_exch{" CBP CMX"}="PC11 CMX";
$resn_exch{" CCP CMX"}="PC12 CMX"; $resn_exch{" CDP CMX"}="PC13 CMX"; $resn_exch{" CEP CMX"}="PC14 CMX";
$resn_exch{" CAP CMX"}="PC10 CMX"; $resn_exch{" OAP CMX"}="PO10 CMX"; $resn_exch{" C9P CMX"}=" PC9 CMX";
$resn_exch{" O9P CMX"}=" PO9 CMX"; $resn_exch{" N8P CMX"}=" PN8 CMX"; $resn_exch{" C7P CMX"}=" PC7 CMX";
$resn_exch{" C6P CMX"}=" PC6 CMX"; $resn_exch{" C5P CMX"}=" PC5 CMX"; $resn_exch{" O5P CMX"}=" PO5 CMX";
$resn_exch{" N4P CMX"}=" PN4 CMX"; $resn_exch{" C3P CMX"}=" PC3 CMX"; $resn_exch{" C2P CMX"}=" PC2 CMX";
$resn_exch{" PA  CNA"}=" AP  CNA"; $resn_exch{" O1A CNA"}=" AO1 CNA"; $resn_exch{" O2A CNA"}=" AO2 CNA";
$resn_exch{" O5B CNA"}="AO5* CNA"; $resn_exch{" C5B CNA"}="AC5* CNA"; $resn_exch{" C4B CNA"}="AC4* CNA";
$resn_exch{" O4B CNA"}="AO4* CNA"; $resn_exch{" C3B CNA"}="AC3* CNA"; $resn_exch{" O3B CNA"}="AO3* CNA";
$resn_exch{" C2B CNA"}="AC2* CNA"; $resn_exch{" O2B CNA"}="AO2* CNA"; $resn_exch{" C1B CNA"}="AC1* CNA";
$resn_exch{" N9A CNA"}=" AN9 CNA"; $resn_exch{" C8A CNA"}=" AC8 CNA"; $resn_exch{" N7A CNA"}=" AN7 CNA";
$resn_exch{" C5A CNA"}=" AC5 CNA"; $resn_exch{" C6A CNA"}=" AC6 CNA"; $resn_exch{" N6A CNA"}=" AN6 CNA";
$resn_exch{" N1A CNA"}=" AN1 CNA"; $resn_exch{" C2A CNA"}=" AC2 CNA"; $resn_exch{" N3A CNA"}=" AN3 CNA";
$resn_exch{" C4A CNA"}=" AC4 CNA"; $resn_exch{" PN  CNA"}=" NP  CNA"; $resn_exch{" O1N CNA"}=" NO1 CNA";
$resn_exch{" O2N CNA"}=" NO2 CNA"; $resn_exch{" O5D CNA"}="NO5* CNA"; $resn_exch{" C5D CNA"}="NC5* CNA";
$resn_exch{" C4D CNA"}="NC4* CNA"; $resn_exch{" C4' CNA"}=" C4* CNA"; $resn_exch{" C3D CNA"}="NC3* CNA";
$resn_exch{" O3D CNA"}="NO3* CNA"; $resn_exch{" C2D CNA"}="NC2* CNA"; $resn_exch{" O2D CNA"}="NO2* CNA";
$resn_exch{" C1D CNA"}="NC1* CNA"; $resn_exch{" N1N CNA"}=" NN1 CNA"; $resn_exch{" C2N CNA"}=" NC2 CNA";
$resn_exch{" C3N CNA"}=" NC3 CNA"; $resn_exch{" C7N CNA"}=" NC7 CNA"; $resn_exch{" O7N CNA"}=" NO7 CNA";
$resn_exch{" N7N CNA"}=" NN7 CNA"; $resn_exch{" C4N CNA"}=" NC4 CNA"; $resn_exch{" C5N CNA"}=" NC5 CNA";
$resn_exch{" C6N CNA"}=" NC6 CNA"; $resn_exch{" PA  CND"}=" AP  CND"; $resn_exch{" O1A CND"}=" AO1 CND";
$resn_exch{" O2A CND"}=" AO2 CND"; $resn_exch{" O5B CND"}="AO5* CND"; $resn_exch{" C5B CND"}="AC5* CND";
$resn_exch{" C4B CND"}="AC4* CND"; $resn_exch{" O4B CND"}="AO4* CND"; $resn_exch{" C3B CND"}="AC3* CND";
$resn_exch{" O3B CND"}="AO3* CND"; $resn_exch{" C2B CND"}="AC2* CND"; $resn_exch{" O2B CND"}="AO2* CND";
$resn_exch{" C1B CND"}="AC1* CND"; $resn_exch{" N9A CND"}=" AN9 CND"; $resn_exch{" C8A CND"}=" AC8 CND";
$resn_exch{" N7A CND"}=" AN7 CND"; $resn_exch{" C5A CND"}=" AC5 CND"; $resn_exch{" C6A CND"}=" AC6 CND";
$resn_exch{" N6A CND"}=" AN6 CND"; $resn_exch{" N1A CND"}=" AN1 CND"; $resn_exch{" C2A CND"}=" AC2 CND";
$resn_exch{" N3A CND"}=" AN3 CND"; $resn_exch{" C4A CND"}=" AC4 CND"; $resn_exch{" PN  CND"}=" NP  CND";
$resn_exch{" O1N CND"}=" NO1 CND"; $resn_exch{" O2N CND"}=" NO2 CND"; $resn_exch{" O5D CND"}="NO5* CND";
$resn_exch{" C5D CND"}="NC5* CND"; $resn_exch{" C4D CND"}="NC4* CND"; $resn_exch{" O4D CND"}="NO4* CND";
$resn_exch{" C3D CND"}="NC3* CND"; $resn_exch{" O3D CND"}="NO3* CND"; $resn_exch{" C2D CND"}="NC2* CND";
$resn_exch{" O2D CND"}="NO2* CND"; $resn_exch{" C1D CND"}="NC1* CND"; $resn_exch{" C1N CND"}=" NC1 CND";
$resn_exch{" C2N CND"}=" NC2 CND"; $resn_exch{" C3N CND"}=" NC3 CND"; $resn_exch{" C7N CND"}=" NC7 CND";
$resn_exch{" O7N CND"}=" NO7 CND"; $resn_exch{" N7N CND"}=" NN7 CND"; $resn_exch{" C4N CND"}=" NC4 CND";
$resn_exch{" N5N CND"}=" NN5 CND"; $resn_exch{" C6N CND"}=" NC6 CND"; $resn_exch{" N2' CO4"}=" N2* CO4";
$resn_exch{" C2D CO4"}=" C2* CO4"; $resn_exch{" N3' CO4"}=" N3* CO4"; $resn_exch{" C4D CO4"}=" C4* CO4";
$resn_exch{" N4' CO4"}=" N4* CO4"; $resn_exch{" C5D CO4"}=" C5* CO4"; $resn_exch{" C6D CO4"}=" C6* CO4";
$resn_exch{" C7' CO4"}=" C7* CO4"; $resn_exch{" N8' CO4"}=" N8* CO4"; $resn_exch{" N1' CO4"}=" N1* CO4";
$resn_exch{" C8' CO4"}=" C8* CO4"; $resn_exch{" N9' CO4"}=" N9* CO4"; $resn_exch{" C9' CO4"}=" C9* CO4";
$resn_exch{" N1A CO6"}=" AN1 CO6"; $resn_exch{" C2A CO6"}=" AC2 CO6"; $resn_exch{" N3A CO6"}=" AN3 CO6";
$resn_exch{" C4A CO6"}=" AC4 CO6"; $resn_exch{" C5A CO6"}=" AC5 CO6"; $resn_exch{" C6A CO6"}=" AC6 CO6";
$resn_exch{" N6A CO6"}=" AN6 CO6"; $resn_exch{" N7A CO6"}=" AN7 CO6"; $resn_exch{" C8A CO6"}=" AC8 CO6";
$resn_exch{" N9A CO6"}=" AN9 CO6"; $resn_exch{" C1B CO6"}="AC1* CO6"; $resn_exch{" C2B CO6"}="AC2* CO6";
$resn_exch{" O2B CO6"}="AO2* CO6"; $resn_exch{" C3B CO6"}="AC3* CO6"; $resn_exch{" O3B CO6"}="AO3* CO6";
$resn_exch{" P3B CO6"}="AP3* CO6"; $resn_exch{" O7A CO6"}=" AO7 CO6"; $resn_exch{" O8A CO6"}=" AO8 CO6";
$resn_exch{" O9A CO6"}=" AO9 CO6"; $resn_exch{" C4B CO6"}="AC4* CO6"; $resn_exch{" O4B CO6"}="AO4* CO6";
$resn_exch{" C5B CO6"}="AC5* CO6"; $resn_exch{" O5B CO6"}="AO5* CO6"; $resn_exch{" P1A CO6"}=" AP1 CO6";
$resn_exch{" O1A CO6"}=" AO1 CO6"; $resn_exch{" O2A CO6"}=" AO2 CO6"; $resn_exch{" O3A CO6"}=" AO3 CO6";
$resn_exch{" P2A CO6"}=" AP2 CO6"; $resn_exch{" O4A CO6"}=" AO4 CO6"; $resn_exch{" O5A CO6"}=" AO5 CO6";
$resn_exch{" O6A CO6"}=" AO6 CO6"; $resn_exch{" CBP CO6"}="PC11 CO6"; $resn_exch{" CCP CO6"}="PC12 CO6";
$resn_exch{" CDP CO6"}="PC13 CO6"; $resn_exch{" CEP CO6"}="PC14 CO6"; $resn_exch{" CAP CO6"}="PC10 CO6";
$resn_exch{" OAP CO6"}="PO10 CO6"; $resn_exch{" C9P CO6"}=" PC9 CO6"; $resn_exch{" O9P CO6"}=" PO9 CO6";
$resn_exch{" N8P CO6"}=" PN8 CO6"; $resn_exch{" C7P CO6"}=" PC7 CO6"; $resn_exch{" C6P CO6"}=" PC6 CO6";
$resn_exch{" C5P CO6"}=" PC5 CO6"; $resn_exch{" O5P CO6"}=" PO5 CO6"; $resn_exch{" N4P CO6"}=" PN4 CO6";
$resn_exch{" C3P CO6"}=" PC3 CO6"; $resn_exch{" C2P CO6"}=" PC2 CO6"; $resn_exch{" S1P CO6"}=" PS1 CO6";
$resn_exch{" N1A CO8"}=" AN1 CO8"; $resn_exch{" C2A CO8"}=" AC2 CO8"; $resn_exch{" N3A CO8"}=" AN3 CO8";
$resn_exch{" C4A CO8"}=" AC4 CO8"; $resn_exch{" C5A CO8"}=" AC5 CO8"; $resn_exch{" C6A CO8"}=" AC6 CO8";
$resn_exch{" N6A CO8"}=" AN6 CO8"; $resn_exch{" N7A CO8"}=" AN7 CO8"; $resn_exch{" C8A CO8"}=" AC8 CO8";
$resn_exch{" N9A CO8"}=" AN9 CO8"; $resn_exch{" C1B CO8"}="AC1* CO8"; $resn_exch{" C2B CO8"}="AC2* CO8";
$resn_exch{" O2B CO8"}="AO2* CO8"; $resn_exch{" C3B CO8"}="AC3* CO8"; $resn_exch{" O3B CO8"}="AO3* CO8";
$resn_exch{" P3B CO8"}="AP3* CO8"; $resn_exch{" O7A CO8"}=" AO7 CO8"; $resn_exch{" O8A CO8"}=" AO8 CO8";
$resn_exch{" O9A CO8"}=" AO9 CO8"; $resn_exch{" C4B CO8"}="AC4* CO8"; $resn_exch{" O4B CO8"}="AO4* CO8";
$resn_exch{" C5B CO8"}="AC5* CO8"; $resn_exch{" O5B CO8"}="AO5* CO8"; $resn_exch{" P1A CO8"}=" AP1 CO8";
$resn_exch{" O1A CO8"}=" AO1 CO8"; $resn_exch{" O2A CO8"}=" AO2 CO8"; $resn_exch{" O3A CO8"}=" AO3 CO8";
$resn_exch{" P2A CO8"}=" AP2 CO8"; $resn_exch{" O4A CO8"}=" AO4 CO8"; $resn_exch{" O5A CO8"}=" AO5 CO8";
$resn_exch{" O6A CO8"}=" AO6 CO8"; $resn_exch{" CBP CO8"}="PC11 CO8"; $resn_exch{" CCP CO8"}="PC12 CO8";
$resn_exch{" CDP CO8"}="PC13 CO8"; $resn_exch{" CEP CO8"}="PC14 CO8"; $resn_exch{" CAP CO8"}="PC10 CO8";
$resn_exch{" OAP CO8"}="PO10 CO8"; $resn_exch{" C9P CO8"}=" PC9 CO8"; $resn_exch{" O9P CO8"}=" PO9 CO8";
$resn_exch{" N8P CO8"}=" PN8 CO8"; $resn_exch{" C7P CO8"}=" PC7 CO8"; $resn_exch{" C6P CO8"}=" PC6 CO8";
$resn_exch{" C5P CO8"}=" PC5 CO8"; $resn_exch{" O5P CO8"}=" PO5 CO8"; $resn_exch{" N4P CO8"}=" PN4 CO8";
$resn_exch{" C3P CO8"}=" PC3 CO8"; $resn_exch{" C2P CO8"}=" PC2 CO8"; $resn_exch{" S1P CO8"}=" PS1 CO8";
$resn_exch{" C1' CO8"}=" C1* CO8"; $resn_exch{" O1' CO8"}=" O1* CO8"; $resn_exch{" C2' CO8"}=" C2* CO8";
$resn_exch{" C3' CO8"}=" C3* CO8"; $resn_exch{" C4' CO8"}=" C4* CO8"; $resn_exch{" C5' CO8"}=" C5* CO8";
$resn_exch{" C6' CO8"}=" C6* CO8"; $resn_exch{" C7' CO8"}=" C7* CO8"; $resn_exch{" C8' CO8"}=" C8* CO8";
$resn_exch{" N1A COA"}=" AN1 COA"; $resn_exch{" C2A COA"}=" AC2 COA"; $resn_exch{" N3A COA"}=" AN3 COA";
$resn_exch{" C4A COA"}=" AC4 COA"; $resn_exch{" C5A COA"}=" AC5 COA"; $resn_exch{" C6A COA"}=" AC6 COA";
$resn_exch{" N6A COA"}=" AN6 COA"; $resn_exch{" N7A COA"}=" AN7 COA"; $resn_exch{" C8A COA"}=" AC8 COA";
$resn_exch{" N9A COA"}=" AN9 COA"; $resn_exch{" C1B COA"}="AC1* COA"; $resn_exch{" C2B COA"}="AC2* COA";
$resn_exch{" O2B COA"}="AO2* COA"; $resn_exch{" C3B COA"}="AC3* COA"; $resn_exch{" O3B COA"}="AO3* COA";
$resn_exch{" P3B COA"}="AP3* COA"; $resn_exch{" O7A COA"}=" AO7 COA"; $resn_exch{" O8A COA"}=" AO8 COA";
$resn_exch{" O9A COA"}=" AO9 COA"; $resn_exch{" C4B COA"}="AC4* COA"; $resn_exch{" O4B COA"}="AO4* COA";
$resn_exch{" C5B COA"}="AC5* COA"; $resn_exch{" O5B COA"}="AO5* COA"; $resn_exch{" P1A COA"}=" AP1 COA";
$resn_exch{" O1A COA"}=" AO1 COA"; $resn_exch{" O2A COA"}=" AO2 COA"; $resn_exch{" O3A COA"}=" AO3 COA";
$resn_exch{" P2A COA"}=" AP2 COA"; $resn_exch{" O4A COA"}=" AO4 COA"; $resn_exch{" O5A COA"}=" AO5 COA";
$resn_exch{" O6A COA"}=" AO6 COA"; $resn_exch{" CBP COA"}="PC11 COA"; $resn_exch{" CCP COA"}="PC12 COA";
$resn_exch{" CDP COA"}="PC13 COA"; $resn_exch{" CEP COA"}="PC14 COA"; $resn_exch{" CAP COA"}="PC10 COA";
$resn_exch{" OAP COA"}="PO10 COA"; $resn_exch{" C9P COA"}=" PC9 COA"; $resn_exch{" O9P COA"}=" PO9 COA";
$resn_exch{" N8P COA"}=" PN8 COA"; $resn_exch{" C7P COA"}=" PC7 COA"; $resn_exch{" C6P COA"}=" PC6 COA";
$resn_exch{" C5P COA"}=" PC5 COA"; $resn_exch{" O5P COA"}=" PO5 COA"; $resn_exch{" N4P COA"}=" PN4 COA";
$resn_exch{" C3P COA"}=" PC3 COA"; $resn_exch{" C2P COA"}=" PC2 COA"; $resn_exch{" S1P COA"}=" PS1 COA";
$resn_exch{" N1A COF"}=" AN1 COF"; $resn_exch{" C2A COF"}=" AC2 COF"; $resn_exch{" N3A COF"}=" AN3 COF";
$resn_exch{" C4A COF"}=" AC4 COF"; $resn_exch{" C5A COF"}=" AC5 COF"; $resn_exch{" C6A COF"}=" AC6 COF";
$resn_exch{" N6A COF"}=" AN6 COF"; $resn_exch{" N7A COF"}=" AN7 COF"; $resn_exch{" C8A COF"}=" AC8 COF";
$resn_exch{" N9A COF"}=" AN9 COF"; $resn_exch{" C1B COF"}="AC1* COF"; $resn_exch{" C2B COF"}="AC2* COF";
$resn_exch{" O2B COF"}="AO2* COF"; $resn_exch{" C3B COF"}="AC3* COF"; $resn_exch{" O3B COF"}="AO3* COF";
$resn_exch{" P3B COF"}="AP3* COF"; $resn_exch{" O7A COF"}=" AO7 COF"; $resn_exch{" O8A COF"}=" AO8 COF";
$resn_exch{" O9A COF"}=" AO9 COF"; $resn_exch{" C4B COF"}="AC4* COF"; $resn_exch{" O4B COF"}="AO4* COF";
$resn_exch{" C5B COF"}="AC5* COF"; $resn_exch{" O5B COF"}="AO5* COF"; $resn_exch{" P1A COF"}=" AP1 COF";
$resn_exch{" O1A COF"}=" AO1 COF"; $resn_exch{" O2A COF"}=" AO2 COF"; $resn_exch{" O3A COF"}=" AO3 COF";
$resn_exch{" P2A COF"}=" AP2 COF"; $resn_exch{" O4A COF"}=" AO4 COF"; $resn_exch{" O5A COF"}=" AO5 COF";
$resn_exch{" O6A COF"}=" AO6 COF"; $resn_exch{" CBP COF"}="PC11 COF"; $resn_exch{" CCP COF"}="PC12 COF";
$resn_exch{" CDP COF"}="PC13 COF"; $resn_exch{" CEP COF"}="PC14 COF"; $resn_exch{" CAP COF"}="PC10 COF";
$resn_exch{" OAP COF"}="PO10 COF"; $resn_exch{" C9P COF"}=" PC9 COF"; $resn_exch{" O9P COF"}=" PO9 COF";
$resn_exch{" N8P COF"}=" PN8 COF"; $resn_exch{" C7P COF"}=" PC7 COF"; $resn_exch{" C6P COF"}=" PC6 COF";
$resn_exch{" C5P COF"}=" PC5 COF"; $resn_exch{" O5P COF"}=" PO5 COF"; $resn_exch{" N4P COF"}=" PN4 COF";
$resn_exch{" C3P COF"}=" PC3 COF"; $resn_exch{" C2P COF"}=" PC2 COF"; $resn_exch{" S1P COF"}=" PS1 COF";
$resn_exch{" C1I COF"}=" IC1 COF"; $resn_exch{" C2I COF"}=" IC2 COF"; $resn_exch{" O1I COF"}=" IO1 COF";
$resn_exch{" C3I COF"}=" IC3 COF"; $resn_exch{" F1I COF"}=" IF1 COF"; $resn_exch{" F2I COF"}=" IF2 COF";
$resn_exch{" F3I COF"}=" IF3 COF"; $resn_exch{" C2B COG"}=" C2* COG"; $resn_exch{" C4B COG"}=" C4* COG";
$resn_exch{" C5B COG"}=" C5* COG"; $resn_exch{" C6B COG"}=" C6* COG"; $resn_exch{" C7B COG"}=" C7* COG";
$resn_exch{" N1' COG"}=" N1* COG"; $resn_exch{" N2' COG"}=" N2* COG"; $resn_exch{" N3' COG"}=" N3* COG";
$resn_exch{" N4' COG"}=" N4* COG"; $resn_exch{" C8' COG"}=" C8* COG"; $resn_exch{" N6' COG"}=" N6* COG";
$resn_exch{" NA  COH"}=" N_A COH"; $resn_exch{" NB  COH"}=" N_B COH"; $resn_exch{" NC  COH"}=" N_C COH";
$resn_exch{" ND  COH"}=" N_D COH"; $resn_exch{" N6A COK"}=" AN6 COK"; $resn_exch{" C6A COK"}=" AC6 COK";
$resn_exch{" N1A COK"}=" AN1 COK"; $resn_exch{" C2A COK"}=" AC2 COK"; $resn_exch{" N3A COK"}=" AN3 COK";
$resn_exch{" C4A COK"}=" AC4 COK"; $resn_exch{" C5A COK"}=" AC5 COK"; $resn_exch{" N7A COK"}=" AN7 COK";
$resn_exch{" C8A COK"}=" AC8 COK"; $resn_exch{" N9A COK"}=" AN9 COK"; $resn_exch{" C1B COK"}="AC1* COK";
$resn_exch{" O4B COK"}="AO4* COK"; $resn_exch{" C2B COK"}="AC2* COK"; $resn_exch{" O2B COK"}="AO2* COK";
$resn_exch{" C3B COK"}="AC3* COK"; $resn_exch{" O3B COK"}="AO3* COK"; $resn_exch{" P3B COK"}="AP3* COK";
$resn_exch{" O8A COK"}=" AO8 COK"; $resn_exch{" O9A COK"}=" AO9 COK"; $resn_exch{" O7A COK"}=" AO7 COK";
$resn_exch{" C4B COK"}="AC4* COK"; $resn_exch{" C5B COK"}="AC5* COK"; $resn_exch{" O5B COK"}="AO5* COK";
$resn_exch{" P1A COK"}=" AP1 COK"; $resn_exch{" O1A COK"}=" AO1 COK"; $resn_exch{" O2A COK"}=" AO2 COK";
$resn_exch{" O3A COK"}=" AO3 COK"; $resn_exch{" P2A COK"}=" AP2 COK"; $resn_exch{" O4A COK"}=" AO4 COK";
$resn_exch{" O5A COK"}=" AO5 COK"; $resn_exch{" O6A COK"}=" AO6 COK"; $resn_exch{" CCP COK"}="PC12 COK";
$resn_exch{" CBP COK"}="PC11 COK"; $resn_exch{" CDP COK"}="PC13 COK"; $resn_exch{" CEP COK"}="PC14 COK";
$resn_exch{" CAP COK"}="PC10 COK"; $resn_exch{" OAP COK"}="PO10 COK"; $resn_exch{" C9P COK"}=" PC9 COK";
$resn_exch{" O9P COK"}=" PO9 COK"; $resn_exch{" N8P COK"}=" PN8 COK"; $resn_exch{" C7P COK"}=" PC7 COK";
$resn_exch{" C6P COK"}=" PC6 COK"; $resn_exch{" C5P COK"}=" PC5 COK"; $resn_exch{" O5P COK"}=" PO5 COK";
$resn_exch{" N4P COK"}=" PN4 COK"; $resn_exch{" C3P COK"}=" PC3 COK"; $resn_exch{" C2P COK"}=" PC2 COK";
$resn_exch{" S1P COK"}=" PS1 COK"; $resn_exch{" C2B COQ"}=" C2* COQ"; $resn_exch{" C4B COQ"}=" C4* COQ";
$resn_exch{" C5B COQ"}=" C5* COQ"; $resn_exch{" C6B COQ"}=" C6* COQ"; $resn_exch{" C7' COQ"}=" C7* COQ";
$resn_exch{" C9' COQ"}=" C9* COQ"; $resn_exch{" CN' COQ"}=" CN* COQ"; $resn_exch{" N1' COQ"}=" N1* COQ";
$resn_exch{" N2' COQ"}=" N2* COQ"; $resn_exch{" N3' COQ"}=" N3* COQ"; $resn_exch{" N4' COQ"}=" N4* COQ";
$resn_exch{" N8' COQ"}=" N8* COQ"; $resn_exch{" N9' COQ"}=" N9* COQ"; $resn_exch{" N1A COS"}=" AN1 COS";
$resn_exch{" C2A COS"}=" AC2 COS"; $resn_exch{" N3A COS"}=" AN3 COS"; $resn_exch{" C4A COS"}=" AC4 COS";
$resn_exch{" C5A COS"}=" AC5 COS"; $resn_exch{" C6A COS"}=" AC6 COS"; $resn_exch{" N6A COS"}=" AN6 COS";
$resn_exch{" N7A COS"}=" AN7 COS"; $resn_exch{" C8A COS"}=" AC8 COS"; $resn_exch{" N9A COS"}=" AN9 COS";
$resn_exch{" C1B COS"}="AC1* COS"; $resn_exch{" C2B COS"}="AC2* COS"; $resn_exch{" O2B COS"}="AO2* COS";
$resn_exch{" C3B COS"}="AC3* COS"; $resn_exch{" O3B COS"}="AO3* COS"; $resn_exch{" P3B COS"}="AP3* COS";
$resn_exch{" O7A COS"}=" AO7 COS"; $resn_exch{" O8A COS"}=" AO8 COS"; $resn_exch{" O9A COS"}=" AO9 COS";
$resn_exch{" C4B COS"}="AC4* COS"; $resn_exch{" O4B COS"}="AO4* COS"; $resn_exch{" C5B COS"}="AC5* COS";
$resn_exch{" O5B COS"}="AO5* COS"; $resn_exch{" P1A COS"}=" AP1 COS"; $resn_exch{" O1A COS"}=" AO1 COS";
$resn_exch{" O2A COS"}=" AO2 COS"; $resn_exch{" O3A COS"}=" AO3 COS"; $resn_exch{" P2A COS"}=" AP2 COS";
$resn_exch{" O4A COS"}=" AO4 COS"; $resn_exch{" O5A COS"}=" AO5 COS"; $resn_exch{" O6A COS"}=" AO6 COS";
$resn_exch{" CBP COS"}="PC11 COS"; $resn_exch{" CCP COS"}="PC12 COS"; $resn_exch{" CDP COS"}="PC13 COS";
$resn_exch{" CEP COS"}="PC14 COS"; $resn_exch{" CAP COS"}="PC10 COS"; $resn_exch{" OAP COS"}="PO10 COS";
$resn_exch{" C9P COS"}=" PC9 COS"; $resn_exch{" O9P COS"}=" PO9 COS"; $resn_exch{" N8P COS"}=" PN8 COS";
$resn_exch{" C7P COS"}=" PC7 COS"; $resn_exch{" C6P COS"}=" PC6 COS"; $resn_exch{" C5P COS"}=" PC5 COS";
$resn_exch{" O5P COS"}=" PO5 COS"; $resn_exch{" N4P COS"}=" PN4 COS"; $resn_exch{" C3P COS"}=" PC3 COS";
$resn_exch{" C2P COS"}=" PC2 COS"; $resn_exch{" S1P COS"}=" PS1 COS"; $resn_exch{" S'P COS"}=" PS' COS";
$resn_exch{" C11 COT"}="CD2T COT"; $resn_exch{" C12 COT"}="CE2T COT"; $resn_exch{" C13 COT"}="CE3T COT";
$resn_exch{" C14 COT"}="CD1T COT"; $resn_exch{" N11 COT"}="NE1T COT"; $resn_exch{" C15 COT"}="CZ2T COT";
$resn_exch{" C16 COT"}="CZ3T COT"; $resn_exch{" C17 COT"}="CH2T COT"; $resn_exch{" O11 COT"}="OPB3 COT";
$resn_exch{" O12 COT"}="OPB1 COT"; $resn_exch{" O13 COT"}="OPB2 COT"; $resn_exch{" O14 COT"}="OPA3 COT";
$resn_exch{" O3B COT"}="O3*A COT"; $resn_exch{" O15 COT"}="OPA2 COT"; $resn_exch{" O16 COT"}="OPA1 COT";
$resn_exch{" O5B COT"}="O5*A COT"; $resn_exch{" C5B COT"}="C5*A COT"; $resn_exch{" C4B COT"}="C4*A COT";
$resn_exch{" C3B COT"}="C3*A COT"; $resn_exch{" O2B COT"}="O2*A COT"; $resn_exch{" O4B COT"}="O4*A COT";
$resn_exch{" O17 COT"}="OPC1 COT"; $resn_exch{" O18 COT"}="OPC2 COT"; $resn_exch{" O19 COT"}="OPC3 COT";
$resn_exch{" C2B COT"}="C2*A COT"; $resn_exch{" C1B COT"}="C1*A COT"; $resn_exch{" OAP COT"}="O10P COT";
$resn_exch{" CBP COT"}="C11P COT"; $resn_exch{" OCP COT"}="O12P COT"; $resn_exch{" CDP COT"}="C13P COT";
$resn_exch{" CGP COT"}="C16P COT"; $resn_exch{" CEP COT"}="C14P COT"; $resn_exch{" CFP COT"}="C15P COT";
$resn_exch{" N1A COZ"}=" AN1 COZ"; $resn_exch{" C2A COZ"}=" AC2 COZ"; $resn_exch{" N3A COZ"}=" AN3 COZ";
$resn_exch{" C4A COZ"}=" AC4 COZ"; $resn_exch{" C5A COZ"}=" AC5 COZ"; $resn_exch{" C6A COZ"}=" AC6 COZ";
$resn_exch{" N6A COZ"}=" AN6 COZ"; $resn_exch{" N7A COZ"}=" AN7 COZ"; $resn_exch{" C8A COZ"}=" AC8 COZ";
$resn_exch{" N9A COZ"}=" AN9 COZ"; $resn_exch{" C1B COZ"}="AC1* COZ"; $resn_exch{" C2B COZ"}="AC2* COZ";
$resn_exch{" O2B COZ"}="AO2* COZ"; $resn_exch{" C3B COZ"}="AC3* COZ"; $resn_exch{" O3B COZ"}="AO3* COZ";
$resn_exch{" P3B COZ"}="AP3* COZ"; $resn_exch{" O7A COZ"}=" AO7 COZ"; $resn_exch{" O8A COZ"}=" AO8 COZ";
$resn_exch{" O9A COZ"}=" AO9 COZ"; $resn_exch{" C4B COZ"}="AC4* COZ"; $resn_exch{" O4B COZ"}="AO4* COZ";
$resn_exch{" C5B COZ"}="AC5* COZ"; $resn_exch{" O5B COZ"}="AO5* COZ"; $resn_exch{" P1A COZ"}=" AP1 COZ";
$resn_exch{" O1A COZ"}=" AO1 COZ"; $resn_exch{" O2A COZ"}=" AO2 COZ"; $resn_exch{" O3A COZ"}=" AO3 COZ";
$resn_exch{" P2A COZ"}=" AP2 COZ"; $resn_exch{" O4A COZ"}=" AO4 COZ"; $resn_exch{" O5A COZ"}=" AO5 COZ";
$resn_exch{" O6A COZ"}=" AO6 COZ"; $resn_exch{" CBP COZ"}="PC11 COZ"; $resn_exch{" CCP COZ"}="PC12 COZ";
$resn_exch{" CDP COZ"}="PC13 COZ"; $resn_exch{" CEP COZ"}="PC14 COZ"; $resn_exch{" CAP COZ"}="PC10 COZ";
$resn_exch{" OAP COZ"}="PO10 COZ"; $resn_exch{" C9P COZ"}=" PC9 COZ"; $resn_exch{" O9P COZ"}=" PO9 COZ";
$resn_exch{" N8P COZ"}=" PN8 COZ"; $resn_exch{" C7P COZ"}=" PC7 COZ"; $resn_exch{" C6P COZ"}=" PC6 COZ";
$resn_exch{" C5P COZ"}=" PC5 COZ"; $resn_exch{" O5P COZ"}=" PO5 COZ"; $resn_exch{" N4P COZ"}=" PN4 COZ";
$resn_exch{" C3P COZ"}=" PC3 COZ"; $resn_exch{" C2P COZ"}=" PC2 COZ"; $resn_exch{" S1P COZ"}=" PS1 COZ";
$resn_exch{" O5D CPA"}="CO5' CPA"; $resn_exch{" C5X CPA"}="CC5' CPA"; $resn_exch{" C4X CPA"}="CC4' CPA";
$resn_exch{" O4D CPA"}="CO4' CPA"; $resn_exch{" C3X CPA"}="CC3' CPA"; $resn_exch{" O3D CPA"}="CO3' CPA";
$resn_exch{" C2X CPA"}="CC2' CPA"; $resn_exch{" C1X CPA"}="CC1' CPA"; $resn_exch{" N1C CPA"}=" CN1 CPA";
$resn_exch{" N3C CPA"}=" CN3 CPA"; $resn_exch{" O2C CPA"}=" CO2 CPA"; $resn_exch{" N4C CPA"}=" CN4 CPA";
$resn_exch{" O5B CPA"}="AO5' CPA"; $resn_exch{" C5B CPA"}="AC5' CPA"; $resn_exch{" C4B CPA"}="AC4' CPA";
$resn_exch{" O4B CPA"}="AO4' CPA"; $resn_exch{" C3B CPA"}="AC3' CPA"; $resn_exch{" O3B CPA"}="AO3' CPA";
$resn_exch{" C2B CPA"}="AC2' CPA"; $resn_exch{" C1B CPA"}="AC1' CPA"; $resn_exch{" N9A CPA"}=" AN9 CPA";
$resn_exch{" C8A CPA"}=" AC8 CPA"; $resn_exch{" N7A CPA"}=" AN7 CPA"; $resn_exch{" C5A CPA"}=" AC5 CPA";
$resn_exch{" C6A CPA"}=" AC6 CPA"; $resn_exch{" N6A CPA"}=" AN6 CPA"; $resn_exch{" N1A CPA"}=" AN1 CPA";
$resn_exch{" C2A CPA"}=" AC2 CPA"; $resn_exch{" N3A CPA"}=" AN3 CPA"; $resn_exch{" C4A CPA"}=" AC4 CPA";
$resn_exch{" O5' CPG"}=" O5* CPG"; $resn_exch{" C5' CPG"}=" C5* CPG"; $resn_exch{" C4' CPG"}=" C4* CPG";
$resn_exch{" O4' CPG"}=" O4* CPG"; $resn_exch{" C3' CPG"}=" C3* CPG"; $resn_exch{" O3' CPG"}=" O3* CPG";
$resn_exch{" C2' CPG"}=" C2* CPG"; $resn_exch{" O2' CPG"}=" O2* CPG"; $resn_exch{" C1' CPG"}=" C1* CPG";
$resn_exch{" C1' CPH"}=" C1* CPH"; $resn_exch{" O1' CPH"}=" O1* CPH"; $resn_exch{" C2' CPH"}=" C2* CPH";
$resn_exch{" O2' CPH"}=" O2* CPH"; $resn_exch{" C3' CPH"}=" C3* CPH"; $resn_exch{" O3' CPH"}=" O3* CPH";
$resn_exch{" C4' CPH"}=" C4* CPH"; $resn_exch{" O4' CPH"}=" O4* CPH"; $resn_exch{" C5' CPH"}=" C5* CPH";
$resn_exch{" CJ' CPO"}="C19* CPO"; $resn_exch{" O5' CPR"}=" O5* CPR"; $resn_exch{" C5' CPR"}=" C5* CPR";
$resn_exch{" C4' CPR"}=" C4* CPR"; $resn_exch{" O4' CPR"}=" O4* CPR"; $resn_exch{" C3' CPR"}=" C3* CPR";
$resn_exch{" O3' CPR"}=" O3* CPR"; $resn_exch{" C2' CPR"}=" C2* CPR"; $resn_exch{" O2' CPR"}=" O2* CPR";
$resn_exch{" C1' CPR"}=" C1* CPR"; $resn_exch{" CA' CPV"}=" CA* CPV"; $resn_exch{" CB' CPV"}=" CB* CPV";
$resn_exch{" C1B CR3"}="C1'' CR3"; $resn_exch{" C2B CR3"}="C2'' CR3"; $resn_exch{" C3B CR3"}="C3'' CR3";
$resn_exch{" C4B CR3"}="C4'' CR3"; $resn_exch{" C5B CR3"}="C5'' CR3"; $resn_exch{" C1B CR9"}="C1'' CR9";
$resn_exch{" C2B CR9"}="C2'' CR9"; $resn_exch{" C3B CR9"}="C3'' CR9"; $resn_exch{" C4B CR9"}="C4'' CR9";
$resn_exch{" C5B CR9"}="C5'' CR9"; $resn_exch{" C6B CR9"}="C6'' CR9"; $resn_exch{" C1' CRH"}=" C1* CRH";
$resn_exch{" C2' CRH"}=" C2* CRH"; $resn_exch{" C3' CRH"}=" C3* CRH"; $resn_exch{" C4' CRH"}=" C4* CRH";
$resn_exch{" O1' CRH"}=" O1* CRH"; $resn_exch{" O2' CRH"}=" O2* CRH"; $resn_exch{" O3' CRH"}=" O3* CRH";
$resn_exch{" O4' CRH"}=" O4* CRH"; $resn_exch{" OXT CS3"}=" O   CS3"; $resn_exch{" O   CS3"}=" OXT CS3";
$resn_exch{" OXT CS4"}=" O   CS4"; $resn_exch{" O   CS4"}=" OXT CS4"; $resn_exch{" OAP CS8"}="PO10 CS8";
$resn_exch{" CAP CS8"}="PC10 CS8"; $resn_exch{" C9P CS8"}=" PC9 CS8"; $resn_exch{" O9P CS8"}=" PO9 CS8";
$resn_exch{" N8P CS8"}=" PN8 CS8"; $resn_exch{" C7P CS8"}=" PC7 CS8"; $resn_exch{" C6P CS8"}=" PC6 CS8";
$resn_exch{" C5P CS8"}=" PC5 CS8"; $resn_exch{" O5P CS8"}=" PO5 CS8"; $resn_exch{" N4P CS8"}=" PN4 CS8";
$resn_exch{" C3P CS8"}=" PC3 CS8"; $resn_exch{" C2P CS8"}=" PC2 CS8"; $resn_exch{" S1P CS8"}=" PS1 CS8";
$resn_exch{" C1' CS8"}=" C1* CS8"; $resn_exch{" O1' CS8"}=" O1* CS8"; $resn_exch{" C2' CS8"}=" C2* CS8";
$resn_exch{" S3' CS8"}=" S3* CS8"; $resn_exch{" C4' CS8"}=" C4* CS8"; $resn_exch{" C5' CS8"}=" C5* CS8";
$resn_exch{" C6' CS8"}=" C6* CS8"; $resn_exch{" C7' CS8"}=" C7* CS8"; $resn_exch{" C8' CS8"}=" C8* CS8";
$resn_exch{" CBP CS8"}="PC11 CS8"; $resn_exch{" CDP CS8"}="PC13 CS8"; $resn_exch{" CEP CS8"}="PC14 CS8";
$resn_exch{" CCP CS8"}="PC12 CS8"; $resn_exch{" O6A CS8"}=" AO6 CS8"; $resn_exch{" P2A CS8"}=" AP2 CS8";
$resn_exch{" O4A CS8"}=" AO4 CS8"; $resn_exch{" O5A CS8"}=" AO5 CS8"; $resn_exch{" O3A CS8"}=" AO3 CS8";
$resn_exch{" P1A CS8"}=" AP1 CS8"; $resn_exch{" O1A CS8"}=" AO1 CS8"; $resn_exch{" O2A CS8"}=" AO2 CS8";
$resn_exch{" O5B CS8"}="AO5* CS8"; $resn_exch{" C5B CS8"}="AC5* CS8"; $resn_exch{" C4B CS8"}="AC4* CS8";
$resn_exch{" O4B CS8"}="AO4* CS8"; $resn_exch{" C1B CS8"}="AC1* CS8"; $resn_exch{" N9A CS8"}=" AN9 CS8";
$resn_exch{" C8A CS8"}=" AC8 CS8"; $resn_exch{" N7A CS8"}=" AN7 CS8"; $resn_exch{" C5A CS8"}=" AC5 CS8";
$resn_exch{" C6A CS8"}=" AC6 CS8"; $resn_exch{" N1A CS8"}=" AN1 CS8"; $resn_exch{" C2A CS8"}=" AC2 CS8";
$resn_exch{" N3A CS8"}=" AN3 CS8"; $resn_exch{" C4A CS8"}=" AC4 CS8"; $resn_exch{" N6A CS8"}=" AN6 CS8";
$resn_exch{" C2B CS8"}="AC2* CS8"; $resn_exch{" O2B CS8"}="AO2* CS8"; $resn_exch{" C3B CS8"}="AC3* CS8";
$resn_exch{" O3B CS8"}="AO3* CS8"; $resn_exch{" P3B CS8"}="AP3* CS8"; $resn_exch{" O7A CS8"}=" AO7 CS8";
$resn_exch{" O8A CS8"}=" AO8 CS8"; $resn_exch{" O9A CS8"}=" AO9 CS8"; $resn_exch{" C3' CSF"}=" C3* CSF";
$resn_exch{" C2' CSF"}=" C2* CSF"; $resn_exch{" C1' CSF"}=" C1* CSF"; $resn_exch{" O4' CSF"}=" O4* CSF";
$resn_exch{" C4' CSF"}=" C4* CSF"; $resn_exch{" O5' CSF"}=" O5* CSF"; $resn_exch{" C5' CSF"}=" C5* CSF";
$resn_exch{" O3' CSF"}=" O3* CSF"; $resn_exch{" O2' CSF"}=" O2* CSF"; $resn_exch{" O5' CSL"}=" O5* CSL";
$resn_exch{" C5' CSL"}=" C5* CSL"; $resn_exch{" C4' CSL"}=" C4* CSL"; $resn_exch{" O4' CSL"}=" O4* CSL";
$resn_exch{" C1' CSL"}=" C1* CSL"; $resn_exch{" C2' CSL"}=" C2* CSL"; $resn_exch{"SE2' CSL"}="SE2* CSL";
$resn_exch{" CA' CSL"}=" CA* CSL"; $resn_exch{" C3' CSL"}=" C3* CSL"; $resn_exch{" O3' CSL"}=" O3* CSL";
$resn_exch{" C4D CTC"}=" C4\" CTC"; $resn_exch{" C1' CTD"}=" C1* CTD"; $resn_exch{" C2' CTD"}=" C2* CTD";
$resn_exch{" O2' CTD"}=" O2* CTD"; $resn_exch{" C3' CTD"}=" C3* CTD"; $resn_exch{" C4' CTD"}=" C4* CTD";
$resn_exch{" O4' CTD"}=" O4* CTD"; $resn_exch{" O3' CTD"}=" O3* CTD"; $resn_exch{" C5' CTD"}=" C5* CTD";
$resn_exch{" O5' CTD"}=" O5* CTD"; $resn_exch{" OP1 CTG"}=" O1P CTG"; $resn_exch{" OP2 CTG"}=" O2P CTG";
$resn_exch{" O5' CTG"}=" O5* CTG"; $resn_exch{" C5' CTG"}=" C5* CTG"; $resn_exch{" C4' CTG"}=" C4* CTG";
$resn_exch{" O4' CTG"}=" O4* CTG"; $resn_exch{" C1' CTG"}=" C1* CTG"; $resn_exch{" C2' CTG"}=" C2* CTG";
$resn_exch{" C3' CTG"}=" C3* CTG"; $resn_exch{" O3' CTG"}=" O3* CTG"; $resn_exch{" OP3 CTG"}=" O3P CTG";
$resn_exch{" O5' CTN"}=" O5* CTN"; $resn_exch{" C5' CTN"}=" C5* CTN"; $resn_exch{" C4' CTN"}=" C4* CTN";
$resn_exch{" O4' CTN"}=" O4* CTN"; $resn_exch{" C1' CTN"}=" C1* CTN"; $resn_exch{" C2' CTN"}=" C2* CTN";
$resn_exch{" O2' CTN"}=" O2* CTN"; $resn_exch{" C3' CTN"}=" C3* CTN"; $resn_exch{" O3' CTN"}=" O3* CTN";
$resn_exch{" C1' CTP"}=" C1* CTP"; $resn_exch{" C2' CTP"}=" C2* CTP"; $resn_exch{" O2' CTP"}=" O2* CTP";
$resn_exch{" C3' CTP"}=" C3* CTP"; $resn_exch{" C4' CTP"}=" C4* CTP"; $resn_exch{" O4' CTP"}=" O4* CTP";
$resn_exch{" O3' CTP"}=" O3* CTP"; $resn_exch{" C5' CTP"}=" C5* CTP"; $resn_exch{" O5' CTP"}=" O5* CTP";
$resn_exch{" ON1 CXM"}=" O1  CXM"; $resn_exch{" ON2 CXM"}=" O2  CXM"; $resn_exch{" O5D CXR"}="RO5* CXR";
$resn_exch{" C5D CXR"}="RC5* CXR"; $resn_exch{" C4D CXR"}="RC4* CXR"; $resn_exch{" C3D CXR"}="RC3* CXR";
$resn_exch{" O3D CXR"}="RO3* CXR"; $resn_exch{" C2D CXR"}="RC2* CXR"; $resn_exch{" O2D CXR"}="RO2* CXR";
$resn_exch{" O4D CXR"}="RO4* CXR"; $resn_exch{" C1D CXR"}="RC1* CXR"; $resn_exch{" C1' CXR"}=" C1* CXR";
$resn_exch{" O4' CXR"}=" O4* CXR"; $resn_exch{" C2' CXR"}=" C2* CXR"; $resn_exch{" O2' CXR"}=" O2* CXR";
$resn_exch{" C3' CXR"}=" C3* CXR"; $resn_exch{" O3' CXR"}=" O3* CXR"; $resn_exch{" C4' CXR"}=" C4* CXR";
$resn_exch{" C5' CXR"}=" C5* CXR"; $resn_exch{" O5' CXR"}=" O5* CXR"; $resn_exch{" O5D CXY"}=" O5* CXY";
$resn_exch{" C5D CXY"}=" C5* CXY"; $resn_exch{" C4D CXY"}=" C4* CXY"; $resn_exch{" O4D CXY"}=" O4* CXY";
$resn_exch{" C3D CXY"}=" C3* CXY"; $resn_exch{" O3D CXY"}=" O3* CXY"; $resn_exch{" C2D CXY"}=" C2* CXY";
$resn_exch{" O2D CXY"}=" O2* CXY"; $resn_exch{" C1D CXY"}=" C1* CXY"; $resn_exch{" NA  CYB"}=" N_A CYB";
$resn_exch{" NB  CYB"}=" N_B CYB"; $resn_exch{" OB  CYB"}=" O_B CYB"; $resn_exch{" NC  CYB"}=" N_C CYB";
$resn_exch{" OC  CYB"}=" O_C CYB"; $resn_exch{" ND  CYB"}=" N_D CYB"; $resn_exch{" NA  CYC"}=" N_A CYC";
$resn_exch{" NB  CYC"}=" N_B CYC"; $resn_exch{" OB  CYC"}=" O_B CYC"; $resn_exch{" NC  CYC"}=" N_C CYC";
$resn_exch{" OC  CYC"}=" O_C CYC"; $resn_exch{" ND  CYC"}=" N_D CYC"; $resn_exch{"CL14 D05"}="4CL1 D05";
$resn_exch{"CL16 D05"}="6CL1 D05"; $resn_exch{"CL12 D05"}="2CL1 D05"; $resn_exch{" CAX D18"}="'C10 D18";
$resn_exch{" CAX D19"}="'C10 D19"; $resn_exch{" OP3 D1P"}=" O3P D1P"; $resn_exch{" OP1 D1P"}=" O1P D1P";
$resn_exch{" OP2 D1P"}=" O2P D1P"; $resn_exch{" O5' D1P"}=" O5* D1P"; $resn_exch{" C5' D1P"}=" C5* D1P";
$resn_exch{" C4' D1P"}=" C4* D1P"; $resn_exch{" O4' D1P"}=" O4* D1P"; $resn_exch{" C3' D1P"}=" C3* D1P";
$resn_exch{" O3' D1P"}=" O3* D1P"; $resn_exch{" C2' D1P"}=" C2* D1P"; $resn_exch{" C1' D1P"}=" C1* D1P";
$resn_exch{" O1' D1P"}=" O1* D1P"; $resn_exch{" CAX D24"}="C10' D24"; $resn_exch{" CBX D24"}="C11' D24";
$resn_exch{" CCX D24"}="C12' D24"; $resn_exch{" OP1  D3"}=" O1P  D3"; $resn_exch{" OP2  D3"}=" O2P  D3";
$resn_exch{" OP3  D3"}=" O3P  D3"; $resn_exch{" O5'  D3"}=" O5*  D3"; $resn_exch{" C5'  D3"}=" C5*  D3";
$resn_exch{" C4'  D3"}=" C4*  D3"; $resn_exch{" O4'  D3"}=" O4*  D3"; $resn_exch{" C3'  D3"}=" C3*  D3";
$resn_exch{" O3'  D3"}=" O3*  D3"; $resn_exch{" C2'  D3"}=" C2*  D3"; $resn_exch{" C1'  D3"}=" C1*  D3";
$resn_exch{" CAX D34"}="C10' D34"; $resn_exch{" CBX D34"}="C11' D34"; $resn_exch{" CAX D35"}="C10' D35";
$resn_exch{" CBX D35"}="C11' D35"; $resn_exch{" CCX D35"}="C12' D35"; $resn_exch{" CDX D35"}="C13' D35";
$resn_exch{" O5' D3T"}=" O5* D3T"; $resn_exch{" C5' D3T"}=" C5* D3T"; $resn_exch{" C4' D3T"}=" C4* D3T";
$resn_exch{" O4' D3T"}=" O4* D3T"; $resn_exch{" C1' D3T"}=" C1* D3T"; $resn_exch{" C2' D3T"}=" C2* D3T";
$resn_exch{" C3' D3T"}=" C3* D3T"; $resn_exch{" OP2 D4M"}=" O2P D4M"; $resn_exch{" OP1 D4M"}=" O1P D4M";
$resn_exch{" OP3 D4M"}=" O3P D4M"; $resn_exch{" O5' D4M"}=" O5* D4M"; $resn_exch{" C5' D4M"}=" C5* D4M";
$resn_exch{" C4' D4M"}=" C4* D4M"; $resn_exch{" O4' D4M"}=" O4* D4M"; $resn_exch{" C3' D4M"}=" C3* D4M";
$resn_exch{" C2' D4M"}=" C2* D4M"; $resn_exch{" C1' D4M"}=" C1* D4M"; $resn_exch{" O5' D5M"}=" O5* D5M";
$resn_exch{" C5' D5M"}=" C5* D5M"; $resn_exch{" C4' D5M"}=" C4* D5M"; $resn_exch{" O4' D5M"}=" O4* D5M";
$resn_exch{" C3' D5M"}=" C3* D5M"; $resn_exch{" O3' D5M"}=" O3* D5M"; $resn_exch{" C2' D5M"}=" C2* D5M";
$resn_exch{" C1' D5M"}=" C1* D5M"; $resn_exch{" OP3  DA"}=" O3P  DA"; $resn_exch{" OP1  DA"}=" O1P  DA";
$resn_exch{" OP2  DA"}=" O2P  DA"; $resn_exch{" O5'  DA"}=" O5*  DA"; $resn_exch{" C5'  DA"}=" C5*  DA";
$resn_exch{" C4'  DA"}=" C4*  DA"; $resn_exch{" O4'  DA"}=" O4*  DA"; $resn_exch{" C3'  DA"}=" C3*  DA";
$resn_exch{" O3'  DA"}=" O3*  DA"; $resn_exch{" C2'  DA"}=" C2*  DA"; $resn_exch{" C1'  DA"}=" C1*  DA";
$resn_exch{" O5' DAD"}=" O5* DAD"; $resn_exch{" C5' DAD"}=" C5* DAD"; $resn_exch{" C4' DAD"}=" C4* DAD";
$resn_exch{" O4' DAD"}=" O4* DAD"; $resn_exch{" C3' DAD"}=" C3* DAD"; $resn_exch{" C2' DAD"}=" C2* DAD";
$resn_exch{" C1' DAD"}=" C1* DAD"; $resn_exch{" C1' DAP"}=" C1* DAP"; $resn_exch{" C2' DAP"}=" C2* DAP";
$resn_exch{" C3' DAP"}=" C3* DAP"; $resn_exch{" C4' DAP"}=" C4* DAP"; $resn_exch{" C5' DAP"}=" C5* DAP";
$resn_exch{" C6' DAP"}=" C6* DAP"; $resn_exch{" O5' DAT"}=" O5* DAT"; $resn_exch{" C5' DAT"}=" C5* DAT";
$resn_exch{" C4' DAT"}=" C4* DAT"; $resn_exch{" O4' DAT"}=" O4* DAT"; $resn_exch{" C3' DAT"}=" C3* DAT";
$resn_exch{" O3' DAT"}=" O3* DAT"; $resn_exch{" C2' DAT"}=" C2* DAT"; $resn_exch{" C1' DAT"}=" C1* DAT";
$resn_exch{" O5' DBG"}=" O5* DBG"; $resn_exch{" C5' DBG"}=" C5* DBG"; $resn_exch{" C4' DBG"}=" C4* DBG";
$resn_exch{" O4' DBG"}=" O4* DBG"; $resn_exch{" C2' DBG"}=" C2* DBG"; $resn_exch{" O2' DBG"}=" O2* DBG";
$resn_exch{" C3' DBG"}=" C3* DBG"; $resn_exch{" O3' DBG"}=" O3* DBG"; $resn_exch{" C1' DBG"}=" C1* DBG";
$resn_exch{" O1' DBL"}=" O1\" DBL"; $resn_exch{" C2B DBL"}=" C2\" DBL"; $resn_exch{" OP3  DC"}=" O3P  DC";
$resn_exch{" OP1  DC"}=" O1P  DC"; $resn_exch{" OP2  DC"}=" O2P  DC"; $resn_exch{" O5'  DC"}=" O5*  DC";
$resn_exch{" C5'  DC"}=" C5*  DC"; $resn_exch{" C4'  DC"}=" C4*  DC"; $resn_exch{" O4'  DC"}=" O4*  DC";
$resn_exch{" C3'  DC"}=" C3*  DC"; $resn_exch{" O3'  DC"}=" O3*  DC"; $resn_exch{" C2'  DC"}=" C2*  DC";
$resn_exch{" C1'  DC"}=" C1*  DC"; $resn_exch{" N1A DCA"}=" AN1 DCA"; $resn_exch{" C2A DCA"}=" AC2 DCA";
$resn_exch{" N3A DCA"}=" AN3 DCA"; $resn_exch{" C4A DCA"}=" AC4 DCA"; $resn_exch{" C5A DCA"}=" AC5 DCA";
$resn_exch{" C6A DCA"}=" AC6 DCA"; $resn_exch{" N6A DCA"}=" AN6 DCA"; $resn_exch{" N7A DCA"}=" AN7 DCA";
$resn_exch{" C8A DCA"}=" AC8 DCA"; $resn_exch{" N9A DCA"}=" AN9 DCA"; $resn_exch{" C1B DCA"}="AC1* DCA";
$resn_exch{" C2B DCA"}="AC2* DCA"; $resn_exch{" O2B DCA"}="AO2* DCA"; $resn_exch{" C3B DCA"}="AC3* DCA";
$resn_exch{" O3B DCA"}="AO3* DCA"; $resn_exch{" P3B DCA"}="AP3* DCA"; $resn_exch{" O7A DCA"}=" AO7 DCA";
$resn_exch{" O8A DCA"}=" AO8 DCA"; $resn_exch{" O9A DCA"}=" AO9 DCA"; $resn_exch{" C4B DCA"}="AC4* DCA";
$resn_exch{" O4B DCA"}="AO4* DCA"; $resn_exch{" C5B DCA"}="AC5* DCA"; $resn_exch{" O5B DCA"}="AO5* DCA";
$resn_exch{" P1A DCA"}=" AP1 DCA"; $resn_exch{" O1A DCA"}=" AO1 DCA"; $resn_exch{" O2A DCA"}=" AO2 DCA";
$resn_exch{" O3A DCA"}=" AO3 DCA"; $resn_exch{" P2A DCA"}=" AP2 DCA"; $resn_exch{" O4A DCA"}=" AO4 DCA";
$resn_exch{" O5A DCA"}=" AO5 DCA"; $resn_exch{" O6A DCA"}=" AO6 DCA"; $resn_exch{" CBP DCA"}="PC11 DCA";
$resn_exch{" CCP DCA"}="PC12 DCA"; $resn_exch{" CDP DCA"}="PC13 DCA"; $resn_exch{" CEP DCA"}="PC14 DCA";
$resn_exch{" CAP DCA"}="PC10 DCA"; $resn_exch{" OAP DCA"}="PO10 DCA"; $resn_exch{" C9P DCA"}=" PC9 DCA";
$resn_exch{" O9P DCA"}=" PO9 DCA"; $resn_exch{" N8P DCA"}=" PN8 DCA"; $resn_exch{" C7P DCA"}=" PC7 DCA";
$resn_exch{" C6P DCA"}=" PC6 DCA"; $resn_exch{" C5P DCA"}=" PC5 DCA"; $resn_exch{" O5P DCA"}=" PO5 DCA";
$resn_exch{" N4P DCA"}=" PN4 DCA"; $resn_exch{" C3P DCA"}=" PC3 DCA"; $resn_exch{" C2P DCA"}=" PC2 DCA";
$resn_exch{" OP1 DCG"}=" O1P DCG"; $resn_exch{" OP2 DCG"}=" O2P DCG"; $resn_exch{" OP3 DCG"}=" O3P DCG";
$resn_exch{" O5' DCG"}=" O5* DCG"; $resn_exch{" C5' DCG"}=" C5* DCG"; $resn_exch{" C4' DCG"}=" C4* DCG";
$resn_exch{" O4' DCG"}=" O4* DCG"; $resn_exch{" C3' DCG"}=" C3* DCG"; $resn_exch{" O3' DCG"}=" O3* DCG";
$resn_exch{" C2' DCG"}=" C2* DCG"; $resn_exch{" C1' DCG"}=" C1* DCG"; $resn_exch{" C1' DCM"}=" C1* DCM";
$resn_exch{" C2' DCM"}=" C2* DCM"; $resn_exch{" C3' DCM"}=" C3* DCM"; $resn_exch{" C4' DCM"}=" C4* DCM";
$resn_exch{" O4' DCM"}=" O4* DCM"; $resn_exch{" O3' DCM"}=" O3* DCM"; $resn_exch{" C5' DCM"}=" C5* DCM";
$resn_exch{" O5' DCM"}=" O5* DCM"; $resn_exch{" C1' DCP"}=" C1* DCP"; $resn_exch{" C2' DCP"}=" C2* DCP";
$resn_exch{" C3' DCP"}=" C3* DCP"; $resn_exch{" C4' DCP"}=" C4* DCP"; $resn_exch{" O4' DCP"}=" O4* DCP";
$resn_exch{" O3' DCP"}=" O3* DCP"; $resn_exch{" C5' DCP"}=" C5* DCP"; $resn_exch{" O5' DCP"}=" O5* DCP";
$resn_exch{" C1' DCT"}=" C1* DCT"; $resn_exch{" C2' DCT"}=" C2* DCT"; $resn_exch{" C3' DCT"}=" C3* DCT";
$resn_exch{" C4' DCT"}=" C4* DCT"; $resn_exch{" O4' DCT"}=" O4* DCT"; $resn_exch{" C5' DCT"}=" C5* DCT";
$resn_exch{" O5' DCT"}=" O5* DCT"; $resn_exch{" C1' DCZ"}=" C1* DCZ"; $resn_exch{" C2' DCZ"}=" C2* DCZ";
$resn_exch{" C3' DCZ"}=" C3* DCZ"; $resn_exch{" C4' DCZ"}=" C4* DCZ"; $resn_exch{" O4' DCZ"}=" O4* DCZ";
$resn_exch{" O3' DCZ"}=" O3* DCZ"; $resn_exch{" C5' DCZ"}=" C5* DCZ"; $resn_exch{" O5' DCZ"}=" O5* DCZ";
$resn_exch{" OP3 DDG"}=" O3P DDG"; $resn_exch{" OP1 DDG"}=" O1P DDG"; $resn_exch{" OP2 DDG"}=" O2P DDG";
$resn_exch{" O5' DDG"}=" O5* DDG"; $resn_exch{" C5' DDG"}=" C5* DDG"; $resn_exch{" C4' DDG"}=" C4* DDG";
$resn_exch{" O4' DDG"}=" O4* DDG"; $resn_exch{" C3' DDG"}=" C3* DDG"; $resn_exch{" C2' DDG"}=" C2* DDG";
$resn_exch{" C1' DDG"}=" C1* DDG"; $resn_exch{" NA  DDH"}=" N_A DDH"; $resn_exch{" NB  DDH"}=" N_B DDH";
$resn_exch{" NC  DDH"}=" N_C DDH"; $resn_exch{" ND  DDH"}=" N_D DDH"; $resn_exch{" OP3 DDN"}=" O3P DDN";
$resn_exch{" OP1 DDN"}=" O1P DDN"; $resn_exch{" OP2 DDN"}=" O2P DDN"; $resn_exch{" O5' DDN"}=" O5* DDN";
$resn_exch{" C5' DDN"}=" C5* DDN"; $resn_exch{" C4' DDN"}=" C4* DDN"; $resn_exch{" O4' DDN"}=" O4* DDN";
$resn_exch{" C3' DDN"}=" C3* DDN"; $resn_exch{" O3' DDN"}=" O3* DDN"; $resn_exch{" C2' DDN"}=" C2* DDN";
$resn_exch{" C1' DDN"}=" C1* DDN"; $resn_exch{" S'  DDT"}=" S*  DDT"; $resn_exch{" O1' DDT"}=" O1* DDT";
$resn_exch{" O2' DDT"}=" O2* DDT"; $resn_exch{" C1' DDT"}=" C1* DDT"; $resn_exch{" C2' DDT"}=" C2* DDT";
$resn_exch{" C3' DDT"}=" C3* DDT"; $resn_exch{" C4' DDT"}=" C4* DDT"; $resn_exch{" C4X DDT"}="C4A* DDT";
$resn_exch{" C5' DDT"}=" C5* DDT"; $resn_exch{" C6' DDT"}=" C6* DDT"; $resn_exch{" C7' DDT"}=" C7* DDT";
$resn_exch{" C8' DDT"}=" C8* DDT"; $resn_exch{" C8X DDT"}="C8A* DDT"; $resn_exch{" C9' DDT"}=" C9* DDT";
$resn_exch{" CA' DDT"}="C10* DDT"; $resn_exch{" N1' DDT"}=" N1* DDT"; $resn_exch{" C1' DDU"}=" C1* DDU";
$resn_exch{" C2' DDU"}=" C2* DDU"; $resn_exch{" C3' DDU"}=" C3* DDU"; $resn_exch{" C4' DDU"}=" C4* DDU";
$resn_exch{" O3' DDU"}=" O3* DDU"; $resn_exch{" O4' DDU"}=" O4* DDU"; $resn_exch{" C5' DDU"}=" C5* DDU";
$resn_exch{" OP1 DDX"}=" O1P DDX"; $resn_exch{" OP2 DDX"}=" O2P DDX"; $resn_exch{" OP3 DDX"}=" O3P DDX";
$resn_exch{" O5' DDX"}=" O5* DDX"; $resn_exch{" C5' DDX"}=" C5* DDX"; $resn_exch{" C4' DDX"}=" C4* DDX";
$resn_exch{" O4' DDX"}=" O4* DDX"; $resn_exch{" C1' DDX"}=" C1* DDX"; $resn_exch{" C3' DDX"}=" C3* DDX";
$resn_exch{" C2' DDX"}=" C2* DDX"; $resn_exch{" O1' DDX"}=" O1* DDX"; $resn_exch{" O5' DDY"}=" O5* DDY";
$resn_exch{" C5' DDY"}=" C5* DDY"; $resn_exch{" C4' DDY"}=" C4* DDY"; $resn_exch{" O4' DDY"}=" O4* DDY";
$resn_exch{" C3' DDY"}=" C3* DDY"; $resn_exch{" C2' DDY"}=" C2* DDY"; $resn_exch{" C1' DDY"}=" C1* DDY";
$resn_exch{" C1' DEA"}=" C1* DEA"; $resn_exch{" C2' DEA"}=" C2* DEA"; $resn_exch{" O2' DEA"}=" O2* DEA";
$resn_exch{" C3' DEA"}=" C3* DEA"; $resn_exch{" O3' DEA"}=" O3* DEA"; $resn_exch{" C4' DEA"}=" C4* DEA";
$resn_exch{" NA  DEU"}=" N_A DEU"; $resn_exch{" NB  DEU"}=" N_B DEU"; $resn_exch{" NC  DEU"}=" N_C DEU";
$resn_exch{" ND  DEU"}=" N_D DEU"; $resn_exch{" OP3 DFC"}=" O3P DFC"; $resn_exch{" OP1 DFC"}=" O1P DFC";
$resn_exch{" OP2 DFC"}=" O2P DFC"; $resn_exch{" O5' DFC"}=" O5* DFC"; $resn_exch{" C5' DFC"}=" C5* DFC";
$resn_exch{" C4' DFC"}=" C4* DFC"; $resn_exch{" O4' DFC"}=" O4* DFC"; $resn_exch{" C3' DFC"}=" C3* DFC";
$resn_exch{" O3' DFC"}=" O3* DFC"; $resn_exch{" C2' DFC"}=" C2* DFC"; $resn_exch{" C1' DFC"}=" C1* DFC";
$resn_exch{" OP3 DFG"}=" O3P DFG"; $resn_exch{" OP1 DFG"}=" O1P DFG"; $resn_exch{" OP2 DFG"}=" O2P DFG";
$resn_exch{" O5' DFG"}=" O5* DFG"; $resn_exch{" C5' DFG"}=" C5* DFG"; $resn_exch{" C4' DFG"}=" C4* DFG";
$resn_exch{" O4' DFG"}=" O4* DFG"; $resn_exch{" C3' DFG"}=" C3* DFG"; $resn_exch{" O3' DFG"}=" O3* DFG";
$resn_exch{" C2' DFG"}=" C2* DFG"; $resn_exch{" C1' DFG"}=" C1* DFG"; $resn_exch{" OP1 DFT"}=" O1P DFT";
$resn_exch{" OP2 DFT"}=" O2P DFT"; $resn_exch{" OP3 DFT"}=" O3P DFT"; $resn_exch{" O5' DFT"}=" O5* DFT";
$resn_exch{" C5' DFT"}=" C5* DFT"; $resn_exch{" C4' DFT"}=" C4* DFT"; $resn_exch{" O4' DFT"}=" O4* DFT";
$resn_exch{" C3' DFT"}=" C3* DFT"; $resn_exch{" O3' DFT"}=" O3* DFT"; $resn_exch{" C2' DFT"}=" C2* DFT";
$resn_exch{" C1' DFT"}=" C1* DFT"; $resn_exch{" OP3  DG"}=" O3P  DG"; $resn_exch{" OP1  DG"}=" O1P  DG";
$resn_exch{" OP2  DG"}=" O2P  DG"; $resn_exch{" O5'  DG"}=" O5*  DG"; $resn_exch{" C5'  DG"}=" C5*  DG";
$resn_exch{" C4'  DG"}=" C4*  DG"; $resn_exch{" O4'  DG"}=" O4*  DG"; $resn_exch{" C3'  DG"}=" C3*  DG";
$resn_exch{" O3'  DG"}=" O3*  DG"; $resn_exch{" C2'  DG"}=" C2*  DG"; $resn_exch{" C1'  DG"}=" C1*  DG";
$resn_exch{" C1' DG1"}=" C1* DG1"; $resn_exch{" O4' DG1"}=" O4* DG1"; $resn_exch{" C2' DG1"}=" C2* DG1";
$resn_exch{" O2' DG1"}=" O2* DG1"; $resn_exch{" C3' DG1"}=" C3* DG1"; $resn_exch{" O3' DG1"}=" O3* DG1";
$resn_exch{" C4' DG1"}=" C4* DG1"; $resn_exch{" C5' DG1"}=" C5* DG1"; $resn_exch{" O5' DG1"}=" O5* DG1";
$resn_exch{" O5' DG3"}=" O5* DG3"; $resn_exch{" C5' DG3"}=" C5* DG3"; $resn_exch{" C4' DG3"}=" C4* DG3";
$resn_exch{" O4' DG3"}=" O4* DG3"; $resn_exch{" C3' DG3"}=" C3* DG3"; $resn_exch{" C2' DG3"}=" C2* DG3";
$resn_exch{" C1' DG3"}=" C1* DG3"; $resn_exch{" CAA DGA"}="AC10 DGA"; $resn_exch{" CBA DGA"}="AC11 DGA";
$resn_exch{" CCA DGA"}="AC12 DGA"; $resn_exch{" CDA DGA"}="AC13 DGA"; $resn_exch{" CEA DGA"}="AC14 DGA";
$resn_exch{" CFA DGA"}="AC15 DGA"; $resn_exch{" CGA DGA"}="AC16 DGA"; $resn_exch{" CHA DGA"}="AC17 DGA";
$resn_exch{" CIA DGA"}="AC18 DGA"; $resn_exch{" CAB DGA"}="BC10 DGA"; $resn_exch{" CBB DGA"}="BC11 DGA";
$resn_exch{" CCB DGA"}="BC12 DGA"; $resn_exch{" CDB DGA"}="BC13 DGA"; $resn_exch{" CEB DGA"}="BC14 DGA";
$resn_exch{" CFB DGA"}="BC15 DGA"; $resn_exch{" CGB DGA"}="BC16 DGA"; $resn_exch{" CHB DGA"}="BC17 DGA";
$resn_exch{" CIB DGA"}="BC18 DGA"; $resn_exch{" C1A DGD"}=" AC1 DGD"; $resn_exch{" C2A DGD"}=" AC2 DGD";
$resn_exch{" C3A DGD"}=" AC3 DGD"; $resn_exch{" C4A DGD"}=" AC4 DGD"; $resn_exch{" C5A DGD"}=" AC5 DGD";
$resn_exch{" C6A DGD"}=" AC6 DGD"; $resn_exch{" C7A DGD"}=" AC7 DGD"; $resn_exch{" C8A DGD"}=" AC8 DGD";
$resn_exch{" C9A DGD"}=" AC9 DGD"; $resn_exch{" CAA DGD"}="AC10 DGD"; $resn_exch{" CBA DGD"}="AC11 DGD";
$resn_exch{" CCA DGD"}="AC12 DGD"; $resn_exch{" CDA DGD"}="AC13 DGD"; $resn_exch{" CEA DGD"}="AC14 DGD";
$resn_exch{" CFA DGD"}="AC15 DGD"; $resn_exch{" CGA DGD"}="AC16 DGD"; $resn_exch{" CHA DGD"}="AC17 DGD";
$resn_exch{" CIA DGD"}="AC18 DGD"; $resn_exch{" O1A DGD"}=" AO1 DGD"; $resn_exch{" C1B DGD"}=" BC1 DGD";
$resn_exch{" C2B DGD"}=" BC2 DGD"; $resn_exch{" C3B DGD"}=" BC3 DGD"; $resn_exch{" C4B DGD"}=" BC4 DGD";
$resn_exch{" C5B DGD"}=" BC5 DGD"; $resn_exch{" C6B DGD"}=" BC6 DGD"; $resn_exch{" C7B DGD"}=" BC7 DGD";
$resn_exch{" C8B DGD"}=" BC8 DGD"; $resn_exch{" C9B DGD"}=" BC9 DGD"; $resn_exch{" CAB DGD"}="BC10 DGD";
$resn_exch{" CBB DGD"}="BC11 DGD"; $resn_exch{" CCB DGD"}="BC12 DGD"; $resn_exch{" CDB DGD"}="BC13 DGD";
$resn_exch{" CEB DGD"}="BC14 DGD"; $resn_exch{" CFB DGD"}="BC15 DGD"; $resn_exch{" CGB DGD"}="BC16 DGD";
$resn_exch{" CHB DGD"}="BC17 DGD"; $resn_exch{" CIB DGD"}="BC18 DGD"; $resn_exch{" O1B DGD"}=" BO1 DGD";
$resn_exch{" O1G DGD"}=" GO1 DGD"; $resn_exch{" C1G DGD"}=" GC1 DGD"; $resn_exch{" C2G DGD"}=" GC2 DGD";
$resn_exch{" O2G DGD"}=" GO2 DGD"; $resn_exch{" C3G DGD"}=" GC3 DGD"; $resn_exch{" O3G DGD"}=" GO3 DGD";
$resn_exch{" C1D DGD"}=" DC1 DGD"; $resn_exch{" C2D DGD"}=" DC2 DGD"; $resn_exch{" O2D DGD"}=" DO2 DGD";
$resn_exch{" C3D DGD"}=" DC3 DGD"; $resn_exch{" O3D DGD"}=" DO3 DGD"; $resn_exch{" C4D DGD"}=" DC4 DGD";
$resn_exch{" O4D DGD"}=" DO4 DGD"; $resn_exch{" C5D DGD"}=" DC5 DGD"; $resn_exch{" O5D DGD"}=" DO5 DGD";
$resn_exch{" C6D DGD"}=" DC6 DGD"; $resn_exch{" O6D DGD"}=" DO6 DGD"; $resn_exch{" C1E DGD"}=" EC1 DGD";
$resn_exch{" C2E DGD"}=" EC2 DGD"; $resn_exch{" O2E DGD"}=" EO2 DGD"; $resn_exch{" C3E DGD"}=" EC3 DGD";
$resn_exch{" O3E DGD"}=" EO3 DGD"; $resn_exch{" C4E DGD"}=" EC4 DGD"; $resn_exch{" O4E DGD"}=" EO4 DGD";
$resn_exch{" C5E DGD"}=" EC5 DGD"; $resn_exch{" O6E DGD"}=" EO6 DGD"; $resn_exch{" C6E DGD"}=" EC6 DGD";
$resn_exch{" O5E DGD"}=" EO5 DGD"; $resn_exch{" O5' DGI"}=" O5* DGI"; $resn_exch{" C5' DGI"}=" C5* DGI";
$resn_exch{" C4' DGI"}=" C4* DGI"; $resn_exch{" O4' DGI"}=" O4* DGI"; $resn_exch{" C3' DGI"}=" C3* DGI";
$resn_exch{" O3' DGI"}=" O3* DGI"; $resn_exch{" C2' DGI"}=" C2* DGI"; $resn_exch{" C1' DGI"}=" C1* DGI";
$resn_exch{" OP1 DGP"}=" O1P DGP"; $resn_exch{" OP2 DGP"}=" O2P DGP"; $resn_exch{" OP3 DGP"}=" O3P DGP";
$resn_exch{" O5' DGP"}=" O5* DGP"; $resn_exch{" C5' DGP"}=" C5* DGP"; $resn_exch{" C4' DGP"}=" C4* DGP";
$resn_exch{" O4' DGP"}=" O4* DGP"; $resn_exch{" C3' DGP"}=" C3* DGP"; $resn_exch{" O3' DGP"}=" O3* DGP";
$resn_exch{" C2' DGP"}=" C2* DGP"; $resn_exch{" C1' DGP"}=" C1* DGP"; $resn_exch{" O5' DGT"}=" O5* DGT";
$resn_exch{" C5' DGT"}=" C5* DGT"; $resn_exch{" C4' DGT"}=" C4* DGT"; $resn_exch{" O4' DGT"}=" O4* DGT";
$resn_exch{" C3' DGT"}=" C3* DGT"; $resn_exch{" O3' DGT"}=" O3* DGT"; $resn_exch{" C2' DGT"}=" C2* DGT";
$resn_exch{" C1' DGT"}=" C1* DGT"; $resn_exch{" C5' DH3"}=" C5* DH3"; $resn_exch{" C4B DH3"}=" C4* DH3";
$resn_exch{" C2' DH9"}=" C2* DH9"; $resn_exch{" C1' DH9"}=" C1* DH9"; $resn_exch{" O1' DH9"}=" O1* DH9";
$resn_exch{" O2' DH9"}=" O2* DH9"; $resn_exch{" C3' DH9"}=" C3* DH9"; $resn_exch{" O3' DH9"}=" O3* DH9";
$resn_exch{" C4' DH9"}=" C4* DH9"; $resn_exch{" C5' DH9"}=" C5* DH9"; $resn_exch{" O4' DH9"}=" O4* DH9";
$resn_exch{" O5' DH9"}=" O5* DH9"; $resn_exch{" N1' DH9"}=" N1* DH9"; $resn_exch{" O6' DH9"}=" O6* DH9";
$resn_exch{" C6' DH9"}=" C6* DH9"; $resn_exch{" C7' DH9"}=" C7* DH9"; $resn_exch{" C8' DH9"}=" C8* DH9";
$resn_exch{" C9' DH9"}=" C9* DH9"; $resn_exch{" NA  DHE"}=" N_A DHE"; $resn_exch{" NB  DHE"}=" N_B DHE";
$resn_exch{" NC  DHE"}=" N_C DHE"; $resn_exch{" ND  DHE"}=" N_D DHE"; $resn_exch{" OP1 DHU"}=" O1P DHU";
$resn_exch{" OP2 DHU"}=" O2P DHU"; $resn_exch{" OP3 DHU"}=" O3P DHU"; $resn_exch{" O5' DHU"}=" O5* DHU";
$resn_exch{" C5' DHU"}=" C5* DHU"; $resn_exch{" C4' DHU"}=" C4* DHU"; $resn_exch{" O4' DHU"}=" O4* DHU";
$resn_exch{" C3' DHU"}=" C3* DHU"; $resn_exch{" O3' DHU"}=" O3* DHU"; $resn_exch{" C2' DHU"}=" C2* DHU";
$resn_exch{" O2' DHU"}=" O2* DHU"; $resn_exch{" C1' DHU"}=" C1* DHU"; $resn_exch{" OP3  DI"}=" O3P  DI";
$resn_exch{" OP1  DI"}=" O1P  DI"; $resn_exch{" OP2  DI"}=" O2P  DI"; $resn_exch{" O5'  DI"}=" O5*  DI";
$resn_exch{" C5'  DI"}=" C5*  DI"; $resn_exch{" C4'  DI"}=" C4*  DI"; $resn_exch{" O4'  DI"}=" O4*  DI";
$resn_exch{" C3'  DI"}=" C3*  DI"; $resn_exch{" O3'  DI"}=" O3*  DI"; $resn_exch{" C2'  DI"}=" C2*  DI";
$resn_exch{" C1'  DI"}=" C1*  DI"; $resn_exch{" O5' DIH"}=" O5* DIH"; $resn_exch{" C5' DIH"}=" C5* DIH";
$resn_exch{" C4' DIH"}=" C4* DIH"; $resn_exch{" C6' DIH"}=" C6* DIH"; $resn_exch{" C3' DIH"}=" C3* DIH";
$resn_exch{" O3' DIH"}=" O3* DIH"; $resn_exch{" C2' DIH"}=" C2* DIH"; $resn_exch{" N1' DIH"}=" N1* DIH";
$resn_exch{" C1X DIT"}=" XC1 DIT"; $resn_exch{" N2X DIT"}=" XN2 DIT"; $resn_exch{" C3X DIT"}=" XC3 DIT";
$resn_exch{" C4X DIT"}=" XC4 DIT"; $resn_exch{" C5X DIT"}=" XC5 DIT"; $resn_exch{" C6X DIT"}=" XC6 DIT";
$resn_exch{" N7X DIT"}=" XN7 DIT"; $resn_exch{" C8X DIT"}=" XC8 DIT"; $resn_exch{" C9X DIT"}=" XC9 DIT";
$resn_exch{" CAX DIT"}="XC10 DIT"; $resn_exch{" OAX DIT"}="XO10 DIT"; $resn_exch{" CBX DIT"}="XC11 DIT";
$resn_exch{" CCX DIT"}="XC12 DIT"; $resn_exch{" CDX DIT"}="XC13 DIT"; $resn_exch{" CEX DIT"}="XC14 DIT";
$resn_exch{" CFX DIT"}="XC15 DIT"; $resn_exch{" CGX DIT"}="XC16 DIT"; $resn_exch{" CHX DIT"}="XC17 DIT";
$resn_exch{" CIX DIT"}="XC18 DIT"; $resn_exch{" CJX DIT"}="XC19 DIT"; $resn_exch{" CKX DIT"}="XC20 DIT";
$resn_exch{" NLX DIT"}="XN21 DIT"; $resn_exch{" CMX DIT"}="XC22 DIT"; $resn_exch{" CNX DIT"}="XC23 DIT";
$resn_exch{" COX DIT"}="XC24 DIT"; $resn_exch{" CPX DIT"}="XC25 DIT"; $resn_exch{" CQX DIT"}="XC26 DIT";
$resn_exch{" C1' DM1"}=" C1* DM1"; $resn_exch{" C2' DM1"}=" C2* DM1"; $resn_exch{" C3' DM1"}=" C3* DM1";
$resn_exch{" N3' DM1"}=" N3* DM1"; $resn_exch{" C4' DM1"}=" C4* DM1"; $resn_exch{" O4' DM1"}=" O4* DM1";
$resn_exch{" C5' DM1"}=" C5* DM1"; $resn_exch{" O5' DM1"}=" O5* DM1"; $resn_exch{" C6' DM1"}=" C6* DM1";
$resn_exch{" C1' DM2"}=" C1* DM2"; $resn_exch{" C2' DM2"}=" C2* DM2"; $resn_exch{" C3' DM2"}=" C3* DM2";
$resn_exch{" N3' DM2"}=" N3* DM2"; $resn_exch{" C4' DM2"}=" C4* DM2"; $resn_exch{" O4' DM2"}=" O4* DM2";
$resn_exch{" C5' DM2"}=" C5* DM2"; $resn_exch{" O5' DM2"}=" O5* DM2"; $resn_exch{" C6' DM2"}=" C6* DM2";
$resn_exch{" C1' DM3"}=" C1* DM3"; $resn_exch{" C2' DM3"}=" C2* DM3"; $resn_exch{" C3' DM3"}=" C3* DM3";
$resn_exch{" N3' DM3"}=" N3* DM3"; $resn_exch{" C4' DM3"}=" C4* DM3"; $resn_exch{" O4' DM3"}=" O4* DM3";
$resn_exch{" C5' DM3"}=" C5* DM3"; $resn_exch{" O5' DM3"}=" O5* DM3"; $resn_exch{" C6' DM3"}=" C6* DM3";
$resn_exch{" C1' DM4"}=" C1* DM4"; $resn_exch{" C2' DM4"}=" C2* DM4"; $resn_exch{" C3' DM4"}=" C3* DM4";
$resn_exch{" N3' DM4"}=" N3* DM4"; $resn_exch{" C4' DM4"}=" C4* DM4"; $resn_exch{" O4' DM4"}=" O4* DM4";
$resn_exch{" C5' DM4"}=" C5* DM4"; $resn_exch{" O5' DM4"}=" O5* DM4"; $resn_exch{" C6' DM4"}=" C6* DM4";
$resn_exch{" C1' DM5"}=" C1* DM5"; $resn_exch{" C2' DM5"}=" C2* DM5"; $resn_exch{" C3' DM5"}=" C3* DM5";
$resn_exch{" N3' DM5"}=" N3* DM5"; $resn_exch{" C4' DM5"}=" C4* DM5"; $resn_exch{" O4' DM5"}=" O4* DM5";
$resn_exch{" C5' DM5"}=" C5* DM5"; $resn_exch{" O5' DM5"}=" O5* DM5"; $resn_exch{" C6' DM5"}=" C6* DM5";
$resn_exch{" C1' DM6"}=" C1* DM6"; $resn_exch{" C2' DM6"}=" C2* DM6"; $resn_exch{" C3' DM6"}=" C3* DM6";
$resn_exch{" C4' DM6"}=" C4* DM6"; $resn_exch{" C5' DM6"}=" C5* DM6"; $resn_exch{" C6' DM6"}=" C6* DM6";
$resn_exch{" O4' DM6"}=" O4* DM6"; $resn_exch{" O5' DM6"}=" O5* DM6"; $resn_exch{" N3' DM6"}=" N3* DM6";
$resn_exch{" C1' DM7"}=" C1* DM7"; $resn_exch{" C2' DM7"}=" C2* DM7"; $resn_exch{" C3' DM7"}=" C3* DM7";
$resn_exch{" C4' DM7"}=" C4* DM7"; $resn_exch{" C5' DM7"}=" C5* DM7"; $resn_exch{" C6' DM7"}=" C6* DM7";
$resn_exch{" O5' DM7"}=" O5* DM7"; $resn_exch{" N3' DM7"}=" N3* DM7"; $resn_exch{" I4' DM7"}=" I4* DM7";
$resn_exch{" C1' DM8"}=" C1* DM8"; $resn_exch{" C2' DM8"}=" C2* DM8"; $resn_exch{" C3' DM8"}=" C3* DM8";
$resn_exch{" C4' DM8"}=" C4* DM8"; $resn_exch{" C5' DM8"}=" C5* DM8"; $resn_exch{" C6' DM8"}=" C6* DM8";
$resn_exch{" O5' DM8"}=" O5* DM8"; $resn_exch{" O4' DM8"}=" O4* DM8"; $resn_exch{" N3' DM8"}=" N3* DM8";
$resn_exch{" C1' DM9"}=" C1* DM9"; $resn_exch{" C2' DM9"}=" C2* DM9"; $resn_exch{" C3' DM9"}=" C3* DM9";
$resn_exch{" C4' DM9"}=" C4* DM9"; $resn_exch{" O4' DM9"}=" O4* DM9"; $resn_exch{" C5' DM9"}=" C5* DM9";
$resn_exch{" O5' DM9"}=" O5* DM9"; $resn_exch{" C6' DM9"}=" C6* DM9"; $resn_exch{" N3' DM9"}=" N3* DM9";
$resn_exch{" OXT DMK"}=" O1  DMK"; $resn_exch{" C1' DMM"}=" C1* DMM"; $resn_exch{" C2' DMM"}=" C2* DMM";
$resn_exch{" C3' DMM"}=" C3* DMM"; $resn_exch{" C4' DMM"}=" C4* DMM"; $resn_exch{" O4' DMM"}=" O4* DMM";
$resn_exch{" C5' DMM"}=" C5* DMM"; $resn_exch{" O5' DMM"}=" O5* DMM"; $resn_exch{" C6' DMM"}=" C6* DMM";
$resn_exch{" N3' DMM"}=" N3* DMM"; $resn_exch{" O11 DND"}="O1PN DND"; $resn_exch{" O12 DND"}="O2PN DND";
$resn_exch{" O5D DND"}="O5'N DND"; $resn_exch{" C5D DND"}="C5'N DND"; $resn_exch{" C4D DND"}="C4'N DND";
$resn_exch{" O4D DND"}="O4'N DND"; $resn_exch{" C3D DND"}="C3'N DND"; $resn_exch{" O3D DND"}="O3'N DND";
$resn_exch{" C2D DND"}="C2'N DND"; $resn_exch{" O2D DND"}="O2'N DND"; $resn_exch{" C1D DND"}="C1'N DND";
$resn_exch{" O13 DND"}="O1PA DND"; $resn_exch{" O14 DND"}="O2PA DND"; $resn_exch{" O5B DND"}="O5'A DND";
$resn_exch{" C5B DND"}="C5'A DND"; $resn_exch{" C4B DND"}="C4'A DND"; $resn_exch{" O4B DND"}="O4'A DND";
$resn_exch{" C3B DND"}="C3'A DND"; $resn_exch{" O3B DND"}="O3'A DND"; $resn_exch{" C2B DND"}="C2'A DND";
$resn_exch{" O2B DND"}="O2'A DND"; $resn_exch{" C1B DND"}="C1'A DND"; $resn_exch{" OXT DNE"}=" O1  DNE";
$resn_exch{" OXT DNM"}=" O1  DNM"; $resn_exch{" OP1 DNR"}=" O1P DNR"; $resn_exch{" OP2 DNR"}=" O2P DNR";
$resn_exch{" O5' DNR"}=" O5* DNR"; $resn_exch{" C5' DNR"}=" C5* DNR"; $resn_exch{" C4' DNR"}=" C4* DNR";
$resn_exch{" O4' DNR"}=" O4* DNR"; $resn_exch{" C1' DNR"}=" C1* DNR"; $resn_exch{" C2' DNR"}=" C2* DNR";
$resn_exch{" C3' DNR"}=" C3* DNR"; $resn_exch{" O3' DNR"}=" O3* DNR"; $resn_exch{" OP3 DNR"}=" O3P DNR";
$resn_exch{" OXT DNS"}=" O   DNS"; $resn_exch{" O   DNS"}=" OXT DNS"; $resn_exch{" C1' DOC"}=" C1* DOC";
$resn_exch{" C2' DOC"}=" C2* DOC"; $resn_exch{" C3' DOC"}=" C3* DOC"; $resn_exch{" C4' DOC"}=" C4* DOC";
$resn_exch{" O4' DOC"}=" O4* DOC"; $resn_exch{" C5' DOC"}=" C5* DOC"; $resn_exch{" O5' DOC"}=" O5* DOC";
$resn_exch{" OP1 DOC"}=" O1P DOC"; $resn_exch{" OP2 DOC"}=" O2P DOC"; $resn_exch{" OP3 DOC"}=" O3P DOC";
$resn_exch{" O5' DOI"}=" O5* DOI"; $resn_exch{" C5' DOI"}=" C5* DOI"; $resn_exch{" C4' DOI"}=" C4* DOI";
$resn_exch{" O4' DOI"}=" O4* DOI"; $resn_exch{" C1' DOI"}=" C1* DOI"; $resn_exch{" C2' DOI"}=" C2* DOI";
$resn_exch{" C3' DOI"}=" C3* DOI"; $resn_exch{" O3' DOI"}=" O3* DOI"; $resn_exch{" O5' DOT"}=" O5* DOT";
$resn_exch{" C5B DOT"}=" C5* DOT"; $resn_exch{" C4B DOT"}=" C4* DOT"; $resn_exch{" O4' DOT"}=" O4* DOT";
$resn_exch{" C3B DOT"}=" C3* DOT"; $resn_exch{" O3' DOT"}=" O3* DOT"; $resn_exch{" C2B DOT"}=" C2* DOT";
$resn_exch{" C1B DOT"}=" C1* DOT"; $resn_exch{" C1' DPB"}=" C1* DPB"; $resn_exch{" O4' DPB"}=" O4* DPB";
$resn_exch{" C4' DPB"}=" C4* DPB"; $resn_exch{" C3' DPB"}=" C3* DPB"; $resn_exch{" C2' DPB"}=" C2* DPB";
$resn_exch{" C5' DPB"}=" C5* DPB"; $resn_exch{" O5' DPB"}=" O5* DPB"; $resn_exch{" C6' DPB"}=" C6* DPB";
$resn_exch{" O3' DPB"}=" O3* DPB"; $resn_exch{" OP2 DPB"}=" O2P DPB"; $resn_exch{" OP1 DPB"}=" O1P DPB";
$resn_exch{" OP3 DPB"}=" O3P DPB"; $resn_exch{" C1D DPI"}=" C1\" DPI"; $resn_exch{" C2D DPI"}=" C2\" DPI";
$resn_exch{" C3D DPI"}=" C3\" DPI"; $resn_exch{" C4D DPI"}=" C4\" DPI"; $resn_exch{" C5D DPI"}=" C5\" DPI";
$resn_exch{" C6' DPI"}=" C6\" DPI"; $resn_exch{" O7' DPI"}=" O7\" DPI"; $resn_exch{" O8' DPI"}=" O8\" DPI";
$resn_exch{" O9D DPI"}=" O9\" DPI"; $resn_exch{" O5' DPY"}=" O5* DPY"; $resn_exch{" C2' DPY"}=" C2* DPY";
$resn_exch{" C5' DPY"}=" C5* DPY"; $resn_exch{" C4' DPY"}=" C4* DPY"; $resn_exch{" O4' DPY"}=" O4* DPY";
$resn_exch{" C1' DPY"}=" C1* DPY"; $resn_exch{" C3' DPY"}=" C3* DPY"; $resn_exch{" O3' DPY"}=" O3* DPY";
$resn_exch{" O1' DR2"}=" O1* DR2"; $resn_exch{" C1' DR2"}=" C1* DR2"; $resn_exch{" O5' DR2"}=" O5* DR2";
$resn_exch{" C5' DR2"}=" C5* DR2"; $resn_exch{" C6' DR2"}=" C6* DR2"; $resn_exch{" O6' DR2"}=" O6* DR2";
$resn_exch{" C4' DR2"}=" C4* DR2"; $resn_exch{" O4' DR2"}=" O4* DR2"; $resn_exch{" C3' DR2"}=" C3* DR2";
$resn_exch{" O3' DR2"}=" O3* DR2"; $resn_exch{" C2' DR2"}=" C2* DR2"; $resn_exch{" O2' DR2"}=" O2* DR2";
$resn_exch{" O2' DR3"}=" O2* DR3"; $resn_exch{" C2' DR3"}=" C2* DR3"; $resn_exch{" C3' DR3"}=" C3* DR3";
$resn_exch{" O3' DR3"}=" O3* DR3"; $resn_exch{" C4' DR3"}=" C4* DR3"; $resn_exch{" O4' DR3"}=" O4* DR3";
$resn_exch{" C5' DR3"}=" C5* DR3"; $resn_exch{" C6' DR3"}=" C6* DR3"; $resn_exch{" O6' DR3"}=" O6* DR3";
$resn_exch{" O5' DR3"}=" O5* DR3"; $resn_exch{" C1' DR3"}=" C1* DR3"; $resn_exch{" O1' DR3"}=" O1* DR3";
$resn_exch{" C1' DRA"}=" C1* DRA"; $resn_exch{" C2' DRA"}=" C2* DRA"; $resn_exch{" C3' DRA"}=" C3* DRA";
$resn_exch{" N3' DRA"}=" N3* DRA"; $resn_exch{" C7' DRA"}=" C7* DRA"; $resn_exch{" C8' DRA"}=" C8* DRA";
$resn_exch{" O5' DRA"}=" O5* DRA"; $resn_exch{" C5' DRA"}=" C5* DRA"; $resn_exch{" C6' DRA"}=" C6* DRA";
$resn_exch{" C4' DRA"}=" C4* DRA"; $resn_exch{" O4' DRA"}=" O4* DRA"; $resn_exch{" OP2 DRM"}=" O2P DRM";
$resn_exch{" OP1 DRM"}=" O1P DRM"; $resn_exch{" OP3 DRM"}=" O3P DRM"; $resn_exch{" C6' DRM"}=" C6* DRM";
$resn_exch{" O5' DRM"}=" O5* DRM"; $resn_exch{" C5' DRM"}=" C5* DRM"; $resn_exch{" C4' DRM"}=" C4* DRM";
$resn_exch{" C3' DRM"}=" C3* DRM"; $resn_exch{" C2' DRM"}=" C2* DRM"; $resn_exch{" C1' DRM"}=" C1* DRM";
$resn_exch{" O5' DRP"}=" O5* DRP"; $resn_exch{" C2' DRP"}=" C2* DRP"; $resn_exch{" C5' DRP"}=" C5* DRP";
$resn_exch{" C4' DRP"}=" C4* DRP"; $resn_exch{" O4' DRP"}=" O4* DRP"; $resn_exch{" C1' DRP"}=" C1* DRP";
$resn_exch{" C3' DRP"}=" C3* DRP"; $resn_exch{" O3' DRP"}=" O3* DRP"; $resn_exch{" OP3 DRT"}=" O3P DRT";
$resn_exch{" OP1 DRT"}=" O1P DRT"; $resn_exch{" OP2 DRT"}=" O2P DRT"; $resn_exch{" O5' DRT"}=" O5* DRT";
$resn_exch{" C5' DRT"}=" C5* DRT"; $resn_exch{" C4' DRT"}=" C4* DRT"; $resn_exch{" O4' DRT"}=" O4* DRT";
$resn_exch{" C3' DRT"}=" C3* DRT"; $resn_exch{" O3' DRT"}=" O3* DRT"; $resn_exch{" C2' DRT"}=" C2* DRT";
$resn_exch{" C1' DRT"}=" C1* DRT"; $resn_exch{" OP1 DRZ"}=" O1P DRZ"; $resn_exch{" OP2 DRZ"}=" O2P DRZ";
$resn_exch{" OP3 DRZ"}=" O3P DRZ"; $resn_exch{" O5' DRZ"}=" O5* DRZ"; $resn_exch{" C5' DRZ"}=" C5* DRZ";
$resn_exch{" C4' DRZ"}=" C4* DRZ"; $resn_exch{" O4' DRZ"}=" O4* DRZ"; $resn_exch{" C3' DRZ"}=" C3* DRZ";
$resn_exch{" C2' DRZ"}=" C2* DRZ"; $resn_exch{" C1' DRZ"}=" C1* DRZ"; $resn_exch{" O3' DRZ"}=" O3* DRZ";
$resn_exch{" O6' DSU"}=" O6* DSU"; $resn_exch{" C6' DSU"}=" C6* DSU"; $resn_exch{" C5' DSU"}=" C5* DSU";
$resn_exch{" O2' DSU"}=" O2* DSU"; $resn_exch{" C4' DSU"}=" C4* DSU"; $resn_exch{" O4' DSU"}=" O4* DSU";
$resn_exch{" C3' DSU"}=" C3* DSU"; $resn_exch{" O3' DSU"}=" O3* DSU"; $resn_exch{" C2' DSU"}=" C2* DSU";
$resn_exch{" C1' DSU"}=" C1* DSU"; $resn_exch{" O1' DSU"}=" O1* DSU"; $resn_exch{" OP3  DT"}=" O3P  DT";
$resn_exch{" OP1  DT"}=" O1P  DT"; $resn_exch{" OP2  DT"}=" O2P  DT"; $resn_exch{" O5'  DT"}=" O5*  DT";
$resn_exch{" C5'  DT"}=" C5*  DT"; $resn_exch{" C4'  DT"}=" C4*  DT"; $resn_exch{" O4'  DT"}=" O4*  DT";
$resn_exch{" C3'  DT"}=" C3*  DT"; $resn_exch{" O3'  DT"}=" O3*  DT"; $resn_exch{" C2'  DT"}=" C2*  DT";
$resn_exch{" C1'  DT"}=" C1*  DT"; $resn_exch{" C7   DT"}=" C5M  DT"; $resn_exch{" C1' DTA"}=" C1* DTA";
$resn_exch{" O4' DTA"}=" O4* DTA"; $resn_exch{" C2' DTA"}=" C2* DTA"; $resn_exch{" O2' DTA"}=" O2* DTA";
$resn_exch{" C3' DTA"}=" C3* DTA"; $resn_exch{" O3' DTA"}=" O3* DTA"; $resn_exch{" C4' DTA"}=" C4* DTA";
$resn_exch{" C5' DTA"}=" C5* DTA"; $resn_exch{" S5' DTA"}=" S5* DTA"; $resn_exch{" C2B DTM"}=" C2* DTM";
$resn_exch{" C4B DTM"}=" C4* DTM"; $resn_exch{" C5B DTM"}=" C5* DTM"; $resn_exch{" C6B DTM"}=" C6* DTM";
$resn_exch{" C7' DTM"}=" C7* DTM"; $resn_exch{" C9' DTM"}=" C9* DTM"; $resn_exch{" CN' DTM"}=" CN* DTM";
$resn_exch{" N1' DTM"}=" N1* DTM"; $resn_exch{" N2' DTM"}=" N2* DTM"; $resn_exch{" N3' DTM"}=" N3* DTM";
$resn_exch{" N4' DTM"}=" N4* DTM"; $resn_exch{" N8' DTM"}=" N8* DTM"; $resn_exch{" N9' DTM"}=" N9* DTM";
$resn_exch{" O5' DTP"}=" O5* DTP"; $resn_exch{" C5' DTP"}=" C5* DTP"; $resn_exch{" C4' DTP"}=" C4* DTP";
$resn_exch{" O4' DTP"}=" O4* DTP"; $resn_exch{" C3' DTP"}=" C3* DTP"; $resn_exch{" O3' DTP"}=" O3* DTP";
$resn_exch{" C2' DTP"}=" C2* DTP"; $resn_exch{" C1' DTP"}=" C1* DTP"; $resn_exch{" OP3  DU"}=" O3P  DU";
$resn_exch{" OP1  DU"}=" O1P  DU"; $resn_exch{" OP2  DU"}=" O2P  DU"; $resn_exch{" O5'  DU"}=" O5*  DU";
$resn_exch{" C5'  DU"}=" C5*  DU"; $resn_exch{" C4'  DU"}=" C4*  DU"; $resn_exch{" O4'  DU"}=" O4*  DU";
$resn_exch{" C3'  DU"}=" C3*  DU"; $resn_exch{" O3'  DU"}=" O3*  DU"; $resn_exch{" C2'  DU"}=" C2*  DU";
$resn_exch{" C1'  DU"}=" C1*  DU"; $resn_exch{" C1' DUD"}=" C1* DUD"; $resn_exch{" C2' DUD"}=" C2* DUD";
$resn_exch{" C3' DUD"}=" C3* DUD"; $resn_exch{" C4' DUD"}=" C4* DUD"; $resn_exch{" O4' DUD"}=" O4* DUD";
$resn_exch{" O3' DUD"}=" O3* DUD"; $resn_exch{" C5' DUD"}=" C5* DUD"; $resn_exch{" O5' DUD"}=" O5* DUD";
$resn_exch{" C1' DUN"}=" C1* DUN"; $resn_exch{" C2' DUN"}=" C2* DUN"; $resn_exch{" C3' DUN"}=" C3* DUN";
$resn_exch{" C4' DUN"}=" C4* DUN"; $resn_exch{" O4' DUN"}=" O4* DUN"; $resn_exch{" O3' DUN"}=" O3* DUN";
$resn_exch{" C5' DUN"}=" C5* DUN"; $resn_exch{" O5' DUN"}=" O5* DUN"; $resn_exch{" C1' DUP"}=" C1* DUP";
$resn_exch{" C2' DUP"}=" C2* DUP"; $resn_exch{" C3' DUP"}=" C3* DUP"; $resn_exch{" O3' DUP"}=" O3* DUP";
$resn_exch{" O4' DUP"}=" O4* DUP"; $resn_exch{" C4' DUP"}=" C4* DUP"; $resn_exch{" C5' DUP"}=" C5* DUP";
$resn_exch{" O5' DUP"}=" O5* DUP"; $resn_exch{" C1' DUR"}=" C1* DUR"; $resn_exch{" C2' DUR"}=" C2* DUR";
$resn_exch{" C3' DUR"}=" C3* DUR"; $resn_exch{" C4' DUR"}=" C4* DUR"; $resn_exch{" O3' DUR"}=" O3* DUR";
$resn_exch{" O4' DUR"}=" O4* DUR"; $resn_exch{" C5' DUR"}=" C5* DUR"; $resn_exch{" O5' DUR"}=" O5* DUR";
$resn_exch{" C1' DUT"}=" C1* DUT"; $resn_exch{" C2' DUT"}=" C2* DUT"; $resn_exch{" C3' DUT"}=" C3* DUT";
$resn_exch{" C4' DUT"}=" C4* DUT"; $resn_exch{" O4' DUT"}=" O4* DUT"; $resn_exch{" O3' DUT"}=" O3* DUT";
$resn_exch{" C5' DUT"}=" C5* DUT"; $resn_exch{" O5' DUT"}=" O5* DUT"; $resn_exch{" OP1 DXD"}=" O1P DXD";
$resn_exch{" OP2 DXD"}=" O2P DXD"; $resn_exch{" O5' DXD"}=" O5* DXD"; $resn_exch{" C5' DXD"}=" C5* DXD";
$resn_exch{" C4' DXD"}=" C4* DXD"; $resn_exch{" C1' DXD"}=" C1* DXD"; $resn_exch{" C6' DXD"}=" C6* DXD";
$resn_exch{" O6' DXD"}=" O6* DXD"; $resn_exch{" C2' DXD"}=" C2* DXD"; $resn_exch{" C3' DXD"}=" C3* DXD";
$resn_exch{" O3' DXD"}=" O3* DXD"; $resn_exch{" OP3 DXD"}=" O3P DXD"; $resn_exch{" OP1 DXN"}=" O1P DXN";
$resn_exch{" OP2 DXN"}=" O2P DXN"; $resn_exch{" O5' DXN"}=" O5* DXN"; $resn_exch{" C5' DXN"}=" C5* DXN";
$resn_exch{" C4' DXN"}=" C4* DXN"; $resn_exch{" C1' DXN"}=" C1* DXN"; $resn_exch{" C6' DXN"}=" C6* DXN";
$resn_exch{" O6' DXN"}=" O6* DXN"; $resn_exch{" C2' DXN"}=" C2* DXN"; $resn_exch{" C3' DXN"}=" C3* DXN";
$resn_exch{" O3' DXN"}=" O3* DXN"; $resn_exch{" OP3 DXN"}=" O3P DXN"; $resn_exch{" OP1   E"}=" O1P   E";
$resn_exch{" OP2   E"}=" O2P   E"; $resn_exch{" OP3   E"}=" O3P   E"; $resn_exch{" O5'   E"}=" O5*   E";
$resn_exch{" C5'   E"}=" C5*   E"; $resn_exch{" C4'   E"}=" C4*   E"; $resn_exch{" O4'   E"}=" O4*   E";
$resn_exch{" C3'   E"}=" C3*   E"; $resn_exch{" O3'   E"}=" O3*   E"; $resn_exch{" C2'   E"}=" C2*   E";
$resn_exch{" C1'   E"}=" C1*   E"; $resn_exch{" OP1 E1X"}=" O1P E1X"; $resn_exch{" OP2 E1X"}=" O2P E1X";
$resn_exch{" O5' E1X"}=" O5* E1X"; $resn_exch{" C2' E1X"}=" C2* E1X"; $resn_exch{" C5' E1X"}=" C5* E1X";
$resn_exch{" C4' E1X"}=" C4* E1X"; $resn_exch{" O4' E1X"}=" O4* E1X"; $resn_exch{" C1' E1X"}=" C1* E1X";
$resn_exch{" C3' E1X"}=" C3* E1X"; $resn_exch{" O3' E1X"}=" O3* E1X"; $resn_exch{" OP3 E1X"}=" O3P E1X";
$resn_exch{" OP1 EDA"}=" O1P EDA"; $resn_exch{" OP2 EDA"}=" O2P EDA"; $resn_exch{" OP3 EDA"}=" O3P EDA";
$resn_exch{" O5' EDA"}=" O5* EDA"; $resn_exch{" C2' EDA"}=" C2* EDA"; $resn_exch{" C5' EDA"}=" C5* EDA";
$resn_exch{" C4' EDA"}=" C4* EDA"; $resn_exch{" O4' EDA"}=" O4* EDA"; $resn_exch{" C1' EDA"}=" C1* EDA";
$resn_exch{" C3' EDA"}=" C3* EDA"; $resn_exch{" O3' EDA"}=" O3* EDA"; $resn_exch{" OP1 EDC"}=" O1P EDC";
$resn_exch{" OP2 EDC"}=" O2P EDC"; $resn_exch{" OP3 EDC"}=" O3P EDC"; $resn_exch{" O5' EDC"}=" O5* EDC";
$resn_exch{" C5' EDC"}=" C5* EDC"; $resn_exch{" C4' EDC"}=" C4* EDC"; $resn_exch{" O4' EDC"}=" O4* EDC";
$resn_exch{" C1' EDC"}=" C1* EDC"; $resn_exch{" C2' EDC"}=" C2* EDC"; $resn_exch{" C3' EDC"}=" C3* EDC";
$resn_exch{" O3' EDC"}=" O3* EDC"; $resn_exch{" N1U EEB"}=" UN1 EEB"; $resn_exch{" C2U EEB"}=" UC2 EEB";
$resn_exch{" N3U EEB"}=" UN3 EEB"; $resn_exch{" C4U EEB"}=" UC4 EEB"; $resn_exch{" C5U EEB"}=" UC5 EEB";
$resn_exch{" C6U EEB"}=" UC6 EEB"; $resn_exch{" O2U EEB"}=" UO2 EEB"; $resn_exch{" O4U EEB"}=" UO4 EEB";
$resn_exch{" C1D EEB"}="UC1* EEB"; $resn_exch{" C2D EEB"}="UC2* EEB"; $resn_exch{" O2D EEB"}="UO2* EEB";
$resn_exch{" C3D EEB"}="UC3* EEB"; $resn_exch{" C4D EEB"}="UC4* EEB"; $resn_exch{" O4D EEB"}="UO4* EEB";
$resn_exch{" O3D EEB"}="UO3* EEB"; $resn_exch{" C5D EEB"}="UC5* EEB"; $resn_exch{" O5D EEB"}="UO5* EEB";
$resn_exch{" C1E EEB"}=" EC1 EEB"; $resn_exch{" C2E EEB"}=" EC2 EEB"; $resn_exch{" C3E EEB"}=" EC3 EEB";
$resn_exch{" C4E EEB"}=" EC4 EEB"; $resn_exch{" O1E EEB"}=" EO1 EEB"; $resn_exch{" O2E EEB"}=" EO2 EEB";
$resn_exch{" OP3 EIT"}=" O3P EIT"; $resn_exch{" OP2 EIT"}=" O2P EIT"; $resn_exch{" O5' EIT"}=" O5* EIT";
$resn_exch{" C5' EIT"}=" C5* EIT"; $resn_exch{" C4' EIT"}=" C4* EIT"; $resn_exch{" O4' EIT"}=" O4* EIT";
$resn_exch{" C1' EIT"}=" C1* EIT"; $resn_exch{" C2' EIT"}=" C2* EIT"; $resn_exch{" O2' EIT"}=" O2* EIT";
$resn_exch{" C3' EIT"}=" C3* EIT"; $resn_exch{" O3' EIT"}=" O3* EIT"; $resn_exch{" OP1 EIT"}=" O1P EIT";
$resn_exch{" C5' EMA"}=" C5* EMA"; $resn_exch{" O5' EMA"}=" O5* EMA"; $resn_exch{" C4' EMA"}=" C4* EMA";
$resn_exch{" C1' EMA"}=" C1* EMA"; $resn_exch{" C1' ENA"}=" C1* ENA"; $resn_exch{" O2' ENA"}=" O2* ENA";
$resn_exch{" C2' ENA"}=" C2* ENA"; $resn_exch{" C4' ENA"}=" C4* ENA"; $resn_exch{" C3' ENA"}=" C3* ENA";
$resn_exch{" O4' ENA"}=" O4* ENA"; $resn_exch{" C5' ENA"}=" C5* ENA"; $resn_exch{" O3' ENA"}=" O3* ENA";
$resn_exch{" O5' ENA"}=" O5* ENA"; $resn_exch{" C6' ENA"}=" C6* ENA"; $resn_exch{" C7' ENA"}=" C7* ENA";
$resn_exch{" O6' ENA"}=" O6* ENA"; $resn_exch{" C8' ENA"}=" C8* ENA"; $resn_exch{" C9' ENA"}=" C9* ENA";
$resn_exch{" C0' ENA"}=" C0* ENA"; $resn_exch{" O7' ENA"}=" O7* ENA"; $resn_exch{" O8' ENA"}=" O8* ENA";
$resn_exch{" O5' ENP"}=" O5* ENP"; $resn_exch{" C5' ENP"}=" C5* ENP"; $resn_exch{" C4' ENP"}=" C4* ENP";
$resn_exch{" O4' ENP"}=" O4* ENP"; $resn_exch{" C3' ENP"}=" C3* ENP"; $resn_exch{" O3' ENP"}=" O3* ENP";
$resn_exch{" C2' ENP"}=" C2* ENP"; $resn_exch{" O2' ENP"}=" O2* ENP"; $resn_exch{" C1' ENP"}=" C1* ENP";
$resn_exch{" O9' ENP"}=" O9* ENP"; $resn_exch{" C0' ENP"}=" C0* ENP"; $resn_exch{" C9' ENP"}=" C9* ENP";
$resn_exch{" O8' ENP"}=" O8* ENP"; $resn_exch{" C8' ENP"}=" C8* ENP"; $resn_exch{" O7' ENP"}=" O7* ENP";
$resn_exch{" C7' ENP"}=" C7* ENP"; $resn_exch{" O6' ENP"}=" O6* ENP"; $resn_exch{" C6' ENP"}=" C6* ENP";
$resn_exch{" O2' EOM"}=" O2* EOM"; $resn_exch{" CA' EOM"}=" CA* EOM"; $resn_exch{" OB' EOM"}=" OB* EOM";
$resn_exch{" CC' EOM"}=" CC* EOM"; $resn_exch{" CD' EOM"}=" CD* EOM"; $resn_exch{" N1U EPU"}=" UN1 EPU";
$resn_exch{" C2U EPU"}=" UC2 EPU"; $resn_exch{" N3U EPU"}=" UN3 EPU"; $resn_exch{" C4U EPU"}=" UC4 EPU";
$resn_exch{" C5U EPU"}=" UC5 EPU"; $resn_exch{" C6U EPU"}=" UC6 EPU"; $resn_exch{" O2U EPU"}=" UO2 EPU";
$resn_exch{" O4U EPU"}=" UO4 EPU"; $resn_exch{" C1D EPU"}="UC1* EPU"; $resn_exch{" C2D EPU"}="UC2* EPU";
$resn_exch{" O2D EPU"}="UO2* EPU"; $resn_exch{" C3D EPU"}="UC3* EPU"; $resn_exch{" C4D EPU"}="UC4* EPU";
$resn_exch{" O4D EPU"}="UO4* EPU"; $resn_exch{" O3D EPU"}="UO3* EPU"; $resn_exch{" C5D EPU"}="UC5* EPU";
$resn_exch{" O5D EPU"}="UO5* EPU"; $resn_exch{" C1E EPU"}=" EC1 EPU"; $resn_exch{" O1E EPU"}=" EO1 EPU";
$resn_exch{" O2E EPU"}=" EO2 EPU"; $resn_exch{" C2E EPU"}=" EC2 EPU"; $resn_exch{" C3E EPU"}=" EC3 EPU";
$resn_exch{" C1' ERT"}=" C1* ERT"; $resn_exch{" C2' ERT"}=" C2* ERT"; $resn_exch{" C3' ERT"}=" C3* ERT";
$resn_exch{" N3' ERT"}=" N3* ERT"; $resn_exch{" C7' ERT"}=" C7* ERT"; $resn_exch{" C8' ERT"}=" C8* ERT";
$resn_exch{" C4' ERT"}=" C4* ERT"; $resn_exch{" O4' ERT"}=" O4* ERT"; $resn_exch{" C5' ERT"}=" C5* ERT";
$resn_exch{" O5' ERT"}=" O5* ERT"; $resn_exch{" C6' ERT"}=" C6* ERT"; $resn_exch{" O5' F2N"}=" O5* F2N";
$resn_exch{" C5' F2N"}=" C5* F2N"; $resn_exch{" C4' F2N"}=" C4* F2N"; $resn_exch{" O4' F2N"}=" O4* F2N";
$resn_exch{" C3' F2N"}=" C3* F2N"; $resn_exch{" O3' F2N"}=" O3* F2N"; $resn_exch{" C2' F2N"}=" C2* F2N";
$resn_exch{" O2' F2N"}=" O2* F2N"; $resn_exch{" C1' F2N"}=" C1* F2N"; $resn_exch{" C1' F42"}=" C1* F42";
$resn_exch{" C2' F42"}=" C2* F42"; $resn_exch{" O2' F42"}=" O2* F42"; $resn_exch{" C3' F42"}=" C3* F42";
$resn_exch{" O3' F42"}=" O3* F42"; $resn_exch{" C4' F42"}=" C4* F42"; $resn_exch{" O4' F42"}=" O4* F42";
$resn_exch{" C5' F42"}=" C5* F42"; $resn_exch{" O5' F42"}=" O5* F42"; $resn_exch{" NA  F43"}=" N_A F43";
$resn_exch{" NB  F43"}=" N_B F43"; $resn_exch{" NC  F43"}=" N_C F43"; $resn_exch{" ND  F43"}=" N_D F43";
$resn_exch{" OP1 FA2"}=" O1P FA2"; $resn_exch{" OP2 FA2"}=" O2P FA2"; $resn_exch{" O3' FA2"}=" O3* FA2";
$resn_exch{" C2' FA2"}=" C2* FA2"; $resn_exch{" C4' FA2"}=" C4* FA2"; $resn_exch{" O4' FA2"}=" O4* FA2";
$resn_exch{" C1' FA2"}=" C1* FA2"; $resn_exch{" C3' FA2"}=" C3* FA2"; $resn_exch{" O2' FA2"}=" O2* FA2";
$resn_exch{" OP3 FA2"}=" O3P FA2"; $resn_exch{" OP1 FA5"}=" O1P FA5"; $resn_exch{" OP2 FA5"}=" O2P FA5";
$resn_exch{" OP3 FA5"}=" O3P FA5"; $resn_exch{" O5' FA5"}=" O5* FA5"; $resn_exch{" C5' FA5"}=" C5* FA5";
$resn_exch{" C4' FA5"}=" C4* FA5"; $resn_exch{" O4' FA5"}=" O4* FA5"; $resn_exch{" C3' FA5"}=" C3* FA5";
$resn_exch{" O3' FA5"}=" O3* FA5"; $resn_exch{" C2' FA5"}=" C2* FA5"; $resn_exch{" O2' FA5"}=" O2* FA5";
$resn_exch{" C1' FA5"}=" C1* FA5"; $resn_exch{" C3B FA9"}=" BC3 FA9"; $resn_exch{" CB1 FA9"}="BC1* FA9";
$resn_exch{" CB2 FA9"}="BC2* FA9"; $resn_exch{" CB3 FA9"}="BC3* FA9"; $resn_exch{" CB4 FA9"}="BC4* FA9";
$resn_exch{" CB5 FA9"}="BC5* FA9"; $resn_exch{" CB6 FA9"}="BC6* FA9"; $resn_exch{" C2B FA9"}=" BC2 FA9";
$resn_exch{" C1B FA9"}=" BC1 FA9"; $resn_exch{" O1B FA9"}=" BO1 FA9"; $resn_exch{" C1' FA9"}=" C1* FA9";
$resn_exch{" C2' FA9"}=" C2* FA9"; $resn_exch{" O2' FA9"}=" O2* FA9"; $resn_exch{" C3' FA9"}=" C3* FA9";
$resn_exch{" O3' FA9"}=" O3* FA9"; $resn_exch{" C4' FA9"}=" C4* FA9"; $resn_exch{" O4' FA9"}=" O4* FA9";
$resn_exch{" C5' FA9"}=" C5* FA9"; $resn_exch{" O5' FA9"}=" O5* FA9"; $resn_exch{" PA  FA9"}=" AP  FA9";
$resn_exch{" O1A FA9"}=" AO1 FA9"; $resn_exch{" O2A FA9"}=" AO2 FA9"; $resn_exch{" OA5 FA9"}="AO5* FA9";
$resn_exch{" CA5 FA9"}="AC5* FA9"; $resn_exch{" CA4 FA9"}="AC4* FA9"; $resn_exch{" OA4 FA9"}="AO4* FA9";
$resn_exch{" CA3 FA9"}="AC3* FA9"; $resn_exch{" OA3 FA9"}="AO3* FA9"; $resn_exch{" CA2 FA9"}="AC2* FA9";
$resn_exch{" OA2 FA9"}="AO2* FA9"; $resn_exch{" CA1 FA9"}="AC1* FA9"; $resn_exch{" N9A FA9"}=" AN9 FA9";
$resn_exch{" C8A FA9"}=" AC8 FA9"; $resn_exch{" N7A FA9"}=" AN7 FA9"; $resn_exch{" C5C FA9"}=" AC5 FA9";
$resn_exch{" C4C FA9"}=" AC4 FA9"; $resn_exch{" N3A FA9"}=" AN3 FA9"; $resn_exch{" C2A FA9"}=" AC2 FA9";
$resn_exch{" N1A FA9"}=" AN1 FA9"; $resn_exch{" C6A FA9"}=" AC6 FA9"; $resn_exch{" N6A FA9"}=" AN6 FA9";
$resn_exch{" PA  FAA"}=" AP  FAA"; $resn_exch{" O1A FAA"}=" AO1 FAA"; $resn_exch{" O2A FAA"}=" AO2 FAA";
$resn_exch{" O5B FAA"}="AO5* FAA"; $resn_exch{" C5B FAA"}="AC5* FAA"; $resn_exch{" C4B FAA"}="AC4* FAA";
$resn_exch{" O4B FAA"}="AO4* FAA"; $resn_exch{" C3B FAA"}="AC3* FAA"; $resn_exch{" O3B FAA"}="AO3* FAA";
$resn_exch{" C2B FAA"}="AC2* FAA"; $resn_exch{" O2B FAA"}="AO2* FAA"; $resn_exch{" C1B FAA"}="AC1* FAA";
$resn_exch{" N9A FAA"}=" AN9 FAA"; $resn_exch{" C8A FAA"}=" AC8 FAA"; $resn_exch{" N7A FAA"}=" AN7 FAA";
$resn_exch{" C5A FAA"}=" AC5 FAA"; $resn_exch{" C6A FAA"}=" AC6 FAA"; $resn_exch{" N6A FAA"}=" AN6 FAA";
$resn_exch{" N1A FAA"}=" AN1 FAA"; $resn_exch{" C2A FAA"}=" AC2 FAA"; $resn_exch{" N3A FAA"}=" AN3 FAA";
$resn_exch{" C4A FAA"}=" AC4 FAA"; $resn_exch{" C4X FAA"}=" C4A FAA"; $resn_exch{" C5X FAA"}=" C5A FAA";
$resn_exch{" C1' FAA"}=" C1* FAA"; $resn_exch{" C2' FAA"}=" C2* FAA"; $resn_exch{" O2' FAA"}=" O2* FAA";
$resn_exch{" C3' FAA"}=" C3* FAA"; $resn_exch{" O3' FAA"}=" O3* FAA"; $resn_exch{" C4' FAA"}=" C4* FAA";
$resn_exch{" O4' FAA"}=" O4* FAA"; $resn_exch{" C5' FAA"}=" C5* FAA"; $resn_exch{" O5' FAA"}=" O5* FAA";
$resn_exch{" PA  FAB"}=" AP  FAB"; $resn_exch{" O1A FAB"}=" AO1 FAB"; $resn_exch{" O2A FAB"}=" AO2 FAB";
$resn_exch{" O5B FAB"}="AO5* FAB"; $resn_exch{" C5B FAB"}="AC5* FAB"; $resn_exch{" C4B FAB"}="AC4* FAB";
$resn_exch{" O4B FAB"}="AO4* FAB"; $resn_exch{" C3B FAB"}="AC3* FAB"; $resn_exch{" O3B FAB"}="AO3* FAB";
$resn_exch{" C2B FAB"}="AC2* FAB"; $resn_exch{" O2B FAB"}="AO2* FAB"; $resn_exch{" C1B FAB"}="AC1* FAB";
$resn_exch{" N9A FAB"}=" AN9 FAB"; $resn_exch{" C8A FAB"}=" AC8 FAB"; $resn_exch{" N7A FAB"}=" AN7 FAB";
$resn_exch{" C5A FAB"}=" AC5 FAB"; $resn_exch{" C6A FAB"}=" AC6 FAB"; $resn_exch{" N6A FAB"}=" AN6 FAB";
$resn_exch{" N1A FAB"}=" AN1 FAB"; $resn_exch{" C2A FAB"}=" AC2 FAB"; $resn_exch{" N3A FAB"}=" AN3 FAB";
$resn_exch{" C4A FAB"}=" AC4 FAB"; $resn_exch{" C4X FAB"}=" C4A FAB"; $resn_exch{" C5X FAB"}=" C5A FAB";
$resn_exch{" C1' FAB"}=" C1* FAB"; $resn_exch{" C2D FAB"}=" C2* FAB"; $resn_exch{" O2' FAB"}=" O2* FAB";
$resn_exch{" C3D FAB"}=" C3* FAB"; $resn_exch{" O3D FAB"}=" O3* FAB"; $resn_exch{" C4D FAB"}=" C4* FAB";
$resn_exch{" O4' FAB"}=" O4* FAB"; $resn_exch{" C5D FAB"}=" C5* FAB"; $resn_exch{" O5' FAB"}=" O5* FAB";
$resn_exch{" PA  FAD"}=" AP  FAD"; $resn_exch{" O1A FAD"}=" AO1 FAD"; $resn_exch{" O2A FAD"}=" AO2 FAD";
$resn_exch{" O5B FAD"}="AO5* FAD"; $resn_exch{" C5B FAD"}="AC5* FAD"; $resn_exch{" C4B FAD"}="AC4* FAD";
$resn_exch{" O4B FAD"}="AO4* FAD"; $resn_exch{" C3B FAD"}="AC3* FAD"; $resn_exch{" O3B FAD"}="AO3* FAD";
$resn_exch{" C2B FAD"}="AC2* FAD"; $resn_exch{" O2B FAD"}="AO2* FAD"; $resn_exch{" C1B FAD"}="AC1* FAD";
$resn_exch{" N9A FAD"}=" AN9 FAD"; $resn_exch{" C8A FAD"}=" AC8 FAD"; $resn_exch{" N7A FAD"}=" AN7 FAD";
$resn_exch{" C5A FAD"}=" AC5 FAD"; $resn_exch{" C6A FAD"}=" AC6 FAD"; $resn_exch{" N6A FAD"}=" AN6 FAD";
$resn_exch{" N1A FAD"}=" AN1 FAD"; $resn_exch{" C2A FAD"}=" AC2 FAD"; $resn_exch{" N3A FAD"}=" AN3 FAD";
$resn_exch{" C4A FAD"}=" AC4 FAD"; $resn_exch{" C4X FAD"}=" C4A FAD"; $resn_exch{" C5X FAD"}=" C5A FAD";
$resn_exch{" C1' FAD"}=" C1* FAD"; $resn_exch{" C2' FAD"}=" C2* FAD"; $resn_exch{" O2' FAD"}=" O2* FAD";
$resn_exch{" C3' FAD"}=" C3* FAD"; $resn_exch{" O3' FAD"}=" O3* FAD"; $resn_exch{" C4' FAD"}=" C4* FAD";
$resn_exch{" O4' FAD"}=" O4* FAD"; $resn_exch{" C5' FAD"}=" C5* FAD"; $resn_exch{" O5' FAD"}=" O5* FAD";
$resn_exch{" PA  FAE"}=" AP  FAE"; $resn_exch{" O1A FAE"}=" AO1 FAE"; $resn_exch{" O2A FAE"}=" AO2 FAE";
$resn_exch{" O5B FAE"}="AO5* FAE"; $resn_exch{" C5B FAE"}="AC5* FAE"; $resn_exch{" C4B FAE"}="AC4* FAE";
$resn_exch{" O4B FAE"}="AO4* FAE"; $resn_exch{" C3B FAE"}="AC3* FAE"; $resn_exch{" O3B FAE"}="AO3* FAE";
$resn_exch{" C2B FAE"}="AC2* FAE"; $resn_exch{" O2B FAE"}="AO2* FAE"; $resn_exch{" C1B FAE"}="AC1* FAE";
$resn_exch{" N9A FAE"}=" AN9 FAE"; $resn_exch{" C8A FAE"}=" AC8 FAE"; $resn_exch{" N7A FAE"}=" AN7 FAE";
$resn_exch{" C5A FAE"}=" AC5 FAE"; $resn_exch{" C6A FAE"}=" AC6 FAE"; $resn_exch{" N6A FAE"}=" AN6 FAE";
$resn_exch{" N1A FAE"}=" AN1 FAE"; $resn_exch{" C2A FAE"}=" AC2 FAE"; $resn_exch{" N3A FAE"}=" AN3 FAE";
$resn_exch{" C4A FAE"}=" AC4 FAE"; $resn_exch{" C4X FAE"}=" C4A FAE"; $resn_exch{" C5X FAE"}=" C5A FAE";
$resn_exch{" C1' FAE"}=" C1* FAE"; $resn_exch{" C2' FAE"}=" C2* FAE"; $resn_exch{" O2' FAE"}=" O2* FAE";
$resn_exch{" C3' FAE"}=" C3* FAE"; $resn_exch{" O3' FAE"}=" O3* FAE"; $resn_exch{" C4' FAE"}=" C4* FAE";
$resn_exch{" O4' FAE"}=" O4* FAE"; $resn_exch{" C5' FAE"}=" C5* FAE"; $resn_exch{" O5' FAE"}=" O5* FAE";
$resn_exch{" O5' FAG"}=" O5* FAG"; $resn_exch{" C5' FAG"}=" C5* FAG"; $resn_exch{" C4' FAG"}=" C4* FAG";
$resn_exch{" O4' FAG"}=" O4* FAG"; $resn_exch{" C1' FAG"}=" C1* FAG"; $resn_exch{" C2' FAG"}=" C2* FAG";
$resn_exch{" C3' FAG"}=" C3* FAG"; $resn_exch{" O3' FAG"}=" O3* FAG"; $resn_exch{" CAA FAG"}="AC10 FAG";
$resn_exch{" CBA FAG"}="AC11 FAG"; $resn_exch{" H31 FAG"}="1H3  FAG"; $resn_exch{" N1A FAM"}=" AN1 FAM";
$resn_exch{" C2A FAM"}=" AC2 FAM"; $resn_exch{" N3A FAM"}=" AN3 FAM"; $resn_exch{" C4A FAM"}=" AC4 FAM";
$resn_exch{" C5A FAM"}=" AC5 FAM"; $resn_exch{" C6A FAM"}=" AC6 FAM"; $resn_exch{" N6A FAM"}=" AN6 FAM";
$resn_exch{" N7A FAM"}=" AN7 FAM"; $resn_exch{" C8A FAM"}=" AC8 FAM"; $resn_exch{" N9A FAM"}=" AN9 FAM";
$resn_exch{" C1B FAM"}="AC1* FAM"; $resn_exch{" C2B FAM"}="AC2* FAM"; $resn_exch{" O2B FAM"}="AO2* FAM";
$resn_exch{" C3B FAM"}="AC3* FAM"; $resn_exch{" O3B FAM"}="AO3* FAM"; $resn_exch{" P3B FAM"}="AP3* FAM";
$resn_exch{" O7A FAM"}=" AO7 FAM"; $resn_exch{" O8A FAM"}=" AO8 FAM"; $resn_exch{" O9A FAM"}=" AO9 FAM";
$resn_exch{" C4B FAM"}="AC4* FAM"; $resn_exch{" O4B FAM"}="AO4* FAM"; $resn_exch{" C5B FAM"}="AC5* FAM";
$resn_exch{" O5B FAM"}="AO5* FAM"; $resn_exch{" P1A FAM"}=" AP1 FAM"; $resn_exch{" O1A FAM"}=" AO1 FAM";
$resn_exch{" O2A FAM"}=" AO2 FAM"; $resn_exch{" O3A FAM"}=" AO3 FAM"; $resn_exch{" P2A FAM"}=" AP2 FAM";
$resn_exch{" O4A FAM"}=" AO4 FAM"; $resn_exch{" O5A FAM"}=" AO5 FAM"; $resn_exch{" O6A FAM"}=" AO6 FAM";
$resn_exch{" CBP FAM"}="PC11 FAM"; $resn_exch{" CCP FAM"}="PC12 FAM"; $resn_exch{" CDP FAM"}="PC13 FAM";
$resn_exch{" CEP FAM"}="PC14 FAM"; $resn_exch{" CAP FAM"}="PC10 FAM"; $resn_exch{" OAP FAM"}="PO10 FAM";
$resn_exch{" C9P FAM"}=" PC9 FAM"; $resn_exch{" O9P FAM"}=" PO9 FAM"; $resn_exch{" N8P FAM"}=" PN8 FAM";
$resn_exch{" C7P FAM"}=" PC7 FAM"; $resn_exch{" C6P FAM"}=" PC6 FAM"; $resn_exch{" C5P FAM"}=" PC5 FAM";
$resn_exch{" O5P FAM"}=" PO5 FAM"; $resn_exch{" N4P FAM"}=" PN4 FAM"; $resn_exch{" C3P FAM"}=" PC3 FAM";
$resn_exch{" C2P FAM"}=" PC2 FAM"; $resn_exch{" PA  FAS"}=" AP  FAS"; $resn_exch{" O1A FAS"}=" AO1 FAS";
$resn_exch{" O2A FAS"}=" AO2 FAS"; $resn_exch{" O5B FAS"}="AO5* FAS"; $resn_exch{" C5B FAS"}="AC5* FAS";
$resn_exch{" C4B FAS"}="AC4* FAS"; $resn_exch{" O4B FAS"}="AO4* FAS"; $resn_exch{" C3B FAS"}="AC3* FAS";
$resn_exch{" O3B FAS"}="AO3* FAS"; $resn_exch{" C2B FAS"}="AC2* FAS"; $resn_exch{" O2B FAS"}="AO2* FAS";
$resn_exch{" C1B FAS"}="AC1* FAS"; $resn_exch{" N9A FAS"}=" AN9 FAS"; $resn_exch{" C8A FAS"}=" AC8 FAS";
$resn_exch{" N7A FAS"}=" AN7 FAS"; $resn_exch{" C5A FAS"}=" AC5 FAS"; $resn_exch{" C6A FAS"}=" AC6 FAS";
$resn_exch{" N6A FAS"}=" AN6 FAS"; $resn_exch{" N1A FAS"}=" AN1 FAS"; $resn_exch{" C2A FAS"}=" AC2 FAS";
$resn_exch{" N3A FAS"}=" AN3 FAS"; $resn_exch{" C4A FAS"}=" AC4 FAS"; $resn_exch{" C4X FAS"}=" C4A FAS";
$resn_exch{" C5X FAS"}=" C5A FAS"; $resn_exch{" C1' FAS"}=" C1* FAS"; $resn_exch{" C2' FAS"}=" C2* FAS";
$resn_exch{" O2' FAS"}=" O2* FAS"; $resn_exch{" C3' FAS"}=" C3* FAS"; $resn_exch{" O3' FAS"}=" O3* FAS";
$resn_exch{" C4' FAS"}=" C4* FAS"; $resn_exch{" O4' FAS"}=" O4* FAS"; $resn_exch{" C5' FAS"}=" C5* FAS";
$resn_exch{" O5' FAS"}=" O5* FAS"; $resn_exch{" OS6 FBQ"}="6OS  FBQ"; $resn_exch{" PA  FCG"}=" AP  FCG";
$resn_exch{" O1A FCG"}=" AO1 FCG"; $resn_exch{" O2A FCG"}=" AO2 FCG"; $resn_exch{" O5B FCG"}="AO5* FCG";
$resn_exch{" C5B FCG"}="AC5* FCG"; $resn_exch{" C4B FCG"}="AC4* FCG"; $resn_exch{" O4B FCG"}="AO4* FCG";
$resn_exch{" C1B FCG"}="AC1* FCG"; $resn_exch{" N9A FCG"}=" AN9 FCG"; $resn_exch{" C4A FCG"}=" AC4 FCG";
$resn_exch{" N3A FCG"}=" AN3 FCG"; $resn_exch{" C2A FCG"}=" AC2 FCG"; $resn_exch{" N1A FCG"}=" AN1 FCG";
$resn_exch{" C6A FCG"}=" AC6 FCG"; $resn_exch{" N6A FCG"}=" AN6 FCG"; $resn_exch{" C5A FCG"}=" AC5 FCG";
$resn_exch{" N7A FCG"}=" AN7 FCG"; $resn_exch{" C8A FCG"}=" AC8 FCG"; $resn_exch{" C2B FCG"}="AC2* FCG";
$resn_exch{" O2B FCG"}="AO2* FCG"; $resn_exch{" C3B FCG"}="AC3* FCG"; $resn_exch{" O3B FCG"}="AO3* FCG";
$resn_exch{" O5' FCG"}=" O5* FCG"; $resn_exch{" C5' FCG"}=" C5* FCG"; $resn_exch{" C4' FCG"}=" C4* FCG";
$resn_exch{" O4' FCG"}=" O4* FCG"; $resn_exch{" C3' FCG"}=" C3* FCG"; $resn_exch{" O3' FCG"}=" O3* FCG";
$resn_exch{" C2' FCG"}=" C2* FCG"; $resn_exch{" O2' FCG"}=" O2* FCG"; $resn_exch{" C1' FCG"}=" C1* FCG";
$resn_exch{" C4X FCG"}=" C4A FCG"; $resn_exch{" C5X FCG"}=" C5A FCG"; $resn_exch{" N1A FCX"}=" AN1 FCX";
$resn_exch{" C2A FCX"}=" AC2 FCX"; $resn_exch{" N3A FCX"}=" AN3 FCX"; $resn_exch{" C4A FCX"}=" AC4 FCX";
$resn_exch{" C5A FCX"}=" AC5 FCX"; $resn_exch{" C6A FCX"}=" AC6 FCX"; $resn_exch{" N6A FCX"}=" AN6 FCX";
$resn_exch{" N7A FCX"}=" AN7 FCX"; $resn_exch{" C8A FCX"}=" AC8 FCX"; $resn_exch{" N9A FCX"}=" AN9 FCX";
$resn_exch{" C1B FCX"}="AC1* FCX"; $resn_exch{" C2B FCX"}="AC2* FCX"; $resn_exch{" O2B FCX"}="AO2* FCX";
$resn_exch{" C3B FCX"}="AC3* FCX"; $resn_exch{" O3B FCX"}="AO3* FCX"; $resn_exch{" P3B FCX"}="AP3* FCX";
$resn_exch{" O7A FCX"}=" AO7 FCX"; $resn_exch{" O8A FCX"}=" AO8 FCX"; $resn_exch{" O9A FCX"}=" AO9 FCX";
$resn_exch{" C4B FCX"}="AC4* FCX"; $resn_exch{" O4B FCX"}="AO4* FCX"; $resn_exch{" C5B FCX"}="AC5* FCX";
$resn_exch{" O5B FCX"}="AO5* FCX"; $resn_exch{" P1A FCX"}=" AP1 FCX"; $resn_exch{" O1A FCX"}=" AO1 FCX";
$resn_exch{" O2A FCX"}=" AO2 FCX"; $resn_exch{" O3A FCX"}=" AO3 FCX"; $resn_exch{" P2A FCX"}=" AP2 FCX";
$resn_exch{" O4A FCX"}=" AO4 FCX"; $resn_exch{" O5A FCX"}=" AO5 FCX"; $resn_exch{" O6A FCX"}=" AO6 FCX";
$resn_exch{" CBP FCX"}="PC11 FCX"; $resn_exch{" CCP FCX"}="PC12 FCX"; $resn_exch{" CDP FCX"}="PC13 FCX";
$resn_exch{" CEP FCX"}="PC14 FCX"; $resn_exch{" CAP FCX"}="PC10 FCX"; $resn_exch{" OAP FCX"}="PO10 FCX";
$resn_exch{" C9P FCX"}=" PC9 FCX"; $resn_exch{" O9P FCX"}=" PO9 FCX"; $resn_exch{" N8P FCX"}=" PN8 FCX";
$resn_exch{" C7P FCX"}=" PC7 FCX"; $resn_exch{" C6P FCX"}=" PC6 FCX"; $resn_exch{" C5P FCX"}=" PC5 FCX";
$resn_exch{" O5P FCX"}=" PO5 FCX"; $resn_exch{" N4P FCX"}=" PN4 FCX"; $resn_exch{" C3P FCX"}=" PC3 FCX";
$resn_exch{" C2P FCX"}=" PC2 FCX"; $resn_exch{" PA  FDA"}=" AP  FDA"; $resn_exch{" O1A FDA"}=" AO1 FDA";
$resn_exch{" O2A FDA"}=" AO2 FDA"; $resn_exch{" O5B FDA"}="AO5* FDA"; $resn_exch{" C5B FDA"}="AC5* FDA";
$resn_exch{" C4B FDA"}="AC4* FDA"; $resn_exch{" O4B FDA"}="AO4* FDA"; $resn_exch{" C3B FDA"}="AC3* FDA";
$resn_exch{" O3B FDA"}="AO3* FDA"; $resn_exch{" C2B FDA"}="AC2* FDA"; $resn_exch{" O2B FDA"}="AO2* FDA";
$resn_exch{" C1B FDA"}="AC1* FDA"; $resn_exch{" N9A FDA"}=" AN9 FDA"; $resn_exch{" C8A FDA"}=" AC8 FDA";
$resn_exch{" N7A FDA"}=" AN7 FDA"; $resn_exch{" C5A FDA"}=" AC5 FDA"; $resn_exch{" C6A FDA"}=" AC6 FDA";
$resn_exch{" N6A FDA"}=" AN6 FDA"; $resn_exch{" N1A FDA"}=" AN1 FDA"; $resn_exch{" C2A FDA"}=" AC2 FDA";
$resn_exch{" N3A FDA"}=" AN3 FDA"; $resn_exch{" C4A FDA"}=" AC4 FDA"; $resn_exch{" C4X FDA"}=" C4A FDA";
$resn_exch{" C5X FDA"}=" C5A FDA"; $resn_exch{" C1' FDA"}=" C1* FDA"; $resn_exch{" C2' FDA"}=" C2* FDA";
$resn_exch{" O2' FDA"}=" O2* FDA"; $resn_exch{" C3' FDA"}=" C3* FDA"; $resn_exch{" O3' FDA"}=" O3* FDA";
$resn_exch{" C4' FDA"}=" C4* FDA"; $resn_exch{" O4' FDA"}=" O4* FDA"; $resn_exch{" C5' FDA"}=" C5* FDA";
$resn_exch{" O5' FDA"}=" O5* FDA"; $resn_exch{" ND  FDD"}=" N_D FDD"; $resn_exch{" NA  FDD"}=" N_A FDD";
$resn_exch{" NC  FDD"}=" N_C FDD"; $resn_exch{" NB  FDD"}=" N_B FDD"; $resn_exch{" NA  FDE"}=" N_A FDE";
$resn_exch{" NB  FDE"}=" N_B FDE"; $resn_exch{" NC  FDE"}=" N_C FDE"; $resn_exch{" ND  FDE"}=" N_D FDE";
$resn_exch{" O4' FDM"}=" O4* FDM"; $resn_exch{" C1' FDM"}=" C1* FDM"; $resn_exch{" C2' FDM"}=" C2* FDM";
$resn_exch{" C3' FDM"}=" C3* FDM"; $resn_exch{" C4' FDM"}=" C4* FDM"; $resn_exch{" O5' FDM"}=" O5* FDM";
$resn_exch{" C5' FDM"}=" C5* FDM"; $resn_exch{" C1' FFD"}=" C1* FFD"; $resn_exch{" O4' FFD"}=" O4* FFD";
$resn_exch{" C2' FFD"}=" C2* FFD"; $resn_exch{" C3' FFD"}=" C3* FFD"; $resn_exch{" O3' FFD"}=" O3* FFD";
$resn_exch{" C4' FFD"}=" C4* FFD"; $resn_exch{" C5' FFD"}=" C5* FFD"; $resn_exch{" O5' FFD"}=" O5* FFD";
$resn_exch{" C1' FHU"}=" C1* FHU"; $resn_exch{" C2' FHU"}=" C2* FHU"; $resn_exch{" O2' FHU"}=" O2* FHU";
$resn_exch{" C3' FHU"}=" C3* FHU"; $resn_exch{" C4' FHU"}=" C4* FHU"; $resn_exch{" O3' FHU"}=" O3* FHU";
$resn_exch{" O4' FHU"}=" O4* FHU"; $resn_exch{" C5' FHU"}=" C5* FHU"; $resn_exch{" O5' FHU"}=" O5* FHU";
$resn_exch{" OP1 FHU"}=" O1P FHU"; $resn_exch{" OP2 FHU"}=" O2P FHU"; $resn_exch{" OP3 FHU"}=" O3P FHU";
$resn_exch{" C3B FL8"}=" C3* FL8"; $resn_exch{" C1' FL9"}=" C1* FL9"; $resn_exch{" C2' FL9"}=" C2* FL9";
$resn_exch{" C3' FL9"}=" C3* FL9"; $resn_exch{" C4' FL9"}=" C4* FL9"; $resn_exch{" C5' FL9"}=" C5* FL9";
$resn_exch{" C6' FL9"}=" C6* FL9"; $resn_exch{" O2' FL9"}=" O2* FL9"; $resn_exch{" O4' FL9"}=" O4* FL9";
$resn_exch{" C1X FLD"}=" XC1 FLD"; $resn_exch{" N2X FLD"}=" XN2 FLD"; $resn_exch{" C3X FLD"}=" XC3 FLD";
$resn_exch{" C4X FLD"}=" XC4 FLD"; $resn_exch{" C5X FLD"}=" XC5 FLD"; $resn_exch{" C6X FLD"}=" XC6 FLD";
$resn_exch{" N7X FLD"}=" XN7 FLD"; $resn_exch{" C8X FLD"}=" XC8 FLD"; $resn_exch{" C9X FLD"}=" XC9 FLD";
$resn_exch{" CAX FLD"}="XC10 FLD"; $resn_exch{" OAX FLD"}="XO10 FLD"; $resn_exch{" CBX FLD"}="XC11 FLD";
$resn_exch{" CCX FLD"}="XC12 FLD"; $resn_exch{" CDX FLD"}="XC13 FLD"; $resn_exch{" CEX FLD"}="XC14 FLD";
$resn_exch{" CFX FLD"}="XC15 FLD"; $resn_exch{" CGX FLD"}="XC16 FLD"; $resn_exch{" CHX FLD"}="XC17 FLD";
$resn_exch{" CIX FLD"}="XC18 FLD"; $resn_exch{" CJX FLD"}="XC19 FLD"; $resn_exch{" CKX FLD"}="XC20 FLD";
$resn_exch{" NLX FLD"}="XN21 FLD"; $resn_exch{" CMX FLD"}="XC22 FLD"; $resn_exch{" CNX FLD"}="XC23 FLD";
$resn_exch{" COX FLD"}="XC24 FLD"; $resn_exch{" C1' FLN"}=" C1* FLN"; $resn_exch{" C2' FLN"}=" C2* FLN";
$resn_exch{" C3' FLN"}=" C3* FLN"; $resn_exch{" C4' FLN"}=" C4* FLN"; $resn_exch{" C5' FLN"}=" C5* FLN";
$resn_exch{" C6' FLN"}=" C6* FLN"; $resn_exch{" C1' FLQ"}=" C1* FLQ"; $resn_exch{" C2' FLQ"}=" C2* FLQ";
$resn_exch{" C3' FLQ"}=" C3* FLQ"; $resn_exch{" C4' FLQ"}=" C4* FLQ"; $resn_exch{" C5' FLQ"}=" C5* FLQ";
$resn_exch{" C6' FLQ"}=" C6* FLQ"; $resn_exch{" O8' FLQ"}=" O8* FLQ"; $resn_exch{" C9' FLQ"}=" C9* FLQ";
$resn_exch{" O9' FLQ"}=" O9* FLQ"; $resn_exch{" C1B FMA"}=" C1* FMA"; $resn_exch{" C2B FMA"}=" C2* FMA";
$resn_exch{" O2B FMA"}=" O2* FMA"; $resn_exch{" C3B FMA"}=" C3* FMA"; $resn_exch{" O3' FMA"}=" O3* FMA";
$resn_exch{" C4B FMA"}=" C4* FMA"; $resn_exch{" O4' FMA"}=" O4* FMA"; $resn_exch{" C5B FMA"}=" C5* FMA";
$resn_exch{" O5' FMA"}=" O5* FMA"; $resn_exch{" NA  FMI"}=" N_A FMI"; $resn_exch{" NB  FMI"}=" N_B FMI";
$resn_exch{" NC  FMI"}=" N_C FMI"; $resn_exch{" ND  FMI"}=" N_D FMI"; $resn_exch{" C1' FMN"}=" C1* FMN";
$resn_exch{" C2' FMN"}=" C2* FMN"; $resn_exch{" O2' FMN"}=" O2* FMN"; $resn_exch{" C3' FMN"}=" C3* FMN";
$resn_exch{" O3' FMN"}=" O3* FMN"; $resn_exch{" C4' FMN"}=" C4* FMN"; $resn_exch{" O4' FMN"}=" O4* FMN";
$resn_exch{" C5' FMN"}=" C5* FMN"; $resn_exch{" O5' FMN"}=" O5* FMN"; $resn_exch{" O5' FMP"}=" O5* FMP";
$resn_exch{" C5' FMP"}=" C5* FMP"; $resn_exch{" C4' FMP"}=" C4* FMP"; $resn_exch{" O4' FMP"}=" O4* FMP";
$resn_exch{" C3' FMP"}=" C3* FMP"; $resn_exch{" O3' FMP"}=" O3* FMP"; $resn_exch{" C2' FMP"}=" C2* FMP";
$resn_exch{" O2' FMP"}=" O2* FMP"; $resn_exch{" C1' FMP"}=" C1* FMP"; $resn_exch{" C1' FMU"}=" C1* FMU";
$resn_exch{" C2' FMU"}=" C2* FMU"; $resn_exch{" O2' FMU"}=" O2* FMU"; $resn_exch{" C3' FMU"}=" C3* FMU";
$resn_exch{" C4' FMU"}=" C4* FMU"; $resn_exch{" O3' FMU"}=" O3* FMU"; $resn_exch{" O4' FMU"}=" O4* FMU";
$resn_exch{" C5' FMU"}=" C5* FMU"; $resn_exch{" O5' FMU"}=" O5* FMU"; $resn_exch{" CAA FNR"}="C10A FNR";
$resn_exch{" C1' FNR"}=" C1* FNR"; $resn_exch{" C2' FNR"}=" C2* FNR"; $resn_exch{" O2' FNR"}=" O2* FNR";
$resn_exch{" C3' FNR"}=" C3* FNR"; $resn_exch{" O3' FNR"}=" O3* FNR"; $resn_exch{" C4' FNR"}=" C4* FNR";
$resn_exch{" O4' FNR"}=" O4* FNR"; $resn_exch{" C5' FNR"}=" C5* FNR"; $resn_exch{" O5' FNR"}=" O5* FNR";
$resn_exch{" C1' FNS"}=" C1* FNS"; $resn_exch{" C2' FNS"}=" C2* FNS"; $resn_exch{" O2' FNS"}=" O2* FNS";
$resn_exch{" C3' FNS"}=" C3* FNS"; $resn_exch{" O3' FNS"}=" O3* FNS"; $resn_exch{" C4' FNS"}=" C4* FNS";
$resn_exch{" O4' FNS"}=" O4* FNS"; $resn_exch{" C5' FNS"}=" C5* FNS"; $resn_exch{" O5' FNS"}=" O5* FNS";
$resn_exch{" O5' FOX"}=" O5* FOX"; $resn_exch{" C5' FOX"}=" C5* FOX"; $resn_exch{" C4' FOX"}=" C4* FOX";
$resn_exch{" C6' FOX"}=" C6* FOX"; $resn_exch{" C3' FOX"}=" C3* FOX"; $resn_exch{" O3' FOX"}=" O3* FOX";
$resn_exch{" C2' FOX"}=" C2* FOX"; $resn_exch{" C1' FOX"}=" C1* FOX"; $resn_exch{" N1A FRE"}=" AN1 FRE";
$resn_exch{" C2A FRE"}=" AC2 FRE"; $resn_exch{" N3A FRE"}=" AN3 FRE"; $resn_exch{" C4A FRE"}=" AC4 FRE";
$resn_exch{" C5A FRE"}=" AC5 FRE"; $resn_exch{" C6A FRE"}=" AC6 FRE"; $resn_exch{" N6A FRE"}=" AN6 FRE";
$resn_exch{" N7A FRE"}=" AN7 FRE"; $resn_exch{" C8A FRE"}=" AC8 FRE"; $resn_exch{" N9A FRE"}=" AN9 FRE";
$resn_exch{" C1B FRE"}="AC1* FRE"; $resn_exch{" C2B FRE"}="AC2* FRE"; $resn_exch{" O2B FRE"}="AO2* FRE";
$resn_exch{" C3B FRE"}="AC3* FRE"; $resn_exch{" O3B FRE"}="AO3* FRE"; $resn_exch{" P3B FRE"}="AP3* FRE";
$resn_exch{" O7A FRE"}=" AO7 FRE"; $resn_exch{" O8A FRE"}=" AO8 FRE"; $resn_exch{" O9A FRE"}=" AO9 FRE";
$resn_exch{" C4B FRE"}="AC4* FRE"; $resn_exch{" O4B FRE"}="AO4* FRE"; $resn_exch{" C5B FRE"}="AC5* FRE";
$resn_exch{" O5B FRE"}="AO5* FRE"; $resn_exch{" P1A FRE"}=" AP1 FRE"; $resn_exch{" O1A FRE"}=" AO1 FRE";
$resn_exch{" O2A FRE"}=" AO2 FRE"; $resn_exch{" O3A FRE"}=" AO3 FRE"; $resn_exch{" P2A FRE"}=" AP2 FRE";
$resn_exch{" O4A FRE"}=" AO4 FRE"; $resn_exch{" O5A FRE"}=" AO5 FRE"; $resn_exch{" O6A FRE"}=" AO6 FRE";
$resn_exch{" CBP FRE"}="PC11 FRE"; $resn_exch{" CCP FRE"}="PC12 FRE"; $resn_exch{" CDP FRE"}="PC13 FRE";
$resn_exch{" CEP FRE"}="PC14 FRE"; $resn_exch{" CAP FRE"}="PC10 FRE"; $resn_exch{" OAP FRE"}="PO10 FRE";
$resn_exch{" C9P FRE"}=" PC9 FRE"; $resn_exch{" O9P FRE"}=" PO9 FRE"; $resn_exch{" N8P FRE"}=" PN8 FRE";
$resn_exch{" C7P FRE"}=" PC7 FRE"; $resn_exch{" C6P FRE"}=" PC6 FRE"; $resn_exch{" C5P FRE"}=" PC5 FRE";
$resn_exch{" O5P FRE"}=" PO5 FRE"; $resn_exch{" N4P FRE"}=" PN4 FRE"; $resn_exch{" C3P FRE"}=" PC3 FRE";
$resn_exch{" C2P FRE"}=" PC2 FRE"; $resn_exch{" S1P FRE"}=" PS1 FRE"; $resn_exch{" O13 FRN"}="1OCT FRN";
$resn_exch{" C2A FSM"}=" AC2 FSM"; $resn_exch{" C3A FSM"}=" AC3 FSM"; $resn_exch{" C4A FSM"}=" AC4 FSM";
$resn_exch{" C5A FSM"}=" AC5 FSM"; $resn_exch{" C6A FSM"}=" AC6 FSM"; $resn_exch{" C7A FSM"}=" AC7 FSM";
$resn_exch{" C8A FSM"}=" AC8 FSM"; $resn_exch{" N2A FSM"}=" AN2 FSM"; $resn_exch{" O1A FSM"}=" AO1 FSM";
$resn_exch{" O3A FSM"}=" AO3 FSM"; $resn_exch{" O4A FSM"}=" AO4 FSM"; $resn_exch{" O5A FSM"}=" AO5 FSM";
$resn_exch{" O6A FSM"}=" AO6 FSM"; $resn_exch{" O7A FSM"}=" AO7 FSM"; $resn_exch{" C1B FSM"}=" BC1 FSM";
$resn_exch{" C2B FSM"}=" BC2 FSM"; $resn_exch{" C3B FSM"}=" BC3 FSM"; $resn_exch{" C4B FSM"}=" BC4 FSM";
$resn_exch{" C5B FSM"}=" BC5 FSM"; $resn_exch{" C6B FSM"}=" BC6 FSM"; $resn_exch{" C7B FSM"}=" BC7 FSM";
$resn_exch{" C8B FSM"}=" BC8 FSM"; $resn_exch{" N2B FSM"}=" BN2 FSM"; $resn_exch{" O1B FSM"}=" BO1 FSM";
$resn_exch{" O3B FSM"}=" BO3 FSM"; $resn_exch{" O5B FSM"}=" BO5 FSM"; $resn_exch{" O6B FSM"}=" BO6 FSM";
$resn_exch{" O7B FSM"}=" BO7 FSM"; $resn_exch{" O5' FUP"}=" O5* FUP"; $resn_exch{" C5' FUP"}=" C5* FUP";
$resn_exch{" C4' FUP"}=" C4* FUP"; $resn_exch{" O4' FUP"}=" O4* FUP"; $resn_exch{" C3' FUP"}=" C3* FUP";
$resn_exch{" F3' FUP"}=" F3* FUP"; $resn_exch{" C2' FUP"}=" C2* FUP"; $resn_exch{" C1' FUP"}=" C1* FUP";
$resn_exch{" C1  FVF"}="1C   FVF"; $resn_exch{" O1  FVF"}="1O   FVF"; $resn_exch{" CB1 FVF"}="1CB  FVF";
$resn_exch{" CG1 FVF"}="1CG  FVF"; $resn_exch{" CDB FVF"}="1CD1 FVF"; $resn_exch{" CDC FVF"}="1CD2 FVF";
$resn_exch{" CEB FVF"}="1CE1 FVF"; $resn_exch{" CEC FVF"}="1CE2 FVF"; $resn_exch{" CZ1 FVF"}="1CZ  FVF";
$resn_exch{" N2  FVF"}="2N   FVF"; $resn_exch{" C2  FVF"}="2C   FVF"; $resn_exch{" O2  FVF"}="2O   FVF";
$resn_exch{" CA2 FVF"}="2CA  FVF"; $resn_exch{" CB2 FVF"}="2CB  FVF"; $resn_exch{" CG2 FVF"}="2CG  FVF";
$resn_exch{" CDL FVF"}="2CD1 FVF"; $resn_exch{" CDM FVF"}="2CD2 FVF"; $resn_exch{" CEL FVF"}="2CE1 FVF";
$resn_exch{" CEM FVF"}="2CE2 FVF"; $resn_exch{" CZ2 FVF"}="2CZ  FVF"; $resn_exch{" N3  FVF"}="3N   FVF";
$resn_exch{" CA3 FVF"}="3CA  FVF"; $resn_exch{" CB3 FVF"}="3CB  FVF"; $resn_exch{" CGD FVF"}="3CG1 FVF";
$resn_exch{" CGN FVF"}="3CG2 FVF"; $resn_exch{" P3  FVF"}="3P   FVF"; $resn_exch{" OPV FVF"}="1OP3 FVF";
$resn_exch{" OPW FVF"}="2OP3 FVF"; $resn_exch{" OPX FVF"}="3OP3 FVF"; $resn_exch{" CA4 FVF"}="4CA  FVF";
$resn_exch{" C4  FVF"}="4C   FVF"; $resn_exch{" O4  FVF"}="4O   FVF"; $resn_exch{" O11 FVF"}="4OXT FVF";
$resn_exch{" CB4 FVF"}="4CB  FVF"; $resn_exch{" CG4 FVF"}="4CG  FVF"; $resn_exch{" CD5 FVF"}="4CD1 FVF";
$resn_exch{" CD6 FVF"}="4CD2 FVF"; $resn_exch{" CE5 FVF"}="4CE1 FVF"; $resn_exch{" CE6 FVF"}="4CE2 FVF";
$resn_exch{" CZ4 FVF"}="4CZ  FVF"; $resn_exch{" O5' FX3"}=" O5* FX3"; $resn_exch{" C5' FX3"}=" C5* FX3";
$resn_exch{" C4' FX3"}=" C4* FX3"; $resn_exch{" O4' FX3"}=" O4* FX3"; $resn_exch{" C3' FX3"}=" C3* FX3";
$resn_exch{" O3' FX3"}=" O3* FX3"; $resn_exch{" C2' FX3"}=" C2* FX3"; $resn_exch{" O2' FX3"}=" O2* FX3";
$resn_exch{" C1' FX3"}=" C1* FX3"; $resn_exch{" O5' FYA"}=" O5* FYA"; $resn_exch{" C5' FYA"}=" C5* FYA";
$resn_exch{" C4' FYA"}=" C4* FYA"; $resn_exch{" O4' FYA"}=" O4* FYA"; $resn_exch{" C3' FYA"}=" C3* FYA";
$resn_exch{" O3' FYA"}=" O3* FYA"; $resn_exch{" C2' FYA"}=" C2* FYA"; $resn_exch{" O2' FYA"}=" O2* FYA";
$resn_exch{" C1' FYA"}=" C1* FYA"; $resn_exch{" OP3   G"}=" O3P   G"; $resn_exch{" OP1   G"}=" O1P   G";
$resn_exch{" OP2   G"}=" O2P   G"; $resn_exch{" O5'   G"}=" O5*   G"; $resn_exch{" C5'   G"}=" C5*   G";
$resn_exch{" C4'   G"}=" C4*   G"; $resn_exch{" O4'   G"}=" O4*   G"; $resn_exch{" C3'   G"}=" C3*   G";
$resn_exch{" O3'   G"}=" O3*   G"; $resn_exch{" C2'   G"}=" C2*   G"; $resn_exch{" O2'   G"}=" O2*   G";
$resn_exch{" C1'   G"}=" C1*   G"; $resn_exch{" P'  G16"}=" P*  G16"; $resn_exch{" O1X G16"}="O1P* G16";
$resn_exch{" O2X G16"}="O2P* G16"; $resn_exch{" O3X G16"}="O3P* G16"; $resn_exch{" C1' G1R"}=" C1* G1R";
$resn_exch{" O4' G1R"}=" O4* G1R"; $resn_exch{" C2' G1R"}=" C2* G1R"; $resn_exch{" O2' G1R"}=" O2* G1R";
$resn_exch{" C3' G1R"}=" C3* G1R"; $resn_exch{" O3' G1R"}=" O3* G1R"; $resn_exch{" C4' G1R"}=" C4* G1R";
$resn_exch{" C5' G1R"}=" C5* G1R"; $resn_exch{" O5' G1R"}=" O5* G1R"; $resn_exch{" OP3 G25"}=" O3P G25";
$resn_exch{" OP1 G25"}=" O1P G25"; $resn_exch{" OP2 G25"}=" O2P G25"; $resn_exch{" O5' G25"}=" O5* G25";
$resn_exch{" C5' G25"}=" C5* G25"; $resn_exch{" C4' G25"}=" C4* G25"; $resn_exch{" O4' G25"}=" O4* G25";
$resn_exch{" C3' G25"}=" C3* G25"; $resn_exch{" O3' G25"}=" O3* G25"; $resn_exch{" C2' G25"}=" C2* G25";
$resn_exch{" O2' G25"}=" O2* G25"; $resn_exch{" C1' G25"}=" C1* G25"; $resn_exch{" OP3 G2L"}=" O3P G2L";
$resn_exch{" OP1 G2L"}=" O1P G2L"; $resn_exch{" OP2 G2L"}=" O2P G2L"; $resn_exch{" O5' G2L"}=" O5* G2L";
$resn_exch{" C5' G2L"}=" C5* G2L"; $resn_exch{" C4' G2L"}=" C4* G2L"; $resn_exch{" O4' G2L"}=" O4* G2L";
$resn_exch{" C3' G2L"}=" C3* G2L"; $resn_exch{" O3' G2L"}=" O3* G2L"; $resn_exch{" C2' G2L"}=" C2* G2L";
$resn_exch{" O2' G2L"}=" O2* G2L"; $resn_exch{" C1' G2L"}=" C1* G2L"; $resn_exch{" CA' G2L"}=" CA* G2L";
$resn_exch{" CB' G2L"}=" CB* G2L"; $resn_exch{" OC' G2L"}=" OC* G2L"; $resn_exch{" CD' G2L"}=" CD* G2L";
$resn_exch{" O5' G2P"}=" O5* G2P"; $resn_exch{" C5' G2P"}=" C5* G2P"; $resn_exch{" C4' G2P"}=" C4* G2P";
$resn_exch{" O4' G2P"}=" O4* G2P"; $resn_exch{" C3' G2P"}=" C3* G2P"; $resn_exch{" O3' G2P"}=" O3* G2P";
$resn_exch{" C2' G2P"}=" C2* G2P"; $resn_exch{" O2' G2P"}=" O2* G2P"; $resn_exch{" C1' G2P"}=" C1* G2P";
$resn_exch{" C1' G2R"}=" C1* G2R"; $resn_exch{" O4' G2R"}=" O4* G2R"; $resn_exch{" C2' G2R"}=" C2* G2R";
$resn_exch{" O2' G2R"}=" O2* G2R"; $resn_exch{" C3' G2R"}=" C3* G2R"; $resn_exch{" O3' G2R"}=" O3* G2R";
$resn_exch{" C4' G2R"}=" C4* G2R"; $resn_exch{" C5' G2R"}=" C5* G2R"; $resn_exch{" O5' G2R"}=" O5* G2R";
$resn_exch{" OP3 G2S"}=" O3P G2S"; $resn_exch{" O5' G2S"}=" O5* G2S"; $resn_exch{" C5' G2S"}=" C5* G2S";
$resn_exch{" C4' G2S"}=" C4* G2S"; $resn_exch{" O4' G2S"}=" O4* G2S"; $resn_exch{" C3' G2S"}=" C3* G2S";
$resn_exch{" O3' G2S"}=" O3* G2S"; $resn_exch{" C2' G2S"}=" C2* G2S"; $resn_exch{" C1' G2S"}=" C1* G2S";
$resn_exch{" O5' G31"}=" O5* G31"; $resn_exch{" C5' G31"}=" C5* G31"; $resn_exch{" C4' G31"}=" C4* G31";
$resn_exch{" O4' G31"}=" O4* G31"; $resn_exch{" C3' G31"}=" C3* G31"; $resn_exch{" C2' G31"}=" C2* G31";
$resn_exch{" C1' G31"}=" C1* G31"; $resn_exch{" O5' G32"}=" O5* G32"; $resn_exch{" C5' G32"}=" C5* G32";
$resn_exch{" C4' G32"}=" C4* G32"; $resn_exch{" O4' G32"}=" O4* G32"; $resn_exch{" C3' G32"}=" C3* G32";
$resn_exch{" O3' G32"}=" O3* G32"; $resn_exch{" C2' G32"}=" C2* G32"; $resn_exch{" C1' G32"}=" C1* G32";
$resn_exch{" O5' G33"}=" O5* G33"; $resn_exch{" C5' G33"}=" C5* G33"; $resn_exch{" C4' G33"}=" C4* G33";
$resn_exch{" O4' G33"}=" O4* G33"; $resn_exch{" C3' G33"}=" C3* G33"; $resn_exch{" O3' G33"}=" O3* G33";
$resn_exch{" C2' G33"}=" C2* G33"; $resn_exch{" C1' G33"}=" C1* G33"; $resn_exch{" O5' G36"}=" O5* G36";
$resn_exch{" C5' G36"}=" C5* G36"; $resn_exch{" C4' G36"}=" C4* G36"; $resn_exch{" O4' G36"}=" O4* G36";
$resn_exch{" C3' G36"}=" C3* G36"; $resn_exch{" O3' G36"}=" O3* G36"; $resn_exch{" C2' G36"}=" C2* G36";
$resn_exch{" C1' G36"}=" C1* G36"; $resn_exch{" O5' G38"}=" O5* G38"; $resn_exch{" C5' G38"}=" C5* G38";
$resn_exch{" C4' G38"}=" C4* G38"; $resn_exch{" O4' G38"}=" O4* G38"; $resn_exch{" C3' G38"}=" C3* G38";
$resn_exch{" C2' G38"}=" C2* G38"; $resn_exch{" C1' G38"}=" C1* G38"; $resn_exch{" O5' G3D"}=" O5* G3D";
$resn_exch{" C5' G3D"}=" C5* G3D"; $resn_exch{" C4' G3D"}=" C4* G3D"; $resn_exch{" O4' G3D"}=" O4* G3D";
$resn_exch{" C3' G3D"}=" C3* G3D"; $resn_exch{" O3' G3D"}=" O3* G3D"; $resn_exch{" C2' G3D"}=" C2* G3D";
$resn_exch{" O2' G3D"}=" O2* G3D"; $resn_exch{" C1' G3D"}=" C1* G3D"; $resn_exch{" O5' G42"}=" O5* G42";
$resn_exch{" C5' G42"}=" C5* G42"; $resn_exch{" C4' G42"}=" C4* G42"; $resn_exch{" O4' G42"}=" O4* G42";
$resn_exch{" C3' G42"}=" C3* G42"; $resn_exch{" O3' G42"}=" O3* G42"; $resn_exch{" C2' G42"}=" C2* G42";
$resn_exch{" C1' G42"}=" C1* G42"; $resn_exch{" O5' G46"}=" O5* G46"; $resn_exch{" C5' G46"}=" C5* G46";
$resn_exch{" C4' G46"}=" C4* G46"; $resn_exch{" O4' G46"}=" O4* G46"; $resn_exch{" C3' G46"}=" C3* G46";
$resn_exch{" O3' G46"}=" O3* G46"; $resn_exch{" C2' G46"}=" C2* G46"; $resn_exch{" C1' G46"}=" C1* G46";
$resn_exch{" O2' G46"}=" O2* G46"; $resn_exch{" O5' G47"}=" O5* G47"; $resn_exch{" C5' G47"}=" C5* G47";
$resn_exch{" C4' G47"}=" C4* G47"; $resn_exch{" O4' G47"}=" O4* G47"; $resn_exch{" C3' G47"}=" C3* G47";
$resn_exch{" O3' G47"}=" O3* G47"; $resn_exch{" C2' G47"}=" C2* G47"; $resn_exch{" C1' G47"}=" C1* G47";
$resn_exch{" O5' G48"}=" O5* G48"; $resn_exch{" C5' G48"}=" C5* G48"; $resn_exch{" C4' G48"}=" C4* G48";
$resn_exch{" O4' G48"}=" O4* G48"; $resn_exch{" C3' G48"}=" C3* G48"; $resn_exch{" O3' G48"}=" O3* G48";
$resn_exch{" C2' G48"}=" C2* G48"; $resn_exch{" O2' G48"}=" O2* G48"; $resn_exch{" C1' G48"}=" C1* G48";
$resn_exch{" CA' G48"}=" CA* G48"; $resn_exch{" CB' G48"}=" CB* G48"; $resn_exch{" OC' G48"}=" OC* G48";
$resn_exch{" CD' G48"}=" CD* G48"; $resn_exch{" O5' G49"}=" O5* G49"; $resn_exch{" C5' G49"}=" C5* G49";
$resn_exch{" C4' G49"}=" C4* G49"; $resn_exch{" O4' G49"}=" O4* G49"; $resn_exch{" C3' G49"}=" C3* G49";
$resn_exch{" O3' G49"}=" O3* G49"; $resn_exch{" C2' G49"}=" C2* G49"; $resn_exch{" C1' G49"}=" C1* G49";
$resn_exch{" O5' G4M"}=" O5* G4M"; $resn_exch{" C5' G4M"}=" C5* G4M"; $resn_exch{" C4' G4M"}=" C4* G4M";
$resn_exch{" O4' G4M"}=" O4* G4M"; $resn_exch{" C3' G4M"}=" C3* G4M"; $resn_exch{" O3' G4M"}=" O3* G4M";
$resn_exch{" C2' G4M"}=" C2* G4M"; $resn_exch{" O2' G4M"}=" O2* G4M"; $resn_exch{" C1' G4M"}=" C1* G4M";
$resn_exch{" O5' G4P"}=" O5* G4P"; $resn_exch{" C5' G4P"}=" C5* G4P"; $resn_exch{" C4' G4P"}=" C4* G4P";
$resn_exch{" O4' G4P"}=" O4* G4P"; $resn_exch{" C3' G4P"}=" C3* G4P"; $resn_exch{" O3' G4P"}=" O3* G4P";
$resn_exch{" C2' G4P"}=" C2* G4P"; $resn_exch{" O2' G4P"}=" O2* G4P"; $resn_exch{" C1' G4P"}=" C1* G4P";
$resn_exch{" O5' G5P"}=" O5* G5P"; $resn_exch{" C5' G5P"}=" C5* G5P"; $resn_exch{" C4' G5P"}=" C4* G5P";
$resn_exch{" O4' G5P"}=" O4* G5P"; $resn_exch{" C3' G5P"}=" C3* G5P"; $resn_exch{" O3' G5P"}=" O3* G5P";
$resn_exch{" C2' G5P"}=" C2* G5P"; $resn_exch{" O2' G5P"}=" O2* G5P"; $resn_exch{" C1' G5P"}=" C1* G5P";
$resn_exch{" O5F G5P"}="AO5* G5P"; $resn_exch{" C5F G5P"}="AC5* G5P"; $resn_exch{" C4F G5P"}="AC4* G5P";
$resn_exch{" O4F G5P"}="AO4* G5P"; $resn_exch{" C1F G5P"}="AC1* G5P"; $resn_exch{" C2F G5P"}="AC2* G5P";
$resn_exch{" O2F G5P"}="AO2* G5P"; $resn_exch{" C3F G5P"}="AC3* G5P"; $resn_exch{" O3F G5P"}="AO3* G5P";
$resn_exch{" N9A G5P"}=" AN9 G5P"; $resn_exch{" C4A G5P"}=" AC4 G5P"; $resn_exch{" N3A G5P"}=" AN3 G5P";
$resn_exch{" C2A G5P"}=" AC2 G5P"; $resn_exch{" N1A G5P"}=" AN1 G5P"; $resn_exch{" C6A G5P"}=" AC6 G5P";
$resn_exch{" N6A G5P"}=" AN6 G5P"; $resn_exch{" C5A G5P"}=" AC5 G5P"; $resn_exch{" N7A G5P"}=" AN7 G5P";
$resn_exch{" C8A G5P"}=" AC8 G5P"; $resn_exch{" OP3 G7M"}=" O3P G7M"; $resn_exch{" OP1 G7M"}=" O1P G7M";
$resn_exch{" OP2 G7M"}=" O2P G7M"; $resn_exch{" O5' G7M"}=" O5* G7M"; $resn_exch{" C5' G7M"}=" C5* G7M";
$resn_exch{" C4' G7M"}=" C4* G7M"; $resn_exch{" O4' G7M"}=" O4* G7M"; $resn_exch{" C3' G7M"}=" C3* G7M";
$resn_exch{" O3' G7M"}=" O3* G7M"; $resn_exch{" C2' G7M"}=" C2* G7M"; $resn_exch{" O2' G7M"}=" O2* G7M";
$resn_exch{" C1' G7M"}=" C1* G7M"; $resn_exch{" C4' GA2"}=" C4* GA2"; $resn_exch{" O4' GA2"}=" O4* GA2";
$resn_exch{" C3' GA2"}=" C3* GA2"; $resn_exch{" O3' GA2"}=" O3* GA2"; $resn_exch{" C2' GA2"}=" C2* GA2";
$resn_exch{" O1' GA2"}=" O1* GA2"; $resn_exch{" C1' GA2"}=" C1* GA2"; $resn_exch{" OP3 GAO"}=" O3P GAO";
$resn_exch{" OP1 GAO"}=" O1P GAO"; $resn_exch{" OP2 GAO"}=" O2P GAO"; $resn_exch{" O5' GAO"}=" O5* GAO";
$resn_exch{" C5' GAO"}=" C5* GAO"; $resn_exch{" C4' GAO"}=" C4* GAO"; $resn_exch{" O4' GAO"}=" O4* GAO";
$resn_exch{" C3' GAO"}=" C3* GAO"; $resn_exch{" O3' GAO"}=" O3* GAO"; $resn_exch{" C2' GAO"}=" C2* GAO";
$resn_exch{" O2' GAO"}=" O2* GAO"; $resn_exch{" C1' GAO"}=" C1* GAO"; $resn_exch{" C5' GAP"}=" C5* GAP";
$resn_exch{" O5' GAP"}=" O5* GAP"; $resn_exch{" C4' GAP"}=" C4* GAP"; $resn_exch{" O4' GAP"}=" O4* GAP";
$resn_exch{" C3' GAP"}=" C3* GAP"; $resn_exch{" O3' GAP"}=" O3* GAP"; $resn_exch{" C2' GAP"}=" C2* GAP";
$resn_exch{" O2' GAP"}=" O2* GAP"; $resn_exch{" C1' GAP"}=" C1* GAP"; $resn_exch{" C1' GAV"}=" C1* GAV";
$resn_exch{" C2' GAV"}=" C2* GAV"; $resn_exch{" O2' GAV"}=" O2* GAV"; $resn_exch{" C3' GAV"}=" C3* GAV";
$resn_exch{" O3' GAV"}=" O3* GAV"; $resn_exch{" O4' GAV"}=" O4* GAV"; $resn_exch{" C4' GAV"}=" C4* GAV";
$resn_exch{" C5' GAV"}=" C5* GAV"; $resn_exch{" O5' GAV"}=" O5* GAV"; $resn_exch{" N11 GCG"}="N10S GCG";
$resn_exch{" O5' GCK"}=" O5* GCK"; $resn_exch{" C2' GCK"}=" C2* GCK"; $resn_exch{" C5' GCK"}=" C5* GCK";
$resn_exch{" C4' GCK"}=" C4* GCK"; $resn_exch{" O4' GCK"}=" O4* GCK"; $resn_exch{" C1' GCK"}=" C1* GCK";
$resn_exch{" C3' GCK"}=" C3* GCK"; $resn_exch{" O3' GCK"}=" O3* GCK"; $resn_exch{" O5' GCP"}=" O5* GCP";
$resn_exch{" C5' GCP"}=" C5* GCP"; $resn_exch{" C4' GCP"}=" C4* GCP"; $resn_exch{" O4' GCP"}=" O4* GCP";
$resn_exch{" C3' GCP"}=" C3* GCP"; $resn_exch{" O3' GCP"}=" O3* GCP"; $resn_exch{" C2' GCP"}=" C2* GCP";
$resn_exch{" O2' GCP"}=" O2* GCP"; $resn_exch{" C1' GCP"}=" C1* GCP"; $resn_exch{" O5' GCQ"}=" O5* GCQ";
$resn_exch{" C5' GCQ"}=" C5* GCQ"; $resn_exch{" C4' GCQ"}=" C4* GCQ"; $resn_exch{" C3' GCQ"}=" C3* GCQ";
$resn_exch{" O3' GCQ"}=" O3* GCQ"; $resn_exch{" C2' GCQ"}=" C2* GCQ"; $resn_exch{" O4' GCQ"}=" O4* GCQ";
$resn_exch{" C1' GCQ"}=" C1* GCQ"; $resn_exch{" O5' GDC"}=" O5* GDC"; $resn_exch{" C5' GDC"}=" C5* GDC";
$resn_exch{" C4' GDC"}=" C4* GDC"; $resn_exch{" O4' GDC"}=" O4* GDC"; $resn_exch{" C3' GDC"}=" C3* GDC";
$resn_exch{" O3' GDC"}=" O3* GDC"; $resn_exch{" C2' GDC"}=" C2* GDC"; $resn_exch{" O2' GDC"}=" O2* GDC";
$resn_exch{" C1' GDC"}=" C1* GDC"; $resn_exch{" O1X GDC"}="AO1P GDC"; $resn_exch{" O2X GDC"}="AO2P GDC";
$resn_exch{" C1' GDD"}=" C1* GDD"; $resn_exch{" C2' GDD"}=" C2* GDD"; $resn_exch{" O2' GDD"}=" O2* GDD";
$resn_exch{" C3' GDD"}=" C3* GDD"; $resn_exch{" O3' GDD"}=" O3* GDD"; $resn_exch{" C4' GDD"}=" C4* GDD";
$resn_exch{" O4' GDD"}=" O4* GDD"; $resn_exch{" C5' GDD"}=" C5* GDD"; $resn_exch{" O5' GDD"}=" O5* GDD";
$resn_exch{" O5' GDP"}=" O5* GDP"; $resn_exch{" C5' GDP"}=" C5* GDP"; $resn_exch{" C4' GDP"}=" C4* GDP";
$resn_exch{" O4' GDP"}=" O4* GDP"; $resn_exch{" C3' GDP"}=" C3* GDP"; $resn_exch{" O3' GDP"}=" O3* GDP";
$resn_exch{" C2' GDP"}=" C2* GDP"; $resn_exch{" O2' GDP"}=" O2* GDP"; $resn_exch{" C1' GDP"}=" C1* GDP";
$resn_exch{" O5D GDR"}=" O5* GDR"; $resn_exch{" C5D GDR"}=" C5* GDR"; $resn_exch{" C4D GDR"}=" C4* GDR";
$resn_exch{" O4D GDR"}=" O4* GDR"; $resn_exch{" C1D GDR"}=" C1* GDR"; $resn_exch{" C2D GDR"}=" C2* GDR";
$resn_exch{" O2D GDR"}=" O2* GDR"; $resn_exch{" C3D GDR"}=" C3* GDR"; $resn_exch{" O3D GDR"}=" O3* GDR";
$resn_exch{" C1D GDU"}=" C1* GDU"; $resn_exch{" C2D GDU"}=" C2* GDU"; $resn_exch{" O2D GDU"}=" O2* GDU";
$resn_exch{" C3D GDU"}=" C3* GDU"; $resn_exch{" C4D GDU"}=" C4* GDU"; $resn_exch{" O4D GDU"}=" O4* GDU";
$resn_exch{" O3D GDU"}=" O3* GDU"; $resn_exch{" C5D GDU"}=" C5* GDU"; $resn_exch{" O5D GDU"}=" O5* GDU";
$resn_exch{" C1D GDX"}=" C1* GDX"; $resn_exch{" C2D GDX"}=" C2* GDX"; $resn_exch{" O2D GDX"}=" O2* GDX";
$resn_exch{" C3D GDX"}=" C3* GDX"; $resn_exch{" O3D GDX"}=" O3* GDX"; $resn_exch{" C4D GDX"}=" C4* GDX";
$resn_exch{" O4D GDX"}=" O4* GDX"; $resn_exch{" C5D GDX"}=" C5* GDX"; $resn_exch{" O5D GDX"}=" O5* GDX";
$resn_exch{" O5' GFL"}=" O5* GFL"; $resn_exch{" C5' GFL"}=" C5* GFL"; $resn_exch{" C4' GFL"}=" C4* GFL";
$resn_exch{" O4' GFL"}=" O4* GFL"; $resn_exch{" C3' GFL"}=" C3* GFL"; $resn_exch{" O3' GFL"}=" O3* GFL";
$resn_exch{" C2' GFL"}=" C2* GFL"; $resn_exch{" C1' GFL"}=" C1* GFL"; $resn_exch{" O2' GGZ"}=" O2* GGZ";
$resn_exch{" O3' GGZ"}=" O3* GGZ"; $resn_exch{" C3' GGZ"}=" C3* GGZ"; $resn_exch{" C4' GGZ"}=" C4* GGZ";
$resn_exch{" C5' GGZ"}=" C5* GGZ"; $resn_exch{" O5' GGZ"}=" O5* GGZ"; $resn_exch{" C2' GGZ"}=" C2* GGZ";
$resn_exch{" C1' GGZ"}=" C1* GGZ"; $resn_exch{" O4' GGZ"}=" O4* GGZ"; $resn_exch{" O5' GH3"}=" O5* GH3";
$resn_exch{" C5' GH3"}=" C5* GH3"; $resn_exch{" C4' GH3"}=" C4* GH3"; $resn_exch{" O4' GH3"}=" O4* GH3";
$resn_exch{" C3' GH3"}=" C3* GH3"; $resn_exch{" C2' GH3"}=" C2* GH3"; $resn_exch{" O2' GH3"}=" O2* GH3";
$resn_exch{" C1' GH3"}=" C1* GH3"; $resn_exch{" C1' GKD"}=" C1* GKD"; $resn_exch{" C2' GKD"}=" C2* GKD";
$resn_exch{" O2' GKD"}=" O2* GKD"; $resn_exch{" C3' GKD"}=" C3* GKD"; $resn_exch{" O3' GKD"}=" O3* GKD";
$resn_exch{" C4' GKD"}=" C4* GKD"; $resn_exch{" O4' GKD"}=" O4* GKD"; $resn_exch{" C5' GKD"}=" C5* GKD";
$resn_exch{" O5' GKD"}=" O5* GKD"; $resn_exch{" C1' GKE"}=" C1* GKE"; $resn_exch{" C2' GKE"}=" C2* GKE";
$resn_exch{" O2' GKE"}=" O2* GKE"; $resn_exch{" C3' GKE"}=" C3* GKE"; $resn_exch{" O3' GKE"}=" O3* GKE";
$resn_exch{" C4' GKE"}=" C4* GKE"; $resn_exch{" O4' GKE"}=" O4* GKE"; $resn_exch{" C5' GKE"}=" C5* GKE";
$resn_exch{" O5' GKE"}=" O5* GKE"; $resn_exch{" CDV GLI"}="CD31 GLI"; $resn_exch{" CDW GLI"}="CD32 GLI";
$resn_exch{" CEV GLI"}="CE31 GLI"; $resn_exch{" CEW GLI"}="CE32 GLI"; $resn_exch{" CGF GLI"}="CG51 GLI";
$resn_exch{" CGG GLI"}="CG52 GLI"; $resn_exch{" CGP GLI"}="CG61 GLI"; $resn_exch{" CGQ GLI"}="CG62 GLI";
$resn_exch{" XE1 GLX"}=" AE1 GLX"; $resn_exch{" XE2 GLX"}=" AE2 GLX"; $resn_exch{" CRA GM2"}="CR10 GM2";
$resn_exch{" CRB GM2"}="CR11 GM2"; $resn_exch{" CRC GM2"}="CR12 GM2"; $resn_exch{" CRD GM2"}="CR13 GM2";
$resn_exch{" CRE GM2"}="CR14 GM2"; $resn_exch{" CRF GM2"}="CR15 GM2"; $resn_exch{" CRG GM2"}="CR16 GM2";
$resn_exch{" CRH GM2"}="CR17 GM2"; $resn_exch{" CLA GM2"}="CL10 GM2"; $resn_exch{" CLB GM2"}="CL11 GM2";
$resn_exch{" CLC GM2"}="CL12 GM2"; $resn_exch{" CLD GM2"}="CL13 GM2"; $resn_exch{" CLE GM2"}="CL14 GM2";
$resn_exch{" CLF GM2"}="CL15 GM2"; $resn_exch{" CLG GM2"}="CL16 GM2"; $resn_exch{" CLH GM2"}="CL17 GM2";
$resn_exch{" CLI GM2"}="CL18 GM2"; $resn_exch{" C1' GMC"}=" C1* GMC"; $resn_exch{" O4' GMC"}=" O4* GMC";
$resn_exch{" C4' GMC"}=" C4* GMC"; $resn_exch{" C5' GMC"}=" C5* GMC"; $resn_exch{" O5' GMC"}=" O5* GMC";
$resn_exch{" C2' GMC"}=" C2* GMC"; $resn_exch{" O2' GMC"}=" O2* GMC"; $resn_exch{" C3' GMC"}=" C3* GMC";
$resn_exch{" N3' GMC"}=" N3* GMC"; $resn_exch{" O5' GMP"}=" O5* GMP"; $resn_exch{" C5' GMP"}=" C5* GMP";
$resn_exch{" C4' GMP"}=" C4* GMP"; $resn_exch{" O4' GMP"}=" O4* GMP"; $resn_exch{" C3' GMP"}=" C3* GMP";
$resn_exch{" O3' GMP"}=" O3* GMP"; $resn_exch{" C2' GMP"}=" C2* GMP"; $resn_exch{" O2' GMP"}=" O2* GMP";
$resn_exch{" C1' GMP"}=" C1* GMP"; $resn_exch{" OP1 GMS"}=" O1P GMS"; $resn_exch{" O5' GMS"}=" O5* GMS";
$resn_exch{" C2' GMS"}=" C2* GMS"; $resn_exch{" C5' GMS"}=" C5* GMS"; $resn_exch{" C4' GMS"}=" C4* GMS";
$resn_exch{" O4' GMS"}=" O4* GMS"; $resn_exch{" C1' GMS"}=" C1* GMS"; $resn_exch{" C3' GMS"}=" C3* GMS";
$resn_exch{" O3' GMS"}=" O3* GMS"; $resn_exch{" OP2 GMS"}=" O2P GMS"; $resn_exch{" OP3 GMU"}=" O3P GMU";
$resn_exch{" OP2 GMU"}=" O2P GMU"; $resn_exch{" O5' GMU"}=" O5* GMU"; $resn_exch{" C5' GMU"}=" C5* GMU";
$resn_exch{" C4' GMU"}=" C4* GMU"; $resn_exch{" O4' GMU"}=" O4* GMU"; $resn_exch{" C3' GMU"}=" C3* GMU";
$resn_exch{" O3' GMU"}=" O3* GMU"; $resn_exch{" C2' GMU"}=" C2* GMU"; $resn_exch{" C1' GMU"}=" C1* GMU";
$resn_exch{" O2' GMU"}=" O2* GMU"; $resn_exch{" CA' GMU"}=" CA* GMU"; $resn_exch{" CB' GMU"}=" CB* GMU";
$resn_exch{" NC' GMU"}=" NC* GMU"; $resn_exch{" CD' GMU"}=" CD* GMU"; $resn_exch{" NE' GMU"}=" NE* GMU";
$resn_exch{" NF' GMU"}=" NF* GMU"; $resn_exch{" OP1 GMU"}=" O1P GMU"; $resn_exch{" OP1 GN7"}=" O1P GN7";
$resn_exch{" OP2 GN7"}=" O2P GN7"; $resn_exch{" OP3 GN7"}=" O3P GN7"; $resn_exch{" O5' GN7"}=" O5* GN7";
$resn_exch{" C5' GN7"}=" C5* GN7"; $resn_exch{" C4' GN7"}=" C4* GN7"; $resn_exch{" O4' GN7"}=" O4* GN7";
$resn_exch{" C3' GN7"}=" C3* GN7"; $resn_exch{" O3' GN7"}=" O3* GN7"; $resn_exch{" C2' GN7"}=" C2* GN7";
$resn_exch{" C1' GN7"}=" C1* GN7"; $resn_exch{" OP1 GNE"}=" O1P GNE"; $resn_exch{" OP2 GNE"}=" O2P GNE";
$resn_exch{" OP3 GNE"}=" O3P GNE"; $resn_exch{" O5' GNE"}=" O5* GNE"; $resn_exch{" C5' GNE"}=" C5* GNE";
$resn_exch{" C4' GNE"}=" C4* GNE"; $resn_exch{" O4' GNE"}=" O4* GNE"; $resn_exch{" C3' GNE"}=" C3* GNE";
$resn_exch{" O3' GNE"}=" O3* GNE"; $resn_exch{" C2' GNE"}=" C2* GNE"; $resn_exch{" C1' GNE"}=" C1* GNE";
$resn_exch{" O5' GNG"}=" O5* GNG"; $resn_exch{" C5' GNG"}=" C5* GNG"; $resn_exch{" C4' GNG"}=" C4* GNG";
$resn_exch{" O4' GNG"}=" O4* GNG"; $resn_exch{" C1' GNG"}=" C1* GNG"; $resn_exch{" C2' GNG"}=" C2* GNG";
$resn_exch{" C3' GNG"}=" C3* GNG"; $resn_exch{" O3' GNG"}=" O3* GNG"; $resn_exch{" O5' GNH"}=" O5* GNH";
$resn_exch{" C5' GNH"}=" C5* GNH"; $resn_exch{" C4' GNH"}=" C4* GNH"; $resn_exch{" O4' GNH"}=" O4* GNH";
$resn_exch{" C3' GNH"}=" C3* GNH"; $resn_exch{" O3' GNH"}=" O3* GNH"; $resn_exch{" C2' GNH"}=" C2* GNH";
$resn_exch{" O2' GNH"}=" O2* GNH"; $resn_exch{" C1' GNH"}=" C1* GNH"; $resn_exch{" O5' GNP"}=" O5* GNP";
$resn_exch{" C5' GNP"}=" C5* GNP"; $resn_exch{" C4' GNP"}=" C4* GNP"; $resn_exch{" O4' GNP"}=" O4* GNP";
$resn_exch{" C3' GNP"}=" C3* GNP"; $resn_exch{" O3' GNP"}=" O3* GNP"; $resn_exch{" C2' GNP"}=" C2* GNP";
$resn_exch{" O2' GNP"}=" O2* GNP"; $resn_exch{" C1' GNP"}=" C1* GNP"; $resn_exch{" OP1 GOM"}=" O1P GOM";
$resn_exch{" OP2 GOM"}=" O2P GOM"; $resn_exch{" OP3 GOM"}=" O3P GOM"; $resn_exch{" O5' GOM"}=" O5* GOM";
$resn_exch{" C5' GOM"}=" C5* GOM"; $resn_exch{" C4' GOM"}=" C4* GOM"; $resn_exch{" O4' GOM"}=" O4* GOM";
$resn_exch{" C1' GOM"}=" C1* GOM"; $resn_exch{" C2' GOM"}=" C2* GOM"; $resn_exch{" O2' GOM"}=" O2* GOM";
$resn_exch{" C3' GOM"}=" C3* GOM"; $resn_exch{" O3' GOM"}=" O3* GOM"; $resn_exch{" C1' GP2"}=" C1* GP2";
$resn_exch{" C2' GP2"}=" C2* GP2"; $resn_exch{" O2' GP2"}=" O2* GP2"; $resn_exch{" C3' GP2"}=" C3* GP2";
$resn_exch{" O3' GP2"}=" O3* GP2"; $resn_exch{" O4' GP2"}=" O4* GP2"; $resn_exch{" C4' GP2"}=" C4* GP2";
$resn_exch{" C5' GP2"}=" C5* GP2"; $resn_exch{" O5' GP2"}=" O5* GP2"; $resn_exch{" N9A GP3"}=" AN9 GP3";
$resn_exch{" C8A GP3"}=" AC8 GP3"; $resn_exch{" N7A GP3"}=" AN7 GP3"; $resn_exch{" C5A GP3"}=" AC5 GP3";
$resn_exch{" C6A GP3"}=" AC6 GP3"; $resn_exch{" O6A GP3"}=" AO6 GP3"; $resn_exch{" N1A GP3"}=" AN1 GP3";
$resn_exch{" C2A GP3"}=" AC2 GP3"; $resn_exch{" N2A GP3"}=" AN2 GP3"; $resn_exch{" N3A GP3"}=" AN3 GP3";
$resn_exch{" C4A GP3"}=" AC4 GP3"; $resn_exch{" O5D GP3"}="AO5* GP3"; $resn_exch{" C5D GP3"}="AC5* GP3";
$resn_exch{" C4D GP3"}="AC4* GP3"; $resn_exch{" O4D GP3"}="AO4* GP3"; $resn_exch{" C3D GP3"}="AC3* GP3";
$resn_exch{" O3D GP3"}="AO3* GP3"; $resn_exch{" C2D GP3"}="AC2* GP3"; $resn_exch{" O2D GP3"}="AO2* GP3";
$resn_exch{" C1D GP3"}="AC1* GP3"; $resn_exch{" O5E GP3"}="BO5* GP3"; $resn_exch{" C5E GP3"}="BC5* GP3";
$resn_exch{" C4E GP3"}="BC4* GP3"; $resn_exch{" O4E GP3"}="BO4* GP3"; $resn_exch{" C3E GP3"}="BC3* GP3";
$resn_exch{" O3E GP3"}="BO3* GP3"; $resn_exch{" C2E GP3"}="BC2* GP3"; $resn_exch{" O2E GP3"}="BO2* GP3";
$resn_exch{" C1E GP3"}="BC1* GP3"; $resn_exch{" N9B GP3"}=" BN9 GP3"; $resn_exch{" C8B GP3"}=" BC8 GP3";
$resn_exch{" N7B GP3"}=" BN7 GP3"; $resn_exch{" C5B GP3"}=" BC5 GP3"; $resn_exch{" C6B GP3"}=" BC6 GP3";
$resn_exch{" O6B GP3"}=" BO6 GP3"; $resn_exch{" N1B GP3"}=" BN1 GP3"; $resn_exch{" C2B GP3"}=" BC2 GP3";
$resn_exch{" N2B GP3"}=" BN2 GP3"; $resn_exch{" N3B GP3"}=" BN3 GP3"; $resn_exch{" C4B GP3"}=" BC4 GP3";
$resn_exch{" C1' GP5"}=" C1* GP5"; $resn_exch{" O4' GP5"}=" O4* GP5"; $resn_exch{" C2' GP5"}=" C2* GP5";
$resn_exch{" O2' GP5"}=" O2* GP5"; $resn_exch{" C3' GP5"}=" C3* GP5"; $resn_exch{" O3' GP5"}=" O3* GP5";
$resn_exch{" C4' GP5"}=" C4* GP5"; $resn_exch{" C5' GP5"}=" C5* GP5"; $resn_exch{" O5' GP5"}=" O5* GP5";
$resn_exch{" N1G GPC"}=" GN1 GPC"; $resn_exch{" C2G GPC"}=" GC2 GPC"; $resn_exch{" N2G GPC"}=" GN2 GPC";
$resn_exch{" N3G GPC"}=" GN3 GPC"; $resn_exch{" C4G GPC"}=" GC4 GPC"; $resn_exch{" C5G GPC"}=" GC5 GPC";
$resn_exch{" C6G GPC"}=" GC6 GPC"; $resn_exch{" O6G GPC"}=" GO6 GPC"; $resn_exch{" N7G GPC"}=" GN7 GPC";
$resn_exch{" C8G GPC"}=" GC8 GPC"; $resn_exch{" N9G GPC"}=" GN9 GPC"; $resn_exch{" C1D GPC"}="GC1* GPC";
$resn_exch{" C2D GPC"}="GC2* GPC"; $resn_exch{" F2D GPC"}="GF2* GPC"; $resn_exch{" C3D GPC"}="GC3* GPC";
$resn_exch{" O3D GPC"}="GO3* GPC"; $resn_exch{" C4D GPC"}="GC4* GPC"; $resn_exch{" O4D GPC"}="GO4* GPC";
$resn_exch{" C5D GPC"}="GC5* GPC"; $resn_exch{" O5D GPC"}="GO5* GPC"; $resn_exch{" PC  GPC"}=" CP  GPC";
$resn_exch{" O1C GPC"}=" CO1 GPC"; $resn_exch{" O2C GPC"}=" CO2 GPC"; $resn_exch{" O5B GPC"}="CO5* GPC";
$resn_exch{" C5X GPC"}="CC5* GPC"; $resn_exch{" C4X GPC"}="CC4* GPC"; $resn_exch{" O4B GPC"}="CO4* GPC";
$resn_exch{" C3X GPC"}="CC3* GPC"; $resn_exch{" O3B GPC"}="CO3* GPC"; $resn_exch{" C2X GPC"}="CC2* GPC";
$resn_exch{" O2B GPC"}="CO2* GPC"; $resn_exch{" C1X GPC"}="CC1* GPC"; $resn_exch{" N1C GPC"}=" CN1 GPC";
$resn_exch{" O2X GPC"}="CO2' GPC"; $resn_exch{" N3C GPC"}=" CN3 GPC"; $resn_exch{" N4C GPC"}=" CN4 GPC";
$resn_exch{" N9A GPG"}=" AN9 GPG"; $resn_exch{" C8A GPG"}=" AC8 GPG"; $resn_exch{" N7A GPG"}=" AN7 GPG";
$resn_exch{" C5A GPG"}=" AC5 GPG"; $resn_exch{" C6A GPG"}=" AC6 GPG"; $resn_exch{" O6A GPG"}=" AO6 GPG";
$resn_exch{" N1A GPG"}=" AN1 GPG"; $resn_exch{" C2A GPG"}=" AC2 GPG"; $resn_exch{" N2A GPG"}=" AN2 GPG";
$resn_exch{" N3A GPG"}=" AN3 GPG"; $resn_exch{" C4A GPG"}=" AC4 GPG"; $resn_exch{" O5D GPG"}="AO5* GPG";
$resn_exch{" C5D GPG"}="AC5* GPG"; $resn_exch{" C4D GPG"}="AC4* GPG"; $resn_exch{" O4D GPG"}="AO4* GPG";
$resn_exch{" C3D GPG"}="AC3* GPG"; $resn_exch{" O3D GPG"}="AO3* GPG"; $resn_exch{" C2D GPG"}="AC2* GPG";
$resn_exch{" O2D GPG"}="AO2* GPG"; $resn_exch{" C1D GPG"}="AC1* GPG"; $resn_exch{" O5E GPG"}="BO5* GPG";
$resn_exch{" C5E GPG"}="BC5* GPG"; $resn_exch{" C4E GPG"}="BC4* GPG"; $resn_exch{" O4E GPG"}="BO4* GPG";
$resn_exch{" C3E GPG"}="BC3* GPG"; $resn_exch{" O3E GPG"}="BO3* GPG"; $resn_exch{" C2E GPG"}="BC2* GPG";
$resn_exch{" O2E GPG"}="BO2* GPG"; $resn_exch{" C1E GPG"}="BC1* GPG"; $resn_exch{" N9B GPG"}=" BN9 GPG";
$resn_exch{" C8B GPG"}=" BC8 GPG"; $resn_exch{" N7B GPG"}=" BN7 GPG"; $resn_exch{" C5B GPG"}=" BC5 GPG";
$resn_exch{" C6B GPG"}=" BC6 GPG"; $resn_exch{" O6B GPG"}=" BO6 GPG"; $resn_exch{" N1B GPG"}=" BN1 GPG";
$resn_exch{" C2B GPG"}=" BC2 GPG"; $resn_exch{" N2B GPG"}=" BN2 GPG"; $resn_exch{" N3B GPG"}=" BN3 GPG";
$resn_exch{" C4B GPG"}=" BC4 GPG"; $resn_exch{" O5' GPL"}=" O5* GPL"; $resn_exch{" C5' GPL"}=" C5* GPL";
$resn_exch{" C4' GPL"}=" C4* GPL"; $resn_exch{" O4' GPL"}=" O4* GPL"; $resn_exch{" C3' GPL"}=" C3* GPL";
$resn_exch{" O3' GPL"}=" O3* GPL"; $resn_exch{" C2' GPL"}=" C2* GPL"; $resn_exch{" O2' GPL"}=" O2* GPL";
$resn_exch{" C1' GPL"}=" C1* GPL"; $resn_exch{" OP1  GS"}=" O1P  GS"; $resn_exch{" OP3  GS"}=" O3P  GS";
$resn_exch{" O5'  GS"}=" O5*  GS"; $resn_exch{" C5'  GS"}=" C5*  GS"; $resn_exch{" C4'  GS"}=" C4*  GS";
$resn_exch{" O4'  GS"}=" O4*  GS"; $resn_exch{" C3'  GS"}=" C3*  GS"; $resn_exch{" O3'  GS"}=" O3*  GS";
$resn_exch{" C2'  GS"}=" C2*  GS"; $resn_exch{" C1'  GS"}=" C1*  GS"; $resn_exch{" O5' GSP"}=" O5* GSP";
$resn_exch{" C5' GSP"}=" C5* GSP"; $resn_exch{" C4' GSP"}=" C4* GSP"; $resn_exch{" O4' GSP"}=" O4* GSP";
$resn_exch{" C3' GSP"}=" C3* GSP"; $resn_exch{" O3' GSP"}=" O3* GSP"; $resn_exch{" C2' GSP"}=" C2* GSP";
$resn_exch{" O2' GSP"}=" O2* GSP"; $resn_exch{" C1' GSP"}=" C1* GSP"; $resn_exch{" OP1 GSR"}=" O1P GSR";
$resn_exch{" OP2 GSR"}=" O2P GSR"; $resn_exch{" OP3 GSR"}=" O3P GSR"; $resn_exch{" O5' GSR"}=" O5* GSR";
$resn_exch{" C5' GSR"}=" C5* GSR"; $resn_exch{" C4' GSR"}=" C4* GSR"; $resn_exch{" O4' GSR"}=" O4* GSR";
$resn_exch{" C3' GSR"}=" C3* GSR"; $resn_exch{" O3' GSR"}=" O3* GSR"; $resn_exch{" C2' GSR"}=" C2* GSR";
$resn_exch{" C1' GSR"}=" C1* GSR"; $resn_exch{" OP1 GSS"}=" O1P GSS"; $resn_exch{" OP2 GSS"}=" O2P GSS";
$resn_exch{" OP3 GSS"}=" O3P GSS"; $resn_exch{" O5' GSS"}=" O5* GSS"; $resn_exch{" C5' GSS"}=" C5* GSS";
$resn_exch{" C4' GSS"}=" C4* GSS"; $resn_exch{" O4' GSS"}=" O4* GSS"; $resn_exch{" C3' GSS"}=" C3* GSS";
$resn_exch{" O3' GSS"}=" O3* GSS"; $resn_exch{" C2' GSS"}=" C2* GSS"; $resn_exch{" C1' GSS"}=" C1* GSS";
$resn_exch{" O5' GSU"}=" O5* GSU"; $resn_exch{" C5' GSU"}=" C5* GSU"; $resn_exch{" C4' GSU"}=" C4* GSU";
$resn_exch{" O4' GSU"}=" O4* GSU"; $resn_exch{" C1' GSU"}=" C1* GSU"; $resn_exch{" C2' GSU"}=" C2* GSU";
$resn_exch{" O2' GSU"}=" O2* GSU"; $resn_exch{" C3' GSU"}=" C3* GSU"; $resn_exch{" O3' GSU"}=" O3* GSU";
$resn_exch{" C7X GTG"}="C7MA GTG"; $resn_exch{" O5D GTG"}="O5*A GTG"; $resn_exch{" C5D GTG"}="C5*A GTG";
$resn_exch{" C4D GTG"}="C4*A GTG"; $resn_exch{" O4D GTG"}="O4*A GTG"; $resn_exch{" C3D GTG"}="C3*A GTG";
$resn_exch{" O3D GTG"}="O3*A GTG"; $resn_exch{" C2D GTG"}="C2*A GTG"; $resn_exch{" O2D GTG"}="O2*A GTG";
$resn_exch{" C1D GTG"}="C1*A GTG"; $resn_exch{" O5E GTG"}="O5*B GTG"; $resn_exch{" C5E GTG"}="C5*B GTG";
$resn_exch{" C4E GTG"}="C4*B GTG"; $resn_exch{" O4E GTG"}="O4*B GTG"; $resn_exch{" C3E GTG"}="C3*B GTG";
$resn_exch{" O3E GTG"}="O3*B GTG"; $resn_exch{" C2E GTG"}="C2*B GTG"; $resn_exch{" O2E GTG"}="O2*B GTG";
$resn_exch{" C1E GTG"}="C1*B GTG"; $resn_exch{" O5' GTN"}=" O5* GTN"; $resn_exch{" C5' GTN"}=" C5* GTN";
$resn_exch{" C4' GTN"}=" C4* GTN"; $resn_exch{" O4' GTN"}=" O4* GTN"; $resn_exch{" C3' GTN"}=" C3* GTN";
$resn_exch{" O3' GTN"}=" O3* GTN"; $resn_exch{" C2' GTN"}=" C2* GTN"; $resn_exch{" O2' GTN"}=" O2* GTN";
$resn_exch{" C1' GTN"}=" C1* GTN"; $resn_exch{" O5' GTO"}=" O5* GTO"; $resn_exch{" C5' GTO"}=" C5* GTO";
$resn_exch{" C4' GTO"}=" C4* GTO"; $resn_exch{" O4' GTO"}=" O4* GTO"; $resn_exch{" C3' GTO"}=" C3* GTO";
$resn_exch{" O3' GTO"}=" O3* GTO"; $resn_exch{" C2' GTO"}=" C2* GTO"; $resn_exch{" O2' GTO"}=" O2* GTO";
$resn_exch{" C1' GTO"}=" C1* GTO"; $resn_exch{" O5' GTP"}=" O5* GTP"; $resn_exch{" C5' GTP"}=" C5* GTP";
$resn_exch{" C4' GTP"}=" C4* GTP"; $resn_exch{" O4' GTP"}=" O4* GTP"; $resn_exch{" C3' GTP"}=" C3* GTP";
$resn_exch{" O3' GTP"}=" O3* GTP"; $resn_exch{" C2' GTP"}=" C2* GTP"; $resn_exch{" O2' GTP"}=" O2* GTP";
$resn_exch{" C1' GTP"}=" C1* GTP"; $resn_exch{" C1D GUD"}=" C1* GUD"; $resn_exch{" C2D GUD"}=" C2* GUD";
$resn_exch{" O2D GUD"}=" O2* GUD"; $resn_exch{" C3D GUD"}=" C3* GUD"; $resn_exch{" C4D GUD"}=" C4* GUD";
$resn_exch{" O4D GUD"}=" O4* GUD"; $resn_exch{" O3D GUD"}=" O3* GUD"; $resn_exch{" C5D GUD"}=" C5* GUD";
$resn_exch{" O5D GUD"}=" O5* GUD"; $resn_exch{" NA  H02"}=" N_A H02"; $resn_exch{" NB  H02"}=" N_B H02";
$resn_exch{" NC  H02"}=" N_C H02"; $resn_exch{" ND  H02"}=" N_D H02"; $resn_exch{" OP1 H2U"}=" O1P H2U";
$resn_exch{" OP2 H2U"}=" O2P H2U"; $resn_exch{" OP3 H2U"}=" O3P H2U"; $resn_exch{" O5' H2U"}=" O5* H2U";
$resn_exch{" C5' H2U"}=" C5* H2U"; $resn_exch{" C4' H2U"}=" C4* H2U"; $resn_exch{" O4' H2U"}=" O4* H2U";
$resn_exch{" C3' H2U"}=" C3* H2U"; $resn_exch{" O3' H2U"}=" O3* H2U"; $resn_exch{" C1' H2U"}=" C1* H2U";
$resn_exch{" C2' H2U"}=" C2* H2U"; $resn_exch{" O2' H2U"}=" O2* H2U"; $resn_exch{" O5' HAM"}=" O5* HAM";
$resn_exch{" C5' HAM"}=" C5* HAM"; $resn_exch{" C4' HAM"}=" C4* HAM"; $resn_exch{" O4' HAM"}=" O4* HAM";
$resn_exch{" C3' HAM"}=" C3* HAM"; $resn_exch{" O3' HAM"}=" O3* HAM"; $resn_exch{" C2' HAM"}=" C2* HAM";
$resn_exch{" O2' HAM"}=" O2* HAM"; $resn_exch{" C1' HAM"}=" C1* HAM"; $resn_exch{" NA  HAS"}=" N_A HAS";
$resn_exch{" NB  HAS"}=" N_B HAS"; $resn_exch{" NC  HAS"}=" N_C HAS"; $resn_exch{" ND  HAS"}=" N_D HAS";
$resn_exch{" N1A HAX"}=" AN1 HAX"; $resn_exch{" C2A HAX"}=" AC2 HAX"; $resn_exch{" N3A HAX"}=" AN3 HAX";
$resn_exch{" C4A HAX"}=" AC4 HAX"; $resn_exch{" C5A HAX"}=" AC5 HAX"; $resn_exch{" C6A HAX"}=" AC6 HAX";
$resn_exch{" N6A HAX"}=" AN6 HAX"; $resn_exch{" N7A HAX"}=" AN7 HAX"; $resn_exch{" C8A HAX"}=" AC8 HAX";
$resn_exch{" N9A HAX"}=" AN9 HAX"; $resn_exch{" C1B HAX"}="AC1* HAX"; $resn_exch{" C2B HAX"}="AC2* HAX";
$resn_exch{" O2B HAX"}="AO2* HAX"; $resn_exch{" C3B HAX"}="AC3* HAX"; $resn_exch{" O3B HAX"}="AO3* HAX";
$resn_exch{" P3B HAX"}="AP3* HAX"; $resn_exch{" O7A HAX"}=" AO7 HAX"; $resn_exch{" O8A HAX"}=" AO8 HAX";
$resn_exch{" O9A HAX"}=" AO9 HAX"; $resn_exch{" C4B HAX"}="AC4* HAX"; $resn_exch{" O4B HAX"}="AO4* HAX";
$resn_exch{" C5B HAX"}="AC5* HAX"; $resn_exch{" O5B HAX"}="AO5* HAX"; $resn_exch{" P1A HAX"}=" AP1 HAX";
$resn_exch{" O1A HAX"}=" AO1 HAX"; $resn_exch{" O2A HAX"}=" AO2 HAX"; $resn_exch{" O3A HAX"}=" AO3 HAX";
$resn_exch{" P2A HAX"}=" AP2 HAX"; $resn_exch{" O4A HAX"}=" AO4 HAX"; $resn_exch{" O5A HAX"}=" AO5 HAX";
$resn_exch{" O6A HAX"}=" AO6 HAX"; $resn_exch{" CBP HAX"}="PC11 HAX"; $resn_exch{" CCP HAX"}="PC12 HAX";
$resn_exch{" CDP HAX"}="PC13 HAX"; $resn_exch{" CEP HAX"}="PC14 HAX"; $resn_exch{" CAP HAX"}="PC10 HAX";
$resn_exch{" OAP HAX"}="PO10 HAX"; $resn_exch{" C9P HAX"}=" PC9 HAX"; $resn_exch{" O9P HAX"}=" PO9 HAX";
$resn_exch{" N8P HAX"}=" PN8 HAX"; $resn_exch{" C7P HAX"}=" PC7 HAX"; $resn_exch{" C6P HAX"}=" PC6 HAX";
$resn_exch{" C5P HAX"}=" PC5 HAX"; $resn_exch{" O5P HAX"}=" PO5 HAX"; $resn_exch{" N4P HAX"}=" PN4 HAX";
$resn_exch{" C3P HAX"}=" PC3 HAX"; $resn_exch{" C2P HAX"}=" PC2 HAX"; $resn_exch{" NA  HCO"}=" N_A HCO";
$resn_exch{" NB  HCO"}=" N_B HCO"; $resn_exch{" NC  HCO"}=" N_C HCO"; $resn_exch{" ND  HCO"}=" N_D HCO";
$resn_exch{" OXT HCS"}=" OT1 HCS"; $resn_exch{" O   HCS"}=" OXT HCS"; $resn_exch{" O5' HCX"}=" O5* HCX";
$resn_exch{" C2' HCX"}=" C2* HCX"; $resn_exch{" C5' HCX"}=" C5* HCX"; $resn_exch{" C4' HCX"}=" C4* HCX";
$resn_exch{" C1' HCX"}=" C1* HCX"; $resn_exch{" C3' HCX"}=" C3* HCX"; $resn_exch{" O3' HCX"}=" O3* HCX";
$resn_exch{" OAP HDC"}="PO10 HDC"; $resn_exch{" CAP HDC"}="PC10 HDC"; $resn_exch{" C9P HDC"}=" PC9 HDC";
$resn_exch{" O9P HDC"}=" PO9 HDC"; $resn_exch{" N8P HDC"}=" PN8 HDC"; $resn_exch{" C7P HDC"}=" PC7 HDC";
$resn_exch{" C6P HDC"}=" PC6 HDC"; $resn_exch{" C5P HDC"}=" PC5 HDC"; $resn_exch{" O5P HDC"}=" PO5 HDC";
$resn_exch{" N4P HDC"}=" PN4 HDC"; $resn_exch{" C3P HDC"}=" PC3 HDC"; $resn_exch{" C2P HDC"}=" PC2 HDC";
$resn_exch{" S1P HDC"}=" PS1 HDC"; $resn_exch{" C1' HDC"}=" C1* HDC"; $resn_exch{" O1' HDC"}=" O1* HDC";
$resn_exch{" C2' HDC"}=" C2* HDC"; $resn_exch{" C3' HDC"}=" C3* HDC"; $resn_exch{" O2' HDC"}=" O2* HDC";
$resn_exch{" C4' HDC"}=" C4* HDC"; $resn_exch{" C5' HDC"}=" C5* HDC"; $resn_exch{" C6' HDC"}=" C6* HDC";
$resn_exch{" C7' HDC"}=" C7* HDC"; $resn_exch{" C8' HDC"}=" C8* HDC"; $resn_exch{" C9' HDC"}=" C9* HDC";
$resn_exch{" CBP HDC"}="PC11 HDC"; $resn_exch{" CDP HDC"}="PC13 HDC"; $resn_exch{" CEP HDC"}="PC14 HDC";
$resn_exch{" CCP HDC"}="PC12 HDC"; $resn_exch{" O6A HDC"}=" AO6 HDC"; $resn_exch{" P2A HDC"}=" AP2 HDC";
$resn_exch{" O4A HDC"}=" AO4 HDC"; $resn_exch{" O5A HDC"}=" AO5 HDC"; $resn_exch{" O3A HDC"}=" AO3 HDC";
$resn_exch{" P1A HDC"}=" AP1 HDC"; $resn_exch{" O1A HDC"}=" AO1 HDC"; $resn_exch{" O2A HDC"}=" AO2 HDC";
$resn_exch{" O5B HDC"}="AO5* HDC"; $resn_exch{" C5B HDC"}="AC5* HDC"; $resn_exch{" C4B HDC"}="AC4* HDC";
$resn_exch{" O4B HDC"}="AO4* HDC"; $resn_exch{" C1B HDC"}="AC1* HDC"; $resn_exch{" N9A HDC"}=" AN9 HDC";
$resn_exch{" C8A HDC"}=" AC8 HDC"; $resn_exch{" N7A HDC"}=" AN7 HDC"; $resn_exch{" C5A HDC"}=" AC5 HDC";
$resn_exch{" C4A HDC"}=" AC4 HDC"; $resn_exch{" N3A HDC"}=" AN3 HDC"; $resn_exch{" C2A HDC"}=" AC2 HDC";
$resn_exch{" N1A HDC"}=" AN1 HDC"; $resn_exch{" C6A HDC"}=" AC6 HDC"; $resn_exch{" N6A HDC"}=" AN6 HDC";
$resn_exch{" C2B HDC"}="AC2* HDC"; $resn_exch{" O2B HDC"}="AO2* HDC"; $resn_exch{" C3B HDC"}="AC3* HDC";
$resn_exch{" O3B HDC"}="AO3* HDC"; $resn_exch{" P3B HDC"}="AP3* HDC"; $resn_exch{" O9A HDC"}=" AO9 HDC";
$resn_exch{" O8A HDC"}=" AO8 HDC"; $resn_exch{" O7A HDC"}=" AO7 HDC"; $resn_exch{" NA  HDD"}=" N_A HDD";
$resn_exch{" NB  HDD"}=" N_B HDD"; $resn_exch{" NC  HDD"}=" N_C HDD"; $resn_exch{" ND  HDD"}=" N_D HDD";
$resn_exch{" C1' HDF"}=" C1* HDF"; $resn_exch{" C2' HDF"}=" C2* HDF"; $resn_exch{" O2' HDF"}=" O2* HDF";
$resn_exch{" C3' HDF"}=" C3* HDF"; $resn_exch{" O3' HDF"}=" O3* HDF"; $resn_exch{" C4' HDF"}=" C4* HDF";
$resn_exch{" O4' HDF"}=" O4* HDF"; $resn_exch{" C5' HDF"}=" C5* HDF"; $resn_exch{" O5' HDF"}=" O5* HDF";
$resn_exch{" NA  HDM"}=" N_A HDM"; $resn_exch{" NB  HDM"}=" N_B HDM"; $resn_exch{" NC  HDM"}=" N_C HDM";
$resn_exch{" ND  HDM"}=" N_D HDM"; $resn_exch{" OP1 HDP"}=" O1P HDP"; $resn_exch{" OP2 HDP"}=" O2P HDP";
$resn_exch{" O5' HDP"}=" O5* HDP"; $resn_exch{" C5B HDP"}=" C5* HDP"; $resn_exch{" O3' HDP"}=" O3* HDP";
$resn_exch{" OP3 HDP"}=" O3P HDP"; $resn_exch{" NA  HE6"}=" N_A HE6"; $resn_exch{" NB  HE6"}=" N_B HE6";
$resn_exch{" NC  HE6"}=" N_C HE6"; $resn_exch{" ND  HE6"}=" N_D HE6"; $resn_exch{" NA  HEA"}=" N_A HEA";
$resn_exch{" NB  HEA"}=" N_B HEA"; $resn_exch{" NC  HEA"}=" N_C HEA"; $resn_exch{" ND  HEA"}=" N_D HEA";
$resn_exch{" NA  HEB"}=" N_A HEB"; $resn_exch{" NB  HEB"}=" N_B HEB"; $resn_exch{" NC  HEB"}=" N_C HEB";
$resn_exch{" ND  HEB"}=" N_D HEB"; $resn_exch{" NA  HEC"}=" N_A HEC"; $resn_exch{" NB  HEC"}=" N_B HEC";
$resn_exch{" NC  HEC"}=" N_C HEC"; $resn_exch{" ND  HEC"}=" N_D HEC"; $resn_exch{" NA  HEG"}=" N_A HEG";
$resn_exch{" NB  HEG"}=" N_B HEG"; $resn_exch{" NC  HEG"}=" N_C HEG"; $resn_exch{" ND  HEG"}=" N_D HEG";
$resn_exch{" CCB HEH"}="C12B HEH"; $resn_exch{" O11 HEH"}="O11D HEH"; $resn_exch{" CBA HEH"}="C11A HEH";
$resn_exch{" CCA HEH"}="C12A HEH"; $resn_exch{" OCC HEH"}="O12C HEH"; $resn_exch{" C1X HEH"}="1CEM HEH";
$resn_exch{" C2X HEH"}="2CEG HEH"; $resn_exch{" C3X HEH"}="3CEG HEH"; $resn_exch{" C1A HEH"}="CFM7 HEH";
$resn_exch{" C5X HEH"}="5CFK HEH"; $resn_exch{" C1B HEH"}="6CKF HEH"; $resn_exch{" C1C HEH"}="CFM4 HEH";
$resn_exch{" NA  HEM"}=" N_A HEM"; $resn_exch{" NB  HEM"}=" N_B HEM"; $resn_exch{" NC  HEM"}=" N_C HEM";
$resn_exch{" ND  HEM"}=" N_D HEM"; $resn_exch{" NA  HEO"}=" N_A HEO"; $resn_exch{" NB  HEO"}=" N_B HEO";
$resn_exch{" NC  HEO"}=" N_C HEO"; $resn_exch{" ND  HEO"}=" N_D HEO"; $resn_exch{" NA  HES"}=" N_A HES";
$resn_exch{" NB  HES"}=" N_B HES"; $resn_exch{" NC  HES"}=" N_C HES"; $resn_exch{" ND  HES"}=" N_D HES";
$resn_exch{" OP2 HEU"}=" O2P HEU"; $resn_exch{" OP1 HEU"}=" O1P HEU"; $resn_exch{" O5' HEU"}=" O5* HEU";
$resn_exch{" C5' HEU"}=" C5* HEU"; $resn_exch{" C4' HEU"}=" C4* HEU"; $resn_exch{" O4' HEU"}=" O4* HEU";
$resn_exch{" C1' HEU"}=" C1* HEU"; $resn_exch{" C3' HEU"}=" C3* HEU"; $resn_exch{" C2' HEU"}=" C2* HEU";
$resn_exch{" O3' HEU"}=" O3* HEU"; $resn_exch{" OP3 HEU"}=" O3P HEU"; $resn_exch{" NA  HEV"}=" N_A HEV";
$resn_exch{" NB  HEV"}=" N_B HEV"; $resn_exch{" NC  HEV"}=" N_C HEV"; $resn_exch{" ND  HEV"}=" N_D HEV";
$resn_exch{" NA  HFM"}=" N_A HFM"; $resn_exch{" NB  HFM"}=" N_B HFM"; $resn_exch{" NC  HFM"}=" N_C HFM";
$resn_exch{" ND  HFM"}=" N_D HFM"; $resn_exch{" NA  HIF"}=" N_A HIF"; $resn_exch{" NB  HIF"}=" N_B HIF";
$resn_exch{" NC  HIF"}=" N_C HIF"; $resn_exch{" ND  HIF"}=" N_D HIF"; $resn_exch{" NC  HME"}=" N_C HME";
$resn_exch{" NB  HME"}=" N_B HME"; $resn_exch{" NA  HME"}=" N_A HME"; $resn_exch{" ND  HME"}=" N_D HME";
$resn_exch{" O5B HMG"}="O5*A HMG"; $resn_exch{" C5B HMG"}="C5*A HMG"; $resn_exch{" C4B HMG"}="C4*A HMG";
$resn_exch{" O4B HMG"}="O4*A HMG"; $resn_exch{" C3B HMG"}="C3*A HMG"; $resn_exch{" O3B HMG"}="O3*A HMG";
$resn_exch{" C2B HMG"}="C2*A HMG"; $resn_exch{" O2B HMG"}="O2*A HMG"; $resn_exch{" C1B HMG"}="C1*A HMG";
$resn_exch{" P3B HMG"}="P3*A HMG"; $resn_exch{" CBP HMG"}="C11P HMG"; $resn_exch{" CCP HMG"}="C12P HMG";
$resn_exch{" CDP HMG"}="C13P HMG"; $resn_exch{" CEP HMG"}="C14P HMG"; $resn_exch{" CAP HMG"}="C10P HMG";
$resn_exch{" OAP HMG"}="O10P HMG"; $resn_exch{" NA  HNI"}=" N_A HNI"; $resn_exch{" NB  HNI"}=" N_B HNI";
$resn_exch{" NC  HNI"}=" N_C HNI"; $resn_exch{" ND  HNI"}=" N_D HNI"; $resn_exch{" OP1 HOB"}=" O1P HOB";
$resn_exch{" OP2 HOB"}=" O2P HOB"; $resn_exch{" O5' HOB"}=" O5* HOB"; $resn_exch{" C5' HOB"}=" C5* HOB";
$resn_exch{" C4' HOB"}=" C4* HOB"; $resn_exch{" C3' HOB"}=" C3* HOB"; $resn_exch{" C2' HOB"}=" C2* HOB";
$resn_exch{" C1' HOB"}=" C1* HOB"; $resn_exch{" O4' HOB"}=" O4* HOB"; $resn_exch{" O3' HOB"}=" O3* HOB";
$resn_exch{" OP3 HOB"}=" O3P HOB"; $resn_exch{" OP1 HOL"}=" O1P HOL"; $resn_exch{" OP2 HOL"}=" O2P HOL";
$resn_exch{" O5' HOL"}=" O5* HOL"; $resn_exch{" C5' HOL"}=" C5* HOL"; $resn_exch{" C4' HOL"}=" C4* HOL";
$resn_exch{" C3' HOL"}=" C3* HOL"; $resn_exch{" C2' HOL"}=" C2* HOL"; $resn_exch{" C1' HOL"}=" C1* HOL";
$resn_exch{" O4' HOL"}=" O4* HOL"; $resn_exch{" O3' HOL"}=" O3* HOL"; $resn_exch{" OP3 HOL"}=" O3P HOL";
$resn_exch{" O5' HPD"}=" O5* HPD"; $resn_exch{" C5' HPD"}=" C5* HPD"; $resn_exch{" O4' HPD"}=" O4* HPD";
$resn_exch{" O3' HPD"}=" O3* HPD"; $resn_exch{" C1' HPD"}=" C1* HPD"; $resn_exch{" O1' HPD"}=" O1* HPD";
$resn_exch{" C3' HPD"}=" C3* HPD"; $resn_exch{" C4' HPD"}=" C4* HPD"; $resn_exch{" C2' HPD"}=" C2* HPD";
$resn_exch{" C1' HPR"}=" C1* HPR"; $resn_exch{" C2' HPR"}=" C2* HPR"; $resn_exch{" O2' HPR"}=" O2* HPR";
$resn_exch{" C3' HPR"}=" C3* HPR"; $resn_exch{" O3' HPR"}=" O3* HPR"; $resn_exch{" C4' HPR"}=" C4* HPR";
$resn_exch{" O4' HPR"}=" O4* HPR"; $resn_exch{" C5' HPR"}=" C5* HPR"; $resn_exch{" O5' HPR"}=" O5* HPR";
$resn_exch{" O5' HSS"}=" O5* HSS"; $resn_exch{" C5' HSS"}=" C5* HSS"; $resn_exch{" C4' HSS"}=" C4* HSS";
$resn_exch{" O4' HSS"}=" O4* HSS"; $resn_exch{" C3' HSS"}=" C3* HSS"; $resn_exch{" O3' HSS"}=" O3* HSS";
$resn_exch{" C2' HSS"}=" C2* HSS"; $resn_exch{" O2' HSS"}=" O2* HSS"; $resn_exch{" C1' HSS"}=" C1* HSS";
$resn_exch{" N1' HTL"}=" N1* HTL"; $resn_exch{" C2' HTL"}=" C2* HTL"; $resn_exch{" N3' HTL"}=" N3* HTL";
$resn_exch{" C4' HTL"}=" C4* HTL"; $resn_exch{" N4' HTL"}=" N4* HTL"; $resn_exch{" C5' HTL"}=" C5* HTL";
$resn_exch{" C6' HTL"}=" C6* HTL"; $resn_exch{" C1' HY1"}=" C1* HY1"; $resn_exch{" C2' HY1"}=" C2* HY1";
$resn_exch{" C3' HY1"}=" C3* HY1"; $resn_exch{" C4' HY1"}=" C4* HY1"; $resn_exch{" C5' HY1"}=" C5* HY1";
$resn_exch{" C6' HY1"}=" C6* HY1"; $resn_exch{" OD1 HYP"}=" OD  HYP"; $resn_exch{" OP3   I"}=" O3P   I";
$resn_exch{" OP1   I"}=" O1P   I"; $resn_exch{" OP2   I"}=" O2P   I"; $resn_exch{" O5'   I"}=" O5*   I";
$resn_exch{" C5'   I"}=" C5*   I"; $resn_exch{" C4'   I"}=" C4*   I"; $resn_exch{" O4'   I"}=" O4*   I";
$resn_exch{" C3'   I"}=" C3*   I"; $resn_exch{" O3'   I"}=" O3*   I"; $resn_exch{" C2'   I"}=" C2*   I";
$resn_exch{" O2'   I"}=" O2*   I"; $resn_exch{" C1'   I"}=" C1*   I"; $resn_exch{" C3' I3A"}=" C3* I3A";
$resn_exch{" OP1 I5C"}=" O1P I5C"; $resn_exch{" OP2 I5C"}=" O2P I5C"; $resn_exch{" O5' I5C"}=" O5* I5C";
$resn_exch{" C2' I5C"}=" C2* I5C"; $resn_exch{" C5' I5C"}=" C5* I5C"; $resn_exch{" C4' I5C"}=" C4* I5C";
$resn_exch{" O4' I5C"}=" O4* I5C"; $resn_exch{" C1' I5C"}=" C1* I5C"; $resn_exch{" C3' I5C"}=" C3* I5C";
$resn_exch{" O3' I5C"}=" O3* I5C"; $resn_exch{" OP3 I5C"}=" O3P I5C"; $resn_exch{" O5D IB2"}=" O5* IB2";
$resn_exch{" C5D IB2"}=" C5* IB2"; $resn_exch{" C4D IB2"}=" C4* IB2"; $resn_exch{" O4D IB2"}=" O4* IB2";
$resn_exch{" C3D IB2"}=" C3* IB2"; $resn_exch{" O3D IB2"}=" O3* IB2"; $resn_exch{" C2D IB2"}=" C2* IB2";
$resn_exch{" O2D IB2"}=" O2* IB2"; $resn_exch{" C1D IB2"}=" C1* IB2"; $resn_exch{" N9A IB2"}=" AN9 IB2";
$resn_exch{" C8A IB2"}=" AC8 IB2"; $resn_exch{" N7A IB2"}=" AN7 IB2"; $resn_exch{" C5A IB2"}=" AC5 IB2";
$resn_exch{" C6A IB2"}=" AC6 IB2"; $resn_exch{" N6A IB2"}=" AN6 IB2"; $resn_exch{" N1A IB2"}=" AN1 IB2";
$resn_exch{" C2A IB2"}=" AC2 IB2"; $resn_exch{" N3A IB2"}=" AN3 IB2"; $resn_exch{" C4A IB2"}=" AC4 IB2";
$resn_exch{" OP1  IC"}=" O1P  IC"; $resn_exch{" OP2  IC"}=" O2P  IC"; $resn_exch{" OP3  IC"}=" O3P  IC";
$resn_exch{" O5'  IC"}=" O5*  IC"; $resn_exch{" O3'  IC"}=" O3*  IC"; $resn_exch{" C1'  IC"}=" C1*  IC";
$resn_exch{" C2'  IC"}=" C2*  IC"; $resn_exch{" C3'  IC"}=" C3*  IC"; $resn_exch{" C4'  IC"}=" C4*  IC";
$resn_exch{" C5'  IC"}=" C5*  IC"; $resn_exch{" O4'  IC"}=" O4*  IC"; $resn_exch{" O2'  IC"}=" O2*  IC";
$resn_exch{" O5' ID2"}=" O5* ID2"; $resn_exch{" C5' ID2"}=" C5* ID2"; $resn_exch{" C4' ID2"}=" C4* ID2";
$resn_exch{" O4' ID2"}=" O4* ID2"; $resn_exch{" C1' ID2"}=" C1* ID2"; $resn_exch{" C2' ID2"}=" C2* ID2";
$resn_exch{" C3' ID2"}=" C3* ID2"; $resn_exch{" O3' ID2"}=" O3* ID2"; $resn_exch{" O5' IDP"}=" O5* IDP";
$resn_exch{" C5' IDP"}=" C5* IDP"; $resn_exch{" C4' IDP"}=" C4* IDP"; $resn_exch{" O4' IDP"}=" O4* IDP";
$resn_exch{" C3' IDP"}=" C3* IDP"; $resn_exch{" O3' IDP"}=" O3* IDP"; $resn_exch{" C2' IDP"}=" C2* IDP";
$resn_exch{" O2' IDP"}=" O2* IDP"; $resn_exch{" C1' IDP"}=" C1* IDP"; $resn_exch{" OP1  IG"}=" O1P  IG";
$resn_exch{" OP2  IG"}=" O2P  IG"; $resn_exch{" OP3  IG"}=" O3P  IG"; $resn_exch{" O5'  IG"}=" O5*  IG";
$resn_exch{" O3'  IG"}=" O3*  IG"; $resn_exch{" C1'  IG"}=" C1*  IG"; $resn_exch{" C2'  IG"}=" C2*  IG";
$resn_exch{" C3'  IG"}=" C3*  IG"; $resn_exch{" C4'  IG"}=" C4*  IG"; $resn_exch{" C5'  IG"}=" C5*  IG";
$resn_exch{" O4'  IG"}=" O4*  IG"; $resn_exch{" O2'  IG"}=" O2*  IG"; $resn_exch{" OP1 IGU"}=" O1P IGU";
$resn_exch{" OP2 IGU"}=" O2P IGU"; $resn_exch{" OP3 IGU"}=" O3P IGU"; $resn_exch{" O5' IGU"}=" O5* IGU";
$resn_exch{" C5' IGU"}=" C5* IGU"; $resn_exch{" C4' IGU"}=" C4* IGU"; $resn_exch{" O4' IGU"}=" O4* IGU";
$resn_exch{" C3' IGU"}=" C3* IGU"; $resn_exch{" O3' IGU"}=" O3* IGU"; $resn_exch{" C2' IGU"}=" C2* IGU";
$resn_exch{" C1' IGU"}=" C1* IGU"; $resn_exch{" O12 IHS"}="2O1  IHS"; $resn_exch{" O32 IHS"}="2O3  IHS";
$resn_exch{" O22 IHS"}="2O2  IHS"; $resn_exch{" O42 IHS"}="2O4  IHS"; $resn_exch{" O13 IHS"}="3O1  IHS";
$resn_exch{" O33 IHS"}="3O3  IHS"; $resn_exch{" O23 IHS"}="3O2  IHS"; $resn_exch{" O43 IHS"}="3O4  IHS";
$resn_exch{" O14 IHS"}="4O1  IHS"; $resn_exch{" O34 IHS"}="4O3  IHS"; $resn_exch{" O24 IHS"}="4O2  IHS";
$resn_exch{" O44 IHS"}="4O4  IHS"; $resn_exch{" O15 IHS"}="5O1  IHS"; $resn_exch{" O35 IHS"}="5O3  IHS";
$resn_exch{" O25 IHS"}="5O2  IHS"; $resn_exch{" O45 IHS"}="5O4  IHS"; $resn_exch{" O16 IHS"}="6O1  IHS";
$resn_exch{" O36 IHS"}="6O3  IHS"; $resn_exch{" O26 IHS"}="6O2  IHS"; $resn_exch{" O46 IHS"}="6O4  IHS";
$resn_exch{" OP1 IMC"}=" O1P IMC"; $resn_exch{" OP2 IMC"}=" O2P IMC"; $resn_exch{" OP3 IMC"}=" O3P IMC";
$resn_exch{" O5' IMC"}=" O5* IMC"; $resn_exch{" C5' IMC"}=" C5* IMC"; $resn_exch{" C4' IMC"}=" C4* IMC";
$resn_exch{" O4' IMC"}=" O4* IMC"; $resn_exch{" C1' IMC"}=" C1* IMC"; $resn_exch{" C2' IMC"}=" C2* IMC";
$resn_exch{" C3' IMC"}=" C3* IMC"; $resn_exch{" O3' IMC"}=" O3* IMC"; $resn_exch{" CG1 IME"}="1CG  IME";
$resn_exch{" CDC IME"}="1CD2 IME"; $resn_exch{" NDB IME"}="1ND1 IME"; $resn_exch{" CEB IME"}="1CE1 IME";
$resn_exch{" NEC IME"}="1NE2 IME"; $resn_exch{" CG2 IME"}="2CG  IME"; $resn_exch{" CDM IME"}="2CD2 IME";
$resn_exch{" NDL IME"}="2ND1 IME"; $resn_exch{" CEL IME"}="2CE1 IME"; $resn_exch{" NEM IME"}="2NE2 IME";
$resn_exch{" CG3 IME"}="3CG  IME"; $resn_exch{" CDW IME"}="3CD2 IME"; $resn_exch{" NDV IME"}="3ND1 IME";
$resn_exch{" CEV IME"}="3CE1 IME"; $resn_exch{" NEW IME"}="3NE2 IME"; $resn_exch{" CG4 IME"}="4CG  IME";
$resn_exch{" CD6 IME"}="4CD2 IME"; $resn_exch{" ND5 IME"}="4ND1 IME"; $resn_exch{" CE5 IME"}="4CE1 IME";
$resn_exch{" NE6 IME"}="4NE2 IME"; $resn_exch{" CG1 IMF"}="1CG  IMF"; $resn_exch{" CDC IMF"}="1CD2 IMF";
$resn_exch{" NDB IMF"}="1ND1 IMF"; $resn_exch{" CEB IMF"}="1CE1 IMF"; $resn_exch{" NEC IMF"}="1NE2 IMF";
$resn_exch{" CG2 IMF"}="2CG  IMF"; $resn_exch{" CDM IMF"}="2CD2 IMF"; $resn_exch{" NDL IMF"}="2ND1 IMF";
$resn_exch{" CEL IMF"}="2CE1 IMF"; $resn_exch{" NEM IMF"}="2NE2 IMF"; $resn_exch{" CG3 IMF"}="3CG  IMF";
$resn_exch{" CDW IMF"}="3CD2 IMF"; $resn_exch{" NDV IMF"}="3ND1 IMF"; $resn_exch{" CEV IMF"}="3CE1 IMF";
$resn_exch{" NEW IMF"}="3NE2 IMF"; $resn_exch{" CG4 IMF"}="4CG  IMF"; $resn_exch{" CD6 IMF"}="4CD2 IMF";
$resn_exch{" ND5 IMF"}="4ND1 IMF"; $resn_exch{" CE5 IMF"}="4CE1 IMF"; $resn_exch{" NE6 IMF"}="4NE2 IMF";
$resn_exch{" O5' IMG"}=" O5* IMG"; $resn_exch{" C5' IMG"}=" C5* IMG"; $resn_exch{" C4' IMG"}=" C4* IMG";
$resn_exch{" N4' IMG"}=" N4* IMG"; $resn_exch{" C3' IMG"}=" C3* IMG"; $resn_exch{" O3' IMG"}=" O3* IMG";
$resn_exch{" C2' IMG"}=" C2* IMG"; $resn_exch{" O2' IMG"}=" O2* IMG"; $resn_exch{" C1' IMG"}=" C1* IMG";
$resn_exch{" O5' IMH"}=" O5* IMH"; $resn_exch{" C5' IMH"}=" C5* IMH"; $resn_exch{" C4' IMH"}=" C4* IMH";
$resn_exch{" N4' IMH"}=" N4* IMH"; $resn_exch{" C3' IMH"}=" C3* IMH"; $resn_exch{" O3' IMH"}=" O3* IMH";
$resn_exch{" C2' IMH"}=" C2* IMH"; $resn_exch{" O2' IMH"}=" O2* IMH"; $resn_exch{" C1' IMH"}=" C1* IMH";
$resn_exch{" O5' IMP"}=" O5* IMP"; $resn_exch{" C5' IMP"}=" C5* IMP"; $resn_exch{" C4' IMP"}=" C4* IMP";
$resn_exch{" O4' IMP"}=" O4* IMP"; $resn_exch{" C3' IMP"}=" C3* IMP"; $resn_exch{" O3' IMP"}=" O3* IMP";
$resn_exch{" C2' IMP"}=" C2* IMP"; $resn_exch{" O2' IMP"}=" O2* IMP"; $resn_exch{" C1' IMP"}=" C1* IMP";
$resn_exch{" O5' IMR"}=" O5* IMR"; $resn_exch{" C5' IMR"}=" C5* IMR"; $resn_exch{" C4' IMR"}=" C4* IMR";
$resn_exch{" N4' IMR"}=" N4* IMR"; $resn_exch{" C3' IMR"}=" C3* IMR"; $resn_exch{" O3' IMR"}=" O3* IMR";
$resn_exch{" C2' IMR"}=" C2* IMR"; $resn_exch{" O2' IMR"}=" O2* IMR"; $resn_exch{" C1' IMR"}=" C1* IMR";
$resn_exch{" O5' IMU"}=" O5* IMU"; $resn_exch{" C5' IMU"}=" C5* IMU"; $resn_exch{" C4' IMU"}=" C4* IMU";
$resn_exch{" N4' IMU"}=" N4* IMU"; $resn_exch{" C3' IMU"}=" C3* IMU"; $resn_exch{" O3' IMU"}=" O3* IMU";
$resn_exch{" C2' IMU"}=" C2* IMU"; $resn_exch{" O2' IMU"}=" O2* IMU"; $resn_exch{" C1' IMU"}=" C1* IMU";
$resn_exch{" OA' IN3"}="'O10 IN3"; $resn_exch{" CB' IN3"}="'C11 IN3"; $resn_exch{" CC' IN3"}="'C12 IN3";
$resn_exch{" CE' IN3"}="'C14 IN3"; $resn_exch{" CF' IN3"}="'C15 IN3"; $resn_exch{" CG' IN3"}="'C16 IN3";
$resn_exch{" CH' IN3"}="'C17 IN3"; $resn_exch{" OI' IN3"}="'O18 IN3"; $resn_exch{" NJ' IN3"}="'N19 IN3";
$resn_exch{" NK' IN3"}="'N20 IN3"; $resn_exch{" NO' IN3"}="'N24 IN3"; $resn_exch{" OA' IN6"}="'O10 IN6";
$resn_exch{" CB' IN6"}="'C11 IN6"; $resn_exch{" CC' IN6"}="'C12 IN6"; $resn_exch{" CE' IN6"}="'C14 IN6";
$resn_exch{" CF' IN6"}="'C15 IN6"; $resn_exch{" CG' IN6"}="'C16 IN6"; $resn_exch{" CH' IN6"}="'C17 IN6";
$resn_exch{" OI' IN6"}="'O18 IN6"; $resn_exch{" NJ' IN6"}="'N19 IN6"; $resn_exch{" NK' IN6"}="'N20 IN6";
$resn_exch{" NO' IN6"}="'N24 IN6"; $resn_exch{" C1B INB"}=" C1\" INB"; $resn_exch{" C2B INB"}=" C2\" INB";
$resn_exch{" CMB INB"}=" CM\" INB"; $resn_exch{" O5' IOT"}=" O5* IOT"; $resn_exch{" C5' IOT"}=" C5* IOT";
$resn_exch{" C4' IOT"}=" C4* IOT"; $resn_exch{" O4' IOT"}=" O4* IOT"; $resn_exch{" C3' IOT"}=" C3* IOT";
$resn_exch{" O3' IOT"}=" O3* IOT"; $resn_exch{" C2' IOT"}=" C2* IOT"; $resn_exch{" O2' IOT"}=" O2* IOT";
$resn_exch{" C1' IOT"}=" C1* IOT"; $resn_exch{" OXT IOY"}=" O   IOY"; $resn_exch{" O   IOY"}=" OXT IOY";
$resn_exch{" C1' IQB"}=" C1* IQB"; $resn_exch{" C2B IQB"}=" C2* IQB"; $resn_exch{" C3B IQB"}=" C3* IQB";
$resn_exch{" C4' IQB"}=" C4* IQB"; $resn_exch{"BR4' IQB"}="BR4* IQB"; $resn_exch{" C5B IQB"}=" C5* IQB";
$resn_exch{" C6B IQB"}=" C6* IQB"; $resn_exch{" C1' IRF"}=" C1* IRF"; $resn_exch{" C2' IRF"}=" C2* IRF";
$resn_exch{" O2' IRF"}=" O2* IRF"; $resn_exch{" C3' IRF"}=" C3* IRF"; $resn_exch{" O3' IRF"}=" O3* IRF";
$resn_exch{" C4' IRF"}=" C4* IRF"; $resn_exch{" O4' IRF"}=" O4* IRF"; $resn_exch{" C5' IRF"}=" C5* IRF";
$resn_exch{" O5' IRF"}=" O5* IRF"; $resn_exch{" O5' IRP"}=" O5* IRP"; $resn_exch{" C5' IRP"}=" C5* IRP";
$resn_exch{" C4' IRP"}=" C4* IRP"; $resn_exch{" N4' IRP"}=" N4* IRP"; $resn_exch{" C3' IRP"}=" C3* IRP";
$resn_exch{" O3' IRP"}=" O3* IRP"; $resn_exch{" C2' IRP"}=" C2* IRP"; $resn_exch{" O2' IRP"}=" O2* IRP";
$resn_exch{" C1' IRP"}=" C1* IRP"; $resn_exch{" O5' ITT"}=" O5* ITT"; $resn_exch{" C5' ITT"}=" C5* ITT";
$resn_exch{" C4' ITT"}=" C4* ITT"; $resn_exch{" O4' ITT"}=" O4* ITT"; $resn_exch{" C3' ITT"}=" C3* ITT";
$resn_exch{" O3' ITT"}=" O3* ITT"; $resn_exch{" C2' ITT"}=" C2* ITT"; $resn_exch{" O2' ITT"}=" O2* ITT";
$resn_exch{" C1' ITT"}=" C1* ITT"; $resn_exch{" OP1  IU"}=" O1P  IU"; $resn_exch{" OP2  IU"}=" O2P  IU";
$resn_exch{" OP3  IU"}=" O3P  IU"; $resn_exch{" O5'  IU"}=" O5*  IU"; $resn_exch{" C5'  IU"}=" C5*  IU";
$resn_exch{" C4'  IU"}=" C4*  IU"; $resn_exch{" O4'  IU"}=" O4*  IU"; $resn_exch{" C3'  IU"}=" C3*  IU";
$resn_exch{" O3'  IU"}=" O3*  IU"; $resn_exch{" C2'  IU"}=" C2*  IU"; $resn_exch{" O2'  IU"}=" O2*  IU";
$resn_exch{" C1'  IU"}=" C1*  IU"; $resn_exch{" C1' JW5"}=" C1* JW5"; $resn_exch{" O4' JW5"}=" O4* JW5";
$resn_exch{" C2' JW5"}=" C2* JW5"; $resn_exch{" O2' JW5"}=" O2* JW5"; $resn_exch{" C3' JW5"}=" C3* JW5";
$resn_exch{" O3' JW5"}=" O3* JW5"; $resn_exch{" C4' JW5"}=" C4* JW5"; $resn_exch{" C5' JW5"}=" C5* JW5";
$resn_exch{" O5' JW5"}=" O5* JW5"; $resn_exch{" C1' K05"}=" C1* K05"; $resn_exch{" C2' K05"}=" C2* K05";
$resn_exch{" C3' K05"}=" C3* K05"; $resn_exch{" C4' K05"}=" C4* K05"; $resn_exch{" C5' K05"}=" C5* K05";
$resn_exch{" C6' K05"}=" C6* K05"; $resn_exch{" OP3 KAG"}=" O3P KAG"; $resn_exch{" OP1 KAG"}=" O1P KAG";
$resn_exch{" OP2 KAG"}=" O2P KAG"; $resn_exch{" O5' KAG"}=" O5* KAG"; $resn_exch{" C5' KAG"}=" C5* KAG";
$resn_exch{" C4' KAG"}=" C4* KAG"; $resn_exch{" O4' KAG"}=" O4* KAG"; $resn_exch{" C3' KAG"}=" C3* KAG";
$resn_exch{" O3' KAG"}=" O3* KAG"; $resn_exch{" C2' KAG"}=" C2* KAG"; $resn_exch{" C1' KAG"}=" C1* KAG";
$resn_exch{" OQ1 KCX"}=" OX1 KCX"; $resn_exch{" OQ2 KCX"}=" OX2 KCX"; $resn_exch{" OXT KGC"}=" O1  KGC";
$resn_exch{" O   KGC"}=" OXT KGC"; $resn_exch{" O1' KHO"}=" O1* KHO"; $resn_exch{" C1' KHO"}=" C1* KHO";
$resn_exch{" C2' KHO"}=" C2* KHO"; $resn_exch{" C3' KHO"}=" C3* KHO"; $resn_exch{" O3' KHO"}=" O3* KHO";
$resn_exch{" O2' KHO"}=" O2* KHO"; $resn_exch{" O4' KHO"}=" O4* KHO"; $resn_exch{" C4' KHO"}=" C4* KHO";
$resn_exch{" C5' KHO"}=" C5* KHO"; $resn_exch{" O5' KHO"}=" O5* KHO"; $resn_exch{" O5' KHP"}=" O5* KHP";
$resn_exch{" C5B KHP"}=" C5* KHP"; $resn_exch{" C4B KHP"}=" C4* KHP"; $resn_exch{" O4' KHP"}=" O4* KHP";
$resn_exch{" C1B KHP"}=" C1* KHP"; $resn_exch{" C2B KHP"}=" C2* KHP"; $resn_exch{" O2B KHP"}=" O2* KHP";
$resn_exch{" C3B KHP"}=" C3* KHP"; $resn_exch{" O3B KHP"}=" O3* KHP"; $resn_exch{" O1' KHP"}=" O1* KHP";
$resn_exch{" N2A KT3"}="2N   KT3"; $resn_exch{" CD1 KT3"}="1CD  KT3"; $resn_exch{" OEB KT3"}="1OE1 KT3";
$resn_exch{" CG1 KT3"}="1CG  KT3"; $resn_exch{" CB1 KT3"}="1CB  KT3"; $resn_exch{" CA1 KT3"}="1CA  KT3";
$resn_exch{" C1A KT3"}="1C   KT3"; $resn_exch{" O11 KT3"}="1OXT KT3"; $resn_exch{" O1A KT3"}="1O   KT3";
$resn_exch{" N1A KT3"}="1N   KT3"; $resn_exch{" OEN KT3"}="3OE2 KT3"; $resn_exch{" OED KT3"}="3OE1 KT3";
$resn_exch{" CD3 KT3"}="3CD  KT3"; $resn_exch{" CG3 KT3"}="3CG  KT3"; $resn_exch{" CB3 KT3"}="3CB  KT3";
$resn_exch{" CA3 KT3"}="3CA  KT3"; $resn_exch{" C3A KT3"}="3C   KT3"; $resn_exch{" O3  KT3"}="3O   KT3";
$resn_exch{" O12 KT3"}="3OXT KT3"; $resn_exch{" N3A KT3"}="3N   KT3"; $resn_exch{" OEC KT3"}="2OE1 KT3";
$resn_exch{" CD2 KT3"}="2CD  KT3"; $resn_exch{" CG2 KT3"}="2CG  KT3"; $resn_exch{" CB2 KT3"}="2CB  KT3";
$resn_exch{" CA2 KT3"}="2CA  KT3"; $resn_exch{" C2A KT3"}="2C   KT3"; $resn_exch{" O2  KT3"}="2O   KT3";
$resn_exch{" O13 KT3"}="2OXT KT3"; $resn_exch{" N2A KT5"}="2N   KT5"; $resn_exch{" CD1 KT5"}="1CD  KT5";
$resn_exch{" OEB KT5"}="1OE1 KT5"; $resn_exch{" CG1 KT5"}="1CG  KT5"; $resn_exch{" CB1 KT5"}="1CB  KT5";
$resn_exch{" CA1 KT5"}="1CA  KT5"; $resn_exch{" C1A KT5"}="1C   KT5"; $resn_exch{" O11 KT5"}="1OXT KT5";
$resn_exch{" O1A KT5"}="1O   KT5"; $resn_exch{" N1A KT5"}="1N   KT5"; $resn_exch{" OEF KT5"}="5OE1 KT5";
$resn_exch{" OEP KT5"}="5OE2 KT5"; $resn_exch{" CD5 KT5"}="5CD  KT5"; $resn_exch{" CG5 KT5"}="5CG  KT5";
$resn_exch{" CB5 KT5"}="5CB  KT5"; $resn_exch{" CA5 KT5"}="5CA  KT5"; $resn_exch{" C5A KT5"}="5C   KT5";
$resn_exch{" O5  KT5"}="5O   KT5"; $resn_exch{" O12 KT5"}="5OXT KT5"; $resn_exch{" N5  KT5"}="5N   KT5";
$resn_exch{" OEE KT5"}="4OE1 KT5"; $resn_exch{" CD4 KT5"}="4CD  KT5"; $resn_exch{" CG4 KT5"}="4CG  KT5";
$resn_exch{" CB4 KT5"}="4CB  KT5"; $resn_exch{" CA4 KT5"}="4CA  KT5"; $resn_exch{" C4A KT5"}="4C   KT5";
$resn_exch{" O4  KT5"}="4O   KT5"; $resn_exch{" O13 KT5"}="4OXT KT5"; $resn_exch{" N4  KT5"}="4N   KT5";
$resn_exch{" OED KT5"}="3OE1 KT5"; $resn_exch{" CD3 KT5"}="3CD  KT5"; $resn_exch{" CG3 KT5"}="3CG  KT5";
$resn_exch{" CB3 KT5"}="3CB  KT5"; $resn_exch{" CA3 KT5"}="3CA  KT5"; $resn_exch{" C3A KT5"}="3C   KT5";
$resn_exch{" O3  KT5"}="3O   KT5"; $resn_exch{" O14 KT5"}="3OXT KT5"; $resn_exch{" N3A KT5"}="3N   KT5";
$resn_exch{" OEC KT5"}="2OE1 KT5"; $resn_exch{" CD2 KT5"}="2CD  KT5"; $resn_exch{" CG2 KT5"}="2CG  KT5";
$resn_exch{" CB2 KT5"}="2CB  KT5"; $resn_exch{" CA2 KT5"}="2CA  KT5"; $resn_exch{" C2A KT5"}="2C   KT5";
$resn_exch{" O2  KT5"}="2O   KT5"; $resn_exch{" O15 KT5"}="2OXT KT5"; $resn_exch{" O'5 LAM"}="1O4' LAM";
$resn_exch{" O'6 LAM"}="2O4' LAM"; $resn_exch{" O5' LAQ"}=" O5* LAQ"; $resn_exch{" C5' LAQ"}=" C5* LAQ";
$resn_exch{" C4' LAQ"}=" C4* LAQ"; $resn_exch{" C3' LAQ"}=" C3* LAQ"; $resn_exch{" O3' LAQ"}=" O3* LAQ";
$resn_exch{" C2' LAQ"}=" C2* LAQ"; $resn_exch{" O2' LAQ"}=" O2* LAQ"; $resn_exch{" C1' LAQ"}=" C1* LAQ";
$resn_exch{" O4' LAQ"}=" O4* LAQ"; $resn_exch{" OP3  LC"}=" O3P  LC"; $resn_exch{" OP1  LC"}=" O1P  LC";
$resn_exch{" OP2  LC"}=" O2P  LC"; $resn_exch{" O5'  LC"}=" O5*  LC"; $resn_exch{" C5'  LC"}=" C5*  LC";
$resn_exch{" C4'  LC"}=" C4*  LC"; $resn_exch{" O4'  LC"}=" O4*  LC"; $resn_exch{" C3'  LC"}=" C3*  LC";
$resn_exch{" O3'  LC"}=" O3*  LC"; $resn_exch{" C2'  LC"}=" C2*  LC"; $resn_exch{" O2'  LC"}=" O2*  LC";
$resn_exch{" C1'  LC"}=" C1*  LC"; $resn_exch{" O5' LCA"}=" O5* LCA"; $resn_exch{" C5' LCA"}=" C5* LCA";
$resn_exch{" C3' LCA"}=" C3* LCA"; $resn_exch{" C6' LCA"}=" C6* LCA"; $resn_exch{" C2' LCA"}=" C2* LCA";
$resn_exch{" C4' LCA"}=" C4* LCA"; $resn_exch{" C1' LCA"}=" C1* LCA"; $resn_exch{" O4' LCA"}=" O4* LCA";
$resn_exch{" O3' LCA"}=" O3* LCA"; $resn_exch{" O2' LCA"}=" O2* LCA"; $resn_exch{" O5' LCC"}=" O5* LCC";
$resn_exch{" C5' LCC"}=" C5* LCC"; $resn_exch{" C4' LCC"}=" C4* LCC"; $resn_exch{" O4' LCC"}=" O4* LCC";
$resn_exch{" C1' LCC"}=" C1* LCC"; $resn_exch{" C3' LCC"}=" C3* LCC"; $resn_exch{" C2' LCC"}=" C2* LCC";
$resn_exch{" O2' LCC"}=" O2* LCC"; $resn_exch{" O3' LCC"}=" O3* LCC"; $resn_exch{" C6' LCC"}=" C6* LCC";
$resn_exch{" OP1 LCG"}=" O1P LCG"; $resn_exch{" O5' LCG"}=" O5* LCG"; $resn_exch{" C5' LCG"}=" C5* LCG";
$resn_exch{" C3' LCG"}=" C3* LCG"; $resn_exch{" C6' LCG"}=" C6* LCG"; $resn_exch{" C2' LCG"}=" C2* LCG";
$resn_exch{" C4' LCG"}=" C4* LCG"; $resn_exch{" C1' LCG"}=" C1* LCG"; $resn_exch{" O4' LCG"}=" O4* LCG";
$resn_exch{" OP2 LCG"}=" O2P LCG"; $resn_exch{" O2' LCG"}=" O2* LCG"; $resn_exch{" O3' LCG"}=" O3* LCG";
$resn_exch{" OP3 LCG"}=" O3P LCG"; $resn_exch{" O5' LCH"}=" O5* LCH"; $resn_exch{" C5' LCH"}=" C5* LCH";
$resn_exch{" C4' LCH"}=" C4* LCH"; $resn_exch{" O4' LCH"}=" O4* LCH"; $resn_exch{" C1' LCH"}=" C1* LCH";
$resn_exch{" C3' LCH"}=" C3* LCH"; $resn_exch{" C2' LCH"}=" C2* LCH"; $resn_exch{" O2' LCH"}=" O2* LCH";
$resn_exch{" O3' LCH"}=" O3* LCH"; $resn_exch{" C6' LCH"}=" C6* LCH"; $resn_exch{" C1' LFN"}=" C1* LFN";
$resn_exch{" OP3  LG"}=" O3P  LG"; $resn_exch{" OP1  LG"}=" O1P  LG"; $resn_exch{" OP2  LG"}=" O2P  LG";
$resn_exch{" O5'  LG"}=" O5*  LG"; $resn_exch{" C5'  LG"}=" C5*  LG"; $resn_exch{" C4'  LG"}=" C4*  LG";
$resn_exch{" O4'  LG"}=" O4*  LG"; $resn_exch{" C3'  LG"}=" C3*  LG"; $resn_exch{" O3'  LG"}=" O3*  LG";
$resn_exch{" C2'  LG"}=" C2*  LG"; $resn_exch{" O2'  LG"}=" O2*  LG"; $resn_exch{" C1'  LG"}=" C1*  LG";
$resn_exch{" OP1 LGP"}=" O1P LGP"; $resn_exch{" OP2 LGP"}=" O2P LGP"; $resn_exch{" OP3 LGP"}=" O3P LGP";
$resn_exch{" O5' LGP"}=" O5* LGP"; $resn_exch{" C5' LGP"}=" C5* LGP"; $resn_exch{" C4' LGP"}=" C4* LGP";
$resn_exch{" O4' LGP"}=" O4* LGP"; $resn_exch{" C3' LGP"}=" C3* LGP"; $resn_exch{" O3' LGP"}=" O3* LGP";
$resn_exch{" C1' LGP"}=" C1* LGP"; $resn_exch{" OP3 LHU"}=" O3P LHU"; $resn_exch{" OP1 LHU"}=" O1P LHU";
$resn_exch{" OP2 LHU"}=" O2P LHU"; $resn_exch{" O5' LHU"}=" O5* LHU"; $resn_exch{" C5' LHU"}=" C5* LHU";
$resn_exch{" C4' LHU"}=" C4* LHU"; $resn_exch{" O4' LHU"}=" O4* LHU"; $resn_exch{" C3' LHU"}=" C3* LHU";
$resn_exch{" O3' LHU"}=" O3* LHU"; $resn_exch{" C2' LHU"}=" C2* LHU"; $resn_exch{" O2' LHU"}=" O2* LHU";
$resn_exch{" C1' LHU"}=" C1* LHU"; $resn_exch{" C4X LIH"}="C4A' LIH"; $resn_exch{" C8X LIH"}="C8A' LIH";
$resn_exch{" C2X LII"}="C2A' LII"; $resn_exch{" C5X LII"}="C5A' LII"; $resn_exch{" C1' LKC"}=" C1* LKC";
$resn_exch{" C2' LKC"}=" C2* LKC"; $resn_exch{" C3' LKC"}=" C3* LKC"; $resn_exch{" C4' LKC"}=" C4* LKC";
$resn_exch{" O4' LKC"}=" O4* LKC"; $resn_exch{" O3' LKC"}=" O3* LKC"; $resn_exch{" C5' LKC"}=" C5* LKC";
$resn_exch{" O5' LKC"}=" O5* LKC"; $resn_exch{" O2' LKC"}=" O2* LKC"; $resn_exch{" C6' LKC"}=" C6* LKC";
$resn_exch{" C2' LLP"}=" C2A LLP"; $resn_exch{" C4' LLP"}=" C4A LLP"; $resn_exch{" C5' LLP"}=" C5A LLP";
$resn_exch{" OP4 LLP"}=" O4P LLP"; $resn_exch{" OP1 LLP"}=" O1P LLP"; $resn_exch{" OP2 LLP"}=" O2P LLP";
$resn_exch{" OP3 LLP"}=" O3P LLP"; $resn_exch{" O5' LMS"}=" O5* LMS"; $resn_exch{" C5' LMS"}=" C5* LMS";
$resn_exch{" C4' LMS"}=" C4* LMS"; $resn_exch{" O4' LMS"}=" O4* LMS"; $resn_exch{" C3' LMS"}=" C3* LMS";
$resn_exch{" C1B LMT"}=" C1* LMT"; $resn_exch{" C2B LMT"}=" C2* LMT"; $resn_exch{" C3B LMT"}=" C3* LMT";
$resn_exch{" C4B LMT"}=" C4* LMT"; $resn_exch{" C5B LMT"}=" C5* LMT"; $resn_exch{" C6B LMT"}=" C6* LMT";
$resn_exch{" O1B LMT"}=" O1* LMT"; $resn_exch{" O2B LMT"}=" O2* LMT"; $resn_exch{" O3B LMT"}=" O3* LMT";
$resn_exch{" O4' LMT"}=" O4* LMT"; $resn_exch{" O5B LMT"}=" O5* LMT"; $resn_exch{" O6B LMT"}=" O6* LMT";
$resn_exch{" C1B LMU"}=" C1* LMU"; $resn_exch{" C2B LMU"}=" C2* LMU"; $resn_exch{" C3B LMU"}=" C3* LMU";
$resn_exch{" C4B LMU"}=" C4* LMU"; $resn_exch{" C5B LMU"}=" C5* LMU"; $resn_exch{" C6B LMU"}=" C6* LMU";
$resn_exch{" O1B LMU"}=" O1* LMU"; $resn_exch{" O2B LMU"}=" O2* LMU"; $resn_exch{" O3B LMU"}=" O3* LMU";
$resn_exch{" O4' LMU"}=" O4* LMU"; $resn_exch{" O5B LMU"}=" O5* LMU"; $resn_exch{" O6B LMU"}=" O6* LMU";
$resn_exch{" N7N LNC"}=" NN7 LNC"; $resn_exch{" C7N LNC"}=" NC7 LNC"; $resn_exch{" O7N LNC"}=" NO7 LNC";
$resn_exch{" C3N LNC"}=" NC3 LNC"; $resn_exch{" C4N LNC"}=" NC4 LNC"; $resn_exch{" C5N LNC"}=" NC5 LNC";
$resn_exch{" CS  LNC"}=" C_S LNC"; $resn_exch{" C6N LNC"}=" NC6 LNC"; $resn_exch{" N1N LNC"}=" NN1 LNC";
$resn_exch{" C2N LNC"}=" NC2 LNC"; $resn_exch{" C1D LNC"}="NC1* LNC"; $resn_exch{" O4D LNC"}="NO4* LNC";
$resn_exch{" C4D LNC"}="NC4* LNC"; $resn_exch{" C5D LNC"}="NC5* LNC"; $resn_exch{" O5D LNC"}="NO5* LNC";
$resn_exch{" PN  LNC"}=" NP  LNC"; $resn_exch{" O1N LNC"}=" NO1 LNC"; $resn_exch{" O2N LNC"}=" NO2 LNC";
$resn_exch{" PA  LNC"}=" AP  LNC"; $resn_exch{" O1A LNC"}=" AO1 LNC"; $resn_exch{" O2A LNC"}=" AO2 LNC";
$resn_exch{" O5B LNC"}="AO5* LNC"; $resn_exch{" C5B LNC"}="AC5* LNC"; $resn_exch{" C4B LNC"}="AC4* LNC";
$resn_exch{" O4B LNC"}="AO4* LNC"; $resn_exch{" C1B LNC"}="AC1* LNC"; $resn_exch{" C2B LNC"}="AC2* LNC";
$resn_exch{" O2B LNC"}="AO2* LNC"; $resn_exch{" C3B LNC"}="AC3* LNC"; $resn_exch{" O3B LNC"}="AO3* LNC";
$resn_exch{" N9A LNC"}=" AN9 LNC"; $resn_exch{" C8A LNC"}=" AC8 LNC"; $resn_exch{" N7A LNC"}=" AN7 LNC";
$resn_exch{" C5A LNC"}=" AC5 LNC"; $resn_exch{" C6A LNC"}=" AC6 LNC"; $resn_exch{" N6A LNC"}=" AN6 LNC";
$resn_exch{" N1A LNC"}=" AN1 LNC"; $resn_exch{" C2A LNC"}=" AC2 LNC"; $resn_exch{" N3A LNC"}=" AN3 LNC";
$resn_exch{" C4A LNC"}=" AC4 LNC"; $resn_exch{" C3D LNC"}="NC3* LNC"; $resn_exch{" O3D LNC"}="NO3* LNC";
$resn_exch{" C2D LNC"}="NC2* LNC"; $resn_exch{" O2D LNC"}="NO2* LNC"; $resn_exch{" C1' LVS"}=" C1* LVS";
$resn_exch{" O1' LVS"}=" O1* LVS"; $resn_exch{" O2' LVS"}=" O2* LVS"; $resn_exch{" C2' LVS"}=" C2* LVS";
$resn_exch{" OP1 M1G"}=" O1P M1G"; $resn_exch{" OP2 M1G"}=" O2P M1G"; $resn_exch{" OP3 M1G"}=" O3P M1G";
$resn_exch{" O5' M1G"}=" O5* M1G"; $resn_exch{" C5' M1G"}=" C5* M1G"; $resn_exch{" C4' M1G"}=" C4* M1G";
$resn_exch{" O4' M1G"}=" O4* M1G"; $resn_exch{" C3' M1G"}=" C3* M1G"; $resn_exch{" O3' M1G"}=" O3* M1G";
$resn_exch{" C2' M1G"}=" C2* M1G"; $resn_exch{" C1' M1G"}=" C1* M1G"; $resn_exch{" OP1 M2G"}=" O1P M2G";
$resn_exch{" OP2 M2G"}=" O2P M2G"; $resn_exch{" OP3 M2G"}=" O3P M2G"; $resn_exch{" O5' M2G"}=" O5* M2G";
$resn_exch{" C5' M2G"}=" C5* M2G"; $resn_exch{" C4' M2G"}=" C4* M2G"; $resn_exch{" O4' M2G"}=" O4* M2G";
$resn_exch{" C3' M2G"}=" C3* M2G"; $resn_exch{" O3' M2G"}=" O3* M2G"; $resn_exch{" C2' M2G"}=" C2* M2G";
$resn_exch{" O2' M2G"}=" O2* M2G"; $resn_exch{" C1' M2G"}=" C1* M2G"; $resn_exch{" N2' M5M"}=" N2* M5M";
$resn_exch{" C6' M5M"}=" C6* M5M"; $resn_exch{" C7' M5M"}=" C7* M5M"; $resn_exch{" O8' M5M"}=" O8* M5M";
$resn_exch{" OP1 M5M"}=" O1P M5M"; $resn_exch{" OP2 M5M"}=" O2P M5M"; $resn_exch{" O5' M5M"}=" O5* M5M";
$resn_exch{" C5' M5M"}=" C5* M5M"; $resn_exch{" C4' M5M"}=" C4* M5M"; $resn_exch{" O4' M5M"}=" O4* M5M";
$resn_exch{" C1' M5M"}=" C1* M5M"; $resn_exch{" C2' M5M"}=" C2* M5M"; $resn_exch{" C3' M5M"}=" C3* M5M";
$resn_exch{" O3' M5M"}=" O3* M5M"; $resn_exch{" OP3 M5M"}=" O3P M5M"; $resn_exch{" O5' M7G"}=" O5* M7G";
$resn_exch{" C5' M7G"}=" C5* M7G"; $resn_exch{" C4' M7G"}=" C4* M7G"; $resn_exch{" O4' M7G"}=" O4* M7G";
$resn_exch{" C3' M7G"}=" C3* M7G"; $resn_exch{" O3' M7G"}=" O3* M7G"; $resn_exch{" C2' M7G"}=" C2* M7G";
$resn_exch{" O2' M7G"}=" O2* M7G"; $resn_exch{" C1' M7G"}=" C1* M7G"; $resn_exch{" OP1 MA6"}=" O1P MA6";
$resn_exch{" OP2 MA6"}=" O2P MA6"; $resn_exch{" OP3 MA6"}=" O3P MA6"; $resn_exch{" O5' MA6"}=" O5* MA6";
$resn_exch{" C5' MA6"}=" C5* MA6"; $resn_exch{" C4' MA6"}=" C4* MA6"; $resn_exch{" O4' MA6"}=" O4* MA6";
$resn_exch{" C1' MA6"}=" C1* MA6"; $resn_exch{" C2' MA6"}=" C2* MA6"; $resn_exch{" O2' MA6"}=" O2* MA6";
$resn_exch{" C3' MA6"}=" C3* MA6"; $resn_exch{" O3' MA6"}=" O3* MA6"; $resn_exch{" OP1 MA7"}=" O1P MA7";
$resn_exch{" OP2 MA7"}=" O2P MA7"; $resn_exch{" O5' MA7"}=" O5* MA7"; $resn_exch{" C2' MA7"}=" C2* MA7";
$resn_exch{" C5' MA7"}=" C5* MA7"; $resn_exch{" C4' MA7"}=" C4* MA7"; $resn_exch{" O4' MA7"}=" O4* MA7";
$resn_exch{" C1' MA7"}=" C1* MA7"; $resn_exch{" C3' MA7"}=" C3* MA7"; $resn_exch{" O3' MA7"}=" O3* MA7";
$resn_exch{" OP3 MA7"}=" O3P MA7"; $resn_exch{" OP1 MAD"}=" O1P MAD"; $resn_exch{" OP2 MAD"}=" O2P MAD";
$resn_exch{" O5' MAD"}=" O5* MAD"; $resn_exch{" C5' MAD"}=" C5* MAD"; $resn_exch{" C4' MAD"}=" C4* MAD";
$resn_exch{" O4' MAD"}=" O4* MAD"; $resn_exch{" C3' MAD"}=" C3* MAD"; $resn_exch{" O3' MAD"}=" O3* MAD";
$resn_exch{" C2' MAD"}=" C2* MAD"; $resn_exch{" O2' MAD"}=" O2* MAD"; $resn_exch{" C1' MAD"}=" C1* MAD";
$resn_exch{" C5' MAO"}=" C5* MAO"; $resn_exch{" C4' MAO"}=" C4* MAO"; $resn_exch{" O4' MAO"}=" O4* MAO";
$resn_exch{" C1' MAO"}=" C1* MAO"; $resn_exch{" C2' MAO"}=" C2* MAO"; $resn_exch{" O2' MAO"}=" O2* MAO";
$resn_exch{" C3' MAO"}=" C3* MAO"; $resn_exch{" O3' MAO"}=" O3* MAO"; $resn_exch{" O5' MAP"}=" O5* MAP";
$resn_exch{" C5' MAP"}=" C5* MAP"; $resn_exch{" C4' MAP"}=" C4* MAP"; $resn_exch{" O4' MAP"}=" O4* MAP";
$resn_exch{" C3' MAP"}=" C3* MAP"; $resn_exch{" O3' MAP"}=" O3* MAP"; $resn_exch{" C2' MAP"}=" C2* MAP";
$resn_exch{" O2' MAP"}=" O2* MAP"; $resn_exch{" C1' MAP"}=" C1* MAP"; $resn_exch{" C1' MAR"}=" C1* MAR";
$resn_exch{" C2' MAR"}=" C2* MAR"; $resn_exch{" C3' MAR"}=" C3* MAR"; $resn_exch{" N3' MAR"}=" N3* MAR";
$resn_exch{" C4' MAR"}=" C4* MAR"; $resn_exch{" O4' MAR"}=" O4* MAR"; $resn_exch{" C5' MAR"}=" C5* MAR";
$resn_exch{" O5' MAR"}=" O5* MAR"; $resn_exch{" C6' MAR"}=" C6* MAR"; $resn_exch{" C4' MBP"}=" C4* MBP";
$resn_exch{" O4' MBP"}=" O4* MBP"; $resn_exch{" C3' MBP"}=" C3* MBP"; $resn_exch{" C2' MBP"}=" C2* MBP";
$resn_exch{" OXT MBQ"}=" O   MBQ"; $resn_exch{" O   MBQ"}=" O1  MBQ"; $resn_exch{" OP1 MBZ"}=" O1P MBZ";
$resn_exch{" OP2 MBZ"}=" O2P MBZ"; $resn_exch{" OP3 MBZ"}=" O3P MBZ"; $resn_exch{" O5' MBZ"}=" O5* MBZ";
$resn_exch{" O3' MBZ"}=" O3* MBZ"; $resn_exch{" C1' MBZ"}=" C1* MBZ"; $resn_exch{" C2' MBZ"}=" C2* MBZ";
$resn_exch{" C3' MBZ"}=" C3* MBZ"; $resn_exch{" C4' MBZ"}=" C4* MBZ"; $resn_exch{" C5' MBZ"}=" C5* MBZ";
$resn_exch{" O4' MBZ"}=" O4* MBZ"; $resn_exch{" OXT MC1"}=" O1  MC1"; $resn_exch{" OXT MCG"}=" OT  MCG";
$resn_exch{" C1' MCN"}=" C1* MCN"; $resn_exch{" C2D MCN"}=" C2* MCN"; $resn_exch{" O2' MCN"}=" O2* MCN";
$resn_exch{" C3' MCN"}=" C3* MCN"; $resn_exch{" C4D MCN"}=" C4* MCN"; $resn_exch{" O4D MCN"}=" O4* MCN";
$resn_exch{" O3' MCN"}=" O3* MCN"; $resn_exch{" C5' MCN"}=" C5* MCN"; $resn_exch{" O5' MCN"}=" O5* MCN";
$resn_exch{" C1' MCY"}=" C1* MCY"; $resn_exch{" C2' MCY"}=" C2* MCY"; $resn_exch{" C3' MCY"}=" C3* MCY";
$resn_exch{" C4' MCY"}=" C4* MCY"; $resn_exch{" O4' MCY"}=" O4* MCY"; $resn_exch{" O3' MCY"}=" O3* MCY";
$resn_exch{" C5' MCY"}=" C5* MCY"; $resn_exch{" O5' MCY"}=" O5* MCY"; $resn_exch{" O5' MD1"}=" O5* MD1";
$resn_exch{" C5' MD1"}=" C5* MD1"; $resn_exch{" C4' MD1"}=" C4* MD1"; $resn_exch{" O4' MD1"}=" O4* MD1";
$resn_exch{" C1' MD1"}=" C1* MD1"; $resn_exch{" C2' MD1"}=" C2* MD1"; $resn_exch{" O2' MD1"}=" O2* MD1";
$resn_exch{" C3' MD1"}=" C3* MD1"; $resn_exch{" O3' MD1"}=" O3* MD1"; $resn_exch{" P1A MDE"}=" AP1 MDE";
$resn_exch{" O1A MDE"}=" AO1 MDE"; $resn_exch{" O2A MDE"}=" AO2 MDE"; $resn_exch{" O5B MDE"}="AO5* MDE";
$resn_exch{" C5B MDE"}="AC5* MDE"; $resn_exch{" C4B MDE"}="AC4* MDE"; $resn_exch{" O4B MDE"}="AO4* MDE";
$resn_exch{" C1B MDE"}="AC1* MDE"; $resn_exch{" N9A MDE"}=" AN9 MDE"; $resn_exch{" C4A MDE"}=" AC4 MDE";
$resn_exch{" N3A MDE"}=" AN3 MDE"; $resn_exch{" C2A MDE"}=" AC2 MDE"; $resn_exch{" N1A MDE"}=" AN1 MDE";
$resn_exch{" C6A MDE"}=" AC6 MDE"; $resn_exch{" N6A MDE"}=" AN6 MDE"; $resn_exch{" C5A MDE"}=" AC5 MDE";
$resn_exch{" N7A MDE"}=" AN7 MDE"; $resn_exch{" C8A MDE"}=" AC8 MDE"; $resn_exch{" C2B MDE"}="AC2* MDE";
$resn_exch{" O2B MDE"}="AO2* MDE"; $resn_exch{" C3B MDE"}="AC3* MDE"; $resn_exch{" O3B MDE"}="AO3* MDE";
$resn_exch{" P3B MDE"}="AP3* MDE"; $resn_exch{" O7B MDE"}="AO7* MDE"; $resn_exch{" O8B MDE"}="AO8* MDE";
$resn_exch{" O9B MDE"}="AO9* MDE"; $resn_exch{" O3A MDE"}=" AO3 MDE"; $resn_exch{" P2A MDE"}=" AP2 MDE";
$resn_exch{" O4A MDE"}=" AO4 MDE"; $resn_exch{" O5A MDE"}=" AO5 MDE"; $resn_exch{" O6A MDE"}=" AO6 MDE";
$resn_exch{" CCP MDE"}="PC12 MDE"; $resn_exch{" CBP MDE"}="PC11 MDE"; $resn_exch{" CDP MDE"}="PC13 MDE";
$resn_exch{" CEP MDE"}="PC14 MDE"; $resn_exch{" CAP MDE"}="PC10 MDE"; $resn_exch{" OAP MDE"}="PO10 MDE";
$resn_exch{" C9P MDE"}=" PC9 MDE"; $resn_exch{" O9P MDE"}=" PO9 MDE"; $resn_exch{" N8P MDE"}=" PN8 MDE";
$resn_exch{" C7P MDE"}=" PC7 MDE"; $resn_exch{" C6P MDE"}=" PC6 MDE"; $resn_exch{" C5P MDE"}=" PC5 MDE";
$resn_exch{" O5P MDE"}=" PO5 MDE"; $resn_exch{" N4P MDE"}=" PN4 MDE"; $resn_exch{" C3P MDE"}=" PC3 MDE";
$resn_exch{" C2P MDE"}=" PC2 MDE"; $resn_exch{" S1P MDE"}=" PS1 MDE"; $resn_exch{" C1' MDE"}=" C1* MDE";
$resn_exch{" O1' MDE"}=" O1* MDE"; $resn_exch{" C2' MDE"}=" C2* MDE"; $resn_exch{" C3' MDE"}=" C3* MDE";
$resn_exch{" C4' MDE"}=" C4* MDE"; $resn_exch{" C5' MDE"}=" C5* MDE"; $resn_exch{" C6' MDE"}=" C6* MDE";
$resn_exch{" C7' MDE"}=" C7* MDE"; $resn_exch{" C8' MDE"}=" C8* MDE"; $resn_exch{" C9' MDE"}=" C9* MDE";
$resn_exch{" O5' MDG"}=" O5* MDG"; $resn_exch{" C5' MDG"}=" C5* MDG"; $resn_exch{" C4' MDG"}=" C4* MDG";
$resn_exch{" O4' MDG"}=" O4* MDG"; $resn_exch{" C3' MDG"}=" C3* MDG"; $resn_exch{" O3' MDG"}=" O3* MDG";
$resn_exch{" C2' MDG"}=" C2* MDG"; $resn_exch{" O2' MDG"}=" O2* MDG"; $resn_exch{" C1' MDG"}=" C1* MDG";
$resn_exch{" OP1 MEP"}=" O1P MEP"; $resn_exch{" OP2 MEP"}=" O2P MEP"; $resn_exch{" OP3 MEP"}=" O3P MEP";
$resn_exch{" O5' MEP"}=" O5* MEP"; $resn_exch{" C5' MEP"}=" C5* MEP"; $resn_exch{" C4' MEP"}=" C4* MEP";
$resn_exch{" O4' MEP"}=" O4* MEP"; $resn_exch{" C3' MEP"}=" C3* MEP"; $resn_exch{" C2' MEP"}=" C2* MEP";
$resn_exch{" O2' MEP"}=" O2* MEP"; $resn_exch{" C1' MEP"}=" C1* MEP"; $resn_exch{" CA' MEP"}=" CA* MEP";
$resn_exch{" OP3 MG1"}=" O3P MG1"; $resn_exch{" OP1 MG1"}=" O1P MG1"; $resn_exch{" OP2 MG1"}=" O2P MG1";
$resn_exch{" O5' MG1"}=" O5* MG1"; $resn_exch{" C5' MG1"}=" C5* MG1"; $resn_exch{" C4' MG1"}=" C4* MG1";
$resn_exch{" O4' MG1"}=" O4* MG1"; $resn_exch{" C3' MG1"}=" C3* MG1"; $resn_exch{" O3' MG1"}=" O3* MG1";
$resn_exch{" C2' MG1"}=" C2* MG1"; $resn_exch{" C1' MG1"}=" C1* MG1"; $resn_exch{" CAA MGE"}="C10A MGE";
$resn_exch{" CBA MGE"}="C11A MGE"; $resn_exch{" CCA MGE"}="C12A MGE"; $resn_exch{" CDA MGE"}="C13A MGE";
$resn_exch{" CAB MGE"}="C10B MGE"; $resn_exch{" CBB MGE"}="C11B MGE"; $resn_exch{" CCB MGE"}="C12B MGE";
$resn_exch{" CDB MGE"}="C13B MGE"; $resn_exch{" CEB MGE"}="C14B MGE"; $resn_exch{" CFB MGE"}="C15B MGE";
$resn_exch{" CGB MGE"}="C16B MGE"; $resn_exch{" O5' MGO"}=" O5* MGO"; $resn_exch{" C5' MGO"}=" C5* MGO";
$resn_exch{" C4' MGO"}=" C4* MGO"; $resn_exch{" O4' MGO"}=" O4* MGO"; $resn_exch{" C3' MGO"}=" C3* MGO";
$resn_exch{" O3' MGO"}=" O3* MGO"; $resn_exch{" C2' MGO"}=" C2* MGO"; $resn_exch{" O2' MGO"}=" O2* MGO";
$resn_exch{" C1' MGO"}=" C1* MGO"; $resn_exch{" O5' MGT"}=" O5* MGT"; $resn_exch{" C5' MGT"}=" C5* MGT";
$resn_exch{" C4' MGT"}=" C4* MGT"; $resn_exch{" O4' MGT"}=" O4* MGT"; $resn_exch{" C3' MGT"}=" C3* MGT";
$resn_exch{" O3' MGT"}=" O3* MGT"; $resn_exch{" C2' MGT"}=" C2* MGT"; $resn_exch{" O2' MGT"}=" O2* MGT";
$resn_exch{" C1' MGT"}=" C1* MGT"; $resn_exch{" NA  MHM"}=" N_A MHM"; $resn_exch{" NB  MHM"}=" N_B MHM";
$resn_exch{" NC  MHM"}=" N_C MHM"; $resn_exch{" ND  MHM"}=" N_D MHM"; $resn_exch{" C5' MHZ"}=" C5* MHZ";
$resn_exch{" C4' MHZ"}=" C4* MHZ"; $resn_exch{" O4' MHZ"}=" O4* MHZ"; $resn_exch{" C1' MHZ"}=" C1* MHZ";
$resn_exch{" C2' MHZ"}=" C2* MHZ"; $resn_exch{" O2' MHZ"}=" O2* MHZ"; $resn_exch{" C3' MHZ"}=" C3* MHZ";
$resn_exch{" O3' MHZ"}=" O3* MHZ"; $resn_exch{" OP3 MIA"}=" O3P MIA"; $resn_exch{" OP1 MIA"}=" O1P MIA";
$resn_exch{" OP2 MIA"}=" O2P MIA"; $resn_exch{" O5' MIA"}=" O5* MIA"; $resn_exch{" C5' MIA"}=" C5* MIA";
$resn_exch{" C4' MIA"}=" C4* MIA"; $resn_exch{" O4' MIA"}=" O4* MIA"; $resn_exch{" C3' MIA"}=" C3* MIA";
$resn_exch{" O3' MIA"}=" O3* MIA"; $resn_exch{" C2' MIA"}=" C2* MIA"; $resn_exch{" O2' MIA"}=" O2* MIA";
$resn_exch{" C1' MIA"}=" C1* MIA"; $resn_exch{" C3X MMT"}="CO3* MMT"; $resn_exch{" O5' MMT"}=" O5* MMT";
$resn_exch{" C5' MMT"}=" C5* MMT"; $resn_exch{" C4' MMT"}=" C4* MMT"; $resn_exch{" O4' MMT"}=" O4* MMT";
$resn_exch{" C1' MMT"}=" C1* MMT"; $resn_exch{" C2' MMT"}=" C2* MMT"; $resn_exch{" C3' MMT"}=" C3* MMT";
$resn_exch{" O3' MMT"}=" O3* MMT"; $resn_exch{" C1' MNG"}=" C1* MNG"; $resn_exch{" C2' MNG"}=" C2* MNG";
$resn_exch{" C3' MNG"}=" C3* MNG"; $resn_exch{" C4' MNG"}=" C4* MNG"; $resn_exch{" C5' MNG"}=" C5* MNG";
$resn_exch{" O1' MNG"}=" O1* MNG"; $resn_exch{" O2' MNG"}=" O2* MNG"; $resn_exch{" O4' MNG"}=" O4* MNG";
$resn_exch{" NA  MNH"}=" N_A MNH"; $resn_exch{" NB  MNH"}=" N_B MNH"; $resn_exch{" NC  MNH"}=" N_C MNH";
$resn_exch{" ND  MNH"}=" N_D MNH"; $resn_exch{" NA  MNR"}=" N_A MNR"; $resn_exch{" NB  MNR"}=" N_B MNR";
$resn_exch{" NC  MNR"}=" N_C MNR"; $resn_exch{" ND  MNR"}=" N_D MNR"; $resn_exch{" O5' MNT"}=" O5* MNT";
$resn_exch{" C5B MNT"}=" C5* MNT"; $resn_exch{" C4B MNT"}=" C4* MNT"; $resn_exch{" O4' MNT"}=" O4* MNT";
$resn_exch{" C3B MNT"}=" C3* MNT"; $resn_exch{" O3' MNT"}=" O3* MNT"; $resn_exch{" C2B MNT"}=" C2* MNT";
$resn_exch{" C1B MNT"}=" C1* MNT"; $resn_exch{" OP2 MNU"}=" O2P MNU"; $resn_exch{" OP1 MNU"}=" O1P MNU";
$resn_exch{" O5' MNU"}=" O5* MNU"; $resn_exch{" C5' MNU"}=" C5* MNU"; $resn_exch{" C4' MNU"}=" C4* MNU";
$resn_exch{" O4' MNU"}=" O4* MNU"; $resn_exch{" C1' MNU"}=" C1* MNU"; $resn_exch{" C2' MNU"}=" C2* MNU";
$resn_exch{" O2' MNU"}=" O2* MNU"; $resn_exch{" C3' MNU"}=" C3* MNU"; $resn_exch{" O3' MNU"}=" O3* MNU";
$resn_exch{" OP3 MNU"}=" O3P MNU"; $resn_exch{" CHX MOB"}="3CH3 MOB"; $resn_exch{" CHZ MOB"}="5CH3 MOB";
$resn_exch{" O5' MOD"}=" O5* MOD"; $resn_exch{" C5' MOD"}=" C5* MOD"; $resn_exch{" C4' MOD"}=" C4* MOD";
$resn_exch{" O4' MOD"}=" O4* MOD"; $resn_exch{" C3' MOD"}=" C3* MOD"; $resn_exch{" O3' MOD"}=" O3* MOD";
$resn_exch{" C2' MOD"}=" C2* MOD"; $resn_exch{" O2' MOD"}=" O2* MOD"; $resn_exch{" C1' MOD"}=" C1* MOD";
$resn_exch{" O2' MOE"}=" O2* MOE"; $resn_exch{" CA' MOE"}=" CA* MOE"; $resn_exch{" CB' MOE"}=" CB* MOE";
$resn_exch{" OC' MOE"}=" OC* MOE"; $resn_exch{" CD' MOE"}=" CD* MOE"; $resn_exch{"CL33 MON"}="3CL3 MON";
$resn_exch{" H11 MPA"}="1H1  MPA"; $resn_exch{" H11 MPC"}="1H1  MPC"; $resn_exch{" CXD MPG"}="1C12 MPG";
$resn_exch{" CX3 MPG"}="1C11 MPG"; $resn_exch{" CDB MQQ"}="1CD1 MQQ"; $resn_exch{" CDC MQQ"}="1CD2 MQQ";
$resn_exch{" C1' MRC"}=" C1* MRC"; $resn_exch{" C2' MRC"}=" C2* MRC"; $resn_exch{" C3' MRC"}=" C3* MRC";
$resn_exch{" C4' MRC"}=" C4* MRC"; $resn_exch{" C5' MRC"}=" C5* MRC"; $resn_exch{" C6' MRC"}=" C6* MRC";
$resn_exch{" C7' MRC"}=" C7* MRC"; $resn_exch{" C8' MRC"}=" C8* MRC"; $resn_exch{" C9' MRC"}=" C9* MRC";
$resn_exch{" OP1 MRG"}=" O1P MRG"; $resn_exch{" OP2 MRG"}=" O2P MRG"; $resn_exch{" O5' MRG"}=" O5* MRG";
$resn_exch{" C2' MRG"}=" C2* MRG"; $resn_exch{" C5' MRG"}=" C5* MRG"; $resn_exch{" C4' MRG"}=" C4* MRG";
$resn_exch{" O4' MRG"}=" O4* MRG"; $resn_exch{" C1' MRG"}=" C1* MRG"; $resn_exch{" C3' MRG"}=" C3* MRG";
$resn_exch{" O3' MRG"}=" O3* MRG"; $resn_exch{" OP3 MRG"}=" O3P MRG"; $resn_exch{" O7A MRR"}=" AO7 MRR";
$resn_exch{" P3B MRR"}="AP3* MRR"; $resn_exch{" O8A MRR"}=" AO8 MRR"; $resn_exch{" O9A MRR"}=" AO9 MRR";
$resn_exch{" O3B MRR"}="AO3* MRR"; $resn_exch{" C3B MRR"}="AC3* MRR"; $resn_exch{" C2B MRR"}="AC2* MRR";
$resn_exch{" O2B MRR"}="AO2* MRR"; $resn_exch{" C1B MRR"}="AC1* MRR"; $resn_exch{" N9A MRR"}=" AN9 MRR";
$resn_exch{" C4A MRR"}=" AC4 MRR"; $resn_exch{" N3A MRR"}=" AN3 MRR"; $resn_exch{" C2A MRR"}=" AC2 MRR";
$resn_exch{" N1A MRR"}=" AN1 MRR"; $resn_exch{" C6A MRR"}=" AC6 MRR"; $resn_exch{" N6A MRR"}=" AN6 MRR";
$resn_exch{" C5A MRR"}=" AC5 MRR"; $resn_exch{" N7A MRR"}=" AN7 MRR"; $resn_exch{" C8A MRR"}=" AC8 MRR";
$resn_exch{" O4B MRR"}="AO4* MRR"; $resn_exch{" C4B MRR"}="AC4* MRR"; $resn_exch{" C5B MRR"}="AC5* MRR";
$resn_exch{" O5B MRR"}="AO5* MRR"; $resn_exch{" P1A MRR"}=" AP1 MRR"; $resn_exch{" O1A MRR"}=" AO1 MRR";
$resn_exch{" O2A MRR"}=" AO2 MRR"; $resn_exch{" O3A MRR"}=" AO3 MRR"; $resn_exch{" P2A MRR"}=" AP2 MRR";
$resn_exch{" O4A MRR"}=" AO4 MRR"; $resn_exch{" O5A MRR"}=" AO5 MRR"; $resn_exch{" O6A MRR"}=" AO6 MRR";
$resn_exch{" CCP MRR"}="PC12 MRR"; $resn_exch{" CBP MRR"}="PC11 MRR"; $resn_exch{" CDP MRR"}="PC13 MRR";
$resn_exch{" CEP MRR"}="PC14 MRR"; $resn_exch{" CAP MRR"}="PC10 MRR"; $resn_exch{" OAP MRR"}="PO10 MRR";
$resn_exch{" C9P MRR"}=" PC9 MRR"; $resn_exch{" O9P MRR"}=" PO9 MRR"; $resn_exch{" N8P MRR"}=" PN8 MRR";
$resn_exch{" C7P MRR"}=" PC7 MRR"; $resn_exch{" C6P MRR"}=" PC6 MRR"; $resn_exch{" C5P MRR"}=" PC5 MRR";
$resn_exch{" O5P MRR"}=" PO5 MRR"; $resn_exch{" N4P MRR"}=" PN4 MRR"; $resn_exch{" C3P MRR"}=" PC3 MRR";
$resn_exch{" C2P MRR"}=" PC2 MRR"; $resn_exch{" S1P MRR"}=" PS1 MRR"; $resn_exch{" O7A MRS"}=" AO7 MRS";
$resn_exch{" P3B MRS"}="AP3* MRS"; $resn_exch{" O8A MRS"}=" AO8 MRS"; $resn_exch{" O9A MRS"}=" AO9 MRS";
$resn_exch{" O3B MRS"}="AO3* MRS"; $resn_exch{" C3B MRS"}="AC3* MRS"; $resn_exch{" C2B MRS"}="AC2* MRS";
$resn_exch{" O2B MRS"}="AO2* MRS"; $resn_exch{" C1B MRS"}="AC1* MRS"; $resn_exch{" N9A MRS"}=" AN9 MRS";
$resn_exch{" C4A MRS"}=" AC4 MRS"; $resn_exch{" N3A MRS"}=" AN3 MRS"; $resn_exch{" C2A MRS"}=" AC2 MRS";
$resn_exch{" N1A MRS"}=" AN1 MRS"; $resn_exch{" C6A MRS"}=" AC6 MRS"; $resn_exch{" N6A MRS"}=" AN6 MRS";
$resn_exch{" C5A MRS"}=" AC5 MRS"; $resn_exch{" N7A MRS"}=" AN7 MRS"; $resn_exch{" C8A MRS"}=" AC8 MRS";
$resn_exch{" O4B MRS"}="AO4* MRS"; $resn_exch{" C4B MRS"}="AC4* MRS"; $resn_exch{" C5B MRS"}="AC5* MRS";
$resn_exch{" O5B MRS"}="AO5* MRS"; $resn_exch{" P1A MRS"}=" AP1 MRS"; $resn_exch{" O1A MRS"}=" AO1 MRS";
$resn_exch{" O2A MRS"}=" AO2 MRS"; $resn_exch{" O3A MRS"}=" AO3 MRS"; $resn_exch{" P2A MRS"}=" AP2 MRS";
$resn_exch{" O4A MRS"}=" AO4 MRS"; $resn_exch{" O5A MRS"}=" AO5 MRS"; $resn_exch{" O6A MRS"}=" AO6 MRS";
$resn_exch{" CCP MRS"}="PC12 MRS"; $resn_exch{" CBP MRS"}="PC11 MRS"; $resn_exch{" CDP MRS"}="PC13 MRS";
$resn_exch{" CEP MRS"}="PC14 MRS"; $resn_exch{" CAP MRS"}="PC10 MRS"; $resn_exch{" OAP MRS"}="PO10 MRS";
$resn_exch{" C9P MRS"}=" PC9 MRS"; $resn_exch{" O9P MRS"}=" PO9 MRS"; $resn_exch{" N8P MRS"}=" PN8 MRS";
$resn_exch{" C7P MRS"}=" PC7 MRS"; $resn_exch{" C6P MRS"}=" PC6 MRS"; $resn_exch{" C5P MRS"}=" PC5 MRS";
$resn_exch{" O5P MRS"}=" PO5 MRS"; $resn_exch{" N4P MRS"}=" PN4 MRS"; $resn_exch{" C3P MRS"}=" PC3 MRS";
$resn_exch{" C2P MRS"}=" PC2 MRS"; $resn_exch{" S1P MRS"}=" PS1 MRS"; $resn_exch{" O5' MSG"}=" O5* MSG";
$resn_exch{" C5' MSG"}=" C5* MSG"; $resn_exch{" C4' MSG"}=" C4* MSG"; $resn_exch{" O4' MSG"}=" O4* MSG";
$resn_exch{" C1' MSG"}=" C1* MSG"; $resn_exch{" C2' MSG"}=" C2* MSG"; $resn_exch{" O2' MSG"}=" O2* MSG";
$resn_exch{" C3' MSG"}=" C3* MSG"; $resn_exch{" O3' MSG"}=" O3* MSG"; $resn_exch{" O5' MSP"}=" O5* MSP";
$resn_exch{" C5' MSP"}=" C5* MSP"; $resn_exch{" C4' MSP"}=" C4* MSP"; $resn_exch{" O4' MSP"}=" O4* MSP";
$resn_exch{" C3' MSP"}=" C3* MSP"; $resn_exch{" O3' MSP"}=" O3* MSP"; $resn_exch{" C2' MSP"}=" C2* MSP";
$resn_exch{" O2' MSP"}=" O2* MSP"; $resn_exch{" C1' MSP"}=" C1* MSP"; $resn_exch{" S5' MTA"}=" S5* MTA";
$resn_exch{" C5' MTA"}=" C5* MTA"; $resn_exch{" C4' MTA"}=" C4* MTA"; $resn_exch{" O4' MTA"}=" O4* MTA";
$resn_exch{" C2' MTA"}=" C2* MTA"; $resn_exch{" O2' MTA"}=" O2* MTA"; $resn_exch{" C3' MTA"}=" C3* MTA";
$resn_exch{" O3' MTA"}=" O3* MTA"; $resn_exch{" C1' MTA"}=" C1* MTA"; $resn_exch{" CHV MTB"}="1CH3 MTB";
$resn_exch{" CHW MTB"}="2CH3 MTB"; $resn_exch{" CHX MTB"}="3CH3 MTB"; $resn_exch{" S5' MTH"}=" S5* MTH";
$resn_exch{" C5' MTH"}=" C5* MTH"; $resn_exch{" C4' MTH"}=" C4* MTH"; $resn_exch{" O4' MTH"}=" O4* MTH";
$resn_exch{" C2' MTH"}=" C2* MTH"; $resn_exch{" O2' MTH"}=" O2* MTH"; $resn_exch{" C3' MTH"}=" C3* MTH";
$resn_exch{" O3' MTH"}=" O3* MTH"; $resn_exch{" C1' MTH"}=" C1* MTH"; $resn_exch{" S5' MTI"}=" S5* MTI";
$resn_exch{" C5' MTI"}=" C5* MTI"; $resn_exch{" C4' MTI"}=" C4* MTI"; $resn_exch{" N4' MTI"}=" N4* MTI";
$resn_exch{" C2' MTI"}=" C2* MTI"; $resn_exch{" O2' MTI"}=" O2* MTI"; $resn_exch{" C3' MTI"}=" C3* MTI";
$resn_exch{" O3' MTI"}=" O3* MTI"; $resn_exch{" C1' MTI"}=" C1* MTI"; $resn_exch{" S5' MTM"}=" S5* MTM";
$resn_exch{" C5' MTM"}=" C5* MTM"; $resn_exch{" C4' MTM"}=" C4* MTM"; $resn_exch{" N4' MTM"}=" N4* MTM";
$resn_exch{" C2' MTM"}=" C2* MTM"; $resn_exch{" O2' MTM"}=" O2* MTM"; $resn_exch{" C3' MTM"}=" C3* MTM";
$resn_exch{" O3' MTM"}=" O3* MTM"; $resn_exch{" C1' MTM"}=" C1* MTM"; $resn_exch{" OP1 MTR"}=" O1P MTR";
$resn_exch{" OP2 MTR"}=" O2P MTR"; $resn_exch{" O5' MTR"}=" O5* MTR"; $resn_exch{" C2' MTR"}=" C2* MTR";
$resn_exch{" C5' MTR"}=" C5* MTR"; $resn_exch{" C4' MTR"}=" C4* MTR"; $resn_exch{" O4' MTR"}=" O4* MTR";
$resn_exch{" C1' MTR"}=" C1* MTR"; $resn_exch{" C3' MTR"}=" C3* MTR"; $resn_exch{" O3' MTR"}=" O3* MTR";
$resn_exch{" OP3 MTR"}=" O3P MTR"; $resn_exch{" OP1 MTU"}=" O1P MTU"; $resn_exch{" OP2 MTU"}=" O2P MTU";
$resn_exch{" O5' MTU"}=" O5* MTU"; $resn_exch{" C5' MTU"}=" C5* MTU"; $resn_exch{" C4' MTU"}=" C4* MTU";
$resn_exch{" O4' MTU"}=" O4* MTU"; $resn_exch{" C1' MTU"}=" C1* MTU"; $resn_exch{" C2' MTU"}=" C2* MTU";
$resn_exch{" O2' MTU"}=" O2* MTU"; $resn_exch{" C3' MTU"}=" C3* MTU"; $resn_exch{" O3' MTU"}=" O3* MTU";
$resn_exch{" O5' MZP"}=" O5* MZP"; $resn_exch{" C5' MZP"}=" C5* MZP"; $resn_exch{" C4' MZP"}=" C4* MZP";
$resn_exch{" O4' MZP"}=" O4* MZP"; $resn_exch{" C3' MZP"}=" C3* MZP"; $resn_exch{" O3' MZP"}=" O3* MZP";
$resn_exch{" C2' MZP"}=" C2* MZP"; $resn_exch{" O2' MZP"}=" O2* MZP"; $resn_exch{" C1' MZP"}=" C1* MZP";
$resn_exch{" OP3   N"}=" O3P   N"; $resn_exch{" OP1   N"}=" O1P   N"; $resn_exch{" OP2   N"}=" O2P   N";
$resn_exch{" O5'   N"}=" O5*   N"; $resn_exch{" C5'   N"}=" C5*   N"; $resn_exch{" C4'   N"}=" C4*   N";
$resn_exch{" O4'   N"}=" O4*   N"; $resn_exch{" C3'   N"}=" C3*   N"; $resn_exch{" O3'   N"}=" O3*   N";
$resn_exch{" C2'   N"}=" C2*   N"; $resn_exch{" O2'   N"}=" O2*   N"; $resn_exch{" C1'   N"}=" C1*   N";
$resn_exch{" C1' N1C"}=" C1* N1C"; $resn_exch{" O4' N1C"}=" O4* N1C"; $resn_exch{" C2' N1C"}=" C2* N1C";
$resn_exch{" O2' N1C"}=" O2* N1C"; $resn_exch{" C3' N1C"}=" C3* N1C"; $resn_exch{" O3' N1C"}=" O3* N1C";
$resn_exch{" C4' N1C"}=" C4* N1C"; $resn_exch{" C5' N1C"}=" C5* N1C"; $resn_exch{" O5' N1C"}=" O5* N1C";
$resn_exch{" CV' N1H"}="C31' N1H"; $resn_exch{" CW' N1H"}="C32' N1H"; $resn_exch{" OX' N1H"}="O33' N1H";
$resn_exch{" OY' N1H"}="O34' N1H"; $resn_exch{" C1B N1H"}="C1'' N1H"; $resn_exch{" C2B N1H"}="C2'' N1H";
$resn_exch{" C3B N1H"}="C3'' N1H"; $resn_exch{" C4B N1H"}="C4'' N1H"; $resn_exch{" C5B N1H"}="C5'' N1H";
$resn_exch{" N5B N1H"}="N5'' N1H"; $resn_exch{" C6B N1H"}="C6'' N1H"; $resn_exch{" N5' N5A"}=" N5* N5A";
$resn_exch{" C5' N5A"}=" C5* N5A"; $resn_exch{" O5' N5A"}=" O5* N5A"; $resn_exch{" C4' N5A"}=" C4* N5A";
$resn_exch{" O4' N5A"}=" O4* N5A"; $resn_exch{" C3' N5A"}=" C3* N5A"; $resn_exch{" O3' N5A"}=" O3* N5A";
$resn_exch{" C2' N5A"}=" C2* N5A"; $resn_exch{" O2' N5A"}=" O2* N5A"; $resn_exch{" C1' N5A"}=" C1* N5A";
$resn_exch{" C1' N5I"}=" C1* N5I"; $resn_exch{" O4' N5I"}=" O4* N5I"; $resn_exch{" C4' N5I"}=" C4* N5I";
$resn_exch{" C3' N5I"}=" C3* N5I"; $resn_exch{" O3' N5I"}=" O3* N5I"; $resn_exch{" C2' N5I"}=" C2* N5I";
$resn_exch{" C5' N5I"}=" C5* N5I"; $resn_exch{" O5' N5I"}=" O5* N5I"; $resn_exch{" N5' N5O"}=" N5* N5O";
$resn_exch{" C5' N5O"}=" C5* N5O"; $resn_exch{" O5' N5O"}=" O5* N5O"; $resn_exch{" C4' N5O"}=" C4* N5O";
$resn_exch{" O4' N5O"}=" O4* N5O"; $resn_exch{" C3' N5O"}=" C3* N5O"; $resn_exch{" O3' N5O"}=" O3* N5O";
$resn_exch{" C2' N5O"}=" C2* N5O"; $resn_exch{" O2' N5O"}=" O2* N5O"; $resn_exch{" C1' N5O"}=" C1* N5O";
$resn_exch{" C1' N5P"}=" C1* N5P"; $resn_exch{" O4' N5P"}=" O4* N5P"; $resn_exch{" C4' N5P"}=" C4* N5P";
$resn_exch{" C3' N5P"}=" C3* N5P"; $resn_exch{" O3' N5P"}=" O3* N5P"; $resn_exch{" C2' N5P"}=" C2* N5P";
$resn_exch{" C5' N5P"}=" C5* N5P"; $resn_exch{" O5' N5P"}=" O5* N5P"; $resn_exch{" OP1 N6G"}=" O1P N6G";
$resn_exch{" OP2 N6G"}=" O2P N6G"; $resn_exch{" O5' N6G"}=" O5* N6G"; $resn_exch{" C5' N6G"}=" C5* N6G";
$resn_exch{" C4' N6G"}=" C4* N6G"; $resn_exch{" O4' N6G"}=" O4* N6G"; $resn_exch{" C1' N6G"}=" C1* N6G";
$resn_exch{" C2' N6G"}=" C2* N6G"; $resn_exch{" O2' N6G"}=" O2* N6G"; $resn_exch{" C3' N6G"}=" C3* N6G";
$resn_exch{" O3' N6G"}=" O3* N6G"; $resn_exch{" OP3 N6G"}=" O3P N6G"; $resn_exch{" PA  NA7"}=" AP  NA7";
$resn_exch{" O1A NA7"}=" AO1 NA7"; $resn_exch{" O2A NA7"}=" AO2 NA7"; $resn_exch{" O5B NA7"}="AO5* NA7";
$resn_exch{" C5B NA7"}="AC5* NA7"; $resn_exch{" C4B NA7"}="AC4* NA7"; $resn_exch{" O4B NA7"}="AO4* NA7";
$resn_exch{" C3B NA7"}="AC3* NA7"; $resn_exch{" O3B NA7"}="AO3* NA7"; $resn_exch{" C2B NA7"}="AC2* NA7";
$resn_exch{" O2B NA7"}="AO2* NA7"; $resn_exch{" C1B NA7"}="AC1* NA7"; $resn_exch{" N9A NA7"}=" AN9 NA7";
$resn_exch{" C8A NA7"}=" AC8 NA7"; $resn_exch{" N7A NA7"}=" AN7 NA7"; $resn_exch{" C5A NA7"}=" AC5 NA7";
$resn_exch{" C6A NA7"}=" AC6 NA7"; $resn_exch{" N6A NA7"}=" AN6 NA7"; $resn_exch{" N1A NA7"}=" AN1 NA7";
$resn_exch{" C2A NA7"}=" AC2 NA7"; $resn_exch{" N3A NA7"}=" AN3 NA7"; $resn_exch{" C4A NA7"}=" AC4 NA7";
$resn_exch{" PN  NA7"}=" NP  NA7"; $resn_exch{" O1N NA7"}=" NO1 NA7"; $resn_exch{" O2N NA7"}=" NO2 NA7";
$resn_exch{" O5D NA7"}="NO5* NA7"; $resn_exch{" C5D NA7"}="NC5* NA7"; $resn_exch{" C4D NA7"}="NC4* NA7";
$resn_exch{" O4D NA7"}="NO4* NA7"; $resn_exch{" C3D NA7"}="NC3* NA7"; $resn_exch{" O3D NA7"}="NO3* NA7";
$resn_exch{" C2D NA7"}="NC2* NA7"; $resn_exch{" O2D NA7"}="NO2* NA7"; $resn_exch{" C1D NA7"}="NC1* NA7";
$resn_exch{" P2B NA7"}="AP2* NA7"; $resn_exch{" O1X NA7"}="AOP1 NA7"; $resn_exch{" O2X NA7"}="AOP2 NA7";
$resn_exch{" O3X NA7"}="AOP3 NA7"; $resn_exch{" PA  NAC"}=" AP  NAC"; $resn_exch{" O1A NAC"}=" AO1 NAC";
$resn_exch{" O2A NAC"}=" AO2 NAC"; $resn_exch{" O5B NAC"}="AO5* NAC"; $resn_exch{" C5B NAC"}="AC5* NAC";
$resn_exch{" C4B NAC"}="AC4* NAC"; $resn_exch{" O4B NAC"}="AO4* NAC"; $resn_exch{" C3B NAC"}="AC3* NAC";
$resn_exch{" O3B NAC"}="AO3* NAC"; $resn_exch{" C2B NAC"}="AC2* NAC"; $resn_exch{" O2B NAC"}="AO2* NAC";
$resn_exch{" C1B NAC"}="AC1* NAC"; $resn_exch{" N9A NAC"}=" AN9 NAC"; $resn_exch{" C8A NAC"}=" AC8 NAC";
$resn_exch{" N7A NAC"}=" AN7 NAC"; $resn_exch{" C5A NAC"}=" AC5 NAC"; $resn_exch{" C6A NAC"}=" AC6 NAC";
$resn_exch{" N6A NAC"}=" AN6 NAC"; $resn_exch{" N1A NAC"}=" AN1 NAC"; $resn_exch{" C2A NAC"}=" AC2 NAC";
$resn_exch{" N3A NAC"}=" AN3 NAC"; $resn_exch{" C4A NAC"}=" AC4 NAC"; $resn_exch{" PN  NAC"}=" NP  NAC";
$resn_exch{" O1N NAC"}=" NO1 NAC"; $resn_exch{" O2N NAC"}=" NO2 NAC"; $resn_exch{" O5D NAC"}="NO5* NAC";
$resn_exch{" C5D NAC"}="NC5* NAC"; $resn_exch{" C4D NAC"}="NC4* NAC"; $resn_exch{" O4D NAC"}="NO4* NAC";
$resn_exch{" C3D NAC"}="NC3* NAC"; $resn_exch{" O3D NAC"}="NO3* NAC"; $resn_exch{" C2D NAC"}="NC2* NAC";
$resn_exch{" O2D NAC"}="NO2* NAC"; $resn_exch{" C1D NAC"}="NC1* NAC"; $resn_exch{" N1N NAC"}=" NN1 NAC";
$resn_exch{" C2N NAC"}=" NC2 NAC"; $resn_exch{" C3N NAC"}=" NC3 NAC"; $resn_exch{" C7N NAC"}=" NC7 NAC";
$resn_exch{" O7N NAC"}=" NO7 NAC"; $resn_exch{" C8N NAC"}=" NC8 NAC"; $resn_exch{" C4N NAC"}=" NC4 NAC";
$resn_exch{" C5N NAC"}=" NC5 NAC"; $resn_exch{" C6N NAC"}=" NC6 NAC"; $resn_exch{" PA  NAD"}=" AP  NAD";
$resn_exch{" O1A NAD"}=" AO1 NAD"; $resn_exch{" O2A NAD"}=" AO2 NAD"; $resn_exch{" O5B NAD"}="AO5* NAD";
$resn_exch{" C5B NAD"}="AC5* NAD"; $resn_exch{" C4B NAD"}="AC4* NAD"; $resn_exch{" O4B NAD"}="AO4* NAD";
$resn_exch{" C3B NAD"}="AC3* NAD"; $resn_exch{" O3B NAD"}="AO3* NAD"; $resn_exch{" C2B NAD"}="AC2* NAD";
$resn_exch{" O2B NAD"}="AO2* NAD"; $resn_exch{" C1B NAD"}="AC1* NAD"; $resn_exch{" N9A NAD"}=" AN9 NAD";
$resn_exch{" C8A NAD"}=" AC8 NAD"; $resn_exch{" N7A NAD"}=" AN7 NAD"; $resn_exch{" C5A NAD"}=" AC5 NAD";
$resn_exch{" C6A NAD"}=" AC6 NAD"; $resn_exch{" N6A NAD"}=" AN6 NAD"; $resn_exch{" N1A NAD"}=" AN1 NAD";
$resn_exch{" C2A NAD"}=" AC2 NAD"; $resn_exch{" N3A NAD"}=" AN3 NAD"; $resn_exch{" C4A NAD"}=" AC4 NAD";
$resn_exch{" PN  NAD"}=" NP  NAD"; $resn_exch{" O1N NAD"}=" NO1 NAD"; $resn_exch{" O2N NAD"}=" NO2 NAD";
$resn_exch{" O5D NAD"}="NO5* NAD"; $resn_exch{" C5D NAD"}="NC5* NAD"; $resn_exch{" C4D NAD"}="NC4* NAD";
$resn_exch{" O4D NAD"}="NO4* NAD"; $resn_exch{" C3D NAD"}="NC3* NAD"; $resn_exch{" O3D NAD"}="NO3* NAD";
$resn_exch{" C2D NAD"}="NC2* NAD"; $resn_exch{" O2D NAD"}="NO2* NAD"; $resn_exch{" C1D NAD"}="NC1* NAD";
$resn_exch{" N1N NAD"}=" NN1 NAD"; $resn_exch{" C2N NAD"}=" NC2 NAD"; $resn_exch{" C3N NAD"}=" NC3 NAD";
$resn_exch{" C7N NAD"}=" NC7 NAD"; $resn_exch{" O7N NAD"}=" NO7 NAD"; $resn_exch{" N7N NAD"}=" NN7 NAD";
$resn_exch{" C4N NAD"}=" NC4 NAD"; $resn_exch{" C5N NAD"}=" NC5 NAD"; $resn_exch{" C6N NAD"}=" NC6 NAD";
$resn_exch{" O5B NAE"}=" O5* NAE"; $resn_exch{" C5B NAE"}=" C5* NAE"; $resn_exch{" C4B NAE"}=" C4* NAE";
$resn_exch{" O4B NAE"}=" O4* NAE"; $resn_exch{" C3B NAE"}=" C3* NAE"; $resn_exch{" O3B NAE"}=" O3* NAE";
$resn_exch{" C2B NAE"}=" C2* NAE"; $resn_exch{" O2B NAE"}=" O2* NAE"; $resn_exch{" C1B NAE"}=" C1* NAE";
$resn_exch{" PA  NAH"}=" AP  NAH"; $resn_exch{" O1A NAH"}=" AO1 NAH"; $resn_exch{" O2A NAH"}=" AO2 NAH";
$resn_exch{" O5B NAH"}="AO5* NAH"; $resn_exch{" C5B NAH"}="AC5* NAH"; $resn_exch{" C4B NAH"}="AC4* NAH";
$resn_exch{" O4B NAH"}="AO4* NAH"; $resn_exch{" C3B NAH"}="AC3* NAH"; $resn_exch{" O3B NAH"}="AO3* NAH";
$resn_exch{" C2B NAH"}="AC2* NAH"; $resn_exch{" O2B NAH"}="AO2* NAH"; $resn_exch{" C1B NAH"}="AC1* NAH";
$resn_exch{" N9A NAH"}=" AN9 NAH"; $resn_exch{" C8A NAH"}=" AC8 NAH"; $resn_exch{" N7A NAH"}=" AN7 NAH";
$resn_exch{" C5A NAH"}=" AC5 NAH"; $resn_exch{" C6A NAH"}=" AC6 NAH"; $resn_exch{" N6A NAH"}=" AN6 NAH";
$resn_exch{" N1A NAH"}=" AN1 NAH"; $resn_exch{" C2A NAH"}=" AC2 NAH"; $resn_exch{" N3A NAH"}=" AN3 NAH";
$resn_exch{" C4A NAH"}=" AC4 NAH"; $resn_exch{" PN  NAH"}=" NP  NAH"; $resn_exch{" O1N NAH"}=" NO1 NAH";
$resn_exch{" O2N NAH"}=" NO2 NAH"; $resn_exch{" O5D NAH"}="NO5* NAH"; $resn_exch{" C5D NAH"}="NC5* NAH";
$resn_exch{" C4D NAH"}="NC4* NAH"; $resn_exch{" O4D NAH"}="NO4* NAH"; $resn_exch{" C3D NAH"}="NC3* NAH";
$resn_exch{" O3D NAH"}="NO3* NAH"; $resn_exch{" C2D NAH"}="NC2* NAH"; $resn_exch{" O2D NAH"}="NO2* NAH";
$resn_exch{" C1D NAH"}="NC1* NAH"; $resn_exch{" N1N NAH"}=" NN1 NAH"; $resn_exch{" C2N NAH"}=" NC2 NAH";
$resn_exch{" C3N NAH"}=" NC3 NAH"; $resn_exch{" C7N NAH"}=" NC7 NAH"; $resn_exch{" O7N NAH"}=" NO7 NAH";
$resn_exch{" N7N NAH"}=" NN7 NAH"; $resn_exch{" C4N NAH"}=" NC4 NAH"; $resn_exch{" C5N NAH"}=" NC5 NAH";
$resn_exch{" C6N NAH"}=" NC6 NAH"; $resn_exch{" PA  NAI"}=" AP  NAI"; $resn_exch{" O1A NAI"}=" AO1 NAI";
$resn_exch{" O2A NAI"}=" AO2 NAI"; $resn_exch{" O5B NAI"}="AO5* NAI"; $resn_exch{" C5B NAI"}="AC5* NAI";
$resn_exch{" C4B NAI"}="AC4* NAI"; $resn_exch{" O4B NAI"}="AO4* NAI"; $resn_exch{" C3B NAI"}="AC3* NAI";
$resn_exch{" O3B NAI"}="AO3* NAI"; $resn_exch{" C2B NAI"}="AC2* NAI"; $resn_exch{" O2B NAI"}="AO2* NAI";
$resn_exch{" C1B NAI"}="AC1* NAI"; $resn_exch{" N9A NAI"}=" AN9 NAI"; $resn_exch{" C8A NAI"}=" AC8 NAI";
$resn_exch{" N7A NAI"}=" AN7 NAI"; $resn_exch{" C5A NAI"}=" AC5 NAI"; $resn_exch{" C6A NAI"}=" AC6 NAI";
$resn_exch{" N6A NAI"}=" AN6 NAI"; $resn_exch{" N1A NAI"}=" AN1 NAI"; $resn_exch{" C2A NAI"}=" AC2 NAI";
$resn_exch{" N3A NAI"}=" AN3 NAI"; $resn_exch{" C4A NAI"}=" AC4 NAI"; $resn_exch{" PN  NAI"}=" NP  NAI";
$resn_exch{" O1N NAI"}=" NO1 NAI"; $resn_exch{" O2N NAI"}=" NO2 NAI"; $resn_exch{" O5D NAI"}="NO5* NAI";
$resn_exch{" C5D NAI"}="NC5* NAI"; $resn_exch{" C4D NAI"}="NC4* NAI"; $resn_exch{" O4D NAI"}="NO4* NAI";
$resn_exch{" C3D NAI"}="NC3* NAI"; $resn_exch{" O3D NAI"}="NO3* NAI"; $resn_exch{" C2D NAI"}="NC2* NAI";
$resn_exch{" O2D NAI"}="NO2* NAI"; $resn_exch{" C1D NAI"}="NC1* NAI"; $resn_exch{" N1N NAI"}=" NN1 NAI";
$resn_exch{" C2N NAI"}=" NC2 NAI"; $resn_exch{" C3N NAI"}=" NC3 NAI"; $resn_exch{" C7N NAI"}=" NC7 NAI";
$resn_exch{" O7N NAI"}=" NO7 NAI"; $resn_exch{" N7N NAI"}=" NN7 NAI"; $resn_exch{" C4N NAI"}=" NC4 NAI";
$resn_exch{" C5N NAI"}=" NC5 NAI"; $resn_exch{" C6N NAI"}=" NC6 NAI"; $resn_exch{" PA  NAJ"}=" AP  NAJ";
$resn_exch{" O1A NAJ"}=" AO1 NAJ"; $resn_exch{" O2A NAJ"}=" AO2 NAJ"; $resn_exch{" O5B NAJ"}="AO5* NAJ";
$resn_exch{" C5B NAJ"}="AC5* NAJ"; $resn_exch{" C4B NAJ"}="AC4* NAJ"; $resn_exch{" O4B NAJ"}="AO4* NAJ";
$resn_exch{" C3B NAJ"}="AC3* NAJ"; $resn_exch{" O3B NAJ"}="AO3* NAJ"; $resn_exch{" C2B NAJ"}="AC2* NAJ";
$resn_exch{" O2B NAJ"}="AO2* NAJ"; $resn_exch{" C1B NAJ"}="AC1* NAJ"; $resn_exch{" N9A NAJ"}=" AN9 NAJ";
$resn_exch{" C8A NAJ"}=" AC8 NAJ"; $resn_exch{" N7A NAJ"}=" AN7 NAJ"; $resn_exch{" C5A NAJ"}=" AC5 NAJ";
$resn_exch{" C6A NAJ"}=" AC6 NAJ"; $resn_exch{" N6A NAJ"}=" AN6 NAJ"; $resn_exch{" N1A NAJ"}=" AN1 NAJ";
$resn_exch{" C2A NAJ"}=" AC2 NAJ"; $resn_exch{" N3A NAJ"}=" AN3 NAJ"; $resn_exch{" C4A NAJ"}=" AC4 NAJ";
$resn_exch{" PN  NAJ"}=" NP  NAJ"; $resn_exch{" O1N NAJ"}=" NO1 NAJ"; $resn_exch{" O2N NAJ"}=" NO2 NAJ";
$resn_exch{" O5D NAJ"}="NO5* NAJ"; $resn_exch{" C5D NAJ"}="NC5* NAJ"; $resn_exch{" C4D NAJ"}="NC4* NAJ";
$resn_exch{" O4D NAJ"}="NO4* NAJ"; $resn_exch{" C3D NAJ"}="NC3* NAJ"; $resn_exch{" O3D NAJ"}="NO3* NAJ";
$resn_exch{" C2D NAJ"}="NC2* NAJ"; $resn_exch{" O2D NAJ"}="NO2* NAJ"; $resn_exch{" C1D NAJ"}="NC1* NAJ";
$resn_exch{" N1N NAJ"}=" NN1 NAJ"; $resn_exch{" C2N NAJ"}=" NC2 NAJ"; $resn_exch{" C3N NAJ"}=" NC3 NAJ";
$resn_exch{" C7N NAJ"}=" NC7 NAJ"; $resn_exch{" O7N NAJ"}=" NO7 NAJ"; $resn_exch{" N7N NAJ"}=" NN7 NAJ";
$resn_exch{" C4N NAJ"}=" NC4 NAJ"; $resn_exch{" C5N NAJ"}=" NC5 NAJ"; $resn_exch{" C6N NAJ"}=" NC6 NAJ";
$resn_exch{" PA  NAP"}=" AP  NAP"; $resn_exch{" O1A NAP"}=" AO1 NAP"; $resn_exch{" O2A NAP"}=" AO2 NAP";
$resn_exch{" O5B NAP"}="AO5* NAP"; $resn_exch{" C5B NAP"}="AC5* NAP"; $resn_exch{" C4B NAP"}="AC4* NAP";
$resn_exch{" O4B NAP"}="AO4* NAP"; $resn_exch{" C3B NAP"}="AC3* NAP"; $resn_exch{" O3B NAP"}="AO3* NAP";
$resn_exch{" C2B NAP"}="AC2* NAP"; $resn_exch{" O2B NAP"}="AO2* NAP"; $resn_exch{" C1B NAP"}="AC1* NAP";
$resn_exch{" N9A NAP"}=" AN9 NAP"; $resn_exch{" C8A NAP"}=" AC8 NAP"; $resn_exch{" N7A NAP"}=" AN7 NAP";
$resn_exch{" C5A NAP"}=" AC5 NAP"; $resn_exch{" C6A NAP"}=" AC6 NAP"; $resn_exch{" N6A NAP"}=" AN6 NAP";
$resn_exch{" N1A NAP"}=" AN1 NAP"; $resn_exch{" C2A NAP"}=" AC2 NAP"; $resn_exch{" N3A NAP"}=" AN3 NAP";
$resn_exch{" C4A NAP"}=" AC4 NAP"; $resn_exch{" PN  NAP"}=" NP  NAP"; $resn_exch{" O1N NAP"}=" NO1 NAP";
$resn_exch{" O2N NAP"}=" NO2 NAP"; $resn_exch{" O5D NAP"}="NO5* NAP"; $resn_exch{" C5D NAP"}="NC5* NAP";
$resn_exch{" C4D NAP"}="NC4* NAP"; $resn_exch{" O4D NAP"}="NO4* NAP"; $resn_exch{" C3D NAP"}="NC3* NAP";
$resn_exch{" O3D NAP"}="NO3* NAP"; $resn_exch{" C2D NAP"}="NC2* NAP"; $resn_exch{" O2D NAP"}="NO2* NAP";
$resn_exch{" C1D NAP"}="NC1* NAP"; $resn_exch{" N1N NAP"}=" NN1 NAP"; $resn_exch{" C2N NAP"}=" NC2 NAP";
$resn_exch{" C3N NAP"}=" NC3 NAP"; $resn_exch{" C7N NAP"}=" NC7 NAP"; $resn_exch{" O7N NAP"}=" NO7 NAP";
$resn_exch{" N7N NAP"}=" NN7 NAP"; $resn_exch{" C4N NAP"}=" NC4 NAP"; $resn_exch{" C5N NAP"}=" NC5 NAP";
$resn_exch{" C6N NAP"}=" NC6 NAP"; $resn_exch{" P2B NAP"}="AP2* NAP"; $resn_exch{" O1X NAP"}="AOP1 NAP";
$resn_exch{" O2X NAP"}="AOP2 NAP"; $resn_exch{" O3X NAP"}="AOP3 NAP"; $resn_exch{" O5B NAQ"}=" O5* NAQ";
$resn_exch{" C5B NAQ"}=" C5* NAQ"; $resn_exch{" C4B NAQ"}=" C4* NAQ"; $resn_exch{" O4B NAQ"}=" O4* NAQ";
$resn_exch{" C3B NAQ"}=" C3* NAQ"; $resn_exch{" O3B NAQ"}=" O3* NAQ"; $resn_exch{" C2B NAQ"}=" C2* NAQ";
$resn_exch{" O2B NAQ"}=" O2* NAQ"; $resn_exch{" C1B NAQ"}=" C1* NAQ"; $resn_exch{" PA  NAX"}=" AP  NAX";
$resn_exch{" O1A NAX"}=" AO1 NAX"; $resn_exch{" O2A NAX"}=" AO2 NAX"; $resn_exch{" O5B NAX"}="AO5* NAX";
$resn_exch{" C5B NAX"}="AC5* NAX"; $resn_exch{" C4B NAX"}="AC4* NAX"; $resn_exch{" O4B NAX"}="AO4* NAX";
$resn_exch{" C3B NAX"}="AC3* NAX"; $resn_exch{" O3B NAX"}="AO3* NAX"; $resn_exch{" C2B NAX"}="AC2* NAX";
$resn_exch{" O2B NAX"}="AO2* NAX"; $resn_exch{" C1B NAX"}="AC1* NAX"; $resn_exch{" N9A NAX"}=" AN9 NAX";
$resn_exch{" C8A NAX"}=" AC8 NAX"; $resn_exch{" N7A NAX"}=" AN7 NAX"; $resn_exch{" C5A NAX"}=" AC5 NAX";
$resn_exch{" C6A NAX"}=" AC6 NAX"; $resn_exch{" N6A NAX"}=" AN6 NAX"; $resn_exch{" N1A NAX"}=" AN1 NAX";
$resn_exch{" C2A NAX"}=" AC2 NAX"; $resn_exch{" N3A NAX"}=" AN3 NAX"; $resn_exch{" C4A NAX"}=" AC4 NAX";
$resn_exch{" PN  NAX"}=" NP  NAX"; $resn_exch{" O1N NAX"}=" NO1 NAX"; $resn_exch{" O2N NAX"}=" NO2 NAX";
$resn_exch{" O5D NAX"}="NO5* NAX"; $resn_exch{" C5D NAX"}="NC5* NAX"; $resn_exch{" C4D NAX"}="NC4* NAX";
$resn_exch{" O4D NAX"}="NO4* NAX"; $resn_exch{" C3D NAX"}="NC3* NAX"; $resn_exch{" O3D NAX"}="NO3* NAX";
$resn_exch{" C2D NAX"}="NC2* NAX"; $resn_exch{" O2D NAX"}="NO2* NAX"; $resn_exch{" C1D NAX"}="NC1* NAX";
$resn_exch{" N1N NAX"}=" NN1 NAX"; $resn_exch{" C2N NAX"}=" NC2 NAX"; $resn_exch{" C3N NAX"}=" NC3 NAX";
$resn_exch{" C7N NAX"}=" NC7 NAX"; $resn_exch{" O7N NAX"}=" NO7 NAX"; $resn_exch{" N7N NAX"}=" NN7 NAX";
$resn_exch{" C4N NAX"}=" NC4 NAX"; $resn_exch{" C5N NAX"}=" NC5 NAX"; $resn_exch{" C6N NAX"}=" NC6 NAX";
$resn_exch{" O6N NAX"}=" NO6 NAX"; $resn_exch{" C1' NBC"}=" C1* NBC"; $resn_exch{" O1' NBC"}=" O1* NBC";
$resn_exch{" C2' NBC"}=" C2* NBC"; $resn_exch{" C3' NBC"}=" C3* NBC"; $resn_exch{" C4' NBC"}=" C4* NBC";
$resn_exch{" N5' NBC"}=" N5* NBC"; $resn_exch{" O2' NBC"}=" O2* NBC"; $resn_exch{" O3' NBC"}=" O3* NBC";
$resn_exch{" PA  NBD"}=" AP  NBD"; $resn_exch{" O11 NBD"}="AO1P NBD"; $resn_exch{" O12 NBD"}="AO2P NBD";
$resn_exch{" O5B NBD"}="AO5* NBD"; $resn_exch{" C5B NBD"}="AC5* NBD"; $resn_exch{" C4B NBD"}="AC4* NBD";
$resn_exch{" O4B NBD"}="AO4* NBD"; $resn_exch{" C3B NBD"}="AC3* NBD"; $resn_exch{" O3B NBD"}="AO3* NBD";
$resn_exch{" C2B NBD"}="AC2* NBD"; $resn_exch{" O2B NBD"}="AO2* NBD"; $resn_exch{" C1B NBD"}="AC1* NBD";
$resn_exch{" N9A NBD"}=" AN9 NBD"; $resn_exch{" C8A NBD"}=" AC8 NBD"; $resn_exch{" N7A NBD"}=" AN7 NBD";
$resn_exch{" C5A NBD"}=" AC5 NBD"; $resn_exch{" C6A NBD"}=" AC6 NBD"; $resn_exch{" N6A NBD"}=" AN6 NBD";
$resn_exch{" N1A NBD"}=" AN1 NBD"; $resn_exch{" C2A NBD"}=" AC2 NBD"; $resn_exch{" N3A NBD"}=" AN3 NBD";
$resn_exch{" C4A NBD"}=" AC4 NBD"; $resn_exch{" PN  NBD"}=" NP  NBD"; $resn_exch{" O13 NBD"}="NO1P NBD";
$resn_exch{" O14 NBD"}="NO2P NBD"; $resn_exch{" O5D NBD"}="NO5* NBD"; $resn_exch{" C5D NBD"}="NC5* NBD";
$resn_exch{" C4D NBD"}="NC4* NBD"; $resn_exch{" O4D NBD"}="NO4* NBD"; $resn_exch{" C3D NBD"}="NC3* NBD";
$resn_exch{" O3D NBD"}="NO3* NBD"; $resn_exch{" C2D NBD"}="NC2* NBD"; $resn_exch{" O2D NBD"}="NO2* NBD";
$resn_exch{" C1D NBD"}="NC1* NBD"; $resn_exch{" N1N NBD"}=" NN1 NBD"; $resn_exch{" C2N NBD"}=" NC2 NBD";
$resn_exch{" C3N NBD"}=" NC3 NBD"; $resn_exch{" C7N NBD"}=" NC7 NBD"; $resn_exch{" O7N NBD"}=" NO7 NBD";
$resn_exch{" N7N NBD"}=" NN7 NBD"; $resn_exch{" C4N NBD"}=" NC4 NBD"; $resn_exch{" C5N NBD"}=" NC5 NBD";
$resn_exch{" C6N NBD"}=" NC6 NBD"; $resn_exch{" PA  NBP"}=" AP  NBP"; $resn_exch{" O1A NBP"}=" AO1 NBP";
$resn_exch{" O2A NBP"}=" AO2 NBP"; $resn_exch{" O5B NBP"}="AO5* NBP"; $resn_exch{" C5B NBP"}="AC5* NBP";
$resn_exch{" C4B NBP"}="AC4* NBP"; $resn_exch{" O4B NBP"}="AO4* NBP"; $resn_exch{" C3B NBP"}="AC3* NBP";
$resn_exch{" O3B NBP"}="AO3* NBP"; $resn_exch{" C2B NBP"}="AC2* NBP"; $resn_exch{" O2B NBP"}="AO2* NBP";
$resn_exch{" C1B NBP"}="AC1* NBP"; $resn_exch{" N9A NBP"}=" AN9 NBP"; $resn_exch{" C8A NBP"}=" AC8 NBP";
$resn_exch{" N7A NBP"}=" AN7 NBP"; $resn_exch{" C5A NBP"}=" AC5 NBP"; $resn_exch{" C6A NBP"}=" AC6 NBP";
$resn_exch{" N6A NBP"}=" AN6 NBP"; $resn_exch{" N1A NBP"}=" AN1 NBP"; $resn_exch{" C2A NBP"}=" AC2 NBP";
$resn_exch{" N3A NBP"}=" AN3 NBP"; $resn_exch{" C4A NBP"}=" AC4 NBP"; $resn_exch{" PN  NBP"}=" NP  NBP";
$resn_exch{" O1N NBP"}=" NO1 NBP"; $resn_exch{" O2N NBP"}=" NO2 NBP"; $resn_exch{" O5D NBP"}="NO5* NBP";
$resn_exch{" C5D NBP"}="NC5* NBP"; $resn_exch{" C4D NBP"}="NC4* NBP"; $resn_exch{" O4D NBP"}="NO4* NBP";
$resn_exch{" C3D NBP"}="NC3* NBP"; $resn_exch{" O3D NBP"}="NO3* NBP"; $resn_exch{" C2D NBP"}="NC2* NBP";
$resn_exch{" O2D NBP"}="NO2* NBP"; $resn_exch{" C1D NBP"}="NC1* NBP"; $resn_exch{" N1N NBP"}=" NN1 NBP";
$resn_exch{" C2N NBP"}=" NC2 NBP"; $resn_exch{" C3N NBP"}=" NC3 NBP"; $resn_exch{" C7N NBP"}=" NC7 NBP";
$resn_exch{" O7N NBP"}=" NO7 NBP"; $resn_exch{" N7N NBP"}=" NN7 NBP"; $resn_exch{" C4N NBP"}=" NC4 NBP";
$resn_exch{" C5N NBP"}=" NC5 NBP"; $resn_exch{" C6N NBP"}=" NC6 NBP"; $resn_exch{" P2B NBP"}="AP2* NBP";
$resn_exch{" O1X NBP"}="AOP1 NBP"; $resn_exch{" O2X NBP"}="AOP2 NBP"; $resn_exch{" O3X NBP"}="AOP3 NBP";
$resn_exch{" OXT NBQ"}=" O   NBQ"; $resn_exch{" O   NBQ"}=" O1  NBQ"; $resn_exch{" O5' NBS"}=" O5* NBS";
$resn_exch{" C5' NBS"}=" C5* NBS"; $resn_exch{" C4' NBS"}=" C4* NBS"; $resn_exch{" O4' NBS"}=" O4* NBS";
$resn_exch{" C1' NBS"}=" C1* NBS"; $resn_exch{" C2' NBS"}=" C2* NBS"; $resn_exch{" O2' NBS"}=" O2* NBS";
$resn_exch{" C3' NBS"}=" C3* NBS"; $resn_exch{" O3' NBS"}=" O3* NBS"; $resn_exch{" OXT NC1"}=" O1  NC1";
$resn_exch{" OXS NCG"}="O100 NCG"; $resn_exch{" CXU NCG"}="C102 NCG"; $resn_exch{" OXX NCG"}="O105 NCG";
$resn_exch{" NX0 NCG"}="N108 NCG"; $resn_exch{" CX2 NCG"}="C110 NCG"; $resn_exch{" OXA NCG"}="O118 NCG";
$resn_exch{" O5' NCN"}=" O5* NCN"; $resn_exch{" C5' NCN"}=" C5* NCN"; $resn_exch{" C4' NCN"}=" C4* NCN";
$resn_exch{" O4' NCN"}=" O4* NCN"; $resn_exch{" C3' NCN"}=" C3* NCN"; $resn_exch{" O3' NCN"}=" O3* NCN";
$resn_exch{" C2' NCN"}=" C2* NCN"; $resn_exch{" O2' NCN"}=" O2* NCN"; $resn_exch{" C1' NCN"}=" C1* NCN";
$resn_exch{" O1' NCS"}=" O1* NCS"; $resn_exch{" C1' NCS"}=" C1* NCS"; $resn_exch{" C2' NCS"}=" C2* NCS";
$resn_exch{" N2' NCS"}=" N2* NCS"; $resn_exch{" C3' NCS"}=" C3* NCS"; $resn_exch{" O3' NCS"}=" O3* NCS";
$resn_exch{" C4' NCS"}=" C4* NCS"; $resn_exch{" O4' NCS"}=" O4* NCS"; $resn_exch{" C5' NCS"}=" C5* NCS";
$resn_exch{" O5' NCS"}=" O5* NCS"; $resn_exch{" OP1 NCX"}=" O1P NCX"; $resn_exch{" OP2 NCX"}=" O2P NCX";
$resn_exch{" OP3 NCX"}=" O3P NCX"; $resn_exch{" O5' NCX"}=" O5* NCX"; $resn_exch{" C5' NCX"}=" C5* NCX";
$resn_exch{" C4' NCX"}=" C4* NCX"; $resn_exch{" O4' NCX"}=" O4* NCX"; $resn_exch{" C1' NCX"}=" C1* NCX";
$resn_exch{" C3' NCX"}=" C3* NCX"; $resn_exch{" C2' NCX"}=" C2* NCX"; $resn_exch{" O3' NCX"}=" O3* NCX";
$resn_exch{" PA  NDA"}=" AP  NDA"; $resn_exch{" O1A NDA"}=" AO1 NDA"; $resn_exch{" O2A NDA"}=" AO2 NDA";
$resn_exch{" O5B NDA"}="AO5* NDA"; $resn_exch{" C5B NDA"}="AC5* NDA"; $resn_exch{" C4B NDA"}="AC4* NDA";
$resn_exch{" O4B NDA"}="AO4* NDA"; $resn_exch{" C3B NDA"}="AC3* NDA"; $resn_exch{" O3B NDA"}="AO3* NDA";
$resn_exch{" C2B NDA"}="AC2* NDA"; $resn_exch{" O2B NDA"}="AO2* NDA"; $resn_exch{" C1B NDA"}="AC1* NDA";
$resn_exch{" N9A NDA"}=" AN9 NDA"; $resn_exch{" C8A NDA"}=" AC8 NDA"; $resn_exch{" N7A NDA"}=" AN7 NDA";
$resn_exch{" C5A NDA"}=" AC5 NDA"; $resn_exch{" C6A NDA"}=" AC6 NDA"; $resn_exch{" N6A NDA"}=" AN6 NDA";
$resn_exch{" N1A NDA"}=" AN1 NDA"; $resn_exch{" C2A NDA"}=" AC2 NDA"; $resn_exch{" N3A NDA"}=" AN3 NDA";
$resn_exch{" C4A NDA"}=" AC4 NDA"; $resn_exch{" PN  NDA"}=" NP  NDA"; $resn_exch{" O1N NDA"}=" NO1 NDA";
$resn_exch{" O2N NDA"}=" NO2 NDA"; $resn_exch{" O5D NDA"}="NO5* NDA"; $resn_exch{" C5D NDA"}="NC5* NDA";
$resn_exch{" C4D NDA"}="NC4* NDA"; $resn_exch{" O4D NDA"}="NO4* NDA"; $resn_exch{" C3D NDA"}="NC3* NDA";
$resn_exch{" O3D NDA"}="NO3* NDA"; $resn_exch{" C2D NDA"}="NC2* NDA"; $resn_exch{" O2D NDA"}="NO2* NDA";
$resn_exch{" C1D NDA"}="NC1* NDA"; $resn_exch{" N1N NDA"}=" NN1 NDA"; $resn_exch{" C2N NDA"}=" NC2 NDA";
$resn_exch{" C3N NDA"}=" NC3 NDA"; $resn_exch{" C7N NDA"}=" NC7 NDA"; $resn_exch{" N7N NDA"}=" NN7 NDA";
$resn_exch{" C4N NDA"}=" NC4 NDA"; $resn_exch{" C5N NDA"}=" NC5 NDA"; $resn_exch{" C6N NDA"}=" NC6 NDA";
$resn_exch{" O5B NDC"}=" O5* NDC"; $resn_exch{" C5B NDC"}=" C5* NDC"; $resn_exch{" C4B NDC"}=" C4* NDC";
$resn_exch{" O4B NDC"}=" O4* NDC"; $resn_exch{" C3B NDC"}=" C3* NDC"; $resn_exch{" O3B NDC"}=" O3* NDC";
$resn_exch{" C2B NDC"}=" C2* NDC"; $resn_exch{" O2B NDC"}=" O2* NDC"; $resn_exch{" C1B NDC"}=" C1* NDC";
$resn_exch{" O7N NDE"}=" NO7 NDE"; $resn_exch{" C7N NDE"}=" NC7 NDE"; $resn_exch{" N7N NDE"}=" NN7 NDE";
$resn_exch{" C3N NDE"}=" NC3 NDE"; $resn_exch{" C4N NDE"}=" NC4 NDE"; $resn_exch{" C5N NDE"}=" NC5 NDE";
$resn_exch{" C6N NDE"}=" NC6 NDE"; $resn_exch{" C2N NDE"}=" NC2 NDE"; $resn_exch{" N1N NDE"}=" NN1 NDE";
$resn_exch{" C1D NDE"}="NC1* NDE"; $resn_exch{" C2D NDE"}="NC2* NDE"; $resn_exch{" O2D NDE"}="NO2* NDE";
$resn_exch{" C3D NDE"}="NC3* NDE"; $resn_exch{" O3D NDE"}="NO3* NDE"; $resn_exch{" O4D NDE"}="NO4* NDE";
$resn_exch{" C4D NDE"}="NC4* NDE"; $resn_exch{" C5D NDE"}="NC5* NDE"; $resn_exch{" O5D NDE"}="NO5* NDE";
$resn_exch{" PN  NDE"}=" NP  NDE"; $resn_exch{" O1N NDE"}=" NO1 NDE"; $resn_exch{" O2N NDE"}=" NO2 NDE";
$resn_exch{" PA  NDE"}=" AP  NDE"; $resn_exch{" O1A NDE"}=" AO1 NDE"; $resn_exch{" O2A NDE"}=" AO2 NDE";
$resn_exch{" O5B NDE"}="AO5* NDE"; $resn_exch{" C5B NDE"}="AC5* NDE"; $resn_exch{" C4B NDE"}="AC4* NDE";
$resn_exch{" O4B NDE"}="AO4* NDE"; $resn_exch{" C1B NDE"}="AC1* NDE"; $resn_exch{" C2B NDE"}="AC2* NDE";
$resn_exch{" O2B NDE"}="AO2* NDE"; $resn_exch{" C3B NDE"}="AC3* NDE"; $resn_exch{" O3B NDE"}="AO3* NDE";
$resn_exch{" N9A NDE"}=" AN9 NDE"; $resn_exch{" C8A NDE"}=" AC8 NDE"; $resn_exch{" N7A NDE"}=" AN7 NDE";
$resn_exch{" C5A NDE"}=" AC5 NDE"; $resn_exch{" C6A NDE"}=" AC6 NDE"; $resn_exch{" N6A NDE"}=" AN6 NDE";
$resn_exch{" C4A NDE"}=" AC4 NDE"; $resn_exch{" N3A NDE"}=" AN3 NDE"; $resn_exch{" C2A NDE"}=" AC2 NDE";
$resn_exch{" N1A NDE"}=" AN1 NDE"; $resn_exch{" PA  NDO"}=" AP  NDO"; $resn_exch{" O1A NDO"}=" AO1 NDO";
$resn_exch{" O2A NDO"}=" AO2 NDO"; $resn_exch{" O5B NDO"}="AO5* NDO"; $resn_exch{" C5B NDO"}="AC5* NDO";
$resn_exch{" C4B NDO"}="AC4* NDO"; $resn_exch{" O4B NDO"}="AO4* NDO"; $resn_exch{" C3B NDO"}="AC3* NDO";
$resn_exch{" O3B NDO"}="AO3* NDO"; $resn_exch{" C2B NDO"}="AC2* NDO"; $resn_exch{" O2B NDO"}="AO2* NDO";
$resn_exch{" C1B NDO"}="AC1* NDO"; $resn_exch{" N9A NDO"}=" AN9 NDO"; $resn_exch{" C8A NDO"}=" AC8 NDO";
$resn_exch{" N7A NDO"}=" AN7 NDO"; $resn_exch{" C5A NDO"}=" AC5 NDO"; $resn_exch{" C6A NDO"}=" AC6 NDO";
$resn_exch{" O6A NDO"}=" AO6 NDO"; $resn_exch{" N1A NDO"}=" AN1 NDO"; $resn_exch{" C2A NDO"}=" AC2 NDO";
$resn_exch{" N3A NDO"}=" AN3 NDO"; $resn_exch{" C4A NDO"}=" AC4 NDO"; $resn_exch{" PN  NDO"}=" NP  NDO";
$resn_exch{" O1N NDO"}=" NO1 NDO"; $resn_exch{" O2N NDO"}=" NO2 NDO"; $resn_exch{" O5D NDO"}="NO5* NDO";
$resn_exch{" C5D NDO"}="NC5* NDO"; $resn_exch{" C4D NDO"}="NC4* NDO"; $resn_exch{" O4D NDO"}="NO4* NDO";
$resn_exch{" C3D NDO"}="NC3* NDO"; $resn_exch{" O3D NDO"}="NO3* NDO"; $resn_exch{" C2D NDO"}="NC2* NDO";
$resn_exch{" O2D NDO"}="NO2* NDO"; $resn_exch{" C1D NDO"}="NC1* NDO"; $resn_exch{" N1N NDO"}=" NN1 NDO";
$resn_exch{" C2N NDO"}=" NC2 NDO"; $resn_exch{" C3N NDO"}=" NC3 NDO"; $resn_exch{" C7N NDO"}=" NC7 NDO";
$resn_exch{" O7N NDO"}=" NO7 NDO"; $resn_exch{" N7N NDO"}=" NN7 NDO"; $resn_exch{" C4N NDO"}=" NC4 NDO";
$resn_exch{" C5N NDO"}=" NC5 NDO"; $resn_exch{" C6N NDO"}=" NC6 NDO"; $resn_exch{" P2B NDO"}="AP2* NDO";
$resn_exch{" O1X NDO"}="AOP1 NDO"; $resn_exch{" O2X NDO"}="AOP2 NDO"; $resn_exch{" O3X NDO"}="AOP3 NDO";
$resn_exch{" PA  NDP"}=" AP  NDP"; $resn_exch{" O1A NDP"}=" AO1 NDP"; $resn_exch{" O2A NDP"}=" AO2 NDP";
$resn_exch{" O5B NDP"}="AO5* NDP"; $resn_exch{" C5B NDP"}="AC5* NDP"; $resn_exch{" C4B NDP"}="AC4* NDP";
$resn_exch{" O4B NDP"}="AO4* NDP"; $resn_exch{" C3B NDP"}="AC3* NDP"; $resn_exch{" O3B NDP"}="AO3* NDP";
$resn_exch{" C2B NDP"}="AC2* NDP"; $resn_exch{" O2B NDP"}="AO2* NDP"; $resn_exch{" C1B NDP"}="AC1* NDP";
$resn_exch{" N9A NDP"}=" AN9 NDP"; $resn_exch{" C8A NDP"}=" AC8 NDP"; $resn_exch{" N7A NDP"}=" AN7 NDP";
$resn_exch{" C5A NDP"}=" AC5 NDP"; $resn_exch{" C6A NDP"}=" AC6 NDP"; $resn_exch{" N6A NDP"}=" AN6 NDP";
$resn_exch{" N1A NDP"}=" AN1 NDP"; $resn_exch{" C2A NDP"}=" AC2 NDP"; $resn_exch{" N3A NDP"}=" AN3 NDP";
$resn_exch{" C4A NDP"}=" AC4 NDP"; $resn_exch{" PN  NDP"}=" NP  NDP"; $resn_exch{" O1N NDP"}=" NO1 NDP";
$resn_exch{" O2N NDP"}=" NO2 NDP"; $resn_exch{" O5D NDP"}="NO5* NDP"; $resn_exch{" C5D NDP"}="NC5* NDP";
$resn_exch{" C4D NDP"}="NC4* NDP"; $resn_exch{" O4D NDP"}="NO4* NDP"; $resn_exch{" C3D NDP"}="NC3* NDP";
$resn_exch{" O3D NDP"}="NO3* NDP"; $resn_exch{" C2D NDP"}="NC2* NDP"; $resn_exch{" O2D NDP"}="NO2* NDP";
$resn_exch{" C1D NDP"}="NC1* NDP"; $resn_exch{" N1N NDP"}=" NN1 NDP"; $resn_exch{" C2N NDP"}=" NC2 NDP";
$resn_exch{" C3N NDP"}=" NC3 NDP"; $resn_exch{" C7N NDP"}=" NC7 NDP"; $resn_exch{" O7N NDP"}=" NO7 NDP";
$resn_exch{" N7N NDP"}=" NN7 NDP"; $resn_exch{" C4N NDP"}=" NC4 NDP"; $resn_exch{" C5N NDP"}=" NC5 NDP";
$resn_exch{" C6N NDP"}=" NC6 NDP"; $resn_exch{" P2B NDP"}="AP2* NDP"; $resn_exch{" O1X NDP"}="AOP1 NDP";
$resn_exch{" O2X NDP"}="AOP2 NDP"; $resn_exch{" O3X NDP"}="AOP3 NDP"; $resn_exch{" C5' NEA"}=" C5* NEA";
$resn_exch{" C4' NEA"}=" C4* NEA"; $resn_exch{" O4' NEA"}=" O4* NEA"; $resn_exch{" C3' NEA"}=" C3* NEA";
$resn_exch{" O3' NEA"}=" O3* NEA"; $resn_exch{" C2' NEA"}=" C2* NEA"; $resn_exch{" O2' NEA"}=" O2* NEA";
$resn_exch{" C1' NEA"}=" C1* NEA"; $resn_exch{" C5' NEC"}=" C5* NEC"; $resn_exch{" O5' NEC"}=" O5* NEC";
$resn_exch{" N5' NEC"}=" N5* NEC"; $resn_exch{" C4' NEC"}=" C4* NEC"; $resn_exch{" O4' NEC"}=" O4* NEC";
$resn_exch{" C3' NEC"}=" C3* NEC"; $resn_exch{" O3' NEC"}=" O3* NEC"; $resn_exch{" C2' NEC"}=" C2* NEC";
$resn_exch{" O2' NEC"}=" O2* NEC"; $resn_exch{" C1' NEC"}=" C1* NEC"; $resn_exch{" N5' NEI"}=" N5* NEI";
$resn_exch{" C5' NEI"}=" C5* NEI"; $resn_exch{" O5' NEI"}=" O5* NEI"; $resn_exch{" C4' NEI"}=" C4* NEI";
$resn_exch{" O4' NEI"}=" O4* NEI"; $resn_exch{" C3' NEI"}=" C3* NEI"; $resn_exch{" O3' NEI"}=" O3* NEI";
$resn_exch{" C2' NEI"}=" C2* NEI"; $resn_exch{" O2' NEI"}=" O2* NEI"; $resn_exch{" C1' NEI"}=" C1* NEI";
$resn_exch{" O11  NF"}="1O1   NF"; $resn_exch{" C11  NF"}="1C1   NF"; $resn_exch{" N21  NF"}="1N2   NF";
$resn_exch{" C12  NF"}="2C1   NF"; $resn_exch{" C31  NF"}="1C3   NF"; $resn_exch{" O31  NF"}="1O3   NF";
$resn_exch{" CX3  NF"}="1C11  NF"; $resn_exch{" CXD  NF"}="1C12  NF"; $resn_exch{" CXT  NF"}="1C10  NF";
$resn_exch{" C91  NF"}="1C9   NF"; $resn_exch{" C81  NF"}="1C8   NF"; $resn_exch{" C71  NF"}="1C7   NF";
$resn_exch{" CXN  NF"}="1C13  NF"; $resn_exch{" C61  NF"}="1C6   NF"; $resn_exch{" C51  NF"}="1C5   NF";
$resn_exch{" C41  NF"}="1C4   NF"; $resn_exch{" C22  NF"}="2C2   NF"; $resn_exch{" N32  NF"}="2N3   NF";
$resn_exch{" C42  NF"}="2C4   NF"; $resn_exch{" C52  NF"}="2C5   NF"; $resn_exch{" C62  NF"}="2C6   NF";
$resn_exch{" N72  NF"}="2N7   NF"; $resn_exch{" C82  NF"}="2C8   NF"; $resn_exch{" O13  NF"}="3O1   NF";
$resn_exch{" C13  NF"}="3C1   NF"; $resn_exch{" N23  NF"}="3N2   NF"; $resn_exch{" C92  NF"}="2C9   NF";
$resn_exch{" C33  NF"}="3C3   NF"; $resn_exch{" O33  NF"}="3O3   NF"; $resn_exch{" CX5  NF"}="3C11  NF";
$resn_exch{" CXF  NF"}="3C12  NF"; $resn_exch{" CXV  NF"}="3C10  NF"; $resn_exch{" C93  NF"}="3C9   NF";
$resn_exch{" C83  NF"}="3C8   NF"; $resn_exch{" C73  NF"}="3C7   NF"; $resn_exch{" CXP  NF"}="3C13  NF";
$resn_exch{" C63  NF"}="3C6   NF"; $resn_exch{" C53  NF"}="3C5   NF"; $resn_exch{" C43  NF"}="3C4   NF";
$resn_exch{" C1' NF2"}=" C1* NF2"; $resn_exch{" O4' NF2"}=" O4* NF2"; $resn_exch{" C2' NF2"}=" C2* NF2";
$resn_exch{" O2' NF2"}=" O2* NF2"; $resn_exch{" C3' NF2"}=" C3* NF2"; $resn_exch{" O3' NF2"}=" O3* NF2";
$resn_exch{" C4' NF2"}=" C4* NF2"; $resn_exch{" C5' NF2"}=" C5* NF2"; $resn_exch{" O5' NF2"}=" O5* NF2";
$resn_exch{" OP1 NF2"}=" O1P NF2"; $resn_exch{" OP2 NF2"}=" O2P NF2"; $resn_exch{" OP3 NF2"}=" O3P NF2";
$resn_exch{" C1' NGD"}=" C1* NGD"; $resn_exch{" O4' NGD"}=" O4* NGD"; $resn_exch{" C2' NGD"}=" C2* NGD";
$resn_exch{" O2' NGD"}=" O2* NGD"; $resn_exch{" C3' NGD"}=" C3* NGD"; $resn_exch{" O3' NGD"}=" O3* NGD";
$resn_exch{" C4' NGD"}=" C4* NGD"; $resn_exch{" C5' NGD"}=" C5* NGD"; $resn_exch{" O5' NGD"}=" O5* NGD";
$resn_exch{"O5'R NGD"}="RO5* NGD"; $resn_exch{"C5'R NGD"}="RC5* NGD"; $resn_exch{"C4'R NGD"}="RC4* NGD";
$resn_exch{"C3'R NGD"}="RC3* NGD"; $resn_exch{"O3'R NGD"}="RO3* NGD"; $resn_exch{"C2'R NGD"}="RC2* NGD";
$resn_exch{"O2'R NGD"}="RO2* NGD"; $resn_exch{"O4'R NGD"}="RO4* NGD"; $resn_exch{"C1'R NGD"}="RC1* NGD";
$resn_exch{" C1' NGM"}=" C1* NGM"; $resn_exch{" C2' NGM"}=" C2* NGM"; $resn_exch{" C3' NGM"}=" C3* NGM";
$resn_exch{" C4' NGM"}=" C4* NGM"; $resn_exch{" C5' NGM"}=" C5* NGM"; $resn_exch{" O1' NGM"}=" O1* NGM";
$resn_exch{" O2' NGM"}=" O2* NGM"; $resn_exch{" O3' NGM"}=" O3* NGM"; $resn_exch{" O4' NGM"}=" O4* NGM";
$resn_exch{" C1' NGP"}=" C1* NGP"; $resn_exch{" C2' NGP"}=" C2* NGP"; $resn_exch{" C3' NGP"}=" C3* NGP";
$resn_exch{" C4' NGP"}=" C4* NGP"; $resn_exch{" C5' NGP"}=" C5* NGP"; $resn_exch{" O1' NGP"}=" O1* NGP";
$resn_exch{" O2' NGP"}=" O2* NGP"; $resn_exch{" O3' NGP"}=" O3* NGP"; $resn_exch{" O4' NGP"}=" O4* NGP";
$resn_exch{" C1' NGU"}=" C1* NGU"; $resn_exch{" C2' NGU"}=" C2* NGU"; $resn_exch{" C3' NGU"}=" C3* NGU";
$resn_exch{" C4' NGU"}=" C4* NGU"; $resn_exch{" C5' NGU"}=" C5* NGU"; $resn_exch{" O1' NGU"}=" O1* NGU";
$resn_exch{" O2' NGU"}=" O2* NGU"; $resn_exch{" O3' NGU"}=" O3* NGU"; $resn_exch{" O4' NGU"}=" O4* NGU";
$resn_exch{" PA  NHD"}=" AP  NHD"; $resn_exch{" O1A NHD"}=" AO1 NHD"; $resn_exch{" O2A NHD"}=" AO2 NHD";
$resn_exch{" O5B NHD"}="AO5* NHD"; $resn_exch{" C5B NHD"}="AC5* NHD"; $resn_exch{" C4B NHD"}="AC4* NHD";
$resn_exch{" O4B NHD"}="AO4* NHD"; $resn_exch{" C3B NHD"}="AC3* NHD"; $resn_exch{" O3B NHD"}="AO3* NHD";
$resn_exch{" C2B NHD"}="AC2* NHD"; $resn_exch{" O2B NHD"}="AO2* NHD"; $resn_exch{" C1B NHD"}="AC1* NHD";
$resn_exch{" N9A NHD"}=" AN9 NHD"; $resn_exch{" C8A NHD"}=" AC8 NHD"; $resn_exch{" N7A NHD"}=" AN7 NHD";
$resn_exch{" C5A NHD"}=" AC5 NHD"; $resn_exch{" C6A NHD"}=" AC6 NHD"; $resn_exch{" O6A NHD"}=" AO6 NHD";
$resn_exch{" N1A NHD"}=" AN1 NHD"; $resn_exch{" C2A NHD"}=" AC2 NHD"; $resn_exch{" N3A NHD"}=" AN3 NHD";
$resn_exch{" C4A NHD"}=" AC4 NHD"; $resn_exch{" PN  NHD"}=" NP  NHD"; $resn_exch{" O1N NHD"}=" NO1 NHD";
$resn_exch{" O2N NHD"}=" NO2 NHD"; $resn_exch{" O5D NHD"}="NO5* NHD"; $resn_exch{" C5D NHD"}="NC5* NHD";
$resn_exch{" C4D NHD"}="NC4* NHD"; $resn_exch{" O4D NHD"}="NO4* NHD"; $resn_exch{" C3D NHD"}="NC3* NHD";
$resn_exch{" O3D NHD"}="NO3* NHD"; $resn_exch{" C2D NHD"}="NC2* NHD"; $resn_exch{" O2D NHD"}="NO2* NHD";
$resn_exch{" C1D NHD"}="NC1* NHD"; $resn_exch{" N1N NHD"}=" NN1 NHD"; $resn_exch{" C2N NHD"}=" NC2 NHD";
$resn_exch{" C3N NHD"}=" NC3 NHD"; $resn_exch{" C7N NHD"}=" NC7 NHD"; $resn_exch{" O7N NHD"}=" NO7 NHD";
$resn_exch{" N7N NHD"}=" NN7 NHD"; $resn_exch{" C4N NHD"}=" NC4 NHD"; $resn_exch{" C5N NHD"}=" NC5 NHD";
$resn_exch{" C6N NHD"}=" NC6 NHD"; $resn_exch{" O5B NHO"}=" O5* NHO"; $resn_exch{" C5B NHO"}=" C5* NHO";
$resn_exch{" C4B NHO"}=" C4* NHO"; $resn_exch{" O4B NHO"}=" O4* NHO"; $resn_exch{" C3B NHO"}=" C3* NHO";
$resn_exch{" O3B NHO"}=" O3* NHO"; $resn_exch{" C2B NHO"}=" C2* NHO"; $resn_exch{" O2B NHO"}=" O2* NHO";
$resn_exch{" C1B NHO"}=" C1* NHO"; $resn_exch{" C1' NIR"}=" C1* NIR"; $resn_exch{" C2' NIR"}=" C2* NIR";
$resn_exch{" O2' NIR"}=" O2* NIR"; $resn_exch{" C3' NIR"}=" C3* NIR"; $resn_exch{" O3' NIR"}=" O3* NIR";
$resn_exch{" C4' NIR"}=" C4* NIR"; $resn_exch{" O4' NIR"}=" O4* NIR"; $resn_exch{" C5' NIR"}=" C5* NIR";
$resn_exch{" OXT NLQ"}=" O   NLQ"; $resn_exch{" O   NLQ"}=" OXT NLQ"; $resn_exch{" C5' NMD"}=" C5* NMD";
$resn_exch{" C4' NMD"}=" C4* NMD"; $resn_exch{" O4' NMD"}=" O4* NMD"; $resn_exch{" C3' NMD"}=" C3* NMD";
$resn_exch{" O3' NMD"}=" O3* NMD"; $resn_exch{" C2' NMD"}=" C2* NMD"; $resn_exch{" N2' NMD"}=" N2* NMD";
$resn_exch{" C1' NMD"}=" C1* NMD"; $resn_exch{" O5' NMD"}=" O5* NMD"; $resn_exch{" OP2 NMS"}=" O2P NMS";
$resn_exch{" OP1 NMS"}=" O1P NMS"; $resn_exch{" O5' NMS"}=" O5* NMS"; $resn_exch{" C2' NMS"}=" C2* NMS";
$resn_exch{" C5' NMS"}=" C5* NMS"; $resn_exch{" C4' NMS"}=" C4* NMS"; $resn_exch{" O4' NMS"}=" O4* NMS";
$resn_exch{" C1' NMS"}=" C1* NMS"; $resn_exch{" C3' NMS"}=" C3* NMS"; $resn_exch{" O3' NMS"}=" O3* NMS";
$resn_exch{" OP3 NMS"}=" O3P NMS"; $resn_exch{" OP3 NMT"}=" O3P NMT"; $resn_exch{" OP2 NMT"}=" O2P NMT";
$resn_exch{" O5' NMT"}=" O5* NMT"; $resn_exch{" C2' NMT"}=" C2* NMT"; $resn_exch{" C5' NMT"}=" C5* NMT";
$resn_exch{" C4' NMT"}=" C4* NMT"; $resn_exch{" O4' NMT"}=" O4* NMT"; $resn_exch{" C1' NMT"}=" C1* NMT";
$resn_exch{" C3' NMT"}=" C3* NMT"; $resn_exch{" O3' NMT"}=" O3* NMT"; $resn_exch{" OP1 NMT"}=" O1P NMT";
$resn_exch{" N1A NMX"}=" AN1 NMX"; $resn_exch{" C2A NMX"}=" AC2 NMX"; $resn_exch{" N3A NMX"}=" AN3 NMX";
$resn_exch{" C4A NMX"}=" AC4 NMX"; $resn_exch{" C5A NMX"}=" AC5 NMX"; $resn_exch{" C6A NMX"}=" AC6 NMX";
$resn_exch{" N6A NMX"}=" AN6 NMX"; $resn_exch{" N7A NMX"}=" AN7 NMX"; $resn_exch{" C8A NMX"}=" AC8 NMX";
$resn_exch{" N9A NMX"}=" AN9 NMX"; $resn_exch{" C1B NMX"}="AC1* NMX"; $resn_exch{" C2B NMX"}="AC2* NMX";
$resn_exch{" O2B NMX"}="AO2* NMX"; $resn_exch{" C3B NMX"}="AC3* NMX"; $resn_exch{" O3B NMX"}="AO3* NMX";
$resn_exch{" P3B NMX"}="AP3* NMX"; $resn_exch{" O7A NMX"}=" AO7 NMX"; $resn_exch{" O8A NMX"}=" AO8 NMX";
$resn_exch{" O9A NMX"}=" AO9 NMX"; $resn_exch{" C4B NMX"}="AC4* NMX"; $resn_exch{" O4B NMX"}="AO4* NMX";
$resn_exch{" C5B NMX"}="AC5* NMX"; $resn_exch{" O5B NMX"}="AO5* NMX"; $resn_exch{" P1A NMX"}=" AP1 NMX";
$resn_exch{" O1A NMX"}=" AO1 NMX"; $resn_exch{" O2A NMX"}=" AO2 NMX"; $resn_exch{" O3A NMX"}=" AO3 NMX";
$resn_exch{" P2A NMX"}=" AP2 NMX"; $resn_exch{" O4A NMX"}=" AO4 NMX"; $resn_exch{" O5A NMX"}=" AO5 NMX";
$resn_exch{" O6A NMX"}=" AO6 NMX"; $resn_exch{" CBP NMX"}="PC11 NMX"; $resn_exch{" CCP NMX"}="PC12 NMX";
$resn_exch{" CDP NMX"}="PC13 NMX"; $resn_exch{" CEP NMX"}="PC14 NMX"; $resn_exch{" CAP NMX"}="PC10 NMX";
$resn_exch{" OAP NMX"}="PO10 NMX"; $resn_exch{" C9P NMX"}=" PC9 NMX"; $resn_exch{" O9P NMX"}=" PO9 NMX";
$resn_exch{" N8P NMX"}=" PN8 NMX"; $resn_exch{" C7P NMX"}=" PC7 NMX"; $resn_exch{" C6P NMX"}=" PC6 NMX";
$resn_exch{" C5P NMX"}=" PC5 NMX"; $resn_exch{" O5P NMX"}=" PO5 NMX"; $resn_exch{" N4P NMX"}=" PN4 NMX";
$resn_exch{" C3P NMX"}=" PC3 NMX"; $resn_exch{" C2P NMX"}=" PC2 NMX"; $resn_exch{" C1I NMX"}=" IC1 NMX";
$resn_exch{" N1I NMX"}=" IN1 NMX"; $resn_exch{" O1I NMX"}=" IO1 NMX"; $resn_exch{" O2I NMX"}=" IO2 NMX";
$resn_exch{" C6' NOC"}=" C6* NOC"; $resn_exch{" O6' NOC"}=" O6* NOC"; $resn_exch{" C5' NOC"}=" C5* NOC";
$resn_exch{" C4' NOC"}=" C4* NOC"; $resn_exch{" C3' NOC"}=" C3* NOC"; $resn_exch{" O3' NOC"}=" O3* NOC";
$resn_exch{" C2' NOC"}=" C2* NOC"; $resn_exch{" O2' NOC"}=" O2* NOC"; $resn_exch{" C1' NOC"}=" C1* NOC";
$resn_exch{" C1' NOD"}=" C1* NOD"; $resn_exch{" C2' NOD"}=" C2* NOD"; $resn_exch{" C3' NOD"}=" C3* NOD";
$resn_exch{" C4' NOD"}=" C4* NOD"; $resn_exch{" O4' NOD"}=" O4* NOD"; $resn_exch{" C5' NOD"}=" C5* NOD";
$resn_exch{" O5' NOD"}=" O5* NOD"; $resn_exch{" C6' NOD"}=" C6* NOD"; $resn_exch{" N3' NOD"}=" N3* NOD";
$resn_exch{" C5' NOS"}=" C5* NOS"; $resn_exch{" O5' NOS"}=" O5* NOS"; $resn_exch{" C4' NOS"}=" C4* NOS";
$resn_exch{" O4' NOS"}=" O4* NOS"; $resn_exch{" C1' NOS"}=" C1* NOS"; $resn_exch{" C2' NOS"}=" C2* NOS";
$resn_exch{" O2' NOS"}=" O2* NOS"; $resn_exch{" C3' NOS"}=" C3* NOS"; $resn_exch{" O3' NOS"}=" O3* NOS";
$resn_exch{" O5' NP3"}=" O5* NP3"; $resn_exch{" C5' NP3"}=" C5* NP3"; $resn_exch{" C4' NP3"}=" C4* NP3";
$resn_exch{" O4' NP3"}=" O4* NP3"; $resn_exch{" C1' NP3"}=" C1* NP3"; $resn_exch{" C2' NP3"}=" C2* NP3";
$resn_exch{" C3' NP3"}=" C3* NP3"; $resn_exch{" O3' NP3"}=" O3* NP3"; $resn_exch{" O5' NRI"}=" O5* NRI";
$resn_exch{" C2' NRI"}=" C2* NRI"; $resn_exch{" C5' NRI"}=" C5* NRI"; $resn_exch{" C4' NRI"}=" C4* NRI";
$resn_exch{" C3' NRI"}=" C3* NRI"; $resn_exch{" C6' NRI"}=" C6* NRI"; $resn_exch{" N1' NRI"}=" N1* NRI";
$resn_exch{" O3' NRI"}=" O3* NRI"; $resn_exch{" C4' NRT"}=" C4* NRT"; $resn_exch{" O4' NRT"}=" O4* NRT";
$resn_exch{" C3' NRT"}=" C3* NRT"; $resn_exch{" C2' NRT"}=" C2* NRT"; $resn_exch{" O5' NSS"}=" O5* NSS";
$resn_exch{" C5' NSS"}=" C5* NSS"; $resn_exch{" C4' NSS"}=" C4* NSS"; $resn_exch{" C3' NSS"}=" C3* NSS";
$resn_exch{" O3' NSS"}=" O3* NSS"; $resn_exch{" C2' NSS"}=" C2* NSS"; $resn_exch{" O2' NSS"}=" O2* NSS";
$resn_exch{" C1' NSS"}=" C1* NSS"; $resn_exch{" O4' NSS"}=" O4* NSS"; $resn_exch{" C2' NTU"}=" C2* NTU";
$resn_exch{" S1' NTU"}=" S1* NTU"; $resn_exch{" C1' NTU"}=" C1* NTU"; $resn_exch{" C4' NTU"}=" C4* NTU";
$resn_exch{" S2' NTU"}=" S2* NTU"; $resn_exch{" C3' NTU"}=" C3* NTU"; $resn_exch{" C1' NUP"}=" C1* NUP";
$resn_exch{" O4' NUP"}=" O4* NUP"; $resn_exch{" C2' NUP"}=" C2* NUP"; $resn_exch{" O2' NUP"}=" O2* NUP";
$resn_exch{" C3' NUP"}=" C3* NUP"; $resn_exch{" O3' NUP"}=" O3* NUP"; $resn_exch{" C4' NUP"}=" C4* NUP";
$resn_exch{" C5' NUP"}=" C5* NUP"; $resn_exch{" O5' NUP"}=" O5* NUP"; $resn_exch{" OXY NYC"}=" OXT NYC";
$resn_exch{" O4' NYM"}=" O4* NYM"; $resn_exch{" N3' NYM"}=" N3* NYM"; $resn_exch{" C1' NYM"}=" C1* NYM";
$resn_exch{" C2' NYM"}=" C2* NYM"; $resn_exch{" C3' NYM"}=" C3* NYM"; $resn_exch{" C4' NYM"}=" C4* NYM";
$resn_exch{" O5' NYM"}=" O5* NYM"; $resn_exch{" C5' NYM"}=" C5* NYM"; $resn_exch{" O1X NZQ"}="AOP1 NZQ";
$resn_exch{" P2B NZQ"}="AP2* NZQ"; $resn_exch{" O2X NZQ"}="AOP2 NZQ"; $resn_exch{" O3X NZQ"}="AOP3 NZQ";
$resn_exch{" O2B NZQ"}="AO2* NZQ"; $resn_exch{" C2B NZQ"}="AC2* NZQ"; $resn_exch{" C3B NZQ"}="AC3* NZQ";
$resn_exch{" O3B NZQ"}="AO3* NZQ"; $resn_exch{" C1B NZQ"}="AC1* NZQ"; $resn_exch{" N9A NZQ"}=" AN9 NZQ";
$resn_exch{" C8A NZQ"}=" AC8 NZQ"; $resn_exch{" N7A NZQ"}=" AN7 NZQ"; $resn_exch{" C5A NZQ"}=" AC5 NZQ";
$resn_exch{" C4A NZQ"}=" AC4 NZQ"; $resn_exch{" N3A NZQ"}=" AN3 NZQ"; $resn_exch{" C2A NZQ"}=" AC2 NZQ";
$resn_exch{" N1A NZQ"}=" AN1 NZQ"; $resn_exch{" C6A NZQ"}=" AC6 NZQ"; $resn_exch{" N6A NZQ"}=" AN6 NZQ";
$resn_exch{" O4B NZQ"}="AO4* NZQ"; $resn_exch{" C4B NZQ"}="AC4* NZQ"; $resn_exch{" C5B NZQ"}="AC5* NZQ";
$resn_exch{" O5B NZQ"}="AO5* NZQ"; $resn_exch{" PA  NZQ"}=" AP  NZQ"; $resn_exch{" O1A NZQ"}=" AO1 NZQ";
$resn_exch{" O2A NZQ"}=" AO2 NZQ"; $resn_exch{" PN  NZQ"}=" NP  NZQ"; $resn_exch{" O1N NZQ"}=" NO1 NZQ";
$resn_exch{" O2N NZQ"}=" NO2 NZQ"; $resn_exch{" O5D NZQ"}="NO5* NZQ"; $resn_exch{" C5D NZQ"}="NC5* NZQ";
$resn_exch{" C4D NZQ"}="NC4* NZQ"; $resn_exch{" O4D NZQ"}="NO4* NZQ"; $resn_exch{" C1D NZQ"}="NC1* NZQ";
$resn_exch{" C2D NZQ"}="NC2* NZQ"; $resn_exch{" O2D NZQ"}="NO2* NZQ"; $resn_exch{" C3D NZQ"}="NC3* NZQ";
$resn_exch{" O3D NZQ"}="NO3* NZQ"; $resn_exch{" N1N NZQ"}=" NN1 NZQ"; $resn_exch{" C2N NZQ"}=" NC2 NZQ";
$resn_exch{" C3N NZQ"}=" NC3 NZQ"; $resn_exch{" C7N NZQ"}=" NC7 NZQ"; $resn_exch{" N7N NZQ"}=" NN7 NZQ";
$resn_exch{" O7N NZQ"}=" NO7 NZQ"; $resn_exch{" C4N NZQ"}=" NC4 NZQ"; $resn_exch{" C5N NZQ"}=" NC5 NZQ";
$resn_exch{" O5N NZQ"}=" NO5 NZQ"; $resn_exch{" C6N NZQ"}=" NC6 NZQ"; $resn_exch{" O6N NZQ"}=" NO6 NZQ";
$resn_exch{" O5' O2C"}=" O5* O2C"; $resn_exch{" C5' O2C"}=" C5* O2C"; $resn_exch{" C4' O2C"}=" C4* O2C";
$resn_exch{" O4' O2C"}=" O4* O2C"; $resn_exch{" C1' O2C"}=" C1* O2C"; $resn_exch{" C2' O2C"}=" C2* O2C";
$resn_exch{" O2' O2C"}=" O2* O2C"; $resn_exch{" C3' O2C"}=" C3* O2C"; $resn_exch{" O5' OAD"}=" O5* OAD";
$resn_exch{" C5' OAD"}=" C5* OAD"; $resn_exch{" C4' OAD"}=" C4* OAD"; $resn_exch{" O4' OAD"}=" O4* OAD";
$resn_exch{" C3' OAD"}=" C3* OAD"; $resn_exch{" O3' OAD"}=" O3* OAD"; $resn_exch{" C2' OAD"}=" C2* OAD";
$resn_exch{" O2' OAD"}=" O2* OAD"; $resn_exch{" C1' OAD"}=" C1* OAD"; $resn_exch{" O2D OAD"}="RO2* OAD";
$resn_exch{" O5D OAD"}="RO5* OAD"; $resn_exch{" C5D OAD"}="RC5* OAD"; $resn_exch{" C4D OAD"}="RC4* OAD";
$resn_exch{" O4D OAD"}="RO4* OAD"; $resn_exch{" C3D OAD"}="RC3* OAD"; $resn_exch{" O3D OAD"}="RO3* OAD";
$resn_exch{" C2D OAD"}="RC2* OAD"; $resn_exch{" C1D OAD"}="RC1* OAD"; $resn_exch{" O1D OAD"}="RO1* OAD";
$resn_exch{" C6D OAD"}="RC6* OAD"; $resn_exch{" O6D OAD"}="RO6* OAD"; $resn_exch{" C7D OAD"}="RC7* OAD";
$resn_exch{" CXA OBP"}="CX10 OBP"; $resn_exch{" CXB OBP"}="CX11 OBP"; $resn_exch{" CXC OBP"}="CX12 OBP";
$resn_exch{" O1X ODP"}="AOP1 ODP"; $resn_exch{" P2B ODP"}="AP2* ODP"; $resn_exch{" O2X ODP"}="AOP2 ODP";
$resn_exch{" O3X ODP"}="AOP3 ODP"; $resn_exch{" O2B ODP"}="AO2* ODP"; $resn_exch{" C2B ODP"}="AC2* ODP";
$resn_exch{" C3B ODP"}="AC3* ODP"; $resn_exch{" O3B ODP"}="AO3* ODP"; $resn_exch{" C1B ODP"}="AC1* ODP";
$resn_exch{" N9A ODP"}=" AN9 ODP"; $resn_exch{" C8A ODP"}=" AC8 ODP"; $resn_exch{" N7A ODP"}=" AN7 ODP";
$resn_exch{" C5A ODP"}=" AC5 ODP"; $resn_exch{" C4A ODP"}=" AC4 ODP"; $resn_exch{" N3A ODP"}=" AN3 ODP";
$resn_exch{" C2A ODP"}=" AC2 ODP"; $resn_exch{" N1A ODP"}=" AN1 ODP"; $resn_exch{" C6A ODP"}=" AC6 ODP";
$resn_exch{" N6A ODP"}=" AN6 ODP"; $resn_exch{" O4B ODP"}="AO4* ODP"; $resn_exch{" C4B ODP"}="AC4* ODP";
$resn_exch{" C5B ODP"}="AC5* ODP"; $resn_exch{" O5B ODP"}="AO5* ODP"; $resn_exch{" PA  ODP"}=" AP  ODP";
$resn_exch{" O1A ODP"}=" AO1 ODP"; $resn_exch{" O2A ODP"}=" AO2 ODP"; $resn_exch{" PN  ODP"}=" NP  ODP";
$resn_exch{" O1N ODP"}=" NO1 ODP"; $resn_exch{" O2N ODP"}=" NO2 ODP"; $resn_exch{" O5D ODP"}="NO5* ODP";
$resn_exch{" C5D ODP"}="NC5* ODP"; $resn_exch{" C4D ODP"}="NC4* ODP"; $resn_exch{" O4D ODP"}="NO4* ODP";
$resn_exch{" C1D ODP"}="NC1* ODP"; $resn_exch{" C2D ODP"}="NC2* ODP"; $resn_exch{" O2D ODP"}="NO2* ODP";
$resn_exch{" C3D ODP"}="NC3* ODP"; $resn_exch{" O3D ODP"}="NO3* ODP"; $resn_exch{" N1N ODP"}=" NN1 ODP";
$resn_exch{" C2N ODP"}=" NC2 ODP"; $resn_exch{" C3N ODP"}=" NC3 ODP"; $resn_exch{" C7N ODP"}=" NC7 ODP";
$resn_exch{" N7N ODP"}=" NN7 ODP"; $resn_exch{" O7N ODP"}=" NO7 ODP"; $resn_exch{" C4N ODP"}=" NC4 ODP";
$resn_exch{" C5N ODP"}=" NC5 ODP"; $resn_exch{" C6N ODP"}=" NC6 ODP"; $resn_exch{" OP1 OIP"}=" O1P OIP";
$resn_exch{" OP2 OIP"}=" O2P OIP"; $resn_exch{" OP3 OIP"}=" O3P OIP"; $resn_exch{" O5' OIP"}=" O5* OIP";
$resn_exch{" C5' OIP"}=" C5* OIP"; $resn_exch{" C4' OIP"}=" C4* OIP"; $resn_exch{" O4' OIP"}=" O4* OIP";
$resn_exch{" C3' OIP"}=" C3* OIP"; $resn_exch{" O3' OIP"}=" O3* OIP"; $resn_exch{" C2' OIP"}=" C2* OIP";
$resn_exch{" C1' OIP"}=" C1* OIP"; $resn_exch{" O6' OLU"}=" O6* OLU"; $resn_exch{" C4' OLU"}=" C4* OLU";
$resn_exch{" N3' OLU"}=" N3* OLU"; $resn_exch{" C2' OLU"}=" C2* OLU"; $resn_exch{" S1' OLU"}=" S1* OLU";
$resn_exch{" C5' OLU"}=" C5* OLU"; $resn_exch{" C1' OMC"}=" C1* OMC"; $resn_exch{" C2' OMC"}=" C2* OMC";
$resn_exch{" O2' OMC"}=" O2* OMC"; $resn_exch{" C3' OMC"}=" C3* OMC"; $resn_exch{" C4' OMC"}=" C4* OMC";
$resn_exch{" O4' OMC"}=" O4* OMC"; $resn_exch{" O3' OMC"}=" O3* OMC"; $resn_exch{" C5' OMC"}=" C5* OMC";
$resn_exch{" O5' OMC"}=" O5* OMC"; $resn_exch{" OP1 OMC"}=" O1P OMC"; $resn_exch{" OP2 OMC"}=" O2P OMC";
$resn_exch{" OP3 OMC"}=" O3P OMC"; $resn_exch{" OP1 OMG"}=" O1P OMG"; $resn_exch{" OP2 OMG"}=" O2P OMG";
$resn_exch{" OP3 OMG"}=" O3P OMG"; $resn_exch{" O5' OMG"}=" O5* OMG"; $resn_exch{" C5' OMG"}=" C5* OMG";
$resn_exch{" C4' OMG"}=" C4* OMG"; $resn_exch{" O4' OMG"}=" O4* OMG"; $resn_exch{" C3' OMG"}=" C3* OMG";
$resn_exch{" O3' OMG"}=" O3* OMG"; $resn_exch{" C2' OMG"}=" C2* OMG"; $resn_exch{" O2' OMG"}=" O2* OMG";
$resn_exch{" C1' OMG"}=" C1* OMG"; $resn_exch{" O5' OMP"}=" O5* OMP"; $resn_exch{" C5' OMP"}=" C5* OMP";
$resn_exch{" C4' OMP"}=" C4* OMP"; $resn_exch{" O4' OMP"}=" O4* OMP"; $resn_exch{" C3' OMP"}=" C3* OMP";
$resn_exch{" O3' OMP"}=" O3* OMP"; $resn_exch{" C2' OMP"}=" C2* OMP"; $resn_exch{" O2' OMP"}=" O2* OMP";
$resn_exch{" C1' OMP"}=" C1* OMP"; $resn_exch{" C1' OMU"}=" C1* OMU"; $resn_exch{" C2' OMU"}=" C2* OMU";
$resn_exch{" O2' OMU"}=" O2* OMU"; $resn_exch{" C3' OMU"}=" C3* OMU"; $resn_exch{" C4' OMU"}=" C4* OMU";
$resn_exch{" O3' OMU"}=" O3* OMU"; $resn_exch{" O4' OMU"}=" O4* OMU"; $resn_exch{" C5' OMU"}=" C5* OMU";
$resn_exch{" O5' OMU"}=" O5* OMU"; $resn_exch{" OP1 OMU"}=" O1P OMU"; $resn_exch{" OP2 OMU"}=" O2P OMU";
$resn_exch{" OP3 OMU"}=" O3P OMU"; $resn_exch{" O3' ONA"}=" O3* ONA"; $resn_exch{" C3' ONA"}=" C3* ONA";
$resn_exch{" C2' ONA"}=" C2* ONA"; $resn_exch{" O2' ONA"}=" O2* ONA"; $resn_exch{" C4' ONA"}=" C4* ONA";
$resn_exch{" C5' ONA"}=" C5* ONA"; $resn_exch{" O5' ONA"}=" O5* ONA"; $resn_exch{" O4' ONA"}=" O4* ONA";
$resn_exch{" C1' ONA"}=" C1* ONA"; $resn_exch{" OP3 ONE"}=" O3P ONE"; $resn_exch{" OP1 ONE"}=" O1P ONE";
$resn_exch{" OP2 ONE"}=" O2P ONE"; $resn_exch{" O5' ONE"}=" O5* ONE"; $resn_exch{" C5' ONE"}=" C5* ONE";
$resn_exch{" C4' ONE"}=" C4* ONE"; $resn_exch{" O4' ONE"}=" O4* ONE"; $resn_exch{" C3' ONE"}=" C3* ONE";
$resn_exch{" C2' ONE"}=" C2* ONE"; $resn_exch{" O2' ONE"}=" O2* ONE"; $resn_exch{" O3' ONE"}=" O3* ONE";
$resn_exch{" C1' ONE"}=" C1* ONE"; $resn_exch{" O3' ONM"}=" O3* ONM"; $resn_exch{" C3' ONM"}=" C3* ONM";
$resn_exch{" C2' ONM"}=" C2* ONM"; $resn_exch{" O2' ONM"}=" O2* ONM"; $resn_exch{" C4' ONM"}=" C4* ONM";
$resn_exch{" C5' ONM"}=" C5* ONM"; $resn_exch{" O5' ONM"}=" O5* ONM"; $resn_exch{" O4' ONM"}=" O4* ONM";
$resn_exch{" C1' ONM"}=" C1* ONM"; $resn_exch{" C1' OR5"}=" C1* OR5"; $resn_exch{" O1' OR5"}=" O1* OR5";
$resn_exch{" C2' OR5"}=" C2* OR5"; $resn_exch{" O2' OR5"}=" O2* OR5"; $resn_exch{" C3' OR5"}=" C3* OR5";
$resn_exch{" O3' OR5"}=" O3* OR5"; $resn_exch{" C4' OR5"}=" C4* OR5"; $resn_exch{" O4' OR5"}=" O4* OR5";
$resn_exch{" C5' OR5"}=" C5* OR5"; $resn_exch{" O5' OR5"}=" O5* OR5"; $resn_exch{" C1' OTG"}=" C1* OTG";
$resn_exch{" C2' OTG"}=" C2* OTG"; $resn_exch{" C3' OTG"}=" C3* OTG"; $resn_exch{" C4' OTG"}=" C4* OTG";
$resn_exch{" C5' OTG"}=" C5* OTG"; $resn_exch{" C6' OTG"}=" C6* OTG"; $resn_exch{" C7' OTG"}=" C7* OTG";
$resn_exch{" C8' OTG"}=" C8* OTG"; $resn_exch{" O8' OTG"}=" O8* OTG"; $resn_exch{" P3' OXT"}=" P3* OXT";
$resn_exch{" O3' OXT"}=" O3* OXT"; $resn_exch{" C3' OXT"}=" C3* OXT"; $resn_exch{" C2' OXT"}=" C2* OXT";
$resn_exch{" O2' OXT"}=" O2* OXT"; $resn_exch{" C1' OXT"}=" C1* OXT"; $resn_exch{" O4' OXT"}=" O4* OXT";
$resn_exch{" C4' OXT"}=" C4* OXT"; $resn_exch{" C5' OXT"}=" C5* OXT"; $resn_exch{" O5' OXT"}=" O5* OXT";
$resn_exch{" OXT OXX"}=" O4  OXX"; $resn_exch{" OP1   P"}=" O1P   P"; $resn_exch{" OP2   P"}=" O2P   P";
$resn_exch{" OP3   P"}=" O3P   P"; $resn_exch{" OP1 P1P"}=" O1P P1P"; $resn_exch{" OP2 P1P"}=" O2P P1P";
$resn_exch{" O5' P1P"}=" O5* P1P"; $resn_exch{" C5' P1P"}=" C5* P1P"; $resn_exch{" C4' P1P"}=" C4* P1P";
$resn_exch{" C3' P1P"}=" C3* P1P"; $resn_exch{" O3' P1P"}=" O3* P1P"; $resn_exch{" O6' P2P"}=" O6* P2P";
$resn_exch{" C5' P2P"}=" C5* P2P"; $resn_exch{" C4' P2P"}=" C4* P2P"; $resn_exch{" O5' P2P"}=" O5* P2P";
$resn_exch{" C3' P2P"}=" C3* P2P"; $resn_exch{" O3' P2P"}=" O3* P2P"; $resn_exch{" C2' P2P"}=" C2* P2P";
$resn_exch{" O2' P2P"}=" O2* P2P"; $resn_exch{" C1' P2P"}=" C1* P2P"; $resn_exch{" OP1 P2T"}=" O1P P2T";
$resn_exch{" OP2 P2T"}=" O2P P2T"; $resn_exch{" O5' P2T"}=" O5* P2T"; $resn_exch{" C5' P2T"}=" C5* P2T";
$resn_exch{" C4' P2T"}=" C4* P2T"; $resn_exch{" O4' P2T"}=" O4* P2T"; $resn_exch{" C1' P2T"}=" C1* P2T";
$resn_exch{" C2' P2T"}=" C2* P2T"; $resn_exch{" O2' P2T"}=" O2* P2T"; $resn_exch{" C3' P2T"}=" C3* P2T";
$resn_exch{" O3' P2T"}=" O3* P2T"; $resn_exch{" OP3 P2T"}=" O3P P2T"; $resn_exch{" C1' P2U"}=" C1* P2U";
$resn_exch{" C2' P2U"}=" C2* P2U"; $resn_exch{" C3' P2U"}=" C3* P2U"; $resn_exch{" C4' P2U"}=" C4* P2U";
$resn_exch{" O3' P2U"}=" O3* P2U"; $resn_exch{" O4' P2U"}=" O4* P2U"; $resn_exch{" C5' P2U"}=" C5* P2U";
$resn_exch{" O5' P2U"}=" O5* P2U"; $resn_exch{" OP1 P2U"}=" O1P P2U"; $resn_exch{" OP2 P2U"}=" O2P P2U";
$resn_exch{" OP3 P2U"}=" O3P P2U"; $resn_exch{" O5' P5A"}=" O5* P5A"; $resn_exch{" C5' P5A"}=" C5* P5A";
$resn_exch{" C4' P5A"}=" C4* P5A"; $resn_exch{" O4' P5A"}=" O4* P5A"; $resn_exch{" C1' P5A"}=" C1* P5A";
$resn_exch{" C2' P5A"}=" C2* P5A"; $resn_exch{" O2' P5A"}=" O2* P5A"; $resn_exch{" C3' P5A"}=" C3* P5A";
$resn_exch{" O3' P5A"}=" O3* P5A"; $resn_exch{" OP1 P5P"}=" O1P P5P"; $resn_exch{" OP2 P5P"}=" O2P P5P";
$resn_exch{" OP3 P5P"}=" O3P P5P"; $resn_exch{" C5' PA7"}=" C5* PA7"; $resn_exch{" O5' PA7"}=" O5* PA7";
$resn_exch{" N5' PA7"}=" N5* PA7"; $resn_exch{" C4' PA7"}=" C4* PA7"; $resn_exch{" O4' PA7"}=" O4* PA7";
$resn_exch{" C3' PA7"}=" C3* PA7"; $resn_exch{" O3' PA7"}=" O3* PA7"; $resn_exch{" C2' PA7"}=" C2* PA7";
$resn_exch{" O2' PA7"}=" O2* PA7"; $resn_exch{" C1' PA7"}=" C1* PA7"; $resn_exch{" PA  PAD"}=" AP  PAD";
$resn_exch{" O1A PAD"}=" AO1 PAD"; $resn_exch{" O2A PAD"}=" AO2 PAD"; $resn_exch{" O5B PAD"}="AO5* PAD";
$resn_exch{" C5B PAD"}="AC5* PAD"; $resn_exch{" C4B PAD"}="AC4* PAD"; $resn_exch{" O4B PAD"}="AO4* PAD";
$resn_exch{" C3B PAD"}="AC3* PAD"; $resn_exch{" O3B PAD"}="AO3* PAD"; $resn_exch{" C2B PAD"}="AC2* PAD";
$resn_exch{" O2B PAD"}="AO2* PAD"; $resn_exch{" C1B PAD"}="AC1* PAD"; $resn_exch{" N9A PAD"}=" AN9 PAD";
$resn_exch{" C8A PAD"}=" AC8 PAD"; $resn_exch{" N7A PAD"}=" AN7 PAD"; $resn_exch{" C5A PAD"}=" AC5 PAD";
$resn_exch{" C6A PAD"}=" AC6 PAD"; $resn_exch{" N6A PAD"}=" AN6 PAD"; $resn_exch{" N1A PAD"}=" AN1 PAD";
$resn_exch{" C2A PAD"}=" AC2 PAD"; $resn_exch{" N3A PAD"}=" AN3 PAD"; $resn_exch{" C4A PAD"}=" AC4 PAD";
$resn_exch{" PN  PAD"}=" NP  PAD"; $resn_exch{" O1N PAD"}=" NO1 PAD"; $resn_exch{" O2N PAD"}=" NO2 PAD";
$resn_exch{" O5D PAD"}="NO5* PAD"; $resn_exch{" C5D PAD"}="NC5* PAD"; $resn_exch{" C4D PAD"}="NC4* PAD";
$resn_exch{" O4D PAD"}="NO4* PAD"; $resn_exch{" C3D PAD"}="NC3* PAD"; $resn_exch{" O3D PAD"}="NO3* PAD";
$resn_exch{" C2D PAD"}="NC2* PAD"; $resn_exch{" O2D PAD"}="NO2* PAD"; $resn_exch{" C1D PAD"}="NC1* PAD";
$resn_exch{" C1N PAD"}=" NC1 PAD"; $resn_exch{" N2N PAD"}=" NN2 PAD"; $resn_exch{" C3N PAD"}=" NC3 PAD";
$resn_exch{" C7N PAD"}=" NC7 PAD"; $resn_exch{" O7N PAD"}=" NO7 PAD"; $resn_exch{" N7N PAD"}=" NN7 PAD";
$resn_exch{" C4N PAD"}=" NC4 PAD"; $resn_exch{" C5N PAD"}=" NC5 PAD"; $resn_exch{" C6N PAD"}=" NC6 PAD";
$resn_exch{" O5' PAJ"}=" O5* PAJ"; $resn_exch{" C5' PAJ"}=" C5* PAJ"; $resn_exch{" C4' PAJ"}=" C4* PAJ";
$resn_exch{" O4' PAJ"}=" O4* PAJ"; $resn_exch{" C3' PAJ"}=" C3* PAJ"; $resn_exch{" O3' PAJ"}=" O3* PAJ";
$resn_exch{" C2' PAJ"}=" C2* PAJ"; $resn_exch{" O2' PAJ"}=" O2* PAJ"; $resn_exch{" C1' PAJ"}=" C1* PAJ";
$resn_exch{" O5' PAP"}=" O5* PAP"; $resn_exch{" C5' PAP"}=" C5* PAP"; $resn_exch{" C4' PAP"}=" C4* PAP";
$resn_exch{" O4' PAP"}=" O4* PAP"; $resn_exch{" C3' PAP"}=" C3* PAP"; $resn_exch{" O3' PAP"}=" O3* PAP";
$resn_exch{" C2' PAP"}=" C2* PAP"; $resn_exch{" O2' PAP"}=" O2* PAP"; $resn_exch{" C1' PAP"}=" C1* PAP";
$resn_exch{" O5' PAU"}=" O5* PAU"; $resn_exch{" O6' PAU"}=" O6* PAU"; $resn_exch{" O5' PAX"}=" O5* PAX";
$resn_exch{" C5' PAX"}=" C5* PAX"; $resn_exch{" C4' PAX"}=" C4* PAX"; $resn_exch{" O4' PAX"}=" O4* PAX";
$resn_exch{" C1' PAX"}=" C1* PAX"; $resn_exch{" C2' PAX"}=" C2* PAX"; $resn_exch{" O2' PAX"}=" O2* PAX";
$resn_exch{" C3' PAX"}=" C3* PAX"; $resn_exch{" O3' PAX"}=" O3* PAX"; $resn_exch{" N1' PBO"}=" N1* PBO";
$resn_exch{" O5' PBT"}=" O5* PBT"; $resn_exch{" C5' PBT"}=" C5* PBT"; $resn_exch{" C4' PBT"}=" C4* PBT";
$resn_exch{" O4' PBT"}=" O4* PBT"; $resn_exch{" C3' PBT"}=" C3* PBT"; $resn_exch{" O3' PBT"}=" O3* PBT";
$resn_exch{" C2' PBT"}=" C2* PBT"; $resn_exch{" C1' PBT"}=" C1* PBT"; $resn_exch{" CX5 PC3"}="1C22 PC3";
$resn_exch{" C1' PCD"}=" C1* PCD"; $resn_exch{" C2D PCD"}=" C2* PCD"; $resn_exch{" O2' PCD"}=" O2* PCD";
$resn_exch{" C3' PCD"}=" C3* PCD"; $resn_exch{" C4D PCD"}=" C4* PCD"; $resn_exch{" O4D PCD"}=" O4* PCD";
$resn_exch{" O3' PCD"}=" O3* PCD"; $resn_exch{" C5' PCD"}=" C5* PCD"; $resn_exch{" O5' PCD"}=" O5* PCD";
$resn_exch{" O5' PCG"}=" O5* PCG"; $resn_exch{" C5' PCG"}=" C5* PCG"; $resn_exch{" C4' PCG"}=" C4* PCG";
$resn_exch{" O4' PCG"}=" O4* PCG"; $resn_exch{" C3' PCG"}=" C3* PCG"; $resn_exch{" O3' PCG"}=" O3* PCG";
$resn_exch{" C2' PCG"}=" C2* PCG"; $resn_exch{" O2' PCG"}=" O2* PCG"; $resn_exch{" C1' PCG"}=" C1* PCG";
$resn_exch{" NA  PCU"}=" N_A PCU"; $resn_exch{" NB  PCU"}=" N_B PCU"; $resn_exch{" NC  PCU"}=" N_C PCU";
$resn_exch{" ND  PCU"}=" N_D PCU"; $resn_exch{" O'' PD7"}=" O\"  PD7"; $resn_exch{" C'' PD7"}=" C\"  PD7";
$resn_exch{"C1'' PD7"}="C1\"  PD7"; $resn_exch{"C2'' PD7"}="C2\"  PD7"; $resn_exch{"C3'' PD7"}="C3\"  PD7";
$resn_exch{"C4'' PD7"}="C4\"  PD7"; $resn_exch{"C5'' PD7"}="C5\"  PD7"; $resn_exch{"C6'' PD7"}="C6\"  PD7";
$resn_exch{" OP3 PDU"}=" O3P PDU"; $resn_exch{" OP1 PDU"}=" O1P PDU"; $resn_exch{" OP2 PDU"}=" O2P PDU";
$resn_exch{" O5' PDU"}=" O5* PDU"; $resn_exch{" C5' PDU"}=" C5* PDU"; $resn_exch{" C4' PDU"}=" C4* PDU";
$resn_exch{" O4' PDU"}=" O4* PDU"; $resn_exch{" C3' PDU"}=" C3* PDU"; $resn_exch{" O3' PDU"}=" O3* PDU";
$resn_exch{" C2' PDU"}=" C2* PDU"; $resn_exch{" C1' PDU"}=" C1* PDU"; $resn_exch{" O1X PDX"}="O1S* PDX";
$resn_exch{" O2X PDX"}="O2S* PDX"; $resn_exch{" O3X PDX"}="O3S* PDX"; $resn_exch{" S'  PDX"}=" S*  PDX";
$resn_exch{" C4' PE2"}=" C4* PE2"; $resn_exch{" O4' PE2"}=" O4* PE2"; $resn_exch{" C3' PE2"}=" C3* PE2";
$resn_exch{" O3' PE2"}=" O3* PE2"; $resn_exch{" C2' PE2"}=" C2* PE2"; $resn_exch{" C1' PE2"}=" C1* PE2";
$resn_exch{" OP1 PE6"}="1OP  PE6"; $resn_exch{" OP2 PE6"}="2OP  PE6"; $resn_exch{" OP3 PE6"}="3OP  PE6";
$resn_exch{" O5' PED"}=" O5* PED"; $resn_exch{" C2' PED"}=" C2* PED"; $resn_exch{" C5' PED"}=" C5* PED";
$resn_exch{" C4' PED"}=" C4* PED"; $resn_exch{" O4' PED"}=" O4* PED"; $resn_exch{" C3' PED"}=" C3* PED";
$resn_exch{" C1' PED"}=" C1* PED"; $resn_exch{" O3' PED"}=" O3* PED"; $resn_exch{" C1' PEL"}=" C1* PEL";
$resn_exch{" C2' PEL"}=" C2* PEL"; $resn_exch{" C3' PEL"}=" C3* PEL"; $resn_exch{" C4' PEL"}=" C4* PEL";
$resn_exch{" C5' PEL"}=" C5* PEL"; $resn_exch{" C6' PEL"}=" C6* PEL"; $resn_exch{" OXT PG1"}=" O1  PG1";
$resn_exch{" OP1 PG7"}=" O1P PG7"; $resn_exch{" OP2 PG7"}=" O2P PG7"; $resn_exch{" O5' PG7"}=" O5* PG7";
$resn_exch{" O3' PG7"}=" O3* PG7"; $resn_exch{" C1' PG7"}=" C1* PG7"; $resn_exch{" C2' PG7"}=" C2* PG7";
$resn_exch{" C3' PG7"}=" C3* PG7"; $resn_exch{" C4' PG7"}=" C4* PG7"; $resn_exch{" C5' PG7"}=" C5* PG7";
$resn_exch{" O4' PG7"}=" O4* PG7"; $resn_exch{" OP3 PG7"}=" O3P PG7"; $resn_exch{" OP1 PGN"}=" O1P PGN";
$resn_exch{" OP2 PGN"}=" O2P PGN"; $resn_exch{" OP3 PGN"}=" O3P PGN"; $resn_exch{" O5' PGN"}=" O5* PGN";
$resn_exch{" C5' PGN"}=" C5* PGN"; $resn_exch{" C4' PGN"}=" C4* PGN"; $resn_exch{" O4' PGN"}=" O4* PGN";
$resn_exch{" C3' PGN"}=" C3* PGN"; $resn_exch{" O3' PGN"}=" O3* PGN"; $resn_exch{" C2' PGN"}=" C2* PGN";
$resn_exch{" C1' PGN"}=" C1* PGN"; $resn_exch{" OP1 PGP"}=" O1P PGP"; $resn_exch{" OP2 PGP"}=" O2P PGP";
$resn_exch{" OP3 PGP"}=" O3P PGP"; $resn_exch{" O5' PGP"}=" O5* PGP"; $resn_exch{" C5' PGP"}=" C5* PGP";
$resn_exch{" C4' PGP"}=" C4* PGP"; $resn_exch{" O4' PGP"}=" O4* PGP"; $resn_exch{" C3' PGP"}=" C3* PGP";
$resn_exch{" O3' PGP"}=" O3* PGP"; $resn_exch{" C2' PGP"}=" C2* PGP"; $resn_exch{" O2' PGP"}=" O2* PGP";
$resn_exch{" C1' PGP"}=" C1* PGP"; $resn_exch{" O5' PGS"}=" O5* PGS"; $resn_exch{" C5' PGS"}=" C5* PGS";
$resn_exch{" C4' PGS"}=" C4* PGS"; $resn_exch{" O4' PGS"}=" O4* PGS"; $resn_exch{" C3' PGS"}=" C3* PGS";
$resn_exch{" O3' PGS"}=" O3* PGS"; $resn_exch{" C2' PGS"}=" C2* PGS"; $resn_exch{" O2' PGS"}=" O2* PGS";
$resn_exch{" C1' PGS"}=" C1* PGS"; $resn_exch{" CAD PGW"}=" C1  PGW"; $resn_exch{" OAE PGW"}=" O1  PGW";
$resn_exch{" OAF PGW"}=" O2  PGW"; $resn_exch{" P   PGW"}=" P1  PGW"; $resn_exch{" C01 PGW"}=" C2  PGW";
$resn_exch{" C1  PGW"}=" C3  PGW"; $resn_exch{" O01 PGW"}=" O3  PGW"; $resn_exch{" C02 PGW"}=" C4  PGW";
$resn_exch{" C2  PGW"}=" C5  PGW"; $resn_exch{" O02 PGW"}=" O4  PGW"; $resn_exch{" C03 PGW"}=" C6  PGW";
$resn_exch{" C3  PGW"}=" C7  PGW"; $resn_exch{" O03 PGW"}=" O5  PGW"; $resn_exch{" C04 PGW"}=" C8  PGW";
$resn_exch{" C4  PGW"}=" C9  PGW"; $resn_exch{" O04 PGW"}=" O6  PGW"; $resn_exch{" C05 PGW"}=" C10 PGW";
$resn_exch{" C5  PGW"}=" C11 PGW"; $resn_exch{" C06 PGW"}=" C12 PGW"; $resn_exch{" C6  PGW"}=" C13 PGW";
$resn_exch{" C07 PGW"}=" C14 PGW"; $resn_exch{" C7  PGW"}=" C15 PGW"; $resn_exch{" C08 PGW"}=" C16 PGW";
$resn_exch{" C8  PGW"}=" C17 PGW"; $resn_exch{" C09 PGW"}=" C18 PGW"; $resn_exch{" C9  PGW"}=" C19 PGW";
$resn_exch{" C10 PGW"}=" C20 PGW"; $resn_exch{" C11 PGW"}=" C21 PGW"; $resn_exch{" O11 PGW"}=" O7  PGW";
$resn_exch{" C12 PGW"}=" C22 PGW"; $resn_exch{" O12 PGW"}=" O8  PGW"; $resn_exch{" C13 PGW"}=" C23 PGW";
$resn_exch{" O13 PGW"}=" O9  PGW"; $resn_exch{" C14 PGW"}=" C24 PGW"; $resn_exch{" O14 PGW"}=" O10 PGW";
$resn_exch{" C15 PGW"}=" C25 PGW"; $resn_exch{" C16 PGW"}=" C26 PGW"; $resn_exch{" C17 PGW"}=" C27 PGW";
$resn_exch{" C18 PGW"}=" C28 PGW"; $resn_exch{" C19 PGW"}=" C29 PGW"; $resn_exch{" C20 PGW"}=" C30 PGW";
$resn_exch{" C21 PGW"}=" C31 PGW"; $resn_exch{" C22 PGW"}=" C32 PGW"; $resn_exch{" C23 PGW"}=" C33 PGW";
$resn_exch{" C24 PGW"}=" C34 PGW"; $resn_exch{" C25 PGW"}=" C35 PGW"; $resn_exch{" C26 PGW"}=" C36 PGW";
$resn_exch{" C27 PGW"}=" C37 PGW"; $resn_exch{" C28 PGW"}=" C38 PGW"; $resn_exch{" C29 PGW"}=" C39 PGW";
$resn_exch{" C30 PGW"}=" C40 PGW"; $resn_exch{" NA  PHO"}=" N_A PHO"; $resn_exch{" NB  PHO"}=" N_B PHO";
$resn_exch{" NC  PHO"}=" N_C PHO"; $resn_exch{" ND  PHO"}=" N_D PHO"; $resn_exch{" CDB PHP"}="1CD1 PHP";
$resn_exch{" CDL PHP"}="2CD1 PHP"; $resn_exch{" CEB PHP"}="1CE1 PHP"; $resn_exch{" CEL PHP"}="2CE1 PHP";
$resn_exch{" CGL PHP"}="1CG2 PHP"; $resn_exch{" CGM PHP"}="2CG2 PHP"; $resn_exch{" CDD PHP"}="1CD3 PHP";
$resn_exch{" CDN PHP"}="2CD3 PHP"; $resn_exch{" CED PHP"}="1CE3 PHP"; $resn_exch{" CEN PHP"}="2CE3 PHP";
$resn_exch{" CDE PHP"}="1CD4 PHP"; $resn_exch{" CDO PHP"}="2CD4 PHP"; $resn_exch{" CEE PHP"}="1CE4 PHP";
$resn_exch{" CEO PHP"}="2CE4 PHP"; $resn_exch{" CGF PHP"}="1CG5 PHP"; $resn_exch{" CGG PHP"}="2CG5 PHP";
$resn_exch{" CDG PHP"}="1CD6 PHP"; $resn_exch{" CEG PHP"}="1CE6 PHP"; $resn_exch{" CEQ PHP"}="2CE6 PHP";
$resn_exch{" CDQ PHP"}="2CD6 PHP"; $resn_exch{" CRA PII"}="CR10 PII"; $resn_exch{" CRB PII"}="CR11 PII";
$resn_exch{" CRC PII"}="CR12 PII"; $resn_exch{" CRD PII"}="CR13 PII"; $resn_exch{" CRE PII"}="CR14 PII";
$resn_exch{" CRF PII"}="CR15 PII"; $resn_exch{" CRG PII"}="CR16 PII"; $resn_exch{" CLA PII"}="CL10 PII";
$resn_exch{" CLB PII"}="CL11 PII"; $resn_exch{" CLC PII"}="CL12 PII"; $resn_exch{" CLD PII"}="CL13 PII";
$resn_exch{" CLE PII"}="CL14 PII"; $resn_exch{" CLF PII"}="CL15 PII"; $resn_exch{" CA' PLC"}="'C10 PLC";
$resn_exch{" CB' PLC"}="'C11 PLC"; $resn_exch{" CB  PLC"}=" C\"  PLC"; $resn_exch{" C1B PLC"}=" C1\" PLC";
$resn_exch{" C2B PLC"}=" C2\" PLC"; $resn_exch{" C3B PLC"}=" C3\" PLC"; $resn_exch{" C4B PLC"}=" C4\" PLC";
$resn_exch{" C5B PLC"}=" C5\" PLC"; $resn_exch{" C6B PLC"}=" C6\" PLC"; $resn_exch{" C7B PLC"}=" C7\" PLC";
$resn_exch{" C8B PLC"}=" C8\" PLC"; $resn_exch{" C9B PLC"}=" C9\" PLC"; $resn_exch{" CAA PLC"}="\"C10 PLC";
$resn_exch{" CBA PLC"}="\"C11 PLC"; $resn_exch{" OB  PLC"}=" O\"  PLC"; $resn_exch{" CA' PLD"}="'C10 PLD";
$resn_exch{" CB' PLD"}="'C11 PLD"; $resn_exch{" CB  PLD"}=" C\"  PLD"; $resn_exch{" C1B PLD"}=" C1\" PLD";
$resn_exch{" C2B PLD"}=" C2\" PLD"; $resn_exch{" C3B PLD"}=" C3\" PLD"; $resn_exch{" C4B PLD"}=" C4\" PLD";
$resn_exch{" C5B PLD"}=" C5\" PLD"; $resn_exch{" C6B PLD"}=" C6\" PLD"; $resn_exch{" C7B PLD"}=" C7\" PLD";
$resn_exch{" C8B PLD"}=" C8\" PLD"; $resn_exch{" C9B PLD"}=" C9\" PLD"; $resn_exch{" CAA PLD"}="\"C10 PLD";
$resn_exch{" CBA PLD"}="\"C11 PLD"; $resn_exch{" OB  PLD"}=" O\"  PLD"; $resn_exch{" CC' PLD"}="'C12 PLD";
$resn_exch{" CD' PLD"}="'C13 PLD"; $resn_exch{" CE' PLD"}="'C14 PLD"; $resn_exch{" CF' PLD"}="'C15 PLD";
$resn_exch{" CG' PLD"}="'C16 PLD"; $resn_exch{" CH' PLD"}="'C17 PLD"; $resn_exch{" CI' PLD"}="'C18 PLD";
$resn_exch{" CJ' PLD"}="'C19 PLD"; $resn_exch{" CK' PLD"}="'C20 PLD"; $resn_exch{" CCA PLD"}="\"C12 PLD";
$resn_exch{" CDA PLD"}="\"C13 PLD"; $resn_exch{" CEA PLD"}="\"C14 PLD"; $resn_exch{" CFA PLD"}="\"C15 PLD";
$resn_exch{" CGA PLD"}="\"C16 PLD"; $resn_exch{" CHA PLD"}="\"C17 PLD"; $resn_exch{" CIA PLD"}="\"C18 PLD";
$resn_exch{" CJA PLD"}="\"C19 PLD"; $resn_exch{" CKA PLD"}="\"C20 PLD"; $resn_exch{" OXT PM3"}=" O   PM3";
$resn_exch{" O   PM3"}=" OXT PM3"; $resn_exch{" C2B PMD"}=" C2* PMD"; $resn_exch{" C4B PMD"}=" C4* PMD";
$resn_exch{" C6B PMD"}=" C6* PMD"; $resn_exch{" C7B PMD"}=" C7* PMD"; $resn_exch{" C9B PMD"}=" C9* PMD";
$resn_exch{" CA' PMD"}="'C10 PMD"; $resn_exch{" CB' PMD"}="'C11 PMD"; $resn_exch{" CC' PMD"}="'C12 PMD";
$resn_exch{" N1' PMD"}=" N1* PMD"; $resn_exch{" N2' PMD"}=" N2* PMD"; $resn_exch{" N3' PMD"}=" N3* PMD";
$resn_exch{" N4' PMD"}=" N4* PMD"; $resn_exch{" N5' PMD"}=" N5* PMD"; $resn_exch{" N8' PMD"}=" N8* PMD";
$resn_exch{" O5' PMO"}=" O5* PMO"; $resn_exch{" C5' PMO"}=" C5* PMO"; $resn_exch{" C4' PMO"}=" C4* PMO";
$resn_exch{" O4' PMO"}=" O4* PMO"; $resn_exch{" C3' PMO"}=" C3* PMO"; $resn_exch{" O3' PMO"}=" O3* PMO";
$resn_exch{" C2' PMO"}=" C2* PMO"; $resn_exch{" O2' PMO"}=" O2* PMO"; $resn_exch{" C1' PMO"}=" C1* PMO";
$resn_exch{" O11 PMT"}="1OP' PMT"; $resn_exch{" O12 PMT"}="3OP' PMT"; $resn_exch{" O13 PMT"}="2OP' PMT";
$resn_exch{" OP2 PMT"}=" O2P PMT"; $resn_exch{" OP1 PMT"}=" O1P PMT"; $resn_exch{" OP3 PMT"}=" O3P PMT";
$resn_exch{" O5B PMT"}=" O5* PMT"; $resn_exch{" C5B PMT"}=" C5* PMT"; $resn_exch{" C4B PMT"}=" C4* PMT";
$resn_exch{" O4B PMT"}=" O4* PMT"; $resn_exch{" C3B PMT"}=" C3* PMT"; $resn_exch{" O3' PMT"}=" O3* PMT";
$resn_exch{" C2B PMT"}=" C2* PMT"; $resn_exch{" O2' PMT"}=" O2* PMT"; $resn_exch{" C1B PMT"}=" C1* PMT";
$resn_exch{" NA  PNI"}=" N_A PNI"; $resn_exch{" NB  PNI"}=" N_B PNI"; $resn_exch{" NC  PNI"}=" N_C PNI";
$resn_exch{" ND  PNI"}=" N_D PNI"; $resn_exch{" CX3 POH"}="1C11 POH"; $resn_exch{" C71 POH"}="1C7  POH";
$resn_exch{" C81 POH"}="1C8  POH"; $resn_exch{" C91 POH"}="1C9  POH"; $resn_exch{" CXN POH"}="1C13 POH";
$resn_exch{" CXD POH"}="1C12 POH"; $resn_exch{" NXT POH"}="1N10 POH"; $resn_exch{" NA  POH"}=" N_A POH";
$resn_exch{" NB  POH"}=" N_B POH"; $resn_exch{" NC  POH"}=" N_C POH"; $resn_exch{" ND  POH"}=" N_D POH";
$resn_exch{" CX4 POH"}="2C11 POH"; $resn_exch{" C72 POH"}="2C7  POH"; $resn_exch{" C82 POH"}="2C8  POH";
$resn_exch{" C92 POH"}="2C9  POH"; $resn_exch{" CXO POH"}="2C13 POH"; $resn_exch{" CXE POH"}="2C12 POH";
$resn_exch{" NXU POH"}="2N10 POH"; $resn_exch{" CX5 POH"}="3C11 POH"; $resn_exch{" C73 POH"}="3C7  POH";
$resn_exch{" C83 POH"}="3C8  POH"; $resn_exch{" C93 POH"}="3C9  POH"; $resn_exch{" CXP POH"}="3C13 POH";
$resn_exch{" CXF POH"}="3C12 POH"; $resn_exch{" NXV POH"}="3N10 POH"; $resn_exch{" CX6 POH"}="4C11 POH";
$resn_exch{" C74 POH"}="4C7  POH"; $resn_exch{" C84 POH"}="4C8  POH"; $resn_exch{" C94 POH"}="4C9  POH";
$resn_exch{" CXQ POH"}="4C13 POH"; $resn_exch{" CXG POH"}="4C12 POH"; $resn_exch{" NXW POH"}="4N10 POH";
$resn_exch{" NA  POR"}=" N_A POR"; $resn_exch{" NB  POR"}=" N_B POR"; $resn_exch{" NC  POR"}=" N_C POR";
$resn_exch{" ND  POR"}=" N_D POR"; $resn_exch{" NA  PP9"}=" N_A PP9"; $resn_exch{" NB  PP9"}=" N_B PP9";
$resn_exch{" NC  PP9"}=" N_C PP9"; $resn_exch{" ND  PP9"}=" N_D PP9"; $resn_exch{" O5' PPS"}=" O5* PPS";
$resn_exch{" C5' PPS"}=" C5* PPS"; $resn_exch{" C4' PPS"}=" C4* PPS"; $resn_exch{" O4' PPS"}=" O4* PPS";
$resn_exch{" C1' PPS"}=" C1* PPS"; $resn_exch{" C2' PPS"}=" C2* PPS"; $resn_exch{" O2' PPS"}=" O2* PPS";
$resn_exch{" C3' PPS"}=" C3* PPS"; $resn_exch{" O3' PPS"}=" O3* PPS"; $resn_exch{" OP1 PPU"}=" O1P PPU";
$resn_exch{" OP2 PPU"}=" O2P PPU"; $resn_exch{" OP3 PPU"}=" O3P PPU"; $resn_exch{" O5' PPU"}=" O5* PPU";
$resn_exch{" C5' PPU"}=" C5* PPU"; $resn_exch{" C4' PPU"}=" C4* PPU"; $resn_exch{" O4' PPU"}=" O4* PPU";
$resn_exch{" C3' PPU"}=" C3* PPU"; $resn_exch{" N3' PPU"}=" N3* PPU"; $resn_exch{" C2' PPU"}=" C2* PPU";
$resn_exch{" O2' PPU"}=" O2* PPU"; $resn_exch{" C1' PPU"}=" C1* PPU"; $resn_exch{" OP1 PPW"}=" O1P PPW";
$resn_exch{" OP2 PPW"}=" O2P PPW"; $resn_exch{" O5' PPW"}=" O5* PPW"; $resn_exch{" C2' PPW"}=" C2* PPW";
$resn_exch{" C5' PPW"}=" C5* PPW"; $resn_exch{" C4' PPW"}=" C4* PPW"; $resn_exch{" O4' PPW"}=" O4* PPW";
$resn_exch{" C1' PPW"}=" C1* PPW"; $resn_exch{" C3' PPW"}=" C3* PPW"; $resn_exch{" O3' PPW"}=" O3* PPW";
$resn_exch{" OP3 PPW"}=" O3P PPW"; $resn_exch{" OP3 PQ1"}=" O3P PQ1"; $resn_exch{" OP2 PQ1"}=" O2P PQ1";
$resn_exch{" O5' PQ1"}=" O5* PQ1"; $resn_exch{" C5' PQ1"}=" C5* PQ1"; $resn_exch{" C4' PQ1"}=" C4* PQ1";
$resn_exch{" O4' PQ1"}=" O4* PQ1"; $resn_exch{" C1' PQ1"}=" C1* PQ1"; $resn_exch{" C2' PQ1"}=" C2* PQ1";
$resn_exch{" O2' PQ1"}=" O2* PQ1"; $resn_exch{" C3' PQ1"}=" C3* PQ1"; $resn_exch{" O3' PQ1"}=" O3* PQ1";
$resn_exch{" OP1 PQ1"}=" O1P PQ1"; $resn_exch{" OXT PR0"}=" OR  PR0"; $resn_exch{" OP1 PR5"}=" O1P PR5";
$resn_exch{" OP2 PR5"}=" O2P PR5"; $resn_exch{" O5' PR5"}=" O5* PR5"; $resn_exch{" C5' PR5"}=" C5* PR5";
$resn_exch{" C4' PR5"}=" C4* PR5"; $resn_exch{" O4' PR5"}=" O4* PR5"; $resn_exch{" C1' PR5"}=" C1* PR5";
$resn_exch{" C3' PR5"}=" C3* PR5"; $resn_exch{" O3' PR5"}=" O3* PR5"; $resn_exch{" C2' PR5"}=" C2* PR5";
$resn_exch{" O2' PR5"}=" O2* PR5"; $resn_exch{" OP3 PR5"}=" O3P PR5"; $resn_exch{" N1' PRD"}=" N1* PRD";
$resn_exch{" N8' PRD"}=" N8* PRD"; $resn_exch{" C7B PRD"}=" C7* PRD"; $resn_exch{" C6B PRD"}=" C6* PRD";
$resn_exch{" N6' PRD"}=" N6* PRD"; $resn_exch{" C5B PRD"}=" C5* PRD"; $resn_exch{" C4B PRD"}=" C4* PRD";
$resn_exch{" N4' PRD"}=" N4* PRD"; $resn_exch{" N3' PRD"}=" N3* PRD"; $resn_exch{" C2B PRD"}=" C2* PRD";
$resn_exch{" N2' PRD"}=" N2* PRD"; $resn_exch{" O71 PRE"}="1O7  PRE"; $resn_exch{" O72 PRE"}="2O7  PRE";
$resn_exch{" O'L PRE"}="1O2' PRE"; $resn_exch{" O'M PRE"}="2O2' PRE"; $resn_exch{" C1' PRH"}=" C1* PRH";
$resn_exch{" C2' PRH"}=" C2* PRH"; $resn_exch{" O2' PRH"}=" O2* PRH"; $resn_exch{" C3' PRH"}=" C3* PRH";
$resn_exch{" O3' PRH"}=" O3* PRH"; $resn_exch{" C4' PRH"}=" C4* PRH"; $resn_exch{" O4' PRH"}=" O4* PRH";
$resn_exch{" C5' PRH"}=" C5* PRH"; $resn_exch{" O5' PRH"}=" O5* PRH"; $resn_exch{" C1' PRN"}=" C1* PRN";
$resn_exch{" C2' PRN"}=" C2* PRN"; $resn_exch{" C3' PRN"}=" C3* PRN"; $resn_exch{" O3' PRN"}=" O3* PRN";
$resn_exch{" C4' PRN"}=" C4* PRN"; $resn_exch{" O4' PRN"}=" O4* PRN"; $resn_exch{" C5' PRN"}=" C5* PRN";
$resn_exch{" O5' PRN"}=" O5* PRN"; $resn_exch{" OP1 PRN"}=" O1P PRN"; $resn_exch{" OP2 PRN"}=" O2P PRN";
$resn_exch{" OP3 PRN"}=" O3P PRN"; $resn_exch{" O5' PRT"}=" O5* PRT"; $resn_exch{" C5' PRT"}=" C5* PRT";
$resn_exch{" C4' PRT"}=" C4* PRT"; $resn_exch{" O4' PRT"}=" O4* PRT"; $resn_exch{" C3' PRT"}=" C3* PRT";
$resn_exch{" O3' PRT"}=" O3* PRT"; $resn_exch{" C2' PRT"}=" C2* PRT"; $resn_exch{" O2' PRT"}=" O2* PRT";
$resn_exch{" C1' PRT"}=" C1* PRT"; $resn_exch{" O2' PRX"}=" O2* PRX"; $resn_exch{" C2' PRX"}=" C2* PRX";
$resn_exch{" C1' PRX"}=" C1* PRX"; $resn_exch{" O4' PRX"}=" O4* PRX"; $resn_exch{" C4' PRX"}=" C4* PRX";
$resn_exch{" C5' PRX"}=" C5* PRX"; $resn_exch{" O5' PRX"}=" O5* PRX"; $resn_exch{" C3' PRX"}=" C3* PRX";
$resn_exch{" O3' PRX"}=" O3* PRX"; $resn_exch{" OP1 PST"}=" O1P PST"; $resn_exch{" OP2 PST"}=" O2P PST";
$resn_exch{" O5' PST"}=" O5* PST"; $resn_exch{" C5' PST"}=" C5* PST"; $resn_exch{" C4' PST"}=" C4* PST";
$resn_exch{" O4' PST"}=" O4* PST"; $resn_exch{" C3' PST"}=" C3* PST"; $resn_exch{" O3' PST"}=" O3* PST";
$resn_exch{" C2' PST"}=" C2* PST"; $resn_exch{" C1' PST"}=" C1* PST"; $resn_exch{" C1' PSU"}=" C1* PSU";
$resn_exch{" C2' PSU"}=" C2* PSU"; $resn_exch{" O2' PSU"}=" O2* PSU"; $resn_exch{" C3' PSU"}=" C3* PSU";
$resn_exch{" C4' PSU"}=" C4* PSU"; $resn_exch{" O3' PSU"}=" O3* PSU"; $resn_exch{" O4' PSU"}=" O4* PSU";
$resn_exch{" C5' PSU"}=" C5* PSU"; $resn_exch{" O5' PSU"}=" O5* PSU"; $resn_exch{" OP1 PSU"}=" O1P PSU";
$resn_exch{" OP2 PSU"}=" O2P PSU"; $resn_exch{" OP3 PSU"}=" O3P PSU"; $resn_exch{" C1' PTP"}=" C1* PTP";
$resn_exch{" C2' PTP"}=" C2* PTP"; $resn_exch{" C3' PTP"}=" C3* PTP"; $resn_exch{" O3' PTP"}=" O3* PTP";
$resn_exch{" P3' PTP"}=" P3* PTP"; $resn_exch{" C4' PTP"}=" C4* PTP"; $resn_exch{" O4' PTP"}=" O4* PTP";
$resn_exch{" C5' PTP"}=" C5* PTP"; $resn_exch{" O5' PTP"}=" O5* PTP"; $resn_exch{" P5' PTP"}=" P5* PTP";
$resn_exch{" OP1  PU"}=" O1P  PU"; $resn_exch{" OP2  PU"}=" O2P  PU"; $resn_exch{" OP3  PU"}=" O3P  PU";
$resn_exch{" O5'  PU"}=" O5*  PU"; $resn_exch{" C5'  PU"}=" C5*  PU"; $resn_exch{" C4'  PU"}=" C4*  PU";
$resn_exch{" O4'  PU"}=" O4*  PU"; $resn_exch{" C3'  PU"}=" C3*  PU"; $resn_exch{" N3'  PU"}=" N3*  PU";
$resn_exch{" C2'  PU"}=" C2*  PU"; $resn_exch{" O2'  PU"}=" O2*  PU"; $resn_exch{" C1'  PU"}=" C1*  PU";
$resn_exch{" O5' PUA"}=" O5* PUA"; $resn_exch{" C5' PUA"}=" C5* PUA"; $resn_exch{" C4' PUA"}=" C4* PUA";
$resn_exch{" O4' PUA"}=" O4* PUA"; $resn_exch{" C1' PUA"}=" C1* PUA"; $resn_exch{" C2' PUA"}=" C2* PUA";
$resn_exch{" O2' PUA"}=" O2* PUA"; $resn_exch{" C3' PUA"}=" C3* PUA"; $resn_exch{" O3' PUA"}=" O3* PUA";
$resn_exch{" C1' PUR"}=" C1* PUR"; $resn_exch{" C2' PUR"}=" C2* PUR"; $resn_exch{" O2' PUR"}=" O2* PUR";
$resn_exch{" C3' PUR"}=" C3* PUR"; $resn_exch{" O3' PUR"}=" O3* PUR"; $resn_exch{" C4' PUR"}=" C4* PUR";
$resn_exch{" O4' PUR"}=" O4* PUR"; $resn_exch{" C5' PUR"}=" C5* PUR"; $resn_exch{" O5' PUR"}=" O5* PUR";
$resn_exch{" O5' PUY"}=" O5* PUY"; $resn_exch{" C5' PUY"}=" C5* PUY"; $resn_exch{" C4' PUY"}=" C4* PUY";
$resn_exch{" O4' PUY"}=" O4* PUY"; $resn_exch{" C1' PUY"}=" C1* PUY"; $resn_exch{" C2' PUY"}=" C2* PUY";
$resn_exch{" O2' PUY"}=" O2* PUY"; $resn_exch{" C3' PUY"}=" C3* PUY"; $resn_exch{" N3' PUY"}=" N3* PUY";
$resn_exch{" C'  PX1"}=" C*  PX1"; $resn_exch{" O'  PX1"}=" O*  PX1"; $resn_exch{" C'  PXA"}=" C*  PXA";
$resn_exch{" O'  PXA"}=" O*  PXA"; $resn_exch{" C'  PXF"}=" C*  PXF"; $resn_exch{" O'  PXF"}=" O*  PXF";
$resn_exch{" C'  PXZ"}=" C*  PXZ"; $resn_exch{" O'  PXZ"}=" O*  PXZ"; $resn_exch{" OP3 PYO"}=" O3P PYO";
$resn_exch{" OP1 PYO"}=" O1P PYO"; $resn_exch{" OP2 PYO"}=" O2P PYO"; $resn_exch{" O5' PYO"}=" O5* PYO";
$resn_exch{" C5' PYO"}=" C5* PYO"; $resn_exch{" C4' PYO"}=" C4* PYO"; $resn_exch{" O4' PYO"}=" O4* PYO";
$resn_exch{" C3' PYO"}=" C3* PYO"; $resn_exch{" C1' PYO"}=" C1* PYO"; $resn_exch{" O3' PYO"}=" O3* PYO";
$resn_exch{" C2' PYO"}=" C2* PYO"; $resn_exch{" O2' PYO"}=" O2* PYO"; $resn_exch{" O5' PYP"}=" O5* PYP";
$resn_exch{" C5' PYP"}=" C5* PYP"; $resn_exch{" C4' PYP"}=" C4* PYP"; $resn_exch{" O4' PYP"}=" O4* PYP";
$resn_exch{" C1' PYP"}=" C1* PYP"; $resn_exch{" C2' PYP"}=" C2* PYP"; $resn_exch{" C3' PYP"}=" C3* PYP";
$resn_exch{" O3' PYP"}=" O3* PYP"; $resn_exch{" O5' PYY"}=" O5* PYY"; $resn_exch{" C5' PYY"}=" C5* PYY";
$resn_exch{" C4' PYY"}=" C4* PYY"; $resn_exch{" O4' PYY"}=" O4* PYY"; $resn_exch{" C1' PYY"}=" C1* PYY";
$resn_exch{" C2' PYY"}=" C2* PYY"; $resn_exch{" O2' PYY"}=" O2* PYY"; $resn_exch{" C3' PYY"}=" C3* PYY";
$resn_exch{" O3' PYY"}=" O3* PYY"; $resn_exch{" OP1 QUO"}=" O1P QUO"; $resn_exch{" OP2 QUO"}=" O2P QUO";
$resn_exch{" OP3 QUO"}=" O3P QUO"; $resn_exch{" O5' QUO"}=" O5* QUO"; $resn_exch{" C5' QUO"}=" C5* QUO";
$resn_exch{" C4' QUO"}=" C4* QUO"; $resn_exch{" O4' QUO"}=" O4* QUO"; $resn_exch{" C1' QUO"}=" C1* QUO";
$resn_exch{" C2' QUO"}=" C2* QUO"; $resn_exch{" O2' QUO"}=" O2* QUO"; $resn_exch{" C3' QUO"}=" C3* QUO";
$resn_exch{" O3' QUO"}=" O3* QUO"; $resn_exch{" OP1   R"}=" O1P   R"; $resn_exch{" OP2   R"}=" O2P   R";
$resn_exch{" OP3   R"}=" O3P   R"; $resn_exch{" O5'   R"}=" O5*   R"; $resn_exch{" C5'   R"}=" C5*   R";
$resn_exch{" C4'   R"}=" C4*   R"; $resn_exch{" O4'   R"}=" O4*   R"; $resn_exch{" C3'   R"}=" C3*   R";
$resn_exch{" O3'   R"}=" O3*   R"; $resn_exch{" C2'   R"}=" C2*   R"; $resn_exch{" C1'   R"}=" C1*   R";
$resn_exch{" C1' R22"}=" C1* R22"; $resn_exch{" C2' R22"}=" C2* R22"; $resn_exch{" C3' R22"}=" C3* R22";
$resn_exch{" C4' R22"}=" C4* R22"; $resn_exch{" C5' R22"}=" C5* R22"; $resn_exch{" C6' R22"}=" C6* R22";
$resn_exch{" O7' R22"}=" O7* R22"; $resn_exch{" C8' R22"}=" C8* R22"; $resn_exch{" C9' R22"}=" C9* R22";
$resn_exch{" OXT R7A"}=" O   R7A"; $resn_exch{" O   R7A"}=" OXT R7A"; $resn_exch{" C1' RBF"}=" C1* RBF";
$resn_exch{" C2' RBF"}=" C2* RBF"; $resn_exch{" O2' RBF"}=" O2* RBF"; $resn_exch{" C3' RBF"}=" C3* RBF";
$resn_exch{" O3' RBF"}=" O3* RBF"; $resn_exch{" C4' RBF"}=" C4* RBF"; $resn_exch{" O4' RBF"}=" O4* RBF";
$resn_exch{" C5' RBF"}=" C5* RBF"; $resn_exch{" O5' RBF"}=" O5* RBF"; $resn_exch{" O5' RBY"}=" O5* RBY";
$resn_exch{" C5' RBY"}=" C5* RBY"; $resn_exch{" C4' RBY"}=" C4* RBY"; $resn_exch{" O4' RBY"}=" O4* RBY";
$resn_exch{" C3' RBY"}=" C3* RBY"; $resn_exch{" O3' RBY"}=" O3* RBY"; $resn_exch{" C2' RBY"}=" C2* RBY";
$resn_exch{" O2' RBY"}=" O2* RBY"; $resn_exch{" C1' RBY"}=" C1* RBY"; $resn_exch{" O5' RBZ"}=" O5* RBZ";
$resn_exch{" C5' RBZ"}=" C5* RBZ"; $resn_exch{" C4' RBZ"}=" C4* RBZ"; $resn_exch{" O4' RBZ"}=" O4* RBZ";
$resn_exch{" C3' RBZ"}=" C3* RBZ"; $resn_exch{" O3' RBZ"}=" O3* RBZ"; $resn_exch{" C2' RBZ"}=" C2* RBZ";
$resn_exch{" O2' RBZ"}=" O2* RBZ"; $resn_exch{" C1' RBZ"}=" C1* RBZ"; $resn_exch{" O1' RDP"}=" O1* RDP";
$resn_exch{" C1' RDP"}=" C1* RDP"; $resn_exch{" C2' RDP"}=" C2* RDP"; $resn_exch{" O2' RDP"}=" O2* RDP";
$resn_exch{" C3' RDP"}=" C3* RDP"; $resn_exch{" O3' RDP"}=" O3* RDP"; $resn_exch{" C4' RDP"}=" C4* RDP";
$resn_exch{" O4' RDP"}=" O4* RDP"; $resn_exch{" C5' RDP"}=" C5* RDP"; $resn_exch{" O5' RDP"}=" O5* RDP";
$resn_exch{" O7A RFC"}=" AO7 RFC"; $resn_exch{" P3B RFC"}="AP3* RFC"; $resn_exch{" O9A RFC"}=" AO9 RFC";
$resn_exch{" O8A RFC"}=" AO8 RFC"; $resn_exch{" O3B RFC"}="AO3* RFC"; $resn_exch{" C3B RFC"}="AC3* RFC";
$resn_exch{" C2B RFC"}="AC2* RFC"; $resn_exch{" O2B RFC"}="AO2* RFC"; $resn_exch{" C1B RFC"}="AC1* RFC";
$resn_exch{" N9A RFC"}=" AN9 RFC"; $resn_exch{" C4A RFC"}=" AC4 RFC"; $resn_exch{" C5A RFC"}=" AC5 RFC";
$resn_exch{" C6A RFC"}=" AC6 RFC"; $resn_exch{" N6A RFC"}=" AN6 RFC"; $resn_exch{" N1A RFC"}=" AN1 RFC";
$resn_exch{" C2A RFC"}=" AC2 RFC"; $resn_exch{" N3A RFC"}=" AN3 RFC"; $resn_exch{" N7A RFC"}=" AN7 RFC";
$resn_exch{" C8A RFC"}=" AC8 RFC"; $resn_exch{" O4B RFC"}="AO4* RFC"; $resn_exch{" C4B RFC"}="AC4* RFC";
$resn_exch{" C5B RFC"}="AC5* RFC"; $resn_exch{" O5B RFC"}="AO5* RFC"; $resn_exch{" P1A RFC"}=" AP1 RFC";
$resn_exch{" O1A RFC"}=" AO1 RFC"; $resn_exch{" O2A RFC"}=" AO2 RFC"; $resn_exch{" O3A RFC"}=" AO3 RFC";
$resn_exch{" P2A RFC"}=" AP2 RFC"; $resn_exch{" O4A RFC"}=" AO4 RFC"; $resn_exch{" O5A RFC"}=" AO5 RFC";
$resn_exch{" O6A RFC"}=" AO6 RFC"; $resn_exch{" CCP RFC"}="PC12 RFC"; $resn_exch{" CBP RFC"}="PC11 RFC";
$resn_exch{" CDP RFC"}="PC13 RFC"; $resn_exch{" CEP RFC"}="PC14 RFC"; $resn_exch{" CAP RFC"}="PC10 RFC";
$resn_exch{" OAP RFC"}="PO10 RFC"; $resn_exch{" C9P RFC"}=" PC9 RFC"; $resn_exch{" O9P RFC"}=" PO9 RFC";
$resn_exch{" N8P RFC"}=" PN8 RFC"; $resn_exch{" C7P RFC"}=" PC7 RFC"; $resn_exch{" C6P RFC"}=" PC6 RFC";
$resn_exch{" C5P RFC"}=" PC5 RFC"; $resn_exch{" O5P RFC"}=" PO5 RFC"; $resn_exch{" N4P RFC"}=" PN4 RFC";
$resn_exch{" C3P RFC"}=" PC3 RFC"; $resn_exch{" C2P RFC"}=" PC2 RFC"; $resn_exch{" S1P RFC"}=" PS1 RFC";
$resn_exch{" SG' RGS"}=" SG* RGS"; $resn_exch{" CB' RGS"}=" CB* RGS"; $resn_exch{" CA' RGS"}=" CA* RGS";
$resn_exch{" N'  RGS"}=" N*  RGS"; $resn_exch{" C5' RGS"}=" C5* RGS"; $resn_exch{" O5' RGS"}=" O5* RGS";
$resn_exch{" C6' RGS"}=" C6* RGS"; $resn_exch{" C7' RGS"}=" C7* RGS"; $resn_exch{" O11 RGS"}="*O7A RGS";
$resn_exch{" O12 RGS"}="*O7B RGS"; $resn_exch{" C'  RGS"}=" C*  RGS"; $resn_exch{" O'  RGS"}=" O*  RGS";
$resn_exch{" N4' RGS"}=" N4* RGS"; $resn_exch{" C4' RGS"}=" C4* RGS"; $resn_exch{" C3' RGS"}=" C3* RGS";
$resn_exch{" C2' RGS"}=" C2* RGS"; $resn_exch{" N2' RGS"}=" N2* RGS"; $resn_exch{" C1' RGS"}=" C1* RGS";
$resn_exch{" O13 RGS"}="*O1A RGS"; $resn_exch{" O14 RGS"}="*O1B RGS"; $resn_exch{" OP1 RIA"}=" O1P RIA";
$resn_exch{" OP2 RIA"}=" O2P RIA"; $resn_exch{" OP3 RIA"}=" O3P RIA"; $resn_exch{" O5A RIA"}=" O5* RIA";
$resn_exch{" C5A RIA"}=" C5* RIA"; $resn_exch{" C4A RIA"}=" C4* RIA"; $resn_exch{" O4' RIA"}=" O4* RIA";
$resn_exch{" C3A RIA"}=" C3* RIA"; $resn_exch{" O3A RIA"}=" O3* RIA"; $resn_exch{" C2A RIA"}=" C2* RIA";
$resn_exch{" O2A RIA"}=" O2* RIA"; $resn_exch{" C1A RIA"}=" C1* RIA"; $resn_exch{" O1X RIA"}="O1P' RIA";
$resn_exch{" O2X RIA"}="O2P' RIA"; $resn_exch{" O3X RIA"}="O3P' RIA"; $resn_exch{" O5' RIB"}=" O5* RIB";
$resn_exch{" C5' RIB"}=" C5* RIB"; $resn_exch{" C4' RIB"}=" C4* RIB"; $resn_exch{" O4' RIB"}=" O4* RIB";
$resn_exch{" C3' RIB"}=" C3* RIB"; $resn_exch{" O3' RIB"}=" O3* RIB"; $resn_exch{" C2' RIB"}=" C2* RIB";
$resn_exch{" O2' RIB"}=" O2* RIB"; $resn_exch{" C1' RIB"}=" C1* RIB"; $resn_exch{" O1' RIB"}=" O1* RIB";
$resn_exch{" O5' RMB"}=" O5* RMB"; $resn_exch{" C5' RMB"}=" C5* RMB"; $resn_exch{" C4' RMB"}=" C4* RMB";
$resn_exch{" O4' RMB"}=" O4* RMB"; $resn_exch{" C3' RMB"}=" C3* RMB"; $resn_exch{" O3' RMB"}=" O3* RMB";
$resn_exch{" C2' RMB"}=" C2* RMB"; $resn_exch{" O2' RMB"}=" O2* RMB"; $resn_exch{" C1' RMB"}=" C1* RMB";
$resn_exch{" OP1 RMP"}=" O1P RMP"; $resn_exch{" O5' RMP"}=" O5* RMP"; $resn_exch{" C5' RMP"}=" C5* RMP";
$resn_exch{" C4' RMP"}=" C4* RMP"; $resn_exch{" O4' RMP"}=" O4* RMP"; $resn_exch{" C1' RMP"}=" C1* RMP";
$resn_exch{" C3' RMP"}=" C3* RMP"; $resn_exch{" C2' RMP"}=" C2* RMP"; $resn_exch{" O3' RMP"}=" O3* RMP";
$resn_exch{" OP3 RMP"}=" O3P RMP"; $resn_exch{" O1X ROB"}="O1P' ROB"; $resn_exch{" O2X ROB"}="O2P' ROB";
$resn_exch{" O3X ROB"}="O3P' ROB"; $resn_exch{" C6D ROM"}="C6'B ROM"; $resn_exch{" C6B ROM"}="C6'A ROM";
$resn_exch{" C4D ROM"}="C4'B ROM"; $resn_exch{" C4B ROM"}="C4'A ROM"; $resn_exch{" C15 ROM"}="C'10 ROM";
$resn_exch{" C1A ROM"}="C'12 ROM"; $resn_exch{" C1B ROM"}="C'11 ROM"; $resn_exch{" C7X ROM"}="C'7A ROM";
$resn_exch{" O11 ROM"}="O'12 ROM"; $resn_exch{" O5' RP1"}=" O5* RP1"; $resn_exch{" C5' RP1"}=" C5* RP1";
$resn_exch{" C4' RP1"}=" C4* RP1"; $resn_exch{" O4' RP1"}=" O4* RP1"; $resn_exch{" C3' RP1"}=" C3* RP1";
$resn_exch{" O3' RP1"}=" O3* RP1"; $resn_exch{" C2' RP1"}=" C2* RP1"; $resn_exch{" O2' RP1"}=" O2* RP1";
$resn_exch{" C1' RP1"}=" C1* RP1"; $resn_exch{" O1X RP5"}="O1P' RP5"; $resn_exch{" O2X RP5"}="O2P' RP5";
$resn_exch{" O3X RP5"}="O3P' RP5"; $resn_exch{" O11 RPA"}="1OM6 RPA"; $resn_exch{" O5' RPD"}=" O5* RPD";
$resn_exch{" C5' RPD"}=" C5* RPD"; $resn_exch{" C4' RPD"}=" C4* RPD"; $resn_exch{" O4' RPD"}=" O4* RPD";
$resn_exch{" C3' RPD"}=" C3* RPD"; $resn_exch{" O3' RPD"}=" O3* RPD"; $resn_exch{" C2' RPD"}=" C2* RPD";
$resn_exch{" O2' RPD"}=" O2* RPD"; $resn_exch{" C1' RPD"}=" C1* RPD"; $resn_exch{" OJA RPD"}="O91A RPD";
$resn_exch{" OKA RPD"}="O92A RPD"; $resn_exch{" NAA RPD"}="N10A RPD"; $resn_exch{" OAA RPD"}="O10A RPD";
$resn_exch{" CBA RPD"}="C11A RPD"; $resn_exch{" OBA RPD"}="O11A RPD"; $resn_exch{" O5' RPL"}=" O5* RPL";
$resn_exch{" C5' RPL"}=" C5* RPL"; $resn_exch{" C4' RPL"}=" C4* RPL"; $resn_exch{" O4' RPL"}=" O4* RPL";
$resn_exch{" C3' RPL"}=" C3* RPL"; $resn_exch{" O3' RPL"}=" O3* RPL"; $resn_exch{" C2' RPL"}=" C2* RPL";
$resn_exch{" O2' RPL"}=" O2* RPL"; $resn_exch{" C1' RPL"}=" C1* RPL"; $resn_exch{" OJA RPL"}="O91A RPL";
$resn_exch{" OKA RPL"}="O92A RPL"; $resn_exch{" NAA RPL"}="N10A RPL"; $resn_exch{" OAA RPL"}="O10A RPL";
$resn_exch{" CBA RPL"}="C11A RPL"; $resn_exch{" OBA RPL"}="O11A RPL"; $resn_exch{" C1' RPP"}=" C1* RPP";
$resn_exch{" C2' RPP"}=" C2* RPP"; $resn_exch{" C3' RPP"}=" C3* RPP"; $resn_exch{" C4' RPP"}=" C4* RPP";
$resn_exch{" O4' RPP"}=" O4* RPP"; $resn_exch{" O2' RPP"}=" O2* RPP"; $resn_exch{" O3' RPP"}=" O3* RPP";
$resn_exch{" C5' RPP"}=" C5* RPP"; $resn_exch{" O5' RPP"}=" O5* RPP"; $resn_exch{" OP3  RT"}=" O3P  RT";
$resn_exch{" OP1  RT"}=" O1P  RT"; $resn_exch{" OP2  RT"}=" O2P  RT"; $resn_exch{" O5'  RT"}=" O5*  RT";
$resn_exch{" C5'  RT"}=" C5*  RT"; $resn_exch{" C4'  RT"}=" C4*  RT"; $resn_exch{" O4'  RT"}=" O4*  RT";
$resn_exch{" C3'  RT"}=" C3*  RT"; $resn_exch{" O3'  RT"}=" O3*  RT"; $resn_exch{" C2'  RT"}=" C2*  RT";
$resn_exch{" O2'  RT"}=" O2*  RT"; $resn_exch{" C1'  RT"}=" C1*  RT"; $resn_exch{" O5' RTP"}=" O5* RTP";
$resn_exch{" C5' RTP"}=" C5* RTP"; $resn_exch{" C4' RTP"}=" C4* RTP"; $resn_exch{" O4' RTP"}=" O4* RTP";
$resn_exch{" C1' RTP"}=" C1* RTP"; $resn_exch{" C2' RTP"}=" C2* RTP"; $resn_exch{" O2' RTP"}=" O2* RTP";
$resn_exch{" C3' RTP"}=" C3* RTP"; $resn_exch{" O3' RTP"}=" O3* RTP"; $resn_exch{" C1' RUN"}=" C1\" RUN";
$resn_exch{" O1' RUN"}=" O1\" RUN"; $resn_exch{" C2D RUN"}=" C2\" RUN"; $resn_exch{" C3' RUN"}=" C3\" RUN";
$resn_exch{" C4D RUN"}=" C4\" RUN"; $resn_exch{" O5' RVP"}=" O5* RVP"; $resn_exch{" C5' RVP"}=" C5* RVP";
$resn_exch{" C4' RVP"}=" C4* RVP"; $resn_exch{" O4' RVP"}=" O4* RVP"; $resn_exch{" C3' RVP"}=" C3* RVP";
$resn_exch{" O3' RVP"}=" O3* RVP"; $resn_exch{" C2' RVP"}=" C2* RVP"; $resn_exch{" O2' RVP"}=" O2* RVP";
$resn_exch{" C1' RVP"}=" C1* RVP"; $resn_exch{" O5' S02"}=" O5* S02"; $resn_exch{" C5' S02"}=" C5* S02";
$resn_exch{" CA' S02"}=" CA* S02"; $resn_exch{" OB' S02"}=" OB* S02"; $resn_exch{" OC' S02"}=" OC* S02";
$resn_exch{" CD' S02"}=" CD* S02"; $resn_exch{" C3' S02"}=" C3* S02"; $resn_exch{" O3' S02"}=" O3* S02";
$resn_exch{" C5' S2M"}=" C5* S2M"; $resn_exch{" O5' S2M"}=" O5* S2M"; $resn_exch{" OP2 S2M"}=" O2P S2M";
$resn_exch{" OP1 S2M"}=" O1P S2M"; $resn_exch{" C4' S2M"}=" C4* S2M"; $resn_exch{" O4' S2M"}=" O4* S2M";
$resn_exch{" C1' S2M"}=" C1* S2M"; $resn_exch{" C2' S2M"}=" C2* S2M"; $resn_exch{" C3' S2M"}=" C3* S2M";
$resn_exch{" O3' S2M"}=" O3* S2M"; $resn_exch{" O2' S2M"}=" O2* S2M"; $resn_exch{" CA' S2M"}=" CA* S2M";
$resn_exch{" CB' S2M"}=" CB* S2M"; $resn_exch{" OC' S2M"}=" OC* S2M"; $resn_exch{" CD' S2M"}=" CD* S2M";
$resn_exch{" OP3 S2M"}=" O3P S2M"; $resn_exch{" C1' S4C"}=" C1* S4C"; $resn_exch{" C2' S4C"}=" C2* S4C";
$resn_exch{" O2' S4C"}=" O2* S4C"; $resn_exch{" C3' S4C"}=" C3* S4C"; $resn_exch{" O3' S4C"}=" O3* S4C";
$resn_exch{" C4' S4C"}=" C4* S4C"; $resn_exch{" S4' S4C"}=" S4* S4C"; $resn_exch{" OP1 S4C"}=" O1P S4C";
$resn_exch{" C5' S4C"}=" C5* S4C"; $resn_exch{" O5' S4C"}=" O5* S4C"; $resn_exch{" OP2 S4C"}=" O2P S4C";
$resn_exch{" OP3 S4C"}=" O3P S4C"; $resn_exch{" C5' S4M"}=" C5* S4M"; $resn_exch{" C4' S4M"}=" C4* S4M";
$resn_exch{" O4' S4M"}=" O4* S4M"; $resn_exch{" C1' S4M"}=" C1* S4M"; $resn_exch{" C2' S4M"}=" C2* S4M";
$resn_exch{" O2' S4M"}=" O2* S4M"; $resn_exch{" C3' S4M"}=" C3* S4M"; $resn_exch{" O3' S4M"}=" O3* S4M";
$resn_exch{" OP1 S4U"}=" O1P S4U"; $resn_exch{" OP2 S4U"}=" O2P S4U"; $resn_exch{" OP3 S4U"}=" O3P S4U";
$resn_exch{" OP1 S6G"}=" O1P S6G"; $resn_exch{" OP2 S6G"}=" O2P S6G"; $resn_exch{" OP3 S6G"}=" O3P S6G";
$resn_exch{" O5' S6G"}=" O5* S6G"; $resn_exch{" C5' S6G"}=" C5* S6G"; $resn_exch{" C4' S6G"}=" C4* S6G";
$resn_exch{" O4' S6G"}=" O4* S6G"; $resn_exch{" C3' S6G"}=" C3* S6G"; $resn_exch{" O3' S6G"}=" O3* S6G";
$resn_exch{" C2' S6G"}=" C2* S6G"; $resn_exch{" C1' S6G"}=" C1* S6G"; $resn_exch{" O5' S9L"}=" O5* S9L";
$resn_exch{" O3' S9L"}=" O3* S9L"; $resn_exch{" C5' SA8"}=" C5* SA8"; $resn_exch{" C4' SA8"}=" C4* SA8";
$resn_exch{" O4' SA8"}=" O4* SA8"; $resn_exch{" C3' SA8"}=" C3* SA8"; $resn_exch{" O3' SA8"}=" O3* SA8";
$resn_exch{" C2' SA8"}=" C2* SA8"; $resn_exch{" O2' SA8"}=" O2* SA8"; $resn_exch{" C1' SA8"}=" C1* SA8";
$resn_exch{" PA  SAD"}=" AP  SAD"; $resn_exch{" O1A SAD"}=" AO1 SAD"; $resn_exch{" O2A SAD"}=" AO2 SAD";
$resn_exch{" O5B SAD"}="AO5* SAD"; $resn_exch{" C5B SAD"}="AC5* SAD"; $resn_exch{" C4B SAD"}="AC4* SAD";
$resn_exch{" O4B SAD"}="AO4* SAD"; $resn_exch{" C3B SAD"}="AC3* SAD"; $resn_exch{" O3B SAD"}="AO3* SAD";
$resn_exch{" C2B SAD"}="AC2* SAD"; $resn_exch{" O2B SAD"}="AO2* SAD"; $resn_exch{" C1B SAD"}="AC1* SAD";
$resn_exch{" N9A SAD"}=" AN9 SAD"; $resn_exch{" C8A SAD"}=" AC8 SAD"; $resn_exch{" N7A SAD"}=" AN7 SAD";
$resn_exch{" C5A SAD"}=" AC5 SAD"; $resn_exch{" C6A SAD"}=" AC6 SAD"; $resn_exch{" N6A SAD"}=" AN6 SAD";
$resn_exch{" N1A SAD"}=" AN1 SAD"; $resn_exch{" C2A SAD"}=" AC2 SAD"; $resn_exch{" N3A SAD"}=" AN3 SAD";
$resn_exch{" C4A SAD"}=" AC4 SAD"; $resn_exch{" PN  SAD"}=" NP  SAD"; $resn_exch{" O1N SAD"}=" NO1 SAD";
$resn_exch{" O2N SAD"}=" NO2 SAD"; $resn_exch{" O5D SAD"}="NO5* SAD"; $resn_exch{" C5D SAD"}="NC5* SAD";
$resn_exch{" C4D SAD"}="NC4* SAD"; $resn_exch{" O4D SAD"}="NO4* SAD"; $resn_exch{" C3D SAD"}="NC3* SAD";
$resn_exch{" O3D SAD"}="NO3* SAD"; $resn_exch{" C2D SAD"}="NC2* SAD"; $resn_exch{" O2D SAD"}="NO2* SAD";
$resn_exch{" C1D SAD"}="NC1* SAD"; $resn_exch{" C2N SAD"}=" NC2 SAD"; $resn_exch{" N3N SAD"}=" NN3 SAD";
$resn_exch{" C4N SAD"}=" NC4 SAD"; $resn_exch{" C5N SAD"}=" NC5 SAD"; $resn_exch{"SE1N SAD"}="NSE1 SAD";
$resn_exch{" C6N SAD"}=" NC6 SAD"; $resn_exch{" O6N SAD"}=" NO6 SAD"; $resn_exch{" N6N SAD"}=" NN6 SAD";
$resn_exch{" PA  SAE"}=" AP  SAE"; $resn_exch{" O1A SAE"}=" AO1 SAE"; $resn_exch{" O2A SAE"}=" AO2 SAE";
$resn_exch{" O5B SAE"}="AO5* SAE"; $resn_exch{" C5B SAE"}="AC5* SAE"; $resn_exch{" C4B SAE"}="AC4* SAE";
$resn_exch{" O4B SAE"}="AO4* SAE"; $resn_exch{" C3B SAE"}="AC3* SAE"; $resn_exch{" O3B SAE"}="AO3* SAE";
$resn_exch{" C2B SAE"}="AC2* SAE"; $resn_exch{" O2B SAE"}="AO2* SAE"; $resn_exch{" C1B SAE"}="AC1* SAE";
$resn_exch{" N9A SAE"}=" AN9 SAE"; $resn_exch{" C8A SAE"}=" AC8 SAE"; $resn_exch{" N7A SAE"}=" AN7 SAE";
$resn_exch{" C5A SAE"}=" AC5 SAE"; $resn_exch{" C6A SAE"}=" AC6 SAE"; $resn_exch{" N6A SAE"}=" AN6 SAE";
$resn_exch{" N1A SAE"}=" AN1 SAE"; $resn_exch{" C2A SAE"}=" AC2 SAE"; $resn_exch{" N3A SAE"}=" AN3 SAE";
$resn_exch{" C4A SAE"}=" AC4 SAE"; $resn_exch{" PN  SAE"}=" NP  SAE"; $resn_exch{" O1N SAE"}=" NO1 SAE";
$resn_exch{" O2N SAE"}=" NO2 SAE"; $resn_exch{" O5D SAE"}="NO5* SAE"; $resn_exch{" C5D SAE"}="NC5* SAE";
$resn_exch{" C4D SAE"}="NC4* SAE"; $resn_exch{" O4D SAE"}="NO4* SAE"; $resn_exch{" C3D SAE"}="NC3* SAE";
$resn_exch{" O3D SAE"}="NO3* SAE"; $resn_exch{" C2D SAE"}="NC2* SAE"; $resn_exch{" O2D SAE"}="NO2* SAE";
$resn_exch{" C1D SAE"}="NC1* SAE"; $resn_exch{" C2N SAE"}=" NC2 SAE"; $resn_exch{" N3N SAE"}=" NN3 SAE";
$resn_exch{" C4N SAE"}=" NC4 SAE"; $resn_exch{" C5N SAE"}=" NC5 SAE"; $resn_exch{"SE1N SAE"}="NSE1 SAE";
$resn_exch{" C6N SAE"}=" NC6 SAE"; $resn_exch{" O6N SAE"}=" NO6 SAE"; $resn_exch{" N6N SAE"}=" NN6 SAE";
$resn_exch{" C5' SAH"}=" C5* SAH"; $resn_exch{" C4' SAH"}=" C4* SAH"; $resn_exch{" O4' SAH"}=" O4* SAH";
$resn_exch{" C3' SAH"}=" C3* SAH"; $resn_exch{" O3' SAH"}=" O3* SAH"; $resn_exch{" C2' SAH"}=" C2* SAH";
$resn_exch{" O2' SAH"}=" O2* SAH"; $resn_exch{" C1' SAH"}=" C1* SAH"; $resn_exch{" C5' SAI"}=" C5* SAI";
$resn_exch{" C4' SAI"}=" C4* SAI"; $resn_exch{" O4' SAI"}=" O4* SAI"; $resn_exch{" C3' SAI"}=" C3* SAI";
$resn_exch{" O3' SAI"}=" O3* SAI"; $resn_exch{" C2' SAI"}=" C2* SAI"; $resn_exch{" O2' SAI"}=" O2* SAI";
$resn_exch{" C1' SAI"}=" C1* SAI"; $resn_exch{" C5' SAM"}=" C5* SAM"; $resn_exch{" C4' SAM"}=" C4* SAM";
$resn_exch{" O4' SAM"}=" O4* SAM"; $resn_exch{" C3' SAM"}=" C3* SAM"; $resn_exch{" O3' SAM"}=" O3* SAM";
$resn_exch{" C2' SAM"}=" C2* SAM"; $resn_exch{" O2' SAM"}=" O2* SAM"; $resn_exch{" C1' SAM"}=" C1* SAM";
$resn_exch{" O5' SAP"}=" O5* SAP"; $resn_exch{" C5' SAP"}=" C5* SAP"; $resn_exch{" C4' SAP"}=" C4* SAP";
$resn_exch{" O4' SAP"}=" O4* SAP"; $resn_exch{" C3' SAP"}=" C3* SAP"; $resn_exch{" O3' SAP"}=" O3* SAP";
$resn_exch{" C2' SAP"}=" C2* SAP"; $resn_exch{" O2' SAP"}=" O2* SAP"; $resn_exch{" C1' SAP"}=" C1* SAP";
$resn_exch{" C1'  SC"}=" C1*  SC"; $resn_exch{" C2'  SC"}=" C2*  SC"; $resn_exch{" C3'  SC"}=" C3*  SC";
$resn_exch{" C4'  SC"}=" C4*  SC"; $resn_exch{" O4'  SC"}=" O4*  SC"; $resn_exch{" O3'  SC"}=" O3*  SC";
$resn_exch{" C5'  SC"}=" C5*  SC"; $resn_exch{" O5'  SC"}=" O5*  SC"; $resn_exch{" OP1  SC"}=" O1P  SC";
$resn_exch{" OP3  SC"}=" O3P  SC"; $resn_exch{" C1' SCO"}=" C1* SCO"; $resn_exch{" C2' SCO"}=" C2* SCO";
$resn_exch{" C3' SCO"}=" C3* SCO"; $resn_exch{" C4' SCO"}=" C4* SCO"; $resn_exch{" C5' SCO"}=" C5* SCO";
$resn_exch{" O1' SCO"}=" O1* SCO"; $resn_exch{" O2' SCO"}=" O2* SCO"; $resn_exch{" O3' SCO"}=" O3* SCO";
$resn_exch{" O4' SCO"}=" O4* SCO"; $resn_exch{" O5' SCO"}=" O5* SCO"; $resn_exch{" O6' SCO"}=" O6* SCO";
$resn_exch{" O7A SFC"}=" AO7 SFC"; $resn_exch{" P3B SFC"}="AP3* SFC"; $resn_exch{" O9A SFC"}=" AO9 SFC";
$resn_exch{" O8A SFC"}=" AO8 SFC"; $resn_exch{" O3B SFC"}="AO3* SFC"; $resn_exch{" C3B SFC"}="AC3* SFC";
$resn_exch{" C2B SFC"}="AC2* SFC"; $resn_exch{" O2B SFC"}="AO2* SFC"; $resn_exch{" C1B SFC"}="AC1* SFC";
$resn_exch{" N9A SFC"}=" AN9 SFC"; $resn_exch{" C4A SFC"}=" AC4 SFC"; $resn_exch{" C5A SFC"}=" AC5 SFC";
$resn_exch{" C6A SFC"}=" AC6 SFC"; $resn_exch{" N6A SFC"}=" AN6 SFC"; $resn_exch{" N1A SFC"}=" AN1 SFC";
$resn_exch{" C2A SFC"}=" AC2 SFC"; $resn_exch{" N3A SFC"}=" AN3 SFC"; $resn_exch{" N7A SFC"}=" AN7 SFC";
$resn_exch{" C8A SFC"}=" AC8 SFC"; $resn_exch{" O4B SFC"}="AO4* SFC"; $resn_exch{" C4B SFC"}="AC4* SFC";
$resn_exch{" C5B SFC"}="AC5* SFC"; $resn_exch{" O5B SFC"}="AO5* SFC"; $resn_exch{" P1A SFC"}=" AP1 SFC";
$resn_exch{" O1A SFC"}=" AO1 SFC"; $resn_exch{" O2A SFC"}=" AO2 SFC"; $resn_exch{" O3A SFC"}=" AO3 SFC";
$resn_exch{" P2A SFC"}=" AP2 SFC"; $resn_exch{" O4A SFC"}=" AO4 SFC"; $resn_exch{" O5A SFC"}=" AO5 SFC";
$resn_exch{" O6A SFC"}=" AO6 SFC"; $resn_exch{" CCP SFC"}="PC12 SFC"; $resn_exch{" CBP SFC"}="PC11 SFC";
$resn_exch{" CDP SFC"}="PC13 SFC"; $resn_exch{" CEP SFC"}="PC14 SFC"; $resn_exch{" CAP SFC"}="PC10 SFC";
$resn_exch{" OAP SFC"}="PO10 SFC"; $resn_exch{" C9P SFC"}=" PC9 SFC"; $resn_exch{" O9P SFC"}=" PO9 SFC";
$resn_exch{" N8P SFC"}=" PN8 SFC"; $resn_exch{" C7P SFC"}=" PC7 SFC"; $resn_exch{" C6P SFC"}=" PC6 SFC";
$resn_exch{" C5P SFC"}=" PC5 SFC"; $resn_exch{" O5P SFC"}=" PO5 SFC"; $resn_exch{" N4P SFC"}=" PN4 SFC";
$resn_exch{" C3P SFC"}=" PC3 SFC"; $resn_exch{" C2P SFC"}=" PC2 SFC"; $resn_exch{" S1P SFC"}=" PS1 SFC";
$resn_exch{" C5' SFG"}=" C5* SFG"; $resn_exch{" C4' SFG"}=" C4* SFG"; $resn_exch{" O4' SFG"}=" O4* SFG";
$resn_exch{" C3' SFG"}=" C3* SFG"; $resn_exch{" O3' SFG"}=" O3* SFG"; $resn_exch{" C2' SFG"}=" C2* SFG";
$resn_exch{" O2' SFG"}=" O2* SFG"; $resn_exch{" C1' SFG"}=" C1* SFG"; $resn_exch{" C42 SFT"}="2C4  SFT";
$resn_exch{" C32 SFT"}="2C3  SFT"; $resn_exch{" C22 SFT"}="2C2  SFT"; $resn_exch{" CCA SFT"}="2C1  SFT";
$resn_exch{" C02 SFT"}="2C0  SFT"; $resn_exch{" C91 SFT"}="1C9  SFT"; $resn_exch{" C81 SFT"}="1C8  SFT";
$resn_exch{" C71 SFT"}="1C7  SFT"; $resn_exch{" C61 SFT"}="1C6  SFT"; $resn_exch{" C51 SFT"}="1C5  SFT";
$resn_exch{" C41 SFT"}="1C4  SFT"; $resn_exch{" C31 SFT"}="1C3  SFT"; $resn_exch{" C21 SFT"}="1C2  SFT";
$resn_exch{" CBA SFT"}="1C1  SFT"; $resn_exch{" C01 SFT"}="1C0  SFT"; $resn_exch{" C1B SFT"}=" C1* SFT";
$resn_exch{" C2B SFT"}=" C2* SFT"; $resn_exch{" O2B SFT"}=" O2* SFT"; $resn_exch{" O5' SFT"}=" O5* SFT";
$resn_exch{" C5B SFT"}=" C5* SFT"; $resn_exch{" C6B SFT"}=" C6* SFT"; $resn_exch{" O6' SFT"}=" O6* SFT";
$resn_exch{" C4B SFT"}=" C4* SFT"; $resn_exch{" O4' SFT"}=" O4* SFT"; $resn_exch{" C3B SFT"}=" C3* SFT";
$resn_exch{" O3' SFT"}=" O3* SFT"; $resn_exch{" O5' SGP"}=" O5* SGP"; $resn_exch{" C5' SGP"}=" C5* SGP";
$resn_exch{" C4' SGP"}=" C4* SGP"; $resn_exch{" O4' SGP"}=" O4* SGP"; $resn_exch{" C3' SGP"}=" C3* SGP";
$resn_exch{" O3' SGP"}=" O3* SGP"; $resn_exch{" C2' SGP"}=" C2* SGP"; $resn_exch{" O2' SGP"}=" O2* SGP";
$resn_exch{" C1' SGP"}=" C1* SGP"; $resn_exch{" O   SHC"}=" O1  SHC"; $resn_exch{" C1B SHP"}=" C1* SHP";
$resn_exch{" C2B SHP"}=" C2* SHP"; $resn_exch{" C3B SHP"}=" C3* SHP"; $resn_exch{" C4B SHP"}=" C4* SHP";
$resn_exch{" O4B SHP"}=" O4* SHP"; $resn_exch{" C5B SHP"}=" C5* SHP"; $resn_exch{" C6B SHP"}=" C6* SHP";
$resn_exch{" OXT SIB"}=" O   SIB"; $resn_exch{" O   SIB"}=" OXT SIB"; $resn_exch{" C5' SIB"}=" C5* SIB";
$resn_exch{" C4' SIB"}=" C4* SIB"; $resn_exch{" O4' SIB"}=" O4* SIB"; $resn_exch{" C3' SIB"}=" C3* SIB";
$resn_exch{" O3' SIB"}=" O3* SIB"; $resn_exch{" C2' SIB"}=" C2* SIB"; $resn_exch{" O2' SIB"}=" O2* SIB";
$resn_exch{" C1' SIB"}=" C1* SIB"; $resn_exch{" C5' SMM"}=" C5* SMM"; $resn_exch{" C4' SMM"}=" C4* SMM";
$resn_exch{" O4' SMM"}=" O4* SMM"; $resn_exch{" C1' SMM"}=" C1* SMM"; $resn_exch{" C2' SMM"}=" C2* SMM";
$resn_exch{" O2' SMM"}=" O2* SMM"; $resn_exch{" C3' SMM"}=" C3* SMM"; $resn_exch{" O3' SMM"}=" O3* SMM";
$resn_exch{" OP1 SMP"}=" O1P SMP"; $resn_exch{" O5' SMP"}=" O5* SMP"; $resn_exch{" C5' SMP"}=" C5* SMP";
$resn_exch{" C4' SMP"}=" C4* SMP"; $resn_exch{" O4' SMP"}=" O4* SMP"; $resn_exch{" C1' SMP"}=" C1* SMP";
$resn_exch{" C3' SMP"}=" C3* SMP"; $resn_exch{" C2' SMP"}=" C2* SMP"; $resn_exch{" O3' SMP"}=" O3* SMP";
$resn_exch{" OP3 SMP"}=" O3P SMP"; $resn_exch{" OP1 SMT"}=" O1P SMT"; $resn_exch{" OP2 SMT"}=" O2P SMT";
$resn_exch{" OP3 SMT"}=" O3P SMT"; $resn_exch{" PA  SND"}=" AP  SND"; $resn_exch{" O1A SND"}=" AO1 SND";
$resn_exch{" O2A SND"}=" AO2 SND"; $resn_exch{" O5B SND"}="AO5* SND"; $resn_exch{" C5B SND"}="AC5* SND";
$resn_exch{" C4B SND"}="AC4* SND"; $resn_exch{" O4B SND"}="AO4* SND"; $resn_exch{" C3B SND"}="AC3* SND";
$resn_exch{" O3B SND"}="AO3* SND"; $resn_exch{" C2B SND"}="AC2* SND"; $resn_exch{" O2B SND"}="AO2* SND";
$resn_exch{" C1B SND"}="AC1* SND"; $resn_exch{" N9A SND"}=" AN9 SND"; $resn_exch{" C8A SND"}=" AC8 SND";
$resn_exch{" N7A SND"}=" AN7 SND"; $resn_exch{" C5A SND"}=" AC5 SND"; $resn_exch{" C6A SND"}=" AC6 SND";
$resn_exch{" N6A SND"}=" AN6 SND"; $resn_exch{" N1A SND"}=" AN1 SND"; $resn_exch{" C2A SND"}=" AC2 SND";
$resn_exch{" N3A SND"}=" AN3 SND"; $resn_exch{" C4A SND"}=" AC4 SND"; $resn_exch{" PN  SND"}=" NP  SND";
$resn_exch{" O1N SND"}=" NO1 SND"; $resn_exch{" O2N SND"}=" NO2 SND"; $resn_exch{" O5D SND"}="NO5* SND";
$resn_exch{" C5D SND"}="NC5* SND"; $resn_exch{" C4D SND"}="NC4* SND"; $resn_exch{" O4D SND"}="NO4* SND";
$resn_exch{" C3D SND"}="NC3* SND"; $resn_exch{" O3D SND"}="NO3* SND"; $resn_exch{" C2D SND"}="NC2* SND";
$resn_exch{" O2D SND"}="NO2* SND"; $resn_exch{" C1D SND"}="NC1* SND"; $resn_exch{" N1N SND"}=" NN1 SND";
$resn_exch{" C2N SND"}=" NC2 SND"; $resn_exch{" C3N SND"}=" NC3 SND"; $resn_exch{" C7N SND"}=" NC7 SND";
$resn_exch{" S7N SND"}=" NS7 SND"; $resn_exch{" N7N SND"}=" NN7 SND"; $resn_exch{" C4N SND"}=" NC4 SND";
$resn_exch{" C5N SND"}=" NC5 SND"; $resn_exch{" C6N SND"}=" NC6 SND"; $resn_exch{" O5' SNI"}=" O5* SNI";
$resn_exch{" C5' SNI"}=" C5* SNI"; $resn_exch{" C4' SNI"}=" C4* SNI"; $resn_exch{" O4' SNI"}=" O4* SNI";
$resn_exch{" C1' SNI"}=" C1* SNI"; $resn_exch{" C2' SNI"}=" C2* SNI"; $resn_exch{" O2' SNI"}=" O2* SNI";
$resn_exch{" C3' SNI"}=" C3* SNI"; $resn_exch{" O3' SNI"}=" O3* SNI"; $resn_exch{" C1' SOP"}=" C1* SOP";
$resn_exch{" O4' SOP"}=" O4* SOP"; $resn_exch{" C2' SOP"}=" C2* SOP"; $resn_exch{" O2' SOP"}=" O2* SOP";
$resn_exch{" C3' SOP"}=" C3* SOP"; $resn_exch{" O3' SOP"}=" O3* SOP"; $resn_exch{" C4' SOP"}=" C4* SOP";
$resn_exch{" C5' SOP"}=" C5* SOP"; $resn_exch{" O5' SOP"}=" O5* SOP"; $resn_exch{" O5' SP1"}=" O5* SP1";
$resn_exch{" C5' SP1"}=" C5* SP1"; $resn_exch{" C4' SP1"}=" C4* SP1"; $resn_exch{" O4' SP1"}=" O4* SP1";
$resn_exch{" C3' SP1"}=" C3* SP1"; $resn_exch{" O3' SP1"}=" O3* SP1"; $resn_exch{" C2' SP1"}=" C2* SP1";
$resn_exch{" O2' SP1"}=" O2* SP1"; $resn_exch{" C1' SP1"}=" C1* SP1"; $resn_exch{" N1A SPF"}=" AN1 SPF";
$resn_exch{" C2A SPF"}=" AC2 SPF"; $resn_exch{" N3A SPF"}=" AN3 SPF"; $resn_exch{" C4A SPF"}=" AC4 SPF";
$resn_exch{" C5A SPF"}=" AC5 SPF"; $resn_exch{" C6A SPF"}=" AC6 SPF"; $resn_exch{" N6A SPF"}=" AN6 SPF";
$resn_exch{" N7A SPF"}=" AN7 SPF"; $resn_exch{" C8A SPF"}=" AC8 SPF"; $resn_exch{" N9A SPF"}=" AN9 SPF";
$resn_exch{" C1B SPF"}="AC1* SPF"; $resn_exch{" C2B SPF"}="AC2* SPF"; $resn_exch{" O2B SPF"}="AO2* SPF";
$resn_exch{" C3B SPF"}="AC3* SPF"; $resn_exch{" O3B SPF"}="AO3* SPF"; $resn_exch{" P3B SPF"}="AP3* SPF";
$resn_exch{" O7A SPF"}=" AO7 SPF"; $resn_exch{" O8A SPF"}=" AO8 SPF"; $resn_exch{" O9A SPF"}=" AO9 SPF";
$resn_exch{" C4B SPF"}="AC4* SPF"; $resn_exch{" O4B SPF"}="AO4* SPF"; $resn_exch{" C5B SPF"}="AC5* SPF";
$resn_exch{" O5B SPF"}="AO5* SPF"; $resn_exch{" P1A SPF"}=" AP1 SPF"; $resn_exch{" O1A SPF"}=" AO1 SPF";
$resn_exch{" O2A SPF"}=" AO2 SPF"; $resn_exch{" O3A SPF"}=" AO3 SPF"; $resn_exch{" P2A SPF"}=" AP2 SPF";
$resn_exch{" O4A SPF"}=" AO4 SPF"; $resn_exch{" O5A SPF"}=" AO5 SPF"; $resn_exch{" O6A SPF"}=" AO6 SPF";
$resn_exch{" CBP SPF"}="PC11 SPF"; $resn_exch{" CCP SPF"}="PC12 SPF"; $resn_exch{" CDP SPF"}="PC13 SPF";
$resn_exch{" CEP SPF"}="PC14 SPF"; $resn_exch{" CAP SPF"}="PC10 SPF"; $resn_exch{" OAP SPF"}="PO10 SPF";
$resn_exch{" C9P SPF"}=" PC9 SPF"; $resn_exch{" O9P SPF"}=" PO9 SPF"; $resn_exch{" N8P SPF"}=" PN8 SPF";
$resn_exch{" C7P SPF"}=" PC7 SPF"; $resn_exch{" C6P SPF"}=" PC6 SPF"; $resn_exch{" C5P SPF"}=" PC5 SPF";
$resn_exch{" O5P SPF"}=" PO5 SPF"; $resn_exch{" N4P SPF"}=" PN4 SPF"; $resn_exch{" C3P SPF"}=" PC3 SPF";
$resn_exch{" C2P SPF"}=" PC2 SPF"; $resn_exch{" S1P SPF"}=" PS1 SPF"; $resn_exch{" O5' SPG"}=" O5* SPG";
$resn_exch{" C5' SPG"}=" C5* SPG"; $resn_exch{" C4' SPG"}=" C4* SPG"; $resn_exch{" O4' SPG"}=" O4* SPG";
$resn_exch{" C3' SPG"}=" C3* SPG"; $resn_exch{" O3' SPG"}=" O3* SPG"; $resn_exch{" C2' SPG"}=" C2* SPG";
$resn_exch{" O2' SPG"}=" O2* SPG"; $resn_exch{" C1' SPG"}=" C1* SPG"; $resn_exch{" S5' SPT"}=" S5* SPT";
$resn_exch{" C2' SPT"}=" C2* SPT"; $resn_exch{" C5' SPT"}=" C5* SPT"; $resn_exch{" C4' SPT"}=" C4* SPT";
$resn_exch{" O4' SPT"}=" O4* SPT"; $resn_exch{" C1' SPT"}=" C1* SPT"; $resn_exch{" C3' SPT"}=" C3* SPT";
$resn_exch{" O3' SPT"}=" O3* SPT"; $resn_exch{" OP3 SPT"}=" O3P SPT"; $resn_exch{" OP1 SPT"}=" O1P SPT";
$resn_exch{" OP2 SPT"}=" O2P SPT"; $resn_exch{" OP1 SRA"}=" O1P SRA"; $resn_exch{" OP3 SRA"}=" O3P SRA";
$resn_exch{" O5' SRA"}=" O5* SRA"; $resn_exch{" C5' SRA"}=" C5* SRA"; $resn_exch{" C4' SRA"}=" C4* SRA";
$resn_exch{" O4' SRA"}=" O4* SRA"; $resn_exch{" C3' SRA"}=" C3* SRA"; $resn_exch{" O3' SRA"}=" O3* SRA";
$resn_exch{" C2' SRA"}=" C2* SRA"; $resn_exch{" C1' SRA"}=" C1* SRA"; $resn_exch{" O2' SRA"}=" O2* SRA";
$resn_exch{" NA  SRM"}=" N_A SRM"; $resn_exch{" NB  SRM"}=" N_B SRM"; $resn_exch{" NC  SRM"}=" N_C SRM";
$resn_exch{" ND  SRM"}=" N_D SRM"; $resn_exch{" O5' SRP"}=" O5* SRP"; $resn_exch{" C5' SRP"}=" C5* SRP";
$resn_exch{" C4' SRP"}=" C4* SRP"; $resn_exch{" O4' SRP"}=" O4* SRP"; $resn_exch{" C3' SRP"}=" C3* SRP";
$resn_exch{" O3' SRP"}=" O3* SRP"; $resn_exch{" C2' SRP"}=" C2* SRP"; $resn_exch{" O2' SRP"}=" O2* SRP";
$resn_exch{" C1' SRP"}=" C1* SRP"; $resn_exch{" O5' SSA"}=" O5* SSA"; $resn_exch{" C5' SSA"}=" C5* SSA";
$resn_exch{" C4' SSA"}=" C4* SSA"; $resn_exch{" O4' SSA"}=" O4* SSA"; $resn_exch{" C3' SSA"}=" C3* SSA";
$resn_exch{" O3' SSA"}=" O3* SSA"; $resn_exch{" C2' SSA"}=" C2* SSA"; $resn_exch{" O2' SSA"}=" O2* SSA";
$resn_exch{" C1' SSA"}=" C1* SSA"; $resn_exch{" O2' SSU"}=" O2* SSU"; $resn_exch{" C2' SSU"}=" C2* SSU";
$resn_exch{" C1' SSU"}=" C1* SSU"; $resn_exch{" O4' SSU"}=" O4* SSU"; $resn_exch{" C4' SSU"}=" C4* SSU";
$resn_exch{" C5' SSU"}=" C5* SSU"; $resn_exch{" O5' SSU"}=" O5* SSU"; $resn_exch{" OP2 SSU"}=" O2P SSU";
$resn_exch{" OP3 SSU"}=" O3P SSU"; $resn_exch{" C3' SSU"}=" C3* SSU"; $resn_exch{" O3' SSU"}=" O3* SSU";
$resn_exch{" O5' STP"}=" O5* STP"; $resn_exch{" C5' STP"}=" C5* STP"; $resn_exch{" C4' STP"}=" C4* STP";
$resn_exch{" O4' STP"}=" O4* STP"; $resn_exch{" C3' STP"}=" C3* STP"; $resn_exch{" O3' STP"}=" O3* STP";
$resn_exch{" C2' STP"}=" C2* STP"; $resn_exch{" C1' STP"}=" C1* STP"; $resn_exch{" N1' SU2"}=" N1* SU2";
$resn_exch{" C2B SU2"}=" C2* SU2"; $resn_exch{" C3B SU2"}=" C3* SU2"; $resn_exch{" N4' SU2"}=" N4* SU2";
$resn_exch{" C5B SU2"}=" C5* SU2"; $resn_exch{" C6B SU2"}=" C6* SU2"; $resn_exch{" C1' SUD"}=" C1* SUD";
$resn_exch{" C2' SUD"}=" C2* SUD"; $resn_exch{" O2' SUD"}=" O2* SUD"; $resn_exch{" C3' SUD"}=" C3* SUD";
$resn_exch{" C4' SUD"}=" C4* SUD"; $resn_exch{" O4' SUD"}=" O4* SUD"; $resn_exch{" O3' SUD"}=" O3* SUD";
$resn_exch{" C5' SUD"}=" C5* SUD"; $resn_exch{" O5' SUD"}=" O5* SUD"; $resn_exch{" OP1 SUR"}=" O1P SUR";
$resn_exch{" OP2 SUR"}=" O2P SUR"; $resn_exch{" OP3 SUR"}=" O3P SUR"; $resn_exch{" OP3   T"}=" O3P   T";
$resn_exch{" OP1   T"}=" O1P   T"; $resn_exch{" OP2   T"}=" O2P   T"; $resn_exch{" O5'   T"}=" O5*   T";
$resn_exch{" C5'   T"}=" C5*   T"; $resn_exch{" C4'   T"}=" C4*   T"; $resn_exch{" O4'   T"}=" O4*   T";
$resn_exch{" C3'   T"}=" C3*   T"; $resn_exch{" O3'   T"}=" O3*   T"; $resn_exch{" C2'   T"}=" C2*   T";
$resn_exch{" C1'   T"}=" C1*   T"; $resn_exch{" C7    T"}=" C5M   T"; $resn_exch{" OP1 T23"}=" O1P T23";
$resn_exch{" OP2 T23"}=" O2P T23"; $resn_exch{" OP3 T23"}=" O3P T23"; $resn_exch{" O5' T23"}=" O5* T23";
$resn_exch{" C2' T23"}=" C2* T23"; $resn_exch{" O2' T23"}=" O2* T23"; $resn_exch{" C5' T23"}=" C5* T23";
$resn_exch{" C4' T23"}=" C4* T23"; $resn_exch{" O4' T23"}=" O4* T23"; $resn_exch{" C1' T23"}=" C1* T23";
$resn_exch{" C3' T23"}=" C3* T23"; $resn_exch{" OP3 T2S"}=" O3P T2S"; $resn_exch{" O5' T2S"}=" O5* T2S";
$resn_exch{" C5' T2S"}=" C5* T2S"; $resn_exch{" C4' T2S"}=" C4* T2S"; $resn_exch{" O4' T2S"}=" O4* T2S";
$resn_exch{" C3' T2S"}=" C3* T2S"; $resn_exch{" O3' T2S"}=" O3* T2S"; $resn_exch{" C2' T2S"}=" C2* T2S";
$resn_exch{" C1' T2S"}=" C1* T2S"; $resn_exch{" OP1 T2T"}=" O1P T2T"; $resn_exch{" OP2 T2T"}=" O2P T2T";
$resn_exch{" O5' T2T"}=" O5* T2T"; $resn_exch{" C5' T2T"}=" C5* T2T"; $resn_exch{" C4' T2T"}=" C4* T2T";
$resn_exch{" O4' T2T"}=" O4* T2T"; $resn_exch{" C1' T2T"}=" C1* T2T"; $resn_exch{" C2' T2T"}=" C2* T2T";
$resn_exch{" O2' T2T"}=" O2* T2T"; $resn_exch{" C3' T2T"}=" C3* T2T"; $resn_exch{" O3' T2T"}=" O3* T2T";
$resn_exch{" OP3 T2T"}=" O3P T2T"; $resn_exch{" O5' T31"}=" O5* T31"; $resn_exch{" C5' T31"}=" C5* T31";
$resn_exch{" C4' T31"}=" C4* T31"; $resn_exch{" O4' T31"}=" O4* T31"; $resn_exch{" C3' T31"}=" C3* T31";
$resn_exch{" O3' T31"}=" O3* T31"; $resn_exch{" C2' T31"}=" C2* T31"; $resn_exch{" O2' T31"}=" O2* T31";
$resn_exch{" C1' T31"}=" C1* T31"; $resn_exch{" O5' T32"}=" O5* T32"; $resn_exch{" C5' T32"}=" C5* T32";
$resn_exch{" C4' T32"}=" C4* T32"; $resn_exch{" C3' T32"}=" C3* T32"; $resn_exch{" O3' T32"}=" O3* T32";
$resn_exch{" C2' T32"}=" C2* T32"; $resn_exch{" C1' T32"}=" C1* T32"; $resn_exch{" C'  T32"}=" C*  T32";
$resn_exch{" CM' T32"}=" CM* T32"; $resn_exch{" O5' T36"}=" O5* T36"; $resn_exch{" C5' T36"}=" C5* T36";
$resn_exch{" C4' T36"}=" C4* T36"; $resn_exch{" O4' T36"}=" O4* T36"; $resn_exch{" C3' T36"}=" C3* T36";
$resn_exch{" O3' T36"}=" O3* T36"; $resn_exch{" C2' T36"}=" C2* T36"; $resn_exch{" C1' T36"}=" C1* T36";
$resn_exch{" O5' T37"}=" O5* T37"; $resn_exch{" C5' T37"}=" C5* T37"; $resn_exch{" C4' T37"}=" C4* T37";
$resn_exch{" O4' T37"}=" O4* T37"; $resn_exch{" C3' T37"}=" C3* T37"; $resn_exch{" N3' T37"}=" N3* T37";
$resn_exch{" C2' T37"}=" C2* T37"; $resn_exch{" C1' T37"}=" C1* T37"; $resn_exch{" O5' T38"}=" O5* T38";
$resn_exch{" C5' T38"}=" C5* T38"; $resn_exch{" C4' T38"}=" C4* T38"; $resn_exch{" O4' T38"}=" O4* T38";
$resn_exch{" C3' T38"}=" C3* T38"; $resn_exch{" O3' T38"}=" O3* T38"; $resn_exch{" C2' T38"}=" C2* T38";
$resn_exch{" O2' T38"}=" O2* T38"; $resn_exch{" C1' T38"}=" C1* T38"; $resn_exch{" CA' T38"}=" CA* T38";
$resn_exch{" CB' T38"}=" CB* T38"; $resn_exch{" OC' T38"}=" OC* T38"; $resn_exch{" CD' T38"}=" CD* T38";
$resn_exch{" O5' T39"}=" O5* T39"; $resn_exch{" C5' T39"}=" C5* T39"; $resn_exch{" C4' T39"}=" C4* T39";
$resn_exch{" O4' T39"}=" O4* T39"; $resn_exch{" C3' T39"}=" C3* T39"; $resn_exch{" O3' T39"}=" O3* T39";
$resn_exch{" C2' T39"}=" C2* T39"; $resn_exch{" O2' T39"}=" O2* T39"; $resn_exch{" C1' T39"}=" C1* T39";
$resn_exch{" CA' T39"}=" CA* T39"; $resn_exch{" CD' T39"}=" CD* T39"; $resn_exch{" CB' T39"}=" CB* T39";
$resn_exch{" OC' T39"}=" OC* T39"; $resn_exch{" OP1 T3P"}=" O1P T3P"; $resn_exch{" OP2 T3P"}=" O2P T3P";
$resn_exch{" OP3 T3P"}=" O3P T3P"; $resn_exch{" O5' T3P"}=" O5* T3P"; $resn_exch{" C5' T3P"}=" C5* T3P";
$resn_exch{" C4' T3P"}=" C4* T3P"; $resn_exch{" O4' T3P"}=" O4* T3P"; $resn_exch{" C3' T3P"}=" C3* T3P";
$resn_exch{" O3' T3P"}=" O3* T3P"; $resn_exch{" C2' T3P"}=" C2* T3P"; $resn_exch{" C1' T3P"}=" C1* T3P";
$resn_exch{" C1' T41"}=" C1* T41"; $resn_exch{" C2' T41"}=" C2* T41"; $resn_exch{" O2' T41"}=" O2* T41";
$resn_exch{" C3' T41"}=" C3* T41"; $resn_exch{" C4' T41"}=" C4* T41"; $resn_exch{" O3' T41"}=" O3* T41";
$resn_exch{" O4' T41"}=" O4* T41"; $resn_exch{" C5' T41"}=" C5* T41"; $resn_exch{" O5' T41"}=" O5* T41";
$resn_exch{" C6' T41"}=" C6* T41"; $resn_exch{" O5' T48"}=" O5* T48"; $resn_exch{" C5' T48"}=" C5* T48";
$resn_exch{" C4' T48"}=" C4* T48"; $resn_exch{" C3' T48"}=" C3* T48"; $resn_exch{" O3' T48"}=" O3* T48";
$resn_exch{" C2' T48"}=" C2* T48"; $resn_exch{" C1' T48"}=" C1* T48"; $resn_exch{" O5' T49"}=" O5* T49";
$resn_exch{" C5' T49"}=" C5* T49"; $resn_exch{" C4' T49"}=" C4* T49"; $resn_exch{" C3' T49"}=" C3* T49";
$resn_exch{" O3' T49"}=" O3* T49"; $resn_exch{" C2' T49"}=" C2* T49"; $resn_exch{" C1' T49"}=" C1* T49";
$resn_exch{" OP3 T4S"}=" O3P T4S"; $resn_exch{" OP1 T4S"}=" O1P T4S"; $resn_exch{" OP2 T4S"}=" O2P T4S";
$resn_exch{" O5' T4S"}=" O5* T4S"; $resn_exch{" C2' T4S"}=" C2* T4S"; $resn_exch{" C5' T4S"}=" C5* T4S";
$resn_exch{" C4' T4S"}=" C4* T4S"; $resn_exch{" O4' T4S"}=" O4* T4S"; $resn_exch{" C1' T4S"}=" C1* T4S";
$resn_exch{" C3' T4S"}=" C3* T4S"; $resn_exch{" O3' T4S"}=" O3* T4S"; $resn_exch{" N1B T5A"}=" BN1 T5A";
$resn_exch{" C6B T5A"}=" BC6 T5A"; $resn_exch{" C2B T5A"}=" BC2 T5A"; $resn_exch{" O2B T5A"}=" BO2 T5A";
$resn_exch{" N3B T5A"}=" BN3 T5A"; $resn_exch{" C4B T5A"}=" BC4 T5A"; $resn_exch{" O4B T5A"}=" BO4 T5A";
$resn_exch{" C5B T5A"}=" BC5 T5A"; $resn_exch{" C7B T5A"}=" BC7 T5A"; $resn_exch{" C5E T5A"}="BC5* T5A";
$resn_exch{" C4E T5A"}="BC4* T5A"; $resn_exch{" O4E T5A"}="BO4* T5A"; $resn_exch{" C1E T5A"}="BC1* T5A";
$resn_exch{" C2E T5A"}="BC2* T5A"; $resn_exch{" C3E T5A"}="BC3* T5A"; $resn_exch{" O3E T5A"}="BO3* T5A";
$resn_exch{" O5E T5A"}="BO5* T5A"; $resn_exch{" O2X T5A"}=" O2B T5A"; $resn_exch{" O5F T5A"}="AO5* T5A";
$resn_exch{" C5F T5A"}="AC5* T5A"; $resn_exch{" C4F T5A"}="AC4* T5A"; $resn_exch{" O4F T5A"}="AO4* T5A";
$resn_exch{" C1F T5A"}="AC1* T5A"; $resn_exch{" C2F T5A"}="AC2* T5A"; $resn_exch{" O2F T5A"}="AO2* T5A";
$resn_exch{" C3F T5A"}="AC3* T5A"; $resn_exch{" O3F T5A"}="AO3* T5A"; $resn_exch{" N9A T5A"}=" AN9 T5A";
$resn_exch{" C4A T5A"}=" AC4 T5A"; $resn_exch{" N3A T5A"}=" AN3 T5A"; $resn_exch{" C2A T5A"}=" AC2 T5A";
$resn_exch{" N1A T5A"}=" AN1 T5A"; $resn_exch{" C6A T5A"}=" AC6 T5A"; $resn_exch{" N6A T5A"}=" AN6 T5A";
$resn_exch{" C5A T5A"}=" AC5 T5A"; $resn_exch{" N7A T5A"}=" AN7 T5A"; $resn_exch{" C8A T5A"}=" AC8 T5A";
$resn_exch{" OP1 T6A"}=" O1P T6A"; $resn_exch{" OP2 T6A"}=" O2P T6A"; $resn_exch{" OP3 T6A"}=" O3P T6A";
$resn_exch{" O5' T6A"}=" O5* T6A"; $resn_exch{" C5' T6A"}=" C5* T6A"; $resn_exch{" C4' T6A"}=" C4* T6A";
$resn_exch{" O4' T6A"}=" O4* T6A"; $resn_exch{" C3' T6A"}=" C3* T6A"; $resn_exch{" O3' T6A"}=" O3* T6A";
$resn_exch{" C2' T6A"}=" C2* T6A"; $resn_exch{" O2' T6A"}=" O2* T6A"; $resn_exch{" C1' T6A"}=" C1* T6A";
$resn_exch{" ODA T6A"}="O13A T6A"; $resn_exch{" ODB T6A"}="O13B T6A"; $resn_exch{" OP3 TA3"}=" O3P TA3";
$resn_exch{" OP1 TA3"}=" O1P TA3"; $resn_exch{" OP2 TA3"}=" O2P TA3"; $resn_exch{" O5' TA3"}=" O5* TA3";
$resn_exch{" C5' TA3"}=" C5* TA3"; $resn_exch{" C4' TA3"}=" C4* TA3"; $resn_exch{" O4' TA3"}=" O4* TA3";
$resn_exch{" C3' TA3"}=" C3* TA3"; $resn_exch{" O3' TA3"}=" O3* TA3"; $resn_exch{" C2' TA3"}=" C2* TA3";
$resn_exch{" C1' TA3"}=" C1* TA3"; $resn_exch{" PA  TAD"}=" AP  TAD"; $resn_exch{" O1A TAD"}=" AO1 TAD";
$resn_exch{" O2A TAD"}=" AO2 TAD"; $resn_exch{" O5B TAD"}="AO5* TAD"; $resn_exch{" C5B TAD"}="AC5* TAD";
$resn_exch{" C4B TAD"}="AC4* TAD"; $resn_exch{" O4B TAD"}="AO4* TAD"; $resn_exch{" C3B TAD"}="AC3* TAD";
$resn_exch{" O3B TAD"}="AO3* TAD"; $resn_exch{" C2B TAD"}="AC2* TAD"; $resn_exch{" O2B TAD"}="AO2* TAD";
$resn_exch{" C1B TAD"}="AC1* TAD"; $resn_exch{" N9A TAD"}=" AN9 TAD"; $resn_exch{" C8A TAD"}=" AC8 TAD";
$resn_exch{" N7A TAD"}=" AN7 TAD"; $resn_exch{" C5A TAD"}=" AC5 TAD"; $resn_exch{" C6A TAD"}=" AC6 TAD";
$resn_exch{" N6A TAD"}=" AN6 TAD"; $resn_exch{" N1A TAD"}=" AN1 TAD"; $resn_exch{" C2A TAD"}=" AC2 TAD";
$resn_exch{" N3A TAD"}=" AN3 TAD"; $resn_exch{" C4A TAD"}=" AC4 TAD"; $resn_exch{" PN  TAD"}=" NP  TAD";
$resn_exch{" O1N TAD"}=" NO1 TAD"; $resn_exch{" O2N TAD"}=" NO2 TAD"; $resn_exch{" O5D TAD"}="NO5* TAD";
$resn_exch{" C5D TAD"}="NC5* TAD"; $resn_exch{" C4D TAD"}="NC4* TAD"; $resn_exch{" O4D TAD"}="NO4* TAD";
$resn_exch{" C3D TAD"}="NC3* TAD"; $resn_exch{" O3D TAD"}="NO3* TAD"; $resn_exch{" C2D TAD"}="NC2* TAD";
$resn_exch{" O2D TAD"}="NO2* TAD"; $resn_exch{" C1D TAD"}="NC1* TAD"; $resn_exch{" C2N TAD"}=" NC2 TAD";
$resn_exch{" N3N TAD"}=" NN3 TAD"; $resn_exch{" C4N TAD"}=" NC4 TAD"; $resn_exch{" C5N TAD"}=" NC5 TAD";
$resn_exch{" S1N TAD"}=" NS1 TAD"; $resn_exch{" C6N TAD"}=" NC6 TAD"; $resn_exch{" O6N TAD"}=" NO6 TAD";
$resn_exch{" N6N TAD"}=" NN6 TAD"; $resn_exch{" OP1 TAF"}=" O1P TAF"; $resn_exch{" OP2 TAF"}=" O2P TAF";
$resn_exch{" OP3 TAF"}=" O3P TAF"; $resn_exch{" O5' TAF"}=" O5* TAF"; $resn_exch{" F2' TAF"}=" F2* TAF";
$resn_exch{" C2' TAF"}=" C2* TAF"; $resn_exch{" C5' TAF"}=" C5* TAF"; $resn_exch{" C4' TAF"}=" C4* TAF";
$resn_exch{" O4' TAF"}=" O4* TAF"; $resn_exch{" C1' TAF"}=" C1* TAF"; $resn_exch{" C3' TAF"}=" C3* TAF";
$resn_exch{" O3' TAF"}=" O3* TAF"; $resn_exch{" PA  TAP"}=" AP  TAP"; $resn_exch{" O1A TAP"}=" AO1 TAP";
$resn_exch{" O2A TAP"}=" AO2 TAP"; $resn_exch{" O5B TAP"}="AO5* TAP"; $resn_exch{" C5B TAP"}="AC5* TAP";
$resn_exch{" C4B TAP"}="AC4* TAP"; $resn_exch{" O4B TAP"}="AO4* TAP"; $resn_exch{" C3B TAP"}="AC3* TAP";
$resn_exch{" O3B TAP"}="AO3* TAP"; $resn_exch{" C2B TAP"}="AC2* TAP"; $resn_exch{" O2B TAP"}="AO2* TAP";
$resn_exch{" C1B TAP"}="AC1* TAP"; $resn_exch{" N9A TAP"}=" AN9 TAP"; $resn_exch{" C8A TAP"}=" AC8 TAP";
$resn_exch{" N7A TAP"}=" AN7 TAP"; $resn_exch{" C5A TAP"}=" AC5 TAP"; $resn_exch{" C6A TAP"}=" AC6 TAP";
$resn_exch{" N6A TAP"}=" AN6 TAP"; $resn_exch{" N1A TAP"}=" AN1 TAP"; $resn_exch{" C2A TAP"}=" AC2 TAP";
$resn_exch{" N3A TAP"}=" AN3 TAP"; $resn_exch{" C4A TAP"}=" AC4 TAP"; $resn_exch{" PN  TAP"}=" NP  TAP";
$resn_exch{" O1N TAP"}=" NO1 TAP"; $resn_exch{" O2N TAP"}=" NO2 TAP"; $resn_exch{" O5D TAP"}="NO5* TAP";
$resn_exch{" C5D TAP"}="NC5* TAP"; $resn_exch{" C4D TAP"}="NC4* TAP"; $resn_exch{" O4D TAP"}="NO4* TAP";
$resn_exch{" C3D TAP"}="NC3* TAP"; $resn_exch{" O3D TAP"}="NO3* TAP"; $resn_exch{" C2D TAP"}="NC2* TAP";
$resn_exch{" O2D TAP"}="NO2* TAP"; $resn_exch{" C1D TAP"}="NC1* TAP"; $resn_exch{" N1N TAP"}=" NN1 TAP";
$resn_exch{" C2N TAP"}=" NC2 TAP"; $resn_exch{" C3N TAP"}=" NC3 TAP"; $resn_exch{" C7N TAP"}=" NC7 TAP";
$resn_exch{" S7N TAP"}=" NS7 TAP"; $resn_exch{" N7N TAP"}=" NN7 TAP"; $resn_exch{" C4N TAP"}=" NC4 TAP";
$resn_exch{" C5N TAP"}=" NC5 TAP"; $resn_exch{" C6N TAP"}=" NC6 TAP"; $resn_exch{" P2B TAP"}="AP2* TAP";
$resn_exch{" O1X TAP"}="AOP1 TAP"; $resn_exch{" O2X TAP"}="AOP2 TAP"; $resn_exch{" O3X TAP"}="AOP3 TAP";
$resn_exch{" O5' TAT"}=" O5* TAT"; $resn_exch{" C5' TAT"}=" C5* TAT"; $resn_exch{" C4' TAT"}=" C4* TAT";
$resn_exch{" O4' TAT"}=" O4* TAT"; $resn_exch{" C3' TAT"}=" C3* TAT"; $resn_exch{" O3' TAT"}=" O3* TAT";
$resn_exch{" C2' TAT"}=" C2* TAT"; $resn_exch{" O2' TAT"}=" O2* TAT"; $resn_exch{" C1' TAT"}=" C1* TAT";
$resn_exch{" O5' TBD"}=" O5* TBD"; $resn_exch{" C5' TBD"}=" C5* TBD"; $resn_exch{" C4' TBD"}=" C4* TBD";
$resn_exch{" O4' TBD"}=" O4* TBD"; $resn_exch{" C1' TBD"}=" C1* TBD"; $resn_exch{" C2' TBD"}=" C2* TBD";
$resn_exch{" C3' TBD"}=" C3* TBD"; $resn_exch{" O3' TBD"}=" O3* TBD"; $resn_exch{" H51 TBS"}="1H5  TBS";
$resn_exch{" O5' TC1"}=" O5* TC1"; $resn_exch{" C5' TC1"}=" C5* TC1"; $resn_exch{" C4' TC1"}=" C4* TC1";
$resn_exch{" C3' TC1"}=" C3* TC1"; $resn_exch{" C2' TC1"}=" C2* TC1"; $resn_exch{" C1' TC1"}=" C1* TC1";
$resn_exch{" O4' TC1"}=" O4* TC1"; $resn_exch{" O3' TC1"}=" O3* TC1"; $resn_exch{" OP3 TC1"}=" O3P TC1";
$resn_exch{" O5' TCP"}=" O5* TCP"; $resn_exch{" C5' TCP"}=" C5* TCP"; $resn_exch{" C4' TCP"}=" C4* TCP";
$resn_exch{" O4' TCP"}=" O4* TCP"; $resn_exch{" C3' TCP"}=" C3* TCP"; $resn_exch{" O3' TCP"}=" O3* TCP";
$resn_exch{" C2' TCP"}=" C2* TCP"; $resn_exch{" C1' TCP"}=" C1* TCP"; $resn_exch{" C5' TDI"}=" C5* TDI";
$resn_exch{" C4' TDI"}=" C4* TDI"; $resn_exch{" C1' TDI"}=" C1* TDI"; $resn_exch{" N1' TDI"}=" N1* TDI";
$resn_exch{" C2' TDI"}=" C2* TDI"; $resn_exch{" C3' TDI"}=" C3* TDI"; $resn_exch{" O3' TDI"}=" O3* TDI";
$resn_exch{" C7' TDK"}=" C7* TDK"; $resn_exch{" C5' TDK"}=" C5* TDK"; $resn_exch{" C6' TDK"}=" C6* TDK";
$resn_exch{" N1' TDK"}=" N1* TDK"; $resn_exch{" C2' TDK"}=" C2* TDK"; $resn_exch{" N3' TDK"}=" N3* TDK";
$resn_exch{" C4' TDK"}=" C4* TDK"; $resn_exch{" N4' TDK"}=" N4* TDK"; $resn_exch{" C7' TDL"}=" C7* TDL";
$resn_exch{" C5' TDL"}=" C5* TDL"; $resn_exch{" C6' TDL"}=" C6* TDL"; $resn_exch{" N1' TDL"}=" N1* TDL";
$resn_exch{" C2' TDL"}=" C2* TDL"; $resn_exch{" N3' TDL"}=" N3* TDL"; $resn_exch{" C4' TDL"}=" C4* TDL";
$resn_exch{" N4' TDL"}=" N4* TDL"; $resn_exch{" C7' TDM"}=" C7* TDM"; $resn_exch{" C5' TDM"}=" C5* TDM";
$resn_exch{" C4' TDM"}=" C4* TDM"; $resn_exch{" N4' TDM"}=" N4* TDM"; $resn_exch{" N3' TDM"}=" N3* TDM";
$resn_exch{" C2' TDM"}=" C2* TDM"; $resn_exch{" N1' TDM"}=" N1* TDM"; $resn_exch{" C6' TDM"}=" C6* TDM";
$resn_exch{" N1' TDP"}=" N1* TDP"; $resn_exch{" C2' TDP"}=" C2* TDP"; $resn_exch{" N3' TDP"}=" N3* TDP";
$resn_exch{" C4' TDP"}=" C4* TDP"; $resn_exch{" N4' TDP"}=" N4* TDP"; $resn_exch{" C5' TDP"}=" C5* TDP";
$resn_exch{" C6' TDP"}=" C6* TDP"; $resn_exch{" C1B TDX"}=" C1* TDX"; $resn_exch{" C2B TDX"}=" C2* TDX";
$resn_exch{" C3B TDX"}=" C3* TDX"; $resn_exch{" O3B TDX"}=" O3* TDX"; $resn_exch{" O4B TDX"}=" O4* TDX";
$resn_exch{" C4B TDX"}=" C4* TDX"; $resn_exch{" C5B TDX"}=" C5* TDX"; $resn_exch{" O5B TDX"}=" O5* TDX";
$resn_exch{" OP1 TFE"}=" O1P TFE"; $resn_exch{" OP2 TFE"}=" O2P TFE"; $resn_exch{" O5' TFE"}=" O5* TFE";
$resn_exch{" C5' TFE"}=" C5* TFE"; $resn_exch{" C4' TFE"}=" C4* TFE"; $resn_exch{" O4' TFE"}=" O4* TFE";
$resn_exch{" C1' TFE"}=" C1* TFE"; $resn_exch{" C2' TFE"}=" C2* TFE"; $resn_exch{" O2' TFE"}=" O2* TFE";
$resn_exch{" C3' TFE"}=" C3* TFE"; $resn_exch{" O3' TFE"}=" O3* TFE"; $resn_exch{" OP3 TFE"}=" O3P TFE";
$resn_exch{" OP2 TFO"}=" O2P TFO"; $resn_exch{" OP1 TFO"}=" O1P TFO"; $resn_exch{" OP3 TFO"}=" O3P TFO";
$resn_exch{" OP1 TFT"}=" O1P TFT"; $resn_exch{" OP2 TFT"}=" O2P TFT"; $resn_exch{" OP3 TFT"}=" O3P TFT";
$resn_exch{" OEB TGF"}="OE11 TGF"; $resn_exch{" OEC TGF"}="OE12 TGF"; $resn_exch{" OED TGF"}="OE13 TGF";
$resn_exch{" OEN TGF"}="OE23 TGF"; $resn_exch{" S5' TGP"}=" S5* TGP"; $resn_exch{" C2' TGP"}=" C2* TGP";
$resn_exch{" C5' TGP"}=" C5* TGP"; $resn_exch{" C4' TGP"}=" C4* TGP"; $resn_exch{" O4' TGP"}=" O4* TGP";
$resn_exch{" C1' TGP"}=" C1* TGP"; $resn_exch{" C3' TGP"}=" C3* TGP"; $resn_exch{" O3' TGP"}=" O3* TGP";
$resn_exch{" OP3 TGP"}=" O3P TGP"; $resn_exch{" OP1 TGP"}=" O1P TGP"; $resn_exch{" OP2 TGP"}=" O2P TGP";
$resn_exch{" O5' THM"}=" O5* THM"; $resn_exch{" C5' THM"}=" C5* THM"; $resn_exch{" C4' THM"}=" C4* THM";
$resn_exch{" O4' THM"}=" O4* THM"; $resn_exch{" C3' THM"}=" C3* THM"; $resn_exch{" O3' THM"}=" O3* THM";
$resn_exch{" C2' THM"}=" C2* THM"; $resn_exch{" C1' THM"}=" C1* THM"; $resn_exch{" O5' THP"}=" O5* THP";
$resn_exch{" C5' THP"}=" C5* THP"; $resn_exch{" C4' THP"}=" C4* THP"; $resn_exch{" O4' THP"}=" O4* THP";
$resn_exch{" C3' THP"}=" C3* THP"; $resn_exch{" O3' THP"}=" O3* THP"; $resn_exch{" C2' THP"}=" C2* THP";
$resn_exch{" C1' THP"}=" C1* THP"; $resn_exch{" O5' THS"}=" O5* THS"; $resn_exch{" C5' THS"}=" C5* THS";
$resn_exch{" C4' THS"}=" C4* THS"; $resn_exch{" O4' THS"}=" O4* THS"; $resn_exch{" C3' THS"}=" C3* THS";
$resn_exch{" O3' THS"}=" O3* THS"; $resn_exch{" C2' THS"}=" C2* THS"; $resn_exch{" C1' THS"}=" C1* THS";
$resn_exch{" C1' THU"}=" C1* THU"; $resn_exch{" C2' THU"}=" C2* THU"; $resn_exch{" C3' THU"}=" C3* THU";
$resn_exch{" O3' THU"}=" O3* THU"; $resn_exch{" C4' THU"}=" C4* THU"; $resn_exch{" O4' THU"}=" O4* THU";
$resn_exch{" C5' THU"}=" C5* THU"; $resn_exch{" O5' THU"}=" O5* THU"; $resn_exch{" C5' THV"}=" C5* THV";
$resn_exch{" C6' THV"}=" C6* THV"; $resn_exch{" N1' THV"}=" N1* THV"; $resn_exch{" C2' THV"}=" C2* THV";
$resn_exch{" N3' THV"}=" N3* THV"; $resn_exch{" C4' THV"}=" C4* THV"; $resn_exch{" N4' THV"}=" N4* THV";
$resn_exch{" C5' THW"}=" C5* THW"; $resn_exch{" C6' THW"}=" C6* THW"; $resn_exch{" N1' THW"}=" N1* THW";
$resn_exch{" C2' THW"}=" C2* THW"; $resn_exch{" N3' THW"}=" N3* THW"; $resn_exch{" C4' THW"}=" C4* THW";
$resn_exch{" N4' THW"}=" N4* THW"; $resn_exch{" C1' THX"}=" C1* THX"; $resn_exch{" C2' THX"}=" C2* THX";
$resn_exch{" C3' THX"}=" C3* THX"; $resn_exch{" C4' THX"}=" C4* THX"; $resn_exch{" C5' THX"}=" C5* THX";
$resn_exch{" O3' THX"}=" O3* THX"; $resn_exch{" O4' THX"}=" O4* THX"; $resn_exch{" O5' THX"}=" O5* THX";
$resn_exch{" C5' THY"}=" C5* THY"; $resn_exch{" C6' THY"}=" C6* THY"; $resn_exch{" N1' THY"}=" N1* THY";
$resn_exch{" C2' THY"}=" C2* THY"; $resn_exch{" N3' THY"}=" N3* THY"; $resn_exch{" C4' THY"}=" C4* THY";
$resn_exch{" N4' THY"}=" N4* THY"; $resn_exch{" C1' TIA"}=" C1* TIA"; $resn_exch{" C2' TIA"}=" C2* TIA";
$resn_exch{" O2' TIA"}=" O2* TIA"; $resn_exch{" C3' TIA"}=" C3* TIA"; $resn_exch{" O3' TIA"}=" O3* TIA";
$resn_exch{" C4' TIA"}=" C4* TIA"; $resn_exch{" O4' TIA"}=" O4* TIA"; $resn_exch{" C5' TIA"}=" C5* TIA";
$resn_exch{" O2' TIZ"}=" O2* TIZ"; $resn_exch{" C2' TIZ"}=" C2* TIZ"; $resn_exch{" C3' TIZ"}=" C3* TIZ";
$resn_exch{" O3' TIZ"}=" O3* TIZ"; $resn_exch{" C4' TIZ"}=" C4* TIZ"; $resn_exch{" C5' TIZ"}=" C5* TIZ";
$resn_exch{" O5' TIZ"}=" O5* TIZ"; $resn_exch{" O4' TIZ"}=" O4* TIZ"; $resn_exch{" C1' TIZ"}=" C1* TIZ";
$resn_exch{" O5' TLB"}=" O5* TLB"; $resn_exch{" C5' TLB"}=" C5* TLB"; $resn_exch{" C4' TLB"}=" C4* TLB";
$resn_exch{" O4' TLB"}=" O4* TLB"; $resn_exch{" C3' TLB"}=" C3* TLB"; $resn_exch{" O3' TLB"}=" O3* TLB";
$resn_exch{" C2' TLB"}=" C2* TLB"; $resn_exch{" O2' TLB"}=" O2* TLB"; $resn_exch{" C1' TLB"}=" C1* TLB";
$resn_exch{" C9' TLB"}=" C9* TLB"; $resn_exch{" OP1 TLC"}=" O1P TLC"; $resn_exch{" OP2 TLC"}=" O2P TLC";
$resn_exch{" OP3 TLC"}=" O3P TLC"; $resn_exch{" O5' TLC"}=" O5* TLC"; $resn_exch{" C5' TLC"}=" C5* TLC";
$resn_exch{" C4' TLC"}=" C4* TLC"; $resn_exch{" O4' TLC"}=" O4* TLC"; $resn_exch{" C1' TLC"}=" C1* TLC";
$resn_exch{" C2' TLC"}=" C2* TLC"; $resn_exch{" C3' TLC"}=" C3* TLC"; $resn_exch{" O3' TLC"}=" O3* TLC";
$resn_exch{" OP1 TLN"}=" O1P TLN"; $resn_exch{" OP2 TLN"}=" O2P TLN"; $resn_exch{" OP3 TLN"}=" O3P TLN";
$resn_exch{" O5' TLN"}=" O5* TLN"; $resn_exch{" C5' TLN"}=" C5* TLN"; $resn_exch{" C4' TLN"}=" C4* TLN";
$resn_exch{" O4' TLN"}=" O4* TLN"; $resn_exch{" C1' TLN"}=" C1* TLN"; $resn_exch{" C3' TLN"}=" C3* TLN";
$resn_exch{" C2' TLN"}=" C2* TLN"; $resn_exch{" O2' TLN"}=" O2* TLN"; $resn_exch{" O3' TLN"}=" O3* TLN";
$resn_exch{" C6' TLN"}=" C6* TLN"; $resn_exch{" O5' TMP"}=" O5* TMP"; $resn_exch{" C5' TMP"}=" C5* TMP";
$resn_exch{" C4' TMP"}=" C4* TMP"; $resn_exch{" O4' TMP"}=" O4* TMP"; $resn_exch{" C3' TMP"}=" C3* TMP";
$resn_exch{" O3' TMP"}=" O3* TMP"; $resn_exch{" C2' TMP"}=" C2* TMP"; $resn_exch{" C1' TMP"}=" C1* TMP";
$resn_exch{" C5' TND"}=" C5* TND"; $resn_exch{" C4' TND"}=" C4* TND"; $resn_exch{" O4' TND"}=" O4* TND";
$resn_exch{" C3' TND"}=" C3* TND"; $resn_exch{" O3' TND"}=" O3* TND"; $resn_exch{" C2' TND"}=" C2* TND";
$resn_exch{" N2' TND"}=" N2* TND"; $resn_exch{" C1' TND"}=" C1* TND"; $resn_exch{" O5' TND"}=" O5* TND";
$resn_exch{" OXT TNR"}=" OB  TNR"; $resn_exch{" O   TNR"}=" OA  TNR"; $resn_exch{" O2' TOE"}=" O2* TOE";
$resn_exch{" CA' TOE"}=" CA* TOE"; $resn_exch{" CB' TOE"}=" CB* TOE"; $resn_exch{" OC' TOE"}=" OC* TOE";
$resn_exch{" CD' TOE"}=" CD* TOE"; $resn_exch{" CE' TOE"}=" CE* TOE"; $resn_exch{" OF' TOE"}=" OF* TOE";
$resn_exch{" CG' TOE"}=" CG* TOE"; $resn_exch{" CH' TOE"}=" CH* TOE"; $resn_exch{" OI' TOE"}=" OI* TOE";
$resn_exch{" CK' TOE"}=" CK* TOE"; $resn_exch{" OP1 TPC"}=" O1P TPC"; $resn_exch{" OP2 TPC"}=" O2P TPC";
$resn_exch{" OP3 TPC"}=" O3P TPC"; $resn_exch{" O5' TPE"}=" O5* TPE"; $resn_exch{" C5' TPE"}=" C5* TPE";
$resn_exch{" C4' TPE"}=" C4* TPE"; $resn_exch{" O4' TPE"}=" O4* TPE"; $resn_exch{" C3' TPE"}=" C3* TPE";
$resn_exch{" O3' TPE"}=" O3* TPE"; $resn_exch{" C2' TPE"}=" C2* TPE"; $resn_exch{" C1' TPE"}=" C1* TPE";
$resn_exch{" O5' TPG"}=" O5* TPG"; $resn_exch{" C5' TPG"}=" C5* TPG"; $resn_exch{" C4' TPG"}=" C4* TPG";
$resn_exch{" O4' TPG"}=" O4* TPG"; $resn_exch{" C3' TPG"}=" C3* TPG"; $resn_exch{" O3' TPG"}=" O3* TPG";
$resn_exch{" C2' TPG"}=" C2* TPG"; $resn_exch{" O2' TPG"}=" O2* TPG"; $resn_exch{" C1' TPG"}=" C1* TPG";
$resn_exch{" N1' TPW"}=" N1* TPW"; $resn_exch{" C2' TPW"}=" C2* TPW"; $resn_exch{" N3' TPW"}=" N3* TPW";
$resn_exch{" C4' TPW"}=" C4* TPW"; $resn_exch{" N4' TPW"}=" N4* TPW"; $resn_exch{" C5' TPW"}=" C5* TPW";
$resn_exch{" C6' TPW"}=" C6* TPW"; $resn_exch{" C2D TQD"}=" C2* TQD"; $resn_exch{" C4D TQD"}=" C4* TQD";
$resn_exch{" C5D TQD"}=" C5* TQD"; $resn_exch{" C6D TQD"}=" C6* TQD"; $resn_exch{" C7' TQD"}=" C7* TQD";
$resn_exch{" C8' TQD"}=" C8* TQD"; $resn_exch{" N1' TQD"}=" N1* TQD"; $resn_exch{" N2' TQD"}=" N2* TQD";
$resn_exch{" N3' TQD"}=" N3* TQD"; $resn_exch{" N4' TQD"}=" N4* TQD"; $resn_exch{" N9' TQD"}=" N9* TQD";
$resn_exch{" C9' TQD"}=" C9* TQD"; $resn_exch{" C4B TQT"}=" C4* TQT"; $resn_exch{" C8B TQT"}=" C8* TQT";
$resn_exch{" C'0 TQT"}=" C*0 TQT"; $resn_exch{" C'1 TQT"}=" C*1 TQT"; $resn_exch{" C'2 TQT"}=" C*2 TQT";
$resn_exch{" C'3 TQT"}=" C*3 TQT"; $resn_exch{" C'4 TQT"}=" C*4 TQT"; $resn_exch{" C'5 TQT"}=" C*5 TQT";
$resn_exch{" C'6 TQT"}=" C*6 TQT"; $resn_exch{" C'7 TQT"}=" C*7 TQT"; $resn_exch{" N'0 TQT"}=" N*0 TQT";
$resn_exch{" OP1  TS"}=" O1P  TS"; $resn_exch{" OP3  TS"}=" O3P  TS"; $resn_exch{" O5'  TS"}=" O5*  TS";
$resn_exch{" C5'  TS"}=" C5*  TS"; $resn_exch{" C4'  TS"}=" C4*  TS"; $resn_exch{" O4'  TS"}=" O4*  TS";
$resn_exch{" C3'  TS"}=" C3*  TS"; $resn_exch{" O3'  TS"}=" O3*  TS"; $resn_exch{" C2'  TS"}=" C2*  TS";
$resn_exch{" C1'  TS"}=" C1*  TS"; $resn_exch{" O5' TSB"}=" O5* TSB"; $resn_exch{" C5' TSB"}=" C5* TSB";
$resn_exch{" C4' TSB"}=" C4* TSB"; $resn_exch{" O4' TSB"}=" O4* TSB"; $resn_exch{" C3' TSB"}=" C3* TSB";
$resn_exch{" O3' TSB"}=" O3* TSB"; $resn_exch{" C2' TSB"}=" C2* TSB"; $resn_exch{" O2' TSB"}=" O2* TSB";
$resn_exch{" C1' TSB"}=" C1* TSB"; $resn_exch{" OP1 TSP"}=" O1P TSP"; $resn_exch{" OP2 TSP"}=" O2P TSP";
$resn_exch{" OP3 TSP"}=" O3P TSP"; $resn_exch{" O5' TSP"}=" O5* TSP"; $resn_exch{" C5' TSP"}=" C5* TSP";
$resn_exch{" C4' TSP"}=" C4* TSP"; $resn_exch{" O4' TSP"}=" O4* TSP"; $resn_exch{" C3' TSP"}=" C3* TSP";
$resn_exch{" S3' TSP"}=" S3* TSP"; $resn_exch{" C2' TSP"}=" C2* TSP"; $resn_exch{" C1' TSP"}=" C1* TSP";
$resn_exch{" O5'  TT"}=" O5*  TT"; $resn_exch{" C5'  TT"}=" C5*  TT"; $resn_exch{" C4'  TT"}=" C4*  TT";
$resn_exch{" O4'  TT"}=" O4*  TT"; $resn_exch{" C3'  TT"}=" C3*  TT"; $resn_exch{" O3'  TT"}=" O3*  TT";
$resn_exch{" C2'  TT"}=" C2*  TT"; $resn_exch{" C1'  TT"}=" C1*  TT"; $resn_exch{" OP1 TTD"}=" O1P TTD";
$resn_exch{" OP2 TTD"}=" O2P TTD"; $resn_exch{" OP3 TTD"}=" O3P TTD"; $resn_exch{" O5' TTD"}=" O5* TTD";
$resn_exch{" C5' TTD"}=" C5* TTD"; $resn_exch{" O4' TTD"}=" O4* TTD"; $resn_exch{" C2' TTD"}=" C2* TTD";
$resn_exch{" C1' TTD"}=" C1* TTD"; $resn_exch{" C4' TTD"}=" C4* TTD"; $resn_exch{" C3' TTD"}=" C3* TTD";
$resn_exch{" O3' TTD"}=" O3* TTD"; $resn_exch{" C1' TTE"}=" C1* TTE"; $resn_exch{" C2' TTE"}=" C2* TTE";
$resn_exch{" C3' TTE"}=" C3* TTE"; $resn_exch{" O3' TTE"}=" O3* TTE"; $resn_exch{" C4' TTE"}=" C4* TTE";
$resn_exch{" O4' TTE"}=" O4* TTE"; $resn_exch{" C5' TTE"}=" C5* TTE"; $resn_exch{" O5' TTE"}=" O5* TTE";
$resn_exch{" OP1 TTM"}=" O1P TTM"; $resn_exch{" OP2 TTM"}=" O2P TTM"; $resn_exch{" O5' TTM"}=" O5* TTM";
$resn_exch{" C5' TTM"}=" C5* TTM"; $resn_exch{" C4' TTM"}=" C4* TTM"; $resn_exch{" O4' TTM"}=" O4* TTM";
$resn_exch{" C1' TTM"}=" C1* TTM"; $resn_exch{" C2' TTM"}=" C2* TTM"; $resn_exch{" C3' TTM"}=" C3* TTM";
$resn_exch{" O3' TTM"}=" O3* TTM"; $resn_exch{" OP3 TTM"}=" O3P TTM"; $resn_exch{" O5' TTP"}=" O5* TTP";
$resn_exch{" C5' TTP"}=" C5* TTP"; $resn_exch{" C4' TTP"}=" C4* TTP"; $resn_exch{" O4' TTP"}=" O4* TTP";
$resn_exch{" C3' TTP"}=" C3* TTP"; $resn_exch{" O3' TTP"}=" O3* TTP"; $resn_exch{" C2' TTP"}=" C2* TTP";
$resn_exch{" C1' TTP"}=" C1* TTP"; $resn_exch{" C3' TTS"}=" C3* TTS"; $resn_exch{" C2' TTS"}=" C2* TTS";
$resn_exch{" C1' TTS"}=" C1* TTS"; $resn_exch{" C6' TTS"}=" C6* TTS"; $resn_exch{" C5' TTS"}=" C5* TTS";
$resn_exch{" C4' TTS"}=" C4* TTS"; $resn_exch{" CAE TUO"}=" C1  TUO"; $resn_exch{" CAA TUO"}=" C2  TUO";
$resn_exch{" CAB TUO"}=" C3  TUO"; $resn_exch{" CAC TUO"}=" C4  TUO"; $resn_exch{" CAG TUO"}=" C5  TUO";
$resn_exch{" CAF TUO"}=" C6  TUO"; $resn_exch{" CAN TUO"}=" C7  TUO"; $resn_exch{" CAM TUO"}=" C8  TUO";
$resn_exch{" CAL TUO"}=" C9  TUO"; $resn_exch{" CAK TUO"}=" C10 TUO"; $resn_exch{" SAJ TUO"}=" S1  TUO";
$resn_exch{" OAS TUO"}=" O1  TUO"; $resn_exch{" OAD TUO"}=" O2  TUO"; $resn_exch{" NAI TUO"}=" N1  TUO";
$resn_exch{" CAT TUO"}=" C11 TUO"; $resn_exch{" CAU TUO"}=" C12 TUO"; $resn_exch{" CAV TUO"}=" C13 TUO";
$resn_exch{" NAW TUO"}=" N2  TUO"; $resn_exch{" CAO TUO"}=" C14 TUO"; $resn_exch{" CAP TUO"}=" C15 TUO";
$resn_exch{" OAH TUO"}=" O3  TUO"; $resn_exch{" NAQ TUO"}=" N3  TUO"; $resn_exch{" NAR TUO"}=" N4  TUO";
$resn_exch{"C1'A TXD"}="C1*  TXD"; $resn_exch{"O4'A TXD"}="O4*  TXD"; $resn_exch{"C2'A TXD"}="C2*  TXD";
$resn_exch{"O2'A TXD"}="O2*  TXD"; $resn_exch{"C3'A TXD"}="C3*  TXD"; $resn_exch{"O3'A TXD"}="O3*  TXD";
$resn_exch{"C4'A TXD"}="C4*  TXD"; $resn_exch{"C5'A TXD"}="C5*  TXD"; $resn_exch{"O5'A TXD"}="O5*  TXD";
$resn_exch{"O5'A TXP"}="O5*  TXP"; $resn_exch{"C5'A TXP"}="C5*  TXP"; $resn_exch{"C4'A TXP"}="C4*  TXP";
$resn_exch{"C3'A TXP"}="C3*  TXP"; $resn_exch{"O3'A TXP"}="O3*  TXP"; $resn_exch{"C2'A TXP"}="C2*  TXP";
$resn_exch{"O2'A TXP"}="O2*  TXP"; $resn_exch{"C1'A TXP"}="C1*  TXP"; $resn_exch{"O4'A TXP"}="O4*  TXP";
$resn_exch{" O5' TYA"}=" O5* TYA"; $resn_exch{" C5' TYA"}=" C5* TYA"; $resn_exch{" C4' TYA"}=" C4* TYA";
$resn_exch{" O4' TYA"}=" O4* TYA"; $resn_exch{" C3' TYA"}=" C3* TYA"; $resn_exch{" O3' TYA"}=" O3* TYA";
$resn_exch{" C2' TYA"}=" C2* TYA"; $resn_exch{" O2' TYA"}=" O2* TYA"; $resn_exch{" C1' TYA"}=" C1* TYA";
$resn_exch{" O5' TYD"}=" O5* TYD"; $resn_exch{" C5' TYD"}=" C5* TYD"; $resn_exch{" C4' TYD"}=" C4* TYD";
$resn_exch{" O4' TYD"}=" O4* TYD"; $resn_exch{" C3' TYD"}=" C3* TYD"; $resn_exch{" O3' TYD"}=" O3* TYD";
$resn_exch{" C2' TYD"}=" C2* TYD"; $resn_exch{" C1' TYD"}=" C1* TYD"; $resn_exch{" O5' TYU"}=" O5* TYU";
$resn_exch{" C5' TYU"}=" C5* TYU"; $resn_exch{" C4' TYU"}=" C4* TYU"; $resn_exch{" O4' TYU"}=" O4* TYU";
$resn_exch{" C1' TYU"}=" C1* TYU"; $resn_exch{" C2' TYU"}=" C2* TYU"; $resn_exch{" O2' TYU"}=" O2* TYU";
$resn_exch{" C3' TYU"}=" C3* TYU"; $resn_exch{" O3' TYU"}=" O3* TYU"; $resn_exch{" N1' TZD"}=" N1* TZD";
$resn_exch{" C2' TZD"}=" C2* TZD"; $resn_exch{" N3' TZD"}=" N3* TZD"; $resn_exch{" C4' TZD"}=" C4* TZD";
$resn_exch{" N4' TZD"}=" N4* TZD"; $resn_exch{" C5' TZD"}=" C5* TZD"; $resn_exch{" C6' TZD"}=" C6* TZD";
$resn_exch{" C1' TZE"}=" C1* TZE"; $resn_exch{" C2' TZE"}=" C2* TZE"; $resn_exch{" OP3   U"}=" O3P   U";
$resn_exch{" OP1   U"}=" O1P   U"; $resn_exch{" OP2   U"}=" O2P   U"; $resn_exch{" O5'   U"}=" O5*   U";
$resn_exch{" C5'   U"}=" C5*   U"; $resn_exch{" C4'   U"}=" C4*   U"; $resn_exch{" O4'   U"}=" O4*   U";
$resn_exch{" C3'   U"}=" C3*   U"; $resn_exch{" O3'   U"}=" O3*   U"; $resn_exch{" C2'   U"}=" C2*   U";
$resn_exch{" O2'   U"}=" O2*   U"; $resn_exch{" C1'   U"}=" C1*   U"; $resn_exch{" OP3 U25"}=" O3P U25";
$resn_exch{" OP1 U25"}=" O1P U25"; $resn_exch{" OP2 U25"}=" O2P U25"; $resn_exch{" O5' U25"}=" O5* U25";
$resn_exch{" C5' U25"}=" C5* U25"; $resn_exch{" C4' U25"}=" C4* U25"; $resn_exch{" O4' U25"}=" O4* U25";
$resn_exch{" C3' U25"}=" C3* U25"; $resn_exch{" O3' U25"}=" O3* U25"; $resn_exch{" C2' U25"}=" C2* U25";
$resn_exch{" O2' U25"}=" O2* U25"; $resn_exch{" C1' U25"}=" C1* U25"; $resn_exch{" C1B U2G"}="C1*G U2G";
$resn_exch{" C2B U2G"}="C2*G U2G"; $resn_exch{" O2B U2G"}="O2*G U2G"; $resn_exch{" C3B U2G"}="C3*G U2G";
$resn_exch{" O3B U2G"}="O3*G U2G"; $resn_exch{" C4B U2G"}="C4*G U2G"; $resn_exch{" O4B U2G"}="O4*G U2G";
$resn_exch{" C5B U2G"}="C5*G U2G"; $resn_exch{" O5B U2G"}="O5*G U2G"; $resn_exch{" O5D U2G"}="O5*U U2G";
$resn_exch{" C5D U2G"}="C5*U U2G"; $resn_exch{" C4D U2G"}="C4*U U2G"; $resn_exch{" O4D U2G"}="O4*U U2G";
$resn_exch{" C3D U2G"}="C3*U U2G"; $resn_exch{" O3D U2G"}="O3*U U2G"; $resn_exch{" C2D U2G"}="C2*U U2G";
$resn_exch{" O2D U2G"}="O2*U U2G"; $resn_exch{" C1D U2G"}="C1*U U2G"; $resn_exch{" OP3 U2L"}=" O3P U2L";
$resn_exch{" OP1 U2L"}=" O1P U2L"; $resn_exch{" OP2 U2L"}=" O2P U2L"; $resn_exch{" O5' U2L"}=" O5* U2L";
$resn_exch{" C5' U2L"}=" C5* U2L"; $resn_exch{" C4' U2L"}=" C4* U2L"; $resn_exch{" O4' U2L"}=" O4* U2L";
$resn_exch{" C3' U2L"}=" C3* U2L"; $resn_exch{" O3' U2L"}=" O3* U2L"; $resn_exch{" C2' U2L"}=" C2* U2L";
$resn_exch{" O2' U2L"}=" O2* U2L"; $resn_exch{" C1' U2L"}=" C1* U2L"; $resn_exch{" CA' U2L"}=" CA* U2L";
$resn_exch{" CB' U2L"}=" CB* U2L"; $resn_exch{" OC' U2L"}=" OC* U2L"; $resn_exch{" CD' U2L"}=" CD* U2L";
$resn_exch{" OP3 U2N"}=" O3P U2N"; $resn_exch{" OP1 U2N"}=" O1P U2N"; $resn_exch{" OP2 U2N"}=" O2P U2N";
$resn_exch{" O5' U2N"}=" O5* U2N"; $resn_exch{" C5' U2N"}=" C5* U2N"; $resn_exch{" C4' U2N"}=" C4* U2N";
$resn_exch{" O4' U2N"}=" O4* U2N"; $resn_exch{" C1' U2N"}=" C1* U2N"; $resn_exch{" C2' U2N"}=" C2* U2N";
$resn_exch{" N2' U2N"}=" N2* U2N"; $resn_exch{" C3' U2N"}=" C3* U2N"; $resn_exch{" O3' U2N"}=" O3* U2N";
$resn_exch{" O5' U31"}=" O5* U31"; $resn_exch{" C5' U31"}=" C5* U31"; $resn_exch{" C4' U31"}=" C4* U31";
$resn_exch{" O4' U31"}=" O4* U31"; $resn_exch{" C3' U31"}=" C3* U31"; $resn_exch{" O3' U31"}=" O3* U31";
$resn_exch{" C2' U31"}=" C2* U31"; $resn_exch{" O2' U31"}=" O2* U31"; $resn_exch{" C1' U31"}=" C1* U31";
$resn_exch{" CA' U31"}=" CA* U31"; $resn_exch{" CB' U31"}=" CB* U31"; $resn_exch{" ND' U31"}=" ND* U31";
$resn_exch{" C1' U33"}=" C1* U33"; $resn_exch{" C2' U33"}=" C2* U33"; $resn_exch{" C3' U33"}=" C3* U33";
$resn_exch{" C4' U33"}=" C4* U33"; $resn_exch{" O3' U33"}=" O3* U33"; $resn_exch{" O4' U33"}=" O4* U33";
$resn_exch{" C5' U33"}=" C5* U33"; $resn_exch{" O5' U33"}=" O5* U33"; $resn_exch{" C5' U34"}=" C5* U34";
$resn_exch{" C4' U34"}=" C4* U34"; $resn_exch{" O4' U34"}=" O4* U34"; $resn_exch{" C3' U34"}=" C3* U34";
$resn_exch{" O3' U34"}=" O3* U34"; $resn_exch{" C2' U34"}=" C2* U34"; $resn_exch{" O2' U34"}=" O2* U34";
$resn_exch{" C1' U34"}=" C1* U34"; $resn_exch{" O5' U36"}=" O5* U36"; $resn_exch{" C5' U36"}=" C5* U36";
$resn_exch{" C4' U36"}=" C4* U36"; $resn_exch{" O4' U36"}=" O4* U36"; $resn_exch{" C3' U36"}=" C3* U36";
$resn_exch{" O3' U36"}=" O3* U36"; $resn_exch{" C2' U36"}=" C2* U36"; $resn_exch{" O2' U36"}=" O2* U36";
$resn_exch{" C1' U36"}=" C1* U36"; $resn_exch{" CA' U36"}=" CA* U36"; $resn_exch{" CB' U36"}=" CB* U36";
$resn_exch{" CD' U36"}=" CD* U36"; $resn_exch{" OC' U36"}=" OC* U36"; $resn_exch{" O5' U37"}=" O5* U37";
$resn_exch{" C5' U37"}=" C5* U37"; $resn_exch{" C4' U37"}=" C4* U37"; $resn_exch{" O4' U37"}=" O4* U37";
$resn_exch{" C3' U37"}=" C3* U37"; $resn_exch{" O3' U37"}=" O3* U37"; $resn_exch{" C2' U37"}=" C2* U37";
$resn_exch{" O2' U37"}=" O2* U37"; $resn_exch{" C1' U37"}=" C1* U37"; $resn_exch{" C1' U3H"}=" C1* U3H";
$resn_exch{" C2' U3H"}=" C2* U3H"; $resn_exch{" O2' U3H"}=" O2* U3H"; $resn_exch{" C3' U3H"}=" C3* U3H";
$resn_exch{" C4' U3H"}=" C4* U3H"; $resn_exch{" O4' U3H"}=" O4* U3H"; $resn_exch{" C5' U3H"}=" C5* U3H";
$resn_exch{" O5' U3H"}=" O5* U3H"; $resn_exch{" O5' U3P"}=" O5* U3P"; $resn_exch{" C5' U3P"}=" C5* U3P";
$resn_exch{" C4' U3P"}=" C4* U3P"; $resn_exch{" O4' U3P"}=" O4* U3P"; $resn_exch{" C3' U3P"}=" C3* U3P";
$resn_exch{" O3' U3P"}=" O3* U3P"; $resn_exch{" C2' U3P"}=" C2* U3P"; $resn_exch{" O2' U3P"}=" O2* U3P";
$resn_exch{" C1' U3P"}=" C1* U3P"; $resn_exch{" C1' U5P"}=" C1* U5P"; $resn_exch{" C2' U5P"}=" C2* U5P";
$resn_exch{" O2' U5P"}=" O2* U5P"; $resn_exch{" C3' U5P"}=" C3* U5P"; $resn_exch{" C4' U5P"}=" C4* U5P";
$resn_exch{" O3' U5P"}=" O3* U5P"; $resn_exch{" O4' U5P"}=" O4* U5P"; $resn_exch{" C5' U5P"}=" C5* U5P";
$resn_exch{" O5' U5P"}=" O5* U5P"; $resn_exch{" C1' U8U"}=" C1* U8U"; $resn_exch{" O2' U8U"}=" O2* U8U";
$resn_exch{" C2' U8U"}=" C2* U8U"; $resn_exch{" O3' U8U"}=" O3* U8U"; $resn_exch{" C3' U8U"}=" C3* U8U";
$resn_exch{" C4' U8U"}=" C4* U8U"; $resn_exch{" O4' U8U"}=" O4* U8U"; $resn_exch{" C5' U8U"}=" C5* U8U";
$resn_exch{" O5' U8U"}=" O5* U8U"; $resn_exch{" OP1 U8U"}=" O1P U8U"; $resn_exch{" OP2 U8U"}=" O2P U8U";
$resn_exch{" OP3 U8U"}=" O3P U8U"; $resn_exch{" O5' UA2"}=" O5* UA2"; $resn_exch{" C5' UA2"}=" C5* UA2";
$resn_exch{" C4' UA2"}=" C4* UA2"; $resn_exch{" N4' UA2"}=" N4* UA2"; $resn_exch{" C3' UA2"}=" C3* UA2";
$resn_exch{" O3' UA2"}=" O3* UA2"; $resn_exch{" C2' UA2"}=" C2* UA2"; $resn_exch{" O2' UA2"}=" O2* UA2";
$resn_exch{" C1' UA2"}=" C1* UA2"; $resn_exch{" O5' UA3"}=" O5* UA3"; $resn_exch{" C5' UA3"}=" C5* UA3";
$resn_exch{" C4' UA3"}=" C4* UA3"; $resn_exch{" O4' UA3"}=" O4* UA3"; $resn_exch{" C3' UA3"}=" C3* UA3";
$resn_exch{" O3' UA3"}=" O3* UA3"; $resn_exch{" C2' UA3"}=" C2* UA3"; $resn_exch{" O2' UA3"}=" O2* UA3";
$resn_exch{" C1' UA3"}=" C1* UA3"; $resn_exch{" C1B UAG"}=" C1* UAG"; $resn_exch{" C2B UAG"}=" C2* UAG";
$resn_exch{" O2' UAG"}=" O2* UAG"; $resn_exch{" C3B UAG"}=" C3* UAG"; $resn_exch{" O3B UAG"}=" O3* UAG";
$resn_exch{" C4B UAG"}=" C4* UAG"; $resn_exch{" O4B UAG"}=" O4* UAG"; $resn_exch{" C5B UAG"}=" C5* UAG";
$resn_exch{" O5B UAG"}=" O5* UAG"; $resn_exch{" OP3 UAR"}=" O3P UAR"; $resn_exch{" OP1 UAR"}=" O1P UAR";
$resn_exch{" OP2 UAR"}=" O2P UAR"; $resn_exch{" O5' UAR"}=" O5* UAR"; $resn_exch{" C5' UAR"}=" C5* UAR";
$resn_exch{" C4' UAR"}=" C4* UAR"; $resn_exch{" O4' UAR"}=" O4* UAR"; $resn_exch{" C3' UAR"}=" C3* UAR";
$resn_exch{" O3' UAR"}=" O3* UAR"; $resn_exch{" C2' UAR"}=" C2* UAR"; $resn_exch{" O2' UAR"}=" O2* UAR";
$resn_exch{" C1' UAR"}=" C1* UAR"; $resn_exch{" O5' UBB"}=" O5* UBB"; $resn_exch{" C5' UBB"}=" C5* UBB";
$resn_exch{" C4' UBB"}=" C4* UBB"; $resn_exch{" O4' UBB"}=" O4* UBB"; $resn_exch{" C1' UBB"}=" C1* UBB";
$resn_exch{" C2' UBB"}=" C2* UBB"; $resn_exch{" C3' UBB"}=" C3* UBB"; $resn_exch{" C1' UC5"}=" C1* UC5";
$resn_exch{" C2' UC5"}=" C2* UC5"; $resn_exch{" C3' UC5"}=" C3* UC5"; $resn_exch{" O3' UC5"}=" O3* UC5";
$resn_exch{" O4' UC5"}=" O4* UC5"; $resn_exch{" C4' UC5"}=" C4* UC5"; $resn_exch{" C5' UC5"}=" C5* UC5";
$resn_exch{" O5' UC5"}=" O5* UC5"; $resn_exch{" O5' UCL"}=" O5* UCL"; $resn_exch{" C5' UCL"}=" C5* UCL";
$resn_exch{" C4' UCL"}=" C4* UCL"; $resn_exch{" O4' UCL"}=" O4* UCL"; $resn_exch{" C3' UCL"}=" C3* UCL";
$resn_exch{" O3' UCL"}=" O3* UCL"; $resn_exch{" C2' UCL"}=" C2* UCL"; $resn_exch{" C1' UCL"}=" C1* UCL";
$resn_exch{" C1' UCP"}=" C1* UCP"; $resn_exch{" C2' UCP"}=" C2* UCP"; $resn_exch{" C3' UCP"}=" C3* UCP";
$resn_exch{" O3' UCP"}=" O3* UCP"; $resn_exch{" C4' UCP"}=" C4* UCP"; $resn_exch{" O4' UCP"}=" O4* UCP";
$resn_exch{" C5' UCP"}=" C5* UCP"; $resn_exch{" O5' UCP"}=" O5* UCP"; $resn_exch{" C1B UD1"}=" C1* UD1";
$resn_exch{" C2B UD1"}=" C2* UD1"; $resn_exch{" O2' UD1"}=" O2* UD1"; $resn_exch{" C3B UD1"}=" C3* UD1";
$resn_exch{" C4B UD1"}=" C4* UD1"; $resn_exch{" O4B UD1"}=" O4* UD1"; $resn_exch{" O3B UD1"}=" O3* UD1";
$resn_exch{" C5B UD1"}=" C5* UD1"; $resn_exch{" O5B UD1"}=" O5* UD1"; $resn_exch{" C1B UD2"}=" C1* UD2";
$resn_exch{" C2B UD2"}=" C2* UD2"; $resn_exch{" O2' UD2"}=" O2* UD2"; $resn_exch{" C3B UD2"}=" C3* UD2";
$resn_exch{" C4B UD2"}=" C4* UD2"; $resn_exch{" O4B UD2"}=" O4* UD2"; $resn_exch{" O3B UD2"}=" O3* UD2";
$resn_exch{" C5B UD2"}=" C5* UD2"; $resn_exch{" O5B UD2"}=" O5* UD2"; $resn_exch{" OP1 UD5"}=" O1P UD5";
$resn_exch{" OP2 UD5"}=" O2P UD5"; $resn_exch{" O5' UD5"}=" O5* UD5"; $resn_exch{" C5' UD5"}=" C5* UD5";
$resn_exch{" C4' UD5"}=" C4* UD5"; $resn_exch{" O4' UD5"}=" O4* UD5"; $resn_exch{" C3' UD5"}=" C3* UD5";
$resn_exch{" O3' UD5"}=" O3* UD5"; $resn_exch{" C2' UD5"}=" C2* UD5"; $resn_exch{" O2' UD5"}=" O2* UD5";
$resn_exch{" C1' UD5"}=" C1* UD5"; $resn_exch{" C1B UDH"}=" C1* UDH"; $resn_exch{" C2B UDH"}=" C2* UDH";
$resn_exch{" O2' UDH"}=" O2* UDH"; $resn_exch{" C3B UDH"}=" C3* UDH"; $resn_exch{" C4B UDH"}=" C4* UDH";
$resn_exch{" O4' UDH"}=" O4* UDH"; $resn_exch{" O3' UDH"}=" O3* UDH"; $resn_exch{" C5B UDH"}=" C5* UDH";
$resn_exch{" O5' UDH"}=" O5* UDH"; $resn_exch{" C1D UDM"}=" C1* UDM"; $resn_exch{" C2D UDM"}=" C2* UDM";
$resn_exch{" O2' UDM"}=" O2* UDM"; $resn_exch{" C3D UDM"}=" C3* UDM"; $resn_exch{" C4D UDM"}=" C4* UDM";
$resn_exch{" O4D UDM"}=" O4* UDM"; $resn_exch{" O3D UDM"}=" O3* UDM"; $resn_exch{" C5D UDM"}=" C5* UDM";
$resn_exch{" O5D UDM"}=" O5* UDM"; $resn_exch{" C1' UDP"}=" C1* UDP"; $resn_exch{" C2' UDP"}=" C2* UDP";
$resn_exch{" O2' UDP"}=" O2* UDP"; $resn_exch{" C3' UDP"}=" C3* UDP"; $resn_exch{" C4' UDP"}=" C4* UDP";
$resn_exch{" O4' UDP"}=" O4* UDP"; $resn_exch{" O3' UDP"}=" O3* UDP"; $resn_exch{" C5' UDP"}=" C5* UDP";
$resn_exch{" O5' UDP"}=" O5* UDP"; $resn_exch{" O5D UDX"}=" O5* UDX"; $resn_exch{" C5D UDX"}=" C5* UDX";
$resn_exch{" C4D UDX"}=" C4* UDX"; $resn_exch{" O4D UDX"}=" O4* UDX"; $resn_exch{" C3D UDX"}=" C3* UDX";
$resn_exch{" O3D UDX"}=" O3* UDX"; $resn_exch{" C2D UDX"}=" C2* UDX"; $resn_exch{" O2D UDX"}=" O2* UDX";
$resn_exch{" C1D UDX"}=" C1* UDX"; $resn_exch{" C1D UFG"}=" C1* UFG"; $resn_exch{" C2D UFG"}=" C2* UFG";
$resn_exch{" O2D UFG"}=" O2* UFG"; $resn_exch{" C3D UFG"}=" C3* UFG"; $resn_exch{" C4D UFG"}=" C4* UFG";
$resn_exch{" O4' UFG"}=" O4* UFG"; $resn_exch{" O3D UFG"}=" O3* UFG"; $resn_exch{" C5D UFG"}=" C5* UFG";
$resn_exch{" O5D UFG"}=" O5* UFG"; $resn_exch{" C1D UFM"}=" C1* UFM"; $resn_exch{" C2D UFM"}=" C2* UFM";
$resn_exch{" O2D UFM"}=" O2* UFM"; $resn_exch{" C3D UFM"}=" C3* UFM"; $resn_exch{" C4D UFM"}=" C4* UFM";
$resn_exch{" O4D UFM"}=" O4* UFM"; $resn_exch{" O3D UFM"}=" O3* UFM"; $resn_exch{" C5D UFM"}=" C5* UFM";
$resn_exch{" O5D UFM"}=" O5* UFM"; $resn_exch{" C1' UFP"}=" C1* UFP"; $resn_exch{" C2' UFP"}=" C2* UFP";
$resn_exch{" C3' UFP"}=" C3* UFP"; $resn_exch{" C4' UFP"}=" C4* UFP"; $resn_exch{" O3' UFP"}=" O3* UFP";
$resn_exch{" O4' UFP"}=" O4* UFP"; $resn_exch{" C5' UFP"}=" C5* UFP"; $resn_exch{" O5' UFP"}=" O5* UFP";
$resn_exch{" O5' UFR"}=" O5* UFR"; $resn_exch{" C5' UFR"}=" C5* UFR"; $resn_exch{" C4' UFR"}=" C4* UFR";
$resn_exch{" O4' UFR"}=" O4* UFR"; $resn_exch{" C3' UFR"}=" C3* UFR"; $resn_exch{" O3' UFR"}=" O3* UFR";
$resn_exch{" C2' UFR"}=" C2* UFR"; $resn_exch{" C1' UFR"}=" C1* UFR"; $resn_exch{" O'P UGA"}="O6'1 UGA";
$resn_exch{" O'Q UGA"}="O6'2 UGA"; $resn_exch{" O5D UGA"}=" O5* UGA"; $resn_exch{" C5D UGA"}=" C5* UGA";
$resn_exch{" C4D UGA"}=" C4* UGA"; $resn_exch{" O4D UGA"}=" O4* UGA"; $resn_exch{" C3D UGA"}=" C3* UGA";
$resn_exch{" O3D UGA"}=" O3* UGA"; $resn_exch{" C2D UGA"}=" C2* UGA"; $resn_exch{" O2D UGA"}=" O2* UGA";
$resn_exch{" C1D UGA"}=" C1* UGA"; $resn_exch{" C1' UM3"}=" C1* UM3"; $resn_exch{" C2' UM3"}=" C2* UM3";
$resn_exch{" C3' UM3"}=" C3* UM3"; $resn_exch{" C4' UM3"}=" C4* UM3"; $resn_exch{" O3' UM3"}=" O3* UM3";
$resn_exch{" O4' UM3"}=" O4* UM3"; $resn_exch{" C5' UM3"}=" C5* UM3"; $resn_exch{" O5' UM3"}=" O5* UM3";
$resn_exch{" C1B UMA"}=" C1* UMA"; $resn_exch{" C2B UMA"}=" C2* UMA"; $resn_exch{" O2' UMA"}=" O2* UMA";
$resn_exch{" C3B UMA"}=" C3* UMA"; $resn_exch{" C4B UMA"}=" C4* UMA"; $resn_exch{" O4B UMA"}=" O4* UMA";
$resn_exch{" O3B UMA"}=" O3* UMA"; $resn_exch{" C5B UMA"}=" C5* UMA"; $resn_exch{" O5B UMA"}=" O5* UMA";
$resn_exch{" O5' UMF"}=" O5* UMF"; $resn_exch{" C5' UMF"}=" C5* UMF"; $resn_exch{" C4' UMF"}=" C4* UMF";
$resn_exch{" O4' UMF"}=" O4* UMF"; $resn_exch{" C3' UMF"}=" C3* UMF"; $resn_exch{" O3' UMF"}=" O3* UMF";
$resn_exch{" C2' UMF"}=" C2* UMF"; $resn_exch{" F2' UMF"}=" F2* UMF"; $resn_exch{" C1' UMF"}=" C1* UMF";
$resn_exch{" C1' UMP"}=" C1* UMP"; $resn_exch{" C2' UMP"}=" C2* UMP"; $resn_exch{" C3' UMP"}=" C3* UMP";
$resn_exch{" C4' UMP"}=" C4* UMP"; $resn_exch{" O3' UMP"}=" O3* UMP"; $resn_exch{" O4' UMP"}=" O4* UMP";
$resn_exch{" C5' UMP"}=" C5* UMP"; $resn_exch{" O5' UMP"}=" O5* UMP"; $resn_exch{" OP1 UMP"}=" O1P UMP";
$resn_exch{" OP2 UMP"}=" O2P UMP"; $resn_exch{" OP3 UMP"}=" O3P UMP"; $resn_exch{" OP3 UMS"}=" O3P UMS";
$resn_exch{" OP1 UMS"}=" O1P UMS"; $resn_exch{" OP2 UMS"}=" O2P UMS"; $resn_exch{" C4' UN4"}=" C4* UN4";
$resn_exch{" C3' UN4"}=" C3* UN4"; $resn_exch{" C2' UN4"}=" C2* UN4"; $resn_exch{" C1' UP1"}=" C1* UP1";
$resn_exch{" C2' UP1"}=" C2* UP1"; $resn_exch{" O2' UP1"}=" O2* UP1"; $resn_exch{" C3' UP1"}=" C3* UP1";
$resn_exch{" C4' UP1"}=" C4* UP1"; $resn_exch{" O4' UP1"}=" O4* UP1"; $resn_exch{" O3' UP1"}=" O3* UP1";
$resn_exch{" C5' UP1"}=" C5* UP1"; $resn_exch{" O5' UP1"}=" O5* UP1"; $resn_exch{" N9A UP5"}=" AN9 UP5";
$resn_exch{" C8A UP5"}=" AC8 UP5"; $resn_exch{" N7A UP5"}=" AN7 UP5"; $resn_exch{" C5A UP5"}=" AC5 UP5";
$resn_exch{" C6A UP5"}=" AC6 UP5"; $resn_exch{" N6A UP5"}=" AN6 UP5"; $resn_exch{" N1A UP5"}=" AN1 UP5";
$resn_exch{" C2A UP5"}=" AC2 UP5"; $resn_exch{" N3A UP5"}=" AN3 UP5"; $resn_exch{" C4A UP5"}=" AC4 UP5";
$resn_exch{" O5F UP5"}="AO5* UP5"; $resn_exch{" C5F UP5"}="AC5* UP5"; $resn_exch{" C4F UP5"}="AC4* UP5";
$resn_exch{" O4F UP5"}="AO4* UP5"; $resn_exch{" C3F UP5"}="AC3* UP5"; $resn_exch{" O3F UP5"}="AO3* UP5";
$resn_exch{" C2F UP5"}="AC2* UP5"; $resn_exch{" O2F UP5"}="AO2* UP5"; $resn_exch{" C1F UP5"}="AC1* UP5";
$resn_exch{" O5J UP5"}="UO5* UP5"; $resn_exch{" C5J UP5"}="UC5* UP5"; $resn_exch{" C4J UP5"}="UC4* UP5";
$resn_exch{" O4J UP5"}="UO4* UP5"; $resn_exch{" C3J UP5"}="UC3* UP5"; $resn_exch{" O3J UP5"}="UO3* UP5";
$resn_exch{" C2J UP5"}="UC2* UP5"; $resn_exch{" O2J UP5"}="UO2* UP5"; $resn_exch{" C1J UP5"}="UC1* UP5";
$resn_exch{" N1U UP5"}=" UN1 UP5"; $resn_exch{" C2U UP5"}=" UC2 UP5"; $resn_exch{" O2X UP5"}="UO2' UP5";
$resn_exch{" N3U UP5"}=" UN3 UP5"; $resn_exch{" C4U UP5"}=" UC4 UP5"; $resn_exch{" O4U UP5"}=" UO4 UP5";
$resn_exch{" C5U UP5"}=" UC5 UP5"; $resn_exch{" C6U UP5"}=" UC6 UP5"; $resn_exch{" C1' UP6"}=" C1* UP6";
$resn_exch{" C2' UP6"}=" C2* UP6"; $resn_exch{" C3' UP6"}=" C3* UP6"; $resn_exch{" C4' UP6"}=" C4* UP6";
$resn_exch{" O2' UP6"}=" O2* UP6"; $resn_exch{" O3' UP6"}=" O3* UP6"; $resn_exch{" O4' UP6"}=" O4* UP6";
$resn_exch{" C5' UP6"}=" C5* UP6"; $resn_exch{" O5' UP6"}=" O5* UP6"; $resn_exch{" C5D UPA"}="C5*U UPA";
$resn_exch{" O5D UPA"}="O5*U UPA"; $resn_exch{" C4D UPA"}="C4*U UPA"; $resn_exch{" O4D UPA"}="O4*U UPA";
$resn_exch{" C3D UPA"}="C3*U UPA"; $resn_exch{" O3D UPA"}="O3*U UPA"; $resn_exch{" C2D UPA"}="C2*U UPA";
$resn_exch{" O2D UPA"}="O2*U UPA"; $resn_exch{" C1D UPA"}="C1*U UPA"; $resn_exch{" O5B UPA"}="O5*A UPA";
$resn_exch{" C5B UPA"}="C5*A UPA"; $resn_exch{" C4B UPA"}="C4*A UPA"; $resn_exch{" O4B UPA"}="O4*A UPA";
$resn_exch{" C3B UPA"}="C3*A UPA"; $resn_exch{" O3B UPA"}="O3*A UPA"; $resn_exch{" C2B UPA"}="C2*A UPA";
$resn_exch{" O2B UPA"}="O2*A UPA"; $resn_exch{" C1B UPA"}="C1*A UPA"; $resn_exch{" C1D UPF"}=" C1* UPF";
$resn_exch{" C2D UPF"}=" C2* UPF"; $resn_exch{" O2' UPF"}=" O2* UPF"; $resn_exch{" C3D UPF"}=" C3* UPF";
$resn_exch{" C4D UPF"}=" C4* UPF"; $resn_exch{" O4D UPF"}=" O4* UPF"; $resn_exch{" O3D UPF"}=" O3* UPF";
$resn_exch{" C5D UPF"}=" C5* UPF"; $resn_exch{" O5D UPF"}=" O5* UPF"; $resn_exch{" C1C UPG"}=" C1* UPG";
$resn_exch{" C2C UPG"}=" C2* UPG"; $resn_exch{" O2C UPG"}=" O2* UPG"; $resn_exch{" C3C UPG"}=" C3* UPG";
$resn_exch{" C4C UPG"}=" C4* UPG"; $resn_exch{" O4C UPG"}=" O4* UPG"; $resn_exch{" O3C UPG"}=" O3* UPG";
$resn_exch{" C5C UPG"}=" C5* UPG"; $resn_exch{" O5C UPG"}=" O5* UPG"; $resn_exch{" C1' UPM"}=" C1* UPM";
$resn_exch{" C2' UPM"}=" C2* UPM"; $resn_exch{" O2' UPM"}=" O2* UPM"; $resn_exch{" C3' UPM"}=" C3* UPM";
$resn_exch{" O3' UPM"}=" O3* UPM"; $resn_exch{" C4' UPM"}=" C4* UPM"; $resn_exch{" O4' UPM"}=" O4* UPM";
$resn_exch{" C5' UPM"}=" C5* UPM"; $resn_exch{" O5' UPM"}=" O5* UPM"; $resn_exch{" C1B UPP"}=" C1* UPP";
$resn_exch{" C2B UPP"}=" C2* UPP"; $resn_exch{" O2' UPP"}=" O2* UPP"; $resn_exch{" C3B UPP"}=" C3* UPP";
$resn_exch{" C4B UPP"}=" C4* UPP"; $resn_exch{" O4' UPP"}=" O4* UPP"; $resn_exch{" O3' UPP"}=" O3* UPP";
$resn_exch{" C5B UPP"}=" C5* UPP"; $resn_exch{" O5' UPP"}=" O5* UPP"; $resn_exch{" C5' UPR"}=" C5* UPR";
$resn_exch{" O5' UPR"}=" O5* UPR"; $resn_exch{" C4' UPR"}=" C4* UPR"; $resn_exch{" O4' UPR"}=" O4* UPR";
$resn_exch{" C3' UPR"}=" C3* UPR"; $resn_exch{" O3' UPR"}=" O3* UPR"; $resn_exch{" C2' UPR"}=" C2* UPR";
$resn_exch{" C1' UPR"}=" C1* UPR"; $resn_exch{"C1'A UPU"}="C1*A UPU"; $resn_exch{"O4'A UPU"}="O4*A UPU";
$resn_exch{"C2'A UPU"}="C2*A UPU"; $resn_exch{"O2'A UPU"}="O2*A UPU"; $resn_exch{"C3'A UPU"}="C3*A UPU";
$resn_exch{"O3'A UPU"}="O3*A UPU"; $resn_exch{"C4'A UPU"}="C4*A UPU"; $resn_exch{"C5'A UPU"}="C5*A UPU";
$resn_exch{"O5'A UPU"}="O5*A UPU"; $resn_exch{"O3'B UPU"}="O3*B UPU"; $resn_exch{"C3'B UPU"}="C3*B UPU";
$resn_exch{"C4'B UPU"}="C4*B UPU"; $resn_exch{"C5'B UPU"}="C5*B UPU"; $resn_exch{"O5'B UPU"}="O5*B UPU";
$resn_exch{"O4'B UPU"}="O4*B UPU"; $resn_exch{"C2'B UPU"}="C2*B UPU"; $resn_exch{"O2'B UPU"}="O2*B UPU";
$resn_exch{"C1'B UPU"}="C1*B UPU"; $resn_exch{" OP1 UR3"}=" O1P UR3"; $resn_exch{" OP2 UR3"}=" O2P UR3";
$resn_exch{" O5' UR3"}=" O5* UR3"; $resn_exch{" C5' UR3"}=" C5* UR3"; $resn_exch{" C4' UR3"}=" C4* UR3";
$resn_exch{" O4' UR3"}=" O4* UR3"; $resn_exch{" C1' UR3"}=" C1* UR3"; $resn_exch{" C2' UR3"}=" C2* UR3";
$resn_exch{" O2' UR3"}=" O2* UR3"; $resn_exch{" C3' UR3"}=" C3* UR3"; $resn_exch{" O3' UR3"}=" O3* UR3";
$resn_exch{" OP3 UR3"}=" O3P UR3"; $resn_exch{" O5' URD"}=" O5* URD"; $resn_exch{" C5' URD"}=" C5* URD";
$resn_exch{" C4' URD"}=" C4* URD"; $resn_exch{" O4' URD"}=" O4* URD"; $resn_exch{" C1' URD"}=" C1* URD";
$resn_exch{" C2' URD"}=" C2* URD"; $resn_exch{" O2' URD"}=" O2* URD"; $resn_exch{" C3' URD"}=" C3* URD";
$resn_exch{" O3' URD"}=" O3* URD"; $resn_exch{" C1' URI"}=" C1* URI"; $resn_exch{" C2' URI"}=" C2* URI";
$resn_exch{" C3' URI"}=" C3* URI"; $resn_exch{" C4' URI"}=" C4* URI"; $resn_exch{" O2' URI"}=" O2* URI";
$resn_exch{" O3' URI"}=" O3* URI"; $resn_exch{" O4' URI"}=" O4* URI"; $resn_exch{" C5' URI"}=" C5* URI";
$resn_exch{" O5' URI"}=" O5* URI"; $resn_exch{" O5' US1"}=" O5* US1"; $resn_exch{" C5' US1"}=" C5* US1";
$resn_exch{" C4' US1"}=" C4* US1"; $resn_exch{" O4' US1"}=" O4* US1"; $resn_exch{" C3' US1"}=" C3* US1";
$resn_exch{" C2' US1"}=" C2* US1"; $resn_exch{" C1' US1"}=" C1* US1"; $resn_exch{" C1D USQ"}=" C1* USQ";
$resn_exch{" C2D USQ"}=" C2* USQ"; $resn_exch{" O2D USQ"}=" O2* USQ"; $resn_exch{" C3D USQ"}=" C3* USQ";
$resn_exch{" C4D USQ"}=" C4* USQ"; $resn_exch{" O4D USQ"}=" O4* USQ"; $resn_exch{" O3D USQ"}=" O3* USQ";
$resn_exch{" C5D USQ"}=" C5* USQ"; $resn_exch{" O5D USQ"}=" O5* USQ"; $resn_exch{" O5' UTP"}=" O5* UTP";
$resn_exch{" C5' UTP"}=" C5* UTP"; $resn_exch{" C4' UTP"}=" C4* UTP"; $resn_exch{" O4' UTP"}=" O4* UTP";
$resn_exch{" C1' UTP"}=" C1* UTP"; $resn_exch{" C2' UTP"}=" C2* UTP"; $resn_exch{" O2' UTP"}=" O2* UTP";
$resn_exch{" C3' UTP"}=" C3* UTP"; $resn_exch{" O3' UTP"}=" O3* UTP"; $resn_exch{" C1' UVC"}=" C1* UVC";
$resn_exch{" C2' UVC"}=" C2* UVC"; $resn_exch{" O2' UVC"}=" O2* UVC"; $resn_exch{" C3' UVC"}=" C3* UVC";
$resn_exch{" C4' UVC"}=" C4* UVC"; $resn_exch{" O3' UVC"}=" O3* UVC"; $resn_exch{" O4' UVC"}=" O4* UVC";
$resn_exch{" C5' UVC"}=" C5* UVC"; $resn_exch{" O5' UVC"}=" O5* UVC"; $resn_exch{" N5' VAA"}=" N5* VAA";
$resn_exch{" C5' VAA"}=" C5* VAA"; $resn_exch{" C4' VAA"}=" C4* VAA"; $resn_exch{" O4' VAA"}=" O4* VAA";
$resn_exch{" C1' VAA"}=" C1* VAA"; $resn_exch{" C2' VAA"}=" C2* VAA"; $resn_exch{" O2' VAA"}=" O2* VAA";
$resn_exch{" C3' VAA"}=" C3* VAA"; $resn_exch{" O3' VAA"}=" O3* VAA"; $resn_exch{" C1B VAN"}=" C1\" VAN";
$resn_exch{" C2B VAN"}=" C2\" VAN"; $resn_exch{" C3B VAN"}=" C3\" VAN"; $resn_exch{" N3' VAN"}=" N3\" VAN";
$resn_exch{" C4B VAN"}=" C4\" VAN"; $resn_exch{" O4B VAN"}=" O4\" VAN"; $resn_exch{" C5B VAN"}=" C5\" VAN";
$resn_exch{" O5B VAN"}=" O5\" VAN"; $resn_exch{" C1B VAX"}=" C1\" VAX"; $resn_exch{" C2B VAX"}=" C2\" VAX";
$resn_exch{" C3B VAX"}=" C3\" VAX"; $resn_exch{" N3' VAX"}=" N3\" VAX"; $resn_exch{" C4B VAX"}=" C4\" VAX";
$resn_exch{" O4B VAX"}=" O4\" VAX"; $resn_exch{" C5B VAX"}=" C5\" VAX"; $resn_exch{" O5B VAX"}=" O5\" VAX";
$resn_exch{" N1' VDM"}=" N1* VDM"; $resn_exch{" C1' VDM"}=" C1* VDM"; $resn_exch{" C2' VDM"}=" C2* VDM";
$resn_exch{" O2' VDM"}=" O2* VDM"; $resn_exch{" C6' VDM"}=" C6* VDM"; $resn_exch{" C5' VDM"}=" C5* VDM";
$resn_exch{" C7' VDM"}=" C7* VDM"; $resn_exch{" O7' VDM"}=" O7* VDM"; $resn_exch{" C4' VDM"}=" C4* VDM";
$resn_exch{" O4' VDM"}=" O4* VDM"; $resn_exch{" C3' VDM"}=" C3* VDM"; $resn_exch{" O3' VDM"}=" O3* VDM";
$resn_exch{" O5' VMS"}=" O5* VMS"; $resn_exch{" C5' VMS"}=" C5* VMS"; $resn_exch{" C4' VMS"}=" C4* VMS";
$resn_exch{" O4' VMS"}=" O4* VMS"; $resn_exch{" C3' VMS"}=" C3* VMS"; $resn_exch{" O3' VMS"}=" O3* VMS";
$resn_exch{" C2' VMS"}=" C2* VMS"; $resn_exch{" O2' VMS"}=" O2* VMS"; $resn_exch{" C1' VMS"}=" C1* VMS";
$resn_exch{" H1   VX"}="1H    VX"; $resn_exch{" C5' WWF"}=" C5* WWF"; $resn_exch{" C6' WWF"}=" C6* WWF";
$resn_exch{" N1' WWF"}=" N1* WWF"; $resn_exch{" C2' WWF"}=" C2* WWF"; $resn_exch{" N3' WWF"}=" N3* WWF";
$resn_exch{" C4' WWF"}=" C4* WWF"; $resn_exch{" N4' WWF"}=" N4* WWF"; $resn_exch{" OP1   X"}=" O1P   X";
$resn_exch{" OP2   X"}=" O2P   X"; $resn_exch{" OP3   X"}=" O3P   X"; $resn_exch{" O5'   X"}=" O5*   X";
$resn_exch{" C5'   X"}=" C5*   X"; $resn_exch{" C4'   X"}=" C4*   X"; $resn_exch{" O4'   X"}=" O4*   X";
$resn_exch{" C3'   X"}=" C3*   X"; $resn_exch{" O3'   X"}=" O3*   X"; $resn_exch{" C2'   X"}=" C2*   X";
$resn_exch{" C1'   X"}=" C1*   X"; $resn_exch{" OP3 X4A"}=" O3P X4A"; $resn_exch{" OP1 X4A"}=" O1P X4A";
$resn_exch{" OP2 X4A"}=" O2P X4A"; $resn_exch{" O5' X4A"}=" O5* X4A"; $resn_exch{" C5' X4A"}=" C5* X4A";
$resn_exch{" C4' X4A"}=" C4* X4A"; $resn_exch{" O4' X4A"}=" O4* X4A"; $resn_exch{" C1' X4A"}=" C1* X4A";
$resn_exch{" C3' X4A"}=" C3* X4A"; $resn_exch{" C2' X4A"}=" C2* X4A"; $resn_exch{" O3' X4A"}=" O3* X4A";
$resn_exch{" OP3 XAD"}=" O3P XAD"; $resn_exch{" OP1 XAD"}=" O1P XAD"; $resn_exch{" OP2 XAD"}=" O2P XAD";
$resn_exch{" OP1 XAE"}=" O1P XAE"; $resn_exch{" OP2 XAE"}=" O2P XAE"; $resn_exch{" OP3 XAE"}=" O3P XAE";
$resn_exch{" OP1 XAL"}=" O1P XAL"; $resn_exch{" OP2 XAL"}=" O2P XAL"; $resn_exch{" O5' XAL"}=" O5* XAL";
$resn_exch{" C5' XAL"}=" C5* XAL"; $resn_exch{" C4' XAL"}=" C4* XAL"; $resn_exch{" C6' XAL"}=" C6* XAL";
$resn_exch{" C7' XAL"}=" C7* XAL"; $resn_exch{" C3' XAL"}=" C3* XAL"; $resn_exch{" O3' XAL"}=" O3* XAL";
$resn_exch{" C2' XAL"}=" C2* XAL"; $resn_exch{" C1' XAL"}=" C1* XAL"; $resn_exch{" OP3 XAL"}=" O3P XAL";
$resn_exch{" O5' XAR"}=" O5* XAR"; $resn_exch{" C5' XAR"}=" C5* XAR"; $resn_exch{" C4' XAR"}=" C4* XAR";
$resn_exch{" C3' XAR"}=" C3* XAR"; $resn_exch{" O3' XAR"}=" O3* XAR"; $resn_exch{" C2' XAR"}=" C2* XAR";
$resn_exch{" C1' XAR"}=" C1* XAR"; $resn_exch{" C7' XAR"}=" C7* XAR"; $resn_exch{" C6' XAR"}=" C6* XAR";
$resn_exch{" OP1 XCL"}=" O1P XCL"; $resn_exch{" OP2 XCL"}=" O2P XCL"; $resn_exch{" O5' XCL"}=" O5* XCL";
$resn_exch{" C5' XCL"}=" C5* XCL"; $resn_exch{" C4' XCL"}=" C4* XCL"; $resn_exch{" C6' XCL"}=" C6* XCL";
$resn_exch{" C7' XCL"}=" C7* XCL"; $resn_exch{" C3' XCL"}=" C3* XCL"; $resn_exch{" O3' XCL"}=" O3* XCL";
$resn_exch{" C2' XCL"}=" C2* XCL"; $resn_exch{" C1' XCL"}=" C1* XCL"; $resn_exch{" OP3 XCL"}=" O3P XCL";
$resn_exch{" OP2 XCS"}=" O2P XCS"; $resn_exch{" OP1 XCS"}=" O1P XCS"; $resn_exch{" OP3 XCS"}=" O3P XCS";
$resn_exch{" OP3 XCT"}=" O3P XCT"; $resn_exch{" OP1 XCT"}=" O1P XCT"; $resn_exch{" OP2 XCT"}=" O2P XCT";
$resn_exch{" OP3 XCY"}=" O3P XCY"; $resn_exch{" OP2 XCY"}=" O2P XCY"; $resn_exch{" O5' XCY"}=" O5* XCY";
$resn_exch{" C2' XCY"}=" C2* XCY"; $resn_exch{" C5' XCY"}=" C5* XCY"; $resn_exch{" C4' XCY"}=" C4* XCY";
$resn_exch{" O4' XCY"}=" O4* XCY"; $resn_exch{" C1' XCY"}=" C1* XCY"; $resn_exch{" C3' XCY"}=" C3* XCY";
$resn_exch{" O3' XCY"}=" O3* XCY"; $resn_exch{" OP1 XCY"}=" O1P XCY"; $resn_exch{" OP1 XGA"}=" O1P XGA";
$resn_exch{" OP2 XGA"}=" O2P XGA"; $resn_exch{" OP3 XGA"}=" O3P XGA"; $resn_exch{" OP1 XGL"}=" O1P XGL";
$resn_exch{" OP2 XGL"}=" O2P XGL"; $resn_exch{" O5' XGL"}=" O5* XGL"; $resn_exch{" C5' XGL"}=" C5* XGL";
$resn_exch{" C4' XGL"}=" C4* XGL"; $resn_exch{" C6' XGL"}=" C6* XGL"; $resn_exch{" C7' XGL"}=" C7* XGL";
$resn_exch{" C3' XGL"}=" C3* XGL"; $resn_exch{" O3' XGL"}=" O3* XGL"; $resn_exch{" C2' XGL"}=" C2* XGL";
$resn_exch{" C1' XGL"}=" C1* XGL"; $resn_exch{" OP3 XGL"}=" O3P XGL"; $resn_exch{" OP3 XGU"}=" O3P XGU";
$resn_exch{" OP1 XGU"}=" O1P XGU"; $resn_exch{" OP2 XGU"}=" O2P XGU"; $resn_exch{" CCX XR2"}="C12' XR2";
$resn_exch{" CDX XR2"}="C13' XR2"; $resn_exch{" CEX XR2"}="C14' XR2"; $resn_exch{" NAX XR2"}="N10' XR2";
$resn_exch{" CBX XR2"}="C11' XR2"; $resn_exch{" C9X XR2"}="C9A' XR2"; $resn_exch{" OP3 XTH"}=" O3P XTH";
$resn_exch{" OP1 XTH"}=" O1P XTH"; $resn_exch{" OP2 XTH"}=" O2P XTH"; $resn_exch{" OP1 XTL"}=" O1P XTL";
$resn_exch{" OP2 XTL"}=" O2P XTL"; $resn_exch{" O5' XTL"}=" O5* XTL"; $resn_exch{" C5' XTL"}=" C5* XTL";
$resn_exch{" C4' XTL"}=" C4* XTL"; $resn_exch{" C6' XTL"}=" C6* XTL"; $resn_exch{" C7' XTL"}=" C7* XTL";
$resn_exch{" C3' XTL"}=" C3* XTL"; $resn_exch{" O3' XTL"}=" O3* XTL"; $resn_exch{" C2' XTL"}=" C2* XTL";
$resn_exch{" C1' XTL"}=" C1* XTL"; $resn_exch{" OP3 XTL"}=" O3P XTL"; $resn_exch{" C1' XTS"}=" C1* XTS";
$resn_exch{" O4' XTS"}=" O4* XTS"; $resn_exch{" C4' XTS"}=" C4* XTS"; $resn_exch{" C5' XTS"}=" C5* XTS";
$resn_exch{" O5' XTS"}=" O5* XTS"; $resn_exch{" C3' XTS"}=" C3* XTS"; $resn_exch{" O3' XTS"}=" O3* XTS";
$resn_exch{" C2' XTS"}=" C2* XTS"; $resn_exch{" O2' XTS"}=" O2* XTS"; $resn_exch{" OP1 XTY"}=" O1P XTY";
$resn_exch{" OP2 XTY"}=" O2P XTY"; $resn_exch{" OP3 XTY"}=" O3P XTY"; $resn_exch{" C1' XUG"}=" C1* XUG";
$resn_exch{" O4' XUG"}=" O4* XUG"; $resn_exch{" C2' XUG"}=" C2* XUG"; $resn_exch{"SE2' XUG"}="SE2* XUG";
$resn_exch{" CA' XUG"}=" CA* XUG"; $resn_exch{" C3' XUG"}=" C3* XUG"; $resn_exch{" O3' XUG"}=" O3* XUG";
$resn_exch{" C4' XUG"}=" C4* XUG"; $resn_exch{" C5' XUG"}=" C5* XUG"; $resn_exch{" O5' XUG"}=" O5* XUG";
$resn_exch{" OP1 XUG"}=" O1P XUG"; $resn_exch{" OP2 XUG"}=" O2P XUG"; $resn_exch{" OP3 XUG"}=" O3P XUG";
$resn_exch{" OP1   Y"}=" O1P   Y"; $resn_exch{" OP2   Y"}=" O2P   Y"; $resn_exch{" OP3   Y"}=" O3P   Y";
$resn_exch{" O5'   Y"}=" O5*   Y"; $resn_exch{" C5'   Y"}=" C5*   Y"; $resn_exch{" C4'   Y"}=" C4*   Y";
$resn_exch{" O4'   Y"}=" O4*   Y"; $resn_exch{" C3'   Y"}=" C3*   Y"; $resn_exch{" O3'   Y"}=" O3*   Y";
$resn_exch{" C2'   Y"}=" C2*   Y"; $resn_exch{" C1'   Y"}=" C1*   Y"; $resn_exch{" N1A YE1"}=" AN1 YE1";
$resn_exch{" C2A YE1"}=" AC2 YE1"; $resn_exch{" N3A YE1"}=" AN3 YE1"; $resn_exch{" C4A YE1"}=" AC4 YE1";
$resn_exch{" C5A YE1"}=" AC5 YE1"; $resn_exch{" C6A YE1"}=" AC6 YE1"; $resn_exch{" N6A YE1"}=" AN6 YE1";
$resn_exch{" N7A YE1"}=" AN7 YE1"; $resn_exch{" C8A YE1"}=" AC8 YE1"; $resn_exch{" N9A YE1"}=" AN9 YE1";
$resn_exch{" C1' YE1"}=" C1* YE1"; $resn_exch{" C2' YE1"}=" C2* YE1"; $resn_exch{" O2' YE1"}=" O2* YE1";
$resn_exch{" C3' YE1"}=" C3* YE1"; $resn_exch{" O3' YE1"}=" O3* YE1"; $resn_exch{" P3' YE1"}=" P3* YE1";
$resn_exch{" O7A YE1"}=" AO7 YE1"; $resn_exch{" O8A YE1"}=" AO8 YE1"; $resn_exch{" O9A YE1"}=" AO9 YE1";
$resn_exch{" C4' YE1"}=" C4* YE1"; $resn_exch{" O4' YE1"}=" O4* YE1"; $resn_exch{" C5' YE1"}=" C5* YE1";
$resn_exch{" O5' YE1"}=" O5* YE1"; $resn_exch{" P1A YE1"}=" AP1 YE1"; $resn_exch{" O1A YE1"}=" AO1 YE1";
$resn_exch{" O2A YE1"}=" AO2 YE1"; $resn_exch{" O3A YE1"}=" AO3 YE1"; $resn_exch{" P2A YE1"}=" AP2 YE1";
$resn_exch{" O4A YE1"}=" AO4 YE1"; $resn_exch{" O5A YE1"}=" AO5 YE1"; $resn_exch{" O6A YE1"}=" AO6 YE1";
$resn_exch{" C9P YE1"}=" PC9 YE1"; $resn_exch{" O9P YE1"}=" PO9 YE1"; $resn_exch{" N8P YE1"}=" PN8 YE1";
$resn_exch{" C7P YE1"}=" PC7 YE1"; $resn_exch{" C6P YE1"}=" PC6 YE1"; $resn_exch{" C5P YE1"}=" PC5 YE1";
$resn_exch{" O5P YE1"}=" PO5 YE1"; $resn_exch{" N4P YE1"}=" PN4 YE1"; $resn_exch{" C3P YE1"}=" PC3 YE1";
$resn_exch{" C2P YE1"}=" PC2 YE1"; $resn_exch{" C1'  YG"}=" C1*  YG"; $resn_exch{" C2'  YG"}=" C2*  YG";
$resn_exch{" O2'  YG"}=" O2*  YG"; $resn_exch{" C3'  YG"}=" C3*  YG"; $resn_exch{" O3'  YG"}=" O3*  YG";
$resn_exch{" C4'  YG"}=" C4*  YG"; $resn_exch{" O4'  YG"}=" O4*  YG"; $resn_exch{" C5'  YG"}=" C5*  YG";
$resn_exch{" O5'  YG"}=" O5*  YG"; $resn_exch{" OP1  YG"}=" O1P  YG"; $resn_exch{" OP2  YG"}=" O2P  YG";
$resn_exch{" OP3  YG"}=" O3P  YG"; $resn_exch{" O5' YLY"}=" O5* YLY"; $resn_exch{" C5' YLY"}=" C5* YLY";
$resn_exch{" C4' YLY"}=" C4* YLY"; $resn_exch{" O4' YLY"}=" O4* YLY"; $resn_exch{" C3' YLY"}=" C3* YLY";
$resn_exch{" O3' YLY"}=" O3* YLY"; $resn_exch{" C2' YLY"}=" C2* YLY"; $resn_exch{" O2' YLY"}=" O2* YLY";
$resn_exch{" C1' YLY"}=" C1* YLY"; $resn_exch{" O5' YMP"}=" O5* YMP"; $resn_exch{" C5' YMP"}=" C5* YMP";
$resn_exch{" C4' YMP"}=" C4* YMP"; $resn_exch{" O4' YMP"}=" O4* YMP"; $resn_exch{" C1' YMP"}=" C1* YMP";
$resn_exch{" C2' YMP"}=" C2* YMP"; $resn_exch{" C3' YMP"}=" C3* YMP"; $resn_exch{" O3' YMP"}=" O3* YMP";
$resn_exch{" O2' YMP"}=" O2* YMP"; $resn_exch{" OP3 YRR"}=" O3P YRR"; $resn_exch{" OP1 YRR"}=" O1P YRR";
$resn_exch{" OP2 YRR"}=" O2P YRR"; $resn_exch{" O5' YRR"}=" O5* YRR"; $resn_exch{" C5' YRR"}=" C5* YRR";
$resn_exch{" C4' YRR"}=" C4* YRR"; $resn_exch{" N4' YRR"}=" N4* YRR"; $resn_exch{" C3' YRR"}=" C3* YRR";
$resn_exch{" O3' YRR"}=" O3* YRR"; $resn_exch{" C2' YRR"}=" C2* YRR"; $resn_exch{" C1' YRR"}=" C1* YRR";
$resn_exch{" O5' YSA"}=" O5* YSA"; $resn_exch{" C5' YSA"}=" C5* YSA"; $resn_exch{" C4' YSA"}=" C4* YSA";
$resn_exch{" O4' YSA"}=" O4* YSA"; $resn_exch{" C3' YSA"}=" C3* YSA"; $resn_exch{" O3' YSA"}=" O3* YSA";
$resn_exch{" C2' YSA"}=" C2* YSA"; $resn_exch{" O2' YSA"}=" O2* YSA"; $resn_exch{" C1' YSA"}=" C1* YSA";
$resn_exch{" OP1 YYG"}=" O1P YYG"; $resn_exch{" OP2 YYG"}=" O2P YYG"; $resn_exch{" OP3 YYG"}=" O3P YYG";
$resn_exch{" O5' YYG"}=" O5* YYG"; $resn_exch{" C5' YYG"}=" C5* YYG"; $resn_exch{" C4' YYG"}=" C4* YYG";
$resn_exch{" O4' YYG"}=" O4* YYG"; $resn_exch{" C3' YYG"}=" C3* YYG"; $resn_exch{" O3' YYG"}=" O3* YYG";
$resn_exch{" C2' YYG"}=" C2* YYG"; $resn_exch{" O2' YYG"}=" O2* YYG"; $resn_exch{" C1' YYG"}=" C1* YYG";
$resn_exch{" O5' YYY"}=" O5* YYY"; $resn_exch{" C5' YYY"}=" C5* YYY"; $resn_exch{" C4' YYY"}=" C4* YYY";
$resn_exch{" O4' YYY"}=" O4* YYY"; $resn_exch{" C3' YYY"}=" C3* YYY"; $resn_exch{" O3' YYY"}=" O3* YYY";
$resn_exch{" C2' YYY"}=" C2* YYY"; $resn_exch{" C1' YYY"}=" C1* YYY"; $resn_exch{" OP1   Z"}=" O1P   Z";
$resn_exch{" OP2   Z"}=" O2P   Z"; $resn_exch{" O5'   Z"}=" O5*   Z"; $resn_exch{" C5'   Z"}=" C5*   Z";
$resn_exch{" C4'   Z"}=" C4*   Z"; $resn_exch{" O4'   Z"}=" O4*   Z"; $resn_exch{" C3'   Z"}=" C3*   Z";
$resn_exch{" O3'   Z"}=" O3*   Z"; $resn_exch{" C2'   Z"}=" C2*   Z"; $resn_exch{" C1'   Z"}=" C1*   Z";
$resn_exch{" OP3   Z"}=" O3P   Z"; $resn_exch{" O5B Z5A"}=" O5* Z5A"; $resn_exch{" C5B Z5A"}=" C5* Z5A";
$resn_exch{" C4B Z5A"}=" C4* Z5A"; $resn_exch{" O4B Z5A"}=" O4* Z5A"; $resn_exch{" C3B Z5A"}=" C3* Z5A";
$resn_exch{" O3' Z5A"}=" O3* Z5A"; $resn_exch{" C2B Z5A"}=" C2* Z5A"; $resn_exch{" O2' Z5A"}=" O2* Z5A";
$resn_exch{" C1B Z5A"}=" C1* Z5A"; $resn_exch{" C1X ZAF"}="*CP1 ZAF"; $resn_exch{" C2X ZAF"}="*CP2 ZAF";
$resn_exch{" C3X ZAF"}="*CP3 ZAF"; $resn_exch{" C4X ZAF"}="*CP4 ZAF"; $resn_exch{" C5X ZAF"}="*CP5 ZAF";
$resn_exch{" C6X ZAF"}="*CP6 ZAF"; $resn_exch{" OP1 ZDU"}=" O1P ZDU"; $resn_exch{" OP2 ZDU"}=" O2P ZDU";
$resn_exch{" O5' ZDU"}=" O5* ZDU"; $resn_exch{" C2' ZDU"}=" C2* ZDU"; $resn_exch{" C5' ZDU"}=" C5* ZDU";
$resn_exch{" C4' ZDU"}=" C4* ZDU"; $resn_exch{" O4' ZDU"}=" O4* ZDU"; $resn_exch{" C1' ZDU"}=" C1* ZDU";
$resn_exch{" C3' ZDU"}=" C3* ZDU"; $resn_exch{" O3' ZDU"}=" O3* ZDU"; $resn_exch{" OP3 ZDU"}=" O3P ZDU";
$resn_exch{" CZA ZED"}="CZ10 ZED"; $resn_exch{" CZB ZED"}="CZ11 ZED"; $resn_exch{" CZC ZED"}="CZ12 ZED";
$resn_exch{" CZD ZED"}="CZ13 ZED"; $resn_exch{" CZE ZED"}="CZ14 ZED"; $resn_exch{" CZF ZED"}="CZ15 ZED";
$resn_exch{" NA  ZEM"}=" N_A ZEM"; $resn_exch{" NB  ZEM"}=" N_B ZEM"; $resn_exch{" NC  ZEM"}=" N_C ZEM";
$resn_exch{" ND  ZEM"}=" N_D ZEM"; $resn_exch{" PA  ZID"}=" AP  ZID"; $resn_exch{" O1A ZID"}=" AO1 ZID";
$resn_exch{" O2A ZID"}=" AO2 ZID"; $resn_exch{" O5B ZID"}="AO5* ZID"; $resn_exch{" C5B ZID"}="AC5* ZID";
$resn_exch{" C4B ZID"}="AC4* ZID"; $resn_exch{" O4B ZID"}="AO4* ZID"; $resn_exch{" C3B ZID"}="AC3* ZID";
$resn_exch{" O3B ZID"}="AO3* ZID"; $resn_exch{" C2B ZID"}="AC2* ZID"; $resn_exch{" O2B ZID"}="AO2* ZID";
$resn_exch{" C1B ZID"}="AC1* ZID"; $resn_exch{" N9A ZID"}=" AN9 ZID"; $resn_exch{" C8A ZID"}=" AC8 ZID";
$resn_exch{" N7A ZID"}=" AN7 ZID"; $resn_exch{" C5A ZID"}=" AC5 ZID"; $resn_exch{" C6A ZID"}=" AC6 ZID";
$resn_exch{" N6A ZID"}=" AN6 ZID"; $resn_exch{" N1A ZID"}=" AN1 ZID"; $resn_exch{" C2A ZID"}=" AC2 ZID";
$resn_exch{" N3A ZID"}=" AN3 ZID"; $resn_exch{" C4A ZID"}=" AC4 ZID"; $resn_exch{" PN  ZID"}=" NP  ZID";
$resn_exch{" O1N ZID"}=" NO1 ZID"; $resn_exch{" O2N ZID"}=" NO2 ZID"; $resn_exch{" O5D ZID"}="NO5* ZID";
$resn_exch{" C5D ZID"}="NC5* ZID"; $resn_exch{" C4D ZID"}="NC4* ZID"; $resn_exch{" O4D ZID"}="NO4* ZID";
$resn_exch{" C3D ZID"}="NC3* ZID"; $resn_exch{" O3D ZID"}="NO3* ZID"; $resn_exch{" C2D ZID"}="NC2* ZID";
$resn_exch{" O2D ZID"}="NO2* ZID"; $resn_exch{" C1D ZID"}="NC1* ZID"; $resn_exch{" N1N ZID"}=" NN1 ZID";
$resn_exch{" C2N ZID"}=" NC2 ZID"; $resn_exch{" C3N ZID"}=" NC3 ZID"; $resn_exch{" C7N ZID"}=" NC7 ZID";
$resn_exch{" O7N ZID"}=" NO7 ZID"; $resn_exch{" N7N ZID"}=" NN7 ZID"; $resn_exch{" C4N ZID"}=" NC4 ZID";
$resn_exch{" C5N ZID"}=" NC5 ZID"; $resn_exch{" C6N ZID"}=" NC6 ZID"; $resn_exch{" C7Z ZID"}=" ZC7 ZID";
$resn_exch{" O7Z ZID"}=" ZO7 ZID"; $resn_exch{" N1Z ZID"}=" ZN1 ZID"; $resn_exch{" C2Z ZID"}=" ZC2 ZID";
$resn_exch{" C3Z ZID"}=" ZC3 ZID"; $resn_exch{" C4Z ZID"}=" ZC4 ZID"; $resn_exch{" C5Z ZID"}=" ZC5 ZID";
$resn_exch{" C6Z ZID"}=" ZC6 ZID"; $resn_exch{" NA  ZNH"}=" N_A ZNH"; $resn_exch{" NB  ZNH"}=" N_B ZNH";
$resn_exch{" NC  ZNH"}=" N_C ZNH"; $resn_exch{" ND  ZNH"}=" N_D ZNH"; $resn_exch{"H021 001"}="1H02 001";
$resn_exch{"H061 001"}="1H06 001"; $resn_exch{"H071 001"}="1H07 001"; $resn_exch{"H072 001"}="2H07 001";
$resn_exch{"H073 001"}="3H07 001"; $resn_exch{"H081 001"}="1H08 001"; $resn_exch{"H082 001"}="2H08 001";
$resn_exch{"H083 001"}="3H08 001"; $resn_exch{"H091 001"}="1H09 001"; $resn_exch{"H092 001"}="2H09 001";
$resn_exch{"H093 001"}="3H09 001"; $resn_exch{"H121 001"}="1H12 001"; $resn_exch{"H122 001"}="2H12 001";
$resn_exch{"H131 001"}="1H13 001"; $resn_exch{"H132 001"}="2H13 001"; $resn_exch{"H141 001"}="1H14 001";
$resn_exch{"H142 001"}="2H14 001"; $resn_exch{"H151 001"}="1H15 001"; $resn_exch{"H152 001"}="2H15 001";
$resn_exch{"H161 001"}="1H16 001"; $resn_exch{"H181 001"}="1H18 001"; $resn_exch{"H191 001"}="1H19 001";
$resn_exch{"H192 001"}="2H19 001"; $resn_exch{"H201 001"}="1H20 001"; $resn_exch{"H202 001"}="2H20 001";
$resn_exch{"H211 001"}="1H21 001"; $resn_exch{"H212 001"}="2H21 001"; $resn_exch{"H231 001"}="1H23 001";
$resn_exch{"H241 001"}="1H24 001"; $resn_exch{"H251 001"}="1H25 001"; $resn_exch{"H261 001"}="1H26 001";
$resn_exch{"H271 001"}="1H27 001"; $resn_exch{"H272 001"}="2H27 001"; $resn_exch{"H281 001"}="1H28 001";
$resn_exch{"H282 001"}="2H28 001"; $resn_exch{"H291 001"}="1H29 001"; $resn_exch{"H292 001"}="2H29 001";
$resn_exch{"H311 001"}="1H31 001"; $resn_exch{"H321 001"}="1H32 001"; $resn_exch{"H331 001"}="1H33 001";
$resn_exch{"H341 001"}="1H34 001"; $resn_exch{"H351 001"}="1H35 001"; $resn_exch{" H31 002"}="1H3  002";
$resn_exch{" H32 002"}="2H3  002"; $resn_exch{" H41 002"}="1H4  002"; $resn_exch{" H42 002"}="2H4  002";
$resn_exch{" H43 002"}="3H4  002"; $resn_exch{" H81 002"}="1H8  002"; $resn_exch{" H82 002"}="2H8  002";
$resn_exch{"H101 002"}="1H10 002"; $resn_exch{"H102 002"}="2H10 002"; $resn_exch{"H121 002"}="1H12 002";
$resn_exch{"H122 002"}="2H12 002"; $resn_exch{"H123 002"}="3H12 002"; $resn_exch{"H201 002"}="1H20 002";
$resn_exch{"H202 002"}="2H20 002"; $resn_exch{"H203 002"}="3H20 002"; $resn_exch{"H211 002"}="1H21 002";
$resn_exch{"H212 002"}="2H21 002"; $resn_exch{"H213 002"}="3H21 002"; $resn_exch{"H231 002"}="1H23 002";
$resn_exch{"H232 002"}="2H23 002"; $resn_exch{"H7C1 003"}="1H7C 003"; $resn_exch{"H7C2 003"}="2H7C 003";
$resn_exch{"H101 003"}="1H10 003"; $resn_exch{"H102 003"}="2H10 003"; $resn_exch{"H103 003"}="3H10 003";
$resn_exch{"H9C1 003"}="1H9C 003"; $resn_exch{"H9C2 003"}="2H9C 003"; $resn_exch{"H9C3 003"}="3H9C 003";
$resn_exch{"H231 003"}="1H23 003"; $resn_exch{"H232 003"}="2H23 003"; $resn_exch{"H121 003"}="1H12 003";
$resn_exch{"H122 003"}="2H12 003"; $resn_exch{"H123 003"}="3H12 003"; $resn_exch{"H1C1 007"}="1H1C 007";
$resn_exch{"H1C2 007"}="2H1C 007"; $resn_exch{"H2C1 007"}="1H2C 007"; $resn_exch{"H2C2 007"}="2H2C 007";
$resn_exch{"H4C1 007"}="1H4C 007"; $resn_exch{"H4C2 007"}="2H4C 007"; $resn_exch{"H5C1 007"}="1H5C 007";
$resn_exch{"H5C2 007"}="2H5C 007"; $resn_exch{"H121 007"}="1H12 007"; $resn_exch{"H122 007"}="2H12 007";
$resn_exch{"H131 007"}="1H13 007"; $resn_exch{"H132 007"}="2H13 007"; $resn_exch{"H8C1 008"}="1H8C 008";
$resn_exch{"H8C2 008"}="2H8C 008"; $resn_exch{"H101 008"}="1H10 008"; $resn_exch{"H102 008"}="2H10 008";
$resn_exch{"H111 008"}="1H11 008"; $resn_exch{"H112 008"}="2H11 008"; $resn_exch{"H191 008"}="1H19 008";
$resn_exch{"H192 008"}="2H19 008"; $resn_exch{"H161 008"}="1H16 008"; $resn_exch{"H162 008"}="2H16 008";
$resn_exch{"H261 008"}="1H26 008"; $resn_exch{"H262 008"}="2H26 008"; $resn_exch{" H11 017"}="1H1  017";
$resn_exch{" H12 017"}="2H1  017"; $resn_exch{"H121 017"}="1H12 017"; $resn_exch{"H122 017"}="2H12 017";
$resn_exch{"H141 017"}="1H14 017"; $resn_exch{"H142 017"}="2H14 017"; $resn_exch{"H143 017"}="3H14 017";
$resn_exch{"H151 017"}="1H15 017"; $resn_exch{"H152 017"}="2H15 017"; $resn_exch{"H153 017"}="3H15 017";
$resn_exch{"H161 017"}="1H16 017"; $resn_exch{"H162 017"}="2H16 017"; $resn_exch{"H251 017"}="1H25 017";
$resn_exch{"H252 017"}="2H25 017"; $resn_exch{"H291 017"}="1H29 017"; $resn_exch{"H292 017"}="2H29 017";
$resn_exch{"H301 017"}="1H30 017"; $resn_exch{"H302 017"}="2H30 017"; $resn_exch{"H321 017"}="1H32 017";
$resn_exch{"H322 017"}="2H32 017"; $resn_exch{"H211 024"}="1H21 024"; $resn_exch{"H212 024"}="2H21 024";
$resn_exch{"H301 024"}="1H30 024"; $resn_exch{"H302 024"}="2H30 024"; $resn_exch{"H311 024"}="1H31 024";
$resn_exch{"H312 024"}="2H31 024"; $resn_exch{"H321 024"}="1H32 024"; $resn_exch{"H322 024"}="2H32 024";
$resn_exch{"H331 024"}="1H33 024"; $resn_exch{"H332 024"}="2H33 024"; $resn_exch{"H341 024"}="1H34 024";
$resn_exch{"H342 024"}="2H34 024"; $resn_exch{" H81 033"}="1H8  033"; $resn_exch{" H82 033"}="2H8  033";
$resn_exch{" H83 033"}="3H8  033"; $resn_exch{" H91 033"}="1H9  033"; $resn_exch{" H92 033"}="2H9  033";
$resn_exch{" H93 033"}="3H9  033"; $resn_exch{"HAU1 039"}="1HAU 039"; $resn_exch{"HAU2 039"}="2HAU 039";
$resn_exch{"HAU3 039"}="3HAU 039"; $resn_exch{"HAS1 039"}="1HAS 039"; $resn_exch{"HAS2 039"}="2HAS 039";
$resn_exch{"H221 03R"}="1H22 03R"; $resn_exch{"H222 03R"}="2H22 03R"; $resn_exch{"H231 03R"}="1H23 03R";
$resn_exch{"H232 03R"}="2H23 03R"; $resn_exch{"H233 03R"}="3H23 03R"; $resn_exch{"H291 03R"}="1H29 03R";
$resn_exch{"H292 03R"}="2H29 03R"; $resn_exch{"H341 03R"}="1H34 03R"; $resn_exch{"H342 03R"}="2H34 03R";
$resn_exch{"H251 041"}="1H25 041"; $resn_exch{"H252 041"}="2H25 041"; $resn_exch{"H101 042"}="1H10 042";
$resn_exch{"H102 042"}="2H10 042"; $resn_exch{"H111 042"}="1H11 042"; $resn_exch{"H112 042"}="2H11 042";
$resn_exch{"H113 042"}="3H11 042"; $resn_exch{"H131 042"}="1H13 042"; $resn_exch{"H132 042"}="2H13 042";
$resn_exch{"H133 042"}="3H13 042"; $resn_exch{"H161 042"}="1H16 042"; $resn_exch{"H162 042"}="2H16 042";
$resn_exch{"H151 042"}="1H15 042"; $resn_exch{"H152 042"}="2H15 042"; $resn_exch{"H153 042"}="3H15 042";
$resn_exch{"H171 042"}="1H17 042"; $resn_exch{"H172 042"}="2H17 042"; $resn_exch{"H173 042"}="3H17 042";
$resn_exch{" H71 047"}="1H7  047"; $resn_exch{" H72 047"}="2H7  047"; $resn_exch{"H121 047"}="1H12 047";
$resn_exch{"H122 047"}="2H12 047"; $resn_exch{"H171 047"}="1H17 047"; $resn_exch{"H172 047"}="2H17 047";
$resn_exch{"H201 047"}="1H20 047"; $resn_exch{"H202 047"}="2H20 047"; $resn_exch{"H231 047"}="1H23 047";
$resn_exch{"H232 047"}="2H23 047"; $resn_exch{"H261 047"}="1H26 047"; $resn_exch{"H262 047"}="2H26 047";
$resn_exch{"H291 047"}="1H29 047"; $resn_exch{"H292 047"}="2H29 047"; $resn_exch{"H501 047"}="1H50 047";
$resn_exch{"H502 047"}="2H50 047"; $resn_exch{"H503 047"}="3H50 047"; $resn_exch{"H341 059"}="1H34 059";
$resn_exch{"H342 059"}="2H34 059"; $resn_exch{"H331 059"}="1H33 059"; $resn_exch{"H332 059"}="2H33 059";
$resn_exch{"H301 059"}="1H30 059"; $resn_exch{"H302 059"}="2H30 059"; $resn_exch{"H303 059"}="3H30 059";
$resn_exch{"H291 059"}="1H29 059"; $resn_exch{"H292 059"}="2H29 059"; $resn_exch{"H293 059"}="3H29 059";
$resn_exch{"H151 059"}="1H15 059"; $resn_exch{"H152 059"}="2H15 059"; $resn_exch{" H91 059"}="1H9  059";
$resn_exch{" H92 059"}="2H9  059"; $resn_exch{"H101 059"}="1H10 059"; $resn_exch{"H102 059"}="2H10 059";
$resn_exch{"H181 061"}="1H18 061"; $resn_exch{"H182 061"}="2H18 061"; $resn_exch{"H211 061"}="1H21 061";
$resn_exch{"H212 061"}="2H21 061"; $resn_exch{"H221 061"}="1H22 061"; $resn_exch{"H222 061"}="2H22 061";
$resn_exch{"H231 061"}="1H23 061"; $resn_exch{"H232 061"}="2H23 061"; $resn_exch{"H241 061"}="1H24 061";
$resn_exch{"H242 061"}="2H24 061"; $resn_exch{"H243 061"}="3H24 061"; $resn_exch{"H251 062"}="1H25 062";
$resn_exch{"H252 062"}="2H25 062"; $resn_exch{"H261 062"}="1H26 062"; $resn_exch{"H262 062"}="2H26 062";
$resn_exch{"H271 062"}="1H27 062"; $resn_exch{"H272 062"}="2H27 062"; $resn_exch{"H371 062"}="1H37 062";
$resn_exch{"H372 062"}="2H37 062"; $resn_exch{"H531 062"}="1H53 062"; $resn_exch{"H532 062"}="2H53 062";
$resn_exch{"H541 062"}="1H54 062"; $resn_exch{"H542 062"}="2H54 062"; $resn_exch{"H141 065"}="1H14 065";
$resn_exch{"H142 065"}="2H14 065"; $resn_exch{"H143 065"}="3H14 065"; $resn_exch{"H151 065"}="1H15 065";
$resn_exch{"H152 065"}="2H15 065"; $resn_exch{"H153 065"}="3H15 065"; $resn_exch{"H121 065"}="1H12 065";
$resn_exch{"H122 065"}="2H12 065"; $resn_exch{"H161 065"}="1H16 065"; $resn_exch{"H162 065"}="2H16 065";
$resn_exch{"H321 065"}="1H32 065"; $resn_exch{"H322 065"}="2H32 065"; $resn_exch{"H251 065"}="1H25 065";
$resn_exch{"H252 065"}="2H25 065"; $resn_exch{"H291 065"}="1H29 065"; $resn_exch{"H292 065"}="2H29 065";
$resn_exch{"H301 065"}="1H30 065"; $resn_exch{"H302 065"}="2H30 065"; $resn_exch{"H401 065"}="1H40 065";
$resn_exch{"H402 065"}="2H40 065"; $resn_exch{"H1C1 072"}="1H1C 072"; $resn_exch{"H1C2 072"}="2H1C 072";
$resn_exch{"H2B1 072"}="1H2B 072"; $resn_exch{"H2B2 072"}="2H2B 072"; $resn_exch{"H2C1 072"}="1H2C 072";
$resn_exch{"H2C2 072"}="2H2C 072"; $resn_exch{"H2D1 072"}="1H2D 072"; $resn_exch{"H2D2 072"}="2H2D 072";
$resn_exch{"H2E1 072"}="1H2E 072"; $resn_exch{"H2E2 072"}="2H2E 072"; $resn_exch{"H2F1 072"}="1H2F 072";
$resn_exch{"H2F2 072"}="2H2F 072"; $resn_exch{"H2G1 072"}="1H2G 072"; $resn_exch{"H2G2 072"}="2H2G 072";
$resn_exch{"H2H1 072"}="1H2H 072"; $resn_exch{"H2H2 072"}="2H2H 072"; $resn_exch{"H2H3 072"}="3H2H 072";
$resn_exch{"H3B1 072"}="1H3B 072"; $resn_exch{"H3B2 072"}="2H3B 072"; $resn_exch{"H3C1 072"}="1H3C 072";
$resn_exch{"H3C2 072"}="2H3C 072"; $resn_exch{"H3D1 072"}="1H3D 072"; $resn_exch{"H3D2 072"}="2H3D 072";
$resn_exch{"H3E1 072"}="1H3E 072"; $resn_exch{"H3E2 072"}="2H3E 072"; $resn_exch{"HOM3 072"}="3HOM 072";
$resn_exch{"H4B1 072"}="1H4B 072"; $resn_exch{"H4B2 072"}="2H4B 072"; $resn_exch{"H5B1 072"}="1H5B 072";
$resn_exch{"H5B2 072"}="2H5B 072"; $resn_exch{"H671 073"}="1H67 073"; $resn_exch{"H672 073"}="2H67 073";
$resn_exch{"H673 073"}="3H67 073"; $resn_exch{"H131 073"}="1H13 073"; $resn_exch{"H132 073"}="2H13 073";
$resn_exch{"H371 073"}="1H37 073"; $resn_exch{"H372 073"}="2H37 073"; $resn_exch{" H11 074"}="1H1  074";
$resn_exch{" H12 074"}="2H1  074"; $resn_exch{" H13 074"}="3H1  074"; $resn_exch{" H21 074"}="1H2  074";
$resn_exch{" H22 074"}="2H2  074"; $resn_exch{" H31 074"}="1H3  074"; $resn_exch{" H32 074"}="2H3  074";
$resn_exch{"HN11 074"}="1HN1 074"; $resn_exch{" H51 074"}="1H5  074"; $resn_exch{" H61 074"}="1H6  074";
$resn_exch{" H62 074"}="2H6  074"; $resn_exch{"HO21 074"}="1HO2 074"; $resn_exch{"HN21 074"}="1HN2 074";
$resn_exch{" H81 074"}="1H8  074"; $resn_exch{" H91 074"}="1H9  074"; $resn_exch{"H101 074"}="1H10 074";
$resn_exch{"H102 074"}="2H10 074"; $resn_exch{"H103 074"}="3H10 074"; $resn_exch{"H111 074"}="1H11 074";
$resn_exch{"H112 074"}="2H11 074"; $resn_exch{"H121 074"}="1H12 074"; $resn_exch{"H122 074"}="2H12 074";
$resn_exch{"H123 074"}="3H12 074"; $resn_exch{"H141 074"}="1H14 074"; $resn_exch{"H142 074"}="2H14 074";
$resn_exch{"H151 074"}="1H15 074"; $resn_exch{"H152 074"}="2H15 074"; $resn_exch{"H161 074"}="1H16 074";
$resn_exch{"H162 074"}="2H16 074"; $resn_exch{"H171 074"}="1H17 074"; $resn_exch{"H391 075"}="1H39 075";
$resn_exch{"H392 075"}="2H39 075"; $resn_exch{"H291 075"}="1H29 075"; $resn_exch{"H292 075"}="2H29 075";
$resn_exch{"H293 075"}="3H29 075"; $resn_exch{"H301 075"}="1H30 075"; $resn_exch{"H302 075"}="2H30 075";
$resn_exch{"H303 075"}="3H30 075"; $resn_exch{"H191 075"}="1H19 075"; $resn_exch{"H192 075"}="2H19 075";
$resn_exch{"H151 075"}="1H15 075"; $resn_exch{"H152 075"}="2H15 075"; $resn_exch{"H111 075"}="1H11 075";
$resn_exch{"H112 075"}="2H11 075"; $resn_exch{"H281 075"}="1H28 075"; $resn_exch{"H282 075"}="2H28 075";
$resn_exch{"H283 075"}="3H28 075"; $resn_exch{" H91 075"}="1H9  075"; $resn_exch{" H92 075"}="2H9  075";
$resn_exch{"H501 084"}="1H50 084"; $resn_exch{"H502 084"}="2H50 084"; $resn_exch{"H503 084"}="3H50 084";
$resn_exch{"H131 084"}="1H13 084"; $resn_exch{"H132 084"}="2H13 084"; $resn_exch{"H141 084"}="1H14 084";
$resn_exch{"H142 084"}="2H14 084"; $resn_exch{"H161 084"}="1H16 084"; $resn_exch{"H162 084"}="2H16 084";
$resn_exch{"H171 084"}="1H17 084"; $resn_exch{"H172 084"}="2H17 084"; $resn_exch{" H61 084"}="1H6  084";
$resn_exch{" H62 084"}="2H6  084"; $resn_exch{" H63 084"}="3H6  084"; $resn_exch{"HAZ1 090"}="1HAZ 090";
$resn_exch{"HAZ2 090"}="2HAZ 090"; $resn_exch{"HAZ3 090"}="3HAZ 090"; $resn_exch{"HAY1 090"}="1HAY 090";
$resn_exch{"HAY2 090"}="2HAY 090"; $resn_exch{"HAY3 090"}="3HAY 090"; $resn_exch{"HAV1 093"}="1HAV 093";
$resn_exch{"HAV2 093"}="2HAV 093"; $resn_exch{"HAW1 093"}="1HAW 093"; $resn_exch{"HAW2 093"}="2HAW 093";
$resn_exch{"HAE1 093"}="1HAE 093"; $resn_exch{"HAE2 093"}="2HAE 093"; $resn_exch{"HAE3 093"}="3HAE 093";
$resn_exch{"HAT1 093"}="1HAT 093"; $resn_exch{"HAT2 093"}="2HAT 093"; $resn_exch{"HAT3 093"}="3HAT 093";
$resn_exch{"H101 094"}="1H10 094"; $resn_exch{"H102 094"}="2H10 094"; $resn_exch{"H201 094"}="1H20 094";
$resn_exch{"H202 094"}="2H20 094"; $resn_exch{"H211 094"}="1H21 094"; $resn_exch{"H212 094"}="2H21 094";
$resn_exch{"H221 094"}="1H22 094"; $resn_exch{"H222 094"}="2H22 094"; $resn_exch{" H21 097"}="1H2  097";
$resn_exch{" H22 097"}="2H2  097"; $resn_exch{" H23 097"}="3H2  097"; $resn_exch{" H81 097"}="1H8  097";
$resn_exch{" H82 097"}="2H8  097"; $resn_exch{" H83 097"}="3H8  097"; $resn_exch{" H91 097"}="1H9  097";
$resn_exch{" H92 097"}="2H9  097"; $resn_exch{" H93 097"}="3H9  097"; $resn_exch{"H111 097"}="1H11 097";
$resn_exch{"H112 097"}="2H11 097"; $resn_exch{"H113 097"}="3H11 097"; $resn_exch{"H151 097"}="1H15 097";
$resn_exch{"H152 097"}="2H15 097"; $resn_exch{" H31 097"}="1H3  097"; $resn_exch{" H32 097"}="2H3  097";
$resn_exch{" H33 097"}="3H3  097"; $resn_exch{" H71 097"}="1H7  097"; $resn_exch{" H72 097"}="2H7  097";
$resn_exch{" H73 097"}="3H7  097"; $resn_exch{" HN1 0A0"}="1HN  0A0"; $resn_exch{" HN2 0A0"}="2HN  0A0";
$resn_exch{"HBC1 0A0"}="1HBC 0A0"; $resn_exch{"HBC2 0A0"}="2HBC 0A0"; $resn_exch{"HMC1 0A0"}="1HMC 0A0";
$resn_exch{"HMC2 0A0"}="2HMC 0A0"; $resn_exch{"HMC3 0A0"}="3HMC 0A0"; $resn_exch{" HN1 0A1"}="1HN  0A1";
$resn_exch{" HN2 0A1"}="2HN  0A1"; $resn_exch{"HBC1 0A1"}="1HBC 0A1"; $resn_exch{"HBC2 0A1"}="2HBC 0A1";
$resn_exch{"HMC1 0A1"}="1HMC 0A1"; $resn_exch{"HMC2 0A1"}="2HMC 0A1"; $resn_exch{"HMC3 0A1"}="3HMC 0A1";
$resn_exch{"H011 0A2"}="1H01 0A2"; $resn_exch{"H012 0A2"}="2H01 0A2"; $resn_exch{"H021 0A2"}="1H02 0A2";
$resn_exch{"H022 0A2"}="2H02 0A2"; $resn_exch{"H023 0A2"}="3H02 0A2"; $resn_exch{"H031 0A2"}="1H03 0A2";
$resn_exch{"H032 0A2"}="2H03 0A2"; $resn_exch{"H051 0A2"}="1H05 0A2"; $resn_exch{"H052 0A2"}="2H05 0A2";
$resn_exch{"H061 0A2"}="1H06 0A2"; $resn_exch{"H062 0A2"}="2H06 0A2"; $resn_exch{"H071 0A2"}="1H07 0A2";
$resn_exch{"H072 0A2"}="2H07 0A2"; $resn_exch{"H081 0A2"}="1H08 0A2"; $resn_exch{"H082 0A2"}="2H08 0A2";
$resn_exch{"H091 0A2"}="1H09 0A2"; $resn_exch{"H092 0A2"}="2H09 0A2"; $resn_exch{"H181 0A2"}="1H18 0A2";
$resn_exch{"H182 0A2"}="2H18 0A2"; $resn_exch{"H191 0A2"}="1H19 0A2"; $resn_exch{"H192 0A2"}="2H19 0A2";
$resn_exch{"H193 0A2"}="3H19 0A2"; $resn_exch{"H2C1 0A3"}="1H2C 0A3"; $resn_exch{"H2C2 0A3"}="2H2C 0A3";
$resn_exch{"H3'1 0A3"}="1H3' 0A3"; $resn_exch{"H3'2 0A3"}="2H3' 0A3"; $resn_exch{"H3'3 0A3"}="3H3' 0A3";
$resn_exch{"H181 0A3"}="1H18 0A3"; $resn_exch{"H182 0A3"}="2H18 0A3"; $resn_exch{"H183 0A3"}="3H18 0A3";
$resn_exch{" HA1 0A3"}="1HA  0A3"; $resn_exch{" HB2 0A3"}="2HB  0A3"; $resn_exch{"H2C1 0A4"}="1H2C 0A4";
$resn_exch{"H2C2 0A4"}="2H2C 0A4"; $resn_exch{"H3'1 0A4"}="1H3' 0A4"; $resn_exch{"H3'2 0A4"}="2H3' 0A4";
$resn_exch{"H131 0A4"}="1H13 0A4"; $resn_exch{"H132 0A4"}="2H13 0A4"; $resn_exch{"H151 0A4"}="1H15 0A4";
$resn_exch{"H2C1 0A5"}="1H2C 0A5"; $resn_exch{"H2C2 0A5"}="2H2C 0A5"; $resn_exch{"H3C1 0A5"}="1H3C 0A5";
$resn_exch{"H3C2 0A5"}="2H3C 0A5"; $resn_exch{"H3C3 0A5"}="3H3C 0A5"; $resn_exch{"HBC1 0A5"}="1HBC 0A5";
$resn_exch{"HBC2 0A5"}="2HBC 0A5"; $resn_exch{"HD21 0A5"}="1HD2 0A5"; $resn_exch{"HD22 0A5"}="2HD2 0A5";
$resn_exch{"H051 0A6"}="1H05 0A6"; $resn_exch{"H052 0A6"}="2H05 0A6"; $resn_exch{"H081 0A6"}="1H08 0A6";
$resn_exch{"H082 0A6"}="2H08 0A6"; $resn_exch{"H091 0A6"}="1H09 0A6"; $resn_exch{"H092 0A6"}="2H09 0A6";
$resn_exch{"H093 0A6"}="3H09 0A6"; $resn_exch{"H101 0A7"}="1H10 0A7"; $resn_exch{"H102 0A7"}="2H10 0A7";
$resn_exch{" HN1 0A8"}="1HN  0A8"; $resn_exch{" HN2 0A8"}="2HN  0A8"; $resn_exch{"HBC1 0A8"}="1HBC 0A8";
$resn_exch{"HBC2 0A8"}="2HBC 0A8"; $resn_exch{"H2C1 0A8"}="1H2C 0A8"; $resn_exch{"H2C2 0A8"}="2H2C 0A8";
$resn_exch{"H3C1 0A8"}="1H3C 0A8"; $resn_exch{"H3C2 0A8"}="2H3C 0A8"; $resn_exch{" HN1 0A9"}="1HN  0A9";
$resn_exch{" HN2 0A9"}="2HN  0A9"; $resn_exch{"HBC1 0A9"}="1HBC 0A9"; $resn_exch{"HBC2 0A9"}="2HBC 0A9";
$resn_exch{"HMC1 0A9"}="1HMC 0A9"; $resn_exch{"HMC2 0A9"}="2HMC 0A9"; $resn_exch{"HMC3 0A9"}="3HMC 0A9";
$resn_exch{" HN1 0AA"}="1HN  0AA"; $resn_exch{" HA2 0AA"}="2HA  0AA"; $resn_exch{"HG11 0AA"}="1HG1 0AA";
$resn_exch{"HG12 0AA"}="2HG1 0AA"; $resn_exch{"HG13 0AA"}="3HG1 0AA"; $resn_exch{"HG21 0AA"}="1HG2 0AA";
$resn_exch{"HG22 0AA"}="2HG2 0AA"; $resn_exch{"HG23 0AA"}="3HG2 0AA"; $resn_exch{"H1C1 0AA"}="1H1C 0AA";
$resn_exch{"H1C2 0AA"}="2H1C 0AA"; $resn_exch{"H1C3 0AA"}="3H1C 0AA"; $resn_exch{" HN1 0AB"}="1HN  0AB";
$resn_exch{" HA2 0AB"}="2HA  0AB"; $resn_exch{"HG11 0AB"}="1HG1 0AB"; $resn_exch{"HG12 0AB"}="2HG1 0AB";
$resn_exch{"HG13 0AB"}="3HG1 0AB"; $resn_exch{" HN1 0AC"}="1HN  0AC"; $resn_exch{" HN2 0AC"}="2HN  0AC";
$resn_exch{"HBC1 0AC"}="1HBC 0AC"; $resn_exch{"HBC2 0AC"}="2HBC 0AC"; $resn_exch{"H1C1 0AC"}="1H1C 0AC";
$resn_exch{"H1C2 0AC"}="2H1C 0AC"; $resn_exch{"HA11 0AC"}="1HA1 0AC"; $resn_exch{"HA12 0AC"}="2HA1 0AC";
$resn_exch{"H5'1 0AD"}="1H5' 0AD"; $resn_exch{"H5'2 0AD"}="2H5' 0AD"; $resn_exch{"H2'1 0AD"}="1H2' 0AD";
$resn_exch{"H2'2 0AD"}="2H2' 0AD"; $resn_exch{"H4A1 0AD"}="1H4A 0AD"; $resn_exch{"H4A2 0AD"}="2H4A 0AD";
$resn_exch{"H5A1 0AD"}="1H5A 0AD"; $resn_exch{"H5A2 0AD"}="2H5A 0AD"; $resn_exch{" HN1 0AF"}="1HN  0AF";
$resn_exch{" HN2 0AF"}="2HN  0AF"; $resn_exch{"HBC1 0AF"}="1HBC 0AF"; $resn_exch{"HBC2 0AF"}="2HBC 0AF";
$resn_exch{" H31 0AG"}="1H3  0AG"; $resn_exch{" H32 0AG"}="2H3  0AG"; $resn_exch{" H33 0AG"}="3H3  0AG";
$resn_exch{" H41 0AG"}="1H4  0AG"; $resn_exch{" H42 0AG"}="2H4  0AG"; $resn_exch{"HBC1 0AG"}="1HBC 0AG";
$resn_exch{"HBC2 0AG"}="2HBC 0AG"; $resn_exch{"HD11 0AG"}="1HD1 0AG"; $resn_exch{"HD12 0AG"}="2HD1 0AG";
$resn_exch{"HD13 0AG"}="3HD1 0AG"; $resn_exch{"HD21 0AG"}="1HD2 0AG"; $resn_exch{"HD22 0AG"}="2HD2 0AG";
$resn_exch{"HD23 0AG"}="3HD2 0AG"; $resn_exch{" HN1 0AH"}="1HN  0AH"; $resn_exch{" HN2 0AH"}="2HN  0AH";
$resn_exch{"HBC1 0AH"}="1HBC 0AH"; $resn_exch{"HBC2 0AH"}="2HBC 0AH"; $resn_exch{"H2C1 0AH"}="1H2C 0AH";
$resn_exch{"H2C2 0AH"}="2H2C 0AH"; $resn_exch{"H8C1 0AI"}="1H8C 0AI"; $resn_exch{"H8C2 0AI"}="2H8C 0AI";
$resn_exch{"H8C3 0AI"}="3H8C 0AI"; $resn_exch{"HMC1 0AI"}="1HMC 0AI"; $resn_exch{"HMC2 0AI"}="2HMC 0AI";
$resn_exch{"HMC3 0AI"}="3HMC 0AI"; $resn_exch{"HAC1 0AJ"}="1HAC 0AJ"; $resn_exch{"HAC2 0AJ"}="2HAC 0AJ";
$resn_exch{"HAC3 0AJ"}="3HAC 0AJ"; $resn_exch{"HDC1 0AJ"}="1HDC 0AJ"; $resn_exch{"HDC2 0AJ"}="2HDC 0AJ";
$resn_exch{"HDC3 0AJ"}="3HDC 0AJ"; $resn_exch{"H1C1 0AJ"}="1H1C 0AJ"; $resn_exch{"H1C2 0AJ"}="2H1C 0AJ";
$resn_exch{"H2C1 0AJ"}="1H2C 0AJ"; $resn_exch{"H2C2 0AJ"}="2H2C 0AJ"; $resn_exch{"H2C3 0AJ"}="3H2C 0AJ";
$resn_exch{" HN1 0AK"}="1HN  0AK"; $resn_exch{" HN2 0AK"}="2HN  0AK"; $resn_exch{"HBC1 0AK"}="1HBC 0AK";
$resn_exch{"HBC2 0AK"}="2HBC 0AK"; $resn_exch{"H1C1 0AK"}="1H1C 0AK"; $resn_exch{"H1C2 0AK"}="2H1C 0AK";
$resn_exch{"H2C1 0AK"}="1H2C 0AK"; $resn_exch{"H2C2 0AK"}="2H2C 0AK"; $resn_exch{"H7C1 0AL"}="1H7C 0AL";
$resn_exch{"H7C2 0AL"}="2H7C 0AL"; $resn_exch{" HN2 0AO"}="2HN  0AO"; $resn_exch{"H5'1 0AP"}="1H5' 0AP";
$resn_exch{"H5'2 0AP"}="2H5' 0AP"; $resn_exch{"H2'1 0AP"}="1H2' 0AP"; $resn_exch{"H2'2 0AP"}="2H2' 0AP";
$resn_exch{"H4N1 0AP"}="1H4N 0AP"; $resn_exch{"H4N2 0AP"}="2H4N 0AP"; $resn_exch{"H6C1 0AT"}="1H6C 0AT";
$resn_exch{"H6C2 0AT"}="2H6C 0AT"; $resn_exch{"H8C1 0AT"}="1H8C 0AT"; $resn_exch{"H8C2 0AT"}="2H8C 0AT";
$resn_exch{"H8C3 0AT"}="3H8C 0AT"; $resn_exch{"H5'1 0AU"}="1H5' 0AU"; $resn_exch{"H5'2 0AU"}="2H5' 0AU";
$resn_exch{"H5'1 0AV"}="1H5' 0AV"; $resn_exch{" HA2 0AV"}="2HA  0AV"; $resn_exch{"HMC1 0AV"}="1HMC 0AV";
$resn_exch{"HMC2 0AV"}="2HMC 0AV"; $resn_exch{"HMC3 0AV"}="3HMC 0AV"; $resn_exch{"H6N1 0AV"}="1H6N 0AV";
$resn_exch{"H6N2 0AV"}="2H6N 0AV"; $resn_exch{" HB2 0AZ"}=" HB1 0AZ"; $resn_exch{" HB3 0AZ"}=" HB2 0AZ";
$resn_exch{"HD22 0AZ"}="HD1  0AZ"; $resn_exch{"HD23 0AZ"}="HD2  0AZ"; $resn_exch{" HD1 0AZ"}=" HOD 0AZ";
$resn_exch{" H   0CS"}="1HN  0CS"; $resn_exch{" H2  0CS"}="2HN  0CS"; $resn_exch{" HB2 0CS"}="1HB  0CS";
$resn_exch{" HB3 0CS"}="2HB  0CS"; $resn_exch{" HD3 0CS"}="HOD3 0CS"; $resn_exch{"HOP2 101"}="2HOP 101";
$resn_exch{"HOP3 101"}="3HOP 101"; $resn_exch{"HO5' 101"}="HO5* 101"; $resn_exch{"H5'1 101"}="1H5* 101";
$resn_exch{"H5'2 101"}="2H5* 101"; $resn_exch{" H4' 101"}=" H4* 101"; $resn_exch{" H3' 101"}=" H3* 101";
$resn_exch{"H2'1 101"}="1H2* 101"; $resn_exch{"H2'2 101"}="2H2* 101"; $resn_exch{" H1' 101"}=" H1* 101";
$resn_exch{"HN61 101"}="1HN6 101"; $resn_exch{"HN62 101"}="2HN6 101"; $resn_exch{"HOG2 102"}="2HOG 102";
$resn_exch{"HOG3 102"}="3HOG 102"; $resn_exch{"HOB2 102"}="2HOB 102"; $resn_exch{"HOA2 102"}="2HOA 102";
$resn_exch{"H5'1 102"}="1H5* 102"; $resn_exch{"H5'2 102"}="2H5* 102"; $resn_exch{" H4' 102"}=" H4* 102";
$resn_exch{" H3' 102"}=" H3* 102"; $resn_exch{"HO3' 102"}="*HO3 102"; $resn_exch{" H2' 102"}=" H2* 102";
$resn_exch{"HO2' 102"}="*HO2 102"; $resn_exch{" H1' 102"}=" H1* 102"; $resn_exch{"HN21 102"}="1HN2 102";
$resn_exch{"HN22 102"}="2HN2 102"; $resn_exch{"HOP2 103"}="2HOP 103"; $resn_exch{"HOP3 103"}="3HOP 103";
$resn_exch{"H5'1 103"}="1H5* 103"; $resn_exch{"H5'2 103"}="2H5* 103"; $resn_exch{"H5'3 103"}="3H5* 103";
$resn_exch{" H4' 103"}=" H4* 103"; $resn_exch{" H3' 103"}=" H3* 103"; $resn_exch{"H2'1 103"}="1H2* 103";
$resn_exch{"H2'2 103"}="2H2* 103"; $resn_exch{" H1' 103"}=" H1* 103"; $resn_exch{"HN61 103"}="1HN6 103";
$resn_exch{"HN62 103"}="2HN6 103"; $resn_exch{" H11 104"}="1H1  104"; $resn_exch{" H12 104"}="2H1  104";
$resn_exch{" H21 104"}="1H2  104"; $resn_exch{" H22 104"}="2H2  104"; $resn_exch{" H31 104"}="1H3  104";
$resn_exch{" H32 104"}="2H3  104"; $resn_exch{" H51 104"}="1H5  104"; $resn_exch{" H52 104"}="2H5  104";
$resn_exch{" H61 104"}="1H6  104"; $resn_exch{" H62 104"}="2H6  104"; $resn_exch{" H81 104"}="1H8  104";
$resn_exch{" H82 104"}="2H8  104"; $resn_exch{" H91 104"}="1H9  104"; $resn_exch{" H92 104"}="2H9  104";
$resn_exch{"H101 104"}="1H10 104"; $resn_exch{"H102 104"}="2H10 104"; $resn_exch{" H31 105"}="1H3  105";
$resn_exch{" H32 105"}="2H3  105"; $resn_exch{" H91 105"}="1H9  105"; $resn_exch{" H92 105"}="2H9  105";
$resn_exch{" H93 105"}="3H9  105"; $resn_exch{"H301 106"}="1H30 106"; $resn_exch{"H302 106"}="2H30 106";
$resn_exch{"H141 107"}="1H14 107"; $resn_exch{"H142 107"}="2H14 107"; $resn_exch{" H71 108"}="1H7  108";
$resn_exch{" H72 108"}="2H7  108"; $resn_exch{"HN11 108"}="1HN1 108"; $resn_exch{"HN12 108"}="2HN1 108";
$resn_exch{" HB1 108"}="1HB  108"; $resn_exch{" HB2 108"}="2HB  108"; $resn_exch{" HB3 108"}="3HB  108";
$resn_exch{" H71 109"}="1H7  109"; $resn_exch{" H72 109"}="2H7  109"; $resn_exch{"HN11 109"}="1HN1 109";
$resn_exch{"HN12 109"}="2HN1 109"; $resn_exch{" HB1 109"}="1HB  109"; $resn_exch{" HB2 109"}="2HB  109";
$resn_exch{" HB3 109"}="3HB  109"; $resn_exch{" H11 10A"}="1H1  10A"; $resn_exch{" H12 10A"}="2H1  10A";
$resn_exch{" H13 10A"}="3H1  10A"; $resn_exch{" H21 10A"}="1H2  10A"; $resn_exch{" H22 10A"}="2H2  10A";
$resn_exch{" H31 10A"}="1H3  10A"; $resn_exch{" H32 10A"}="2H3  10A"; $resn_exch{" H41 10A"}="1H4  10A";
$resn_exch{" H42 10A"}="2H4  10A"; $resn_exch{" H51 10A"}="1H5  10A"; $resn_exch{" H52 10A"}="2H5  10A";
$resn_exch{" H61 10A"}="1H6  10A"; $resn_exch{" H62 10A"}="2H6  10A"; $resn_exch{" H71 10A"}="1H7  10A";
$resn_exch{" H72 10A"}="2H7  10A"; $resn_exch{" H81 10A"}="1H8  10A"; $resn_exch{" H82 10A"}="2H8  10A";
$resn_exch{" H91 10A"}="1H9  10A"; $resn_exch{" H92 10A"}="2H9  10A"; $resn_exch{"H101 10A"}="1H10 10A";
$resn_exch{"H102 10A"}="2H10 10A"; $resn_exch{"H111 10A"}="1H11 10A"; $resn_exch{"H112 10A"}="2H11 10A";
$resn_exch{"H113 10A"}="3H11 10A"; $resn_exch{"H121 10A"}="1H12 10A"; $resn_exch{"H122 10A"}="2H12 10A";
$resn_exch{"H123 10A"}="3H12 10A"; $resn_exch{"H131 10A"}="1H13 10A"; $resn_exch{"H132 10A"}="2H13 10A";
$resn_exch{"H141 10A"}="1H14 10A"; $resn_exch{"H142 10A"}="2H14 10A"; $resn_exch{"H151 10A"}="1H15 10A";
$resn_exch{"H152 10A"}="2H15 10A"; $resn_exch{"H161 10A"}="1H16 10A"; $resn_exch{"H162 10A"}="2H16 10A";
$resn_exch{"H171 10A"}="1H17 10A"; $resn_exch{"H172 10A"}="2H17 10A"; $resn_exch{"H181 10A"}="1H18 10A";
$resn_exch{"H182 10A"}="2H18 10A"; $resn_exch{"H191 10A"}="1H19 10A"; $resn_exch{"H192 10A"}="2H19 10A";
$resn_exch{"H201 10A"}="1H20 10A"; $resn_exch{"H202 10A"}="2H20 10A"; $resn_exch{"H211 10A"}="1H21 10A";
$resn_exch{"H212 10A"}="2H21 10A"; $resn_exch{"H221 10A"}="1H22 10A"; $resn_exch{"H222 10A"}="2H22 10A";
$resn_exch{"H223 10A"}="3H22 10A"; $resn_exch{"HOP1 10C"}="1HOP 10C"; $resn_exch{"HOP2 10C"}="2HOP 10C";
$resn_exch{"H5'1 10C"}="1H5* 10C"; $resn_exch{"H5'2 10C"}="2H5* 10C"; $resn_exch{" H1' 10C"}=" H1* 10C";
$resn_exch{" H3' 10C"}=" H3* 10C"; $resn_exch{"H6'1 10C"}="1H6* 10C"; $resn_exch{"H6'2 10C"}="2H6* 10C";
$resn_exch{" H41 10C"}="1H4  10C"; $resn_exch{" H42 10C"}="2H4  10C"; $resn_exch{"H2'1 10C"}="1H2* 10C";
$resn_exch{"H101 110"}="1H10 110"; $resn_exch{"H102 110"}="2H10 110"; $resn_exch{"H111 110"}="1H11 110";
$resn_exch{"H112 110"}="2H11 110"; $resn_exch{"H211 110"}="1H21 110"; $resn_exch{"H212 110"}="2H21 110";
$resn_exch{"H221 110"}="1H22 110"; $resn_exch{"H222 110"}="2H22 110"; $resn_exch{"H191 110"}="1H19 110";
$resn_exch{"H192 110"}="2H19 110"; $resn_exch{"H121 110"}="1H12 110"; $resn_exch{"H122 110"}="2H12 110";
$resn_exch{"H131 110"}="1H13 110"; $resn_exch{"H132 110"}="2H13 110"; $resn_exch{"H133 110"}="3H13 110";
$resn_exch{" H41 110"}="1H4  110"; $resn_exch{" H42 110"}="2H4  110"; $resn_exch{" H51 110"}="1H5  110";
$resn_exch{" H52 110"}="2H5  110"; $resn_exch{"HD21 110"}="1HD2 110"; $resn_exch{"HD22 110"}="2HD2 110";
$resn_exch{" H71 110"}="1H7  110"; $resn_exch{" H72 110"}="2H7  110"; $resn_exch{" H81 110"}="1H8  110";
$resn_exch{" H82 110"}="2H8  110"; $resn_exch{"H131 111"}="1H13 111"; $resn_exch{"H132 111"}="2H13 111";
$resn_exch{"H133 111"}="3H13 111"; $resn_exch{"H111 111"}="1H11 111"; $resn_exch{"H112 111"}="2H11 111";
$resn_exch{" H91 111"}="1H9  111"; $resn_exch{" H92 111"}="2H9  111"; $resn_exch{" H71 111"}="1H7  111";
$resn_exch{" H72 111"}="2H7  111"; $resn_exch{"H101 111"}="1H10 111"; $resn_exch{"H102 111"}="2H10 111";
$resn_exch{"H121 111"}="1H12 111"; $resn_exch{"H122 111"}="2H12 111"; $resn_exch{"H141 111"}="1H14 111";
$resn_exch{"H142 111"}="2H14 111"; $resn_exch{"H143 111"}="3H14 111"; $resn_exch{"HNS1 112"}="1HNS 112";
$resn_exch{"HNS2 112"}="2HNS 112"; $resn_exch{"H1S1 112"}="1H1S 112"; $resn_exch{"H1S2 112"}="2H1S 112";
$resn_exch{"HOG2 112"}="2HOG 112"; $resn_exch{"HOB2 112"}="2HOB 112"; $resn_exch{"HOA2 112"}="2HOA 112";
$resn_exch{"H5'1 112"}="1H5* 112"; $resn_exch{"H5'2 112"}="2H5* 112"; $resn_exch{" H4' 112"}=" H4* 112";
$resn_exch{" H3' 112"}=" H3* 112"; $resn_exch{"HO3' 112"}="*HO3 112"; $resn_exch{" H2' 112"}=" H2* 112";
$resn_exch{"HO2' 112"}="*HO2 112"; $resn_exch{" H1' 112"}=" H1* 112"; $resn_exch{"HN61 112"}="1HN6 112";
$resn_exch{"HN62 112"}="2HN6 112"; $resn_exch{"H121 113"}="1H12 113"; $resn_exch{"H122 113"}="2H12 113";
$resn_exch{"H221 113"}="1H22 113"; $resn_exch{"H222 113"}="2H22 113"; $resn_exch{"H223 113"}="3H22 113";
$resn_exch{"H261 113"}="1H26 113"; $resn_exch{"H262 113"}="2H26 113"; $resn_exch{"H263 113"}="3H26 113";
$resn_exch{" H21 114"}="1H2  114"; $resn_exch{" H22 114"}="2H2  114"; $resn_exch{" H41 114"}="1H4  114";
$resn_exch{" H42 114"}="2H4  114"; $resn_exch{" H61 114"}="1H6  114"; $resn_exch{" H62 114"}="2H6  114";
$resn_exch{" H71 114"}="1H7  114"; $resn_exch{" H72 114"}="2H7  114"; $resn_exch{"H161 114"}="1H16 114";
$resn_exch{"H162 114"}="2H16 114"; $resn_exch{"H151 114"}="1H15 114"; $resn_exch{"H152 114"}="2H15 114";
$resn_exch{"H9A1 114"}="1H9A 114"; $resn_exch{"H9A2 114"}="2H9A 114"; $resn_exch{"H9A3 114"}="3H9A 114";
$resn_exch{"H201 114"}="1H20 114"; $resn_exch{"H202 114"}="2H20 114"; $resn_exch{"H211 114"}="1H21 114";
$resn_exch{"H212 114"}="2H21 114"; $resn_exch{"H213 114"}="3H21 114"; $resn_exch{"H221 114"}="1H22 114";
$resn_exch{"H222 114"}="2H22 114"; $resn_exch{"H223 114"}="3H22 114"; $resn_exch{" H21 115"}="1H2  115";
$resn_exch{" H22 115"}="2H2  115"; $resn_exch{" H41 115"}="1H4  115"; $resn_exch{" H42 115"}="2H4  115";
$resn_exch{" H61 115"}="1H6  115"; $resn_exch{" H62 115"}="2H6  115"; $resn_exch{" H71 115"}="1H7  115";
$resn_exch{" H72 115"}="2H7  115"; $resn_exch{"H921 115"}="1H92 115"; $resn_exch{"H922 115"}="2H92 115";
$resn_exch{"H923 115"}="3H92 115"; $resn_exch{"H931 115"}="1H93 115"; $resn_exch{"H932 115"}="2H93 115";
$resn_exch{"H933 115"}="3H93 115"; $resn_exch{"H141 116"}="1H14 116"; $resn_exch{"H142 116"}="2H14 116";
$resn_exch{"H143 116"}="3H14 116"; $resn_exch{"H131 116"}="1H13 116"; $resn_exch{"H132 116"}="2H13 116";
$resn_exch{"H171 116"}="1H17 116"; $resn_exch{"H172 116"}="2H17 116"; $resn_exch{"H173 116"}="3H17 116";
$resn_exch{"H161 116"}="1H16 116"; $resn_exch{"H162 116"}="2H16 116"; $resn_exch{"H163 116"}="3H16 116";
$resn_exch{" H21 116"}="1H2  116"; $resn_exch{" H22 116"}="2H2  116"; $resn_exch{" H41 116"}="1H4  116";
$resn_exch{" H42 116"}="2H4  116"; $resn_exch{" H61 116"}="1H6  116"; $resn_exch{" H62 116"}="2H6  116";
$resn_exch{" H71 116"}="1H7  116"; $resn_exch{" H72 116"}="2H7  116"; $resn_exch{"H921 116"}="1H92 116";
$resn_exch{"H922 116"}="2H92 116"; $resn_exch{"H923 116"}="3H92 116"; $resn_exch{"H931 116"}="1H93 116";
$resn_exch{"H932 116"}="2H93 116"; $resn_exch{"H933 116"}="3H93 116"; $resn_exch{"H21A 117"}="1H2  117";
$resn_exch{"H22A 117"}="2H2  117"; $resn_exch{" H41 117"}="1H4  117"; $resn_exch{" H42 117"}="2H4  117";
$resn_exch{" H61 117"}="1H6  117"; $resn_exch{" H62 117"}="2H6  117"; $resn_exch{" H71 117"}="1H7  117";
$resn_exch{" H72 117"}="2H7  117"; $resn_exch{"H931 117"}="1H93 117"; $resn_exch{"H932 117"}="2H93 117";
$resn_exch{"H933 117"}="3H93 117"; $resn_exch{"H921 117"}="1H92 117"; $resn_exch{"H922 117"}="2H92 117";
$resn_exch{"H923 117"}="3H92 117"; $resn_exch{"H351 11B"}="1H35 11B"; $resn_exch{"H352 11B"}="2H35 11B";
$resn_exch{"H261 11B"}="1H26 11B"; $resn_exch{"H262 11B"}="2H26 11B"; $resn_exch{"H271 11B"}="1H27 11B";
$resn_exch{"H272 11B"}="2H27 11B"; $resn_exch{"H281 11B"}="1H28 11B"; $resn_exch{"H282 11B"}="2H28 11B";
$resn_exch{"H291 11B"}="1H29 11B"; $resn_exch{"H292 11B"}="2H29 11B"; $resn_exch{"H301 11B"}="1H30 11B";
$resn_exch{"H302 11B"}="2H30 11B"; $resn_exch{"H303 11B"}="3H30 11B"; $resn_exch{"H111 11B"}="1H11 11B";
$resn_exch{"H112 11B"}="2H11 11B"; $resn_exch{" H21 11O"}="1H2  11O"; $resn_exch{" H22 11O"}="2H2  11O";
$resn_exch{" H31 11O"}="1H3  11O"; $resn_exch{" H32 11O"}="2H3  11O"; $resn_exch{" H41 11O"}="1H4  11O";
$resn_exch{" H42 11O"}="2H4  11O"; $resn_exch{" H51 11O"}="1H5  11O"; $resn_exch{" H52 11O"}="2H5  11O";
$resn_exch{" H61 11O"}="1H6  11O"; $resn_exch{" H62 11O"}="2H6  11O"; $resn_exch{" H71 11O"}="1H7  11O";
$resn_exch{" H72 11O"}="2H7  11O"; $resn_exch{" H81 11O"}="1H8  11O"; $resn_exch{" H82 11O"}="2H8  11O";
$resn_exch{" H91 11O"}="1H9  11O"; $resn_exch{"H101 11O"}="1H10 11O"; $resn_exch{"H111 11O"}="1H11 11O";
$resn_exch{"H121 11O"}="1H12 11O"; $resn_exch{"H131 11O"}="1H13 11O"; $resn_exch{"H141 11O"}="1H14 11O";
$resn_exch{"H142 11O"}="2H14 11O"; $resn_exch{"H151 11O"}="1H15 11O"; $resn_exch{"H152 11O"}="2H15 11O";
$resn_exch{"H161 11O"}="1H16 11O"; $resn_exch{"H162 11O"}="2H16 11O"; $resn_exch{"H171 11O"}="1H17 11O";
$resn_exch{"H172 11O"}="2H17 11O"; $resn_exch{"H181 11O"}="1H18 11O"; $resn_exch{"H182 11O"}="2H18 11O";
$resn_exch{"H183 11O"}="3H18 11O"; $resn_exch{"H2O1 11O"}="1H2O 11O"; $resn_exch{"H211 11O"}="1H21 11O";
$resn_exch{"H161 11P"}="1H16 11P"; $resn_exch{"H162 11P"}="2H16 11P"; $resn_exch{"H171 11P"}="1H17 11P";
$resn_exch{"H172 11P"}="2H17 11P"; $resn_exch{"HM23 11R"}="3HM2 11R"; $resn_exch{"HM22 11R"}="2HM2 11R";
$resn_exch{"HM21 11R"}="1HM2 11R"; $resn_exch{"HM13 11R"}="3HM1 11R"; $resn_exch{"HM12 11R"}="2HM1 11R";
$resn_exch{"HM11 11R"}="1HM1 11R"; $resn_exch{"HX61 11R"}="1HX6 11R"; $resn_exch{"HX62 11R"}="2HX6 11R";
$resn_exch{"HX51 11R"}="1HX5 11R"; $resn_exch{"HX52 11R"}="2HX5 11R"; $resn_exch{"HX41 11R"}="1HX4 11R";
$resn_exch{"HX42 11R"}="2HX4 11R"; $resn_exch{"HX31 11R"}="1HX3 11R"; $resn_exch{"HX32 11R"}="2HX3 11R";
$resn_exch{"HX21 11R"}="1HX2 11R"; $resn_exch{"HX22 11R"}="2HX2 11R"; $resn_exch{"HL61 11R"}="1HL6 11R";
$resn_exch{"HL62 11R"}="2HL6 11R"; $resn_exch{"HL51 11R"}="1HL5 11R"; $resn_exch{"HL52 11R"}="2HL5 11R";
$resn_exch{"HL41 11R"}="1HL4 11R"; $resn_exch{"HL42 11R"}="2HL4 11R"; $resn_exch{"HL31 11R"}="1HL3 11R";
$resn_exch{"HL32 11R"}="2HL3 11R"; $resn_exch{"HL21 11R"}="1HL2 11R"; $resn_exch{"HL22 11R"}="2HL2 11R";
$resn_exch{"HL11 11R"}="1HL1 11R"; $resn_exch{"HL12 11R"}="2HL1 11R"; $resn_exch{"HAN1 11R"}="1HAN 11R";
$resn_exch{"HCK1 11R"}="1HCK 11R"; $resn_exch{"HCJ1 11R"}="1HCJ 11R"; $resn_exch{"HCE1 11R"}="1HCE 11R";
$resn_exch{"HCD1 11R"}="1HCD 11R"; $resn_exch{"HCC1 11R"}="1HCC 11R"; $resn_exch{"HCB1 11R"}="1HCB 11R";
$resn_exch{"HBK1 11R"}="1HBK 11R"; $resn_exch{"HBJ1 11R"}="1HBJ 11R"; $resn_exch{"HBI1 11R"}="1HBI 11R";
$resn_exch{"HBH1 11R"}="1HBH 11R"; $resn_exch{"HBE1 11R"}="1HBE 11R"; $resn_exch{"HBD1 11R"}="1HBD 11R";
$resn_exch{"HBC1 11R"}="1HBC 11R"; $resn_exch{"HBB1 11R"}="1HBB 11R"; $resn_exch{"HH11 120"}="1HH1 120";
$resn_exch{"HH12 120"}="2HH1 120"; $resn_exch{"HH21 120"}="1HH2 120"; $resn_exch{"HH22 120"}="2HH2 120";
$resn_exch{"HH11 121"}="1HH1 121"; $resn_exch{"HH12 121"}="2HH1 121"; $resn_exch{"HH21 121"}="1HH2 121";
$resn_exch{"HH22 121"}="2HH2 121"; $resn_exch{"HH11 122"}="1HH1 122"; $resn_exch{"HH12 122"}="2HH1 122";
$resn_exch{"HH21 122"}="1HH2 122"; $resn_exch{"HH22 122"}="2HH2 122"; $resn_exch{"HH11 123"}="1HH1 123";
$resn_exch{"HH12 123"}="2HH1 123"; $resn_exch{"HH21 123"}="1HH2 123"; $resn_exch{"HH22 123"}="2HH2 123";
$resn_exch{"H3'1 123"}="1H3' 123"; $resn_exch{"H3'2 123"}="2H3' 123"; $resn_exch{"H3'3 123"}="3H3' 123";
$resn_exch{"HH11 124"}="1HH1 124"; $resn_exch{"HH12 124"}="2HH1 124"; $resn_exch{"HH21 124"}="1HH2 124";
$resn_exch{"HH22 124"}="2HH2 124"; $resn_exch{"HOP2 125"}="2HOP 125"; $resn_exch{"HOP3 125"}="3HOP 125";
$resn_exch{" H5' 125"}="1H5* 125"; $resn_exch{"H5'' 125"}="2H5* 125"; $resn_exch{" H4' 125"}=" H4* 125";
$resn_exch{" H3' 125"}=" H3* 125"; $resn_exch{"HO3' 125"}="H3T  125"; $resn_exch{" H2' 125"}=" H2* 125";
$resn_exch{" H1' 125"}=" H1* 125"; $resn_exch{" H71 125"}="1H5M 125"; $resn_exch{" H72 125"}="2H5M 125";
$resn_exch{" H73 125"}="3H5M 125"; $resn_exch{"HA'1 125"}="1HA* 125"; $resn_exch{"HA'2 125"}="2HA* 125";
$resn_exch{"HB'1 125"}="1HB* 125"; $resn_exch{"HB'2 125"}="2HB* 125"; $resn_exch{"HOP2 126"}="2HOP 126";
$resn_exch{"HOP3 126"}="3HOP 126"; $resn_exch{" H5' 126"}="1H5* 126"; $resn_exch{"H5'' 126"}="2H5* 126";
$resn_exch{" H4' 126"}=" H4* 126"; $resn_exch{" H3' 126"}=" H3* 126"; $resn_exch{"HO3' 126"}="H3T  126";
$resn_exch{" H2' 126"}=" H2* 126"; $resn_exch{" H1' 126"}=" H1* 126"; $resn_exch{" H71 126"}="1H5M 126";
$resn_exch{" H72 126"}="2H5M 126"; $resn_exch{" H73 126"}="3H5M 126"; $resn_exch{"HA'1 126"}="1HA* 126";
$resn_exch{"HA'2 126"}="2HA* 126"; $resn_exch{"HB'1 126"}="1HB* 126"; $resn_exch{"HB'2 126"}="2HB* 126";
$resn_exch{"HD'1 126"}="1HD* 126"; $resn_exch{"HD'2 126"}="2HD* 126"; $resn_exch{"HE'1 126"}="1HE* 126";
$resn_exch{"HE'2 126"}="2HE* 126"; $resn_exch{"HG'1 126"}="1HG* 126"; $resn_exch{"HG'2 126"}="2HG* 126";
$resn_exch{"HH'1 126"}="1HH* 126"; $resn_exch{"HH'2 126"}="2HH* 126"; $resn_exch{"HK'1 126"}="1HK* 126";
$resn_exch{"HK'2 126"}="2HK* 126"; $resn_exch{"HK'3 126"}="3HK* 126"; $resn_exch{"HOP2 127"}="2HOP 127";
$resn_exch{"HOP3 127"}="3HOP 127"; $resn_exch{" H5' 127"}="1H5* 127"; $resn_exch{"H5'' 127"}="2H5* 127";
$resn_exch{" H4' 127"}=" H4* 127"; $resn_exch{" H3' 127"}=" H3* 127"; $resn_exch{"HO3' 127"}="H3T  127";
$resn_exch{" H2' 127"}=" H2* 127"; $resn_exch{" H1' 127"}=" H1* 127"; $resn_exch{" H71 127"}="1H5M 127";
$resn_exch{" H72 127"}="2H5M 127"; $resn_exch{" H73 127"}="3H5M 127"; $resn_exch{"HA'1 127"}="1HA* 127";
$resn_exch{"HA'2 127"}="2HA* 127"; $resn_exch{"HB'1 127"}="1HB* 127"; $resn_exch{"HB'2 127"}="2HB* 127";
$resn_exch{"HD'1 127"}="1HD* 127"; $resn_exch{"HD'2 127"}="2HD* 127"; $resn_exch{"HOG2 128"}="2HOG 128";
$resn_exch{"HOG3 128"}="3HOG 128"; $resn_exch{"HOB2 128"}="2HOB 128"; $resn_exch{"HOA2 128"}="2HOA 128";
$resn_exch{" H5' 128"}="1H5* 128"; $resn_exch{"H5'' 128"}="2H5* 128"; $resn_exch{" H4' 128"}=" H4* 128";
$resn_exch{" H3' 128"}=" H3* 128"; $resn_exch{" H2' 128"}=" H2* 128"; $resn_exch{" H1' 128"}=" H1* 128";
$resn_exch{"HN61 128"}="1HN6 128"; $resn_exch{"HN62 128"}="2HN6 128"; $resn_exch{"HOP2 129"}="2HOP 129";
$resn_exch{"HOP3 129"}="3HOP 129"; $resn_exch{" H41 129"}="1H4  129"; $resn_exch{" H42 129"}="2H4  129";
$resn_exch{" H31 129"}="1H3  129"; $resn_exch{" H32 129"}="2H3  129"; $resn_exch{"HOP3 12A"}="3HOP 12A";
$resn_exch{"HOP2 12A"}="2HOP 12A"; $resn_exch{" H5' 12A"}="1H5* 12A"; $resn_exch{"H5'' 12A"}="2H5* 12A";
$resn_exch{" H4' 12A"}=" H4* 12A"; $resn_exch{" H3' 12A"}=" H3* 12A"; $resn_exch{"HO3' 12A"}="H3T  12A";
$resn_exch{" H2' 12A"}=" H2* 12A"; $resn_exch{"HO2' 12A"}="2HO* 12A"; $resn_exch{" H1' 12A"}=" H1* 12A";
$resn_exch{"H2M1 12A"}="1H2M 12A"; $resn_exch{"H2M2 12A"}="2H2M 12A"; $resn_exch{"H2M3 12A"}="3H2M 12A";
$resn_exch{"HG21 12A"}="1HG2 12A"; $resn_exch{"HG22 12A"}="2HG2 12A"; $resn_exch{"HG23 12A"}="3HG2 12A";
$resn_exch{"H181 12C"}="1H18 12C"; $resn_exch{"H182 12C"}="2H18 12C"; $resn_exch{"H191 12C"}="1H19 12C";
$resn_exch{"H192 12C"}="2H19 12C"; $resn_exch{"H201 12C"}="1H20 12C"; $resn_exch{"H202 12C"}="2H20 12C";
$resn_exch{"H211 12C"}="1H21 12C"; $resn_exch{"H212 12C"}="2H21 12C"; $resn_exch{"H221 12C"}="1H22 12C";
$resn_exch{"H222 12C"}="2H22 12C"; $resn_exch{" H11 12H"}="1H1  12H"; $resn_exch{" H12 12H"}="2H1  12H";
$resn_exch{" H21 12H"}="1H2  12H"; $resn_exch{" H22 12H"}="2H2  12H"; $resn_exch{" H31 12H"}="1H3  12H";
$resn_exch{" H32 12H"}="2H3  12H"; $resn_exch{" H41 12H"}="1H4  12H"; $resn_exch{" H42 12H"}="2H4  12H";
$resn_exch{" H51 12H"}="1H5  12H"; $resn_exch{" H52 12H"}="2H5  12H"; $resn_exch{" H61 12H"}="1H6  12H";
$resn_exch{" H62 12H"}="2H6  12H"; $resn_exch{" H71 12H"}="1H7  12H"; $resn_exch{" H72 12H"}="2H7  12H";
$resn_exch{" H81 12H"}="1H8  12H"; $resn_exch{" H82 12H"}="2H8  12H"; $resn_exch{" H91 12H"}="1H9  12H";
$resn_exch{" H92 12H"}="2H9  12H"; $resn_exch{"H101 12H"}="1H10 12H"; $resn_exch{"H102 12H"}="2H10 12H";
$resn_exch{"H111 12H"}="1H11 12H"; $resn_exch{"H112 12H"}="2H11 12H"; $resn_exch{"HAH1 12M"}="1HAH 12M";
$resn_exch{"HAH2 12M"}="2HAH 12M"; $resn_exch{"HAH3 12M"}="3HAH 12M"; $resn_exch{"HAD1 12M"}="1HAD 12M";
$resn_exch{"HAD2 12M"}="2HAD 12M"; $resn_exch{"HAF1 12M"}="1HAF 12M"; $resn_exch{"HAF2 12M"}="2HAF 12M";
$resn_exch{"H361 12P"}="1H36 12P"; $resn_exch{"H362 12P"}="2H36 12P"; $resn_exch{"H351 12P"}="1H35 12P";
$resn_exch{"H352 12P"}="2H35 12P"; $resn_exch{"H331 12P"}="1H33 12P"; $resn_exch{"H332 12P"}="2H33 12P";
$resn_exch{"H321 12P"}="1H32 12P"; $resn_exch{"H322 12P"}="2H32 12P"; $resn_exch{"H301 12P"}="1H30 12P";
$resn_exch{"H302 12P"}="2H30 12P"; $resn_exch{"H291 12P"}="1H29 12P"; $resn_exch{"H292 12P"}="2H29 12P";
$resn_exch{"H271 12P"}="1H27 12P"; $resn_exch{"H272 12P"}="2H27 12P"; $resn_exch{"H261 12P"}="1H26 12P";
$resn_exch{"H262 12P"}="2H26 12P"; $resn_exch{"H241 12P"}="1H24 12P"; $resn_exch{"H242 12P"}="2H24 12P";
$resn_exch{"H231 12P"}="1H23 12P"; $resn_exch{"H232 12P"}="2H23 12P"; $resn_exch{"H211 12P"}="1H21 12P";
$resn_exch{"H212 12P"}="2H21 12P"; $resn_exch{"H201 12P"}="1H20 12P"; $resn_exch{"H202 12P"}="2H20 12P";
$resn_exch{"H181 12P"}="1H18 12P"; $resn_exch{"H182 12P"}="2H18 12P"; $resn_exch{"H171 12P"}="1H17 12P";
$resn_exch{"H172 12P"}="2H17 12P"; $resn_exch{"H151 12P"}="1H15 12P"; $resn_exch{"H152 12P"}="2H15 12P";
$resn_exch{"H141 12P"}="1H14 12P"; $resn_exch{"H142 12P"}="2H14 12P"; $resn_exch{"H121 12P"}="1H12 12P";
$resn_exch{"H122 12P"}="2H12 12P"; $resn_exch{"H111 12P"}="1H11 12P"; $resn_exch{"H112 12P"}="2H11 12P";
$resn_exch{" H91 12P"}="1H9  12P"; $resn_exch{" H92 12P"}="2H9  12P"; $resn_exch{" H81 12P"}="1H8  12P";
$resn_exch{" H82 12P"}="2H8  12P"; $resn_exch{" H61 12P"}="1H6  12P"; $resn_exch{" H62 12P"}="2H6  12P";
$resn_exch{" H51 12P"}="1H5  12P"; $resn_exch{" H52 12P"}="2H5  12P"; $resn_exch{" H31 12P"}="1H3  12P";
$resn_exch{" H32 12P"}="2H3  12P"; $resn_exch{" H21 12P"}="1H2  12P"; $resn_exch{" H22 12P"}="2H2  12P";
$resn_exch{"HAH1 12Q"}="1HAH 12Q"; $resn_exch{"HAH2 12Q"}="2HAH 12Q"; $resn_exch{"HAH3 12Q"}="3HAH 12Q";
$resn_exch{"HH11 130"}="1HH1 130"; $resn_exch{"HH12 130"}="2HH1 130"; $resn_exch{"HH21 130"}="1HH2 130";
$resn_exch{"HC'2 130"}="2HC' 130"; $resn_exch{"HC'3 130"}="3HC' 130"; $resn_exch{"HC'4 130"}="4HC' 130";
$resn_exch{" H2' 130"}=" H2* 130"; $resn_exch{" H3' 130"}=" H3* 130"; $resn_exch{" H4' 130"}=" H4* 130";
$resn_exch{" H5' 130"}=" H5* 130"; $resn_exch{" H6' 130"}=" H6* 130"; $resn_exch{"HH11 132"}="1HH1 132";
$resn_exch{"HH12 132"}="2HH1 132"; $resn_exch{"HH21 132"}="1HH2 132"; $resn_exch{"HH22 132"}="2HH2 132";
$resn_exch{"HC'2 132"}="2HC' 132"; $resn_exch{"HC'3 132"}="3HC' 132"; $resn_exch{"HC'4 132"}="4HC' 132";
$resn_exch{" H2' 132"}=" H2* 132"; $resn_exch{" H3' 132"}=" H3* 132"; $resn_exch{" H4' 132"}=" H4* 132";
$resn_exch{" H5' 132"}=" H5* 132"; $resn_exch{" H6' 132"}=" H6* 132"; $resn_exch{"HH11 133"}="1HH1 133";
$resn_exch{"HH12 133"}="2HH1 133"; $resn_exch{"HH21 133"}="1HH2 133"; $resn_exch{"HC'2 133"}="2HC' 133";
$resn_exch{"HC'3 133"}="3HC' 133"; $resn_exch{"HC'4 133"}="4HC' 133"; $resn_exch{"HO'6 133"}="6HO' 133";
$resn_exch{"H101 133"}="1H10 133"; $resn_exch{"H102 133"}="2H10 133"; $resn_exch{"H301 133"}="1H30 133";
$resn_exch{"H302 133"}="2H30 133"; $resn_exch{"H303 133"}="3H30 133"; $resn_exch{"H401 133"}="1H40 133";
$resn_exch{"H402 133"}="2H40 133"; $resn_exch{"H403 133"}="3H40 133"; $resn_exch{"HH11 134"}="1HH1 134";
$resn_exch{"HH12 134"}="2HH1 134"; $resn_exch{"HH21 134"}="1HH2 134"; $resn_exch{"HC'2 134"}="2HC' 134";
$resn_exch{"HC'3 134"}="3HC' 134"; $resn_exch{"HC'4 134"}="4HC' 134"; $resn_exch{"HO'6 134"}="6HO' 134";
$resn_exch{" H1' 134"}=" H1* 134"; $resn_exch{"H2'1 134"}="1H2* 134"; $resn_exch{"H2'2 134"}="2H2* 134";
$resn_exch{"H3'1 134"}="1H3* 134"; $resn_exch{"H3'2 134"}="2H3* 134"; $resn_exch{"H4'1 134"}="1H4* 134";
$resn_exch{"H4'2 134"}="2H4* 134"; $resn_exch{"H5'1 134"}="1H5* 134"; $resn_exch{"H5'2 134"}="2H5* 134";
$resn_exch{" H6' 134"}=" H6* 134"; $resn_exch{" HM1 134"}="1HM  134"; $resn_exch{" HM2 134"}="2HM  134";
$resn_exch{" HM3 134"}="3HM  134"; $resn_exch{"HH11 135"}="1HH1 135"; $resn_exch{"HH12 135"}="2HH1 135";
$resn_exch{"HH21 135"}="1HH2 135"; $resn_exch{"HH11 136"}="1HH1 136"; $resn_exch{"HH12 136"}="2HH1 136";
$resn_exch{"HH21 136"}="1HH2 136"; $resn_exch{"H211 136"}="1H21 136"; $resn_exch{"H212 136"}="2H21 136";
$resn_exch{"H213 136"}="3H21 136"; $resn_exch{"HOP2 137"}="2HOP 137"; $resn_exch{"HOP3 137"}="3HOP 137";
$resn_exch{"H5'1 137"}="1H5' 137"; $resn_exch{"H5'2 137"}="2H5' 137"; $resn_exch{"HO'4 137"}="4HO' 137";
$resn_exch{"HO'3 137"}="3HO' 137"; $resn_exch{"HO'2 137"}="2HO' 137"; $resn_exch{"H1'1 137"}="1H1' 137";
$resn_exch{"H1'2 137"}="2H1' 137"; $resn_exch{"HA21 138"}="1HA2 138"; $resn_exch{"HA22 138"}="2HA2 138";
$resn_exch{" H91 138"}="1H9  138"; $resn_exch{" H92 138"}="2H9  138"; $resn_exch{"HA11 138"}="1HA1 138";
$resn_exch{"HA12 138"}="2HA1 138"; $resn_exch{" HB1 138"}="1HB  138"; $resn_exch{" HB2 138"}="2HB  138";
$resn_exch{" HG1 138"}="1HG  138"; $resn_exch{" HG2 138"}="2HG  138"; $resn_exch{"H271 138"}="1H27 138";
$resn_exch{"H272 138"}="2H27 138"; $resn_exch{"H191 138"}="1H19 138"; $resn_exch{"H192 138"}="2H19 138";
$resn_exch{"HOA2 139"}="2HOA 139"; $resn_exch{"HOB2 139"}="2HOB 139"; $resn_exch{"H6A1 139"}="1H6A 139";
$resn_exch{"H6A2 139"}="2H6A 139"; $resn_exch{"HA51 139"}="1HA5 139"; $resn_exch{"HA52 139"}="2HA5 139";
$resn_exch{" H4D 139"}="H4*A 139"; $resn_exch{" H1D 139"}="H1*A 139"; $resn_exch{" H2D 139"}="H2*A 139";
$resn_exch{" HD2 139"}="2H*A 139"; $resn_exch{" H3D 139"}="H3*A 139"; $resn_exch{" HD3 139"}="3H*A 139";
$resn_exch{" HB5 139"}="5H*U 139"; $resn_exch{"HU51 139"}="1HU5 139"; $resn_exch{"HU52 139"}="2HU5 139";
$resn_exch{" H4B 139"}="H4*U 139"; $resn_exch{" H1B 139"}="H1*U 139"; $resn_exch{"HU21 139"}="1HU2 139";
$resn_exch{"HU22 139"}="2HU2 139"; $resn_exch{" H3B 139"}="H3*U 139"; $resn_exch{" H81 13A"}="1H8  13A";
$resn_exch{" H82 13A"}="2H8  13A"; $resn_exch{"HAC1 13D"}="1HAC 13D"; $resn_exch{"HAC2 13D"}="2HAC 13D";
$resn_exch{"HBC1 13D"}="1HBC 13D"; $resn_exch{"HBC2 13D"}="2HBC 13D"; $resn_exch{"HCC1 13D"}="1HCC 13D";
$resn_exch{"HCC2 13D"}="2HCC 13D"; $resn_exch{"HDN1 13D"}="1HDN 13D"; $resn_exch{"HDN2 13D"}="2HDN 13D";
$resn_exch{"HAA1 13D"}="1HAA 13D"; $resn_exch{"HAA2 13D"}="2HAA 13D"; $resn_exch{"HOP2 13P"}="2HOP 13P";
$resn_exch{"HOP3 13P"}="3HOP 13P"; $resn_exch{" H11 13P"}="1H1  13P"; $resn_exch{" H12 13P"}="2H1  13P";
$resn_exch{" H31 13P"}="1H3  13P"; $resn_exch{" H32 13P"}="2H3  13P"; $resn_exch{" H21 13R"}="1H2  13R";
$resn_exch{" H22 13R"}="2H2  13R"; $resn_exch{" H31 13R"}="1H3  13R"; $resn_exch{" H32 13R"}="2H3  13R";
$resn_exch{" H41 13R"}="1H4  13R"; $resn_exch{" H42 13R"}="2H4  13R"; $resn_exch{" H51 13R"}="1H5  13R";
$resn_exch{" H52 13R"}="2H5  13R"; $resn_exch{" H61 13R"}="1H6  13R"; $resn_exch{" H62 13R"}="2H6  13R";
$resn_exch{" H71 13R"}="1H7  13R"; $resn_exch{" H72 13R"}="2H7  13R"; $resn_exch{" H81 13R"}="1H8  13R";
$resn_exch{" H82 13R"}="2H8  13R"; $resn_exch{" H91 13R"}="1H9  13R"; $resn_exch{"H101 13R"}="1H10 13R";
$resn_exch{"H111 13R"}="1H11 13R"; $resn_exch{"H121 13R"}="1H12 13R"; $resn_exch{"H131 13R"}="1H13 13R";
$resn_exch{"H141 13R"}="1H14 13R"; $resn_exch{"H142 13R"}="2H14 13R"; $resn_exch{"H151 13R"}="1H15 13R";
$resn_exch{"H152 13R"}="2H15 13R"; $resn_exch{"H161 13R"}="1H16 13R"; $resn_exch{"H162 13R"}="2H16 13R";
$resn_exch{"H171 13R"}="1H17 13R"; $resn_exch{"H172 13R"}="2H17 13R"; $resn_exch{"H181 13R"}="1H18 13R";
$resn_exch{"H182 13R"}="2H18 13R"; $resn_exch{"H183 13R"}="3H18 13R"; $resn_exch{"H201 13R"}="1H20 13R";
$resn_exch{"H221 13R"}="1H22 13R"; $resn_exch{" H21 13S"}="1H2  13S"; $resn_exch{" H22 13S"}="2H2  13S";
$resn_exch{" H31 13S"}="1H3  13S"; $resn_exch{" H32 13S"}="2H3  13S"; $resn_exch{" H41 13S"}="1H4  13S";
$resn_exch{" H42 13S"}="2H4  13S"; $resn_exch{" H51 13S"}="1H5  13S"; $resn_exch{" H52 13S"}="2H5  13S";
$resn_exch{" H61 13S"}="1H6  13S"; $resn_exch{" H62 13S"}="2H6  13S"; $resn_exch{" H71 13S"}="1H7  13S";
$resn_exch{" H72 13S"}="2H7  13S"; $resn_exch{" H81 13S"}="1H8  13S"; $resn_exch{" H82 13S"}="2H8  13S";
$resn_exch{" H91 13S"}="1H9  13S"; $resn_exch{"H101 13S"}="1H10 13S"; $resn_exch{"H111 13S"}="1H11 13S";
$resn_exch{"H121 13S"}="1H12 13S"; $resn_exch{"H131 13S"}="1H13 13S"; $resn_exch{"H141 13S"}="1H14 13S";
$resn_exch{"H142 13S"}="2H14 13S"; $resn_exch{"H151 13S"}="1H15 13S"; $resn_exch{"H152 13S"}="2H15 13S";
$resn_exch{"H161 13S"}="1H16 13S"; $resn_exch{"H162 13S"}="2H16 13S"; $resn_exch{"H171 13S"}="1H17 13S";
$resn_exch{"H172 13S"}="2H17 13S"; $resn_exch{"H181 13S"}="1H18 13S"; $resn_exch{"H182 13S"}="2H18 13S";
$resn_exch{"H183 13S"}="3H18 13S"; $resn_exch{"H201 13S"}="1H20 13S"; $resn_exch{"H221 13S"}="1H22 13S";
$resn_exch{"H121 13T"}="1H12 13T"; $resn_exch{"H122 13T"}="2H12 13T"; $resn_exch{"H131 13T"}="1H13 13T";
$resn_exch{"H132 13T"}="2H13 13T"; $resn_exch{"H231 13T"}="1H23 13T"; $resn_exch{"H232 13T"}="2H23 13T";
$resn_exch{"H233 13T"}="3H23 13T"; $resn_exch{"H241 13T"}="1H24 13T"; $resn_exch{"H242 13T"}="2H24 13T";
$resn_exch{"H243 13T"}="3H24 13T"; $resn_exch{"H251 13T"}="1H25 13T"; $resn_exch{"H252 13T"}="2H25 13T";
$resn_exch{"H253 13T"}="3H25 13T"; $resn_exch{"H261 13T"}="1H26 13T"; $resn_exch{"H262 13T"}="2H26 13T";
$resn_exch{"H263 13T"}="3H26 13T"; $resn_exch{"H271 13T"}="1H27 13T"; $resn_exch{"H272 13T"}="2H27 13T";
$resn_exch{"H273 13T"}="3H27 13T"; $resn_exch{"H281 13T"}="1H28 13T"; $resn_exch{"H282 13T"}="2H28 13T";
$resn_exch{"H283 13T"}="3H28 13T"; $resn_exch{"H291 13T"}="1H29 13T"; $resn_exch{"H292 13T"}="2H29 13T";
$resn_exch{"H301 13T"}="1H30 13T"; $resn_exch{"H302 13T"}="2H30 13T"; $resn_exch{"H303 13T"}="3H30 13T";
$resn_exch{"H311 13T"}="1H31 13T"; $resn_exch{"H312 13T"}="2H31 13T"; $resn_exch{"H313 13T"}="3H31 13T";
$resn_exch{"H321 13T"}="1H32 13T"; $resn_exch{"H322 13T"}="2H32 13T"; $resn_exch{"H323 13T"}="3H32 13T";
$resn_exch{" H21 140"}="1H2  140"; $resn_exch{" H22 140"}="2H2  140"; $resn_exch{" H41 140"}="1H4  140";
$resn_exch{" H42 140"}="2H4  140"; $resn_exch{" H51 140"}="1H5  140"; $resn_exch{" H52 140"}="2H5  140";
$resn_exch{" H61 140"}="1H6  140"; $resn_exch{" H62 140"}="2H6  140"; $resn_exch{" H71 140"}="1H7  140";
$resn_exch{" H72 140"}="2H7  140"; $resn_exch{" H81 140"}="1H8  140"; $resn_exch{" H82 140"}="2H8  140";
$resn_exch{" H91 140"}="1H9  140"; $resn_exch{" H92 140"}="2H9  140"; $resn_exch{"H101 140"}="1H10 140";
$resn_exch{"H102 140"}="2H10 140"; $resn_exch{"H111 140"}="1H11 140"; $resn_exch{"H112 140"}="2H11 140";
$resn_exch{"H121 140"}="1H12 140"; $resn_exch{"H122 140"}="2H12 140"; $resn_exch{"H131 140"}="1H13 140";
$resn_exch{"H132 140"}="2H13 140"; $resn_exch{"H141 140"}="1H14 140"; $resn_exch{"H142 140"}="2H14 140";
$resn_exch{"H151 140"}="1H15 140"; $resn_exch{"H152 140"}="2H15 140"; $resn_exch{"H161 140"}="1H16 140";
$resn_exch{"H162 140"}="2H16 140"; $resn_exch{"H171 140"}="1H17 140"; $resn_exch{"H172 140"}="2H17 140";
$resn_exch{"H181 140"}="1H18 140"; $resn_exch{"H182 140"}="2H18 140"; $resn_exch{"H183 140"}="3H18 140";
$resn_exch{" HN1 142"}="1HN  142"; $resn_exch{" HN2 142"}="2HN  142"; $resn_exch{" HB1 142"}="1HB  142";
$resn_exch{" HB2 142"}="2HB  142"; $resn_exch{"HB11 142"}="1HB1 142"; $resn_exch{"HB12 142"}="2HB1 142";
$resn_exch{"HB13 142"}="3HB1 142"; $resn_exch{" H2  143"}=" HN2 143"; $resn_exch{" HB2 143"}="1HB  143";
$resn_exch{" HB3 143"}="2HB  143"; $resn_exch{" HI1 143"}="1HI  143"; $resn_exch{" HI2 143"}="2HI  143";
$resn_exch{" H11 144"}="1H1  144"; $resn_exch{" H12 144"}="2H1  144"; $resn_exch{" H13 144"}="3H1  144";
$resn_exch{" H21 144"}="1H2  144"; $resn_exch{" H22 144"}="2H2  144"; $resn_exch{" H31 144"}="1H3  144";
$resn_exch{" H32 144"}="2H3  144"; $resn_exch{" H41 144"}="1H4  144"; $resn_exch{" H42 144"}="2H4  144";
$resn_exch{" H61 145"}="1H6  145"; $resn_exch{" H62 145"}="2H6  145"; $resn_exch{"H21A 146"}="1H2  146";
$resn_exch{" H22 146"}="2H2  146"; $resn_exch{"H71A 146"}="1H7  146"; $resn_exch{" H72 146"}="2H7  146";
$resn_exch{"H311 146"}="1H31 146"; $resn_exch{"H312 146"}="2H31 146"; $resn_exch{"H611 146"}="1H61 146";
$resn_exch{"H612 146"}="2H61 146"; $resn_exch{"HN77 146"}="7HN7 146"; $resn_exch{" H61 147"}="1H6  147";
$resn_exch{" H62 147"}="2H6  147"; $resn_exch{" H21 148"}="1H2  148"; $resn_exch{" H22 148"}="2H2  148";
$resn_exch{" H31 148"}="1H3  148"; $resn_exch{" H32 148"}="2H3  148"; $resn_exch{" H41 148"}="1H4  148";
$resn_exch{" H42 148"}="2H4  148"; $resn_exch{" H51 148"}="1H5  148"; $resn_exch{" H52 148"}="2H5  148";
$resn_exch{" H61 148"}="1H6  148"; $resn_exch{" H62 148"}="2H6  148"; $resn_exch{" H71 148"}="1H7  148";
$resn_exch{" H72 148"}="2H7  148"; $resn_exch{" H81 148"}="1H8  148"; $resn_exch{" H82 148"}="2H8  148";
$resn_exch{" H61 149"}="1H6  149"; $resn_exch{" H62 149"}="2H6  149"; $resn_exch{"HN31 14A"}="1HN3 14A";
$resn_exch{"HN32 14A"}="2HN3 14A"; $resn_exch{" H61 14A"}="1H6  14A"; $resn_exch{" H62 14A"}="2H6  14A";
$resn_exch{" H71 14A"}="1H7  14A"; $resn_exch{" H72 14A"}="2H7  14A"; $resn_exch{" H81 14A"}="1H8  14A";
$resn_exch{" H82 14A"}="2H8  14A"; $resn_exch{"H201 14A"}="1H20 14A"; $resn_exch{"H202 14A"}="2H20 14A";
$resn_exch{"H221 14A"}="1H22 14A"; $resn_exch{"H222 14A"}="2H22 14A"; $resn_exch{"H241 14A"}="1H24 14A";
$resn_exch{"H242 14A"}="2H24 14A"; $resn_exch{"H251 14A"}="1H25 14A"; $resn_exch{"H252 14A"}="2H25 14A";
$resn_exch{"H261 14A"}="1H26 14A"; $resn_exch{"H262 14A"}="2H26 14A"; $resn_exch{"H271 14A"}="1H27 14A";
$resn_exch{"H272 14A"}="2H27 14A"; $resn_exch{" H71 14W"}="1H7  14W"; $resn_exch{" H72 14W"}="2H7  14W";
$resn_exch{"H101 14W"}="1H10 14W"; $resn_exch{"H102 14W"}="2H10 14W"; $resn_exch{"H103 14W"}="3H10 14W";
$resn_exch{"H121 14W"}="1H12 14W"; $resn_exch{"H122 14W"}="2H12 14W"; $resn_exch{"H131 14W"}="1H13 14W";
$resn_exch{"H132 14W"}="2H13 14W"; $resn_exch{"HN11 150"}="1HN1 150"; $resn_exch{"HN12 150"}="2HN1 150";
$resn_exch{"HN21 150"}="1HN2 150"; $resn_exch{"HN22 150"}="2HN2 150"; $resn_exch{" H71 150"}="1H7  150";
$resn_exch{" H72 150"}="2H7  150"; $resn_exch{" H73 150"}="3H7  150"; $resn_exch{" H81 150"}="1H8  150";
$resn_exch{" H82 150"}="2H8  150"; $resn_exch{" H83 150"}="3H8  150"; $resn_exch{"H261 151"}="1H26 151";
$resn_exch{"H262 151"}="2H26 151"; $resn_exch{"H281 151"}="1H28 151"; $resn_exch{"H282 151"}="2H28 151";
$resn_exch{"H3C1 151"}="1H3C 151"; $resn_exch{"H3C2 151"}="2H3C 151"; $resn_exch{"H3C3 151"}="3H3C 151";
$resn_exch{"H1C1 151"}="1H1C 151"; $resn_exch{"H1C2 151"}="2H1C 151"; $resn_exch{"H1C3 151"}="3H1C 151";
$resn_exch{"H171 151"}="1H17 151"; $resn_exch{"H172 151"}="2H17 151"; $resn_exch{"H173 151"}="3H17 151";
$resn_exch{"H9C1 151"}="1H9C 151"; $resn_exch{"H9C2 151"}="2H9C 151"; $resn_exch{"H111 151"}="1H11 151";
$resn_exch{"H112 151"}="2H11 151"; $resn_exch{"H131 151"}="1H13 151"; $resn_exch{"H132 151"}="2H13 151";
$resn_exch{"H141 151"}="1H14 151"; $resn_exch{"H142 151"}="2H14 151"; $resn_exch{"H121 151"}="1H12 151";
$resn_exch{"H122 151"}="2H12 151"; $resn_exch{" H21 152"}="1H2  152"; $resn_exch{" H22 152"}="2H2  152";
$resn_exch{" H41 152"}="1H4  152"; $resn_exch{" H42 152"}="2H4  152"; $resn_exch{"H5A1 152"}="1H5A 152";
$resn_exch{"H5A2 152"}="2H5A 152"; $resn_exch{"H5A3 152"}="3H5A 152"; $resn_exch{"H5B1 152"}="1H5B 152";
$resn_exch{"H5B2 152"}="2H5B 152"; $resn_exch{"H5B3 152"}="3H5B 152"; $resn_exch{"H5C1 152"}="1H5C 152";
$resn_exch{"H5C2 152"}="2H5C 152"; $resn_exch{"H5C3 152"}="3H5C 152"; $resn_exch{"H161 153"}="1H16 153";
$resn_exch{"H162 153"}="2H16 153"; $resn_exch{"H231 153"}="1H23 153"; $resn_exch{"H232 153"}="2H23 153";
$resn_exch{"H161 154"}="1H16 154"; $resn_exch{"H162 154"}="2H16 154"; $resn_exch{"H231 154"}="1H23 154";
$resn_exch{"H232 154"}="2H23 154"; $resn_exch{"H191 155"}="1H19 155"; $resn_exch{"H192 155"}="2H19 155";
$resn_exch{"H321 155"}="1H32 155"; $resn_exch{"H322 155"}="2H32 155"; $resn_exch{"H331 155"}="1H33 155";
$resn_exch{"H332 155"}="2H33 155"; $resn_exch{"H411 155"}="1H41 155"; $resn_exch{"H412 155"}="2H41 155";
$resn_exch{"H413 155"}="3H41 155"; $resn_exch{"H451 155"}="1H45 155"; $resn_exch{"H452 155"}="2H45 155";
$resn_exch{"H453 155"}="3H45 155"; $resn_exch{"H151 156"}="1H15 156"; $resn_exch{"H152 156"}="2H15 156";
$resn_exch{"H161 156"}="1H16 156"; $resn_exch{"H162 156"}="2H16 156"; $resn_exch{"H211 156"}="1H21 156";
$resn_exch{"H212 156"}="2H21 156"; $resn_exch{"H213 156"}="3H21 156"; $resn_exch{"H221 156"}="1H22 156";
$resn_exch{"H222 156"}="2H22 156"; $resn_exch{"H223 156"}="3H22 156"; $resn_exch{"H231 156"}="1H23 156";
$resn_exch{"H232 156"}="2H23 156"; $resn_exch{"H233 156"}="3H23 156"; $resn_exch{"H241 156"}="1H24 156";
$resn_exch{"H242 156"}="2H24 156"; $resn_exch{"H243 156"}="3H24 156"; $resn_exch{" H11 157"}="1H1  157";
$resn_exch{" H12 157"}="2H1  157"; $resn_exch{" H21 157"}="1H2  157"; $resn_exch{" H22 157"}="2H2  157";
$resn_exch{" H31 157"}="1H3  157"; $resn_exch{" H32 157"}="2H3  157"; $resn_exch{" H41 157"}="1H4  157";
$resn_exch{" H42 157"}="2H4  157"; $resn_exch{"HN61 157"}="1HN6 157"; $resn_exch{"HN62 157"}="2HN6 157";
$resn_exch{"H111 157"}="1H11 157"; $resn_exch{"H112 157"}="2H11 157"; $resn_exch{"H121 157"}="1H12 157";
$resn_exch{"H122 157"}="2H12 157"; $resn_exch{"H141 157"}="1H14 157"; $resn_exch{"H142 157"}="2H14 157";
$resn_exch{" H61 158"}="1H6  158"; $resn_exch{" H62 158"}="2H6  158"; $resn_exch{" H11 158"}="1H1  158";
$resn_exch{"H12A 158"}="2H1  158"; $resn_exch{" H13 158"}="3H1  158"; $resn_exch{" H91 158"}="1H9  158";
$resn_exch{" H92 158"}="2H9  158"; $resn_exch{" H51 159"}="1H5  159"; $resn_exch{" H52 159"}="2H5  159";
$resn_exch{" H41 159"}="1H4  159"; $resn_exch{" H42 159"}="2H4  159"; $resn_exch{"H6C1 15A"}="1H6C 15A";
$resn_exch{"H6C2 15A"}="2H6C 15A"; $resn_exch{"H7C1 15A"}="1H7C 15A"; $resn_exch{"H7C2 15A"}="2H7C 15A";
$resn_exch{"H8C1 15A"}="1H8C 15A"; $resn_exch{"H8C2 15A"}="2H8C 15A"; $resn_exch{"H2N1 15A"}="1H2N 15A";
$resn_exch{"H2N2 15A"}="2H2N 15A"; $resn_exch{" H61 15B"}="1H6  15B"; $resn_exch{" H62 15B"}="2H6  15B";
$resn_exch{"H191 15B"}="1H19 15B"; $resn_exch{"H192 15B"}="2H19 15B"; $resn_exch{"H201 15B"}="1H20 15B";
$resn_exch{"H202 15B"}="2H20 15B"; $resn_exch{"H211 15B"}="1H21 15B"; $resn_exch{"H212 15B"}="2H21 15B";
$resn_exch{"H231 15B"}="1H23 15B"; $resn_exch{"H232 15B"}="2H23 15B"; $resn_exch{"H241 15B"}="1H24 15B";
$resn_exch{"H242 15B"}="2H24 15B"; $resn_exch{"H261 15B"}="1H26 15B"; $resn_exch{"H262 15B"}="2H26 15B";
$resn_exch{"H271 15B"}="1H27 15B"; $resn_exch{"H272 15B"}="2H27 15B"; $resn_exch{"H281 15B"}="1H28 15B";
$resn_exch{"H282 15B"}="2H28 15B"; $resn_exch{"H351 15B"}="1H35 15B"; $resn_exch{"H352 15B"}="2H35 15B";
$resn_exch{"H361 15B"}="1H36 15B"; $resn_exch{"H362 15B"}="2H36 15B"; $resn_exch{"H371 15B"}="1H37 15B";
$resn_exch{"H372 15B"}="2H37 15B"; $resn_exch{"H21A 15M"}="1H2  15M"; $resn_exch{"H22A 15M"}="2H2  15M";
$resn_exch{" H31 15M"}="1H3  15M"; $resn_exch{" H32 15M"}="2H3  15M"; $resn_exch{" H41 15M"}="1H4  15M";
$resn_exch{" H42 15M"}="2H4  15M"; $resn_exch{" H71 15M"}="1H7  15M"; $resn_exch{" H72 15M"}="2H7  15M";
$resn_exch{"H101 15M"}="1H10 15M"; $resn_exch{"H102 15M"}="2H10 15M"; $resn_exch{"H161 15M"}="1H16 15M";
$resn_exch{"H162 15M"}="2H16 15M"; $resn_exch{"H171 15M"}="1H17 15M"; $resn_exch{"H172 15M"}="2H17 15M";
$resn_exch{"H241 15M"}="1H24 15M"; $resn_exch{"H242 15M"}="2H24 15M"; $resn_exch{"H243 15M"}="3H24 15M";
$resn_exch{"H251 15M"}="1H25 15M"; $resn_exch{"H252 15M"}="2H25 15M"; $resn_exch{"H101 15P"}="1H10 15P";
$resn_exch{"H102 15P"}="2H10 15P"; $resn_exch{"H111 15P"}="1H11 15P"; $resn_exch{"H112 15P"}="2H11 15P";
$resn_exch{"H121 15P"}="1H12 15P"; $resn_exch{"H122 15P"}="2H12 15P"; $resn_exch{"H131 15P"}="1H13 15P";
$resn_exch{"H132 15P"}="2H13 15P"; $resn_exch{"H141 15P"}="1H14 15P"; $resn_exch{"H142 15P"}="2H14 15P";
$resn_exch{"H151 15P"}="1H15 15P"; $resn_exch{"H152 15P"}="2H15 15P"; $resn_exch{"H161 15P"}="1H16 15P";
$resn_exch{"H162 15P"}="2H16 15P"; $resn_exch{"H171 15P"}="1H17 15P"; $resn_exch{"H172 15P"}="2H17 15P";
$resn_exch{"H181 15P"}="1H18 15P"; $resn_exch{"H182 15P"}="2H18 15P"; $resn_exch{"H191 15P"}="1H19 15P";
$resn_exch{"H192 15P"}="2H19 15P"; $resn_exch{"H201 15P"}="1H20 15P"; $resn_exch{"H202 15P"}="2H20 15P";
$resn_exch{"H211 15P"}="1H21 15P"; $resn_exch{"H212 15P"}="2H21 15P"; $resn_exch{"H221 15P"}="1H22 15P";
$resn_exch{"H222 15P"}="2H22 15P"; $resn_exch{"H231 15P"}="1H23 15P"; $resn_exch{"H232 15P"}="2H23 15P";
$resn_exch{"H241 15P"}="1H24 15P"; $resn_exch{"H242 15P"}="2H24 15P"; $resn_exch{"H251 15P"}="1H25 15P";
$resn_exch{"H252 15P"}="2H25 15P"; $resn_exch{"H261 15P"}="1H26 15P"; $resn_exch{"H262 15P"}="2H26 15P";
$resn_exch{"H271 15P"}="1H27 15P"; $resn_exch{"H272 15P"}="2H27 15P"; $resn_exch{"H281 15P"}="1H28 15P";
$resn_exch{"H282 15P"}="2H28 15P"; $resn_exch{"H291 15P"}="1H29 15P"; $resn_exch{"H292 15P"}="2H29 15P";
$resn_exch{"H301 15P"}="1H30 15P"; $resn_exch{"H302 15P"}="2H30 15P"; $resn_exch{"H311 15P"}="1H31 15P";
$resn_exch{"H312 15P"}="2H31 15P"; $resn_exch{"H321 15P"}="1H32 15P"; $resn_exch{"H322 15P"}="2H32 15P";
$resn_exch{"H331 15P"}="1H33 15P"; $resn_exch{"H332 15P"}="2H33 15P"; $resn_exch{"H341 15P"}="1H34 15P";
$resn_exch{"H342 15P"}="2H34 15P"; $resn_exch{"H351 15P"}="1H35 15P"; $resn_exch{"H352 15P"}="2H35 15P";
$resn_exch{"H361 15P"}="1H36 15P"; $resn_exch{"H362 15P"}="2H36 15P"; $resn_exch{"H371 15P"}="1H37 15P";
$resn_exch{"H372 15P"}="2H37 15P"; $resn_exch{"H381 15P"}="1H38 15P"; $resn_exch{"H382 15P"}="2H38 15P";
$resn_exch{"H391 15P"}="1H39 15P"; $resn_exch{"H392 15P"}="2H39 15P"; $resn_exch{"H401 15P"}="1H40 15P";
$resn_exch{"H402 15P"}="2H40 15P"; $resn_exch{"H411 15P"}="1H41 15P"; $resn_exch{"H412 15P"}="2H41 15P";
$resn_exch{"H421 15P"}="1H42 15P"; $resn_exch{"H422 15P"}="2H42 15P"; $resn_exch{"H431 15P"}="1H43 15P";
$resn_exch{"H432 15P"}="2H43 15P"; $resn_exch{"H441 15P"}="1H44 15P"; $resn_exch{"H442 15P"}="2H44 15P";
$resn_exch{"H451 15P"}="1H45 15P"; $resn_exch{"H452 15P"}="2H45 15P"; $resn_exch{"H461 15P"}="1H46 15P";
$resn_exch{"H462 15P"}="2H46 15P"; $resn_exch{"H471 15P"}="1H47 15P"; $resn_exch{"H472 15P"}="2H47 15P";
$resn_exch{"H481 15P"}="1H48 15P"; $resn_exch{"H482 15P"}="2H48 15P"; $resn_exch{"H491 15P"}="1H49 15P";
$resn_exch{"H492 15P"}="2H49 15P"; $resn_exch{"H501 15P"}="1H50 15P"; $resn_exch{"H502 15P"}="2H50 15P";
$resn_exch{"H511 15P"}="1H51 15P"; $resn_exch{"H512 15P"}="2H51 15P"; $resn_exch{"H521 15P"}="1H52 15P";
$resn_exch{"H522 15P"}="2H52 15P"; $resn_exch{"H531 15P"}="1H53 15P"; $resn_exch{"H532 15P"}="2H53 15P";
$resn_exch{"H541 15P"}="1H54 15P"; $resn_exch{"H542 15P"}="2H54 15P"; $resn_exch{"H551 15P"}="1H55 15P";
$resn_exch{"H552 15P"}="2H55 15P"; $resn_exch{"H561 15P"}="1H56 15P"; $resn_exch{"H562 15P"}="2H56 15P";
$resn_exch{"H571 15P"}="1H57 15P"; $resn_exch{"H572 15P"}="2H57 15P"; $resn_exch{"H581 15P"}="1H58 15P";
$resn_exch{"H582 15P"}="2H58 15P"; $resn_exch{"H591 15P"}="1H59 15P"; $resn_exch{"H592 15P"}="2H59 15P";
$resn_exch{"H601 15P"}="1H60 15P"; $resn_exch{"H602 15P"}="2H60 15P"; $resn_exch{"H611 15P"}="1H61 15P";
$resn_exch{"H612 15P"}="2H61 15P"; $resn_exch{"H621 15P"}="1H62 15P"; $resn_exch{"H622 15P"}="2H62 15P";
$resn_exch{"H631 15P"}="1H63 15P"; $resn_exch{"H632 15P"}="2H63 15P"; $resn_exch{"H641 15P"}="1H64 15P";
$resn_exch{"H642 15P"}="2H64 15P"; $resn_exch{"H651 15P"}="1H65 15P"; $resn_exch{"H652 15P"}="2H65 15P";
$resn_exch{"H661 15P"}="1H66 15P"; $resn_exch{"H662 15P"}="2H66 15P"; $resn_exch{"H671 15P"}="1H67 15P";
$resn_exch{"H672 15P"}="2H67 15P"; $resn_exch{"H681 15P"}="1H68 15P"; $resn_exch{"H682 15P"}="2H68 15P";
$resn_exch{"HCM1 15P"}="1HCM 15P"; $resn_exch{"HCM2 15P"}="2HCM 15P"; $resn_exch{"HCM3 15P"}="3HCM 15P";
$resn_exch{"H251 160"}="1H25 160"; $resn_exch{"H252 160"}="2H25 160"; $resn_exch{"H253 160"}="3H25 160";
$resn_exch{"H161 160"}="1H16 160"; $resn_exch{"H162 160"}="2H16 160"; $resn_exch{"H151 160"}="1H15 160";
$resn_exch{"H152 160"}="2H15 160"; $resn_exch{"H121 160"}="1H12 160"; $resn_exch{"H122 160"}="2H12 160";
$resn_exch{"H11A 160"}="1H1  160"; $resn_exch{" H12 160"}="2H1  160"; $resn_exch{" H13 160"}="3H1  160";
$resn_exch{"H291 160"}="1H29 160"; $resn_exch{"H292 160"}="2H29 160"; $resn_exch{"H11A 161"}="1H1  161";
$resn_exch{" H12 161"}="2H1  161"; $resn_exch{"H13A 161"}="3H1  161"; $resn_exch{"H251 161"}="1H25 161";
$resn_exch{"H252 161"}="2H25 161"; $resn_exch{"H121 161"}="1H12 161"; $resn_exch{"H122 161"}="2H12 161";
$resn_exch{"H11A 162"}="1H1  162"; $resn_exch{" H12 162"}="2H1  162"; $resn_exch{" H21 162"}="1H2  162";
$resn_exch{" H22 162"}="2H2  162"; $resn_exch{" H23 162"}="3H2  162"; $resn_exch{" H61 162"}="1H6  162";
$resn_exch{" H62 162"}="2H6  162"; $resn_exch{"H141 162"}="1H14 162"; $resn_exch{"H142 162"}="2H14 162";
$resn_exch{"H161 162"}="1H16 162"; $resn_exch{"H162 162"}="2H16 162"; $resn_exch{"H171 162"}="1H17 162";
$resn_exch{"H172 162"}="2H17 162"; $resn_exch{"H181 162"}="1H18 162"; $resn_exch{"H182 162"}="2H18 162";
$resn_exch{"H191 162"}="1H19 162"; $resn_exch{"H192 162"}="2H19 162"; $resn_exch{"H201 162"}="1H20 162";
$resn_exch{"H202 162"}="2H20 162"; $resn_exch{"H211 162"}="1H21 162"; $resn_exch{"H212 162"}="2H21 162";
$resn_exch{"H11A 163"}="1H1  163"; $resn_exch{" H12 163"}="2H1  163"; $resn_exch{" H21 163"}="1H2  163";
$resn_exch{" H22 163"}="2H2  163"; $resn_exch{" H23 163"}="3H2  163"; $resn_exch{" H61 163"}="1H6  163";
$resn_exch{" H62 163"}="2H6  163"; $resn_exch{"H141 163"}="1H14 163"; $resn_exch{"H142 163"}="2H14 163";
$resn_exch{"H161 163"}="1H16 163"; $resn_exch{"H162 163"}="2H16 163"; $resn_exch{"H171 163"}="1H17 163";
$resn_exch{"H172 163"}="2H17 163"; $resn_exch{"H181 163"}="1H18 163"; $resn_exch{"H182 163"}="2H18 163";
$resn_exch{"H191 163"}="1H19 163"; $resn_exch{"H192 163"}="2H19 163"; $resn_exch{"H201 163"}="1H20 163";
$resn_exch{"H202 163"}="2H20 163"; $resn_exch{"H211 163"}="1H21 163"; $resn_exch{"H212 163"}="2H21 163";
$resn_exch{" H91 164"}="1H9  164"; $resn_exch{" H92 164"}="2H9  164"; $resn_exch{"H101 164"}="1H10 164";
$resn_exch{"H102 164"}="2H10 164"; $resn_exch{"H221 165"}="1H22 165"; $resn_exch{"H222 165"}="2H22 165";
$resn_exch{"H211 165"}="1H21 165"; $resn_exch{"H212 165"}="2H21 165"; $resn_exch{"H201 165"}="1H20 165";
$resn_exch{"H202 165"}="2H20 165"; $resn_exch{"H351 165"}="1H35 165"; $resn_exch{"H352 165"}="2H35 165";
$resn_exch{" H71 165"}="1H7  165"; $resn_exch{" H72 165"}="2H7  165"; $resn_exch{"H111 165"}="1H11 165";
$resn_exch{"H112 165"}="2H11 165"; $resn_exch{"H131 165"}="1H13 165"; $resn_exch{"H132 165"}="2H13 165";
$resn_exch{"H141 165"}="1H14 165"; $resn_exch{"H142 165"}="2H14 165"; $resn_exch{"H151 165"}="1H15 165";
$resn_exch{"H152 165"}="2H15 165"; $resn_exch{"H161 165"}="1H16 165"; $resn_exch{"H162 165"}="2H16 165";
$resn_exch{"H171 165"}="1H17 165"; $resn_exch{"H172 165"}="2H17 165"; $resn_exch{" H12 166"}="2H1  166";
$resn_exch{" H11 166"}="1H1  166"; $resn_exch{" H22 166"}="2H2  166"; $resn_exch{" H21 166"}="1H2  166";
$resn_exch{" H32 166"}="2H3  166"; $resn_exch{" H31 166"}="1H3  166"; $resn_exch{" H42 166"}="2H4  166";
$resn_exch{" H41 166"}="1H4  166"; $resn_exch{" H62 166"}="2H6  166"; $resn_exch{" H61 166"}="1H6  166";
$resn_exch{"H142 166"}="2H14 166"; $resn_exch{"H141 166"}="1H14 166"; $resn_exch{" H12 167"}="2H1  167";
$resn_exch{" H11 167"}="1H1  167"; $resn_exch{" H22 167"}="2H2  167"; $resn_exch{" H21 167"}="1H2  167";
$resn_exch{" H32 167"}="2H3  167"; $resn_exch{" H31 167"}="1H3  167"; $resn_exch{" H42 167"}="2H4  167";
$resn_exch{" H41 167"}="1H4  167"; $resn_exch{" H62 167"}="2H6  167"; $resn_exch{" H61 167"}="1H6  167";
$resn_exch{"H142 167"}="2H14 167"; $resn_exch{"H141 167"}="1H14 167"; $resn_exch{"H533 167"}="3H53 167";
$resn_exch{"H532 167"}="2H53 167"; $resn_exch{"H531 167"}="1H53 167"; $resn_exch{" H21 168"}="1H2  168";
$resn_exch{" H22 168"}="2H2  168"; $resn_exch{" H23 168"}="3H2  168"; $resn_exch{"HP21 168"}="1HP2 168";
$resn_exch{"HP22 168"}="2HP2 168"; $resn_exch{"HP31 168"}="1HP3 168"; $resn_exch{"HP32 168"}="2HP3 168";
$resn_exch{"HP61 168"}="1HP6 168"; $resn_exch{"HP62 168"}="2HP6 168"; $resn_exch{"HP71 168"}="1HP7 168";
$resn_exch{"HP72 168"}="2HP7 168"; $resn_exch{"HPA0 168"}="0HPA 168"; $resn_exch{"HPD1 168"}="1HPD 168";
$resn_exch{"HPD2 168"}="2HPD 168"; $resn_exch{"HPD3 168"}="3HPD 168"; $resn_exch{"HPE1 168"}="1HPE 168";
$resn_exch{"HPE2 168"}="2HPE 168"; $resn_exch{"HPE3 168"}="3HPE 168"; $resn_exch{"HPC1 168"}="1HPC 168";
$resn_exch{"HPC2 168"}="2HPC 168"; $resn_exch{"HT31 168"}="1HT3 168"; $resn_exch{"HT32 168"}="2HT3 168";
$resn_exch{"HT33 168"}="3HT3 168"; $resn_exch{"HT41 168"}="1HT4 168"; $resn_exch{"HT42 168"}="2HT4 168";
$resn_exch{"HT43 168"}="3HT4 168"; $resn_exch{"HT51 168"}="1HT5 168"; $resn_exch{"HT52 168"}="2HT5 168";
$resn_exch{"HT53 168"}="3HT5 168"; $resn_exch{"H11A 169"}="1H1  169"; $resn_exch{"H12A 169"}="2H1  169";
$resn_exch{" H13 169"}="3H1  169"; $resn_exch{" H31 169"}="1H3  169"; $resn_exch{" H32 169"}="2H3  169";
$resn_exch{" H33 169"}="3H3  169"; $resn_exch{" H41 169"}="1H4  169"; $resn_exch{" H42 169"}="2H4  169";
$resn_exch{" H43 169"}="3H4  169"; $resn_exch{" H51 169"}="1H5  169"; $resn_exch{" H52 169"}="2H5  169";
$resn_exch{" H61 169"}="1H6  169"; $resn_exch{" H62 169"}="2H6  169"; $resn_exch{" H71 169"}="1H7  169";
$resn_exch{" H72 169"}="2H7  169"; $resn_exch{" H81 169"}="1H8  169"; $resn_exch{" H82 169"}="2H8  169";
$resn_exch{"H131 169"}="1H13 169"; $resn_exch{"H132 169"}="2H13 169"; $resn_exch{"H141 169"}="1H14 169";
$resn_exch{"H142 169"}="2H14 169"; $resn_exch{"H151 169"}="1H15 169"; $resn_exch{"H152 169"}="2H15 169";
$resn_exch{"HN51 169"}="1HN5 169"; $resn_exch{"HN55 169"}="5HN5 169"; $resn_exch{" H11 16A"}="1H1  16A";
$resn_exch{" H12 16A"}="2H1  16A"; $resn_exch{" H13 16A"}="3H1  16A"; $resn_exch{" H21 16A"}="1H2  16A";
$resn_exch{" H22 16A"}="2H2  16A"; $resn_exch{" H31 16A"}="1H3  16A"; $resn_exch{" H32 16A"}="2H3  16A";
$resn_exch{" H41 16A"}="1H4  16A"; $resn_exch{" H42 16A"}="2H4  16A"; $resn_exch{" H51 16A"}="1H5  16A";
$resn_exch{" H52 16A"}="2H5  16A"; $resn_exch{" H61 16A"}="1H6  16A"; $resn_exch{" H62 16A"}="2H6  16A";
$resn_exch{" H71 16A"}="1H7  16A"; $resn_exch{" H72 16A"}="2H7  16A"; $resn_exch{" H81 16A"}="1H8  16A";
$resn_exch{" H82 16A"}="2H8  16A"; $resn_exch{" H91 16A"}="1H9  16A"; $resn_exch{" H92 16A"}="2H9  16A";
$resn_exch{"H101 16A"}="1H10 16A"; $resn_exch{"H102 16A"}="2H10 16A"; $resn_exch{"H111 16A"}="1H11 16A";
$resn_exch{"H112 16A"}="2H11 16A"; $resn_exch{"H121 16A"}="1H12 16A"; $resn_exch{"H122 16A"}="2H12 16A";
$resn_exch{"H131 16A"}="1H13 16A"; $resn_exch{"H132 16A"}="2H13 16A"; $resn_exch{"H141 16A"}="1H14 16A";
$resn_exch{"H142 16A"}="2H14 16A"; $resn_exch{"H151 16A"}="1H15 16A"; $resn_exch{"H152 16A"}="2H15 16A";
$resn_exch{"H161 16A"}="1H16 16A"; $resn_exch{"H162 16A"}="2H16 16A"; $resn_exch{"H171 16A"}="1H17 16A";
$resn_exch{"H172 16A"}="2H17 16A"; $resn_exch{"H173 16A"}="3H17 16A"; $resn_exch{"H181 16A"}="1H18 16A";
$resn_exch{"H182 16A"}="2H18 16A"; $resn_exch{"H183 16A"}="3H18 16A"; $resn_exch{"H191 16A"}="1H19 16A";
$resn_exch{"H192 16A"}="2H19 16A"; $resn_exch{"H193 16A"}="3H19 16A"; $resn_exch{" H11 16C"}="1H1  16C";
$resn_exch{" H12 16C"}="2H1  16C"; $resn_exch{" H61 16C"}="1H6  16C"; $resn_exch{" H62 16C"}="2H6  16C";
$resn_exch{" H71 16C"}="1H7  16C"; $resn_exch{" H72 16C"}="2H7  16C"; $resn_exch{" H81 16C"}="1H8  16C";
$resn_exch{" H82 16C"}="2H8  16C"; $resn_exch{" H91 16C"}="1H9  16C"; $resn_exch{" H92 16C"}="2H9  16C";
$resn_exch{"H101 16C"}="1H10 16C"; $resn_exch{"H102 16C"}="2H10 16C"; $resn_exch{"H111 16C"}="1H11 16C";
$resn_exch{"H112 16C"}="2H11 16C"; $resn_exch{"H121 16C"}="1H12 16C"; $resn_exch{"H122 16C"}="2H12 16C";
$resn_exch{"H131 16C"}="1H13 16C"; $resn_exch{"H132 16C"}="2H13 16C"; $resn_exch{"H141 16C"}="1H14 16C";
$resn_exch{"H142 16C"}="2H14 16C"; $resn_exch{"H151 16C"}="1H15 16C"; $resn_exch{"H152 16C"}="2H15 16C";
$resn_exch{"H161 16C"}="1H16 16C"; $resn_exch{"H162 16C"}="2H16 16C"; $resn_exch{"H171 16C"}="1H17 16C";
$resn_exch{"H172 16C"}="2H17 16C"; $resn_exch{"H181 16C"}="1H18 16C"; $resn_exch{"H182 16C"}="2H18 16C";
$resn_exch{"H183 16C"}="3H18 16C"; $resn_exch{"H201 16C"}="1H20 16C"; $resn_exch{"H202 16C"}="2H20 16C";
$resn_exch{"H211 16C"}="1H21 16C"; $resn_exch{"H212 16C"}="2H21 16C"; $resn_exch{"H221 16C"}="1H22 16C";
$resn_exch{"H222 16C"}="2H22 16C"; $resn_exch{"H231 16C"}="1H23 16C"; $resn_exch{"H232 16C"}="2H23 16C";
$resn_exch{"H241 16C"}="1H24 16C"; $resn_exch{"H242 16C"}="2H24 16C"; $resn_exch{"H251 16C"}="1H25 16C";
$resn_exch{"H252 16C"}="2H25 16C"; $resn_exch{"H261 16C"}="1H26 16C"; $resn_exch{"H262 16C"}="2H26 16C";
$resn_exch{"H271 16C"}="1H27 16C"; $resn_exch{"H272 16C"}="2H27 16C"; $resn_exch{"H281 16C"}="1H28 16C";
$resn_exch{"H282 16C"}="2H28 16C"; $resn_exch{"H291 16C"}="1H29 16C"; $resn_exch{"H292 16C"}="2H29 16C";
$resn_exch{"H301 16C"}="1H30 16C"; $resn_exch{"H302 16C"}="2H30 16C"; $resn_exch{"H311 16C"}="1H31 16C";
$resn_exch{"H312 16C"}="2H31 16C"; $resn_exch{"H321 16C"}="1H32 16C"; $resn_exch{"H322 16C"}="2H32 16C";
$resn_exch{"H331 16C"}="1H33 16C"; $resn_exch{"H332 16C"}="2H33 16C"; $resn_exch{"H341 16C"}="1H34 16C";
$resn_exch{"H342 16C"}="2H34 16C"; $resn_exch{"H343 16C"}="3H34 16C"; $resn_exch{"HN11 16D"}="1HN1 16D";
$resn_exch{"HN12 16D"}="2HN1 16D"; $resn_exch{" H11 16D"}="1H1  16D"; $resn_exch{" H12 16D"}="2H1  16D";
$resn_exch{" H21 16D"}="1H2  16D"; $resn_exch{" H22 16D"}="2H2  16D"; $resn_exch{" H31 16D"}="1H3  16D";
$resn_exch{" H32 16D"}="2H3  16D"; $resn_exch{" H41 16D"}="1H4  16D"; $resn_exch{" H42 16D"}="2H4  16D";
$resn_exch{" H51 16D"}="1H5  16D"; $resn_exch{" H52 16D"}="2H5  16D"; $resn_exch{" H61 16D"}="1H6  16D";
$resn_exch{" H62 16D"}="2H6  16D"; $resn_exch{"HN21 16D"}="1HN2 16D"; $resn_exch{"HN22 16D"}="2HN2 16D";
$resn_exch{" H61 16G"}="1H6  16G"; $resn_exch{" H62 16G"}="2H6  16G"; $resn_exch{" H81 16G"}="1H8  16G";
$resn_exch{" H82 16G"}="2H8  16G"; $resn_exch{" H83 16G"}="3H8  16G"; $resn_exch{"HOP2 16G"}="2HOP 16G";
$resn_exch{"HOP3 16G"}="3HOP 16G"; $resn_exch{" H11 16P"}="1H1  16P"; $resn_exch{" H12 16P"}="2H1  16P";
$resn_exch{" H13 16P"}="3H1  16P"; $resn_exch{" H21 16P"}="1H2  16P"; $resn_exch{" H22 16P"}="2H2  16P";
$resn_exch{" H31 16P"}="1H3  16P"; $resn_exch{" H32 16P"}="2H3  16P"; $resn_exch{" H41 16P"}="1H4  16P";
$resn_exch{" H42 16P"}="2H4  16P"; $resn_exch{" H51 16P"}="1H5  16P"; $resn_exch{" H52 16P"}="2H5  16P";
$resn_exch{" H61 16P"}="1H6  16P"; $resn_exch{" H62 16P"}="2H6  16P"; $resn_exch{" H71 16P"}="1H7  16P";
$resn_exch{" H72 16P"}="2H7  16P"; $resn_exch{" H81 16P"}="1H8  16P"; $resn_exch{" H82 16P"}="2H8  16P";
$resn_exch{" H91 16P"}="1H9  16P"; $resn_exch{" H92 16P"}="2H9  16P"; $resn_exch{"H101 16P"}="1H10 16P";
$resn_exch{"H102 16P"}="2H10 16P"; $resn_exch{"H111 16P"}="1H11 16P"; $resn_exch{"H112 16P"}="2H11 16P";
$resn_exch{"H121 16P"}="1H12 16P"; $resn_exch{"H122 16P"}="2H12 16P"; $resn_exch{"H131 16P"}="1H13 16P";
$resn_exch{"H132 16P"}="2H13 16P"; $resn_exch{"H141 16P"}="1H14 16P"; $resn_exch{"H142 16P"}="2H14 16P";
$resn_exch{"H143 16P"}="3H14 16P"; $resn_exch{" H11 170"}="1H1  170"; $resn_exch{" H12 170"}="2H1  170";
$resn_exch{"H331 170"}="1H33 170"; $resn_exch{"H332 170"}="2H33 170"; $resn_exch{"H391 170"}="1H39 170";
$resn_exch{"H392 170"}="2H39 170"; $resn_exch{" H11 171"}="1H1  171"; $resn_exch{" H12 171"}="2H1  171";
$resn_exch{" H21 171"}="1H2  171"; $resn_exch{" H22 171"}="2H2  171"; $resn_exch{"H141 172"}="1H14 172";
$resn_exch{"H142 172"}="2H14 172"; $resn_exch{" H2  175"}=" HN2 175"; $resn_exch{" HC2 175"}=" H2  175";
$resn_exch{" HB1 175"}="1HB  175"; $resn_exch{" HB2 175"}="2HB  175"; $resn_exch{" HB3 175"}="3HB  175";
$resn_exch{"HN2A 175"}="2HN  175"; $resn_exch{" H41 175"}="1H4  175"; $resn_exch{" H42 175"}="2H4  175";
$resn_exch{" HA1 175"}="1HA  175"; $resn_exch{" HA2 175"}="2HA  175"; $resn_exch{"HN51 175"}="1HN5 175";
$resn_exch{"HN52 175"}="2HN5 175"; $resn_exch{"HN01 176"}="1HN0 176"; $resn_exch{"HN02 176"}="2HN0 176";
$resn_exch{"HD1A 176"}="1HD  176"; $resn_exch{"HD2A 176"}="2HD  176"; $resn_exch{" HB1 176"}="1HB  176";
$resn_exch{" HB2 176"}="2HB  176"; $resn_exch{" HG1 176"}="1HG  176"; $resn_exch{" HG2 176"}="2HG  176";
$resn_exch{"HA11 176"}="1HA1 176"; $resn_exch{"HA12 176"}="2HA1 176"; $resn_exch{" HG' 176"}=" HG* 176";
$resn_exch{"HG11 177"}="1HG1 177"; $resn_exch{"HG12 177"}="2HG1 177"; $resn_exch{"HG21 177"}="1HG2 177";
$resn_exch{"HG22 177"}="2HG2 177"; $resn_exch{"HD11 177"}="1HD1 177"; $resn_exch{"HD12 177"}="2HD1 177";
$resn_exch{"HD21 177"}="1HD2 177"; $resn_exch{"HD22 177"}="2HD2 177"; $resn_exch{"HE01 177"}="1HE0 177";
$resn_exch{"HE02 177"}="2HE0 177"; $resn_exch{" H01 177"}="1H0  177"; $resn_exch{" H02 177"}="2H0  177";
$resn_exch{" HD1 177"}="1HD  177"; $resn_exch{" HD2 177"}="2HD  177"; $resn_exch{" HB1 177"}="1HB  177";
$resn_exch{" HB2 177"}="2HB  177"; $resn_exch{" HG1 177"}="1HG  177"; $resn_exch{" HG2 177"}="2HG  177";
$resn_exch{"HA'1 177"}="1HA' 177"; $resn_exch{"HA'2 177"}="2HA' 177"; $resn_exch{" HG' 177"}=" HG* 177";
$resn_exch{" HF1 177"}="1HF  177"; $resn_exch{" HF2 177"}="2HF  177"; $resn_exch{" HM1 177"}="1HM  177";
$resn_exch{" HM2 177"}="2HM  177"; $resn_exch{" HN1 177"}="1HN  177"; $resn_exch{"HN2A 177"}="2HN  177";
$resn_exch{" HN3 177"}="3HN  177"; $resn_exch{"H2B1 178"}="1H2B 178"; $resn_exch{"H2B2 178"}="2H2B 178";
$resn_exch{"H2B3 178"}="3H2B 178"; $resn_exch{"H2G1 178"}="1H2G 178"; $resn_exch{"H2G2 178"}="2H2G 178";
$resn_exch{"H2G3 178"}="3H2G 178"; $resn_exch{"H2Y1 178"}="1H2Y 178"; $resn_exch{"H2Y2 178"}="2H2Y 178";
$resn_exch{"H2Y3 178"}="3H2Y 178"; $resn_exch{"H2Z1 178"}="1H2Z 178"; $resn_exch{"H2Z2 178"}="2H2Z 178";
$resn_exch{"H2Z3 178"}="3H2Z 178"; $resn_exch{"H301 17A"}="1H30 17A"; $resn_exch{"H302 17A"}="2H30 17A";
$resn_exch{"H303 17A"}="3H30 17A"; $resn_exch{"H291 17A"}="1H29 17A"; $resn_exch{"H292 17A"}="2H29 17A";
$resn_exch{"H293 17A"}="3H29 17A"; $resn_exch{"H151 17A"}="1H15 17A"; $resn_exch{"H152 17A"}="2H15 17A";
$resn_exch{" H91 17A"}="1H9  17A"; $resn_exch{" H92 17A"}="2H9  17A"; $resn_exch{"H101 17A"}="1H10 17A";
$resn_exch{"H102 17A"}="2H10 17A"; $resn_exch{"H8C1 17B"}="1H8C 17B"; $resn_exch{"H8C2 17B"}="2H8C 17B";
$resn_exch{"H7C1 17B"}="1H7C 17B"; $resn_exch{"H7C2 17B"}="2H7C 17B"; $resn_exch{"H9C1 17B"}="1H9C 17B";
$resn_exch{"H9C2 17B"}="2H9C 17B"; $resn_exch{"H6C1 17B"}="1H6C 17B"; $resn_exch{"H6C2 17B"}="2H6C 17B";
$resn_exch{"H1N1 17B"}="1H1N 17B"; $resn_exch{"H1N2 17B"}="2H1N 17B"; $resn_exch{"H101 17B"}="1H10 17B";
$resn_exch{"H102 17B"}="2H10 17B"; $resn_exch{"H1N3 17B"}="3H1N 17B"; $resn_exch{" H21 17H"}="1H2  17H";
$resn_exch{" H22 17H"}="2H2  17H"; $resn_exch{" H71 17H"}="1H7  17H"; $resn_exch{" H72 17H"}="2H7  17H";
$resn_exch{" H61 17H"}="1H6  17H"; $resn_exch{" H62 17H"}="2H6  17H"; $resn_exch{"H11A 17H"}="1H1  17H";
$resn_exch{"H12A 17H"}="2H1  17H"; $resn_exch{"H151 17H"}="1H15 17H"; $resn_exch{"H152 17H"}="2H15 17H";
$resn_exch{"H161 17H"}="1H16 17H"; $resn_exch{"H162 17H"}="2H16 17H"; $resn_exch{"H181 17H"}="1H18 17H";
$resn_exch{"H182 17H"}="2H18 17H"; $resn_exch{"H221 17H"}="1H22 17H"; $resn_exch{"H222 17H"}="2H22 17H";
$resn_exch{"H223 17H"}="3H22 17H"; $resn_exch{"H211 17H"}="1H21 17H"; $resn_exch{"H212 17H"}="2H21 17H";
$resn_exch{"H213 17H"}="3H21 17H"; $resn_exch{"H201 17H"}="1H20 17H"; $resn_exch{"H202 17H"}="2H20 17H";
$resn_exch{"HAR1 17M"}="1HAR 17M"; $resn_exch{"HAR2 17M"}="2HAR 17M"; $resn_exch{"HAR3 17M"}="3HAR 17M";
$resn_exch{"HAP1 17M"}="1HAP 17M"; $resn_exch{"HAP2 17M"}="2HAP 17M"; $resn_exch{"HAJ1 17M"}="1HAJ 17M";
$resn_exch{"HAJ2 17M"}="2HAJ 17M"; $resn_exch{"HAQ1 17M"}="1HAQ 17M"; $resn_exch{"HAQ2 17M"}="2HAQ 17M";
$resn_exch{"HAC1 17M"}="1HAC 17M"; $resn_exch{"HAC2 17M"}="2HAC 17M"; $resn_exch{"HAS1 17M"}="1HAS 17M";
$resn_exch{"HAS2 17M"}="2HAS 17M"; $resn_exch{"HAS3 17M"}="3HAS 17M"; $resn_exch{" H31 180"}="1H3  180";
$resn_exch{" H32 180"}="2H3  180"; $resn_exch{"H101 180"}="1H10 180"; $resn_exch{"H102 180"}="2H10 180";
$resn_exch{"H111 180"}="1H11 180"; $resn_exch{"H112 180"}="2H11 180"; $resn_exch{"H131 180"}="1H13 180";
$resn_exch{"H132 180"}="2H13 180"; $resn_exch{"H141 180"}="1H14 180"; $resn_exch{"H142 180"}="2H14 180";
$resn_exch{"H151 180"}="1H15 180"; $resn_exch{"H152 180"}="2H15 180"; $resn_exch{"H161 180"}="1H16 180";
$resn_exch{"H162 180"}="2H16 180"; $resn_exch{"HAJ1 181"}="1HAJ 181"; $resn_exch{"HAJ2 181"}="2HAJ 181";
$resn_exch{"HAK1 181"}="1HAK 181"; $resn_exch{"HAK2 181"}="2HAK 181"; $resn_exch{"HAL1 181"}="1HAL 181";
$resn_exch{"HAL2 181"}="2HAL 181"; $resn_exch{"HAJ1 182"}="1HAJ 182"; $resn_exch{"HAJ2 182"}="2HAJ 182";
$resn_exch{"HAK1 182"}="1HAK 182"; $resn_exch{"HAK2 182"}="2HAK 182"; $resn_exch{"HAL1 182"}="1HAL 182";
$resn_exch{"HAL2 182"}="2HAL 182"; $resn_exch{"HAN1 183"}="1HAN 183"; $resn_exch{"HAN2 183"}="2HAN 183";
$resn_exch{"HAO1 183"}="1HAO 183"; $resn_exch{"HAO2 183"}="2HAO 183"; $resn_exch{"HAR1 183"}="1HAR 183";
$resn_exch{"HAR2 183"}="2HAR 183"; $resn_exch{"HAQ1 183"}="1HAQ 183"; $resn_exch{"HAQ2 183"}="2HAQ 183";
$resn_exch{"HAP1 183"}="1HAP 183"; $resn_exch{"HAP2 183"}="2HAP 183"; $resn_exch{"H171 184"}="1H17 184";
$resn_exch{"H172 184"}="2H17 184"; $resn_exch{"H181 184"}="1H18 184"; $resn_exch{"H182 184"}="2H18 184";
$resn_exch{"H231 184"}="1H23 184"; $resn_exch{"H232 184"}="2H23 184"; $resn_exch{"H233 184"}="3H23 184";
$resn_exch{"H241 184"}="1H24 184"; $resn_exch{"H242 184"}="2H24 184"; $resn_exch{"H243 184"}="3H24 184";
$resn_exch{"H251 184"}="1H25 184"; $resn_exch{"H252 184"}="2H25 184"; $resn_exch{"H253 184"}="3H25 184";
$resn_exch{"H261 184"}="1H26 184"; $resn_exch{"H262 184"}="2H26 184"; $resn_exch{"H263 184"}="3H26 184";
$resn_exch{"H2'1 185"}="1H2' 185"; $resn_exch{"H2'2 185"}="2H2' 185"; $resn_exch{"H2'3 185"}="3H2' 185";
$resn_exch{"H6'1 185"}="1H6' 185"; $resn_exch{"H6'2 185"}="2H6' 185"; $resn_exch{"H6'3 185"}="3H6' 185";
$resn_exch{"H181 185"}="1H18 185"; $resn_exch{"H182 185"}="2H18 185"; $resn_exch{"H271 186"}="1H27 186";
$resn_exch{"H272 186"}="2H27 186"; $resn_exch{"H161 186"}="1H16 186"; $resn_exch{"H162 186"}="2H16 186";
$resn_exch{"H141 186"}="1H14 186"; $resn_exch{"H142 186"}="2H14 186"; $resn_exch{"H143 186"}="3H14 186";
$resn_exch{"HN41 187"}="1HN4 187"; $resn_exch{"HN42 187"}="2HN4 187"; $resn_exch{"H131 188"}="1H13 188";
$resn_exch{"H132 188"}="2H13 188"; $resn_exch{"H161 188"}="1H16 188"; $resn_exch{"H162 188"}="2H16 188";
$resn_exch{"H181 188"}="1H18 188"; $resn_exch{"H182 188"}="2H18 188"; $resn_exch{"H183 188"}="3H18 188";
$resn_exch{"H201 189"}="1H20 189"; $resn_exch{"H202 189"}="2H20 189"; $resn_exch{"H261 189"}="1H26 189";
$resn_exch{"H262 189"}="2H26 189"; $resn_exch{"H361 189"}="1H36 189"; $resn_exch{"H362 189"}="2H36 189";
$resn_exch{"H451 189"}="1H45 189"; $resn_exch{"H452 189"}="2H45 189"; $resn_exch{"H453 189"}="3H45 189";
$resn_exch{"H441 189"}="1H44 189"; $resn_exch{"H442 189"}="2H44 189"; $resn_exch{"H443 189"}="3H44 189";
$resn_exch{"H241 189"}="1H24 189"; $resn_exch{"H242 189"}="2H24 189"; $resn_exch{"H221 189"}="1H22 189";
$resn_exch{"H222 189"}="2H22 189"; $resn_exch{"H161 189"}="1H16 189"; $resn_exch{"H162 189"}="2H16 189";
$resn_exch{"H163 189"}="3H16 189"; $resn_exch{"H191 189"}="1H19 189"; $resn_exch{"H192 189"}="2H19 189";
$resn_exch{"H181 189"}="1H18 189"; $resn_exch{"H182 189"}="2H18 189"; $resn_exch{"H183 189"}="3H18 189";
$resn_exch{"H281 189"}="1H28 189"; $resn_exch{"H282 189"}="2H28 189"; $resn_exch{"H361 18C"}="1H36 18C";
$resn_exch{"H362 18C"}="2H36 18C"; $resn_exch{"H363 18C"}="3H36 18C"; $resn_exch{"H351 18C"}="1H35 18C";
$resn_exch{"H352 18C"}="2H35 18C"; $resn_exch{"H341 18C"}="1H34 18C"; $resn_exch{"H342 18C"}="2H34 18C";
$resn_exch{"H331 18C"}="1H33 18C"; $resn_exch{"H332 18C"}="2H33 18C"; $resn_exch{"H321 18C"}="1H32 18C";
$resn_exch{"H322 18C"}="2H32 18C"; $resn_exch{"H311 18C"}="1H31 18C"; $resn_exch{"H312 18C"}="2H31 18C";
$resn_exch{"H301 18C"}="1H30 18C"; $resn_exch{"H302 18C"}="2H30 18C"; $resn_exch{"H291 18C"}="1H29 18C";
$resn_exch{"H292 18C"}="2H29 18C"; $resn_exch{"H281 18C"}="1H28 18C"; $resn_exch{"H282 18C"}="2H28 18C";
$resn_exch{"H271 18C"}="1H27 18C"; $resn_exch{"H272 18C"}="2H27 18C"; $resn_exch{"H261 18C"}="1H26 18C";
$resn_exch{"H262 18C"}="2H26 18C"; $resn_exch{"H251 18C"}="1H25 18C"; $resn_exch{"H252 18C"}="2H25 18C";
$resn_exch{"H241 18C"}="1H24 18C"; $resn_exch{"H242 18C"}="2H24 18C"; $resn_exch{"H231 18C"}="1H23 18C";
$resn_exch{"H232 18C"}="2H23 18C"; $resn_exch{"H221 18C"}="1H22 18C"; $resn_exch{"H222 18C"}="2H22 18C";
$resn_exch{"H211 18C"}="1H21 18C"; $resn_exch{"H212 18C"}="2H21 18C"; $resn_exch{"H201 18C"}="1H20 18C";
$resn_exch{"H202 18C"}="2H20 18C"; $resn_exch{" H11 18C"}="1H1  18C"; $resn_exch{" H12 18C"}="2H1  18C";
$resn_exch{" H61 18C"}="1H6  18C"; $resn_exch{" H62 18C"}="2H6  18C"; $resn_exch{" H71 18C"}="1H7  18C";
$resn_exch{" H72 18C"}="2H7  18C"; $resn_exch{" H81 18C"}="1H8  18C"; $resn_exch{" H82 18C"}="2H8  18C";
$resn_exch{" H91 18C"}="1H9  18C"; $resn_exch{" H92 18C"}="2H9  18C"; $resn_exch{"H101 18C"}="1H10 18C";
$resn_exch{"H102 18C"}="2H10 18C"; $resn_exch{"H111 18C"}="1H11 18C"; $resn_exch{"H112 18C"}="2H11 18C";
$resn_exch{"H121 18C"}="1H12 18C"; $resn_exch{"H122 18C"}="2H12 18C"; $resn_exch{"H131 18C"}="1H13 18C";
$resn_exch{"H132 18C"}="2H13 18C"; $resn_exch{"H141 18C"}="1H14 18C"; $resn_exch{"H142 18C"}="2H14 18C";
$resn_exch{"H151 18C"}="1H15 18C"; $resn_exch{"H152 18C"}="2H15 18C"; $resn_exch{"H161 18C"}="1H16 18C";
$resn_exch{"H162 18C"}="2H16 18C"; $resn_exch{"H171 18C"}="1H17 18C"; $resn_exch{"H172 18C"}="2H17 18C";
$resn_exch{"H181 18C"}="1H18 18C"; $resn_exch{"H182 18C"}="2H18 18C"; $resn_exch{"H183 18C"}="3H18 18C";
$resn_exch{" H81 190"}="1H8  190"; $resn_exch{" H82 190"}="2H8  190"; $resn_exch{" H83 190"}="3H8  190";
$resn_exch{" H71 190"}="1H7  190"; $resn_exch{" H72 190"}="2H7  190"; $resn_exch{" H73 190"}="3H7  190";
$resn_exch{"H101 190"}="1H10 190"; $resn_exch{"H102 190"}="2H10 190"; $resn_exch{"H161 190"}="1H16 190";
$resn_exch{"H162 190"}="2H16 190"; $resn_exch{"H241 190"}="1H24 190"; $resn_exch{"H242 190"}="2H24 190";
$resn_exch{"H261 190"}="1H26 190"; $resn_exch{"H262 190"}="2H26 190"; $resn_exch{"H281 190"}="1H28 190";
$resn_exch{"H282 190"}="2H28 190"; $resn_exch{"H283 190"}="3H28 190"; $resn_exch{"H291 190"}="1H29 190";
$resn_exch{"H292 190"}="2H29 190"; $resn_exch{"H293 190"}="3H29 190"; $resn_exch{" H31 191"}="1H3  191";
$resn_exch{"H32A 191"}="2H3  191"; $resn_exch{"H33A 191"}="3H3  191"; $resn_exch{" H21 191"}="1H2  191";
$resn_exch{"H22A 191"}="2H2  191"; $resn_exch{" H51 191"}="1H5  191"; $resn_exch{" H52 191"}="2H5  191";
$resn_exch{" H61 191"}="1H6  191"; $resn_exch{" H62 191"}="2H6  191"; $resn_exch{" H91 191"}="1H9  191";
$resn_exch{" H92 191"}="2H9  191"; $resn_exch{"H101 191"}="1H10 191"; $resn_exch{"H102 191"}="2H10 191";
$resn_exch{"H171 191"}="1H17 191"; $resn_exch{"H172 191"}="2H17 191"; $resn_exch{"H173 191"}="3H17 191";
$resn_exch{"H181 191"}="1H18 191"; $resn_exch{"H182 191"}="2H18 191"; $resn_exch{"H183 191"}="3H18 191";
$resn_exch{"H191 191"}="1H19 191"; $resn_exch{"H192 191"}="2H19 191"; $resn_exch{"H5'1 191"}="1H5* 191";
$resn_exch{"H5'2 191"}="2H5* 191"; $resn_exch{" H4' 191"}=" H4* 191"; $resn_exch{" H3' 191"}=" H3* 191";
$resn_exch{" H2' 191"}=" H2* 191"; $resn_exch{"HO'2 191"}="2HO* 191"; $resn_exch{" H1' 191"}=" H1* 191";
$resn_exch{"H411 191"}="1H41 191"; $resn_exch{"H412 191"}="2H41 191"; $resn_exch{"HB11 192"}="1HB1 192";
$resn_exch{"HB12 192"}="2HB1 192"; $resn_exch{" HG1 192"}="1HG  192"; $resn_exch{" HG2 192"}="2HG  192";
$resn_exch{"HB21 192"}="1HB2 192"; $resn_exch{"HB22 192"}="2HB2 192"; $resn_exch{" HB2 193"}="2HB  193";
$resn_exch{" HB1 193"}="1HB  193"; $resn_exch{" HG2 193"}="2HG  193"; $resn_exch{" HG1 193"}="1HG  193";
$resn_exch{" HZ2 193"}="2HZ  193"; $resn_exch{" HZ1 193"}="1HZ  193"; $resn_exch{" HT2 193"}="2HT  193";
$resn_exch{" HT1 193"}="1HT  193"; $resn_exch{"H71A 197"}="1H7  197"; $resn_exch{" H72 197"}="2H7  197";
$resn_exch{"H131 197"}="1H13 197"; $resn_exch{"H132 197"}="2H13 197"; $resn_exch{"H133 197"}="3H13 197";
$resn_exch{"H171 197"}="1H17 197"; $resn_exch{"H172 197"}="2H17 197"; $resn_exch{"H181 197"}="1H18 197";
$resn_exch{"H182 197"}="2H18 197"; $resn_exch{"H191 197"}="1H19 197"; $resn_exch{"H192 197"}="2H19 197";
$resn_exch{"H201 197"}="1H20 197"; $resn_exch{"H202 197"}="2H20 197"; $resn_exch{"H231 197"}="1H23 197";
$resn_exch{"H232 197"}="2H23 197"; $resn_exch{"H131 198"}="1H13 198"; $resn_exch{"H132 198"}="2H13 198";
$resn_exch{"H121 198"}="1H12 198"; $resn_exch{"H122 198"}="2H12 198"; $resn_exch{"H123 198"}="3H12 198";
$resn_exch{"H251 199"}="1H25 199"; $resn_exch{"H252 199"}="2H25 199"; $resn_exch{"H261 199"}="1H26 199";
$resn_exch{"H262 199"}="2H26 199"; $resn_exch{"H281 199"}="1H28 199"; $resn_exch{"H282 199"}="2H28 199";
$resn_exch{"H291 199"}="1H29 199"; $resn_exch{"H292 199"}="2H29 199"; $resn_exch{"H311 199"}="1H31 199";
$resn_exch{"H312 199"}="2H31 199"; $resn_exch{"H313 199"}="3H31 199"; $resn_exch{" H11 19A"}="1H1  19A";
$resn_exch{" H12 19A"}="2H1  19A"; $resn_exch{" H13 19A"}="3H1  19A"; $resn_exch{" H31 19A"}="1H3  19A";
$resn_exch{" H32 19A"}="2H3  19A"; $resn_exch{" H33 19A"}="3H3  19A"; $resn_exch{"HN31 1AA"}="1HN3 1AA";
$resn_exch{"HN32 1AA"}="2HN3 1AA"; $resn_exch{" H8' 1AA"}=" H8* 1AA"; $resn_exch{" H7' 1AA"}=" H7* 1AA";
$resn_exch{" H4' 1AA"}=" H4* 1AA"; $resn_exch{" H61 1AA"}="1H6  1AA"; $resn_exch{" H62 1AA"}="2H6  1AA";
$resn_exch{" H71 1AA"}="1H7  1AA"; $resn_exch{" H72 1AA"}="2H7  1AA"; $resn_exch{" H81 1AA"}="1H8  1AA";
$resn_exch{" H82 1AA"}="2H8  1AA"; $resn_exch{" H2' 1AA"}=" H2* 1AA"; $resn_exch{" H3' 1AA"}=" H3* 1AA";
$resn_exch{" H5' 1AA"}=" H5* 1AA"; $resn_exch{"HN11 1AA"}="1HN1 1AA"; $resn_exch{"HN12 1AA"}="2HN1 1AA";
$resn_exch{" H21 1AB"}="1H2  1AB"; $resn_exch{" H22 1AB"}="2H2  1AB"; $resn_exch{" H61 1AB"}="1H6  1AB";
$resn_exch{" H62 1AB"}="2H6  1AB"; $resn_exch{" H31 1AC"}="1H3  1AC"; $resn_exch{" H32 1AC"}="2H3  1AC";
$resn_exch{" H41 1AC"}="1H4  1AC"; $resn_exch{" H42 1AC"}="2H4  1AC"; $resn_exch{" H11 1AC"}="1H1  1AC";
$resn_exch{" H12 1AC"}="2H1  1AC"; $resn_exch{" H51 1AD"}="1H5  1AD"; $resn_exch{" H52 1AD"}="2H5  1AD";
$resn_exch{" H71 1AD"}="1H7  1AD"; $resn_exch{" H72 1AD"}="2H7  1AD"; $resn_exch{" H81 1AD"}="1H8  1AD";
$resn_exch{" H82 1AD"}="2H8  1AD"; $resn_exch{"H121 1AD"}="1H12 1AD"; $resn_exch{"H122 1AD"}="2H12 1AD";
$resn_exch{"H141 1AD"}="1H14 1AD"; $resn_exch{"H142 1AD"}="2H14 1AD"; $resn_exch{"H151 1AD"}="1H15 1AD";
$resn_exch{"H152 1AD"}="2H15 1AD"; $resn_exch{"H153 1AD"}="3H15 1AD"; $resn_exch{"H161 1AD"}="1H16 1AD";
$resn_exch{"H162 1AD"}="2H16 1AD"; $resn_exch{"H171 1AD"}="1H17 1AD"; $resn_exch{"H172 1AD"}="2H17 1AD";
$resn_exch{"H181 1AD"}="1H18 1AD"; $resn_exch{"H182 1AD"}="2H18 1AD"; $resn_exch{"H191 1AD"}="1H19 1AD";
$resn_exch{"H192 1AD"}="2H19 1AD"; $resn_exch{" H11 1AE"}="1H1  1AE"; $resn_exch{" H12 1AE"}="2H1  1AE";
$resn_exch{" H13 1AE"}="3H1  1AE"; $resn_exch{" H21 1AE"}="1H2  1AE"; $resn_exch{" H22 1AE"}="2H2  1AE";
$resn_exch{" H31 1AE"}="1H3  1AE"; $resn_exch{" H32 1AE"}="2H3  1AE"; $resn_exch{" H62 1AE"}="2H6  1AE";
$resn_exch{" H41 1AF"}="1H4  1AF"; $resn_exch{" H42 1AF"}="2H4  1AF"; $resn_exch{" H43 1AF"}="3H4  1AF";
$resn_exch{" H51 1AF"}="1H5  1AF"; $resn_exch{" H52 1AF"}="2H5  1AF"; $resn_exch{" H53 1AF"}="3H5  1AF";
$resn_exch{" H61 1AF"}="1H6  1AF"; $resn_exch{" H62 1AF"}="2H6  1AF"; $resn_exch{" H63 1AF"}="3H6  1AF";
$resn_exch{"H321 1AH"}="1H32 1AH"; $resn_exch{"H322 1AH"}="2H32 1AH"; $resn_exch{"H7C1 1AH"}="1H7C 1AH";
$resn_exch{"H7C2 1AH"}="2H7C 1AH"; $resn_exch{"H7C3 1AH"}="3H7C 1AH"; $resn_exch{"H4C1 1AH"}="1H4C 1AH";
$resn_exch{"H4C2 1AH"}="2H4C 1AH"; $resn_exch{"H221 1AH"}="1H22 1AH"; $resn_exch{"H222 1AH"}="2H22 1AH";
$resn_exch{"H581 1AH"}="1H58 1AH"; $resn_exch{"H582 1AH"}="2H58 1AH"; $resn_exch{"H431 1AH"}="1H43 1AH";
$resn_exch{"H432 1AH"}="2H43 1AH"; $resn_exch{"H433 1AH"}="3H43 1AH"; $resn_exch{"H441 1AH"}="1H44 1AH";
$resn_exch{"H442 1AH"}="2H44 1AH"; $resn_exch{"H443 1AH"}="3H44 1AH"; $resn_exch{"H861 1AH"}="1H86 1AH";
$resn_exch{"H862 1AH"}="2H86 1AH"; $resn_exch{"H863 1AH"}="3H86 1AH"; $resn_exch{"HN31 1AL"}="1HN3 1AL";
$resn_exch{"HN32 1AL"}="2HN3 1AL"; $resn_exch{"H111 1AL"}="1H11 1AL"; $resn_exch{"H112 1AL"}="2H11 1AL";
$resn_exch{" HN1 1AN"}="1HN  1AN"; $resn_exch{" HN2 1AN"}="2HN  1AN"; $resn_exch{" H81 1AP"}="1H8  1AP";
$resn_exch{"HN21 1AP"}="1HN2 1AP"; $resn_exch{"HN22 1AP"}="2HN2 1AP"; $resn_exch{"HN61 1AP"}="1HN6 1AP";
$resn_exch{"HN62 1AP"}="2HN6 1AP"; $resn_exch{" H52 1AP"}="2H5  1AP"; $resn_exch{" H51 1AP"}="1H5  1AP";
$resn_exch{" H11 1AP"}="1H1  1AP"; $resn_exch{" H21 1AP"}="1H2  1AP"; $resn_exch{" H22 1AP"}="2H2  1AP";
$resn_exch{" H31 1AP"}="1H3  1AP"; $resn_exch{"HO31 1AP"}="1HO3 1AP"; $resn_exch{" H21 1AR"}="1H2  1AR";
$resn_exch{" H22 1AR"}="2H2  1AR"; $resn_exch{" H61 1AR"}="1H6  1AR"; $resn_exch{" H62 1AR"}="2H6  1AR";
$resn_exch{" H63 1AR"}="3H6  1AR"; $resn_exch{" H81 1BA"}="1H8  1BA"; $resn_exch{" H82 1BA"}="2H8  1BA";
$resn_exch{"H11A 1BA"}="1H1  1BA"; $resn_exch{"H12A 1BA"}="2H1  1BA"; $resn_exch{" H13 1BA"}="3H1  1BA";
$resn_exch{" H31 1BA"}="1H3  1BA"; $resn_exch{" H32 1BA"}="2H3  1BA"; $resn_exch{" H33 1BA"}="3H3  1BA";
$resn_exch{" H11 1BH"}="1H1  1BH"; $resn_exch{" H12 1BH"}="2H1  1BH"; $resn_exch{"H411 1BH"}="1H41 1BH";
$resn_exch{"H412 1BH"}="2H41 1BH"; $resn_exch{"H413 1BH"}="3H41 1BH"; $resn_exch{" HD1 1BH"}="1HD  1BH";
$resn_exch{" HD2 1BH"}="2HD  1BH"; $resn_exch{"H611 1BH"}="1H61 1BH"; $resn_exch{"H612 1BH"}="2H61 1BH";
$resn_exch{" HG1 1BH"}="1HG  1BH"; $resn_exch{" HG2 1BH"}="2HG  1BH"; $resn_exch{"H811 1BH"}="1H81 1BH";
$resn_exch{"H812 1BH"}="2H81 1BH"; $resn_exch{" HC1 1BH"}="1HC  1BH"; $resn_exch{" HC2 1BH"}="2HC  1BH";
$resn_exch{" HC3 1BH"}="3HC  1BH"; $resn_exch{"H191 1BM"}="1H19 1BM"; $resn_exch{"H192 1BM"}="2H19 1BM";
$resn_exch{"H193 1BM"}="3H19 1BM"; $resn_exch{"H271 1BM"}="1H27 1BM"; $resn_exch{"H272 1BM"}="2H27 1BM";
$resn_exch{"H301 1BM"}="1H30 1BM"; $resn_exch{"H302 1BM"}="2H30 1BM"; $resn_exch{"H361 1BM"}="1H36 1BM";
$resn_exch{"H362 1BM"}="2H36 1BM"; $resn_exch{"H421 1BM"}="1H42 1BM"; $resn_exch{"H422 1BM"}="2H42 1BM";
$resn_exch{"H423 1BM"}="3H42 1BM"; $resn_exch{"H351 1BM"}="1H35 1BM"; $resn_exch{"H352 1BM"}="2H35 1BM";
$resn_exch{"H381 1BM"}="1H38 1BM"; $resn_exch{"H382 1BM"}="2H38 1BM"; $resn_exch{"H383 1BM"}="3H38 1BM";
$resn_exch{" H41 1BN"}="1H4  1BN"; $resn_exch{" H42 1BN"}="2H4  1BN"; $resn_exch{" H11 1BO"}="1H1  1BO";
$resn_exch{" H12 1BO"}="2H1  1BO"; $resn_exch{" H13 1BO"}="3H1  1BO"; $resn_exch{" H21 1BO"}="1H2  1BO";
$resn_exch{" H22 1BO"}="2H2  1BO"; $resn_exch{" H31 1BO"}="1H3  1BO"; $resn_exch{" H32 1BO"}="2H3  1BO";
$resn_exch{" H41 1BO"}="1H4  1BO"; $resn_exch{" H42 1BO"}="2H4  1BO"; $resn_exch{" H11 1BP"}="1H1  1BP";
$resn_exch{" H12 1BP"}="2H1  1BP"; $resn_exch{" H21 1BP"}="1H2  1BP"; $resn_exch{" H31 1BP"}="1H3  1BP";
$resn_exch{" H32 1BP"}="2H3  1BP"; $resn_exch{" H33 1BP"}="3H3  1BP"; $resn_exch{"HO21 1BP"}="1HO2 1BP";
$resn_exch{" H21 1BY"}="1H2  1BY"; $resn_exch{" H22 1BY"}="2H2  1BY"; $resn_exch{"H121 1C2"}="1H12 1C2";
$resn_exch{"H122 1C2"}="2H12 1C2"; $resn_exch{"H111 1C2"}="1H11 1C2"; $resn_exch{"H112 1C2"}="2H11 1C2";
$resn_exch{"H113 1C2"}="3H11 1C2"; $resn_exch{"H101 1C2"}="1H10 1C2"; $resn_exch{"H102 1C2"}="2H10 1C2";
$resn_exch{"H103 1C2"}="3H10 1C2"; $resn_exch{" H91 1C2"}="1H9  1C2"; $resn_exch{" H92 1C2"}="2H9  1C2";
$resn_exch{" H93 1C2"}="3H9  1C2"; $resn_exch{" H61 1C2"}="1H6  1C2"; $resn_exch{" H62 1C2"}="2H6  1C2";
$resn_exch{" H63 1C2"}="3H6  1C2"; $resn_exch{" H31 1C2"}="1H3  1C2"; $resn_exch{" H32 1C2"}="2H3  1C2";
$resn_exch{" H11 1C5"}="1H1  1C5"; $resn_exch{"H12A 1C5"}="2H1  1C5"; $resn_exch{"H21A 1C5"}="1H2  1C5";
$resn_exch{" H22 1C5"}="2H2  1C5"; $resn_exch{" H31 1C5"}="1H3  1C5"; $resn_exch{" H32 1C5"}="2H3  1C5";
$resn_exch{" H51 1C5"}="1H5  1C5"; $resn_exch{" H52 1C5"}="2H5  1C5"; $resn_exch{" H61 1C5"}="1H6  1C5";
$resn_exch{" H62 1C5"}="2H6  1C5"; $resn_exch{" H71 1C5"}="1H7  1C5"; $resn_exch{" H72 1C5"}="2H7  1C5";
$resn_exch{" H81 1C5"}="1H8  1C5"; $resn_exch{" H82 1C5"}="2H8  1C5"; $resn_exch{"H131 1C5"}="1H13 1C5";
$resn_exch{"H132 1C5"}="2H13 1C5"; $resn_exch{"H141 1C5"}="1H14 1C5"; $resn_exch{"H142 1C5"}="2H14 1C5";
$resn_exch{"H181 1C5"}="1H18 1C5"; $resn_exch{"H182 1C5"}="2H18 1C5"; $resn_exch{"H261 1C5"}="1H26 1C5";
$resn_exch{"H262 1C5"}="2H26 1C5"; $resn_exch{"H263 1C5"}="3H26 1C5"; $resn_exch{"HN11 1C5"}="1HN1 1C5";
$resn_exch{"HN12 1C5"}="2HN1 1C5"; $resn_exch{"HOP2 1C5"}="2HOP 1C5"; $resn_exch{"HOP3 1C5"}="3HOP 1C5";
$resn_exch{" H11 1CA"}="1H1  1CA"; $resn_exch{" H12 1CA"}="2H1  1CA"; $resn_exch{" H21 1CA"}="1H2  1CA";
$resn_exch{" H22 1CA"}="2H2  1CA"; $resn_exch{" H61 1CA"}="1H6  1CA"; $resn_exch{" H62 1CA"}="2H6  1CA";
$resn_exch{" H71 1CA"}="1H7  1CA"; $resn_exch{" H72 1CA"}="2H7  1CA"; $resn_exch{"H111 1CA"}="1H11 1CA";
$resn_exch{"H112 1CA"}="2H11 1CA"; $resn_exch{"H121 1CA"}="1H12 1CA"; $resn_exch{"H122 1CA"}="2H12 1CA";
$resn_exch{"H151 1CA"}="1H15 1CA"; $resn_exch{"H152 1CA"}="2H15 1CA"; $resn_exch{"H161 1CA"}="1H16 1CA";
$resn_exch{"H162 1CA"}="2H16 1CA"; $resn_exch{"H181 1CA"}="1H18 1CA"; $resn_exch{"H182 1CA"}="2H18 1CA";
$resn_exch{"H183 1CA"}="3H18 1CA"; $resn_exch{"H191 1CA"}="1H19 1CA"; $resn_exch{"H192 1CA"}="2H19 1CA";
$resn_exch{"H193 1CA"}="3H19 1CA"; $resn_exch{"H211 1CA"}="1H21 1CA"; $resn_exch{"H212 1CA"}="2H21 1CA";
$resn_exch{" H21 1CB"}="1H2  1CB"; $resn_exch{" H22 1CB"}="2H2  1CB"; $resn_exch{" H41 1CB"}="1H4  1CB";
$resn_exch{" H42 1CB"}="2H4  1CB"; $resn_exch{" H51 1CB"}="1H5  1CB"; $resn_exch{" H52 1CB"}="2H5  1CB";
$resn_exch{" H31 1CB"}="1H3  1CB"; $resn_exch{" H32 1CB"}="2H3  1CB"; $resn_exch{"HN21 1CB"}="1HN2 1CB";
$resn_exch{"HN22 1CB"}="2HN2 1CB"; $resn_exch{"H11A 1CD"}="1H1  1CD"; $resn_exch{" H12 1CD"}="2H1  1CD";
$resn_exch{"H13A 1CD"}="3H1  1CD"; $resn_exch{" H31 1CD"}="1H3  1CD"; $resn_exch{" H32 1CD"}="2H3  1CD";
$resn_exch{" H51 1CD"}="1H5  1CD"; $resn_exch{" H52 1CD"}="2H5  1CD"; $resn_exch{"H281 1CD"}="1H28 1CD";
$resn_exch{"H282 1CD"}="2H28 1CD"; $resn_exch{"H301 1CD"}="1H30 1CD"; $resn_exch{"H302 1CD"}="2H30 1CD";
$resn_exch{"H361 1CM"}="1H36 1CM"; $resn_exch{"H362 1CM"}="2H36 1CM"; $resn_exch{"H351 1CM"}="1H35 1CM";
$resn_exch{"H352 1CM"}="2H35 1CM"; $resn_exch{"H353 1CM"}="3H35 1CM"; $resn_exch{"HN11 1CN"}="1HN1 1CN";
$resn_exch{"HN12 1CN"}="2HN1 1CN"; $resn_exch{"HN51 1CN"}="1HN5 1CN"; $resn_exch{"HN52 1CN"}="2HN5 1CN";
$resn_exch{"H421 1CP"}="1H42 1CP"; $resn_exch{"H422 1CP"}="2H42 1CP"; $resn_exch{"H411 1CP"}="1H41 1CP";
$resn_exch{"H412 1CP"}="2H41 1CP"; $resn_exch{"H7A1 1CP"}="1H7A 1CP"; $resn_exch{"H7A2 1CP"}="2H7A 1CP";
$resn_exch{"H7A3 1CP"}="3H7A 1CP"; $resn_exch{"H101 1CP"}="1H10 1CP"; $resn_exch{"H102 1CP"}="2H10 1CP";
$resn_exch{" H51 1CP"}="1H5  1CP"; $resn_exch{" H52 1CP"}="2H5  1CP"; $resn_exch{"H311 1CP"}="1H31 1CP";
$resn_exch{"H312 1CP"}="2H31 1CP"; $resn_exch{"H321 1CP"}="1H32 1CP"; $resn_exch{"H322 1CP"}="2H32 1CP";
$resn_exch{"H2A1 1CP"}="1H2A 1CP"; $resn_exch{"H2A2 1CP"}="2H2A 1CP"; $resn_exch{"H2A3 1CP"}="3H2A 1CP";
$resn_exch{"H201 1CP"}="1H20 1CP"; $resn_exch{"H202 1CP"}="2H20 1CP"; $resn_exch{"H611 1CP"}="1H61 1CP";
$resn_exch{"H612 1CP"}="2H61 1CP"; $resn_exch{"H621 1CP"}="1H62 1CP"; $resn_exch{"H622 1CP"}="2H62 1CP";
$resn_exch{"H861 1CP"}="1H86 1CP"; $resn_exch{"H862 1CP"}="2H86 1CP"; $resn_exch{"H863 1CP"}="3H86 1CP";
$resn_exch{"H151 1CP"}="1H15 1CP"; $resn_exch{"H152 1CP"}="2H15 1CP"; $resn_exch{"H811 1CP"}="1H81 1CP";
$resn_exch{"H812 1CP"}="2H81 1CP"; $resn_exch{"H813 1CP"}="3H81 1CP"; $resn_exch{"H911 1CP"}="1H91 1CP";
$resn_exch{"H912 1CP"}="2H91 1CP"; $resn_exch{"H921 1CP"}="1H92 1CP"; $resn_exch{"H922 1CP"}="2H92 1CP";
$resn_exch{"H5'1 1CS"}="1H5' 1CS"; $resn_exch{"H5'2 1CS"}="2H5' 1CS"; $resn_exch{"H5'3 1CS"}="3H5' 1CS";
$resn_exch{"H7'1 1CS"}="1H7' 1CS"; $resn_exch{"H7'2 1CS"}="2H7' 1CS"; $resn_exch{"H7'3 1CS"}="3H7' 1CS";
$resn_exch{"HO11 1CU"}="1HO1 1CU"; $resn_exch{"HO12 1CU"}="2HO1 1CU"; $resn_exch{"HO5' 1DA"}="*HO5 1DA";
$resn_exch{"H5'1 1DA"}="1H5* 1DA"; $resn_exch{"H5'2 1DA"}="2H5* 1DA"; $resn_exch{" H4' 1DA"}=" H4* 1DA";
$resn_exch{" H3' 1DA"}=" H3* 1DA"; $resn_exch{"HO3' 1DA"}="*HO3 1DA"; $resn_exch{" H2' 1DA"}=" H2* 1DA";
$resn_exch{"HO2' 1DA"}="*HO2 1DA"; $resn_exch{" H1' 1DA"}=" H1* 1DA"; $resn_exch{"HN61 1DA"}="1HN6 1DA";
$resn_exch{"HN62 1DA"}="2HN6 1DA"; $resn_exch{"H1C1 1DB"}="1H1C 1DB"; $resn_exch{"H1C2 1DB"}="2H1C 1DB";
$resn_exch{"H2C1 1DB"}="1H2C 1DB"; $resn_exch{"H2C2 1DB"}="2H2C 1DB"; $resn_exch{"H3C1 1DB"}="1H3C 1DB";
$resn_exch{"H3C2 1DB"}="2H3C 1DB"; $resn_exch{"H4C1 1DB"}="1H4C 1DB"; $resn_exch{"H4C2 1DB"}="2H4C 1DB";
$resn_exch{"H5C1 1DB"}="1H5C 1DB"; $resn_exch{"H5C2 1DB"}="2H5C 1DB"; $resn_exch{"H6C1 1DB"}="1H6C 1DB";
$resn_exch{"H6C2 1DB"}="2H6C 1DB"; $resn_exch{"H7C1 1DB"}="1H7C 1DB"; $resn_exch{"H7C2 1DB"}="2H7C 1DB";
$resn_exch{"H8C1 1DB"}="1H8C 1DB"; $resn_exch{"H8C2 1DB"}="2H8C 1DB"; $resn_exch{"H9C1 1DB"}="1H9C 1DB";
$resn_exch{"H9C2 1DB"}="2H9C 1DB"; $resn_exch{"H101 1DB"}="1H10 1DB"; $resn_exch{"H102 1DB"}="2H10 1DB";
$resn_exch{"H103 1DB"}="3H10 1DB"; $resn_exch{"HAA1 1DG"}="1HAA 1DG"; $resn_exch{"HAA2 1DG"}="2HAA 1DG";
$resn_exch{" H1' 1DG"}=" H1* 1DG"; $resn_exch{" H2' 1DG"}=" H2* 1DG"; $resn_exch{" H3' 1DG"}=" H3* 1DG";
$resn_exch{" H4' 1DG"}=" H4* 1DG"; $resn_exch{"H5'1 1DG"}="1H5* 1DG"; $resn_exch{"H5'2 1DG"}="2H5* 1DG";
$resn_exch{"HAY1 1DG"}="1HAY 1DG"; $resn_exch{"HAY2 1DG"}="2HAY 1DG"; $resn_exch{"H6N1 1DG"}="1H6N 1DG";
$resn_exch{"H6N2 1DG"}="2H6N 1DG"; $resn_exch{"H211 1DM"}="1H21 1DM"; $resn_exch{"H212 1DM"}="2H21 1DM";
$resn_exch{"H221 1DM"}="1H22 1DM"; $resn_exch{"H222 1DM"}="2H22 1DM"; $resn_exch{"H231 1DM"}="1H23 1DM";
$resn_exch{"H232 1DM"}="2H23 1DM"; $resn_exch{"H241 1DM"}="1H24 1DM"; $resn_exch{"H242 1DM"}="2H24 1DM";
$resn_exch{"H201 1DM"}="1H20 1DM"; $resn_exch{"H202 1DM"}="2H20 1DM"; $resn_exch{"H301 1DM"}="1H30 1DM";
$resn_exch{"H302 1DM"}="2H30 1DM"; $resn_exch{"H251 1DM"}="1H25 1DM"; $resn_exch{"H252 1DM"}="2H25 1DM";
$resn_exch{"H261 1DM"}="1H26 1DM"; $resn_exch{"H262 1DM"}="2H26 1DM"; $resn_exch{"H271 1DM"}="1H27 1DM";
$resn_exch{"H272 1DM"}="2H27 1DM"; $resn_exch{"H281 1DM"}="1H28 1DM"; $resn_exch{"H282 1DM"}="2H28 1DM";
$resn_exch{" H61 1DM"}="1H6  1DM"; $resn_exch{" H62 1DM"}="2H6  1DM"; $resn_exch{"H1C1 1DO"}="1H1C 1DO";
$resn_exch{"H1C2 1DO"}="2H1C 1DO"; $resn_exch{"H2C1 1DO"}="1H2C 1DO"; $resn_exch{"H2C2 1DO"}="2H2C 1DO";
$resn_exch{"H3C1 1DO"}="1H3C 1DO"; $resn_exch{"H3C2 1DO"}="2H3C 1DO"; $resn_exch{"H4C1 1DO"}="1H4C 1DO";
$resn_exch{"H4C2 1DO"}="2H4C 1DO"; $resn_exch{"H5C1 1DO"}="1H5C 1DO"; $resn_exch{"H5C2 1DO"}="2H5C 1DO";
$resn_exch{"H6C1 1DO"}="1H6C 1DO"; $resn_exch{"H6C2 1DO"}="2H6C 1DO"; $resn_exch{"H7C1 1DO"}="1H7C 1DO";
$resn_exch{"H7C2 1DO"}="2H7C 1DO"; $resn_exch{"H8C1 1DO"}="1H8C 1DO"; $resn_exch{"H8C2 1DO"}="2H8C 1DO";
$resn_exch{"H9C1 1DO"}="1H9C 1DO"; $resn_exch{"H9C2 1DO"}="2H9C 1DO"; $resn_exch{"H101 1DO"}="1H10 1DO";
$resn_exch{"H102 1DO"}="2H10 1DO"; $resn_exch{"H111 1DO"}="1H11 1DO"; $resn_exch{"H112 1DO"}="2H11 1DO";
$resn_exch{"H121 1DO"}="1H12 1DO"; $resn_exch{"H122 1DO"}="2H12 1DO"; $resn_exch{"H123 1DO"}="3H12 1DO";
$resn_exch{"H121 1EM"}="1H12 1EM"; $resn_exch{"H122 1EM"}="2H12 1EM"; $resn_exch{"H131 1EM"}="1H13 1EM";
$resn_exch{"H132 1EM"}="2H13 1EM"; $resn_exch{"H141 1EM"}="1H14 1EM"; $resn_exch{"H142 1EM"}="2H14 1EM";
$resn_exch{"H151 1EM"}="1H15 1EM"; $resn_exch{"H152 1EM"}="2H15 1EM"; $resn_exch{"H161 1EM"}="1H16 1EM";
$resn_exch{"H162 1EM"}="2H16 1EM"; $resn_exch{"H171 1EM"}="1H17 1EM"; $resn_exch{"H172 1EM"}="2H17 1EM";
$resn_exch{"H181 1EM"}="1H18 1EM"; $resn_exch{"H182 1EM"}="2H18 1EM"; $resn_exch{"H191 1EM"}="1H19 1EM";
$resn_exch{"H192 1EM"}="2H19 1EM"; $resn_exch{"H193 1EM"}="3H19 1EM"; $resn_exch{"H221 1EM"}="1H22 1EM";
$resn_exch{"H222 1EM"}="2H22 1EM"; $resn_exch{"H231 1EM"}="1H23 1EM"; $resn_exch{"H232 1EM"}="2H23 1EM";
$resn_exch{"H241 1EM"}="1H24 1EM"; $resn_exch{"H242 1EM"}="2H24 1EM"; $resn_exch{"H251 1EM"}="1H25 1EM";
$resn_exch{"H252 1EM"}="2H25 1EM"; $resn_exch{"H261 1EM"}="1H26 1EM"; $resn_exch{"H262 1EM"}="2H26 1EM";
$resn_exch{"H271 1EM"}="1H27 1EM"; $resn_exch{"H272 1EM"}="2H27 1EM"; $resn_exch{"H281 1EM"}="1H28 1EM";
$resn_exch{"H282 1EM"}="2H28 1EM"; $resn_exch{"H291 1EM"}="1H29 1EM"; $resn_exch{"H292 1EM"}="2H29 1EM";
$resn_exch{"H301 1EM"}="1H30 1EM"; $resn_exch{"H302 1EM"}="2H30 1EM"; $resn_exch{"H311 1EM"}="1H31 1EM";
$resn_exch{"H312 1EM"}="2H31 1EM"; $resn_exch{"H321 1EM"}="1H32 1EM"; $resn_exch{"H322 1EM"}="2H32 1EM";
$resn_exch{"H331 1EM"}="1H33 1EM"; $resn_exch{"H332 1EM"}="2H33 1EM"; $resn_exch{"H341 1EM"}="1H34 1EM";
$resn_exch{"H342 1EM"}="2H34 1EM"; $resn_exch{"H343 1EM"}="3H34 1EM"; $resn_exch{"H411 1EM"}="1H41 1EM";
$resn_exch{"H412 1EM"}="2H41 1EM"; $resn_exch{"H431 1EM"}="1H43 1EM"; $resn_exch{"H432 1EM"}="2H43 1EM";
$resn_exch{"HAA1 1FH"}="1HAA 1FH"; $resn_exch{"HAA2 1FH"}="2HAA 1FH"; $resn_exch{"HMA1 1FH"}="1HMA 1FH";
$resn_exch{"HMA2 1FH"}="2HMA 1FH"; $resn_exch{"HMA3 1FH"}="3HMA 1FH"; $resn_exch{"HBA1 1FH"}="1HBA 1FH";
$resn_exch{"HBA2 1FH"}="2HBA 1FH"; $resn_exch{"HMB1 1FH"}="1HMB 1FH"; $resn_exch{"HMB2 1FH"}="2HMB 1FH";
$resn_exch{"HMB3 1FH"}="3HMB 1FH"; $resn_exch{"HBB1 1FH"}="1HBB 1FH"; $resn_exch{"HBB2 1FH"}="2HBB 1FH";
$resn_exch{"HMC1 1FH"}="1HMC 1FH"; $resn_exch{"HMC2 1FH"}="2HMC 1FH"; $resn_exch{"HMC3 1FH"}="3HMC 1FH";
$resn_exch{"HBC1 1FH"}="1HBC 1FH"; $resn_exch{"HBC2 1FH"}="2HBC 1FH"; $resn_exch{"HMD1 1FH"}="1HMD 1FH";
$resn_exch{"HMD2 1FH"}="2HMD 1FH"; $resn_exch{"HMD3 1FH"}="3HMD 1FH"; $resn_exch{"HAD1 1FH"}="1HAD 1FH";
$resn_exch{"HAD2 1FH"}="2HAD 1FH"; $resn_exch{"HBD1 1FH"}="1HBD 1FH"; $resn_exch{"HBD2 1FH"}="2HBD 1FH";
$resn_exch{" H21 1GL"}="1H2  1GL"; $resn_exch{" H22 1GL"}="2H2  1GL"; $resn_exch{" H61 1GL"}="1H6  1GL";
$resn_exch{" H62 1GL"}="2H6  1GL"; $resn_exch{" H63 1GL"}="3H6  1GL"; $resn_exch{"HN21 1GN"}="1HN2 1GN";
$resn_exch{"HN22 1GN"}="2HN2 1GN"; $resn_exch{" H61 1GN"}="1H6  1GN"; $resn_exch{" H62 1GN"}="2H6  1GN";
$resn_exch{" H31 1GP"}="1H3  1GP"; $resn_exch{" H32 1GP"}="2H3  1GP"; $resn_exch{" H11 1GP"}="1H1  1GP";
$resn_exch{" H12 1GP"}="2H1  1GP"; $resn_exch{"HAA1 1HP"}="1HAA 1HP"; $resn_exch{"HAA2 1HP"}="2HAA 1HP";
$resn_exch{"HAA3 1HP"}="3HAA 1HP"; $resn_exch{"HAB1 1HP"}="1HAB 1HP"; $resn_exch{"HAB2 1HP"}="2HAB 1HP";
$resn_exch{"HN31 1IG"}="1HN3 1IG"; $resn_exch{"HN32 1IG"}="2HN3 1IG"; $resn_exch{"HN41 1IG"}="1HN4 1IG";
$resn_exch{"HN42 1IG"}="2HN4 1IG"; $resn_exch{" H51 1IG"}="1H5  1IG"; $resn_exch{" H52 1IG"}="2H5  1IG";
$resn_exch{" H61 1IG"}="1H6  1IG"; $resn_exch{" H62 1IG"}="2H6  1IG"; $resn_exch{" H63 1IG"}="3H6  1IG";
$resn_exch{"H161 1IG"}="1H16 1IG"; $resn_exch{"H162 1IG"}="2H16 1IG"; $resn_exch{"H171 1IG"}="1H17 1IG";
$resn_exch{"H172 1IG"}="2H17 1IG"; $resn_exch{"H191 1IG"}="1H19 1IG"; $resn_exch{"H192 1IG"}="2H19 1IG";
$resn_exch{"H193 1IG"}="3H19 1IG"; $resn_exch{"H211 1IG"}="1H21 1IG"; $resn_exch{"H212 1IG"}="2H21 1IG";
$resn_exch{"H213 1IG"}="3H21 1IG"; $resn_exch{"H201 1IG"}="1H20 1IG"; $resn_exch{"H202 1IG"}="2H20 1IG";
$resn_exch{"H203 1IG"}="3H20 1IG"; $resn_exch{"H11A 1IN"}="1H1  1IN"; $resn_exch{" H12 1IN"}="2H1  1IN";
$resn_exch{" H71 1IN"}="1H7  1IN"; $resn_exch{" H72 1IN"}="2H7  1IN"; $resn_exch{" H73 1IN"}="3H7  1IN";
$resn_exch{" H81 1IN"}="1H8  1IN"; $resn_exch{" H82 1IN"}="2H8  1IN"; $resn_exch{" H91 1IN"}="1H9  1IN";
$resn_exch{" H92 1IN"}="2H9  1IN"; $resn_exch{"H101 1IN"}="1H10 1IN"; $resn_exch{"H102 1IN"}="2H10 1IN";
$resn_exch{"H121 1IN"}="1H12 1IN"; $resn_exch{"H122 1IN"}="2H12 1IN"; $resn_exch{"H141 1IN"}="1H14 1IN";
$resn_exch{"H142 1IN"}="2H14 1IN"; $resn_exch{"H241 1IN"}="1H24 1IN"; $resn_exch{"H242 1IN"}="2H24 1IN";
$resn_exch{"H271 1IN"}="1H27 1IN"; $resn_exch{"H272 1IN"}="2H27 1IN"; $resn_exch{" H51 1IN"}="1H5  1IN";
$resn_exch{" H52 1IN"}="2H5  1IN"; $resn_exch{" H53 1IN"}="3H5  1IN"; $resn_exch{" H61 1IN"}="1H6  1IN";
$resn_exch{" H62 1IN"}="2H6  1IN"; $resn_exch{" H63 1IN"}="3H6  1IN"; $resn_exch{"H251 1IN"}="1H25 1IN";
$resn_exch{"H252 1IN"}="2H25 1IN"; $resn_exch{"H291 1IN"}="1H29 1IN"; $resn_exch{"H292 1IN"}="2H29 1IN";
$resn_exch{"H293 1IN"}="3H29 1IN"; $resn_exch{"H261 1IN"}="1H26 1IN"; $resn_exch{"H262 1IN"}="2H26 1IN";
$resn_exch{"HAJ1 1IP"}="1HAJ 1IP"; $resn_exch{"HAJ2 1IP"}="2HAJ 1IP"; $resn_exch{" HB1 1IP"}="1HB  1IP";
$resn_exch{" HB2 1IP"}="2HB  1IP"; $resn_exch{"HD21 1IP"}="1HD2 1IP"; $resn_exch{"HD22 1IP"}="2HD2 1IP";
$resn_exch{" H71 1IQ"}="1H7  1IQ"; $resn_exch{" H72 1IQ"}="2H7  1IQ"; $resn_exch{" H73 1IQ"}="3H7  1IQ";
$resn_exch{"H101 1IQ"}="1H10 1IQ"; $resn_exch{"H102 1IQ"}="2H10 1IQ"; $resn_exch{"H103 1IQ"}="3H10 1IQ";
$resn_exch{" H91 1IQ"}="1H9  1IQ"; $resn_exch{" H92 1IQ"}="2H9  1IQ"; $resn_exch{" H93 1IQ"}="3H9  1IQ";
$resn_exch{" H41 1LG"}="1H4  1LG"; $resn_exch{" H42 1LG"}="2H4  1LG"; $resn_exch{" H81 1LG"}="1H8  1LG";
$resn_exch{" H82 1LG"}="2H8  1LG"; $resn_exch{"H281 1LG"}="1H28 1LG"; $resn_exch{"H282 1LG"}="2H28 1LG";
$resn_exch{" H91 1LG"}="1H9  1LG"; $resn_exch{" H92 1LG"}="2H9  1LG"; $resn_exch{"H121 1LG"}="1H12 1LG";
$resn_exch{"H122 1LG"}="2H12 1LG"; $resn_exch{"H271 1LG"}="1H27 1LG"; $resn_exch{"H272 1LG"}="2H27 1LG";
$resn_exch{"H291 1LG"}="1H29 1LG"; $resn_exch{"H292 1LG"}="2H29 1LG"; $resn_exch{"H301 1LG"}="1H30 1LG";
$resn_exch{"H302 1LG"}="2H30 1LG"; $resn_exch{"H1C1 1LP"}="1H1C 1LP"; $resn_exch{"H3C1 1LP"}="1H3C 1LP";
$resn_exch{"H3C2 1LP"}="2H3C 1LP"; $resn_exch{"H3C3 1LP"}="3H3C 1LP"; $resn_exch{" HB1 1LU"}="1HB  1LU";
$resn_exch{" HB2 1LU"}="2HB  1LU"; $resn_exch{"HD11 1LU"}="1HD1 1LU"; $resn_exch{"HD12 1LU"}="2HD1 1LU";
$resn_exch{"HD13 1LU"}="3HD1 1LU"; $resn_exch{"HD21 1LU"}="1HD2 1LU"; $resn_exch{"HD22 1LU"}="2HD2 1LU";
$resn_exch{"HD23 1LU"}="3HD2 1LU"; $resn_exch{"HOP2 1MA"}="2HOP 1MA"; $resn_exch{"HOP3 1MA"}="3HOP 1MA";
$resn_exch{" H5' 1MA"}="1H5* 1MA"; $resn_exch{"H5'' 1MA"}="2H5* 1MA"; $resn_exch{" H4' 1MA"}=" H4* 1MA";
$resn_exch{" H3' 1MA"}=" H3* 1MA"; $resn_exch{"HO3' 1MA"}="*HO3 1MA"; $resn_exch{" H2' 1MA"}=" H2* 1MA";
$resn_exch{"HO2' 1MA"}="*HO2 1MA"; $resn_exch{" H1' 1MA"}=" H1* 1MA"; $resn_exch{"HN61 1MA"}="1HN6 1MA";
$resn_exch{"HN62 1MA"}="2HN6 1MA"; $resn_exch{"HM11 1MA"}="1HM1 1MA"; $resn_exch{"HM12 1MA"}="2HM1 1MA";
$resn_exch{"HM13 1MA"}="3HM1 1MA"; $resn_exch{" H11 1MC"}="1H1  1MC"; $resn_exch{" H12 1MC"}="2H1  1MC";
$resn_exch{" H13 1MC"}="3H1  1MC"; $resn_exch{"HN41 1MC"}="1HN4 1MC"; $resn_exch{"HN42 1MC"}="2HN4 1MC";
$resn_exch{"HOP2 1MG"}="2HOP 1MG"; $resn_exch{"HOP3 1MG"}="3HOP 1MG"; $resn_exch{" H5' 1MG"}="1H5* 1MG";
$resn_exch{"H5'' 1MG"}="2H5* 1MG"; $resn_exch{" H4' 1MG"}=" H4* 1MG"; $resn_exch{" H3' 1MG"}=" H3* 1MG";
$resn_exch{"HO3' 1MG"}="*HO3 1MG"; $resn_exch{" H2' 1MG"}=" H2* 1MG"; $resn_exch{"HO2' 1MG"}="*HO2 1MG";
$resn_exch{" H1' 1MG"}=" H1* 1MG"; $resn_exch{"HM11 1MG"}="1HM1 1MG"; $resn_exch{"HM12 1MG"}="2HM1 1MG";
$resn_exch{"HM13 1MG"}="3HM1 1MG"; $resn_exch{"HN21 1MG"}="1HN2 1MG"; $resn_exch{"HN22 1MG"}="2HN2 1MG";
$resn_exch{"H131 1MM"}="1H13 1MM"; $resn_exch{"H132 1MM"}="2H13 1MM"; $resn_exch{"H133 1MM"}="3H13 1MM";
$resn_exch{"H5'1 1MM"}="1H5' 1MM"; $resn_exch{"H5'2 1MM"}="2H5' 1MM"; $resn_exch{"H5'3 1MM"}="3H5' 1MM";
$resn_exch{"H7'1 1MM"}="1H7' 1MM"; $resn_exch{"H7'2 1MM"}="2H7' 1MM"; $resn_exch{"H7'3 1MM"}="3H7' 1MM";
$resn_exch{"HAA1 1MR"}="1HAA 1MR"; $resn_exch{"HAA2 1MR"}="2HAA 1MR"; $resn_exch{"HAA3 1MR"}="3HAA 1MR";
$resn_exch{"HM11 1MZ"}="1HM1 1MZ"; $resn_exch{"HM12 1MZ"}="2HM1 1MZ"; $resn_exch{"HM13 1MZ"}="3HM1 1MZ";
$resn_exch{"H101 1N1"}="1H10 1N1"; $resn_exch{"H102 1N1"}="2H10 1N1"; $resn_exch{"H103 1N1"}="3H10 1N1";
$resn_exch{"H151 1N1"}="1H15 1N1"; $resn_exch{"H152 1N1"}="2H15 1N1"; $resn_exch{"H153 1N1"}="3H15 1N1";
$resn_exch{"H161 1N1"}="1H16 1N1"; $resn_exch{"H162 1N1"}="2H16 1N1"; $resn_exch{"H191 1N1"}="1H19 1N1";
$resn_exch{"H192 1N1"}="2H19 1N1"; $resn_exch{"H201 1N1"}="1H20 1N1"; $resn_exch{"H202 1N1"}="2H20 1N1";
$resn_exch{"H211 1N1"}="1H21 1N1"; $resn_exch{"H212 1N1"}="2H21 1N1"; $resn_exch{"H171 1N1"}="1H17 1N1";
$resn_exch{"H172 1N1"}="2H17 1N1"; $resn_exch{"H181 1N1"}="1H18 1N1"; $resn_exch{"H182 1N1"}="2H18 1N1";
$resn_exch{"H251 1N8"}="1H25 1N8"; $resn_exch{"H252 1N8"}="2H25 1N8"; $resn_exch{"H253 1N8"}="3H25 1N8";
$resn_exch{"H271 1N8"}="1H27 1N8"; $resn_exch{"H272 1N8"}="2H27 1N8"; $resn_exch{"H281 1N8"}="1H28 1N8";
$resn_exch{"H282 1N8"}="2H28 1N8"; $resn_exch{"H291 1N8"}="1H29 1N8"; $resn_exch{"H292 1N8"}="2H29 1N8";
$resn_exch{"H301 1N8"}="1H30 1N8"; $resn_exch{"H302 1N8"}="2H30 1N8"; $resn_exch{"H303 1N8"}="3H30 1N8";
$resn_exch{" H51 1N8"}="1H5  1N8"; $resn_exch{" H52 1N8"}="2H5  1N8"; $resn_exch{" H53 1N8"}="3H5  1N8";
$resn_exch{" H11 1N8"}="1H1  1N8"; $resn_exch{"H12A 1N8"}="2H1  1N8"; $resn_exch{" H13 1N8"}="3H1  1N8";
$resn_exch{"H311 1N9"}="1H31 1N9"; $resn_exch{"H312 1N9"}="2H31 1N9"; $resn_exch{"H313 1N9"}="3H31 1N9";
$resn_exch{"H321 1N9"}="1H32 1N9"; $resn_exch{"H322 1N9"}="2H32 1N9"; $resn_exch{"H323 1N9"}="3H32 1N9";
$resn_exch{"H331 1N9"}="1H33 1N9"; $resn_exch{"H332 1N9"}="2H33 1N9"; $resn_exch{"H333 1N9"}="3H33 1N9";
$resn_exch{"H361 1N9"}="1H36 1N9"; $resn_exch{"H362 1N9"}="2H36 1N9"; $resn_exch{"H363 1N9"}="3H36 1N9";
$resn_exch{"H371 1N9"}="1H37 1N9"; $resn_exch{"H372 1N9"}="2H37 1N9"; $resn_exch{"H373 1N9"}="3H37 1N9";
$resn_exch{" H71 1N9"}="1H7  1N9"; $resn_exch{" H72 1N9"}="2H7  1N9"; $resn_exch{" H81 1N9"}="1H8  1N9";
$resn_exch{" H82 1N9"}="2H8  1N9"; $resn_exch{"H351 1N9"}="1H35 1N9"; $resn_exch{"H352 1N9"}="2H35 1N9";
$resn_exch{"H341 1N9"}="1H34 1N9"; $resn_exch{"H342 1N9"}="2H34 1N9"; $resn_exch{"H343 1N9"}="3H34 1N9";
$resn_exch{" H11 1N9"}="1H1  1N9"; $resn_exch{" H12 1N9"}="2H1  1N9"; $resn_exch{" H31 1N9"}="1H3  1N9";
$resn_exch{" H32 1N9"}="2H3  1N9"; $resn_exch{" H41 1N9"}="1H4  1N9"; $resn_exch{" H42 1N9"}="2H4  1N9";
$resn_exch{" H61 1N9"}="1H6  1N9"; $resn_exch{" H62 1N9"}="2H6  1N9"; $resn_exch{" H21 1N9"}="1H2  1N9";
$resn_exch{" H22 1N9"}="2H2  1N9"; $resn_exch{" H23 1N9"}="3H2  1N9"; $resn_exch{" H61 1NA"}="1H6  1NA";
$resn_exch{" H62 1NA"}="2H6  1NA"; $resn_exch{" H81 1NA"}="1H8  1NA"; $resn_exch{" H82 1NA"}="2H8  1NA";
$resn_exch{" H83 1NA"}="3H8  1NA"; $resn_exch{" HM1 1NA"}="1HM  1NA"; $resn_exch{" HM2 1NA"}="2HM  1NA";
$resn_exch{" HM3 1NA"}="3HM  1NA"; $resn_exch{" H91 1NB"}="1H9  1NB"; $resn_exch{" H92 1NB"}="2H9  1NB";
$resn_exch{"H261 1NB"}="1H26 1NB"; $resn_exch{"H262 1NB"}="2H26 1NB"; $resn_exch{"H271 1NB"}="1H27 1NB";
$resn_exch{"H272 1NB"}="2H27 1NB"; $resn_exch{"H281 1NB"}="1H28 1NB"; $resn_exch{"H282 1NB"}="2H28 1NB";
$resn_exch{"H191 1NB"}="1H19 1NB"; $resn_exch{"H192 1NB"}="2H19 1NB"; $resn_exch{" H41 1NH"}="1H4  1NH";
$resn_exch{" H42 1NH"}="2H4  1NH"; $resn_exch{" H43 1NH"}="3H4  1NH"; $resn_exch{" H31 1NH"}="1H3  1NH";
$resn_exch{" H32 1NH"}="2H3  1NH"; $resn_exch{" H51 1NH"}="1H5  1NH"; $resn_exch{" H52 1NH"}="2H5  1NH";
$resn_exch{" H81 1NH"}="1H8  1NH"; $resn_exch{" H82 1NH"}="2H8  1NH"; $resn_exch{" H83 1NH"}="3H8  1NH";
$resn_exch{" H91 1NH"}="1H9  1NH"; $resn_exch{" H92 1NH"}="2H9  1NH"; $resn_exch{"H111 1NH"}="1H11 1NH";
$resn_exch{"H112 1NH"}="2H11 1NH"; $resn_exch{"H101 1NH"}="1H10 1NH"; $resn_exch{"H102 1NH"}="2H10 1NH";
$resn_exch{" H11 1NI"}="1H1  1NI"; $resn_exch{" H12 1NI"}="2H1  1NI"; $resn_exch{"H13A 1NI"}="3H1  1NI";
$resn_exch{"H241 1NI"}="1H24 1NI"; $resn_exch{"H242 1NI"}="2H24 1NI"; $resn_exch{"H243 1NI"}="3H24 1NI";
$resn_exch{"H301 1NI"}="1H30 1NI"; $resn_exch{"H302 1NI"}="2H30 1NI"; $resn_exch{"H303 1NI"}="3H30 1NI";
$resn_exch{"H311 1NI"}="1H31 1NI"; $resn_exch{"H312 1NI"}="2H31 1NI"; $resn_exch{"H313 1NI"}="3H31 1NI";
$resn_exch{"H331 1NI"}="1H33 1NI"; $resn_exch{"H332 1NI"}="2H33 1NI"; $resn_exch{" H61 1NI"}="1H6  1NI";
$resn_exch{" H62 1NI"}="2H6  1NI"; $resn_exch{"H251 1NI"}="1H25 1NI"; $resn_exch{"H252 1NI"}="2H25 1NI";
$resn_exch{"H253 1NI"}="3H25 1NI"; $resn_exch{"H281 1NI"}="1H28 1NI"; $resn_exch{"H282 1NI"}="2H28 1NI";
$resn_exch{"H391 1NI"}="1H39 1NI"; $resn_exch{"H392 1NI"}="2H39 1NI"; $resn_exch{"H401 1NI"}="1H40 1NI";
$resn_exch{"H402 1NI"}="2H40 1NI"; $resn_exch{"H403 1NI"}="3H40 1NI"; $resn_exch{"H451 1NI"}="1H45 1NI";
$resn_exch{"H452 1NI"}="2H45 1NI"; $resn_exch{" HN1 1NI"}="1HN  1NI"; $resn_exch{" HN2 1NI"}="2HN  1NI";
$resn_exch{"HI31 1NT"}="1HI3 1NT"; $resn_exch{"HI32 1NT"}="2HI3 1NT"; $resn_exch{"HA51 1NT"}="1HA5 1NT";
$resn_exch{"HA52 1NT"}="2HA5 1NT"; $resn_exch{"H2C1 1P3"}="1H2C 1P3"; $resn_exch{"H2C2 1P3"}="2H2C 1P3";
$resn_exch{"H2C3 1P3"}="3H2C 1P3"; $resn_exch{"H201 1P3"}="1H20 1P3"; $resn_exch{"H202 1P3"}="2H20 1P3";
$resn_exch{"H4C1 1P3"}="1H4C 1P3"; $resn_exch{"H4C2 1P3"}="2H4C 1P3"; $resn_exch{"H5C1 1P3"}="1H5C 1P3";
$resn_exch{"H5C2 1P3"}="2H5C 1P3"; $resn_exch{"H6C1 1P3"}="1H6C 1P3"; $resn_exch{"H6C2 1P3"}="2H6C 1P3";
$resn_exch{"H7C1 1P3"}="1H7C 1P3"; $resn_exch{"H7C2 1P3"}="2H7C 1P3"; $resn_exch{"H7C3 1P3"}="3H7C 1P3";
$resn_exch{" H   1PA"}="1HN  1PA"; $resn_exch{" H2  1PA"}="2HN  1PA"; $resn_exch{" HB2 1PA"}="1HB  1PA";
$resn_exch{" HB3 1PA"}="2HB  1PA"; $resn_exch{" HH2 1PA"}="1HH  1PA"; $resn_exch{" HH3 1PA"}="2HH  1PA";
$resn_exch{"HCC1 1PB"}="1HCC 1PB"; $resn_exch{"HCC2 1PB"}="2HCC 1PB"; $resn_exch{"HDC1 1PB"}="1HDC 1PB";
$resn_exch{"HDC2 1PB"}="2HDC 1PB"; $resn_exch{" HP1 1PC"}="1HP  1PC"; $resn_exch{" HP2 1PC"}="2HP  1PC";
$resn_exch{" HH1 1PC"}="1HH  1PC"; $resn_exch{" HH2 1PC"}="2HH  1PC"; $resn_exch{" HI1 1PC"}="1HI  1PC";
$resn_exch{" HI2 1PC"}="2HI  1PC"; $resn_exch{" HJ1 1PC"}="1HJ  1PC"; $resn_exch{" HJ2 1PC"}="2HJ  1PC";
$resn_exch{" HK1 1PC"}="1HK  1PC"; $resn_exch{" HK2 1PC"}="2HK  1PC"; $resn_exch{" HL1 1PC"}="1HL  1PC";
$resn_exch{" HL2 1PC"}="2HL  1PC"; $resn_exch{" HN1 1PC"}="1HN  1PC"; $resn_exch{" HN2 1PC"}="2HN  1PC";
$resn_exch{" HO1 1PC"}="1HO  1PC"; $resn_exch{" HO2 1PC"}="2HO  1PC"; $resn_exch{" HQ1 1PC"}="1HQ  1PC";
$resn_exch{" HQ2 1PC"}="2HQ  1PC"; $resn_exch{" HR1 1PC"}="1HR  1PC"; $resn_exch{" HR2 1PC"}="2HR  1PC";
$resn_exch{"H121 1PE"}="1H12 1PE"; $resn_exch{"H122 1PE"}="2H12 1PE"; $resn_exch{"H221 1PE"}="1H22 1PE";
$resn_exch{"H222 1PE"}="2H22 1PE"; $resn_exch{"H131 1PE"}="1H13 1PE"; $resn_exch{"H132 1PE"}="2H13 1PE";
$resn_exch{"H231 1PE"}="1H23 1PE"; $resn_exch{"H232 1PE"}="2H23 1PE"; $resn_exch{"H141 1PE"}="1H14 1PE";
$resn_exch{"H142 1PE"}="2H14 1PE"; $resn_exch{"H241 1PE"}="1H24 1PE"; $resn_exch{"H242 1PE"}="2H24 1PE";
$resn_exch{"H151 1PE"}="1H15 1PE"; $resn_exch{"H152 1PE"}="2H15 1PE"; $resn_exch{"H251 1PE"}="1H25 1PE";
$resn_exch{"H252 1PE"}="2H25 1PE"; $resn_exch{"H161 1PE"}="1H16 1PE"; $resn_exch{"H162 1PE"}="2H16 1PE";
$resn_exch{"H261 1PE"}="1H26 1PE"; $resn_exch{"H262 1PE"}="2H26 1PE"; $resn_exch{" H21 1PG"}="1H2  1PG";
$resn_exch{" H22 1PG"}="2H2  1PG"; $resn_exch{" H11 1PG"}="1H1  1PG"; $resn_exch{" H12 1PG"}="2H1  1PG";
$resn_exch{" H13 1PG"}="3H1  1PG"; $resn_exch{" H31 1PG"}="1H3  1PG"; $resn_exch{" H32 1PG"}="2H3  1PG";
$resn_exch{" H41 1PG"}="1H4  1PG"; $resn_exch{" H42 1PG"}="2H4  1PG"; $resn_exch{" H51 1PG"}="1H5  1PG";
$resn_exch{" H52 1PG"}="2H5  1PG"; $resn_exch{" H61 1PG"}="1H6  1PG"; $resn_exch{" H62 1PG"}="2H6  1PG";
$resn_exch{" H71 1PG"}="1H7  1PG"; $resn_exch{" H72 1PG"}="2H7  1PG"; $resn_exch{" H81 1PG"}="1H8  1PG";
$resn_exch{" H82 1PG"}="2H8  1PG"; $resn_exch{" H91 1PG"}="1H9  1PG"; $resn_exch{" H92 1PG"}="2H9  1PG";
$resn_exch{"H101 1PG"}="1H10 1PG"; $resn_exch{"H102 1PG"}="2H10 1PG"; $resn_exch{"H111 1PG"}="1H11 1PG";
$resn_exch{"H112 1PG"}="2H11 1PG"; $resn_exch{" H31 1PI"}="1H3  1PI"; $resn_exch{" H32 1PI"}="2H3  1PI";
$resn_exch{"HN1A 1PI"}="1HN  1PI"; $resn_exch{" HN2 1PI"}="2HN  1PI"; $resn_exch{"H171 1PI"}="1H17 1PI";
$resn_exch{"H172 1PI"}="2H17 1PI"; $resn_exch{" HD1 1PI"}="1HD  1PI"; $resn_exch{" HD2 1PI"}="2HD  1PI";
$resn_exch{" HG1 1PI"}="1HG  1PI"; $resn_exch{" HG2 1PI"}="2HG  1PI"; $resn_exch{" H11 1PI"}="1H1  1PI";
$resn_exch{" H12 1PI"}="2H1  1PI"; $resn_exch{"HN21 1PI"}="1HN2 1PI"; $resn_exch{"HN22 1PI"}="2HN2 1PI";
$resn_exch{"H101 1PL"}="1H10 1PL"; $resn_exch{"H102 1PL"}="2H10 1PL"; $resn_exch{"H103 1PL"}="3H10 1PL";
$resn_exch{"H111 1PL"}="1H11 1PL"; $resn_exch{"H112 1PL"}="2H11 1PL"; $resn_exch{"H121 1PL"}="1H12 1PL";
$resn_exch{"H122 1PL"}="2H12 1PL"; $resn_exch{" H31 1PL"}="1H3  1PL"; $resn_exch{" H32 1PL"}="2H3  1PL";
$resn_exch{"H141 1PL"}="1H14 1PL"; $resn_exch{"H142 1PL"}="2H14 1PL"; $resn_exch{"H143 1PL"}="3H14 1PL";
$resn_exch{"H151 1PL"}="1H15 1PL"; $resn_exch{"H152 1PL"}="2H15 1PL"; $resn_exch{"H153 1PL"}="3H15 1PL";
$resn_exch{" H11 1PL"}="1H1  1PL"; $resn_exch{" H12 1PL"}="2H1  1PL"; $resn_exch{"H301 1PM"}="1H30 1PM";
$resn_exch{"H302 1PM"}="2H30 1PM"; $resn_exch{"H303 1PM"}="3H30 1PM"; $resn_exch{"H311 1PM"}="1H31 1PM";
$resn_exch{"H312 1PM"}="2H31 1PM"; $resn_exch{"H361 1PM"}="1H36 1PM"; $resn_exch{"H362 1PM"}="2H36 1PM";
$resn_exch{"H481 1PM"}="1H48 1PM"; $resn_exch{"H482 1PM"}="2H48 1PM"; $resn_exch{"H121 1PN"}="1H12 1PN";
$resn_exch{"H122 1PN"}="2H12 1PN"; $resn_exch{"H131 1PN"}="1H13 1PN"; $resn_exch{"H132 1PN"}="2H13 1PN";
$resn_exch{"H141 1PN"}="1H14 1PN"; $resn_exch{"H142 1PN"}="2H14 1PN"; $resn_exch{"H143 1PN"}="3H14 1PN";
$resn_exch{"H161 1PN"}="1H16 1PN"; $resn_exch{"H162 1PN"}="2H16 1PN"; $resn_exch{"H163 1PN"}="3H16 1PN";
$resn_exch{"H201 1PN"}="1H20 1PN"; $resn_exch{"H202 1PN"}="2H20 1PN"; $resn_exch{"H203 1PN"}="3H20 1PN";
$resn_exch{"H151 1PP"}="1H15 1PP"; $resn_exch{"H152 1PP"}="2H15 1PP"; $resn_exch{"H153 1PP"}="3H15 1PP";
$resn_exch{"H141 1PP"}="1H14 1PP"; $resn_exch{"H142 1PP"}="2H14 1PP"; $resn_exch{"H143 1PP"}="3H14 1PP";
$resn_exch{"H131 1PP"}="1H13 1PP"; $resn_exch{"H132 1PP"}="2H13 1PP"; $resn_exch{"H133 1PP"}="3H13 1PP";
$resn_exch{"H5'1 1PR"}="1H5' 1PR"; $resn_exch{"H5'2 1PR"}="2H5' 1PR"; $resn_exch{"HO3' 1PR"}="'HO3 1PR";
$resn_exch{"HO2' 1PR"}="'HO2 1PR"; $resn_exch{"HN91 1PR"}="1HN9 1PR"; $resn_exch{"HN92 1PR"}="2HN9 1PR";
$resn_exch{" H71 1PR"}="1H7  1PR"; $resn_exch{" H72 1PR"}="2H7  1PR"; $resn_exch{" H'1 1PR"}="1H\"  1PR";
$resn_exch{" H'2 1PR"}="2H\"  1PR"; $resn_exch{" H2B 1PR"}=" H2\" 1PR"; $resn_exch{" H3B 1PR"}=" H3\" 1PR";
$resn_exch{" H4B 1PR"}=" H4\" 1PR"; $resn_exch{"H5A1 1PR"}="1H5\" 1PR"; $resn_exch{"H5A2 1PR"}="2H5\" 1PR";
$resn_exch{" H8B 1PR"}=" H8\" 1PR"; $resn_exch{" H9B 1PR"}=" H9\" 1PR"; $resn_exch{" H61 1PS"}="1H6  1PS";
$resn_exch{" H62 1PS"}="2H6  1PS"; $resn_exch{" H71 1PS"}="1H7  1PS"; $resn_exch{" H72 1PS"}="2H7  1PS";
$resn_exch{" H81 1PS"}="1H8  1PS"; $resn_exch{" H82 1PS"}="2H8  1PS"; $resn_exch{" H11 1PT"}="1H1  1PT";
$resn_exch{" H12 1PT"}="2H1  1PT"; $resn_exch{" H21 1PT"}="1H2  1PT"; $resn_exch{" H22 1PT"}="2H2  1PT";
$resn_exch{" H31 1PT"}="1H3  1PT"; $resn_exch{" H32 1PT"}="2H3  1PT"; $resn_exch{" H41 1PT"}="1H4  1PT";
$resn_exch{" H42 1PT"}="2H4  1PT"; $resn_exch{" H51 1PT"}="1H5  1PT"; $resn_exch{" H52 1PT"}="2H5  1PT";
$resn_exch{" H61 1PT"}="1H6  1PT"; $resn_exch{" H62 1PT"}="2H6  1PT"; $resn_exch{" H61 1PU"}="1H6  1PU";
$resn_exch{" H62 1PU"}="2H6  1PU"; $resn_exch{"H111 1PU"}="1H11 1PU"; $resn_exch{"H112 1PU"}="2H11 1PU";
$resn_exch{"H121 1PU"}="1H12 1PU"; $resn_exch{"H122 1PU"}="2H12 1PU"; $resn_exch{" HB1 1PY"}="1HB  1PY";
$resn_exch{" HB2 1PY"}="2HB  1PY"; $resn_exch{"HOP2 1RB"}="2HOP 1RB"; $resn_exch{"HOP3 1RB"}="3HOP 1RB";
$resn_exch{"H5'1 1RB"}="1H5* 1RB"; $resn_exch{"H5'2 1RB"}="2H5* 1RB"; $resn_exch{" H4' 1RB"}=" H4* 1RB";
$resn_exch{" H3' 1RB"}=" H3* 1RB"; $resn_exch{"HO23 1RB"}="3HO2 1RB"; $resn_exch{" H2' 1RB"}=" H2* 1RB";
$resn_exch{"HO22 1RB"}="2HO2 1RB"; $resn_exch{" H1' 1RB"}=" H1* 1RB"; $resn_exch{"HN11 1SA"}="1HN1 1SA";
$resn_exch{"HN12 1SA"}="2HN1 1SA"; $resn_exch{"HN41 1SA"}="1HN4 1SA"; $resn_exch{"HN42 1SA"}="2HN4 1SA";
$resn_exch{"H5'1 1SC"}="1H5* 1SC"; $resn_exch{"H5'2 1SC"}="2H5* 1SC"; $resn_exch{" H4' 1SC"}=" H4* 1SC";
$resn_exch{" H3' 1SC"}=" H3* 1SC"; $resn_exch{"HO3' 1SC"}="HO3* 1SC"; $resn_exch{" H2' 1SC"}=" H2* 1SC";
$resn_exch{"HO2' 1SC"}="HO2* 1SC"; $resn_exch{" H1' 1SC"}=" H1* 1SC"; $resn_exch{"HN41 1SC"}="1HN4 1SC";
$resn_exch{"HN42 1SC"}="2HN4 1SC"; $resn_exch{"H131 1SM"}="1H13 1SM"; $resn_exch{"H132 1SM"}="2H13 1SM";
$resn_exch{"H133 1SM"}="3H13 1SM"; $resn_exch{"H8'1 1SM"}="1H8' 1SM"; $resn_exch{"H8'2 1SM"}="2H8' 1SM";
$resn_exch{"H8'3 1SM"}="3H8' 1SM"; $resn_exch{"H7'1 1SM"}="1H7' 1SM"; $resn_exch{"H7'2 1SM"}="2H7' 1SM";
$resn_exch{"H7'3 1SM"}="3H7' 1SM"; $resn_exch{"HN11 1SQ"}="1HN1 1SQ"; $resn_exch{"HN12 1SQ"}="2HN1 1SQ";
$resn_exch{"H271 1ST"}="1H27 1ST"; $resn_exch{"H272 1ST"}="2H27 1ST"; $resn_exch{"H321 1ST"}="1H32 1ST";
$resn_exch{"H322 1ST"}="2H32 1ST"; $resn_exch{"H323 1ST"}="3H32 1ST"; $resn_exch{"H341 1ST"}="1H34 1ST";
$resn_exch{"H342 1ST"}="2H34 1ST"; $resn_exch{"H343 1ST"}="3H34 1ST"; $resn_exch{"H351 1ST"}="1H35 1ST";
$resn_exch{"H352 1ST"}="2H35 1ST"; $resn_exch{"H353 1ST"}="3H35 1ST"; $resn_exch{"H131 1TB"}="1H13 1TB";
$resn_exch{"H132 1TB"}="2H13 1TB"; $resn_exch{"H133 1TB"}="3H13 1TB"; $resn_exch{"H101 1TB"}="1H10 1TB";
$resn_exch{"H102 1TB"}="2H10 1TB"; $resn_exch{"H103 1TB"}="3H10 1TB"; $resn_exch{"H5'1 1TB"}="1H5' 1TB";
$resn_exch{"H5'2 1TB"}="2H5' 1TB"; $resn_exch{"H5'3 1TB"}="3H5' 1TB"; $resn_exch{"H7'1 1TB"}="1H7' 1TB";
$resn_exch{"H7'2 1TB"}="2H7' 1TB"; $resn_exch{"H7'3 1TB"}="3H7' 1TB"; $resn_exch{"H2A1 1TP"}="1H2A 1TP";
$resn_exch{"H2A2 1TP"}="2H2A 1TP"; $resn_exch{"H2A3 1TP"}="3H2A 1TP"; $resn_exch{"H4'1 1TP"}="1H4* 1TP";
$resn_exch{"H4'2 1TP"}="2H4* 1TP"; $resn_exch{" H6' 1TP"}=" H6* 1TP"; $resn_exch{"H7'1 1TP"}="1H7* 1TP";
$resn_exch{"H7'2 1TP"}="2H7* 1TP"; $resn_exch{"H4A1 1TP"}="1H4A 1TP"; $resn_exch{"H4A2 1TP"}="2H4A 1TP";
$resn_exch{"H4A3 1TP"}="3H4A 1TP"; $resn_exch{"H5A1 1TP"}="1H5A 1TP"; $resn_exch{"H5A2 1TP"}="2H5A 1TP";
$resn_exch{"H5B1 1TP"}="1H5B 1TP"; $resn_exch{"H5B2 1TP"}="2H5B 1TP"; $resn_exch{"HO13 1TP"}="3HO1 1TP";
$resn_exch{"H6B1 1TP"}="1H6B 1TP"; $resn_exch{"HO23 1TP"}="3HO2 1TP"; $resn_exch{"H351 1TP"}="1H35 1TP";
$resn_exch{"H352 1TP"}="2H35 1TP"; $resn_exch{"H353 1TP"}="3H35 1TP"; $resn_exch{" H   1TQ"}="1HN  1TQ";
$resn_exch{" H2  1TQ"}="2HN  1TQ"; $resn_exch{" HB2 1TQ"}="1HB  1TQ"; $resn_exch{" HB3 1TQ"}="2HB  1TQ";
$resn_exch{" HE1 1TQ"}="HNE1 1TQ"; $resn_exch{" HXT 1TQ"}="HOXT 1TQ"; $resn_exch{" H   1TY"}="1HN  1TY";
$resn_exch{" H2  1TY"}="2HN  1TY"; $resn_exch{" HB2 1TY"}="1HB  1TY"; $resn_exch{" HB3 1TY"}="2HB  1TY";
$resn_exch{" HH  1TY"}=" HOH 1TY"; $resn_exch{" H21 1TY"}="1H2  1TY"; $resn_exch{" H22 1TY"}="2H2  1TY";
$resn_exch{" H11 1TY"}="1H1  1TY"; $resn_exch{" H12 1TY"}="2H1  1TY"; $resn_exch{"H31A 1UN"}="1H3  1UN";
$resn_exch{"H32A 1UN"}="2H3  1UN"; $resn_exch{" H41 1UN"}="1H4  1UN"; $resn_exch{" H42 1UN"}="2H4  1UN";
$resn_exch{" H51 1UN"}="1H5  1UN"; $resn_exch{" H52 1UN"}="2H5  1UN"; $resn_exch{" H61 1UN"}="1H6  1UN";
$resn_exch{" H62 1UN"}="2H6  1UN"; $resn_exch{" H91 1UN"}="1H9  1UN"; $resn_exch{" H92 1UN"}="2H9  1UN";
$resn_exch{"H101 1UN"}="1H10 1UN"; $resn_exch{"H102 1UN"}="2H10 1UN"; $resn_exch{"H141 1UN"}="1H14 1UN";
$resn_exch{"H142 1UN"}="2H14 1UN"; $resn_exch{"H143 1UN"}="3H14 1UN"; $resn_exch{"H151 1UN"}="1H15 1UN";
$resn_exch{"H152 1UN"}="2H15 1UN"; $resn_exch{"H153 1UN"}="3H15 1UN"; $resn_exch{"H161 1UN"}="1H16 1UN";
$resn_exch{"H162 1UN"}="2H16 1UN"; $resn_exch{"H163 1UN"}="3H16 1UN"; $resn_exch{"H181 1UN"}="1H18 1UN";
$resn_exch{"H182 1UN"}="2H18 1UN"; $resn_exch{"H231 1UN"}="1H23 1UN"; $resn_exch{"H232 1UN"}="2H23 1UN";
$resn_exch{"H391 1UN"}="1H39 1UN"; $resn_exch{"H392 1UN"}="2H39 1UN"; $resn_exch{"H393 1UN"}="3H39 1UN";
$resn_exch{" H2  200"}="1HN  200"; $resn_exch{" HB2 200"}="1HB  200"; $resn_exch{" HB3 200"}="2HB  200";
$resn_exch{" HD1 200"}="1HD1 200"; $resn_exch{" HD2 200"}="1HD2 200"; $resn_exch{" HE1 200"}="1HE1 200";
$resn_exch{" HE2 200"}="1HE2 200"; $resn_exch{"HN11 201"}="1HN1 201"; $resn_exch{"HN12 201"}="2HN1 201";
$resn_exch{" H11 204"}="1H1  204"; $resn_exch{" H12 204"}="2H1  204"; $resn_exch{"H13A 204"}="3H1  204";
$resn_exch{" H21 204"}="1H2  204"; $resn_exch{" H22 204"}="2H2  204"; $resn_exch{"H101 204"}="1H10 204";
$resn_exch{"H102 204"}="2H10 204"; $resn_exch{"H9C1 207"}="1H9C 207"; $resn_exch{"H9C2 207"}="2H9C 207";
$resn_exch{"H2N1 207"}="1H2N 207"; $resn_exch{"H2N2 207"}="2H2N 207"; $resn_exch{"H111 207"}="1H11 207";
$resn_exch{"H112 207"}="2H11 207"; $resn_exch{"H121 207"}="1H12 207"; $resn_exch{"H122 207"}="2H12 207";
$resn_exch{"H131 207"}="1H13 207"; $resn_exch{"H132 207"}="2H13 207"; $resn_exch{"H141 207"}="1H14 207";
$resn_exch{"H142 207"}="2H14 207"; $resn_exch{"H151 207"}="1H15 207"; $resn_exch{"H152 207"}="2H15 207";
$resn_exch{"H301 208"}="1H30 208"; $resn_exch{"H302 208"}="2H30 208"; $resn_exch{"H303 208"}="3H30 208";
$resn_exch{"H281 208"}="1H28 208"; $resn_exch{"H282 208"}="2H28 208"; $resn_exch{"H191 208"}="1H19 208";
$resn_exch{"H192 208"}="2H19 208"; $resn_exch{" H91 208"}="1H9  208"; $resn_exch{" H92 208"}="2H9  208";
$resn_exch{"H261 208"}="1H26 208"; $resn_exch{"H262 208"}="2H26 208"; $resn_exch{"H263 208"}="3H26 208";
$resn_exch{"H101 209"}="1H10 209"; $resn_exch{"H102 209"}="2H10 209"; $resn_exch{"H111 209"}="1H11 209";
$resn_exch{"H112 209"}="2H11 209"; $resn_exch{"H113 209"}="3H11 209"; $resn_exch{"H151 209"}="1H15 209";
$resn_exch{"H152 209"}="2H15 209"; $resn_exch{"H153 209"}="3H15 209"; $resn_exch{"H191 20E"}="1H19 20E";
$resn_exch{"H192 20E"}="2H19 20E"; $resn_exch{"H193 20E"}="3H19 20E"; $resn_exch{" H11 20E"}="1H1  20E";
$resn_exch{" H12 20E"}="2H1  20E"; $resn_exch{" H41 20E"}="1H4  20E"; $resn_exch{" H42 20E"}="2H4  20E";
$resn_exch{"H111 20E"}="1H11 20E"; $resn_exch{"H112 20E"}="2H11 20E"; $resn_exch{"H121 20E"}="1H12 20E";
$resn_exch{"H122 20E"}="2H12 20E"; $resn_exch{"H181 20E"}="1H18 20E"; $resn_exch{"H182 20E"}="2H18 20E";
$resn_exch{"H183 20E"}="3H18 20E"; $resn_exch{"H151 20E"}="1H15 20E"; $resn_exch{"H152 20E"}="2H15 20E";
$resn_exch{"H161 20E"}="1H16 20E"; $resn_exch{"H162 20E"}="2H16 20E"; $resn_exch{"H211 20E"}="1H21 20E";
$resn_exch{"H212 20E"}="2H21 20E"; $resn_exch{"H213 20E"}="3H21 20E"; $resn_exch{"H231 20E"}="1H23 20E";
$resn_exch{"H232 20E"}="2H23 20E"; $resn_exch{"H241 20E"}="1H24 20E"; $resn_exch{"H242 20E"}="2H24 20E";
$resn_exch{"H261 20E"}="1H26 20E"; $resn_exch{"H262 20E"}="2H26 20E"; $resn_exch{"H263 20E"}="3H26 20E";
$resn_exch{"H271 20E"}="1H27 20E"; $resn_exch{"H272 20E"}="2H27 20E"; $resn_exch{"H273 20E"}="3H27 20E";
$resn_exch{" H71 20S"}="1H7  20S"; $resn_exch{" H72 20S"}="2H7  20S"; $resn_exch{" H81 20S"}="1H8  20S";
$resn_exch{" H82 20S"}="2H8  20S"; $resn_exch{" H91 20S"}="1H9  20S"; $resn_exch{" H92 20S"}="2H9  20S";
$resn_exch{"H101 20S"}="1H10 20S"; $resn_exch{"H102 20S"}="2H10 20S"; $resn_exch{"H171 20S"}="1H17 20S";
$resn_exch{"H172 20S"}="2H17 20S"; $resn_exch{"H181 20S"}="1H18 20S"; $resn_exch{"H182 20S"}="2H18 20S";
$resn_exch{"H321 20S"}="1H32 20S"; $resn_exch{"H322 20S"}="2H32 20S"; $resn_exch{"H311 20S"}="1H31 20S";
$resn_exch{"H312 20S"}="2H31 20S"; $resn_exch{"H313 20S"}="3H31 20S"; $resn_exch{"H161 210"}="1H16 210";
$resn_exch{"H162 210"}="2H16 210"; $resn_exch{"H191 210"}="1H19 210"; $resn_exch{"H192 210"}="2H19 210";
$resn_exch{"H221 210"}="1H22 210"; $resn_exch{"H222 210"}="2H22 210"; $resn_exch{" H11 211"}="1H1  211";
$resn_exch{" H12 211"}="2H1  211"; $resn_exch{" H21 211"}="1H2  211"; $resn_exch{" H22 211"}="2H2  211";
$resn_exch{" H31 211"}="1H3  211"; $resn_exch{" H32 211"}="2H3  211"; $resn_exch{" H41 211"}="1H4  211";
$resn_exch{" H42 211"}="2H4  211"; $resn_exch{" H51 211"}="1H5  211"; $resn_exch{" H52 211"}="2H5  211";
$resn_exch{" H61 211"}="1H6  211"; $resn_exch{" H62 211"}="2H6  211"; $resn_exch{"H321 213"}="1H32 213";
$resn_exch{"H322 213"}="2H32 213"; $resn_exch{"H351 213"}="1H35 213"; $resn_exch{"H352 213"}="2H35 213";
$resn_exch{"H321 214"}="1H32 214"; $resn_exch{"H322 214"}="2H32 214"; $resn_exch{"H351 214"}="1H35 214";
$resn_exch{"H352 214"}="2H35 214"; $resn_exch{"H711 214"}="1H71 214"; $resn_exch{"H712 214"}="2H71 214";
$resn_exch{"H713 214"}="3H71 214"; $resn_exch{"H231 215"}="1H23 215"; $resn_exch{"H232 215"}="2H23 215";
$resn_exch{"H311 215"}="1H31 215"; $resn_exch{"H312 215"}="2H31 215"; $resn_exch{"H301 215"}="1H30 215";
$resn_exch{"H302 215"}="2H30 215"; $resn_exch{"H241 215"}="1H24 215"; $resn_exch{"H242 215"}="2H24 215";
$resn_exch{"H271 215"}="1H27 215"; $resn_exch{"H272 215"}="2H27 215"; $resn_exch{"H273 215"}="3H27 215";
$resn_exch{"H261 215"}="1H26 215"; $resn_exch{"H262 215"}="2H26 215"; $resn_exch{"H263 215"}="3H26 215";
$resn_exch{"H211 216"}="1H21 216"; $resn_exch{"H212 216"}="2H21 216"; $resn_exch{"H231 216"}="1H23 216";
$resn_exch{"H232 216"}="2H23 216"; $resn_exch{"H311 216"}="1H31 216"; $resn_exch{"H312 216"}="2H31 216";
$resn_exch{"H611 216"}="1H61 216"; $resn_exch{"H612 216"}="2H61 216"; $resn_exch{"H711 216"}="1H71 216";
$resn_exch{"H712 216"}="2H71 216"; $resn_exch{"H731 216"}="1H73 216"; $resn_exch{"H732 216"}="2H73 216";
$resn_exch{" H11 217"}="1H1  217"; $resn_exch{" H12 217"}="2H1  217"; $resn_exch{" H31 217"}="1H3  217";
$resn_exch{" H32 217"}="2H3  217"; $resn_exch{" H41 217"}="1H4  217"; $resn_exch{" H42 217"}="2H4  217";
$resn_exch{" H61 217"}="1H6  217"; $resn_exch{" H62 217"}="2H6  217"; $resn_exch{"HM21 218"}="1HM2 218";
$resn_exch{"HM22 218"}="2HM2 218"; $resn_exch{"HM23 218"}="3HM2 218"; $resn_exch{"H7A1 218"}="1H7A 218";
$resn_exch{"H7A2 218"}="2H7A 218"; $resn_exch{"HM61 218"}="1HM6 218"; $resn_exch{"HM62 218"}="2HM6 218";
$resn_exch{"HM71 218"}="1HM7 218"; $resn_exch{"HM72 218"}="2HM7 218"; $resn_exch{"HM41 218"}="1HM4 218";
$resn_exch{"HM42 218"}="2HM4 218"; $resn_exch{"HM43 218"}="3HM4 218"; $resn_exch{"H4A1 218"}="1H4A 218";
$resn_exch{"H4A2 218"}="2H4A 218"; $resn_exch{"H121 219"}="1H12 219"; $resn_exch{"H122 219"}="2H12 219";
$resn_exch{"H241 219"}="1H24 219"; $resn_exch{"H242 219"}="2H24 219"; $resn_exch{"H251 219"}="1H25 219";
$resn_exch{"H252 219"}="2H25 219"; $resn_exch{"H261 219"}="1H26 219"; $resn_exch{"H262 219"}="2H26 219";
$resn_exch{"H271 219"}="1H27 219"; $resn_exch{"H272 219"}="2H27 219"; $resn_exch{"H281 219"}="1H28 219";
$resn_exch{"H282 219"}="2H28 219"; $resn_exch{" H21 220"}="1H2  220"; $resn_exch{" H22 220"}="2H2  220";
$resn_exch{" H31 220"}="1H3  220"; $resn_exch{" H32 220"}="2H3  220"; $resn_exch{" H41 220"}="1H4  220";
$resn_exch{" H42 220"}="2H4  220"; $resn_exch{" H71 220"}="1H7  220"; $resn_exch{" H72 220"}="2H7  220";
$resn_exch{" H81 220"}="1H8  220"; $resn_exch{" H82 220"}="2H8  220"; $resn_exch{"H111 220"}="1H11 220";
$resn_exch{"H112 220"}="2H11 220"; $resn_exch{"H121 220"}="1H12 220"; $resn_exch{"H122 220"}="2H12 220";
$resn_exch{"H131 221"}="1H13 221"; $resn_exch{"H132 221"}="2H13 221"; $resn_exch{"H221 222"}="1H22 222";
$resn_exch{"H222 222"}="2H22 222"; $resn_exch{"H241 222"}="1H24 222"; $resn_exch{"H242 222"}="2H24 222";
$resn_exch{"H271 222"}="1H27 222"; $resn_exch{"H272 222"}="2H27 222"; $resn_exch{"HN61 223"}="1HN6 223";
$resn_exch{"HN62 223"}="2HN6 223"; $resn_exch{"H101 223"}="1H10 223"; $resn_exch{"H102 223"}="2H10 223";
$resn_exch{"H6'1 223"}="1H6* 223"; $resn_exch{"H6'2 223"}="2H6* 223"; $resn_exch{" H4' 223"}=" H4* 223";
$resn_exch{"H5'1 223"}="1H5* 223"; $resn_exch{"H5'2 223"}="2H5* 223"; $resn_exch{"HO5' 223"}="HO5* 223";
$resn_exch{" H3' 223"}=" H3* 223"; $resn_exch{"HO3' 223"}="HO3* 223"; $resn_exch{"H2'1 223"}="1H2* 223";
$resn_exch{"H2'2 223"}="2H2* 223"; $resn_exch{" H41 224"}="1H4  224"; $resn_exch{" H42 224"}="2H4  224";
$resn_exch{" H61 224"}="1H6  224"; $resn_exch{" H62 224"}="2H6  224"; $resn_exch{" H71 224"}="1H7  224";
$resn_exch{" H72 224"}="2H7  224"; $resn_exch{" H81 224"}="1H8  224"; $resn_exch{" H82 224"}="2H8  224";
$resn_exch{" H91 224"}="1H9  224"; $resn_exch{" H92 224"}="2H9  224"; $resn_exch{"H151 225"}="1H15 225";
$resn_exch{"H152 225"}="2H15 225"; $resn_exch{"H153 225"}="3H15 225"; $resn_exch{"H141 225"}="1H14 225";
$resn_exch{"H142 225"}="2H14 225"; $resn_exch{"H121 225"}="1H12 225"; $resn_exch{"H122 225"}="2H12 225";
$resn_exch{"H123 225"}="3H12 225"; $resn_exch{"H181 225"}="1H18 225"; $resn_exch{"H182 225"}="2H18 225";
$resn_exch{"H183 225"}="3H18 225"; $resn_exch{"H171 225"}="1H17 225"; $resn_exch{"H172 225"}="2H17 225";
$resn_exch{"H173 225"}="3H17 225"; $resn_exch{"H341 227"}="1H34 227"; $resn_exch{"H342 227"}="2H34 227";
$resn_exch{"H343 227"}="3H34 227"; $resn_exch{"H321 227"}="1H32 227"; $resn_exch{"H322 227"}="2H32 227";
$resn_exch{"H211 227"}="1H21 227"; $resn_exch{"H212 227"}="2H21 227"; $resn_exch{"H213 227"}="3H21 227";
$resn_exch{"H181 227"}="1H18 227"; $resn_exch{"H182 227"}="2H18 227"; $resn_exch{"H183 227"}="3H18 227";
$resn_exch{"H161 227"}="1H16 227"; $resn_exch{"H162 227"}="2H16 227"; $resn_exch{"H301 227"}="1H30 227";
$resn_exch{"H302 227"}="2H30 227"; $resn_exch{"H311 227"}="1H31 227"; $resn_exch{"H312 227"}="2H31 227";
$resn_exch{"H291 227"}="1H29 227"; $resn_exch{"H292 227"}="2H29 227"; $resn_exch{"H101 227"}="1H10 227";
$resn_exch{"H102 227"}="2H10 227"; $resn_exch{"H111 227"}="1H11 227"; $resn_exch{"H112 227"}="2H11 227";
$resn_exch{"H301 228"}="1H30 228"; $resn_exch{"H302 228"}="2H30 228"; $resn_exch{"H221 228"}="1H22 228";
$resn_exch{"H222 228"}="2H22 228"; $resn_exch{"H211 228"}="1H21 228"; $resn_exch{"H212 228"}="2H21 228";
$resn_exch{"H161 228"}="1H16 228"; $resn_exch{"H162 228"}="2H16 228"; $resn_exch{"H151 228"}="1H15 228";
$resn_exch{"H152 228"}="2H15 228"; $resn_exch{"H141 228"}="1H14 228"; $resn_exch{"H142 228"}="2H14 228";
$resn_exch{"H121 228"}="1H12 228"; $resn_exch{"H122 228"}="2H12 228"; $resn_exch{"H123 228"}="3H12 228";
$resn_exch{"H501 22B"}="1H50 22B"; $resn_exch{"H502 22B"}="2H50 22B"; $resn_exch{"H503 22B"}="3H50 22B";
$resn_exch{"H201 22B"}="1H20 22B"; $resn_exch{"H202 22B"}="2H20 22B"; $resn_exch{"H203 22B"}="3H20 22B";
$resn_exch{"H191 22B"}="1H19 22B"; $resn_exch{"H192 22B"}="2H19 22B"; $resn_exch{"H193 22B"}="3H19 22B";
$resn_exch{"H181 22B"}="1H18 22B"; $resn_exch{"H182 22B"}="2H18 22B"; $resn_exch{"H183 22B"}="3H18 22B";
$resn_exch{"H211 22B"}="1H21 22B"; $resn_exch{"H212 22B"}="2H21 22B"; $resn_exch{"H221 22B"}="1H22 22B";
$resn_exch{"H222 22B"}="2H22 22B"; $resn_exch{"H241 22B"}="1H24 22B"; $resn_exch{"H242 22B"}="2H24 22B";
$resn_exch{"H243 22B"}="3H24 22B"; $resn_exch{"H251 22B"}="1H25 22B"; $resn_exch{"H252 22B"}="2H25 22B";
$resn_exch{"H253 22B"}="3H25 22B"; $resn_exch{"H161 22B"}="1H16 22B"; $resn_exch{"H162 22B"}="2H16 22B";
$resn_exch{"H163 22B"}="3H16 22B"; $resn_exch{"H171 22B"}="1H17 22B"; $resn_exch{"H172 22B"}="2H17 22B";
$resn_exch{"H173 22B"}="3H17 22B"; $resn_exch{"H261 22B"}="1H26 22B"; $resn_exch{"H262 22B"}="2H26 22B";
$resn_exch{"H263 22B"}="3H26 22B"; $resn_exch{"H301 22B"}="1H30 22B"; $resn_exch{"H302 22B"}="2H30 22B";
$resn_exch{"H303 22B"}="3H30 22B"; $resn_exch{"H341 22B"}="1H34 22B"; $resn_exch{"H342 22B"}="2H34 22B";
$resn_exch{"H351 22B"}="1H35 22B"; $resn_exch{"H352 22B"}="2H35 22B"; $resn_exch{"H371 22B"}="1H37 22B";
$resn_exch{"H372 22B"}="2H37 22B"; $resn_exch{"H373 22B"}="3H37 22B"; $resn_exch{"H461 22B"}="1H46 22B";
$resn_exch{"H462 22B"}="2H46 22B"; $resn_exch{"H463 22B"}="3H46 22B"; $resn_exch{"H481 22B"}="1H48 22B";
$resn_exch{"H482 22B"}="2H48 22B"; $resn_exch{"H483 22B"}="3H48 22B"; $resn_exch{"H491 22B"}="1H49 22B";
$resn_exch{"H492 22B"}="2H49 22B"; $resn_exch{"H493 22B"}="3H49 22B"; $resn_exch{"HN51 22M"}="1HN5 22M";
$resn_exch{"HN52 22M"}="2HN5 22M"; $resn_exch{"H181 22M"}="1H18 22M"; $resn_exch{"H182 22M"}="2H18 22M";
$resn_exch{"H183 22M"}="3H18 22M"; $resn_exch{"H201 230"}="1H20 230"; $resn_exch{"H202 230"}="2H20 230";
$resn_exch{"H221 230"}="1H22 230"; $resn_exch{"H222 230"}="2H22 230"; $resn_exch{"H241 230"}="1H24 230";
$resn_exch{"H242 230"}="2H24 230"; $resn_exch{"H243 230"}="3H24 230"; $resn_exch{"H191 232"}="1H19 232";
$resn_exch{"H192 232"}="2H19 232"; $resn_exch{"H181 232"}="1H18 232"; $resn_exch{"H182 232"}="2H18 232";
$resn_exch{"H183 232"}="3H18 232"; $resn_exch{"H201 232"}="1H20 232"; $resn_exch{"H202 232"}="2H20 232";
$resn_exch{"H221 232"}="1H22 232"; $resn_exch{"H222 232"}="2H22 232"; $resn_exch{"H231 232"}="1H23 232";
$resn_exch{"H232 232"}="2H23 232"; $resn_exch{"H251 232"}="1H25 232"; $resn_exch{"H252 232"}="2H25 232";
$resn_exch{"H261 232"}="1H26 232"; $resn_exch{"H262 232"}="2H26 232"; $resn_exch{"H271 232"}="1H27 232";
$resn_exch{"H272 232"}="2H27 232"; $resn_exch{"H281 232"}="1H28 232"; $resn_exch{"H282 232"}="2H28 232";
$resn_exch{"H193 232"}="3H19 232"; $resn_exch{"H194 232"}="4H19 232"; $resn_exch{"H184 232"}="4H18 232";
$resn_exch{"H185 232"}="5H18 232"; $resn_exch{"H186 232"}="6H18 232"; $resn_exch{"H203 232"}="3H20 232";
$resn_exch{"H204 232"}="4H20 232"; $resn_exch{"H223 232"}="3H22 232"; $resn_exch{"H224 232"}="4H22 232";
$resn_exch{"H233 232"}="3H23 232"; $resn_exch{"H234 232"}="4H23 232"; $resn_exch{"H253 232"}="3H25 232";
$resn_exch{"H254 232"}="4H25 232"; $resn_exch{"H263 232"}="3H26 232"; $resn_exch{"H264 232"}="4H26 232";
$resn_exch{"H273 232"}="3H27 232"; $resn_exch{"H274 232"}="4H27 232"; $resn_exch{"H391 234"}="1H39 234";
$resn_exch{"H392 234"}="2H39 234"; $resn_exch{"H393 234"}="3H39 234"; $resn_exch{"H401 234"}="1H40 234";
$resn_exch{"H402 234"}="2H40 234"; $resn_exch{"H301 235"}="1H30 235"; $resn_exch{"H302 235"}="2H30 235";
$resn_exch{"H303 235"}="3H30 235"; $resn_exch{"H291 235"}="1H29 235"; $resn_exch{"H292 235"}="2H29 235";
$resn_exch{"H293 235"}="3H29 235"; $resn_exch{"H151 235"}="1H15 235"; $resn_exch{"H152 235"}="2H15 235";
$resn_exch{" H91 235"}="1H9  235"; $resn_exch{" H92 235"}="2H9  235"; $resn_exch{"H101 235"}="1H10 235";
$resn_exch{"H102 235"}="2H10 235"; $resn_exch{"H201 237"}="1H20 237"; $resn_exch{"H202 237"}="2H20 237";
$resn_exch{"H203 237"}="3H20 237"; $resn_exch{" H91 237"}="1H9  237"; $resn_exch{" H92 237"}="2H9  237";
$resn_exch{"H111 237"}="1H11 237"; $resn_exch{"H112 237"}="2H11 237"; $resn_exch{"H191 237"}="1H19 237";
$resn_exch{"H192 237"}="2H19 237"; $resn_exch{"H193 237"}="3H19 237"; $resn_exch{"H191 239"}="1H19 239";
$resn_exch{"H192 239"}="2H19 239"; $resn_exch{"H371 239"}="1H37 239"; $resn_exch{"H372 239"}="2H37 239";
$resn_exch{"H381 239"}="1H38 239"; $resn_exch{"H382 239"}="2H38 239"; $resn_exch{"HAZ1 23D"}="1HAZ 23D";
$resn_exch{"HAZ2 23D"}="2HAZ 23D"; $resn_exch{"HBA1 23D"}="1HBA 23D"; $resn_exch{"HBA2 23D"}="2HBA 23D";
$resn_exch{"HBA3 23D"}="3HBA 23D"; $resn_exch{"HAF1 23D"}="1HAF 23D"; $resn_exch{"HAF2 23D"}="2HAF 23D";
$resn_exch{"HAE1 23D"}="1HAE 23D"; $resn_exch{"HAE2 23D"}="2HAE 23D"; $resn_exch{"HAD1 23D"}="1HAD 23D";
$resn_exch{"HAD2 23D"}="2HAD 23D"; $resn_exch{"HAC1 23D"}="1HAC 23D"; $resn_exch{"HAC2 23D"}="2HAC 23D";
$resn_exch{"HAB1 23D"}="1HAB 23D"; $resn_exch{"HAB2 23D"}="2HAB 23D"; $resn_exch{" H   23F"}=" HN  23F";
$resn_exch{" H2  23F"}=" HN2 23F"; $resn_exch{" HB  23F"}="2HB  23F"; $resn_exch{"H201 23I"}="1H20 23I";
$resn_exch{"H202 23I"}="2H20 23I"; $resn_exch{"H203 23I"}="3H20 23I"; $resn_exch{"H331 23I"}="1H33 23I";
$resn_exch{"H332 23I"}="2H33 23I"; $resn_exch{"H351 23I"}="1H35 23I"; $resn_exch{"H352 23I"}="2H35 23I";
$resn_exch{"H353 23I"}="3H35 23I"; $resn_exch{"H181 23I"}="1H18 23I"; $resn_exch{"H182 23I"}="2H18 23I";
$resn_exch{"H183 23I"}="3H18 23I"; $resn_exch{"H291 23I"}="1H29 23I"; $resn_exch{"H292 23I"}="2H29 23I";
$resn_exch{"H293 23I"}="3H29 23I"; $resn_exch{"H361 23I"}="1H36 23I"; $resn_exch{"H362 23I"}="2H36 23I";
$resn_exch{"H363 23I"}="3H36 23I"; $resn_exch{"H371 23I"}="1H37 23I"; $resn_exch{"H372 23I"}="2H37 23I";
$resn_exch{"H391 23I"}="1H39 23I"; $resn_exch{"H392 23I"}="2H39 23I"; $resn_exch{"H393 23I"}="3H39 23I";
$resn_exch{"H431 23I"}="1H43 23I"; $resn_exch{"H432 23I"}="2H43 23I"; $resn_exch{"H433 23I"}="3H43 23I";
$resn_exch{"H441 23I"}="1H44 23I"; $resn_exch{"H442 23I"}="2H44 23I"; $resn_exch{"H443 23I"}="3H44 23I";
$resn_exch{"H531 23I"}="1H53 23I"; $resn_exch{"H532 23I"}="2H53 23I"; $resn_exch{"H533 23I"}="3H53 23I";
$resn_exch{"H521 23I"}="1H52 23I"; $resn_exch{"H522 23I"}="2H52 23I"; $resn_exch{"H523 23I"}="3H52 23I";
$resn_exch{"H121 23M"}="1H12 23M"; $resn_exch{"H122 23M"}="2H12 23M"; $resn_exch{"H123 23M"}="3H12 23M";
$resn_exch{"H191 23M"}="1H19 23M"; $resn_exch{"H192 23M"}="2H19 23M"; $resn_exch{"HAJ1 23N"}="1HAJ 23N";
$resn_exch{"HAJ2 23N"}="2HAJ 23N"; $resn_exch{"HAK1 23N"}="1HAK 23N"; $resn_exch{"HAK2 23N"}="2HAK 23N";
$resn_exch{" HB1 23S"}="1HB  23S"; $resn_exch{" HB2 23S"}="2HB  23S"; $resn_exch{"H5'1 23T"}="1H5* 23T";
$resn_exch{"H5'2 23T"}="2H5* 23T"; $resn_exch{" H4' 23T"}=" H4* 23T"; $resn_exch{" H1' 23T"}=" H1* 23T";
$resn_exch{"H5M1 23T"}="1H5M 23T"; $resn_exch{"H5M2 23T"}="2H5M 23T"; $resn_exch{"H5M3 23T"}="3H5M 23T";
$resn_exch{"H2'1 23T"}="1H2* 23T"; $resn_exch{"H2'2 23T"}="2H2* 23T"; $resn_exch{"H3'1 23T"}="1H3* 23T";
$resn_exch{"H3'2 23T"}="2H3* 23T"; $resn_exch{"HAC1 240"}="1HAC 240"; $resn_exch{"HAC2 240"}="2HAC 240";
$resn_exch{"HAC3 240"}="3HAC 240"; $resn_exch{"HAU1 240"}="1HAU 240"; $resn_exch{"HAU2 240"}="2HAU 240";
$resn_exch{"HAB1 240"}="1HAB 240"; $resn_exch{"HAB2 240"}="2HAB 240"; $resn_exch{"HAB3 240"}="3HAB 240";
$resn_exch{"HAA1 240"}="1HAA 240"; $resn_exch{"HAA2 240"}="2HAA 240"; $resn_exch{"HAA3 240"}="3HAA 240";
$resn_exch{"HAA1 241"}="1HAA 241"; $resn_exch{"HAA2 241"}="2HAA 241"; $resn_exch{"HAA3 241"}="3HAA 241";
$resn_exch{"HAB1 241"}="1HAB 241"; $resn_exch{"HAB2 241"}="2HAB 241"; $resn_exch{"HAB3 241"}="3HAB 241";
$resn_exch{"HAU1 241"}="1HAU 241"; $resn_exch{"HAU2 241"}="2HAU 241"; $resn_exch{"HAC1 241"}="1HAC 241";
$resn_exch{"HAC2 241"}="2HAC 241"; $resn_exch{"HAC3 241"}="3HAC 241"; $resn_exch{"H151 242"}="1H15 242";
$resn_exch{"H152 242"}="2H15 242"; $resn_exch{"H251 242"}="1H25 242"; $resn_exch{"H252 242"}="2H25 242";
$resn_exch{"H253 242"}="3H25 242"; $resn_exch{"H131 244"}="1H13 244"; $resn_exch{"H132 244"}="2H13 244";
$resn_exch{"H0A1 24A"}="1H0A 24A"; $resn_exch{"H0A2 24A"}="2H0A 24A"; $resn_exch{"H0A3 24A"}="3H0A 24A";
$resn_exch{"H1A1 24A"}="1H1A 24A"; $resn_exch{"H1A2 24A"}="2H1A 24A"; $resn_exch{"HAB1 24A"}="1HAB 24A";
$resn_exch{"HAB2 24A"}="2HAB 24A"; $resn_exch{"HS11 24A"}="1HS1 24A"; $resn_exch{"HS12 24A"}="2HS1 24A";
$resn_exch{"HS21 24A"}="1HS2 24A"; $resn_exch{"HS22 24A"}="2HS2 24A"; $resn_exch{"HS31 24A"}="1HS3 24A";
$resn_exch{"HS32 24A"}="2HS3 24A"; $resn_exch{"HS51 24A"}="1HS5 24A"; $resn_exch{"HS52 24A"}="2HS5 24A";
$resn_exch{"HS61 24A"}="1HS6 24A"; $resn_exch{"HS62 24A"}="2HS6 24A"; $resn_exch{" H71 24B"}="1H7  24B";
$resn_exch{" H72 24B"}="2H7  24B"; $resn_exch{"H131 24I"}="1H13 24I"; $resn_exch{"H132 24I"}="2H13 24I";
$resn_exch{"H121 24I"}="1H12 24I"; $resn_exch{"H122 24I"}="2H12 24I"; $resn_exch{"H7C1 24I"}="1H7C 24I";
$resn_exch{"H7C2 24I"}="2H7C 24I"; $resn_exch{"H6C1 24I"}="1H6C 24I"; $resn_exch{"H6C2 24I"}="2H6C 24I";
$resn_exch{" HN1 24T"}="1HN  24T"; $resn_exch{" HN2 24T"}="2HN  24T"; $resn_exch{" HM1 24T"}="1HM  24T";
$resn_exch{" HM2 24T"}="2HM  24T"; $resn_exch{" HM3 24T"}="3HM  24T"; $resn_exch{"HN41 24X"}="1HN4 24X";
$resn_exch{"HN42 24X"}="2HN4 24X"; $resn_exch{" H71 24X"}="1H7  24X"; $resn_exch{" H72 24X"}="2H7  24X";
$resn_exch{"H161 24X"}="1H16 24X"; $resn_exch{"H162 24X"}="2H16 24X"; $resn_exch{"H111 24X"}="1H11 24X";
$resn_exch{"H112 24X"}="2H11 24X"; $resn_exch{"H121 24X"}="1H12 24X"; $resn_exch{"H122 24X"}="2H12 24X";
$resn_exch{"H123 24X"}="3H12 24X"; $resn_exch{"H131 24X"}="1H13 24X"; $resn_exch{"H132 24X"}="2H13 24X";
$resn_exch{"H133 24X"}="3H13 24X"; $resn_exch{"H171 254"}="1H17 254"; $resn_exch{"H172 254"}="2H17 254";
$resn_exch{"H181 254"}="1H18 254"; $resn_exch{"H182 254"}="2H18 254"; $resn_exch{"H231 254"}="1H23 254";
$resn_exch{"H232 254"}="2H23 254"; $resn_exch{"H233 254"}="3H23 254"; $resn_exch{"H241 254"}="1H24 254";
$resn_exch{"H242 254"}="2H24 254"; $resn_exch{"H243 254"}="3H24 254"; $resn_exch{"H251 254"}="1H25 254";
$resn_exch{"H252 254"}="2H25 254"; $resn_exch{"H253 254"}="3H25 254"; $resn_exch{"H261 254"}="1H26 254";
$resn_exch{"H262 254"}="2H26 254"; $resn_exch{"H263 254"}="3H26 254"; $resn_exch{"H011 255"}="1H01 255";
$resn_exch{"H012 255"}="2H01 255"; $resn_exch{"H013 255"}="3H01 255"; $resn_exch{"H021 255"}="1H02 255";
$resn_exch{"H022 255"}="2H02 255"; $resn_exch{"H023 255"}="3H02 255"; $resn_exch{"H031 255"}="1H03 255";
$resn_exch{"H032 255"}="2H03 255"; $resn_exch{"H033 255"}="3H03 255"; $resn_exch{"H5'1 25A"}="1H5* 25A";
$resn_exch{"H5'2 25A"}="2H5* 25A"; $resn_exch{" H4' 25A"}=" H4* 25A"; $resn_exch{" H1' 25A"}=" H1* 25A";
$resn_exch{" H2' 25A"}=" H2* 25A"; $resn_exch{" H3' 25A"}=" H3* 25A"; $resn_exch{"HN61 25A"}="1HN6 25A";
$resn_exch{"HN62 25A"}="2HN6 25A"; $resn_exch{"H151 25A"}="1H15 25A"; $resn_exch{"H152 25A"}="2H15 25A";
$resn_exch{"H14' 25A"}="H14* 25A"; $resn_exch{"H11' 25A"}="H11* 25A"; $resn_exch{"H161 25A"}="1H16 25A";
$resn_exch{"H162 25A"}="2H16 25A"; $resn_exch{"H12' 25A"}="H12* 25A"; $resn_exch{"H13' 25A"}="H13* 25A";
$resn_exch{"H251 25A"}="1H25 25A"; $resn_exch{"H252 25A"}="2H25 25A"; $resn_exch{"H24' 25A"}="H24* 25A";
$resn_exch{"H21' 25A"}="H21* 25A"; $resn_exch{"H261 25A"}="1H26 25A"; $resn_exch{"H262 25A"}="2H26 25A";
$resn_exch{"H22' 25A"}="H22* 25A"; $resn_exch{"H23' 25A"}="H23* 25A"; $resn_exch{" HN1 25T"}="1HN  25T";
$resn_exch{" HN2 25T"}="2HN  25T"; $resn_exch{" HM1 25T"}="1HM  25T"; $resn_exch{" HM2 25T"}="2HM  25T";
$resn_exch{" HM3 25T"}="3HM  25T"; $resn_exch{"H101 26A"}="1H10 26A"; $resn_exch{"H102 26A"}="2H10 26A";
$resn_exch{"H103 26A"}="3H10 26A"; $resn_exch{" H91 26A"}="1H9  26A"; $resn_exch{" H92 26A"}="2H9  26A";
$resn_exch{" H93 26A"}="3H9  26A"; $resn_exch{" H1' 26A"}=" H1* 26A"; $resn_exch{" H4' 26A"}=" H4* 26A";
$resn_exch{"H5'1 26A"}="1H5* 26A"; $resn_exch{"H5'2 26A"}="2H5* 26A"; $resn_exch{" H5' 26A"}=" H5* 26A";
$resn_exch{" H2' 26A"}=" H2* 26A"; $resn_exch{"HO'2 26A"}="2HO* 26A"; $resn_exch{" H3' 26A"}=" H3* 26A";
$resn_exch{"HO'3 26A"}="3HO* 26A"; $resn_exch{"HN11 26D"}="1HN1 26D"; $resn_exch{"HN12 26D"}="2HN1 26D";
$resn_exch{"HN21 26D"}="1HN2 26D"; $resn_exch{"HN22 26D"}="2HN2 26D"; $resn_exch{"HN21 26P"}="1HN2 26P";
$resn_exch{"HN22 26P"}="2HN2 26P"; $resn_exch{" H31 26P"}="1H3  26P"; $resn_exch{" H32 26P"}="2H3  26P";
$resn_exch{" H41 26P"}="1H4  26P"; $resn_exch{" H42 26P"}="2H4  26P"; $resn_exch{" H51 26P"}="1H5  26P";
$resn_exch{" H52 26P"}="2H5  26P"; $resn_exch{" H21 270"}="1H2  270"; $resn_exch{" H22 270"}="2H2  270";
$resn_exch{"H211 274"}="1H21 274"; $resn_exch{"H212 274"}="2H21 274"; $resn_exch{"H213 274"}="3H21 274";
$resn_exch{"H231 274"}="1H23 274"; $resn_exch{"H232 274"}="2H23 274"; $resn_exch{"H233 274"}="3H23 274";
$resn_exch{"H7C1 274"}="1H7C 274"; $resn_exch{"H7C2 274"}="2H7C 274"; $resn_exch{"H7C3 274"}="3H7C 274";
$resn_exch{"H241 274"}="1H24 274"; $resn_exch{"H242 274"}="2H24 274"; $resn_exch{"H271 274"}="1H27 274";
$resn_exch{"H272 274"}="2H27 274"; $resn_exch{"H281 274"}="1H28 274"; $resn_exch{"H282 274"}="2H28 274";
$resn_exch{"H283 274"}="3H28 274"; $resn_exch{"H261 274"}="1H26 274"; $resn_exch{"H262 274"}="2H26 274";
$resn_exch{"H251 274"}="1H25 274"; $resn_exch{"H252 274"}="2H25 274"; $resn_exch{"H131 276"}="1H13 276";
$resn_exch{"H132 276"}="2H13 276"; $resn_exch{"H133 276"}="3H13 276"; $resn_exch{" H11 277"}="1H1  277";
$resn_exch{" H12 277"}="2H1  277"; $resn_exch{" H21 277"}="1H2  277"; $resn_exch{" H22 277"}="2H2  277";
$resn_exch{" H41 277"}="1H4  277"; $resn_exch{" H42 277"}="2H4  277"; $resn_exch{" H51 277"}="1H5  277";
$resn_exch{" H52 277"}="2H5  277"; $resn_exch{"H231 277"}="1H23 277"; $resn_exch{"H232 277"}="2H23 277";
$resn_exch{"H233 277"}="3H23 277"; $resn_exch{"H271 277"}="1H27 277"; $resn_exch{"H272 277"}="2H27 277";
$resn_exch{"H273 277"}="3H27 277"; $resn_exch{"H331 277"}="1H33 277"; $resn_exch{"H332 277"}="2H33 277";
$resn_exch{"H351 277"}="1H35 277"; $resn_exch{"H352 277"}="2H35 277"; $resn_exch{"H353 277"}="3H35 277";
$resn_exch{"H421 277"}="1H42 277"; $resn_exch{"H422 277"}="2H42 277"; $resn_exch{"H441 277"}="1H44 277";
$resn_exch{"H442 277"}="2H44 277"; $resn_exch{"H451 277"}="1H45 277"; $resn_exch{"H452 277"}="2H45 277";
$resn_exch{"H301 281"}="1H30 281"; $resn_exch{"H302 281"}="2H30 281"; $resn_exch{"H303 281"}="3H30 281";
$resn_exch{"H291 281"}="1H29 281"; $resn_exch{"H292 281"}="2H29 281"; $resn_exch{"H293 281"}="3H29 281";
$resn_exch{"H151 281"}="1H15 281"; $resn_exch{"H152 281"}="2H15 281"; $resn_exch{" H91 281"}="1H9  281";
$resn_exch{" H92 281"}="2H9  281"; $resn_exch{"H101 281"}="1H10 281"; $resn_exch{"H102 281"}="2H10 281";
$resn_exch{" H51 283"}="1H5  283"; $resn_exch{" H52 283"}="2H5  283"; $resn_exch{" H61 283"}="1H6  283";
$resn_exch{" H62 283"}="2H6  283"; $resn_exch{" H63 283"}="3H6  283"; $resn_exch{"H101 283"}="1H10 283";
$resn_exch{"H102 283"}="2H10 283"; $resn_exch{"H111 283"}="1H11 283"; $resn_exch{"H112 283"}="2H11 283";
$resn_exch{"H131 283"}="1H13 283"; $resn_exch{"H132 283"}="2H13 283"; $resn_exch{"H141 288"}="1H14 288";
$resn_exch{"H142 288"}="2H14 288"; $resn_exch{"H191 288"}="1H19 288"; $resn_exch{"H192 288"}="2H19 288";
$resn_exch{"H201 288"}="1H20 288"; $resn_exch{"H202 288"}="2H20 288"; $resn_exch{"H211 288"}="1H21 288";
$resn_exch{"H212 288"}="2H21 288"; $resn_exch{"H221 288"}="1H22 288"; $resn_exch{"H222 288"}="2H22 288";
$resn_exch{"H231 288"}="1H23 288"; $resn_exch{"H232 288"}="2H23 288"; $resn_exch{"H241 288"}="1H24 288";
$resn_exch{"H242 288"}="2H24 288"; $resn_exch{"H1C1 292"}="1H1C 292"; $resn_exch{"H1C2 292"}="2H1C 292";
$resn_exch{"H2C1 292"}="1H2C 292"; $resn_exch{"H2C2 292"}="2H2C 292"; $resn_exch{"H111 292"}="1H11 292";
$resn_exch{"H112 292"}="2H11 292"; $resn_exch{" H71 299"}="1H7  299"; $resn_exch{" H72 299"}="2H7  299";
$resn_exch{" H73 299"}="3H7  299"; $resn_exch{"H101 2A6"}="1H10 2A6"; $resn_exch{"H102 2A6"}="2H10 2A6";
$resn_exch{"H121 2A6"}="1H12 2A6"; $resn_exch{"H122 2A6"}="2H12 2A6"; $resn_exch{"H131 2A6"}="1H13 2A6";
$resn_exch{"H132 2A6"}="2H13 2A6"; $resn_exch{"H141 2A6"}="1H14 2A6"; $resn_exch{"H142 2A6"}="2H14 2A6";
$resn_exch{"H151 2A6"}="1H15 2A6"; $resn_exch{"H152 2A6"}="2H15 2A6"; $resn_exch{"H161 2A6"}="1H16 2A6";
$resn_exch{"H162 2A6"}="2H16 2A6"; $resn_exch{"H131 2AA"}="1H13 2AA"; $resn_exch{"H132 2AA"}="2H13 2AA";
$resn_exch{"H133 2AA"}="3H13 2AA"; $resn_exch{"H121 2AA"}="1H12 2AA"; $resn_exch{"H122 2AA"}="2H12 2AA";
$resn_exch{"H123 2AA"}="3H12 2AA"; $resn_exch{" H9' 2AA"}=" H9* 2AA"; $resn_exch{" H8' 2AA"}=" H8* 2AA";
$resn_exch{" H5' 2AA"}=" H5* 2AA"; $resn_exch{" H4' 2AA"}=" H4* 2AA"; $resn_exch{" H3' 2AA"}=" H3* 2AA";
$resn_exch{"H1'1 2AA"}="1H1* 2AA"; $resn_exch{"H1'2 2AA"}="2H1* 2AA"; $resn_exch{" H91 2AA"}="1H9  2AA";
$resn_exch{" H92 2AA"}="2H9  2AA"; $resn_exch{"H141 2AA"}="1H14 2AA"; $resn_exch{"H142 2AA"}="2H14 2AA";
$resn_exch{"HN21 2AA"}="1HN2 2AA"; $resn_exch{"HN22 2AA"}="2HN2 2AA"; $resn_exch{" H11 2AB"}="1H1  2AB";
$resn_exch{"H12A 2AB"}="2H1  2AB"; $resn_exch{" H21 2AB"}="1H2  2AB"; $resn_exch{" H22 2AB"}="2H2  2AB";
$resn_exch{"HN91 2AB"}="1HN9 2AB"; $resn_exch{"HN92 2AB"}="2HN9 2AB"; $resn_exch{" HN1 2AC"}="1HN  2AC";
$resn_exch{" HN2 2AC"}="2HN  2AC"; $resn_exch{" H81 2AC"}="1H8  2AC"; $resn_exch{" H82 2AC"}="2H8  2AC";
$resn_exch{" H83 2AC"}="3H8  2AC"; $resn_exch{" H5' 2AD"}=" H5* 2AD"; $resn_exch{"H5'1 2AD"}="1H5* 2AD";
$resn_exch{"H5'2 2AD"}="2H5* 2AD"; $resn_exch{" H4' 2AD"}=" H4* 2AD"; $resn_exch{" H3' 2AD"}=" H3* 2AD";
$resn_exch{" H2' 2AD"}=" H2* 2AD"; $resn_exch{" H1' 2AD"}=" H1* 2AD"; $resn_exch{"H6N1 2AD"}="1H6N 2AD";
$resn_exch{"H6N2 2AD"}="2H6N 2AD"; $resn_exch{" HN1 2AD"}="1HN  2AD"; $resn_exch{" HN2 2AD"}="2HN  2AD";
$resn_exch{" HN1 2AF"}="1HN  2AF"; $resn_exch{" HN2 2AF"}="2HN  2AF"; $resn_exch{"H121 2AH"}="1H12 2AH";
$resn_exch{"H122 2AH"}="2H12 2AH"; $resn_exch{"H241 2AH"}="1H24 2AH"; $resn_exch{"H242 2AH"}="2H24 2AH";
$resn_exch{"H411 2AH"}="1H41 2AH"; $resn_exch{"H412 2AH"}="2H41 2AH"; $resn_exch{"H441 2AH"}="1H44 2AH";
$resn_exch{"H442 2AH"}="2H44 2AH"; $resn_exch{"H491 2AH"}="1H49 2AH"; $resn_exch{"H492 2AH"}="2H49 2AH";
$resn_exch{"H493 2AH"}="3H49 2AH"; $resn_exch{"H501 2AH"}="1H50 2AH"; $resn_exch{"H502 2AH"}="2H50 2AH";
$resn_exch{"H503 2AH"}="3H50 2AH"; $resn_exch{"H911 2AH"}="1H91 2AH"; $resn_exch{"H912 2AH"}="2H91 2AH";
$resn_exch{"H913 2AH"}="3H91 2AH"; $resn_exch{"H921 2AH"}="1H92 2AH"; $resn_exch{"H922 2AH"}="2H92 2AH";
$resn_exch{"H923 2AH"}="3H92 2AH"; $resn_exch{"HN11 2AL"}="1HN1 2AL"; $resn_exch{"HN12 2AL"}="2HN1 2AL";
$resn_exch{"HOP2 2AM"}="2HOP 2AM"; $resn_exch{"HOP3 2AM"}="3HOP 2AM"; $resn_exch{"H5'1 2AM"}="1H5* 2AM";
$resn_exch{"H5'2 2AM"}="2H5* 2AM"; $resn_exch{"HO5' 2AM"}="*HO5 2AM"; $resn_exch{" H4' 2AM"}=" H4* 2AM";
$resn_exch{" H3' 2AM"}=" H3* 2AM"; $resn_exch{"HO3' 2AM"}="*HO3 2AM"; $resn_exch{" H2' 2AM"}=" H2* 2AM";
$resn_exch{" H1' 2AM"}=" H1* 2AM"; $resn_exch{"HN61 2AM"}="1HN6 2AM"; $resn_exch{"HN62 2AM"}="2HN6 2AM";
$resn_exch{" H   2AO"}=" HN1 2AO"; $resn_exch{" H2  2AO"}=" HN2 2AO"; $resn_exch{" HB2 2AO"}="HCB1 2AO";
$resn_exch{" HB3 2AO"}="HCB2 2AO"; $resn_exch{" HG2 2AO"}="HCG1 2AO"; $resn_exch{" HG3 2AO"}="HCG2 2AO";
$resn_exch{" HD2 2AO"}="HCD1 2AO"; $resn_exch{" HD3 2AO"}="HCD2 2AO"; $resn_exch{" HE1 2AO"}="HCE1 2AO";
$resn_exch{" HE2 2AO"}="HCE2 2AO"; $resn_exch{" HE3 2AO"}="HCE3 2AO"; $resn_exch{"HN1A 2AP"}="1HN  2AP";
$resn_exch{" HN2 2AP"}="2HN  2AP"; $resn_exch{"HN11 2AQ"}="1HN1 2AQ"; $resn_exch{"HN12 2AQ"}="2HN1 2AQ";
$resn_exch{"HOP2 2AR"}="2HOP 2AR"; $resn_exch{"HOP3 2AR"}="3HOP 2AR"; $resn_exch{" H5' 2AR"}="1H5* 2AR";
$resn_exch{"H5'' 2AR"}="2H5* 2AR"; $resn_exch{" H4' 2AR"}=" H4* 2AR"; $resn_exch{"HX'1 2AR"}="1HX* 2AR";
$resn_exch{"HX'2 2AR"}="2HX* 2AR"; $resn_exch{" H1' 2AR"}=" H1* 2AR"; $resn_exch{" H61 2AR"}="1H6  2AR";
$resn_exch{" H62 2AR"}="2H6  2AR"; $resn_exch{"H2'' 2AR"}="2H2* 2AR"; $resn_exch{" H2' 2AR"}="1H2* 2AR";
$resn_exch{" H3' 2AR"}=" H3* 2AR"; $resn_exch{"HO3' 2AR"}="H3T  2AR"; $resn_exch{"HBB1 2AS"}="1HBB 2AS";
$resn_exch{"HBB2 2AS"}="2HBB 2AS"; $resn_exch{"HBB3 2AS"}="3HBB 2AS"; $resn_exch{"HOP2 2AT"}="2HOP 2AT";
$resn_exch{" H5' 2AT"}="1H5* 2AT"; $resn_exch{"H5'' 2AT"}="2H5* 2AT"; $resn_exch{" H4' 2AT"}=" H4* 2AT";
$resn_exch{" H1' 2AT"}=" H1* 2AT"; $resn_exch{" H71 2AT"}="1H5M 2AT"; $resn_exch{" H72 2AT"}="2H5M 2AT";
$resn_exch{" H73 2AT"}="3H5M 2AT"; $resn_exch{" H2' 2AT"}=" H2* 2AT"; $resn_exch{"HB'1 2AT"}="1HB' 2AT";
$resn_exch{"HB'2 2AT"}="2HB' 2AT"; $resn_exch{"HD'1 2AT"}="1HD' 2AT"; $resn_exch{"HD'2 2AT"}="2HD' 2AT";
$resn_exch{" H3' 2AT"}=" H3* 2AT"; $resn_exch{"HO3' 2AT"}="H3T  2AT"; $resn_exch{"HOP3 2AT"}="3HOP 2AT";
$resn_exch{" H5' 2AU"}="1H5* 2AU"; $resn_exch{"H5'' 2AU"}="2H5* 2AU"; $resn_exch{" H4' 2AU"}=" H4* 2AU";
$resn_exch{" H3' 2AU"}=" H3* 2AU"; $resn_exch{"HO3' 2AU"}="H3T  2AU"; $resn_exch{" H2' 2AU"}="1H2* 2AU";
$resn_exch{" H1' 2AU"}=" H1* 2AU"; $resn_exch{" HA1 2AU"}="1HA  2AU"; $resn_exch{"H2'' 2AU"}="2H2* 2AU";
$resn_exch{" OP2 2AU"}=" O2P 2AU"; $resn_exch{"H2P1 2AU"}="2H1P 2AU"; $resn_exch{" H11 2AZ"}="1H1  2AZ";
$resn_exch{" H12 2AZ"}="2H1  2AZ"; $resn_exch{"H61A 2AZ"}="1H6  2AZ"; $resn_exch{" H62 2AZ"}="2H6  2AZ";
$resn_exch{"H231 2AZ"}="1H23 2AZ"; $resn_exch{"H232 2AZ"}="2H23 2AZ"; $resn_exch{"H391 2AZ"}="1H39 2AZ";
$resn_exch{"H392 2AZ"}="2H39 2AZ"; $resn_exch{"H421 2AZ"}="1H42 2AZ"; $resn_exch{"H422 2AZ"}="2H42 2AZ";
$resn_exch{"H451 2AZ"}="1H45 2AZ"; $resn_exch{"H452 2AZ"}="2H45 2AZ"; $resn_exch{"H481 2AZ"}="1H48 2AZ";
$resn_exch{"H482 2AZ"}="2H48 2AZ"; $resn_exch{"H631 2AZ"}="1H63 2AZ"; $resn_exch{"H632 2AZ"}="2H63 2AZ";
$resn_exch{"H633 2AZ"}="3H63 2AZ"; $resn_exch{" H5' 2BA"}=" H5* 2BA"; $resn_exch{"H5'A 2BA"}="H5*A 2BA";
$resn_exch{" H4' 2BA"}=" H4* 2BA"; $resn_exch{" H3' 2BA"}=" H3* 2BA"; $resn_exch{" H2' 2BA"}=" H2* 2BA";
$resn_exch{"HO2' 2BA"}="HO2* 2BA"; $resn_exch{" H1' 2BA"}=" H1* 2BA"; $resn_exch{"HC5' 2BA"}="HC5* 2BA";
$resn_exch{"HC4' 2BA"}="HC4* 2BA"; $resn_exch{"HC3' 2BA"}="HC3* 2BA"; $resn_exch{"HC2' 2BA"}="HC2* 2BA";
$resn_exch{"HC1' 2BA"}="HC1* 2BA"; $resn_exch{"HOP2 2BD"}="2HOP 2BD"; $resn_exch{"HOP3 2BD"}="3HOP 2BD";
$resn_exch{" H5' 2BD"}="1H5* 2BD"; $resn_exch{"H5'' 2BD"}="2H5* 2BD"; $resn_exch{" H4' 2BD"}=" H4* 2BD";
$resn_exch{" H1' 2BD"}=" H1* 2BD"; $resn_exch{"HAL2 2BD"}="2HAL 2BD"; $resn_exch{"HAM1 2BD"}="1HAM 2BD";
$resn_exch{"HAM2 2BD"}="2HAM 2BD"; $resn_exch{"HAO1 2BD"}="1HAO 2BD"; $resn_exch{"HAO2 2BD"}="2HAO 2BD";
$resn_exch{" H2' 2BD"}="1H2* 2BD"; $resn_exch{"H2'' 2BD"}="2H2* 2BD"; $resn_exch{" H3' 2BD"}=" H3* 2BD";
$resn_exch{"HO3' 2BD"}="H3T  2BD"; $resn_exch{"HOP2 2BF"}="2HOP 2BF"; $resn_exch{"HOP3 2BF"}="3HOP 2BF";
$resn_exch{"H191 2BF"}="1H19 2BF"; $resn_exch{"H192 2BF"}="2H19 2BF"; $resn_exch{"HP12 2BH"}="2HP1 2BH";
$resn_exch{"HP11 2BH"}="1HP1 2BH"; $resn_exch{"H061 2BH"}="1H06 2BH"; $resn_exch{"H062 2BH"}="2H06 2BH";
$resn_exch{"H071 2BH"}="1H07 2BH"; $resn_exch{"H072 2BH"}="2H07 2BH"; $resn_exch{" HD1 2BL"}="1HD  2BL";
$resn_exch{" HD2 2BL"}="2HD  2BL"; $resn_exch{" HE1 2BL"}="1HE  2BL"; $resn_exch{" HE2 2BL"}="2HE  2BL";
$resn_exch{" HE3 2BL"}="3HE  2BL"; $resn_exch{" HJ1 2BL"}="1HJ  2BL"; $resn_exch{" HJ2 2BL"}="2HJ  2BL";
$resn_exch{" HJ3 2BL"}="3HJ  2BL"; $resn_exch{"HC41 2BN"}="1HC4 2BN"; $resn_exch{"HC42 2BN"}="2HC4 2BN";
$resn_exch{"HC61 2BN"}="1HC6 2BN"; $resn_exch{"HC62 2BN"}="2HC6 2BN"; $resn_exch{"HC71 2BN"}="1HC7 2BN";
$resn_exch{"HC72 2BN"}="2HC7 2BN"; $resn_exch{"HC91 2BN"}="1HC9 2BN"; $resn_exch{"HC92 2BN"}="2HC9 2BN";
$resn_exch{"HC93 2BN"}="3HC9 2BN"; $resn_exch{"H101 2BN"}="1H10 2BN"; $resn_exch{"H102 2BN"}="2H10 2BN";
$resn_exch{"H103 2BN"}="3H10 2BN"; $resn_exch{"H111 2BN"}="1H11 2BN"; $resn_exch{"H112 2BN"}="2H11 2BN";
$resn_exch{"H113 2BN"}="3H11 2BN"; $resn_exch{"HOP3 2BT"}="3HOP 2BT"; $resn_exch{"HOP2 2BT"}="2HOP 2BT";
$resn_exch{" H5' 2BT"}="1H5* 2BT"; $resn_exch{"H5'' 2BT"}="2H5* 2BT"; $resn_exch{" H4' 2BT"}=" H4* 2BT";
$resn_exch{" H1' 2BT"}=" H1* 2BT"; $resn_exch{" H71 2BT"}="1H5M 2BT"; $resn_exch{" H72 2BT"}="2H5M 2BT";
$resn_exch{" H73 2BT"}="3H5M 2BT"; $resn_exch{" H2' 2BT"}=" H2* 2BT"; $resn_exch{"HA'1 2BT"}="1HA* 2BT";
$resn_exch{"HA'2 2BT"}="2HA* 2BT"; $resn_exch{"HB'1 2BT"}="1HB* 2BT"; $resn_exch{"HB'2 2BT"}="2HB* 2BT";
$resn_exch{"HC'1 2BT"}="1HC* 2BT"; $resn_exch{"HC'2 2BT"}="2HC* 2BT"; $resn_exch{" H3' 2BT"}=" H3* 2BT";
$resn_exch{"HO3' 2BT"}="H3T  2BT"; $resn_exch{"HD'1 2BT"}="1HD* 2BT"; $resn_exch{"HD'2 2BT"}="2HD* 2BT";
$resn_exch{"HD'3 2BT"}="3HD* 2BT"; $resn_exch{"HOP2 2BU"}="2HOP 2BU"; $resn_exch{"HOP3 2BU"}="3HOP 2BU";
$resn_exch{" H5' 2BU"}=" H5* 2BU"; $resn_exch{"H5'' 2BU"}="H5** 2BU"; $resn_exch{" H4' 2BU"}=" H4* 2BU";
$resn_exch{" H1' 2BU"}=" H1* 2BU"; $resn_exch{" H2' 2BU"}=" H2* 2BU"; $resn_exch{"H2'' 2BU"}="H2** 2BU";
$resn_exch{" H3' 2BU"}=" H3* 2BU"; $resn_exch{"HO3' 2BU"}="H3T  2BU"; $resn_exch{"HC11 2C5"}="1HC1 2C5";
$resn_exch{"HC12 2C5"}="2HC1 2C5"; $resn_exch{"H151 2C5"}="1H15 2C5"; $resn_exch{"H152 2C5"}="2H15 2C5";
$resn_exch{"H153 2C5"}="3H15 2C5"; $resn_exch{"H101 2CA"}="1H10 2CA"; $resn_exch{"H102 2CA"}="2H10 2CA";
$resn_exch{"H121 2CA"}="1H12 2CA"; $resn_exch{"H122 2CA"}="2H12 2CA"; $resn_exch{"H123 2CA"}="3H12 2CA";
$resn_exch{"H131 2CA"}="1H13 2CA"; $resn_exch{"H132 2CA"}="2H13 2CA"; $resn_exch{"H133 2CA"}="3H13 2CA";
$resn_exch{"H021 2CA"}="1H02 2CA"; $resn_exch{"H022 2CA"}="2H02 2CA"; $resn_exch{"H031 2CA"}="1H03 2CA";
$resn_exch{"H032 2CA"}="2H03 2CA"; $resn_exch{"H041 2CA"}="1H04 2CA"; $resn_exch{"H042 2CA"}="2H04 2CA";
$resn_exch{"H061 2CA"}="1H06 2CA"; $resn_exch{"H062 2CA"}="2H06 2CA"; $resn_exch{"H9C1 2CE"}="1H9C 2CE";
$resn_exch{"H9C2 2CE"}="2H9C 2CE"; $resn_exch{"H141 2CE"}="1H14 2CE"; $resn_exch{"H142 2CE"}="2H14 2CE";
$resn_exch{"H241 2CE"}="1H24 2CE"; $resn_exch{"H242 2CE"}="2H24 2CE"; $resn_exch{"H281 2CE"}="1H28 2CE";
$resn_exch{"H282 2CE"}="2H28 2CE"; $resn_exch{"H291 2CE"}="1H29 2CE"; $resn_exch{"H292 2CE"}="2H29 2CE";
$resn_exch{"H311 2CE"}="1H31 2CE"; $resn_exch{"H312 2CE"}="2H31 2CE"; $resn_exch{"H321 2CE"}="1H32 2CE";
$resn_exch{"H322 2CE"}="2H32 2CE"; $resn_exch{"H341 2CE"}="1H34 2CE"; $resn_exch{"H342 2CE"}="2H34 2CE";
$resn_exch{"H371 2CE"}="1H37 2CE"; $resn_exch{"H372 2CE"}="2H37 2CE"; $resn_exch{" HN1 2CG"}="1HN  2CG";
$resn_exch{" HN2 2CG"}="2HN  2CG"; $resn_exch{" H71 2CL"}="1H7  2CL"; $resn_exch{" H72 2CL"}="2H7  2CL";
$resn_exch{"HN41 2CM"}="1HN4 2CM"; $resn_exch{"HN42 2CM"}="2HN4 2CM"; $resn_exch{"HC61 2CM"}="1HC6 2CM";
$resn_exch{"HC62 2CM"}="2HC6 2CM"; $resn_exch{"HC63 2CM"}="3HC6 2CM"; $resn_exch{" H61 2CP"}="1H6  2CP";
$resn_exch{" H62 2CP"}="2H6  2CP"; $resn_exch{"HO'2 2CP"}="2HO' 2CP"; $resn_exch{"H5'1 2CP"}="1H5' 2CP";
$resn_exch{"H5'2 2CP"}="2H5' 2CP"; $resn_exch{" HB1 2CP"}="1HB  2CP"; $resn_exch{" HB2 2CP"}="2HB  2CP";
$resn_exch{"HP91 2CP"}="1HP9 2CP"; $resn_exch{"HP92 2CP"}="2HP9 2CP"; $resn_exch{"HP93 2CP"}="3HP9 2CP";
$resn_exch{"HP81 2CP"}="1HP8 2CP"; $resn_exch{"HP82 2CP"}="2HP8 2CP"; $resn_exch{"HP83 2CP"}="3HP8 2CP";
$resn_exch{"HOP3 2CP"}="3HOP 2CP"; $resn_exch{"HP51 2CP"}="1HP5 2CP"; $resn_exch{"HP52 2CP"}="2HP5 2CP";
$resn_exch{"HP41 2CP"}="1HP4 2CP"; $resn_exch{"HP42 2CP"}="2HP4 2CP"; $resn_exch{"HP21 2CP"}="1HP2 2CP";
$resn_exch{"HP22 2CP"}="2HP2 2CP"; $resn_exch{"HP11 2CP"}="1HP1 2CP"; $resn_exch{"HP12 2CP"}="2HP1 2CP";
$resn_exch{"HS11 2CP"}="1HS1 2CP"; $resn_exch{"HS12 2CP"}="2HS1 2CP"; $resn_exch{"HS31 2CP"}="1HS3 2CP";
$resn_exch{"HS32 2CP"}="2HS3 2CP"; $resn_exch{"HS33 2CP"}="3HS3 2CP"; $resn_exch{"HOS5 2CP"}="5HOS 2CP";
$resn_exch{" H11 2CS"}="1H1  2CS"; $resn_exch{"H12A 2CS"}="2H1  2CS"; $resn_exch{"H13A 2CS"}="3H1  2CS";
$resn_exch{" H71 2CS"}="1H7  2CS"; $resn_exch{" H72 2CS"}="2H7  2CS"; $resn_exch{"H101 2CS"}="1H10 2CS";
$resn_exch{"H102 2CS"}="2H10 2CS"; $resn_exch{"H251 2CS"}="1H25 2CS"; $resn_exch{"H252 2CS"}="2H25 2CS";
$resn_exch{"H411 2CS"}="1H41 2CS"; $resn_exch{"H412 2CS"}="2H41 2CS"; $resn_exch{"H413 2CS"}="3H41 2CS";
$resn_exch{"H391 2CS"}="1H39 2CS"; $resn_exch{"H392 2CS"}="2H39 2CS"; $resn_exch{"H393 2CS"}="3H39 2CS";
$resn_exch{"H401 2CS"}="1H40 2CS"; $resn_exch{"H402 2CS"}="2H40 2CS"; $resn_exch{"H403 2CS"}="3H40 2CS";
$resn_exch{" H61 2CS"}="1H6  2CS"; $resn_exch{" H62 2CS"}="2H6  2CS"; $resn_exch{" H63 2CS"}="3H6  2CS";
$resn_exch{"H181 2D7"}="1H18 2D7"; $resn_exch{"H182 2D7"}="2H18 2D7"; $resn_exch{"H183 2D7"}="3H18 2D7";
$resn_exch{"H311 2D9"}="1H31 2D9"; $resn_exch{"H312 2D9"}="2H31 2D9"; $resn_exch{"H313 2D9"}="3H31 2D9";
$resn_exch{"H181 2D9"}="1H18 2D9"; $resn_exch{"H182 2D9"}="2H18 2D9"; $resn_exch{"H161 2D9"}="1H16 2D9";
$resn_exch{"H162 2D9"}="2H16 2D9"; $resn_exch{"H171 2D9"}="1H17 2D9"; $resn_exch{"H172 2D9"}="2H17 2D9";
$resn_exch{"H141 2D9"}="1H14 2D9"; $resn_exch{"H142 2D9"}="2H14 2D9"; $resn_exch{"H131 2D9"}="1H13 2D9";
$resn_exch{"H132 2D9"}="2H13 2D9"; $resn_exch{"HOP2 2DA"}="2HOP 2DA"; $resn_exch{"HOP3 2DA"}="3HOP 2DA";
$resn_exch{" H5' 2DA"}="1H5* 2DA"; $resn_exch{"H5'' 2DA"}="2H5* 2DA"; $resn_exch{" H4' 2DA"}=" H4* 2DA";
$resn_exch{"H3'1 2DA"}="1H3* 2DA"; $resn_exch{"H3'2 2DA"}="2H3* 2DA"; $resn_exch{" H2' 2DA"}="1H2* 2DA";
$resn_exch{"H2'' 2DA"}="2H2* 2DA"; $resn_exch{" H1' 2DA"}=" H1* 2DA"; $resn_exch{"HN61 2DA"}="1HN6 2DA";
$resn_exch{"HN62 2DA"}="2HN6 2DA"; $resn_exch{" H11 2DB"}="1H1  2DB"; $resn_exch{" H12 2DB"}="2H1  2DB";
$resn_exch{" H31 2DB"}="1H3  2DB"; $resn_exch{" H32 2DB"}="2H3  2DB"; $resn_exch{"H171 2DD"}="1H17 2DD";
$resn_exch{"H172 2DD"}="2H17 2DD"; $resn_exch{"H4N1 2DD"}="1H4N 2DD"; $resn_exch{"H4N2 2DD"}="2H4N 2DD";
$resn_exch{"HOP2 2DF"}="2HOP 2DF"; $resn_exch{"HOP3 2DF"}="3HOP 2DF"; $resn_exch{"H5'1 2DF"}="1H5* 2DF";
$resn_exch{"H5'2 2DF"}="2H5* 2DF"; $resn_exch{" H4' 2DF"}=" H4* 2DF"; $resn_exch{" H1' 2DF"}=" H1* 2DF";
$resn_exch{" H3' 2DF"}=" H3* 2DF"; $resn_exch{"H2'1 2DF"}="1H2* 2DF"; $resn_exch{"H2'2 2DF"}="2H2* 2DF";
$resn_exch{" H21 2DG"}="1H2  2DG"; $resn_exch{" H22 2DG"}="2H2  2DG"; $resn_exch{" H61 2DG"}="1H6  2DG";
$resn_exch{" H62 2DG"}="2H6  2DG"; $resn_exch{"H5'1 2DI"}="1H5* 2DI"; $resn_exch{"H5'2 2DI"}="2H5* 2DI";
$resn_exch{" H5' 2DI"}=" H5* 2DI"; $resn_exch{" H4' 2DI"}=" H4* 2DI"; $resn_exch{" H1' 2DI"}=" H1* 2DI";
$resn_exch{"H2'1 2DI"}="1H2* 2DI"; $resn_exch{"H2'2 2DI"}="2H2* 2DI"; $resn_exch{"H3'1 2DI"}="1H3* 2DI";
$resn_exch{"H3'2 2DI"}="2H3* 2DI"; $resn_exch{"HOP2 2DM"}="2HOP 2DM"; $resn_exch{"HO3' 2DM"}="H3T  2DM";
$resn_exch{"HOP3 2DM"}="3HOP 2DM"; $resn_exch{" H   2DO"}="1HN  2DO"; $resn_exch{" H2  2DO"}="2HN  2DO";
$resn_exch{" HC  2DO"}=" H   2DO"; $resn_exch{" HB2 2DO"}="1HCB 2DO"; $resn_exch{" HB3 2DO"}="2HCB 2DO";
$resn_exch{" HG2 2DO"}="1HCG 2DO"; $resn_exch{" HG3 2DO"}="2HCG 2DO"; $resn_exch{" HD2 2DO"}="1HCD 2DO";
$resn_exch{" HD3 2DO"}="2HCD 2DO"; $resn_exch{" HE1 2DO"}="1HCE 2DO"; $resn_exch{" HE2 2DO"}="2HCE 2DO";
$resn_exch{" HE3 2DO"}="3HCE 2DO"; $resn_exch{" H11 2DP"}="1H1  2DP"; $resn_exch{" H12 2DP"}="2H1  2DP";
$resn_exch{" H31 2DP"}="1H3  2DP"; $resn_exch{" H32 2DP"}="2H3  2DP"; $resn_exch{" H41 2DP"}="1H4  2DP";
$resn_exch{" H42 2DP"}="2H4  2DP"; $resn_exch{" H61 2DP"}="1H6  2DP"; $resn_exch{" H62 2DP"}="2H6  2DP";
$resn_exch{"H111 2DP"}="1H11 2DP"; $resn_exch{"H112 2DP"}="2H11 2DP"; $resn_exch{"H121 2DP"}="1H12 2DP";
$resn_exch{"H122 2DP"}="2H12 2DP"; $resn_exch{"H141 2DP"}="1H14 2DP"; $resn_exch{"H142 2DP"}="2H14 2DP";
$resn_exch{"H143 2DP"}="3H14 2DP"; $resn_exch{"H151 2DP"}="1H15 2DP"; $resn_exch{"H152 2DP"}="2H15 2DP";
$resn_exch{"H161 2DP"}="1H16 2DP"; $resn_exch{"H162 2DP"}="2H16 2DP"; $resn_exch{"H171 2DP"}="1H17 2DP";
$resn_exch{"H172 2DP"}="2H17 2DP"; $resn_exch{"H191 2DP"}="1H19 2DP"; $resn_exch{"H192 2DP"}="2H19 2DP";
$resn_exch{"H193 2DP"}="3H19 2DP"; $resn_exch{"H201 2DP"}="1H20 2DP"; $resn_exch{"H202 2DP"}="2H20 2DP";
$resn_exch{"H211 2DP"}="1H21 2DP"; $resn_exch{"H212 2DP"}="2H21 2DP"; $resn_exch{"H221 2DP"}="1H22 2DP";
$resn_exch{"H222 2DP"}="2H22 2DP"; $resn_exch{"H241 2DP"}="1H24 2DP"; $resn_exch{"H242 2DP"}="2H24 2DP";
$resn_exch{"H243 2DP"}="3H24 2DP"; $resn_exch{"H251 2DP"}="1H25 2DP"; $resn_exch{"H252 2DP"}="2H25 2DP";
$resn_exch{"H261 2DP"}="1H26 2DP"; $resn_exch{"H262 2DP"}="2H26 2DP"; $resn_exch{"H271 2DP"}="1H27 2DP";
$resn_exch{"H272 2DP"}="2H27 2DP"; $resn_exch{"H291 2DP"}="1H29 2DP"; $resn_exch{"H292 2DP"}="2H29 2DP";
$resn_exch{"H293 2DP"}="3H29 2DP"; $resn_exch{"H301 2DP"}="1H30 2DP"; $resn_exch{"H302 2DP"}="2H30 2DP";
$resn_exch{"H303 2DP"}="3H30 2DP"; $resn_exch{"H411 2DP"}="1H41 2DP"; $resn_exch{"H412 2DP"}="2H41 2DP";
$resn_exch{"H421 2DP"}="1H42 2DP"; $resn_exch{"H422 2DP"}="2H42 2DP"; $resn_exch{"H441 2DP"}="1H44 2DP";
$resn_exch{"H442 2DP"}="2H44 2DP"; $resn_exch{"H443 2DP"}="3H44 2DP"; $resn_exch{"H451 2DP"}="1H45 2DP";
$resn_exch{"H452 2DP"}="2H45 2DP"; $resn_exch{"H461 2DP"}="1H46 2DP"; $resn_exch{"H462 2DP"}="2H46 2DP";
$resn_exch{"H471 2DP"}="1H47 2DP"; $resn_exch{"H472 2DP"}="2H47 2DP"; $resn_exch{"H491 2DP"}="1H49 2DP";
$resn_exch{"H492 2DP"}="2H49 2DP"; $resn_exch{"H493 2DP"}="3H49 2DP"; $resn_exch{"H501 2DP"}="1H50 2DP";
$resn_exch{"H502 2DP"}="2H50 2DP"; $resn_exch{"H511 2DP"}="1H51 2DP"; $resn_exch{"H512 2DP"}="2H51 2DP";
$resn_exch{"H521 2DP"}="1H52 2DP"; $resn_exch{"H522 2DP"}="2H52 2DP"; $resn_exch{"H541 2DP"}="1H54 2DP";
$resn_exch{"H542 2DP"}="2H54 2DP"; $resn_exch{"H543 2DP"}="3H54 2DP"; $resn_exch{"H551 2DP"}="1H55 2DP";
$resn_exch{"H552 2DP"}="2H55 2DP"; $resn_exch{"H561 2DP"}="1H56 2DP"; $resn_exch{"H562 2DP"}="2H56 2DP";
$resn_exch{"H571 2DP"}="1H57 2DP"; $resn_exch{"H572 2DP"}="2H57 2DP"; $resn_exch{"H591 2DP"}="1H59 2DP";
$resn_exch{"H592 2DP"}="2H59 2DP"; $resn_exch{"H593 2DP"}="3H59 2DP"; $resn_exch{"H601 2DP"}="1H60 2DP";
$resn_exch{"H602 2DP"}="2H60 2DP"; $resn_exch{"H603 2DP"}="3H60 2DP"; $resn_exch{" HM1 2DP"}="1HM  2DP";
$resn_exch{" HM2 2DP"}="2HM  2DP"; $resn_exch{" HM3 2DP"}="3HM  2DP"; $resn_exch{"HOP2 2DT"}="2HOP 2DT";
$resn_exch{"HOP3 2DT"}="3HOP 2DT"; $resn_exch{" H71 2DT"}="1H5M 2DT"; $resn_exch{" H72 2DT"}="2H5M 2DT";
$resn_exch{" H73 2DT"}="3H5M 2DT"; $resn_exch{" H2' 2DT"}="1H2* 2DT"; $resn_exch{"H2'' 2DT"}="2H2* 2DT";
$resn_exch{" H5' 2DT"}="1H5* 2DT"; $resn_exch{"H5'' 2DT"}="2H5* 2DT"; $resn_exch{" H4' 2DT"}=" H4* 2DT";
$resn_exch{" H1' 2DT"}=" H1* 2DT"; $resn_exch{"H3'1 2DT"}="1H3* 2DT"; $resn_exch{"H3'2 2DT"}="2H3* 2DT";
$resn_exch{"H191 2E1"}="1H19 2E1"; $resn_exch{"H192 2E1"}="2H19 2E1"; $resn_exch{"H201 2E1"}="1H20 2E1";
$resn_exch{"H202 2E1"}="2H20 2E1"; $resn_exch{"H171 2E1"}="1H17 2E1"; $resn_exch{"H172 2E1"}="2H17 2E1";
$resn_exch{"H161 2E1"}="1H16 2E1"; $resn_exch{"H162 2E1"}="2H16 2E1"; $resn_exch{"H261 2E1"}="1H26 2E1";
$resn_exch{"H262 2E1"}="2H26 2E1"; $resn_exch{"H271 2E1"}="1H27 2E1"; $resn_exch{"H272 2E1"}="2H27 2E1";
$resn_exch{"H273 2E1"}="3H27 2E1"; $resn_exch{"H241 2EA"}="1H24 2EA"; $resn_exch{"H242 2EA"}="2H24 2EA";
$resn_exch{"H251 2EA"}="1H25 2EA"; $resn_exch{"H252 2EA"}="2H25 2EA"; $resn_exch{"HN41 2EA"}="1HN4 2EA";
$resn_exch{"HN42 2EA"}="2HN4 2EA"; $resn_exch{" H5' 2EG"}="1H5* 2EG"; $resn_exch{"H5'' 2EG"}="2H5* 2EG";
$resn_exch{" H4' 2EG"}=" H4* 2EG"; $resn_exch{" H3' 2EG"}=" H3* 2EG"; $resn_exch{" H2' 2EG"}="1H2* 2EG";
$resn_exch{"H2'' 2EG"}="2H2* 2EG"; $resn_exch{" H1' 2EG"}=" H1* 2EG"; $resn_exch{" H21 2EG"}="1H2  2EG";
$resn_exch{"H211 2EG"}="1H21 2EG"; $resn_exch{"H212 2EG"}="2H21 2EG"; $resn_exch{"H221 2EG"}="1H22 2EG";
$resn_exch{"H222 2EG"}="2H22 2EG"; $resn_exch{"H223 2EG"}="3H22 2EG"; $resn_exch{" H31 2EP"}="1H3  2EP";
$resn_exch{" H32 2EP"}="2H3  2EP"; $resn_exch{" H41 2EP"}="1H4  2EP"; $resn_exch{" H42 2EP"}="2H4  2EP";
$resn_exch{" H51 2EP"}="1H5  2EP"; $resn_exch{" H52 2EP"}="2H5  2EP"; $resn_exch{" H61 2EP"}="1H6  2EP";
$resn_exch{" H62 2EP"}="2H6  2EP"; $resn_exch{"H1'1 2EP"}="1H1' 2EP"; $resn_exch{"H1'2 2EP"}="2H1' 2EP";
$resn_exch{"H2'1 2EP"}="1H2' 2EP"; $resn_exch{"H2'2 2EP"}="2H2' 2EP"; $resn_exch{"H2'3 2EP"}="3H2' 2EP";
$resn_exch{"H201 2EQ"}="1H20 2EQ"; $resn_exch{"H202 2EQ"}="2H20 2EQ"; $resn_exch{"H211 2EQ"}="1H21 2EQ";
$resn_exch{"H212 2EQ"}="2H21 2EQ"; $resn_exch{"H221 2EQ"}="1H22 2EQ"; $resn_exch{"H222 2EQ"}="2H22 2EQ";
$resn_exch{"H231 2EQ"}="1H23 2EQ"; $resn_exch{"H232 2EQ"}="2H23 2EQ"; $resn_exch{"H191 2EQ"}="1H19 2EQ";
$resn_exch{"H192 2EQ"}="2H19 2EQ"; $resn_exch{"H111 2EQ"}="1H11 2EQ"; $resn_exch{"H112 2EQ"}="2H11 2EQ";
$resn_exch{"H121 2EQ"}="1H12 2EQ"; $resn_exch{"H122 2EQ"}="2H12 2EQ"; $resn_exch{"H123 2EQ"}="3H12 2EQ";
$resn_exch{" H61 2EZ"}="1H6  2EZ"; $resn_exch{" H62 2EZ"}="2H6  2EZ"; $resn_exch{" H71 2EZ"}="1H7  2EZ";
$resn_exch{" H72 2EZ"}="2H7  2EZ"; $resn_exch{" H73 2EZ"}="3H7  2EZ"; $resn_exch{"H5'1 2FA"}="1H5' 2FA";
$resn_exch{"H5'2 2FA"}="2H5' 2FA"; $resn_exch{"HN61 2FA"}="1HN6 2FA"; $resn_exch{"HN62 2FA"}="2HN6 2FA";
$resn_exch{"H5'1 2FD"}="1H5' 2FD"; $resn_exch{"H5'2 2FD"}="2H5' 2FD"; $resn_exch{"HN61 2FD"}="1HN6 2FD";
$resn_exch{"HN62 2FD"}="2HN6 2FD"; $resn_exch{"H2'1 2FD"}="1H2' 2FD"; $resn_exch{"H2'2 2FD"}="2H2' 2FD";
$resn_exch{" H61 2FG"}="1H6  2FG"; $resn_exch{" H62 2FG"}="2H6  2FG"; $resn_exch{"HAA1 2FH"}="1HAA 2FH";
$resn_exch{"HAA2 2FH"}="2HAA 2FH"; $resn_exch{"HBA1 2FH"}="1HBA 2FH"; $resn_exch{"HBA2 2FH"}="2HBA 2FH";
$resn_exch{"HMA1 2FH"}="1HMA 2FH"; $resn_exch{"HMA2 2FH"}="2HMA 2FH"; $resn_exch{"HMA3 2FH"}="3HMA 2FH";
$resn_exch{"HMB1 2FH"}="1HMB 2FH"; $resn_exch{"HMB2 2FH"}="2HMB 2FH"; $resn_exch{"HMB3 2FH"}="3HMB 2FH";
$resn_exch{"HBB1 2FH"}="1HBB 2FH"; $resn_exch{"HBB2 2FH"}="2HBB 2FH"; $resn_exch{"HMC1 2FH"}="1HMC 2FH";
$resn_exch{"HMC2 2FH"}="2HMC 2FH"; $resn_exch{"HMC3 2FH"}="3HMC 2FH"; $resn_exch{"HBC1 2FH"}="1HBC 2FH";
$resn_exch{"HBC2 2FH"}="2HBC 2FH"; $resn_exch{"HMD1 2FH"}="1HMD 2FH"; $resn_exch{"HMD2 2FH"}="2HMD 2FH";
$resn_exch{"HMD3 2FH"}="3HMD 2FH"; $resn_exch{"HAD1 2FH"}="1HAD 2FH"; $resn_exch{"HAD2 2FH"}="2HAD 2FH";
$resn_exch{"HBD1 2FH"}="1HBD 2FH"; $resn_exch{"HBD2 2FH"}="2HBD 2FH"; $resn_exch{" H61 2FL"}="1H6  2FL";
$resn_exch{" H62 2FL"}="2H6  2FL"; $resn_exch{"H6'1 2FL"}="1H6' 2FL"; $resn_exch{"H6'2 2FL"}="2H6' 2FL";
$resn_exch{"HO1' 2FL"}="'HO1 2FL"; $resn_exch{"HO2' 2FL"}="'HO2 2FL"; $resn_exch{"HO3' 2FL"}="'HO3 2FL";
$resn_exch{"HO6' 2FL"}="'HO6 2FL"; $resn_exch{" HN1 2FM"}="1HN  2FM"; $resn_exch{" HN2 2FM"}="2HN  2FM";
$resn_exch{" HB1 2FM"}="1HB  2FM"; $resn_exch{" HB2 2FM"}="2HB  2FM"; $resn_exch{" HG1 2FM"}="1HG  2FM";
$resn_exch{" HG2 2FM"}="2HG  2FM"; $resn_exch{" H11 2FP"}="1H1  2FP"; $resn_exch{" H12 2FP"}="2H1  2FP";
$resn_exch{" H61 2FP"}="1H6  2FP"; $resn_exch{"H62A 2FP"}="2H6  2FP"; $resn_exch{"H221 2GG"}="1H22 2GG";
$resn_exch{"H222 2GG"}="2H22 2GG"; $resn_exch{"H223 2GG"}="3H22 2GG"; $resn_exch{"H261 2GG"}="1H26 2GG";
$resn_exch{"H262 2GG"}="2H26 2GG"; $resn_exch{"H271 2GG"}="1H27 2GG"; $resn_exch{"H272 2GG"}="2H27 2GG";
$resn_exch{"H273 2GG"}="3H27 2GG"; $resn_exch{"H251 2GJ"}="1H25 2GJ"; $resn_exch{"H252 2GJ"}="2H25 2GJ";
$resn_exch{"H261 2GJ"}="1H26 2GJ"; $resn_exch{"H262 2GJ"}="2H26 2GJ"; $resn_exch{"H281 2GJ"}="1H28 2GJ";
$resn_exch{"H282 2GJ"}="2H28 2GJ"; $resn_exch{"H291 2GJ"}="1H29 2GJ"; $resn_exch{"H292 2GJ"}="2H29 2GJ";
$resn_exch{"H231 2GJ"}="1H23 2GJ"; $resn_exch{"H232 2GJ"}="2H23 2GJ"; $resn_exch{"H111 2GJ"}="1H11 2GJ";
$resn_exch{"H112 2GJ"}="2H11 2GJ"; $resn_exch{"H121 2GJ"}="1H12 2GJ"; $resn_exch{"H122 2GJ"}="2H12 2GJ";
$resn_exch{"H123 2GJ"}="3H12 2GJ"; $resn_exch{"H321 2GJ"}="1H32 2GJ"; $resn_exch{"H322 2GJ"}="2H32 2GJ";
$resn_exch{"H323 2GJ"}="3H32 2GJ"; $resn_exch{"H311 2GJ"}="1H31 2GJ"; $resn_exch{"H312 2GJ"}="2H31 2GJ";
$resn_exch{"H313 2GJ"}="3H31 2GJ"; $resn_exch{" H21 2GL"}="1H2  2GL"; $resn_exch{" H22 2GL"}="2H2  2GL";
$resn_exch{" H61 2GL"}="1H6  2GL"; $resn_exch{" H62 2GL"}="2H6  2GL"; $resn_exch{" H63 2GL"}="3H6  2GL";
$resn_exch{"HOP2 2GP"}="2HOP 2GP"; $resn_exch{"HOP3 2GP"}="3HOP 2GP"; $resn_exch{"HO5' 2GP"}="*HO5 2GP";
$resn_exch{"H5'1 2GP"}="1H5* 2GP"; $resn_exch{"H5'2 2GP"}="2H5* 2GP"; $resn_exch{" H4' 2GP"}=" H4* 2GP";
$resn_exch{" H3' 2GP"}=" H3* 2GP"; $resn_exch{"HO3' 2GP"}="*HO3 2GP"; $resn_exch{" H2' 2GP"}=" H2* 2GP";
$resn_exch{" H1' 2GP"}=" H1* 2GP"; $resn_exch{"HN21 2GP"}="1HN2 2GP"; $resn_exch{"HN22 2GP"}="2HN2 2GP";
$resn_exch{" H61 2GS"}="1H6  2GS"; $resn_exch{" H62 2GS"}="2H6  2GS"; $resn_exch{" H71 2GS"}="1H7  2GS";
$resn_exch{" H72 2GS"}="2H7  2GS"; $resn_exch{" H73 2GS"}="3H7  2GS"; $resn_exch{"HOP2 2GT"}="2HOP 2GT";
$resn_exch{" H5' 2GT"}="1H5* 2GT"; $resn_exch{"H5'' 2GT"}="2H5* 2GT"; $resn_exch{" H4' 2GT"}=" H4* 2GT";
$resn_exch{" H1' 2GT"}=" H1* 2GT"; $resn_exch{" H71 2GT"}="1H5M 2GT"; $resn_exch{" H72 2GT"}="2H5M 2GT";
$resn_exch{" H73 2GT"}="3H5M 2GT"; $resn_exch{" H2' 2GT"}=" H2* 2GT"; $resn_exch{"HB'1 2GT"}="1HB' 2GT";
$resn_exch{"HB'2 2GT"}="2HB' 2GT"; $resn_exch{" H3' 2GT"}=" H3* 2GT"; $resn_exch{"HO3' 2GT"}="H3T  2GT";
$resn_exch{"HOP3 2GT"}="3HOP 2GT"; $resn_exch{"H1C1 2HA"}="1H1C 2HA"; $resn_exch{"H1C2 2HA"}="2H1C 2HA";
$resn_exch{"H3C1 2HA"}="1H3C 2HA"; $resn_exch{"H3C2 2HA"}="2H3C 2HA"; $resn_exch{" H61 2HR"}="1H6  2HR";
$resn_exch{" H62 2HR"}="2H6  2HR"; $resn_exch{" H63 2HR"}="3H6  2HR"; $resn_exch{" H51 2HR"}="1H5  2HR";
$resn_exch{" H52 2HR"}="2H5  2HR"; $resn_exch{" H41 2HR"}="1H4  2HR"; $resn_exch{" H42 2HR"}="2H4  2HR";
$resn_exch{" H31 2HR"}="1H3  2HR"; $resn_exch{" H32 2HR"}="2H3  2HR"; $resn_exch{" H21 2HR"}="1H2  2HR";
$resn_exch{" H22 2HR"}="2H2  2HR"; $resn_exch{" H11 2HR"}="1H1  2HR"; $resn_exch{"H12A 2HR"}="2H1  2HR";
$resn_exch{" H71 2HR"}="1H7  2HR"; $resn_exch{" H72 2HR"}="2H7  2HR"; $resn_exch{" H91 2HR"}="1H9  2HR";
$resn_exch{" H92 2HR"}="2H9  2HR"; $resn_exch{" H93 2HR"}="3H9  2HR"; $resn_exch{"H101 2HR"}="1H10 2HR";
$resn_exch{"H102 2HR"}="2H10 2HR"; $resn_exch{" H61 2HS"}="1H6  2HS"; $resn_exch{" H62 2HS"}="2H6  2HS";
$resn_exch{" H63 2HS"}="3H6  2HS"; $resn_exch{" H51 2HS"}="1H5  2HS"; $resn_exch{" H52 2HS"}="2H5  2HS";
$resn_exch{" H41 2HS"}="1H4  2HS"; $resn_exch{" H42 2HS"}="2H4  2HS"; $resn_exch{" H31 2HS"}="1H3  2HS";
$resn_exch{" H32 2HS"}="2H3  2HS"; $resn_exch{" H21 2HS"}="1H2  2HS"; $resn_exch{" H22 2HS"}="2H2  2HS";
$resn_exch{" H11 2HS"}="1H1  2HS"; $resn_exch{"H12A 2HS"}="2H1  2HS"; $resn_exch{" H71 2HS"}="1H7  2HS";
$resn_exch{" H72 2HS"}="2H7  2HS"; $resn_exch{" H91 2HS"}="1H9  2HS"; $resn_exch{" H92 2HS"}="2H9  2HS";
$resn_exch{" H93 2HS"}="3H9  2HS"; $resn_exch{"H101 2HS"}="1H10 2HS"; $resn_exch{"H102 2HS"}="2H10 2HS";
$resn_exch{" H1  2IB"}="1H   2IB"; $resn_exch{" H2  2IB"}="2H   2IB"; $resn_exch{" H3A 2IB"}="3H   2IB";
$resn_exch{"H151 2IC"}="1H15 2IC"; $resn_exch{"H152 2IC"}="2H15 2IC"; $resn_exch{"H191 2IC"}="1H19 2IC";
$resn_exch{"H192 2IC"}="2H19 2IC"; $resn_exch{"H361 2IC"}="1H36 2IC"; $resn_exch{"H362 2IC"}="2H36 2IC";
$resn_exch{"H521 2IC"}="1H52 2IC"; $resn_exch{"H522 2IC"}="2H52 2IC"; $resn_exch{"H523 2IC"}="3H52 2IC";
$resn_exch{"HN31 2IG"}="1HN3 2IG"; $resn_exch{"HN32 2IG"}="2HN3 2IG"; $resn_exch{"HN41 2IG"}="1HN4 2IG";
$resn_exch{"HN42 2IG"}="2HN4 2IG"; $resn_exch{" H51 2IG"}="1H5  2IG"; $resn_exch{" H52 2IG"}="2H5  2IG";
$resn_exch{" H61 2IG"}="1H6  2IG"; $resn_exch{" H62 2IG"}="2H6  2IG"; $resn_exch{" H63 2IG"}="3H6  2IG";
$resn_exch{"H131 2IG"}="1H13 2IG"; $resn_exch{"H132 2IG"}="2H13 2IG"; $resn_exch{"H141 2IG"}="1H14 2IG";
$resn_exch{"H142 2IG"}="2H14 2IG"; $resn_exch{"H151 2IG"}="1H15 2IG"; $resn_exch{"H152 2IG"}="2H15 2IG";
$resn_exch{"H161 2IG"}="1H16 2IG"; $resn_exch{"H162 2IG"}="2H16 2IG"; $resn_exch{"H163 2IG"}="3H16 2IG";
$resn_exch{"H6C1 2IH"}="1H6C 2IH"; $resn_exch{"H6C2 2IH"}="2H6C 2IH"; $resn_exch{"H7C1 2IH"}="1H7C 2IH";
$resn_exch{"H7C2 2IH"}="2H7C 2IH"; $resn_exch{"H8C1 2IH"}="1H8C 2IH"; $resn_exch{"H8C2 2IH"}="2H8C 2IH";
$resn_exch{"H9C1 2IH"}="1H9C 2IH"; $resn_exch{"H9C2 2IH"}="2H9C 2IH"; $resn_exch{"H101 2IH"}="1H10 2IH";
$resn_exch{"H102 2IH"}="2H10 2IH"; $resn_exch{"H111 2IH"}="1H11 2IH"; $resn_exch{"H112 2IH"}="2H11 2IH";
$resn_exch{" H41 2IM"}="1H4  2IM"; $resn_exch{" H42 2IM"}="2H4  2IM"; $resn_exch{" H51 2IM"}="1H5  2IM";
$resn_exch{" H52 2IM"}="2H5  2IM"; $resn_exch{" H31 2IM"}="1H3  2IM"; $resn_exch{" H32 2IM"}="2H3  2IM";
$resn_exch{" H21 2IM"}="1H2  2IM"; $resn_exch{" H22 2IM"}="2H2  2IM"; $resn_exch{"HN41 2IN"}="1HN4 2IN";
$resn_exch{"HN42 2IN"}="2HN4 2IN"; $resn_exch{"H111 2IN"}="1H11 2IN"; $resn_exch{"H112 2IN"}="2H11 2IN";
$resn_exch{" H81 2IN"}="1H8  2IN"; $resn_exch{" H82 2IN"}="2H8  2IN"; $resn_exch{" H61 2IN"}="1H6  2IN";
$resn_exch{" H62 2IN"}="2H6  2IN"; $resn_exch{" H11 2IN"}="1H1  2IN"; $resn_exch{"H12A 2IN"}="2H1  2IN";
$resn_exch{"HO42 2IP"}="2HO4 2IP"; $resn_exch{"HO43 2IP"}="3HO4 2IP"; $resn_exch{"HO12 2IP"}="2HO1 2IP";
$resn_exch{"HO13 2IP"}="3HO1 2IP"; $resn_exch{" H41 2KT"}="1H4  2KT"; $resn_exch{" H42 2KT"}="2H4  2KT";
$resn_exch{" H43 2KT"}="3H4  2KT"; $resn_exch{" H31 2KT"}="1H3  2KT"; $resn_exch{" H32 2KT"}="2H3  2KT";
$resn_exch{" H41 2LG"}="1H4  2LG"; $resn_exch{" H42 2LG"}="2H4  2LG"; $resn_exch{" H61 2LG"}="1H6  2LG";
$resn_exch{" H62 2LG"}="2H6  2LG"; $resn_exch{"H261 2LG"}="1H26 2LG"; $resn_exch{"H262 2LG"}="2H26 2LG";
$resn_exch{"H281 2LG"}="1H28 2LG"; $resn_exch{"H282 2LG"}="2H28 2LG"; $resn_exch{"H283 2LG"}="3H28 2LG";
$resn_exch{" H91 2LG"}="1H9  2LG"; $resn_exch{" H92 2LG"}="2H9  2LG"; $resn_exch{"H251 2LG"}="1H25 2LG";
$resn_exch{"H252 2LG"}="2H25 2LG"; $resn_exch{"H271 2LG"}="1H27 2LG"; $resn_exch{"H272 2LG"}="2H27 2LG";
$resn_exch{"H273 2LG"}="3H27 2LG"; $resn_exch{"HC41 2LP"}="1HC4 2LP"; $resn_exch{"HC42 2LP"}="2HC4 2LP";
$resn_exch{"H141 2LP"}="1H14 2LP"; $resn_exch{"H142 2LP"}="2H14 2LP"; $resn_exch{" H2  2LU"}=" HN2 2LU";
$resn_exch{" HB2 2LU"}="1HB  2LU"; $resn_exch{" HB3 2LU"}="2HB  2LU"; $resn_exch{"HD11 2LU"}="1HD1 2LU";
$resn_exch{"HD12 2LU"}="2HD1 2LU"; $resn_exch{"HD13 2LU"}="3HD1 2LU"; $resn_exch{"HD21 2LU"}="1HD2 2LU";
$resn_exch{"HD22 2LU"}="2HD2 2LU"; $resn_exch{"HD23 2LU"}="3HD2 2LU"; $resn_exch{"HOP2 2MA"}="2HOP 2MA";
$resn_exch{"HOP3 2MA"}="3HOP 2MA"; $resn_exch{" H5' 2MA"}="1H5* 2MA"; $resn_exch{"H5'' 2MA"}="2H5* 2MA";
$resn_exch{" H4' 2MA"}=" H4* 2MA"; $resn_exch{" H3' 2MA"}=" H3* 2MA"; $resn_exch{"HO3' 2MA"}="*HO3 2MA";
$resn_exch{" H2' 2MA"}=" H2* 2MA"; $resn_exch{"HO2' 2MA"}="*HO2 2MA"; $resn_exch{" H1' 2MA"}=" H1* 2MA";
$resn_exch{"HM21 2MA"}="1HM2 2MA"; $resn_exch{"HM22 2MA"}="2HM2 2MA"; $resn_exch{"HM23 2MA"}="3HM2 2MA";
$resn_exch{"H4C1 2MB"}="1H4C 2MB"; $resn_exch{"H4C2 2MB"}="2H4C 2MB"; $resn_exch{"H4C3 2MB"}="3H4C 2MB";
$resn_exch{"H1C1 2MB"}="1H1C 2MB"; $resn_exch{"H1C2 2MB"}="2H1C 2MB"; $resn_exch{"H1C3 2MB"}="3H1C 2MB";
$resn_exch{"H2'1 2MB"}="1H2' 2MB"; $resn_exch{"H2'2 2MB"}="2H2' 2MB"; $resn_exch{"H2'3 2MB"}="3H2' 2MB";
$resn_exch{"HN61 2MC"}="1HN6 2MC"; $resn_exch{"HN62 2MC"}="2HN6 2MC"; $resn_exch{" H1' 2MC"}=" H1* 2MC";
$resn_exch{" H2' 2MC"}=" H2* 2MC"; $resn_exch{" H3' 2MC"}=" H3* 2MC"; $resn_exch{" H4' 2MC"}=" H4* 2MC";
$resn_exch{"H5'1 2MC"}="1H5* 2MC"; $resn_exch{"H5'2 2MC"}="2H5* 2MC"; $resn_exch{"H121 2MC"}="1H12 2MC";
$resn_exch{"H122 2MC"}="2H12 2MC"; $resn_exch{"H131 2MC"}="1H13 2MC"; $resn_exch{"H132 2MC"}="2H13 2MC";
$resn_exch{"H133 2MC"}="3H13 2MC"; $resn_exch{"H141 2MC"}="1H14 2MC"; $resn_exch{"H142 2MC"}="2H14 2MC";
$resn_exch{"H143 2MC"}="3H14 2MC"; $resn_exch{" H71 2MC"}="1H7  2MC"; $resn_exch{" H72 2MC"}="2H7  2MC";
$resn_exch{" H61 2MC"}="1H6  2MC"; $resn_exch{" H62 2MC"}="2H6  2MC"; $resn_exch{" H31 2MC"}="1H3  2MC";
$resn_exch{" H32 2MC"}="2H3  2MC"; $resn_exch{" H21 2MC"}="1H2  2MC"; $resn_exch{" H22 2MC"}="2H2  2MC";
$resn_exch{" H11 2MC"}="1H1  2MC"; $resn_exch{" H12 2MC"}="2H1  2MC"; $resn_exch{"H2M1 2MC"}="1H2M 2MC";
$resn_exch{"H2M2 2MC"}="2H2M 2MC"; $resn_exch{"H2M3 2MC"}="3H2M 2MC"; $resn_exch{"HO2B 2MD"}="BHO2 2MD";
$resn_exch{"H101 2MD"}="1H10 2MD"; $resn_exch{"H102 2MD"}="2H10 2MD"; $resn_exch{"HNG1 2MD"}="1HNG 2MD";
$resn_exch{"HNG2 2MD"}="2HNG 2MD"; $resn_exch{"HO2A 2MD"}="AHO2 2MD"; $resn_exch{"H5'1 2MD"}="1H5* 2MD";
$resn_exch{"H5'2 2MD"}="2H5* 2MD"; $resn_exch{" H4' 2MD"}=" H4* 2MD"; $resn_exch{" H1' 2MD"}=" H1* 2MD";
$resn_exch{"HN21 2MD"}="1HN2 2MD"; $resn_exch{"HN22 2MD"}="2HN2 2MD"; $resn_exch{" H2' 2MD"}=" H2* 2MD";
$resn_exch{"HO2' 2MD"}="*HO2 2MD"; $resn_exch{" H3' 2MD"}=" H3* 2MD"; $resn_exch{"HO3' 2MD"}="*HO3 2MD";
$resn_exch{"HA'1 2ME"}="1HA* 2ME"; $resn_exch{"HA'2 2ME"}="2HA* 2ME"; $resn_exch{"HA'3 2ME"}="3HA* 2ME";
$resn_exch{"HB'1 2ME"}="1HB* 2ME"; $resn_exch{"HB'2 2ME"}="2HB* 2ME"; $resn_exch{"HD'1 2ME"}="1HD* 2ME";
$resn_exch{"HD'2 2ME"}="2HD* 2ME"; $resn_exch{"HD'3 2ME"}="3HD* 2ME"; $resn_exch{"HOP2 2MG"}="2HOP 2MG";
$resn_exch{"HOP3 2MG"}="3HOP 2MG"; $resn_exch{" H5' 2MG"}="1H5* 2MG"; $resn_exch{"H5'' 2MG"}="2H5* 2MG";
$resn_exch{" H4' 2MG"}=" H4* 2MG"; $resn_exch{" H3' 2MG"}=" H3* 2MG"; $resn_exch{"HO3' 2MG"}="*HO3 2MG";
$resn_exch{" H2' 2MG"}=" H2* 2MG"; $resn_exch{"HO2' 2MG"}="*HO2 2MG"; $resn_exch{" H1' 2MG"}=" H1* 2MG";
$resn_exch{"HM21 2MG"}="1HM2 2MG"; $resn_exch{"HM22 2MG"}="2HM2 2MG"; $resn_exch{"HM23 2MG"}="3HM2 2MG";
$resn_exch{"HN11 2MH"}="1HN1 2MH"; $resn_exch{"HN12 2MH"}="2HN1 2MH"; $resn_exch{" H61 2MH"}="1H6  2MH";
$resn_exch{" H62 2MH"}="2H6  2MH"; $resn_exch{" H41 2MH"}="1H4  2MH"; $resn_exch{" H42 2MH"}="2H4  2MH";
$resn_exch{" H31 2MH"}="1H3  2MH"; $resn_exch{" H32 2MH"}="2H3  2MH"; $resn_exch{" H11 2MH"}="1H1  2MH";
$resn_exch{" H12 2MH"}="2H1  2MH"; $resn_exch{" H13 2MH"}="3H1  2MH"; $resn_exch{" H71 2MH"}="1H7  2MH";
$resn_exch{" H72 2MH"}="2H7  2MH"; $resn_exch{"HAJ1 2MI"}="1HAJ 2MI"; $resn_exch{"HAJ2 2MI"}="2HAJ 2MI";
$resn_exch{"HAJ3 2MI"}="3HAJ 2MI"; $resn_exch{" H2  2ML"}=" HN2 2ML"; $resn_exch{"HB11 2ML"}="1HB1 2ML";
$resn_exch{"HB12 2ML"}="2HB1 2ML"; $resn_exch{"HB21 2ML"}="1HB2 2ML"; $resn_exch{"HB22 2ML"}="2HB2 2ML";
$resn_exch{"HB23 2ML"}="3HB2 2ML"; $resn_exch{"HD11 2ML"}="1HD1 2ML"; $resn_exch{"HD12 2ML"}="2HD1 2ML";
$resn_exch{"HD13 2ML"}="3HD1 2ML"; $resn_exch{"HD21 2ML"}="1HD2 2ML"; $resn_exch{"HD22 2ML"}="2HD2 2ML";
$resn_exch{"HD23 2ML"}="3HD2 2ML"; $resn_exch{"HA51 2MN"}="1HA5 2MN"; $resn_exch{"HA52 2MN"}="2HA5 2MN";
$resn_exch{"HA53 2MN"}="3HA5 2MN"; $resn_exch{" H71 2MP"}="1H7  2MP"; $resn_exch{" H72 2MP"}="2H7  2MP";
$resn_exch{" H73 2MP"}="3H7  2MP"; $resn_exch{" H81 2MP"}="1H8  2MP"; $resn_exch{" H82 2MP"}="2H8  2MP";
$resn_exch{" H83 2MP"}="3H8  2MP"; $resn_exch{" H2  2MR"}=" HN2 2MR"; $resn_exch{" HB2 2MR"}="1HB  2MR";
$resn_exch{" HB3 2MR"}="2HB  2MR"; $resn_exch{" HG2 2MR"}="1HG  2MR"; $resn_exch{" HG3 2MR"}="2HG  2MR";
$resn_exch{" HD2 2MR"}="1HD  2MR"; $resn_exch{" HD3 2MR"}="2HD  2MR"; $resn_exch{"HQ11 2MR"}="1HQ1 2MR";
$resn_exch{"HQ12 2MR"}="2HQ1 2MR"; $resn_exch{"HQ13 2MR"}="3HQ1 2MR"; $resn_exch{" HH2 2MR"}="1HH2 2MR";
$resn_exch{"HQ21 2MR"}="1HQ2 2MR"; $resn_exch{"HQ22 2MR"}="2HQ2 2MR"; $resn_exch{"HQ23 2MR"}="3HQ2 2MR";
$resn_exch{" H   2MT"}=" HN  2MT"; $resn_exch{" HB2 2MT"}="2HB  2MT"; $resn_exch{" HB3 2MT"}="1HB  2MT";
$resn_exch{"HD21 2MT"}="1HD2 2MT"; $resn_exch{"HD22 2MT"}="2HD2 2MT"; $resn_exch{"HD23 2MT"}="3HD2 2MT";
$resn_exch{"HD31 2MT"}="1HD3 2MT"; $resn_exch{"HD32 2MT"}="2HD3 2MT"; $resn_exch{"HD33 2MT"}="3HD3 2MT";
$resn_exch{"HM51 2MU"}="1HM5 2MU"; $resn_exch{"HM52 2MU"}="2HM5 2MU"; $resn_exch{"HM53 2MU"}="3HM5 2MU";
$resn_exch{" H1' 2MU"}=" H1* 2MU"; $resn_exch{" H2' 2MU"}=" H2* 2MU"; $resn_exch{" H3' 2MU"}=" H3* 2MU";
$resn_exch{" H4' 2MU"}=" H4* 2MU"; $resn_exch{"HO3' 2MU"}="*HO3 2MU"; $resn_exch{" H5' 2MU"}="1H5* 2MU";
$resn_exch{"H5'' 2MU"}="2H5* 2MU"; $resn_exch{"H6'1 2MU"}="1H6* 2MU"; $resn_exch{"H6'2 2MU"}="2H6* 2MU";
$resn_exch{"H6'3 2MU"}="3H6* 2MU"; $resn_exch{"HOP2 2MU"}="2HOP 2MU"; $resn_exch{"HOP3 2MU"}="3HOP 2MU";
$resn_exch{" H61 2MZ"}="1H6  2MZ"; $resn_exch{" H62 2MZ"}="2H6  2MZ"; $resn_exch{" H63 2MZ"}="3H6  2MZ";
$resn_exch{" H1' 2ND"}=" H1* 2ND"; $resn_exch{"H2'1 2ND"}="1H2* 2ND"; $resn_exch{"H2'2 2ND"}="2H2* 2ND";
$resn_exch{" H3' 2ND"}=" H3* 2ND"; $resn_exch{" H4' 2ND"}=" H4* 2ND"; $resn_exch{"H5'1 2ND"}="1H5* 2ND";
$resn_exch{"H5'2 2ND"}="2H5* 2ND"; $resn_exch{" H5' 2ND"}=" H5* 2ND"; $resn_exch{"HD11 2NH"}="1HD1 2NH";
$resn_exch{"HD12 2NH"}="2HD1 2NH"; $resn_exch{"HD13 2NH"}="3HD1 2NH"; $resn_exch{"HG41 2NH"}="1HG4 2NH";
$resn_exch{"HG42 2NH"}="2HG4 2NH"; $resn_exch{"HG31 2NH"}="1HG3 2NH"; $resn_exch{"HG32 2NH"}="2HG3 2NH";
$resn_exch{"HG33 2NH"}="3HG3 2NH"; $resn_exch{"HB21 2NH"}="1HB2 2NH"; $resn_exch{"HB22 2NH"}="2HB2 2NH";
$resn_exch{" H1  2NH"}="1H   2NH"; $resn_exch{" H2A 2NH"}="2H   2NH"; $resn_exch{" H41 2NH"}="1H4  2NH";
$resn_exch{" H42 2NH"}="2H4  2NH"; $resn_exch{" H43 2NH"}="3H4  2NH"; $resn_exch{" H31 2NH"}="1H3  2NH";
$resn_exch{" H32 2NH"}="2H3  2NH"; $resn_exch{" H51 2NH"}="1H5  2NH"; $resn_exch{" H52 2NH"}="2H5  2NH";
$resn_exch{" H81 2NH"}="1H8  2NH"; $resn_exch{" H82 2NH"}="2H8  2NH"; $resn_exch{" H83 2NH"}="3H8  2NH";
$resn_exch{" H91 2NH"}="1H9  2NH"; $resn_exch{" H92 2NH"}="2H9  2NH"; $resn_exch{"H101 2NH"}="1H10 2NH";
$resn_exch{"H102 2NH"}="2H10 2NH"; $resn_exch{"H131 2NH"}="1H13 2NH"; $resn_exch{"H132 2NH"}="2H13 2NH";
$resn_exch{"H133 2NH"}="3H13 2NH"; $resn_exch{"H141 2NH"}="1H14 2NH"; $resn_exch{"H142 2NH"}="2H14 2NH";
$resn_exch{"H143 2NH"}="3H14 2NH"; $resn_exch{" HA1 2NH"}="1HA  2NH"; $resn_exch{"HA2A 2NH"}="2HA  2NH";
$resn_exch{"HA3A 2NH"}="3HA  2NH"; $resn_exch{"HG11 2NH"}="1HG1 2NH"; $resn_exch{"HG12 2NH"}="2HG1 2NH";
$resn_exch{"HG13 2NH"}="3HG1 2NH"; $resn_exch{"HG21 2NH"}="1HG2 2NH"; $resn_exch{"HG22 2NH"}="2HG2 2NH";
$resn_exch{"HG23 2NH"}="3HG2 2NH"; $resn_exch{"HB51 2NH"}="1HB5 2NH"; $resn_exch{"HB52 2NH"}="2HB5 2NH";
$resn_exch{" HG1 2NH"}="1HG  2NH"; $resn_exch{" HG2 2NH"}="2HG  2NH"; $resn_exch{" HE1 2NH"}="1HE  2NH";
$resn_exch{" HE2 2NH"}="2HE  2NH"; $resn_exch{" HE3 2NH"}="3HE  2NH"; $resn_exch{"H141 2NI"}="1H14 2NI";
$resn_exch{"H142 2NI"}="2H14 2NI"; $resn_exch{"H151 2NI"}="1H15 2NI"; $resn_exch{"H152 2NI"}="2H15 2NI";
$resn_exch{"H161 2NI"}="1H16 2NI"; $resn_exch{"H162 2NI"}="2H16 2NI"; $resn_exch{"H163 2NI"}="3H16 2NI";
$resn_exch{" H21 2NP"}="1H2  2NP"; $resn_exch{"HN21 2NP"}="1HN2 2NP"; $resn_exch{"HN22 2NP"}="2HN2 2NP";
$resn_exch{" H31 2NP"}="1H3  2NP"; $resn_exch{" H32 2NP"}="2H3  2NP"; $resn_exch{" H41 2NP"}="1H4  2NP";
$resn_exch{" H42 2NP"}="2H4  2NP"; $resn_exch{" H51 2NP"}="1H5  2NP"; $resn_exch{" H52 2NP"}="2H5  2NP";
$resn_exch{" H81 2NP"}="1H8  2NP"; $resn_exch{" H82 2NP"}="2H8  2NP"; $resn_exch{"HOP3 2NT"}="3HOP 2NT";
$resn_exch{"HOP2 2NT"}="2HOP 2NT"; $resn_exch{" H5' 2NT"}="1H5* 2NT"; $resn_exch{"H5'' 2NT"}="2H5* 2NT";
$resn_exch{" H4' 2NT"}=" H4* 2NT"; $resn_exch{" H1' 2NT"}=" H1* 2NT"; $resn_exch{" H71 2NT"}="1H5M 2NT";
$resn_exch{" H72 2NT"}="2H5M 2NT"; $resn_exch{" H73 2NT"}="3H5M 2NT"; $resn_exch{" H2' 2NT"}=" H2* 2NT";
$resn_exch{"HB'1 2NT"}="1HB' 2NT"; $resn_exch{"HB'2 2NT"}="2HB' 2NT"; $resn_exch{"HC'1 2NT"}="1HC' 2NT";
$resn_exch{"HC'2 2NT"}="2HC' 2NT"; $resn_exch{"HF'1 2NT"}="1HF' 2NT"; $resn_exch{"HF'2 2NT"}="2HF' 2NT";
$resn_exch{" H3' 2NT"}=" H3* 2NT"; $resn_exch{"HO3' 2NT"}="H3T  2NT"; $resn_exch{" H31 2O7"}="1H3  2O7";
$resn_exch{" H32 2O7"}="2H3  2O7"; $resn_exch{" H91 2O7"}="1H9  2O7"; $resn_exch{" H92 2O7"}="2H9  2O7";
$resn_exch{" H61 2O7"}="1H6  2O7"; $resn_exch{" H62 2O7"}="2H6  2O7"; $resn_exch{"H291 2O7"}="1H29 2O7";
$resn_exch{"H292 2O7"}="2H29 2O7"; $resn_exch{" H21 2OB"}="1H2  2OB"; $resn_exch{" H22 2OB"}="2H2  2OB";
$resn_exch{" H41 2OB"}="1H4  2OB"; $resn_exch{" H42 2OB"}="2H4  2OB"; $resn_exch{"H111 2OB"}="1H11 2OB";
$resn_exch{"H112 2OB"}="2H11 2OB"; $resn_exch{"H121 2OB"}="1H12 2OB"; $resn_exch{"H122 2OB"}="2H12 2OB";
$resn_exch{"H181 2OB"}="1H18 2OB"; $resn_exch{"H182 2OB"}="2H18 2OB"; $resn_exch{"H183 2OB"}="3H18 2OB";
$resn_exch{"H221 2OB"}="1H22 2OB"; $resn_exch{"H222 2OB"}="2H22 2OB"; $resn_exch{"H231 2OB"}="1H23 2OB";
$resn_exch{"H232 2OB"}="2H23 2OB"; $resn_exch{"H241 2OB"}="1H24 2OB"; $resn_exch{"H242 2OB"}="2H24 2OB";
$resn_exch{"H271 2OB"}="1H27 2OB"; $resn_exch{"H272 2OB"}="2H27 2OB"; $resn_exch{"H273 2OB"}="3H27 2OB";
$resn_exch{"H261 2OB"}="1H26 2OB"; $resn_exch{"H262 2OB"}="2H26 2OB"; $resn_exch{"H263 2OB"}="3H26 2OB";
$resn_exch{"H211 2OB"}="1H21 2OB"; $resn_exch{"H212 2OB"}="2H21 2OB"; $resn_exch{"H213 2OB"}="3H21 2OB";
$resn_exch{"H161 2OB"}="1H16 2OB"; $resn_exch{"H162 2OB"}="2H16 2OB"; $resn_exch{"H151 2OB"}="1H15 2OB";
$resn_exch{"H152 2OB"}="2H15 2OB"; $resn_exch{" H71 2OB"}="1H7  2OB"; $resn_exch{" H72 2OB"}="2H7  2OB";
$resn_exch{"H191 2OB"}="1H19 2OB"; $resn_exch{"H192 2OB"}="2H19 2OB"; $resn_exch{"H193 2OB"}="3H19 2OB";
$resn_exch{" H11 2OB"}="1H1  2OB"; $resn_exch{" H12 2OB"}="2H1  2OB"; $resn_exch{"H471 2OB"}="1H47 2OB";
$resn_exch{"H472 2OB"}="2H47 2OB"; $resn_exch{"H461 2OB"}="1H46 2OB"; $resn_exch{"H462 2OB"}="2H46 2OB";
$resn_exch{"H451 2OB"}="1H45 2OB"; $resn_exch{"H452 2OB"}="2H45 2OB"; $resn_exch{"H441 2OB"}="1H44 2OB";
$resn_exch{"H442 2OB"}="2H44 2OB"; $resn_exch{"H431 2OB"}="1H43 2OB"; $resn_exch{"H432 2OB"}="2H43 2OB";
$resn_exch{"H421 2OB"}="1H42 2OB"; $resn_exch{"H422 2OB"}="2H42 2OB"; $resn_exch{"H411 2OB"}="1H41 2OB";
$resn_exch{"H412 2OB"}="2H41 2OB"; $resn_exch{"H381 2OB"}="1H38 2OB"; $resn_exch{"H382 2OB"}="2H38 2OB";
$resn_exch{"H371 2OB"}="1H37 2OB"; $resn_exch{"H372 2OB"}="2H37 2OB"; $resn_exch{"H361 2OB"}="1H36 2OB";
$resn_exch{"H362 2OB"}="2H36 2OB"; $resn_exch{"H351 2OB"}="1H35 2OB"; $resn_exch{"H352 2OB"}="2H35 2OB";
$resn_exch{"H341 2OB"}="1H34 2OB"; $resn_exch{"H342 2OB"}="2H34 2OB"; $resn_exch{"H331 2OB"}="1H33 2OB";
$resn_exch{"H332 2OB"}="2H33 2OB"; $resn_exch{"H321 2OB"}="1H32 2OB"; $resn_exch{"H322 2OB"}="2H32 2OB";
$resn_exch{"H311 2OB"}="1H31 2OB"; $resn_exch{"H312 2OB"}="2H31 2OB"; $resn_exch{"H313 2OB"}="3H31 2OB";
$resn_exch{"HO11 2OF"}="1HO1 2OF"; $resn_exch{"HO12 2OF"}="2HO1 2OF"; $resn_exch{"HO21 2OF"}="1HO2 2OF";
$resn_exch{"HO22 2OF"}="2HO2 2OF"; $resn_exch{" H31 2OG"}="1H3  2OG"; $resn_exch{" H32 2OG"}="2H3  2OG";
$resn_exch{" H41 2OG"}="1H4  2OG"; $resn_exch{" H42 2OG"}="2H4  2OG"; $resn_exch{" H31 2OH"}="1H3  2OH";
$resn_exch{" H32 2OH"}="2H3  2OH"; $resn_exch{" H33 2OH"}="3H3  2OH"; $resn_exch{" H51 2OH"}="1H5  2OH";
$resn_exch{" H52 2OH"}="2H5  2OH"; $resn_exch{" H53 2OH"}="3H5  2OH"; $resn_exch{" HB1 2OP"}="1HB  2OP";
$resn_exch{" HB2 2OP"}="2HB  2OP"; $resn_exch{" HB3 2OP"}="3HB  2OP"; $resn_exch{" H71 2OS"}="1H7  2OS";
$resn_exch{" H72 2OS"}="2H7  2OS"; $resn_exch{" H81 2OS"}="1H8  2OS"; $resn_exch{" H82 2OS"}="2H8  2OS";
$resn_exch{" H91 2OS"}="1H9  2OS"; $resn_exch{" H92 2OS"}="2H9  2OS"; $resn_exch{"H101 2OS"}="1H10 2OS";
$resn_exch{"H102 2OS"}="2H10 2OS"; $resn_exch{"H111 2OS"}="1H11 2OS"; $resn_exch{"H112 2OS"}="2H11 2OS";
$resn_exch{"H121 2OS"}="1H12 2OS"; $resn_exch{"H122 2OS"}="2H12 2OS"; $resn_exch{"H171 2OS"}="1H17 2OS";
$resn_exch{"H172 2OS"}="2H17 2OS"; $resn_exch{"H181 2OS"}="1H18 2OS"; $resn_exch{"H182 2OS"}="2H18 2OS";
$resn_exch{"H321 2OS"}="1H32 2OS"; $resn_exch{"H322 2OS"}="2H32 2OS"; $resn_exch{"H311 2OS"}="1H31 2OS";
$resn_exch{"H312 2OS"}="2H31 2OS"; $resn_exch{"H313 2OS"}="3H31 2OS"; $resn_exch{"HOP2 2OT"}="2HOP 2OT";
$resn_exch{" H5' 2OT"}="1H5* 2OT"; $resn_exch{"H5'' 2OT"}="2H5* 2OT"; $resn_exch{" H4' 2OT"}=" H4* 2OT";
$resn_exch{" H1' 2OT"}=" H1* 2OT"; $resn_exch{" H71 2OT"}="1H5M 2OT"; $resn_exch{" H72 2OT"}="2H5M 2OT";
$resn_exch{" H73 2OT"}="3H5M 2OT"; $resn_exch{" H2' 2OT"}=" H2* 2OT"; $resn_exch{"HB'1 2OT"}="1HB' 2OT";
$resn_exch{"HB'2 2OT"}="2HB' 2OT"; $resn_exch{"HC'1 2OT"}="1HC' 2OT"; $resn_exch{"HC'2 2OT"}="2HC' 2OT";
$resn_exch{"HF'1 2OT"}="1HF' 2OT"; $resn_exch{"HF'2 2OT"}="2HF' 2OT"; $resn_exch{"HF'3 2OT"}="3HF' 2OT";
$resn_exch{"HG'1 2OT"}="1HG' 2OT"; $resn_exch{"HG'2 2OT"}="2HG' 2OT"; $resn_exch{"HG'3 2OT"}="3HG' 2OT";
$resn_exch{" H3' 2OT"}=" H3* 2OT"; $resn_exch{"HO3' 2OT"}="H3T  2OT"; $resn_exch{"HOP3 2OT"}="3HOP 2OT";
$resn_exch{"HN41 2PA"}="1HN4 2PA"; $resn_exch{"HN42 2PA"}="2HN4 2PA"; $resn_exch{"HN51 2PA"}="1HN5 2PA";
$resn_exch{"HN52 2PA"}="2HN5 2PA"; $resn_exch{"HO31 2PA"}="1HO3 2PA"; $resn_exch{" H31 2PC"}="1H3  2PC";
$resn_exch{" H32 2PC"}="2H3  2PC"; $resn_exch{" H41 2PC"}="1H4  2PC"; $resn_exch{" H42 2PC"}="2H4  2PC";
$resn_exch{" H51 2PC"}="1H5  2PC"; $resn_exch{" H52 2PC"}="2H5  2PC"; $resn_exch{" HO1 2PE"}="1HO  2PE";
$resn_exch{" H21 2PE"}="1H2  2PE"; $resn_exch{" H22 2PE"}="2H2  2PE"; $resn_exch{" H31 2PE"}="1H3  2PE";
$resn_exch{" H32 2PE"}="2H3  2PE"; $resn_exch{" H51 2PE"}="1H5  2PE"; $resn_exch{" H52 2PE"}="2H5  2PE";
$resn_exch{" H61 2PE"}="1H6  2PE"; $resn_exch{" H62 2PE"}="2H6  2PE"; $resn_exch{" H81 2PE"}="1H8  2PE";
$resn_exch{" H82 2PE"}="2H8  2PE"; $resn_exch{" H91 2PE"}="1H9  2PE"; $resn_exch{" H92 2PE"}="2H9  2PE";
$resn_exch{"H111 2PE"}="1H11 2PE"; $resn_exch{"H112 2PE"}="2H11 2PE"; $resn_exch{"H121 2PE"}="1H12 2PE";
$resn_exch{"H122 2PE"}="2H12 2PE"; $resn_exch{"H141 2PE"}="1H14 2PE"; $resn_exch{"H142 2PE"}="2H14 2PE";
$resn_exch{"H151 2PE"}="1H15 2PE"; $resn_exch{"H152 2PE"}="2H15 2PE"; $resn_exch{"H171 2PE"}="1H17 2PE";
$resn_exch{"H172 2PE"}="2H17 2PE"; $resn_exch{"H181 2PE"}="1H18 2PE"; $resn_exch{"H182 2PE"}="2H18 2PE";
$resn_exch{"H201 2PE"}="1H20 2PE"; $resn_exch{"H202 2PE"}="2H20 2PE"; $resn_exch{"H211 2PE"}="1H21 2PE";
$resn_exch{"H212 2PE"}="2H21 2PE"; $resn_exch{"H231 2PE"}="1H23 2PE"; $resn_exch{"H232 2PE"}="2H23 2PE";
$resn_exch{"H241 2PE"}="1H24 2PE"; $resn_exch{"H242 2PE"}="2H24 2PE"; $resn_exch{"H261 2PE"}="1H26 2PE";
$resn_exch{"H262 2PE"}="2H26 2PE"; $resn_exch{"H271 2PE"}="1H27 2PE"; $resn_exch{"H272 2PE"}="2H27 2PE";
$resn_exch{" HO2 2PE"}="2HO  2PE"; $resn_exch{" H31 2PG"}="1H3  2PG"; $resn_exch{" H32 2PG"}="2H3  2PG";
$resn_exch{"HOP3 2PG"}="3HOP 2PG"; $resn_exch{"HOP4 2PG"}="4HOP 2PG"; $resn_exch{" H31 2PH"}="1H3  2PH";
$resn_exch{" H32 2PH"}="2H3  2PH"; $resn_exch{"HN61 2PH"}="1HN6 2PH"; $resn_exch{"HN62 2PH"}="2HN6 2PH";
$resn_exch{"H111 2PH"}="1H11 2PH"; $resn_exch{"H112 2PH"}="2H11 2PH"; $resn_exch{"HOP2 2PH"}="2HOP 2PH";
$resn_exch{"HOP5 2PH"}="5HOP 2PH"; $resn_exch{"HOP6 2PH"}="6HOP 2PH"; $resn_exch{" HB1 2PI"}="1HB  2PI";
$resn_exch{" HB2 2PI"}="2HB  2PI"; $resn_exch{" HG1 2PI"}="1HG  2PI"; $resn_exch{" HG2 2PI"}="2HG  2PI";
$resn_exch{" HD1 2PI"}="1HD  2PI"; $resn_exch{" HD2 2PI"}="2HD  2PI"; $resn_exch{" HD3 2PI"}="3HD  2PI";
$resn_exch{" H21 2PL"}="1H2  2PL"; $resn_exch{" H22 2PL"}="2H2  2PL"; $resn_exch{"HOP3 2PL"}="3HOP 2PL";
$resn_exch{"HOP4 2PL"}="4HOP 2PL"; $resn_exch{"H141 2PM"}="1H14 2PM"; $resn_exch{"H142 2PM"}="2H14 2PM";
$resn_exch{"H151 2PM"}="1H15 2PM"; $resn_exch{"H152 2PM"}="2H15 2PM"; $resn_exch{"H161 2PM"}="1H16 2PM";
$resn_exch{"H162 2PM"}="2H16 2PM"; $resn_exch{"H163 2PM"}="3H16 2PM"; $resn_exch{"H171 2PM"}="1H17 2PM";
$resn_exch{"H172 2PM"}="2H17 2PM"; $resn_exch{"H173 2PM"}="3H17 2PM"; $resn_exch{" H31 2PP"}="1H3  2PP";
$resn_exch{" H32 2PP"}="2H3  2PP"; $resn_exch{" H41 2PP"}="1H4  2PP"; $resn_exch{" H42 2PP"}="2H4  2PP";
$resn_exch{" H51 2PP"}="1H5  2PP"; $resn_exch{" H52 2PP"}="2H5  2PP"; $resn_exch{" H53 2PP"}="3H5  2PP";
$resn_exch{"H1'1 2PP"}="1H1' 2PP"; $resn_exch{"H1'2 2PP"}="2H1' 2PP"; $resn_exch{"H2'1 2PP"}="1H2' 2PP";
$resn_exch{"H2'2 2PP"}="2H2' 2PP"; $resn_exch{"H3'1 2PP"}="1H3' 2PP"; $resn_exch{"H3'2 2PP"}="2H3' 2PP";
$resn_exch{"H3'3 2PP"}="3H3' 2PP"; $resn_exch{"HOP3 2PR"}="3HOP 2PR"; $resn_exch{"HOP2 2PR"}="2HOP 2PR";
$resn_exch{" H5' 2PR"}="1H5* 2PR"; $resn_exch{"H5'' 2PR"}="2H5* 2PR"; $resn_exch{" H4' 2PR"}=" H4* 2PR";
$resn_exch{" H3' 2PR"}=" H3* 2PR"; $resn_exch{"HO3' 2PR"}="H3T  2PR"; $resn_exch{" H2' 2PR"}="1H2* 2PR";
$resn_exch{"H2'' 2PR"}="2H2* 2PR"; $resn_exch{" H1' 2PR"}=" H1* 2PR"; $resn_exch{"HN21 2PR"}="1HN2 2PR";
$resn_exch{"HN22 2PR"}="2HN2 2PR"; $resn_exch{" H7N 2PT"}=" NH7 2PT"; $resn_exch{"H231 2PT"}="1H23 2PT";
$resn_exch{"H232 2PT"}="2H23 2PT"; $resn_exch{"H233 2PT"}="3H23 2PT"; $resn_exch{"H241 2PT"}="1H24 2PT";
$resn_exch{"H242 2PT"}="2H24 2PT"; $resn_exch{"H243 2PT"}="3H24 2PT"; $resn_exch{"HN51 2PT"}="1HN5 2PT";
$resn_exch{"HN52 2PT"}="2HN5 2PT"; $resn_exch{" H61 2PU"}="1H6  2PU"; $resn_exch{" H62 2PU"}="2H6  2PU";
$resn_exch{"H111 2PU"}="1H11 2PU"; $resn_exch{"H112 2PU"}="2H11 2PU"; $resn_exch{"H121 2PU"}="1H12 2PU";
$resn_exch{"H122 2PU"}="2H12 2PU"; $resn_exch{"H241 2PU"}="1H24 2PU"; $resn_exch{"H242 2PU"}="2H24 2PU";
$resn_exch{"H261 2PU"}="1H26 2PU"; $resn_exch{"H262 2PU"}="2H26 2PU"; $resn_exch{"H271 2PU"}="1H27 2PU";
$resn_exch{"H272 2PU"}="2H27 2PU"; $resn_exch{"H241 2PY"}="1H24 2PY"; $resn_exch{"H242 2PY"}="2H24 2PY";
$resn_exch{"H251 2PY"}="1H25 2PY"; $resn_exch{"H252 2PY"}="2H25 2PY"; $resn_exch{"HN41 2PY"}="1HN4 2PY";
$resn_exch{"HN42 2PY"}="2HN4 2PY"; $resn_exch{"HAA1 2RD"}="1HAA 2RD"; $resn_exch{"HAA2 2RD"}="2HAA 2RD";
$resn_exch{"HAA3 2RD"}="3HAA 2RD"; $resn_exch{"HAD1 2RD"}="1HAD 2RD"; $resn_exch{"HAD2 2RD"}="2HAD 2RD";
$resn_exch{"HAE1 2RD"}="1HAE 2RD"; $resn_exch{"HAE2 2RD"}="2HAE 2RD"; $resn_exch{"HAF1 2RD"}="1HAF 2RD";
$resn_exch{"HAF2 2RD"}="2HAF 2RD"; $resn_exch{"HAG1 2RD"}="1HAG 2RD"; $resn_exch{"HAG2 2RD"}="2HAG 2RD";
$resn_exch{"HAH1 2RD"}="1HAH 2RD"; $resn_exch{"HAH2 2RD"}="2HAH 2RD"; $resn_exch{"HAI1 2RD"}="1HAI 2RD";
$resn_exch{"HAI2 2RD"}="2HAI 2RD"; $resn_exch{"HAJ1 2RD"}="1HAJ 2RD"; $resn_exch{"HAJ2 2RD"}="2HAJ 2RD";
$resn_exch{"HAK1 2RD"}="1HAK 2RD"; $resn_exch{"HAK2 2RD"}="2HAK 2RD"; $resn_exch{"HAL1 2RD"}="1HAL 2RD";
$resn_exch{"HAL2 2RD"}="2HAL 2RD"; $resn_exch{"HAM1 2RD"}="1HAM 2RD"; $resn_exch{"HAM2 2RD"}="2HAM 2RD";
$resn_exch{"HAN1 2RD"}="1HAN 2RD"; $resn_exch{"HAN2 2RD"}="2HAN 2RD"; $resn_exch{"HC51 2SA"}="1HC5 2SA";
$resn_exch{"HC52 2SA"}="2HC5 2SA"; $resn_exch{"H631 2SA"}="1H63 2SA"; $resn_exch{"H632 2SA"}="2H63 2SA";
$resn_exch{"H121 2SC"}="1H12 2SC"; $resn_exch{"H122 2SC"}="2H12 2SC"; $resn_exch{"H241 2SC"}="1H24 2SC";
$resn_exch{"H242 2SC"}="2H24 2SC"; $resn_exch{"H1O1 2SI"}="1H1O 2SI"; $resn_exch{"H3O1 2SI"}="1H3O 2SI";
$resn_exch{"HO61 2SI"}="1HO6 2SI"; $resn_exch{"HOS3 2SI"}="3HOS 2SI"; $resn_exch{"H4O1 2SI"}="1H4O 2SI";
$resn_exch{" H71 2SK"}="1H7  2SK"; $resn_exch{" H72 2SK"}="2H7  2SK"; $resn_exch{" H73 2SK"}="3H7  2SK";
$resn_exch{" H81 2SK"}="1H8  2SK"; $resn_exch{" H82 2SK"}="2H8  2SK"; $resn_exch{" H11 2SP"}="1H1  2SP";
$resn_exch{" H12 2SP"}="2H1  2SP"; $resn_exch{"H321 2SP"}="1H32 2SP"; $resn_exch{"H322 2SP"}="2H32 2SP";
$resn_exch{"H331 2SP"}="1H33 2SP"; $resn_exch{"H332 2SP"}="2H33 2SP"; $resn_exch{"H341 2SP"}="1H34 2SP";
$resn_exch{"H342 2SP"}="2H34 2SP"; $resn_exch{"H351 2SP"}="1H35 2SP"; $resn_exch{"H352 2SP"}="2H35 2SP";
$resn_exch{"H361 2SP"}="1H36 2SP"; $resn_exch{"H362 2SP"}="2H36 2SP"; $resn_exch{"H371 2SP"}="1H37 2SP";
$resn_exch{"H372 2SP"}="2H37 2SP"; $resn_exch{"H381 2SP"}="1H38 2SP"; $resn_exch{"H382 2SP"}="2H38 2SP";
$resn_exch{"H391 2SP"}="1H39 2SP"; $resn_exch{"H392 2SP"}="2H39 2SP"; $resn_exch{"H401 2SP"}="1H40 2SP";
$resn_exch{"H402 2SP"}="2H40 2SP"; $resn_exch{"H411 2SP"}="1H41 2SP"; $resn_exch{"H412 2SP"}="2H41 2SP";
$resn_exch{"H421 2SP"}="1H42 2SP"; $resn_exch{"H422 2SP"}="2H42 2SP"; $resn_exch{"H431 2SP"}="1H43 2SP";
$resn_exch{"H432 2SP"}="2H43 2SP"; $resn_exch{"H441 2SP"}="1H44 2SP"; $resn_exch{"H442 2SP"}="2H44 2SP";
$resn_exch{"H451 2SP"}="1H45 2SP"; $resn_exch{"H452 2SP"}="2H45 2SP"; $resn_exch{"H461 2SP"}="1H46 2SP";
$resn_exch{"H462 2SP"}="2H46 2SP"; $resn_exch{"H471 2SP"}="1H47 2SP"; $resn_exch{"H472 2SP"}="2H47 2SP";
$resn_exch{"H481 2SP"}="1H48 2SP"; $resn_exch{"H482 2SP"}="2H48 2SP"; $resn_exch{"H483 2SP"}="3H48 2SP";
$resn_exch{" H31 2SP"}="1H3  2SP"; $resn_exch{" H32 2SP"}="2H3  2SP"; $resn_exch{"H121 2SP"}="1H12 2SP";
$resn_exch{"H122 2SP"}="2H12 2SP"; $resn_exch{"H131 2SP"}="1H13 2SP"; $resn_exch{"H132 2SP"}="2H13 2SP";
$resn_exch{"H141 2SP"}="1H14 2SP"; $resn_exch{"H142 2SP"}="2H14 2SP"; $resn_exch{"H151 2SP"}="1H15 2SP";
$resn_exch{"H152 2SP"}="2H15 2SP"; $resn_exch{"H161 2SP"}="1H16 2SP"; $resn_exch{"H162 2SP"}="2H16 2SP";
$resn_exch{"H171 2SP"}="1H17 2SP"; $resn_exch{"H172 2SP"}="2H17 2SP"; $resn_exch{"H181 2SP"}="1H18 2SP";
$resn_exch{"H182 2SP"}="2H18 2SP"; $resn_exch{"H191 2SP"}="1H19 2SP"; $resn_exch{"H192 2SP"}="2H19 2SP";
$resn_exch{"H201 2SP"}="1H20 2SP"; $resn_exch{"H202 2SP"}="2H20 2SP"; $resn_exch{"H211 2SP"}="1H21 2SP";
$resn_exch{"H212 2SP"}="2H21 2SP"; $resn_exch{"H221 2SP"}="1H22 2SP"; $resn_exch{"H222 2SP"}="2H22 2SP";
$resn_exch{"H231 2SP"}="1H23 2SP"; $resn_exch{"H232 2SP"}="2H23 2SP"; $resn_exch{"H241 2SP"}="1H24 2SP";
$resn_exch{"H242 2SP"}="2H24 2SP"; $resn_exch{"H251 2SP"}="1H25 2SP"; $resn_exch{"H252 2SP"}="2H25 2SP";
$resn_exch{"H261 2SP"}="1H26 2SP"; $resn_exch{"H262 2SP"}="2H26 2SP"; $resn_exch{"H271 2SP"}="1H27 2SP";
$resn_exch{"H272 2SP"}="2H27 2SP"; $resn_exch{"H281 2SP"}="1H28 2SP"; $resn_exch{"H282 2SP"}="2H28 2SP";
$resn_exch{"H283 2SP"}="3H28 2SP"; $resn_exch{"HOP3 2ST"}="3HOP 2ST"; $resn_exch{"HOP2 2ST"}="2HOP 2ST";
$resn_exch{"H5A1 2ST"}="1H5A 2ST"; $resn_exch{"H5A2 2ST"}="2H5A 2ST"; $resn_exch{"H5A3 2ST"}="3H5A 2ST";
$resn_exch{" H2' 2ST"}=" H2* 2ST"; $resn_exch{" H5' 2ST"}="1H5* 2ST"; $resn_exch{"H5'' 2ST"}="2H5* 2ST";
$resn_exch{" H4' 2ST"}=" H4* 2ST"; $resn_exch{" H1' 2ST"}=" H1* 2ST"; $resn_exch{" H3' 2ST"}=" H3* 2ST";
$resn_exch{"HA'1 2ST"}="1HA* 2ST"; $resn_exch{"HA'2 2ST"}="2HA* 2ST"; $resn_exch{"HA'3 2ST"}="3HA* 2ST";
$resn_exch{"HO3' 2ST"}="H3T  2ST"; $resn_exch{" H21 2TB"}="1H2  2TB"; $resn_exch{" H22 2TB"}="2H2  2TB";
$resn_exch{"HN11 2TB"}="1HN1 2TB"; $resn_exch{"HN12 2TB"}="2HN1 2TB"; $resn_exch{"HN13 2TB"}="3HN1 2TB";
$resn_exch{"HN31 2TB"}="1HN3 2TB"; $resn_exch{"HN32 2TB"}="2HN3 2TB"; $resn_exch{"HN33 2TB"}="3HN3 2TB";
$resn_exch{" H81 2TH"}="1H8  2TH"; $resn_exch{" H82 2TH"}="2H8  2TH"; $resn_exch{"HN21 2TN"}="1HN2 2TN";
$resn_exch{"HN22 2TN"}="2HN2 2TN"; $resn_exch{"H101 2TN"}="1H10 2TN"; $resn_exch{"H102 2TN"}="2H10 2TN";
$resn_exch{" H11 2TN"}="1H1  2TN"; $resn_exch{"H12A 2TN"}="2H1  2TN"; $resn_exch{" H31 2TN"}="1H3  2TN";
$resn_exch{" H32 2TN"}="2H3  2TN"; $resn_exch{"H141 2TN"}="1H14 2TN"; $resn_exch{"H142 2TN"}="2H14 2TN";
$resn_exch{"H143 2TN"}="3H14 2TN"; $resn_exch{"H131 2TN"}="1H13 2TN"; $resn_exch{"H132 2TN"}="2H13 2TN";
$resn_exch{"H133 2TN"}="3H13 2TN"; $resn_exch{"H2A1 2TP"}="1H2A 2TP"; $resn_exch{"H2A2 2TP"}="2H2A 2TP";
$resn_exch{"H2A3 2TP"}="3H2A 2TP"; $resn_exch{" H4' 2TP"}=" H4* 2TP"; $resn_exch{" H6' 2TP"}=" H6* 2TP";
$resn_exch{"H351 2TP"}="1H35 2TP"; $resn_exch{"H352 2TP"}="2H35 2TP"; $resn_exch{"H4A1 2TP"}="1H4A 2TP";
$resn_exch{"H4A2 2TP"}="2H4A 2TP"; $resn_exch{"H4A3 2TP"}="3H4A 2TP"; $resn_exch{"H5A1 2TP"}="1H5A 2TP";
$resn_exch{"H5A2 2TP"}="2H5A 2TP"; $resn_exch{"H5B1 2TP"}="1H5B 2TP"; $resn_exch{"H5B2 2TP"}="2H5B 2TP";
$resn_exch{" H   2TY"}="1HN  2TY"; $resn_exch{" H2  2TY"}="2HN  2TY"; $resn_exch{" HB2 2TY"}="1HB  2TY";
$resn_exch{" HB3 2TY"}="2HB  2TY"; $resn_exch{" HH  2TY"}=" HOH 2TY"; $resn_exch{" H21 2TY"}="1H2  2TY";
$resn_exch{" H22 2TY"}="2H2  2TY"; $resn_exch{"H191 2UN"}="1H19 2UN"; $resn_exch{"H192 2UN"}="2H19 2UN";
$resn_exch{"H171 2UN"}="1H17 2UN"; $resn_exch{"H172 2UN"}="2H17 2UN"; $resn_exch{"H173 2UN"}="3H17 2UN";
$resn_exch{"H181 2UN"}="1H18 2UN"; $resn_exch{"H182 2UN"}="2H18 2UN"; $resn_exch{"H183 2UN"}="3H18 2UN";
$resn_exch{"H101 2UN"}="1H10 2UN"; $resn_exch{"H102 2UN"}="2H10 2UN"; $resn_exch{"HG21 2VA"}="1HG2 2VA";
$resn_exch{"HG22 2VA"}="2HG2 2VA"; $resn_exch{"HG23 2VA"}="3HG2 2VA"; $resn_exch{"HG11 2VA"}="1HG1 2VA";
$resn_exch{"HG12 2VA"}="2HG1 2VA"; $resn_exch{"HG13 2VA"}="3HG1 2VA"; $resn_exch{" H   2VA"}="1HN  2VA";
$resn_exch{" H2  2VA"}="2HN  2VA"; $resn_exch{" H2' 2VA"}=" H2* 2VA"; $resn_exch{" H3' 2VA"}=" H3* 2VA";
$resn_exch{" H4' 2VA"}=" H4* 2VA"; $resn_exch{"H5'1 2VA"}="1H5* 2VA"; $resn_exch{"H5'2 2VA"}="2H5* 2VA";
$resn_exch{" H5' 2VA"}=" H5* 2VA"; $resn_exch{" H1' 2VA"}=" H1* 2VA"; $resn_exch{" HC2 2VA"}=" H2  2VA";
$resn_exch{"HN61 2VA"}="1HN6 2VA"; $resn_exch{"HN62 2VA"}="2HN6 2VA"; $resn_exch{"H131 300"}="1H13 300";
$resn_exch{"H132 300"}="2H13 300"; $resn_exch{"H133 300"}="3H13 300"; $resn_exch{"H131 302"}="1H13 302";
$resn_exch{"H132 302"}="2H13 302"; $resn_exch{"H121 302"}="1H12 302"; $resn_exch{"H122 302"}="2H12 302";
$resn_exch{"H191 303"}="1H19 303"; $resn_exch{"H192 303"}="2H19 303"; $resn_exch{"H321 303"}="1H32 303";
$resn_exch{"H322 303"}="2H32 303"; $resn_exch{"H331 303"}="1H33 303"; $resn_exch{"H332 303"}="2H33 303";
$resn_exch{"H421 303"}="1H42 303"; $resn_exch{"H422 303"}="2H42 303"; $resn_exch{"H423 303"}="3H42 303";
$resn_exch{"H461 303"}="1H46 303"; $resn_exch{"H462 303"}="2H46 303"; $resn_exch{"H463 303"}="3H46 303";
$resn_exch{"H131 306"}="1H13 306"; $resn_exch{"H132 306"}="2H13 306"; $resn_exch{"H251 306"}="1H25 306";
$resn_exch{"H252 306"}="2H25 306"; $resn_exch{"H261 306"}="1H26 306"; $resn_exch{"H262 306"}="2H26 306";
$resn_exch{"H271 306"}="1H27 306"; $resn_exch{"H272 306"}="2H27 306"; $resn_exch{"H281 306"}="1H28 306";
$resn_exch{"H282 306"}="2H28 306"; $resn_exch{"H291 306"}="1H29 306"; $resn_exch{"H292 306"}="2H29 306";
$resn_exch{"H141 307"}="1H14 307"; $resn_exch{"H142 307"}="2H14 307"; $resn_exch{"H221 307"}="1H22 307";
$resn_exch{"H222 307"}="2H22 307"; $resn_exch{"H223 307"}="3H22 307"; $resn_exch{"H231 307"}="1H23 307";
$resn_exch{"H232 307"}="2H23 307"; $resn_exch{"H241 307"}="1H24 307"; $resn_exch{"H242 307"}="2H24 307";
$resn_exch{"H281 307"}="1H28 307"; $resn_exch{"H282 307"}="2H28 307"; $resn_exch{"H283 307"}="3H28 307";
$resn_exch{"H261 307"}="1H26 307"; $resn_exch{"H262 307"}="2H26 307"; $resn_exch{"H391 307"}="1H39 307";
$resn_exch{"H392 307"}="2H39 307"; $resn_exch{"H393 307"}="3H39 307"; $resn_exch{"H371 307"}="1H37 307";
$resn_exch{"H372 307"}="2H37 307"; $resn_exch{"H373 307"}="3H37 307"; $resn_exch{" H3B 312"}="H3'' 312";
$resn_exch{" H4B 312"}="H4'' 312"; $resn_exch{" H5B 312"}="H5'' 312"; $resn_exch{" H6B 312"}="H6'' 312";
$resn_exch{"H171 326"}="1H17 326"; $resn_exch{"H172 326"}="2H17 326"; $resn_exch{"H173 326"}="3H17 326";
$resn_exch{"H211 326"}="1H21 326"; $resn_exch{"H212 326"}="2H21 326"; $resn_exch{"H213 326"}="3H21 326";
$resn_exch{" H71 32P"}="1H7  32P"; $resn_exch{" H72 32P"}="2H7  32P"; $resn_exch{"H151 32P"}="1H15 32P";
$resn_exch{"H152 32P"}="2H15 32P"; $resn_exch{"H181 32P"}="1H18 32P"; $resn_exch{"H182 32P"}="2H18 32P";
$resn_exch{"H183 32P"}="3H18 32P"; $resn_exch{"H191 32P"}="1H19 32P"; $resn_exch{"H192 32P"}="2H19 32P";
$resn_exch{"H201 32P"}="1H20 32P"; $resn_exch{"H202 32P"}="2H20 32P"; $resn_exch{"H211 32P"}="1H21 32P";
$resn_exch{"H212 32P"}="2H21 32P"; $resn_exch{"H221 32P"}="1H22 32P"; $resn_exch{"H222 32P"}="2H22 32P";
$resn_exch{"HN31 32P"}="1HN3 32P"; $resn_exch{"HN32 32P"}="2HN3 32P"; $resn_exch{"H231 32P"}="1H23 32P";
$resn_exch{"H232 32P"}="2H23 32P"; $resn_exch{" HB1 32S"}="1HB  32S"; $resn_exch{" HB2 32S"}="2HB  32S";
$resn_exch{" HB1 32T"}="1HB  32T"; $resn_exch{" HB2 32T"}="2HB  32T"; $resn_exch{"H211 330"}="1H21 330";
$resn_exch{"H212 330"}="2H21 330"; $resn_exch{"H221 330"}="1H22 330"; $resn_exch{"H222 330"}="2H22 330";
$resn_exch{"H231 330"}="1H23 330"; $resn_exch{"H232 330"}="2H23 330"; $resn_exch{"H241 330"}="1H24 330";
$resn_exch{"H242 330"}="2H24 330"; $resn_exch{"H243 330"}="3H24 330"; $resn_exch{"H381 333"}="1H38 333";
$resn_exch{"H382 333"}="2H38 333"; $resn_exch{"H321 335"}="1H32 335"; $resn_exch{"H322 335"}="2H32 335";
$resn_exch{"H121 335"}="1H12 335"; $resn_exch{"H122 335"}="2H12 335"; $resn_exch{" H11 336"}="1H1  336";
$resn_exch{" H12 336"}="2H1  336"; $resn_exch{" H61 336"}="1H6  336"; $resn_exch{" H62 336"}="2H6  336";
$resn_exch{" H91 336"}="1H9  336"; $resn_exch{" H92 336"}="2H9  336"; $resn_exch{"H131 336"}="1H13 336";
$resn_exch{"H132 336"}="2H13 336"; $resn_exch{"H221 336"}="1H22 336"; $resn_exch{"H222 336"}="2H22 336";
$resn_exch{"H251 336"}="1H25 336"; $resn_exch{"H252 336"}="2H25 336"; $resn_exch{"H431 336"}="1H43 336";
$resn_exch{"H432 336"}="2H43 336"; $resn_exch{"H461 336"}="1H46 336"; $resn_exch{"H462 336"}="2H46 336";
$resn_exch{"H511 336"}="1H51 336"; $resn_exch{"H512 336"}="2H51 336"; $resn_exch{"H561 336"}="1H56 336";
$resn_exch{"H562 336"}="2H56 336"; $resn_exch{"H581 336"}="1H58 336"; $resn_exch{"H582 336"}="2H58 336";
$resn_exch{"H611 336"}="1H61 336"; $resn_exch{"H612 336"}="2H61 336"; $resn_exch{"H21A 339"}="1H2  339";
$resn_exch{" H22 339"}="2H2  339"; $resn_exch{" H31 339"}="1H3  339"; $resn_exch{" H32 339"}="2H3  339";
$resn_exch{" H41 339"}="1H4  339"; $resn_exch{" H42 339"}="2H4  339"; $resn_exch{"HN71 339"}="1HN7 339";
$resn_exch{"HN72 339"}="2HN7 339"; $resn_exch{"HN81 339"}="1HN8 339"; $resn_exch{"HN82 339"}="2HN8 339";
$resn_exch{"H181 339"}="1H18 339"; $resn_exch{"H182 339"}="2H18 339"; $resn_exch{"H183 339"}="3H18 339";
$resn_exch{"H191 339"}="1H19 339"; $resn_exch{"H192 339"}="2H19 339"; $resn_exch{"H193 339"}="3H19 339";
$resn_exch{"H241 339"}="1H24 339"; $resn_exch{"H242 339"}="2H24 339"; $resn_exch{"H251 339"}="1H25 339";
$resn_exch{"H252 339"}="2H25 339"; $resn_exch{"H261 339"}="1H26 339"; $resn_exch{"H262 339"}="2H26 339";
$resn_exch{"H291 339"}="1H29 339"; $resn_exch{"H292 339"}="2H29 339"; $resn_exch{"H421 339"}="1H42 339";
$resn_exch{"H422 339"}="2H42 339"; $resn_exch{"H423 339"}="3H42 339"; $resn_exch{"H231 33A"}="1H23 33A";
$resn_exch{"H232 33A"}="2H23 33A"; $resn_exch{"H2A1 33P"}="1H2A 33P"; $resn_exch{"H2A2 33P"}="2H2A 33P";
$resn_exch{"H2A3 33P"}="3H2A 33P"; $resn_exch{"H4A1 33P"}="1H4A 33P"; $resn_exch{"H4A2 33P"}="2H4A 33P";
$resn_exch{"H5A1 33P"}="1H5A 33P"; $resn_exch{"H5A2 33P"}="2H5A 33P"; $resn_exch{"HOA2 33P"}="2HOA 33P";
$resn_exch{"HOA3 33P"}="3HOA 33P"; $resn_exch{"HOB2 33P"}="2HOB 33P"; $resn_exch{"HOB3 33P"}="3HOB 33P";
$resn_exch{" H81 33P"}="1H8  33P"; $resn_exch{" H82 33P"}="2H8  33P"; $resn_exch{"H101 33P"}="1H10 33P";
$resn_exch{"H102 33P"}="2H10 33P"; $resn_exch{"H103 33P"}="3H10 33P"; $resn_exch{"H471 33Z"}="1H47 33Z";
$resn_exch{"H472 33Z"}="2H47 33Z"; $resn_exch{"H351 33Z"}="1H35 33Z"; $resn_exch{"H352 33Z"}="2H35 33Z";
$resn_exch{"H301 33Z"}="1H30 33Z"; $resn_exch{"H302 33Z"}="2H30 33Z"; $resn_exch{"H303 33Z"}="3H30 33Z";
$resn_exch{"H291 33Z"}="1H29 33Z"; $resn_exch{"H292 33Z"}="2H29 33Z"; $resn_exch{"H293 33Z"}="3H29 33Z";
$resn_exch{"H231 33Z"}="1H23 33Z"; $resn_exch{"H232 33Z"}="2H23 33Z"; $resn_exch{"H111 33Z"}="1H11 33Z";
$resn_exch{"H112 33Z"}="2H11 33Z"; $resn_exch{" H11 340"}="1H1  340"; $resn_exch{"H121 340"}="1H12 340";
$resn_exch{"H122 340"}="2H12 340"; $resn_exch{"H123 340"}="3H12 340"; $resn_exch{"H191 340"}="1H19 340";
$resn_exch{"H192 340"}="2H19 340"; $resn_exch{"H251 340"}="1H25 340"; $resn_exch{"H261 340"}="1H26 340";
$resn_exch{"H271 340"}="1H27 340"; $resn_exch{"H281 340"}="1H28 340"; $resn_exch{"H291 340"}="1H29 340";
$resn_exch{"H371 340"}="1H37 340"; $resn_exch{"H372 340"}="2H37 340"; $resn_exch{"H373 340"}="3H37 340";
$resn_exch{"H451 340"}="1H45 340"; $resn_exch{"H452 340"}="2H45 340"; $resn_exch{"H453 340"}="3H45 340";
$resn_exch{"H381 342"}="1H38 342"; $resn_exch{"H382 342"}="2H38 342"; $resn_exch{"H271 342"}="1H27 342";
$resn_exch{"H272 342"}="2H27 342"; $resn_exch{"H221 342"}="1H22 342"; $resn_exch{"H222 342"}="2H22 342";
$resn_exch{"H241 342"}="1H24 342"; $resn_exch{"H242 342"}="2H24 342"; $resn_exch{"H251 342"}="1H25 342";
$resn_exch{"H252 342"}="2H25 342"; $resn_exch{"H261 342"}="1H26 342"; $resn_exch{"H262 342"}="2H26 342";
$resn_exch{"H471 343"}="1H47 343"; $resn_exch{"H472 343"}="2H47 343"; $resn_exch{"H351 343"}="1H35 343";
$resn_exch{"H352 343"}="2H35 343"; $resn_exch{"H301 343"}="1H30 343"; $resn_exch{"H302 343"}="2H30 343";
$resn_exch{"H303 343"}="3H30 343"; $resn_exch{"H291 343"}="1H29 343"; $resn_exch{"H292 343"}="2H29 343";
$resn_exch{"H293 343"}="3H29 343"; $resn_exch{"H111 343"}="1H11 343"; $resn_exch{"H112 343"}="2H11 343";
$resn_exch{"HN21 344"}="1HN2 344"; $resn_exch{"HN22 344"}="2HN2 344"; $resn_exch{"H261 345"}="1H26 345";
$resn_exch{"H262 345"}="2H26 345"; $resn_exch{"H271 345"}="1H27 345"; $resn_exch{"H272 345"}="2H27 345";
$resn_exch{"H291 345"}="1H29 345"; $resn_exch{"H292 345"}="2H29 345"; $resn_exch{"H301 345"}="1H30 345";
$resn_exch{"H302 345"}="2H30 345"; $resn_exch{"H391 345"}="1H39 345"; $resn_exch{"H392 345"}="2H39 345";
$resn_exch{"H272 346"}="2H27 346"; $resn_exch{"H271 346"}="1H27 346"; $resn_exch{"H292 346"}="2H29 346";
$resn_exch{"H291 346"}="1H29 346"; $resn_exch{"H312 346"}="2H31 346"; $resn_exch{"H311 346"}="1H31 346";
$resn_exch{"H241 346"}="1H24 346"; $resn_exch{"H242 346"}="2H24 346"; $resn_exch{"H303 346"}="3H30 346";
$resn_exch{"H302 346"}="2H30 346"; $resn_exch{"H301 346"}="1H30 346"; $resn_exch{" H11 347"}="1H1  347";
$resn_exch{" H12 347"}="2H1  347"; $resn_exch{" H13 347"}="3H1  347"; $resn_exch{" H41 347"}="1H4  347";
$resn_exch{" H42 347"}="2H4  347"; $resn_exch{"H111 347"}="1H11 347"; $resn_exch{"H112 347"}="2H11 347";
$resn_exch{"H131 347"}="1H13 347"; $resn_exch{"H132 347"}="2H13 347"; $resn_exch{"H141 347"}="1H14 347";
$resn_exch{"H142 347"}="2H14 347"; $resn_exch{"H151 347"}="1H15 347"; $resn_exch{"H152 347"}="2H15 347";
$resn_exch{"H161 347"}="1H16 347"; $resn_exch{"H162 347"}="2H16 347"; $resn_exch{"H191 347"}="1H19 347";
$resn_exch{"H192 347"}="2H19 347"; $resn_exch{"H193 347"}="3H19 347"; $resn_exch{"H201 347"}="1H20 347";
$resn_exch{"H202 347"}="2H20 347"; $resn_exch{"H203 347"}="3H20 347"; $resn_exch{"H211 347"}="1H21 347";
$resn_exch{"H212 347"}="2H21 347"; $resn_exch{"H213 347"}="3H21 347"; $resn_exch{"H1N1 348"}="1H1N 348";
$resn_exch{"H1N2 348"}="2H1N 348"; $resn_exch{"H121 348"}="1H12 348"; $resn_exch{"H122 348"}="2H12 348";
$resn_exch{"H131 348"}="1H13 348"; $resn_exch{"H132 348"}="2H13 348"; $resn_exch{"H141 348"}="1H14 348";
$resn_exch{"H142 348"}="2H14 348"; $resn_exch{"H231 348"}="1H23 348"; $resn_exch{"H232 348"}="2H23 348";
$resn_exch{"H281 348"}="1H28 348"; $resn_exch{"H282 348"}="2H28 348"; $resn_exch{"H283 348"}="3H28 348";
$resn_exch{"H411 348"}="1H41 348"; $resn_exch{"H412 348"}="2H41 348"; $resn_exch{"H413 348"}="3H41 348";
$resn_exch{"H421 348"}="1H42 348"; $resn_exch{"H422 348"}="2H42 348"; $resn_exch{"H423 348"}="3H42 348";
$resn_exch{"HN11 34A"}="1HN1 34A"; $resn_exch{"HN12 34A"}="2HN1 34A"; $resn_exch{" H71 34A"}="1H7  34A";
$resn_exch{" H72 34A"}="2H7  34A"; $resn_exch{" H73 34A"}="3H7  34A"; $resn_exch{" H81 34A"}="1H8  34A";
$resn_exch{" H82 34A"}="2H8  34A"; $resn_exch{" H83 34A"}="3H8  34A"; $resn_exch{"H5'1 34B"}="1H5' 34B";
$resn_exch{"H5'2 34B"}="2H5' 34B"; $resn_exch{"H6'1 34B"}="1H6' 34B"; $resn_exch{"H6'2 34B"}="2H6' 34B";
$resn_exch{"H6'3 34B"}="3H6' 34B"; $resn_exch{"HC21 34C"}="1HC2 34C"; $resn_exch{"HC41 34C"}="1HC4 34C";
$resn_exch{"HC51 34C"}="1HC5 34C"; $resn_exch{"HC71 34C"}="1HC7 34C"; $resn_exch{"HC72 34C"}="2HC7 34C";
$resn_exch{"HC91 34C"}="1HC9 34C"; $resn_exch{"HC92 34C"}="2HC9 34C"; $resn_exch{"H101 34C"}="1H10 34C";
$resn_exch{"H102 34C"}="2H10 34C"; $resn_exch{"H111 34C"}="1H11 34C"; $resn_exch{"H112 34C"}="2H11 34C";
$resn_exch{"H113 34C"}="3H11 34C"; $resn_exch{" H71 34P"}="1H7  34P"; $resn_exch{" H72 34P"}="2H7  34P";
$resn_exch{"H121 34P"}="1H12 34P"; $resn_exch{"H122 34P"}="2H12 34P"; $resn_exch{"H131 34P"}="1H13 34P";
$resn_exch{"H132 34P"}="2H13 34P"; $resn_exch{"H141 34P"}="1H14 34P"; $resn_exch{"H142 34P"}="2H14 34P";
$resn_exch{"H201 34P"}="1H20 34P"; $resn_exch{"H202 34P"}="2H20 34P"; $resn_exch{"H211 34P"}="1H21 34P";
$resn_exch{"H212 34P"}="2H21 34P"; $resn_exch{"H231 34P"}="1H23 34P"; $resn_exch{"H232 34P"}="2H23 34P";
$resn_exch{"H241 34P"}="1H24 34P"; $resn_exch{"H242 34P"}="2H24 34P"; $resn_exch{"H251 34P"}="1H25 34P";
$resn_exch{"H252 34P"}="2H25 34P"; $resn_exch{"H261 34P"}="1H26 34P"; $resn_exch{"H262 34P"}="2H26 34P";
$resn_exch{"H271 34P"}="1H27 34P"; $resn_exch{"H272 34P"}="2H27 34P"; $resn_exch{"H291 34P"}="1H29 34P";
$resn_exch{"H292 34P"}="2H29 34P"; $resn_exch{"HN21 354"}="1HN2 354"; $resn_exch{"HN22 354"}="2HN2 354";
$resn_exch{" HB1 354"}="1HB  354"; $resn_exch{" HB2 354"}="2HB  354"; $resn_exch{" HG1 354"}="1HG  354";
$resn_exch{" HG2 354"}="2HG  354"; $resn_exch{"H101 357"}="1H10 357"; $resn_exch{"H102 357"}="2H10 357";
$resn_exch{"H111 357"}="1H11 357"; $resn_exch{"H112 357"}="2H11 357"; $resn_exch{"H113 357"}="3H11 357";
$resn_exch{"H7'1 357"}="1H7* 357"; $resn_exch{"H7'2 357"}="2H7* 357"; $resn_exch{" H71 357"}="1H7  357";
$resn_exch{" H72 357"}="2H7  357"; $resn_exch{" H73 357"}="3H7  357"; $resn_exch{"H6'1 357"}="1H6\" 357";
$resn_exch{"H6'2 357"}="2H6\" 357"; $resn_exch{"H6'3 357"}="3H6\" 357"; $resn_exch{"H2'1 357"}="1H2\" 357";
$resn_exch{"H2'2 357"}="2H2\" 357"; $resn_exch{"H3'1 357"}="1H3\" 357"; $resn_exch{"H3'2 357"}="2H3\" 357";
$resn_exch{"H5'1 357"}="1H5\" 357"; $resn_exch{"H5'2 357"}="2H5\" 357"; $resn_exch{"H5'3 357"}="3H5\" 357";
$resn_exch{" H6' 357"}=" H6* 357"; $resn_exch{" H5' 357"}=" H5* 357"; $resn_exch{" H4' 357"}=" H4* 357";
$resn_exch{"H8'1 357"}="1H8* 357"; $resn_exch{"H8'2 357"}="2H8* 357"; $resn_exch{"H8'3 357"}="3H8* 357";
$resn_exch{" H2' 357"}=" H2* 357"; $resn_exch{" H81 358"}="1H8  358"; $resn_exch{" H82 358"}="2H8  358";
$resn_exch{" H91 358"}="1H9  358"; $resn_exch{" H92 358"}="2H9  358"; $resn_exch{"H231 358"}="1H23 358";
$resn_exch{"H232 358"}="2H23 358"; $resn_exch{"H241 358"}="1H24 358"; $resn_exch{"H242 358"}="2H24 358";
$resn_exch{"H251 358"}="1H25 358"; $resn_exch{"H252 358"}="2H25 358"; $resn_exch{"H261 358"}="1H26 358";
$resn_exch{"H262 358"}="2H26 358"; $resn_exch{" H41 35A"}="1H4  35A"; $resn_exch{" H42 35A"}="2H4  35A";
$resn_exch{" H61 35A"}="1H6  35A"; $resn_exch{" H62 35A"}="2H6  35A"; $resn_exch{"H111 35A"}="1H11 35A";
$resn_exch{"H112 35A"}="2H11 35A"; $resn_exch{"H131 35A"}="1H13 35A"; $resn_exch{"H132 35A"}="2H13 35A";
$resn_exch{"H141 35A"}="1H14 35A"; $resn_exch{"H142 35A"}="2H14 35A"; $resn_exch{"H161 35A"}="1H16 35A";
$resn_exch{"H162 35A"}="2H16 35A"; $resn_exch{"H201 35A"}="1H20 35A"; $resn_exch{"H202 35A"}="2H20 35A";
$resn_exch{"H203 35A"}="3H20 35A"; $resn_exch{"H231 35A"}="1H23 35A"; $resn_exch{"H232 35A"}="2H23 35A";
$resn_exch{"H233 35A"}="3H23 35A"; $resn_exch{"H241 35A"}="1H24 35A"; $resn_exch{"H242 35A"}="2H24 35A";
$resn_exch{"H243 35A"}="3H24 35A"; $resn_exch{"H271 35A"}="1H27 35A"; $resn_exch{"H272 35A"}="2H27 35A";
$resn_exch{"H273 35A"}="3H27 35A"; $resn_exch{"H281 35A"}="1H28 35A"; $resn_exch{"H282 35A"}="2H28 35A";
$resn_exch{"HOP2 35G"}="2HOP 35G"; $resn_exch{"H5'1 35G"}="1H5* 35G"; $resn_exch{"H5'2 35G"}="2H5* 35G";
$resn_exch{" H4' 35G"}=" H4* 35G"; $resn_exch{" H3' 35G"}=" H3* 35G"; $resn_exch{" H2' 35G"}=" H2* 35G";
$resn_exch{"HO2' 35G"}="*HO2 35G"; $resn_exch{" H1' 35G"}=" H1* 35G"; $resn_exch{"HN21 35G"}="1HN2 35G";
$resn_exch{"HN22 35G"}="2HN2 35G"; $resn_exch{"HAU1 364"}="1HAU 364"; $resn_exch{"HAU2 364"}="2HAU 364";
$resn_exch{"H191 367"}="1H19 367"; $resn_exch{"H192 367"}="2H19 367"; $resn_exch{"H441 367"}="1H44 367";
$resn_exch{"H442 367"}="2H44 367"; $resn_exch{"H451 367"}="1H45 367"; $resn_exch{"H452 367"}="2H45 367";
$resn_exch{"H461 367"}="1H46 367"; $resn_exch{"H462 367"}="2H46 367"; $resn_exch{"H091 368"}="1H09 368";
$resn_exch{"H092 368"}="2H09 368"; $resn_exch{"H131 368"}="1H13 368"; $resn_exch{"H132 368"}="2H13 368";
$resn_exch{"HP61 368"}="1HP6 368"; $resn_exch{"HP62 368"}="2HP6 368"; $resn_exch{"H271 368"}="1H27 368";
$resn_exch{"H272 368"}="2H27 368"; $resn_exch{"H161 371"}="1H16 371"; $resn_exch{"H162 371"}="2H16 371";
$resn_exch{"H163 371"}="3H16 371"; $resn_exch{"H171 371"}="1H17 371"; $resn_exch{"H172 371"}="2H17 371";
$resn_exch{"H181 371"}="1H18 371"; $resn_exch{"H182 371"}="2H18 371"; $resn_exch{"H191 371"}="1H19 371";
$resn_exch{"H192 371"}="2H19 371"; $resn_exch{"H311 373"}="1H31 373"; $resn_exch{"H312 373"}="2H31 373";
$resn_exch{"H351 373"}="1H35 373"; $resn_exch{"H352 373"}="2H35 373"; $resn_exch{"H361 373"}="1H36 373";
$resn_exch{"H362 373"}="2H36 373"; $resn_exch{"H381 373"}="1H38 373"; $resn_exch{"H382 373"}="2H38 373";
$resn_exch{"H391 373"}="1H39 373"; $resn_exch{"H392 373"}="2H39 373"; $resn_exch{"H481 373"}="1H48 373";
$resn_exch{"H482 373"}="2H48 373"; $resn_exch{"H501 373"}="1H50 373"; $resn_exch{"H502 373"}="2H50 373";
$resn_exch{" H71 37A"}="1H7  37A"; $resn_exch{" H72 37A"}="2H7  37A"; $resn_exch{" H31 37A"}="1H3  37A";
$resn_exch{" H32 37A"}="2H3  37A"; $resn_exch{"H121 37A"}="1H12 37A"; $resn_exch{"H122 37A"}="2H12 37A";
$resn_exch{"H131 37T"}="1H13 37T"; $resn_exch{"H132 37T"}="2H13 37T"; $resn_exch{"H133 37T"}="3H13 37T";
$resn_exch{"H121 37T"}="1H12 37T"; $resn_exch{"H122 37T"}="2H12 37T"; $resn_exch{"H123 37T"}="3H12 37T";
$resn_exch{"H1N1 380"}="1H1N 380"; $resn_exch{"H1N2 380"}="2H1N 380"; $resn_exch{"H251 380"}="1H25 380";
$resn_exch{"H252 380"}="2H25 380"; $resn_exch{"H241 380"}="1H24 380"; $resn_exch{"H242 380"}="2H24 380";
$resn_exch{"H243 380"}="3H24 380"; $resn_exch{"H321 380"}="1H32 380"; $resn_exch{"H322 380"}="2H32 380";
$resn_exch{"H323 380"}="3H32 380"; $resn_exch{" H11 382"}="1H1  382"; $resn_exch{" H12 382"}="2H1  382";
$resn_exch{" H21 382"}="1H2  382"; $resn_exch{" H22 382"}="2H2  382"; $resn_exch{" H51 382"}="1H5  382";
$resn_exch{"H52A 382"}="2H5  382"; $resn_exch{" H61 382"}="1H6  382"; $resn_exch{" H62 382"}="2H6  382";
$resn_exch{"H191 382"}="1H19 382"; $resn_exch{"H192 382"}="2H19 382"; $resn_exch{"H341 382"}="1H34 382";
$resn_exch{"H342 382"}="2H34 382"; $resn_exch{"H141 383"}="1H14 383"; $resn_exch{"H142 383"}="2H14 383";
$resn_exch{"H161 383"}="1H16 383"; $resn_exch{"H162 383"}="2H16 383"; $resn_exch{"H301 383"}="1H30 383";
$resn_exch{"H302 383"}="2H30 383"; $resn_exch{" H41 385"}="1H4  385"; $resn_exch{" H42 385"}="2H4  385";
$resn_exch{" H71 385"}="1H7  385"; $resn_exch{" H72 385"}="2H7  385"; $resn_exch{" H91 385"}="1H9  385";
$resn_exch{" H92 385"}="2H9  385"; $resn_exch{"H141 385"}="1H14 385"; $resn_exch{"H142 385"}="2H14 385";
$resn_exch{"H171 385"}="1H17 385"; $resn_exch{"H172 385"}="2H17 385"; $resn_exch{"H251 385"}="1H25 385";
$resn_exch{"H252 385"}="2H25 385"; $resn_exch{"H341 385"}="1H34 385"; $resn_exch{"H342 385"}="2H34 385";
$resn_exch{"H343 385"}="3H34 385"; $resn_exch{"H351 385"}="1H35 385"; $resn_exch{"H352 385"}="2H35 385";
$resn_exch{"H353 385"}="3H35 385"; $resn_exch{"H361 385"}="1H36 385"; $resn_exch{"H362 385"}="2H36 385";
$resn_exch{"H451 385"}="1H45 385"; $resn_exch{"H452 385"}="2H45 385"; $resn_exch{"H481 385"}="1H48 385";
$resn_exch{"H482 385"}="2H48 385"; $resn_exch{"H483 385"}="3H48 385"; $resn_exch{" H11 388"}="1H1  388";
$resn_exch{" H12 388"}="2H1  388"; $resn_exch{"H211 388"}="1H21 388"; $resn_exch{"H212 388"}="2H21 388";
$resn_exch{"H381 391"}="1H38 391"; $resn_exch{"H382 391"}="2H38 391"; $resn_exch{"H271 391"}="1H27 391";
$resn_exch{"H272 391"}="2H27 391"; $resn_exch{"H273 391"}="3H27 391"; $resn_exch{" H71 393"}="1H7  393";
$resn_exch{" H72 393"}="2H7  393"; $resn_exch{"H181 393"}="1H18 393"; $resn_exch{"H182 393"}="2H18 393";
$resn_exch{"H141 394"}="1H14 394"; $resn_exch{"H142 394"}="2H14 394"; $resn_exch{"H151 394"}="1H15 394";
$resn_exch{"H152 394"}="2H15 394"; $resn_exch{"H181 394"}="1H18 394"; $resn_exch{"H182 394"}="2H18 394";
$resn_exch{"H183 394"}="3H18 394"; $resn_exch{"H191 394"}="1H19 394"; $resn_exch{"H192 394"}="2H19 394";
$resn_exch{"H193 394"}="3H19 394"; $resn_exch{"H201 394"}="1H20 394"; $resn_exch{"H202 394"}="2H20 394";
$resn_exch{"H203 394"}="3H20 394"; $resn_exch{"H211 394"}="1H21 394"; $resn_exch{"H212 394"}="2H21 394";
$resn_exch{"H213 394"}="3H21 394"; $resn_exch{"HO22 394"}="2HO2 394"; $resn_exch{"HO27 394"}="7HO2 394";
$resn_exch{"H141 395"}="1H14 395"; $resn_exch{"H142 395"}="2H14 395"; $resn_exch{"H151 395"}="1H15 395";
$resn_exch{"H152 395"}="2H15 395"; $resn_exch{"H181 395"}="1H18 395"; $resn_exch{"H182 395"}="2H18 395";
$resn_exch{"H183 395"}="3H18 395"; $resn_exch{"H191 395"}="1H19 395"; $resn_exch{"H192 395"}="2H19 395";
$resn_exch{"H193 395"}="3H19 395"; $resn_exch{"H201 395"}="1H20 395"; $resn_exch{"H202 395"}="2H20 395";
$resn_exch{"H203 395"}="3H20 395"; $resn_exch{"H211 395"}="1H21 395"; $resn_exch{"H212 395"}="2H21 395";
$resn_exch{"H213 395"}="3H21 395"; $resn_exch{"HO22 395"}="2HO2 395"; $resn_exch{"HO27 395"}="7HO2 395";
$resn_exch{"HN61 39A"}="1HN6 39A"; $resn_exch{"HN62 39A"}="2HN6 39A"; $resn_exch{"H9M1 39A"}="1H9M 39A";
$resn_exch{"H9M2 39A"}="2H9M 39A"; $resn_exch{"H9M3 39A"}="3H9M 39A"; $resn_exch{"H3M1 39A"}="1H3M 39A";
$resn_exch{"H3M2 39A"}="2H3M 39A"; $resn_exch{"H3M3 39A"}="3H3M 39A"; $resn_exch{"H2'1 39B"}="1H2' 39B";
$resn_exch{"H2'2 39B"}="2H2' 39B"; $resn_exch{"H4'1 39B"}="1H4' 39B"; $resn_exch{"H4'2 39B"}="2H4' 39B";
$resn_exch{"H5'1 39B"}="1H5' 39B"; $resn_exch{"H5'2 39B"}="2H5' 39B"; $resn_exch{"H6'1 39B"}="1H6' 39B";
$resn_exch{"H6'2 39B"}="2H6' 39B"; $resn_exch{"H6'3 39B"}="3H6' 39B"; $resn_exch{"H2'1 39E"}="1H2' 39E";
$resn_exch{" HA2 39E"}="2HA  39E"; $resn_exch{" HA1 39E"}="1HA  39E"; $resn_exch{" HB2 39E"}="2HB  39E";
$resn_exch{" HB1 39E"}="1HB  39E"; $resn_exch{" HC2 39E"}="2HC  39E"; $resn_exch{" HC1 39E"}="1HC  39E";
$resn_exch{" HD2 39E"}="2HD  39E"; $resn_exch{" HE3 39E"}="3HE  39E"; $resn_exch{"H201 39Z"}="1H20 39Z";
$resn_exch{"H202 39Z"}="2H20 39Z"; $resn_exch{"H203 39Z"}="3H20 39Z"; $resn_exch{"H191 3A3"}="1H19 3A3";
$resn_exch{"H192 3A3"}="2H19 3A3"; $resn_exch{"H201 3A3"}="1H20 3A3"; $resn_exch{"H202 3A3"}="2H20 3A3";
$resn_exch{"H5'1 3AA"}="1H5* 3AA"; $resn_exch{"H5'2 3AA"}="2H5* 3AA"; $resn_exch{" H'4 3AA"}="4H*  3AA";
$resn_exch{" H'1 3AA"}="1H*  3AA"; $resn_exch{"HN61 3AA"}="1HN6 3AA"; $resn_exch{"HN62 3AA"}="2HN6 3AA";
$resn_exch{" H'2 3AA"}="2H*  3AA"; $resn_exch{" H'3 3AA"}="3H*  3AA"; $resn_exch{"HA3' 3AA"}="HA3* 3AA";
$resn_exch{"H15' 3AA"}="H15* 3AA"; $resn_exch{"H25' 3AA"}="H25* 3AA"; $resn_exch{"HC4' 3AA"}="HC4* 3AA";
$resn_exch{" H3' 3AA"}=" H3* 3AA"; $resn_exch{"HO3' 3AA"}="HO3* 3AA"; $resn_exch{" H2' 3AA"}=" H2* 3AA";
$resn_exch{"HO2' 3AA"}="HO2* 3AA"; $resn_exch{" H1' 3AA"}=" H1* 3AA"; $resn_exch{"HN71 3AA"}="1HN7 3AA";
$resn_exch{"HN72 3AA"}="2HN7 3AA"; $resn_exch{"H511 3AC"}="1H51 3AC"; $resn_exch{"H512 3AC"}="2H51 3AC";
$resn_exch{"H513 3AC"}="3H51 3AC"; $resn_exch{"H611 3AC"}="1H61 3AC"; $resn_exch{"H612 3AC"}="2H61 3AC";
$resn_exch{"H621 3AC"}="1H62 3AC"; $resn_exch{"H622 3AC"}="2H62 3AC"; $resn_exch{"H623 3AC"}="3H62 3AC";
$resn_exch{"HO5' 3AD"}="*HO5 3AD"; $resn_exch{"H5'1 3AD"}="1H5* 3AD"; $resn_exch{"H5'2 3AD"}="2H5* 3AD";
$resn_exch{" H4' 3AD"}=" H4* 3AD"; $resn_exch{"H3'1 3AD"}="1H3* 3AD"; $resn_exch{"H3'2 3AD"}="2H3* 3AD";
$resn_exch{" H2' 3AD"}=" H2* 3AD"; $resn_exch{"HO2' 3AD"}="*HO2 3AD"; $resn_exch{" H1' 3AD"}=" H1* 3AD";
$resn_exch{"HN61 3AD"}="1HN6 3AD"; $resn_exch{"HN62 3AD"}="2HN6 3AD"; $resn_exch{" H41 3AF"}="1H4  3AF";
$resn_exch{" H43 3AF"}="3H4  3AF"; $resn_exch{" H42 3AF"}="2H4  3AF"; $resn_exch{" H51 3AF"}="1H5  3AF";
$resn_exch{" H52 3AF"}="2H5  3AF"; $resn_exch{" H53 3AF"}="3H5  3AF"; $resn_exch{" H61 3AF"}="1H6  3AF";
$resn_exch{" H63 3AF"}="3H6  3AF"; $resn_exch{" H62 3AF"}="2H6  3AF"; $resn_exch{" H71 3AF"}="1H7  3AF";
$resn_exch{"HC11 3AG"}="1HC1 3AG"; $resn_exch{"HC12 3AG"}="2HC1 3AG"; $resn_exch{"HC13 3AG"}="3HC1 3AG";
$resn_exch{"HC31 3AG"}="1HC3 3AG"; $resn_exch{"HC32 3AG"}="2HC3 3AG"; $resn_exch{"HC33 3AG"}="3HC3 3AG";
$resn_exch{"HC51 3AG"}="1HC5 3AG"; $resn_exch{"HC52 3AG"}="2HC5 3AG"; $resn_exch{"HC61 3AG"}="1HC6 3AG";
$resn_exch{"HC62 3AG"}="2HC6 3AG"; $resn_exch{"HC71 3AG"}="1HC7 3AG"; $resn_exch{"HC72 3AG"}="2HC7 3AG";
$resn_exch{"HC81 3AG"}="1HC8 3AG"; $resn_exch{"HC82 3AG"}="2HC8 3AG"; $resn_exch{"HC91 3AG"}="1HC9 3AG";
$resn_exch{"HC92 3AG"}="2HC9 3AG"; $resn_exch{"HC93 3AG"}="3HC9 3AG"; $resn_exch{" H2  3AH"}=" HN2 3AH";
$resn_exch{" HB2 3AH"}="1HB  3AH"; $resn_exch{" HB3 3AH"}="2HB  3AH"; $resn_exch{"HN31 3AH"}="1HN3 3AH";
$resn_exch{"HN32 3AH"}="2HN3 3AH"; $resn_exch{"HN11 3AL"}="1HN1 3AL"; $resn_exch{"HN12 3AL"}="2HN1 3AL";
$resn_exch{"HO2' 3AM"}="HO2* 3AM"; $resn_exch{" H2' 3AM"}=" H2* 3AM"; $resn_exch{" H3' 3AM"}=" H3* 3AM";
$resn_exch{" H1' 3AM"}=" H1* 3AM"; $resn_exch{" H4' 3AM"}=" H4* 3AM"; $resn_exch{" H5' 3AM"}=" H5* 3AM";
$resn_exch{"H5'A 3AM"}="H5*A 3AM"; $resn_exch{"HO5' 3AM"}="HO5* 3AM"; $resn_exch{"HOB2 3AN"}="2HOB 3AN";
$resn_exch{"HOB3 3AN"}="3HOB 3AN"; $resn_exch{"HOA1 3AN"}="1HOA 3AN"; $resn_exch{"H5'1 3AN"}="1H5* 3AN";
$resn_exch{"H5'2 3AN"}="2H5* 3AN"; $resn_exch{"HC'4 3AN"}="4HC* 3AN"; $resn_exch{"HC'3 3AN"}="3HC* 3AN";
$resn_exch{"H3'1 3AN"}="1H3* 3AN"; $resn_exch{"H3'2 3AN"}="2H3* 3AN"; $resn_exch{"HC'2 3AN"}="2HC* 3AN";
$resn_exch{"HO'2 3AN"}="2HO* 3AN"; $resn_exch{"HC'1 3AN"}="1HC* 3AN"; $resn_exch{" HC8 3AN"}="8HC  3AN";
$resn_exch{"HN61 3AN"}="1HN6 3AN"; $resn_exch{"HN62 3AN"}="2HN6 3AN"; $resn_exch{" HC2 3AN"}="2HC  3AN";
$resn_exch{"HN31 3AP"}="1HN3 3AP"; $resn_exch{"HN32 3AP"}="2HN3 3AP"; $resn_exch{" H2  3AR"}=" HN2 3AR";
$resn_exch{" HB2 3AR"}="1HB  3AR"; $resn_exch{" HB3 3AR"}="2HB  3AR"; $resn_exch{" HG2 3AR"}="1HG  3AR";
$resn_exch{" HG3 3AR"}="2HG  3AR"; $resn_exch{" HD2 3AR"}="1HD  3AR"; $resn_exch{" HD3 3AR"}="2HD  3AR";
$resn_exch{" HH1 3AR"}="1HH1 3AR"; $resn_exch{"HH21 3AR"}="1HH2 3AR"; $resn_exch{"HH22 3AR"}="2HH2 3AR";
$resn_exch{" H11 3AR"}="1H1  3AR"; $resn_exch{" H12 3AR"}="2H1  3AR"; $resn_exch{" H21 3AR"}="1H2  3AR";
$resn_exch{" H22 3AR"}="2H2  3AR"; $resn_exch{" H31 3AR"}="1H3  3AR"; $resn_exch{" H32 3AR"}="2H3  3AR";
$resn_exch{" H33 3AR"}="3H3  3AR"; $resn_exch{"HOG2 3AT"}="2HOG 3AT"; $resn_exch{"HOG3 3AT"}="3HOG 3AT";
$resn_exch{"HOB2 3AT"}="2HOB 3AT"; $resn_exch{"HOA2 3AT"}="2HOA 3AT"; $resn_exch{"H5'1 3AT"}="1H5* 3AT";
$resn_exch{"H5'2 3AT"}="2H5* 3AT"; $resn_exch{" H4' 3AT"}=" H4* 3AT"; $resn_exch{"H3'1 3AT"}="1H3* 3AT";
$resn_exch{"H3'2 3AT"}="2H3* 3AT"; $resn_exch{"H2'1 3AT"}="1H2* 3AT"; $resn_exch{"HO2' 3AT"}="HO2* 3AT";
$resn_exch{" H1' 3AT"}=" H1* 3AT"; $resn_exch{"HN61 3AT"}="1HN6 3AT"; $resn_exch{"HN62 3AT"}="2HN6 3AT";
$resn_exch{"HAA1 3AY"}="1HAA 3AY"; $resn_exch{"HAA2 3AY"}="2HAA 3AY"; $resn_exch{"HAH1 3AY"}="1HAH 3AY";
$resn_exch{"HAH2 3AY"}="2HAH 3AY"; $resn_exch{"HAI1 3AY"}="1HAI 3AY"; $resn_exch{"HAI2 3AY"}="2HAI 3AY";
$resn_exch{"H6C1 3B3"}="1H6C 3B3"; $resn_exch{"H6C2 3B3"}="2H6C 3B3"; $resn_exch{"HAC1 3B3"}="1HAC 3B3";
$resn_exch{"HAC2 3B3"}="2HAC 3B3"; $resn_exch{" HN1 3B3"}="1HN  3B3"; $resn_exch{" HN2 3B3"}="2HN  3B3";
$resn_exch{" H11 3BB"}="1H1  3BB"; $resn_exch{" H12 3BB"}="2H1  3BB"; $resn_exch{" H31 3BB"}="1H3  3BB";
$resn_exch{" H32 3BB"}="2H3  3BB"; $resn_exch{" H41 3BB"}="1H4  3BB"; $resn_exch{" H42 3BB"}="2H4  3BB";
$resn_exch{" H51 3BC"}="1H5  3BC"; $resn_exch{" H52 3BC"}="2H5  3BC"; $resn_exch{" H53 3BC"}="3H5  3BC";
$resn_exch{" H61 3BC"}="1H6  3BC"; $resn_exch{" H62 3BC"}="2H6  3BC"; $resn_exch{" H63 3BC"}="3H6  3BC";
$resn_exch{" H71 3BC"}="1H7  3BC"; $resn_exch{" H72 3BC"}="2H7  3BC"; $resn_exch{" H73 3BC"}="3H7  3BC";
$resn_exch{"H111 3BC"}="1H11 3BC"; $resn_exch{"H112 3BC"}="2H11 3BC"; $resn_exch{"H113 3BC"}="3H11 3BC";
$resn_exch{"H121 3BC"}="1H12 3BC"; $resn_exch{"H122 3BC"}="2H12 3BC"; $resn_exch{"H123 3BC"}="3H12 3BC";
$resn_exch{"H131 3BC"}="1H13 3BC"; $resn_exch{"H132 3BC"}="2H13 3BC"; $resn_exch{"H133 3BC"}="3H13 3BC";
$resn_exch{"H171 3BC"}="1H17 3BC"; $resn_exch{"H172 3BC"}="2H17 3BC"; $resn_exch{"H221 3BC"}="1H22 3BC";
$resn_exch{"H222 3BC"}="2H22 3BC"; $resn_exch{"H223 3BC"}="3H22 3BC"; $resn_exch{"H231 3BC"}="1H23 3BC";
$resn_exch{"H232 3BC"}="2H23 3BC"; $resn_exch{"H233 3BC"}="3H23 3BC"; $resn_exch{"H281 3BC"}="1H28 3BC";
$resn_exch{"H282 3BC"}="2H28 3BC"; $resn_exch{"H301 3BC"}="1H30 3BC"; $resn_exch{"H302 3BC"}="2H30 3BC";
$resn_exch{"H311 3BC"}="1H31 3BC"; $resn_exch{"H312 3BC"}="2H31 3BC"; $resn_exch{"H361 3BC"}="1H36 3BC";
$resn_exch{"H362 3BC"}="2H36 3BC"; $resn_exch{"H511 3BC"}="1H51 3BC"; $resn_exch{"H512 3BC"}="2H51 3BC";
$resn_exch{"H513 3BC"}="3H51 3BC"; $resn_exch{"H521 3BC"}="1H52 3BC"; $resn_exch{"H522 3BC"}="2H52 3BC";
$resn_exch{"H523 3BC"}="3H52 3BC"; $resn_exch{" H11 3BN"}="1H1  3BN"; $resn_exch{"H12A 3BN"}="2H1  3BN";
$resn_exch{" H31 3BN"}="1H3  3BN"; $resn_exch{" H32 3BN"}="2H3  3BN"; $resn_exch{" H41 3BN"}="1H4  3BN";
$resn_exch{" H42 3BN"}="2H4  3BN"; $resn_exch{" H81 3BN"}="1H8  3BN"; $resn_exch{" H82 3BN"}="2H8  3BN";
$resn_exch{"H161 3BN"}="1H16 3BN"; $resn_exch{"H162 3BN"}="2H16 3BN"; $resn_exch{"H163 3BN"}="3H16 3BN";
$resn_exch{"H231 3BN"}="1H23 3BN"; $resn_exch{"H232 3BN"}="2H23 3BN"; $resn_exch{"H233 3BN"}="3H23 3BN";
$resn_exch{"H191 3BP"}="1H19 3BP"; $resn_exch{"H192 3BP"}="2H19 3BP"; $resn_exch{"H201 3BP"}="1H20 3BP";
$resn_exch{"H202 3BP"}="2H20 3BP"; $resn_exch{"H321 3BP"}="1H32 3BP"; $resn_exch{"H322 3BP"}="2H32 3BP";
$resn_exch{"HN31 3BP"}="1HN3 3BP"; $resn_exch{"HN32 3BP"}="2HN3 3BP"; $resn_exch{"H11A 3BP"}="1H1  3BP";
$resn_exch{"H12A 3BP"}="2H1  3BP"; $resn_exch{" H91 3BP"}="1H9  3BP"; $resn_exch{" H92 3BP"}="2H9  3BP";
$resn_exch{" H81 3BP"}="1H8  3BP"; $resn_exch{" H82 3BP"}="2H8  3BP"; $resn_exch{" H83 3BP"}="3H8  3BP";
$resn_exch{"HN21 3BP"}="1HN2 3BP"; $resn_exch{"HN22 3BP"}="2HN2 3BP"; $resn_exch{" H11 3BR"}="1H1  3BR";
$resn_exch{" H12 3BR"}="2H1  3BR"; $resn_exch{" H21 3BR"}="1H2  3BR"; $resn_exch{" H22 3BR"}="2H2  3BR";
$resn_exch{" H31 3BR"}="1H3  3BR"; $resn_exch{" H32 3BR"}="2H3  3BR"; $resn_exch{" H33 3BR"}="3H3  3BR";
$resn_exch{"HCA1 3BT"}="1HCA 3BT"; $resn_exch{"HCA2 3BT"}="2HCA 3BT"; $resn_exch{"HCB1 3BT"}="1HCB 3BT";
$resn_exch{"HCB2 3BT"}="2HCB 3BT"; $resn_exch{"HN11 3BT"}="1HN1 3BT"; $resn_exch{"HN12 3BT"}="2HN1 3BT";
$resn_exch{"HN21 3BT"}="1HN2 3BT"; $resn_exch{" H21 3BT"}="1H2  3BT"; $resn_exch{" H61 3BT"}="1H6  3BT";
$resn_exch{" H51 3BT"}="1H5  3BT"; $resn_exch{" H41 3BT"}="1H4  3BT"; $resn_exch{"HCC1 3BT"}="1HCC 3BT";
$resn_exch{"HCC2 3BT"}="2HCC 3BT"; $resn_exch{"HCD1 3BT"}="1HCD 3BT"; $resn_exch{"HCD2 3BT"}="2HCD 3BT";
$resn_exch{"H1'1 3BT"}="1H1' 3BT"; $resn_exch{"H1'2 3BT"}="2H1' 3BT"; $resn_exch{"HN2' 3BT"}="'HN2 3BT";
$resn_exch{"H161 3C3"}="1H16 3C3"; $resn_exch{"H162 3C3"}="2H16 3C3"; $resn_exch{"H181 3C3"}="1H18 3C3";
$resn_exch{"H182 3C3"}="2H18 3C3"; $resn_exch{"H191 3C3"}="1H19 3C3"; $resn_exch{"H192 3C3"}="2H19 3C3";
$resn_exch{"H201 3C3"}="1H20 3C3"; $resn_exch{"H202 3C3"}="2H20 3C3"; $resn_exch{"H203 3C3"}="3H20 3C3";
$resn_exch{"H171 3C3"}="1H17 3C3"; $resn_exch{"H172 3C3"}="2H17 3C3"; $resn_exch{" HG1 3C4"}="1HG  3C4";
$resn_exch{" HG2 3C4"}="2HG  3C4"; $resn_exch{"H191 3CB"}="1H19 3CB"; $resn_exch{"H192 3CB"}="2H19 3CB";
$resn_exch{"H201 3CB"}="1H20 3CB"; $resn_exch{"H202 3CB"}="2H20 3CB"; $resn_exch{"H321 3CB"}="1H32 3CB";
$resn_exch{"H322 3CB"}="2H32 3CB"; $resn_exch{"HN51 3CB"}="1HN5 3CB"; $resn_exch{"HN52 3CB"}="2HN5 3CB";
$resn_exch{" H41 3CB"}="1H4  3CB"; $resn_exch{" H42 3CB"}="2H4  3CB"; $resn_exch{"H31A 3CB"}="1H3  3CB";
$resn_exch{" H32 3CB"}="2H3  3CB"; $resn_exch{" H33 3CB"}="3H3  3CB"; $resn_exch{" H91 3CB"}="1H9  3CB";
$resn_exch{" H92 3CB"}="2H9  3CB"; $resn_exch{" H61 3CB"}="1H6  3CB"; $resn_exch{" H62 3CB"}="2H6  3CB";
$resn_exch{" H63 3CB"}="3H6  3CB"; $resn_exch{"HN21 3CB"}="1HN2 3CB"; $resn_exch{"HN22 3CB"}="2HN2 3CB";
$resn_exch{" H71 3CC"}="1H7  3CC"; $resn_exch{" H72 3CC"}="2H7  3CC"; $resn_exch{" H91 3CC"}="1H9  3CC";
$resn_exch{" H92 3CC"}="2H9  3CC"; $resn_exch{"HN11 3CC"}="1HN1 3CC"; $resn_exch{"HN12 3CC"}="2HN1 3CC";
$resn_exch{"H121 3CC"}="1H12 3CC"; $resn_exch{"H122 3CC"}="2H12 3CC"; $resn_exch{"H131 3CC"}="1H13 3CC";
$resn_exch{"H132 3CC"}="2H13 3CC"; $resn_exch{"H141 3CC"}="1H14 3CC"; $resn_exch{"H142 3CC"}="2H14 3CC";
$resn_exch{"H143 3CC"}="3H14 3CC"; $resn_exch{"H151 3CC"}="1H15 3CC"; $resn_exch{"H152 3CC"}="2H15 3CC";
$resn_exch{"H161 3CC"}="1H16 3CC"; $resn_exch{"H162 3CC"}="2H16 3CC"; $resn_exch{"H171 3CC"}="1H17 3CC";
$resn_exch{"H172 3CC"}="2H17 3CC"; $resn_exch{"H173 3CC"}="3H17 3CC"; $resn_exch{" H21 3CL"}="1H2  3CL";
$resn_exch{" H22 3CL"}="2H2  3CL"; $resn_exch{" H31 3CL"}="1H3  3CL"; $resn_exch{" H32 3CL"}="2H3  3CL";
$resn_exch{" H41 3CL"}="1H4  3CL"; $resn_exch{" H42 3CL"}="2H4  3CL"; $resn_exch{" H61 3CM"}="1H6  3CM";
$resn_exch{"H62A 3CM"}="2H6  3CM"; $resn_exch{"H111 3CM"}="1H11 3CM"; $resn_exch{"H112 3CM"}="2H11 3CM";
$resn_exch{"H211 3CM"}="1H21 3CM"; $resn_exch{"H212 3CM"}="2H21 3CM"; $resn_exch{"H311 3CM"}="1H31 3CM";
$resn_exch{"H312 3CM"}="2H31 3CM"; $resn_exch{"H421 3CM"}="1H42 3CM"; $resn_exch{"H422 3CM"}="2H42 3CM";
$resn_exch{"H601 3CM"}="1H60 3CM"; $resn_exch{"H602 3CM"}="2H60 3CM"; $resn_exch{" HA1 3CN"}="1HA  3CN";
$resn_exch{" HA2 3CN"}="2HA  3CN"; $resn_exch{" HA3 3CN"}="3HA  3CN"; $resn_exch{" HB1 3CN"}="1HB  3CN";
$resn_exch{" HB2 3CN"}="2HB  3CN"; $resn_exch{" HC1 3CN"}="1HC  3CN"; $resn_exch{" HC2 3CN"}="2HC  3CN";
$resn_exch{"HND1 3CN"}="1HND 3CN"; $resn_exch{"HND2 3CN"}="2HND 3CN"; $resn_exch{" H61 3CP"}="1H6  3CP";
$resn_exch{" H62 3CP"}="2H6  3CP"; $resn_exch{"HO'2 3CP"}="2HO' 3CP"; $resn_exch{"H5'1 3CP"}="1H5' 3CP";
$resn_exch{"H5'2 3CP"}="2H5' 3CP"; $resn_exch{" HB1 3CP"}="1HB  3CP"; $resn_exch{" HB2 3CP"}="2HB  3CP";
$resn_exch{"HP91 3CP"}="1HP9 3CP"; $resn_exch{"HP92 3CP"}="2HP9 3CP"; $resn_exch{"HP93 3CP"}="3HP9 3CP";
$resn_exch{"HP81 3CP"}="1HP8 3CP"; $resn_exch{"HP82 3CP"}="2HP8 3CP"; $resn_exch{"HP83 3CP"}="3HP8 3CP";
$resn_exch{"HOP3 3CP"}="3HOP 3CP"; $resn_exch{"HP51 3CP"}="1HP5 3CP"; $resn_exch{"HP52 3CP"}="2HP5 3CP";
$resn_exch{"HP41 3CP"}="1HP4 3CP"; $resn_exch{"HP42 3CP"}="2HP4 3CP"; $resn_exch{"HP21 3CP"}="1HP2 3CP";
$resn_exch{"HP22 3CP"}="2HP2 3CP"; $resn_exch{"HP11 3CP"}="1HP1 3CP"; $resn_exch{"HP12 3CP"}="2HP1 3CP";
$resn_exch{"HS11 3CP"}="1HS1 3CP"; $resn_exch{"HS12 3CP"}="2HS1 3CP"; $resn_exch{"HS21 3CP"}="1HS2 3CP";
$resn_exch{"HS22 3CP"}="2HS2 3CP"; $resn_exch{"HS31 3CP"}="1HS3 3CP"; $resn_exch{"HS32 3CP"}="2HS3 3CP";
$resn_exch{"HOS5 3CP"}="5HOS 3CP"; $resn_exch{"H101 3CS"}="1H10 3CS"; $resn_exch{"H102 3CS"}="2H10 3CS";
$resn_exch{"H301 3CS"}="1H30 3CS"; $resn_exch{"H302 3CS"}="2H30 3CS"; $resn_exch{"H311 3CS"}="1H31 3CS";
$resn_exch{"H312 3CS"}="2H31 3CS"; $resn_exch{"H313 3CS"}="3H31 3CS"; $resn_exch{"H321 3CS"}="1H32 3CS";
$resn_exch{"H322 3CS"}="2H32 3CS"; $resn_exch{"H323 3CS"}="3H32 3CS"; $resn_exch{"H341 3CS"}="1H34 3CS";
$resn_exch{"H342 3CS"}="2H34 3CS"; $resn_exch{"H343 3CS"}="3H34 3CS"; $resn_exch{"H181 3CS"}="1H18 3CS";
$resn_exch{"H182 3CS"}="2H18 3CS"; $resn_exch{"H371 3CS"}="1H37 3CS"; $resn_exch{"H372 3CS"}="2H37 3CS";
$resn_exch{"H381 3CS"}="1H38 3CS"; $resn_exch{"H382 3CS"}="2H38 3CS"; $resn_exch{"H383 3CS"}="3H38 3CS";
$resn_exch{"H391 3CS"}="1H39 3CS"; $resn_exch{"H392 3CS"}="2H39 3CS"; $resn_exch{"H393 3CS"}="3H39 3CS";
$resn_exch{" H71 3CY"}="1H7  3CY"; $resn_exch{" H72 3CY"}="2H7  3CY"; $resn_exch{" H81 3CY"}="1H8  3CY";
$resn_exch{" H82 3CY"}="2H8  3CY"; $resn_exch{"H111 3CY"}="1H11 3CY"; $resn_exch{"H112 3CY"}="2H11 3CY";
$resn_exch{"H151 3CY"}="1H15 3CY"; $resn_exch{"H152 3CY"}="2H15 3CY"; $resn_exch{"H221 3CY"}="1H22 3CY";
$resn_exch{"H222 3CY"}="2H22 3CY"; $resn_exch{"H231 3CY"}="1H23 3CY"; $resn_exch{"H232 3CY"}="2H23 3CY";
$resn_exch{"H311 3CY"}="1H31 3CY"; $resn_exch{"H312 3CY"}="2H31 3CY"; $resn_exch{"H313 3CY"}="3H31 3CY";
$resn_exch{"H281 3CY"}="1H28 3CY"; $resn_exch{"H282 3CY"}="2H28 3CY"; $resn_exch{"H131 3D3"}="1H13 3D3";
$resn_exch{"H132 3D3"}="2H13 3D3"; $resn_exch{"H141 3D3"}="1H14 3D3"; $resn_exch{"H142 3D3"}="2H14 3D3";
$resn_exch{"H151 3D3"}="1H15 3D3"; $resn_exch{"H152 3D3"}="2H15 3D3"; $resn_exch{"HOP2 3DA"}="2HOP 3DA";
$resn_exch{"H5'1 3DA"}="1H5* 3DA"; $resn_exch{"H5'2 3DA"}="2H5* 3DA"; $resn_exch{" H4' 3DA"}=" H4* 3DA";
$resn_exch{" H1' 3DA"}=" H1* 3DA"; $resn_exch{"HN61 3DA"}="1HN6 3DA"; $resn_exch{"HN62 3DA"}="2HN6 3DA";
$resn_exch{"H2'1 3DA"}="1H2* 3DA"; $resn_exch{"HO'2 3DA"}="2HO* 3DA"; $resn_exch{"H3'1 3DA"}="1H3* 3DA";
$resn_exch{"H3'2 3DA"}="2H3* 3DA"; $resn_exch{"HOP3 3DA"}="3HOP 3DA"; $resn_exch{"HN61 3DD"}="1HN6 3DD";
$resn_exch{"HN62 3DD"}="2HN6 3DD"; $resn_exch{"H1'1 3DD"}="1H1' 3DD"; $resn_exch{"H1'2 3DD"}="2H1' 3DD";
$resn_exch{"H161 3DE"}="1H16 3DE"; $resn_exch{"H162 3DE"}="2H16 3DE"; $resn_exch{"H171 3DE"}="1H17 3DE";
$resn_exch{"H172 3DE"}="2H17 3DE"; $resn_exch{"H173 3DE"}="3H17 3DE"; $resn_exch{"H121 3DE"}="1H12 3DE";
$resn_exch{"H122 3DE"}="2H12 3DE"; $resn_exch{"H123 3DE"}="3H12 3DE"; $resn_exch{" H91 3DE"}="1H9  3DE";
$resn_exch{" H92 3DE"}="2H9  3DE"; $resn_exch{" H93 3DE"}="3H9  3DE"; $resn_exch{"H3'1 3DG"}="1H3' 3DG";
$resn_exch{"H3'2 3DG"}="2H3' 3DG"; $resn_exch{"H5'1 3DG"}="1H5' 3DG"; $resn_exch{"H5'2 3DG"}="2H5' 3DG";
$resn_exch{"HN21 3DG"}="1HN2 3DG"; $resn_exch{"HN22 3DG"}="2HN2 3DG"; $resn_exch{"HAA1 3DP"}="1HAA 3DP";
$resn_exch{"HAA2 3DP"}="2HAA 3DP"; $resn_exch{"HAA3 3DP"}="3HAA 3DP"; $resn_exch{"HAG1 3DP"}="1HAG 3DP";
$resn_exch{"HAG2 3DP"}="2HAG 3DP"; $resn_exch{"HAH1 3DP"}="1HAH 3DP"; $resn_exch{"HAH2 3DP"}="2HAH 3DP";
$resn_exch{"HAI1 3DP"}="1HAI 3DP"; $resn_exch{"HAI2 3DP"}="2HAI 3DP"; $resn_exch{"HAJ1 3DP"}="1HAJ 3DP";
$resn_exch{"HAJ2 3DP"}="2HAJ 3DP"; $resn_exch{"HAK1 3DP"}="1HAK 3DP"; $resn_exch{"HAK2 3DP"}="2HAK 3DP";
$resn_exch{"HAL1 3DP"}="1HAL 3DP"; $resn_exch{"HAL2 3DP"}="2HAL 3DP"; $resn_exch{"HAN1 3DP"}="1HAN 3DP";
$resn_exch{"HAN2 3DP"}="2HAN 3DP"; $resn_exch{"HAB1 3DP"}="1HAB 3DP"; $resn_exch{"HAB2 3DP"}="2HAB 3DP";
$resn_exch{"HAB3 3DP"}="3HAB 3DP"; $resn_exch{"HAC1 3DP"}="1HAC 3DP"; $resn_exch{"HAC2 3DP"}="2HAC 3DP";
$resn_exch{"HAC3 3DP"}="3HAC 3DP"; $resn_exch{"HAO1 3DP"}="1HAO 3DP"; $resn_exch{"HAO2 3DP"}="2HAO 3DP";
$resn_exch{"HAM1 3DP"}="1HAM 3DP"; $resn_exch{"HAM2 3DP"}="2HAM 3DP"; $resn_exch{"HAP1 3DP"}="1HAP 3DP";
$resn_exch{"HAP2 3DP"}="2HAP 3DP"; $resn_exch{"HOP2 3DR"}="2HOP 3DR"; $resn_exch{"HOP3 3DR"}="3HOP 3DR";
$resn_exch{" H2' 3DR"}="1H2* 3DR"; $resn_exch{"H2'' 3DR"}="2H2* 3DR"; $resn_exch{" H5' 3DR"}="1H5* 3DR";
$resn_exch{"H5'' 3DR"}="2H5* 3DR"; $resn_exch{"H4'1 3DR"}="1H4* 3DR"; $resn_exch{"H1'1 3DR"}="1H1* 3DR";
$resn_exch{"H1'2 3DR"}="2H1* 3DR"; $resn_exch{" H3' 3DR"}=" H3* 3DR"; $resn_exch{"HO3' 3DR"}="HO3* 3DR";
$resn_exch{"H131 3EA"}="1H13 3EA"; $resn_exch{"H132 3EA"}="2H13 3EA"; $resn_exch{"H133 3EA"}="3H13 3EA";
$resn_exch{"H121 3EA"}="1H12 3EA"; $resn_exch{"H122 3EA"}="2H12 3EA"; $resn_exch{"H111 3EA"}="1H11 3EA";
$resn_exch{"H112 3EA"}="2H11 3EA"; $resn_exch{"H151 3EA"}="1H15 3EA"; $resn_exch{"H152 3EA"}="2H15 3EA";
$resn_exch{"H161 3EA"}="1H16 3EA"; $resn_exch{"H162 3EA"}="2H16 3EA"; $resn_exch{"H171 3EA"}="1H17 3EA";
$resn_exch{"H172 3EA"}="2H17 3EA"; $resn_exch{"H281 3EA"}="1H28 3EA"; $resn_exch{"H282 3EA"}="2H28 3EA";
$resn_exch{" H12 3EP"}="2H1  3EP"; $resn_exch{" H11 3EP"}="1H1  3EP"; $resn_exch{" H23 3EP"}="3H2  3EP";
$resn_exch{" H22 3EP"}="2H2  3EP"; $resn_exch{" H21 3EP"}="1H2  3EP"; $resn_exch{" H32 3EP"}="2H3  3EP";
$resn_exch{" H31 3EP"}="1H3  3EP"; $resn_exch{" H43 3EP"}="3H4  3EP"; $resn_exch{" H42 3EP"}="2H4  3EP";
$resn_exch{" H41 3EP"}="1H4  3EP"; $resn_exch{" H52 3EP"}="2H5  3EP"; $resn_exch{" H51 3EP"}="1H5  3EP";
$resn_exch{" H63 3EP"}="3H6  3EP"; $resn_exch{" H62 3EP"}="2H6  3EP"; $resn_exch{" H61 3EP"}="1H6  3EP";
$resn_exch{" H11 3FC"}="1H1  3FC"; $resn_exch{" H12 3FC"}="2H1  3FC"; $resn_exch{" H13 3FC"}="3H1  3FC";
$resn_exch{"H21A 3FC"}="1H2  3FC"; $resn_exch{"H22A 3FC"}="2H2  3FC"; $resn_exch{" H31 3FC"}="1H3  3FC";
$resn_exch{" H32 3FC"}="2H3  3FC"; $resn_exch{" H41 3FC"}="1H4  3FC"; $resn_exch{" H42 3FC"}="2H4  3FC";
$resn_exch{"H111 3FC"}="1H11 3FC"; $resn_exch{"H112 3FC"}="2H11 3FC"; $resn_exch{"H281 3FC"}="1H28 3FC";
$resn_exch{"H282 3FC"}="2H28 3FC"; $resn_exch{"H283 3FC"}="3H28 3FC"; $resn_exch{"H291 3FC"}="1H29 3FC";
$resn_exch{"H292 3FC"}="2H29 3FC"; $resn_exch{"H293 3FC"}="3H29 3FC"; $resn_exch{"H301 3FC"}="1H30 3FC";
$resn_exch{"H302 3FC"}="2H30 3FC"; $resn_exch{"H303 3FC"}="3H30 3FC"; $resn_exch{"H371 3FC"}="1H37 3FC";
$resn_exch{"H372 3FC"}="2H37 3FC"; $resn_exch{"H381 3FC"}="1H38 3FC"; $resn_exch{"H382 3FC"}="2H38 3FC";
$resn_exch{"HNC1 3FM"}="1HNC 3FM"; $resn_exch{"HNC2 3FM"}="2HNC 3FM"; $resn_exch{" H61 3FM"}="1H6  3FM";
$resn_exch{" H62 3FM"}="2H6  3FM"; $resn_exch{"H161 3FP"}="1H16 3FP"; $resn_exch{"H162 3FP"}="2H16 3FP";
$resn_exch{"H181 3FP"}="1H18 3FP"; $resn_exch{"H182 3FP"}="2H18 3FP"; $resn_exch{"H191 3FP"}="1H19 3FP";
$resn_exch{"H192 3FP"}="2H19 3FP"; $resn_exch{"H193 3FP"}="3H19 3FP"; $resn_exch{"H201 3FP"}="1H20 3FP";
$resn_exch{"H202 3FP"}="2H20 3FP"; $resn_exch{"H203 3FP"}="3H20 3FP"; $resn_exch{"H161 3FQ"}="1H16 3FQ";
$resn_exch{"H162 3FQ"}="2H16 3FQ"; $resn_exch{"H181 3FQ"}="1H18 3FQ"; $resn_exch{"H182 3FQ"}="2H18 3FQ";
$resn_exch{"H191 3FQ"}="1H19 3FQ"; $resn_exch{"H192 3FQ"}="2H19 3FQ"; $resn_exch{"H193 3FQ"}="3H19 3FQ";
$resn_exch{"H201 3FQ"}="1H20 3FQ"; $resn_exch{"H202 3FQ"}="2H20 3FQ"; $resn_exch{"H203 3FQ"}="3H20 3FQ";
$resn_exch{"H241 3FT"}="1H24 3FT"; $resn_exch{"H242 3FT"}="2H24 3FT"; $resn_exch{"H251 3FT"}="1H25 3FT";
$resn_exch{"H252 3FT"}="2H25 3FT"; $resn_exch{"H261 3FT"}="1H26 3FT"; $resn_exch{"H262 3FT"}="2H26 3FT";
$resn_exch{" H21 3GA"}="1H2  3GA"; $resn_exch{" H22 3GA"}="2H2  3GA"; $resn_exch{" H31 3GA"}="1H3  3GA";
$resn_exch{" H32 3GA"}="2H3  3GA"; $resn_exch{"HN1A 3GA"}="1HN  3GA"; $resn_exch{" HN2 3GA"}="2HN  3GA";
$resn_exch{" HB1 3GA"}="1HB  3GA"; $resn_exch{" HB2 3GA"}="2HB  3GA"; $resn_exch{" H11 3GA"}="1H1  3GA";
$resn_exch{" H12 3GA"}="2H1  3GA"; $resn_exch{" HD1 3GA"}="1HD  3GA"; $resn_exch{" HD2 3GA"}="2HD  3GA";
$resn_exch{"HN21 3GA"}="1HN2 3GA"; $resn_exch{"HN22 3GA"}="2HN2 3GA"; $resn_exch{" H11 3GC"}="1H1  3GC";
$resn_exch{" H22 3GC"}="2H2  3GC"; $resn_exch{"HB11 3GC"}="1HB1 3GC"; $resn_exch{"HB12 3GC"}="2HB1 3GC";
$resn_exch{"HG11 3GC"}="1HG1 3GC"; $resn_exch{"HG12 3GC"}="2HG1 3GC"; $resn_exch{"HB21 3GC"}="1HB2 3GC";
$resn_exch{"HB22 3GC"}="2HB2 3GC"; $resn_exch{"HOP2 3GP"}="2HOP 3GP"; $resn_exch{"HOP3 3GP"}="3HOP 3GP";
$resn_exch{"HO5' 3GP"}="*HO5 3GP"; $resn_exch{"H5'1 3GP"}="1H5* 3GP"; $resn_exch{"H5'2 3GP"}="2H5* 3GP";
$resn_exch{" H4' 3GP"}=" H4* 3GP"; $resn_exch{" H3' 3GP"}=" H3* 3GP"; $resn_exch{" H2' 3GP"}=" H2* 3GP";
$resn_exch{"HO2' 3GP"}="*HO2 3GP"; $resn_exch{" H1' 3GP"}=" H1* 3GP"; $resn_exch{"HN21 3GP"}="1HN2 3GP";
$resn_exch{"HN22 3GP"}="2HN2 3GP"; $resn_exch{"H1C1 3GR"}="1H1C 3GR"; $resn_exch{"H1C2 3GR"}="2H1C 3GR";
$resn_exch{"H101 3HA"}="1H10 3HA"; $resn_exch{"H102 3HA"}="2H10 3HA"; $resn_exch{"HO2' 3HB"}="'HO2 3HB";
$resn_exch{" H2A 3HC"}=" AH2 3HC"; $resn_exch{"H61A 3HC"}="AH61 3HC"; $resn_exch{"H62A 3HC"}="AH62 3HC";
$resn_exch{" H8A 3HC"}=" AH8 3HC"; $resn_exch{" H1B 3HC"}="AH1* 3HC"; $resn_exch{" H2B 3HC"}="AH2* 3HC";
$resn_exch{"HO2A 3HC"}="AHO2 3HC"; $resn_exch{" H3B 3HC"}="AH3* 3HC"; $resn_exch{"HOA8 3HC"}="8HOA 3HC";
$resn_exch{"HOA9 3HC"}="9HOA 3HC"; $resn_exch{" H4B 3HC"}="AH4* 3HC"; $resn_exch{"H51A 3HC"}="AH51 3HC";
$resn_exch{"H52A 3HC"}="AH52 3HC"; $resn_exch{"HOA2 3HC"}="2HOA 3HC"; $resn_exch{"HOA5 3HC"}="5HOA 3HC";
$resn_exch{"H121 3HC"}="1H12 3HC"; $resn_exch{"H122 3HC"}="2H12 3HC"; $resn_exch{"H131 3HC"}="1H13 3HC";
$resn_exch{"H132 3HC"}="2H13 3HC"; $resn_exch{"H133 3HC"}="3H13 3HC"; $resn_exch{"H141 3HC"}="1H14 3HC";
$resn_exch{"H142 3HC"}="2H14 3HC"; $resn_exch{"H143 3HC"}="3H14 3HC"; $resn_exch{" H71 3HC"}="1H7  3HC";
$resn_exch{" H72 3HC"}="2H7  3HC"; $resn_exch{" H61 3HC"}="1H6  3HC"; $resn_exch{" H62 3HC"}="2H6  3HC";
$resn_exch{" H31 3HC"}="1H3  3HC"; $resn_exch{" H32 3HC"}="2H3  3HC"; $resn_exch{" H21 3HC"}="1H2  3HC";
$resn_exch{" H22 3HC"}="2H2  3HC"; $resn_exch{"HC21 3HC"}="1HC2 3HC"; $resn_exch{"HC22 3HC"}="2HC2 3HC";
$resn_exch{"HC31 3HC"}="1HC3 3HC"; $resn_exch{"HO31 3HC"}="1HO3 3HC"; $resn_exch{"HC41 3HC"}="1HC4 3HC";
$resn_exch{"HC42 3HC"}="2HC4 3HC"; $resn_exch{"HC43 3HC"}="3HC4 3HC"; $resn_exch{" H1  3HD"}="1H   3HD";
$resn_exch{" H2  3HD"}="2H   3HD"; $resn_exch{" HO2 3HD"}="2HO  3HD"; $resn_exch{" H3  3HD"}="3H   3HD";
$resn_exch{" H31 3HD"}="1H3  3HD"; $resn_exch{" H32 3HD"}="2H3  3HD"; $resn_exch{" H33 3HD"}="3H3  3HD";
$resn_exch{" H4  3HD"}="4H   3HD"; $resn_exch{" HO4 3HD"}="4HO  3HD"; $resn_exch{" H5  3HD"}="5H   3HD";
$resn_exch{" H61 3HD"}="1H6  3HD"; $resn_exch{" H62 3HD"}="2H6  3HD"; $resn_exch{" HO6 3HD"}="6HO  3HD";
$resn_exch{" H21 3HG"}="1H2  3HG"; $resn_exch{" H22 3HG"}="2H2  3HG"; $resn_exch{" H41 3HG"}="1H4  3HG";
$resn_exch{" H42 3HG"}="2H4  3HG"; $resn_exch{" HA1 3HL"}="1HA  3HL"; $resn_exch{" HA2 3HL"}="2HA  3HL";
$resn_exch{"HG21 3HL"}="1HG2 3HL"; $resn_exch{"HG22 3HL"}="2HG2 3HL"; $resn_exch{"HG23 3HL"}="3HG2 3HL";
$resn_exch{" H71 3HP"}="1H7  3HP"; $resn_exch{" H72 3HP"}="2H7  3HP"; $resn_exch{" HN1 3HX"}="1HN  3HX";
$resn_exch{" HN2 3HX"}="2HN  3HX"; $resn_exch{" H1  3HX"}="1H   3HX"; $resn_exch{" H2  3HX"}="2H   3HX";
$resn_exch{" HB1 3HX"}="1HB  3HX"; $resn_exch{" HB2 3HX"}="2HB  3HX"; $resn_exch{" HG1 3HX"}="1HG  3HX";
$resn_exch{" HG2 3HX"}="2HG  3HX"; $resn_exch{" HD1 3HX"}="1HD  3HX"; $resn_exch{" HD2 3HX"}="2HD  3HX";
$resn_exch{"HNB1 3HX"}="1HNB 3HX"; $resn_exch{"HNB2 3HX"}="2HNB 3HX"; $resn_exch{"H1'1 3HX"}="1H1' 3HX";
$resn_exch{"H1'2 3HX"}="2H1' 3HX"; $resn_exch{"H2'1 3HX"}="1H2' 3HX"; $resn_exch{"H2'2 3HX"}="2H2' 3HX";
$resn_exch{"H101 3IB"}="1H10 3IB"; $resn_exch{"H102 3IB"}="2H10 3IB"; $resn_exch{"H111 3IB"}="1H11 3IB";
$resn_exch{"H112 3IB"}="2H11 3IB"; $resn_exch{"H121 3IB"}="1H12 3IB"; $resn_exch{"H122 3IB"}="2H12 3IB";
$resn_exch{" HN1 3ID"}="1HN  3ID"; $resn_exch{" H1O 3ID"}="1OH  3ID"; $resn_exch{" H2O 3ID"}="2OH  3ID";
$resn_exch{"HN31 3IG"}="1HN3 3IG"; $resn_exch{"HN32 3IG"}="2HN3 3IG"; $resn_exch{"HN41 3IG"}="1HN4 3IG";
$resn_exch{"HN42 3IG"}="2HN4 3IG"; $resn_exch{" H51 3IG"}="1H5  3IG"; $resn_exch{" H52 3IG"}="2H5  3IG";
$resn_exch{" H61 3IG"}="1H6  3IG"; $resn_exch{" H62 3IG"}="2H6  3IG"; $resn_exch{" H63 3IG"}="3H6  3IG";
$resn_exch{"H161 3IG"}="1H16 3IG"; $resn_exch{"H162 3IG"}="2H16 3IG"; $resn_exch{"H171 3IG"}="1H17 3IG";
$resn_exch{"H172 3IG"}="2H17 3IG"; $resn_exch{"H191 3IG"}="1H19 3IG"; $resn_exch{"H192 3IG"}="2H19 3IG";
$resn_exch{"H193 3IG"}="3H19 3IG"; $resn_exch{"H211 3IG"}="1H21 3IG"; $resn_exch{"H212 3IG"}="2H21 3IG";
$resn_exch{"H213 3IG"}="3H21 3IG"; $resn_exch{"H201 3IG"}="1H20 3IG"; $resn_exch{"H202 3IG"}="2H20 3IG";
$resn_exch{"H203 3IG"}="3H20 3IG"; $resn_exch{" H61 3IH"}="1H6  3IH"; $resn_exch{" H62 3IH"}="2H6  3IH";
$resn_exch{" H63 3IH"}="3H6  3IH"; $resn_exch{"H101 3IH"}="1H10 3IH"; $resn_exch{"H102 3IH"}="2H10 3IH";
$resn_exch{"H103 3IH"}="3H10 3IH"; $resn_exch{"H111 3IH"}="1H11 3IH"; $resn_exch{"H112 3IH"}="2H11 3IH";
$resn_exch{"H113 3IH"}="3H11 3IH"; $resn_exch{"H141 3IH"}="1H14 3IH"; $resn_exch{"H142 3IH"}="2H14 3IH";
$resn_exch{"H161 3IH"}="1H16 3IH"; $resn_exch{"H162 3IH"}="2H16 3IH"; $resn_exch{"H163 3IH"}="3H16 3IH";
$resn_exch{"H171 3IH"}="1H17 3IH"; $resn_exch{"H172 3IH"}="2H17 3IH"; $resn_exch{"H173 3IH"}="3H17 3IH";
$resn_exch{"H231 3IH"}="1H23 3IH"; $resn_exch{"H232 3IH"}="2H23 3IH"; $resn_exch{"H251 3IH"}="1H25 3IH";
$resn_exch{"H252 3IH"}="2H25 3IH"; $resn_exch{"H271 3IH"}="1H27 3IH"; $resn_exch{"H272 3IH"}="2H27 3IH";
$resn_exch{"H281 3IH"}="1H28 3IH"; $resn_exch{"H282 3IH"}="2H28 3IH"; $resn_exch{"H351 3IH"}="1H35 3IH";
$resn_exch{"H352 3IH"}="2H35 3IH"; $resn_exch{"H353 3IH"}="3H35 3IH"; $resn_exch{"H151 3IL"}="1H15 3IL";
$resn_exch{"H152 3IL"}="2H15 3IL"; $resn_exch{"H11A 3IN"}="1H1  3IN"; $resn_exch{" H12 3IN"}="2H1  3IN";
$resn_exch{" H71 3IN"}="1H7  3IN"; $resn_exch{" H72 3IN"}="2H7  3IN"; $resn_exch{" H73 3IN"}="3H7  3IN";
$resn_exch{" H81 3IN"}="1H8  3IN"; $resn_exch{" H82 3IN"}="2H8  3IN"; $resn_exch{" H91 3IN"}="1H9  3IN";
$resn_exch{" H92 3IN"}="2H9  3IN"; $resn_exch{"H101 3IN"}="1H10 3IN"; $resn_exch{"H102 3IN"}="2H10 3IN";
$resn_exch{"H121 3IN"}="1H12 3IN"; $resn_exch{"H122 3IN"}="2H12 3IN"; $resn_exch{"H141 3IN"}="1H14 3IN";
$resn_exch{"H142 3IN"}="2H14 3IN"; $resn_exch{"H241 3IN"}="1H24 3IN"; $resn_exch{"H242 3IN"}="2H24 3IN";
$resn_exch{"H271 3IN"}="1H27 3IN"; $resn_exch{"H272 3IN"}="2H27 3IN"; $resn_exch{"H351 3IN"}="1H35 3IN";
$resn_exch{"H352 3IN"}="2H35 3IN"; $resn_exch{" H51 3IN"}="1H5  3IN"; $resn_exch{" H52 3IN"}="2H5  3IN";
$resn_exch{" H53 3IN"}="3H5  3IN"; $resn_exch{" H61 3IN"}="1H6  3IN"; $resn_exch{" H62 3IN"}="2H6  3IN";
$resn_exch{" H63 3IN"}="3H6  3IN"; $resn_exch{"H251 3IN"}="1H25 3IN"; $resn_exch{"H252 3IN"}="2H25 3IN";
$resn_exch{"H291 3IN"}="1H29 3IN"; $resn_exch{"H292 3IN"}="2H29 3IN"; $resn_exch{"H293 3IN"}="3H29 3IN";
$resn_exch{"H261 3IN"}="1H26 3IN"; $resn_exch{"H262 3IN"}="2H26 3IN"; $resn_exch{"H371 3IN"}="1H37 3IN";
$resn_exch{"H372 3IN"}="2H37 3IN"; $resn_exch{"H281 3IN"}="1H28 3IN"; $resn_exch{"H282 3IN"}="2H28 3IN";
$resn_exch{"H283 3IN"}="3H28 3IN"; $resn_exch{"HAI1 3IO"}="1HAI 3IO"; $resn_exch{"HAI2 3IO"}="2HAI 3IO";
$resn_exch{"H1N1 3IP"}="1H1N 3IP"; $resn_exch{"H1N2 3IP"}="2H1N 3IP"; $resn_exch{"H9C1 3IP"}="1H9C 3IP";
$resn_exch{"H9C2 3IP"}="2H9C 3IP"; $resn_exch{"H11A 3LG"}="1H1  3LG"; $resn_exch{"H12A 3LG"}="2H1  3LG";
$resn_exch{"H13A 3LG"}="3H1  3LG"; $resn_exch{" H41 3LG"}="1H4  3LG"; $resn_exch{" H42 3LG"}="2H4  3LG";
$resn_exch{"H211 3LG"}="1H21 3LG"; $resn_exch{"H212 3LG"}="2H21 3LG"; $resn_exch{"H221 3LG"}="1H22 3LG";
$resn_exch{"H222 3LG"}="2H22 3LG"; $resn_exch{"H241 3LG"}="1H24 3LG"; $resn_exch{"H242 3LG"}="2H24 3LG";
$resn_exch{"H311 3LG"}="1H31 3LG"; $resn_exch{"H312 3LG"}="2H31 3LG"; $resn_exch{"H321 3LG"}="1H32 3LG";
$resn_exch{"H322 3LG"}="2H32 3LG"; $resn_exch{"H331 3LG"}="1H33 3LG"; $resn_exch{"H332 3LG"}="2H33 3LG";
$resn_exch{"H171 3LG"}="1H17 3LG"; $resn_exch{"H172 3LG"}="2H17 3LG"; $resn_exch{"HAA1 3LP"}="1HAA 3LP";
$resn_exch{"HAA2 3LP"}="2HAA 3LP"; $resn_exch{"HAA3 3LP"}="3HAA 3LP"; $resn_exch{"HAK1 3LP"}="1HAK 3LP";
$resn_exch{"HAK2 3LP"}="2HAK 3LP"; $resn_exch{"HAG1 3LP"}="1HAG 3LP"; $resn_exch{"HAG2 3LP"}="2HAG 3LP";
$resn_exch{"HAH1 3LP"}="1HAH 3LP"; $resn_exch{"HAH2 3LP"}="2HAH 3LP"; $resn_exch{"HAL1 3LP"}="1HAL 3LP";
$resn_exch{"HAL2 3LP"}="2HAL 3LP"; $resn_exch{"HAP1 3LP"}="1HAP 3LP"; $resn_exch{"HAP2 3LP"}="2HAP 3LP";
$resn_exch{"HAO1 3LP"}="1HAO 3LP"; $resn_exch{"HAO2 3LP"}="2HAO 3LP"; $resn_exch{"HAM1 3LP"}="1HAM 3LP";
$resn_exch{"HAM2 3LP"}="2HAM 3LP"; $resn_exch{"HAI1 3LP"}="1HAI 3LP"; $resn_exch{"HAI2 3LP"}="2HAI 3LP";
$resn_exch{"HAF1 3LP"}="1HAF 3LP"; $resn_exch{"HAF2 3LP"}="2HAF 3LP"; $resn_exch{"HAJ1 3LP"}="1HAJ 3LP";
$resn_exch{"HAJ2 3LP"}="2HAJ 3LP"; $resn_exch{"HAN1 3LP"}="1HAN 3LP"; $resn_exch{"HAN2 3LP"}="2HAN 3LP";
$resn_exch{" H31 3MA"}="1H3  3MA"; $resn_exch{" H32 3MA"}="2H3  3MA"; $resn_exch{" H33 3MA"}="3H3  3MA";
$resn_exch{"HN61 3MA"}="1HN6 3MA"; $resn_exch{"HN62 3MA"}="2HN6 3MA"; $resn_exch{" H71 3MB"}="1H7  3MB";
$resn_exch{" H72 3MB"}="2H7  3MB"; $resn_exch{" H73 3MB"}="3H7  3MB"; $resn_exch{" HN1 3MB"}="1HN  3MB";
$resn_exch{" HN2 3MB"}="2HN  3MB"; $resn_exch{" H31 3MC"}="1H3  3MC"; $resn_exch{" H32 3MC"}="2H3  3MC";
$resn_exch{" H33 3MC"}="3H3  3MC"; $resn_exch{"HN41 3MC"}="1HN4 3MC"; $resn_exch{"HN42 3MC"}="2HN4 3MC";
$resn_exch{" H2  3MD"}=" HN  3MD"; $resn_exch{"HB11 3MD"}="1HB1 3MD"; $resn_exch{"HB12 3MD"}="2HB1 3MD";
$resn_exch{"HB13 3MD"}="3HB1 3MD"; $resn_exch{"H5'1 3ME"}="1H5' 3ME"; $resn_exch{"H5'2 3ME"}="2H5' 3ME";
$resn_exch{"H5A1 3ME"}="1H5A 3ME"; $resn_exch{"H5A2 3ME"}="2H5A 3ME"; $resn_exch{"H5A3 3ME"}="3H5A 3ME";
$resn_exch{"HB'1 3ME"}="1HB' 3ME"; $resn_exch{"HB'2 3ME"}="2HB' 3ME"; $resn_exch{"HC'1 3ME"}="1HC' 3ME";
$resn_exch{"HC'2 3ME"}="2HC' 3ME"; $resn_exch{"HE'1 3ME"}="1HE' 3ME"; $resn_exch{"HE'2 3ME"}="2HE' 3ME";
$resn_exch{"HF'1 3ME"}="1HF' 3ME"; $resn_exch{"HF'2 3ME"}="2HF' 3ME"; $resn_exch{"HI'1 3ME"}="1HI' 3ME";
$resn_exch{"HI'2 3ME"}="2HI' 3ME"; $resn_exch{"HI'3 3ME"}="3HI' 3ME"; $resn_exch{"HJ'1 3ME"}="1HJ' 3ME";
$resn_exch{"HJ'2 3ME"}="2HJ' 3ME"; $resn_exch{"HJ'3 3ME"}="3HJ' 3ME"; $resn_exch{" H11 3MF"}="1H1  3MF";
$resn_exch{" H12 3MF"}="2H1  3MF"; $resn_exch{" H61 3MF"}="1H6  3MF"; $resn_exch{" H62 3MF"}="2H6  3MF";
$resn_exch{" H71 3MF"}="1H7  3MF"; $resn_exch{" H72 3MF"}="2H7  3MF"; $resn_exch{" H73 3MF"}="3H7  3MF";
$resn_exch{"H21A 3MN"}="1H2  3MN"; $resn_exch{" H22 3MN"}="2H2  3MN"; $resn_exch{" H11 3MN"}="1H1  3MN";
$resn_exch{"H12A 3MN"}="2H1  3MN"; $resn_exch{"H161 3MN"}="1H16 3MN"; $resn_exch{"H162 3MN"}="2H16 3MN";
$resn_exch{"H111 3MN"}="1H11 3MN"; $resn_exch{"H112 3MN"}="2H11 3MN"; $resn_exch{"H281 3MN"}="1H28 3MN";
$resn_exch{"H282 3MN"}="2H28 3MN"; $resn_exch{"H283 3MN"}="3H28 3MN"; $resn_exch{" H41 3MO"}="1H4  3MO";
$resn_exch{" H42 3MO"}="2H4  3MO"; $resn_exch{" H31 3MO"}="1H3  3MO"; $resn_exch{" H32 3MO"}="2H3  3MO";
$resn_exch{" H71 3MO"}="1H7  3MO"; $resn_exch{" H72 3MO"}="2H7  3MO"; $resn_exch{" H73 3MO"}="3H7  3MO";
$resn_exch{" HB1 3MP"}="1HB  3MP"; $resn_exch{" HB2 3MP"}="2HB  3MP"; $resn_exch{" HB3 3MP"}="3HB  3MP";
$resn_exch{" H11 3MR"}="1H1  3MR"; $resn_exch{" H12 3MR"}="2H1  3MR"; $resn_exch{" H13 3MR"}="3H1  3MR";
$resn_exch{" H51 3MR"}="1H5  3MR"; $resn_exch{" H52 3MR"}="2H5  3MR"; $resn_exch{" H61 3MR"}="1H6  3MR";
$resn_exch{" H62 3MR"}="2H6  3MR"; $resn_exch{"H101 3MR"}="1H10 3MR"; $resn_exch{"H102 3MR"}="2H10 3MR";
$resn_exch{"H111 3MR"}="1H11 3MR"; $resn_exch{"H112 3MR"}="2H11 3MR"; $resn_exch{"H121 3MR"}="1H12 3MR";
$resn_exch{"H122 3MR"}="2H12 3MR"; $resn_exch{"H131 3MR"}="1H13 3MR"; $resn_exch{"H132 3MR"}="2H13 3MR";
$resn_exch{"H141 3MR"}="1H14 3MR"; $resn_exch{"H142 3MR"}="2H14 3MR"; $resn_exch{"H161 3MR"}="1H16 3MR";
$resn_exch{"H162 3MR"}="2H16 3MR"; $resn_exch{"H321 3MR"}="1H32 3MR"; $resn_exch{"H322 3MR"}="2H32 3MR";
$resn_exch{"H341 3MR"}="1H34 3MR"; $resn_exch{"H342 3MR"}="2H34 3MR"; $resn_exch{"H351 3MR"}="1H35 3MR";
$resn_exch{"H352 3MR"}="2H35 3MR"; $resn_exch{"H361 3MR"}="1H36 3MR"; $resn_exch{"H362 3MR"}="2H36 3MR";
$resn_exch{"H371 3MR"}="1H37 3MR"; $resn_exch{"H372 3MR"}="2H37 3MR"; $resn_exch{"H381 3MR"}="1H38 3MR";
$resn_exch{"H382 3MR"}="2H38 3MR"; $resn_exch{"H11A 3MT"}="1H1  3MT"; $resn_exch{" H12 3MT"}="2H1  3MT";
$resn_exch{" H13 3MT"}="3H1  3MT"; $resn_exch{"H171 3NA"}="1H17 3NA"; $resn_exch{"H172 3NA"}="2H17 3NA";
$resn_exch{"H201 3NA"}="1H20 3NA"; $resn_exch{"H202 3NA"}="2H20 3NA"; $resn_exch{"H221 3NH"}="1H22 3NH";
$resn_exch{"H222 3NH"}="2H22 3NH"; $resn_exch{"H301 3NH"}="1H30 3NH"; $resn_exch{"H302 3NH"}="2H30 3NH";
$resn_exch{"H101 3NH"}="1H10 3NH"; $resn_exch{"H102 3NH"}="2H10 3NH"; $resn_exch{"H081 3NH"}="1H08 3NH";
$resn_exch{"H082 3NH"}="2H08 3NH"; $resn_exch{"H451 3NH"}="1H45 3NH"; $resn_exch{"H452 3NH"}="2H45 3NH";
$resn_exch{"H471 3NH"}="1H47 3NH"; $resn_exch{"H472 3NH"}="2H47 3NH"; $resn_exch{"H481 3NH"}="1H48 3NH";
$resn_exch{"H482 3NH"}="2H48 3NH"; $resn_exch{"H361 3NH"}="1H36 3NH"; $resn_exch{"H362 3NH"}="2H36 3NH";
$resn_exch{"HN11 3NH"}="1HN1 3NH"; $resn_exch{"HN12 3NH"}="2HN1 3NH"; $resn_exch{" H21 3NP"}="1H2  3NP";
$resn_exch{" H22 3NP"}="2H2  3NP"; $resn_exch{" H31 3NP"}="1H3  3NP"; $resn_exch{" H32 3NP"}="2H3  3NP";
$resn_exch{"H7C1 3NT"}="1H7C 3NT"; $resn_exch{"H7C2 3NT"}="2H7C 3NT"; $resn_exch{"H7C3 3NT"}="3H7C 3NT";
$resn_exch{"H171 3OD"}="1H17 3OD"; $resn_exch{"H172 3OD"}="2H17 3OD"; $resn_exch{"H173 3OD"}="3H17 3OD";
$resn_exch{"H111 3OD"}="1H11 3OD"; $resn_exch{"H112 3OD"}="2H11 3OD"; $resn_exch{" H11 3OD"}="1H1  3OD";
$resn_exch{"H12A 3OD"}="2H1  3OD"; $resn_exch{"HN31 3OD"}="1HN3 3OD"; $resn_exch{"HN32 3OD"}="2HN3 3OD";
$resn_exch{"HO11 3OF"}="1HO1 3OF"; $resn_exch{"HO12 3OF"}="2HO1 3OF"; $resn_exch{"HO21 3OF"}="1HO2 3OF";
$resn_exch{"HO22 3OF"}="2HO2 3OF"; $resn_exch{" H21 3OH"}="1H2  3OH"; $resn_exch{" H22 3OH"}="2H2  3OH";
$resn_exch{" H31 3OH"}="1H3  3OH"; $resn_exch{" H32 3OH"}="2H3  3OH"; $resn_exch{" H11 3OL"}="1H1  3OL";
$resn_exch{" H12 3OL"}="2H1  3OL"; $resn_exch{" H21 3OL"}="1H2  3OL"; $resn_exch{" H31 3OL"}="1H3  3OL";
$resn_exch{" H41 3OL"}="1H4  3OL"; $resn_exch{" H42 3OL"}="2H4  3OL"; $resn_exch{" H51 3OL"}="1H5  3OL";
$resn_exch{" H52 3OL"}="2H5  3OL"; $resn_exch{" H61 3OL"}="1H6  3OL"; $resn_exch{" H62 3OL"}="2H6  3OL";
$resn_exch{" H71 3OL"}="1H7  3OL"; $resn_exch{" H72 3OL"}="2H7  3OL"; $resn_exch{" H81 3OL"}="1H8  3OL";
$resn_exch{" H82 3OL"}="2H8  3OL"; $resn_exch{" H83 3OL"}="3H8  3OL"; $resn_exch{"H4C1 3ON"}="1H4C 3ON";
$resn_exch{"H4C2 3ON"}="2H4C 3ON"; $resn_exch{"H111 3ON"}="1H11 3ON"; $resn_exch{"H112 3ON"}="2H11 3ON";
$resn_exch{"H113 3ON"}="3H11 3ON"; $resn_exch{"H6C1 3ON"}="1H6C 3ON"; $resn_exch{"H6C2 3ON"}="2H6C 3ON";
$resn_exch{"H9C1 3ON"}="1H9C 3ON"; $resn_exch{"H9C2 3ON"}="2H9C 3ON"; $resn_exch{"H9C3 3ON"}="3H9C 3ON";
$resn_exch{"H8C1 3ON"}="1H8C 3ON"; $resn_exch{"H8C2 3ON"}="2H8C 3ON"; $resn_exch{"H8C3 3ON"}="3H8C 3ON";
$resn_exch{"H141 3ON"}="1H14 3ON"; $resn_exch{"H142 3ON"}="2H14 3ON"; $resn_exch{"H143 3ON"}="3H14 3ON";
$resn_exch{"H191 3ON"}="1H19 3ON"; $resn_exch{"H192 3ON"}="2H19 3ON"; $resn_exch{"H193 3ON"}="3H19 3ON";
$resn_exch{"H261 3ON"}="1H26 3ON"; $resn_exch{"H262 3ON"}="2H26 3ON"; $resn_exch{"H263 3ON"}="3H26 3ON";
$resn_exch{"H301 3ON"}="1H30 3ON"; $resn_exch{"H302 3ON"}="2H30 3ON"; $resn_exch{"H303 3ON"}="3H30 3ON";
$resn_exch{"H311 3ON"}="1H31 3ON"; $resn_exch{"H312 3ON"}="2H31 3ON"; $resn_exch{"H4C1 3P4"}="1H4C 3P4";
$resn_exch{"H4C2 3P4"}="2H4C 3P4"; $resn_exch{"H8C1 3P4"}="1H8C 3P4"; $resn_exch{"H8C2 3P4"}="2H8C 3P4";
$resn_exch{"H8C3 3P4"}="3H8C 3P4"; $resn_exch{"H101 3P4"}="1H10 3P4"; $resn_exch{"H102 3P4"}="2H10 3P4";
$resn_exch{"H111 3P4"}="1H11 3P4"; $resn_exch{"H112 3P4"}="2H11 3P4"; $resn_exch{"H151 3P4"}="1H15 3P4";
$resn_exch{"H152 3P4"}="2H15 3P4"; $resn_exch{"H171 3P4"}="1H17 3P4"; $resn_exch{"H172 3P4"}="2H17 3P4";
$resn_exch{"H181 3P4"}="1H18 3P4"; $resn_exch{"H182 3P4"}="2H18 3P4"; $resn_exch{"H183 3P4"}="3H18 3P4";
$resn_exch{"H191 3P4"}="1H19 3P4"; $resn_exch{"H192 3P4"}="2H19 3P4"; $resn_exch{"H221 3P4"}="1H22 3P4";
$resn_exch{"H222 3P4"}="2H22 3P4"; $resn_exch{"H231 3P4"}="1H23 3P4"; $resn_exch{"H232 3P4"}="2H23 3P4";
$resn_exch{"H151 3PC"}="1H15 3PC"; $resn_exch{"H152 3PC"}="2H15 3PC"; $resn_exch{"H153 3PC"}="3H15 3PC";
$resn_exch{"H141 3PC"}="1H14 3PC"; $resn_exch{"H142 3PC"}="2H14 3PC"; $resn_exch{"H131 3PC"}="1H13 3PC";
$resn_exch{"H132 3PC"}="2H13 3PC"; $resn_exch{"H121 3PC"}="1H12 3PC"; $resn_exch{"H122 3PC"}="2H12 3PC";
$resn_exch{"H111 3PC"}="1H11 3PC"; $resn_exch{"H112 3PC"}="2H11 3PC"; $resn_exch{" H91 3PC"}="1H9  3PC";
$resn_exch{" H92 3PC"}="2H9  3PC"; $resn_exch{"H171 3PC"}="1H17 3PC"; $resn_exch{"H172 3PC"}="2H17 3PC";
$resn_exch{"H181 3PC"}="1H18 3PC"; $resn_exch{"H182 3PC"}="2H18 3PC"; $resn_exch{"H191 3PC"}="1H19 3PC";
$resn_exch{"H192 3PC"}="2H19 3PC"; $resn_exch{"H201 3PC"}="1H20 3PC"; $resn_exch{"H202 3PC"}="2H20 3PC";
$resn_exch{"H211 3PC"}="1H21 3PC"; $resn_exch{"H212 3PC"}="2H21 3PC"; $resn_exch{"H213 3PC"}="3H21 3PC";
$resn_exch{" H71 3PC"}="1H7  3PC"; $resn_exch{" H72 3PC"}="2H7  3PC"; $resn_exch{" H61 3PC"}="1H6  3PC";
$resn_exch{" H62 3PC"}="2H6  3PC"; $resn_exch{" H51 3PC"}="1H5  3PC"; $resn_exch{" H52 3PC"}="2H5  3PC";
$resn_exch{" H41 3PC"}="1H4  3PC"; $resn_exch{" H42 3PC"}="2H4  3PC"; $resn_exch{" H11 3PC"}="1H1  3PC";
$resn_exch{" H12 3PC"}="2H1  3PC"; $resn_exch{" H13 3PC"}="3H1  3PC"; $resn_exch{" H21 3PC"}="1H2  3PC";
$resn_exch{" H22 3PC"}="2H2  3PC"; $resn_exch{" H23 3PC"}="3H2  3PC"; $resn_exch{" H31 3PC"}="1H3  3PC";
$resn_exch{" H32 3PC"}="2H3  3PC"; $resn_exch{" H33 3PC"}="3H3  3PC"; $resn_exch{"H5'1 3PD"}="1H5' 3PD";
$resn_exch{"H5'2 3PD"}="2H5' 3PD"; $resn_exch{"H2'1 3PD"}="1H2' 3PD"; $resn_exch{"H2'2 3PD"}="2H2' 3PD";
$resn_exch{"HN21 3PD"}="1HN2 3PD"; $resn_exch{"HN22 3PD"}="2HN2 3PD"; $resn_exch{" HN1 3PE"}="1HN  3PE";
$resn_exch{" HN2 3PE"}="2HN  3PE"; $resn_exch{" HN3 3PE"}="3HN  3PE"; $resn_exch{"H111 3PE"}="1H11 3PE";
$resn_exch{"H112 3PE"}="2H11 3PE"; $resn_exch{"H121 3PE"}="1H12 3PE"; $resn_exch{"H122 3PE"}="2H12 3PE";
$resn_exch{" H11 3PE"}="1H1  3PE"; $resn_exch{" H12 3PE"}="2H1  3PE"; $resn_exch{" H31 3PE"}="1H3  3PE";
$resn_exch{" H32 3PE"}="2H3  3PE"; $resn_exch{"H321 3PE"}="1H32 3PE"; $resn_exch{"H322 3PE"}="2H32 3PE";
$resn_exch{"H331 3PE"}="1H33 3PE"; $resn_exch{"H332 3PE"}="2H33 3PE"; $resn_exch{"H341 3PE"}="1H34 3PE";
$resn_exch{"H342 3PE"}="2H34 3PE"; $resn_exch{"H351 3PE"}="1H35 3PE"; $resn_exch{"H352 3PE"}="2H35 3PE";
$resn_exch{"H361 3PE"}="1H36 3PE"; $resn_exch{"H362 3PE"}="2H36 3PE"; $resn_exch{"H371 3PE"}="1H37 3PE";
$resn_exch{"H372 3PE"}="2H37 3PE"; $resn_exch{"H381 3PE"}="1H38 3PE"; $resn_exch{"H382 3PE"}="2H38 3PE";
$resn_exch{"H391 3PE"}="1H39 3PE"; $resn_exch{"H392 3PE"}="2H39 3PE"; $resn_exch{"H3A1 3PE"}="1H3A 3PE";
$resn_exch{"H3A2 3PE"}="2H3A 3PE"; $resn_exch{"H3B1 3PE"}="1H3B 3PE"; $resn_exch{"H3B2 3PE"}="2H3B 3PE";
$resn_exch{"H3C1 3PE"}="1H3C 3PE"; $resn_exch{"H3C2 3PE"}="2H3C 3PE"; $resn_exch{"H3D1 3PE"}="1H3D 3PE";
$resn_exch{"H3D2 3PE"}="2H3D 3PE"; $resn_exch{"H3E1 3PE"}="1H3E 3PE"; $resn_exch{"H3E2 3PE"}="2H3E 3PE";
$resn_exch{"H3F1 3PE"}="1H3F 3PE"; $resn_exch{"H3F2 3PE"}="2H3F 3PE"; $resn_exch{"H3G1 3PE"}="1H3G 3PE";
$resn_exch{"H3G2 3PE"}="2H3G 3PE"; $resn_exch{"H3H1 3PE"}="1H3H 3PE"; $resn_exch{"H3H2 3PE"}="2H3H 3PE";
$resn_exch{"H3I1 3PE"}="1H3I 3PE"; $resn_exch{"H3I2 3PE"}="2H3I 3PE"; $resn_exch{"H3I3 3PE"}="3H3I 3PE";
$resn_exch{"H221 3PE"}="1H22 3PE"; $resn_exch{"H222 3PE"}="2H22 3PE"; $resn_exch{"H231 3PE"}="1H23 3PE";
$resn_exch{"H232 3PE"}="2H23 3PE"; $resn_exch{"H241 3PE"}="1H24 3PE"; $resn_exch{"H242 3PE"}="2H24 3PE";
$resn_exch{"H251 3PE"}="1H25 3PE"; $resn_exch{"H252 3PE"}="2H25 3PE"; $resn_exch{"H261 3PE"}="1H26 3PE";
$resn_exch{"H262 3PE"}="2H26 3PE"; $resn_exch{"H271 3PE"}="1H27 3PE"; $resn_exch{"H272 3PE"}="2H27 3PE";
$resn_exch{"H281 3PE"}="1H28 3PE"; $resn_exch{"H282 3PE"}="2H28 3PE"; $resn_exch{"H291 3PE"}="1H29 3PE";
$resn_exch{"H292 3PE"}="2H29 3PE"; $resn_exch{"H2A1 3PE"}="1H2A 3PE"; $resn_exch{"H2A2 3PE"}="2H2A 3PE";
$resn_exch{"H2B1 3PE"}="1H2B 3PE"; $resn_exch{"H2B2 3PE"}="2H2B 3PE"; $resn_exch{"H2C1 3PE"}="1H2C 3PE";
$resn_exch{"H2C2 3PE"}="2H2C 3PE"; $resn_exch{"H2D1 3PE"}="1H2D 3PE"; $resn_exch{"H2D2 3PE"}="2H2D 3PE";
$resn_exch{"H2E1 3PE"}="1H2E 3PE"; $resn_exch{"H2E2 3PE"}="2H2E 3PE"; $resn_exch{"H2F1 3PE"}="1H2F 3PE";
$resn_exch{"H2F2 3PE"}="2H2F 3PE"; $resn_exch{"H2G1 3PE"}="1H2G 3PE"; $resn_exch{"H2G2 3PE"}="2H2G 3PE";
$resn_exch{"H2H1 3PE"}="1H2H 3PE"; $resn_exch{"H2H2 3PE"}="2H2H 3PE"; $resn_exch{"H2I1 3PE"}="1H2I 3PE";
$resn_exch{"H2I2 3PE"}="2H2I 3PE"; $resn_exch{"H2I3 3PE"}="3H2I 3PE"; $resn_exch{" H31 3PG"}="1H3  3PG";
$resn_exch{" H32 3PG"}="2H3  3PG"; $resn_exch{"HOP3 3PG"}="3HOP 3PG"; $resn_exch{"HOP4 3PG"}="4HOP 3PG";
$resn_exch{" H71 3PI"}="1H7  3PI"; $resn_exch{" H72 3PI"}="2H7  3PI"; $resn_exch{" H91 3PI"}="1H9  3PI";
$resn_exch{" H92 3PI"}="2H9  3PI"; $resn_exch{"H111 3PI"}="1H11 3PI"; $resn_exch{"H112 3PI"}="2H11 3PI";
$resn_exch{"H121 3PI"}="1H12 3PI"; $resn_exch{"H122 3PI"}="2H12 3PI"; $resn_exch{"H131 3PI"}="1H13 3PI";
$resn_exch{"H132 3PI"}="2H13 3PI"; $resn_exch{"H133 3PI"}="3H13 3PI"; $resn_exch{"H151 3PI"}="1H15 3PI";
$resn_exch{"H152 3PI"}="2H15 3PI"; $resn_exch{"H161 3PI"}="1H16 3PI"; $resn_exch{"H162 3PI"}="2H16 3PI";
$resn_exch{"H171 3PI"}="1H17 3PI"; $resn_exch{"H172 3PI"}="2H17 3PI"; $resn_exch{"H173 3PI"}="3H17 3PI";
$resn_exch{" HA1 3PL"}="1HA  3PL"; $resn_exch{" HA2 3PL"}="2HA  3PL"; $resn_exch{" HB1 3PL"}="1HB  3PL";
$resn_exch{" HB2 3PL"}="2HB  3PL"; $resn_exch{"HOG2 3PO"}="2HOG 3PO"; $resn_exch{"HOG3 3PO"}="3HOG 3PO";
$resn_exch{"HOB2 3PO"}="2HOB 3PO"; $resn_exch{"HOA2 3PO"}="2HOA 3PO"; $resn_exch{"HO5' 3PO"}="HO5* 3PO";
$resn_exch{"HCB1 3PP"}="1HCB 3PP"; $resn_exch{"HCB2 3PP"}="2HCB 3PP"; $resn_exch{"HCG1 3PP"}="1HCG 3PP";
$resn_exch{"HCG2 3PP"}="2HCG 3PP"; $resn_exch{" H31 3PY"}="1H3  3PY"; $resn_exch{" H32 3PY"}="2H3  3PY";
$resn_exch{"H21A 3QC"}="1H2  3QC"; $resn_exch{"H22A 3QC"}="2H2  3QC"; $resn_exch{"H331 3QC"}="1H33 3QC";
$resn_exch{"H332 3QC"}="2H33 3QC"; $resn_exch{"H333 3QC"}="3H33 3QC"; $resn_exch{" H41 3QC"}="1H4  3QC";
$resn_exch{" H42 3QC"}="2H4  3QC"; $resn_exch{"H291 3QC"}="1H29 3QC"; $resn_exch{"H292 3QC"}="2H29 3QC";
$resn_exch{"H293 3QC"}="3H29 3QC"; $resn_exch{"H401 3QC"}="1H40 3QC"; $resn_exch{"H402 3QC"}="2H40 3QC";
$resn_exch{"H403 3QC"}="3H40 3QC"; $resn_exch{"H441 3QC"}="1H44 3QC"; $resn_exch{"H442 3QC"}="2H44 3QC";
$resn_exch{"H443 3QC"}="3H44 3QC"; $resn_exch{"H6X1 3SA"}="1H6X 3SA"; $resn_exch{"H6X2 3SA"}="2H6X 3SA";
$resn_exch{"H6Y1 3SA"}="1H6Y 3SA"; $resn_exch{"H6Y2 3SA"}="2H6Y 3SA"; $resn_exch{"H6Y3 3SA"}="3H6Y 3SA";
$resn_exch{"H7Z1 3SA"}="1H7Z 3SA"; $resn_exch{"H7Z2 3SA"}="2H7Z 3SA"; $resn_exch{"H121 3SC"}="1H12 3SC";
$resn_exch{"H122 3SC"}="2H12 3SC"; $resn_exch{" H31 3SL"}="1H3  3SL"; $resn_exch{" H32 3SL"}="2H3  3SL";
$resn_exch{"HN81 3SP"}="1HN8 3SP"; $resn_exch{"HN82 3SP"}="2HN8 3SP"; $resn_exch{" H91 3SP"}="1H9  3SP";
$resn_exch{" H92 3SP"}="2H9  3SP"; $resn_exch{"H141 3SP"}="1H14 3SP"; $resn_exch{"H142 3SP"}="2H14 3SP";
$resn_exch{"H151 3SP"}="1H15 3SP"; $resn_exch{"H152 3SP"}="2H15 3SP"; $resn_exch{"H161 3SP"}="1H16 3SP";
$resn_exch{"H162 3SP"}="2H16 3SP"; $resn_exch{"H221 3SP"}="1H22 3SP"; $resn_exch{"H222 3SP"}="2H22 3SP";
$resn_exch{"H241 3SP"}="1H24 3SP"; $resn_exch{"H242 3SP"}="2H24 3SP"; $resn_exch{"H261 3SP"}="1H26 3SP";
$resn_exch{"H262 3SP"}="2H26 3SP"; $resn_exch{"H271 3SP"}="1H27 3SP"; $resn_exch{"H272 3SP"}="2H27 3SP";
$resn_exch{"H281 3SP"}="1H28 3SP"; $resn_exch{"H282 3SP"}="2H28 3SP"; $resn_exch{"H291 3SP"}="1H29 3SP";
$resn_exch{"H292 3SP"}="2H29 3SP"; $resn_exch{"H301 3SP"}="1H30 3SP"; $resn_exch{"H302 3SP"}="2H30 3SP";
$resn_exch{"HN31 3TC"}="1HN3 3TC"; $resn_exch{"HN32 3TC"}="2HN3 3TC"; $resn_exch{" H61 3TC"}="1H6  3TC";
$resn_exch{" H62 3TC"}="2H6  3TC"; $resn_exch{" H81 3TC"}="1H8  3TC"; $resn_exch{" H82 3TC"}="2H8  3TC";
$resn_exch{"H151 3TH"}="1H15 3TH"; $resn_exch{"H152 3TH"}="2H15 3TH"; $resn_exch{" H31 3TL"}="1H3  3TL";
$resn_exch{" H32 3TL"}="2H3  3TL"; $resn_exch{"H201 3TL"}="1H20 3TL"; $resn_exch{"H202 3TL"}="2H20 3TL";
$resn_exch{"H203 3TL"}="3H20 3TL"; $resn_exch{"HA51 3TL"}="1HA5 3TL"; $resn_exch{"HA52 3TL"}="2HA5 3TL";
$resn_exch{"H701 3TL"}="1H70 3TL"; $resn_exch{"H702 3TL"}="2H70 3TL"; $resn_exch{"H703 3TL"}="3H70 3TL";
$resn_exch{"HG61 3TL"}="1HG6 3TL"; $resn_exch{"HG62 3TL"}="2HG6 3TL"; $resn_exch{"HG63 3TL"}="3HG6 3TL";
$resn_exch{"HG51 3TL"}="1HG5 3TL"; $resn_exch{"HG52 3TL"}="2HG5 3TL"; $resn_exch{"HG53 3TL"}="3HG5 3TL";
$resn_exch{"H531 3TL"}="1H53 3TL"; $resn_exch{"H532 3TL"}="2H53 3TL"; $resn_exch{"HG21 3TL"}="1HG2 3TL";
$resn_exch{"HG22 3TL"}="2HG2 3TL"; $resn_exch{"HG23 3TL"}="3HG2 3TL"; $resn_exch{"HG11 3TL"}="1HG1 3TL";
$resn_exch{"HG12 3TL"}="2HG1 3TL"; $resn_exch{"HG13 3TL"}="3HG1 3TL"; $resn_exch{" HA1 3TL"}="1HA  3TL";
$resn_exch{" HA2 3TL"}="2HA  3TL"; $resn_exch{"HAA1 3TN"}="1HAA 3TN"; $resn_exch{"HAA2 3TN"}="2HAA 3TN";
$resn_exch{"HAA3 3TN"}="3HAA 3TN"; $resn_exch{"HAB1 3TN"}="1HAB 3TN"; $resn_exch{"HAB2 3TN"}="2HAB 3TN";
$resn_exch{"HAB3 3TN"}="3HAB 3TN"; $resn_exch{"HAC1 3TN"}="1HAC 3TN"; $resn_exch{"HAC2 3TN"}="2HAC 3TN";
$resn_exch{"HAC3 3TN"}="3HAC 3TN"; $resn_exch{"HAP1 3TN"}="1HAP 3TN"; $resn_exch{"HAP2 3TN"}="2HAP 3TN";
$resn_exch{"HAQ1 3TN"}="1HAQ 3TN"; $resn_exch{"HAQ2 3TN"}="2HAQ 3TN"; $resn_exch{"HAD1 3TN"}="1HAD 3TN";
$resn_exch{"HAD2 3TN"}="2HAD 3TN"; $resn_exch{"HAD3 3TN"}="3HAD 3TN"; $resn_exch{"HAE1 3TN"}="1HAE 3TN";
$resn_exch{"HAE2 3TN"}="2HAE 3TN"; $resn_exch{"HAE3 3TN"}="3HAE 3TN"; $resn_exch{"HN61 3TP"}="1HN6 3TP";
$resn_exch{"HN62 3TP"}="2HN6 3TP"; $resn_exch{"H371 3TP"}="1H37 3TP"; $resn_exch{"H372 3TP"}="2H37 3TP";
$resn_exch{"H373 3TP"}="3H37 3TP"; $resn_exch{"H441 3TP"}="1H44 3TP"; $resn_exch{"H442 3TP"}="2H44 3TP";
$resn_exch{"H443 3TP"}="3H44 3TP"; $resn_exch{"H3A1 3TR"}="1H3A 3TR"; $resn_exch{"H3A2 3TR"}="2H3A 3TR";
$resn_exch{"HT31 3TY"}="1HT3 3TY"; $resn_exch{"HT32 3TY"}="2HT3 3TY"; $resn_exch{"H511 3UN"}="1H51 3UN";
$resn_exch{"H512 3UN"}="2H51 3UN"; $resn_exch{"H541 3UN"}="1H54 3UN"; $resn_exch{"H542 3UN"}="2H54 3UN";
$resn_exch{"H543 3UN"}="3H54 3UN"; $resn_exch{"H151 3UN"}="1H15 3UN"; $resn_exch{"H152 3UN"}="2H15 3UN";
$resn_exch{"H441 3UN"}="1H44 3UN"; $resn_exch{"H442 3UN"}="2H44 3UN"; $resn_exch{"H231 3UN"}="1H23 3UN";
$resn_exch{"H232 3UN"}="2H23 3UN"; $resn_exch{"H233 3UN"}="3H23 3UN"; $resn_exch{"H221 3UN"}="1H22 3UN";
$resn_exch{"H222 3UN"}="2H22 3UN"; $resn_exch{"H223 3UN"}="3H22 3UN"; $resn_exch{"H211 3UN"}="1H21 3UN";
$resn_exch{"H212 3UN"}="2H21 3UN"; $resn_exch{"H213 3UN"}="3H21 3UN"; $resn_exch{"H501 3UN"}="1H50 3UN";
$resn_exch{"H502 3UN"}="2H50 3UN"; $resn_exch{"H601 3UN"}="1H60 3UN"; $resn_exch{"H602 3UN"}="2H60 3UN";
$resn_exch{"H603 3UN"}="3H60 3UN"; $resn_exch{"H351 3UN"}="1H35 3UN"; $resn_exch{"H352 3UN"}="2H35 3UN";
$resn_exch{"HAF1 3YP"}="1HAF 3YP"; $resn_exch{"HAF2 3YP"}="2HAF 3YP"; $resn_exch{"HAC1 3YP"}="1HAC 3YP";
$resn_exch{"HAC2 3YP"}="2HAC 3YP"; $resn_exch{"HAD1 3YP"}="1HAD 3YP"; $resn_exch{"HAD2 3YP"}="2HAD 3YP";
$resn_exch{"HAG1 3YP"}="1HAG 3YP"; $resn_exch{"HAG2 3YP"}="2HAG 3YP"; $resn_exch{"HAH1 3YP"}="1HAH 3YP";
$resn_exch{"HAH2 3YP"}="2HAH 3YP"; $resn_exch{"HAE1 3YP"}="1HAE 3YP"; $resn_exch{"HAE2 3YP"}="2HAE 3YP";
$resn_exch{"H391 406"}="1H39 406"; $resn_exch{"H392 406"}="2H39 406"; $resn_exch{"H393 406"}="3H39 406";
$resn_exch{"H111 406"}="1H11 406"; $resn_exch{"H112 406"}="2H11 406"; $resn_exch{"H131 406"}="1H13 406";
$resn_exch{"H132 406"}="2H13 406"; $resn_exch{"H141 406"}="1H14 406"; $resn_exch{"H142 406"}="2H14 406";
$resn_exch{"H161 406"}="1H16 406"; $resn_exch{"H162 406"}="2H16 406"; $resn_exch{"H181 406"}="1H18 406";
$resn_exch{"H182 406"}="2H18 406"; $resn_exch{"H183 406"}="3H18 406"; $resn_exch{"H191 406"}="1H19 406";
$resn_exch{"H192 406"}="2H19 406"; $resn_exch{"H193 406"}="3H19 406"; $resn_exch{"H071 412"}="1H07 412";
$resn_exch{"H072 412"}="2H07 412"; $resn_exch{"H081 412"}="1H08 412"; $resn_exch{"H082 412"}="2H08 412";
$resn_exch{"H121 412"}="1H12 412"; $resn_exch{"H122 412"}="2H12 412"; $resn_exch{"H141 412"}="1H14 412";
$resn_exch{"H142 412"}="2H14 412"; $resn_exch{"H191 412"}="1H19 412"; $resn_exch{"H192 412"}="2H19 412";
$resn_exch{"H1N1 413"}="1H1N 413"; $resn_exch{"H1N2 413"}="2H1N 413"; $resn_exch{"H151 413"}="1H15 413";
$resn_exch{"H152 413"}="2H15 413"; $resn_exch{"H153 413"}="3H15 413"; $resn_exch{"H161 413"}="1H16 413";
$resn_exch{"H162 413"}="2H16 413"; $resn_exch{"H331 414"}="1H33 414"; $resn_exch{"H332 414"}="2H33 414";
$resn_exch{"H511 414"}="1H51 414"; $resn_exch{"H512 414"}="2H51 414"; $resn_exch{"H521 414"}="1H52 414";
$resn_exch{"H522 414"}="2H52 414"; $resn_exch{"H531 414"}="1H53 414"; $resn_exch{"H532 414"}="2H53 414";
$resn_exch{"H541 414"}="1H54 414"; $resn_exch{"H542 414"}="2H54 414"; $resn_exch{"H551 414"}="1H55 414";
$resn_exch{"H552 414"}="2H55 414"; $resn_exch{"H111 414"}="1H11 414"; $resn_exch{"H112 414"}="2H11 414";
$resn_exch{"H101 417"}="1H10 417"; $resn_exch{"H102 417"}="2H10 417"; $resn_exch{"H111 417"}="1H11 417";
$resn_exch{"H112 417"}="2H11 417"; $resn_exch{"H131 417"}="1H13 417"; $resn_exch{"H132 417"}="2H13 417";
$resn_exch{"H221 417"}="1H22 417"; $resn_exch{"H222 417"}="2H22 417"; $resn_exch{"H231 417"}="1H23 417";
$resn_exch{"H232 417"}="2H23 417"; $resn_exch{"H261 417"}="1H26 417"; $resn_exch{"H262 417"}="2H26 417";
$resn_exch{"H381 417"}="1H38 417"; $resn_exch{"H382 417"}="2H38 417"; $resn_exch{" H71 418"}="1H7  418";
$resn_exch{" H72 418"}="2H7  418"; $resn_exch{"H401 418"}="1H40 418"; $resn_exch{"H402 418"}="2H40 418";
$resn_exch{"H421 418"}="1H42 418"; $resn_exch{"H422 418"}="2H42 418"; $resn_exch{"H121 418"}="1H12 418";
$resn_exch{"H122 418"}="2H12 418"; $resn_exch{"H131 418"}="1H13 418"; $resn_exch{"H132 418"}="2H13 418";
$resn_exch{"H151 418"}="1H15 418"; $resn_exch{"H152 418"}="2H15 418"; $resn_exch{"H141 418"}="1H14 418";
$resn_exch{"H142 418"}="2H14 418"; $resn_exch{"H391 418"}="1H39 418"; $resn_exch{"H392 418"}="2H39 418";
$resn_exch{"H393 418"}="3H39 418"; $resn_exch{"H121 421"}="1H12 421"; $resn_exch{"H122 421"}="2H12 421";
$resn_exch{"H231 421"}="1H23 421"; $resn_exch{"H232 421"}="2H23 421"; $resn_exch{"H233 421"}="3H23 421";
$resn_exch{"H281 421"}="1H28 421"; $resn_exch{"H282 421"}="2H28 421"; $resn_exch{"H283 421"}="3H28 421";
$resn_exch{" H71 422"}="1H7  422"; $resn_exch{" H72 422"}="2H7  422"; $resn_exch{" H81 422"}="1H8  422";
$resn_exch{" H82 422"}="2H8  422"; $resn_exch{" H91 422"}="1H9  422"; $resn_exch{" H92 422"}="2H9  422";
$resn_exch{"H101 422"}="1H10 422"; $resn_exch{"H102 422"}="2H10 422"; $resn_exch{"H131 422"}="1H13 422";
$resn_exch{"H132 422"}="2H13 422"; $resn_exch{"H181 422"}="1H18 422"; $resn_exch{"H182 422"}="2H18 422";
$resn_exch{"HN31 422"}="1HN3 422"; $resn_exch{"HN32 422"}="2HN3 422"; $resn_exch{"H191 426"}="1H19 426";
$resn_exch{"H192 426"}="2H19 426"; $resn_exch{"H381 426"}="1H38 426"; $resn_exch{"H382 426"}="2H38 426";
$resn_exch{"H371 426"}="1H37 426"; $resn_exch{"H372 426"}="2H37 426"; $resn_exch{"H361 426"}="1H36 426";
$resn_exch{"H362 426"}="2H36 426"; $resn_exch{"H351 426"}="1H35 426"; $resn_exch{"H352 426"}="2H35 426";
$resn_exch{"H121 427"}="1H12 427"; $resn_exch{"H122 427"}="2H12 427"; $resn_exch{"H141 427"}="1H14 427";
$resn_exch{"H142 427"}="2H14 427"; $resn_exch{"H171 427"}="1H17 427"; $resn_exch{"H172 427"}="2H17 427";
$resn_exch{"H121 429"}="1H12 429"; $resn_exch{"H122 429"}="2H12 429"; $resn_exch{"H131 429"}="1H13 429";
$resn_exch{"H132 429"}="2H13 429"; $resn_exch{"H141 429"}="1H14 429"; $resn_exch{"H142 429"}="2H14 429";
$resn_exch{"H151 429"}="1H15 429"; $resn_exch{"H152 429"}="2H15 429"; $resn_exch{"H231 429"}="1H23 429";
$resn_exch{"H232 429"}="2H23 429"; $resn_exch{"H233 429"}="3H23 429"; $resn_exch{" H81 429"}="1H8  429";
$resn_exch{" H82 429"}="2H8  429"; $resn_exch{" H71 429"}="1H7  429"; $resn_exch{" H72 429"}="2H7  429";
$resn_exch{"H391 429"}="1H39 429"; $resn_exch{"H392 429"}="2H39 429"; $resn_exch{"H393 429"}="3H39 429";
$resn_exch{"H1N1 42B"}="1H1N 42B"; $resn_exch{"H1N2 42B"}="2H1N 42B"; $resn_exch{"H5C1 42B"}="1H5C 42B";
$resn_exch{"H5C2 42B"}="2H5C 42B"; $resn_exch{"H6C1 42B"}="1H6C 42B"; $resn_exch{"H6C2 42B"}="2H6C 42B";
$resn_exch{"HOB1 433"}="1HOB 433"; $resn_exch{"H1X1 433"}="1H1X 433"; $resn_exch{"H1X2 433"}="2H1X 433";
$resn_exch{"H1X3 433"}="3H1X 433"; $resn_exch{"H1Y1 433"}="1H1Y 433"; $resn_exch{"H1Y2 433"}="2H1Y 433";
$resn_exch{"H1Y3 433"}="3H1Y 433"; $resn_exch{"H1G1 433"}="1H1G 433"; $resn_exch{"H1I1 433"}="1H1I 433";
$resn_exch{"H1J1 433"}="1H1J 433"; $resn_exch{"H1H1 433"}="1H1H 433"; $resn_exch{"H1L1 433"}="1H1L 433";
$resn_exch{"H1L2 433"}="2H1L 433"; $resn_exch{"H1M1 433"}="1H1M 433"; $resn_exch{"H1M2 433"}="2H1M 433";
$resn_exch{"H1N1 433"}="1H1N 433"; $resn_exch{"H1N2 433"}="2H1N 433"; $resn_exch{"H2C1 433"}="1H2C 433";
$resn_exch{"H2C2 433"}="2H2C 433"; $resn_exch{"HN31 433"}="1HN3 433"; $resn_exch{"H3C1 433"}="1H3C 433";
$resn_exch{"H3E1 433"}="1H3E 433"; $resn_exch{"H3G1 433"}="1H3G 433"; $resn_exch{"H2D1 433"}="1H2D 433";
$resn_exch{"H2D2 433"}="2H2D 433"; $resn_exch{"H2H1 433"}="1H2H 433"; $resn_exch{"H2J1 433"}="1H2J 433";
$resn_exch{"H2I1 433"}="1H2I 433"; $resn_exch{" H31 434"}="1H3  434"; $resn_exch{" H32 434"}="2H3  434";
$resn_exch{"H33A 434"}="3H3  434"; $resn_exch{" H91 43A"}="1H9  43A"; $resn_exch{" H92 43A"}="2H9  43A";
$resn_exch{" H93 43A"}="3H9  43A"; $resn_exch{" H71 43M"}="1H7  43M"; $resn_exch{" H72 43M"}="2H7  43M";
$resn_exch{" H73 43M"}="3H7  43M"; $resn_exch{"HAS1 43P"}="1HAS 43P"; $resn_exch{"HAS2 43P"}="2HAS 43P";
$resn_exch{"HAT1 43P"}="1HAT 43P"; $resn_exch{"HAT2 43P"}="2HAT 43P"; $resn_exch{"HAT3 43P"}="3HAT 43P";
$resn_exch{" H71 442"}="1H7  442"; $resn_exch{" H72 442"}="2H7  442"; $resn_exch{"H291 442"}="1H29 442";
$resn_exch{"H292 442"}="2H29 442"; $resn_exch{"H301 442"}="1H30 442"; $resn_exch{"H302 442"}="2H30 442";
$resn_exch{"H161 444"}="1H16 444"; $resn_exch{"H162 444"}="2H16 444"; $resn_exch{"H111 448"}="1H11 448";
$resn_exch{"H112 448"}="2H11 448"; $resn_exch{"H201 448"}="1H20 448"; $resn_exch{"H202 448"}="2H20 448";
$resn_exch{"H151 448"}="1H15 448"; $resn_exch{"H152 448"}="2H15 448"; $resn_exch{"H251 448"}="1H25 448";
$resn_exch{"H252 448"}="2H25 448"; $resn_exch{"H261 448"}="1H26 448"; $resn_exch{"H262 448"}="2H26 448";
$resn_exch{"H341 448"}="1H34 448"; $resn_exch{"H342 448"}="2H34 448"; $resn_exch{"H343 448"}="3H34 448";
$resn_exch{"H161 44B"}="1H16 44B"; $resn_exch{"H162 44B"}="2H16 44B"; $resn_exch{"H171 44C"}="1H17 44C";
$resn_exch{"H172 44C"}="2H17 44C"; $resn_exch{"H181 44C"}="1H18 44C"; $resn_exch{"H182 44C"}="2H18 44C";
$resn_exch{"H191 44C"}="1H19 44C"; $resn_exch{"H192 44C"}="2H19 44C"; $resn_exch{"H201 44C"}="1H20 44C";
$resn_exch{"H202 44C"}="2H20 44C"; $resn_exch{"HC81 44D"}="1HC8 44D"; $resn_exch{"HC82 44D"}="2HC8 44D";
$resn_exch{"H101 44D"}="1H10 44D"; $resn_exch{"H102 44D"}="2H10 44D"; $resn_exch{"H141 44D"}="1H14 44D";
$resn_exch{"H142 44D"}="2H14 44D"; $resn_exch{" H1B 44D"}=" H1* 44D"; $resn_exch{"H2'1 44D"}="1H2* 44D";
$resn_exch{"H2'2 44D"}="2H2* 44D"; $resn_exch{" H3B 44D"}=" H3* 44D"; $resn_exch{" H4B 44D"}=" H4* 44D";
$resn_exch{" H5B 44D"}=" H5* 44D"; $resn_exch{"H6'1 44D"}="1H6* 44D"; $resn_exch{"H6'2 44D"}="2H6* 44D";
$resn_exch{"H6'3 44D"}="3H6* 44D"; $resn_exch{"H3'O 44D"}="OH3* 44D"; $resn_exch{" H2B 44D"}="1H2' 44D";
$resn_exch{" H2D 44D"}="2H2' 44D"; $resn_exch{" H6B 44D"}="1H6' 44D"; $resn_exch{" H6D 44D"}="2H6' 44D";
$resn_exch{" H6E 44D"}="3H6' 44D"; $resn_exch{"H4'O 44D"}="OH4' 44D"; $resn_exch{"H3'1 44D"}="1H3' 44D";
$resn_exch{"H3'2 44D"}="2H3' 44D"; $resn_exch{"H21A 450"}="1H2  450"; $resn_exch{"H22A 450"}="2H2  450";
$resn_exch{"H71A 450"}="1H7  450"; $resn_exch{" H72 450"}="2H7  450"; $resn_exch{"HN21 450"}="1HN2 450";
$resn_exch{"HN22 450"}="2HN2 450"; $resn_exch{"H311 450"}="1H31 450"; $resn_exch{"H312 450"}="2H31 450";
$resn_exch{"H611 450"}="1H61 450"; $resn_exch{"H612 450"}="2H61 450"; $resn_exch{"HN71 450"}="1HN7 450";
$resn_exch{"HN72 450"}="2HN7 450"; $resn_exch{" H11 452"}="1H1  452"; $resn_exch{"H12A 452"}="2H1  452";
$resn_exch{"H13A 452"}="3H1  452"; $resn_exch{" H91 452"}="1H9  452"; $resn_exch{" H92 452"}="2H9  452";
$resn_exch{"HAK1 458"}="1HAK 458"; $resn_exch{"HAK2 458"}="2HAK 458"; $resn_exch{"HAR1 458"}="1HAR 458";
$resn_exch{"HAR2 458"}="2HAR 458"; $resn_exch{"HAR3 458"}="3HAR 458"; $resn_exch{"HAP1 458"}="1HAP 458";
$resn_exch{"HAP2 458"}="2HAP 458"; $resn_exch{"HAT1 458"}="1HAT 458"; $resn_exch{"HAT2 458"}="2HAT 458";
$resn_exch{"HAM1 458"}="1HAM 458"; $resn_exch{"HAM2 458"}="2HAM 458"; $resn_exch{"HAT1 459"}="1HAT 459";
$resn_exch{"HAT2 459"}="2HAT 459"; $resn_exch{"HAT3 459"}="3HAT 459"; $resn_exch{"HAQ1 459"}="1HAQ 459";
$resn_exch{"HAQ2 459"}="2HAQ 459"; $resn_exch{"HAQ3 459"}="3HAQ 459"; $resn_exch{"HAP1 459"}="1HAP 459";
$resn_exch{"HAP2 459"}="2HAP 459"; $resn_exch{"HAO1 459"}="1HAO 459"; $resn_exch{"HAO2 459"}="2HAO 459";
$resn_exch{"HAR1 459"}="1HAR 459"; $resn_exch{"HAR2 459"}="2HAR 459"; $resn_exch{"HAM1 459"}="1HAM 459";
$resn_exch{"HAM2 459"}="2HAM 459"; $resn_exch{" H11 460"}="1H1  460"; $resn_exch{" H12 460"}="2H1  460";
$resn_exch{" H13 460"}="3H1  460"; $resn_exch{"H121 460"}="1H12 460"; $resn_exch{"H122 460"}="2H12 460";
$resn_exch{"H381 464"}="1H38 464"; $resn_exch{"H382 464"}="2H38 464"; $resn_exch{"H391 464"}="1H39 464";
$resn_exch{"H392 464"}="2H39 464"; $resn_exch{"H401 464"}="1H40 464"; $resn_exch{"H402 464"}="2H40 464";
$resn_exch{"H411 464"}="1H41 464"; $resn_exch{"H412 464"}="2H41 464"; $resn_exch{"H421 464"}="1H42 464";
$resn_exch{"H422 464"}="2H42 464"; $resn_exch{"H361 464"}="1H36 464"; $resn_exch{"H362 464"}="2H36 464";
$resn_exch{"H351 464"}="1H35 464"; $resn_exch{"H352 464"}="2H35 464"; $resn_exch{"H181 464"}="1H18 464";
$resn_exch{"H182 464"}="2H18 464"; $resn_exch{"H141 464"}="1H14 464"; $resn_exch{"H142 464"}="2H14 464";
$resn_exch{"H101 468"}="1H10 468"; $resn_exch{"H102 468"}="2H10 468"; $resn_exch{" H81 468"}="1H8  468";
$resn_exch{" H82 468"}="2H8  468"; $resn_exch{" H41 468"}="1H4  468"; $resn_exch{" H42 468"}="2H4  468";
$resn_exch{"H231 468"}="1H23 468"; $resn_exch{"H232 468"}="2H23 468"; $resn_exch{"H233 468"}="3H23 468";
$resn_exch{"HP61 46D"}="1HP6 46D"; $resn_exch{"HP62 46D"}="2HP6 46D"; $resn_exch{"HP81 46D"}="1HP8 46D";
$resn_exch{"HP82 46D"}="2HP8 46D"; $resn_exch{"HP83 46D"}="3HP8 46D"; $resn_exch{" H81 46M"}="1H8  46M";
$resn_exch{" H82 46M"}="2H8  46M"; $resn_exch{" H83 46M"}="3H8  46M"; $resn_exch{" H61 46M"}="1H6  46M";
$resn_exch{" H62 46M"}="2H6  46M"; $resn_exch{" HB1 471"}="1HB  471"; $resn_exch{" HB2 471"}="2HB  471";
$resn_exch{"H1B1 471"}="1H1B 471"; $resn_exch{"H1B2 471"}="2H1B 471"; $resn_exch{"HIB3 471"}="3HIB 471";
$resn_exch{"H3E1 471"}="1H3E 471"; $resn_exch{"H3E2 471"}="2H3E 471"; $resn_exch{"H3E3 471"}="3H3E 471";
$resn_exch{"H3A1 471"}="1H3A 471"; $resn_exch{"H3A2 471"}="2H3A 471"; $resn_exch{"H3B1 471"}="1H3B 471";
$resn_exch{"H3B2 471"}="2H3B 471"; $resn_exch{"H4A1 471"}="1H4A 471"; $resn_exch{"H4A2 471"}="2H4A 471";
$resn_exch{"H4D1 471"}="1H4D 471"; $resn_exch{"H4D2 471"}="2H4D 471"; $resn_exch{"H4E1 471"}="1H4E 471";
$resn_exch{"H4E2 471"}="2H4E 471"; $resn_exch{"H4E3 471"}="3H4E 471"; $resn_exch{"HN61 474"}="1HN6 474";
$resn_exch{"HN62 474"}="2HN6 474"; $resn_exch{"H211 474"}="1H21 474"; $resn_exch{"H212 474"}="2H21 474";
$resn_exch{"H201 474"}="1H20 474"; $resn_exch{"H202 474"}="2H20 474"; $resn_exch{"H181 474"}="1H18 474";
$resn_exch{"H182 474"}="2H18 474"; $resn_exch{"H101 474"}="1H10 474"; $resn_exch{"H102 474"}="2H10 474";
$resn_exch{"H103 474"}="3H10 474"; $resn_exch{"H141 474"}="1H14 474"; $resn_exch{"H142 474"}="2H14 474";
$resn_exch{"H143 474"}="3H14 474"; $resn_exch{"H311 474"}="1H31 474"; $resn_exch{"H312 474"}="2H31 474";
$resn_exch{"H301 474"}="1H30 474"; $resn_exch{"H302 474"}="2H30 474"; $resn_exch{"H341 474"}="1H34 474";
$resn_exch{"H342 474"}="2H34 474"; $resn_exch{"H331 474"}="1H33 474"; $resn_exch{"H332 474"}="2H33 474";
$resn_exch{"H11A 478"}="1H1  478"; $resn_exch{"H12A 478"}="2H1  478"; $resn_exch{"H21A 478"}="1H2  478";
$resn_exch{"H22A 478"}="2H2  478"; $resn_exch{" H71 478"}="1H7  478"; $resn_exch{" H72 478"}="2H7  478";
$resn_exch{"H141 478"}="1H14 478"; $resn_exch{"H142 478"}="2H14 478"; $resn_exch{"H151 478"}="1H15 478";
$resn_exch{"H152 478"}="2H15 478"; $resn_exch{"H231 478"}="1H23 478"; $resn_exch{"H232 478"}="2H23 478";
$resn_exch{"H233 478"}="3H23 478"; $resn_exch{"H241 478"}="1H24 478"; $resn_exch{"H242 478"}="2H24 478";
$resn_exch{"H243 478"}="3H24 478"; $resn_exch{"H251 478"}="1H25 478"; $resn_exch{"H252 478"}="2H25 478";
$resn_exch{"HN31 478"}="1HN3 478"; $resn_exch{"HN32 478"}="2HN3 478"; $resn_exch{" H31 47D"}="1H3  47D";
$resn_exch{" H32 47D"}="2H3  47D"; $resn_exch{" H51 47D"}="1H5  47D"; $resn_exch{" H52 47D"}="2H5  47D";
$resn_exch{" H81 485"}="1H8  485"; $resn_exch{" H82 485"}="2H8  485"; $resn_exch{"H101 485"}="1H10 485";
$resn_exch{"H102 485"}="2H10 485"; $resn_exch{"H271 485"}="1H27 485"; $resn_exch{"H272 485"}="2H27 485";
$resn_exch{"H273 485"}="3H27 485"; $resn_exch{"HC1A 486"}="1HC1 486"; $resn_exch{"HC12 486"}="2HC1 486";
$resn_exch{"HC71 486"}="1HC7 486"; $resn_exch{"HC72 486"}="2HC7 486"; $resn_exch{"HC61 486"}="1HC6 486";
$resn_exch{"HC62 486"}="2HC6 486"; $resn_exch{"H291 486"}="1H29 486"; $resn_exch{"H292 486"}="2H29 486";
$resn_exch{"H293 486"}="3H29 486"; $resn_exch{"H281 486"}="1H28 486"; $resn_exch{"H282 486"}="2H28 486";
$resn_exch{"H283 486"}="3H28 486"; $resn_exch{"H131 486"}="1H13 486"; $resn_exch{"H132 486"}="2H13 486";
$resn_exch{"H191 486"}="1H19 486"; $resn_exch{"H192 486"}="2H19 486"; $resn_exch{"H193 486"}="3H19 486";
$resn_exch{"H321 486"}="1H32 486"; $resn_exch{"H322 486"}="2H32 486"; $resn_exch{"H323 486"}="3H32 486";
$resn_exch{"H161 486"}="1H16 486"; $resn_exch{"H162 486"}="2H16 486"; $resn_exch{"H151 486"}="1H15 486";
$resn_exch{"H152 486"}="2H15 486"; $resn_exch{"HC81 486"}="1HC8 486"; $resn_exch{"HC82 486"}="2HC8 486";
$resn_exch{" H71 493"}="1H7  493"; $resn_exch{"H72A 493"}="2H7  493"; $resn_exch{" H91 493"}="1H9  493";
$resn_exch{" H92 493"}="2H9  493"; $resn_exch{"H151 493"}="1H15 493"; $resn_exch{"H152 493"}="2H15 493";
$resn_exch{"H153 493"}="3H15 493"; $resn_exch{"H191 493"}="1H19 493"; $resn_exch{"H192 493"}="2H19 493";
$resn_exch{"H201 493"}="1H20 493"; $resn_exch{"H202 493"}="2H20 493"; $resn_exch{"H211 493"}="1H21 493";
$resn_exch{"H212 493"}="2H21 493"; $resn_exch{"H221 493"}="1H22 493"; $resn_exch{"H222 493"}="2H22 493";
$resn_exch{"H251 493"}="1H25 493"; $resn_exch{"H252 493"}="2H25 493"; $resn_exch{"H471 497"}="1H47 497";
$resn_exch{"H472 497"}="2H47 497"; $resn_exch{"H473 497"}="3H47 497"; $resn_exch{"H451 497"}="1H45 497";
$resn_exch{"H452 497"}="2H45 497"; $resn_exch{"H381 497"}="1H38 497"; $resn_exch{"H382 497"}="2H38 497";
$resn_exch{"H361 497"}="1H36 497"; $resn_exch{"H362 497"}="2H36 497"; $resn_exch{"H181 497"}="1H18 497";
$resn_exch{"H182 497"}="2H18 497"; $resn_exch{" H91 49A"}="1H9  49A"; $resn_exch{" H92 49A"}="2H9  49A";
$resn_exch{"H111 49A"}="1H11 49A"; $resn_exch{"H112 49A"}="2H11 49A"; $resn_exch{"H113 49A"}="3H11 49A";
$resn_exch{"HOB1 49A"}="1HOB 49A"; $resn_exch{"HN41 49A"}="1HN4 49A"; $resn_exch{"HN42 49A"}="2HN4 49A";
$resn_exch{"HN91 49A"}="1HN9 49A"; $resn_exch{"HN92 49A"}="2HN9 49A"; $resn_exch{"HN11 4A3"}="1HN1 4A3";
$resn_exch{"HN12 4A3"}="2HN1 4A3"; $resn_exch{"HAA1 4AA"}="1HAA 4AA"; $resn_exch{"HAA2 4AA"}="2HAA 4AA";
$resn_exch{" H22 4AB"}="2H2  4AB"; $resn_exch{" H21 4AB"}="1H2  4AB"; $resn_exch{"HH41 4AB"}="1HH4 4AB";
$resn_exch{"HH42 4AB"}="2HH4 4AB"; $resn_exch{" H71 4AB"}="1H7  4AB"; $resn_exch{" H72 4AB"}="2H7  4AB";
$resn_exch{"H9O1 4AB"}="1H9O 4AB"; $resn_exch{"H113 4AB"}="3H11 4AB"; $resn_exch{"H112 4AB"}="2H11 4AB";
$resn_exch{"H111 4AB"}="1H11 4AB"; $resn_exch{"H1O1 4AB"}="1H1O 4AB"; $resn_exch{"HOP3 4AC"}="3HOP 4AC";
$resn_exch{"HOP2 4AC"}="2HOP 4AC"; $resn_exch{" H5' 4AC"}="1H5* 4AC"; $resn_exch{"H5'' 4AC"}="2H5* 4AC";
$resn_exch{" H4' 4AC"}=" H4* 4AC"; $resn_exch{" H3' 4AC"}=" H3* 4AC"; $resn_exch{"HO3' 4AC"}="H3T  4AC";
$resn_exch{" H2' 4AC"}=" H2* 4AC"; $resn_exch{"HO2' 4AC"}="2HO* 4AC"; $resn_exch{" H1' 4AC"}=" H1* 4AC";
$resn_exch{"HM71 4AC"}="1HM7 4AC"; $resn_exch{"HM72 4AC"}="2HM7 4AC"; $resn_exch{"HM73 4AC"}="3HM7 4AC";
$resn_exch{"HOP1 4AD"}="1HOP 4AD"; $resn_exch{" HN1 4AD"}="1HN  4AD"; $resn_exch{" HN2 4AD"}="2HN  4AD";
$resn_exch{" HN3 4AD"}="3HN  4AD"; $resn_exch{" HB1 4AD"}="1HB  4AD"; $resn_exch{" HB2 4AD"}="2HB  4AD";
$resn_exch{"HD21 4AD"}="1HD2 4AD"; $resn_exch{"HD22 4AD"}="2HD2 4AD"; $resn_exch{"H5'1 4AD"}="1H5* 4AD";
$resn_exch{"H5'2 4AD"}="2H5* 4AD"; $resn_exch{" H4' 4AD"}=" H4* 4AD"; $resn_exch{" H3' 4AD"}=" H3* 4AD";
$resn_exch{" H2' 4AD"}=" H2* 4AD"; $resn_exch{" H1' 4AD"}=" H1* 4AD"; $resn_exch{"HN61 4AD"}="1HN6 4AD";
$resn_exch{"HN62 4AD"}="2HN6 4AD"; $resn_exch{"H111 4AF"}="1H11 4AF"; $resn_exch{"H112 4AF"}="2H11 4AF";
$resn_exch{"H113 4AF"}="3H11 4AF"; $resn_exch{" H31 4AF"}="1H3  4AF"; $resn_exch{" H32 4AF"}="2H3  4AF";
$resn_exch{" HN1 4AF"}="1HN  4AF"; $resn_exch{" HN2 4AF"}="2HN  4AF"; $resn_exch{"H121 4AH"}="1H12 4AH";
$resn_exch{"H122 4AH"}="2H12 4AH"; $resn_exch{"H241 4AH"}="1H24 4AH"; $resn_exch{"H242 4AH"}="2H24 4AH";
$resn_exch{"H411 4AH"}="1H41 4AH"; $resn_exch{"H412 4AH"}="2H41 4AH"; $resn_exch{"H441 4AH"}="1H44 4AH";
$resn_exch{"H442 4AH"}="2H44 4AH"; $resn_exch{"H491 4AH"}="1H49 4AH"; $resn_exch{"H492 4AH"}="2H49 4AH";
$resn_exch{"H493 4AH"}="3H49 4AH"; $resn_exch{"H501 4AH"}="1H50 4AH"; $resn_exch{"H502 4AH"}="2H50 4AH";
$resn_exch{"H503 4AH"}="3H50 4AH"; $resn_exch{"H911 4AH"}="1H91 4AH"; $resn_exch{"H912 4AH"}="2H91 4AH";
$resn_exch{"H913 4AH"}="3H91 4AH"; $resn_exch{"H921 4AH"}="1H92 4AH"; $resn_exch{"H922 4AH"}="2H92 4AH";
$resn_exch{"H923 4AH"}="3H92 4AH"; $resn_exch{"HOB1 4AM"}="1HOB 4AM"; $resn_exch{"HN41 4AM"}="1HN4 4AM";
$resn_exch{"HN42 4AM"}="2HN4 4AM"; $resn_exch{" H5N 4AM"}=" NH5 4AM"; $resn_exch{"H111 4AM"}="1H11 4AM";
$resn_exch{"H112 4AM"}="2H11 4AM"; $resn_exch{"H113 4AM"}="3H11 4AM"; $resn_exch{" H7O 4AM"}=" OH7 4AM";
$resn_exch{" H8O 4AM"}=" OH8 4AM"; $resn_exch{" H91 4AM"}="1H9  4AM"; $resn_exch{" H92 4AM"}="2H9  4AM";
$resn_exch{"HN41 4AN"}="1HN4 4AN"; $resn_exch{"HN42 4AN"}="2HN4 4AN"; $resn_exch{"HN41 4AP"}="1HN4 4AP";
$resn_exch{"HN42 4AP"}="2HN4 4AP"; $resn_exch{" HA1 4AT"}="1HA  4AT"; $resn_exch{" HA2 4AT"}="2HA  4AT";
$resn_exch{" HB1 4AT"}="1HB  4AT"; $resn_exch{" HB2 4AT"}="2HB  4AT"; $resn_exch{" HG1 4AT"}="1HG  4AT";
$resn_exch{" HG2 4AT"}="2HG  4AT"; $resn_exch{"H2A1 4AT"}="1H2A 4AT"; $resn_exch{"H2A2 4AT"}="2H2A 4AT";
$resn_exch{"H2A3 4AT"}="3H2A 4AT"; $resn_exch{" HB1 4AX"}="1HB  4AX"; $resn_exch{" HB2 4AX"}="2HB  4AX";
$resn_exch{"H211 4B3"}="1H21 4B3"; $resn_exch{"H212 4B3"}="2H21 4B3"; $resn_exch{"H301 4B3"}="1H30 4B3";
$resn_exch{"H302 4B3"}="2H30 4B3"; $resn_exch{"H321 4B3"}="1H32 4B3"; $resn_exch{"H322 4B3"}="2H32 4B3";
$resn_exch{"H341 4B3"}="1H34 4B3"; $resn_exch{"H342 4B3"}="2H34 4B3"; $resn_exch{"H421 4B3"}="1H42 4B3";
$resn_exch{"H422 4B3"}="2H42 4B3"; $resn_exch{"H423 4B3"}="3H42 4B3"; $resn_exch{"H461 4B3"}="1H46 4B3";
$resn_exch{"H462 4B3"}="2H46 4B3"; $resn_exch{"H463 4B3"}="3H46 4B3"; $resn_exch{"H501 4B3"}="1H50 4B3";
$resn_exch{"H502 4B3"}="2H50 4B3"; $resn_exch{"H503 4B3"}="3H50 4B3"; $resn_exch{"H541 4B3"}="1H54 4B3";
$resn_exch{"H542 4B3"}="2H54 4B3"; $resn_exch{"H543 4B3"}="3H54 4B3"; $resn_exch{"H191 4BC"}="1H19 4BC";
$resn_exch{"H192 4BC"}="2H19 4BC"; $resn_exch{"H261 4BC"}="1H26 4BC"; $resn_exch{"H262 4BC"}="2H26 4BC";
$resn_exch{"H271 4BC"}="1H27 4BC"; $resn_exch{"H272 4BC"}="2H27 4BC"; $resn_exch{"H273 4BC"}="3H27 4BC";
$resn_exch{" HB3 4BF"}="1HCB 4BF"; $resn_exch{" HB2 4BF"}="2HCB 4BF"; $resn_exch{" H   4BF"}="1HN  4BF";
$resn_exch{" H2  4BF"}="2HN  4BF"; $resn_exch{"H111 4BH"}="1H11 4BH"; $resn_exch{"H112 4BH"}="2H11 4BH";
$resn_exch{"H121 4BH"}="1H12 4BH"; $resn_exch{"H122 4BH"}="2H12 4BH"; $resn_exch{"H151 4BH"}="1H15 4BH";
$resn_exch{"H152 4BH"}="2H15 4BH"; $resn_exch{"H141 4BH"}="1H14 4BH"; $resn_exch{"H142 4BH"}="2H14 4BH";
$resn_exch{"HZ21 4BR"}="1HZ2 4BR"; $resn_exch{"HZ22 4BR"}="2HZ2 4BR"; $resn_exch{"HA21 4BR"}="1HA2 4BR";
$resn_exch{"HA22 4BR"}="2HA2 4BR"; $resn_exch{"HH21 4BR"}="1HH2 4BR"; $resn_exch{"HH22 4BR"}="2HH2 4BR";
$resn_exch{"HH23 4BR"}="3HH2 4BR"; $resn_exch{"HCA1 4BT"}="1HCA 4BT"; $resn_exch{"HCA2 4BT"}="2HCA 4BT";
$resn_exch{"HCB1 4BT"}="1HCB 4BT"; $resn_exch{"HCB2 4BT"}="2HCB 4BT"; $resn_exch{" H51 4BT"}="1H5  4BT";
$resn_exch{" H61 4BT"}="1H6  4BT"; $resn_exch{" H21 4BT"}="1H2  4BT"; $resn_exch{" H31 4BT"}="1H3  4BT";
$resn_exch{"HCD1 4BT"}="1HCD 4BT"; $resn_exch{"HCD2 4BT"}="2HCD 4BT"; $resn_exch{"HCC1 4BT"}="1HCC 4BT";
$resn_exch{"HCC2 4BT"}="2HCC 4BT"; $resn_exch{"H1'1 4BT"}="1H1' 4BT"; $resn_exch{"H1'2 4BT"}="2H1' 4BT";
$resn_exch{"HN11 4BT"}="1HN1 4BT"; $resn_exch{"HN12 4BT"}="2HN1 4BT"; $resn_exch{"HN21 4BT"}="1HN2 4BT";
$resn_exch{" HB2 4BZ"}="2HB  4BZ"; $resn_exch{" HB3 4BZ"}="3HB  4BZ"; $resn_exch{"HQ11 4BZ"}="1HQ1 4BZ";
$resn_exch{"HQ12 4BZ"}="2HQ1 4BZ"; $resn_exch{"HQ21 4BZ"}="1HQ2 4BZ"; $resn_exch{" H2A 4CA"}=" AH2 4CA";
$resn_exch{"H61A 4CA"}="AH61 4CA"; $resn_exch{"H62A 4CA"}="AH62 4CA"; $resn_exch{" H8A 4CA"}=" AH8 4CA";
$resn_exch{" H1D 4CA"}="AH1* 4CA"; $resn_exch{" H2D 4CA"}="AH2* 4CA"; $resn_exch{"HO2A 4CA"}="AHO2 4CA";
$resn_exch{" H3D 4CA"}="AH3* 4CA"; $resn_exch{"HOA8 4CA"}="8HOA 4CA"; $resn_exch{"HOA9 4CA"}="9HOA 4CA";
$resn_exch{" H4D 4CA"}="AH4* 4CA"; $resn_exch{"H51A 4CA"}="AH51 4CA"; $resn_exch{"H52A 4CA"}="AH52 4CA";
$resn_exch{"HOA2 4CA"}="2HOA 4CA"; $resn_exch{"HOA5 4CA"}="5HOA 4CA"; $resn_exch{"H121 4CA"}="1H12 4CA";
$resn_exch{"H122 4CA"}="2H12 4CA"; $resn_exch{"H131 4CA"}="1H13 4CA"; $resn_exch{"H132 4CA"}="2H13 4CA";
$resn_exch{"H133 4CA"}="3H13 4CA"; $resn_exch{"H141 4CA"}="1H14 4CA"; $resn_exch{"H142 4CA"}="2H14 4CA";
$resn_exch{"H143 4CA"}="3H14 4CA"; $resn_exch{" H71 4CA"}="1H7  4CA"; $resn_exch{" H72 4CA"}="2H7  4CA";
$resn_exch{" H61 4CA"}="1H6  4CA"; $resn_exch{" H62 4CA"}="2H6  4CA"; $resn_exch{" H31 4CA"}="1H3  4CA";
$resn_exch{" H32 4CA"}="2H3  4CA"; $resn_exch{" H21 4CA"}="1H2  4CA"; $resn_exch{" H22 4CA"}="2H2  4CA";
$resn_exch{" H11 4CA"}="1H1  4CA"; $resn_exch{" H12 4CA"}="2H1  4CA"; $resn_exch{" H3B 4CA"}=" BH3 4CA";
$resn_exch{" H4B 4CA"}=" BH4 4CA"; $resn_exch{"HO2B 4CA"}="BHO2 4CA"; $resn_exch{" H6B 4CA"}=" BH6 4CA";
$resn_exch{" H7B 4CA"}=" BH7 4CA"; $resn_exch{"HO11 4CB"}="1HO1 4CB"; $resn_exch{"HO21 4CB"}="1HO2 4CB";
$resn_exch{"HC21 4CB"}="1HC2 4CB"; $resn_exch{"HC31 4CB"}="1HC3 4CB"; $resn_exch{"HOX1 4CB"}="1HOX 4CB";
$resn_exch{"HC51 4CB"}="1HC5 4CB"; $resn_exch{"HC61 4CB"}="1HC6 4CB"; $resn_exch{"HN11 4CM"}="1HN1 4CM";
$resn_exch{"HN12 4CM"}="2HN1 4CM"; $resn_exch{" H81 4CM"}="1H8  4CM"; $resn_exch{" H82 4CM"}="2H8  4CM";
$resn_exch{" H83 4CM"}="3H8  4CM"; $resn_exch{" H2A 4CO"}=" AH2 4CO"; $resn_exch{"H61A 4CO"}="AH61 4CO";
$resn_exch{"H62A 4CO"}="AH62 4CO"; $resn_exch{" H8A 4CO"}=" AH8 4CO"; $resn_exch{" H1D 4CO"}="AH1* 4CO";
$resn_exch{" H2D 4CO"}="AH2* 4CO"; $resn_exch{"HO2A 4CO"}="AHO2 4CO"; $resn_exch{" H3D 4CO"}="AH3* 4CO";
$resn_exch{"HOA8 4CO"}="8HOA 4CO"; $resn_exch{"HOA9 4CO"}="9HOA 4CO"; $resn_exch{" H4D 4CO"}="AH4* 4CO";
$resn_exch{"H51A 4CO"}="AH51 4CO"; $resn_exch{"H52A 4CO"}="AH52 4CO"; $resn_exch{"HOA2 4CO"}="2HOA 4CO";
$resn_exch{"HOA5 4CO"}="5HOA 4CO"; $resn_exch{"H121 4CO"}="1H12 4CO"; $resn_exch{"H122 4CO"}="2H12 4CO";
$resn_exch{"H131 4CO"}="1H13 4CO"; $resn_exch{"H132 4CO"}="2H13 4CO"; $resn_exch{"H133 4CO"}="3H13 4CO";
$resn_exch{"H141 4CO"}="1H14 4CO"; $resn_exch{"H142 4CO"}="2H14 4CO"; $resn_exch{"H143 4CO"}="3H14 4CO";
$resn_exch{" H71 4CO"}="1H7  4CO"; $resn_exch{" H72 4CO"}="2H7  4CO"; $resn_exch{" H61 4CO"}="1H6  4CO";
$resn_exch{" H62 4CO"}="2H6  4CO"; $resn_exch{" H31 4CO"}="1H3  4CO"; $resn_exch{" H32 4CO"}="2H3  4CO";
$resn_exch{" H21 4CO"}="1H2  4CO"; $resn_exch{" H22 4CO"}="2H2  4CO"; $resn_exch{" H3B 4CO"}=" BH3 4CO";
$resn_exch{" H4B 4CO"}=" BH4 4CO"; $resn_exch{" H2B 4CO"}=" BH2 4CO"; $resn_exch{" H6B 4CO"}=" BH6 4CO";
$resn_exch{" H7B 4CO"}=" BH7 4CO"; $resn_exch{" H1B 4CO"}="1BH  4CO"; $resn_exch{"H2BX 4CO"}="2BH  4CO";
$resn_exch{"H9C1 4CP"}="1H9C 4CP"; $resn_exch{"H9C2 4CP"}="2H9C 4CP"; $resn_exch{"H141 4CP"}="1H14 4CP";
$resn_exch{"H142 4CP"}="2H14 4CP"; $resn_exch{"H241 4CP"}="1H24 4CP"; $resn_exch{"H242 4CP"}="2H24 4CP";
$resn_exch{"H271 4CP"}="1H27 4CP"; $resn_exch{"H272 4CP"}="2H27 4CP"; $resn_exch{"H4B1 4CR"}="1H4B 4CR";
$resn_exch{"H4B2 4CR"}="2H4B 4CR"; $resn_exch{"H4C1 4CR"}="1H4C 4CR"; $resn_exch{"H4C2 4CR"}="2H4C 4CR";
$resn_exch{"H4C3 4CR"}="3H4C 4CR"; $resn_exch{"HN41 4CR"}="1HN4 4CR"; $resn_exch{"HN42 4CR"}="2HN4 4CR";
$resn_exch{"HN43 4CR"}="3HN4 4CR"; $resn_exch{"H2C1 4CR"}="1H2C 4CR"; $resn_exch{"H2C2 4CR"}="2H2C 4CR";
$resn_exch{"H1C1 4CR"}="1H1C 4CR"; $resn_exch{"H1C2 4CR"}="2H1C 4CR"; $resn_exch{"H101 4CR"}="1H10 4CR";
$resn_exch{"H102 4CR"}="2H10 4CR"; $resn_exch{"H103 4CR"}="3H10 4CR"; $resn_exch{"H131 4CR"}="1H13 4CR";
$resn_exch{"H132 4CR"}="2H13 4CR"; $resn_exch{" HB1 4CS"}="1HB  4CS"; $resn_exch{" HB2 4CS"}="2HB  4CS";
$resn_exch{"HAD1 4CS"}="1HAD 4CS"; $resn_exch{"HAD2 4CS"}="2HAD 4CS"; $resn_exch{"HAA1 4CS"}="1HAA 4CS";
$resn_exch{"HAA2 4CS"}="2HAA 4CS"; $resn_exch{"HAA3 4CS"}="3HAA 4CS"; $resn_exch{" H51 4DE"}="1H5  4DE";
$resn_exch{" H52 4DE"}="2H5  4DE"; $resn_exch{" H61 4DE"}="1H6  4DE"; $resn_exch{" H62 4DE"}="2H6  4DE";
$resn_exch{" H63 4DE"}="3H6  4DE"; $resn_exch{" H81 4DE"}="1H8  4DE"; $resn_exch{" H82 4DE"}="2H8  4DE";
$resn_exch{" H83 4DE"}="3H8  4DE"; $resn_exch{"H131 4DE"}="1H13 4DE"; $resn_exch{"H132 4DE"}="2H13 4DE";
$resn_exch{"H133 4DE"}="3H13 4DE"; $resn_exch{"H181 4DE"}="1H18 4DE"; $resn_exch{"H182 4DE"}="2H18 4DE";
$resn_exch{"H183 4DE"}="3H18 4DE"; $resn_exch{" H21 4DI"}="1H2  4DI"; $resn_exch{" H22 4DI"}="2H2  4DI";
$resn_exch{" H31 4DI"}="1H3  4DI"; $resn_exch{" H32 4DI"}="2H3  4DI"; $resn_exch{" H41 4DI"}="1H4  4DI";
$resn_exch{" H42 4DI"}="2H4  4DI"; $resn_exch{" H51 4DI"}="1H5  4DI"; $resn_exch{" H52 4DI"}="2H5  4DI";
$resn_exch{" H61 4DI"}="1H6  4DI"; $resn_exch{" H62 4DI"}="2H6  4DI"; $resn_exch{" H63 4DI"}="3H6  4DI";
$resn_exch{" H71 4DI"}="1H7  4DI"; $resn_exch{" H72 4DI"}="2H7  4DI"; $resn_exch{" H73 4DI"}="3H7  4DI";
$resn_exch{"HN21 4DI"}="1HN2 4DI"; $resn_exch{"HN22 4DI"}="2HN2 4DI"; $resn_exch{"HAD1 4DP"}="1HAD 4DP";
$resn_exch{"HAD2 4DP"}="2HAD 4DP"; $resn_exch{"HAD3 4DP"}="3HAD 4DP"; $resn_exch{"HAC1 4DP"}="1HAC 4DP";
$resn_exch{"HAC2 4DP"}="2HAC 4DP"; $resn_exch{"HAC3 4DP"}="3HAC 4DP"; $resn_exch{" HB3 4DP"}="1HB  4DP";
$resn_exch{" HB2 4DP"}="2HB  4DP"; $resn_exch{" H   4DP"}="1HN  4DP"; $resn_exch{" H2  4DP"}="2HN  4DP";
$resn_exch{" HXT 4DP"}="HOXT 4DP"; $resn_exch{"H241 4EA"}="1H24 4EA"; $resn_exch{"H242 4EA"}="2H24 4EA";
$resn_exch{"H251 4EA"}="1H25 4EA"; $resn_exch{"H252 4EA"}="2H25 4EA"; $resn_exch{"HN41 4EA"}="1HN4 4EA";
$resn_exch{"HN42 4EA"}="2HN4 4EA"; $resn_exch{" H10 4EB"}="0H1  4EB"; $resn_exch{" H11 4EB"}="1H1  4EB";
$resn_exch{" H12 4EB"}="2H1  4EB"; $resn_exch{" H13 4EB"}="3H1  4EB"; $resn_exch{"H111 4F3"}="1H11 4F3";
$resn_exch{"H112 4F3"}="2H11 4F3"; $resn_exch{"H181 4F3"}="1H18 4F3"; $resn_exch{"H182 4F3"}="2H18 4F3";
$resn_exch{"HO42 4F3"}="2HO4 4F3"; $resn_exch{"H251 4F3"}="1H25 4F3"; $resn_exch{"H252 4F3"}="2H25 4F3";
$resn_exch{"H253 4F3"}="3H25 4F3"; $resn_exch{"HC41 4FA"}="1HC4 4FA"; $resn_exch{"HC42 4FA"}="2HC4 4FA";
$resn_exch{"H131 4FA"}="1H13 4FA"; $resn_exch{"H132 4FA"}="2H13 4FA"; $resn_exch{"HBX1 4FB"}="1HBX 4FB";
$resn_exch{"HBX2 4FB"}="2HBX 4FB"; $resn_exch{"HDX1 4FB"}="1HDX 4FB"; $resn_exch{"HDX2 4FB"}="2HDX 4FB";
$resn_exch{" H61 4FP"}="1H6  4FP"; $resn_exch{" H62 4FP"}="2H6  4FP"; $resn_exch{" H81 4FP"}="1H8  4FP";
$resn_exch{" H82 4FP"}="2H8  4FP"; $resn_exch{" H91 4FP"}="1H9  4FP"; $resn_exch{" H92 4FP"}="2H9  4FP";
$resn_exch{"H111 4FP"}="1H11 4FP"; $resn_exch{"H112 4FP"}="2H11 4FP"; $resn_exch{"H121 4FP"}="1H12 4FP";
$resn_exch{"H122 4FP"}="2H12 4FP"; $resn_exch{" H   4FW"}="1HN  4FW"; $resn_exch{" H2  4FW"}="2HN  4FW";
$resn_exch{" HB2 4FW"}="1HB  4FW"; $resn_exch{" HB3 4FW"}="2HB  4FW"; $resn_exch{" HE1 4FW"}=" HNE 4FW";
$resn_exch{" H61 4GP"}="1H6  4GP"; $resn_exch{" H62 4GP"}="2H6  4GP"; $resn_exch{" HB2 4HA"}="2HB  4HA";
$resn_exch{" HB3 4HA"}="3HB  4HA"; $resn_exch{" HG2 4HA"}="2HG  4HA"; $resn_exch{" HG3 4HA"}="3HG  4HA";
$resn_exch{" HD2 4HA"}="2HD  4HA"; $resn_exch{" HD3 4HA"}="3HD  4HA"; $resn_exch{" HE2 4HA"}="2HE  4HA";
$resn_exch{" HE3 4HA"}="3HE  4HA"; $resn_exch{" HZ1 4HA"}="1HZ  4HA"; $resn_exch{" HZ2 4HA"}="2HZ  4HA";
$resn_exch{" HZ3 4HA"}="3HZ  4HA"; $resn_exch{" H51 4HF"}="1H5  4HF"; $resn_exch{" H52 4HF"}="2H5  4HF";
$resn_exch{"HN21 4HF"}="1HN2 4HF"; $resn_exch{"HN22 4HF"}="2HN2 4HF"; $resn_exch{" H71 4HF"}="1H7  4HF";
$resn_exch{" H72 4HF"}="2H7  4HF"; $resn_exch{" H91 4HF"}="1H9  4HF"; $resn_exch{" H92 4HF"}="2H9  4HF";
$resn_exch{"H101 4HF"}="1H10 4HF"; $resn_exch{"H102 4HF"}="2H10 4HF"; $resn_exch{" HB1 4HF"}="1HB  4HF";
$resn_exch{" HB2 4HF"}="2HB  4HF"; $resn_exch{" HG1 4HF"}="1HG  4HF"; $resn_exch{" HG2 4HF"}="2HG  4HF";
$resn_exch{"H111 4HG"}="1H11 4HG"; $resn_exch{"H112 4HG"}="2H11 4HG"; $resn_exch{" H11 4HL"}="1H1  4HL";
$resn_exch{" H12 4HL"}="2H1  4HL"; $resn_exch{"HX21 4HL"}="1HX2 4HL"; $resn_exch{"HX22 4HL"}="2HX2 4HL";
$resn_exch{" H71 4HP"}="1H7  4HP"; $resn_exch{" H72 4HP"}="2H7  4HP"; $resn_exch{" H   4HT"}="1HN  4HT";
$resn_exch{" H2  4HT"}="2HN  4HT"; $resn_exch{" HB2 4HT"}="1HB  4HT"; $resn_exch{" HB3 4HT"}="2HB  4HT";
$resn_exch{" HN1 4HX"}="1HN  4HX"; $resn_exch{" HN2 4HX"}="2HN  4HX"; $resn_exch{" HB1 4HX"}="1HB  4HX";
$resn_exch{" HB2 4HX"}="2HB  4HX"; $resn_exch{" HG1 4HX"}="1HG  4HX"; $resn_exch{" HG2 4HX"}="2HG  4HX";
$resn_exch{" HD1 4HX"}="1HD  4HX"; $resn_exch{" HD2 4HX"}="2HD  4HX"; $resn_exch{"H1'1 4HX"}="1H1' 4HX";
$resn_exch{"H1'2 4HX"}="2H1' 4HX"; $resn_exch{"HB'1 4HX"}="1HB' 4HX"; $resn_exch{"HB'2 4HX"}="2HB' 4HX";
$resn_exch{"HD'1 4HX"}="1HD' 4HX"; $resn_exch{"HD'2 4HX"}="2HD' 4HX"; $resn_exch{"HC31 4HY"}="1HC3 4HY";
$resn_exch{"HC41 4HY"}="1HC4 4HY"; $resn_exch{"H101 4HY"}="1H10 4HY"; $resn_exch{"H111 4HY"}="1H11 4HY";
$resn_exch{"H121 4HY"}="1H12 4HY"; $resn_exch{"H131 4HY"}="1H13 4HY"; $resn_exch{"H132 4HY"}="2H13 4HY";
$resn_exch{"HO11 4HY"}="1HO1 4HY"; $resn_exch{"HO41 4HY"}="1HO4 4HY"; $resn_exch{"HN31 4IG"}="1HN3 4IG";
$resn_exch{"HN32 4IG"}="2HN3 4IG"; $resn_exch{"HN41 4IG"}="1HN4 4IG"; $resn_exch{"HN42 4IG"}="2HN4 4IG";
$resn_exch{" H51 4IG"}="1H5  4IG"; $resn_exch{" H52 4IG"}="2H5  4IG"; $resn_exch{" H61 4IG"}="1H6  4IG";
$resn_exch{" H62 4IG"}="2H6  4IG"; $resn_exch{" H63 4IG"}="3H6  4IG"; $resn_exch{"H161 4IG"}="1H16 4IG";
$resn_exch{"H162 4IG"}="2H16 4IG"; $resn_exch{"H171 4IG"}="1H17 4IG"; $resn_exch{"H172 4IG"}="2H17 4IG";
$resn_exch{"H181 4IG"}="1H18 4IG"; $resn_exch{"H182 4IG"}="2H18 4IG"; $resn_exch{"H191 4IG"}="1H19 4IG";
$resn_exch{"H192 4IG"}="2H19 4IG"; $resn_exch{"H193 4IG"}="3H19 4IG"; $resn_exch{" H   4IN"}="1HN  4IN";
$resn_exch{" H2  4IN"}="2HN  4IN"; $resn_exch{" HB2 4IN"}="1HB  4IN"; $resn_exch{" HB3 4IN"}="2HB  4IN";
$resn_exch{"H161 4IN"}="1H16 4IN"; $resn_exch{"H162 4IN"}="2H16 4IN"; $resn_exch{"HO2P 4IP"}="PHO2 4IP";
$resn_exch{"HO3P 4IP"}="PHO3 4IP"; $resn_exch{"HO5P 4IP"}="PHO5 4IP"; $resn_exch{"HO6P 4IP"}="PHO6 4IP";
$resn_exch{"HO7P 4IP"}="PHO7 4IP"; $resn_exch{"HO8P 4IP"}="PHO8 4IP"; $resn_exch{"HOP1 4IP"}="1HOP 4IP";
$resn_exch{"HOP3 4IP"}="3HOP 4IP"; $resn_exch{" H11 4LG"}="1H1  4LG"; $resn_exch{" H12 4LG"}="2H1  4LG";
$resn_exch{"H13A 4LG"}="3H1  4LG"; $resn_exch{" H41 4LG"}="1H4  4LG"; $resn_exch{" H42 4LG"}="2H4  4LG";
$resn_exch{"H171 4LG"}="1H17 4LG"; $resn_exch{"H172 4LG"}="2H17 4LG"; $resn_exch{"H191 4LG"}="1H19 4LG";
$resn_exch{"H192 4LG"}="2H19 4LG"; $resn_exch{"H193 4LG"}="3H19 4LG"; $resn_exch{"H211 4LG"}="1H21 4LG";
$resn_exch{"H212 4LG"}="2H21 4LG"; $resn_exch{"H221 4LG"}="1H22 4LG"; $resn_exch{"H222 4LG"}="2H22 4LG";
$resn_exch{"H241 4LG"}="1H24 4LG"; $resn_exch{"H242 4LG"}="2H24 4LG"; $resn_exch{"H311 4LG"}="1H31 4LG";
$resn_exch{"H312 4LG"}="2H31 4LG"; $resn_exch{"H321 4LG"}="1H32 4LG"; $resn_exch{"H322 4LG"}="2H32 4LG";
$resn_exch{"H331 4LG"}="1H33 4LG"; $resn_exch{"H332 4LG"}="2H33 4LG"; $resn_exch{" H81 4MA"}="1H8  4MA";
$resn_exch{" H82 4MA"}="2H8  4MA"; $resn_exch{" H83 4MA"}="3H8  4MA"; $resn_exch{"H121 4MB"}="1H12 4MB";
$resn_exch{"H122 4MB"}="2H12 4MB"; $resn_exch{"H123 4MB"}="3H12 4MB"; $resn_exch{"H331 4MC"}="1H33 4MC";
$resn_exch{"H332 4MC"}="2H33 4MC"; $resn_exch{"H333 4MC"}="3H33 4MC"; $resn_exch{"H491 4MC"}="1H49 4MC";
$resn_exch{"H492 4MC"}="2H49 4MC"; $resn_exch{"H481 4MC"}="1H48 4MC"; $resn_exch{"H482 4MC"}="2H48 4MC";
$resn_exch{"H391 4MC"}="1H39 4MC"; $resn_exch{"H392 4MC"}="2H39 4MC"; $resn_exch{"H401 4MC"}="1H40 4MC";
$resn_exch{"H402 4MC"}="2H40 4MC"; $resn_exch{"H101 4MC"}="1H10 4MC"; $resn_exch{"H102 4MC"}="2H10 4MC";
$resn_exch{"H111 4MC"}="1H11 4MC"; $resn_exch{"H112 4MC"}="2H11 4MC"; $resn_exch{"H011 4MC"}="1H01 4MC";
$resn_exch{"H012 4MC"}="2H01 4MC"; $resn_exch{"H021 4MC"}="1H02 4MC"; $resn_exch{"H022 4MC"}="2H02 4MC";
$resn_exch{"H191 4MC"}="1H19 4MC"; $resn_exch{"H192 4MC"}="2H19 4MC"; $resn_exch{"H201 4MC"}="1H20 4MC";
$resn_exch{"H202 4MC"}="2H20 4MC"; $resn_exch{"HOP3 4MF"}="3HOP 4MF"; $resn_exch{"HOP2 4MF"}="2HOP 4MF";
$resn_exch{"H5'1 4MF"}="1H5* 4MF"; $resn_exch{"H5'2 4MF"}="2H5* 4MF"; $resn_exch{" H4' 4MF"}=" H4* 4MF";
$resn_exch{" H3' 4MF"}=" H3* 4MF"; $resn_exch{"H2'1 4MF"}="1H2* 4MF"; $resn_exch{"H2'2 4MF"}="2H2* 4MF";
$resn_exch{" H1' 4MF"}=" H1* 4MF"; $resn_exch{"H4M1 4MF"}="1H4M 4MF"; $resn_exch{"H4M2 4MF"}="2H4M 4MF";
$resn_exch{"H4M3 4MF"}="3H4M 4MF"; $resn_exch{" H21 4MP"}="1H2  4MP"; $resn_exch{" H22 4MP"}="2H2  4MP";
$resn_exch{" H31 4MP"}="1H3  4MP"; $resn_exch{" H32 4MP"}="2H3  4MP"; $resn_exch{" H41 4MP"}="1H4  4MP";
$resn_exch{" H42 4MP"}="2H4  4MP"; $resn_exch{"H121 4MP"}="1H12 4MP"; $resn_exch{"H122 4MP"}="2H12 4MP";
$resn_exch{"H123 4MP"}="3H12 4MP"; $resn_exch{"HAR1 4MR"}="1HAR 4MR"; $resn_exch{"HAR2 4MR"}="2HAR 4MR";
$resn_exch{"HAQ1 4MR"}="1HAQ 4MR"; $resn_exch{"HAQ2 4MR"}="2HAQ 4MR"; $resn_exch{"HBC1 4MR"}="1HBC 4MR";
$resn_exch{"HBC2 4MR"}="2HBC 4MR"; $resn_exch{"HBB1 4MR"}="1HBB 4MR"; $resn_exch{"HBB2 4MR"}="2HBB 4MR";
$resn_exch{" HA1 4MV"}="1HA  4MV"; $resn_exch{" HA2 4MV"}="2HA  4MV"; $resn_exch{" HB1 4MV"}="1HB  4MV";
$resn_exch{" HB2 4MV"}="2HB  4MV"; $resn_exch{"HD11 4MV"}="1HD1 4MV"; $resn_exch{"HD12 4MV"}="2HD1 4MV";
$resn_exch{"HD13 4MV"}="3HD1 4MV"; $resn_exch{"HD21 4MV"}="1HD2 4MV"; $resn_exch{"HD22 4MV"}="2HD2 4MV";
$resn_exch{"HD23 4MV"}="3HD2 4MV"; $resn_exch{" H41 4MZ"}="1H4  4MZ"; $resn_exch{" H42 4MZ"}="2H4  4MZ";
$resn_exch{" H43 4MZ"}="3H4  4MZ"; $resn_exch{"HC41 4ND"}="1HC4 4ND"; $resn_exch{"HC51 4ND"}="1HC5 4ND";
$resn_exch{"HC61 4ND"}="1HC6 4ND"; $resn_exch{"HC71 4ND"}="1HC7 4ND"; $resn_exch{"HC81 4ND"}="1HC8 4ND";
$resn_exch{"H101 4ND"}="1H10 4ND"; $resn_exch{"H111 4ND"}="1H11 4ND"; $resn_exch{"H121 4ND"}="1H12 4ND";
$resn_exch{"H131 4ND"}="1H13 4ND"; $resn_exch{"H141 4ND"}="1H14 4ND"; $resn_exch{"HN41 4ND"}="1HN4 4ND";
$resn_exch{"HN31 4ND"}="1HN3 4ND"; $resn_exch{"HN21 4ND"}="1HN2 4ND"; $resn_exch{"HN22 4ND"}="2HN2 4ND";
$resn_exch{"HN11 4ND"}="1HN1 4ND"; $resn_exch{"H11A 4NH"}="1H1  4NH"; $resn_exch{" H12 4NH"}="2H1  4NH";
$resn_exch{" H13 4NH"}="3H1  4NH"; $resn_exch{" H41 4NH"}="1H4  4NH"; $resn_exch{" H42 4NH"}="2H4  4NH";
$resn_exch{" H51 4NH"}="1H5  4NH"; $resn_exch{" H52 4NH"}="2H5  4NH"; $resn_exch{"H121 4NH"}="1H12 4NH";
$resn_exch{"H122 4NH"}="2H12 4NH"; $resn_exch{"H151 4NH"}="1H15 4NH"; $resn_exch{"H152 4NH"}="2H15 4NH";
$resn_exch{"HN21 4NH"}="1HN2 4NH"; $resn_exch{"HN22 4NH"}="2HN2 4NH"; $resn_exch{"H171 4NH"}="1H17 4NH";
$resn_exch{"H172 4NH"}="2H17 4NH"; $resn_exch{"H173 4NH"}="3H17 4NH"; $resn_exch{" H21 4NI"}="1H2  4NI";
$resn_exch{" H22 4NI"}="2H2  4NI"; $resn_exch{" H31 4NI"}="1H3  4NI"; $resn_exch{" H32 4NI"}="2H3  4NI";
$resn_exch{" H41 4NI"}="1H4  4NI"; $resn_exch{" H42 4NI"}="2H4  4NI"; $resn_exch{" H21 4NP"}="1H2  4NP";
$resn_exch{" H31 4NP"}="1H3  4NP"; $resn_exch{" H51 4NP"}="1H5  4NP"; $resn_exch{" H61 4NP"}="1H6  4NP";
$resn_exch{"HM41 4OC"}="1HM4 4OC"; $resn_exch{"HM42 4OC"}="2HM4 4OC"; $resn_exch{"HM43 4OC"}="3HM4 4OC";
$resn_exch{" H1' 4OC"}=" H1* 4OC"; $resn_exch{" H4' 4OC"}=" H4* 4OC"; $resn_exch{" H5' 4OC"}="1H5* 4OC";
$resn_exch{"H5'' 4OC"}="2H5* 4OC"; $resn_exch{"HOP3 4OC"}="3HOP 4OC"; $resn_exch{" H3' 4OC"}=" H3* 4OC";
$resn_exch{"HO3' 4OC"}="H3T  4OC"; $resn_exch{" H2' 4OC"}=" H2* 4OC"; $resn_exch{"HM21 4OC"}="1HM2 4OC";
$resn_exch{"HM22 4OC"}="2HM2 4OC"; $resn_exch{"HM23 4OC"}="3HM2 4OC"; $resn_exch{"HOP2 4OC"}="2HOP 4OC";
$resn_exch{"HC71 4OX"}="1HC7 4OX"; $resn_exch{"HC72 4OX"}="2HC7 4OX"; $resn_exch{"HC81 4OX"}="1HC8 4OX";
$resn_exch{"HC82 4OX"}="2HC8 4OX"; $resn_exch{"HC91 4OX"}="1HC9 4OX"; $resn_exch{"HC92 4OX"}="2HC9 4OX";
$resn_exch{"HC21 4OX"}="1HC2 4OX"; $resn_exch{"HC22 4OX"}="2HC2 4OX"; $resn_exch{"HC31 4OX"}="1HC3 4OX";
$resn_exch{"HC32 4OX"}="2HC3 4OX"; $resn_exch{"HC51 4OX"}="1HC5 4OX"; $resn_exch{"HC52 4OX"}="2HC5 4OX";
$resn_exch{"HC61 4OX"}="1HC6 4OX"; $resn_exch{"HC62 4OX"}="2HC6 4OX"; $resn_exch{"HAC1 4PA"}="1HAC 4PA";
$resn_exch{"HAC2 4PA"}="2HAC 4PA"; $resn_exch{"HAC3 4PA"}="3HAC 4PA"; $resn_exch{"HAF1 4PA"}="1HAF 4PA";
$resn_exch{"HAF2 4PA"}="2HAF 4PA"; $resn_exch{"HAE1 4PA"}="1HAE 4PA"; $resn_exch{"HAE2 4PA"}="2HAE 4PA";
$resn_exch{"HAA1 4PA"}="1HAA 4PA"; $resn_exch{"HAA2 4PA"}="2HAA 4PA"; $resn_exch{"HAA3 4PA"}="3HAA 4PA";
$resn_exch{" H21 4PB"}="1H2  4PB"; $resn_exch{" H22 4PB"}="2H2  4PB"; $resn_exch{" H31 4PB"}="1H3  4PB";
$resn_exch{" H32 4PB"}="2H3  4PB"; $resn_exch{" H41 4PB"}="1H4  4PB"; $resn_exch{" H42 4PB"}="2H4  4PB";
$resn_exch{"HOP2 4PC"}="2HOP 4PC"; $resn_exch{" H5' 4PC"}="1H5* 4PC"; $resn_exch{"H5'' 4PC"}="2H5* 4PC";
$resn_exch{" H4' 4PC"}=" H4* 4PC"; $resn_exch{" H3' 4PC"}=" H3* 4PC"; $resn_exch{"HO3' 4PC"}="H3T  4PC";
$resn_exch{" H2' 4PC"}="1H2* 4PC"; $resn_exch{"H2'' 4PC"}="2H2* 4PC"; $resn_exch{" H1' 4PC"}=" H1* 4PC";
$resn_exch{"H161 4PC"}="1H16 4PC"; $resn_exch{"H162 4PC"}="2H16 4PC"; $resn_exch{"H163 4PC"}="3H16 4PC";
$resn_exch{"HOP3 4PC"}="3HOP 4PC"; $resn_exch{"HOP2 4PD"}="2HOP 4PD"; $resn_exch{" H5' 4PD"}="1H5* 4PD";
$resn_exch{"H5'' 4PD"}="2H5* 4PD"; $resn_exch{" H4' 4PD"}=" H4* 4PD"; $resn_exch{" H3' 4PD"}=" H3* 4PD";
$resn_exch{"HO3' 4PD"}="H3T  4PD"; $resn_exch{" H2' 4PD"}="1H2* 4PD"; $resn_exch{"H2'' 4PD"}="2H2* 4PD";
$resn_exch{" H1' 4PD"}=" H1* 4PD"; $resn_exch{"H161 4PD"}="1H16 4PD"; $resn_exch{"H162 4PD"}="2H16 4PD";
$resn_exch{"H163 4PD"}="3H16 4PD"; $resn_exch{"HOP3 4PD"}="3HOP 4PD"; $resn_exch{"HOP2 4PE"}="2HOP 4PE";
$resn_exch{" H5' 4PE"}="1H5* 4PE"; $resn_exch{"H5'' 4PE"}="2H5* 4PE"; $resn_exch{" H4' 4PE"}=" H4* 4PE";
$resn_exch{" H3' 4PE"}=" H3* 4PE"; $resn_exch{"HO3' 4PE"}="H3T  4PE"; $resn_exch{" H2' 4PE"}="1H2* 4PE";
$resn_exch{"H2'' 4PE"}="2H2* 4PE"; $resn_exch{" H1' 4PE"}=" H1* 4PE"; $resn_exch{"H161 4PE"}="1H16 4PE";
$resn_exch{"H162 4PE"}="2H16 4PE"; $resn_exch{"H163 4PE"}="3H16 4PE"; $resn_exch{"HOP3 4PE"}="3HOP 4PE";
$resn_exch{" H81 4PG"}="1H8  4PG"; $resn_exch{" H82 4PG"}="2H8  4PG"; $resn_exch{"HN21 4PG"}="1HN2 4PG";
$resn_exch{"HN22 4PG"}="2HN2 4PG"; $resn_exch{" H71 4PI"}="1H7  4PI"; $resn_exch{" H72 4PI"}="2H7  4PI";
$resn_exch{" H81 4PI"}="1H8  4PI"; $resn_exch{" H82 4PI"}="2H8  4PI"; $resn_exch{"H101 4PI"}="1H10 4PI";
$resn_exch{"H102 4PI"}="2H10 4PI"; $resn_exch{"H111 4PI"}="1H11 4PI"; $resn_exch{"H112 4PI"}="2H11 4PI";
$resn_exch{"H271 4PI"}="1H27 4PI"; $resn_exch{"H272 4PI"}="2H27 4PI"; $resn_exch{"H273 4PI"}="3H27 4PI";
$resn_exch{"H371 4PI"}="1H37 4PI"; $resn_exch{"H372 4PI"}="2H37 4PI"; $resn_exch{" H81 4PN"}="1H8  4PN";
$resn_exch{" H82 4PN"}="2H8  4PN"; $resn_exch{" H91 4PN"}="1H9  4PN"; $resn_exch{" H92 4PN"}="2H9  4PN";
$resn_exch{"H101 4PN"}="1H10 4PN"; $resn_exch{"H102 4PN"}="2H10 4PN"; $resn_exch{"H111 4PN"}="1H11 4PN";
$resn_exch{"H112 4PN"}="2H11 4PN"; $resn_exch{"H121 4PN"}="1H12 4PN"; $resn_exch{"H122 4PN"}="2H12 4PN";
$resn_exch{" H51 4PN"}="1H5  4PN"; $resn_exch{" H52 4PN"}="2H5  4PN"; $resn_exch{" H61 4PN"}="1H6  4PN";
$resn_exch{" H62 4PN"}="2H6  4PN"; $resn_exch{" H21 4PN"}="1H2  4PN"; $resn_exch{" H22 4PN"}="2H2  4PN";
$resn_exch{" H31 4PN"}="1H3  4PN"; $resn_exch{" H32 4PN"}="2H3  4PN"; $resn_exch{" H21 4PP"}="1H2  4PP";
$resn_exch{" H22 4PP"}="2H2  4PP"; $resn_exch{" H31 4PP"}="1H3  4PP"; $resn_exch{" H32 4PP"}="2H3  4PP";
$resn_exch{" H41 4PP"}="1H4  4PP"; $resn_exch{" H42 4PP"}="2H4  4PP"; $resn_exch{"HN21 4PP"}="1HN2 4PP";
$resn_exch{"HN22 4PP"}="2HN2 4PP"; $resn_exch{" H11 4PR"}="1H1  4PR"; $resn_exch{" H12 4PR"}="2H1  4PR";
$resn_exch{" H13 4PR"}="3H1  4PR"; $resn_exch{"H21A 4PR"}="1H2  4PR"; $resn_exch{" H22 4PR"}="2H2  4PR";
$resn_exch{" H31 4PR"}="1H3  4PR"; $resn_exch{" H32 4PR"}="2H3  4PR"; $resn_exch{" H41 4PR"}="1H4  4PR";
$resn_exch{" H42 4PR"}="2H4  4PR"; $resn_exch{" H81 4PR"}="1H8  4PR"; $resn_exch{" H82 4PR"}="2H8  4PR";
$resn_exch{" H91 4PR"}="1H9  4PR"; $resn_exch{" H92 4PR"}="2H9  4PR"; $resn_exch{"H111 4PR"}="1H11 4PR";
$resn_exch{"H112 4PR"}="2H11 4PR"; $resn_exch{"H113 4PR"}="3H11 4PR"; $resn_exch{"H121 4PR"}="1H12 4PR";
$resn_exch{"H122 4PR"}="2H12 4PR"; $resn_exch{"H123 4PR"}="3H12 4PR"; $resn_exch{"H161 4PR"}="1H16 4PR";
$resn_exch{"H162 4PR"}="2H16 4PR"; $resn_exch{"H163 4PR"}="3H16 4PR"; $resn_exch{"H181 4PR"}="1H18 4PR";
$resn_exch{"H182 4PR"}="2H18 4PR"; $resn_exch{"H183 4PR"}="3H18 4PR"; $resn_exch{"H221 4PR"}="1H22 4PR";
$resn_exch{"H222 4PR"}="2H22 4PR"; $resn_exch{"H223 4PR"}="3H22 4PR"; $resn_exch{"H0E1 4PT"}="1H0E 4PT";
$resn_exch{"H0E2 4PT"}="2H0E 4PT"; $resn_exch{"H0B1 4PT"}="1H0B 4PT"; $resn_exch{"H0B2 4PT"}="2H0B 4PT";
$resn_exch{"H0N1 4PT"}="1H0N 4PT"; $resn_exch{"H0N2 4PT"}="2H0N 4PT"; $resn_exch{"H0P1 4PT"}="1H0P 4PT";
$resn_exch{"H0P2 4PT"}="2H0P 4PT"; $resn_exch{"H0Q1 4PT"}="1H0Q 4PT"; $resn_exch{"H0Q2 4PT"}="2H0Q 4PT";
$resn_exch{"H0Q3 4PT"}="3H0Q 4PT"; $resn_exch{"H0I1 4PT"}="1H0I 4PT"; $resn_exch{"H0I2 4PT"}="2H0I 4PT";
$resn_exch{"H0J1 4PT"}="1H0J 4PT"; $resn_exch{"H0J2 4PT"}="2H0J 4PT"; $resn_exch{"H0K1 4PT"}="1H0K 4PT";
$resn_exch{"H0K2 4PT"}="2H0K 4PT"; $resn_exch{"H0K3 4PT"}="3H0K 4PT"; $resn_exch{" H61 4PY"}="1H6  4PY";
$resn_exch{" H62 4PY"}="2H6  4PY"; $resn_exch{"H111 4PY"}="1H11 4PY"; $resn_exch{"H112 4PY"}="2H11 4PY";
$resn_exch{"H121 4PY"}="1H12 4PY"; $resn_exch{"H122 4PY"}="2H12 4PY"; $resn_exch{"H131 4PY"}="1H13 4PY";
$resn_exch{"H132 4PY"}="2H13 4PY"; $resn_exch{"H251 4PY"}="1H25 4PY"; $resn_exch{"H252 4PY"}="2H25 4PY";
$resn_exch{" H71 4QB"}="1H7  4QB"; $resn_exch{" H72 4QB"}="2H7  4QB"; $resn_exch{" H91 4QB"}="1H9  4QB";
$resn_exch{" H92 4QB"}="2H9  4QB"; $resn_exch{"H311 4QB"}="1H31 4QB"; $resn_exch{"H312 4QB"}="2H31 4QB";
$resn_exch{"H313 4QB"}="3H31 4QB"; $resn_exch{"H111 4QB"}="1H11 4QB"; $resn_exch{"H112 4QB"}="2H11 4QB";
$resn_exch{"H121 4QB"}="1H12 4QB"; $resn_exch{"H122 4QB"}="2H12 4QB"; $resn_exch{"H131 4QB"}="1H13 4QB";
$resn_exch{"H132 4QB"}="2H13 4QB"; $resn_exch{"H141 4QB"}="1H14 4QB"; $resn_exch{"H142 4QB"}="2H14 4QB";
$resn_exch{"H111 4QC"}="1H11 4QC"; $resn_exch{"H112 4QC"}="2H11 4QC"; $resn_exch{"H121 4QC"}="1H12 4QC";
$resn_exch{"H122 4QC"}="2H12 4QC"; $resn_exch{"H201 4QC"}="1H20 4QC"; $resn_exch{"H202 4QC"}="2H20 4QC";
$resn_exch{"H203 4QC"}="3H20 4QC"; $resn_exch{"H241 4QC"}="1H24 4QC"; $resn_exch{"H242 4QC"}="2H24 4QC";
$resn_exch{"H231 4QC"}="1H23 4QC"; $resn_exch{"H232 4QC"}="2H23 4QC"; $resn_exch{"H233 4QC"}="3H23 4QC";
$resn_exch{"HN11 4QC"}="1HN1 4QC"; $resn_exch{"HN12 4QC"}="2HN1 4QC"; $resn_exch{"H181 4RR"}="1H18 4RR";
$resn_exch{"H182 4RR"}="2H18 4RR"; $resn_exch{"H183 4RR"}="3H18 4RR"; $resn_exch{"H151 4RR"}="1H15 4RR";
$resn_exch{"H152 4RR"}="2H15 4RR"; $resn_exch{"H141 4RR"}="1H14 4RR"; $resn_exch{"H142 4RR"}="2H14 4RR";
$resn_exch{"H131 4RR"}="1H13 4RR"; $resn_exch{"H132 4RR"}="2H13 4RR"; $resn_exch{"H121 4RR"}="1H12 4RR";
$resn_exch{"H122 4RR"}="2H12 4RR"; $resn_exch{" H51 4RR"}="1H5  4RR"; $resn_exch{" H52 4RR"}="2H5  4RR";
$resn_exch{"H11A 4RR"}="1H1  4RR"; $resn_exch{" H12 4RR"}="2H1  4RR"; $resn_exch{" H5' 4SC"}="1H5* 4SC";
$resn_exch{"H5'' 4SC"}="2H5* 4SC"; $resn_exch{" H4' 4SC"}=" H4* 4SC"; $resn_exch{" H3' 4SC"}=" H3* 4SC";
$resn_exch{"HO3' 4SC"}="*HO3 4SC"; $resn_exch{" H2' 4SC"}="1H2* 4SC"; $resn_exch{"H2'' 4SC"}="2H2* 4SC";
$resn_exch{" H1' 4SC"}=" H1* 4SC"; $resn_exch{"HN41 4SC"}="1HN4 4SC"; $resn_exch{"HN42 4SC"}="2HN4 4SC";
$resn_exch{" H61 4SC"}="1H6  4SC"; $resn_exch{" H62 4SC"}="2H6  4SC"; $resn_exch{" H71 4SC"}="1H5M 4SC";
$resn_exch{" H72 4SC"}="2H5M 4SC"; $resn_exch{" H73 4SC"}="3H5M 4SC"; $resn_exch{"H101 4SP"}="1H10 4SP";
$resn_exch{"H102 4SP"}="2H10 4SP"; $resn_exch{"H121 4SP"}="1H12 4SP"; $resn_exch{"H122 4SP"}="2H12 4SP";
$resn_exch{"H131 4SP"}="1H13 4SP"; $resn_exch{"H132 4SP"}="2H13 4SP"; $resn_exch{"H141 4SP"}="1H14 4SP";
$resn_exch{"H142 4SP"}="2H14 4SP"; $resn_exch{"H151 4SP"}="1H15 4SP"; $resn_exch{"H152 4SP"}="2H15 4SP";
$resn_exch{"H161 4SP"}="1H16 4SP"; $resn_exch{"H162 4SP"}="2H16 4SP"; $resn_exch{"H261 4SP"}="1H26 4SP";
$resn_exch{"H262 4SP"}="2H26 4SP"; $resn_exch{"H161 4SR"}="1H16 4SR"; $resn_exch{"H162 4SR"}="2H16 4SR";
$resn_exch{"H121 4SR"}="1H12 4SR"; $resn_exch{"H122 4SR"}="2H12 4SR"; $resn_exch{"H201 4SR"}="1H20 4SR";
$resn_exch{"H202 4SR"}="2H20 4SR"; $resn_exch{"H203 4SR"}="3H20 4SR"; $resn_exch{" H51 4SR"}="1H5  4SR";
$resn_exch{" H52 4SR"}="2H5  4SR"; $resn_exch{" H61 4SR"}="1H6  4SR"; $resn_exch{" H62 4SR"}="2H6  4SR";
$resn_exch{" H71 4SR"}="1H7  4SR"; $resn_exch{" H72 4SR"}="2H7  4SR"; $resn_exch{" H81 4SR"}="1H8  4SR";
$resn_exch{" H82 4SR"}="2H8  4SR"; $resn_exch{"HAB1 4ST"}="1HAB 4ST"; $resn_exch{"HAB2 4ST"}="2HAB 4ST";
$resn_exch{"HAB3 4ST"}="3HAB 4ST"; $resn_exch{"HAA1 4ST"}="1HAA 4ST"; $resn_exch{"HAA2 4ST"}="2HAA 4ST";
$resn_exch{"HAA3 4ST"}="3HAA 4ST"; $resn_exch{"HAN1 4ST"}="1HAN 4ST"; $resn_exch{"HAN2 4ST"}="2HAN 4ST";
$resn_exch{"HAC1 4ST"}="1HAC 4ST"; $resn_exch{"HAC2 4ST"}="2HAC 4ST"; $resn_exch{"HAC3 4ST"}="3HAC 4ST";
$resn_exch{"HAL1 4ST"}="1HAL 4ST"; $resn_exch{"HAL2 4ST"}="2HAL 4ST"; $resn_exch{"HAJ1 4ST"}="1HAJ 4ST";
$resn_exch{"HAJ2 4ST"}="2HAJ 4ST"; $resn_exch{"HAI1 4ST"}="1HAI 4ST"; $resn_exch{"HAI2 4ST"}="2HAI 4ST";
$resn_exch{"HAK1 4ST"}="1HAK 4ST"; $resn_exch{"HAK2 4ST"}="2HAK 4ST"; $resn_exch{"HAM1 4ST"}="1HAM 4ST";
$resn_exch{"HAM2 4ST"}="2HAM 4ST"; $resn_exch{" H1' 4SU"}=" H1* 4SU"; $resn_exch{" H2' 4SU"}=" H2* 4SU";
$resn_exch{"HO2' 4SU"}="*HO2 4SU"; $resn_exch{" H3' 4SU"}=" H3* 4SU"; $resn_exch{" H4' 4SU"}=" H4* 4SU";
$resn_exch{"HO3' 4SU"}="*HO3 4SU"; $resn_exch{" H5' 4SU"}="1H5* 4SU"; $resn_exch{"H5'' 4SU"}="2H5* 4SU";
$resn_exch{"HOP2 4SU"}="2HOP 4SU"; $resn_exch{"HOP3 4SU"}="3HOP 4SU"; $resn_exch{"H5M1 4TA"}="1H5M 4TA";
$resn_exch{"H5M2 4TA"}="2H5M 4TA"; $resn_exch{"H5M3 4TA"}="3H5M 4TA"; $resn_exch{" H1B 4TA"}=" H1* 4TA";
$resn_exch{"H2'1 4TA"}="1H2* 4TA"; $resn_exch{"H2'2 4TA"}="2H2* 4TA"; $resn_exch{" H3B 4TA"}=" H3* 4TA";
$resn_exch{" H4B 4TA"}=" H4* 4TA"; $resn_exch{"H5'1 4TA"}="1H5* 4TA"; $resn_exch{"H5'2 4TA"}="2H5* 4TA";
$resn_exch{" H5B 4TA"}="1H5' 4TA"; $resn_exch{" H5D 4TA"}="2H5' 4TA"; $resn_exch{"H6A1 4TA"}="1H6A 4TA";
$resn_exch{"H6A2 4TA"}="2H6A 4TA"; $resn_exch{" H61 4TB"}="1H6  4TB"; $resn_exch{" H62 4TB"}="2H6  4TB";
$resn_exch{" H71 4TB"}="1H7  4TB"; $resn_exch{" H72 4TB"}="2H7  4TB"; $resn_exch{" H81 4TB"}="1H8  4TB";
$resn_exch{" H82 4TB"}="2H8  4TB"; $resn_exch{"H3B1 4TC"}="1H3B 4TC"; $resn_exch{"H3B2 4TC"}="2H3B 4TC";
$resn_exch{"H5U1 4TC"}="1H5U 4TC"; $resn_exch{"H5U2 4TC"}="2H5U 4TC"; $resn_exch{"H5'1 4TC"}="1H5* 4TC";
$resn_exch{"H5'2 4TC"}="2H5* 4TC"; $resn_exch{" H4' 4TC"}=" H4* 4TC"; $resn_exch{" H3' 4TC"}=" H3* 4TC";
$resn_exch{" H2' 4TC"}=" H2* 4TC"; $resn_exch{" H1' 4TC"}=" H1* 4TC"; $resn_exch{"H6N1 4TC"}="1H6N 4TC";
$resn_exch{"H6N2 4TC"}="2H6N 4TC"; $resn_exch{"HAP1 4TN"}="1HAP 4TN"; $resn_exch{"HAP2 4TN"}="2HAP 4TN";
$resn_exch{"HAA1 4TN"}="1HAA 4TN"; $resn_exch{"HAA2 4TN"}="2HAA 4TN"; $resn_exch{"HAA3 4TN"}="3HAA 4TN";
$resn_exch{"HAB1 4TN"}="1HAB 4TN"; $resn_exch{"HAB2 4TN"}="2HAB 4TN"; $resn_exch{"HAB3 4TN"}="3HAB 4TN";
$resn_exch{"HAC1 4TN"}="1HAC 4TN"; $resn_exch{"HAC2 4TN"}="2HAC 4TN"; $resn_exch{"HAC3 4TN"}="3HAC 4TN";
$resn_exch{"HAQ1 4TN"}="1HAQ 4TN"; $resn_exch{"HAQ2 4TN"}="2HAQ 4TN"; $resn_exch{"HAR1 4TN"}="1HAR 4TN";
$resn_exch{"HAR2 4TN"}="2HAR 4TN"; $resn_exch{"HAD1 4TN"}="1HAD 4TN"; $resn_exch{"HAD2 4TN"}="2HAD 4TN";
$resn_exch{"HAD3 4TN"}="3HAD 4TN"; $resn_exch{"HAE1 4TN"}="1HAE 4TN"; $resn_exch{"HAE2 4TN"}="2HAE 4TN";
$resn_exch{"HAE3 4TN"}="3HAE 4TN"; $resn_exch{" HN2 4TP"}="2HN  4TP"; $resn_exch{"HG21 4TP"}="1HG2 4TP";
$resn_exch{"HG22 4TP"}="2HG2 4TP"; $resn_exch{"HOP2 4TP"}="2HOP 4TP"; $resn_exch{"HOP3 4TP"}="3HOP 4TP";
$resn_exch{"HN11 4TR"}="1HN1 4TR"; $resn_exch{"HN12 4TR"}="2HN1 4TR"; $resn_exch{" H71 4TR"}="1H7  4TR";
$resn_exch{" H72 4TR"}="2H7  4TR"; $resn_exch{"HN11 4TZ"}="1HN1 4TZ"; $resn_exch{"HN12 4TZ"}="2HN1 4TZ";
$resn_exch{"H101 4TZ"}="1H10 4TZ"; $resn_exch{"H102 4TZ"}="2H10 4TZ"; $resn_exch{"H491 4UN"}="1H49 4UN";
$resn_exch{"H492 4UN"}="2H49 4UN"; $resn_exch{"H281 4UN"}="1H28 4UN"; $resn_exch{"H282 4UN"}="2H28 4UN";
$resn_exch{"H283 4UN"}="3H28 4UN"; $resn_exch{"H241 4UN"}="1H24 4UN"; $resn_exch{"H242 4UN"}="2H24 4UN";
$resn_exch{"H243 4UN"}="3H24 4UN"; $resn_exch{"H191 4UN"}="1H19 4UN"; $resn_exch{"H192 4UN"}="2H19 4UN";
$resn_exch{"H221 501"}="1H22 501"; $resn_exch{"H222 501"}="2H22 501"; $resn_exch{"H231 501"}="1H23 501";
$resn_exch{"H232 501"}="2H23 501"; $resn_exch{"H261 501"}="1H26 501"; $resn_exch{"H262 501"}="2H26 501";
$resn_exch{"H291 501"}="1H29 501"; $resn_exch{"H292 501"}="2H29 501"; $resn_exch{"H293 501"}="3H29 501";
$resn_exch{"H371 501"}="1H37 501"; $resn_exch{"H372 501"}="2H37 501"; $resn_exch{"H373 501"}="3H37 501";
$resn_exch{"H421 501"}="1H42 501"; $resn_exch{"H422 501"}="2H42 501"; $resn_exch{"H423 501"}="3H42 501";
$resn_exch{"H1C1 505"}="1H1C 505"; $resn_exch{"H1C2 505"}="2H1C 505"; $resn_exch{"H1C3 505"}="3H1C 505";
$resn_exch{"H3N1 505"}="1H3N 505"; $resn_exch{"H3N2 505"}="2H3N 505"; $resn_exch{"H4C1 505"}="1H4C 505";
$resn_exch{"H4C2 505"}="2H4C 505"; $resn_exch{"H8C1 505"}="1H8C 505"; $resn_exch{"H8C2 505"}="2H8C 505";
$resn_exch{"H8C3 505"}="3H8C 505"; $resn_exch{"H131 505"}="1H13 505"; $resn_exch{"H132 505"}="2H13 505";
$resn_exch{"H133 505"}="3H13 505"; $resn_exch{"H121 509"}="1H12 509"; $resn_exch{"H122 509"}="2H12 509";
$resn_exch{" H91 512"}="1H9  512"; $resn_exch{" H92 512"}="2H9  512"; $resn_exch{"H181 514"}="1H18 514";
$resn_exch{"H182 514"}="2H18 514"; $resn_exch{"H201 514"}="1H20 514"; $resn_exch{"H202 514"}="2H20 514";
$resn_exch{"H203 514"}="3H20 514"; $resn_exch{" H31 515"}="1H3  515"; $resn_exch{" H41 515"}="1H4  515";
$resn_exch{" H91 515"}="1H9  515"; $resn_exch{"H121 515"}="1H12 515"; $resn_exch{"H122 515"}="2H12 515";
$resn_exch{"H131 515"}="1H13 515"; $resn_exch{"H132 515"}="2H13 515"; $resn_exch{"H141 515"}="1H14 515";
$resn_exch{"H142 515"}="2H14 515"; $resn_exch{"H151 515"}="1H15 515"; $resn_exch{"H152 515"}="2H15 515";
$resn_exch{"H231 515"}="1H23 515"; $resn_exch{"H232 515"}="2H23 515"; $resn_exch{"H233 515"}="3H23 515";
$resn_exch{" H71 515"}="1H7  515"; $resn_exch{" H72 515"}="2H7  515"; $resn_exch{"H271 515"}="1H27 515";
$resn_exch{"H281 515"}="1H28 515"; $resn_exch{"H291 515"}="1H29 515"; $resn_exch{"H301 515"}="1H30 515";
$resn_exch{"H331 515"}="1H33 515"; $resn_exch{"H361 515"}="1H36 515"; $resn_exch{"H161 515"}="1H16 515";
$resn_exch{"H171 515"}="1H17 515"; $resn_exch{"H181 515"}="1H18 515"; $resn_exch{"H191 515"}="1H19 515";
$resn_exch{"H201 515"}="1H20 515"; $resn_exch{"H202 515"}="2H20 515"; $resn_exch{" H21 515"}="1H2  515";
$resn_exch{"H411 515"}="1H41 515"; $resn_exch{"H421 515"}="1H42 515"; $resn_exch{"H431 515"}="1H43 515";
$resn_exch{"H441 515"}="1H44 515"; $resn_exch{"H201 517"}="1H20 517"; $resn_exch{"H202 517"}="2H20 517";
$resn_exch{"H203 517"}="3H20 517"; $resn_exch{"H161 517"}="1H16 517"; $resn_exch{"H162 517"}="2H16 517";
$resn_exch{"H111 521"}="1H11 521"; $resn_exch{"H112 521"}="2H11 521"; $resn_exch{"H5A1 523"}="1H5A 523";
$resn_exch{"H5A2 523"}="2H5A 523"; $resn_exch{"H5A3 523"}="3H5A 523"; $resn_exch{"HN41 523"}="1HN4 523";
$resn_exch{"HN42 523"}="2HN4 523"; $resn_exch{" H1' 523"}=" H1* 523"; $resn_exch{"H2'1 523"}="1H2* 523";
$resn_exch{"H2'2 523"}="2H2* 523"; $resn_exch{" H3' 523"}=" H3* 523"; $resn_exch{" H4' 523"}=" H4* 523";
$resn_exch{"HO3' 523"}="HO3* 523"; $resn_exch{"H5'1 523"}="1H5* 523"; $resn_exch{"H5'2 523"}="2H5* 523";
$resn_exch{"H191 524"}="1H19 524"; $resn_exch{"H192 524"}="2H19 524"; $resn_exch{"H341 524"}="1H34 524";
$resn_exch{"H342 524"}="2H34 524"; $resn_exch{"H331 524"}="1H33 524"; $resn_exch{"H332 524"}="2H33 524";
$resn_exch{"H311 524"}="1H31 524"; $resn_exch{"H312 524"}="2H31 524"; $resn_exch{"H231 524"}="1H23 524";
$resn_exch{"H232 524"}="2H23 524"; $resn_exch{"H233 524"}="3H23 524"; $resn_exch{"H271 524"}="1H27 524";
$resn_exch{"H272 524"}="2H27 524"; $resn_exch{"H273 524"}="3H27 524"; $resn_exch{"H451 524"}="1H45 524";
$resn_exch{"H452 524"}="2H45 524"; $resn_exch{"H441 524"}="1H44 524"; $resn_exch{"H442 524"}="2H44 524";
$resn_exch{"H481 524"}="1H48 524"; $resn_exch{"H482 524"}="2H48 524"; $resn_exch{"H471 524"}="1H47 524";
$resn_exch{"H472 524"}="2H47 524"; $resn_exch{"H211 527"}="1H21 527"; $resn_exch{"H212 527"}="2H21 527";
$resn_exch{"H311 527"}="1H31 527"; $resn_exch{"H312 527"}="2H31 527"; $resn_exch{"H321 527"}="1H32 527";
$resn_exch{"H322 527"}="2H32 527"; $resn_exch{"H341 527"}="1H34 527"; $resn_exch{"H342 527"}="2H34 527";
$resn_exch{"H351 527"}="1H35 527"; $resn_exch{"H352 527"}="2H35 527"; $resn_exch{"H491 527"}="1H49 527";
$resn_exch{"H492 527"}="2H49 527"; $resn_exch{"H351 528"}="1H35 528"; $resn_exch{"H352 528"}="2H35 528";
$resn_exch{"H341 528"}="1H34 528"; $resn_exch{"H342 528"}="2H34 528"; $resn_exch{"H331 528"}="1H33 528";
$resn_exch{"H332 528"}="2H33 528"; $resn_exch{"H301 528"}="1H30 528"; $resn_exch{"H302 528"}="2H30 528";
$resn_exch{"H303 528"}="3H30 528"; $resn_exch{"H291 528"}="1H29 528"; $resn_exch{"H292 528"}="2H29 528";
$resn_exch{"H293 528"}="3H29 528"; $resn_exch{"H151 528"}="1H15 528"; $resn_exch{"H152 528"}="2H15 528";
$resn_exch{" H91 528"}="1H9  528"; $resn_exch{" H92 528"}="2H9  528"; $resn_exch{"H101 528"}="1H10 528";
$resn_exch{"H102 528"}="2H10 528"; $resn_exch{"H121 529"}="1H12 529"; $resn_exch{"H122 529"}="2H12 529";
$resn_exch{"H131 529"}="1H13 529"; $resn_exch{"H132 529"}="2H13 529"; $resn_exch{"H251 529"}="1H25 529";
$resn_exch{"H252 529"}="2H25 529"; $resn_exch{"H253 529"}="3H25 529"; $resn_exch{" HN1 52A"}="1HN  52A";
$resn_exch{" HN2 52A"}="2HN  52A"; $resn_exch{"HB11 52A"}="1HB1 52A"; $resn_exch{"HB12 52A"}="2HB1 52A";
$resn_exch{"HB21 52A"}="1HB2 52A"; $resn_exch{"HB22 52A"}="2HB2 52A"; $resn_exch{" H41 541"}="1H4  541";
$resn_exch{" H42 541"}="2H4  541"; $resn_exch{" H43 541"}="3H4  541"; $resn_exch{" H81 541"}="1H8  541";
$resn_exch{" H82 541"}="2H8  541"; $resn_exch{"H151 541"}="1H15 541"; $resn_exch{"H152 541"}="2H15 541";
$resn_exch{"H171 541"}="1H17 541"; $resn_exch{"H172 541"}="2H17 541"; $resn_exch{"H173 541"}="3H17 541";
$resn_exch{"H191 541"}="1H19 541"; $resn_exch{"H192 541"}="2H19 541"; $resn_exch{"H251 541"}="1H25 541";
$resn_exch{"H252 541"}="2H25 541"; $resn_exch{"H253 541"}="3H25 541"; $resn_exch{"HO11 543"}="1HO1 543";
$resn_exch{"HO12 543"}="2HO1 543"; $resn_exch{"HO21 543"}="1HO2 543"; $resn_exch{"HO22 543"}="2HO2 543";
$resn_exch{"HO31 543"}="1HO3 543"; $resn_exch{"HO32 543"}="2HO3 543"; $resn_exch{"HO41 543"}="1HO4 543";
$resn_exch{"HO42 543"}="2HO4 543"; $resn_exch{"HO51 543"}="1HO5 543"; $resn_exch{"HO52 543"}="2HO5 543";
$resn_exch{"HO61 543"}="1HO6 543"; $resn_exch{"HO62 543"}="2HO6 543"; $resn_exch{" H11 543"}="1H1  543";
$resn_exch{" H12 543"}="2H1  543"; $resn_exch{" H21 543"}="1H2  543"; $resn_exch{" H22 543"}="2H2  543";
$resn_exch{" H23 543"}="3H2  543"; $resn_exch{" HA1 544"}="1HA  544"; $resn_exch{" HB1 544"}="1HB  544";
$resn_exch{" HB2 544"}="2HB  544"; $resn_exch{"HD11 544"}="1HD1 544"; $resn_exch{"HD21 544"}="1HD2 544";
$resn_exch{"HE11 544"}="1HE1 544"; $resn_exch{"HE21 544"}="1HE2 544"; $resn_exch{"H1F1 544"}="1H1F 544";
$resn_exch{"H1B1 544"}="1H1B 544"; $resn_exch{"H1B2 544"}="2H1B 544"; $resn_exch{"H1B3 544"}="3H1B 544";
$resn_exch{"H1I1 544"}="1H1I 544"; $resn_exch{"H1J1 544"}="1H1J 544"; $resn_exch{"H1K1 544"}="1H1K 544";
$resn_exch{"H1L1 544"}="1H1L 544"; $resn_exch{"H1M1 544"}="1H1M 544"; $resn_exch{"H3E1 544"}="1H3E 544";
$resn_exch{"H3E2 544"}="2H3E 544"; $resn_exch{"H3E3 544"}="3H3E 544"; $resn_exch{"H3N1 544"}="1H3N 544";
$resn_exch{"H3M1 544"}="1H3M 544"; $resn_exch{"H3L1 544"}="1H3L 544"; $resn_exch{"H3K1 544"}="1H3K 544";
$resn_exch{"H3J1 544"}="1H3J 544"; $resn_exch{"H3A1 544"}="1H3A 544"; $resn_exch{"H3A2 544"}="2H3A 544";
$resn_exch{"H3B1 544"}="1H3B 544"; $resn_exch{"H3B2 544"}="2H3B 544"; $resn_exch{"HO11 544"}="1HO1 544";
$resn_exch{"H141 545"}="1H14 545"; $resn_exch{"H142 545"}="2H14 545"; $resn_exch{"H241 545"}="1H24 545";
$resn_exch{"H242 545"}="2H24 545"; $resn_exch{"H251 545"}="1H25 545"; $resn_exch{"H252 545"}="2H25 545";
$resn_exch{"H261 545"}="1H26 545"; $resn_exch{"H262 545"}="2H26 545"; $resn_exch{"H271 545"}="1H27 545";
$resn_exch{"H272 545"}="2H27 545"; $resn_exch{"H273 545"}="3H27 545"; $resn_exch{"H161 545"}="1H16 545";
$resn_exch{"H162 545"}="2H16 545"; $resn_exch{" H41 545"}="1H4  545"; $resn_exch{" H42 545"}="2H4  545";
$resn_exch{"H301 545"}="1H30 545"; $resn_exch{"H302 545"}="2H30 545"; $resn_exch{"H351 547"}="1H35 547";
$resn_exch{"H352 547"}="2H35 547"; $resn_exch{"H361 547"}="1H36 547"; $resn_exch{"H362 547"}="2H36 547";
$resn_exch{"H421 547"}="1H42 547"; $resn_exch{"H422 547"}="2H42 547"; $resn_exch{"H431 547"}="1H43 547";
$resn_exch{"H432 547"}="2H43 547"; $resn_exch{"H451 547"}="1H45 547"; $resn_exch{"H452 547"}="2H45 547";
$resn_exch{"H461 547"}="1H46 547"; $resn_exch{"H462 547"}="2H46 547"; $resn_exch{"H101 561"}="1H10 561";
$resn_exch{"H102 561"}="2H10 561"; $resn_exch{"H311 561"}="1H31 561"; $resn_exch{"H312 561"}="2H31 561";
$resn_exch{"H321 561"}="1H32 561"; $resn_exch{"H322 561"}="2H32 561"; $resn_exch{"H331 561"}="1H33 561";
$resn_exch{"H332 561"}="2H33 561"; $resn_exch{"H171 564"}="1H17 564"; $resn_exch{"H172 564"}="2H17 564";
$resn_exch{"H181 564"}="1H18 564"; $resn_exch{"H182 564"}="2H18 564"; $resn_exch{"H231 564"}="1H23 564";
$resn_exch{"H232 564"}="2H23 564"; $resn_exch{"H233 564"}="3H23 564"; $resn_exch{"H241 564"}="1H24 564";
$resn_exch{"H242 564"}="2H24 564"; $resn_exch{"H243 564"}="3H24 564"; $resn_exch{"H251 564"}="1H25 564";
$resn_exch{"H252 564"}="2H25 564"; $resn_exch{"H253 564"}="3H25 564"; $resn_exch{"H261 564"}="1H26 564";
$resn_exch{"H262 564"}="2H26 564"; $resn_exch{"H263 564"}="3H26 564"; $resn_exch{"H111 565"}="1H11 565";
$resn_exch{"H112 565"}="2H11 565"; $resn_exch{"H151 565"}="1H15 565"; $resn_exch{"H152 565"}="2H15 565";
$resn_exch{"H201 565"}="1H20 565"; $resn_exch{"H202 565"}="2H20 565"; $resn_exch{"H251 565"}="1H25 565";
$resn_exch{"H252 565"}="2H25 565"; $resn_exch{"H261 565"}="1H26 565"; $resn_exch{"H262 565"}="2H26 565";
$resn_exch{"H271 565"}="1H27 565"; $resn_exch{"H272 565"}="2H27 565"; $resn_exch{"H371 565"}="1H37 565";
$resn_exch{"H372 565"}="2H37 565"; $resn_exch{"H373 565"}="3H37 565"; $resn_exch{" H11 566"}="1H1  566";
$resn_exch{" H12 566"}="2H1  566"; $resn_exch{" H21 566"}="1H2  566"; $resn_exch{" H22 566"}="2H2  566";
$resn_exch{" H41 566"}="1H4  566"; $resn_exch{" H42 566"}="2H4  566"; $resn_exch{" H51 566"}="1H5  566";
$resn_exch{" H52 566"}="2H5  566"; $resn_exch{" H61 566"}="1H6  566"; $resn_exch{" H62 566"}="2H6  566";
$resn_exch{"H161 566"}="1H16 566"; $resn_exch{"H162 566"}="2H16 566"; $resn_exch{"H181 566"}="1H18 566";
$resn_exch{"H182 566"}="2H18 566"; $resn_exch{"H2C1 568"}="1H2C 568"; $resn_exch{"H2C2 568"}="2H2C 568";
$resn_exch{"H4C1 568"}="1H4C 568"; $resn_exch{"H4C2 568"}="2H4C 568"; $resn_exch{"H6C1 568"}="1H6C 568";
$resn_exch{"H6C2 568"}="2H6C 568"; $resn_exch{"H131 568"}="1H13 568"; $resn_exch{"H132 568"}="2H13 568";
$resn_exch{"H291 568"}="1H29 568"; $resn_exch{"H292 568"}="2H29 568"; $resn_exch{"H391 568"}="1H39 568";
$resn_exch{"H392 568"}="2H39 568"; $resn_exch{"H3E1 570"}="1H3E 570"; $resn_exch{"H3E2 570"}="2H3E 570";
$resn_exch{"H3E3 570"}="3H3E 570"; $resn_exch{"H3A1 570"}="1H3A 570"; $resn_exch{"H3A2 570"}="2H3A 570";
$resn_exch{"H3B1 570"}="1H3B 570"; $resn_exch{"H3B2 570"}="2H3B 570"; $resn_exch{"H141 572"}="1H14 572";
$resn_exch{"H142 572"}="2H14 572"; $resn_exch{"H191 572"}="1H19 572"; $resn_exch{"H192 572"}="2H19 572";
$resn_exch{"H151 572"}="1H15 572"; $resn_exch{"H152 572"}="2H15 572"; $resn_exch{"H131 572"}="1H13 572";
$resn_exch{"H132 572"}="2H13 572"; $resn_exch{"H133 572"}="3H13 572"; $resn_exch{"H111 572"}="1H11 572";
$resn_exch{"H112 572"}="2H11 572"; $resn_exch{"H113 572"}="3H11 572"; $resn_exch{"H161 572"}="1H16 572";
$resn_exch{"H162 572"}="2H16 572"; $resn_exch{"H181 572"}="1H18 572"; $resn_exch{"H182 572"}="2H18 572";
$resn_exch{"H211 577"}="1H21 577"; $resn_exch{"H212 577"}="2H21 577"; $resn_exch{"H213 577"}="3H21 577";
$resn_exch{"H231 578"}="1H23 578"; $resn_exch{"H232 578"}="2H23 578"; $resn_exch{"H181 578"}="1H18 578";
$resn_exch{"H182 578"}="2H18 578"; $resn_exch{"H183 578"}="3H18 578"; $resn_exch{"H171 578"}="1H17 578";
$resn_exch{"H172 578"}="2H17 578"; $resn_exch{"H173 578"}="3H17 578"; $resn_exch{" H91 578"}="1H9  578";
$resn_exch{" H92 578"}="2H9  578"; $resn_exch{"H101 578"}="1H10 578"; $resn_exch{"H102 578"}="2H10 578";
$resn_exch{" H61 580"}="1H6  580"; $resn_exch{" H62 580"}="2H6  580"; $resn_exch{" H71 580"}="1H7  580";
$resn_exch{" H72 580"}="2H7  580"; $resn_exch{" H81 580"}="1H8  580"; $resn_exch{" H82 580"}="2H8  580";
$resn_exch{"H351 580"}="1H35 580"; $resn_exch{"H352 580"}="2H35 580"; $resn_exch{"H353 580"}="3H35 580";
$resn_exch{"H181 582"}="1H18 582"; $resn_exch{"H182 582"}="2H18 582"; $resn_exch{" H91 582"}="1H9  582";
$resn_exch{" H92 582"}="2H9  582"; $resn_exch{" H93 582"}="3H9  582"; $resn_exch{" H71 582"}="1H7  582";
$resn_exch{" H72 582"}="2H7  582"; $resn_exch{" H73 582"}="3H7  582"; $resn_exch{"H31A 587"}="1H3  587";
$resn_exch{"H32A 587"}="2H3  587"; $resn_exch{" H41 587"}="1H4  587"; $resn_exch{" H42 587"}="2H4  587";
$resn_exch{" H51 587"}="1H5  587"; $resn_exch{" H52 587"}="2H5  587"; $resn_exch{" H61 587"}="1H6  587";
$resn_exch{"H62A 587"}="2H6  587"; $resn_exch{"H121 587"}="1H12 587"; $resn_exch{"H122 587"}="2H12 587";
$resn_exch{"H161 587"}="1H16 587"; $resn_exch{"H162 587"}="2H16 587"; $resn_exch{"H231 587"}="1H23 587";
$resn_exch{"H232 587"}="2H23 587"; $resn_exch{"H301 587"}="1H30 587"; $resn_exch{"H302 587"}="2H30 587";
$resn_exch{"H331 587"}="1H33 587"; $resn_exch{"H332 587"}="2H33 587"; $resn_exch{"H341 587"}="1H34 587";
$resn_exch{"H342 587"}="2H34 587"; $resn_exch{"H351 587"}="1H35 587"; $resn_exch{"H352 587"}="2H35 587";
$resn_exch{"H353 587"}="3H35 587"; $resn_exch{"H361 587"}="1H36 587"; $resn_exch{"H362 587"}="2H36 587";
$resn_exch{"H363 587"}="3H36 587"; $resn_exch{"H371 587"}="1H37 587"; $resn_exch{"H372 587"}="2H37 587";
$resn_exch{"H373 587"}="3H37 587"; $resn_exch{"H381 587"}="1H38 587"; $resn_exch{"H382 587"}="2H38 587";
$resn_exch{"H391 587"}="1H39 587"; $resn_exch{"H392 587"}="2H39 587"; $resn_exch{"H393 587"}="3H39 587";
$resn_exch{"H411 587"}="1H41 587"; $resn_exch{"H412 587"}="2H41 587"; $resn_exch{"H413 587"}="3H41 587";
$resn_exch{"H421 587"}="1H42 587"; $resn_exch{"H422 587"}="2H42 587"; $resn_exch{"H423 587"}="3H42 587";
$resn_exch{"H431 587"}="1H43 587"; $resn_exch{"H432 587"}="2H43 587"; $resn_exch{"H433 587"}="3H43 587";
$resn_exch{"H441 587"}="1H44 587"; $resn_exch{"H442 587"}="2H44 587"; $resn_exch{"H443 587"}="3H44 587";
$resn_exch{"H451 587"}="1H45 587"; $resn_exch{"H452 587"}="2H45 587"; $resn_exch{"H453 587"}="3H45 587";
$resn_exch{"H671 587"}="1H67 587"; $resn_exch{"H672 587"}="2H67 587"; $resn_exch{"H673 587"}="3H67 587";
$resn_exch{"HO10 587"}="0HO1 587"; $resn_exch{"HO13 587"}="3HO1 587"; $resn_exch{"HN31 588"}="1HN3 588";
$resn_exch{"HN32 588"}="2HN3 588"; $resn_exch{" H71 588"}="1H7  588"; $resn_exch{" H72 588"}="2H7  588";
$resn_exch{"H201 588"}="1H20 588"; $resn_exch{"H202 588"}="2H20 588"; $resn_exch{"H203 588"}="3H20 588";
$resn_exch{"H121 588"}="1H12 588"; $resn_exch{"H122 588"}="2H12 588"; $resn_exch{"H131 588"}="1H13 588";
$resn_exch{"H132 588"}="2H13 588"; $resn_exch{"H151 588"}="1H15 588"; $resn_exch{"H152 588"}="2H15 588";
$resn_exch{"H141 588"}="1H14 588"; $resn_exch{"H142 588"}="2H14 588"; $resn_exch{"H161 588"}="1H16 588";
$resn_exch{"H162 588"}="2H16 588"; $resn_exch{"H163 588"}="3H16 588"; $resn_exch{"H101 598"}="1H10 598";
$resn_exch{"H102 598"}="2H10 598"; $resn_exch{"H301 598"}="1H30 598"; $resn_exch{"H302 598"}="2H30 598";
$resn_exch{"H311 598"}="1H31 598"; $resn_exch{"H312 598"}="2H31 598"; $resn_exch{"H331 598"}="1H33 598";
$resn_exch{"H332 598"}="2H33 598"; $resn_exch{"H341 598"}="1H34 598"; $resn_exch{"H342 598"}="2H34 598";
$resn_exch{"H451 598"}="1H45 598"; $resn_exch{"H452 598"}="2H45 598"; $resn_exch{"H121 59A"}="1H12 59A";
$resn_exch{"H122 59A"}="2H12 59A"; $resn_exch{"H111 59A"}="1H11 59A"; $resn_exch{"H112 59A"}="2H11 59A";
$resn_exch{"H113 59A"}="3H11 59A"; $resn_exch{"H131 59A"}="1H13 59A"; $resn_exch{"H132 59A"}="2H13 59A";
$resn_exch{"H133 59A"}="3H13 59A"; $resn_exch{" H81 59A"}="1H8  59A"; $resn_exch{" H82 59A"}="2H8  59A";
$resn_exch{" H91 59A"}="1H9  59A"; $resn_exch{" H92 59A"}="2H9  59A"; $resn_exch{" H93 59A"}="3H9  59A";
$resn_exch{"HOP3 5AA"}="3HOP 5AA"; $resn_exch{"HOP2 5AA"}="2HOP 5AA"; $resn_exch{" H5' 5AA"}="1H5* 5AA";
$resn_exch{"H5'' 5AA"}="2H5* 5AA"; $resn_exch{" H4' 5AA"}=" H4* 5AA"; $resn_exch{" H1' 5AA"}=" H1* 5AA";
$resn_exch{" H91 5AA"}="1H9  5AA"; $resn_exch{" H92 5AA"}="2H9  5AA"; $resn_exch{" H93 5AA"}="3H9  5AA";
$resn_exch{"H101 5AA"}="1H10 5AA"; $resn_exch{"H102 5AA"}="2H10 5AA"; $resn_exch{"H103 5AA"}="3H10 5AA";
$resn_exch{" H2' 5AA"}=" H2* 5AA"; $resn_exch{" H3' 5AA"}=" H3* 5AA"; $resn_exch{"H3'1 5AA"}="1H3* 5AA";
$resn_exch{"H3'2 5AA"}="2H3* 5AA"; $resn_exch{"HN61 5AD"}="1HN6 5AD"; $resn_exch{"HN62 5AD"}="2HN6 5AD";
$resn_exch{"H5'1 5AD"}="1H5' 5AD"; $resn_exch{"H5'2 5AD"}="2H5' 5AD"; $resn_exch{"H5'3 5AD"}="3H5' 5AD";
$resn_exch{"HO2' 5AD"}="'HO2 5AD"; $resn_exch{"HO3' 5AD"}="'HO3 5AD"; $resn_exch{" H11 5AF"}="1H1  5AF";
$resn_exch{" H13 5AF"}="3H1  5AF"; $resn_exch{" H12 5AF"}="2H1  5AF"; $resn_exch{" H21 5AF"}="1H2  5AF";
$resn_exch{" H22 5AF"}="2H2  5AF"; $resn_exch{" H31 5AF"}="1H3  5AF"; $resn_exch{" H42 5AF"}="2H4  5AF";
$resn_exch{" H52 5AF"}="2H5  5AF"; $resn_exch{" H53 5AF"}="3H5  5AF"; $resn_exch{" H71 5AF"}="1H7  5AF";
$resn_exch{" H62 5AF"}="2H6  5AF"; $resn_exch{" H63 5AF"}="3H6  5AF"; $resn_exch{"H11A 5AM"}="1H1  5AM";
$resn_exch{"H12A 5AM"}="2H1  5AM"; $resn_exch{"H151 5AM"}="1H15 5AM"; $resn_exch{"H152 5AM"}="2H15 5AM";
$resn_exch{"H161 5AM"}="1H16 5AM"; $resn_exch{"H162 5AM"}="2H16 5AM"; $resn_exch{"H271 5AM"}="1H27 5AM";
$resn_exch{"H272 5AM"}="2H27 5AM"; $resn_exch{"H301 5AM"}="1H30 5AM"; $resn_exch{"H302 5AM"}="2H30 5AM";
$resn_exch{"H311 5AM"}="1H31 5AM"; $resn_exch{"H312 5AM"}="2H31 5AM"; $resn_exch{"H331 5AM"}="1H33 5AM";
$resn_exch{"H332 5AM"}="2H33 5AM"; $resn_exch{"H351 5AM"}="1H35 5AM"; $resn_exch{"H352 5AM"}="2H35 5AM";
$resn_exch{"H171 5AM"}="1H17 5AM"; $resn_exch{"H172 5AM"}="2H17 5AM"; $resn_exch{"H181 5AM"}="1H18 5AM";
$resn_exch{"H182 5AM"}="2H18 5AM"; $resn_exch{"H191 5AM"}="1H19 5AM"; $resn_exch{"H192 5AM"}="2H19 5AM";
$resn_exch{"H251 5AM"}="1H25 5AM"; $resn_exch{"H252 5AM"}="2H25 5AM"; $resn_exch{"H281 5AM"}="1H28 5AM";
$resn_exch{"H282 5AM"}="2H28 5AM"; $resn_exch{"H361 5AM"}="1H36 5AM"; $resn_exch{"H362 5AM"}="2H36 5AM";
$resn_exch{" HN1 5AN"}="1HN  5AN"; $resn_exch{" HN2 5AN"}="2HN  5AN"; $resn_exch{"H201 5AP"}="1H20 5AP";
$resn_exch{"H202 5AP"}="2H20 5AP"; $resn_exch{"H211 5AP"}="1H21 5AP"; $resn_exch{"H212 5AP"}="2H21 5AP";
$resn_exch{"H231 5AP"}="1H23 5AP"; $resn_exch{"H232 5AP"}="2H23 5AP"; $resn_exch{"H281 5AP"}="1H28 5AP";
$resn_exch{"H282 5AP"}="2H28 5AP"; $resn_exch{"H283 5AP"}="3H28 5AP"; $resn_exch{"H291 5AP"}="1H29 5AP";
$resn_exch{"H292 5AP"}="2H29 5AP"; $resn_exch{"H293 5AP"}="3H29 5AP"; $resn_exch{"H5'1 5AS"}="1H5* 5AS";
$resn_exch{"H5'2 5AS"}="2H5* 5AS"; $resn_exch{" H4' 5AS"}=" H4* 5AS"; $resn_exch{" H3' 5AS"}=" H3* 5AS";
$resn_exch{"HO'3 5AS"}="3HO* 5AS"; $resn_exch{" H2' 5AS"}=" H2* 5AS"; $resn_exch{"HO'2 5AS"}="2HO* 5AS";
$resn_exch{" H1' 5AS"}=" H1* 5AS"; $resn_exch{" H61 5AS"}="1H6  5AS"; $resn_exch{" H62 5AS"}="2H6  5AS";
$resn_exch{"H101 5AS"}="1H10 5AS"; $resn_exch{"H102 5AS"}="2H10 5AS"; $resn_exch{"H111 5AS"}="1H11 5AS";
$resn_exch{"H112 5AS"}="2H11 5AS"; $resn_exch{"H113 5AS"}="3H11 5AS"; $resn_exch{"HN51 5AT"}="1HN5 5AT";
$resn_exch{"HN52 5AT"}="2HN5 5AT"; $resn_exch{" H2' 5AT"}="1H2* 5AT"; $resn_exch{"H2'' 5AT"}="2H2* 5AT";
$resn_exch{" H5' 5AT"}="1H5* 5AT"; $resn_exch{"H5'' 5AT"}="2H5* 5AT"; $resn_exch{" H4' 5AT"}=" H4* 5AT";
$resn_exch{" H1' 5AT"}=" H1* 5AT"; $resn_exch{" H3' 5AT"}=" H3* 5AT"; $resn_exch{"HO3' 5AT"}="H3T  5AT";
$resn_exch{" H61 5AX"}="1H6  5AX"; $resn_exch{" H62 5AX"}="2H6  5AX"; $resn_exch{" H81 5AX"}="1H8  5AX";
$resn_exch{" H82 5AX"}="2H8  5AX"; $resn_exch{" H83 5AX"}="3H8  5AX"; $resn_exch{" H11 5AX"}="1H1  5AX";
$resn_exch{" H12 5AX"}="2H1  5AX"; $resn_exch{"H221 5BN"}="1H22 5BN"; $resn_exch{"H222 5BN"}="2H22 5BN";
$resn_exch{"H211 5BN"}="1H21 5BN"; $resn_exch{"H212 5BN"}="2H21 5BN"; $resn_exch{"H201 5BN"}="1H20 5BN";
$resn_exch{"H202 5BN"}="2H20 5BN"; $resn_exch{" H11 5BR"}="1H1  5BR"; $resn_exch{" H12 5BR"}="2H1  5BR";
$resn_exch{" H21 5BR"}="1H2  5BR"; $resn_exch{" H22 5BR"}="2H2  5BR"; $resn_exch{" H31 5BR"}="1H3  5BR";
$resn_exch{" H32 5BR"}="2H3  5BR"; $resn_exch{" H41 5BR"}="1H4  5BR"; $resn_exch{" H42 5BR"}="2H4  5BR";
$resn_exch{" H51 5BR"}="1H5  5BR"; $resn_exch{" H52 5BR"}="2H5  5BR"; $resn_exch{" H53 5BR"}="3H5  5BR";
$resn_exch{"HOP2 5BU"}="2HOP 5BU"; $resn_exch{"HOP3 5BU"}="3HOP 5BU"; $resn_exch{" H5' 5BU"}="1H5* 5BU";
$resn_exch{"H5'' 5BU"}="2H5* 5BU"; $resn_exch{" H4' 5BU"}=" H4* 5BU"; $resn_exch{" H3' 5BU"}=" H3* 5BU";
$resn_exch{"HO3' 5BU"}="H3T  5BU"; $resn_exch{" H2' 5BU"}=" H2* 5BU"; $resn_exch{"HO2' 5BU"}="2HO* 5BU";
$resn_exch{" H1' 5BU"}=" H1* 5BU"; $resn_exch{" HB1 5CA"}="1HB  5CA"; $resn_exch{" HB2 5CA"}="2HB  5CA";
$resn_exch{"H5'1 5CA"}="1H5* 5CA"; $resn_exch{"H5'2 5CA"}="2H5* 5CA"; $resn_exch{" H4' 5CA"}=" H4* 5CA";
$resn_exch{" H3' 5CA"}=" H3* 5CA"; $resn_exch{" H2' 5CA"}=" H2* 5CA"; $resn_exch{"HO'2 5CA"}="2HO* 5CA";
$resn_exch{" H1' 5CA"}=" H1* 5CA"; $resn_exch{" H61 5CA"}="1H6  5CA"; $resn_exch{" H62 5CA"}="2H6  5CA";
$resn_exch{"H131 5CB"}="1H13 5CB"; $resn_exch{"H132 5CB"}="2H13 5CB"; $resn_exch{"H161 5CB"}="1H16 5CB";
$resn_exch{"H162 5CB"}="2H16 5CB"; $resn_exch{"H117 5CB"}="17H1 5CB"; $resn_exch{"H191 5CB"}="1H19 5CB";
$resn_exch{"H220 5CB"}="20H2 5CB"; $resn_exch{"H241 5CB"}="1H24 5CB"; $resn_exch{"H242 5CB"}="2H24 5CB";
$resn_exch{"H251 5CB"}="1H25 5CB"; $resn_exch{"H252 5CB"}="2H25 5CB"; $resn_exch{"H6N1 5CD"}="1H6N 5CD";
$resn_exch{"H6N2 5CD"}="2H6N 5CD"; $resn_exch{" H2' 5CD"}=" H2* 5CD"; $resn_exch{" H3' 5CD"}=" H3* 5CD";
$resn_exch{" H4' 5CD"}=" H4* 5CD"; $resn_exch{"H5'1 5CD"}="1H5* 5CD"; $resn_exch{"H5'2 5CD"}="2H5* 5CD";
$resn_exch{" H1' 5CD"}=" H1* 5CD"; $resn_exch{" H5' 5CG"}="1H5* 5CG"; $resn_exch{"H5'' 5CG"}="2H5* 5CG";
$resn_exch{" H4' 5CG"}=" H4* 5CG"; $resn_exch{" H3' 5CG"}=" H3* 5CG"; $resn_exch{"HO3' 5CG"}="*HO3 5CG";
$resn_exch{" H2' 5CG"}=" H2* 5CG"; $resn_exch{"HO2' 5CG"}="*HO2 5CG"; $resn_exch{" H1' 5CG"}=" H1* 5CG";
$resn_exch{"HN21 5CG"}="1HN2 5CG"; $resn_exch{"HN22 5CG"}="2HN2 5CG"; $resn_exch{"H5A1 5CM"}="1H5A 5CM";
$resn_exch{"H5A2 5CM"}="2H5A 5CM"; $resn_exch{"H5A3 5CM"}="3H5A 5CM"; $resn_exch{"HN41 5CM"}="1HN4 5CM";
$resn_exch{"HN42 5CM"}="2HN4 5CM"; $resn_exch{" H1' 5CM"}=" H1* 5CM"; $resn_exch{" H2' 5CM"}="1H2* 5CM";
$resn_exch{"H2'' 5CM"}="2H2* 5CM"; $resn_exch{" H3' 5CM"}=" H3* 5CM"; $resn_exch{" H4' 5CM"}=" H4* 5CM";
$resn_exch{"HO3' 5CM"}="*HO3 5CM"; $resn_exch{" H5' 5CM"}="1H5* 5CM"; $resn_exch{"H5'' 5CM"}="2H5* 5CM";
$resn_exch{"HOP2 5CM"}="2HOP 5CM"; $resn_exch{"HOP3 5CM"}="3HOP 5CM"; $resn_exch{" H71 5CN"}="1H7  5CN";
$resn_exch{" H72 5CN"}="2H7  5CN"; $resn_exch{" H81 5CN"}="1H8  5CN"; $resn_exch{" H82 5CN"}="2H8  5CN";
$resn_exch{" H91 5CN"}="1H9  5CN"; $resn_exch{" H92 5CN"}="2H9  5CN"; $resn_exch{"H101 5CN"}="1H10 5CN";
$resn_exch{"H102 5CN"}="2H10 5CN"; $resn_exch{"H121 5CN"}="1H12 5CN"; $resn_exch{"H122 5CN"}="2H12 5CN";
$resn_exch{"H123 5CN"}="3H12 5CN"; $resn_exch{"H131 5CN"}="1H13 5CN"; $resn_exch{"H132 5CN"}="2H13 5CN";
$resn_exch{" H2  5CS"}=" HN2 5CS"; $resn_exch{" HB2 5CS"}="1HB  5CS"; $resn_exch{" HB3 5CS"}="2HB  5CS";
$resn_exch{" HN1 5CS"}="1HN  5CS"; $resn_exch{"HN2A 5CS"}="2HN  5CS"; $resn_exch{" HE3 5CS"}="1HE  5CS";
$resn_exch{" HE2 5CS"}="2HE  5CS"; $resn_exch{"H171 5DE"}="1H17 5DE"; $resn_exch{"H172 5DE"}="2H17 5DE";
$resn_exch{"H181 5DE"}="1H18 5DE"; $resn_exch{"H182 5DE"}="2H18 5DE"; $resn_exch{"H183 5DE"}="3H18 5DE";
$resn_exch{" H91 5DE"}="1H9  5DE"; $resn_exch{" H92 5DE"}="2H9  5DE"; $resn_exch{" H93 5DE"}="3H9  5DE";
$resn_exch{" H61 5DE"}="1H6  5DE"; $resn_exch{" H62 5DE"}="2H6  5DE"; $resn_exch{" H63 5DE"}="3H6  5DE";
$resn_exch{"H141 5DE"}="1H14 5DE"; $resn_exch{"H142 5DE"}="2H14 5DE"; $resn_exch{" H11 5DS"}="1H1  5DS";
$resn_exch{" H12 5DS"}="2H1  5DS"; $resn_exch{" H13 5DS"}="3H1  5DS"; $resn_exch{" H21 5DS"}="1H2  5DS";
$resn_exch{" H22 5DS"}="2H2  5DS"; $resn_exch{" H23 5DS"}="3H2  5DS"; $resn_exch{"HN41 5DS"}="1HN4 5DS";
$resn_exch{"HN42 5DS"}="2HN4 5DS"; $resn_exch{"HN51 5DS"}="1HN5 5DS"; $resn_exch{"HN52 5DS"}="2HN5 5DS";
$resn_exch{"H221 5EA"}="1H22 5EA"; $resn_exch{"H222 5EA"}="2H22 5EA"; $resn_exch{"H241 5EA"}="1H24 5EA";
$resn_exch{"H242 5EA"}="2H24 5EA"; $resn_exch{"H271 5EA"}="1H27 5EA"; $resn_exch{"H272 5EA"}="2H27 5EA";
$resn_exch{"H291 5EA"}="1H29 5EA"; $resn_exch{"H292 5EA"}="2H29 5EA"; $resn_exch{"H301 5EA"}="1H30 5EA";
$resn_exch{"H302 5EA"}="2H30 5EA"; $resn_exch{"H281 5EA"}="1H28 5EA"; $resn_exch{"H282 5EA"}="2H28 5EA";
$resn_exch{" H11 5EE"}="1H1  5EE"; $resn_exch{" H12 5EE"}="2H1  5EE"; $resn_exch{"H2'1 5F1"}="1H2* 5F1";
$resn_exch{"H2'2 5F1"}="2H2* 5F1"; $resn_exch{" H3' 5F1"}=" H3* 5F1"; $resn_exch{" H4' 5F1"}=" H4* 5F1";
$resn_exch{"H5'1 5F1"}="1H5* 5F1"; $resn_exch{"H5'2 5F1"}="2H5* 5F1"; $resn_exch{" H1' 5F1"}=" H1* 5F1";
$resn_exch{"H6N1 5F1"}="1H6N 5F1"; $resn_exch{"H6N2 5F1"}="2H6N 5F1"; $resn_exch{" H61 5FA"}="1H6  5FA";
$resn_exch{" H62 5FA"}="2H6  5FA"; $resn_exch{"H5'1 5FA"}="1H5* 5FA"; $resn_exch{"H5'2 5FA"}="2H5* 5FA";
$resn_exch{" H4' 5FA"}=" H4* 5FA"; $resn_exch{" H3' 5FA"}=" H3* 5FA"; $resn_exch{" H2' 5FA"}=" H2* 5FA";
$resn_exch{"HO'2 5FA"}="2HO* 5FA"; $resn_exch{" H1' 5FA"}=" H1* 5FA"; $resn_exch{"HOP2 5FC"}="2HOP 5FC";
$resn_exch{"HN41 5FC"}="1HN4 5FC"; $resn_exch{"HN42 5FC"}="2HN4 5FC"; $resn_exch{" H2' 5FC"}="1H2* 5FC";
$resn_exch{"H2'' 5FC"}="2H2* 5FC"; $resn_exch{" H5' 5FC"}="1H5* 5FC"; $resn_exch{"H5'' 5FC"}="2H5* 5FC";
$resn_exch{" H4' 5FC"}=" H4* 5FC"; $resn_exch{" H1' 5FC"}=" H1* 5FC"; $resn_exch{" H3' 5FC"}=" H3* 5FC";
$resn_exch{"HO3' 5FC"}="H3T  5FC"; $resn_exch{"HOP3 5FC"}="3HOP 5FC"; $resn_exch{" H2' 5FD"}=" H2* 5FD";
$resn_exch{"H3'' 5FD"}="*H3* 5FD"; $resn_exch{" H4' 5FD"}=" H4* 5FD"; $resn_exch{"H5'1 5FD"}="1H5* 5FD";
$resn_exch{"H5'2 5FD"}="2H5* 5FD"; $resn_exch{" H1' 5FD"}=" H1* 5FD"; $resn_exch{" H55 5FD"}="5H5  5FD";
$resn_exch{" H44 5FD"}="4H4  5FD"; $resn_exch{" H21 5FE"}="1H2  5FE"; $resn_exch{" H22 5FE"}="2H2  5FE";
$resn_exch{" HA1 5FE"}="1HA  5FE"; $resn_exch{" HA2 5FE"}="2HA  5FE"; $resn_exch{" HA3 5FE"}="3HA  5FE";
$resn_exch{"H291 5FE"}="1H29 5FE"; $resn_exch{"H292 5FE"}="2H29 5FE"; $resn_exch{"H293 5FE"}="3H29 5FE";
$resn_exch{"H101 5FE"}="1H10 5FE"; $resn_exch{"H102 5FE"}="2H10 5FE"; $resn_exch{"H351 5FE"}="1H35 5FE";
$resn_exch{"H352 5FE"}="2H35 5FE"; $resn_exch{"H353 5FE"}="3H35 5FE"; $resn_exch{"H341 5FE"}="1H34 5FE";
$resn_exch{"H342 5FE"}="2H34 5FE"; $resn_exch{"H343 5FE"}="3H34 5FE"; $resn_exch{"H381 5FE"}="1H38 5FE";
$resn_exch{"H382 5FE"}="2H38 5FE"; $resn_exch{"H383 5FE"}="3H38 5FE"; $resn_exch{"H371 5FE"}="1H37 5FE";
$resn_exch{"H372 5FE"}="2H37 5FE"; $resn_exch{"H373 5FE"}="3H37 5FE"; $resn_exch{"H221 5FE"}="1H22 5FE";
$resn_exch{"H222 5FE"}="2H22 5FE"; $resn_exch{"H391 5FE"}="1H39 5FE"; $resn_exch{"H392 5FE"}="2H39 5FE";
$resn_exch{"H393 5FE"}="3H39 5FE"; $resn_exch{"H241 5FE"}="1H24 5FE"; $resn_exch{"H242 5FE"}="2H24 5FE";
$resn_exch{"H243 5FE"}="3H24 5FE"; $resn_exch{" H21 5FP"}="1H2  5FP"; $resn_exch{" H22 5FP"}="2H2  5FP";
$resn_exch{"H291 5FP"}="1H29 5FP"; $resn_exch{"H292 5FP"}="2H29 5FP"; $resn_exch{"H293 5FP"}="3H29 5FP";
$resn_exch{"H101 5FP"}="1H10 5FP"; $resn_exch{"H102 5FP"}="2H10 5FP"; $resn_exch{"H351 5FP"}="1H35 5FP";
$resn_exch{"H352 5FP"}="2H35 5FP"; $resn_exch{"H353 5FP"}="3H35 5FP"; $resn_exch{"H341 5FP"}="1H34 5FP";
$resn_exch{"H342 5FP"}="2H34 5FP"; $resn_exch{"H343 5FP"}="3H34 5FP"; $resn_exch{"H381 5FP"}="1H38 5FP";
$resn_exch{"H382 5FP"}="2H38 5FP"; $resn_exch{"H383 5FP"}="3H38 5FP"; $resn_exch{"H371 5FP"}="1H37 5FP";
$resn_exch{"H372 5FP"}="2H37 5FP"; $resn_exch{"H373 5FP"}="3H37 5FP"; $resn_exch{"H221 5FP"}="1H22 5FP";
$resn_exch{"H222 5FP"}="2H22 5FP"; $resn_exch{"H391 5FP"}="1H39 5FP"; $resn_exch{"H392 5FP"}="2H39 5FP";
$resn_exch{"H393 5FP"}="3H39 5FP"; $resn_exch{"H241 5FP"}="1H24 5FP"; $resn_exch{"H242 5FP"}="2H24 5FP";
$resn_exch{"H243 5FP"}="3H24 5FP"; $resn_exch{"HO'2 5FU"}="2HO* 5FU"; $resn_exch{" H2' 5FU"}=" H2* 5FU";
$resn_exch{" H1' 5FU"}=" H1* 5FU"; $resn_exch{" H4' 5FU"}=" H4* 5FU"; $resn_exch{"H5'1 5FU"}="1H5* 5FU";
$resn_exch{"H5'2 5FU"}="2H5* 5FU"; $resn_exch{"HOP2 5FU"}="2HOP 5FU"; $resn_exch{"HOP3 5FU"}="3HOP 5FU";
$resn_exch{" H3' 5FU"}=" H3* 5FU"; $resn_exch{"HOP2 5GP"}="2HOP 5GP"; $resn_exch{"HOP3 5GP"}="3HOP 5GP";
$resn_exch{"H5'1 5GP"}="1H5* 5GP"; $resn_exch{"H5'2 5GP"}="2H5* 5GP"; $resn_exch{" H4' 5GP"}=" H4* 5GP";
$resn_exch{" H3' 5GP"}=" H3* 5GP"; $resn_exch{"HO3' 5GP"}="*HO3 5GP"; $resn_exch{" H2' 5GP"}=" H2* 5GP";
$resn_exch{"HO2' 5GP"}="*HO2 5GP"; $resn_exch{" H1' 5GP"}=" H1* 5GP"; $resn_exch{"HN21 5GP"}="1HN2 5GP";
$resn_exch{"HN22 5GP"}="2HN2 5GP"; $resn_exch{" H11  5H"}="1H1   5H"; $resn_exch{" H12  5H"}="2H1   5H";
$resn_exch{" H11 5HA"}="1H1  5HA"; $resn_exch{"H12A 5HA"}="2H1  5HA"; $resn_exch{"H31A 5HA"}="1H3  5HA";
$resn_exch{"H32A 5HA"}="2H3  5HA"; $resn_exch{" H41 5HA"}="1H4  5HA"; $resn_exch{" H42 5HA"}="2H4  5HA";
$resn_exch{" H81 5HA"}="1H8  5HA"; $resn_exch{" H82 5HA"}="2H8  5HA"; $resn_exch{"H161 5HA"}="1H16 5HA";
$resn_exch{"H162 5HA"}="2H16 5HA"; $resn_exch{"H163 5HA"}="3H16 5HA"; $resn_exch{"H231 5HA"}="1H23 5HA";
$resn_exch{"H232 5HA"}="2H23 5HA"; $resn_exch{"H233 5HA"}="3H23 5HA"; $resn_exch{"H181 5HA"}="1H18 5HA";
$resn_exch{"H182 5HA"}="2H18 5HA"; $resn_exch{"H183 5HA"}="3H18 5HA"; $resn_exch{"H281 5HG"}="1H28 5HG";
$resn_exch{"H282 5HG"}="2H28 5HG"; $resn_exch{"H241 5HG"}="1H24 5HG"; $resn_exch{"H242 5HG"}="2H24 5HG";
$resn_exch{"H221 5HG"}="1H22 5HG"; $resn_exch{"H222 5HG"}="2H22 5HG"; $resn_exch{"H201 5HG"}="1H20 5HG";
$resn_exch{"H202 5HG"}="2H20 5HG"; $resn_exch{" HB1 5HP"}="1HB  5HP"; $resn_exch{" HB2 5HP"}="2HB  5HP";
$resn_exch{" HG1 5HP"}="1HG  5HP"; $resn_exch{" HG2 5HP"}="2HG  5HP"; $resn_exch{"HO5' 5HT"}="H5T  5HT";
$resn_exch{" H5' 5HT"}="1H5* 5HT"; $resn_exch{"H5'' 5HT"}="2H5* 5HT"; $resn_exch{" H4' 5HT"}=" H4* 5HT";
$resn_exch{" H3' 5HT"}=" H3* 5HT"; $resn_exch{"HO3' 5HT"}="H3T  5HT"; $resn_exch{" H2' 5HT"}="1H2* 5HT";
$resn_exch{"H2'' 5HT"}="2H2* 5HT"; $resn_exch{" H1' 5HT"}=" H1* 5HT"; $resn_exch{" H71 5HT"}="1H5M 5HT";
$resn_exch{" H72 5HT"}="2H5M 5HT"; $resn_exch{" H73 5HT"}="3H5M 5HT"; $resn_exch{" H61 5HT"}="1H6  5HT";
$resn_exch{" H62 5HT"}="2H6  5HT"; $resn_exch{" H5' 5HU"}="1H5* 5HU"; $resn_exch{"H5'' 5HU"}="2H5* 5HU";
$resn_exch{"HC4' 5HU"}="HC4* 5HU"; $resn_exch{"HC3' 5HU"}="HC3* 5HU"; $resn_exch{"HO3' 5HU"}="HO3* 5HU";
$resn_exch{" H2' 5HU"}="1H2* 5HU"; $resn_exch{"H2'' 5HU"}="2H2* 5HU"; $resn_exch{"HC1' 5HU"}="HC1* 5HU";
$resn_exch{"H5A1 5HU"}="1H5A 5HU"; $resn_exch{"H5A2 5HU"}="2H5A 5HU"; $resn_exch{"HN41 5I5"}="1HN4 5I5";
$resn_exch{"HN42 5I5"}="2HN4 5I5"; $resn_exch{"H111 5I5"}="1H11 5I5"; $resn_exch{"H112 5I5"}="2H11 5I5";
$resn_exch{"H113 5I5"}="3H11 5I5"; $resn_exch{"HO2' 5IC"}="2HO* 5IC"; $resn_exch{" H2' 5IC"}=" H2* 5IC";
$resn_exch{" H3' 5IC"}=" H3* 5IC"; $resn_exch{"HO3' 5IC"}="H3T  5IC"; $resn_exch{" H4' 5IC"}=" H4* 5IC";
$resn_exch{" H1' 5IC"}=" H1* 5IC"; $resn_exch{" H41 5IC"}="1H4  5IC"; $resn_exch{" H42 5IC"}="2H4  5IC";
$resn_exch{" H5' 5IC"}="1H5* 5IC"; $resn_exch{"H5'' 5IC"}="2H5* 5IC"; $resn_exch{"HOP3 5IC"}="3HOP 5IC";
$resn_exch{"HOP2 5IC"}="2HOP 5IC"; $resn_exch{"H5'1 5ID"}="1H5' 5ID"; $resn_exch{"H5'2 5ID"}="2H5' 5ID";
$resn_exch{"H6N1 5ID"}="1H6N 5ID"; $resn_exch{"H6N2 5ID"}="2H6N 5ID"; $resn_exch{"HN31 5IG"}="1HN3 5IG";
$resn_exch{"HN32 5IG"}="2HN3 5IG"; $resn_exch{"HN41 5IG"}="1HN4 5IG"; $resn_exch{"HN42 5IG"}="2HN4 5IG";
$resn_exch{" H61 5IG"}="1H6  5IG"; $resn_exch{" H62 5IG"}="2H6  5IG"; $resn_exch{"H161 5IG"}="1H16 5IG";
$resn_exch{"H162 5IG"}="2H16 5IG"; $resn_exch{"H171 5IG"}="1H17 5IG"; $resn_exch{"H172 5IG"}="2H17 5IG";
$resn_exch{"H181 5IG"}="1H18 5IG"; $resn_exch{"H182 5IG"}="2H18 5IG"; $resn_exch{"H191 5IG"}="1H19 5IG";
$resn_exch{"H192 5IG"}="2H19 5IG"; $resn_exch{"H193 5IG"}="3H19 5IG"; $resn_exch{"H211 5IG"}="1H21 5IG";
$resn_exch{"H212 5IG"}="2H21 5IG"; $resn_exch{"H213 5IG"}="3H21 5IG"; $resn_exch{"H201 5IG"}="1H20 5IG";
$resn_exch{"H202 5IG"}="2H20 5IG"; $resn_exch{"H203 5IG"}="3H20 5IG"; $resn_exch{" H51 5IG"}="1H5  5IG";
$resn_exch{" H52 5IG"}="2H5  5IG"; $resn_exch{" H53 5IG"}="3H5  5IG"; $resn_exch{" H11 5IN"}="1H1  5IN";
$resn_exch{"H12A 5IN"}="2H1  5IN"; $resn_exch{"H31A 5IN"}="1H3  5IN"; $resn_exch{" H32 5IN"}="2H3  5IN";
$resn_exch{"H101 5IN"}="1H10 5IN"; $resn_exch{"H102 5IN"}="2H10 5IN"; $resn_exch{"H191 5IN"}="1H19 5IN";
$resn_exch{"H192 5IN"}="2H19 5IN"; $resn_exch{"H251 5IN"}="1H25 5IN"; $resn_exch{"H252 5IN"}="2H25 5IN";
$resn_exch{"H341 5IN"}="1H34 5IN"; $resn_exch{"H342 5IN"}="2H34 5IN"; $resn_exch{"HAJ1 5IQ"}="1HAJ 5IQ";
$resn_exch{"HAJ2 5IQ"}="2HAJ 5IQ"; $resn_exch{"HOP2 5IT"}="2HOP 5IT"; $resn_exch{"HOP3 5IT"}="3HOP 5IT";
$resn_exch{" H5' 5IT"}="1H5* 5IT"; $resn_exch{"H5'' 5IT"}="2H5* 5IT"; $resn_exch{" H4' 5IT"}=" H4* 5IT";
$resn_exch{" H3' 5IT"}=" H3* 5IT"; $resn_exch{"HO3' 5IT"}="*HO3 5IT"; $resn_exch{" H2' 5IT"}="1H2* 5IT";
$resn_exch{"H2'' 5IT"}="2H2* 5IT"; $resn_exch{" H1' 5IT"}=" H1* 5IT"; $resn_exch{" H1' 5IU"}=" H1* 5IU";
$resn_exch{" H2' 5IU"}="1H2* 5IU"; $resn_exch{"H2'' 5IU"}="2H2* 5IU"; $resn_exch{" H3' 5IU"}=" H3* 5IU";
$resn_exch{" H4' 5IU"}=" H4* 5IU"; $resn_exch{"HO3' 5IU"}="*HO3 5IU"; $resn_exch{" H5' 5IU"}="1H5* 5IU";
$resn_exch{"H5'' 5IU"}="2H5* 5IU"; $resn_exch{"HOP2 5IU"}="2HOP 5IU"; $resn_exch{"HOP3 5IU"}="3HOP 5IU";
$resn_exch{"HN31 5MB"}="1HN3 5MB"; $resn_exch{" H21 5MB"}="1H2  5MB"; $resn_exch{" H41 5MB"}="1H4  5MB";
$resn_exch{" H61 5MB"}="1H6  5MB"; $resn_exch{" H71 5MB"}="1H7  5MB"; $resn_exch{" H81 5MB"}="1H8  5MB";
$resn_exch{" H82 5MB"}="2H8  5MB"; $resn_exch{" H83 5MB"}="3H8  5MB"; $resn_exch{"HOP2 5MC"}="2HOP 5MC";
$resn_exch{"HOP3 5MC"}="3HOP 5MC"; $resn_exch{" H5' 5MC"}="1H5* 5MC"; $resn_exch{"H5'' 5MC"}="2H5* 5MC";
$resn_exch{" H4' 5MC"}=" H4* 5MC"; $resn_exch{" H3' 5MC"}=" H3* 5MC"; $resn_exch{"HO3' 5MC"}="*HO3 5MC";
$resn_exch{" H2' 5MC"}=" H2* 5MC"; $resn_exch{"HO2' 5MC"}="*HO2 5MC"; $resn_exch{" H1' 5MC"}=" H1* 5MC";
$resn_exch{"HN41 5MC"}="1HN4 5MC"; $resn_exch{"HN42 5MC"}="2HN4 5MC"; $resn_exch{"HM51 5MC"}="1HM5 5MC";
$resn_exch{"HM52 5MC"}="2HM5 5MC"; $resn_exch{"HM53 5MC"}="3HM5 5MC"; $resn_exch{" H5' 5MD"}=" H5* 5MD";
$resn_exch{"H5'1 5MD"}="1H5' 5MD"; $resn_exch{"H5'2 5MD"}="2H5' 5MD"; $resn_exch{" H51 5MD"}="1H5  5MD";
$resn_exch{" H52 5MD"}="2H5  5MD"; $resn_exch{" H53 5MD"}="3H5  5MD"; $resn_exch{"H3'A 5MD"}="H3*  5MD";
$resn_exch{"H2'1 5MD"}="1H2' 5MD"; $resn_exch{"H2'2 5MD"}="2H2' 5MD"; $resn_exch{"HC41 5MP"}="1HC4 5MP";
$resn_exch{"HC42 5MP"}="2HC4 5MP"; $resn_exch{"HC43 5MP"}="3HC4 5MP"; $resn_exch{" HB1 5MR"}="1HB  5MR";
$resn_exch{" HB2 5MR"}="2HB  5MR"; $resn_exch{"HAA1 5MR"}="1HAA 5MR"; $resn_exch{"HAA2 5MR"}="2HAA 5MR";
$resn_exch{"HAX1 5MR"}="1HAX 5MR"; $resn_exch{"HAX2 5MR"}="2HAX 5MR"; $resn_exch{"HAV1 5MR"}="1HAV 5MR";
$resn_exch{"HAV2 5MR"}="2HAV 5MR"; $resn_exch{" H71 5MU"}="1H5M 5MU"; $resn_exch{" H72 5MU"}="2H5M 5MU";
$resn_exch{" H73 5MU"}="3H5M 5MU"; $resn_exch{" H1' 5MU"}=" H1* 5MU"; $resn_exch{" H2' 5MU"}=" H2* 5MU";
$resn_exch{"HO2' 5MU"}="*HO2 5MU"; $resn_exch{" H3' 5MU"}=" H3* 5MU"; $resn_exch{" H4' 5MU"}=" H4* 5MU";
$resn_exch{"HO3' 5MU"}="*HO3 5MU"; $resn_exch{" H5' 5MU"}="1H5* 5MU"; $resn_exch{"H5'' 5MU"}="2H5* 5MU";
$resn_exch{"HOP2 5MU"}="2HOP 5MU"; $resn_exch{"HOP3 5MU"}="3HOP 5MU"; $resn_exch{"HOP2 5NC"}="2HOP 5NC";
$resn_exch{"HOP3 5NC"}="3HOP 5NC"; $resn_exch{" H5' 5NC"}="1H5* 5NC"; $resn_exch{"H5'' 5NC"}="2H5* 5NC";
$resn_exch{" H4' 5NC"}=" H4* 5NC"; $resn_exch{" H3' 5NC"}=" H3* 5NC"; $resn_exch{"HO'3 5NC"}="3HO* 5NC";
$resn_exch{" H2' 5NC"}="1H2* 5NC"; $resn_exch{"H2'' 5NC"}="2H2* 5NC"; $resn_exch{"H1'1 5NC"}="1H1* 5NC";
$resn_exch{"HN41 5NC"}="1HN4 5NC"; $resn_exch{"HN42 5NC"}="2HN4 5NC"; $resn_exch{" H51 5NH"}="1H5  5NH";
$resn_exch{" H52 5NH"}="2H5  5NH"; $resn_exch{" H53 5NH"}="3H5  5NH"; $resn_exch{" H61 5NH"}="1H6  5NH";
$resn_exch{" H62 5NH"}="2H6  5NH"; $resn_exch{" H63 5NH"}="3H6  5NH"; $resn_exch{" H71 5NH"}="1H7  5NH";
$resn_exch{" H72 5NH"}="2H7  5NH"; $resn_exch{" H73 5NH"}="3H7  5NH"; $resn_exch{"H111 5NH"}="1H11 5NH";
$resn_exch{"H112 5NH"}="2H11 5NH"; $resn_exch{"H121 5NH"}="1H12 5NH"; $resn_exch{"H122 5NH"}="2H12 5NH";
$resn_exch{"H131 5NH"}="1H13 5NH"; $resn_exch{"H132 5NH"}="2H13 5NH"; $resn_exch{"H141 5NH"}="1H14 5NH";
$resn_exch{"H142 5NH"}="2H14 5NH"; $resn_exch{"H151 5NH"}="1H15 5NH"; $resn_exch{"H152 5NH"}="2H15 5NH";
$resn_exch{"H231 5NH"}="1H23 5NH"; $resn_exch{"H232 5NH"}="2H23 5NH"; $resn_exch{"H241 5NH"}="1H24 5NH";
$resn_exch{"H242 5NH"}="2H24 5NH"; $resn_exch{"H261 5NH"}="1H26 5NH"; $resn_exch{"H262 5NH"}="2H26 5NH";
$resn_exch{"H291 5NH"}="1H29 5NH"; $resn_exch{"H292 5NH"}="2H29 5NH"; $resn_exch{"H301 5NH"}="1H30 5NH";
$resn_exch{"H302 5NH"}="2H30 5NH"; $resn_exch{"H311 5NH"}="1H31 5NH"; $resn_exch{"H312 5NH"}="2H31 5NH";
$resn_exch{"H321 5NH"}="1H32 5NH"; $resn_exch{"H322 5NH"}="2H32 5NH"; $resn_exch{"H323 5NH"}="3H32 5NH";
$resn_exch{"H371 5NH"}="1H37 5NH"; $resn_exch{"H372 5NH"}="2H37 5NH"; $resn_exch{"H521 5NH"}="1H52 5NH";
$resn_exch{"H522 5NH"}="2H52 5NH"; $resn_exch{"H523 5NH"}="3H52 5NH"; $resn_exch{"H531 5NH"}="1H53 5NH";
$resn_exch{"H532 5NH"}="2H53 5NH"; $resn_exch{"H533 5NH"}="3H53 5NH"; $resn_exch{" H61 5NI"}="1H6  5NI";
$resn_exch{" H71 5NI"}="1H7  5NI"; $resn_exch{" H41 5NI"}="1H4  5NI"; $resn_exch{"HN11 5NI"}="1HN1 5NI";
$resn_exch{" H31 5NI"}="1H3  5NI"; $resn_exch{"H31N 5OB"}="1NH3 5OB"; $resn_exch{" H21 5OB"}="1H2  5OB";
$resn_exch{" H41 5OB"}="1H4  5OB"; $resn_exch{" H61 5OB"}="1H6  5OB"; $resn_exch{" H71 5OB"}="1H7  5OB";
$resn_exch{"H111 5OB"}="1H11 5OB"; $resn_exch{"H112 5OB"}="2H11 5OB"; $resn_exch{"H113 5OB"}="3H11 5OB";
$resn_exch{" H11 5OP"}="1H1  5OP"; $resn_exch{" H12 5OP"}="2H1  5OP"; $resn_exch{"H13A 5OP"}="3H1  5OP";
$resn_exch{" H21 5OP"}="1H2  5OP"; $resn_exch{" H22 5OP"}="2H2  5OP"; $resn_exch{" H31 5OP"}="1H3  5OP";
$resn_exch{" H32 5OP"}="2H3  5OP"; $resn_exch{" H41 5OP"}="1H4  5OP"; $resn_exch{" H42 5OP"}="2H4  5OP";
$resn_exch{" H51 5OP"}="1H5  5OP"; $resn_exch{" H52 5OP"}="2H5  5OP"; $resn_exch{"H111 5OP"}="1H11 5OP";
$resn_exch{"H112 5OP"}="2H11 5OP"; $resn_exch{"H141 5OP"}="1H14 5OP"; $resn_exch{"H142 5OP"}="2H14 5OP";
$resn_exch{"H171 5OP"}="1H17 5OP"; $resn_exch{"H172 5OP"}="2H17 5OP"; $resn_exch{"H181 5OP"}="1H18 5OP";
$resn_exch{"H182 5OP"}="2H18 5OP"; $resn_exch{"H191 5OP"}="1H19 5OP"; $resn_exch{"H192 5OP"}="2H19 5OP";
$resn_exch{"H4A1 5PA"}="1H4A 5PA"; $resn_exch{"H4A2 5PA"}="2H4A 5PA"; $resn_exch{"HOP2 5PA"}="2HOP 5PA";
$resn_exch{"HOP3 5PA"}="3HOP 5PA"; $resn_exch{"H5A1 5PA"}="1H5A 5PA"; $resn_exch{"H5A2 5PA"}="2H5A 5PA";
$resn_exch{"H2A1 5PA"}="1H2A 5PA"; $resn_exch{"H2A2 5PA"}="2H2A 5PA"; $resn_exch{"H2A3 5PA"}="3H2A 5PA";
$resn_exch{" H91 5PA"}="1H9  5PA"; $resn_exch{" H92 5PA"}="2H9  5PA"; $resn_exch{"H101 5PA"}="1H10 5PA";
$resn_exch{"H102 5PA"}="2H10 5PA"; $resn_exch{"HOP3 5PC"}="3HOP 5PC"; $resn_exch{"HOP2 5PC"}="2HOP 5PC";
$resn_exch{" H5' 5PC"}="1H5* 5PC"; $resn_exch{"H5'' 5PC"}="2H5* 5PC"; $resn_exch{" H4' 5PC"}=" H4* 5PC";
$resn_exch{" H3' 5PC"}=" H3* 5PC"; $resn_exch{"HO3' 5PC"}="H3T  5PC"; $resn_exch{" H2' 5PC"}="1H2* 5PC";
$resn_exch{"H2'' 5PC"}="2H2* 5PC"; $resn_exch{" H1' 5PC"}=" H1* 5PC"; $resn_exch{"H251 5PH"}="1H25 5PH";
$resn_exch{"H252 5PH"}="2H25 5PH"; $resn_exch{"H361 5PH"}="1H36 5PH"; $resn_exch{"H362 5PH"}="2H36 5PH";
$resn_exch{"H363 5PH"}="3H36 5PH"; $resn_exch{" H41 5PH"}="1H4  5PH"; $resn_exch{" H42 5PH"}="2H4  5PH";
$resn_exch{" H11 5PH"}="1H1  5PH"; $resn_exch{" H12 5PH"}="2H1  5PH"; $resn_exch{"H121 5PH"}="1H12 5PH";
$resn_exch{"H122 5PH"}="2H12 5PH"; $resn_exch{" H81 5PH"}="1H8  5PH"; $resn_exch{" H82 5PH"}="2H8  5PH";
$resn_exch{"H181 5PI"}="1H18 5PI"; $resn_exch{"H182 5PI"}="2H18 5PI"; $resn_exch{"H191 5PI"}="1H19 5PI";
$resn_exch{"H192 5PI"}="2H19 5PI"; $resn_exch{"H211 5PI"}="1H21 5PI"; $resn_exch{"H212 5PI"}="2H21 5PI";
$resn_exch{"HN11 5PI"}="1HN1 5PI"; $resn_exch{"HN12 5PI"}="2HN1 5PI"; $resn_exch{"HN31 5PI"}="1HN3 5PI";
$resn_exch{"HN32 5PI"}="2HN3 5PI"; $resn_exch{" H11 5PI"}="1H1  5PI"; $resn_exch{"H12A 5PI"}="2H1  5PI";
$resn_exch{" H91 5PI"}="1H9  5PI"; $resn_exch{" H92 5PI"}="2H9  5PI"; $resn_exch{" H81 5PI"}="1H8  5PI";
$resn_exch{" H82 5PI"}="2H8  5PI"; $resn_exch{" H83 5PI"}="3H8  5PI"; $resn_exch{" H71 5PI"}="1H7  5PI";
$resn_exch{" H72 5PI"}="2H7  5PI"; $resn_exch{"H101 5PI"}="1H10 5PI"; $resn_exch{"H102 5PI"}="2H10 5PI";
$resn_exch{"H111 5PI"}="1H11 5PI"; $resn_exch{"H112 5PI"}="2H11 5PI"; $resn_exch{"H113 5PI"}="3H11 5PI";
$resn_exch{"H141 5PP"}="1H14 5PP"; $resn_exch{"H142 5PP"}="2H14 5PP"; $resn_exch{"H151 5PP"}="1H15 5PP";
$resn_exch{"H152 5PP"}="2H15 5PP"; $resn_exch{"H161 5PP"}="1H16 5PP"; $resn_exch{"H162 5PP"}="2H16 5PP";
$resn_exch{"H171 5PP"}="1H17 5PP"; $resn_exch{"H172 5PP"}="2H17 5PP"; $resn_exch{"H181 5PP"}="1H18 5PP";
$resn_exch{"H182 5PP"}="2H18 5PP"; $resn_exch{"H183 5PP"}="3H18 5PP"; $resn_exch{" H71 5PV"}="1H7  5PV";
$resn_exch{" H72 5PV"}="2H7  5PV"; $resn_exch{" H81 5PV"}="1H8  5PV"; $resn_exch{" H82 5PV"}="2H8  5PV";
$resn_exch{" H91 5PV"}="1H9  5PV"; $resn_exch{" H92 5PV"}="2H9  5PV"; $resn_exch{"H101 5PV"}="1H10 5PV";
$resn_exch{"H102 5PV"}="2H10 5PV"; $resn_exch{" H71 5PY"}="1H5M 5PY"; $resn_exch{" H72 5PY"}="2H5M 5PY";
$resn_exch{" H73 5PY"}="3H5M 5PY"; $resn_exch{" H1' 5PY"}=" H1* 5PY"; $resn_exch{" H2' 5PY"}="1H2* 5PY";
$resn_exch{"H2'' 5PY"}="2H2* 5PY"; $resn_exch{" H3' 5PY"}=" H3* 5PY"; $resn_exch{"HO3' 5PY"}="H3T  5PY";
$resn_exch{" H4' 5PY"}=" H4* 5PY"; $resn_exch{" H5' 5PY"}="1H5* 5PY"; $resn_exch{"H5'' 5PY"}="2H5* 5PY";
$resn_exch{"HOP2 5PY"}="2HOP 5PY"; $resn_exch{"HOP3 5PY"}="3HOP 5PY"; $resn_exch{"HN11 5QC"}="1HN1 5QC";
$resn_exch{"HN12 5QC"}="2HN1 5QC"; $resn_exch{"H111 5QC"}="1H11 5QC"; $resn_exch{"H112 5QC"}="2H11 5QC";
$resn_exch{"H201 5QC"}="1H20 5QC"; $resn_exch{"H202 5QC"}="2H20 5QC"; $resn_exch{"H231 5QC"}="1H23 5QC";
$resn_exch{"H232 5QC"}="2H23 5QC"; $resn_exch{"H281 5QC"}="1H28 5QC"; $resn_exch{"H282 5QC"}="2H28 5QC";
$resn_exch{"H291 5QC"}="1H29 5QC"; $resn_exch{"H292 5QC"}="2H29 5QC"; $resn_exch{"H311 5QC"}="1H31 5QC";
$resn_exch{"H312 5QC"}="2H31 5QC"; $resn_exch{"H181 5RM"}="1H18 5RM"; $resn_exch{"H182 5RM"}="2H18 5RM";
$resn_exch{"H183 5RM"}="3H18 5RM"; $resn_exch{"H111 5RM"}="1H11 5RM"; $resn_exch{"H112 5RM"}="2H11 5RM";
$resn_exch{"H113 5RM"}="3H11 5RM"; $resn_exch{"H141 5RM"}="1H14 5RM"; $resn_exch{"H142 5RM"}="2H14 5RM";
$resn_exch{"H151 5RM"}="1H15 5RM"; $resn_exch{"H152 5RM"}="2H15 5RM"; $resn_exch{"H161 5RM"}="1H16 5RM";
$resn_exch{"H162 5RM"}="2H16 5RM"; $resn_exch{"H163 5RM"}="3H16 5RM"; $resn_exch{" H21 5RP"}="1H2  5RP";
$resn_exch{" H22 5RP"}="2H2  5RP"; $resn_exch{" H71 5RP"}="1H7  5RP"; $resn_exch{" H72 5RP"}="2H7  5RP";
$resn_exch{"HR61 5SA"}="1HR6 5SA"; $resn_exch{"HR62 5SA"}="2HR6 5SA"; $resn_exch{"HV61 5SA"}="1HV6 5SA";
$resn_exch{"HV62 5SA"}="2HV6 5SA"; $resn_exch{"HV63 5SA"}="3HV6 5SA"; $resn_exch{"HQ61 5SA"}="1HQ6 5SA";
$resn_exch{"HQ62 5SA"}="2HQ6 5SA"; $resn_exch{"HW61 5SA"}="1HW6 5SA"; $resn_exch{"HW62 5SA"}="2HW6 5SA";
$resn_exch{"HY71 5SA"}="1HY7 5SA"; $resn_exch{"HY72 5SA"}="2HY7 5SA"; $resn_exch{"H121 5SC"}="1H12 5SC";
$resn_exch{"H122 5SC"}="2H12 5SC"; $resn_exch{"H251 5SC"}="1H25 5SC"; $resn_exch{"H252 5SC"}="2H25 5SC";
$resn_exch{"H253 5SC"}="3H25 5SC"; $resn_exch{"H1C1 5SD"}="1H1C 5SD"; $resn_exch{"H1C2 5SD"}="2H1C 5SD";
$resn_exch{"H2C1 5SD"}="1H2C 5SD"; $resn_exch{"H2C2 5SD"}="2H2C 5SD"; $resn_exch{"H4C1 5SD"}="1H4C 5SD";
$resn_exch{"H4C2 5SD"}="2H4C 5SD"; $resn_exch{"H6C1 5SD"}="1H6C 5SD"; $resn_exch{"H6C2 5SD"}="2H6C 5SD";
$resn_exch{"H7C1 5SD"}="1H7C 5SD"; $resn_exch{"H7C2 5SD"}="2H7C 5SD"; $resn_exch{"H111 5SD"}="1H11 5SD";
$resn_exch{"H112 5SD"}="2H11 5SD"; $resn_exch{"H121 5SD"}="1H12 5SD"; $resn_exch{"H122 5SD"}="2H12 5SD";
$resn_exch{"H151 5SD"}="1H15 5SD"; $resn_exch{"H152 5SD"}="2H15 5SD"; $resn_exch{"H161 5SD"}="1H16 5SD";
$resn_exch{"H162 5SD"}="2H16 5SD"; $resn_exch{"H181 5SD"}="1H18 5SD"; $resn_exch{"H182 5SD"}="2H18 5SD";
$resn_exch{"H183 5SD"}="3H18 5SD"; $resn_exch{"H191 5SD"}="1H19 5SD"; $resn_exch{"H192 5SD"}="2H19 5SD";
$resn_exch{"H193 5SD"}="3H19 5SD"; $resn_exch{"HAQ1 5TN"}="1HAQ 5TN"; $resn_exch{"HAQ2 5TN"}="2HAQ 5TN";
$resn_exch{"HAP1 5TN"}="1HAP 5TN"; $resn_exch{"HAP2 5TN"}="2HAP 5TN"; $resn_exch{"HAA1 5TN"}="1HAA 5TN";
$resn_exch{"HAA2 5TN"}="2HAA 5TN"; $resn_exch{"HAA3 5TN"}="3HAA 5TN"; $resn_exch{"HAB1 5TN"}="1HAB 5TN";
$resn_exch{"HAB2 5TN"}="2HAB 5TN"; $resn_exch{"HAB3 5TN"}="3HAB 5TN"; $resn_exch{"HAC1 5TN"}="1HAC 5TN";
$resn_exch{"HAC2 5TN"}="2HAC 5TN"; $resn_exch{"HAC3 5TN"}="3HAC 5TN"; $resn_exch{"HAR1 5TN"}="1HAR 5TN";
$resn_exch{"HAR2 5TN"}="2HAR 5TN"; $resn_exch{"HAS1 5TN"}="1HAS 5TN"; $resn_exch{"HAS2 5TN"}="2HAS 5TN";
$resn_exch{"HAD1 5TN"}="1HAD 5TN"; $resn_exch{"HAD2 5TN"}="2HAD 5TN"; $resn_exch{"HAD3 5TN"}="3HAD 5TN";
$resn_exch{"HAE1 5TN"}="1HAE 5TN"; $resn_exch{"HAE2 5TN"}="2HAE 5TN"; $resn_exch{"HAE3 5TN"}="3HAE 5TN";
$resn_exch{" H1' 5UD"}=" H1* 5UD"; $resn_exch{" H2' 5UD"}=" H2* 5UD"; $resn_exch{"HO2' 5UD"}="*HO2 5UD";
$resn_exch{" H3' 5UD"}=" H3* 5UD"; $resn_exch{"HO3' 5UD"}="*HO3 5UD"; $resn_exch{" H4' 5UD"}=" H4* 5UD";
$resn_exch{"H5'1 5UD"}="1H5* 5UD"; $resn_exch{"H5'2 5UD"}="2H5* 5UD"; $resn_exch{"HO5' 5UD"}="*HO5 5UD";
$resn_exch{"H11A  5X"}="1H1   5X"; $resn_exch{" H12  5X"}="2H1   5X"; $resn_exch{" H41  5X"}="1H4   5X";
$resn_exch{" H42  5X"}="2H4   5X"; $resn_exch{" H43  5X"}="3H4   5X"; $resn_exch{"HOP3 5YL"}="3HOP 5YL";
$resn_exch{"HOP2 5YL"}="2HOP 5YL"; $resn_exch{"H171 5YL"}="1H17 5YL"; $resn_exch{"H172 5YL"}="2H17 5YL";
$resn_exch{"H161 5YL"}="1H16 5YL"; $resn_exch{"H162 5YL"}="2H16 5YL"; $resn_exch{"H151 5YL"}="1H15 5YL";
$resn_exch{"H152 5YL"}="2H15 5YL"; $resn_exch{"H141 5YL"}="1H14 5YL"; $resn_exch{"H142 5YL"}="2H14 5YL";
$resn_exch{"H131 5YL"}="1H13 5YL"; $resn_exch{"H132 5YL"}="2H13 5YL"; $resn_exch{" H81 5YL"}="1H8  5YL";
$resn_exch{" H82 5YL"}="2H8  5YL"; $resn_exch{" H91 5YL"}="1H9  5YL"; $resn_exch{"H101 5YL"}="1H10 5YL";
$resn_exch{"H10O 5YL"}="OH10 5YL"; $resn_exch{"H111 5YL"}="1H11 5YL"; $resn_exch{"H11O 5YL"}="OH11 5YL";
$resn_exch{"H121 5YL"}="1H12 5YL"; $resn_exch{"H122 5YL"}="2H12 5YL"; $resn_exch{"H12O 5YL"}="OH12 5YL";
$resn_exch{"H171 5Z5"}="1H17 5Z5"; $resn_exch{"H172 5Z5"}="2H17 5Z5"; $resn_exch{"H173 5Z5"}="3H17 5Z5";
$resn_exch{"H141 5Z5"}="1H14 5Z5"; $resn_exch{"H142 5Z5"}="2H14 5Z5"; $resn_exch{"H143 5Z5"}="3H14 5Z5";
$resn_exch{"H151 5Z5"}="1H15 5Z5"; $resn_exch{"H152 5Z5"}="2H15 5Z5"; $resn_exch{"H153 5Z5"}="3H15 5Z5";
$resn_exch{"H251 5ZA"}="1H25 5ZA"; $resn_exch{"H252 5ZA"}="2H25 5ZA"; $resn_exch{"H253 5ZA"}="3H25 5ZA";
$resn_exch{"H261 5ZA"}="1H26 5ZA"; $resn_exch{"H262 5ZA"}="2H26 5ZA"; $resn_exch{"H101 5ZA"}="1H10 5ZA";
$resn_exch{"H102 5ZA"}="2H10 5ZA"; $resn_exch{"H181 5ZA"}="1H18 5ZA"; $resn_exch{"H182 5ZA"}="2H18 5ZA";
$resn_exch{"H191 5ZA"}="1H19 5ZA"; $resn_exch{"H192 5ZA"}="2H19 5ZA"; $resn_exch{"H321 600"}="1H32 600";
$resn_exch{"H322 600"}="2H32 600"; $resn_exch{"H351 600"}="1H35 600"; $resn_exch{"H352 600"}="2H35 600";
$resn_exch{"H761 600"}="1H76 600"; $resn_exch{"H762 600"}="2H76 600"; $resn_exch{"H763 600"}="3H76 600";
$resn_exch{"H721 600"}="1H72 600"; $resn_exch{"H722 600"}="2H72 600"; $resn_exch{"H741 600"}="1H74 600";
$resn_exch{"H742 600"}="2H74 600"; $resn_exch{"H743 600"}="3H74 600"; $resn_exch{"H751 600"}="1H75 600";
$resn_exch{"H752 600"}="2H75 600"; $resn_exch{"H753 600"}="3H75 600"; $resn_exch{"H271 606"}="1H27 606";
$resn_exch{"H272 606"}="2H27 606"; $resn_exch{"H261 606"}="1H26 606"; $resn_exch{"H262 606"}="2H26 606";
$resn_exch{"H251 606"}="1H25 606"; $resn_exch{"H252 606"}="2H25 606"; $resn_exch{"H231 606"}="1H23 606";
$resn_exch{"H232 606"}="2H23 606"; $resn_exch{"H121 606"}="1H12 606"; $resn_exch{"H122 606"}="2H12 606";
$resn_exch{" H3B 607"}="H3'' 607"; $resn_exch{" H4B 607"}="H4'' 607"; $resn_exch{" H5B 607"}="H5'' 607";
$resn_exch{" H6B 607"}="H6'' 607"; $resn_exch{" H71 608"}="1H7  608"; $resn_exch{" H72 608"}="2H7  608";
$resn_exch{" H21 612"}="1H2  612"; $resn_exch{" H31 612"}="1H3  612"; $resn_exch{" H41 612"}="1H4  612";
$resn_exch{" H51 612"}="1H5  612"; $resn_exch{" H61 612"}="1H6  612"; $resn_exch{" HF1 612"}="1HF  612";
$resn_exch{" HH1 612"}="1HH  612"; $resn_exch{" HH2 612"}="2HH  612"; $resn_exch{" HH3 612"}="3HH  612";
$resn_exch{" HI1 612"}="1HI  612"; $resn_exch{" HI2 612"}="2HI  612"; $resn_exch{" HI3 612"}="3HI  612";
$resn_exch{" HN1 612"}="1HN  612"; $resn_exch{" HA1 612"}="1HA  612"; $resn_exch{" HA2 612"}="2HA  612";
$resn_exch{" HC1 612"}="1HC  612"; $resn_exch{" HC2 612"}="2HC  612"; $resn_exch{" HD1 612"}="1HD  612";
$resn_exch{" HD2 612"}="2HD  612"; $resn_exch{" HD3 612"}="3HD  612"; $resn_exch{"H30C 616"}="CH30 616";
$resn_exch{"H27C 616"}="CH27 616"; $resn_exch{"H171 616"}="1H17 616"; $resn_exch{"H172 616"}="2H17 616";
$resn_exch{"H173 616"}="3H17 616"; $resn_exch{"HC11 616"}="1HC1 616"; $resn_exch{"HC12 616"}="2HC1 616";
$resn_exch{"HC13 616"}="3HC1 616"; $resn_exch{"H29C 616"}="CH29 616"; $resn_exch{"HC31 616"}="1HC3 616";
$resn_exch{"HC32 616"}="2HC3 616"; $resn_exch{"HC33 616"}="3HC3 616"; $resn_exch{"H281 616"}="1H28 616";
$resn_exch{"H282 616"}="2H28 616"; $resn_exch{"H261 616"}="1H26 616"; $resn_exch{"H262 616"}="2H26 616";
$resn_exch{"HC61 616"}="1HC6 616"; $resn_exch{"HC62 616"}="2HC6 616"; $resn_exch{"H501 616"}="1H50 616";
$resn_exch{"H502 616"}="2H50 616"; $resn_exch{"H451 616"}="1H45 616"; $resn_exch{"H452 616"}="2H45 616";
$resn_exch{"H461 616"}="1H46 616"; $resn_exch{"H462 616"}="2H46 616"; $resn_exch{"H471 616"}="1H47 616";
$resn_exch{"H472 616"}="2H47 616"; $resn_exch{"H481 616"}="1H48 616"; $resn_exch{"H482 616"}="2H48 616";
$resn_exch{" H91 617"}="1H9  617"; $resn_exch{" H92 617"}="2H9  617"; $resn_exch{"H11A 618"}="1H1  618";
$resn_exch{" H12 618"}="2H1  618"; $resn_exch{"H13A 618"}="3H1  618"; $resn_exch{" H41 618"}="1H4  618";
$resn_exch{" H42 618"}="2H4  618"; $resn_exch{" H43 618"}="3H4  618"; $resn_exch{"H121 618"}="1H12 618";
$resn_exch{"H122 618"}="2H12 618"; $resn_exch{"H191 618"}="1H19 618"; $resn_exch{"H192 618"}="2H19 618";
$resn_exch{"H193 618"}="3H19 618"; $resn_exch{"H201 618"}="1H20 618"; $resn_exch{"H202 618"}="2H20 618";
$resn_exch{"H203 618"}="3H20 618"; $resn_exch{"H291 618"}="1H29 618"; $resn_exch{"H292 618"}="2H29 618";
$resn_exch{"H293 618"}="3H29 618"; $resn_exch{" H51 623"}="1H5  623"; $resn_exch{" H52 623"}="2H5  623";
$resn_exch{" H53 623"}="3H5  623"; $resn_exch{"H221 623"}="1H22 623"; $resn_exch{"H222 623"}="2H22 623";
$resn_exch{"H121 623"}="1H12 623"; $resn_exch{"H122 623"}="2H12 623"; $resn_exch{"H061 624"}="1H06 624";
$resn_exch{"H062 624"}="2H06 624"; $resn_exch{"H071 624"}="1H07 624"; $resn_exch{"H072 624"}="2H07 624";
$resn_exch{"H081 624"}="1H08 624"; $resn_exch{"H082 624"}="2H08 624"; $resn_exch{"H111 624"}="1H11 624";
$resn_exch{"H112 624"}="2H11 624"; $resn_exch{"H201 624"}="1H20 624"; $resn_exch{"H202 624"}="2H20 624";
$resn_exch{"H311 624"}="1H31 624"; $resn_exch{"H312 624"}="2H31 624"; $resn_exch{"H313 624"}="3H31 624";
$resn_exch{"H321 624"}="1H32 624"; $resn_exch{"H322 624"}="2H32 624"; $resn_exch{"H323 624"}="3H32 624";
$resn_exch{"H321 626"}="1H32 626"; $resn_exch{"H322 626"}="2H32 626"; $resn_exch{"H201 626"}="1H20 626";
$resn_exch{"H202 626"}="2H20 626"; $resn_exch{"H241 626"}="1H24 626"; $resn_exch{"H242 626"}="2H24 626";
$resn_exch{"H211 626"}="1H21 626"; $resn_exch{"H212 626"}="2H21 626"; $resn_exch{"H231 626"}="1H23 626";
$resn_exch{"H232 626"}="2H23 626"; $resn_exch{"H251 626"}="1H25 626"; $resn_exch{"H252 626"}="2H25 626";
$resn_exch{"H253 626"}="3H25 626"; $resn_exch{"H181 627"}="1H18 627"; $resn_exch{"H182 627"}="2H18 627";
$resn_exch{"H221 627"}="1H22 627"; $resn_exch{"H222 627"}="2H22 627"; $resn_exch{"H191 627"}="1H19 627";
$resn_exch{"H192 627"}="2H19 627"; $resn_exch{"H211 627"}="1H21 627"; $resn_exch{"H212 627"}="2H21 627";
$resn_exch{"H231 627"}="1H23 627"; $resn_exch{"H232 627"}="2H23 627"; $resn_exch{"H233 627"}="3H23 627";
$resn_exch{"H361 627"}="1H36 627"; $resn_exch{"H362 627"}="2H36 627"; $resn_exch{"H363 627"}="3H36 627";
$resn_exch{"HN11 628"}="1HN1 628"; $resn_exch{"HN12 628"}="2HN1 628"; $resn_exch{"H141 629"}="1H14 629";
$resn_exch{"H142 629"}="2H14 629"; $resn_exch{"H161 629"}="1H16 629"; $resn_exch{"H162 629"}="2H16 629";
$resn_exch{" H41 629"}="1H4  629"; $resn_exch{" H42 629"}="2H4  629"; $resn_exch{"H301 629"}="1H30 629";
$resn_exch{"H302 629"}="2H30 629"; $resn_exch{"H21A 62A"}="1H2  62A"; $resn_exch{"H22A 62A"}="2H2  62A";
$resn_exch{" H31 62A"}="1H3  62A"; $resn_exch{" H32 62A"}="2H3  62A"; $resn_exch{" H41 62A"}="1H4  62A";
$resn_exch{" H42 62A"}="2H4  62A"; $resn_exch{"HN71 62A"}="1HN7 62A"; $resn_exch{"HN72 62A"}="2HN7 62A";
$resn_exch{"H111 62A"}="1H11 62A"; $resn_exch{"H112 62A"}="2H11 62A"; $resn_exch{"H181 62A"}="1H18 62A";
$resn_exch{"H182 62A"}="2H18 62A"; $resn_exch{"H251 62A"}="1H25 62A"; $resn_exch{"H252 62A"}="2H25 62A";
$resn_exch{"H253 62A"}="3H25 62A"; $resn_exch{"H201 62P"}="1H20 62P"; $resn_exch{"H202 62P"}="2H20 62P";
$resn_exch{"H203 62P"}="3H20 62P"; $resn_exch{" H61 632"}="1H6  632"; $resn_exch{" H62 632"}="2H6  632";
$resn_exch{" H71 632"}="1H7  632"; $resn_exch{" H72 632"}="2H7  632"; $resn_exch{" H81 632"}="1H8  632";
$resn_exch{" H82 632"}="2H8  632"; $resn_exch{"H111 632"}="1H11 632"; $resn_exch{"H112 632"}="2H11 632";
$resn_exch{"H151 632"}="1H15 632"; $resn_exch{"H152 632"}="2H15 632"; $resn_exch{"H221 632"}="1H22 632";
$resn_exch{"H222 632"}="2H22 632"; $resn_exch{"H321 632"}="1H32 632"; $resn_exch{"H322 632"}="2H32 632";
$resn_exch{"H323 632"}="3H32 632"; $resn_exch{"H21A 638"}="1H2  638"; $resn_exch{" H22 638"}="2H2  638";
$resn_exch{"H71A 638"}="1H7  638"; $resn_exch{" H72 638"}="2H7  638"; $resn_exch{"H311 638"}="1H31 638";
$resn_exch{"H312 638"}="2H31 638"; $resn_exch{"H611 638"}="1H61 638"; $resn_exch{"H612 638"}="2H61 638";
$resn_exch{"H101 641"}="1H10 641"; $resn_exch{"H102 641"}="2H10 641"; $resn_exch{" H81 641"}="1H8  641";
$resn_exch{" H82 641"}="2H8  641"; $resn_exch{" H41 641"}="1H4  641"; $resn_exch{" H42 641"}="2H4  641";
$resn_exch{" H51 641"}="1H5  641"; $resn_exch{" H52 641"}="2H5  641"; $resn_exch{" H61 641"}="1H6  641";
$resn_exch{" H62 641"}="2H6  641"; $resn_exch{"H101 645"}="1H10 645"; $resn_exch{"H102 645"}="2H10 645";
$resn_exch{"H103 645"}="3H10 645"; $resn_exch{"H131 645"}="1H13 645"; $resn_exch{"H132 645"}="2H13 645";
$resn_exch{"H133 645"}="3H13 645"; $resn_exch{"H141 645"}="1H14 645"; $resn_exch{"H142 645"}="2H14 645";
$resn_exch{"H143 645"}="3H14 645"; $resn_exch{"H151 645"}="1H15 645"; $resn_exch{"H152 645"}="2H15 645";
$resn_exch{"H153 645"}="3H15 645"; $resn_exch{"HOP2 64T"}="2HOP 64T"; $resn_exch{" H5' 64T"}="1H5* 64T";
$resn_exch{"H5'' 64T"}="2H5* 64T"; $resn_exch{" H4' 64T"}=" H4* 64T"; $resn_exch{" H3' 64T"}=" H3* 64T";
$resn_exch{"HO3' 64T"}="H3T  64T"; $resn_exch{" H2' 64T"}="1H2* 64T"; $resn_exch{"H2'' 64T"}="2H2* 64T";
$resn_exch{" H1' 64T"}=" H1* 64T"; $resn_exch{" H71 64T"}="1H5M 64T"; $resn_exch{" H72 64T"}="2H5M 64T";
$resn_exch{" H73 64T"}="3H5M 64T"; $resn_exch{" H61 64T"}="1H6  64T"; $resn_exch{" H62 64T"}="2H6  64T";
$resn_exch{"HOP3 64T"}="3HOP 64T"; $resn_exch{" H2B 653"}="H2'' 653"; $resn_exch{" H3B 653"}="H3'' 653";
$resn_exch{" H4B 653"}="H4'' 653"; $resn_exch{" H5B 653"}="H5'' 653"; $resn_exch{" H6B 653"}="H6'' 653";
$resn_exch{"H3TC 654"}="CH3T 654"; $resn_exch{"H4TC 654"}="CH4T 654"; $resn_exch{"H5TC 654"}="CH5T 654";
$resn_exch{"H3IC 654"}="CH3I 654"; $resn_exch{"H5IC 654"}="CH5I 654"; $resn_exch{"HC11 654"}="1HC1 654";
$resn_exch{"HC12 654"}="2HC1 654"; $resn_exch{"HC81 654"}="1HC8 654"; $resn_exch{"HC82 654"}="2HC8 654";
$resn_exch{"HC83 654"}="3HC8 654"; $resn_exch{"H2'1 65B"}="1H2' 65B"; $resn_exch{"H2'2 65B"}="2H2' 65B";
$resn_exch{"H2'3 65B"}="3H2' 65B"; $resn_exch{"H6'1 65B"}="1H6' 65B"; $resn_exch{"H6'2 65B"}="2H6' 65B";
$resn_exch{"H6'3 65B"}="3H6' 65B"; $resn_exch{"H181 65B"}="1H18 65B"; $resn_exch{"H182 65B"}="2H18 65B";
$resn_exch{"H341 663"}="1H34 663"; $resn_exch{"H342 663"}="2H34 663"; $resn_exch{"H343 663"}="3H34 663";
$resn_exch{"H281 663"}="1H28 663"; $resn_exch{"H282 663"}="2H28 663"; $resn_exch{"H181 663"}="1H18 663";
$resn_exch{"H182 663"}="2H18 663"; $resn_exch{"H183 663"}="3H18 663"; $resn_exch{" H41 665"}="1H4  665";
$resn_exch{" H42 665"}="2H4  665"; $resn_exch{" H81 665"}="1H8  665"; $resn_exch{" H82 665"}="2H8  665";
$resn_exch{"H101 665"}="1H10 665"; $resn_exch{"H102 665"}="2H10 665"; $resn_exch{"H111 666"}="1H11 666";
$resn_exch{"H112 666"}="2H11 666"; $resn_exch{"H161 666"}="1H16 666"; $resn_exch{"H162 666"}="2H16 666";
$resn_exch{"H171 666"}="1H17 666"; $resn_exch{"H172 666"}="2H17 666"; $resn_exch{"H181 666"}="1H18 666";
$resn_exch{"H182 666"}="2H18 666"; $resn_exch{"H411 666"}="1H41 666"; $resn_exch{"H412 666"}="2H41 666";
$resn_exch{"H471 666"}="1H47 666"; $resn_exch{"H472 666"}="2H47 666"; $resn_exch{"H473 666"}="3H47 666";
$resn_exch{"HN11 667"}="1HN1 667"; $resn_exch{"HN12 667"}="2HN1 667"; $resn_exch{"H101 667"}="1H10 667";
$resn_exch{"H102 667"}="2H10 667"; $resn_exch{"H111 667"}="1H11 667"; $resn_exch{"H112 667"}="2H11 667";
$resn_exch{"H121 667"}="1H12 667"; $resn_exch{"H122 667"}="2H12 667"; $resn_exch{"H131 667"}="1H13 667";
$resn_exch{"H132 667"}="2H13 667"; $resn_exch{"H141 667"}="1H14 667"; $resn_exch{"H142 667"}="2H14 667";
$resn_exch{"H041 669"}="1H04 669"; $resn_exch{"H042 669"}="2H04 669"; $resn_exch{"H051 669"}="1H05 669";
$resn_exch{"H052 669"}="2H05 669"; $resn_exch{"H061 669"}="1H06 669"; $resn_exch{"H062 669"}="2H06 669";
$resn_exch{"H071 669"}="1H07 669"; $resn_exch{"H072 669"}="2H07 669"; $resn_exch{"H081 669"}="1H08 669";
$resn_exch{"H082 669"}="2H08 669"; $resn_exch{"H161 669"}="1H16 669"; $resn_exch{"H162 669"}="2H16 669";
$resn_exch{"H41A 672"}="1H4  672"; $resn_exch{" H42 672"}="2H4  672"; $resn_exch{" H43 672"}="3H4  672";
$resn_exch{"H151 672"}="1H15 672"; $resn_exch{"H152 672"}="2H15 672"; $resn_exch{"H153 672"}="3H15 672";
$resn_exch{"H141 672"}="1H14 672"; $resn_exch{"H142 672"}="2H14 672"; $resn_exch{"H143 672"}="3H14 672";
$resn_exch{"H231 672"}="1H23 672"; $resn_exch{"H232 672"}="2H23 672"; $resn_exch{"H321 672"}="1H32 672";
$resn_exch{"H322 672"}="2H32 672"; $resn_exch{"H381 672"}="1H38 672"; $resn_exch{"H382 672"}="2H38 672";
$resn_exch{"H371 672"}="1H37 672"; $resn_exch{"H372 672"}="2H37 672"; $resn_exch{"H461 672"}="1H46 672";
$resn_exch{"H462 672"}="2H46 672"; $resn_exch{"H451 672"}="1H45 672"; $resn_exch{"H452 672"}="2H45 672";
$resn_exch{"H491 672"}="1H49 672"; $resn_exch{"H492 672"}="2H49 672"; $resn_exch{"H431 672"}="1H43 672";
$resn_exch{"H432 672"}="2H43 672"; $resn_exch{"H421 672"}="1H42 672"; $resn_exch{"H422 672"}="2H42 672";
$resn_exch{"H351 672"}="1H35 672"; $resn_exch{"H352 672"}="2H35 672"; $resn_exch{"H341 672"}="1H34 672";
$resn_exch{"H342 672"}="2H34 672"; $resn_exch{"H121 672"}="1H12 672"; $resn_exch{"H122 672"}="2H12 672";
$resn_exch{"H123 672"}="3H12 672"; $resn_exch{"H111 672"}="1H11 672"; $resn_exch{"H112 672"}="2H11 672";
$resn_exch{"H113 672"}="3H11 672"; $resn_exch{" H11 672"}="1H1  672"; $resn_exch{" H12 672"}="2H1  672";
$resn_exch{"H13A 672"}="3H1  672"; $resn_exch{"H191 675"}="1H19 675"; $resn_exch{"H192 675"}="2H19 675";
$resn_exch{" H51 680"}="1H5  680"; $resn_exch{" H52 680"}="2H5  680"; $resn_exch{" H53 680"}="3H5  680";
$resn_exch{"HN51 680"}="1HN5 680"; $resn_exch{"HN52 680"}="2HN5 680"; $resn_exch{" H11 681"}="1H1  681";
$resn_exch{" H51 681"}="1H5  681"; $resn_exch{" H52 681"}="2H5  681"; $resn_exch{" H53 681"}="3H5  681";
$resn_exch{" H61 681"}="1H6  681"; $resn_exch{" H62 681"}="2H6  681"; $resn_exch{" H63 681"}="3H6  681";
$resn_exch{" H71 681"}="1H7  681"; $resn_exch{" H72 681"}="2H7  681"; $resn_exch{" H73 681"}="3H7  681";
$resn_exch{"HVA1 681"}="1HVA 681"; $resn_exch{"HVB1 681"}="1HVB 681"; $resn_exch{"HV11 681"}="1HV1 681";
$resn_exch{"HV12 681"}="2HV1 681"; $resn_exch{"HV13 681"}="3HV1 681"; $resn_exch{"HV21 681"}="1HV2 681";
$resn_exch{"HV22 681"}="2HV2 681"; $resn_exch{"HV23 681"}="3HV2 681"; $resn_exch{"HNV1 681"}="1HNV 681";
$resn_exch{"HAG1 681"}="1HAG 681"; $resn_exch{"HAG2 681"}="2HAG 681"; $resn_exch{"H2X1 681"}="1H2X 681";
$resn_exch{"H3X1 681"}="1H3X 681"; $resn_exch{"H3X2 681"}="2H3X 681"; $resn_exch{"H6X1 681"}="1H6X 681";
$resn_exch{"H7X1 681"}="1H7X 681"; $resn_exch{"H8X1 681"}="1H8X 681"; $resn_exch{"H9X1 681"}="1H9X 681";
$resn_exch{"H0X1 681"}="1H0X 681"; $resn_exch{" H61 682"}="1H6  682"; $resn_exch{" H62 682"}="2H6  682";
$resn_exch{" H71 682"}="1H7  682"; $resn_exch{" H72 682"}="2H7  682"; $resn_exch{" H81 682"}="1H8  682";
$resn_exch{" H82 682"}="2H8  682"; $resn_exch{"H121 682"}="1H12 682"; $resn_exch{"H122 682"}="2H12 682";
$resn_exch{"H211 682"}="1H21 682"; $resn_exch{"H212 682"}="2H21 682"; $resn_exch{"H213 682"}="3H21 682";
$resn_exch{"H221 682"}="1H22 682"; $resn_exch{"H222 682"}="2H22 682"; $resn_exch{"H223 682"}="3H22 682";
$resn_exch{"H241 682"}="1H24 682"; $resn_exch{"H242 682"}="2H24 682"; $resn_exch{"H111 687"}="1H11 687";
$resn_exch{"H112 687"}="2H11 687"; $resn_exch{"H171 687"}="1H17 687"; $resn_exch{"H172 687"}="2H17 687";
$resn_exch{"H173 687"}="3H17 687"; $resn_exch{"H211 687"}="1H21 687"; $resn_exch{"H212 687"}="2H21 687";
$resn_exch{"H221 687"}="1H22 687"; $resn_exch{"H222 687"}="2H22 687"; $resn_exch{"H231 687"}="1H23 687";
$resn_exch{"H232 687"}="2H23 687"; $resn_exch{"H241 687"}="1H24 687"; $resn_exch{"H242 687"}="2H24 687";
$resn_exch{"H271 687"}="1H27 687"; $resn_exch{"H272 687"}="2H27 687"; $resn_exch{"H121 688"}="1H12 688";
$resn_exch{"H122 688"}="2H12 688"; $resn_exch{"HAN1 689"}="1HAN 689"; $resn_exch{"HAN2 689"}="2HAN 689";
$resn_exch{"HAN3 689"}="3HAN 689"; $resn_exch{"HAR1 689"}="1HAR 689"; $resn_exch{"HAR2 689"}="2HAR 689";
$resn_exch{"HAR3 689"}="3HAR 689"; $resn_exch{"HAS1 689"}="1HAS 689"; $resn_exch{"HAS2 689"}="2HAS 689";
$resn_exch{"HAS3 689"}="3HAS 689"; $resn_exch{"HAT1 689"}="1HAT 689"; $resn_exch{"HAT2 689"}="2HAT 689";
$resn_exch{"HAM1 689"}="1HAM 689"; $resn_exch{"HAM2 689"}="2HAM 689"; $resn_exch{" H3B 693"}="H3'' 693";
$resn_exch{" H4B 693"}="H4'' 693"; $resn_exch{" H5B 693"}="H5'' 693"; $resn_exch{" H6B 693"}="H6'' 693";
$resn_exch{"H121 694"}="1H12 694"; $resn_exch{"H122 694"}="2H12 694"; $resn_exch{" H2B 696"}="H2'' 696";
$resn_exch{" H3B 696"}="H3'' 696"; $resn_exch{" H4B 696"}="H4'' 696"; $resn_exch{" H5B 696"}="H5'' 696";
$resn_exch{" H6B 696"}="H6'' 696"; $resn_exch{"H161 698"}="1H16 698"; $resn_exch{"H162 698"}="2H16 698";
$resn_exch{"H331 698"}="1H33 698"; $resn_exch{"H332 698"}="2H33 698"; $resn_exch{"H401 698"}="1H40 698";
$resn_exch{"H402 698"}="2H40 698"; $resn_exch{"H403 698"}="3H40 698"; $resn_exch{"H441 698"}="1H44 698";
$resn_exch{"H442 698"}="2H44 698"; $resn_exch{"H443 698"}="3H44 698"; $resn_exch{"H9C1 698"}="1H9C 698";
$resn_exch{"H9C2 698"}="2H9C 698"; $resn_exch{"H9C3 698"}="3H9C 698"; $resn_exch{"H8C1 698"}="1H8C 698";
$resn_exch{"H8C2 698"}="2H8C 698"; $resn_exch{"H8C3 698"}="3H8C 698"; $resn_exch{"H7C1 698"}="1H7C 698";
$resn_exch{"H7C2 698"}="2H7C 698"; $resn_exch{"H7C3 698"}="3H7C 698"; $resn_exch{"H331 699"}="1H33 699";
$resn_exch{"H332 699"}="2H33 699"; $resn_exch{"H161 699"}="1H16 699"; $resn_exch{"H162 699"}="2H16 699";
$resn_exch{"H441 699"}="1H44 699"; $resn_exch{"H442 699"}="2H44 699"; $resn_exch{"H443 699"}="3H44 699";
$resn_exch{"H401 699"}="1H40 699"; $resn_exch{"H402 699"}="2H40 699"; $resn_exch{"H403 699"}="3H40 699";
$resn_exch{"HO2' 6AB"}="'HO2 6AB"; $resn_exch{"HN61 6AB"}="1HN6 6AB"; $resn_exch{"HN62 6AB"}="2HN6 6AB";
$resn_exch{"HN21 6AP"}="1HN2 6AP"; $resn_exch{"HN22 6AP"}="2HN2 6AP"; $resn_exch{"HN61 6AP"}="1HN6 6AP";
$resn_exch{"HN62 6AP"}="2HN6 6AP"; $resn_exch{"H191 6C3"}="1H19 6C3"; $resn_exch{"H192 6C3"}="2H19 6C3";
$resn_exch{"H193 6C3"}="3H19 6C3"; $resn_exch{"H221 6CA"}="1H22 6CA"; $resn_exch{"H222 6CA"}="2H22 6CA";
$resn_exch{"H211 6CA"}="1H21 6CA"; $resn_exch{"H212 6CA"}="2H21 6CA"; $resn_exch{"H201 6CA"}="1H20 6CA";
$resn_exch{"H202 6CA"}="2H20 6CA"; $resn_exch{"H191 6CA"}="1H19 6CA"; $resn_exch{"H192 6CA"}="2H19 6CA";
$resn_exch{"H181 6CA"}="1H18 6CA"; $resn_exch{"H182 6CA"}="2H18 6CA"; $resn_exch{"H171 6CA"}="1H17 6CA";
$resn_exch{"H172 6CA"}="2H17 6CA"; $resn_exch{" H2  6CL"}=" HN2 6CL"; $resn_exch{" HB2 6CL"}=" HB1 6CL";
$resn_exch{" HB3 6CL"}=" HB2 6CL"; $resn_exch{" HG2 6CL"}=" HG1 6CL"; $resn_exch{" HG3 6CL"}=" HG2 6CL";
$resn_exch{" HD2 6CL"}=" HD1 6CL"; $resn_exch{" HD3 6CL"}=" HD2 6CL"; $resn_exch{" H11 6CM"}="1H1  6CM";
$resn_exch{" H12 6CM"}="2H1  6CM"; $resn_exch{" H61 6CM"}="1H6  6CM"; $resn_exch{" H62 6CM"}="2H6  6CM";
$resn_exch{" H71 6CM"}="1H7  6CM"; $resn_exch{" H72 6CM"}="2H7  6CM"; $resn_exch{" H81 6CM"}="1H8  6CM";
$resn_exch{" H82 6CM"}="2H8  6CM"; $resn_exch{" H91 6CM"}="1H9  6CM"; $resn_exch{" H92 6CM"}="2H9  6CM";
$resn_exch{"H101 6CM"}="1H10 6CM"; $resn_exch{"H102 6CM"}="2H10 6CM"; $resn_exch{"H111 6CM"}="1H11 6CM";
$resn_exch{"H112 6CM"}="2H11 6CM"; $resn_exch{"H121 6CM"}="1H12 6CM"; $resn_exch{"H122 6CM"}="2H12 6CM";
$resn_exch{"H131 6CM"}="1H13 6CM"; $resn_exch{"H132 6CM"}="2H13 6CM"; $resn_exch{"H141 6CM"}="1H14 6CM";
$resn_exch{"H142 6CM"}="2H14 6CM"; $resn_exch{"H151 6CM"}="1H15 6CM"; $resn_exch{"H152 6CM"}="2H15 6CM";
$resn_exch{"H161 6CM"}="1H16 6CM"; $resn_exch{"H162 6CM"}="2H16 6CM"; $resn_exch{"H171 6CM"}="1H17 6CM";
$resn_exch{"H172 6CM"}="2H17 6CM"; $resn_exch{"H181 6CM"}="1H18 6CM"; $resn_exch{"H182 6CM"}="2H18 6CM";
$resn_exch{"H183 6CM"}="3H18 6CM"; $resn_exch{"H201 6CM"}="1H20 6CM"; $resn_exch{"H202 6CM"}="2H20 6CM";
$resn_exch{"H211 6CM"}="1H21 6CM"; $resn_exch{"H212 6CM"}="2H21 6CM"; $resn_exch{"H221 6CM"}="1H22 6CM";
$resn_exch{"H222 6CM"}="2H22 6CM"; $resn_exch{"H231 6CM"}="1H23 6CM"; $resn_exch{"H232 6CM"}="2H23 6CM";
$resn_exch{"H241 6CM"}="1H24 6CM"; $resn_exch{"H242 6CM"}="2H24 6CM"; $resn_exch{"H243 6CM"}="3H24 6CM";
$resn_exch{"H101 6CP"}="1H10 6CP"; $resn_exch{"H102 6CP"}="2H10 6CP"; $resn_exch{"H121 6CP"}="1H12 6CP";
$resn_exch{"H122 6CP"}="2H12 6CP"; $resn_exch{"H131 6CP"}="1H13 6CP"; $resn_exch{"H132 6CP"}="2H13 6CP";
$resn_exch{"H141 6CP"}="1H14 6CP"; $resn_exch{"H142 6CP"}="2H14 6CP"; $resn_exch{"H151 6CP"}="1H15 6CP";
$resn_exch{"H152 6CP"}="2H15 6CP"; $resn_exch{"H161 6CP"}="1H16 6CP"; $resn_exch{"H162 6CP"}="2H16 6CP";
$resn_exch{"HAA1 6CS"}="1HAA 6CS"; $resn_exch{"HAA2 6CS"}="2HAA 6CS"; $resn_exch{"HAA3 6CS"}="3HAA 6CS";
$resn_exch{"HG21 6CS"}="1HG2 6CS"; $resn_exch{"HG22 6CS"}="2HG2 6CS"; $resn_exch{" H5' 6CT"}="1H5* 6CT";
$resn_exch{"H5'' 6CT"}="2H5* 6CT"; $resn_exch{" H4' 6CT"}=" H4* 6CT"; $resn_exch{" H3' 6CT"}=" H3* 6CT";
$resn_exch{"HO3' 6CT"}="*HO3 6CT"; $resn_exch{" H2' 6CT"}="1H2* 6CT"; $resn_exch{"H2'' 6CT"}="2H2* 6CT";
$resn_exch{" H1' 6CT"}=" H1* 6CT"; $resn_exch{" H71 6CT"}="1H5M 6CT"; $resn_exch{" H72 6CT"}="2H5M 6CT";
$resn_exch{" H73 6CT"}="3H5M 6CT"; $resn_exch{" H'  6CT"}=" H*  6CT"; $resn_exch{"HM'1 6CT"}="1HM* 6CT";
$resn_exch{"HM'2 6CT"}="2HM* 6CT"; $resn_exch{"HM'3 6CT"}="3HM* 6CT"; $resn_exch{" OP3 6CT"}=" O3P 6CT";
$resn_exch{"HOP3 6CT"}="3HOP 6CT"; $resn_exch{" H   6CW"}="1HN  6CW"; $resn_exch{" H2  6CW"}="2HN  6CW";
$resn_exch{" HB2 6CW"}="1HB  6CW"; $resn_exch{" HB3 6CW"}="2HB  6CW"; $resn_exch{"H161 6DE"}="1H16 6DE";
$resn_exch{"H162 6DE"}="2H16 6DE"; $resn_exch{"H171 6DE"}="1H17 6DE"; $resn_exch{"H172 6DE"}="2H17 6DE";
$resn_exch{"H173 6DE"}="3H17 6DE"; $resn_exch{"H191 6DE"}="1H19 6DE"; $resn_exch{"H192 6DE"}="2H19 6DE";
$resn_exch{"H193 6DE"}="3H19 6DE"; $resn_exch{"H111 6DE"}="1H11 6DE"; $resn_exch{"H112 6DE"}="2H11 6DE";
$resn_exch{"H113 6DE"}="3H11 6DE"; $resn_exch{"H261 6EA"}="1H26 6EA"; $resn_exch{"H262 6EA"}="2H26 6EA";
$resn_exch{"H301 6EA"}="1H30 6EA"; $resn_exch{"H302 6EA"}="2H30 6EA"; $resn_exch{"H311 6EA"}="1H31 6EA";
$resn_exch{"H312 6EA"}="2H31 6EA"; $resn_exch{"HM71 6FA"}="1HM7 6FA"; $resn_exch{"HM72 6FA"}="2HM7 6FA";
$resn_exch{"HM73 6FA"}="3HM7 6FA"; $resn_exch{"HM81 6FA"}="1HM8 6FA"; $resn_exch{"HM82 6FA"}="2HM8 6FA";
$resn_exch{"HM83 6FA"}="3HM8 6FA"; $resn_exch{"H1'1 6FA"}="1H1* 6FA"; $resn_exch{"H1'2 6FA"}="2H1* 6FA";
$resn_exch{" H2' 6FA"}=" H2* 6FA"; $resn_exch{"HO2' 6FA"}="*HO2 6FA"; $resn_exch{" H3' 6FA"}=" H3* 6FA";
$resn_exch{"HO3' 6FA"}="*HO3 6FA"; $resn_exch{" H4' 6FA"}=" H4* 6FA"; $resn_exch{"HO4' 6FA"}="*HO4 6FA";
$resn_exch{"H5'1 6FA"}="1H5* 6FA"; $resn_exch{"H5'2 6FA"}="2H5* 6FA"; $resn_exch{"HOP2 6FA"}="2HOP 6FA";
$resn_exch{"HOA2 6FA"}="2HOA 6FA"; $resn_exch{"H51A 6FA"}="AH51 6FA"; $resn_exch{"H52A 6FA"}="AH52 6FA";
$resn_exch{" H4B 6FA"}="AH4* 6FA"; $resn_exch{" H3B 6FA"}="AH3* 6FA"; $resn_exch{"HO3A 6FA"}="AHO3 6FA";
$resn_exch{" H2B 6FA"}="AH2* 6FA"; $resn_exch{"HO2A 6FA"}="AHO2 6FA"; $resn_exch{" H1B 6FA"}="AH1* 6FA";
$resn_exch{" H8A 6FA"}=" AH8 6FA"; $resn_exch{"H61A 6FA"}="AH61 6FA"; $resn_exch{"H62A 6FA"}="AH62 6FA";
$resn_exch{" H2A 6FA"}=" AH2 6FA"; $resn_exch{" H61 6GP"}="1H6  6GP"; $resn_exch{" H62 6GP"}="2H6  6GP";
$resn_exch{" H91 6GP"}="1H9  6GP"; $resn_exch{" H92 6GP"}="2H9  6GP"; $resn_exch{" H93 6GP"}="3H9  6GP";
$resn_exch{"HOP2 6HA"}="2HOP 6HA"; $resn_exch{"HOP3 6HA"}="3HOP 6HA"; $resn_exch{" H5' 6HA"}="1H5* 6HA";
$resn_exch{"H5'' 6HA"}="2H5* 6HA"; $resn_exch{" H4' 6HA"}=" H4* 6HA"; $resn_exch{" H3' 6HA"}=" H3* 6HA";
$resn_exch{"HO3' 6HA"}="HO3* 6HA"; $resn_exch{" H2' 6HA"}="1H2* 6HA"; $resn_exch{"H2'' 6HA"}="2H2* 6HA";
$resn_exch{" H1' 6HA"}=" H1* 6HA"; $resn_exch{"H6'1 6HA"}="1H6* 6HA"; $resn_exch{"H6'2 6HA"}="2H6* 6HA";
$resn_exch{" H61 6HA"}="1H6  6HA"; $resn_exch{" H62 6HA"}="2H6  6HA"; $resn_exch{"HOP2 6HC"}="2HOP 6HC";
$resn_exch{"HOP3 6HC"}="3HOP 6HC"; $resn_exch{" H5' 6HC"}="1H5* 6HC"; $resn_exch{"H5'' 6HC"}="2H5* 6HC";
$resn_exch{" H4' 6HC"}=" H4* 6HC"; $resn_exch{" H3' 6HC"}=" H3* 6HC"; $resn_exch{"HO3' 6HC"}="HO3* 6HC";
$resn_exch{" H2' 6HC"}="1H2* 6HC"; $resn_exch{"H2'' 6HC"}="2H2* 6HC"; $resn_exch{" H1' 6HC"}=" H1* 6HC";
$resn_exch{"H6'1 6HC"}="1H6* 6HC"; $resn_exch{"H6'2 6HC"}="2H6* 6HC"; $resn_exch{" H41 6HC"}="1H4  6HC";
$resn_exch{" H42 6HC"}="2H4  6HC"; $resn_exch{"HMA1 6HE"}="1HMA 6HE"; $resn_exch{"HMA2 6HE"}="2HMA 6HE";
$resn_exch{"HMA3 6HE"}="3HMA 6HE"; $resn_exch{"HAA1 6HE"}="1HAA 6HE"; $resn_exch{"HAA2 6HE"}="2HAA 6HE";
$resn_exch{"HBA1 6HE"}="1HBA 6HE"; $resn_exch{"HBA2 6HE"}="2HBA 6HE"; $resn_exch{"HMB1 6HE"}="1HMB 6HE";
$resn_exch{"HMB2 6HE"}="2HMB 6HE"; $resn_exch{"HMB3 6HE"}="3HMB 6HE"; $resn_exch{"HBB1 6HE"}="1HBB 6HE";
$resn_exch{"HBB2 6HE"}="2HBB 6HE"; $resn_exch{"HMC1 6HE"}="1HMC 6HE"; $resn_exch{"HMC2 6HE"}="2HMC 6HE";
$resn_exch{"HMC3 6HE"}="3HMC 6HE"; $resn_exch{"HBC1 6HE"}="1HBC 6HE"; $resn_exch{"HBC2 6HE"}="2HBC 6HE";
$resn_exch{"HMD1 6HE"}="1HMD 6HE"; $resn_exch{"HMD2 6HE"}="2HMD 6HE"; $resn_exch{"HMD3 6HE"}="3HMD 6HE";
$resn_exch{"HAD1 6HE"}="1HAD 6HE"; $resn_exch{"HAD2 6HE"}="2HAD 6HE"; $resn_exch{"HAD3 6HE"}="3HAD 6HE";
$resn_exch{"HOP2 6HG"}="2HOP 6HG"; $resn_exch{"HOP3 6HG"}="3HOP 6HG"; $resn_exch{" H5' 6HG"}="1H5* 6HG";
$resn_exch{"H5'' 6HG"}="2H5* 6HG"; $resn_exch{" H4' 6HG"}=" H4* 6HG"; $resn_exch{" H3' 6HG"}=" H3* 6HG";
$resn_exch{"HO3' 6HG"}="HO3* 6HG"; $resn_exch{" H2' 6HG"}="1H2* 6HG"; $resn_exch{"H2'' 6HG"}="2H2* 6HG";
$resn_exch{" H1' 6HG"}=" H1* 6HG"; $resn_exch{"H6'1 6HG"}="1H6* 6HG"; $resn_exch{"H6'2 6HG"}="2H6* 6HG";
$resn_exch{" H21 6HG"}="1H2  6HG"; $resn_exch{" H22 6HG"}="2H2  6HG"; $resn_exch{"HOP2 6HT"}="2HOP 6HT";
$resn_exch{"HOP3 6HT"}="3HOP 6HT"; $resn_exch{" H5' 6HT"}="1H5* 6HT"; $resn_exch{"H5'' 6HT"}="2H5* 6HT";
$resn_exch{" H4' 6HT"}=" H4* 6HT"; $resn_exch{" H3' 6HT"}=" H3* 6HT"; $resn_exch{"HO3' 6HT"}="HO3* 6HT";
$resn_exch{" H2' 6HT"}="1H2* 6HT"; $resn_exch{"H2'' 6HT"}="2H2* 6HT"; $resn_exch{" H1' 6HT"}=" H1* 6HT";
$resn_exch{"H6'1 6HT"}="1H6* 6HT"; $resn_exch{"H6'2 6HT"}="2H6* 6HT"; $resn_exch{" H71 6HT"}="1H5M 6HT";
$resn_exch{" H72 6HT"}="2H5M 6HT"; $resn_exch{" H73 6HT"}="3H5M 6HT"; $resn_exch{"HOP3 6IA"}="3HOP 6IA";
$resn_exch{"HOP2 6IA"}="2HOP 6IA"; $resn_exch{" H5' 6IA"}="1H5* 6IA"; $resn_exch{"H5'' 6IA"}="2H5* 6IA";
$resn_exch{" H4' 6IA"}=" H4* 6IA"; $resn_exch{" H3' 6IA"}=" H3* 6IA"; $resn_exch{"HO3' 6IA"}="H3T  6IA";
$resn_exch{" H2' 6IA"}=" H2* 6IA"; $resn_exch{"HO2' 6IA"}="2HO* 6IA"; $resn_exch{" H1' 6IA"}=" H1* 6IA";
$resn_exch{"H121 6IA"}="1H12 6IA"; $resn_exch{"H122 6IA"}="2H12 6IA"; $resn_exch{"H131 6IA"}="1H13 6IA";
$resn_exch{"H132 6IA"}="2H13 6IA"; $resn_exch{"H151 6IA"}="1H15 6IA"; $resn_exch{"H152 6IA"}="2H15 6IA";
$resn_exch{"H153 6IA"}="3H15 6IA"; $resn_exch{"H161 6IA"}="1H16 6IA"; $resn_exch{"H162 6IA"}="2H16 6IA";
$resn_exch{"H163 6IA"}="3H16 6IA"; $resn_exch{"HN41 6IG"}="1HN4 6IG"; $resn_exch{"HN42 6IG"}="2HN4 6IG";
$resn_exch{" H51 6IG"}="1H5  6IG"; $resn_exch{" H52 6IG"}="2H5  6IG"; $resn_exch{" H61 6IG"}="1H6  6IG";
$resn_exch{" H62 6IG"}="2H6  6IG"; $resn_exch{" H63 6IG"}="3H6  6IG"; $resn_exch{"H131 6IG"}="1H13 6IG";
$resn_exch{"H132 6IG"}="2H13 6IG"; $resn_exch{"H141 6IG"}="1H14 6IG"; $resn_exch{"H142 6IG"}="2H14 6IG";
$resn_exch{"H151 6IG"}="1H15 6IG"; $resn_exch{"H152 6IG"}="2H15 6IG"; $resn_exch{"H161 6IG"}="1H16 6IG";
$resn_exch{"H162 6IG"}="2H16 6IG"; $resn_exch{"H171 6IG"}="1H17 6IG"; $resn_exch{"H172 6IG"}="2H17 6IG";
$resn_exch{"H181 6IG"}="1H18 6IG"; $resn_exch{"H182 6IG"}="2H18 6IG"; $resn_exch{"H191 6IG"}="1H19 6IG";
$resn_exch{"H192 6IG"}="2H19 6IG"; $resn_exch{"H193 6IG"}="3H19 6IG"; $resn_exch{"H201 6IG"}="1H20 6IG";
$resn_exch{"H202 6IG"}="2H20 6IG"; $resn_exch{"H211 6IG"}="1H21 6IG"; $resn_exch{"H212 6IG"}="2H21 6IG";
$resn_exch{"H131 6IN"}="1H13 6IN"; $resn_exch{"H132 6IN"}="2H13 6IN"; $resn_exch{"H271 6IN"}="1H27 6IN";
$resn_exch{"H272 6IN"}="2H27 6IN"; $resn_exch{"H273 6IN"}="3H27 6IN"; $resn_exch{"H321 6IN"}="1H32 6IN";
$resn_exch{"H322 6IN"}="2H32 6IN"; $resn_exch{"H371 6IN"}="1H37 6IN"; $resn_exch{"H372 6IN"}="2H37 6IN";
$resn_exch{"H421 6IN"}="1H42 6IN"; $resn_exch{"H422 6IN"}="2H42 6IN"; $resn_exch{"H431 6IN"}="1H43 6IN";
$resn_exch{"H432 6IN"}="2H43 6IN"; $resn_exch{"H461 6IN"}="1H46 6IN"; $resn_exch{"H462 6IN"}="2H46 6IN";
$resn_exch{"HN11 6IP"}="1HN1 6IP"; $resn_exch{"HN12 6IP"}="2HN1 6IP"; $resn_exch{" H71 6IP"}="1H7  6IP";
$resn_exch{" H72 6IP"}="2H7  6IP"; $resn_exch{" H81 6IP"}="1H8  6IP"; $resn_exch{" H82 6IP"}="2H8  6IP";
$resn_exch{"HOP3 6MA"}="3HOP 6MA"; $resn_exch{"HOP2 6MA"}="2HOP 6MA"; $resn_exch{" H5' 6MA"}="1H5* 6MA";
$resn_exch{"H5'' 6MA"}="2H5* 6MA"; $resn_exch{" H4' 6MA"}=" H4* 6MA"; $resn_exch{" H3' 6MA"}=" H3* 6MA";
$resn_exch{"HO3' 6MA"}="H3T  6MA"; $resn_exch{" H2' 6MA"}="1H2* 6MA"; $resn_exch{"H2'' 6MA"}="2H2* 6MA";
$resn_exch{" H1' 6MA"}=" H1* 6MA"; $resn_exch{" H61 6MA"}="1H6  6MA"; $resn_exch{" H11 6MA"}="1H1  6MA";
$resn_exch{" H12 6MA"}="2H1  6MA"; $resn_exch{" H13 6MA"}="3H1  6MA"; $resn_exch{"HOP3 6MC"}="3HOP 6MC";
$resn_exch{"HOP2 6MC"}="2HOP 6MC"; $resn_exch{" H5' 6MC"}="1H5* 6MC"; $resn_exch{"H5'' 6MC"}="2H5* 6MC";
$resn_exch{" H4' 6MC"}=" H4* 6MC"; $resn_exch{" H3' 6MC"}=" H3* 6MC"; $resn_exch{"HO3' 6MC"}="H3T  6MC";
$resn_exch{" H2' 6MC"}="1H2* 6MC"; $resn_exch{"H2'' 6MC"}="2H2* 6MC"; $resn_exch{" H1' 6MC"}=" H1* 6MC";
$resn_exch{" H61 6MC"}="1H6  6MC"; $resn_exch{" H11 6MC"}="1H1  6MC"; $resn_exch{" H12 6MC"}="2H1  6MC";
$resn_exch{" H13 6MC"}="3H1  6MC"; $resn_exch{"HOP2 6MI"}="2HOP 6MI"; $resn_exch{"HOP3 6MI"}="3HOP 6MI";
$resn_exch{"H7M1 6MI"}="1H7M 6MI"; $resn_exch{"H7M2 6MI"}="2H7M 6MI"; $resn_exch{"H3I1 6MI"}="1H3I 6MI";
$resn_exch{"H3I2 6MI"}="2H3I 6MI"; $resn_exch{"H3I3 6MI"}="3H3I 6MI"; $resn_exch{"H2'1 6MI"}="1H2* 6MI";
$resn_exch{"H2'2 6MI"}="2H2* 6MI"; $resn_exch{"H5'1 6MI"}="1H5* 6MI"; $resn_exch{"H5'2 6MI"}="2H5* 6MI";
$resn_exch{" H4' 6MI"}=" H4* 6MI"; $resn_exch{" H1' 6MI"}=" H1* 6MI"; $resn_exch{" H3' 6MI"}=" H3* 6MI";
$resn_exch{" H71 6MP"}="1H7  6MP"; $resn_exch{" H72 6MP"}="2H7  6MP"; $resn_exch{" H73 6MP"}="3H7  6MP";
$resn_exch{" HB1 6MR"}="1HB  6MR"; $resn_exch{" HB2 6MR"}="2HB  6MR"; $resn_exch{"HOP3 6MT"}="3HOP 6MT";
$resn_exch{"HOP2 6MT"}="2HOP 6MT"; $resn_exch{" H5' 6MT"}="1H5* 6MT"; $resn_exch{"H5'' 6MT"}="2H5* 6MT";
$resn_exch{" H4' 6MT"}=" H4* 6MT"; $resn_exch{" H3' 6MT"}=" H3* 6MT"; $resn_exch{"HO3' 6MT"}="H3T  6MT";
$resn_exch{" H2' 6MT"}="1H2* 6MT"; $resn_exch{"H2'' 6MT"}="2H2* 6MT"; $resn_exch{" H1' 6MT"}=" H1* 6MT";
$resn_exch{" H61 6MT"}="1H6  6MT"; $resn_exch{" H11 6MT"}="1H1  6MT"; $resn_exch{" H12 6MT"}="2H1  6MT";
$resn_exch{" H13 6MT"}="3H1  6MT"; $resn_exch{" H1' 6MZ"}=" H1* 6MZ"; $resn_exch{"H9C1 6MZ"}="1H9C 6MZ";
$resn_exch{"H9C2 6MZ"}="2H9C 6MZ"; $resn_exch{"H5'1 6MZ"}="1H5* 6MZ"; $resn_exch{"H5'2 6MZ"}="2H5* 6MZ";
$resn_exch{" H4' 6MZ"}=" H4* 6MZ"; $resn_exch{" H3' 6MZ"}=" H3* 6MZ"; $resn_exch{" H2' 6MZ"}=" H2* 6MZ";
$resn_exch{"HAC1 6NA"}="1HAC 6NA"; $resn_exch{"HAC2 6NA"}="2HAC 6NA"; $resn_exch{"HBC1 6NA"}="1HBC 6NA";
$resn_exch{"HBC2 6NA"}="2HBC 6NA"; $resn_exch{"HGC1 6NA"}="1HGC 6NA"; $resn_exch{"HGC2 6NA"}="2HGC 6NA";
$resn_exch{"HDC1 6NA"}="1HDC 6NA"; $resn_exch{"HDC2 6NA"}="2HDC 6NA"; $resn_exch{"H6C1 6NA"}="1H6C 6NA";
$resn_exch{"H6C2 6NA"}="2H6C 6NA"; $resn_exch{"H6C3 6NA"}="3H6C 6NA"; $resn_exch{"HN11 6NH"}="1HN1 6NH";
$resn_exch{"HN12 6NH"}="2HN1 6NH"; $resn_exch{" H2B 6NH"}="H2'' 6NH"; $resn_exch{" H3B 6NH"}="H3'' 6NH";
$resn_exch{" H4B 6NH"}="H4'' 6NH"; $resn_exch{" H5B 6NH"}="H5'' 6NH"; $resn_exch{" H6B 6NH"}="H6'' 6NH";
$resn_exch{" H71 6NI"}="1H7  6NI"; $resn_exch{" H41 6NI"}="1H4  6NI"; $resn_exch{" H51 6NI"}="1H5  6NI";
$resn_exch{"HN11 6NI"}="1HN1 6NI"; $resn_exch{" H31 6NI"}="1H3  6NI"; $resn_exch{"H101 6NP"}="1H10 6NP";
$resn_exch{"H102 6NP"}="2H10 6NP"; $resn_exch{"H281 6NP"}="1H28 6NP"; $resn_exch{"H282 6NP"}="2H28 6NP";
$resn_exch{"H291 6NP"}="1H29 6NP"; $resn_exch{"H292 6NP"}="2H29 6NP"; $resn_exch{"H301 6NP"}="1H30 6NP";
$resn_exch{"H302 6NP"}="2H30 6NP"; $resn_exch{"HOP2 6OG"}="2HOP 6OG"; $resn_exch{"HN21 6OG"}="1HN2 6OG";
$resn_exch{"HN22 6OG"}="2HN2 6OG"; $resn_exch{" H2' 6OG"}="1H2* 6OG"; $resn_exch{"H2'' 6OG"}="2H2* 6OG";
$resn_exch{" H5' 6OG"}="1H5* 6OG"; $resn_exch{"H5'' 6OG"}="2H5* 6OG"; $resn_exch{" H4' 6OG"}=" H4* 6OG";
$resn_exch{" H1' 6OG"}=" H1* 6OG"; $resn_exch{" H3' 6OG"}=" H3* 6OG"; $resn_exch{"HO3' 6OG"}="H3T  6OG";
$resn_exch{" H1  6OG"}="1H   6OG"; $resn_exch{" H2  6OG"}="2H   6OG"; $resn_exch{" H3  6OG"}="3H   6OG";
$resn_exch{"HOP3 6OG"}="3HOP 6OG"; $resn_exch{" H61 6PG"}="1H6  6PG"; $resn_exch{" H62 6PG"}="2H6  6PG";
$resn_exch{"HOP2 6PG"}="2HOP 6PG"; $resn_exch{"HOP3 6PG"}="3HOP 6PG"; $resn_exch{"H261 6PL"}="1H26 6PL";
$resn_exch{"H262 6PL"}="2H26 6PL"; $resn_exch{"H263 6PL"}="3H26 6PL"; $resn_exch{"H251 6PL"}="1H25 6PL";
$resn_exch{"H252 6PL"}="2H25 6PL"; $resn_exch{"H241 6PL"}="1H24 6PL"; $resn_exch{"H242 6PL"}="2H24 6PL";
$resn_exch{"H231 6PL"}="1H23 6PL"; $resn_exch{"H232 6PL"}="2H23 6PL"; $resn_exch{"H221 6PL"}="1H22 6PL";
$resn_exch{"H222 6PL"}="2H22 6PL"; $resn_exch{"H211 6PL"}="1H21 6PL"; $resn_exch{"H212 6PL"}="2H21 6PL";
$resn_exch{"H201 6PL"}="1H20 6PL"; $resn_exch{"H202 6PL"}="2H20 6PL"; $resn_exch{"H191 6PL"}="1H19 6PL";
$resn_exch{"H192 6PL"}="2H19 6PL"; $resn_exch{"H181 6PL"}="1H18 6PL"; $resn_exch{"H182 6PL"}="2H18 6PL";
$resn_exch{"H171 6PL"}="1H17 6PL"; $resn_exch{"H172 6PL"}="2H17 6PL"; $resn_exch{"H161 6PL"}="1H16 6PL";
$resn_exch{"H162 6PL"}="2H16 6PL"; $resn_exch{"H151 6PL"}="1H15 6PL"; $resn_exch{"H152 6PL"}="2H15 6PL";
$resn_exch{"H141 6PL"}="1H14 6PL"; $resn_exch{"H142 6PL"}="2H14 6PL"; $resn_exch{"H131 6PL"}="1H13 6PL";
$resn_exch{"H132 6PL"}="2H13 6PL"; $resn_exch{"H121 6PL"}="1H12 6PL"; $resn_exch{"H122 6PL"}="2H12 6PL";
$resn_exch{" H31 6PL"}="1H3  6PL"; $resn_exch{" H32 6PL"}="2H3  6PL"; $resn_exch{" H11 6PL"}="1H1  6PL";
$resn_exch{" H12 6PL"}="2H1  6PL"; $resn_exch{" H41 6PL"}="1H4  6PL"; $resn_exch{" H42 6PL"}="2H4  6PL";
$resn_exch{" H51 6PL"}="1H5  6PL"; $resn_exch{" H52 6PL"}="2H5  6PL"; $resn_exch{" H71 6PL"}="1H7  6PL";
$resn_exch{" H72 6PL"}="2H7  6PL"; $resn_exch{" H73 6PL"}="3H7  6PL"; $resn_exch{" H81 6PL"}="1H8  6PL";
$resn_exch{" H82 6PL"}="2H8  6PL"; $resn_exch{" H83 6PL"}="3H8  6PL"; $resn_exch{" H61 6PL"}="1H6  6PL";
$resn_exch{" H62 6PL"}="2H6  6PL"; $resn_exch{" H63 6PL"}="3H6  6PL"; $resn_exch{"H321 6PL"}="1H32 6PL";
$resn_exch{"H322 6PL"}="2H32 6PL"; $resn_exch{"H331 6PL"}="1H33 6PL"; $resn_exch{"H332 6PL"}="2H33 6PL";
$resn_exch{"H341 6PL"}="1H34 6PL"; $resn_exch{"H342 6PL"}="2H34 6PL"; $resn_exch{"H351 6PL"}="1H35 6PL";
$resn_exch{"H352 6PL"}="2H35 6PL"; $resn_exch{"H361 6PL"}="1H36 6PL"; $resn_exch{"H362 6PL"}="2H36 6PL";
$resn_exch{"H371 6PL"}="1H37 6PL"; $resn_exch{"H372 6PL"}="2H37 6PL"; $resn_exch{"H381 6PL"}="1H38 6PL";
$resn_exch{"H382 6PL"}="2H38 6PL"; $resn_exch{"H391 6PL"}="1H39 6PL"; $resn_exch{"H392 6PL"}="2H39 6PL";
$resn_exch{"H401 6PL"}="1H40 6PL"; $resn_exch{"H402 6PL"}="2H40 6PL"; $resn_exch{"H411 6PL"}="1H41 6PL";
$resn_exch{"H412 6PL"}="2H41 6PL"; $resn_exch{"H421 6PL"}="1H42 6PL"; $resn_exch{"H422 6PL"}="2H42 6PL";
$resn_exch{"H431 6PL"}="1H43 6PL"; $resn_exch{"H432 6PL"}="2H43 6PL"; $resn_exch{"H441 6PL"}="1H44 6PL";
$resn_exch{"H442 6PL"}="2H44 6PL"; $resn_exch{"H451 6PL"}="1H45 6PL"; $resn_exch{"H452 6PL"}="2H45 6PL";
$resn_exch{"H461 6PL"}="1H46 6PL"; $resn_exch{"H462 6PL"}="2H46 6PL"; $resn_exch{"H471 6PL"}="1H47 6PL";
$resn_exch{"H472 6PL"}="2H47 6PL"; $resn_exch{"H481 6PL"}="1H48 6PL"; $resn_exch{"H482 6PL"}="2H48 6PL";
$resn_exch{"H483 6PL"}="3H48 6PL"; $resn_exch{"HAJ1 6PR"}="1HAJ 6PR"; $resn_exch{"HAJ2 6PR"}="2HAJ 6PR";
$resn_exch{" HB1 6PR"}="1HB  6PR"; $resn_exch{" HB2 6PR"}="2HB  6PR"; $resn_exch{"HAA1 6PR"}="1HAA 6PR";
$resn_exch{"HAA2 6PR"}="2HAA 6PR"; $resn_exch{"HR61 6SA"}="1HR6 6SA"; $resn_exch{"HR62 6SA"}="2HR6 6SA";
$resn_exch{"HY61 6SA"}="1HY6 6SA"; $resn_exch{"HY62 6SA"}="2HY6 6SA"; $resn_exch{"HY63 6SA"}="3HY6 6SA";
$resn_exch{"HQ71 6SA"}="1HQ7 6SA"; $resn_exch{"HQ72 6SA"}="2HQ7 6SA"; $resn_exch{"HW61 6SA"}="1HW6 6SA";
$resn_exch{"HW62 6SA"}="2HW6 6SA"; $resn_exch{"HP61 6SA"}="1HP6 6SA"; $resn_exch{"HP62 6SA"}="2HP6 6SA";
$resn_exch{"HV61 6SA"}="1HV6 6SA"; $resn_exch{"HV62 6SA"}="2HV6 6SA"; $resn_exch{"H121 6SC"}="1H12 6SC";
$resn_exch{"H122 6SC"}="2H12 6SC"; $resn_exch{"HBP1 6UL"}="1HBP 6UL"; $resn_exch{"HBP2 6UL"}="2HBP 6UL";
$resn_exch{"HBP3 6UL"}="3HBP 6UL"; $resn_exch{"HBN1 6UL"}="1HBN 6UL"; $resn_exch{"HBN2 6UL"}="2HBN 6UL";
$resn_exch{"HAX1 6UL"}="1HAX 6UL"; $resn_exch{"HAX2 6UL"}="2HAX 6UL"; $resn_exch{"HAY1 6UL"}="1HAY 6UL";
$resn_exch{"HAY2 6UL"}="2HAY 6UL"; $resn_exch{"HAZ1 6UL"}="1HAZ 6UL"; $resn_exch{"HAZ2 6UL"}="2HAZ 6UL";
$resn_exch{"HBA1 6UL"}="1HBA 6UL"; $resn_exch{"HBA2 6UL"}="2HBA 6UL"; $resn_exch{"HBB1 6UL"}="1HBB 6UL";
$resn_exch{"HBB2 6UL"}="2HBB 6UL"; $resn_exch{"HBC1 6UL"}="1HBC 6UL"; $resn_exch{"HBC2 6UL"}="2HBC 6UL";
$resn_exch{"HBD1 6UL"}="1HBD 6UL"; $resn_exch{"HBD2 6UL"}="2HBD 6UL"; $resn_exch{"HBE1 6UL"}="1HBE 6UL";
$resn_exch{"HBE2 6UL"}="2HBE 6UL"; $resn_exch{"HBF1 6UL"}="1HBF 6UL"; $resn_exch{"HBF2 6UL"}="2HBF 6UL";
$resn_exch{"HBG1 6UL"}="1HBG 6UL"; $resn_exch{"HBG2 6UL"}="2HBG 6UL"; $resn_exch{"HBH1 6UL"}="1HBH 6UL";
$resn_exch{"HBH2 6UL"}="2HBH 6UL"; $resn_exch{"HBI1 6UL"}="1HBI 6UL"; $resn_exch{"HBI2 6UL"}="2HBI 6UL";
$resn_exch{"HBJ1 6UL"}="1HBJ 6UL"; $resn_exch{"HBJ2 6UL"}="2HBJ 6UL"; $resn_exch{"HBK1 6UL"}="1HBK 6UL";
$resn_exch{"HBK2 6UL"}="2HBK 6UL"; $resn_exch{"HBL1 6UL"}="1HBL 6UL"; $resn_exch{"HBL2 6UL"}="2HBL 6UL";
$resn_exch{"HBM1 6UL"}="1HBM 6UL"; $resn_exch{"HBM2 6UL"}="2HBM 6UL"; $resn_exch{"HAW1 6UL"}="1HAW 6UL";
$resn_exch{"HAW2 6UL"}="2HAW 6UL"; $resn_exch{"HAV1 6UL"}="1HAV 6UL"; $resn_exch{"HAV2 6UL"}="2HAV 6UL";
$resn_exch{"HAU1 6UL"}="1HAU 6UL"; $resn_exch{"HAU2 6UL"}="2HAU 6UL"; $resn_exch{"HAT1 6UL"}="1HAT 6UL";
$resn_exch{"HAT2 6UL"}="2HAT 6UL"; $resn_exch{"HAS1 6UL"}="1HAS 6UL"; $resn_exch{"HAS2 6UL"}="2HAS 6UL";
$resn_exch{"HAR1 6UL"}="1HAR 6UL"; $resn_exch{"HAR2 6UL"}="2HAR 6UL"; $resn_exch{"HAO1 6UL"}="1HAO 6UL";
$resn_exch{"HAO2 6UL"}="2HAO 6UL"; $resn_exch{"HAN1 6UL"}="1HAN 6UL"; $resn_exch{"HAN2 6UL"}="2HAN 6UL";
$resn_exch{"HAM1 6UL"}="1HAM 6UL"; $resn_exch{"HAM2 6UL"}="2HAM 6UL"; $resn_exch{"HAL1 6UL"}="1HAL 6UL";
$resn_exch{"HAL2 6UL"}="2HAL 6UL"; $resn_exch{"HAK1 6UL"}="1HAK 6UL"; $resn_exch{"HAK2 6UL"}="2HAK 6UL";
$resn_exch{"HAJ1 6UL"}="1HAJ 6UL"; $resn_exch{"HAJ2 6UL"}="2HAJ 6UL"; $resn_exch{"HAI1 6UL"}="1HAI 6UL";
$resn_exch{"HAI2 6UL"}="2HAI 6UL"; $resn_exch{"HAH1 6UL"}="1HAH 6UL"; $resn_exch{"HAH2 6UL"}="2HAH 6UL";
$resn_exch{"HAG1 6UL"}="1HAG 6UL"; $resn_exch{"HAG2 6UL"}="2HAG 6UL"; $resn_exch{"HAF1 6UL"}="1HAF 6UL";
$resn_exch{"HAF2 6UL"}="2HAF 6UL"; $resn_exch{"HAE1 6UL"}="1HAE 6UL"; $resn_exch{"HAE2 6UL"}="2HAE 6UL";
$resn_exch{"HAD1 6UL"}="1HAD 6UL"; $resn_exch{"HAD2 6UL"}="2HAD 6UL"; $resn_exch{"HAC1 6UL"}="1HAC 6UL";
$resn_exch{"HAC2 6UL"}="2HAC 6UL"; $resn_exch{"HAB1 6UL"}="1HAB 6UL"; $resn_exch{"HAB2 6UL"}="2HAB 6UL";
$resn_exch{"HAA1 6UL"}="1HAA 6UL"; $resn_exch{"HAA2 6UL"}="2HAA 6UL"; $resn_exch{"HAA3 6UL"}="3HAA 6UL";
$resn_exch{"H101 700"}="1H10 700"; $resn_exch{"H102 700"}="2H10 700"; $resn_exch{"H121 700"}="1H12 700";
$resn_exch{"H131 700"}="1H13 700"; $resn_exch{"H141 700"}="1H14 700"; $resn_exch{"H151 700"}="1H15 700";
$resn_exch{"H161 700"}="1H16 700"; $resn_exch{"H171 700"}="1H17 700"; $resn_exch{"H191 700"}="1H19 700";
$resn_exch{"H192 700"}="2H19 700"; $resn_exch{"H201 700"}="1H20 700"; $resn_exch{"H221 700"}="1H22 700";
$resn_exch{"H222 700"}="2H22 700"; $resn_exch{"H8C1 701"}="1H8C 701"; $resn_exch{"H8C2 701"}="2H8C 701";
$resn_exch{"H8C3 701"}="3H8C 701"; $resn_exch{"H131 701"}="1H13 701"; $resn_exch{"H132 701"}="2H13 701";
$resn_exch{"H141 701"}="1H14 701"; $resn_exch{"H142 701"}="2H14 701"; $resn_exch{"H171 701"}="1H17 701";
$resn_exch{"H172 701"}="2H17 701"; $resn_exch{"H173 701"}="3H17 701"; $resn_exch{"H191 701"}="1H19 701";
$resn_exch{"H192 701"}="2H19 701"; $resn_exch{"H231 701"}="1H23 701"; $resn_exch{"H232 701"}="2H23 701";
$resn_exch{"H201 701"}="1H20 701"; $resn_exch{"H202 701"}="2H20 701"; $resn_exch{"H221 701"}="1H22 701";
$resn_exch{"H222 701"}="2H22 701"; $resn_exch{"H231 703"}="1H23 703"; $resn_exch{"H232 703"}="2H23 703";
$resn_exch{"H241 703"}="1H24 703"; $resn_exch{"H242 703"}="2H24 703"; $resn_exch{"H251 703"}="1H25 703";
$resn_exch{"H252 703"}="2H25 703"; $resn_exch{"H271 703"}="1H27 703"; $resn_exch{"H272 703"}="2H27 703";
$resn_exch{"H281 703"}="1H28 703"; $resn_exch{"H282 703"}="2H28 703"; $resn_exch{"H301 703"}="1H30 703";
$resn_exch{"H302 703"}="2H30 703"; $resn_exch{"H311 703"}="1H31 703"; $resn_exch{"H312 703"}="2H31 703";
$resn_exch{"H121 709"}="1H12 709"; $resn_exch{"H122 709"}="2H12 709"; $resn_exch{"H231 709"}="1H23 709";
$resn_exch{"H232 709"}="2H23 709"; $resn_exch{"H233 709"}="3H23 709"; $resn_exch{" H71 70U"}="1H5M 70U";
$resn_exch{" H72 70U"}="2H5M 70U"; $resn_exch{" H91 70U"}="1H9  70U"; $resn_exch{" H92 70U"}="2H9  70U";
$resn_exch{" H93 70U"}="3H9  70U"; $resn_exch{" H1' 70U"}=" H1* 70U"; $resn_exch{"HO2' 70U"}="2HO* 70U";
$resn_exch{" H2' 70U"}=" H2* 70U"; $resn_exch{"HO3' 70U"}="H3T  70U"; $resn_exch{" H3' 70U"}=" H3* 70U";
$resn_exch{" H4' 70U"}=" H4* 70U"; $resn_exch{" H5' 70U"}="1H5* 70U"; $resn_exch{"H5'' 70U"}="2H5* 70U";
$resn_exch{"HOP2 70U"}="2HOP 70U"; $resn_exch{"HOP3 70U"}="3HOP 70U"; $resn_exch{"H171 710"}="1H17 710";
$resn_exch{"H172 710"}="2H17 710"; $resn_exch{"H191 710"}="1H19 710"; $resn_exch{"H192 710"}="2H19 710";
$resn_exch{"H201 710"}="1H20 710"; $resn_exch{"H202 710"}="2H20 710"; $resn_exch{"H211 710"}="1H21 710";
$resn_exch{"H212 710"}="2H21 710"; $resn_exch{"H221 710"}="1H22 710"; $resn_exch{"H222 710"}="2H22 710";
$resn_exch{"H231 710"}="1H23 710"; $resn_exch{"H232 710"}="2H23 710"; $resn_exch{"H291 710"}="1H29 710";
$resn_exch{"H292 710"}="2H29 710"; $resn_exch{"HN21 711"}="1HN2 711"; $resn_exch{"HN2A 711"}="2HN2 711";
$resn_exch{"H121 711"}="1H12 711"; $resn_exch{"H122 711"}="2H12 711"; $resn_exch{"H221 711"}="1H22 711";
$resn_exch{"H222 711"}="2H22 711"; $resn_exch{"H223 711"}="3H22 711"; $resn_exch{"H321 711"}="1H32 711";
$resn_exch{"H322 711"}="2H32 711"; $resn_exch{"H331 711"}="1H33 711"; $resn_exch{"H332 711"}="2H33 711";
$resn_exch{"H341 711"}="1H34 711"; $resn_exch{"H342 711"}="2H34 711"; $resn_exch{"H351 711"}="1H35 711";
$resn_exch{"H352 711"}="2H35 711"; $resn_exch{"H371 711"}="1H37 711"; $resn_exch{"H372 711"}="2H37 711";
$resn_exch{"H373 711"}="3H37 711"; $resn_exch{" HN1 712"}="1HN  712"; $resn_exch{" HN2 712"}="2HN  712";
$resn_exch{"H31A 712"}="1H3  712"; $resn_exch{" H32 712"}="2H3  712"; $resn_exch{" H33 712"}="3H3  712";
$resn_exch{" H41 712"}="1H4  712"; $resn_exch{" H42 712"}="2H4  712"; $resn_exch{"H211 712"}="1H21 712";
$resn_exch{"H212 712"}="2H21 712"; $resn_exch{"H213 712"}="3H21 712"; $resn_exch{"H241 712"}="1H24 712";
$resn_exch{"H242 712"}="2H24 712"; $resn_exch{"H243 712"}="3H24 712"; $resn_exch{"H321 712"}="1H32 712";
$resn_exch{"H322 712"}="2H32 712"; $resn_exch{"H323 712"}="3H32 712"; $resn_exch{"H331 712"}="1H33 712";
$resn_exch{"H332 712"}="2H33 712"; $resn_exch{"H111 715"}="1H11 715"; $resn_exch{"H112 715"}="2H11 715";
$resn_exch{"H151 715"}="1H15 715"; $resn_exch{"H152 715"}="2H15 715"; $resn_exch{"H201 715"}="1H20 715";
$resn_exch{"H202 715"}="2H20 715"; $resn_exch{"H251 715"}="1H25 715"; $resn_exch{"H252 715"}="2H25 715";
$resn_exch{"H261 715"}="1H26 715"; $resn_exch{"H262 715"}="2H26 715"; $resn_exch{"H291 715"}="1H29 715";
$resn_exch{"H292 715"}="2H29 715"; $resn_exch{" H31 716"}="1H3  716"; $resn_exch{" H32 716"}="2H3  716";
$resn_exch{" H41 716"}="1H4  716"; $resn_exch{" H42 716"}="2H4  716"; $resn_exch{" H51 716"}="1H5  716";
$resn_exch{" H52 716"}="2H5  716"; $resn_exch{"HN81 716"}="1HN8 716"; $resn_exch{"HN82 716"}="2HN8 716";
$resn_exch{"H191 716"}="1H19 716"; $resn_exch{"H192 716"}="2H19 716"; $resn_exch{"H193 716"}="3H19 716";
$resn_exch{"H201 716"}="1H20 716"; $resn_exch{"H202 716"}="2H20 716"; $resn_exch{"H203 716"}="3H20 716";
$resn_exch{" HN1 720"}="1HN  720"; $resn_exch{" HN2 720"}="2HN  720"; $resn_exch{"HAC1 720"}="1HAC 720";
$resn_exch{"HAC2 720"}="2HAC 720"; $resn_exch{"HHC1 720"}="1HHC 720"; $resn_exch{"HHC2 720"}="2HHC 720";
$resn_exch{"HE21 720"}="1HE2 720"; $resn_exch{"HE22 720"}="2HE2 720"; $resn_exch{"HZC1 720"}="1HZC 720";
$resn_exch{"HZC2 720"}="2HZC 720"; $resn_exch{"HE11 720"}="1HE1 720"; $resn_exch{"HE12 720"}="2HE1 720";
$resn_exch{"HE13 720"}="3HE1 720"; $resn_exch{"HMC1 720"}="1HMC 720"; $resn_exch{"HMC2 720"}="2HMC 720";
$resn_exch{"HB21 720"}="1HB2 720"; $resn_exch{"HB22 720"}="2HB2 720"; $resn_exch{"H311 730"}="1H31 730";
$resn_exch{"H312 730"}="2H31 730"; $resn_exch{"H301 730"}="1H30 730"; $resn_exch{"H302 730"}="2H30 730";
$resn_exch{"H321 730"}="1H32 730"; $resn_exch{"H322 730"}="2H32 730"; $resn_exch{"H331 730"}="1H33 730";
$resn_exch{"H332 730"}="2H33 730"; $resn_exch{"H361 730"}="1H36 730"; $resn_exch{"H362 730"}="2H36 730";
$resn_exch{"H281 730"}="1H28 730"; $resn_exch{"H282 730"}="2H28 730"; $resn_exch{"H271 730"}="1H27 730";
$resn_exch{"H272 730"}="2H27 730"; $resn_exch{"H141 730"}="1H14 730"; $resn_exch{"H142 730"}="2H14 730";
$resn_exch{"H221 730"}="1H22 730"; $resn_exch{"H222 730"}="2H22 730"; $resn_exch{"H251 736"}="1H25 736";
$resn_exch{"H252 736"}="2H25 736"; $resn_exch{"H261 736"}="1H26 736"; $resn_exch{"H262 736"}="2H26 736";
$resn_exch{"H281 736"}="1H28 736"; $resn_exch{"H282 736"}="2H28 736"; $resn_exch{"H291 736"}="1H29 736";
$resn_exch{"H292 736"}="2H29 736"; $resn_exch{"H21A 736"}="1H2  736"; $resn_exch{" H22 736"}="2H2  736";
$resn_exch{" H71 736"}="1H7  736"; $resn_exch{" H72 736"}="2H7  736"; $resn_exch{" H81 736"}="1H8  736";
$resn_exch{" H82 736"}="2H8  736"; $resn_exch{"H141 736"}="1H14 736"; $resn_exch{"H142 736"}="2H14 736";
$resn_exch{"H151 736"}="1H15 736"; $resn_exch{"H152 736"}="2H15 736"; $resn_exch{"H153 736"}="3H15 736";
$resn_exch{"H061 737"}="1H06 737"; $resn_exch{"H062 737"}="2H06 737"; $resn_exch{"H071 737"}="1H07 737";
$resn_exch{"H072 737"}="2H07 737"; $resn_exch{"H081 737"}="1H08 737"; $resn_exch{"H082 737"}="2H08 737";
$resn_exch{"H111 737"}="1H11 737"; $resn_exch{"H112 737"}="2H11 737"; $resn_exch{"H151 737"}="1H15 737";
$resn_exch{"H152 737"}="2H15 737"; $resn_exch{"H291 737"}="1H29 737"; $resn_exch{"H292 737"}="2H29 737";
$resn_exch{"H293 737"}="3H29 737"; $resn_exch{"H351 737"}="1H35 737"; $resn_exch{"H352 737"}="2H35 737";
$resn_exch{" H81 738"}="1H8  738"; $resn_exch{" H82 738"}="2H8  738"; $resn_exch{"H121 738"}="1H12 738";
$resn_exch{"H122 738"}="2H12 738"; $resn_exch{" H91 738"}="1H9  738"; $resn_exch{" H92 738"}="2H9  738";
$resn_exch{" H41 738"}="1H4  738"; $resn_exch{" H42 738"}="2H4  738"; $resn_exch{"HBC1 739"}="1HBC 739";
$resn_exch{"HBC2 739"}="2HBC 739"; $resn_exch{"HCC1 739"}="1HCC 739"; $resn_exch{"HCC2 739"}="2HCC 739";
$resn_exch{"HNC1 739"}="1HNC 739"; $resn_exch{"HNC2 739"}="2HNC 739"; $resn_exch{"HCI1 739"}="1HCI 739";
$resn_exch{"HCI2 739"}="2HCI 739"; $resn_exch{"HCI3 739"}="3HCI 739"; $resn_exch{"HDI1 739"}="1HDI 739";
$resn_exch{"HDI2 739"}="2HDI 739"; $resn_exch{"HEI1 739"}="1HEI 739"; $resn_exch{"HEI2 739"}="2HEI 739";
$resn_exch{"HEI3 739"}="3HEI 739"; $resn_exch{" HI1 739"}="1HI  739"; $resn_exch{" HI2 739"}="2HI  739";
$resn_exch{"HBF1 739"}="1HBF 739"; $resn_exch{"HBF2 739"}="2HBF 739"; $resn_exch{"HBM1 739"}="1HBM 739";
$resn_exch{"HBM2 739"}="2HBM 739"; $resn_exch{"HCM1 739"}="1HCM 739"; $resn_exch{"HCM2 739"}="2HCM 739";
$resn_exch{"HEM1 739"}="1HEM 739"; $resn_exch{"HEM2 739"}="2HEM 739"; $resn_exch{"HEM3 739"}="3HEM 739";
$resn_exch{" H11 73Q"}="1H1  73Q"; $resn_exch{" H12 73Q"}="2H1  73Q"; $resn_exch{" H13 73Q"}="3H1  73Q";
$resn_exch{" H31 73Q"}="1H3  73Q"; $resn_exch{" H32 73Q"}="2H3  73Q"; $resn_exch{" H33 73Q"}="3H3  73Q";
$resn_exch{"H191 73V"}="1H19 73V"; $resn_exch{"H192 73V"}="2H19 73V"; $resn_exch{"H193 73V"}="3H19 73V";
$resn_exch{"H181 73V"}="1H18 73V"; $resn_exch{"H182 73V"}="2H18 73V"; $resn_exch{"H171 73V"}="1H17 73V";
$resn_exch{"H172 73V"}="2H17 73V"; $resn_exch{" H91 73V"}="1H9  73V"; $resn_exch{" H92 73V"}="2H9  73V";
$resn_exch{"H221 73V"}="1H22 73V"; $resn_exch{"H222 73V"}="2H22 73V"; $resn_exch{"H211 73V"}="1H21 73V";
$resn_exch{"H212 73V"}="2H21 73V"; $resn_exch{"H213 73V"}="3H21 73V"; $resn_exch{"H231 73V"}="1H23 73V";
$resn_exch{"H232 73V"}="2H23 73V"; $resn_exch{"H233 73V"}="3H23 73V"; $resn_exch{"H261 73V"}="1H26 73V";
$resn_exch{"H262 73V"}="2H26 73V"; $resn_exch{"H251 73V"}="1H25 73V"; $resn_exch{"H252 73V"}="2H25 73V";
$resn_exch{"H241 73V"}="1H24 73V"; $resn_exch{"H242 73V"}="2H24 73V"; $resn_exch{"H101 744"}="1H10 744";
$resn_exch{"H102 744"}="2H10 744"; $resn_exch{" H81 744"}="1H8  744"; $resn_exch{" H82 744"}="2H8  744";
$resn_exch{" H41 744"}="1H4  744"; $resn_exch{" H42 744"}="2H4  744"; $resn_exch{"H231 744"}="1H23 744";
$resn_exch{"H232 744"}="2H23 744"; $resn_exch{"H233 744"}="3H23 744"; $resn_exch{"H191 745"}="1H19 745";
$resn_exch{"H192 745"}="2H19 745"; $resn_exch{" H71 745"}="1H7  745"; $resn_exch{" H72 745"}="2H7  745";
$resn_exch{"H161 74M"}="1H16 74M"; $resn_exch{"H162 74M"}="2H16 74M"; $resn_exch{"H151 74M"}="1H15 74M";
$resn_exch{"H152 74M"}="2H15 74M"; $resn_exch{"H153 74M"}="3H15 74M"; $resn_exch{"H171 74M"}="1H17 74M";
$resn_exch{"H172 74M"}="2H17 74M"; $resn_exch{"H173 74M"}="3H17 74M"; $resn_exch{"H201 74M"}="1H20 74M";
$resn_exch{"H202 74M"}="2H20 74M"; $resn_exch{"H191 74M"}="1H19 74M"; $resn_exch{"H192 74M"}="2H19 74M";
$resn_exch{"H181 74M"}="1H18 74M"; $resn_exch{"H182 74M"}="2H18 74M"; $resn_exch{" H91 74M"}="1H9  74M";
$resn_exch{" H92 74M"}="2H9  74M"; $resn_exch{" H93 74M"}="3H9  74M"; $resn_exch{"H111 74M"}="1H11 74M";
$resn_exch{"H112 74M"}="2H11 74M"; $resn_exch{"H121 74M"}="1H12 74M"; $resn_exch{"H122 74M"}="2H12 74M";
$resn_exch{"H131 74M"}="1H13 74M"; $resn_exch{"H132 74M"}="2H13 74M"; $resn_exch{"H133 74M"}="3H13 74M";
$resn_exch{"HC11 750"}="1HC1 750"; $resn_exch{"HC12 750"}="2HC1 750"; $resn_exch{"HC31 750"}="1HC3 750";
$resn_exch{"HC32 750"}="2HC3 750"; $resn_exch{"HC41 750"}="1HC4 750"; $resn_exch{"HC42 750"}="2HC4 750";
$resn_exch{"HC61 750"}="1HC6 750"; $resn_exch{"HC62 750"}="2HC6 750"; $resn_exch{"HC71 750"}="1HC7 750";
$resn_exch{"HC72 750"}="2HC7 750"; $resn_exch{"HC81 750"}="1HC8 750"; $resn_exch{"HC82 750"}="2HC8 750";
$resn_exch{"H231 750"}="1H23 750"; $resn_exch{"H232 750"}="2H23 750"; $resn_exch{"H251 750"}="1H25 750";
$resn_exch{"H252 750"}="2H25 750"; $resn_exch{"H253 750"}="3H25 750"; $resn_exch{"H261 750"}="1H26 750";
$resn_exch{"H262 750"}="2H26 750"; $resn_exch{"H263 750"}="3H26 750"; $resn_exch{"H331 750"}="1H33 750";
$resn_exch{"H332 750"}="2H33 750"; $resn_exch{"H351 750"}="1H35 750"; $resn_exch{"H352 750"}="2H35 750";
$resn_exch{"H361 750"}="1H36 750"; $resn_exch{"H362 750"}="2H36 750"; $resn_exch{"H371 750"}="1H37 750";
$resn_exch{"H372 750"}="2H37 750"; $resn_exch{"H401 750"}="1H40 750"; $resn_exch{"H402 750"}="2H40 750";
$resn_exch{"H101 75V"}="1H10 75V"; $resn_exch{"H102 75V"}="2H10 75V"; $resn_exch{"H111 75V"}="1H11 75V";
$resn_exch{"H112 75V"}="2H11 75V"; $resn_exch{"H113 75V"}="3H11 75V"; $resn_exch{"H141 75V"}="1H14 75V";
$resn_exch{"H142 75V"}="2H14 75V"; $resn_exch{"H131 75V"}="1H13 75V"; $resn_exch{"H132 75V"}="2H13 75V";
$resn_exch{"H133 75V"}="3H13 75V"; $resn_exch{"H151 75V"}="1H15 75V"; $resn_exch{"H152 75V"}="2H15 75V";
$resn_exch{"H153 75V"}="3H15 75V"; $resn_exch{"H161 75V"}="1H16 75V"; $resn_exch{"H162 75V"}="2H16 75V";
$resn_exch{"H163 75V"}="3H16 75V"; $resn_exch{" H12 760"}="2H1  760"; $resn_exch{" H11 760"}="1H1  760";
$resn_exch{"H22A 760"}="2H2  760"; $resn_exch{"H21A 760"}="1H2  760"; $resn_exch{" H32 760"}="2H3  760";
$resn_exch{" H31 760"}="1H3  760"; $resn_exch{" H42 760"}="2H4  760"; $resn_exch{" H41 760"}="1H4  760";
$resn_exch{"H171 760"}="1H17 760"; $resn_exch{"H172 760"}="2H17 760"; $resn_exch{" H11 761"}="1H1  761";
$resn_exch{" H21 761"}="1H2  761"; $resn_exch{" H51 761"}="1H5  761"; $resn_exch{" H61 761"}="1H6  761";
$resn_exch{"H111 761"}="1H11 761"; $resn_exch{"H141 761"}="1H14 761"; $resn_exch{" H2B 762"}="H2'' 762";
$resn_exch{" H3B 762"}="H3'' 762"; $resn_exch{" H4B 762"}="H4'' 762"; $resn_exch{" H5B 762"}="H5'' 762";
$resn_exch{" H6B 762"}="H6'' 762"; $resn_exch{"H331 76A"}="1H33 76A"; $resn_exch{"H332 76A"}="2H33 76A";
$resn_exch{"H341 76A"}="1H34 76A"; $resn_exch{"H342 76A"}="2H34 76A"; $resn_exch{"H301 76A"}="1H30 76A";
$resn_exch{"H302 76A"}="2H30 76A"; $resn_exch{"H311 76A"}="1H31 76A"; $resn_exch{"H312 76A"}="2H31 76A";
$resn_exch{"H211 76A"}="1H21 76A"; $resn_exch{"H212 76A"}="2H21 76A"; $resn_exch{"H101 76V"}="1H10 76V";
$resn_exch{"H102 76V"}="2H10 76V"; $resn_exch{"H111 76V"}="1H11 76V"; $resn_exch{"H112 76V"}="2H11 76V";
$resn_exch{"H113 76V"}="3H11 76V"; $resn_exch{"H141 76V"}="1H14 76V"; $resn_exch{"H142 76V"}="2H14 76V";
$resn_exch{"H131 76V"}="1H13 76V"; $resn_exch{"H132 76V"}="2H13 76V"; $resn_exch{"H133 76V"}="3H13 76V";
$resn_exch{"H151 76V"}="1H15 76V"; $resn_exch{"H152 76V"}="2H15 76V"; $resn_exch{"H153 76V"}="3H15 76V";
$resn_exch{"H181 76V"}="1H18 76V"; $resn_exch{"H182 76V"}="2H18 76V"; $resn_exch{"H171 76V"}="1H17 76V";
$resn_exch{"H172 76V"}="2H17 76V"; $resn_exch{"H173 76V"}="3H17 76V"; $resn_exch{"H191 76V"}="1H19 76V";
$resn_exch{"H192 76V"}="2H19 76V"; $resn_exch{"H193 76V"}="3H19 76V"; $resn_exch{"H2N1 771"}="1H2N 771";
$resn_exch{"H2N2 771"}="2H2N 771"; $resn_exch{"H8C1 771"}="1H8C 771"; $resn_exch{"H8C2 771"}="2H8C 771";
$resn_exch{"H151 771"}="1H15 771"; $resn_exch{"H152 771"}="2H15 771"; $resn_exch{"H221 771"}="1H22 771";
$resn_exch{"H222 771"}="2H22 771"; $resn_exch{"H223 771"}="3H22 771"; $resn_exch{"H231 771"}="1H23 771";
$resn_exch{"H232 771"}="2H23 771"; $resn_exch{"H233 771"}="3H23 771"; $resn_exch{"H131 773"}="1H13 773";
$resn_exch{"H132 773"}="2H13 773"; $resn_exch{"H133 773"}="3H13 773"; $resn_exch{"H111 773"}="1H11 773";
$resn_exch{"H112 773"}="2H11 773"; $resn_exch{"H141 773"}="1H14 773"; $resn_exch{"H142 773"}="2H14 773";
$resn_exch{"H143 773"}="3H14 773"; $resn_exch{"H151 773"}="1H15 773"; $resn_exch{"H152 773"}="2H15 773";
$resn_exch{"H153 773"}="3H15 773"; $resn_exch{"H421 773"}="1H42 773"; $resn_exch{"H422 773"}="2H42 773";
$resn_exch{"H431 773"}="1H43 773"; $resn_exch{"H432 773"}="2H43 773"; $resn_exch{"H433 773"}="3H43 773";
$resn_exch{"H201 773"}="1H20 773"; $resn_exch{"H202 773"}="2H20 773"; $resn_exch{"H203 773"}="3H20 773";
$resn_exch{"H181 773"}="1H18 773"; $resn_exch{"H182 773"}="2H18 773"; $resn_exch{"H281 773"}="1H28 773";
$resn_exch{"H282 773"}="2H28 773"; $resn_exch{"H171 773"}="1H17 773"; $resn_exch{"H172 773"}="2H17 773";
$resn_exch{"H173 773"}="3H17 773"; $resn_exch{"H411 773"}="1H41 773"; $resn_exch{"H412 773"}="2H41 773";
$resn_exch{"H413 773"}="3H41 773"; $resn_exch{"H271 773"}="1H27 773"; $resn_exch{"H272 773"}="2H27 773";
$resn_exch{"H273 773"}="3H27 773"; $resn_exch{" H31 773"}="1H3  773"; $resn_exch{"H32A 773"}="2H3  773";
$resn_exch{" H33 773"}="3H3  773"; $resn_exch{" H61 773"}="1H6  773"; $resn_exch{" H62 773"}="2H6  773";
$resn_exch{" H63 773"}="3H6  773"; $resn_exch{"H21A 778"}="1H2  778"; $resn_exch{" H22 778"}="2H2  778";
$resn_exch{" H31 778"}="1H3  778"; $resn_exch{"H32A 778"}="2H3  778"; $resn_exch{" H41 778"}="1H4  778";
$resn_exch{" H42 778"}="2H4  778"; $resn_exch{" H51 778"}="1H5  778"; $resn_exch{" H52 778"}="2H5  778";
$resn_exch{"H161 778"}="1H16 778"; $resn_exch{"H162 778"}="2H16 778"; $resn_exch{" H91 77A"}="1H9  77A";
$resn_exch{" H92 77A"}="2H9  77A"; $resn_exch{" H93 77A"}="3H9  77A"; $resn_exch{"H221 77A"}="1H22 77A";
$resn_exch{"H222 77A"}="2H22 77A"; $resn_exch{"H241 77A"}="1H24 77A"; $resn_exch{"H242 77A"}="2H24 77A";
$resn_exch{"H243 77A"}="3H24 77A"; $resn_exch{"H251 77A"}="1H25 77A"; $resn_exch{"H252 77A"}="2H25 77A";
$resn_exch{"H253 77A"}="3H25 77A"; $resn_exch{"H261 77A"}="1H26 77A"; $resn_exch{"H262 77A"}="2H26 77A";
$resn_exch{"H263 77A"}="3H26 77A"; $resn_exch{"H171 77B"}="1H17 77B"; $resn_exch{"H172 77B"}="2H17 77B";
$resn_exch{"H261 77B"}="1H26 77B"; $resn_exch{"H262 77B"}="2H26 77B"; $resn_exch{"H251 77B"}="1H25 77B";
$resn_exch{"H252 77B"}="2H25 77B"; $resn_exch{"H253 77B"}="3H25 77B"; $resn_exch{"H271 77B"}="1H27 77B";
$resn_exch{"H272 77B"}="2H27 77B"; $resn_exch{"H273 77B"}="3H27 77B"; $resn_exch{" H91 77B"}="1H9  77B";
$resn_exch{" H92 77B"}="2H9  77B"; $resn_exch{"H301 77B"}="1H30 77B"; $resn_exch{"H302 77B"}="2H30 77B";
$resn_exch{"H291 77B"}="1H29 77B"; $resn_exch{"H292 77B"}="2H29 77B"; $resn_exch{"H281 77B"}="1H28 77B";
$resn_exch{"H282 77B"}="2H28 77B"; $resn_exch{" H2B 780"}="H2'' 780"; $resn_exch{" H3B 780"}="H3'' 780";
$resn_exch{" H4B 780"}="H4'' 780"; $resn_exch{" H5B 780"}="H5'' 780"; $resn_exch{" H6B 780"}="H6'' 780";
$resn_exch{" H2B 783"}="H2'' 783"; $resn_exch{" H3B 783"}="H3'' 783"; $resn_exch{" H4B 783"}="H4'' 783";
$resn_exch{" H5B 783"}="H5'' 783"; $resn_exch{" H6B 783"}="H6'' 783"; $resn_exch{"H191 784"}="1H19 784";
$resn_exch{"H192 784"}="2H19 784"; $resn_exch{"H211 784"}="1H21 784"; $resn_exch{"H212 784"}="2H21 784";
$resn_exch{"H231 784"}="1H23 784"; $resn_exch{"H232 784"}="2H23 784"; $resn_exch{"H321 784"}="1H32 784";
$resn_exch{"H322 784"}="2H32 784"; $resn_exch{"H323 784"}="3H32 784"; $resn_exch{"H311 784"}="1H31 784";
$resn_exch{"H312 784"}="2H31 784"; $resn_exch{"H313 784"}="3H31 784"; $resn_exch{"H301 784"}="1H30 784";
$resn_exch{"H302 784"}="2H30 784"; $resn_exch{"H303 784"}="3H30 784"; $resn_exch{"H141 784"}="1H14 784";
$resn_exch{"H142 784"}="2H14 784"; $resn_exch{"H151 784"}="1H15 784"; $resn_exch{"H152 784"}="2H15 784";
$resn_exch{"H8C1 784"}="1H8C 784"; $resn_exch{"H8C2 784"}="2H8C 784"; $resn_exch{"H241 784"}="1H24 784";
$resn_exch{"H242 784"}="2H24 784"; $resn_exch{"H101 78A"}="1H10 78A"; $resn_exch{"H102 78A"}="2H10 78A";
$resn_exch{"H191 78A"}="1H19 78A"; $resn_exch{"H192 78A"}="2H19 78A"; $resn_exch{"H181 78A"}="1H18 78A";
$resn_exch{"H182 78A"}="2H18 78A"; $resn_exch{"H183 78A"}="3H18 78A"; $resn_exch{"H201 78A"}="1H20 78A";
$resn_exch{"H202 78A"}="2H20 78A"; $resn_exch{"H203 78A"}="3H20 78A"; $resn_exch{"H231 78A"}="1H23 78A";
$resn_exch{"H232 78A"}="2H23 78A"; $resn_exch{"H221 78A"}="1H22 78A"; $resn_exch{"H222 78A"}="2H22 78A";
$resn_exch{"H211 78A"}="1H21 78A"; $resn_exch{"H212 78A"}="2H21 78A"; $resn_exch{" H11 790"}="1H1  790";
$resn_exch{" H12 790"}="2H1  790"; $resn_exch{"H21A 790"}="1H2  790"; $resn_exch{" H22 790"}="2H2  790";
$resn_exch{" H61 790"}="1H6  790"; $resn_exch{" H62 790"}="2H6  790"; $resn_exch{"H101 796"}="1H10 796";
$resn_exch{"H102 796"}="2H10 796"; $resn_exch{"H103 796"}="3H10 796"; $resn_exch{"H271 796"}="1H27 796";
$resn_exch{"H272 796"}="2H27 796"; $resn_exch{"H431 796"}="1H43 796"; $resn_exch{"H432 796"}="2H43 796";
$resn_exch{"H281 796"}="1H28 796"; $resn_exch{"H282 796"}="2H28 796"; $resn_exch{" H51 7A8"}="1H5  7A8";
$resn_exch{" H52 7A8"}="2H5  7A8"; $resn_exch{" H81 7A8"}="1H8  7A8"; $resn_exch{" H82 7A8"}="2H8  7A8";
$resn_exch{" H91 7A8"}="1H9  7A8"; $resn_exch{" H92 7A8"}="2H9  7A8"; $resn_exch{"H131 7A8"}="1H13 7A8";
$resn_exch{"H132 7A8"}="2H13 7A8"; $resn_exch{"H133 7A8"}="3H13 7A8"; $resn_exch{"H141 7A8"}="1H14 7A8";
$resn_exch{"H142 7A8"}="2H14 7A8"; $resn_exch{"H143 7A8"}="3H14 7A8"; $resn_exch{"H151 7A8"}="1H15 7A8";
$resn_exch{"H152 7A8"}="2H15 7A8"; $resn_exch{"H153 7A8"}="3H15 7A8"; $resn_exch{"HN11 7AD"}="1HN1 7AD";
$resn_exch{"HN12 7AD"}="2HN1 7AD"; $resn_exch{"H151 7AD"}="1H15 7AD"; $resn_exch{"H152 7AD"}="2H15 7AD";
$resn_exch{"H161 7AD"}="1H16 7AD"; $resn_exch{"H162 7AD"}="2H16 7AD"; $resn_exch{"H171 7AD"}="1H17 7AD";
$resn_exch{"H172 7AD"}="2H17 7AD"; $resn_exch{"H181 7AD"}="1H18 7AD"; $resn_exch{"H182 7AD"}="2H18 7AD";
$resn_exch{"H183 7AD"}="3H18 7AD"; $resn_exch{"H191 7AD"}="1H19 7AD"; $resn_exch{"H192 7AD"}="2H19 7AD";
$resn_exch{"H193 7AD"}="3H19 7AD"; $resn_exch{" H77 7AP"}="7H7  7AP"; $resn_exch{"H1'' 7AP"}="'H1' 7AP";
$resn_exch{"H2'' 7AP"}="'H2' 7AP"; $resn_exch{"H3'1 7AP"}="1H3' 7AP"; $resn_exch{"H3'2 7AP"}="2H3' 7AP";
$resn_exch{"H3'3 7AP"}="3H3' 7AP"; $resn_exch{" H11 7CA"}="1H1  7CA"; $resn_exch{" H12 7CA"}="2H1  7CA";
$resn_exch{" H13 7CA"}="3H1  7CA"; $resn_exch{"H171 7CA"}="1H17 7CA"; $resn_exch{"H172 7CA"}="2H17 7CA";
$resn_exch{"H181 7CA"}="1H18 7CA"; $resn_exch{"H182 7CA"}="2H18 7CA"; $resn_exch{"H201 7CA"}="1H20 7CA";
$resn_exch{"H202 7CA"}="2H20 7CA"; $resn_exch{"H203 7CA"}="3H20 7CA"; $resn_exch{"H211 7CA"}="1H21 7CA";
$resn_exch{"H212 7CA"}="2H21 7CA"; $resn_exch{"H213 7CA"}="3H21 7CA"; $resn_exch{"H271 7CA"}="1H27 7CA";
$resn_exch{"H272 7CA"}="2H27 7CA"; $resn_exch{"H273 7CA"}="3H27 7CA"; $resn_exch{" H81 7CS"}="1H8  7CS";
$resn_exch{" H82 7CS"}="2H8  7CS"; $resn_exch{" H83 7CS"}="3H8  7CS"; $resn_exch{" H71 7CS"}="1H7  7CS";
$resn_exch{" H72 7CS"}="2H7  7CS"; $resn_exch{" H61 7CS"}="1H6  7CS"; $resn_exch{" H62 7CS"}="2H6  7CS";
$resn_exch{" H41 7CS"}="1H4  7CS"; $resn_exch{" H42 7CS"}="2H4  7CS"; $resn_exch{" H21 7CS"}="1H2  7CS";
$resn_exch{" H22 7CS"}="2H2  7CS"; $resn_exch{"HOP2 7DA"}="2HOP 7DA"; $resn_exch{"HN61 7DA"}="1HN6 7DA";
$resn_exch{"HN62 7DA"}="2HN6 7DA"; $resn_exch{" H2' 7DA"}="1H2* 7DA"; $resn_exch{"H2'' 7DA"}="2H2* 7DA";
$resn_exch{" H5' 7DA"}="1H5* 7DA"; $resn_exch{"H5'' 7DA"}="2H5* 7DA"; $resn_exch{" H4' 7DA"}=" H4* 7DA";
$resn_exch{" H1' 7DA"}=" H1* 7DA"; $resn_exch{"HO3' 7DA"}="H3T  7DA"; $resn_exch{"HOP3 7DA"}="3HOP 7DA";
$resn_exch{" H61 7DE"}="1H6  7DE"; $resn_exch{" H62 7DE"}="2H6  7DE"; $resn_exch{" H63 7DE"}="3H6  7DE";
$resn_exch{" H91 7DE"}="1H9  7DE"; $resn_exch{" H92 7DE"}="2H9  7DE"; $resn_exch{" H93 7DE"}="3H9  7DE";
$resn_exch{"H161 7DE"}="1H16 7DE"; $resn_exch{"H162 7DE"}="2H16 7DE"; $resn_exch{"H171 7DE"}="1H17 7DE";
$resn_exch{"H172 7DE"}="2H17 7DE"; $resn_exch{"H173 7DE"}="3H17 7DE"; $resn_exch{" H71 7DG"}="1H7  7DG";
$resn_exch{" H72 7DG"}="2H7  7DG"; $resn_exch{"HN21 7DG"}="1HN2 7DG"; $resn_exch{"HN22 7DG"}="2HN2 7DG";
$resn_exch{" H61 7GP"}="1H6  7GP"; $resn_exch{" H62 7GP"}="2H6  7GP"; $resn_exch{" H91 7GP"}="1H9  7GP";
$resn_exch{" H92 7GP"}="2H9  7GP"; $resn_exch{"H101 7GP"}="1H10 7GP"; $resn_exch{"H102 7GP"}="2H10 7GP";
$resn_exch{"H103 7GP"}="3H10 7GP"; $resn_exch{"HOP2 7GU"}="2HOP 7GU"; $resn_exch{"HN21 7GU"}="1HN2 7GU";
$resn_exch{"HN22 7GU"}="2HN2 7GU"; $resn_exch{" H2' 7GU"}="1H2* 7GU"; $resn_exch{"H2'' 7GU"}="2H2* 7GU";
$resn_exch{" H5' 7GU"}="1H5* 7GU"; $resn_exch{"H5'' 7GU"}="2H5* 7GU"; $resn_exch{" H4' 7GU"}=" H4* 7GU";
$resn_exch{" H1' 7GU"}=" H1* 7GU"; $resn_exch{" H3' 7GU"}=" H3* 7GU"; $resn_exch{"HO3' 7GU"}="H3T  7GU";
$resn_exch{"HOP3 7GU"}="3HOP 7GU"; $resn_exch{"HMA1 7HE"}="1HMA 7HE"; $resn_exch{"HMA2 7HE"}="2HMA 7HE";
$resn_exch{"HMA3 7HE"}="3HMA 7HE"; $resn_exch{"HAA1 7HE"}="1HAA 7HE"; $resn_exch{"HAA2 7HE"}="2HAA 7HE";
$resn_exch{"HAA3 7HE"}="3HAA 7HE"; $resn_exch{"HMB1 7HE"}="1HMB 7HE"; $resn_exch{"HMB2 7HE"}="2HMB 7HE";
$resn_exch{"HMB3 7HE"}="3HMB 7HE"; $resn_exch{"HBB1 7HE"}="1HBB 7HE"; $resn_exch{"HBB2 7HE"}="2HBB 7HE";
$resn_exch{"HMC1 7HE"}="1HMC 7HE"; $resn_exch{"HMC2 7HE"}="2HMC 7HE"; $resn_exch{"HMC3 7HE"}="3HMC 7HE";
$resn_exch{"HBC1 7HE"}="1HBC 7HE"; $resn_exch{"HBC2 7HE"}="2HBC 7HE"; $resn_exch{"HMD1 7HE"}="1HMD 7HE";
$resn_exch{"HMD2 7HE"}="2HMD 7HE"; $resn_exch{"HMD3 7HE"}="3HMD 7HE"; $resn_exch{"HAD1 7HE"}="1HAD 7HE";
$resn_exch{"HAD2 7HE"}="2HAD 7HE"; $resn_exch{"HBD1 7HE"}="1HBD 7HE"; $resn_exch{"HBD2 7HE"}="2HBD 7HE";
$resn_exch{" H31 7I2"}="1H3  7I2"; $resn_exch{" H41 7I2"}="1H4  7I2"; $resn_exch{" H51 7I2"}="1H5  7I2";
$resn_exch{" H61 7I2"}="1H6  7I2"; $resn_exch{"H151 7I2"}="1H15 7I2"; $resn_exch{"H152 7I2"}="2H15 7I2";
$resn_exch{"HN31 7IG"}="1HN3 7IG"; $resn_exch{"HN32 7IG"}="2HN3 7IG"; $resn_exch{"HN41 7IG"}="1HN4 7IG";
$resn_exch{"HN42 7IG"}="2HN4 7IG"; $resn_exch{" H51 7IG"}="1H5  7IG"; $resn_exch{" H52 7IG"}="2H5  7IG";
$resn_exch{" H61 7IG"}="1H6  7IG"; $resn_exch{" H62 7IG"}="2H6  7IG"; $resn_exch{" H63 7IG"}="3H6  7IG";
$resn_exch{"H191 7IG"}="1H19 7IG"; $resn_exch{"H192 7IG"}="2H19 7IG"; $resn_exch{"H31A 7IN"}="1H3  7IN";
$resn_exch{" H32 7IN"}="2H3  7IN"; $resn_exch{"H101 7IN"}="1H10 7IN"; $resn_exch{"H102 7IN"}="2H10 7IN";
$resn_exch{"H191 7IN"}="1H19 7IN"; $resn_exch{"H192 7IN"}="2H19 7IN"; $resn_exch{"H251 7IN"}="1H25 7IN";
$resn_exch{"H252 7IN"}="2H25 7IN"; $resn_exch{"H341 7IN"}="1H34 7IN"; $resn_exch{"H342 7IN"}="2H34 7IN";
$resn_exch{"H11A 7IP"}="1H1  7IP"; $resn_exch{"H12A 7IP"}="2H1  7IP"; $resn_exch{" H13 7IP"}="3H1  7IP";
$resn_exch{"H141 7IP"}="1H14 7IP"; $resn_exch{"H142 7IP"}="2H14 7IP"; $resn_exch{"H151 7IP"}="1H15 7IP";
$resn_exch{"H152 7IP"}="2H15 7IP"; $resn_exch{"H211 7IP"}="1H21 7IP"; $resn_exch{"H212 7IP"}="2H21 7IP";
$resn_exch{"HOP2 7MG"}="2HOP 7MG"; $resn_exch{"HOP3 7MG"}="3HOP 7MG"; $resn_exch{" H5' 7MG"}="1H5* 7MG";
$resn_exch{"H5'' 7MG"}="2H5* 7MG"; $resn_exch{" H4' 7MG"}=" H4* 7MG"; $resn_exch{" H3' 7MG"}=" H3* 7MG";
$resn_exch{"HO3' 7MG"}="*HO3 7MG"; $resn_exch{" H2' 7MG"}=" H2* 7MG"; $resn_exch{"HO2' 7MG"}="*HO2 7MG";
$resn_exch{" H1' 7MG"}=" H1* 7MG"; $resn_exch{" H81 7MG"}="1H8  7MG"; $resn_exch{" H82 7MG"}="2H8  7MG";
$resn_exch{"HN21 7MG"}="1HN2 7MG"; $resn_exch{"HN22 7MG"}="2HN2 7MG"; $resn_exch{"HM71 7MG"}="1HM7 7MG";
$resn_exch{"HM72 7MG"}="2HM7 7MG"; $resn_exch{"HM73 7MG"}="3HM7 7MG"; $resn_exch{"HAT1 7MP"}="1HAT 7MP";
$resn_exch{"HAT2 7MP"}="2HAT 7MP"; $resn_exch{"HAT3 7MP"}="3HAT 7MP"; $resn_exch{"HAS1 7MP"}="1HAS 7MP";
$resn_exch{"HAS2 7MP"}="2HAS 7MP"; $resn_exch{"HAR1 7MP"}="1HAR 7MP"; $resn_exch{"HAR2 7MP"}="2HAR 7MP";
$resn_exch{"HAR3 7MP"}="3HAR 7MP"; $resn_exch{"H3M1 7MQ"}="1H3M 7MQ"; $resn_exch{"H3M2 7MQ"}="2H3M 7MQ";
$resn_exch{"H3M3 7MQ"}="3H3M 7MQ"; $resn_exch{"H111 7MQ"}="1H11 7MQ"; $resn_exch{"H112 7MQ"}="2H11 7MQ";
$resn_exch{"H141 7MQ"}="1H14 7MQ"; $resn_exch{"H142 7MQ"}="2H14 7MQ"; $resn_exch{"H143 7MQ"}="3H14 7MQ";
$resn_exch{"H151 7MQ"}="1H15 7MQ"; $resn_exch{"H152 7MQ"}="2H15 7MQ"; $resn_exch{"H161 7MQ"}="1H16 7MQ";
$resn_exch{"H162 7MQ"}="2H16 7MQ"; $resn_exch{"H191 7MQ"}="1H19 7MQ"; $resn_exch{"H192 7MQ"}="2H19 7MQ";
$resn_exch{"H193 7MQ"}="3H19 7MQ"; $resn_exch{"H201 7MQ"}="1H20 7MQ"; $resn_exch{"H202 7MQ"}="2H20 7MQ";
$resn_exch{"H211 7MQ"}="1H21 7MQ"; $resn_exch{"H212 7MQ"}="2H21 7MQ"; $resn_exch{"H241 7MQ"}="1H24 7MQ";
$resn_exch{"H242 7MQ"}="2H24 7MQ"; $resn_exch{"H243 7MQ"}="3H24 7MQ"; $resn_exch{"H251 7MQ"}="1H25 7MQ";
$resn_exch{"H252 7MQ"}="2H25 7MQ"; $resn_exch{"H261 7MQ"}="1H26 7MQ"; $resn_exch{"H262 7MQ"}="2H26 7MQ";
$resn_exch{"H291 7MQ"}="1H29 7MQ"; $resn_exch{"H292 7MQ"}="2H29 7MQ"; $resn_exch{"H293 7MQ"}="3H29 7MQ";
$resn_exch{"H301 7MQ"}="1H30 7MQ"; $resn_exch{"H302 7MQ"}="2H30 7MQ"; $resn_exch{"H311 7MQ"}="1H31 7MQ";
$resn_exch{"H312 7MQ"}="2H31 7MQ"; $resn_exch{"H341 7MQ"}="1H34 7MQ"; $resn_exch{"H342 7MQ"}="2H34 7MQ";
$resn_exch{"H343 7MQ"}="3H34 7MQ"; $resn_exch{"H351 7MQ"}="1H35 7MQ"; $resn_exch{"H352 7MQ"}="2H35 7MQ";
$resn_exch{"H361 7MQ"}="1H36 7MQ"; $resn_exch{"H362 7MQ"}="2H36 7MQ"; $resn_exch{"H391 7MQ"}="1H39 7MQ";
$resn_exch{"H392 7MQ"}="2H39 7MQ"; $resn_exch{"H393 7MQ"}="3H39 7MQ"; $resn_exch{"H401 7MQ"}="1H40 7MQ";
$resn_exch{"H402 7MQ"}="2H40 7MQ"; $resn_exch{"H411 7MQ"}="1H41 7MQ"; $resn_exch{"H412 7MQ"}="2H41 7MQ";
$resn_exch{"H441 7MQ"}="1H44 7MQ"; $resn_exch{"H442 7MQ"}="2H44 7MQ"; $resn_exch{"H443 7MQ"}="3H44 7MQ";
$resn_exch{"H451 7MQ"}="1H45 7MQ"; $resn_exch{"H452 7MQ"}="2H45 7MQ"; $resn_exch{"H453 7MQ"}="3H45 7MQ";
$resn_exch{"HAN1 7MR"}="1HAN 7MR"; $resn_exch{"HAN2 7MR"}="2HAN 7MR"; $resn_exch{"HAZ1 7MR"}="1HAZ 7MR";
$resn_exch{"HAZ2 7MR"}="2HAZ 7MR"; $resn_exch{" H2B 7NH"}="H2'' 7NH"; $resn_exch{" H3B 7NH"}="H3'' 7NH";
$resn_exch{" H4B 7NH"}="H4'' 7NH"; $resn_exch{" H6B 7NH"}="H6'' 7NH"; $resn_exch{"H511 7NH"}="1H51 7NH";
$resn_exch{"H512 7NH"}="2H51 7NH"; $resn_exch{"H541 7NH"}="1H54 7NH"; $resn_exch{"H542 7NH"}="2H54 7NH";
$resn_exch{" H11 7NI"}="1H1  7NI"; $resn_exch{" H31 7NI"}="1H3  7NI"; $resn_exch{" H41 7NI"}="1H4  7NI";
$resn_exch{" H51 7NI"}="1H5  7NI"; $resn_exch{" H61 7NI"}="1H6  7NI"; $resn_exch{" H11 7PA"}="1H1  7PA";
$resn_exch{" H12 7PA"}="2H1  7PA"; $resn_exch{" H21 7PA"}="1H2  7PA"; $resn_exch{" H22 7PA"}="2H2  7PA";
$resn_exch{" H31 7PA"}="1H3  7PA"; $resn_exch{" H32 7PA"}="2H3  7PA"; $resn_exch{"H291 7PC"}="1H29 7PC";
$resn_exch{"H292 7PC"}="2H29 7PC"; $resn_exch{"H301 7PC"}="1H30 7PC"; $resn_exch{"H302 7PC"}="2H30 7PC";
$resn_exch{"H211 7PE"}="1H21 7PE"; $resn_exch{"H212 7PE"}="2H21 7PE"; $resn_exch{"H213 7PE"}="3H21 7PE";
$resn_exch{"H201 7PE"}="1H20 7PE"; $resn_exch{"H202 7PE"}="2H20 7PE"; $resn_exch{"H181 7PE"}="1H18 7PE";
$resn_exch{"H182 7PE"}="2H18 7PE"; $resn_exch{"H171 7PE"}="1H17 7PE"; $resn_exch{"H172 7PE"}="2H17 7PE";
$resn_exch{"H151 7PE"}="1H15 7PE"; $resn_exch{"H152 7PE"}="2H15 7PE"; $resn_exch{"H141 7PE"}="1H14 7PE";
$resn_exch{"H142 7PE"}="2H14 7PE"; $resn_exch{"H121 7PE"}="1H12 7PE"; $resn_exch{"H122 7PE"}="2H12 7PE";
$resn_exch{"H111 7PE"}="1H11 7PE"; $resn_exch{"H112 7PE"}="2H11 7PE"; $resn_exch{" H91 7PE"}="1H9  7PE";
$resn_exch{" H92 7PE"}="2H9  7PE"; $resn_exch{" H81 7PE"}="1H8  7PE"; $resn_exch{" H82 7PE"}="2H8  7PE";
$resn_exch{" H61 7PE"}="1H6  7PE"; $resn_exch{" H62 7PE"}="2H6  7PE"; $resn_exch{" H51 7PE"}="1H5  7PE";
$resn_exch{" H52 7PE"}="2H5  7PE"; $resn_exch{" H31 7PE"}="1H3  7PE"; $resn_exch{" H32 7PE"}="2H3  7PE";
$resn_exch{" H21 7PE"}="1H2  7PE"; $resn_exch{" H22 7PE"}="2H2  7PE"; $resn_exch{" H11 7PG"}="1H1  7PG";
$resn_exch{" H12 7PG"}="2H1  7PG"; $resn_exch{" H21 7PG"}="1H2  7PG"; $resn_exch{" H22 7PG"}="2H2  7PG";
$resn_exch{" H31 7PG"}="1H3  7PG"; $resn_exch{" H32 7PG"}="2H3  7PG"; $resn_exch{" H41 7PG"}="1H4  7PG";
$resn_exch{" H42 7PG"}="2H4  7PG"; $resn_exch{" H51 7PG"}="1H5  7PG"; $resn_exch{" H52 7PG"}="2H5  7PG";
$resn_exch{" H61 7PG"}="1H6  7PG"; $resn_exch{" H62 7PG"}="2H6  7PG"; $resn_exch{" H71 7PG"}="1H7  7PG";
$resn_exch{" H72 7PG"}="2H7  7PG"; $resn_exch{" H81 7PG"}="1H8  7PG"; $resn_exch{" H82 7PG"}="2H8  7PG";
$resn_exch{" H91 7PG"}="1H9  7PG"; $resn_exch{" H92 7PG"}="2H9  7PG"; $resn_exch{"H101 7PG"}="1H10 7PG";
$resn_exch{"H102 7PG"}="2H10 7PG"; $resn_exch{"H111 7PG"}="1H11 7PG"; $resn_exch{"H112 7PG"}="2H11 7PG";
$resn_exch{"H121 7PG"}="1H12 7PG"; $resn_exch{"H122 7PG"}="2H12 7PG"; $resn_exch{"H131 7PG"}="1H13 7PG";
$resn_exch{"H132 7PG"}="2H13 7PG"; $resn_exch{"H141 7PG"}="1H14 7PG"; $resn_exch{"H142 7PG"}="2H14 7PG";
$resn_exch{"H151 7PG"}="1H15 7PG"; $resn_exch{"H152 7PG"}="2H15 7PG"; $resn_exch{"H161 7PG"}="1H16 7PG";
$resn_exch{"H162 7PG"}="2H16 7PG"; $resn_exch{"H171 7PG"}="1H17 7PG"; $resn_exch{"H172 7PG"}="2H17 7PG";
$resn_exch{"H173 7PG"}="3H17 7PG"; $resn_exch{"HAX1 7PY"}="1HAX 7PY"; $resn_exch{"HAX2 7PY"}="2HAX 7PY";
$resn_exch{"HAX3 7PY"}="3HAX 7PY"; $resn_exch{"HAZ1 7PY"}="1HAZ 7PY"; $resn_exch{"HAZ2 7PY"}="2HAZ 7PY";
$resn_exch{"HAZ3 7PY"}="3HAZ 7PY"; $resn_exch{"HBB1 7PY"}="1HBB 7PY"; $resn_exch{"HBB2 7PY"}="2HBB 7PY";
$resn_exch{"HBB3 7PY"}="3HBB 7PY"; $resn_exch{"H101 7RA"}="1H10 7RA"; $resn_exch{"H102 7RA"}="2H10 7RA";
$resn_exch{"HOP2 7RA"}="2HOP 7RA"; $resn_exch{"HOP3 7RA"}="3HOP 7RA"; $resn_exch{"H5'1 7RA"}="1H5* 7RA";
$resn_exch{"H5'2 7RA"}="2H5* 7RA"; $resn_exch{" H4' 7RA"}=" H4* 7RA"; $resn_exch{" H3' 7RA"}=" H3* 7RA";
$resn_exch{"HO'3 7RA"}="3HO* 7RA"; $resn_exch{" H2' 7RA"}=" H2* 7RA"; $resn_exch{"HO'2 7RA"}="2HO* 7RA";
$resn_exch{" H1' 7RA"}=" H1* 7RA"; $resn_exch{"HOP2 7RP"}="2HOP 7RP"; $resn_exch{"HOP3 7RP"}="3HOP 7RP";
$resn_exch{"H5'1 7RP"}="1H5* 7RP"; $resn_exch{"H5'2 7RP"}="2H5* 7RP"; $resn_exch{" H4' 7RP"}=" H4* 7RP";
$resn_exch{" H3' 7RP"}=" H3* 7RP"; $resn_exch{"HO'3 7RP"}="3HO* 7RP"; $resn_exch{" H2' 7RP"}=" H2* 7RP";
$resn_exch{"HO'2 7RP"}="2HO* 7RP"; $resn_exch{" H1' 7RP"}=" H1* 7RP"; $resn_exch{"HAY1 7XY"}="1HAY 7XY";
$resn_exch{"HAY2 7XY"}="2HAY 7XY"; $resn_exch{"HAZ1 7XY"}="1HAZ 7XY"; $resn_exch{"HAZ2 7XY"}="2HAZ 7XY";
$resn_exch{"HAW1 7XY"}="1HAW 7XY"; $resn_exch{"HAW2 7XY"}="2HAW 7XY"; $resn_exch{"HAX1 7XY"}="1HAX 7XY";
$resn_exch{"HAX2 7XY"}="2HAX 7XY"; $resn_exch{" H21 803"}="1H2  803"; $resn_exch{" H22 803"}="2H2  803";
$resn_exch{"H211 803"}="1H21 803"; $resn_exch{"H212 803"}="2H21 803"; $resn_exch{"H213 803"}="3H21 803";
$resn_exch{"H201 803"}="1H20 803"; $resn_exch{"H202 803"}="2H20 803"; $resn_exch{"H203 803"}="3H20 803";
$resn_exch{"H101 803"}="1H10 803"; $resn_exch{"H102 803"}="2H10 803"; $resn_exch{"H111 803"}="1H11 803";
$resn_exch{"H112 803"}="2H11 803"; $resn_exch{"H131 803"}="1H13 803"; $resn_exch{"H132 803"}="2H13 803";
$resn_exch{"H151 803"}="1H15 803"; $resn_exch{"H152 803"}="2H15 803"; $resn_exch{"H191 803"}="1H19 803";
$resn_exch{"H192 803"}="2H19 803"; $resn_exch{"H193 803"}="3H19 803"; $resn_exch{"H171 803"}="1H17 803";
$resn_exch{"H172 803"}="2H17 803"; $resn_exch{"H181 803"}="1H18 803"; $resn_exch{"H182 803"}="2H18 803";
$resn_exch{"H183 803"}="3H18 803"; $resn_exch{"HN21 806"}="1HN2 806"; $resn_exch{"HN22 806"}="2HN2 806";
$resn_exch{"H121 806"}="1H12 806"; $resn_exch{"H122 806"}="2H12 806"; $resn_exch{"H421 806"}="1H42 806";
$resn_exch{"H422 806"}="2H42 806"; $resn_exch{"H431 806"}="1H43 806"; $resn_exch{"H432 806"}="2H43 806";
$resn_exch{"H441 806"}="1H44 806"; $resn_exch{"H442 806"}="2H44 806"; $resn_exch{"H451 806"}="1H45 806";
$resn_exch{"H452 806"}="2H45 806"; $resn_exch{"H471 806"}="1H47 806"; $resn_exch{"H472 806"}="2H47 806";
$resn_exch{"H473 806"}="3H47 806"; $resn_exch{" H11 815"}="1H1  815"; $resn_exch{" H21 815"}="1H2  815";
$resn_exch{" H51 815"}="1H5  815"; $resn_exch{"H101 815"}="1H10 815"; $resn_exch{"H111 815"}="1H11 815";
$resn_exch{"H171 815"}="1H17 815"; $resn_exch{"H181 815"}="1H18 815"; $resn_exch{"H191 815"}="1H19 815";
$resn_exch{"H251 815"}="1H25 815"; $resn_exch{"H301 815"}="1H30 815"; $resn_exch{"H311 815"}="1H31 815";
$resn_exch{"H312 815"}="2H31 815"; $resn_exch{"H321 815"}="1H32 815"; $resn_exch{"H322 815"}="2H32 815";
$resn_exch{"H371 815"}="1H37 815"; $resn_exch{"H421 815"}="1H42 815"; $resn_exch{"H422 815"}="2H42 815";
$resn_exch{"H431 815"}="1H43 815"; $resn_exch{"H432 815"}="2H43 815"; $resn_exch{"HN21 817"}="1HN2 817";
$resn_exch{"HN22 817"}="2HN2 817"; $resn_exch{" H31 817"}="1H3  817"; $resn_exch{" H32 817"}="2H3  817";
$resn_exch{"H31A 818"}="1H3  818"; $resn_exch{"H32A 818"}="2H3  818"; $resn_exch{" H41 818"}="1H4  818";
$resn_exch{" H42 818"}="2H4  818"; $resn_exch{" H51 818"}="1H5  818"; $resn_exch{" H52 818"}="2H5  818";
$resn_exch{" H61 818"}="1H6  818"; $resn_exch{" H62 818"}="2H6  818"; $resn_exch{"H121 818"}="1H12 818";
$resn_exch{"H122 818"}="2H12 818"; $resn_exch{"H161 818"}="1H16 818"; $resn_exch{"H162 818"}="2H16 818";
$resn_exch{"H231 818"}="1H23 818"; $resn_exch{"H232 818"}="2H23 818"; $resn_exch{"H301 818"}="1H30 818";
$resn_exch{"H302 818"}="2H30 818"; $resn_exch{"H331 818"}="1H33 818"; $resn_exch{"H332 818"}="2H33 818";
$resn_exch{"H341 818"}="1H34 818"; $resn_exch{"H342 818"}="2H34 818"; $resn_exch{"H351 818"}="1H35 818";
$resn_exch{"H352 818"}="2H35 818"; $resn_exch{"H353 818"}="3H35 818"; $resn_exch{"H361 818"}="1H36 818";
$resn_exch{"H362 818"}="2H36 818"; $resn_exch{"H363 818"}="3H36 818"; $resn_exch{"H371 818"}="1H37 818";
$resn_exch{"H372 818"}="2H37 818"; $resn_exch{"H373 818"}="3H37 818"; $resn_exch{"H381 818"}="1H38 818";
$resn_exch{"H382 818"}="2H38 818"; $resn_exch{"H391 818"}="1H39 818"; $resn_exch{"H392 818"}="2H39 818";
$resn_exch{"H393 818"}="3H39 818"; $resn_exch{"H411 818"}="1H41 818"; $resn_exch{"H412 818"}="2H41 818";
$resn_exch{"H413 818"}="3H41 818"; $resn_exch{"H421 818"}="1H42 818"; $resn_exch{"H422 818"}="2H42 818";
$resn_exch{"H423 818"}="3H42 818"; $resn_exch{"H431 818"}="1H43 818"; $resn_exch{"H432 818"}="2H43 818";
$resn_exch{"H433 818"}="3H43 818"; $resn_exch{"H441 818"}="1H44 818"; $resn_exch{"H442 818"}="2H44 818";
$resn_exch{"H443 818"}="3H44 818"; $resn_exch{"H451 818"}="1H45 818"; $resn_exch{"H452 818"}="2H45 818";
$resn_exch{"H453 818"}="3H45 818"; $resn_exch{"HO10 818"}="0HO1 818"; $resn_exch{"HO12 818"}="2HO1 818";
$resn_exch{"HO13 818"}="3HO1 818"; $resn_exch{"H161 81A"}="1H16 81A"; $resn_exch{"H162 81A"}="2H16 81A";
$resn_exch{"H163 81A"}="3H16 81A"; $resn_exch{"H21A 81A"}="1H2A 81A"; $resn_exch{"H22A 81A"}="2H2A 81A";
$resn_exch{"H23A 81A"}="3H2A 81A"; $resn_exch{"H101 81A"}="1H10 81A"; $resn_exch{"H102 81A"}="2H10 81A";
$resn_exch{"H141 81A"}="1H14 81A"; $resn_exch{"H142 81A"}="2H14 81A"; $resn_exch{"H143 81A"}="3H14 81A";
$resn_exch{"H144 81A"}="4H14 81A"; $resn_exch{"H151 81A"}="1H15 81A"; $resn_exch{"H152 81A"}="2H15 81A";
$resn_exch{" H51 821"}="1H5  821"; $resn_exch{" H52 821"}="2H5  821"; $resn_exch{" H53 821"}="3H5  821";
$resn_exch{"H151 821"}="1H15 821"; $resn_exch{"H152 821"}="2H15 821"; $resn_exch{"H161 821"}="1H16 821";
$resn_exch{"H162 821"}="2H16 821"; $resn_exch{"H191 821"}="1H19 821"; $resn_exch{"H192 821"}="2H19 821";
$resn_exch{"H221 821"}="1H22 821"; $resn_exch{"H222 821"}="2H22 821"; $resn_exch{"H301 821"}="1H30 821";
$resn_exch{"H302 821"}="2H30 821"; $resn_exch{"H651 821"}="1H65 821"; $resn_exch{"H652 821"}="2H65 821";
$resn_exch{"H181 822"}="1H18 822"; $resn_exch{"H182 822"}="2H18 822"; $resn_exch{"H281 822"}="1H28 822";
$resn_exch{"H282 822"}="2H28 822"; $resn_exch{"H283 822"}="3H28 822"; $resn_exch{"H381 822"}="1H38 822";
$resn_exch{"H382 822"}="2H38 822"; $resn_exch{"H391 822"}="1H39 822"; $resn_exch{"H392 822"}="2H39 822";
$resn_exch{"H421 822"}="1H42 822"; $resn_exch{"H422 822"}="2H42 822"; $resn_exch{"H461 822"}="1H46 822";
$resn_exch{"H462 822"}="2H46 822"; $resn_exch{"H491 822"}="1H49 822"; $resn_exch{"H492 822"}="2H49 822";
$resn_exch{"H111 826"}="1H11 826"; $resn_exch{"H112 826"}="2H11 826"; $resn_exch{"H121 826"}="1H12 826";
$resn_exch{"H122 826"}="2H12 826"; $resn_exch{"H151 826"}="1H15 826"; $resn_exch{"H152 826"}="2H15 826";
$resn_exch{"H231 826"}="1H23 826"; $resn_exch{"H232 826"}="2H23 826"; $resn_exch{"H21A 82A"}="1H2  82A";
$resn_exch{"H22A 82A"}="2H2  82A"; $resn_exch{"H211 839"}="1H21 839"; $resn_exch{"H212 839"}="2H21 839";
$resn_exch{"H221 839"}="1H22 839"; $resn_exch{"H222 839"}="2H22 839"; $resn_exch{"H111 843"}="1H11 843";
$resn_exch{"H112 843"}="2H11 843"; $resn_exch{"H171 843"}="1H17 843"; $resn_exch{"H172 843"}="2H17 843";
$resn_exch{"H173 843"}="3H17 843"; $resn_exch{"H211 843"}="1H21 843"; $resn_exch{"H212 843"}="2H21 843";
$resn_exch{"H221 843"}="1H22 843"; $resn_exch{"H222 843"}="2H22 843"; $resn_exch{"H231 843"}="1H23 843";
$resn_exch{"H232 843"}="2H23 843"; $resn_exch{"H241 843"}="1H24 843"; $resn_exch{"H242 843"}="2H24 843";
$resn_exch{"H271 843"}="1H27 843"; $resn_exch{"H272 843"}="2H27 843"; $resn_exch{"H21A 846"}="1H2  846";
$resn_exch{" H22 846"}="2H2  846"; $resn_exch{" H71 846"}="1H7  846"; $resn_exch{" H72 846"}="2H7  846";
$resn_exch{"H311 846"}="1H31 846"; $resn_exch{"H312 846"}="2H31 846"; $resn_exch{"H611 846"}="1H61 846";
$resn_exch{"H612 846"}="2H61 846"; $resn_exch{"H711 846"}="1H71 846"; $resn_exch{"H712 846"}="2H71 846";
$resn_exch{"H721 846"}="1H72 846"; $resn_exch{"H722 846"}="2H72 846"; $resn_exch{" H71 852"}="1H7  852";
$resn_exch{" H72 852"}="2H7  852"; $resn_exch{"H151 852"}="1H15 852"; $resn_exch{"H152 852"}="2H15 852";
$resn_exch{"H153 852"}="3H15 852"; $resn_exch{"H201 852"}="1H20 852"; $resn_exch{"H202 852"}="2H20 852";
$resn_exch{"H211 852"}="1H21 852"; $resn_exch{"H212 852"}="2H21 852"; $resn_exch{"H221 852"}="1H22 852";
$resn_exch{"H222 852"}="2H22 852"; $resn_exch{"H241 852"}="1H24 852"; $resn_exch{"H242 852"}="2H24 852";
$resn_exch{"H251 852"}="1H25 852"; $resn_exch{"H252 852"}="2H25 852"; $resn_exch{"H101 853"}="1H10 853";
$resn_exch{"H102 853"}="2H10 853"; $resn_exch{"H201 853"}="1H20 853"; $resn_exch{"H202 853"}="2H20 853";
$resn_exch{"H203 853"}="3H20 853"; $resn_exch{"H281 853"}="1H28 853"; $resn_exch{"H282 853"}="2H28 853";
$resn_exch{"H291 853"}="1H29 853"; $resn_exch{"H292 853"}="2H29 853"; $resn_exch{"H321 853"}="1H32 853";
$resn_exch{"H322 853"}="2H32 853"; $resn_exch{"H361 853"}="1H36 853"; $resn_exch{"H362 853"}="2H36 853";
$resn_exch{"H391 853"}="1H39 853"; $resn_exch{"H392 853"}="2H39 853"; $resn_exch{"H711 853"}="1H71 853";
$resn_exch{"H712 853"}="2H71 853"; $resn_exch{"H221 857"}="1H22 857"; $resn_exch{"H222 857"}="2H22 857";
$resn_exch{"H223 857"}="3H22 857"; $resn_exch{"H231 857"}="1H23 857"; $resn_exch{"H232 857"}="2H23 857";
$resn_exch{"H21A 857"}="1H2  857"; $resn_exch{" H22 857"}="2H2  857"; $resn_exch{" H31 857"}="1H3  857";
$resn_exch{" H32 857"}="2H3  857"; $resn_exch{" H41 857"}="1H4  857"; $resn_exch{" H42 857"}="2H4  857";
$resn_exch{"H261 857"}="1H26 857"; $resn_exch{"H262 857"}="2H26 857"; $resn_exch{"H263 857"}="3H26 857";
$resn_exch{"H31A 858"}="1H3  858"; $resn_exch{"H32A 858"}="2H3  858"; $resn_exch{" H41 858"}="1H4  858";
$resn_exch{" H42 858"}="2H4  858"; $resn_exch{" H51 858"}="1H5  858"; $resn_exch{" H52 858"}="2H5  858";
$resn_exch{" H61 858"}="1H6  858"; $resn_exch{"H62A 858"}="2H6  858"; $resn_exch{"H121 858"}="1H12 858";
$resn_exch{"H122 858"}="2H12 858"; $resn_exch{"H161 858"}="1H16 858"; $resn_exch{"H162 858"}="2H16 858";
$resn_exch{"H182 858"}="2H18 858"; $resn_exch{"H231 858"}="1H23 858"; $resn_exch{"H232 858"}="2H23 858";
$resn_exch{"H301 858"}="1H30 858"; $resn_exch{"H302 858"}="2H30 858"; $resn_exch{"H331 858"}="1H33 858";
$resn_exch{"H332 858"}="2H33 858"; $resn_exch{"H341 858"}="1H34 858"; $resn_exch{"H342 858"}="2H34 858";
$resn_exch{"H351 858"}="1H35 858"; $resn_exch{"H352 858"}="2H35 858"; $resn_exch{"H353 858"}="3H35 858";
$resn_exch{"H361 858"}="1H36 858"; $resn_exch{"H362 858"}="2H36 858"; $resn_exch{"H363 858"}="3H36 858";
$resn_exch{"H371 858"}="1H37 858"; $resn_exch{"H372 858"}="2H37 858"; $resn_exch{"H373 858"}="3H37 858";
$resn_exch{"H381 858"}="1H38 858"; $resn_exch{"H382 858"}="2H38 858"; $resn_exch{"H391 858"}="1H39 858";
$resn_exch{"H392 858"}="2H39 858"; $resn_exch{"H393 858"}="3H39 858"; $resn_exch{"H411 858"}="1H41 858";
$resn_exch{"H412 858"}="2H41 858"; $resn_exch{"H413 858"}="3H41 858"; $resn_exch{"H421 858"}="1H42 858";
$resn_exch{"H422 858"}="2H42 858"; $resn_exch{"H423 858"}="3H42 858"; $resn_exch{"H431 858"}="1H43 858";
$resn_exch{"H432 858"}="2H43 858"; $resn_exch{"H433 858"}="3H43 858"; $resn_exch{"H441 858"}="1H44 858";
$resn_exch{"H442 858"}="2H44 858"; $resn_exch{"H443 858"}="3H44 858"; $resn_exch{"H451 858"}="1H45 858";
$resn_exch{"H452 858"}="2H45 858"; $resn_exch{"H453 858"}="3H45 858"; $resn_exch{"H671 858"}="1H67 858";
$resn_exch{"H672 858"}="2H67 858"; $resn_exch{"H681 858"}="1H68 858"; $resn_exch{"H682 858"}="2H68 858";
$resn_exch{"H683 858"}="3H68 858"; $resn_exch{"HO10 858"}="0HO1 858"; $resn_exch{"H231 859"}="1H23 859";
$resn_exch{"H232 859"}="2H23 859"; $resn_exch{"H221 85A"}="1H22 85A"; $resn_exch{"H222 85A"}="2H22 85A";
$resn_exch{"H261 85A"}="1H26 85A"; $resn_exch{"H262 85A"}="2H26 85A"; $resn_exch{"H271 85A"}="1H27 85A";
$resn_exch{"H272 85A"}="2H27 85A"; $resn_exch{"H231 85A"}="1H23 85A"; $resn_exch{"H232 85A"}="2H23 85A";
$resn_exch{"H241 85A"}="1H24 85A"; $resn_exch{"H242 85A"}="2H24 85A"; $resn_exch{"H281 85A"}="1H28 85A";
$resn_exch{"H282 85A"}="2H28 85A"; $resn_exch{" H91 864"}="1H9  864"; $resn_exch{" H92 864"}="2H9  864";
$resn_exch{"H101 864"}="1H10 864"; $resn_exch{"H102 864"}="2H10 864"; $resn_exch{"H151 864"}="1H15 864";
$resn_exch{"H152 864"}="2H15 864"; $resn_exch{"H271 864"}="1H27 864"; $resn_exch{"H272 864"}="2H27 864";
$resn_exch{"H273 864"}="3H27 864"; $resn_exch{"H281 864"}="1H28 864"; $resn_exch{"H282 864"}="2H28 864";
$resn_exch{"H283 864"}="3H28 864"; $resn_exch{"H331 864"}="1H33 864"; $resn_exch{"H332 864"}="2H33 864";
$resn_exch{"H311 869"}="1H31 869"; $resn_exch{"H312 869"}="2H31 869"; $resn_exch{"H313 869"}="3H31 869";
$resn_exch{" H71 869"}="1H7  869"; $resn_exch{" H72 869"}="2H7  869"; $resn_exch{" H73 869"}="3H7  869";
$resn_exch{"H141 869"}="1H14 869"; $resn_exch{"H142 869"}="2H14 869"; $resn_exch{"H143 869"}="3H14 869";
$resn_exch{"H211 869"}="1H21 869"; $resn_exch{"H212 869"}="2H21 869"; $resn_exch{"H213 869"}="3H21 869";
$resn_exch{"H221 869"}="1H22 869"; $resn_exch{"H222 869"}="2H22 869"; $resn_exch{"H223 869"}="3H22 869";
$resn_exch{"H231 869"}="1H23 869"; $resn_exch{"H232 869"}="2H23 869"; $resn_exch{"H233 869"}="3H23 869";
$resn_exch{"H331 86A"}="1H33 86A"; $resn_exch{"H332 86A"}="2H33 86A"; $resn_exch{"H241 86A"}="1H24 86A";
$resn_exch{"H242 86A"}="2H24 86A"; $resn_exch{"H281 86A"}="1H28 86A"; $resn_exch{"H282 86A"}="2H28 86A";
$resn_exch{"H283 86A"}="3H28 86A"; $resn_exch{"H271 86A"}="1H27 86A"; $resn_exch{"H272 86A"}="2H27 86A";
$resn_exch{"H273 86A"}="3H27 86A"; $resn_exch{"H111 86A"}="1H11 86A"; $resn_exch{"H112 86A"}="2H11 86A";
$resn_exch{"H131 870"}="1H13 870"; $resn_exch{"H132 870"}="2H13 870"; $resn_exch{"H133 870"}="3H13 870";
$resn_exch{"HN21 870"}="1HN2 870"; $resn_exch{"HN22 870"}="2HN2 870"; $resn_exch{"H111 872"}="1H11 872";
$resn_exch{"H112 872"}="2H11 872"; $resn_exch{"H151 872"}="1H15 872"; $resn_exch{"H152 872"}="2H15 872";
$resn_exch{"H201 872"}="1H20 872"; $resn_exch{"H202 872"}="2H20 872"; $resn_exch{"H251 872"}="1H25 872";
$resn_exch{"H252 872"}="2H25 872"; $resn_exch{"H261 872"}="1H26 872"; $resn_exch{"H262 872"}="2H26 872";
$resn_exch{"H271 872"}="1H27 872"; $resn_exch{"H272 872"}="2H27 872"; $resn_exch{"H371 872"}="1H37 872";
$resn_exch{"H372 872"}="2H37 872"; $resn_exch{" H71 876"}="1H7  876"; $resn_exch{"H72A 876"}="2H7  876";
$resn_exch{"H131 876"}="1H13 876"; $resn_exch{"H132 876"}="2H13 876"; $resn_exch{"H133 876"}="3H13 876";
$resn_exch{"H171 876"}="1H17 876"; $resn_exch{"H172 876"}="2H17 876"; $resn_exch{"H181 876"}="1H18 876";
$resn_exch{"H182 876"}="2H18 876"; $resn_exch{"H191 876"}="1H19 876"; $resn_exch{"H192 876"}="2H19 876";
$resn_exch{"H201 876"}="1H20 876"; $resn_exch{"H202 876"}="2H20 876"; $resn_exch{"H231 876"}="1H23 876";
$resn_exch{"H232 876"}="2H23 876"; $resn_exch{" H91 877"}="1H9  877"; $resn_exch{" H92 877"}="2H9  877";
$resn_exch{"H231 877"}="1H23 877"; $resn_exch{"H232 877"}="2H23 877"; $resn_exch{"H233 877"}="3H23 877";
$resn_exch{"H281 877"}="1H28 877"; $resn_exch{"H282 877"}="2H28 877"; $resn_exch{"H283 877"}="3H28 877";
$resn_exch{"H331 877"}="1H33 877"; $resn_exch{"H332 877"}="2H33 877"; $resn_exch{"H371 877"}="1H37 877";
$resn_exch{"H372 877"}="2H37 877"; $resn_exch{"H381 877"}="1H38 877"; $resn_exch{"H382 877"}="2H38 877";
$resn_exch{"H383 877"}="3H38 877"; $resn_exch{"H152 87Y"}="2H15 87Y"; $resn_exch{"H151 87Y"}="1H15 87Y";
$resn_exch{"H142 87Y"}="2H14 87Y"; $resn_exch{"H141 87Y"}="1H14 87Y"; $resn_exch{"H133 87Y"}="3H13 87Y";
$resn_exch{"H132 87Y"}="2H13 87Y"; $resn_exch{"H131 87Y"}="1H13 87Y"; $resn_exch{" H52 87Y"}="2H5  87Y";
$resn_exch{" H51 87Y"}="1H5  87Y"; $resn_exch{"H172 87Y"}="2H17 87Y"; $resn_exch{"H171 87Y"}="1H17 87Y";
$resn_exch{"H061 880"}="1H06 880"; $resn_exch{"H062 880"}="2H06 880"; $resn_exch{"H071 880"}="1H07 880";
$resn_exch{"H072 880"}="2H07 880"; $resn_exch{"H101 880"}="1H10 880"; $resn_exch{"H102 880"}="2H10 880";
$resn_exch{"H103 880"}="3H10 880"; $resn_exch{"H171 880"}="1H17 880"; $resn_exch{"H172 880"}="2H17 880";
$resn_exch{"H181 880"}="1H18 880"; $resn_exch{"H182 880"}="2H18 880"; $resn_exch{"H201 880"}="1H20 880";
$resn_exch{"H202 880"}="2H20 880"; $resn_exch{"H211 880"}="1H21 880"; $resn_exch{"H212 880"}="2H21 880";
$resn_exch{"H561 880"}="1H56 880"; $resn_exch{"H562 880"}="2H56 880"; $resn_exch{"H571 880"}="1H57 880";
$resn_exch{"H572 880"}="2H57 880"; $resn_exch{"H581 880"}="1H58 880"; $resn_exch{"H582 880"}="2H58 880";
$resn_exch{"H591 880"}="1H59 880"; $resn_exch{"H592 880"}="2H59 880"; $resn_exch{"H601 880"}="1H60 880";
$resn_exch{"H602 880"}="2H60 880"; $resn_exch{" H71 882"}="1H7  882"; $resn_exch{" H72 882"}="2H7  882";
$resn_exch{" H81 882"}="1H8  882"; $resn_exch{" H82 882"}="2H8  882"; $resn_exch{"H101 882"}="1H10 882";
$resn_exch{"H102 882"}="2H10 882"; $resn_exch{"H131 882"}="1H13 882"; $resn_exch{"H132 882"}="2H13 882";
$resn_exch{"H133 882"}="3H13 882"; $resn_exch{"H351 882"}="1H35 882"; $resn_exch{"H352 882"}="2H35 882";
$resn_exch{"H141 882"}="1H14 882"; $resn_exch{"H142 882"}="2H14 882"; $resn_exch{"H143 882"}="3H14 882";
$resn_exch{"H11A 885"}="1H1  885"; $resn_exch{" H12 885"}="2H1  885"; $resn_exch{" H13 885"}="3H1  885";
$resn_exch{"HN11 885"}="1HN1 885"; $resn_exch{"HN12 885"}="2HN1 885"; $resn_exch{" H81 885"}="1H8  885";
$resn_exch{" H82 885"}="2H8  885"; $resn_exch{" H91 885"}="1H9  885"; $resn_exch{" H92 885"}="2H9  885";
$resn_exch{"H101 885"}="1H10 885"; $resn_exch{"H102 885"}="2H10 885"; $resn_exch{"H311 888"}="1H31 888";
$resn_exch{"H312 888"}="2H31 888"; $resn_exch{"H321 888"}="1H32 888"; $resn_exch{"H322 888"}="2H32 888";
$resn_exch{"H381 888"}="1H38 888"; $resn_exch{"H382 888"}="2H38 888"; $resn_exch{"H383 888"}="3H38 888";
$resn_exch{" H71 888"}="1H7  888"; $resn_exch{" H72 888"}="2H7  888"; $resn_exch{" H73 888"}="3H7  888";
$resn_exch{"H171 892"}="1H17 892"; $resn_exch{"H172 892"}="2H17 892"; $resn_exch{"H181 892"}="1H18 892";
$resn_exch{"H182 892"}="2H18 892"; $resn_exch{"H183 892"}="3H18 892"; $resn_exch{"H021 892"}="1H02 892";
$resn_exch{"H022 892"}="2H02 892"; $resn_exch{" H91 893"}="1H9  893"; $resn_exch{" H92 893"}="2H9  893";
$resn_exch{"H231 893"}="1H23 893"; $resn_exch{"H232 893"}="2H23 893"; $resn_exch{"H233 893"}="3H23 893";
$resn_exch{"H281 893"}="1H28 893"; $resn_exch{"H282 893"}="2H28 893"; $resn_exch{"H283 893"}="3H28 893";
$resn_exch{"H371 893"}="1H37 893"; $resn_exch{"H372 893"}="2H37 893"; $resn_exch{" H71 893"}="1H7  893";
$resn_exch{" H72 893"}="2H7  893"; $resn_exch{" H73 893"}="3H7  893"; $resn_exch{"H9C1 894"}="1H9C 894";
$resn_exch{"H9C2 894"}="2H9C 894"; $resn_exch{"H6C1 894"}="1H6C 894"; $resn_exch{"H6C2 894"}="2H6C 894";
$resn_exch{"H131 894"}="1H13 894"; $resn_exch{"H132 894"}="2H13 894"; $resn_exch{"H141 894"}="1H14 894";
$resn_exch{"H142 894"}="2H14 894"; $resn_exch{"H171 894"}="1H17 894"; $resn_exch{"H172 894"}="2H17 894";
$resn_exch{"H173 894"}="3H17 894"; $resn_exch{"H191 894"}="1H19 894"; $resn_exch{"H192 894"}="2H19 894";
$resn_exch{"H231 894"}="1H23 894"; $resn_exch{"H232 894"}="2H23 894"; $resn_exch{"H201 894"}="1H20 894";
$resn_exch{"H202 894"}="2H20 894"; $resn_exch{"H221 894"}="1H22 894"; $resn_exch{"H222 894"}="2H22 894";
$resn_exch{"H131 895"}="1H13 895"; $resn_exch{"H132 895"}="2H13 895"; $resn_exch{"H141 895"}="1H14 895";
$resn_exch{"H142 895"}="2H14 895"; $resn_exch{"H171 895"}="1H17 895"; $resn_exch{"H172 895"}="2H17 895";
$resn_exch{"H173 895"}="3H17 895"; $resn_exch{"H191 895"}="1H19 895"; $resn_exch{"H192 895"}="2H19 895";
$resn_exch{"H231 895"}="1H23 895"; $resn_exch{"H232 895"}="2H23 895"; $resn_exch{"H201 895"}="1H20 895";
$resn_exch{"H202 895"}="2H20 895"; $resn_exch{"H221 895"}="1H22 895"; $resn_exch{"H222 895"}="2H22 895";
$resn_exch{"H6C1 896"}="1H6C 896"; $resn_exch{"H6C2 896"}="2H6C 896"; $resn_exch{"H5C1 896"}="1H5C 896";
$resn_exch{"H5C2 896"}="2H5C 896"; $resn_exch{"H5C3 896"}="3H5C 896"; $resn_exch{"H241 896"}="1H24 896";
$resn_exch{"H242 896"}="2H24 896"; $resn_exch{"H243 896"}="3H24 896"; $resn_exch{"H231 896"}="1H23 896";
$resn_exch{"H232 896"}="2H23 896"; $resn_exch{"H233 896"}="3H23 896"; $resn_exch{"H201 896"}="1H20 896";
$resn_exch{"H202 896"}="2H20 896"; $resn_exch{"H203 896"}="3H20 896"; $resn_exch{"H1C1 896"}="1H1C 896";
$resn_exch{"H1C2 896"}="2H1C 896"; $resn_exch{"H8C1 896"}="1H8C 896"; $resn_exch{"H8C2 896"}="2H8C 896";
$resn_exch{"H8C3 896"}="3H8C 896"; $resn_exch{"H8C1 897"}="1H8C 897"; $resn_exch{"H8C2 897"}="2H8C 897";
$resn_exch{"H8C3 897"}="3H8C 897"; $resn_exch{"H1C1 897"}="1H1C 897"; $resn_exch{"H1C2 897"}="2H1C 897";
$resn_exch{"H201 897"}="1H20 897"; $resn_exch{"H202 897"}="2H20 897"; $resn_exch{"H203 897"}="3H20 897";
$resn_exch{"H6C1 897"}="1H6C 897"; $resn_exch{"H6C2 897"}="2H6C 897"; $resn_exch{"H231 897"}="1H23 897";
$resn_exch{"H232 897"}="2H23 897"; $resn_exch{"H233 897"}="3H23 897"; $resn_exch{"H241 897"}="1H24 897";
$resn_exch{"H242 897"}="2H24 897"; $resn_exch{"H243 897"}="3H24 897"; $resn_exch{"H5C1 897"}="1H5C 897";
$resn_exch{"H5C2 897"}="2H5C 897"; $resn_exch{"H251 897"}="1H25 897"; $resn_exch{"H252 897"}="2H25 897";
$resn_exch{"H253 897"}="3H25 897"; $resn_exch{"H261 897"}="1H26 897"; $resn_exch{"H262 897"}="2H26 897";
$resn_exch{"H263 897"}="3H26 897"; $resn_exch{"H2C1 897"}="1H2C 897"; $resn_exch{"H2C2 897"}="2H2C 897";
$resn_exch{"H2C3 897"}="3H2C 897"; $resn_exch{"H311 89I"}="1H31 89I"; $resn_exch{"H312 89I"}="2H31 89I";
$resn_exch{"H301 89I"}="1H30 89I"; $resn_exch{"H302 89I"}="2H30 89I"; $resn_exch{"H281 89I"}="1H28 89I";
$resn_exch{"H282 89I"}="2H28 89I"; $resn_exch{"H271 89I"}="1H27 89I"; $resn_exch{"H272 89I"}="2H27 89I";
$resn_exch{"H441 89I"}="1H44 89I"; $resn_exch{"H442 89I"}="2H44 89I"; $resn_exch{"H443 89I"}="3H44 89I";
$resn_exch{"H481 89I"}="1H48 89I"; $resn_exch{"H482 89I"}="2H48 89I"; $resn_exch{"H483 89I"}="3H48 89I";
$resn_exch{"H521 89I"}="1H52 89I"; $resn_exch{"H522 89I"}="2H52 89I"; $resn_exch{"HD21 8AD"}="1HD2 8AD";
$resn_exch{"HD22 8AD"}="2HD2 8AD"; $resn_exch{"HD31 8AD"}="1HD3 8AD"; $resn_exch{"HD32 8AD"}="2HD3 8AD";
$resn_exch{"HD41 8AD"}="1HD4 8AD"; $resn_exch{"HD42 8AD"}="2HD4 8AD"; $resn_exch{"HD51 8AD"}="1HD5 8AD";
$resn_exch{"HD52 8AD"}="2HD5 8AD"; $resn_exch{"HD71 8AD"}="1HD7 8AD"; $resn_exch{"HD72 8AD"}="2HD7 8AD";
$resn_exch{"HD73 8AD"}="3HD7 8AD"; $resn_exch{"HD81 8AD"}="1HD8 8AD"; $resn_exch{"HD82 8AD"}="2HD8 8AD";
$resn_exch{"HD83 8AD"}="3HD8 8AD"; $resn_exch{"HN11 8AP"}="1HN1 8AP"; $resn_exch{"HN12 8AP"}="2HN1 8AP";
$resn_exch{" H91 8AP"}="1H9  8AP"; $resn_exch{" H92 8AP"}="2H9  8AP"; $resn_exch{"HOP2 8BR"}="2HOP 8BR";
$resn_exch{"HOP3 8BR"}="3HOP 8BR"; $resn_exch{"H5'1 8BR"}="1H5* 8BR"; $resn_exch{"H5'2 8BR"}="2H5* 8BR";
$resn_exch{" H4' 8BR"}=" H4* 8BR"; $resn_exch{" H3' 8BR"}=" H3* 8BR"; $resn_exch{"HO3' 8BR"}="*HO3 8BR";
$resn_exch{" H2' 8BR"}=" H2* 8BR"; $resn_exch{"HO2' 8BR"}="*HO2 8BR"; $resn_exch{" H1' 8BR"}=" H1* 8BR";
$resn_exch{"HN61 8BR"}="1HN6 8BR"; $resn_exch{"HN62 8BR"}="2HN6 8BR"; $resn_exch{"H2'1 8CS"}="1H2' 8CS";
$resn_exch{"H2'2 8CS"}="2H2' 8CS"; $resn_exch{"H101 8CS"}="1H10 8CS"; $resn_exch{"H102 8CS"}="2H10 8CS";
$resn_exch{"H5'1 8DA"}="1H5* 8DA"; $resn_exch{"H5'2 8DA"}="2H5* 8DA"; $resn_exch{" H4' 8DA"}=" H4* 8DA";
$resn_exch{" H3' 8DA"}=" H3* 8DA"; $resn_exch{" H1' 8DA"}=" H1* 8DA"; $resn_exch{"H2'1 8DA"}="1H2* 8DA";
$resn_exch{"H2'2 8DA"}="2H2* 8DA"; $resn_exch{"HN61 8DA"}="1HN6 8DA"; $resn_exch{"HN62 8DA"}="2HN6 8DA";
$resn_exch{" H21 8DA"}="1H2  8DA"; $resn_exch{" H22 8DA"}="2H2  8DA"; $resn_exch{"HOG2 8DG"}="2HOG 8DG";
$resn_exch{"HOB2 8DG"}="2HOB 8DG"; $resn_exch{"HOA2 8DG"}="2HOA 8DG"; $resn_exch{"H5'1 8DG"}="1H5* 8DG";
$resn_exch{"H5'2 8DG"}="2H5* 8DG"; $resn_exch{" H4' 8DG"}=" H4* 8DG"; $resn_exch{" H3' 8DG"}=" H3* 8DG";
$resn_exch{"H2'1 8DG"}="1H2* 8DG"; $resn_exch{"H2'2 8DG"}="2H2* 8DG"; $resn_exch{" H1' 8DG"}=" H1* 8DG";
$resn_exch{"HN21 8DG"}="1HN2 8DG"; $resn_exch{"HN22 8DG"}="2HN2 8DG"; $resn_exch{"HOP3 8FG"}="3HOP 8FG";
$resn_exch{"HOP2 8FG"}="2HOP 8FG"; $resn_exch{"HN21 8FG"}="1HN2 8FG"; $resn_exch{"HN22 8FG"}="2HN2 8FG";
$resn_exch{"H371 8FG"}="1H37 8FG"; $resn_exch{"H372 8FG"}="2H37 8FG"; $resn_exch{"H451 8FG"}="1H45 8FG";
$resn_exch{"H452 8FG"}="2H45 8FG"; $resn_exch{"H453 8FG"}="3H45 8FG"; $resn_exch{" H2' 8FG"}="1H2* 8FG";
$resn_exch{"H2'' 8FG"}="2H2* 8FG"; $resn_exch{" H5' 8FG"}="1H5* 8FG"; $resn_exch{"H5'' 8FG"}="2H5* 8FG";
$resn_exch{" H4' 8FG"}=" H4* 8FG"; $resn_exch{" H1' 8FG"}=" H1* 8FG"; $resn_exch{" H3' 8FG"}=" H3* 8FG";
$resn_exch{"HO3' 8FG"}="H3T  8FG"; $resn_exch{" H61 8GP"}="1H6  8GP"; $resn_exch{" H62 8GP"}="2H6  8GP";
$resn_exch{"H101 8GP"}="1H10 8GP"; $resn_exch{"H102 8GP"}="2H10 8GP"; $resn_exch{"H111 8GP"}="1H11 8GP";
$resn_exch{"H112 8GP"}="2H11 8GP"; $resn_exch{"HOG2 8GT"}="2HOG 8GT"; $resn_exch{"HOB2 8GT"}="2HOB 8GT";
$resn_exch{"HOA2 8GT"}="2HOA 8GT"; $resn_exch{"H5'1 8GT"}="1H5* 8GT"; $resn_exch{"H5'2 8GT"}="2H5* 8GT";
$resn_exch{" H4' 8GT"}=" H4* 8GT"; $resn_exch{" H3' 8GT"}=" H3* 8GT"; $resn_exch{" H2' 8GT"}=" H2* 8GT";
$resn_exch{" H1' 8GT"}=" H1* 8GT"; $resn_exch{"HN21 8GT"}="1HN2 8GT"; $resn_exch{"HN22 8GT"}="2HN2 8GT";
$resn_exch{"HN21 8HG"}="1HN2 8HG"; $resn_exch{"HN22 8HG"}="2HN2 8HG"; $resn_exch{" H1' 8HG"}=" H1* 8HG";
$resn_exch{"H2'1 8HG"}="1H2* 8HG"; $resn_exch{"H2'2 8HG"}="2H2* 8HG"; $resn_exch{" H3' 8HG"}=" H3* 8HG";
$resn_exch{" H4' 8HG"}=" H4* 8HG"; $resn_exch{"H5'1 8HG"}="1H5* 8HG"; $resn_exch{"H5'2 8HG"}="2H5* 8HG";
$resn_exch{" H5' 8HG"}=" H5* 8HG"; $resn_exch{"H7N1 8ID"}="1H7N 8ID"; $resn_exch{"H7N2 8ID"}="2H7N 8ID";
$resn_exch{"H5'1 8ID"}="1H5' 8ID"; $resn_exch{"H5'2 8ID"}="2H5' 8ID"; $resn_exch{"H5A1 8ID"}="1H5* 8ID";
$resn_exch{"H5A2 8ID"}="2H5* 8ID"; $resn_exch{" H4B 8ID"}=" H4* 8ID"; $resn_exch{" H1B 8ID"}=" H1* 8ID";
$resn_exch{" H2B 8ID"}=" H2* 8ID"; $resn_exch{"HO2A 8ID"}="HO2* 8ID"; $resn_exch{" H3B 8ID"}=" H3* 8ID";
$resn_exch{"HO3A 8ID"}="HO3* 8ID"; $resn_exch{"H6A1 8ID"}="1H6A 8ID"; $resn_exch{"H6A2 8ID"}="2H6A 8ID";
$resn_exch{"HN21 8IG"}="1HN2 8IG"; $resn_exch{"HN22 8IG"}="2HN2 8IG"; $resn_exch{"HN11 8IG"}="1HN1 8IG";
$resn_exch{"H101 8IN"}="1H10 8IN"; $resn_exch{"H102 8IN"}="2H10 8IN"; $resn_exch{"H131 8IN"}="1H13 8IN";
$resn_exch{"H132 8IN"}="2H13 8IN"; $resn_exch{"H141 8IN"}="1H14 8IN"; $resn_exch{"H142 8IN"}="2H14 8IN";
$resn_exch{"H143 8IN"}="3H14 8IN"; $resn_exch{"H151 8IN"}="1H15 8IN"; $resn_exch{"H152 8IN"}="2H15 8IN";
$resn_exch{"H231 8IN"}="1H23 8IN"; $resn_exch{"H232 8IN"}="2H23 8IN"; $resn_exch{"H241 8IN"}="1H24 8IN";
$resn_exch{"H242 8IN"}="2H24 8IN"; $resn_exch{"H251 8IN"}="1H25 8IN"; $resn_exch{"H252 8IN"}="2H25 8IN";
$resn_exch{"HN11 8IP"}="1HN1 8IP"; $resn_exch{"HN12 8IP"}="2HN1 8IP"; $resn_exch{" H91 8IP"}="1H9  8IP";
$resn_exch{" H92 8IP"}="2H9  8IP"; $resn_exch{" H21 8MG"}="1H2  8MG"; $resn_exch{" H22 8MG"}="2H2  8MG";
$resn_exch{" H2' 8MG"}="1H2* 8MG"; $resn_exch{"H2'' 8MG"}="2H2* 8MG"; $resn_exch{" H5' 8MG"}="1H5* 8MG";
$resn_exch{"H5'' 8MG"}="2H5* 8MG"; $resn_exch{" H4' 8MG"}=" H4* 8MG"; $resn_exch{" H1' 8MG"}=" H1* 8MG";
$resn_exch{" H3' 8MG"}=" H3* 8MG"; $resn_exch{"HO3' 8MG"}="H3T  8MG"; $resn_exch{" HC1 8MG"}="1HC  8MG";
$resn_exch{" HC2 8MG"}="2HC  8MG"; $resn_exch{" HC3 8MG"}="3HC  8MG"; $resn_exch{" H41 8MO"}="1H4  8MO";
$resn_exch{" H42 8MO"}="2H4  8MO"; $resn_exch{" H43 8MO"}="3H4  8MO"; $resn_exch{"HAI1 8MR"}="1HAI 8MR";
$resn_exch{"HAI2 8MR"}="2HAI 8MR"; $resn_exch{"HAI3 8MR"}="3HAI 8MR"; $resn_exch{"HAL1 8MR"}="1HAL 8MR";
$resn_exch{"HAL2 8MR"}="2HAL 8MR"; $resn_exch{"H191 8NH"}="1H19 8NH"; $resn_exch{"H192 8NH"}="2H19 8NH";
$resn_exch{"H221 8NH"}="1H22 8NH"; $resn_exch{"H222 8NH"}="2H22 8NH"; $resn_exch{"H301 8NH"}="1H30 8NH";
$resn_exch{"H302 8NH"}="2H30 8NH"; $resn_exch{"H303 8NH"}="3H30 8NH"; $resn_exch{"HOP3 8OG"}="3HOP 8OG";
$resn_exch{"HOP2 8OG"}="2HOP 8OG"; $resn_exch{" H5' 8OG"}="1H5* 8OG"; $resn_exch{"H5'' 8OG"}="2H5* 8OG";
$resn_exch{" H4' 8OG"}=" H4* 8OG"; $resn_exch{" H3' 8OG"}=" H3* 8OG"; $resn_exch{"HO3' 8OG"}="H3T  8OG";
$resn_exch{" H2' 8OG"}="1H2* 8OG"; $resn_exch{"H2'' 8OG"}="2H2* 8OG"; $resn_exch{" H1' 8OG"}=" H1* 8OG";
$resn_exch{" H21 8OG"}="1H2  8OG"; $resn_exch{" H22 8OG"}="2H2  8OG"; $resn_exch{" H71 8PC"}="1H7  8PC";
$resn_exch{" H72 8PC"}="2H7  8PC"; $resn_exch{"HAA1 8PG"}="1HAA 8PG"; $resn_exch{"HAA2 8PG"}="2HAA 8PG";
$resn_exch{"HAA3 8PG"}="3HAA 8PG"; $resn_exch{"HAJ1 8PG"}="1HAJ 8PG"; $resn_exch{"HAJ2 8PG"}="2HAJ 8PG";
$resn_exch{"HAK1 8PG"}="1HAK 8PG"; $resn_exch{"HAK2 8PG"}="2HAK 8PG"; $resn_exch{"HAO1 8PG"}="1HAO 8PG";
$resn_exch{"HAO2 8PG"}="2HAO 8PG"; $resn_exch{"HAR1 8PG"}="1HAR 8PG"; $resn_exch{"HAR2 8PG"}="2HAR 8PG";
$resn_exch{"HAS1 8PG"}="1HAS 8PG"; $resn_exch{"HAS2 8PG"}="2HAS 8PG"; $resn_exch{"HAP1 8PG"}="1HAP 8PG";
$resn_exch{"HAP2 8PG"}="2HAP 8PG"; $resn_exch{"HAM1 8PG"}="1HAM 8PG"; $resn_exch{"HAM2 8PG"}="2HAM 8PG";
$resn_exch{"HAL1 8PG"}="1HAL 8PG"; $resn_exch{"HAL2 8PG"}="2HAL 8PG"; $resn_exch{"HAN1 8PG"}="1HAN 8PG";
$resn_exch{"HAN2 8PG"}="2HAN 8PG"; $resn_exch{"HAQ1 8PG"}="1HAQ 8PG"; $resn_exch{"HAQ2 8PG"}="2HAQ 8PG";
$resn_exch{" H71 8PP"}="1H7  8PP"; $resn_exch{" H72 8PP"}="2H7  8PP"; $resn_exch{"H101 8PP"}="1H10 8PP";
$resn_exch{"H102 8PP"}="2H10 8PP"; $resn_exch{"H103 8PP"}="3H10 8PP"; $resn_exch{"H111 8PP"}="1H11 8PP";
$resn_exch{"H112 8PP"}="2H11 8PP"; $resn_exch{"H121 8PP"}="1H12 8PP"; $resn_exch{"H122 8PP"}="2H12 8PP";
$resn_exch{"H151 8PP"}="1H15 8PP"; $resn_exch{"H152 8PP"}="2H15 8PP"; $resn_exch{"H153 8PP"}="3H15 8PP";
$resn_exch{"H161 8PP"}="1H16 8PP"; $resn_exch{"H162 8PP"}="2H16 8PP"; $resn_exch{"H171 8PP"}="1H17 8PP";
$resn_exch{"H172 8PP"}="2H17 8PP"; $resn_exch{"H201 8PP"}="1H20 8PP"; $resn_exch{"H202 8PP"}="2H20 8PP";
$resn_exch{"H203 8PP"}="3H20 8PP"; $resn_exch{"H211 8PP"}="1H21 8PP"; $resn_exch{"H212 8PP"}="2H21 8PP";
$resn_exch{"H221 8PP"}="1H22 8PP"; $resn_exch{"H222 8PP"}="2H22 8PP"; $resn_exch{"H251 8PP"}="1H25 8PP";
$resn_exch{"H252 8PP"}="2H25 8PP"; $resn_exch{"H253 8PP"}="3H25 8PP"; $resn_exch{"H261 8PP"}="1H26 8PP";
$resn_exch{"H262 8PP"}="2H26 8PP"; $resn_exch{"H271 8PP"}="1H27 8PP"; $resn_exch{"H272 8PP"}="2H27 8PP";
$resn_exch{"H301 8PP"}="1H30 8PP"; $resn_exch{"H302 8PP"}="2H30 8PP"; $resn_exch{"H303 8PP"}="3H30 8PP";
$resn_exch{"H311 8PP"}="1H31 8PP"; $resn_exch{"H312 8PP"}="2H31 8PP"; $resn_exch{"H321 8PP"}="1H32 8PP";
$resn_exch{"H322 8PP"}="2H32 8PP"; $resn_exch{"H351 8PP"}="1H35 8PP"; $resn_exch{"H352 8PP"}="2H35 8PP";
$resn_exch{"H353 8PP"}="3H35 8PP"; $resn_exch{"H361 8PP"}="1H36 8PP"; $resn_exch{"H362 8PP"}="2H36 8PP";
$resn_exch{"H371 8PP"}="1H37 8PP"; $resn_exch{"H372 8PP"}="2H37 8PP"; $resn_exch{"H401 8PP"}="1H40 8PP";
$resn_exch{"H402 8PP"}="2H40 8PP"; $resn_exch{"H411 8PP"}="1H41 8PP"; $resn_exch{"H412 8PP"}="2H41 8PP";
$resn_exch{"H413 8PP"}="3H41 8PP"; $resn_exch{"H421 8PP"}="1H42 8PP"; $resn_exch{"H422 8PP"}="2H42 8PP";
$resn_exch{"H451 8PP"}="1H45 8PP"; $resn_exch{"H452 8PP"}="2H45 8PP"; $resn_exch{"H453 8PP"}="3H45 8PP";
$resn_exch{"H461 8PP"}="1H46 8PP"; $resn_exch{"H462 8PP"}="2H46 8PP"; $resn_exch{"H463 8PP"}="3H46 8PP";
$resn_exch{"H141 8PS"}="1H14 8PS"; $resn_exch{"H142 8PS"}="2H14 8PS"; $resn_exch{"H151 8PS"}="1H15 8PS";
$resn_exch{"H152 8PS"}="2H15 8PS"; $resn_exch{"H161 8PS"}="1H16 8PS"; $resn_exch{"H162 8PS"}="2H16 8PS";
$resn_exch{"H171 8PS"}="1H17 8PS"; $resn_exch{"H172 8PS"}="2H17 8PS"; $resn_exch{"H181 8PS"}="1H18 8PS";
$resn_exch{"H182 8PS"}="2H18 8PS"; $resn_exch{"H191 8PS"}="1H19 8PS"; $resn_exch{"H192 8PS"}="2H19 8PS";
$resn_exch{"H201 8PS"}="1H20 8PS"; $resn_exch{"H202 8PS"}="2H20 8PS"; $resn_exch{"H211 8PS"}="1H21 8PS";
$resn_exch{"H212 8PS"}="2H21 8PS"; $resn_exch{"H213 8PS"}="3H21 8PS"; $resn_exch{"H321 901"}="1H32 901";
$resn_exch{"H322 901"}="2H32 901"; $resn_exch{"H331 901"}="1H33 901"; $resn_exch{"H332 901"}="2H33 901";
$resn_exch{"H333 901"}="3H33 901"; $resn_exch{" H71 901"}="1H7  901"; $resn_exch{" H72 901"}="2H7  901";
$resn_exch{"H231 901"}="1H23 901"; $resn_exch{"H232 901"}="2H23 901"; $resn_exch{"H233 901"}="3H23 901";
$resn_exch{"H121 901"}="1H12 901"; $resn_exch{"H122 901"}="2H12 901"; $resn_exch{"H131 901"}="1H13 901";
$resn_exch{"H132 901"}="2H13 901"; $resn_exch{"H151 901"}="1H15 901"; $resn_exch{"H152 901"}="2H15 901";
$resn_exch{"H141 901"}="1H14 901"; $resn_exch{"H142 901"}="2H14 901"; $resn_exch{"H181 901"}="1H18 901";
$resn_exch{"H182 901"}="2H18 901"; $resn_exch{"H341 901"}="1H34 901"; $resn_exch{"H342 901"}="2H34 901";
$resn_exch{"H351 901"}="1H35 901"; $resn_exch{"H352 901"}="2H35 901"; $resn_exch{"H353 901"}="3H35 901";
$resn_exch{"H211 902"}="1H21 902"; $resn_exch{"H212 902"}="2H21 902"; $resn_exch{"H301 902"}="1H30 902";
$resn_exch{"H302 902"}="2H30 902"; $resn_exch{"H311 902"}="1H31 902"; $resn_exch{"H312 902"}="2H31 902";
$resn_exch{"H331 902"}="1H33 902"; $resn_exch{"H332 902"}="2H33 902"; $resn_exch{"H341 902"}="1H34 902";
$resn_exch{"H342 902"}="2H34 902"; $resn_exch{"H471 902"}="1H47 902"; $resn_exch{"H472 902"}="2H47 902";
$resn_exch{" H71 903"}="1H7  903"; $resn_exch{" H72 903"}="2H7  903"; $resn_exch{"H151 903"}="1H15 903";
$resn_exch{"H152 903"}="2H15 903"; $resn_exch{"H153 903"}="3H15 903"; $resn_exch{"H201 903"}="1H20 903";
$resn_exch{"H202 903"}="2H20 903"; $resn_exch{"H211 903"}="1H21 903"; $resn_exch{"H212 903"}="2H21 903";
$resn_exch{"H221 903"}="1H22 903"; $resn_exch{"H222 903"}="2H22 903"; $resn_exch{"H241 903"}="1H24 903";
$resn_exch{"H242 903"}="2H24 903"; $resn_exch{"H251 903"}="1H25 903"; $resn_exch{"H252 903"}="2H25 903";
$resn_exch{"H811 903"}="1H81 903"; $resn_exch{"H812 903"}="2H81 903"; $resn_exch{"H813 903"}="3H81 903";
$resn_exch{"HN31 905"}="1HN3 905"; $resn_exch{"HN32 905"}="2HN3 905"; $resn_exch{"H151 905"}="1H15 905";
$resn_exch{"H152 905"}="2H15 905"; $resn_exch{"H161 905"}="1H16 905"; $resn_exch{"H162 905"}="2H16 905";
$resn_exch{"H163 905"}="3H16 905"; $resn_exch{"H191 905"}="1H19 905"; $resn_exch{"H192 905"}="2H19 905";
$resn_exch{"H201 905"}="1H20 905"; $resn_exch{"H202 905"}="2H20 905"; $resn_exch{"H203 905"}="3H20 905";
$resn_exch{"HN51 905"}="1HN5 905"; $resn_exch{"HN52 905"}="2HN5 905"; $resn_exch{"H211 910"}="1H21 910";
$resn_exch{"H212 910"}="2H21 910"; $resn_exch{"H301 910"}="1H30 910"; $resn_exch{"H302 910"}="2H30 910";
$resn_exch{"H111 915"}="1H11 915"; $resn_exch{"H112 915"}="2H11 915"; $resn_exch{"H113 915"}="3H11 915";
$resn_exch{"H291 915"}="1H29 915"; $resn_exch{"H292 915"}="2H29 915"; $resn_exch{"H301 915"}="1H30 915";
$resn_exch{"H302 915"}="2H30 915"; $resn_exch{"H321 915"}="1H32 915"; $resn_exch{"H322 915"}="2H32 915";
$resn_exch{"H331 915"}="1H33 915"; $resn_exch{"H332 915"}="2H33 915"; $resn_exch{"H331 922"}="1H33 922";
$resn_exch{"H332 922"}="2H33 922"; $resn_exch{"H281 922"}="1H28 922"; $resn_exch{"H282 922"}="2H28 922";
$resn_exch{"H283 922"}="3H28 922"; $resn_exch{"H271 922"}="1H27 922"; $resn_exch{"H272 922"}="2H27 922";
$resn_exch{"H273 922"}="3H27 922"; $resn_exch{"H111 922"}="1H11 922"; $resn_exch{"H112 922"}="2H11 922";
$resn_exch{" H11 936"}="1H1  936"; $resn_exch{"H121 936"}="1H12 936"; $resn_exch{"H122 936"}="2H12 936";
$resn_exch{"H123 936"}="3H12 936"; $resn_exch{"H191 936"}="1H19 936"; $resn_exch{"H192 936"}="2H19 936";
$resn_exch{"H251 936"}="1H25 936"; $resn_exch{"H261 936"}="1H26 936"; $resn_exch{"H281 936"}="1H28 936";
$resn_exch{"H291 936"}="1H29 936"; $resn_exch{"H371 936"}="1H37 936"; $resn_exch{"H372 936"}="2H37 936";
$resn_exch{"H373 936"}="3H37 936"; $resn_exch{"H451 936"}="1H45 936"; $resn_exch{"H452 936"}="2H45 936";
$resn_exch{"H453 936"}="3H45 936"; $resn_exch{"H5'1 93A"}="1H5* 93A"; $resn_exch{"H5'2 93A"}="2H5* 93A";
$resn_exch{"H121 941"}="1H12 941"; $resn_exch{"H122 941"}="2H12 941"; $resn_exch{"H131 941"}="1H13 941";
$resn_exch{"H132 941"}="2H13 941"; $resn_exch{"H141 941"}="1H14 941"; $resn_exch{"H142 941"}="2H14 941";
$resn_exch{"H151 941"}="1H15 941"; $resn_exch{"H152 941"}="2H15 941"; $resn_exch{" H71 941"}="1H7  941";
$resn_exch{" H72 941"}="2H7  941"; $resn_exch{"H391 941"}="1H39 941"; $resn_exch{"H392 941"}="2H39 941";
$resn_exch{"H393 941"}="3H39 941"; $resn_exch{"H421 941"}="1H42 941"; $resn_exch{"H422 941"}="2H42 941";
$resn_exch{"H423 941"}="3H42 941"; $resn_exch{"H431 941"}="1H43 941"; $resn_exch{"H432 941"}="2H43 941";
$resn_exch{"H433 941"}="3H43 941"; $resn_exch{"H441 941"}="1H44 941"; $resn_exch{"H442 941"}="2H44 941";
$resn_exch{"H443 941"}="3H44 941"; $resn_exch{"H141 95A"}="1H14 95A"; $resn_exch{"H142 95A"}="2H14 95A";
$resn_exch{"H481 95A"}="1H48 95A"; $resn_exch{"H482 95A"}="2H48 95A"; $resn_exch{"H483 95A"}="3H48 95A";
$resn_exch{"H471 95A"}="1H47 95A"; $resn_exch{"H472 95A"}="2H47 95A"; $resn_exch{"H491 95A"}="1H49 95A";
$resn_exch{"H492 95A"}="2H49 95A"; $resn_exch{"H493 95A"}="3H49 95A"; $resn_exch{"H381 95A"}="1H38 95A";
$resn_exch{"H382 95A"}="2H38 95A"; $resn_exch{"H391 95A"}="1H39 95A"; $resn_exch{"H392 95A"}="2H39 95A";
$resn_exch{"H401 95A"}="1H40 95A"; $resn_exch{"H402 95A"}="2H40 95A"; $resn_exch{"H403 95A"}="3H40 95A";
$resn_exch{"H201 95A"}="1H20 95A"; $resn_exch{"H202 95A"}="2H20 95A"; $resn_exch{"H231 95A"}="1H23 95A";
$resn_exch{"H232 95A"}="2H23 95A"; $resn_exch{"H141 961"}="1H14 961"; $resn_exch{"H142 961"}="2H14 961";
$resn_exch{"H151 961"}="1H15 961"; $resn_exch{"H152 961"}="2H15 961"; $resn_exch{"H181 961"}="1H18 961";
$resn_exch{"H182 961"}="2H18 961"; $resn_exch{"H183 961"}="3H18 961"; $resn_exch{"H191 961"}="1H19 961";
$resn_exch{"H192 961"}="2H19 961"; $resn_exch{"H193 961"}="3H19 961"; $resn_exch{"H201 961"}="1H20 961";
$resn_exch{"H202 961"}="2H20 961"; $resn_exch{"H203 961"}="3H20 961"; $resn_exch{"H211 961"}="1H21 961";
$resn_exch{"H212 961"}="2H21 961"; $resn_exch{"H213 961"}="3H21 961"; $resn_exch{"HO22 961"}="2HO2 961";
$resn_exch{"HO27 961"}="7HO2 961"; $resn_exch{"H11A 964"}="1H1  964"; $resn_exch{" H12 964"}="2H1  964";
$resn_exch{"H13A 964"}="3H1  964"; $resn_exch{"H181 964"}="1H18 964"; $resn_exch{"H182 964"}="2H18 964";
$resn_exch{"H191 964"}="1H19 964"; $resn_exch{"H192 964"}="2H19 964"; $resn_exch{"H161 964"}="1H16 964";
$resn_exch{"H162 964"}="2H16 964"; $resn_exch{"H151 964"}="1H15 964"; $resn_exch{"H152 964"}="2H15 964";
$resn_exch{"H101 964"}="1H10 964"; $resn_exch{"H102 964"}="2H10 964"; $resn_exch{" H91 964"}="1H9  964";
$resn_exch{" H92 964"}="2H9  964"; $resn_exch{" H51 964"}="1H5  964"; $resn_exch{" H52 964"}="2H5  964";
$resn_exch{" H61 964"}="1H6  964"; $resn_exch{" H62 964"}="2H6  964"; $resn_exch{"H341 965"}="1H34 965";
$resn_exch{"H342 965"}="2H34 965"; $resn_exch{"H261 965"}="1H26 965"; $resn_exch{"H262 965"}="2H26 965";
$resn_exch{"H251 965"}="1H25 965"; $resn_exch{"H252 965"}="2H25 965"; $resn_exch{"H171 965"}="1H17 965";
$resn_exch{"H172 965"}="2H17 965"; $resn_exch{"H161 965"}="1H16 965"; $resn_exch{"H162 965"}="2H16 965";
$resn_exch{"H081 965"}="1H08 965"; $resn_exch{"H082 965"}="2H08 965"; $resn_exch{"H111 965"}="1H11 965";
$resn_exch{"H112 965"}="2H11 965"; $resn_exch{"H121 974"}="1H12 974"; $resn_exch{"H122 974"}="2H12 974";
$resn_exch{"H321 974"}="1H32 974"; $resn_exch{"H322 974"}="2H32 974"; $resn_exch{"H341 974"}="1H34 974";
$resn_exch{"H342 974"}="2H34 974"; $resn_exch{"H351 974"}="1H35 974"; $resn_exch{"H352 974"}="2H35 974";
$resn_exch{"H361 974"}="1H36 974"; $resn_exch{"H362 974"}="2H36 974"; $resn_exch{"H301 974"}="1H30 974";
$resn_exch{"H302 974"}="2H30 974"; $resn_exch{"H303 974"}="3H30 974"; $resn_exch{"HN11 977"}="1HN1 977";
$resn_exch{"HN12 977"}="2HN1 977"; $resn_exch{"H121 979"}="1H12 979"; $resn_exch{"H122 979"}="2H12 979";
$resn_exch{"H131 979"}="1H13 979"; $resn_exch{"H132 979"}="2H13 979"; $resn_exch{" H21 979"}="1H2  979";
$resn_exch{" H22 979"}="2H2  979"; $resn_exch{"H111 979"}="1H11 979"; $resn_exch{"H112 979"}="2H11 979";
$resn_exch{"H161 982"}="1H16 982"; $resn_exch{"H162 982"}="2H16 982"; $resn_exch{"H471 983"}="1H47 983";
$resn_exch{"H472 983"}="2H47 983"; $resn_exch{"H351 983"}="1H35 983"; $resn_exch{"H352 983"}="2H35 983";
$resn_exch{"H331 983"}="1H33 983"; $resn_exch{"H332 983"}="2H33 983"; $resn_exch{"H301 983"}="1H30 983";
$resn_exch{"H302 983"}="2H30 983"; $resn_exch{"H303 983"}="3H30 983"; $resn_exch{"H291 983"}="1H29 983";
$resn_exch{"H292 983"}="2H29 983"; $resn_exch{"H293 983"}="3H29 983"; $resn_exch{"H151 983"}="1H15 983";
$resn_exch{"H152 983"}="2H15 983"; $resn_exch{" H91 983"}="1H9  983"; $resn_exch{" H92 983"}="2H9  983";
$resn_exch{"H101 983"}="1H10 983"; $resn_exch{"H102 983"}="2H10 983"; $resn_exch{"HC61 984"}="1HC6 984";
$resn_exch{"HC62 984"}="2HC6 984"; $resn_exch{"HC71 984"}="1HC7 984"; $resn_exch{"HC72 984"}="2HC7 984";
$resn_exch{"H101 984"}="1H10 984"; $resn_exch{"H102 984"}="2H10 984"; $resn_exch{"H103 984"}="3H10 984";
$resn_exch{"H171 984"}="1H17 984"; $resn_exch{"H172 984"}="2H17 984"; $resn_exch{"H181 984"}="1H18 984";
$resn_exch{"H182 984"}="2H18 984"; $resn_exch{"H201 984"}="1H20 984"; $resn_exch{"H202 984"}="2H20 984";
$resn_exch{"H211 984"}="1H21 984"; $resn_exch{"H212 984"}="2H21 984"; $resn_exch{"H311 984"}="1H31 984";
$resn_exch{"H312 984"}="2H31 984"; $resn_exch{"H313 984"}="3H31 984"; $resn_exch{"H561 984"}="1H56 984";
$resn_exch{"H562 984"}="2H56 984"; $resn_exch{"H571 984"}="1H57 984"; $resn_exch{"H572 984"}="2H57 984";
$resn_exch{"H361 989"}="1H36 989"; $resn_exch{"H362 989"}="2H36 989"; $resn_exch{"H621 989"}="1H62 989";
$resn_exch{"H622 989"}="2H62 989"; $resn_exch{"H623 989"}="3H62 989"; $resn_exch{"H451 989"}="1H45 989";
$resn_exch{"H452 989"}="2H45 989"; $resn_exch{"H461 989"}="1H46 989"; $resn_exch{"H462 989"}="2H46 989";
$resn_exch{"H491 989"}="1H49 989"; $resn_exch{"H492 989"}="2H49 989"; $resn_exch{"H521 989"}="1H52 989";
$resn_exch{"H522 989"}="2H52 989"; $resn_exch{"H551 989"}="1H55 989"; $resn_exch{"H552 989"}="2H55 989";
$resn_exch{"H553 989"}="3H55 989"; $resn_exch{"H351 993"}="1H35 993"; $resn_exch{"H352 993"}="2H35 993";
$resn_exch{"H353 993"}="3H35 993"; $resn_exch{"H281 993"}="1H28 993"; $resn_exch{"H282 993"}="2H28 993";
$resn_exch{"H181 993"}="1H18 993"; $resn_exch{"H182 993"}="2H18 993"; $resn_exch{"H183 993"}="3H18 993";
$resn_exch{"H181 994"}="1H18 994"; $resn_exch{"H182 994"}="2H18 994"; $resn_exch{"H183 994"}="3H18 994";
$resn_exch{"H231 994"}="1H23 994"; $resn_exch{"H232 994"}="2H23 994"; $resn_exch{"H233 994"}="3H23 994";
$resn_exch{"H281 994"}="1H28 994"; $resn_exch{"H282 994"}="2H28 994"; $resn_exch{"H283 994"}="3H28 994";
$resn_exch{" H61 995"}="1H6  995"; $resn_exch{" H62 995"}="2H6  995"; $resn_exch{" H71 995"}="1H7  995";
$resn_exch{" H72 995"}="2H7  995"; $resn_exch{"H111 995"}="1H11 995"; $resn_exch{"H112 995"}="2H11 995";
$resn_exch{"H113 995"}="3H11 995"; $resn_exch{"H121 995"}="1H12 995"; $resn_exch{"H122 995"}="2H12 995";
$resn_exch{"H123 995"}="3H12 995"; $resn_exch{"H141 995"}="1H14 995"; $resn_exch{"H142 995"}="2H14 995";
$resn_exch{"H151 995"}="1H15 995"; $resn_exch{"H152 995"}="2H15 995"; $resn_exch{"H171 995"}="1H17 995";
$resn_exch{"H172 995"}="2H17 995"; $resn_exch{"H173 995"}="3H17 995"; $resn_exch{"H181 995"}="1H18 995";
$resn_exch{"H182 995"}="2H18 995"; $resn_exch{"H183 995"}="3H18 995"; $resn_exch{"H191 995"}="1H19 995";
$resn_exch{"H192 995"}="2H19 995"; $resn_exch{"HN51 995"}="1HN5 995"; $resn_exch{"HN52 995"}="2HN5 995";
$resn_exch{"HN61 995"}="1HN6 995"; $resn_exch{"HN62 995"}="2HN6 995"; $resn_exch{"HD21 9AD"}="1HD2 9AD";
$resn_exch{"HD22 9AD"}="2HD2 9AD"; $resn_exch{"HD31 9AD"}="1HD3 9AD"; $resn_exch{"HD32 9AD"}="2HD3 9AD";
$resn_exch{"HD71 9AD"}="1HD7 9AD"; $resn_exch{"HD72 9AD"}="2HD7 9AD"; $resn_exch{"HD73 9AD"}="3HD7 9AD";
$resn_exch{"HD81 9AD"}="1HD8 9AD"; $resn_exch{"HD82 9AD"}="2HD8 9AD"; $resn_exch{"HD83 9AD"}="3HD8 9AD";
$resn_exch{" H91 9AM"}="1H9  9AM"; $resn_exch{" H92 9AM"}="2H9  9AM"; $resn_exch{"H111 9AM"}="1H11 9AM";
$resn_exch{"H112 9AM"}="2H11 9AM"; $resn_exch{"H113 9AM"}="3H11 9AM"; $resn_exch{"HOB1 9AM"}="1HOB 9AM";
$resn_exch{"HN91 9AM"}="1HN9 9AM"; $resn_exch{"HN92 9AM"}="2HN9 9AM"; $resn_exch{"HN11 9AP"}="1HN1 9AP";
$resn_exch{"HN12 9AP"}="2HN1 9AP"; $resn_exch{" H21 9AR"}="1H2  9AR"; $resn_exch{" H91 9AR"}="1H9  9AR";
$resn_exch{"H111 9AR"}="1H11 9AR"; $resn_exch{"H121 9AR"}="1H12 9AR"; $resn_exch{"H151 9AR"}="1H15 9AR";
$resn_exch{"H152 9AR"}="2H15 9AR"; $resn_exch{"H171 9AR"}="1H17 9AR"; $resn_exch{"H172 9AR"}="2H17 9AR";
$resn_exch{"H173 9AR"}="3H17 9AR"; $resn_exch{" H51 9AR"}="1H5  9AR"; $resn_exch{" H61 9AR"}="1H6  9AR";
$resn_exch{" H61 9CS"}="1H6  9CS"; $resn_exch{" H62 9CS"}="2H6  9CS"; $resn_exch{"H121 9CS"}="1H12 9CS";
$resn_exch{"H122 9CS"}="2H12 9CS"; $resn_exch{"H181 9CS"}="1H18 9CS"; $resn_exch{"H182 9CS"}="2H18 9CS";
$resn_exch{"HN11 9CS"}="1HN1 9CS"; $resn_exch{"HN12 9CS"}="2HN1 9CS"; $resn_exch{"HN21 9CS"}="1HN2 9CS";
$resn_exch{"HN22 9CS"}="2HN2 9CS"; $resn_exch{"HN31 9CS"}="1HN3 9CS"; $resn_exch{"HN32 9CS"}="2HN3 9CS";
$resn_exch{"HN41 9CS"}="1HN4 9CS"; $resn_exch{"HN42 9CS"}="2HN4 9CS"; $resn_exch{"HN61 9CS"}="1HN6 9CS";
$resn_exch{"HN62 9CS"}="2HN6 9CS"; $resn_exch{"HN61 9DA"}="1HN6 9DA"; $resn_exch{"HN62 9DA"}="2HN6 9DA";
$resn_exch{"HN21 9DG"}="1HN2 9DG"; $resn_exch{"HN22 9DG"}="2HN2 9DG"; $resn_exch{"H5'1 9DI"}="1H5' 9DI";
$resn_exch{"H5'2 9DI"}="2H5' 9DI"; $resn_exch{"HO5' 9DI"}="'HO5 9DI"; $resn_exch{"HO2' 9DI"}="'HO2 9DI";
$resn_exch{"HO3' 9DI"}="'HO3 9DI"; $resn_exch{" H91 9IN"}="1H9  9IN"; $resn_exch{" H92 9IN"}="2H9  9IN";
$resn_exch{" H93 9IN"}="3H9  9IN"; $resn_exch{"H101 9IN"}="1H10 9IN"; $resn_exch{"H102 9IN"}="2H10 9IN";
$resn_exch{"H103 9IN"}="3H10 9IN"; $resn_exch{"H131 9IN"}="1H13 9IN"; $resn_exch{"H132 9IN"}="2H13 9IN";
$resn_exch{"H133 9IN"}="3H13 9IN"; $resn_exch{"H141 9IN"}="1H14 9IN"; $resn_exch{"H142 9IN"}="2H14 9IN";
$resn_exch{"H143 9IN"}="3H14 9IN"; $resn_exch{"H171 9IN"}="1H17 9IN"; $resn_exch{"H172 9IN"}="2H17 9IN";
$resn_exch{"H191 9IN"}="1H19 9IN"; $resn_exch{"H192 9IN"}="2H19 9IN"; $resn_exch{"H193 9IN"}="3H19 9IN";
$resn_exch{"H201 9IN"}="1H20 9IN"; $resn_exch{"H202 9IN"}="2H20 9IN"; $resn_exch{"H203 9IN"}="3H20 9IN";
$resn_exch{"H281 9IN"}="1H28 9IN"; $resn_exch{"H282 9IN"}="2H28 9IN"; $resn_exch{"H301 9IN"}="1H30 9IN";
$resn_exch{"H302 9IN"}="2H30 9IN"; $resn_exch{"H311 9IN"}="1H31 9IN"; $resn_exch{"H312 9IN"}="2H31 9IN";
$resn_exch{"H261 9IN"}="1H26 9IN"; $resn_exch{"H262 9IN"}="2H26 9IN"; $resn_exch{"H271 9IN"}="1H27 9IN";
$resn_exch{"H272 9IN"}="2H27 9IN"; $resn_exch{"H273 9IN"}="3H27 9IN"; $resn_exch{"H11A 9IP"}="1H1  9IP";
$resn_exch{"H12A 9IP"}="2H1  9IP"; $resn_exch{" H13 9IP"}="3H1  9IP"; $resn_exch{"H141 9IP"}="1H14 9IP";
$resn_exch{"H142 9IP"}="2H14 9IP"; $resn_exch{"H221 9IP"}="1H22 9IP"; $resn_exch{"H222 9IP"}="2H22 9IP";
$resn_exch{" H11 9MG"}="1H1  9MG"; $resn_exch{" H12 9MG"}="2H1  9MG"; $resn_exch{" H13 9MG"}="3H1  9MG";
$resn_exch{"H101 9MG"}="1H10 9MG"; $resn_exch{"H102 9MG"}="2H10 9MG"; $resn_exch{"H6B1 9MR"}="1H6B 9MR";
$resn_exch{"H6B2 9MR"}="2H6B 9MR"; $resn_exch{"H5M1 9MR"}="1H5M 9MR"; $resn_exch{"H5M2 9MR"}="2H5M 9MR";
$resn_exch{"H1A1 9MR"}="1H1A 9MR"; $resn_exch{"H1A2 9MR"}="2H1A 9MR"; $resn_exch{"H6A1 9MR"}="1H6A 9MR";
$resn_exch{"H6A2 9MR"}="2H6A 9MR"; $resn_exch{" H21 9OH"}="1H2  9OH"; $resn_exch{" H22 9OH"}="2H2  9OH";
$resn_exch{" H31 9OH"}="1H3  9OH"; $resn_exch{" H32 9OH"}="2H3  9OH"; $resn_exch{" H41 9OH"}="1H4  9OH";
$resn_exch{" H42 9OH"}="2H4  9OH"; $resn_exch{" H51 9OH"}="1H5  9OH"; $resn_exch{" H52 9OH"}="2H5  9OH";
$resn_exch{" H61 9OH"}="1H6  9OH"; $resn_exch{" H62 9OH"}="2H6  9OH"; $resn_exch{" H71 9OH"}="1H7  9OH";
$resn_exch{" H72 9OH"}="2H7  9OH"; $resn_exch{" H81 9OH"}="1H8  9OH"; $resn_exch{" H82 9OH"}="2H8  9OH";
$resn_exch{" H91 9OH"}="1H9  9OH"; $resn_exch{"H101 9OH"}="1H10 9OH"; $resn_exch{"H111 9OH"}="1H11 9OH";
$resn_exch{"H121 9OH"}="1H12 9OH"; $resn_exch{"H131 9OH"}="1H13 9OH"; $resn_exch{"H141 9OH"}="1H14 9OH";
$resn_exch{"H142 9OH"}="2H14 9OH"; $resn_exch{"H151 9OH"}="1H15 9OH"; $resn_exch{"H152 9OH"}="2H15 9OH";
$resn_exch{"H161 9OH"}="1H16 9OH"; $resn_exch{"H162 9OH"}="2H16 9OH"; $resn_exch{"H171 9OH"}="1H17 9OH";
$resn_exch{"H172 9OH"}="2H17 9OH"; $resn_exch{"H181 9OH"}="1H18 9OH"; $resn_exch{"H182 9OH"}="2H18 9OH";
$resn_exch{"H183 9OH"}="3H18 9OH"; $resn_exch{"H201 9OH"}="1H20 9OH"; $resn_exch{"H211 9OH"}="1H21 9OH";
$resn_exch{" H81 9PP"}="1H8  9PP"; $resn_exch{"HN61 9PP"}="1HN6 9PP"; $resn_exch{"HN62 9PP"}="2HN6 9PP";
$resn_exch{"HN21 9PP"}="1HN2 9PP"; $resn_exch{"HN22 9PP"}="2HN2 9PP"; $resn_exch{"H101 9PP"}="1H10 9PP";
$resn_exch{"H102 9PP"}="2H10 9PP"; $resn_exch{"H111 9PP"}="1H11 9PP"; $resn_exch{"H121 9PP"}="1H12 9PP";
$resn_exch{"H122 9PP"}="2H12 9PP"; $resn_exch{"H123 9PP"}="3H12 9PP"; $resn_exch{"H141 9PP"}="1H14 9PP";
$resn_exch{"H142 9PP"}="2H14 9PP"; $resn_exch{"HOP3   A"}="3HOP   A"; $resn_exch{"HOP2   A"}="2HOP   A";
$resn_exch{" H5'   A"}="1H5*   A"; $resn_exch{"H5''   A"}="2H5*   A"; $resn_exch{" H4'   A"}=" H4*   A";
$resn_exch{" H3'   A"}=" H3*   A"; $resn_exch{"HO3'   A"}="H3T    A"; $resn_exch{" H2'   A"}=" H2*   A";
$resn_exch{"HO2'   A"}="2HO*   A"; $resn_exch{" H1'   A"}=" H1*   A"; $resn_exch{" H61   A"}="1H6    A";
$resn_exch{" H62   A"}="2H6    A"; $resn_exch{" HN1 A0A"}="1HN  A0A"; $resn_exch{" HN2 A0A"}="2HN  A0A";
$resn_exch{"HBC1 A0A"}="1HBC A0A"; $resn_exch{"HBC2 A0A"}="2HBC A0A"; $resn_exch{"HOB2 A12"}="2HOB A12";
$resn_exch{"HOA2 A12"}="2HOA A12"; $resn_exch{"H3A1 A12"}="1H3A A12"; $resn_exch{"H3A2 A12"}="2H3A A12";
$resn_exch{"H5'1 A12"}="1H5* A12"; $resn_exch{"H5'2 A12"}="2H5* A12"; $resn_exch{" H4' A12"}=" H4* A12";
$resn_exch{" H3' A12"}=" H3* A12"; $resn_exch{"HO3' A12"}="*HO3 A12"; $resn_exch{" H2' A12"}=" H2* A12";
$resn_exch{"HO2' A12"}="*HO2 A12"; $resn_exch{" H1' A12"}=" H1* A12"; $resn_exch{"HN61 A12"}="1HN6 A12";
$resn_exch{"HN62 A12"}="2HN6 A12"; $resn_exch{"H3C1 A14"}="1H3C A14"; $resn_exch{"H3C2 A14"}="2H3C A14";
$resn_exch{"H141 A14"}="1H14 A14"; $resn_exch{"H142 A14"}="2H14 A14"; $resn_exch{"H4C1 A14"}="1H4C A14";
$resn_exch{"H4C2 A14"}="2H4C A14"; $resn_exch{"H7C1 A14"}="1H7C A14"; $resn_exch{"H7C2 A14"}="2H7C A14";
$resn_exch{"H331 A14"}="1H33 A14"; $resn_exch{"H332 A14"}="2H33 A14"; $resn_exch{"H333 A14"}="3H33 A14";
$resn_exch{"H371 A14"}="1H37 A14"; $resn_exch{"H372 A14"}="2H37 A14"; $resn_exch{"H373 A14"}="3H37 A14";
$resn_exch{"HAL1 A18"}="1HAL A18"; $resn_exch{"HAL2 A18"}="2HAL A18"; $resn_exch{"H321 A1A"}="1H32 A1A";
$resn_exch{"H322 A1A"}="2H32 A1A"; $resn_exch{"H7C1 A1A"}="1H7C A1A"; $resn_exch{"H7C2 A1A"}="2H7C A1A";
$resn_exch{"H7C3 A1A"}="3H7C A1A"; $resn_exch{"H4C1 A1A"}="1H4C A1A"; $resn_exch{"H4C2 A1A"}="2H4C A1A";
$resn_exch{"H221 A1A"}="1H22 A1A"; $resn_exch{"H222 A1A"}="2H22 A1A"; $resn_exch{"H581 A1A"}="1H58 A1A";
$resn_exch{"H582 A1A"}="2H58 A1A"; $resn_exch{"H431 A1A"}="1H43 A1A"; $resn_exch{"H432 A1A"}="2H43 A1A";
$resn_exch{"H433 A1A"}="3H43 A1A"; $resn_exch{"H441 A1A"}="1H44 A1A"; $resn_exch{"H442 A1A"}="2H44 A1A";
$resn_exch{"H443 A1A"}="3H44 A1A"; $resn_exch{" H11 A1E"}="1H1  A1E"; $resn_exch{"H12A A1E"}="2H1  A1E";
$resn_exch{" H21 A1E"}="1H2  A1E"; $resn_exch{" H22 A1E"}="2H2  A1E"; $resn_exch{" H61 A1E"}="1H6  A1E";
$resn_exch{" H62 A1E"}="2H6  A1E"; $resn_exch{"H131 A1E"}="1H13 A1E"; $resn_exch{"H132 A1E"}="2H13 A1E";
$resn_exch{"H141 A1E"}="1H14 A1E"; $resn_exch{"H142 A1E"}="2H14 A1E"; $resn_exch{"H151 A1E"}="1H15 A1E";
$resn_exch{"H152 A1E"}="2H15 A1E"; $resn_exch{"H161 A1E"}="1H16 A1E"; $resn_exch{"H162 A1E"}="2H16 A1E";
$resn_exch{"H171 A1E"}="1H17 A1E"; $resn_exch{"H172 A1E"}="2H17 A1E"; $resn_exch{"H181 A1E"}="1H18 A1E";
$resn_exch{"H182 A1E"}="2H18 A1E"; $resn_exch{"H191 A1E"}="1H19 A1E"; $resn_exch{"H192 A1E"}="2H19 A1E";
$resn_exch{"H201 A1E"}="1H20 A1E"; $resn_exch{"H202 A1E"}="2H20 A1E"; $resn_exch{"H211 A1E"}="1H21 A1E";
$resn_exch{"H212 A1E"}="2H21 A1E"; $resn_exch{"H221 A1E"}="1H22 A1E"; $resn_exch{"H222 A1E"}="2H22 A1E";
$resn_exch{"H341 A1E"}="1H34 A1E"; $resn_exch{"H342 A1E"}="2H34 A1E"; $resn_exch{"H351 A1E"}="1H35 A1E";
$resn_exch{"H352 A1E"}="2H35 A1E"; $resn_exch{"H361 A1E"}="1H36 A1E"; $resn_exch{"H362 A1E"}="2H36 A1E";
$resn_exch{"H371 A1E"}="1H37 A1E"; $resn_exch{"H372 A1E"}="2H37 A1E"; $resn_exch{"H2N1 A1P"}="1H2N A1P";
$resn_exch{"H2N2 A1P"}="2H2N A1P"; $resn_exch{"H2'1 A1P"}="1H2* A1P"; $resn_exch{"H2'2 A1P"}="2H2* A1P";
$resn_exch{"H5'1 A1P"}="1H5* A1P"; $resn_exch{"H5'2 A1P"}="2H5* A1P"; $resn_exch{" H4' A1P"}=" H4* A1P";
$resn_exch{" H1' A1P"}=" H1* A1P"; $resn_exch{" H3' A1P"}=" H3* A1P"; $resn_exch{"HN61 A1R"}="1HN6 A1R";
$resn_exch{"HN62 A1R"}="2HN6 A1R"; $resn_exch{" H1' A1R"}=" H1* A1R"; $resn_exch{" H2' A1R"}=" H2* A1R";
$resn_exch{"HO2' A1R"}="HO2* A1R"; $resn_exch{" H3' A1R"}=" H3* A1R"; $resn_exch{"HO3' A1R"}="HO3* A1R";
$resn_exch{" H4' A1R"}=" H4* A1R"; $resn_exch{"H5'1 A1R"}="1H5* A1R"; $resn_exch{"H5'2 A1R"}="2H5* A1R";
$resn_exch{"H5N1 A1R"}="1H5N A1R"; $resn_exch{"H5N2 A1R"}="2H5N A1R"; $resn_exch{"H1N1 A1R"}="1H1N A1R";
$resn_exch{"H1N2 A1R"}="2H1N A1R"; $resn_exch{"H381 A1T"}="1H38 A1T"; $resn_exch{"H382 A1T"}="2H38 A1T";
$resn_exch{"H401 A1T"}="1H40 A1T"; $resn_exch{"H402 A1T"}="2H40 A1T"; $resn_exch{"H431 A1T"}="1H43 A1T";
$resn_exch{"H432 A1T"}="2H43 A1T"; $resn_exch{"H461 A1T"}="1H46 A1T"; $resn_exch{"H462 A1T"}="2H46 A1T";
$resn_exch{"H481 A1T"}="1H48 A1T"; $resn_exch{"H482 A1T"}="2H48 A1T"; $resn_exch{"H483 A1T"}="3H48 A1T";
$resn_exch{"H281 A1T"}="1H28 A1T"; $resn_exch{"H282 A1T"}="2H28 A1T"; $resn_exch{"H321 A1T"}="1H32 A1T";
$resn_exch{"H322 A1T"}="2H32 A1T"; $resn_exch{"H371 A1T"}="1H37 A1T"; $resn_exch{"H372 A1T"}="2H37 A1T";
$resn_exch{"H311 A1T"}="1H31 A1T"; $resn_exch{"H312 A1T"}="2H31 A1T"; $resn_exch{"H271 A1T"}="1H27 A1T";
$resn_exch{"H272 A1T"}="2H27 A1T"; $resn_exch{"H211 A1T"}="1H21 A1T"; $resn_exch{"H212 A1T"}="2H21 A1T";
$resn_exch{" H61 A1T"}="1H6  A1T"; $resn_exch{" H62 A1T"}="2H6  A1T"; $resn_exch{"H101 A1T"}="1H10 A1T";
$resn_exch{"H102 A1T"}="2H10 A1T"; $resn_exch{"H131 A1T"}="1H13 A1T"; $resn_exch{"H132 A1T"}="2H13 A1T";
$resn_exch{" H91 A1T"}="1H9  A1T"; $resn_exch{" H92 A1T"}="2H9  A1T"; $resn_exch{" H51 A1T"}="1H5  A1T";
$resn_exch{" H52 A1T"}="2H5  A1T"; $resn_exch{" HD1 A22"}="1H*A A22"; $resn_exch{" HD2 A22"}="2H*A A22";
$resn_exch{" H4D A22"}="H4*A A22"; $resn_exch{" H2D A22"}="H2*A A22"; $resn_exch{" H1D A22"}="H1*A A22";
$resn_exch{"H6A1 A22"}="1H6A A22"; $resn_exch{"H6A2 A22"}="2H6A A22"; $resn_exch{" HE1 A22"}="1H*B A22";
$resn_exch{" HE2 A22"}="2H*B A22"; $resn_exch{" H4E A22"}="H4*B A22"; $resn_exch{" H1E A22"}="H1*B A22";
$resn_exch{"H6B1 A22"}="1H6B A22"; $resn_exch{"H6B2 A22"}="2H6B A22"; $resn_exch{"HOC2 A23"}="2HOC A23";
$resn_exch{"HOP2 A23"}="2HOP A23"; $resn_exch{"HOP3 A23"}="3HOP A23"; $resn_exch{" H5' A23"}="1H5* A23";
$resn_exch{"H5'' A23"}="2H5* A23"; $resn_exch{" H4' A23"}=" H4* A23"; $resn_exch{" H3' A23"}=" H3* A23";
$resn_exch{" H2' A23"}=" H2* A23"; $resn_exch{" H1' A23"}=" H1* A23"; $resn_exch{"HN61 A23"}="1HN6 A23";
$resn_exch{"HN62 A23"}="2HN6 A23"; $resn_exch{" H11 A24"}="1H1  A24"; $resn_exch{" H21 A24"}="1H2  A24";
$resn_exch{"HO21 A24"}="1HO2 A24"; $resn_exch{" H31 A24"}="1H3  A24"; $resn_exch{"HO31 A24"}="1HO3 A24";
$resn_exch{" H41 A24"}="1H4  A24"; $resn_exch{"HO41 A24"}="1HO4 A24"; $resn_exch{" H51 A24"}="1H5  A24";
$resn_exch{" H61 A24"}="1H6  A24"; $resn_exch{" H62 A24"}="2H6  A24"; $resn_exch{" HO1 A24"}="1HO  A24";
$resn_exch{"H2'1 A24"}="1H2' A24"; $resn_exch{"H4'1 A24"}="1H4' A24"; $resn_exch{"H6'1 A24"}="1H6' A24";
$resn_exch{"HN21 A24"}="1HN2 A24"; $resn_exch{"H2A1 A24"}="1H2* A24"; $resn_exch{"H2'2 A24"}="2H2* A24";
$resn_exch{"H3'1 A24"}="1H3* A24"; $resn_exch{"H3'2 A24"}="2H3* A24"; $resn_exch{"H5'1 A24"}="1H5* A24";
$resn_exch{"H5'2 A24"}="2H5* A24"; $resn_exch{"H6A1 A24"}="1H6* A24"; $resn_exch{"H6'2 A24"}="2H6* A24";
$resn_exch{"H7'1 A24"}="1H7* A24"; $resn_exch{"H7'2 A24"}="2H7* A24"; $resn_exch{"H8'1 A24"}="1H8* A24";
$resn_exch{"H8'2 A24"}="2H8* A24"; $resn_exch{"H211 A25"}="1H21 A25"; $resn_exch{"H212 A25"}="2H21 A25";
$resn_exch{"H221 A25"}="1H22 A25"; $resn_exch{"H222 A25"}="2H22 A25"; $resn_exch{"H241 A25"}="1H24 A25";
$resn_exch{"H242 A25"}="2H24 A25"; $resn_exch{"H231 A25"}="1H23 A25"; $resn_exch{"H232 A25"}="2H23 A25";
$resn_exch{" H91 A25"}="1H9  A25"; $resn_exch{" H92 A25"}="2H9  A25"; $resn_exch{" H11 A26"}="1H1  A26";
$resn_exch{" H21 A26"}="1H2  A26"; $resn_exch{" H41 A26"}="1H4  A26"; $resn_exch{" H51 A26"}="1H5  A26";
$resn_exch{"HN11 A26"}="1HN1 A26"; $resn_exch{"H211 A26"}="1H21 A26"; $resn_exch{"H212 A26"}="2H21 A26";
$resn_exch{"H213 A26"}="3H21 A26"; $resn_exch{" HD1 A2D"}="1H*A A2D"; $resn_exch{" HD2 A2D"}="2H*A A2D";
$resn_exch{" H4D A2D"}="H4*A A2D"; $resn_exch{" H1D A2D"}="H1*A A2D"; $resn_exch{"H6A1 A2D"}="1H6A A2D";
$resn_exch{"H6A2 A2D"}="2H6A A2D"; $resn_exch{" HE1 A2D"}="1H*B A2D"; $resn_exch{" HE2 A2D"}="2H*B A2D";
$resn_exch{" H4E A2D"}="H4*B A2D"; $resn_exch{" H1E A2D"}="H1*B A2D"; $resn_exch{"H6B1 A2D"}="1H6B A2D";
$resn_exch{"H6B2 A2D"}="2H6B A2D"; $resn_exch{" H61 A2E"}="1H6  A2E"; $resn_exch{" H62 A2E"}="2H6  A2E";
$resn_exch{" H21 A2E"}="1H2  A2E"; $resn_exch{" H22 A2E"}="2H2  A2E"; $resn_exch{" H11 A2E"}="1H1  A2E";
$resn_exch{"H12A A2E"}="2H1  A2E"; $resn_exch{"H131 A2E"}="1H13 A2E"; $resn_exch{"H132 A2E"}="2H13 A2E";
$resn_exch{"H141 A2E"}="1H14 A2E"; $resn_exch{"H142 A2E"}="2H14 A2E"; $resn_exch{"H151 A2E"}="1H15 A2E";
$resn_exch{"H152 A2E"}="2H15 A2E"; $resn_exch{"H161 A2E"}="1H16 A2E"; $resn_exch{"H162 A2E"}="2H16 A2E";
$resn_exch{"H171 A2E"}="1H17 A2E"; $resn_exch{"H172 A2E"}="2H17 A2E"; $resn_exch{"H181 A2E"}="1H18 A2E";
$resn_exch{"H182 A2E"}="2H18 A2E"; $resn_exch{"H191 A2E"}="1H19 A2E"; $resn_exch{"H192 A2E"}="2H19 A2E";
$resn_exch{"H201 A2E"}="1H20 A2E"; $resn_exch{"H202 A2E"}="2H20 A2E"; $resn_exch{"H211 A2E"}="1H21 A2E";
$resn_exch{"H212 A2E"}="2H21 A2E"; $resn_exch{"H221 A2E"}="1H22 A2E"; $resn_exch{"H222 A2E"}="2H22 A2E";
$resn_exch{"H301 A2E"}="1H30 A2E"; $resn_exch{"H302 A2E"}="2H30 A2E"; $resn_exch{"H331 A2E"}="1H33 A2E";
$resn_exch{"H332 A2E"}="2H33 A2E"; $resn_exch{"H321 A2E"}="1H32 A2E"; $resn_exch{"H322 A2E"}="2H32 A2E";
$resn_exch{"H311 A2E"}="1H31 A2E"; $resn_exch{"H312 A2E"}="2H31 A2E"; $resn_exch{" H61 A2G"}="1H6  A2G";
$resn_exch{" H62 A2G"}="2H6  A2G"; $resn_exch{" H81 A2G"}="1H8  A2G"; $resn_exch{" H82 A2G"}="2H8  A2G";
$resn_exch{" H83 A2G"}="3H8  A2G"; $resn_exch{"HO2' A2L"}="2HO* A2L"; $resn_exch{" H2' A2L"}=" H2* A2L";
$resn_exch{" H1' A2L"}=" H1* A2L"; $resn_exch{" H4' A2L"}=" H4* A2L"; $resn_exch{" H5' A2L"}="1H5* A2L";
$resn_exch{"H5'' A2L"}="2H5* A2L"; $resn_exch{"HOP3 A2L"}="3HOP A2L"; $resn_exch{"HOP2 A2L"}="2HOP A2L";
$resn_exch{" H3' A2L"}=" H3* A2L"; $resn_exch{"HA'1 A2L"}="1HA* A2L"; $resn_exch{"HA'2 A2L"}="2HA* A2L";
$resn_exch{"HB'1 A2L"}="1HB* A2L"; $resn_exch{"HB'2 A2L"}="2HB* A2L"; $resn_exch{"HD'1 A2L"}="1HD* A2L";
$resn_exch{"HD'2 A2L"}="2HD* A2L"; $resn_exch{"HD'3 A2L"}="3HD* A2L"; $resn_exch{" H61 A2L"}="1H6  A2L";
$resn_exch{" H62 A2L"}="2H6  A2L"; $resn_exch{"HOP3 A2M"}="3HOP A2M"; $resn_exch{"HOP2 A2M"}="2HOP A2M";
$resn_exch{" H5' A2M"}="1H5* A2M"; $resn_exch{"H5'' A2M"}="2H5* A2M"; $resn_exch{" H4' A2M"}=" H4* A2M";
$resn_exch{" H3' A2M"}=" H3* A2M"; $resn_exch{"HO3' A2M"}="H3T  A2M"; $resn_exch{" H2' A2M"}=" H2* A2M";
$resn_exch{" H1' A2M"}=" H1* A2M"; $resn_exch{"HM'1 A2M"}="1HM* A2M"; $resn_exch{"HM'2 A2M"}="2HM* A2M";
$resn_exch{"HM'3 A2M"}="3HM* A2M"; $resn_exch{" H61 A2M"}="1H6  A2M"; $resn_exch{" H62 A2M"}="2H6  A2M";
$resn_exch{"HOP2 A2P"}="2HOP A2P"; $resn_exch{"HOP3 A2P"}="3HOP A2P"; $resn_exch{"HOP5 A2P"}="5HOP A2P";
$resn_exch{"HOP6 A2P"}="6HOP A2P"; $resn_exch{"H5'1 A2P"}="1H5* A2P"; $resn_exch{"H5'2 A2P"}="2H5* A2P";
$resn_exch{" H4' A2P"}=" H4* A2P"; $resn_exch{" H3' A2P"}=" H3* A2P"; $resn_exch{"HO3' A2P"}="*HO3 A2P";
$resn_exch{" H2' A2P"}=" H2* A2P"; $resn_exch{" H1' A2P"}=" H1* A2P"; $resn_exch{"HN61 A2P"}="1HN6 A2P";
$resn_exch{"HN62 A2P"}="2HN6 A2P"; $resn_exch{" H2' A2R"}=" H2* A2R"; $resn_exch{" H1' A2R"}=" H1* A2R";
$resn_exch{"HN61 A2R"}="1HN6 A2R"; $resn_exch{"HN62 A2R"}="2HN6 A2R"; $resn_exch{" H3' A2R"}=" H3* A2R";
$resn_exch{"HO3' A2R"}="HO3* A2R"; $resn_exch{" H4' A2R"}=" H4* A2R"; $resn_exch{"H5'1 A2R"}="1H5* A2R";
$resn_exch{"H5'2 A2R"}="2H5* A2R"; $resn_exch{"H5'3 A2R"}="3H5* A2R"; $resn_exch{"H5'4 A2R"}="4H5* A2R";
$resn_exch{"H14' A2R"}="H14* A2R"; $resn_exch{"H13' A2R"}="H13* A2R"; $resn_exch{"H16' A2R"}="H16* A2R";
$resn_exch{"H12' A2R"}="H12* A2R"; $resn_exch{"H15' A2R"}="H15* A2R"; $resn_exch{"H11' A2R"}="H11* A2R";
$resn_exch{"H17' A2R"}="H17* A2R"; $resn_exch{"H351 A2T"}="1H35 A2T"; $resn_exch{"H352 A2T"}="2H35 A2T";
$resn_exch{"H371 A2T"}="1H37 A2T"; $resn_exch{"H372 A2T"}="2H37 A2T"; $resn_exch{"H391 A2T"}="1H39 A2T";
$resn_exch{"H392 A2T"}="2H39 A2T"; $resn_exch{"H421 A2T"}="1H42 A2T"; $resn_exch{"H422 A2T"}="2H42 A2T";
$resn_exch{"H431 A2T"}="1H43 A2T"; $resn_exch{"H432 A2T"}="2H43 A2T"; $resn_exch{"H433 A2T"}="3H43 A2T";
$resn_exch{"H251 A2T"}="1H25 A2T"; $resn_exch{"H252 A2T"}="2H25 A2T"; $resn_exch{"H291 A2T"}="1H29 A2T";
$resn_exch{"H292 A2T"}="2H29 A2T"; $resn_exch{"H341 A2T"}="1H34 A2T"; $resn_exch{"H342 A2T"}="2H34 A2T";
$resn_exch{"H361 A2T"}="1H36 A2T"; $resn_exch{"H362 A2T"}="2H36 A2T"; $resn_exch{"H411 A2T"}="1H41 A2T";
$resn_exch{"H412 A2T"}="2H41 A2T"; $resn_exch{"H413 A2T"}="3H41 A2T"; $resn_exch{"H401 A2T"}="1H40 A2T";
$resn_exch{"H402 A2T"}="2H40 A2T"; $resn_exch{"H403 A2T"}="3H40 A2T"; $resn_exch{"H281 A2T"}="1H28 A2T";
$resn_exch{"H282 A2T"}="2H28 A2T"; $resn_exch{"H241 A2T"}="1H24 A2T"; $resn_exch{"H242 A2T"}="2H24 A2T";
$resn_exch{"H181 A2T"}="1H18 A2T"; $resn_exch{"H182 A2T"}="2H18 A2T"; $resn_exch{"H451 A2T"}="1H45 A2T";
$resn_exch{"H452 A2T"}="2H45 A2T"; $resn_exch{"H453 A2T"}="3H45 A2T"; $resn_exch{" H11 A32"}="1H1  A32";
$resn_exch{" H21 A32"}="1H2  A32"; $resn_exch{"HO21 A32"}="1HO2 A32"; $resn_exch{" H31 A32"}="1H3  A32";
$resn_exch{"HO31 A32"}="1HO3 A32"; $resn_exch{" H41 A32"}="1H4  A32"; $resn_exch{"HO41 A32"}="1HO4 A32";
$resn_exch{" H51 A32"}="1H5  A32"; $resn_exch{" H61 A32"}="1H6  A32"; $resn_exch{" H62 A32"}="2H6  A32";
$resn_exch{" HO1 A32"}="1HO  A32"; $resn_exch{"H2'1 A32"}="1H2' A32"; $resn_exch{"H4'1 A32"}="1H4' A32";
$resn_exch{"H6'1 A32"}="1H6' A32"; $resn_exch{"HN21 A32"}="1HN2 A32"; $resn_exch{"H2A1 A32"}="1H2* A32";
$resn_exch{"H2'2 A32"}="2H2* A32"; $resn_exch{"H3'1 A32"}="1H3* A32"; $resn_exch{"H3'2 A32"}="2H3* A32";
$resn_exch{"H5'1 A32"}="1H5* A32"; $resn_exch{"H5'2 A32"}="2H5* A32"; $resn_exch{"H6A1 A32"}="1H6* A32";
$resn_exch{"H6'2 A32"}="2H6* A32"; $resn_exch{"H7'1 A32"}="1H7* A32"; $resn_exch{"H7'2 A32"}="2H7* A32";
$resn_exch{"H8'1 A32"}="1H8* A32"; $resn_exch{"H8'2 A32"}="2H8* A32"; $resn_exch{"H9'1 A32"}="1H9* A32";
$resn_exch{"H9'2 A32"}="2H9* A32"; $resn_exch{"H111 A34"}="1H11 A34"; $resn_exch{"H112 A34"}="2H11 A34";
$resn_exch{"H113 A34"}="3H11 A34"; $resn_exch{"H2'1 A34"}="1H2* A34"; $resn_exch{"H2'2 A34"}="2H2* A34";
$resn_exch{"H5'1 A34"}="1H5* A34"; $resn_exch{"H5'2 A34"}="2H5* A34"; $resn_exch{" H4' A34"}=" H4* A34";
$resn_exch{" H1' A34"}=" H1* A34"; $resn_exch{" H3' A34"}=" H3* A34"; $resn_exch{"H6N1 A35"}="1H6N A35";
$resn_exch{"H6N2 A35"}="2H6N A35"; $resn_exch{"H5'1 A35"}="1H5* A35"; $resn_exch{"H5'2 A35"}="2H5* A35";
$resn_exch{" H4' A35"}=" H4* A35"; $resn_exch{" H1' A35"}=" H1* A35"; $resn_exch{"H2'1 A35"}="1H2* A35";
$resn_exch{"H2'2 A35"}="2H2* A35"; $resn_exch{" H3' A35"}=" H3* A35"; $resn_exch{"HH21 A35"}="1HH2 A35";
$resn_exch{"HH22 A35"}="2HH2 A35"; $resn_exch{"HH23 A35"}="3HH2 A35"; $resn_exch{"H5'1 A38"}="1H5* A38";
$resn_exch{"H5'2 A38"}="2H5* A38"; $resn_exch{" H4' A38"}=" H4* A38"; $resn_exch{" H3' A38"}=" H3* A38";
$resn_exch{"H2'1 A38"}="1H2* A38"; $resn_exch{"H2'2 A38"}="2H2* A38"; $resn_exch{" H1' A38"}=" H1* A38";
$resn_exch{"H6N1 A38"}="1H6N A38"; $resn_exch{"H6N2 A38"}="2H6N A38"; $resn_exch{"H5'1 A39"}="1H5* A39";
$resn_exch{"H5'2 A39"}="2H5* A39"; $resn_exch{" H4' A39"}=" H4* A39"; $resn_exch{" H3' A39"}=" H3* A39";
$resn_exch{" H2' A39"}=" H2* A39"; $resn_exch{" H1' A39"}=" H1* A39"; $resn_exch{"H6N1 A39"}="1H6N A39";
$resn_exch{"H6N2 A39"}="2H6N A39"; $resn_exch{"HH31 A39"}="1HH3 A39"; $resn_exch{"HH32 A39"}="2HH3 A39";
$resn_exch{"HH33 A39"}="3HH3 A39"; $resn_exch{"HN61 A3A"}="1HN6 A3A"; $resn_exch{"HN62 A3A"}="2HN6 A3A";
$resn_exch{" H5' A3A"}="1H5* A3A"; $resn_exch{"H5'' A3A"}="2H5* A3A"; $resn_exch{" H4' A3A"}=" H4* A3A";
$resn_exch{" H3' A3A"}=" H3* A3A"; $resn_exch{" H2' A3A"}="1H2* A3A"; $resn_exch{"H2'' A3A"}="2H2* A3A";
$resn_exch{" H1' A3A"}=" H1* A3A"; $resn_exch{"HAA1 A3B"}="1HAA A3B"; $resn_exch{"HAA2 A3B"}="2HAA A3B";
$resn_exch{"HAB1 A3B"}="1HAB A3B"; $resn_exch{"HAB2 A3B"}="2HAB A3B"; $resn_exch{"HAB3 A3B"}="3HAB A3B";
$resn_exch{"HOA2 A3D"}="2HOA A3D"; $resn_exch{"H51A A3D"}="AH51 A3D"; $resn_exch{"H52A A3D"}="AH52 A3D";
$resn_exch{" H4B A3D"}="AH4* A3D"; $resn_exch{" H3B A3D"}="AH3* A3D"; $resn_exch{"HO3A A3D"}="AHO3 A3D";
$resn_exch{" H2B A3D"}="AH2* A3D"; $resn_exch{"HO2A A3D"}="AHO2 A3D"; $resn_exch{" H1B A3D"}="AH1* A3D";
$resn_exch{" H8A A3D"}=" AH8 A3D"; $resn_exch{"H61A A3D"}="AH61 A3D"; $resn_exch{"H62A A3D"}="AH62 A3D";
$resn_exch{" H2A A3D"}=" AH2 A3D"; $resn_exch{"H51N A3D"}="NH51 A3D"; $resn_exch{"H52N A3D"}="NH52 A3D";
$resn_exch{" H4D A3D"}="NH4* A3D"; $resn_exch{" H3D A3D"}="NH3* A3D"; $resn_exch{"HO3N A3D"}="NHO3 A3D";
$resn_exch{" H2D A3D"}="NH2* A3D"; $resn_exch{"HO2N A3D"}="NHO2 A3D"; $resn_exch{" H1D A3D"}="NH1* A3D";
$resn_exch{" H2N A3D"}=" NH2 A3D"; $resn_exch{"H81N A3D"}="NH81 A3D"; $resn_exch{"H82N A3D"}="NH82 A3D";
$resn_exch{"H83N A3D"}="NH83 A3D"; $resn_exch{" H4N A3D"}=" NH4 A3D"; $resn_exch{" H5N A3D"}=" NH5 A3D";
$resn_exch{" H6N A3D"}=" NH6 A3D"; $resn_exch{"HC11 A3M"}="1HC1 A3M"; $resn_exch{"HC12 A3M"}="2HC1 A3M";
$resn_exch{"HC21 A3M"}="1HC2 A3M"; $resn_exch{"HC22 A3M"}="2HC2 A3M"; $resn_exch{"HC41 A3M"}="1HC4 A3M";
$resn_exch{"HC42 A3M"}="2HC4 A3M"; $resn_exch{"HC51 A3M"}="1HC5 A3M"; $resn_exch{"HC52 A3M"}="2HC5 A3M";
$resn_exch{"H171 A3M"}="1H17 A3M"; $resn_exch{"H172 A3M"}="2H17 A3M"; $resn_exch{"H173 A3M"}="3H17 A3M";
$resn_exch{"H191 A3M"}="1H19 A3M"; $resn_exch{"H192 A3M"}="2H19 A3M"; $resn_exch{"H261 A3M"}="1H26 A3M";
$resn_exch{"H262 A3M"}="2H26 A3M"; $resn_exch{"H263 A3M"}="3H26 A3M"; $resn_exch{"HOP2 A3P"}="2HOP A3P";
$resn_exch{"HOP3 A3P"}="3HOP A3P"; $resn_exch{"HOP5 A3P"}="5HOP A3P"; $resn_exch{"HOP6 A3P"}="6HOP A3P";
$resn_exch{"H5'1 A3P"}="1H5* A3P"; $resn_exch{"H5'2 A3P"}="2H5* A3P"; $resn_exch{" H4' A3P"}=" H4* A3P";
$resn_exch{" H3' A3P"}=" H3* A3P"; $resn_exch{" H2' A3P"}=" H2* A3P"; $resn_exch{"HO2' A3P"}="*HO2 A3P";
$resn_exch{" H1' A3P"}=" H1* A3P"; $resn_exch{"HN61 A3P"}="1HN6 A3P"; $resn_exch{"HN62 A3P"}="2HN6 A3P";
$resn_exch{"H5'1 A3S"}="1H5' A3S"; $resn_exch{"H5'2 A3S"}="2H5' A3S"; $resn_exch{"HN61 A3S"}="1HN6 A3S";
$resn_exch{"HN62 A3S"}="2HN6 A3S"; $resn_exch{" HN1 A3S"}="1HN  A3S"; $resn_exch{" HN2 A3S"}="2HN  A3S";
$resn_exch{" HB1 A3S"}="1HB  A3S"; $resn_exch{" HB2 A3S"}="2HB  A3S"; $resn_exch{"H6N1 A40"}="1H6N A40";
$resn_exch{"H6N2 A40"}="2H6N A40"; $resn_exch{"H5'1 A40"}="1H5* A40"; $resn_exch{"H5'2 A40"}="2H5* A40";
$resn_exch{" H4' A40"}=" H4* A40"; $resn_exch{" H1' A40"}=" H1* A40"; $resn_exch{"H2'1 A40"}="1H2* A40";
$resn_exch{"H2'2 A40"}="2H2* A40"; $resn_exch{" H3' A40"}=" H3* A40"; $resn_exch{"HH21 A40"}="1HH2 A40";
$resn_exch{"HH22 A40"}="2HH2 A40"; $resn_exch{"HH23 A40"}="3HH2 A40"; $resn_exch{"H201 A41"}="1H20 A41";
$resn_exch{"H202 A41"}="2H20 A41"; $resn_exch{"H203 A41"}="3H20 A41"; $resn_exch{"H201 A42"}="1H20 A42";
$resn_exch{"H202 A42"}="2H20 A42"; $resn_exch{"H203 A42"}="3H20 A42"; $resn_exch{"H221 A42"}="1H22 A42";
$resn_exch{"H222 A42"}="2H22 A42"; $resn_exch{"H223 A42"}="3H22 A42"; $resn_exch{"H5'1 A43"}="1H5* A43";
$resn_exch{"H5'2 A43"}="2H5* A43"; $resn_exch{" H4' A43"}=" H4* A43"; $resn_exch{" H3' A43"}=" H3* A43";
$resn_exch{"H2'1 A43"}="1H2* A43"; $resn_exch{"H2'2 A43"}="2H2* A43"; $resn_exch{" H1' A43"}=" H1* A43";
$resn_exch{"H6N1 A43"}="1H6N A43"; $resn_exch{"H6N2 A43"}="2H6N A43"; $resn_exch{" HN1 A43"}="1HN  A43";
$resn_exch{" HN2 A43"}="2HN  A43"; $resn_exch{"H5'1 A44"}="1H5* A44"; $resn_exch{"H5'2 A44"}="2H5* A44";
$resn_exch{" H4' A44"}=" H4* A44"; $resn_exch{" H3' A44"}=" H3* A44"; $resn_exch{" H2' A44"}=" H2* A44";
$resn_exch{" H1' A44"}=" H1* A44"; $resn_exch{"H6N1 A44"}="1H6N A44"; $resn_exch{"H6N2 A44"}="2H6N A44";
$resn_exch{"HA'1 A44"}="1HA* A44"; $resn_exch{"HA'2 A44"}="2HA* A44"; $resn_exch{"H131 A44"}="1H13 A44";
$resn_exch{"H132 A44"}="2H13 A44"; $resn_exch{"H133 A44"}="3H13 A44"; $resn_exch{"H151 A44"}="1H15 A44";
$resn_exch{"H152 A44"}="2H15 A44"; $resn_exch{"HN11 A45"}="1HN1 A45"; $resn_exch{"HN12 A45"}="2HN1 A45";
$resn_exch{"H5'1 A47"}="1H5* A47"; $resn_exch{"H5'2 A47"}="2H5* A47"; $resn_exch{" H4' A47"}=" H4* A47";
$resn_exch{" H3' A47"}=" H3* A47"; $resn_exch{"H2'1 A47"}="1H2* A47"; $resn_exch{"H2'2 A47"}="2H2* A47";
$resn_exch{" H1' A47"}=" H1* A47"; $resn_exch{"H1C1 A47"}="1H1C A47"; $resn_exch{"H1C2 A47"}="2H1C A47";
$resn_exch{"H1C3 A47"}="3H1C A47"; $resn_exch{"HAD1 A48"}="1HAD A48"; $resn_exch{"HAD2 A48"}="2HAD A48";
$resn_exch{"HAD3 A48"}="3HAD A48"; $resn_exch{"HAC1 A48"}="1HAC A48"; $resn_exch{"HAC2 A48"}="2HAC A48";
$resn_exch{"HAC3 A48"}="3HAC A48"; $resn_exch{"HAJ1 A48"}="1HAJ A48"; $resn_exch{"HAJ2 A48"}="2HAJ A48";
$resn_exch{"HAJ3 A48"}="3HAJ A48"; $resn_exch{"HAW1 A48"}="1HAW A48"; $resn_exch{"HAW2 A48"}="2HAW A48";
$resn_exch{"HAG1 A48"}="1HAG A48"; $resn_exch{"HAG2 A48"}="2HAG A48"; $resn_exch{"HAG3 A48"}="3HAG A48";
$resn_exch{"HAS1 A48"}="1HAS A48"; $resn_exch{"HAS2 A48"}="2HAS A48"; $resn_exch{"HAS3 A48"}="3HAS A48";
$resn_exch{"HBB1 A48"}="1HBB A48"; $resn_exch{"HBB2 A48"}="2HBB A48"; $resn_exch{"HBB3 A48"}="3HBB A48";
$resn_exch{"HD21 A4C"}="1HD2 A4C"; $resn_exch{"HD22 A4C"}="2HD2 A4C"; $resn_exch{"HD31 A4C"}="1HD3 A4C";
$resn_exch{"HD32 A4C"}="2HD3 A4C"; $resn_exch{"HD81 A4C"}="1HD8 A4C"; $resn_exch{"HD82 A4C"}="2HD8 A4C";
$resn_exch{"HD83 A4C"}="3HD8 A4C"; $resn_exch{"HD71 A4C"}="1HD7 A4C"; $resn_exch{"HD72 A4C"}="2HD7 A4C";
$resn_exch{"HD73 A4C"}="3HD7 A4C"; $resn_exch{"HX11 A4C"}="1HX1 A4C"; $resn_exch{"HX12 A4C"}="2HX1 A4C";
$resn_exch{"HX21 A4C"}="1HX2 A4C"; $resn_exch{"HX22 A4C"}="2HX2 A4C"; $resn_exch{"HX31 A4C"}="1HX3 A4C";
$resn_exch{"HX32 A4C"}="2HX3 A4C"; $resn_exch{"HD51 A4C"}="1HD5 A4C"; $resn_exch{"HD52 A4C"}="2HD5 A4C";
$resn_exch{"HD61 A4C"}="1HD6 A4C"; $resn_exch{"HD62 A4C"}="2HD6 A4C"; $resn_exch{"HD01 A4C"}="1HD0 A4C";
$resn_exch{"HD02 A4C"}="2HD0 A4C"; $resn_exch{"HD03 A4C"}="3HD0 A4C"; $resn_exch{"HD91 A4C"}="1HD9 A4C";
$resn_exch{"HD92 A4C"}="2HD9 A4C"; $resn_exch{"HD93 A4C"}="3HD9 A4C"; $resn_exch{"HX41 A4C"}="1HX4 A4C";
$resn_exch{"HX42 A4C"}="2HX4 A4C"; $resn_exch{"HX51 A4C"}="1HX5 A4C"; $resn_exch{"HX52 A4C"}="2HX5 A4C";
$resn_exch{"HX61 A4C"}="1HX6 A4C"; $resn_exch{"HX62 A4C"}="2HX6 A4C"; $resn_exch{"H281 A4L"}="1H28 A4L";
$resn_exch{"H282 A4L"}="2H28 A4L"; $resn_exch{"H283 A4L"}="3H28 A4L"; $resn_exch{"H221 A4L"}="1H22 A4L";
$resn_exch{"H222 A4L"}="2H22 A4L"; $resn_exch{"H211 A4L"}="1H21 A4L"; $resn_exch{"H212 A4L"}="2H21 A4L";
$resn_exch{"H201 A4L"}="1H20 A4L"; $resn_exch{"H202 A4L"}="2H20 A4L"; $resn_exch{"H191 A4L"}="1H19 A4L";
$resn_exch{"H192 A4L"}="2H19 A4L"; $resn_exch{"H171 A4L"}="1H17 A4L"; $resn_exch{"H172 A4L"}="2H17 A4L";
$resn_exch{"H173 A4L"}="3H17 A4L"; $resn_exch{"HN11 A4L"}="1HN1 A4L"; $resn_exch{"HN12 A4L"}="2HN1 A4L";
$resn_exch{" H71 A4L"}="1H7  A4L"; $resn_exch{" H72 A4L"}="2H7  A4L"; $resn_exch{"H151 A4L"}="1H15 A4L";
$resn_exch{"H152 A4L"}="2H15 A4L"; $resn_exch{"H161 A4L"}="1H16 A4L"; $resn_exch{"H162 A4L"}="2H16 A4L";
$resn_exch{"H181 A4L"}="1H18 A4L"; $resn_exch{"H182 A4L"}="2H18 A4L"; $resn_exch{"H341 A4L"}="1H34 A4L";
$resn_exch{"H342 A4L"}="2H34 A4L"; $resn_exch{"H351 A4L"}="1H35 A4L"; $resn_exch{"H352 A4L"}="2H35 A4L";
$resn_exch{"H361 A4L"}="1H36 A4L"; $resn_exch{"H362 A4L"}="2H36 A4L"; $resn_exch{"H371 A4L"}="1H37 A4L";
$resn_exch{"H372 A4L"}="2H37 A4L"; $resn_exch{"HN61 A4L"}="1HN6 A4L"; $resn_exch{"HN62 A4L"}="2HN6 A4L";
$resn_exch{"H381 A4L"}="1H38 A4L"; $resn_exch{"H382 A4L"}="2H38 A4L"; $resn_exch{"H391 A4L"}="1H39 A4L";
$resn_exch{"H392 A4L"}="2H39 A4L"; $resn_exch{"H401 A4L"}="1H40 A4L"; $resn_exch{"H402 A4L"}="2H40 A4L";
$resn_exch{"H411 A4L"}="1H41 A4L"; $resn_exch{"H412 A4L"}="2H41 A4L"; $resn_exch{"HN81 A4L"}="1HN8 A4L";
$resn_exch{"HN82 A4L"}="2HN8 A4L"; $resn_exch{"H421 A4L"}="1H42 A4L"; $resn_exch{"H422 A4L"}="2H42 A4L";
$resn_exch{"H431 A4L"}="1H43 A4L"; $resn_exch{"H432 A4L"}="2H43 A4L"; $resn_exch{"H441 A4L"}="1H44 A4L";
$resn_exch{"H442 A4L"}="2H44 A4L"; $resn_exch{"H451 A4L"}="1H45 A4L"; $resn_exch{"H452 A4L"}="2H45 A4L";
$resn_exch{"HN91 A4L"}="1HN9 A4L"; $resn_exch{"HN92 A4L"}="2HN9 A4L"; $resn_exch{"H6R1 A4P"}="1H6R A4P";
$resn_exch{"H6R2 A4P"}="2H6R A4P"; $resn_exch{"H5'1 A4P"}="1H5* A4P"; $resn_exch{"H5'2 A4P"}="2H5* A4P";
$resn_exch{" H4' A4P"}=" H4* A4P"; $resn_exch{" H3' A4P"}=" H3* A4P"; $resn_exch{"HO'3 A4P"}="3HO* A4P";
$resn_exch{" H2' A4P"}=" H2* A4P"; $resn_exch{"HO'2 A4P"}="2HO* A4P"; $resn_exch{"HOA1 A4P"}="1HOA A4P";
$resn_exch{"HOB1 A4P"}="1HOB A4P"; $resn_exch{"HOG1 A4P"}="1HOG A4P"; $resn_exch{"HOD1 A4P"}="1HOD A4P";
$resn_exch{"HN21 A4P"}="1HN2 A4P"; $resn_exch{"HN22 A4P"}="2HN2 A4P"; $resn_exch{"H6A1 A4P"}="1H6A A4P";
$resn_exch{"H6A2 A4P"}="2H6A A4P"; $resn_exch{" H71 A4P"}="1H7  A4P"; $resn_exch{" H72 A4P"}="2H7  A4P";
$resn_exch{"H241 A53"}="1H24 A53"; $resn_exch{"H242 A53"}="2H24 A53"; $resn_exch{"H261 A53"}="1H26 A53";
$resn_exch{"H262 A53"}="2H26 A53"; $resn_exch{"H271 A53"}="1H27 A53"; $resn_exch{"H272 A53"}="2H27 A53";
$resn_exch{"H11A A58"}="1H1  A58"; $resn_exch{" H12 A58"}="2H1  A58"; $resn_exch{" H13 A58"}="3H1  A58";
$resn_exch{" H61 A58"}="1H6  A58"; $resn_exch{" H62 A58"}="2H6  A58"; $resn_exch{" H71 A58"}="1H7  A58";
$resn_exch{" H72 A58"}="2H7  A58"; $resn_exch{" H31 A58"}="1H3  A58"; $resn_exch{" H32 A58"}="2H3  A58";
$resn_exch{" H41 A58"}="1H4  A58"; $resn_exch{" H42 A58"}="2H4  A58"; $resn_exch{" H91 A58"}="1H9  A58";
$resn_exch{" H92 A58"}="2H9  A58"; $resn_exch{"H161 A58"}="1H16 A58"; $resn_exch{"H162 A58"}="2H16 A58";
$resn_exch{" HB1 A5A"}="1HB  A5A"; $resn_exch{" HB2 A5A"}="2HB  A5A"; $resn_exch{" HB3 A5A"}="3HB  A5A";
$resn_exch{"H5'1 A5A"}="1H5* A5A"; $resn_exch{"H5'2 A5A"}="2H5* A5A"; $resn_exch{" H4' A5A"}=" H4* A5A";
$resn_exch{" H3' A5A"}=" H3* A5A"; $resn_exch{" H2' A5A"}=" H2* A5A"; $resn_exch{"HO'2 A5A"}="2HO* A5A";
$resn_exch{" H1' A5A"}=" H1* A5A"; $resn_exch{" H61 A5A"}="1H6  A5A"; $resn_exch{" H62 A5A"}="2H6  A5A";
$resn_exch{"HOP2 A5M"}="2HOP A5M"; $resn_exch{" H5' A5M"}="1H5* A5M"; $resn_exch{"H5'' A5M"}="2H5* A5M";
$resn_exch{" H4' A5M"}=" H4* A5M"; $resn_exch{" H1' A5M"}=" H1* A5M"; $resn_exch{" H41 A5M"}="1H4  A5M";
$resn_exch{" H42 A5M"}="2H4  A5M"; $resn_exch{" H2' A5M"}=" H2* A5M"; $resn_exch{"H'1N A5M"}="1NH* A5M";
$resn_exch{"H'2N A5M"}="2NH* A5M"; $resn_exch{" H3' A5M"}=" H3* A5M"; $resn_exch{"HO3' A5M"}="H3T  A5M";
$resn_exch{"HOP3 A5M"}="3HOP A5M"; $resn_exch{"HOP2 A5P"}="2HOP A5P"; $resn_exch{"HOP3 A5P"}="3HOP A5P";
$resn_exch{"H8'1 A66"}="1H8' A66"; $resn_exch{"H8'2 A66"}="2H8' A66"; $resn_exch{"H5'1 A66"}="1H5' A66";
$resn_exch{"H3'1 A66"}="1H3' A66"; $resn_exch{"H3'2 A66"}="2H3' A66"; $resn_exch{"H2'1 A66"}="1H2' A66";
$resn_exch{"H2'2 A66"}="2H2' A66"; $resn_exch{"H1'1 A66"}="1H1' A66"; $resn_exch{"H1'2 A66"}="2H1' A66";
$resn_exch{"HN61 A66"}="1HN6 A66"; $resn_exch{"HN62 A66"}="2HN6 A66"; $resn_exch{" HB1 A66"}="1HB  A66";
$resn_exch{" HB2 A66"}="2HB  A66"; $resn_exch{" HG1 A66"}="1HG  A66"; $resn_exch{" HG2 A66"}="2HG  A66";
$resn_exch{" HD1 A66"}="1HD  A66"; $resn_exch{" HD2 A66"}="2HD  A66"; $resn_exch{" HE1 A66"}="1HE  A66";
$resn_exch{" HE2 A66"}="2HE  A66"; $resn_exch{" HZ1 A66"}="1HZ  A66"; $resn_exch{" HZ2 A66"}="2HZ  A66";
$resn_exch{" H11 A70"}="1H1  A70"; $resn_exch{" H12 A70"}="2H1  A70"; $resn_exch{"H13A A70"}="3H1  A70";
$resn_exch{" H21 A70"}="1H2  A70"; $resn_exch{" H22 A70"}="2H2  A70"; $resn_exch{" H31 A70"}="1H3  A70";
$resn_exch{"H32A A70"}="2H3  A70"; $resn_exch{" H71 A70"}="1H7  A70"; $resn_exch{" H72 A70"}="2H7  A70";
$resn_exch{" H91 A70"}="1H9  A70"; $resn_exch{" H92 A70"}="2H9  A70"; $resn_exch{"H101 A70"}="1H10 A70";
$resn_exch{"H102 A70"}="2H10 A70"; $resn_exch{"H111 A70"}="1H11 A70"; $resn_exch{"H112 A70"}="2H11 A70";
$resn_exch{"H121 A70"}="1H12 A70"; $resn_exch{"H122 A70"}="2H12 A70"; $resn_exch{"H123 A70"}="3H12 A70";
$resn_exch{"H141 A70"}="1H14 A70"; $resn_exch{"H142 A70"}="2H14 A70"; $resn_exch{"H143 A70"}="3H14 A70";
$resn_exch{"H151 A70"}="1H15 A70"; $resn_exch{"H152 A70"}="2H15 A70"; $resn_exch{"H161 A70"}="1H16 A70";
$resn_exch{"H162 A70"}="2H16 A70"; $resn_exch{"H163 A70"}="3H16 A70"; $resn_exch{"H191 A70"}="1H19 A70";
$resn_exch{"H192 A70"}="2H19 A70"; $resn_exch{"H211 A70"}="1H21 A70"; $resn_exch{"H212 A70"}="2H21 A70";
$resn_exch{"H221 A70"}="1H22 A70"; $resn_exch{"H222 A70"}="2H22 A70"; $resn_exch{"H311 A70"}="1H31 A70";
$resn_exch{"H312 A70"}="2H31 A70"; $resn_exch{"H341 A70"}="1H34 A70"; $resn_exch{"H342 A70"}="2H34 A70";
$resn_exch{"H181 A70"}="1H18 A70"; $resn_exch{"H182 A70"}="2H18 A70"; $resn_exch{"H231 A70"}="1H23 A70";
$resn_exch{"H232 A70"}="2H23 A70"; $resn_exch{"H251 A70"}="1H25 A70"; $resn_exch{"H252 A70"}="2H25 A70";
$resn_exch{"H253 A70"}="3H25 A70"; $resn_exch{"H261 A70"}="1H26 A70"; $resn_exch{"H262 A70"}="2H26 A70";
$resn_exch{"H281 A70"}="1H28 A70"; $resn_exch{"H282 A70"}="2H28 A70"; $resn_exch{"H291 A70"}="1H29 A70";
$resn_exch{"H292 A70"}="2H29 A70"; $resn_exch{" H51 A70"}="1H5  A70"; $resn_exch{" H52 A70"}="2H5  A70";
$resn_exch{" H61 A70"}="1H6  A70"; $resn_exch{" H62 A70"}="2H6  A70"; $resn_exch{"H41A A70"}="1H4  A70";
$resn_exch{" H42 A70"}="2H4  A70"; $resn_exch{"HN11 A71"}="1HN1 A71"; $resn_exch{"HN12 A71"}="2HN1 A71";
$resn_exch{" H21 A71"}="1H2  A71"; $resn_exch{" H22 A71"}="2H2  A71"; $resn_exch{" H31 A71"}="1H3  A71";
$resn_exch{" H32 A71"}="2H3  A71"; $resn_exch{" H41 A71"}="1H4  A71"; $resn_exch{" H42 A71"}="2H4  A71";
$resn_exch{" H51 A71"}="1H5  A71"; $resn_exch{" H52 A71"}="2H5  A71"; $resn_exch{" H61 A71"}="1H6  A71";
$resn_exch{" H62 A71"}="2H6  A71"; $resn_exch{" H71 A71"}="1H7  A71"; $resn_exch{" H72 A71"}="2H7  A71";
$resn_exch{"HN81 A71"}="1HN8 A71"; $resn_exch{"HN82 A71"}="2HN8 A71"; $resn_exch{"HN91 A71"}="1HN9 A71";
$resn_exch{"HN92 A71"}="2HN9 A71"; $resn_exch{"HN93 A71"}="3HN9 A71"; $resn_exch{"H101 A71"}="1H10 A71";
$resn_exch{"H102 A71"}="2H10 A71"; $resn_exch{"H103 A71"}="3H10 A71"; $resn_exch{"H111 A71"}="1H11 A71";
$resn_exch{"H112 A71"}="2H11 A71"; $resn_exch{"H113 A71"}="3H11 A71"; $resn_exch{"H121 A72"}="1H12 A72";
$resn_exch{"H122 A72"}="2H12 A72"; $resn_exch{"H123 A72"}="3H12 A72"; $resn_exch{"H131 A72"}="1H13 A72";
$resn_exch{"H132 A72"}="2H13 A72"; $resn_exch{"H141 A72"}="1H14 A72"; $resn_exch{"H142 A72"}="2H14 A72";
$resn_exch{"H151 A72"}="1H15 A72"; $resn_exch{"H152 A72"}="2H15 A72"; $resn_exch{"H161 A72"}="1H16 A72";
$resn_exch{"H162 A72"}="2H16 A72"; $resn_exch{"H171 A72"}="1H17 A72"; $resn_exch{"H172 A72"}="2H17 A72";
$resn_exch{"H181 A72"}="1H18 A72"; $resn_exch{"H182 A72"}="2H18 A72"; $resn_exch{"H191 A72"}="1H19 A72";
$resn_exch{"H192 A72"}="2H19 A72"; $resn_exch{"H193 A72"}="3H19 A72"; $resn_exch{"H201 A72"}="1H20 A72";
$resn_exch{"H202 A72"}="2H20 A72"; $resn_exch{"H203 A72"}="3H20 A72"; $resn_exch{"H211 A72"}="1H21 A72";
$resn_exch{"H212 A72"}="2H21 A72"; $resn_exch{"H221 A72"}="1H22 A72"; $resn_exch{"H222 A72"}="2H22 A72";
$resn_exch{"H231 A72"}="1H23 A72"; $resn_exch{"H232 A72"}="2H23 A72"; $resn_exch{"H241 A72"}="1H24 A72";
$resn_exch{"H242 A72"}="2H24 A72"; $resn_exch{"H251 A72"}="1H25 A72"; $resn_exch{"H252 A72"}="2H25 A72";
$resn_exch{"H261 A72"}="1H26 A72"; $resn_exch{"H262 A72"}="2H26 A72"; $resn_exch{"H271 A72"}="1H27 A72";
$resn_exch{"H272 A72"}="2H27 A72"; $resn_exch{"H273 A72"}="3H27 A72"; $resn_exch{"H131 A74"}="1H13 A74";
$resn_exch{"H132 A74"}="2H13 A74"; $resn_exch{"H133 A74"}="3H13 A74"; $resn_exch{"H191 A74"}="1H19 A74";
$resn_exch{"H192 A74"}="2H19 A74"; $resn_exch{"H193 A74"}="3H19 A74"; $resn_exch{" H11 A75"}="1H1  A75";
$resn_exch{"H12A A75"}="2H1  A75"; $resn_exch{"H21A A75"}="1H2  A75"; $resn_exch{"H22A A75"}="2H2  A75";
$resn_exch{" H51 A75"}="1H5  A75"; $resn_exch{" H52 A75"}="2H5  A75"; $resn_exch{" H61 A75"}="1H6  A75";
$resn_exch{" H62 A75"}="2H6  A75"; $resn_exch{"H201 A80"}="1H20 A80"; $resn_exch{"H202 A80"}="2H20 A80";
$resn_exch{"H203 A80"}="3H20 A80"; $resn_exch{"H211 A80"}="1H21 A80"; $resn_exch{"H212 A80"}="2H21 A80";
$resn_exch{"H213 A80"}="3H21 A80"; $resn_exch{"H221 A80"}="1H22 A80"; $resn_exch{"H222 A80"}="2H22 A80";
$resn_exch{"H223 A80"}="3H22 A80"; $resn_exch{"H231 A80"}="1H23 A80"; $resn_exch{"H232 A80"}="2H23 A80";
$resn_exch{"H233 A80"}="3H23 A80"; $resn_exch{" H11 A83"}="1H1  A83"; $resn_exch{"H12A A83"}="2H1  A83";
$resn_exch{" H61 A83"}="1H6  A83"; $resn_exch{" H62 A83"}="2H6  A83"; $resn_exch{" H71 A83"}="1H7  A83";
$resn_exch{" H72 A83"}="2H7  A83"; $resn_exch{" H91 A83"}="1H9  A83"; $resn_exch{" H92 A83"}="2H9  A83";
$resn_exch{" H93 A83"}="3H9  A83"; $resn_exch{"H101 A83"}="1H10 A83"; $resn_exch{"H102 A83"}="2H10 A83";
$resn_exch{"H103 A83"}="3H10 A83"; $resn_exch{"H171 A83"}="1H17 A83"; $resn_exch{"H172 A83"}="2H17 A83";
$resn_exch{"H173 A83"}="3H17 A83"; $resn_exch{"H181 A83"}="1H18 A83"; $resn_exch{"H182 A83"}="2H18 A83";
$resn_exch{"H311 A83"}="1H31 A83"; $resn_exch{"H312 A83"}="2H31 A83"; $resn_exch{"H491 A83"}="1H49 A83";
$resn_exch{"H492 A83"}="2H49 A83"; $resn_exch{"H493 A83"}="3H49 A83"; $resn_exch{"H501 A83"}="1H50 A83";
$resn_exch{"H502 A83"}="2H50 A83"; $resn_exch{"H503 A83"}="3H50 A83"; $resn_exch{"H511 A83"}="1H51 A83";
$resn_exch{"H512 A83"}="2H51 A83"; $resn_exch{"H513 A83"}="3H51 A83"; $resn_exch{" H11 A84"}="1H1  A84";
$resn_exch{" H12 A84"}="2H1  A84"; $resn_exch{" H21 A84"}="1H2  A84"; $resn_exch{"H22A A84"}="2H2  A84";
$resn_exch{" H51 A84"}="1H5  A84"; $resn_exch{" H52 A84"}="2H5  A84"; $resn_exch{" H61 A84"}="1H6  A84";
$resn_exch{" H62 A84"}="2H6  A84"; $resn_exch{"H271 A84"}="1H27 A84"; $resn_exch{"H272 A84"}="2H27 A84";
$resn_exch{"H291 A84"}="1H29 A84"; $resn_exch{"H292 A84"}="2H29 A84"; $resn_exch{"H293 A84"}="3H29 A84";
$resn_exch{"H301 A84"}="1H30 A84"; $resn_exch{"H302 A84"}="2H30 A84"; $resn_exch{"H303 A84"}="3H30 A84";
$resn_exch{" H71 A88"}="1H7  A88"; $resn_exch{" H72 A88"}="2H7  A88"; $resn_exch{" H81 A88"}="1H8  A88";
$resn_exch{" H82 A88"}="2H8  A88"; $resn_exch{"H151 A88"}="1H15 A88"; $resn_exch{"H152 A88"}="2H15 A88";
$resn_exch{"H221 A88"}="1H22 A88"; $resn_exch{"H222 A88"}="2H22 A88"; $resn_exch{"H311 A88"}="1H31 A88";
$resn_exch{"H312 A88"}="2H31 A88"; $resn_exch{"H313 A88"}="3H31 A88"; $resn_exch{"H321 A88"}="1H32 A88";
$resn_exch{"H322 A88"}="2H32 A88"; $resn_exch{"H411 A88"}="1H41 A88"; $resn_exch{"H412 A88"}="2H41 A88";
$resn_exch{"H413 A88"}="3H41 A88"; $resn_exch{"H101 A8B"}="1H10 A8B"; $resn_exch{"H102 A8B"}="2H10 A8B";
$resn_exch{"H111 A8B"}="1H11 A8B"; $resn_exch{"H112 A8B"}="2H11 A8B"; $resn_exch{"H121 A8B"}="1H12 A8B";
$resn_exch{"H122 A8B"}="2H12 A8B"; $resn_exch{"H131 A8B"}="1H13 A8B"; $resn_exch{"H132 A8B"}="2H13 A8B";
$resn_exch{"H161 A8B"}="1H16 A8B"; $resn_exch{"H162 A8B"}="2H16 A8B"; $resn_exch{"H171 A8B"}="1H17 A8B";
$resn_exch{"H172 A8B"}="2H17 A8B"; $resn_exch{"H181 A8B"}="1H18 A8B"; $resn_exch{"H182 A8B"}="2H18 A8B";
$resn_exch{"H191 A8B"}="1H19 A8B"; $resn_exch{"H192 A8B"}="2H19 A8B"; $resn_exch{"H201 A8B"}="1H20 A8B";
$resn_exch{"H202 A8B"}="2H20 A8B"; $resn_exch{"H211 A8B"}="1H21 A8B"; $resn_exch{"H212 A8B"}="2H21 A8B";
$resn_exch{"H221 A8B"}="1H22 A8B"; $resn_exch{"H222 A8B"}="2H22 A8B"; $resn_exch{"H231 A8B"}="1H23 A8B";
$resn_exch{"H232 A8B"}="2H23 A8B"; $resn_exch{"H272 A8B"}="2H27 A8B"; $resn_exch{"H282 A8B"}="2H28 A8B";
$resn_exch{"H292 A8B"}="2H29 A8B"; $resn_exch{"H302 A8B"}="2H30 A8B"; $resn_exch{"H332 A8B"}="2H33 A8B";
$resn_exch{"H342 A8B"}="2H34 A8B"; $resn_exch{"H101 A8N"}="1H10 A8N"; $resn_exch{"H131 A8N"}="1H13 A8N";
$resn_exch{"H132 A8N"}="2H13 A8N"; $resn_exch{"H141 A8N"}="1H14 A8N"; $resn_exch{"H142 A8N"}="2H14 A8N";
$resn_exch{"H151 A8N"}="1H15 A8N"; $resn_exch{"H152 A8N"}="2H15 A8N"; $resn_exch{"H161 A8N"}="1H16 A8N";
$resn_exch{"H162 A8N"}="2H16 A8N"; $resn_exch{"H171 A8N"}="1H17 A8N"; $resn_exch{"H172 A8N"}="2H17 A8N";
$resn_exch{"H181 A8N"}="1H18 A8N"; $resn_exch{"H182 A8N"}="2H18 A8N"; $resn_exch{"H191 A8N"}="1H19 A8N";
$resn_exch{"H192 A8N"}="2H19 A8N"; $resn_exch{"H201 A8N"}="1H20 A8N"; $resn_exch{"H202 A8N"}="2H20 A8N";
$resn_exch{"H211 A8N"}="1H21 A8N"; $resn_exch{"H212 A8N"}="2H21 A8N"; $resn_exch{"H221 A8N"}="1H22 A8N";
$resn_exch{"H222 A8N"}="2H22 A8N"; $resn_exch{"H231 A8N"}="1H23 A8N"; $resn_exch{"H232 A8N"}="2H23 A8N";
$resn_exch{"H241 A8N"}="1H24 A8N"; $resn_exch{"H242 A8N"}="2H24 A8N"; $resn_exch{"H251 A8N"}="1H25 A8N";
$resn_exch{"H252 A8N"}="2H25 A8N"; $resn_exch{"H253 A8N"}="3H25 A8N"; $resn_exch{"H261 A9A"}="1H26 A9A";
$resn_exch{"H262 A9A"}="2H26 A9A"; $resn_exch{"H291 A9A"}="1H29 A9A"; $resn_exch{"H292 A9A"}="2H29 A9A";
$resn_exch{"H293 A9A"}="3H29 A9A"; $resn_exch{"H301 A9A"}="1H30 A9A"; $resn_exch{"H302 A9A"}="2H30 A9A";
$resn_exch{"H303 A9A"}="3H30 A9A"; $resn_exch{"H361 A9A"}="1H36 A9A"; $resn_exch{"H362 A9A"}="2H36 A9A";
$resn_exch{"H441 A9A"}="1H44 A9A"; $resn_exch{"H442 A9A"}="2H44 A9A"; $resn_exch{"H443 A9A"}="3H44 A9A";
$resn_exch{"H451 A9A"}="1H45 A9A"; $resn_exch{"H452 A9A"}="2H45 A9A"; $resn_exch{"H471 A9A"}="1H47 A9A";
$resn_exch{"H472 A9A"}="2H47 A9A"; $resn_exch{"H473 A9A"}="3H47 A9A"; $resn_exch{"H481 A9A"}="1H48 A9A";
$resn_exch{"H482 A9A"}="2H48 A9A"; $resn_exch{"H483 A9A"}="3H48 A9A"; $resn_exch{"H491 A9A"}="1H49 A9A";
$resn_exch{"H492 A9A"}="2H49 A9A"; $resn_exch{"H501 A9A"}="1H50 A9A"; $resn_exch{"H502 A9A"}="2H50 A9A";
$resn_exch{"H531 A9A"}="1H53 A9A"; $resn_exch{"H532 A9A"}="2H53 A9A"; $resn_exch{"H541 A9A"}="1H54 A9A";
$resn_exch{"H542 A9A"}="2H54 A9A"; $resn_exch{"H551 A9A"}="1H55 A9A"; $resn_exch{"H552 A9A"}="2H55 A9A";
$resn_exch{"H581 A9A"}="1H58 A9A"; $resn_exch{"H582 A9A"}="2H58 A9A"; $resn_exch{"H601 A9A"}="1H60 A9A";
$resn_exch{"H602 A9A"}="2H60 A9A"; $resn_exch{"H603 A9A"}="3H60 A9A"; $resn_exch{"H621 A9A"}="1H62 A9A";
$resn_exch{"H622 A9A"}="2H62 A9A"; $resn_exch{"H641 A9A"}="1H64 A9A"; $resn_exch{"H642 A9A"}="2H64 A9A";
$resn_exch{"H643 A9A"}="3H64 A9A"; $resn_exch{"H651 A9A"}="1H65 A9A"; $resn_exch{"H652 A9A"}="2H65 A9A";
$resn_exch{"H653 A9A"}="3H65 A9A"; $resn_exch{"H3N1 AA1"}="1H3N AA1"; $resn_exch{"H3N2 AA1"}="2H3N AA1";
$resn_exch{"H7C1 AA1"}="1H7C AA1"; $resn_exch{"H7C2 AA1"}="2H7C AA1"; $resn_exch{"H111 AA1"}="1H11 AA1";
$resn_exch{"H112 AA1"}="2H11 AA1"; $resn_exch{"H121 AA1"}="1H12 AA1"; $resn_exch{"H122 AA1"}="2H12 AA1";
$resn_exch{"H181 AA1"}="1H18 AA1"; $resn_exch{"H182 AA1"}="2H18 AA1"; $resn_exch{"H191 AA1"}="1H19 AA1";
$resn_exch{"H192 AA1"}="2H19 AA1"; $resn_exch{"H193 AA1"}="3H19 AA1"; $resn_exch{"H221 AA2"}="1H22 AA2";
$resn_exch{"H222 AA2"}="2H22 AA2"; $resn_exch{"H223 AA2"}="3H22 AA2"; $resn_exch{"H231 AA2"}="1H23 AA2";
$resn_exch{"H232 AA2"}="2H23 AA2"; $resn_exch{"H233 AA2"}="3H23 AA2"; $resn_exch{"H241 AA2"}="1H24 AA2";
$resn_exch{"H242 AA2"}="2H24 AA2"; $resn_exch{"H251 AA2"}="1H25 AA2"; $resn_exch{"H252 AA2"}="2H25 AA2";
$resn_exch{"H261 AA2"}="1H26 AA2"; $resn_exch{"H262 AA2"}="2H26 AA2"; $resn_exch{"H281 AA2"}="1H28 AA2";
$resn_exch{"H282 AA2"}="2H28 AA2"; $resn_exch{"H291 AA2"}="1H29 AA2"; $resn_exch{"H292 AA2"}="2H29 AA2";
$resn_exch{"H311 AA2"}="1H31 AA2"; $resn_exch{"H312 AA2"}="2H31 AA2"; $resn_exch{"H321 AA2"}="1H32 AA2";
$resn_exch{"H322 AA2"}="2H32 AA2"; $resn_exch{" H2  AA4"}=" HN  AA4"; $resn_exch{" HB2 AA4"}=" HB1 AA4";
$resn_exch{" HB3 AA4"}=" HB2 AA4"; $resn_exch{" HG2 AA4"}=" HG1 AA4"; $resn_exch{" HG3 AA4"}=" HG2 AA4";
$resn_exch{" HD2 AA4"}=" HD1 AA4"; $resn_exch{" HD3 AA4"}=" HD2 AA4"; $resn_exch{" HE  AA4"}=" HOE AA4";
$resn_exch{" HB1 AA5"}="1HB  AA5"; $resn_exch{" HB2 AA5"}="2HB  AA5"; $resn_exch{" HG1 AA5"}="1HG  AA5";
$resn_exch{" HG2 AA5"}="2HG  AA5"; $resn_exch{" HE1 AA5"}="1HE  AA5"; $resn_exch{" HE2 AA5"}="2HE  AA5";
$resn_exch{" HE3 AA5"}="3HE  AA5"; $resn_exch{"H2A1 AA5"}="1H2A AA5"; $resn_exch{"H2A2 AA5"}="2H2A AA5";
$resn_exch{"H2A3 AA5"}="3H2A AA5"; $resn_exch{"H5A1 AA5"}="1H5A AA5"; $resn_exch{"H5A2 AA5"}="2H5A AA5";
$resn_exch{"H3C1 AA7"}="1H3C AA7"; $resn_exch{"H3C2 AA7"}="2H3C AA7"; $resn_exch{"H4C1 AA7"}="1H4C AA7";
$resn_exch{"H4C2 AA7"}="2H4C AA7"; $resn_exch{"H5C1 AA7"}="1H5C AA7"; $resn_exch{"H5C2 AA7"}="2H5C AA7";
$resn_exch{"H6C1 AA7"}="1H6C AA7"; $resn_exch{"H6C2 AA7"}="2H6C AA7"; $resn_exch{"H191 AA7"}="1H19 AA7";
$resn_exch{"H192 AA7"}="2H19 AA7"; $resn_exch{"H201 AA7"}="1H20 AA7"; $resn_exch{"H202 AA7"}="2H20 AA7";
$resn_exch{"H211 AA7"}="1H21 AA7"; $resn_exch{"H212 AA7"}="2H21 AA7"; $resn_exch{"H221 AA7"}="1H22 AA7";
$resn_exch{"H222 AA7"}="2H22 AA7"; $resn_exch{"H231 AA7"}="1H23 AA7"; $resn_exch{"H232 AA7"}="2H23 AA7";
$resn_exch{"H241 AA7"}="1H24 AA7"; $resn_exch{"H242 AA7"}="2H24 AA7"; $resn_exch{"H251 AA7"}="1H25 AA7";
$resn_exch{"H252 AA7"}="2H25 AA7"; $resn_exch{"H281 AA7"}="1H28 AA7"; $resn_exch{"H282 AA7"}="2H28 AA7";
$resn_exch{"H291 AA7"}="1H29 AA7"; $resn_exch{"H292 AA7"}="2H29 AA7"; $resn_exch{"H301 AA7"}="1H30 AA7";
$resn_exch{"H302 AA7"}="2H30 AA7"; $resn_exch{"H311 AA7"}="1H31 AA7"; $resn_exch{"H312 AA7"}="2H31 AA7";
$resn_exch{"H1B1 AAA"}="1H1B AAA"; $resn_exch{"H1B2 AAA"}="2H1B AAA"; $resn_exch{"H1B3 AAA"}="3H1B AAA";
$resn_exch{"H4A1 AAA"}="1H4A AAA"; $resn_exch{"H4A2 AAA"}="2H4A AAA"; $resn_exch{"H4A3 AAA"}="3H4A AAA";
$resn_exch{"HN81 AAA"}="1HN8 AAA"; $resn_exch{"HN82 AAA"}="2HN8 AAA"; $resn_exch{"H5'1 AAB"}="1H5' AAB";
$resn_exch{"H5'2 AAB"}="2H5' AAB"; $resn_exch{"HO1' AAB"}="'HO1 AAB"; $resn_exch{"H2'1 AAB"}="1H2' AAB";
$resn_exch{"H2'2 AAB"}="2H2' AAB"; $resn_exch{"HO3' AAB"}="'HO3 AAB"; $resn_exch{"HC21 AAC"}="1HC2 AAC";
$resn_exch{"HC22 AAC"}="2HC2 AAC"; $resn_exch{"HC41 AAC"}="1HC4 AAC"; $resn_exch{"HC42 AAC"}="2HC4 AAC";
$resn_exch{"HC43 AAC"}="3HC4 AAC"; $resn_exch{"HN61 AAD"}="1HN6 AAD"; $resn_exch{"HN62 AAD"}="2HN6 AAD";
$resn_exch{"HN21 AAD"}="1HN2 AAD"; $resn_exch{"HN22 AAD"}="2HN2 AAD"; $resn_exch{"HCB1 AAD"}="1HCB AAD";
$resn_exch{"HCB2 AAD"}="2HCB AAD"; $resn_exch{"HCG1 AAD"}="1HCG AAD"; $resn_exch{"HCG2 AAD"}="2HCG AAD";
$resn_exch{"HCE1 AAD"}="1HCE AAD"; $resn_exch{"HCE2 AAD"}="2HCE AAD"; $resn_exch{"HCE3 AAD"}="3HCE AAD";
$resn_exch{"HC51 AAD"}="1HC5 AAD"; $resn_exch{"HC52 AAD"}="2HC5 AAD"; $resn_exch{" H41 AAE"}="1H4  AAE";
$resn_exch{" H42 AAE"}="2H4  AAE"; $resn_exch{" H91 AAE"}="1H9  AAE"; $resn_exch{" H92 AAE"}="2H9  AAE";
$resn_exch{" H93 AAE"}="3H9  AAE"; $resn_exch{" H41 AAF"}="1H4  AAF"; $resn_exch{" H42 AAF"}="2H4  AAF";
$resn_exch{" H51 AAF"}="1H5  AAF"; $resn_exch{" H52 AAF"}="2H5  AAF"; $resn_exch{" H91 AAF"}="1H9  AAF";
$resn_exch{" H92 AAF"}="2H9  AAF"; $resn_exch{"H111 AAF"}="1H11 AAF"; $resn_exch{"H112 AAF"}="2H11 AAF";
$resn_exch{"H121 AAF"}="1H12 AAF"; $resn_exch{"H122 AAF"}="2H12 AAF"; $resn_exch{"H131 AAF"}="1H13 AAF";
$resn_exch{"H132 AAF"}="2H13 AAF"; $resn_exch{"H141 AAF"}="1H14 AAF"; $resn_exch{"H142 AAF"}="2H14 AAF";
$resn_exch{"H181 AAF"}="1H18 AAF"; $resn_exch{"H182 AAF"}="2H18 AAF"; $resn_exch{"H291 AAF"}="1H29 AAF";
$resn_exch{"H292 AAF"}="2H29 AAF"; $resn_exch{"H293 AAF"}="3H29 AAF"; $resn_exch{"HC21 AAG"}="1HC2 AAG";
$resn_exch{"HC22 AAG"}="2HC2 AAG"; $resn_exch{"HC41 AAG"}="1HC4 AAG"; $resn_exch{"HC42 AAG"}="2HC4 AAG";
$resn_exch{"HC51 AAG"}="1HC5 AAG"; $resn_exch{"HC52 AAG"}="2HC5 AAG"; $resn_exch{"HN31 AAG"}="1HN3 AAG";
$resn_exch{"HN32 AAG"}="2HN3 AAG"; $resn_exch{"HC91 AAG"}="1HC9 AAG"; $resn_exch{"HC92 AAG"}="2HC9 AAG";
$resn_exch{"HC93 AAG"}="3HC9 AAG"; $resn_exch{"H101 AAH"}="1H10 AAH"; $resn_exch{"H102 AAH"}="2H10 AAH";
$resn_exch{"H121 AAH"}="1H12 AAH"; $resn_exch{"H122 AAH"}="2H12 AAH"; $resn_exch{"H171 AAH"}="1H17 AAH";
$resn_exch{"H172 AAH"}="2H17 AAH"; $resn_exch{"H181 AAH"}="1H18 AAH"; $resn_exch{"H182 AAH"}="2H18 AAH";
$resn_exch{"H191 AAH"}="1H19 AAH"; $resn_exch{"H192 AAH"}="2H19 AAH"; $resn_exch{"H201 AAH"}="1H20 AAH";
$resn_exch{"H202 AAH"}="2H20 AAH"; $resn_exch{" H61 AAL"}="1H6  AAL"; $resn_exch{" H62 AAL"}="2H6  AAL";
$resn_exch{"HN61 AAM"}="1HN6 AAM"; $resn_exch{"HN62 AAM"}="2HN6 AAM"; $resn_exch{"HOP2 AAM"}="2HOP AAM";
$resn_exch{"HOP3 AAM"}="3HOP AAM"; $resn_exch{"H5'1 AAM"}="1H5* AAM"; $resn_exch{"H5'2 AAM"}="2H5* AAM";
$resn_exch{" H4' AAM"}=" H4* AAM"; $resn_exch{" H3' AAM"}=" H3* AAM"; $resn_exch{"HO3' AAM"}="*HO3 AAM";
$resn_exch{" H2' AAM"}=" H2* AAM"; $resn_exch{"HO2' AAM"}="*HO2 AAM"; $resn_exch{" H1' AAM"}=" H1* AAM";
$resn_exch{" H21 AAN"}="1H2  AAN"; $resn_exch{" H22 AAN"}="2H2  AAN"; $resn_exch{"H6A1 AAO"}="1H6A AAO";
$resn_exch{"H6A2 AAO"}="2H6A AAO"; $resn_exch{"H6B1 AAO"}="1H6B AAO"; $resn_exch{"H6B2 AAO"}="2H6B AAO";
$resn_exch{"H6C1 AAO"}="1H6C AAO"; $resn_exch{"H6C2 AAO"}="2H6C AAO"; $resn_exch{"H6C3 AAO"}="3H6C AAO";
$resn_exch{"H7H1 AAO"}="1H7H AAO"; $resn_exch{"H7H2 AAO"}="2H7H AAO"; $resn_exch{"H6G1 AAO"}="1H6G AAO";
$resn_exch{"H6G2 AAO"}="2H6G AAO"; $resn_exch{"H6T1 AAO"}="1H6T AAO"; $resn_exch{"H6T2 AAO"}="2H6T AAO";
$resn_exch{" HN1 AAP"}="1HN  AAP"; $resn_exch{" HN2 AAP"}="2HN  AAP"; $resn_exch{"HM'1 AAP"}="1HM' AAP";
$resn_exch{"HM'2 AAP"}="2HM' AAP"; $resn_exch{"HM'3 AAP"}="3HM' AAP"; $resn_exch{"HM51 AAP"}="1HM5 AAP";
$resn_exch{"HM52 AAP"}="2HM5 AAP"; $resn_exch{"HM53 AAP"}="3HM5 AAP"; $resn_exch{" H   AAR"}="1HN  AAR";
$resn_exch{" H2  AAR"}="2HN  AAR"; $resn_exch{" HB2 AAR"}="1HB  AAR"; $resn_exch{" HB3 AAR"}="2HB  AAR";
$resn_exch{" HG2 AAR"}="1HG  AAR"; $resn_exch{" HG3 AAR"}="2HG  AAR"; $resn_exch{" HD2 AAR"}="1HD  AAR";
$resn_exch{" HD3 AAR"}="2HD  AAR"; $resn_exch{"HH11 AAR"}="1HH1 AAR"; $resn_exch{"HH12 AAR"}="2HH1 AAR";
$resn_exch{"HH21 AAR"}="1HH2 AAR"; $resn_exch{"HH22 AAR"}="2HH2 AAR"; $resn_exch{"HNT1 AAR"}="1HNT AAR";
$resn_exch{"HNT2 AAR"}="2HNT AAR"; $resn_exch{" H81 AAS"}="1H8  AAS"; $resn_exch{" H82 AAS"}="2H8  AAS";
$resn_exch{" H83 AAS"}="3H8  AAS"; $resn_exch{"HN11 AAS"}="1HN1 AAS"; $resn_exch{"HN12 AAS"}="2HN1 AAS";
$resn_exch{"HN21 AAS"}="1HN2 AAS"; $resn_exch{"HN22 AAS"}="2HN2 AAS"; $resn_exch{"HCA1 AAT"}="1HCA AAT";
$resn_exch{"HCA2 AAT"}="2HCA AAT"; $resn_exch{"H5'1 AAT"}="1H5* AAT"; $resn_exch{"H5'2 AAT"}="2H5* AAT";
$resn_exch{"H4'C AAT"}="CH4* AAT"; $resn_exch{"H3'C AAT"}="CH3* AAT"; $resn_exch{"H3'O AAT"}="OH3* AAT";
$resn_exch{"H2'C AAT"}="CH2* AAT"; $resn_exch{"H2'O AAT"}="OH2* AAT"; $resn_exch{"H1'C AAT"}="CH1* AAT";
$resn_exch{"HN61 AAT"}="1HN6 AAT"; $resn_exch{"HN62 AAT"}="2HN6 AAT"; $resn_exch{"H111 AAT"}="1H11 AAT";
$resn_exch{"H112 AAT"}="2H11 AAT"; $resn_exch{"H121 AAT"}="1H12 AAT"; $resn_exch{"H122 AAT"}="2H12 AAT";
$resn_exch{"H131 AAT"}="1H13 AAT"; $resn_exch{"H132 AAT"}="2H13 AAT"; $resn_exch{"H141 AAT"}="1H14 AAT";
$resn_exch{"H142 AAT"}="2H14 AAT"; $resn_exch{"H151 AAT"}="1H15 AAT"; $resn_exch{"H152 AAT"}="2H15 AAT";
$resn_exch{"H161 AAT"}="1H16 AAT"; $resn_exch{"H162 AAT"}="2H16 AAT"; $resn_exch{"H11A AAU"}="1H1  AAU";
$resn_exch{"H12A AAU"}="2H1  AAU"; $resn_exch{"H13A AAU"}="3H1  AAU"; $resn_exch{" H61 AAU"}="1H6  AAU";
$resn_exch{" H62 AAU"}="2H6  AAU"; $resn_exch{" H91 AAU"}="1H9  AAU"; $resn_exch{" H92 AAU"}="2H9  AAU";
$resn_exch{"H181 AAU"}="1H18 AAU"; $resn_exch{"H182 AAU"}="2H18 AAU"; $resn_exch{"H211 AAU"}="1H21 AAU";
$resn_exch{"H212 AAU"}="2H21 AAU"; $resn_exch{"H291 AAU"}="1H29 AAU"; $resn_exch{"H292 AAU"}="2H29 AAU";
$resn_exch{"H293 AAU"}="3H29 AAU"; $resn_exch{"H351 AAU"}="1H35 AAU"; $resn_exch{"H352 AAU"}="2H35 AAU";
$resn_exch{"H361 AAU"}="1H36 AAU"; $resn_exch{"H362 AAU"}="2H36 AAU"; $resn_exch{"H311 AAX"}="1H31 AAX";
$resn_exch{"H312 AAX"}="2H31 AAX"; $resn_exch{"H341 AAX"}="1H34 AAX"; $resn_exch{"H342 AAX"}="2H34 AAX";
$resn_exch{"H331 AAX"}="1H33 AAX"; $resn_exch{"H332 AAX"}="2H33 AAX"; $resn_exch{" H11 AAX"}="1H1  AAX";
$resn_exch{"H12A AAX"}="2H1  AAX"; $resn_exch{" H13 AAX"}="3H1  AAX"; $resn_exch{"H621 AAY"}="1H62 AAY";
$resn_exch{"H622 AAY"}="2H62 AAY"; $resn_exch{"H623 AAY"}="3H62 AAY"; $resn_exch{"H551 AAY"}="1H55 AAY";
$resn_exch{"H552 AAY"}="2H55 AAY"; $resn_exch{"H581 AAY"}="1H58 AAY"; $resn_exch{"H582 AAY"}="2H58 AAY";
$resn_exch{"H583 AAY"}="3H58 AAY"; $resn_exch{"H281 AAY"}="1H28 AAY"; $resn_exch{"H282 AAY"}="2H28 AAY";
$resn_exch{"H461 AAY"}="1H46 AAY"; $resn_exch{"H462 AAY"}="2H46 AAY"; $resn_exch{"H463 AAY"}="3H46 AAY";
$resn_exch{"H421 AAY"}="1H42 AAY"; $resn_exch{"H422 AAY"}="2H42 AAY"; $resn_exch{"H423 AAY"}="3H42 AAY";
$resn_exch{"H191 AAY"}="1H19 AAY"; $resn_exch{"H192 AAY"}="2H19 AAY"; $resn_exch{"H161 AAY"}="1H16 AAY";
$resn_exch{"H162 AAY"}="2H16 AAY"; $resn_exch{" H91 AAY"}="1H9  AAY"; $resn_exch{"H92A AAY"}="2H9  AAY";
$resn_exch{" H41 AAY"}="1H4  AAY"; $resn_exch{" H42 AAY"}="2H4  AAY"; $resn_exch{"H661 AAY"}="1H66 AAY";
$resn_exch{"H662 AAY"}="2H66 AAY"; $resn_exch{"H851 AAY"}="1H85 AAY"; $resn_exch{"H852 AAY"}="2H85 AAY";
$resn_exch{"H811 AAY"}="1H81 AAY"; $resn_exch{"H812 AAY"}="2H81 AAY"; $resn_exch{"H813 AAY"}="3H81 AAY";
$resn_exch{"H21A AAZ"}="1H2  AAZ"; $resn_exch{"H22A AAZ"}="2H2  AAZ"; $resn_exch{" H11 AAZ"}="1H1  AAZ";
$resn_exch{" H12 AAZ"}="2H1  AAZ"; $resn_exch{"H13A AAZ"}="3H1  AAZ"; $resn_exch{"H111 AAZ"}="1H11 AAZ";
$resn_exch{"H112 AAZ"}="2H11 AAZ"; $resn_exch{"H113 AAZ"}="3H11 AAZ"; $resn_exch{"H181 AB0"}="1H18 AB0";
$resn_exch{"H182 AB0"}="2H18 AB0"; $resn_exch{"H021 AB0"}="1H02 AB0"; $resn_exch{"H022 AB0"}="2H02 AB0";
$resn_exch{"H061 AB0"}="1H06 AB0"; $resn_exch{"H062 AB0"}="2H06 AB0"; $resn_exch{"H151 AB0"}="1H15 AB0";
$resn_exch{"H152 AB0"}="2H15 AB0"; $resn_exch{"H201 AB0"}="1H20 AB0"; $resn_exch{"H202 AB0"}="2H20 AB0";
$resn_exch{"H221 AB0"}="1H22 AB0"; $resn_exch{"H222 AB0"}="2H22 AB0"; $resn_exch{"H223 AB0"}="3H22 AB0";
$resn_exch{" H11 AB2"}="1H1  AB2"; $resn_exch{" H12 AB2"}="2H1  AB2"; $resn_exch{"H13A AB2"}="3H1  AB2";
$resn_exch{" H81 AB2"}="1H8  AB2"; $resn_exch{" H82 AB2"}="2H8  AB2"; $resn_exch{"H101 AB2"}="1H10 AB2";
$resn_exch{"H102 AB2"}="2H10 AB2"; $resn_exch{"H103 AB2"}="3H10 AB2"; $resn_exch{"H111 AB2"}="1H11 AB2";
$resn_exch{"H112 AB2"}="2H11 AB2"; $resn_exch{"H113 AB2"}="3H11 AB2"; $resn_exch{"H121 AB2"}="1H12 AB2";
$resn_exch{"H122 AB2"}="2H12 AB2"; $resn_exch{"H151 AB2"}="1H15 AB2"; $resn_exch{"H152 AB2"}="2H15 AB2";
$resn_exch{"H241 AB2"}="1H24 AB2"; $resn_exch{"H242 AB2"}="2H24 AB2"; $resn_exch{"H281 AB2"}="1H28 AB2";
$resn_exch{"H282 AB2"}="2H28 AB2"; $resn_exch{"H201 AB3"}="1H20 AB3"; $resn_exch{"H202 AB3"}="2H20 AB3";
$resn_exch{"H203 AB3"}="3H20 AB3"; $resn_exch{"H191 AB3"}="1H19 AB3"; $resn_exch{"H192 AB3"}="2H19 AB3";
$resn_exch{"H193 AB3"}="3H19 AB3"; $resn_exch{"H211 AB3"}="1H21 AB3"; $resn_exch{"H212 AB3"}="2H21 AB3";
$resn_exch{"H213 AB3"}="3H21 AB3"; $resn_exch{"HN11 AB3"}="1HN1 AB3"; $resn_exch{"HN12 AB3"}="2HN1 AB3";
$resn_exch{"H221 AB6"}="1H22 AB6"; $resn_exch{"H222 AB6"}="2H22 AB6"; $resn_exch{"H251 AB6"}="1H25 AB6";
$resn_exch{"H252 AB6"}="2H25 AB6"; $resn_exch{"H261 AB6"}="1H26 AB6"; $resn_exch{"H262 AB6"}="2H26 AB6";
$resn_exch{"H291 AB6"}="1H29 AB6"; $resn_exch{"H292 AB6"}="2H29 AB6"; $resn_exch{"H301 AB6"}="1H30 AB6";
$resn_exch{"H302 AB6"}="2H30 AB6"; $resn_exch{"H331 AB6"}="1H33 AB6"; $resn_exch{"H332 AB6"}="2H33 AB6";
$resn_exch{"H341 AB6"}="1H34 AB6"; $resn_exch{"H342 AB6"}="2H34 AB6"; $resn_exch{"H211 AB6"}="1H21 AB6";
$resn_exch{"H212 AB6"}="2H21 AB6"; $resn_exch{"H611 AB6"}="1H61 AB6"; $resn_exch{"H612 AB6"}="2H61 AB6";
$resn_exch{"H321 AB6"}="1H32 AB6"; $resn_exch{"H322 AB6"}="2H32 AB6"; $resn_exch{"H11A AB6"}="1H1  AB6";
$resn_exch{"H12A AB6"}="2H1  AB6"; $resn_exch{"H351 AB6"}="1H35 AB6"; $resn_exch{"H352 AB6"}="2H35 AB6";
$resn_exch{"H271 AB6"}="1H27 AB6"; $resn_exch{"H272 AB6"}="2H27 AB6"; $resn_exch{" HN1 AB7"}="1HN  AB7";
$resn_exch{" HN2 AB7"}="2HN  AB7"; $resn_exch{"HBC1 AB7"}="1HBC AB7"; $resn_exch{"HBC2 AB7"}="2HBC AB7";
$resn_exch{"HGC1 AB7"}="1HGC AB7"; $resn_exch{"HGC2 AB7"}="2HGC AB7"; $resn_exch{"HGC3 AB7"}="3HGC AB7";
$resn_exch{"H881 AB8"}="1H88 AB8"; $resn_exch{"H882 AB8"}="2H88 AB8"; $resn_exch{"H883 AB8"}="3H88 AB8";
$resn_exch{"H731 AB8"}="1H73 AB8"; $resn_exch{"H732 AB8"}="2H73 AB8"; $resn_exch{"H621 AB8"}="1H62 AB8";
$resn_exch{"H622 AB8"}="2H62 AB8"; $resn_exch{"H691 AB8"}="1H69 AB8"; $resn_exch{"H692 AB8"}="2H69 AB8";
$resn_exch{"H693 AB8"}="3H69 AB8"; $resn_exch{" H41 AB8"}="1H4  AB8"; $resn_exch{" H42 AB8"}="2H4  AB8";
$resn_exch{"H121 AB8"}="1H12 AB8"; $resn_exch{"H122 AB8"}="2H12 AB8"; $resn_exch{"H151 AB8"}="1H15 AB8";
$resn_exch{"H152 AB8"}="2H15 AB8"; $resn_exch{"H381 AB8"}="1H38 AB8"; $resn_exch{"H382 AB8"}="2H38 AB8";
$resn_exch{"H383 AB8"}="3H38 AB8"; $resn_exch{"H421 AB8"}="1H42 AB8"; $resn_exch{"H422 AB8"}="2H42 AB8";
$resn_exch{"H423 AB8"}="3H42 AB8"; $resn_exch{"H241 AB8"}="1H24 AB8"; $resn_exch{"H242 AB8"}="2H24 AB8";
$resn_exch{"H581 AB8"}="1H58 AB8"; $resn_exch{"H582 AB8"}="2H58 AB8"; $resn_exch{"H583 AB8"}="3H58 AB8";
$resn_exch{"H511 AB8"}="1H51 AB8"; $resn_exch{"H512 AB8"}="2H51 AB8"; $resn_exch{"H541 AB8"}="1H54 AB8";
$resn_exch{"H542 AB8"}="2H54 AB8"; $resn_exch{"H543 AB8"}="3H54 AB8"; $resn_exch{"H221 AB9"}="1H22 AB9";
$resn_exch{"H222 AB9"}="2H22 AB9"; $resn_exch{"H251 AB9"}="1H25 AB9"; $resn_exch{"H252 AB9"}="2H25 AB9";
$resn_exch{"H261 AB9"}="1H26 AB9"; $resn_exch{"H262 AB9"}="2H26 AB9"; $resn_exch{"H291 AB9"}="1H29 AB9";
$resn_exch{"H292 AB9"}="2H29 AB9"; $resn_exch{"H301 AB9"}="1H30 AB9"; $resn_exch{"H302 AB9"}="2H30 AB9";
$resn_exch{"H331 AB9"}="1H33 AB9"; $resn_exch{"H332 AB9"}="2H33 AB9"; $resn_exch{"H351 AB9"}="1H35 AB9";
$resn_exch{"H352 AB9"}="2H35 AB9"; $resn_exch{"H211 AB9"}="1H21 AB9"; $resn_exch{"H212 AB9"}="2H21 AB9";
$resn_exch{"H611 AB9"}="1H61 AB9"; $resn_exch{"H612 AB9"}="2H61 AB9"; $resn_exch{"H11A AB9"}="1H1  AB9";
$resn_exch{"H12A AB9"}="2H1  AB9"; $resn_exch{"H321 AB9"}="1H32 AB9"; $resn_exch{"H322 AB9"}="2H32 AB9";
$resn_exch{"H341 AB9"}="1H34 AB9"; $resn_exch{"H342 AB9"}="2H34 AB9"; $resn_exch{"H271 AB9"}="1H27 AB9";
$resn_exch{"H272 AB9"}="2H27 AB9"; $resn_exch{" H   ABA"}="1HN  ABA"; $resn_exch{" HN2 ABA"}="2HN  ABA";
$resn_exch{" HB2 ABA"}="1HB  ABA"; $resn_exch{" HB3 ABA"}="2HB  ABA"; $resn_exch{" HG1 ABA"}="1HG  ABA";
$resn_exch{" HG2 ABA"}="2HG  ABA"; $resn_exch{" HG3 ABA"}="3HG  ABA"; $resn_exch{"H541 ABB"}="1H54 ABB";
$resn_exch{"H542 ABB"}="2H54 ABB"; $resn_exch{"H521 ABB"}="1H52 ABB"; $resn_exch{"H522 ABB"}="2H52 ABB";
$resn_exch{"HCB1 ABB"}="1HCB ABB"; $resn_exch{"HCB2 ABB"}="2HCB ABB"; $resn_exch{"HCG1 ABB"}="1HCG ABB";
$resn_exch{"HCG2 ABB"}="2HCG ABB"; $resn_exch{"HCD1 ABB"}="1HCD ABB"; $resn_exch{"HCD2 ABB"}="2HCD ABB";
$resn_exch{"HH21 ABB"}="1HH2 ABB"; $resn_exch{"HH22 ABB"}="2HH2 ABB"; $resn_exch{"HC21 ABB"}="1HC2 ABB";
$resn_exch{"HC22 ABB"}="2HC2 ABB"; $resn_exch{"HC23 ABB"}="3HC2 ABB"; $resn_exch{"H6G1 ABC"}="1H6G ABC";
$resn_exch{"H6G2 ABC"}="2H6G ABC"; $resn_exch{"H6G3 ABC"}="3H6G ABC"; $resn_exch{"HO2G ABC"}="GHO2 ABC";
$resn_exch{"HO3G ABC"}="GHO3 ABC"; $resn_exch{"HO4G ABC"}="GHO4 ABC"; $resn_exch{"H6H1 ABC"}="1H6H ABC";
$resn_exch{"H6H2 ABC"}="2H6H ABC"; $resn_exch{"H6I1 ABC"}="1H6I ABC"; $resn_exch{"H6I2 ABC"}="2H6I ABC";
$resn_exch{"HO2I ABC"}="IHO2 ABC"; $resn_exch{"HO3I ABC"}="IHO3 ABC"; $resn_exch{"HO6I ABC"}="IHO6 ABC";
$resn_exch{"H6J1 ABC"}="1H6J ABC"; $resn_exch{"H6J2 ABC"}="2H6J ABC"; $resn_exch{"H6J3 ABC"}="3H6J ABC";
$resn_exch{"HO2J ABC"}="JHO2 ABC"; $resn_exch{"HO3J ABC"}="JHO3 ABC"; $resn_exch{"H6K1 ABC"}="1H6K ABC";
$resn_exch{"H6K2 ABC"}="2H6K ABC"; $resn_exch{"HO2K ABC"}="KHO2 ABC"; $resn_exch{"HO3K ABC"}="KHO3 ABC";
$resn_exch{"HO6K ABC"}="KHO6 ABC"; $resn_exch{"H6L1 ABC"}="1H6L ABC"; $resn_exch{"H6L2 ABC"}="2H6L ABC";
$resn_exch{"H6L3 ABC"}="3H6L ABC"; $resn_exch{"HO2L ABC"}="LHO2 ABC"; $resn_exch{"HO3L ABC"}="LHO3 ABC";
$resn_exch{"HO1L ABC"}="LHO1 ABC"; $resn_exch{"H6L1 ABD"}="1H6L ABD"; $resn_exch{"H6L2 ABD"}="2H6L ABD";
$resn_exch{"H6K1 ABD"}="1H6K ABD"; $resn_exch{"H6K2 ABD"}="2H6K ABD"; $resn_exch{"H6J1 ABD"}="1H6J ABD";
$resn_exch{"H6J2 ABD"}="2H6J ABD"; $resn_exch{"H6J3 ABD"}="3H6J ABD"; $resn_exch{"H6I1 ABD"}="1H6I ABD";
$resn_exch{"H6I2 ABD"}="2H6I ABD"; $resn_exch{"H6H1 ABD"}="1H6H ABD"; $resn_exch{"H6H2 ABD"}="2H6H ABD";
$resn_exch{"HOG4 ABD"}="4HOG ABD"; $resn_exch{"H6G1 ABD"}="1H6G ABD"; $resn_exch{"H6G2 ABD"}="2H6G ABD";
$resn_exch{" H31 ABE"}="1H3  ABE"; $resn_exch{" H32 ABE"}="2H3  ABE"; $resn_exch{" H61 ABE"}="1H6  ABE";
$resn_exch{" H62 ABE"}="2H6  ABE"; $resn_exch{" H63 ABE"}="3H6  ABE"; $resn_exch{"H5'1 ABF"}="1H5' ABF";
$resn_exch{"H5'2 ABF"}="2H5' ABF"; $resn_exch{"H3B2 ABG"}="2H3B ABG"; $resn_exch{"H3B1 ABG"}="1H3B ABG";
$resn_exch{"H5'2 ABG"}="2H5' ABG"; $resn_exch{"H5'1 ABG"}="1H5' ABG"; $resn_exch{" H62 ABG"}="2H6  ABG";
$resn_exch{" H61 ABG"}="1H6  ABG"; $resn_exch{"H2O1 ABG"}="1H2O ABG"; $resn_exch{"H3O1 ABG"}="1H3O ABG";
$resn_exch{" HN1 ABH"}="1HN  ABH"; $resn_exch{" HN2 ABH"}="2HN  ABH"; $resn_exch{"HOT1 ABH"}="1HOT ABH";
$resn_exch{"HCB1 ABH"}="1HCB ABH"; $resn_exch{"HCB2 ABH"}="2HCB ABH"; $resn_exch{"HCG1 ABH"}="1HCG ABH";
$resn_exch{"HCG2 ABH"}="2HCG ABH"; $resn_exch{"HCD1 ABH"}="1HCD ABH"; $resn_exch{"HCD2 ABH"}="2HCD ABH";
$resn_exch{"HCE1 ABH"}="1HCE ABH"; $resn_exch{"HCE2 ABH"}="2HCE ABH"; $resn_exch{"HH11 ABI"}="1HH1 ABI";
$resn_exch{"HH12 ABI"}="2HH1 ABI"; $resn_exch{"HH21 ABI"}="1HH2 ABI"; $resn_exch{"HH22 ABI"}="2HH2 ABI";
$resn_exch{" HB1 ABK"}="1HB  ABK"; $resn_exch{" HB2 ABK"}="2HB  ABK"; $resn_exch{" H61 ABL"}="1H6  ABL";
$resn_exch{" H62 ABL"}="2H6  ABL"; $resn_exch{"H2AO ABL"}="OH2A ABL"; $resn_exch{"H3AO ABL"}="OH3A ABL";
$resn_exch{"H4AO ABL"}="OH4A ABL"; $resn_exch{"H6A1 ABL"}="1H6A ABL"; $resn_exch{"H6A2 ABL"}="2H6A ABL";
$resn_exch{"H6AO ABL"}="OH6A ABL"; $resn_exch{"HOP3 ABM"}="3HOP ABM"; $resn_exch{"H5'1 ABM"}="1H5* ABM";
$resn_exch{"H5'2 ABM"}="2H5* ABM"; $resn_exch{" H4' ABM"}=" H4* ABM"; $resn_exch{" H3' ABM"}=" H3* ABM";
$resn_exch{"HO3' ABM"}="*HO3 ABM"; $resn_exch{" H2' ABM"}=" H2* ABM"; $resn_exch{"HO2' ABM"}="*HO2 ABM";
$resn_exch{" H1' ABM"}=" H1* ABM"; $resn_exch{"HN61 ABM"}="1HN6 ABM"; $resn_exch{"HN62 ABM"}="2HN6 ABM";
$resn_exch{" HN1 ABN"}="1HN  ABN"; $resn_exch{" HN2 ABN"}="2HN  ABN"; $resn_exch{" H1  ABN"}="1H   ABN";
$resn_exch{" H2A ABN"}="2H   ABN"; $resn_exch{"H181 ABO"}="1H18 ABO"; $resn_exch{"H182 ABO"}="2H18 ABO";
$resn_exch{"H183 ABO"}="3H18 ABO"; $resn_exch{"HOB2 ABP"}="2HOB ABP"; $resn_exch{"HOB3 ABP"}="3HOB ABP";
$resn_exch{"HOA2 ABP"}="2HOA ABP"; $resn_exch{"H5'1 ABP"}="1H5* ABP"; $resn_exch{"H5'2 ABP"}="2H5* ABP";
$resn_exch{" H4' ABP"}=" H4* ABP"; $resn_exch{" H3' ABP"}=" H3* ABP"; $resn_exch{"HO3' ABP"}="*HO3 ABP";
$resn_exch{" H2' ABP"}=" H2* ABP"; $resn_exch{"HO2' ABP"}="*HO2 ABP"; $resn_exch{" H1' ABP"}=" H1* ABP";
$resn_exch{"HN61 ABP"}="1HN6 ABP"; $resn_exch{"HN62 ABP"}="2HN6 ABP"; $resn_exch{"HOP2 ABR"}="2HOP ABR";
$resn_exch{" H5' ABR"}="1H5* ABR"; $resn_exch{"H5'' ABR"}="2H5* ABR"; $resn_exch{" H4' ABR"}=" H4* ABR";
$resn_exch{" H1' ABR"}=" H1* ABR"; $resn_exch{" H2' ABR"}="1H2* ABR"; $resn_exch{"H2'' ABR"}="2H2* ABR";
$resn_exch{" H3' ABR"}=" H3* ABR"; $resn_exch{"HO3' ABR"}="H3T  ABR"; $resn_exch{"HOP3 ABR"}="3HOP ABR";
$resn_exch{"HOP2 ABS"}="2HOP ABS"; $resn_exch{" H5' ABS"}="1H5* ABS"; $resn_exch{"H5'' ABS"}="2H5* ABS";
$resn_exch{" H4' ABS"}=" H4* ABS"; $resn_exch{" H1' ABS"}=" H1* ABS"; $resn_exch{" H2' ABS"}="1H2* ABS";
$resn_exch{"H2'' ABS"}="2H2* ABS"; $resn_exch{" H3' ABS"}=" H3* ABS"; $resn_exch{"HO3' ABS"}="H3T  ABS";
$resn_exch{"HOP3 ABS"}="3HOP ABS"; $resn_exch{"HOG2 ABT"}="2HOG ABT"; $resn_exch{"HOG3 ABT"}="3HOG ABT";
$resn_exch{"HOB2 ABT"}="2HOB ABT"; $resn_exch{"HOA2 ABT"}="2HOA ABT"; $resn_exch{"H5'1 ABT"}="1H5* ABT";
$resn_exch{"H5'2 ABT"}="2H5* ABT"; $resn_exch{" H4' ABT"}=" H4* ABT"; $resn_exch{" H1' ABT"}=" H1* ABT";
$resn_exch{" H51 ABT"}="1H5  ABT"; $resn_exch{" H52 ABT"}="2H5  ABT"; $resn_exch{" H53 ABT"}="3H5  ABT";
$resn_exch{"H2'1 ABT"}="1H2* ABT"; $resn_exch{"H2'2 ABT"}="2H2* ABT"; $resn_exch{" H3' ABT"}=" H3* ABT";
$resn_exch{" HA1 ABU"}="1HA  ABU"; $resn_exch{" HA2 ABU"}="2HA  ABU"; $resn_exch{" HB1 ABU"}="1HB  ABU";
$resn_exch{" HB2 ABU"}="2HB  ABU"; $resn_exch{" HG1 ABU"}="1HG  ABU"; $resn_exch{" HG2 ABU"}="2HG  ABU";
$resn_exch{" H41 ABW"}="1H4  ABW"; $resn_exch{" H42 ABW"}="2H4  ABW"; $resn_exch{" H71 ABW"}="1H7  ABW";
$resn_exch{" H72 ABW"}="2H7  ABW"; $resn_exch{" H73 ABW"}="3H7  ABW"; $resn_exch{" H21 ABW"}="1H2  ABW";
$resn_exch{" H22 ABW"}="2H2  ABW"; $resn_exch{"H101 ABW"}="1H10 ABW"; $resn_exch{"H102 ABW"}="2H10 ABW";
$resn_exch{"H103 ABW"}="3H10 ABW"; $resn_exch{"H151 ABW"}="1H15 ABW"; $resn_exch{"H152 ABW"}="2H15 ABW";
$resn_exch{"H153 ABW"}="3H15 ABW"; $resn_exch{"H161 ABW"}="1H16 ABW"; $resn_exch{"H162 ABW"}="2H16 ABW";
$resn_exch{"H163 ABW"}="3H16 ABW"; $resn_exch{" H41 ABX"}="1H4  ABX"; $resn_exch{" H42 ABX"}="2H4  ABX";
$resn_exch{" H71 ABX"}="1H7  ABX"; $resn_exch{" H72 ABX"}="2H7  ABX"; $resn_exch{" H73 ABX"}="3H7  ABX";
$resn_exch{" H91 ABX"}="1H9  ABX"; $resn_exch{" H92 ABX"}="2H9  ABX"; $resn_exch{"H111 ABX"}="1H11 ABX";
$resn_exch{"H112 ABX"}="2H11 ABX"; $resn_exch{"H113 ABX"}="3H11 ABX"; $resn_exch{"H121 ABX"}="1H12 ABX";
$resn_exch{"H122 ABX"}="2H12 ABX"; $resn_exch{"H123 ABX"}="3H12 ABX"; $resn_exch{"H141 ABX"}="1H14 ABX";
$resn_exch{"H142 ABX"}="2H14 ABX"; $resn_exch{"H143 ABX"}="3H14 ABX"; $resn_exch{"H7'1 ABY"}="1H7' ABY";
$resn_exch{"H7'2 ABY"}="2H7' ABY"; $resn_exch{"H7'3 ABY"}="3H7' ABY"; $resn_exch{" H'1 ABY"}="1H'  ABY";
$resn_exch{" H'2 ABY"}="2H'  ABY"; $resn_exch{"HB21 ABY"}="1HB2 ABY"; $resn_exch{"HB22 ABY"}="2HB2 ABY";
$resn_exch{"HG11 ABY"}="1HG1 ABY"; $resn_exch{"HG12 ABY"}="2HG1 ABY"; $resn_exch{"HB11 ABY"}="1HB1 ABY";
$resn_exch{"HB12 ABY"}="2HB1 ABY"; $resn_exch{"HA11 ABY"}="1HA1 ABY"; $resn_exch{"HA12 ABY"}="2HA1 ABY";
$resn_exch{"HN11 ABY"}="1HN1 ABY"; $resn_exch{"HN12 ABY"}="2HN1 ABY"; $resn_exch{"HA31 ABY"}="1HA3 ABY";
$resn_exch{"HA32 ABY"}="2HA3 ABY"; $resn_exch{" H71 ABZ"}="1H7  ABZ"; $resn_exch{" H72 ABZ"}="2H7  ABZ";
$resn_exch{"HN41 ABZ"}="1HN4 ABZ"; $resn_exch{"HN42 ABZ"}="2HN4 ABZ"; $resn_exch{" H81 AC0"}="1H8  AC0";
$resn_exch{" H82 AC0"}="2H8  AC0"; $resn_exch{" H83 AC0"}="3H8  AC0"; $resn_exch{"HCA3 AC1"}="3HCA AC1";
$resn_exch{"HOA3 AC1"}="3HOA AC1"; $resn_exch{"HCA4 AC1"}="4HCA AC1"; $resn_exch{"HCA5 AC1"}="5HCA AC1";
$resn_exch{"H6A1 AC1"}="1H6A AC1"; $resn_exch{"H6A2 AC1"}="2H6A AC1"; $resn_exch{"H6A3 AC1"}="3H6A AC1";
$resn_exch{"HCB1 AC1"}="1HCB AC1"; $resn_exch{"HCB2 AC1"}="2HCB AC1"; $resn_exch{"HCB3 AC1"}="3HCB AC1";
$resn_exch{"HOB3 AC1"}="3HOB AC1"; $resn_exch{"HC61 AC1"}="1HC6 AC1"; $resn_exch{"HC62 AC1"}="2HC6 AC1";
$resn_exch{"H3'1 AC2"}="1H3* AC2"; $resn_exch{"H3'2 AC2"}="2H3* AC2"; $resn_exch{"HO'3 AC2"}="3HO* AC2";
$resn_exch{"H2'1 AC2"}="1H2* AC2"; $resn_exch{"H2'2 AC2"}="2H2* AC2"; $resn_exch{"H1'1 AC2"}="1H1* AC2";
$resn_exch{"H1'2 AC2"}="2H1* AC2"; $resn_exch{"HN21 AC2"}="1HN2 AC2"; $resn_exch{"HN22 AC2"}="2HN2 AC2";
$resn_exch{"HB11 AC5"}="1HB1 AC5"; $resn_exch{"HB12 AC5"}="2HB1 AC5"; $resn_exch{"HB21 AC5"}="1HB2 AC5";
$resn_exch{"HB22 AC5"}="2HB2 AC5"; $resn_exch{"HG11 AC5"}="1HG1 AC5"; $resn_exch{"HG12 AC5"}="2HG1 AC5";
$resn_exch{"HG21 AC5"}="1HG2 AC5"; $resn_exch{"HG22 AC5"}="2HG2 AC5"; $resn_exch{" H81 AC6"}="1H8  AC6";
$resn_exch{" H82 AC6"}="2H8  AC6"; $resn_exch{" H83 AC6"}="3H8  AC6"; $resn_exch{" H41 AC9"}="1H4  AC9";
$resn_exch{" H42 AC9"}="2H4  AC9"; $resn_exch{" H43 AC9"}="3H4  AC9"; $resn_exch{" H51 AC9"}="1H5  AC9";
$resn_exch{" H52 AC9"}="2H5  AC9"; $resn_exch{" H53 AC9"}="3H5  AC9"; $resn_exch{" H91 AC9"}="1H9  AC9";
$resn_exch{" H92 AC9"}="2H9  AC9"; $resn_exch{" H93 AC9"}="3H9  AC9"; $resn_exch{"H101 AC9"}="1H10 AC9";
$resn_exch{"H102 AC9"}="2H10 AC9"; $resn_exch{"H103 AC9"}="3H10 AC9"; $resn_exch{"H141 AC9"}="1H14 AC9";
$resn_exch{"H142 AC9"}="2H14 AC9"; $resn_exch{"H143 AC9"}="3H14 AC9"; $resn_exch{"H151 AC9"}="1H15 AC9";
$resn_exch{"H152 AC9"}="2H15 AC9"; $resn_exch{"H153 AC9"}="3H15 AC9"; $resn_exch{" H21 ACA"}="1H2  ACA";
$resn_exch{" H22 ACA"}="2H2  ACA"; $resn_exch{" H31 ACA"}="1H3  ACA"; $resn_exch{" H32 ACA"}="2H3  ACA";
$resn_exch{" H41 ACA"}="1H4  ACA"; $resn_exch{" H42 ACA"}="2H4  ACA"; $resn_exch{" H51 ACA"}="1H5  ACA";
$resn_exch{" H52 ACA"}="2H5  ACA"; $resn_exch{" H61 ACA"}="1H6  ACA"; $resn_exch{" H62 ACA"}="2H6  ACA";
$resn_exch{"HN61 ACA"}="1HN6 ACA"; $resn_exch{"HN62 ACA"}="2HN6 ACA"; $resn_exch{"HN21 ACB"}="1HN2 ACB";
$resn_exch{"HN22 ACB"}="2HN2 ACB"; $resn_exch{" H41 ACB"}="1H4  ACB"; $resn_exch{" H42 ACB"}="2H4  ACB";
$resn_exch{" H43 ACB"}="3H4  ACB"; $resn_exch{" H52 ACC"}="2H5  ACC"; $resn_exch{" H51 ACC"}="1H5  ACC";
$resn_exch{" H92 ACC"}="2H9  ACC"; $resn_exch{" H91 ACC"}="1H9  ACC"; $resn_exch{"H102 ACC"}="2H10 ACC";
$resn_exch{"H101 ACC"}="1H10 ACC"; $resn_exch{"H132 ACC"}="2H13 ACC"; $resn_exch{"H131 ACC"}="1H13 ACC";
$resn_exch{"H193 ACC"}="3H19 ACC"; $resn_exch{"H192 ACC"}="2H19 ACC"; $resn_exch{"H191 ACC"}="1H19 ACC";
$resn_exch{"H352 ACC"}="2H35 ACC"; $resn_exch{"H351 ACC"}="1H35 ACC"; $resn_exch{"H423 ACC"}="3H42 ACC";
$resn_exch{"H422 ACC"}="2H42 ACC"; $resn_exch{"H421 ACC"}="1H42 ACC"; $resn_exch{" H21 ACD"}="1H2  ACD";
$resn_exch{" H22 ACD"}="2H2  ACD"; $resn_exch{" H31 ACD"}="1H3  ACD"; $resn_exch{" H32 ACD"}="2H3  ACD";
$resn_exch{" H41 ACD"}="1H4  ACD"; $resn_exch{" H42 ACD"}="2H4  ACD"; $resn_exch{" H71 ACD"}="1H7  ACD";
$resn_exch{" H72 ACD"}="2H7  ACD"; $resn_exch{"H101 ACD"}="1H10 ACD"; $resn_exch{"H102 ACD"}="2H10 ACD";
$resn_exch{"H131 ACD"}="1H13 ACD"; $resn_exch{"H132 ACD"}="2H13 ACD"; $resn_exch{"H161 ACD"}="1H16 ACD";
$resn_exch{"H162 ACD"}="2H16 ACD"; $resn_exch{"H171 ACD"}="1H17 ACD"; $resn_exch{"H172 ACD"}="2H17 ACD";
$resn_exch{"H181 ACD"}="1H18 ACD"; $resn_exch{"H182 ACD"}="2H18 ACD"; $resn_exch{"H191 ACD"}="1H19 ACD";
$resn_exch{"H192 ACD"}="2H19 ACD"; $resn_exch{"H201 ACD"}="1H20 ACD"; $resn_exch{"H202 ACD"}="2H20 ACD";
$resn_exch{"H203 ACD"}="3H20 ACD"; $resn_exch{" H1  ACE"}="1H   ACE"; $resn_exch{" H2  ACE"}="2H   ACE";
$resn_exch{" H3  ACE"}="3H   ACE"; $resn_exch{" H41 ACF"}="1H4  ACF"; $resn_exch{" H42 ACF"}="2H4  ACF";
$resn_exch{" H51 ACF"}="1H5  ACF"; $resn_exch{" H52 ACF"}="2H5  ACF"; $resn_exch{" H91 ACF"}="1H9  ACF";
$resn_exch{" H92 ACF"}="2H9  ACF"; $resn_exch{"H181 ACF"}="1H18 ACF"; $resn_exch{"H182 ACF"}="2H18 ACF";
$resn_exch{"H191 ACF"}="1H19 ACF"; $resn_exch{"H192 ACF"}="2H19 ACF"; $resn_exch{"H201 ACF"}="1H20 ACF";
$resn_exch{"H202 ACF"}="2H20 ACF"; $resn_exch{"H211 ACF"}="1H21 ACF"; $resn_exch{"H212 ACF"}="2H21 ACF";
$resn_exch{"H221 ACF"}="1H22 ACF"; $resn_exch{"H222 ACF"}="2H22 ACF"; $resn_exch{"H6A1 ACG"}="1H6A ACG";
$resn_exch{"H6A2 ACG"}="2H6A ACG"; $resn_exch{"HOA2 ACG"}="2HOA ACG"; $resn_exch{"HOA3 ACG"}="3HOA ACG";
$resn_exch{"HOA6 ACG"}="6HOA ACG"; $resn_exch{"H6B1 ACG"}="1H6B ACG"; $resn_exch{"H6B2 ACG"}="2H6B ACG";
$resn_exch{"H6B3 ACG"}="3H6B ACG"; $resn_exch{"HOB2 ACG"}="2HOB ACG"; $resn_exch{"HOB3 ACG"}="3HOB ACG";
$resn_exch{"H6C1 ACG"}="1H6C ACG"; $resn_exch{"H6C2 ACG"}="2H6C ACG"; $resn_exch{"HOC2 ACG"}="2HOC ACG";
$resn_exch{"HOC3 ACG"}="3HOC ACG"; $resn_exch{"HOC6 ACG"}="6HOC ACG"; $resn_exch{"H6D1 ACG"}="1H6D ACG";
$resn_exch{"H6D2 ACG"}="2H6D ACG"; $resn_exch{"HOD1 ACG"}="1HOD ACG"; $resn_exch{"HOD2 ACG"}="2HOD ACG";
$resn_exch{"HOD3 ACG"}="3HOD ACG"; $resn_exch{"HOD6 ACG"}="6HOD ACG"; $resn_exch{"H6E1 ACG"}="1H6E ACG";
$resn_exch{"H6E2 ACG"}="2H6E ACG"; $resn_exch{"H6E3 ACG"}="3H6E ACG"; $resn_exch{"HOE2 ACG"}="2HOE ACG";
$resn_exch{"HOE3 ACG"}="3HOE ACG"; $resn_exch{"HOE4 ACG"}="4HOE ACG"; $resn_exch{" H21 ACH"}="1H2  ACH";
$resn_exch{" H22 ACH"}="2H2  ACH"; $resn_exch{" H31 ACH"}="1H3  ACH"; $resn_exch{" H32 ACH"}="2H3  ACH";
$resn_exch{" H61 ACH"}="1H6  ACH"; $resn_exch{" H62 ACH"}="2H6  ACH"; $resn_exch{" H63 ACH"}="3H6  ACH";
$resn_exch{" H81 ACH"}="1H8  ACH"; $resn_exch{" H82 ACH"}="2H8  ACH"; $resn_exch{" H83 ACH"}="3H8  ACH";
$resn_exch{" H91 ACH"}="1H9  ACH"; $resn_exch{" H92 ACH"}="2H9  ACH"; $resn_exch{" H93 ACH"}="3H9  ACH";
$resn_exch{"H101 ACH"}="1H10 ACH"; $resn_exch{"H102 ACH"}="2H10 ACH"; $resn_exch{"H103 ACH"}="3H10 ACH";
$resn_exch{" H61 ACI"}="1H6  ACI"; $resn_exch{" H62 ACI"}="2H6  ACI"; $resn_exch{"HN11 ACI"}="1HN1 ACI";
$resn_exch{"HN12 ACI"}="2HN1 ACI"; $resn_exch{" H5' ACK"}=" H5* ACK"; $resn_exch{" H4' ACK"}=" H4* ACK";
$resn_exch{"H3'1 ACK"}="1H3* ACK"; $resn_exch{"H3'2 ACK"}="2H3* ACK"; $resn_exch{" H3' ACK"}=" H3* ACK";
$resn_exch{" H1' ACK"}=" H1* ACK"; $resn_exch{"H6N1 ACK"}="1H6N ACK"; $resn_exch{"H6N2 ACK"}="2H6N ACK";
$resn_exch{" H2' ACK"}=" H2* ACK"; $resn_exch{" H   ACL"}="1HN  ACL"; $resn_exch{" H2  ACL"}="2HN  ACL";
$resn_exch{" HB2 ACL"}="1HB  ACL"; $resn_exch{" HB3 ACL"}="2HB  ACL"; $resn_exch{" HG2 ACL"}="1HG  ACL";
$resn_exch{" HG3 ACL"}="2HG  ACL"; $resn_exch{" HD2 ACL"}="1HD  ACL"; $resn_exch{" HD3 ACL"}="2HD  ACL";
$resn_exch{" HE  ACL"}=" HNE ACL"; $resn_exch{" HH1 ACL"}=" HN1 ACL"; $resn_exch{"HH21 ACL"}="1HN2 ACL";
$resn_exch{"HH22 ACL"}="2HN2 ACL"; $resn_exch{" HM1 ACL"}="1HM  ACL"; $resn_exch{" HM2 ACL"}="2HM  ACL";
$resn_exch{" HN1 ACM"}="1HN  ACM"; $resn_exch{" HN2 ACM"}="2HN  ACM"; $resn_exch{" H21 ACM"}="1H2  ACM";
$resn_exch{" H22 ACM"}="2H2  ACM"; $resn_exch{" H23 ACM"}="3H2  ACM"; $resn_exch{" H11 ACN"}="1H1  ACN";
$resn_exch{" H12 ACN"}="2H1  ACN"; $resn_exch{" H13 ACN"}="3H1  ACN"; $resn_exch{" H21 ACN"}="1H2  ACN";
$resn_exch{" H22 ACN"}="2H2  ACN"; $resn_exch{" H23 ACN"}="3H2  ACN"; $resn_exch{" H2A ACO"}=" AH2 ACO";
$resn_exch{"H61A ACO"}="AH61 ACO"; $resn_exch{"H62A ACO"}="AH62 ACO"; $resn_exch{" H8A ACO"}=" AH8 ACO";
$resn_exch{" H1B ACO"}="AH1* ACO"; $resn_exch{" H2B ACO"}="AH2* ACO"; $resn_exch{"HO2A ACO"}="AHO2 ACO";
$resn_exch{" H3B ACO"}="AH3* ACO"; $resn_exch{"HOA8 ACO"}="8HOA ACO"; $resn_exch{"HOA9 ACO"}="9HOA ACO";
$resn_exch{" H4B ACO"}="AH4* ACO"; $resn_exch{"H51A ACO"}="AH51 ACO"; $resn_exch{"H52A ACO"}="AH52 ACO";
$resn_exch{"HOA2 ACO"}="2HOA ACO"; $resn_exch{"HOA5 ACO"}="5HOA ACO"; $resn_exch{"H121 ACO"}="1H12 ACO";
$resn_exch{"H122 ACO"}="2H12 ACO"; $resn_exch{"H131 ACO"}="1H13 ACO"; $resn_exch{"H132 ACO"}="2H13 ACO";
$resn_exch{"H133 ACO"}="3H13 ACO"; $resn_exch{"H141 ACO"}="1H14 ACO"; $resn_exch{"H142 ACO"}="2H14 ACO";
$resn_exch{"H143 ACO"}="3H14 ACO"; $resn_exch{" H71 ACO"}="1H7  ACO"; $resn_exch{" H72 ACO"}="2H7  ACO";
$resn_exch{" H61 ACO"}="1H6  ACO"; $resn_exch{" H62 ACO"}="2H6  ACO"; $resn_exch{" H31 ACO"}="1H3  ACO";
$resn_exch{" H32 ACO"}="2H3  ACO"; $resn_exch{" H21 ACO"}="1H2  ACO"; $resn_exch{" H22 ACO"}="2H2  ACO";
$resn_exch{"HH31 ACO"}="1HH3 ACO"; $resn_exch{"HH32 ACO"}="2HH3 ACO"; $resn_exch{"HH33 ACO"}="3HH3 ACO";
$resn_exch{"HOG2 ACP"}="2HOG ACP"; $resn_exch{"HOG3 ACP"}="3HOG ACP"; $resn_exch{"HOB2 ACP"}="2HOB ACP";
$resn_exch{"H3B1 ACP"}="1H3B ACP"; $resn_exch{"H3B2 ACP"}="2H3B ACP"; $resn_exch{"HOA2 ACP"}="2HOA ACP";
$resn_exch{"H5'1 ACP"}="1H5* ACP"; $resn_exch{"H5'2 ACP"}="2H5* ACP"; $resn_exch{" H4' ACP"}=" H4* ACP";
$resn_exch{" H3' ACP"}=" H3* ACP"; $resn_exch{"HO3' ACP"}="*HO3 ACP"; $resn_exch{" H2' ACP"}=" H2* ACP";
$resn_exch{"HO2' ACP"}="*HO2 ACP"; $resn_exch{" H1' ACP"}=" H1* ACP"; $resn_exch{"HN61 ACP"}="1HN6 ACP";
$resn_exch{"HN62 ACP"}="2HN6 ACP"; $resn_exch{"HOG2 ACQ"}="2HOG ACQ"; $resn_exch{"HOB2 ACQ"}="2HOB ACQ";
$resn_exch{"H3B1 ACQ"}="1H3B ACQ"; $resn_exch{"H3B2 ACQ"}="2H3B ACQ"; $resn_exch{"HOA2 ACQ"}="2HOA ACQ";
$resn_exch{"H5'1 ACQ"}="1H5* ACQ"; $resn_exch{"H5'2 ACQ"}="2H5* ACQ"; $resn_exch{" H4' ACQ"}=" H4* ACQ";
$resn_exch{" H3' ACQ"}=" H3* ACQ"; $resn_exch{"HO3' ACQ"}="*HO3 ACQ"; $resn_exch{" H2' ACQ"}=" H2* ACQ";
$resn_exch{"HO2' ACQ"}="*HO2 ACQ"; $resn_exch{" H1' ACQ"}=" H1* ACQ"; $resn_exch{"HN61 ACQ"}="1HN6 ACQ";
$resn_exch{"HN62 ACQ"}="2HN6 ACQ"; $resn_exch{"HOD2 ACQ"}="2HOD ACQ"; $resn_exch{"HOD3 ACQ"}="3HOD ACQ";
$resn_exch{"H6A1 ACR"}="1H6A ACR"; $resn_exch{"H6A2 ACR"}="2H6A ACR"; $resn_exch{"HOA2 ACR"}="2HOA ACR";
$resn_exch{"HOA3 ACR"}="3HOA ACR"; $resn_exch{"HOA4 ACR"}="4HOA ACR"; $resn_exch{"HOA6 ACR"}="6HOA ACR";
$resn_exch{"H6B1 ACR"}="1H6B ACR"; $resn_exch{"H6B2 ACR"}="2H6B ACR"; $resn_exch{"H6B3 ACR"}="3H6B ACR";
$resn_exch{"HOB2 ACR"}="2HOB ACR"; $resn_exch{"HOB3 ACR"}="3HOB ACR"; $resn_exch{"H6C1 ACR"}="1H6C ACR";
$resn_exch{"H6C2 ACR"}="2H6C ACR"; $resn_exch{"HOC2 ACR"}="2HOC ACR"; $resn_exch{"HOC3 ACR"}="3HOC ACR";
$resn_exch{"HOC6 ACR"}="6HOC ACR"; $resn_exch{"H6D1 ACR"}="1H6D ACR"; $resn_exch{"H6D2 ACR"}="2H6D ACR";
$resn_exch{"HOD1 ACR"}="1HOD ACR"; $resn_exch{"HOD2 ACR"}="2HOD ACR"; $resn_exch{"HOD3 ACR"}="3HOD ACR";
$resn_exch{"HOD6 ACR"}="6HOD ACR"; $resn_exch{" H12 ACS"}="2H1  ACS"; $resn_exch{" H11 ACS"}="1H1  ACS";
$resn_exch{" H62 ACS"}="2H6  ACS"; $resn_exch{" H61 ACS"}="1H6  ACS"; $resn_exch{" H72 ACS"}="2H7  ACS";
$resn_exch{" H71 ACS"}="1H7  ACS"; $resn_exch{"H102 ACS"}="2H10 ACS"; $resn_exch{"H101 ACS"}="1H10 ACS";
$resn_exch{"H242 ACS"}="2H24 ACS"; $resn_exch{"H241 ACS"}="1H24 ACS"; $resn_exch{"H293 ACS"}="3H29 ACS";
$resn_exch{"H292 ACS"}="2H29 ACS"; $resn_exch{"H291 ACS"}="1H29 ACS"; $resn_exch{" H1  ACT"}="1H   ACT";
$resn_exch{" H2  ACT"}="2H   ACT"; $resn_exch{" H3  ACT"}="3H   ACT"; $resn_exch{" H31 ACV"}="1H3  ACV";
$resn_exch{"H32A ACV"}="2H3  ACV"; $resn_exch{" H41 ACV"}="1H4  ACV"; $resn_exch{" H42 ACV"}="2H4  ACV";
$resn_exch{" H71 ACV"}="1H7  ACV"; $resn_exch{" H72 ACV"}="2H7  ACV"; $resn_exch{"HNE1 ACV"}="1HNE ACV";
$resn_exch{"HNE2 ACV"}="2HNE ACV"; $resn_exch{"H161 ACV"}="1H16 ACV"; $resn_exch{"H162 ACV"}="2H16 ACV";
$resn_exch{"H331 ACV"}="1H33 ACV"; $resn_exch{"H332 ACV"}="2H33 ACV"; $resn_exch{"H333 ACV"}="3H33 ACV";
$resn_exch{"H371 ACV"}="1H37 ACV"; $resn_exch{"H372 ACV"}="2H37 ACV"; $resn_exch{"H373 ACV"}="3H37 ACV";
$resn_exch{"H3C1 ACW"}="1H3C ACW"; $resn_exch{"H3C2 ACW"}="2H3C ACW"; $resn_exch{"H4C1 ACW"}="1H4C ACW";
$resn_exch{"H4C2 ACW"}="2H4C ACW"; $resn_exch{"H7C1 ACW"}="1H7C ACW"; $resn_exch{"H7C2 ACW"}="2H7C ACW";
$resn_exch{"H141 ACW"}="1H14 ACW"; $resn_exch{"H142 ACW"}="2H14 ACW"; $resn_exch{"H161 ACW"}="1H16 ACW";
$resn_exch{"H162 ACW"}="2H16 ACW"; $resn_exch{"H331 ACW"}="1H33 ACW"; $resn_exch{"H332 ACW"}="2H33 ACW";
$resn_exch{"H371 ACW"}="1H37 ACW"; $resn_exch{"H372 ACW"}="2H37 ACW"; $resn_exch{"H373 ACW"}="3H37 ACW";
$resn_exch{"H401 ACW"}="1H40 ACW"; $resn_exch{"H402 ACW"}="2H40 ACW"; $resn_exch{"H6A1 ACX"}="1H6A ACX";
$resn_exch{"H6A2 ACX"}="2H6A ACX"; $resn_exch{"HO2A ACX"}="AHO2 ACX"; $resn_exch{"HO3A ACX"}="AHO3 ACX";
$resn_exch{"HO6A ACX"}="AHO6 ACX"; $resn_exch{"H6B1 ACX"}="1H6B ACX"; $resn_exch{"H6B2 ACX"}="2H6B ACX";
$resn_exch{"HO2B ACX"}="BHO2 ACX"; $resn_exch{"HO3B ACX"}="BHO3 ACX"; $resn_exch{"HO6B ACX"}="BHO6 ACX";
$resn_exch{"H6C1 ACX"}="1H6C ACX"; $resn_exch{"H6C2 ACX"}="2H6C ACX"; $resn_exch{"HO2C ACX"}="CHO2 ACX";
$resn_exch{"HO3C ACX"}="CHO3 ACX"; $resn_exch{"HO6C ACX"}="CHO6 ACX"; $resn_exch{"H6D1 ACX"}="1H6D ACX";
$resn_exch{"H6D2 ACX"}="2H6D ACX"; $resn_exch{"HO2D ACX"}="DHO2 ACX"; $resn_exch{"HO3D ACX"}="DHO3 ACX";
$resn_exch{"HO6D ACX"}="DHO6 ACX"; $resn_exch{"H6E1 ACX"}="1H6E ACX"; $resn_exch{"H6E2 ACX"}="2H6E ACX";
$resn_exch{"HO2E ACX"}="EHO2 ACX"; $resn_exch{"HO3E ACX"}="EHO3 ACX"; $resn_exch{"HO6E ACX"}="EHO6 ACX";
$resn_exch{"H6F1 ACX"}="1H6F ACX"; $resn_exch{"H6F2 ACX"}="2H6F ACX"; $resn_exch{"HO2F ACX"}="FHO2 ACX";
$resn_exch{"HO3F ACX"}="FHO3 ACX"; $resn_exch{"HO6F ACX"}="FHO6 ACX"; $resn_exch{" H1  ACY"}="1H   ACY";
$resn_exch{" H2  ACY"}="2H   ACY"; $resn_exch{" H3  ACY"}="3H   ACY"; $resn_exch{"HNH1 ACZ"}="1HNH ACZ";
$resn_exch{"HNH2 ACZ"}="2HNH ACZ"; $resn_exch{"HE22 ACZ"}="2HE2 ACZ"; $resn_exch{"HD22 ACZ"}="2HD2 ACZ";
$resn_exch{"HD11 ACZ"}="1HD1 ACZ"; $resn_exch{"HE11 ACZ"}="1HE1 ACZ"; $resn_exch{"HCX1 ACZ"}="1HCX ACZ";
$resn_exch{"HCX2 ACZ"}="2HCX ACZ"; $resn_exch{"HCB1 ACZ"}="1HCB ACZ"; $resn_exch{"HCB2 ACZ"}="2HCB ACZ";
$resn_exch{" HN1 ACZ"}="1HN  ACZ"; $resn_exch{" HN2 ACZ"}="2HN  ACZ"; $resn_exch{"HOP2 AD2"}="2HOP AD2";
$resn_exch{"HOP5 AD2"}="5HOP AD2"; $resn_exch{"HOP6 AD2"}="6HOP AD2"; $resn_exch{" H61 AD2"}="1H6  AD2";
$resn_exch{" H62 AD2"}="2H6  AD2"; $resn_exch{"H2'1 AD2"}="1H2' AD2"; $resn_exch{"H2'2 AD2"}="2H2' AD2";
$resn_exch{"H5'1 AD2"}="1H5' AD2"; $resn_exch{"H5'2 AD2"}="2H5' AD2"; $resn_exch{"HOP3 AD2"}="3HOP AD2";
$resn_exch{"HO5' AD3"}="*HO5 AD3"; $resn_exch{"H5'1 AD3"}="1H5* AD3"; $resn_exch{"H5'2 AD3"}="2H5* AD3";
$resn_exch{" H4' AD3"}=" H4* AD3"; $resn_exch{" H3' AD3"}=" H3* AD3"; $resn_exch{"HO3' AD3"}="*HO3 AD3";
$resn_exch{" H2' AD3"}=" H2* AD3"; $resn_exch{"HO2' AD3"}="*HO2 AD3"; $resn_exch{" H1' AD3"}=" H1* AD3";
$resn_exch{"HN61 AD3"}="1HN6 AD3"; $resn_exch{"HN62 AD3"}="2HN6 AD3"; $resn_exch{"H341 AD4"}="1H34 AD4";
$resn_exch{"H342 AD4"}="2H34 AD4"; $resn_exch{"H343 AD4"}="3H34 AD4"; $resn_exch{"H9C1 AD4"}="1H9C AD4";
$resn_exch{"H9C2 AD4"}="2H9C AD4"; $resn_exch{"H141 AD4"}="1H14 AD4"; $resn_exch{"H142 AD4"}="2H14 AD4";
$resn_exch{"H151 AD4"}="1H15 AD4"; $resn_exch{"H152 AD4"}="2H15 AD4"; $resn_exch{"H161 AD4"}="1H16 AD4";
$resn_exch{"H162 AD4"}="2H16 AD4"; $resn_exch{"H171 AD4"}="1H17 AD4"; $resn_exch{"H172 AD4"}="2H17 AD4";
$resn_exch{"H181 AD4"}="1H18 AD4"; $resn_exch{"H182 AD4"}="2H18 AD4"; $resn_exch{"H191 AD4"}="1H19 AD4";
$resn_exch{"H192 AD4"}="2H19 AD4"; $resn_exch{"H201 AD4"}="1H20 AD4"; $resn_exch{"H202 AD4"}="2H20 AD4";
$resn_exch{"H203 AD4"}="3H20 AD4"; $resn_exch{"H231 AD4"}="1H23 AD4"; $resn_exch{"H232 AD4"}="2H23 AD4";
$resn_exch{"H233 AD4"}="3H23 AD4"; $resn_exch{"H251 AD4"}="1H25 AD4"; $resn_exch{"H252 AD4"}="2H25 AD4";
$resn_exch{"H253 AD4"}="3H25 AD4"; $resn_exch{"H261 AD4"}="1H26 AD4"; $resn_exch{"H262 AD4"}="2H26 AD4";
$resn_exch{"H263 AD4"}="3H26 AD4"; $resn_exch{"H311 AD4"}="1H31 AD4"; $resn_exch{"H312 AD4"}="2H31 AD4";
$resn_exch{"H313 AD4"}="3H31 AD4"; $resn_exch{"H331 AD4"}="1H33 AD4"; $resn_exch{"H332 AD4"}="2H33 AD4";
$resn_exch{"H333 AD4"}="3H33 AD4"; $resn_exch{"H281 AD4"}="1H28 AD4"; $resn_exch{"H282 AD4"}="2H28 AD4";
$resn_exch{"H291 AD4"}="1H29 AD4"; $resn_exch{"H292 AD4"}="2H29 AD4"; $resn_exch{"H301 AD4"}="1H30 AD4";
$resn_exch{"H302 AD4"}="2H30 AD4"; $resn_exch{"H571 AD4"}="1H57 AD4"; $resn_exch{"H572 AD4"}="2H57 AD4";
$resn_exch{"H573 AD4"}="3H57 AD4"; $resn_exch{"H551 AD4"}="1H55 AD4"; $resn_exch{"H552 AD4"}="2H55 AD4";
$resn_exch{"H553 AD4"}="3H55 AD4"; $resn_exch{"H541 AD4"}="1H54 AD4"; $resn_exch{"H542 AD4"}="2H54 AD4";
$resn_exch{"H543 AD4"}="3H54 AD4"; $resn_exch{"H521 AD4"}="1H52 AD4"; $resn_exch{"H522 AD4"}="2H52 AD4";
$resn_exch{"H511 AD4"}="1H51 AD4"; $resn_exch{"H512 AD4"}="2H51 AD4"; $resn_exch{"H501 AD4"}="1H50 AD4";
$resn_exch{"H502 AD4"}="2H50 AD4"; $resn_exch{"H491 AD4"}="1H49 AD4"; $resn_exch{"H492 AD4"}="2H49 AD4";
$resn_exch{"H481 AD4"}="1H48 AD4"; $resn_exch{"H482 AD4"}="2H48 AD4"; $resn_exch{"H471 AD4"}="1H47 AD4";
$resn_exch{"H472 AD4"}="2H47 AD4"; $resn_exch{"H461 AD4"}="1H46 AD4"; $resn_exch{"H462 AD4"}="2H46 AD4";
$resn_exch{"H451 AD4"}="1H45 AD4"; $resn_exch{"H452 AD4"}="2H45 AD4"; $resn_exch{"HAA1 AD6"}="1HAA AD6";
$resn_exch{"HAA2 AD6"}="2HAA AD6"; $resn_exch{"HAA3 AD6"}="3HAA AD6"; $resn_exch{"HN41 ADB"}="1HN4 ADB";
$resn_exch{"HN42 ADB"}="2HN4 ADB"; $resn_exch{"HN61 ADC"}="1HN6 ADC"; $resn_exch{"HN62 ADC"}="2HN6 ADC";
$resn_exch{"HM21 ADD"}="1HM2 ADD"; $resn_exch{"HM22 ADD"}="2HM2 ADD"; $resn_exch{"HM23 ADD"}="3HM2 ADD";
$resn_exch{"HN31 ADD"}="1HN3 ADD"; $resn_exch{"HN32 ADD"}="2HN3 ADD"; $resn_exch{" H41 ADD"}="1H4  ADD";
$resn_exch{" H42 ADD"}="2H4  ADD"; $resn_exch{" H51 ADD"}="1H5  ADD"; $resn_exch{" H52 ADD"}="2H5  ADD";
$resn_exch{"HM61 ADD"}="1HM6 ADD"; $resn_exch{"HM62 ADD"}="2HM6 ADD"; $resn_exch{"HM63 ADD"}="3HM6 ADD";
$resn_exch{" H71 ADD"}="1H7  ADD"; $resn_exch{" H72 ADD"}="2H7  ADD"; $resn_exch{"HM81 ADD"}="1HM8 ADD";
$resn_exch{"HM82 ADD"}="2HM8 ADD"; $resn_exch{"HM83 ADD"}="3HM8 ADD"; $resn_exch{"HM91 ADD"}="1HM9 ADD";
$resn_exch{"HM92 ADD"}="2HM9 ADD"; $resn_exch{"HM93 ADD"}="3HM9 ADD"; $resn_exch{" H'1 ADD"}="1H'  ADD";
$resn_exch{" H'2 ADD"}="2H'  ADD"; $resn_exch{"HN61 ADE"}="1HN6 ADE"; $resn_exch{"HN62 ADE"}="2HN6 ADE";
$resn_exch{" H41 ADF"}="1H4  ADF"; $resn_exch{" H42 ADF"}="2H4  ADF"; $resn_exch{" H51 ADF"}="1H5  ADF";
$resn_exch{" H52 ADF"}="2H5  ADF"; $resn_exch{" H91 ADF"}="1H9  ADF"; $resn_exch{" H92 ADF"}="2H9  ADF";
$resn_exch{"H131 ADF"}="1H13 ADF"; $resn_exch{"H132 ADF"}="2H13 ADF"; $resn_exch{"H141 ADF"}="1H14 ADF";
$resn_exch{"H142 ADF"}="2H14 ADF"; $resn_exch{"H151 ADF"}="1H15 ADF"; $resn_exch{"H152 ADF"}="2H15 ADF";
$resn_exch{"H211 ADF"}="1H21 ADF"; $resn_exch{"H212 ADF"}="2H21 ADF"; $resn_exch{"H221 ADF"}="1H22 ADF";
$resn_exch{"H222 ADF"}="2H22 ADF"; $resn_exch{"H251 ADF"}="1H25 ADF"; $resn_exch{"H252 ADF"}="2H25 ADF";
$resn_exch{"H253 ADF"}="3H25 ADF"; $resn_exch{"H261 ADF"}="1H26 ADF"; $resn_exch{"H262 ADF"}="2H26 ADF";
$resn_exch{"H263 ADF"}="3H26 ADF"; $resn_exch{"H271 ADF"}="1H27 ADF"; $resn_exch{"H272 ADF"}="2H27 ADF";
$resn_exch{"H273 ADF"}="3H27 ADF"; $resn_exch{" H31 ADG"}="1H3  ADG"; $resn_exch{" H32 ADG"}="2H3  ADG";
$resn_exch{" H33 ADG"}="3H3  ADG"; $resn_exch{" H61 ADG"}="1H6  ADG"; $resn_exch{" H62 ADG"}="2H6  ADG";
$resn_exch{" H41 ADH"}="1H4  ADH"; $resn_exch{" H42 ADH"}="2H4  ADH"; $resn_exch{" H61 ADH"}="1H6  ADH";
$resn_exch{" H62 ADH"}="2H6  ADH"; $resn_exch{" H71 ADH"}="1H7  ADH"; $resn_exch{"HN12 ADH"}="2HN1 ADH";
$resn_exch{"HN22 ADH"}="2HN2 ADH"; $resn_exch{"HOB2 ADI"}="2HOB ADI"; $resn_exch{"HOB3 ADI"}="3HOB ADI";
$resn_exch{"HOA2 ADI"}="2HOA ADI"; $resn_exch{"H5'1 ADI"}="1H5* ADI"; $resn_exch{"H5'2 ADI"}="2H5* ADI";
$resn_exch{" H4' ADI"}=" H4* ADI"; $resn_exch{"H3'1 ADI"}="1H3* ADI"; $resn_exch{"H3'2 ADI"}="2H3* ADI";
$resn_exch{"H2'1 ADI"}="1H2* ADI"; $resn_exch{"H2'2 ADI"}="2H2* ADI"; $resn_exch{" H1' ADI"}=" H1* ADI";
$resn_exch{"HN61 ADI"}="1HN6 ADI"; $resn_exch{"HN62 ADI"}="2HN6 ADI"; $resn_exch{"H5'1 ADJ"}="1H5' ADJ";
$resn_exch{"H5'2 ADJ"}="2H5' ADJ"; $resn_exch{"HN61 ADJ"}="1HN6 ADJ"; $resn_exch{"HN62 ADJ"}="2HN6 ADJ";
$resn_exch{"HO'2 ADJ"}="2HO' ADJ"; $resn_exch{"HO'3 ADJ"}="3HO' ADJ"; $resn_exch{"HON7 ADJ"}="7HON ADJ";
$resn_exch{"H5R1 ADJ"}="1H5R ADJ"; $resn_exch{"H5R2 ADJ"}="2H5R ADJ"; $resn_exch{"HOR2 ADJ"}="2HOR ADJ";
$resn_exch{"HOR3 ADJ"}="3HOR ADJ"; $resn_exch{"H5S1 ADJ"}="1H5S ADJ"; $resn_exch{"H5S2 ADJ"}="2H5S ADJ";
$resn_exch{"HX61 ADJ"}="1HX6 ADJ"; $resn_exch{"HX62 ADJ"}="2HX6 ADJ"; $resn_exch{"HOS2 ADJ"}="2HOS ADJ";
$resn_exch{"HOS3 ADJ"}="3HOS ADJ"; $resn_exch{"HN61 ADK"}="1HN6 ADK"; $resn_exch{"HN62 ADK"}="2HN6 ADK";
$resn_exch{"H3A1 ADK"}="1H3A ADK"; $resn_exch{"H3A2 ADK"}="2H3A ADK"; $resn_exch{"H3A3 ADK"}="3H3A ADK";
$resn_exch{"H101 ADL"}="1H10 ADL"; $resn_exch{"H102 ADL"}="2H10 ADL"; $resn_exch{"H151 ADL"}="1H15 ADL";
$resn_exch{"H152 ADL"}="2H15 ADL"; $resn_exch{" H21 ADM"}="1H2  ADM"; $resn_exch{" H22 ADM"}="2H2  ADM";
$resn_exch{" H41 ADM"}="1H4  ADM"; $resn_exch{" H42 ADM"}="2H4  ADM"; $resn_exch{" H61 ADM"}="1H6  ADM";
$resn_exch{" H62 ADM"}="2H6  ADM"; $resn_exch{" H81 ADM"}="1H8  ADM"; $resn_exch{" H82 ADM"}="2H8  ADM";
$resn_exch{" H91 ADM"}="1H9  ADM"; $resn_exch{" H92 ADM"}="2H9  ADM"; $resn_exch{"H101 ADM"}="1H10 ADM";
$resn_exch{"H102 ADM"}="2H10 ADM"; $resn_exch{"HO5' ADN"}="*HO5 ADN"; $resn_exch{"H5'1 ADN"}="1H5* ADN";
$resn_exch{"H5'2 ADN"}="2H5* ADN"; $resn_exch{" H4' ADN"}=" H4* ADN"; $resn_exch{" H3' ADN"}=" H3* ADN";
$resn_exch{"HO3' ADN"}="*HO3 ADN"; $resn_exch{" H2' ADN"}=" H2* ADN"; $resn_exch{"HO2' ADN"}="*HO2 ADN";
$resn_exch{" H1' ADN"}=" H1* ADN"; $resn_exch{"HN61 ADN"}="1HN6 ADN"; $resn_exch{"HN62 ADN"}="2HN6 ADN";
$resn_exch{" H41 ADO"}="1H4  ADO"; $resn_exch{" H42 ADO"}="2H4  ADO"; $resn_exch{" H61 ADO"}="1H6  ADO";
$resn_exch{" H62 ADO"}="2H6  ADO"; $resn_exch{" H81 ADO"}="1H8  ADO"; $resn_exch{" H82 ADO"}="2H8  ADO";
$resn_exch{" H91 ADO"}="1H9  ADO"; $resn_exch{" H92 ADO"}="2H9  ADO"; $resn_exch{"H101 ADO"}="1H10 ADO";
$resn_exch{"H102 ADO"}="2H10 ADO"; $resn_exch{"HOB2 ADP"}="2HOB ADP"; $resn_exch{"HOB3 ADP"}="3HOB ADP";
$resn_exch{"HOA2 ADP"}="2HOA ADP"; $resn_exch{"H5'1 ADP"}="1H5* ADP"; $resn_exch{"H5'2 ADP"}="2H5* ADP";
$resn_exch{" H4' ADP"}=" H4* ADP"; $resn_exch{" H3' ADP"}=" H3* ADP"; $resn_exch{"HO3' ADP"}="*HO3 ADP";
$resn_exch{" H2' ADP"}=" H2* ADP"; $resn_exch{"HO2' ADP"}="*HO2 ADP"; $resn_exch{" H1' ADP"}=" H1* ADP";
$resn_exch{"HN61 ADP"}="1HN6 ADP"; $resn_exch{"HN62 ADP"}="2HN6 ADP"; $resn_exch{"HOB2 ADQ"}="2HOB ADQ";
$resn_exch{"HOA2 ADQ"}="2HOA ADQ"; $resn_exch{"H5'1 ADQ"}="1H5* ADQ"; $resn_exch{"H5'2 ADQ"}="2H5* ADQ";
$resn_exch{" H4D ADQ"}=" H4* ADQ"; $resn_exch{" H3D ADQ"}=" H3* ADQ"; $resn_exch{"HO3' ADQ"}="*HO3 ADQ";
$resn_exch{" H2D ADQ"}=" H2* ADQ"; $resn_exch{"HO2' ADQ"}="*HO2 ADQ"; $resn_exch{" H1D ADQ"}=" H1* ADQ";
$resn_exch{"HN61 ADQ"}="1HN6 ADQ"; $resn_exch{"HN62 ADQ"}="2HN6 ADQ"; $resn_exch{"H6'1 ADQ"}="1H6' ADQ";
$resn_exch{"H6'2 ADQ"}="2H6' ADQ"; $resn_exch{"HO2A ADQ"}="'HO2 ADQ"; $resn_exch{"HO3A ADQ"}="'HO3 ADQ";
$resn_exch{"HO4' ADQ"}="'HO4 ADQ"; $resn_exch{"HO6' ADQ"}="'HO6 ADQ"; $resn_exch{" H11 ADR"}="1H1  ADR";
$resn_exch{" H12 ADR"}="2H1  ADR"; $resn_exch{" H21 ADR"}="1H2  ADR"; $resn_exch{" H22 ADR"}="2H2  ADR";
$resn_exch{" H31 ADR"}="1H3  ADR"; $resn_exch{" H32 ADR"}="2H3  ADR"; $resn_exch{" H51 ADR"}="1H5  ADR";
$resn_exch{" H52 ADR"}="2H5  ADR"; $resn_exch{" H61 ADR"}="1H6  ADR"; $resn_exch{" H62 ADR"}="2H6  ADR";
$resn_exch{"HSP2 ADS"}="2HSP ADS"; $resn_exch{"HSP3 ADS"}="3HSP ADS"; $resn_exch{"H5'1 ADS"}="1H5* ADS";
$resn_exch{"H5'2 ADS"}="2H5* ADS"; $resn_exch{" H4' ADS"}=" H4* ADS"; $resn_exch{" H3' ADS"}=" H3* ADS";
$resn_exch{"HO3' ADS"}="*HO3 ADS"; $resn_exch{" H2' ADS"}=" H2* ADS"; $resn_exch{"HO2' ADS"}="*HO2 ADS";
$resn_exch{" H1' ADS"}=" H1* ADS"; $resn_exch{"HN61 ADS"}="1HN6 ADS"; $resn_exch{"HN62 ADS"}="2HN6 ADS";
$resn_exch{"HM51 ADT"}="1HM5 ADT"; $resn_exch{"HM52 ADT"}="2HM5 ADT"; $resn_exch{"HM53 ADT"}="3HM5 ADT";
$resn_exch{" H1' ADT"}=" H1* ADT"; $resn_exch{" H4' ADT"}=" H4* ADT"; $resn_exch{"H5'1 ADT"}="1H5* ADT";
$resn_exch{"H5'2 ADT"}="2H5* ADT"; $resn_exch{"HO5' ADT"}="*HO5 ADT"; $resn_exch{" H3' ADT"}=" H3* ADT";
$resn_exch{"HN3' ADT"}="*HN3 ADT"; $resn_exch{"H9'1 ADT"}="1H9* ADT"; $resn_exch{"H9'2 ADT"}="2H9* ADT";
$resn_exch{"H9'3 ADT"}="3H9* ADT"; $resn_exch{"H2'1 ADT"}="1H2* ADT"; $resn_exch{"H2'2 ADT"}="2H2* ADT";
$resn_exch{" H1' ADU"}=" H1* ADU"; $resn_exch{" H4' ADU"}=" H4* ADU"; $resn_exch{"H5'1 ADU"}="1H5* ADU";
$resn_exch{"H5'2 ADU"}="2H5* ADU"; $resn_exch{"HO5' ADU"}="*HO5 ADU"; $resn_exch{" H3' ADU"}=" H3* ADU";
$resn_exch{"HN3' ADU"}="*HN3 ADU"; $resn_exch{"H9'1 ADU"}="1H9* ADU"; $resn_exch{"H9'2 ADU"}="2H9* ADU";
$resn_exch{"H9'3 ADU"}="3H9* ADU"; $resn_exch{" H2' ADU"}=" H2* ADU"; $resn_exch{"HO2' ADU"}="*HO2 ADU";
$resn_exch{"H5'1 ADV"}="1H5* ADV"; $resn_exch{"H5'2 ADV"}="2H5* ADV"; $resn_exch{" H'4 ADV"}="4H*  ADV";
$resn_exch{" H'3 ADV"}="3H*  ADV"; $resn_exch{"HO'3 ADV"}="3HO* ADV"; $resn_exch{" H'2 ADV"}="2H*  ADV";
$resn_exch{"HO'2 ADV"}="2HO* ADV"; $resn_exch{" H'1 ADV"}="1H*  ADV"; $resn_exch{" H61 ADV"}="1H6  ADV";
$resn_exch{" H62 ADV"}="2H6  ADV"; $resn_exch{" HX1 ADV"}="1HX  ADV"; $resn_exch{" HX2 ADV"}="2HX  ADV";
$resn_exch{"H5R1 ADV"}="1H5R ADV"; $resn_exch{"H5R2 ADV"}="2H5R ADV"; $resn_exch{" HR4 ADV"}="4HR  ADV";
$resn_exch{" HR3 ADV"}="3HR  ADV"; $resn_exch{"HOR3 ADV"}="3HOR ADV"; $resn_exch{" HR2 ADV"}="2HR  ADV";
$resn_exch{"HOR2 ADV"}="2HOR ADV"; $resn_exch{" HR1 ADV"}="1HR  ADV"; $resn_exch{"HOR1 ADV"}="1HOR ADV";
$resn_exch{"HOA2 ADV"}="2HOA ADV"; $resn_exch{"HOB2 ADV"}="2HOB ADV"; $resn_exch{"HOB2 ADW"}="2HOB ADW";
$resn_exch{"HOB3 ADW"}="3HOB ADW"; $resn_exch{"HOA2 ADW"}="2HOA ADW"; $resn_exch{"H5'1 ADW"}="1H5* ADW";
$resn_exch{"H5'2 ADW"}="2H5* ADW"; $resn_exch{" H4' ADW"}=" H4* ADW"; $resn_exch{" H3' ADW"}=" H3* ADW";
$resn_exch{"HO3' ADW"}="*HO3 ADW"; $resn_exch{" H2' ADW"}=" H2* ADW"; $resn_exch{"HO2' ADW"}="*HO2 ADW";
$resn_exch{" H1' ADW"}=" H1* ADW"; $resn_exch{"HN61 ADW"}="1HN6 ADW"; $resn_exch{"HN62 ADW"}="2HN6 ADW";
$resn_exch{"HOB3 ADX"}="3HOB ADX"; $resn_exch{"HOA2 ADX"}="2HOA ADX"; $resn_exch{" H5' ADX"}="1H5* ADX";
$resn_exch{"H5'' ADX"}="2H5* ADX"; $resn_exch{" H4' ADX"}=" H4* ADX"; $resn_exch{" H3' ADX"}=" H3* ADX";
$resn_exch{"HO3' ADX"}="*HO3 ADX"; $resn_exch{" H2' ADX"}=" H2* ADX"; $resn_exch{"HO2' ADX"}="*HO2 ADX";
$resn_exch{" H1' ADX"}=" H1* ADX"; $resn_exch{"HN61 ADX"}="1HN6 ADX"; $resn_exch{"HN62 ADX"}="2HN6 ADX";
$resn_exch{"HO5' ADY"}="*HO5 ADY"; $resn_exch{"H5'1 ADY"}="1H5* ADY"; $resn_exch{"H5'2 ADY"}="2H5* ADY";
$resn_exch{" H4' ADY"}=" H4* ADY"; $resn_exch{" H2' ADY"}=" H2* ADY"; $resn_exch{"HO2' ADY"}="*HO2 ADY";
$resn_exch{" H1' ADY"}=" H1* ADY"; $resn_exch{"HN61 ADY"}="1HN6 ADY"; $resn_exch{"HN62 ADY"}="2HN6 ADY";
$resn_exch{"H6N1 ADZ"}="1H6N ADZ"; $resn_exch{"H6N2 ADZ"}="2H6N ADZ"; $resn_exch{"H1'1 ADZ"}="1H1* ADZ";
$resn_exch{"H1'2 ADZ"}="2H1* ADZ"; $resn_exch{"H1'3 ADZ"}="3H1* ADZ"; $resn_exch{" H21 AE1"}="1H2  AE1";
$resn_exch{" H22 AE1"}="2H2  AE1"; $resn_exch{" HN1 AE1"}="1HN  AE1"; $resn_exch{" HN2 AE1"}="2HN  AE1";
$resn_exch{" H81 AE1"}="1H8  AE1"; $resn_exch{" H82 AE1"}="2H8  AE1"; $resn_exch{" H51 AE1"}="1H5  AE1";
$resn_exch{" H52 AE1"}="2H5  AE1"; $resn_exch{"H171 AE1"}="1H17 AE1"; $resn_exch{"H172 AE1"}="2H17 AE1";
$resn_exch{"H173 AE1"}="3H17 AE1"; $resn_exch{"H183 AE1"}="3H18 AE1"; $resn_exch{"H181 AE1"}="1H18 AE1";
$resn_exch{"H182 AE1"}="2H18 AE1"; $resn_exch{" H11 AE2"}="1H1  AE2"; $resn_exch{" H12 AE2"}="2H1  AE2";
$resn_exch{" H21 AE2"}="1H2  AE2"; $resn_exch{" H22 AE2"}="2H2  AE2"; $resn_exch{" H41 AE2"}="1H4  AE2";
$resn_exch{" H42 AE2"}="2H4  AE2"; $resn_exch{" H61 AE2"}="1H6  AE2"; $resn_exch{" H62 AE2"}="2H6  AE2";
$resn_exch{" H71 AE2"}="1H7  AE2"; $resn_exch{" H72 AE2"}="2H7  AE2"; $resn_exch{"H111 AE2"}="1H11 AE2";
$resn_exch{"H112 AE2"}="2H11 AE2"; $resn_exch{"H121 AE2"}="1H12 AE2"; $resn_exch{"H122 AE2"}="2H12 AE2";
$resn_exch{"H151 AE2"}="1H15 AE2"; $resn_exch{"H152 AE2"}="2H15 AE2"; $resn_exch{"H161 AE2"}="1H16 AE2";
$resn_exch{"H162 AE2"}="2H16 AE2"; $resn_exch{"H181 AE2"}="1H18 AE2"; $resn_exch{"H182 AE2"}="2H18 AE2";
$resn_exch{"H183 AE2"}="3H18 AE2"; $resn_exch{"H191 AE2"}="1H19 AE2"; $resn_exch{"H192 AE2"}="2H19 AE2";
$resn_exch{"H193 AE2"}="3H19 AE2"; $resn_exch{"H1C1 AE3"}="1H1C AE3"; $resn_exch{"H1C2 AE3"}="2H1C AE3";
$resn_exch{"H1C3 AE3"}="3H1C AE3"; $resn_exch{"H2C1 AE3"}="1H2C AE3"; $resn_exch{"H2C2 AE3"}="2H2C AE3";
$resn_exch{"H3C1 AE3"}="1H3C AE3"; $resn_exch{"H3C2 AE3"}="2H3C AE3"; $resn_exch{"H4C1 AE3"}="1H4C AE3";
$resn_exch{"H4C2 AE3"}="2H4C AE3"; $resn_exch{"H5C1 AE3"}="1H5C AE3"; $resn_exch{"H5C2 AE3"}="2H5C AE3";
$resn_exch{"H6C1 AE3"}="1H6C AE3"; $resn_exch{"H6C2 AE3"}="2H6C AE3"; $resn_exch{"H2C1 AE4"}="1H2C AE4";
$resn_exch{"H2C2 AE4"}="2H2C AE4"; $resn_exch{"H2C3 AE4"}="3H2C AE4"; $resn_exch{"H3C1 AE4"}="1H3C AE4";
$resn_exch{"H3C2 AE4"}="2H3C AE4"; $resn_exch{"H5C1 AE4"}="1H5C AE4"; $resn_exch{"H5C2 AE4"}="2H5C AE4";
$resn_exch{"H6C1 AE4"}="1H6C AE4"; $resn_exch{"H6C2 AE4"}="2H6C AE4"; $resn_exch{"H8C1 AE4"}="1H8C AE4";
$resn_exch{"H8C2 AE4"}="2H8C AE4"; $resn_exch{"H9C1 AE4"}="1H9C AE4"; $resn_exch{"H9C2 AE4"}="2H9C AE4";
$resn_exch{"H111 AE4"}="1H11 AE4"; $resn_exch{"H112 AE4"}="2H11 AE4"; $resn_exch{"H121 AE4"}="1H12 AE4";
$resn_exch{"H122 AE4"}="2H12 AE4"; $resn_exch{"H141 AE4"}="1H14 AE4"; $resn_exch{"H142 AE4"}="2H14 AE4";
$resn_exch{"H151 AE4"}="1H15 AE4"; $resn_exch{"H152 AE4"}="2H15 AE4"; $resn_exch{"H171 AE4"}="1H17 AE4";
$resn_exch{"H172 AE4"}="2H17 AE4"; $resn_exch{"H181 AE4"}="1H18 AE4"; $resn_exch{"H182 AE4"}="2H18 AE4";
$resn_exch{"HN11 AEA"}="1HN1 AEA"; $resn_exch{"HN12 AEA"}="2HN1 AEA"; $resn_exch{" H11 AEA"}="1H1  AEA";
$resn_exch{" H31 AEA"}="1H3  AEA"; $resn_exch{" H32 AEA"}="2H3  AEA"; $resn_exch{"HN21 AEA"}="1HN2 AEA";
$resn_exch{"HN22 AEA"}="2HN2 AEA"; $resn_exch{" H41 AEA"}="1H4  AEA"; $resn_exch{" H42 AEA"}="2H4  AEA";
$resn_exch{"HO21 AEA"}="1HO2 AEA"; $resn_exch{"HAQ1 AEE"}="1HAQ AEE"; $resn_exch{"HAQ2 AEE"}="2HAQ AEE";
$resn_exch{"HAO1 AEE"}="1HAO AEE"; $resn_exch{"HAO2 AEE"}="2HAO AEE"; $resn_exch{"HAN1 AEE"}="1HAN AEE";
$resn_exch{"HAN2 AEE"}="2HAN AEE"; $resn_exch{"HAA1 AEE"}="1HAA AEE"; $resn_exch{"HAA2 AEE"}="2HAA AEE";
$resn_exch{"HAA3 AEE"}="3HAA AEE"; $resn_exch{"HAP1 AEE"}="1HAP AEE"; $resn_exch{"HAP2 AEE"}="2HAP AEE";
$resn_exch{"HAR1 AEE"}="1HAR AEE"; $resn_exch{"HAR2 AEE"}="2HAR AEE"; $resn_exch{"HAS1 AEE"}="1HAS AEE";
$resn_exch{"HAS2 AEE"}="2HAS AEE"; $resn_exch{"HAB1 AEE"}="1HAB AEE"; $resn_exch{"HAB2 AEE"}="2HAB AEE";
$resn_exch{"HAB3 AEE"}="3HAB AEE"; $resn_exch{" H   AEI"}="1HN  AEI"; $resn_exch{" H2  AEI"}="2HN  AEI";
$resn_exch{"HG21 AEI"}="1HG  AEI"; $resn_exch{"HG22 AEI"}="2HG  AEI"; $resn_exch{"HG23 AEI"}="3HG  AEI";
$resn_exch{" HE1 AEI"}=" HOE AEI"; $resn_exch{"HE22 AEI"}="1HE2 AEI"; $resn_exch{"HE23 AEI"}="2HE2 AEI";
$resn_exch{"HH11 AEI"}="1HN1 AEI"; $resn_exch{"HH12 AEI"}="2HN1 AEI"; $resn_exch{"H271 AEJ"}="1H27 AEJ";
$resn_exch{"H272 AEJ"}="2H27 AEJ"; $resn_exch{"H301 AEJ"}="1H30 AEJ"; $resn_exch{"H302 AEJ"}="2H30 AEJ";
$resn_exch{" H81 AEJ"}="1H8  AEJ"; $resn_exch{" H82 AEJ"}="2H8  AEJ"; $resn_exch{"H111 AEJ"}="1H11 AEJ";
$resn_exch{"H112 AEJ"}="2H11 AEJ"; $resn_exch{" H41 AEJ"}="1H4  AEJ"; $resn_exch{"H42A AEJ"}="2H4  AEJ";
$resn_exch{"H551 AEJ"}="1H55 AEJ"; $resn_exch{"H552 AEJ"}="2H55 AEJ"; $resn_exch{"H581 AEJ"}="1H58 AEJ";
$resn_exch{"H582 AEJ"}="2H58 AEJ"; $resn_exch{"H611 AEJ"}="1H61 AEJ"; $resn_exch{"H612 AEJ"}="2H61 AEJ";
$resn_exch{"H641 AEJ"}="1H64 AEJ"; $resn_exch{"H642 AEJ"}="2H64 AEJ"; $resn_exch{"H2C1 AEM"}="1H2C AEM";
$resn_exch{"H2C2 AEM"}="2H2C AEM"; $resn_exch{"H2N1 AEM"}="1H2N AEM"; $resn_exch{"H2N2 AEM"}="2H2N AEM";
$resn_exch{"H1'1 AEN"}="1H1' AEN"; $resn_exch{"H1'2 AEN"}="2H1' AEN"; $resn_exch{"H1'3 AEN"}="3H1' AEN";
$resn_exch{"HN'3 AEN"}="3HN' AEN"; $resn_exch{"H4'1 AEN"}="1H4' AEN"; $resn_exch{"H4'2 AEN"}="2H4' AEN";
$resn_exch{"H5'1 AEN"}="1H5' AEN"; $resn_exch{"H5'2 AEN"}="2H5' AEN"; $resn_exch{"HN'6 AEN"}="6HN' AEN";
$resn_exch{"HOS3 AEN"}="3HOS AEN"; $resn_exch{" HN1 AEP"}="1HN  AEP"; $resn_exch{" HN2 AEP"}="2HN  AEP";
$resn_exch{" H21 AEP"}="1H2  AEP"; $resn_exch{" H22 AEP"}="2H2  AEP"; $resn_exch{" H41 AEP"}="1H4  AEP";
$resn_exch{" H42 AEP"}="2H4  AEP"; $resn_exch{" H51 AEP"}="1H5  AEP"; $resn_exch{" H52 AEP"}="2H5  AEP";
$resn_exch{" H53 AEP"}="3H5  AEP"; $resn_exch{"HOP2 AEP"}="2HOP AEP"; $resn_exch{" H71 AES"}="1H7  AES";
$resn_exch{" H72 AES"}="2H7  AES"; $resn_exch{" H81 AES"}="1H8  AES"; $resn_exch{" H82 AES"}="2H8  AES";
$resn_exch{"HN81 AES"}="1HN8 AES"; $resn_exch{"HN82 AES"}="2HN8 AES"; $resn_exch{"HOP2 AET"}="2HOP AET";
$resn_exch{"HOP3 AET"}="3HOP AET"; $resn_exch{" H5' AET"}="1H5* AET"; $resn_exch{"H5'' AET"}="2H5* AET";
$resn_exch{" H4' AET"}=" H4* AET"; $resn_exch{" H3' AET"}=" H3* AET"; $resn_exch{"HO3' AET"}="*HO3 AET";
$resn_exch{" H2' AET"}=" H2* AET"; $resn_exch{"HO2' AET"}="*HO2 AET"; $resn_exch{" H1' AET"}=" H1* AET";
$resn_exch{"HM61 AET"}="1HM6 AET"; $resn_exch{"HM62 AET"}="2HM6 AET"; $resn_exch{"HM63 AET"}="3HM6 AET";
$resn_exch{" HO4 AET"}="4HO  AET"; $resn_exch{"H151 AET"}="1H15 AET"; $resn_exch{"H152 AET"}="2H15 AET";
$resn_exch{"H153 AET"}="3H15 AET"; $resn_exch{" HN1  AF"}="1HN   AF"; $resn_exch{" HN2  AF"}="2HN   AF";
$resn_exch{" H91  AF"}="1H9   AF"; $resn_exch{" H92  AF"}="2H9   AF"; $resn_exch{" H61 AF1"}="1H6  AF1";
$resn_exch{" H62 AF1"}="2H6  AF1"; $resn_exch{" H63 AF1"}="3H6  AF1"; $resn_exch{"H7H1 AF1"}="1H7H AF1";
$resn_exch{"H7H2 AF1"}="2H7H AF1"; $resn_exch{"HO2' AF1"}="'HO2 AF1"; $resn_exch{"HO3' AF1"}="'HO3 AF1";
$resn_exch{"HO4' AF1"}="'HO4 AF1"; $resn_exch{" H32 AF4"}="2H3  AF4"; $resn_exch{" H31 AF4"}="1H3  AF4";
$resn_exch{" H33 AF4"}="3H3  AF4"; $resn_exch{" H42 AF4"}="2H4  AF4"; $resn_exch{" H43 AF4"}="3H4  AF4";
$resn_exch{" H41 AF4"}="1H4  AF4"; $resn_exch{" H51 AF4"}="1H5  AF4"; $resn_exch{" H52 AF4"}="2H5  AF4";
$resn_exch{" H53 AF4"}="3H5  AF4"; $resn_exch{" H   AFA"}=" HN  AFA"; $resn_exch{" HB2 AFA"}="1HB  AFA";
$resn_exch{" HB3 AFA"}="2HB  AFA"; $resn_exch{"HD21 AFA"}="1HND AFA"; $resn_exch{"HD22 AFA"}="2HND AFA";
$resn_exch{" H61 AFA"}="1H6  AFA"; $resn_exch{" H62 AFA"}="2H6  AFA"; $resn_exch{" H81 AFA"}="1H8  AFA";
$resn_exch{" H82 AFA"}="2H8  AFA"; $resn_exch{" H83 AFA"}="3H8  AFA"; $resn_exch{" H91 AFA"}="1H9  AFA";
$resn_exch{" H92 AFA"}="2H9  AFA"; $resn_exch{" H93 AFA"}="3H9  AFA"; $resn_exch{" H81 AFB"}="1H8  AFB";
$resn_exch{" H82 AFB"}="2H8  AFB"; $resn_exch{" H61 AFB"}="1H6  AFB"; $resn_exch{" H62 AFB"}="2H6  AFB";
$resn_exch{"H101 AFB"}="1H10 AFB"; $resn_exch{"H111 AFB"}="1H11 AFB"; $resn_exch{"H121 AFB"}="1H12 AFB";
$resn_exch{"H122 AFB"}="2H12 AFB"; $resn_exch{"H131 AFB"}="1H13 AFB"; $resn_exch{"H132 AFB"}="2H13 AFB";
$resn_exch{"H141 AFB"}="1H14 AFB"; $resn_exch{"H142 AFB"}="2H14 AFB"; $resn_exch{"H161 AFB"}="1H16 AFB";
$resn_exch{"H162 AFB"}="2H16 AFB"; $resn_exch{"H163 AFB"}="3H16 AFB"; $resn_exch{" H21 AFB"}="1H2  AFB";
$resn_exch{" H31 AFB"}="1H3  AFB"; $resn_exch{" HG1 AFC"}="1HG  AFC"; $resn_exch{" HG2 AFC"}="2HG  AFC";
$resn_exch{" HB2 AFC"}="2HB  AFC"; $resn_exch{" HB3 AFC"}="3HB  AFC"; $resn_exch{" HD1 AFC"}="1HD  AFC";
$resn_exch{" HD2 AFC"}="2HD  AFC"; $resn_exch{" HE1 AFC"}="1HE  AFC"; $resn_exch{" HE2 AFC"}="2HE  AFC";
$resn_exch{" HF1 AFC"}="1HF  AFC"; $resn_exch{" HF2 AFC"}="2HF  AFC"; $resn_exch{" HI1 AFC"}="1HI  AFC";
$resn_exch{" HI2 AFC"}="2HI  AFC"; $resn_exch{" HJ1 AFC"}="1HJ  AFC"; $resn_exch{" HJ2 AFC"}="2HJ  AFC";
$resn_exch{" HK1 AFC"}="1HK  AFC"; $resn_exch{" HK2 AFC"}="2HK  AFC"; $resn_exch{" HL1 AFC"}="1HL  AFC";
$resn_exch{" HL2 AFC"}="2HL  AFC"; $resn_exch{"HN21 AFC"}="1HN2 AFC"; $resn_exch{"HN22 AFC"}="2HN2 AFC";
$resn_exch{"HN23 AFC"}="3HN2 AFC"; $resn_exch{"HN11 AFC"}="1HN1 AFC"; $resn_exch{"HN12 AFC"}="2HN1 AFC";
$resn_exch{"HO11 AFC"}="1HO1 AFC"; $resn_exch{"HO12 AFC"}="2HO1 AFC"; $resn_exch{"HO13 AFC"}="3HO1 AFC";
$resn_exch{"HOP3 AFG"}="3HOP AFG"; $resn_exch{"HOP2 AFG"}="2HOP AFG"; $resn_exch{"HN21 AFG"}="1HN2 AFG";
$resn_exch{"HN22 AFG"}="2HN2 AFG"; $resn_exch{"H371 AFG"}="1H37 AFG"; $resn_exch{"H372 AFG"}="2H37 AFG";
$resn_exch{" H2' AFG"}="1H2* AFG"; $resn_exch{"H2'' AFG"}="2H2* AFG"; $resn_exch{" H5' AFG"}="1H5* AFG";
$resn_exch{"H5'' AFG"}="2H5* AFG"; $resn_exch{" H4' AFG"}=" H4* AFG"; $resn_exch{" H1' AFG"}=" H1* AFG";
$resn_exch{" H3' AFG"}=" H3* AFG"; $resn_exch{"HO3' AFG"}="H3T  AFG"; $resn_exch{"HN61 AFH"}="1HN6 AFH";
$resn_exch{"HN62 AFH"}="2HN6 AFH"; $resn_exch{" H1D AFH"}=" H1* AFH"; $resn_exch{" H2D AFH"}=" H2* AFH";
$resn_exch{"HO2' AFH"}="HO2* AFH"; $resn_exch{" H3D AFH"}=" H3* AFH"; $resn_exch{" H4D AFH"}=" H4* AFH";
$resn_exch{"HO3A AFH"}="HO3* AFH"; $resn_exch{"H5'1 AFH"}="1H5* AFH"; $resn_exch{"H5'2 AFH"}="2H5* AFH";
$resn_exch{"H7'1 AFH"}="1H7' AFH"; $resn_exch{"H7'2 AFH"}="2H7' AFH"; $resn_exch{"H111 AFI"}="1H11 AFI";
$resn_exch{"H112 AFI"}="2H11 AFI"; $resn_exch{"H121 AFI"}="1H12 AFI"; $resn_exch{"H122 AFI"}="2H12 AFI";
$resn_exch{"H141 AFI"}="1H14 AFI"; $resn_exch{"H142 AFI"}="2H14 AFI"; $resn_exch{"H151 AFI"}="1H15 AFI";
$resn_exch{"H152 AFI"}="2H15 AFI"; $resn_exch{" HA1 AFL"}="1HA  AFL"; $resn_exch{" HB2 AFL"}="2HB  AFL";
$resn_exch{" HC3 AFL"}="3HC  AFL"; $resn_exch{"H8A2 AFN"}="2H8A AFN"; $resn_exch{" HM1 AFN"}="1HM  AFN";
$resn_exch{" HM2 AFN"}="2HM  AFN"; $resn_exch{" HM3 AFN"}="3HM  AFN"; $resn_exch{" H31 AFN"}="1H3  AFN";
$resn_exch{" H32 AFN"}="2H3  AFN"; $resn_exch{"H2A1 AFN"}="1H2A AFN"; $resn_exch{"H2A2 AFN"}="2H2A AFN";
$resn_exch{"HOP2 AFP"}="2HOP AFP"; $resn_exch{"HOP3 AFP"}="3HOP AFP"; $resn_exch{" H11 AFP"}="1H1  AFP";
$resn_exch{" H12 AFP"}="2H1  AFP"; $resn_exch{" H61 AFP"}="1H6  AFP"; $resn_exch{" H62 AFP"}="2H6  AFP";
$resn_exch{"HOP5 AFP"}="5HOP AFP"; $resn_exch{"HOP6 AFP"}="6HOP AFP"; $resn_exch{" H31 AFZ"}="1H3  AFZ";
$resn_exch{" H33 AFZ"}="3H3  AFZ"; $resn_exch{" H32 AFZ"}="2H3  AFZ"; $resn_exch{" H41 AFZ"}="1H4  AFZ";
$resn_exch{" HN1 AG2"}="1HN  AG2"; $resn_exch{" HN2 AG2"}="2HN  AG2"; $resn_exch{" HA1 AG2"}="1HA  AG2";
$resn_exch{" HA2 AG2"}="2HA  AG2"; $resn_exch{" HB1 AG2"}="1HB  AG2"; $resn_exch{" HB2 AG2"}="2HB  AG2";
$resn_exch{" HG1 AG2"}="1HG  AG2"; $resn_exch{" HG2 AG2"}="2HG  AG2"; $resn_exch{" HD1 AG2"}="1HD  AG2";
$resn_exch{" HD2 AG2"}="2HD  AG2"; $resn_exch{" HE1 AG2"}="1HE  AG2"; $resn_exch{"HH11 AG2"}="1HH1 AG2";
$resn_exch{"HH21 AG2"}="1HH2 AG2"; $resn_exch{"HH22 AG2"}="2HH2 AG2"; $resn_exch{" HA1 AG3"}="1HA  AG3";
$resn_exch{" HA2 AG3"}="2HA  AG3"; $resn_exch{" HB1 AG3"}="1HB  AG3"; $resn_exch{" HB2 AG3"}="2HB  AG3";
$resn_exch{" HG1 AG3"}="1HG  AG3"; $resn_exch{" HG2 AG3"}="2HG  AG3"; $resn_exch{" HD1 AG3"}="1HD  AG3";
$resn_exch{" HD2 AG3"}="2HD  AG3"; $resn_exch{"HH11 AG3"}="1HH1 AG3"; $resn_exch{"HH12 AG3"}="2HH1 AG3";
$resn_exch{" H71 AG3"}="1H7  AG3"; $resn_exch{" H72 AG3"}="2H7  AG3"; $resn_exch{" H81 AG3"}="1H8  AG3";
$resn_exch{" H82 AG3"}="2H8  AG3"; $resn_exch{" H91 AG3"}="1H9  AG3"; $resn_exch{" H92 AG3"}="2H9  AG3";
$resn_exch{"H101 AG3"}="1H10 AG3"; $resn_exch{"H102 AG3"}="2H10 AG3"; $resn_exch{"H191 AG7"}="1H19 AG7";
$resn_exch{"H192 AG7"}="2H19 AG7"; $resn_exch{" H11 AGA"}="1H1  AGA"; $resn_exch{" H12 AGA"}="2H1  AGA";
$resn_exch{" H31 AGA"}="1H3  AGA"; $resn_exch{" H32 AGA"}="2H3  AGA"; $resn_exch{" H41 AGA"}="1H4  AGA";
$resn_exch{" H42 AGA"}="2H4  AGA"; $resn_exch{" H61 AGA"}="1H6  AGA"; $resn_exch{" H62 AGA"}="2H6  AGA";
$resn_exch{" H81 AGA"}="1H8  AGA"; $resn_exch{" H82 AGA"}="2H8  AGA"; $resn_exch{" H91 AGA"}="1H9  AGA";
$resn_exch{" H92 AGA"}="2H9  AGA"; $resn_exch{"H101 AGA"}="1H10 AGA"; $resn_exch{"H102 AGA"}="2H10 AGA";
$resn_exch{"H111 AGA"}="1H11 AGA"; $resn_exch{"H112 AGA"}="2H11 AGA"; $resn_exch{"H113 AGA"}="3H11 AGA";
$resn_exch{"H131 AGA"}="1H13 AGA"; $resn_exch{"H132 AGA"}="2H13 AGA"; $resn_exch{"H141 AGA"}="1H14 AGA";
$resn_exch{"H142 AGA"}="2H14 AGA"; $resn_exch{"H151 AGA"}="1H15 AGA"; $resn_exch{"H152 AGA"}="2H15 AGA";
$resn_exch{"H161 AGA"}="1H16 AGA"; $resn_exch{"H162 AGA"}="2H16 AGA"; $resn_exch{"H171 AGA"}="1H17 AGA";
$resn_exch{"H172 AGA"}="2H17 AGA"; $resn_exch{"H181 AGA"}="1H18 AGA"; $resn_exch{"H182 AGA"}="2H18 AGA";
$resn_exch{"H191 AGA"}="1H19 AGA"; $resn_exch{"H192 AGA"}="2H19 AGA"; $resn_exch{"H193 AGA"}="3H19 AGA";
$resn_exch{" H21 AGB"}="1H2  AGB"; $resn_exch{" H22 AGB"}="2H2  AGB"; $resn_exch{" H31 AGB"}="1H3  AGB";
$resn_exch{" H32 AGB"}="2H3  AGB"; $resn_exch{" H41 AGB"}="1H4  AGB"; $resn_exch{" H42 AGB"}="2H4  AGB";
$resn_exch{" H51 AGB"}="1H5  AGB"; $resn_exch{" H52 AGB"}="2H5  AGB"; $resn_exch{" H81 AGB"}="1H8  AGB";
$resn_exch{" H82 AGB"}="2H8  AGB"; $resn_exch{" H91 AGB"}="1H9  AGB"; $resn_exch{" H92 AGB"}="2H9  AGB";
$resn_exch{"H111 AGB"}="1H11 AGB"; $resn_exch{"H112 AGB"}="2H11 AGB"; $resn_exch{"HN51 AGB"}="1HN5 AGB";
$resn_exch{"HN52 AGB"}="2HN5 AGB"; $resn_exch{" H61 AGC"}="1H6  AGC"; $resn_exch{" H62 AGC"}="2H6  AGC";
$resn_exch{"HB11 AGF"}="1HB1 AGF"; $resn_exch{"HB12 AGF"}="2HB1 AGF"; $resn_exch{"HD11 AGF"}="1HD1 AGF";
$resn_exch{"HD12 AGF"}="2HD1 AGF"; $resn_exch{"HE11 AGF"}="1HE1 AGF"; $resn_exch{"HE12 AGF"}="2HE1 AGF";
$resn_exch{"HB21 AGF"}="1HB2 AGF"; $resn_exch{"HB22 AGF"}="2HB2 AGF"; $resn_exch{"HB23 AGF"}="3HB2 AGF";
$resn_exch{"HA31 AGF"}="1HA3 AGF"; $resn_exch{"HA32 AGF"}="2HA3 AGF"; $resn_exch{"HOP2 AGF"}="2HOP AGF";
$resn_exch{"HB41 AGF"}="1HB4 AGF"; $resn_exch{"HB42 AGF"}="2HB4 AGF"; $resn_exch{"HD41 AGF"}="1HD4 AGF";
$resn_exch{"HD42 AGF"}="2HD4 AGF"; $resn_exch{"HE41 AGF"}="1HE4 AGF"; $resn_exch{"HE42 AGF"}="2HE4 AGF";
$resn_exch{" H11 AGG"}="1H1  AGG"; $resn_exch{" H12 AGG"}="2H1  AGG"; $resn_exch{" H13 AGG"}="3H1  AGG";
$resn_exch{" H21 AGG"}="1H2  AGG"; $resn_exch{" H22 AGG"}="2H2  AGG"; $resn_exch{" H31 AGG"}="1H3  AGG";
$resn_exch{" H32 AGG"}="2H3  AGG"; $resn_exch{" H41 AGG"}="1H4  AGG"; $resn_exch{" H42 AGG"}="2H4  AGG";
$resn_exch{" HB1 AGG"}="1HB  AGG"; $resn_exch{" HB2 AGG"}="2HB  AGG"; $resn_exch{"H111 AGG"}="1H11 AGG";
$resn_exch{"H112 AGG"}="2H11 AGG"; $resn_exch{"H121 AGG"}="1H12 AGG"; $resn_exch{"H122 AGG"}="2H12 AGG";
$resn_exch{"H131 AGG"}="1H13 AGG"; $resn_exch{"H132 AGG"}="2H13 AGG"; $resn_exch{"H141 AGG"}="1H14 AGG";
$resn_exch{"H142 AGG"}="2H14 AGG"; $resn_exch{"H161 AGG"}="1H16 AGG"; $resn_exch{"H162 AGG"}="2H16 AGG";
$resn_exch{"H171 AGG"}="1H17 AGG"; $resn_exch{"H172 AGG"}="2H17 AGG"; $resn_exch{"H181 AGG"}="1H18 AGG";
$resn_exch{"H182 AGG"}="2H18 AGG"; $resn_exch{"H191 AGG"}="1H19 AGG"; $resn_exch{"H192 AGG"}="2H19 AGG";
$resn_exch{" H11 AGH"}="1H1  AGH"; $resn_exch{" H12 AGH"}="2H1  AGH"; $resn_exch{" H51 AGH"}="1H5  AGH";
$resn_exch{" H52 AGH"}="2H5  AGH"; $resn_exch{" H61 AGH"}="1H6  AGH"; $resn_exch{" H62 AGH"}="2H6  AGH";
$resn_exch{"H181 AGH"}="1H18 AGH"; $resn_exch{"H182 AGH"}="2H18 AGH"; $resn_exch{"H183 AGH"}="3H18 AGH";
$resn_exch{"H171 AGH"}="1H17 AGH"; $resn_exch{"H172 AGH"}="2H17 AGH"; $resn_exch{"H161 AGH"}="1H16 AGH";
$resn_exch{"H162 AGH"}="2H16 AGH"; $resn_exch{"H151 AGH"}="1H15 AGH"; $resn_exch{"H152 AGH"}="2H15 AGH";
$resn_exch{"H141 AGH"}="1H14 AGH"; $resn_exch{"H142 AGH"}="2H14 AGH"; $resn_exch{"H131 AGH"}="1H13 AGH";
$resn_exch{"H132 AGH"}="2H13 AGH"; $resn_exch{"H121 AGH"}="1H12 AGH"; $resn_exch{"H122 AGH"}="2H12 AGH";
$resn_exch{"H111 AGH"}="1H11 AGH"; $resn_exch{"H112 AGH"}="2H11 AGH"; $resn_exch{"H101 AGH"}="1H10 AGH";
$resn_exch{"H102 AGH"}="2H10 AGH"; $resn_exch{" H91 AGH"}="1H9  AGH"; $resn_exch{" H92 AGH"}="2H9  AGH";
$resn_exch{" H81 AGH"}="1H8  AGH"; $resn_exch{" H82 AGH"}="2H8  AGH"; $resn_exch{" H71 AGH"}="1H7  AGH";
$resn_exch{" H72 AGH"}="2H7  AGH"; $resn_exch{"H6A1 AGH"}="1H6A AGH"; $resn_exch{"H6A2 AGH"}="2H6A AGH";
$resn_exch{"HAB1 AGH"}="1HAB AGH"; $resn_exch{"HAB2 AGH"}="2HAB AGH"; $resn_exch{"HAC1 AGH"}="1HAC AGH";
$resn_exch{"HAC2 AGH"}="2HAC AGH"; $resn_exch{"HAD1 AGH"}="1HAD AGH"; $resn_exch{"HAD2 AGH"}="2HAD AGH";
$resn_exch{"HAE1 AGH"}="1HAE AGH"; $resn_exch{"HAE2 AGH"}="2HAE AGH"; $resn_exch{"HAF1 AGH"}="1HAF AGH";
$resn_exch{"HAF2 AGH"}="2HAF AGH"; $resn_exch{"HAG1 AGH"}="1HAG AGH"; $resn_exch{"HAG2 AGH"}="2HAG AGH";
$resn_exch{"HAH1 AGH"}="1HAH AGH"; $resn_exch{"HAH2 AGH"}="2HAH AGH"; $resn_exch{"HAI1 AGH"}="1HAI AGH";
$resn_exch{"HAI2 AGH"}="2HAI AGH"; $resn_exch{"HAJ1 AGH"}="1HAJ AGH"; $resn_exch{"HAJ2 AGH"}="2HAJ AGH";
$resn_exch{"HAK1 AGH"}="1HAK AGH"; $resn_exch{"HAK2 AGH"}="2HAK AGH"; $resn_exch{"HAL1 AGH"}="1HAL AGH";
$resn_exch{"HAL2 AGH"}="2HAL AGH"; $resn_exch{"HAM1 AGH"}="1HAM AGH"; $resn_exch{"HAM2 AGH"}="2HAM AGH";
$resn_exch{"HAN1 AGH"}="1HAN AGH"; $resn_exch{"HAN2 AGH"}="2HAN AGH"; $resn_exch{"HAO1 AGH"}="1HAO AGH";
$resn_exch{"HAO2 AGH"}="2HAO AGH"; $resn_exch{"HAP1 AGH"}="1HAP AGH"; $resn_exch{"HAP2 AGH"}="2HAP AGH";
$resn_exch{"HAQ1 AGH"}="1HAQ AGH"; $resn_exch{"HAQ2 AGH"}="2HAQ AGH"; $resn_exch{"HAR1 AGH"}="1HAR AGH";
$resn_exch{"HAR2 AGH"}="2HAR AGH"; $resn_exch{"HAS1 AGH"}="1HAS AGH"; $resn_exch{"HAS2 AGH"}="2HAS AGH";
$resn_exch{"HAT1 AGH"}="1HAT AGH"; $resn_exch{"HAT2 AGH"}="2HAT AGH"; $resn_exch{"HAU1 AGH"}="1HAU AGH";
$resn_exch{"HAU2 AGH"}="2HAU AGH"; $resn_exch{"HAV1 AGH"}="1HAV AGH"; $resn_exch{"HAV2 AGH"}="2HAV AGH";
$resn_exch{"HAW1 AGH"}="1HAW AGH"; $resn_exch{"HAW2 AGH"}="2HAW AGH"; $resn_exch{"HAX1 AGH"}="1HAX AGH";
$resn_exch{"HAX2 AGH"}="2HAX AGH"; $resn_exch{"HAY1 AGH"}="1HAY AGH"; $resn_exch{"HAY2 AGH"}="2HAY AGH";
$resn_exch{"HAZ1 AGH"}="1HAZ AGH"; $resn_exch{"HAZ2 AGH"}="2HAZ AGH"; $resn_exch{"HAZ3 AGH"}="3HAZ AGH";
$resn_exch{" H61 AGL"}="1H6  AGL"; $resn_exch{" H62 AGL"}="2H6  AGL"; $resn_exch{" H63 AGL"}="3H6  AGL";
$resn_exch{"HN41 AGL"}="1HN4 AGL"; $resn_exch{"HN42 AGL"}="2HN4 AGL"; $resn_exch{" H   AGM"}=" HN1 AGM";
$resn_exch{" H2  AGM"}=" HN2 AGM"; $resn_exch{" HB2 AGM"}=" HB1 AGM"; $resn_exch{" HB3 AGM"}=" HB2 AGM";
$resn_exch{" HG2 AGM"}=" HG1 AGM"; $resn_exch{" HG3 AGM"}=" HG2 AGM"; $resn_exch{"HOG2 AGN"}="2HOG AGN";
$resn_exch{"HOG3 AGN"}="3HOG AGN"; $resn_exch{"HNB3 AGN"}="3HNB AGN"; $resn_exch{"HOB2 AGN"}="2HOB AGN";
$resn_exch{"HOA2 AGN"}="2HOA AGN"; $resn_exch{"H5'2 AGN"}="2H5* AGN"; $resn_exch{"H5'1 AGN"}="1H5* AGN";
$resn_exch{" H4B AGN"}=" H4* AGN"; $resn_exch{" H3B AGN"}=" H3* AGN"; $resn_exch{"H2'1 AGN"}="1H2* AGN";
$resn_exch{"H2'2 AGN"}="2H2* AGN"; $resn_exch{" H1' AGN"}=" H1* AGN"; $resn_exch{"HN2' AGN"}="'HN2 AGN";
$resn_exch{"HM'1 AGN"}="1HM' AGN"; $resn_exch{"HM'2 AGN"}="2HM' AGN"; $resn_exch{"HM'3 AGN"}="3HM' AGN";
$resn_exch{"HN21 AGN"}="1HN2 AGN"; $resn_exch{"HN22 AGN"}="2HN2 AGN"; $resn_exch{" H11 AGP"}="1H1  AGP";
$resn_exch{" H12 AGP"}="2H1  AGP"; $resn_exch{"HN21 AGP"}="1HN2 AGP"; $resn_exch{"HN22 AGP"}="2HN2 AGP";
$resn_exch{" H61 AGP"}="1H6  AGP"; $resn_exch{" H62 AGP"}="2H6  AGP"; $resn_exch{"HOP2 AGP"}="2HOP AGP";
$resn_exch{"HOP3 AGP"}="3HOP AGP"; $resn_exch{"HOG2 AGS"}="2HOG AGS"; $resn_exch{"HOB2 AGS"}="2HOB AGS";
$resn_exch{"HOA2 AGS"}="2HOA AGS"; $resn_exch{"H5'1 AGS"}="1H5* AGS"; $resn_exch{"H5'2 AGS"}="2H5* AGS";
$resn_exch{" H4' AGS"}=" H4* AGS"; $resn_exch{" H3' AGS"}=" H3* AGS"; $resn_exch{"HO3' AGS"}="*HO3 AGS";
$resn_exch{" H2' AGS"}=" H2* AGS"; $resn_exch{"HO2' AGS"}="*HO2 AGS"; $resn_exch{" H1' AGS"}=" H1* AGS";
$resn_exch{"HN61 AGS"}="1HN6 AGS"; $resn_exch{"HN62 AGS"}="2HN6 AGS"; $resn_exch{" HN1 AGT"}="1HN  AGT";
$resn_exch{" HN2 AGT"}="2HN  AGT"; $resn_exch{"HH21 AGT"}="1HH2 AGT"; $resn_exch{"HH22 AGT"}="2HH2 AGT";
$resn_exch{"HDC1 AGT"}="1HDC AGT"; $resn_exch{"HDC2 AGT"}="2HDC AGT"; $resn_exch{"HGC1 AGT"}="1HGC AGT";
$resn_exch{"HGC2 AGT"}="2HGC AGT"; $resn_exch{"HKC1 AGT"}="1HKC AGT"; $resn_exch{"HKC2 AGT"}="2HKC AGT";
$resn_exch{"HJC1 AGT"}="1HJC AGT"; $resn_exch{"HJC2 AGT"}="2HJC AGT"; $resn_exch{"HBN1 AGT"}="1HBN AGT";
$resn_exch{"HBN2 AGT"}="2HBN AGT"; $resn_exch{" HB2 AGT"}="2HB  AGT"; $resn_exch{"HN21 AGU"}="1HN2 AGU";
$resn_exch{"HN22 AGU"}="2HN2 AGU"; $resn_exch{"HN41 AGU"}="1HN4 AGU"; $resn_exch{"HN42 AGU"}="2HN4 AGU";
$resn_exch{" H81 AH0"}="1H8  AH0"; $resn_exch{" H82 AH0"}="2H8  AH0"; $resn_exch{" H83 AH0"}="3H8  AH0";
$resn_exch{" H61 AH0"}="1H6  AH0"; $resn_exch{" H62 AH0"}="2H6  AH0"; $resn_exch{" HB1 AH0"}="1HB  AH0";
$resn_exch{" HB2 AH0"}="2HB  AH0"; $resn_exch{" HB3 AH0"}="3HB  AH0"; $resn_exch{"H081 AH1"}="1H08 AH1";
$resn_exch{"H082 AH1"}="2H08 AH1"; $resn_exch{"H161 AH1"}="1H16 AH1"; $resn_exch{"H162 AH1"}="2H16 AH1";
$resn_exch{"H231 AH1"}="1H23 AH1"; $resn_exch{"H232 AH1"}="2H23 AH1"; $resn_exch{"H331 AH1"}="1H33 AH1";
$resn_exch{"H332 AH1"}="2H33 AH1"; $resn_exch{" H21 AHA"}="1H2  AHA"; $resn_exch{" H22 AHA"}="2H2  AHA";
$resn_exch{" H31 AHA"}="1H3  AHA"; $resn_exch{" H32 AHA"}="2H3  AHA"; $resn_exch{" H41 AHA"}="1H4  AHA";
$resn_exch{" H42 AHA"}="2H4  AHA"; $resn_exch{" H51 AHA"}="1H5  AHA"; $resn_exch{" H52 AHA"}="2H5  AHA";
$resn_exch{" H61 AHA"}="1H6  AHA"; $resn_exch{" H62 AHA"}="2H6  AHA"; $resn_exch{" HN1 AHA"}="1HN  AHA";
$resn_exch{" HN2 AHA"}="2HN  AHA"; $resn_exch{" H   AHB"}=" HN1 AHB"; $resn_exch{" H2  AHB"}=" HN2 AHB";
$resn_exch{"HD21 AHB"}="HND1 AHB"; $resn_exch{"HD22 AHB"}="HND2 AHB"; $resn_exch{" HA1 AHC"}="1HA  AHC";
$resn_exch{" HA2 AHC"}="2HA  AHC"; $resn_exch{" HB1 AHC"}="1HB  AHC"; $resn_exch{" HB2 AHC"}="2HB  AHC";
$resn_exch{"HNH1 AHC"}="1HNH AHC"; $resn_exch{"HNH2 AHC"}="2HNH AHC"; $resn_exch{" H21 AHD"}="1H2  AHD";
$resn_exch{" H22 AHD"}="2H2  AHD"; $resn_exch{" H31 AHD"}="1H3  AHD"; $resn_exch{" H32 AHD"}="2H3  AHD";
$resn_exch{" H71 AHD"}="1H7  AHD"; $resn_exch{" H72 AHD"}="2H7  AHD"; $resn_exch{"HN41 AHD"}="1HN4 AHD";
$resn_exch{"HN42 AHD"}="2HN4 AHD"; $resn_exch{"HN11 AHE"}="1HN1 AHE"; $resn_exch{"HN12 AHE"}="2HN1 AHE";
$resn_exch{"HB11 AHE"}="1HB1 AHE"; $resn_exch{"HB12 AHE"}="2HB1 AHE"; $resn_exch{"HG11 AHE"}="1HG1 AHE";
$resn_exch{"HG12 AHE"}="2HG1 AHE"; $resn_exch{"HB21 AHE"}="1HB2 AHE"; $resn_exch{"HB22 AHE"}="2HB2 AHE";
$resn_exch{"HD21 AHE"}="1HD2 AHE"; $resn_exch{"HD22 AHE"}="2HD2 AHE"; $resn_exch{"HA31 AHE"}="1HA3 AHE";
$resn_exch{"HA32 AHE"}="2HA3 AHE"; $resn_exch{"H082 AHF"}="2H08 AHF"; $resn_exch{"H081 AHF"}="1H08 AHF";
$resn_exch{"H231 AHF"}="1H23 AHF"; $resn_exch{"H232 AHF"}="2H23 AHF"; $resn_exch{"H321 AHF"}="1H32 AHF";
$resn_exch{"H322 AHF"}="2H32 AHF"; $resn_exch{"H411 AHF"}="1H41 AHF"; $resn_exch{"H412 AHF"}="2H41 AHF";
$resn_exch{"H451 AHF"}="1H45 AHF"; $resn_exch{"H452 AHF"}="2H45 AHF"; $resn_exch{"H453 AHF"}="3H45 AHF";
$resn_exch{"H461 AHF"}="1H46 AHF"; $resn_exch{"H462 AHF"}="2H46 AHF"; $resn_exch{"HOP2 AHG"}="2HOP AHG";
$resn_exch{"HOP3 AHG"}="3HOP AHG"; $resn_exch{" H11 AHG"}="1H1  AHG"; $resn_exch{" H12 AHG"}="2H1  AHG";
$resn_exch{" H61 AHG"}="1H6  AHG"; $resn_exch{" H62 AHG"}="2H6  AHG"; $resn_exch{"HOP5 AHG"}="5HOP AHG";
$resn_exch{"HOP6 AHG"}="6HOP AHG"; $resn_exch{" HN1 AHH"}="1HN  AHH"; $resn_exch{" HN2 AHH"}="2HN  AHH";
$resn_exch{"HCA1 AHH"}="1HCA AHH"; $resn_exch{"HCB1 AHH"}="1HCB AHH"; $resn_exch{"HCB2 AHH"}="2HCB AHH";
$resn_exch{"HCG1 AHH"}="1HCG AHH"; $resn_exch{"HCG2 AHH"}="2HCG AHH"; $resn_exch{"HO11 AHH"}="1HO1 AHH";
$resn_exch{"HCD1 AHH"}="1HCD AHH"; $resn_exch{"HCD2 AHH"}="2HCD AHH"; $resn_exch{"HCE1 AHH"}="1HCE AHH";
$resn_exch{"HCE2 AHH"}="2HCE AHH"; $resn_exch{"HCE3 AHH"}="3HCE AHH"; $resn_exch{"HC11 AHH"}="1HC1 AHH";
$resn_exch{"HO31 AHH"}="1HO3 AHH"; $resn_exch{"H101 AHI"}="1H10 AHI"; $resn_exch{"H102 AHI"}="2H10 AHI";
$resn_exch{"H071 AHI"}="1H07 AHI"; $resn_exch{"H072 AHI"}="2H07 AHI"; $resn_exch{"H061 AHI"}="1H06 AHI";
$resn_exch{"H062 AHI"}="2H06 AHI"; $resn_exch{"H031 AHI"}="1H03 AHI"; $resn_exch{"H032 AHI"}="2H03 AHI";
$resn_exch{"H021 AHI"}="1H02 AHI"; $resn_exch{"H022 AHI"}="2H02 AHI"; $resn_exch{"H023 AHI"}="3H02 AHI";
$resn_exch{"HN11 AHL"}="1HN1 AHL"; $resn_exch{"HN12 AHL"}="2HN1 AHL"; $resn_exch{" HB1 AHL"}="1HB  AHL";
$resn_exch{" HB2 AHL"}="2HB  AHL"; $resn_exch{" HG1 AHL"}="1HG  AHL"; $resn_exch{" HG2 AHL"}="2HG  AHL";
$resn_exch{" HD1 AHL"}="1HD  AHL"; $resn_exch{" HD2 AHL"}="2HD  AHL"; $resn_exch{"HH11 AHL"}="1HH1 AHL";
$resn_exch{"HH12 AHL"}="2HH1 AHL"; $resn_exch{"HOP2 AHM"}="2HOP AHM"; $resn_exch{"HOP3 AHM"}="3HOP AHM";
$resn_exch{" H11 AHM"}="1H1  AHM"; $resn_exch{" H12 AHM"}="2H1  AHM"; $resn_exch{" H61 AHM"}="1H6  AHM";
$resn_exch{" H62 AHM"}="2H6  AHM"; $resn_exch{"HOP5 AHM"}="5HOP AHM"; $resn_exch{"HOP6 AHM"}="6HOP AHM";
$resn_exch{" HT1 AHN"}="1HT  AHN"; $resn_exch{" HT2 AHN"}="2HT  AHN"; $resn_exch{" HT3 AHN"}="3HT  AHN";
$resn_exch{" HA1 AHN"}="1HA  AHN"; $resn_exch{" HA2 AHN"}="2HA  AHN"; $resn_exch{" HB1 AHN"}="1HB  AHN";
$resn_exch{" HB2 AHN"}="2HB  AHN"; $resn_exch{"HC21 AHO"}="1HC2 AHO"; $resn_exch{"HC31 AHO"}="1HC3 AHO";
$resn_exch{"HC32 AHO"}="2HC3 AHO"; $resn_exch{"HC41 AHO"}="1HC4 AHO"; $resn_exch{"HC42 AHO"}="2HC4 AHO";
$resn_exch{"HC51 AHO"}="1HC5 AHO"; $resn_exch{"HC52 AHO"}="2HC5 AHO"; $resn_exch{"HC71 AHO"}="1HC7 AHO";
$resn_exch{"HC72 AHO"}="2HC7 AHO"; $resn_exch{"HC73 AHO"}="3HC7 AHO"; $resn_exch{"HN11 AHO"}="1HN1 AHO";
$resn_exch{"HN12 AHO"}="2HN1 AHO"; $resn_exch{"HO21 AHO"}="1HO2 AHO"; $resn_exch{" H   AHP"}="1HN  AHP";
$resn_exch{" H2  AHP"}="2HN  AHP"; $resn_exch{" HB2 AHP"}="1HB  AHP"; $resn_exch{" HB3 AHP"}="2HB  AHP";
$resn_exch{" HG2 AHP"}="1HG  AHP"; $resn_exch{" HG3 AHP"}="2HG  AHP"; $resn_exch{" HD2 AHP"}="1HD  AHP";
$resn_exch{" HD3 AHP"}="2HD  AHP"; $resn_exch{" HE2 AHP"}="1HE  AHP"; $resn_exch{" HE3 AHP"}="2HE  AHP";
$resn_exch{" HZ1 AHP"}="1HZ  AHP"; $resn_exch{" HZ2 AHP"}="2HZ  AHP"; $resn_exch{" HZ3 AHP"}="3HZ  AHP";
$resn_exch{" H5' AHR"}=" H5* AHR"; $resn_exch{"H5'1 AHR"}="1H5* AHR"; $resn_exch{"H5'2 AHR"}="2H5* AHR";
$resn_exch{" H4' AHR"}=" H4* AHR"; $resn_exch{" H3' AHR"}=" H3* AHR"; $resn_exch{" H2' AHR"}=" H2* AHR";
$resn_exch{" H1' AHR"}=" H1* AHR"; $resn_exch{" H   AHS"}="1HN  AHS"; $resn_exch{" H2  AHS"}="2HN  AHS";
$resn_exch{" HB2 AHS"}="1HB  AHS"; $resn_exch{" HB3 AHS"}="2HB  AHS"; $resn_exch{"HD12 AHS"}="1HD1 AHS";
$resn_exch{"HD13 AHS"}="2HD1 AHS"; $resn_exch{"HD23 AHS"}="1HD2 AHS"; $resn_exch{"HD22 AHS"}="2HD2 AHS";
$resn_exch{"HE12 AHS"}="1HE1 AHS"; $resn_exch{"HE13 AHS"}="2HE1 AHS"; $resn_exch{"HE23 AHS"}="1HE2 AHS";
$resn_exch{"HE22 AHS"}="2HE2 AHS"; $resn_exch{" HZ2 AHS"}="1HZ  AHS"; $resn_exch{" HZ3 AHS"}="2HZ  AHS";
$resn_exch{" HM1 AHS"}="1HM  AHS"; $resn_exch{" HM2 AHS"}="2HM  AHS"; $resn_exch{"HB21 AHS"}="1HB2 AHS";
$resn_exch{"HB22 AHS"}="2HB2 AHS"; $resn_exch{"HD31 AHS"}="1HD3 AHS"; $resn_exch{"HD32 AHS"}="2HD3 AHS";
$resn_exch{"HD33 AHS"}="3HD3 AHS"; $resn_exch{"HD41 AHS"}="1HD4 AHS"; $resn_exch{"HD42 AHS"}="2HD4 AHS";
$resn_exch{"HD43 AHS"}="3HD4 AHS"; $resn_exch{" H   AHT"}="1HN  AHT"; $resn_exch{" H2  AHT"}="2HN  AHT";
$resn_exch{" HC1 AHT"}="1HC  AHT"; $resn_exch{" HC2 AHT"}="2HC  AHT"; $resn_exch{" HB2 AHT"}="1HB  AHT";
$resn_exch{" HB3 AHT"}="2HB  AHT"; $resn_exch{" HH  AHT"}=" HO  AHT"; $resn_exch{" H11 AHT"}="1H1  AHT";
$resn_exch{" H12 AHT"}="2H1  AHT"; $resn_exch{"HC22 AHT"}="H2   AHT"; $resn_exch{"HN21 AHT"}="1HN2 AHT";
$resn_exch{"HN22 AHT"}="2HN2 AHT"; $resn_exch{" H4' AHU"}=" H4* AHU"; $resn_exch{"H7'1 AHU"}="1H7* AHU";
$resn_exch{"H7'2 AHU"}="2H7* AHU"; $resn_exch{"HO'7 AHU"}="7HO* AHU"; $resn_exch{"H6'1 AHU"}="1H6* AHU";
$resn_exch{"H6'2 AHU"}="2H6* AHU"; $resn_exch{" H1' AHU"}=" H1* AHU"; $resn_exch{"H2'1 AHU"}="1H2* AHU";
$resn_exch{"H2'2 AHU"}="2H2* AHU"; $resn_exch{" H3' AHU"}=" H3* AHU"; $resn_exch{"HO'3 AHU"}="3HO* AHU";
$resn_exch{"HN11 AHX"}="1HN1 AHX"; $resn_exch{"HN12 AHX"}="2HN1 AHX"; $resn_exch{" HB1 AHX"}="1HB  AHX";
$resn_exch{" HB2 AHX"}="2HB  AHX"; $resn_exch{"HOA2 AHX"}="2HOA AHX"; $resn_exch{"H5'1 AHX"}="1H5* AHX";
$resn_exch{"H5'2 AHX"}="2H5* AHX"; $resn_exch{" H4' AHX"}=" H4* AHX"; $resn_exch{" H3' AHX"}=" H3* AHX";
$resn_exch{"HO3' AHX"}="*HO3 AHX"; $resn_exch{" H2' AHX"}=" H2* AHX"; $resn_exch{"HO2' AHX"}="*HO2 AHX";
$resn_exch{" H1' AHX"}=" H1* AHX"; $resn_exch{"HN61 AHX"}="1HN6 AHX"; $resn_exch{"HN62 AHX"}="2HN6 AHX";
$resn_exch{"H101 AHY"}="1H10 AHY"; $resn_exch{"H102 AHY"}="2H10 AHY"; $resn_exch{"H103 AHY"}="3H10 AHY";
$resn_exch{"H9C1 AHY"}="1H9C AHY"; $resn_exch{"H9C2 AHY"}="2H9C AHY"; $resn_exch{"H8C1 AHY"}="1H8C AHY";
$resn_exch{"H8C2 AHY"}="2H8C AHY"; $resn_exch{"H7C1 AHY"}="1H7C AHY"; $resn_exch{"H7C2 AHY"}="2H7C AHY";
$resn_exch{"H6C1 AHY"}="1H6C AHY"; $resn_exch{"H6C2 AHY"}="2H6C AHY"; $resn_exch{"H5C1 AHY"}="1H5C AHY";
$resn_exch{"H5C2 AHY"}="2H5C AHY"; $resn_exch{"H4C1 AHY"}="1H4C AHY"; $resn_exch{"H4C2 AHY"}="2H4C AHY";
$resn_exch{"H3N1 AHY"}="1H3N AHY"; $resn_exch{"H3N2 AHY"}="2H3N AHY"; $resn_exch{" H2' AHZ"}=" H2* AHZ";
$resn_exch{" H3' AHZ"}=" H3* AHZ"; $resn_exch{" H1' AHZ"}=" H1* AHZ"; $resn_exch{"HN61 AHZ"}="1HN6 AHZ";
$resn_exch{"HN62 AHZ"}="2HN6 AHZ"; $resn_exch{" H4' AHZ"}=" H4* AHZ"; $resn_exch{"H5'1 AHZ"}="1H5* AHZ";
$resn_exch{"H5'2 AHZ"}="2H5* AHZ"; $resn_exch{"H7T1 AHZ"}="1H7T AHZ"; $resn_exch{"H7T2 AHZ"}="2H7T AHZ";
$resn_exch{"H6T1 AHZ"}="1H6T AHZ"; $resn_exch{"H6T2 AHZ"}="2H6T AHZ"; $resn_exch{"H1M1 AHZ"}="1H1M AHZ";
$resn_exch{"H1M2 AHZ"}="2H1M AHZ"; $resn_exch{"H1M3 AHZ"}="3H1M AHZ"; $resn_exch{"H111  AI"}="1H11  AI";
$resn_exch{"H112  AI"}="2H11  AI"; $resn_exch{"H113  AI"}="3H11  AI"; $resn_exch{" H81  AI"}="1H8   AI";
$resn_exch{" H82  AI"}="2H8   AI"; $resn_exch{" H83  AI"}="3H8   AI"; $resn_exch{"H121  AI"}="1H12  AI";
$resn_exch{"H122  AI"}="2H12  AI"; $resn_exch{"H123  AI"}="3H12  AI"; $resn_exch{"H21A  AI"}="1H2   AI";
$resn_exch{"H22A  AI"}="2H2   AI"; $resn_exch{"H151  AI"}="1H15  AI"; $resn_exch{"H152  AI"}="2H15  AI";
$resn_exch{"H251  AI"}="1H25  AI"; $resn_exch{"H252  AI"}="2H25  AI"; $resn_exch{"H271  AI"}="1H27  AI";
$resn_exch{"H272  AI"}="2H27  AI"; $resn_exch{"H281  AI"}="1H28  AI"; $resn_exch{"H282  AI"}="2H28  AI";
$resn_exch{"H291  AI"}="1H29  AI"; $resn_exch{"H292  AI"}="2H29  AI"; $resn_exch{"H301  AI"}="1H30  AI";
$resn_exch{"H302  AI"}="2H30  AI"; $resn_exch{"H311  AI"}="1H31  AI"; $resn_exch{"H312  AI"}="2H31  AI";
$resn_exch{"H351  AI"}="1H35  AI"; $resn_exch{"H352  AI"}="2H35  AI"; $resn_exch{"H353  AI"}="3H35  AI";
$resn_exch{"H361  AI"}="1H36  AI"; $resn_exch{"H362  AI"}="2H36  AI"; $resn_exch{"H363  AI"}="3H36  AI";
$resn_exch{"H381  AI"}="1H38  AI"; $resn_exch{"H382  AI"}="2H38  AI"; $resn_exch{"H383  AI"}="3H38  AI";
$resn_exch{"H391  AI"}="1H39  AI"; $resn_exch{"H392  AI"}="2H39  AI"; $resn_exch{"HC61 AI1"}="1HC6 AI1";
$resn_exch{"HC62 AI1"}="2HC6 AI1"; $resn_exch{"HN'1 AI1"}="1HN' AI1"; $resn_exch{"H1'1 AI1"}="1H1* AI1";
$resn_exch{"H1'2 AI1"}="2H1* AI1"; $resn_exch{" H3B AI1"}=" H3* AI1"; $resn_exch{" H4B AI1"}=" H4* AI1";
$resn_exch{" H5' AI1"}=" H5* AI1"; $resn_exch{" H6B AI1"}=" H6* AI1"; $resn_exch{" H7' AI1"}=" H7* AI1";
$resn_exch{"H111 AI2"}="1H11 AI2"; $resn_exch{"H112 AI2"}="2H11 AI2"; $resn_exch{"H113 AI2"}="3H11 AI2";
$resn_exch{" H61 AI2"}="1H6  AI2"; $resn_exch{" H62 AI2"}="2H6  AI2"; $resn_exch{"H221 AI3"}="1H22 AI3";
$resn_exch{"H222 AI3"}="2H22 AI3"; $resn_exch{"H391 AI3"}="1H39 AI3"; $resn_exch{"H392 AI3"}="2H39 AI3";
$resn_exch{"H471 AI3"}="1H47 AI3"; $resn_exch{"H472 AI3"}="2H47 AI3"; $resn_exch{"H473 AI3"}="3H47 AI3";
$resn_exch{"H511 AI3"}="1H51 AI3"; $resn_exch{"H512 AI3"}="2H51 AI3"; $resn_exch{"H513 AI3"}="3H51 AI3";
$resn_exch{" H41 AI7"}="1H4  AI7"; $resn_exch{" H42 AI7"}="2H4  AI7"; $resn_exch{" H61 AI7"}="1H6  AI7";
$resn_exch{" H62 AI7"}="2H6  AI7"; $resn_exch{" H81 AI7"}="1H8  AI7"; $resn_exch{" H82 AI7"}="2H8  AI7";
$resn_exch{" H91 AI7"}="1H9  AI7"; $resn_exch{" H92 AI7"}="2H9  AI7"; $resn_exch{" H93 AI7"}="3H9  AI7";
$resn_exch{"H101 AI7"}="1H10 AI7"; $resn_exch{"H102 AI7"}="2H10 AI7"; $resn_exch{"H121 AI7"}="1H12 AI7";
$resn_exch{"H122 AI7"}="2H12 AI7"; $resn_exch{"H131 AI7"}="1H13 AI7"; $resn_exch{"H132 AI7"}="2H13 AI7";
$resn_exch{" H51 AIA"}="1H5  AIA"; $resn_exch{" H52 AIA"}="2H5  AIA"; $resn_exch{" H71 AIA"}="1H7  AIA";
$resn_exch{" H72 AIA"}="2H7  AIA"; $resn_exch{" H81 AIA"}="1H8  AIA"; $resn_exch{" H82 AIA"}="2H8  AIA";
$resn_exch{"H121 AIA"}="1H12 AIA"; $resn_exch{"H122 AIA"}="2H12 AIA"; $resn_exch{"H141 AIA"}="1H14 AIA";
$resn_exch{"H142 AIA"}="2H14 AIA"; $resn_exch{"H161 AIA"}="1H16 AIA"; $resn_exch{"H162 AIA"}="2H16 AIA";
$resn_exch{"H171 AIA"}="1H17 AIA"; $resn_exch{"H172 AIA"}="2H17 AIA"; $resn_exch{"H181 AIA"}="1H18 AIA";
$resn_exch{"H182 AIA"}="2H18 AIA"; $resn_exch{"H191 AIA"}="1H19 AIA"; $resn_exch{"H192 AIA"}="2H19 AIA";
$resn_exch{" H   AIB"}="1HN  AIB"; $resn_exch{" H2  AIB"}="2HN  AIB"; $resn_exch{"HB11 AIB"}="1HB1 AIB";
$resn_exch{"HB12 AIB"}="2HB1 AIB"; $resn_exch{"HB13 AIB"}="3HB1 AIB"; $resn_exch{"HB21 AIB"}="1HB2 AIB";
$resn_exch{"HB22 AIB"}="2HB2 AIB"; $resn_exch{"HB23 AIB"}="3HB2 AIB"; $resn_exch{" HC8 AIC"}="8HC  AIC";
$resn_exch{" HN1 AIC"}="1HN  AIC"; $resn_exch{" HC4 AIC"}="4HC  AIC"; $resn_exch{" HC1 AIC"}="1HC  AIC";
$resn_exch{" HC2 AIC"}="2HC  AIC"; $resn_exch{" HC3 AIC"}="3HC  AIC"; $resn_exch{" HC7 AIC"}="7HC  AIC";
$resn_exch{" HC9 AIC"}="9HC  AIC"; $resn_exch{"HC01 AIC"}="1HC0 AIC"; $resn_exch{"HC11 AIC"}="1HC1 AIC";
$resn_exch{"HC21 AIC"}="1HC2 AIC"; $resn_exch{"HC31 AIC"}="1HC3 AIC"; $resn_exch{"HC41 AIC"}="1HC4 AIC";
$resn_exch{"H161 AIC"}="1H16 AIC"; $resn_exch{"H162 AIC"}="2H16 AIC"; $resn_exch{"H163 AIC"}="3H16 AIC";
$resn_exch{"HN21 AIC"}="1HN2 AIC"; $resn_exch{"HN22 AIC"}="2HN2 AIC"; $resn_exch{" HO2 AIC"}="2HO  AIC";
$resn_exch{" H61 AIG"}="1H6  AIG"; $resn_exch{" H62 AIG"}="2H6  AIG"; $resn_exch{"HN31 AIG"}="1HN3 AIG";
$resn_exch{"HN32 AIG"}="2HN3 AIG"; $resn_exch{"H1'1 AIG"}="1H1' AIG"; $resn_exch{"H1'2 AIG"}="2H1' AIG";
$resn_exch{"H2'1 AIG"}="1H2' AIG"; $resn_exch{"H2'2 AIG"}="2H2' AIG"; $resn_exch{"H3'1 AIG"}="1H3' AIG";
$resn_exch{"H3'2 AIG"}="2H3' AIG"; $resn_exch{"H4'1 AIG"}="1H4' AIG"; $resn_exch{"H4'2 AIG"}="2H4' AIG";
$resn_exch{"H5'1 AIG"}="1H5' AIG"; $resn_exch{"H5'2 AIG"}="2H5' AIG"; $resn_exch{"H6'1 AIG"}="1H6' AIG";
$resn_exch{"H6'2 AIG"}="2H6' AIG"; $resn_exch{"H6'3 AIG"}="3H6' AIG"; $resn_exch{"H261 AIH"}="1H26 AIH";
$resn_exch{"H262 AIH"}="2H26 AIH"; $resn_exch{"H271 AIH"}="1H27 AIH"; $resn_exch{"H272 AIH"}="2H27 AIH";
$resn_exch{"H291 AIH"}="1H29 AIH"; $resn_exch{"H292 AIH"}="2H29 AIH"; $resn_exch{"H311 AIH"}="1H31 AIH";
$resn_exch{"H312 AIH"}="2H31 AIH"; $resn_exch{"H313 AIH"}="3H31 AIH"; $resn_exch{"H331 AIH"}="1H33 AIH";
$resn_exch{"H332 AIH"}="2H33 AIH"; $resn_exch{"H333 AIH"}="3H33 AIH"; $resn_exch{"H341 AIH"}="1H34 AIH";
$resn_exch{"H342 AIH"}="2H34 AIH"; $resn_exch{"H261 AIJ"}="1H26 AIJ"; $resn_exch{"H262 AIJ"}="2H26 AIJ";
$resn_exch{"H281 AIJ"}="1H28 AIJ"; $resn_exch{"H282 AIJ"}="2H28 AIJ"; $resn_exch{"H283 AIJ"}="3H28 AIJ";
$resn_exch{"H301 AIJ"}="1H30 AIJ"; $resn_exch{"H302 AIJ"}="2H30 AIJ"; $resn_exch{"H311 AIJ"}="1H31 AIJ";
$resn_exch{"H312 AIJ"}="2H31 AIJ"; $resn_exch{"H321 AIJ"}="1H32 AIJ"; $resn_exch{"H322 AIJ"}="2H32 AIJ";
$resn_exch{"H331 AIJ"}="1H33 AIJ"; $resn_exch{"H332 AIJ"}="2H33 AIJ"; $resn_exch{" H51 AIK"}="1H5  AIK";
$resn_exch{" H52 AIK"}="2H5  AIK"; $resn_exch{" H53 AIK"}="3H5  AIK"; $resn_exch{"H121 AIK"}="1H12 AIK";
$resn_exch{"H122 AIK"}="2H12 AIK"; $resn_exch{"H123 AIK"}="3H12 AIK"; $resn_exch{"H271 AIK"}="1H27 AIK";
$resn_exch{"H272 AIK"}="2H27 AIK"; $resn_exch{"H281 AIK"}="1H28 AIK"; $resn_exch{"H282 AIK"}="2H28 AIK";
$resn_exch{"H291 AIK"}="1H29 AIK"; $resn_exch{"H292 AIK"}="2H29 AIK"; $resn_exch{"H321 AIK"}="1H32 AIK";
$resn_exch{"H322 AIK"}="2H32 AIK"; $resn_exch{"H323 AIK"}="3H32 AIK"; $resn_exch{"H311 AIK"}="1H31 AIK";
$resn_exch{"H312 AIK"}="2H31 AIK"; $resn_exch{"H313 AIK"}="3H31 AIK"; $resn_exch{"H231 AIK"}="1H23 AIK";
$resn_exch{"H232 AIK"}="2H23 AIK"; $resn_exch{"H233 AIK"}="3H23 AIK"; $resn_exch{"H241 AIK"}="1H24 AIK";
$resn_exch{"H242 AIK"}="2H24 AIK"; $resn_exch{"H243 AIK"}="3H24 AIK"; $resn_exch{" H11 AIL"}="1H1  AIL";
$resn_exch{" H12 AIL"}="2H1  AIL"; $resn_exch{" H41 AIL"}="1H4  AIL"; $resn_exch{" H42 AIL"}="2H4  AIL";
$resn_exch{" H61 AIL"}="1H6  AIL"; $resn_exch{" H62 AIL"}="2H6  AIL"; $resn_exch{"H101 AIL"}="1H10 AIL";
$resn_exch{"H102 AIL"}="2H10 AIL"; $resn_exch{"H111 AIL"}="1H11 AIL"; $resn_exch{"H112 AIL"}="2H11 AIL";
$resn_exch{"H131 AIL"}="1H13 AIL"; $resn_exch{"H132 AIL"}="2H13 AIL"; $resn_exch{"HC11 AIM"}="1HC1 AIM";
$resn_exch{"HC12 AIM"}="2HC1 AIM"; $resn_exch{"HC21 AIM"}="1HC2 AIM"; $resn_exch{"HC31 AIM"}="1HC3 AIM";
$resn_exch{"HC32 AIM"}="2HC3 AIM"; $resn_exch{"HC41 AIM"}="1HC4 AIM"; $resn_exch{"HC42 AIM"}="2HC4 AIM";
$resn_exch{"HC61 AIM"}="1HC6 AIM"; $resn_exch{"HC62 AIM"}="2HC6 AIM"; $resn_exch{"HN11 AIM"}="1HN1 AIM";
$resn_exch{"HN12 AIM"}="2HN1 AIM"; $resn_exch{"H261 AIM"}="1H26 AIM"; $resn_exch{"HN21 AIM"}="1HN2 AIM";
$resn_exch{"HN22 AIM"}="2HN2 AIM"; $resn_exch{" H91 AIN"}="1H9  AIN"; $resn_exch{" H92 AIN"}="2H9  AIN";
$resn_exch{" H93 AIN"}="3H9  AIN"; $resn_exch{" H71 AIO"}="1H7  AIO"; $resn_exch{" H72 AIO"}="2H7  AIO";
$resn_exch{" H81 AIO"}="1H8  AIO"; $resn_exch{" H82 AIO"}="2H8  AIO"; $resn_exch{" H83 AIO"}="3H8  AIO";
$resn_exch{" H61 AIO"}="1H6  AIO"; $resn_exch{" H62 AIO"}="2H6  AIO"; $resn_exch{" H63 AIO"}="3H6  AIO";
$resn_exch{" H21 AIO"}="1H2  AIO"; $resn_exch{" H22 AIO"}="2H2  AIO"; $resn_exch{" H23 AIO"}="3H2  AIO";
$resn_exch{"HN11 AIO"}="1HN1 AIO"; $resn_exch{"HN12 AIO"}="2HN1 AIO"; $resn_exch{"H131 AIO"}="1H13 AIO";
$resn_exch{"H132 AIO"}="2H13 AIO"; $resn_exch{"H121 AIO"}="1H12 AIO"; $resn_exch{"H122 AIO"}="2H12 AIO";
$resn_exch{"H111 AIO"}="1H11 AIO"; $resn_exch{"H112 AIO"}="2H11 AIO"; $resn_exch{" H51 AIP"}="1H5  AIP";
$resn_exch{" H52 AIP"}="2H5  AIP"; $resn_exch{" H61 AIP"}="1H6  AIP"; $resn_exch{" H62 AIP"}="2H6  AIP";
$resn_exch{" H71 AIP"}="1H7  AIP"; $resn_exch{" H72 AIP"}="2H7  AIP"; $resn_exch{"H131 AIP"}="1H13 AIP";
$resn_exch{"H132 AIP"}="2H13 AIP"; $resn_exch{"H133 AIP"}="3H13 AIP"; $resn_exch{"H141 AIP"}="1H14 AIP";
$resn_exch{"H142 AIP"}="2H14 AIP"; $resn_exch{"H143 AIP"}="3H14 AIP"; $resn_exch{"H181 AIP"}="1H18 AIP";
$resn_exch{"H182 AIP"}="2H18 AIP"; $resn_exch{"H191 AIP"}="1H19 AIP"; $resn_exch{"H192 AIP"}="2H19 AIP";
$resn_exch{"H201 AIP"}="1H20 AIP"; $resn_exch{"H202 AIP"}="2H20 AIP"; $resn_exch{"H251 AIP"}="1H25 AIP";
$resn_exch{"H252 AIP"}="2H25 AIP"; $resn_exch{"HN41 AIP"}="1HN4 AIP"; $resn_exch{"HN42 AIP"}="2HN4 AIP";
$resn_exch{"HN91 AIP"}="1HN9 AIP"; $resn_exch{"HN92 AIP"}="2HN9 AIP"; $resn_exch{" H11 AIQ"}="1H1  AIQ";
$resn_exch{" H91 AIQ"}="1H9  AIQ"; $resn_exch{" H92 AIQ"}="2H9  AIQ"; $resn_exch{"HN21 AIQ"}="1HN2 AIQ";
$resn_exch{"HN22 AIQ"}="2HN2 AIQ"; $resn_exch{"HN11 AIQ"}="1HN1 AIQ"; $resn_exch{" H51 AIQ"}="1H5  AIQ";
$resn_exch{"HN41 AIQ"}="1HN4 AIQ"; $resn_exch{"HN42 AIQ"}="2HN4 AIQ"; $resn_exch{"HN61 AIQ"}="1HN6 AIQ";
$resn_exch{"H111 AIQ"}="1H11 AIQ"; $resn_exch{"H121 AIQ"}="1H12 AIQ"; $resn_exch{"HN61 AIR"}="1HN6 AIR";
$resn_exch{"HN62 AIR"}="2HN6 AIR"; $resn_exch{"HO'2 AIR"}="2HO' AIR"; $resn_exch{"HO'3 AIR"}="3HO' AIR";
$resn_exch{"HC51 AIR"}="1HC5 AIR"; $resn_exch{"HC52 AIR"}="2HC5 AIR"; $resn_exch{" H2' AIS"}=" H2* AIS";
$resn_exch{" H3' AIS"}=" H3* AIS"; $resn_exch{" H4' AIS"}=" H4* AIS"; $resn_exch{"H5'1 AIS"}="1H5* AIS";
$resn_exch{"H5'2 AIS"}="2H5* AIS"; $resn_exch{" H5' AIS"}=" H5* AIS"; $resn_exch{" H1' AIS"}=" H1* AIS";
$resn_exch{"HN61 AIS"}="1HN6 AIS"; $resn_exch{"HN62 AIS"}="2HN6 AIS"; $resn_exch{"H261 AIT"}="1H26 AIT";
$resn_exch{"H262 AIT"}="2H26 AIT"; $resn_exch{"H281 AIT"}="1H28 AIT"; $resn_exch{"H282 AIT"}="2H28 AIT";
$resn_exch{"H283 AIT"}="3H28 AIT"; $resn_exch{"H301 AIT"}="1H30 AIT"; $resn_exch{"H302 AIT"}="2H30 AIT";
$resn_exch{"H311 AIT"}="1H31 AIT"; $resn_exch{"H312 AIT"}="2H31 AIT"; $resn_exch{"H321 AIT"}="1H32 AIT";
$resn_exch{"H322 AIT"}="2H32 AIT"; $resn_exch{"H331 AIT"}="1H33 AIT"; $resn_exch{"H332 AIT"}="2H33 AIT";
$resn_exch{"H261 AIU"}="1H26 AIU"; $resn_exch{"H262 AIU"}="2H26 AIU"; $resn_exch{"H271 AIU"}="1H27 AIU";
$resn_exch{"H272 AIU"}="2H27 AIU"; $resn_exch{"H291 AIU"}="1H29 AIU"; $resn_exch{"H292 AIU"}="2H29 AIU";
$resn_exch{"H311 AIU"}="1H31 AIU"; $resn_exch{"H312 AIU"}="2H31 AIU"; $resn_exch{"H313 AIU"}="3H31 AIU";
$resn_exch{"H331 AIU"}="1H33 AIU"; $resn_exch{"H332 AIU"}="2H33 AIU"; $resn_exch{"H333 AIU"}="3H33 AIU";
$resn_exch{"H341 AIU"}="1H34 AIU"; $resn_exch{"H342 AIU"}="2H34 AIU"; $resn_exch{"H221 AIZ"}="1H22 AIZ";
$resn_exch{"H222 AIZ"}="2H22 AIZ"; $resn_exch{"H261 AIZ"}="1H26 AIZ"; $resn_exch{"H262 AIZ"}="2H26 AIZ";
$resn_exch{"H301 AIZ"}="1H30 AIZ"; $resn_exch{"H302 AIZ"}="2H30 AIZ"; $resn_exch{"H331 AIZ"}="1H33 AIZ";
$resn_exch{"H332 AIZ"}="2H33 AIZ"; $resn_exch{"H333 AIZ"}="3H33 AIZ"; $resn_exch{"H341 AIZ"}="1H34 AIZ";
$resn_exch{"H342 AIZ"}="2H34 AIZ"; $resn_exch{"H343 AIZ"}="3H34 AIZ"; $resn_exch{"H8B1 AJ3"}="1H8B AJ3";
$resn_exch{"H8B2 AJ3"}="2H8B AJ3"; $resn_exch{"H101 AJ3"}="1H10 AJ3"; $resn_exch{"H102 AJ3"}="2H10 AJ3";
$resn_exch{"H121 AJ3"}="1H12 AJ3"; $resn_exch{"H122 AJ3"}="2H12 AJ3"; $resn_exch{" H91 AJA"}="1H9  AJA";
$resn_exch{" H92 AJA"}="2H9  AJA"; $resn_exch{"H251 AJA"}="1H25 AJA"; $resn_exch{"H252 AJA"}="2H25 AJA";
$resn_exch{"H253 AJA"}="3H25 AJA"; $resn_exch{"H201 AJA"}="1H20 AJA"; $resn_exch{"H202 AJA"}="2H20 AJA";
$resn_exch{"H211 AJA"}="1H21 AJA"; $resn_exch{"H212 AJA"}="2H21 AJA"; $resn_exch{"H221 AJA"}="1H22 AJA";
$resn_exch{"H222 AJA"}="2H22 AJA"; $resn_exch{"H231 AJA"}="1H23 AJA"; $resn_exch{"H232 AJA"}="2H23 AJA";
$resn_exch{"H281 AJA"}="1H28 AJA"; $resn_exch{"H282 AJA"}="2H28 AJA"; $resn_exch{"H291 AJA"}="1H29 AJA";
$resn_exch{"H292 AJA"}="2H29 AJA"; $resn_exch{"H293 AJA"}="3H29 AJA"; $resn_exch{"H241 AJA"}="1H24 AJA";
$resn_exch{"H242 AJA"}="2H24 AJA"; $resn_exch{"H243 AJA"}="3H24 AJA"; $resn_exch{"H181 AJA"}="1H18 AJA";
$resn_exch{"H182 AJA"}="2H18 AJA"; $resn_exch{"H183 AJA"}="3H18 AJA"; $resn_exch{"H171 AJA"}="1H17 AJA";
$resn_exch{"H172 AJA"}="2H17 AJA"; $resn_exch{"H173 AJA"}="3H17 AJA"; $resn_exch{"H121 AJA"}="1H12 AJA";
$resn_exch{"H122 AJA"}="2H12 AJA"; $resn_exch{" H41 AJM"}="1H4  AJM"; $resn_exch{" H42 AJM"}="2H4  AJM";
$resn_exch{"H161 AJM"}="1H16 AJM"; $resn_exch{"H162 AJM"}="2H16 AJM"; $resn_exch{"H121 AJM"}="1H12 AJM";
$resn_exch{"H122 AJM"}="2H12 AJM"; $resn_exch{"H181 AJM"}="1H18 AJM"; $resn_exch{"H182 AJM"}="2H18 AJM";
$resn_exch{"H183 AJM"}="3H18 AJM"; $resn_exch{"H401 AKA"}="1H40 AKA"; $resn_exch{"H402 AKA"}="2H40 AKA";
$resn_exch{"H381 AKA"}="1H38 AKA"; $resn_exch{"H382 AKA"}="2H38 AKA"; $resn_exch{"H411 AKA"}="1H41 AKA";
$resn_exch{"H412 AKA"}="2H41 AKA"; $resn_exch{"H413 AKA"}="3H41 AKA"; $resn_exch{"H361 AKA"}="1H36 AKA";
$resn_exch{"H362 AKA"}="2H36 AKA"; $resn_exch{"H363 AKA"}="3H36 AKA"; $resn_exch{"H321 AKA"}="1H32 AKA";
$resn_exch{"H322 AKA"}="2H32 AKA"; $resn_exch{" H4' AKA"}=" H4* AKA"; $resn_exch{" H5' AKA"}=" H5* AKA";
$resn_exch{"H6'1 AKA"}="1H6* AKA"; $resn_exch{"H6'2 AKA"}="2H6* AKA"; $resn_exch{"H6'3 AKA"}="3H6* AKA";
$resn_exch{" H3' AKA"}=" H3* AKA"; $resn_exch{"H8'1 AKA"}="1H8* AKA"; $resn_exch{"H8'2 AKA"}="2H8* AKA";
$resn_exch{"H8'3 AKA"}="3H8* AKA"; $resn_exch{"H7'1 AKA"}="1H7* AKA"; $resn_exch{"H7'2 AKA"}="2H7* AKA";
$resn_exch{"H7'3 AKA"}="3H7* AKA"; $resn_exch{"H2'1 AKA"}="1H2* AKA"; $resn_exch{"H2'2 AKA"}="2H2* AKA";
$resn_exch{" H1' AKA"}=" H1* AKA"; $resn_exch{" H81 AKA"}="1H8  AKA"; $resn_exch{" H82 AKA"}="2H8  AKA";
$resn_exch{"H131 AKA"}="1H13 AKA"; $resn_exch{"H132 AKA"}="2H13 AKA"; $resn_exch{"H141 AKA"}="1H14 AKA";
$resn_exch{"H142 AKA"}="2H14 AKA"; $resn_exch{"H143 AKA"}="3H14 AKA"; $resn_exch{"H101 AKA"}="1H10 AKA";
$resn_exch{"H102 AKA"}="2H10 AKA"; $resn_exch{" H21 AKB"}="1H2  AKB"; $resn_exch{" H41 AKB"}="1H4  AKB";
$resn_exch{" H42 AKB"}="2H4  AKB"; $resn_exch{" H43 AKB"}="3H4  AKB"; $resn_exch{"HN41 AKB"}="1HN4 AKB";
$resn_exch{"HN42 AKB"}="2HN4 AKB"; $resn_exch{"H081 AKC"}="1H08 AKC"; $resn_exch{"H082 AKC"}="2H08 AKC";
$resn_exch{"H101 AKC"}="1H10 AKC"; $resn_exch{"H102 AKC"}="2H10 AKC"; $resn_exch{"H361 AKC"}="1H36 AKC";
$resn_exch{"H362 AKC"}="2H36 AKC"; $resn_exch{"H441 AKC"}="1H44 AKC"; $resn_exch{"H442 AKC"}="2H44 AKC";
$resn_exch{"H451 AKC"}="1H45 AKC"; $resn_exch{"H452 AKC"}="2H45 AKC"; $resn_exch{"H471 AKC"}="1H47 AKC";
$resn_exch{"H472 AKC"}="2H47 AKC"; $resn_exch{"H481 AKC"}="1H48 AKC"; $resn_exch{"H482 AKC"}="2H48 AKC";
$resn_exch{"H483 AKC"}="3H48 AKC"; $resn_exch{" H31 AKG"}="1H3  AKG"; $resn_exch{" H32 AKG"}="2H3  AKG";
$resn_exch{" H41 AKG"}="1H4  AKG"; $resn_exch{" H42 AKG"}="2H4  AKG"; $resn_exch{" H   AKL"}="1HN  AKL";
$resn_exch{" H2  AKL"}="2HN  AKL"; $resn_exch{" HB2 AKL"}="1HB  AKL"; $resn_exch{" HB3 AKL"}="2HB  AKL";
$resn_exch{" HM1 AKL"}="1HM  AKL"; $resn_exch{" HM2 AKL"}="2HM  AKL"; $resn_exch{" H91 AKN"}="1H9  AKN";
$resn_exch{" H92 AKN"}="2H9  AKN"; $resn_exch{"H101 AKN"}="1H10 AKN"; $resn_exch{"H102 AKN"}="2H10 AKN";
$resn_exch{"H131 AKN"}="1H13 AKN"; $resn_exch{"H132 AKN"}="2H13 AKN"; $resn_exch{"H151 AKN"}="1H15 AKN";
$resn_exch{"H152 AKN"}="2H15 AKN"; $resn_exch{"H251 AKN"}="1H25 AKN"; $resn_exch{"H252 AKN"}="2H25 AKN";
$resn_exch{"H301 AKN"}="1H30 AKN"; $resn_exch{"H302 AKN"}="2H30 AKN"; $resn_exch{"H341 AKN"}="1H34 AKN";
$resn_exch{"H342 AKN"}="2H34 AKN"; $resn_exch{"H381 AKN"}="1H38 AKN"; $resn_exch{"H382 AKN"}="2H38 AKN";
$resn_exch{"H391 AKN"}="1H39 AKN"; $resn_exch{"H392 AKN"}="2H39 AKN"; $resn_exch{"H31A AKP"}="1H3  AKP";
$resn_exch{" H32 AKP"}="2H3  AKP"; $resn_exch{" H51 AKP"}="1H5  AKP"; $resn_exch{" H52 AKP"}="2H5  AKP";
$resn_exch{" H81 AKP"}="1H8  AKP"; $resn_exch{" H82 AKP"}="2H8  AKP"; $resn_exch{"H181 AKP"}="1H18 AKP";
$resn_exch{"H182 AKP"}="2H18 AKP"; $resn_exch{"H231 AKP"}="1H23 AKP"; $resn_exch{"H232 AKP"}="2H23 AKP";
$resn_exch{"H233 AKP"}="3H23 AKP"; $resn_exch{"H241 AKP"}="1H24 AKP"; $resn_exch{"H242 AKP"}="2H24 AKP";
$resn_exch{"H271 AKP"}="1H27 AKP"; $resn_exch{"H272 AKP"}="2H27 AKP"; $resn_exch{"H281 AKP"}="1H28 AKP";
$resn_exch{"H282 AKP"}="2H28 AKP"; $resn_exch{"H283 AKP"}="3H28 AKP"; $resn_exch{"H291 AKP"}="1H29 AKP";
$resn_exch{"H292 AKP"}="2H29 AKP"; $resn_exch{"H293 AKP"}="3H29 AKP"; $resn_exch{"H301 AKP"}="1H30 AKP";
$resn_exch{"H302 AKP"}="2H30 AKP"; $resn_exch{"H303 AKP"}="3H30 AKP"; $resn_exch{"H321 AKP"}="1H32 AKP";
$resn_exch{"H322 AKP"}="2H32 AKP"; $resn_exch{"H331 AKP"}="1H33 AKP"; $resn_exch{"H332 AKP"}="2H33 AKP";
$resn_exch{"H341 AKP"}="1H34 AKP"; $resn_exch{"H342 AKP"}="2H34 AKP"; $resn_exch{"H351 AKP"}="1H35 AKP";
$resn_exch{"H352 AKP"}="2H35 AKP"; $resn_exch{"H361 AKP"}="1H36 AKP"; $resn_exch{"H362 AKP"}="2H36 AKP";
$resn_exch{" HA1 AKR"}="1HA  AKR"; $resn_exch{" HB2 AKR"}="2HB  AKR"; $resn_exch{" HB3 AKR"}="3HB  AKR";
$resn_exch{"H101 AKT"}="1H10 AKT"; $resn_exch{"H102 AKT"}="2H10 AKT"; $resn_exch{" H81 AKT"}="1H8  AKT";
$resn_exch{" H82 AKT"}="2H8  AKT"; $resn_exch{"H131 AKT"}="1H13 AKT"; $resn_exch{"H132 AKT"}="2H13 AKT";
$resn_exch{"H141 AKT"}="1H14 AKT"; $resn_exch{"H142 AKT"}="2H14 AKT"; $resn_exch{"H143 AKT"}="3H14 AKT";
$resn_exch{" H1' AKT"}=" H1* AKT"; $resn_exch{"H2'1 AKT"}="1H2* AKT"; $resn_exch{"H2'2 AKT"}="2H2* AKT";
$resn_exch{" H3' AKT"}=" H3* AKT"; $resn_exch{"H8'1 AKT"}="1H8* AKT"; $resn_exch{"H8'2 AKT"}="2H8* AKT";
$resn_exch{"H8'3 AKT"}="3H8* AKT"; $resn_exch{"H7'1 AKT"}="1H7* AKT"; $resn_exch{"H7'2 AKT"}="2H7* AKT";
$resn_exch{"H7'3 AKT"}="3H7* AKT"; $resn_exch{" H4' AKT"}=" H4* AKT"; $resn_exch{" H5' AKT"}=" H5* AKT";
$resn_exch{"H6'1 AKT"}="1H6* AKT"; $resn_exch{"H6'2 AKT"}="2H6* AKT"; $resn_exch{"H6'3 AKT"}="3H6* AKT";
$resn_exch{"H101 AKV"}="1H10 AKV"; $resn_exch{"H102 AKV"}="2H10 AKV"; $resn_exch{"H151 AKV"}="1H15 AKV";
$resn_exch{"H152 AKV"}="2H15 AKV"; $resn_exch{"H153 AKV"}="3H15 AKV"; $resn_exch{" H81 AKV"}="1H8  AKV";
$resn_exch{" H82 AKV"}="2H8  AKV"; $resn_exch{"H131 AKV"}="1H13 AKV"; $resn_exch{"H132 AKV"}="2H13 AKV";
$resn_exch{"H221 AKV"}="1H22 AKV"; $resn_exch{"H222 AKV"}="2H22 AKV"; $resn_exch{"H223 AKV"}="3H22 AKV";
$resn_exch{"H141 AKY"}="1H14 AKY"; $resn_exch{"H142 AKY"}="2H14 AKY"; $resn_exch{"H143 AKY"}="3H14 AKY";
$resn_exch{"H131 AKY"}="1H13 AKY"; $resn_exch{"H132 AKY"}="2H13 AKY"; $resn_exch{" H81 AKY"}="1H8  AKY";
$resn_exch{" H82 AKY"}="2H8  AKY"; $resn_exch{"H461 AKY"}="1H46 AKY"; $resn_exch{"H462 AKY"}="2H46 AKY";
$resn_exch{"H321 AKY"}="1H32 AKY"; $resn_exch{"H322 AKY"}="2H32 AKY"; $resn_exch{"H381 AKY"}="1H38 AKY";
$resn_exch{"H382 AKY"}="2H38 AKY"; $resn_exch{"H401 AKY"}="1H40 AKY"; $resn_exch{"H402 AKY"}="2H40 AKY";
$resn_exch{"H411 AKY"}="1H41 AKY"; $resn_exch{"H412 AKY"}="2H41 AKY"; $resn_exch{"H413 AKY"}="3H41 AKY";
$resn_exch{"H361 AKY"}="1H36 AKY"; $resn_exch{"H362 AKY"}="2H36 AKY"; $resn_exch{"H363 AKY"}="3H36 AKY";
$resn_exch{"H501 AKY"}="1H50 AKY"; $resn_exch{"H502 AKY"}="2H50 AKY"; $resn_exch{"H503 AKY"}="3H50 AKY";
$resn_exch{"H521 AKY"}="1H52 AKY"; $resn_exch{"H522 AKY"}="2H52 AKY"; $resn_exch{"H523 AKY"}="3H52 AKY";
$resn_exch{"H511 AKY"}="1H51 AKY"; $resn_exch{"H512 AKY"}="2H51 AKY"; $resn_exch{"H513 AKY"}="3H51 AKY";
$resn_exch{"H441 AKY"}="1H44 AKY"; $resn_exch{"H442 AKY"}="2H44 AKY"; $resn_exch{"H443 AKY"}="3H44 AKY";
$resn_exch{" HN1 AL0"}="1HN  AL0"; $resn_exch{" HN2 AL0"}="2HN  AL0"; $resn_exch{" HB1 AL0"}="1HB  AL0";
$resn_exch{" HB2 AL0"}="2HB  AL0"; $resn_exch{" H91 AL1"}="1H9  AL1"; $resn_exch{" H92 AL1"}="2H9  AL1";
$resn_exch{"H101 AL1"}="1H10 AL1"; $resn_exch{"H102 AL1"}="2H10 AL1"; $resn_exch{"H241 AL1"}="1H24 AL1";
$resn_exch{"H242 AL1"}="2H24 AL1"; $resn_exch{"H243 AL1"}="3H24 AL1"; $resn_exch{"HNL1 AL1"}="1HNL AL1";
$resn_exch{"HNL2 AL1"}="2HNL AL1"; $resn_exch{" H91 AL2"}="1H9  AL2"; $resn_exch{" H92 AL2"}="2H9  AL2";
$resn_exch{"HNL1 AL2"}="1HNL AL2"; $resn_exch{"HNL2 AL2"}="2HNL AL2"; $resn_exch{"H171 AL2"}="1H17 AL2";
$resn_exch{"H172 AL2"}="2H17 AL2"; $resn_exch{"H173 AL2"}="3H17 AL2"; $resn_exch{" H91 AL3"}="1H9  AL3";
$resn_exch{" H92 AL3"}="2H9  AL3"; $resn_exch{"H111 AL3"}="1H11 AL3"; $resn_exch{"H112 AL3"}="2H11 AL3";
$resn_exch{"HNL1 AL3"}="1HNL AL3"; $resn_exch{"HNL2 AL3"}="2HNL AL3"; $resn_exch{" H91 AL4"}="1H9  AL4";
$resn_exch{" H92 AL4"}="2H9  AL4"; $resn_exch{"H111 AL4"}="1H11 AL4"; $resn_exch{"H112 AL4"}="2H11 AL4";
$resn_exch{"H121 AL4"}="1H12 AL4"; $resn_exch{"H122 AL4"}="2H12 AL4"; $resn_exch{"H141 AL4"}="1H14 AL4";
$resn_exch{"H142 AL4"}="2H14 AL4"; $resn_exch{"H143 AL4"}="3H14 AL4"; $resn_exch{"H161 AL4"}="1H16 AL4";
$resn_exch{"H162 AL4"}="2H16 AL4"; $resn_exch{"HNL1 AL4"}="1HNL AL4"; $resn_exch{"HNL2 AL4"}="2HNL AL4";
$resn_exch{"H171 AL4"}="1H17 AL4"; $resn_exch{"H172 AL4"}="2H17 AL4"; $resn_exch{"H173 AL4"}="3H17 AL4";
$resn_exch{" H91 AL5"}="1H9  AL5"; $resn_exch{" H92 AL5"}="2H9  AL5"; $resn_exch{"H161 AL5"}="1H16 AL5";
$resn_exch{"H162 AL5"}="2H16 AL5"; $resn_exch{"H163 AL5"}="3H16 AL5"; $resn_exch{"HNL1 AL5"}="1HNL AL5";
$resn_exch{"HNL2 AL5"}="2HNL AL5"; $resn_exch{"HNL1 AL6"}="1HNL AL6"; $resn_exch{"HNL2 AL6"}="2HNL AL6";
$resn_exch{"H171 AL6"}="1H17 AL6"; $resn_exch{"H172 AL6"}="2H17 AL6"; $resn_exch{"H173 AL6"}="3H17 AL6";
$resn_exch{" H91 AL7"}="1H9  AL7"; $resn_exch{" H92 AL7"}="2H9  AL7"; $resn_exch{"H161 AL7"}="1H16 AL7";
$resn_exch{"H162 AL7"}="2H16 AL7"; $resn_exch{"H163 AL7"}="3H16 AL7"; $resn_exch{"H241 AL7"}="1H24 AL7";
$resn_exch{"H242 AL7"}="2H24 AL7"; $resn_exch{"H243 AL7"}="3H24 AL7"; $resn_exch{"HNL1 AL7"}="1HNL AL7";
$resn_exch{"HNL2 AL7"}="2HNL AL7"; $resn_exch{" H91 AL8"}="1H9  AL8"; $resn_exch{" H92 AL8"}="2H9  AL8";
$resn_exch{"H161 AL8"}="1H16 AL8"; $resn_exch{"H162 AL8"}="2H16 AL8"; $resn_exch{"H163 AL8"}="3H16 AL8";
$resn_exch{"H241 AL8"}="1H24 AL8"; $resn_exch{"H242 AL8"}="2H24 AL8"; $resn_exch{"H243 AL8"}="3H24 AL8";
$resn_exch{"HNL1 AL8"}="1HNL AL8"; $resn_exch{"HNL2 AL8"}="2HNL AL8"; $resn_exch{" H91 AL9"}="1H9  AL9";
$resn_exch{" H92 AL9"}="2H9  AL9"; $resn_exch{"HNL1 AL9"}="1HNL AL9"; $resn_exch{"HNL2 AL9"}="2HNL AL9";
$resn_exch{"H171 AL9"}="1H17 AL9"; $resn_exch{"H172 AL9"}="2H17 AL9"; $resn_exch{"H173 AL9"}="3H17 AL9";
$resn_exch{" H2  ALA"}=" HN2 ALA"; $resn_exch{" HB1 ALA"}="1HB  ALA"; $resn_exch{" HB2 ALA"}="2HB  ALA";
$resn_exch{" HB3 ALA"}="3HB  ALA"; $resn_exch{" H92 ALB"}="2H9  ALB"; $resn_exch{" H91 ALB"}="1H9  ALB";
$resn_exch{"H102 ALB"}="2H10 ALB"; $resn_exch{"H101 ALB"}="1H10 ALB"; $resn_exch{"H112 ALB"}="2H11 ALB";
$resn_exch{"H111 ALB"}="1H11 ALB"; $resn_exch{"H122 ALB"}="2H12 ALB"; $resn_exch{"H121 ALB"}="1H12 ALB";
$resn_exch{"H132 ALB"}="2H13 ALB"; $resn_exch{"H131 ALB"}="1H13 ALB"; $resn_exch{"H142 ALB"}="2H14 ALB";
$resn_exch{"H141 ALB"}="1H14 ALB"; $resn_exch{"H152 ALB"}="2H15 ALB"; $resn_exch{"H151 ALB"}="1H15 ALB";
$resn_exch{"H162 ALB"}="2H16 ALB"; $resn_exch{"H161 ALB"}="1H16 ALB"; $resn_exch{"H172 ALB"}="2H17 ALB";
$resn_exch{"H171 ALB"}="1H17 ALB"; $resn_exch{"H183 ALB"}="3H18 ALB"; $resn_exch{"H182 ALB"}="2H18 ALB";
$resn_exch{"H181 ALB"}="1H18 ALB"; $resn_exch{"H193 ALB"}="3H19 ALB"; $resn_exch{"H192 ALB"}="2H19 ALB";
$resn_exch{"H191 ALB"}="1H19 ALB"; $resn_exch{"H203 ALB"}="3H20 ALB"; $resn_exch{"H202 ALB"}="2H20 ALB";
$resn_exch{"H201 ALB"}="1H20 ALB"; $resn_exch{"H232 ALB"}="2H23 ALB"; $resn_exch{"H231 ALB"}="1H23 ALB";
$resn_exch{"H363 ALB"}="3H36 ALB"; $resn_exch{"H362 ALB"}="2H36 ALB"; $resn_exch{"H361 ALB"}="1H36 ALB";
$resn_exch{"H7N1 ALB"}="1H7N ALB"; $resn_exch{"H1N3 ALB"}="3H1N ALB"; $resn_exch{"H1N2 ALB"}="2H1N ALB";
$resn_exch{"H1N4 ALB"}="4H1N ALB"; $resn_exch{"H1N5 ALB"}="5H1N ALB"; $resn_exch{" H2  ALC"}=" HN2 ALC";
$resn_exch{" HB2 ALC"}="1HB  ALC"; $resn_exch{" HB3 ALC"}="2HB  ALC"; $resn_exch{"HD23 ALC"}="1HD2 ALC";
$resn_exch{"HD22 ALC"}="2HD2 ALC"; $resn_exch{"HE23 ALC"}="1HE2 ALC"; $resn_exch{"HE22 ALC"}="2HE2 ALC";
$resn_exch{" HZ3 ALC"}="1HZ  ALC"; $resn_exch{" HZ2 ALC"}="2HZ  ALC"; $resn_exch{"HE13 ALC"}="1HE1 ALC";
$resn_exch{"HE12 ALC"}="2HE1 ALC"; $resn_exch{"HD12 ALC"}="1HD1 ALC"; $resn_exch{"HD13 ALC"}="2HD1 ALC";
$resn_exch{" H71 ALD"}="1H7  ALD"; $resn_exch{" H72 ALD"}="2H7  ALD"; $resn_exch{"H181 ALD"}="1H18 ALD";
$resn_exch{"H182 ALD"}="2H18 ALD"; $resn_exch{"H201 ALD"}="1H20 ALD"; $resn_exch{"H202 ALD"}="2H20 ALD";
$resn_exch{"H203 ALD"}="3H20 ALD"; $resn_exch{"H211 ALD"}="1H21 ALD"; $resn_exch{"H212 ALD"}="2H21 ALD";
$resn_exch{"H213 ALD"}="3H21 ALD"; $resn_exch{"H221 ALD"}="1H22 ALD"; $resn_exch{"H222 ALD"}="2H22 ALD";
$resn_exch{"H241 ALD"}="1H24 ALD"; $resn_exch{"H242 ALD"}="2H24 ALD"; $resn_exch{"H261 ALD"}="1H26 ALD";
$resn_exch{"H262 ALD"}="2H26 ALD"; $resn_exch{"H263 ALD"}="3H26 ALD"; $resn_exch{"H271 ALD"}="1H27 ALD";
$resn_exch{"H272 ALD"}="2H27 ALD"; $resn_exch{"H273 ALD"}="3H27 ALD"; $resn_exch{"H301 ALD"}="1H30 ALD";
$resn_exch{"H302 ALD"}="2H30 ALD"; $resn_exch{"H321 ALD"}="1H32 ALD"; $resn_exch{"H322 ALD"}="2H32 ALD";
$resn_exch{"H323 ALD"}="3H32 ALD"; $resn_exch{"H331 ALD"}="1H33 ALD"; $resn_exch{"H332 ALD"}="2H33 ALD";
$resn_exch{"H333 ALD"}="3H33 ALD"; $resn_exch{" H81 ALE"}="1H8  ALE"; $resn_exch{" H82 ALE"}="2H8  ALE";
$resn_exch{" H91 ALE"}="1H9  ALE"; $resn_exch{" H92 ALE"}="2H9  ALE"; $resn_exch{" H93 ALE"}="3H9  ALE";
$resn_exch{" HA1 ALG"}="1HA  ALG"; $resn_exch{" HA2 ALG"}="2HA  ALG"; $resn_exch{" HB1 ALG"}="1HB  ALG";
$resn_exch{" HB2 ALG"}="2HB  ALG"; $resn_exch{" HD1 ALG"}="1HD  ALG"; $resn_exch{" HD2 ALG"}="2HD  ALG";
$resn_exch{"HH21 ALG"}="1HH2 ALG"; $resn_exch{"HH22 ALG"}="2HH2 ALG"; $resn_exch{"HAG1 ALH"}="1HAG ALH";
$resn_exch{"HAG2 ALH"}="2HAG ALH"; $resn_exch{"HAD1 ALH"}="1HAD ALH"; $resn_exch{"HAD2 ALH"}="2HAD ALH";
$resn_exch{"HAC1 ALH"}="1HAC ALH"; $resn_exch{"HAC2 ALH"}="2HAC ALH"; $resn_exch{"HAB1 ALH"}="1HAB ALH";
$resn_exch{"HAB2 ALH"}="2HAB ALH"; $resn_exch{"HAB3 ALH"}="3HAB ALH"; $resn_exch{"H6C1 ALI"}="1H6C ALI";
$resn_exch{"H6C2 ALI"}="2H6C ALI"; $resn_exch{"H8C1 ALI"}="1H8C ALI"; $resn_exch{"H8C2 ALI"}="2H8C ALI";
$resn_exch{"HDP1 ALJ"}="1HDP ALJ"; $resn_exch{"HDP2 ALJ"}="2HDP ALJ"; $resn_exch{"HGP1 ALJ"}="1HGP ALJ";
$resn_exch{"HGP2 ALJ"}="2HGP ALJ"; $resn_exch{"HBP1 ALJ"}="1HBP ALJ"; $resn_exch{"HBP2 ALJ"}="2HBP ALJ";
$resn_exch{"HBC1 ALJ"}="1HBC ALJ"; $resn_exch{"HBC2 ALJ"}="2HBC ALJ"; $resn_exch{"HGC1 ALJ"}="1HGC ALJ";
$resn_exch{"HGC2 ALJ"}="2HGC ALJ"; $resn_exch{"HDC1 ALJ"}="1HDC ALJ"; $resn_exch{"HDC2 ALJ"}="2HDC ALJ";
$resn_exch{"HH11 ALJ"}="1HH1 ALJ"; $resn_exch{"HH12 ALJ"}="2HH1 ALJ"; $resn_exch{"HH21 ALJ"}="1HH2 ALJ";
$resn_exch{"HH22 ALJ"}="2HH2 ALJ"; $resn_exch{" H1' ALL"}=" H1* ALL"; $resn_exch{" H2' ALL"}=" H2* ALL";
$resn_exch{" H3' ALL"}=" H3* ALL"; $resn_exch{" H4' ALL"}=" H4* ALL"; $resn_exch{" H5' ALL"}=" H5* ALL";
$resn_exch{"H6'1 ALL"}="1H6* ALL"; $resn_exch{"H6'2 ALL"}="2H6* ALL"; $resn_exch{"HO'1 ALL"}="1HO* ALL";
$resn_exch{"HO'2 ALL"}="2HO* ALL"; $resn_exch{"HO'3 ALL"}="3HO* ALL"; $resn_exch{"HO'4 ALL"}="4HO* ALL";
$resn_exch{"HO'6 ALL"}="6HO* ALL"; $resn_exch{" H   ALM"}="1HN  ALM"; $resn_exch{" H2  ALM"}="2HN  ALM";
$resn_exch{" HB1 ALM"}="1HB  ALM"; $resn_exch{" HB2 ALM"}="2HB  ALM"; $resn_exch{" HB3 ALM"}="3HB  ALM";
$resn_exch{" HM1 ALM"}="1HM  ALM"; $resn_exch{" HM2 ALM"}="2HM  ALM"; $resn_exch{" HM3 ALM"}="3HM  ALM";
$resn_exch{" H   ALN"}="1HN  ALN"; $resn_exch{" H2  ALN"}="2HN  ALN"; $resn_exch{" HB2 ALN"}="1HB  ALN";
$resn_exch{" HB3 ALN"}="2HB  ALN"; $resn_exch{" H   ALO"}="1HN  ALO"; $resn_exch{" H2  ALO"}="2HN  ALO";
$resn_exch{"HG21 ALO"}="1HG2 ALO"; $resn_exch{"HG22 ALO"}="2HG2 ALO"; $resn_exch{"HG23 ALO"}="3HG2 ALO";
$resn_exch{" HG1 ALO"}=" HOG ALO"; $resn_exch{"H211 ALP"}="1H21 ALP"; $resn_exch{"H212 ALP"}="2H21 ALP";
$resn_exch{"H511 ALP"}="1H51 ALP"; $resn_exch{"H512 ALP"}="2H51 ALP"; $resn_exch{"H513 ALP"}="3H51 ALP";
$resn_exch{"H521 ALP"}="1H52 ALP"; $resn_exch{"H522 ALP"}="2H52 ALP"; $resn_exch{"H523 ALP"}="3H52 ALP";
$resn_exch{" HM1 ALQ"}="1HM  ALQ"; $resn_exch{" HM2 ALQ"}="2HM  ALQ"; $resn_exch{" HM3 ALQ"}="3HM  ALQ";
$resn_exch{" HB1 ALQ"}="1HB  ALQ"; $resn_exch{" HB2 ALQ"}="2HB  ALQ"; $resn_exch{" HB3 ALQ"}="3HB  ALQ";
$resn_exch{" H71 ALR"}="1H7  ALR"; $resn_exch{" H72 ALR"}="2H7  ALR"; $resn_exch{"HN21 ALS"}="1HN2 ALS";
$resn_exch{"HN22 ALS"}="2HN2 ALS"; $resn_exch{"HOS4 ALS"}="4HOS ALS"; $resn_exch{" H   ALT"}="1HN  ALT";
$resn_exch{" H2  ALT"}="2HN  ALT"; $resn_exch{" HB1 ALT"}="1HB  ALT"; $resn_exch{" HB2 ALT"}="2HB  ALT";
$resn_exch{" HB3 ALT"}="3HB  ALT"; $resn_exch{"H111 ALW"}="1H11 ALW"; $resn_exch{"H112 ALW"}="2H11 ALW";
$resn_exch{"H3'1 ALW"}="1H3' ALW"; $resn_exch{"H3'2 ALW"}="2H3' ALW"; $resn_exch{" HC1 ALW"}="1HC  ALW";
$resn_exch{" HC2 ALW"}="2HC  ALW"; $resn_exch{" HC3 ALW"}="3HC  ALW"; $resn_exch{"H6C1 ALW"}="1H6C ALW";
$resn_exch{"H6C2 ALW"}="2H6C ALW"; $resn_exch{"H6C3 ALW"}="3H6C ALW"; $resn_exch{"HH31 ALY"}="1HH3 ALY";
$resn_exch{"HH32 ALY"}="2HH3 ALY"; $resn_exch{"HH33 ALY"}="3HH3 ALY"; $resn_exch{" HZ  ALY"}=" HNZ ALY";
$resn_exch{" HE3 ALY"}="1HCE ALY"; $resn_exch{" HE2 ALY"}="2HCE ALY"; $resn_exch{" HD3 ALY"}="1HCD ALY";
$resn_exch{" HD2 ALY"}="2HCD ALY"; $resn_exch{" HG3 ALY"}="1HCG ALY"; $resn_exch{" HG2 ALY"}="2HCG ALY";
$resn_exch{" HB3 ALY"}="1HCB ALY"; $resn_exch{" HB2 ALY"}="2HCB ALY"; $resn_exch{" H   ALY"}="1HN  ALY";
$resn_exch{" H2  ALY"}="2HN  ALY"; $resn_exch{"HN11 ALZ"}="1HN1 ALZ"; $resn_exch{"HN12 ALZ"}="2HN1 ALZ";
$resn_exch{"H21A ALZ"}="1H2  ALZ"; $resn_exch{"H22A ALZ"}="2H2  ALZ"; $resn_exch{" H31 ALZ"}="1H3  ALZ";
$resn_exch{" H32 ALZ"}="2H3  ALZ"; $resn_exch{" H41 ALZ"}="1H4  ALZ"; $resn_exch{" H42 ALZ"}="2H4  ALZ";
$resn_exch{" H51 ALZ"}="1H5  ALZ"; $resn_exch{" H52 ALZ"}="2H5  ALZ"; $resn_exch{"H101 ALZ"}="1H10 ALZ";
$resn_exch{"H102 ALZ"}="2H10 ALZ"; $resn_exch{"H111 ALZ"}="1H11 ALZ"; $resn_exch{"H112 ALZ"}="2H11 ALZ";
$resn_exch{"H121 ALZ"}="1H12 ALZ"; $resn_exch{"H122 ALZ"}="2H12 ALZ"; $resn_exch{"H161 ALZ"}="1H16 ALZ";
$resn_exch{"H162 ALZ"}="2H16 ALZ"; $resn_exch{"HA31 AM2"}="1HA3 AM2"; $resn_exch{"HA32 AM2"}="2HA3 AM2";
$resn_exch{"HA91 AM2"}="1HA9 AM2"; $resn_exch{"HA92 AM2"}="2HA9 AM2"; $resn_exch{"HA93 AM2"}="3HA9 AM2";
$resn_exch{"HA21 AM2"}="1HA2 AM2"; $resn_exch{"HA22 AM2"}="2HA2 AM2"; $resn_exch{"HB61 AM2"}="1HB6 AM2";
$resn_exch{"HB62 AM2"}="2HB6 AM2"; $resn_exch{"HB41 AM2"}="1HB4 AM2"; $resn_exch{"HB42 AM2"}="2HB4 AM2";
$resn_exch{"HC51 AM2"}="1HC5 AM2"; $resn_exch{"HC52 AM2"}="2HC5 AM2"; $resn_exch{"HC41 AM2"}="1HC4 AM2";
$resn_exch{"HC42 AM2"}="2HC4 AM2"; $resn_exch{"HC61 AM2"}="1HC6 AM2"; $resn_exch{"HC62 AM2"}="2HC6 AM2";
$resn_exch{"H121 AM4"}="1H12 AM4"; $resn_exch{"H122 AM4"}="2H12 AM4"; $resn_exch{"H101 AM4"}="1H10 AM4";
$resn_exch{"H102 AM4"}="2H10 AM4"; $resn_exch{"H103 AM4"}="3H10 AM4"; $resn_exch{" H41 AM7"}="1H4  AM7";
$resn_exch{" H42 AM7"}="2H4  AM7"; $resn_exch{" H11 AM7"}="1H1  AM7"; $resn_exch{" H12 AM7"}="2H1  AM7";
$resn_exch{" H21 AM7"}="1H2  AM7"; $resn_exch{"H22A AM7"}="2H2  AM7"; $resn_exch{" H31 AM7"}="1H3  AM7";
$resn_exch{" H32 AM7"}="2H3  AM7"; $resn_exch{"H331 AM7"}="1H33 AM7"; $resn_exch{"H332 AM7"}="2H33 AM7";
$resn_exch{"H321 AM7"}="1H32 AM7"; $resn_exch{"H322 AM7"}="2H32 AM7"; $resn_exch{"H311 AM7"}="1H31 AM7";
$resn_exch{"H312 AM7"}="2H31 AM7"; $resn_exch{"H351 AM7"}="1H35 AM7"; $resn_exch{"H352 AM7"}="2H35 AM7";
$resn_exch{"H353 AM7"}="3H35 AM7"; $resn_exch{"H341 AM7"}="1H34 AM7"; $resn_exch{"H342 AM7"}="2H34 AM7";
$resn_exch{"H343 AM7"}="3H34 AM7"; $resn_exch{"H301 AM7"}="1H30 AM7"; $resn_exch{"H302 AM7"}="2H30 AM7";
$resn_exch{"HAA1 AM8"}="1HAA AM8"; $resn_exch{"HAA2 AM8"}="2HAA AM8"; $resn_exch{"HAA3 AM8"}="3HAA AM8";
$resn_exch{"HAE1 AM8"}="1HAE AM8"; $resn_exch{"HAE2 AM8"}="2HAE AM8"; $resn_exch{"HAE3 AM8"}="3HAE AM8";
$resn_exch{"HAG1 AM8"}="1HAG AM8"; $resn_exch{"HAG2 AM8"}="2HAG AM8"; $resn_exch{"HAG3 AM8"}="3HAG AM8";
$resn_exch{"HAJ1 AM8"}="1HAJ AM8"; $resn_exch{"HAJ2 AM8"}="2HAJ AM8"; $resn_exch{"HBE1 AM8"}="1HBE AM8";
$resn_exch{"HBE2 AM8"}="2HBE AM8"; $resn_exch{"HBE3 AM8"}="3HBE AM8"; $resn_exch{"H2A1 AMA"}="1H2A AMA";
$resn_exch{"H2A2 AMA"}="2H2A AMA"; $resn_exch{"H2A3 AMA"}="3H2A AMA"; $resn_exch{"H4A1 AMA"}="1H4A AMA";
$resn_exch{"H4A2 AMA"}="2H4A AMA"; $resn_exch{"H5A1 AMA"}="1H5A AMA"; $resn_exch{"H5A2 AMA"}="2H5A AMA";
$resn_exch{"HOP2 AMA"}="2HOP AMA"; $resn_exch{"HOP3 AMA"}="3HOP AMA"; $resn_exch{"HB21 AMA"}="1HB2 AMA";
$resn_exch{"HB22 AMA"}="2HB2 AMA"; $resn_exch{"HB23 AMA"}="3HB2 AMA"; $resn_exch{"HB11 AMA"}="1HB1 AMA";
$resn_exch{"HB12 AMA"}="2HB1 AMA"; $resn_exch{" HE3 AMB"}="3HE  AMB"; $resn_exch{" HE2 AMB"}="2HE  AMB";
$resn_exch{" HE1 AMB"}="1HE  AMB"; $resn_exch{" H2  AMB"}="2H   AMB"; $resn_exch{" H1  AMB"}="1H   AMB";
$resn_exch{" H21 AMC"}="1H2  AMC"; $resn_exch{" H22 AMC"}="2H2  AMC"; $resn_exch{" H31 AMC"}="1H3  AMC";
$resn_exch{" H32 AMC"}="2H3  AMC"; $resn_exch{" H41 AMC"}="1H4  AMC"; $resn_exch{" H42 AMC"}="2H4  AMC";
$resn_exch{" H51 AMC"}="1H5  AMC"; $resn_exch{" H52 AMC"}="2H5  AMC"; $resn_exch{" H61 AMC"}="1H6  AMC";
$resn_exch{" H62 AMC"}="2H6  AMC"; $resn_exch{" HM1 AMC"}="1HM  AMC"; $resn_exch{" HM2 AMC"}="2HM  AMC";
$resn_exch{" HN1 AMC"}="1HN  AMC"; $resn_exch{" HN2 AMC"}="2HN  AMC"; $resn_exch{" HN3 AMC"}="3HN  AMC";
$resn_exch{"HOP2 AMD"}="2HOP AMD"; $resn_exch{"H5'1 AMD"}="1H5* AMD"; $resn_exch{"H5'2 AMD"}="2H5* AMD";
$resn_exch{" H4' AMD"}=" H4* AMD"; $resn_exch{" H1' AMD"}=" H1* AMD"; $resn_exch{" H61 AMD"}="1H6  AMD";
$resn_exch{" H62 AMD"}="2H6  AMD"; $resn_exch{"H2'1 AMD"}="1H2* AMD"; $resn_exch{"H2'2 AMD"}="2H2* AMD";
$resn_exch{" H3' AMD"}=" H3* AMD"; $resn_exch{"HO'3 AMD"}="3HO* AMD"; $resn_exch{" HB1 AMD"}="1HB  AMD";
$resn_exch{" HB2 AMD"}="2HB  AMD"; $resn_exch{"HT23 AME"}="3HT2 AME"; $resn_exch{"HT22 AME"}="2HT2 AME";
$resn_exch{"HT21 AME"}="1HT2 AME"; $resn_exch{" HB2 AME"}="2HB  AME"; $resn_exch{" HB1 AME"}="1HB  AME";
$resn_exch{" HG2 AME"}="2HG  AME"; $resn_exch{" HG1 AME"}="1HG  AME"; $resn_exch{" HE3 AME"}="3HE  AME";
$resn_exch{" HE2 AME"}="2HE  AME"; $resn_exch{" HE1 AME"}="1HE  AME"; $resn_exch{" HN1 AME"}="1HN  AME";
$resn_exch{" H61 AMG"}="1H6  AMG"; $resn_exch{" H62 AMG"}="2H6  AMG"; $resn_exch{" H71 AMG"}="1H7  AMG";
$resn_exch{" H72 AMG"}="2H7  AMG"; $resn_exch{" H73 AMG"}="3H7  AMG"; $resn_exch{" H21 AMH"}="1H2  AMH";
$resn_exch{" H22 AMH"}="2H2  AMH"; $resn_exch{" H31 AMH"}="1H3  AMH"; $resn_exch{" H32 AMH"}="2H3  AMH";
$resn_exch{" H51 AMH"}="1H5  AMH"; $resn_exch{" H52 AMH"}="2H5  AMH"; $resn_exch{" H61 AMH"}="1H6  AMH";
$resn_exch{" H62 AMH"}="2H6  AMH"; $resn_exch{" H71 AMH"}="1H7  AMH"; $resn_exch{" H72 AMH"}="2H7  AMH";
$resn_exch{" HN1 AMH"}="1HN  AMH"; $resn_exch{" HN2 AMH"}="2HN  AMH"; $resn_exch{" H61 AMI"}="1H6  AMI";
$resn_exch{" H62 AMI"}="2H6  AMI"; $resn_exch{" H81 AMI"}="1H8  AMI"; $resn_exch{" H82 AMI"}="2H8  AMI";
$resn_exch{" H83 AMI"}="3H8  AMI"; $resn_exch{" H91 AMI"}="1H9  AMI"; $resn_exch{" H92 AMI"}="2H9  AMI";
$resn_exch{" H93 AMI"}="3H9  AMI"; $resn_exch{"H131 AMK"}="1H13 AMK"; $resn_exch{"H132 AMK"}="2H13 AMK";
$resn_exch{"H161 AMK"}="1H16 AMK"; $resn_exch{"H162 AMK"}="2H16 AMK"; $resn_exch{"H191 AMK"}="1H19 AMK";
$resn_exch{"H192 AMK"}="2H19 AMK"; $resn_exch{"H281 AMK"}="1H28 AMK"; $resn_exch{"H282 AMK"}="2H28 AMK";
$resn_exch{"H283 AMK"}="3H28 AMK"; $resn_exch{"H361 AMK"}="1H36 AMK"; $resn_exch{"H362 AMK"}="2H36 AMK";
$resn_exch{"H411 AMK"}="1H41 AMK"; $resn_exch{"H412 AMK"}="2H41 AMK"; $resn_exch{"H413 AMK"}="3H41 AMK";
$resn_exch{"H451 AMK"}="1H45 AMK"; $resn_exch{"H452 AMK"}="2H45 AMK"; $resn_exch{"H453 AMK"}="3H45 AMK";
$resn_exch{"H651 AMK"}="1H65 AMK"; $resn_exch{"H652 AMK"}="2H65 AMK"; $resn_exch{"H681 AMK"}="1H68 AMK";
$resn_exch{"H682 AMK"}="2H68 AMK"; $resn_exch{"H711 AMK"}="1H71 AMK"; $resn_exch{"H712 AMK"}="2H71 AMK";
$resn_exch{"H801 AMK"}="1H80 AMK"; $resn_exch{"H802 AMK"}="2H80 AMK"; $resn_exch{"H803 AMK"}="3H80 AMK";
$resn_exch{"H841 AMK"}="1H84 AMK"; $resn_exch{"H842 AMK"}="2H84 AMK"; $resn_exch{"H843 AMK"}="3H84 AMK";
$resn_exch{"H881 AMK"}="1H88 AMK"; $resn_exch{"H882 AMK"}="2H88 AMK"; $resn_exch{"H931 AMK"}="1H93 AMK";
$resn_exch{"H932 AMK"}="2H93 AMK"; $resn_exch{"HC11 AML"}="1HC1 AML"; $resn_exch{"HC12 AML"}="2HC1 AML";
$resn_exch{"HC13 AML"}="3HC1 AML"; $resn_exch{"HC21 AML"}="1HC2 AML"; $resn_exch{"HC22 AML"}="2HC2 AML";
$resn_exch{"HC31 AML"}="1HC3 AML"; $resn_exch{"HC32 AML"}="2HC3 AML"; $resn_exch{"HC41 AML"}="1HC4 AML";
$resn_exch{"HC42 AML"}="2HC4 AML"; $resn_exch{"HC51 AML"}="1HC5 AML"; $resn_exch{"HC52 AML"}="2HC5 AML";
$resn_exch{"HN11 AML"}="1HN1 AML"; $resn_exch{"HN12 AML"}="2HN1 AML"; $resn_exch{" HN1 AMM"}="1HN  AMM";
$resn_exch{" HN2 AMM"}="2HN  AMM"; $resn_exch{" HA1 AMM"}="1HA  AMM"; $resn_exch{" HA2 AMM"}="2HA  AMM";
$resn_exch{"HH21 AMM"}="1HH2 AMM"; $resn_exch{"HH22 AMM"}="2HH2 AMM"; $resn_exch{" HB1 AMM"}="1HB  AMM";
$resn_exch{" HB2 AMM"}="2HB  AMM"; $resn_exch{"HD11 AMM"}="1HD1 AMM"; $resn_exch{"HD12 AMM"}="2HD1 AMM";
$resn_exch{"HD13 AMM"}="3HD1 AMM"; $resn_exch{"HD21 AMM"}="1HD2 AMM"; $resn_exch{"HD22 AMM"}="2HD2 AMM";
$resn_exch{"HD23 AMM"}="3HD2 AMM"; $resn_exch{" H31 AMN"}="1H3  AMN"; $resn_exch{" H32 AMN"}="2H3  AMN";
$resn_exch{" H91 AMN"}="1H9  AMN"; $resn_exch{" H92 AMN"}="2H9  AMN"; $resn_exch{"H111 AMN"}="1H11 AMN";
$resn_exch{"H112 AMN"}="2H11 AMN"; $resn_exch{"H113 AMN"}="3H11 AMN"; $resn_exch{"H121 AMN"}="1H12 AMN";
$resn_exch{"H122 AMN"}="2H12 AMN"; $resn_exch{"H123 AMN"}="3H12 AMN"; $resn_exch{"HN91 AMN"}="1HN9 AMN";
$resn_exch{"HN92 AMN"}="2HN9 AMN"; $resn_exch{"HOB1 AMN"}="1HOB AMN"; $resn_exch{"HOP2 AMO"}="2HOP AMO";
$resn_exch{"H5'1 AMO"}="1H5* AMO"; $resn_exch{"H5'2 AMO"}="2H5* AMO"; $resn_exch{" H4' AMO"}=" H4* AMO";
$resn_exch{" H1' AMO"}=" H1* AMO"; $resn_exch{" H61 AMO"}="1H6  AMO"; $resn_exch{" H62 AMO"}="2H6  AMO";
$resn_exch{"H2'2 AMO"}="2H2* AMO"; $resn_exch{" H3' AMO"}=" H3* AMO"; $resn_exch{"HO'3 AMO"}="3HO* AMO";
$resn_exch{" HB1 AMO"}="1HB  AMO"; $resn_exch{" HB2 AMO"}="2HB  AMO"; $resn_exch{"H2'1 AMO"}="1H2* AMO";
$resn_exch{"HOP2 AMP"}="2HOP AMP"; $resn_exch{"HOP3 AMP"}="3HOP AMP"; $resn_exch{"H5'1 AMP"}="1H5* AMP";
$resn_exch{"H5'2 AMP"}="2H5* AMP"; $resn_exch{" H4' AMP"}=" H4* AMP"; $resn_exch{" H3' AMP"}=" H3* AMP";
$resn_exch{"HO3' AMP"}="*HO3 AMP"; $resn_exch{" H2' AMP"}=" H2* AMP"; $resn_exch{"HO2' AMP"}="*HO2 AMP";
$resn_exch{" H1' AMP"}=" H1* AMP"; $resn_exch{"HN61 AMP"}="1HN6 AMP"; $resn_exch{"HN62 AMP"}="2HN6 AMP";
$resn_exch{" HB1 AMQ"}="1HB  AMQ"; $resn_exch{" HB2 AMQ"}="2HB  AMQ"; $resn_exch{"HE21 AMQ"}="1HE2 AMQ";
$resn_exch{"HE22 AMQ"}="2HE2 AMQ"; $resn_exch{"HE23 AMQ"}="3HE2 AMQ"; $resn_exch{"HN71 AMR"}="1HN7 AMR";
$resn_exch{"HN72 AMR"}="2HN7 AMR"; $resn_exch{"HN41 AMR"}="1HN4 AMR"; $resn_exch{"HN42 AMR"}="2HN4 AMR";
$resn_exch{"HN21 AMR"}="1HN2 AMR"; $resn_exch{"HN22 AMR"}="2HN2 AMR"; $resn_exch{"HN11 AMS"}="1HN1 AMS";
$resn_exch{"HN12 AMS"}="2HN1 AMS"; $resn_exch{"HN21 AMS"}="1HN2 AMS"; $resn_exch{"HN22 AMS"}="2HN2 AMS";
$resn_exch{" HN1 AMT"}="1HN  AMT"; $resn_exch{" HN2 AMT"}="2HN  AMT"; $resn_exch{" H61 AMU"}="1H6  AMU";
$resn_exch{" H62 AMU"}="2H6  AMU"; $resn_exch{" H81 AMU"}="1H8  AMU"; $resn_exch{" H82 AMU"}="2H8  AMU";
$resn_exch{" H83 AMU"}="3H8  AMU"; $resn_exch{"H111 AMU"}="1H11 AMU"; $resn_exch{"H112 AMU"}="2H11 AMU";
$resn_exch{"H113 AMU"}="3H11 AMU"; $resn_exch{"H6C1 AMV"}="1H6C AMV"; $resn_exch{"H6C2 AMV"}="2H6C AMV";
$resn_exch{"H8C1 AMV"}="1H8C AMV"; $resn_exch{"H8C2 AMV"}="2H8C AMV"; $resn_exch{"H8C3 AMV"}="3H8C AMV";
$resn_exch{"H111 AMV"}="1H11 AMV"; $resn_exch{"H112 AMV"}="2H11 AMV"; $resn_exch{"H113 AMV"}="3H11 AMV";
$resn_exch{"H121 AMV"}="1H12 AMV"; $resn_exch{"H122 AMV"}="2H12 AMV"; $resn_exch{"H123 AMV"}="3H12 AMV";
$resn_exch{"HOW2 AMW"}="2HOW AMW"; $resn_exch{"HOW3 AMW"}="3HOW AMW"; $resn_exch{"H5'1 AMW"}="1H5* AMW";
$resn_exch{"H5'2 AMW"}="2H5* AMW"; $resn_exch{" H4' AMW"}=" H4* AMW"; $resn_exch{" H3' AMW"}=" H3* AMW";
$resn_exch{"HO3' AMW"}="*HO3 AMW"; $resn_exch{" H2' AMW"}=" H2* AMW"; $resn_exch{"HO2' AMW"}="*HO2 AMW";
$resn_exch{" H1' AMW"}=" H1* AMW"; $resn_exch{"HN61 AMW"}="1HN6 AMW"; $resn_exch{"HN62 AMW"}="2HN6 AMW";
$resn_exch{" H2A AMX"}=" AH2 AMX"; $resn_exch{"H61A AMX"}="AH61 AMX"; $resn_exch{"H62A AMX"}="AH62 AMX";
$resn_exch{" H8A AMX"}=" AH8 AMX"; $resn_exch{" H1B AMX"}="AH1* AMX"; $resn_exch{" H2B AMX"}="AH2* AMX";
$resn_exch{"HO2A AMX"}="AHO2 AMX"; $resn_exch{" H3B AMX"}="AH3* AMX"; $resn_exch{"HOA8 AMX"}="8HOA AMX";
$resn_exch{"HOA9 AMX"}="9HOA AMX"; $resn_exch{" H4B AMX"}="AH4* AMX"; $resn_exch{"H51A AMX"}="AH51 AMX";
$resn_exch{"H52A AMX"}="AH52 AMX"; $resn_exch{"HOA2 AMX"}="2HOA AMX"; $resn_exch{"HOA5 AMX"}="5HOA AMX";
$resn_exch{"H121 AMX"}="1H12 AMX"; $resn_exch{"H122 AMX"}="2H12 AMX"; $resn_exch{"H131 AMX"}="1H13 AMX";
$resn_exch{"H132 AMX"}="2H13 AMX"; $resn_exch{"H133 AMX"}="3H13 AMX"; $resn_exch{"H141 AMX"}="1H14 AMX";
$resn_exch{"H142 AMX"}="2H14 AMX"; $resn_exch{"H143 AMX"}="3H14 AMX"; $resn_exch{" H71 AMX"}="1H7  AMX";
$resn_exch{" H72 AMX"}="2H7  AMX"; $resn_exch{" H61 AMX"}="1H6  AMX"; $resn_exch{" H62 AMX"}="2H6  AMX";
$resn_exch{" H31 AMX"}="1H3  AMX"; $resn_exch{" H32 AMX"}="2H3  AMX"; $resn_exch{" H21 AMX"}="1H2  AMX";
$resn_exch{" H22 AMX"}="2H2  AMX"; $resn_exch{" H11 AMX"}="1H1  AMX"; $resn_exch{" H12 AMX"}="2H1  AMX";
$resn_exch{"HN21 AMX"}="1HN2 AMX"; $resn_exch{"HN22 AMX"}="2HN2 AMX"; $resn_exch{"H151 AMY"}="1H15 AMY";
$resn_exch{"H152 AMY"}="2H15 AMY"; $resn_exch{"H161 AMY"}="1H16 AMY"; $resn_exch{"H162 AMY"}="2H16 AMY";
$resn_exch{"H171 AMY"}="1H17 AMY"; $resn_exch{"H172 AMY"}="2H17 AMY"; $resn_exch{"H181 AMY"}="1H18 AMY";
$resn_exch{"H182 AMY"}="2H18 AMY"; $resn_exch{"H191 AMY"}="1H19 AMY"; $resn_exch{"H192 AMY"}="2H19 AMY";
$resn_exch{"H193 AMY"}="3H19 AMY"; $resn_exch{"H221 AMY"}="1H22 AMY"; $resn_exch{"H222 AMY"}="2H22 AMY";
$resn_exch{"H241 AMY"}="1H24 AMY"; $resn_exch{"H242 AMY"}="2H24 AMY"; $resn_exch{"H243 AMY"}="3H24 AMY";
$resn_exch{"H251 AMY"}="1H25 AMY"; $resn_exch{"H252 AMY"}="2H25 AMY"; $resn_exch{"H253 AMY"}="3H25 AMY";
$resn_exch{"H261 AMY"}="1H26 AMY"; $resn_exch{"H262 AMY"}="2H26 AMY"; $resn_exch{"H263 AMY"}="3H26 AMY";
$resn_exch{"H271 AMY"}="1H27 AMY"; $resn_exch{"H272 AMY"}="2H27 AMY"; $resn_exch{"H273 AMY"}="3H27 AMY";
$resn_exch{" HB1 AN0"}="1HB  AN0"; $resn_exch{" HB2 AN0"}="2HB  AN0"; $resn_exch{" HG1 AN0"}="1HG  AN0";
$resn_exch{" HG2 AN0"}="2HG  AN0"; $resn_exch{" HD1 AN0"}="1HD  AN0"; $resn_exch{" HD2 AN0"}="2HD  AN0";
$resn_exch{" HD3 AN0"}="3HD  AN0"; $resn_exch{" H21 AN0"}="1H2  AN0"; $resn_exch{" H22 AN0"}="2H2  AN0";
$resn_exch{" H23 AN0"}="3H2  AN0"; $resn_exch{"H3B1 AN2"}="1H3B AN2"; $resn_exch{"H3B2 AN2"}="2H3B AN2";
$resn_exch{"HO2B AN2"}="BHO2 AN2"; $resn_exch{"HO2A AN2"}="AHO2 AN2"; $resn_exch{"H5'1 AN2"}="1H5* AN2";
$resn_exch{"H5'2 AN2"}="2H5* AN2"; $resn_exch{" H4' AN2"}=" H4* AN2"; $resn_exch{" H3' AN2"}=" H3* AN2";
$resn_exch{" H2' AN2"}=" H2* AN2"; $resn_exch{" H1' AN2"}=" H1* AN2"; $resn_exch{"H6N1 AN2"}="1H6N AN2";
$resn_exch{"H6N2 AN2"}="2H6N AN2"; $resn_exch{"H201 AN9"}="1H20 AN9"; $resn_exch{"H202 AN9"}="2H20 AN9";
$resn_exch{"H151 AN9"}="1H15 AN9"; $resn_exch{"H152 AN9"}="2H15 AN9"; $resn_exch{"H161 AN9"}="1H16 AN9";
$resn_exch{"H162 AN9"}="2H16 AN9"; $resn_exch{"H171 AN9"}="1H17 AN9"; $resn_exch{"H172 AN9"}="2H17 AN9";
$resn_exch{"H191 AN9"}="1H19 AN9"; $resn_exch{"H192 AN9"}="2H19 AN9"; $resn_exch{"H193 AN9"}="3H19 AN9";
$resn_exch{"H211 AN9"}="1H21 AN9"; $resn_exch{"H212 AN9"}="2H21 AN9"; $resn_exch{"H221 AN9"}="1H22 AN9";
$resn_exch{"H222 AN9"}="2H22 AN9"; $resn_exch{"H223 AN9"}="3H22 AN9"; $resn_exch{"H231 AN9"}="1H23 AN9";
$resn_exch{"H232 AN9"}="2H23 AN9"; $resn_exch{"H241 AN9"}="1H24 AN9"; $resn_exch{"H242 AN9"}="2H24 AN9";
$resn_exch{"H251 AN9"}="1H25 AN9"; $resn_exch{"H252 AN9"}="2H25 AN9"; $resn_exch{"H253 AN9"}="3H25 AN9";
$resn_exch{"H261 AN9"}="1H26 AN9"; $resn_exch{"H262 AN9"}="2H26 AN9"; $resn_exch{"H271 AN9"}="1H27 AN9";
$resn_exch{"H272 AN9"}="2H27 AN9"; $resn_exch{"H273 AN9"}="3H27 AN9"; $resn_exch{" H31 ANA"}="1H3  ANA";
$resn_exch{" H32 ANA"}="2H3  ANA"; $resn_exch{" H91 ANA"}="1H9  ANA"; $resn_exch{" H92 ANA"}="2H9  ANA";
$resn_exch{"H111 ANA"}="1H11 ANA"; $resn_exch{"H112 ANA"}="2H11 ANA"; $resn_exch{"H113 ANA"}="3H11 ANA";
$resn_exch{"H121 ANA"}="1H12 ANA"; $resn_exch{"H122 ANA"}="2H12 ANA"; $resn_exch{"H123 ANA"}="3H12 ANA";
$resn_exch{"HM41 ANA"}="1HM4 ANA"; $resn_exch{"HM42 ANA"}="2HM4 ANA"; $resn_exch{"HM43 ANA"}="3HM4 ANA";
$resn_exch{"HOB1 ANA"}="1HOB ANA"; $resn_exch{" HC1 ANB"}="1HC  ANB"; $resn_exch{"H111 ANB"}="1H11 ANB";
$resn_exch{"H112 ANB"}="2H11 ANB"; $resn_exch{"H121 ANB"}="1H12 ANB"; $resn_exch{"H122 ANB"}="2H12 ANB";
$resn_exch{"HC41 ANB"}="1HC4 ANB"; $resn_exch{"H151 ANB"}="1H15 ANB"; $resn_exch{"H152 ANB"}="2H15 ANB";
$resn_exch{"H161 ANB"}="1H16 ANB"; $resn_exch{"H162 ANB"}="2H16 ANB"; $resn_exch{"H181 ANB"}="1H18 ANB";
$resn_exch{"H182 ANB"}="2H18 ANB"; $resn_exch{"H183 ANB"}="3H18 ANB"; $resn_exch{"H191 ANB"}="1H19 ANB";
$resn_exch{"H192 ANB"}="2H19 ANB"; $resn_exch{"H193 ANB"}="3H19 ANB"; $resn_exch{" HC2 ANB"}="2HC  ANB";
$resn_exch{" HC4 ANB"}="4HC  ANB"; $resn_exch{"HC61 ANB"}="1HC6 ANB"; $resn_exch{"HC62 ANB"}="2HC6 ANB";
$resn_exch{"HC71 ANB"}="1HC7 ANB"; $resn_exch{"HC72 ANB"}="2HC7 ANB"; $resn_exch{" HC8 ANB"}="8HC  ANB";
$resn_exch{" HC9 ANB"}="9HC  ANB"; $resn_exch{"H10C ANC"}="CH10 ANC"; $resn_exch{"H13C ANC"}="CH13 ANC";
$resn_exch{"H14C ANC"}="CH14 ANC"; $resn_exch{" H11 AND"}="1H1  AND"; $resn_exch{" H12 AND"}="2H1  AND";
$resn_exch{" H21 AND"}="1H2  AND"; $resn_exch{" H22 AND"}="2H2  AND"; $resn_exch{" H41 AND"}="1H4  AND";
$resn_exch{" H42 AND"}="2H4  AND"; $resn_exch{" H71 AND"}="1H7  AND"; $resn_exch{" H72 AND"}="2H7  AND";
$resn_exch{"H111 AND"}="1H11 AND"; $resn_exch{"H112 AND"}="2H11 AND"; $resn_exch{"H121 AND"}="1H12 AND";
$resn_exch{"H122 AND"}="2H12 AND"; $resn_exch{"H151 AND"}="1H15 AND"; $resn_exch{"H152 AND"}="2H15 AND";
$resn_exch{"H161 AND"}="1H16 AND"; $resn_exch{"H162 AND"}="2H16 AND"; $resn_exch{"H181 AND"}="1H18 AND";
$resn_exch{"H182 AND"}="2H18 AND"; $resn_exch{"H183 AND"}="3H18 AND"; $resn_exch{"H191 AND"}="1H19 AND";
$resn_exch{"H192 AND"}="2H19 AND"; $resn_exch{"H193 AND"}="3H19 AND"; $resn_exch{"HN61 ANE"}="1HN6 ANE";
$resn_exch{"HN62 ANE"}="2HN6 ANE"; $resn_exch{" H21 ANG"}="1H2  ANG"; $resn_exch{" H22 ANG"}="2H2  ANG";
$resn_exch{" H81 ANG"}="1H8  ANG"; $resn_exch{" H82 ANG"}="2H8  ANG"; $resn_exch{"H301 ANH"}="1H30 ANH";
$resn_exch{"H302 ANH"}="2H30 ANH"; $resn_exch{"H161 ANH"}="1H16 ANH"; $resn_exch{"H162 ANH"}="2H16 ANH";
$resn_exch{"H163 ANH"}="3H16 ANH"; $resn_exch{"H281 ANH"}="1H28 ANH"; $resn_exch{"H282 ANH"}="2H28 ANH";
$resn_exch{"H101 ANH"}="1H10 ANH"; $resn_exch{"H102 ANH"}="2H10 ANH"; $resn_exch{" H11 ANH"}="1H1  ANH";
$resn_exch{"H12A ANH"}="2H1  ANH"; $resn_exch{"H13A ANH"}="3H1  ANH"; $resn_exch{" HN1 ANI"}="1HN  ANI";
$resn_exch{" HN2 ANI"}="2HN  ANI"; $resn_exch{"H111 ANJ"}="1H11 ANJ"; $resn_exch{"H112 ANJ"}="2H11 ANJ";
$resn_exch{"H113 ANJ"}="3H11 ANJ"; $resn_exch{"H161 ANJ"}="1H16 ANJ"; $resn_exch{"H162 ANJ"}="2H16 ANJ";
$resn_exch{"H163 ANJ"}="3H16 ANJ"; $resn_exch{"H181 ANJ"}="1H18 ANJ"; $resn_exch{"H182 ANJ"}="2H18 ANJ";
$resn_exch{"H191 ANJ"}="1H19 ANJ"; $resn_exch{"H192 ANJ"}="2H19 ANJ"; $resn_exch{"H201 ANJ"}="1H20 ANJ";
$resn_exch{"H202 ANJ"}="2H20 ANJ"; $resn_exch{"H211 ANJ"}="1H21 ANJ"; $resn_exch{"H212 ANJ"}="2H21 ANJ";
$resn_exch{"H221 ANJ"}="1H22 ANJ"; $resn_exch{"H222 ANJ"}="2H22 ANJ"; $resn_exch{"H231 ANJ"}="1H23 ANJ";
$resn_exch{"H232 ANJ"}="2H23 ANJ"; $resn_exch{"H233 ANJ"}="3H23 ANJ"; $resn_exch{"H261 ANJ"}="1H26 ANJ";
$resn_exch{"H262 ANJ"}="2H26 ANJ"; $resn_exch{"H263 ANJ"}="3H26 ANJ"; $resn_exch{"H271 ANJ"}="1H27 ANJ";
$resn_exch{"H272 ANJ"}="2H27 ANJ"; $resn_exch{"H281 ANJ"}="1H28 ANJ"; $resn_exch{"H282 ANJ"}="2H28 ANJ";
$resn_exch{"H283 ANJ"}="3H28 ANJ"; $resn_exch{" HN1 ANL"}="1HN  ANL"; $resn_exch{" HN2 ANL"}="2HN  ANL";
$resn_exch{" H41 ANM"}="1H4  ANM"; $resn_exch{" H42 ANM"}="2H4  ANM"; $resn_exch{" H61 ANM"}="1H6  ANM";
$resn_exch{" H62 ANM"}="2H6  ANM"; $resn_exch{" H63 ANM"}="3H6  ANM"; $resn_exch{"H141 ANM"}="1H14 ANM";
$resn_exch{"H142 ANM"}="2H14 ANM"; $resn_exch{"H143 ANM"}="3H14 ANM"; $resn_exch{"H151 ANM"}="1H15 ANM";
$resn_exch{"H152 ANM"}="2H15 ANM"; $resn_exch{" H81 ANN"}="1H8  ANN"; $resn_exch{" H82 ANN"}="2H8  ANN";
$resn_exch{" H83 ANN"}="3H8  ANN"; $resn_exch{" H11 ANO"}="1H1  ANO"; $resn_exch{" H12 ANO"}="2H1  ANO";
$resn_exch{" H21 ANO"}="1H2  ANO"; $resn_exch{" H22 ANO"}="2H2  ANO"; $resn_exch{" H41 ANO"}="1H4  ANO";
$resn_exch{" H42 ANO"}="2H4  ANO"; $resn_exch{" H61 ANO"}="1H6  ANO"; $resn_exch{" H62 ANO"}="2H6  ANO";
$resn_exch{" H71 ANO"}="1H7  ANO"; $resn_exch{" H72 ANO"}="2H7  ANO"; $resn_exch{"H111 ANO"}="1H11 ANO";
$resn_exch{"H112 ANO"}="2H11 ANO"; $resn_exch{"H121 ANO"}="1H12 ANO"; $resn_exch{"H122 ANO"}="2H12 ANO";
$resn_exch{"H151 ANO"}="1H15 ANO"; $resn_exch{"H152 ANO"}="2H15 ANO"; $resn_exch{"H161 ANO"}="1H16 ANO";
$resn_exch{"H162 ANO"}="2H16 ANO"; $resn_exch{"H181 ANO"}="1H18 ANO"; $resn_exch{"H182 ANO"}="2H18 ANO";
$resn_exch{"H183 ANO"}="3H18 ANO"; $resn_exch{"H191 ANO"}="1H19 ANO"; $resn_exch{"H192 ANO"}="2H19 ANO";
$resn_exch{"H193 ANO"}="3H19 ANO"; $resn_exch{"HOG2 ANP"}="2HOG ANP"; $resn_exch{"HOG3 ANP"}="3HOG ANP";
$resn_exch{"HOB2 ANP"}="2HOB ANP"; $resn_exch{"HNB1 ANP"}="1HNB ANP"; $resn_exch{"HOA2 ANP"}="2HOA ANP";
$resn_exch{"H5'1 ANP"}="1H5* ANP"; $resn_exch{"H5'2 ANP"}="2H5* ANP"; $resn_exch{" H4' ANP"}=" H4* ANP";
$resn_exch{" H3' ANP"}=" H3* ANP"; $resn_exch{"HO3' ANP"}="*HO3 ANP"; $resn_exch{" H2' ANP"}=" H2* ANP";
$resn_exch{"HO2' ANP"}="*HO2 ANP"; $resn_exch{" H1' ANP"}=" H1* ANP"; $resn_exch{"HN61 ANP"}="1HN6 ANP";
$resn_exch{"HN62 ANP"}="2HN6 ANP"; $resn_exch{"H151 ANR"}="1H15 ANR"; $resn_exch{"H152 ANR"}="2H15 ANR";
$resn_exch{"H201 ANR"}="1H20 ANR"; $resn_exch{"H202 ANR"}="2H20 ANR"; $resn_exch{"H203 ANR"}="3H20 ANR";
$resn_exch{"H191 ANR"}="1H19 ANR"; $resn_exch{"H192 ANR"}="2H19 ANR"; $resn_exch{"H193 ANR"}="3H19 ANR";
$resn_exch{"H171 ANR"}="1H17 ANR"; $resn_exch{"H172 ANR"}="2H17 ANR"; $resn_exch{"H173 ANR"}="3H17 ANR";
$resn_exch{"H181 ANR"}="1H18 ANR"; $resn_exch{"H182 ANR"}="2H18 ANR"; $resn_exch{"H183 ANR"}="3H18 ANR";
$resn_exch{" H21 ANR"}="1H2  ANR"; $resn_exch{" H22 ANR"}="2H2  ANR"; $resn_exch{" H31 ANR"}="1H3  ANR";
$resn_exch{" H32 ANR"}="2H3  ANR"; $resn_exch{"H161 ANR"}="1H16 ANR"; $resn_exch{"H162 ANR"}="2H16 ANR";
$resn_exch{"H163 ANR"}="3H16 ANR"; $resn_exch{"HM11 ANS"}="1HM1 ANS"; $resn_exch{"HM12 ANS"}="2HM1 ANS";
$resn_exch{"HM13 ANS"}="3HM1 ANS"; $resn_exch{"HM21 ANS"}="1HM2 ANS"; $resn_exch{"HM22 ANS"}="2HM2 ANS";
$resn_exch{"HM23 ANS"}="3HM2 ANS"; $resn_exch{"HOS3 ANS"}="3HOS ANS"; $resn_exch{"H11A ANT"}="1H1  ANT";
$resn_exch{"H12A ANT"}="2H1  ANT"; $resn_exch{"HM81 ANT"}="1HM8 ANT"; $resn_exch{"HM82 ANT"}="2HM8 ANT";
$resn_exch{"HM83 ANT"}="3HM8 ANT"; $resn_exch{"HN51 ANT"}="1HN5 ANT"; $resn_exch{"HN52 ANT"}="2HN5 ANT";
$resn_exch{" H3' ANU"}=" H3* ANU"; $resn_exch{" H2' ANU"}=" H2* ANU"; $resn_exch{" H1' ANU"}=" H1* ANU";
$resn_exch{" H4' ANU"}=" H4* ANU"; $resn_exch{"H5'1 ANU"}="1H5* ANU"; $resn_exch{"H5'2 ANU"}="2H5* ANU";
$resn_exch{" H5' ANU"}=" H5* ANU"; $resn_exch{"H151 ANY"}="1H15 ANY"; $resn_exch{"H152 ANY"}="2H15 ANY";
$resn_exch{"H161 ANY"}="1H16 ANY"; $resn_exch{"H162 ANY"}="2H16 ANY"; $resn_exch{"H261 ANY"}="1H26 ANY";
$resn_exch{"H262 ANY"}="2H26 ANY"; $resn_exch{"H263 ANY"}="3H26 ANY"; $resn_exch{"H271 ANY"}="1H27 ANY";
$resn_exch{"H272 ANY"}="2H27 ANY"; $resn_exch{"H273 ANY"}="3H27 ANY"; $resn_exch{"H231 ANY"}="1H23 ANY";
$resn_exch{"H232 ANY"}="2H23 ANY"; $resn_exch{"H171 ANY"}="1H17 ANY"; $resn_exch{"H172 ANY"}="2H17 ANY";
$resn_exch{"H241 ANY"}="1H24 ANY"; $resn_exch{"H242 ANY"}="2H24 ANY"; $resn_exch{"H243 ANY"}="3H24 ANY";
$resn_exch{"H251 ANY"}="1H25 ANY"; $resn_exch{"H252 ANY"}="2H25 ANY"; $resn_exch{"H253 ANY"}="3H25 ANY";
$resn_exch{"H181 ANY"}="1H18 ANY"; $resn_exch{"H182 ANY"}="2H18 ANY"; $resn_exch{"H191 ANY"}="1H19 ANY";
$resn_exch{"H192 ANY"}="2H19 ANY"; $resn_exch{"H281 ANY"}="1H28 ANY"; $resn_exch{"H282 ANY"}="2H28 ANY";
$resn_exch{"H291 ANY"}="1H29 ANY"; $resn_exch{"H292 ANY"}="2H29 ANY"; $resn_exch{"H293 ANY"}="3H29 ANY";
$resn_exch{"HBC1 ANZ"}="1HBC ANZ"; $resn_exch{"HBC2 ANZ"}="2HBC ANZ"; $resn_exch{" H1' ANZ"}=" H1* ANZ";
$resn_exch{"H6N1 ANZ"}="1H6N ANZ"; $resn_exch{"H6N2 ANZ"}="2H6N ANZ"; $resn_exch{"H5'1 ANZ"}="1H5* ANZ";
$resn_exch{"H5'2 ANZ"}="2H5* ANZ"; $resn_exch{" H4' ANZ"}=" H4* ANZ"; $resn_exch{" H3' ANZ"}=" H3* ANZ";
$resn_exch{" H2' ANZ"}=" H2* ANZ"; $resn_exch{"H11A AO1"}="1H1  AO1"; $resn_exch{" H12 AO1"}="2H1  AO1";
$resn_exch{" H13 AO1"}="3H1  AO1"; $resn_exch{"H21A AO1"}="1H2  AO1"; $resn_exch{"H22A AO1"}="2H2  AO1";
$resn_exch{" H41 AO1"}="1H4  AO1"; $resn_exch{" H42 AO1"}="2H4  AO1"; $resn_exch{" H51 AO1"}="1H5  AO1";
$resn_exch{" H52 AO1"}="2H5  AO1"; $resn_exch{"H141 AO1"}="1H14 AO1"; $resn_exch{"H142 AO1"}="2H14 AO1";
$resn_exch{"H143 AO1"}="3H14 AO1"; $resn_exch{"HN71 AO1"}="1HN7 AO1"; $resn_exch{"HN72 AO1"}="2HN7 AO1";
$resn_exch{"H11A AO2"}="1H1  AO2"; $resn_exch{" H12 AO2"}="2H1  AO2"; $resn_exch{"H21A AO2"}="1H2  AO2";
$resn_exch{" H22 AO2"}="2H2  AO2"; $resn_exch{" H31 AO2"}="1H3  AO2"; $resn_exch{" H32 AO2"}="2H3  AO2";
$resn_exch{" H41 AO2"}="1H4  AO2"; $resn_exch{" H42 AO2"}="2H4  AO2"; $resn_exch{" H51 AO2"}="1H5  AO2";
$resn_exch{" H52 AO2"}="2H5  AO2"; $resn_exch{" H71 AO2"}="1H7  AO2"; $resn_exch{" H72 AO2"}="2H7  AO2";
$resn_exch{"HN91 AO2"}="1HN9 AO2"; $resn_exch{"HN92 AO2"}="2HN9 AO2"; $resn_exch{"H221 AO2"}="1H22 AO2";
$resn_exch{"H222 AO2"}="2H22 AO2"; $resn_exch{"H223 AO2"}="3H22 AO2"; $resn_exch{" H61 AO3"}="1H6  AO3";
$resn_exch{" H62 AO3"}="2H6  AO3"; $resn_exch{" H81 AO3"}="1H8  AO3"; $resn_exch{" H82 AO3"}="2H8  AO3";
$resn_exch{" H83 AO3"}="3H8  AO3"; $resn_exch{"H161 AO3"}="1H16 AO3"; $resn_exch{"H162 AO3"}="2H16 AO3";
$resn_exch{"H181 AO3"}="1H18 AO3"; $resn_exch{"H182 AO3"}="2H18 AO3"; $resn_exch{"H183 AO3"}="3H18 AO3";
$resn_exch{"H261 AO3"}="1H26 AO3"; $resn_exch{"H262 AO3"}="2H26 AO3"; $resn_exch{"H281 AO3"}="1H28 AO3";
$resn_exch{"H282 AO3"}="2H28 AO3"; $resn_exch{"H283 AO3"}="3H28 AO3"; $resn_exch{"H291 AO3"}="1H29 AO3";
$resn_exch{"H292 AO3"}="2H29 AO3"; $resn_exch{"H293 AO3"}="3H29 AO3"; $resn_exch{" H21 AO5"}="1H2  AO5";
$resn_exch{"H22A AO5"}="2H2  AO5"; $resn_exch{" H23 AO5"}="3H2  AO5"; $resn_exch{" H31 AO5"}="1H3  AO5";
$resn_exch{" H32 AO5"}="2H3  AO5"; $resn_exch{" H33 AO5"}="3H3  AO5"; $resn_exch{" H51 AO5"}="1H5  AO5";
$resn_exch{" H52 AO5"}="2H5  AO5"; $resn_exch{" H61 AO5"}="1H6  AO5"; $resn_exch{" H62 AO5"}="2H6  AO5";
$resn_exch{"HN81 AO5"}="1HN8 AO5"; $resn_exch{"HN82 AO5"}="2HN8 AO5"; $resn_exch{"H1C1 AOA"}="1H1C AOA";
$resn_exch{"H1C2 AOA"}="2H1C AOA"; $resn_exch{"H1N1 AOA"}="1H1N AOA"; $resn_exch{"H1N2 AOA"}="2H1N AOA";
$resn_exch{" HC1 AOE"}="1HC  AOE"; $resn_exch{" HC2 AOE"}="2HC  AOE"; $resn_exch{" HO3 AOE"}="3HO  AOE";
$resn_exch{" HC4 AOE"}="4HC  AOE"; $resn_exch{"HC61 AOE"}="1HC6 AOE"; $resn_exch{"HC62 AOE"}="2HC6 AOE";
$resn_exch{" HC7 AOE"}="7HC  AOE"; $resn_exch{" HC8 AOE"}="8HC  AOE"; $resn_exch{" HC9 AOE"}="9HC  AOE";
$resn_exch{"H111 AOE"}="1H11 AOE"; $resn_exch{"H112 AOE"}="2H11 AOE"; $resn_exch{"H121 AOE"}="1H12 AOE";
$resn_exch{"H122 AOE"}="2H12 AOE"; $resn_exch{"HC41 AOE"}="1HC4 AOE"; $resn_exch{"H151 AOE"}="1H15 AOE";
$resn_exch{"H152 AOE"}="2H15 AOE"; $resn_exch{"H161 AOE"}="1H16 AOE"; $resn_exch{"H162 AOE"}="2H16 AOE";
$resn_exch{"HC71 AOE"}="1HC7 AOE"; $resn_exch{"H181 AOE"}="1H18 AOE"; $resn_exch{"H182 AOE"}="2H18 AOE";
$resn_exch{"H183 AOE"}="3H18 AOE"; $resn_exch{"HO71 AOE"}="1HO7 AOE"; $resn_exch{"H191 AOE"}="1H19 AOE";
$resn_exch{"H192 AOE"}="2H19 AOE"; $resn_exch{"H201 AOE"}="1H20 AOE"; $resn_exch{"H202 AOE"}="2H20 AOE";
$resn_exch{"H211 AOE"}="1H21 AOE"; $resn_exch{"H212 AOE"}="2H21 AOE"; $resn_exch{"H221 AOE"}="1H22 AOE";
$resn_exch{"H222 AOE"}="2H22 AOE"; $resn_exch{"H231 AOE"}="1H23 AOE"; $resn_exch{"H232 AOE"}="2H23 AOE";
$resn_exch{"H241 AOE"}="1H24 AOE"; $resn_exch{"H242 AOE"}="2H24 AOE"; $resn_exch{"H251 AOE"}="1H25 AOE";
$resn_exch{"H252 AOE"}="2H25 AOE"; $resn_exch{"H261 AOE"}="1H26 AOE"; $resn_exch{"H262 AOE"}="2H26 AOE";
$resn_exch{"H271 AOE"}="1H27 AOE"; $resn_exch{"H272 AOE"}="2H27 AOE"; $resn_exch{"H281 AOE"}="1H28 AOE";
$resn_exch{"H282 AOE"}="2H28 AOE"; $resn_exch{"H301 AOE"}="1H30 AOE"; $resn_exch{"H302 AOE"}="2H30 AOE";
$resn_exch{"H303 AOE"}="3H30 AOE"; $resn_exch{"H311 AOE"}="1H31 AOE"; $resn_exch{"H312 AOE"}="2H31 AOE";
$resn_exch{"H321 AOE"}="1H32 AOE"; $resn_exch{"H322 AOE"}="2H32 AOE"; $resn_exch{"H331 AOE"}="1H33 AOE";
$resn_exch{"H332 AOE"}="2H33 AOE"; $resn_exch{"H341 AOE"}="1H34 AOE"; $resn_exch{"H342 AOE"}="2H34 AOE";
$resn_exch{"H343 AOE"}="3H34 AOE"; $resn_exch{" H61 AOG"}="1H6  AOG"; $resn_exch{" H62 AOG"}="2H6  AOG";
$resn_exch{"HN31 AOG"}="1HN3 AOG"; $resn_exch{"HN32 AOG"}="2HN3 AOG"; $resn_exch{"H1'1 AOG"}="1H1' AOG";
$resn_exch{"H1'2 AOG"}="2H1' AOG"; $resn_exch{"H2'1 AOG"}="1H2' AOG"; $resn_exch{"H2'2 AOG"}="2H2' AOG";
$resn_exch{"H3'1 AOG"}="1H3' AOG"; $resn_exch{"H3'2 AOG"}="2H3' AOG"; $resn_exch{"H4'1 AOG"}="1H4' AOG";
$resn_exch{"H4'2 AOG"}="2H4' AOG"; $resn_exch{"H5'1 AOG"}="1H5' AOG"; $resn_exch{"H5'2 AOG"}="2H5' AOG";
$resn_exch{"H6'1 AOG"}="1H6' AOG"; $resn_exch{"H6'2 AOG"}="2H6' AOG"; $resn_exch{"H7'1 AOG"}="1H7' AOG";
$resn_exch{"H7'2 AOG"}="2H7' AOG"; $resn_exch{"H8'1 AOG"}="1H8' AOG"; $resn_exch{"H8'2 AOG"}="2H8' AOG";
$resn_exch{"H8'3 AOG"}="3H8' AOG"; $resn_exch{"HN51 AOL"}="1HN5 AOL"; $resn_exch{"HN52 AOL"}="2HN5 AOL";
$resn_exch{"HC11 AOM"}="1HC1 AOM"; $resn_exch{"HC12 AOM"}="2HC1 AOM"; $resn_exch{"H111 AOM"}="1H11 AOM";
$resn_exch{"H112 AOM"}="2H11 AOM"; $resn_exch{"H121 AOM"}="1H12 AOM"; $resn_exch{"H122 AOM"}="2H12 AOM";
$resn_exch{"H14C AOM"}="CH14 AOM"; $resn_exch{"H151 AOM"}="1H15 AOM"; $resn_exch{"H152 AOM"}="2H15 AOM";
$resn_exch{"H161 AOM"}="1H16 AOM"; $resn_exch{"H162 AOM"}="2H16 AOM"; $resn_exch{"H17C AOM"}="CH17 AOM";
$resn_exch{"H181 AOM"}="1H18 AOM"; $resn_exch{"H182 AOM"}="2H18 AOM"; $resn_exch{"H183 AOM"}="3H18 AOM";
$resn_exch{"H191 AOM"}="1H19 AOM"; $resn_exch{"H192 AOM"}="2H19 AOM"; $resn_exch{"H193 AOM"}="3H19 AOM";
$resn_exch{"HC21 AOM"}="1HC2 AOM"; $resn_exch{"HC22 AOM"}="2HC2 AOM"; $resn_exch{"HC41 AOM"}="1HC4 AOM";
$resn_exch{"HC42 AOM"}="2HC4 AOM"; $resn_exch{"HC61 AOM"}="1HC6 AOM"; $resn_exch{"HC62 AOM"}="2HC6 AOM";
$resn_exch{"HC71 AOM"}="1HC7 AOM"; $resn_exch{"HC72 AOM"}="2HC7 AOM"; $resn_exch{"HC11 AON"}="1HC1 AON";
$resn_exch{"HC12 AON"}="2HC1 AON"; $resn_exch{"H111 AON"}="1H11 AON"; $resn_exch{"H112 AON"}="2H11 AON";
$resn_exch{"H121 AON"}="1H12 AON"; $resn_exch{"H122 AON"}="2H12 AON"; $resn_exch{"H14C AON"}="CH14 AON";
$resn_exch{"H151 AON"}="1H15 AON"; $resn_exch{"H152 AON"}="2H15 AON"; $resn_exch{"H161 AON"}="1H16 AON";
$resn_exch{"H162 AON"}="2H16 AON"; $resn_exch{"H17C AON"}="CH17 AON"; $resn_exch{"H181 AON"}="1H18 AON";
$resn_exch{"H182 AON"}="2H18 AON"; $resn_exch{"H183 AON"}="3H18 AON"; $resn_exch{"H191 AON"}="1H19 AON";
$resn_exch{"H192 AON"}="2H19 AON"; $resn_exch{"H193 AON"}="3H19 AON"; $resn_exch{"HC21 AON"}="1HC2 AON";
$resn_exch{"HC22 AON"}="2HC2 AON"; $resn_exch{"HC41 AON"}="1HC4 AON"; $resn_exch{"HC42 AON"}="2HC4 AON";
$resn_exch{"HC61 AON"}="1HC6 AON"; $resn_exch{"HC62 AON"}="2HC6 AON"; $resn_exch{"HC71 AON"}="1HC7 AON";
$resn_exch{"HC72 AON"}="2HC7 AON"; $resn_exch{" H21 AOP"}="1H2  AOP"; $resn_exch{" H22 AOP"}="2H2  AOP";
$resn_exch{" H51 AOP"}="1H5  AOP"; $resn_exch{" H52 AOP"}="2H5  AOP"; $resn_exch{" H61 AOP"}="1H6  AOP";
$resn_exch{" H62 AOP"}="2H6  AOP"; $resn_exch{" H71 AOP"}="1H7  AOP"; $resn_exch{" H72 AOP"}="2H7  AOP";
$resn_exch{" H81 AOP"}="1H8  AOP"; $resn_exch{" H82 AOP"}="2H8  AOP"; $resn_exch{" H91 AOP"}="1H9  AOP";
$resn_exch{" H92 AOP"}="2H9  AOP"; $resn_exch{" H93 AOP"}="3H9  AOP"; $resn_exch{" HB1 AOR"}="1HB  AOR";
$resn_exch{" HB2 AOR"}="2HB  AOR"; $resn_exch{" HG1 AOR"}="1HG  AOR"; $resn_exch{" HG2 AOR"}="2HG  AOR";
$resn_exch{" HD1 AOR"}="1HD  AOR"; $resn_exch{" HD2 AOR"}="2HD  AOR"; $resn_exch{"HNE1 AOR"}="1HNE AOR";
$resn_exch{"HNE2 AOR"}="2HNE AOR"; $resn_exch{" H21 AOR"}="1H2  AOR"; $resn_exch{" H22 AOR"}="2H2  AOR";
$resn_exch{" H23 AOR"}="3H2  AOR"; $resn_exch{" H61 AOS"}="1H6  AOS"; $resn_exch{" H62 AOS"}="2H6  AOS";
$resn_exch{" HN1  AP"}="1HN   AP"; $resn_exch{" HN2  AP"}="2HN   AP"; $resn_exch{" H81 AP0"}="1H8  AP0";
$resn_exch{" H82 AP0"}="2H8  AP0"; $resn_exch{" H83 AP0"}="3H8  AP0"; $resn_exch{" H41 AP0"}="1H4  AP0";
$resn_exch{" H42 AP0"}="2H4  AP0"; $resn_exch{" H1' AP0"}=" H1* AP0"; $resn_exch{" H4' AP0"}=" H4* AP0";
$resn_exch{"H5'1 AP0"}="1H5* AP0"; $resn_exch{"H5'2 AP0"}="2H5* AP0"; $resn_exch{" H11 AP0"}="1H1  AP0";
$resn_exch{"H12X AP0"}="2H1  AP0"; $resn_exch{" H2' AP0"}=" H2* AP0"; $resn_exch{" H3' AP0"}=" H3* AP0";
$resn_exch{"HN61 AP0"}="1HN6 AP0"; $resn_exch{"HN62 AP0"}="2HN6 AP0"; $resn_exch{"H111 AP0"}="H11  AP0";
$resn_exch{" H31 AP1"}="1H3  AP1"; $resn_exch{" H32 AP1"}="2H3  AP1"; $resn_exch{"H41A AP1"}="1H4  AP1";
$resn_exch{" H42 AP1"}="2H4  AP1"; $resn_exch{" H51 AP1"}="1H5  AP1"; $resn_exch{" H52 AP1"}="2H5  AP1";
$resn_exch{" H61 AP1"}="1H6  AP1"; $resn_exch{" H62 AP1"}="2H6  AP1"; $resn_exch{"H161 AP1"}="1H16 AP1";
$resn_exch{"H162 AP1"}="2H16 AP1"; $resn_exch{"H163 AP1"}="3H16 AP1"; $resn_exch{"H181 AP1"}="1H18 AP1";
$resn_exch{"H182 AP1"}="2H18 AP1"; $resn_exch{"H183 AP1"}="3H18 AP1"; $resn_exch{"H191 AP1"}="1H19 AP1";
$resn_exch{"H192 AP1"}="2H19 AP1"; $resn_exch{"H193 AP1"}="3H19 AP1"; $resn_exch{"H201 AP1"}="1H20 AP1";
$resn_exch{"H202 AP1"}="2H20 AP1"; $resn_exch{"H203 AP1"}="3H20 AP1"; $resn_exch{"H211 AP1"}="1H21 AP1";
$resn_exch{"H212 AP1"}="2H21 AP1"; $resn_exch{"H213 AP1"}="3H21 AP1"; $resn_exch{"H271 AP1"}="1H27 AP1";
$resn_exch{"H272 AP1"}="2H27 AP1"; $resn_exch{"H281 AP1"}="1H28 AP1"; $resn_exch{"H282 AP1"}="2H28 AP1";
$resn_exch{"H371 AP1"}="1H37 AP1"; $resn_exch{"H372 AP1"}="2H37 AP1"; $resn_exch{"H421 AP1"}="1H42 AP1";
$resn_exch{"H422 AP1"}="2H42 AP1"; $resn_exch{"H423 AP1"}="3H42 AP1"; $resn_exch{"H461 AP1"}="1H46 AP1";
$resn_exch{"H462 AP1"}="2H46 AP1"; $resn_exch{"HOB2 AP2"}="2HOB AP2"; $resn_exch{"HOB3 AP2"}="3HOB AP2";
$resn_exch{"H3A1 AP2"}="1H3A AP2"; $resn_exch{"H3A2 AP2"}="2H3A AP2"; $resn_exch{"HOA2 AP2"}="2HOA AP2";
$resn_exch{"H5'1 AP2"}="1H5* AP2"; $resn_exch{"H5'2 AP2"}="2H5* AP2"; $resn_exch{" H4' AP2"}=" H4* AP2";
$resn_exch{" H3' AP2"}=" H3* AP2"; $resn_exch{"HO3' AP2"}="*HO3 AP2"; $resn_exch{" H2' AP2"}=" H2* AP2";
$resn_exch{"HO2' AP2"}="*HO2 AP2"; $resn_exch{" H1' AP2"}=" H1* AP2"; $resn_exch{"HN61 AP2"}="1HN6 AP2";
$resn_exch{"HN62 AP2"}="2HN6 AP2"; $resn_exch{"H511 AP3"}="1H51 AP3"; $resn_exch{"H512 AP3"}="2H51 AP3";
$resn_exch{"H513 AP3"}="3H51 AP3"; $resn_exch{"H521 AP3"}="1H52 AP3"; $resn_exch{"H522 AP3"}="2H52 AP3";
$resn_exch{"H523 AP3"}="3H52 AP3"; $resn_exch{" H91 AP3"}="1H9  AP3"; $resn_exch{" H92 AP3"}="2H9  AP3";
$resn_exch{" H93 AP3"}="3H9  AP3"; $resn_exch{"H101 AP3"}="1H10 AP3"; $resn_exch{"H102 AP3"}="2H10 AP3";
$resn_exch{"H103 AP3"}="3H10 AP3"; $resn_exch{"HN21 AP4"}="1HN2 AP4"; $resn_exch{"HN22 AP4"}="2HN2 AP4";
$resn_exch{" H71 AP4"}="1H7  AP4"; $resn_exch{" H72 AP4"}="2H7  AP4"; $resn_exch{"H111 AP4"}="1H11 AP4";
$resn_exch{"H112 AP4"}="2H11 AP4"; $resn_exch{"HOA2 AP5"}="2HOA AP5"; $resn_exch{"HOB2 AP5"}="2HOB AP5";
$resn_exch{"HOG2 AP5"}="2HOG AP5"; $resn_exch{"HOD2 AP5"}="2HOD AP5"; $resn_exch{"HOE2 AP5"}="2HOE AP5";
$resn_exch{"H51A AP5"}="AH51 AP5"; $resn_exch{"H52A AP5"}="AH52 AP5"; $resn_exch{" H4F AP5"}="AH4* AP5";
$resn_exch{" H3F AP5"}="AH3* AP5"; $resn_exch{"HO3A AP5"}="AHO3 AP5"; $resn_exch{" H2F AP5"}="AH2* AP5";
$resn_exch{"HO2A AP5"}="AHO2 AP5"; $resn_exch{" H1F AP5"}="AH1* AP5"; $resn_exch{" H8A AP5"}=" AH8 AP5";
$resn_exch{"H61A AP5"}="AH61 AP5"; $resn_exch{"H62A AP5"}="AH62 AP5"; $resn_exch{" H2A AP5"}=" AH2 AP5";
$resn_exch{"H51B AP5"}="BH51 AP5"; $resn_exch{"H52B AP5"}="BH52 AP5"; $resn_exch{" H4J AP5"}="BH4* AP5";
$resn_exch{" H3J AP5"}="BH3* AP5"; $resn_exch{"HO3B AP5"}="BHO3 AP5"; $resn_exch{" H2J AP5"}="BH2* AP5";
$resn_exch{"HO2B AP5"}="BHO2 AP5"; $resn_exch{" H1J AP5"}="BH1* AP5"; $resn_exch{" H8B AP5"}=" BH8 AP5";
$resn_exch{"H61B AP5"}="BH61 AP5"; $resn_exch{"H62B AP5"}="BH62 AP5"; $resn_exch{" H2B AP5"}=" BH2 AP5";
$resn_exch{"HN21 AP6"}="1HN2 AP6"; $resn_exch{"HN22 AP6"}="2HN2 AP6"; $resn_exch{"HN41 AP6"}="1HN4 AP6";
$resn_exch{"HN42 AP6"}="2HN4 AP6"; $resn_exch{" H71 AP6"}="1H7  AP6"; $resn_exch{" H72 AP6"}="2H7  AP6";
$resn_exch{" H1' AP7"}=" H1* AP7"; $resn_exch{" H61 AP7"}="1H6  AP7"; $resn_exch{" H62 AP7"}="2H6  AP7";
$resn_exch{" H2' AP7"}=" H2* AP7"; $resn_exch{"HO2' AP7"}="2HO* AP7"; $resn_exch{" H3' AP7"}=" H3* AP7";
$resn_exch{"HO3' AP7"}="H3T  AP7"; $resn_exch{" H4' AP7"}=" H4* AP7"; $resn_exch{"H5'' AP7"}="2H5* AP7";
$resn_exch{" H5' AP7"}="1H5* AP7"; $resn_exch{"HOP2 AP7"}="2HOP AP7"; $resn_exch{"HOP3 AP7"}="3HOP AP7";
$resn_exch{"H561 AP8"}="1H56 AP8"; $resn_exch{"H562 AP8"}="2H56 AP8"; $resn_exch{"H573 AP8"}="3H57 AP8";
$resn_exch{"H572 AP8"}="2H57 AP8"; $resn_exch{"H571 AP8"}="1H57 AP8"; $resn_exch{"H593 AP8"}="3H59 AP8";
$resn_exch{"H592 AP8"}="2H59 AP8"; $resn_exch{"H591 AP8"}="1H59 AP8"; $resn_exch{"H583 AP8"}="3H58 AP8";
$resn_exch{"H582 AP8"}="2H58 AP8"; $resn_exch{"H581 AP8"}="1H58 AP8"; $resn_exch{"H353 AP8"}="3H35 AP8";
$resn_exch{"H352 AP8"}="2H35 AP8"; $resn_exch{"H351 AP8"}="1H35 AP8"; $resn_exch{"H363 AP8"}="3H36 AP8";
$resn_exch{"H362 AP8"}="2H36 AP8"; $resn_exch{"H361 AP8"}="1H36 AP8"; $resn_exch{"H111 AP8"}="1H11 AP8";
$resn_exch{"H112 AP8"}="2H11 AP8"; $resn_exch{"H121 AP8"}="1H12 AP8"; $resn_exch{"H122 AP8"}="2H12 AP8";
$resn_exch{"H533 AP8"}="3H53 AP8"; $resn_exch{"H532 AP8"}="2H53 AP8"; $resn_exch{"H531 AP8"}="1H53 AP8";
$resn_exch{"H373 AP8"}="3H37 AP8"; $resn_exch{"H372 AP8"}="2H37 AP8"; $resn_exch{"H371 AP8"}="1H37 AP8";
$resn_exch{"H161 AP8"}="1H16 AP8"; $resn_exch{"H162 AP8"}="2H16 AP8"; $resn_exch{"H383 AP8"}="3H38 AP8";
$resn_exch{"H382 AP8"}="2H38 AP8"; $resn_exch{"H381 AP8"}="1H38 AP8"; $resn_exch{"H181 AP8"}="1H18 AP8";
$resn_exch{"H182 AP8"}="2H18 AP8"; $resn_exch{"H523 AP8"}="3H52 AP8"; $resn_exch{"H522 AP8"}="2H52 AP8";
$resn_exch{"H521 AP8"}="1H52 AP8"; $resn_exch{"H221 AP8"}="1H22 AP8"; $resn_exch{"H222 AP8"}="2H22 AP8";
$resn_exch{"H393 AP8"}="3H39 AP8"; $resn_exch{"H392 AP8"}="2H39 AP8"; $resn_exch{"H391 AP8"}="1H39 AP8";
$resn_exch{"H403 AP8"}="3H40 AP8"; $resn_exch{"H402 AP8"}="2H40 AP8"; $resn_exch{"H401 AP8"}="1H40 AP8";
$resn_exch{"H271 AP8"}="1H27 AP8"; $resn_exch{"H272 AP8"}="2H27 AP8"; $resn_exch{"H281 AP8"}="1H28 AP8";
$resn_exch{"H282 AP8"}="2H28 AP8"; $resn_exch{"H493 AP8"}="3H49 AP8"; $resn_exch{"H492 AP8"}="2H49 AP8";
$resn_exch{"H491 AP8"}="1H49 AP8"; $resn_exch{"H513 AP8"}="3H51 AP8"; $resn_exch{"H512 AP8"}="2H51 AP8";
$resn_exch{"H511 AP8"}="1H51 AP8"; $resn_exch{"H503 AP8"}="3H50 AP8"; $resn_exch{"H502 AP8"}="2H50 AP8";
$resn_exch{"H501 AP8"}="1H50 AP8"; $resn_exch{"H413 AP8"}="3H41 AP8"; $resn_exch{"H412 AP8"}="2H41 AP8";
$resn_exch{"H411 AP8"}="1H41 AP8"; $resn_exch{"H462 AP8"}="2H46 AP8"; $resn_exch{"H461 AP8"}="1H46 AP8";
$resn_exch{"H423 AP8"}="3H42 AP8"; $resn_exch{"H422 AP8"}="2H42 AP8"; $resn_exch{"H421 AP8"}="1H42 AP8";
$resn_exch{"H443 AP8"}="3H44 AP8"; $resn_exch{"H442 AP8"}="2H44 AP8"; $resn_exch{"H441 AP8"}="1H44 AP8";
$resn_exch{"H151 AP9"}="1H15 AP9"; $resn_exch{"H152 AP9"}="2H15 AP9"; $resn_exch{"H141 AP9"}="1H14 AP9";
$resn_exch{"H142 AP9"}="2H14 AP9"; $resn_exch{"H143 AP9"}="3H14 AP9"; $resn_exch{"H161 AP9"}="1H16 AP9";
$resn_exch{"H162 AP9"}="2H16 AP9"; $resn_exch{"H163 AP9"}="3H16 AP9"; $resn_exch{"H101 AP9"}="1H10 AP9";
$resn_exch{"H102 AP9"}="2H10 AP9"; $resn_exch{"H103 AP9"}="3H10 AP9"; $resn_exch{"H111 AP9"}="1H11 AP9";
$resn_exch{"H112 AP9"}="2H11 AP9"; $resn_exch{"H113 AP9"}="3H11 AP9"; $resn_exch{" H11 AP9"}="1H1  AP9";
$resn_exch{"H12A AP9"}="2H1  AP9"; $resn_exch{" H31 APA"}="1H3  APA"; $resn_exch{" H32 APA"}="2H3  APA";
$resn_exch{"HN11 APA"}="1HN1 APA"; $resn_exch{"HN12 APA"}="2HN1 APA"; $resn_exch{"HN11 APB"}="1HN1 APB";
$resn_exch{"HN12 APB"}="2HN1 APB"; $resn_exch{"HOG2 APC"}="2HOG APC"; $resn_exch{"HOG3 APC"}="3HOG APC";
$resn_exch{"HOB2 APC"}="2HOB APC"; $resn_exch{"HOA2 APC"}="2HOA APC"; $resn_exch{"H3A1 APC"}="1H3A APC";
$resn_exch{"H3A2 APC"}="2H3A APC"; $resn_exch{"H5'1 APC"}="1H5* APC"; $resn_exch{"H5'2 APC"}="2H5* APC";
$resn_exch{" H4' APC"}=" H4* APC"; $resn_exch{" H3' APC"}=" H3* APC"; $resn_exch{"HO3' APC"}="*HO3 APC";
$resn_exch{" H2' APC"}=" H2* APC"; $resn_exch{"HO2' APC"}="*HO2 APC"; $resn_exch{" H1' APC"}=" H1* APC";
$resn_exch{"HN61 APC"}="1HN6 APC"; $resn_exch{"HN62 APC"}="2HN6 APC"; $resn_exch{" HN1 APD"}="1HN  APD";
$resn_exch{" HN2 APD"}="2HN  APD"; $resn_exch{"HBC1 APD"}="1HBC APD"; $resn_exch{"HBC2 APD"}="2HBC APD";
$resn_exch{"H5C1 APD"}="1H5C APD"; $resn_exch{"H5C2 APD"}="2H5C APD"; $resn_exch{"H5C3 APD"}="3H5C APD";
$resn_exch{" H   APE"}="1HN  APE"; $resn_exch{" H2  APE"}="2HN  APE"; $resn_exch{" HB2 APE"}="1HB  APE";
$resn_exch{" HB3 APE"}="2HB  APE"; $resn_exch{"HC21 APF"}="1HC2 APF"; $resn_exch{"HC22 APF"}="2HC2 APF";
$resn_exch{"HC23 APF"}="3HC2 APF"; $resn_exch{" H41 APF"}="1H4  APF"; $resn_exch{" H42 APF"}="2H4  APF";
$resn_exch{"H141 APG"}="1H14 APG"; $resn_exch{"H142 APG"}="2H14 APG"; $resn_exch{"H143 APG"}="3H14 APG";
$resn_exch{" H   APH"}="1HN  APH"; $resn_exch{" H2  APH"}="2HN  APH"; $resn_exch{" HB2 APH"}="1HB  APH";
$resn_exch{" HB3 APH"}="2HB  APH"; $resn_exch{"HG21 APH"}="1HG2 APH"; $resn_exch{"HG22 APH"}="2HG2 APH";
$resn_exch{" H31 API"}="1H3  API"; $resn_exch{" H32 API"}="2H3  API"; $resn_exch{" H41 API"}="1H4  API";
$resn_exch{" H42 API"}="2H4  API"; $resn_exch{" H51 API"}="1H5  API"; $resn_exch{" H52 API"}="2H5  API";
$resn_exch{"HN21 API"}="1HN2 API"; $resn_exch{"HN22 API"}="2HN2 API"; $resn_exch{"HN61 API"}="1HN6 API";
$resn_exch{"HN62 API"}="2HN6 API"; $resn_exch{" HC  APK"}=" H   APK"; $resn_exch{" H   APK"}="1HN  APK";
$resn_exch{" H2  APK"}="2HN  APK"; $resn_exch{" HB2 APK"}="1HB  APK"; $resn_exch{" HB3 APK"}="2HB  APK";
$resn_exch{" HG2 APK"}="1HG  APK"; $resn_exch{" HG3 APK"}="2HG  APK"; $resn_exch{" HD2 APK"}="1HD  APK";
$resn_exch{" HD3 APK"}="2HD  APK"; $resn_exch{" HE2 APK"}="1HE  APK"; $resn_exch{" HE3 APK"}="2HE  APK";
$resn_exch{" HZ  APK"}=" HNZ APK"; $resn_exch{"H5'1 APK"}="1H5* APK"; $resn_exch{"H5'2 APK"}="2H5* APK";
$resn_exch{" H4' APK"}=" H4* APK"; $resn_exch{" H3' APK"}=" H3* APK"; $resn_exch{"HO3' APK"}="HO3* APK";
$resn_exch{" H2' APK"}=" H2* APK"; $resn_exch{"HO2' APK"}="HO2* APK"; $resn_exch{" H1' APK"}=" H1* APK";
$resn_exch{" HC2 APK"}=" H2  APK"; $resn_exch{"HN61 APK"}="1HN6 APK"; $resn_exch{"HN62 APK"}="2HN6 APK";
$resn_exch{"HC21 APL"}="1HC2 APL"; $resn_exch{"HC22 APL"}="2HC2 APL"; $resn_exch{"HC23 APL"}="3HC2 APL";
$resn_exch{" H41 APL"}="1H4  APL"; $resn_exch{" H42 APL"}="2H4  APL"; $resn_exch{" H   APM"}="1HN  APM";
$resn_exch{" H2  APM"}="2HN  APM"; $resn_exch{" HB2 APM"}="1HB  APM"; $resn_exch{" HB3 APM"}="2HB  APM";
$resn_exch{"HG21 APM"}="1HG2 APM"; $resn_exch{"HG22 APM"}="2HG2 APM"; $resn_exch{"H8'1 APN"}="1H8' APN";
$resn_exch{"H8'2 APN"}="2H8' APN"; $resn_exch{"H5'1 APN"}="1H5' APN"; $resn_exch{"H5'2 APN"}="2H5' APN";
$resn_exch{"H3'1 APN"}="1H3' APN"; $resn_exch{"H3'2 APN"}="2H3' APN"; $resn_exch{"H2'1 APN"}="1H2' APN";
$resn_exch{"H2'2 APN"}="2H2' APN"; $resn_exch{"H1'1 APN"}="1H1' APN"; $resn_exch{"H1'2 APN"}="2H1' APN";
$resn_exch{"H1'3 APN"}="3H1' APN"; $resn_exch{"HN61 APN"}="1HN6 APN"; $resn_exch{"HN62 APN"}="2HN6 APN";
$resn_exch{"HN21 APO"}="1HN2 APO"; $resn_exch{"HN22 APO"}="2HN2 APO"; $resn_exch{"HC61 APO"}="1HC6 APO";
$resn_exch{"HC62 APO"}="2HC6 APO"; $resn_exch{" HM1 APP"}="1HM  APP"; $resn_exch{" HM2 APP"}="2HM  APP";
$resn_exch{" HM3 APP"}="3HM  APP"; $resn_exch{" H31 APP"}="1H3  APP"; $resn_exch{" H32 APP"}="2H3  APP";
$resn_exch{" H41 APP"}="1H4  APP"; $resn_exch{" H42 APP"}="2H4  APP"; $resn_exch{" H51 APP"}="1H5  APP";
$resn_exch{" H52 APP"}="2H5  APP"; $resn_exch{" H61 APP"}="1H6  APP"; $resn_exch{" H62 APP"}="2H6  APP";
$resn_exch{"H101 APQ"}="1H10 APQ"; $resn_exch{"H102 APQ"}="2H10 APQ"; $resn_exch{"H111 APQ"}="1H11 APQ";
$resn_exch{"H112 APQ"}="2H11 APQ"; $resn_exch{"H121 APQ"}="1H12 APQ"; $resn_exch{"H122 APQ"}="2H12 APQ";
$resn_exch{"H123 APQ"}="3H12 APQ"; $resn_exch{" H91 APQ"}="1H9  APQ"; $resn_exch{" H92 APQ"}="2H9  APQ";
$resn_exch{"HN11 APQ"}="1HN1 APQ"; $resn_exch{" H51 APQ"}="1H5  APQ"; $resn_exch{"HN41 APQ"}="1HN4 APQ";
$resn_exch{"HN42 APQ"}="2HN4 APQ"; $resn_exch{" H11 APQ"}="1H1  APQ"; $resn_exch{"HN21 APQ"}="1HN2 APQ";
$resn_exch{"HN22 APQ"}="2HN2 APQ"; $resn_exch{" H61 APR"}="1H6  APR"; $resn_exch{" H62 APR"}="2H6  APR";
$resn_exch{" H'1 APR"}="1H*  APR"; $resn_exch{" H'2 APR"}="2H*  APR"; $resn_exch{"HO'2 APR"}="2HO* APR";
$resn_exch{" H'3 APR"}="3H*  APR"; $resn_exch{"HO'3 APR"}="3HO* APR"; $resn_exch{" H'4 APR"}="4H*  APR";
$resn_exch{"H5'1 APR"}="1H5* APR"; $resn_exch{"H5'2 APR"}="2H5* APR"; $resn_exch{"HOA2 APR"}="2HOA APR";
$resn_exch{"HOB2 APR"}="2HOB APR"; $resn_exch{"H5R1 APR"}="1H5R APR"; $resn_exch{"H5R2 APR"}="2H5R APR";
$resn_exch{"HOR1 APR"}="1HOR APR"; $resn_exch{"HR'1 APR"}="1HR* APR"; $resn_exch{"HOR2 APR"}="2HOR APR";
$resn_exch{"HR'2 APR"}="2HR* APR"; $resn_exch{"HOR3 APR"}="3HOR APR"; $resn_exch{"HR'3 APR"}="3HR* APR";
$resn_exch{"HR'4 APR"}="4HR* APR"; $resn_exch{" H72 APS"}="2H7  APS"; $resn_exch{" H71 APS"}="1H7  APS";
$resn_exch{" H93 APS"}="3H9  APS"; $resn_exch{" H92 APS"}="2H9  APS"; $resn_exch{" H91 APS"}="1H9  APS";
$resn_exch{"H1C1 APS"}="1H1C APS"; $resn_exch{" H52 APS"}="2H5  APS"; $resn_exch{" H51 APS"}="1H5  APS";
$resn_exch{" H82 APT"}="2H8  APT"; $resn_exch{" H81 APT"}="1H8  APT"; $resn_exch{" H21 APT"}="1H2  APT";
$resn_exch{" H22 APT"}="2H2  APT"; $resn_exch{"HOA2 APU"}="2HOA APU"; $resn_exch{"HO5A APU"}="AHO5 APU";
$resn_exch{"H51A APU"}="AH51 APU"; $resn_exch{"H52A APU"}="AH52 APU"; $resn_exch{" H4B APU"}="AH4* APU";
$resn_exch{" H3B APU"}="AH3* APU"; $resn_exch{" H2B APU"}="AH2* APU"; $resn_exch{"HO2A APU"}="AHO2 APU";
$resn_exch{" H1B APU"}="AH1* APU"; $resn_exch{" H8A APU"}=" AH8 APU"; $resn_exch{"H61A APU"}="AH61 APU";
$resn_exch{"H62A APU"}="AH62 APU"; $resn_exch{" H2A APU"}=" AH2 APU"; $resn_exch{" H3U APU"}=" UH3 APU";
$resn_exch{" H5U APU"}=" UH5 APU"; $resn_exch{" H6U APU"}=" UH6 APU"; $resn_exch{" H1D APU"}="UH1* APU";
$resn_exch{" H2D APU"}="UH2* APU"; $resn_exch{"HO2' APU"}="*HO2 APU"; $resn_exch{" H3D APU"}="UH3* APU";
$resn_exch{" H4D APU"}="UH4* APU"; $resn_exch{"H51U APU"}="UH51 APU"; $resn_exch{"H52U APU"}="UH52 APU";
$resn_exch{"HO2U APU"}="UHO2 APU"; $resn_exch{"HO3U APU"}="UHO3 APU"; $resn_exch{"H3C1 APV"}="1H3C APV";
$resn_exch{"H3C2 APV"}="2H3C APV"; $resn_exch{"H4C1 APV"}="1H4C APV"; $resn_exch{"H4C2 APV"}="2H4C APV";
$resn_exch{"H7C1 APV"}="1H7C APV"; $resn_exch{"H7C2 APV"}="2H7C APV"; $resn_exch{"H141 APV"}="1H14 APV";
$resn_exch{"H142 APV"}="2H14 APV"; $resn_exch{"H331 APV"}="1H33 APV"; $resn_exch{"H332 APV"}="2H33 APV";
$resn_exch{"H3B1 APW"}="1H3B APW"; $resn_exch{"H3B2 APW"}="2H3B APW"; $resn_exch{"H5'1 APW"}="1H5* APW";
$resn_exch{"H5'2 APW"}="2H5* APW"; $resn_exch{" H4' APW"}=" H4* APW"; $resn_exch{" H3' APW"}=" H3* APW";
$resn_exch{" H2' APW"}=" H2* APW"; $resn_exch{" H1' APW"}=" H1* APW"; $resn_exch{"HN61 APW"}="1HN6 APW";
$resn_exch{"HN62 APW"}="2HN6 APW"; $resn_exch{"HOM7 APX"}="7HOM APX"; $resn_exch{"H5M2 APX"}="2H5M APX";
$resn_exch{"H5M1 APX"}="1H5M APX"; $resn_exch{"H3O1 APX"}="1H3O APX"; $resn_exch{"H2O1 APX"}="1H2O APX";
$resn_exch{"HOB7 APX"}="7HOB APX"; $resn_exch{"H5B2 APX"}="2H5B APX"; $resn_exch{"H5B1 APX"}="1H5B APX";
$resn_exch{"H3O2 APX"}="2H3O APX"; $resn_exch{"H6A2 APX"}="2H6A APX"; $resn_exch{"H6A1 APX"}="1H6A APX";
$resn_exch{"HOP2 APX"}="2HOP APX"; $resn_exch{"HOP3 APX"}="3HOP APX"; $resn_exch{" HN1 APY"}="1HN  APY";
$resn_exch{" HN2 APY"}="2HN  APY"; $resn_exch{" HM1 APY"}="1HM  APY"; $resn_exch{" HM2 APY"}="2HM  APY";
$resn_exch{"H131 APZ"}="1H13 APZ"; $resn_exch{"H132 APZ"}="2H13 APZ"; $resn_exch{" H71 AQO"}="1H7  AQO";
$resn_exch{" H72 AQO"}="2H7  AQO"; $resn_exch{"HN61 AQP"}="1HN6 AQP"; $resn_exch{"HN62 AQP"}="2HN6 AQP";
$resn_exch{" H1' AQP"}=" H1* AQP"; $resn_exch{" H4' AQP"}=" H4* AQP"; $resn_exch{"H5'1 AQP"}="1H5* AQP";
$resn_exch{"H5'2 AQP"}="2H5* AQP"; $resn_exch{" H3' AQP"}=" H3* AQP"; $resn_exch{" H2' AQP"}=" H2* AQP";
$resn_exch{"HA11 AQS"}="1HA1 AQS"; $resn_exch{"HA12 AQS"}="2HA1 AQS"; $resn_exch{"HA21 AQS"}="1HA2 AQS";
$resn_exch{"HA22 AQS"}="2HA2 AQS"; $resn_exch{"HA31 AQS"}="1HA3 AQS"; $resn_exch{"HA32 AQS"}="2HA3 AQS";
$resn_exch{"HNA1 AQS"}="1HNA AQS"; $resn_exch{"HNA2 AQS"}="2HNA AQS"; $resn_exch{"HNA3 AQS"}="3HNA AQS";
$resn_exch{"HB11 AQS"}="1HB1 AQS"; $resn_exch{"HB12 AQS"}="2HB1 AQS"; $resn_exch{"HB21 AQS"}="1HB2 AQS";
$resn_exch{"HB22 AQS"}="2HB2 AQS"; $resn_exch{"HB31 AQS"}="1HB3 AQS"; $resn_exch{"HB32 AQS"}="2HB3 AQS";
$resn_exch{"HNB1 AQS"}="1HNB AQS"; $resn_exch{"HNB2 AQS"}="2HNB AQS"; $resn_exch{"HNB3 AQS"}="3HNB AQS";
$resn_exch{" H11 AQZ"}="1H1  AQZ"; $resn_exch{"H12A AQZ"}="2H1  AQZ"; $resn_exch{" H13 AQZ"}="3H1  AQZ";
$resn_exch{"H241 AQZ"}="1H24 AQZ"; $resn_exch{"H242 AQZ"}="2H24 AQZ"; $resn_exch{"H251 AQZ"}="1H25 AQZ";
$resn_exch{"H252 AQZ"}="2H25 AQZ"; $resn_exch{"H271 AQZ"}="1H27 AQZ"; $resn_exch{"H272 AQZ"}="2H27 AQZ";
$resn_exch{"H281 AQZ"}="1H28 AQZ"; $resn_exch{"H282 AQZ"}="2H28 AQZ"; $resn_exch{"H321 AQZ"}="1H32 AQZ";
$resn_exch{"H322 AQZ"}="2H32 AQZ"; $resn_exch{"H323 AQZ"}="3H32 AQZ"; $resn_exch{"H341 AQZ"}="1H34 AQZ";
$resn_exch{"H371 AQZ"}="1H37 AQZ"; $resn_exch{"H372 AQZ"}="2H37 AQZ"; $resn_exch{"H381 AQZ"}="1H38 AQZ";
$resn_exch{"H382 AQZ"}="2H38 AQZ"; $resn_exch{"H401 AQZ"}="1H40 AQZ"; $resn_exch{"H402 AQZ"}="2H40 AQZ";
$resn_exch{"H411 AQZ"}="1H41 AQZ"; $resn_exch{"H412 AQZ"}="2H41 AQZ"; $resn_exch{"H421 AQZ"}="1H42 AQZ";
$resn_exch{"H422 AQZ"}="2H42 AQZ"; $resn_exch{"H431 AQZ"}="1H43 AQZ"; $resn_exch{"H432 AQZ"}="2H43 AQZ";
$resn_exch{"HN11 AR1"}="1HN1 AR1"; $resn_exch{"HN12 AR1"}="2HN1 AR1"; $resn_exch{"HN13 AR1"}="3HN1 AR1";
$resn_exch{"HN21 AR1"}="1HN2 AR1"; $resn_exch{"HN22 AR1"}="2HN2 AR1"; $resn_exch{"HN23 AR1"}="3HN2 AR1";
$resn_exch{"HN31 AR1"}="1HN3 AR1"; $resn_exch{"HN32 AR1"}="2HN3 AR1"; $resn_exch{"HN33 AR1"}="3HN3 AR1";
$resn_exch{"H1T1 AR1"}="1H1T AR1"; $resn_exch{"H1T2 AR1"}="2H1T AR1"; $resn_exch{"H2T1 AR1"}="1H2T AR1";
$resn_exch{"H2T2 AR1"}="2H2T AR1"; $resn_exch{"HNT1 AR1"}="1HNT AR1"; $resn_exch{"HNT2 AR1"}="2HNT AR1";
$resn_exch{"HNT3 AR1"}="3HNT AR1"; $resn_exch{"HMT1 AR1"}="1HMT AR1"; $resn_exch{"HMT2 AR1"}="2HMT AR1";
$resn_exch{"HMT3 AR1"}="3HMT AR1"; $resn_exch{" HB1 AR2"}="1HB  AR2"; $resn_exch{" HB2 AR2"}="2HB  AR2";
$resn_exch{" HG1 AR2"}="1HG  AR2"; $resn_exch{" HG2 AR2"}="2HG  AR2"; $resn_exch{" HD1 AR2"}="1HD  AR2";
$resn_exch{" HD2 AR2"}="2HD  AR2"; $resn_exch{"HH11 AR2"}="1HH1 AR2"; $resn_exch{"HH21 AR2"}="1HH2 AR2";
$resn_exch{"HH22 AR2"}="2HH2 AR2"; $resn_exch{"HO21 AR2"}="1HO2 AR2"; $resn_exch{"H5'1 AR3"}="1H5' AR3";
$resn_exch{"H5'2 AR3"}="2H5' AR3"; $resn_exch{"HN41 AR3"}="1HN4 AR3"; $resn_exch{"HN42 AR3"}="2HN4 AR3";
$resn_exch{" H   AR4"}="1HN  AR4"; $resn_exch{" H2  AR4"}="2HN  AR4"; $resn_exch{" HB2 AR4"}="1HB  AR4";
$resn_exch{" HB3 AR4"}="2HB  AR4"; $resn_exch{" HG2 AR4"}="1HG  AR4"; $resn_exch{" HG3 AR4"}="2HG  AR4";
$resn_exch{" H5' AR4"}=" H5* AR4"; $resn_exch{"H5'1 AR4"}="1H5* AR4"; $resn_exch{"H5'2 AR4"}="2H5* AR4";
$resn_exch{" H4' AR4"}=" H4* AR4"; $resn_exch{" H1' AR4"}=" H1* AR4"; $resn_exch{" H3' AR4"}=" H3* AR4";
$resn_exch{"H3'1 AR4"}="1H3* AR4"; $resn_exch{"HO2' AR4"}="HO2* AR4"; $resn_exch{" H51 ARA"}="1H5  ARA";
$resn_exch{" H52 ARA"}="2H5  ARA"; $resn_exch{" H51 ARB"}="1H5  ARB"; $resn_exch{" H52 ARB"}="2H5  ARB";
$resn_exch{" H11 ARC"}="1H1  ARC"; $resn_exch{" H12 ARC"}="2H1  ARC"; $resn_exch{" H21 ARC"}="1H2  ARC";
$resn_exch{" H22 ARC"}="2H2  ARC"; $resn_exch{" H41 ARC"}="1H4  ARC"; $resn_exch{" H42 ARC"}="2H4  ARC";
$resn_exch{" H43 ARC"}="3H4  ARC"; $resn_exch{" H51 ARC"}="1H5  ARC"; $resn_exch{" H52 ARC"}="2H5  ARC";
$resn_exch{" H61 ARC"}="1H6  ARC"; $resn_exch{" H62 ARC"}="2H6  ARC"; $resn_exch{" H71 ARC"}="1H7  ARC";
$resn_exch{" H72 ARC"}="2H7  ARC"; $resn_exch{" H91 ARC"}="1H9  ARC"; $resn_exch{" H92 ARC"}="2H9  ARC";
$resn_exch{" H93 ARC"}="3H9  ARC"; $resn_exch{"H101 ARC"}="1H10 ARC"; $resn_exch{"H102 ARC"}="2H10 ARC";
$resn_exch{"H111 ARC"}="1H11 ARC"; $resn_exch{"H112 ARC"}="2H11 ARC"; $resn_exch{"H121 ARC"}="1H12 ARC";
$resn_exch{"H122 ARC"}="2H12 ARC"; $resn_exch{"H141 ARC"}="1H14 ARC"; $resn_exch{"H142 ARC"}="2H14 ARC";
$resn_exch{"H143 ARC"}="3H14 ARC"; $resn_exch{"H151 ARC"}="1H15 ARC"; $resn_exch{"H152 ARC"}="2H15 ARC";
$resn_exch{"H161 ARC"}="1H16 ARC"; $resn_exch{"H162 ARC"}="2H16 ARC"; $resn_exch{"H171 ARC"}="1H17 ARC";
$resn_exch{"H172 ARC"}="2H17 ARC"; $resn_exch{"H191 ARC"}="1H19 ARC"; $resn_exch{"H192 ARC"}="2H19 ARC";
$resn_exch{"H193 ARC"}="3H19 ARC"; $resn_exch{"H201 ARC"}="1H20 ARC"; $resn_exch{"H202 ARC"}="2H20 ARC";
$resn_exch{"H203 ARC"}="3H20 ARC"; $resn_exch{" H31 ARD"}="1H3  ARD"; $resn_exch{" H32 ARD"}="2H3  ARD";
$resn_exch{" H41 ARD"}="1H4  ARD"; $resn_exch{" H42 ARD"}="2H4  ARD"; $resn_exch{" H51 ARD"}="1H5  ARD";
$resn_exch{" H52 ARD"}="2H5  ARD"; $resn_exch{" H61 ARD"}="1H6  ARD"; $resn_exch{" H62 ARD"}="2H6  ARD";
$resn_exch{"H111 ARD"}="1H11 ARD"; $resn_exch{"H112 ARD"}="2H11 ARD"; $resn_exch{"H121 ARD"}="1H12 ARD";
$resn_exch{"H122 ARD"}="2H12 ARD"; $resn_exch{"H421 ARD"}="1H42 ARD"; $resn_exch{"H422 ARD"}="2H42 ARD";
$resn_exch{"H423 ARD"}="3H42 ARD"; $resn_exch{"H141 ARD"}="1H14 ARD"; $resn_exch{"H142 ARD"}="2H14 ARD";
$resn_exch{"H431 ARD"}="1H43 ARD"; $resn_exch{"H432 ARD"}="2H43 ARD"; $resn_exch{"H433 ARD"}="3H43 ARD";
$resn_exch{"H441 ARD"}="1H44 ARD"; $resn_exch{"H442 ARD"}="2H44 ARD"; $resn_exch{"H443 ARD"}="3H44 ARD";
$resn_exch{"H231 ARD"}="1H23 ARD"; $resn_exch{"H232 ARD"}="2H23 ARD"; $resn_exch{"H451 ARD"}="1H45 ARD";
$resn_exch{"H452 ARD"}="2H45 ARD"; $resn_exch{"H453 ARD"}="3H45 ARD"; $resn_exch{"H501 ARD"}="1H50 ARD";
$resn_exch{"H502 ARD"}="2H50 ARD"; $resn_exch{"H503 ARD"}="3H50 ARD"; $resn_exch{"H461 ARD"}="1H46 ARD";
$resn_exch{"H462 ARD"}="2H46 ARD"; $resn_exch{"H463 ARD"}="3H46 ARD"; $resn_exch{"H471 ARD"}="1H47 ARD";
$resn_exch{"H472 ARD"}="2H47 ARD"; $resn_exch{"H473 ARD"}="3H47 ARD"; $resn_exch{"H321 ARD"}="1H32 ARD";
$resn_exch{"H322 ARD"}="2H32 ARD"; $resn_exch{"H481 ARD"}="1H48 ARD"; $resn_exch{"H482 ARD"}="2H48 ARD";
$resn_exch{"H483 ARD"}="3H48 ARD"; $resn_exch{"H351 ARD"}="1H35 ARD"; $resn_exch{"H352 ARD"}="2H35 ARD";
$resn_exch{"H371 ARD"}="1H37 ARD"; $resn_exch{"H372 ARD"}="2H37 ARD"; $resn_exch{"H511 ARD"}="1H51 ARD";
$resn_exch{"H512 ARD"}="2H51 ARD"; $resn_exch{"H513 ARD"}="3H51 ARD"; $resn_exch{"H401 ARD"}="1H40 ARD";
$resn_exch{"H402 ARD"}="2H40 ARD"; $resn_exch{"H411 ARD"}="1H41 ARD"; $resn_exch{"H412 ARD"}="2H41 ARD";
$resn_exch{"H551 ARD"}="1H55 ARD"; $resn_exch{"H552 ARD"}="2H55 ARD"; $resn_exch{"H553 ARD"}="3H55 ARD";
$resn_exch{"H6A1 ARE"}="1H6A ARE"; $resn_exch{"H6A2 ARE"}="2H6A ARE"; $resn_exch{"H6B1 ARE"}="1H6B ARE";
$resn_exch{"H6B2 ARE"}="2H6B ARE"; $resn_exch{"H6C1 ARE"}="1H6C ARE"; $resn_exch{"H6C2 ARE"}="2H6C ARE";
$resn_exch{"H6C3 ARE"}="3H6C ARE"; $resn_exch{"H7H1 ARE"}="1H7H ARE"; $resn_exch{"H7H2 ARE"}="2H7H ARE";
$resn_exch{"H6G1 ARE"}="1H6G ARE"; $resn_exch{"H6G2 ARE"}="2H6G ARE"; $resn_exch{" HN1 ARF"}="1HN  ARF";
$resn_exch{" HN2 ARF"}="2HN  ARF"; $resn_exch{" H2  ARG"}=" HN2 ARG"; $resn_exch{" HB2 ARG"}="1HB  ARG";
$resn_exch{" HB3 ARG"}="2HB  ARG"; $resn_exch{" HG2 ARG"}="1HG  ARG"; $resn_exch{" HG3 ARG"}="2HG  ARG";
$resn_exch{" HD2 ARG"}="1HD  ARG"; $resn_exch{" HD3 ARG"}="2HD  ARG"; $resn_exch{"HH11 ARG"}="1HH1 ARG";
$resn_exch{"HH12 ARG"}="2HH1 ARG"; $resn_exch{"HH21 ARG"}="1HH2 ARG"; $resn_exch{"HH22 ARG"}="2HH2 ARG";
$resn_exch{" HC1 ARH"}="1HC  ARH"; $resn_exch{"H131 ARH"}="1H13 ARH"; $resn_exch{"H132 ARH"}="2H13 ARH";
$resn_exch{"HC41 ARH"}="1HC4 ARH"; $resn_exch{"H171 ARH"}="1H17 ARH"; $resn_exch{"H172 ARH"}="2H17 ARH";
$resn_exch{"H173 ARH"}="3H17 ARH"; $resn_exch{"H251 ARH"}="1H25 ARH"; $resn_exch{"H252 ARH"}="2H25 ARH";
$resn_exch{"H253 ARH"}="3H25 ARH"; $resn_exch{"H331 ARH"}="1H33 ARH"; $resn_exch{"H332 ARH"}="2H33 ARH";
$resn_exch{"HO43 ARH"}="3HO4 ARH"; $resn_exch{"HC83 ARH"}="3HC8 ARH"; $resn_exch{"HC93 ARH"}="3HC9 ARH";
$resn_exch{"HC04 ARH"}="4HC0 ARH"; $resn_exch{"HC14 ARH"}="4HC1 ARH"; $resn_exch{"HC24 ARH"}="4HC2 ARH";
$resn_exch{" H21 ARI"}="1H2  ARI"; $resn_exch{" H22 ARI"}="2H2  ARI"; $resn_exch{" H31 ARI"}="1H3  ARI";
$resn_exch{" H32 ARI"}="2H3  ARI"; $resn_exch{" H41 ARI"}="1H4  ARI"; $resn_exch{" H42 ARI"}="2H4  ARI";
$resn_exch{" H43 ARI"}="3H4  ARI"; $resn_exch{" H61 ARI"}="1H6  ARI"; $resn_exch{" H62 ARI"}="2H6  ARI";
$resn_exch{" H63 ARI"}="3H6  ARI"; $resn_exch{"H161 ARL"}="1H16 ARL"; $resn_exch{"H162 ARL"}="2H16 ARL";
$resn_exch{"H163 ARL"}="3H16 ARL"; $resn_exch{" H71 ARL"}="1H7  ARL"; $resn_exch{" H72 ARL"}="2H7  ARL";
$resn_exch{" H73 ARL"}="3H7  ARL"; $resn_exch{"H111 ARL"}="1H11 ARL"; $resn_exch{"H112 ARL"}="2H11 ARL";
$resn_exch{"H113 ARL"}="3H11 ARL"; $resn_exch{"H151 ARL"}="1H15 ARL"; $resn_exch{"H152 ARL"}="2H15 ARL";
$resn_exch{"H153 ARL"}="3H15 ARL"; $resn_exch{"H191 ARL"}="1H19 ARL"; $resn_exch{"H192 ARL"}="2H19 ARL";
$resn_exch{"H193 ARL"}="3H19 ARL"; $resn_exch{"H241 ARL"}="1H24 ARL"; $resn_exch{"H242 ARL"}="2H24 ARL";
$resn_exch{"H243 ARL"}="3H24 ARL"; $resn_exch{"H281 ARL"}="1H28 ARL"; $resn_exch{"H282 ARL"}="2H28 ARL";
$resn_exch{"H283 ARL"}="3H28 ARL"; $resn_exch{"H321 ARL"}="1H32 ARL"; $resn_exch{"H322 ARL"}="2H32 ARL";
$resn_exch{"H323 ARL"}="3H32 ARL"; $resn_exch{" H   ARM"}="1HN  ARM"; $resn_exch{" H2  ARM"}="2HN  ARM";
$resn_exch{" HB2 ARM"}="1HB  ARM"; $resn_exch{" HB3 ARM"}="2HB  ARM"; $resn_exch{" HG2 ARM"}="1HG  ARM";
$resn_exch{" HG3 ARM"}="2HG  ARM"; $resn_exch{" HD2 ARM"}="1HD  ARM"; $resn_exch{" HD3 ARM"}="2HD  ARM";
$resn_exch{" HE  ARM"}=" HNE ARM"; $resn_exch{" HH1 ARM"}=" HN1 ARM"; $resn_exch{"HH21 ARM"}="1HN2 ARM";
$resn_exch{"HH22 ARM"}="2HN2 ARM"; $resn_exch{" HM1 ARM"}="1HM  ARM"; $resn_exch{" HM2 ARM"}="2HM  ARM";
$resn_exch{" HM3 ARM"}="3HM  ARM"; $resn_exch{" HN1 ARN"}="1HN  ARN"; $resn_exch{" H11 ARN"}="1H1  ARN";
$resn_exch{" H21 ARN"}="1H2  ARN"; $resn_exch{" H22 ARN"}="2H2  ARN"; $resn_exch{" H31 ARN"}="1H3  ARN";
$resn_exch{" H32 ARN"}="2H3  ARN"; $resn_exch{"HCA1 ARN"}="1HCA ARN"; $resn_exch{"HCA2 ARN"}="2HCA ARN";
$resn_exch{" HC1 ARN"}="1HC  ARN"; $resn_exch{" H2  ARO"}=" HN2 ARO"; $resn_exch{" HB2 ARO"}="1HB  ARO";
$resn_exch{" HB3 ARO"}="2HB  ARO"; $resn_exch{" HG  ARO"}="1HG  ARO"; $resn_exch{" HH  ARO"}=" HOH ARO";
$resn_exch{" HD2 ARO"}="1HD  ARO"; $resn_exch{" HD3 ARO"}="2HD  ARO"; $resn_exch{"HH11 ARO"}="1HH1 ARO";
$resn_exch{"HH12 ARO"}="2HH1 ARO"; $resn_exch{"HH21 ARO"}="1HH2 ARO"; $resn_exch{"HH22 ARO"}="2HH2 ARO";
$resn_exch{" H72 ARP"}="2H7  ARP"; $resn_exch{" H71 ARP"}="1H7  ARP"; $resn_exch{" H93 ARP"}="3H9  ARP";
$resn_exch{" H92 ARP"}="2H9  ARP"; $resn_exch{" H91 ARP"}="1H9  ARP"; $resn_exch{"H1C1 ARP"}="1H1C ARP";
$resn_exch{" H52 ARP"}="2H5  ARP"; $resn_exch{" H51 ARP"}="1H5  ARP"; $resn_exch{"H21A ARQ"}="1H2  ARQ";
$resn_exch{" H22 ARQ"}="2H2  ARQ"; $resn_exch{" H23 ARQ"}="3H2  ARQ"; $resn_exch{" H31 ARQ"}="1H3  ARQ";
$resn_exch{" H32 ARQ"}="2H3  ARQ"; $resn_exch{" H33 ARQ"}="3H3  ARQ"; $resn_exch{" H41 ARQ"}="1H4  ARQ";
$resn_exch{" H42 ARQ"}="2H4  ARQ"; $resn_exch{" H43 ARQ"}="3H4  ARQ"; $resn_exch{" H61 ARQ"}="1H6  ARQ";
$resn_exch{" H62 ARQ"}="2H6  ARQ"; $resn_exch{"H141 ARQ"}="1H14 ARQ"; $resn_exch{"H142 ARQ"}="2H14 ARQ";
$resn_exch{"H221 ARQ"}="1H22 ARQ"; $resn_exch{"H222 ARQ"}="2H22 ARQ"; $resn_exch{"H223 ARQ"}="3H22 ARQ";
$resn_exch{"H151 ARQ"}="1H15 ARQ"; $resn_exch{"H152 ARQ"}="2H15 ARQ"; $resn_exch{"H121 ARR"}="1H12 ARR";
$resn_exch{"H122 ARR"}="2H12 ARR"; $resn_exch{"H131 ARR"}="1H13 ARR"; $resn_exch{"H132 ARR"}="2H13 ARR";
$resn_exch{"H141 ARR"}="1H14 ARR"; $resn_exch{"H142 ARR"}="2H14 ARR"; $resn_exch{" H2  ARV"}=" HN2 ARV";
$resn_exch{" HB2 ARV"}="1HB  ARV"; $resn_exch{" HB3 ARV"}="2HB  ARV"; $resn_exch{" HG2 ARV"}="1HG  ARV";
$resn_exch{" HG3 ARV"}="2HG  ARV"; $resn_exch{" HD2 ARV"}="1HD  ARV"; $resn_exch{" HD3 ARV"}="2HD  ARV";
$resn_exch{"HH11 ARV"}="2HH1 ARV"; $resn_exch{"HH12 ARV"}="HNH1 ARV"; $resn_exch{" HH2 ARV"}="1HH2 ARV";
$resn_exch{"HC11 ARV"}="1HC1 ARV"; $resn_exch{"HC12 ARV"}="2HC1 ARV"; $resn_exch{"HC31 ARV"}="1HC3 ARV";
$resn_exch{"HC32 ARV"}="2HC3 ARV"; $resn_exch{"H5C1 ARW"}="1H5C ARW"; $resn_exch{"H5C2 ARW"}="2H5C ARW";
$resn_exch{"HMC1 ARW"}="1HMC ARW"; $resn_exch{"HMC2 ARW"}="2HMC ARW"; $resn_exch{"HMC3 ARW"}="3HMC ARW";
$resn_exch{"H501 ARY"}="1H50 ARY"; $resn_exch{"H502 ARY"}="2H50 ARY"; $resn_exch{"H511 ARY"}="1H51 ARY";
$resn_exch{"H512 ARY"}="2H51 ARY"; $resn_exch{"H521 ARY"}="1H52 ARY"; $resn_exch{"H522 ARY"}="2H52 ARY";
$resn_exch{"H531 ARY"}="1H53 ARY"; $resn_exch{"H532 ARY"}="2H53 ARY"; $resn_exch{"H541 ARY"}="1H54 ARY";
$resn_exch{"H542 ARY"}="2H54 ARY"; $resn_exch{"H551 ARY"}="1H55 ARY"; $resn_exch{"H552 ARY"}="2H55 ARY";
$resn_exch{"H561 ARY"}="1H56 ARY"; $resn_exch{"H562 ARY"}="2H56 ARY"; $resn_exch{"H581 ARY"}="1H58 ARY";
$resn_exch{"H582 ARY"}="2H58 ARY"; $resn_exch{"H583 ARY"}="3H58 ARY"; $resn_exch{"H591 ARY"}="1H59 ARY";
$resn_exch{"H592 ARY"}="2H59 ARY"; $resn_exch{"H593 ARY"}="3H59 ARY"; $resn_exch{"H481 ARY"}="1H48 ARY";
$resn_exch{"H482 ARY"}="2H48 ARY"; $resn_exch{"H491 ARY"}="1H49 ARY"; $resn_exch{"H492 ARY"}="2H49 ARY";
$resn_exch{"H493 ARY"}="3H49 ARY"; $resn_exch{" H31 ARY"}="1H3  ARY"; $resn_exch{" H32 ARY"}="2H3  ARY";
$resn_exch{" H91 ARY"}="1H9  ARY"; $resn_exch{" H92 ARY"}="2H9  ARY"; $resn_exch{" H93 ARY"}="3H9  ARY";
$resn_exch{"H131 ARY"}="1H13 ARY"; $resn_exch{"H132 ARY"}="2H13 ARY"; $resn_exch{"H171 ARY"}="1H17 ARY";
$resn_exch{"H172 ARY"}="2H17 ARY"; $resn_exch{"H173 ARY"}="3H17 ARY"; $resn_exch{"H301 ARY"}="1H30 ARY";
$resn_exch{"H302 ARY"}="2H30 ARY"; $resn_exch{"H303 ARY"}="3H30 ARY"; $resn_exch{"H351 ARY"}="1H35 ARY";
$resn_exch{"H352 ARY"}="2H35 ARY"; $resn_exch{"HOP1  AS"}="1HOP  AS"; $resn_exch{"HOP3  AS"}="3HOP  AS";
$resn_exch{" H5'  AS"}="1H5*  AS"; $resn_exch{"H5''  AS"}="2H5*  AS"; $resn_exch{" H4'  AS"}=" H4*  AS";
$resn_exch{" H3'  AS"}=" H3*  AS"; $resn_exch{"HO3'  AS"}="*HO3  AS"; $resn_exch{" H2'  AS"}="1H2*  AS";
$resn_exch{"H2''  AS"}="2H2*  AS"; $resn_exch{" H1'  AS"}=" H1*  AS"; $resn_exch{"HN61  AS"}="1HN6  AS";
$resn_exch{"HN62  AS"}="2HN6  AS"; $resn_exch{" H11 AS0"}="1H1  AS0"; $resn_exch{"H12A AS0"}="2H1  AS0";
$resn_exch{" H13 AS0"}="3H1  AS0"; $resn_exch{"H31A AS0"}="1H3  AS0"; $resn_exch{" H32 AS0"}="2H3  AS0";
$resn_exch{" H51 AS0"}="1H5  AS0"; $resn_exch{" H52 AS0"}="2H5  AS0"; $resn_exch{" H61 AS0"}="1H6  AS0";
$resn_exch{" H62 AS0"}="2H6  AS0"; $resn_exch{" H91 AS0"}="1H9  AS0"; $resn_exch{" H92 AS0"}="2H9  AS0";
$resn_exch{"H101 AS0"}="1H10 AS0"; $resn_exch{"H102 AS0"}="2H10 AS0"; $resn_exch{"H151 AS0"}="1H15 AS0";
$resn_exch{"H152 AS0"}="2H15 AS0"; $resn_exch{"H161 AS0"}="1H16 AS0"; $resn_exch{"H162 AS0"}="2H16 AS0";
$resn_exch{"H201 AS0"}="1H20 AS0"; $resn_exch{"H202 AS0"}="2H20 AS0"; $resn_exch{"H221 AS0"}="1H22 AS0";
$resn_exch{"H222 AS0"}="2H22 AS0"; $resn_exch{"H223 AS0"}="3H22 AS0"; $resn_exch{"H331 AS0"}="1H33 AS0";
$resn_exch{"H332 AS0"}="2H33 AS0"; $resn_exch{"H333 AS0"}="3H33 AS0"; $resn_exch{"HN41 AS1"}="1HN4 AS1";
$resn_exch{"HN42 AS1"}="2HN4 AS1"; $resn_exch{" H31 AS1"}="1H3  AS1"; $resn_exch{" H32 AS1"}="2H3  AS1";
$resn_exch{" H21 AS1"}="1H2  AS1"; $resn_exch{" H22 AS1"}="2H2  AS1"; $resn_exch{" H11 AS1"}="1H1  AS1";
$resn_exch{" H12 AS1"}="2H1  AS1"; $resn_exch{" HB1 AS1"}="1HB  AS1"; $resn_exch{" HB2 AS1"}="2HB  AS1";
$resn_exch{" HB1 AS2"}="1HB  AS2"; $resn_exch{" HB2 AS2"}="2HB  AS2"; $resn_exch{" HN1 AS2"}="1HN  AS2";
$resn_exch{" HN2 AS2"}="2HN  AS2"; $resn_exch{" HN3 AS2"}="3HN  AS2"; $resn_exch{" H51 AS4"}="1H5  AS4";
$resn_exch{" H52 AS4"}="2H5  AS4"; $resn_exch{"H151 AS4"}="1H15 AS4"; $resn_exch{"H152 AS4"}="2H15 AS4";
$resn_exch{"H153 AS4"}="3H15 AS4"; $resn_exch{"H161 AS4"}="1H16 AS4"; $resn_exch{"H162 AS4"}="2H16 AS4";
$resn_exch{"H191 AS4"}="1H19 AS4"; $resn_exch{"H192 AS4"}="2H19 AS4"; $resn_exch{"H201 AS4"}="1H20 AS4";
$resn_exch{"H202 AS4"}="2H20 AS4"; $resn_exch{"H111 AS4"}="1H11 AS4"; $resn_exch{"H112 AS4"}="2H11 AS4";
$resn_exch{"H121 AS4"}="1H12 AS4"; $resn_exch{"H122 AS4"}="2H12 AS4"; $resn_exch{"H131 AS4"}="1H13 AS4";
$resn_exch{"H132 AS4"}="2H13 AS4"; $resn_exch{"H141 AS4"}="1H14 AS4"; $resn_exch{"H142 AS4"}="2H14 AS4";
$resn_exch{"HAA1 AS9"}="1HAA AS9"; $resn_exch{"HAA2 AS9"}="2HAA AS9"; $resn_exch{"HAA3 AS9"}="3HAA AS9";
$resn_exch{" H   AS9"}=" HN  AS9"; $resn_exch{" HXT AS9"}="HOXT AS9"; $resn_exch{" HB2 AS9"}="1HB  AS9";
$resn_exch{" HB3 AS9"}="2HB  AS9"; $resn_exch{" HD1 AS9"}="HOD1 AS9"; $resn_exch{" H2  ASA"}=" HN2 ASA";
$resn_exch{" HB2 ASA"}="1HB  ASA"; $resn_exch{" HB3 ASA"}="2HB  ASA"; $resn_exch{" H2  ASB"}=" HN2 ASB";
$resn_exch{" HB2 ASB"}="1HB  ASB"; $resn_exch{" HB3 ASB"}="2HB  ASB"; $resn_exch{"HC21 ASB"}="1HC2 ASB";
$resn_exch{"HC22 ASB"}="2HC2 ASB"; $resn_exch{" H61 ASC"}="1H6  ASC"; $resn_exch{" H62 ASC"}="2H6  ASC";
$resn_exch{" H11 ASD"}="1H1  ASD"; $resn_exch{" H12 ASD"}="2H1  ASD"; $resn_exch{"H111 ASD"}="1H11 ASD";
$resn_exch{"H112 ASD"}="2H11 ASD"; $resn_exch{"H121 ASD"}="1H12 ASD"; $resn_exch{"H122 ASD"}="2H12 ASD";
$resn_exch{"H151 ASD"}="1H15 ASD"; $resn_exch{"H152 ASD"}="2H15 ASD"; $resn_exch{"H161 ASD"}="1H16 ASD";
$resn_exch{"H162 ASD"}="2H16 ASD"; $resn_exch{"H181 ASD"}="1H18 ASD"; $resn_exch{"H182 ASD"}="2H18 ASD";
$resn_exch{"H183 ASD"}="3H18 ASD"; $resn_exch{"H191 ASD"}="1H19 ASD"; $resn_exch{"H192 ASD"}="2H19 ASD";
$resn_exch{"H193 ASD"}="3H19 ASD"; $resn_exch{" H21 ASD"}="1H2  ASD"; $resn_exch{" H22 ASD"}="2H2  ASD";
$resn_exch{" H61 ASD"}="1H6  ASD"; $resn_exch{" H62 ASD"}="2H6  ASD"; $resn_exch{" H71 ASD"}="1H7  ASD";
$resn_exch{" H72 ASD"}="2H7  ASD"; $resn_exch{"H111 ASE"}="1H11 ASE"; $resn_exch{"H112 ASE"}="2H11 ASE";
$resn_exch{"H121 ASE"}="1H12 ASE"; $resn_exch{"H122 ASE"}="2H12 ASE"; $resn_exch{"H151 ASE"}="1H15 ASE";
$resn_exch{"H152 ASE"}="2H15 ASE"; $resn_exch{"H153 ASE"}="3H15 ASE"; $resn_exch{"H2C1 ASF"}="1H2C ASF";
$resn_exch{"H2C2 ASF"}="2H2C ASF"; $resn_exch{"H2C3 ASF"}="3H2C ASF"; $resn_exch{"H3C1 ASF"}="1H3C ASF";
$resn_exch{"H3C2 ASF"}="2H3C ASF"; $resn_exch{"H4C1 ASF"}="1H4C ASF"; $resn_exch{"H4C2 ASF"}="2H4C ASF";
$resn_exch{"H5C1 ASF"}="1H5C ASF"; $resn_exch{"H5C2 ASF"}="2H5C ASF"; $resn_exch{"H7C1 ASF"}="1H7C ASF";
$resn_exch{"H7C2 ASF"}="2H7C ASF"; $resn_exch{"H8C1 ASF"}="1H8C ASF"; $resn_exch{"H8C2 ASF"}="2H8C ASF";
$resn_exch{"HO31 ASF"}="1HO3 ASF"; $resn_exch{" H2  ASI"}=" HN2 ASI"; $resn_exch{" H21 ASI"}="1H2  ASI";
$resn_exch{" H22 ASI"}="2H2  ASI"; $resn_exch{" H   ASK"}="1HN  ASK"; $resn_exch{" H2  ASK"}="2HN  ASK";
$resn_exch{" HB2 ASK"}="1HB  ASK"; $resn_exch{" HB3 ASK"}="2HB  ASK"; $resn_exch{" HD2 ASK"}=" HOD ASK";
$resn_exch{" HM1 ASK"}="1HM  ASK"; $resn_exch{" HM2 ASK"}="2HM  ASK"; $resn_exch{" HM3 ASK"}="3HM  ASK";
$resn_exch{" H2  ASL"}=" HN2 ASL"; $resn_exch{" HB1 ASL"}="1HB  ASL"; $resn_exch{" HB2 ASL"}="2HB  ASL";
$resn_exch{"HC31 ASL"}="1HC3 ASL"; $resn_exch{"HC32 ASL"}="2HC3 ASL"; $resn_exch{"HC33 ASL"}="3HC3 ASL";
$resn_exch{" H   ASM"}="1HN  ASM"; $resn_exch{" H2  ASM"}="2HN  ASM"; $resn_exch{" HB2 ASM"}="1HB  ASM";
$resn_exch{" HB3 ASM"}="2HB  ASM"; $resn_exch{" H2  ASN"}=" HN2 ASN"; $resn_exch{" HB2 ASN"}="1HB  ASN";
$resn_exch{" HB3 ASN"}="2HB  ASN"; $resn_exch{"HD21 ASN"}="1HD2 ASN"; $resn_exch{"HD22 ASN"}="2HD2 ASN";
$resn_exch{" H11 ASO"}="1H1  ASO"; $resn_exch{" H12 ASO"}="2H1  ASO"; $resn_exch{" H61 ASO"}="1H6  ASO";
$resn_exch{" H62 ASO"}="2H6  ASO"; $resn_exch{" H2  ASP"}=" HN2 ASP"; $resn_exch{" HB2 ASP"}="1HB  ASP";
$resn_exch{" HB3 ASP"}="2HB  ASP"; $resn_exch{" H2  ASQ"}=" HN2 ASQ"; $resn_exch{" HB2 ASQ"}="1HB  ASQ";
$resn_exch{" HB3 ASQ"}="2HB  ASQ"; $resn_exch{"HOP2 ASQ"}="2HOP ASQ"; $resn_exch{"HOP3 ASQ"}="3HOP ASQ";
$resn_exch{" H71 ASR"}="1H7  ASR"; $resn_exch{" H72 ASR"}="2H7  ASR"; $resn_exch{"HOP2 ASU"}="2HOP ASU";
$resn_exch{"HOP3 ASU"}="3HOP ASU"; $resn_exch{"HO'1 ASU"}="1HO* ASU"; $resn_exch{" H2' ASU"}="1H2* ASU";
$resn_exch{"H2'' ASU"}="2H2* ASU"; $resn_exch{" H5' ASU"}="1H5* ASU"; $resn_exch{"H5'' ASU"}="2H5* ASU";
$resn_exch{" H4' ASU"}=" H4* ASU"; $resn_exch{" H1' ASU"}=" H1* ASU"; $resn_exch{" H3' ASU"}=" H3* ASU";
$resn_exch{"HO'3 ASU"}="3HO* ASU"; $resn_exch{"H3C1 ASV"}="1H3C ASV"; $resn_exch{"H3C2 ASV"}="2H3C ASV";
$resn_exch{"H4C1 ASV"}="1H4C ASV"; $resn_exch{"H4C2 ASV"}="2H4C ASV"; $resn_exch{"H7C1 ASV"}="1H7C ASV";
$resn_exch{"H7C2 ASV"}="2H7C ASV"; $resn_exch{"H141 ASV"}="1H14 ASV"; $resn_exch{"H142 ASV"}="2H14 ASV";
$resn_exch{"H161 ASV"}="1H16 ASV"; $resn_exch{"H162 ASV"}="2H16 ASV"; $resn_exch{"H321 ASV"}="1H32 ASV";
$resn_exch{"H331 ASV"}="1H33 ASV"; $resn_exch{"H332 ASV"}="2H33 ASV"; $resn_exch{" HB1 ASX"}="1HB  ASX";
$resn_exch{" HB2 ASX"}="2HB  ASX"; $resn_exch{"H2C1 AT3"}="1H2C AT3"; $resn_exch{"H2C2 AT3"}="2H2C AT3";
$resn_exch{"H3C1 AT3"}="1H3C AT3"; $resn_exch{"H3C2 AT3"}="2H3C AT3"; $resn_exch{"H6C1 AT3"}="1H6C AT3";
$resn_exch{"H6C2 AT3"}="2H6C AT3"; $resn_exch{"H6C3 AT3"}="3H6C AT3"; $resn_exch{"H8C1 AT3"}="1H8C AT3";
$resn_exch{"H8C2 AT3"}="2H8C AT3"; $resn_exch{"H8C3 AT3"}="3H8C AT3"; $resn_exch{"H9C1 AT3"}="1H9C AT3";
$resn_exch{"H9C2 AT3"}="2H9C AT3"; $resn_exch{"H9C3 AT3"}="3H9C AT3"; $resn_exch{"H103 AT3"}="3H10 AT3";
$resn_exch{"H101 AT3"}="1H10 AT3"; $resn_exch{"H102 AT3"}="2H10 AT3"; $resn_exch{"H5'1 AT4"}="1H5* AT4";
$resn_exch{"H5'2 AT4"}="2H5* AT4"; $resn_exch{" H4' AT4"}=" H4* AT4"; $resn_exch{" H3' AT4"}=" H3* AT4";
$resn_exch{"HO3' AT4"}="HO3* AT4"; $resn_exch{" H2' AT4"}=" H2* AT4"; $resn_exch{" H1' AT4"}=" H1* AT4";
$resn_exch{"HN61 AT4"}="1HN6 AT4"; $resn_exch{"HN62 AT4"}="2HN6 AT4"; $resn_exch{"HO2' AT4"}="HO2* AT4";
$resn_exch{"H111 AT5"}="1H11 AT5"; $resn_exch{"H112 AT5"}="2H11 AT5"; $resn_exch{"H171 AT5"}="1H17 AT5";
$resn_exch{"H172 AT5"}="2H17 AT5"; $resn_exch{"H173 AT5"}="3H17 AT5"; $resn_exch{"H141 AT5"}="1H14 AT5";
$resn_exch{"H142 AT5"}="2H14 AT5"; $resn_exch{"H101 AT5"}="1H10 AT5"; $resn_exch{"H102 AT5"}="2H10 AT5";
$resn_exch{"H103 AT5"}="3H10 AT5"; $resn_exch{"HM21 AT5"}="1HM2 AT5"; $resn_exch{"HM22 AT5"}="2HM2 AT5";
$resn_exch{"HM23 AT5"}="3HM2 AT5"; $resn_exch{"HM31 AT5"}="1HM3 AT5"; $resn_exch{"HM32 AT5"}="2HM3 AT5";
$resn_exch{"HM33 AT5"}="3HM3 AT5"; $resn_exch{"HN'1 ATA"}="1HN' ATA"; $resn_exch{"HN'2 ATA"}="2HN' ATA";
$resn_exch{"HB'1 ATA"}="1HB' ATA"; $resn_exch{"HB'2 ATA"}="2HB' ATA"; $resn_exch{"HG'1 ATA"}="1HG' ATA";
$resn_exch{"HG'2 ATA"}="2HG' ATA"; $resn_exch{"HO'2 ATA"}="2HO' ATA"; $resn_exch{" HB1 ATA"}="1HB  ATA";
$resn_exch{" HB2 ATA"}="2HB  ATA"; $resn_exch{"H121 ATA"}="1H12 ATA"; $resn_exch{"H122 ATA"}="2H12 ATA";
$resn_exch{"H131 ATA"}="1H13 ATA"; $resn_exch{"H132 ATA"}="2H13 ATA"; $resn_exch{"H133 ATA"}="3H13 ATA";
$resn_exch{" H91 ATA"}="1H9  ATA"; $resn_exch{" H92 ATA"}="2H9  ATA"; $resn_exch{" H93 ATA"}="3H9  ATA";
$resn_exch{"H101 ATA"}="1H10 ATA"; $resn_exch{"H102 ATA"}="2H10 ATA"; $resn_exch{"H103 ATA"}="3H10 ATA";
$resn_exch{" HN' ATA"}=" HN\" ATA"; $resn_exch{"HA'1 ATA"}="1HA\" ATA"; $resn_exch{"HA'2 ATA"}="2HA\" ATA";
$resn_exch{" HO' ATA"}=" HO\" ATA"; $resn_exch{"HN21 ATC"}="1HN2 ATC"; $resn_exch{"HN22 ATC"}="2HN2 ATC";
$resn_exch{"H421 ATC"}="1H42 ATC"; $resn_exch{"H422 ATC"}="2H42 ATC"; $resn_exch{"H423 ATC"}="3H42 ATC";
$resn_exch{"H431 ATC"}="1H43 ATC"; $resn_exch{"H432 ATC"}="2H43 ATC"; $resn_exch{"H433 ATC"}="3H43 ATC";
$resn_exch{"H51A ATC"}="1H5  ATC"; $resn_exch{" H52 ATC"}="2H5  ATC"; $resn_exch{" H61 ATC"}="1H6  ATC";
$resn_exch{" H62 ATC"}="2H6  ATC"; $resn_exch{"HOC1 ATC"}="1HOC ATC"; $resn_exch{"H921 ATC"}="1H92 ATC";
$resn_exch{"H922 ATC"}="2H92 ATC"; $resn_exch{"H931 ATC"}="1H93 ATC"; $resn_exch{"H932 ATC"}="2H93 ATC";
$resn_exch{"H933 ATC"}="3H93 ATC"; $resn_exch{"H941 ATC"}="1H94 ATC"; $resn_exch{"H942 ATC"}="2H94 ATC";
$resn_exch{"H943 ATC"}="3H94 ATC"; $resn_exch{"HOP2 ATD"}="2HOP ATD"; $resn_exch{"HOP3 ATD"}="3HOP ATD";
$resn_exch{"HO'5 ATD"}="5HO* ATD"; $resn_exch{" H5' ATD"}="1H5* ATD"; $resn_exch{"H5'' ATD"}="2H5* ATD";
$resn_exch{" H4' ATD"}=" H4* ATD"; $resn_exch{" H3' ATD"}=" H3* ATD"; $resn_exch{" H2' ATD"}="1H2* ATD";
$resn_exch{"H2'' ATD"}="2H2* ATD"; $resn_exch{" H1' ATD"}=" H1* ATD"; $resn_exch{" H71 ATD"}="1H5M ATD";
$resn_exch{" H72 ATD"}="2H5M ATD"; $resn_exch{" H73 ATD"}="3H5M ATD"; $resn_exch{" H21 ATE"}="1H2  ATE";
$resn_exch{" H22 ATE"}="2H2  ATE"; $resn_exch{" H11 ATE"}="1H1  ATE"; $resn_exch{" H12 ATE"}="2H1  ATE";
$resn_exch{" H41 ATE"}="1H4  ATE"; $resn_exch{" H42 ATE"}="2H4  ATE"; $resn_exch{" H61 ATE"}="1H6  ATE";
$resn_exch{" H62 ATE"}="2H6  ATE"; $resn_exch{"H191 ATE"}="1H19 ATE"; $resn_exch{"H192 ATE"}="2H19 ATE";
$resn_exch{"H193 ATE"}="3H19 ATE"; $resn_exch{"H111 ATE"}="1H11 ATE"; $resn_exch{"H112 ATE"}="2H11 ATE";
$resn_exch{" H71 ATE"}="1H7  ATE"; $resn_exch{" H72 ATE"}="2H7  ATE"; $resn_exch{"H151 ATE"}="1H15 ATE";
$resn_exch{"H152 ATE"}="2H15 ATE"; $resn_exch{"H121 ATE"}="1H12 ATE"; $resn_exch{"H122 ATE"}="2H12 ATE";
$resn_exch{"H181 ATE"}="1H18 ATE"; $resn_exch{"H182 ATE"}="2H18 ATE"; $resn_exch{"H183 ATE"}="3H18 ATE";
$resn_exch{"HOG2 ATF"}="2HOG ATF"; $resn_exch{"HOG3 ATF"}="3HOG ATF"; $resn_exch{"HOB2 ATF"}="2HOB ATF";
$resn_exch{"HOA2 ATF"}="2HOA ATF"; $resn_exch{"H5'1 ATF"}="1H5* ATF"; $resn_exch{"H5'2 ATF"}="2H5* ATF";
$resn_exch{" H4' ATF"}=" H4* ATF"; $resn_exch{" H3' ATF"}=" H3* ATF"; $resn_exch{"HO3' ATF"}="*HO3 ATF";
$resn_exch{" H2' ATF"}=" H2* ATF"; $resn_exch{"HO2' ATF"}="*HO2 ATF"; $resn_exch{" H1' ATF"}=" H1* ATF";
$resn_exch{"HN61 ATF"}="1HN6 ATF"; $resn_exch{"HN62 ATF"}="2HN6 ATF"; $resn_exch{"HOG2 ATG"}="2HOG ATG";
$resn_exch{"HSG3 ATG"}="3HSG ATG"; $resn_exch{"HOB2 ATG"}="2HOB ATG"; $resn_exch{"HOA2 ATG"}="2HOA ATG";
$resn_exch{"H5'1 ATG"}="1H5* ATG"; $resn_exch{"H5'2 ATG"}="2H5* ATG"; $resn_exch{" H4' ATG"}=" H4* ATG";
$resn_exch{" H3' ATG"}=" H3* ATG"; $resn_exch{"HO3' ATG"}="*HO3 ATG"; $resn_exch{" H2' ATG"}=" H2* ATG";
$resn_exch{"HO2' ATG"}="*HO2 ATG"; $resn_exch{" H1' ATG"}=" H1* ATG"; $resn_exch{"HN61 ATG"}="1HN6 ATG";
$resn_exch{"HN62 ATG"}="2HN6 ATG"; $resn_exch{"HN11 ATI"}="1HN1 ATI"; $resn_exch{"HN12 ATI"}="2HN1 ATI";
$resn_exch{" H51 ATI"}="1H5  ATI"; $resn_exch{" H52 ATI"}="2H5  ATI"; $resn_exch{" H71 ATI"}="1H7  ATI";
$resn_exch{" H72 ATI"}="2H7  ATI"; $resn_exch{" H73 ATI"}="3H7  ATI"; $resn_exch{" H81 ATI"}="1H8  ATI";
$resn_exch{" H82 ATI"}="2H8  ATI"; $resn_exch{" H83 ATI"}="3H8  ATI"; $resn_exch{"H161 ATI"}="1H16 ATI";
$resn_exch{"H162 ATI"}="2H16 ATI"; $resn_exch{"H163 ATI"}="3H16 ATI"; $resn_exch{"H171 ATI"}="1H17 ATI";
$resn_exch{"H172 ATI"}="2H17 ATI"; $resn_exch{"H173 ATI"}="3H17 ATI"; $resn_exch{"H231 ATI"}="1H23 ATI";
$resn_exch{"H232 ATI"}="2H23 ATI"; $resn_exch{"H233 ATI"}="3H23 ATI"; $resn_exch{"H241 ATI"}="1H24 ATI";
$resn_exch{"H242 ATI"}="2H24 ATI"; $resn_exch{"H243 ATI"}="3H24 ATI"; $resn_exch{"H291 ATI"}="1H29 ATI";
$resn_exch{"H292 ATI"}="2H29 ATI"; $resn_exch{"H1A1 ATJ"}="1H1A ATJ"; $resn_exch{"H2B1 ATJ"}="1H2B ATJ";
$resn_exch{"H2B2 ATJ"}="2H2B ATJ"; $resn_exch{"H1B1 ATJ"}="1H1B ATJ"; $resn_exch{"H1B2 ATJ"}="2H1B ATJ";
$resn_exch{"H1B3 ATJ"}="3H1B ATJ"; $resn_exch{" H5' ATL"}="1H5* ATL"; $resn_exch{"H5'' ATL"}="2H5* ATL";
$resn_exch{" H3' ATL"}=" H3* ATL"; $resn_exch{" H2' ATL"}="1H2* ATL"; $resn_exch{" H1' ATL"}=" H1* ATL";
$resn_exch{"H9'1 ATL"}="1H9* ATL"; $resn_exch{"H9'2 ATL"}="2H9* ATL"; $resn_exch{" H71 ATL"}="1H5M ATL";
$resn_exch{" H72 ATL"}="2H5M ATL"; $resn_exch{" H73 ATL"}="3H5M ATL"; $resn_exch{"HOP2 ATM"}="2HOP ATM";
$resn_exch{"HOP3 ATM"}="3HOP ATM"; $resn_exch{" H5' ATM"}="1H5* ATM"; $resn_exch{"H5'' ATM"}="2H5* ATM";
$resn_exch{" H4' ATM"}=" H4* ATM"; $resn_exch{" H3' ATM"}=" H3* ATM"; $resn_exch{" H2' ATM"}="1H2* ATM";
$resn_exch{"H2'' ATM"}="2H2* ATM"; $resn_exch{" H1' ATM"}=" H1* ATM"; $resn_exch{" H51 ATM"}="1H5  ATM";
$resn_exch{" H52 ATM"}="2H5  ATM"; $resn_exch{" H53 ATM"}="3H5  ATM"; $resn_exch{" HB1 ATN"}="1HB  ATN";
$resn_exch{" HB2 ATN"}="2HB  ATN"; $resn_exch{" HN1 ATN"}="1HN  ATN"; $resn_exch{" HN2 ATN"}="2HN  ATN";
$resn_exch{"HB11 ATN"}="1HB1 ATN"; $resn_exch{"HB12 ATN"}="2HB1 ATN"; $resn_exch{"HG11 ATN"}="1HG1 ATN";
$resn_exch{"HG12 ATN"}="2HG1 ATN"; $resn_exch{"HD1A ATN"}="1HD  ATN"; $resn_exch{"HD2A ATN"}="2HD  ATN";
$resn_exch{"HB21 ATN"}="1HB2 ATN"; $resn_exch{"HB22 ATN"}="2HB2 ATN"; $resn_exch{"HG21 ATN"}="1HG2 ATN";
$resn_exch{"HG22 ATN"}="2HG2 ATN"; $resn_exch{"HD31 ATN"}="1HD3 ATN"; $resn_exch{"HD32 ATN"}="2HD3 ATN";
$resn_exch{"HB31 ATN"}="1HB3 ATN"; $resn_exch{"HB32 ATN"}="2HB3 ATN"; $resn_exch{"HB33 ATN"}="3HB3 ATN";
$resn_exch{"HN41 ATN"}="1HN4 ATN"; $resn_exch{"HN42 ATN"}="2HN4 ATN"; $resn_exch{" H13 ATO"}="3H1  ATO";
$resn_exch{" H12 ATO"}="2H1  ATO"; $resn_exch{" H11 ATO"}="1H1  ATO"; $resn_exch{" H32 ATO"}="2H3  ATO";
$resn_exch{" H31 ATO"}="1H3  ATO"; $resn_exch{"HOG2 ATP"}="2HOG ATP"; $resn_exch{"HOG3 ATP"}="3HOG ATP";
$resn_exch{"HOB2 ATP"}="2HOB ATP"; $resn_exch{"HOA2 ATP"}="2HOA ATP"; $resn_exch{"H5'1 ATP"}="1H5* ATP";
$resn_exch{"H5'2 ATP"}="2H5* ATP"; $resn_exch{" H4' ATP"}=" H4* ATP"; $resn_exch{" H3' ATP"}=" H3* ATP";
$resn_exch{"HO3' ATP"}="*HO3 ATP"; $resn_exch{" H2' ATP"}=" H2* ATP"; $resn_exch{"HO2' ATP"}="*HO2 ATP";
$resn_exch{" H1' ATP"}=" H1* ATP"; $resn_exch{"HN61 ATP"}="1HN6 ATP"; $resn_exch{"HN62 ATP"}="2HN6 ATP";
$resn_exch{" H41 ATQ"}="1H4  ATQ"; $resn_exch{" H42 ATQ"}="2H4  ATQ"; $resn_exch{" H51 ATQ"}="1H5  ATQ";
$resn_exch{" H52 ATQ"}="2H5  ATQ"; $resn_exch{"HN21 ATQ"}="1HN2 ATQ"; $resn_exch{"HN22 ATQ"}="2HN2 ATQ";
$resn_exch{"HOB2 ATR"}="2HOB ATR"; $resn_exch{"HOB3 ATR"}="3HOB ATR"; $resn_exch{"HOA2 ATR"}="2HOA ATR";
$resn_exch{"H5'1 ATR"}="1H5* ATR"; $resn_exch{"H5'2 ATR"}="2H5* ATR"; $resn_exch{" H4' ATR"}=" H4* ATR";
$resn_exch{" H3' ATR"}=" H3* ATR"; $resn_exch{"HO3' ATR"}="*HO3 ATR"; $resn_exch{" H2' ATR"}=" H2* ATR";
$resn_exch{"HOP2 ATR"}="2HOP ATR"; $resn_exch{"HOP3 ATR"}="3HOP ATR"; $resn_exch{" H1' ATR"}=" H1* ATR";
$resn_exch{"HN61 ATR"}="1HN6 ATR"; $resn_exch{"HN62 ATR"}="2HN6 ATR"; $resn_exch{"H5'1 ATS"}="1H5' ATS";
$resn_exch{"H5'2 ATS"}="2H5' ATS"; $resn_exch{"HO2' ATS"}="'HO2 ATS"; $resn_exch{"HO3' ATS"}="'HO3 ATS";
$resn_exch{"HN61 ATS"}="1HN6 ATS"; $resn_exch{"HN62 ATS"}="2HN6 ATS"; $resn_exch{"H3B1 ATS"}="1H3B ATS";
$resn_exch{"H3B2 ATS"}="2H3B ATS"; $resn_exch{" H31 ATT"}="1H3  ATT"; $resn_exch{" H32 ATT"}="2H3  ATT";
$resn_exch{" H33 ATT"}="3H3  ATT"; $resn_exch{" H41 ATT"}="1H4  ATT"; $resn_exch{" H42 ATT"}="2H4  ATT";
$resn_exch{"HC31 ATU"}="1HC3 ATU"; $resn_exch{"HC32 ATU"}="2HC3 ATU"; $resn_exch{" HN1 ATX"}="1HN  ATX";
$resn_exch{" HN2 ATX"}="2HN  ATX"; $resn_exch{" HB1 ATX"}="1HB  ATX"; $resn_exch{" HB2 ATX"}="2HB  ATX";
$resn_exch{" HB3 ATX"}="3HB  ATX"; $resn_exch{" H71 ATX"}="1H7  ATX"; $resn_exch{" H72 ATX"}="2H7  ATX";
$resn_exch{" H73 ATX"}="3H7  ATX"; $resn_exch{" H81 ATX"}="1H8  ATX"; $resn_exch{" H82 ATX"}="2H8  ATX";
$resn_exch{" H83 ATX"}="3H8  ATX"; $resn_exch{" H91 ATX"}="1H9  ATX"; $resn_exch{" H92 ATX"}="2H9  ATX";
$resn_exch{" H93 ATX"}="3H9  ATX"; $resn_exch{"HPA2 ATY"}="2HPA ATY"; $resn_exch{"HPB2 ATY"}="2HPB ATY";
$resn_exch{"HPB3 ATY"}="3HPB ATY"; $resn_exch{"H5'2 ATY"}="2H5* ATY"; $resn_exch{"H5'1 ATY"}="1H5* ATY";
$resn_exch{" H4' ATY"}=" H4* ATY"; $resn_exch{" H3' ATY"}=" H3* ATY"; $resn_exch{"HHR3 ATY"}="3HHR ATY";
$resn_exch{"HHR2 ATY"}="2HHR ATY"; $resn_exch{"HHR1 ATY"}="1HHR ATY"; $resn_exch{"H2'2 ATY"}="2H2* ATY";
$resn_exch{"H2'1 ATY"}="1H2* ATY"; $resn_exch{" H1' ATY"}=" H1* ATY"; $resn_exch{"H5M3 ATY"}="3H5M ATY";
$resn_exch{"H5M2 ATY"}="2H5M ATY"; $resn_exch{"H5M1 ATY"}="1H5M ATY"; $resn_exch{" H91 ATZ"}="1H9  ATZ";
$resn_exch{" H92 ATZ"}="2H9  ATZ"; $resn_exch{" H93 ATZ"}="3H9  ATZ"; $resn_exch{"H101 ATZ"}="1H10 ATZ";
$resn_exch{"H102 ATZ"}="2H10 ATZ"; $resn_exch{"H103 ATZ"}="3H10 ATZ"; $resn_exch{"H121 ATZ"}="1H12 ATZ";
$resn_exch{"H122 ATZ"}="2H12 ATZ"; $resn_exch{"H131 ATZ"}="1H13 ATZ"; $resn_exch{"H132 ATZ"}="2H13 ATZ";
$resn_exch{"H133 ATZ"}="3H13 ATZ"; $resn_exch{"H141 AU4"}="1H14 AU4"; $resn_exch{"H142 AU4"}="2H14 AU4";
$resn_exch{"H143 AU4"}="3H14 AU4"; $resn_exch{"H151 AU4"}="1H15 AU4"; $resn_exch{"H152 AU4"}="2H15 AU4";
$resn_exch{"H153 AU4"}="3H15 AU4"; $resn_exch{"HN81 AU4"}="1HN8 AU4"; $resn_exch{"HN82 AU4"}="2HN8 AU4";
$resn_exch{"H171 AU4"}="1H17 AU4"; $resn_exch{"H172 AU4"}="2H17 AU4"; $resn_exch{"H173 AU4"}="3H17 AU4";
$resn_exch{"H161 AU4"}="1H16 AU4"; $resn_exch{"H162 AU4"}="2H16 AU4"; $resn_exch{"H163 AU4"}="3H16 AU4";
$resn_exch{" H21 AUA"}="1H2  AUA"; $resn_exch{" H22 AUA"}="2H2  AUA"; $resn_exch{" H51 AUA"}="1H5  AUA";
$resn_exch{" H52 AUA"}="2H5  AUA"; $resn_exch{"H201 AUA"}="1H20 AUA"; $resn_exch{"H202 AUA"}="2H20 AUA";
$resn_exch{"H203 AUA"}="3H20 AUA"; $resn_exch{"H241 AUA"}="1H24 AUA"; $resn_exch{"H242 AUA"}="2H24 AUA";
$resn_exch{"H291 AUA"}="1H29 AUA"; $resn_exch{"H292 AUA"}="2H29 AUA"; $resn_exch{"H293 AUA"}="3H29 AUA";
$resn_exch{"H331 AUA"}="1H33 AUA"; $resn_exch{"H332 AUA"}="2H33 AUA"; $resn_exch{"H333 AUA"}="3H33 AUA";
$resn_exch{"H511 AUA"}="1H51 AUA"; $resn_exch{"H512 AUA"}="2H51 AUA"; $resn_exch{"H551 AUA"}="1H55 AUA";
$resn_exch{"H552 AUA"}="2H55 AUA"; $resn_exch{"H671 AUA"}="1H67 AUA"; $resn_exch{"H672 AUA"}="2H67 AUA";
$resn_exch{"H673 AUA"}="3H67 AUA"; $resn_exch{"H711 AUA"}="1H71 AUA"; $resn_exch{"H712 AUA"}="2H71 AUA";
$resn_exch{"H713 AUA"}="3H71 AUA"; $resn_exch{"H791 AUA"}="1H79 AUA"; $resn_exch{"H792 AUA"}="2H79 AUA";
$resn_exch{"H821 AUA"}="1H82 AUA"; $resn_exch{"H822 AUA"}="2H82 AUA"; $resn_exch{"H851 AUA"}="1H85 AUA";
$resn_exch{"H852 AUA"}="2H85 AUA"; $resn_exch{"H881 AUA"}="1H88 AUA"; $resn_exch{"H882 AUA"}="2H88 AUA";
$resn_exch{"H883 AUA"}="3H88 AUA"; $resn_exch{"H921 AUA"}="1H92 AUA"; $resn_exch{"H922 AUA"}="2H92 AUA";
$resn_exch{"H971 AUA"}="1H97 AUA"; $resn_exch{"H972 AUA"}="2H97 AUA"; $resn_exch{"H973 AUA"}="3H97 AUA";
$resn_exch{"H981 AUA"}="1H98 AUA"; $resn_exch{"H982 AUA"}="2H98 AUA"; $resn_exch{"H983 AUA"}="3H98 AUA";
$resn_exch{"H991 AUA"}="1H99 AUA"; $resn_exch{"H992 AUA"}="2H99 AUA"; $resn_exch{"H993 AUA"}="3H99 AUA";
$resn_exch{" H31 AUP"}="1H3  AUP"; $resn_exch{" H32 AUP"}="2H3  AUP"; $resn_exch{" H41 AUP"}="1H4  AUP";
$resn_exch{" H42 AUP"}="2H4  AUP"; $resn_exch{" H51 AUP"}="1H5  AUP"; $resn_exch{" H52 AUP"}="2H5  AUP";
$resn_exch{" H61 AUP"}="1H6  AUP"; $resn_exch{" H62 AUP"}="2H6  AUP"; $resn_exch{"H11A AUR"}="1H1  AUR";
$resn_exch{"H12A AUR"}="2H1  AUR"; $resn_exch{"H13A AUR"}="3H1  AUR"; $resn_exch{" H21 AUR"}="1H2  AUR";
$resn_exch{" H22 AUR"}="2H2  AUR"; $resn_exch{"H201 AUR"}="1H20 AUR"; $resn_exch{"H202 AUR"}="2H20 AUR";
$resn_exch{"H203 AUR"}="3H20 AUR"; $resn_exch{"H211 AUR"}="1H21 AUR"; $resn_exch{"H212 AUR"}="2H21 AUR";
$resn_exch{"H213 AUR"}="3H21 AUR"; $resn_exch{"H241 AUR"}="1H24 AUR"; $resn_exch{"H242 AUR"}="2H24 AUR";
$resn_exch{"H243 AUR"}="3H24 AUR"; $resn_exch{"H221 AUR"}="1H22 AUR"; $resn_exch{"H222 AUR"}="2H22 AUR";
$resn_exch{"H223 AUR"}="3H22 AUR"; $resn_exch{"H231 AUR"}="1H23 AUR"; $resn_exch{"H232 AUR"}="2H23 AUR";
$resn_exch{"H233 AUR"}="3H23 AUR"; $resn_exch{"HOB2 AV2"}="2HOB AV2"; $resn_exch{"HOB3 AV2"}="3HOB AV2";
$resn_exch{"HOA2 AV2"}="2HOA AV2"; $resn_exch{"H5'1 AV2"}="1H5* AV2"; $resn_exch{"H5'2 AV2"}="2H5* AV2";
$resn_exch{" H4' AV2"}=" H4* AV2"; $resn_exch{" H3' AV2"}=" H3* AV2"; $resn_exch{" H2' AV2"}=" H2* AV2";
$resn_exch{" H1' AV2"}=" H1* AV2"; $resn_exch{"HN61 AV2"}="1HN6 AV2"; $resn_exch{"HN62 AV2"}="2HN6 AV2";
$resn_exch{"HOP3 AVC"}="3HOP AVC"; $resn_exch{"HOP2 AVC"}="2HOP AVC"; $resn_exch{" H5' AVC"}="1H5* AVC";
$resn_exch{"H5'' AVC"}="2H5* AVC"; $resn_exch{" H4' AVC"}=" H4* AVC"; $resn_exch{" H3' AVC"}=" H3* AVC";
$resn_exch{" H2' AVC"}=" H2* AVC"; $resn_exch{" H1' AVC"}=" H1* AVC"; $resn_exch{" H61 AVC"}="1H6  AVC";
$resn_exch{" H62 AVC"}="2H6  AVC"; $resn_exch{"HN11 AVG"}="1HN1 AVG"; $resn_exch{"HN12 AVG"}="2HN1 AVG";
$resn_exch{"HC31 AVG"}="1HC3 AVG"; $resn_exch{"HC32 AVG"}="2HC3 AVG"; $resn_exch{"HC41 AVG"}="1HC4 AVG";
$resn_exch{"HC42 AVG"}="2HC4 AVG"; $resn_exch{"HC51 AVG"}="1HC5 AVG"; $resn_exch{"HC52 AVG"}="2HC5 AVG";
$resn_exch{"HC61 AVG"}="1HC6 AVG"; $resn_exch{"HC62 AVG"}="2HC6 AVG"; $resn_exch{"HN21 AVG"}="1HN2 AVG";
$resn_exch{"HN22 AVG"}="2HN2 AVG"; $resn_exch{"H2A1 AVP"}="1H2A AVP"; $resn_exch{"H'51 AVP"}="1H'5 AVP";
$resn_exch{"H'52 AVP"}="2H'5 AVP"; $resn_exch{"H'41 AVP"}="1H'4 AVP"; $resn_exch{"H'42 AVP"}="2H'4 AVP";
$resn_exch{"H'11 AVP"}="1H'1 AVP"; $resn_exch{"H'12 AVP"}="2H'1 AVP"; $resn_exch{"HN21 AVP"}="1HN2 AVP";
$resn_exch{"HN22 AVP"}="2HN2 AVP"; $resn_exch{"HN11 AVP"}="1HN1 AVP"; $resn_exch{"HC81 AVP"}="1HC8 AVP";
$resn_exch{"H2B1 AVP"}="1H2B AVP"; $resn_exch{"H1G1 AVP"}="1H1G AVP"; $resn_exch{"H2G1 AVP"}="1H2G AVP";
$resn_exch{" H31 AXD"}="1H3  AXD"; $resn_exch{" H32 AXD"}="2H3  AXD"; $resn_exch{" H41 AXD"}="1H4  AXD";
$resn_exch{" H42 AXD"}="2H4  AXD"; $resn_exch{" H61 AXD"}="1H6  AXD"; $resn_exch{" H62 AXD"}="2H6  AXD";
$resn_exch{" H71 AXD"}="1H7  AXD"; $resn_exch{" H72 AXD"}="2H7  AXD"; $resn_exch{"H111 AXD"}="1H11 AXD";
$resn_exch{"H112 AXD"}="2H11 AXD"; $resn_exch{"H121 AXD"}="1H12 AXD"; $resn_exch{"H122 AXD"}="2H12 AXD";
$resn_exch{"H131 AXD"}="1H13 AXD"; $resn_exch{"H132 AXD"}="2H13 AXD"; $resn_exch{"H141 AXD"}="1H14 AXD";
$resn_exch{"H142 AXD"}="2H14 AXD"; $resn_exch{"H171 AXD"}="1H17 AXD"; $resn_exch{"H172 AXD"}="2H17 AXD";
$resn_exch{"H261 AXD"}="1H26 AXD"; $resn_exch{"H262 AXD"}="2H26 AXD"; $resn_exch{"H263 AXD"}="3H26 AXD";
$resn_exch{" H81 AXF"}="1H8  AXF"; $resn_exch{" H82 AXF"}="2H8  AXF"; $resn_exch{"H111 AXF"}="1H11 AXF";
$resn_exch{"H112 AXF"}="2H11 AXF"; $resn_exch{"H141 AXF"}="1H14 AXF"; $resn_exch{"H142 AXF"}="2H14 AXF";
$resn_exch{"H271 AXF"}="1H27 AXF"; $resn_exch{"H272 AXF"}="2H27 AXF"; $resn_exch{"H311 AXF"}="1H31 AXF";
$resn_exch{"H312 AXF"}="2H31 AXF"; $resn_exch{"H381 AXF"}="1H38 AXF"; $resn_exch{"H382 AXF"}="2H38 AXF";
$resn_exch{"H451 AXF"}="1H45 AXF"; $resn_exch{"H452 AXF"}="2H45 AXF"; $resn_exch{"H501 AXF"}="1H50 AXF";
$resn_exch{"H502 AXF"}="2H50 AXF"; $resn_exch{"H503 AXF"}="3H50 AXF"; $resn_exch{"H581 AXF"}="1H58 AXF";
$resn_exch{"H582 AXF"}="2H58 AXF"; $resn_exch{"H611 AXF"}="1H61 AXF"; $resn_exch{"H612 AXF"}="2H61 AXF";
$resn_exch{"H641 AXF"}="1H64 AXF"; $resn_exch{"H642 AXF"}="2H64 AXF"; $resn_exch{"H671 AXF"}="1H67 AXF";
$resn_exch{"H672 AXF"}="2H67 AXF"; $resn_exch{"H673 AXF"}="3H67 AXF"; $resn_exch{"H721 AXF"}="1H72 AXF";
$resn_exch{"H722 AXF"}="2H72 AXF"; $resn_exch{"H723 AXF"}="3H72 AXF"; $resn_exch{"H511 AXL"}="1H51 AXL";
$resn_exch{"H512 AXL"}="2H51 AXL"; $resn_exch{"H513 AXL"}="3H51 AXL"; $resn_exch{"H521 AXL"}="1H52 AXL";
$resn_exch{"H522 AXL"}="2H52 AXL"; $resn_exch{"H523 AXL"}="3H52 AXL"; $resn_exch{"H62O AXL"}="OH62 AXL";
$resn_exch{"H111 AXL"}="1H11 AXL"; $resn_exch{"H112 AXL"}="2H11 AXL"; $resn_exch{"H18O AXL"}="OH18 AXL";
$resn_exch{" H31 AXP"}="1H3  AXP"; $resn_exch{" H32 AXP"}="2H3  AXP"; $resn_exch{" H91 AXP"}="1H9  AXP";
$resn_exch{" H92 AXP"}="2H9  AXP"; $resn_exch{"H111 AXP"}="1H11 AXP"; $resn_exch{"H112 AXP"}="2H11 AXP";
$resn_exch{"H113 AXP"}="3H11 AXP"; $resn_exch{"HOP2 AXP"}="2HOP AXP"; $resn_exch{"HOP3 AXP"}="3HOP AXP";
$resn_exch{" H21 AXQ"}="1H2  AXQ"; $resn_exch{" H22 AXQ"}="2H2  AXQ"; $resn_exch{"H171 AXQ"}="1H17 AXQ";
$resn_exch{"H172 AXQ"}="2H17 AXQ"; $resn_exch{"H211 AXQ"}="1H21 AXQ"; $resn_exch{"H212 AXQ"}="2H21 AXQ";
$resn_exch{"H281 AXQ"}="1H28 AXQ"; $resn_exch{"H282 AXQ"}="2H28 AXQ"; $resn_exch{"H371 AXQ"}="1H37 AXQ";
$resn_exch{"H372 AXQ"}="2H37 AXQ"; $resn_exch{"H373 AXQ"}="3H37 AXQ"; $resn_exch{"H411 AXQ"}="1H41 AXQ";
$resn_exch{"H412 AXQ"}="2H41 AXQ"; $resn_exch{"H413 AXQ"}="3H41 AXQ"; $resn_exch{"H451 AXQ"}="1H45 AXQ";
$resn_exch{"H452 AXQ"}="2H45 AXQ"; $resn_exch{"H453 AXQ"}="3H45 AXQ"; $resn_exch{"H501 AXQ"}="1H50 AXQ";
$resn_exch{"H502 AXQ"}="2H50 AXQ"; $resn_exch{"H503 AXQ"}="3H50 AXQ"; $resn_exch{"H591 AXQ"}="1H59 AXQ";
$resn_exch{"H592 AXQ"}="2H59 AXQ"; $resn_exch{"H641 AXQ"}="1H64 AXQ"; $resn_exch{"H642 AXQ"}="2H64 AXQ";
$resn_exch{"H643 AXQ"}="3H64 AXQ"; $resn_exch{"H761 AXQ"}="1H76 AXQ"; $resn_exch{"H762 AXQ"}="2H76 AXQ";
$resn_exch{"H763 AXQ"}="3H76 AXQ"; $resn_exch{"H801 AXQ"}="1H80 AXQ"; $resn_exch{"H802 AXQ"}="2H80 AXQ";
$resn_exch{"H803 AXQ"}="3H80 AXQ"; $resn_exch{"H881 AXQ"}="1H88 AXQ"; $resn_exch{"H882 AXQ"}="2H88 AXQ";
$resn_exch{"H911 AXQ"}="1H91 AXQ"; $resn_exch{"H912 AXQ"}="2H91 AXQ"; $resn_exch{"H941 AXQ"}="1H94 AXQ";
$resn_exch{"H942 AXQ"}="2H94 AXQ"; $resn_exch{"H971 AXQ"}="1H97 AXQ"; $resn_exch{"H972 AXQ"}="2H97 AXQ";
$resn_exch{"H973 AXQ"}="3H97 AXQ"; $resn_exch{"HC21 AXT"}="1HC2 AXT"; $resn_exch{"HC22 AXT"}="2HC2 AXT";
$resn_exch{"H161 AXT"}="1H16 AXT"; $resn_exch{"H162 AXT"}="2H16 AXT"; $resn_exch{"H163 AXT"}="3H16 AXT";
$resn_exch{"H171 AXT"}="1H17 AXT"; $resn_exch{"H172 AXT"}="2H17 AXT"; $resn_exch{"H173 AXT"}="3H17 AXT";
$resn_exch{"H181 AXT"}="1H18 AXT"; $resn_exch{"H182 AXT"}="2H18 AXT"; $resn_exch{"H183 AXT"}="3H18 AXT";
$resn_exch{"H191 AXT"}="1H19 AXT"; $resn_exch{"H192 AXT"}="2H19 AXT"; $resn_exch{"H193 AXT"}="3H19 AXT";
$resn_exch{"H201 AXT"}="1H20 AXT"; $resn_exch{"H202 AXT"}="2H20 AXT"; $resn_exch{"H203 AXT"}="3H20 AXT";
$resn_exch{"H221 AXT"}="1H22 AXT"; $resn_exch{"H222 AXT"}="2H22 AXT"; $resn_exch{"H361 AXT"}="1H36 AXT";
$resn_exch{"H362 AXT"}="2H36 AXT"; $resn_exch{"H363 AXT"}="3H36 AXT"; $resn_exch{"H371 AXT"}="1H37 AXT";
$resn_exch{"H372 AXT"}="2H37 AXT"; $resn_exch{"H373 AXT"}="3H37 AXT"; $resn_exch{"H381 AXT"}="1H38 AXT";
$resn_exch{"H382 AXT"}="2H38 AXT"; $resn_exch{"H383 AXT"}="3H38 AXT"; $resn_exch{"H391 AXT"}="1H39 AXT";
$resn_exch{"H392 AXT"}="2H39 AXT"; $resn_exch{"H393 AXT"}="3H39 AXT"; $resn_exch{"H401 AXT"}="1H40 AXT";
$resn_exch{"H402 AXT"}="2H40 AXT"; $resn_exch{"H403 AXT"}="3H40 AXT"; $resn_exch{" H31 AXX"}="1H3  AXX";
$resn_exch{" H32 AXX"}="2H3  AXX"; $resn_exch{" H11 AXX"}="1H1  AXX"; $resn_exch{" H12 AXX"}="2H1  AXX";
$resn_exch{"H181 AXX"}="1H18 AXX"; $resn_exch{"H182 AXX"}="2H18 AXX"; $resn_exch{"H171 AXX"}="1H17 AXX";
$resn_exch{"H172 AXX"}="2H17 AXX"; $resn_exch{"H161 AXX"}="1H16 AXX"; $resn_exch{"H162 AXX"}="2H16 AXX";
$resn_exch{"H151 AXX"}="1H15 AXX"; $resn_exch{"H152 AXX"}="2H15 AXX"; $resn_exch{"H141 AXX"}="1H14 AXX";
$resn_exch{"H142 AXX"}="2H14 AXX"; $resn_exch{"H111 AY1"}="1H11 AY1"; $resn_exch{"H112 AY1"}="2H11 AY1";
$resn_exch{"H113 AY1"}="3H11 AY1"; $resn_exch{"H161 AY1"}="1H16 AY1"; $resn_exch{"H162 AY1"}="2H16 AY1";
$resn_exch{"H163 AY1"}="3H16 AY1"; $resn_exch{"H181 AY1"}="1H18 AY1"; $resn_exch{"H182 AY1"}="2H18 AY1";
$resn_exch{"H191 AY1"}="1H19 AY1"; $resn_exch{"H192 AY1"}="2H19 AY1"; $resn_exch{"H201 AY1"}="1H20 AY1";
$resn_exch{"H202 AY1"}="2H20 AY1"; $resn_exch{"H211 AY1"}="1H21 AY1"; $resn_exch{"H212 AY1"}="2H21 AY1";
$resn_exch{"H221 AY1"}="1H22 AY1"; $resn_exch{"H222 AY1"}="2H22 AY1"; $resn_exch{"H231 AY1"}="1H23 AY1";
$resn_exch{"H232 AY1"}="2H23 AY1"; $resn_exch{"H233 AY1"}="3H23 AY1"; $resn_exch{"H261 AY1"}="1H26 AY1";
$resn_exch{"H262 AY1"}="2H26 AY1"; $resn_exch{"H263 AY1"}="3H26 AY1"; $resn_exch{"H271 AY1"}="1H27 AY1";
$resn_exch{"H272 AY1"}="2H27 AY1"; $resn_exch{"H273 AY1"}="3H27 AY1"; $resn_exch{" H   AYA"}=" HN  AYA";
$resn_exch{" HB1 AYA"}="1HB  AYA"; $resn_exch{" HB2 AYA"}="2HB  AYA"; $resn_exch{" HB3 AYA"}="3HB  AYA";
$resn_exch{" HM1 AYA"}="1HM  AYA"; $resn_exch{" HM2 AYA"}="2HM  AYA"; $resn_exch{" HM3 AYA"}="3HM  AYA";
$resn_exch{"HM21 AYD"}="1HM2 AYD"; $resn_exch{"HM22 AYD"}="2HM2 AYD"; $resn_exch{"HM23 AYD"}="3HM2 AYD";
$resn_exch{"H4'1 AYD"}="1H4' AYD"; $resn_exch{"H4'2 AYD"}="2H4' AYD"; $resn_exch{"H7'1 AYD"}="1H7' AYD";
$resn_exch{"H7'2 AYD"}="2H7' AYD"; $resn_exch{"HM41 AYD"}="1HM4 AYD"; $resn_exch{"HM42 AYD"}="2HM4 AYD";
$resn_exch{"HM43 AYD"}="3HM4 AYD"; $resn_exch{" H61 AYD"}="1H6  AYD"; $resn_exch{" H62 AYD"}="2H6  AYD";
$resn_exch{" H71 AYD"}="1H7  AYD"; $resn_exch{" H72 AYD"}="2H7  AYD"; $resn_exch{"HA31 AYG"}="1HA3 AYG";
$resn_exch{"HA32 AYG"}="2HA3 AYG"; $resn_exch{"HB11 AYG"}="1HB1 AYG"; $resn_exch{"HB12 AYG"}="2HB1 AYG";
$resn_exch{"HB13 AYG"}="3HB1 AYG"; $resn_exch{"HN11 AYG"}="1HN1 AYG"; $resn_exch{" H2  AYG"}=" HN2 AYG";
$resn_exch{"H011 AYM"}="1H01 AYM"; $resn_exch{"H012 AYM"}="2H01 AYM"; $resn_exch{"H131 AYM"}="1H13 AYM";
$resn_exch{"H132 AYM"}="2H13 AYM"; $resn_exch{"H133 AYM"}="3H13 AYM"; $resn_exch{"H141 AYM"}="1H14 AYM";
$resn_exch{"H142 AYM"}="2H14 AYM"; $resn_exch{"H201 AYM"}="1H20 AYM"; $resn_exch{"H202 AYM"}="2H20 AYM";
$resn_exch{"H203 AYM"}="3H20 AYM"; $resn_exch{" H21 AZ1"}="1H2  AZ1"; $resn_exch{" H22 AZ1"}="2H2  AZ1";
$resn_exch{" H31 AZ1"}="1H3  AZ1"; $resn_exch{" H32 AZ1"}="2H3  AZ1"; $resn_exch{" H41 AZ1"}="1H4  AZ1";
$resn_exch{" H42 AZ1"}="2H4  AZ1"; $resn_exch{" H51 AZ1"}="1H5  AZ1"; $resn_exch{" H52 AZ1"}="2H5  AZ1";
$resn_exch{" H61 AZ1"}="1H6  AZ1"; $resn_exch{" H62 AZ1"}="2H6  AZ1"; $resn_exch{" H71 AZ1"}="1H7  AZ1";
$resn_exch{" H72 AZ1"}="2H7  AZ1"; $resn_exch{" H81 AZ1"}="1H8  AZ1"; $resn_exch{" H82 AZ1"}="2H8  AZ1";
$resn_exch{" H21 AZ2"}="1H2  AZ2"; $resn_exch{" H22 AZ2"}="2H2  AZ2"; $resn_exch{" H51 AZ2"}="1H5  AZ2";
$resn_exch{" H81 AZ2"}="1H8  AZ2"; $resn_exch{" H82 AZ2"}="2H8  AZ2"; $resn_exch{"H111 AZ2"}="1H11 AZ2";
$resn_exch{"H112 AZ2"}="2H11 AZ2"; $resn_exch{"H113 AZ2"}="3H11 AZ2"; $resn_exch{"H161 AZ2"}="1H16 AZ2";
$resn_exch{"H191 AZ2"}="1H19 AZ2"; $resn_exch{"H211 AZ2"}="1H21 AZ2"; $resn_exch{"H241 AZ2"}="1H24 AZ2";
$resn_exch{"H261 AZ2"}="1H26 AZ2"; $resn_exch{"H291 AZ2"}="1H29 AZ2"; $resn_exch{"H292 AZ2"}="2H29 AZ2";
$resn_exch{"H321 AZ2"}="1H32 AZ2"; $resn_exch{"H322 AZ2"}="2H32 AZ2"; $resn_exch{"H361 AZ2"}="1H36 AZ2";
$resn_exch{"H381 AZ2"}="1H38 AZ2"; $resn_exch{"H411 AZ2"}="1H41 AZ2"; $resn_exch{"H431 AZ2"}="1H43 AZ2";
$resn_exch{"H471 AZ2"}="1H47 AZ2"; $resn_exch{"H472 AZ2"}="2H47 AZ2"; $resn_exch{"H473 AZ2"}="3H47 AZ2";
$resn_exch{" H61 AZC"}="1H6  AZC"; $resn_exch{" H62 AZC"}="2H6  AZC"; $resn_exch{" H91 AZC"}="1H9  AZC";
$resn_exch{" H92 AZC"}="2H9  AZC"; $resn_exch{" H93 AZC"}="3H9  AZC"; $resn_exch{"HOA2 AZD"}="2HOA AZD";
$resn_exch{"HOB2 AZD"}="2HOB AZD"; $resn_exch{"HOB3 AZD"}="3HOB AZD"; $resn_exch{"H5'1 AZD"}="1H5* AZD";
$resn_exch{"H5'2 AZD"}="2H5* AZD"; $resn_exch{" H4' AZD"}=" H4* AZD"; $resn_exch{" H3' AZD"}=" H3* AZD";
$resn_exch{"H2'1 AZD"}="1H2* AZD"; $resn_exch{"H2'2 AZD"}="2H2* AZD"; $resn_exch{" H1' AZD"}=" H1* AZD";
$resn_exch{" H51 AZD"}="1H5  AZD"; $resn_exch{" H52 AZD"}="2H5  AZD"; $resn_exch{" H53 AZD"}="3H5  AZD";
$resn_exch{" H21 AZE"}="1H2  AZE"; $resn_exch{" H22 AZE"}="2H2  AZE"; $resn_exch{" H31 AZE"}="1H3  AZE";
$resn_exch{" H32 AZE"}="2H3  AZE"; $resn_exch{" H41 AZE"}="1H4  AZE"; $resn_exch{" H42 AZE"}="2H4  AZE";
$resn_exch{"H151 AZE"}="1H15 AZE"; $resn_exch{"H152 AZE"}="2H15 AZE"; $resn_exch{"H153 AZE"}="3H15 AZE";
$resn_exch{"H161 AZE"}="1H16 AZE"; $resn_exch{"H162 AZE"}="2H16 AZE"; $resn_exch{"H163 AZE"}="3H16 AZE";
$resn_exch{"H171 AZE"}="1H17 AZE"; $resn_exch{"H172 AZE"}="2H17 AZE"; $resn_exch{"H173 AZE"}="3H17 AZE";
$resn_exch{"H181 AZE"}="1H18 AZE"; $resn_exch{"H182 AZE"}="2H18 AZE"; $resn_exch{"H183 AZE"}="3H18 AZE";
$resn_exch{"H191 AZE"}="1H19 AZE"; $resn_exch{"H192 AZE"}="2H19 AZE"; $resn_exch{"H193 AZE"}="3H19 AZE";
$resn_exch{"H201 AZE"}="1H20 AZE"; $resn_exch{"H202 AZE"}="2H20 AZE"; $resn_exch{"H203 AZE"}="3H20 AZE";
$resn_exch{"H2C1 AZF"}="1H2C AZF"; $resn_exch{"H2C2 AZF"}="2H2C AZF"; $resn_exch{"H6C1 AZF"}="1H6C AZF";
$resn_exch{"H6C2 AZF"}="2H6C AZF"; $resn_exch{"HN21 AZG"}="1HN2 AZG"; $resn_exch{"HN22 AZG"}="2HN2 AZG";
$resn_exch{" H   AZK"}="1HN  AZK"; $resn_exch{" H2  AZK"}="2HN  AZK"; $resn_exch{" HB2 AZK"}="1HB  AZK";
$resn_exch{" HB3 AZK"}="2HB  AZK"; $resn_exch{" HG2 AZK"}="1HG  AZK"; $resn_exch{" HG3 AZK"}="2HG  AZK";
$resn_exch{" HD2 AZK"}="1HD  AZK"; $resn_exch{" HD3 AZK"}="2HD  AZK"; $resn_exch{" HE2 AZK"}="1HE  AZK";
$resn_exch{" HE3 AZK"}="2HE  AZK"; $resn_exch{" HC1 AZK"}="1H   AZK"; $resn_exch{" HC2 AZK"}="2H   AZK";
$resn_exch{"HN11 AZL"}="1HN1 AZL"; $resn_exch{"HN12 AZL"}="2HN1 AZL"; $resn_exch{"H21A AZL"}="1H2  AZL";
$resn_exch{"H22A AZL"}="2H2  AZL"; $resn_exch{" H31 AZL"}="1H3  AZL"; $resn_exch{" H32 AZL"}="2H3  AZL";
$resn_exch{" H41 AZL"}="1H4  AZL"; $resn_exch{" H42 AZL"}="2H4  AZL"; $resn_exch{" H51 AZL"}="1H5  AZL";
$resn_exch{" H52 AZL"}="2H5  AZL"; $resn_exch{"H101 AZL"}="1H10 AZL"; $resn_exch{"H102 AZL"}="2H10 AZL";
$resn_exch{"H111 AZL"}="1H11 AZL"; $resn_exch{"H112 AZL"}="2H11 AZL"; $resn_exch{"H121 AZL"}="1H12 AZL";
$resn_exch{"H122 AZL"}="2H12 AZL"; $resn_exch{"H161 AZL"}="1H16 AZL"; $resn_exch{"H162 AZL"}="2H16 AZL";
$resn_exch{"H241 AZL"}="1H24 AZL"; $resn_exch{"H242 AZL"}="2H24 AZL"; $resn_exch{"H251 AZL"}="1H25 AZL";
$resn_exch{"H252 AZL"}="2H25 AZL"; $resn_exch{"H253 AZL"}="3H25 AZL"; $resn_exch{" H41 AZM"}="1H4  AZM";
$resn_exch{" H42 AZM"}="2H4  AZM"; $resn_exch{" H43 AZM"}="3H4  AZM"; $resn_exch{"HN11 AZM"}="1HN1 AZM";
$resn_exch{"HN12 AZM"}="2HN1 AZM"; $resn_exch{"H201 AZO"}="1H20 AZO"; $resn_exch{"H202 AZO"}="2H20 AZO";
$resn_exch{"H203 AZO"}="3H20 AZO"; $resn_exch{"H221 AZO"}="1H22 AZO"; $resn_exch{"H222 AZO"}="2H22 AZO";
$resn_exch{"H223 AZO"}="3H22 AZO"; $resn_exch{"HAG1 AZP"}="1HAG AZP"; $resn_exch{"HAG2 AZP"}="2HAG AZP";
$resn_exch{" HB1 AZP"}="1HB  AZP"; $resn_exch{" HB2 AZP"}="2HB  AZP"; $resn_exch{"HD11 AZP"}="1HD1 AZP";
$resn_exch{"HD12 AZP"}="2HD1 AZP"; $resn_exch{"HD13 AZP"}="3HD1 AZP"; $resn_exch{"HD21 AZP"}="1HD2 AZP";
$resn_exch{"HD22 AZP"}="2HD2 AZP"; $resn_exch{"HD23 AZP"}="3HD2 AZP"; $resn_exch{"HAV1 AZP"}="1HAV AZP";
$resn_exch{"HAV2 AZP"}="2HAV AZP"; $resn_exch{"HBA1 AZP"}="1HBA AZP"; $resn_exch{"HBA2 AZP"}="2HBA AZP";
$resn_exch{"HBB1 AZP"}="1HBB AZP"; $resn_exch{"HBB2 AZP"}="2HBB AZP"; $resn_exch{"HBE1 AZP"}="1HBE AZP";
$resn_exch{"HBE2 AZP"}="2HBE AZP"; $resn_exch{"HBM1 AZP"}="1HBM AZP"; $resn_exch{"HBM2 AZP"}="2HBM AZP";
$resn_exch{"HBS1 AZP"}="1HBS AZP"; $resn_exch{"HBS2 AZP"}="2HBS AZP"; $resn_exch{"HBT1 AZP"}="1HBT AZP";
$resn_exch{"HBT2 AZP"}="2HBT AZP"; $resn_exch{"HBT3 AZP"}="3HBT AZP"; $resn_exch{"H111 AZQ"}="1H11 AZQ";
$resn_exch{"H112 AZQ"}="2H11 AZQ"; $resn_exch{"H131 AZQ"}="1H13 AZQ"; $resn_exch{"H132 AZQ"}="2H13 AZQ";
$resn_exch{"H141 AZQ"}="1H14 AZQ"; $resn_exch{"H142 AZQ"}="2H14 AZQ"; $resn_exch{"H143 AZQ"}="3H14 AZQ";
$resn_exch{"H151 AZQ"}="1H15 AZQ"; $resn_exch{"H152 AZQ"}="2H15 AZQ"; $resn_exch{"H153 AZQ"}="3H15 AZQ";
$resn_exch{"H161 AZQ"}="1H16 AZQ"; $resn_exch{"H162 AZQ"}="2H16 AZQ"; $resn_exch{"H163 AZQ"}="3H16 AZQ";
$resn_exch{" H71 AZR"}="1H7  AZR"; $resn_exch{" H72 AZR"}="2H7  AZR"; $resn_exch{" H73 AZR"}="3H7  AZR";
$resn_exch{"H161 AZR"}="1H16 AZR"; $resn_exch{"H162 AZR"}="2H16 AZR"; $resn_exch{"H271 AZR"}="1H27 AZR";
$resn_exch{"H272 AZR"}="2H27 AZR"; $resn_exch{"H273 AZR"}="3H27 AZR"; $resn_exch{"H281 AZR"}="1H28 AZR";
$resn_exch{"H282 AZR"}="2H28 AZR"; $resn_exch{"H283 AZR"}="3H28 AZR"; $resn_exch{"HOA2 AZT"}="2HOA AZT";
$resn_exch{"HOB2 AZT"}="2HOB AZT"; $resn_exch{"HOG2 AZT"}="2HOG AZT"; $resn_exch{"HOG3 AZT"}="3HOG AZT";
$resn_exch{"H5'1 AZT"}="1H5* AZT"; $resn_exch{"H5'2 AZT"}="2H5* AZT"; $resn_exch{" H4' AZT"}=" H4* AZT";
$resn_exch{" H3' AZT"}=" H3* AZT"; $resn_exch{"H2'1 AZT"}="1H2* AZT"; $resn_exch{"H2'2 AZT"}="2H2* AZT";
$resn_exch{" H1' AZT"}=" H1* AZT"; $resn_exch{" H51 AZT"}="1H5  AZT"; $resn_exch{" H52 AZT"}="2H5  AZT";
$resn_exch{" H53 AZT"}="3H5  AZT"; $resn_exch{"H261 AZX"}="1H26 AZX"; $resn_exch{"H262 AZX"}="2H26 AZX";
$resn_exch{"H263 AZX"}="3H26 AZX"; $resn_exch{"H111 AZX"}="1H11 AZX"; $resn_exch{"H112 AZX"}="2H11 AZX";
$resn_exch{"H113 AZX"}="3H11 AZX"; $resn_exch{"H101 AZX"}="1H10 AZX"; $resn_exch{"H102 AZX"}="2H10 AZX";
$resn_exch{"H103 AZX"}="3H10 AZX"; $resn_exch{" HN1 AZY"}="1HN  AZY"; $resn_exch{" HN2 AZY"}="2HN  AZY";
$resn_exch{" HB1 AZY"}="1HB  AZY"; $resn_exch{" HB2 AZY"}="2HB  AZY"; $resn_exch{"HAU1 B08"}="1HAU B08";
$resn_exch{"HAU2 B08"}="2HAU B08"; $resn_exch{" H81 B11"}="1H8  B11"; $resn_exch{" H82 B11"}="2H8  B11";
$resn_exch{"H111 B11"}="1H11 B11"; $resn_exch{"H112 B11"}="2H11 B11"; $resn_exch{"H121 B11"}="1H12 B11";
$resn_exch{"H122 B11"}="2H12 B11"; $resn_exch{"H141 B11"}="1H14 B11"; $resn_exch{"H142 B11"}="2H14 B11";
$resn_exch{"H151 B11"}="1H15 B11"; $resn_exch{"H152 B11"}="2H15 B11"; $resn_exch{"H201 B12"}="1H20 B12";
$resn_exch{"H202 B12"}="2H20 B12"; $resn_exch{"H203 B12"}="3H20 B12"; $resn_exch{"H251 B12"}="1H25 B12";
$resn_exch{"H252 B12"}="2H25 B12"; $resn_exch{"H253 B12"}="3H25 B12"; $resn_exch{"H261 B12"}="1H26 B12";
$resn_exch{"H262 B12"}="2H26 B12"; $resn_exch{"H291 B12"}="1H29 B12"; $resn_exch{"H292 B12"}="2H29 B12";
$resn_exch{"H301 B12"}="1H30 B12"; $resn_exch{"H302 B12"}="2H30 B12"; $resn_exch{"H311 B12"}="1H31 B12";
$resn_exch{"H312 B12"}="2H31 B12"; $resn_exch{"H331 B12"}="1H33 B12"; $resn_exch{"H332 B12"}="2H33 B12";
$resn_exch{"H351 B12"}="1H35 B12"; $resn_exch{"H352 B12"}="2H35 B12"; $resn_exch{"H353 B12"}="3H35 B12";
$resn_exch{"H361 B12"}="1H36 B12"; $resn_exch{"H362 B12"}="2H36 B12"; $resn_exch{"H363 B12"}="3H36 B12";
$resn_exch{"H371 B12"}="1H37 B12"; $resn_exch{"H372 B12"}="2H37 B12"; $resn_exch{"H401 B12"}="1H40 B12";
$resn_exch{"H402 B12"}="2H40 B12"; $resn_exch{"H411 B12"}="1H41 B12"; $resn_exch{"H412 B12"}="2H41 B12";
$resn_exch{"H421 B12"}="1H42 B12"; $resn_exch{"H422 B12"}="2H42 B12"; $resn_exch{"H451 B12"}="1H45 B12";
$resn_exch{"H452 B12"}="2H45 B12"; $resn_exch{"H461 B12"}="1H46 B12"; $resn_exch{"H462 B12"}="2H46 B12";
$resn_exch{"H463 B12"}="3H46 B12"; $resn_exch{"H471 B12"}="1H47 B12"; $resn_exch{"H472 B12"}="2H47 B12";
$resn_exch{"H473 B12"}="3H47 B12"; $resn_exch{"H481 B12"}="1H48 B12"; $resn_exch{"H482 B12"}="2H48 B12";
$resn_exch{"H491 B12"}="1H49 B12"; $resn_exch{"H492 B12"}="2H49 B12"; $resn_exch{"H521 B12"}="1H52 B12";
$resn_exch{"H522 B12"}="2H52 B12"; $resn_exch{"H531 B12"}="1H53 B12"; $resn_exch{"H532 B12"}="2H53 B12";
$resn_exch{"H533 B12"}="3H53 B12"; $resn_exch{"H541 B12"}="1H54 B12"; $resn_exch{"H542 B12"}="2H54 B12";
$resn_exch{"H543 B12"}="3H54 B12"; $resn_exch{"H551 B12"}="1H55 B12"; $resn_exch{"H552 B12"}="2H55 B12";
$resn_exch{"H561 B12"}="1H56 B12"; $resn_exch{"H562 B12"}="2H56 B12"; $resn_exch{"H601 B12"}="1H60 B12";
$resn_exch{"H602 B12"}="2H60 B12"; $resn_exch{"H621 B12"}="1H62 B12"; $resn_exch{"H622 B12"}="2H62 B12";
$resn_exch{"H1P1 B12"}="1H1P B12"; $resn_exch{"H1P2 B12"}="2H1P B12"; $resn_exch{"H3P1 B12"}="1H3P B12";
$resn_exch{"H3P2 B12"}="2H3P B12"; $resn_exch{"H3P3 B12"}="3H3P B12"; $resn_exch{"HOP5 B12"}="5HOP B12";
$resn_exch{"HOR7 B12"}="7HOR B12"; $resn_exch{"H5R1 B12"}="1H5R B12"; $resn_exch{"H5R2 B12"}="2H5R B12";
$resn_exch{"HOR8 B12"}="8HOR B12"; $resn_exch{"HM51 B12"}="1HM5 B12"; $resn_exch{"HM52 B12"}="2HM5 B12";
$resn_exch{"HM53 B12"}="3HM5 B12"; $resn_exch{"HM61 B12"}="1HM6 B12"; $resn_exch{"HM62 B12"}="2HM6 B12";
$resn_exch{"HM63 B12"}="3HM6 B12"; $resn_exch{"H3P1 B13"}="1H3P B13"; $resn_exch{"H3P2 B13"}="2H3P B13";
$resn_exch{"H3P3 B13"}="3H3P B13"; $resn_exch{"H1P1 B13"}="1H1P B13"; $resn_exch{"H1P2 B13"}="2H1P B13";
$resn_exch{"H561 B13"}="1H56 B13"; $resn_exch{"H562 B13"}="2H56 B13"; $resn_exch{"H551 B13"}="1H55 B13";
$resn_exch{"H552 B13"}="2H55 B13"; $resn_exch{"H541 B13"}="1H54 B13"; $resn_exch{"H542 B13"}="2H54 B13";
$resn_exch{"H543 B13"}="3H54 B13"; $resn_exch{"H601 B13"}="1H60 B13"; $resn_exch{"H602 B13"}="2H60 B13";
$resn_exch{"H621 B13"}="1H62 B13"; $resn_exch{"H622 B13"}="2H62 B13"; $resn_exch{"H351 B13"}="1H35 B13";
$resn_exch{"H352 B13"}="2H35 B13"; $resn_exch{"H353 B13"}="3H35 B13"; $resn_exch{"H301 B13"}="1H30 B13";
$resn_exch{"H302 B13"}="2H30 B13"; $resn_exch{"H311 B13"}="1H31 B13"; $resn_exch{"H312 B13"}="2H31 B13";
$resn_exch{"H331 B13"}="1H33 B13"; $resn_exch{"H332 B13"}="2H33 B13"; $resn_exch{"H261 B13"}="1H26 B13";
$resn_exch{"H262 B13"}="2H26 B13"; $resn_exch{"H291 B13"}="1H29 B13"; $resn_exch{"H292 B13"}="2H29 B13";
$resn_exch{"H251 B13"}="1H25 B13"; $resn_exch{"H252 B13"}="2H25 B13"; $resn_exch{"H253 B13"}="3H25 B13";
$resn_exch{"H201 B13"}="1H20 B13"; $resn_exch{"H202 B13"}="2H20 B13"; $resn_exch{"H203 B13"}="3H20 B13";
$resn_exch{"H531 B13"}="1H53 B13"; $resn_exch{"H532 B13"}="2H53 B13"; $resn_exch{"H533 B13"}="3H53 B13";
$resn_exch{"H481 B13"}="1H48 B13"; $resn_exch{"H482 B13"}="2H48 B13"; $resn_exch{"H491 B13"}="1H49 B13";
$resn_exch{"H492 B13"}="2H49 B13"; $resn_exch{"H521 B13"}="1H52 B13"; $resn_exch{"H522 B13"}="2H52 B13";
$resn_exch{"H461 B13"}="1H46 B13"; $resn_exch{"H462 B13"}="2H46 B13"; $resn_exch{"H463 B13"}="3H46 B13";
$resn_exch{"H471 B13"}="1H47 B13"; $resn_exch{"H472 B13"}="2H47 B13"; $resn_exch{"H473 B13"}="3H47 B13";
$resn_exch{"H411 B13"}="1H41 B13"; $resn_exch{"H412 B13"}="2H41 B13"; $resn_exch{"H421 B13"}="1H42 B13";
$resn_exch{"H422 B13"}="2H42 B13"; $resn_exch{"H451 B13"}="1H45 B13"; $resn_exch{"H452 B13"}="2H45 B13";
$resn_exch{"H361 B13"}="1H36 B13"; $resn_exch{"H362 B13"}="2H36 B13"; $resn_exch{"H363 B13"}="3H36 B13";
$resn_exch{"H371 B13"}="1H37 B13"; $resn_exch{"H372 B13"}="2H37 B13"; $resn_exch{"H401 B13"}="1H40 B13";
$resn_exch{"H402 B13"}="2H40 B13"; $resn_exch{" H71 B15"}="1H7  B15"; $resn_exch{" H72 B15"}="2H7  B15";
$resn_exch{" H81 B15"}="1H8  B15"; $resn_exch{" H82 B15"}="2H8  B15"; $resn_exch{"H171 B15"}="1H17 B15";
$resn_exch{"H172 B15"}="2H17 B15"; $resn_exch{"H191 B15"}="1H19 B15"; $resn_exch{"H192 B15"}="2H19 B15";
$resn_exch{"H211 B15"}="1H21 B15"; $resn_exch{"H212 B15"}="2H21 B15"; $resn_exch{"HN11 B15"}="1HN1 B15";
$resn_exch{"HN12 B15"}="2HN1 B15"; $resn_exch{" HN1 B17"}="1HN  B17"; $resn_exch{" HN2 B17"}="2HN  B17";
$resn_exch{" H91 B17"}="1H9  B17"; $resn_exch{" H92 B17"}="2H9  B17"; $resn_exch{"H101 B17"}="1H10 B17";
$resn_exch{"H102 B17"}="2H10 B17"; $resn_exch{"H121 B17"}="1H12 B17"; $resn_exch{"H122 B17"}="2H12 B17";
$resn_exch{"H131 B17"}="1H13 B17"; $resn_exch{"H132 B17"}="2H13 B17"; $resn_exch{"H152 B17"}="2H15 B17";
$resn_exch{"H153 B17"}="3H15 B17"; $resn_exch{" H81 B17"}="1H8  B17"; $resn_exch{" H82 B17"}="2H8  B17";
$resn_exch{"H111 B17"}="1H11 B17"; $resn_exch{"H112 B17"}="2H11 B17"; $resn_exch{"H161 B17"}="1H16 B17";
$resn_exch{"H162 B17"}="2H16 B17"; $resn_exch{"H111 B18"}="1H11 B18"; $resn_exch{"H112 B18"}="2H11 B18";
$resn_exch{"H261 B18"}="1H26 B18"; $resn_exch{"H262 B18"}="2H26 B18"; $resn_exch{"H271 B18"}="1H27 B18";
$resn_exch{"H272 B18"}="2H27 B18"; $resn_exch{"H291 B18"}="1H29 B18"; $resn_exch{"H292 B18"}="2H29 B18";
$resn_exch{"H293 B18"}="3H29 B18"; $resn_exch{"H301 B18"}="1H30 B18"; $resn_exch{"H302 B18"}="2H30 B18";
$resn_exch{"H303 B18"}="3H30 B18"; $resn_exch{"H311 B18"}="1H31 B18"; $resn_exch{"H312 B18"}="2H31 B18";
$resn_exch{"H313 B18"}="3H31 B18"; $resn_exch{"H111 B19"}="1H11 B19"; $resn_exch{"H112 B19"}="2H11 B19";
$resn_exch{"H113 B19"}="3H11 B19"; $resn_exch{"H121 B19"}="1H12 B19"; $resn_exch{"H122 B19"}="2H12 B19";
$resn_exch{"H123 B19"}="3H12 B19"; $resn_exch{" H81 B19"}="1H8  B19"; $resn_exch{" H82 B19"}="2H8  B19";
$resn_exch{" H83 B19"}="3H8  B19"; $resn_exch{" H91 B19"}="1H9  B19"; $resn_exch{" H92 B19"}="2H9  B19";
$resn_exch{" H93 B19"}="3H9  B19"; $resn_exch{" H61 B19"}="1H6  B19"; $resn_exch{" H62 B19"}="2H6  B19";
$resn_exch{" H11 B19"}="1H1  B19"; $resn_exch{" H12 B19"}="2H1  B19"; $resn_exch{"HN11 B19"}="1HN1 B19";
$resn_exch{"HN12 B19"}="2HN1 B19"; $resn_exch{" HN1 B1F"}="1HN  B1F"; $resn_exch{" HN2 B1F"}="2HN  B1F";
$resn_exch{" HB1 B1F"}="1HB  B1F"; $resn_exch{" HB2 B1F"}="2HB  B1F"; $resn_exch{"H101 B1L"}="1H10 B1L";
$resn_exch{"H102 B1L"}="2H10 B1L"; $resn_exch{"H111 B1L"}="1H11 B1L"; $resn_exch{"H112 B1L"}="2H11 B1L";
$resn_exch{"H121 B1L"}="1H12 B1L"; $resn_exch{"H122 B1L"}="2H12 B1L"; $resn_exch{"H141 B1L"}="1H14 B1L";
$resn_exch{"H142 B1L"}="2H14 B1L"; $resn_exch{"H201 B1M"}="1H20 B1M"; $resn_exch{"H202 B1M"}="2H20 B1M";
$resn_exch{"H203 B1M"}="3H20 B1M"; $resn_exch{"H251 B1M"}="1H25 B1M"; $resn_exch{"H252 B1M"}="2H25 B1M";
$resn_exch{"H253 B1M"}="3H25 B1M"; $resn_exch{"H261 B1M"}="1H26 B1M"; $resn_exch{"H262 B1M"}="2H26 B1M";
$resn_exch{"H291 B1M"}="1H29 B1M"; $resn_exch{"H292 B1M"}="2H29 B1M"; $resn_exch{"H301 B1M"}="1H30 B1M";
$resn_exch{"H302 B1M"}="2H30 B1M"; $resn_exch{"H311 B1M"}="1H31 B1M"; $resn_exch{"H312 B1M"}="2H31 B1M";
$resn_exch{"H331 B1M"}="1H33 B1M"; $resn_exch{"H332 B1M"}="2H33 B1M"; $resn_exch{"H351 B1M"}="1H35 B1M";
$resn_exch{"H352 B1M"}="2H35 B1M"; $resn_exch{"H353 B1M"}="3H35 B1M"; $resn_exch{"H361 B1M"}="1H36 B1M";
$resn_exch{"H362 B1M"}="2H36 B1M"; $resn_exch{"H363 B1M"}="3H36 B1M"; $resn_exch{"H371 B1M"}="1H37 B1M";
$resn_exch{"H372 B1M"}="2H37 B1M"; $resn_exch{"H401 B1M"}="1H40 B1M"; $resn_exch{"H402 B1M"}="2H40 B1M";
$resn_exch{"H411 B1M"}="1H41 B1M"; $resn_exch{"H412 B1M"}="2H41 B1M"; $resn_exch{"H421 B1M"}="1H42 B1M";
$resn_exch{"H422 B1M"}="2H42 B1M"; $resn_exch{"H451 B1M"}="1H45 B1M"; $resn_exch{"H452 B1M"}="2H45 B1M";
$resn_exch{"H461 B1M"}="1H46 B1M"; $resn_exch{"H462 B1M"}="2H46 B1M"; $resn_exch{"H463 B1M"}="3H46 B1M";
$resn_exch{"H471 B1M"}="1H47 B1M"; $resn_exch{"H472 B1M"}="2H47 B1M"; $resn_exch{"H473 B1M"}="3H47 B1M";
$resn_exch{"H481 B1M"}="1H48 B1M"; $resn_exch{"H482 B1M"}="2H48 B1M"; $resn_exch{"H491 B1M"}="1H49 B1M";
$resn_exch{"H492 B1M"}="2H49 B1M"; $resn_exch{"H521 B1M"}="1H52 B1M"; $resn_exch{"H522 B1M"}="2H52 B1M";
$resn_exch{"H531 B1M"}="1H53 B1M"; $resn_exch{"H532 B1M"}="2H53 B1M"; $resn_exch{"H533 B1M"}="3H53 B1M";
$resn_exch{"H541 B1M"}="1H54 B1M"; $resn_exch{"H542 B1M"}="2H54 B1M"; $resn_exch{"H543 B1M"}="3H54 B1M";
$resn_exch{"H551 B1M"}="1H55 B1M"; $resn_exch{"H552 B1M"}="2H55 B1M"; $resn_exch{"H561 B1M"}="1H56 B1M";
$resn_exch{"H562 B1M"}="2H56 B1M"; $resn_exch{"H601 B1M"}="1H60 B1M"; $resn_exch{"H602 B1M"}="2H60 B1M";
$resn_exch{"H621 B1M"}="1H62 B1M"; $resn_exch{"H622 B1M"}="2H62 B1M"; $resn_exch{"H1P1 B1M"}="1H1P B1M";
$resn_exch{"H1P2 B1M"}="2H1P B1M"; $resn_exch{"H3P1 B1M"}="1H3P B1M"; $resn_exch{"H3P2 B1M"}="2H3P B1M";
$resn_exch{"H3P3 B1M"}="3H3P B1M"; $resn_exch{"HOP5 B1M"}="5HOP B1M"; $resn_exch{"HOR7 B1M"}="7HOR B1M";
$resn_exch{"H5R1 B1M"}="1H5R B1M"; $resn_exch{"H5R2 B1M"}="2H5R B1M"; $resn_exch{"HOR8 B1M"}="8HOR B1M";
$resn_exch{"HO51 B1M"}="1HO5 B1M"; $resn_exch{"HO52 B1M"}="2HO5 B1M"; $resn_exch{"HO53 B1M"}="3HO5 B1M";
$resn_exch{"HOP2 B1P"}="2HOP B1P"; $resn_exch{"HOP3 B1P"}="3HOP B1P"; $resn_exch{" H5' B1P"}="1H5* B1P";
$resn_exch{"H5'' B1P"}="2H5* B1P"; $resn_exch{" H4' B1P"}=" H4* B1P"; $resn_exch{" H1' B1P"}=" H1* B1P";
$resn_exch{"HO1' B1P"}="HO1* B1P"; $resn_exch{" H2' B1P"}="1H2* B1P"; $resn_exch{"H2'' B1P"}="2H2* B1P";
$resn_exch{" H3' B1P"}=" H3* B1P"; $resn_exch{"HO3' B1P"}="H3T  B1P"; $resn_exch{"H151 B1V"}="1H15 B1V";
$resn_exch{"H152 B1V"}="2H15 B1V"; $resn_exch{"H181 B1V"}="1H18 B1V"; $resn_exch{"H182 B1V"}="2H18 B1V";
$resn_exch{"H191 B1V"}="1H19 B1V"; $resn_exch{"H192 B1V"}="2H19 B1V"; $resn_exch{"H201 B1V"}="1H20 B1V";
$resn_exch{"H202 B1V"}="2H20 B1V"; $resn_exch{"H211 B1V"}="1H21 B1V"; $resn_exch{"H212 B1V"}="2H21 B1V";
$resn_exch{"H221 B1V"}="1H22 B1V"; $resn_exch{"H222 B1V"}="2H22 B1V"; $resn_exch{"HAL1 B21"}="1HAL B21";
$resn_exch{"HAL2 B21"}="2HAL B21"; $resn_exch{"HAL3 B21"}="3HAL B21"; $resn_exch{"HN12 B22"}="2HN1 B22";
$resn_exch{"HN11 B22"}="1HN1 B22"; $resn_exch{" H71 B22"}="1H7  B22"; $resn_exch{" H72 B22"}="2H7  B22";
$resn_exch{" H81 B22"}="1H8  B22"; $resn_exch{" H82 B22"}="2H8  B22"; $resn_exch{"H191 B22"}="1H19 B22";
$resn_exch{"H192 B22"}="2H19 B22"; $resn_exch{"H212 B22"}="2H21 B22"; $resn_exch{"H211 B22"}="1H21 B22";
$resn_exch{"H231 B22"}="1H23 B22"; $resn_exch{"H232 B22"}="2H23 B22"; $resn_exch{"HAU1 B28"}="1HAU B28";
$resn_exch{"HAU2 B28"}="2HAU B28"; $resn_exch{"HAS1 B29"}="1HAS B29"; $resn_exch{"HAS2 B29"}="2HAS B29";
$resn_exch{" H   B2A"}="1HN  B2A"; $resn_exch{" HN2 B2A"}="2HN  B2A"; $resn_exch{" HB1 B2A"}="1HB  B2A";
$resn_exch{" HB2 B2A"}="2HB  B2A"; $resn_exch{" HB3 B2A"}="3HB  B2A"; $resn_exch{" HN1 B2F"}="1HN  B2F";
$resn_exch{" HN2 B2F"}="2HN  B2F"; $resn_exch{" HB1 B2F"}="1HB  B2F"; $resn_exch{" HB2 B2F"}="2HB  B2F";
$resn_exch{"H6A1 B2G"}="1H6A B2G"; $resn_exch{"H6A2 B2G"}="2H6A B2G"; $resn_exch{"H6B1 B2G"}="1H6B B2G";
$resn_exch{"H6B2 B2G"}="2H6B B2G"; $resn_exch{" HN1 B2I"}="1HN  B2I"; $resn_exch{" HN2 B2I"}="2HN  B2I";
$resn_exch{"HG11 B2I"}="1HG1 B2I"; $resn_exch{"HG12 B2I"}="2HG1 B2I"; $resn_exch{"HG21 B2I"}="1HG2 B2I";
$resn_exch{"HG22 B2I"}="2HG2 B2I"; $resn_exch{"HG23 B2I"}="3HG2 B2I"; $resn_exch{"HD11 B2I"}="1HD1 B2I";
$resn_exch{"HD12 B2I"}="2HD1 B2I"; $resn_exch{"HD13 B2I"}="3HD1 B2I"; $resn_exch{" HN1 B2V"}="1HN  B2V";
$resn_exch{" HN2 B2V"}="2HN  B2V"; $resn_exch{"HG11 B2V"}="1HG1 B2V"; $resn_exch{"HG12 B2V"}="2HG1 B2V";
$resn_exch{"HG13 B2V"}="3HG1 B2V"; $resn_exch{"HG21 B2V"}="1HG2 B2V"; $resn_exch{"HG22 B2V"}="2HG2 B2V";
$resn_exch{"HG23 B2V"}="3HG2 B2V"; $resn_exch{"HN11 B30"}="1HN1 B30"; $resn_exch{"HN12 B30"}="2HN1 B30";
$resn_exch{" H91 B30"}="1H9  B30"; $resn_exch{" H92 B30"}="2H9  B30"; $resn_exch{"H101 B30"}="1H10 B30";
$resn_exch{"H102 B30"}="2H10 B30"; $resn_exch{"H121 B30"}="1H12 B30"; $resn_exch{"H122 B30"}="2H12 B30";
$resn_exch{"H141 B30"}="1H14 B30"; $resn_exch{"H142 B30"}="2H14 B30"; $resn_exch{"H101 B33"}="1H10 B33";
$resn_exch{"H102 B33"}="2H10 B33"; $resn_exch{"H121 B33"}="1H12 B33"; $resn_exch{"H122 B33"}="2H12 B33";
$resn_exch{"H131 B33"}="1H13 B33"; $resn_exch{"H132 B33"}="2H13 B33"; $resn_exch{" H11 B33"}="1H1  B33";
$resn_exch{" H12 B33"}="2H1  B33"; $resn_exch{" H13 B33"}="3H1  B33"; $resn_exch{" H21 B33"}="1H2  B33";
$resn_exch{" H22 B33"}="2H2  B33"; $resn_exch{" H41 B33"}="1H4  B33"; $resn_exch{" H42 B33"}="2H4  B33";
$resn_exch{" H51 B33"}="1H5  B33"; $resn_exch{" H52 B33"}="2H5  B33"; $resn_exch{" H61 B33"}="1H6  B33";
$resn_exch{" H62 B33"}="2H6  B33"; $resn_exch{" H81 B33"}="1H8  B33"; $resn_exch{" H82 B33"}="2H8  B33";
$resn_exch{" H91 B33"}="1H9  B33"; $resn_exch{" H92 B33"}="2H9  B33"; $resn_exch{"H141 B33"}="1H14 B33";
$resn_exch{"H142 B33"}="2H14 B33"; $resn_exch{"H143 B33"}="3H14 B33"; $resn_exch{" HG1 B3A"}="1HG  B3A";
$resn_exch{" HG2 B3A"}="2HG  B3A"; $resn_exch{" HG3 B3A"}="3HG  B3A"; $resn_exch{" H   B3A"}="1HN  B3A";
$resn_exch{" H2  B3A"}="2HN  B3A"; $resn_exch{" HA1 B3A"}="1HA  B3A"; $resn_exch{" HA2 B3A"}="2HA  B3A";
$resn_exch{" HE1 B3D"}="HOE1 B3D"; $resn_exch{" HG3 B3D"}="1HG  B3D"; $resn_exch{" HG2 B3D"}="2HG  B3D";
$resn_exch{" H   B3D"}="1HN  B3D"; $resn_exch{" H2  B3D"}="2HN  B3D"; $resn_exch{" HA1 B3D"}="1HA  B3D";
$resn_exch{" HA2 B3D"}="2HA  B3D"; $resn_exch{" H   B3E"}="1HN  B3E"; $resn_exch{" H2  B3E"}="2HN  B3E";
$resn_exch{" HG2 B3E"}="1HG  B3E"; $resn_exch{" HG3 B3E"}="2HG  B3E"; $resn_exch{" HD2 B3E"}="1HD  B3E";
$resn_exch{" HD3 B3E"}="2HD  B3E"; $resn_exch{" HA1 B3E"}="1HA  B3E"; $resn_exch{" HA2 B3E"}="2HA  B3E";
$resn_exch{"H121 B3H"}="1H12 B3H"; $resn_exch{"H122 B3H"}="2H12 B3H"; $resn_exch{"H131 B3H"}="1H13 B3H";
$resn_exch{"H132 B3H"}="2H13 B3H"; $resn_exch{"H141 B3H"}="1H14 B3H"; $resn_exch{"H142 B3H"}="2H14 B3H";
$resn_exch{"H151 B3H"}="1H15 B3H"; $resn_exch{"H152 B3H"}="2H15 B3H"; $resn_exch{"H161 B3H"}="1H16 B3H";
$resn_exch{"H162 B3H"}="2H16 B3H"; $resn_exch{"H171 B3H"}="1H17 B3H"; $resn_exch{"H172 B3H"}="2H17 B3H";
$resn_exch{"H181 B3H"}="1H18 B3H"; $resn_exch{"H182 B3H"}="2H18 B3H"; $resn_exch{"H191 B3H"}="1H19 B3H";
$resn_exch{"H192 B3H"}="2H19 B3H"; $resn_exch{"H193 B3H"}="3H19 B3H"; $resn_exch{"H221 B3H"}="1H22 B3H";
$resn_exch{"H222 B3H"}="2H22 B3H"; $resn_exch{"H231 B3H"}="1H23 B3H"; $resn_exch{"H232 B3H"}="2H23 B3H";
$resn_exch{"H241 B3H"}="1H24 B3H"; $resn_exch{"H242 B3H"}="2H24 B3H"; $resn_exch{"H243 B3H"}="3H24 B3H";
$resn_exch{"H411 B3H"}="1H41 B3H"; $resn_exch{"H412 B3H"}="2H41 B3H"; $resn_exch{"H431 B3H"}="1H43 B3H";
$resn_exch{"H432 B3H"}="2H43 B3H"; $resn_exch{" H   B3K"}="1HN  B3K"; $resn_exch{" H2  B3K"}="2HN  B3K";
$resn_exch{" HG2 B3K"}="1HG  B3K"; $resn_exch{" HG3 B3K"}="2HG  B3K"; $resn_exch{" HD2 B3K"}="1HD  B3K";
$resn_exch{" HD3 B3K"}="2HD  B3K"; $resn_exch{" HE2 B3K"}="1HE  B3K"; $resn_exch{" HE3 B3K"}="2HE  B3K";
$resn_exch{" HF1 B3K"}="1HF  B3K"; $resn_exch{" HF2 B3K"}="2HF  B3K"; $resn_exch{"HNZ1 B3K"}="1HNZ B3K";
$resn_exch{"HNZ2 B3K"}="2HNZ B3K"; $resn_exch{" HA1 B3K"}="1HA  B3K"; $resn_exch{" HA2 B3K"}="2HA  B3K";
$resn_exch{"H221 B3N"}="1H22 B3N"; $resn_exch{"H222 B3N"}="2H22 B3N"; $resn_exch{"H223 B3N"}="3H22 B3N";
$resn_exch{"H211 B3N"}="1H21 B3N"; $resn_exch{"H212 B3N"}="2H21 B3N"; $resn_exch{"H213 B3N"}="3H21 B3N";
$resn_exch{"H101 B3N"}="1H10 B3N"; $resn_exch{"H102 B3N"}="2H10 B3N"; $resn_exch{" H91 B3N"}="1H9  B3N";
$resn_exch{" H92 B3N"}="2H9  B3N"; $resn_exch{" H81 B3N"}="1H8  B3N"; $resn_exch{" H82 B3N"}="2H8  B3N";
$resn_exch{" H71 B3N"}="1H7  B3N"; $resn_exch{" H72 B3N"}="2H7  B3N"; $resn_exch{" H61 B3N"}="1H6  B3N";
$resn_exch{" H62 B3N"}="2H6  B3N"; $resn_exch{" H51 B3N"}="1H5  B3N"; $resn_exch{" H52 B3N"}="2H5  B3N";
$resn_exch{" H11 B3P"}="1H1  B3P"; $resn_exch{" H12 B3P"}="2H1  B3P"; $resn_exch{" H21 B3P"}="1H2  B3P";
$resn_exch{" H22 B3P"}="2H2  B3P"; $resn_exch{" H31 B3P"}="1H3  B3P"; $resn_exch{" H32 B3P"}="2H3  B3P";
$resn_exch{" H51 B3P"}="1H5  B3P"; $resn_exch{" H52 B3P"}="2H5  B3P"; $resn_exch{" H61 B3P"}="1H6  B3P";
$resn_exch{" H62 B3P"}="2H6  B3P"; $resn_exch{" H71 B3P"}="1H7  B3P"; $resn_exch{" H72 B3P"}="2H7  B3P";
$resn_exch{" H91 B3P"}="1H9  B3P"; $resn_exch{" H92 B3P"}="2H9  B3P"; $resn_exch{"H101 B3P"}="1H10 B3P";
$resn_exch{"H102 B3P"}="2H10 B3P"; $resn_exch{"H111 B3P"}="1H11 B3P"; $resn_exch{"H112 B3P"}="2H11 B3P";
$resn_exch{" H   B3S"}="1HN  B3S"; $resn_exch{" H2  B3S"}="2HN  B3S"; $resn_exch{" HD  B3S"}=" HOD B3S";
$resn_exch{" HG3 B3S"}="1HG  B3S"; $resn_exch{" HG2 B3S"}="2HG  B3S"; $resn_exch{" HA1 B3S"}="1HA  B3S";
$resn_exch{" HA2 B3S"}="2HA  B3S"; $resn_exch{"HE21 B3X"}="1HE2 B3X"; $resn_exch{"HE22 B3X"}="2HE2 B3X";
$resn_exch{" HG3 B3X"}="1HG  B3X"; $resn_exch{" HG2 B3X"}="2HG  B3X"; $resn_exch{" H   B3X"}="1HN  B3X";
$resn_exch{" H2  B3X"}="2HN  B3X"; $resn_exch{" HA1 B3X"}="1HA  B3X"; $resn_exch{" HA2 B3X"}="2HA  B3X";
$resn_exch{" HA1 B3Y"}="1HA  B3Y"; $resn_exch{" HA2 B3Y"}="2HA  B3Y"; $resn_exch{" H   B3Y"}="1HN  B3Y";
$resn_exch{" H2  B3Y"}="2HN  B3Y"; $resn_exch{" HG2 B3Y"}="1HG  B3Y"; $resn_exch{" HG3 B3Y"}="2HG  B3Y";
$resn_exch{"HJ61 B4G"}="1HJ6 B4G"; $resn_exch{"HJ62 B4G"}="2HJ6 B4G"; $resn_exch{"HQ61 B4G"}="1HQ6 B4G";
$resn_exch{"HQ62 B4G"}="2HQ6 B4G"; $resn_exch{"H6A1 B4G"}="1H6A B4G"; $resn_exch{"HOA2 B4P"}="2HOA B4P";
$resn_exch{"HOB2 B4P"}="2HOB B4P"; $resn_exch{"HOG2 B4P"}="2HOG B4P"; $resn_exch{"HOD2 B4P"}="2HOD B4P";
$resn_exch{"H51A B4P"}="AH51 B4P"; $resn_exch{"H52A B4P"}="AH52 B4P"; $resn_exch{" H4E B4P"}="AH4* B4P";
$resn_exch{" H3E B4P"}="AH3* B4P"; $resn_exch{"HO3A B4P"}="AHO3 B4P"; $resn_exch{" H2E B4P"}="AH2* B4P";
$resn_exch{"HO2A B4P"}="AHO2 B4P"; $resn_exch{" H1E B4P"}="AH1* B4P"; $resn_exch{" H8A B4P"}=" AH8 B4P";
$resn_exch{"H61A B4P"}="AH61 B4P"; $resn_exch{"H62A B4P"}="AH62 B4P"; $resn_exch{" H2A B4P"}=" AH2 B4P";
$resn_exch{"H51B B4P"}="BH51 B4P"; $resn_exch{"H52B B4P"}="BH52 B4P"; $resn_exch{" H4F B4P"}="BH4* B4P";
$resn_exch{" H3F B4P"}="BH3* B4P"; $resn_exch{"HO3B B4P"}="BHO3 B4P"; $resn_exch{" H2F B4P"}="BH2* B4P";
$resn_exch{"HO2B B4P"}="BHO2 B4P"; $resn_exch{" H1F B4P"}="BH1* B4P"; $resn_exch{" H8B B4P"}=" BH8 B4P";
$resn_exch{"H61B B4P"}="BH61 B4P"; $resn_exch{"H62B B4P"}="BH62 B4P"; $resn_exch{" H2B B4P"}=" BH2 B4P";
$resn_exch{"HAY1 B75"}="1HAY B75"; $resn_exch{"HAY2 B75"}="2HAY B75"; $resn_exch{"HAU1 B76"}="1HAU B76";
$resn_exch{"HAU2 B76"}="2HAU B76"; $resn_exch{" H51 B7G"}="1H5  B7G"; $resn_exch{" H11 B7G"}="1H1  B7G";
$resn_exch{" H21 B7G"}="1H2  B7G"; $resn_exch{" H31 B7G"}="1H3  B7G"; $resn_exch{" H41 B7G"}="1H4  B7G";
$resn_exch{" H61 B7G"}="1H6  B7G"; $resn_exch{" H62 B7G"}="2H6  B7G"; $resn_exch{" H71 B7G"}="1H7  B7G";
$resn_exch{" H72 B7G"}="2H7  B7G"; $resn_exch{" H81 B7G"}="1H8  B7G"; $resn_exch{" H82 B7G"}="2H8  B7G";
$resn_exch{" H91 B7G"}="1H9  B7G"; $resn_exch{" H92 B7G"}="2H9  B7G"; $resn_exch{"H101 B7G"}="1H10 B7G";
$resn_exch{"H102 B7G"}="2H10 B7G"; $resn_exch{"H111 B7G"}="1H11 B7G"; $resn_exch{"H112 B7G"}="2H11 B7G";
$resn_exch{"H121 B7G"}="1H12 B7G"; $resn_exch{"H122 B7G"}="2H12 B7G"; $resn_exch{"H131 B7G"}="1H13 B7G";
$resn_exch{"H132 B7G"}="2H13 B7G"; $resn_exch{"H133 B7G"}="3H13 B7G"; $resn_exch{"H101 B8L"}="1H10 B8L";
$resn_exch{"H102 B8L"}="2H10 B8L"; $resn_exch{"H111 B8L"}="1H11 B8L"; $resn_exch{"H112 B8L"}="2H11 B8L";
$resn_exch{"H121 B8L"}="1H12 B8L"; $resn_exch{"H122 B8L"}="2H12 B8L"; $resn_exch{"H141 B8L"}="1H14 B8L";
$resn_exch{"H142 B8L"}="2H14 B8L"; $resn_exch{"H711 B8L"}="1H71 B8L"; $resn_exch{"H712 B8L"}="2H71 B8L";
$resn_exch{"H721 B8L"}="1H72 B8L"; $resn_exch{"H722 B8L"}="2H72 B8L"; $resn_exch{"H723 B8L"}="3H72 B8L";
$resn_exch{"H811 B8L"}="1H81 B8L"; $resn_exch{"H812 B8L"}="2H81 B8L"; $resn_exch{"H813 B8L"}="3H81 B8L";
$resn_exch{"H301 B8L"}="1H30 B8L"; $resn_exch{"H302 B8L"}="2H30 B8L"; $resn_exch{"H303 B8L"}="3H30 B8L";
$resn_exch{"H171 B96"}="1H17 B96"; $resn_exch{"H172 B96"}="2H17 B96"; $resn_exch{"H173 B96"}="3H17 B96";
$resn_exch{"H181 B96"}="1H18 B96"; $resn_exch{"H182 B96"}="2H18 B96"; $resn_exch{"H183 B96"}="3H18 B96";
$resn_exch{"H191 B96"}="1H19 B96"; $resn_exch{"H192 B96"}="2H19 B96"; $resn_exch{"H193 B96"}="3H19 B96";
$resn_exch{"H251 B96"}="1H25 B96"; $resn_exch{"H252 B96"}="2H25 B96"; $resn_exch{"H253 B96"}="3H25 B96";
$resn_exch{"H421 B96"}="1H42 B96"; $resn_exch{"H422 B96"}="2H42 B96"; $resn_exch{"H431 B96"}="1H43 B96";
$resn_exch{"H432 B96"}="2H43 B96"; $resn_exch{"H451 B96"}="1H45 B96"; $resn_exch{"H452 B96"}="2H45 B96";
$resn_exch{"H461 B96"}="1H46 B96"; $resn_exch{"H462 B96"}="2H46 B96"; $resn_exch{"H481 B96"}="1H48 B96";
$resn_exch{"H482 B96"}="2H48 B96"; $resn_exch{"H491 B96"}="1H49 B96"; $resn_exch{"H492 B96"}="2H49 B96";
$resn_exch{" H1A B9A"}=" AH1 B9A"; $resn_exch{" H2A B9A"}=" AH2 B9A"; $resn_exch{" H3A B9A"}=" AH3 B9A";
$resn_exch{" H5A B9A"}=" AH5 B9A"; $resn_exch{" H6A B9A"}=" AH6 B9A"; $resn_exch{" H7A B9A"}=" AH7 B9A";
$resn_exch{" H8A B9A"}=" AH8 B9A"; $resn_exch{" H9A B9A"}=" AH9 B9A"; $resn_exch{"H151 B9A"}="1H15 B9A";
$resn_exch{"H152 B9A"}="2H15 B9A"; $resn_exch{"H161 B9A"}="1H16 B9A"; $resn_exch{"H162 B9A"}="2H16 B9A";
$resn_exch{"H171 B9A"}="1H17 B9A"; $resn_exch{"H172 B9A"}="2H17 B9A"; $resn_exch{"H191 B9A"}="1H19 B9A";
$resn_exch{"H192 B9A"}="2H19 B9A"; $resn_exch{"H201 B9A"}="1H20 B9A"; $resn_exch{"H202 B9A"}="2H20 B9A";
$resn_exch{"H211 B9A"}="1H21 B9A"; $resn_exch{"H212 B9A"}="2H21 B9A"; $resn_exch{"H221 B9A"}="1H22 B9A";
$resn_exch{"H222 B9A"}="2H22 B9A"; $resn_exch{"H241 B9A"}="1H24 B9A"; $resn_exch{"H242 B9A"}="2H24 B9A";
$resn_exch{"H251 B9A"}="1H25 B9A"; $resn_exch{"H252 B9A"}="2H25 B9A"; $resn_exch{"H261 B9A"}="1H26 B9A";
$resn_exch{"H262 B9A"}="2H26 B9A"; $resn_exch{"H281 B9A"}="1H28 B9A"; $resn_exch{"H282 B9A"}="2H28 B9A";
$resn_exch{"H301 B9A"}="1H30 B9A"; $resn_exch{"H302 B9A"}="2H30 B9A"; $resn_exch{"H311 B9A"}="1H31 B9A";
$resn_exch{"H312 B9A"}="2H31 B9A"; $resn_exch{"H351 B9A"}="1H35 B9A"; $resn_exch{"H352 B9A"}="2H35 B9A";
$resn_exch{"H361 B9A"}="1H36 B9A"; $resn_exch{"H362 B9A"}="2H36 B9A"; $resn_exch{"H401 B9A"}="1H40 B9A";
$resn_exch{"H402 B9A"}="2H40 B9A"; $resn_exch{"H411 B9A"}="1H41 B9A"; $resn_exch{"H412 B9A"}="2H41 B9A";
$resn_exch{"H431 B9A"}="1H43 B9A"; $resn_exch{"H432 B9A"}="2H43 B9A"; $resn_exch{" H1B B9A"}=" BH1 B9A";
$resn_exch{" H2B B9A"}=" BH2 B9A"; $resn_exch{" H3B B9A"}=" BH3 B9A"; $resn_exch{" H4B B9A"}=" BH4 B9A";
$resn_exch{" H6B B9A"}=" BH6 B9A"; $resn_exch{" H7B B9A"}=" BH7 B9A"; $resn_exch{" H8B B9A"}=" BH8 B9A";
$resn_exch{" H9B B9A"}=" BH9 B9A"; $resn_exch{"HO5' BA1"}="'HO5 BA1"; $resn_exch{"HN1' BA1"}="'HN1 BA1";
$resn_exch{" H21 BA1"}="1H2  BA1"; $resn_exch{" H22 BA1"}="2H2  BA1"; $resn_exch{" H51 BA1"}="1H5  BA1";
$resn_exch{" H52 BA1"}="2H5  BA1"; $resn_exch{" H61 BA1"}="1H6  BA1"; $resn_exch{" H62 BA1"}="2H6  BA1";
$resn_exch{" H71 BA1"}="1H7  BA1"; $resn_exch{" H72 BA1"}="2H7  BA1"; $resn_exch{" H3B BA1"}=" H3\" BA1";
$resn_exch{"HO4' BA1"}="\"HO4 BA1"; $resn_exch{"HO6' BA1"}="\"HO6 BA1"; $resn_exch{" H7B BA1"}=" H7\" BA1";
$resn_exch{"H2OA BA1"}="AH2O BA1"; $resn_exch{"H112 BA1"}="2H11 BA1"; $resn_exch{"H122 BA1"}="2H12 BA1";
$resn_exch{"H132 BA1"}="2H13 BA1"; $resn_exch{"H2OF BA1"}="FH2O BA1"; $resn_exch{"H271 BA2"}="1H27 BA2";
$resn_exch{"H272 BA2"}="2H27 BA2"; $resn_exch{"H241 BA2"}="1H24 BA2"; $resn_exch{"H242 BA2"}="2H24 BA2";
$resn_exch{"H231 BA2"}="1H23 BA2"; $resn_exch{"H232 BA2"}="2H23 BA2"; $resn_exch{"H221 BA2"}="1H22 BA2";
$resn_exch{"H222 BA2"}="2H22 BA2"; $resn_exch{"H211 BA2"}="1H21 BA2"; $resn_exch{"H212 BA2"}="2H21 BA2";
$resn_exch{"H171 BA2"}="1H17 BA2"; $resn_exch{"H172 BA2"}="2H17 BA2"; $resn_exch{"H161 BA2"}="1H16 BA2";
$resn_exch{"H162 BA2"}="2H16 BA2"; $resn_exch{"H151 BA2"}="1H15 BA2"; $resn_exch{"H152 BA2"}="2H15 BA2";
$resn_exch{"H141 BA2"}="1H14 BA2"; $resn_exch{"H142 BA2"}="2H14 BA2"; $resn_exch{" H1' BA2"}=" H1* BA2";
$resn_exch{"H6N1 BA2"}="1H6N BA2"; $resn_exch{"H6N2 BA2"}="2H6N BA2"; $resn_exch{" H2' BA2"}=" H2* BA2";
$resn_exch{" H3' BA2"}=" H3* BA2"; $resn_exch{" H4' BA2"}=" H4* BA2"; $resn_exch{"H5'1 BA2"}="1H5* BA2";
$resn_exch{"H5'2 BA2"}="2H5* BA2"; $resn_exch{"H111 BA2"}="1H11 BA2"; $resn_exch{"H112 BA2"}="2H11 BA2";
$resn_exch{"H121 BA2"}="1H12 BA2"; $resn_exch{"H122 BA2"}="2H12 BA2"; $resn_exch{"H123 BA2"}="3H12 BA2";
$resn_exch{"H6A1 BA3"}="1H6A BA3"; $resn_exch{"H6A2 BA3"}="2H6A BA3"; $resn_exch{"H5B1 BA3"}="1H5B BA3";
$resn_exch{"H5B2 BA3"}="2H5B BA3"; $resn_exch{"H5D1 BA3"}="1H5D BA3"; $resn_exch{"H5D2 BA3"}="2H5D BA3";
$resn_exch{"H6G1 BA3"}="1H6G BA3"; $resn_exch{"H6G2 BA3"}="2H6G BA3"; $resn_exch{"HN31 BAA"}="1HN3 BAA";
$resn_exch{"HN32 BAA"}="2HN3 BAA"; $resn_exch{"HB11 BAA"}="1HB1 BAA"; $resn_exch{"HB12 BAA"}="2HB1 BAA";
$resn_exch{"HB13 BAA"}="3HB1 BAA"; $resn_exch{"HB21 BAA"}="1HB2 BAA"; $resn_exch{"HB22 BAA"}="2HB2 BAA";
$resn_exch{"HB23 BAA"}="3HB2 BAA"; $resn_exch{"H411 BAA"}="1H41 BAA"; $resn_exch{"H412 BAA"}="2H41 BAA";
$resn_exch{"H413 BAA"}="3H41 BAA"; $resn_exch{"H421 BAA"}="1H42 BAA"; $resn_exch{"H422 BAA"}="2H42 BAA";
$resn_exch{"H423 BAA"}="3H42 BAA"; $resn_exch{"H431 BAA"}="1H43 BAA"; $resn_exch{"H432 BAA"}="2H43 BAA";
$resn_exch{"H433 BAA"}="3H43 BAA"; $resn_exch{" H91 BAB"}="1H9  BAB"; $resn_exch{" H92 BAB"}="2H9  BAB";
$resn_exch{"HN21 BAB"}="1HN2 BAB"; $resn_exch{"HN22 BAB"}="2HN2 BAB"; $resn_exch{"H2'1 BAB"}="1H2' BAB";
$resn_exch{"H2'2 BAB"}="2H2' BAB"; $resn_exch{"HN4' BAB"}="'HN4 BAB"; $resn_exch{" HN1 BAB"}="1H1' BAB";
$resn_exch{" HN2 BAB"}="2H1' BAB"; $resn_exch{" HN3 BAB"}="'HN3 BAB"; $resn_exch{" HN5 BAB"}="1HN1 BAB";
$resn_exch{" HN6 BAB"}="2HN1 BAB"; $resn_exch{" H31 BAC"}="1H3  BAC"; $resn_exch{" H32 BAC"}="2H3  BAC";
$resn_exch{"HB11 BAF"}="1HB1 BAF"; $resn_exch{"HB12 BAF"}="2HB1 BAF"; $resn_exch{"HB13 BAF"}="3HB1 BAF";
$resn_exch{"HB21 BAF"}="1HB2 BAF"; $resn_exch{"HB22 BAF"}="2HB2 BAF"; $resn_exch{"HB23 BAF"}="3HB2 BAF";
$resn_exch{"H411 BAF"}="1H41 BAF"; $resn_exch{"H412 BAF"}="2H41 BAF"; $resn_exch{"H413 BAF"}="3H41 BAF";
$resn_exch{"H421 BAF"}="1H42 BAF"; $resn_exch{"H422 BAF"}="2H42 BAF"; $resn_exch{"H423 BAF"}="3H42 BAF";
$resn_exch{"H431 BAF"}="1H43 BAF"; $resn_exch{"H432 BAF"}="2H43 BAF"; $resn_exch{"H433 BAF"}="3H43 BAF";
$resn_exch{"H181 BAG"}="1H18 BAG"; $resn_exch{"H182 BAG"}="2H18 BAG"; $resn_exch{" H31 BAG"}="1H3  BAG";
$resn_exch{" H32 BAG"}="2H3  BAG"; $resn_exch{" H41 BAG"}="1H4  BAG"; $resn_exch{" H42 BAG"}="2H4  BAG";
$resn_exch{" HD1 BAG"}="1HD  BAG"; $resn_exch{" HD2 BAG"}="2HD  BAG"; $resn_exch{"HH21 BAG"}="1HH2 BAG";
$resn_exch{"HH22 BAG"}="2HH2 BAG"; $resn_exch{"HN11 BAH"}="1HN1 BAH"; $resn_exch{"HN12 BAH"}="2HN1 BAH";
$resn_exch{"HN21 BAH"}="1HN2 BAH"; $resn_exch{"HN22 BAH"}="2HN2 BAH"; $resn_exch{"HC2' BAH"}="'HC2 BAH";
$resn_exch{"HC3' BAH"}="'HC3 BAH"; $resn_exch{"HC6' BAH"}="'HC6 BAH"; $resn_exch{"H1'1 BAH"}="1H1' BAH";
$resn_exch{"H1'2 BAH"}="2H1' BAH"; $resn_exch{"H2'1 BAH"}="1H2' BAH"; $resn_exch{"H2'2 BAH"}="2H2' BAH";
$resn_exch{"HN4' BAH"}="'HN4 BAH"; $resn_exch{" H91 BAI"}="1H9  BAI"; $resn_exch{" H92 BAI"}="2H9  BAI";
$resn_exch{"HH11 BAI"}="1HH1 BAI"; $resn_exch{"HH12 BAI"}="2HH1 BAI"; $resn_exch{"HH21 BAI"}="1HH2 BAI";
$resn_exch{"HN3' BAI"}="'HN3 BAI"; $resn_exch{"HH11 BAK"}="1HH1 BAK"; $resn_exch{"HH12 BAK"}="2HH1 BAK";
$resn_exch{"HH21 BAK"}="1HH2 BAK"; $resn_exch{"HH22 BAK"}="2HH2 BAK"; $resn_exch{"H1'1 BAK"}="1H1' BAK";
$resn_exch{"H1'2 BAK"}="2H1' BAK"; $resn_exch{"H2'1 BAK"}="1H2' BAK"; $resn_exch{"H2'2 BAK"}="2H2' BAK";
$resn_exch{"HN4' BAK"}="'HN4 BAK"; $resn_exch{" H   BAL"}="1HN  BAL"; $resn_exch{" H2  BAL"}="2HN  BAL";
$resn_exch{" HB3 BAL"}="1HB  BAL"; $resn_exch{" HB2 BAL"}="2HB  BAL"; $resn_exch{" HA1 BAL"}="1HA  BAL";
$resn_exch{" HA2 BAL"}="2HA  BAL"; $resn_exch{"HC21 BAM"}="1HC2 BAM"; $resn_exch{"HC31 BAM"}="1HC3 BAM";
$resn_exch{"HC41 BAM"}="1HC4 BAM"; $resn_exch{"HC51 BAM"}="1HC5 BAM"; $resn_exch{"HC61 BAM"}="1HC6 BAM";
$resn_exch{"HN11 BAM"}="1HN1 BAM"; $resn_exch{"HN12 BAM"}="2HN1 BAM"; $resn_exch{"HN21 BAM"}="1HN2 BAM";
$resn_exch{"HN22 BAM"}="2HN2 BAM"; $resn_exch{" H6  BAN"}="6H   BAN"; $resn_exch{" H7  BAN"}="7H   BAN";
$resn_exch{" H23 BAN"}="3H2  BAN"; $resn_exch{" H24 BAN"}="4H2  BAN"; $resn_exch{" H25 BAN"}="5H2  BAN";
$resn_exch{" H29 BAN"}="9H2  BAN"; $resn_exch{" H39 BAN"}="9H3  BAN"; $resn_exch{"HN21 BAO"}="1HN2 BAO";
$resn_exch{"HN22 BAO"}="2HN2 BAO"; $resn_exch{"H2'1 BAO"}="1H2' BAO"; $resn_exch{"H2'2 BAO"}="2H2' BAO";
$resn_exch{"HN4' BAO"}="'HN4 BAO"; $resn_exch{"H4'1 BAP"}="1H4' BAP"; $resn_exch{"H4'2 BAP"}="2H4' BAP";
$resn_exch{" H31 BAR"}="1H3  BAR"; $resn_exch{" H32 BAR"}="2H3  BAR"; $resn_exch{" H61 BAR"}="1H6  BAR";
$resn_exch{" H62 BAR"}="2H6  BAR"; $resn_exch{"HC11 BAS"}="1HC1 BAS"; $resn_exch{"H311 BAS"}="1H31 BAS";
$resn_exch{"H312 BAS"}="2H31 BAS"; $resn_exch{"H321 BAS"}="1H32 BAS"; $resn_exch{"H322 BAS"}="2H32 BAS";
$resn_exch{"H323 BAS"}="3H32 BAS"; $resn_exch{" H31 BAT"}="1H3  BAT"; $resn_exch{" H32 BAT"}="2H3  BAT";
$resn_exch{" H91 BAT"}="1H9  BAT"; $resn_exch{" H92 BAT"}="2H9  BAT"; $resn_exch{"H111 BAT"}="1H11 BAT";
$resn_exch{"H112 BAT"}="2H11 BAT"; $resn_exch{"H113 BAT"}="3H11 BAT"; $resn_exch{"H121 BAT"}="1H12 BAT";
$resn_exch{"H122 BAT"}="2H12 BAT"; $resn_exch{"H123 BAT"}="3H12 BAT"; $resn_exch{"H161 BAT"}="1H16 BAT";
$resn_exch{"H162 BAT"}="2H16 BAT"; $resn_exch{"H163 BAT"}="3H16 BAT"; $resn_exch{"H171 BAT"}="1H17 BAT";
$resn_exch{"H172 BAT"}="2H17 BAT"; $resn_exch{"HAJ1 BAU"}="1HAJ BAU"; $resn_exch{"HAJ2 BAU"}="2HAJ BAU";
$resn_exch{"HAK1 BAU"}="1HAK BAU"; $resn_exch{"HAK2 BAU"}="2HAK BAU"; $resn_exch{"HAM1 BAU"}="1HAM BAU";
$resn_exch{"HAM2 BAU"}="2HAM BAU"; $resn_exch{"HAL1 BAU"}="1HAL BAU"; $resn_exch{"HAL2 BAU"}="2HAL BAU";
$resn_exch{"H311 BAX"}="1H31 BAX"; $resn_exch{"H312 BAX"}="2H31 BAX"; $resn_exch{"H313 BAX"}="3H31 BAX";
$resn_exch{" H11 BAY"}="1H1  BAY"; $resn_exch{"H12A BAY"}="2H1  BAY"; $resn_exch{"H13A BAY"}="3H1  BAY";
$resn_exch{" H21 BAY"}="1H2  BAY"; $resn_exch{" H22 BAY"}="2H2  BAY"; $resn_exch{" H23 BAY"}="3H2  BAY";
$resn_exch{"H31A BAY"}="1H3  BAY"; $resn_exch{" H32 BAY"}="2H3  BAY"; $resn_exch{" H33 BAY"}="3H3  BAY";
$resn_exch{" H81 BAY"}="1H8  BAY"; $resn_exch{" H82 BAY"}="2H8  BAY"; $resn_exch{"H101 BAY"}="1H10 BAY";
$resn_exch{"H102 BAY"}="2H10 BAY"; $resn_exch{"H261 BAY"}="1H26 BAY"; $resn_exch{"H262 BAY"}="2H26 BAY";
$resn_exch{"H263 BAY"}="3H26 BAY"; $resn_exch{"H271 BAY"}="1H27 BAY"; $resn_exch{"H272 BAY"}="2H27 BAY";
$resn_exch{"H273 BAY"}="3H27 BAY"; $resn_exch{"H321 BAY"}="1H32 BAY"; $resn_exch{"H322 BAY"}="2H32 BAY";
$resn_exch{"H441 BAY"}="1H44 BAY"; $resn_exch{"H442 BAY"}="2H44 BAY"; $resn_exch{"H561 BAY"}="1H56 BAY";
$resn_exch{"H562 BAY"}="2H56 BAY"; $resn_exch{"H563 BAY"}="3H56 BAY"; $resn_exch{"H571 BAY"}="1H57 BAY";
$resn_exch{"H572 BAY"}="2H57 BAY"; $resn_exch{"H573 BAY"}="3H57 BAY"; $resn_exch{"H621 BAY"}="1H62 BAY";
$resn_exch{"H622 BAY"}="2H62 BAY"; $resn_exch{"H731 BAY"}="1H73 BAY"; $resn_exch{"H732 BAY"}="2H73 BAY";
$resn_exch{"H781 BAY"}="1H78 BAY"; $resn_exch{"H782 BAY"}="2H78 BAY"; $resn_exch{"H783 BAY"}="3H78 BAY";
$resn_exch{"H791 BAY"}="1H79 BAY"; $resn_exch{"H792 BAY"}="2H79 BAY"; $resn_exch{"H793 BAY"}="3H79 BAY";
$resn_exch{"H801 BAY"}="1H80 BAY"; $resn_exch{"H802 BAY"}="2H80 BAY"; $resn_exch{"H803 BAY"}="3H80 BAY";
$resn_exch{"HN21 BAZ"}="1HN2 BAZ"; $resn_exch{"HN22 BAZ"}="2HN2 BAZ"; $resn_exch{"H2'1 BAZ"}="1H2' BAZ";
$resn_exch{"H2'2 BAZ"}="2H2' BAZ"; $resn_exch{"HN4' BAZ"}="'HN4 BAZ"; $resn_exch{" H91 BAZ"}="1H9  BAZ";
$resn_exch{" H92 BAZ"}="2H9  BAZ"; $resn_exch{" H21 BB1"}="1H2  BB1"; $resn_exch{" H22 BB1"}="2H2  BB1";
$resn_exch{" H71 BB1"}="1H7  BB1"; $resn_exch{" H72 BB1"}="2H7  BB1"; $resn_exch{" H81 BB1"}="1H8  BB1";
$resn_exch{" H82 BB1"}="2H8  BB1"; $resn_exch{" H91 BB1"}="1H9  BB1"; $resn_exch{" H92 BB1"}="2H9  BB1";
$resn_exch{"H101 BB1"}="1H10 BB1"; $resn_exch{"H102 BB1"}="2H10 BB1"; $resn_exch{"H103 BB1"}="3H10 BB1";
$resn_exch{"H161 BB1"}="1H16 BB1"; $resn_exch{"H162 BB1"}="2H16 BB1"; $resn_exch{"H163 BB1"}="3H16 BB1";
$resn_exch{"H151 BB1"}="1H15 BB1"; $resn_exch{"H152 BB1"}="2H15 BB1"; $resn_exch{"H153 BB1"}="3H15 BB1";
$resn_exch{"H121 BB1"}="1H12 BB1"; $resn_exch{"H122 BB1"}="2H12 BB1"; $resn_exch{"H123 BB1"}="3H12 BB1";
$resn_exch{"H131 BB1"}="1H13 BB1"; $resn_exch{"H132 BB1"}="2H13 BB1"; $resn_exch{"H133 BB1"}="3H13 BB1";
$resn_exch{"H141 BB1"}="1H14 BB1"; $resn_exch{"H142 BB1"}="2H14 BB1"; $resn_exch{"H143 BB1"}="3H14 BB1";
$resn_exch{" H51 BB2"}="1H5  BB2"; $resn_exch{" H52 BB2"}="2H5  BB2"; $resn_exch{" H71 BB2"}="1H7  BB2";
$resn_exch{" H72 BB2"}="2H7  BB2"; $resn_exch{" H81 BB2"}="1H8  BB2"; $resn_exch{" H82 BB2"}="2H8  BB2";
$resn_exch{" H91 BB2"}="1H9  BB2"; $resn_exch{" H92 BB2"}="2H9  BB2"; $resn_exch{"H101 BB2"}="1H10 BB2";
$resn_exch{"H102 BB2"}="2H10 BB2"; $resn_exch{"H111 BB2"}="1H11 BB2"; $resn_exch{"H112 BB2"}="2H11 BB2";
$resn_exch{"H113 BB2"}="3H11 BB2"; $resn_exch{"H181 BB2"}="1H18 BB2"; $resn_exch{"H182 BB2"}="2H18 BB2";
$resn_exch{"H183 BB2"}="3H18 BB2"; $resn_exch{"H171 BB2"}="1H17 BB2"; $resn_exch{"H172 BB2"}="2H17 BB2";
$resn_exch{"H173 BB2"}="3H17 BB2"; $resn_exch{"H231 BB2"}="1H23 BB2"; $resn_exch{"H232 BB2"}="2H23 BB2";
$resn_exch{"H241 BB2"}="1H24 BB2"; $resn_exch{"H242 BB2"}="2H24 BB2"; $resn_exch{"H251 BB2"}="1H25 BB2";
$resn_exch{"H252 BB2"}="2H25 BB2"; $resn_exch{"H261 BB2"}="1H26 BB2"; $resn_exch{"H262 BB2"}="2H26 BB2";
$resn_exch{"H171 BB3"}="1H17 BB3"; $resn_exch{"H172 BB3"}="2H17 BB3"; $resn_exch{"H181 BB3"}="1H18 BB3";
$resn_exch{"H182 BB3"}="2H18 BB3"; $resn_exch{"H183 BB3"}="3H18 BB3"; $resn_exch{"H011 BB3"}="1H01 BB3";
$resn_exch{"H012 BB3"}="2H01 BB3"; $resn_exch{"H013 BB3"}="3H01 BB3"; $resn_exch{"H021 BB3"}="1H02 BB3";
$resn_exch{"H022 BB3"}="2H02 BB3"; $resn_exch{"H023 BB3"}="3H02 BB3"; $resn_exch{" H81 BBA"}="1H8  BBA";
$resn_exch{" H82 BBA"}="2H8  BBA"; $resn_exch{"HG11 BBA"}="1HG1 BBA"; $resn_exch{"HG12 BBA"}="2HG1 BBA";
$resn_exch{"H181 BBA"}="1H18 BBA"; $resn_exch{"H182 BBA"}="2H18 BBA"; $resn_exch{"HG31 BBA"}="1HG3 BBA";
$resn_exch{"HG32 BBA"}="2HG3 BBA"; $resn_exch{"H21A BBA"}="1H2  BBA"; $resn_exch{" H22 BBA"}="2H2  BBA";
$resn_exch{" H31 BBA"}="1H3  BBA"; $resn_exch{" H32 BBA"}="2H3  BBA"; $resn_exch{" H1A BBA"}="1H   BBA";
$resn_exch{" H2  BBA"}="2H   BBA"; $resn_exch{" H41 BBA"}="1H4  BBA"; $resn_exch{" H42 BBA"}="2H4  BBA";
$resn_exch{"HAN1 BBB"}="1HAN BBB"; $resn_exch{"HAN2 BBB"}="2HAN BBB"; $resn_exch{"HAO1 BBB"}="1HAO BBB";
$resn_exch{"HAO2 BBB"}="2HAO BBB"; $resn_exch{"HAR1 BBB"}="1HAR BBB"; $resn_exch{"HAR2 BBB"}="2HAR BBB";
$resn_exch{"HAQ1 BBB"}="1HAQ BBB"; $resn_exch{"HAQ2 BBB"}="2HAQ BBB"; $resn_exch{"HAP1 BBB"}="1HAP BBB";
$resn_exch{"HAP2 BBB"}="2HAP BBB"; $resn_exch{" HB1 BBC"}="1HB  BBC"; $resn_exch{" HB2 BBC"}="2HB  BBC";
$resn_exch{"H051 BBC"}="1H05 BBC"; $resn_exch{"H052 BBC"}="2H05 BBC"; $resn_exch{" H1  BBC"}="1H   BBC";
$resn_exch{" H2  BBC"}="2H   BBC"; $resn_exch{"H081 BBC"}="1H08 BBC"; $resn_exch{"H082 BBC"}="2H08 BBC";
$resn_exch{"H091 BBC"}="1H09 BBC"; $resn_exch{"H092 BBC"}="2H09 BBC"; $resn_exch{"H101 BBC"}="1H10 BBC";
$resn_exch{"H102 BBC"}="2H10 BBC"; $resn_exch{"H111 BBC"}="1H11 BBC"; $resn_exch{"H112 BBC"}="2H11 BBC";
$resn_exch{"HC21 BBH"}="1HC2 BBH"; $resn_exch{"HC31 BBH"}="1HC3 BBH"; $resn_exch{"HC51 BBH"}="1HC5 BBH";
$resn_exch{"HC61 BBH"}="1HC6 BBH"; $resn_exch{"HC81 BBH"}="1HC8 BBH"; $resn_exch{"HC82 BBH"}="2HC8 BBH";
$resn_exch{"HC83 BBH"}="3HC8 BBH"; $resn_exch{"H131 BBH"}="1H13 BBH"; $resn_exch{"H141 BBH"}="1H14 BBH";
$resn_exch{"H142 BBH"}="2H14 BBH"; $resn_exch{"H171 BBH"}="1H17 BBH"; $resn_exch{"H172 BBH"}="2H17 BBH";
$resn_exch{"HC71 BBH"}="1HC7 BBH"; $resn_exch{"HC72 BBH"}="2HC7 BBH"; $resn_exch{"HC91 BBH"}="1HC9 BBH";
$resn_exch{"H101 BBH"}="1H10 BBH"; $resn_exch{"H121 BBH"}="1H12 BBH"; $resn_exch{"H161 BBH"}="1H16 BBH";
$resn_exch{"H181 BBH"}="1H18 BBH"; $resn_exch{"HBZ1 BBL"}="1HBZ BBL"; $resn_exch{"HBZ2 BBL"}="2HBZ BBL";
$resn_exch{"HBZ3 BBL"}="3HBZ BBL"; $resn_exch{" HB1 BBL"}="1HB  BBL"; $resn_exch{" HB2 BBL"}="2HB  BBL";
$resn_exch{"H151 BBM"}="1H15 BBM"; $resn_exch{"H152 BBM"}="2H15 BBM"; $resn_exch{"H171 BBM"}="1H17 BBM";
$resn_exch{"H172 BBM"}="2H17 BBM"; $resn_exch{"H1C1 BBR"}="1H1C BBR"; $resn_exch{"H1C2 BBR"}="2H1C BBR";
$resn_exch{"H3C1 BBR"}="1H3C BBR"; $resn_exch{"H3C2 BBR"}="2H3C BBR"; $resn_exch{" H11 BBS"}="1H1  BBS";
$resn_exch{" H12 BBS"}="2H1  BBS"; $resn_exch{" H13 BBS"}="3H1  BBS"; $resn_exch{" H31 BBS"}="1H3  BBS";
$resn_exch{" H32 BBS"}="2H3  BBS"; $resn_exch{" H33 BBS"}="3H3  BBS"; $resn_exch{" H41 BBS"}="1H4  BBS";
$resn_exch{" H42 BBS"}="2H4  BBS"; $resn_exch{" H43 BBS"}="3H4  BBS"; $resn_exch{"HP21 BBT"}="1HP2 BBT";
$resn_exch{"HP22 BBT"}="2HP2 BBT"; $resn_exch{"HP31 BBT"}="1HP3 BBT"; $resn_exch{"HP32 BBT"}="2HP3 BBT";
$resn_exch{"HP51 BBT"}="1HP5 BBT"; $resn_exch{"HP52 BBT"}="2HP5 BBT"; $resn_exch{"HP61 BBT"}="1HP6 BBT";
$resn_exch{"HP62 BBT"}="2HP6 BBT"; $resn_exch{"HP71 BBT"}="1HP7 BBT"; $resn_exch{"HP72 BBT"}="2HP7 BBT";
$resn_exch{"HP81 BBT"}="1HP8 BBT"; $resn_exch{"HP82 BBT"}="2HP8 BBT"; $resn_exch{" H11 BBU"}="1H1  BBU";
$resn_exch{" H12 BBU"}="2H1  BBU"; $resn_exch{" H21 BBU"}="1H2  BBU"; $resn_exch{" H22 BBU"}="2H2  BBU";
$resn_exch{" H31 BBU"}="1H3  BBU"; $resn_exch{" H32 BBU"}="2H3  BBU"; $resn_exch{" H41 BBU"}="1H4  BBU";
$resn_exch{" H42 BBU"}="2H4  BBU"; $resn_exch{" H43 BBU"}="3H4  BBU"; $resn_exch{" H11 BBX"}="1H1  BBX";
$resn_exch{" H12 BBX"}="2H1  BBX"; $resn_exch{" H21 BBX"}="1H2  BBX"; $resn_exch{" H22 BBX"}="2H2  BBX";
$resn_exch{" H23 BBX"}="3H2  BBX"; $resn_exch{"H201 BBZ"}="1H20 BBZ"; $resn_exch{"H202 BBZ"}="2H20 BBZ";
$resn_exch{"H211 BBZ"}="1H21 BBZ"; $resn_exch{"H212 BBZ"}="2H21 BBZ"; $resn_exch{"H221 BBZ"}="1H22 BBZ";
$resn_exch{"H222 BBZ"}="2H22 BBZ"; $resn_exch{"H231 BBZ"}="1H23 BBZ"; $resn_exch{"H232 BBZ"}="2H23 BBZ";
$resn_exch{"H241 BBZ"}="1H24 BBZ"; $resn_exch{"H242 BBZ"}="2H24 BBZ"; $resn_exch{"H243 BBZ"}="3H24 BBZ";
$resn_exch{"HN11 BBZ"}="1HN1 BBZ"; $resn_exch{"HN12 BBZ"}="2HN1 BBZ"; $resn_exch{"HN13 BBZ"}="3HN1 BBZ";
$resn_exch{"HN21 BBZ"}="1HN2 BBZ"; $resn_exch{"HN22 BBZ"}="2HN2 BBZ"; $resn_exch{"HN23 BBZ"}="3HN2 BBZ";
$resn_exch{"H141  BC"}="1H14  BC"; $resn_exch{"H142  BC"}="2H14  BC"; $resn_exch{"H4C1 BC2"}="1H4C BC2";
$resn_exch{"H6C1 BC2"}="1H6C BC2"; $resn_exch{"H361 BC2"}="1H36 BC2"; $resn_exch{"H362 BC2"}="2H36 BC2";
$resn_exch{"H363 BC2"}="3H36 BC2"; $resn_exch{"H351 BC2"}="1H35 BC2"; $resn_exch{"H352 BC2"}="2H35 BC2";
$resn_exch{"H353 BC2"}="3H35 BC2"; $resn_exch{"H391 BC2"}="1H39 BC2"; $resn_exch{"H392 BC2"}="2H39 BC2";
$resn_exch{"H393 BC2"}="3H39 BC2"; $resn_exch{"H251 BC2"}="1H25 BC2"; $resn_exch{"H252 BC2"}="2H25 BC2";
$resn_exch{"H253 BC2"}="3H25 BC2"; $resn_exch{"H9C1 BC2"}="1H9C BC2"; $resn_exch{"H9C2 BC2"}="2H9C BC2";
$resn_exch{"H271 BC2"}="1H27 BC2"; $resn_exch{"H272 BC2"}="2H27 BC2"; $resn_exch{"H273 BC2"}="3H27 BC2";
$resn_exch{"H401 BC2"}="1H40 BC2"; $resn_exch{"H402 BC2"}="2H40 BC2"; $resn_exch{"H403 BC2"}="3H40 BC2";
$resn_exch{"H291 BC2"}="1H29 BC2"; $resn_exch{"H292 BC2"}="2H29 BC2"; $resn_exch{"H293 BC2"}="3H29 BC2";
$resn_exch{"H301 BC2"}="1H30 BC2"; $resn_exch{"H302 BC2"}="2H30 BC2"; $resn_exch{"H303 BC2"}="3H30 BC2";
$resn_exch{"H341 BC2"}="1H34 BC2"; $resn_exch{"H342 BC2"}="2H34 BC2"; $resn_exch{" H2A BCA"}=" AH2 BCA";
$resn_exch{"H61A BCA"}="AH61 BCA"; $resn_exch{"H62A BCA"}="AH62 BCA"; $resn_exch{" H8A BCA"}=" AH8 BCA";
$resn_exch{" H1D BCA"}="AH1* BCA"; $resn_exch{" H2D BCA"}="AH2* BCA"; $resn_exch{"HO2A BCA"}="AHO2 BCA";
$resn_exch{" H3D BCA"}="AH3* BCA"; $resn_exch{"HOA8 BCA"}="8HOA BCA"; $resn_exch{"HOA9 BCA"}="9HOA BCA";
$resn_exch{" H4D BCA"}="AH4* BCA"; $resn_exch{"H51A BCA"}="AH51 BCA"; $resn_exch{"H52A BCA"}="AH52 BCA";
$resn_exch{"HOA2 BCA"}="2HOA BCA"; $resn_exch{"HOA5 BCA"}="5HOA BCA"; $resn_exch{"H121 BCA"}="1H12 BCA";
$resn_exch{"H122 BCA"}="2H12 BCA"; $resn_exch{"H131 BCA"}="1H13 BCA"; $resn_exch{"H132 BCA"}="2H13 BCA";
$resn_exch{"H133 BCA"}="3H13 BCA"; $resn_exch{"H141 BCA"}="1H14 BCA"; $resn_exch{"H142 BCA"}="2H14 BCA";
$resn_exch{"H143 BCA"}="3H14 BCA"; $resn_exch{" H71 BCA"}="1H7  BCA"; $resn_exch{" H72 BCA"}="2H7  BCA";
$resn_exch{" H61 BCA"}="1H6  BCA"; $resn_exch{" H62 BCA"}="2H6  BCA"; $resn_exch{" H31 BCA"}="1H3  BCA";
$resn_exch{" H32 BCA"}="2H3  BCA"; $resn_exch{" H21 BCA"}="1H2  BCA"; $resn_exch{" H22 BCA"}="2H2  BCA";
$resn_exch{" H3B BCA"}=" BH3 BCA"; $resn_exch{" H4B BCA"}=" BH4 BCA"; $resn_exch{"HO2B BCA"}="BHO2 BCA";
$resn_exch{" H6B BCA"}=" BH6 BCA"; $resn_exch{" H7B BCA"}=" BH7 BCA"; $resn_exch{"HMA1 BCB"}="1HMA BCB";
$resn_exch{"HMA2 BCB"}="2HMA BCB"; $resn_exch{"HMA3 BCB"}="3HMA BCB"; $resn_exch{"HAA1 BCB"}="1HAA BCB";
$resn_exch{"HAA2 BCB"}="2HAA BCB"; $resn_exch{"HBA1 BCB"}="1HBA BCB"; $resn_exch{"HBA2 BCB"}="2HBA BCB";
$resn_exch{"HMB1 BCB"}="1HMB BCB"; $resn_exch{"HMB2 BCB"}="2HMB BCB"; $resn_exch{"HMB3 BCB"}="3HMB BCB";
$resn_exch{"HBB1 BCB"}="1HBB BCB"; $resn_exch{"HBB2 BCB"}="2HBB BCB"; $resn_exch{"HBB3 BCB"}="3HBB BCB";
$resn_exch{"HMC1 BCB"}="1HMC BCB"; $resn_exch{"HMC2 BCB"}="2HMC BCB"; $resn_exch{"HMC3 BCB"}="3HMC BCB";
$resn_exch{"HAC1 BCB"}="1HAC BCB"; $resn_exch{"HBC1 BCB"}="1HBC BCB"; $resn_exch{"HBC2 BCB"}="2HBC BCB";
$resn_exch{"HBC3 BCB"}="3HBC BCB"; $resn_exch{"HMD1 BCB"}="1HMD BCB"; $resn_exch{"HMD2 BCB"}="2HMD BCB";
$resn_exch{"HMD3 BCB"}="3HMD BCB"; $resn_exch{"HED1 BCB"}="1HED BCB"; $resn_exch{"HED2 BCB"}="2HED BCB";
$resn_exch{"HED3 BCB"}="3HED BCB"; $resn_exch{" H11 BCB"}="1H1  BCB"; $resn_exch{" H12 BCB"}="2H1  BCB";
$resn_exch{" H41 BCB"}="1H4  BCB"; $resn_exch{" H42 BCB"}="2H4  BCB"; $resn_exch{" H43 BCB"}="3H4  BCB";
$resn_exch{" H51 BCB"}="1H5  BCB"; $resn_exch{" H52 BCB"}="2H5  BCB"; $resn_exch{" H61 BCB"}="1H6  BCB";
$resn_exch{" H62 BCB"}="2H6  BCB"; $resn_exch{" H71 BCB"}="1H7  BCB"; $resn_exch{" H72 BCB"}="2H7  BCB";
$resn_exch{" H91 BCB"}="1H9  BCB"; $resn_exch{" H92 BCB"}="2H9  BCB"; $resn_exch{" H93 BCB"}="3H9  BCB";
$resn_exch{"H101 BCB"}="1H10 BCB"; $resn_exch{"H102 BCB"}="2H10 BCB"; $resn_exch{"H111 BCB"}="1H11 BCB";
$resn_exch{"H112 BCB"}="2H11 BCB"; $resn_exch{"H121 BCB"}="1H12 BCB"; $resn_exch{"H122 BCB"}="2H12 BCB";
$resn_exch{"H141 BCB"}="1H14 BCB"; $resn_exch{"H142 BCB"}="2H14 BCB"; $resn_exch{"H143 BCB"}="3H14 BCB";
$resn_exch{"H151 BCB"}="1H15 BCB"; $resn_exch{"H152 BCB"}="2H15 BCB"; $resn_exch{"H161 BCB"}="1H16 BCB";
$resn_exch{"H162 BCB"}="2H16 BCB"; $resn_exch{"H171 BCB"}="1H17 BCB"; $resn_exch{"H172 BCB"}="2H17 BCB";
$resn_exch{"H191 BCB"}="1H19 BCB"; $resn_exch{"H192 BCB"}="2H19 BCB"; $resn_exch{"H193 BCB"}="3H19 BCB";
$resn_exch{"H201 BCB"}="1H20 BCB"; $resn_exch{"H202 BCB"}="2H20 BCB"; $resn_exch{"H203 BCB"}="3H20 BCB";
$resn_exch{" H31 BCC"}="1H3  BCC"; $resn_exch{" H32 BCC"}="2H3  BCC"; $resn_exch{" H41 BCC"}="1H4  BCC";
$resn_exch{" H42 BCC"}="2H4  BCC"; $resn_exch{" H61 BCC"}="1H6  BCC"; $resn_exch{" H62 BCC"}="2H6  BCC";
$resn_exch{" H71 BCC"}="1H7  BCC"; $resn_exch{" H72 BCC"}="2H7  BCC"; $resn_exch{"HN21 BCC"}="1HN2 BCC";
$resn_exch{"HN22 BCC"}="2HN2 BCC"; $resn_exch{"H101 BCC"}="1H10 BCC"; $resn_exch{"H102 BCC"}="2H10 BCC";
$resn_exch{"HO21 BCD"}="1HO2 BCD"; $resn_exch{"HO31 BCD"}="1HO3 BCD"; $resn_exch{"H611 BCD"}="1H61 BCD";
$resn_exch{"H612 BCD"}="2H61 BCD"; $resn_exch{"HO61 BCD"}="1HO6 BCD"; $resn_exch{"HO22 BCD"}="2HO2 BCD";
$resn_exch{"HO32 BCD"}="2HO3 BCD"; $resn_exch{"H621 BCD"}="1H62 BCD"; $resn_exch{"H622 BCD"}="2H62 BCD";
$resn_exch{"HO62 BCD"}="2HO6 BCD"; $resn_exch{"HO23 BCD"}="3HO2 BCD"; $resn_exch{"HO33 BCD"}="3HO3 BCD";
$resn_exch{"H631 BCD"}="1H63 BCD"; $resn_exch{"H632 BCD"}="2H63 BCD"; $resn_exch{"HO63 BCD"}="3HO6 BCD";
$resn_exch{"HO24 BCD"}="4HO2 BCD"; $resn_exch{"HO34 BCD"}="4HO3 BCD"; $resn_exch{"H641 BCD"}="1H64 BCD";
$resn_exch{"H642 BCD"}="2H64 BCD"; $resn_exch{"HO64 BCD"}="4HO6 BCD"; $resn_exch{"HO25 BCD"}="5HO2 BCD";
$resn_exch{"HO35 BCD"}="5HO3 BCD"; $resn_exch{"H651 BCD"}="1H65 BCD"; $resn_exch{"H652 BCD"}="2H65 BCD";
$resn_exch{"HO65 BCD"}="5HO6 BCD"; $resn_exch{"HO26 BCD"}="6HO2 BCD"; $resn_exch{"HO36 BCD"}="6HO3 BCD";
$resn_exch{"H661 BCD"}="1H66 BCD"; $resn_exch{"H662 BCD"}="2H66 BCD"; $resn_exch{"HO66 BCD"}="6HO6 BCD";
$resn_exch{"HO27 BCD"}="7HO2 BCD"; $resn_exch{"HO37 BCD"}="7HO3 BCD"; $resn_exch{"H671 BCD"}="1H67 BCD";
$resn_exch{"H672 BCD"}="2H67 BCD"; $resn_exch{"HO67 BCD"}="7HO6 BCD"; $resn_exch{" H41 BCG"}="1H4  BCG";
$resn_exch{" H42 BCG"}="2H4  BCG"; $resn_exch{" H61 BCG"}="1H6  BCG"; $resn_exch{" H62 BCG"}="2H6  BCG";
$resn_exch{" H71 BCG"}="1H7  BCG"; $resn_exch{" H72 BCG"}="2H7  BCG"; $resn_exch{"H161 BCG"}="1H16 BCG";
$resn_exch{"H162 BCG"}="2H16 BCG"; $resn_exch{"H163 BCG"}="3H16 BCG"; $resn_exch{"H211 BCH"}="1H21 BCH";
$resn_exch{"H212 BCH"}="2H21 BCH"; $resn_exch{"H213 BCH"}="3H21 BCH"; $resn_exch{"H201 BCH"}="1H20 BCH";
$resn_exch{"H202 BCH"}="2H20 BCH"; $resn_exch{"H191 BCH"}="1H19 BCH"; $resn_exch{"H192 BCH"}="2H19 BCH";
$resn_exch{"H161 BCH"}="1H16 BCH"; $resn_exch{"H162 BCH"}="2H16 BCH"; $resn_exch{"H151 BCH"}="1H15 BCH";
$resn_exch{"H152 BCH"}="2H15 BCH"; $resn_exch{"H131 BCH"}="1H13 BCH"; $resn_exch{"H132 BCH"}="2H13 BCH";
$resn_exch{"H133 BCH"}="3H13 BCH"; $resn_exch{"H221 BCH"}="1H22 BCH"; $resn_exch{"H222 BCH"}="2H22 BCH";
$resn_exch{"H223 BCH"}="3H22 BCH"; $resn_exch{"H111 BCH"}="1H11 BCH"; $resn_exch{"H112 BCH"}="2H11 BCH";
$resn_exch{"H113 BCH"}="3H11 BCH"; $resn_exch{"HMA1 BCL"}="1HMA BCL"; $resn_exch{"HMA2 BCL"}="2HMA BCL";
$resn_exch{"HMA3 BCL"}="3HMA BCL"; $resn_exch{"HAA1 BCL"}="1HAA BCL"; $resn_exch{"HAA2 BCL"}="2HAA BCL";
$resn_exch{"HBA1 BCL"}="1HBA BCL"; $resn_exch{"HBA2 BCL"}="2HBA BCL"; $resn_exch{"HMB1 BCL"}="1HMB BCL";
$resn_exch{"HMB2 BCL"}="2HMB BCL"; $resn_exch{"HMB3 BCL"}="3HMB BCL"; $resn_exch{"HBB1 BCL"}="1HBB BCL";
$resn_exch{"HBB2 BCL"}="2HBB BCL"; $resn_exch{"HBB3 BCL"}="3HBB BCL"; $resn_exch{"HMC1 BCL"}="1HMC BCL";
$resn_exch{"HMC2 BCL"}="2HMC BCL"; $resn_exch{"HMC3 BCL"}="3HMC BCL"; $resn_exch{"HAC1 BCL"}="1HAC BCL";
$resn_exch{"HAC2 BCL"}="2HAC BCL"; $resn_exch{"HBC1 BCL"}="1HBC BCL"; $resn_exch{"HBC2 BCL"}="2HBC BCL";
$resn_exch{"HBC3 BCL"}="3HBC BCL"; $resn_exch{"HMD1 BCL"}="1HMD BCL"; $resn_exch{"HMD2 BCL"}="2HMD BCL";
$resn_exch{"HMD3 BCL"}="3HMD BCL"; $resn_exch{"HED1 BCL"}="1HED BCL"; $resn_exch{"HED2 BCL"}="2HED BCL";
$resn_exch{"HED3 BCL"}="3HED BCL"; $resn_exch{" H11 BCL"}="1H1  BCL"; $resn_exch{" H12 BCL"}="2H1  BCL";
$resn_exch{" H41 BCL"}="1H4  BCL"; $resn_exch{" H42 BCL"}="2H4  BCL"; $resn_exch{" H43 BCL"}="3H4  BCL";
$resn_exch{" H51 BCL"}="1H5  BCL"; $resn_exch{" H52 BCL"}="2H5  BCL"; $resn_exch{" H61 BCL"}="1H6  BCL";
$resn_exch{" H62 BCL"}="2H6  BCL"; $resn_exch{" H71 BCL"}="1H7  BCL"; $resn_exch{" H72 BCL"}="2H7  BCL";
$resn_exch{" H91 BCL"}="1H9  BCL"; $resn_exch{" H92 BCL"}="2H9  BCL"; $resn_exch{" H93 BCL"}="3H9  BCL";
$resn_exch{"H101 BCL"}="1H10 BCL"; $resn_exch{"H102 BCL"}="2H10 BCL"; $resn_exch{"H111 BCL"}="1H11 BCL";
$resn_exch{"H112 BCL"}="2H11 BCL"; $resn_exch{"H121 BCL"}="1H12 BCL"; $resn_exch{"H122 BCL"}="2H12 BCL";
$resn_exch{"H141 BCL"}="1H14 BCL"; $resn_exch{"H142 BCL"}="2H14 BCL"; $resn_exch{"H143 BCL"}="3H14 BCL";
$resn_exch{"H151 BCL"}="1H15 BCL"; $resn_exch{"H152 BCL"}="2H15 BCL"; $resn_exch{"H161 BCL"}="1H16 BCL";
$resn_exch{"H162 BCL"}="2H16 BCL"; $resn_exch{"H171 BCL"}="1H17 BCL"; $resn_exch{"H172 BCL"}="2H17 BCL";
$resn_exch{"H191 BCL"}="1H19 BCL"; $resn_exch{"H192 BCL"}="2H19 BCL"; $resn_exch{"H193 BCL"}="3H19 BCL";
$resn_exch{"H201 BCL"}="1H20 BCL"; $resn_exch{"H202 BCL"}="2H20 BCL"; $resn_exch{"H203 BCL"}="3H20 BCL";
$resn_exch{"H5A1 BCM"}="1H5A BCM"; $resn_exch{"H5A2 BCM"}="2H5A BCM"; $resn_exch{" H41 BCM"}="1H4  BCM";
$resn_exch{" H42 BCM"}="2H4  BCM"; $resn_exch{" H31 BCM"}="1H3  BCM"; $resn_exch{" H32 BCM"}="2H3  BCM";
$resn_exch{" HB1 BCM"}="1HB  BCM"; $resn_exch{" HB2 BCM"}="2HB  BCM"; $resn_exch{" HB3 BCM"}="3HB  BCM";
$resn_exch{" HA1 BCM"}="1HA  BCM"; $resn_exch{" HA2 BCM"}="2HA  BCM"; $resn_exch{" H11 BCN"}="1H1  BCN";
$resn_exch{" H12 BCN"}="2H1  BCN"; $resn_exch{" H31 BCN"}="1H3  BCN"; $resn_exch{" H32 BCN"}="2H3  BCN";
$resn_exch{" H41 BCN"}="1H4  BCN"; $resn_exch{" H42 BCN"}="2H4  BCN"; $resn_exch{" H51 BCN"}="1H5  BCN";
$resn_exch{" H52 BCN"}="2H5  BCN"; $resn_exch{" H61 BCN"}="1H6  BCN"; $resn_exch{" H62 BCN"}="2H6  BCN";
$resn_exch{" H21 BCQ"}="1H2  BCQ"; $resn_exch{" H22 BCQ"}="2H2  BCQ"; $resn_exch{" H31 BCQ"}="1H3  BCQ";
$resn_exch{" H32 BCQ"}="2H3  BCQ"; $resn_exch{" H41 BCQ"}="1H4  BCQ"; $resn_exch{" H42 BCQ"}="2H4  BCQ";
$resn_exch{" H51 BCQ"}="1H5  BCQ"; $resn_exch{" H52 BCQ"}="2H5  BCQ"; $resn_exch{"H031 BCQ"}="1H03 BCQ";
$resn_exch{"H032 BCQ"}="2H03 BCQ"; $resn_exch{"H091 BCQ"}="1H09 BCQ"; $resn_exch{"H092 BCQ"}="2H09 BCQ";
$resn_exch{"H181 BCQ"}="1H18 BCQ"; $resn_exch{"H182 BCQ"}="2H18 BCQ"; $resn_exch{"H183 BCQ"}="3H18 BCQ";
$resn_exch{"H171 BCQ"}="1H17 BCQ"; $resn_exch{"H172 BCQ"}="2H17 BCQ"; $resn_exch{"H173 BCQ"}="3H17 BCQ";
$resn_exch{"HB21 BCQ"}="1HB2 BCQ"; $resn_exch{"HB22 BCQ"}="2HB2 BCQ"; $resn_exch{"HB81 BCQ"}="1HB8 BCQ";
$resn_exch{"HB82 BCQ"}="2HB8 BCQ"; $resn_exch{"H7A1 BCQ"}="1H7A BCQ"; $resn_exch{"H7A2 BCQ"}="2H7A BCQ";
$resn_exch{" HT1 BCQ"}="1HT  BCQ"; $resn_exch{" HT2 BCQ"}="2HT  BCQ"; $resn_exch{"HC21 BCR"}="1HC2 BCR";
$resn_exch{"HC22 BCR"}="2HC2 BCR"; $resn_exch{"HC31 BCR"}="1HC3 BCR"; $resn_exch{"HC32 BCR"}="2HC3 BCR";
$resn_exch{"HC41 BCR"}="1HC4 BCR"; $resn_exch{"HC42 BCR"}="2HC4 BCR"; $resn_exch{"H10C BCR"}="CH10 BCR";
$resn_exch{"H11C BCR"}="CH11 BCR"; $resn_exch{"H331 BCR"}="1H33 BCR"; $resn_exch{"H332 BCR"}="2H33 BCR";
$resn_exch{"H333 BCR"}="3H33 BCR"; $resn_exch{"H311 BCR"}="1H31 BCR"; $resn_exch{"H312 BCR"}="2H31 BCR";
$resn_exch{"H313 BCR"}="3H31 BCR"; $resn_exch{"H321 BCR"}="1H32 BCR"; $resn_exch{"H322 BCR"}="2H32 BCR";
$resn_exch{"H323 BCR"}="3H32 BCR"; $resn_exch{"H341 BCR"}="1H34 BCR"; $resn_exch{"H342 BCR"}="2H34 BCR";
$resn_exch{"H343 BCR"}="3H34 BCR"; $resn_exch{"H12C BCR"}="CH12 BCR"; $resn_exch{"H14C BCR"}="CH14 BCR";
$resn_exch{"H15C BCR"}="CH15 BCR"; $resn_exch{"H16C BCR"}="CH16 BCR"; $resn_exch{"H17C BCR"}="CH17 BCR";
$resn_exch{"H19C BCR"}="CH19 BCR"; $resn_exch{"H20C BCR"}="CH20 BCR"; $resn_exch{"H21C BCR"}="CH21 BCR";
$resn_exch{"H23C BCR"}="CH23 BCR"; $resn_exch{"H24C BCR"}="CH24 BCR"; $resn_exch{"H271 BCR"}="1H27 BCR";
$resn_exch{"H272 BCR"}="2H27 BCR"; $resn_exch{"H281 BCR"}="1H28 BCR"; $resn_exch{"H282 BCR"}="2H28 BCR";
$resn_exch{"H291 BCR"}="1H29 BCR"; $resn_exch{"H292 BCR"}="2H29 BCR"; $resn_exch{"H351 BCR"}="1H35 BCR";
$resn_exch{"H352 BCR"}="2H35 BCR"; $resn_exch{"H353 BCR"}="3H35 BCR"; $resn_exch{"H361 BCR"}="1H36 BCR";
$resn_exch{"H362 BCR"}="2H36 BCR"; $resn_exch{"H363 BCR"}="3H36 BCR"; $resn_exch{"H371 BCR"}="1H37 BCR";
$resn_exch{"H372 BCR"}="2H37 BCR"; $resn_exch{"H373 BCR"}="3H37 BCR"; $resn_exch{"H381 BCR"}="1H38 BCR";
$resn_exch{"H382 BCR"}="2H38 BCR"; $resn_exch{"H383 BCR"}="3H38 BCR"; $resn_exch{"H391 BCR"}="1H39 BCR";
$resn_exch{"H392 BCR"}="2H39 BCR"; $resn_exch{"H393 BCR"}="3H39 BCR"; $resn_exch{"H401 BCR"}="1H40 BCR";
$resn_exch{"H402 BCR"}="2H40 BCR"; $resn_exch{"H403 BCR"}="3H40 BCR"; $resn_exch{" H   BCS"}="1HN  BCS";
$resn_exch{" H2  BCS"}="2HN  BCS"; $resn_exch{" HB2 BCS"}="1HB  BCS"; $resn_exch{" HB3 BCS"}="2HB  BCS";
$resn_exch{" HD2 BCS"}="1HD  BCS"; $resn_exch{" HD3 BCS"}="2HD  BCS"; $resn_exch{"H141 BCV"}="1H14 BCV";
$resn_exch{"H142 BCV"}="2H14 BCV"; $resn_exch{"H3C1 BCV"}="1H3C BCV"; $resn_exch{"H3C2 BCV"}="2H3C BCV";
$resn_exch{"H4C1 BCV"}="1H4C BCV"; $resn_exch{"H4C2 BCV"}="2H4C BCV"; $resn_exch{"H7C1 BCV"}="1H7C BCV";
$resn_exch{"H7C2 BCV"}="2H7C BCV"; $resn_exch{"H161 BCV"}="1H16 BCV"; $resn_exch{"H162 BCV"}="2H16 BCV";
$resn_exch{"H371 BCV"}="1H37 BCV"; $resn_exch{"H372 BCV"}="2H37 BCV"; $resn_exch{"H331 BCV"}="1H33 BCV";
$resn_exch{"H332 BCV"}="2H33 BCV"; $resn_exch{" H2  BCX"}=" HN2 BCX"; $resn_exch{" HC1 BCX"}="1HC  BCX";
$resn_exch{" HC2 BCX"}="2HC  BCX"; $resn_exch{" HB2 BCX"}="1HB  BCX"; $resn_exch{" HB3 BCX"}="2HB  BCX";
$resn_exch{" HN1 BCY"}="1HN  BCY"; $resn_exch{" HN2 BCY"}="2HN  BCY"; $resn_exch{" HB1 BCY"}="1HB  BCY";
$resn_exch{" HB2 BCY"}="2HB  BCY"; $resn_exch{" HB3 BCY"}="3HB  BCY"; $resn_exch{" H31 BCY"}="1H3  BCY";
$resn_exch{" H32 BCY"}="2H3  BCY"; $resn_exch{" H41 BCY"}="1H4  BCY"; $resn_exch{" H42 BCY"}="2H4  BCY";
$resn_exch{" H61 BCY"}="1H6  BCY"; $resn_exch{" H62 BCY"}="2H6  BCY"; $resn_exch{" H71 BCY"}="1H7  BCY";
$resn_exch{" H72 BCY"}="2H7  BCY"; $resn_exch{" H21 BCZ"}="1H2  BCZ"; $resn_exch{"H251 BCZ"}="1H25 BCZ";
$resn_exch{"H301 BCZ"}="1H30 BCZ"; $resn_exch{"H271 BCZ"}="1H27 BCZ"; $resn_exch{"H272 BCZ"}="2H27 BCZ";
$resn_exch{" H31 BCZ"}="1H3  BCZ"; $resn_exch{"H101 BCZ"}="1H10 BCZ"; $resn_exch{"H241 BCZ"}="1H24 BCZ";
$resn_exch{"H371 BCZ"}="1H37 BCZ"; $resn_exch{"H372 BCZ"}="2H37 BCZ"; $resn_exch{"H381 BCZ"}="1H38 BCZ";
$resn_exch{"H382 BCZ"}="2H38 BCZ"; $resn_exch{"H383 BCZ"}="3H38 BCZ"; $resn_exch{"H361 BCZ"}="1H36 BCZ";
$resn_exch{"H362 BCZ"}="2H36 BCZ"; $resn_exch{"H391 BCZ"}="1H39 BCZ"; $resn_exch{"H392 BCZ"}="2H39 BCZ";
$resn_exch{"H393 BCZ"}="3H39 BCZ"; $resn_exch{"H111 BCZ"}="1H11 BCZ"; $resn_exch{"H151 BCZ"}="1H15 BCZ";
$resn_exch{"H152 BCZ"}="2H15 BCZ"; $resn_exch{"H153 BCZ"}="3H15 BCZ"; $resn_exch{" H41 BCZ"}="1H4  BCZ";
$resn_exch{"HO91 BCZ"}="1HO9 BCZ"; $resn_exch{" H51 BCZ"}="1H5  BCZ"; $resn_exch{"HO71 BCZ"}="1HO7 BCZ";
$resn_exch{" H11 BCZ"}="1H1  BCZ"; $resn_exch{" H12 BCZ"}="2H1  BCZ"; $resn_exch{"H2C1 BD1"}="1H2C BD1";
$resn_exch{"H2C2 BD1"}="2H2C BD1"; $resn_exch{"H5C1 BD1"}="1H5C BD1"; $resn_exch{"H5C2 BD1"}="2H5C BD1";
$resn_exch{"H101 BD2"}="1H10 BD2"; $resn_exch{"H102 BD2"}="2H10 BD2"; $resn_exch{"H111 BD2"}="1H11 BD2";
$resn_exch{"H112 BD2"}="2H11 BD2"; $resn_exch{"H121 BD2"}="1H12 BD2"; $resn_exch{"H122 BD2"}="2H12 BD2";
$resn_exch{"H141 BD2"}="1H14 BD2"; $resn_exch{"H142 BD2"}="2H14 BD2"; $resn_exch{"H151 BD4"}="1H15 BD4";
$resn_exch{"H152 BD4"}="2H15 BD4"; $resn_exch{"H153 BD4"}="3H15 BD4"; $resn_exch{"H141 BD4"}="1H14 BD4";
$resn_exch{"H142 BD4"}="2H14 BD4"; $resn_exch{"H143 BD4"}="3H14 BD4"; $resn_exch{"H161 BD4"}="1H16 BD4";
$resn_exch{"H162 BD4"}="2H16 BD4"; $resn_exch{"H171 BD4"}="1H17 BD4"; $resn_exch{"H172 BD4"}="2H17 BD4";
$resn_exch{"H181 BD4"}="1H18 BD4"; $resn_exch{"H182 BD4"}="2H18 BD4"; $resn_exch{"H191 BD4"}="1H19 BD4";
$resn_exch{"H192 BD4"}="2H19 BD4"; $resn_exch{"H111 BDA"}="1H11 BDA"; $resn_exch{"H112 BDA"}="2H11 BDA";
$resn_exch{"H141 BDA"}="1H14 BDA"; $resn_exch{"H142 BDA"}="2H14 BDA"; $resn_exch{"H143 BDA"}="3H14 BDA";
$resn_exch{"H151 BDA"}="1H15 BDA"; $resn_exch{"H152 BDA"}="2H15 BDA"; $resn_exch{"H211 BDA"}="1H21 BDA";
$resn_exch{"H212 BDA"}="2H21 BDA"; $resn_exch{"H213 BDA"}="3H21 BDA"; $resn_exch{" H1B BDA"}=" H1* BDA";
$resn_exch{"H2'1 BDA"}="1H2* BDA"; $resn_exch{"H2'2 BDA"}="2H2* BDA"; $resn_exch{" H3B BDA"}=" H3* BDA";
$resn_exch{"HN'1 BDA"}="1HN* BDA"; $resn_exch{"HN'2 BDA"}="2HN* BDA"; $resn_exch{" H4' BDA"}=" H4* BDA";
$resn_exch{"HO4' BDA"}="*HO4 BDA"; $resn_exch{" H5' BDA"}=" H5* BDA"; $resn_exch{"H6'1 BDA"}="1H6* BDA";
$resn_exch{"H6'2 BDA"}="2H6* BDA"; $resn_exch{"H6'3 BDA"}="3H6* BDA"; $resn_exch{"H7'1 BDA"}="1H7* BDA";
$resn_exch{"H7'2 BDA"}="2H7* BDA"; $resn_exch{"HO8' BDA"}="'HO8 BDA"; $resn_exch{"H10' BDA"}="'H10 BDA";
$resn_exch{"HL'1 BDA"}="1HL' BDA"; $resn_exch{"HL'2 BDA"}="2HL' BDA"; $resn_exch{"H12' BDA"}="'H12 BDA";
$resn_exch{"HK'1 BDA"}="1HK' BDA"; $resn_exch{"HK'2 BDA"}="2HK' BDA"; $resn_exch{"HK'3 BDA"}="3HK' BDA";
$resn_exch{"HJ'1 BDA"}="1HJ' BDA"; $resn_exch{"HJ'2 BDA"}="2HJ' BDA"; $resn_exch{"H17' BDA"}="'H17 BDA";
$resn_exch{"HI'1 BDA"}="1HI' BDA"; $resn_exch{"HI'2 BDA"}="2HI' BDA"; $resn_exch{"HI'3 BDA"}="3HI' BDA";
$resn_exch{" H1D BDA"}="'H1* BDA"; $resn_exch{" H2B BDA"}="1H2' BDA"; $resn_exch{" H2D BDA"}="2H2' BDA";
$resn_exch{" H3D BDA"}="'H3* BDA"; $resn_exch{" HNB BDA"}="1HN' BDA"; $resn_exch{" HND BDA"}="2HN' BDA";
$resn_exch{" H4D BDA"}="'H4* BDA"; $resn_exch{"HO4A BDA"}="'HO4 BDA"; $resn_exch{" H5D BDA"}="'H5* BDA";
$resn_exch{" H6B BDA"}="1H6' BDA"; $resn_exch{" H6D BDA"}="2H6' BDA"; $resn_exch{" H6E BDA"}="3H6' BDA";
$resn_exch{" H7B BDA"}="1H7' BDA"; $resn_exch{" H7D BDA"}="2H7' BDA"; $resn_exch{"H9A' BDA"}="'H9A BDA";
$resn_exch{"H9B' BDA"}="'H9B BDA"; $resn_exch{"HO11 BDB"}="1HO1 BDB"; $resn_exch{"HO21 BDB"}="1HO2 BDB";
$resn_exch{"HC21 BDB"}="1HC2 BDB"; $resn_exch{"HC31 BDB"}="1HC3 BDB"; $resn_exch{"HC51 BDB"}="1HC5 BDB";
$resn_exch{"HC61 BDB"}="1HC6 BDB"; $resn_exch{"HB21 BDB"}="1HB2 BDB"; $resn_exch{"HB31 BDB"}="1HB3 BDB";
$resn_exch{"HB51 BDB"}="1HB5 BDB"; $resn_exch{"HB61 BDB"}="1HB6 BDB"; $resn_exch{"HH11 BDB"}="1HH1 BDB";
$resn_exch{"HH21 BDB"}="1HH2 BDB"; $resn_exch{"HE11 BDC"}="1HE1 BDC"; $resn_exch{"HE12 BDC"}="2HE1 BDC";
$resn_exch{"HE31 BDC"}="1HE3 BDC"; $resn_exch{"HE32 BDC"}="2HE3 BDC"; $resn_exch{"HE21 BDC"}="1HE2 BDC";
$resn_exch{"HE22 BDC"}="2HE2 BDC"; $resn_exch{"HE41 BDC"}="1HE4 BDC"; $resn_exch{"HE42 BDC"}="2HE4 BDC";
$resn_exch{"HD21 BDC"}="1HD2 BDC"; $resn_exch{"HD22 BDC"}="2HD2 BDC"; $resn_exch{"HD31 BDC"}="1HD3 BDC";
$resn_exch{"HD32 BDC"}="2HD3 BDC"; $resn_exch{"HD71 BDC"}="1HD7 BDC"; $resn_exch{"HD72 BDC"}="2HD7 BDC";
$resn_exch{"HD73 BDC"}="3HD7 BDC"; $resn_exch{"HD81 BDC"}="1HD8 BDC"; $resn_exch{"HD82 BDC"}="2HD8 BDC";
$resn_exch{"HD83 BDC"}="3HD8 BDC"; $resn_exch{"H4'1 BDC"}="1H4' BDC"; $resn_exch{"H4'2 BDC"}="2H4' BDC";
$resn_exch{"H3'1 BDC"}="1H3' BDC"; $resn_exch{"H3'2 BDC"}="2H3' BDC"; $resn_exch{"H2'1 BDC"}="1H2' BDC";
$resn_exch{"H2'2 BDC"}="2H2' BDC"; $resn_exch{"H1'1 BDC"}="1H1' BDC"; $resn_exch{"H1'2 BDC"}="2H1' BDC";
$resn_exch{" H9' BDC"}="'H9  BDC"; $resn_exch{" H5' BDC"}="'H5  BDC"; $resn_exch{" H6' BDC"}="'H6  BDC";
$resn_exch{" H7' BDC"}="'H7  BDC"; $resn_exch{" H8' BDC"}="'H8  BDC"; $resn_exch{" H1' BDC"}="'H1  BDC";
$resn_exch{" H2' BDC"}="'H2  BDC"; $resn_exch{" H3' BDC"}="'H3  BDC"; $resn_exch{"HD1' BDC"}="'HD1 BDC";
$resn_exch{"H'21 BDC"}="1H'2 BDC"; $resn_exch{"H'22 BDC"}="2H'2 BDC"; $resn_exch{"H'31 BDC"}="1H'3 BDC";
$resn_exch{"H'32 BDC"}="2H'3 BDC"; $resn_exch{"H'71 BDC"}="1H'7 BDC"; $resn_exch{"H'72 BDC"}="2H'7 BDC";
$resn_exch{"H'73 BDC"}="3H'7 BDC"; $resn_exch{"H'81 BDC"}="1H'8 BDC"; $resn_exch{"H'82 BDC"}="2H'8 BDC";
$resn_exch{"H'83 BDC"}="3H'8 BDC"; $resn_exch{" H11 BDD"}="1H1  BDD"; $resn_exch{" H12 BDD"}="2H1  BDD";
$resn_exch{" H21 BDD"}="1H2  BDD"; $resn_exch{" H22 BDD"}="2H2  BDD"; $resn_exch{" H31 BDD"}="1H3  BDD";
$resn_exch{" H32 BDD"}="2H3  BDD"; $resn_exch{" H41 BDD"}="1H4  BDD"; $resn_exch{" H42 BDD"}="2H4  BDD";
$resn_exch{" H51 BDD"}="1H5  BDD"; $resn_exch{" H52 BDD"}="2H5  BDD"; $resn_exch{" H61 BDD"}="1H6  BDD";
$resn_exch{" H62 BDD"}="2H6  BDD"; $resn_exch{" H71 BDD"}="1H7  BDD"; $resn_exch{" H72 BDD"}="2H7  BDD";
$resn_exch{" H81 BDD"}="1H8  BDD"; $resn_exch{" H82 BDD"}="2H8  BDD"; $resn_exch{" H91 BDD"}="1H9  BDD";
$resn_exch{" H92 BDD"}="2H9  BDD"; $resn_exch{"H101 BDD"}="1H10 BDD"; $resn_exch{"H102 BDD"}="2H10 BDD";
$resn_exch{"H111 BDD"}="1H11 BDD"; $resn_exch{"H112 BDD"}="2H11 BDD"; $resn_exch{"H121 BDD"}="1H12 BDD";
$resn_exch{"H122 BDD"}="2H12 BDD"; $resn_exch{" H11 BDF"}="1H1  BDF"; $resn_exch{" H12 BDF"}="2H1  BDF";
$resn_exch{" H61 BDF"}="1H6  BDF"; $resn_exch{" H62 BDF"}="2H6  BDF"; $resn_exch{"HN21 BDG"}="1HN2 BDG";
$resn_exch{"HN22 BDG"}="2HN2 BDG"; $resn_exch{" H61 BDG"}="1H6  BDG"; $resn_exch{" H62 BDG"}="2H6  BDG";
$resn_exch{"HN61 BDG"}="1HN6 BDG"; $resn_exch{"HN62 BDG"}="2HN6 BDG"; $resn_exch{"HN11 BDH"}="1HN1 BDH";
$resn_exch{"HN12 BDH"}="2HN1 BDH"; $resn_exch{" H21 BDH"}="1H2  BDH"; $resn_exch{" H22 BDH"}="2H2  BDH";
$resn_exch{" HB1 BDH"}="1HB  BDH"; $resn_exch{" HB2 BDH"}="2HB  BDH"; $resn_exch{"HC61 BDI"}="1HC6 BDI";
$resn_exch{"HC62 BDI"}="2HC6 BDI"; $resn_exch{"HC71 BDI"}="1HC7 BDI"; $resn_exch{"HC72 BDI"}="2HC7 BDI";
$resn_exch{"HC81 BDI"}="1HC8 BDI"; $resn_exch{"HC82 BDI"}="2HC8 BDI"; $resn_exch{"HC91 BDI"}="1HC9 BDI";
$resn_exch{"HC92 BDI"}="2HC9 BDI"; $resn_exch{"HC93 BDI"}="3HC9 BDI"; $resn_exch{" H51 BDK"}="1H5  BDK";
$resn_exch{" H52 BDK"}="2H5  BDK"; $resn_exch{" H53 BDK"}="3H5  BDK"; $resn_exch{" H61 BDK"}="1H6  BDK";
$resn_exch{" H62 BDK"}="2H6  BDK"; $resn_exch{" H63 BDK"}="3H6  BDK"; $resn_exch{" H81 BDK"}="1H8  BDK";
$resn_exch{" H82 BDK"}="2H8  BDK"; $resn_exch{"H181 BDK"}="1H18 BDK"; $resn_exch{"H182 BDK"}="2H18 BDK";
$resn_exch{"H191 BDK"}="1H19 BDK"; $resn_exch{"H192 BDK"}="2H19 BDK"; $resn_exch{"H201 BDK"}="1H20 BDK";
$resn_exch{"H202 BDK"}="2H20 BDK"; $resn_exch{"H211 BDK"}="1H21 BDK"; $resn_exch{"H212 BDK"}="2H21 BDK";
$resn_exch{"H221 BDK"}="1H22 BDK"; $resn_exch{"H222 BDK"}="2H22 BDK"; $resn_exch{"H231 BDK"}="1H23 BDK";
$resn_exch{"H232 BDK"}="2H23 BDK"; $resn_exch{"HN41 BDK"}="1HN4 BDK"; $resn_exch{"HN42 BDK"}="2HN4 BDK";
$resn_exch{"HN21 BDM"}="1HN2 BDM"; $resn_exch{"HN22 BDM"}="2HN2 BDM"; $resn_exch{"HN41 BDM"}="1HN4 BDM";
$resn_exch{"HN42 BDM"}="2HN4 BDM"; $resn_exch{" H71 BDM"}="1H7  BDM"; $resn_exch{" H72 BDM"}="2H7  BDM";
$resn_exch{"H141 BDM"}="1H14 BDM"; $resn_exch{"H142 BDM"}="2H14 BDM"; $resn_exch{"H143 BDM"}="3H14 BDM";
$resn_exch{"H151 BDM"}="1H15 BDM"; $resn_exch{"H152 BDM"}="2H15 BDM"; $resn_exch{"H161 BDM"}="1H16 BDM";
$resn_exch{"H162 BDM"}="2H16 BDM"; $resn_exch{"H171 BDM"}="1H17 BDM"; $resn_exch{"H172 BDM"}="2H17 BDM";
$resn_exch{"H191 BDM"}="1H19 BDM"; $resn_exch{"H192 BDM"}="2H19 BDM"; $resn_exch{"H201 BDM"}="1H20 BDM";
$resn_exch{"H202 BDM"}="2H20 BDM"; $resn_exch{"HN21 BDN"}="1HN2 BDN"; $resn_exch{"HN22 BDN"}="2HN2 BDN";
$resn_exch{" H51 BDR"}="1H5  BDR"; $resn_exch{" H52 BDR"}="2H5  BDR"; $resn_exch{"HC11 BDS"}="1HC1 BDS";
$resn_exch{"HXT2 BDS"}="2HXT BDS"; $resn_exch{"HC51 BDS"}="1HC5 BDS"; $resn_exch{"HC52 BDS"}="2HC5 BDS";
$resn_exch{"HC71 BDS"}="1HC7 BDS"; $resn_exch{"HC81 BDS"}="1HC8 BDS"; $resn_exch{"H111 BDS"}="1H11 BDS";
$resn_exch{"H112 BDS"}="2H11 BDS"; $resn_exch{"H141 BDS"}="1H14 BDS"; $resn_exch{"H151 BDS"}="1H15 BDS";
$resn_exch{"HXT1 BDS"}="1HXT BDS"; $resn_exch{"H191 BDS"}="1H19 BDS"; $resn_exch{"H192 BDS"}="2H19 BDS";
$resn_exch{"H211 BDS"}="1H21 BDS"; $resn_exch{"H221 BDS"}="1H22 BDS"; $resn_exch{"H251 BDS"}="1H25 BDS";
$resn_exch{"H252 BDS"}="2H25 BDS"; $resn_exch{"H281 BDS"}="1H28 BDS"; $resn_exch{"HO21 BDU"}="1HO2 BDU";
$resn_exch{"HO22 BDU"}="2HO2 BDU"; $resn_exch{"HO23 BDU"}="3HO2 BDU"; $resn_exch{"HX31 BDU"}="1HX3 BDU";
$resn_exch{"HX32 BDU"}="2HX3 BDU"; $resn_exch{"H111 BDU"}="1H11 BDU"; $resn_exch{"H112 BDU"}="2H11 BDU";
$resn_exch{"H113 BDU"}="3H11 BDU"; $resn_exch{"HX91 BDU"}="1HX9 BDU"; $resn_exch{"HX92 BDU"}="2HX9 BDU";
$resn_exch{"HOP2 BE1"}="2HOP BE1"; $resn_exch{"HOP3 BE1"}="3HOP BE1"; $resn_exch{"HN21 BE2"}="1HN2 BE2";
$resn_exch{"HN22 BE2"}="2HN2 BE2"; $resn_exch{"H131 BE3"}="1H13 BE3"; $resn_exch{"H132 BE3"}="2H13 BE3";
$resn_exch{"H231 BE3"}="1H23 BE3"; $resn_exch{"H232 BE3"}="2H23 BE3"; $resn_exch{"H381 BE3"}="1H38 BE3";
$resn_exch{"H382 BE3"}="2H38 BE3"; $resn_exch{"H481 BE3"}="1H48 BE3"; $resn_exch{"H482 BE3"}="2H48 BE3";
$resn_exch{"H131 BE4"}="1H13 BE4"; $resn_exch{"H132 BE4"}="2H13 BE4"; $resn_exch{"H231 BE4"}="1H23 BE4";
$resn_exch{"H232 BE4"}="2H23 BE4"; $resn_exch{"H381 BE4"}="1H38 BE4"; $resn_exch{"H382 BE4"}="2H38 BE4";
$resn_exch{"H481 BE4"}="1H48 BE4"; $resn_exch{"H482 BE4"}="2H48 BE4"; $resn_exch{"H131 BE5"}="1H13 BE5";
$resn_exch{"H132 BE5"}="2H13 BE5"; $resn_exch{"H231 BE5"}="1H23 BE5"; $resn_exch{"H232 BE5"}="2H23 BE5";
$resn_exch{"H381 BE5"}="1H38 BE5"; $resn_exch{"H382 BE5"}="2H38 BE5"; $resn_exch{"H481 BE5"}="1H48 BE5";
$resn_exch{"H482 BE5"}="2H48 BE5"; $resn_exch{"H131 BE6"}="1H13 BE6"; $resn_exch{"H132 BE6"}="2H13 BE6";
$resn_exch{"H231 BE6"}="1H23 BE6"; $resn_exch{"H232 BE6"}="2H23 BE6"; $resn_exch{"H381 BE6"}="1H38 BE6";
$resn_exch{"H382 BE6"}="2H38 BE6"; $resn_exch{"H481 BE6"}="1H48 BE6"; $resn_exch{"H482 BE6"}="2H48 BE6";
$resn_exch{" H21 BEA"}="1H2  BEA"; $resn_exch{" H22 BEA"}="2H2  BEA"; $resn_exch{" H23 BEA"}="3H2  BEA";
$resn_exch{"H321 BEB"}="1H32 BEB"; $resn_exch{"H322 BEB"}="2H32 BEB"; $resn_exch{"H221 BEB"}="1H22 BEB";
$resn_exch{"H222 BEB"}="2H22 BEB"; $resn_exch{"H581 BEB"}="1H58 BEB"; $resn_exch{"H582 BEB"}="2H58 BEB";
$resn_exch{"H481 BEB"}="1H48 BEB"; $resn_exch{"H482 BEB"}="2H48 BEB"; $resn_exch{"H321 BEC"}="1H32 BEC";
$resn_exch{"H322 BEC"}="2H32 BEC"; $resn_exch{"H221 BEC"}="1H22 BEC"; $resn_exch{"H222 BEC"}="2H22 BEC";
$resn_exch{"H581 BEC"}="1H58 BEC"; $resn_exch{"H582 BEC"}="2H58 BEC"; $resn_exch{"H431 BEC"}="1H43 BEC";
$resn_exch{"H432 BEC"}="2H43 BEC"; $resn_exch{"H433 BEC"}="3H43 BEC"; $resn_exch{"H441 BEC"}="1H44 BEC";
$resn_exch{"H442 BEC"}="2H44 BEC"; $resn_exch{"H443 BEC"}="3H44 BEC"; $resn_exch{"H481 BEC"}="1H48 BEC";
$resn_exch{"H482 BEC"}="2H48 BEC"; $resn_exch{"H483 BEC"}="3H48 BEC"; $resn_exch{"H131 BED"}="1H13 BED";
$resn_exch{"H132 BED"}="2H13 BED"; $resn_exch{"H231 BED"}="1H23 BED"; $resn_exch{"H232 BED"}="2H23 BED";
$resn_exch{"H381 BED"}="1H38 BED"; $resn_exch{"H382 BED"}="2H38 BED"; $resn_exch{"H481 BED"}="1H48 BED";
$resn_exch{"H482 BED"}="2H48 BED"; $resn_exch{"H131 BEE"}="1H13 BEE"; $resn_exch{"H132 BEE"}="2H13 BEE";
$resn_exch{"H241 BEE"}="1H24 BEE"; $resn_exch{"H242 BEE"}="2H24 BEE"; $resn_exch{"H243 BEE"}="3H24 BEE";
$resn_exch{"H251 BEE"}="1H25 BEE"; $resn_exch{"H252 BEE"}="2H25 BEE"; $resn_exch{"H253 BEE"}="3H25 BEE";
$resn_exch{"H291 BEE"}="1H29 BEE"; $resn_exch{"H292 BEE"}="2H29 BEE"; $resn_exch{"H293 BEE"}="3H29 BEE";
$resn_exch{"H311 BEE"}="1H31 BEE"; $resn_exch{"H312 BEE"}="2H31 BEE"; $resn_exch{"H391 BEE"}="1H39 BEE";
$resn_exch{"H392 BEE"}="2H39 BEE"; $resn_exch{"H393 BEE"}="3H39 BEE"; $resn_exch{"H401 BEE"}="1H40 BEE";
$resn_exch{"H402 BEE"}="2H40 BEE"; $resn_exch{"H403 BEE"}="3H40 BEE"; $resn_exch{"H441 BEE"}="1H44 BEE";
$resn_exch{"H442 BEE"}="2H44 BEE"; $resn_exch{"H443 BEE"}="3H44 BEE"; $resn_exch{"H021 BEG"}="1H02 BEG";
$resn_exch{"H031 BEG"}="1H03 BEG"; $resn_exch{"H041 BEG"}="1H04 BEG"; $resn_exch{"H051 BEG"}="1H05 BEG";
$resn_exch{"H061 BEG"}="1H06 BEG"; $resn_exch{"H081 BEG"}="1H08 BEG"; $resn_exch{"H091 BEG"}="1H09 BEG";
$resn_exch{"H101 BEG"}="1H10 BEG"; $resn_exch{"H111 BEG"}="1H11 BEG"; $resn_exch{"H121 BEG"}="1H12 BEG";
$resn_exch{"H131 BEG"}="1H13 BEG"; $resn_exch{"H132 BEG"}="2H13 BEG"; $resn_exch{"H231 BEG"}="1H23 BEG";
$resn_exch{"H232 BEG"}="2H23 BEG"; $resn_exch{"H151 BEG"}="1H15 BEG"; $resn_exch{"H161 BEG"}="1H16 BEG";
$resn_exch{"H162 BEG"}="2H16 BEG"; $resn_exch{"H171 BEG"}="1H17 BEG"; $resn_exch{"H241 BEG"}="1H24 BEG";
$resn_exch{"H181 BEG"}="1H18 BEG"; $resn_exch{"H211 BEG"}="1H21 BEG"; $resn_exch{"H271 BEG"}="1H27 BEG";
$resn_exch{"H281 BEG"}="1H28 BEG"; $resn_exch{"H311 BEG"}="1H31 BEG"; $resn_exch{"H321 BEG"}="1H32 BEG";
$resn_exch{"H331 BEG"}="1H33 BEG"; $resn_exch{"H341 BEG"}="1H34 BEG"; $resn_exch{"H351 BEG"}="1H35 BEG";
$resn_exch{"H361 BEG"}="1H36 BEG"; $resn_exch{"H371 BEG"}="1H37 BEG"; $resn_exch{"H372 BEG"}="2H37 BEG";
$resn_exch{"H381 BEG"}="1H38 BEG"; $resn_exch{"H411 BEG"}="1H41 BEG"; $resn_exch{"H421 BEG"}="1H42 BEG";
$resn_exch{"H431 BEG"}="1H43 BEG"; $resn_exch{"H441 BEG"}="1H44 BEG"; $resn_exch{"H451 BEG"}="1H45 BEG";
$resn_exch{"H461 BEG"}="1H46 BEG"; $resn_exch{"H471 BEG"}="1H47 BEG"; $resn_exch{"H472 BEG"}="2H47 BEG";
$resn_exch{"H011 BEH"}="1H01 BEH"; $resn_exch{"H021 BEH"}="1H02 BEH"; $resn_exch{"H031 BEH"}="1H03 BEH";
$resn_exch{"H051 BEH"}="1H05 BEH"; $resn_exch{"H052 BEH"}="2H05 BEH"; $resn_exch{"H071 BEH"}="1H07 BEH";
$resn_exch{"H081 BEH"}="1H08 BEH"; $resn_exch{"H091 BEH"}="1H09 BEH"; $resn_exch{"H101 BEH"}="1H10 BEH";
$resn_exch{"H111 BEH"}="1H11 BEH"; $resn_exch{"H141 BEH"}="1H14 BEH"; $resn_exch{"H151 BEH"}="1H15 BEH";
$resn_exch{"H171 BEH"}="1H17 BEH"; $resn_exch{"H181 BEH"}="1H18 BEH"; $resn_exch{"H191 BEH"}="1H19 BEH";
$resn_exch{"H201 BEH"}="1H20 BEH"; $resn_exch{"H221 BEH"}="1H22 BEH"; $resn_exch{"H222 BEH"}="2H22 BEH";
$resn_exch{"H231 BEH"}="1H23 BEH"; $resn_exch{"H241 BEH"}="1H24 BEH"; $resn_exch{"H311 BEH"}="1H31 BEH";
$resn_exch{"H321 BEH"}="1H32 BEH"; $resn_exch{"H331 BEH"}="1H33 BEH"; $resn_exch{"H351 BEH"}="1H35 BEH";
$resn_exch{"H352 BEH"}="2H35 BEH"; $resn_exch{"H371 BEH"}="1H37 BEH"; $resn_exch{"H381 BEH"}="1H38 BEH";
$resn_exch{"H391 BEH"}="1H39 BEH"; $resn_exch{"H401 BEH"}="1H40 BEH"; $resn_exch{"H411 BEH"}="1H41 BEH";
$resn_exch{"H441 BEH"}="1H44 BEH"; $resn_exch{"H451 BEH"}="1H45 BEH"; $resn_exch{"H452 BEH"}="2H45 BEH";
$resn_exch{"H471 BEH"}="1H47 BEH"; $resn_exch{"H481 BEH"}="1H48 BEH"; $resn_exch{"H491 BEH"}="1H49 BEH";
$resn_exch{"H501 BEH"}="1H50 BEH"; $resn_exch{"H511 BEH"}="1H51 BEH"; $resn_exch{"H301 BEI"}="1H30 BEI";
$resn_exch{"H302 BEI"}="2H30 BEI"; $resn_exch{"H211 BEI"}="1H21 BEI"; $resn_exch{"H212 BEI"}="2H21 BEI";
$resn_exch{"H213 BEI"}="3H21 BEI"; $resn_exch{"H221 BEI"}="1H22 BEI"; $resn_exch{"H222 BEI"}="2H22 BEI";
$resn_exch{"H461 BEI"}="1H46 BEI"; $resn_exch{"H462 BEI"}="2H46 BEI"; $resn_exch{"H463 BEI"}="3H46 BEI";
$resn_exch{"H261 BEI"}="1H26 BEI"; $resn_exch{"H262 BEI"}="2H26 BEI"; $resn_exch{"H263 BEI"}="3H26 BEI";
$resn_exch{"H331 BEI"}="1H33 BEI"; $resn_exch{"H332 BEI"}="2H33 BEI"; $resn_exch{"H111 BEI"}="1H11 BEI";
$resn_exch{"H112 BEI"}="2H11 BEI"; $resn_exch{"H113 BEI"}="3H11 BEI"; $resn_exch{"H121 BEI"}="1H12 BEI";
$resn_exch{"H122 BEI"}="2H12 BEI"; $resn_exch{"H471 BEI"}="1H47 BEI"; $resn_exch{"H472 BEI"}="2H47 BEI";
$resn_exch{"H473 BEI"}="3H47 BEI"; $resn_exch{"H161 BEI"}="1H16 BEI"; $resn_exch{"H162 BEI"}="2H16 BEI";
$resn_exch{"H163 BEI"}="3H16 BEI"; $resn_exch{"H131 BEJ"}="1H13 BEJ"; $resn_exch{"H132 BEJ"}="2H13 BEJ";
$resn_exch{"H241 BEJ"}="1H24 BEJ"; $resn_exch{"H242 BEJ"}="2H24 BEJ"; $resn_exch{"H243 BEJ"}="3H24 BEJ";
$resn_exch{"H251 BEJ"}="1H25 BEJ"; $resn_exch{"H252 BEJ"}="2H25 BEJ"; $resn_exch{"H253 BEJ"}="3H25 BEJ";
$resn_exch{"H291 BEJ"}="1H29 BEJ"; $resn_exch{"H292 BEJ"}="2H29 BEJ"; $resn_exch{"H293 BEJ"}="3H29 BEJ";
$resn_exch{"H311 BEJ"}="1H31 BEJ"; $resn_exch{"H312 BEJ"}="2H31 BEJ"; $resn_exch{"H391 BEJ"}="1H39 BEJ";
$resn_exch{"H392 BEJ"}="2H39 BEJ"; $resn_exch{"H393 BEJ"}="3H39 BEJ"; $resn_exch{"H401 BEJ"}="1H40 BEJ";
$resn_exch{"H402 BEJ"}="2H40 BEJ"; $resn_exch{"H403 BEJ"}="3H40 BEJ"; $resn_exch{"H441 BEJ"}="1H44 BEJ";
$resn_exch{"H442 BEJ"}="2H44 BEJ"; $resn_exch{"H443 BEJ"}="3H44 BEJ"; $resn_exch{" HN1 BEK"}="1HN  BEK";
$resn_exch{" HN2 BEK"}="2HN  BEK"; $resn_exch{" H81 BEK"}="1H8  BEK"; $resn_exch{" H82 BEK"}="2H8  BEK";
$resn_exch{" H83 BEK"}="3H8  BEK"; $resn_exch{" H71 BEL"}="1H7  BEL"; $resn_exch{" H72 BEL"}="2H7  BEL";
$resn_exch{" H81 BEL"}="1H8  BEL"; $resn_exch{" H82 BEL"}="2H8  BEL"; $resn_exch{" H91 BEL"}="1H9  BEL";
$resn_exch{" H92 BEL"}="2H9  BEL"; $resn_exch{"H101 BEL"}="1H10 BEL"; $resn_exch{"H102 BEL"}="2H10 BEL";
$resn_exch{"H121 BEL"}="1H12 BEL"; $resn_exch{"H122 BEL"}="2H12 BEL"; $resn_exch{"HN21 BEN"}="1HN2 BEN";
$resn_exch{"HN22 BEN"}="2HN2 BEN"; $resn_exch{" H41 BEO"}="1H4  BEO"; $resn_exch{" H42 BEO"}="2H4  BEO";
$resn_exch{" H43 BEO"}="3H4  BEO"; $resn_exch{" H11 BEP"}="1H1  BEP"; $resn_exch{" H12 BEP"}="2H1  BEP";
$resn_exch{" H21 BEP"}="1H2  BEP"; $resn_exch{" H31 BEP"}="1H3  BEP"; $resn_exch{" H32 BEP"}="2H3  BEP";
$resn_exch{" H41 BEP"}="1H4  BEP"; $resn_exch{" H42 BEP"}="2H4  BEP"; $resn_exch{" H51 BEP"}="1H5  BEP";
$resn_exch{" H61 BEP"}="1H6  BEP"; $resn_exch{" H62 BEP"}="2H6  BEP"; $resn_exch{" H63 BEP"}="3H6  BEP";
$resn_exch{" H71 BEP"}="1H7  BEP"; $resn_exch{" H72 BEP"}="2H7  BEP"; $resn_exch{" H73 BEP"}="3H7  BEP";
$resn_exch{" H81 BEP"}="1H8  BEP"; $resn_exch{" H82 BEP"}="2H8  BEP"; $resn_exch{" H91 BEP"}="1H9  BEP";
$resn_exch{" H92 BEP"}="2H9  BEP"; $resn_exch{"H101 BEP"}="1H10 BEP"; $resn_exch{"H102 BEP"}="2H10 BEP";
$resn_exch{"H111 BEP"}="1H11 BEP"; $resn_exch{"H112 BEP"}="2H11 BEP"; $resn_exch{"H121 BEP"}="1H12 BEP";
$resn_exch{"H122 BEP"}="2H12 BEP"; $resn_exch{"H141 BEP"}="1H14 BEP"; $resn_exch{"H151 BEP"}="1H15 BEP";
$resn_exch{"H161 BEP"}="1H16 BEP"; $resn_exch{"H171 BEP"}="1H17 BEP"; $resn_exch{"H181 BEP"}="1H18 BEP";
$resn_exch{"H201 BEP"}="1H20 BEP"; $resn_exch{"H211 BEP"}="1H21 BEP"; $resn_exch{"H221 BEP"}="1H22 BEP";
$resn_exch{"H231 BEP"}="1H23 BEP"; $resn_exch{"H241 BEP"}="1H24 BEP"; $resn_exch{" HB1 BEQ"}="1HB  BEQ";
$resn_exch{" HB2 BEQ"}="2HB  BEQ"; $resn_exch{" HB3 BEQ"}="3HB  BEQ"; $resn_exch{" HG1 BEQ"}="1HG  BEQ";
$resn_exch{" HG2 BEQ"}="2HG  BEQ"; $resn_exch{" HG3 BEQ"}="3HG  BEQ"; $resn_exch{" HD1 BEQ"}="1HD  BEQ";
$resn_exch{" HD2 BEQ"}="2HD  BEQ"; $resn_exch{" H11 BEQ"}="1H1  BEQ"; $resn_exch{" H12 BEQ"}="2H1  BEQ";
$resn_exch{" H21 BEQ"}="1H2  BEQ"; $resn_exch{" H22 BEQ"}="2H2  BEQ"; $resn_exch{" H31 BEQ"}="1H3  BEQ";
$resn_exch{" H32 BEQ"}="2H3  BEQ"; $resn_exch{" H51 BEQ"}="1H5  BEQ"; $resn_exch{" H52 BEQ"}="2H5  BEQ";
$resn_exch{" H61 BEQ"}="1H6  BEQ"; $resn_exch{" H62 BEQ"}="2H6  BEQ"; $resn_exch{" H71 BEQ"}="1H7  BEQ";
$resn_exch{" H72 BEQ"}="2H7  BEQ"; $resn_exch{" H81 BEQ"}="1H8  BEQ"; $resn_exch{" H82 BEQ"}="2H8  BEQ";
$resn_exch{" H91 BEQ"}="1H9  BEQ"; $resn_exch{" H92 BEQ"}="2H9  BEQ"; $resn_exch{"H101 BEQ"}="1H10 BEQ";
$resn_exch{"H102 BEQ"}="2H10 BEQ"; $resn_exch{"H111 BEQ"}="1H11 BEQ"; $resn_exch{"H112 BEQ"}="2H11 BEQ";
$resn_exch{"H121 BEQ"}="1H12 BEQ"; $resn_exch{"H122 BEQ"}="2H12 BEQ"; $resn_exch{"H131 BEQ"}="1H13 BEQ";
$resn_exch{"H132 BEQ"}="2H13 BEQ"; $resn_exch{"H141 BEQ"}="1H14 BEQ"; $resn_exch{"H142 BEQ"}="2H14 BEQ";
$resn_exch{"H151 BEQ"}="1H15 BEQ"; $resn_exch{"H152 BEQ"}="2H15 BEQ"; $resn_exch{"H153 BEQ"}="3H15 BEQ";
$resn_exch{" H31 BER"}="1H3  BER"; $resn_exch{" H51 BER"}="1H5  BER"; $resn_exch{" H61 BER"}="1H6  BER";
$resn_exch{" H71 BER"}="1H7  BER"; $resn_exch{" H72 BER"}="2H7  BER"; $resn_exch{" H91 BER"}="1H9  BER";
$resn_exch{"H101 BER"}="1H10 BER"; $resn_exch{"H102 BER"}="2H10 BER"; $resn_exch{"H131 BER"}="1H13 BER";
$resn_exch{"H161 BER"}="1H16 BER"; $resn_exch{"H171 BER"}="1H17 BER"; $resn_exch{"H172 BER"}="2H17 BER";
$resn_exch{"H191 BER"}="1H19 BER"; $resn_exch{"H192 BER"}="2H19 BER"; $resn_exch{"H193 BER"}="3H19 BER";
$resn_exch{"H201 BER"}="1H20 BER"; $resn_exch{"H202 BER"}="2H20 BER"; $resn_exch{"H203 BER"}="3H20 BER";
$resn_exch{"HN21 BES"}="1HN2 BES"; $resn_exch{"HN22 BES"}="2HN2 BES"; $resn_exch{" H61 BES"}="1H6  BES";
$resn_exch{" H62 BES"}="2H6  BES"; $resn_exch{"H131 BES"}="1H13 BES"; $resn_exch{"H132 BES"}="2H13 BES";
$resn_exch{"H151 BES"}="1H15 BES"; $resn_exch{"H152 BES"}="2H15 BES"; $resn_exch{"H153 BES"}="3H15 BES";
$resn_exch{"H161 BES"}="1H16 BES"; $resn_exch{"H162 BES"}="2H16 BES"; $resn_exch{"H163 BES"}="3H16 BES";
$resn_exch{" HA1 BET"}="1HA  BET"; $resn_exch{" HA2 BET"}="2HA  BET"; $resn_exch{" H11 BET"}="1H1  BET";
$resn_exch{" H12 BET"}="2H1  BET"; $resn_exch{" H13 BET"}="3H1  BET"; $resn_exch{" H21 BET"}="1H2  BET";
$resn_exch{" H22 BET"}="2H2  BET"; $resn_exch{" H23 BET"}="3H2  BET"; $resn_exch{" H31 BET"}="1H3  BET";
$resn_exch{" H32 BET"}="2H3  BET"; $resn_exch{" H33 BET"}="3H3  BET"; $resn_exch{"H'C1 BEU"}="1H'C BEU";
$resn_exch{"H'C2 BEU"}="2H'C BEU"; $resn_exch{"H3N1 BEU"}="1H3N BEU"; $resn_exch{"H3N2 BEU"}="2H3N BEU";
$resn_exch{"HM41 BEU"}="1HM4 BEU"; $resn_exch{"HM42 BEU"}="2HM4 BEU"; $resn_exch{"HM43 BEU"}="3HM4 BEU";
$resn_exch{"HD21 BFA"}="1HD2 BFA"; $resn_exch{"HD22 BFA"}="2HD2 BFA"; $resn_exch{"HD31 BFA"}="1HD3 BFA";
$resn_exch{"HD32 BFA"}="2HD3 BFA"; $resn_exch{"HD71 BFA"}="1HD7 BFA"; $resn_exch{"HD72 BFA"}="2HD7 BFA";
$resn_exch{"HD73 BFA"}="3HD7 BFA"; $resn_exch{"HD81 BFA"}="1HD8 BFA"; $resn_exch{"HD82 BFA"}="2HD8 BFA";
$resn_exch{"HD83 BFA"}="3HD8 BFA"; $resn_exch{"HAA1 BFB"}="1HAA BFB"; $resn_exch{"HAA2 BFB"}="2HAA BFB";
$resn_exch{"HAO1 BFB"}="1HAO BFB"; $resn_exch{"HAO2 BFB"}="2HAO BFB"; $resn_exch{"HAM1 BFB"}="1HAM BFB";
$resn_exch{"HAM2 BFB"}="2HAM BFB"; $resn_exch{"HAN1 BFB"}="1HAN BFB"; $resn_exch{"HAN2 BFB"}="2HAN BFB";
$resn_exch{"HAP1 BFB"}="1HAP BFB"; $resn_exch{"HAP2 BFB"}="2HAP BFB"; $resn_exch{"HAP3 BFB"}="3HAP BFB";
$resn_exch{" HA1 BFC"}="1HA  BFC"; $resn_exch{" HA2 BFC"}="2HA  BFC"; $resn_exch{" HC1 BFC"}="1HC  BFC";
$resn_exch{" HC2 BFC"}="2HC  BFC"; $resn_exch{" HD1 BFC"}="1HD  BFC"; $resn_exch{" HD2 BFC"}="2HD  BFC";
$resn_exch{" HE1 BFC"}="1HE  BFC"; $resn_exch{" HE2 BFC"}="2HE  BFC"; $resn_exch{" HF1 BFC"}="1HF  BFC";
$resn_exch{" HF2 BFC"}="2HF  BFC"; $resn_exch{" HG1 BFC"}="1HG  BFC"; $resn_exch{" HG2 BFC"}="2HG  BFC";
$resn_exch{" HH1 BFC"}="1HH  BFC"; $resn_exch{" HH2 BFC"}="2HH  BFC"; $resn_exch{" HI1 BFC"}="1HI  BFC";
$resn_exch{" HI2 BFC"}="2HI  BFC"; $resn_exch{" HJ1 BFC"}="1HJ  BFC"; $resn_exch{" HJ2 BFC"}="2HJ  BFC";
$resn_exch{" HK1 BFC"}="1HK  BFC"; $resn_exch{" HK2 BFC"}="2HK  BFC"; $resn_exch{" HL1 BFC"}="1HL  BFC";
$resn_exch{" HL2 BFC"}="2HL  BFC"; $resn_exch{" HM1 BFC"}="1HM  BFC"; $resn_exch{" HM2 BFC"}="2HM  BFC";
$resn_exch{" HM3 BFC"}="3HM  BFC"; $resn_exch{" H2  BFD"}=" HN2 BFD"; $resn_exch{" HB2 BFD"}="1HB  BFD";
$resn_exch{" HB3 BFD"}="2HB  BFD"; $resn_exch{"H181 BFI"}="1H18 BFI"; $resn_exch{"H182 BFI"}="2H18 BFI";
$resn_exch{"H231 BFI"}="1H23 BFI"; $resn_exch{"H232 BFI"}="2H23 BFI"; $resn_exch{" H71 BFL"}="1H7  BFL";
$resn_exch{" H72 BFL"}="2H7  BFL"; $resn_exch{" H73 BFL"}="3H7  BFL"; $resn_exch{"HOP2 BFP"}="2HOP BFP";
$resn_exch{"HOP3 BFP"}="3HOP BFP"; $resn_exch{" H11 BFP"}="1H1  BFP"; $resn_exch{" H12 BFP"}="2H1  BFP";
$resn_exch{" H61 BFP"}="1H6  BFP"; $resn_exch{" H62 BFP"}="2H6  BFP"; $resn_exch{"HOP5 BFP"}="5HOP BFP";
$resn_exch{"HOP6 BFP"}="6HOP BFP"; $resn_exch{"H161 BFQ"}="1H16 BFQ"; $resn_exch{"H162 BFQ"}="2H16 BFQ";
$resn_exch{"H191 BFQ"}="1H19 BFQ"; $resn_exch{"H192 BFQ"}="2H19 BFQ"; $resn_exch{"H231 BFQ"}="1H23 BFQ";
$resn_exch{"H232 BFQ"}="2H23 BFQ"; $resn_exch{"H261 BFQ"}="1H26 BFQ"; $resn_exch{"H262 BFQ"}="2H26 BFQ";
$resn_exch{"H291 BFQ"}="1H29 BFQ"; $resn_exch{"H292 BFQ"}="2H29 BFQ"; $resn_exch{"H321 BFQ"}="1H32 BFQ";
$resn_exch{"H322 BFQ"}="2H32 BFQ"; $resn_exch{"H351 BFQ"}="1H35 BFQ"; $resn_exch{"H352 BFQ"}="2H35 BFQ";
$resn_exch{"H353 BFQ"}="3H35 BFQ"; $resn_exch{"H391 BFQ"}="1H39 BFQ"; $resn_exch{"H392 BFQ"}="2H39 BFQ";
$resn_exch{"H393 BFQ"}="3H39 BFQ"; $resn_exch{"HE21 BFS"}="1HE2 BFS"; $resn_exch{"HE22 BFS"}="2HE2 BFS";
$resn_exch{"HE23 BFS"}="3HE2 BFS"; $resn_exch{"HM21 BFT"}="1HM2 BFT"; $resn_exch{"HM22 BFT"}="2HM2 BFT";
$resn_exch{"HM23 BFT"}="3HM2 BFT"; $resn_exch{"HM41 BFT"}="1HM4 BFT"; $resn_exch{"HM42 BFT"}="2HM4 BFT";
$resn_exch{"HM43 BFT"}="3HM4 BFT"; $resn_exch{" H61 BFT"}="1H6  BFT"; $resn_exch{" H62 BFT"}="2H6  BFT";
$resn_exch{" H71 BFT"}="1H7  BFT"; $resn_exch{" H72 BFT"}="2H7  BFT"; $resn_exch{"H4A1 BFT"}="1H4A BFT";
$resn_exch{"H4A2 BFT"}="2H4A BFT"; $resn_exch{"H7A1 BFT"}="1H7A BFT"; $resn_exch{"H7A2 BFT"}="2H7A BFT";
$resn_exch{"H131 BFU"}="1H13 BFU"; $resn_exch{"H132 BFU"}="2H13 BFU"; $resn_exch{"H251 BFU"}="1H25 BFU";
$resn_exch{"H252 BFU"}="2H25 BFU"; $resn_exch{"H261 BFU"}="1H26 BFU"; $resn_exch{"H262 BFU"}="2H26 BFU";
$resn_exch{"H263 BFU"}="3H26 BFU"; $resn_exch{" H41 BG3"}="1H4  BG3"; $resn_exch{" H42 BG3"}="2H4  BG3";
$resn_exch{" H61 BG3"}="1H6  BG3"; $resn_exch{" H62 BG3"}="2H6  BG3"; $resn_exch{" H63 BG3"}="3H6  BG3";
$resn_exch{" H41 BG4"}="1H4  BG4"; $resn_exch{" H42 BG4"}="2H4  BG4"; $resn_exch{" H61 BG4"}="1H6  BG4";
$resn_exch{" H62 BG4"}="2H6  BG4"; $resn_exch{" H63 BG4"}="3H6  BG4"; $resn_exch{" H71 BG4"}="1H7  BG4";
$resn_exch{" H72 BG4"}="2H7  BG4"; $resn_exch{" H73 BG4"}="3H7  BG4"; $resn_exch{" H41 BG5"}="1H4  BG5";
$resn_exch{" H42 BG5"}="2H4  BG5"; $resn_exch{" H61 BG5"}="1H6  BG5"; $resn_exch{" H62 BG5"}="2H6  BG5";
$resn_exch{" H63 BG5"}="3H6  BG5"; $resn_exch{"HC61 BG6"}="1HC6 BG6"; $resn_exch{"HC62 BG6"}="2HC6 BG6";
$resn_exch{"H1O1 BG6"}="1H1O BG6"; $resn_exch{"H2O2 BG6"}="2H2O BG6"; $resn_exch{"H6C1 BGC"}="1H6C BGC";
$resn_exch{"H6C2 BGC"}="2H6C BGC"; $resn_exch{" H31 BGD"}="1H3  BGD"; $resn_exch{" H32 BGD"}="2H3  BGD";
$resn_exch{"H41A BGD"}="1H4  BGD"; $resn_exch{"H42A BGD"}="2H4  BGD"; $resn_exch{"H221 BGD"}="1H22 BGD";
$resn_exch{"H222 BGD"}="2H22 BGD"; $resn_exch{"H381 BGD"}="1H38 BGD"; $resn_exch{"H382 BGD"}="2H38 BGD";
$resn_exch{"H541 BGD"}="1H54 BGD"; $resn_exch{"H542 BGD"}="2H54 BGD"; $resn_exch{"HN11 BGF"}="1HN1 BGF";
$resn_exch{"HN12 BGF"}="2HN1 BGF"; $resn_exch{"HN21 BGF"}="1HN2 BGF"; $resn_exch{"HN22 BGF"}="2HN2 BGF";
$resn_exch{"H1'1 BGF"}="1H1' BGF"; $resn_exch{"H1'2 BGF"}="2H1' BGF"; $resn_exch{"H2'1 BGF"}="1H2' BGF";
$resn_exch{"H2'2 BGF"}="2H2' BGF"; $resn_exch{" H71 BGG"}="1H7  BGG"; $resn_exch{" H72 BGG"}="2H7  BGG";
$resn_exch{"H1'1 BGL"}="1H1' BGL"; $resn_exch{"H1'2 BGL"}="2H1' BGL"; $resn_exch{"H2'1 BGL"}="1H2' BGL";
$resn_exch{"H2'2 BGL"}="2H2' BGL"; $resn_exch{"H3'1 BGL"}="1H3' BGL"; $resn_exch{"H3'2 BGL"}="2H3' BGL";
$resn_exch{"H4'1 BGL"}="1H4' BGL"; $resn_exch{"H4'2 BGL"}="2H4' BGL"; $resn_exch{"H5'1 BGL"}="1H5' BGL";
$resn_exch{"H5'2 BGL"}="2H5' BGL"; $resn_exch{"H6'1 BGL"}="1H6' BGL"; $resn_exch{"H6'2 BGL"}="2H6' BGL";
$resn_exch{"H7'1 BGL"}="1H7' BGL"; $resn_exch{"H7'2 BGL"}="2H7' BGL"; $resn_exch{"H8'1 BGL"}="1H8' BGL";
$resn_exch{"H8'2 BGL"}="2H8' BGL"; $resn_exch{"H8'3 BGL"}="3H8' BGL"; $resn_exch{"HOP2 BGM"}="2HOP BGM";
$resn_exch{" H5' BGM"}="1H5* BGM"; $resn_exch{"H5'' BGM"}="2H5* BGM"; $resn_exch{" H4' BGM"}=" H4* BGM";
$resn_exch{" H1' BGM"}=" H1* BGM"; $resn_exch{" H21 BGM"}="1H2  BGM"; $resn_exch{" H22 BGM"}="2H2  BGM";
$resn_exch{" H2' BGM"}="1H2* BGM"; $resn_exch{"H2'' BGM"}="2H2* BGM"; $resn_exch{" H3' BGM"}=" H3* BGM";
$resn_exch{"HO3' BGM"}="H3T  BGM"; $resn_exch{"HOP3 BGM"}="3HOP BGM"; $resn_exch{" H61 BGN"}="1H6  BGN";
$resn_exch{" H62 BGN"}="2H6  BGN"; $resn_exch{" H81 BGN"}="1H8  BGN"; $resn_exch{" H82 BGN"}="2H8  BGN";
$resn_exch{" H91 BGN"}="1H9  BGN"; $resn_exch{" H92 BGN"}="2H9  BGN"; $resn_exch{"H101 BGN"}="1H10 BGN";
$resn_exch{"H102 BGN"}="2H10 BGN"; $resn_exch{"H103 BGN"}="3H10 BGN"; $resn_exch{" H61 BGP"}="1H6  BGP";
$resn_exch{" H62 BGP"}="2H6  BGP"; $resn_exch{"HOP2 BGP"}="2HOP BGP"; $resn_exch{"HOP3 BGP"}="3HOP BGP";
$resn_exch{"H2'1 BGS"}="1H2* BGS"; $resn_exch{"H2'2 BGS"}="2H2* BGS"; $resn_exch{" H61 BGS"}="1H6  BGS";
$resn_exch{" H62 BGS"}="2H6  BGS"; $resn_exch{" HS1 BGS"}="1HS  BGS"; $resn_exch{" HS2 BGS"}="2HS  BGS";
$resn_exch{" HS3 BGS"}="3HS  BGS"; $resn_exch{" HB1 BGT"}="1HB  BGT"; $resn_exch{" HB2 BGT"}="2HB  BGT";
$resn_exch{" HG1 BGT"}="1HG  BGT"; $resn_exch{" HG2 BGT"}="2HG  BGT"; $resn_exch{"HE21 BGT"}="1HE2 BGT";
$resn_exch{"HE22 BGT"}="2HE2 BGT"; $resn_exch{" H11 BGT"}="1H1  BGT"; $resn_exch{" H12 BGT"}="2H1  BGT";
$resn_exch{" H13 BGT"}="3H1  BGT"; $resn_exch{" H21 BGT"}="1H2  BGT"; $resn_exch{" H22 BGT"}="2H2  BGT";
$resn_exch{" H23 BGT"}="3H2  BGT"; $resn_exch{" H31 BGT"}="1H3  BGT"; $resn_exch{" H32 BGT"}="2H3  BGT";
$resn_exch{" H33 BGT"}="3H3  BGT"; $resn_exch{" HN1 BGT"}="1HN  BGT"; $resn_exch{" HN2 BGT"}="2HN  BGT";
$resn_exch{"HC61 BGX"}="1HC6 BGX"; $resn_exch{"HC62 BGX"}="2HC6 BGX"; $resn_exch{"HC63 BGX"}="3HC6 BGX";
$resn_exch{" H11 BH0"}="1H1  BH0"; $resn_exch{" H21 BH0"}="1H2  BH0"; $resn_exch{" H22 BH0"}="2H2  BH0";
$resn_exch{" H44 BH0"}="4H4  BH0"; $resn_exch{" H55 BH0"}="5H5  BH0"; $resn_exch{" H66 BH0"}="6H6  BH0";
$resn_exch{" H77 BH0"}="7H7  BH0"; $resn_exch{" H88 BH0"}="8H8  BH0"; $resn_exch{" H99 BH0"}="9H9  BH0";
$resn_exch{"H101 BH0"}="1H10 BH0"; $resn_exch{"H102 BH0"}="2H10 BH0"; $resn_exch{"H141 BH0"}="1H14 BH0";
$resn_exch{"H142 BH0"}="2H14 BH0"; $resn_exch{"H151 BH0"}="1H15 BH0"; $resn_exch{"H152 BH0"}="2H15 BH0";
$resn_exch{"H117 BH0"}="17H1 BH0"; $resn_exch{"H118 BH0"}="18H1 BH0"; $resn_exch{"H220 BH0"}="20H2 BH0";
$resn_exch{"H221 BH0"}="21H2 BH0"; $resn_exch{"H223 BH0"}="23H2 BH0"; $resn_exch{"H224 BH0"}="24H2 BH0";
$resn_exch{"H226 BH0"}="26H2 BH0"; $resn_exch{"H227 BH0"}="27H2 BH0"; $resn_exch{"H331 BH0"}="31H3 BH0";
$resn_exch{"H332 BH0"}="32H3 BH0"; $resn_exch{"H334 BH0"}="34H3 BH0"; $resn_exch{"H113 BH0"}="13H1 BH0";
$resn_exch{"HED1 BH1"}="1HED BH1"; $resn_exch{"HED2 BH1"}="2HED BH1"; $resn_exch{"HED3 BH1"}="3HED BH1";
$resn_exch{"HMD1 BH1"}="1HMD BH1"; $resn_exch{"HMD2 BH1"}="2HMD BH1"; $resn_exch{"HMD3 BH1"}="3HMD BH1";
$resn_exch{"HAC1 BH1"}="1HAC BH1"; $resn_exch{"HAC2 BH1"}="2HAC BH1"; $resn_exch{"HBC1 BH1"}="1HBC BH1";
$resn_exch{"HBC2 BH1"}="2HBC BH1"; $resn_exch{"HBC3 BH1"}="3HBC BH1"; $resn_exch{"HMC1 BH1"}="1HMC BH1";
$resn_exch{"HMC2 BH1"}="2HMC BH1"; $resn_exch{"HMC3 BH1"}="3HMC BH1"; $resn_exch{"HBB1 BH1"}="1HBB BH1";
$resn_exch{"HBB2 BH1"}="2HBB BH1"; $resn_exch{"HBB3 BH1"}="3HBB BH1"; $resn_exch{"HMB1 BH1"}="1HMB BH1";
$resn_exch{"HMB2 BH1"}="2HMB BH1"; $resn_exch{"HMB3 BH1"}="3HMB BH1"; $resn_exch{"HMA1 BH1"}="1HMA BH1";
$resn_exch{"HMA2 BH1"}="2HMA BH1"; $resn_exch{"HMA3 BH1"}="3HMA BH1"; $resn_exch{"HAA1 BH1"}="1HAA BH1";
$resn_exch{"HAA2 BH1"}="2HAA BH1"; $resn_exch{"HBA1 BH1"}="1HBA BH1"; $resn_exch{"HBA2 BH1"}="2HBA BH1";
$resn_exch{"H1C1 BH1"}="1H1C BH1"; $resn_exch{"H1C2 BH1"}="2H1C BH1"; $resn_exch{"H4C1 BH1"}="1H4C BH1";
$resn_exch{"H4C2 BH1"}="2H4C BH1"; $resn_exch{"H4C3 BH1"}="3H4C BH1"; $resn_exch{"H5C1 BH1"}="1H5C BH1";
$resn_exch{"H5C2 BH1"}="2H5C BH1"; $resn_exch{"H6C1 BH1"}="1H6C BH1"; $resn_exch{"H6C2 BH1"}="2H6C BH1";
$resn_exch{"H7C1 BH1"}="1H7C BH1"; $resn_exch{"H7C2 BH1"}="2H7C BH1"; $resn_exch{"H9C1 BH1"}="1H9C BH1";
$resn_exch{"H9C2 BH1"}="2H9C BH1"; $resn_exch{"H9C3 BH1"}="3H9C BH1"; $resn_exch{"H101 BH1"}="1H10 BH1";
$resn_exch{"H102 BH1"}="2H10 BH1"; $resn_exch{"H111 BH1"}="1H11 BH1"; $resn_exch{"H112 BH1"}="2H11 BH1";
$resn_exch{"H121 BH1"}="1H12 BH1"; $resn_exch{"H122 BH1"}="2H12 BH1"; $resn_exch{"H141 BH1"}="1H14 BH1";
$resn_exch{"H142 BH1"}="2H14 BH1"; $resn_exch{"H143 BH1"}="3H14 BH1"; $resn_exch{"H151 BH1"}="1H15 BH1";
$resn_exch{"H152 BH1"}="2H15 BH1"; $resn_exch{"H161 BH1"}="1H16 BH1"; $resn_exch{"H162 BH1"}="2H16 BH1";
$resn_exch{"H171 BH1"}="1H17 BH1"; $resn_exch{"H172 BH1"}="2H17 BH1"; $resn_exch{"H191 BH1"}="1H19 BH1";
$resn_exch{"H192 BH1"}="2H19 BH1"; $resn_exch{"H193 BH1"}="3H19 BH1"; $resn_exch{"H201 BH1"}="1H20 BH1";
$resn_exch{"H202 BH1"}="2H20 BH1"; $resn_exch{"H203 BH1"}="3H20 BH1"; $resn_exch{" H71 BH4"}="1H7  BH4";
$resn_exch{" H72 BH4"}="2H7  BH4"; $resn_exch{"H171 BH4"}="1H17 BH4"; $resn_exch{"H172 BH4"}="2H17 BH4";
$resn_exch{"H173 BH4"}="3H17 BH4"; $resn_exch{"H111 BH4"}="1H11 BH4"; $resn_exch{"H112 BH4"}="2H11 BH4";
$resn_exch{"H21A BH7"}="1H2  BH7"; $resn_exch{" H31 BH7"}="1H3  BH7"; $resn_exch{" H61 BH7"}="1H6  BH7";
$resn_exch{" H62 BH7"}="2H6  BH7"; $resn_exch{" H81 BH7"}="1H8  BH7"; $resn_exch{"HO2' BHA"}="'HO2 BHA";
$resn_exch{"HN41 BHA"}="1HN4 BHA"; $resn_exch{"HN42 BHA"}="2HN4 BHA"; $resn_exch{"HN41 BHB"}="1HN4 BHB";
$resn_exch{"HN42 BHB"}="2HN4 BHB"; $resn_exch{"HN21 BHB"}="1HN2 BHB"; $resn_exch{"HN22 BHB"}="2HN2 BHB";
$resn_exch{" H91 BHB"}="1H9  BHB"; $resn_exch{" H92 BHB"}="2H9  BHB"; $resn_exch{"H161 BHB"}="1H16 BHB";
$resn_exch{"H162 BHB"}="2H16 BHB"; $resn_exch{"H171 BHB"}="1H17 BHB"; $resn_exch{"H172 BHB"}="2H17 BHB";
$resn_exch{"H201 BHB"}="1H20 BHB"; $resn_exch{"H202 BHB"}="2H20 BHB"; $resn_exch{"H203 BHB"}="3H20 BHB";
$resn_exch{"H191 BHB"}="1H19 BHB"; $resn_exch{"H192 BHB"}="2H19 BHB"; $resn_exch{"H193 BHB"}="3H19 BHB";
$resn_exch{" H   BHD"}="1HN  BHD"; $resn_exch{" H2  BHD"}="2HN  BHD"; $resn_exch{" HD2 BHD"}=" HOD BHD";
$resn_exch{"H1'1 BHG"}="1H1' BHG"; $resn_exch{"H1'2 BHG"}="2H1' BHG"; $resn_exch{"H2'1 BHG"}="1H2' BHG";
$resn_exch{"H2'2 BHG"}="2H2' BHG"; $resn_exch{"H3'1 BHG"}="1H3' BHG"; $resn_exch{"H3'2 BHG"}="2H3' BHG";
$resn_exch{"H4'1 BHG"}="1H4' BHG"; $resn_exch{"H4'2 BHG"}="2H4' BHG"; $resn_exch{"H5'1 BHG"}="1H5' BHG";
$resn_exch{"H5'2 BHG"}="2H5' BHG"; $resn_exch{"H6'1 BHG"}="1H6' BHG"; $resn_exch{"H6'2 BHG"}="2H6' BHG";
$resn_exch{"H6'3 BHG"}="3H6' BHG"; $resn_exch{" H41 BHH"}="1H4  BHH"; $resn_exch{" H42 BHH"}="2H4  BHH";
$resn_exch{" H43 BHH"}="3H4  BHH"; $resn_exch{" H31 BHH"}="1H3  BHH"; $resn_exch{" H32 BHH"}="2H3  BHH";
$resn_exch{" H21 BHH"}="1H2  BHH"; $resn_exch{" H22 BHH"}="2H2  BHH"; $resn_exch{" H11 BHH"}="1H1  BHH";
$resn_exch{" H12 BHH"}="2H1  BHH"; $resn_exch{" HE1 BHH"}="1HE  BHH"; $resn_exch{"HH11 BHH"}="1HH1 BHH";
$resn_exch{"HH21 BHH"}="1HH2 BHH"; $resn_exch{" HH1 BHH"}="1HH  BHH"; $resn_exch{"H101 BHI"}="1H10 BHI";
$resn_exch{"H102 BHI"}="2H10 BHI"; $resn_exch{"H111 BHI"}="1H11 BHI"; $resn_exch{"H112 BHI"}="2H11 BHI";
$resn_exch{"H113 BHI"}="3H11 BHI"; $resn_exch{"H121 BHI"}="1H12 BHI"; $resn_exch{"H122 BHI"}="2H12 BHI";
$resn_exch{"H131 BHI"}="1H13 BHI"; $resn_exch{"H132 BHI"}="2H13 BHI"; $resn_exch{" H11 BHL"}="1H1  BHL";
$resn_exch{" H12 BHL"}="2H1  BHL"; $resn_exch{" H21 BHL"}="1H2  BHL"; $resn_exch{" H22 BHL"}="2H2  BHL";
$resn_exch{" H31 BHL"}="1H3  BHL"; $resn_exch{" H32 BHL"}="2H3  BHL"; $resn_exch{" H41 BHL"}="1H4  BHL";
$resn_exch{" H42 BHL"}="2H4  BHL"; $resn_exch{" H51 BHL"}="1H5  BHL"; $resn_exch{" H52 BHL"}="2H5  BHL";
$resn_exch{" H61 BHL"}="1H6  BHL"; $resn_exch{" H62 BHL"}="2H6  BHL"; $resn_exch{"H131 BHM"}="1H13 BHM";
$resn_exch{"H132 BHM"}="2H13 BHM"; $resn_exch{"H121 BHM"}="1H12 BHM"; $resn_exch{"H122 BHM"}="2H12 BHM";
$resn_exch{"H123 BHM"}="3H12 BHM"; $resn_exch{" H91 BHP"}="1H9  BHP"; $resn_exch{" H92 BHP"}="2H9  BHP";
$resn_exch{" H81 BHP"}="1H8  BHP"; $resn_exch{" H82 BHP"}="2H8  BHP"; $resn_exch{" H71 BHP"}="1H7  BHP";
$resn_exch{" H72 BHP"}="2H7  BHP"; $resn_exch{"H121 BHP"}="1H12 BHP"; $resn_exch{"H122 BHP"}="2H12 BHP";
$resn_exch{"H111 BHP"}="1H11 BHP"; $resn_exch{"H112 BHP"}="2H11 BHP"; $resn_exch{"H101 BHP"}="1H10 BHP";
$resn_exch{"H102 BHP"}="2H10 BHP"; $resn_exch{"H151 BHP"}="1H15 BHP"; $resn_exch{"H152 BHP"}="2H15 BHP";
$resn_exch{"H291 BHP"}="1H29 BHP"; $resn_exch{"H292 BHP"}="2H29 BHP"; $resn_exch{"H301 BHP"}="1H30 BHP";
$resn_exch{"H302 BHP"}="2H30 BHP"; $resn_exch{"H221 BHP"}="1H22 BHP"; $resn_exch{"H222 BHP"}="2H22 BHP";
$resn_exch{" H81 BHQ"}="1H8  BHQ"; $resn_exch{" H82 BHQ"}="2H8  BHQ"; $resn_exch{" H83 BHQ"}="3H8  BHQ";
$resn_exch{" H91 BHQ"}="1H9  BHQ"; $resn_exch{" H92 BHQ"}="2H9  BHQ"; $resn_exch{" H93 BHQ"}="3H9  BHQ";
$resn_exch{"H101 BHQ"}="1H10 BHQ"; $resn_exch{"H102 BHQ"}="2H10 BHQ"; $resn_exch{"H103 BHQ"}="3H10 BHQ";
$resn_exch{"H121 BHQ"}="1H12 BHQ"; $resn_exch{"H122 BHQ"}="2H12 BHQ"; $resn_exch{"H123 BHQ"}="3H12 BHQ";
$resn_exch{"H131 BHQ"}="1H13 BHQ"; $resn_exch{"H132 BHQ"}="2H13 BHQ"; $resn_exch{"H133 BHQ"}="3H13 BHQ";
$resn_exch{"H141 BHQ"}="1H14 BHQ"; $resn_exch{"H142 BHQ"}="2H14 BHQ"; $resn_exch{"H143 BHQ"}="3H14 BHQ";
$resn_exch{"HN21 BHS"}="1HN2 BHS"; $resn_exch{"HN22 BHS"}="2HN2 BHS"; $resn_exch{" H71 BHS"}="1H7  BHS";
$resn_exch{" H72 BHS"}="2H7  BHS"; $resn_exch{"H3'1 BHS"}="1H3' BHS"; $resn_exch{"H3'2 BHS"}="2H3' BHS";
$resn_exch{"H3'3 BHS"}="3H3' BHS"; $resn_exch{"HBD1 BI1"}="1HBD BI1"; $resn_exch{"HBD2 BI1"}="2HBD BI1";
$resn_exch{"HBD3 BI1"}="3HBD BI1"; $resn_exch{"HBE1 BI1"}="1HBE BI1"; $resn_exch{"HBE2 BI1"}="2HBE BI1";
$resn_exch{"HBE3 BI1"}="3HBE BI1"; $resn_exch{"HBB1 BI1"}="1HBB BI1"; $resn_exch{"HBB2 BI1"}="2HBB BI1";
$resn_exch{"HBA1 BI1"}="1HBA BI1"; $resn_exch{"HBA2 BI1"}="2HBA BI1"; $resn_exch{"HAY1 BI1"}="1HAY BI1";
$resn_exch{"HAY2 BI1"}="2HAY BI1"; $resn_exch{"HBD1 BI2"}="1HBD BI2"; $resn_exch{"HBD2 BI2"}="2HBD BI2";
$resn_exch{"HBD3 BI2"}="3HBD BI2"; $resn_exch{"HBG1 BI2"}="1HBG BI2"; $resn_exch{"HBG2 BI2"}="2HBG BI2";
$resn_exch{"HBF1 BI2"}="1HBF BI2"; $resn_exch{"HBF2 BI2"}="2HBF BI2"; $resn_exch{"HBE1 BI2"}="1HBE BI2";
$resn_exch{"HBE2 BI2"}="2HBE BI2"; $resn_exch{"HBA1 BI2"}="1HBA BI2"; $resn_exch{"HBA2 BI2"}="2HBA BI2";
$resn_exch{"HAY1 BI2"}="1HAY BI2"; $resn_exch{"HAY2 BI2"}="2HAY BI2"; $resn_exch{"HBC1 BI3"}="1HBC BI3";
$resn_exch{"HBC2 BI3"}="2HBC BI3"; $resn_exch{"HBB1 BI3"}="1HBB BI3"; $resn_exch{"HBB2 BI3"}="2HBB BI3";
$resn_exch{"HBA1 BI3"}="1HBA BI3"; $resn_exch{"HBA2 BI3"}="2HBA BI3"; $resn_exch{"HAY1 BI3"}="1HAY BI3";
$resn_exch{"HAY2 BI3"}="2HAY BI3"; $resn_exch{"H241 BI4"}="1H24 BI4"; $resn_exch{"H242 BI4"}="2H24 BI4";
$resn_exch{"H251 BI4"}="1H25 BI4"; $resn_exch{"H252 BI4"}="2H25 BI4"; $resn_exch{"H271 BI4"}="1H27 BI4";
$resn_exch{"H272 BI4"}="2H27 BI4"; $resn_exch{"H281 BI4"}="1H28 BI4"; $resn_exch{"H282 BI4"}="2H28 BI4";
$resn_exch{"H291 BI4"}="1H29 BI4"; $resn_exch{"H292 BI4"}="2H29 BI4"; $resn_exch{"H311 BI4"}="1H31 BI4";
$resn_exch{"H312 BI4"}="2H31 BI4"; $resn_exch{"H313 BI4"}="3H31 BI4"; $resn_exch{" H61 BI5"}="1H6  BI5";
$resn_exch{" H62 BI5"}="2H6  BI5"; $resn_exch{"HB01 BI8"}="1HB0 BI8"; $resn_exch{"HB02 BI8"}="2HB0 BI8";
$resn_exch{"HB03 BI8"}="3HB0 BI8"; $resn_exch{"HBC1 BI8"}="1HBC BI8"; $resn_exch{"HBC2 BI8"}="2HBC BI8";
$resn_exch{"HBB1 BI8"}="1HBB BI8"; $resn_exch{"HBB2 BI8"}="2HBB BI8"; $resn_exch{"HBA1 BI8"}="1HBA BI8";
$resn_exch{"HBA2 BI8"}="2HBA BI8"; $resn_exch{"HAY1 BI8"}="1HAY BI8"; $resn_exch{"HAY2 BI8"}="2HAY BI8";
$resn_exch{"H121 BIA"}="1H12 BIA"; $resn_exch{"H122 BIA"}="2H12 BIA"; $resn_exch{"H161 BIA"}="1H16 BIA";
$resn_exch{"H162 BIA"}="2H16 BIA"; $resn_exch{"H211 BIA"}="1H21 BIA"; $resn_exch{"H212 BIA"}="2H21 BIA";
$resn_exch{"H221 BIA"}="1H22 BIA"; $resn_exch{"H222 BIA"}="2H22 BIA"; $resn_exch{"H251 BIA"}="1H25 BIA";
$resn_exch{"H252 BIA"}="2H25 BIA"; $resn_exch{"H271 BIA"}="1H27 BIA"; $resn_exch{"H272 BIA"}="2H27 BIA";
$resn_exch{" H21 BIB"}="1H2  BIB"; $resn_exch{" H22 BIB"}="2H2  BIB"; $resn_exch{" H23 BIB"}="3H2  BIB";
$resn_exch{" H41 BIB"}="1H4  BIB"; $resn_exch{" H42 BIB"}="2H4  BIB"; $resn_exch{"HN11 BIB"}="1HN1 BIB";
$resn_exch{"HN12 BIB"}="2HN1 BIB"; $resn_exch{" H21 BIC"}="1H2  BIC"; $resn_exch{" H22 BIC"}="2H2  BIC";
$resn_exch{" H31 BIC"}="1H3  BIC"; $resn_exch{" H32 BIC"}="2H3  BIC"; $resn_exch{" H41 BIC"}="1H4  BIC";
$resn_exch{" H42 BIC"}="2H4  BIC"; $resn_exch{" H51 BIC"}="1H5  BIC"; $resn_exch{" H52 BIC"}="2H5  BIC";
$resn_exch{"HN31 BIC"}="1HN3 BIC"; $resn_exch{"HN32 BIC"}="2HN3 BIC"; $resn_exch{" H91 BIC"}="1H9  BIC";
$resn_exch{" H92 BIC"}="2H9  BIC"; $resn_exch{"H171 BID"}="1H17 BID"; $resn_exch{"H172 BID"}="2H17 BID";
$resn_exch{"H173 BID"}="3H17 BID"; $resn_exch{"H141 BID"}="1H14 BID"; $resn_exch{"H142 BID"}="2H14 BID";
$resn_exch{"H121 BID"}="1H12 BID"; $resn_exch{"H122 BID"}="2H12 BID"; $resn_exch{"H101 BID"}="1H10 BID";
$resn_exch{"H102 BID"}="2H10 BID"; $resn_exch{"H103 BID"}="3H10 BID"; $resn_exch{" H81 BID"}="1H8  BID";
$resn_exch{" H82 BID"}="2H8  BID"; $resn_exch{" H71 BID"}="1H7  BID"; $resn_exch{" H72 BID"}="2H7  BID";
$resn_exch{" H51 BID"}="1H5  BID"; $resn_exch{" H52 BID"}="2H5  BID"; $resn_exch{"H11A BID"}="1H1  BID";
$resn_exch{" H12 BID"}="2H1  BID"; $resn_exch{" H13 BID"}="3H1  BID"; $resn_exch{"H191 BID"}="1H19 BID";
$resn_exch{"H192 BID"}="2H19 BID"; $resn_exch{"H201 BID"}="1H20 BID"; $resn_exch{"H202 BID"}="2H20 BID";
$resn_exch{"H211 BID"}="1H21 BID"; $resn_exch{"H212 BID"}="2H21 BID"; $resn_exch{"H241 BID"}="1H24 BID";
$resn_exch{"H242 BID"}="2H24 BID"; $resn_exch{"H243 BID"}="3H24 BID"; $resn_exch{"H251 BID"}="1H25 BID";
$resn_exch{"H252 BID"}="2H25 BID"; $resn_exch{"H261 BID"}="1H26 BID"; $resn_exch{"H262 BID"}="2H26 BID";
$resn_exch{"H281 BID"}="1H28 BID"; $resn_exch{"H282 BID"}="2H28 BID"; $resn_exch{"H291 BID"}="1H29 BID";
$resn_exch{"H292 BID"}="2H29 BID"; $resn_exch{"H301 BID"}="1H30 BID"; $resn_exch{"H302 BID"}="2H30 BID";
$resn_exch{"H321 BID"}="1H32 BID"; $resn_exch{"H322 BID"}="2H32 BID"; $resn_exch{"H331 BID"}="1H33 BID";
$resn_exch{"H332 BID"}="2H33 BID"; $resn_exch{"H351 BID"}="1H35 BID"; $resn_exch{"H352 BID"}="2H35 BID";
$resn_exch{"H353 BID"}="3H35 BID"; $resn_exch{"H381 BID"}="1H38 BID"; $resn_exch{"H382 BID"}="2H38 BID";
$resn_exch{"H383 BID"}="3H38 BID"; $resn_exch{"H401 BID"}="1H40 BID"; $resn_exch{"H402 BID"}="2H40 BID";
$resn_exch{"H403 BID"}="3H40 BID"; $resn_exch{" H   BIF"}="1HN  BIF"; $resn_exch{" H2  BIF"}="2HN  BIF";
$resn_exch{" HB2 BIF"}="1HB  BIF"; $resn_exch{" HB3 BIF"}="2HB  BIF"; $resn_exch{"H271 BIM"}="1H27 BIM";
$resn_exch{"H272 BIM"}="2H27 BIM"; $resn_exch{"H351 BIM"}="1H35 BIM"; $resn_exch{"H352 BIM"}="2H35 BIM";
$resn_exch{"H361 BIM"}="1H36 BIM"; $resn_exch{"H362 BIM"}="2H36 BIM"; $resn_exch{"H371 BIM"}="1H37 BIM";
$resn_exch{"H372 BIM"}="2H37 BIM"; $resn_exch{"H381 BIM"}="1H38 BIM"; $resn_exch{"H382 BIM"}="2H38 BIM";
$resn_exch{"H401 BIM"}="1H40 BIM"; $resn_exch{"H402 BIM"}="2H40 BIM"; $resn_exch{"H403 BIM"}="3H40 BIM";
$resn_exch{"H261 BIM"}="1H26 BIM"; $resn_exch{"H262 BIM"}="2H26 BIM"; $resn_exch{"H281 BIM"}="1H28 BIM";
$resn_exch{"H282 BIM"}="2H28 BIM"; $resn_exch{"H411 BIM"}="1H41 BIM"; $resn_exch{"H412 BIM"}="2H41 BIM";
$resn_exch{"H413 BIM"}="3H41 BIM"; $resn_exch{"H151 BIN"}="1H15 BIN"; $resn_exch{"H152 BIN"}="2H15 BIN";
$resn_exch{"H153 BIN"}="3H15 BIN"; $resn_exch{"H161 BIN"}="1H16 BIN"; $resn_exch{"H162 BIN"}="2H16 BIN";
$resn_exch{"H163 BIN"}="3H16 BIN"; $resn_exch{"H171 BIN"}="1H17 BIN"; $resn_exch{"H172 BIN"}="2H17 BIN";
$resn_exch{"H201 BIN"}="1H20 BIN"; $resn_exch{"H202 BIN"}="2H20 BIN"; $resn_exch{"H203 BIN"}="3H20 BIN";
$resn_exch{"H211 BIN"}="1H21 BIN"; $resn_exch{"H212 BIN"}="2H21 BIN"; $resn_exch{"H213 BIN"}="3H21 BIN";
$resn_exch{"HOB3 BIN"}="3HOB BIN"; $resn_exch{"HOB4 BIN"}="4HOB BIN"; $resn_exch{"HN21 BIO"}="1HN2 BIO";
$resn_exch{"HN22 BIO"}="2HN2 BIO"; $resn_exch{"H111 BIO"}="1H11 BIO"; $resn_exch{"H112 BIO"}="2H11 BIO";
$resn_exch{"H113 BIO"}="3H11 BIO"; $resn_exch{" H31 BIP"}="1H3  BIP"; $resn_exch{" H32 BIP"}="2H3  BIP";
$resn_exch{" H1  BIP"}="1H   BIP"; $resn_exch{" H2A BIP"}="2H   BIP"; $resn_exch{" H31 BIQ"}="1H3  BIQ";
$resn_exch{" H32 BIQ"}="2H3  BIQ"; $resn_exch{"H121 BIQ"}="1H12 BIQ"; $resn_exch{"H122 BIQ"}="2H12 BIQ";
$resn_exch{"H141 BIQ"}="1H14 BIQ"; $resn_exch{"H142 BIQ"}="2H14 BIQ"; $resn_exch{"H231 BIQ"}="1H23 BIQ";
$resn_exch{"H232 BIQ"}="2H23 BIQ"; $resn_exch{"H241 BIQ"}="1H24 BIQ"; $resn_exch{"H242 BIQ"}="2H24 BIQ";
$resn_exch{"H251 BIQ"}="1H25 BIQ"; $resn_exch{"H252 BIQ"}="2H25 BIQ"; $resn_exch{"H253 BIQ"}="3H25 BIQ";
$resn_exch{"H261 BIQ"}="1H26 BIQ"; $resn_exch{"H262 BIQ"}="2H26 BIQ"; $resn_exch{"H411 BIQ"}="1H41 BIQ";
$resn_exch{"H412 BIQ"}="2H41 BIQ"; $resn_exch{" H31 BIR"}="1H3  BIR"; $resn_exch{" H32 BIR"}="2H3  BIR";
$resn_exch{" H33 BIR"}="3H3  BIR"; $resn_exch{"HN11 BIR"}="1HN1 BIR"; $resn_exch{"HN12 BIR"}="2HN1 BIR";
$resn_exch{" H71 BIR"}="1H7  BIR"; $resn_exch{" H72 BIR"}="2H7  BIR"; $resn_exch{"H261 BIR"}="1H26 BIR";
$resn_exch{"H262 BIR"}="2H26 BIR"; $resn_exch{"H263 BIR"}="3H26 BIR"; $resn_exch{" H91 BIR"}="1H9  BIR";
$resn_exch{" H92 BIR"}="2H9  BIR"; $resn_exch{"HO2B BIS"}="BHO2 BIS"; $resn_exch{"HO1A BIS"}="AHO1 BIS";
$resn_exch{"H5'1 BIS"}="1H5* BIS"; $resn_exch{"H5'2 BIS"}="2H5* BIS"; $resn_exch{" H4' BIS"}=" H4* BIS";
$resn_exch{" H3' BIS"}=" H3* BIS"; $resn_exch{"HO'3 BIS"}="3HO* BIS"; $resn_exch{" H2' BIS"}=" H2* BIS";
$resn_exch{"HO'2 BIS"}="2HO* BIS"; $resn_exch{" H1' BIS"}=" H1* BIS"; $resn_exch{"HN61 BIS"}="1HN6 BIS";
$resn_exch{"HN62 BIS"}="2HN6 BIS"; $resn_exch{"H121 BIS"}="1H12 BIS"; $resn_exch{"H122 BIS"}="2H12 BIS";
$resn_exch{"H131 BIS"}="1H13 BIS"; $resn_exch{"H132 BIS"}="2H13 BIS"; $resn_exch{"H141 BIS"}="1H14 BIS";
$resn_exch{"H142 BIS"}="2H14 BIS"; $resn_exch{"H181 BIT"}="1H18 BIT"; $resn_exch{"H182 BIT"}="2H18 BIT";
$resn_exch{"H183 BIT"}="3H18 BIT"; $resn_exch{" H21 BIT"}="1H2  BIT"; $resn_exch{" H22 BIT"}="2H2  BIT";
$resn_exch{" H11 BIT"}="1H1  BIT"; $resn_exch{" H12 BIT"}="2H1  BIT"; $resn_exch{" H21 BIV"}="1H2  BIV";
$resn_exch{" H22 BIV"}="2H2  BIV"; $resn_exch{" H41 BIV"}="1H4  BIV"; $resn_exch{" H42 BIV"}="2H4  BIV";
$resn_exch{" H91 BIV"}="1H9  BIV"; $resn_exch{" H92 BIV"}="2H9  BIV"; $resn_exch{"H111 BIV"}="1H11 BIV";
$resn_exch{"H112 BIV"}="2H11 BIV"; $resn_exch{"H121 BIV"}="1H12 BIV"; $resn_exch{"H122 BIV"}="2H12 BIV";
$resn_exch{"H151 BIV"}="1H15 BIV"; $resn_exch{"H152 BIV"}="2H15 BIV"; $resn_exch{"H161 BIV"}="1H16 BIV";
$resn_exch{"H162 BIV"}="2H16 BIV"; $resn_exch{"H181 BIV"}="1H18 BIV"; $resn_exch{"H182 BIV"}="2H18 BIV";
$resn_exch{"H183 BIV"}="3H18 BIV"; $resn_exch{"H191 BIV"}="1H19 BIV"; $resn_exch{"H192 BIV"}="2H19 BIV";
$resn_exch{"H221 BIV"}="1H22 BIV"; $resn_exch{"H222 BIV"}="2H22 BIV"; $resn_exch{"H231 BIV"}="1H23 BIV";
$resn_exch{"H232 BIV"}="2H23 BIV"; $resn_exch{"H241 BIV"}="1H24 BIV"; $resn_exch{"H242 BIV"}="2H24 BIV";
$resn_exch{"H261 BIV"}="1H26 BIV"; $resn_exch{"H262 BIV"}="2H26 BIV"; $resn_exch{"H263 BIV"}="3H26 BIV";
$resn_exch{"H271 BIV"}="1H27 BIV"; $resn_exch{"H272 BIV"}="2H27 BIV"; $resn_exch{"H273 BIV"}="3H27 BIV";
$resn_exch{"H211 BIV"}="1H21 BIV"; $resn_exch{"H212 BIV"}="2H21 BIV"; $resn_exch{"H281 BIV"}="1H28 BIV";
$resn_exch{"H282 BIV"}="2H28 BIV"; $resn_exch{"H291 BIV"}="1H29 BIV"; $resn_exch{"H292 BIV"}="2H29 BIV";
$resn_exch{"H321 BIV"}="1H32 BIV"; $resn_exch{"H322 BIV"}="2H32 BIV"; $resn_exch{"H323 BIV"}="3H32 BIV";
$resn_exch{"H331 BIV"}="1H33 BIV"; $resn_exch{"H332 BIV"}="2H33 BIV"; $resn_exch{"H333 BIV"}="3H33 BIV";
$resn_exch{"H141 BIZ"}="1H14 BIZ"; $resn_exch{"H142 BIZ"}="2H14 BIZ"; $resn_exch{"H181 BIZ"}="1H18 BIZ";
$resn_exch{"H182 BIZ"}="2H18 BIZ"; $resn_exch{"H183 BIZ"}="3H18 BIZ"; $resn_exch{"H261 BIZ"}="1H26 BIZ";
$resn_exch{"H262 BIZ"}="2H26 BIZ"; $resn_exch{"H263 BIZ"}="3H26 BIZ"; $resn_exch{"HN1' BIZ"}="*HN1 BIZ";
$resn_exch{" H3' BIZ"}=" *H3 BIZ"; $resn_exch{" H6' BIZ"}=" *H6 BIZ"; $resn_exch{" H7' BIZ"}=" *H7 BIZ";
$resn_exch{"HN8' BIZ"}="*HN8 BIZ"; $resn_exch{"H10' BIZ"}="*H10 BIZ"; $resn_exch{"H4'1 BIZ"}="1H4* BIZ";
$resn_exch{"H4'2 BIZ"}="2H4* BIZ"; $resn_exch{"H15' BIZ"}="*H15 BIZ"; $resn_exch{"H8'1 BIZ"}="1H8* BIZ";
$resn_exch{"H8'2 BIZ"}="2H8* BIZ"; $resn_exch{"H8'3 BIZ"}="3H8* BIZ"; $resn_exch{"H19' BIZ"}="*H19 BIZ";
$resn_exch{"HO2' BIZ"}="*HO2 BIZ"; $resn_exch{"H25' BIZ"}="*H25 BIZ"; $resn_exch{"H6'1 BIZ"}="1H6* BIZ";
$resn_exch{"H6'2 BIZ"}="2H6* BIZ"; $resn_exch{"H6'3 BIZ"}="3H6* BIZ"; $resn_exch{"HN7' BIZ"}="*HN7 BIZ";
$resn_exch{" H21 BJH"}="1H2  BJH"; $resn_exch{" H22 BJH"}="2H2  BJH"; $resn_exch{" H41 BJH"}="1H4  BJH";
$resn_exch{" H42 BJH"}="2H4  BJH"; $resn_exch{" H43 BJH"}="3H4  BJH"; $resn_exch{"HOB1 BJH"}="1HOB BJH";
$resn_exch{"HOB2 BJH"}="2HOB BJH"; $resn_exch{" H21 BJI"}="1H2  BJI"; $resn_exch{" H22 BJI"}="2H2  BJI";
$resn_exch{" H41 BJI"}="1H4  BJI"; $resn_exch{" H42 BJI"}="2H4  BJI"; $resn_exch{" H43 BJI"}="3H4  BJI";
$resn_exch{"HOB1 BJI"}="1HOB BJI"; $resn_exch{"HOB2 BJI"}="2HOB BJI"; $resn_exch{" H21 BJP"}="1H2  BJP";
$resn_exch{" H22 BJP"}="2H2  BJP"; $resn_exch{"HOB1 BJP"}="1HOB BJP"; $resn_exch{"HOB2 BJP"}="2HOB BJP";
$resn_exch{"H201 BJP"}="1H20 BJP"; $resn_exch{"H211 BJP"}="1H21 BJP"; $resn_exch{"H221 BJP"}="1H22 BJP";
$resn_exch{"H231 BJP"}="1H23 BJP"; $resn_exch{"H241 BJP"}="1H24 BJP"; $resn_exch{"H251 BJP"}="1H25 BJP";
$resn_exch{"H252 BJP"}="2H25 BJP"; $resn_exch{"H111 BL5"}="1H11 BL5"; $resn_exch{"H112 BL5"}="2H11 BL5";
$resn_exch{"H121 BL5"}="1H12 BL5"; $resn_exch{"H122 BL5"}="2H12 BL5"; $resn_exch{"H131 BL5"}="1H13 BL5";
$resn_exch{"H132 BL5"}="2H13 BL5"; $resn_exch{"H141 BL5"}="1H14 BL5"; $resn_exch{"H142 BL5"}="2H14 BL5";
$resn_exch{"H151 BL5"}="1H15 BL5"; $resn_exch{"H152 BL5"}="2H15 BL5"; $resn_exch{"H201 BL5"}="1H20 BL5";
$resn_exch{"H202 BL5"}="2H20 BL5"; $resn_exch{"H211 BL5"}="1H21 BL5"; $resn_exch{"H212 BL5"}="2H21 BL5";
$resn_exch{"H221 BL5"}="1H22 BL5"; $resn_exch{"H222 BL5"}="2H22 BL5"; $resn_exch{"H231 BL5"}="1H23 BL5";
$resn_exch{"H232 BL5"}="2H23 BL5"; $resn_exch{"H233 BL5"}="3H23 BL5"; $resn_exch{"H241 BL5"}="1H24 BL5";
$resn_exch{"H242 BL5"}="2H24 BL5"; $resn_exch{"H271 BL5"}="1H27 BL5"; $resn_exch{"H272 BL5"}="2H27 BL5";
$resn_exch{"H311 BL5"}="1H31 BL5"; $resn_exch{"H312 BL5"}="2H31 BL5"; $resn_exch{"H301 BL5"}="1H30 BL5";
$resn_exch{"H302 BL5"}="2H30 BL5"; $resn_exch{"H291 BL5"}="1H29 BL5"; $resn_exch{"H292 BL5"}="2H29 BL5";
$resn_exch{"H281 BL5"}="1H28 BL5"; $resn_exch{"H282 BL5"}="2H28 BL5"; $resn_exch{"H283 BL5"}="3H28 BL5";
$resn_exch{"H401 BL5"}="1H40 BL5"; $resn_exch{"H402 BL5"}="2H40 BL5"; $resn_exch{"H391 BL5"}="1H39 BL5";
$resn_exch{"H392 BL5"}="2H39 BL5"; $resn_exch{"H381 BL5"}="1H38 BL5"; $resn_exch{"H382 BL5"}="2H38 BL5";
$resn_exch{"H371 BL5"}="1H37 BL5"; $resn_exch{"H372 BL5"}="2H37 BL5"; $resn_exch{"H361 BL5"}="1H36 BL5";
$resn_exch{"H362 BL5"}="2H36 BL5"; $resn_exch{" HA  BLA"}=" H_A BLA"; $resn_exch{"HMA1 BLA"}="1HMA BLA";
$resn_exch{"HMA2 BLA"}="2HMA BLA"; $resn_exch{"HMA3 BLA"}="3HMA BLA"; $resn_exch{"HAA1 BLA"}="1HAA BLA";
$resn_exch{"HAA2 BLA"}="2HAA BLA"; $resn_exch{"HBA1 BLA"}="1HBA BLA"; $resn_exch{"HBA2 BLA"}="2HBA BLA";
$resn_exch{" HB  BLA"}=" H_B BLA"; $resn_exch{"HMB1 BLA"}="1HMB BLA"; $resn_exch{"HMB2 BLA"}="2HMB BLA";
$resn_exch{"HMB3 BLA"}="3HMB BLA"; $resn_exch{"HBB1 BLA"}="1HBB BLA"; $resn_exch{"HBB2 BLA"}="2HBB BLA";
$resn_exch{" HC  BLA"}=" H_C BLA"; $resn_exch{"HMC1 BLA"}="1HMC BLA"; $resn_exch{"HMC2 BLA"}="2HMC BLA";
$resn_exch{"HMC3 BLA"}="3HMC BLA"; $resn_exch{"HBC1 BLA"}="1HBC BLA"; $resn_exch{"HBC2 BLA"}="2HBC BLA";
$resn_exch{"HMD1 BLA"}="1HMD BLA"; $resn_exch{"HMD2 BLA"}="2HMD BLA"; $resn_exch{"HMD3 BLA"}="3HMD BLA";
$resn_exch{"HAD1 BLA"}="1HAD BLA"; $resn_exch{"HAD2 BLA"}="2HAD BLA"; $resn_exch{"HBD1 BLA"}="1HBD BLA";
$resn_exch{"HBD2 BLA"}="2HBD BLA"; $resn_exch{"H501 BLB"}="1H50 BLB"; $resn_exch{"H502 BLB"}="2H50 BLB";
$resn_exch{"H511 BLB"}="1H51 BLB"; $resn_exch{"H512 BLB"}="2H51 BLB"; $resn_exch{"H521 BLB"}="1H52 BLB";
$resn_exch{"H522 BLB"}="2H52 BLB"; $resn_exch{"H531 BLB"}="1H53 BLB"; $resn_exch{"H532 BLB"}="2H53 BLB";
$resn_exch{"HNS1 BLB"}="1HNS BLB"; $resn_exch{"HNS2 BLB"}="2HNS BLB"; $resn_exch{"HNT1 BLB"}="1HNT BLB";
$resn_exch{"HNT2 BLB"}="2HNT BLB"; $resn_exch{"H611 BLB"}="1H61 BLB"; $resn_exch{"H612 BLB"}="2H61 BLB";
$resn_exch{"HNQ1 BLB"}="1HNQ BLB"; $resn_exch{"HNQ2 BLB"}="2HNQ BLB"; $resn_exch{"H661 BLB"}="1H66 BLB";
$resn_exch{"H662 BLB"}="2H66 BLB"; $resn_exch{" HB2 BLE"}="1HB  BLE"; $resn_exch{" HB3 BLE"}="2HB  BLE";
$resn_exch{"HD11 BLE"}="1HD1 BLE"; $resn_exch{"HD12 BLE"}="2HD1 BLE"; $resn_exch{"HD13 BLE"}="3HD1 BLE";
$resn_exch{"HD21 BLE"}="1HD2 BLE"; $resn_exch{"HD22 BLE"}="2HD2 BLE"; $resn_exch{"HD23 BLE"}="3HD2 BLE";
$resn_exch{" H   BLE"}="1HN  BLE"; $resn_exch{" HN2 BLE"}="2HN  BLE"; $resn_exch{" H81 BLG"}="1H8  BLG";
$resn_exch{" H82 BLG"}="2H8  BLG"; $resn_exch{" H83 BLG"}="3H8  BLG"; $resn_exch{"HO43 BLG"}="3HO4 BLG";
$resn_exch{" H61 BLG"}="1H6  BLG"; $resn_exch{" H62 BLG"}="2H6  BLG"; $resn_exch{" HG1 BLG"}="1HG  BLG";
$resn_exch{" HG2 BLG"}="2HG  BLG"; $resn_exch{" HN1 BLG"}="1HN  BLG"; $resn_exch{"HN2A BLG"}="2HN  BLG";
$resn_exch{" H91 BLG"}="1H9  BLG"; $resn_exch{" H92 BLG"}="2H9  BLG"; $resn_exch{"H111 BLG"}="1H11 BLG";
$resn_exch{"H112 BLG"}="2H11 BLG"; $resn_exch{"H121 BLG"}="1H12 BLG"; $resn_exch{"H122 BLG"}="2H12 BLG";
$resn_exch{"HOS3 BLG"}="3HOS BLG"; $resn_exch{" H11 BLI"}="1H1  BLI"; $resn_exch{" H21 BLI"}="1H2  BLI";
$resn_exch{" H22 BLI"}="2H2  BLI"; $resn_exch{" H41 BLI"}="1H4  BLI"; $resn_exch{" H42 BLI"}="2H4  BLI";
$resn_exch{"H011 BLI"}="1H01 BLI"; $resn_exch{"H101 BLI"}="1H10 BLI"; $resn_exch{"H031 BLI"}="1H03 BLI";
$resn_exch{"H032 BLI"}="2H03 BLI"; $resn_exch{"H061 BLI"}="1H06 BLI"; $resn_exch{"H062 BLI"}="2H06 BLI";
$resn_exch{"H021 BLI"}="1H02 BLI"; $resn_exch{"H022 BLI"}="2H02 BLI"; $resn_exch{"H1'1 BLI"}="1H1' BLI";
$resn_exch{"H1'2 BLI"}="2H1' BLI"; $resn_exch{"H2'1 BLI"}="1H2' BLI"; $resn_exch{"H3'1 BLI"}="1H3' BLI";
$resn_exch{"H3'2 BLI"}="2H3' BLI"; $resn_exch{"H7'1 BLI"}="1H7' BLI"; $resn_exch{"H7'2 BLI"}="2H7' BLI";
$resn_exch{"H4'1 BLI"}="1H4' BLI"; $resn_exch{"H4'2 BLI"}="2H4' BLI"; $resn_exch{"H5'1 BLI"}="1H5' BLI";
$resn_exch{"H5'2 BLI"}="2H5' BLI"; $resn_exch{"HN21 BLI"}="1HN2 BLI"; $resn_exch{"HN22 BLI"}="2HN2 BLI";
$resn_exch{"HO31 BLI"}="1HO3 BLI"; $resn_exch{"H1A1 BLI"}="1H1* BLI"; $resn_exch{"H2A1 BLI"}="1H2* BLI";
$resn_exch{"H3A1 BLI"}="1H3* BLI"; $resn_exch{"H4A1 BLI"}="1H4* BLI"; $resn_exch{"H5A1 BLI"}="1H5* BLI";
$resn_exch{"H131 BLL"}="1H13 BLL"; $resn_exch{"H132 BLL"}="2H13 BLL"; $resn_exch{"H141 BLL"}="1H14 BLL";
$resn_exch{"H142 BLL"}="2H14 BLL"; $resn_exch{"H221 BLL"}="1H22 BLL"; $resn_exch{"H222 BLL"}="2H22 BLL";
$resn_exch{"H231 BLL"}="1H23 BLL"; $resn_exch{"H232 BLL"}="2H23 BLL"; $resn_exch{"H381 BLL"}="1H38 BLL";
$resn_exch{"H382 BLL"}="2H38 BLL"; $resn_exch{"H481 BLL"}="1H48 BLL"; $resn_exch{"H482 BLL"}="2H48 BLL";
$resn_exch{"H501 BLM"}="1H50 BLM"; $resn_exch{"H502 BLM"}="2H50 BLM"; $resn_exch{"H511 BLM"}="1H51 BLM";
$resn_exch{"H512 BLM"}="2H51 BLM"; $resn_exch{"H521 BLM"}="1H52 BLM"; $resn_exch{"H522 BLM"}="2H52 BLM";
$resn_exch{"H551 BLM"}="1H55 BLM"; $resn_exch{"H552 BLM"}="2H55 BLM"; $resn_exch{"H553 BLM"}="3H55 BLM";
$resn_exch{"H541 BLM"}="1H54 BLM"; $resn_exch{"H542 BLM"}="2H54 BLM"; $resn_exch{"H543 BLM"}="3H54 BLM";
$resn_exch{"H611 BLM"}="1H61 BLM"; $resn_exch{"H612 BLM"}="2H61 BLM"; $resn_exch{"HNQ1 BLM"}="1HNQ BLM";
$resn_exch{"HNQ2 BLM"}="2HNQ BLM"; $resn_exch{"H661 BLM"}="1H66 BLM"; $resn_exch{"H662 BLM"}="2H66 BLM";
$resn_exch{"HC21 BLN"}="1HC2 BLN"; $resn_exch{"HC22 BLN"}="2HC2 BLN"; $resn_exch{"HC31 BLN"}="1HC3 BLN";
$resn_exch{"HC32 BLN"}="2HC3 BLN"; $resn_exch{"HC51 BLN"}="1HC5 BLN"; $resn_exch{"HC52 BLN"}="2HC5 BLN";
$resn_exch{"HC61 BLN"}="1HC6 BLN"; $resn_exch{"HC62 BLN"}="2HC6 BLN"; $resn_exch{"HCB1 BLN"}="1HCB BLN";
$resn_exch{"HCB2 BLN"}="2HCB BLN"; $resn_exch{"HD11 BLN"}="1HD1 BLN"; $resn_exch{"HD12 BLN"}="2HD1 BLN";
$resn_exch{"HD13 BLN"}="3HD1 BLN"; $resn_exch{"HD21 BLN"}="1HD2 BLN"; $resn_exch{"HD22 BLN"}="2HD2 BLN";
$resn_exch{"HD23 BLN"}="3HD2 BLN"; $resn_exch{"HB'1 BLN"}="1HB' BLN"; $resn_exch{"HB'2 BLN"}="2HB' BLN";
$resn_exch{"HCD1 BLN"}="1HCD BLN"; $resn_exch{"HCD2 BLN"}="2HCD BLN"; $resn_exch{"H2'C BLN"}="CH2' BLN";
$resn_exch{"H3'C BLN"}="CH3' BLN"; $resn_exch{"H4'C BLN"}="CH4' BLN"; $resn_exch{"H5'C BLN"}="CH5' BLN";
$resn_exch{"H6'C BLN"}="CH6' BLN"; $resn_exch{"HA'C BLN"}="CHA' BLN"; $resn_exch{" H81 BLO"}="1H8  BLO";
$resn_exch{" H82 BLO"}="2H8  BLO"; $resn_exch{"H101 BLO"}="1H10 BLO"; $resn_exch{"H102 BLO"}="2H10 BLO";
$resn_exch{"H111 BLO"}="1H11 BLO"; $resn_exch{"H112 BLO"}="2H11 BLO"; $resn_exch{"H141 BLO"}="1H14 BLO";
$resn_exch{"H142 BLO"}="2H14 BLO"; $resn_exch{"H131 BLO"}="1H13 BLO"; $resn_exch{"H132 BLO"}="2H13 BLO";
$resn_exch{"H133 BLO"}="3H13 BLO"; $resn_exch{"HN91 BLO"}="1HN9 BLO"; $resn_exch{"HN92 BLO"}="2HN9 BLO";
$resn_exch{" H91 BLP"}="1H9  BLP"; $resn_exch{" H92 BLP"}="2H9  BLP"; $resn_exch{" H71 BLP"}="1H7  BLP";
$resn_exch{" H72 BLP"}="2H7  BLP"; $resn_exch{" H11 BLP"}="1H1  BLP"; $resn_exch{"H12A BLP"}="2H1  BLP";
$resn_exch{"H13A BLP"}="3H1  BLP"; $resn_exch{" H51 BLP"}="1H5  BLP"; $resn_exch{" H52 BLP"}="2H5  BLP";
$resn_exch{"HN41 BLS"}="1HN4 BLS"; $resn_exch{"HN42 BLS"}="2HN4 BLS"; $resn_exch{" H1' BLS"}=" H1* BLS";
$resn_exch{" H2' BLS"}=" H2* BLS"; $resn_exch{" H3' BLS"}=" H3* BLS"; $resn_exch{" H4' BLS"}=" H4* BLS";
$resn_exch{" H5' BLS"}=" H5* BLS"; $resn_exch{" H81 BLS"}="1H8  BLS"; $resn_exch{" H82 BLS"}="2H8  BLS";
$resn_exch{"HN91 BLS"}="1HN9 BLS"; $resn_exch{"HN92 BLS"}="2HN9 BLS"; $resn_exch{"H101 BLS"}="1H10 BLS";
$resn_exch{"H102 BLS"}="2H10 BLS"; $resn_exch{"H111 BLS"}="1H11 BLS"; $resn_exch{"H112 BLS"}="2H11 BLS";
$resn_exch{"H131 BLS"}="1H13 BLS"; $resn_exch{"H132 BLS"}="2H13 BLS"; $resn_exch{"H133 BLS"}="3H13 BLS";
$resn_exch{"H151 BLS"}="1H15 BLS"; $resn_exch{"H152 BLS"}="2H15 BLS"; $resn_exch{" H31 BLT"}="1H3  BLT";
$resn_exch{" H32 BLT"}="2H3  BLT"; $resn_exch{" H61 BLT"}="1H6  BLT"; $resn_exch{" H62 BLT"}="2H6  BLT";
$resn_exch{" H71 BLT"}="1H7  BLT"; $resn_exch{" H72 BLT"}="2H7  BLT"; $resn_exch{"H101 BLT"}="1H10 BLT";
$resn_exch{"H102 BLT"}="2H10 BLT"; $resn_exch{" HA  BLV"}=" H_A BLV"; $resn_exch{"HMA1 BLV"}="1HMA BLV";
$resn_exch{"HMA2 BLV"}="2HMA BLV"; $resn_exch{"HMA3 BLV"}="3HMA BLV"; $resn_exch{"HAA1 BLV"}="1HAA BLV";
$resn_exch{"HAA2 BLV"}="2HAA BLV"; $resn_exch{"HBA1 BLV"}="1HBA BLV"; $resn_exch{"HBA2 BLV"}="2HBA BLV";
$resn_exch{" HB  BLV"}=" H_B BLV"; $resn_exch{"HMB1 BLV"}="1HMB BLV"; $resn_exch{"HMB2 BLV"}="2HMB BLV";
$resn_exch{"HMB3 BLV"}="3HMB BLV"; $resn_exch{"HBB1 BLV"}="1HBB BLV"; $resn_exch{"HBB2 BLV"}="2HBB BLV";
$resn_exch{" HC  BLV"}=" H_C BLV"; $resn_exch{"HMC1 BLV"}="1HMC BLV"; $resn_exch{"HMC2 BLV"}="2HMC BLV";
$resn_exch{"HMC3 BLV"}="3HMC BLV"; $resn_exch{"HBC1 BLV"}="1HBC BLV"; $resn_exch{"HBC2 BLV"}="2HBC BLV";
$resn_exch{"HMD1 BLV"}="1HMD BLV"; $resn_exch{"HMD2 BLV"}="2HMD BLV"; $resn_exch{"HMD3 BLV"}="3HMD BLV";
$resn_exch{"HAD1 BLV"}="1HAD BLV"; $resn_exch{"HAD2 BLV"}="2HAD BLV"; $resn_exch{"HBD1 BLV"}="1HBD BLV";
$resn_exch{"HBD2 BLV"}="2HBD BLV"; $resn_exch{" HN1 BLY"}="1HN  BLY"; $resn_exch{" HN2 BLY"}="2HN  BLY";
$resn_exch{" HB1 BLY"}="1HB  BLY"; $resn_exch{" HB2 BLY"}="2HB  BLY"; $resn_exch{" HG1 BLY"}="1HG  BLY";
$resn_exch{" HG2 BLY"}="2HG  BLY"; $resn_exch{" HD1 BLY"}="1HD  BLY"; $resn_exch{" HD2 BLY"}="2HD  BLY";
$resn_exch{" HE1 BLY"}="1HE  BLY"; $resn_exch{" HE2 BLY"}="2HE  BLY"; $resn_exch{" HZ1 BLY"}="1HZ  BLY";
$resn_exch{" HZ2 BLY"}="2HZ  BLY"; $resn_exch{" HZ3 BLY"}="3HZ  BLY"; $resn_exch{" H11 BLZ"}="1H1  BLZ";
$resn_exch{" H12 BLZ"}="2H1  BLZ"; $resn_exch{"H13A BLZ"}="3H1  BLZ"; $resn_exch{"H141 BLZ"}="1H14 BLZ";
$resn_exch{"H142 BLZ"}="2H14 BLZ"; $resn_exch{"H143 BLZ"}="3H14 BLZ"; $resn_exch{"H261 BLZ"}="1H26 BLZ";
$resn_exch{"H262 BLZ"}="2H26 BLZ"; $resn_exch{"H311 BLZ"}="1H31 BLZ"; $resn_exch{"H312 BLZ"}="2H31 BLZ";
$resn_exch{"H313 BLZ"}="3H31 BLZ"; $resn_exch{" H21 BM1"}="1H2  BM1"; $resn_exch{" H22 BM1"}="2H2  BM1";
$resn_exch{" H31 BM1"}="1H3  BM1"; $resn_exch{" H32 BM1"}="2H3  BM1"; $resn_exch{" H81 BM1"}="1H8  BM1";
$resn_exch{" H91 BM1"}="1H9  BM1"; $resn_exch{"H101 BM1"}="1H10 BM1"; $resn_exch{"H111 BM1"}="1H11 BM1";
$resn_exch{"H141 BM1"}="1H14 BM1"; $resn_exch{" HA1 BM1"}="1HA  BM1"; $resn_exch{" HA2 BM1"}="2HA  BM1";
$resn_exch{" HA3 BM1"}="3HA  BM1"; $resn_exch{"H161 BM1"}="1H16 BM1"; $resn_exch{"H171 BM1"}="1H17 BM1";
$resn_exch{"H181 BM1"}="1H18 BM1"; $resn_exch{"H161 BM2"}="1H16 BM2"; $resn_exch{"H162 BM2"}="2H16 BM2";
$resn_exch{"H211 BM2"}="1H21 BM2"; $resn_exch{"H212 BM2"}="2H21 BM2"; $resn_exch{"H221 BM2"}="1H22 BM2";
$resn_exch{"H222 BM2"}="2H22 BM2"; $resn_exch{"H231 BM2"}="1H23 BM2"; $resn_exch{"H232 BM2"}="2H23 BM2";
$resn_exch{"H251 BM2"}="1H25 BM2"; $resn_exch{"H252 BM2"}="2H25 BM2"; $resn_exch{"H291 BM2"}="1H29 BM2";
$resn_exch{"H292 BM2"}="2H29 BM2"; $resn_exch{"H301 BM2"}="1H30 BM2"; $resn_exch{"H302 BM2"}="2H30 BM2";
$resn_exch{"H311 BM2"}="1H31 BM2"; $resn_exch{"H312 BM2"}="2H31 BM2"; $resn_exch{"HN41 BM2"}="1HN4 BM2";
$resn_exch{"HN42 BM2"}="2HN4 BM2"; $resn_exch{"H8C1 BM3"}="1H8C BM3"; $resn_exch{"H8C2 BM3"}="2H8C BM3";
$resn_exch{"H8C3 BM3"}="3H8C BM3"; $resn_exch{"H6C1 BM3"}="1H6C BM3"; $resn_exch{"H6C2 BM3"}="2H6C BM3";
$resn_exch{" H21 BM5"}="1H2  BM5"; $resn_exch{" H22 BM5"}="2H2  BM5"; $resn_exch{" H81 BM5"}="1H8  BM5";
$resn_exch{" H91 BM5"}="1H9  BM5"; $resn_exch{"H101 BM5"}="1H10 BM5"; $resn_exch{"H111 BM5"}="1H11 BM5";
$resn_exch{"H141 BM5"}="1H14 BM5"; $resn_exch{"H151 BM5"}="1H15 BM5"; $resn_exch{"H161 BM5"}="1H16 BM5";
$resn_exch{"H171 BM5"}="1H17 BM5"; $resn_exch{"H181 BM5"}="1H18 BM5"; $resn_exch{" HD1 BM5"}="1HD  BM5";
$resn_exch{" HD2 BM5"}="2HD  BM5"; $resn_exch{" HD3 BM5"}="3HD  BM5"; $resn_exch{"H161 BM6"}="1H16 BM6";
$resn_exch{"H162 BM6"}="2H16 BM6"; $resn_exch{"H171 BM6"}="1H17 BM6"; $resn_exch{"H172 BM6"}="2H17 BM6";
$resn_exch{"H191 BM6"}="1H19 BM6"; $resn_exch{"H192 BM6"}="2H19 BM6"; $resn_exch{"H201 BM6"}="1H20 BM6";
$resn_exch{"H202 BM6"}="2H20 BM6"; $resn_exch{"H211 BM6"}="1H21 BM6"; $resn_exch{"H212 BM6"}="2H21 BM6";
$resn_exch{"H213 BM6"}="3H21 BM6"; $resn_exch{"H221 BM6"}="1H22 BM6"; $resn_exch{"H222 BM6"}="2H22 BM6";
$resn_exch{"H223 BM6"}="3H22 BM6"; $resn_exch{"H231 BM6"}="1H23 BM6"; $resn_exch{"H232 BM6"}="2H23 BM6";
$resn_exch{"H233 BM6"}="3H23 BM6"; $resn_exch{"H241 BM6"}="1H24 BM6"; $resn_exch{"H242 BM6"}="2H24 BM6";
$resn_exch{"H243 BM6"}="3H24 BM6"; $resn_exch{"H161 BM9"}="1H16 BM9"; $resn_exch{"H162 BM9"}="2H16 BM9";
$resn_exch{"H211 BM9"}="1H21 BM9"; $resn_exch{"H212 BM9"}="2H21 BM9"; $resn_exch{"H221 BM9"}="1H22 BM9";
$resn_exch{"H222 BM9"}="2H22 BM9"; $resn_exch{"H231 BM9"}="1H23 BM9"; $resn_exch{"H232 BM9"}="2H23 BM9";
$resn_exch{"H251 BM9"}="1H25 BM9"; $resn_exch{"H252 BM9"}="2H25 BM9"; $resn_exch{"H301 BM9"}="1H30 BM9";
$resn_exch{"H302 BM9"}="2H30 BM9"; $resn_exch{"H321 BM9"}="1H32 BM9"; $resn_exch{"H322 BM9"}="2H32 BM9";
$resn_exch{"H331 BM9"}="1H33 BM9"; $resn_exch{"H332 BM9"}="2H33 BM9"; $resn_exch{"H341 BM9"}="1H34 BM9";
$resn_exch{"H342 BM9"}="2H34 BM9"; $resn_exch{"HN61 BM9"}="1HN6 BM9"; $resn_exch{"HN62 BM9"}="2HN6 BM9";
$resn_exch{" H61 BMA"}="1H6  BMA"; $resn_exch{" H62 BMA"}="2H6  BMA"; $resn_exch{" H33 BMD"}="3H3  BMD";
$resn_exch{" H32 BMD"}="2H3  BMD"; $resn_exch{" H31 BMD"}="1H3  BMD"; $resn_exch{" H22 BMD"}="2H2  BMD";
$resn_exch{" H21 BMD"}="1H2  BMD"; $resn_exch{" H12 BMD"}="2H1  BMD"; $resn_exch{" H11 BMD"}="1H1  BMD";
$resn_exch{" HN1 BMD"}="1HN  BMD"; $resn_exch{" HN2 BMD"}="2HN  BMD"; $resn_exch{" H11 BME"}="1H1  BME";
$resn_exch{" H12 BME"}="2H1  BME"; $resn_exch{" H21 BME"}="1H2  BME"; $resn_exch{" H22 BME"}="2H2  BME";
$resn_exch{"H621 BMH"}="1H62 BMH"; $resn_exch{"H622 BMH"}="2H62 BMH"; $resn_exch{"H623 BMH"}="3H62 BMH";
$resn_exch{"H111 BMH"}="1H11 BMH"; $resn_exch{"H112 BMH"}="2H11 BMH"; $resn_exch{"H113 BMH"}="3H11 BMH";
$resn_exch{"H231 BMH"}="1H23 BMH"; $resn_exch{"H232 BMH"}="2H23 BMH"; $resn_exch{"H271 BMH"}="1H27 BMH";
$resn_exch{"H272 BMH"}="2H27 BMH"; $resn_exch{" H41 BMI"}="1H4  BMI"; $resn_exch{" H42 BMI"}="2H4  BMI";
$resn_exch{"H211 BMI"}="1H21 BMI"; $resn_exch{"H212 BMI"}="2H21 BMI"; $resn_exch{"H213 BMI"}="3H21 BMI";
$resn_exch{" H11 BMM"}="1H1  BMM"; $resn_exch{" H12 BMM"}="2H1  BMM"; $resn_exch{" H13 BMM"}="3H1  BMM";
$resn_exch{"H171 BMO"}="1H17 BMO"; $resn_exch{"H191 BMO"}="1H19 BMO"; $resn_exch{"H192 BMO"}="2H19 BMO";
$resn_exch{"H361 BMO"}="1H36 BMO"; $resn_exch{"H362 BMO"}="2H36 BMO"; $resn_exch{"H221 BMO"}="1H22 BMO";
$resn_exch{"H222 BMO"}="2H22 BMO"; $resn_exch{" H11 BMO"}="1H1  BMO"; $resn_exch{"H12A BMO"}="2H1  BMO";
$resn_exch{" H21 BMO"}="1H2  BMO"; $resn_exch{" H22 BMO"}="2H2  BMO"; $resn_exch{" H31 BMO"}="1H3  BMO";
$resn_exch{"H32A BMO"}="2H3  BMO"; $resn_exch{"HOP2 BMP"}="2HOP BMP"; $resn_exch{"HOP3 BMP"}="3HOP BMP";
$resn_exch{" H5' BMP"}="1H5* BMP"; $resn_exch{"H5'' BMP"}="2H5* BMP"; $resn_exch{" H4' BMP"}=" H4* BMP";
$resn_exch{" H3' BMP"}=" H3* BMP"; $resn_exch{"HO3' BMP"}="*HO3 BMP"; $resn_exch{" H2' BMP"}=" H2* BMP";
$resn_exch{"HO2' BMP"}="*HO2 BMP"; $resn_exch{" H1' BMP"}=" H1* BMP"; $resn_exch{" H51 BMP"}="1H5  BMP";
$resn_exch{"HOP2 BMQ"}="2HOP BMQ"; $resn_exch{"HOP3 BMQ"}="3HOP BMQ"; $resn_exch{"H5'1 BMQ"}="1H5* BMQ";
$resn_exch{"H5'2 BMQ"}="2H5* BMQ"; $resn_exch{" H4' BMQ"}=" H4* BMQ"; $resn_exch{" H3' BMQ"}=" H3* BMQ";
$resn_exch{"HO3' BMQ"}="*HO3 BMQ"; $resn_exch{" H2' BMQ"}=" H2* BMQ"; $resn_exch{"HO2' BMQ"}="*HO2 BMQ";
$resn_exch{" H1' BMQ"}=" H1* BMQ"; $resn_exch{" H51 BMQ"}="1H5  BMQ"; $resn_exch{" H52 BMQ"}="2H5  BMQ";
$resn_exch{"H161 BMS"}="1H16 BMS"; $resn_exch{"H162 BMS"}="2H16 BMS"; $resn_exch{"H171 BMS"}="1H17 BMS";
$resn_exch{"H172 BMS"}="2H17 BMS"; $resn_exch{"H191 BMS"}="1H19 BMS"; $resn_exch{"H192 BMS"}="2H19 BMS";
$resn_exch{"H193 BMS"}="3H19 BMS"; $resn_exch{"H201 BMS"}="1H20 BMS"; $resn_exch{"H202 BMS"}="2H20 BMS";
$resn_exch{"H203 BMS"}="3H20 BMS"; $resn_exch{" H   BMT"}=" HN  BMT"; $resn_exch{" HN1 BMT"}="1HCN BMT";
$resn_exch{" HN2 BMT"}="2HCN BMT"; $resn_exch{" HN3 BMT"}="3HCN BMT"; $resn_exch{"HD11 BMT"}="1HD1 BMT";
$resn_exch{"HD12 BMT"}="2HD1 BMT"; $resn_exch{"HD13 BMT"}="3HD1 BMT"; $resn_exch{"HD22 BMT"}="1HD2 BMT";
$resn_exch{"HD23 BMT"}="2HD2 BMT"; $resn_exch{" HH1 BMT"}="1HH  BMT"; $resn_exch{" HH2 BMT"}="2HH  BMT";
$resn_exch{" HH3 BMT"}="3HH  BMT"; $resn_exch{"H151 BMU"}="1H15 BMU"; $resn_exch{"H152 BMU"}="2H15 BMU";
$resn_exch{"H153 BMU"}="3H15 BMU"; $resn_exch{"H171 BMU"}="1H17 BMU"; $resn_exch{"H172 BMU"}="2H17 BMU";
$resn_exch{"H173 BMU"}="3H17 BMU"; $resn_exch{"H181 BMU"}="1H18 BMU"; $resn_exch{"H182 BMU"}="2H18 BMU";
$resn_exch{"H183 BMU"}="3H18 BMU"; $resn_exch{"H191 BMU"}="1H19 BMU"; $resn_exch{"H192 BMU"}="2H19 BMU";
$resn_exch{"H193 BMU"}="3H19 BMU"; $resn_exch{"HAR1 BMV"}="1HAR BMV"; $resn_exch{"HAR2 BMV"}="2HAR BMV";
$resn_exch{"HAU1 BMV"}="1HAU BMV"; $resn_exch{"HAU2 BMV"}="2HAU BMV"; $resn_exch{"HAS1 BMV"}="1HAS BMV";
$resn_exch{"HAS2 BMV"}="2HAS BMV"; $resn_exch{"HAT1 BMV"}="1HAT BMV"; $resn_exch{"HAT2 BMV"}="2HAT BMV";
$resn_exch{"H281 BMY"}="1H28 BMY"; $resn_exch{"H282 BMY"}="2H28 BMY"; $resn_exch{"H311 BMY"}="1H31 BMY";
$resn_exch{"H312 BMY"}="2H31 BMY"; $resn_exch{"HO08 BMY"}="8HO0 BMY"; $resn_exch{"H2'1 BMY"}="1H2' BMY";
$resn_exch{"H2'2 BMY"}="2H2' BMY"; $resn_exch{"H7'1 BMY"}="1H7' BMY"; $resn_exch{"H7'2 BMY"}="2H7' BMY";
$resn_exch{"H7'3 BMY"}="3H7' BMY"; $resn_exch{"H8'1 BMY"}="1H8' BMY"; $resn_exch{"H8'2 BMY"}="2H8' BMY";
$resn_exch{"HO'9 BMY"}="9HO' BMY"; $resn_exch{"HO'1 BMY"}="1HO\" BMY"; $resn_exch{"H2A1 BMY"}="1H2\" BMY";
$resn_exch{"H2A2 BMY"}="2H2\" BMY"; $resn_exch{"H2'3 BMY"}="3H2\" BMY"; $resn_exch{" H3' BMY"}=" H3\" BMY";
$resn_exch{" H4' BMY"}=" H4\" BMY"; $resn_exch{" H5B BMY"}=" H5\" BMY"; $resn_exch{" H6' BMY"}=" H6\" BMY";
$resn_exch{" H7' BMY"}=" H7\" BMY"; $resn_exch{" H9' BMY"}=" H9\" BMY"; $resn_exch{"HOA1 BMY"}="1HOA BMY";
$resn_exch{"HOA2 BMY"}="2HOA BMY"; $resn_exch{"H3A1 BMY"}="1H3A BMY"; $resn_exch{"H3A2 BMY"}="2H3A BMY";
$resn_exch{"HH11 BMZ"}="1HH1 BMZ"; $resn_exch{"HH12 BMZ"}="2HH1 BMZ"; $resn_exch{"HH21 BMZ"}="1HH2 BMZ";
$resn_exch{"HH22 BMZ"}="2HH2 BMZ"; $resn_exch{"H151 BN2"}="1H15 BN2"; $resn_exch{"H152 BN2"}="2H15 BN2";
$resn_exch{"H161 BN2"}="1H16 BN2"; $resn_exch{"H162 BN2"}="2H16 BN2"; $resn_exch{"H171 BN2"}="1H17 BN2";
$resn_exch{"H172 BN2"}="2H17 BN2"; $resn_exch{"H181 BN2"}="1H18 BN2"; $resn_exch{"H182 BN2"}="2H18 BN2";
$resn_exch{"H191 BN2"}="1H19 BN2"; $resn_exch{"H192 BN2"}="2H19 BN2"; $resn_exch{"H201 BN2"}="1H20 BN2";
$resn_exch{"H202 BN2"}="2H20 BN2"; $resn_exch{"H151 BN3"}="1H15 BN3"; $resn_exch{"H152 BN3"}="2H15 BN3";
$resn_exch{"H161 BN3"}="1H16 BN3"; $resn_exch{"H162 BN3"}="2H16 BN3"; $resn_exch{"H171 BN3"}="1H17 BN3";
$resn_exch{"H172 BN3"}="2H17 BN3"; $resn_exch{"H151 BN4"}="1H15 BN4"; $resn_exch{"H152 BN4"}="2H15 BN4";
$resn_exch{"H161 BN4"}="1H16 BN4"; $resn_exch{"H162 BN4"}="2H16 BN4"; $resn_exch{"H171 BN4"}="1H17 BN4";
$resn_exch{"H172 BN4"}="2H17 BN4"; $resn_exch{"H191 BN4"}="1H19 BN4"; $resn_exch{"H192 BN4"}="2H19 BN4";
$resn_exch{"H193 BN4"}="3H19 BN4"; $resn_exch{"H201 BN4"}="1H20 BN4"; $resn_exch{"H202 BN4"}="2H20 BN4";
$resn_exch{"H203 BN4"}="3H20 BN4"; $resn_exch{"H151 BN5"}="1H15 BN5"; $resn_exch{"H152 BN5"}="2H15 BN5";
$resn_exch{"H153 BN5"}="3H15 BN5"; $resn_exch{"H161 BN5"}="1H16 BN5"; $resn_exch{"H162 BN5"}="2H16 BN5";
$resn_exch{"H171 BN5"}="1H17 BN5"; $resn_exch{"H172 BN5"}="2H17 BN5"; $resn_exch{"H181 BN5"}="1H18 BN5";
$resn_exch{"H182 BN5"}="2H18 BN5"; $resn_exch{"H191 BN5"}="1H19 BN5"; $resn_exch{"H192 BN5"}="2H19 BN5";
$resn_exch{"H41A BN6"}="1H4  BN6"; $resn_exch{" H42 BN6"}="2H4  BN6"; $resn_exch{" H61 BN6"}="1H6  BN6";
$resn_exch{" H62 BN6"}="2H6  BN6"; $resn_exch{" H63 BN6"}="3H6  BN6"; $resn_exch{" H71 BN6"}="1H7  BN6";
$resn_exch{" H72 BN6"}="2H7  BN6"; $resn_exch{" H73 BN6"}="3H7  BN6"; $resn_exch{"H111 BN6"}="1H11 BN6";
$resn_exch{"H112 BN6"}="2H11 BN6"; $resn_exch{"H121 BN6"}="1H12 BN6"; $resn_exch{"H122 BN6"}="2H12 BN6";
$resn_exch{"H131 BN6"}="1H13 BN6"; $resn_exch{"H132 BN6"}="2H13 BN6"; $resn_exch{"H141 BN6"}="1H14 BN6";
$resn_exch{"H142 BN6"}="2H14 BN6"; $resn_exch{"H151 BN6"}="1H15 BN6"; $resn_exch{"H152 BN6"}="2H15 BN6";
$resn_exch{"H221 BN6"}="1H22 BN6"; $resn_exch{"H222 BN6"}="2H22 BN6"; $resn_exch{"H241 BN6"}="1H24 BN6";
$resn_exch{"H242 BN6"}="2H24 BN6"; $resn_exch{"H251 BN6"}="1H25 BN6"; $resn_exch{"H252 BN6"}="2H25 BN6";
$resn_exch{"H281 BN6"}="1H28 BN6"; $resn_exch{"H282 BN6"}="2H28 BN6"; $resn_exch{"H301 BN6"}="1H30 BN6";
$resn_exch{"H302 BN6"}="2H30 BN6"; $resn_exch{"H311 BN6"}="1H31 BN6"; $resn_exch{"H312 BN6"}="2H31 BN6";
$resn_exch{"H361 BN6"}="1H36 BN6"; $resn_exch{"H362 BN6"}="2H36 BN6"; $resn_exch{"HB11 BNA"}="1HB1 BNA";
$resn_exch{"HB12 BNA"}="2HB1 BNA"; $resn_exch{"HB21 BNA"}="1HB2 BNA"; $resn_exch{"HB22 BNA"}="2HB2 BNA";
$resn_exch{"HY11 BND"}="1HY1 BND"; $resn_exch{"HY12 BND"}="2HY1 BND"; $resn_exch{"HY13 BND"}="3HY1 BND";
$resn_exch{"HX31 BND"}="1HX3 BND"; $resn_exch{"HX32 BND"}="2HX3 BND"; $resn_exch{"H111 BND"}="1H11 BND";
$resn_exch{"H112 BND"}="2H11 BND"; $resn_exch{"H113 BND"}="3H11 BND"; $resn_exch{"HX91 BND"}="1HX9 BND";
$resn_exch{"HX92 BND"}="2HX9 BND"; $resn_exch{" H41 BNE"}="1H4  BNE"; $resn_exch{" H42 BNE"}="2H4  BNE";
$resn_exch{" H51 BNE"}="1H5  BNE"; $resn_exch{" H52 BNE"}="2H5  BNE"; $resn_exch{" H61 BNE"}="1H6  BNE";
$resn_exch{" H71 BNE"}="1H7  BNE"; $resn_exch{"H101 BNE"}="1H10 BNE"; $resn_exch{"H102 BNE"}="2H10 BNE";
$resn_exch{"H111 BNE"}="1H11 BNE"; $resn_exch{"H112 BNE"}="2H11 BNE"; $resn_exch{"H113 BNE"}="3H11 BNE";
$resn_exch{"H131 BNE"}="1H13 BNE"; $resn_exch{"H141 BNE"}="1H14 BNE"; $resn_exch{"H142 BNE"}="2H14 BNE";
$resn_exch{"H151 BNE"}="1H15 BNE"; $resn_exch{"H152 BNE"}="2H15 BNE"; $resn_exch{"H153 BNE"}="3H15 BNE";
$resn_exch{"H161 BNE"}="1H16 BNE"; $resn_exch{"H162 BNE"}="2H16 BNE"; $resn_exch{"H181 BNE"}="1H18 BNE";
$resn_exch{"H211 BNE"}="1H21 BNE"; $resn_exch{"H221 BNE"}="1H22 BNE"; $resn_exch{"H251 BNE"}="1H25 BNE";
$resn_exch{"H291 BNE"}="1H29 BNE"; $resn_exch{"H292 BNE"}="2H29 BNE"; $resn_exch{"H293 BNE"}="3H29 BNE";
$resn_exch{"H301 BNE"}="1H30 BNE"; $resn_exch{"H302 BNE"}="2H30 BNE"; $resn_exch{"H321 BNE"}="1H32 BNE";
$resn_exch{"H331 BNE"}="1H33 BNE"; $resn_exch{"H341 BNE"}="1H34 BNE"; $resn_exch{"H342 BNE"}="2H34 BNE";
$resn_exch{"H343 BNE"}="3H34 BNE"; $resn_exch{"H121 BNF"}="1H12 BNF"; $resn_exch{"H122 BNF"}="2H12 BNF";
$resn_exch{" H61 BNG"}="1H6  BNG"; $resn_exch{" H62 BNG"}="2H6  BNG"; $resn_exch{"H1'1 BNG"}="1H1' BNG";
$resn_exch{"H1'2 BNG"}="2H1' BNG"; $resn_exch{"H2'1 BNG"}="1H2' BNG"; $resn_exch{"H2'2 BNG"}="2H2' BNG";
$resn_exch{"H3'1 BNG"}="1H3' BNG"; $resn_exch{"H3'2 BNG"}="2H3' BNG"; $resn_exch{"H4'1 BNG"}="1H4' BNG";
$resn_exch{"H4'2 BNG"}="2H4' BNG"; $resn_exch{"H5'1 BNG"}="1H5' BNG"; $resn_exch{"H5'2 BNG"}="2H5' BNG";
$resn_exch{"H6'1 BNG"}="1H6' BNG"; $resn_exch{"H6'2 BNG"}="2H6' BNG"; $resn_exch{"H7'1 BNG"}="1H7' BNG";
$resn_exch{"H7'2 BNG"}="2H7' BNG"; $resn_exch{"H8'1 BNG"}="1H8' BNG"; $resn_exch{"H8'2 BNG"}="2H8' BNG";
$resn_exch{"H9'1 BNG"}="1H9' BNG"; $resn_exch{"H9'2 BNG"}="2H9' BNG"; $resn_exch{"H9'3 BNG"}="3H9' BNG";
$resn_exch{"H41A BNH"}="1H4  BNH"; $resn_exch{" H42 BNH"}="2H4  BNH"; $resn_exch{" H61 BNH"}="1H6  BNH";
$resn_exch{" H62 BNH"}="2H6  BNH"; $resn_exch{" H63 BNH"}="3H6  BNH"; $resn_exch{" H71 BNH"}="1H7  BNH";
$resn_exch{" H72 BNH"}="2H7  BNH"; $resn_exch{" H73 BNH"}="3H7  BNH"; $resn_exch{"H111 BNH"}="1H11 BNH";
$resn_exch{"H112 BNH"}="2H11 BNH"; $resn_exch{"H121 BNH"}="1H12 BNH"; $resn_exch{"H122 BNH"}="2H12 BNH";
$resn_exch{"H131 BNH"}="1H13 BNH"; $resn_exch{"H132 BNH"}="2H13 BNH"; $resn_exch{"H141 BNH"}="1H14 BNH";
$resn_exch{"H142 BNH"}="2H14 BNH"; $resn_exch{"H151 BNH"}="1H15 BNH"; $resn_exch{"H152 BNH"}="2H15 BNH";
$resn_exch{"H221 BNH"}="1H22 BNH"; $resn_exch{"H222 BNH"}="2H22 BNH"; $resn_exch{"H241 BNH"}="1H24 BNH";
$resn_exch{"H242 BNH"}="2H24 BNH"; $resn_exch{"H251 BNH"}="1H25 BNH"; $resn_exch{"H252 BNH"}="2H25 BNH";
$resn_exch{"H281 BNH"}="1H28 BNH"; $resn_exch{"H282 BNH"}="2H28 BNH"; $resn_exch{"H301 BNH"}="1H30 BNH";
$resn_exch{"H302 BNH"}="2H30 BNH"; $resn_exch{"H311 BNH"}="1H31 BNH"; $resn_exch{"H312 BNH"}="2H31 BNH";
$resn_exch{"H371 BNH"}="1H37 BNH"; $resn_exch{"H372 BNH"}="2H37 BNH"; $resn_exch{"H101 BNI"}="1H10 BNI";
$resn_exch{"H102 BNI"}="2H10 BNI"; $resn_exch{" H91 BNI"}="1H9  BNI"; $resn_exch{" H92 BNI"}="2H9  BNI";
$resn_exch{" H81 BNI"}="1H8  BNI"; $resn_exch{" H82 BNI"}="2H8  BNI"; $resn_exch{" H71 BNI"}="1H7  BNI";
$resn_exch{" H72 BNI"}="2H7  BNI"; $resn_exch{"H21A BNI"}="1H2  BNI"; $resn_exch{" H61 BNI"}="1H6  BNI";
$resn_exch{" H62 BNI"}="2H6  BNI"; $resn_exch{"HN11 BNI"}="1HN1 BNI"; $resn_exch{"HN21 BNI"}="1HN2 BNI";
$resn_exch{"H171 BNI"}="1H17 BNI"; $resn_exch{"H171 BNL"}="1H17 BNL"; $resn_exch{"H172 BNL"}="2H17 BNL";
$resn_exch{"H151 BNL"}="1H15 BNL"; $resn_exch{"H152 BNL"}="2H15 BNL"; $resn_exch{"H141 BNL"}="1H14 BNL";
$resn_exch{"H142 BNL"}="2H14 BNL"; $resn_exch{"H131 BNL"}="1H13 BNL"; $resn_exch{"H132 BNL"}="2H13 BNL";
$resn_exch{"H121 BNL"}="1H12 BNL"; $resn_exch{"H122 BNL"}="2H12 BNL"; $resn_exch{" H11 BNL"}="1H1  BNL";
$resn_exch{" H12 BNL"}="2H1  BNL"; $resn_exch{" H31 BNL"}="1H3  BNL"; $resn_exch{" H32 BNL"}="2H3  BNL";
$resn_exch{" H41 BNL"}="1H4  BNL"; $resn_exch{" H42 BNL"}="2H4  BNL"; $resn_exch{" H51 BNL"}="1H5  BNL";
$resn_exch{" H52 BNL"}="2H5  BNL"; $resn_exch{" H61 BNL"}="1H6  BNL"; $resn_exch{" H62 BNL"}="2H6  BNL";
$resn_exch{"HH11 BNN"}="1HH1 BNN"; $resn_exch{"HH12 BNN"}="2HH1 BNN"; $resn_exch{"HH13 BNN"}="3HH1 BNN";
$resn_exch{"H161 BNN"}="1H16 BNN"; $resn_exch{"H162 BNN"}="2H16 BNN"; $resn_exch{" H71 BNN"}="1H7  BNN";
$resn_exch{" H72 BNN"}="2H7  BNN"; $resn_exch{" HN1 BNO"}="1HN  BNO"; $resn_exch{" HN2 BNO"}="2HN  BNO";
$resn_exch{" HB1 BNO"}="1HB  BNO"; $resn_exch{" HB2 BNO"}="2HB  BNO"; $resn_exch{" HG1 BNO"}="1HG  BNO";
$resn_exch{" HG2 BNO"}="2HG  BNO"; $resn_exch{" HD1 BNO"}="1HD  BNO"; $resn_exch{" HD2 BNO"}="2HD  BNO";
$resn_exch{" HE1 BNO"}="1HE  BNO"; $resn_exch{" HE2 BNO"}="2HE  BNO"; $resn_exch{" HE3 BNO"}="3HE  BNO";
$resn_exch{" H81 BNP"}="1H8  BNP"; $resn_exch{" H82 BNP"}="2H8  BNP"; $resn_exch{" H91 BNP"}="1H9  BNP";
$resn_exch{" H92 BNP"}="2H9  BNP"; $resn_exch{"H101 BNP"}="1H10 BNP"; $resn_exch{"H102 BNP"}="2H10 BNP";
$resn_exch{"H111 BNP"}="1H11 BNP"; $resn_exch{"H112 BNP"}="2H11 BNP"; $resn_exch{"H201 BNP"}="1H20 BNP";
$resn_exch{"H202 BNP"}="2H20 BNP"; $resn_exch{"H211 BNP"}="1H21 BNP"; $resn_exch{"H212 BNP"}="2H21 BNP";
$resn_exch{"H221 BNP"}="1H22 BNP"; $resn_exch{"H222 BNP"}="2H22 BNP"; $resn_exch{"H231 BNP"}="1H23 BNP";
$resn_exch{"H232 BNP"}="2H23 BNP"; $resn_exch{"H111 BNR"}="1H11 BNR"; $resn_exch{"H112 BNR"}="2H11 BNR";
$resn_exch{"H141 BNR"}="1H14 BNR"; $resn_exch{"H142 BNR"}="2H14 BNR"; $resn_exch{"H143 BNR"}="3H14 BNR";
$resn_exch{"H151 BNR"}="1H15 BNR"; $resn_exch{"H152 BNR"}="2H15 BNR"; $resn_exch{"H211 BNR"}="1H21 BNR";
$resn_exch{"H212 BNR"}="2H21 BNR"; $resn_exch{"H213 BNR"}="3H21 BNR"; $resn_exch{" H1' BNR"}=" H1* BNR";
$resn_exch{"H2'1 BNR"}="1H2* BNR"; $resn_exch{"H2'2 BNR"}="2H2* BNR"; $resn_exch{" H3' BNR"}=" H3* BNR";
$resn_exch{"HO3' BNR"}="*HO3 BNR"; $resn_exch{" H4' BNR"}=" H4* BNR"; $resn_exch{"HN'1 BNR"}="1HN* BNR";
$resn_exch{"HN'2 BNR"}="2HN* BNR"; $resn_exch{" H5' BNR"}=" H5* BNR"; $resn_exch{"H6'1 BNR"}="1H6* BNR";
$resn_exch{"H6'2 BNR"}="2H6* BNR"; $resn_exch{"H6'3 BNR"}="3H6* BNR"; $resn_exch{"H7'1 BNR"}="1H7* BNR";
$resn_exch{"H7'2 BNR"}="2H7* BNR"; $resn_exch{" H1B BNR"}="'H1  BNR"; $resn_exch{" H2' BNR"}="'H2  BNR";
$resn_exch{" H3B BNR"}="'H3  BNR"; $resn_exch{"HO8' BNR"}="'HO8 BNR"; $resn_exch{"H10' BNR"}="'H10 BNR";
$resn_exch{"HL'1 BNR"}="1HL' BNR"; $resn_exch{"HL'2 BNR"}="2HL' BNR"; $resn_exch{"H12' BNR"}="'H12 BNR";
$resn_exch{"HK'1 BNR"}="1HK' BNR"; $resn_exch{"HK'2 BNR"}="2HK' BNR"; $resn_exch{"HK'3 BNR"}="3HK' BNR";
$resn_exch{"HJ'1 BNR"}="1HJ' BNR"; $resn_exch{"HJ'2 BNR"}="2HJ' BNR"; $resn_exch{"H17' BNR"}="'H17 BNR";
$resn_exch{"HI'1 BNR"}="1HI' BNR"; $resn_exch{"HI'2 BNR"}="2HI' BNR"; $resn_exch{"HI'3 BNR"}="3HI' BNR";
$resn_exch{" H1D BNR"}="'H1* BNR"; $resn_exch{" H2B BNR"}="1H2' BNR"; $resn_exch{" H2D BNR"}="2H2' BNR";
$resn_exch{" H3D BNR"}="'H3* BNR"; $resn_exch{"HO3A BNR"}="'HO3 BNR"; $resn_exch{" H4D BNR"}="'H4* BNR";
$resn_exch{" HNB BNR"}="1HN' BNR"; $resn_exch{" HND BNR"}="2HN' BNR"; $resn_exch{" H5D BNR"}="'H5* BNR";
$resn_exch{" H6B BNR"}="1H6' BNR"; $resn_exch{" H6D BNR"}="2H6' BNR"; $resn_exch{" H6E BNR"}="3H6' BNR";
$resn_exch{" H7B BNR"}="1H7' BNR"; $resn_exch{" H7D BNR"}="2H7' BNR"; $resn_exch{"H9A' BNR"}="'H9A BNR";
$resn_exch{"H9B' BNR"}="'H9B BNR"; $resn_exch{"HAA1 BNT"}="1HAA BNT"; $resn_exch{"HAA2 BNT"}="2HAA BNT";
$resn_exch{"HAA3 BNT"}="3HAA BNT"; $resn_exch{"HAC1 BNT"}="1HAC BNT"; $resn_exch{"HAC2 BNT"}="2HAC BNT";
$resn_exch{"HAC3 BNT"}="3HAC BNT"; $resn_exch{"HAM1 BNT"}="1HAM BNT"; $resn_exch{"HAM2 BNT"}="2HAM BNT";
$resn_exch{"HAM3 BNT"}="3HAM BNT"; $resn_exch{"HN11 BO1"}="1HN1 BO1"; $resn_exch{"HN12 BO1"}="2HN1 BO1";
$resn_exch{"H121 BO1"}="1H12 BO1"; $resn_exch{"H122 BO1"}="2H12 BO1"; $resn_exch{"H111 BO2"}="1H11 BO2";
$resn_exch{"H112 BO2"}="2H11 BO2"; $resn_exch{"H221 BO2"}="1H22 BO2"; $resn_exch{"H222 BO2"}="2H22 BO2";
$resn_exch{"H241 BO2"}="1H24 BO2"; $resn_exch{"H242 BO2"}="2H24 BO2"; $resn_exch{"H243 BO2"}="3H24 BO2";
$resn_exch{"H251 BO2"}="1H25 BO2"; $resn_exch{"H252 BO2"}="2H25 BO2"; $resn_exch{"H253 BO2"}="3H25 BO2";
$resn_exch{"H411 BO9"}="1H41 BO9"; $resn_exch{"H412 BO9"}="2H41 BO9"; $resn_exch{"H413 BO9"}="3H41 BO9";
$resn_exch{"H401 BO9"}="1H40 BO9"; $resn_exch{"H402 BO9"}="2H40 BO9"; $resn_exch{"H403 BO9"}="3H40 BO9";
$resn_exch{"H391 BO9"}="1H39 BO9"; $resn_exch{"H392 BO9"}="2H39 BO9"; $resn_exch{"H393 BO9"}="3H39 BO9";
$resn_exch{"H291 BO9"}="1H29 BO9"; $resn_exch{"H292 BO9"}="2H29 BO9"; $resn_exch{"H281 BO9"}="1H28 BO9";
$resn_exch{"H282 BO9"}="2H28 BO9"; $resn_exch{" H51 BO9"}="1H5  BO9"; $resn_exch{" H52 BO9"}="2H5  BO9";
$resn_exch{" H53 BO9"}="3H5  BO9"; $resn_exch{"H101 BO9"}="1H10 BO9"; $resn_exch{"H102 BO9"}="2H10 BO9";
$resn_exch{"H111 BO9"}="1H11 BO9"; $resn_exch{"H112 BO9"}="2H11 BO9"; $resn_exch{"H121 BO9"}="1H12 BO9";
$resn_exch{"H122 BO9"}="2H12 BO9"; $resn_exch{"H131 BO9"}="1H13 BO9"; $resn_exch{"H132 BO9"}="2H13 BO9";
$resn_exch{"H181 BO9"}="1H18 BO9"; $resn_exch{"H182 BO9"}="2H18 BO9"; $resn_exch{" H11 BOC"}="1H1  BOC";
$resn_exch{" H12 BOC"}="2H1  BOC"; $resn_exch{" H13 BOC"}="3H1  BOC"; $resn_exch{" H21 BOC"}="1H2  BOC";
$resn_exch{" H22 BOC"}="2H2  BOC"; $resn_exch{" H23 BOC"}="3H2  BOC"; $resn_exch{" H31 BOC"}="1H3  BOC";
$resn_exch{" H32 BOC"}="2H3  BOC"; $resn_exch{" H33 BOC"}="3H3  BOC"; $resn_exch{"HOP2 BOE"}="2HOP BOE";
$resn_exch{" H5' BOE"}="1H5* BOE"; $resn_exch{"H5'' BOE"}="2H5* BOE"; $resn_exch{" H4' BOE"}=" H4* BOE";
$resn_exch{" H1' BOE"}=" H1* BOE"; $resn_exch{" H71 BOE"}="1H5M BOE"; $resn_exch{" H72 BOE"}="2H5M BOE";
$resn_exch{" H73 BOE"}="3H5M BOE"; $resn_exch{" H2' BOE"}=" H2* BOE"; $resn_exch{"HB'1 BOE"}="1HB' BOE";
$resn_exch{"HB'2 BOE"}="2HB' BOE"; $resn_exch{"HC'1 BOE"}="1HC' BOE"; $resn_exch{"HC'2 BOE"}="2HC' BOE";
$resn_exch{"HE'1 BOE"}="1HE' BOE"; $resn_exch{"HE'2 BOE"}="2HE' BOE"; $resn_exch{" H3' BOE"}=" H3* BOE";
$resn_exch{"HO3' BOE"}="H3T  BOE"; $resn_exch{"HOP3 BOE"}="3HOP BOE"; $resn_exch{"H1'1 BOG"}="1H1' BOG";
$resn_exch{"H1'2 BOG"}="2H1' BOG"; $resn_exch{"H2'1 BOG"}="1H2' BOG"; $resn_exch{"H2'2 BOG"}="2H2' BOG";
$resn_exch{"H3'1 BOG"}="1H3' BOG"; $resn_exch{"H3'2 BOG"}="2H3' BOG"; $resn_exch{"H4'1 BOG"}="1H4' BOG";
$resn_exch{"H4'2 BOG"}="2H4' BOG"; $resn_exch{"H5'1 BOG"}="1H5' BOG"; $resn_exch{"H5'2 BOG"}="2H5' BOG";
$resn_exch{"H6'1 BOG"}="1H6' BOG"; $resn_exch{"H6'2 BOG"}="2H6' BOG"; $resn_exch{"H7'1 BOG"}="1H7' BOG";
$resn_exch{"H7'2 BOG"}="2H7' BOG"; $resn_exch{"H8'1 BOG"}="1H8' BOG"; $resn_exch{"H8'2 BOG"}="2H8' BOG";
$resn_exch{"H8'3 BOG"}="3H8' BOG"; $resn_exch{" H21 BOM"}="1H2  BOM"; $resn_exch{" H22 BOM"}="2H2  BOM";
$resn_exch{" H41 BOM"}="1H4  BOM"; $resn_exch{" H42 BOM"}="2H4  BOM"; $resn_exch{" H71 BOM"}="1H7  BOM";
$resn_exch{" H72 BOM"}="2H7  BOM"; $resn_exch{"H101 BOM"}="1H10 BOM"; $resn_exch{"H102 BOM"}="2H10 BOM";
$resn_exch{"H131 BOM"}="1H13 BOM"; $resn_exch{"H132 BOM"}="2H13 BOM"; $resn_exch{"H161 BOM"}="1H16 BOM";
$resn_exch{"H162 BOM"}="2H16 BOM"; $resn_exch{"H191 BOM"}="1H19 BOM"; $resn_exch{"H192 BOM"}="2H19 BOM";
$resn_exch{"H221 BOM"}="1H22 BOM"; $resn_exch{"H222 BOM"}="2H22 BOM"; $resn_exch{"H251 BOM"}="1H25 BOM";
$resn_exch{"H252 BOM"}="2H25 BOM"; $resn_exch{"H281 BOM"}="1H28 BOM"; $resn_exch{"H311 BOM"}="1H31 BOM";
$resn_exch{"H331 BOM"}="1H33 BOM"; $resn_exch{"H351 BOM"}="1H35 BOM"; $resn_exch{"H371 BOM"}="1H37 BOM";
$resn_exch{"H372 BOM"}="2H37 BOM"; $resn_exch{"H391 BOM"}="1H39 BOM"; $resn_exch{"H392 BOM"}="2H39 BOM";
$resn_exch{"H421 BOM"}="1H42 BOM"; $resn_exch{"H422 BOM"}="2H42 BOM"; $resn_exch{"H423 BOM"}="3H42 BOM";
$resn_exch{" H71 BOP"}="1H7  BOP"; $resn_exch{" H72 BOP"}="2H7  BOP"; $resn_exch{" H73 BOP"}="3H7  BOP";
$resn_exch{" HN1 BOR"}="1HN  BOR"; $resn_exch{" HN2 BOR"}="2HN  BOR"; $resn_exch{" HB1 BOR"}="1HB  BOR";
$resn_exch{" HB2 BOR"}="2HB  BOR"; $resn_exch{" HG1 BOR"}="1HG  BOR"; $resn_exch{" HG2 BOR"}="2HG  BOR";
$resn_exch{" HD1 BOR"}="1HD  BOR"; $resn_exch{" HD2 BOR"}="2HD  BOR"; $resn_exch{"HH21 BOR"}="1HH2 BOR";
$resn_exch{"HH22 BOR"}="2HH2 BOR"; $resn_exch{"HN11 BOS"}="1HN1 BOS"; $resn_exch{"HN12 BOS"}="2HN1 BOS";
$resn_exch{"HN11 BOZ"}="1HN1 BOZ"; $resn_exch{"HN12 BOZ"}="2HN1 BOZ"; $resn_exch{"HN21 BOZ"}="1HN2 BOZ";
$resn_exch{"HN22 BOZ"}="2HN2 BOZ"; $resn_exch{"H1'1 BOZ"}="1H1' BOZ"; $resn_exch{"H1'2 BOZ"}="2H1' BOZ";
$resn_exch{"H2'1 BOZ"}="1H2' BOZ"; $resn_exch{"H2'2 BOZ"}="2H2' BOZ"; $resn_exch{"HN4' BOZ"}="'HN4 BOZ";
$resn_exch{"H101  BP"}="1H10  BP"; $resn_exch{"H102  BP"}="2H10  BP"; $resn_exch{"HC41 BP2"}="1HC4 BP2";
$resn_exch{"HC42 BP2"}="2HC4 BP2"; $resn_exch{"HC61 BP2"}="1HC6 BP2"; $resn_exch{"HC62 BP2"}="2HC6 BP2";
$resn_exch{"HC71 BP2"}="1HC7 BP2"; $resn_exch{"HC72 BP2"}="2HC7 BP2"; $resn_exch{"HC91 BP2"}="1HC9 BP2";
$resn_exch{"HC92 BP2"}="2HC9 BP2"; $resn_exch{"HC93 BP2"}="3HC9 BP2"; $resn_exch{"H101 BP2"}="1H10 BP2";
$resn_exch{"H102 BP2"}="2H10 BP2"; $resn_exch{"H103 BP2"}="3H10 BP2"; $resn_exch{"H111 BP2"}="1H11 BP2";
$resn_exch{"H112 BP2"}="2H11 BP2"; $resn_exch{"H113 BP2"}="3H11 BP2"; $resn_exch{" HA1 BP4"}="1HA  BP4";
$resn_exch{" HA2 BP4"}="2HA  BP4"; $resn_exch{"H121 BP5"}="1H12 BP5"; $resn_exch{"H122 BP5"}="2H12 BP5";
$resn_exch{" HN1 BP5"}="1HN  BP5"; $resn_exch{" HN2 BP5"}="2HN  BP5"; $resn_exch{"H101 BPA"}="1H10 BPA";
$resn_exch{"H102 BPA"}="2H10 BPA"; $resn_exch{"HMA1 BPB"}="1HMA BPB"; $resn_exch{"HMA2 BPB"}="2HMA BPB";
$resn_exch{"HMA3 BPB"}="3HMA BPB"; $resn_exch{"HAA1 BPB"}="1HAA BPB"; $resn_exch{"HAA2 BPB"}="2HAA BPB";
$resn_exch{"HBA1 BPB"}="1HBA BPB"; $resn_exch{"HBA2 BPB"}="2HBA BPB"; $resn_exch{"HMB1 BPB"}="1HMB BPB";
$resn_exch{"HMB2 BPB"}="2HMB BPB"; $resn_exch{"HMB3 BPB"}="3HMB BPB"; $resn_exch{"HBB1 BPB"}="1HBB BPB";
$resn_exch{"HBB2 BPB"}="2HBB BPB"; $resn_exch{"HBB3 BPB"}="3HBB BPB"; $resn_exch{"HMC1 BPB"}="1HMC BPB";
$resn_exch{"HMC2 BPB"}="2HMC BPB"; $resn_exch{"HMC3 BPB"}="3HMC BPB"; $resn_exch{"HAC1 BPB"}="1HAC BPB";
$resn_exch{"HAC2 BPB"}="2HAC BPB"; $resn_exch{"HBC1 BPB"}="1HBC BPB"; $resn_exch{"HBC2 BPB"}="2HBC BPB";
$resn_exch{"HBC3 BPB"}="3HBC BPB"; $resn_exch{"HMD1 BPB"}="1HMD BPB"; $resn_exch{"HMD2 BPB"}="2HMD BPB";
$resn_exch{"HMD3 BPB"}="3HMD BPB"; $resn_exch{"HED1 BPB"}="1HED BPB"; $resn_exch{"HED2 BPB"}="2HED BPB";
$resn_exch{"HED3 BPB"}="3HED BPB"; $resn_exch{" H11 BPB"}="1H1  BPB"; $resn_exch{" H12 BPB"}="2H1  BPB";
$resn_exch{" H41 BPB"}="1H4  BPB"; $resn_exch{" H42 BPB"}="2H4  BPB"; $resn_exch{" H43 BPB"}="3H4  BPB";
$resn_exch{" H51 BPB"}="1H5  BPB"; $resn_exch{" H52 BPB"}="2H5  BPB"; $resn_exch{" H61 BPB"}="1H6  BPB";
$resn_exch{" H62 BPB"}="2H6  BPB"; $resn_exch{" H71 BPB"}="1H7  BPB"; $resn_exch{" H72 BPB"}="2H7  BPB";
$resn_exch{" H91 BPB"}="1H9  BPB"; $resn_exch{" H92 BPB"}="2H9  BPB"; $resn_exch{" H93 BPB"}="3H9  BPB";
$resn_exch{"H101 BPB"}="1H10 BPB"; $resn_exch{"H102 BPB"}="2H10 BPB"; $resn_exch{"H111 BPB"}="1H11 BPB";
$resn_exch{"H112 BPB"}="2H11 BPB"; $resn_exch{"H121 BPB"}="1H12 BPB"; $resn_exch{"H122 BPB"}="2H12 BPB";
$resn_exch{"H141 BPB"}="1H14 BPB"; $resn_exch{"H142 BPB"}="2H14 BPB"; $resn_exch{"H143 BPB"}="3H14 BPB";
$resn_exch{"H151 BPB"}="1H15 BPB"; $resn_exch{"H152 BPB"}="2H15 BPB"; $resn_exch{"H161 BPB"}="1H16 BPB";
$resn_exch{"H162 BPB"}="2H16 BPB"; $resn_exch{"H171 BPB"}="1H17 BPB"; $resn_exch{"H172 BPB"}="2H17 BPB";
$resn_exch{"H191 BPB"}="1H19 BPB"; $resn_exch{"H192 BPB"}="2H19 BPB"; $resn_exch{"H193 BPB"}="3H19 BPB";
$resn_exch{"H201 BPB"}="1H20 BPB"; $resn_exch{"H202 BPB"}="2H20 BPB"; $resn_exch{"H203 BPB"}="3H20 BPB";
$resn_exch{"H4'1 BPC"}="1H4' BPC"; $resn_exch{"H4'2 BPC"}="2H4' BPC"; $resn_exch{" HN1 BPE"}="1HN  BPE";
$resn_exch{" HN2 BPE"}="2HN  BPE"; $resn_exch{" HB1 BPE"}="1HB  BPE"; $resn_exch{" HB2 BPE"}="2HB  BPE";
$resn_exch{"H051 BPE"}="1H05 BPE"; $resn_exch{"H052 BPE"}="2H05 BPE"; $resn_exch{" H1  BPE"}="1H   BPE";
$resn_exch{" H2  BPE"}="2H   BPE"; $resn_exch{"H081 BPE"}="1H08 BPE"; $resn_exch{"H082 BPE"}="2H08 BPE";
$resn_exch{"H091 BPE"}="1H09 BPE"; $resn_exch{"H092 BPE"}="2H09 BPE"; $resn_exch{"H101 BPE"}="1H10 BPE";
$resn_exch{"H102 BPE"}="2H10 BPE"; $resn_exch{" H91 BPF"}="1H9  BPF"; $resn_exch{" H92 BPF"}="2H9  BPF";
$resn_exch{" H93 BPF"}="3H9  BPF"; $resn_exch{"HN11 BPF"}="1HN1 BPF"; $resn_exch{"HN12 BPF"}="2HN1 BPF";
$resn_exch{"H8'1 BPF"}="1H8' BPF"; $resn_exch{"H8'2 BPF"}="2H8' BPF"; $resn_exch{"H9'1 BPF"}="1H9' BPF";
$resn_exch{"H9'2 BPF"}="2H9' BPF"; $resn_exch{"H9'3 BPF"}="3H9' BPF"; $resn_exch{"H3'1 BPG"}="1H3* BPG";
$resn_exch{"H3'2 BPG"}="2H3* BPG"; $resn_exch{" H3' BPG"}=" H3* BPG"; $resn_exch{"H2'1 BPG"}="1H2* BPG";
$resn_exch{"H2'2 BPG"}="2H2* BPG"; $resn_exch{"H121 BPG"}="1H12 BPG"; $resn_exch{"H122 BPG"}="2H12 BPG";
$resn_exch{"H1'1 BPG"}="1H1* BPG"; $resn_exch{"H1'2 BPG"}="2H1* BPG"; $resn_exch{"HN21 BPG"}="1HN2 BPG";
$resn_exch{"H041 BPG"}="1H04 BPG"; $resn_exch{"H051 BPG"}="1H05 BPG"; $resn_exch{"H061 BPG"}="1H06 BPG";
$resn_exch{"H021 BPG"}="1H02 BPG"; $resn_exch{"H031 BPG"}="1H03 BPG"; $resn_exch{"HMA1 BPH"}="1HMA BPH";
$resn_exch{"HMA2 BPH"}="2HMA BPH"; $resn_exch{"HMA3 BPH"}="3HMA BPH"; $resn_exch{"HAA1 BPH"}="1HAA BPH";
$resn_exch{"HAA2 BPH"}="2HAA BPH"; $resn_exch{"HBA1 BPH"}="1HBA BPH"; $resn_exch{"HBA2 BPH"}="2HBA BPH";
$resn_exch{"HMB1 BPH"}="1HMB BPH"; $resn_exch{"HMB2 BPH"}="2HMB BPH"; $resn_exch{"HMB3 BPH"}="3HMB BPH";
$resn_exch{"HBB1 BPH"}="1HBB BPH"; $resn_exch{"HBB2 BPH"}="2HBB BPH"; $resn_exch{"HBB3 BPH"}="3HBB BPH";
$resn_exch{"HMC1 BPH"}="1HMC BPH"; $resn_exch{"HMC2 BPH"}="2HMC BPH"; $resn_exch{"HMC3 BPH"}="3HMC BPH";
$resn_exch{"HAC1 BPH"}="1HAC BPH"; $resn_exch{"HAC2 BPH"}="2HAC BPH"; $resn_exch{"HBC1 BPH"}="1HBC BPH";
$resn_exch{"HBC2 BPH"}="2HBC BPH"; $resn_exch{"HBC3 BPH"}="3HBC BPH"; $resn_exch{"HMD1 BPH"}="1HMD BPH";
$resn_exch{"HMD2 BPH"}="2HMD BPH"; $resn_exch{"HMD3 BPH"}="3HMD BPH"; $resn_exch{"HED1 BPH"}="1HED BPH";
$resn_exch{"HED2 BPH"}="2HED BPH"; $resn_exch{"HED3 BPH"}="3HED BPH"; $resn_exch{" H11 BPH"}="1H1  BPH";
$resn_exch{" H12 BPH"}="2H1  BPH"; $resn_exch{" H41 BPH"}="1H4  BPH"; $resn_exch{" H42 BPH"}="2H4  BPH";
$resn_exch{" H43 BPH"}="3H4  BPH"; $resn_exch{" H51 BPH"}="1H5  BPH"; $resn_exch{" H52 BPH"}="2H5  BPH";
$resn_exch{" H61 BPH"}="1H6  BPH"; $resn_exch{" H62 BPH"}="2H6  BPH"; $resn_exch{" H71 BPH"}="1H7  BPH";
$resn_exch{" H72 BPH"}="2H7  BPH"; $resn_exch{" H91 BPH"}="1H9  BPH"; $resn_exch{" H92 BPH"}="2H9  BPH";
$resn_exch{" H93 BPH"}="3H9  BPH"; $resn_exch{"H101 BPH"}="1H10 BPH"; $resn_exch{"H102 BPH"}="2H10 BPH";
$resn_exch{"H111 BPH"}="1H11 BPH"; $resn_exch{"H112 BPH"}="2H11 BPH"; $resn_exch{"H121 BPH"}="1H12 BPH";
$resn_exch{"H122 BPH"}="2H12 BPH"; $resn_exch{"H141 BPH"}="1H14 BPH"; $resn_exch{"H142 BPH"}="2H14 BPH";
$resn_exch{"H143 BPH"}="3H14 BPH"; $resn_exch{"H151 BPH"}="1H15 BPH"; $resn_exch{"H152 BPH"}="2H15 BPH";
$resn_exch{"H161 BPH"}="1H16 BPH"; $resn_exch{"H162 BPH"}="2H16 BPH"; $resn_exch{"H171 BPH"}="1H17 BPH";
$resn_exch{"H172 BPH"}="2H17 BPH"; $resn_exch{"H191 BPH"}="1H19 BPH"; $resn_exch{"H192 BPH"}="2H19 BPH";
$resn_exch{"H193 BPH"}="3H19 BPH"; $resn_exch{"H201 BPH"}="1H20 BPH"; $resn_exch{"H202 BPH"}="2H20 BPH";
$resn_exch{"H203 BPH"}="3H20 BPH"; $resn_exch{"H11A BPI"}="1H1  BPI"; $resn_exch{"H12A BPI"}="2H1  BPI";
$resn_exch{" HO2 BPI"}="2HO  BPI"; $resn_exch{" HO3 BPI"}="3HO  BPI"; $resn_exch{" HO4 BPI"}="4HO  BPI";
$resn_exch{"H11A BPJ"}="1H1  BPJ"; $resn_exch{"H12A BPJ"}="2H1  BPJ"; $resn_exch{" HO2 BPJ"}="2HO  BPJ";
$resn_exch{" HO3 BPJ"}="3HO  BPJ"; $resn_exch{" HO4 BPJ"}="4HO  BPJ"; $resn_exch{"HC01 BPM"}="1HC0 BPM";
$resn_exch{"HC02 BPM"}="2HC0 BPM"; $resn_exch{"HOA3 BPM"}="3HOA BPM"; $resn_exch{"HOA4 BPM"}="4HOA BPM";
$resn_exch{"HOB3 BPM"}="3HOB BPM"; $resn_exch{"HOB4 BPM"}="4HOB BPM"; $resn_exch{" H71 BPN"}="1H7  BPN";
$resn_exch{" H72 BPN"}="2H7  BPN"; $resn_exch{"HN11 BPO"}="1HN1 BPO"; $resn_exch{"HN12 BPO"}="2HN1 BPO";
$resn_exch{"H171 BPO"}="1H17 BPO"; $resn_exch{"H172 BPO"}="2H17 BPO"; $resn_exch{"H161 BPO"}="1H16 BPO";
$resn_exch{"H162 BPO"}="2H16 BPO"; $resn_exch{"H201 BPO"}="1H20 BPO"; $resn_exch{"H202 BPO"}="2H20 BPO";
$resn_exch{"H211 BPO"}="1H21 BPO"; $resn_exch{"H212 BPO"}="2H21 BPO"; $resn_exch{"H141 BPP"}="1H14 BPP";
$resn_exch{"H142 BPP"}="2H14 BPP"; $resn_exch{"H151 BPP"}="1H15 BPP"; $resn_exch{"H152 BPP"}="2H15 BPP";
$resn_exch{"H201 BPP"}="1H20 BPP"; $resn_exch{"H202 BPP"}="2H20 BPP"; $resn_exch{"H211 BPP"}="1H21 BPP";
$resn_exch{"H212 BPP"}="2H21 BPP"; $resn_exch{"H231 BPP"}="1H23 BPP"; $resn_exch{"H232 BPP"}="2H23 BPP";
$resn_exch{"H241 BPP"}="1H24 BPP"; $resn_exch{"H242 BPP"}="2H24 BPP"; $resn_exch{"H331 BPP"}="1H33 BPP";
$resn_exch{"H332 BPP"}="2H33 BPP"; $resn_exch{" HB1 BPR"}="1HB  BPR"; $resn_exch{" HB2 BPR"}="2HB  BPR";
$resn_exch{" HG1 BPR"}="1HG  BPR"; $resn_exch{" HG2 BPR"}="2HG  BPR"; $resn_exch{" HD1 BPR"}="1HD  BPR";
$resn_exch{" HD2 BPR"}="2HD  BPR"; $resn_exch{" H11 BPR"}="1H1  BPR"; $resn_exch{" H12 BPR"}="2H1  BPR";
$resn_exch{" H31 BPR"}="1H3  BPR"; $resn_exch{" H32 BPR"}="2H3  BPR"; $resn_exch{" H41 BPR"}="1H4  BPR";
$resn_exch{" H42 BPR"}="2H4  BPR"; $resn_exch{"HN11 BPT"}="1HN1 BPT"; $resn_exch{"HN12 BPT"}="2HN1 BPT";
$resn_exch{"HC11 BPT"}="1HC1 BPT"; $resn_exch{"HC12 BPT"}="2HC1 BPT"; $resn_exch{"HC21 BPT"}="1HC2 BPT";
$resn_exch{"HC22 BPT"}="2HC2 BPT"; $resn_exch{"HC31 BPT"}="1HC3 BPT"; $resn_exch{"HC32 BPT"}="2HC3 BPT";
$resn_exch{"HC41 BPT"}="1HC4 BPT"; $resn_exch{"HC42 BPT"}="2HC4 BPT"; $resn_exch{"HN21 BPT"}="1HN2 BPT";
$resn_exch{"HN22 BPT"}="2HN2 BPT"; $resn_exch{"HN31 BPT"}="1HN3 BPT"; $resn_exch{"HN32 BPT"}="2HN3 BPT";
$resn_exch{"HN33 BPT"}="3HN3 BPT"; $resn_exch{"HN41 BPT"}="1HN4 BPT"; $resn_exch{"HN42 BPT"}="2HN4 BPT";
$resn_exch{"HN43 BPT"}="3HN4 BPT"; $resn_exch{"HN51 BPT"}="1HN5 BPT"; $resn_exch{"HN52 BPT"}="2HN5 BPT";
$resn_exch{"HN53 BPT"}="3HN5 BPT"; $resn_exch{"HN61 BPT"}="1HN6 BPT"; $resn_exch{"HN62 BPT"}="2HN6 BPT";
$resn_exch{"HN63 BPT"}="3HN6 BPT"; $resn_exch{"HA21 BR3"}="1HA2 BR3"; $resn_exch{"HA22 BR3"}="2HA2 BR3";
$resn_exch{"HZ31 BR3"}="1HZ3 BR3"; $resn_exch{"HZ32 BR3"}="2HZ3 BR3"; $resn_exch{"HE31 BR3"}="1HE3 BR3";
$resn_exch{"HE32 BR3"}="2HE3 BR3"; $resn_exch{"HH21 BR3"}="1HH2 BR3"; $resn_exch{"HH22 BR3"}="2HH2 BR3";
$resn_exch{"HH23 BR3"}="3HH2 BR3"; $resn_exch{" H91 BR4"}="1H9  BR4"; $resn_exch{" H92 BR4"}="2H9  BR4";
$resn_exch{" H81 BR4"}="1H8  BR4"; $resn_exch{" H82 BR4"}="2H8  BR4"; $resn_exch{" H71 BR4"}="1H7  BR4";
$resn_exch{" H72 BR4"}="2H7  BR4"; $resn_exch{" H11 BR4"}="1H1  BR4"; $resn_exch{" H51 BR4"}="1H5  BR4";
$resn_exch{" H21 BR4"}="1H2  BR4"; $resn_exch{" H41 BR4"}="1H4  BR4"; $resn_exch{" H31 BR4"}="1H3  BR4";
$resn_exch{"H121 BR4"}="1H12 BR4"; $resn_exch{"H122 BR4"}="2H12 BR4"; $resn_exch{"H111 BR4"}="1H11 BR4";
$resn_exch{"H112 BR4"}="2H11 BR4"; $resn_exch{"H101 BR4"}="1H10 BR4"; $resn_exch{"H102 BR4"}="2H10 BR4";
$resn_exch{"HN11 BR4"}="1HN1 BR4"; $resn_exch{"H231 BR4"}="1H23 BR4"; $resn_exch{"H232 BR4"}="2H23 BR4";
$resn_exch{"H141 BR4"}="1H14 BR4"; $resn_exch{"H151 BR4"}="1H15 BR4"; $resn_exch{"H152 BR4"}="2H15 BR4";
$resn_exch{"H161 BR4"}="1H16 BR4"; $resn_exch{"H162 BR4"}="2H16 BR4"; $resn_exch{"H181 BR4"}="1H18 BR4";
$resn_exch{"H221 BR4"}="1H22 BR4"; $resn_exch{"H191 BR4"}="1H19 BR4"; $resn_exch{"H211 BR4"}="1H21 BR4";
$resn_exch{"H201 BR4"}="1H20 BR4"; $resn_exch{"H241 BR4"}="1H24 BR4"; $resn_exch{"H242 BR4"}="2H24 BR4";
$resn_exch{"HO31 BR4"}="1HO3 BR4"; $resn_exch{"HA11 BR5"}="1HA1 BR5"; $resn_exch{"HA12 BR5"}="2HA1 BR5";
$resn_exch{"HA13 BR5"}="3HA1 BR5"; $resn_exch{"HB21 BR5"}="1HB2 BR5"; $resn_exch{"HB22 BR5"}="2HB2 BR5";
$resn_exch{"HZ21 BR5"}="1HZ2 BR5"; $resn_exch{"HZ22 BR5"}="2HZ2 BR5"; $resn_exch{"HA21 BR5"}="1HA2 BR5";
$resn_exch{"HA22 BR5"}="2HA2 BR5"; $resn_exch{" H71 BRB"}="1H7  BRB"; $resn_exch{" H72 BRB"}="2H7  BRB";
$resn_exch{" H21 BRC"}="1H2  BRC"; $resn_exch{" H22 BRC"}="2H2  BRC"; $resn_exch{" H31 BRC"}="1H3  BRC";
$resn_exch{" H32 BRC"}="2H3  BRC"; $resn_exch{" H41 BRC"}="1H4  BRC"; $resn_exch{" H42 BRC"}="2H4  BRC";
$resn_exch{" H51 BRC"}="1H5  BRC"; $resn_exch{" H52 BRC"}="2H5  BRC"; $resn_exch{" H61 BRC"}="1H6  BRC";
$resn_exch{" H62 BRC"}="2H6  BRC"; $resn_exch{" H71 BRC"}="1H7  BRC"; $resn_exch{" H72 BRC"}="2H7  BRC";
$resn_exch{" H81 BRC"}="1H8  BRC"; $resn_exch{" H82 BRC"}="2H8  BRC"; $resn_exch{" H91 BRC"}="1H9  BRC";
$resn_exch{" H92 BRC"}="2H9  BRC"; $resn_exch{"H101 BRC"}="1H10 BRC"; $resn_exch{"H102 BRC"}="2H10 BRC";
$resn_exch{"H111 BRC"}="1H11 BRC"; $resn_exch{"H112 BRC"}="2H11 BRC"; $resn_exch{"H121 BRC"}="1H12 BRC";
$resn_exch{"H122 BRC"}="2H12 BRC"; $resn_exch{"H3'1 BRD"}="1H3' BRD"; $resn_exch{"H2'1 BRD"}="1H2' BRD";
$resn_exch{"H5'1 BRD"}="1H5' BRD"; $resn_exch{"H5'2 BRD"}="2H5' BRD"; $resn_exch{" H11 BRE"}="1H1  BRE";
$resn_exch{" H12 BRE"}="2H1  BRE"; $resn_exch{" H13 BRE"}="3H1  BRE"; $resn_exch{" H81 BRE"}="1H8  BRE";
$resn_exch{" H91 BRE"}="1H9  BRE"; $resn_exch{"H101 BRE"}="1H10 BRE"; $resn_exch{"H111 BRE"}="1H11 BRE";
$resn_exch{"H121 BRE"}="1H12 BRE"; $resn_exch{"H131 BRE"}="1H13 BRE"; $resn_exch{"H151 BRE"}="1H15 BRE";
$resn_exch{"H181 BRE"}="1H18 BRE"; $resn_exch{"H191 BRE"}="1H19 BRE"; $resn_exch{"H201 BRE"}="1H20 BRE";
$resn_exch{"H211 BRE"}="1H21 BRE"; $resn_exch{"H221 BRE"}="1H22 BRE"; $resn_exch{"H1C1 BRF"}="1H1C BRF";
$resn_exch{"H1C2 BRF"}="2H1C BRF"; $resn_exch{"H1C3 BRF"}="3H1C BRF"; $resn_exch{" H21 BRG"}="1H2  BRG";
$resn_exch{" H22 BRG"}="2H2  BRG"; $resn_exch{" H21 BRI"}="1H2  BRI"; $resn_exch{" H22 BRI"}="2H2  BRI";
$resn_exch{" H41 BRI"}="1H4  BRI"; $resn_exch{" H42 BRI"}="2H4  BRI"; $resn_exch{" H43 BRI"}="3H4  BRI";
$resn_exch{" H61 BRI"}="1H6  BRI"; $resn_exch{" H62 BRI"}="2H6  BRI"; $resn_exch{" H63 BRI"}="3H6  BRI";
$resn_exch{"HB11 BRJ"}="1HB1 BRJ"; $resn_exch{"HB12 BRJ"}="2HB1 BRJ"; $resn_exch{"HB1A BRJ"}="1HB  BRJ";
$resn_exch{" HB2 BRJ"}="2HB  BRJ"; $resn_exch{"HAB1 BRK"}="1HAB BRK"; $resn_exch{"HAB2 BRK"}="2HAB BRK";
$resn_exch{"HAB3 BRK"}="3HAB BRK"; $resn_exch{"HAC1 BRK"}="1HAC BRK"; $resn_exch{"HAC2 BRK"}="2HAC BRK";
$resn_exch{"HAA1 BRK"}="1HAA BRK"; $resn_exch{"HAA2 BRK"}="2HAA BRK"; $resn_exch{"HAA3 BRK"}="3HAA BRK";
$resn_exch{" H61 BRL"}="1H6  BRL"; $resn_exch{" H62 BRL"}="2H6  BRL"; $resn_exch{"H141 BRL"}="1H14 BRL";
$resn_exch{"H142 BRL"}="2H14 BRL"; $resn_exch{"H151 BRL"}="1H15 BRL"; $resn_exch{"H152 BRL"}="2H15 BRL";
$resn_exch{"H161 BRL"}="1H16 BRL"; $resn_exch{"H162 BRL"}="2H16 BRL"; $resn_exch{"H163 BRL"}="3H16 BRL";
$resn_exch{" H21 BRM"}="1H2  BRM"; $resn_exch{" H22 BRM"}="2H2  BRM"; $resn_exch{" HB1 BRN"}="1HB  BRN";
$resn_exch{" HB2 BRN"}="2HB  BRN"; $resn_exch{"HB'1 BRN"}="1HB' BRN"; $resn_exch{"HB'2 BRN"}="2HB' BRN";
$resn_exch{" H11 BRP"}="1H1  BRP"; $resn_exch{" H12 BRP"}="2H1  BRP"; $resn_exch{" H31 BRP"}="1H3  BRP";
$resn_exch{" H32 BRP"}="2H3  BRP"; $resn_exch{" H1  BRQ"}="1H   BRQ"; $resn_exch{" H2  BRQ"}="2H   BRQ";
$resn_exch{"H101 BRR"}="1H10 BRR"; $resn_exch{"H102 BRR"}="2H10 BRR"; $resn_exch{"H103 BRR"}="3H10 BRR";
$resn_exch{" H91 BRR"}="1H9  BRR"; $resn_exch{" H92 BRR"}="2H9  BRR"; $resn_exch{" H81 BRR"}="1H8  BRR";
$resn_exch{" H82 BRR"}="2H8  BRR"; $resn_exch{" H71 BRR"}="1H7  BRR"; $resn_exch{" H72 BRR"}="2H7  BRR";
$resn_exch{" H51 BRR"}="1H5  BRR"; $resn_exch{" H52 BRR"}="2H5  BRR"; $resn_exch{"H221 BRS"}="1H22 BRS";
$resn_exch{"H222 BRS"}="2H22 BRS"; $resn_exch{"H223 BRS"}="3H22 BRS"; $resn_exch{"H8N1 BRT"}="1H8N BRT";
$resn_exch{"H8N2 BRT"}="2H8N BRT"; $resn_exch{" H1' BRU"}=" H1* BRU"; $resn_exch{" H2' BRU"}="1H2* BRU";
$resn_exch{"H2'' BRU"}="2H2* BRU"; $resn_exch{" H3' BRU"}=" H3* BRU"; $resn_exch{" H4' BRU"}=" H4* BRU";
$resn_exch{"HO3' BRU"}="*HO3 BRU"; $resn_exch{" H5' BRU"}="1H5* BRU"; $resn_exch{"H5'' BRU"}="2H5* BRU";
$resn_exch{"HOP2 BRU"}="2HOP BRU"; $resn_exch{"HOP3 BRU"}="3HOP BRU"; $resn_exch{"HC11 BRZ"}="1HC1 BRZ";
$resn_exch{"HC1A BRZ"}="2HC1 BRZ"; $resn_exch{"HC61 BRZ"}="1HC6 BRZ"; $resn_exch{"HC62 BRZ"}="2HC6 BRZ";
$resn_exch{"HC63 BRZ"}="3HC6 BRZ"; $resn_exch{"HC71 BRZ"}="1HC7 BRZ"; $resn_exch{"HC72 BRZ"}="2HC7 BRZ";
$resn_exch{"HC73 BRZ"}="3HC7 BRZ"; $resn_exch{"HC91 BRZ"}="1HC9 BRZ"; $resn_exch{"HC92 BRZ"}="2HC9 BRZ";
$resn_exch{"HC21 BRZ"}="1HC2 BRZ"; $resn_exch{"HC22 BRZ"}="2HC2 BRZ"; $resn_exch{"HC23 BRZ"}="3HC2 BRZ";
$resn_exch{"HC31 BRZ"}="1HC3 BRZ"; $resn_exch{"HC32 BRZ"}="2HC3 BRZ"; $resn_exch{"HC33 BRZ"}="3HC3 BRZ";
$resn_exch{" H71 BSA"}="1H7  BSA"; $resn_exch{" H72 BSA"}="2H7  BSA"; $resn_exch{"H101 BSA"}="1H10 BSA";
$resn_exch{"H102 BSA"}="2H10 BSA"; $resn_exch{"H01C BSB"}="CH01 BSB"; $resn_exch{"H02C BSB"}="CH02 BSB";
$resn_exch{"H04C BSB"}="CH04 BSB"; $resn_exch{"H05C BSB"}="CH05 BSB"; $resn_exch{"H09N BSB"}="NH09 BSB";
$resn_exch{"H101 BSB"}="1H10 BSB"; $resn_exch{"H102 BSB"}="2H10 BSB"; $resn_exch{"HNP1 BSB"}="1HNP BSB";
$resn_exch{"HNP2 BSB"}="2HNP BSB"; $resn_exch{"H16C BSB"}="CH16 BSB"; $resn_exch{"H17C BSB"}="CH17 BSB";
$resn_exch{"H18C BSB"}="CH18 BSB"; $resn_exch{"H19C BSB"}="CH19 BSB"; $resn_exch{"H20C BSB"}="CH20 BSB";
$resn_exch{"HAB1 BSC"}="1HAB BSC"; $resn_exch{"HAB2 BSC"}="2HAB BSC"; $resn_exch{"HAB3 BSC"}="3HAB BSC";
$resn_exch{"HAC1 BSC"}="1HAC BSC"; $resn_exch{"HAC2 BSC"}="2HAC BSC"; $resn_exch{"HAD1 BSC"}="1HAD BSC";
$resn_exch{"HAD2 BSC"}="2HAD BSC"; $resn_exch{"HAE1 BSC"}="1HAE BSC"; $resn_exch{"HAE2 BSC"}="2HAE BSC";
$resn_exch{"HAA1 BSC"}="1HAA BSC"; $resn_exch{"HAA2 BSC"}="2HAA BSC"; $resn_exch{"HAH1 BSC"}="1HAH BSC";
$resn_exch{"HAH2 BSC"}="2HAH BSC"; $resn_exch{" HB1 BSC"}="1HB  BSC"; $resn_exch{" HB2 BSC"}="2HB  BSC";
$resn_exch{" HXT BSC"}="HOXT BSC"; $resn_exch{" HN1 BSC"}="1HN  BSC"; $resn_exch{" HN2 BSC"}="2HN  BSC";
$resn_exch{" H2  BSE"}=" HN2 BSE"; $resn_exch{" HC1 BSE"}="1HC  BSE"; $resn_exch{" HC2 BSE"}="2HC  BSE";
$resn_exch{" HB2 BSE"}="1HB  BSE"; $resn_exch{" HB3 BSE"}="2HB  BSE"; $resn_exch{"HC71 BSI"}="1HC7 BSI";
$resn_exch{"HC72 BSI"}="2HC7 BSI"; $resn_exch{"HC01 BSI"}="1HC0 BSI"; $resn_exch{"HC02 BSI"}="2HC0 BSI";
$resn_exch{"H531 BSJ"}="1H53 BSJ"; $resn_exch{"H532 BSJ"}="2H53 BSJ"; $resn_exch{"H631 BSJ"}="1H63 BSJ";
$resn_exch{"H632 BSJ"}="2H63 BSJ"; $resn_exch{"H521 BSJ"}="1H52 BSJ"; $resn_exch{"H522 BSJ"}="2H52 BSJ";
$resn_exch{"H501 BSJ"}="1H50 BSJ"; $resn_exch{"H502 BSJ"}="2H50 BSJ"; $resn_exch{"H581 BSJ"}="1H58 BSJ";
$resn_exch{"H582 BSJ"}="2H58 BSJ"; $resn_exch{" HA1 BSJ"}="1HA  BSJ"; $resn_exch{" HB2 BSJ"}="2HB  BSJ";
$resn_exch{" HB1 BSJ"}="1HB  BSJ"; $resn_exch{" HC2 BSJ"}="2HC  BSJ"; $resn_exch{"H511 BSJ"}="1H51 BSJ";
$resn_exch{"H512 BSJ"}="2H51 BSJ"; $resn_exch{"H481 BSJ"}="1H48 BSJ"; $resn_exch{"H482 BSJ"}="2H48 BSJ";
$resn_exch{"H471 BSJ"}="1H47 BSJ"; $resn_exch{"H472 BSJ"}="2H47 BSJ"; $resn_exch{"H431 BSJ"}="1H43 BSJ";
$resn_exch{"H432 BSJ"}="2H43 BSJ"; $resn_exch{"H433 BSJ"}="3H43 BSJ"; $resn_exch{"H441 BSJ"}="1H44 BSJ";
$resn_exch{"H442 BSJ"}="2H44 BSJ"; $resn_exch{"H443 BSJ"}="3H44 BSJ"; $resn_exch{"H411 BSJ"}="1H41 BSJ";
$resn_exch{"H412 BSJ"}="2H41 BSJ"; $resn_exch{"H151 BSJ"}="1H15 BSJ"; $resn_exch{"H152 BSJ"}="2H15 BSJ";
$resn_exch{"H3N1 BSJ"}="1H3N BSJ"; $resn_exch{"H3N2 BSJ"}="2H3N BSJ"; $resn_exch{"H221 BSM"}="1H22 BSM";
$resn_exch{"H222 BSM"}="2H22 BSM"; $resn_exch{"H223 BSM"}="3H22 BSM"; $resn_exch{"H261 BSM"}="1H26 BSM";
$resn_exch{"H262 BSM"}="2H26 BSM"; $resn_exch{"H271 BSM"}="1H27 BSM"; $resn_exch{"H272 BSM"}="2H27 BSM";
$resn_exch{"H273 BSM"}="3H27 BSM"; $resn_exch{"H101 BSO"}="1H10 BSO"; $resn_exch{"H102 BSO"}="2H10 BSO";
$resn_exch{"H9C1 BSO"}="1H9C BSO"; $resn_exch{"H9C2 BSO"}="2H9C BSO"; $resn_exch{"H8C1 BSO"}="1H8C BSO";
$resn_exch{"H8C2 BSO"}="2H8C BSO"; $resn_exch{"H7C1 BSO"}="1H7C BSO"; $resn_exch{"H7C2 BSO"}="2H7C BSO";
$resn_exch{"H6C1 BSO"}="1H6C BSO"; $resn_exch{"H6C2 BSO"}="2H6C BSO"; $resn_exch{" H2B BSP"}=" H2* BSP";
$resn_exch{" H5B BSP"}=" H5* BSP"; $resn_exch{" H6B BSP"}=" H6* BSP"; $resn_exch{" H4B BSP"}=" H4* BSP";
$resn_exch{" H81 BST"}="1H8  BST"; $resn_exch{" H82 BST"}="2H8  BST"; $resn_exch{"HN91 BST"}="1HN9 BST";
$resn_exch{"HN92 BST"}="2HN9 BST"; $resn_exch{"H101 BST"}="1H10 BST"; $resn_exch{"H102 BST"}="2H10 BST";
$resn_exch{"H111 BST"}="1H11 BST"; $resn_exch{"H112 BST"}="2H11 BST"; $resn_exch{"H131 BST"}="1H13 BST";
$resn_exch{"H132 BST"}="2H13 BST"; $resn_exch{"H133 BST"}="3H13 BST"; $resn_exch{"H151 BST"}="1H15 BST";
$resn_exch{"H152 BST"}="2H15 BST"; $resn_exch{"HN41 BST"}="1HN4 BST"; $resn_exch{"HN42 BST"}="2HN4 BST";
$resn_exch{" H41 BSU"}="1H4  BSU"; $resn_exch{" H31 BSU"}="1H3  BSU"; $resn_exch{" H21 BSU"}="1H2  BSU";
$resn_exch{" H11 BSU"}="1H1  BSU"; $resn_exch{" H61 BSU"}="1H6  BSU"; $resn_exch{"H161 BSU"}="1H16 BSU";
$resn_exch{"H151 BSU"}="1H15 BSU"; $resn_exch{"H141 BSU"}="1H14 BSU"; $resn_exch{"H131 BSU"}="1H13 BSU";
$resn_exch{"H121 BSU"}="1H12 BSU"; $resn_exch{"H241 BT1"}="1H24 BT1"; $resn_exch{"H242 BT1"}="2H24 BT1";
$resn_exch{"H251 BT1"}="1H25 BT1"; $resn_exch{"H252 BT1"}="2H25 BT1"; $resn_exch{"H171 BT1"}="1H17 BT1";
$resn_exch{"H172 BT1"}="2H17 BT1"; $resn_exch{"H231 BT1"}="1H23 BT1"; $resn_exch{"H232 BT1"}="2H23 BT1";
$resn_exch{"H261 BT1"}="1H26 BT1"; $resn_exch{"H262 BT1"}="2H26 BT1"; $resn_exch{"H271 BT1"}="1H27 BT1";
$resn_exch{"H272 BT1"}="2H27 BT1"; $resn_exch{"H221 BT1"}="1H22 BT1"; $resn_exch{"H222 BT1"}="2H22 BT1";
$resn_exch{"H281 BT1"}="1H28 BT1"; $resn_exch{"H282 BT1"}="2H28 BT1"; $resn_exch{"H291 BT1"}="1H29 BT1";
$resn_exch{"H292 BT1"}="2H29 BT1"; $resn_exch{"H301 BT1"}="1H30 BT1"; $resn_exch{"H302 BT1"}="2H30 BT1";
$resn_exch{"H331 BT1"}="1H33 BT1"; $resn_exch{"H332 BT1"}="2H33 BT1"; $resn_exch{"H341 BT1"}="1H34 BT1";
$resn_exch{"H342 BT1"}="2H34 BT1"; $resn_exch{"H241 BT2"}="1H24 BT2"; $resn_exch{"H242 BT2"}="2H24 BT2";
$resn_exch{"H251 BT2"}="1H25 BT2"; $resn_exch{"H252 BT2"}="2H25 BT2"; $resn_exch{"H171 BT2"}="1H17 BT2";
$resn_exch{"H172 BT2"}="2H17 BT2"; $resn_exch{"H231 BT2"}="1H23 BT2"; $resn_exch{"H232 BT2"}="2H23 BT2";
$resn_exch{"H261 BT2"}="1H26 BT2"; $resn_exch{"H262 BT2"}="2H26 BT2"; $resn_exch{"H271 BT2"}="1H27 BT2";
$resn_exch{"H272 BT2"}="2H27 BT2"; $resn_exch{"H221 BT2"}="1H22 BT2"; $resn_exch{"H222 BT2"}="2H22 BT2";
$resn_exch{"H281 BT2"}="1H28 BT2"; $resn_exch{"H282 BT2"}="2H28 BT2"; $resn_exch{"H291 BT2"}="1H29 BT2";
$resn_exch{"H292 BT2"}="2H29 BT2"; $resn_exch{"H301 BT2"}="1H30 BT2"; $resn_exch{"H302 BT2"}="2H30 BT2";
$resn_exch{"H331 BT2"}="1H33 BT2"; $resn_exch{"H332 BT2"}="2H33 BT2"; $resn_exch{"H341 BT2"}="1H34 BT2";
$resn_exch{"H342 BT2"}="2H34 BT2"; $resn_exch{"H241 BT3"}="1H24 BT3"; $resn_exch{"H242 BT3"}="2H24 BT3";
$resn_exch{"H251 BT3"}="1H25 BT3"; $resn_exch{"H252 BT3"}="2H25 BT3"; $resn_exch{"H171 BT3"}="1H17 BT3";
$resn_exch{"H172 BT3"}="2H17 BT3"; $resn_exch{"H231 BT3"}="1H23 BT3"; $resn_exch{"H232 BT3"}="2H23 BT3";
$resn_exch{"H261 BT3"}="1H26 BT3"; $resn_exch{"H262 BT3"}="2H26 BT3"; $resn_exch{"H271 BT3"}="1H27 BT3";
$resn_exch{"H272 BT3"}="2H27 BT3"; $resn_exch{"H221 BT3"}="1H22 BT3"; $resn_exch{"H222 BT3"}="2H22 BT3";
$resn_exch{"H281 BT3"}="1H28 BT3"; $resn_exch{"H282 BT3"}="2H28 BT3"; $resn_exch{"H291 BT3"}="1H29 BT3";
$resn_exch{"H292 BT3"}="2H29 BT3"; $resn_exch{"H301 BT3"}="1H30 BT3"; $resn_exch{"H302 BT3"}="2H30 BT3";
$resn_exch{"H331 BT3"}="1H33 BT3"; $resn_exch{"H332 BT3"}="2H33 BT3"; $resn_exch{"H341 BT3"}="1H34 BT3";
$resn_exch{"H342 BT3"}="2H34 BT3"; $resn_exch{"H101 BT5"}="1H10 BT5"; $resn_exch{"H102 BT5"}="2H10 BT5";
$resn_exch{"H9B1 BT5"}="1H9B BT5"; $resn_exch{"H9B2 BT5"}="2H9B BT5"; $resn_exch{"H8B1 BT5"}="1H8B BT5";
$resn_exch{"H8B2 BT5"}="2H8B BT5"; $resn_exch{"H7B1 BT5"}="1H7B BT5"; $resn_exch{"H7B2 BT5"}="2H7B BT5";
$resn_exch{"H6B1 BT5"}="1H6B BT5"; $resn_exch{"H6B2 BT5"}="2H6B BT5"; $resn_exch{" H5' BT5"}="1H5* BT5";
$resn_exch{"H5'' BT5"}="2H5* BT5"; $resn_exch{" H4' BT5"}=" H4* BT5"; $resn_exch{" H3' BT5"}=" H3* BT5";
$resn_exch{" H2' BT5"}=" H2* BT5"; $resn_exch{" H1' BT5"}=" H1* BT5"; $resn_exch{"HN61 BT5"}="1HN6 BT5";
$resn_exch{"HN62 BT5"}="2HN6 BT5"; $resn_exch{" HN1 BTA"}="1HN  BTA"; $resn_exch{" HN2 BTA"}="2HN  BTA";
$resn_exch{" HB1 BTA"}="1HB  BTA"; $resn_exch{" HB2 BTA"}="2HB  BTA"; $resn_exch{" HG1 BTA"}="1HG  BTA";
$resn_exch{" HG2 BTA"}="2HG  BTA"; $resn_exch{" HD1 BTA"}="1HD  BTA"; $resn_exch{" HD2 BTA"}="2HD  BTA";
$resn_exch{" HD3 BTA"}="3HD  BTA"; $resn_exch{" H11 BTB"}="1H1  BTB"; $resn_exch{" H12 BTB"}="2H1  BTB";
$resn_exch{" H31 BTB"}="1H3  BTB"; $resn_exch{" H32 BTB"}="2H3  BTB"; $resn_exch{" H41 BTB"}="1H4  BTB";
$resn_exch{" H42 BTB"}="2H4  BTB"; $resn_exch{" H51 BTB"}="1H5  BTB"; $resn_exch{" H52 BTB"}="2H5  BTB";
$resn_exch{" H61 BTB"}="1H6  BTB"; $resn_exch{" H62 BTB"}="2H6  BTB"; $resn_exch{" H71 BTB"}="1H7  BTB";
$resn_exch{" H72 BTB"}="2H7  BTB"; $resn_exch{" H81 BTB"}="1H8  BTB"; $resn_exch{" H82 BTB"}="2H8  BTB";
$resn_exch{" H   BTC"}="1HN  BTC"; $resn_exch{" H2  BTC"}="2HN  BTC"; $resn_exch{" HB2 BTC"}="1HB  BTC";
$resn_exch{" HB3 BTC"}="2HB  BTC"; $resn_exch{" HG  BTC"}=" HSG BTC"; $resn_exch{"HO5' BTD"}="*HO5 BTD";
$resn_exch{"H5'1 BTD"}="1H5* BTD"; $resn_exch{"H5'2 BTD"}="2H5* BTD"; $resn_exch{" H4' BTD"}=" H4* BTD";
$resn_exch{" H1' BTD"}=" H1* BTD"; $resn_exch{"H2'1 BTD"}="1H2* BTD"; $resn_exch{"H2'2 BTD"}="2H2* BTD";
$resn_exch{" H3' BTD"}=" H3* BTD"; $resn_exch{"HO3' BTD"}="*HO3 BTD"; $resn_exch{"H311 BTH"}="1H31 BTH";
$resn_exch{"H312 BTH"}="2H31 BTH"; $resn_exch{"H313 BTH"}="3H31 BTH"; $resn_exch{"H351 BTH"}="1H35 BTH";
$resn_exch{"H352 BTH"}="2H35 BTH"; $resn_exch{"H231 BTH"}="1H23 BTH"; $resn_exch{"H232 BTH"}="2H23 BTH";
$resn_exch{"H221 BTH"}="1H22 BTH"; $resn_exch{"H222 BTH"}="2H22 BTH"; $resn_exch{" H91 BTH"}="1H9  BTH";
$resn_exch{" H92 BTH"}="2H9  BTH"; $resn_exch{"H101 BTH"}="1H10 BTH"; $resn_exch{"H102 BTH"}="2H10 BTH";
$resn_exch{" HA1 BTL"}="1HA  BTL"; $resn_exch{" HA2 BTL"}="2HA  BTL"; $resn_exch{" H31 BTL"}="1H3  BTL";
$resn_exch{" H32 BTL"}="2H3  BTL"; $resn_exch{" H33 BTL"}="3H3  BTL"; $resn_exch{" H21 BTL"}="1H2  BTL";
$resn_exch{" H22 BTL"}="2H2  BTL"; $resn_exch{" H23 BTL"}="3H2  BTL"; $resn_exch{" H11 BTL"}="1H1  BTL";
$resn_exch{" H12 BTL"}="2H1  BTL"; $resn_exch{" H13 BTL"}="3H1  BTL"; $resn_exch{"H101 BTN"}="1H10 BTN";
$resn_exch{"H102 BTN"}="2H10 BTN"; $resn_exch{" H91 BTN"}="1H9  BTN"; $resn_exch{" H92 BTN"}="2H9  BTN";
$resn_exch{" H81 BTN"}="1H8  BTN"; $resn_exch{" H82 BTN"}="2H8  BTN"; $resn_exch{" H71 BTN"}="1H7  BTN";
$resn_exch{" H72 BTN"}="2H7  BTN"; $resn_exch{" H61 BTN"}="1H6  BTN"; $resn_exch{" H62 BTN"}="2H6  BTN";
$resn_exch{"H111 BTO"}="1H11 BTO"; $resn_exch{"H112 BTO"}="2H11 BTO"; $resn_exch{"H101 BTO"}="1H10 BTO";
$resn_exch{"H102 BTO"}="2H10 BTO"; $resn_exch{" H91 BTO"}="1H9  BTO"; $resn_exch{" H92 BTO"}="2H9  BTO";
$resn_exch{" H31 BTP"}="1H3  BTP"; $resn_exch{" H32 BTP"}="2H3  BTP"; $resn_exch{" H'1 BTP"}="1H*  BTP";
$resn_exch{" H'2 BTP"}="2H*  BTP"; $resn_exch{" HS' BTP"}=" HS* BTP"; $resn_exch{"H101 BTQ"}="1H10 BTQ";
$resn_exch{"H102 BTQ"}="2H10 BTQ"; $resn_exch{" H91 BTQ"}="1H9  BTQ"; $resn_exch{" H92 BTQ"}="2H9  BTQ";
$resn_exch{" H81 BTQ"}="1H8  BTQ"; $resn_exch{" H82 BTQ"}="2H8  BTQ"; $resn_exch{" H71 BTQ"}="1H7  BTQ";
$resn_exch{" H72 BTQ"}="2H7  BTQ"; $resn_exch{" H61 BTQ"}="1H6  BTQ"; $resn_exch{" H62 BTQ"}="2H6  BTQ";
$resn_exch{" H   BTR"}="1HN  BTR"; $resn_exch{" H2  BTR"}="2HN  BTR"; $resn_exch{" HB2 BTR"}="1HB  BTR";
$resn_exch{" HB3 BTR"}="2HB  BTR"; $resn_exch{" H11 BTS"}="1H1  BTS"; $resn_exch{" H12 BTS"}="2H1  BTS";
$resn_exch{" H81 BTS"}="1H8  BTS"; $resn_exch{" H82 BTS"}="2H8  BTS"; $resn_exch{" H91 BTS"}="1H9  BTS";
$resn_exch{" H92 BTS"}="2H9  BTS"; $resn_exch{"H5'1 BTX"}="1H5* BTX"; $resn_exch{"H5'2 BTX"}="2H5* BTX";
$resn_exch{" H4' BTX"}=" H4* BTX"; $resn_exch{" H3' BTX"}=" H3* BTX"; $resn_exch{" H2' BTX"}=" H2* BTX";
$resn_exch{" H1' BTX"}=" H1* BTX"; $resn_exch{"HN61 BTX"}="1HN6 BTX"; $resn_exch{"HN62 BTX"}="2HN6 BTX";
$resn_exch{"H111 BTX"}="1H11 BTX"; $resn_exch{"H112 BTX"}="2H11 BTX"; $resn_exch{"H101 BTX"}="1H10 BTX";
$resn_exch{"H102 BTX"}="2H10 BTX"; $resn_exch{"H9B1 BTX"}="1H9B BTX"; $resn_exch{"H9B2 BTX"}="2H9B BTX";
$resn_exch{"H8B1 BTX"}="1H8B BTX"; $resn_exch{"H8B2 BTX"}="2H8B BTX"; $resn_exch{"H7B1 BTX"}="1H7B BTX";
$resn_exch{"H7B2 BTX"}="2H7B BTX"; $resn_exch{"H6B1 BTX"}="1H6B BTX"; $resn_exch{"H6B2 BTX"}="2H6B BTX";
$resn_exch{"H101 BTY"}="1H10 BTY"; $resn_exch{"H102 BTY"}="2H10 BTY"; $resn_exch{"H103 BTY"}="3H10 BTY";
$resn_exch{"H111 BTY"}="1H11 BTY"; $resn_exch{"H112 BTY"}="2H11 BTY"; $resn_exch{" H11 BU1"}="1H1  BU1";
$resn_exch{" H12 BU1"}="2H1  BU1"; $resn_exch{" H21 BU1"}="1H2  BU1"; $resn_exch{" H22 BU1"}="2H2  BU1";
$resn_exch{" H31 BU1"}="1H3  BU1"; $resn_exch{" H32 BU1"}="2H3  BU1"; $resn_exch{" H41 BU1"}="1H4  BU1";
$resn_exch{" H42 BU1"}="2H4  BU1"; $resn_exch{"HC11 BU2"}="1HC1 BU2"; $resn_exch{"HC12 BU2"}="2HC1 BU2";
$resn_exch{"HC21 BU2"}="1HC2 BU2"; $resn_exch{"HC22 BU2"}="2HC2 BU2"; $resn_exch{"HC41 BU2"}="1HC4 BU2";
$resn_exch{"HC42 BU2"}="2HC4 BU2"; $resn_exch{"HC43 BU2"}="3HC4 BU2"; $resn_exch{" H41 BU3"}="1H4  BU3";
$resn_exch{" H42 BU3"}="2H4  BU3"; $resn_exch{" H43 BU3"}="3H4  BU3"; $resn_exch{" H11 BU3"}="1H1  BU3";
$resn_exch{" H12 BU3"}="2H1  BU3"; $resn_exch{" H13 BU3"}="3H1  BU3"; $resn_exch{" H11 BUA"}="1H1  BUA";
$resn_exch{" H12 BUA"}="2H1  BUA"; $resn_exch{" H13 BUA"}="3H1  BUA"; $resn_exch{" H21 BUA"}="1H2  BUA";
$resn_exch{" H22 BUA"}="2H2  BUA"; $resn_exch{" H31 BUA"}="1H3  BUA"; $resn_exch{" H32 BUA"}="2H3  BUA";
$resn_exch{" H11 BUB"}="1H1  BUB"; $resn_exch{" H12 BUB"}="2H1  BUB"; $resn_exch{" H13 BUB"}="3H1  BUB";
$resn_exch{" H21 BUB"}="1H2  BUB"; $resn_exch{" H22 BUB"}="2H2  BUB"; $resn_exch{" H61 BUB"}="1H6  BUB";
$resn_exch{" H62 BUB"}="2H6  BUB"; $resn_exch{" H91 BUB"}="1H9  BUB"; $resn_exch{" H92 BUB"}="2H9  BUB";
$resn_exch{" H   BUC"}="1HN  BUC"; $resn_exch{" H2  BUC"}="2HN  BUC"; $resn_exch{" HB2 BUC"}="1HB  BUC";
$resn_exch{" HB3 BUC"}="2HB  BUC"; $resn_exch{" H11 BUC"}="1H1  BUC"; $resn_exch{" H12 BUC"}="2H1  BUC";
$resn_exch{" H21 BUC"}="1H2  BUC"; $resn_exch{" H22 BUC"}="2H2  BUC"; $resn_exch{" H31 BUC"}="1H3  BUC";
$resn_exch{" H32 BUC"}="2H3  BUC"; $resn_exch{" H41 BUC"}="1H4  BUC"; $resn_exch{" H42 BUC"}="2H4  BUC";
$resn_exch{" H43 BUC"}="3H4  BUC"; $resn_exch{" H   BUG"}="1H1  BUG"; $resn_exch{" H2  BUG"}="2H1  BUG";
$resn_exch{" HA1 BUG"}="1HA  BUG"; $resn_exch{"HG11 BUG"}="1HG1 BUG"; $resn_exch{"HG12 BUG"}="2HG1 BUG";
$resn_exch{"HG13 BUG"}="3HG1 BUG"; $resn_exch{"HG21 BUG"}="1HG2 BUG"; $resn_exch{"HG22 BUG"}="2HG2 BUG";
$resn_exch{"HG23 BUG"}="3HG2 BUG"; $resn_exch{"HG31 BUG"}="1HG3 BUG"; $resn_exch{"HG32 BUG"}="2HG3 BUG";
$resn_exch{"HG33 BUG"}="3HG3 BUG"; $resn_exch{"HN21 BUG"}="1HN2 BUG"; $resn_exch{"HN22 BUG"}="2HN2 BUG";
$resn_exch{" H21 BUI"}="1H2  BUI"; $resn_exch{" H22 BUI"}="2H2  BUI"; $resn_exch{"H101 BUI"}="1H10 BUI";
$resn_exch{"H102 BUI"}="2H10 BUI"; $resn_exch{"H141 BUI"}="1H14 BUI"; $resn_exch{"H142 BUI"}="2H14 BUI";
$resn_exch{"H143 BUI"}="3H14 BUI"; $resn_exch{"H131 BUI"}="1H13 BUI"; $resn_exch{"H132 BUI"}="2H13 BUI";
$resn_exch{"H133 BUI"}="3H13 BUI"; $resn_exch{"H121 BUI"}="1H12 BUI"; $resn_exch{"H122 BUI"}="2H12 BUI";
$resn_exch{"H123 BUI"}="3H12 BUI"; $resn_exch{"H151 BUI"}="1H15 BUI"; $resn_exch{"H152 BUI"}="2H15 BUI";
$resn_exch{"H161 BUI"}="1H16 BUI"; $resn_exch{"H162 BUI"}="2H16 BUI"; $resn_exch{"H241 BUI"}="1H24 BUI";
$resn_exch{"H242 BUI"}="2H24 BUI"; $resn_exch{"H261 BUI"}="1H26 BUI"; $resn_exch{"H262 BUI"}="2H26 BUI";
$resn_exch{"H271 BUI"}="1H27 BUI"; $resn_exch{"H272 BUI"}="2H27 BUI"; $resn_exch{"H191 BUI"}="1H19 BUI";
$resn_exch{"H192 BUI"}="2H19 BUI"; $resn_exch{"H201 BUI"}="1H20 BUI"; $resn_exch{"H202 BUI"}="2H20 BUI";
$resn_exch{"H211 BUI"}="1H21 BUI"; $resn_exch{"H212 BUI"}="2H21 BUI"; $resn_exch{"H221 BUI"}="1H22 BUI";
$resn_exch{"H222 BUI"}="2H22 BUI"; $resn_exch{"H231 BUI"}="1H23 BUI"; $resn_exch{"H232 BUI"}="2H23 BUI";
$resn_exch{"H251 BUI"}="1H25 BUI"; $resn_exch{"H252 BUI"}="2H25 BUI"; $resn_exch{"H181 BUI"}="1H18 BUI";
$resn_exch{"H182 BUI"}="2H18 BUI"; $resn_exch{"H171 BUI"}="1H17 BUI"; $resn_exch{"H172 BUI"}="2H17 BUI";
$resn_exch{"H281 BUI"}="1H28 BUI"; $resn_exch{"H282 BUI"}="2H28 BUI"; $resn_exch{"H283 BUI"}="3H28 BUI";
$resn_exch{"H061 BUK"}="1H06 BUK"; $resn_exch{"H062 BUK"}="2H06 BUK"; $resn_exch{"H071 BUK"}="1H07 BUK";
$resn_exch{"H072 BUK"}="2H07 BUK"; $resn_exch{"H081 BUK"}="1H08 BUK"; $resn_exch{"H082 BUK"}="2H08 BUK";
$resn_exch{"H111 BUK"}="1H11 BUK"; $resn_exch{"H112 BUK"}="2H11 BUK"; $resn_exch{"H221 BUK"}="1H22 BUK";
$resn_exch{"H222 BUK"}="2H22 BUK"; $resn_exch{"H223 BUK"}="3H22 BUK"; $resn_exch{"H231 BUK"}="1H23 BUK";
$resn_exch{"H232 BUK"}="2H23 BUK"; $resn_exch{"H233 BUK"}="3H23 BUK"; $resn_exch{"H281 BUK"}="1H28 BUK";
$resn_exch{"H282 BUK"}="2H28 BUK"; $resn_exch{"H311 BUK"}="1H31 BUK"; $resn_exch{"H312 BUK"}="2H31 BUK";
$resn_exch{"H431 BUK"}="1H43 BUK"; $resn_exch{"H432 BUK"}="2H43 BUK"; $resn_exch{"H433 BUK"}="3H43 BUK";
$resn_exch{" H91 BUL"}="1H9  BUL"; $resn_exch{" H92 BUL"}="2H9  BUL"; $resn_exch{"H121 BUL"}="1H12 BUL";
$resn_exch{"H122 BUL"}="2H12 BUL"; $resn_exch{"H191 BUL"}="1H19 BUL"; $resn_exch{"H192 BUL"}="2H19 BUL";
$resn_exch{"H193 BUL"}="3H19 BUL"; $resn_exch{"H211 BUL"}="1H21 BUL"; $resn_exch{"H212 BUL"}="2H21 BUL";
$resn_exch{"H261 BUL"}="1H26 BUL"; $resn_exch{"H262 BUL"}="2H26 BUL"; $resn_exch{"H271 BUL"}="1H27 BUL";
$resn_exch{"H272 BUL"}="2H27 BUL"; $resn_exch{"HG21 BUM"}="1HG2 BUM"; $resn_exch{"HG22 BUM"}="2HG2 BUM";
$resn_exch{"HG23 BUM"}="3HG2 BUM"; $resn_exch{"HG31 BUM"}="1HG3 BUM"; $resn_exch{"HG32 BUM"}="2HG3 BUM";
$resn_exch{"HG33 BUM"}="3HG3 BUM"; $resn_exch{"HG11 BUM"}="1HG1 BUM"; $resn_exch{"HG12 BUM"}="2HG1 BUM";
$resn_exch{"HG13 BUM"}="3HG1 BUM"; $resn_exch{" HN1 BUM"}="1HN  BUM"; $resn_exch{" HN2 BUM"}="2HN  BUM";
$resn_exch{"HN21 BUM"}="1HN2 BUM"; $resn_exch{"HN22 BUM"}="2HN2 BUM"; $resn_exch{"HN23 BUM"}="3HN2 BUM";
$resn_exch{" H11 BUQ"}="1H1  BUQ"; $resn_exch{" H12 BUQ"}="2H1  BUQ"; $resn_exch{" H13 BUQ"}="3H1  BUQ";
$resn_exch{" H31 BUQ"}="1H3  BUQ"; $resn_exch{" H32 BUQ"}="2H3  BUQ"; $resn_exch{" H41 BUQ"}="1H4  BUQ";
$resn_exch{" H42 BUQ"}="2H4  BUQ"; $resn_exch{"HO51 BUQ"}="1HO5 BUQ"; $resn_exch{" H11 BUT"}="1H1  BUT";
$resn_exch{" H12 BUT"}="2H1  BUT"; $resn_exch{" H13 BUT"}="3H1  BUT"; $resn_exch{" H21 BUT"}="1H2  BUT";
$resn_exch{" H22 BUT"}="2H2  BUT"; $resn_exch{" H31 BUT"}="1H3  BUT"; $resn_exch{" H32 BUT"}="2H3  BUT";
$resn_exch{" H41 BUT"}="1H4  BUT"; $resn_exch{" H42 BUT"}="2H4  BUT"; $resn_exch{" H43 BUT"}="3H4  BUT";
$resn_exch{"H501 BV1"}="1H50 BV1"; $resn_exch{"H502 BV1"}="2H50 BV1"; $resn_exch{"H11A BV1"}="1H1  BV1";
$resn_exch{" H12 BV1"}="2H1  BV1"; $resn_exch{"H111 BV1"}="1H11 BV1"; $resn_exch{"H112 BV1"}="2H11 BV1";
$resn_exch{"H121 BV1"}="1H12 BV1"; $resn_exch{"H122 BV1"}="2H12 BV1"; $resn_exch{"H131 BV1"}="1H13 BV1";
$resn_exch{"H132 BV1"}="2H13 BV1"; $resn_exch{"H141 BV1"}="1H14 BV1"; $resn_exch{"H142 BV1"}="2H14 BV1";
$resn_exch{"H151 BV1"}="1H15 BV1"; $resn_exch{"H152 BV1"}="2H15 BV1"; $resn_exch{"H161 BV1"}="1H16 BV1";
$resn_exch{"H162 BV1"}="2H16 BV1"; $resn_exch{"H171 BV1"}="1H17 BV1"; $resn_exch{"H172 BV1"}="2H17 BV1";
$resn_exch{"H181 BV1"}="1H18 BV1"; $resn_exch{"H182 BV1"}="2H18 BV1"; $resn_exch{"H191 BV1"}="1H19 BV1";
$resn_exch{"H192 BV1"}="2H19 BV1"; $resn_exch{"H201 BV1"}="1H20 BV1"; $resn_exch{"H202 BV1"}="2H20 BV1";
$resn_exch{"H231 BV1"}="1H23 BV1"; $resn_exch{"H232 BV1"}="2H23 BV1"; $resn_exch{"H241 BV1"}="1H24 BV1";
$resn_exch{"H242 BV1"}="2H24 BV1"; $resn_exch{"H251 BV1"}="1H25 BV1"; $resn_exch{"H252 BV1"}="2H25 BV1";
$resn_exch{"H261 BV1"}="1H26 BV1"; $resn_exch{"H262 BV1"}="2H26 BV1"; $resn_exch{"H271 BV1"}="1H27 BV1";
$resn_exch{"H272 BV1"}="2H27 BV1"; $resn_exch{"H281 BV1"}="1H28 BV1"; $resn_exch{"H282 BV1"}="2H28 BV1";
$resn_exch{"H291 BV1"}="1H29 BV1"; $resn_exch{"H292 BV1"}="2H29 BV1"; $resn_exch{"H301 BV1"}="1H30 BV1";
$resn_exch{"H302 BV1"}="2H30 BV1"; $resn_exch{"H311 BV1"}="1H31 BV1"; $resn_exch{"H312 BV1"}="2H31 BV1";
$resn_exch{"H321 BV1"}="1H32 BV1"; $resn_exch{"H322 BV1"}="2H32 BV1"; $resn_exch{"H511 BV2"}="1H51 BV2";
$resn_exch{"H512 BV2"}="2H51 BV2"; $resn_exch{"H11A BV2"}="1H1  BV2"; $resn_exch{"H12A BV2"}="2H1  BV2";
$resn_exch{"H111 BV2"}="1H11 BV2"; $resn_exch{"H112 BV2"}="2H11 BV2"; $resn_exch{"H121 BV2"}="1H12 BV2";
$resn_exch{"H122 BV2"}="2H12 BV2"; $resn_exch{"H131 BV2"}="1H13 BV2"; $resn_exch{"H132 BV2"}="2H13 BV2";
$resn_exch{"H141 BV2"}="1H14 BV2"; $resn_exch{"H142 BV2"}="2H14 BV2"; $resn_exch{"H151 BV2"}="1H15 BV2";
$resn_exch{"H152 BV2"}="2H15 BV2"; $resn_exch{"H161 BV2"}="1H16 BV2"; $resn_exch{"H162 BV2"}="2H16 BV2";
$resn_exch{"H171 BV2"}="1H17 BV2"; $resn_exch{"H172 BV2"}="2H17 BV2"; $resn_exch{"H181 BV2"}="1H18 BV2";
$resn_exch{"H182 BV2"}="2H18 BV2"; $resn_exch{"H191 BV2"}="1H19 BV2"; $resn_exch{"H192 BV2"}="2H19 BV2";
$resn_exch{"H201 BV2"}="1H20 BV2"; $resn_exch{"H202 BV2"}="2H20 BV2"; $resn_exch{"H221 BV2"}="1H22 BV2";
$resn_exch{"H222 BV2"}="2H22 BV2"; $resn_exch{"HN61 BV2"}="1HN6 BV2"; $resn_exch{"HN62 BV2"}="2HN6 BV2";
$resn_exch{"H241 BV2"}="1H24 BV2"; $resn_exch{"H242 BV2"}="2H24 BV2"; $resn_exch{"H261 BV2"}="1H26 BV2";
$resn_exch{"H262 BV2"}="2H26 BV2"; $resn_exch{"H271 BV2"}="1H27 BV2"; $resn_exch{"H272 BV2"}="2H27 BV2";
$resn_exch{"H281 BV2"}="1H28 BV2"; $resn_exch{"H282 BV2"}="2H28 BV2"; $resn_exch{"H291 BV2"}="1H29 BV2";
$resn_exch{"H292 BV2"}="2H29 BV2"; $resn_exch{"H301 BV2"}="1H30 BV2"; $resn_exch{"H302 BV2"}="2H30 BV2";
$resn_exch{"H311 BV2"}="1H31 BV2"; $resn_exch{"H312 BV2"}="2H31 BV2"; $resn_exch{"H321 BV2"}="1H32 BV2";
$resn_exch{"H322 BV2"}="2H32 BV2"; $resn_exch{"H331 BV2"}="1H33 BV2"; $resn_exch{"H332 BV2"}="2H33 BV2";
$resn_exch{"H341 BV2"}="1H34 BV2"; $resn_exch{"H342 BV2"}="2H34 BV2"; $resn_exch{"H351 BV2"}="1H35 BV2";
$resn_exch{"H352 BV2"}="2H35 BV2"; $resn_exch{"H631 BV3"}="1H63 BV3"; $resn_exch{"H632 BV3"}="2H63 BV3";
$resn_exch{"H11A BV3"}="1H1  BV3"; $resn_exch{" H12 BV3"}="2H1  BV3"; $resn_exch{"H111 BV3"}="1H11 BV3";
$resn_exch{"H112 BV3"}="2H11 BV3"; $resn_exch{"H121 BV3"}="1H12 BV3"; $resn_exch{"H122 BV3"}="2H12 BV3";
$resn_exch{"H131 BV3"}="1H13 BV3"; $resn_exch{"H132 BV3"}="2H13 BV3"; $resn_exch{"H141 BV3"}="1H14 BV3";
$resn_exch{"H142 BV3"}="2H14 BV3"; $resn_exch{"H151 BV3"}="1H15 BV3"; $resn_exch{"H152 BV3"}="2H15 BV3";
$resn_exch{"H161 BV3"}="1H16 BV3"; $resn_exch{"H162 BV3"}="2H16 BV3"; $resn_exch{"H171 BV3"}="1H17 BV3";
$resn_exch{"H172 BV3"}="2H17 BV3"; $resn_exch{"H181 BV3"}="1H18 BV3"; $resn_exch{"H182 BV3"}="2H18 BV3";
$resn_exch{"H191 BV3"}="1H19 BV3"; $resn_exch{"H192 BV3"}="2H19 BV3"; $resn_exch{"H201 BV3"}="1H20 BV3";
$resn_exch{"H202 BV3"}="2H20 BV3"; $resn_exch{"H231 BV3"}="1H23 BV3"; $resn_exch{"H232 BV3"}="2H23 BV3";
$resn_exch{"H241 BV3"}="1H24 BV3"; $resn_exch{"H242 BV3"}="2H24 BV3"; $resn_exch{"H261 BV3"}="1H26 BV3";
$resn_exch{"H262 BV3"}="2H26 BV3"; $resn_exch{"HN81 BV3"}="1HN8 BV3"; $resn_exch{"HN82 BV3"}="2HN8 BV3";
$resn_exch{"H281 BV3"}="1H28 BV3"; $resn_exch{"H282 BV3"}="2H28 BV3"; $resn_exch{"H301 BV3"}="1H30 BV3";
$resn_exch{"H302 BV3"}="2H30 BV3"; $resn_exch{"H311 BV3"}="1H31 BV3"; $resn_exch{"H312 BV3"}="2H31 BV3";
$resn_exch{"H341 BV3"}="1H34 BV3"; $resn_exch{"H342 BV3"}="2H34 BV3"; $resn_exch{"H351 BV3"}="1H35 BV3";
$resn_exch{"H352 BV3"}="2H35 BV3"; $resn_exch{"H361 BV3"}="1H36 BV3"; $resn_exch{"H362 BV3"}="2H36 BV3";
$resn_exch{"H371 BV3"}="1H37 BV3"; $resn_exch{"H372 BV3"}="2H37 BV3"; $resn_exch{"H381 BV3"}="1H38 BV3";
$resn_exch{"H382 BV3"}="2H38 BV3"; $resn_exch{"H391 BV3"}="1H39 BV3"; $resn_exch{"H392 BV3"}="2H39 BV3";
$resn_exch{"H401 BV3"}="1H40 BV3"; $resn_exch{"H402 BV3"}="2H40 BV3"; $resn_exch{"H411 BV3"}="1H41 BV3";
$resn_exch{"H412 BV3"}="2H41 BV3"; $resn_exch{"H421 BV3"}="1H42 BV3"; $resn_exch{"H422 BV3"}="2H42 BV3";
$resn_exch{"H431 BV3"}="1H43 BV3"; $resn_exch{"H432 BV3"}="2H43 BV3"; $resn_exch{"H751 BV4"}="1H75 BV4";
$resn_exch{"H752 BV4"}="2H75 BV4"; $resn_exch{"H11A BV4"}="1H1  BV4"; $resn_exch{" H12 BV4"}="2H1  BV4";
$resn_exch{"H111 BV4"}="1H11 BV4"; $resn_exch{"H112 BV4"}="2H11 BV4"; $resn_exch{"H121 BV4"}="1H12 BV4";
$resn_exch{"H122 BV4"}="2H12 BV4"; $resn_exch{"H141 BV4"}="1H14 BV4"; $resn_exch{"H142 BV4"}="2H14 BV4";
$resn_exch{"H151 BV4"}="1H15 BV4"; $resn_exch{"H152 BV4"}="2H15 BV4"; $resn_exch{"H161 BV4"}="1H16 BV4";
$resn_exch{"H162 BV4"}="2H16 BV4"; $resn_exch{"H171 BV4"}="1H17 BV4"; $resn_exch{"H172 BV4"}="2H17 BV4";
$resn_exch{"H181 BV4"}="1H18 BV4"; $resn_exch{"H182 BV4"}="2H18 BV4"; $resn_exch{"H191 BV4"}="1H19 BV4";
$resn_exch{"H192 BV4"}="2H19 BV4"; $resn_exch{"H201 BV4"}="1H20 BV4"; $resn_exch{"H202 BV4"}="2H20 BV4";
$resn_exch{"H211 BV4"}="1H21 BV4"; $resn_exch{"H212 BV4"}="2H21 BV4"; $resn_exch{"H241 BV4"}="1H24 BV4";
$resn_exch{"H242 BV4"}="2H24 BV4"; $resn_exch{"H251 BV4"}="1H25 BV4"; $resn_exch{"H252 BV4"}="2H25 BV4";
$resn_exch{"H261 BV4"}="1H26 BV4"; $resn_exch{"H262 BV4"}="2H26 BV4"; $resn_exch{"H271 BV4"}="1H27 BV4";
$resn_exch{"H272 BV4"}="2H27 BV4"; $resn_exch{"H281 BV4"}="1H28 BV4"; $resn_exch{"H282 BV4"}="2H28 BV4";
$resn_exch{"H291 BV4"}="1H29 BV4"; $resn_exch{"H292 BV4"}="2H29 BV4"; $resn_exch{"H301 BV4"}="1H30 BV4";
$resn_exch{"H302 BV4"}="2H30 BV4"; $resn_exch{"H311 BV4"}="1H31 BV4"; $resn_exch{"H312 BV4"}="2H31 BV4";
$resn_exch{"H321 BV4"}="1H32 BV4"; $resn_exch{"H322 BV4"}="2H32 BV4"; $resn_exch{"H331 BV4"}="1H33 BV4";
$resn_exch{"H332 BV4"}="2H33 BV4"; $resn_exch{"H351 BV4"}="1H35 BV4"; $resn_exch{"H352 BV4"}="2H35 BV4";
$resn_exch{"HN81 BV4"}="1HN8 BV4"; $resn_exch{"HN82 BV4"}="2HN8 BV4"; $resn_exch{"H371 BV4"}="1H37 BV4";
$resn_exch{"H372 BV4"}="2H37 BV4"; $resn_exch{"H391 BV4"}="1H39 BV4"; $resn_exch{"H392 BV4"}="2H39 BV4";
$resn_exch{"H401 BV4"}="1H40 BV4"; $resn_exch{"H402 BV4"}="2H40 BV4"; $resn_exch{"H411 BV4"}="1H41 BV4";
$resn_exch{"H412 BV4"}="2H41 BV4"; $resn_exch{"H421 BV4"}="1H42 BV4"; $resn_exch{"H422 BV4"}="2H42 BV4";
$resn_exch{"H431 BV4"}="1H43 BV4"; $resn_exch{"H432 BV4"}="2H43 BV4"; $resn_exch{"H441 BV4"}="1H44 BV4";
$resn_exch{"H442 BV4"}="2H44 BV4"; $resn_exch{"H451 BV4"}="1H45 BV4"; $resn_exch{"H452 BV4"}="2H45 BV4";
$resn_exch{"H461 BV4"}="1H46 BV4"; $resn_exch{"H462 BV4"}="2H46 BV4"; $resn_exch{"H471 BV4"}="1H47 BV4";
$resn_exch{"H472 BV4"}="2H47 BV4"; $resn_exch{"H481 BV4"}="1H48 BV4"; $resn_exch{"H482 BV4"}="2H48 BV4";
$resn_exch{"H511 BV4"}="1H51 BV4"; $resn_exch{"H512 BV4"}="2H51 BV4"; $resn_exch{"H521 BV4"}="1H52 BV4";
$resn_exch{"H522 BV4"}="2H52 BV4"; $resn_exch{"H531 BV4"}="1H53 BV4"; $resn_exch{"H532 BV4"}="2H53 BV4";
$resn_exch{"H541 BV4"}="1H54 BV4"; $resn_exch{"H542 BV4"}="2H54 BV4"; $resn_exch{"H551 BV4"}="1H55 BV4";
$resn_exch{"H552 BV4"}="2H55 BV4"; $resn_exch{"H561 BV4"}="1H56 BV4"; $resn_exch{"H562 BV4"}="2H56 BV4";
$resn_exch{"H571 BV4"}="1H57 BV4"; $resn_exch{"H572 BV4"}="2H57 BV4"; $resn_exch{"H581 BV4"}="1H58 BV4";
$resn_exch{"H582 BV4"}="2H58 BV4"; $resn_exch{"H591 BV4"}="1H59 BV4"; $resn_exch{"H592 BV4"}="2H59 BV4";
$resn_exch{"H601 BV4"}="1H60 BV4"; $resn_exch{"H602 BV4"}="2H60 BV4"; $resn_exch{" H61 BVC"}="1H6  BVC";
$resn_exch{" H62 BVC"}="2H6  BVC"; $resn_exch{"HN71 BVC"}="1HN7 BVC"; $resn_exch{"HN72 BVC"}="2HN7 BVC";
$resn_exch{"HO5' BVD"}="*HO5 BVD"; $resn_exch{"H5'1 BVD"}="1H5* BVD"; $resn_exch{"H5'2 BVD"}="2H5* BVD";
$resn_exch{" H4' BVD"}=" H4* BVD"; $resn_exch{" H1' BVD"}=" H1* BVD"; $resn_exch{"H2'1 BVD"}="1H2* BVD";
$resn_exch{"H2'2 BVD"}="2H2* BVD"; $resn_exch{" H3' BVD"}=" H3* BVD"; $resn_exch{"HO3' BVD"}="*HO3 BVD";
$resn_exch{"HN71 BVF"}="1HN7 BVF"; $resn_exch{"HN72 BVF"}="2HN7 BVF"; $resn_exch{" H81 BVF"}="1H8  BVF";
$resn_exch{" H82 BVF"}="2H8  BVF"; $resn_exch{" H83 BVF"}="3H8  BVF"; $resn_exch{" H61 BVG"}="1H6  BVG";
$resn_exch{" H62 BVG"}="2H6  BVG"; $resn_exch{" H3' BVP"}=" H3* BVP"; $resn_exch{"H2'1 BVP"}="1H2* BVP";
$resn_exch{"H2'2 BVP"}="2H2* BVP"; $resn_exch{" H1' BVP"}=" H1* BVP"; $resn_exch{" H4' BVP"}=" H4* BVP";
$resn_exch{"H5'1 BVP"}="1H5* BVP"; $resn_exch{"H5'2 BVP"}="2H5* BVP"; $resn_exch{"H211 BW2"}="1H21 BW2";
$resn_exch{"H212 BW2"}="2H21 BW2"; $resn_exch{"H201 BW2"}="1H20 BW2"; $resn_exch{"H202 BW2"}="2H20 BW2";
$resn_exch{"HN21 BW2"}="1HN2 BW2"; $resn_exch{"HN22 BW2"}="2HN2 BW2"; $resn_exch{" H71 BWD"}="1H7  BWD";
$resn_exch{" H72 BWD"}="2H7  BWD"; $resn_exch{" H81 BWD"}="1H8  BWD"; $resn_exch{" H82 BWD"}="2H8  BWD";
$resn_exch{"H92O BWD"}="OH92 BWD"; $resn_exch{"H221 BWP"}="1H22 BWP"; $resn_exch{"H222 BWP"}="2H22 BWP";
$resn_exch{"H241 BWP"}="1H24 BWP"; $resn_exch{"H242 BWP"}="2H24 BWP"; $resn_exch{"H261 BWP"}="1H26 BWP";
$resn_exch{"H262 BWP"}="2H26 BWP"; $resn_exch{"H263 BWP"}="3H26 BWP"; $resn_exch{"H271 BWP"}="1H27 BWP";
$resn_exch{"H272 BWP"}="2H27 BWP"; $resn_exch{"H273 BWP"}="3H27 BWP"; $resn_exch{"HN31 BX3"}="1HN3 BX3";
$resn_exch{"HN32 BX3"}="2HN3 BX3"; $resn_exch{"H141 BX3"}="1H14 BX3"; $resn_exch{"H142 BX3"}="2H14 BX3";
$resn_exch{"H271 BX3"}="1H27 BX3"; $resn_exch{"H272 BX3"}="2H27 BX3"; $resn_exch{"H291 BX3"}="1H29 BX3";
$resn_exch{"H292 BX3"}="2H29 BX3"; $resn_exch{"H301 BX3"}="1H30 BX3"; $resn_exch{"H302 BX3"}="2H30 BX3";
$resn_exch{"H331 BX3"}="1H33 BX3"; $resn_exch{"H332 BX3"}="2H33 BX3"; $resn_exch{"H333 BX3"}="3H33 BX3";
$resn_exch{"H341 BX3"}="1H34 BX3"; $resn_exch{"H342 BX3"}="2H34 BX3"; $resn_exch{"H5B1 BXP"}="1H5B BXP";
$resn_exch{"H5B2 BXP"}="2H5B BXP"; $resn_exch{"H5A1 BXP"}="1H5A BXP"; $resn_exch{"H5A2 BXP"}="2H5A BXP";
$resn_exch{"H1A1 BXP"}="1H1A BXP"; $resn_exch{"H1A2 BXP"}="2H1A BXP"; $resn_exch{"H221 BYP"}="1H22 BYP";
$resn_exch{"H222 BYP"}="2H22 BYP"; $resn_exch{"H241 BYP"}="1H24 BYP"; $resn_exch{"H242 BYP"}="2H24 BYP";
$resn_exch{"H261 BYP"}="1H26 BYP"; $resn_exch{"H262 BYP"}="2H26 BYP"; $resn_exch{"H263 BYP"}="3H26 BYP";
$resn_exch{"H271 BYP"}="1H27 BYP"; $resn_exch{"H272 BYP"}="2H27 BYP"; $resn_exch{"H273 BYP"}="3H27 BYP";
$resn_exch{"HC11 BYS"}="1HC1 BYS"; $resn_exch{"HXT1 BYS"}="1HXT BYS"; $resn_exch{"HC51 BYS"}="1HC5 BYS";
$resn_exch{"HC52 BYS"}="2HC5 BYS"; $resn_exch{"HC71 BYS"}="1HC7 BYS"; $resn_exch{"HC81 BYS"}="1HC8 BYS";
$resn_exch{"HC91 BYS"}="1HC9 BYS"; $resn_exch{"H101 BYS"}="1H10 BYS"; $resn_exch{"H111 BYS"}="1H11 BYS";
$resn_exch{"H121 BYS"}="1H12 BYS"; $resn_exch{"HXT2 BYS"}="2HXT BYS"; $resn_exch{"H161 BYS"}="1H16 BYS";
$resn_exch{"H162 BYS"}="2H16 BYS"; $resn_exch{"H181 BYS"}="1H18 BYS"; $resn_exch{"H211 BYS"}="1H21 BYS";
$resn_exch{"H212 BYS"}="2H21 BYS"; $resn_exch{"H241 BYS"}="1H24 BYS"; $resn_exch{"H251 BYS"}="1H25 BYS";
$resn_exch{" H31 BZ1"}="1H3  BZ1"; $resn_exch{" H32 BZ1"}="2H3  BZ1"; $resn_exch{"H121 BZ1"}="1H12 BZ1";
$resn_exch{"H122 BZ1"}="2H12 BZ1"; $resn_exch{" H41 BZ1"}="1H4  BZ1"; $resn_exch{" H42 BZ1"}="2H4  BZ1";
$resn_exch{" H51 BZ1"}="1H5  BZ1"; $resn_exch{" H52 BZ1"}="2H5  BZ1"; $resn_exch{" H53 BZ1"}="3H5  BZ1";
$resn_exch{" H61 BZ1"}="1H6  BZ1"; $resn_exch{" H62 BZ1"}="2H6  BZ1"; $resn_exch{"HN21 BZ1"}="1HN2 BZ1";
$resn_exch{"HN22 BZ1"}="2HN2 BZ1"; $resn_exch{" H21 BZ1"}="1H2  BZ1"; $resn_exch{" H22 BZ1"}="2H2  BZ1";
$resn_exch{"H11A BZ1"}="1H1  BZ1"; $resn_exch{" H12 BZ1"}="2H1  BZ1"; $resn_exch{" H13 BZ1"}="3H1  BZ1";
$resn_exch{"H4C1 BZA"}="1H4C BZA"; $resn_exch{"H4C2 BZA"}="2H4C BZA"; $resn_exch{"HO11 BZB"}="1HO1 BZB";
$resn_exch{"HO21 BZB"}="1HO2 BZB"; $resn_exch{"HC21 BZB"}="1HC2 BZB"; $resn_exch{"HC71 BZB"}="1HC7 BZB";
$resn_exch{"HC41 BZB"}="1HC4 BZB"; $resn_exch{"HC61 BZB"}="1HC6 BZB"; $resn_exch{"HC51 BZB"}="1HC5 BZB";
$resn_exch{"HN11 BZC"}="1HN1 BZC"; $resn_exch{"HN12 BZC"}="2HN1 BZC"; $resn_exch{"H181 BZC"}="1H18 BZC";
$resn_exch{"H182 BZC"}="2H18 BZC"; $resn_exch{"H183 BZC"}="3H18 BZC"; $resn_exch{" H61 BZD"}="1H6  BZD";
$resn_exch{" H62 BZD"}="2H6  BZD"; $resn_exch{"H101 BZD"}="1H10 BZD"; $resn_exch{"H111 BZD"}="1H11 BZD";
$resn_exch{"H121 BZD"}="1H12 BZD"; $resn_exch{"H131 BZD"}="1H13 BZD"; $resn_exch{"H141 BZD"}="1H14 BZD";
$resn_exch{"H5'1 BZG"}="1H5* BZG"; $resn_exch{"H5'2 BZG"}="2H5* BZG"; $resn_exch{"HWC1 BZG"}="1HWC BZG";
$resn_exch{"HWC2 BZG"}="2HWC BZG"; $resn_exch{"HIN1 BZG"}="1HIN BZG"; $resn_exch{"HIN2 BZG"}="2HIN BZG";
$resn_exch{" HT' BZG"}=" HT* BZG"; $resn_exch{"HP'1 BZG"}="1HP* BZG"; $resn_exch{"HP'2 BZG"}="2HP* BZG";
$resn_exch{" H4' BZG"}=" H4* BZG"; $resn_exch{" H3' BZG"}=" H3* BZG"; $resn_exch{" H82 BZM"}="2H8  BZM";
$resn_exch{" H81 BZM"}="1H8  BZM"; $resn_exch{" H11 BZO"}="1H1  BZO"; $resn_exch{" H21 BZO"}="1H2  BZO";
$resn_exch{" H22 BZO"}="2H2  BZO"; $resn_exch{" H21 BZP"}="1H2  BZP"; $resn_exch{" H22 BZP"}="2H2  BZP";
$resn_exch{" H31 BZP"}="1H3  BZP"; $resn_exch{" H32 BZP"}="2H3  BZP"; $resn_exch{" H51 BZP"}="1H5  BZP";
$resn_exch{" H52 BZP"}="2H5  BZP"; $resn_exch{" H61 BZP"}="1H6  BZP"; $resn_exch{" H62 BZP"}="2H6  BZP";
$resn_exch{"H4C1 BZR"}="1H4C BZR"; $resn_exch{"H4C2 BZR"}="2H4C BZR"; $resn_exch{" H21 BZS"}="1H2  BZS";
$resn_exch{" H22 BZS"}="2H2  BZS"; $resn_exch{" HB1 BZS"}="1HB  BZS"; $resn_exch{" HB2 BZS"}="2HB  BZS";
$resn_exch{"H241 BZT"}="1H24 BZT"; $resn_exch{"H242 BZT"}="2H24 BZT"; $resn_exch{"H311 BZT"}="1H31 BZT";
$resn_exch{"H312 BZT"}="2H31 BZT"; $resn_exch{"H171 BZT"}="1H17 BZT"; $resn_exch{"H172 BZT"}="2H17 BZT";
$resn_exch{"H231 BZT"}="1H23 BZT"; $resn_exch{"H232 BZT"}="2H23 BZT"; $resn_exch{"H261 BZT"}="1H26 BZT";
$resn_exch{"H262 BZT"}="2H26 BZT"; $resn_exch{"H271 BZT"}="1H27 BZT"; $resn_exch{"H272 BZT"}="2H27 BZT";
$resn_exch{"H221 BZT"}="1H22 BZT"; $resn_exch{"H222 BZT"}="2H22 BZT"; $resn_exch{"H281 BZT"}="1H28 BZT";
$resn_exch{"H282 BZT"}="2H28 BZT"; $resn_exch{"H291 BZT"}="1H29 BZT"; $resn_exch{"H292 BZT"}="2H29 BZT";
$resn_exch{"H301 BZT"}="1H30 BZT"; $resn_exch{"H302 BZT"}="2H30 BZT"; $resn_exch{"H341 BZT"}="1H34 BZT";
$resn_exch{"H342 BZT"}="2H34 BZT"; $resn_exch{" H91 BZU"}="1H9  BZU"; $resn_exch{" H92 BZU"}="2H9  BZU";
$resn_exch{"H111 BZU"}="1H11 BZU"; $resn_exch{"H112 BZU"}="2H11 BZU"; $resn_exch{"H121 BZU"}="1H12 BZU";
$resn_exch{"H122 BZU"}="2H12 BZU"; $resn_exch{"H141 BZU"}="1H14 BZU"; $resn_exch{"H142 BZU"}="2H14 BZU";
$resn_exch{"H151 BZU"}="1H15 BZU"; $resn_exch{"H152 BZU"}="2H15 BZU"; $resn_exch{"H153 BZU"}="3H15 BZU";
$resn_exch{"HN11 BZU"}="1HN1 BZU"; $resn_exch{"HN12 BZU"}="2HN1 BZU"; $resn_exch{"H171 BZU"}="1H17 BZU";
$resn_exch{"H172 BZU"}="2H17 BZU"; $resn_exch{"H181 BZU"}="1H18 BZU"; $resn_exch{"H182 BZU"}="2H18 BZU";
$resn_exch{"H183 BZU"}="3H18 BZU"; $resn_exch{"HX21 BZZ"}="1HX2 BZZ"; $resn_exch{"HX22 BZZ"}="2HX2 BZZ";
$resn_exch{" H11 BZZ"}="1H1  BZZ"; $resn_exch{" H12 BZZ"}="2H1  BZZ"; $resn_exch{"HOP3   C"}="3HOP   C";
$resn_exch{"HOP2   C"}="2HOP   C"; $resn_exch{" H5'   C"}="1H5*   C"; $resn_exch{"H5''   C"}="2H5*   C";
$resn_exch{" H4'   C"}=" H4*   C"; $resn_exch{" H3'   C"}=" H3*   C"; $resn_exch{"HO3'   C"}="H3T    C";
$resn_exch{" H2'   C"}=" H2*   C"; $resn_exch{"HO2'   C"}="2HO*   C"; $resn_exch{" H1'   C"}=" H1*   C";
$resn_exch{" H41   C"}="1H4    C"; $resn_exch{" H42   C"}="2H4    C"; $resn_exch{"H111 C01"}="1H11 C01";
$resn_exch{"H112 C01"}="2H11 C01"; $resn_exch{"H113 C01"}="3H11 C01"; $resn_exch{"H161 C01"}="1H16 C01";
$resn_exch{"H162 C01"}="2H16 C01"; $resn_exch{"H171 C01"}="1H17 C01"; $resn_exch{"H172 C01"}="2H17 C01";
$resn_exch{"H201 C01"}="1H20 C01"; $resn_exch{"H202 C01"}="2H20 C01"; $resn_exch{"H203 C01"}="3H20 C01";
$resn_exch{"H261 C01"}="1H26 C01"; $resn_exch{"H262 C01"}="2H26 C01"; $resn_exch{"H271 C01"}="1H27 C01";
$resn_exch{"H272 C01"}="2H27 C01"; $resn_exch{"H301 C01"}="1H30 C01"; $resn_exch{"H302 C01"}="2H30 C01";
$resn_exch{"H303 C01"}="3H30 C01"; $resn_exch{"H501 C01"}="1H50 C01"; $resn_exch{"H502 C01"}="2H50 C01";
$resn_exch{"H503 C01"}="3H50 C01"; $resn_exch{"H541 C01"}="1H54 C01"; $resn_exch{"H542 C01"}="2H54 C01";
$resn_exch{"H543 C01"}="3H54 C01"; $resn_exch{"HN31 C02"}="1HN3 C02"; $resn_exch{"HN32 C02"}="2HN3 C02";
$resn_exch{"H101 C02"}="1H10 C02"; $resn_exch{"H102 C02"}="2H10 C02"; $resn_exch{"H161 C02"}="1H16 C02";
$resn_exch{"H162 C02"}="2H16 C02"; $resn_exch{"H163 C02"}="3H16 C02"; $resn_exch{"H21N C02"}="NH21 C02";
$resn_exch{"H141 C03"}="1H14 C03"; $resn_exch{"H142 C03"}="2H14 C03"; $resn_exch{"H143 C03"}="3H14 C03";
$resn_exch{"H181 C03"}="1H18 C03"; $resn_exch{"H182 C03"}="2H18 C03"; $resn_exch{"H381 C03"}="1H38 C03";
$resn_exch{"H382 C03"}="2H38 C03"; $resn_exch{"H383 C03"}="3H38 C03"; $resn_exch{"H581 C03"}="1H58 C03";
$resn_exch{"H582 C03"}="2H58 C03"; $resn_exch{"H583 C03"}="3H58 C03"; $resn_exch{" H11 C0M"}="1H1  C0M";
$resn_exch{" H12 C0M"}="2H1  C0M"; $resn_exch{"H271 C0M"}="1H27 C0M"; $resn_exch{"H272 C0M"}="2H27 C0M";
$resn_exch{"H281 C0M"}="1H28 C0M"; $resn_exch{"H282 C0M"}="2H28 C0M"; $resn_exch{"H341 C0M"}="1H34 C0M";
$resn_exch{"H342 C0M"}="2H34 C0M"; $resn_exch{"H331 C0M"}="1H33 C0M"; $resn_exch{"H332 C0M"}="2H33 C0M";
$resn_exch{"H321 C0M"}="1H32 C0M"; $resn_exch{"H322 C0M"}="2H32 C0M"; $resn_exch{"H311 C0M"}="1H31 C0M";
$resn_exch{"H312 C0M"}="2H31 C0M"; $resn_exch{"H301 C0M"}="1H30 C0M"; $resn_exch{"H302 C0M"}="2H30 C0M";
$resn_exch{" H61 C0M"}="1H6  C0M"; $resn_exch{" H62 C0M"}="2H6  C0M"; $resn_exch{" H41 C0M"}="1H4  C0M";
$resn_exch{" H42 C0M"}="2H4  C0M"; $resn_exch{"H11A C0R"}="1H1  C0R"; $resn_exch{" H12 C0R"}="2H1  C0R";
$resn_exch{" H21 C0R"}="1H2  C0R"; $resn_exch{" H22 C0R"}="2H2  C0R"; $resn_exch{" H61 C0R"}="1H6  C0R";
$resn_exch{" H62 C0R"}="2H6  C0R"; $resn_exch{" H71 C0R"}="1H7  C0R"; $resn_exch{" H72 C0R"}="2H7  C0R";
$resn_exch{"H121 C0R"}="1H12 C0R"; $resn_exch{"H122 C0R"}="2H12 C0R"; $resn_exch{"H151 C0R"}="1H15 C0R";
$resn_exch{"H152 C0R"}="2H15 C0R"; $resn_exch{"H161 C0R"}="1H16 C0R"; $resn_exch{"H162 C0R"}="2H16 C0R";
$resn_exch{"H181 C0R"}="1H18 C0R"; $resn_exch{"H182 C0R"}="2H18 C0R"; $resn_exch{"H183 C0R"}="3H18 C0R";
$resn_exch{"H191 C0R"}="1H19 C0R"; $resn_exch{"H192 C0R"}="2H19 C0R"; $resn_exch{"H193 C0R"}="3H19 C0R";
$resn_exch{"H211 C0R"}="1H21 C0R"; $resn_exch{"H212 C0R"}="2H21 C0R"; $resn_exch{"H1C1 C10"}="1H1C C10";
$resn_exch{"H1C2 C10"}="2H1C C10"; $resn_exch{"H1C3 C10"}="3H1C C10"; $resn_exch{"H2C1 C10"}="1H2C C10";
$resn_exch{"H2C2 C10"}="2H2C C10"; $resn_exch{"H3C1 C10"}="1H3C C10"; $resn_exch{"H3C2 C10"}="2H3C C10";
$resn_exch{"H4C1 C10"}="1H4C C10"; $resn_exch{"H4C2 C10"}="2H4C C10"; $resn_exch{"H5C1 C10"}="1H5C C10";
$resn_exch{"H5C2 C10"}="2H5C C10"; $resn_exch{"H6C1 C10"}="1H6C C10"; $resn_exch{"H6C2 C10"}="2H6C C10";
$resn_exch{"H7C1 C10"}="1H7C C10"; $resn_exch{"H7C2 C10"}="2H7C C10"; $resn_exch{"H8C1 C10"}="1H8C C10";
$resn_exch{"H8C2 C10"}="2H8C C10"; $resn_exch{"H9C1 C10"}="1H9C C10"; $resn_exch{"H9C2 C10"}="2H9C C10";
$resn_exch{"H101 C10"}="1H10 C10"; $resn_exch{"H102 C10"}="2H10 C10"; $resn_exch{"H121 C10"}="1H12 C10";
$resn_exch{"H122 C10"}="2H12 C10"; $resn_exch{"H131 C10"}="1H13 C10"; $resn_exch{"H132 C10"}="2H13 C10";
$resn_exch{"H151 C10"}="1H15 C10"; $resn_exch{"H152 C10"}="2H15 C10"; $resn_exch{"H161 C10"}="1H16 C10";
$resn_exch{"H162 C10"}="2H16 C10"; $resn_exch{"H181 C10"}="1H18 C10"; $resn_exch{"H182 C10"}="2H18 C10";
$resn_exch{"H191 C10"}="1H19 C10"; $resn_exch{"H192 C10"}="2H19 C10"; $resn_exch{"H211 C10"}="1H21 C10";
$resn_exch{"H212 C10"}="2H21 C10"; $resn_exch{"H221 C10"}="1H22 C10"; $resn_exch{"H222 C10"}="2H22 C10";
$resn_exch{"H241 C10"}="1H24 C10"; $resn_exch{"H242 C10"}="2H24 C10"; $resn_exch{"H251 C10"}="1H25 C10";
$resn_exch{"H252 C10"}="2H25 C10"; $resn_exch{"H271 C10"}="1H27 C10"; $resn_exch{"H272 C10"}="2H27 C10";
$resn_exch{"H281 C10"}="1H28 C10"; $resn_exch{"H282 C10"}="2H28 C10"; $resn_exch{" H11 C11"}="1H1  C11";
$resn_exch{" H12 C11"}="2H1  C11"; $resn_exch{" H21 C11"}="1H2  C11"; $resn_exch{" H22 C11"}="2H2  C11";
$resn_exch{" H31 C11"}="1H3  C11"; $resn_exch{" H32 C11"}="2H3  C11"; $resn_exch{" H41 C11"}="1H4  C11";
$resn_exch{" H42 C11"}="2H4  C11"; $resn_exch{" H51 C11"}="1H5  C11"; $resn_exch{" H52 C11"}="2H5  C11";
$resn_exch{" H61 C11"}="1H6  C11"; $resn_exch{" H62 C11"}="2H6  C11"; $resn_exch{" H71 C11"}="1H7  C11";
$resn_exch{" H72 C11"}="2H7  C11"; $resn_exch{" H81 C11"}="1H8  C11"; $resn_exch{" H82 C11"}="2H8  C11";
$resn_exch{" H91 C11"}="1H9  C11"; $resn_exch{" H92 C11"}="2H9  C11"; $resn_exch{"H101 C11"}="1H10 C11";
$resn_exch{"H102 C11"}="2H10 C11"; $resn_exch{"H111 C11"}="1H11 C11"; $resn_exch{"H112 C11"}="2H11 C11";
$resn_exch{"H113 C11"}="3H11 C11"; $resn_exch{"HY11 C11"}="1HY1 C11"; $resn_exch{"HY12 C11"}="2HY1 C11";
$resn_exch{"HY21 C11"}="1HY2 C11"; $resn_exch{"HY22 C11"}="2HY2 C11"; $resn_exch{"HY31 C11"}="1HY3 C11";
$resn_exch{"HY32 C11"}="2HY3 C11"; $resn_exch{"HY41 C11"}="1HY4 C11"; $resn_exch{"HY42 C11"}="2HY4 C11";
$resn_exch{"HY43 C11"}="3HY4 C11"; $resn_exch{"HG11 C12"}="1HG1 C12"; $resn_exch{"HG12 C12"}="2HG1 C12";
$resn_exch{"HG13 C12"}="3HG1 C12"; $resn_exch{"HN11 C12"}="1HN1 C12"; $resn_exch{"HN12 C12"}="2HN1 C12";
$resn_exch{"HA31 C12"}="1HA3 C12"; $resn_exch{"HA32 C12"}="2HA3 C12"; $resn_exch{"HB21 C12"}="1HB2 C12";
$resn_exch{"HB22 C12"}="2HB2 C12"; $resn_exch{"H011 C14"}="1H01 C14"; $resn_exch{"H012 C14"}="2H01 C14";
$resn_exch{"H013 C14"}="3H01 C14"; $resn_exch{"H021 C14"}="1H02 C14"; $resn_exch{"H022 C14"}="2H02 C14";
$resn_exch{"H031 C14"}="1H03 C14"; $resn_exch{"H032 C14"}="2H03 C14"; $resn_exch{"H041 C14"}="1H04 C14";
$resn_exch{"H042 C14"}="2H04 C14"; $resn_exch{"H051 C14"}="1H05 C14"; $resn_exch{"H052 C14"}="2H05 C14";
$resn_exch{"H061 C14"}="1H06 C14"; $resn_exch{"H062 C14"}="2H06 C14"; $resn_exch{"H071 C14"}="1H07 C14";
$resn_exch{"H072 C14"}="2H07 C14"; $resn_exch{"H081 C14"}="1H08 C14"; $resn_exch{"H082 C14"}="2H08 C14";
$resn_exch{"H091 C14"}="1H09 C14"; $resn_exch{"H092 C14"}="2H09 C14"; $resn_exch{"H101 C14"}="1H10 C14";
$resn_exch{"H102 C14"}="2H10 C14"; $resn_exch{"H111 C14"}="1H11 C14"; $resn_exch{"H112 C14"}="2H11 C14";
$resn_exch{"H121 C14"}="1H12 C14"; $resn_exch{"H122 C14"}="2H12 C14"; $resn_exch{"H131 C14"}="1H13 C14";
$resn_exch{"H132 C14"}="2H13 C14"; $resn_exch{"H141 C14"}="1H14 C14"; $resn_exch{"H142 C14"}="2H14 C14";
$resn_exch{"H143 C14"}="3H14 C14"; $resn_exch{"H1C1 C15"}="1H1C C15"; $resn_exch{"H1C2 C15"}="2H1C C15";
$resn_exch{"H2C1 C15"}="1H2C C15"; $resn_exch{"H2C2 C15"}="2H2C C15"; $resn_exch{"H3C1 C15"}="1H3C C15";
$resn_exch{"H3C2 C15"}="2H3C C15"; $resn_exch{"H5C1 C15"}="1H5C C15"; $resn_exch{"H5C2 C15"}="2H5C C15";
$resn_exch{"H5C3 C15"}="3H5C C15"; $resn_exch{"H6C1 C15"}="1H6C C15"; $resn_exch{"H6C2 C15"}="2H6C C15";
$resn_exch{"H7C1 C15"}="1H7C C15"; $resn_exch{"H7C2 C15"}="2H7C C15"; $resn_exch{"H8C1 C15"}="1H8C C15";
$resn_exch{"H8C2 C15"}="2H8C C15"; $resn_exch{"HCN1 C15"}="1HCN C15"; $resn_exch{"HCN2 C15"}="2HCN C15";
$resn_exch{"HCN3 C15"}="3HCN C15"; $resn_exch{"HCN4 C15"}="4HCN C15"; $resn_exch{"HCN5 C15"}="5HCN C15";
$resn_exch{"HCN6 C15"}="6HCN C15"; $resn_exch{"H9C1 C15"}="1H9C C15"; $resn_exch{"H9C2 C15"}="2H9C C15";
$resn_exch{"H101 C15"}="1H10 C15"; $resn_exch{"H102 C15"}="2H10 C15"; $resn_exch{"H111 C15"}="1H11 C15";
$resn_exch{"H112 C15"}="2H11 C15"; $resn_exch{"H121 C15"}="1H12 C15"; $resn_exch{"H122 C15"}="2H12 C15";
$resn_exch{"H131 C15"}="1H13 C15"; $resn_exch{"H132 C15"}="2H13 C15"; $resn_exch{"H141 C15"}="1H14 C15";
$resn_exch{"H142 C15"}="2H14 C15"; $resn_exch{"H151 C15"}="1H15 C15"; $resn_exch{"H152 C15"}="2H15 C15";
$resn_exch{"H161 C15"}="1H16 C15"; $resn_exch{"H162 C15"}="2H16 C15"; $resn_exch{"H7B1 C17"}="1H7B C17";
$resn_exch{"H7B2 C17"}="2H7B C17"; $resn_exch{"H4C1 C18"}="1H4C C18"; $resn_exch{"H4C2 C18"}="2H4C C18";
$resn_exch{"H4N1 C18"}="1H4N C18"; $resn_exch{"H4N2 C18"}="2H4N C18"; $resn_exch{"H4N3 C18"}="3H4N C18";
$resn_exch{"H7B1 C18"}="1H7B C18"; $resn_exch{"H7B2 C18"}="2H7B C18"; $resn_exch{" H31 C1A"}="1H3  C1A";
$resn_exch{" H32 C1A"}="2H3  C1A"; $resn_exch{" H33 C1A"}="3H3  C1A"; $resn_exch{" H51 C1A"}="1H5  C1A";
$resn_exch{" H52 C1A"}="2H5  C1A"; $resn_exch{"HN41 C1A"}="1HN4 C1A"; $resn_exch{"HN42 C1A"}="2HN4 C1A";
$resn_exch{" H91 C1A"}="1H9  C1A"; $resn_exch{" H92 C1A"}="2H9  C1A"; $resn_exch{" H93 C1A"}="3H9  C1A";
$resn_exch{"H121 C1A"}="1H12 C1A"; $resn_exch{"H122 C1A"}="2H12 C1A"; $resn_exch{"HN81 C1A"}="1HN8 C1A";
$resn_exch{"HN82 C1A"}="2HN8 C1A"; $resn_exch{"HN21 C1B"}="1HN2 C1B"; $resn_exch{"HN22 C1B"}="2HN2 C1B";
$resn_exch{"H421 C1B"}="1H42 C1B"; $resn_exch{"H422 C1B"}="2H42 C1B"; $resn_exch{" H3B C1B"}="H3'' C1B";
$resn_exch{" H4B C1B"}="H4'' C1B"; $resn_exch{"H511 C1B"}="1H51 C1B"; $resn_exch{"H512 C1B"}="2H51 C1B";
$resn_exch{"H551 C1B"}="1H55 C1B"; $resn_exch{"H552 C1B"}="2H55 C1B"; $resn_exch{" H6B C1B"}="H6'' C1B";
$resn_exch{" H2B C1B"}="H2'' C1B"; $resn_exch{"H121 C1D"}="1H12 C1D"; $resn_exch{"H122 C1D"}="2H12 C1D";
$resn_exch{"H119 C1D"}="19H1 C1D"; $resn_exch{"H241 C1D"}="1H24 C1D"; $resn_exch{"H242 C1D"}="2H24 C1D";
$resn_exch{"H251 C1D"}="1H25 C1D"; $resn_exch{"H252 C1D"}="2H25 C1D"; $resn_exch{"H291 C1D"}="1H29 C1D";
$resn_exch{"H292 C1D"}="2H29 C1D"; $resn_exch{"H330 C1D"}="30H3 C1D"; $resn_exch{"H331 C1D"}="1H33 C1D";
$resn_exch{"H341 C1D"}="1H34 C1D"; $resn_exch{"H2C1 C1D"}="1H2C C1D"; $resn_exch{"H2C2 C1D"}="2H2C C1D";
$resn_exch{"H2C3 C1D"}="3H2C C1D"; $resn_exch{"H335 C1D"}="35H3 C1D"; $resn_exch{"H337 C1D"}="37H3 C1D";
$resn_exch{"H6N1 C1D"}="1H6N C1D"; $resn_exch{"H6N2 C1D"}="2H6N C1D"; $resn_exch{"H4C1 C1F"}="1H4C C1F";
$resn_exch{"H4C2 C1F"}="2H4C C1F"; $resn_exch{"H6C1 C1F"}="1H6C C1F"; $resn_exch{"H6C2 C1F"}="2H6C C1F";
$resn_exch{"H6C3 C1F"}="3H6C C1F"; $resn_exch{"H7C1 C1F"}="1H7C C1F"; $resn_exch{"H7C2 C1F"}="2H7C C1F";
$resn_exch{"H7C3 C1F"}="3H7C C1F"; $resn_exch{"H9C1 C1F"}="1H9C C1F"; $resn_exch{"H9C2 C1F"}="2H9C C1F";
$resn_exch{"H111 C1F"}="1H11 C1F"; $resn_exch{"H112 C1F"}="2H11 C1F"; $resn_exch{"H121 C1F"}="1H12 C1F";
$resn_exch{"H122 C1F"}="2H12 C1F"; $resn_exch{" H71 C1N"}="1H7  C1N"; $resn_exch{" H72 C1N"}="2H7  C1N";
$resn_exch{"H141 C1N"}="1H14 C1N"; $resn_exch{"H142 C1N"}="2H14 C1N"; $resn_exch{"H143 C1N"}="3H14 C1N";
$resn_exch{"H151 C1N"}="1H15 C1N"; $resn_exch{"H152 C1N"}="2H15 C1N"; $resn_exch{"H153 C1N"}="3H15 C1N";
$resn_exch{"H171 C1N"}="1H17 C1N"; $resn_exch{"H172 C1N"}="2H17 C1N"; $resn_exch{"H211 C1N"}="1H21 C1N";
$resn_exch{"H212 C1N"}="2H21 C1N"; $resn_exch{"H231 C1N"}="1H23 C1N"; $resn_exch{"H232 C1N"}="2H23 C1N";
$resn_exch{"H233 C1N"}="3H23 C1N"; $resn_exch{"H241 C1N"}="1H24 C1N"; $resn_exch{"H242 C1N"}="2H24 C1N";
$resn_exch{"H243 C1N"}="3H24 C1N"; $resn_exch{"H281 C1N"}="1H28 C1N"; $resn_exch{"H282 C1N"}="2H28 C1N";
$resn_exch{"H321 C1N"}="1H32 C1N"; $resn_exch{"H322 C1N"}="2H32 C1N"; $resn_exch{"H141 C1P"}="1H14 C1P";
$resn_exch{"H142 C1P"}="2H14 C1P"; $resn_exch{"H131 C1P"}="1H13 C1P"; $resn_exch{"H132 C1P"}="2H13 C1P";
$resn_exch{"H111 C1P"}="1H11 C1P"; $resn_exch{"H112 C1P"}="2H11 C1P"; $resn_exch{"H101 C1P"}="1H10 C1P";
$resn_exch{"H102 C1P"}="2H10 C1P"; $resn_exch{"H201 C1P"}="1H20 C1P"; $resn_exch{"H202 C1P"}="2H20 C1P";
$resn_exch{"H211 C1P"}="1H21 C1P"; $resn_exch{"H212 C1P"}="2H21 C1P"; $resn_exch{" H51 C1P"}="1H5  C1P";
$resn_exch{" H52 C1P"}="2H5  C1P"; $resn_exch{" H71 C1P"}="1H7  C1P"; $resn_exch{" H72 C1P"}="2H7  C1P";
$resn_exch{" H73 C1P"}="3H7  C1P"; $resn_exch{" H81 C1P"}="1H8  C1P"; $resn_exch{" H82 C1P"}="2H8  C1P";
$resn_exch{" H83 C1P"}="3H8  C1P"; $resn_exch{" H11 C1R"}="1H1  C1R"; $resn_exch{" H12 C1R"}="2H1  C1R";
$resn_exch{" H21 C1R"}="1H2  C1R"; $resn_exch{" H22 C1R"}="2H2  C1R"; $resn_exch{" H41 C1R"}="1H4  C1R";
$resn_exch{" H42 C1R"}="2H4  C1R"; $resn_exch{" H51 C1R"}="1H5  C1R"; $resn_exch{" H52 C1R"}="2H5  C1R";
$resn_exch{" H71 C1R"}="1H7  C1R"; $resn_exch{" H72 C1R"}="2H7  C1R"; $resn_exch{" H81 C1R"}="1H8  C1R";
$resn_exch{" H82 C1R"}="2H8  C1R"; $resn_exch{" H91 C1R"}="1H9  C1R"; $resn_exch{" H92 C1R"}="2H9  C1R";
$resn_exch{"H101 C1X"}="1H10 C1X"; $resn_exch{"H102 C1X"}="2H10 C1X"; $resn_exch{"H9C1 C1X"}="1H9C C1X";
$resn_exch{"H9C2 C1X"}="2H9C C1X"; $resn_exch{"H7C1 C1X"}="1H7C C1X"; $resn_exch{"H7C2 C1X"}="2H7C C1X";
$resn_exch{"H5C1 C1X"}="1H5C C1X"; $resn_exch{"H5C2 C1X"}="2H5C C1X"; $resn_exch{"H2C1 C1X"}="1H2C C1X";
$resn_exch{"H2C2 C1X"}="2H2C C1X"; $resn_exch{"HEC1 C1X"}="1HEC C1X"; $resn_exch{"HEC2 C1X"}="2HEC C1X";
$resn_exch{"HDC1 C1X"}="1HDC C1X"; $resn_exch{"HDC2 C1X"}="2HDC C1X"; $resn_exch{"HGC1 C1X"}="1HGC C1X";
$resn_exch{"HGC2 C1X"}="2HGC C1X"; $resn_exch{"HBC1 C1X"}="1HBC C1X"; $resn_exch{"HBC2 C1X"}="2HBC C1X";
$resn_exch{" HN1 C1X"}="1HN  C1X"; $resn_exch{" HN2 C1X"}="2HN  C1X"; $resn_exch{" H11 C20"}="1H1  C20";
$resn_exch{"H12A C20"}="2H1  C20"; $resn_exch{" H13 C20"}="3H1  C20"; $resn_exch{" H71 C20"}="1H7  C20";
$resn_exch{" H72 C20"}="2H7  C20"; $resn_exch{" H73 C20"}="3H7  C20"; $resn_exch{" H81 C20"}="1H8  C20";
$resn_exch{" H82 C20"}="2H8  C20"; $resn_exch{" H83 C20"}="3H8  C20"; $resn_exch{"H131 C20"}="1H13 C20";
$resn_exch{"H132 C20"}="2H13 C20"; $resn_exch{"H221 C20"}="1H22 C20"; $resn_exch{"H222 C20"}="2H22 C20";
$resn_exch{"H241 C20"}="1H24 C20"; $resn_exch{"H242 C20"}="2H24 C20"; $resn_exch{"H261 C20"}="1H26 C20";
$resn_exch{"H262 C20"}="2H26 C20"; $resn_exch{"H271 C20"}="1H27 C20"; $resn_exch{"H272 C20"}="2H27 C20";
$resn_exch{"H281 C20"}="1H28 C20"; $resn_exch{"H282 C20"}="2H28 C20"; $resn_exch{"H291 C20"}="1H29 C20";
$resn_exch{"H292 C20"}="2H29 C20"; $resn_exch{"H301 C20"}="1H30 C20"; $resn_exch{"H302 C20"}="2H30 C20";
$resn_exch{"H371 C20"}="1H37 C20"; $resn_exch{"H372 C20"}="2H37 C20"; $resn_exch{"H373 C20"}="3H37 C20";
$resn_exch{"H381 C20"}="1H38 C20"; $resn_exch{"H382 C20"}="2H38 C20"; $resn_exch{"H383 C20"}="3H38 C20";
$resn_exch{"H411 C20"}="1H41 C20"; $resn_exch{"H412 C20"}="2H41 C20"; $resn_exch{"H413 C20"}="3H41 C20";
$resn_exch{"H10N C24"}="NH10 C24"; $resn_exch{"H111 C24"}="1H11 C24"; $resn_exch{"H112 C24"}="2H11 C24";
$resn_exch{"H291 C24"}="1H29 C24"; $resn_exch{"H292 C24"}="2H29 C24"; $resn_exch{"H301 C24"}="1H30 C24";
$resn_exch{"H302 C24"}="2H30 C24"; $resn_exch{"H341 C24"}="1H34 C24"; $resn_exch{"H342 C24"}="2H34 C24";
$resn_exch{"H351 C24"}="1H35 C24"; $resn_exch{"H352 C24"}="2H35 C24"; $resn_exch{"H353 C24"}="3H35 C24";
$resn_exch{"H171 C24"}="1H17 C24"; $resn_exch{"H172 C24"}="2H17 C24"; $resn_exch{"H173 C24"}="3H17 C24";
$resn_exch{"HN81 C24"}="1HN8 C24"; $resn_exch{"HN82 C24"}="2HN8 C24"; $resn_exch{"HOP3 C25"}="3HOP C25";
$resn_exch{"HOP2 C25"}="2HOP C25"; $resn_exch{" H5' C25"}="1H5* C25"; $resn_exch{"H5'' C25"}="2H5* C25";
$resn_exch{" H4' C25"}=" H4* C25"; $resn_exch{" H3' C25"}=" H3* C25"; $resn_exch{"HO'3 C25"}="3HO* C25";
$resn_exch{" H2' C25"}=" H2* C25"; $resn_exch{"HO2' C25"}="2HO* C25"; $resn_exch{" H1' C25"}=" H1* C25";
$resn_exch{" H41 C25"}="1H4  C25"; $resn_exch{" H42 C25"}="2H4  C25"; $resn_exch{"H1C1 C26"}="1H1C C26";
$resn_exch{"H1C2 C26"}="2H1C C26"; $resn_exch{"H7C1 C26"}="1H7C C26"; $resn_exch{"H7C2 C26"}="2H7C C26";
$resn_exch{"H8C1 C26"}="1H8C C26"; $resn_exch{"H8C2 C26"}="2H8C C26"; $resn_exch{"H8C3 C26"}="3H8C C26";
$resn_exch{"H3C1 C26"}="1H3C C26"; $resn_exch{"H3C2 C26"}="2H3C C26"; $resn_exch{"H4C1 C26"}="1H4C C26";
$resn_exch{"H4C2 C26"}="2H4C C26"; $resn_exch{"H5C1 C26"}="1H5C C26"; $resn_exch{"H5C2 C26"}="2H5C C26";
$resn_exch{"H6C1 C26"}="1H6C C26"; $resn_exch{"H6C2 C26"}="2H6C C26"; $resn_exch{"H6C3 C26"}="3H6C C26";
$resn_exch{"H1C1 C27"}="1H1C C27"; $resn_exch{"H1C2 C27"}="2H1C C27"; $resn_exch{"H1C3 C27"}="3H1C C27";
$resn_exch{"H141 C27"}="1H14 C27"; $resn_exch{"H142 C27"}="2H14 C27"; $resn_exch{"H151 C27"}="1H15 C27";
$resn_exch{"H152 C27"}="2H15 C27"; $resn_exch{"H171 C27"}="1H17 C27"; $resn_exch{"H172 C27"}="2H17 C27";
$resn_exch{"H173 C27"}="3H17 C27"; $resn_exch{"H181 C27"}="1H18 C27"; $resn_exch{"H182 C27"}="2H18 C27";
$resn_exch{"H221 C27"}="1H22 C27"; $resn_exch{"H222 C27"}="2H22 C27"; $resn_exch{"H223 C27"}="3H22 C27";
$resn_exch{"H251 C27"}="1H25 C27"; $resn_exch{"H252 C27"}="2H25 C27"; $resn_exch{"H131 C28"}="1H13 C28";
$resn_exch{"H132 C28"}="2H13 C28"; $resn_exch{"H191 C28"}="1H19 C28"; $resn_exch{"H192 C28"}="2H19 C28";
$resn_exch{"H201 C28"}="1H20 C28"; $resn_exch{"H202 C28"}="2H20 C28"; $resn_exch{"H261 C28"}="1H26 C28";
$resn_exch{"H262 C28"}="2H26 C28"; $resn_exch{"H271 C28"}="1H27 C28"; $resn_exch{"H272 C28"}="2H27 C28";
$resn_exch{"H291 C28"}="1H29 C28"; $resn_exch{"H292 C28"}="2H29 C28"; $resn_exch{"H301 C28"}="1H30 C28";
$resn_exch{"H302 C28"}="2H30 C28"; $resn_exch{"H321 C28"}="1H32 C28"; $resn_exch{"H322 C28"}="2H32 C28";
$resn_exch{"H311 C28"}="1H31 C28"; $resn_exch{"H312 C28"}="2H31 C28"; $resn_exch{" HN1 C2A"}="1HN  C2A";
$resn_exch{" HN2 C2A"}="2HN  C2A"; $resn_exch{" HC1 C2A"}="1HC  C2A"; $resn_exch{" HC2 C2A"}="2HC  C2A";
$resn_exch{"HN11 C2B"}="1HN1 C2B"; $resn_exch{"HN12 C2B"}="2HN1 C2B"; $resn_exch{" HB1 C2B"}="1HB  C2B";
$resn_exch{" HB2 C2B"}="2HB  C2B"; $resn_exch{"H131 C2D"}="1H13 C2D"; $resn_exch{"H132 C2D"}="2H13 C2D";
$resn_exch{"H151 C2D"}="1H15 C2D"; $resn_exch{"H152 C2D"}="2H15 C2D"; $resn_exch{"H161 C2D"}="1H16 C2D";
$resn_exch{"H162 C2D"}="2H16 C2D"; $resn_exch{"H117 C2D"}="17H1 C2D"; $resn_exch{"H191 C2D"}="1H19 C2D";
$resn_exch{"H220 C2D"}="20H2 C2D"; $resn_exch{"H241 C2D"}="1H24 C2D"; $resn_exch{"H242 C2D"}="2H24 C2D";
$resn_exch{"H251 C2D"}="1H25 C2D"; $resn_exch{"H252 C2D"}="2H25 C2D"; $resn_exch{"H261 C2D"}="1H26 C2D";
$resn_exch{"H227 C2D"}="27H2 C2D"; $resn_exch{"H281 C2D"}="1H28 C2D"; $resn_exch{"H301 C2D"}="1H30 C2D";
$resn_exch{"H311 C2D"}="1H31 C2D"; $resn_exch{"HN21 C2F"}="1HN2 C2F"; $resn_exch{"HN22 C2F"}="2HN2 C2F";
$resn_exch{" H71 C2F"}="1H7  C2F"; $resn_exch{" H72 C2F"}="2H7  C2F"; $resn_exch{" H91 C2F"}="1H9  C2F";
$resn_exch{" H92 C2F"}="2H9  C2F"; $resn_exch{"H111 C2F"}="1H11 C2F"; $resn_exch{"H112 C2F"}="2H11 C2F";
$resn_exch{"H113 C2F"}="3H11 C2F"; $resn_exch{" HB1 C2F"}="1HB  C2F"; $resn_exch{" HB2 C2F"}="2HB  C2F";
$resn_exch{" HG1 C2F"}="1HG  C2F"; $resn_exch{" HG2 C2F"}="2HG  C2F"; $resn_exch{"HOE2 C2F"}="2HOE C2F";
$resn_exch{"HN41 C2G"}="1HN4 C2G"; $resn_exch{"HN42 C2G"}="2HN4 C2G"; $resn_exch{"HC51 C2G"}="1HC5 C2G";
$resn_exch{"HC52 C2G"}="2HC5 C2G"; $resn_exch{"H1G1 C2G"}="1H1G C2G"; $resn_exch{"H1G2 C2G"}="2H1G C2G";
$resn_exch{"H3G1 C2G"}="1H3G C2G"; $resn_exch{"H3G2 C2G"}="2H3G C2G"; $resn_exch{"HOP3 C2L"}="3HOP C2L";
$resn_exch{"HOP2 C2L"}="2HOP C2L"; $resn_exch{" H5' C2L"}="1H5* C2L"; $resn_exch{"H5'' C2L"}="2H5* C2L";
$resn_exch{" H4' C2L"}=" H4* C2L"; $resn_exch{" H3' C2L"}=" H3* C2L"; $resn_exch{" H2' C2L"}=" H2* C2L";
$resn_exch{"HO2' C2L"}="2HO* C2L"; $resn_exch{" H1' C2L"}=" H1* C2L"; $resn_exch{" H41 C2L"}="1H4  C2L";
$resn_exch{" H42 C2L"}="2H4  C2L"; $resn_exch{"HA'1 C2L"}="1HA* C2L"; $resn_exch{"HA'2 C2L"}="2HA* C2L";
$resn_exch{"HB'1 C2L"}="1HB* C2L"; $resn_exch{"HB'2 C2L"}="2HB* C2L"; $resn_exch{"HD'1 C2L"}="1HD* C2L";
$resn_exch{"HD'2 C2L"}="2HD* C2L"; $resn_exch{"HD'3 C2L"}="3HD* C2L"; $resn_exch{" H71 C2L"}="1H7  C2L";
$resn_exch{" H72 C2L"}="2H7  C2L"; $resn_exch{" H73 C2L"}="3H7  C2L"; $resn_exch{"HN31 C2M"}="1HN3 C2M";
$resn_exch{"HN32 C2M"}="2HN3 C2M"; $resn_exch{"HN41 C2M"}="1HN4 C2M"; $resn_exch{"HN42 C2M"}="2HN4 C2M";
$resn_exch{"H121 C2M"}="1H12 C2M"; $resn_exch{"H122 C2M"}="2H12 C2M"; $resn_exch{"H123 C2M"}="3H12 C2M";
$resn_exch{" HB1 C2N"}="1HB  C2N"; $resn_exch{" HB2 C2N"}="2HB  C2N"; $resn_exch{" HN1 C2N"}="1HN  C2N";
$resn_exch{" HN2 C2N"}="2HN  C2N"; $resn_exch{" HN3 C2N"}="3HN  C2N"; $resn_exch{"HN41 C2P"}="1HN4 C2P";
$resn_exch{"HN42 C2P"}="2HN4 C2P"; $resn_exch{" H1' C2P"}=" H1* C2P"; $resn_exch{" H2' C2P"}=" H2* C2P";
$resn_exch{" H3' C2P"}=" H3* C2P"; $resn_exch{" H4' C2P"}=" H4* C2P"; $resn_exch{"HO3' C2P"}="*HO3 C2P";
$resn_exch{"H5'1 C2P"}="1H5* C2P"; $resn_exch{"H5'2 C2P"}="2H5* C2P"; $resn_exch{"HO5' C2P"}="*HO5 C2P";
$resn_exch{"HOP2 C2P"}="2HOP C2P"; $resn_exch{"HOP3 C2P"}="3HOP C2P"; $resn_exch{" H2' C2R"}=" H2* C2R";
$resn_exch{" H3' C2R"}=" H3* C2R"; $resn_exch{"H5'1 C2R"}="1H5* C2R"; $resn_exch{"H5'2 C2R"}="2H5* C2R";
$resn_exch{" H4' C2R"}=" H4* C2R"; $resn_exch{" H1' C2R"}=" H1* C2R"; $resn_exch{"HN51 C2R"}="1HN5 C2R";
$resn_exch{"HN52 C2R"}="2HN5 C2R"; $resn_exch{"HOP3 C2S"}="3HOP C2S"; $resn_exch{"HOP2 C2S"}="2HOP C2S";
$resn_exch{" H5' C2S"}="1H5* C2S"; $resn_exch{"H5'' C2S"}="2H5* C2S"; $resn_exch{" H4' C2S"}=" H4* C2S";
$resn_exch{" H3' C2S"}=" H3* C2S"; $resn_exch{"HO3' C2S"}="H3T  C2S"; $resn_exch{"H2'1 C2S"}="1H2* C2S";
$resn_exch{" H1' C2S"}=" H1* C2S"; $resn_exch{" H41 C2S"}="1H4  C2S"; $resn_exch{" H42 C2S"}="2H4  C2S";
$resn_exch{"H2'2 C2S"}="2H2* C2S"; $resn_exch{"H5'1 C31"}="1H5* C31"; $resn_exch{"H5'2 C31"}="2H5* C31";
$resn_exch{" H4' C31"}=" H4* C31"; $resn_exch{" H3' C31"}=" H3* C31"; $resn_exch{" H2' C31"}=" H2* C31";
$resn_exch{" H1' C31"}=" H1* C31"; $resn_exch{"H4N1 C31"}="1H4N C31"; $resn_exch{"H4N2 C31"}="2H4N C31";
$resn_exch{"HA'1 C31"}="1HA* C31"; $resn_exch{"HA'2 C31"}="2HA* C31"; $resn_exch{"HB'1 C31"}="1HB* C31";
$resn_exch{"HB'2 C31"}="2HB* C31"; $resn_exch{"HCC1 C31"}="1HCC C31"; $resn_exch{"HCC2 C31"}="2HCC C31";
$resn_exch{"HD'1 C31"}="1HD* C31"; $resn_exch{"HD'2 C31"}="2HD* C31"; $resn_exch{"H5'1 C32"}="1H5* C32";
$resn_exch{"H5'2 C32"}="2H5* C32"; $resn_exch{" H4' C32"}=" H4* C32"; $resn_exch{" H3' C32"}=" H3* C32";
$resn_exch{"H2'1 C32"}="1H2* C32"; $resn_exch{"H2'2 C32"}="2H2* C32"; $resn_exch{" H1' C32"}=" H1* C32";
$resn_exch{"H4N1 C32"}="1H4N C32"; $resn_exch{"H4N2 C32"}="2H4N C32"; $resn_exch{" H41 C33"}="1H4  C33";
$resn_exch{" H42 C33"}="2H4  C33"; $resn_exch{"H121 C33"}="1H12 C33"; $resn_exch{"H122 C33"}="2H12 C33";
$resn_exch{"H111 C33"}="1H11 C33"; $resn_exch{"H112 C33"}="2H11 C33"; $resn_exch{" H91 C33"}="1H9  C33";
$resn_exch{" H92 C33"}="2H9  C33"; $resn_exch{"H151 C33"}="1H15 C33"; $resn_exch{"H152 C33"}="2H15 C33";
$resn_exch{"H161 C33"}="1H16 C33"; $resn_exch{"H162 C33"}="2H16 C33"; $resn_exch{"H191 C33"}="1H19 C33";
$resn_exch{"H192 C33"}="2H19 C33"; $resn_exch{"H221 C33"}="1H22 C33"; $resn_exch{"H222 C33"}="2H22 C33";
$resn_exch{"H181 C33"}="1H18 C33"; $resn_exch{"H182 C33"}="2H18 C33"; $resn_exch{"H183 C33"}="3H18 C33";
$resn_exch{"H211 C33"}="1H21 C33"; $resn_exch{"H212 C33"}="2H21 C33"; $resn_exch{"H213 C33"}="3H21 C33";
$resn_exch{"H231 C33"}="1H23 C33"; $resn_exch{"H232 C33"}="2H23 C33"; $resn_exch{"H241 C33"}="1H24 C33";
$resn_exch{"H242 C33"}="2H24 C33"; $resn_exch{"H271 C33"}="1H27 C33"; $resn_exch{"H272 C33"}="2H27 C33";
$resn_exch{"H273 C33"}="3H27 C33"; $resn_exch{"H261 C33"}="1H26 C33"; $resn_exch{"H262 C33"}="2H26 C33";
$resn_exch{"H263 C33"}="3H26 C33"; $resn_exch{"H281 C33"}="1H28 C33"; $resn_exch{"H282 C33"}="2H28 C33";
$resn_exch{"H291 C33"}="1H29 C33"; $resn_exch{"H292 C33"}="2H29 C33"; $resn_exch{"H301 C33"}="1H30 C33";
$resn_exch{"H302 C33"}="2H30 C33"; $resn_exch{"H303 C33"}="3H30 C33"; $resn_exch{"HOP2 C34"}="2HOP C34";
$resn_exch{"HOP3 C34"}="3HOP C34"; $resn_exch{" H5' C34"}="1H5* C34"; $resn_exch{"H5'' C34"}="2H5* C34";
$resn_exch{" H4' C34"}=" H4* C34"; $resn_exch{" H1' C34"}=" H1* C34"; $resn_exch{"H7C1 C34"}="1H7C C34";
$resn_exch{"H7C2 C34"}="2H7C C34"; $resn_exch{"H7C3 C34"}="3H7C C34"; $resn_exch{" H2' C34"}="1H2* C34";
$resn_exch{"H2'' C34"}="2H2* C34"; $resn_exch{" H3' C34"}=" H3* C34"; $resn_exch{"HO3' C34"}="H3T  C34";
$resn_exch{"H311 C35"}="1H31 C35"; $resn_exch{"H312 C35"}="2H31 C35"; $resn_exch{"H313 C35"}="3H31 C35";
$resn_exch{"H241 C35"}="1H24 C35"; $resn_exch{"H242 C35"}="2H24 C35"; $resn_exch{"H251 C35"}="1H25 C35";
$resn_exch{"H252 C35"}="2H25 C35"; $resn_exch{"H261 C35"}="1H26 C35"; $resn_exch{"H262 C35"}="2H26 C35";
$resn_exch{"H6N1 C35"}="1H6N C35"; $resn_exch{"H6N2 C35"}="2H6N C35"; $resn_exch{"H7N1 C35"}="1H7N C35";
$resn_exch{"H7N2 C35"}="2H7N C35"; $resn_exch{"H181 C35"}="1H18 C35"; $resn_exch{"H182 C35"}="2H18 C35";
$resn_exch{"H201 C35"}="1H20 C35"; $resn_exch{"H202 C35"}="2H20 C35"; $resn_exch{"H203 C35"}="3H20 C35";
$resn_exch{"H211 C35"}="1H21 C35"; $resn_exch{"H212 C35"}="2H21 C35"; $resn_exch{"H213 C35"}="3H21 C35";
$resn_exch{"H151 C35"}="1H15 C35"; $resn_exch{"H152 C35"}="2H15 C35"; $resn_exch{"H153 C35"}="3H15 C35";
$resn_exch{"H131 C35"}="1H13 C35"; $resn_exch{"H132 C35"}="2H13 C35"; $resn_exch{"H141 C35"}="1H14 C35";
$resn_exch{"H142 C35"}="2H14 C35"; $resn_exch{"H143 C35"}="3H14 C35"; $resn_exch{"H7C1 C35"}="1H7C C35";
$resn_exch{"H7C2 C35"}="2H7C C35"; $resn_exch{"H1N1 C35"}="1H1N C35"; $resn_exch{"H1N2 C35"}="2H1N C35";
$resn_exch{"H5'1 C36"}="1H5* C36"; $resn_exch{"H5'2 C36"}="2H5* C36"; $resn_exch{" H4' C36"}=" H4* C36";
$resn_exch{" H3' C36"}=" H3* C36"; $resn_exch{"H2'1 C36"}="1H2* C36"; $resn_exch{"H2'2 C36"}="2H2* C36";
$resn_exch{" H1' C36"}=" H1* C36"; $resn_exch{"H4N1 C36"}="1H4N C36"; $resn_exch{"H4N2 C36"}="2H4N C36";
$resn_exch{"H5'1 C37"}="1H5* C37"; $resn_exch{"H5'2 C37"}="2H5* C37"; $resn_exch{" H4' C37"}=" H4* C37";
$resn_exch{" H3' C37"}=" H3* C37"; $resn_exch{"H2'1 C37"}="1H2* C37"; $resn_exch{"H2'2 C37"}="2H2* C37";
$resn_exch{" H1' C37"}=" H1* C37"; $resn_exch{"H4N1 C37"}="1H4N C37"; $resn_exch{"H4N2 C37"}="2H4N C37";
$resn_exch{"H5'1 C38"}="1H5* C38"; $resn_exch{"H5'2 C38"}="2H5* C38"; $resn_exch{" H4' C38"}=" H4* C38";
$resn_exch{" H3' C38"}=" H3* C38"; $resn_exch{"H2'1 C38"}="1H2* C38"; $resn_exch{"H2'2 C38"}="2H2* C38";
$resn_exch{" H1' C38"}=" H1* C38"; $resn_exch{"H4N1 C38"}="1H4N C38"; $resn_exch{"H4N2 C38"}="2H4N C38";
$resn_exch{"H1C1 C39"}="1H1C C39"; $resn_exch{"H1C2 C39"}="2H1C C39"; $resn_exch{"H1C3 C39"}="3H1C C39";
$resn_exch{"H3C1 C39"}="1H3C C39"; $resn_exch{"H3C2 C39"}="2H3C C39"; $resn_exch{"H4C1 C39"}="1H4C C39";
$resn_exch{"H4C2 C39"}="2H4C C39"; $resn_exch{"H5C1 C39"}="1H5C C39"; $resn_exch{"H5C2 C39"}="2H5C C39";
$resn_exch{"H161 C39"}="1H16 C39"; $resn_exch{"H162 C39"}="2H16 C39"; $resn_exch{"H211 C39"}="1H21 C39";
$resn_exch{"H212 C39"}="2H21 C39"; $resn_exch{"H191 C39"}="1H19 C39"; $resn_exch{"H192 C39"}="2H19 C39";
$resn_exch{"H193 C39"}="3H19 C39"; $resn_exch{"H201 C39"}="1H20 C39"; $resn_exch{"H202 C39"}="2H20 C39";
$resn_exch{"H203 C39"}="3H20 C39"; $resn_exch{"H231 C39"}="1H23 C39"; $resn_exch{"H232 C39"}="2H23 C39";
$resn_exch{"H261 C39"}="1H26 C39"; $resn_exch{"H262 C39"}="2H26 C39"; $resn_exch{"H281 C39"}="1H28 C39";
$resn_exch{"H282 C39"}="2H28 C39"; $resn_exch{"H283 C39"}="3H28 C39"; $resn_exch{"H321 C39"}="1H32 C39";
$resn_exch{"H322 C39"}="2H32 C39"; $resn_exch{"H331 C39"}="1H33 C39"; $resn_exch{"H332 C39"}="2H33 C39";
$resn_exch{"H341 C39"}="1H34 C39"; $resn_exch{"H342 C39"}="2H34 C39"; $resn_exch{"H351 C39"}="1H35 C39";
$resn_exch{"H352 C39"}="2H35 C39"; $resn_exch{"H353 C39"}="3H35 C39"; $resn_exch{"H11A C3A"}="1H1  C3A";
$resn_exch{" H12 C3A"}="2H1  C3A"; $resn_exch{"H31A C3A"}="1H3  C3A"; $resn_exch{"H32A C3A"}="2H3  C3A";
$resn_exch{" H91 C3A"}="1H9  C3A"; $resn_exch{" H92 C3A"}="2H9  C3A"; $resn_exch{"H101 C3A"}="1H10 C3A";
$resn_exch{"H102 C3A"}="2H10 C3A"; $resn_exch{"H121 C3A"}="1H12 C3A"; $resn_exch{"H122 C3A"}="2H12 C3A";
$resn_exch{"H131 C3A"}="1H13 C3A"; $resn_exch{"H132 C3A"}="2H13 C3A"; $resn_exch{"H151 C3A"}="1H15 C3A";
$resn_exch{"H152 C3A"}="2H15 C3A"; $resn_exch{"H221 C3A"}="1H22 C3A"; $resn_exch{"H222 C3A"}="2H22 C3A";
$resn_exch{"H331 C3A"}="1H33 C3A"; $resn_exch{"H332 C3A"}="2H33 C3A"; $resn_exch{"H401 C3A"}="1H40 C3A";
$resn_exch{"H402 C3A"}="2H40 C3A"; $resn_exch{" H31 C3D"}="1H3  C3D"; $resn_exch{" H32 C3D"}="2H3  C3D";
$resn_exch{" H41 C3D"}="1H4  C3D"; $resn_exch{" H42 C3D"}="2H4  C3D"; $resn_exch{"H241 C3D"}="1H24 C3D";
$resn_exch{"H242 C3D"}="2H24 C3D"; $resn_exch{"H251 C3D"}="1H25 C3D"; $resn_exch{"H252 C3D"}="2H25 C3D";
$resn_exch{"H271 C3D"}="1H27 C3D"; $resn_exch{"H272 C3D"}="2H27 C3D"; $resn_exch{"H281 C3D"}="1H28 C3D";
$resn_exch{"H282 C3D"}="2H28 C3D"; $resn_exch{"H291 C3D"}="1H29 C3D"; $resn_exch{"H292 C3D"}="2H29 C3D";
$resn_exch{"H301 C3D"}="1H30 C3D"; $resn_exch{"H302 C3D"}="2H30 C3D"; $resn_exch{"H1N1 C3M"}="1H1N C3M";
$resn_exch{"H1N2 C3M"}="2H1N C3M"; $resn_exch{"H3C1 C3M"}="1H3C C3M"; $resn_exch{"H3C2 C3M"}="2H3C C3M";
$resn_exch{"H121 C3M"}="1H12 C3M"; $resn_exch{"H122 C3M"}="2H12 C3M"; $resn_exch{" H41 C3O"}="1H4  C3O";
$resn_exch{" H42 C3O"}="2H4  C3O"; $resn_exch{"H121 C3O"}="1H12 C3O"; $resn_exch{"H122 C3O"}="2H12 C3O";
$resn_exch{"H111 C3O"}="1H11 C3O"; $resn_exch{"H112 C3O"}="2H11 C3O"; $resn_exch{" H91 C3O"}="1H9  C3O";
$resn_exch{" H92 C3O"}="2H9  C3O"; $resn_exch{"H151 C3O"}="1H15 C3O"; $resn_exch{"H152 C3O"}="2H15 C3O";
$resn_exch{"H161 C3O"}="1H16 C3O"; $resn_exch{"H162 C3O"}="2H16 C3O"; $resn_exch{"H191 C3O"}="1H19 C3O";
$resn_exch{"H192 C3O"}="2H19 C3O"; $resn_exch{"H221 C3O"}="1H22 C3O"; $resn_exch{"H222 C3O"}="2H22 C3O";
$resn_exch{"H181 C3O"}="1H18 C3O"; $resn_exch{"H182 C3O"}="2H18 C3O"; $resn_exch{"H183 C3O"}="3H18 C3O";
$resn_exch{"H211 C3O"}="1H21 C3O"; $resn_exch{"H212 C3O"}="2H21 C3O"; $resn_exch{"H213 C3O"}="3H21 C3O";
$resn_exch{"H231 C3O"}="1H23 C3O"; $resn_exch{"H232 C3O"}="2H23 C3O"; $resn_exch{"H241 C3O"}="1H24 C3O";
$resn_exch{"H242 C3O"}="2H24 C3O"; $resn_exch{"H271 C3O"}="1H27 C3O"; $resn_exch{"H272 C3O"}="2H27 C3O";
$resn_exch{"H273 C3O"}="3H27 C3O"; $resn_exch{"H261 C3O"}="1H26 C3O"; $resn_exch{"H262 C3O"}="2H26 C3O";
$resn_exch{"H263 C3O"}="3H26 C3O"; $resn_exch{"H291 C3O"}="1H29 C3O"; $resn_exch{"H292 C3O"}="2H29 C3O";
$resn_exch{"H301 C3O"}="1H30 C3O"; $resn_exch{"H302 C3O"}="2H30 C3O"; $resn_exch{"H311 C3O"}="1H31 C3O";
$resn_exch{"H312 C3O"}="2H31 C3O"; $resn_exch{"H313 C3O"}="3H31 C3O"; $resn_exch{"HOP2 C3P"}="2HOP C3P";
$resn_exch{"HOP3 C3P"}="3HOP C3P"; $resn_exch{"HO5' C3P"}="*HO5 C3P"; $resn_exch{"H5'1 C3P"}="1H5* C3P";
$resn_exch{"H5'2 C3P"}="2H5* C3P"; $resn_exch{" H4' C3P"}=" H4* C3P"; $resn_exch{" H3' C3P"}=" H3* C3P";
$resn_exch{"H2'1 C3P"}="1H2* C3P"; $resn_exch{"HO2' C3P"}="*HO2 C3P"; $resn_exch{" H1' C3P"}=" H1* C3P";
$resn_exch{"HN41 C3P"}="1HN4 C3P"; $resn_exch{"HN42 C3P"}="2HN4 C3P"; $resn_exch{"H651 C3S"}="1H65 C3S";
$resn_exch{"H652 C3S"}="2H65 C3S"; $resn_exch{"H653 C3S"}="3H65 C3S"; $resn_exch{"H691 C3S"}="1H69 C3S";
$resn_exch{"H692 C3S"}="2H69 C3S"; $resn_exch{"H693 C3S"}="3H69 C3S"; $resn_exch{"H601 C3S"}="1H60 C3S";
$resn_exch{"H602 C3S"}="2H60 C3S"; $resn_exch{"H571 C3S"}="1H57 C3S"; $resn_exch{"H572 C3S"}="2H57 C3S";
$resn_exch{"H541 C3S"}="1H54 C3S"; $resn_exch{"H542 C3S"}="2H54 C3S"; $resn_exch{"H501 C3S"}="1H50 C3S";
$resn_exch{"H502 C3S"}="2H50 C3S"; $resn_exch{"H503 C3S"}="3H50 C3S"; $resn_exch{"H351 C3S"}="1H35 C3S";
$resn_exch{"H352 C3S"}="2H35 C3S"; $resn_exch{"H261 C3S"}="1H26 C3S"; $resn_exch{"H262 C3S"}="2H26 C3S";
$resn_exch{"H401 C3S"}="1H40 C3S"; $resn_exch{"H402 C3S"}="2H40 C3S"; $resn_exch{"H403 C3S"}="3H40 C3S";
$resn_exch{"H321 C3S"}="1H32 C3S"; $resn_exch{"H322 C3S"}="2H32 C3S"; $resn_exch{"H151 C3S"}="1H15 C3S";
$resn_exch{"H152 C3S"}="2H15 C3S"; $resn_exch{"H231 C3S"}="1H23 C3S"; $resn_exch{"H232 C3S"}="2H23 C3S";
$resn_exch{"H441 C3S"}="1H44 C3S"; $resn_exch{"H442 C3S"}="2H44 C3S"; $resn_exch{"H443 C3S"}="3H44 C3S";
$resn_exch{" H91 C3S"}="1H9  C3S"; $resn_exch{" H92 C3S"}="2H9  C3S"; $resn_exch{" H11 C3S"}="1H1  C3S";
$resn_exch{" H12 C3S"}="2H1  C3S"; $resn_exch{" H41 C3S"}="1H4  C3S"; $resn_exch{" H42 C3S"}="2H4  C3S";
$resn_exch{" H51 C3X"}="1H5  C3X"; $resn_exch{" H52 C3X"}="2H5  C3X"; $resn_exch{"H1'1 C3X"}="1H1' C3X";
$resn_exch{"H1'2 C3X"}="2H1' C3X"; $resn_exch{"H3'1 C3X"}="1H3' C3X"; $resn_exch{"H3'2 C3X"}="2H3' C3X";
$resn_exch{" HN1 C3Y"}="1HN  C3Y"; $resn_exch{" HN2 C3Y"}="2HN  C3Y"; $resn_exch{"HBC1 C3Y"}="1HBC C3Y";
$resn_exch{"HBC2 C3Y"}="2HBC C3Y"; $resn_exch{"H1C1 C3Y"}="1H1C C3Y"; $resn_exch{"H1C2 C3Y"}="2H1C C3Y";
$resn_exch{"H1C3 C3Y"}="3H1C C3Y"; $resn_exch{"H1C1 C40"}="1H1C C40"; $resn_exch{"H1C2 C40"}="2H1C C40";
$resn_exch{"H1C3 C40"}="3H1C C40"; $resn_exch{"H3C1 C40"}="1H3C C40"; $resn_exch{"H3C2 C40"}="2H3C C40";
$resn_exch{"H4C1 C40"}="1H4C C40"; $resn_exch{"H4C2 C40"}="2H4C C40"; $resn_exch{"H5C1 C40"}="1H5C C40";
$resn_exch{"H5C2 C40"}="2H5C C40"; $resn_exch{"H161 C40"}="1H16 C40"; $resn_exch{"H162 C40"}="2H16 C40";
$resn_exch{"H181 C40"}="1H18 C40"; $resn_exch{"H182 C40"}="2H18 C40"; $resn_exch{"H183 C40"}="3H18 C40";
$resn_exch{"H191 C40"}="1H19 C40"; $resn_exch{"H192 C40"}="2H19 C40"; $resn_exch{"H211 C40"}="1H21 C40";
$resn_exch{"H212 C40"}="2H21 C40"; $resn_exch{"H241 C40"}="1H24 C40"; $resn_exch{"H242 C40"}="2H24 C40";
$resn_exch{"H261 C40"}="1H26 C40"; $resn_exch{"H262 C40"}="2H26 C40"; $resn_exch{"H263 C40"}="3H26 C40";
$resn_exch{"H301 C40"}="1H30 C40"; $resn_exch{"H302 C40"}="2H30 C40"; $resn_exch{"H311 C40"}="1H31 C40";
$resn_exch{"H312 C40"}="2H31 C40"; $resn_exch{"H321 C40"}="1H32 C40"; $resn_exch{"H322 C40"}="2H32 C40";
$resn_exch{"H331 C40"}="1H33 C40"; $resn_exch{"H332 C40"}="2H33 C40"; $resn_exch{"H333 C40"}="3H33 C40";
$resn_exch{"H1C1 C41"}="1H1C C41"; $resn_exch{"H1C2 C41"}="2H1C C41"; $resn_exch{"H1C3 C41"}="3H1C C41";
$resn_exch{"H3C1 C41"}="1H3C C41"; $resn_exch{"H3C2 C41"}="2H3C C41"; $resn_exch{"H4C1 C41"}="1H4C C41";
$resn_exch{"H4C2 C41"}="2H4C C41"; $resn_exch{"H5C1 C41"}="1H5C C41"; $resn_exch{"H5C2 C41"}="2H5C C41";
$resn_exch{"H151 C41"}="1H15 C41"; $resn_exch{"H152 C41"}="2H15 C41"; $resn_exch{"H141 C41"}="1H14 C41";
$resn_exch{"H142 C41"}="2H14 C41"; $resn_exch{"H143 C41"}="3H14 C41"; $resn_exch{"H201 C41"}="1H20 C41";
$resn_exch{"H202 C41"}="2H20 C41"; $resn_exch{"H181 C41"}="1H18 C41"; $resn_exch{"H182 C41"}="2H18 C41";
$resn_exch{"H183 C41"}="3H18 C41"; $resn_exch{"H191 C41"}="1H19 C41"; $resn_exch{"H192 C41"}="2H19 C41";
$resn_exch{"H193 C41"}="3H19 C41"; $resn_exch{"H221 C41"}="1H22 C41"; $resn_exch{"H222 C41"}="2H22 C41";
$resn_exch{"H251 C41"}="1H25 C41"; $resn_exch{"H252 C41"}="2H25 C41"; $resn_exch{"H281 C41"}="1H28 C41";
$resn_exch{"H282 C41"}="2H28 C41"; $resn_exch{"H283 C41"}="3H28 C41"; $resn_exch{"H291 C41"}="1H29 C41";
$resn_exch{"H292 C41"}="2H29 C41"; $resn_exch{"H293 C41"}="3H29 C41"; $resn_exch{"H331 C41"}="1H33 C41";
$resn_exch{"H332 C41"}="2H33 C41"; $resn_exch{"H351 C41"}="1H35 C41"; $resn_exch{"H352 C41"}="2H35 C41";
$resn_exch{"H353 C41"}="3H35 C41"; $resn_exch{"H361 C41"}="1H36 C41"; $resn_exch{"H362 C41"}="2H36 C41";
$resn_exch{"H363 C41"}="3H36 C41"; $resn_exch{"H391 C41"}="1H39 C41"; $resn_exch{"H392 C41"}="2H39 C41";
$resn_exch{"H4N1 C42"}="1H4N C42"; $resn_exch{"H4N2 C42"}="2H4N C42"; $resn_exch{" H1' C42"}=" H1* C42";
$resn_exch{"H2'1 C42"}="1H2* C42"; $resn_exch{"H2'2 C42"}="2H2* C42"; $resn_exch{" H3' C42"}=" H3* C42";
$resn_exch{" H4' C42"}=" H4* C42"; $resn_exch{"H5'1 C42"}="1H5* C42"; $resn_exch{"H5'2 C42"}="2H5* C42";
$resn_exch{" HN1 C42"}="1HN  C42"; $resn_exch{" HN2 C42"}="2HN  C42"; $resn_exch{"H5'1 C43"}="1H5* C43";
$resn_exch{"H5'2 C43"}="2H5* C43"; $resn_exch{" H4' C43"}=" H4* C43"; $resn_exch{" H3' C43"}=" H3* C43";
$resn_exch{" H2' C43"}=" H2* C43"; $resn_exch{" H1' C43"}=" H1* C43"; $resn_exch{"H4N1 C43"}="1H4N C43";
$resn_exch{"H4N2 C43"}="2H4N C43"; $resn_exch{"HA'1 C43"}="1HA* C43"; $resn_exch{"HA'2 C43"}="2HA* C43";
$resn_exch{"HB'1 C43"}="1HB* C43"; $resn_exch{"HB'2 C43"}="2HB* C43"; $resn_exch{"HD'1 C43"}="1HD* C43";
$resn_exch{"HD'2 C43"}="2HD* C43"; $resn_exch{"HD'3 C43"}="3HD* C43"; $resn_exch{"H5C1 C44"}="1H5C C44";
$resn_exch{"H5C2 C44"}="2H5C C44"; $resn_exch{"H6C1 C44"}="1H6C C44"; $resn_exch{"H6C2 C44"}="2H6C C44";
$resn_exch{"H331 C44"}="1H33 C44"; $resn_exch{"H332 C44"}="2H33 C44"; $resn_exch{"H341 C44"}="1H34 C44";
$resn_exch{"H342 C44"}="2H34 C44"; $resn_exch{"H343 C44"}="3H34 C44"; $resn_exch{"H421 C44"}="1H42 C44";
$resn_exch{"H422 C44"}="2H42 C44"; $resn_exch{"H431 C44"}="1H43 C44"; $resn_exch{"H432 C44"}="2H43 C44";
$resn_exch{"H441 C44"}="1H44 C44"; $resn_exch{"H442 C44"}="2H44 C44"; $resn_exch{"H451 C44"}="1H45 C44";
$resn_exch{"H452 C44"}="2H45 C44"; $resn_exch{"H561 C44"}="1H56 C44"; $resn_exch{"H562 C44"}="2H56 C44";
$resn_exch{"H571 C44"}="1H57 C44"; $resn_exch{"H572 C44"}="2H57 C44"; $resn_exch{"H581 C44"}="1H58 C44";
$resn_exch{"H582 C44"}="2H58 C44"; $resn_exch{" H1' C45"}=" H1* C45"; $resn_exch{"H2'1 C45"}="1H2* C45";
$resn_exch{"H2'2 C45"}="2H2* C45"; $resn_exch{"HM21 C45"}="1HM2 C45"; $resn_exch{"HM22 C45"}="2HM2 C45";
$resn_exch{"HM23 C45"}="3HM2 C45"; $resn_exch{" H3' C45"}=" H3* C45"; $resn_exch{" H4' C45"}=" H4* C45";
$resn_exch{"H5'1 C45"}="1H5* C45"; $resn_exch{"H5'2 C45"}="2H5* C45"; $resn_exch{"H5'1 C46"}="1H5* C46";
$resn_exch{"H5'2 C46"}="2H5* C46"; $resn_exch{" H4' C46"}=" H4* C46"; $resn_exch{" H3' C46"}=" H3* C46";
$resn_exch{"H2'1 C46"}="1H2* C46"; $resn_exch{"H2'2 C46"}="2H2* C46"; $resn_exch{" H1' C46"}=" H1* C46";
$resn_exch{"HM51 C46"}="1HM5 C46"; $resn_exch{"HM52 C46"}="2HM5 C46"; $resn_exch{"H7C1 C46"}="1H7C C46";
$resn_exch{"H7C2 C46"}="2H7C C46"; $resn_exch{"H101 C48"}="1H10 C48"; $resn_exch{"H102 C48"}="2H10 C48";
$resn_exch{"H141 C48"}="1H14 C48"; $resn_exch{"H142 C48"}="2H14 C48"; $resn_exch{"H221 C48"}="1H22 C48";
$resn_exch{"H222 C48"}="2H22 C48"; $resn_exch{"H223 C48"}="3H22 C48"; $resn_exch{"H231 C48"}="1H23 C48";
$resn_exch{"H232 C48"}="2H23 C48"; $resn_exch{"H233 C48"}="3H23 C48"; $resn_exch{"H241 C48"}="1H24 C48";
$resn_exch{"H242 C48"}="2H24 C48"; $resn_exch{"H251 C48"}="1H25 C48"; $resn_exch{"H252 C48"}="2H25 C48";
$resn_exch{"H5'1 C49"}="1H5* C49"; $resn_exch{"H5'2 C49"}="2H5* C49"; $resn_exch{" H4' C49"}=" H4* C49";
$resn_exch{" H3' C49"}=" H3* C49"; $resn_exch{"H2'1 C49"}="1H2* C49"; $resn_exch{"H2'2 C49"}="2H2* C49";
$resn_exch{" H1' C49"}=" H1* C49"; $resn_exch{"HM51 C49"}="1HM5 C49"; $resn_exch{"HM52 C49"}="2HM5 C49";
$resn_exch{"HM53 C49"}="3HM5 C49"; $resn_exch{"HN11 C4A"}="1HN1 C4A"; $resn_exch{"HN12 C4A"}="2HN1 C4A";
$resn_exch{"H21A C4A"}="1H2  C4A"; $resn_exch{"H22A C4A"}="2H2  C4A"; $resn_exch{" H41 C4A"}="1H4  C4A";
$resn_exch{" H42 C4A"}="2H4  C4A"; $resn_exch{" H51 C4A"}="1H5  C4A"; $resn_exch{" H52 C4A"}="2H5  C4A";
$resn_exch{" H31 C4A"}="1H3  C4A"; $resn_exch{" H32 C4A"}="2H3  C4A"; $resn_exch{"H111 C4A"}="1H11 C4A";
$resn_exch{"H112 C4A"}="2H11 C4A"; $resn_exch{"H121 C4A"}="1H12 C4A"; $resn_exch{"H122 C4A"}="2H12 C4A";
$resn_exch{"H123 C4A"}="3H12 C4A"; $resn_exch{"H191 C4A"}="1H19 C4A"; $resn_exch{"H192 C4A"}="2H19 C4A";
$resn_exch{"H301 C4A"}="1H30 C4A"; $resn_exch{"H302 C4A"}="2H30 C4A"; $resn_exch{"H331 C4A"}="1H33 C4A";
$resn_exch{"H332 C4A"}="2H33 C4A"; $resn_exch{" H51 C4H"}="1H5  C4H"; $resn_exch{" H52 C4H"}="2H5  C4H";
$resn_exch{" H53 C4H"}="3H5  C4H"; $resn_exch{" H61 C4H"}="1H6  C4H"; $resn_exch{" H62 C4H"}="2H6  C4H";
$resn_exch{" H63 C4H"}="3H6  C4H"; $resn_exch{"H161 C4H"}="1H16 C4H"; $resn_exch{"H162 C4H"}="2H16 C4H";
$resn_exch{"H171 C4H"}="1H17 C4H"; $resn_exch{"H172 C4H"}="2H17 C4H"; $resn_exch{"H181 C4H"}="1H18 C4H";
$resn_exch{"H182 C4H"}="2H18 C4H"; $resn_exch{"H191 C4H"}="1H19 C4H"; $resn_exch{"H192 C4H"}="2H19 C4H";
$resn_exch{"H231 C4H"}="1H23 C4H"; $resn_exch{"H232 C4H"}="2H23 C4H"; $resn_exch{"H241 C4H"}="1H24 C4H";
$resn_exch{"H242 C4H"}="2H24 C4H"; $resn_exch{"H1C1 C4M"}="1H1C C4M"; $resn_exch{"H1C2 C4M"}="2H1C C4M";
$resn_exch{"H1C3 C4M"}="3H1C C4M"; $resn_exch{"H6C1 C4M"}="1H6C C4M"; $resn_exch{"H6C2 C4M"}="2H6C C4M";
$resn_exch{"H6C3 C4M"}="3H6C C4M"; $resn_exch{"H9C1 C4M"}="1H9C C4M"; $resn_exch{"H9C2 C4M"}="2H9C C4M";
$resn_exch{"H9C3 C4M"}="3H9C C4M"; $resn_exch{"H111 C4M"}="1H11 C4M"; $resn_exch{"H112 C4M"}="2H11 C4M";
$resn_exch{"H113 C4M"}="3H11 C4M"; $resn_exch{"H161 C4M"}="1H16 C4M"; $resn_exch{"H162 C4M"}="2H16 C4M";
$resn_exch{"H201 C4M"}="1H20 C4M"; $resn_exch{"H202 C4M"}="2H20 C4M"; $resn_exch{"H211 C4M"}="1H21 C4M";
$resn_exch{"H212 C4M"}="2H21 C4M"; $resn_exch{" H21 C4P"}="1H2  C4P"; $resn_exch{" H22 C4P"}="2H2  C4P";
$resn_exch{" H31 C4P"}="1H3  C4P"; $resn_exch{" H32 C4P"}="2H3  C4P"; $resn_exch{"H51A C4P"}="1H5  C4P";
$resn_exch{"H52A C4P"}="2H5  C4P"; $resn_exch{" H61 C4P"}="1H6  C4P"; $resn_exch{" H62 C4P"}="2H6  C4P";
$resn_exch{"H581 C4P"}="1H58 C4P"; $resn_exch{"H582 C4P"}="2H58 C4P"; $resn_exch{" H11 C4P"}="1H1  C4P";
$resn_exch{" H12 C4P"}="2H1  C4P"; $resn_exch{"H471 C4P"}="1H47 C4P"; $resn_exch{"H472 C4P"}="2H47 C4P";
$resn_exch{" H51 C4X"}="1H5  C4X"; $resn_exch{" H52 C4X"}="2H5  C4X"; $resn_exch{"H1'1 C4X"}="1H1' C4X";
$resn_exch{"H1'2 C4X"}="2H1' C4X"; $resn_exch{"H2'1 C4X"}="1H2' C4X"; $resn_exch{"H2'2 C4X"}="2H2' C4X";
$resn_exch{"H4'1 C4X"}="1H4' C4X"; $resn_exch{"H4'2 C4X"}="2H4' C4X"; $resn_exch{"H6C1 C53"}="1H6C C53";
$resn_exch{"H3N1 C53"}="1H3N C53"; $resn_exch{" HN1 C5A"}="1HN  C5A"; $resn_exch{" HN2 C5A"}="2HN  C5A";
$resn_exch{"HB11 C5A"}="1HB1 C5A"; $resn_exch{"HB12 C5A"}="2HB1 C5A"; $resn_exch{"HB21 C5A"}="1HB2 C5A";
$resn_exch{"HB22 C5A"}="2HB2 C5A"; $resn_exch{"HG21 C5A"}="1HG2 C5A"; $resn_exch{"HG22 C5A"}="2HG2 C5A";
$resn_exch{" HN1 C5B"}="1HN  C5B"; $resn_exch{" HN2 C5B"}="2HN  C5B"; $resn_exch{"HB11 C5B"}="1HB1 C5B";
$resn_exch{"HB12 C5B"}="2HB1 C5B"; $resn_exch{"HB21 C5B"}="1HB2 C5B"; $resn_exch{"HB22 C5B"}="2HB2 C5B";
$resn_exch{"HG21 C5B"}="1HG2 C5B"; $resn_exch{"HG22 C5B"}="2HG2 C5B"; $resn_exch{" H   C5C"}="1HN  C5C";
$resn_exch{" H2  C5C"}="2HN  C5C"; $resn_exch{" HB2 C5C"}="1HB  C5C"; $resn_exch{" HB3 C5C"}="2HB  C5C";
$resn_exch{" H21 C5C"}="1H2  C5C"; $resn_exch{" H22 C5C"}="2H2  C5C"; $resn_exch{" H31 C5C"}="1H3  C5C";
$resn_exch{" H32 C5C"}="2H3  C5C"; $resn_exch{" H41 C5C"}="1H4  C5C"; $resn_exch{" H42 C5C"}="2H4  C5C";
$resn_exch{" H51 C5C"}="1H5  C5C"; $resn_exch{" H52 C5C"}="2H5  C5C"; $resn_exch{" H41 C5G"}="1H4  C5G";
$resn_exch{" H42 C5G"}="2H4  C5G"; $resn_exch{" H1D C5G"}=" H1* C5G"; $resn_exch{" H2D C5G"}=" H2* C5G";
$resn_exch{"HO'2 C5G"}="2HO* C5G"; $resn_exch{" H3D C5G"}=" H3* C5G"; $resn_exch{" H4D C5G"}=" H4* C5G";
$resn_exch{"H5'1 C5G"}="1H5* C5G"; $resn_exch{"H5'2 C5G"}="2H5* C5G"; $resn_exch{" HOD C5G"}="2HO' C5G";
$resn_exch{"HO'3 C5G"}="3HO' C5G"; $resn_exch{"HO'4 C5G"}="4HO' C5G"; $resn_exch{"H6'1 C5G"}="1H6' C5G";
$resn_exch{"H6'2 C5G"}="2H6' C5G"; $resn_exch{"HO'6 C5G"}="6HO' C5G"; $resn_exch{"H1C1 C5M"}="1H1C C5M";
$resn_exch{"H1C2 C5M"}="2H1C C5M"; $resn_exch{"H1C3 C5M"}="3H1C C5M"; $resn_exch{"H6C1 C5M"}="1H6C C5M";
$resn_exch{"H6C2 C5M"}="2H6C C5M"; $resn_exch{"H6C3 C5M"}="3H6C C5M"; $resn_exch{"H9C1 C5M"}="1H9C C5M";
$resn_exch{"H9C2 C5M"}="2H9C C5M"; $resn_exch{"H9C3 C5M"}="3H9C C5M"; $resn_exch{"H111 C5M"}="1H11 C5M";
$resn_exch{"H112 C5M"}="2H11 C5M"; $resn_exch{"H113 C5M"}="3H11 C5M"; $resn_exch{"H161 C5M"}="1H16 C5M";
$resn_exch{"H162 C5M"}="2H16 C5M"; $resn_exch{"H201 C5M"}="1H20 C5M"; $resn_exch{"H202 C5M"}="2H20 C5M";
$resn_exch{"H281 C5M"}="1H28 C5M"; $resn_exch{"H282 C5M"}="2H28 C5M"; $resn_exch{"HOP3 C5P"}="3HOP C5P";
$resn_exch{"HOP2 C5P"}="2HOP C5P"; $resn_exch{"H5'1 C5P"}="1H5* C5P"; $resn_exch{"H5'2 C5P"}="2H5* C5P";
$resn_exch{" H4' C5P"}=" H4* C5P"; $resn_exch{" H3' C5P"}=" H3* C5P"; $resn_exch{"HO3' C5P"}="*HO3 C5P";
$resn_exch{"H2'1 C5P"}="1H2* C5P"; $resn_exch{"HO2' C5P"}="*HO2 C5P"; $resn_exch{" H1' C5P"}=" H1* C5P";
$resn_exch{"HN41 C5P"}="1HN4 C5P"; $resn_exch{"HN42 C5P"}="2HN4 C5P"; $resn_exch{" H51 C5X"}="1H5  C5X";
$resn_exch{" H52 C5X"}="2H5  C5X"; $resn_exch{"H1'1 C5X"}="1H1' C5X"; $resn_exch{"H1'2 C5X"}="2H1' C5X";
$resn_exch{"H2'1 C5X"}="1H2' C5X"; $resn_exch{"H2'2 C5X"}="2H2' C5X"; $resn_exch{"H3'1 C5X"}="1H3' C5X";
$resn_exch{"H3'2 C5X"}="2H3' C5X"; $resn_exch{"H5'1 C5X"}="1H5' C5X"; $resn_exch{"H5'2 C5X"}="2H5' C5X";
$resn_exch{"HS21 C60"}="1HS2 C60"; $resn_exch{"HS22 C60"}="2HS2 C60"; $resn_exch{"HS23 C60"}="3HS2 C60";
$resn_exch{"HS31 C60"}="1HS3 C60"; $resn_exch{"HS32 C60"}="2HS3 C60"; $resn_exch{"HS33 C60"}="3HS3 C60";
$resn_exch{"HS41 C60"}="1HS4 C60"; $resn_exch{"HS42 C60"}="2HS4 C60"; $resn_exch{"HS43 C60"}="3HS4 C60";
$resn_exch{"HNF1 C60"}="1HNF C60"; $resn_exch{"HNF2 C60"}="2HNF C60"; $resn_exch{"HBF1 C60"}="1HBF C60";
$resn_exch{"HBF2 C60"}="2HBF C60"; $resn_exch{"HBH1 C60"}="1HBH C60"; $resn_exch{"HBH2 C60"}="2HBH C60";
$resn_exch{"HNH1 C60"}="1HNH C60"; $resn_exch{"HBC1 C60"}="1HBC C60"; $resn_exch{"HBC2 C60"}="2HBC C60";
$resn_exch{"H2C1 C60"}="1H2C C60"; $resn_exch{"H2C2 C60"}="2H2C C60"; $resn_exch{"H3C1 C60"}="1H3C C60";
$resn_exch{"H3C2 C60"}="2H3C C60"; $resn_exch{"H4C1 C60"}="1H4C C60"; $resn_exch{"H4C2 C60"}="2H4C C60";
$resn_exch{"H5C1 C60"}="1H5C C60"; $resn_exch{"H5C2 C60"}="2H5C C60"; $resn_exch{"H6C1 C60"}="1H6C C60";
$resn_exch{"H6C2 C60"}="2H6C C60"; $resn_exch{"HNV1 C60"}="1HNV C60"; $resn_exch{"HNV2 C60"}="2HNV C60";
$resn_exch{"H1V1 C60"}="1H1V C60"; $resn_exch{"H1V2 C60"}="2H1V C60"; $resn_exch{"H1V3 C60"}="3H1V C60";
$resn_exch{"H2V1 C60"}="1H2V C60"; $resn_exch{"H2V2 C60"}="2H2V C60"; $resn_exch{"H2V3 C60"}="3H2V C60";
$resn_exch{"H1T1 C60"}="1H1T C60"; $resn_exch{"H1T2 C60"}="2H1T C60"; $resn_exch{"H2T1 C60"}="1H2T C60";
$resn_exch{"H2T2 C60"}="2H2T C60"; $resn_exch{"H3T1 C60"}="1H3T C60"; $resn_exch{"H3T2 C60"}="2H3T C60";
$resn_exch{"H4T1 C60"}="1H4T C60"; $resn_exch{"H4T2 C60"}="2H4T C60"; $resn_exch{"H4T3 C60"}="3H4T C60";
$resn_exch{"H2C1 C61"}="1H2C C61"; $resn_exch{"H2C2 C61"}="2H2C C61"; $resn_exch{"H2C3 C61"}="3H2C C61";
$resn_exch{"H4C1 C61"}="1H4C C61"; $resn_exch{"H4C2 C61"}="2H4C C61"; $resn_exch{"H5C1 C61"}="1H5C C61";
$resn_exch{"H5C2 C61"}="2H5C C61"; $resn_exch{"H6C1 C61"}="1H6C C61"; $resn_exch{"H6C2 C61"}="2H6C C61";
$resn_exch{"H9C1 C61"}="1H9C C61"; $resn_exch{"H9C2 C61"}="2H9C C61"; $resn_exch{"H9C3 C61"}="3H9C C61";
$resn_exch{"H161 C61"}="1H16 C61"; $resn_exch{"H162 C61"}="2H16 C61"; $resn_exch{"H211 C61"}="1H21 C61";
$resn_exch{"H212 C61"}="2H21 C61"; $resn_exch{"H191 C61"}="1H19 C61"; $resn_exch{"H192 C61"}="2H19 C61";
$resn_exch{"H193 C61"}="3H19 C61"; $resn_exch{"H201 C61"}="1H20 C61"; $resn_exch{"H202 C61"}="2H20 C61";
$resn_exch{"H203 C61"}="3H20 C61"; $resn_exch{"H231 C61"}="1H23 C61"; $resn_exch{"H232 C61"}="2H23 C61";
$resn_exch{"H261 C61"}="1H26 C61"; $resn_exch{"H262 C61"}="2H26 C61"; $resn_exch{"H281 C61"}="1H28 C61";
$resn_exch{"H282 C61"}="2H28 C61"; $resn_exch{"H283 C61"}="3H28 C61"; $resn_exch{"H321 C61"}="1H32 C61";
$resn_exch{"H322 C61"}="2H32 C61"; $resn_exch{"H331 C61"}="1H33 C61"; $resn_exch{"H332 C61"}="2H33 C61";
$resn_exch{"H341 C61"}="1H34 C61"; $resn_exch{"H342 C61"}="2H34 C61"; $resn_exch{"H351 C61"}="1H35 C61";
$resn_exch{"H352 C61"}="2H35 C61"; $resn_exch{"H353 C61"}="3H35 C61"; $resn_exch{"H6N1 C62"}="1H6N C62";
$resn_exch{"H6N2 C62"}="2H6N C62"; $resn_exch{"H6C1 C62"}="1H6C C62"; $resn_exch{"H8'1 C66"}="1H8' C66";
$resn_exch{"H8'2 C66"}="2H8' C66"; $resn_exch{"H5'1 C66"}="1H5' C66"; $resn_exch{"H3'1 C66"}="1H3' C66";
$resn_exch{"H3'2 C66"}="2H3' C66"; $resn_exch{"H2'1 C66"}="1H2' C66"; $resn_exch{"H2'2 C66"}="2H2' C66";
$resn_exch{"H1'1 C66"}="1H1' C66"; $resn_exch{"H1'2 C66"}="2H1' C66"; $resn_exch{"HN41 C66"}="1HN4 C66";
$resn_exch{"HN42 C66"}="2HN4 C66"; $resn_exch{" HB1 C66"}="1HB  C66"; $resn_exch{" HB2 C66"}="2HB  C66";
$resn_exch{" HG1 C66"}="1HG  C66"; $resn_exch{" HG2 C66"}="2HG  C66"; $resn_exch{" HD1 C66"}="1HD  C66";
$resn_exch{" HD2 C66"}="2HD  C66"; $resn_exch{" HE1 C66"}="1HE  C66"; $resn_exch{" HE2 C66"}="2HE  C66";
$resn_exch{" HZ1 C66"}="1HZ  C66"; $resn_exch{" HZ2 C66"}="2HZ  C66"; $resn_exch{" H   C6C"}="1HN  C6C";
$resn_exch{" H2  C6C"}="2HN  C6C"; $resn_exch{" HB2 C6C"}="1HB  C6C"; $resn_exch{" HB3 C6C"}="2HB  C6C";
$resn_exch{" H21 C6C"}="1H2  C6C"; $resn_exch{" H22 C6C"}="2H2  C6C"; $resn_exch{" H31 C6C"}="1H3  C6C";
$resn_exch{" H32 C6C"}="2H3  C6C"; $resn_exch{" H41 C6C"}="1H4  C6C"; $resn_exch{" H42 C6C"}="2H4  C6C";
$resn_exch{" H51 C6C"}="1H5  C6C"; $resn_exch{" H52 C6C"}="2H5  C6C"; $resn_exch{" H61 C6C"}="1H6  C6C";
$resn_exch{" H62 C6C"}="2H6  C6C"; $resn_exch{"H5M1 C6P"}="1H5M C6P"; $resn_exch{"H5M2 C6P"}="2H5M C6P";
$resn_exch{"H2A1 C6P"}="1H2A C6P"; $resn_exch{"H2A2 C6P"}="2H2A C6P"; $resn_exch{"H2A3 C6P"}="3H2A C6P";
$resn_exch{"H4A1 C6P"}="1H4A C6P"; $resn_exch{"H4A2 C6P"}="2H4A C6P"; $resn_exch{" HB1 C6P"}="1HB  C6P";
$resn_exch{" HB2 C6P"}="2HB  C6P"; $resn_exch{" H21 C71"}="1H2  C71"; $resn_exch{"H22A C71"}="2H2  C71";
$resn_exch{" H61 C71"}="1H6  C71"; $resn_exch{" H62 C71"}="2H6  C71"; $resn_exch{" H63 C71"}="3H6  C71";
$resn_exch{"H171 C71"}="1H17 C71"; $resn_exch{"H172 C71"}="2H17 C71"; $resn_exch{"H181 C71"}="1H18 C71";
$resn_exch{"H182 C71"}="2H18 C71"; $resn_exch{"H201 C71"}="1H20 C71"; $resn_exch{"H202 C71"}="2H20 C71";
$resn_exch{"H211 C71"}="1H21 C71"; $resn_exch{"H212 C71"}="2H21 C71"; $resn_exch{"H271 C71"}="1H27 C71";
$resn_exch{"H272 C71"}="2H27 C71"; $resn_exch{"H281 C71"}="1H28 C71"; $resn_exch{"H282 C71"}="2H28 C71";
$resn_exch{"H291 C71"}="1H29 C71"; $resn_exch{"H292 C71"}="2H29 C71"; $resn_exch{"H301 C71"}="1H30 C71";
$resn_exch{"H302 C71"}="2H30 C71"; $resn_exch{"H311 C71"}="1H31 C71"; $resn_exch{"H312 C71"}="2H31 C71";
$resn_exch{"H313 C71"}="3H31 C71"; $resn_exch{"H341 C71"}="1H34 C71"; $resn_exch{"H342 C71"}="2H34 C71";
$resn_exch{"H161 C71"}="1H16 C71"; $resn_exch{"H162 C71"}="2H16 C71"; $resn_exch{"H611 C75"}="1H61 C75";
$resn_exch{"H612 C75"}="2H61 C75"; $resn_exch{"H613 C75"}="3H61 C75"; $resn_exch{"HN11 C78"}="1HN1 C78";
$resn_exch{"HN12 C78"}="2HN1 C78"; $resn_exch{"HC31 C78"}="1HC3 C78"; $resn_exch{"HC41 C78"}="1HC4 C78";
$resn_exch{"HC51 C78"}="1HC5 C78"; $resn_exch{"HC61 C78"}="1HC6 C78"; $resn_exch{"HC81 C78"}="1HC8 C78";
$resn_exch{"HC82 C78"}="2HC8 C78"; $resn_exch{"H121 C78"}="1H12 C78"; $resn_exch{"H131 C78"}="1H13 C78";
$resn_exch{"H141 C78"}="1H14 C78"; $resn_exch{"H142 C78"}="2H14 C78"; $resn_exch{"H161 C78"}="1H16 C78";
$resn_exch{"H171 C78"}="1H17 C78"; $resn_exch{"H181 C78"}="1H18 C78"; $resn_exch{"H191 C78"}="1H19 C78";
$resn_exch{"H251 C78"}="1H25 C78"; $resn_exch{"H271 C78"}="1H27 C78"; $resn_exch{"H281 C78"}="1H28 C78";
$resn_exch{"H301 C78"}="1H30 C78"; $resn_exch{"H302 C78"}="2H30 C78"; $resn_exch{"H311 C78"}="1H31 C78";
$resn_exch{"H312 C78"}="2H31 C78"; $resn_exch{"H321 C78"}="1H32 C78"; $resn_exch{"H322 C78"}="2H32 C78";
$resn_exch{"H331 C78"}="1H33 C78"; $resn_exch{"H332 C78"}="2H33 C78"; $resn_exch{"H341 C78"}="1H34 C78";
$resn_exch{"H342 C78"}="2H34 C78"; $resn_exch{"H371 C78"}="1H37 C78"; $resn_exch{"H381 C78"}="1H38 C78";
$resn_exch{"H391 C78"}="1H39 C78"; $resn_exch{"H392 C78"}="2H39 C78"; $resn_exch{"H401 C78"}="1H40 C78";
$resn_exch{"H402 C78"}="2H40 C78"; $resn_exch{"H411 C78"}="1H41 C78"; $resn_exch{"H412 C78"}="2H41 C78";
$resn_exch{"H421 C78"}="1H42 C78"; $resn_exch{"H422 C78"}="2H42 C78"; $resn_exch{"H431 C78"}="1H43 C78";
$resn_exch{"H451 C78"}="1H45 C78"; $resn_exch{"H452 C78"}="2H45 C78"; $resn_exch{"H7C1 C80"}="1H7C C80";
$resn_exch{"H7C2 C80"}="2H7C C80"; $resn_exch{"H201 C80"}="1H20 C80"; $resn_exch{"H202 C80"}="2H20 C80";
$resn_exch{"H221 C80"}="1H22 C80"; $resn_exch{"H222 C80"}="2H22 C80"; $resn_exch{"H223 C80"}="3H22 C80";
$resn_exch{"H231 C80"}="1H23 C80"; $resn_exch{"H232 C80"}="2H23 C80"; $resn_exch{"H251 C80"}="1H25 C80";
$resn_exch{"H252 C80"}="2H25 C80"; $resn_exch{"H281 C80"}="1H28 C80"; $resn_exch{"H282 C80"}="2H28 C80";
$resn_exch{"H301 C80"}="1H30 C80"; $resn_exch{"H302 C80"}="2H30 C80"; $resn_exch{"H303 C80"}="3H30 C80";
$resn_exch{"H341 C80"}="1H34 C80"; $resn_exch{"H342 C80"}="2H34 C80"; $resn_exch{"H351 C80"}="1H35 C80";
$resn_exch{"H352 C80"}="2H35 C80"; $resn_exch{"H361 C80"}="1H36 C80"; $resn_exch{"H362 C80"}="2H36 C80";
$resn_exch{"H371 C80"}="1H37 C80"; $resn_exch{"H372 C80"}="2H37 C80"; $resn_exch{"H373 C80"}="3H37 C80";
$resn_exch{"H6N1 C85"}="1H6N C85"; $resn_exch{"H6N2 C85"}="2H6N C85"; $resn_exch{"H3N1 C85"}="1H3N C85";
$resn_exch{"H3N2 C85"}="2H3N C85"; $resn_exch{"H1N1 C8C"}="1H1N C8C"; $resn_exch{"H1N2 C8C"}="2H1N C8C";
$resn_exch{"H7C1 C8C"}="1H7C C8C"; $resn_exch{"H7C2 C8C"}="2H7C C8C"; $resn_exch{"H8C1 C8C"}="1H8C C8C";
$resn_exch{"H8C2 C8C"}="2H8C C8C"; $resn_exch{" H11 C8E"}="1H1  C8E"; $resn_exch{" H12 C8E"}="2H1  C8E";
$resn_exch{" H13 C8E"}="3H1  C8E"; $resn_exch{" H21 C8E"}="1H2  C8E"; $resn_exch{" H22 C8E"}="2H2  C8E";
$resn_exch{" H31 C8E"}="1H3  C8E"; $resn_exch{" H32 C8E"}="2H3  C8E"; $resn_exch{" H41 C8E"}="1H4  C8E";
$resn_exch{" H42 C8E"}="2H4  C8E"; $resn_exch{" H51 C8E"}="1H5  C8E"; $resn_exch{" H52 C8E"}="2H5  C8E";
$resn_exch{" H61 C8E"}="1H6  C8E"; $resn_exch{" H62 C8E"}="2H6  C8E"; $resn_exch{" H71 C8E"}="1H7  C8E";
$resn_exch{" H72 C8E"}="2H7  C8E"; $resn_exch{" H81 C8E"}="1H8  C8E"; $resn_exch{" H82 C8E"}="2H8  C8E";
$resn_exch{"H101 C8E"}="1H10 C8E"; $resn_exch{"H102 C8E"}="2H10 C8E"; $resn_exch{"H111 C8E"}="1H11 C8E";
$resn_exch{"H112 C8E"}="2H11 C8E"; $resn_exch{"H131 C8E"}="1H13 C8E"; $resn_exch{"H132 C8E"}="2H13 C8E";
$resn_exch{"H141 C8E"}="1H14 C8E"; $resn_exch{"H142 C8E"}="2H14 C8E"; $resn_exch{"H161 C8E"}="1H16 C8E";
$resn_exch{"H162 C8E"}="2H16 C8E"; $resn_exch{"H171 C8E"}="1H17 C8E"; $resn_exch{"H172 C8E"}="2H17 C8E";
$resn_exch{"H191 C8E"}="1H19 C8E"; $resn_exch{"H192 C8E"}="2H19 C8E"; $resn_exch{"H201 C8E"}="1H20 C8E";
$resn_exch{"H202 C8E"}="2H20 C8E"; $resn_exch{" H51 C90"}="1H5  C90"; $resn_exch{" H52 C90"}="2H5  C90";
$resn_exch{"H211 C90"}="1H21 C90"; $resn_exch{"H212 C90"}="2H21 C90"; $resn_exch{"H213 C90"}="3H21 C90";
$resn_exch{"HA11 C90"}="1HA1 C90"; $resn_exch{"HA12 C90"}="2HA1 C90"; $resn_exch{"HA21 C90"}="1HA2 C90";
$resn_exch{"HA22 C90"}="2HA2 C90"; $resn_exch{"HB11 C90"}="1HB1 C90"; $resn_exch{"HB12 C90"}="2HB1 C90";
$resn_exch{"HB21 C90"}="1HB2 C90"; $resn_exch{"HB22 C90"}="2HB2 C90"; $resn_exch{"H6N1 C94"}="1H6N C94";
$resn_exch{"H6N2 C94"}="2H6N C94"; $resn_exch{"H3N1 C94"}="1H3N C94"; $resn_exch{"H3N2 C94"}="2H3N C94";
$resn_exch{"H3N1 C95"}="1H3N C95"; $resn_exch{"H3N2 C95"}="2H3N C95"; $resn_exch{"H6N1 C96"}="1H6N C96";
$resn_exch{"H6N2 C96"}="2H6N C96"; $resn_exch{"H201 C96"}="1H20 C96"; $resn_exch{"H202 C96"}="2H20 C96";
$resn_exch{"H203 C96"}="3H20 C96"; $resn_exch{"H3N1 C96"}="1H3N C96"; $resn_exch{"H3N2 C96"}="2H3N C96";
$resn_exch{"HX51 CA1"}="1HX5 CA1"; $resn_exch{"HX52 CA1"}="2HX5 CA1"; $resn_exch{"H2C1 CA2"}="1H2C CA2";
$resn_exch{"H2C2 CA2"}="2H2C CA2"; $resn_exch{"H6C1 CA2"}="1H6C CA2"; $resn_exch{"H6C2 CA2"}="2H6C CA2";
$resn_exch{"H8C1 CA2"}="1H8C CA2"; $resn_exch{"H8C2 CA2"}="2H8C CA2"; $resn_exch{"H9C1 CA2"}="1H9C CA2";
$resn_exch{"H9C2 CA2"}="2H9C CA2"; $resn_exch{"H101 CA2"}="1H10 CA2"; $resn_exch{"H102 CA2"}="2H10 CA2";
$resn_exch{" H61 CA3"}="1H6  CA3"; $resn_exch{" H62 CA3"}="2H6  CA3"; $resn_exch{"HO'2 CA3"}="2HO' CA3";
$resn_exch{"H5'1 CA3"}="1H5' CA3"; $resn_exch{"H5'2 CA3"}="2H5' CA3"; $resn_exch{"HP91 CA3"}="1HP9 CA3";
$resn_exch{"HP92 CA3"}="2HP9 CA3"; $resn_exch{"HPA1 CA3"}="1HPA CA3"; $resn_exch{"HPA2 CA3"}="2HPA CA3";
$resn_exch{"HPA3 CA3"}="3HPA CA3"; $resn_exch{"HPB1 CA3"}="1HPB CA3"; $resn_exch{"HPB2 CA3"}="2HPB CA3";
$resn_exch{"HPB3 CA3"}="3HPB CA3"; $resn_exch{"HP51 CA3"}="1HP5 CA3"; $resn_exch{"HP52 CA3"}="2HP5 CA3";
$resn_exch{"HP41 CA3"}="1HP4 CA3"; $resn_exch{"HP42 CA3"}="2HP4 CA3"; $resn_exch{"HP21 CA3"}="1HP2 CA3";
$resn_exch{"HP22 CA3"}="2HP2 CA3"; $resn_exch{"HP11 CA3"}="1HP1 CA3"; $resn_exch{"HP12 CA3"}="2HP1 CA3";
$resn_exch{"HT21 CA3"}="1HT2 CA3"; $resn_exch{"HT22 CA3"}="2HT2 CA3"; $resn_exch{"HT10 CA3"}="0HT1 CA3";
$resn_exch{"HNT6 CA3"}="6HNT CA3"; $resn_exch{"HA11 CA3"}="1HA1 CA3"; $resn_exch{"HA12 CA3"}="2HA1 CA3";
$resn_exch{"HT11 CA3"}="1HT1 CA3"; $resn_exch{"HT12 CA3"}="2HT1 CA3"; $resn_exch{"HA31 CA3"}="1HA3 CA3";
$resn_exch{"HA32 CA3"}="2HA3 CA3"; $resn_exch{"HA41 CA3"}="1HA4 CA3"; $resn_exch{"HA42 CA3"}="2HA4 CA3";
$resn_exch{"HA51 CA3"}="1HA5 CA3"; $resn_exch{"HA52 CA3"}="2HA5 CA3"; $resn_exch{"H111 CA4"}="1H11 CA4";
$resn_exch{"H112 CA4"}="2H11 CA4"; $resn_exch{"H121 CA4"}="1H12 CA4"; $resn_exch{"H122 CA4"}="2H12 CA4";
$resn_exch{"H191 CA4"}="1H19 CA4"; $resn_exch{"H192 CA4"}="2H19 CA4"; $resn_exch{"H193 CA4"}="3H19 CA4";
$resn_exch{"H221 CA4"}="1H22 CA4"; $resn_exch{"H222 CA4"}="2H22 CA4"; $resn_exch{"H181 CA4"}="1H18 CA4";
$resn_exch{"H182 CA4"}="2H18 CA4"; $resn_exch{"H183 CA4"}="3H18 CA4"; $resn_exch{"H151 CA4"}="1H15 CA4";
$resn_exch{"H152 CA4"}="2H15 CA4"; $resn_exch{"H161 CA4"}="1H16 CA4"; $resn_exch{"H162 CA4"}="2H16 CA4";
$resn_exch{"H211 CA4"}="1H21 CA4"; $resn_exch{"H212 CA4"}="2H21 CA4"; $resn_exch{"H213 CA4"}="3H21 CA4";
$resn_exch{"H241 CA4"}="1H24 CA4"; $resn_exch{"H242 CA4"}="2H24 CA4"; $resn_exch{"H243 CA4"}="3H24 CA4";
$resn_exch{"HN61 CA5"}="1HN6 CA5"; $resn_exch{"HN62 CA5"}="2HN6 CA5"; $resn_exch{"HO'2 CA5"}="2HO' CA5";
$resn_exch{"H5'1 CA5"}="1H5' CA5"; $resn_exch{"H5'2 CA5"}="2H5' CA5"; $resn_exch{"HP91 CA5"}="1HP9 CA5";
$resn_exch{"HP92 CA5"}="2HP9 CA5"; $resn_exch{"HCA1 CA5"}="1HCA CA5"; $resn_exch{"HCA2 CA5"}="2HCA CA5";
$resn_exch{"HCA3 CA5"}="3HCA CA5"; $resn_exch{"HCB1 CA5"}="1HCB CA5"; $resn_exch{"HCB2 CA5"}="2HCB CA5";
$resn_exch{"HCB3 CA5"}="3HCB CA5"; $resn_exch{"HP51 CA5"}="1HP5 CA5"; $resn_exch{"HP52 CA5"}="2HP5 CA5";
$resn_exch{"HP41 CA5"}="1HP4 CA5"; $resn_exch{"HP42 CA5"}="2HP4 CA5"; $resn_exch{"HP21 CA5"}="1HP2 CA5";
$resn_exch{"HP22 CA5"}="2HP2 CA5"; $resn_exch{"HP11 CA5"}="1HP1 CA5"; $resn_exch{"HP12 CA5"}="2HP1 CA5";
$resn_exch{"HT21 CA5"}="1HT2 CA5"; $resn_exch{"HT22 CA5"}="2HT2 CA5"; $resn_exch{"HT10 CA5"}="0HT1 CA5";
$resn_exch{"HNT6 CA5"}="6HNT CA5"; $resn_exch{"HA11 CA5"}="1HA1 CA5"; $resn_exch{"HA12 CA5"}="2HA1 CA5";
$resn_exch{"HT11 CA5"}="1HT1 CA5"; $resn_exch{"HT12 CA5"}="2HT1 CA5"; $resn_exch{" H2A CAA"}=" AH2 CAA";
$resn_exch{"H61A CAA"}="AH61 CAA"; $resn_exch{"H62A CAA"}="AH62 CAA"; $resn_exch{" H8A CAA"}=" AH8 CAA";
$resn_exch{" H1B CAA"}="AH1* CAA"; $resn_exch{" H2B CAA"}="AH2* CAA"; $resn_exch{"HO2A CAA"}="AHO2 CAA";
$resn_exch{" H3B CAA"}="AH3* CAA"; $resn_exch{"HOA8 CAA"}="8HOA CAA"; $resn_exch{"HOA9 CAA"}="9HOA CAA";
$resn_exch{" H4B CAA"}="AH4* CAA"; $resn_exch{"H51A CAA"}="AH51 CAA"; $resn_exch{"H52A CAA"}="AH52 CAA";
$resn_exch{"HOA2 CAA"}="2HOA CAA"; $resn_exch{"HOA5 CAA"}="5HOA CAA"; $resn_exch{"H121 CAA"}="1H12 CAA";
$resn_exch{"H122 CAA"}="2H12 CAA"; $resn_exch{"H131 CAA"}="1H13 CAA"; $resn_exch{"H132 CAA"}="2H13 CAA";
$resn_exch{"H133 CAA"}="3H13 CAA"; $resn_exch{"H141 CAA"}="1H14 CAA"; $resn_exch{"H142 CAA"}="2H14 CAA";
$resn_exch{"H143 CAA"}="3H14 CAA"; $resn_exch{" H71 CAA"}="1H7  CAA"; $resn_exch{" H72 CAA"}="2H7  CAA";
$resn_exch{" H61 CAA"}="1H6  CAA"; $resn_exch{" H62 CAA"}="2H6  CAA"; $resn_exch{" H31 CAA"}="1H3  CAA";
$resn_exch{" H32 CAA"}="2H3  CAA"; $resn_exch{" H21 CAA"}="1H2  CAA"; $resn_exch{" H22 CAA"}="2H2  CAA";
$resn_exch{"H2'1 CAA"}="1H2' CAA"; $resn_exch{"H2'2 CAA"}="2H2' CAA"; $resn_exch{"H4'1 CAA"}="1H4' CAA";
$resn_exch{"H4'2 CAA"}="2H4' CAA"; $resn_exch{"H4'3 CAA"}="3H4' CAA"; $resn_exch{" H21 CAB"}="1H2  CAB";
$resn_exch{" H22 CAB"}="2H2  CAB"; $resn_exch{" H31 CAB"}="1H3  CAB"; $resn_exch{" H32 CAB"}="2H3  CAB";
$resn_exch{"HN41 CAB"}="1HN4 CAB"; $resn_exch{"HN42 CAB"}="2HN4 CAB"; $resn_exch{" H11 CAC"}="1H1  CAC";
$resn_exch{" H12 CAC"}="2H1  CAC"; $resn_exch{" H13 CAC"}="3H1  CAC"; $resn_exch{" H21 CAC"}="1H2  CAC";
$resn_exch{" H22 CAC"}="2H2  CAC"; $resn_exch{" H23 CAC"}="3H2  CAC"; $resn_exch{" H11 CAD"}="1H1  CAD";
$resn_exch{" H12 CAD"}="2H1  CAD"; $resn_exch{" H13 CAD"}="3H1  CAD"; $resn_exch{" H21 CAD"}="1H2  CAD";
$resn_exch{" H22 CAD"}="2H2  CAD"; $resn_exch{" H23 CAD"}="3H2  CAD"; $resn_exch{" H21 CAE"}="1H2  CAE";
$resn_exch{" H22 CAE"}="2H2  CAE"; $resn_exch{" H31 CAE"}="1H3  CAE"; $resn_exch{" H32 CAE"}="2H3  CAE";
$resn_exch{" H51 CAE"}="1H5  CAE"; $resn_exch{" H52 CAE"}="2H5  CAE"; $resn_exch{" H61 CAE"}="1H6  CAE";
$resn_exch{" H62 CAE"}="2H6  CAE"; $resn_exch{" H81 CAE"}="1H8  CAE"; $resn_exch{" H82 CAE"}="2H8  CAE";
$resn_exch{" H83 CAE"}="3H8  CAE"; $resn_exch{" H91 CAE"}="1H9  CAE"; $resn_exch{" H92 CAE"}="2H9  CAE";
$resn_exch{" H93 CAE"}="3H9  CAE"; $resn_exch{"H101 CAE"}="1H10 CAE"; $resn_exch{"H102 CAE"}="2H10 CAE";
$resn_exch{"H103 CAE"}="3H10 CAE"; $resn_exch{" H   CAF"}="1HN  CAF"; $resn_exch{" H2  CAF"}="2HN  CAF";
$resn_exch{" HB2 CAF"}="1HB  CAF"; $resn_exch{" HB3 CAF"}="2HB  CAF"; $resn_exch{"HE11 CAF"}="1HE1 CAF";
$resn_exch{"HE12 CAF"}="2HE1 CAF"; $resn_exch{"HE13 CAF"}="3HE1 CAF"; $resn_exch{"HE21 CAF"}="1HE2 CAF";
$resn_exch{"HE22 CAF"}="2HE2 CAF"; $resn_exch{"HE23 CAF"}="3HE2 CAF"; $resn_exch{"HOG2 CAG"}="2HOG CAG";
$resn_exch{"HOB2 CAG"}="2HOB CAG"; $resn_exch{"HOA2 CAG"}="2HOA CAG"; $resn_exch{"H5'1 CAG"}="1H5* CAG";
$resn_exch{"H5'2 CAG"}="2H5* CAG"; $resn_exch{" H4B CAG"}=" H4* CAG"; $resn_exch{" H3B CAG"}=" H3* CAG";
$resn_exch{" H2' CAG"}=" H2* CAG"; $resn_exch{" H1' CAG"}=" H1* CAG"; $resn_exch{" H21 CAG"}="1H2  CAG";
$resn_exch{" H22 CAG"}="2H2  CAG"; $resn_exch{"HCM1 CAG"}="1HCM CAG"; $resn_exch{"HCM2 CAG"}="2HCM CAG";
$resn_exch{"HCM3 CAG"}="3HCM CAG"; $resn_exch{" H31 CAH"}="1H3  CAH"; $resn_exch{" H32 CAH"}="2H3  CAH";
$resn_exch{" H61 CAH"}="1H6  CAH"; $resn_exch{" H62 CAH"}="2H6  CAH"; $resn_exch{" H81 CAH"}="1H8  CAH";
$resn_exch{" H82 CAH"}="2H8  CAH"; $resn_exch{" H83 CAH"}="3H8  CAH"; $resn_exch{" H91 CAH"}="1H9  CAH";
$resn_exch{" H92 CAH"}="2H9  CAH"; $resn_exch{" H93 CAH"}="3H9  CAH"; $resn_exch{"H101 CAH"}="1H10 CAH";
$resn_exch{"H102 CAH"}="2H10 CAH"; $resn_exch{"H103 CAH"}="3H10 CAH"; $resn_exch{"H201 CAI"}="1H20 CAI";
$resn_exch{"H202 CAI"}="2H20 CAI"; $resn_exch{"H21A CAI"}="1H2  CAI"; $resn_exch{" H22 CAI"}="2H2  CAI";
$resn_exch{" H71 CAI"}="1H7  CAI"; $resn_exch{" H72 CAI"}="2H7  CAI"; $resn_exch{"H141 CAI"}="1H14 CAI";
$resn_exch{"H142 CAI"}="2H14 CAI"; $resn_exch{"H161 CAI"}="1H16 CAI"; $resn_exch{"H162 CAI"}="2H16 CAI";
$resn_exch{"H163 CAI"}="3H16 CAI"; $resn_exch{"H221 CAI"}="1H22 CAI"; $resn_exch{"H222 CAI"}="2H22 CAI";
$resn_exch{"H223 CAI"}="3H22 CAI"; $resn_exch{"H6A1 CAI"}="1H6A CAI"; $resn_exch{"H6A2 CAI"}="2H6A CAI";
$resn_exch{"H6A3 CAI"}="3H6A CAI"; $resn_exch{"H7A1 CAI"}="1H7A CAI"; $resn_exch{"H7A2 CAI"}="2H7A CAI";
$resn_exch{"H7A3 CAI"}="3H7A CAI"; $resn_exch{"H2B1 CAI"}="1H2B CAI"; $resn_exch{"H2B2 CAI"}="2H2B CAI";
$resn_exch{"H6B1 CAI"}="1H6B CAI"; $resn_exch{"H6B2 CAI"}="2H6B CAI"; $resn_exch{"H6B3 CAI"}="3H6B CAI";
$resn_exch{"H7B1 CAI"}="1H7B CAI"; $resn_exch{"H7B2 CAI"}="2H7B CAI"; $resn_exch{"H7B3 CAI"}="3H7B CAI";
$resn_exch{"H2C1 CAI"}="1H2C CAI"; $resn_exch{"H2C2 CAI"}="2H2C CAI"; $resn_exch{"H4C1 CAI"}="1H4C CAI";
$resn_exch{"H4C2 CAI"}="2H4C CAI"; $resn_exch{"H4C3 CAI"}="3H4C CAI"; $resn_exch{"H5C1 CAI"}="1H5C CAI";
$resn_exch{"H5C2 CAI"}="2H5C CAI"; $resn_exch{"H5C3 CAI"}="3H5C CAI"; $resn_exch{"H181 CAI"}="1H18 CAI";
$resn_exch{"H182 CAI"}="2H18 CAI"; $resn_exch{"H183 CAI"}="3H18 CAI"; $resn_exch{"H191 CAI"}="1H19 CAI";
$resn_exch{"H192 CAI"}="2H19 CAI"; $resn_exch{"H193 CAI"}="3H19 CAI"; $resn_exch{"H8A1 CAI"}="1H8A CAI";
$resn_exch{"H8A2 CAI"}="2H8A CAI"; $resn_exch{"H8A3 CAI"}="3H8A CAI"; $resn_exch{"HAK1 CAK"}="1HAK CAK";
$resn_exch{"HAK2 CAK"}="2HAK CAK"; $resn_exch{"HAE1 CAK"}="1HAE CAK"; $resn_exch{"HAE2 CAK"}="2HAE CAK";
$resn_exch{" H   CAL"}="1HN  CAL"; $resn_exch{" H2  CAL"}="2HN  CAL"; $resn_exch{" HB2 CAL"}="1HB  CAL";
$resn_exch{" HB3 CAL"}="2HB  CAL"; $resn_exch{"HD12 CAL"}="1HD1 CAL"; $resn_exch{"HD13 CAL"}="2HD1 CAL";
$resn_exch{"HD23 CAL"}="1HD2 CAL"; $resn_exch{"HD22 CAL"}="2HD2 CAL"; $resn_exch{"HE12 CAL"}="1HE1 CAL";
$resn_exch{"HE13 CAL"}="2HE1 CAL"; $resn_exch{"HE23 CAL"}="1HE2 CAL"; $resn_exch{"HE22 CAL"}="2HE2 CAL";
$resn_exch{" HZ2 CAL"}="1HZ  CAL"; $resn_exch{" HZ3 CAL"}="2HZ  CAL"; $resn_exch{" HM1 CAL"}="1HM  CAL";
$resn_exch{" HM2 CAL"}="2HM  CAL"; $resn_exch{"HB21 CAL"}="1HB2 CAL"; $resn_exch{"HB22 CAL"}="2HB2 CAL";
$resn_exch{"HD31 CAL"}="1HD3 CAL"; $resn_exch{"HD32 CAL"}="2HD3 CAL"; $resn_exch{"HD33 CAL"}="3HD3 CAL";
$resn_exch{"HD41 CAL"}="1HD4 CAL"; $resn_exch{"HD42 CAL"}="2HD4 CAL"; $resn_exch{"HD43 CAL"}="3HD4 CAL";
$resn_exch{" H31 CAM"}="1H3  CAM"; $resn_exch{" H32 CAM"}="2H3  CAM"; $resn_exch{" H51 CAM"}="1H5  CAM";
$resn_exch{" H52 CAM"}="2H5  CAM"; $resn_exch{" H61 CAM"}="1H6  CAM"; $resn_exch{" H62 CAM"}="2H6  CAM";
$resn_exch{" H81 CAM"}="1H8  CAM"; $resn_exch{" H82 CAM"}="2H8  CAM"; $resn_exch{" H83 CAM"}="3H8  CAM";
$resn_exch{" H91 CAM"}="1H9  CAM"; $resn_exch{" H92 CAM"}="2H9  CAM"; $resn_exch{" H93 CAM"}="3H9  CAM";
$resn_exch{"H101 CAM"}="1H10 CAM"; $resn_exch{"H102 CAM"}="2H10 CAM"; $resn_exch{"H103 CAM"}="3H10 CAM";
$resn_exch{" HN1 CAN"}="1HN  CAN"; $resn_exch{" HN2 CAN"}="2HN  CAN"; $resn_exch{" HB1 CAN"}="1HB  CAN";
$resn_exch{" HB2 CAN"}="2HB  CAN"; $resn_exch{" HG1 CAN"}="1HG  CAN"; $resn_exch{" HG2 CAN"}="2HG  CAN";
$resn_exch{"HNZ1 CAN"}="1HNZ CAN"; $resn_exch{"HNZ2 CAN"}="2HNZ CAN"; $resn_exch{" H2A CAO"}=" AH2 CAO";
$resn_exch{"H61A CAO"}="AH61 CAO"; $resn_exch{"H62A CAO"}="AH62 CAO"; $resn_exch{" H8A CAO"}=" AH8 CAO";
$resn_exch{" H1B CAO"}="AH1* CAO"; $resn_exch{" H2B CAO"}="AH2* CAO"; $resn_exch{"HO2A CAO"}="AHO2 CAO";
$resn_exch{" H3B CAO"}="AH3* CAO"; $resn_exch{"HOA8 CAO"}="8HOA CAO"; $resn_exch{"HOA9 CAO"}="9HOA CAO";
$resn_exch{" H4B CAO"}="AH4* CAO"; $resn_exch{"H51A CAO"}="AH51 CAO"; $resn_exch{"H52A CAO"}="AH52 CAO";
$resn_exch{"HOA2 CAO"}="2HOA CAO"; $resn_exch{"HOA5 CAO"}="5HOA CAO"; $resn_exch{"H121 CAO"}="1H12 CAO";
$resn_exch{"H122 CAO"}="2H12 CAO"; $resn_exch{"H131 CAO"}="1H13 CAO"; $resn_exch{"H132 CAO"}="2H13 CAO";
$resn_exch{"H133 CAO"}="3H13 CAO"; $resn_exch{"H141 CAO"}="1H14 CAO"; $resn_exch{"H142 CAO"}="2H14 CAO";
$resn_exch{"H143 CAO"}="3H14 CAO"; $resn_exch{" H71 CAO"}="1H7  CAO"; $resn_exch{" H72 CAO"}="2H7  CAO";
$resn_exch{" H61 CAO"}="1H6  CAO"; $resn_exch{" H62 CAO"}="2H6  CAO"; $resn_exch{" H31 CAO"}="1H3  CAO";
$resn_exch{" H32 CAO"}="2H3  CAO"; $resn_exch{" H21 CAO"}="1H2  CAO"; $resn_exch{" H22 CAO"}="2H2  CAO";
$resn_exch{"HOS1 CAO"}="1HOS CAO"; $resn_exch{" H11 CAP"}="1H1  CAP"; $resn_exch{" H12 CAP"}="2H1  CAP";
$resn_exch{" H51 CAP"}="1H5  CAP"; $resn_exch{" H52 CAP"}="2H5  CAP"; $resn_exch{"HOP2 CAP"}="2HOP CAP";
$resn_exch{"HOP3 CAP"}="3HOP CAP"; $resn_exch{"HOP5 CAP"}="5HOP CAP"; $resn_exch{"HOP6 CAP"}="6HOP CAP";
$resn_exch{"HOP2 CAR"}="2HOP CAR"; $resn_exch{"HOP3 CAR"}="3HOP CAR"; $resn_exch{" H5' CAR"}="1H5* CAR";
$resn_exch{"H5'' CAR"}="2H5* CAR"; $resn_exch{" H4' CAR"}=" H4* CAR"; $resn_exch{" H3' CAR"}=" H3* CAR";
$resn_exch{"HO3' CAR"}="H3T  CAR"; $resn_exch{" H2' CAR"}=" H2* CAR"; $resn_exch{"HO2' CAR"}="2HO* CAR";
$resn_exch{" H1' CAR"}=" H1* CAR"; $resn_exch{"HN41 CAR"}="1HN4 CAR"; $resn_exch{"HN42 CAR"}="2HN4 CAR";
$resn_exch{" H   CAS"}="1HN  CAS"; $resn_exch{" H2  CAS"}="2HN  CAS"; $resn_exch{" HB2 CAS"}="1HB  CAS";
$resn_exch{" HB3 CAS"}="2HB  CAS"; $resn_exch{"HE11 CAS"}="1HE1 CAS"; $resn_exch{"HE12 CAS"}="2HE1 CAS";
$resn_exch{"HE13 CAS"}="3HE1 CAS"; $resn_exch{"HE21 CAS"}="1HE2 CAS"; $resn_exch{"HE22 CAS"}="2HE2 CAS";
$resn_exch{"HE23 CAS"}="3HE2 CAS"; $resn_exch{" H11 CAT"}="1H1  CAT"; $resn_exch{" H12 CAT"}="2H1  CAT";
$resn_exch{" H21 CAT"}="1H2  CAT"; $resn_exch{" H22 CAT"}="2H2  CAT"; $resn_exch{" H31 CAT"}="1H3  CAT";
$resn_exch{" H32 CAT"}="2H3  CAT"; $resn_exch{" H41 CAT"}="1H4  CAT"; $resn_exch{" H42 CAT"}="2H4  CAT";
$resn_exch{" H51 CAT"}="1H5  CAT"; $resn_exch{" H52 CAT"}="2H5  CAT"; $resn_exch{" H61 CAT"}="1H6  CAT";
$resn_exch{" H62 CAT"}="2H6  CAT"; $resn_exch{" H71 CAT"}="1H7  CAT"; $resn_exch{" H72 CAT"}="2H7  CAT";
$resn_exch{" H81 CAT"}="1H8  CAT"; $resn_exch{" H82 CAT"}="2H8  CAT"; $resn_exch{" H91 CAT"}="1H9  CAT";
$resn_exch{" H92 CAT"}="2H9  CAT"; $resn_exch{"H101 CAT"}="1H10 CAT"; $resn_exch{"H102 CAT"}="2H10 CAT";
$resn_exch{"H111 CAT"}="1H11 CAT"; $resn_exch{"H112 CAT"}="2H11 CAT"; $resn_exch{"H121 CAT"}="1H12 CAT";
$resn_exch{"H122 CAT"}="2H12 CAT"; $resn_exch{"H123 CAT"}="3H12 CAT"; $resn_exch{"HA11 CAT"}="1HA1 CAT";
$resn_exch{"HA12 CAT"}="2HA1 CAT"; $resn_exch{"HA13 CAT"}="3HA1 CAT"; $resn_exch{"HB11 CAT"}="1HB1 CAT";
$resn_exch{"HB12 CAT"}="2HB1 CAT"; $resn_exch{"HB13 CAT"}="3HB1 CAT"; $resn_exch{"HC11 CAT"}="1HC1 CAT";
$resn_exch{"HC12 CAT"}="2HC1 CAT"; $resn_exch{"HC13 CAT"}="3HC1 CAT"; $resn_exch{" H   CAV"}="1HN  CAV";
$resn_exch{" H2  CAV"}="2HN  CAV"; $resn_exch{" HB2 CAV"}="1HB  CAV"; $resn_exch{" HB3 CAV"}="2HB  CAV";
$resn_exch{"HD12 CAV"}="1HD1 CAV"; $resn_exch{"HD13 CAV"}="2HD1 CAV"; $resn_exch{"HD23 CAV"}="1HD2 CAV";
$resn_exch{"HD22 CAV"}="2HD2 CAV"; $resn_exch{"HE12 CAV"}="1HE1 CAV"; $resn_exch{"HE13 CAV"}="2HE1 CAV";
$resn_exch{"HE23 CAV"}="1HE2 CAV"; $resn_exch{"HE22 CAV"}="2HE2 CAV"; $resn_exch{" HZ2 CAV"}="1HZ  CAV";
$resn_exch{" HZ3 CAV"}="2HZ  CAV"; $resn_exch{" HA' CAV"}=" HA* CAV"; $resn_exch{" HB' CAV"}=" HB* CAV";
$resn_exch{"HG11 CAV"}="1HG1 CAV"; $resn_exch{"HG12 CAV"}="2HG1 CAV"; $resn_exch{"HG13 CAV"}="3HG1 CAV";
$resn_exch{"HG21 CAV"}="1HG2 CAV"; $resn_exch{"HG22 CAV"}="2HG2 CAV"; $resn_exch{"HG23 CAV"}="3HG2 CAV";
$resn_exch{"HOB1 CAV"}="1HOB CAV"; $resn_exch{" H91 CAX"}="1H9  CAX"; $resn_exch{" H92 CAX"}="2H9  CAX";
$resn_exch{" H81 CAX"}="1H8  CAX"; $resn_exch{" H82 CAX"}="2H8  CAX"; $resn_exch{" H83 CAX"}="3H8  CAX";
$resn_exch{" H71 CAX"}="1H7  CAX"; $resn_exch{" H72 CAX"}="2H7  CAX"; $resn_exch{" H73 CAX"}="3H7  CAX";
$resn_exch{" H61 CAX"}="1H6  CAX"; $resn_exch{" H62 CAX"}="2H6  CAX"; $resn_exch{" H63 CAX"}="3H6  CAX";
$resn_exch{" H51 CAX"}="1H5  CAX"; $resn_exch{" H52 CAX"}="2H5  CAX"; $resn_exch{" HN1 CAY"}="1HN  CAY";
$resn_exch{" HN2 CAY"}="2HN  CAY"; $resn_exch{" HB1 CAY"}="1HB  CAY"; $resn_exch{" HB2 CAY"}="2HB  CAY";
$resn_exch{" HG1 CAY"}="1HG  CAY"; $resn_exch{" HG2 CAY"}="2HG  CAY"; $resn_exch{"HC21 CAZ"}="1HC2 CAZ";
$resn_exch{"HC22 CAZ"}="2HC2 CAZ"; $resn_exch{"HC31 CAZ"}="1HC3 CAZ"; $resn_exch{"HC32 CAZ"}="2HC3 CAZ";
$resn_exch{"H4AO CAZ"}="OH4A CAZ"; $resn_exch{"H10N CAZ"}="NH10 CAZ"; $resn_exch{"H15C CAZ"}="CH15 CAZ";
$resn_exch{"H181 CAZ"}="1H18 CAZ"; $resn_exch{"H182 CAZ"}="2H18 CAZ"; $resn_exch{"H191 CAZ"}="1H19 CAZ";
$resn_exch{"H192 CAZ"}="2H19 CAZ"; $resn_exch{"H193 CAZ"}="3H19 CAZ"; $resn_exch{"H201 CAZ"}="1H20 CAZ";
$resn_exch{"H202 CAZ"}="2H20 CAZ"; $resn_exch{"H203 CAZ"}="3H20 CAZ"; $resn_exch{"H2AO CAZ"}="OH2A CAZ";
$resn_exch{" H1  CB1"}="1H   CB1"; $resn_exch{" H41 CB1"}="1H4  CB1"; $resn_exch{"HN11 CB1"}="1HN1 CB1";
$resn_exch{"HN12 CB1"}="2HN1 CB1"; $resn_exch{" H71 CB1"}="1H7  CB1"; $resn_exch{" H72 CB1"}="2H7  CB1";
$resn_exch{" H91 CB1"}="1H9  CB1"; $resn_exch{" H92 CB1"}="2H9  CB1"; $resn_exch{"HO3' CB2"}="H3T  CB2";
$resn_exch{" H3' CB2"}=" H3* CB2"; $resn_exch{" H2' CB2"}="1H2* CB2"; $resn_exch{"HO2' CB2"}="2HO* CB2";
$resn_exch{" H1' CB2"}=" H1* CB2"; $resn_exch{" H4' CB2"}=" H4* CB2"; $resn_exch{" H5' CB2"}="1H5* CB2";
$resn_exch{"H5'' CB2"}="2H5* CB2"; $resn_exch{"HN41 CB2"}="1HN4 CB2"; $resn_exch{"HN42 CB2"}="2HN4 CB2";
$resn_exch{"HN21 CB3"}="1HN2 CB3"; $resn_exch{"HN22 CB3"}="2HN2 CB3"; $resn_exch{" H91 CB3"}="1H9  CB3";
$resn_exch{" H92 CB3"}="2H9  CB3"; $resn_exch{" HB1 CB3"}="1HB  CB3"; $resn_exch{" HB2 CB3"}="2HB  CB3";
$resn_exch{" HG1 CB3"}="1HG  CB3"; $resn_exch{" HG2 CB3"}="2HG  CB3"; $resn_exch{"HOE2 CB3"}="2HOE CB3";
$resn_exch{"HP11 CB3"}="1HP1 CB3"; $resn_exch{"HP12 CB3"}="2HP1 CB3"; $resn_exch{"HB1O CB4"}="OHB1 CB4";
$resn_exch{"HB2O CB4"}="OHB2 CB4"; $resn_exch{"HC71 CB4"}="1HC7 CB4"; $resn_exch{"HC72 CB4"}="2HC7 CB4";
$resn_exch{"H10N CB4"}="NH10 CB4"; $resn_exch{"H15C CB4"}="CH15 CB4"; $resn_exch{"H181 CB4"}="1H18 CB4";
$resn_exch{"H182 CB4"}="2H18 CB4"; $resn_exch{"H191 CB4"}="1H19 CB4"; $resn_exch{"H192 CB4"}="2H19 CB4";
$resn_exch{"H193 CB4"}="3H19 CB4"; $resn_exch{"H201 CB4"}="1H20 CB4"; $resn_exch{"H202 CB4"}="2H20 CB4";
$resn_exch{"H203 CB4"}="3H20 CB4"; $resn_exch{"H2BO CB4"}="OH2B CB4"; $resn_exch{"H2A1 CBA"}="1H2A CBA";
$resn_exch{"H2A2 CBA"}="2H2A CBA"; $resn_exch{"H2A3 CBA"}="3H2A CBA"; $resn_exch{"H4A1 CBA"}="1H4A CBA";
$resn_exch{"H4A2 CBA"}="2H4A CBA"; $resn_exch{"H5A1 CBA"}="1H5A CBA"; $resn_exch{"H5A2 CBA"}="2H5A CBA";
$resn_exch{"HOP2 CBA"}="2HOP CBA"; $resn_exch{"HOP3 CBA"}="3HOP CBA"; $resn_exch{"HOD2 CBA"}="2HOD CBA";
$resn_exch{"H101 CBB"}="1H10 CBB"; $resn_exch{"H102 CBB"}="2H10 CBB"; $resn_exch{"H161 CBB"}="1H16 CBB";
$resn_exch{"H118 CBB"}="18H1 CBB"; $resn_exch{"H201 CBB"}="1H20 CBB"; $resn_exch{"H202 CBB"}="2H20 CBB";
$resn_exch{"H331 CBB"}="1H33 CBB"; $resn_exch{"H332 CBB"}="2H33 CBB"; $resn_exch{"H334 CBB"}="34H3 CBB";
$resn_exch{"H371 CBB"}="1H37 CBB"; $resn_exch{"H381 CBB"}="1H38 CBB"; $resn_exch{"H391 CBB"}="1H39 CBB";
$resn_exch{"H411 CBB"}="1H41 CBB"; $resn_exch{"H412 CBB"}="2H41 CBB"; $resn_exch{"H431 CBB"}="1H43 CBB";
$resn_exch{"H432 CBB"}="2H43 CBB"; $resn_exch{"HOA3 CBD"}="3HOA CBD"; $resn_exch{"HN21 CBD"}="1HN2 CBD";
$resn_exch{"HN22 CBD"}="2HN2 CBD"; $resn_exch{"HOB3 CBD"}="3HOB CBD"; $resn_exch{"HOD3 CBD"}="3HOD CBD";
$resn_exch{" H11 CBE"}="1H1  CBE"; $resn_exch{"H12A CBE"}="2H1  CBE"; $resn_exch{"H13A CBE"}="3H1  CBE";
$resn_exch{" H51 CBE"}="1H5  CBE"; $resn_exch{" H52 CBE"}="2H5  CBE"; $resn_exch{" H61 CBE"}="1H6  CBE";
$resn_exch{" H62 CBE"}="2H6  CBE"; $resn_exch{" H61 CBF"}="1H6  CBF"; $resn_exch{" H62 CBF"}="2H6  CBF";
$resn_exch{"HN11 CBF"}="1HN1 CBF"; $resn_exch{"HN12 CBF"}="2HN1 CBF"; $resn_exch{" H11 CBG"}="1H1  CBG";
$resn_exch{" H21 CBG"}="1H2  CBG"; $resn_exch{" H22 CBG"}="2H2  CBG"; $resn_exch{" H31 CBG"}="1H3  CBG";
$resn_exch{" H32 CBG"}="2H3  CBG"; $resn_exch{" H33 CBG"}="3H3  CBG"; $resn_exch{"H151 CBH"}="1H15 CBH";
$resn_exch{"H152 CBH"}="2H15 CBH"; $resn_exch{"H161 CBH"}="1H16 CBH"; $resn_exch{"H162 CBH"}="2H16 CBH";
$resn_exch{"H171 CBH"}="1H17 CBH"; $resn_exch{"H172 CBH"}="2H17 CBH"; $resn_exch{"H191 CBH"}="1H19 CBH";
$resn_exch{"H192 CBH"}="2H19 CBH"; $resn_exch{"H201 CBH"}="1H20 CBH"; $resn_exch{"H102 CBH"}="2H10 CBH";
$resn_exch{"H211 CBH"}="1H21 CBH"; $resn_exch{"H212 CBH"}="2H21 CBH"; $resn_exch{"H221 CBH"}="1H22 CBH";
$resn_exch{"H222 CBH"}="2H22 CBH"; $resn_exch{" H61 CBI"}="1H6  CBI"; $resn_exch{" H62 CBI"}="2H6  CBI";
$resn_exch{"H6'1 CBI"}="1H6' CBI"; $resn_exch{"H6'2 CBI"}="2H6' CBI"; $resn_exch{"HO1' CBI"}="'HO1 CBI";
$resn_exch{"HO2' CBI"}="'HO2 CBI"; $resn_exch{"HO3' CBI"}="'HO3 CBI"; $resn_exch{"HO6' CBI"}="'HO6 CBI";
$resn_exch{" H71 CBL"}="1H7  CBL"; $resn_exch{" H72 CBL"}="2H7  CBL"; $resn_exch{" H81 CBL"}="1H8  CBL";
$resn_exch{" H82 CBL"}="2H8  CBL"; $resn_exch{" H91 CBL"}="1H9  CBL"; $resn_exch{" H92 CBL"}="2H9  CBL";
$resn_exch{"H121 CBL"}="1H12 CBL"; $resn_exch{"H122 CBL"}="2H12 CBL"; $resn_exch{"H131 CBL"}="1H13 CBL";
$resn_exch{"H132 CBL"}="2H13 CBL"; $resn_exch{"H151 CBL"}="1H15 CBL"; $resn_exch{"H152 CBL"}="2H15 CBL";
$resn_exch{"H161 CBL"}="1H16 CBL"; $resn_exch{"H162 CBL"}="2H16 CBL"; $resn_exch{" H21 CBM"}="1H2  CBM";
$resn_exch{" H22 CBM"}="2H2  CBM"; $resn_exch{" H23 CBM"}="3H2  CBM"; $resn_exch{" H71 CBN"}="1H7  CBN";
$resn_exch{" H72 CBN"}="2H7  CBN"; $resn_exch{" H73 CBN"}="3H7  CBN"; $resn_exch{" H81 CBN"}="1H8  CBN";
$resn_exch{" H82 CBN"}="2H8  CBN"; $resn_exch{" H83 CBN"}="3H8  CBN"; $resn_exch{"H121 CBN"}="1H12 CBN";
$resn_exch{"H122 CBN"}="2H12 CBN"; $resn_exch{"H123 CBN"}="3H12 CBN"; $resn_exch{"H251 CBN"}="1H25 CBN";
$resn_exch{"H252 CBN"}="2H25 CBN"; $resn_exch{"H301 CBN"}="1H30 CBN"; $resn_exch{"H302 CBN"}="2H30 CBN";
$resn_exch{"H303 CBN"}="3H30 CBN"; $resn_exch{"H341 CBN"}="1H34 CBN"; $resn_exch{"H342 CBN"}="2H34 CBN";
$resn_exch{"H343 CBN"}="3H34 CBN"; $resn_exch{" H61 CBN"}="1H6  CBN"; $resn_exch{" H62 CBN"}="2H6  CBN";
$resn_exch{" H63 CBN"}="3H6  CBN"; $resn_exch{" H11 CBO"}="1H1  CBO"; $resn_exch{"H12A CBO"}="2H1  CBO";
$resn_exch{" H21 CBO"}="1H2  CBO"; $resn_exch{" H22 CBO"}="2H2  CBO"; $resn_exch{" H61 CBO"}="1H6  CBO";
$resn_exch{" H62 CBO"}="2H6  CBO"; $resn_exch{" H71 CBO"}="1H7  CBO"; $resn_exch{" H72 CBO"}="2H7  CBO";
$resn_exch{"H151 CBO"}="1H15 CBO"; $resn_exch{"H152 CBO"}="2H15 CBO"; $resn_exch{"H161 CBO"}="1H16 CBO";
$resn_exch{"H162 CBO"}="2H16 CBO"; $resn_exch{"H191 CBO"}="1H19 CBO"; $resn_exch{"H192 CBO"}="2H19 CBO";
$resn_exch{"H193 CBO"}="3H19 CBO"; $resn_exch{"H201 CBO"}="1H20 CBO"; $resn_exch{"H202 CBO"}="2H20 CBO";
$resn_exch{"H221 CBO"}="1H22 CBO"; $resn_exch{"H222 CBO"}="2H22 CBO"; $resn_exch{"H231 CBO"}="1H23 CBO";
$resn_exch{"H232 CBO"}="2H23 CBO"; $resn_exch{"H241 CBO"}="1H24 CBO"; $resn_exch{"H242 CBO"}="2H24 CBO";
$resn_exch{"H243 CBO"}="3H24 CBO"; $resn_exch{"H251 CBO"}="1H25 CBO"; $resn_exch{"H252 CBO"}="2H25 CBO";
$resn_exch{"H253 CBO"}="3H25 CBO"; $resn_exch{"H261 CBO"}="1H26 CBO"; $resn_exch{"H262 CBO"}="2H26 CBO";
$resn_exch{"H263 CBO"}="3H26 CBO"; $resn_exch{"H271 CBO"}="1H27 CBO"; $resn_exch{"H272 CBO"}="2H27 CBO";
$resn_exch{"H273 CBO"}="3H27 CBO"; $resn_exch{"H281 CBO"}="1H28 CBO"; $resn_exch{"H282 CBO"}="2H28 CBO";
$resn_exch{"H283 CBO"}="3H28 CBO"; $resn_exch{"H301 CBO"}="1H30 CBO"; $resn_exch{"H302 CBO"}="2H30 CBO";
$resn_exch{"H311 CBO"}="1H31 CBO"; $resn_exch{"H312 CBO"}="2H31 CBO"; $resn_exch{"H341 CBO"}="1H34 CBO";
$resn_exch{"H342 CBO"}="2H34 CBO"; $resn_exch{"H343 CBO"}="3H34 CBO"; $resn_exch{"HO1I CBP"}="IHO1 CBP";
$resn_exch{"HN21 CBP"}="1HN2 CBP"; $resn_exch{"HC51 CBP"}="1HC5 CBP"; $resn_exch{"HC52 CBP"}="2HC5 CBP";
$resn_exch{"HC71 CBP"}="1HC7 CBP"; $resn_exch{"HC72 CBP"}="2HC7 CBP"; $resn_exch{"HC81 CBP"}="1HC8 CBP";
$resn_exch{"HC82 CBP"}="2HC8 CBP"; $resn_exch{"H101 CBP"}="1H10 CBP"; $resn_exch{"H102 CBP"}="2H10 CBP";
$resn_exch{"H111 CBP"}="1H11 CBP"; $resn_exch{"H112 CBP"}="2H11 CBP"; $resn_exch{"H161 CBP"}="1H16 CBP";
$resn_exch{"H171 CBP"}="1H17 CBP"; $resn_exch{"H191 CBP"}="1H19 CBP"; $resn_exch{"H201 CBP"}="1H20 CBP";
$resn_exch{"H231 CBP"}="1H23 CBP"; $resn_exch{"H241 CBP"}="1H24 CBP"; $resn_exch{"H261 CBP"}="1H26 CBP";
$resn_exch{"H271 CBP"}="1H27 CBP"; $resn_exch{"HOP2 CBR"}="2HOP CBR"; $resn_exch{" H41 CBR"}="1H4  CBR";
$resn_exch{" H42 CBR"}="2H4  CBR"; $resn_exch{" H2' CBR"}="1H2* CBR"; $resn_exch{"H2'' CBR"}="2H2* CBR";
$resn_exch{" H5' CBR"}="1H5* CBR"; $resn_exch{"H5'' CBR"}="2H5* CBR"; $resn_exch{" H4' CBR"}=" H4* CBR";
$resn_exch{" H1' CBR"}=" H1* CBR"; $resn_exch{" H3' CBR"}=" H3* CBR"; $resn_exch{"HO3' CBR"}="H3T  CBR";
$resn_exch{"HOP3 CBR"}="3HOP CBR"; $resn_exch{" H1A CBS"}=" AH1 CBS"; $resn_exch{" H2A CBS"}=" AH2 CBS";
$resn_exch{" H3A CBS"}=" AH3 CBS"; $resn_exch{" H4A CBS"}=" AH4 CBS"; $resn_exch{" H5A CBS"}=" AH5 CBS";
$resn_exch{"H61A CBS"}="AH61 CBS"; $resn_exch{"H62A CBS"}="AH62 CBS"; $resn_exch{"H81A CBS"}="AH81 CBS";
$resn_exch{"H82A CBS"}="AH82 CBS"; $resn_exch{"H83A CBS"}="AH83 CBS"; $resn_exch{"HN2A CBS"}="AHN2 CBS";
$resn_exch{"HO3A CBS"}="AHO3 CBS"; $resn_exch{"HO4A CBS"}="AHO4 CBS"; $resn_exch{"HO6A CBS"}="AHO6 CBS";
$resn_exch{" H1B CBS"}=" BH1 CBS"; $resn_exch{" H2B CBS"}=" BH2 CBS"; $resn_exch{" H3B CBS"}=" BH3 CBS";
$resn_exch{" H4B CBS"}=" BH4 CBS"; $resn_exch{" H5B CBS"}=" BH5 CBS"; $resn_exch{"H61B CBS"}="BH61 CBS";
$resn_exch{"H62B CBS"}="BH62 CBS"; $resn_exch{"H81B CBS"}="BH81 CBS"; $resn_exch{"H82B CBS"}="BH82 CBS";
$resn_exch{"H83B CBS"}="BH83 CBS"; $resn_exch{"HN2B CBS"}="BHN2 CBS"; $resn_exch{"HO1B CBS"}="BHO1 CBS";
$resn_exch{"HO3B CBS"}="BHO3 CBS"; $resn_exch{"HO6B CBS"}="BHO6 CBS"; $resn_exch{"H171 CBT"}="1H17 CBT";
$resn_exch{"H172 CBT"}="2H17 CBT"; $resn_exch{" H11 CBT"}="1H1  CBT"; $resn_exch{" H12 CBT"}="2H1  CBT";
$resn_exch{"H5'1 CBV"}="1H5* CBV"; $resn_exch{"H5'2 CBV"}="2H5* CBV"; $resn_exch{" H4' CBV"}=" H4* CBV";
$resn_exch{" H3' CBV"}=" H3* CBV"; $resn_exch{"HO3' CBV"}="HO3* CBV"; $resn_exch{" H2' CBV"}=" H2* CBV";
$resn_exch{"HO2' CBV"}="HO2* CBV"; $resn_exch{" H1' CBV"}=" H1* CBV"; $resn_exch{"HN41 CBV"}="1HN4 CBV";
$resn_exch{"HN42 CBV"}="2HN4 CBV"; $resn_exch{"H1'1 CBZ"}="1H1' CBZ"; $resn_exch{"H1'2 CBZ"}="2H1' CBZ";
$resn_exch{"HC11 CC0"}="1HC1 CC0"; $resn_exch{"HN11 CC0"}="1HN1 CC0"; $resn_exch{"HC01 CC0"}="1HC0 CC0";
$resn_exch{"HN21 CC0"}="1HN2 CC0"; $resn_exch{"HC41 CC0"}="1HC4 CC0"; $resn_exch{"HC42 CC0"}="2HC4 CC0";
$resn_exch{"HC61 CC0"}="1HC6 CC0"; $resn_exch{"HC71 CC0"}="1HC7 CC0"; $resn_exch{"HC91 CC0"}="1HC9 CC0";
$resn_exch{"H111 CC0"}="1H11 CC0"; $resn_exch{"H112 CC0"}="2H11 CC0"; $resn_exch{"H113 CC0"}="3H11 CC0";
$resn_exch{"H131 CC0"}="1H13 CC0"; $resn_exch{"H132 CC0"}="2H13 CC0"; $resn_exch{"H133 CC0"}="3H13 CC0";
$resn_exch{"H151 CC0"}="1H15 CC0"; $resn_exch{"H181 CC0"}="1H18 CC0"; $resn_exch{"H191 CC0"}="1H19 CC0";
$resn_exch{"H201 CC0"}="1H20 CC0"; $resn_exch{"H202 CC0"}="2H20 CC0"; $resn_exch{"H211 CC0"}="1H21 CC0";
$resn_exch{"H221 CC0"}="1H22 CC0"; $resn_exch{"H222 CC0"}="2H22 CC0"; $resn_exch{"H231 CC0"}="1H23 CC0";
$resn_exch{"H232 CC0"}="2H23 CC0"; $resn_exch{"H241 CC0"}="1H24 CC0"; $resn_exch{"H242 CC0"}="2H24 CC0";
$resn_exch{"H251 CC0"}="1H25 CC0"; $resn_exch{"H252 CC0"}="2H25 CC0"; $resn_exch{"H261 CC0"}="1H26 CC0";
$resn_exch{"H262 CC0"}="2H26 CC0"; $resn_exch{"HN31 CC0"}="1HN3 CC0"; $resn_exch{"HN32 CC0"}="2HN3 CC0";
$resn_exch{"H281 CC0"}="1H28 CC0"; $resn_exch{"H282 CC0"}="2H28 CC0"; $resn_exch{"HC11 CC1"}="1HC1 CC1";
$resn_exch{"HN11 CC1"}="1HN1 CC1"; $resn_exch{"HCO1 CC1"}="1HCO CC1"; $resn_exch{"HN21 CC1"}="1HN2 CC1";
$resn_exch{"H131 CC1"}="1H13 CC1"; $resn_exch{"H132 CC1"}="2H13 CC1"; $resn_exch{"H133 CC1"}="3H13 CC1";
$resn_exch{"HC41 CC1"}="1HC4 CC1"; $resn_exch{"HC42 CC1"}="2HC4 CC1"; $resn_exch{"HC61 CC1"}="1HC6 CC1";
$resn_exch{"HR11 CC1"}="1HR1 CC1"; $resn_exch{"HR31 CC1"}="1HR3 CC1"; $resn_exch{"HR41 CC1"}="1HR4 CC1";
$resn_exch{"HR51 CC1"}="1HR5 CC1"; $resn_exch{"HC91 CC1"}="1HC9 CC1"; $resn_exch{"HC71 CC1"}="1HC7 CC1";
$resn_exch{"H111 CC1"}="1H11 CC1"; $resn_exch{"H112 CC1"}="2H11 CC1"; $resn_exch{"H301 CC1"}="1H30 CC1";
$resn_exch{"H302 CC1"}="2H30 CC1"; $resn_exch{"H281 CC1"}="1H28 CC1"; $resn_exch{"H282 CC1"}="2H28 CC1";
$resn_exch{"H311 CC1"}="1H31 CC1"; $resn_exch{"H312 CC1"}="2H31 CC1"; $resn_exch{"H181 CC1"}="1H18 CC1";
$resn_exch{"H201 CC1"}="1H20 CC1"; $resn_exch{"H202 CC1"}="2H20 CC1"; $resn_exch{"H211 CC1"}="1H21 CC1";
$resn_exch{"H221 CC1"}="1H22 CC1"; $resn_exch{"H222 CC1"}="2H22 CC1"; $resn_exch{"H231 CC1"}="1H23 CC1";
$resn_exch{"H232 CC1"}="2H23 CC1"; $resn_exch{"H241 CC1"}="1H24 CC1"; $resn_exch{"H242 CC1"}="2H24 CC1";
$resn_exch{"H251 CC1"}="1H25 CC1"; $resn_exch{"H252 CC1"}="2H25 CC1"; $resn_exch{"H261 CC1"}="1H26 CC1";
$resn_exch{"H262 CC1"}="2H26 CC1"; $resn_exch{"H151 CC1"}="1H15 CC1"; $resn_exch{"HN31 CC1"}="1HN3 CC1";
$resn_exch{"HN32 CC1"}="2HN3 CC1"; $resn_exch{"H301 CC3"}="1H30 CC3"; $resn_exch{"H302 CC3"}="2H30 CC3";
$resn_exch{"H291 CC3"}="1H29 CC3"; $resn_exch{"H292 CC3"}="2H29 CC3"; $resn_exch{"H6N1 CC5"}="1H6N CC5";
$resn_exch{"H6N2 CC5"}="2H6N CC5"; $resn_exch{"H4'1 CC5"}="1H4' CC5"; $resn_exch{"H4'2 CC5"}="2H4' CC5";
$resn_exch{" H71 CCB"}="1H7  CCB"; $resn_exch{" H72 CCB"}="2H7  CCB"; $resn_exch{" H81 CCB"}="1H8  CCB";
$resn_exch{" H82 CCB"}="2H8  CCB"; $resn_exch{" H91 CCB"}="1H9  CCB"; $resn_exch{" H92 CCB"}="2H9  CCB";
$resn_exch{"H101 CCB"}="1H10 CCB"; $resn_exch{"H102 CCB"}="2H10 CCB"; $resn_exch{"H201 CCB"}="1H20 CCB";
$resn_exch{"H202 CCB"}="2H20 CCB"; $resn_exch{"H211 CCB"}="1H21 CCB"; $resn_exch{"H212 CCB"}="2H21 CCB";
$resn_exch{"H221 CCB"}="1H22 CCB"; $resn_exch{"H222 CCB"}="2H22 CCB"; $resn_exch{"HOC2 CCC"}="2HOC CCC";
$resn_exch{"HOP2 CCC"}="2HOP CCC"; $resn_exch{"HOP3 CCC"}="3HOP CCC"; $resn_exch{" H5' CCC"}="1H5* CCC";
$resn_exch{"H5'' CCC"}="2H5* CCC"; $resn_exch{" H4' CCC"}=" H4* CCC"; $resn_exch{" H3' CCC"}=" H3* CCC";
$resn_exch{" H2' CCC"}=" H2* CCC"; $resn_exch{" H1' CCC"}=" H1* CCC"; $resn_exch{" H41 CCC"}="1H4  CCC";
$resn_exch{" H42 CCC"}="2H4  CCC"; $resn_exch{"H2C1 CCE"}="1H2C CCE"; $resn_exch{"H2C2 CCE"}="2H2C CCE";
$resn_exch{"H3C1 CCE"}="1H3C CCE"; $resn_exch{"H3C2 CCE"}="2H3C CCE"; $resn_exch{"H8C1 CCE"}="1H8C CCE";
$resn_exch{"H8C2 CCE"}="2H8C CCE"; $resn_exch{"H8C3 CCE"}="3H8C CCE"; $resn_exch{"H9C1 CCE"}="1H9C CCE";
$resn_exch{"H9C2 CCE"}="2H9C CCE"; $resn_exch{"H9C3 CCE"}="3H9C CCE"; $resn_exch{"H101 CCE"}="1H10 CCE";
$resn_exch{"H102 CCE"}="2H10 CCE"; $resn_exch{"H103 CCE"}="3H10 CCE"; $resn_exch{"H6N1 CCE"}="1H6N CCE";
$resn_exch{"H6N2 CCE"}="2H6N CCE"; $resn_exch{"HBC1 CCG"}="1HBC CCG"; $resn_exch{"HBC2 CCG"}="2HBC CCG";
$resn_exch{"HBC3 CCG"}="3HBC CCG"; $resn_exch{"HCV1 CCG"}="1HCV CCG"; $resn_exch{"HCV2 CCG"}="2HCV CCG";
$resn_exch{"HCV3 CCG"}="3HCV CCG"; $resn_exch{"HCU1 CCG"}="1HCU CCG"; $resn_exch{"HCU2 CCG"}="2HCU CCG";
$resn_exch{"HCX1 CCG"}="1HCX CCG"; $resn_exch{"HCX2 CCG"}="2HCX CCG"; $resn_exch{"HCX3 CCG"}="3HCX CCG";
$resn_exch{"HCO1 CCG"}="1HCO CCG"; $resn_exch{"HCO2 CCG"}="2HCO CCG"; $resn_exch{"HBW1 CCG"}="1HBW CCG";
$resn_exch{"HBW2 CCG"}="2HBW CCG"; $resn_exch{"HCC1 CCG"}="1HCC CCG"; $resn_exch{"HCC2 CCG"}="2HCC CCG";
$resn_exch{"HCC3 CCG"}="3HCC CCG"; $resn_exch{"HBZ1 CCG"}="1HBZ CCG"; $resn_exch{"HBZ2 CCG"}="2HBZ CCG";
$resn_exch{"HCA1 CCG"}="1HCA CCG"; $resn_exch{"HCA2 CCG"}="2HCA CCG"; $resn_exch{"HCB1 CCG"}="1HCB CCG";
$resn_exch{"HCB2 CCG"}="2HCB CCG"; $resn_exch{"HCB3 CCG"}="3HCB CCG"; $resn_exch{"HAF1 CCG"}="1HAF CCG";
$resn_exch{"HAF2 CCG"}="2HAF CCG"; $resn_exch{"HAF3 CCG"}="3HAF CCG"; $resn_exch{"HAM1 CCG"}="1HAM CCG";
$resn_exch{"HAM2 CCG"}="2HAM CCG"; $resn_exch{"HAQ1 CCG"}="1HAQ CCG"; $resn_exch{"HAQ2 CCG"}="2HAQ CCG";
$resn_exch{"HAQ3 CCG"}="3HAQ CCG"; $resn_exch{"HBE1 CCG"}="1HBE CCG"; $resn_exch{"HBE2 CCG"}="2HBE CCG";
$resn_exch{"HBE3 CCG"}="3HBE CCG"; $resn_exch{"HBD1 CCG"}="1HBD CCG"; $resn_exch{"HBD2 CCG"}="2HBD CCG";
$resn_exch{"HBD3 CCG"}="3HBD CCG"; $resn_exch{"HBO1 CCG"}="1HBO CCG"; $resn_exch{"HBO2 CCG"}="2HBO CCG";
$resn_exch{"HBO3 CCG"}="3HBO CCG"; $resn_exch{"HBP1 CCG"}="1HBP CCG"; $resn_exch{"HBP2 CCG"}="2HBP CCG";
$resn_exch{"HBP3 CCG"}="3HBP CCG"; $resn_exch{"HAA1 CCH"}="1HAA CCH"; $resn_exch{"HAA2 CCH"}="2HAA CCH";
$resn_exch{"HMA1 CCH"}="1HMA CCH"; $resn_exch{"HMA2 CCH"}="2HMA CCH"; $resn_exch{"HMA3 CCH"}="3HMA CCH";
$resn_exch{"HBA1 CCH"}="1HBA CCH"; $resn_exch{"HBA2 CCH"}="2HBA CCH"; $resn_exch{"HO1A CCH"}="AHO1 CCH";
$resn_exch{"HMB1 CCH"}="1HMB CCH"; $resn_exch{"HMB2 CCH"}="2HMB CCH"; $resn_exch{"HMB3 CCH"}="3HMB CCH";
$resn_exch{"HMC1 CCH"}="1HMC CCH"; $resn_exch{"HMC2 CCH"}="2HMC CCH"; $resn_exch{"HMC3 CCH"}="3HMC CCH";
$resn_exch{"HBC1 CCH"}="1HBC CCH"; $resn_exch{"HBC2 CCH"}="2HBC CCH"; $resn_exch{"HMD1 CCH"}="1HMD CCH";
$resn_exch{"HMD2 CCH"}="2HMD CCH"; $resn_exch{"HMD3 CCH"}="3HMD CCH"; $resn_exch{"HAD1 CCH"}="1HAD CCH";
$resn_exch{"HAD2 CCH"}="2HAD CCH"; $resn_exch{"HBD1 CCH"}="1HBD CCH"; $resn_exch{"HBD2 CCH"}="2HBD CCH";
$resn_exch{"HO1D CCH"}="DHO1 CCH"; $resn_exch{"H6D1 CCI"}="1H6D CCI"; $resn_exch{"H6D2 CCI"}="2H6D CCI";
$resn_exch{"H6D3 CCI"}="3H6D CCI"; $resn_exch{"HOD2 CCI"}="2HOD CCI"; $resn_exch{"HOD4 CCI"}="4HOD CCI";
$resn_exch{"H7D1 CCI"}="1H7D CCI"; $resn_exch{"H7D2 CCI"}="2H7D CCI"; $resn_exch{"H7D3 CCI"}="3H7D CCI";
$resn_exch{"H8C1 CCI"}="1H8C CCI"; $resn_exch{"H8C2 CCI"}="2H8C CCI"; $resn_exch{"H8C3 CCI"}="3H8C CCI";
$resn_exch{"H9C1 CCI"}="1H9C CCI"; $resn_exch{"H9C2 CCI"}="2H9C CCI"; $resn_exch{"H9C3 CCI"}="3H9C CCI";
$resn_exch{"H7C1 CCI"}="1H7C CCI"; $resn_exch{"H7C2 CCI"}="2H7C CCI"; $resn_exch{"H7C3 CCI"}="3H7C CCI";
$resn_exch{"H2B1 CCI"}="1H2B CCI"; $resn_exch{"H2B2 CCI"}="2H2B CCI"; $resn_exch{"H6B1 CCI"}="1H6B CCI";
$resn_exch{"H6B2 CCI"}="2H6B CCI"; $resn_exch{"H6B3 CCI"}="3H6B CCI"; $resn_exch{"HOB3 CCI"}="3HOB CCI";
$resn_exch{"H6A1 CCI"}="1H6A CCI"; $resn_exch{"H6A2 CCI"}="2H6A CCI"; $resn_exch{"H6A3 CCI"}="3H6A CCI";
$resn_exch{"HOA3 CCI"}="3HOA CCI"; $resn_exch{"HAN1 CCI"}="1HAN CCI"; $resn_exch{"HAN2 CCI"}="2HAN CCI";
$resn_exch{"HAN3 CCI"}="3HAN CCI"; $resn_exch{"H2E1 CCI"}="1H2E CCI"; $resn_exch{"H2E2 CCI"}="2H2E CCI";
$resn_exch{"H5E1 CCI"}="1H5E CCI"; $resn_exch{"H5E2 CCI"}="2H5E CCI"; $resn_exch{"H6E1 CCI"}="1H6E CCI";
$resn_exch{"H6E2 CCI"}="2H6E CCI"; $resn_exch{"H6E3 CCI"}="3H6E CCI"; $resn_exch{"HNE2 CCI"}="2HNE CCI";
$resn_exch{"H7E1 CCI"}="1H7E CCI"; $resn_exch{"H7E2 CCI"}="2H7E CCI"; $resn_exch{"H8E1 CCI"}="1H8E CCI";
$resn_exch{"H8E2 CCI"}="2H8E CCI"; $resn_exch{"H8E3 CCI"}="3H8E CCI"; $resn_exch{"HAK1 CCL"}="1HAK CCL";
$resn_exch{"HAK2 CCL"}="2HAK CCL"; $resn_exch{"HAG1 CCL"}="1HAG CCL"; $resn_exch{"HAG2 CCL"}="2HAG CCL";
$resn_exch{"HAH1 CCL"}="1HAH CCL"; $resn_exch{"HAH2 CCL"}="2HAH CCL"; $resn_exch{"HAL1 CCL"}="1HAL CCL";
$resn_exch{"HAL2 CCL"}="2HAL CCL"; $resn_exch{"HAI1 CCL"}="1HAI CCL"; $resn_exch{"HAI2 CCL"}="2HAI CCL";
$resn_exch{"HAE1 CCL"}="1HAE CCL"; $resn_exch{"HAE2 CCL"}="2HAE CCL"; $resn_exch{"HAF1 CCL"}="1HAF CCL";
$resn_exch{"HAF2 CCL"}="2HAF CCL"; $resn_exch{" HB1 CCL"}="1HB  CCL"; $resn_exch{" HB2 CCL"}="2HB  CCL";
$resn_exch{" HN1 CCL"}="1HN  CCL"; $resn_exch{" HN2 CCL"}="2HN  CCL"; $resn_exch{"HH21 CCM"}="1HH2 CCM";
$resn_exch{"HH22 CCM"}="2HH2 CCM"; $resn_exch{"HH23 CCM"}="3HH2 CCM"; $resn_exch{"HB11 CCM"}="1HB1 CCM";
$resn_exch{"HB12 CCM"}="2HB1 CCM"; $resn_exch{"HB21 CCM"}="1HB2 CCM"; $resn_exch{"HB22 CCM"}="2HB2 CCM";
$resn_exch{"HG11 CCM"}="1HG1 CCM"; $resn_exch{"HG12 CCM"}="2HG1 CCM"; $resn_exch{"HG21 CCM"}="1HG2 CCM";
$resn_exch{"HG22 CCM"}="2HG2 CCM"; $resn_exch{" H21 CCN"}="1H2  CCN"; $resn_exch{" H22 CCN"}="2H2  CCN";
$resn_exch{" H23 CCN"}="3H2  CCN"; $resn_exch{"H1B1 CCO"}="1H1B CCO"; $resn_exch{"H1B2 CCO"}="2H1B CCO";
$resn_exch{"HOP2 CCP"}="2HOP CCP"; $resn_exch{"HOP3 CCP"}="3HOP CCP"; $resn_exch{" H11 CCP"}="1H1  CCP";
$resn_exch{" H12 CCP"}="2H1  CCP"; $resn_exch{" H21 CCP"}="1H2  CCP"; $resn_exch{" H22 CCP"}="2H2  CCP";
$resn_exch{" H31 CCP"}="1H3  CCP"; $resn_exch{" H32 CCP"}="2H3  CCP"; $resn_exch{" H41 CCP"}="1H4  CCP";
$resn_exch{" H42 CCP"}="2H4  CCP"; $resn_exch{" H43 CCP"}="3H4  CCP"; $resn_exch{" H21 CCQ"}="1H2  CCQ";
$resn_exch{" H22 CCQ"}="2H2  CCQ"; $resn_exch{" H41 CCQ"}="1H4  CCQ"; $resn_exch{" H42 CCQ"}="2H4  CCQ";
$resn_exch{"H5C1 CCQ"}="1H5C CCQ"; $resn_exch{"H5C2 CCQ"}="2H5C CCQ"; $resn_exch{"H5C3 CCQ"}="3H5C CCQ";
$resn_exch{"H5B1 CCQ"}="1H5B CCQ"; $resn_exch{"H5B2 CCQ"}="2H5B CCQ"; $resn_exch{"H5B3 CCQ"}="3H5B CCQ";
$resn_exch{"H5A1 CCQ"}="1H5A CCQ"; $resn_exch{"H5A2 CCQ"}="2H5A CCQ"; $resn_exch{"H5A3 CCQ"}="3H5A CCQ";
$resn_exch{" H11 CCQ"}="1H1  CCQ"; $resn_exch{" H12 CCQ"}="2H1  CCQ"; $resn_exch{" H31 CCQ"}="1H3  CCQ";
$resn_exch{" H32 CCQ"}="2H3  CCQ"; $resn_exch{" H61 CCQ"}="1H6  CCQ"; $resn_exch{" H62 CCQ"}="2H6  CCQ";
$resn_exch{" H71 CCQ"}="1H7  CCQ"; $resn_exch{" H72 CCQ"}="2H7  CCQ"; $resn_exch{"H131 CCQ"}="1H13 CCQ";
$resn_exch{"H132 CCQ"}="2H13 CCQ"; $resn_exch{"H133 CCQ"}="3H13 CCQ"; $resn_exch{"H141 CCQ"}="1H14 CCQ";
$resn_exch{"H142 CCQ"}="2H14 CCQ"; $resn_exch{"H143 CCQ"}="3H14 CCQ"; $resn_exch{"H121 CCQ"}="1H12 CCQ";
$resn_exch{"H122 CCQ"}="2H12 CCQ"; $resn_exch{"H5'1 CCQ"}="1H5* CCQ"; $resn_exch{"H5'2 CCQ"}="2H5* CCQ";
$resn_exch{" H4' CCQ"}=" H4* CCQ"; $resn_exch{" H3' CCQ"}=" H3* CCQ"; $resn_exch{" H2' CCQ"}=" H2* CCQ";
$resn_exch{" H1' CCQ"}=" H1* CCQ"; $resn_exch{"HN61 CCQ"}="1HN6 CCQ"; $resn_exch{"HN62 CCQ"}="2HN6 CCQ";
$resn_exch{"H521 CCR"}="1H52 CCR"; $resn_exch{"H531 CCR"}="1H53 CCR"; $resn_exch{"H551 CCR"}="1H55 CCR";
$resn_exch{"H561 CCR"}="1H56 CCR"; $resn_exch{"H581 CCR"}="1H58 CCR"; $resn_exch{"H582 CCR"}="2H58 CCR";
$resn_exch{"H591 CCR"}="1H59 CCR"; $resn_exch{"H441 CCR"}="1H44 CCR"; $resn_exch{"H442 CCR"}="2H44 CCR";
$resn_exch{"H411 CCR"}="1H41 CCR"; $resn_exch{"H401 CCR"}="1H40 CCR"; $resn_exch{"H611 CCR"}="1H61 CCR";
$resn_exch{"H612 CCR"}="2H61 CCR"; $resn_exch{"H621 CCR"}="1H62 CCR"; $resn_exch{"H622 CCR"}="2H62 CCR";
$resn_exch{"H631 CCR"}="1H63 CCR"; $resn_exch{"H632 CCR"}="2H63 CCR"; $resn_exch{"H641 CCR"}="1H64 CCR";
$resn_exch{"H642 CCR"}="2H64 CCR"; $resn_exch{"H651 CCR"}="1H65 CCR"; $resn_exch{"H652 CCR"}="2H65 CCR";
$resn_exch{"H231 CCR"}="1H23 CCR"; $resn_exch{"H151 CCR"}="1H15 CCR"; $resn_exch{"H171 CCR"}="1H17 CCR";
$resn_exch{"H172 CCR"}="2H17 CCR"; $resn_exch{"H173 CCR"}="3H17 CCR"; $resn_exch{"H181 CCR"}="1H18 CCR";
$resn_exch{"H182 CCR"}="2H18 CCR"; $resn_exch{"H183 CCR"}="3H18 CCR"; $resn_exch{"H101 CCR"}="1H10 CCR";
$resn_exch{"H102 CCR"}="2H10 CCR"; $resn_exch{"H103 CCR"}="3H10 CCR"; $resn_exch{"H191 CCR"}="1H19 CCR";
$resn_exch{"H192 CCR"}="2H19 CCR"; $resn_exch{"H193 CCR"}="3H19 CCR"; $resn_exch{"H301 CCR"}="1H30 CCR";
$resn_exch{"H311 CCR"}="1H31 CCR"; $resn_exch{"H312 CCR"}="2H31 CCR"; $resn_exch{"H341 CCR"}="1H34 CCR";
$resn_exch{" H   CCS"}="1HN  CCS"; $resn_exch{" H2  CCS"}="2HN  CCS"; $resn_exch{" HB2 CCS"}="1HB  CCS";
$resn_exch{" HB3 CCS"}="2HB  CCS"; $resn_exch{" HD2 CCS"}="1HD  CCS"; $resn_exch{" HD3 CCS"}="2HD  CCS";
$resn_exch{" HZ2 CCS"}=" HOZ CCS"; $resn_exch{"H251 CCT"}="1H25 CCT"; $resn_exch{"H252 CCT"}="2H25 CCT";
$resn_exch{"H253 CCT"}="3H25 CCT"; $resn_exch{" H43 CCV"}="3H4  CCV"; $resn_exch{" H42 CCV"}="2H4  CCV";
$resn_exch{" H41 CCV"}="1H4  CCV"; $resn_exch{"H112 CCV"}="2H11 CCV"; $resn_exch{"H111 CCV"}="1H11 CCV";
$resn_exch{"H132 CCV"}="2H13 CCV"; $resn_exch{"H131 CCV"}="1H13 CCV"; $resn_exch{"H142 CCV"}="2H14 CCV";
$resn_exch{"H141 CCV"}="1H14 CCV"; $resn_exch{"HA31 CCY"}="1HA3 CCY"; $resn_exch{"HA32 CCY"}="2HA3 CCY";
$resn_exch{"HN11 CCY"}="1HN1 CCY"; $resn_exch{"HN12 CCY"}="2HN1 CCY"; $resn_exch{"HB11 CCY"}="1HB1 CCY";
$resn_exch{"HB12 CCY"}="2HB1 CCY"; $resn_exch{"HB21 CCY"}="1HB2 CCY"; $resn_exch{"HB22 CCY"}="2HB2 CCY";
$resn_exch{"HO11 CD1"}="1HO1 CD1"; $resn_exch{"HO12 CD1"}="2HO1 CD1"; $resn_exch{"HO11 CD3"}="1HO1 CD3";
$resn_exch{"HO12 CD3"}="2HO1 CD3"; $resn_exch{"HO21 CD3"}="1HO2 CD3"; $resn_exch{"HO22 CD3"}="2HO2 CD3";
$resn_exch{"HO31 CD3"}="1HO3 CD3"; $resn_exch{"HO32 CD3"}="2HO3 CD3"; $resn_exch{"HO11 CD5"}="1HO1 CD5";
$resn_exch{"HO12 CD5"}="2HO1 CD5"; $resn_exch{"HO21 CD5"}="1HO2 CD5"; $resn_exch{"HO22 CD5"}="2HO2 CD5";
$resn_exch{"HO31 CD5"}="1HO3 CD5"; $resn_exch{"HO32 CD5"}="2HO3 CD5"; $resn_exch{"HO41 CD5"}="1HO4 CD5";
$resn_exch{"HO42 CD5"}="2HO4 CD5"; $resn_exch{"HO51 CD5"}="1HO5 CD5"; $resn_exch{"HO52 CD5"}="2HO5 CD5";
$resn_exch{"H121 CDA"}="1H12 CDA"; $resn_exch{"H122 CDA"}="2H12 CDA"; $resn_exch{"H291 CDA"}="1H29 CDA";
$resn_exch{"H292 CDA"}="2H29 CDA"; $resn_exch{"H293 CDA"}="3H29 CDA"; $resn_exch{"H331 CDA"}="1H33 CDA";
$resn_exch{"H332 CDA"}="2H33 CDA"; $resn_exch{"H391 CDA"}="1H39 CDA"; $resn_exch{"H392 CDA"}="2H39 CDA";
$resn_exch{"H121 CDB"}="1H12 CDB"; $resn_exch{"H122 CDB"}="2H12 CDB"; $resn_exch{"H291 CDB"}="1H29 CDB";
$resn_exch{"H292 CDB"}="2H29 CDB"; $resn_exch{"H293 CDB"}="3H29 CDB"; $resn_exch{"H331 CDB"}="1H33 CDB";
$resn_exch{"H332 CDB"}="2H33 CDB"; $resn_exch{"H391 CDB"}="1H39 CDB"; $resn_exch{"H392 CDB"}="2H39 CDB";
$resn_exch{" H11 CDB"}="1H1  CDB"; $resn_exch{" H12 CDB"}="2H1  CDB"; $resn_exch{" H13 CDB"}="3H1  CDB";
$resn_exch{" H41 CDC"}="1H4  CDC"; $resn_exch{" H42 CDC"}="2H4  CDC"; $resn_exch{" H1' CDC"}=" H1* CDC";
$resn_exch{" H2' CDC"}=" H2* CDC"; $resn_exch{"HO'2 CDC"}="2HO* CDC"; $resn_exch{" H3' CDC"}=" H3* CDC";
$resn_exch{" H4' CDC"}=" H4* CDC"; $resn_exch{"H5'1 CDC"}="1H5* CDC"; $resn_exch{"H5'2 CDC"}="2H5* CDC";
$resn_exch{"H141 CDC"}="1H14 CDC"; $resn_exch{"H142 CDC"}="2H14 CDC"; $resn_exch{"H151 CDC"}="1H15 CDC";
$resn_exch{"H152 CDC"}="2H15 CDC"; $resn_exch{"H161 CDC"}="1H16 CDC"; $resn_exch{"H162 CDC"}="2H16 CDC";
$resn_exch{"H163 CDC"}="3H16 CDC"; $resn_exch{"H171 CDC"}="1H17 CDC"; $resn_exch{"H172 CDC"}="2H17 CDC";
$resn_exch{"H173 CDC"}="3H17 CDC"; $resn_exch{"H181 CDC"}="1H18 CDC"; $resn_exch{"H182 CDC"}="2H18 CDC";
$resn_exch{"H183 CDC"}="3H18 CDC"; $resn_exch{"H121 CDD"}="1H12 CDD"; $resn_exch{"H122 CDD"}="2H12 CDD";
$resn_exch{"H241 CDD"}="1H24 CDD"; $resn_exch{"H242 CDD"}="2H24 CDD"; $resn_exch{"H291 CDD"}="1H29 CDD";
$resn_exch{"H292 CDD"}="2H29 CDD"; $resn_exch{" H   CDE"}="1HN  CDE"; $resn_exch{" H2  CDE"}="2HN  CDE";
$resn_exch{"HG11 CDE"}="1HG1 CDE"; $resn_exch{"HG12 CDE"}="2HG1 CDE"; $resn_exch{"HG13 CDE"}="3HG1 CDE";
$resn_exch{"HG21 CDE"}="1HG2 CDE"; $resn_exch{"HG22 CDE"}="2HG2 CDE"; $resn_exch{"HG23 CDE"}="3HG2 CDE";
$resn_exch{" HC1 CDE"}="1H   CDE"; $resn_exch{" HC2 CDE"}="2H   CDE"; $resn_exch{" HC3 CDE"}="3H   CDE";
$resn_exch{"H5'1 CDF"}="1H5* CDF"; $resn_exch{"H5'2 CDF"}="2H5* CDF"; $resn_exch{" H4' CDF"}=" H4* CDF";
$resn_exch{" H3' CDF"}=" H3* CDF"; $resn_exch{" H2' CDF"}=" H2* CDF"; $resn_exch{" H1' CDF"}=" H1* CDF";
$resn_exch{"H4N1 CDF"}="1H4N CDF"; $resn_exch{"H4N2 CDF"}="2H4N CDF"; $resn_exch{"H6C1 CDG"}="1H6C CDG";
$resn_exch{"H6C2 CDG"}="2H6C CDG"; $resn_exch{"H9C1 CDG"}="1H9C CDG"; $resn_exch{"H9C2 CDG"}="2H9C CDG";
$resn_exch{"H9C3 CDG"}="3H9C CDG"; $resn_exch{"H101 CDG"}="1H10 CDG"; $resn_exch{"H102 CDG"}="2H10 CDG";
$resn_exch{"H103 CDG"}="3H10 CDG"; $resn_exch{"H371 CDH"}="1H37 CDH"; $resn_exch{"H372 CDH"}="2H37 CDH";
$resn_exch{"H331 CDH"}="1H33 CDH"; $resn_exch{"H332 CDH"}="2H33 CDH"; $resn_exch{"H333 CDH"}="3H33 CDH";
$resn_exch{"H121 CDH"}="1H12 CDH"; $resn_exch{"H161 CDH"}="1H16 CDH"; $resn_exch{"H162 CDH"}="2H16 CDH";
$resn_exch{"H30N CDH"}="NH30 CDH"; $resn_exch{"H241 CDH"}="1H24 CDH"; $resn_exch{"H242 CDH"}="2H24 CDH";
$resn_exch{"H231 CDH"}="1H23 CDH"; $resn_exch{"H232 CDH"}="2H23 CDH"; $resn_exch{"H221 CDH"}="1H22 CDH";
$resn_exch{"H222 CDH"}="2H22 CDH"; $resn_exch{"H211 CDH"}="1H21 CDH"; $resn_exch{"H311 CDH"}="1H31 CDH";
$resn_exch{"H312 CDH"}="2H31 CDH"; $resn_exch{"HC41 CDI"}="1HC4 CDI"; $resn_exch{"HC42 CDI"}="2HC4 CDI";
$resn_exch{"HB2O CDI"}="OHB2 CDI"; $resn_exch{"HA1O CDI"}="OHA1 CDI"; $resn_exch{"HC11 CDI"}="1HC1 CDI";
$resn_exch{"HC12 CDI"}="2HC1 CDI"; $resn_exch{"HC51 CDI"}="1HC5 CDI"; $resn_exch{"HC52 CDI"}="2HC5 CDI";
$resn_exch{"HC53 CDI"}="3HC5 CDI"; $resn_exch{"H1O1 CDL"}="1H1O CDL"; $resn_exch{"HA22 CDL"}="2HA2 CDL";
$resn_exch{"HA21 CDL"}="1HA2 CDL"; $resn_exch{"HA32 CDL"}="2HA3 CDL"; $resn_exch{"HA31 CDL"}="1HA3 CDL";
$resn_exch{"H112 CDL"}="2H11 CDL"; $resn_exch{"H111 CDL"}="1H11 CDL"; $resn_exch{"H122 CDL"}="2H12 CDL";
$resn_exch{"H121 CDL"}="1H12 CDL"; $resn_exch{"H132 CDL"}="2H13 CDL"; $resn_exch{"H131 CDL"}="1H13 CDL";
$resn_exch{"H142 CDL"}="2H14 CDL"; $resn_exch{"H141 CDL"}="1H14 CDL"; $resn_exch{"H152 CDL"}="2H15 CDL";
$resn_exch{"H151 CDL"}="1H15 CDL"; $resn_exch{"H162 CDL"}="2H16 CDL"; $resn_exch{"H161 CDL"}="1H16 CDL";
$resn_exch{"H172 CDL"}="2H17 CDL"; $resn_exch{"H171 CDL"}="1H17 CDL"; $resn_exch{"H182 CDL"}="2H18 CDL";
$resn_exch{"H181 CDL"}="1H18 CDL"; $resn_exch{"H192 CDL"}="2H19 CDL"; $resn_exch{"H191 CDL"}="1H19 CDL";
$resn_exch{"H202 CDL"}="2H20 CDL"; $resn_exch{"H201 CDL"}="1H20 CDL"; $resn_exch{"H212 CDL"}="2H21 CDL";
$resn_exch{"H211 CDL"}="1H21 CDL"; $resn_exch{"H222 CDL"}="2H22 CDL"; $resn_exch{"H221 CDL"}="1H22 CDL";
$resn_exch{"H232 CDL"}="2H23 CDL"; $resn_exch{"H231 CDL"}="1H23 CDL"; $resn_exch{"H242 CDL"}="2H24 CDL";
$resn_exch{"H241 CDL"}="1H24 CDL"; $resn_exch{"H252 CDL"}="2H25 CDL"; $resn_exch{"H251 CDL"}="1H25 CDL";
$resn_exch{"H262 CDL"}="2H26 CDL"; $resn_exch{"H261 CDL"}="1H26 CDL"; $resn_exch{"H273 CDL"}="3H27 CDL";
$resn_exch{"H272 CDL"}="2H27 CDL"; $resn_exch{"H271 CDL"}="1H27 CDL"; $resn_exch{"HA62 CDL"}="2HA6 CDL";
$resn_exch{"HA61 CDL"}="1HA6 CDL"; $resn_exch{"H312 CDL"}="2H31 CDL"; $resn_exch{"H311 CDL"}="1H31 CDL";
$resn_exch{"H322 CDL"}="2H32 CDL"; $resn_exch{"H321 CDL"}="1H32 CDL"; $resn_exch{"H332 CDL"}="2H33 CDL";
$resn_exch{"H331 CDL"}="1H33 CDL"; $resn_exch{"H342 CDL"}="2H34 CDL"; $resn_exch{"H341 CDL"}="1H34 CDL";
$resn_exch{"H352 CDL"}="2H35 CDL"; $resn_exch{"H351 CDL"}="1H35 CDL"; $resn_exch{"H362 CDL"}="2H36 CDL";
$resn_exch{"H361 CDL"}="1H36 CDL"; $resn_exch{"H372 CDL"}="2H37 CDL"; $resn_exch{"H371 CDL"}="1H37 CDL";
$resn_exch{"H382 CDL"}="2H38 CDL"; $resn_exch{"H381 CDL"}="1H38 CDL"; $resn_exch{"H392 CDL"}="2H39 CDL";
$resn_exch{"H391 CDL"}="1H39 CDL"; $resn_exch{"H402 CDL"}="2H40 CDL"; $resn_exch{"H401 CDL"}="1H40 CDL";
$resn_exch{"H412 CDL"}="2H41 CDL"; $resn_exch{"H411 CDL"}="1H41 CDL"; $resn_exch{"H422 CDL"}="2H42 CDL";
$resn_exch{"H421 CDL"}="1H42 CDL"; $resn_exch{"H432 CDL"}="2H43 CDL"; $resn_exch{"H431 CDL"}="1H43 CDL";
$resn_exch{"H442 CDL"}="2H44 CDL"; $resn_exch{"H441 CDL"}="1H44 CDL"; $resn_exch{"H452 CDL"}="2H45 CDL";
$resn_exch{"H451 CDL"}="1H45 CDL"; $resn_exch{"H462 CDL"}="2H46 CDL"; $resn_exch{"H461 CDL"}="1H46 CDL";
$resn_exch{"H473 CDL"}="3H47 CDL"; $resn_exch{"H472 CDL"}="2H47 CDL"; $resn_exch{"H471 CDL"}="1H47 CDL";
$resn_exch{"HB22 CDL"}="2HB2 CDL"; $resn_exch{"HB21 CDL"}="1HB2 CDL"; $resn_exch{"HB32 CDL"}="2HB3 CDL";
$resn_exch{"HB31 CDL"}="1HB3 CDL"; $resn_exch{"H512 CDL"}="2H51 CDL"; $resn_exch{"H511 CDL"}="1H51 CDL";
$resn_exch{"H522 CDL"}="2H52 CDL"; $resn_exch{"H521 CDL"}="1H52 CDL"; $resn_exch{"H532 CDL"}="2H53 CDL";
$resn_exch{"H531 CDL"}="1H53 CDL"; $resn_exch{"H542 CDL"}="2H54 CDL"; $resn_exch{"H541 CDL"}="1H54 CDL";
$resn_exch{"H552 CDL"}="2H55 CDL"; $resn_exch{"H551 CDL"}="1H55 CDL"; $resn_exch{"H562 CDL"}="2H56 CDL";
$resn_exch{"H561 CDL"}="1H56 CDL"; $resn_exch{"H572 CDL"}="2H57 CDL"; $resn_exch{"H571 CDL"}="1H57 CDL";
$resn_exch{"H582 CDL"}="2H58 CDL"; $resn_exch{"H581 CDL"}="1H58 CDL"; $resn_exch{"H592 CDL"}="2H59 CDL";
$resn_exch{"H591 CDL"}="1H59 CDL"; $resn_exch{"H602 CDL"}="2H60 CDL"; $resn_exch{"H601 CDL"}="1H60 CDL";
$resn_exch{"H612 CDL"}="2H61 CDL"; $resn_exch{"H611 CDL"}="1H61 CDL"; $resn_exch{"H622 CDL"}="2H62 CDL";
$resn_exch{"H621 CDL"}="1H62 CDL"; $resn_exch{"H632 CDL"}="2H63 CDL"; $resn_exch{"H631 CDL"}="1H63 CDL";
$resn_exch{"H642 CDL"}="2H64 CDL"; $resn_exch{"H641 CDL"}="1H64 CDL"; $resn_exch{"H652 CDL"}="2H65 CDL";
$resn_exch{"H651 CDL"}="1H65 CDL"; $resn_exch{"H662 CDL"}="2H66 CDL"; $resn_exch{"H661 CDL"}="1H66 CDL";
$resn_exch{"H673 CDL"}="3H67 CDL"; $resn_exch{"H672 CDL"}="2H67 CDL"; $resn_exch{"H671 CDL"}="1H67 CDL";
$resn_exch{"HB62 CDL"}="2HB6 CDL"; $resn_exch{"HB61 CDL"}="1HB6 CDL"; $resn_exch{"H712 CDL"}="2H71 CDL";
$resn_exch{"H711 CDL"}="1H71 CDL"; $resn_exch{"H722 CDL"}="2H72 CDL"; $resn_exch{"H721 CDL"}="1H72 CDL";
$resn_exch{"H732 CDL"}="2H73 CDL"; $resn_exch{"H731 CDL"}="1H73 CDL"; $resn_exch{"H742 CDL"}="2H74 CDL";
$resn_exch{"H741 CDL"}="1H74 CDL"; $resn_exch{"H752 CDL"}="2H75 CDL"; $resn_exch{"H751 CDL"}="1H75 CDL";
$resn_exch{"H762 CDL"}="2H76 CDL"; $resn_exch{"H761 CDL"}="1H76 CDL"; $resn_exch{"H772 CDL"}="2H77 CDL";
$resn_exch{"H771 CDL"}="1H77 CDL"; $resn_exch{"H782 CDL"}="2H78 CDL"; $resn_exch{"H781 CDL"}="1H78 CDL";
$resn_exch{"H792 CDL"}="2H79 CDL"; $resn_exch{"H791 CDL"}="1H79 CDL"; $resn_exch{"H802 CDL"}="2H80 CDL";
$resn_exch{"H801 CDL"}="1H80 CDL"; $resn_exch{"H812 CDL"}="2H81 CDL"; $resn_exch{"H811 CDL"}="1H81 CDL";
$resn_exch{"H822 CDL"}="2H82 CDL"; $resn_exch{"H821 CDL"}="1H82 CDL"; $resn_exch{"H832 CDL"}="2H83 CDL";
$resn_exch{"H831 CDL"}="1H83 CDL"; $resn_exch{"H842 CDL"}="2H84 CDL"; $resn_exch{"H841 CDL"}="1H84 CDL";
$resn_exch{"H852 CDL"}="2H85 CDL"; $resn_exch{"H851 CDL"}="1H85 CDL"; $resn_exch{"H862 CDL"}="2H86 CDL";
$resn_exch{"H861 CDL"}="1H86 CDL"; $resn_exch{"H873 CDL"}="3H87 CDL"; $resn_exch{"H872 CDL"}="2H87 CDL";
$resn_exch{"H871 CDL"}="1H87 CDL"; $resn_exch{" H41 CDM"}="1H4  CDM"; $resn_exch{" H42 CDM"}="2H4  CDM";
$resn_exch{" H1' CDM"}=" H1* CDM"; $resn_exch{" H2' CDM"}=" H2* CDM"; $resn_exch{"HO'2 CDM"}="2HO* CDM";
$resn_exch{" H3' CDM"}=" H3* CDM"; $resn_exch{" H4' CDM"}=" H4* CDM"; $resn_exch{"H5'1 CDM"}="1H5* CDM";
$resn_exch{"H5'2 CDM"}="2H5* CDM"; $resn_exch{"H1M1 CDM"}="1H1M CDM"; $resn_exch{"H1M2 CDM"}="2H1M CDM";
$resn_exch{"H2M1 CDM"}="1H2M CDM"; $resn_exch{"H2M2 CDM"}="2H2M CDM"; $resn_exch{"H4M1 CDM"}="1H4M CDM";
$resn_exch{"H4M2 CDM"}="2H4M CDM"; $resn_exch{"H5M1 CDM"}="1H5M CDM"; $resn_exch{"H5M2 CDM"}="2H5M CDM";
$resn_exch{"H5M3 CDM"}="3H5M CDM"; $resn_exch{"HO11 CDN"}="1HO1 CDN"; $resn_exch{"HB31 CDN"}="1HB3 CDN";
$resn_exch{"HB21 CDN"}="1HB2 CDN"; $resn_exch{"HB22 CDN"}="2HB2 CDN"; $resn_exch{"HC11 CDN"}="1HC1 CDN";
$resn_exch{"HB32 CDN"}="2HB3 CDN"; $resn_exch{"HB33 CDN"}="3HB3 CDN"; $resn_exch{"HB41 CDN"}="1HB4 CDN";
$resn_exch{"HB61 CDN"}="1HB6 CDN"; $resn_exch{"HB62 CDN"}="2HB6 CDN"; $resn_exch{"HB91 CDN"}="1HB9 CDN";
$resn_exch{"HB71 CDN"}="1HB7 CDN"; $resn_exch{"H711 CDN"}="1H71 CDN"; $resn_exch{"H712 CDN"}="2H71 CDN";
$resn_exch{"H721 CDN"}="1H72 CDN"; $resn_exch{"H722 CDN"}="2H72 CDN"; $resn_exch{"H731 CDN"}="1H73 CDN";
$resn_exch{"H732 CDN"}="2H73 CDN"; $resn_exch{"H741 CDN"}="1H74 CDN"; $resn_exch{"H742 CDN"}="2H74 CDN";
$resn_exch{"H751 CDN"}="1H75 CDN"; $resn_exch{"H752 CDN"}="2H75 CDN"; $resn_exch{"H753 CDN"}="3H75 CDN";
$resn_exch{"HB7O CDN"}="OHB7 CDN"; $resn_exch{"HB51 CDN"}="1HB5 CDN"; $resn_exch{"H511 CDN"}="1H51 CDN";
$resn_exch{"H512 CDN"}="2H51 CDN"; $resn_exch{"H521 CDN"}="1H52 CDN"; $resn_exch{"H522 CDN"}="2H52 CDN";
$resn_exch{"H531 CDN"}="1H53 CDN"; $resn_exch{"H532 CDN"}="2H53 CDN"; $resn_exch{"H541 CDN"}="1H54 CDN";
$resn_exch{"H542 CDN"}="2H54 CDN"; $resn_exch{"H551 CDN"}="1H55 CDN"; $resn_exch{"H552 CDN"}="2H55 CDN";
$resn_exch{"H561 CDN"}="1H56 CDN"; $resn_exch{"H562 CDN"}="2H56 CDN"; $resn_exch{"H571 CDN"}="1H57 CDN";
$resn_exch{"H572 CDN"}="2H57 CDN"; $resn_exch{"H581 CDN"}="1H58 CDN"; $resn_exch{"H582 CDN"}="2H58 CDN";
$resn_exch{"H591 CDN"}="1H59 CDN"; $resn_exch{"H592 CDN"}="2H59 CDN"; $resn_exch{"H601 CDN"}="1H60 CDN";
$resn_exch{"H602 CDN"}="2H60 CDN"; $resn_exch{"H611 CDN"}="1H61 CDN"; $resn_exch{"H612 CDN"}="2H61 CDN";
$resn_exch{"H621 CDN"}="1H62 CDN"; $resn_exch{"H622 CDN"}="2H62 CDN"; $resn_exch{"H631 CDN"}="1H63 CDN";
$resn_exch{"H632 CDN"}="2H63 CDN"; $resn_exch{"H641 CDN"}="1H64 CDN"; $resn_exch{"H642 CDN"}="2H64 CDN";
$resn_exch{"H643 CDN"}="3H64 CDN"; $resn_exch{"HA3O CDN"}="OHA3 CDN"; $resn_exch{"HA21 CDN"}="1HA2 CDN";
$resn_exch{"HA22 CDN"}="2HA2 CDN"; $resn_exch{"HA31 CDN"}="1HA3 CDN"; $resn_exch{"HA32 CDN"}="2HA3 CDN";
$resn_exch{"HA41 CDN"}="1HA4 CDN"; $resn_exch{"HA61 CDN"}="1HA6 CDN"; $resn_exch{"HA62 CDN"}="2HA6 CDN";
$resn_exch{"HA91 CDN"}="1HA9 CDN"; $resn_exch{"HA71 CDN"}="1HA7 CDN"; $resn_exch{"H311 CDN"}="1H31 CDN";
$resn_exch{"H312 CDN"}="2H31 CDN"; $resn_exch{"H321 CDN"}="1H32 CDN"; $resn_exch{"H322 CDN"}="2H32 CDN";
$resn_exch{"H331 CDN"}="1H33 CDN"; $resn_exch{"H332 CDN"}="2H33 CDN"; $resn_exch{"H341 CDN"}="1H34 CDN";
$resn_exch{"H342 CDN"}="2H34 CDN"; $resn_exch{"H351 CDN"}="1H35 CDN"; $resn_exch{"H352 CDN"}="2H35 CDN";
$resn_exch{"H361 CDN"}="1H36 CDN"; $resn_exch{"H362 CDN"}="2H36 CDN"; $resn_exch{"H371 CDN"}="1H37 CDN";
$resn_exch{"H372 CDN"}="2H37 CDN"; $resn_exch{"H381 CDN"}="1H38 CDN"; $resn_exch{"H382 CDN"}="2H38 CDN";
$resn_exch{"H391 CDN"}="1H39 CDN"; $resn_exch{"H392 CDN"}="2H39 CDN"; $resn_exch{"H401 CDN"}="1H40 CDN";
$resn_exch{"H402 CDN"}="2H40 CDN"; $resn_exch{"H411 CDN"}="1H41 CDN"; $resn_exch{"H412 CDN"}="2H41 CDN";
$resn_exch{"H421 CDN"}="1H42 CDN"; $resn_exch{"H422 CDN"}="2H42 CDN"; $resn_exch{"H423 CDN"}="3H42 CDN";
$resn_exch{"HA7O CDN"}="OHA7 CDN"; $resn_exch{"HA51 CDN"}="1HA5 CDN"; $resn_exch{"H111 CDN"}="1H11 CDN";
$resn_exch{"H112 CDN"}="2H11 CDN"; $resn_exch{"H121 CDN"}="1H12 CDN"; $resn_exch{"H122 CDN"}="2H12 CDN";
$resn_exch{"H131 CDN"}="1H13 CDN"; $resn_exch{"H132 CDN"}="2H13 CDN"; $resn_exch{"H141 CDN"}="1H14 CDN";
$resn_exch{"H142 CDN"}="2H14 CDN"; $resn_exch{"H151 CDN"}="1H15 CDN"; $resn_exch{"H152 CDN"}="2H15 CDN";
$resn_exch{"H161 CDN"}="1H16 CDN"; $resn_exch{"H162 CDN"}="2H16 CDN"; $resn_exch{"H171 CDN"}="1H17 CDN";
$resn_exch{"H172 CDN"}="2H17 CDN"; $resn_exch{"H181 CDN"}="1H18 CDN"; $resn_exch{"H182 CDN"}="2H18 CDN";
$resn_exch{"H191 CDN"}="1H19 CDN"; $resn_exch{"H192 CDN"}="2H19 CDN"; $resn_exch{"H201 CDN"}="1H20 CDN";
$resn_exch{"H202 CDN"}="2H20 CDN"; $resn_exch{"H211 CDN"}="1H21 CDN"; $resn_exch{"H212 CDN"}="2H21 CDN";
$resn_exch{"H221 CDN"}="1H22 CDN"; $resn_exch{"H222 CDN"}="2H22 CDN"; $resn_exch{"H231 CDN"}="1H23 CDN";
$resn_exch{"H232 CDN"}="2H23 CDN"; $resn_exch{"H241 CDN"}="1H24 CDN"; $resn_exch{"H242 CDN"}="2H24 CDN";
$resn_exch{"H243 CDN"}="3H24 CDN"; $resn_exch{" H71 CDO"}="1H7  CDO"; $resn_exch{" H72 CDO"}="2H7  CDO";
$resn_exch{"H101 CDO"}="1H10 CDO"; $resn_exch{"H102 CDO"}="2H10 CDO"; $resn_exch{"H171 CDO"}="1H17 CDO";
$resn_exch{"H172 CDO"}="2H17 CDO"; $resn_exch{"H231 CDO"}="1H23 CDO"; $resn_exch{"H232 CDO"}="2H23 CDO";
$resn_exch{"H271 CDO"}="1H27 CDO"; $resn_exch{"H272 CDO"}="2H27 CDO"; $resn_exch{"H291 CDO"}="1H29 CDO";
$resn_exch{"H292 CDO"}="2H29 CDO"; $resn_exch{"H301 CDO"}="1H30 CDO"; $resn_exch{"H302 CDO"}="2H30 CDO";
$resn_exch{"H311 CDO"}="1H31 CDO"; $resn_exch{"H312 CDO"}="2H31 CDO"; $resn_exch{"H321 CDO"}="1H32 CDO";
$resn_exch{"H322 CDO"}="2H32 CDO"; $resn_exch{"H331 CDO"}="1H33 CDO"; $resn_exch{"H332 CDO"}="2H33 CDO";
$resn_exch{" H41 CDP"}="1H4  CDP"; $resn_exch{" H42 CDP"}="2H4  CDP"; $resn_exch{" H1' CDP"}=" H1* CDP";
$resn_exch{" H2' CDP"}=" H2* CDP"; $resn_exch{"HO'2 CDP"}="2HO* CDP"; $resn_exch{" H3' CDP"}=" H3* CDP";
$resn_exch{" H4' CDP"}=" H4* CDP"; $resn_exch{"H5'1 CDP"}="1H5* CDP"; $resn_exch{"H5'2 CDP"}="2H5* CDP";
$resn_exch{" H21 CDR"}="1H2  CDR"; $resn_exch{" H22 CDR"}="2H2  CDR"; $resn_exch{" H31 CDR"}="1H3  CDR";
$resn_exch{" H32 CDR"}="2H3  CDR"; $resn_exch{" H61 CDR"}="1H6  CDR"; $resn_exch{" H62 CDR"}="2H6  CDR";
$resn_exch{" H63 CDR"}="3H6  CDR"; $resn_exch{"H331 CDS"}="1H33 CDS"; $resn_exch{"H332 CDS"}="2H33 CDS";
$resn_exch{"H271 CDS"}="1H27 CDS"; $resn_exch{"H272 CDS"}="2H27 CDS"; $resn_exch{"H273 CDS"}="3H27 CDS";
$resn_exch{" H21 CDS"}="1H2  CDS"; $resn_exch{" H22 CDS"}="2H2  CDS"; $resn_exch{"H201 CDS"}="1H20 CDS";
$resn_exch{"H202 CDS"}="2H20 CDS"; $resn_exch{"H203 CDS"}="3H20 CDS"; $resn_exch{" HT1 CDT"}="1HT  CDT";
$resn_exch{" HT2 CDT"}="2HT  CDT"; $resn_exch{" HN1 CDT"}="1HN  CDT"; $resn_exch{" HB1 CDT"}="1HB  CDT";
$resn_exch{" HB2 CDT"}="2HB  CDT"; $resn_exch{" HG1 CDT"}="1HG  CDT"; $resn_exch{" HG2 CDT"}="2HG  CDT";
$resn_exch{" HE1 CDT"}="1HE  CDT"; $resn_exch{" HE2 CDT"}="2HE  CDT"; $resn_exch{" HE3 CDT"}="3HE  CDT";
$resn_exch{"HC11 CDU"}="1HC1 CDU"; $resn_exch{"HC12 CDU"}="2HC1 CDU"; $resn_exch{"HC21 CDU"}="1HC2 CDU";
$resn_exch{"HC22 CDU"}="2HC2 CDU"; $resn_exch{"HC31 CDU"}="1HC3 CDU"; $resn_exch{"HC32 CDU"}="2HC3 CDU";
$resn_exch{"HC41 CDU"}="1HC4 CDU"; $resn_exch{"HC42 CDU"}="2HC4 CDU"; $resn_exch{"HC51 CDU"}="1HC5 CDU";
$resn_exch{"HC61 CDU"}="1HC6 CDU"; $resn_exch{"HC62 CDU"}="2HC6 CDU"; $resn_exch{"H111 CDU"}="1H11 CDU";
$resn_exch{"H112 CDU"}="2H11 CDU"; $resn_exch{"H121 CDU"}="1H12 CDU"; $resn_exch{"H122 CDU"}="2H12 CDU";
$resn_exch{"H131 CDU"}="1H13 CDU"; $resn_exch{"H132 CDU"}="2H13 CDU"; $resn_exch{"H141 CDU"}="1H14 CDU";
$resn_exch{"H142 CDU"}="2H14 CDU"; $resn_exch{"H151 CDU"}="1H15 CDU"; $resn_exch{"H152 CDU"}="2H15 CDU";
$resn_exch{"H161 CDU"}="1H16 CDU"; $resn_exch{"H162 CDU"}="2H16 CDU"; $resn_exch{"H171 CDU"}="1H17 CDU";
$resn_exch{"H172 CDU"}="2H17 CDU"; $resn_exch{"H181 CDU"}="1H18 CDU"; $resn_exch{"H182 CDU"}="2H18 CDU";
$resn_exch{"H191 CDU"}="1H19 CDU"; $resn_exch{"H192 CDU"}="2H19 CDU"; $resn_exch{"H201 CDU"}="1H20 CDU";
$resn_exch{"H202 CDU"}="2H20 CDU"; $resn_exch{"H203 CDU"}="3H20 CDU"; $resn_exch{" HT1 CDV"}="1HT  CDV";
$resn_exch{" HT2 CDV"}="2HT  CDV"; $resn_exch{" HN1 CDV"}="1HN  CDV"; $resn_exch{"HG11 CDV"}="1HG1 CDV";
$resn_exch{"HG12 CDV"}="2HG1 CDV"; $resn_exch{"HG13 CDV"}="3HG1 CDV"; $resn_exch{"HG21 CDV"}="1HG2 CDV";
$resn_exch{"HG22 CDV"}="2HG2 CDV"; $resn_exch{"HG23 CDV"}="3HG2 CDV"; $resn_exch{"HAD1 CDX"}="1HAD CDX";
$resn_exch{"HAD2 CDX"}="2HAD CDX"; $resn_exch{"HAQ1 CDX"}="1HAQ CDX"; $resn_exch{"HAQ2 CDX"}="2HAQ CDX";
$resn_exch{"HAK1 CDX"}="1HAK CDX"; $resn_exch{"HAK2 CDX"}="2HAK CDX"; $resn_exch{"HAE1 CDX"}="1HAE CDX";
$resn_exch{"HAE2 CDX"}="2HAE CDX"; $resn_exch{"HAE3 CDX"}="3HAE CDX"; $resn_exch{"HAR1 CDX"}="1HAR CDX";
$resn_exch{"HAR2 CDX"}="2HAR CDX"; $resn_exch{"HAF1 CDX"}="1HAF CDX"; $resn_exch{"HAF2 CDX"}="2HAF CDX";
$resn_exch{"HC51 CDY"}="1HC5 CDY"; $resn_exch{"HC52 CDY"}="2HC5 CDY"; $resn_exch{" H4' CDY"}=" H4* CDY";
$resn_exch{"HC31 CDY"}="1HC3 CDY"; $resn_exch{"HC32 CDY"}="2HC3 CDY"; $resn_exch{"HC21 CDY"}="1HC2 CDY";
$resn_exch{"HC22 CDY"}="2HC2 CDY"; $resn_exch{" H1' CDY"}=" H1* CDY"; $resn_exch{"HN61 CDY"}="1HN6 CDY";
$resn_exch{"HN62 CDY"}="2HN6 CDY"; $resn_exch{" H11 CE1"}="1H1  CE1"; $resn_exch{" H12 CE1"}="2H1  CE1";
$resn_exch{" H13 CE1"}="3H1  CE1"; $resn_exch{" H21 CE1"}="1H2  CE1"; $resn_exch{" H22 CE1"}="2H2  CE1";
$resn_exch{" H31 CE1"}="1H3  CE1"; $resn_exch{" H32 CE1"}="2H3  CE1"; $resn_exch{" H41 CE1"}="1H4  CE1";
$resn_exch{" H42 CE1"}="2H4  CE1"; $resn_exch{" H51 CE1"}="1H5  CE1"; $resn_exch{" H52 CE1"}="2H5  CE1";
$resn_exch{" H61 CE1"}="1H6  CE1"; $resn_exch{" H62 CE1"}="2H6  CE1"; $resn_exch{" H71 CE1"}="1H7  CE1";
$resn_exch{" H72 CE1"}="2H7  CE1"; $resn_exch{" H81 CE1"}="1H8  CE1"; $resn_exch{" H82 CE1"}="2H8  CE1";
$resn_exch{" H91 CE1"}="1H9  CE1"; $resn_exch{" H92 CE1"}="2H9  CE1"; $resn_exch{"H101 CE1"}="1H10 CE1";
$resn_exch{"H102 CE1"}="2H10 CE1"; $resn_exch{"H111 CE1"}="1H11 CE1"; $resn_exch{"H112 CE1"}="2H11 CE1";
$resn_exch{"H121 CE1"}="1H12 CE1"; $resn_exch{"H122 CE1"}="2H12 CE1"; $resn_exch{"H141 CE1"}="1H14 CE1";
$resn_exch{"H142 CE1"}="2H14 CE1"; $resn_exch{"H151 CE1"}="1H15 CE1"; $resn_exch{"H152 CE1"}="2H15 CE1";
$resn_exch{"H171 CE1"}="1H17 CE1"; $resn_exch{"H172 CE1"}="2H17 CE1"; $resn_exch{"H181 CE1"}="1H18 CE1";
$resn_exch{"H182 CE1"}="2H18 CE1"; $resn_exch{"H201 CE1"}="1H20 CE1"; $resn_exch{"H202 CE1"}="2H20 CE1";
$resn_exch{"H211 CE1"}="1H21 CE1"; $resn_exch{"H212 CE1"}="2H21 CE1"; $resn_exch{"H231 CE1"}="1H23 CE1";
$resn_exch{"H232 CE1"}="2H23 CE1"; $resn_exch{"H241 CE1"}="1H24 CE1"; $resn_exch{"H242 CE1"}="2H24 CE1";
$resn_exch{"H261 CE1"}="1H26 CE1"; $resn_exch{"H262 CE1"}="2H26 CE1"; $resn_exch{"H271 CE1"}="1H27 CE1";
$resn_exch{"H272 CE1"}="2H27 CE1"; $resn_exch{"H291 CE1"}="1H29 CE1"; $resn_exch{"H292 CE1"}="2H29 CE1";
$resn_exch{"H301 CE1"}="1H30 CE1"; $resn_exch{"H302 CE1"}="2H30 CE1"; $resn_exch{"H321 CE1"}="1H32 CE1";
$resn_exch{"H322 CE1"}="2H32 CE1"; $resn_exch{"H331 CE1"}="1H33 CE1"; $resn_exch{"H332 CE1"}="2H33 CE1";
$resn_exch{"H351 CE1"}="1H35 CE1"; $resn_exch{"H352 CE1"}="2H35 CE1"; $resn_exch{"H361 CE1"}="1H36 CE1";
$resn_exch{"H362 CE1"}="2H36 CE1"; $resn_exch{"H121 CE2"}="1H12 CE2"; $resn_exch{"H122 CE2"}="2H12 CE2";
$resn_exch{"H123 CE2"}="3H12 CE2"; $resn_exch{" H61 CE2"}="1H6  CE2"; $resn_exch{" H62 CE2"}="2H6  CE2";
$resn_exch{"H241 CE2"}="1H24 CE2"; $resn_exch{"H242 CE2"}="2H24 CE2"; $resn_exch{"H243 CE2"}="3H24 CE2";
$resn_exch{"H131 CE2"}="1H13 CE2"; $resn_exch{"H132 CE2"}="2H13 CE2"; $resn_exch{"H133 CE2"}="3H13 CE2";
$resn_exch{"H141 CE2"}="1H14 CE2"; $resn_exch{"H142 CE2"}="2H14 CE2"; $resn_exch{"H143 CE2"}="3H14 CE2";
$resn_exch{" H21 CE3"}="1H2  CE3"; $resn_exch{" H22 CE3"}="2H2  CE3"; $resn_exch{" H81 CE3"}="1H8  CE3";
$resn_exch{" H82 CE3"}="2H8  CE3"; $resn_exch{" H83 CE3"}="3H8  CE3"; $resn_exch{" H41 CE3"}="1H4  CE3";
$resn_exch{" H42 CE3"}="2H4  CE3"; $resn_exch{" H91 CE3"}="1H9  CE3"; $resn_exch{" H92 CE3"}="2H9  CE3";
$resn_exch{"H111 CE3"}="1H11 CE3"; $resn_exch{"H112 CE3"}="2H11 CE3"; $resn_exch{"H113 CE3"}="3H11 CE3";
$resn_exch{"H6E1 CE5"}="1H6E CE5"; $resn_exch{"H6E2 CE5"}="2H6E CE5"; $resn_exch{"H6D1 CE5"}="1H6D CE5";
$resn_exch{"H6D2 CE5"}="2H6D CE5"; $resn_exch{"H6C1 CE5"}="1H6C CE5"; $resn_exch{"H6C2 CE5"}="2H6C CE5";
$resn_exch{"H6B1 CE5"}="1H6B CE5"; $resn_exch{"H6B2 CE5"}="2H6B CE5"; $resn_exch{"H6A1 CE5"}="1H6A CE5";
$resn_exch{"H6A2 CE5"}="2H6A CE5"; $resn_exch{"H6A1 CE6"}="1H6A CE6"; $resn_exch{"H6A2 CE6"}="2H6A CE6";
$resn_exch{"H6B1 CE6"}="1H6B CE6"; $resn_exch{"H6B2 CE6"}="2H6B CE6"; $resn_exch{"H6C1 CE6"}="1H6C CE6";
$resn_exch{"H6C2 CE6"}="2H6C CE6"; $resn_exch{"H6D1 CE6"}="1H6D CE6"; $resn_exch{"H6D2 CE6"}="2H6D CE6";
$resn_exch{"H6F1 CE6"}="1H6F CE6"; $resn_exch{"H6F2 CE6"}="2H6F CE6"; $resn_exch{"H6E1 CE6"}="1H6E CE6";
$resn_exch{"H6E2 CE6"}="2H6E CE6"; $resn_exch{"H6H1 CE8"}="1H6H CE8"; $resn_exch{"H6H2 CE8"}="2H6H CE8";
$resn_exch{"H6G1 CE8"}="1H6G CE8"; $resn_exch{"H6G2 CE8"}="2H6G CE8"; $resn_exch{"H6F1 CE8"}="1H6F CE8";
$resn_exch{"H6F2 CE8"}="2H6F CE8"; $resn_exch{"H6E1 CE8"}="1H6E CE8"; $resn_exch{"H6E2 CE8"}="2H6E CE8";
$resn_exch{"H6D1 CE8"}="1H6D CE8"; $resn_exch{"H6D2 CE8"}="2H6D CE8"; $resn_exch{"H6C1 CE8"}="1H6C CE8";
$resn_exch{"H6C2 CE8"}="2H6C CE8"; $resn_exch{"H6B1 CE8"}="1H6B CE8"; $resn_exch{"H6B2 CE8"}="2H6B CE8";
$resn_exch{"H6A1 CE8"}="1H6A CE8"; $resn_exch{"H6A2 CE8"}="2H6A CE8"; $resn_exch{" HN1 CEA"}="1HN  CEA";
$resn_exch{" HN2 CEA"}="2HN  CEA"; $resn_exch{" HB1 CEA"}="1HB  CEA"; $resn_exch{" HB2 CEA"}="2HB  CEA";
$resn_exch{"H101 CEB"}="1H10 CEB"; $resn_exch{"H102 CEB"}="2H10 CEB"; $resn_exch{"H111 CEB"}="1H11 CEB";
$resn_exch{"H112 CEB"}="2H11 CEB"; $resn_exch{"H113 CEB"}="3H11 CEB"; $resn_exch{"H121 CEB"}="1H12 CEB";
$resn_exch{"H122 CEB"}="2H12 CEB"; $resn_exch{"H123 CEB"}="3H12 CEB"; $resn_exch{"H141 CEB"}="1H14 CEB";
$resn_exch{"H142 CEB"}="2H14 CEB"; $resn_exch{"H151 CEB"}="1H15 CEB"; $resn_exch{"H152 CEB"}="2H15 CEB";
$resn_exch{"H153 CEB"}="3H15 CEB"; $resn_exch{" H11 CEC"}="1H1  CEC"; $resn_exch{" H21 CEC"}="1H2  CEC";
$resn_exch{" H22 CEC"}="2H2  CEC"; $resn_exch{" H31 CEC"}="1H3  CEC"; $resn_exch{" H32 CEC"}="2H3  CEC";
$resn_exch{" H21 CED"}="1H2  CED"; $resn_exch{" H22 CED"}="2H2  CED"; $resn_exch{"H3'1 CED"}="1H3' CED";
$resn_exch{"H3'2 CED"}="2H3' CED"; $resn_exch{"H3'3 CED"}="3H3' CED"; $resn_exch{"H131 CED"}="1H13 CED";
$resn_exch{"H132 CED"}="2H13 CED"; $resn_exch{" H21 CEF"}="1H2  CEF"; $resn_exch{" H22 CEF"}="2H2  CEF";
$resn_exch{"H3'1 CEF"}="1H3' CEF"; $resn_exch{"H3'2 CEF"}="2H3' CEF"; $resn_exch{"H1'1 CEF"}="1H1' CEF";
$resn_exch{"H1'2 CEF"}="2H1' CEF"; $resn_exch{"H1'3 CEF"}="3H1' CEF"; $resn_exch{" H88 CEF"}="8H8  CEF";
$resn_exch{"H181 CEF"}="1H18 CEF"; $resn_exch{"H182 CEF"}="2H18 CEF"; $resn_exch{"H183 CEF"}="3H18 CEF";
$resn_exch{"HN81 CEF"}="1HN8 CEF"; $resn_exch{"HN82 CEF"}="2HN8 CEF"; $resn_exch{"HOB8 CEG"}="8HOB CEG";
$resn_exch{" H11 CEH"}="1H1  CEH"; $resn_exch{"H12A CEH"}="2H1  CEH"; $resn_exch{" H51 CEH"}="1H5  CEH";
$resn_exch{" H52 CEH"}="2H5  CEH"; $resn_exch{"H101 CEH"}="1H10 CEH"; $resn_exch{"H102 CEH"}="2H10 CEH";
$resn_exch{"H171 CEH"}="1H17 CEH"; $resn_exch{"H172 CEH"}="2H17 CEH"; $resn_exch{"H201 CEH"}="1H20 CEH";
$resn_exch{"H202 CEH"}="2H20 CEH"; $resn_exch{"H251 CEH"}="1H25 CEH"; $resn_exch{"H252 CEH"}="2H25 CEH";
$resn_exch{"H253 CEH"}="3H25 CEH"; $resn_exch{"H281 CEH"}="1H28 CEH"; $resn_exch{"H282 CEH"}="2H28 CEH";
$resn_exch{"H283 CEH"}="3H28 CEH"; $resn_exch{"H211 CEH"}="1H21 CEH"; $resn_exch{"H212 CEH"}="2H21 CEH";
$resn_exch{"H321 CEH"}="1H32 CEH"; $resn_exch{"H322 CEH"}="2H32 CEH"; $resn_exch{"H323 CEH"}="3H32 CEH";
$resn_exch{"H101 CEI"}="1H10 CEI"; $resn_exch{"H102 CEI"}="2H10 CEI"; $resn_exch{"H261 CEI"}="1H26 CEI";
$resn_exch{"H262 CEI"}="2H26 CEI"; $resn_exch{"H1C1 CEJ"}="1H1C CEJ"; $resn_exch{"H1C2 CEJ"}="2H1C CEJ";
$resn_exch{"H5C1 CEJ"}="1H5C CEJ"; $resn_exch{"H5C2 CEJ"}="2H5C CEJ"; $resn_exch{"H101 CEJ"}="1H10 CEJ";
$resn_exch{"H102 CEJ"}="2H10 CEJ"; $resn_exch{"H14C CEL"}="CH14 CEL"; $resn_exch{"H13C CEL"}="CH13 CEL";
$resn_exch{"H17C CEL"}="CH17 CEL"; $resn_exch{"H16C CEL"}="CH16 CEL"; $resn_exch{"H10C CEL"}="CH10 CEL";
$resn_exch{"H111 CEL"}="1H11 CEL"; $resn_exch{"H112 CEL"}="2H11 CEL"; $resn_exch{"H113 CEL"}="3H11 CEL";
$resn_exch{"HN31 CEL"}="1HN3 CEL"; $resn_exch{"HN32 CEL"}="2HN3 CEL"; $resn_exch{" H91 CEM"}="1H9  CEM";
$resn_exch{" H92 CEM"}="2H9  CEM"; $resn_exch{"HO14 CEM"}="4HO1 CEM"; $resn_exch{"HC21 CEN"}="1HC2 CEN";
$resn_exch{"HC22 CEN"}="2HC2 CEN"; $resn_exch{"H131 CEN"}="1H13 CEN"; $resn_exch{"H132 CEN"}="2H13 CEN";
$resn_exch{"H171 CEN"}="1H17 CEN"; $resn_exch{"H172 CEN"}="2H17 CEN"; $resn_exch{" HC1 CEN"}="1HC  CEN";
$resn_exch{" HC2 CEN"}="2HC  CEN"; $resn_exch{" HC3 CEN"}="3HC  CEN"; $resn_exch{"HC51 CEN"}="1HC5 CEN";
$resn_exch{"HC52 CEN"}="2HC5 CEN"; $resn_exch{"HC91 CEN"}="1HC9 CEN"; $resn_exch{"HC92 CEN"}="2HC9 CEN";
$resn_exch{"HC21 CEO"}="1HC2 CEO"; $resn_exch{"HC22 CEO"}="2HC2 CEO"; $resn_exch{"H3'1 CEO"}="1H3' CEO";
$resn_exch{"H3'2 CEO"}="2H3' CEO"; $resn_exch{"H131 CEO"}="1H13 CEO"; $resn_exch{"H132 CEO"}="2H13 CEO";
$resn_exch{" H21 CEP"}="1H2  CEP"; $resn_exch{" H22 CEP"}="2H2  CEP"; $resn_exch{"H3'1 CEP"}="1H3' CEP";
$resn_exch{"H3'2 CEP"}="2H3' CEP"; $resn_exch{"H1'1 CEP"}="1H1' CEP"; $resn_exch{"H1'2 CEP"}="2H1' CEP";
$resn_exch{"H1'3 CEP"}="3H1' CEP"; $resn_exch{" H88 CEP"}="8H8  CEP"; $resn_exch{"H131 CEP"}="1H13 CEP";
$resn_exch{"H132 CEP"}="2H13 CEP"; $resn_exch{" H21 CEQ"}="1H2  CEQ"; $resn_exch{" H22 CEQ"}="2H2  CEQ";
$resn_exch{" H23 CEQ"}="3H2  CEQ"; $resn_exch{" H31 CEQ"}="1H3  CEQ"; $resn_exch{" H32 CEQ"}="2H3  CEQ";
$resn_exch{" H33 CEQ"}="3H3  CEQ"; $resn_exch{" H41 CEQ"}="1H4  CEQ"; $resn_exch{" H42 CEQ"}="2H4  CEQ";
$resn_exch{" H43 CEQ"}="3H4  CEQ"; $resn_exch{" H51 CEQ"}="1H5  CEQ"; $resn_exch{" H52 CEQ"}="2H5  CEQ";
$resn_exch{" H61 CEQ"}="1H6  CEQ"; $resn_exch{" H62 CEQ"}="2H6  CEQ"; $resn_exch{" H63 CEQ"}="3H6  CEQ";
$resn_exch{"HN11 CER"}="1HN1 CER"; $resn_exch{"HN12 CER"}="2HN1 CER"; $resn_exch{" H21 CER"}="1H2  CER";
$resn_exch{" H22 CER"}="2H2  CER"; $resn_exch{" H31 CER"}="1H3  CER"; $resn_exch{" H51 CER"}="1H5  CER";
$resn_exch{" H52 CER"}="2H5  CER"; $resn_exch{" H61 CER"}="1H6  CER"; $resn_exch{" H62 CER"}="2H6  CER";
$resn_exch{" H71 CER"}="1H7  CER"; $resn_exch{" H81 CER"}="1H8  CER"; $resn_exch{" H91 CER"}="1H9  CER";
$resn_exch{" H92 CER"}="2H9  CER"; $resn_exch{"H101 CER"}="1H10 CER"; $resn_exch{"H111 CER"}="1H11 CER";
$resn_exch{"H121 CER"}="1H12 CER"; $resn_exch{"H122 CER"}="2H12 CER"; $resn_exch{"H123 CER"}="3H12 CER";
$resn_exch{"H183 CES"}="3H18 CES"; $resn_exch{"H182 CES"}="2H18 CES"; $resn_exch{"H181 CES"}="1H18 CES";
$resn_exch{"H3'3 CES"}="3H3' CES"; $resn_exch{"H3'2 CES"}="2H3' CES"; $resn_exch{"H3'1 CES"}="1H3' CES";
$resn_exch{" H21 CES"}="1H2  CES"; $resn_exch{" H22 CES"}="2H2  CES"; $resn_exch{"H101 CET"}="1H10 CET";
$resn_exch{"H102 CET"}="2H10 CET"; $resn_exch{"H111 CET"}="1H11 CET"; $resn_exch{"H112 CET"}="2H11 CET";
$resn_exch{"H113 CET"}="3H11 CET"; $resn_exch{"H121 CET"}="1H12 CET"; $resn_exch{"H122 CET"}="2H12 CET";
$resn_exch{"H123 CET"}="3H12 CET"; $resn_exch{"H141 CET"}="1H14 CET"; $resn_exch{"H142 CET"}="2H14 CET";
$resn_exch{"H151 CET"}="1H15 CET"; $resn_exch{"H152 CET"}="2H15 CET"; $resn_exch{"H153 CET"}="3H15 CET";
$resn_exch{"H101 CF3"}="1H10 CF3"; $resn_exch{"H102 CF3"}="2H10 CF3"; $resn_exch{" H81 CF3"}="1H8  CF3";
$resn_exch{" H82 CF3"}="2H8  CF3"; $resn_exch{" H61 CF3"}="1H6  CF3"; $resn_exch{" H62 CF3"}="2H6  CF3";
$resn_exch{" H71 CF3"}="1H7  CF3"; $resn_exch{" H72 CF3"}="2H7  CF3"; $resn_exch{" H91 CF3"}="1H9  CF3";
$resn_exch{" H92 CF3"}="2H9  CF3"; $resn_exch{"H111 CF3"}="1H11 CF3"; $resn_exch{"H112 CF3"}="2H11 CF3";
$resn_exch{"H5S1 CF5"}="1H5S CF5"; $resn_exch{"H5S2 CF5"}="2H5S CF5"; $resn_exch{" H71 CF5"}="1H7  CF5";
$resn_exch{" H72 CF5"}="2H7  CF5"; $resn_exch{"H2C1 CFA"}="1H2C CFA"; $resn_exch{"H2C2 CFA"}="2H2C CFA";
$resn_exch{"H5'1 CFB"}="1H5' CFB"; $resn_exch{"H5'2 CFB"}="2H5' CFB"; $resn_exch{"HN61 CFB"}="1HN6 CFB";
$resn_exch{"HN62 CFB"}="2HN6 CFB"; $resn_exch{"H121 CFC"}="1H12 CFC"; $resn_exch{"H122 CFC"}="2H12 CFC";
$resn_exch{"H123 CFC"}="3H12 CFC"; $resn_exch{"H131 CFC"}="1H13 CFC"; $resn_exch{"H132 CFC"}="2H13 CFC";
$resn_exch{"H133 CFC"}="3H13 CFC"; $resn_exch{"H101 CFF"}="1H10 CFF"; $resn_exch{"H102 CFF"}="2H10 CFF";
$resn_exch{"H103 CFF"}="3H10 CFF"; $resn_exch{"H121 CFF"}="1H12 CFF"; $resn_exch{"H122 CFF"}="2H12 CFF";
$resn_exch{"H123 CFF"}="3H12 CFF"; $resn_exch{" H81 CFF"}="1H8  CFF"; $resn_exch{"H141 CFF"}="1H14 CFF";
$resn_exch{"H142 CFF"}="2H14 CFF"; $resn_exch{"H143 CFF"}="3H14 CFF"; $resn_exch{"H5'1 CFL"}="1H5* CFL";
$resn_exch{"H5'2 CFL"}="2H5* CFL"; $resn_exch{" H4' CFL"}=" H4* CFL"; $resn_exch{" H3' CFL"}=" H3* CFL";
$resn_exch{"HO3' CFL"}="HO3* CFL"; $resn_exch{" H2' CFL"}=" H2* CFL"; $resn_exch{" H1' CFL"}=" H1* CFL";
$resn_exch{"HN41 CFL"}="1HN4 CFL"; $resn_exch{"HN42 CFL"}="2HN4 CFL"; $resn_exch{"H1C1 CFQ"}="1H1C CFQ";
$resn_exch{"H1C2 CFQ"}="2H1C CFQ"; $resn_exch{"H1C3 CFQ"}="3H1C CFQ"; $resn_exch{"H2C1 CFQ"}="1H2C CFQ";
$resn_exch{"H2C2 CFQ"}="2H2C CFQ"; $resn_exch{"H2C3 CFQ"}="3H2C CFQ"; $resn_exch{"H3C1 CFQ"}="1H3C CFQ";
$resn_exch{"H3C2 CFQ"}="2H3C CFQ"; $resn_exch{"H3C3 CFQ"}="3H3C CFQ"; $resn_exch{"H4C1 CFQ"}="1H4C CFQ";
$resn_exch{"H4C2 CFQ"}="2H4C CFQ"; $resn_exch{"H5C1 CFQ"}="1H5C CFQ"; $resn_exch{"H5C2 CFQ"}="2H5C CFQ";
$resn_exch{" HN1 CFS"}="1HN  CFS"; $resn_exch{" HN2 CFS"}="2HN  CFS"; $resn_exch{" HB1 CFS"}="1HB  CFS";
$resn_exch{" HB2 CFS"}="2HB  CFS"; $resn_exch{"HD11 CFS"}="1HD1 CFS"; $resn_exch{"HD12 CFS"}="2HD1 CFS";
$resn_exch{"HD21 CFS"}="1HD2 CFS"; $resn_exch{"HD22 CFS"}="2HD2 CFS"; $resn_exch{"HE11 CFS"}="1HE1 CFS";
$resn_exch{"HE12 CFS"}="2HE1 CFS"; $resn_exch{"HE21 CFS"}="1HE2 CFS"; $resn_exch{"HE22 CFS"}="2HE2 CFS";
$resn_exch{" HZ1 CFS"}="1HZ  CFS"; $resn_exch{" HZ2 CFS"}="2HZ  CFS"; $resn_exch{"HC91 CFX"}="1HC9 CFX";
$resn_exch{"HC92 CFX"}="2HC9 CFX"; $resn_exch{"H151 CFX"}="1H15 CFX"; $resn_exch{"H152 CFX"}="2H15 CFX";
$resn_exch{"H171 CFX"}="1H17 CFX"; $resn_exch{"H172 CFX"}="2H17 CFX"; $resn_exch{"H173 CFX"}="3H17 CFX";
$resn_exch{"H201 CFX"}="1H20 CFX"; $resn_exch{"H202 CFX"}="2H20 CFX"; $resn_exch{"H331 CFX"}="1H33 CFX";
$resn_exch{"H332 CFX"}="2H33 CFX"; $resn_exch{" H11  CG"}="1H1   CG"; $resn_exch{"HN61  CG"}="1HN6  CG";
$resn_exch{"HN71  CG"}="1HN7  CG"; $resn_exch{"HN72  CG"}="2HN7  CG"; $resn_exch{" H81  CG"}="1H8   CG";
$resn_exch{" H91  CG"}="1H9   CG"; $resn_exch{"H101  CG"}="1H10  CG"; $resn_exch{"H102  CG"}="2H10  CG";
$resn_exch{"H111  CG"}="1H11  CG"; $resn_exch{"H112  CG"}="2H11  CG"; $resn_exch{"H141  CG"}="1H14  CG";
$resn_exch{"H161  CG"}="1H16  CG"; $resn_exch{"H171  CG"}="1H17  CG"; $resn_exch{"H172  CG"}="2H17  CG";
$resn_exch{"HNG1 CG2"}="1HNG CG2"; $resn_exch{"H2G1 CG2"}="1H2G CG2"; $resn_exch{"H2G2 CG2"}="2H2G CG2";
$resn_exch{"HC'1 CG2"}="1HC' CG2"; $resn_exch{"HC'2 CG2"}="2HC' CG2"; $resn_exch{"HCA1 CG2"}="1HCA CG2";
$resn_exch{"HCA2 CG2"}="2HCA CG2"; $resn_exch{"H4C1 CG2"}="1H4C CG2"; $resn_exch{"H4C2 CG2"}="2H4C CG2";
$resn_exch{" H   CGA"}="1HN  CGA"; $resn_exch{" H2  CGA"}="2HN  CGA"; $resn_exch{" HB1 CGA"}="1HB  CGA";
$resn_exch{" HB2 CGA"}="2HB  CGA"; $resn_exch{" HG1 CGA"}="1HG  CGA"; $resn_exch{" HG2 CGA"}="2HG  CGA";
$resn_exch{" H21 CGA"}="1H2  CGA"; $resn_exch{" H22 CGA"}="2H2  CGA"; $resn_exch{"H8C1 CGB"}="1H8C CGB";
$resn_exch{"H8C2 CGB"}="2H8C CGB"; $resn_exch{"H7C1 CGB"}="1H7C CGB"; $resn_exch{"H7C2 CGB"}="2H7C CGB";
$resn_exch{"HN71 CGF"}="1HN7 CGF"; $resn_exch{"HN72 CGF"}="2HN7 CGF"; $resn_exch{" H61 CGF"}="1H6  CGF";
$resn_exch{" H62 CGF"}="2H6  CGF"; $resn_exch{" HB1 CGN"}="1HB  CGN"; $resn_exch{" HB2 CGN"}="2HB  CGN";
$resn_exch{" HG1 CGN"}="1HG  CGN"; $resn_exch{" HG2 CGN"}="2HG  CGN"; $resn_exch{"HOP2 CGP"}="2HOP CGP";
$resn_exch{"HO5C CGP"}="CHO5 CGP"; $resn_exch{"H51C CGP"}="CH51 CGP"; $resn_exch{"H52C CGP"}="CH52 CGP";
$resn_exch{" H4B CGP"}="CH4* CGP"; $resn_exch{" H3B CGP"}="CH3* CGP"; $resn_exch{"H21C CGP"}="CH21 CGP";
$resn_exch{"H22C CGP"}="CH22 CGP"; $resn_exch{" H1B CGP"}="CH1* CGP"; $resn_exch{" H5C CGP"}=" CH5 CGP";
$resn_exch{" H6C CGP"}=" CH6 CGP"; $resn_exch{"H41C CGP"}="CH41 CGP"; $resn_exch{"H42C CGP"}="CH42 CGP";
$resn_exch{"H51G CGP"}="GH51 CGP"; $resn_exch{"H52G CGP"}="GH52 CGP"; $resn_exch{" H4D CGP"}="GH4* CGP";
$resn_exch{" H3D CGP"}="GH3* CGP"; $resn_exch{"HO3G CGP"}="GHO3 CGP"; $resn_exch{"H21G CGP"}="GH21 CGP";
$resn_exch{"H22G CGP"}="GH22 CGP"; $resn_exch{" H1D CGP"}="GH1* CGP"; $resn_exch{" H8G CGP"}=" GH8 CGP";
$resn_exch{" H1G CGP"}=" GH1 CGP"; $resn_exch{"HN1G CGP"}="GHN1 CGP"; $resn_exch{"HN2G CGP"}="GHN2 CGP";
$resn_exch{" H21 CGQ"}="1H2  CGQ"; $resn_exch{" H41 CGQ"}="1H4  CGQ"; $resn_exch{" H51 CGQ"}="1H5  CGQ";
$resn_exch{" H61 CGQ"}="1H6  CGQ"; $resn_exch{" H71 CGQ"}="1H7  CGQ"; $resn_exch{"H101 CGQ"}="1H10 CGQ";
$resn_exch{"H102 CGQ"}="2H10 CGQ"; $resn_exch{"H111 CGQ"}="1H11 CGQ"; $resn_exch{"H112 CGQ"}="2H11 CGQ";
$resn_exch{"H121 CGQ"}="1H12 CGQ"; $resn_exch{"H151 CGQ"}="1H15 CGQ"; $resn_exch{"H152 CGQ"}="2H15 CGQ";
$resn_exch{"H191 CGQ"}="1H19 CGQ"; $resn_exch{"H211 CGQ"}="1H21 CGQ"; $resn_exch{"H231 CGQ"}="1H23 CGQ";
$resn_exch{"H241 CGQ"}="1H24 CGQ"; $resn_exch{"H251 CGQ"}="1H25 CGQ"; $resn_exch{"H271 CGQ"}="1H27 CGQ";
$resn_exch{"H291 CGQ"}="1H29 CGQ"; $resn_exch{"H292 CGQ"}="2H29 CGQ"; $resn_exch{"H301 CGQ"}="1H30 CGQ";
$resn_exch{"H302 CGQ"}="2H30 CGQ"; $resn_exch{" H51 CGR"}="1H5  CGR"; $resn_exch{" H52 CGR"}="2H5  CGR";
$resn_exch{"HC4' CGR"}="HC4* CGR"; $resn_exch{"HC1' CGR"}="HC1* CGR"; $resn_exch{"HC2' CGR"}="HC2* CGR";
$resn_exch{"HC3' CGR"}="HC3* CGR"; $resn_exch{" H81 CGR"}="1H8  CGR"; $resn_exch{" H82 CGR"}="2H8  CGR";
$resn_exch{" H1' CGR"}=" H1* CGR"; $resn_exch{" H2' CGR"}=" H2* CGR"; $resn_exch{"HO2' CGR"}="HO2* CGR";
$resn_exch{" H3' CGR"}=" H3* CGR"; $resn_exch{"HO3' CGR"}="HO3* CGR"; $resn_exch{" H4' CGR"}=" H4* CGR";
$resn_exch{"H5'1 CGR"}="1H5* CGR"; $resn_exch{"H5'2 CGR"}="2H5* CGR"; $resn_exch{"H2A1 CGR"}="1H2A CGR";
$resn_exch{"H2A2 CGR"}="2H2A CGR"; $resn_exch{"H7C1 CGT"}="1H7C CGT"; $resn_exch{"H7C2 CGT"}="2H7C CGT";
$resn_exch{"H6C1 CGT"}="1H6C CGT"; $resn_exch{"H6C2 CGT"}="2H6C CGT"; $resn_exch{"HBC1 CGT"}="1HBC CGT";
$resn_exch{"HBC2 CGT"}="2HBC CGT"; $resn_exch{" H   CGU"}="1HN  CGU"; $resn_exch{" HN2 CGU"}="2HN  CGU";
$resn_exch{" HB2 CGU"}="1HB  CGU"; $resn_exch{" HB3 CGU"}="2HB  CGU"; $resn_exch{"HE12 CGU"}="HE2  CGU";
$resn_exch{"HE22 CGU"}="HE4  CGU"; $resn_exch{"HOP3  CH"}="3HOP  CH"; $resn_exch{"HOP2  CH"}="2HOP  CH";
$resn_exch{" H5'  CH"}="1H5*  CH"; $resn_exch{"H5''  CH"}="2H5*  CH"; $resn_exch{" H4'  CH"}=" H4*  CH";
$resn_exch{" H3'  CH"}=" H3*  CH"; $resn_exch{"HO3'  CH"}="H3T   CH"; $resn_exch{" H2'  CH"}=" H2*  CH";
$resn_exch{"HO2'  CH"}="2HO*  CH"; $resn_exch{" H1'  CH"}=" H1*  CH"; $resn_exch{" H41  CH"}="1H4   CH";
$resn_exch{" H42  CH"}="2H4   CH"; $resn_exch{"HN41 CH1"}="1HN4 CH1"; $resn_exch{"HN42 CH1"}="2HN4 CH1";
$resn_exch{" H1' CH1"}=" H1* CH1"; $resn_exch{" H2' CH1"}=" H2* CH1"; $resn_exch{"HO2' CH1"}="*HO2 CH1";
$resn_exch{"H3'2 CH1"}="2H3* CH1"; $resn_exch{" H3' CH1"}=" H3* CH1"; $resn_exch{" H4' CH1"}=" H4* CH1";
$resn_exch{"H5'1 CH1"}="1H5* CH1"; $resn_exch{"H5'2 CH1"}="2H5* CH1"; $resn_exch{"HOA2 CH1"}="2HOA CH1";
$resn_exch{"HOB2 CH1"}="2HOB CH1"; $resn_exch{"HOG2 CH1"}="2HOG CH1"; $resn_exch{"HOG3 CH1"}="3HOG CH1";
$resn_exch{" H1  CH2"}="1H   CH2"; $resn_exch{" H2  CH2"}="2H   CH2"; $resn_exch{" HC1 CH3"}="1HC  CH3";
$resn_exch{" HC2 CH3"}="2HC  CH3"; $resn_exch{" HC3 CH3"}="3HC  CH3"; $resn_exch{" HC4 CH3"}="4HC  CH3";
$resn_exch{"HB12 CH4"}="2HB1 CH4"; $resn_exch{"HB13 CH4"}="3HB1 CH4"; $resn_exch{" HG1 CH4"}="1HG  CH4";
$resn_exch{"HD12 CH4"}="2HD1 CH4"; $resn_exch{"HD13 CH4"}="3HD1 CH4"; $resn_exch{"HE12 CH4"}="2HE1 CH4";
$resn_exch{"HE13 CH4"}="3HE1 CH4"; $resn_exch{"HZ11 CH4"}="1HZ1 CH4"; $resn_exch{"HZ12 CH4"}="2HZ1 CH4";
$resn_exch{"HB22 CH4"}="2HB2 CH4"; $resn_exch{"HB23 CH4"}="3HB2 CH4"; $resn_exch{" HG2 CH4"}="2HG  CH4";
$resn_exch{"HD22 CH4"}="2HD2 CH4"; $resn_exch{"HD23 CH4"}="3HD2 CH4"; $resn_exch{"HE22 CH4"}="2HE2 CH4";
$resn_exch{"HE23 CH4"}="3HE2 CH4"; $resn_exch{"HZ21 CH4"}="1HZ2 CH4"; $resn_exch{"HZ22 CH4"}="2HZ2 CH4";
$resn_exch{" H11 CH5"}="1H1  CH5"; $resn_exch{" H12 CH5"}="2H1  CH5"; $resn_exch{" H31 CH5"}="1H3  CH5";
$resn_exch{" H32 CH5"}="2H3  CH5"; $resn_exch{" H41 CH5"}="1H4  CH5"; $resn_exch{" H42 CH5"}="2H4  CH5";
$resn_exch{" H51 CH5"}="1H5  CH5"; $resn_exch{" H52 CH5"}="2H5  CH5"; $resn_exch{" H61 CH5"}="1H6  CH5";
$resn_exch{" H62 CH5"}="2H6  CH5"; $resn_exch{" H63 CH5"}="3H6  CH5"; $resn_exch{" H71 CH5"}="1H7  CH5";
$resn_exch{" H72 CH5"}="2H7  CH5"; $resn_exch{" H73 CH5"}="3H7  CH5"; $resn_exch{" H81 CH5"}="1H8  CH5";
$resn_exch{" H82 CH5"}="2H8  CH5"; $resn_exch{" H83 CH5"}="3H8  CH5"; $resn_exch{" HB1 CH7"}="1HB  CH7";
$resn_exch{"HB2A CH7"}="2HB  CH7"; $resn_exch{" HG1 CH7"}="1HG  CH7"; $resn_exch{" HG2 CH7"}="2HG  CH7";
$resn_exch{"HD1A CH7"}="1HD  CH7"; $resn_exch{"HD2A CH7"}="2HD  CH7"; $resn_exch{"HE1A CH7"}="1HE  CH7";
$resn_exch{"HE2A CH7"}="2HE  CH7"; $resn_exch{"HA31 CH7"}="1HA3 CH7"; $resn_exch{"HA32 CH7"}="2HA3 CH7";
$resn_exch{" H11 CHA"}="1H1  CHA"; $resn_exch{" H12 CHA"}="2H1  CHA"; $resn_exch{" H31 CHA"}="1H3  CHA";
$resn_exch{" H32 CHA"}="2H3  CHA"; $resn_exch{"H2'1 CHA"}="1H2' CHA"; $resn_exch{"H2'2 CHA"}="2H2' CHA";
$resn_exch{"H3'1 CHA"}="1H3' CHA"; $resn_exch{"H3'2 CHA"}="2H3' CHA"; $resn_exch{"H4'1 CHA"}="1H4' CHA";
$resn_exch{"H4'2 CHA"}="2H4' CHA"; $resn_exch{"H5'1 CHA"}="1H5' CHA"; $resn_exch{"H5'2 CHA"}="2H5' CHA";
$resn_exch{"H6'1 CHA"}="1H6' CHA"; $resn_exch{"H6'2 CHA"}="2H6' CHA"; $resn_exch{"HN21 CHA"}="1HN2 CHA";
$resn_exch{"HN22 CHA"}="2HN2 CHA"; $resn_exch{"HC11 CHC"}="1HC1 CHC"; $resn_exch{"HC12 CHC"}="2HC1 CHC";
$resn_exch{"HC21 CHC"}="1HC2 CHC"; $resn_exch{"HC22 CHC"}="2HC2 CHC"; $resn_exch{"HC41 CHC"}="1HC4 CHC";
$resn_exch{"HC42 CHC"}="2HC4 CHC"; $resn_exch{"H111 CHC"}="1H11 CHC"; $resn_exch{"H112 CHC"}="2H11 CHC";
$resn_exch{"H121 CHC"}="1H12 CHC"; $resn_exch{"H122 CHC"}="2H12 CHC"; $resn_exch{"H14C CHC"}="CH14 CHC";
$resn_exch{"H151 CHC"}="1H15 CHC"; $resn_exch{"H152 CHC"}="2H15 CHC"; $resn_exch{"H161 CHC"}="1H16 CHC";
$resn_exch{"H162 CHC"}="2H16 CHC"; $resn_exch{"H17C CHC"}="CH17 CHC"; $resn_exch{"H181 CHC"}="1H18 CHC";
$resn_exch{"H182 CHC"}="2H18 CHC"; $resn_exch{"H183 CHC"}="3H18 CHC"; $resn_exch{"H191 CHC"}="1H19 CHC";
$resn_exch{"H192 CHC"}="2H19 CHC"; $resn_exch{"H193 CHC"}="3H19 CHC"; $resn_exch{"H20C CHC"}="CH20 CHC";
$resn_exch{"H211 CHC"}="1H21 CHC"; $resn_exch{"H212 CHC"}="2H21 CHC"; $resn_exch{"H213 CHC"}="3H21 CHC";
$resn_exch{"H221 CHC"}="1H22 CHC"; $resn_exch{"H222 CHC"}="2H22 CHC"; $resn_exch{"H231 CHC"}="1H23 CHC";
$resn_exch{"H232 CHC"}="2H23 CHC"; $resn_exch{"H251 CHC"}="1H25 CHC"; $resn_exch{"H252 CHC"}="2H25 CHC";
$resn_exch{"H253 CHC"}="3H25 CHC"; $resn_exch{"H261 CHC"}="1H26 CHC"; $resn_exch{"H262 CHC"}="2H26 CHC";
$resn_exch{"HT1O CHC"}="OHT1 CHC"; $resn_exch{" H11 CHD"}="1H1  CHD"; $resn_exch{"H12A CHD"}="2H1  CHD";
$resn_exch{" H21 CHD"}="1H2  CHD"; $resn_exch{" H22 CHD"}="2H2  CHD"; $resn_exch{" H41 CHD"}="1H4  CHD";
$resn_exch{" H42 CHD"}="2H4  CHD"; $resn_exch{" H61 CHD"}="1H6  CHD"; $resn_exch{" H62 CHD"}="2H6  CHD";
$resn_exch{"H111 CHD"}="1H11 CHD"; $resn_exch{"H112 CHD"}="2H11 CHD"; $resn_exch{"H151 CHD"}="1H15 CHD";
$resn_exch{"H152 CHD"}="2H15 CHD"; $resn_exch{"H161 CHD"}="1H16 CHD"; $resn_exch{"H162 CHD"}="2H16 CHD";
$resn_exch{"H181 CHD"}="1H18 CHD"; $resn_exch{"H182 CHD"}="2H18 CHD"; $resn_exch{"H183 CHD"}="3H18 CHD";
$resn_exch{"H191 CHD"}="1H19 CHD"; $resn_exch{"H192 CHD"}="2H19 CHD"; $resn_exch{"H193 CHD"}="3H19 CHD";
$resn_exch{"H211 CHD"}="1H21 CHD"; $resn_exch{"H212 CHD"}="2H21 CHD"; $resn_exch{"H213 CHD"}="3H21 CHD";
$resn_exch{"H221 CHD"}="1H22 CHD"; $resn_exch{"H222 CHD"}="2H22 CHD"; $resn_exch{"H231 CHD"}="1H23 CHD";
$resn_exch{"H232 CHD"}="2H23 CHD"; $resn_exch{" H   CHF"}="1HN  CHF"; $resn_exch{" H2  CHF"}="2HN  CHF";
$resn_exch{" HB2 CHF"}="1HB  CHF"; $resn_exch{" HB3 CHF"}="2HB  CHF"; $resn_exch{"HD12 CHF"}="1HD1 CHF";
$resn_exch{"HD13 CHF"}="2HD1 CHF"; $resn_exch{"HD23 CHF"}="1HD2 CHF"; $resn_exch{"HD22 CHF"}="2HD2 CHF";
$resn_exch{"HE12 CHF"}="1HE1 CHF"; $resn_exch{"HE13 CHF"}="2HE1 CHF"; $resn_exch{"HE23 CHF"}="1HE2 CHF";
$resn_exch{"HE22 CHF"}="2HE2 CHF"; $resn_exch{" HZ2 CHF"}="1HZ  CHF"; $resn_exch{" HZ3 CHF"}="2HZ  CHF";
$resn_exch{" H21 CHG"}="1H2  CHG"; $resn_exch{" H22 CHG"}="2H2  CHG"; $resn_exch{" H31 CHG"}="1H3  CHG";
$resn_exch{" H32 CHG"}="2H3  CHG"; $resn_exch{" H41 CHG"}="1H4  CHG"; $resn_exch{" H42 CHG"}="2H4  CHG";
$resn_exch{" H51 CHG"}="1H5  CHG"; $resn_exch{" H52 CHG"}="2H5  CHG"; $resn_exch{" H61 CHG"}="1H6  CHG";
$resn_exch{" H62 CHG"}="2H6  CHG"; $resn_exch{"HN71 CHG"}="1HN7 CHG"; $resn_exch{"HN72 CHG"}="2HN7 CHG";
$resn_exch{" H61 CHH"}="1H6  CHH"; $resn_exch{" H62 CHH"}="2H6  CHH"; $resn_exch{" H63 CHH"}="3H6  CHH";
$resn_exch{" H41 CHH"}="1H4  CHH"; $resn_exch{" H42 CHH"}="2H4  CHH"; $resn_exch{" H31 CHH"}="1H3  CHH";
$resn_exch{" H32 CHH"}="2H3  CHH"; $resn_exch{" H21 CHH"}="1H2  CHH"; $resn_exch{" H22 CHH"}="2H2  CHH";
$resn_exch{" H91 CHH"}="1H9  CHH"; $resn_exch{" H92 CHH"}="2H9  CHH"; $resn_exch{" H93 CHH"}="3H9  CHH";
$resn_exch{"H101 CHH"}="1H10 CHH"; $resn_exch{"H102 CHH"}="2H10 CHH"; $resn_exch{"H103 CHH"}="3H10 CHH";
$resn_exch{" H81 CHH"}="1H8  CHH"; $resn_exch{" H82 CHH"}="2H8  CHH"; $resn_exch{" H83 CHH"}="3H8  CHH";
$resn_exch{" H21 CHI"}="1H2  CHI"; $resn_exch{" H31 CHI"}="1H3  CHI"; $resn_exch{" H61 CHI"}="1H6  CHI";
$resn_exch{"H101 CHI"}="1H10 CHI"; $resn_exch{"H121 CHI"}="1H12 CHI"; $resn_exch{"H122 CHI"}="2H12 CHI";
$resn_exch{"H131 CHI"}="1H13 CHI"; $resn_exch{"H132 CHI"}="2H13 CHI"; $resn_exch{"H151 CHI"}="1H15 CHI";
$resn_exch{"H152 CHI"}="2H15 CHI"; $resn_exch{"H161 CHI"}="1H16 CHI"; $resn_exch{"H162 CHI"}="2H16 CHI";
$resn_exch{"H171 CHI"}="1H17 CHI"; $resn_exch{"H172 CHI"}="2H17 CHI"; $resn_exch{"H191 CHI"}="1H19 CHI";
$resn_exch{"H201 CHI"}="1H20 CHI"; $resn_exch{"H221 CHI"}="1H22 CHI"; $resn_exch{"H231 CHI"}="1H23 CHI";
$resn_exch{"HX21 CHK"}="1HX2 CHK"; $resn_exch{"HX22 CHK"}="2HX2 CHK"; $resn_exch{"HX41 CHK"}="1HX4 CHK";
$resn_exch{"HX42 CHK"}="2HX4 CHK"; $resn_exch{"HX51 CHK"}="1HX5 CHK"; $resn_exch{"HX52 CHK"}="2HX5 CHK";
$resn_exch{"HX61 CHK"}="1HX6 CHK"; $resn_exch{"HX62 CHK"}="2HX6 CHK"; $resn_exch{"HX71 CHK"}="1HX7 CHK";
$resn_exch{"HX72 CHK"}="2HX7 CHK"; $resn_exch{"HX81 CHK"}="1HX8 CHK"; $resn_exch{"HX82 CHK"}="2HX8 CHK";
$resn_exch{"HX91 CHK"}="1HX9 CHK"; $resn_exch{"HX92 CHK"}="2HX9 CHK"; $resn_exch{"HXA1 CHK"}="1HXA CHK";
$resn_exch{"HXA2 CHK"}="2HXA CHK"; $resn_exch{"HXB1 CHK"}="1HXB CHK"; $resn_exch{"HXB2 CHK"}="2HXB CHK";
$resn_exch{"HXG1 CHK"}="1HXG CHK"; $resn_exch{"HXG2 CHK"}="2HXG CHK"; $resn_exch{"HMA1 CHL"}="1HMA CHL";
$resn_exch{"HMA2 CHL"}="2HMA CHL"; $resn_exch{"HMA3 CHL"}="3HMA CHL"; $resn_exch{"HAA1 CHL"}="1HAA CHL";
$resn_exch{"HAA2 CHL"}="2HAA CHL"; $resn_exch{"HBA1 CHL"}="1HBA CHL"; $resn_exch{"HBA2 CHL"}="2HBA CHL";
$resn_exch{"HMB1 CHL"}="1HMB CHL"; $resn_exch{"HMB2 CHL"}="2HMB CHL"; $resn_exch{"HMB3 CHL"}="3HMB CHL";
$resn_exch{"HBB1 CHL"}="1HBB CHL"; $resn_exch{"HBB2 CHL"}="2HBB CHL"; $resn_exch{"HAC1 CHL"}="1HAC CHL";
$resn_exch{"HAC2 CHL"}="2HAC CHL"; $resn_exch{"HBC1 CHL"}="1HBC CHL"; $resn_exch{"HBC2 CHL"}="2HBC CHL";
$resn_exch{"HBC3 CHL"}="3HBC CHL"; $resn_exch{"HMD1 CHL"}="1HMD CHL"; $resn_exch{"HMD2 CHL"}="2HMD CHL";
$resn_exch{"HMD3 CHL"}="3HMD CHL"; $resn_exch{"HED1 CHL"}="1HED CHL"; $resn_exch{"HED2 CHL"}="2HED CHL";
$resn_exch{"HED3 CHL"}="3HED CHL"; $resn_exch{" H11 CHL"}="1H1  CHL"; $resn_exch{" H12 CHL"}="2H1  CHL";
$resn_exch{" H41 CHL"}="1H4  CHL"; $resn_exch{" H42 CHL"}="2H4  CHL"; $resn_exch{" H43 CHL"}="3H4  CHL";
$resn_exch{" H51 CHL"}="1H5  CHL"; $resn_exch{" H52 CHL"}="2H5  CHL"; $resn_exch{" H61 CHL"}="1H6  CHL";
$resn_exch{" H62 CHL"}="2H6  CHL"; $resn_exch{" H71 CHL"}="1H7  CHL"; $resn_exch{" H72 CHL"}="2H7  CHL";
$resn_exch{" H91 CHL"}="1H9  CHL"; $resn_exch{" H92 CHL"}="2H9  CHL"; $resn_exch{" H93 CHL"}="3H9  CHL";
$resn_exch{"H101 CHL"}="1H10 CHL"; $resn_exch{"H102 CHL"}="2H10 CHL"; $resn_exch{"H111 CHL"}="1H11 CHL";
$resn_exch{"H112 CHL"}="2H11 CHL"; $resn_exch{"H121 CHL"}="1H12 CHL"; $resn_exch{"H122 CHL"}="2H12 CHL";
$resn_exch{"H141 CHL"}="1H14 CHL"; $resn_exch{"H142 CHL"}="2H14 CHL"; $resn_exch{"H143 CHL"}="3H14 CHL";
$resn_exch{"H151 CHL"}="1H15 CHL"; $resn_exch{"H152 CHL"}="2H15 CHL"; $resn_exch{"H161 CHL"}="1H16 CHL";
$resn_exch{"H162 CHL"}="2H16 CHL"; $resn_exch{"H171 CHL"}="1H17 CHL"; $resn_exch{"H172 CHL"}="2H17 CHL";
$resn_exch{"H191 CHL"}="1H19 CHL"; $resn_exch{"H192 CHL"}="2H19 CHL"; $resn_exch{"H193 CHL"}="3H19 CHL";
$resn_exch{"H201 CHL"}="1H20 CHL"; $resn_exch{"H202 CHL"}="2H20 CHL"; $resn_exch{"H203 CHL"}="3H20 CHL";
$resn_exch{" H11 CHM"}="1H1  CHM"; $resn_exch{" H12 CHM"}="2H1  CHM"; $resn_exch{" H31 CHM"}="1H3  CHM";
$resn_exch{" H32 CHM"}="2H3  CHM"; $resn_exch{"HN11 CHN"}="1HN1 CHN"; $resn_exch{"HN12 CHN"}="2HN1 CHN";
$resn_exch{" H11 CHN"}="1H1  CHN"; $resn_exch{" H12 CHN"}="2H1  CHN"; $resn_exch{" H21 CHN"}="1H2  CHN";
$resn_exch{" H71 CHN"}="1H7  CHN"; $resn_exch{" H72 CHN"}="2H7  CHN"; $resn_exch{" H61 CHN"}="1H6  CHN";
$resn_exch{" H62 CHN"}="2H6  CHN"; $resn_exch{" H51 CHN"}="1H5  CHN"; $resn_exch{" H41 CHN"}="1H4  CHN";
$resn_exch{" H42 CHN"}="2H4  CHN"; $resn_exch{" H31 CHN"}="1H3  CHN"; $resn_exch{" H32 CHN"}="2H3  CHN";
$resn_exch{"HN21 CHN"}="1HN2 CHN"; $resn_exch{"HN22 CHN"}="2HN2 CHN"; $resn_exch{" H11 CHO"}="1H1  CHO";
$resn_exch{" H12 CHO"}="2H1  CHO"; $resn_exch{" H21 CHO"}="1H2  CHO"; $resn_exch{" H22 CHO"}="2H2  CHO";
$resn_exch{" H41 CHO"}="1H4  CHO"; $resn_exch{" H42 CHO"}="2H4  CHO"; $resn_exch{" H61 CHO"}="1H6  CHO";
$resn_exch{" H62 CHO"}="2H6  CHO"; $resn_exch{"H111 CHO"}="1H11 CHO"; $resn_exch{"H112 CHO"}="2H11 CHO";
$resn_exch{"H121 CHO"}="1H12 CHO"; $resn_exch{"H122 CHO"}="2H12 CHO"; $resn_exch{"H151 CHO"}="1H15 CHO";
$resn_exch{"H152 CHO"}="2H15 CHO"; $resn_exch{"H161 CHO"}="1H16 CHO"; $resn_exch{"H162 CHO"}="2H16 CHO";
$resn_exch{"H181 CHO"}="1H18 CHO"; $resn_exch{"H182 CHO"}="2H18 CHO"; $resn_exch{"H183 CHO"}="3H18 CHO";
$resn_exch{"H191 CHO"}="1H19 CHO"; $resn_exch{"H192 CHO"}="2H19 CHO"; $resn_exch{"H193 CHO"}="3H19 CHO";
$resn_exch{"H211 CHO"}="1H21 CHO"; $resn_exch{"H212 CHO"}="2H21 CHO"; $resn_exch{"H213 CHO"}="3H21 CHO";
$resn_exch{"H221 CHO"}="1H22 CHO"; $resn_exch{"H222 CHO"}="2H22 CHO"; $resn_exch{"H231 CHO"}="1H23 CHO";
$resn_exch{"H232 CHO"}="2H23 CHO"; $resn_exch{"H261 CHO"}="1H26 CHO"; $resn_exch{"H262 CHO"}="2H26 CHO";
$resn_exch{" H   CHP"}="1HN  CHP"; $resn_exch{" H2  CHP"}="2HN  CHP"; $resn_exch{" HC2 CHP"}=" H2  CHP";
$resn_exch{"HBC1 CHQ"}="1HBC CHQ"; $resn_exch{"HBC2 CHQ"}="2HBC CHQ"; $resn_exch{"HBP1 CHQ"}="1HBP CHQ";
$resn_exch{"HBP2 CHQ"}="2HBP CHQ"; $resn_exch{"HGP1 CHQ"}="1HGP CHQ"; $resn_exch{"HGP2 CHQ"}="2HGP CHQ";
$resn_exch{"HDP1 CHQ"}="1HDP CHQ"; $resn_exch{"HDP2 CHQ"}="2HDP CHQ"; $resn_exch{"H191 CHR"}="1H19 CHR";
$resn_exch{"H192 CHR"}="2H19 CHR"; $resn_exch{"H193 CHR"}="3H19 CHR"; $resn_exch{"H181 CHR"}="1H18 CHR";
$resn_exch{"H182 CHR"}="2H18 CHR"; $resn_exch{"H183 CHR"}="3H18 CHR"; $resn_exch{"H311 CHR"}="1H31 CHR";
$resn_exch{"H312 CHR"}="2H31 CHR"; $resn_exch{"H313 CHR"}="3H31 CHR"; $resn_exch{"H321 CHR"}="1H32 CHR";
$resn_exch{"H322 CHR"}="2H32 CHR"; $resn_exch{"H323 CHR"}="3H32 CHR"; $resn_exch{"H351 CHR"}="1H35 CHR";
$resn_exch{"H352 CHR"}="2H35 CHR"; $resn_exch{" H   CHS"}="1HN  CHS"; $resn_exch{" H2  CHS"}="2HN  CHS";
$resn_exch{" HB2 CHS"}="1HB  CHS"; $resn_exch{" HB3 CHS"}="2HB  CHS"; $resn_exch{"HD12 CHS"}="1HD1 CHS";
$resn_exch{"HD13 CHS"}="2HD1 CHS"; $resn_exch{"HD23 CHS"}="1HD2 CHS"; $resn_exch{"HD22 CHS"}="2HD2 CHS";
$resn_exch{"HE12 CHS"}="1HE1 CHS"; $resn_exch{"HE13 CHS"}="2HE1 CHS"; $resn_exch{"HE23 CHS"}="1HE2 CHS";
$resn_exch{"HE22 CHS"}="2HE2 CHS"; $resn_exch{" HZ2 CHS"}="1HZ  CHS"; $resn_exch{" HZ3 CHS"}="2HZ  CHS";
$resn_exch{" HM1 CHS"}="1HM  CHS"; $resn_exch{" HM2 CHS"}="2HM  CHS"; $resn_exch{"HC41 CHT"}="1HC4 CHT";
$resn_exch{"HC42 CHT"}="2HC4 CHT"; $resn_exch{"HC51 CHT"}="1HC5 CHT"; $resn_exch{"HC52 CHT"}="2HC5 CHT";
$resn_exch{" H61 CHT"}="1H6  CHT"; $resn_exch{" H62 CHT"}="2H6  CHT"; $resn_exch{" H63 CHT"}="3H6  CHT";
$resn_exch{" H71 CHT"}="1H7  CHT"; $resn_exch{" H72 CHT"}="2H7  CHT"; $resn_exch{" H73 CHT"}="3H7  CHT";
$resn_exch{" H81 CHT"}="1H8  CHT"; $resn_exch{" H82 CHT"}="2H8  CHT"; $resn_exch{" H83 CHT"}="3H8  CHT";
$resn_exch{" HO6 CHT"}="6HO  CHT"; $resn_exch{" H11 CHX"}="1H1  CHX"; $resn_exch{" H12 CHX"}="2H1  CHX";
$resn_exch{" H21 CHX"}="1H2  CHX"; $resn_exch{" H22 CHX"}="2H2  CHX"; $resn_exch{" H31 CHX"}="1H3  CHX";
$resn_exch{" H32 CHX"}="2H3  CHX"; $resn_exch{" H41 CHX"}="1H4  CHX"; $resn_exch{" H42 CHX"}="2H4  CHX";
$resn_exch{" H51 CHX"}="1H5  CHX"; $resn_exch{" H52 CHX"}="2H5  CHX"; $resn_exch{" H61 CHX"}="1H6  CHX";
$resn_exch{" H62 CHX"}="2H6  CHX"; $resn_exch{" H71 CHY"}="1H7  CHY"; $resn_exch{" H72 CHY"}="2H7  CHY";
$resn_exch{"H121 CHY"}="1H12 CHY"; $resn_exch{"H122 CHY"}="2H12 CHY"; $resn_exch{"H141 CHY"}="1H14 CHY";
$resn_exch{"H142 CHY"}="2H14 CHY"; $resn_exch{"H143 CHY"}="3H14 CHY"; $resn_exch{"H151 CHY"}="1H15 CHY";
$resn_exch{"H152 CHY"}="2H15 CHY"; $resn_exch{"H153 CHY"}="3H15 CHY"; $resn_exch{"H191 CHY"}="1H19 CHY";
$resn_exch{"H192 CHY"}="2H19 CHY"; $resn_exch{"H201 CHY"}="1H20 CHY"; $resn_exch{"H202 CHY"}="2H20 CHY";
$resn_exch{"H251 CHY"}="1H25 CHY"; $resn_exch{"H252 CHY"}="2H25 CHY"; $resn_exch{"HN41 CHY"}="1HN4 CHY";
$resn_exch{"HN42 CHY"}="2HN4 CHY"; $resn_exch{"H161 CIA"}="1H16 CIA"; $resn_exch{"H162 CIA"}="2H16 CIA";
$resn_exch{"H101 CIA"}="1H10 CIA"; $resn_exch{"H102 CIA"}="2H10 CIA"; $resn_exch{"H181 CIA"}="1H18 CIA";
$resn_exch{"H182 CIA"}="2H18 CIA"; $resn_exch{"H183 CIA"}="3H18 CIA"; $resn_exch{"H291 CIA"}="1H29 CIA";
$resn_exch{"H292 CIA"}="2H29 CIA"; $resn_exch{"HB11 CIB"}="1HB1 CIB"; $resn_exch{"HB12 CIB"}="2HB1 CIB";
$resn_exch{"HD11 CIB"}="1HD1 CIB"; $resn_exch{"HD12 CIB"}="2HD1 CIB"; $resn_exch{"HD13 CIB"}="3HD1 CIB";
$resn_exch{"HD21 CIB"}="1HD2 CIB"; $resn_exch{"HD22 CIB"}="2HD2 CIB"; $resn_exch{"HD23 CIB"}="3HD2 CIB";
$resn_exch{"H251 CIB"}="1H25 CIB"; $resn_exch{"H252 CIB"}="2H25 CIB"; $resn_exch{"H253 CIB"}="3H25 CIB";
$resn_exch{"HB21 CIB"}="1HB2 CIB"; $resn_exch{"HB22 CIB"}="2HB2 CIB"; $resn_exch{"HD31 CIB"}="1HD3 CIB";
$resn_exch{"HD32 CIB"}="2HD3 CIB"; $resn_exch{"HD33 CIB"}="3HD3 CIB"; $resn_exch{"HD41 CIB"}="1HD4 CIB";
$resn_exch{"HD42 CIB"}="2HD4 CIB"; $resn_exch{"HD43 CIB"}="3HD4 CIB"; $resn_exch{"HB31 CIB"}="1HB3 CIB";
$resn_exch{"HB32 CIB"}="2HB3 CIB"; $resn_exch{"HG31 CIB"}="1HG3 CIB"; $resn_exch{"HG32 CIB"}="2HG3 CIB";
$resn_exch{"HE31 CIB"}="1HE3 CIB"; $resn_exch{"HE32 CIB"}="2HE3 CIB"; $resn_exch{"HE33 CIB"}="3HE3 CIB";
$resn_exch{"H191 CIB"}="1H19 CIB"; $resn_exch{"H192 CIB"}="2H19 CIB"; $resn_exch{" H2A CIC"}=" AH2 CIC";
$resn_exch{"H61A CIC"}="AH61 CIC"; $resn_exch{"H62A CIC"}="AH62 CIC"; $resn_exch{" H8A CIC"}=" AH8 CIC";
$resn_exch{" H1B CIC"}="AH1* CIC"; $resn_exch{" H2B CIC"}="AH2* CIC"; $resn_exch{"HO2A CIC"}="AHO2 CIC";
$resn_exch{" H3B CIC"}="AH3* CIC"; $resn_exch{"HOA8 CIC"}="8HOA CIC"; $resn_exch{"HOA9 CIC"}="9HOA CIC";
$resn_exch{" H4B CIC"}="AH4* CIC"; $resn_exch{"H51A CIC"}="AH51 CIC"; $resn_exch{"H52A CIC"}="AH52 CIC";
$resn_exch{"HOA2 CIC"}="2HOA CIC"; $resn_exch{"HOA5 CIC"}="5HOA CIC"; $resn_exch{"H121 CIC"}="1H12 CIC";
$resn_exch{"H122 CIC"}="2H12 CIC"; $resn_exch{"H131 CIC"}="1H13 CIC"; $resn_exch{"H132 CIC"}="2H13 CIC";
$resn_exch{"H133 CIC"}="3H13 CIC"; $resn_exch{"H141 CIC"}="1H14 CIC"; $resn_exch{"H142 CIC"}="2H14 CIC";
$resn_exch{"H143 CIC"}="3H14 CIC"; $resn_exch{" H71 CIC"}="1H7  CIC"; $resn_exch{" H72 CIC"}="2H7  CIC";
$resn_exch{" H61 CIC"}="1H6  CIC"; $resn_exch{" H62 CIC"}="2H6  CIC"; $resn_exch{" H31 CIC"}="1H3  CIC";
$resn_exch{" H32 CIC"}="2H3  CIC"; $resn_exch{" H21 CIC"}="1H2  CIC"; $resn_exch{" H22 CIC"}="2H2  CIC";
$resn_exch{" H11 CIC"}="1H1  CIC"; $resn_exch{" H12 CIC"}="2H1  CIC"; $resn_exch{"H111 CIC"}="H11  CIC";
$resn_exch{"H123 CIC"}="H12  CIC"; $resn_exch{"HO32 CIC"}="2HO3 CIC"; $resn_exch{"HO52 CIC"}="2HO5 CIC";
$resn_exch{"H181 CID"}="1H18 CID"; $resn_exch{"H182 CID"}="2H18 CID"; $resn_exch{" H91 CIE"}="1H9  CIE";
$resn_exch{" H92 CIE"}="2H9  CIE"; $resn_exch{"H101 CIE"}="1H10 CIE"; $resn_exch{"H102 CIE"}="2H10 CIE";
$resn_exch{"H103 CIE"}="3H10 CIE"; $resn_exch{"H8'1 CIE"}="1H8' CIE"; $resn_exch{"H8'2 CIE"}="2H8' CIE";
$resn_exch{"H8'3 CIE"}="3H8' CIE"; $resn_exch{"H011 CIG"}="1H01 CIG"; $resn_exch{"H012 CIG"}="2H01 CIG";
$resn_exch{" H11 CIL"}="1H1  CIL"; $resn_exch{"H12A CIL"}="2H1  CIL"; $resn_exch{" H41 CIL"}="1H4  CIL";
$resn_exch{" H42 CIL"}="2H4  CIL"; $resn_exch{" H43 CIL"}="3H4  CIL"; $resn_exch{" H51 CIL"}="1H5  CIL";
$resn_exch{" H52 CIL"}="2H5  CIL"; $resn_exch{" H53 CIL"}="3H5  CIL"; $resn_exch{"H141 CIL"}="1H14 CIL";
$resn_exch{"H142 CIL"}="2H14 CIL"; $resn_exch{"H151 CIL"}="1H15 CIL"; $resn_exch{"H152 CIL"}="2H15 CIL";
$resn_exch{"H161 CIL"}="1H16 CIL"; $resn_exch{"H162 CIL"}="2H16 CIL"; $resn_exch{"H171 CIL"}="1H17 CIL";
$resn_exch{"H172 CIL"}="2H17 CIL"; $resn_exch{"H181 CIL"}="1H18 CIL"; $resn_exch{"H182 CIL"}="2H18 CIL";
$resn_exch{"H211 CIL"}="1H21 CIL"; $resn_exch{"H122 CIL"}="2H12 CIL"; $resn_exch{" H51 CIO"}="1H5  CIO";
$resn_exch{" H52 CIO"}="2H5  CIO"; $resn_exch{" H61 CIO"}="1H6  CIO"; $resn_exch{" H62 CIO"}="2H6  CIO";
$resn_exch{" H31 CIO"}="1H3  CIO"; $resn_exch{" H32 CIO"}="2H3  CIO"; $resn_exch{" H21 CIO"}="1H2  CIO";
$resn_exch{" H22 CIO"}="2H2  CIO"; $resn_exch{"H121 CIO"}="1H12 CIO"; $resn_exch{"H122 CIO"}="2H12 CIO";
$resn_exch{"H123 CIO"}="3H12 CIO"; $resn_exch{"H161 CIO"}="1H16 CIO"; $resn_exch{"H162 CIO"}="2H16 CIO";
$resn_exch{"H171 CIO"}="1H17 CIO"; $resn_exch{"H172 CIO"}="2H17 CIO"; $resn_exch{"H181 CIO"}="1H18 CIO";
$resn_exch{"H182 CIO"}="2H18 CIO"; $resn_exch{"H191 CIO"}="1H19 CIO"; $resn_exch{"H192 CIO"}="2H19 CIO";
$resn_exch{" H71 CIP"}="1H7  CIP"; $resn_exch{" H72 CIP"}="2H7  CIP"; $resn_exch{"HOP2 CIP"}="2HOP CIP";
$resn_exch{"H171 CIS"}="1H17 CIS"; $resn_exch{"H172 CIS"}="2H17 CIS"; $resn_exch{"H173 CIS"}="3H17 CIS";
$resn_exch{"H161 CIS"}="1H16 CIS"; $resn_exch{"H162 CIS"}="2H16 CIS"; $resn_exch{"H151 CIS"}="1H15 CIS";
$resn_exch{"H152 CIS"}="2H15 CIS"; $resn_exch{"H141 CIS"}="1H14 CIS"; $resn_exch{"H142 CIS"}="2H14 CIS";
$resn_exch{"H131 CIS"}="1H13 CIS"; $resn_exch{"H132 CIS"}="2H13 CIS"; $resn_exch{"H121 CIS"}="1H12 CIS";
$resn_exch{"H122 CIS"}="2H12 CIS"; $resn_exch{"H111 CIS"}="1H11 CIS"; $resn_exch{"H112 CIS"}="2H11 CIS";
$resn_exch{"H101 CIS"}="1H10 CIS"; $resn_exch{"H102 CIS"}="2H10 CIS"; $resn_exch{" H91 CIS"}="1H9  CIS";
$resn_exch{" H92 CIS"}="2H9  CIS"; $resn_exch{" H81 CIS"}="1H8  CIS"; $resn_exch{" H82 CIS"}="2H8  CIS";
$resn_exch{" H71 CIS"}="1H7  CIS"; $resn_exch{" H72 CIS"}="2H7  CIS"; $resn_exch{" H61 CIS"}="1H6  CIS";
$resn_exch{" H62 CIS"}="2H6  CIS"; $resn_exch{" H51 CIS"}="1H5  CIS"; $resn_exch{" H52 CIS"}="2H5  CIS";
$resn_exch{" H1A CIS"}="1H   CIS"; $resn_exch{" H2A CIS"}="2H   CIS"; $resn_exch{"H481 CIS"}="1H48 CIS";
$resn_exch{"H482 CIS"}="2H48 CIS"; $resn_exch{"H191 CIS"}="1H19 CIS"; $resn_exch{"H192 CIS"}="2H19 CIS";
$resn_exch{"H201 CIS"}="1H20 CIS"; $resn_exch{"H202 CIS"}="2H20 CIS"; $resn_exch{"H211 CIS"}="1H21 CIS";
$resn_exch{"H212 CIS"}="2H21 CIS"; $resn_exch{"H221 CIS"}="1H22 CIS"; $resn_exch{"H222 CIS"}="2H22 CIS";
$resn_exch{"H231 CIS"}="1H23 CIS"; $resn_exch{"H232 CIS"}="2H23 CIS"; $resn_exch{"H241 CIS"}="1H24 CIS";
$resn_exch{"H242 CIS"}="2H24 CIS"; $resn_exch{"H251 CIS"}="1H25 CIS"; $resn_exch{"H252 CIS"}="2H25 CIS";
$resn_exch{"H261 CIS"}="1H26 CIS"; $resn_exch{"H262 CIS"}="2H26 CIS"; $resn_exch{"H271 CIS"}="1H27 CIS";
$resn_exch{"H272 CIS"}="2H27 CIS"; $resn_exch{"H281 CIS"}="1H28 CIS"; $resn_exch{"H282 CIS"}="2H28 CIS";
$resn_exch{"H291 CIS"}="1H29 CIS"; $resn_exch{"H292 CIS"}="2H29 CIS"; $resn_exch{"H301 CIS"}="1H30 CIS";
$resn_exch{"H302 CIS"}="2H30 CIS"; $resn_exch{"H311 CIS"}="1H31 CIS"; $resn_exch{"H312 CIS"}="2H31 CIS";
$resn_exch{"H341 CIS"}="1H34 CIS"; $resn_exch{"H342 CIS"}="2H34 CIS"; $resn_exch{"H351 CIS"}="1H35 CIS";
$resn_exch{"H352 CIS"}="2H35 CIS"; $resn_exch{"H361 CIS"}="1H36 CIS"; $resn_exch{"H362 CIS"}="2H36 CIS";
$resn_exch{"H371 CIS"}="1H37 CIS"; $resn_exch{"H372 CIS"}="2H37 CIS"; $resn_exch{"H381 CIS"}="1H38 CIS";
$resn_exch{"H382 CIS"}="2H38 CIS"; $resn_exch{"H391 CIS"}="1H39 CIS"; $resn_exch{"H392 CIS"}="2H39 CIS";
$resn_exch{"H401 CIS"}="1H40 CIS"; $resn_exch{"H402 CIS"}="2H40 CIS"; $resn_exch{"H411 CIS"}="1H41 CIS";
$resn_exch{"H412 CIS"}="2H41 CIS"; $resn_exch{"H413 CIS"}="3H41 CIS"; $resn_exch{" H21 CIT"}="1H2  CIT";
$resn_exch{" H22 CIT"}="2H2  CIT"; $resn_exch{" H41 CIT"}="1H4  CIT"; $resn_exch{" H42 CIT"}="2H4  CIT";
$resn_exch{"HC11 CIU"}="1HC1 CIU"; $resn_exch{"HC12 CIU"}="2HC1 CIU"; $resn_exch{"HC21 CIU"}="1HC2 CIU";
$resn_exch{"HC22 CIU"}="2HC2 CIU"; $resn_exch{"HC31 CIU"}="1HC3 CIU"; $resn_exch{"HC41 CIU"}="1HC4 CIU";
$resn_exch{"HC42 CIU"}="2HC4 CIU"; $resn_exch{"HC51 CIU"}="1HC5 CIU"; $resn_exch{"HC52 CIU"}="2HC5 CIU";
$resn_exch{"HC61 CIU"}="1HC6 CIU"; $resn_exch{"HC62 CIU"}="2HC6 CIU"; $resn_exch{"HN71 CK1"}="1HN7 CK1";
$resn_exch{"HN72 CK1"}="2HN7 CK1"; $resn_exch{"H7A1 CK2"}="1H7A CK2"; $resn_exch{"H7A2 CK2"}="2H7A CK2";
$resn_exch{"H7A3 CK2"}="3H7A CK2"; $resn_exch{"H6A1 CK2"}="1H6A CK2"; $resn_exch{"H6A2 CK2"}="2H6A CK2";
$resn_exch{"H6A3 CK2"}="3H6A CK2"; $resn_exch{"HN71 CK2"}="1HN7 CK2"; $resn_exch{"HN72 CK2"}="2HN7 CK2";
$resn_exch{"H7A1 CK3"}="1H7A CK3"; $resn_exch{"H7A2 CK3"}="2H7A CK3"; $resn_exch{"H7A3 CK3"}="3H7A CK3";
$resn_exch{"H6A1 CK3"}="1H6A CK3"; $resn_exch{"H6A2 CK3"}="2H6A CK3"; $resn_exch{"H6A3 CK3"}="3H6A CK3";
$resn_exch{"H6A1 CK4"}="1H6A CK4"; $resn_exch{"H6A2 CK4"}="2H6A CK4"; $resn_exch{"H6A3 CK4"}="3H6A CK4";
$resn_exch{"H7A1 CK4"}="1H7A CK4"; $resn_exch{"H7A2 CK4"}="2H7A CK4"; $resn_exch{"H7A3 CK4"}="3H7A CK4";
$resn_exch{"H6A1 CK5"}="1H6A CK5"; $resn_exch{"H6A2 CK5"}="2H6A CK5"; $resn_exch{"H6A3 CK5"}="3H6A CK5";
$resn_exch{"H7A1 CK5"}="1H7A CK5"; $resn_exch{"H7A2 CK5"}="2H7A CK5"; $resn_exch{"H7A3 CK5"}="3H7A CK5";
$resn_exch{"H6A1 CK6"}="1H6A CK6"; $resn_exch{"H6A2 CK6"}="2H6A CK6"; $resn_exch{"H6A3 CK6"}="3H6A CK6";
$resn_exch{"H8A1 CK6"}="1H8A CK6"; $resn_exch{"H8A2 CK6"}="2H8A CK6"; $resn_exch{"H8A3 CK6"}="3H8A CK6";
$resn_exch{"H7A1 CK7"}="1H7A CK7"; $resn_exch{"H7A2 CK7"}="2H7A CK7"; $resn_exch{"H6A1 CK7"}="1H6A CK7";
$resn_exch{"H6A2 CK7"}="2H6A CK7"; $resn_exch{"H6A3 CK7"}="3H6A CK7"; $resn_exch{"H6A1 CK8"}="1H6A CK8";
$resn_exch{"H6A2 CK8"}="2H6A CK8"; $resn_exch{"H6A3 CK8"}="3H6A CK8"; $resn_exch{"H7A1 CK8"}="1H7A CK8";
$resn_exch{"H7A2 CK8"}="2H7A CK8"; $resn_exch{"H7A3 CK8"}="3H7A CK8"; $resn_exch{"H8B1 CK8"}="1H8B CK8";
$resn_exch{"H8B2 CK8"}="2H8B CK8"; $resn_exch{"H8B3 CK8"}="3H8B CK8"; $resn_exch{"H9B1 CK8"}="1H9B CK8";
$resn_exch{"H9B2 CK8"}="2H9B CK8"; $resn_exch{"H9B3 CK8"}="3H9B CK8"; $resn_exch{"H181 CK9"}="1H18 CK9";
$resn_exch{"H182 CK9"}="2H18 CK9"; $resn_exch{"H121 CK9"}="1H12 CK9"; $resn_exch{"H122 CK9"}="2H12 CK9";
$resn_exch{"H123 CK9"}="3H12 CK9"; $resn_exch{"H111 CK9"}="1H11 CK9"; $resn_exch{"H112 CK9"}="2H11 CK9";
$resn_exch{"H113 CK9"}="3H11 CK9"; $resn_exch{"H251 CK9"}="1H25 CK9"; $resn_exch{"H252 CK9"}="2H25 CK9";
$resn_exch{"H261 CK9"}="1H26 CK9"; $resn_exch{"H262 CK9"}="2H26 CK9"; $resn_exch{"H263 CK9"}="3H26 CK9";
$resn_exch{"H151 CK9"}="1H15 CK9"; $resn_exch{"H152 CK9"}="2H15 CK9"; $resn_exch{"H121 CKE"}="1H12 CKE";
$resn_exch{"H122 CKE"}="2H12 CKE"; $resn_exch{"H201 CKE"}="1H20 CKE"; $resn_exch{"H202 CKE"}="2H20 CKE";
$resn_exch{"H211 CKE"}="1H21 CKE"; $resn_exch{"H212 CKE"}="2H21 CKE"; $resn_exch{"H241 CKE"}="1H24 CKE";
$resn_exch{"H242 CKE"}="2H24 CKE"; $resn_exch{"H271 CKE"}="1H27 CKE"; $resn_exch{"H272 CKE"}="2H27 CKE";
$resn_exch{"H301 CKE"}="1H30 CKE"; $resn_exch{"H302 CKE"}="2H30 CKE"; $resn_exch{"H311 CKE"}="1H31 CKE";
$resn_exch{"H312 CKE"}="2H31 CKE"; $resn_exch{"H313 CKE"}="3H31 CKE"; $resn_exch{"H101 CKE"}="1H10 CKE";
$resn_exch{"H102 CKE"}="2H10 CKE"; $resn_exch{"H231 CKE"}="1H23 CKE"; $resn_exch{"H232 CKE"}="2H23 CKE";
$resn_exch{"H261 CKE"}="1H26 CKE"; $resn_exch{"H262 CKE"}="2H26 CKE"; $resn_exch{"H281 CKE"}="1H28 CKE";
$resn_exch{"H282 CKE"}="2H28 CKE"; $resn_exch{"H291 CKE"}="1H29 CKE"; $resn_exch{"H292 CKE"}="2H29 CKE";
$resn_exch{"H091 CKE"}="1H09 CKE"; $resn_exch{"H092 CKE"}="2H09 CKE"; $resn_exch{"H1'1 CKI"}="1H1' CKI";
$resn_exch{"H1'2 CKI"}="2H1' CKI"; $resn_exch{"H2'1 CKI"}="1H2' CKI"; $resn_exch{"H2'2 CKI"}="2H2' CKI";
$resn_exch{"HN21 CKI"}="1HN2 CKI"; $resn_exch{"HN22 CKI"}="2HN2 CKI"; $resn_exch{" H11 CKP"}="1H1  CKP";
$resn_exch{" H12 CKP"}="2H1  CKP"; $resn_exch{"HMA1 CL1"}="1HMA CL1"; $resn_exch{"HMA2 CL1"}="2HMA CL1";
$resn_exch{"HMA3 CL1"}="3HMA CL1"; $resn_exch{"HAA1 CL1"}="1HAA CL1"; $resn_exch{"HAA2 CL1"}="2HAA CL1";
$resn_exch{"HBA1 CL1"}="1HBA CL1"; $resn_exch{"HBA2 CL1"}="2HBA CL1"; $resn_exch{"HMB1 CL1"}="1HMB CL1";
$resn_exch{"HMB2 CL1"}="2HMB CL1"; $resn_exch{"HMB3 CL1"}="3HMB CL1"; $resn_exch{"HBB1 CL1"}="1HBB CL1";
$resn_exch{"HBB2 CL1"}="2HBB CL1"; $resn_exch{"HMC1 CL1"}="1HMC CL1"; $resn_exch{"HMC2 CL1"}="2HMC CL1";
$resn_exch{"HMC3 CL1"}="3HMC CL1"; $resn_exch{"HAC1 CL1"}="1HAC CL1"; $resn_exch{"HAC2 CL1"}="2HAC CL1";
$resn_exch{"HBC1 CL1"}="1HBC CL1"; $resn_exch{"HBC2 CL1"}="2HBC CL1"; $resn_exch{"HBC3 CL1"}="3HBC CL1";
$resn_exch{"HMD1 CL1"}="1HMD CL1"; $resn_exch{"HMD2 CL1"}="2HMD CL1"; $resn_exch{"HMD3 CL1"}="3HMD CL1";
$resn_exch{"HED1 CL1"}="1HED CL1"; $resn_exch{"HED2 CL1"}="2HED CL1"; $resn_exch{"HED3 CL1"}="3HED CL1";
$resn_exch{" H11 CL1"}="1H1  CL1"; $resn_exch{" H12 CL1"}="2H1  CL1"; $resn_exch{" H41 CL1"}="1H4  CL1";
$resn_exch{" H42 CL1"}="2H4  CL1"; $resn_exch{" H43 CL1"}="3H4  CL1"; $resn_exch{" H51 CL1"}="1H5  CL1";
$resn_exch{" H52 CL1"}="2H5  CL1"; $resn_exch{" H61 CL1"}="1H6  CL1"; $resn_exch{" H62 CL1"}="2H6  CL1";
$resn_exch{" H71 CL1"}="1H7  CL1"; $resn_exch{" H72 CL1"}="2H7  CL1"; $resn_exch{" H91 CL1"}="1H9  CL1";
$resn_exch{" H92 CL1"}="2H9  CL1"; $resn_exch{" H93 CL1"}="3H9  CL1"; $resn_exch{"H101 CL1"}="1H10 CL1";
$resn_exch{"H102 CL1"}="2H10 CL1"; $resn_exch{"H111 CL1"}="1H11 CL1"; $resn_exch{"H112 CL1"}="2H11 CL1";
$resn_exch{"H121 CL1"}="1H12 CL1"; $resn_exch{"H122 CL1"}="2H12 CL1"; $resn_exch{"H141 CL1"}="1H14 CL1";
$resn_exch{"H142 CL1"}="2H14 CL1"; $resn_exch{"H143 CL1"}="3H14 CL1"; $resn_exch{"H151 CL1"}="1H15 CL1";
$resn_exch{"H152 CL1"}="2H15 CL1"; $resn_exch{"H161 CL1"}="1H16 CL1"; $resn_exch{"H162 CL1"}="2H16 CL1";
$resn_exch{"H171 CL1"}="1H17 CL1"; $resn_exch{"H172 CL1"}="2H17 CL1"; $resn_exch{"H191 CL1"}="1H19 CL1";
$resn_exch{"H192 CL1"}="2H19 CL1"; $resn_exch{"H193 CL1"}="3H19 CL1"; $resn_exch{"H201 CL1"}="1H20 CL1";
$resn_exch{"H202 CL1"}="2H20 CL1"; $resn_exch{"H203 CL1"}="3H20 CL1"; $resn_exch{"HMA1 CL2"}="1HMA CL2";
$resn_exch{"HMA2 CL2"}="2HMA CL2"; $resn_exch{"HMA3 CL2"}="3HMA CL2"; $resn_exch{"HAA1 CL2"}="1HAA CL2";
$resn_exch{"HAA2 CL2"}="2HAA CL2"; $resn_exch{"HBA1 CL2"}="1HBA CL2"; $resn_exch{"HBA2 CL2"}="2HBA CL2";
$resn_exch{"HMB1 CL2"}="1HMB CL2"; $resn_exch{"HMB2 CL2"}="2HMB CL2"; $resn_exch{"HMB3 CL2"}="3HMB CL2";
$resn_exch{"HBB1 CL2"}="1HBB CL2"; $resn_exch{"HBB2 CL2"}="2HBB CL2"; $resn_exch{"HMC1 CL2"}="1HMC CL2";
$resn_exch{"HMC2 CL2"}="2HMC CL2"; $resn_exch{"HMC3 CL2"}="3HMC CL2"; $resn_exch{"HAC1 CL2"}="1HAC CL2";
$resn_exch{"HAC2 CL2"}="2HAC CL2"; $resn_exch{"HBC1 CL2"}="1HBC CL2"; $resn_exch{"HBC2 CL2"}="2HBC CL2";
$resn_exch{"HBC3 CL2"}="3HBC CL2"; $resn_exch{"HMD1 CL2"}="1HMD CL2"; $resn_exch{"HMD2 CL2"}="2HMD CL2";
$resn_exch{"HMD3 CL2"}="3HMD CL2"; $resn_exch{"HED1 CL2"}="1HED CL2"; $resn_exch{"HED2 CL2"}="2HED CL2";
$resn_exch{"HED3 CL2"}="3HED CL2"; $resn_exch{" H11 CL2"}="1H1  CL2"; $resn_exch{" H12 CL2"}="2H1  CL2";
$resn_exch{" H41 CL2"}="1H4  CL2"; $resn_exch{" H42 CL2"}="2H4  CL2"; $resn_exch{" H43 CL2"}="3H4  CL2";
$resn_exch{" H51 CL2"}="1H5  CL2"; $resn_exch{" H52 CL2"}="2H5  CL2"; $resn_exch{" H61 CL2"}="1H6  CL2";
$resn_exch{" H62 CL2"}="2H6  CL2"; $resn_exch{" H71 CL2"}="1H7  CL2"; $resn_exch{" H72 CL2"}="2H7  CL2";
$resn_exch{" H91 CL2"}="1H9  CL2"; $resn_exch{" H92 CL2"}="2H9  CL2"; $resn_exch{" H93 CL2"}="3H9  CL2";
$resn_exch{"H101 CL2"}="1H10 CL2"; $resn_exch{"H102 CL2"}="2H10 CL2"; $resn_exch{"H111 CL2"}="1H11 CL2";
$resn_exch{"H112 CL2"}="2H11 CL2"; $resn_exch{"H121 CL2"}="1H12 CL2"; $resn_exch{"H122 CL2"}="2H12 CL2";
$resn_exch{"H141 CL2"}="1H14 CL2"; $resn_exch{"H142 CL2"}="2H14 CL2"; $resn_exch{"H143 CL2"}="3H14 CL2";
$resn_exch{"H151 CL2"}="1H15 CL2"; $resn_exch{"H152 CL2"}="2H15 CL2"; $resn_exch{"H161 CL2"}="1H16 CL2";
$resn_exch{"H162 CL2"}="2H16 CL2"; $resn_exch{"H171 CL2"}="1H17 CL2"; $resn_exch{"H172 CL2"}="2H17 CL2";
$resn_exch{"H191 CL2"}="1H19 CL2"; $resn_exch{"H192 CL2"}="2H19 CL2"; $resn_exch{"H193 CL2"}="3H19 CL2";
$resn_exch{"H201 CL2"}="1H20 CL2"; $resn_exch{"H202 CL2"}="2H20 CL2"; $resn_exch{"H203 CL2"}="3H20 CL2";
$resn_exch{"H231 CL3"}="1H23 CL3"; $resn_exch{"H232 CL3"}="2H23 CL3"; $resn_exch{"H233 CL3"}="3H23 CL3";
$resn_exch{"H281 CL3"}="1H28 CL3"; $resn_exch{"H282 CL3"}="2H28 CL3"; $resn_exch{"H283 CL3"}="3H28 CL3";
$resn_exch{"HC21 CL4"}="1HC2 CL4"; $resn_exch{"HC22 CL4"}="2HC2 CL4"; $resn_exch{"H171 CL4"}="1H17 CL4";
$resn_exch{"H172 CL4"}="2H17 CL4"; $resn_exch{" H61 CL5"}="1H6  CL5"; $resn_exch{" H62 CL5"}="2H6  CL5";
$resn_exch{" H51 CL5"}="1H5  CL5"; $resn_exch{" H52 CL5"}="2H5  CL5"; $resn_exch{" H31 CL5"}="1H3  CL5";
$resn_exch{" H32 CL5"}="2H3  CL5"; $resn_exch{"HMA1 CLA"}="1HMA CLA"; $resn_exch{"HMA2 CLA"}="2HMA CLA";
$resn_exch{"HMA3 CLA"}="3HMA CLA"; $resn_exch{"HAA1 CLA"}="1HAA CLA"; $resn_exch{"HAA2 CLA"}="2HAA CLA";
$resn_exch{"HBA1 CLA"}="1HBA CLA"; $resn_exch{"HBA2 CLA"}="2HBA CLA"; $resn_exch{"HMB1 CLA"}="1HMB CLA";
$resn_exch{"HMB2 CLA"}="2HMB CLA"; $resn_exch{"HMB3 CLA"}="3HMB CLA"; $resn_exch{"HBB1 CLA"}="1HBB CLA";
$resn_exch{"HBB2 CLA"}="2HBB CLA"; $resn_exch{"HMC1 CLA"}="1HMC CLA"; $resn_exch{"HMC2 CLA"}="2HMC CLA";
$resn_exch{"HMC3 CLA"}="3HMC CLA"; $resn_exch{"HAC1 CLA"}="1HAC CLA"; $resn_exch{"HAC2 CLA"}="2HAC CLA";
$resn_exch{"HBC1 CLA"}="1HBC CLA"; $resn_exch{"HBC2 CLA"}="2HBC CLA"; $resn_exch{"HBC3 CLA"}="3HBC CLA";
$resn_exch{"HMD1 CLA"}="1HMD CLA"; $resn_exch{"HMD2 CLA"}="2HMD CLA"; $resn_exch{"HMD3 CLA"}="3HMD CLA";
$resn_exch{"HED1 CLA"}="1HED CLA"; $resn_exch{"HED2 CLA"}="2HED CLA"; $resn_exch{"HED3 CLA"}="3HED CLA";
$resn_exch{" H11 CLA"}="1H1  CLA"; $resn_exch{" H12 CLA"}="2H1  CLA"; $resn_exch{" H41 CLA"}="1H4  CLA";
$resn_exch{" H42 CLA"}="2H4  CLA"; $resn_exch{" H43 CLA"}="3H4  CLA"; $resn_exch{" H51 CLA"}="1H5  CLA";
$resn_exch{" H52 CLA"}="2H5  CLA"; $resn_exch{" H61 CLA"}="1H6  CLA"; $resn_exch{" H62 CLA"}="2H6  CLA";
$resn_exch{" H71 CLA"}="1H7  CLA"; $resn_exch{" H72 CLA"}="2H7  CLA"; $resn_exch{" H91 CLA"}="1H9  CLA";
$resn_exch{" H92 CLA"}="2H9  CLA"; $resn_exch{" H93 CLA"}="3H9  CLA"; $resn_exch{"H101 CLA"}="1H10 CLA";
$resn_exch{"H102 CLA"}="2H10 CLA"; $resn_exch{"H111 CLA"}="1H11 CLA"; $resn_exch{"H112 CLA"}="2H11 CLA";
$resn_exch{"H121 CLA"}="1H12 CLA"; $resn_exch{"H122 CLA"}="2H12 CLA"; $resn_exch{"H141 CLA"}="1H14 CLA";
$resn_exch{"H142 CLA"}="2H14 CLA"; $resn_exch{"H143 CLA"}="3H14 CLA"; $resn_exch{"H151 CLA"}="1H15 CLA";
$resn_exch{"H152 CLA"}="2H15 CLA"; $resn_exch{"H161 CLA"}="1H16 CLA"; $resn_exch{"H162 CLA"}="2H16 CLA";
$resn_exch{"H171 CLA"}="1H17 CLA"; $resn_exch{"H172 CLA"}="2H17 CLA"; $resn_exch{"H191 CLA"}="1H19 CLA";
$resn_exch{"H192 CLA"}="2H19 CLA"; $resn_exch{"H193 CLA"}="3H19 CLA"; $resn_exch{"H201 CLA"}="1H20 CLA";
$resn_exch{"H202 CLA"}="2H20 CLA"; $resn_exch{"H203 CLA"}="3H20 CLA"; $resn_exch{" H   CLB"}="1HN  CLB";
$resn_exch{" H2  CLB"}="2HN  CLB"; $resn_exch{" HB2 CLB"}="1HB  CLB"; $resn_exch{" HB3 CLB"}="2HB  CLB";
$resn_exch{" H71 CLB"}="1H7  CLB"; $resn_exch{" H72 CLB"}="2H7  CLB"; $resn_exch{" HC2 CLB"}=" H2  CLB";
$resn_exch{"HOB1 CLB"}="1HOB CLB"; $resn_exch{"HOB2 CLB"}="2HOB CLB"; $resn_exch{"H111 CLB"}="1H11 CLB";
$resn_exch{"H112 CLB"}="2H11 CLB"; $resn_exch{"H113 CLB"}="3H11 CLB"; $resn_exch{"H121 CLC"}="1H12 CLC";
$resn_exch{"H122 CLC"}="2H12 CLC"; $resn_exch{"H123 CLC"}="3H12 CLC"; $resn_exch{"H211 CLC"}="1H21 CLC";
$resn_exch{"H212 CLC"}="2H21 CLC"; $resn_exch{" H   CLD"}="1HN  CLD"; $resn_exch{" H2  CLD"}="2HN  CLD";
$resn_exch{" HB2 CLD"}="1HB  CLD"; $resn_exch{" HB3 CLD"}="2HB  CLD"; $resn_exch{" H71 CLD"}="1H7  CLD";
$resn_exch{" H72 CLD"}="2H7  CLD"; $resn_exch{" HC2 CLD"}=" H2  CLD"; $resn_exch{"HOB1 CLD"}="1HOB CLD";
$resn_exch{"HOB2 CLD"}="2HOB CLD"; $resn_exch{"H111 CLD"}="1H11 CLD"; $resn_exch{"H112 CLD"}="2H11 CLD";
$resn_exch{"H113 CLD"}="3H11 CLD"; $resn_exch{" H2  CLE"}=" HN  CLE"; $resn_exch{" HB2 CLE"}=" HB1 CLE";
$resn_exch{" HB3 CLE"}=" HB2 CLE"; $resn_exch{"HD11 CLE"}="1HD1 CLE"; $resn_exch{"HD12 CLE"}="2HD1 CLE";
$resn_exch{"HD13 CLE"}="3HD1 CLE"; $resn_exch{"HD21 CLE"}="1HD2 CLE"; $resn_exch{"HD22 CLE"}="2HD2 CLE";
$resn_exch{"HD23 CLE"}="3HD2 CLE"; $resn_exch{"HN21 CLE"}="1HN2 CLE"; $resn_exch{"HN22 CLE"}="2HN2 CLE";
$resn_exch{" H2  CLG"}=" HN  CLG"; $resn_exch{" HB2 CLG"}="1HB  CLG"; $resn_exch{" HB3 CLG"}="2HB  CLG";
$resn_exch{" HG2 CLG"}="1HG  CLG"; $resn_exch{" HG3 CLG"}="2HG  CLG"; $resn_exch{" HD2 CLG"}="1HD  CLG";
$resn_exch{" HD3 CLG"}="2HD  CLG"; $resn_exch{" HE2 CLG"}="1HE  CLG"; $resn_exch{" HE3 CLG"}="2HE  CLG";
$resn_exch{" HZ  CLG"}=" HNZ CLG"; $resn_exch{" HI1 CLG"}="1HI  CLG"; $resn_exch{" HI2 CLG"}="2HI  CLG";
$resn_exch{" HL1 CLG"}="1HL  CLG"; $resn_exch{" HL2 CLG"}="2HL  CLG"; $resn_exch{"HNM1 CLG"}="1HNM CLG";
$resn_exch{"HNM2 CLG"}="2HNM CLG"; $resn_exch{" H2  CLH"}=" HN  CLH"; $resn_exch{" HB2 CLH"}="1HB  CLH";
$resn_exch{" HB3 CLH"}="2HB  CLH"; $resn_exch{" HG2 CLH"}="1HG  CLH"; $resn_exch{" HG3 CLH"}="2HG  CLH";
$resn_exch{" HD2 CLH"}="1HD  CLH"; $resn_exch{" HD3 CLH"}="2HD  CLH"; $resn_exch{" HE2 CLH"}="1HE  CLH";
$resn_exch{" HE3 CLH"}="2HE  CLH"; $resn_exch{" HZ  CLH"}=" HNZ CLH"; $resn_exch{" HI1 CLH"}="1HI  CLH";
$resn_exch{" HI2 CLH"}="2HI  CLH"; $resn_exch{"H121 CLI"}="1H12 CLI"; $resn_exch{"H122 CLI"}="2H12 CLI";
$resn_exch{" H61 CLI"}="1H6  CLI"; $resn_exch{" H62 CLI"}="2H6  CLI"; $resn_exch{"H4C1 CLK"}="1H4C CLK";
$resn_exch{"H4C2 CLK"}="2H4C CLK"; $resn_exch{"H9N1 CLK"}="1H9N CLK"; $resn_exch{"H9N2 CLK"}="2H9N CLK";
$resn_exch{" H11 CLL"}="1H1  CLL"; $resn_exch{" H12 CLL"}="2H1  CLL"; $resn_exch{" H21 CLL"}="1H2  CLL";
$resn_exch{" H22 CLL"}="2H2  CLL"; $resn_exch{" H41 CLL"}="1H4  CLL"; $resn_exch{" H42 CLL"}="2H4  CLL";
$resn_exch{" H71 CLL"}="1H7  CLL"; $resn_exch{" H72 CLL"}="2H7  CLL"; $resn_exch{"H111 CLL"}="1H11 CLL";
$resn_exch{"H112 CLL"}="2H11 CLL"; $resn_exch{"H121 CLL"}="1H12 CLL"; $resn_exch{"H122 CLL"}="2H12 CLL";
$resn_exch{"H151 CLL"}="1H15 CLL"; $resn_exch{"H152 CLL"}="2H15 CLL"; $resn_exch{"H161 CLL"}="1H16 CLL";
$resn_exch{"H162 CLL"}="2H16 CLL"; $resn_exch{"H181 CLL"}="1H18 CLL"; $resn_exch{"H182 CLL"}="2H18 CLL";
$resn_exch{"H183 CLL"}="3H18 CLL"; $resn_exch{"H191 CLL"}="1H19 CLL"; $resn_exch{"H192 CLL"}="2H19 CLL";
$resn_exch{"H193 CLL"}="3H19 CLL"; $resn_exch{"H211 CLL"}="1H21 CLL"; $resn_exch{"H212 CLL"}="2H21 CLL";
$resn_exch{"H213 CLL"}="3H21 CLL"; $resn_exch{"H221 CLL"}="1H22 CLL"; $resn_exch{"H222 CLL"}="2H22 CLL";
$resn_exch{"H231 CLL"}="1H23 CLL"; $resn_exch{"H232 CLL"}="2H23 CLL"; $resn_exch{"H241 CLL"}="1H24 CLL";
$resn_exch{"H242 CLL"}="2H24 CLL"; $resn_exch{"H261 CLL"}="1H26 CLL"; $resn_exch{"H262 CLL"}="2H26 CLL";
$resn_exch{"H263 CLL"}="3H26 CLL"; $resn_exch{"H271 CLL"}="1H27 CLL"; $resn_exch{"H272 CLL"}="2H27 CLL";
$resn_exch{"H273 CLL"}="3H27 CLL"; $resn_exch{"H291 CLL"}="1H29 CLL"; $resn_exch{"H292 CLL"}="2H29 CLL";
$resn_exch{"H301 CLL"}="1H30 CLL"; $resn_exch{"H302 CLL"}="2H30 CLL"; $resn_exch{"H311 CLL"}="1H31 CLL";
$resn_exch{"H312 CLL"}="2H31 CLL"; $resn_exch{"H321 CLL"}="1H32 CLL"; $resn_exch{"H322 CLL"}="2H32 CLL";
$resn_exch{"H331 CLL"}="1H33 CLL"; $resn_exch{"H332 CLL"}="2H33 CLL"; $resn_exch{"H341 CLL"}="1H34 CLL";
$resn_exch{"H342 CLL"}="2H34 CLL"; $resn_exch{"H351 CLL"}="1H35 CLL"; $resn_exch{"H352 CLL"}="2H35 CLL";
$resn_exch{"H381 CLL"}="1H38 CLL"; $resn_exch{"H382 CLL"}="2H38 CLL"; $resn_exch{"H411 CLL"}="1H41 CLL";
$resn_exch{"H412 CLL"}="2H41 CLL"; $resn_exch{"H421 CLL"}="1H42 CLL"; $resn_exch{"H422 CLL"}="2H42 CLL";
$resn_exch{"H431 CLL"}="1H43 CLL"; $resn_exch{"H432 CLL"}="2H43 CLL"; $resn_exch{"H441 CLL"}="1H44 CLL";
$resn_exch{"H442 CLL"}="2H44 CLL"; $resn_exch{"H451 CLL"}="1H45 CLL"; $resn_exch{"H452 CLL"}="2H45 CLL";
$resn_exch{"H453 CLL"}="3H45 CLL"; $resn_exch{" H41 CLM"}="1H4  CLM"; $resn_exch{" H42 CLM"}="2H4  CLM";
$resn_exch{"HMA1 CLN"}="1HMA CLN"; $resn_exch{"HMA2 CLN"}="2HMA CLN"; $resn_exch{"HMA3 CLN"}="3HMA CLN";
$resn_exch{"HAA1 CLN"}="1HAA CLN"; $resn_exch{"HAA2 CLN"}="2HAA CLN"; $resn_exch{"HBA1 CLN"}="1HBA CLN";
$resn_exch{"HBA2 CLN"}="2HBA CLN"; $resn_exch{"HMB1 CLN"}="1HMB CLN"; $resn_exch{"HMB2 CLN"}="2HMB CLN";
$resn_exch{"HMB3 CLN"}="3HMB CLN"; $resn_exch{"HBB1 CLN"}="1HBB CLN"; $resn_exch{"HBB2 CLN"}="2HBB CLN";
$resn_exch{"HMC1 CLN"}="1HMC CLN"; $resn_exch{"HMC2 CLN"}="2HMC CLN"; $resn_exch{"HMC3 CLN"}="3HMC CLN";
$resn_exch{"HMD1 CLN"}="1HMD CLN"; $resn_exch{"HMD2 CLN"}="2HMD CLN"; $resn_exch{"HMD3 CLN"}="3HMD CLN";
$resn_exch{"HAD1 CLN"}="1HAD CLN"; $resn_exch{"HAD2 CLN"}="2HAD CLN"; $resn_exch{"HBD1 CLN"}="1HBD CLN";
$resn_exch{"HBD2 CLN"}="2HBD CLN"; $resn_exch{" H11 CLQ"}="1H1  CLQ"; $resn_exch{" H21 CLQ"}="1H2  CLQ";
$resn_exch{" H51 CLQ"}="1H5  CLQ"; $resn_exch{" H61 CLQ"}="1H6  CLQ"; $resn_exch{" H81 CLQ"}="1H8  CLQ";
$resn_exch{"HN21 CLQ"}="1HN2 CLQ"; $resn_exch{"H101 CLQ"}="1H10 CLQ"; $resn_exch{"H111 CLQ"}="1H11 CLQ";
$resn_exch{"H112 CLQ"}="2H11 CLQ"; $resn_exch{"H121 CLQ"}="1H12 CLQ"; $resn_exch{"H122 CLQ"}="2H12 CLQ";
$resn_exch{"H131 CLQ"}="1H13 CLQ"; $resn_exch{"H132 CLQ"}="2H13 CLQ"; $resn_exch{"H141 CLQ"}="1H14 CLQ";
$resn_exch{"H142 CLQ"}="2H14 CLQ"; $resn_exch{"H151 CLQ"}="1H15 CLQ"; $resn_exch{"H152 CLQ"}="2H15 CLQ";
$resn_exch{"H153 CLQ"}="3H15 CLQ"; $resn_exch{"H161 CLQ"}="1H16 CLQ"; $resn_exch{"H162 CLQ"}="2H16 CLQ";
$resn_exch{"H171 CLQ"}="1H17 CLQ"; $resn_exch{"H172 CLQ"}="2H17 CLQ"; $resn_exch{"H173 CLQ"}="3H17 CLQ";
$resn_exch{"H181 CLQ"}="1H18 CLQ"; $resn_exch{"H182 CLQ"}="2H18 CLQ"; $resn_exch{"H183 CLQ"}="3H18 CLQ";
$resn_exch{" H11 CLR"}="1H1  CLR"; $resn_exch{" H12 CLR"}="2H1  CLR"; $resn_exch{" H21 CLR"}="1H2  CLR";
$resn_exch{" H22 CLR"}="2H2  CLR"; $resn_exch{" H41 CLR"}="1H4  CLR"; $resn_exch{" H42 CLR"}="2H4  CLR";
$resn_exch{" H71 CLR"}="1H7  CLR"; $resn_exch{" H72 CLR"}="2H7  CLR"; $resn_exch{"H111 CLR"}="1H11 CLR";
$resn_exch{"H112 CLR"}="2H11 CLR"; $resn_exch{"H121 CLR"}="1H12 CLR"; $resn_exch{"H122 CLR"}="2H12 CLR";
$resn_exch{"H151 CLR"}="1H15 CLR"; $resn_exch{"H152 CLR"}="2H15 CLR"; $resn_exch{"H161 CLR"}="1H16 CLR";
$resn_exch{"H162 CLR"}="2H16 CLR"; $resn_exch{"H181 CLR"}="1H18 CLR"; $resn_exch{"H182 CLR"}="2H18 CLR";
$resn_exch{"H183 CLR"}="3H18 CLR"; $resn_exch{"H191 CLR"}="1H19 CLR"; $resn_exch{"H192 CLR"}="2H19 CLR";
$resn_exch{"H193 CLR"}="3H19 CLR"; $resn_exch{"H211 CLR"}="1H21 CLR"; $resn_exch{"H212 CLR"}="2H21 CLR";
$resn_exch{"H213 CLR"}="3H21 CLR"; $resn_exch{"H221 CLR"}="1H22 CLR"; $resn_exch{"H222 CLR"}="2H22 CLR";
$resn_exch{"H231 CLR"}="1H23 CLR"; $resn_exch{"H232 CLR"}="2H23 CLR"; $resn_exch{"H241 CLR"}="1H24 CLR";
$resn_exch{"H242 CLR"}="2H24 CLR"; $resn_exch{"H261 CLR"}="1H26 CLR"; $resn_exch{"H262 CLR"}="2H26 CLR";
$resn_exch{"H263 CLR"}="3H26 CLR"; $resn_exch{"H271 CLR"}="1H27 CLR"; $resn_exch{"H272 CLR"}="2H27 CLR";
$resn_exch{"H273 CLR"}="3H27 CLR"; $resn_exch{" H21 CLS"}="1H2  CLS"; $resn_exch{" H22 CLS"}="2H2  CLS";
$resn_exch{"H3'1 CLS"}="1H3' CLS"; $resn_exch{"H3'2 CLS"}="2H3' CLS"; $resn_exch{"H231 CLS"}="1H23 CLS";
$resn_exch{"H232 CLS"}="2H23 CLS"; $resn_exch{"H233 CLS"}="3H23 CLS"; $resn_exch{"H131 CLS"}="1H13 CLS";
$resn_exch{"H132 CLS"}="2H13 CLS"; $resn_exch{" HA1 CLT"}="1HA  CLT"; $resn_exch{" HA2 CLT"}="2HA  CLT";
$resn_exch{"HB11 CLT"}="1HB1 CLT"; $resn_exch{"HB12 CLT"}="2HB1 CLT"; $resn_exch{" HG1 CLT"}="1HG  CLT";
$resn_exch{" HG2 CLT"}="2HG  CLT"; $resn_exch{"HAG1 CLU"}="1HAG CLU"; $resn_exch{"HAG2 CLU"}="2HAG CLU";
$resn_exch{"HAF1 CLU"}="1HAF CLU"; $resn_exch{"HAF2 CLU"}="2HAF CLU"; $resn_exch{" H11 CLW"}="1H1  CLW";
$resn_exch{" H21 CLW"}="1H2  CLW"; $resn_exch{" H51 CLW"}="1H5  CLW"; $resn_exch{"HN11 CLW"}="1HN1 CLW";
$resn_exch{" H61 CLY"}="1H6  CLY"; $resn_exch{" H62 CLY"}="2H6  CLY"; $resn_exch{" H63 CLY"}="3H6  CLY";
$resn_exch{" H91 CLY"}="1H9  CLY"; $resn_exch{" H92 CLY"}="2H9  CLY"; $resn_exch{" H93 CLY"}="3H9  CLY";
$resn_exch{"H121 CLY"}="1H12 CLY"; $resn_exch{"H122 CLY"}="2H12 CLY"; $resn_exch{"H141 CLY"}="1H14 CLY";
$resn_exch{"H142 CLY"}="2H14 CLY"; $resn_exch{"H151 CLY"}="1H15 CLY"; $resn_exch{"H152 CLY"}="2H15 CLY";
$resn_exch{"H153 CLY"}="3H15 CLY"; $resn_exch{"H161 CLY"}="1H16 CLY"; $resn_exch{"H162 CLY"}="2H16 CLY";
$resn_exch{"H171 CLY"}="1H17 CLY"; $resn_exch{"H172 CLY"}="2H17 CLY"; $resn_exch{"H181 CLY"}="1H18 CLY";
$resn_exch{"H182 CLY"}="2H18 CLY"; $resn_exch{"H183 CLY"}="3H18 CLY"; $resn_exch{" H51 CLZ"}="1H5  CLZ";
$resn_exch{" H52 CLZ"}="2H5  CLZ"; $resn_exch{" H61 CLZ"}="1H6  CLZ"; $resn_exch{" H62 CLZ"}="2H6  CLZ";
$resn_exch{"H131 CLZ"}="1H13 CLZ"; $resn_exch{"H132 CLZ"}="2H13 CLZ"; $resn_exch{" H21  CM"}="1H2   CM";
$resn_exch{" H22  CM"}="2H2   CM"; $resn_exch{" H23  CM"}="3H2   CM"; $resn_exch{"H5'1 CM0"}="1H5* CM0";
$resn_exch{"H5'2 CM0"}="2H5* CM0"; $resn_exch{" H4' CM0"}=" H4* CM0"; $resn_exch{" H3' CM0"}=" H3* CM0";
$resn_exch{" H1' CM0"}=" H1* CM0"; $resn_exch{" H2' CM0"}=" H2* CM0"; $resn_exch{"H7C1 CM0"}="1H7C CM0";
$resn_exch{"H7C2 CM0"}="2H7C CM0"; $resn_exch{"H131 CM1"}="1H13 CM1"; $resn_exch{"H132 CM1"}="2H13 CM1";
$resn_exch{"H231 CM1"}="1H23 CM1"; $resn_exch{"H232 CM1"}="2H23 CM1"; $resn_exch{"H233 CM1"}="3H23 CM1";
$resn_exch{"H251 CM1"}="1H25 CM1"; $resn_exch{"H252 CM1"}="2H25 CM1"; $resn_exch{"H311 CM1"}="1H31 CM1";
$resn_exch{"H312 CM1"}="2H31 CM1"; $resn_exch{"H321 CM1"}="1H32 CM1"; $resn_exch{"H322 CM1"}="2H32 CM1";
$resn_exch{"H331 CM1"}="1H33 CM1"; $resn_exch{"H332 CM1"}="2H33 CM1"; $resn_exch{"H341 CM1"}="1H34 CM1";
$resn_exch{"H342 CM1"}="2H34 CM1"; $resn_exch{"H391 CM1"}="1H39 CM1"; $resn_exch{"H392 CM1"}="2H39 CM1";
$resn_exch{"H401 CM1"}="1H40 CM1"; $resn_exch{"H402 CM1"}="2H40 CM1"; $resn_exch{"H411 CM1"}="1H41 CM1";
$resn_exch{"H412 CM1"}="2H41 CM1"; $resn_exch{"H421 CM1"}="1H42 CM1"; $resn_exch{"H422 CM1"}="2H42 CM1";
$resn_exch{"H491 CM1"}="1H49 CM1"; $resn_exch{"H492 CM1"}="2H49 CM1"; $resn_exch{"H501 CM1"}="1H50 CM1";
$resn_exch{"H502 CM1"}="2H50 CM1"; $resn_exch{"H511 CM1"}="1H51 CM1"; $resn_exch{"H512 CM1"}="2H51 CM1";
$resn_exch{"H521 CM1"}="1H52 CM1"; $resn_exch{"H522 CM1"}="2H52 CM1"; $resn_exch{"H531 CM1"}="1H53 CM1";
$resn_exch{"H532 CM1"}="2H53 CM1"; $resn_exch{"H541 CM1"}="1H54 CM1"; $resn_exch{"H542 CM1"}="2H54 CM1";
$resn_exch{"H131 CM2"}="1H13 CM2"; $resn_exch{"H132 CM2"}="2H13 CM2"; $resn_exch{"H221 CM2"}="1H22 CM2";
$resn_exch{"H222 CM2"}="2H22 CM2"; $resn_exch{"H251 CM2"}="1H25 CM2"; $resn_exch{"H252 CM2"}="2H25 CM2";
$resn_exch{"H311 CM2"}="1H31 CM2"; $resn_exch{"H312 CM2"}="2H31 CM2"; $resn_exch{"H321 CM2"}="1H32 CM2";
$resn_exch{"H322 CM2"}="2H32 CM2"; $resn_exch{"H331 CM2"}="1H33 CM2"; $resn_exch{"H332 CM2"}="2H33 CM2";
$resn_exch{"H341 CM2"}="1H34 CM2"; $resn_exch{"H342 CM2"}="2H34 CM2"; $resn_exch{"H391 CM2"}="1H39 CM2";
$resn_exch{"H392 CM2"}="2H39 CM2"; $resn_exch{"H401 CM2"}="1H40 CM2"; $resn_exch{"H402 CM2"}="2H40 CM2";
$resn_exch{"H411 CM2"}="1H41 CM2"; $resn_exch{"H412 CM2"}="2H41 CM2"; $resn_exch{"H421 CM2"}="1H42 CM2";
$resn_exch{"H422 CM2"}="2H42 CM2"; $resn_exch{"H491 CM2"}="1H49 CM2"; $resn_exch{"H492 CM2"}="2H49 CM2";
$resn_exch{"H501 CM2"}="1H50 CM2"; $resn_exch{"H502 CM2"}="2H50 CM2"; $resn_exch{"H511 CM2"}="1H51 CM2";
$resn_exch{"H512 CM2"}="2H51 CM2"; $resn_exch{"H521 CM2"}="1H52 CM2"; $resn_exch{"H522 CM2"}="2H52 CM2";
$resn_exch{"H531 CM2"}="1H53 CM2"; $resn_exch{"H532 CM2"}="2H53 CM2"; $resn_exch{"H541 CM2"}="1H54 CM2";
$resn_exch{"H542 CM2"}="2H54 CM2"; $resn_exch{"H131 CM3"}="1H13 CM3"; $resn_exch{"H132 CM3"}="2H13 CM3";
$resn_exch{"H133 CM3"}="3H13 CM3"; $resn_exch{"H281 CM3"}="1H28 CM3"; $resn_exch{"H282 CM3"}="2H28 CM3";
$resn_exch{"H291 CM3"}="1H29 CM3"; $resn_exch{"H292 CM3"}="2H29 CM3"; $resn_exch{"H311 CM3"}="1H31 CM3";
$resn_exch{"H312 CM3"}="2H31 CM3"; $resn_exch{"H321 CM3"}="1H32 CM3"; $resn_exch{"H322 CM3"}="2H32 CM3";
$resn_exch{"H331 CM3"}="1H33 CM3"; $resn_exch{"H332 CM3"}="2H33 CM3"; $resn_exch{"H341 CM3"}="1H34 CM3";
$resn_exch{"H342 CM3"}="2H34 CM3"; $resn_exch{"H351 CM3"}="1H35 CM3"; $resn_exch{"H352 CM3"}="2H35 CM3";
$resn_exch{"H121 CM4"}="1H12 CM4"; $resn_exch{"H122 CM4"}="2H12 CM4"; $resn_exch{"H123 CM4"}="3H12 CM4";
$resn_exch{"H271 CM4"}="1H27 CM4"; $resn_exch{"H272 CM4"}="2H27 CM4"; $resn_exch{"H281 CM4"}="1H28 CM4";
$resn_exch{"H282 CM4"}="2H28 CM4"; $resn_exch{"H301 CM4"}="1H30 CM4"; $resn_exch{"H302 CM4"}="2H30 CM4";
$resn_exch{"H311 CM4"}="1H31 CM4"; $resn_exch{"H312 CM4"}="2H31 CM4"; $resn_exch{"H321 CM4"}="1H32 CM4";
$resn_exch{"H322 CM4"}="2H32 CM4"; $resn_exch{"H331 CM4"}="1H33 CM4"; $resn_exch{"H332 CM4"}="2H33 CM4";
$resn_exch{" H71 CM5"}="1H7  CM5"; $resn_exch{" H72 CM5"}="2H7  CM5"; $resn_exch{" H81 CM5"}="1H8  CM5";
$resn_exch{" H82 CM5"}="2H8  CM5"; $resn_exch{" H91 CM5"}="1H9  CM5"; $resn_exch{" H92 CM5"}="2H9  CM5";
$resn_exch{"H101 CM5"}="1H10 CM5"; $resn_exch{"H102 CM5"}="2H10 CM5"; $resn_exch{"H111 CM5"}="1H11 CM5";
$resn_exch{"H112 CM5"}="2H11 CM5"; $resn_exch{" H51 CM5"}="1H5  CM5"; $resn_exch{" H52 CM5"}="2H5  CM5";
$resn_exch{" H41 CM5"}="1H4  CM5"; $resn_exch{" H42 CM5"}="2H4  CM5"; $resn_exch{"H31A CM5"}="1H3  CM5";
$resn_exch{"H32A CM5"}="2H3  CM5"; $resn_exch{"H21A CM5"}="1H2  CM5"; $resn_exch{"H22A CM5"}="2H2  CM5";
$resn_exch{" H11 CM5"}="1H1  CM5"; $resn_exch{" H12 CM5"}="2H1  CM5"; $resn_exch{"H191 CM5"}="1H19 CM5";
$resn_exch{"H192 CM5"}="2H19 CM5"; $resn_exch{"H301 CM5"}="1H30 CM5"; $resn_exch{"H302 CM5"}="2H30 CM5";
$resn_exch{" HB1 CMA"}="1HB  CMA"; $resn_exch{" HB2 CMA"}="2HB  CMA"; $resn_exch{" H11 CMA"}="1H1  CMA";
$resn_exch{" H12 CMA"}="2H1  CMA"; $resn_exch{" H21 CMA"}="1H2  CMA"; $resn_exch{" H22 CMA"}="2H2  CMA";
$resn_exch{" HG1 CMA"}="1HG  CMA"; $resn_exch{" HG2 CMA"}="2HG  CMA"; $resn_exch{" HD1 CMA"}="1HD  CMA";
$resn_exch{" HD2 CMA"}="2HD  CMA"; $resn_exch{"HH11 CMA"}="1HH1 CMA"; $resn_exch{"HH12 CMA"}="2HH1 CMA";
$resn_exch{"HH21 CMA"}="1HH2 CMA"; $resn_exch{"H101 CMB"}="1H10 CMB"; $resn_exch{"H102 CMB"}="2H10 CMB";
$resn_exch{"H103 CMB"}="3H10 CMB"; $resn_exch{"H131 CMB"}="1H13 CMB"; $resn_exch{"H132 CMB"}="2H13 CMB";
$resn_exch{"H171 CMB"}="1H17 CMB"; $resn_exch{"H172 CMB"}="2H17 CMB"; $resn_exch{"H118 CMB"}="18H1 CMB";
$resn_exch{"H211 CMB"}="1H21 CMB"; $resn_exch{"H221 CMB"}="1H22 CMB"; $resn_exch{"H231 CMB"}="1H23 CMB";
$resn_exch{"H224 CMB"}="24H2 CMB"; $resn_exch{"H251 CMB"}="1H25 CMB"; $resn_exch{"H252 CMB"}="2H25 CMB";
$resn_exch{"H281 CMB"}="1H28 CMB"; $resn_exch{"H531 CMB"}="1H53 CMB"; $resn_exch{"H532 CMB"}="2H53 CMB";
$resn_exch{" H2A CMC"}=" AH2 CMC"; $resn_exch{"H61A CMC"}="AH61 CMC"; $resn_exch{"H62A CMC"}="AH62 CMC";
$resn_exch{" H8A CMC"}=" AH8 CMC"; $resn_exch{" H1B CMC"}="AH1* CMC"; $resn_exch{" H2B CMC"}="AH2* CMC";
$resn_exch{"HO2A CMC"}="AHO2 CMC"; $resn_exch{" H3B CMC"}="AH3* CMC"; $resn_exch{"HOA8 CMC"}="8HOA CMC";
$resn_exch{"HOA9 CMC"}="9HOA CMC"; $resn_exch{" H4B CMC"}="AH4* CMC"; $resn_exch{"H51A CMC"}="AH51 CMC";
$resn_exch{"H52A CMC"}="AH52 CMC"; $resn_exch{"HOA2 CMC"}="2HOA CMC"; $resn_exch{"HOA5 CMC"}="5HOA CMC";
$resn_exch{"H121 CMC"}="1H12 CMC"; $resn_exch{"H122 CMC"}="2H12 CMC"; $resn_exch{"H131 CMC"}="1H13 CMC";
$resn_exch{"H132 CMC"}="2H13 CMC"; $resn_exch{"H133 CMC"}="3H13 CMC"; $resn_exch{"H141 CMC"}="1H14 CMC";
$resn_exch{"H142 CMC"}="2H14 CMC"; $resn_exch{"H143 CMC"}="3H14 CMC"; $resn_exch{" H71 CMC"}="1H7  CMC";
$resn_exch{" H72 CMC"}="2H7  CMC"; $resn_exch{" H61 CMC"}="1H6  CMC"; $resn_exch{" H62 CMC"}="2H6  CMC";
$resn_exch{" H31 CMC"}="1H3  CMC"; $resn_exch{" H32 CMC"}="2H3  CMC"; $resn_exch{" H21 CMC"}="1H2  CMC";
$resn_exch{" H22 CMC"}="2H2  CMC"; $resn_exch{"HO22 CMC"}="2HO2 CMC"; $resn_exch{"H111 CMD"}="1H11 CMD";
$resn_exch{"H112 CMD"}="2H11 CMD"; $resn_exch{"HO12 CMD"}="2HO1 CMD"; $resn_exch{"H141 CMD"}="1H14 CMD";
$resn_exch{"H142 CMD"}="2H14 CMD"; $resn_exch{"HO14 CMD"}="4HO1 CMD"; $resn_exch{"H151 CMD"}="1H15 CMD";
$resn_exch{"H152 CMD"}="2H15 CMD"; $resn_exch{"HO17 CMD"}="7HO1 CMD"; $resn_exch{"H211 CMD"}="1H21 CMD";
$resn_exch{"H212 CMD"}="2H21 CMD"; $resn_exch{"H213 CMD"}="3H21 CMD"; $resn_exch{" H1' CMD"}=" H1* CMD";
$resn_exch{"H2'1 CMD"}="1H2* CMD"; $resn_exch{"H2'2 CMD"}="2H2* CMD"; $resn_exch{" H3' CMD"}=" H3* CMD";
$resn_exch{" H4' CMD"}=" H4* CMD"; $resn_exch{"HO4' CMD"}="*HO4 CMD"; $resn_exch{" H5' CMD"}=" H5* CMD";
$resn_exch{"H6'1 CMD"}="1H6* CMD"; $resn_exch{"H6'2 CMD"}="2H6* CMD"; $resn_exch{"H6'3 CMD"}="3H6* CMD";
$resn_exch{"H231 CMD"}="1H23 CMD"; $resn_exch{"H232 CMD"}="2H23 CMD"; $resn_exch{"H631 CMD"}="1H63 CMD";
$resn_exch{"H632 CMD"}="2H63 CMD"; $resn_exch{"H531 CMD"}="1H53 CMD"; $resn_exch{"H532 CMD"}="2H53 CMD";
$resn_exch{" H   CME"}="1HN  CME"; $resn_exch{" H2  CME"}="2HN  CME"; $resn_exch{" HB2 CME"}="1HB  CME";
$resn_exch{" HB3 CME"}="2HB  CME"; $resn_exch{" HE2 CME"}="1HE  CME"; $resn_exch{" HE3 CME"}="2HE  CME";
$resn_exch{" HZ2 CME"}="1HZ  CME"; $resn_exch{" HZ3 CME"}="2HZ  CME"; $resn_exch{" HH  CME"}=" HO  CME";
$resn_exch{"H241 CMF"}="1H24 CMF"; $resn_exch{"H242 CMF"}="2H24 CMF"; $resn_exch{"H261 CMF"}="1H26 CMF";
$resn_exch{"H262 CMF"}="2H26 CMF"; $resn_exch{"H271 CMF"}="1H27 CMF"; $resn_exch{"H272 CMF"}="2H27 CMF";
$resn_exch{"H251 CMF"}="1H25 CMF"; $resn_exch{"H252 CMF"}="2H25 CMF"; $resn_exch{"H221 CMF"}="1H22 CMF";
$resn_exch{"H222 CMF"}="2H22 CMF"; $resn_exch{"H171 CMF"}="1H17 CMF"; $resn_exch{"H172 CMF"}="2H17 CMF";
$resn_exch{"H201 CMF"}="1H20 CMF"; $resn_exch{"H202 CMF"}="2H20 CMF"; $resn_exch{"H211 CMF"}="1H21 CMF";
$resn_exch{"H212 CMF"}="2H21 CMF"; $resn_exch{"H191 CMF"}="1H19 CMF"; $resn_exch{"H192 CMF"}="2H19 CMF";
$resn_exch{"H181 CMF"}="1H18 CMF"; $resn_exch{"H182 CMF"}="2H18 CMF"; $resn_exch{" H92 CMG"}="2H9  CMG";
$resn_exch{" H91 CMG"}="1H9  CMG"; $resn_exch{" H22 CMG"}="2H2  CMG"; $resn_exch{" H21 CMG"}="1H2  CMG";
$resn_exch{"H112 CMG"}="2H11 CMG"; $resn_exch{"H111 CMG"}="1H11 CMG"; $resn_exch{"H122 CMG"}="2H12 CMG";
$resn_exch{"H121 CMG"}="1H12 CMG"; $resn_exch{"H132 CMG"}="2H13 CMG"; $resn_exch{"H131 CMG"}="1H13 CMG";
$resn_exch{"H142 CMG"}="2H14 CMG"; $resn_exch{"H141 CMG"}="1H14 CMG"; $resn_exch{"H152 CMG"}="2H15 CMG";
$resn_exch{"H151 CMG"}="1H15 CMG"; $resn_exch{" H   CMH"}="1HN  CMH"; $resn_exch{" H2  CMH"}="2HN  CMH";
$resn_exch{" HB2 CMH"}="1HBC CMH"; $resn_exch{" HB3 CMH"}="2HBC CMH"; $resn_exch{" HM1 CMH"}="1HM  CMH";
$resn_exch{" HM2 CMH"}="2HM  CMH"; $resn_exch{" HM3 CMH"}="3HM  CMH"; $resn_exch{"H101 CMI"}="1H10 CMI";
$resn_exch{"H102 CMI"}="2H10 CMI"; $resn_exch{"H103 CMI"}="3H10 CMI"; $resn_exch{"H131 CMI"}="1H13 CMI";
$resn_exch{"H132 CMI"}="2H13 CMI"; $resn_exch{"H171 CMI"}="1H17 CMI"; $resn_exch{"H172 CMI"}="2H17 CMI";
$resn_exch{"H321 CMI"}="1H32 CMI"; $resn_exch{"H322 CMI"}="2H32 CMI"; $resn_exch{"H331 CMI"}="1H33 CMI";
$resn_exch{"H332 CMI"}="2H33 CMI"; $resn_exch{"H333 CMI"}="3H33 CMI"; $resn_exch{"H371 CMI"}="1H37 CMI";
$resn_exch{"H372 CMI"}="2H37 CMI"; $resn_exch{"H373 CMI"}="3H37 CMI"; $resn_exch{"H4C1 CMK"}="1H4C CMK";
$resn_exch{"H4C2 CMK"}="2H4C CMK"; $resn_exch{" H1' CMK"}=" H1* CMK"; $resn_exch{" H2' CMK"}=" H2* CMK";
$resn_exch{" H3' CMK"}=" H3* CMK"; $resn_exch{" H4' CMK"}=" H4* CMK"; $resn_exch{"H5'1 CMK"}="1H5* CMK";
$resn_exch{"H5'2 CMK"}="2H5* CMK"; $resn_exch{"H3C1 CMK"}="1H3C CMK"; $resn_exch{"H3C2 CMK"}="2H3C CMK";
$resn_exch{"H8C1 CMK"}="1H8C CMK"; $resn_exch{"H8C2 CMK"}="2H8C CMK"; $resn_exch{"H151 CMM"}="1H15 CMM";
$resn_exch{"H152 CMM"}="2H15 CMM"; $resn_exch{"H161 CMM"}="1H16 CMM"; $resn_exch{"H162 CMM"}="2H16 CMM";
$resn_exch{"H231 CMM"}="1H23 CMM"; $resn_exch{"H232 CMM"}="2H23 CMM"; $resn_exch{"H271 CMM"}="1H27 CMM";
$resn_exch{"H272 CMM"}="2H27 CMM"; $resn_exch{"H273 CMM"}="3H27 CMM"; $resn_exch{"H311 CMM"}="1H31 CMM";
$resn_exch{"H312 CMM"}="2H31 CMM"; $resn_exch{"H331 CMM"}="1H33 CMM"; $resn_exch{"H332 CMM"}="2H33 CMM";
$resn_exch{"H371 CMM"}="1H37 CMM"; $resn_exch{"H372 CMM"}="2H37 CMM"; $resn_exch{"H361 CMM"}="1H36 CMM";
$resn_exch{"H362 CMM"}="2H36 CMM"; $resn_exch{"H351 CMM"}="1H35 CMM"; $resn_exch{"H352 CMM"}="2H35 CMM";
$resn_exch{"H401 CMM"}="1H40 CMM"; $resn_exch{"H402 CMM"}="2H40 CMM"; $resn_exch{" H62 CMN"}="2H6  CMN";
$resn_exch{" H61 CMN"}="1H6  CMN"; $resn_exch{" H12 CMN"}="2H1  CMN"; $resn_exch{" H11 CMN"}="1H1  CMN";
$resn_exch{"H2O1 CMN"}="1H2O CMN"; $resn_exch{"H3O1 CMN"}="1H3O CMN"; $resn_exch{"H4O1 CMN"}="1H4O CMN";
$resn_exch{"HOP2 CMP"}="2HOP CMP"; $resn_exch{"H5'1 CMP"}="1H5* CMP"; $resn_exch{"H5'2 CMP"}="2H5* CMP";
$resn_exch{" H4' CMP"}=" H4* CMP"; $resn_exch{" H3' CMP"}=" H3* CMP"; $resn_exch{" H2' CMP"}=" H2* CMP";
$resn_exch{"HO2' CMP"}="*HO2 CMP"; $resn_exch{" H1' CMP"}=" H1* CMP"; $resn_exch{"HN61 CMP"}="1HN6 CMP";
$resn_exch{"HN62 CMP"}="2HN6 CMP"; $resn_exch{"H21A CMQ"}="1H2  CMQ"; $resn_exch{"H22A CMQ"}="2H2  CMQ";
$resn_exch{"H111 CMQ"}="1H11 CMQ"; $resn_exch{"H112 CMQ"}="2H11 CMQ"; $resn_exch{"H131 CMQ"}="1H13 CMQ";
$resn_exch{"H132 CMQ"}="2H13 CMQ"; $resn_exch{"H133 CMQ"}="3H13 CMQ"; $resn_exch{"H141 CMQ"}="1H14 CMQ";
$resn_exch{"H142 CMQ"}="2H14 CMQ"; $resn_exch{"H143 CMQ"}="3H14 CMQ"; $resn_exch{"H241 CMQ"}="1H24 CMQ";
$resn_exch{"H242 CMQ"}="2H24 CMQ"; $resn_exch{"H243 CMQ"}="3H24 CMQ"; $resn_exch{"H171 CMQ"}="1H17 CMQ";
$resn_exch{"H172 CMQ"}="2H17 CMQ"; $resn_exch{"HMP1 CMR"}="1HMP CMR"; $resn_exch{"HMP2 CMR"}="2HMP CMR";
$resn_exch{"HMP3 CMR"}="3HMP CMR"; $resn_exch{" H5' CMR"}="1H5* CMR"; $resn_exch{"H5'' CMR"}="2H5* CMR";
$resn_exch{" H4' CMR"}=" H4* CMR"; $resn_exch{" H1' CMR"}=" H1* CMR"; $resn_exch{" H41 CMR"}="1H4  CMR";
$resn_exch{" H42 CMR"}="2H4  CMR"; $resn_exch{" H3' CMR"}=" H3* CMR"; $resn_exch{" H2' CMR"}="1H2* CMR";
$resn_exch{"H2'' CMR"}="2H2* CMR"; $resn_exch{"HO3' CMR"}="H3T  CMR"; $resn_exch{"HOP3 CMR"}="3HOP CMR";
$resn_exch{" H21 CMS"}="1H2  CMS"; $resn_exch{" H22 CMS"}="2H2  CMS"; $resn_exch{" H23 CMS"}="3H2  CMS";
$resn_exch{" H31 CMS"}="1H3  CMS"; $resn_exch{" H32 CMS"}="2H3  CMS"; $resn_exch{"HN11 CMS"}="1HN1 CMS";
$resn_exch{"HN12 CMS"}="2HN1 CMS"; $resn_exch{" H2  CMT"}=" HN2 CMT"; $resn_exch{" HB2 CMT"}="1HB  CMT";
$resn_exch{" HB3 CMT"}="2HB  CMT"; $resn_exch{" H11 CMT"}="1H1  CMT"; $resn_exch{" H12 CMT"}="2H1  CMT";
$resn_exch{" H13 CMT"}="3H1  CMT"; $resn_exch{"H4C1 CMU"}="1H4C CMU"; $resn_exch{"H4C2 CMU"}="2H4C CMU";
$resn_exch{"H6C1 CMU"}="1H6C CMU"; $resn_exch{"H6C2 CMU"}="2H6C CMU"; $resn_exch{"H7C1 CMU"}="1H7C CMU";
$resn_exch{"H7C2 CMU"}="2H7C CMU"; $resn_exch{"H8C1 CMU"}="1H8C CMU"; $resn_exch{"H8C2 CMU"}="2H8C CMU";
$resn_exch{" H2A CMX"}=" AH2 CMX"; $resn_exch{"H61A CMX"}="AH61 CMX"; $resn_exch{"H62A CMX"}="AH62 CMX";
$resn_exch{" H8A CMX"}=" AH8 CMX"; $resn_exch{" H1B CMX"}="AH1* CMX"; $resn_exch{" H2B CMX"}="AH2* CMX";
$resn_exch{"HO2A CMX"}="AHO2 CMX"; $resn_exch{" H3B CMX"}="AH3* CMX"; $resn_exch{"HOA8 CMX"}="8HOA CMX";
$resn_exch{"HOA9 CMX"}="9HOA CMX"; $resn_exch{" H4B CMX"}="AH4* CMX"; $resn_exch{"H51A CMX"}="AH51 CMX";
$resn_exch{"H52A CMX"}="AH52 CMX"; $resn_exch{"HOA2 CMX"}="2HOA CMX"; $resn_exch{"HOA5 CMX"}="5HOA CMX";
$resn_exch{"H121 CMX"}="1H12 CMX"; $resn_exch{"H122 CMX"}="2H12 CMX"; $resn_exch{"H131 CMX"}="1H13 CMX";
$resn_exch{"H132 CMX"}="2H13 CMX"; $resn_exch{"H133 CMX"}="3H13 CMX"; $resn_exch{"H141 CMX"}="1H14 CMX";
$resn_exch{"H142 CMX"}="2H14 CMX"; $resn_exch{"H143 CMX"}="3H14 CMX"; $resn_exch{" H71 CMX"}="1H7  CMX";
$resn_exch{" H72 CMX"}="2H7  CMX"; $resn_exch{" H61 CMX"}="1H6  CMX"; $resn_exch{" H62 CMX"}="2H6  CMX";
$resn_exch{" H31 CMX"}="1H3  CMX"; $resn_exch{" H32 CMX"}="2H3  CMX"; $resn_exch{" H21 CMX"}="1H2  CMX";
$resn_exch{" H22 CMX"}="2H2  CMX"; $resn_exch{" H11 CMX"}="1H1  CMX"; $resn_exch{" H12 CMX"}="2H1  CMX";
$resn_exch{"H2Y1 CMY"}="1H2Y CMY"; $resn_exch{"H2Y2 CMY"}="2H2Y CMY"; $resn_exch{"H3Y1 CMY"}="1H3Y CMY";
$resn_exch{"H3Y2 CMY"}="2H3Y CMY"; $resn_exch{"H4Y1 CMY"}="1H4Y CMY"; $resn_exch{"H4Y2 CMY"}="2H4Y CMY";
$resn_exch{"H5Y1 CMY"}="1H5Y CMY"; $resn_exch{"H5Y2 CMY"}="2H5Y CMY"; $resn_exch{"H6Y1 CMY"}="1H6Y CMY";
$resn_exch{"H6Y2 CMY"}="2H6Y CMY"; $resn_exch{"H7Y1 CMY"}="1H7Y CMY"; $resn_exch{"H7Y2 CMY"}="2H7Y CMY";
$resn_exch{" H11 CMY"}="1H1  CMY"; $resn_exch{" H12 CMY"}="2H1  CMY"; $resn_exch{" H11 CMZ"}="1H1  CMZ";
$resn_exch{" H12 CMZ"}="2H1  CMZ"; $resn_exch{" H13 CMZ"}="3H1  CMZ"; $resn_exch{" H61 CMZ"}="1H6  CMZ";
$resn_exch{" H62 CMZ"}="2H6  CMZ"; $resn_exch{" H63 CMZ"}="3H6  CMZ"; $resn_exch{" H91 CMZ"}="1H9  CMZ";
$resn_exch{" H92 CMZ"}="2H9  CMZ"; $resn_exch{"H121 CMZ"}="1H12 CMZ"; $resn_exch{"H122 CMZ"}="2H12 CMZ";
$resn_exch{"H141 CMZ"}="1H14 CMZ"; $resn_exch{"H142 CMZ"}="2H14 CMZ"; $resn_exch{"H151 CMZ"}="1H15 CMZ";
$resn_exch{"H152 CMZ"}="2H15 CMZ"; $resn_exch{"H171 CMZ"}="1H17 CMZ"; $resn_exch{"H172 CMZ"}="2H17 CMZ";
$resn_exch{"H181 CMZ"}="1H18 CMZ"; $resn_exch{"H182 CMZ"}="2H18 CMZ"; $resn_exch{"H131 CN2"}="1H13 CN2";
$resn_exch{"H132 CN2"}="2H13 CN2"; $resn_exch{"H101 CN2"}="1H10 CN2"; $resn_exch{"H102 CN2"}="2H10 CN2";
$resn_exch{" H91 CN2"}="1H9  CN2"; $resn_exch{" H92 CN2"}="2H9  CN2"; $resn_exch{" H21 CN2"}="1H2  CN2";
$resn_exch{" H22 CN2"}="2H2  CN2"; $resn_exch{" H23 CN2"}="3H2  CN2"; $resn_exch{" H41 CN2"}="1H4  CN2";
$resn_exch{" H42 CN2"}="2H4  CN2"; $resn_exch{" H43 CN2"}="3H4  CN2"; $resn_exch{" H61 CN2"}="1H6  CN2";
$resn_exch{" H62 CN2"}="2H6  CN2"; $resn_exch{" H63 CN2"}="3H6  CN2"; $resn_exch{"H181 CN2"}="1H18 CN2";
$resn_exch{"H182 CN2"}="2H18 CN2"; $resn_exch{"H183 CN2"}="3H18 CN2"; $resn_exch{"HOA2 CNA"}="2HOA CNA";
$resn_exch{"H51A CNA"}="AH51 CNA"; $resn_exch{"H52A CNA"}="AH52 CNA"; $resn_exch{" H4B CNA"}="AH4* CNA";
$resn_exch{" H3B CNA"}="AH3* CNA"; $resn_exch{"HO3A CNA"}="AHO3 CNA"; $resn_exch{" H2B CNA"}="AH2* CNA";
$resn_exch{"HO2A CNA"}="AHO2 CNA"; $resn_exch{" H1B CNA"}="AH1* CNA"; $resn_exch{" H8A CNA"}=" AH8 CNA";
$resn_exch{"H61A CNA"}="AH61 CNA"; $resn_exch{"H62A CNA"}="AH62 CNA"; $resn_exch{" H2A CNA"}=" AH2 CNA";
$resn_exch{"H51N CNA"}="NH51 CNA"; $resn_exch{"H52N CNA"}="NH52 CNA"; $resn_exch{" H4D CNA"}="NH4* CNA";
$resn_exch{"H4'1 CNA"}="1H4* CNA"; $resn_exch{"H4'2 CNA"}="2H4* CNA"; $resn_exch{" H3D CNA"}="NH3* CNA";
$resn_exch{"HO3N CNA"}="NHO3 CNA"; $resn_exch{" H2D CNA"}="NH2* CNA"; $resn_exch{"HO2N CNA"}="NHO2 CNA";
$resn_exch{" H1D CNA"}="NH1* CNA"; $resn_exch{" H2N CNA"}=" NH2 CNA"; $resn_exch{"H71N CNA"}="NH71 CNA";
$resn_exch{"H72N CNA"}="NH72 CNA"; $resn_exch{" H4N CNA"}=" NH4 CNA"; $resn_exch{" H5N CNA"}=" NH5 CNA";
$resn_exch{" H6N CNA"}=" NH6 CNA"; $resn_exch{"H201 CNC"}="1H20 CNC"; $resn_exch{"H202 CNC"}="2H20 CNC";
$resn_exch{"H203 CNC"}="3H20 CNC"; $resn_exch{"H251 CNC"}="1H25 CNC"; $resn_exch{"H252 CNC"}="2H25 CNC";
$resn_exch{"H253 CNC"}="3H25 CNC"; $resn_exch{"H261 CNC"}="1H26 CNC"; $resn_exch{"H262 CNC"}="2H26 CNC";
$resn_exch{"H291 CNC"}="1H29 CNC"; $resn_exch{"H292 CNC"}="2H29 CNC"; $resn_exch{"H301 CNC"}="1H30 CNC";
$resn_exch{"H302 CNC"}="2H30 CNC"; $resn_exch{"H311 CNC"}="1H31 CNC"; $resn_exch{"H312 CNC"}="2H31 CNC";
$resn_exch{"H331 CNC"}="1H33 CNC"; $resn_exch{"H332 CNC"}="2H33 CNC"; $resn_exch{"H351 CNC"}="1H35 CNC";
$resn_exch{"H352 CNC"}="2H35 CNC"; $resn_exch{"H353 CNC"}="3H35 CNC"; $resn_exch{"H361 CNC"}="1H36 CNC";
$resn_exch{"H362 CNC"}="2H36 CNC"; $resn_exch{"H363 CNC"}="3H36 CNC"; $resn_exch{"H371 CNC"}="1H37 CNC";
$resn_exch{"H372 CNC"}="2H37 CNC"; $resn_exch{"H401 CNC"}="1H40 CNC"; $resn_exch{"H402 CNC"}="2H40 CNC";
$resn_exch{"H411 CNC"}="1H41 CNC"; $resn_exch{"H412 CNC"}="2H41 CNC"; $resn_exch{"H421 CNC"}="1H42 CNC";
$resn_exch{"H422 CNC"}="2H42 CNC"; $resn_exch{"H451 CNC"}="1H45 CNC"; $resn_exch{"H452 CNC"}="2H45 CNC";
$resn_exch{"H461 CNC"}="1H46 CNC"; $resn_exch{"H462 CNC"}="2H46 CNC"; $resn_exch{"H463 CNC"}="3H46 CNC";
$resn_exch{"H471 CNC"}="1H47 CNC"; $resn_exch{"H472 CNC"}="2H47 CNC"; $resn_exch{"H473 CNC"}="3H47 CNC";
$resn_exch{"H481 CNC"}="1H48 CNC"; $resn_exch{"H482 CNC"}="2H48 CNC"; $resn_exch{"H491 CNC"}="1H49 CNC";
$resn_exch{"H492 CNC"}="2H49 CNC"; $resn_exch{"H521 CNC"}="1H52 CNC"; $resn_exch{"H522 CNC"}="2H52 CNC";
$resn_exch{"H531 CNC"}="1H53 CNC"; $resn_exch{"H532 CNC"}="2H53 CNC"; $resn_exch{"H533 CNC"}="3H53 CNC";
$resn_exch{"H541 CNC"}="1H54 CNC"; $resn_exch{"H542 CNC"}="2H54 CNC"; $resn_exch{"H543 CNC"}="3H54 CNC";
$resn_exch{"H551 CNC"}="1H55 CNC"; $resn_exch{"H552 CNC"}="2H55 CNC"; $resn_exch{"H561 CNC"}="1H56 CNC";
$resn_exch{"H562 CNC"}="2H56 CNC"; $resn_exch{"H601 CNC"}="1H60 CNC"; $resn_exch{"H602 CNC"}="2H60 CNC";
$resn_exch{"H621 CNC"}="1H62 CNC"; $resn_exch{"H622 CNC"}="2H62 CNC"; $resn_exch{"H1P1 CNC"}="1H1P CNC";
$resn_exch{"H1P2 CNC"}="2H1P CNC"; $resn_exch{"H3P1 CNC"}="1H3P CNC"; $resn_exch{"H3P2 CNC"}="2H3P CNC";
$resn_exch{"H3P3 CNC"}="3H3P CNC"; $resn_exch{"HOP5 CNC"}="5HOP CNC"; $resn_exch{"HOR7 CNC"}="7HOR CNC";
$resn_exch{"H5R1 CNC"}="1H5R CNC"; $resn_exch{"H5R2 CNC"}="2H5R CNC"; $resn_exch{"HOR8 CNC"}="8HOR CNC";
$resn_exch{"HM51 CNC"}="1HM5 CNC"; $resn_exch{"HM52 CNC"}="2HM5 CNC"; $resn_exch{"HM53 CNC"}="3HM5 CNC";
$resn_exch{"HM61 CNC"}="1HM6 CNC"; $resn_exch{"HM62 CNC"}="2HM6 CNC"; $resn_exch{"HM63 CNC"}="3HM6 CNC";
$resn_exch{"HOA2 CND"}="2HOA CND"; $resn_exch{"H51A CND"}="AH51 CND"; $resn_exch{"H52A CND"}="AH52 CND";
$resn_exch{" H4B CND"}="AH4* CND"; $resn_exch{" H3B CND"}="AH3* CND"; $resn_exch{"HO3A CND"}="AHO3 CND";
$resn_exch{" H2B CND"}="AH2* CND"; $resn_exch{"HO2A CND"}="AHO2 CND"; $resn_exch{" H1B CND"}="AH1* CND";
$resn_exch{" H8A CND"}=" AH8 CND"; $resn_exch{"H61A CND"}="AH61 CND"; $resn_exch{"H62A CND"}="AH62 CND";
$resn_exch{" H2A CND"}=" AH2 CND"; $resn_exch{"H51N CND"}="NH51 CND"; $resn_exch{"H52N CND"}="NH52 CND";
$resn_exch{" H4D CND"}="NH4* CND"; $resn_exch{" H3D CND"}="NH3* CND"; $resn_exch{"HO3N CND"}="NHO3 CND";
$resn_exch{" H2D CND"}="NH2* CND"; $resn_exch{"HO2N CND"}="NHO2 CND"; $resn_exch{" H1D CND"}="NH1* CND";
$resn_exch{" H2N CND"}=" NH2 CND"; $resn_exch{"H71N CND"}="NH71 CND"; $resn_exch{"H72N CND"}="NH72 CND";
$resn_exch{" H4N CND"}=" NH4 CND"; $resn_exch{" H5N CND"}=" NH5 CND"; $resn_exch{" H6N CND"}=" NH6 CND";
$resn_exch{"H291 CNE"}="1H29 CNE"; $resn_exch{"H292 CNE"}="2H29 CNE"; $resn_exch{"H293 CNE"}="3H29 CNE";
$resn_exch{"H301 CNE"}="1H30 CNE"; $resn_exch{"H302 CNE"}="2H30 CNE"; $resn_exch{"H311 CNE"}="1H31 CNE";
$resn_exch{"H312 CNE"}="2H31 CNE"; $resn_exch{"H313 CNE"}="3H31 CNE"; $resn_exch{"H191 CNE"}="1H19 CNE";
$resn_exch{"H192 CNE"}="2H19 CNE"; $resn_exch{"H171 CNE"}="1H17 CNE"; $resn_exch{"H172 CNE"}="2H17 CNE";
$resn_exch{"H173 CNE"}="3H17 CNE"; $resn_exch{"H161 CNE"}="1H16 CNE"; $resn_exch{"H162 CNE"}="2H16 CNE";
$resn_exch{"H163 CNE"}="3H16 CNE"; $resn_exch{"H111 CNE"}="1H11 CNE"; $resn_exch{"H112 CNE"}="2H11 CNE";
$resn_exch{" H81 CNE"}="1H8  CNE"; $resn_exch{" H82 CNE"}="2H8  CNE"; $resn_exch{" H71 CNE"}="1H7  CNE";
$resn_exch{" H72 CNE"}="2H7  CNE"; $resn_exch{"HC21 CNH"}="1HC2 CNH"; $resn_exch{"HC22 CNH"}="2HC2 CNH";
$resn_exch{"HC23 CNH"}="3HC2 CNH"; $resn_exch{"HC31 CNH"}="1HC3 CNH"; $resn_exch{"HC32 CNH"}="2HC3 CNH";
$resn_exch{"HC33 CNH"}="3HC3 CNH"; $resn_exch{" H21 CNL"}="1H2  CNL"; $resn_exch{" H22 CNL"}="2H2  CNL";
$resn_exch{" H31 CNL"}="1H3  CNL"; $resn_exch{" H32 CNL"}="2H3  CNL"; $resn_exch{" H51 CNL"}="1H5  CNL";
$resn_exch{" H52 CNL"}="2H5  CNL"; $resn_exch{" H61 CNL"}="1H6  CNL"; $resn_exch{" H62 CNL"}="2H6  CNL";
$resn_exch{" H71 CNL"}="1H7  CNL"; $resn_exch{" H72 CNL"}="2H7  CNL"; $resn_exch{" H73 CNL"}="3H7  CNL";
$resn_exch{" H91 CNL"}="1H9  CNL"; $resn_exch{" H92 CNL"}="2H9  CNL"; $resn_exch{" H93 CNL"}="3H9  CNL";
$resn_exch{"H101 CNL"}="1H10 CNL"; $resn_exch{"H102 CNL"}="2H10 CNL"; $resn_exch{"H103 CNL"}="3H10 CNL";
$resn_exch{" HN1 CNM"}="1HN  CNM"; $resn_exch{" HN2 CNM"}="2HN  CNM"; $resn_exch{" H21 CNM"}="1H2  CNM";
$resn_exch{" H22 CNM"}="2H2  CNM"; $resn_exch{" H23 CNM"}="3H2  CNM"; $resn_exch{"HN11 CNN"}="1HN1 CNN";
$resn_exch{"HN12 CNN"}="2HN1 CNN"; $resn_exch{"HC51 CNO"}="1HC5 CNO"; $resn_exch{"HC52 CNO"}="2HC5 CNO";
$resn_exch{"H53C CNO"}="3CH5 CNO"; $resn_exch{"HC61 CNO"}="1HC6 CNO"; $resn_exch{"HC62 CNO"}="2HC6 CNO";
$resn_exch{"HC63 CNO"}="3HC6 CNO"; $resn_exch{"H10C CNO"}="CH10 CNO"; $resn_exch{"H11C CNO"}="CH11 CNO";
$resn_exch{"H12C CNO"}="CH12 CNO"; $resn_exch{"H141 CNO"}="1H14 CNO"; $resn_exch{"H142 CNO"}="2H14 CNO";
$resn_exch{"H17N CNO"}="NH17 CNO"; $resn_exch{"H19C CNO"}="CH19 CNO"; $resn_exch{"H20C CNO"}="CH20 CNO";
$resn_exch{"H21C CNO"}="CH21 CNO"; $resn_exch{"H22C CNO"}="CH22 CNO"; $resn_exch{"H24N CNO"}="NH24 CNO";
$resn_exch{"H271 CNO"}="1H27 CNO"; $resn_exch{"H272 CNO"}="2H27 CNO"; $resn_exch{"H29C CNO"}="CH29 CNO";
$resn_exch{"H30C CNO"}="CH30 CNO"; $resn_exch{"H31C CNO"}="CH31 CNO"; $resn_exch{"H32C CNO"}="CH32 CNO";
$resn_exch{"H361 CNO"}="1H36 CNO"; $resn_exch{"H362 CNO"}="2H36 CNO"; $resn_exch{"H363 CNO"}="3H36 CNO";
$resn_exch{"H371 CNO"}="1H37 CNO"; $resn_exch{"H372 CNO"}="2H37 CNO"; $resn_exch{"H373 CNO"}="3H37 CNO";
$resn_exch{"H40O CNO"}="OH40 CNO"; $resn_exch{"HO1B CNP"}="BHO1 CNP"; $resn_exch{"H121 CNP"}="1H12 CNP";
$resn_exch{"H122 CNP"}="2H12 CNP"; $resn_exch{"H141 CNP"}="1H14 CNP"; $resn_exch{"H142 CNP"}="2H14 CNP";
$resn_exch{" H31 CNP"}="1H3  CNP"; $resn_exch{" H32 CNP"}="2H3  CNP"; $resn_exch{"H111 CNP"}="1H11 CNP";
$resn_exch{"H112 CNP"}="2H11 CNP"; $resn_exch{"H113 CNP"}="3H11 CNP"; $resn_exch{" H91 CNP"}="1H9  CNP";
$resn_exch{" H92 CNP"}="2H9  CNP"; $resn_exch{"H121 CNQ"}="1H12 CNQ"; $resn_exch{"H122 CNQ"}="2H12 CNQ";
$resn_exch{" H31 CNR"}="1H3  CNR"; $resn_exch{" H32 CNR"}="2H3  CNR"; $resn_exch{" H33 CNR"}="3H3  CNR";
$resn_exch{"H241 CNS"}="1H24 CNS"; $resn_exch{"H242 CNS"}="2H24 CNS"; $resn_exch{"H243 CNS"}="3H24 CNS";
$resn_exch{"H231 CNS"}="1H23 CNS"; $resn_exch{"H232 CNS"}="2H23 CNS"; $resn_exch{"H221 CNS"}="1H22 CNS";
$resn_exch{"H222 CNS"}="2H22 CNS"; $resn_exch{"H211 CNS"}="1H21 CNS"; $resn_exch{"H212 CNS"}="2H21 CNS";
$resn_exch{"H201 CNS"}="1H20 CNS"; $resn_exch{"H202 CNS"}="2H20 CNS"; $resn_exch{"H191 CNS"}="1H19 CNS";
$resn_exch{"H192 CNS"}="2H19 CNS"; $resn_exch{"H181 CNS"}="1H18 CNS"; $resn_exch{"H182 CNS"}="2H18 CNS";
$resn_exch{"H171 CNS"}="1H17 CNS"; $resn_exch{"H172 CNS"}="2H17 CNS"; $resn_exch{"H161 CNS"}="1H16 CNS";
$resn_exch{"H162 CNS"}="2H16 CNS"; $resn_exch{"H151 CNS"}="1H15 CNS"; $resn_exch{"H152 CNS"}="2H15 CNS";
$resn_exch{"H141 CNS"}="1H14 CNS"; $resn_exch{"H142 CNS"}="2H14 CNS"; $resn_exch{"H131 CNS"}="1H13 CNS";
$resn_exch{"H132 CNS"}="2H13 CNS"; $resn_exch{"H121 CNS"}="1H12 CNS"; $resn_exch{"H122 CNS"}="2H12 CNS";
$resn_exch{"H111 CNS"}="1H11 CNS"; $resn_exch{"H112 CNS"}="2H11 CNS"; $resn_exch{"H101 CNS"}="1H10 CNS";
$resn_exch{"H102 CNS"}="2H10 CNS"; $resn_exch{" H91 CNS"}="1H9  CNS"; $resn_exch{" H92 CNS"}="2H9  CNS";
$resn_exch{" H71 CNS"}="1H7  CNS"; $resn_exch{" H72 CNS"}="2H7  CNS"; $resn_exch{" H61 CNS"}="1H6  CNS";
$resn_exch{" H62 CNS"}="2H6  CNS"; $resn_exch{" H31 CNS"}="1H3  CNS"; $resn_exch{" H32 CNS"}="2H3  CNS";
$resn_exch{" H41 CNS"}="1H4  CNS"; $resn_exch{" H42 CNS"}="2H4  CNS"; $resn_exch{" H51 CNS"}="1H5  CNS";
$resn_exch{" H52 CNS"}="2H5  CNS"; $resn_exch{" H21 CNS"}="1H2  CNS"; $resn_exch{" H22 CNS"}="2H2  CNS";
$resn_exch{" H11 CNS"}="1H1  CNS"; $resn_exch{" H12 CNS"}="2H1  CNS"; $resn_exch{" H13 CNS"}="3H1  CNS";
$resn_exch{"HMC1 CNT"}="1HMC CNT"; $resn_exch{"HMC2 CNT"}="2HMC CNT"; $resn_exch{"HMC3 CNT"}="3HMC CNT";
$resn_exch{"HBC1 CNT"}="1HBC CNT"; $resn_exch{"HBC2 CNT"}="2HBC CNT"; $resn_exch{"H611 CNY"}="1H61 CNY";
$resn_exch{"H612 CNY"}="2H61 CNY"; $resn_exch{"H11A CNY"}="1H1  CNY"; $resn_exch{"H12A CNY"}="2H1  CNY";
$resn_exch{"H121 CNY"}="1H12 CNY"; $resn_exch{"H122 CNY"}="2H12 CNY"; $resn_exch{"H221 CNY"}="1H22 CNY";
$resn_exch{"H222 CNY"}="2H22 CNY"; $resn_exch{"H321 CNY"}="1H32 CNY"; $resn_exch{"H322 CNY"}="2H32 CNY";
$resn_exch{"H531 CNY"}="1H53 CNY"; $resn_exch{"H532 CNY"}="2H53 CNY"; $resn_exch{"H241 CNY"}="1H24 CNY";
$resn_exch{"H242 CNY"}="2H24 CNY"; $resn_exch{"H641 CNY"}="1H64 CNY"; $resn_exch{"H642 CNY"}="2H64 CNY";
$resn_exch{"H21A CNY"}="1H2  CNY"; $resn_exch{" H22 CNY"}="2H2  CNY"; $resn_exch{"H181 CO1"}="1H18 CO1";
$resn_exch{"H182 CO1"}="2H18 CO1"; $resn_exch{" H41 CO1"}="1H4  CO1"; $resn_exch{" H42 CO1"}="2H4  CO1";
$resn_exch{" H81 CO1"}="1H8  CO1"; $resn_exch{" H82 CO1"}="2H8  CO1"; $resn_exch{"H111 CO1"}="1H11 CO1";
$resn_exch{"H112 CO1"}="2H11 CO1"; $resn_exch{"H121 CO1"}="1H12 CO1"; $resn_exch{"H122 CO1"}="2H12 CO1";
$resn_exch{"H123 CO1"}="3H12 CO1"; $resn_exch{"H131 CO1"}="1H13 CO1"; $resn_exch{"H132 CO1"}="2H13 CO1";
$resn_exch{" H61 CO1"}="1H6  CO1"; $resn_exch{" H62 CO1"}="2H6  CO1"; $resn_exch{"H211 CO1"}="1H21 CO1";
$resn_exch{"H212 CO1"}="2H21 CO1"; $resn_exch{"H213 CO1"}="3H21 CO1"; $resn_exch{"H201 CO1"}="1H20 CO1";
$resn_exch{"H202 CO1"}="2H20 CO1"; $resn_exch{"H231 CO1"}="1H23 CO1"; $resn_exch{"H232 CO1"}="2H23 CO1";
$resn_exch{"H281 CO1"}="1H28 CO1"; $resn_exch{"H282 CO1"}="2H28 CO1"; $resn_exch{"H283 CO1"}="3H28 CO1";
$resn_exch{"H291 CO1"}="1H29 CO1"; $resn_exch{"H292 CO1"}="2H29 CO1"; $resn_exch{"H293 CO1"}="3H29 CO1";
$resn_exch{"H161 CO1"}="1H16 CO1"; $resn_exch{"H162 CO1"}="2H16 CO1"; $resn_exch{"H163 CO1"}="3H16 CO1";
$resn_exch{"H151 CO1"}="1H15 CO1"; $resn_exch{"H152 CO1"}="2H15 CO1"; $resn_exch{"H191 CO1"}="1H19 CO1";
$resn_exch{"H192 CO1"}="2H19 CO1"; $resn_exch{"H2'1 CO4"}="1H2* CO4"; $resn_exch{"H2'2 CO4"}="2H2* CO4";
$resn_exch{"H4'1 CO4"}="1H4* CO4"; $resn_exch{"H4'2 CO4"}="2H4* CO4"; $resn_exch{" H7' CO4"}=" H7* CO4";
$resn_exch{"H8'1 CO4"}="1H8* CO4"; $resn_exch{"H8'2 CO4"}="2H8* CO4"; $resn_exch{"H5B1 CO4"}="1H5B CO4";
$resn_exch{"H5B2 CO4"}="2H5B CO4"; $resn_exch{"H5B3 CO4"}="3H5B CO4"; $resn_exch{"H4B1 CO4"}="1H4B CO4";
$resn_exch{"H4B2 CO4"}="2H4B CO4"; $resn_exch{"H4B3 CO4"}="3H4B CO4"; $resn_exch{"H3B1 CO4"}="1H3B CO4";
$resn_exch{"H3B2 CO4"}="2H3B CO4"; $resn_exch{"H3B3 CO4"}="3H3B CO4"; $resn_exch{"H9'1 CO4"}="1H9* CO4";
$resn_exch{"H9'2 CO4"}="2H9* CO4"; $resn_exch{"H9'3 CO4"}="3H9* CO4"; $resn_exch{"H511 CO4"}="1H51 CO4";
$resn_exch{"H512 CO4"}="2H51 CO4"; $resn_exch{"H513 CO4"}="3H51 CO4"; $resn_exch{"HO11 CO5"}="1HO1 CO5";
$resn_exch{"HO12 CO5"}="2HO1 CO5"; $resn_exch{"HO21 CO5"}="1HO2 CO5"; $resn_exch{"HO22 CO5"}="2HO2 CO5";
$resn_exch{"HO31 CO5"}="1HO3 CO5"; $resn_exch{"HO32 CO5"}="2HO3 CO5"; $resn_exch{"HO41 CO5"}="1HO4 CO5";
$resn_exch{"HO42 CO5"}="2HO4 CO5"; $resn_exch{"HO51 CO5"}="1HO5 CO5"; $resn_exch{"HO52 CO5"}="2HO5 CO5";
$resn_exch{"HN61 CO6"}="1HN6 CO6"; $resn_exch{"HN62 CO6"}="2HN6 CO6"; $resn_exch{" H1' CO6"}=" H1* CO6";
$resn_exch{" H2' CO6"}=" H2* CO6"; $resn_exch{" H3' CO6"}=" H3* CO6"; $resn_exch{" H4' CO6"}=" H4* CO6";
$resn_exch{"H5'1 CO6"}="1H5* CO6"; $resn_exch{"H5'2 CO6"}="2H5* CO6"; $resn_exch{"H121 CO6"}="1H12 CO6";
$resn_exch{"H122 CO6"}="2H12 CO6"; $resn_exch{"H131 CO6"}="1H13 CO6"; $resn_exch{"H132 CO6"}="2H13 CO6";
$resn_exch{"H133 CO6"}="3H13 CO6"; $resn_exch{"H141 CO6"}="1H14 CO6"; $resn_exch{"H142 CO6"}="2H14 CO6";
$resn_exch{"H143 CO6"}="3H14 CO6"; $resn_exch{" H71 CO6"}="1H7  CO6"; $resn_exch{" H72 CO6"}="2H7  CO6";
$resn_exch{" H61 CO6"}="1H6  CO6"; $resn_exch{" H62 CO6"}="2H6  CO6"; $resn_exch{" H31 CO6"}="1H3  CO6";
$resn_exch{" H32 CO6"}="2H3  CO6"; $resn_exch{" H21 CO6"}="1H2  CO6"; $resn_exch{" H22 CO6"}="2H2  CO6";
$resn_exch{" H11 CO6"}="1H1  CO6"; $resn_exch{" H12 CO6"}="2H1  CO6"; $resn_exch{" H13 CO6"}="3H1  CO6";
$resn_exch{" H41 CO6"}="1H4  CO6"; $resn_exch{" H42 CO6"}="2H4  CO6"; $resn_exch{" H43 CO6"}="3H4  CO6";
$resn_exch{" H2A CO8"}=" AH2 CO8"; $resn_exch{"H61A CO8"}="AH61 CO8"; $resn_exch{"H62A CO8"}="AH62 CO8";
$resn_exch{" H8A CO8"}=" AH8 CO8"; $resn_exch{" H1B CO8"}="AH1* CO8"; $resn_exch{" H2B CO8"}="AH2* CO8";
$resn_exch{"HO2A CO8"}="AHO2 CO8"; $resn_exch{" H3B CO8"}="AH3* CO8"; $resn_exch{"HOA8 CO8"}="8HOA CO8";
$resn_exch{"HOA9 CO8"}="9HOA CO8"; $resn_exch{" H4B CO8"}="AH4* CO8"; $resn_exch{"H51A CO8"}="AH51 CO8";
$resn_exch{"H52A CO8"}="AH52 CO8"; $resn_exch{"HOA2 CO8"}="2HOA CO8"; $resn_exch{"HOA5 CO8"}="5HOA CO8";
$resn_exch{"H121 CO8"}="1H12 CO8"; $resn_exch{"H122 CO8"}="2H12 CO8"; $resn_exch{"H131 CO8"}="1H13 CO8";
$resn_exch{"H132 CO8"}="2H13 CO8"; $resn_exch{"H133 CO8"}="3H13 CO8"; $resn_exch{"H141 CO8"}="1H14 CO8";
$resn_exch{"H142 CO8"}="2H14 CO8"; $resn_exch{"H143 CO8"}="3H14 CO8"; $resn_exch{" H71 CO8"}="1H7  CO8";
$resn_exch{" H72 CO8"}="2H7  CO8"; $resn_exch{" H61 CO8"}="1H6  CO8"; $resn_exch{" H62 CO8"}="2H6  CO8";
$resn_exch{" H31 CO8"}="1H3  CO8"; $resn_exch{" H32 CO8"}="2H3  CO8"; $resn_exch{" H21 CO8"}="1H2  CO8";
$resn_exch{" H22 CO8"}="2H2  CO8"; $resn_exch{"H2'1 CO8"}="1H2* CO8"; $resn_exch{"H2'2 CO8"}="2H2* CO8";
$resn_exch{"H3'1 CO8"}="1H3* CO8"; $resn_exch{"H3'2 CO8"}="2H3* CO8"; $resn_exch{"H4'1 CO8"}="1H4* CO8";
$resn_exch{"H4'2 CO8"}="2H4* CO8"; $resn_exch{"H5'1 CO8"}="1H5* CO8"; $resn_exch{"H5'2 CO8"}="2H5* CO8";
$resn_exch{"H6'1 CO8"}="1H6* CO8"; $resn_exch{"H6'2 CO8"}="2H6* CO8"; $resn_exch{"H7'1 CO8"}="1H7* CO8";
$resn_exch{"H7'2 CO8"}="2H7* CO8"; $resn_exch{"H8'1 CO8"}="1H8* CO8"; $resn_exch{"H8'2 CO8"}="2H8* CO8";
$resn_exch{"H8'3 CO8"}="3H8* CO8"; $resn_exch{"H11A CO9"}="1H1  CO9"; $resn_exch{" H12 CO9"}="2H1  CO9";
$resn_exch{" H13 CO9"}="3H1  CO9"; $resn_exch{"HN11 CO9"}="1HN1 CO9"; $resn_exch{"HN12 CO9"}="2HN1 CO9";
$resn_exch{" H61 CO9"}="1H6  CO9"; $resn_exch{" H62 CO9"}="2H6  CO9"; $resn_exch{" H71 CO9"}="1H7  CO9";
$resn_exch{" H72 CO9"}="2H7  CO9"; $resn_exch{" H81 CO9"}="1H8  CO9"; $resn_exch{" H82 CO9"}="2H8  CO9";
$resn_exch{"H121 CO9"}="1H12 CO9"; $resn_exch{"H122 CO9"}="2H12 CO9"; $resn_exch{" H2A COA"}=" AH2 COA";
$resn_exch{"H61A COA"}="AH61 COA"; $resn_exch{"H62A COA"}="AH62 COA"; $resn_exch{" H8A COA"}=" AH8 COA";
$resn_exch{" H1B COA"}="AH1* COA"; $resn_exch{" H2B COA"}="AH2* COA"; $resn_exch{"HO2A COA"}="AHO2 COA";
$resn_exch{" H3B COA"}="AH3* COA"; $resn_exch{"HOA8 COA"}="8HOA COA"; $resn_exch{"HOA9 COA"}="9HOA COA";
$resn_exch{" H4B COA"}="AH4* COA"; $resn_exch{"H51A COA"}="AH51 COA"; $resn_exch{"H52A COA"}="AH52 COA";
$resn_exch{"HOA2 COA"}="2HOA COA"; $resn_exch{"HOA5 COA"}="5HOA COA"; $resn_exch{"H121 COA"}="1H12 COA";
$resn_exch{"H122 COA"}="2H12 COA"; $resn_exch{"H131 COA"}="1H13 COA"; $resn_exch{"H132 COA"}="2H13 COA";
$resn_exch{"H133 COA"}="3H13 COA"; $resn_exch{"H141 COA"}="1H14 COA"; $resn_exch{"H142 COA"}="2H14 COA";
$resn_exch{"H143 COA"}="3H14 COA"; $resn_exch{" H71 COA"}="1H7  COA"; $resn_exch{" H72 COA"}="2H7  COA";
$resn_exch{" H61 COA"}="1H6  COA"; $resn_exch{" H62 COA"}="2H6  COA"; $resn_exch{" H31 COA"}="1H3  COA";
$resn_exch{" H32 COA"}="2H3  COA"; $resn_exch{" H21 COA"}="1H2  COA"; $resn_exch{" H22 COA"}="2H2  COA";
$resn_exch{"HN71 COD"}="1HN7 COD"; $resn_exch{"HN72 COD"}="2HN7 COD"; $resn_exch{"H181 COD"}="1H18 COD";
$resn_exch{"H182 COD"}="2H18 COD"; $resn_exch{"H281 COD"}="1H28 COD"; $resn_exch{"H282 COD"}="2H28 COD";
$resn_exch{"H301 COD"}="1H30 COD"; $resn_exch{"H302 COD"}="2H30 COD"; $resn_exch{"H303 COD"}="3H30 COD";
$resn_exch{"H311 COD"}="1H31 COD"; $resn_exch{"H312 COD"}="2H31 COD"; $resn_exch{"H313 COD"}="3H31 COD";
$resn_exch{"H371 COD"}="1H37 COD"; $resn_exch{"H372 COD"}="2H37 COD"; $resn_exch{"H381 COD"}="1H38 COD";
$resn_exch{"H382 COD"}="2H38 COD"; $resn_exch{"H421 COD"}="1H42 COD"; $resn_exch{"H422 COD"}="2H42 COD";
$resn_exch{"H431 COD"}="1H43 COD"; $resn_exch{"H432 COD"}="2H43 COD"; $resn_exch{"HN21 COE"}="1HN2 COE";
$resn_exch{"HN22 COE"}="2HN2 COE"; $resn_exch{"HN41 COE"}="1HN4 COE"; $resn_exch{"HN42 COE"}="2HN4 COE";
$resn_exch{" H91 COE"}="1H9  COE"; $resn_exch{" H92 COE"}="2H9  COE"; $resn_exch{"H101 COE"}="1H10 COE";
$resn_exch{"H102 COE"}="2H10 COE"; $resn_exch{"H103 COE"}="3H10 COE"; $resn_exch{" HB1 COE"}="1HB  COE";
$resn_exch{" HB2 COE"}="2HB  COE"; $resn_exch{" HG1 COE"}="1HG  COE"; $resn_exch{" HG2 COE"}="2HG  COE";
$resn_exch{"HOE2 COE"}="2HOE COE"; $resn_exch{" H2A COF"}=" AH2 COF"; $resn_exch{"H61A COF"}="AH61 COF";
$resn_exch{"H62A COF"}="AH62 COF"; $resn_exch{" H8A COF"}=" AH8 COF"; $resn_exch{" H1B COF"}="AH1* COF";
$resn_exch{" H2B COF"}="AH2* COF"; $resn_exch{"HO2A COF"}="AHO2 COF"; $resn_exch{" H3B COF"}="AH3* COF";
$resn_exch{"HOA8 COF"}="8HOA COF"; $resn_exch{"HOA9 COF"}="9HOA COF"; $resn_exch{" H4B COF"}="AH4* COF";
$resn_exch{"H51A COF"}="AH51 COF"; $resn_exch{"H52A COF"}="AH52 COF"; $resn_exch{"HOA2 COF"}="2HOA COF";
$resn_exch{"HOA5 COF"}="5HOA COF"; $resn_exch{"H121 COF"}="1H12 COF"; $resn_exch{"H122 COF"}="2H12 COF";
$resn_exch{"H131 COF"}="1H13 COF"; $resn_exch{"H132 COF"}="2H13 COF"; $resn_exch{"H133 COF"}="3H13 COF";
$resn_exch{"H141 COF"}="1H14 COF"; $resn_exch{"H142 COF"}="2H14 COF"; $resn_exch{"H143 COF"}="3H14 COF";
$resn_exch{" H71 COF"}="1H7  COF"; $resn_exch{" H72 COF"}="2H7  COF"; $resn_exch{" H61 COF"}="1H6  COF";
$resn_exch{" H62 COF"}="2H6  COF"; $resn_exch{" H31 COF"}="1H3  COF"; $resn_exch{" H32 COF"}="2H3  COF";
$resn_exch{" H21 COF"}="1H2  COF"; $resn_exch{" H22 COF"}="2H2  COF"; $resn_exch{"HI11 COF"}="1HI1 COF";
$resn_exch{"HI12 COF"}="2HI1 COF"; $resn_exch{" H5' COG"}=" H5* COG"; $resn_exch{" H7' COG"}=" H7* COG";
$resn_exch{"H211 COG"}="1H21 COG"; $resn_exch{"H212 COG"}="2H21 COG"; $resn_exch{"H213 COG"}="3H21 COG";
$resn_exch{"H7'1 COG"}="1H7' COG"; $resn_exch{"H7'2 COG"}="2H7' COG"; $resn_exch{"H611 COG"}="1H61 COG";
$resn_exch{"H612 COG"}="2H61 COG"; $resn_exch{"H613 COG"}="3H61 COG"; $resn_exch{"H511 COG"}="1H51 COG";
$resn_exch{"H512 COG"}="2H51 COG"; $resn_exch{"H513 COG"}="3H51 COG"; $resn_exch{"HN21 COG"}="1HN2 COG";
$resn_exch{"HN22 COG"}="2HN2 COG"; $resn_exch{"HN41 COG"}="1HN4 COG"; $resn_exch{"HN42 COG"}="2HN4 COG";
$resn_exch{" H8' COG"}=" H8* COG"; $resn_exch{"HMA1 COH"}="1HMA COH"; $resn_exch{"HMA2 COH"}="2HMA COH";
$resn_exch{"HMA3 COH"}="3HMA COH"; $resn_exch{"HAA1 COH"}="1HAA COH"; $resn_exch{"HAA2 COH"}="2HAA COH";
$resn_exch{"HBA1 COH"}="1HBA COH"; $resn_exch{"HBA2 COH"}="2HBA COH"; $resn_exch{"HMB1 COH"}="1HMB COH";
$resn_exch{"HMB2 COH"}="2HMB COH"; $resn_exch{"HMB3 COH"}="3HMB COH"; $resn_exch{"HBB1 COH"}="1HBB COH";
$resn_exch{"HBB2 COH"}="2HBB COH"; $resn_exch{"HMC1 COH"}="1HMC COH"; $resn_exch{"HMC2 COH"}="2HMC COH";
$resn_exch{"HMC3 COH"}="3HMC COH"; $resn_exch{"HBC1 COH"}="1HBC COH"; $resn_exch{"HBC2 COH"}="2HBC COH";
$resn_exch{"HMD1 COH"}="1HMD COH"; $resn_exch{"HMD2 COH"}="2HMD COH"; $resn_exch{"HMD3 COH"}="3HMD COH";
$resn_exch{"HAD1 COH"}="1HAD COH"; $resn_exch{"HAD2 COH"}="2HAD COH"; $resn_exch{"HBD1 COH"}="1HBD COH";
$resn_exch{"HBD2 COH"}="2HBD COH"; $resn_exch{" H31 COI"}="1H3  COI"; $resn_exch{" H32 COI"}="2H3  COI";
$resn_exch{" H51 COI"}="1H5  COI"; $resn_exch{" H52 COI"}="2H5  COI"; $resn_exch{" H53 COI"}="3H5  COI";
$resn_exch{" H61 COI"}="1H6  COI"; $resn_exch{" H62 COI"}="2H6  COI"; $resn_exch{" H63 COI"}="3H6  COI";
$resn_exch{"H481 COJ"}="1H48 COJ"; $resn_exch{"H482 COJ"}="2H48 COJ"; $resn_exch{"H491 COJ"}="1H49 COJ";
$resn_exch{"H492 COJ"}="2H49 COJ"; $resn_exch{"H461 COJ"}="1H46 COJ"; $resn_exch{"H462 COJ"}="2H46 COJ";
$resn_exch{"H463 COJ"}="3H46 COJ"; $resn_exch{"H471 COJ"}="1H47 COJ"; $resn_exch{"H472 COJ"}="2H47 COJ";
$resn_exch{"H473 COJ"}="3H47 COJ"; $resn_exch{"H601 COJ"}="1H60 COJ"; $resn_exch{"H602 COJ"}="2H60 COJ";
$resn_exch{"H541 COJ"}="1H54 COJ"; $resn_exch{"H542 COJ"}="2H54 COJ"; $resn_exch{"H543 COJ"}="3H54 COJ";
$resn_exch{"H551 COJ"}="1H55 COJ"; $resn_exch{"H552 COJ"}="2H55 COJ"; $resn_exch{"H561 COJ"}="1H56 COJ";
$resn_exch{"H562 COJ"}="2H56 COJ"; $resn_exch{"H421 COJ"}="1H42 COJ"; $resn_exch{"H422 COJ"}="2H42 COJ";
$resn_exch{"H251 COJ"}="1H25 COJ"; $resn_exch{"H252 COJ"}="2H25 COJ"; $resn_exch{"H253 COJ"}="3H25 COJ";
$resn_exch{"H261 COJ"}="1H26 COJ"; $resn_exch{"H262 COJ"}="2H26 COJ"; $resn_exch{" H31 COJ"}="1H3  COJ";
$resn_exch{"H301 COJ"}="1H30 COJ"; $resn_exch{"H302 COJ"}="2H30 COJ"; $resn_exch{"H641 COJ"}="1H64 COJ";
$resn_exch{"H642 COJ"}="2H64 COJ"; $resn_exch{"H643 COJ"}="3H64 COJ"; $resn_exch{"H201 COJ"}="1H20 COJ";
$resn_exch{"H202 COJ"}="2H20 COJ"; $resn_exch{"H203 COJ"}="3H20 COJ"; $resn_exch{"H311 COJ"}="1H31 COJ";
$resn_exch{"H312 COJ"}="2H31 COJ"; $resn_exch{"H361 COJ"}="1H36 COJ"; $resn_exch{"H362 COJ"}="2H36 COJ";
$resn_exch{"H363 COJ"}="3H36 COJ"; $resn_exch{"H371 COJ"}="1H37 COJ"; $resn_exch{"H372 COJ"}="2H37 COJ";
$resn_exch{"H411 COJ"}="1H41 COJ"; $resn_exch{"H412 COJ"}="2H41 COJ"; $resn_exch{"H351 COJ"}="1H35 COJ";
$resn_exch{"H352 COJ"}="2H35 COJ"; $resn_exch{"H353 COJ"}="3H35 COJ"; $resn_exch{"HN61 COK"}="1HN6 COK";
$resn_exch{"HN62 COK"}="2HN6 COK"; $resn_exch{" H1' COK"}=" H1* COK"; $resn_exch{" H2' COK"}=" H2* COK";
$resn_exch{" H3' COK"}=" H3* COK"; $resn_exch{" H4' COK"}=" H4* COK"; $resn_exch{"H5'1 COK"}="1H5* COK";
$resn_exch{"H5'2 COK"}="2H5* COK"; $resn_exch{"H121 COK"}="1H12 COK"; $resn_exch{"H122 COK"}="2H12 COK";
$resn_exch{"H131 COK"}="1H13 COK"; $resn_exch{"H132 COK"}="2H13 COK"; $resn_exch{"H133 COK"}="3H13 COK";
$resn_exch{"H141 COK"}="1H14 COK"; $resn_exch{"H142 COK"}="2H14 COK"; $resn_exch{"H143 COK"}="3H14 COK";
$resn_exch{" H71 COK"}="1H7  COK"; $resn_exch{" H72 COK"}="2H7  COK"; $resn_exch{" H61 COK"}="1H6  COK";
$resn_exch{" H62 COK"}="2H6  COK"; $resn_exch{" H31 COK"}="1H3  COK"; $resn_exch{" H32 COK"}="2H3  COK";
$resn_exch{" H21 COK"}="1H2  COK"; $resn_exch{" H22 COK"}="2H2  COK"; $resn_exch{"H501 COK"}="1H50 COK";
$resn_exch{"H502 COK"}="2H50 COK"; $resn_exch{"H511 COK"}="1H51 COK"; $resn_exch{"H512 COK"}="2H51 COK";
$resn_exch{" H21 COL"}="1H2  COL"; $resn_exch{"H22A COL"}="2H2  COL"; $resn_exch{" H51 COL"}="1H5  COL";
$resn_exch{" H52 COL"}="2H5  COL"; $resn_exch{" H61 COL"}="1H6  COL"; $resn_exch{" H62 COL"}="2H6  COL";
$resn_exch{" H11 COM"}="1H1  COM"; $resn_exch{" H12 COM"}="2H1  COM"; $resn_exch{" H21 COM"}="1H2  COM";
$resn_exch{" H22 COM"}="2H2  COM"; $resn_exch{"HOS3 COM"}="3HOS COM"; $resn_exch{"HN11 CON"}="1HN1 CON";
$resn_exch{"HN12 CON"}="2HN1 CON"; $resn_exch{"HN13 CON"}="3HN1 CON"; $resn_exch{"HN21 CON"}="1HN2 CON";
$resn_exch{"HN22 CON"}="2HN2 CON"; $resn_exch{"HN23 CON"}="3HN2 CON"; $resn_exch{"HN31 CON"}="1HN3 CON";
$resn_exch{"HN32 CON"}="2HN3 CON"; $resn_exch{"HN33 CON"}="3HN3 CON"; $resn_exch{"HN41 CON"}="1HN4 CON";
$resn_exch{"HN42 CON"}="2HN4 CON"; $resn_exch{"HN43 CON"}="3HN4 CON"; $resn_exch{" H21 COO"}="1H2  COO";
$resn_exch{" H22 COO"}="2H2  COO"; $resn_exch{" H31 COO"}="1H3  COO"; $resn_exch{" H32 COO"}="2H3  COO";
$resn_exch{" H51 COO"}="1H5  COO"; $resn_exch{" H52 COO"}="2H5  COO"; $resn_exch{" H61 COO"}="1H6  COO";
$resn_exch{" H62 COO"}="2H6  COO"; $resn_exch{"H101 COO"}="1H10 COO"; $resn_exch{"H102 COO"}="2H10 COO";
$resn_exch{"H103 COO"}="3H10 COO"; $resn_exch{"H151 COO"}="1H15 COO"; $resn_exch{"H152 COO"}="2H15 COO";
$resn_exch{"H153 COO"}="3H15 COO"; $resn_exch{"H141 COO"}="1H14 COO"; $resn_exch{"H142 COO"}="2H14 COO";
$resn_exch{"H143 COO"}="3H14 COO"; $resn_exch{"H121 COO"}="1H12 COO"; $resn_exch{"H122 COO"}="2H12 COO";
$resn_exch{"H5X1 COO"}="1H5X COO"; $resn_exch{"H5X2 COO"}="2H5X COO"; $resn_exch{"H6A1 COO"}="1H6A COO";
$resn_exch{"H6A2 COO"}="2H6A COO"; $resn_exch{"H111 COP"}="1H11 COP"; $resn_exch{"H112 COP"}="2H11 COP";
$resn_exch{"H271 COP"}="1H27 COP"; $resn_exch{"H272 COP"}="2H27 COP"; $resn_exch{"H281 COP"}="1H28 COP";
$resn_exch{"H282 COP"}="2H28 COP"; $resn_exch{"H291 COP"}="1H29 COP"; $resn_exch{"H292 COP"}="2H29 COP";
$resn_exch{"HN31 COP"}="1HN3 COP"; $resn_exch{"HN32 COP"}="2HN3 COP"; $resn_exch{"HN11 COP"}="1HN1 COP";
$resn_exch{"HN12 COP"}="2HN1 COP"; $resn_exch{" H5' COQ"}=" H5* COQ"; $resn_exch{" H7' COQ"}=" H7* COQ";
$resn_exch{"H9'1 COQ"}="1H9* COQ"; $resn_exch{"H9'2 COQ"}="2H9* COQ"; $resn_exch{"H311 COQ"}="1H31 COQ";
$resn_exch{"H312 COQ"}="2H31 COQ"; $resn_exch{"H313 COQ"}="3H31 COQ"; $resn_exch{"H511 COQ"}="1H51 COQ";
$resn_exch{"H512 COQ"}="2H51 COQ"; $resn_exch{"H513 COQ"}="3H51 COQ"; $resn_exch{"HCN1 COQ"}="1HCN COQ";
$resn_exch{"HCN2 COQ"}="2HCN COQ"; $resn_exch{"HCN3 COQ"}="3HCN COQ"; $resn_exch{"HN21 COQ"}="1HN2 COQ";
$resn_exch{"HN22 COQ"}="2HN2 COQ"; $resn_exch{"HN41 COQ"}="1HN4 COQ"; $resn_exch{"HN42 COQ"}="2HN4 COQ";
$resn_exch{" H11 COR"}="1H1  COR"; $resn_exch{" H12 COR"}="2H1  COR"; $resn_exch{"HN21 COR"}="1HN2 COR";
$resn_exch{"HN22 COR"}="2HN2 COR"; $resn_exch{"HN41 COR"}="1HN4 COR"; $resn_exch{"HN42 COR"}="2HN4 COR";
$resn_exch{" H51 COR"}="1H5  COR"; $resn_exch{" H52 COR"}="2H5  COR"; $resn_exch{" H2A COS"}=" AH2 COS";
$resn_exch{"H61A COS"}="AH61 COS"; $resn_exch{"H62A COS"}="AH62 COS"; $resn_exch{" H8A COS"}=" AH8 COS";
$resn_exch{" H1B COS"}="AH1* COS"; $resn_exch{" H2B COS"}="AH2* COS"; $resn_exch{"HO2A COS"}="AHO2 COS";
$resn_exch{" H3B COS"}="AH3* COS"; $resn_exch{"HOA8 COS"}="8HOA COS"; $resn_exch{"HOA9 COS"}="9HOA COS";
$resn_exch{" H4B COS"}="AH4* COS"; $resn_exch{"H51A COS"}="AH51 COS"; $resn_exch{"H52A COS"}="AH52 COS";
$resn_exch{"HOA2 COS"}="2HOA COS"; $resn_exch{"HOA5 COS"}="5HOA COS"; $resn_exch{"H121 COS"}="1H12 COS";
$resn_exch{"H122 COS"}="2H12 COS"; $resn_exch{"H131 COS"}="1H13 COS"; $resn_exch{"H132 COS"}="2H13 COS";
$resn_exch{"H133 COS"}="3H13 COS"; $resn_exch{"H141 COS"}="1H14 COS"; $resn_exch{"H142 COS"}="2H14 COS";
$resn_exch{"H143 COS"}="3H14 COS"; $resn_exch{" H71 COS"}="1H7  COS"; $resn_exch{" H72 COS"}="2H7  COS";
$resn_exch{" H61 COS"}="1H6  COS"; $resn_exch{" H62 COS"}="2H6  COS"; $resn_exch{" H31 COS"}="1H3  COS";
$resn_exch{" H32 COS"}="2H3  COS"; $resn_exch{" H21 COS"}="1H2  COS"; $resn_exch{" H22 COS"}="2H2  COS";
$resn_exch{"HS'P COS"}="PHS' COS"; $resn_exch{"H1T1 COT"}="1H1T COT"; $resn_exch{"H1T2 COT"}="2H1T COT";
$resn_exch{"H6T1 COT"}="1H6T COT"; $resn_exch{"H6T2 COT"}="2H6T COT"; $resn_exch{"H2T1 COT"}="1H2T COT";
$resn_exch{"H2T2 COT"}="2H2T COT"; $resn_exch{"HC51 COT"}="1HC5 COT"; $resn_exch{"HC52 COT"}="2HC5 COT";
$resn_exch{"HN61 COT"}="1HN6 COT"; $resn_exch{"HN62 COT"}="2HN6 COT"; $resn_exch{"HCA2 COT"}="2HCA COT";
$resn_exch{"H6P1 COT"}="1H6P COT"; $resn_exch{"H6P2 COT"}="2H6P COT"; $resn_exch{"H7P1 COT"}="1H7P COT";
$resn_exch{"H7P2 COT"}="2H7P COT"; $resn_exch{"H161 COT"}="1H16 COT"; $resn_exch{"H162 COT"}="2H16 COT";
$resn_exch{"H141 COT"}="1H14 COT"; $resn_exch{"H142 COT"}="2H14 COT"; $resn_exch{"H143 COT"}="3H14 COT";
$resn_exch{"H151 COT"}="1H15 COT"; $resn_exch{"H152 COT"}="2H15 COT"; $resn_exch{"H153 COT"}="3H15 COT";
$resn_exch{"H2P1 COT"}="1H2P COT"; $resn_exch{"H2P2 COT"}="2H2P COT"; $resn_exch{"H1P1 COT"}="1H1P COT";
$resn_exch{"H1P2 COT"}="2H1P COT"; $resn_exch{"HN11 COX"}="1HN1 COX"; $resn_exch{"HN12 COX"}="2HN1 COX";
$resn_exch{" H91 COX"}="1H9  COX"; $resn_exch{" H92 COX"}="2H9  COX"; $resn_exch{" H93 COX"}="3H9  COX";
$resn_exch{"H201 COY"}="1H20 COY"; $resn_exch{"H202 COY"}="2H20 COY"; $resn_exch{"H203 COY"}="3H20 COY";
$resn_exch{"H251 COY"}="1H25 COY"; $resn_exch{"H252 COY"}="2H25 COY"; $resn_exch{"H253 COY"}="3H25 COY";
$resn_exch{"H261 COY"}="1H26 COY"; $resn_exch{"H262 COY"}="2H26 COY"; $resn_exch{"H291 COY"}="1H29 COY";
$resn_exch{"H292 COY"}="2H29 COY"; $resn_exch{"H301 COY"}="1H30 COY"; $resn_exch{"H302 COY"}="2H30 COY";
$resn_exch{"H311 COY"}="1H31 COY"; $resn_exch{"H312 COY"}="2H31 COY"; $resn_exch{"H331 COY"}="1H33 COY";
$resn_exch{"H332 COY"}="2H33 COY"; $resn_exch{"H351 COY"}="1H35 COY"; $resn_exch{"H352 COY"}="2H35 COY";
$resn_exch{"H353 COY"}="3H35 COY"; $resn_exch{" H66 COY"}="6H6  COY"; $resn_exch{"H361 COY"}="1H36 COY";
$resn_exch{"H362 COY"}="2H36 COY"; $resn_exch{"H363 COY"}="3H36 COY"; $resn_exch{"H371 COY"}="1H37 COY";
$resn_exch{"H372 COY"}="2H37 COY"; $resn_exch{"H401 COY"}="1H40 COY"; $resn_exch{"H402 COY"}="2H40 COY";
$resn_exch{"H411 COY"}="1H41 COY"; $resn_exch{"H412 COY"}="2H41 COY"; $resn_exch{"H421 COY"}="1H42 COY";
$resn_exch{"H422 COY"}="2H42 COY"; $resn_exch{"H451 COY"}="1H45 COY"; $resn_exch{"H452 COY"}="2H45 COY";
$resn_exch{"H111 COY"}="11H1 COY"; $resn_exch{"H461 COY"}="1H46 COY"; $resn_exch{"H462 COY"}="2H46 COY";
$resn_exch{"H463 COY"}="3H46 COY"; $resn_exch{"H471 COY"}="1H47 COY"; $resn_exch{"H472 COY"}="2H47 COY";
$resn_exch{"H473 COY"}="3H47 COY"; $resn_exch{"H481 COY"}="1H48 COY"; $resn_exch{"H482 COY"}="2H48 COY";
$resn_exch{"H491 COY"}="1H49 COY"; $resn_exch{"H492 COY"}="2H49 COY"; $resn_exch{"H521 COY"}="1H52 COY";
$resn_exch{"H522 COY"}="2H52 COY"; $resn_exch{"H531 COY"}="1H53 COY"; $resn_exch{"H532 COY"}="2H53 COY";
$resn_exch{"H533 COY"}="3H53 COY"; $resn_exch{"H116 COY"}="16H1 COY"; $resn_exch{"H541 COY"}="1H54 COY";
$resn_exch{"H542 COY"}="2H54 COY"; $resn_exch{"H543 COY"}="3H54 COY"; $resn_exch{"H551 COY"}="1H55 COY";
$resn_exch{"H552 COY"}="2H55 COY"; $resn_exch{"H561 COY"}="1H56 COY"; $resn_exch{"H562 COY"}="2H56 COY";
$resn_exch{"H601 COY"}="1H60 COY"; $resn_exch{"H602 COY"}="2H60 COY"; $resn_exch{"H621 COY"}="1H62 COY";
$resn_exch{"H622 COY"}="2H62 COY"; $resn_exch{"H119 COY"}="19H1 COY"; $resn_exch{"H1P1 COY"}="1H1P COY";
$resn_exch{"H1P2 COY"}="2H1P COY"; $resn_exch{"H3P1 COY"}="1H3P COY"; $resn_exch{"H3P2 COY"}="2H3P COY";
$resn_exch{"H3P3 COY"}="3H3P COY"; $resn_exch{"HOP5 COY"}="5HOP COY"; $resn_exch{"HOR7 COY"}="7HOR COY";
$resn_exch{"H5R1 COY"}="1H5R COY"; $resn_exch{"H5R2 COY"}="2H5R COY"; $resn_exch{"HOR8 COY"}="8HOR COY";
$resn_exch{"HM51 COY"}="1HM5 COY"; $resn_exch{"HM52 COY"}="2HM5 COY"; $resn_exch{"HM53 COY"}="3HM5 COY";
$resn_exch{"HM61 COY"}="1HM6 COY"; $resn_exch{"HM62 COY"}="2HM6 COY"; $resn_exch{"HM63 COY"}="3HM6 COY";
$resn_exch{"H711 COY"}="1H71 COY"; $resn_exch{"H712 COY"}="2H71 COY"; $resn_exch{"H721 COY"}="1H72 COY";
$resn_exch{"H722 COY"}="2H72 COY"; $resn_exch{"H731 COY"}="1H73 COY"; $resn_exch{"H732 COY"}="2H73 COY";
$resn_exch{"H741 COY"}="1H74 COY"; $resn_exch{"H742 COY"}="2H74 COY"; $resn_exch{"H751 COY"}="1H75 COY";
$resn_exch{"H752 COY"}="2H75 COY"; $resn_exch{"H6A1 COY"}="1H6A COY"; $resn_exch{"H6A2 COY"}="2H6A COY";
$resn_exch{" H2A COZ"}=" AH2 COZ"; $resn_exch{"H61A COZ"}="AH61 COZ"; $resn_exch{"H62A COZ"}="AH62 COZ";
$resn_exch{" H8A COZ"}=" AH8 COZ"; $resn_exch{" H1B COZ"}="AH1* COZ"; $resn_exch{" H2B COZ"}="AH2* COZ";
$resn_exch{"HO2A COZ"}="AHO2 COZ"; $resn_exch{" H3B COZ"}="AH3* COZ"; $resn_exch{"HOA8 COZ"}="8HOA COZ";
$resn_exch{"HOA9 COZ"}="9HOA COZ"; $resn_exch{" H4B COZ"}="AH4* COZ"; $resn_exch{"H51A COZ"}="AH51 COZ";
$resn_exch{"H52A COZ"}="AH52 COZ"; $resn_exch{"HOA2 COZ"}="2HOA COZ"; $resn_exch{"HOA5 COZ"}="5HOA COZ";
$resn_exch{"H121 COZ"}="1H12 COZ"; $resn_exch{"H122 COZ"}="2H12 COZ"; $resn_exch{"H131 COZ"}="1H13 COZ";
$resn_exch{"H132 COZ"}="2H13 COZ"; $resn_exch{"H133 COZ"}="3H13 COZ"; $resn_exch{"H141 COZ"}="1H14 COZ";
$resn_exch{"H142 COZ"}="2H14 COZ"; $resn_exch{"H143 COZ"}="3H14 COZ"; $resn_exch{" H71 COZ"}="1H7  COZ";
$resn_exch{" H72 COZ"}="2H7  COZ"; $resn_exch{" H61 COZ"}="1H6  COZ"; $resn_exch{" H62 COZ"}="2H6  COZ";
$resn_exch{" H31 COZ"}="1H3  COZ"; $resn_exch{" H32 COZ"}="2H3  COZ"; $resn_exch{" H21 COZ"}="1H2  COZ";
$resn_exch{" H22 COZ"}="2H2  COZ"; $resn_exch{" HN1  CP"}="1HN   CP"; $resn_exch{" HN2  CP"}="2HN   CP";
$resn_exch{"HOP2  CP"}="2HOP  CP"; $resn_exch{"HOP3  CP"}="3HOP  CP"; $resn_exch{"H171 CP0"}="1H17 CP0";
$resn_exch{"H172 CP0"}="2H17 CP0"; $resn_exch{" H51 CP0"}="1H5  CP0"; $resn_exch{" H52 CP0"}="2H5  CP0";
$resn_exch{"H281 CP0"}="1H28 CP0"; $resn_exch{"H282 CP0"}="2H28 CP0"; $resn_exch{"H311 CP0"}="1H31 CP0";
$resn_exch{"H312 CP0"}="2H31 CP0"; $resn_exch{"H321 CP0"}="1H32 CP0"; $resn_exch{"H322 CP0"}="2H32 CP0";
$resn_exch{"H323 CP0"}="3H32 CP0"; $resn_exch{"H331 CP0"}="1H33 CP0"; $resn_exch{"H332 CP0"}="2H33 CP0";
$resn_exch{"H333 CP0"}="3H33 CP0"; $resn_exch{"H251 CP0"}="1H25 CP0"; $resn_exch{"H252 CP0"}="2H25 CP0";
$resn_exch{"H221 CP0"}="1H22 CP0"; $resn_exch{"H222 CP0"}="2H22 CP0"; $resn_exch{"H261 CP0"}="1H26 CP0";
$resn_exch{"H262 CP0"}="2H26 CP0"; $resn_exch{"H291 CP0"}="1H29 CP0"; $resn_exch{"H292 CP0"}="2H29 CP0";
$resn_exch{"H341 CP0"}="1H34 CP0"; $resn_exch{"H342 CP0"}="2H34 CP0"; $resn_exch{"H351 CP0"}="1H35 CP0";
$resn_exch{"H352 CP0"}="2H35 CP0"; $resn_exch{"H271 CP0"}="1H27 CP0"; $resn_exch{"H272 CP0"}="2H27 CP0";
$resn_exch{"H301 CP0"}="1H30 CP0"; $resn_exch{"H302 CP0"}="2H30 CP0"; $resn_exch{"H11A CP0"}="1H1  CP0";
$resn_exch{"H12A CP0"}="2H1  CP0"; $resn_exch{"H8'1 CP1"}="1H8' CP1"; $resn_exch{"H8'2 CP1"}="2H8' CP1";
$resn_exch{"H5'1 CP1"}="1H5' CP1"; $resn_exch{"H5'2 CP1"}="2H5' CP1"; $resn_exch{"H3'1 CP1"}="1H3' CP1";
$resn_exch{"H3'2 CP1"}="2H3' CP1"; $resn_exch{"H2'1 CP1"}="1H2' CP1"; $resn_exch{"H2'2 CP1"}="2H2' CP1";
$resn_exch{"H1'1 CP1"}="1H1' CP1"; $resn_exch{"H1'2 CP1"}="2H1' CP1"; $resn_exch{"H1'3 CP1"}="3H1' CP1";
$resn_exch{"HN41 CP1"}="1HN4 CP1"; $resn_exch{"HN42 CP1"}="2HN4 CP1"; $resn_exch{"HC21 CP2"}="1HC2 CP2";
$resn_exch{"HC22 CP2"}="2HC2 CP2"; $resn_exch{"HC31 CP2"}="1HC3 CP2"; $resn_exch{"HC41 CP2"}="1HC4 CP2";
$resn_exch{"HC42 CP2"}="2HC4 CP2"; $resn_exch{"HC43 CP2"}="3HC4 CP2"; $resn_exch{"H621 CP3"}="1H62 CP3";
$resn_exch{"H622 CP3"}="2H62 CP3"; $resn_exch{"H611 CP3"}="1H61 CP3"; $resn_exch{"H612 CP3"}="2H61 CP3";
$resn_exch{"H201 CP3"}="1H20 CP3"; $resn_exch{"H202 CP3"}="2H20 CP3"; $resn_exch{"H861 CP3"}="1H86 CP3";
$resn_exch{"H862 CP3"}="2H86 CP3"; $resn_exch{"H863 CP3"}="3H86 CP3"; $resn_exch{"H151 CP3"}="1H15 CP3";
$resn_exch{"H152 CP3"}="2H15 CP3"; $resn_exch{"H511 CP3"}="1H51 CP3"; $resn_exch{"H512 CP3"}="2H51 CP3";
$resn_exch{"H521 CP3"}="1H52 CP3"; $resn_exch{"H522 CP3"}="2H52 CP3"; $resn_exch{"H811 CP3"}="1H81 CP3";
$resn_exch{"H812 CP3"}="2H81 CP3"; $resn_exch{"H813 CP3"}="3H81 CP3"; $resn_exch{"H101 CP3"}="1H10 CP3";
$resn_exch{"H102 CP3"}="2H10 CP3"; $resn_exch{"H7A1 CP3"}="1H7A CP3"; $resn_exch{"H7A2 CP3"}="2H7A CP3";
$resn_exch{"H7A3 CP3"}="3H7A CP3"; $resn_exch{"H411 CP3"}="1H41 CP3"; $resn_exch{"H412 CP3"}="2H41 CP3";
$resn_exch{"H421 CP3"}="1H42 CP3"; $resn_exch{"H422 CP3"}="2H42 CP3"; $resn_exch{" H51 CP3"}="1H5  CP3";
$resn_exch{" H52 CP3"}="2H5  CP3"; $resn_exch{"H2A1 CP3"}="1H2A CP3"; $resn_exch{"H2A2 CP3"}="2H2A CP3";
$resn_exch{"H2A3 CP3"}="3H2A CP3"; $resn_exch{"H311 CP3"}="1H31 CP3"; $resn_exch{"H312 CP3"}="2H31 CP3";
$resn_exch{"H321 CP3"}="1H32 CP3"; $resn_exch{"H322 CP3"}="2H32 CP3"; $resn_exch{"H131 CP4"}="1H13 CP4";
$resn_exch{"H132 CP4"}="2H13 CP4"; $resn_exch{"H141 CP4"}="1H14 CP4"; $resn_exch{"H142 CP4"}="2H14 CP4";
$resn_exch{"H161 CP4"}="1H16 CP4"; $resn_exch{"H162 CP4"}="2H16 CP4"; $resn_exch{"H171 CP4"}="1H17 CP4";
$resn_exch{"H172 CP4"}="2H17 CP4"; $resn_exch{"H191 CP4"}="1H19 CP4"; $resn_exch{"H192 CP4"}="2H19 CP4";
$resn_exch{"H201 CP4"}="1H20 CP4"; $resn_exch{"H202 CP4"}="2H20 CP4"; $resn_exch{"H131 CP6"}="1H13 CP6";
$resn_exch{"H132 CP6"}="2H13 CP6"; $resn_exch{"H141 CP6"}="1H14 CP6"; $resn_exch{"H142 CP6"}="2H14 CP6";
$resn_exch{"H151 CP6"}="1H15 CP6"; $resn_exch{"H152 CP6"}="2H15 CP6"; $resn_exch{"H161 CP6"}="1H16 CP6";
$resn_exch{"H162 CP6"}="2H16 CP6"; $resn_exch{"H163 CP6"}="3H16 CP6"; $resn_exch{"H131 CP7"}="1H13 CP7";
$resn_exch{"H132 CP7"}="2H13 CP7"; $resn_exch{"H141 CP7"}="1H14 CP7"; $resn_exch{"H142 CP7"}="2H14 CP7";
$resn_exch{"H151 CP7"}="1H15 CP7"; $resn_exch{"H152 CP7"}="2H15 CP7"; $resn_exch{"H161 CP7"}="1H16 CP7";
$resn_exch{"H162 CP7"}="2H16 CP7"; $resn_exch{"H163 CP7"}="3H16 CP7"; $resn_exch{"H6C1 CP8"}="1H6C CP8";
$resn_exch{"H6C2 CP8"}="2H6C CP8"; $resn_exch{"H8C1 CP8"}="1H8C CP8"; $resn_exch{"H8C2 CP8"}="2H8C CP8";
$resn_exch{"H8C3 CP8"}="3H8C CP8"; $resn_exch{"H141 CP8"}="1H14 CP8"; $resn_exch{"H142 CP8"}="2H14 CP8";
$resn_exch{"H171 CP8"}="1H17 CP8"; $resn_exch{"H172 CP8"}="2H17 CP8"; $resn_exch{"H181 CP8"}="1H18 CP8";
$resn_exch{"H182 CP8"}="2H18 CP8"; $resn_exch{"H101 CP9"}="1H10 CP9"; $resn_exch{"H102 CP9"}="2H10 CP9";
$resn_exch{"H261 CP9"}="1H26 CP9"; $resn_exch{"H262 CP9"}="2H26 CP9"; $resn_exch{"H263 CP9"}="3H26 CP9";
$resn_exch{"HOP2 CPA"}="2HOP CPA"; $resn_exch{"HO5C CPA"}="CHO5 CPA"; $resn_exch{"H51C CPA"}="CH51 CPA";
$resn_exch{"H52C CPA"}="CH52 CPA"; $resn_exch{" H4D CPA"}="CH4' CPA"; $resn_exch{" H3D CPA"}="CH3' CPA";
$resn_exch{"H21C CPA"}="CH21 CPA"; $resn_exch{"H22C CPA"}="CH22 CPA"; $resn_exch{" H1D CPA"}="CH1' CPA";
$resn_exch{" H5C CPA"}=" CH5 CPA"; $resn_exch{" H6C CPA"}=" CH6 CPA"; $resn_exch{"H41C CPA"}="CH41 CPA";
$resn_exch{"H42C CPA"}="CH42 CPA"; $resn_exch{"H51A CPA"}="AH51 CPA"; $resn_exch{"H52A CPA"}="AH52 CPA";
$resn_exch{" H4B CPA"}="AH4' CPA"; $resn_exch{" H3B CPA"}="AH3' CPA"; $resn_exch{"HO3A CPA"}="AHO3 CPA";
$resn_exch{"H21A CPA"}="AH21 CPA"; $resn_exch{"H22A CPA"}="AH22 CPA"; $resn_exch{" H1B CPA"}="AH1' CPA";
$resn_exch{" H8A CPA"}=" AH8 CPA"; $resn_exch{"H61A CPA"}="AH61 CPA"; $resn_exch{"H62A CPA"}="AH62 CPA";
$resn_exch{" H2A CPA"}=" AH2 CPA"; $resn_exch{" H11 CPB"}="1H1  CPB"; $resn_exch{" H12 CPB"}="2H1  CPB";
$resn_exch{" H13 CPB"}="3H1  CPB"; $resn_exch{"H161 CPB"}="1H16 CPB"; $resn_exch{"H162 CPB"}="2H16 CPB";
$resn_exch{"H151 CPB"}="1H15 CPB"; $resn_exch{"H152 CPB"}="2H15 CPB"; $resn_exch{"H141 CPB"}="1H14 CPB";
$resn_exch{"H131 CPB"}="1H13 CPB"; $resn_exch{"H121 CPB"}="1H12 CPB"; $resn_exch{"H122 CPB"}="2H12 CPB";
$resn_exch{" H31 CPB"}="1H3  CPB"; $resn_exch{" H61 CPB"}="1H6  CPB"; $resn_exch{"H231 CPB"}="1H23 CPB";
$resn_exch{"H241 CPB"}="1H24 CPB"; $resn_exch{"H251 CPB"}="1H25 CPB"; $resn_exch{"H261 CPB"}="1H26 CPB";
$resn_exch{" H   CPC"}=" HN  CPC"; $resn_exch{"HCN1 CPC"}="1HCN CPC"; $resn_exch{"HCN2 CPC"}="2HCN CPC";
$resn_exch{"HCN3 CPC"}="3HCN CPC"; $resn_exch{"HG13 CPC"}="1HG1 CPC"; $resn_exch{"HG12 CPC"}="2HG1 CPC";
$resn_exch{"HG21 CPC"}="1HG2 CPC"; $resn_exch{"HG22 CPC"}="2HG2 CPC"; $resn_exch{"HG23 CPC"}="3HG2 CPC";
$resn_exch{" H41 CPD"}="1H4  CPD"; $resn_exch{" H42 CPD"}="2H4  CPD"; $resn_exch{"H121 CPD"}="1H12 CPD";
$resn_exch{"H122 CPD"}="2H12 CPD"; $resn_exch{"H221 CPD"}="1H22 CPD"; $resn_exch{"H222 CPD"}="2H22 CPD";
$resn_exch{"H231 CPD"}="1H23 CPD"; $resn_exch{"H232 CPD"}="2H23 CPD"; $resn_exch{"H233 CPD"}="3H23 CPD";
$resn_exch{"H241 CPD"}="1H24 CPD"; $resn_exch{"H242 CPD"}="2H24 CPD"; $resn_exch{"H251 CPD"}="1H25 CPD";
$resn_exch{"H252 CPD"}="2H25 CPD"; $resn_exch{"H261 CPD"}="1H26 CPD"; $resn_exch{"H262 CPD"}="2H26 CPD";
$resn_exch{"H271 CPD"}="1H27 CPD"; $resn_exch{"H272 CPD"}="2H27 CPD"; $resn_exch{"H273 CPD"}="3H27 CPD";
$resn_exch{"H281 CPD"}="1H28 CPD"; $resn_exch{"H282 CPD"}="2H28 CPD"; $resn_exch{"H283 CPD"}="3H28 CPD";
$resn_exch{"H171 CPF"}="1H17 CPF"; $resn_exch{"H172 CPF"}="2H17 CPF"; $resn_exch{"H161 CPF"}="1H16 CPF";
$resn_exch{"H162 CPF"}="2H16 CPF"; $resn_exch{"H151 CPF"}="1H15 CPF"; $resn_exch{"H152 CPF"}="2H15 CPF";
$resn_exch{"H141 CPF"}="1H14 CPF"; $resn_exch{"H142 CPF"}="2H14 CPF"; $resn_exch{"H121 CPF"}="1H12 CPF";
$resn_exch{"H122 CPF"}="2H12 CPF"; $resn_exch{"H131 CPF"}="1H13 CPF"; $resn_exch{"H132 CPF"}="2H13 CPF";
$resn_exch{"HOP2 CPG"}="2HOP CPG"; $resn_exch{"HOP3 CPG"}="3HOP CPG"; $resn_exch{"H5'1 CPG"}="1H5* CPG";
$resn_exch{"H5'2 CPG"}="2H5* CPG"; $resn_exch{" H4' CPG"}=" H4* CPG"; $resn_exch{" H3' CPG"}=" H3* CPG";
$resn_exch{"HO3' CPG"}="*HO3 CPG"; $resn_exch{" H2' CPG"}=" H2* CPG"; $resn_exch{"HO2' CPG"}="*HO2 CPG";
$resn_exch{" H1' CPG"}=" H1* CPG"; $resn_exch{"HN21 CPG"}="1HN2 CPG"; $resn_exch{"HN22 CPG"}="2HN2 CPG";
$resn_exch{" H41 CPH"}="1H4  CPH"; $resn_exch{" H42 CPH"}="2H4  CPH"; $resn_exch{"H101 CPH"}="1H10 CPH";
$resn_exch{"H102 CPH"}="2H10 CPH"; $resn_exch{" H71 CPH"}="1H7  CPH"; $resn_exch{" H72 CPH"}="2H7  CPH";
$resn_exch{" H73 CPH"}="3H7  CPH"; $resn_exch{" H1' CPH"}=" H1* CPH"; $resn_exch{" H3' CPH"}=" H3* CPH";
$resn_exch{" H4' CPH"}=" H4* CPH"; $resn_exch{" H51 CPH"}="1H5  CPH"; $resn_exch{" H52 CPH"}="2H5  CPH";
$resn_exch{" H53 CPH"}="3H5  CPH"; $resn_exch{" H21 CPI"}="1H2  CPI"; $resn_exch{" H22 CPI"}="2H2  CPI";
$resn_exch{" H31 CPI"}="1H3  CPI"; $resn_exch{" H32 CPI"}="2H3  CPI"; $resn_exch{" H41 CPI"}="1H4  CPI";
$resn_exch{" H42 CPI"}="2H4  CPI"; $resn_exch{" H51 CPI"}="1H5  CPI"; $resn_exch{" H52 CPI"}="2H5  CPI";
$resn_exch{"H281 CPJ"}="1H28 CPJ"; $resn_exch{"H282 CPJ"}="2H28 CPJ"; $resn_exch{"H321 CPJ"}="1H32 CPJ";
$resn_exch{"H322 CPJ"}="2H32 CPJ"; $resn_exch{"H323 CPJ"}="3H32 CPJ"; $resn_exch{"H241 CPJ"}="1H24 CPJ";
$resn_exch{"H242 CPJ"}="2H24 CPJ"; $resn_exch{"H161 CPJ"}="1H16 CPJ"; $resn_exch{"H162 CPJ"}="2H16 CPJ";
$resn_exch{"H151 CPJ"}="1H15 CPJ"; $resn_exch{"H152 CPJ"}="2H15 CPJ"; $resn_exch{" H21 CPJ"}="1H2  CPJ";
$resn_exch{" H22 CPJ"}="2H2  CPJ"; $resn_exch{" H41 CPJ"}="1H4  CPJ"; $resn_exch{" H42 CPJ"}="2H4  CPJ";
$resn_exch{"H101 CPJ"}="1H10 CPJ"; $resn_exch{"H102 CPJ"}="2H10 CPJ"; $resn_exch{"H103 CPJ"}="3H10 CPJ";
$resn_exch{"H281 CPK"}="1H28 CPK"; $resn_exch{"H282 CPK"}="2H28 CPK"; $resn_exch{"H321 CPK"}="1H32 CPK";
$resn_exch{"H322 CPK"}="2H32 CPK"; $resn_exch{"H323 CPK"}="3H32 CPK"; $resn_exch{"H241 CPK"}="1H24 CPK";
$resn_exch{"H242 CPK"}="2H24 CPK"; $resn_exch{"H161 CPK"}="1H16 CPK"; $resn_exch{"H162 CPK"}="2H16 CPK";
$resn_exch{"H151 CPK"}="1H15 CPK"; $resn_exch{"H152 CPK"}="2H15 CPK"; $resn_exch{"H181 CPK"}="1H18 CPK";
$resn_exch{"H182 CPK"}="2H18 CPK"; $resn_exch{"H191 CPK"}="1H19 CPK"; $resn_exch{"H192 CPK"}="2H19 CPK";
$resn_exch{" H21 CPK"}="1H2  CPK"; $resn_exch{" H22 CPK"}="2H2  CPK"; $resn_exch{" H41 CPK"}="1H4  CPK";
$resn_exch{" H42 CPK"}="2H4  CPK"; $resn_exch{"H101 CPK"}="1H10 CPK"; $resn_exch{"H102 CPK"}="2H10 CPK";
$resn_exch{"H103 CPK"}="3H10 CPK"; $resn_exch{"HC11 CPL"}="1HC1 CPL"; $resn_exch{"HC12 CPL"}="2HC1 CPL";
$resn_exch{"HC31 CPL"}="1HC3 CPL"; $resn_exch{"HC32 CPL"}="2HC3 CPL"; $resn_exch{"HC41 CPL"}="1HC4 CPL";
$resn_exch{"HC42 CPL"}="2HC4 CPL"; $resn_exch{"HC51 CPL"}="1HC5 CPL"; $resn_exch{"HC52 CPL"}="2HC5 CPL";
$resn_exch{"HC61 CPL"}="1HC6 CPL"; $resn_exch{"HC62 CPL"}="2HC6 CPL"; $resn_exch{"HC63 CPL"}="3HC6 CPL";
$resn_exch{"HC71 CPL"}="1HC7 CPL"; $resn_exch{"HC72 CPL"}="2HC7 CPL"; $resn_exch{"HC73 CPL"}="3HC7 CPL";
$resn_exch{"HC81 CPL"}="1HC8 CPL"; $resn_exch{"HC82 CPL"}="2HC8 CPL"; $resn_exch{"HC83 CPL"}="3HC8 CPL";
$resn_exch{"H121 CPL"}="1H12 CPL"; $resn_exch{"H122 CPL"}="2H12 CPL"; $resn_exch{"H131 CPL"}="1H13 CPL";
$resn_exch{"H132 CPL"}="2H13 CPL"; $resn_exch{"H141 CPL"}="1H14 CPL"; $resn_exch{"H142 CPL"}="2H14 CPL";
$resn_exch{"H151 CPL"}="1H15 CPL"; $resn_exch{"H152 CPL"}="2H15 CPL"; $resn_exch{"H161 CPL"}="1H16 CPL";
$resn_exch{"H162 CPL"}="2H16 CPL"; $resn_exch{"H171 CPL"}="1H17 CPL"; $resn_exch{"H172 CPL"}="2H17 CPL";
$resn_exch{"H181 CPL"}="1H18 CPL"; $resn_exch{"H182 CPL"}="2H18 CPL"; $resn_exch{"H191 CPL"}="1H19 CPL";
$resn_exch{"H192 CPL"}="2H19 CPL"; $resn_exch{"H201 CPL"}="1H20 CPL"; $resn_exch{"H202 CPL"}="2H20 CPL";
$resn_exch{"H211 CPL"}="1H21 CPL"; $resn_exch{"H212 CPL"}="2H21 CPL"; $resn_exch{"H221 CPL"}="1H22 CPL";
$resn_exch{"H222 CPL"}="2H22 CPL"; $resn_exch{"H231 CPL"}="1H23 CPL"; $resn_exch{"H232 CPL"}="2H23 CPL";
$resn_exch{"H241 CPL"}="1H24 CPL"; $resn_exch{"H242 CPL"}="2H24 CPL"; $resn_exch{"H251 CPL"}="1H25 CPL";
$resn_exch{"H252 CPL"}="2H25 CPL"; $resn_exch{"H261 CPL"}="1H26 CPL"; $resn_exch{"H262 CPL"}="2H26 CPL";
$resn_exch{"H263 CPL"}="3H26 CPL"; $resn_exch{"H321 CPL"}="1H32 CPL"; $resn_exch{"H322 CPL"}="2H32 CPL";
$resn_exch{"H331 CPL"}="1H33 CPL"; $resn_exch{"H332 CPL"}="2H33 CPL"; $resn_exch{"H341 CPL"}="1H34 CPL";
$resn_exch{"H342 CPL"}="2H34 CPL"; $resn_exch{"H351 CPL"}="1H35 CPL"; $resn_exch{"H352 CPL"}="2H35 CPL";
$resn_exch{"H361 CPL"}="1H36 CPL"; $resn_exch{"H362 CPL"}="2H36 CPL"; $resn_exch{"H371 CPL"}="1H37 CPL";
$resn_exch{"H372 CPL"}="2H37 CPL"; $resn_exch{"H381 CPL"}="1H38 CPL"; $resn_exch{"H382 CPL"}="2H38 CPL";
$resn_exch{"H411 CPL"}="1H41 CPL"; $resn_exch{"H412 CPL"}="2H41 CPL"; $resn_exch{"H441 CPL"}="1H44 CPL";
$resn_exch{"H442 CPL"}="2H44 CPL"; $resn_exch{"H451 CPL"}="1H45 CPL"; $resn_exch{"H452 CPL"}="2H45 CPL";
$resn_exch{"H461 CPL"}="1H46 CPL"; $resn_exch{"H462 CPL"}="2H46 CPL"; $resn_exch{"H471 CPL"}="1H47 CPL";
$resn_exch{"H472 CPL"}="2H47 CPL"; $resn_exch{"H481 CPL"}="1H48 CPL"; $resn_exch{"H482 CPL"}="2H48 CPL";
$resn_exch{"H483 CPL"}="3H48 CPL"; $resn_exch{"HA11 CPM"}="1HA1 CPM"; $resn_exch{"HA12 CPM"}="2HA1 CPM";
$resn_exch{" HB1 CPM"}="1HB  CPM"; $resn_exch{" HB2 CPM"}="2HB  CPM"; $resn_exch{" HM1 CPM"}="1HM  CPM";
$resn_exch{" HM2 CPM"}="2HM  CPM"; $resn_exch{" HM3 CPM"}="3HM  CPM"; $resn_exch{"H8'1 CPN"}="1H8' CPN";
$resn_exch{"H8'2 CPN"}="2H8' CPN"; $resn_exch{"H5'1 CPN"}="1H5' CPN"; $resn_exch{"H5'2 CPN"}="2H5' CPN";
$resn_exch{"H3'1 CPN"}="1H3' CPN"; $resn_exch{"H3'2 CPN"}="2H3' CPN"; $resn_exch{"H2'1 CPN"}="1H2' CPN";
$resn_exch{"H2'2 CPN"}="2H2' CPN"; $resn_exch{"H1'1 CPN"}="1H1' CPN"; $resn_exch{"H1'2 CPN"}="2H1' CPN";
$resn_exch{"H1'3 CPN"}="3H1' CPN"; $resn_exch{"HN41 CPN"}="1HN4 CPN"; $resn_exch{"HN42 CPN"}="2HN4 CPN";
$resn_exch{" H21 CPO"}="1H2  CPO"; $resn_exch{" H31 CPO"}="1H3  CPO"; $resn_exch{" H32 CPO"}="2H3  CPO";
$resn_exch{" H41 CPO"}="1H4  CPO"; $resn_exch{" H42 CPO"}="2H4  CPO"; $resn_exch{" H51 CPO"}="1H5  CPO";
$resn_exch{" H52 CPO"}="2H5  CPO"; $resn_exch{"HN21 CPO"}="1HN2 CPO"; $resn_exch{" H81 CPO"}="1H8  CPO";
$resn_exch{" H91 CPO"}="1H9  CPO"; $resn_exch{" H92 CPO"}="2H9  CPO"; $resn_exch{"H101 CPO"}="1H10 CPO";
$resn_exch{"H102 CPO"}="2H10 CPO"; $resn_exch{"H111 CPO"}="1H11 CPO"; $resn_exch{"H112 CPO"}="2H11 CPO";
$resn_exch{"HN31 CPO"}="1HN3 CPO"; $resn_exch{"H121 CPO"}="1H12 CPO"; $resn_exch{"H122 CPO"}="2H12 CPO";
$resn_exch{"H123 CPO"}="3H12 CPO"; $resn_exch{"H141 CPO"}="1H14 CPO"; $resn_exch{"H142 CPO"}="2H14 CPO";
$resn_exch{"H151 CPO"}="1H15 CPO"; $resn_exch{"H152 CPO"}="2H15 CPO"; $resn_exch{"H161 CPO"}="1H16 CPO";
$resn_exch{"H181 CPO"}="1H18 CPO"; $resn_exch{"H201 CPO"}="1H20 CPO"; $resn_exch{"H202 CPO"}="2H20 CPO";
$resn_exch{"H211 CPO"}="1H21 CPO"; $resn_exch{"H212 CPO"}="2H21 CPO"; $resn_exch{"H241 CPO"}="1H24 CPO";
$resn_exch{"H251 CPO"}="1H25 CPO"; $resn_exch{"H252 CPO"}="2H25 CPO"; $resn_exch{"H261 CPO"}="1H26 CPO";
$resn_exch{"H262 CPO"}="2H26 CPO"; $resn_exch{"H271 CPO"}="1H27 CPO"; $resn_exch{"H272 CPO"}="2H27 CPO";
$resn_exch{"H292 CPO"}="2H29 CPO"; $resn_exch{"H311 CPO"}="1H31 CPO"; $resn_exch{"H312 CPO"}="2H31 CPO";
$resn_exch{"H321 CPO"}="1H32 CPO"; $resn_exch{"H322 CPO"}="2H32 CPO"; $resn_exch{"H331 CPO"}="1H33 CPO";
$resn_exch{"H332 CPO"}="2H33 CPO"; $resn_exch{"H333 CPO"}="3H33 CPO"; $resn_exch{"HN61 CPO"}="1HN6 CPO";
$resn_exch{"H351 CPO"}="1H35 CPO"; $resn_exch{"H352 CPO"}="2H35 CPO"; $resn_exch{"H353 CPO"}="3H35 CPO";
$resn_exch{"H191 CPO"}="1H19 CPO"; $resn_exch{"H192 CPO"}="2H19 CPO"; $resn_exch{"H193 CPO"}="3H19 CPO";
$resn_exch{" H31 CPP"}="1H3  CPP"; $resn_exch{" H32 CPP"}="2H3  CPP"; $resn_exch{" H33 CPP"}="3H3  CPP";
$resn_exch{"H2'1 CPP"}="1H2' CPP"; $resn_exch{"H2'2 CPP"}="2H2' CPP"; $resn_exch{"H3'1 CPP"}="1H3' CPP";
$resn_exch{"H3'2 CPP"}="2H3' CPP"; $resn_exch{"H4'1 CPP"}="1H4' CPP"; $resn_exch{"H4'2 CPP"}="2H4' CPP";
$resn_exch{" H11 CPQ"}="1H1  CPQ"; $resn_exch{" H12 CPQ"}="2H1  CPQ"; $resn_exch{" H31 CPQ"}="1H3  CPQ";
$resn_exch{" H32 CPQ"}="2H3  CPQ"; $resn_exch{" H71 CPQ"}="1H7  CPQ"; $resn_exch{" H72 CPQ"}="2H7  CPQ";
$resn_exch{" H81 CPQ"}="1H8  CPQ"; $resn_exch{" H82 CPQ"}="2H8  CPQ"; $resn_exch{"H101 CPQ"}="1H10 CPQ";
$resn_exch{"H102 CPQ"}="2H10 CPQ"; $resn_exch{"H103 CPQ"}="3H10 CPQ"; $resn_exch{"H111 CPQ"}="1H11 CPQ";
$resn_exch{"H112 CPQ"}="2H11 CPQ"; $resn_exch{"H113 CPQ"}="3H11 CPQ"; $resn_exch{"H121 CPQ"}="1H12 CPQ";
$resn_exch{"H122 CPQ"}="2H12 CPQ"; $resn_exch{"H141 CPQ"}="1H14 CPQ"; $resn_exch{"H142 CPQ"}="2H14 CPQ";
$resn_exch{"H151 CPQ"}="1H15 CPQ"; $resn_exch{"H161 CPQ"}="1H16 CPQ"; $resn_exch{"H162 CPQ"}="2H16 CPQ";
$resn_exch{"H171 CPQ"}="1H17 CPQ"; $resn_exch{"H172 CPQ"}="2H17 CPQ"; $resn_exch{"H211 CPQ"}="1H21 CPQ";
$resn_exch{"H212 CPQ"}="2H21 CPQ"; $resn_exch{"H213 CPQ"}="3H21 CPQ"; $resn_exch{"H221 CPQ"}="1H22 CPQ";
$resn_exch{"H222 CPQ"}="2H22 CPQ"; $resn_exch{"H231 CPQ"}="1H23 CPQ"; $resn_exch{"H232 CPQ"}="2H23 CPQ";
$resn_exch{"H741 CPQ"}="1H74 CPQ"; $resn_exch{"H742 CPQ"}="2H74 CPQ"; $resn_exch{"H751 CPQ"}="1H75 CPQ";
$resn_exch{"H752 CPQ"}="2H75 CPQ"; $resn_exch{"H761 CPQ"}="1H76 CPQ"; $resn_exch{"H762 CPQ"}="2H76 CPQ";
$resn_exch{"H771 CPQ"}="1H77 CPQ"; $resn_exch{"H801 CPQ"}="1H80 CPQ"; $resn_exch{"H861 CPQ"}="1H86 CPQ";
$resn_exch{"H811 CPQ"}="1H81 CPQ"; $resn_exch{"H871 CPQ"}="1H87 CPQ"; $resn_exch{"H821 CPQ"}="1H82 CPQ";
$resn_exch{"H881 CPQ"}="1H88 CPQ"; $resn_exch{"H831 CPQ"}="1H83 CPQ"; $resn_exch{"H891 CPQ"}="1H89 CPQ";
$resn_exch{"H841 CPQ"}="1H84 CPQ"; $resn_exch{"H842 CPQ"}="2H84 CPQ"; $resn_exch{"H851 CPQ"}="1H85 CPQ";
$resn_exch{"H681 CPQ"}="1H68 CPQ"; $resn_exch{"H682 CPQ"}="2H68 CPQ"; $resn_exch{"H691 CPQ"}="1H69 CPQ";
$resn_exch{"H692 CPQ"}="2H69 CPQ"; $resn_exch{"H701 CPQ"}="1H70 CPQ"; $resn_exch{"H702 CPQ"}="2H70 CPQ";
$resn_exch{"H711 CPQ"}="1H71 CPQ"; $resn_exch{"H901 CPQ"}="1H90 CPQ"; $resn_exch{"H961 CPQ"}="1H96 CPQ";
$resn_exch{"H911 CPQ"}="1H91 CPQ"; $resn_exch{"H971 CPQ"}="1H97 CPQ"; $resn_exch{"H921 CPQ"}="1H92 CPQ";
$resn_exch{"H981 CPQ"}="1H98 CPQ"; $resn_exch{"H931 CPQ"}="1H93 CPQ"; $resn_exch{"H991 CPQ"}="1H99 CPQ";
$resn_exch{"H941 CPQ"}="1H94 CPQ"; $resn_exch{"H942 CPQ"}="2H94 CPQ"; $resn_exch{"H951 CPQ"}="1H95 CPQ";
$resn_exch{"HOP2 CPR"}="2HOP CPR"; $resn_exch{"HOP3 CPR"}="3HOP CPR"; $resn_exch{"H5'1 CPR"}="1H5* CPR";
$resn_exch{"H5'2 CPR"}="2H5* CPR"; $resn_exch{" H4' CPR"}=" H4* CPR"; $resn_exch{" H3' CPR"}=" H3* CPR";
$resn_exch{"HO3' CPR"}="*HO3 CPR"; $resn_exch{" H2' CPR"}=" H2* CPR"; $resn_exch{"HO2' CPR"}="*HO2 CPR";
$resn_exch{" H1' CPR"}=" H1* CPR"; $resn_exch{" H11 CPS"}="1H1  CPS"; $resn_exch{" H12 CPS"}="2H1  CPS";
$resn_exch{" H31 CPS"}="1H3  CPS"; $resn_exch{" H32 CPS"}="2H3  CPS"; $resn_exch{" H71 CPS"}="1H7  CPS";
$resn_exch{" H72 CPS"}="2H7  CPS"; $resn_exch{" H81 CPS"}="1H8  CPS"; $resn_exch{" H82 CPS"}="2H8  CPS";
$resn_exch{"H101 CPS"}="1H10 CPS"; $resn_exch{"H102 CPS"}="2H10 CPS"; $resn_exch{"H103 CPS"}="3H10 CPS";
$resn_exch{"H111 CPS"}="1H11 CPS"; $resn_exch{"H112 CPS"}="2H11 CPS"; $resn_exch{"H113 CPS"}="3H11 CPS";
$resn_exch{"H121 CPS"}="1H12 CPS"; $resn_exch{"H122 CPS"}="2H12 CPS"; $resn_exch{"H141 CPS"}="1H14 CPS";
$resn_exch{"H142 CPS"}="2H14 CPS"; $resn_exch{"H161 CPS"}="1H16 CPS"; $resn_exch{"H162 CPS"}="2H16 CPS";
$resn_exch{"H211 CPS"}="1H21 CPS"; $resn_exch{"H212 CPS"}="2H21 CPS"; $resn_exch{"H213 CPS"}="3H21 CPS";
$resn_exch{"H221 CPS"}="1H22 CPS"; $resn_exch{"H222 CPS"}="2H22 CPS"; $resn_exch{"H231 CPS"}="1H23 CPS";
$resn_exch{"H232 CPS"}="2H23 CPS"; $resn_exch{"H251 CPS"}="1H25 CPS"; $resn_exch{"H252 CPS"}="2H25 CPS";
$resn_exch{"H261 CPS"}="1H26 CPS"; $resn_exch{"H262 CPS"}="2H26 CPS"; $resn_exch{"H271 CPS"}="1H27 CPS";
$resn_exch{"H272 CPS"}="2H27 CPS"; $resn_exch{"H281 CPS"}="1H28 CPS"; $resn_exch{"H282 CPS"}="2H28 CPS";
$resn_exch{"H283 CPS"}="3H28 CPS"; $resn_exch{"H291 CPS"}="1H29 CPS"; $resn_exch{"H292 CPS"}="2H29 CPS";
$resn_exch{"H293 CPS"}="3H29 CPS"; $resn_exch{"H301 CPS"}="1H30 CPS"; $resn_exch{"H302 CPS"}="2H30 CPS";
$resn_exch{"H311 CPS"}="1H31 CPS"; $resn_exch{"H312 CPS"}="2H31 CPS"; $resn_exch{"H321 CPS"}="1H32 CPS";
$resn_exch{"H322 CPS"}="2H32 CPS"; $resn_exch{" H11 CPT"}="1H1  CPT"; $resn_exch{" H12 CPT"}="2H1  CPT";
$resn_exch{" H13 CPT"}="3H1  CPT"; $resn_exch{" H21 CPT"}="1H2  CPT"; $resn_exch{" H22 CPT"}="2H2  CPT";
$resn_exch{" H23 CPT"}="3H2  CPT"; $resn_exch{" H   CPV"}="1HN  CPV"; $resn_exch{" H2  CPV"}="2HN  CPV";
$resn_exch{" HB2 CPV"}="1HB  CPV"; $resn_exch{" HB3 CPV"}="2HB  CPV"; $resn_exch{"HD12 CPV"}="1HD1 CPV";
$resn_exch{"HD13 CPV"}="2HD1 CPV"; $resn_exch{"HD23 CPV"}="1HD2 CPV"; $resn_exch{"HD22 CPV"}="2HD2 CPV";
$resn_exch{"HE12 CPV"}="1HE1 CPV"; $resn_exch{"HE13 CPV"}="2HE1 CPV"; $resn_exch{"HE23 CPV"}="1HE2 CPV";
$resn_exch{"HE22 CPV"}="2HE2 CPV"; $resn_exch{" HZ2 CPV"}="1HZ  CPV"; $resn_exch{" HZ3 CPV"}="2HZ  CPV";
$resn_exch{"HB11 CPV"}="1HB1 CPV"; $resn_exch{"HB12 CPV"}="2HB1 CPV"; $resn_exch{" HA' CPV"}=" HA* CPV";
$resn_exch{" HB' CPV"}=" HB* CPV"; $resn_exch{"HG11 CPV"}="1HG1 CPV"; $resn_exch{"HG12 CPV"}="2HG1 CPV";
$resn_exch{"HG13 CPV"}="3HG1 CPV"; $resn_exch{"HG21 CPV"}="1HG2 CPV"; $resn_exch{"HG22 CPV"}="2HG2 CPV";
$resn_exch{"HG23 CPV"}="3HG2 CPV"; $resn_exch{" H11 CPW"}="1H1  CPW"; $resn_exch{" H12 CPW"}="2H1  CPW";
$resn_exch{" H61 CPW"}="1H6  CPW"; $resn_exch{" H62 CPW"}="2H6  CPW"; $resn_exch{" H71 CPW"}="1H7  CPW";
$resn_exch{" H72 CPW"}="2H7  CPW"; $resn_exch{" H81 CPW"}="1H8  CPW"; $resn_exch{" H82 CPW"}="2H8  CPW";
$resn_exch{"HN31 CPW"}="1HN3 CPW"; $resn_exch{"HN32 CPW"}="2HN3 CPW"; $resn_exch{"H171 CPX"}="1H17 CPX";
$resn_exch{"H172 CPX"}="2H17 CPX"; $resn_exch{"H161 CPX"}="1H16 CPX"; $resn_exch{"H162 CPX"}="2H16 CPX";
$resn_exch{"H301 CPX"}="1H30 CPX"; $resn_exch{"H302 CPX"}="2H30 CPX"; $resn_exch{"H321 CPX"}="1H32 CPX";
$resn_exch{"H322 CPX"}="2H32 CPX"; $resn_exch{"H311 CPX"}="1H31 CPX"; $resn_exch{"H312 CPX"}="2H31 CPX";
$resn_exch{"H281 CPX"}="1H28 CPX"; $resn_exch{"H282 CPX"}="2H28 CPX"; $resn_exch{"H221 CPX"}="1H22 CPX";
$resn_exch{"H222 CPX"}="2H22 CPX"; $resn_exch{"H223 CPX"}="3H22 CPX"; $resn_exch{"H181 CPX"}="1H18 CPX";
$resn_exch{"H182 CPX"}="2H18 CPX"; $resn_exch{"H183 CPX"}="3H18 CPX"; $resn_exch{" H91 CPX"}="1H9  CPX";
$resn_exch{" H92 CPX"}="2H9  CPX"; $resn_exch{" H71 CPX"}="1H7  CPX"; $resn_exch{" H72 CPX"}="2H7  CPX";
$resn_exch{"H231 CPX"}="1H23 CPX"; $resn_exch{"H232 CPX"}="2H23 CPX"; $resn_exch{"H211 CPX"}="1H21 CPX";
$resn_exch{"H212 CPX"}="2H21 CPX"; $resn_exch{"H201 CPX"}="1H20 CPX"; $resn_exch{"H202 CPX"}="2H20 CPX";
$resn_exch{" HN1 CPY"}="1HN  CPY"; $resn_exch{" HN2 CPY"}="2HN  CPY"; $resn_exch{" HB1 CPY"}="1HB  CPY";
$resn_exch{" HB2 CPY"}="2HB  CPY"; $resn_exch{"HD11 CPY"}="1HD1 CPY"; $resn_exch{"HD12 CPY"}="2HD1 CPY";
$resn_exch{"HD21 CPY"}="1HD2 CPY"; $resn_exch{"HD22 CPY"}="2HD2 CPY"; $resn_exch{"HE11 CPY"}="1HE1 CPY";
$resn_exch{"HE12 CPY"}="2HE1 CPY"; $resn_exch{"HE21 CPY"}="1HE2 CPY"; $resn_exch{"HE22 CPY"}="2HE2 CPY";
$resn_exch{"HCZ1 CPY"}="1HCZ CPY"; $resn_exch{"HCZ2 CPY"}="2HCZ CPY"; $resn_exch{" HM1 CPY"}="1HM  CPY";
$resn_exch{" HM2 CPY"}="2HM  CPY"; $resn_exch{"H161 CQA"}="1H16 CQA"; $resn_exch{"H162 CQA"}="2H16 CQA";
$resn_exch{"H171 CQA"}="1H17 CQA"; $resn_exch{"H172 CQA"}="2H17 CQA"; $resn_exch{"H181 CQA"}="1H18 CQA";
$resn_exch{"H182 CQA"}="2H18 CQA"; $resn_exch{"H183 CQA"}="3H18 CQA"; $resn_exch{"H191 CQA"}="1H19 CQA";
$resn_exch{"H192 CQA"}="2H19 CQA"; $resn_exch{"H201 CQA"}="1H20 CQA"; $resn_exch{"H202 CQA"}="2H20 CQA";
$resn_exch{"H203 CQA"}="3H20 CQA"; $resn_exch{"H6C1 CQP"}="1H6C CQP"; $resn_exch{"H6C2 CQP"}="2H6C CQP";
$resn_exch{"H101 CQP"}="1H10 CQP"; $resn_exch{"H102 CQP"}="2H10 CQP"; $resn_exch{"H141 CQP"}="1H14 CQP";
$resn_exch{"H142 CQP"}="2H14 CQP"; $resn_exch{"H311 CQP"}="1H31 CQP"; $resn_exch{"H312 CQP"}="2H31 CQP";
$resn_exch{" HN1 CQR"}="1HN  CQR"; $resn_exch{" HN2 CQR"}="2HN  CQR"; $resn_exch{"HA11 CQR"}="1HA1 CQR";
$resn_exch{"HA12 CQR"}="2HA1 CQR"; $resn_exch{"HA31 CQR"}="1HA3 CQR"; $resn_exch{"HA32 CQR"}="2HA3 CQR";
$resn_exch{"HN11 CR0"}="1HN1 CR0"; $resn_exch{"HN12 CR0"}="2HN1 CR0"; $resn_exch{"HG11 CR0"}="1HG1 CR0";
$resn_exch{"HG12 CR0"}="2HG1 CR0"; $resn_exch{"HG13 CR0"}="3HG1 CR0"; $resn_exch{"HA31 CR0"}="1HA3 CR0";
$resn_exch{"HA32 CR0"}="2HA3 CR0"; $resn_exch{"HB21 CR0"}="1HB2 CR0"; $resn_exch{"HB22 CR0"}="2HB2 CR0";
$resn_exch{"HD11 CR0"}="1HD1 CR0"; $resn_exch{"HD12 CR0"}="2HD1 CR0"; $resn_exch{"HD13 CR0"}="3HD1 CR0";
$resn_exch{"HD21 CR0"}="1HD2 CR0"; $resn_exch{"HD22 CR0"}="2HD2 CR0"; $resn_exch{"HD23 CR0"}="3HD2 CR0";
$resn_exch{" H21 CR1"}="1H2  CR1"; $resn_exch{" H41 CR1"}="1H4  CR1"; $resn_exch{" H51 CR1"}="1H5  CR1";
$resn_exch{" H61 CR1"}="1H6  CR1"; $resn_exch{" H62 CR1"}="2H6  CR1"; $resn_exch{" H11 CR1"}="1H1  CR1";
$resn_exch{" H31 CR1"}="1H3  CR1"; $resn_exch{" H81 CR1"}="1H8  CR1"; $resn_exch{" H82 CR1"}="2H8  CR1";
$resn_exch{" H83 CR1"}="3H8  CR1"; $resn_exch{"HN11 CR2"}="1HN1 CR2"; $resn_exch{"HN12 CR2"}="2HN1 CR2";
$resn_exch{"HA11 CR2"}="1HA1 CR2"; $resn_exch{"HA12 CR2"}="2HA1 CR2"; $resn_exch{"HA31 CR2"}="1HA3 CR2";
$resn_exch{"HA32 CR2"}="2HA3 CR2"; $resn_exch{" HN1 CR5"}="1HN  CR5"; $resn_exch{"HN2A CR5"}="2HN  CR5";
$resn_exch{"HA11 CR5"}="1HA1 CR5"; $resn_exch{"HA12 CR5"}="2HA1 CR5"; $resn_exch{"HA31 CR5"}="1HA3 CR5";
$resn_exch{"HA32 CR5"}="2HA3 CR5"; $resn_exch{" H21 CR6"}="1H2  CR6"; $resn_exch{" H41 CR6"}="1H4  CR6";
$resn_exch{" H51 CR6"}="1H5  CR6"; $resn_exch{" H61 CR6"}="1H6  CR6"; $resn_exch{" H62 CR6"}="2H6  CR6";
$resn_exch{" H31 CR6"}="1H3  CR6"; $resn_exch{" H81 CR6"}="1H8  CR6"; $resn_exch{" H82 CR6"}="2H8  CR6";
$resn_exch{" H83 CR6"}="3H8  CR6"; $resn_exch{"HN11 CR7"}="1HN1 CR7"; $resn_exch{"HN12 CR7"}="2HN1 CR7";
$resn_exch{"HB11 CR7"}="1HB1 CR7"; $resn_exch{"HB12 CR7"}="2HB1 CR7"; $resn_exch{"HG11 CR7"}="1HG1 CR7";
$resn_exch{"HG12 CR7"}="2HG1 CR7"; $resn_exch{"HA31 CR7"}="1HA3 CR7"; $resn_exch{"HA32 CR7"}="2HA3 CR7";
$resn_exch{"HD1A CR7"}="1HD  CR7"; $resn_exch{"HD2A CR7"}="2HD  CR7"; $resn_exch{"HE1A CR7"}="1HE  CR7";
$resn_exch{"HE2A CR7"}="2HE  CR7"; $resn_exch{"HNZ1 CR7"}="1HNZ CR7"; $resn_exch{"HNZ2 CR7"}="2HNZ CR7";
$resn_exch{"H201 CR8"}="1H20 CR8"; $resn_exch{"H202 CR8"}="2H20 CR8"; $resn_exch{" H1  CR8"}="1H   CR8";
$resn_exch{" H2A CR8"}="2H   CR8"; $resn_exch{"H171 CR8"}="1H17 CR8"; $resn_exch{"H172 CR8"}="2H17 CR8";
$resn_exch{" H6B CR9"}="H6'' CR9"; $resn_exch{" H21 CRA"}="1H2  CRA"; $resn_exch{" H41 CRA"}="1H4  CRA";
$resn_exch{" H51 CRA"}="1H5  CRA"; $resn_exch{" H61 CRA"}="1H6  CRA"; $resn_exch{" H62 CRA"}="2H6  CRA";
$resn_exch{" H31 CRA"}="1H3  CRA"; $resn_exch{" H81 CRA"}="1H8  CRA"; $resn_exch{" H82 CRA"}="2H8  CRA";
$resn_exch{" H83 CRA"}="3H8  CRA"; $resn_exch{" H31 CRB"}="1H3  CRB"; $resn_exch{" H32 CRB"}="2H3  CRB";
$resn_exch{" H71 CRB"}="1H7  CRB"; $resn_exch{" H72 CRB"}="2H7  CRB"; $resn_exch{" H81 CRB"}="1H8  CRB";
$resn_exch{" H82 CRB"}="2H8  CRB"; $resn_exch{"HOP2 CRB"}="2HOP CRB"; $resn_exch{"HOP3 CRB"}="3HOP CRB";
$resn_exch{" H21 CRC"}="1H2  CRC"; $resn_exch{" H22 CRC"}="2H2  CRC"; $resn_exch{" H31 CRC"}="1H3  CRC";
$resn_exch{" H32 CRC"}="2H3  CRC"; $resn_exch{" H41 CRC"}="1H4  CRC"; $resn_exch{" H42 CRC"}="2H4  CRC";
$resn_exch{" H51 CRC"}="1H5  CRC"; $resn_exch{" H52 CRC"}="2H5  CRC"; $resn_exch{" H61 CRC"}="1H6  CRC";
$resn_exch{" H62 CRC"}="2H6  CRC"; $resn_exch{" H71 CRC"}="1H7  CRC"; $resn_exch{" H72 CRC"}="2H7  CRC";
$resn_exch{" H81 CRC"}="1H8  CRC"; $resn_exch{" H82 CRC"}="2H8  CRC"; $resn_exch{" H91 CRC"}="1H9  CRC";
$resn_exch{" H92 CRC"}="2H9  CRC"; $resn_exch{"H101 CRC"}="1H10 CRC"; $resn_exch{"H102 CRC"}="2H10 CRC";
$resn_exch{"H103 CRC"}="3H10 CRC"; $resn_exch{"HC11 CRD"}="1HC1 CRD"; $resn_exch{"HC12 CRD"}="2HC1 CRD";
$resn_exch{"HC13 CRD"}="3HC1 CRD"; $resn_exch{" H11 CRE"}="1H1  CRE"; $resn_exch{"H12A CRE"}="2H1  CRE";
$resn_exch{" H21 CRE"}="1H2  CRE"; $resn_exch{" H22 CRE"}="2H2  CRE"; $resn_exch{" H41 CRE"}="1H4  CRE";
$resn_exch{" H42 CRE"}="2H4  CRE"; $resn_exch{" H43 CRE"}="3H4  CRE"; $resn_exch{" H51 CRE"}="1H5  CRE";
$resn_exch{" H61 CRE"}="1H6  CRE"; $resn_exch{" H62 CRE"}="2H6  CRE"; $resn_exch{" H71 CRE"}="1H7  CRE";
$resn_exch{" H72 CRE"}="2H7  CRE"; $resn_exch{" H91 CRE"}="1H9  CRE"; $resn_exch{" H92 CRE"}="2H9  CRE";
$resn_exch{" H93 CRE"}="3H9  CRE"; $resn_exch{"H111 CRE"}="1H11 CRE"; $resn_exch{"H112 CRE"}="2H11 CRE";
$resn_exch{"H141 CRE"}="1H14 CRE"; $resn_exch{"H142 CRE"}="2H14 CRE"; $resn_exch{"H143 CRE"}="3H14 CRE";
$resn_exch{"H151 CRE"}="1H15 CRE"; $resn_exch{"H152 CRE"}="2H15 CRE"; $resn_exch{"HN11 CRF"}="1HN1 CRF";
$resn_exch{"HN12 CRF"}="2HN1 CRF"; $resn_exch{"HG11 CRF"}="1HG1 CRF"; $resn_exch{"HG12 CRF"}="2HG1 CRF";
$resn_exch{"HG13 CRF"}="3HG1 CRF"; $resn_exch{"HA31 CRF"}="1HA3 CRF"; $resn_exch{"HA32 CRF"}="2HA3 CRF";
$resn_exch{"HN11 CRG"}="1HN1 CRG"; $resn_exch{"HN12 CRG"}="2HN1 CRG"; $resn_exch{"HG11 CRG"}="1HG1 CRG";
$resn_exch{"HG12 CRG"}="2HG1 CRG"; $resn_exch{"HG13 CRG"}="3HG1 CRG"; $resn_exch{"HG1O CRG"}="OHG1 CRG";
$resn_exch{"HA31 CRG"}="1HA3 CRG"; $resn_exch{"HA32 CRG"}="2HA3 CRG"; $resn_exch{" H21 CRH"}="1H2  CRH";
$resn_exch{" H22 CRH"}="2H2  CRH"; $resn_exch{" H41 CRH"}="1H4  CRH"; $resn_exch{" H42 CRH"}="2H4  CRH";
$resn_exch{"H151 CRH"}="1H15 CRH"; $resn_exch{"H152 CRH"}="2H15 CRH"; $resn_exch{"H153 CRH"}="3H15 CRH";
$resn_exch{" H1' CRH"}=" H1* CRH"; $resn_exch{" H3' CRH"}=" H3* CRH"; $resn_exch{" H4' CRH"}=" H4* CRH";
$resn_exch{"H1'1 CRH"}="1H1* CRH"; $resn_exch{"H1'2 CRH"}="2H1* CRH"; $resn_exch{"H1'3 CRH"}="3H1* CRH";
$resn_exch{"H4'1 CRH"}="1H4* CRH"; $resn_exch{"H4'2 CRH"}="2H4* CRH"; $resn_exch{"H4'3 CRH"}="3H4* CRH";
$resn_exch{"H131 CRJ"}="1H13 CRJ"; $resn_exch{"H132 CRJ"}="2H13 CRJ"; $resn_exch{"H151 CRJ"}="1H15 CRJ";
$resn_exch{"H152 CRJ"}="2H15 CRJ"; $resn_exch{"H191 CRJ"}="1H19 CRJ"; $resn_exch{"H192 CRJ"}="2H19 CRJ";
$resn_exch{"H201 CRJ"}="1H20 CRJ"; $resn_exch{"H202 CRJ"}="2H20 CRJ"; $resn_exch{"H251 CRJ"}="1H25 CRJ";
$resn_exch{"H252 CRJ"}="2H25 CRJ"; $resn_exch{"H261 CRJ"}="1H26 CRJ"; $resn_exch{"H262 CRJ"}="2H26 CRJ";
$resn_exch{"HA31 CRK"}="1HA3 CRK"; $resn_exch{"HA32 CRK"}="2HA3 CRK"; $resn_exch{" HB1 CRK"}="1HB  CRK";
$resn_exch{"HB2A CRK"}="2HB  CRK"; $resn_exch{"HE1A CRK"}="1HE  CRK"; $resn_exch{"HE2A CRK"}="2HE  CRK";
$resn_exch{" HE3 CRK"}="3HE  CRK"; $resn_exch{" HG1 CRK"}="1HG  CRK"; $resn_exch{" HG2 CRK"}="2HG  CRK";
$resn_exch{" H21 CRL"}="1H2  CRL"; $resn_exch{" H22 CRL"}="2H2  CRL"; $resn_exch{" H31 CRL"}="1H3  CRL";
$resn_exch{" H32 CRL"}="2H3  CRL"; $resn_exch{" H41 CRL"}="1H4  CRL"; $resn_exch{" H42 CRL"}="2H4  CRL";
$resn_exch{" H51 CRL"}="1H5  CRL"; $resn_exch{" H52 CRL"}="2H5  CRL"; $resn_exch{"H101 CRL"}="1H10 CRL";
$resn_exch{"H102 CRL"}="2H10 CRL"; $resn_exch{"H111 CRL"}="1H11 CRL"; $resn_exch{"H112 CRL"}="2H11 CRL";
$resn_exch{"H121 CRL"}="1H12 CRL"; $resn_exch{"H122 CRL"}="2H12 CRL"; $resn_exch{"H131 CRL"}="1H13 CRL";
$resn_exch{"H132 CRL"}="2H13 CRL"; $resn_exch{"H141 CRL"}="1H14 CRL"; $resn_exch{"H142 CRL"}="2H14 CRL";
$resn_exch{"H151 CRL"}="1H15 CRL"; $resn_exch{"H152 CRL"}="2H15 CRL"; $resn_exch{"H153 CRL"}="3H15 CRL";
$resn_exch{"H161 CRL"}="1H16 CRL"; $resn_exch{"H162 CRL"}="2H16 CRL"; $resn_exch{"H163 CRL"}="3H16 CRL";
$resn_exch{"H171 CRL"}="1H17 CRL"; $resn_exch{"H172 CRL"}="2H17 CRL"; $resn_exch{"H173 CRL"}="3H17 CRL";
$resn_exch{"H171 CRM"}="1H17 CRM"; $resn_exch{"H172 CRM"}="2H17 CRM"; $resn_exch{"H251 CRM"}="1H25 CRM";
$resn_exch{"H252 CRM"}="2H25 CRM"; $resn_exch{"H111 CRM"}="1H11 CRM"; $resn_exch{"H112 CRM"}="2H11 CRM";
$resn_exch{"H121 CRM"}="1H12 CRM"; $resn_exch{"H122 CRM"}="2H12 CRM"; $resn_exch{" H51 CRN"}="1H5  CRN";
$resn_exch{" H52 CRN"}="2H5  CRN"; $resn_exch{" H41 CRN"}="1H4  CRN"; $resn_exch{" H42 CRN"}="2H4  CRN";
$resn_exch{" H43 CRN"}="3H4  CRN"; $resn_exch{"HN31 CRN"}="1HN3 CRN"; $resn_exch{"HN32 CRN"}="2HN3 CRN";
$resn_exch{"HN11 CRO"}="1HN1 CRO"; $resn_exch{" H2  CRO"}=" HN2 CRO"; $resn_exch{"HG11 CRO"}="1HG1 CRO";
$resn_exch{"HG12 CRO"}="2HG1 CRO"; $resn_exch{"HG13 CRO"}="3HG1 CRO"; $resn_exch{"HA31 CRO"}="1HA3 CRO";
$resn_exch{"HA32 CRO"}="2HA3 CRO"; $resn_exch{"H8'1 CRP"}="1H8' CRP"; $resn_exch{"H8'2 CRP"}="2H8' CRP";
$resn_exch{"H8'3 CRP"}="3H8' CRP"; $resn_exch{" H41 CRP"}="1H4  CRP"; $resn_exch{" H42 CRP"}="2H4  CRP";
$resn_exch{" H43 CRP"}="3H4  CRP"; $resn_exch{" H51 CRP"}="1H5  CRP"; $resn_exch{" H52 CRP"}="2H5  CRP";
$resn_exch{" H61 CRP"}="1H6  CRP"; $resn_exch{" H62 CRP"}="2H6  CRP"; $resn_exch{" H63 CRP"}="3H6  CRP";
$resn_exch{"HB11 CRQ"}="1HB1 CRQ"; $resn_exch{"HB12 CRQ"}="2HB1 CRQ"; $resn_exch{"HG11 CRQ"}="1HG1 CRQ";
$resn_exch{"HG12 CRQ"}="2HG1 CRQ"; $resn_exch{"HA31 CRQ"}="1HA3 CRQ"; $resn_exch{"HA32 CRQ"}="2HA3 CRQ";
$resn_exch{" HB1 CRQ"}="1HB  CRQ"; $resn_exch{"HE11 CRQ"}="1HE1 CRQ"; $resn_exch{"HE12 CRQ"}="2HE1 CRQ";
$resn_exch{" H71 CRS"}="1H7  CRS"; $resn_exch{" H72 CRS"}="2H7  CRS"; $resn_exch{" H73 CRS"}="3H7  CRS";
$resn_exch{"H1M1 CRT"}="1H1M CRT"; $resn_exch{"H1M2 CRT"}="2H1M CRT"; $resn_exch{"H1M3 CRT"}="3H1M CRT";
$resn_exch{"H21A CRT"}="1H2  CRT"; $resn_exch{"H22A CRT"}="2H2  CRT"; $resn_exch{" H23 CRT"}="3H2  CRT";
$resn_exch{"H31A CRT"}="1H3  CRT"; $resn_exch{"H32A CRT"}="2H3  CRT"; $resn_exch{" H33 CRT"}="3H3  CRT";
$resn_exch{" H41 CRT"}="1H4  CRT"; $resn_exch{" H42 CRT"}="2H4  CRT"; $resn_exch{" H81 CRT"}="1H8  CRT";
$resn_exch{" H82 CRT"}="2H8  CRT"; $resn_exch{" H83 CRT"}="3H8  CRT"; $resn_exch{"H131 CRT"}="1H13 CRT";
$resn_exch{"H132 CRT"}="2H13 CRT"; $resn_exch{"H133 CRT"}="3H13 CRT"; $resn_exch{"H181 CRT"}="1H18 CRT";
$resn_exch{"H182 CRT"}="2H18 CRT"; $resn_exch{"H183 CRT"}="3H18 CRT"; $resn_exch{"H241 CRT"}="1H24 CRT";
$resn_exch{"H242 CRT"}="2H24 CRT"; $resn_exch{"H243 CRT"}="3H24 CRT"; $resn_exch{"H291 CRT"}="1H29 CRT";
$resn_exch{"H292 CRT"}="2H29 CRT"; $resn_exch{"H293 CRT"}="3H29 CRT"; $resn_exch{"H341 CRT"}="1H34 CRT";
$resn_exch{"H342 CRT"}="2H34 CRT"; $resn_exch{"H343 CRT"}="3H34 CRT"; $resn_exch{"H371 CRT"}="1H37 CRT";
$resn_exch{"H372 CRT"}="2H37 CRT"; $resn_exch{"H391 CRT"}="1H39 CRT"; $resn_exch{"H392 CRT"}="2H39 CRT";
$resn_exch{"H393 CRT"}="3H39 CRT"; $resn_exch{"H401 CRT"}="1H40 CRT"; $resn_exch{"H402 CRT"}="2H40 CRT";
$resn_exch{"H403 CRT"}="3H40 CRT"; $resn_exch{"H2M1 CRT"}="1H2M CRT"; $resn_exch{"H2M2 CRT"}="2H2M CRT";
$resn_exch{"H2M3 CRT"}="3H2M CRT"; $resn_exch{"HE21 CRU"}="1HE2 CRU"; $resn_exch{"HG11 CRU"}="1HG1 CRU";
$resn_exch{"HG12 CRU"}="2HG1 CRU"; $resn_exch{"HB11 CRU"}="1HB1 CRU"; $resn_exch{"HB12 CRU"}="2HB1 CRU";
$resn_exch{"HA31 CRU"}="1HA3 CRU"; $resn_exch{"HA32 CRU"}="2HA3 CRU"; $resn_exch{" HB1 CRU"}="1HB  CRU";
$resn_exch{"H101 CRV"}="1H10 CRV"; $resn_exch{"H102 CRV"}="2H10 CRV"; $resn_exch{"H121 CRV"}="1H12 CRV";
$resn_exch{"H122 CRV"}="2H12 CRV"; $resn_exch{"H171 CRV"}="1H17 CRV"; $resn_exch{"H172 CRV"}="2H17 CRV";
$resn_exch{"H181 CRV"}="1H18 CRV"; $resn_exch{"H182 CRV"}="2H18 CRV"; $resn_exch{"H191 CRV"}="1H19 CRV";
$resn_exch{"H192 CRV"}="2H19 CRV"; $resn_exch{"H161 CRV"}="1H16 CRV"; $resn_exch{"H162 CRV"}="2H16 CRV";
$resn_exch{"HN11 CRW"}="1HN1 CRW"; $resn_exch{"HN12 CRW"}="2HN1 CRW"; $resn_exch{"HB11 CRW"}="1HB1 CRW";
$resn_exch{"HB12 CRW"}="2HB1 CRW"; $resn_exch{"HB13 CRW"}="3HB1 CRW"; $resn_exch{"HA31 CRW"}="1HA3 CRW";
$resn_exch{"HA32 CRW"}="2HA3 CRW"; $resn_exch{"HB21 CRW"}="1HB2 CRW"; $resn_exch{"HB22 CRW"}="2HB2 CRW";
$resn_exch{"HN11 CRX"}="1HN1 CRX"; $resn_exch{"HN12 CRX"}="2HN1 CRX"; $resn_exch{"HB11 CRX"}="1HB1 CRX";
$resn_exch{"HB12 CRX"}="2HB1 CRX"; $resn_exch{"HB13 CRX"}="3HB1 CRX"; $resn_exch{"HA31 CRX"}="1HA3 CRX";
$resn_exch{"HA32 CRX"}="2HA3 CRX"; $resn_exch{"HB21 CRX"}="1HB2 CRX"; $resn_exch{"HB22 CRX"}="2HB2 CRX";
$resn_exch{" H11 CRY"}="1H1  CRY"; $resn_exch{" H12 CRY"}="2H1  CRY"; $resn_exch{" H21 CRY"}="1H2  CRY";
$resn_exch{" H31 CRY"}="1H3  CRY"; $resn_exch{" H32 CRY"}="2H3  CRY"; $resn_exch{" H21 CRZ"}="1H2  CRZ";
$resn_exch{" H22 CRZ"}="2H2  CRZ"; $resn_exch{" H31 CRZ"}="1H3  CRZ"; $resn_exch{" H32 CRZ"}="2H3  CRZ";
$resn_exch{" H41 CRZ"}="1H4  CRZ"; $resn_exch{" H42 CRZ"}="2H4  CRZ"; $resn_exch{" H   CS1"}=" HN1 CS1";
$resn_exch{" H2  CS1"}=" HN2 CS1"; $resn_exch{" HB2 CS1"}=" HB1 CS1"; $resn_exch{" HB3 CS1"}=" HB2 CS1";
$resn_exch{" HC2 CS1"}=" H2  CS1"; $resn_exch{" H61 CS2"}="1H6  CS2"; $resn_exch{" H62 CS2"}="2H6  CS2";
$resn_exch{" H   CS3"}="1HN  CS3"; $resn_exch{" H2  CS3"}="2HN  CS3"; $resn_exch{" HB2 CS3"}="1HB  CS3";
$resn_exch{" HB3 CS3"}="2HB  CS3"; $resn_exch{"H1H1 CS3"}="1H1H CS3"; $resn_exch{"H1H2 CS3"}="2H1H CS3";
$resn_exch{"H1I1 CS3"}="1H1I CS3"; $resn_exch{"H1I2 CS3"}="2H1I CS3"; $resn_exch{" H   CS4"}="1HN  CS4";
$resn_exch{" H2  CS4"}="2HN  CS4"; $resn_exch{" HB2 CS4"}="1HB  CS4"; $resn_exch{" HB3 CS4"}="2HB  CS4";
$resn_exch{"H1J1 CS4"}="1H1J CS4"; $resn_exch{"H1J2 CS4"}="2H1J CS4"; $resn_exch{"H1K1 CS4"}="1H1K CS4";
$resn_exch{"H1K2 CS4"}="2H1K CS4"; $resn_exch{" H81 CS5"}="1H8  CS5"; $resn_exch{" H82 CS5"}="2H8  CS5";
$resn_exch{" H91 CS5"}="1H9  CS5"; $resn_exch{" H92 CS5"}="2H9  CS5"; $resn_exch{"H101 CS5"}="1H10 CS5";
$resn_exch{"H102 CS5"}="2H10 CS5"; $resn_exch{"H103 CS5"}="3H10 CS5"; $resn_exch{"H111 CS5"}="1H11 CS5";
$resn_exch{"H112 CS5"}="2H11 CS5"; $resn_exch{"H121 CS5"}="1H12 CS5"; $resn_exch{"H122 CS5"}="2H12 CS5";
$resn_exch{"H131 CS5"}="1H13 CS5"; $resn_exch{"H132 CS5"}="2H13 CS5"; $resn_exch{"H133 CS5"}="3H13 CS5";
$resn_exch{"H171 CS5"}="1H17 CS5"; $resn_exch{"H172 CS5"}="2H17 CS5"; $resn_exch{"H181 CS5"}="1H18 CS5";
$resn_exch{"H182 CS5"}="2H18 CS5"; $resn_exch{"H191 CS5"}="1H19 CS5"; $resn_exch{"H192 CS5"}="2H19 CS5";
$resn_exch{"H321 CS5"}="1H32 CS5"; $resn_exch{"H322 CS5"}="2H32 CS5"; $resn_exch{"H323 CS5"}="3H32 CS5";
$resn_exch{"H331 CS5"}="1H33 CS5"; $resn_exch{"H332 CS5"}="2H33 CS5"; $resn_exch{"H333 CS5"}="3H33 CS5";
$resn_exch{" H41 CS7"}="1H4  CS7"; $resn_exch{" H42 CS7"}="2H4  CS7"; $resn_exch{" H61 CS7"}="1H6  CS7";
$resn_exch{" H62 CS7"}="2H6  CS7"; $resn_exch{" H71 CS7"}="1H7  CS7"; $resn_exch{" H72 CS7"}="2H7  CS7";
$resn_exch{"H221 CS7"}="1H22 CS7"; $resn_exch{"H222 CS7"}="2H22 CS7"; $resn_exch{"H223 CS7"}="3H22 CS7";
$resn_exch{"H231 CS7"}="1H23 CS7"; $resn_exch{"H232 CS7"}="2H23 CS7"; $resn_exch{"H301 CS7"}="1H30 CS7";
$resn_exch{"H302 CS7"}="2H30 CS7"; $resn_exch{"H311 CS7"}="1H31 CS7"; $resn_exch{"H312 CS7"}="2H31 CS7";
$resn_exch{"H321 CS7"}="1H32 CS7"; $resn_exch{"H322 CS7"}="2H32 CS7"; $resn_exch{"H323 CS7"}="3H32 CS7";
$resn_exch{"H331 CS7"}="1H33 CS7"; $resn_exch{"H332 CS7"}="2H33 CS7"; $resn_exch{"H341 CS7"}="1H34 CS7";
$resn_exch{"H342 CS7"}="2H34 CS7"; $resn_exch{"H351 CS7"}="1H35 CS7"; $resn_exch{"H352 CS7"}="2H35 CS7";
$resn_exch{"H353 CS7"}="3H35 CS7"; $resn_exch{" H71 CS8"}="1H7  CS8"; $resn_exch{" H72 CS8"}="2H7  CS8";
$resn_exch{" H61 CS8"}="1H6  CS8"; $resn_exch{" H62 CS8"}="2H6  CS8"; $resn_exch{" H31 CS8"}="1H3  CS8";
$resn_exch{" H32 CS8"}="2H3  CS8"; $resn_exch{" H21 CS8"}="1H2  CS8"; $resn_exch{" H22 CS8"}="2H2  CS8";
$resn_exch{"H2'1 CS8"}="1H2* CS8"; $resn_exch{"H2'2 CS8"}="2H2* CS8"; $resn_exch{"H4'1 CS8"}="1H4* CS8";
$resn_exch{"H4'2 CS8"}="2H4* CS8"; $resn_exch{"H5'1 CS8"}="1H5* CS8"; $resn_exch{"H5'2 CS8"}="2H5* CS8";
$resn_exch{"H6'1 CS8"}="1H6* CS8"; $resn_exch{"H6'2 CS8"}="2H6* CS8"; $resn_exch{"H7'1 CS8"}="1H7* CS8";
$resn_exch{"H7'2 CS8"}="2H7* CS8"; $resn_exch{"H8'1 CS8"}="1H8* CS8"; $resn_exch{"H8'2 CS8"}="2H8* CS8";
$resn_exch{"H8'3 CS8"}="3H8* CS8"; $resn_exch{"H131 CS8"}="1H13 CS8"; $resn_exch{"H132 CS8"}="2H13 CS8";
$resn_exch{"H133 CS8"}="3H13 CS8"; $resn_exch{"H141 CS8"}="1H14 CS8"; $resn_exch{"H142 CS8"}="2H14 CS8";
$resn_exch{"H143 CS8"}="3H14 CS8"; $resn_exch{"H121 CS8"}="1H12 CS8"; $resn_exch{"H122 CS8"}="2H12 CS8";
$resn_exch{" H11 CS8"}="1H1  CS8"; $resn_exch{" H12 CS8"}="2H1  CS8"; $resn_exch{" H4' CS8"}=" H4* CS8";
$resn_exch{" H1' CS8"}=" H1* CS8"; $resn_exch{"HN61 CS8"}="1HN6 CS8"; $resn_exch{"HN62 CS8"}="2HN6 CS8";
$resn_exch{" H2' CS8"}=" H2* CS8"; $resn_exch{" H3' CS8"}=" H3* CS8"; $resn_exch{" H81 CS9"}="1H8  CS9";
$resn_exch{" H82 CS9"}="2H8  CS9"; $resn_exch{" H91 CS9"}="1H9  CS9"; $resn_exch{" H92 CS9"}="2H9  CS9";
$resn_exch{"H101 CS9"}="1H10 CS9"; $resn_exch{"H102 CS9"}="2H10 CS9"; $resn_exch{"H103 CS9"}="3H10 CS9";
$resn_exch{"H111 CS9"}="1H11 CS9"; $resn_exch{"H112 CS9"}="2H11 CS9"; $resn_exch{"H121 CS9"}="1H12 CS9";
$resn_exch{"H122 CS9"}="2H12 CS9"; $resn_exch{"H131 CS9"}="1H13 CS9"; $resn_exch{"H132 CS9"}="2H13 CS9";
$resn_exch{"H133 CS9"}="3H13 CS9"; $resn_exch{"H301 CS9"}="1H30 CS9"; $resn_exch{"H302 CS9"}="2H30 CS9";
$resn_exch{"H303 CS9"}="3H30 CS9"; $resn_exch{"H171 CS9"}="1H17 CS9"; $resn_exch{"H172 CS9"}="2H17 CS9";
$resn_exch{"H181 CS9"}="1H18 CS9"; $resn_exch{"H182 CS9"}="2H18 CS9"; $resn_exch{"H201 CS9"}="1H20 CS9";
$resn_exch{"H202 CS9"}="2H20 CS9"; $resn_exch{" H   CSA"}="1HN  CSA"; $resn_exch{" H2  CSA"}="2HN  CSA";
$resn_exch{" HB2 CSA"}="1HB  CSA"; $resn_exch{" HB3 CSA"}="2HB  CSA"; $resn_exch{" H31 CSA"}="1H3  CSA";
$resn_exch{" H32 CSA"}="2H3  CSA"; $resn_exch{" H11 CSA"}="1H1  CSA"; $resn_exch{" H12 CSA"}="2H1  CSA";
$resn_exch{" H13 CSA"}="3H1  CSA"; $resn_exch{" H   CSB"}="1HN  CSB"; $resn_exch{" H2  CSB"}="2HN  CSB";
$resn_exch{" HB2 CSB"}="1HB  CSB"; $resn_exch{" HB3 CSB"}="2HB  CSB"; $resn_exch{" H11 CSC"}="1H1  CSC";
$resn_exch{" H12 CSC"}="2H1  CSC"; $resn_exch{"H3'1 CSC"}="1H3' CSC"; $resn_exch{"H3'2 CSC"}="2H3' CSC";
$resn_exch{"H131 CSC"}="1H13 CSC"; $resn_exch{"H132 CSC"}="2H13 CSC"; $resn_exch{"H141 CSC"}="1H14 CSC";
$resn_exch{"H142 CSC"}="2H14 CSC"; $resn_exch{"H151 CSC"}="1H15 CSC"; $resn_exch{"H152 CSC"}="2H15 CSC";
$resn_exch{"H171 CSC"}="1H17 CSC"; $resn_exch{"H172 CSC"}="2H17 CSC"; $resn_exch{"H173 CSC"}="3H17 CSC";
$resn_exch{"H201 CSC"}="1H20 CSC"; $resn_exch{"H202 CSC"}="2H20 CSC"; $resn_exch{"H203 CSC"}="3H20 CSC";
$resn_exch{" H   CSD"}="1HN  CSD"; $resn_exch{" H2  CSD"}="2HN  CSD"; $resn_exch{" HB2 CSD"}="1HB  CSD";
$resn_exch{" HB3 CSD"}="2HB  CSD"; $resn_exch{" H   CSE"}="1HN  CSE"; $resn_exch{" H2  CSE"}="2HN  CSE";
$resn_exch{" HB2 CSE"}="1HB  CSE"; $resn_exch{" HB3 CSE"}="2HB  CSE"; $resn_exch{"H9A1 CSF"}="1H9A CSF";
$resn_exch{"H9A2 CSF"}="2H9A CSF"; $resn_exch{" H3' CSF"}=" H3* CSF"; $resn_exch{" H2' CSF"}=" H2* CSF";
$resn_exch{" H1' CSF"}=" H1* CSF"; $resn_exch{" H4' CSF"}=" H4* CSF"; $resn_exch{" H5' CSF"}="1H5* CSF";
$resn_exch{"H5'' CSF"}="2H5* CSF"; $resn_exch{"HO3' CSF"}="H3T  CSF"; $resn_exch{" H41 CSF"}="1H4  CSF";
$resn_exch{" H42 CSF"}="2H4  CSF"; $resn_exch{"H111 CSF"}="1H11 CSF"; $resn_exch{"H112 CSF"}="2H11 CSF";
$resn_exch{"H113 CSF"}="3H11 CSF"; $resn_exch{"HA31 CSH"}="1HA3 CSH"; $resn_exch{"HA32 CSH"}="2HA3 CSH";
$resn_exch{"HN11 CSH"}="1HN1 CSH"; $resn_exch{"HN12 CSH"}="2HN1 CSH"; $resn_exch{"HB11 CSH"}="1HB1 CSH";
$resn_exch{"HB12 CSH"}="2HB1 CSH"; $resn_exch{"HB21 CSH"}="1HB2 CSH"; $resn_exch{"HB22 CSH"}="2HB2 CSH";
$resn_exch{" HE  CSI"}=" HNE CSI"; $resn_exch{"HD13 CSI"}="1HCD CSI"; $resn_exch{"HD12 CSI"}="2HCD CSI";
$resn_exch{" HH  CSI"}=" HNH CSI"; $resn_exch{" HB  CSI"}=" HCB CSI"; $resn_exch{"HG13 CSI"}="1HCG CSI";
$resn_exch{"HG12 CSI"}="2HCG CSI"; $resn_exch{" HG2 CSI"}=" HNG CSI"; $resn_exch{" H   CSI"}="1HN  CSI";
$resn_exch{" H2  CSI"}="2HN  CSI"; $resn_exch{"HOP2 CSL"}="2HOP CSL"; $resn_exch{"H5'1 CSL"}="1H5* CSL";
$resn_exch{"H5'2 CSL"}="2H5* CSL"; $resn_exch{" H4' CSL"}=" H4* CSL"; $resn_exch{" H1' CSL"}=" H1* CSL";
$resn_exch{"HN41 CSL"}="1HN4 CSL"; $resn_exch{"HN42 CSL"}="2HN4 CSL"; $resn_exch{" H2' CSL"}=" H2* CSL";
$resn_exch{"HA'1 CSL"}="1HA* CSL"; $resn_exch{"HA'2 CSL"}="2HA* CSL"; $resn_exch{"HA'3 CSL"}="3HA* CSL";
$resn_exch{" H3' CSL"}=" H3* CSL"; $resn_exch{"HOP3 CSL"}="3HOP CSL"; $resn_exch{" H   CSO"}="1HN  CSO";
$resn_exch{" HN2 CSO"}="2HN  CSO"; $resn_exch{" HB2 CSO"}="1HB  CSO"; $resn_exch{" HB3 CSO"}="2HB  CSO";
$resn_exch{" H   CSP"}="1HN  CSP"; $resn_exch{" H2  CSP"}="2HN  CSP"; $resn_exch{" HB2 CSP"}="1HB  CSP";
$resn_exch{" HB3 CSP"}="2HB  CSP"; $resn_exch{"HO2P CSP"}="PHO2 CSP"; $resn_exch{"HO3P CSP"}="PHO3 CSP";
$resn_exch{" H2  CSR"}=" HN2 CSR"; $resn_exch{" HB2 CSR"}="1HB  CSR"; $resn_exch{" HB3 CSR"}="2HB  CSR";
$resn_exch{" H   CSS"}="1HN  CSS"; $resn_exch{" HN2 CSS"}="2HN  CSS"; $resn_exch{" HB2 CSS"}="1HB  CSS";
$resn_exch{" HB3 CSS"}="2HB  CSS"; $resn_exch{" H71 CST"}="1H7  CST"; $resn_exch{" H72 CST"}="2H7  CST";
$resn_exch{"H121 CST"}="1H12 CST"; $resn_exch{"H122 CST"}="2H12 CST"; $resn_exch{"H141 CST"}="1H14 CST";
$resn_exch{"H142 CST"}="2H14 CST"; $resn_exch{"H143 CST"}="3H14 CST"; $resn_exch{"H151 CST"}="1H15 CST";
$resn_exch{"H152 CST"}="2H15 CST"; $resn_exch{"H153 CST"}="3H15 CST"; $resn_exch{"H191 CST"}="1H19 CST";
$resn_exch{"H192 CST"}="2H19 CST"; $resn_exch{"H201 CST"}="1H20 CST"; $resn_exch{"H202 CST"}="2H20 CST";
$resn_exch{"H251 CST"}="1H25 CST"; $resn_exch{"H252 CST"}="2H25 CST"; $resn_exch{"HN41 CST"}="1HN4 CST";
$resn_exch{"HN42 CST"}="2HN4 CST"; $resn_exch{" H   CSU"}="1HN  CSU"; $resn_exch{" H2  CSU"}="2HN  CSU";
$resn_exch{" HB2 CSU"}="1HB  CSU"; $resn_exch{" HB3 CSU"}="2HB  CSU"; $resn_exch{" H   CSW"}="1HN  CSW";
$resn_exch{" H2  CSW"}="2HN  CSW"; $resn_exch{" HB2 CSW"}="1HB  CSW"; $resn_exch{" HB3 CSW"}="2HB  CSW";
$resn_exch{" HG  CSW"}=" HS  CSW"; $resn_exch{" H   CSX"}="1HN  CSX"; $resn_exch{" H2  CSX"}="2HN  CSX";
$resn_exch{" HB2 CSX"}="1HB  CSX"; $resn_exch{" HB3 CSX"}="2HB  CSX"; $resn_exch{" HG  CSX"}=" HS  CSX";
$resn_exch{"HA31 CSY"}="1HA3 CSY"; $resn_exch{"HA32 CSY"}="2HA3 CSY"; $resn_exch{"HN11 CSY"}="1HN1 CSY";
$resn_exch{"HN12 CSY"}="2HN1 CSY"; $resn_exch{"HB11 CSY"}="1HB1 CSY"; $resn_exch{"HB12 CSY"}="2HB1 CSY";
$resn_exch{"HB21 CSY"}="1HB2 CSY"; $resn_exch{"HB22 CSY"}="2HB2 CSY"; $resn_exch{" H2  CSZ"}=" HN2 CSZ";
$resn_exch{" HB2 CSZ"}="1HB  CSZ"; $resn_exch{" HB3 CSZ"}="2HB  CSZ"; $resn_exch{" HE  CSZ"}="HSE1 CSZ";
$resn_exch{" H91 CT0"}="1H9  CT0"; $resn_exch{" H92 CT0"}="2H9  CT0"; $resn_exch{"H101 CT0"}="1H10 CT0";
$resn_exch{"H102 CT0"}="2H10 CT0"; $resn_exch{"H111 CT0"}="1H11 CT0"; $resn_exch{"H112 CT0"}="2H11 CT0";
$resn_exch{"H121 CT0"}="1H12 CT0"; $resn_exch{"H122 CT0"}="2H12 CT0"; $resn_exch{" H11 CT1"}="1H1  CT1";
$resn_exch{"H12A CT1"}="2H1  CT1"; $resn_exch{" H13 CT1"}="3H1  CT1"; $resn_exch{" H21 CT1"}="1H2  CT1";
$resn_exch{" H22 CT1"}="2H2  CT1"; $resn_exch{" H31 CT1"}="1H3  CT1"; $resn_exch{" H32 CT1"}="2H3  CT1";
$resn_exch{" H41 CT1"}="1H4  CT1"; $resn_exch{" H42 CT1"}="2H4  CT1"; $resn_exch{"H131 CT1"}="1H13 CT1";
$resn_exch{"H132 CT1"}="2H13 CT1"; $resn_exch{"H211 CT1"}="1H21 CT1"; $resn_exch{"H212 CT1"}="2H21 CT1";
$resn_exch{"H213 CT1"}="3H21 CT1"; $resn_exch{"H221 CT1"}="1H22 CT1"; $resn_exch{"H222 CT1"}="2H22 CT1";
$resn_exch{"H223 CT1"}="3H22 CT1"; $resn_exch{" H11 CT2"}="1H1  CT2"; $resn_exch{" H12 CT2"}="2H1  CT2";
$resn_exch{" H13 CT2"}="3H1  CT2"; $resn_exch{" H21 CT2"}="1H2  CT2"; $resn_exch{" H22 CT2"}="2H2  CT2";
$resn_exch{" H31 CT2"}="1H3  CT2"; $resn_exch{" H32 CT2"}="2H3  CT2"; $resn_exch{" H41 CT2"}="1H4  CT2";
$resn_exch{" H42 CT2"}="2H4  CT2"; $resn_exch{"H131 CT2"}="1H13 CT2"; $resn_exch{"H132 CT2"}="2H13 CT2";
$resn_exch{"H201 CT2"}="1H20 CT2"; $resn_exch{"H202 CT2"}="2H20 CT2"; $resn_exch{"H211 CT2"}="1H21 CT2";
$resn_exch{"H212 CT2"}="2H21 CT2"; $resn_exch{"H221 CT2"}="1H22 CT2"; $resn_exch{"H222 CT2"}="2H22 CT2";
$resn_exch{"H231 CT2"}="1H23 CT2"; $resn_exch{"H232 CT2"}="2H23 CT2"; $resn_exch{"H6A1 CT3"}="1H6A CT3";
$resn_exch{"H6A2 CT3"}="2H6A CT3"; $resn_exch{"HOA2 CT3"}="2HOA CT3"; $resn_exch{"HOA3 CT3"}="3HOA CT3";
$resn_exch{"HOA6 CT3"}="6HOA CT3"; $resn_exch{"H6B1 CT3"}="1H6B CT3"; $resn_exch{"H6B2 CT3"}="2H6B CT3";
$resn_exch{"HOB2 CT3"}="2HOB CT3"; $resn_exch{"HOB3 CT3"}="3HOB CT3"; $resn_exch{"HOB6 CT3"}="6HOB CT3";
$resn_exch{"H6C1 CT3"}="1H6C CT3"; $resn_exch{"H6C2 CT3"}="2H6C CT3"; $resn_exch{"HOC2 CT3"}="2HOC CT3";
$resn_exch{"HOC3 CT3"}="3HOC CT3"; $resn_exch{"HOC4 CT3"}="4HOC CT3"; $resn_exch{"HOC6 CT3"}="6HOC CT3";
$resn_exch{"H7C1 CT5"}="1H7C CT5"; $resn_exch{"H7C2 CT5"}="2H7C CT5"; $resn_exch{"H7C3 CT5"}="3H7C CT5";
$resn_exch{"H261 CT5"}="1H26 CT5"; $resn_exch{"H262 CT5"}="2H26 CT5"; $resn_exch{"H271 CT5"}="1H27 CT5";
$resn_exch{"H272 CT5"}="2H27 CT5"; $resn_exch{"H273 CT5"}="3H27 CT5"; $resn_exch{"H171 CT5"}="1H17 CT5";
$resn_exch{"H172 CT5"}="2H17 CT5"; $resn_exch{"H181 CT5"}="1H18 CT5"; $resn_exch{"H182 CT5"}="2H18 CT5";
$resn_exch{"H131 CT6"}="1H13 CT6"; $resn_exch{"H132 CT6"}="2H13 CT6"; $resn_exch{"H201 CT7"}="1H20 CT7";
$resn_exch{"H202 CT7"}="2H20 CT7"; $resn_exch{"H203 CT7"}="3H20 CT7"; $resn_exch{"H131 CT8"}="1H13 CT8";
$resn_exch{"H132 CT8"}="2H13 CT8"; $resn_exch{"HN81 CT9"}="1HN8 CT9"; $resn_exch{"HN82 CT9"}="2HN8 CT9";
$resn_exch{"H611 CT9"}="1H61 CT9"; $resn_exch{"H612 CT9"}="2H61 CT9"; $resn_exch{"H613 CT9"}="3H61 CT9";
$resn_exch{"H621 CT9"}="1H62 CT9"; $resn_exch{"H622 CT9"}="2H62 CT9"; $resn_exch{"H623 CT9"}="3H62 CT9";
$resn_exch{" H51 CT9"}="1H5  CT9"; $resn_exch{" H52 CT9"}="2H5  CT9"; $resn_exch{" H61 CT9"}="1H6  CT9";
$resn_exch{" H62 CT9"}="2H6  CT9"; $resn_exch{" H31 CT9"}="1H3  CT9"; $resn_exch{" H32 CT9"}="2H3  CT9";
$resn_exch{" H21 CT9"}="1H2  CT9"; $resn_exch{" H22 CT9"}="2H2  CT9"; $resn_exch{"H211 CT9"}="1H21 CT9";
$resn_exch{"H212 CT9"}="2H21 CT9"; $resn_exch{"H213 CT9"}="3H21 CT9"; $resn_exch{"H221 CT9"}="1H22 CT9";
$resn_exch{"H222 CT9"}="2H22 CT9"; $resn_exch{"H223 CT9"}="3H22 CT9"; $resn_exch{"H151 CTA"}="1H15 CTA";
$resn_exch{"H152 CTA"}="2H15 CTA"; $resn_exch{"H161 CTA"}="1H16 CTA"; $resn_exch{"H162 CTA"}="2H16 CTA";
$resn_exch{"H171 CTA"}="1H17 CTA"; $resn_exch{"H172 CTA"}="2H17 CTA"; $resn_exch{"H211 CTA"}="1H21 CTA";
$resn_exch{"H212 CTA"}="2H21 CTA"; $resn_exch{"H241 CTA"}="1H24 CTA"; $resn_exch{"H242 CTA"}="2H24 CTA";
$resn_exch{"H331 CTA"}="1H33 CTA"; $resn_exch{"H332 CTA"}="2H33 CTA"; $resn_exch{"H421 CTA"}="1H42 CTA";
$resn_exch{"H422 CTA"}="2H42 CTA"; $resn_exch{"H431 CTA"}="1H43 CTA"; $resn_exch{"H432 CTA"}="2H43 CTA";
$resn_exch{"H441 CTA"}="1H44 CTA"; $resn_exch{"H442 CTA"}="2H44 CTA"; $resn_exch{"H471 CTA"}="1H47 CTA";
$resn_exch{"H472 CTA"}="2H47 CTA"; $resn_exch{" H31 CTB"}="1H3  CTB"; $resn_exch{" H32 CTB"}="2H3  CTB";
$resn_exch{" H71 CTB"}="1H7  CTB"; $resn_exch{" H72 CTB"}="2H7  CTB"; $resn_exch{"HN21 CTC"}="1HN2 CTC";
$resn_exch{"HN22 CTC"}="2HN2 CTC"; $resn_exch{"H4'1 CTC"}="1H4' CTC"; $resn_exch{"H4'2 CTC"}="2H4' CTC";
$resn_exch{"H4'3 CTC"}="3H4' CTC"; $resn_exch{"H4A1 CTC"}="1H4\" CTC"; $resn_exch{"H4A2 CTC"}="2H4\" CTC";
$resn_exch{"H4A3 CTC"}="3H4\" CTC"; $resn_exch{" H51 CTC"}="1H5  CTC"; $resn_exch{" H52 CTC"}="2H5  CTC";
$resn_exch{"H6'1 CTC"}="1H6' CTC"; $resn_exch{"H6'2 CTC"}="2H6' CTC"; $resn_exch{"H6'3 CTC"}="3H6' CTC";
$resn_exch{"HN41 CTD"}="1HN4 CTD"; $resn_exch{"HN42 CTD"}="2HN4 CTD"; $resn_exch{" H1' CTD"}=" H1* CTD";
$resn_exch{" H2' CTD"}=" H2* CTD"; $resn_exch{"HO2' CTD"}="*HO2 CTD"; $resn_exch{" H3' CTD"}=" H3* CTD";
$resn_exch{" H4' CTD"}=" H4* CTD"; $resn_exch{"HO3' CTD"}="*HO3 CTD"; $resn_exch{"H5'1 CTD"}="1H5* CTD";
$resn_exch{"H5'2 CTD"}="2H5* CTD"; $resn_exch{"HO5' CTD"}="*HO5 CTD"; $resn_exch{" HN1 CTE"}="1HN  CTE";
$resn_exch{" HN2 CTE"}="2HN  CTE"; $resn_exch{" HB1 CTE"}="1HB  CTE"; $resn_exch{" HB2 CTE"}="2HB  CTE";
$resn_exch{" H71 CTG"}="1H5M CTG"; $resn_exch{" H72 CTG"}="2H5M CTG"; $resn_exch{" H73 CTG"}="3H5M CTG";
$resn_exch{" H1' CTG"}=" H1* CTG"; $resn_exch{" H2' CTG"}="1H2* CTG"; $resn_exch{"H2'' CTG"}="2H2* CTG";
$resn_exch{" H3' CTG"}=" H3* CTG"; $resn_exch{"HO3' CTG"}="H3T  CTG"; $resn_exch{" H4' CTG"}=" H4* CTG";
$resn_exch{" H5' CTG"}="1H5* CTG"; $resn_exch{"H5'' CTG"}="2H5* CTG"; $resn_exch{"HOP3 CTG"}="3HOP CTG";
$resn_exch{"HOP2 CTG"}="2HOP CTG"; $resn_exch{" H   CTH"}="1HN  CTH"; $resn_exch{" H2  CTH"}="2HN  CTH";
$resn_exch{"HG22 CTH"}="1HG2 CTH"; $resn_exch{"HG23 CTH"}="2HG2 CTH"; $resn_exch{" HG1 CTH"}=" HO1 CTH";
$resn_exch{" H31 CTL"}="1H3  CTL"; $resn_exch{" H32 CTL"}="2H3  CTL"; $resn_exch{" H41 CTL"}="1H4  CTL";
$resn_exch{" H42 CTL"}="2H4  CTL"; $resn_exch{"H121 CTL"}="1H12 CTL"; $resn_exch{"H122 CTL"}="2H12 CTL";
$resn_exch{" H5' CTN"}=" H5* CTN"; $resn_exch{"H5'1 CTN"}="1H5* CTN"; $resn_exch{"H5'2 CTN"}="2H5* CTN";
$resn_exch{" H4' CTN"}=" H4* CTN"; $resn_exch{" H1' CTN"}=" H1* CTN"; $resn_exch{"HN41 CTN"}="1HN4 CTN";
$resn_exch{"HN42 CTN"}="2HN4 CTN"; $resn_exch{" H2' CTN"}=" H2* CTN"; $resn_exch{" H3' CTN"}=" H3* CTN";
$resn_exch{"H811 CTO"}="1H81 CTO"; $resn_exch{"H812 CTO"}="2H81 CTO"; $resn_exch{"H813 CTO"}="3H81 CTO";
$resn_exch{"H611 CTO"}="1H61 CTO"; $resn_exch{"H612 CTO"}="2H61 CTO"; $resn_exch{"H821 CTO"}="1H82 CTO";
$resn_exch{"H822 CTO"}="2H82 CTO"; $resn_exch{"H823 CTO"}="3H82 CTO"; $resn_exch{"H621 CTO"}="1H62 CTO";
$resn_exch{"H622 CTO"}="2H62 CTO"; $resn_exch{"H831 CTO"}="1H83 CTO"; $resn_exch{"H832 CTO"}="2H83 CTO";
$resn_exch{"H833 CTO"}="3H83 CTO"; $resn_exch{"H631 CTO"}="1H63 CTO"; $resn_exch{"H632 CTO"}="2H63 CTO";
$resn_exch{"HN41 CTP"}="1HN4 CTP"; $resn_exch{"HN42 CTP"}="2HN4 CTP"; $resn_exch{" H1' CTP"}=" H1* CTP";
$resn_exch{" H2' CTP"}=" H2* CTP"; $resn_exch{"HO2' CTP"}="*HO2 CTP"; $resn_exch{" H3' CTP"}=" H3* CTP";
$resn_exch{" H4' CTP"}=" H4* CTP"; $resn_exch{"HO3' CTP"}="*HO3 CTP"; $resn_exch{"H5'1 CTP"}="1H5* CTP";
$resn_exch{"H5'2 CTP"}="2H5* CTP"; $resn_exch{"HOA2 CTP"}="2HOA CTP"; $resn_exch{"HOB2 CTP"}="2HOB CTP";
$resn_exch{"HOG2 CTP"}="2HOG CTP"; $resn_exch{"HOG3 CTP"}="3HOG CTP"; $resn_exch{"H6A1 CTR"}="1H6A CTR";
$resn_exch{"H6A2 CTR"}="2H6A CTR"; $resn_exch{"HO2A CTR"}="AHO2 CTR"; $resn_exch{"HO3A CTR"}="AHO3 CTR";
$resn_exch{"HO4A CTR"}="AHO4 CTR"; $resn_exch{"HO6A CTR"}="AHO6 CTR"; $resn_exch{"H6B1 CTR"}="1H6B CTR";
$resn_exch{"H6B2 CTR"}="2H6B CTR"; $resn_exch{"HO2B CTR"}="BHO2 CTR"; $resn_exch{"HO3B CTR"}="BHO3 CTR";
$resn_exch{"HO6B CTR"}="BHO6 CTR"; $resn_exch{"H6C1 CTR"}="1H6C CTR"; $resn_exch{"H6C2 CTR"}="2H6C CTR";
$resn_exch{"HO1C CTR"}="CHO1 CTR"; $resn_exch{"HO2C CTR"}="CHO2 CTR"; $resn_exch{"HO3C CTR"}="CHO3 CTR";
$resn_exch{"HO6C CTR"}="CHO6 CTR"; $resn_exch{" H11 CTS"}="1H1  CTS"; $resn_exch{" H12 CTS"}="2H1  CTS";
$resn_exch{" H71 CTS"}="1H7  CTS"; $resn_exch{" H72 CTS"}="2H7  CTS"; $resn_exch{" H81 CTS"}="1H8  CTS";
$resn_exch{" H82 CTS"}="2H8  CTS"; $resn_exch{"H6A1 CTT"}="1H6A CTT"; $resn_exch{"H6A2 CTT"}="2H6A CTT";
$resn_exch{"HOA2 CTT"}="2HOA CTT"; $resn_exch{"HOA3 CTT"}="3HOA CTT"; $resn_exch{"HOA6 CTT"}="6HOA CTT";
$resn_exch{"H6B1 CTT"}="1H6B CTT"; $resn_exch{"H6B2 CTT"}="2H6B CTT"; $resn_exch{"HOB2 CTT"}="2HOB CTT";
$resn_exch{"HOB3 CTT"}="3HOB CTT"; $resn_exch{"HOB6 CTT"}="6HOB CTT"; $resn_exch{"H6C1 CTT"}="1H6C CTT";
$resn_exch{"H6C2 CTT"}="2H6C CTT"; $resn_exch{"HOC2 CTT"}="2HOC CTT"; $resn_exch{"HOC3 CTT"}="3HOC CTT";
$resn_exch{"HOC6 CTT"}="6HOC CTT"; $resn_exch{"H6D1 CTT"}="1H6D CTT"; $resn_exch{"H6D2 CTT"}="2H6D CTT";
$resn_exch{"HOD2 CTT"}="2HOD CTT"; $resn_exch{"HOD3 CTT"}="3HOD CTT"; $resn_exch{"HOD4 CTT"}="4HOD CTT";
$resn_exch{"HOD6 CTT"}="6HOD CTT"; $resn_exch{" HA1 CTX"}="1HA  CTX"; $resn_exch{" HA2 CTX"}="2HA  CTX";
$resn_exch{" HA3 CTX"}="3HA  CTX"; $resn_exch{" HB1 CTX"}="1HB  CTX"; $resn_exch{" HB2 CTX"}="2HB  CTX";
$resn_exch{"HA11 CTX"}="1HA1 CTX"; $resn_exch{"HA12 CTX"}="2HA1 CTX"; $resn_exch{"HB11 CTX"}="1HB1 CTX";
$resn_exch{"HB12 CTX"}="2HB1 CTX"; $resn_exch{"H211 CTX"}="1H21 CTX"; $resn_exch{"H212 CTX"}="2H21 CTX";
$resn_exch{"H213 CTX"}="3H21 CTX"; $resn_exch{"H221 CTX"}="1H22 CTX"; $resn_exch{"H222 CTX"}="2H22 CTX";
$resn_exch{"H223 CTX"}="3H22 CTX"; $resn_exch{" H71 CTY"}="1H7  CTY"; $resn_exch{" H72 CTY"}="2H7  CTY";
$resn_exch{"H341 CTY"}="1H34 CTY"; $resn_exch{"H342 CTY"}="2H34 CTY"; $resn_exch{"H343 CTY"}="3H34 CTY";
$resn_exch{"H331 CTY"}="1H33 CTY"; $resn_exch{"H332 CTY"}="2H33 CTY"; $resn_exch{"H333 CTY"}="3H33 CTY";
$resn_exch{"H351 CTY"}="1H35 CTY"; $resn_exch{"H352 CTY"}="2H35 CTY"; $resn_exch{"H353 CTY"}="3H35 CTY";
$resn_exch{"H361 CTY"}="1H36 CTY"; $resn_exch{"H362 CTY"}="2H36 CTY"; $resn_exch{"H301 CTY"}="1H30 CTY";
$resn_exch{"H302 CTY"}="2H30 CTY"; $resn_exch{"H303 CTY"}="3H30 CTY"; $resn_exch{"H321 CTY"}="1H32 CTY";
$resn_exch{"H322 CTY"}="2H32 CTY"; $resn_exch{"H323 CTY"}="3H32 CTY"; $resn_exch{"H251 CTY"}="1H25 CTY";
$resn_exch{"H252 CTY"}="2H25 CTY"; $resn_exch{"H271 CTY"}="1H27 CTY"; $resn_exch{"H272 CTY"}="2H27 CTY";
$resn_exch{"H273 CTY"}="3H27 CTY"; $resn_exch{"H281 CTY"}="1H28 CTY"; $resn_exch{"H282 CTY"}="2H28 CTY";
$resn_exch{"H283 CTY"}="3H28 CTY"; $resn_exch{"H151 CTY"}="1H15 CTY"; $resn_exch{"H152 CTY"}="2H15 CTY";
$resn_exch{"H201 CTY"}="1H20 CTY"; $resn_exch{"H202 CTY"}="2H20 CTY"; $resn_exch{"H203 CTY"}="3H20 CTY";
$resn_exch{"H291 CTY"}="1H29 CTY"; $resn_exch{"H292 CTY"}="2H29 CTY"; $resn_exch{"H293 CTY"}="3H29 CTY";
$resn_exch{"H211 CTY"}="1H21 CTY"; $resn_exch{"H212 CTY"}="2H21 CTY"; $resn_exch{"H213 CTY"}="3H21 CTY";
$resn_exch{"H371 CTY"}="1H37 CTY"; $resn_exch{"H372 CTY"}="2H37 CTY"; $resn_exch{"H373 CTY"}="3H37 CTY";
$resn_exch{"H311 CTY"}="1H31 CTY"; $resn_exch{"H312 CTY"}="2H31 CTY"; $resn_exch{"H313 CTY"}="3H31 CTY";
$resn_exch{"H191 CTY"}="1H19 CTY"; $resn_exch{"H192 CTY"}="2H19 CTY"; $resn_exch{"H193 CTY"}="3H19 CTY";
$resn_exch{"H381 CTY"}="1H38 CTY"; $resn_exch{"H382 CTY"}="2H38 CTY"; $resn_exch{"H383 CTY"}="3H38 CTY";
$resn_exch{"H101 CTZ"}="1H10 CTZ"; $resn_exch{"H102 CTZ"}="2H10 CTZ"; $resn_exch{"H261 CTZ"}="1H26 CTZ";
$resn_exch{"H262 CTZ"}="2H26 CTZ"; $resn_exch{" H31 CUB"}="1H3  CUB"; $resn_exch{" H32 CUB"}="2H3  CUB";
$resn_exch{" H21 CUB"}="1H2  CUB"; $resn_exch{" H22 CUB"}="2H2  CUB"; $resn_exch{" H11 CUB"}="1H1  CUB";
$resn_exch{" H12 CUB"}="2H1  CUB"; $resn_exch{" H41 CUB"}="1H4  CUB"; $resn_exch{" H42 CUB"}="2H4  CUB";
$resn_exch{" H43 CUB"}="3H4  CUB"; $resn_exch{"H101 CUP"}="1H10 CUP"; $resn_exch{"H102 CUP"}="2H10 CUP";
$resn_exch{"H101 CUS"}="1H10 CUS"; $resn_exch{"H102 CUS"}="2H10 CUS"; $resn_exch{"H161 CUS"}="1H16 CUS";
$resn_exch{"H162 CUS"}="2H16 CUS"; $resn_exch{"H163 CUS"}="3H16 CUS"; $resn_exch{"H121 CUS"}="1H12 CUS";
$resn_exch{"H122 CUS"}="2H12 CUS"; $resn_exch{"H123 CUS"}="3H12 CUS"; $resn_exch{"HO11 CVB"}="1HO1 CVB";
$resn_exch{"HO21 CVB"}="1HO2 CVB"; $resn_exch{"HC21 CVB"}="1HC2 CVB"; $resn_exch{"HC31 CVB"}="1HC3 CVB";
$resn_exch{"HX11 CVB"}="1HX1 CVB"; $resn_exch{"HX21 CVB"}="1HX2 CVB"; $resn_exch{" H11 CVB"}="1H1  CVB";
$resn_exch{"HC51 CVB"}="1HC5 CVB"; $resn_exch{"HC61 CVB"}="1HC6 CVB"; $resn_exch{" H31 CVI"}="1H3  CVI";
$resn_exch{" H41 CVI"}="1H4  CVI"; $resn_exch{" H61 CVI"}="1H6  CVI"; $resn_exch{" H71 CVI"}="1H7  CVI";
$resn_exch{" H91 CVI"}="1H9  CVI"; $resn_exch{"H101 CVI"}="1H10 CVI"; $resn_exch{"H121 CVI"}="1H12 CVI";
$resn_exch{"H131 CVI"}="1H13 CVI"; $resn_exch{"H151 CVI"}="1H15 CVI"; $resn_exch{"H161 CVI"}="1H16 CVI";
$resn_exch{"H181 CVI"}="1H18 CVI"; $resn_exch{"H191 CVI"}="1H19 CVI"; $resn_exch{"H201 CVI"}="1H20 CVI";
$resn_exch{"H202 CVI"}="2H20 CVI"; $resn_exch{"H203 CVI"}="3H20 CVI"; $resn_exch{"H211 CVI"}="1H21 CVI";
$resn_exch{"H212 CVI"}="2H21 CVI"; $resn_exch{"H213 CVI"}="3H21 CVI"; $resn_exch{"H221 CVI"}="1H22 CVI";
$resn_exch{"H222 CVI"}="2H22 CVI"; $resn_exch{"H223 CVI"}="3H22 CVI"; $resn_exch{"H231 CVI"}="1H23 CVI";
$resn_exch{"H232 CVI"}="2H23 CVI"; $resn_exch{"H233 CVI"}="3H23 CVI"; $resn_exch{"H241 CVI"}="1H24 CVI";
$resn_exch{"H242 CVI"}="2H24 CVI"; $resn_exch{"H243 CVI"}="3H24 CVI"; $resn_exch{"H251 CVI"}="1H25 CVI";
$resn_exch{"H252 CVI"}="2H25 CVI"; $resn_exch{"H253 CVI"}="3H25 CVI"; $resn_exch{" H21 CX6"}="1H2  CX6";
$resn_exch{" H22 CX6"}="2H2  CX6"; $resn_exch{" H31 CX6"}="1H3  CX6"; $resn_exch{" H32 CX6"}="2H3  CX6";
$resn_exch{" H41 CX6"}="1H4  CX6"; $resn_exch{" H42 CX6"}="2H4  CX6"; $resn_exch{"H121 CX6"}="1H12 CX6";
$resn_exch{"H122 CX6"}="2H12 CX6"; $resn_exch{"H131 CX6"}="1H13 CX6"; $resn_exch{"H132 CX6"}="2H13 CX6";
$resn_exch{"H121 CXA"}="1H12 CXA"; $resn_exch{"H122 CXA"}="2H12 CXA"; $resn_exch{"H191 CXA"}="1H19 CXA";
$resn_exch{"H192 CXA"}="2H19 CXA"; $resn_exch{"H181 CXB"}="1H18 CXB"; $resn_exch{"H182 CXB"}="2H18 CXB";
$resn_exch{"H183 CXB"}="3H18 CXB"; $resn_exch{" H71 CXB"}="1H7  CXB"; $resn_exch{" H72 CXB"}="2H7  CXB";
$resn_exch{" H11 CXB"}="1H1  CXB"; $resn_exch{" H12 CXB"}="2H1  CXB"; $resn_exch{"H011 CXE"}="1H01 CXE";
$resn_exch{"H012 CXE"}="2H01 CXE"; $resn_exch{"H013 CXE"}="3H01 CXE"; $resn_exch{"H021 CXE"}="1H02 CXE";
$resn_exch{"H022 CXE"}="2H02 CXE"; $resn_exch{"H031 CXE"}="1H03 CXE"; $resn_exch{"H032 CXE"}="2H03 CXE";
$resn_exch{"H041 CXE"}="1H04 CXE"; $resn_exch{"H042 CXE"}="2H04 CXE"; $resn_exch{"H051 CXE"}="1H05 CXE";
$resn_exch{"H052 CXE"}="2H05 CXE"; $resn_exch{"H061 CXE"}="1H06 CXE"; $resn_exch{"H062 CXE"}="2H06 CXE";
$resn_exch{"H071 CXE"}="1H07 CXE"; $resn_exch{"H072 CXE"}="2H07 CXE"; $resn_exch{"H081 CXE"}="1H08 CXE";
$resn_exch{"H082 CXE"}="2H08 CXE"; $resn_exch{"H091 CXE"}="1H09 CXE"; $resn_exch{"H092 CXE"}="2H09 CXE";
$resn_exch{"H101 CXE"}="1H10 CXE"; $resn_exch{"H102 CXE"}="2H10 CXE"; $resn_exch{"H121 CXE"}="1H12 CXE";
$resn_exch{"H122 CXE"}="2H12 CXE"; $resn_exch{"H131 CXE"}="1H13 CXE"; $resn_exch{"H132 CXE"}="2H13 CXE";
$resn_exch{"H151 CXE"}="1H15 CXE"; $resn_exch{"H152 CXE"}="2H15 CXE"; $resn_exch{"H161 CXE"}="1H16 CXE";
$resn_exch{"H162 CXE"}="2H16 CXE"; $resn_exch{"H181 CXE"}="1H18 CXE"; $resn_exch{"H182 CXE"}="2H18 CXE";
$resn_exch{"H191 CXE"}="1H19 CXE"; $resn_exch{"H192 CXE"}="2H19 CXE"; $resn_exch{"H211 CXE"}="1H21 CXE";
$resn_exch{"H212 CXE"}="2H21 CXE"; $resn_exch{"H221 CXE"}="1H22 CXE"; $resn_exch{"H222 CXE"}="2H22 CXE";
$resn_exch{"H241 CXE"}="1H24 CXE"; $resn_exch{"H242 CXE"}="2H24 CXE"; $resn_exch{"H251 CXE"}="1H25 CXE";
$resn_exch{"H252 CXE"}="2H25 CXE"; $resn_exch{" H21 CXF"}="1H2  CXF"; $resn_exch{" H22 CXF"}="2H2  CXF";
$resn_exch{" H31 CXF"}="1H3  CXF"; $resn_exch{" H32 CXF"}="2H3  CXF"; $resn_exch{" H41 CXF"}="1H4  CXF";
$resn_exch{" H42 CXF"}="2H4  CXF"; $resn_exch{" H51 CXF"}="1H5  CXF"; $resn_exch{" H52 CXF"}="2H5  CXF";
$resn_exch{" H61 CXF"}="1H6  CXF"; $resn_exch{" H62 CXF"}="2H6  CXF"; $resn_exch{" H21 CXL"}="1H2  CXL";
$resn_exch{" H22 CXL"}="2H2  CXL"; $resn_exch{" H31 CXL"}="1H3  CXL"; $resn_exch{" H32 CXL"}="2H3  CXL";
$resn_exch{" H41 CXL"}="1H4  CXL"; $resn_exch{" H42 CXL"}="2H4  CXL"; $resn_exch{" H51 CXL"}="1H5  CXL";
$resn_exch{" H52 CXL"}="2H5  CXL"; $resn_exch{" H61 CXL"}="1H6  CXL"; $resn_exch{" H62 CXL"}="2H6  CXL";
$resn_exch{" H   CXM"}=" HN  CXM"; $resn_exch{" HB2 CXM"}="1HB  CXM"; $resn_exch{" HB3 CXM"}="2HB  CXM";
$resn_exch{" HG2 CXM"}="1HG  CXM"; $resn_exch{" HG3 CXM"}="2HG  CXM"; $resn_exch{" HE1 CXM"}="1HE  CXM";
$resn_exch{" HE2 CXM"}="2HE  CXM"; $resn_exch{" HE3 CXM"}="3HE  CXM"; $resn_exch{"H511 CXN"}="1H51 CXN";
$resn_exch{"H512 CXN"}="2H51 CXN"; $resn_exch{"H513 CXN"}="3H51 CXN"; $resn_exch{"H521 CXN"}="1H52 CXN";
$resn_exch{"H522 CXN"}="2H52 CXN"; $resn_exch{"H523 CXN"}="3H52 CXN"; $resn_exch{"H271 CXN"}="1H27 CXN";
$resn_exch{"H272 CXN"}="2H27 CXN"; $resn_exch{"H273 CXN"}="3H27 CXN"; $resn_exch{" H31 CXO"}="1H3  CXO";
$resn_exch{" H32 CXO"}="2H3  CXO"; $resn_exch{" H41 CXO"}="1H4  CXO"; $resn_exch{" H42 CXO"}="2H4  CXO";
$resn_exch{" H51 CXO"}="1H5  CXO"; $resn_exch{" H52 CXO"}="2H5  CXO"; $resn_exch{" H61 CXO"}="1H6  CXO";
$resn_exch{" H62 CXO"}="2H6  CXO"; $resn_exch{" H21 CXP"}="1H2  CXP"; $resn_exch{" H22 CXP"}="2H2  CXP";
$resn_exch{" H31 CXP"}="1H3  CXP"; $resn_exch{" H32 CXP"}="2H3  CXP"; $resn_exch{" H41 CXP"}="1H4  CXP";
$resn_exch{" H42 CXP"}="2H4  CXP"; $resn_exch{" H51 CXP"}="1H5  CXP"; $resn_exch{" H52 CXP"}="2H5  CXP";
$resn_exch{" H61 CXP"}="1H6  CXP"; $resn_exch{" H62 CXP"}="2H6  CXP"; $resn_exch{" H71 CXP"}="1H7  CXP";
$resn_exch{" H72 CXP"}="2H7  CXP"; $resn_exch{" H81 CXP"}="1H8  CXP"; $resn_exch{" H82 CXP"}="2H8  CXP";
$resn_exch{" H11 CXR"}="1H1  CXR"; $resn_exch{" H12 CXR"}="2H1  CXR"; $resn_exch{" H4' CXR"}=" H4* CXR";
$resn_exch{" H3' CXR"}=" H3* CXR"; $resn_exch{" H2' CXR"}=" H2* CXR"; $resn_exch{" H1' CXR"}=" H1* CXR";
$resn_exch{"H5'1 CXR"}="1H5* CXR"; $resn_exch{"H5'2 CXR"}="2H5* CXR"; $resn_exch{" H11 CXS"}="1H1  CXS";
$resn_exch{" H12 CXS"}="2H1  CXS"; $resn_exch{" H21 CXS"}="1H2  CXS"; $resn_exch{" H22 CXS"}="2H2  CXS";
$resn_exch{" H31 CXS"}="1H3  CXS"; $resn_exch{" H32 CXS"}="2H3  CXS"; $resn_exch{" H51 CXS"}="1H5  CXS";
$resn_exch{" H52 CXS"}="2H5  CXS"; $resn_exch{" H61 CXS"}="1H6  CXS"; $resn_exch{" H62 CXS"}="2H6  CXS";
$resn_exch{" H71 CXS"}="1H7  CXS"; $resn_exch{" H72 CXS"}="2H7  CXS"; $resn_exch{" H81 CXS"}="1H8  CXS";
$resn_exch{" H82 CXS"}="2H8  CXS"; $resn_exch{" H91 CXS"}="1H9  CXS"; $resn_exch{" H92 CXS"}="2H9  CXS";
$resn_exch{" H61 CXT"}="1H6  CXT"; $resn_exch{" H62 CXT"}="2H6  CXT"; $resn_exch{"H8C1 CXT"}="1H8C CXT";
$resn_exch{"H8C2 CXT"}="2H8C CXT"; $resn_exch{"H101 CXT"}="1H10 CXT"; $resn_exch{"H102 CXT"}="2H10 CXT";
$resn_exch{"H103 CXT"}="3H10 CXT"; $resn_exch{"H111 CXT"}="1H11 CXT"; $resn_exch{"H112 CXT"}="2H11 CXT";
$resn_exch{"H113 CXT"}="3H11 CXT"; $resn_exch{"H121 CXT"}="1H12 CXT"; $resn_exch{"H211 CXT"}="1H21 CXT";
$resn_exch{"H212 CXT"}="2H21 CXT"; $resn_exch{"H231 CXT"}="1H23 CXT"; $resn_exch{"H232 CXT"}="2H23 CXT";
$resn_exch{"H261 CXT"}="1H26 CXT"; $resn_exch{"H262 CXT"}="2H26 CXT"; $resn_exch{"H271 CXT"}="1H27 CXT";
$resn_exch{"H272 CXT"}="2H27 CXT"; $resn_exch{"H311 CXT"}="1H31 CXT"; $resn_exch{"H312 CXT"}="2H31 CXT";
$resn_exch{"H321 CXT"}="1H32 CXT"; $resn_exch{"H322 CXT"}="2H32 CXT"; $resn_exch{"H341 CXT"}="1H34 CXT";
$resn_exch{"H342 CXT"}="2H34 CXT"; $resn_exch{"H391 CXT"}="1H39 CXT"; $resn_exch{"H392 CXT"}="2H39 CXT";
$resn_exch{"H393 CXT"}="3H39 CXT"; $resn_exch{"H401 CXT"}="1H40 CXT"; $resn_exch{"H402 CXT"}="2H40 CXT";
$resn_exch{" H51 CXX"}="1H5  CXX"; $resn_exch{" H61 CXX"}="1H6  CXX"; $resn_exch{"H171 CXX"}="1H17 CXX";
$resn_exch{"H172 CXX"}="2H17 CXX"; $resn_exch{"H181 CXX"}="1H18 CXX"; $resn_exch{"H182 CXX"}="2H18 CXX";
$resn_exch{"H183 CXX"}="3H18 CXX"; $resn_exch{"H151 CXX"}="1H15 CXX"; $resn_exch{"H152 CXX"}="2H15 CXX";
$resn_exch{"H161 CXX"}="1H16 CXX"; $resn_exch{"H162 CXX"}="2H16 CXX"; $resn_exch{"H191 CXX"}="1H19 CXX";
$resn_exch{"H192 CXX"}="2H19 CXX"; $resn_exch{"H193 CXX"}="3H19 CXX"; $resn_exch{" H52 CXX"}="2H5  CXX";
$resn_exch{" H62 CXX"}="2H6  CXX"; $resn_exch{"H5'1 CXY"}="1H5* CXY"; $resn_exch{"H5'2 CXY"}="2H5* CXY";
$resn_exch{" H4D CXY"}=" H4* CXY"; $resn_exch{" H3D CXY"}=" H3* CXY"; $resn_exch{" H2D CXY"}=" H2* CXY";
$resn_exch{" H1D CXY"}=" H1* CXY"; $resn_exch{"HN41 CXY"}="1HN4 CXY"; $resn_exch{"HN42 CXY"}="2HN4 CXY";
$resn_exch{" H5D CXY"}="1H5' CXY"; $resn_exch{" H5B CXY"}="2H5' CXY"; $resn_exch{"HAO1 CY0"}="1HAO CY0";
$resn_exch{"HAO2 CY0"}="2HAO CY0"; $resn_exch{"HAN1 CY0"}="1HAN CY0"; $resn_exch{"HAN2 CY0"}="2HAN CY0";
$resn_exch{"HBC1 CY0"}="1HBC CY0"; $resn_exch{"HBC2 CY0"}="2HBC CY0"; $resn_exch{" HN1 CY0"}="1HN  CY0";
$resn_exch{" HN2 CY0"}="2HN  CY0"; $resn_exch{" H2  CY1"}=" HN1 CY1"; $resn_exch{" HB2 CY1"}=" HB1 CY1";
$resn_exch{" HB3 CY1"}=" HB2 CY1"; $resn_exch{" HD2 CY1"}=" HD1 CY1"; $resn_exch{" HD3 CY1"}=" HD2 CY1";
$resn_exch{" HE  CY1"}=" HE1 CY1"; $resn_exch{" HM1 CY1"}="1HM  CY1"; $resn_exch{" HM2 CY1"}="2HM  CY1";
$resn_exch{" HM3 CY1"}="3HM  CY1"; $resn_exch{" H2  CY3"}=" HN2 CY3"; $resn_exch{" HB2 CY3"}="1HB  CY3";
$resn_exch{" HB3 CY3"}="2HB  CY3"; $resn_exch{"HN11 CY3"}="1HN1 CY3"; $resn_exch{"HN12 CY3"}="2HN1 CY3";
$resn_exch{" H2  CY4"}=" HN2 CY4"; $resn_exch{" HB2 CY4"}="1HB  CY4"; $resn_exch{" HB3 CY4"}="2HB  CY4";
$resn_exch{"HA22 CY4"}="1HA2 CY4"; $resn_exch{"HA23 CY4"}="2HA2 CY4"; $resn_exch{"HA32 CY4"}="1HA3 CY4";
$resn_exch{"HA33 CY4"}="2HA3 CY4"; $resn_exch{"HA41 CY4"}="1HA4 CY4"; $resn_exch{"HA42 CY4"}="2HA4 CY4";
$resn_exch{"HA43 CY4"}="3HA4 CY4"; $resn_exch{" H51 CY6"}="1H5  CY6"; $resn_exch{" H52 CY6"}="2H5  CY6";
$resn_exch{" H53 CY6"}="3H5  CY6"; $resn_exch{"H161 CY6"}="1H16 CY6"; $resn_exch{"H162 CY6"}="2H16 CY6";
$resn_exch{"H163 CY6"}="3H16 CY6"; $resn_exch{"H171 CY6"}="1H17 CY6"; $resn_exch{"H172 CY6"}="2H17 CY6";
$resn_exch{"H173 CY6"}="3H17 CY6"; $resn_exch{"H201 CY6"}="1H20 CY6"; $resn_exch{"H202 CY6"}="2H20 CY6";
$resn_exch{"H221 CY6"}="1H22 CY6"; $resn_exch{"H222 CY6"}="2H22 CY6"; $resn_exch{"H251 CY6"}="1H25 CY6";
$resn_exch{"H252 CY6"}="2H25 CY6"; $resn_exch{"H253 CY6"}="3H25 CY6"; $resn_exch{"H261 CY6"}="1H26 CY6";
$resn_exch{"H262 CY6"}="2H26 CY6"; $resn_exch{"H263 CY6"}="3H26 CY6"; $resn_exch{"H381 CY6"}="1H38 CY6";
$resn_exch{"H382 CY6"}="2H38 CY6"; $resn_exch{"H391 CY6"}="1H39 CY6"; $resn_exch{"H392 CY6"}="2H39 CY6";
$resn_exch{"H393 CY6"}="3H39 CY6"; $resn_exch{"H401 CY6"}="1H40 CY6"; $resn_exch{"H402 CY6"}="2H40 CY6";
$resn_exch{"H421 CY6"}="1H42 CY6"; $resn_exch{"H422 CY6"}="2H42 CY6"; $resn_exch{"H431 CY6"}="1H43 CY6";
$resn_exch{"H432 CY6"}="2H43 CY6"; $resn_exch{"HAV1 CY7"}="1HAV CY7"; $resn_exch{"HAV2 CY7"}="2HAV CY7";
$resn_exch{"HAU1 CY7"}="1HAU CY7"; $resn_exch{"HAU2 CY7"}="2HAU CY7"; $resn_exch{"HAA1 CY7"}="1HAA CY7";
$resn_exch{"HAA2 CY7"}="2HAA CY7"; $resn_exch{"HAA3 CY7"}="3HAA CY7"; $resn_exch{"HAX1 CY7"}="1HAX CY7";
$resn_exch{"HAX2 CY7"}="2HAX CY7"; $resn_exch{"HAY1 CY7"}="1HAY CY7"; $resn_exch{"HAY2 CY7"}="2HAY CY7";
$resn_exch{"HAB1 CY7"}="1HAB CY7"; $resn_exch{"HAB2 CY7"}="2HAB CY7"; $resn_exch{"HAB3 CY7"}="3HAB CY7";
$resn_exch{"HAC1 CY7"}="1HAC CY7"; $resn_exch{"HAC2 CY7"}="2HAC CY7"; $resn_exch{"HAC3 CY7"}="3HAC CY7";
$resn_exch{"HBC1 CY7"}="1HBC CY7"; $resn_exch{"HBC2 CY7"}="2HBC CY7"; $resn_exch{" HN1 CY7"}="1HN  CY7";
$resn_exch{" HN2 CY7"}="2HN  CY7"; $resn_exch{"HMA1 CYB"}="1HMA CYB"; $resn_exch{"HMA2 CYB"}="2HMA CYB";
$resn_exch{"HMA3 CYB"}="3HMA CYB"; $resn_exch{"HAA1 CYB"}="1HAA CYB"; $resn_exch{"HAA2 CYB"}="2HAA CYB";
$resn_exch{"HBA1 CYB"}="1HBA CYB"; $resn_exch{"HBA2 CYB"}="2HBA CYB"; $resn_exch{"HHB1 CYB"}="1HHB CYB";
$resn_exch{"HHB2 CYB"}="2HHB CYB"; $resn_exch{" HB  CYB"}=" H_B CYB"; $resn_exch{"HMB1 CYB"}="1HMB CYB";
$resn_exch{"HMB2 CYB"}="2HMB CYB"; $resn_exch{"HMB3 CYB"}="3HMB CYB"; $resn_exch{"HAB1 CYB"}="1HAB CYB";
$resn_exch{"HAB2 CYB"}="2HAB CYB"; $resn_exch{"HBB1 CYB"}="1HBB CYB"; $resn_exch{"HBB2 CYB"}="2HBB CYB";
$resn_exch{"HBB3 CYB"}="3HBB CYB"; $resn_exch{" HC  CYB"}=" H_C CYB"; $resn_exch{"HAC1 CYB"}="1HAC CYB";
$resn_exch{"HAC2 CYB"}="2HAC CYB"; $resn_exch{"HBC1 CYB"}="1HBC CYB"; $resn_exch{"HBC2 CYB"}="2HBC CYB";
$resn_exch{"HBC3 CYB"}="3HBC CYB"; $resn_exch{"HHD1 CYB"}="1HHD CYB"; $resn_exch{"HHD2 CYB"}="2HHD CYB";
$resn_exch{" HD  CYB"}=" H_D CYB"; $resn_exch{"HMD1 CYB"}="1HMD CYB"; $resn_exch{"HMD2 CYB"}="2HMD CYB";
$resn_exch{"HMD3 CYB"}="3HMD CYB"; $resn_exch{"HAD1 CYB"}="1HAD CYB"; $resn_exch{"HAD2 CYB"}="2HAD CYB";
$resn_exch{"HBD1 CYB"}="1HBD CYB"; $resn_exch{"HBD2 CYB"}="2HBD CYB"; $resn_exch{"HMA1 CYC"}="1HMA CYC";
$resn_exch{"HMA2 CYC"}="2HMA CYC"; $resn_exch{"HMA3 CYC"}="3HMA CYC"; $resn_exch{"HAA1 CYC"}="1HAA CYC";
$resn_exch{"HAA2 CYC"}="2HAA CYC"; $resn_exch{"HBA1 CYC"}="1HBA CYC"; $resn_exch{"HBA2 CYC"}="2HBA CYC";
$resn_exch{" HB  CYC"}=" H_B CYC"; $resn_exch{"HMB1 CYC"}="1HMB CYC"; $resn_exch{"HMB2 CYC"}="2HMB CYC";
$resn_exch{"HMB3 CYC"}="3HMB CYC"; $resn_exch{"HAB1 CYC"}="1HAB CYC"; $resn_exch{"HAB2 CYC"}="2HAB CYC";
$resn_exch{"HBB1 CYC"}="1HBB CYC"; $resn_exch{"HBB2 CYC"}="2HBB CYC"; $resn_exch{"HBB3 CYC"}="3HBB CYC";
$resn_exch{" HC  CYC"}=" H_C CYC"; $resn_exch{"HMC1 CYC"}="1HMC CYC"; $resn_exch{"HMC2 CYC"}="2HMC CYC";
$resn_exch{"HMC3 CYC"}="3HMC CYC"; $resn_exch{"HAC1 CYC"}="1HAC CYC"; $resn_exch{"HAC2 CYC"}="2HAC CYC";
$resn_exch{"HBC1 CYC"}="1HBC CYC"; $resn_exch{"HBC2 CYC"}="2HBC CYC"; $resn_exch{"HBC3 CYC"}="3HBC CYC";
$resn_exch{" HD  CYC"}=" H_D CYC"; $resn_exch{"HMD1 CYC"}="1HMD CYC"; $resn_exch{"HMD2 CYC"}="2HMD CYC";
$resn_exch{"HMD3 CYC"}="3HMD CYC"; $resn_exch{"HAD1 CYC"}="1HAD CYC"; $resn_exch{"HAD2 CYC"}="2HAD CYC";
$resn_exch{"HBD1 CYC"}="1HBD CYC"; $resn_exch{"HBD2 CYC"}="2HBD CYC"; $resn_exch{" H2  CYD"}=" HN2 CYD";
$resn_exch{" HB2 CYD"}="1HB  CYD"; $resn_exch{" HB3 CYD"}="2HB  CYD"; $resn_exch{" H11 CYD"}="1H1  CYD";
$resn_exch{" H12 CYD"}="2H1  CYD"; $resn_exch{"HN11 CYD"}="1HN1 CYD"; $resn_exch{"HN12 CYD"}="2HN1 CYD";
$resn_exch{" H41 CYD"}="1H4  CYD"; $resn_exch{" H42 CYD"}="2H4  CYD"; $resn_exch{" H31 CYD"}="1H3  CYD";
$resn_exch{" H32 CYD"}="2H3  CYD"; $resn_exch{" H42 CYE"}="2H4  CYE"; $resn_exch{" H41 CYE"}="1H4  CYE";
$resn_exch{" H22 CYE"}="2H2  CYE"; $resn_exch{" HC2 CYF"}=" H2  CYF"; $resn_exch{"HAL3 CYF"}="1HAL CYF";
$resn_exch{"HAL2 CYF"}="2HAL CYF"; $resn_exch{" HB3 CYF"}="1HB  CYF"; $resn_exch{" HB2 CYF"}="2HB  CYF";
$resn_exch{" H   CYF"}="1HN  CYF"; $resn_exch{" H2  CYF"}="2HN  CYF"; $resn_exch{"HNT1 CYF"}="1HNT CYF";
$resn_exch{"HNT2 CYF"}="2HNT CYF"; $resn_exch{" H   CYG"}="1HN  CYG"; $resn_exch{" H2  CYG"}="2HN  CYG";
$resn_exch{" HB2 CYG"}="1HB  CYG"; $resn_exch{" HB3 CYG"}="2HB  CYG"; $resn_exch{"HN11 CYG"}="1HN1 CYG";
$resn_exch{"HN12 CYG"}="2HN1 CYG"; $resn_exch{"HB13 CYG"}="1HB1 CYG"; $resn_exch{"HB12 CYG"}="2HB1 CYG";
$resn_exch{"HG13 CYG"}="1HG1 CYG"; $resn_exch{"HG12 CYG"}="2HG1 CYG"; $resn_exch{" H21 CYH"}="1H2  CYH";
$resn_exch{" H22 CYH"}="2H2  CYH"; $resn_exch{" H31 CYH"}="1H3  CYH"; $resn_exch{" H32 CYH"}="2H3  CYH";
$resn_exch{" H41 CYH"}="1H4  CYH"; $resn_exch{" H42 CYH"}="2H4  CYH"; $resn_exch{" H51 CYH"}="1H5  CYH";
$resn_exch{" H52 CYH"}="2H5  CYH"; $resn_exch{" H61 CYH"}="1H6  CYH"; $resn_exch{" H62 CYH"}="2H6  CYH";
$resn_exch{" H11 CYI"}="1H1  CYI"; $resn_exch{" H12 CYI"}="2H1  CYI"; $resn_exch{" H21 CYI"}="1H2  CYI";
$resn_exch{" H22 CYI"}="2H2  CYI"; $resn_exch{" H31 CYI"}="1H3  CYI"; $resn_exch{" H32 CYI"}="2H3  CYI";
$resn_exch{" H51 CYI"}="1H5  CYI"; $resn_exch{" H52 CYI"}="2H5  CYI"; $resn_exch{" H61 CYI"}="1H6  CYI";
$resn_exch{" H62 CYI"}="2H6  CYI"; $resn_exch{"H101 CYJ"}="1H10 CYJ"; $resn_exch{"H102 CYJ"}="2H10 CYJ";
$resn_exch{"H9C1 CYJ"}="1H9C CYJ"; $resn_exch{"H9C2 CYJ"}="2H9C CYJ"; $resn_exch{"H7C1 CYJ"}="1H7C CYJ";
$resn_exch{"H7C2 CYJ"}="2H7C CYJ"; $resn_exch{"H5C1 CYJ"}="1H5C CYJ"; $resn_exch{"H5C2 CYJ"}="2H5C CYJ";
$resn_exch{"H2C1 CYJ"}="1H2C CYJ"; $resn_exch{"H2C2 CYJ"}="2H2C CYJ"; $resn_exch{"HEC1 CYJ"}="1HEC CYJ";
$resn_exch{"HEC2 CYJ"}="2HEC CYJ"; $resn_exch{"HDC1 CYJ"}="1HDC CYJ"; $resn_exch{"HDC2 CYJ"}="2HDC CYJ";
$resn_exch{"HGC1 CYJ"}="1HGC CYJ"; $resn_exch{"HGC2 CYJ"}="2HGC CYJ"; $resn_exch{"HBC1 CYJ"}="1HBC CYJ";
$resn_exch{"HBC2 CYJ"}="2HBC CYJ"; $resn_exch{" HN1 CYJ"}="1HN  CYJ"; $resn_exch{" HN2 CYJ"}="2HN  CYJ";
$resn_exch{" H61 CYL"}="1H6  CYL"; $resn_exch{" H62 CYL"}="2H6  CYL"; $resn_exch{"HN11 CYL"}="1HN1 CYL";
$resn_exch{"HN12 CYL"}="2HN1 CYL"; $resn_exch{" HN1 CYM"}="1HN  CYM"; $resn_exch{" HN2 CYM"}="2HN  CYM";
$resn_exch{" HB1 CYM"}="1HB  CYM"; $resn_exch{" HB2 CYM"}="2HB  CYM"; $resn_exch{" HD1 CYM"}="1HD  CYM";
$resn_exch{" HD2 CYM"}="2HD  CYM"; $resn_exch{" HD3 CYM"}="3HD  CYM"; $resn_exch{"HN11 CYP"}="1HN1 CYP";
$resn_exch{"HN12 CYP"}="2HN1 CYP"; $resn_exch{"HB11 CYP"}="1HB1 CYP"; $resn_exch{"HB12 CYP"}="2HB1 CYP";
$resn_exch{"HG11 CYP"}="1HG1 CYP"; $resn_exch{"HG12 CYP"}="2HG1 CYP"; $resn_exch{"HB21 CYP"}="1HB2 CYP";
$resn_exch{"HB22 CYP"}="2HB2 CYP"; $resn_exch{"HA31 CYP"}="1HA3 CYP"; $resn_exch{"HA32 CYP"}="2HA3 CYP";
$resn_exch{" H   CYQ"}="1HN  CYQ"; $resn_exch{" H2  CYQ"}="2HN  CYQ"; $resn_exch{" HB2 CYQ"}="1HB  CYQ";
$resn_exch{" HB3 CYQ"}="2HB  CYQ"; $resn_exch{" HD2 CYQ"}="1HD  CYQ"; $resn_exch{" HD3 CYQ"}="2HD  CYQ";
$resn_exch{"HOP2 CYQ"}="2HOP CYQ"; $resn_exch{"HOP3 CYQ"}="3HOP CYQ"; $resn_exch{" HC2 CYR"}=" H2  CYR";
$resn_exch{"HN21 CYR"}="1HN2 CYR"; $resn_exch{"HN22 CYR"}="2HN2 CYR"; $resn_exch{" H31 CYR"}="1H3  CYR";
$resn_exch{" H32 CYR"}="2H3  CYR"; $resn_exch{" H41 CYR"}="1H4  CYR"; $resn_exch{" H42 CYR"}="2H4  CYR";
$resn_exch{" H51 CYR"}="1H5  CYR"; $resn_exch{" H52 CYR"}="2H5  CYR"; $resn_exch{"HN71 CYR"}="1HN7 CYR";
$resn_exch{"HN72 CYR"}="2HN7 CYR"; $resn_exch{" HB3 CYR"}="1HB  CYR"; $resn_exch{" HB2 CYR"}="2HB  CYR";
$resn_exch{" H   CYR"}="1HN  CYR"; $resn_exch{" H2  CYR"}="2HN  CYR"; $resn_exch{" H2  CYS"}=" HN2 CYS";
$resn_exch{" HB2 CYS"}="1HB  CYS"; $resn_exch{" HB3 CYS"}="2HB  CYS"; $resn_exch{"HN41 CYT"}="1HN4 CYT";
$resn_exch{"HN42 CYT"}="2HN4 CYT"; $resn_exch{"H141 CYU"}="1H14 CYU"; $resn_exch{"H142 CYU"}="2H14 CYU";
$resn_exch{"H201 CYU"}="1H20 CYU"; $resn_exch{"H202 CYU"}="2H20 CYU"; $resn_exch{"H241 CYU"}="1H24 CYU";
$resn_exch{"H242 CYU"}="2H24 CYU"; $resn_exch{"H311 CYU"}="1H31 CYU"; $resn_exch{"H312 CYU"}="2H31 CYU";
$resn_exch{"H321 CYU"}="1H32 CYU"; $resn_exch{"H322 CYU"}="2H32 CYU"; $resn_exch{"H411 CYU"}="1H41 CYU";
$resn_exch{"H412 CYU"}="2H41 CYU"; $resn_exch{"H413 CYU"}="3H41 CYU"; $resn_exch{"H421 CYU"}="1H42 CYU";
$resn_exch{"H422 CYU"}="2H42 CYU"; $resn_exch{"H423 CYU"}="3H42 CYU"; $resn_exch{"H431 CYU"}="1H43 CYU";
$resn_exch{"H432 CYU"}="2H43 CYU"; $resn_exch{"H433 CYU"}="3H43 CYU"; $resn_exch{"H441 CYU"}="1H44 CYU";
$resn_exch{"H442 CYU"}="2H44 CYU"; $resn_exch{"H443 CYU"}="3H44 CYU"; $resn_exch{"H451 CYU"}="1H45 CYU";
$resn_exch{"H452 CYU"}="2H45 CYU"; $resn_exch{"H453 CYU"}="3H45 CYU"; $resn_exch{"H461 CYU"}="1H46 CYU";
$resn_exch{"H462 CYU"}="2H46 CYU"; $resn_exch{"H463 CYU"}="3H46 CYU"; $resn_exch{"H471 CYU"}="1H47 CYU";
$resn_exch{"H472 CYU"}="2H47 CYU"; $resn_exch{"H473 CYU"}="3H47 CYU"; $resn_exch{"H481 CYU"}="1H48 CYU";
$resn_exch{"H482 CYU"}="2H48 CYU"; $resn_exch{"H483 CYU"}="3H48 CYU"; $resn_exch{"H491 CYU"}="1H49 CYU";
$resn_exch{"H492 CYU"}="2H49 CYU"; $resn_exch{"H493 CYU"}="3H49 CYU"; $resn_exch{"H501 CYU"}="1H50 CYU";
$resn_exch{"H502 CYU"}="2H50 CYU"; $resn_exch{"H503 CYU"}="3H50 CYU"; $resn_exch{"H371 CYU"}="1H37 CYU";
$resn_exch{"H372 CYU"}="2H37 CYU"; $resn_exch{"H381 CYU"}="1H38 CYU"; $resn_exch{"H382 CYU"}="2H38 CYU";
$resn_exch{"H383 CYU"}="3H38 CYU"; $resn_exch{"H391 CYU"}="1H39 CYU"; $resn_exch{"H392 CYU"}="2H39 CYU";
$resn_exch{"H393 CYU"}="3H39 CYU"; $resn_exch{"H401 CYU"}="1H40 CYU"; $resn_exch{"H402 CYU"}="2H40 CYU";
$resn_exch{"H403 CYU"}="3H40 CYU"; $resn_exch{"H491 CYV"}="1H49 CYV"; $resn_exch{"H492 CYV"}="2H49 CYV";
$resn_exch{"H493 CYV"}="3H49 CYV"; $resn_exch{"H481 CYV"}="1H48 CYV"; $resn_exch{"H482 CYV"}="2H48 CYV";
$resn_exch{"H483 CYV"}="3H48 CYV"; $resn_exch{"H471 CYV"}="1H47 CYV"; $resn_exch{"H472 CYV"}="2H47 CYV";
$resn_exch{"H473 CYV"}="3H47 CYV"; $resn_exch{"H401 CYV"}="1H40 CYV"; $resn_exch{"H402 CYV"}="2H40 CYV";
$resn_exch{"H351 CYV"}="1H35 CYV"; $resn_exch{"H352 CYV"}="2H35 CYV"; $resn_exch{"H353 CYV"}="3H35 CYV";
$resn_exch{"H341 CYV"}="1H34 CYV"; $resn_exch{"H342 CYV"}="2H34 CYV"; $resn_exch{"H343 CYV"}="3H34 CYV";
$resn_exch{"H291 CYV"}="1H29 CYV"; $resn_exch{"H292 CYV"}="2H29 CYV"; $resn_exch{"H231 CYV"}="1H23 CYV";
$resn_exch{"H232 CYV"}="2H23 CYV"; $resn_exch{"H261 CYV"}="1H26 CYV"; $resn_exch{"H262 CYV"}="2H26 CYV";
$resn_exch{"H263 CYV"}="3H26 CYV"; $resn_exch{"H271 CYV"}="1H27 CYV"; $resn_exch{"H272 CYV"}="2H27 CYV";
$resn_exch{"H273 CYV"}="3H27 CYV"; $resn_exch{" H51 CYV"}="1H5  CYV"; $resn_exch{" H52 CYV"}="2H5  CYV";
$resn_exch{" H61 CYV"}="1H6  CYV"; $resn_exch{" H62 CYV"}="2H6  CYV"; $resn_exch{" H91 CYV"}="1H9  CYV";
$resn_exch{" H92 CYV"}="2H9  CYV"; $resn_exch{"H101 CYV"}="1H10 CYV"; $resn_exch{"H102 CYV"}="2H10 CYV";
$resn_exch{"H103 CYV"}="3H10 CYV"; $resn_exch{"H191 CYV"}="1H19 CYV"; $resn_exch{"H192 CYV"}="2H19 CYV";
$resn_exch{"H141 CYV"}="1H14 CYV"; $resn_exch{"H142 CYV"}="2H14 CYV"; $resn_exch{"H151 CYV"}="1H15 CYV";
$resn_exch{"H152 CYV"}="2H15 CYV"; $resn_exch{" H11 CYX"}="1H1  CYX"; $resn_exch{" H41 CYX"}="1H4  CYX";
$resn_exch{" H42 CYX"}="2H4  CYX"; $resn_exch{" H51 CYX"}="1H5  CYX"; $resn_exch{" H52 CYX"}="2H5  CYX";
$resn_exch{" H61 CYX"}="1H6  CYX"; $resn_exch{" H62 CYX"}="2H6  CYX"; $resn_exch{" H81 CYX"}="1H8  CYX";
$resn_exch{" H91 CYX"}="1H9  CYX"; $resn_exch{" H21 CYY"}="1H2  CYY"; $resn_exch{" H22 CYY"}="2H2  CYY";
$resn_exch{"HN11 CYY"}="1HN1 CYY"; $resn_exch{"HN12 CYY"}="2HN1 CYY"; $resn_exch{"HN31 CYY"}="1HN3 CYY";
$resn_exch{"HN32 CYY"}="2HN3 CYY"; $resn_exch{"H11A CYZ"}="1H1  CYZ"; $resn_exch{" H21 CYZ"}="1H2  CYZ";
$resn_exch{"H31A CYZ"}="1H3  CYZ"; $resn_exch{" H41 CYZ"}="1H4  CYZ"; $resn_exch{" H51 CYZ"}="1H5  CYZ";
$resn_exch{" H61 CYZ"}="1H6  CYZ"; $resn_exch{" H62 CYZ"}="2H6  CYZ"; $resn_exch{" H71 CYZ"}="1H7  CYZ";
$resn_exch{" H72 CYZ"}="2H7  CYZ"; $resn_exch{" H81 CYZ"}="1H8  CYZ"; $resn_exch{" H2  CZ2"}=" HN2 CZ2";
$resn_exch{" H   CZ2"}=" HN1 CZ2"; $resn_exch{" HB2 CZ2"}="1HCB CZ2"; $resn_exch{" HB3 CZ2"}="2HCB CZ2";
$resn_exch{" H11 CZA"}="1H1  CZA"; $resn_exch{" H12 CZA"}="2H1  CZA"; $resn_exch{" H13 CZA"}="3H1  CZA";
$resn_exch{"H161 CZA"}="1H16 CZA"; $resn_exch{"H121 CZA"}="1H12 CZA"; $resn_exch{"H122 CZA"}="2H12 CZA";
$resn_exch{"H101 CZA"}="1H10 CZA"; $resn_exch{"H102 CZA"}="2H10 CZA"; $resn_exch{"H103 CZA"}="3H10 CZA";
$resn_exch{"H111 CZA"}="1H11 CZA"; $resn_exch{"H112 CZA"}="2H11 CZA"; $resn_exch{"H113 CZA"}="3H11 CZA";
$resn_exch{"H261 CZB"}="1H26 CZB"; $resn_exch{"H262 CZB"}="2H26 CZB"; $resn_exch{"H101 CZB"}="1H10 CZB";
$resn_exch{"H102 CZB"}="2H10 CZB"; $resn_exch{"H101 CZH"}="1H10 CZH"; $resn_exch{"H102 CZH"}="2H10 CZH";
$resn_exch{"H261 CZH"}="1H26 CZH"; $resn_exch{"H262 CZH"}="2H26 CZH"; $resn_exch{"H101 CZI"}="1H10 CZI";
$resn_exch{"H102 CZI"}="2H10 CZI"; $resn_exch{"H261 CZI"}="1H26 CZI"; $resn_exch{"H262 CZI"}="2H26 CZI";
$resn_exch{"HB71 CZM"}="1HB7 CZM"; $resn_exch{"HB72 CZM"}="2HB7 CZM"; $resn_exch{"HB73 CZM"}="3HB7 CZM";
$resn_exch{"HA71 CZM"}="1HA7 CZM"; $resn_exch{"HA72 CZM"}="2HA7 CZM"; $resn_exch{"HA73 CZM"}="3HA7 CZM";
$resn_exch{"H291 CZN"}="1H29 CZN"; $resn_exch{"H292 CZN"}="2H29 CZN"; $resn_exch{"H101 CZN"}="1H10 CZN";
$resn_exch{"H102 CZN"}="2H10 CZN"; $resn_exch{"H261 CZP"}="1H26 CZP"; $resn_exch{"H262 CZP"}="2H26 CZP";
$resn_exch{"H281 CZP"}="1H28 CZP"; $resn_exch{"H282 CZP"}="2H28 CZP"; $resn_exch{"H291 CZP"}="1H29 CZP";
$resn_exch{"H292 CZP"}="2H29 CZP"; $resn_exch{"H301 CZP"}="1H30 CZP"; $resn_exch{"H302 CZP"}="2H30 CZP";
$resn_exch{"H311 CZP"}="1H31 CZP"; $resn_exch{"H312 CZP"}="2H31 CZP"; $resn_exch{"H101 CZP"}="1H10 CZP";
$resn_exch{"H102 CZP"}="2H10 CZP"; $resn_exch{" H   CZZ"}=" HN1 CZZ"; $resn_exch{" H2  CZZ"}=" HN2 CZZ";
$resn_exch{"HCB1 CZZ"}="1HCB CZZ"; $resn_exch{"HCB2 CZZ"}="2HCB CZZ"; $resn_exch{" H91 D01"}="1H9  D01";
$resn_exch{" H92 D01"}="2H9  D01"; $resn_exch{"H101 D01"}="1H10 D01"; $resn_exch{"H102 D01"}="2H10 D01";
$resn_exch{"H191 D01"}="1H19 D01"; $resn_exch{"H192 D01"}="2H19 D01"; $resn_exch{"H201 D01"}="1H20 D01";
$resn_exch{"H202 D01"}="2H20 D01"; $resn_exch{" H11 D05"}="1H1  D05"; $resn_exch{" H12 D05"}="2H1  D05";
$resn_exch{" H21 D05"}="1H2  D05"; $resn_exch{"H22A D05"}="2H2  D05"; $resn_exch{"H23A D05"}="3H2  D05";
$resn_exch{"H201 D05"}="1H20 D05"; $resn_exch{"H202 D05"}="2H20 D05"; $resn_exch{" H11 D10"}="1H1  D10";
$resn_exch{" H12 D10"}="2H1  D10"; $resn_exch{" H13 D10"}="3H1  D10"; $resn_exch{" H21 D10"}="1H2  D10";
$resn_exch{" H22 D10"}="2H2  D10"; $resn_exch{" H31 D10"}="1H3  D10"; $resn_exch{" H32 D10"}="2H3  D10";
$resn_exch{" H41 D10"}="1H4  D10"; $resn_exch{" H42 D10"}="2H4  D10"; $resn_exch{" H51 D10"}="1H5  D10";
$resn_exch{" H52 D10"}="2H5  D10"; $resn_exch{" H61 D10"}="1H6  D10"; $resn_exch{" H62 D10"}="2H6  D10";
$resn_exch{" H71 D10"}="1H7  D10"; $resn_exch{" H72 D10"}="2H7  D10"; $resn_exch{" H81 D10"}="1H8  D10";
$resn_exch{" H82 D10"}="2H8  D10"; $resn_exch{" H91 D10"}="1H9  D10"; $resn_exch{" H92 D10"}="2H9  D10";
$resn_exch{"H101 D10"}="1H10 D10"; $resn_exch{"H102 D10"}="2H10 D10"; $resn_exch{"H103 D10"}="3H10 D10";
$resn_exch{" H   D11"}="1HN  D11"; $resn_exch{" H2  D11"}="2HN  D11"; $resn_exch{"HG21 D11"}="1HG2 D11";
$resn_exch{"HG22 D11"}="2HG2 D11"; $resn_exch{"HG23 D11"}="3HG2 D11"; $resn_exch{" HXT D11"}="HOXT D11";
$resn_exch{" H11 D12"}="1H1  D12"; $resn_exch{" H12 D12"}="2H1  D12"; $resn_exch{" H13 D12"}="3H1  D12";
$resn_exch{" H21 D12"}="1H2  D12"; $resn_exch{" H22 D12"}="2H2  D12"; $resn_exch{" H31 D12"}="1H3  D12";
$resn_exch{" H32 D12"}="2H3  D12"; $resn_exch{" H41 D12"}="1H4  D12"; $resn_exch{" H42 D12"}="2H4  D12";
$resn_exch{" H51 D12"}="1H5  D12"; $resn_exch{" H52 D12"}="2H5  D12"; $resn_exch{" H61 D12"}="1H6  D12";
$resn_exch{" H62 D12"}="2H6  D12"; $resn_exch{" H71 D12"}="1H7  D12"; $resn_exch{" H72 D12"}="2H7  D12";
$resn_exch{" H81 D12"}="1H8  D12"; $resn_exch{" H82 D12"}="2H8  D12"; $resn_exch{" H91 D12"}="1H9  D12";
$resn_exch{" H92 D12"}="2H9  D12"; $resn_exch{"H101 D12"}="1H10 D12"; $resn_exch{"H102 D12"}="2H10 D12";
$resn_exch{"H111 D12"}="1H11 D12"; $resn_exch{"H112 D12"}="2H11 D12"; $resn_exch{"H121 D12"}="1H12 D12";
$resn_exch{"H122 D12"}="2H12 D12"; $resn_exch{"H123 D12"}="3H12 D12"; $resn_exch{"H251 D13"}="1H25 D13";
$resn_exch{"H252 D13"}="2H25 D13"; $resn_exch{"H253 D13"}="3H25 D13"; $resn_exch{"HM21 D16"}="1HM2 D16";
$resn_exch{"HM22 D16"}="2HM2 D16"; $resn_exch{"HM23 D16"}="3HM2 D16"; $resn_exch{" H91 D16"}="1H9  D16";
$resn_exch{" H92 D16"}="2H9  D16"; $resn_exch{" HB1 D16"}="1HB  D16"; $resn_exch{" HB2 D16"}="2HB  D16";
$resn_exch{" HG1 D16"}="1HG  D16"; $resn_exch{" HG2 D16"}="2HG  D16"; $resn_exch{"HOE2 D16"}="2HOE D16";
$resn_exch{"HP11 D16"}="1HP1 D16"; $resn_exch{"HP12 D16"}="2HP1 D16"; $resn_exch{"HP13 D16"}="3HP1 D16";
$resn_exch{"H101 D18"}="1H10 D18"; $resn_exch{"H102 D18"}="2H10 D18"; $resn_exch{"H103 D18"}="3H10 D18";
$resn_exch{" H91 D18"}="1H9  D18"; $resn_exch{" H92 D18"}="2H9  D18"; $resn_exch{" H93 D18"}="3H9  D18";
$resn_exch{"HN11 D18"}="1HN1 D18"; $resn_exch{"HN12 D18"}="2HN1 D18"; $resn_exch{"HN'1 D18"}="1HN' D18";
$resn_exch{"HN'2 D18"}="2HN' D18"; $resn_exch{"HN2' D18"}="'HN2 D18"; $resn_exch{"H9'1 D18"}="1H9' D18";
$resn_exch{"H9'2 D18"}="2H9' D18"; $resn_exch{"H9'3 D18"}="3H9' D18"; $resn_exch{"H1'1 D18"}="1H1' D18";
$resn_exch{"H1'2 D18"}="2H1' D18"; $resn_exch{"H1'3 D18"}="3H1' D18"; $resn_exch{"H101 D19"}="1H10 D19";
$resn_exch{"H102 D19"}="2H10 D19"; $resn_exch{" H91 D19"}="1H9  D19"; $resn_exch{" H92 D19"}="2H9  D19";
$resn_exch{"HN11 D19"}="1HN1 D19"; $resn_exch{"HN12 D19"}="2HN1 D19"; $resn_exch{"HN'1 D19"}="1HN' D19";
$resn_exch{"HN'2 D19"}="2HN' D19"; $resn_exch{"HN2' D19"}="'HN2 D19"; $resn_exch{"H9'1 D19"}="1H9' D19";
$resn_exch{"H9'2 D19"}="2H9' D19"; $resn_exch{"H1'1 D19"}="1H1' D19"; $resn_exch{"H1'2 D19"}="2H1' D19";
$resn_exch{"H221 D1B"}="1H22 D1B"; $resn_exch{"H222 D1B"}="2H22 D1B"; $resn_exch{"H261 D1B"}="1H26 D1B";
$resn_exch{"H262 D1B"}="2H26 D1B"; $resn_exch{"H1C1 D1D"}="1H1C D1D"; $resn_exch{"H1C2 D1D"}="2H1C D1D";
$resn_exch{"H4C1 D1D"}="1H4C D1D"; $resn_exch{"H4C2 D1D"}="2H4C D1D"; $resn_exch{"H101 D1G"}="1H10 D1G";
$resn_exch{"H102 D1G"}="2H10 D1G"; $resn_exch{"H111 D1G"}="1H11 D1G"; $resn_exch{"H112 D1G"}="2H11 D1G";
$resn_exch{"H113 D1G"}="3H11 D1G"; $resn_exch{"H121 D1G"}="1H12 D1G"; $resn_exch{"H122 D1G"}="2H12 D1G";
$resn_exch{"H123 D1G"}="3H12 D1G"; $resn_exch{"HAA1 D1H"}="1HAA D1H"; $resn_exch{"HAA2 D1H"}="2HAA D1H";
$resn_exch{"HAA3 D1H"}="3HAA D1H"; $resn_exch{"HAC1 D1H"}="1HAC D1H"; $resn_exch{"HAC2 D1H"}="2HAC D1H";
$resn_exch{"HAC3 D1H"}="3HAC D1H"; $resn_exch{"HAK1 D1H"}="1HAK D1H"; $resn_exch{"HAK2 D1H"}="2HAK D1H";
$resn_exch{"HAL1 D1H"}="1HAL D1H"; $resn_exch{"HAL2 D1H"}="2HAL D1H"; $resn_exch{"HAB1 D1H"}="1HAB D1H";
$resn_exch{"HAB2 D1H"}="2HAB D1H"; $resn_exch{"HAB3 D1H"}="3HAB D1H"; $resn_exch{"HAD1 D1H"}="1HAD D1H";
$resn_exch{"HAD2 D1H"}="2HAD D1H"; $resn_exch{"HAD3 D1H"}="3HAD D1H"; $resn_exch{"H2A1 D1L"}="1H2A D1L";
$resn_exch{"H2A2 D1L"}="2H2A D1L"; $resn_exch{"H2A3 D1L"}="3H2A D1L"; $resn_exch{"HOP3 D1P"}="3HOP D1P";
$resn_exch{"HOP2 D1P"}="2HOP D1P"; $resn_exch{" H5' D1P"}="1H5* D1P"; $resn_exch{"H5'' D1P"}="2H5* D1P";
$resn_exch{" H4' D1P"}=" H4* D1P"; $resn_exch{" H3' D1P"}=" H3* D1P"; $resn_exch{"HO'3 D1P"}="3HO* D1P";
$resn_exch{" H2' D1P"}="1H2* D1P"; $resn_exch{"H2'' D1P"}="2H2* D1P"; $resn_exch{" H1' D1P"}=" H1* D1P";
$resn_exch{"HO1' D1P"}="HO1* D1P"; $resn_exch{"H391 D1R"}="1H39 D1R"; $resn_exch{"H392 D1R"}="2H39 D1R";
$resn_exch{"H401 D1R"}="1H40 D1R"; $resn_exch{"H402 D1R"}="2H40 D1R"; $resn_exch{"H331 D1R"}="1H33 D1R";
$resn_exch{"H332 D1R"}="2H33 D1R"; $resn_exch{"H333 D1R"}="3H33 D1R"; $resn_exch{"H491 D1R"}="1H49 D1R";
$resn_exch{"H492 D1R"}="2H49 D1R"; $resn_exch{"H481 D1R"}="1H48 D1R"; $resn_exch{"H482 D1R"}="2H48 D1R";
$resn_exch{"H101 D1R"}="1H10 D1R"; $resn_exch{"H102 D1R"}="2H10 D1R"; $resn_exch{"H191 D1R"}="1H19 D1R";
$resn_exch{"H192 D1R"}="2H19 D1R"; $resn_exch{"H201 D1R"}="1H20 D1R"; $resn_exch{"H202 D1R"}="2H20 D1R";
$resn_exch{"H271 D1R"}="1H27 D1R"; $resn_exch{"H272 D1R"}="2H27 D1R"; $resn_exch{"H281 D1R"}="1H28 D1R";
$resn_exch{"H282 D1R"}="2H28 D1R"; $resn_exch{"H101 D1T"}="1H10 D1T"; $resn_exch{"H102 D1T"}="2H10 D1T";
$resn_exch{"H103 D1T"}="3H10 D1T"; $resn_exch{" H91 D1T"}="1H9  D1T"; $resn_exch{" H92 D1T"}="2H9  D1T";
$resn_exch{" H81 D1T"}="1H8  D1T"; $resn_exch{" H82 D1T"}="2H8  D1T"; $resn_exch{" H71 D1T"}="1H7  D1T";
$resn_exch{" H72 D1T"}="2H7  D1T"; $resn_exch{" H61 D1T"}="1H6  D1T"; $resn_exch{" H62 D1T"}="2H6  D1T";
$resn_exch{" H51 D1T"}="1H5  D1T"; $resn_exch{" H52 D1T"}="2H5  D1T"; $resn_exch{" H41 D1T"}="1H4  D1T";
$resn_exch{" H42 D1T"}="2H4  D1T"; $resn_exch{" H31 D1T"}="1H3  D1T"; $resn_exch{" H32 D1T"}="2H3  D1T";
$resn_exch{" H21 D1T"}="1H2  D1T"; $resn_exch{" H22 D1T"}="2H2  D1T"; $resn_exch{" H11 D1T"}="1H1  D1T";
$resn_exch{" H12 D1T"}="2H1  D1T"; $resn_exch{" HN1 D20"}="1HN  D20"; $resn_exch{" HN2 D20"}="2HN  D20";
$resn_exch{"H2C1 D20"}="1H2C D20"; $resn_exch{"H2C2 D20"}="2H2C D20"; $resn_exch{"H3C1 D20"}="1H3C D20";
$resn_exch{"H3C2 D20"}="2H3C D20"; $resn_exch{"H4C1 D20"}="1H4C D20"; $resn_exch{"H4C2 D20"}="2H4C D20";
$resn_exch{"H7C1 D20"}="1H7C D20"; $resn_exch{"H7C2 D20"}="2H7C D20"; $resn_exch{"H101 D20"}="1H10 D20";
$resn_exch{"H102 D20"}="2H10 D20"; $resn_exch{"H8C1 D20"}="1H8C D20"; $resn_exch{"H8C2 D20"}="2H8C D20";
$resn_exch{"H8C3 D20"}="3H8C D20"; $resn_exch{"HN71 D23"}="1HN7 D23"; $resn_exch{"HN72 D23"}="2HN7 D23";
$resn_exch{"H201 D23"}="1H20 D23"; $resn_exch{"H202 D23"}="2H20 D23"; $resn_exch{"H203 D23"}="3H20 D23";
$resn_exch{"H211 D23"}="1H21 D23"; $resn_exch{"H212 D23"}="2H21 D23"; $resn_exch{"H213 D23"}="3H21 D23";
$resn_exch{"H221 D23"}="1H22 D23"; $resn_exch{"H222 D23"}="2H22 D23"; $resn_exch{"H223 D23"}="3H22 D23";
$resn_exch{"H11A D24"}="1H1  D24"; $resn_exch{"H12A D24"}="2H1  D24"; $resn_exch{"H1'1 D24"}="1H1' D24";
$resn_exch{"H1'2 D24"}="2H1' D24"; $resn_exch{"H211 D27"}="1H21 D27"; $resn_exch{"H212 D27"}="2H21 D27";
$resn_exch{"H213 D27"}="3H21 D27"; $resn_exch{"H221 D27"}="1H22 D27"; $resn_exch{"H222 D27"}="2H22 D27";
$resn_exch{"H223 D27"}="3H22 D27"; $resn_exch{"H131 D28"}="1H13 D28"; $resn_exch{"H132 D28"}="2H13 D28";
$resn_exch{"H133 D28"}="3H13 D28"; $resn_exch{"H231 D2A"}="1H23 D2A"; $resn_exch{"H232 D2A"}="2H23 D2A";
$resn_exch{"H261 D2A"}="1H26 D2A"; $resn_exch{"H262 D2A"}="2H26 D2A"; $resn_exch{"H11A D2C"}="1H1  D2C";
$resn_exch{"H12A D2C"}="2H1  D2C"; $resn_exch{" H13 D2C"}="3H1  D2C"; $resn_exch{" H21 D2C"}="1H2  D2C";
$resn_exch{" H22 D2C"}="2H2  D2C"; $resn_exch{" H23 D2C"}="3H2  D2C"; $resn_exch{" H61 D2C"}="1H6  D2C";
$resn_exch{" H62 D2C"}="2H6  D2C"; $resn_exch{"HN21 D2C"}="1HN2 D2C"; $resn_exch{"HN22 D2C"}="2HN2 D2C";
$resn_exch{"H101 D2G"}="1H10 D2G"; $resn_exch{"H102 D2G"}="2H10 D2G"; $resn_exch{"H111 D2G"}="1H11 D2G";
$resn_exch{"H112 D2G"}="2H11 D2G"; $resn_exch{"H113 D2G"}="3H11 D2G"; $resn_exch{"H2C1 D2P"}="1H2C D2P";
$resn_exch{"H2C2 D2P"}="2H2C D2P"; $resn_exch{"H4C1 D2P"}="1H4C D2P"; $resn_exch{"H4C2 D2P"}="2H4C D2P";
$resn_exch{"HOP2  D3"}="2HOP  D3"; $resn_exch{"HOP3  D3"}="3HOP  D3"; $resn_exch{" H5'  D3"}="1H5*  D3";
$resn_exch{"H5''  D3"}="2H5*  D3"; $resn_exch{" H4'  D3"}=" H4*  D3"; $resn_exch{" H3'  D3"}=" H3*  D3";
$resn_exch{"HO3'  D3"}="*HO3  D3"; $resn_exch{" H2'  D3"}="1H2*  D3"; $resn_exch{"H2''  D3"}="2H2*  D3";
$resn_exch{" H1'  D3"}=" H1*  D3"; $resn_exch{"HN11 D31"}="1HN1 D31"; $resn_exch{"HN12 D31"}="2HN1 D31";
$resn_exch{" H21 D31"}="1H2  D31"; $resn_exch{" H22 D31"}="2H2  D31"; $resn_exch{" H61 D31"}="1H6  D31";
$resn_exch{" H62 D31"}="2H6  D31"; $resn_exch{" H71 D31"}="1H7  D31"; $resn_exch{" H72 D31"}="2H7  D31";
$resn_exch{" H91 D31"}="1H9  D31"; $resn_exch{" H92 D31"}="2H9  D31"; $resn_exch{"H101 D31"}="1H10 D31";
$resn_exch{"H102 D31"}="2H10 D31"; $resn_exch{"H201 D31"}="1H20 D31"; $resn_exch{"H202 D31"}="2H20 D31";
$resn_exch{"H371 D31"}="1H37 D31"; $resn_exch{"H372 D31"}="2H37 D31"; $resn_exch{"H381 D31"}="1H38 D31";
$resn_exch{"H382 D31"}="2H38 D31"; $resn_exch{"H391 D31"}="1H39 D31"; $resn_exch{"H392 D31"}="2H39 D31";
$resn_exch{"H401 D31"}="1H40 D31"; $resn_exch{"H402 D31"}="2H40 D31"; $resn_exch{"H411 D31"}="1H41 D31";
$resn_exch{"H412 D31"}="2H41 D31"; $resn_exch{" H77 D34"}="7H7  D34"; $resn_exch{" HN1 D34"}="1HN  D34";
$resn_exch{"HN2A D34"}="2HN  D34"; $resn_exch{" H91 D34"}="1H9  D34"; $resn_exch{" H92 D34"}="2H9  D34";
$resn_exch{" H01 D34"}="1H0  D34"; $resn_exch{" H02 D34"}="2H0  D34"; $resn_exch{" H11 D34"}="1H1  D34";
$resn_exch{" H12 D34"}="2H1  D34"; $resn_exch{" H7B D34"}="'H7' D34"; $resn_exch{"HN'1 D34"}="1HN' D34";
$resn_exch{"HN'2 D34"}="2HN' D34"; $resn_exch{"H9'1 D34"}="1H9' D34"; $resn_exch{"H9'2 D34"}="2H9' D34";
$resn_exch{"H0'1 D34"}="1H0' D34"; $resn_exch{"H0'2 D34"}="2H0' D34"; $resn_exch{"H1'1 D34"}="1H1' D34";
$resn_exch{"H1'2 D34"}="2H1' D34"; $resn_exch{" H77 D35"}="7H7  D35"; $resn_exch{" HN1 D35"}="1HN  D35";
$resn_exch{"HN2A D35"}="2HN  D35"; $resn_exch{" H91 D35"}="1H9  D35"; $resn_exch{" H92 D35"}="2H9  D35";
$resn_exch{" H01 D35"}="1H0  D35"; $resn_exch{" H02 D35"}="2H0  D35"; $resn_exch{" H11 D35"}="1H1  D35";
$resn_exch{" H12 D35"}="2H1  D35"; $resn_exch{" H21 D35"}="1H2  D35"; $resn_exch{" H22 D35"}="2H2  D35";
$resn_exch{" H31 D35"}="1H3  D35"; $resn_exch{" H32 D35"}="2H3  D35"; $resn_exch{" H7B D35"}="'H7' D35";
$resn_exch{"HN'1 D35"}="1HN' D35"; $resn_exch{"HN'2 D35"}="2HN' D35"; $resn_exch{"H9'1 D35"}="1H9' D35";
$resn_exch{"H9'2 D35"}="2H9' D35"; $resn_exch{"H0'1 D35"}="1H0' D35"; $resn_exch{"H0'2 D35"}="2H0' D35";
$resn_exch{"H1'1 D35"}="1H1' D35"; $resn_exch{"H1'2 D35"}="2H1' D35"; $resn_exch{"H2'1 D35"}="1H2' D35";
$resn_exch{"H2'2 D35"}="2H2' D35"; $resn_exch{"H3'1 D35"}="1H3' D35"; $resn_exch{"H3'2 D35"}="2H3' D35";
$resn_exch{" H71 D3F"}="1H7  D3F"; $resn_exch{" H72 D3F"}="2H7  D3F"; $resn_exch{" H73 D3F"}="3H7  D3F";
$resn_exch{"H101 D3G"}="1H10 D3G"; $resn_exch{"H102 D3G"}="2H10 D3G"; $resn_exch{"H_21 D3G"}="1H_2 D3G";
$resn_exch{"H_22 D3G"}="2H_2 D3G"; $resn_exch{" HN1 D3P"}="1HN  D3P"; $resn_exch{" HN2 D3P"}="2HN  D3P";
$resn_exch{"HOA2 D3T"}="2HOA D3T"; $resn_exch{"H5'1 D3T"}="1H5* D3T"; $resn_exch{"H5'2 D3T"}="2H5* D3T";
$resn_exch{" H4' D3T"}=" H4* D3T"; $resn_exch{" H1' D3T"}=" H1* D3T"; $resn_exch{"H5M1 D3T"}="1H5M D3T";
$resn_exch{"H5M2 D3T"}="2H5M D3T"; $resn_exch{"H5M3 D3T"}="3H5M D3T"; $resn_exch{"H2'1 D3T"}="1H2* D3T";
$resn_exch{"H2'2 D3T"}="2H2* D3T"; $resn_exch{"H3'1 D3T"}="1H3* D3T"; $resn_exch{"H3'2 D3T"}="2H3* D3T";
$resn_exch{"HOB2 D3T"}="2HOB D3T"; $resn_exch{"HOG3 D3T"}="3HOG D3T"; $resn_exch{" H11 D42"}="1H1  D42";
$resn_exch{" H12 D42"}="2H1  D42"; $resn_exch{"H21A D42"}="1H2  D42"; $resn_exch{" H22 D42"}="2H2  D42";
$resn_exch{" H41 D42"}="1H4  D42"; $resn_exch{" H42 D42"}="2H4  D42"; $resn_exch{" H51 D42"}="1H5  D42";
$resn_exch{" H52 D42"}="2H5  D42"; $resn_exch{"H151 D42"}="1H15 D42"; $resn_exch{"H152 D42"}="2H15 D42";
$resn_exch{"H153 D42"}="3H15 D42"; $resn_exch{"H421 D42"}="1H42 D42"; $resn_exch{"H422 D42"}="2H42 D42";
$resn_exch{"H423 D42"}="3H42 D42"; $resn_exch{"H461 D42"}="1H46 D42"; $resn_exch{"H462 D42"}="2H46 D42";
$resn_exch{"H463 D42"}="3H46 D42"; $resn_exch{"H2AO D4D"}="OH2A D4D"; $resn_exch{"H1BO D4D"}="OH1B D4D";
$resn_exch{"H3BO D4D"}="OH3B D4D"; $resn_exch{"H5'1 D4D"}="1H5' D4D"; $resn_exch{"H5'2 D4D"}="2H5' D4D";
$resn_exch{"H4'C D4D"}="CH4' D4D"; $resn_exch{"H3'C D4D"}="CH3' D4D"; $resn_exch{"H2'C D4D"}="CH2' D4D";
$resn_exch{"H1'C D4D"}="CH1' D4D"; $resn_exch{"H5M1 D4D"}="1H5M D4D"; $resn_exch{"H5M2 D4D"}="2H5M D4D";
$resn_exch{"H5M3 D4D"}="3H5M D4D"; $resn_exch{" H66 D4D"}="6H6  D4D"; $resn_exch{"HOP2 D4M"}="2HOP D4M";
$resn_exch{"HOP3 D4M"}="3HOP D4M"; $resn_exch{" H5' D4M"}="1H5* D4M"; $resn_exch{"H5'' D4M"}="2H5* D4M";
$resn_exch{" H4' D4M"}=" H4* D4M"; $resn_exch{" H3' D4M"}=" H3* D4M"; $resn_exch{" H2' D4M"}=" H2* D4M";
$resn_exch{" H1' D4M"}=" H1* D4M"; $resn_exch{" H71 D4M"}="1H5M D4M"; $resn_exch{" H72 D4M"}="2H5M D4M";
$resn_exch{" H73 D4M"}="3H5M D4M"; $resn_exch{" HN1 D4P"}="1HN  D4P"; $resn_exch{" HN2 D4P"}="2HN  D4P";
$resn_exch{"H1CO D4T"}="OH1C D4T"; $resn_exch{"H2CO D4T"}="OH2C D4T"; $resn_exch{"H2BO D4T"}="OH2B D4T";
$resn_exch{"H2AO D4T"}="OH2A D4T"; $resn_exch{"H5'1 D4T"}="1H5' D4T"; $resn_exch{"H5'2 D4T"}="2H5' D4T";
$resn_exch{"H4'C D4T"}="CH4' D4T"; $resn_exch{"H1'C D4T"}="CH1' D4T"; $resn_exch{" H66 D4T"}="6H6  D4T";
$resn_exch{"HCA1 D4T"}="1HCA D4T"; $resn_exch{"HCA2 D4T"}="2HCA D4T"; $resn_exch{"HCA3 D4T"}="3HCA D4T";
$resn_exch{"H2'C D4T"}="CH2' D4T"; $resn_exch{"H3'C D4T"}="CH3' D4T"; $resn_exch{"H2C1 D55"}="1H2C D55";
$resn_exch{"H2C2 D55"}="2H2C D55"; $resn_exch{"H2C3 D55"}="3H2C D55"; $resn_exch{"H3C1 D55"}="1H3C D55";
$resn_exch{"H3C2 D55"}="2H3C D55"; $resn_exch{"H5C1 D55"}="1H5C D55"; $resn_exch{"H5C2 D55"}="2H5C D55";
$resn_exch{"H6C1 D55"}="1H6C D55"; $resn_exch{"H6C2 D55"}="2H6C D55"; $resn_exch{"H6C3 D55"}="3H6C D55";
$resn_exch{"H191 D55"}="1H19 D55"; $resn_exch{"H192 D55"}="2H19 D55"; $resn_exch{"H193 D55"}="3H19 D55";
$resn_exch{"H221 D55"}="1H22 D55"; $resn_exch{"H222 D55"}="2H22 D55"; $resn_exch{"H231 D55"}="1H23 D55";
$resn_exch{"H232 D55"}="2H23 D55"; $resn_exch{"H241 D55"}="1H24 D55"; $resn_exch{"H242 D55"}="2H24 D55";
$resn_exch{"H291 D55"}="1H29 D55"; $resn_exch{"H292 D55"}="2H29 D55"; $resn_exch{"H293 D55"}="3H29 D55";
$resn_exch{"H2C1 D56"}="1H2C D56"; $resn_exch{"H2C2 D56"}="2H2C D56"; $resn_exch{"H6C1 D56"}="1H6C D56";
$resn_exch{"H6C2 D56"}="2H6C D56"; $resn_exch{"H3C1 D56"}="1H3C D56"; $resn_exch{"H3C2 D56"}="2H3C D56";
$resn_exch{"H4C1 D56"}="1H4C D56"; $resn_exch{"H4C2 D56"}="2H4C D56"; $resn_exch{"H5C1 D56"}="1H5C D56";
$resn_exch{"H5C2 D56"}="2H5C D56"; $resn_exch{"H191 D56"}="1H19 D56"; $resn_exch{"H192 D56"}="2H19 D56";
$resn_exch{"H193 D56"}="3H19 D56"; $resn_exch{"H101 D56"}="1H10 D56"; $resn_exch{"H102 D56"}="2H10 D56";
$resn_exch{"H103 D56"}="3H10 D56"; $resn_exch{"H131 D56"}="1H13 D56"; $resn_exch{"H132 D56"}="2H13 D56";
$resn_exch{"H221 D56"}="1H22 D56"; $resn_exch{"H222 D56"}="2H22 D56"; $resn_exch{"H231 D56"}="1H23 D56";
$resn_exch{"H232 D56"}="2H23 D56"; $resn_exch{"H241 D56"}="1H24 D56"; $resn_exch{"H242 D56"}="2H24 D56";
$resn_exch{"H291 D56"}="1H29 D56"; $resn_exch{"H292 D56"}="2H29 D56"; $resn_exch{"H293 D56"}="3H29 D56";
$resn_exch{" HB1 D5G"}="1HB  D5G"; $resn_exch{" HB2 D5G"}="2HB  D5G"; $resn_exch{"HD21 D5G"}="1HD2 D5G";
$resn_exch{"HD22 D5G"}="2HD2 D5G"; $resn_exch{"HD23 D5G"}="3HD2 D5G"; $resn_exch{"HD11 D5G"}="1HD1 D5G";
$resn_exch{"HD12 D5G"}="2HD1 D5G"; $resn_exch{"HD13 D5G"}="3HD1 D5G"; $resn_exch{"HAX1 D5G"}="1HAX D5G";
$resn_exch{"HAX2 D5G"}="2HAX D5G"; $resn_exch{"HAN1 D5G"}="1HAN D5G"; $resn_exch{"HAN2 D5G"}="2HAN D5G";
$resn_exch{"H5'1 D5M"}="1H5* D5M"; $resn_exch{"H5'2 D5M"}="2H5* D5M"; $resn_exch{" H4' D5M"}=" H4* D5M";
$resn_exch{" H3' D5M"}=" H3* D5M"; $resn_exch{"H2'1 D5M"}="1H2* D5M"; $resn_exch{"H2'2 D5M"}="2H2* D5M";
$resn_exch{" H1' D5M"}=" H1* D5M"; $resn_exch{"HN61 D5M"}="1HN6 D5M"; $resn_exch{"HN62 D5M"}="2HN6 D5M";
$resn_exch{" H21 D6G"}="1H2  D6G"; $resn_exch{" H22 D6G"}="2H2  D6G"; $resn_exch{" H61 D6G"}="1H6  D6G";
$resn_exch{" H62 D6G"}="2H6  D6G"; $resn_exch{"HOP2 D6G"}="2HOP D6G"; $resn_exch{"HOP3 D6G"}="3HOP D6G";
$resn_exch{" H11 D6P"}="1H1  D6P"; $resn_exch{" H12 D6P"}="2H1  D6P"; $resn_exch{" H21 D6P"}="1H2  D6P";
$resn_exch{" H22 D6P"}="2H2  D6P"; $resn_exch{"HZ71 D76"}="1HZ7 D76"; $resn_exch{"HZ72 D76"}="2HZ7 D76";
$resn_exch{"HZ73 D76"}="3HZ7 D76"; $resn_exch{"HZ31 D76"}="1HZ3 D76"; $resn_exch{"HZ32 D76"}="2HZ3 D76";
$resn_exch{"HZ41 D76"}="1HZ4 D76"; $resn_exch{"HZ42 D76"}="2HZ4 D76"; $resn_exch{"HZ51 D76"}="1HZ5 D76";
$resn_exch{"HZ52 D76"}="2HZ5 D76"; $resn_exch{"HP31 D76"}="1HP3 D76"; $resn_exch{"HP32 D76"}="2HP3 D76";
$resn_exch{"HP41 D76"}="1HP4 D76"; $resn_exch{"HP42 D76"}="2HP4 D76"; $resn_exch{"HC21 D76"}="1HC2 D76";
$resn_exch{"HC22 D76"}="2HC2 D76"; $resn_exch{"HP11 D76"}="1HP1 D76"; $resn_exch{"HP12 D76"}="2HP1 D76";
$resn_exch{" H41 D7G"}="1H4  D7G"; $resn_exch{" H42 D7G"}="2H4  D7G"; $resn_exch{" H51 D7G"}="1H5  D7G";
$resn_exch{" H52 D7G"}="2H5  D7G"; $resn_exch{"H101 D7G"}="1H10 D7G"; $resn_exch{"H102 D7G"}="2H10 D7G";
$resn_exch{"H111 D7G"}="1H11 D7G"; $resn_exch{"H112 D7G"}="2H11 D7G"; $resn_exch{"H121 D7G"}="1H12 D7G";
$resn_exch{"H122 D7G"}="2H12 D7G"; $resn_exch{"H131 D7G"}="1H13 D7G"; $resn_exch{"H132 D7G"}="2H13 D7G";
$resn_exch{"H141 D7G"}="1H14 D7G"; $resn_exch{"H142 D7G"}="2H14 D7G"; $resn_exch{"H143 D7G"}="3H14 D7G";
$resn_exch{"H151 D7G"}="1H15 D7G"; $resn_exch{"H152 D7G"}="2H15 D7G"; $resn_exch{"H171 D7G"}="1H17 D7G";
$resn_exch{"H172 D7G"}="2H17 D7G"; $resn_exch{"H173 D7G"}="3H17 D7G"; $resn_exch{"H181 D7G"}="1H18 D7G";
$resn_exch{"H182 D7G"}="2H18 D7G"; $resn_exch{"H183 D7G"}="3H18 D7G"; $resn_exch{"H191 D7G"}="1H19 D7G";
$resn_exch{"H192 D7G"}="2H19 D7G"; $resn_exch{"H1'1 D7P"}="1H1' D7P"; $resn_exch{"H1'2 D7P"}="2H1' D7P";
$resn_exch{"HN'1 D7P"}="1HN' D7P"; $resn_exch{"HN'2 D7P"}="2HN' D7P"; $resn_exch{"HB'1 D7P"}="1HB' D7P";
$resn_exch{"HB'2 D7P"}="2HB' D7P"; $resn_exch{" HB1 D7P"}="1HB  D7P"; $resn_exch{" HB2 D7P"}="2HB  D7P";
$resn_exch{" HG1 D7P"}="1HG  D7P"; $resn_exch{" HG2 D7P"}="2HG  D7P"; $resn_exch{"HD1A D7P"}="1HD  D7P";
$resn_exch{"HD2A D7P"}="2HD  D7P"; $resn_exch{"H191 D91"}="1H19 D91"; $resn_exch{"H192 D91"}="2H19 D91";
$resn_exch{"H161 D91"}="1H16 D91"; $resn_exch{"H162 D91"}="2H16 D91"; $resn_exch{"H151 D91"}="1H15 D91";
$resn_exch{"H152 D91"}="2H15 D91"; $resn_exch{" H71 D91"}="1H7  D91"; $resn_exch{" H72 D91"}="2H7  D91";
$resn_exch{"H111 D91"}="1H11 D91"; $resn_exch{"H112 D91"}="2H11 D91"; $resn_exch{"H113 D91"}="3H11 D91";
$resn_exch{"H121 D91"}="1H12 D91"; $resn_exch{"H122 D91"}="2H12 D91"; $resn_exch{"H123 D91"}="3H12 D91";
$resn_exch{" H61 D92"}="1H6  D92"; $resn_exch{" H62 D92"}="2H6  D92"; $resn_exch{" H51 D92"}="1H5  D92";
$resn_exch{" H52 D92"}="2H5  D92"; $resn_exch{" H41 D92"}="1H4  D92"; $resn_exch{" H42 D92"}="2H4  D92";
$resn_exch{" H31 D92"}="1H3  D92"; $resn_exch{" H32 D92"}="2H3  D92"; $resn_exch{"H261 D92"}="1H26 D92";
$resn_exch{"H262 D92"}="2H26 D92"; $resn_exch{"H301 D92"}="1H30 D92"; $resn_exch{"H302 D92"}="2H30 D92";
$resn_exch{"H303 D92"}="3H30 D92"; $resn_exch{"H281 D92"}="1H28 D92"; $resn_exch{"H282 D92"}="2H28 D92";
$resn_exch{"H291 D92"}="1H29 D92"; $resn_exch{"H292 D92"}="2H29 D92"; $resn_exch{" H61 D93"}="1H6  D93";
$resn_exch{" H62 D93"}="2H6  D93"; $resn_exch{" H51 D93"}="1H5  D93"; $resn_exch{" H52 D93"}="2H5  D93";
$resn_exch{" H41 D93"}="1H4  D93"; $resn_exch{" H42 D93"}="2H4  D93"; $resn_exch{" H31 D93"}="1H3  D93";
$resn_exch{" H32 D93"}="2H3  D93"; $resn_exch{"H261 D93"}="1H26 D93"; $resn_exch{"H262 D93"}="2H26 D93";
$resn_exch{"H301 D93"}="1H30 D93"; $resn_exch{"H302 D93"}="2H30 D93"; $resn_exch{"H303 D93"}="3H30 D93";
$resn_exch{"H281 D93"}="1H28 D93"; $resn_exch{"H282 D93"}="2H28 D93"; $resn_exch{"H291 D93"}="1H29 D93";
$resn_exch{"H292 D93"}="2H29 D93"; $resn_exch{"HAC1 D9G"}="1HAC D9G"; $resn_exch{"HAC2 D9G"}="2HAC D9G";
$resn_exch{"HAC3 D9G"}="3HAC D9G"; $resn_exch{"HAD1 D9G"}="1HAD D9G"; $resn_exch{"HAD2 D9G"}="2HAD D9G";
$resn_exch{"HAE1 D9G"}="1HAE D9G"; $resn_exch{"HAE2 D9G"}="2HAE D9G"; $resn_exch{"HAF1 D9G"}="1HAF D9G";
$resn_exch{"HAF2 D9G"}="2HAF D9G"; $resn_exch{"HAG1 D9G"}="1HAG D9G"; $resn_exch{"HAG2 D9G"}="2HAG D9G";
$resn_exch{"HAH1 D9G"}="1HAH D9G"; $resn_exch{"HAH2 D9G"}="2HAH D9G"; $resn_exch{"HAI1 D9G"}="1HAI D9G";
$resn_exch{"HAI2 D9G"}="2HAI D9G"; $resn_exch{"HAJ1 D9G"}="1HAJ D9G"; $resn_exch{"HAJ2 D9G"}="2HAJ D9G";
$resn_exch{"HAK1 D9G"}="1HAK D9G"; $resn_exch{"HAK2 D9G"}="2HAK D9G"; $resn_exch{"HAL1 D9G"}="1HAL D9G";
$resn_exch{"HAL2 D9G"}="2HAL D9G"; $resn_exch{"HAM1 D9G"}="1HAM D9G"; $resn_exch{"HAM2 D9G"}="2HAM D9G";
$resn_exch{"HAN1 D9G"}="1HAN D9G"; $resn_exch{"HAN2 D9G"}="2HAN D9G"; $resn_exch{"HAS1 D9G"}="1HAS D9G";
$resn_exch{"HAS2 D9G"}="2HAS D9G"; $resn_exch{"HAS3 D9G"}="3HAS D9G"; $resn_exch{"HAA1 D9G"}="1HAA D9G";
$resn_exch{"HAA2 D9G"}="2HAA D9G"; $resn_exch{"HAA3 D9G"}="3HAA D9G"; $resn_exch{"HAP1 D9G"}="1HAP D9G";
$resn_exch{"HAP2 D9G"}="2HAP D9G"; $resn_exch{"HOP3  DA"}="3HOP  DA"; $resn_exch{"HOP2  DA"}="2HOP  DA";
$resn_exch{" H5'  DA"}="1H5*  DA"; $resn_exch{"H5''  DA"}="2H5*  DA"; $resn_exch{" H4'  DA"}=" H4*  DA";
$resn_exch{" H3'  DA"}=" H3*  DA"; $resn_exch{"HO3'  DA"}="H3T   DA"; $resn_exch{" H2'  DA"}="1H2*  DA";
$resn_exch{"H2''  DA"}="2H2*  DA"; $resn_exch{" H1'  DA"}=" H1*  DA"; $resn_exch{" H61  DA"}="1H6   DA";
$resn_exch{" H62  DA"}="2H6   DA"; $resn_exch{"HN21 DA1"}="1HN2 DA1"; $resn_exch{"HN22 DA1"}="2HN2 DA1";
$resn_exch{"HN31 DA1"}="1HN3 DA1"; $resn_exch{"HN32 DA1"}="2HN3 DA1"; $resn_exch{"HC11 DA2"}="1HC1 DA2";
$resn_exch{"HC12 DA2"}="2HC1 DA2"; $resn_exch{"HC13 DA2"}="3HC1 DA2"; $resn_exch{"HC21 DA2"}="1HC2 DA2";
$resn_exch{"HC22 DA2"}="2HC2 DA2"; $resn_exch{"HC23 DA2"}="3HC2 DA2"; $resn_exch{" HN1 DA2"}="1HN  DA2";
$resn_exch{" HN2 DA2"}="2HN  DA2"; $resn_exch{"HCB1 DA2"}="1HCB DA2"; $resn_exch{"HCB2 DA2"}="2HCB DA2";
$resn_exch{"HCG1 DA2"}="1HCG DA2"; $resn_exch{"HCG2 DA2"}="2HCG DA2"; $resn_exch{"HCD1 DA2"}="1HCD DA2";
$resn_exch{"HCD2 DA2"}="2HCD DA2"; $resn_exch{"HN11 DA3"}="1HN1 DA3"; $resn_exch{"HN12 DA3"}="2HN1 DA3";
$resn_exch{" H31 DA3"}="1H3  DA3"; $resn_exch{" H32 DA3"}="2H3  DA3"; $resn_exch{" H51 DA3"}="1H5  DA3";
$resn_exch{" H52 DA3"}="2H5  DA3"; $resn_exch{"HN91 DA5"}="1HN9 DA5"; $resn_exch{"HN92 DA5"}="2HN9 DA5";
$resn_exch{"HN93 DA5"}="3HN9 DA5"; $resn_exch{"HND1 DA5"}="1HND DA5"; $resn_exch{"HD21 DA5"}="1HD2 DA5";
$resn_exch{"HD22 DA5"}="2HD2 DA5"; $resn_exch{"HD31 DA5"}="1HD3 DA5"; $resn_exch{"HD32 DA5"}="2HD3 DA5";
$resn_exch{"HD71 DA5"}="1HD7 DA5"; $resn_exch{"HD72 DA5"}="2HD7 DA5"; $resn_exch{"HD73 DA5"}="3HD7 DA5";
$resn_exch{"HD81 DA5"}="1HD8 DA5"; $resn_exch{"HD82 DA5"}="2HD8 DA5"; $resn_exch{"HD83 DA5"}="3HD8 DA5";
$resn_exch{"HN91 DA6"}="1HN9 DA6"; $resn_exch{"HN92 DA6"}="2HN9 DA6"; $resn_exch{"HN93 DA6"}="3HN9 DA6";
$resn_exch{"HND1 DA6"}="1HND DA6"; $resn_exch{"HD21 DA6"}="1HD2 DA6"; $resn_exch{"HD22 DA6"}="2HD2 DA6";
$resn_exch{"HD31 DA6"}="1HD3 DA6"; $resn_exch{"HD32 DA6"}="2HD3 DA6"; $resn_exch{"HD71 DA6"}="1HD7 DA6";
$resn_exch{"HD72 DA6"}="2HD7 DA6"; $resn_exch{"HD73 DA6"}="3HD7 DA6"; $resn_exch{"HD81 DA6"}="1HD8 DA6";
$resn_exch{"HD82 DA6"}="2HD8 DA6"; $resn_exch{"HD83 DA6"}="3HD8 DA6"; $resn_exch{"HN91 DA7"}="1HN9 DA7";
$resn_exch{"HN92 DA7"}="2HN9 DA7"; $resn_exch{"H171 DA7"}="1H17 DA7"; $resn_exch{"H172 DA7"}="2H17 DA7";
$resn_exch{"H181 DA7"}="1H18 DA7"; $resn_exch{"H182 DA7"}="2H18 DA7"; $resn_exch{"H201 DA7"}="1H20 DA7";
$resn_exch{"H202 DA7"}="2H20 DA7"; $resn_exch{"H203 DA7"}="3H20 DA7"; $resn_exch{"H211 DA7"}="1H21 DA7";
$resn_exch{"H212 DA7"}="2H21 DA7"; $resn_exch{"H213 DA7"}="3H21 DA7"; $resn_exch{"HCA1 DAA"}="1HCA DAA";
$resn_exch{"HCA2 DAA"}="2HCA DAA"; $resn_exch{"HCB1 DAA"}="1HCB DAA"; $resn_exch{"HCB2 DAA"}="2HCB DAA";
$resn_exch{"HCG1 DAA"}="1HCG DAA"; $resn_exch{"HCG2 DAA"}="2HCG DAA"; $resn_exch{"HCD1 DAA"}="1HCD DAA";
$resn_exch{"HCD2 DAA"}="2HCD DAA"; $resn_exch{"HCE1 DAA"}="1HCE DAA"; $resn_exch{"HCE2 DAA"}="2HCE DAA";
$resn_exch{"HCS1 DAA"}="1HCS DAA"; $resn_exch{"HCS2 DAA"}="2HCS DAA"; $resn_exch{"HCS3 DAA"}="3HCS DAA";
$resn_exch{"HN21 DAA"}="1HN2 DAA"; $resn_exch{"HN22 DAA"}="2HN2 DAA"; $resn_exch{" H   DAB"}="1HN  DAB";
$resn_exch{" H2  DAB"}="2HN  DAB"; $resn_exch{" HB2 DAB"}="1HB  DAB"; $resn_exch{" HB3 DAB"}="2HB  DAB";
$resn_exch{" HG2 DAB"}="1HG  DAB"; $resn_exch{" HG3 DAB"}="2HG  DAB"; $resn_exch{" HD1 DAB"}="1HD  DAB";
$resn_exch{" HD2 DAB"}="2HD  DAB"; $resn_exch{" H41 DAC"}="1H4  DAC"; $resn_exch{" H42 DAC"}="2H4  DAC";
$resn_exch{" H51 DAC"}="1H5  DAC"; $resn_exch{" H52 DAC"}="2H5  DAC"; $resn_exch{" H61 DAC"}="1H6  DAC";
$resn_exch{" H62 DAC"}="2H6  DAC"; $resn_exch{" H71 DAC"}="1H7  DAC"; $resn_exch{" H72 DAC"}="2H7  DAC";
$resn_exch{" H81 DAC"}="1H8  DAC"; $resn_exch{" H82 DAC"}="2H8  DAC"; $resn_exch{" H91 DAC"}="1H9  DAC";
$resn_exch{" H92 DAC"}="2H9  DAC"; $resn_exch{"H101 DAC"}="1H10 DAC"; $resn_exch{"H102 DAC"}="2H10 DAC";
$resn_exch{"H103 DAC"}="3H10 DAC"; $resn_exch{"H1'1 DAC"}="1H1' DAC"; $resn_exch{"H1'2 DAC"}="2H1' DAC";
$resn_exch{"H2'1 DAC"}="1H2' DAC"; $resn_exch{"H2'2 DAC"}="2H2' DAC"; $resn_exch{"H5'1 DAC"}="1H5' DAC";
$resn_exch{"H5'2 DAC"}="2H5' DAC"; $resn_exch{"H5'3 DAC"}="3H5' DAC"; $resn_exch{"HOG2 DAD"}="2HOG DAD";
$resn_exch{"HOG3 DAD"}="3HOG DAD"; $resn_exch{"HOB2 DAD"}="2HOB DAD"; $resn_exch{"HOA2 DAD"}="2HOA DAD";
$resn_exch{"H5'1 DAD"}="1H5* DAD"; $resn_exch{"H5'2 DAD"}="2H5* DAD"; $resn_exch{" H4' DAD"}=" H4* DAD";
$resn_exch{"H3'1 DAD"}="1H3* DAD"; $resn_exch{"H3'2 DAD"}="2H3* DAD"; $resn_exch{"H2'1 DAD"}="1H2* DAD";
$resn_exch{"H2'2 DAD"}="2H2* DAD"; $resn_exch{" H1' DAD"}=" H1* DAD"; $resn_exch{"HN61 DAD"}="1HN6 DAD";
$resn_exch{"HN62 DAD"}="2HN6 DAD"; $resn_exch{"HOB2 DAE"}="2HOB DAE"; $resn_exch{"HOA2 DAE"}="2HOA DAE";
$resn_exch{"HNA1 DAE"}="1HNA DAE"; $resn_exch{"HA21 DAE"}="1HA2 DAE"; $resn_exch{"HA22 DAE"}="2HA2 DAE";
$resn_exch{"HA11 DAE"}="1HA1 DAE"; $resn_exch{"HA12 DAE"}="2HA1 DAE"; $resn_exch{" H11 DAF"}="1H1  DAF";
$resn_exch{" H12 DAF"}="2H1  DAF"; $resn_exch{" H61 DAF"}="1H6  DAF"; $resn_exch{" H62 DAF"}="2H6  DAF";
$resn_exch{" H63 DAF"}="3H6  DAF"; $resn_exch{"H7H1 DAF"}="1H7H DAF"; $resn_exch{"H7H2 DAF"}="2H7H DAF";
$resn_exch{"HO2' DAF"}="'HO2 DAF"; $resn_exch{"HO3' DAF"}="'HO3 DAF"; $resn_exch{"HO4' DAF"}="'HO4 DAF";
$resn_exch{" H61 DAG"}="1H6  DAG"; $resn_exch{" H62 DAG"}="2H6  DAG"; $resn_exch{" H63 DAG"}="3H6  DAG";
$resn_exch{"HN41 DAG"}="1HN4 DAG"; $resn_exch{"HN42 DAG"}="2HN4 DAG"; $resn_exch{" H2  DAH"}=" HN2 DAH";
$resn_exch{" HB2 DAH"}="1HB  DAH"; $resn_exch{" HB3 DAH"}="2HB  DAH"; $resn_exch{" HE2 DAH"}=" HOE DAH";
$resn_exch{" HZ  DAH"}=" HOZ DAH"; $resn_exch{" H51 DAI"}="1H5  DAI"; $resn_exch{" H52 DAI"}="2H5  DAI";
$resn_exch{"H211 DAI"}="1H21 DAI"; $resn_exch{"H212 DAI"}="2H21 DAI"; $resn_exch{"H201 DAI"}="1H20 DAI";
$resn_exch{"H202 DAI"}="2H20 DAI"; $resn_exch{" H31 DAI"}="1H3  DAI"; $resn_exch{" H32 DAI"}="2H3  DAI";
$resn_exch{" H21 DAI"}="1H2  DAI"; $resn_exch{" H22 DAI"}="2H2  DAI"; $resn_exch{" H11 DAI"}="1H1  DAI";
$resn_exch{" H12 DAI"}="2H1  DAI"; $resn_exch{"H13A DAI"}="3H1  DAI"; $resn_exch{" H11 DAJ"}="1H1  DAJ";
$resn_exch{" H12 DAJ"}="2H1  DAJ"; $resn_exch{" HN1 DAJ"}="1HN  DAJ"; $resn_exch{" HN2 DAJ"}="2HN  DAJ";
$resn_exch{" HN3 DAJ"}="3HN  DAJ"; $resn_exch{" H31 DAJ"}="1H3  DAJ"; $resn_exch{" H32 DAJ"}="2H3  DAJ";
$resn_exch{"H131 DAJ"}="1H13 DAJ"; $resn_exch{"H132 DAJ"}="2H13 DAJ"; $resn_exch{"H133 DAJ"}="3H13 DAJ";
$resn_exch{" HN4 DAJ"}="4HN  DAJ"; $resn_exch{" HN5 DAJ"}="5HN  DAJ"; $resn_exch{"H101 DAJ"}="1H10 DAJ";
$resn_exch{"H102 DAJ"}="2H10 DAJ"; $resn_exch{"H103 DAJ"}="3H10 DAJ"; $resn_exch{"H22A DAK"}="2H2  DAK";
$resn_exch{"HN61 DAK"}="1HN6 DAK"; $resn_exch{"HN62 DAK"}="2HN6 DAK"; $resn_exch{" H81 DAK"}="1H8  DAK";
$resn_exch{"H1'1 DAK"}="1H1' DAK"; $resn_exch{"H2'1 DAK"}="1H2' DAK"; $resn_exch{"HO21 DAK"}="1HO2 DAK";
$resn_exch{"H3'1 DAK"}="1H3' DAK"; $resn_exch{"H4'1 DAK"}="1H4' DAK"; $resn_exch{"H5'1 DAK"}="1H5' DAK";
$resn_exch{"H5'2 DAK"}="2H5' DAK"; $resn_exch{"HP91 DAK"}="1HP9 DAK"; $resn_exch{"HP92 DAK"}="2HP9 DAK";
$resn_exch{"HPA1 DAK"}="1HPA DAK"; $resn_exch{"HPA2 DAK"}="2HPA DAK"; $resn_exch{"HPA3 DAK"}="3HPA DAK";
$resn_exch{"HPB1 DAK"}="1HPB DAK"; $resn_exch{"HPB2 DAK"}="2HPB DAK"; $resn_exch{"HPB3 DAK"}="3HPB DAK";
$resn_exch{"HP71 DAK"}="1HP7 DAK"; $resn_exch{"HO31 DAK"}="1HO3 DAK"; $resn_exch{"HN21 DAK"}="1HN2 DAK";
$resn_exch{"HP51 DAK"}="1HP5 DAK"; $resn_exch{"HP52 DAK"}="2HP5 DAK"; $resn_exch{"HP41 DAK"}="1HP4 DAK";
$resn_exch{"HP42 DAK"}="2HP4 DAK"; $resn_exch{"HN11 DAK"}="1HN1 DAK"; $resn_exch{"HP21 DAK"}="1HP2 DAK";
$resn_exch{"HP22 DAK"}="2HP2 DAK"; $resn_exch{"HP11 DAK"}="1HP1 DAK"; $resn_exch{"HP12 DAK"}="2HP1 DAK";
$resn_exch{"HD61 DAK"}="1HD6 DAK"; $resn_exch{"HD51 DAK"}="1HD5 DAK"; $resn_exch{"HD91 DAK"}="1HD9 DAK";
$resn_exch{"HD81 DAK"}="1HD8 DAK"; $resn_exch{"HDA1 DAK"}="1HDA DAK"; $resn_exch{"HDA2 DAK"}="2HDA DAK";
$resn_exch{"HDA3 DAK"}="3HDA DAK"; $resn_exch{"HDB1 DAK"}="1HDB DAK"; $resn_exch{"HDB2 DAK"}="2HDB DAK";
$resn_exch{"HDB3 DAK"}="3HDB DAK"; $resn_exch{"HD31 DAK"}="1HD3 DAK"; $resn_exch{"HD21 DAK"}="1HD2 DAK";
$resn_exch{" H2  DAL"}=" HN2 DAL"; $resn_exch{" HB1 DAL"}="1HB  DAL"; $resn_exch{" HB2 DAL"}="2HB  DAL";
$resn_exch{" HB3 DAL"}="3HB  DAL"; $resn_exch{" H   DAM"}=" HN  DAM"; $resn_exch{" HM1 DAM"}="1HM  DAM";
$resn_exch{" HM2 DAM"}="2HM  DAM"; $resn_exch{" HM3 DAM"}="3HM  DAM"; $resn_exch{" HB1 DAM"}="1HB  DAM";
$resn_exch{" HB2 DAM"}="2HB  DAM"; $resn_exch{" H91 DAN"}="1H9  DAN"; $resn_exch{" H92 DAN"}="2H9  DAN";
$resn_exch{"H111 DAN"}="1H11 DAN"; $resn_exch{"H112 DAN"}="2H11 DAN"; $resn_exch{"H113 DAN"}="3H11 DAN";
$resn_exch{"HOB1 DAN"}="1HOB DAN"; $resn_exch{" H21 DAO"}="1H2  DAO"; $resn_exch{" H22 DAO"}="2H2  DAO";
$resn_exch{" H31 DAO"}="1H3  DAO"; $resn_exch{" H32 DAO"}="2H3  DAO"; $resn_exch{" H41 DAO"}="1H4  DAO";
$resn_exch{" H42 DAO"}="2H4  DAO"; $resn_exch{" H51 DAO"}="1H5  DAO"; $resn_exch{" H52 DAO"}="2H5  DAO";
$resn_exch{" H61 DAO"}="1H6  DAO"; $resn_exch{" H62 DAO"}="2H6  DAO"; $resn_exch{" H71 DAO"}="1H7  DAO";
$resn_exch{" H72 DAO"}="2H7  DAO"; $resn_exch{" H81 DAO"}="1H8  DAO"; $resn_exch{" H82 DAO"}="2H8  DAO";
$resn_exch{" H91 DAO"}="1H9  DAO"; $resn_exch{" H92 DAO"}="2H9  DAO"; $resn_exch{"H101 DAO"}="1H10 DAO";
$resn_exch{"H102 DAO"}="2H10 DAO"; $resn_exch{"H111 DAO"}="1H11 DAO"; $resn_exch{"H112 DAO"}="2H11 DAO";
$resn_exch{"H121 DAO"}="1H12 DAO"; $resn_exch{"H122 DAO"}="2H12 DAO"; $resn_exch{"H123 DAO"}="3H12 DAO";
$resn_exch{"HN31 DAP"}="1HN3 DAP"; $resn_exch{"HN32 DAP"}="2HN3 DAP"; $resn_exch{" H2' DAP"}=" H2* DAP";
$resn_exch{" H3' DAP"}=" H3* DAP"; $resn_exch{" H5' DAP"}=" H5* DAP"; $resn_exch{" H6' DAP"}=" H6* DAP";
$resn_exch{"HN51 DAP"}="1HN5 DAP"; $resn_exch{"HN52 DAP"}="2HN5 DAP"; $resn_exch{"HOB2 DAQ"}="2HOB DAQ";
$resn_exch{"HOA2 DAQ"}="2HOA DAQ"; $resn_exch{"HNA1 DAQ"}="1HNA DAQ"; $resn_exch{"HA21 DAQ"}="1HA2 DAQ";
$resn_exch{"HA22 DAQ"}="2HA2 DAQ"; $resn_exch{"HA11 DAQ"}="1HA1 DAQ"; $resn_exch{"HA12 DAQ"}="2HA1 DAQ";
$resn_exch{"HA31 DAQ"}="1HA3 DAQ"; $resn_exch{"HA32 DAQ"}="2HA3 DAQ"; $resn_exch{" H2  DAR"}=" HN2 DAR";
$resn_exch{" HB2 DAR"}="1HB  DAR"; $resn_exch{" HB3 DAR"}="2HB  DAR"; $resn_exch{" HG2 DAR"}="1HG  DAR";
$resn_exch{" HG3 DAR"}="2HG  DAR"; $resn_exch{" HD2 DAR"}="1HD  DAR"; $resn_exch{" HD3 DAR"}="2HD  DAR";
$resn_exch{" HE  DAR"}=" HNE DAR"; $resn_exch{"HH11 DAR"}="1HH1 DAR"; $resn_exch{"HH12 DAR"}="2HH1 DAR";
$resn_exch{"HH21 DAR"}="1HH2 DAR"; $resn_exch{"HH22 DAR"}="2HH2 DAR"; $resn_exch{" H2  DAS"}=" HN2 DAS";
$resn_exch{" HB2 DAS"}="1HB  DAS"; $resn_exch{" HB3 DAS"}="2HB  DAS"; $resn_exch{"HOB2 DAT"}="2HOB DAT";
$resn_exch{"HOB3 DAT"}="3HOB DAT"; $resn_exch{"HOA2 DAT"}="2HOA DAT"; $resn_exch{"H5'1 DAT"}="1H5* DAT";
$resn_exch{"H5'2 DAT"}="2H5* DAT"; $resn_exch{" H4' DAT"}=" H4* DAT"; $resn_exch{" H3' DAT"}=" H3* DAT";
$resn_exch{"HO3' DAT"}="*HO3 DAT"; $resn_exch{"H2'1 DAT"}="1H2* DAT"; $resn_exch{"H2'2 DAT"}="2H2* DAT";
$resn_exch{" H1' DAT"}=" H1* DAT"; $resn_exch{"HN61 DAT"}="1HN6 DAT"; $resn_exch{"HN62 DAT"}="2HN6 DAT";
$resn_exch{"HC61 DAU"}="1HC6 DAU"; $resn_exch{"HC62 DAU"}="2HC6 DAU"; $resn_exch{"H4PO DAU"}="OH4P DAU";
$resn_exch{"H2PO DAU"}="OH2P DAU"; $resn_exch{"HC51 DAU"}="1HC5 DAU"; $resn_exch{"HC52 DAU"}="2HC5 DAU";
$resn_exch{"H4,C DAU"}="CH4, DAU"; $resn_exch{"H3,C DAU"}="CH3, DAU"; $resn_exch{"H3,O DAU"}="OH3, DAU";
$resn_exch{"HC21 DAU"}="1HC2 DAU"; $resn_exch{"HC22 DAU"}="2HC2 DAU"; $resn_exch{"H1,C DAU"}="CH1, DAU";
$resn_exch{"H61C DAU"}="CH61 DAU"; $resn_exch{"H5A1 DAU"}="1H5A DAU"; $resn_exch{"H5A2 DAU"}="2H5A DAU";
$resn_exch{"H5A3 DAU"}="3H5A DAU"; $resn_exch{"H31N DAU"}="NH31 DAU"; $resn_exch{"HCB1 DAV"}="1HCB DAV";
$resn_exch{"HCB2 DAV"}="2HCB DAV"; $resn_exch{"HCG1 DAV"}="1HCG DAV"; $resn_exch{"HCG2 DAV"}="2HCG DAV";
$resn_exch{"HCD1 DAV"}="1HCD DAV"; $resn_exch{"HCD2 DAV"}="2HCD DAV"; $resn_exch{"HCA1 DAV"}="1HCA DAV";
$resn_exch{"HCA2 DAV"}="2HCA DAV"; $resn_exch{" H91 DAX"}="1H9  DAX"; $resn_exch{" H92 DAX"}="2H9  DAX";
$resn_exch{"HD21 DAX"}="1HD2 DAX"; $resn_exch{"HD22 DAX"}="2HD2 DAX"; $resn_exch{"HD31 DAX"}="1HD3 DAX";
$resn_exch{"HD32 DAX"}="2HD3 DAX"; $resn_exch{"HD71 DAX"}="1HD7 DAX"; $resn_exch{"HD72 DAX"}="2HD7 DAX";
$resn_exch{"HD73 DAX"}="3HD7 DAX"; $resn_exch{"HD81 DAX"}="1HD8 DAX"; $resn_exch{"HD82 DAX"}="2HD8 DAX";
$resn_exch{"HD83 DAX"}="3HD8 DAX"; $resn_exch{"HAF1 DB0"}="1HAF DB0"; $resn_exch{"HAF2 DB0"}="2HAF DB0";
$resn_exch{"HAA1 DB0"}="1HAA DB0"; $resn_exch{"HAA2 DB0"}="2HAA DB0"; $resn_exch{"HAA3 DB0"}="3HAA DB0";
$resn_exch{"HAC1 DB0"}="1HAC DB0"; $resn_exch{"HAC2 DB0"}="2HAC DB0"; $resn_exch{"HN11 DB1"}="1HN1 DB1";
$resn_exch{"HN12 DB1"}="2HN1 DB1"; $resn_exch{"H141 DB9"}="1H14 DB9"; $resn_exch{"H142 DB9"}="2H14 DB9";
$resn_exch{"H261 DB9"}="1H26 DB9"; $resn_exch{"H262 DB9"}="2H26 DB9"; $resn_exch{" H21 DBA"}="1H2  DBA";
$resn_exch{" H22 DBA"}="2H2  DBA"; $resn_exch{"HM21 DBA"}="1HM2 DBA"; $resn_exch{"HM22 DBA"}="2HM2 DBA";
$resn_exch{"HM23 DBA"}="3HM2 DBA"; $resn_exch{"HM61 DBA"}="1HM6 DBA"; $resn_exch{"HM62 DBA"}="2HM6 DBA";
$resn_exch{"HM63 DBA"}="3HM6 DBA"; $resn_exch{" H51 DBD"}="1H5  DBD"; $resn_exch{" H52 DBD"}="2H5  DBD";
$resn_exch{" H61 DBD"}="1H6  DBD"; $resn_exch{" H62 DBD"}="2H6  DBD"; $resn_exch{"H191 DBD"}="1H19 DBD";
$resn_exch{"H192 DBD"}="2H19 DBD"; $resn_exch{"H151 DBG"}="1H15 DBG"; $resn_exch{"H152 DBG"}="2H15 DBG";
$resn_exch{"HNG3 DBG"}="3HNG DBG"; $resn_exch{"HOG1 DBG"}="1HOG DBG"; $resn_exch{"HNB3 DBG"}="3HNB DBG";
$resn_exch{"HOB1 DBG"}="1HOB DBG"; $resn_exch{"HOA1 DBG"}="1HOA DBG"; $resn_exch{"H5'1 DBG"}="1H5* DBG";
$resn_exch{"H5'2 DBG"}="2H5* DBG"; $resn_exch{" H4' DBG"}=" H4* DBG"; $resn_exch{" H2' DBG"}=" H2* DBG";
$resn_exch{"HO2' DBG"}="HO2* DBG"; $resn_exch{" H3' DBG"}=" H3* DBG"; $resn_exch{"HO3' DBG"}="HO3* DBG";
$resn_exch{" H1' DBG"}=" H1* DBG"; $resn_exch{"HN21 DBG"}="1HN2 DBG"; $resn_exch{"HN22 DBG"}="2HN2 DBG";
$resn_exch{"H5'1 DBI"}="1H5' DBI"; $resn_exch{"H5'2 DBI"}="2H5' DBI"; $resn_exch{"H111 DBI"}="1H11 DBI";
$resn_exch{"H112 DBI"}="2H11 DBI"; $resn_exch{"H113 DBI"}="3H11 DBI"; $resn_exch{"H101 DBI"}="1H10 DBI";
$resn_exch{"H102 DBI"}="2H10 DBI"; $resn_exch{"H103 DBI"}="3H10 DBI"; $resn_exch{"HO21 DBI"}="1HO2 DBI";
$resn_exch{" HC1 DBL"}="1HC  DBL"; $resn_exch{" HC2 DBL"}="2HC  DBL"; $resn_exch{" HC3 DBL"}="3HC  DBL";
$resn_exch{" HN2 DBL"}="2HN  DBL"; $resn_exch{"HC61 DBL"}="1HC6 DBL"; $resn_exch{"HC62 DBL"}="2HC6 DBL";
$resn_exch{" HC7 DBL"}="7HC  DBL"; $resn_exch{" HA1 DBL"}="1HA  DBL"; $resn_exch{" HB2 DBL"}="2HB  DBL";
$resn_exch{" HD3 DBL"}="3HD  DBL"; $resn_exch{" HB1 DBL"}="1HB  DBL"; $resn_exch{" HD2 DBL"}="2HD  DBL";
$resn_exch{" HE3 DBL"}="3HE  DBL"; $resn_exch{"HN01 DBL"}="1HN0 DBL"; $resn_exch{"HC11 DBL"}="1HC1 DBL";
$resn_exch{"HC41 DBL"}="1HC4 DBL"; $resn_exch{"HO51 DBL"}="1HO5 DBL"; $resn_exch{"HC71 DBL"}="1HC7 DBL";
$resn_exch{"HC81 DBL"}="1HC8 DBL"; $resn_exch{"HC02 DBL"}="2HC0 DBL"; $resn_exch{"HN42 DBL"}="2HN4 DBL";
$resn_exch{"HC52 DBL"}="2HC5 DBL"; $resn_exch{"H281 DBL"}="1H28 DBL"; $resn_exch{"H282 DBL"}="2H28 DBL";
$resn_exch{"H311 DBL"}="1H31 DBL"; $resn_exch{"H312 DBL"}="2H31 DBL"; $resn_exch{"HN23 DBL"}="3HN2 DBL";
$resn_exch{"HC33 DBL"}="3HC3 DBL"; $resn_exch{"HC73 DBL"}="3HC7 DBL"; $resn_exch{"HC83 DBL"}="3HC8 DBL";
$resn_exch{"HO24 DBL"}="4HO2 DBL"; $resn_exch{"HN34 DBL"}="4HN3 DBL"; $resn_exch{"HC44 DBL"}="4HC4 DBL";
$resn_exch{"HC84 DBL"}="4HC8 DBL"; $resn_exch{"HC94 DBL"}="4HC9 DBL"; $resn_exch{"HC15 DBL"}="5HC1 DBL";
$resn_exch{"HO35 DBL"}="5HO3 DBL"; $resn_exch{"HN45 DBL"}="5HN4 DBL"; $resn_exch{"HC55 DBL"}="5HC5 DBL";
$resn_exch{"HC85 DBL"}="5HC8 DBL"; $resn_exch{"HC16 DBL"}="6HC1 DBL"; $resn_exch{"HC26 DBL"}="6HC2 DBL";
$resn_exch{"HC36 DBL"}="6HC3 DBL"; $resn_exch{"HN86 DBL"}="6HN8 DBL"; $resn_exch{"HC96 DBL"}="6HC9 DBL";
$resn_exch{"HC47 DBL"}="7HC4 DBL"; $resn_exch{"HC67 DBL"}="7HC6 DBL"; $resn_exch{"HO87 DBL"}="7HO8 DBL";
$resn_exch{"HO97 DBL"}="7HO9 DBL"; $resn_exch{"HO08 DBL"}="8HO0 DBL"; $resn_exch{"HC'1 DBL"}="1HC' DBL";
$resn_exch{"H2'1 DBL"}="1H2' DBL"; $resn_exch{"H2'2 DBL"}="2H2' DBL"; $resn_exch{"HC'5 DBL"}="5HC' DBL";
$resn_exch{"H7'1 DBL"}="1H7' DBL"; $resn_exch{"H7'2 DBL"}="2H7' DBL"; $resn_exch{"H7'3 DBL"}="3H7' DBL";
$resn_exch{"H8'1 DBL"}="1H8' DBL"; $resn_exch{"H8'2 DBL"}="2H8' DBL"; $resn_exch{"H9'O DBL"}="OH9' DBL";
$resn_exch{"HO'1 DBL"}="1HO\" DBL"; $resn_exch{"H2A1 DBL"}="1H2\" DBL"; $resn_exch{"H2A2 DBL"}="2H2\" DBL";
$resn_exch{"H2'3 DBL"}="3H2\" DBL"; $resn_exch{"H6'1 DBM"}="1H6' DBM"; $resn_exch{"H6'2 DBM"}="2H6' DBM";
$resn_exch{"H6'3 DBM"}="3H6' DBM"; $resn_exch{" H71 DBM"}="1H7  DBM"; $resn_exch{" H72 DBM"}="2H7  DBM";
$resn_exch{" H73 DBM"}="3H7  DBM"; $resn_exch{"H271 DBN"}="1H27 DBN"; $resn_exch{"H272 DBN"}="2H27 DBN";
$resn_exch{"H281 DBN"}="1H28 DBN"; $resn_exch{"H282 DBN"}="2H28 DBN"; $resn_exch{"H291 DBN"}="1H29 DBN";
$resn_exch{"H292 DBN"}="2H29 DBN"; $resn_exch{"H301 DBN"}="1H30 DBN"; $resn_exch{"H302 DBN"}="2H30 DBN";
$resn_exch{" H91 DBO"}="1H9  DBO"; $resn_exch{" H92 DBO"}="2H9  DBO"; $resn_exch{"H141 DBO"}="1H14 DBO";
$resn_exch{"H142 DBO"}="2H14 DBO"; $resn_exch{"H271 DBO"}="1H27 DBO"; $resn_exch{"H272 DBO"}="2H27 DBO";
$resn_exch{"H273 DBO"}="3H27 DBO"; $resn_exch{"H311 DBO"}="1H31 DBO"; $resn_exch{"H312 DBO"}="2H31 DBO";
$resn_exch{" HN1 DBP"}="1HN  DBP"; $resn_exch{"HN21 DBP"}="1HN2 DBP"; $resn_exch{"HB11 DBP"}="1HB1 DBP";
$resn_exch{"HB21 DBP"}="1HB2 DBP"; $resn_exch{"HD11 DBP"}="1HD1 DBP"; $resn_exch{"HC11 DBP"}="1HC1 DBP";
$resn_exch{"HC21 DBP"}="1HC2 DBP"; $resn_exch{"HCZ1 DBP"}="1HCZ DBP"; $resn_exch{"HE21 DBP"}="1HE2 DBP";
$resn_exch{"HE22 DBP"}="2HE2 DBP"; $resn_exch{" H61 DBQ"}="1H6  DBQ"; $resn_exch{" H62 DBQ"}="2H6  DBQ";
$resn_exch{" H71 DBQ"}="1H7  DBQ"; $resn_exch{" H72 DBQ"}="2H7  DBQ"; $resn_exch{"HN51 DBQ"}="1HN5 DBQ";
$resn_exch{"HN52 DBQ"}="2HN5 DBQ"; $resn_exch{"H11A DBR"}="1H1  DBR"; $resn_exch{"H12A DBR"}="2H1  DBR";
$resn_exch{"H31A DBR"}="1H3  DBR"; $resn_exch{" H32 DBR"}="2H3  DBR"; $resn_exch{" H41 DBR"}="1H4  DBR";
$resn_exch{" H42 DBR"}="2H4  DBR"; $resn_exch{" H61 DBR"}="1H6  DBR"; $resn_exch{" H62 DBR"}="2H6  DBR";
$resn_exch{" H71 DBR"}="1H7  DBR"; $resn_exch{" H72 DBR"}="2H7  DBR"; $resn_exch{"H101 DBR"}="1H10 DBR";
$resn_exch{"H102 DBR"}="2H10 DBR"; $resn_exch{"H131 DBR"}="1H13 DBR"; $resn_exch{"H132 DBR"}="2H13 DBR";
$resn_exch{"H141 DBR"}="1H14 DBR"; $resn_exch{"H142 DBR"}="2H14 DBR"; $resn_exch{"H151 DBR"}="1H15 DBR";
$resn_exch{"H152 DBR"}="2H15 DBR"; $resn_exch{"H161 DBR"}="1H16 DBR"; $resn_exch{"H162 DBR"}="2H16 DBR";
$resn_exch{"H281 DBR"}="1H28 DBR"; $resn_exch{"H282 DBR"}="2H28 DBR"; $resn_exch{"H283 DBR"}="3H28 DBR";
$resn_exch{"H291 DBR"}="1H29 DBR"; $resn_exch{"H292 DBR"}="2H29 DBR"; $resn_exch{"H293 DBR"}="3H29 DBR";
$resn_exch{"H281 DBS"}="1H28 DBS"; $resn_exch{"H282 DBS"}="2H28 DBS"; $resn_exch{" H81 DBT"}="1H8  DBT";
$resn_exch{" H82 DBT"}="2H8  DBT"; $resn_exch{" H91 DBT"}="1H9  DBT"; $resn_exch{" H92 DBT"}="2H9  DBT";
$resn_exch{"H101 DBT"}="1H10 DBT"; $resn_exch{"H102 DBT"}="2H10 DBT"; $resn_exch{"H111 DBT"}="1H11 DBT";
$resn_exch{"H112 DBT"}="2H11 DBT"; $resn_exch{"H121 DBT"}="1H12 DBT"; $resn_exch{"H122 DBT"}="2H12 DBT";
$resn_exch{"H131 DBT"}="1H13 DBT"; $resn_exch{"H132 DBT"}="2H13 DBT"; $resn_exch{"H141 DBT"}="1H14 DBT";
$resn_exch{"H142 DBT"}="2H14 DBT"; $resn_exch{"H143 DBT"}="3H14 DBT"; $resn_exch{" HB1 DBU"}="1HB  DBU";
$resn_exch{" HG1 DBU"}="1HG  DBU"; $resn_exch{" HG2 DBU"}="2HG  DBU"; $resn_exch{" HG3 DBU"}="3HG  DBU";
$resn_exch{"HMA1 DBV"}="1HMA DBV"; $resn_exch{"HMA2 DBV"}="2HMA DBV"; $resn_exch{"HMA3 DBV"}="3HMA DBV";
$resn_exch{"HBA1 DBV"}="1HBA DBV"; $resn_exch{"HBA2 DBV"}="2HBA DBV"; $resn_exch{"HHA1 DBV"}="1HHA DBV";
$resn_exch{"HAA1 DBV"}="1HAA DBV"; $resn_exch{"HHB1 DBV"}="1HHB DBV"; $resn_exch{"HMB1 DBV"}="1HMB DBV";
$resn_exch{"HMB2 DBV"}="2HMB DBV"; $resn_exch{"HMB3 DBV"}="3HMB DBV"; $resn_exch{"HAB1 DBV"}="1HAB DBV";
$resn_exch{"HAB2 DBV"}="2HAB DBV"; $resn_exch{"HBB1 DBV"}="1HBB DBV"; $resn_exch{"HBB2 DBV"}="2HBB DBV";
$resn_exch{"H2B1 DBV"}="1H2B DBV"; $resn_exch{"HHC1 DBV"}="1HHC DBV"; $resn_exch{"HHC2 DBV"}="2HHC DBV";
$resn_exch{"HMC1 DBV"}="1HMC DBV"; $resn_exch{"HMC2 DBV"}="2HMC DBV"; $resn_exch{"HMC3 DBV"}="3HMC DBV";
$resn_exch{"HAC1 DBV"}="1HAC DBV"; $resn_exch{"HAC2 DBV"}="2HAC DBV"; $resn_exch{"HBC1 DBV"}="1HBC DBV";
$resn_exch{"HBC2 DBV"}="2HBC DBV"; $resn_exch{"H2C1 DBV"}="1H2C DBV"; $resn_exch{"H1D1 DBV"}="1H1D DBV";
$resn_exch{"HMD1 DBV"}="1HMD DBV"; $resn_exch{"HMD2 DBV"}="2HMD DBV"; $resn_exch{"HMD3 DBV"}="3HMD DBV";
$resn_exch{"HAD1 DBV"}="1HAD DBV"; $resn_exch{"HBD1 DBV"}="1HBD DBV"; $resn_exch{"HBD2 DBV"}="2HBD DBV";
$resn_exch{" H   DBY"}="1HN  DBY"; $resn_exch{" H2  DBY"}="2HN  DBY"; $resn_exch{" HB2 DBY"}="1HB  DBY";
$resn_exch{" HB3 DBY"}="2HB  DBY"; $resn_exch{" HN1 DBZ"}="1HN  DBZ"; $resn_exch{" HN2 DBZ"}="2HN  DBZ";
$resn_exch{" HB1 DBZ"}="1HB  DBZ"; $resn_exch{" HB2 DBZ"}="2HB  DBZ"; $resn_exch{"HOP3  DC"}="3HOP  DC";
$resn_exch{"HOP2  DC"}="2HOP  DC"; $resn_exch{" H5'  DC"}="1H5*  DC"; $resn_exch{"H5''  DC"}="2H5*  DC";
$resn_exch{" H4'  DC"}=" H4*  DC"; $resn_exch{" H3'  DC"}=" H3*  DC"; $resn_exch{"HO3'  DC"}="H3T   DC";
$resn_exch{" H2'  DC"}="1H2*  DC"; $resn_exch{"H2''  DC"}="2H2*  DC"; $resn_exch{" H1'  DC"}=" H1*  DC";
$resn_exch{" H41  DC"}="1H4   DC"; $resn_exch{" H42  DC"}="2H4   DC"; $resn_exch{"H101 DC8"}="1H10 DC8";
$resn_exch{"H102 DC8"}="2H10 DC8"; $resn_exch{"H141 DC8"}="1H14 DC8"; $resn_exch{"H142 DC8"}="2H14 DC8";
$resn_exch{" H2A DCA"}=" AH2 DCA"; $resn_exch{"H61A DCA"}="AH61 DCA"; $resn_exch{"H62A DCA"}="AH62 DCA";
$resn_exch{" H8A DCA"}=" AH8 DCA"; $resn_exch{" H1B DCA"}="AH1* DCA"; $resn_exch{" H2B DCA"}="AH2* DCA";
$resn_exch{"HO2A DCA"}="AHO2 DCA"; $resn_exch{" H3B DCA"}="AH3* DCA"; $resn_exch{"HOA8 DCA"}="8HOA DCA";
$resn_exch{"HOA9 DCA"}="9HOA DCA"; $resn_exch{" H4B DCA"}="AH4* DCA"; $resn_exch{"H51A DCA"}="AH51 DCA";
$resn_exch{"H52A DCA"}="AH52 DCA"; $resn_exch{"HOA2 DCA"}="2HOA DCA"; $resn_exch{"HOA5 DCA"}="5HOA DCA";
$resn_exch{"H121 DCA"}="1H12 DCA"; $resn_exch{"H122 DCA"}="2H12 DCA"; $resn_exch{"H131 DCA"}="1H13 DCA";
$resn_exch{"H132 DCA"}="2H13 DCA"; $resn_exch{"H133 DCA"}="3H13 DCA"; $resn_exch{"H141 DCA"}="1H14 DCA";
$resn_exch{"H142 DCA"}="2H14 DCA"; $resn_exch{"H143 DCA"}="3H14 DCA"; $resn_exch{" H71 DCA"}="1H7  DCA";
$resn_exch{" H72 DCA"}="2H7  DCA"; $resn_exch{" H61 DCA"}="1H6  DCA"; $resn_exch{" H62 DCA"}="2H6  DCA";
$resn_exch{" H31 DCA"}="1H3  DCA"; $resn_exch{" H32 DCA"}="2H3  DCA"; $resn_exch{" H21 DCA"}="1H2  DCA";
$resn_exch{" H22 DCA"}="2H2  DCA"; $resn_exch{" H23 DCA"}="3H2  DCA"; $resn_exch{"H6A1 DCB"}="1H6A DCB";
$resn_exch{"HCA2 DCB"}="2HCA DCB"; $resn_exch{"H6B1 DCB"}="1H6B DCB"; $resn_exch{"H6B2 DCB"}="2H6B DCB";
$resn_exch{"H101 DCC"}="1H10 DCC"; $resn_exch{"H102 DCC"}="2H10 DCC"; $resn_exch{"H111 DCC"}="1H11 DCC";
$resn_exch{"H112 DCC"}="2H11 DCC"; $resn_exch{"H121 DCC"}="1H12 DCC"; $resn_exch{"H122 DCC"}="2H12 DCC";
$resn_exch{"H123 DCC"}="3H12 DCC"; $resn_exch{"HN61 DCC"}="1HN6 DCC"; $resn_exch{"HN62 DCC"}="2HN6 DCC";
$resn_exch{"H5'1 DCC"}="1H5' DCC"; $resn_exch{"H5'2 DCC"}="2H5' DCC"; $resn_exch{"HPA1 DCC"}="1HPA DCC";
$resn_exch{"HPA2 DCC"}="2HPA DCC"; $resn_exch{"HP71 DCC"}="1HP7 DCC"; $resn_exch{"HP72 DCC"}="2HP7 DCC";
$resn_exch{"HP73 DCC"}="3HP7 DCC"; $resn_exch{"HP91 DCC"}="1HP9 DCC"; $resn_exch{"HP92 DCC"}="2HP9 DCC";
$resn_exch{"HP93 DCC"}="3HP9 DCC"; $resn_exch{"HP51 DCC"}="1HP5 DCC"; $resn_exch{"HP52 DCC"}="2HP5 DCC";
$resn_exch{"HP41 DCC"}="1HP4 DCC"; $resn_exch{"HP42 DCC"}="2HP4 DCC"; $resn_exch{"HP21 DCC"}="1HP2 DCC";
$resn_exch{"HP22 DCC"}="2HP2 DCC"; $resn_exch{"HP11 DCC"}="1HP1 DCC"; $resn_exch{"HP12 DCC"}="2HP1 DCC";
$resn_exch{"HA21 DCC"}="1HA2 DCC"; $resn_exch{"HA22 DCC"}="2HA2 DCC"; $resn_exch{"HA31 DCC"}="1HA3 DCC";
$resn_exch{"HA32 DCC"}="2HA3 DCC"; $resn_exch{"HA41 DCC"}="1HA4 DCC"; $resn_exch{"HA42 DCC"}="2HA4 DCC";
$resn_exch{"HA51 DCC"}="1HA5 DCC"; $resn_exch{"HA52 DCC"}="2HA5 DCC"; $resn_exch{"HA61 DCC"}="1HA6 DCC";
$resn_exch{"HA62 DCC"}="2HA6 DCC"; $resn_exch{"HA71 DCC"}="1HA7 DCC"; $resn_exch{"HA72 DCC"}="2HA7 DCC";
$resn_exch{"HA81 DCC"}="1HA8 DCC"; $resn_exch{"HA82 DCC"}="2HA8 DCC"; $resn_exch{"HA91 DCC"}="1HA9 DCC";
$resn_exch{"HA92 DCC"}="2HA9 DCC"; $resn_exch{"H5C1 DCD"}="1H5C DCD"; $resn_exch{"H5C2 DCD"}="2H5C DCD";
$resn_exch{"H5C3 DCD"}="3H5C DCD"; $resn_exch{"H3C1 DCD"}="1H3C DCD"; $resn_exch{"H3C2 DCD"}="2H3C DCD";
$resn_exch{"H2C1 DCD"}="1H2C DCD"; $resn_exch{"H2C2 DCD"}="2H2C DCD"; $resn_exch{"H4C1 DCD"}="1H4C DCD";
$resn_exch{"H4C2 DCD"}="2H4C DCD"; $resn_exch{"H4C3 DCD"}="3H4C DCD"; $resn_exch{" H11 DCE"}="1H1  DCE";
$resn_exch{" H12 DCE"}="2H1  DCE"; $resn_exch{" H21 DCE"}="1H2  DCE"; $resn_exch{" H22 DCE"}="2H2  DCE";
$resn_exch{" H71 DCF"}="1H7  DCF"; $resn_exch{" H72 DCF"}="2H7  DCF"; $resn_exch{"H2S1 DCF"}="1H2S DCF";
$resn_exch{"H2S2 DCF"}="2H2S DCF"; $resn_exch{"H5S1 DCF"}="1H5S DCF"; $resn_exch{"H5S2 DCF"}="2H5S DCF";
$resn_exch{"HOP2 DCG"}="2HOP DCG"; $resn_exch{"HOP3 DCG"}="3HOP DCG"; $resn_exch{" H5' DCG"}="1H5* DCG";
$resn_exch{"H5'' DCG"}="2H5* DCG"; $resn_exch{" H4' DCG"}=" H4* DCG"; $resn_exch{" H3' DCG"}=" H3* DCG";
$resn_exch{"HO3' DCG"}="*HO3 DCG"; $resn_exch{" H2' DCG"}="1H2* DCG"; $resn_exch{"H2'' DCG"}="2H2* DCG";
$resn_exch{" H1' DCG"}=" H1* DCG"; $resn_exch{"HN21 DCG"}="1HN2 DCG"; $resn_exch{"HN22 DCG"}="2HN2 DCG";
$resn_exch{"HZ11 DCH"}="1HZ1 DCH"; $resn_exch{"HZ12 DCH"}="2HZ1 DCH"; $resn_exch{"HZ21 DCH"}="1HZ2 DCH";
$resn_exch{"HZ22 DCH"}="2HZ2 DCH"; $resn_exch{"HA11 DCH"}="1HA1 DCH"; $resn_exch{"HA12 DCH"}="2HA1 DCH";
$resn_exch{"HA51 DCH"}="1HA5 DCH"; $resn_exch{"HA52 DCH"}="2HA5 DCH"; $resn_exch{"HA61 DCH"}="1HA6 DCH";
$resn_exch{"HA62 DCH"}="2HA6 DCH"; $resn_exch{"HA63 DCH"}="3HA6 DCH"; $resn_exch{"H321 DCH"}="1H32 DCH";
$resn_exch{"H322 DCH"}="2H32 DCH"; $resn_exch{"H341 DCH"}="1H34 DCH"; $resn_exch{"H342 DCH"}="2H34 DCH";
$resn_exch{"H351 DCH"}="1H35 DCH"; $resn_exch{"H352 DCH"}="2H35 DCH"; $resn_exch{" HN1 DCI"}="1HN  DCI";
$resn_exch{" HN2 DCI"}="2HN  DCI"; $resn_exch{" HA1 DCI"}="1HA  DCI"; $resn_exch{" HA2 DCI"}="2HA  DCI";
$resn_exch{"HG11 DCI"}="1HG1 DCI"; $resn_exch{"HG12 DCI"}="2HG1 DCI"; $resn_exch{"HG21 DCI"}="1HG2 DCI";
$resn_exch{"HG22 DCI"}="2HG2 DCI"; $resn_exch{"HG23 DCI"}="3HG2 DCI"; $resn_exch{" HD1 DCI"}="1HD  DCI";
$resn_exch{" HD2 DCI"}="2HD  DCI"; $resn_exch{" HD3 DCI"}="3HD  DCI"; $resn_exch{" HN1 DCL"}="1HN  DCL";
$resn_exch{" HN2 DCL"}="2HN  DCL"; $resn_exch{" HC1 DCL"}="1HC  DCL"; $resn_exch{" HC2 DCL"}="2HC  DCL";
$resn_exch{" HB1 DCL"}="1HB  DCL"; $resn_exch{" HB2 DCL"}="2HB  DCL"; $resn_exch{"HD11 DCL"}="1HD1 DCL";
$resn_exch{"HD12 DCL"}="2HD1 DCL"; $resn_exch{"HD13 DCL"}="3HD1 DCL"; $resn_exch{"HD21 DCL"}="1HD2 DCL";
$resn_exch{"HD22 DCL"}="2HD2 DCL"; $resn_exch{"HD23 DCL"}="3HD2 DCL"; $resn_exch{"HN41 DCM"}="1HN4 DCM";
$resn_exch{"HN42 DCM"}="2HN4 DCM"; $resn_exch{" H1' DCM"}=" H1* DCM"; $resn_exch{"H2'1 DCM"}="1H2* DCM";
$resn_exch{"H2'2 DCM"}="2H2* DCM"; $resn_exch{" H3' DCM"}=" H3* DCM"; $resn_exch{" H4' DCM"}=" H4* DCM";
$resn_exch{"HO3' DCM"}="*HO3 DCM"; $resn_exch{"H5'1 DCM"}="1H5* DCM"; $resn_exch{"H5'2 DCM"}="2H5* DCM";
$resn_exch{"HOP2 DCM"}="2HOP DCM"; $resn_exch{"HOP3 DCM"}="3HOP DCM"; $resn_exch{" H31 DCO"}="1H3  DCO";
$resn_exch{" H32 DCO"}="2H3  DCO"; $resn_exch{"HN41 DCP"}="1HN4 DCP"; $resn_exch{"HN42 DCP"}="2HN4 DCP";
$resn_exch{" H1' DCP"}=" H1* DCP"; $resn_exch{"H2'1 DCP"}="1H2* DCP"; $resn_exch{"H2'2 DCP"}="2H2* DCP";
$resn_exch{" H3' DCP"}=" H3* DCP"; $resn_exch{" H4' DCP"}=" H4* DCP"; $resn_exch{"HO3' DCP"}="*HO3 DCP";
$resn_exch{"H5'1 DCP"}="1H5* DCP"; $resn_exch{"H5'2 DCP"}="2H5* DCP"; $resn_exch{"HOA2 DCP"}="2HOA DCP";
$resn_exch{"HOB2 DCP"}="2HOB DCP"; $resn_exch{"HOG2 DCP"}="2HOG DCP"; $resn_exch{"HOG3 DCP"}="3HOG DCP";
$resn_exch{"H2A1 DCS"}="1H2A DCS"; $resn_exch{"H2A2 DCS"}="2H2A DCS"; $resn_exch{"H2A3 DCS"}="3H2A DCS";
$resn_exch{"H4A1 DCS"}="1H4A DCS"; $resn_exch{"H4A2 DCS"}="2H4A DCS"; $resn_exch{"H5A1 DCS"}="1H5A DCS";
$resn_exch{"H5A2 DCS"}="2H5A DCS"; $resn_exch{"HOP2 DCS"}="2HOP DCS"; $resn_exch{"HOP3 DCS"}="3HOP DCS";
$resn_exch{" HB1 DCS"}="1HB  DCS"; $resn_exch{" HB2 DCS"}="2HB  DCS"; $resn_exch{"HN41 DCT"}="1HN4 DCT";
$resn_exch{"HN42 DCT"}="2HN4 DCT"; $resn_exch{" H1' DCT"}=" H1* DCT"; $resn_exch{" H2' DCT"}="1H2* DCT";
$resn_exch{"H2'' DCT"}="2H2* DCT"; $resn_exch{"H3'1 DCT"}="1H3* DCT"; $resn_exch{"H3'2 DCT"}="2H3* DCT";
$resn_exch{" H4' DCT"}=" H4* DCT"; $resn_exch{" H5' DCT"}="1H5* DCT"; $resn_exch{"H5'' DCT"}="2H5* DCT";
$resn_exch{"HOA2 DCT"}="2HOA DCT"; $resn_exch{"HOB2 DCT"}="2HOB DCT"; $resn_exch{"HOG2 DCT"}="2HOG DCT";
$resn_exch{"HOG3 DCT"}="3HOG DCT"; $resn_exch{"H21A DCU"}="1H2  DCU"; $resn_exch{" H22 DCU"}="2H2  DCU";
$resn_exch{"H31A DCU"}="1H3  DCU"; $resn_exch{"H32A DCU"}="2H3  DCU"; $resn_exch{" H41 DCU"}="1H4  DCU";
$resn_exch{" H42 DCU"}="2H4  DCU"; $resn_exch{" H51 DCU"}="1H5  DCU"; $resn_exch{" H52 DCU"}="2H5  DCU";
$resn_exch{" H61 DCU"}="1H6  DCU"; $resn_exch{" H62 DCU"}="2H6  DCU"; $resn_exch{" H71 DCU"}="1H7  DCU";
$resn_exch{" H72 DCU"}="2H7  DCU"; $resn_exch{" H81 DCU"}="1H8  DCU"; $resn_exch{" H82 DCU"}="2H8  DCU";
$resn_exch{" H91 DCU"}="1H9  DCU"; $resn_exch{" H92 DCU"}="2H9  DCU"; $resn_exch{"H101 DCU"}="1H10 DCU";
$resn_exch{"H102 DCU"}="2H10 DCU"; $resn_exch{"H111 DCU"}="1H11 DCU"; $resn_exch{"H112 DCU"}="2H11 DCU";
$resn_exch{"H131 DCU"}="1H13 DCU"; $resn_exch{"H132 DCU"}="2H13 DCU"; $resn_exch{"H133 DCU"}="3H13 DCU";
$resn_exch{"H141 DCU"}="1H14 DCU"; $resn_exch{"H142 DCU"}="2H14 DCU"; $resn_exch{"H143 DCU"}="3H14 DCU";
$resn_exch{"H151 DCU"}="1H15 DCU"; $resn_exch{"H152 DCU"}="2H15 DCU"; $resn_exch{"H153 DCU"}="3H15 DCU";
$resn_exch{"H361 DCU"}="1H36 DCU"; $resn_exch{"H362 DCU"}="2H36 DCU"; $resn_exch{"H371 DCU"}="1H37 DCU";
$resn_exch{"H372 DCU"}="2H37 DCU"; $resn_exch{" H31 DCX"}="1H3  DCX"; $resn_exch{" H32 DCX"}="2H3  DCX";
$resn_exch{" H33 DCX"}="3H3  DCX"; $resn_exch{" H61 DCX"}="1H6  DCX"; $resn_exch{" H62 DCX"}="2H6  DCX";
$resn_exch{"H141 DCX"}="1H14 DCX"; $resn_exch{"H142 DCX"}="2H14 DCX"; $resn_exch{"H143 DCX"}="3H14 DCX";
$resn_exch{"H131 DCX"}="1H13 DCX"; $resn_exch{"H132 DCX"}="2H13 DCX"; $resn_exch{"H121 DCX"}="1H12 DCX";
$resn_exch{"H122 DCX"}="2H12 DCX"; $resn_exch{"H111 DCX"}="1H11 DCX"; $resn_exch{"H112 DCX"}="2H11 DCX";
$resn_exch{"H101 DCX"}="1H10 DCX"; $resn_exch{"H102 DCX"}="2H10 DCX"; $resn_exch{" H91 DCX"}="1H9  DCX";
$resn_exch{" H92 DCX"}="2H9  DCX"; $resn_exch{" H81 DCX"}="1H8  DCX"; $resn_exch{" H82 DCX"}="2H8  DCX";
$resn_exch{" H71 DCX"}="1H7  DCX"; $resn_exch{" H72 DCX"}="2H7  DCX"; $resn_exch{" H41 DCX"}="1H4  DCX";
$resn_exch{" H42 DCX"}="2H4  DCX"; $resn_exch{" H21 DCX"}="1H2  DCX"; $resn_exch{" H22 DCX"}="2H2  DCX";
$resn_exch{" H23 DCX"}="3H2  DCX"; $resn_exch{" H2  DCY"}=" HN2 DCY"; $resn_exch{" HB2 DCY"}="1HB  DCY";
$resn_exch{" HB3 DCY"}="2HB  DCY"; $resn_exch{"HN41 DCZ"}="1HN4 DCZ"; $resn_exch{"HN42 DCZ"}="2HN4 DCZ";
$resn_exch{" H1' DCZ"}=" H1* DCZ"; $resn_exch{"H2'1 DCZ"}="1H2* DCZ"; $resn_exch{"H2'2 DCZ"}="2H2* DCZ";
$resn_exch{" H3' DCZ"}=" H3* DCZ"; $resn_exch{" H4' DCZ"}=" H4* DCZ"; $resn_exch{"HO3' DCZ"}="*HO3 DCZ";
$resn_exch{"H5'1 DCZ"}="1H5* DCZ"; $resn_exch{"H5'2 DCZ"}="2H5* DCZ"; $resn_exch{"HO51 DCZ"}="1HO5 DCZ";
$resn_exch{" H21 DDA"}="1H2  DDA"; $resn_exch{" H22 DDA"}="2H2  DDA"; $resn_exch{" H61 DDA"}="1H6  DDA";
$resn_exch{" H62 DDA"}="2H6  DDA"; $resn_exch{" H63 DDA"}="3H6  DDA"; $resn_exch{" H21 DDB"}="1H2  DDB";
$resn_exch{" H22 DDB"}="2H2  DDB"; $resn_exch{"H3M1 DDB"}="1H3M DDB"; $resn_exch{"H3M2 DDB"}="2H3M DDB";
$resn_exch{"H3M3 DDB"}="3H3M DDB"; $resn_exch{" H61 DDB"}="1H6  DDB"; $resn_exch{" H62 DDB"}="2H6  DDB";
$resn_exch{" H63 DDB"}="3H6  DDB"; $resn_exch{" H81 DDC"}="1H8  DDC"; $resn_exch{" H82 DDC"}="2H8  DDC";
$resn_exch{"HB21 DDD"}="1HB2 DDD"; $resn_exch{"HB22 DDD"}="2HB2 DDD"; $resn_exch{"HG21 DDD"}="1HG2 DDD";
$resn_exch{"HG22 DDD"}="2HG2 DDD"; $resn_exch{"HA31 DDD"}="1HA3 DDD"; $resn_exch{"HA32 DDD"}="2HA3 DDD";
$resn_exch{"HD21 DDD"}="1HD2 DDD"; $resn_exch{"HD22 DDD"}="2HD2 DDD"; $resn_exch{"HB11 DDD"}="1HB1 DDD";
$resn_exch{"HB12 DDD"}="2HB1 DDD"; $resn_exch{"HA11 DDD"}="1HA1 DDD"; $resn_exch{"HA12 DDD"}="2HA1 DDD";
$resn_exch{" H   DDE"}="1HN  DDE"; $resn_exch{" H2  DDE"}="2HN  DDE"; $resn_exch{" HB2 DDE"}="1HB  DDE";
$resn_exch{" HB3 DDE"}="2HB  DDE"; $resn_exch{"HAD1 DDE"}="1HAD DDE"; $resn_exch{"HAD2 DDE"}="2HAD DDE";
$resn_exch{"HAB1 DDE"}="1HAB DDE"; $resn_exch{"HAB2 DDE"}="2HAB DDE"; $resn_exch{"HAB3 DDE"}="3HAB DDE";
$resn_exch{"HAC1 DDE"}="1HAC DDE"; $resn_exch{"HAC2 DDE"}="2HAC DDE"; $resn_exch{"HAC3 DDE"}="3HAC DDE";
$resn_exch{"HAA1 DDE"}="1HAA DDE"; $resn_exch{"HAA2 DDE"}="2HAA DDE"; $resn_exch{"HAA3 DDE"}="3HAA DDE";
$resn_exch{"HAU3 DDE"}="1HAU DDE"; $resn_exch{"HAU2 DDE"}="2HAU DDE"; $resn_exch{"HAT2 DDE"}="1HAT DDE";
$resn_exch{"HAT3 DDE"}="2HAT DDE"; $resn_exch{"HN21 DDF"}="1HN2 DDF"; $resn_exch{"HN22 DDF"}="2HN2 DDF";
$resn_exch{" H51 DDF"}="1H5  DDF"; $resn_exch{" H52 DDF"}="2H5  DDF"; $resn_exch{" H71 DDF"}="1H7  DDF";
$resn_exch{" H72 DDF"}="2H7  DDF"; $resn_exch{" H91 DDF"}="1H9  DDF"; $resn_exch{" H92 DDF"}="2H9  DDF";
$resn_exch{"H101 DDF"}="1H10 DDF"; $resn_exch{"H102 DDF"}="2H10 DDF"; $resn_exch{" HB1 DDF"}="1HB  DDF";
$resn_exch{" HB2 DDF"}="2HB  DDF"; $resn_exch{" HG1 DDF"}="1HG  DDF"; $resn_exch{" HG2 DDF"}="2HG  DDF";
$resn_exch{"HOE2 DDF"}="2HOE DDF"; $resn_exch{"HOP3 DDG"}="3HOP DDG"; $resn_exch{"HOP2 DDG"}="2HOP DDG";
$resn_exch{" H5' DDG"}="1H5* DDG"; $resn_exch{"H5'' DDG"}="2H5* DDG"; $resn_exch{" H4' DDG"}=" H4* DDG";
$resn_exch{"H3'1 DDG"}="1H3* DDG"; $resn_exch{"H3'2 DDG"}="2H3* DDG"; $resn_exch{" H2' DDG"}="1H2* DDG";
$resn_exch{"H2'' DDG"}="2H2* DDG"; $resn_exch{" H1' DDG"}=" H1* DDG"; $resn_exch{" H21 DDG"}="1H2  DDG";
$resn_exch{" H22 DDG"}="2H2  DDG"; $resn_exch{"HAA1 DDH"}="1HAA DDH"; $resn_exch{"HAA2 DDH"}="2HAA DDH";
$resn_exch{"HMA1 DDH"}="1HMA DDH"; $resn_exch{"HMA2 DDH"}="2HMA DDH"; $resn_exch{"HMA3 DDH"}="3HMA DDH";
$resn_exch{"HBA1 DDH"}="1HBA DDH"; $resn_exch{"HBA2 DDH"}="2HBA DDH"; $resn_exch{"HO2A DDH"}="AHO2 DDH";
$resn_exch{"HMB1 DDH"}="1HMB DDH"; $resn_exch{"HMB2 DDH"}="2HMB DDH"; $resn_exch{"HMB3 DDH"}="3HMB DDH";
$resn_exch{"HOCB DDH"}="BHOC DDH"; $resn_exch{"HBB1 DDH"}="1HBB DDH"; $resn_exch{"HBB2 DDH"}="2HBB DDH";
$resn_exch{"HMC1 DDH"}="1HMC DDH"; $resn_exch{"HMC2 DDH"}="2HMC DDH"; $resn_exch{"HMC3 DDH"}="3HMC DDH";
$resn_exch{"HBC1 DDH"}="1HBC DDH"; $resn_exch{"HBC2 DDH"}="2HBC DDH"; $resn_exch{"HBC3 DDH"}="3HBC DDH";
$resn_exch{"HMD1 DDH"}="1HMD DDH"; $resn_exch{"HMD2 DDH"}="2HMD DDH"; $resn_exch{"HMD3 DDH"}="3HMD DDH";
$resn_exch{"HAD1 DDH"}="1HAD DDH"; $resn_exch{"HAD2 DDH"}="2HAD DDH"; $resn_exch{"HBD1 DDH"}="1HBD DDH";
$resn_exch{"HBD2 DDH"}="2HBD DDH"; $resn_exch{"HO1D DDH"}="DHO1 DDH"; $resn_exch{" HN1 DDI"}="1HN  DDI";
$resn_exch{" HN2 DDI"}="2HN  DDI"; $resn_exch{" H21 DDL"}="1H2  DDL"; $resn_exch{" H22 DDL"}="2H2  DDL";
$resn_exch{" H61 DDL"}="1H6  DDL"; $resn_exch{" H62 DDL"}="2H6  DDL"; $resn_exch{" H63 DDL"}="3H6  DDL";
$resn_exch{"HC11 DDM"}="1HC1 DDM"; $resn_exch{"HC12 DDM"}="2HC1 DDM"; $resn_exch{"HC61 DDM"}="1HC6 DDM";
$resn_exch{"HC62 DDM"}="2HC6 DDM"; $resn_exch{" H5' DDN"}="1H5* DDN"; $resn_exch{"H5'' DDN"}="2H5* DDN";
$resn_exch{" H4' DDN"}=" H4* DDN"; $resn_exch{" H3' DDN"}=" H3* DDN"; $resn_exch{" H2' DDN"}="1H2* DDN";
$resn_exch{"H2'' DDN"}="2H2* DDN"; $resn_exch{" H1' DDN"}=" H1* DDN"; $resn_exch{" HN1 DDO"}="1HN  DDO";
$resn_exch{" HN2 DDO"}="2HN  DDO"; $resn_exch{" HA1 DDO"}="1HA  DDO"; $resn_exch{" HB1 DDO"}="1HB  DDO";
$resn_exch{" HB2 DDO"}="2HB  DDO"; $resn_exch{" HG1 DDO"}="1HG  DDO"; $resn_exch{" HG2 DDO"}="2HG  DDO";
$resn_exch{" HD1 DDO"}="1HD  DDO"; $resn_exch{" HD2 DDO"}="2HD  DDO"; $resn_exch{" HE1 DDO"}="1HE  DDO";
$resn_exch{" HE2 DDO"}="2HE  DDO"; $resn_exch{" H71 DDP"}="1H7  DDP"; $resn_exch{" H72 DDP"}="2H7  DDP";
$resn_exch{" H21 DDP"}="1H2  DDP"; $resn_exch{" H22 DDP"}="2H2  DDP"; $resn_exch{" H61 DDP"}="1H6  DDP";
$resn_exch{" H91 DDP"}="1H9  DDP"; $resn_exch{"HM11 DDQ"}="1HM1 DDQ"; $resn_exch{"HM12 DDQ"}="2HM1 DDQ";
$resn_exch{"HM13 DDQ"}="3HM1 DDQ"; $resn_exch{"HM21 DDQ"}="1HM2 DDQ"; $resn_exch{"HM22 DDQ"}="2HM2 DDQ";
$resn_exch{"HM23 DDQ"}="3HM2 DDQ"; $resn_exch{" H11 DDQ"}="1H1  DDQ"; $resn_exch{" H12 DDQ"}="2H1  DDQ";
$resn_exch{" H21 DDQ"}="1H2  DDQ"; $resn_exch{" H22 DDQ"}="2H2  DDQ"; $resn_exch{" H31 DDQ"}="1H3  DDQ";
$resn_exch{" H32 DDQ"}="2H3  DDQ"; $resn_exch{" H41 DDQ"}="1H4  DDQ"; $resn_exch{" H42 DDQ"}="2H4  DDQ";
$resn_exch{" H51 DDQ"}="1H5  DDQ"; $resn_exch{" H52 DDQ"}="2H5  DDQ"; $resn_exch{" H61 DDQ"}="1H6  DDQ";
$resn_exch{" H62 DDQ"}="2H6  DDQ"; $resn_exch{" H71 DDQ"}="1H7  DDQ"; $resn_exch{" H72 DDQ"}="2H7  DDQ";
$resn_exch{" H81 DDQ"}="1H8  DDQ"; $resn_exch{" H82 DDQ"}="2H8  DDQ"; $resn_exch{" H91 DDQ"}="1H9  DDQ";
$resn_exch{" H92 DDQ"}="2H9  DDQ"; $resn_exch{"H101 DDQ"}="1H10 DDQ"; $resn_exch{"H102 DDQ"}="2H10 DDQ";
$resn_exch{"H103 DDQ"}="3H10 DDQ"; $resn_exch{"H221 DDR"}="1H22 DDR"; $resn_exch{"H222 DDR"}="2H22 DDR";
$resn_exch{"H231 DDR"}="1H23 DDR"; $resn_exch{"H232 DDR"}="2H23 DDR"; $resn_exch{"H241 DDR"}="1H24 DDR";
$resn_exch{"H242 DDR"}="2H24 DDR"; $resn_exch{"H251 DDR"}="1H25 DDR"; $resn_exch{"H252 DDR"}="2H25 DDR";
$resn_exch{"H261 DDR"}="1H26 DDR"; $resn_exch{"H262 DDR"}="2H26 DDR"; $resn_exch{"H271 DDR"}="1H27 DDR";
$resn_exch{"H272 DDR"}="2H27 DDR"; $resn_exch{"H281 DDR"}="1H28 DDR"; $resn_exch{"H282 DDR"}="2H28 DDR";
$resn_exch{"H291 DDR"}="1H29 DDR"; $resn_exch{"H292 DDR"}="2H29 DDR"; $resn_exch{"H301 DDR"}="1H30 DDR";
$resn_exch{"H302 DDR"}="2H30 DDR"; $resn_exch{"H303 DDR"}="3H30 DDR"; $resn_exch{"H531 DDR"}="1H53 DDR";
$resn_exch{"H532 DDR"}="2H53 DDR"; $resn_exch{"H511 DDR"}="1H51 DDR"; $resn_exch{"H512 DDR"}="2H51 DDR";
$resn_exch{" H21 DDR"}="1H2  DDR"; $resn_exch{" H22 DDR"}="2H2  DDR"; $resn_exch{" H31 DDR"}="1H3  DDR";
$resn_exch{" H32 DDR"}="2H3  DDR"; $resn_exch{" H41 DDR"}="1H4  DDR"; $resn_exch{" H42 DDR"}="2H4  DDR";
$resn_exch{" H51 DDR"}="1H5  DDR"; $resn_exch{"H52A DDR"}="2H5  DDR"; $resn_exch{" H61 DDR"}="1H6  DDR";
$resn_exch{" H62 DDR"}="2H6  DDR"; $resn_exch{" H71 DDR"}="1H7  DDR"; $resn_exch{" H72 DDR"}="2H7  DDR";
$resn_exch{" H81 DDR"}="1H8  DDR"; $resn_exch{" H82 DDR"}="2H8  DDR"; $resn_exch{" H91 DDR"}="1H9  DDR";
$resn_exch{" H92 DDR"}="2H9  DDR"; $resn_exch{"H101 DDR"}="1H10 DDR"; $resn_exch{"H102 DDR"}="2H10 DDR";
$resn_exch{"H103 DDR"}="3H10 DDR"; $resn_exch{"HC91 DDT"}="1HC9 DDT"; $resn_exch{"HC92 DDT"}="2HC9 DDT";
$resn_exch{"HC93 DDT"}="3HC9 DDT"; $resn_exch{"H101 DDT"}="1H10 DDT"; $resn_exch{"H102 DDT"}="2H10 DDT";
$resn_exch{"H103 DDT"}="3H10 DDT"; $resn_exch{"H131 DDT"}="1H13 DDT"; $resn_exch{"H132 DDT"}="2H13 DDT";
$resn_exch{"H2'1 DDT"}="1H2* DDT"; $resn_exch{"H3'1 DDT"}="1H3* DDT"; $resn_exch{"H4'1 DDT"}="1H4* DDT";
$resn_exch{"H6'1 DDT"}="1H6* DDT"; $resn_exch{"H7'1 DDT"}="1H7* DDT"; $resn_exch{"H8'1 DDT"}="1H8* DDT";
$resn_exch{"H9'1 DDT"}="1H9* DDT"; $resn_exch{"H9'2 DDT"}="2H9* DDT"; $resn_exch{"H9'3 DDT"}="3H9* DDT";
$resn_exch{" HB1 DDT"}="1H** DDT"; $resn_exch{" HB2 DDT"}="2H** DDT"; $resn_exch{" HB3 DDT"}="3H** DDT";
$resn_exch{" H1' DDU"}=" H1* DDU"; $resn_exch{"H2'1 DDU"}="1H2* DDU"; $resn_exch{"H2'2 DDU"}="2H2* DDU";
$resn_exch{" H3' DDU"}=" H3* DDU"; $resn_exch{" H4' DDU"}=" H4* DDU"; $resn_exch{"HO3' DDU"}="*HO3 DDU";
$resn_exch{"H5'1 DDU"}="1H5* DDU"; $resn_exch{"H5'2 DDU"}="2H5* DDU"; $resn_exch{"H5'3 DDU"}="3H5* DDU";
$resn_exch{"HOP2 DDX"}="2HOP DDX"; $resn_exch{"HOP3 DDX"}="3HOP DDX"; $resn_exch{" H5' DDX"}="1H5* DDX";
$resn_exch{"H5'' DDX"}="2H5* DDX"; $resn_exch{" H4' DDX"}=" H4* DDX"; $resn_exch{" H1' DDX"}=" H1* DDX";
$resn_exch{" H3' DDX"}=" H3* DDX"; $resn_exch{" H2' DDX"}=" H2* DDX"; $resn_exch{"HO1' DDX"}="HO1* DDX";
$resn_exch{"H5'1 DDY"}="1H5* DDY"; $resn_exch{"H5'2 DDY"}="2H5* DDY"; $resn_exch{" H4' DDY"}=" H4* DDY";
$resn_exch{"H3'1 DDY"}="1H3* DDY"; $resn_exch{"H3'2 DDY"}="2H3* DDY"; $resn_exch{"H2'1 DDY"}="1H2* DDY";
$resn_exch{"H2'2 DDY"}="2H2* DDY"; $resn_exch{" H1' DDY"}=" H1* DDY"; $resn_exch{"HN41 DDY"}="1HN4 DDY";
$resn_exch{"HN42 DDY"}="2HN4 DDY"; $resn_exch{"HAA1 DE1"}="1HAA DE1"; $resn_exch{"HAA2 DE1"}="2HAA DE1";
$resn_exch{"HAA3 DE1"}="3HAA DE1"; $resn_exch{"HAC1 DE1"}="1HAC DE1"; $resn_exch{"HAC2 DE1"}="2HAC DE1";
$resn_exch{"HAE1 DE1"}="1HAE DE1"; $resn_exch{"HAE2 DE1"}="2HAE DE1"; $resn_exch{"HAG1 DE1"}="1HAG DE1";
$resn_exch{"HAG2 DE1"}="2HAG DE1"; $resn_exch{"HAI1 DE1"}="1HAI DE1"; $resn_exch{"HAI2 DE1"}="2HAI DE1";
$resn_exch{"HAK1 DE1"}="1HAK DE1"; $resn_exch{"HAK2 DE1"}="2HAK DE1"; $resn_exch{"HAJ1 DE1"}="1HAJ DE1";
$resn_exch{"HAJ2 DE1"}="2HAJ DE1"; $resn_exch{"HAH1 DE1"}="1HAH DE1"; $resn_exch{"HAH2 DE1"}="2HAH DE1";
$resn_exch{"HAF1 DE1"}="1HAF DE1"; $resn_exch{"HAF2 DE1"}="2HAF DE1"; $resn_exch{"HAD1 DE1"}="1HAD DE1";
$resn_exch{"HAD2 DE1"}="2HAD DE1"; $resn_exch{"HN61 DEA"}="1HN6 DEA"; $resn_exch{"HN62 DEA"}="2HN6 DEA";
$resn_exch{"H1'1 DEA"}="1H1* DEA"; $resn_exch{"H1'2 DEA"}="2H1* DEA"; $resn_exch{" H2' DEA"}=" H2* DEA";
$resn_exch{"HO'2 DEA"}="2HO* DEA"; $resn_exch{" H3' DEA"}=" H3* DEA"; $resn_exch{"HO'3 DEA"}="3HO* DEA";
$resn_exch{" H71 DEB"}="1H7  DEB"; $resn_exch{" H72 DEB"}="2H7  DEB"; $resn_exch{"H141 DEB"}="1H14 DEB";
$resn_exch{"H142 DEB"}="2H14 DEB"; $resn_exch{"H151 DEB"}="1H15 DEB"; $resn_exch{"H152 DEB"}="2H15 DEB";
$resn_exch{"H153 DEB"}="3H15 DEB"; $resn_exch{"H181 DEB"}="1H18 DEB"; $resn_exch{"H182 DEB"}="2H18 DEB";
$resn_exch{"H183 DEB"}="3H18 DEB"; $resn_exch{"H201 DEB"}="1H20 DEB"; $resn_exch{"H202 DEB"}="2H20 DEB";
$resn_exch{"H203 DEB"}="3H20 DEB"; $resn_exch{"H221 DEB"}="1H22 DEB"; $resn_exch{"H222 DEB"}="2H22 DEB";
$resn_exch{"H223 DEB"}="3H22 DEB"; $resn_exch{"H231 DEB"}="1H23 DEB"; $resn_exch{"H232 DEB"}="2H23 DEB";
$resn_exch{"H233 DEB"}="3H23 DEB"; $resn_exch{"H251 DEB"}="1H25 DEB"; $resn_exch{"H252 DEB"}="2H25 DEB";
$resn_exch{"H253 DEB"}="3H25 DEB"; $resn_exch{"H271 DEB"}="1H27 DEB"; $resn_exch{"H272 DEB"}="2H27 DEB";
$resn_exch{"H273 DEB"}="3H27 DEB"; $resn_exch{" H21 DEC"}="1H2  DEC"; $resn_exch{" H22 DEC"}="2H2  DEC";
$resn_exch{" H31 DEC"}="1H3  DEC"; $resn_exch{" H32 DEC"}="2H3  DEC"; $resn_exch{" H41 DEC"}="1H4  DEC";
$resn_exch{" H42 DEC"}="2H4  DEC"; $resn_exch{" H51 DEC"}="1H5  DEC"; $resn_exch{" H52 DEC"}="2H5  DEC";
$resn_exch{" H61 DEC"}="1H6  DEC"; $resn_exch{" H62 DEC"}="2H6  DEC"; $resn_exch{" H71 DEC"}="1H7  DEC";
$resn_exch{" H72 DEC"}="2H7  DEC"; $resn_exch{" H81 DEC"}="1H8  DEC"; $resn_exch{" H82 DEC"}="2H8  DEC";
$resn_exch{" H91 DEC"}="1H9  DEC"; $resn_exch{" H92 DEC"}="2H9  DEC"; $resn_exch{"H101 DED"}="1H10 DED";
$resn_exch{"H102 DED"}="2H10 DED"; $resn_exch{"H111 DED"}="1H11 DED"; $resn_exch{"H112 DED"}="2H11 DED";
$resn_exch{"H131 DED"}="1H13 DED"; $resn_exch{"H132 DED"}="2H13 DED"; $resn_exch{"H133 DED"}="3H13 DED";
$resn_exch{"H141 DED"}="1H14 DED"; $resn_exch{"H142 DED"}="2H14 DED"; $resn_exch{"H143 DED"}="3H14 DED";
$resn_exch{"H111 DEE"}="1H11 DEE"; $resn_exch{"H112 DEE"}="2H11 DEE"; $resn_exch{"H121 DEE"}="1H12 DEE";
$resn_exch{"H122 DEE"}="2H12 DEE"; $resn_exch{"H123 DEE"}="3H12 DEE"; $resn_exch{" H91 DEE"}="1H9  DEE";
$resn_exch{" H92 DEE"}="2H9  DEE"; $resn_exch{" H93 DEE"}="3H9  DEE"; $resn_exch{"H101 DEE"}="1H10 DEE";
$resn_exch{"H102 DEE"}="2H10 DEE"; $resn_exch{"H103 DEE"}="3H10 DEE"; $resn_exch{"H121 DEF"}="1H12 DEF";
$resn_exch{"H122 DEF"}="2H12 DEF"; $resn_exch{"H131 DEF"}="1H13 DEF"; $resn_exch{"H132 DEF"}="2H13 DEF";
$resn_exch{"H12N DEF"}="NH12 DEF"; $resn_exch{"H151 DEF"}="1H15 DEF"; $resn_exch{"H152 DEF"}="2H15 DEF";
$resn_exch{"H161 DEF"}="1H16 DEF"; $resn_exch{"H162 DEF"}="2H16 DEF"; $resn_exch{"H181 DEF"}="1H18 DEF";
$resn_exch{"H182 DEF"}="2H18 DEF"; $resn_exch{"H171 DEF"}="1H17 DEF"; $resn_exch{"H172 DEF"}="2H17 DEF";
$resn_exch{"H191 DEF"}="1H19 DEF"; $resn_exch{"H192 DEF"}="2H19 DEF"; $resn_exch{"H221 DEF"}="1H22 DEF";
$resn_exch{"H222 DEF"}="2H22 DEF"; $resn_exch{"H231 DEF"}="1H23 DEF"; $resn_exch{"H232 DEF"}="2H23 DEF";
$resn_exch{"H22N DEF"}="NH22 DEF"; $resn_exch{"H251 DEF"}="1H25 DEF"; $resn_exch{"H252 DEF"}="2H25 DEF";
$resn_exch{"H261 DEF"}="1H26 DEF"; $resn_exch{"H262 DEF"}="2H26 DEF"; $resn_exch{"H271 DEF"}="1H27 DEF";
$resn_exch{"H272 DEF"}="2H27 DEF"; $resn_exch{"H281 DEF"}="1H28 DEF"; $resn_exch{"H282 DEF"}="2H28 DEF";
$resn_exch{"H291 DEF"}="1H29 DEF"; $resn_exch{"H292 DEF"}="2H29 DEF"; $resn_exch{"H321 DEF"}="1H32 DEF";
$resn_exch{"H322 DEF"}="2H32 DEF"; $resn_exch{"H391 DEF"}="1H39 DEF"; $resn_exch{"H392 DEF"}="2H39 DEF";
$resn_exch{"H331 DEF"}="1H33 DEF"; $resn_exch{"H332 DEF"}="2H33 DEF"; $resn_exch{"H333 DEF"}="3H33 DEF";
$resn_exch{"H401 DEF"}="1H40 DEF"; $resn_exch{"H402 DEF"}="2H40 DEF"; $resn_exch{"H411 DEF"}="1H41 DEF";
$resn_exch{"H412 DEF"}="2H41 DEF"; $resn_exch{"H421 DEF"}="1H42 DEF"; $resn_exch{"H422 DEF"}="2H42 DEF";
$resn_exch{"H431 DEF"}="1H43 DEF"; $resn_exch{"H432 DEF"}="2H43 DEF"; $resn_exch{"H44N DEF"}="NH44 DEF";
$resn_exch{"H491 DEF"}="1H49 DEF"; $resn_exch{"H492 DEF"}="2H49 DEF"; $resn_exch{"H493 DEF"}="3H49 DEF";
$resn_exch{"H3'1 DEG"}="1H3' DEG"; $resn_exch{"H3'2 DEG"}="2H3' DEG"; $resn_exch{"H4'1 DEG"}="1H4' DEG";
$resn_exch{"H4'2 DEG"}="2H4' DEG"; $resn_exch{"H4'3 DEG"}="3H4' DEG"; $resn_exch{"H1'1 DEG"}="1H1' DEG";
$resn_exch{"H1'2 DEG"}="2H1' DEG"; $resn_exch{"H2'1 DEG"}="1H2' DEG"; $resn_exch{"H2'2 DEG"}="2H2' DEG";
$resn_exch{"H6C1 DEG"}="1H6C DEG"; $resn_exch{"H6C2 DEG"}="2H6C DEG"; $resn_exch{"HC41 DEL"}="1HC4 DEL";
$resn_exch{"HC42 DEL"}="2HC4 DEL"; $resn_exch{" H61 DEL"}="1H6  DEL"; $resn_exch{" H62 DEL"}="2H6  DEL";
$resn_exch{"H6'1 DEL"}="1H6' DEL"; $resn_exch{"H6'2 DEL"}="2H6' DEL"; $resn_exch{"HO1' DEL"}="'HO1 DEL";
$resn_exch{"HO2' DEL"}="'HO2 DEL"; $resn_exch{"HO3' DEL"}="'HO3 DEL"; $resn_exch{"HO6' DEL"}="'HO6 DEL";
$resn_exch{"HCM1 DEM"}="1HCM DEM"; $resn_exch{"HCM2 DEM"}="2HCM DEM"; $resn_exch{"HC11 DEM"}="1HC1 DEM";
$resn_exch{"HC12 DEM"}="2HC1 DEM"; $resn_exch{"HC21 DEM"}="1HC2 DEM"; $resn_exch{"HC22 DEM"}="2HC2 DEM";
$resn_exch{"HC31 DEM"}="1HC3 DEM"; $resn_exch{"HC32 DEM"}="2HC3 DEM"; $resn_exch{"HC41 DEM"}="1HC4 DEM";
$resn_exch{"HC42 DEM"}="2HC4 DEM"; $resn_exch{"HC51 DEM"}="1HC5 DEM"; $resn_exch{"HC52 DEM"}="2HC5 DEM";
$resn_exch{"HC61 DEM"}="1HC6 DEM"; $resn_exch{"HC62 DEM"}="2HC6 DEM"; $resn_exch{"HC71 DEM"}="1HC7 DEM";
$resn_exch{"HC72 DEM"}="2HC7 DEM"; $resn_exch{"HC81 DEM"}="1HC8 DEM"; $resn_exch{"HC82 DEM"}="2HC8 DEM";
$resn_exch{"HC91 DEM"}="1HC9 DEM"; $resn_exch{"HC92 DEM"}="2HC9 DEM"; $resn_exch{"H101 DEM"}="1H10 DEM";
$resn_exch{"H102 DEM"}="2H10 DEM"; $resn_exch{"H103 DEM"}="3H10 DEM"; $resn_exch{" H11 DEN"}="1H1  DEN";
$resn_exch{" H12 DEN"}="2H1  DEN"; $resn_exch{" H91 DEO"}="1H9  DEO"; $resn_exch{" H92 DEO"}="2H9  DEO";
$resn_exch{"H101 DEO"}="1H10 DEO"; $resn_exch{"H102 DEO"}="2H10 DEO"; $resn_exch{"H121 DEO"}="1H12 DEO";
$resn_exch{"H122 DEO"}="2H12 DEO"; $resn_exch{"H261 DEO"}="1H26 DEO"; $resn_exch{"H262 DEO"}="2H26 DEO";
$resn_exch{"H271 DEO"}="1H27 DEO"; $resn_exch{"H272 DEO"}="2H27 DEO"; $resn_exch{"H273 DEO"}="3H27 DEO";
$resn_exch{" H11 DEP"}="1H1  DEP"; $resn_exch{" H12 DEP"}="2H1  DEP"; $resn_exch{" H21 DEP"}="1H2  DEP";
$resn_exch{" H22 DEP"}="2H2  DEP"; $resn_exch{" H23 DEP"}="3H2  DEP"; $resn_exch{" H31 DEP"}="1H3  DEP";
$resn_exch{" H32 DEP"}="2H3  DEP"; $resn_exch{" H41 DEP"}="1H4  DEP"; $resn_exch{" H42 DEP"}="2H4  DEP";
$resn_exch{" H43 DEP"}="3H4  DEP"; $resn_exch{" H11 DEQ"}="1H1  DEQ"; $resn_exch{" H21 DEQ"}="1H2  DEQ";
$resn_exch{" H31 DEQ"}="1H3  DEQ"; $resn_exch{" H41 DEQ"}="1H4  DEQ"; $resn_exch{" H81 DEQ"}="1H8  DEQ";
$resn_exch{"H131 DEQ"}="1H13 DEQ"; $resn_exch{"H151 DEQ"}="1H15 DEQ"; $resn_exch{"H161 DEQ"}="1H16 DEQ";
$resn_exch{"H171 DEQ"}="1H17 DEQ"; $resn_exch{"H181 DEQ"}="1H18 DEQ"; $resn_exch{"H191 DEQ"}="1H19 DEQ";
$resn_exch{"H192 DEQ"}="2H19 DEQ"; $resn_exch{"H201 DEQ"}="1H20 DEQ"; $resn_exch{"H202 DEQ"}="2H20 DEQ";
$resn_exch{"H211 DEQ"}="1H21 DEQ"; $resn_exch{"H212 DEQ"}="2H21 DEQ"; $resn_exch{"H221 DEQ"}="1H22 DEQ";
$resn_exch{"H222 DEQ"}="2H22 DEQ"; $resn_exch{"H231 DEQ"}="1H23 DEQ"; $resn_exch{"H232 DEQ"}="2H23 DEQ";
$resn_exch{"H241 DEQ"}="1H24 DEQ"; $resn_exch{"H242 DEQ"}="2H24 DEQ"; $resn_exch{"H251 DEQ"}="1H25 DEQ";
$resn_exch{"H252 DEQ"}="2H25 DEQ"; $resn_exch{"H261 DEQ"}="1H26 DEQ"; $resn_exch{"H262 DEQ"}="2H26 DEQ";
$resn_exch{"H271 DEQ"}="1H27 DEQ"; $resn_exch{"H272 DEQ"}="2H27 DEQ"; $resn_exch{"H281 DEQ"}="1H28 DEQ";
$resn_exch{"H282 DEQ"}="2H28 DEQ"; $resn_exch{"H301 DEQ"}="1H30 DEQ"; $resn_exch{"H302 DEQ"}="2H30 DEQ";
$resn_exch{"H303 DEQ"}="3H30 DEQ"; $resn_exch{"H291 DEQ"}="1H29 DEQ"; $resn_exch{"H292 DEQ"}="2H29 DEQ";
$resn_exch{"H293 DEQ"}="3H29 DEQ"; $resn_exch{"HN41 DEQ"}="1HN4 DEQ"; $resn_exch{"HN42 DEQ"}="2HN4 DEQ";
$resn_exch{"HN31 DEQ"}="1HN3 DEQ"; $resn_exch{"HN32 DEQ"}="2HN3 DEQ"; $resn_exch{" H41 DER"}="1H4  DER";
$resn_exch{" H42 DER"}="2H4  DER"; $resn_exch{"HP91 DES"}="1HP9 DES"; $resn_exch{"HP92 DES"}="2HP9 DES";
$resn_exch{"HP93 DES"}="3HP9 DES"; $resn_exch{"HP81 DES"}="1HP8 DES"; $resn_exch{"HP82 DES"}="2HP8 DES";
$resn_exch{"HOP3 DES"}="3HOP DES"; $resn_exch{" HO3 DES"}="3HO  DES"; $resn_exch{" H81 DES"}="1H8  DES";
$resn_exch{" H82 DES"}="2H8  DES"; $resn_exch{" H91 DES"}="1H9  DES"; $resn_exch{" H92 DES"}="2H9  DES";
$resn_exch{" H93 DES"}="3H9  DES"; $resn_exch{"HM11 DET"}="1HM1 DET"; $resn_exch{"HM12 DET"}="2HM1 DET";
$resn_exch{"HM13 DET"}="3HM1 DET"; $resn_exch{"HM21 DET"}="1HM2 DET"; $resn_exch{"HM22 DET"}="2HM2 DET";
$resn_exch{"HM23 DET"}="3HM2 DET"; $resn_exch{" H11 DET"}="1H1  DET"; $resn_exch{" H12 DET"}="2H1  DET";
$resn_exch{" H21 DET"}="1H2  DET"; $resn_exch{" H22 DET"}="2H2  DET"; $resn_exch{" H31 DET"}="1H3  DET";
$resn_exch{" H32 DET"}="2H3  DET"; $resn_exch{" H41 DET"}="1H4  DET"; $resn_exch{" H42 DET"}="2H4  DET";
$resn_exch{" H51 DET"}="1H5  DET"; $resn_exch{" H52 DET"}="2H5  DET"; $resn_exch{" H61 DET"}="1H6  DET";
$resn_exch{" H62 DET"}="2H6  DET"; $resn_exch{" H71 DET"}="1H7  DET"; $resn_exch{" H72 DET"}="2H7  DET";
$resn_exch{" H81 DET"}="1H8  DET"; $resn_exch{" H82 DET"}="2H8  DET"; $resn_exch{" H91 DET"}="1H9  DET";
$resn_exch{" H92 DET"}="2H9  DET"; $resn_exch{"H101 DET"}="1H10 DET"; $resn_exch{"H102 DET"}="2H10 DET";
$resn_exch{"H111 DET"}="1H11 DET"; $resn_exch{"H112 DET"}="2H11 DET"; $resn_exch{"H113 DET"}="3H11 DET";
$resn_exch{"HMB1 DEU"}="1HMB DEU"; $resn_exch{"HMB2 DEU"}="2HMB DEU"; $resn_exch{"HMB3 DEU"}="3HMB DEU";
$resn_exch{"HMC1 DEU"}="1HMC DEU"; $resn_exch{"HMC2 DEU"}="2HMC DEU"; $resn_exch{"HMC3 DEU"}="3HMC DEU";
$resn_exch{"HMA1 DEU"}="1HMA DEU"; $resn_exch{"HMA2 DEU"}="2HMA DEU"; $resn_exch{"HMA3 DEU"}="3HMA DEU";
$resn_exch{"HMD1 DEU"}="1HMD DEU"; $resn_exch{"HMD2 DEU"}="2HMD DEU"; $resn_exch{"HMD3 DEU"}="3HMD DEU";
$resn_exch{"HAA1 DEU"}="1HAA DEU"; $resn_exch{"HAA2 DEU"}="2HAA DEU"; $resn_exch{"HBA1 DEU"}="1HBA DEU";
$resn_exch{"HBA2 DEU"}="2HBA DEU"; $resn_exch{"HAD1 DEU"}="1HAD DEU"; $resn_exch{"HAD2 DEU"}="2HAD DEU";
$resn_exch{"HBD1 DEU"}="1HBD DEU"; $resn_exch{"HBD2 DEU"}="2HBD DEU"; $resn_exch{"H211 DEX"}="1H21 DEX";
$resn_exch{"H221 DEX"}="1H22 DEX"; $resn_exch{"H511 DEX"}="1H51 DEX"; $resn_exch{"H521 DEX"}="1H52 DEX";
$resn_exch{"H811 DEX"}="1H81 DEX"; $resn_exch{"H821 DEX"}="1H82 DEX"; $resn_exch{"H831 DEX"}="1H83 DEX";
$resn_exch{"H911 DEX"}="1H91 DEX"; $resn_exch{"H921 DEX"}="1H92 DEX"; $resn_exch{"H931 DEX"}="1H93 DEX";
$resn_exch{"H112 DEX"}="2H11 DEX"; $resn_exch{"H122 DEX"}="2H12 DEX"; $resn_exch{"H212 DEX"}="2H21 DEX";
$resn_exch{"H222 DEX"}="2H22 DEX"; $resn_exch{"H232 DEX"}="2H23 DEX"; $resn_exch{"H4C1 DEZ"}="1H4C DEZ";
$resn_exch{"H4C2 DEZ"}="2H4C DEZ"; $resn_exch{"H111 DF1"}="1H11 DF1"; $resn_exch{"H112 DF1"}="2H11 DF1";
$resn_exch{"H141 DF1"}="1H14 DF1"; $resn_exch{"H142 DF1"}="2H14 DF1"; $resn_exch{"H143 DF1"}="3H14 DF1";
$resn_exch{"H261 DF2"}="1H26 DF2"; $resn_exch{"H262 DF2"}="2H26 DF2"; $resn_exch{"H8C1 DF2"}="1H8C DF2";
$resn_exch{"H8C2 DF2"}="2H8C DF2"; $resn_exch{"HC71 DFB"}="1HC7 DFB"; $resn_exch{"HC72 DFB"}="2HC7 DFB";
$resn_exch{"HOP3 DFC"}="3HOP DFC"; $resn_exch{"HOP2 DFC"}="2HOP DFC"; $resn_exch{" H5' DFC"}="1H5* DFC";
$resn_exch{"H5'' DFC"}="2H5* DFC"; $resn_exch{" H4' DFC"}=" H4* DFC"; $resn_exch{" H3' DFC"}=" H3* DFC";
$resn_exch{"HO3' DFC"}="H3T  DFC"; $resn_exch{" H2' DFC"}="1H2* DFC"; $resn_exch{"H2'' DFC"}="2H2* DFC";
$resn_exch{" H1' DFC"}=" H1* DFC"; $resn_exch{" H41 DFC"}="1H4  DFC"; $resn_exch{" H42 DFC"}="2H4  DFC";
$resn_exch{"H111 DFD"}="1H11 DFD"; $resn_exch{"H112 DFD"}="2H11 DFD"; $resn_exch{"H113 DFD"}="3H11 DFD";
$resn_exch{"H101 DFD"}="1H10 DFD"; $resn_exch{"H102 DFD"}="2H10 DFD"; $resn_exch{" H91 DFD"}="1H9  DFD";
$resn_exch{" H92 DFD"}="2H9  DFD"; $resn_exch{" H81 DFD"}="1H8  DFD"; $resn_exch{" H82 DFD"}="2H8  DFD";
$resn_exch{" H71 DFD"}="1H7  DFD"; $resn_exch{" H72 DFD"}="2H7  DFD"; $resn_exch{" H61 DFD"}="1H6  DFD";
$resn_exch{" H62 DFD"}="2H6  DFD"; $resn_exch{" H51 DFD"}="1H5  DFD"; $resn_exch{" H52 DFD"}="2H5  DFD";
$resn_exch{" H41 DFD"}="1H4  DFD"; $resn_exch{" H42 DFD"}="2H4  DFD"; $resn_exch{" H31 DFD"}="1H3  DFD";
$resn_exch{" H32 DFD"}="2H3  DFD"; $resn_exch{" H21 DFD"}="1H2  DFD"; $resn_exch{" H22 DFD"}="2H2  DFD";
$resn_exch{"HAA1 DFE"}="1HAA DFE"; $resn_exch{"HAA2 DFE"}="2HAA DFE"; $resn_exch{"HAA3 DFE"}="3HAA DFE";
$resn_exch{"HAG1 DFE"}="1HAG DFE"; $resn_exch{"HAG2 DFE"}="2HAG DFE"; $resn_exch{"HOP3 DFG"}="3HOP DFG";
$resn_exch{"HOP2 DFG"}="2HOP DFG"; $resn_exch{" H5' DFG"}="1H5* DFG"; $resn_exch{"H5'' DFG"}="2H5* DFG";
$resn_exch{" H4' DFG"}=" H4* DFG"; $resn_exch{" H3' DFG"}=" H3* DFG"; $resn_exch{"HO3' DFG"}="H3T  DFG";
$resn_exch{" H2' DFG"}="1H2* DFG"; $resn_exch{"H2'' DFG"}="2H2* DFG"; $resn_exch{" H1' DFG"}=" H1* DFG";
$resn_exch{" H21 DFG"}="1H2  DFG"; $resn_exch{" H22 DFG"}="2H2  DFG"; $resn_exch{" H   DFI"}="1HN  DFI";
$resn_exch{" H2  DFI"}="2HN  DFI"; $resn_exch{" HB2 DFI"}="1HB  DFI"; $resn_exch{" HB3 DFI"}="2HB  DFI";
$resn_exch{"HD11 DFI"}="1HD1 DFI"; $resn_exch{"HD12 DFI"}="2HD1 DFI"; $resn_exch{"HD13 DFI"}="3HD1 DFI";
$resn_exch{"HD21 DFI"}="1HD2 DFI"; $resn_exch{"HD22 DFI"}="2HD2 DFI"; $resn_exch{"HD23 DFI"}="3HD2 DFI";
$resn_exch{"H201 DFJ"}="1H20 DFJ"; $resn_exch{"H202 DFJ"}="2H20 DFJ"; $resn_exch{"H351 DFJ"}="1H35 DFJ";
$resn_exch{"H352 DFJ"}="2H35 DFJ"; $resn_exch{"H401 DFJ"}="1H40 DFJ"; $resn_exch{"H402 DFJ"}="2H40 DFJ";
$resn_exch{"H7C1 DFJ"}="1H7C DFJ"; $resn_exch{"H7C2 DFJ"}="2H7C DFJ"; $resn_exch{" H81 DFL"}="1H8  DFL";
$resn_exch{" H82 DFL"}="2H8  DFL"; $resn_exch{"H211 DFM"}="1H21 DFM"; $resn_exch{"H212 DFM"}="2H21 DFM";
$resn_exch{"H271 DFM"}="1H27 DFM"; $resn_exch{"H272 DFM"}="2H27 DFM"; $resn_exch{"H4C1 DFM"}="1H4C DFM";
$resn_exch{"H4C2 DFM"}="2H4C DFM"; $resn_exch{"H711 DFM"}="1H71 DFM"; $resn_exch{"H712 DFM"}="2H71 DFM";
$resn_exch{"H713 DFM"}="3H71 DFM"; $resn_exch{"H281 DFN"}="1H28 DFN"; $resn_exch{"H282 DFN"}="2H28 DFN";
$resn_exch{"H283 DFN"}="3H28 DFN"; $resn_exch{"H261 DFN"}="1H26 DFN"; $resn_exch{"H262 DFN"}="2H26 DFN";
$resn_exch{"H401 DFN"}="1H40 DFN"; $resn_exch{"H402 DFN"}="2H40 DFN"; $resn_exch{" H   DFO"}="1HN  DFO";
$resn_exch{" H2  DFO"}="2HN  DFO"; $resn_exch{" HB2 DFO"}="1HB  DFO"; $resn_exch{" HB3 DFO"}="2HB  DFO";
$resn_exch{"HD11 DFO"}="1HD1 DFO"; $resn_exch{"HD12 DFO"}="2HD1 DFO"; $resn_exch{"HD13 DFO"}="3HD1 DFO";
$resn_exch{"HD21 DFO"}="1HD2 DFO"; $resn_exch{"HD22 DFO"}="2HD2 DFO"; $resn_exch{"HD23 DFO"}="3HD2 DFO";
$resn_exch{" H21 DFP"}="1H2  DFP"; $resn_exch{" H22 DFP"}="2H2  DFP"; $resn_exch{" H23 DFP"}="3H2  DFP";
$resn_exch{" H31 DFP"}="1H3  DFP"; $resn_exch{" H32 DFP"}="2H3  DFP"; $resn_exch{" H33 DFP"}="3H3  DFP";
$resn_exch{"H2'1 DFP"}="1H2' DFP"; $resn_exch{"H2'2 DFP"}="2H2' DFP"; $resn_exch{"H2'3 DFP"}="3H2' DFP";
$resn_exch{"H3'1 DFP"}="1H3' DFP"; $resn_exch{"H3'2 DFP"}="2H3' DFP"; $resn_exch{"H3'3 DFP"}="3H3' DFP";
$resn_exch{" H51 DFR"}="1H5  DFR"; $resn_exch{" H52 DFR"}="2H5  DFR"; $resn_exch{" H61 DFR"}="1H6  DFR";
$resn_exch{" H62 DFR"}="2H6  DFR"; $resn_exch{" H71 DFR"}="1H7  DFR"; $resn_exch{" H72 DFR"}="2H7  DFR";
$resn_exch{" H73 DFR"}="3H7  DFR"; $resn_exch{"HOP2 DFT"}="2HOP DFT"; $resn_exch{"HOP3 DFT"}="3HOP DFT";
$resn_exch{" H5' DFT"}="1H5* DFT"; $resn_exch{"H5'' DFT"}="2H5* DFT"; $resn_exch{" H4' DFT"}=" H4* DFT";
$resn_exch{" H3' DFT"}=" H3* DFT"; $resn_exch{"HO3' DFT"}="*HO3 DFT"; $resn_exch{" H2' DFT"}="1H2* DFT";
$resn_exch{"H2'' DFT"}="2H2* DFT"; $resn_exch{" H1' DFT"}=" H1* DFT"; $resn_exch{" H71 DFT"}="1H5M DFT";
$resn_exch{" H72 DFT"}="2H5M DFT"; $resn_exch{" H73 DFT"}="3H5M DFT"; $resn_exch{" H11 DFU"}="1H1  DFU";
$resn_exch{" H12 DFU"}="2H1  DFU"; $resn_exch{" H61 DFU"}="1H6  DFU"; $resn_exch{" H62 DFU"}="2H6  DFU";
$resn_exch{" H63 DFU"}="3H6  DFU"; $resn_exch{" H81 DFV"}="1H8  DFV"; $resn_exch{" H82 DFV"}="2H8  DFV";
$resn_exch{"H8C1 DFW"}="1H8C DFW"; $resn_exch{"H8C2 DFW"}="2H8C DFW"; $resn_exch{" H11 DFX"}="1H1  DFX";
$resn_exch{" H12 DFX"}="2H1  DFX"; $resn_exch{" H51 DFX"}="1H5  DFX"; $resn_exch{" H52 DFX"}="2H5  DFX";
$resn_exch{"H231 DFY"}="1H23 DFY"; $resn_exch{"H232 DFY"}="2H23 DFY"; $resn_exch{"H8C1 DFY"}="1H8C DFY";
$resn_exch{"H8C2 DFY"}="2H8C DFY"; $resn_exch{"HAA1 DFY"}="1HAA DFY"; $resn_exch{"HAA2 DFY"}="2HAA DFY";
$resn_exch{"HAA3 DFY"}="3HAA DFY"; $resn_exch{"H231 DFZ"}="1H23 DFZ"; $resn_exch{"H232 DFZ"}="2H23 DFZ";
$resn_exch{"H8C1 DFZ"}="1H8C DFZ"; $resn_exch{"H8C2 DFZ"}="2H8C DFZ"; $resn_exch{"HOP3  DG"}="3HOP  DG";
$resn_exch{"HOP2  DG"}="2HOP  DG"; $resn_exch{" H5'  DG"}="1H5*  DG"; $resn_exch{"H5''  DG"}="2H5*  DG";
$resn_exch{" H4'  DG"}=" H4*  DG"; $resn_exch{" H3'  DG"}=" H3*  DG"; $resn_exch{"HO3'  DG"}="H3T   DG";
$resn_exch{" H2'  DG"}="1H2*  DG"; $resn_exch{"H2''  DG"}="2H2*  DG"; $resn_exch{" H1'  DG"}=" H1*  DG";
$resn_exch{" H21  DG"}="1H2   DG"; $resn_exch{" H22  DG"}="2H2   DG"; $resn_exch{"HAA1 DG1"}="1HAA DG1";
$resn_exch{"HAA2 DG1"}="2HAA DG1"; $resn_exch{" H1' DG1"}=" H1* DG1"; $resn_exch{" H2' DG1"}=" H2* DG1";
$resn_exch{"HO2' DG1"}="HO2* DG1"; $resn_exch{" H3' DG1"}=" H3* DG1"; $resn_exch{"HO3' DG1"}="HO3* DG1";
$resn_exch{" H4' DG1"}=" H4* DG1"; $resn_exch{"H5'1 DG1"}="1H5* DG1"; $resn_exch{"H5'2 DG1"}="2H5* DG1";
$resn_exch{"HAY1 DG1"}="1HAY DG1"; $resn_exch{"HAY2 DG1"}="2HAY DG1"; $resn_exch{"HN61 DG1"}="1HN6 DG1";
$resn_exch{"HN62 DG1"}="2HN6 DG1"; $resn_exch{" H41 DG2"}="1H4  DG2"; $resn_exch{" H42 DG2"}="2H4  DG2";
$resn_exch{"HOG2 DG3"}="2HOG DG3"; $resn_exch{"HOG3 DG3"}="3HOG DG3"; $resn_exch{"HOB2 DG3"}="2HOB DG3";
$resn_exch{"HOA2 DG3"}="2HOA DG3"; $resn_exch{"H5'1 DG3"}="1H5* DG3"; $resn_exch{"H5'2 DG3"}="2H5* DG3";
$resn_exch{" H4' DG3"}=" H4* DG3"; $resn_exch{"H3'1 DG3"}="1H3* DG3"; $resn_exch{"H3'2 DG3"}="2H3* DG3";
$resn_exch{"H2'1 DG3"}="1H2* DG3"; $resn_exch{"H2'2 DG3"}="2H2* DG3"; $resn_exch{" H1' DG3"}=" H1* DG3";
$resn_exch{"HN21 DG3"}="1HN2 DG3"; $resn_exch{"HN22 DG3"}="2HN2 DG3"; $resn_exch{"HC11 DG6"}="1HC1 DG6";
$resn_exch{"HC12 DG6"}="2HC1 DG6"; $resn_exch{"HC21 DG6"}="1HC2 DG6"; $resn_exch{"HC22 DG6"}="2HC2 DG6";
$resn_exch{"HC61 DG6"}="1HC6 DG6"; $resn_exch{"HC62 DG6"}="2HC6 DG6"; $resn_exch{"HA21 DGA"}="1HA2 DGA";
$resn_exch{"HA22 DGA"}="2HA2 DGA"; $resn_exch{"HA31 DGA"}="1HA3 DGA"; $resn_exch{"HA32 DGA"}="2HA3 DGA";
$resn_exch{"HA41 DGA"}="1HA4 DGA"; $resn_exch{"HA42 DGA"}="2HA4 DGA"; $resn_exch{"HA51 DGA"}="1HA5 DGA";
$resn_exch{"HA52 DGA"}="2HA5 DGA"; $resn_exch{"HA61 DGA"}="1HA6 DGA"; $resn_exch{"HA62 DGA"}="2HA6 DGA";
$resn_exch{"HA71 DGA"}="1HA7 DGA"; $resn_exch{"HA72 DGA"}="2HA7 DGA"; $resn_exch{"HA81 DGA"}="1HA8 DGA";
$resn_exch{"HA82 DGA"}="2HA8 DGA"; $resn_exch{"HA91 DGA"}="1HA9 DGA"; $resn_exch{"HA92 DGA"}="2HA9 DGA";
$resn_exch{"HAT1 DGA"}="1HAT DGA"; $resn_exch{"HAT2 DGA"}="2HAT DGA"; $resn_exch{"HAE1 DGA"}="1HAE DGA";
$resn_exch{"HAE2 DGA"}="2HAE DGA"; $resn_exch{"HAW1 DGA"}="1HAW DGA"; $resn_exch{"HAW2 DGA"}="2HAW DGA";
$resn_exch{"HAH1 DGA"}="1HAH DGA"; $resn_exch{"HAH2 DGA"}="2HAH DGA"; $resn_exch{"HAF1 DGA"}="1HAF DGA";
$resn_exch{"HAF2 DGA"}="2HAF DGA"; $resn_exch{"HAN1 DGA"}="1HAN DGA"; $resn_exch{"HAN2 DGA"}="2HAN DGA";
$resn_exch{"HAS1 DGA"}="1HAS DGA"; $resn_exch{"HAS2 DGA"}="2HAS DGA"; $resn_exch{"HAV1 DGA"}="1HAV DGA";
$resn_exch{"HAV2 DGA"}="2HAV DGA"; $resn_exch{"HAG1 DGA"}="1HAG DGA"; $resn_exch{"HAG2 DGA"}="2HAG DGA";
$resn_exch{"HAG3 DGA"}="3HAG DGA"; $resn_exch{"HB21 DGA"}="1HB2 DGA"; $resn_exch{"HB22 DGA"}="2HB2 DGA";
$resn_exch{"HB31 DGA"}="1HB3 DGA"; $resn_exch{"HB32 DGA"}="2HB3 DGA"; $resn_exch{"HB41 DGA"}="1HB4 DGA";
$resn_exch{"HB42 DGA"}="2HB4 DGA"; $resn_exch{"HB51 DGA"}="1HB5 DGA"; $resn_exch{"HB52 DGA"}="2HB5 DGA";
$resn_exch{"HB61 DGA"}="1HB6 DGA"; $resn_exch{"HB62 DGA"}="2HB6 DGA"; $resn_exch{"HB71 DGA"}="1HB7 DGA";
$resn_exch{"HB72 DGA"}="2HB7 DGA"; $resn_exch{"HB81 DGA"}="1HB8 DGA"; $resn_exch{"HB82 DGA"}="2HB8 DGA";
$resn_exch{"HB91 DGA"}="1HB9 DGA"; $resn_exch{"HB92 DGA"}="2HB9 DGA"; $resn_exch{"HBT1 DGA"}="1HBT DGA";
$resn_exch{"HBT2 DGA"}="2HBT DGA"; $resn_exch{"HBE1 DGA"}="1HBE DGA"; $resn_exch{"HBE2 DGA"}="2HBE DGA";
$resn_exch{"HBW1 DGA"}="1HBW DGA"; $resn_exch{"HBW2 DGA"}="2HBW DGA"; $resn_exch{"HBH1 DGA"}="1HBH DGA";
$resn_exch{"HBH2 DGA"}="2HBH DGA"; $resn_exch{"HBF1 DGA"}="1HBF DGA"; $resn_exch{"HBF2 DGA"}="2HBF DGA";
$resn_exch{"HBN1 DGA"}="1HBN DGA"; $resn_exch{"HBN2 DGA"}="2HBN DGA"; $resn_exch{"HBS1 DGA"}="1HBS DGA";
$resn_exch{"HBS2 DGA"}="2HBS DGA"; $resn_exch{"HBV1 DGA"}="1HBV DGA"; $resn_exch{"HBV2 DGA"}="2HBV DGA";
$resn_exch{"HBG1 DGA"}="1HBG DGA"; $resn_exch{"HBG2 DGA"}="2HBG DGA"; $resn_exch{"HBG3 DGA"}="3HBG DGA";
$resn_exch{"HG11 DGA"}="1HG1 DGA"; $resn_exch{"HG12 DGA"}="2HG1 DGA"; $resn_exch{"HG31 DGA"}="1HG3 DGA";
$resn_exch{"HG32 DGA"}="2HG3 DGA"; $resn_exch{"HAU1 DGB"}="1HAU DGB"; $resn_exch{"HAU2 DGB"}="2HAU DGB";
$resn_exch{"HAS1 DGB"}="1HAS DGB"; $resn_exch{"HAS2 DGB"}="2HAS DGB"; $resn_exch{"HAT1 DGB"}="1HAT DGB";
$resn_exch{"HAT2 DGB"}="2HAT DGB"; $resn_exch{"HAR1 DGB"}="1HAR DGB"; $resn_exch{"HAR2 DGB"}="2HAR DGB";
$resn_exch{"HAQ1 DGB"}="1HAQ DGB"; $resn_exch{"HAQ2 DGB"}="2HAQ DGB"; $resn_exch{"HAO1 DGB"}="1HAO DGB";
$resn_exch{"HAO2 DGB"}="2HAO DGB"; $resn_exch{"HAN1 DGB"}="1HAN DGB"; $resn_exch{"HAN2 DGB"}="2HAN DGB";
$resn_exch{"HAP1 DGB"}="1HAP DGB"; $resn_exch{"HAP2 DGB"}="2HAP DGB"; $resn_exch{"HAC1 DGB"}="1HAC DGB";
$resn_exch{"HAC2 DGB"}="2HAC DGB"; $resn_exch{"HAD1 DGB"}="1HAD DGB"; $resn_exch{"HAD2 DGB"}="2HAD DGB";
$resn_exch{"HA21 DGD"}="1HA2 DGD"; $resn_exch{"HA22 DGD"}="2HA2 DGD"; $resn_exch{"HA31 DGD"}="1HA3 DGD";
$resn_exch{"HA32 DGD"}="2HA3 DGD"; $resn_exch{"HA41 DGD"}="1HA4 DGD"; $resn_exch{"HA42 DGD"}="2HA4 DGD";
$resn_exch{"HA51 DGD"}="1HA5 DGD"; $resn_exch{"HA52 DGD"}="2HA5 DGD"; $resn_exch{"HA61 DGD"}="1HA6 DGD";
$resn_exch{"HA62 DGD"}="2HA6 DGD"; $resn_exch{"HA71 DGD"}="1HA7 DGD"; $resn_exch{"HA72 DGD"}="2HA7 DGD";
$resn_exch{"HA81 DGD"}="1HA8 DGD"; $resn_exch{"HA82 DGD"}="2HA8 DGD"; $resn_exch{"HA91 DGD"}="1HA9 DGD";
$resn_exch{"HA92 DGD"}="2HA9 DGD"; $resn_exch{"HAT1 DGD"}="1HAT DGD"; $resn_exch{"HAT2 DGD"}="2HAT DGD";
$resn_exch{"HAE1 DGD"}="1HAE DGD"; $resn_exch{"HAE2 DGD"}="2HAE DGD"; $resn_exch{"HAW1 DGD"}="1HAW DGD";
$resn_exch{"HAW2 DGD"}="2HAW DGD"; $resn_exch{"HAH1 DGD"}="1HAH DGD"; $resn_exch{"HAH2 DGD"}="2HAH DGD";
$resn_exch{"HAF1 DGD"}="1HAF DGD"; $resn_exch{"HAF2 DGD"}="2HAF DGD"; $resn_exch{"HAN1 DGD"}="1HAN DGD";
$resn_exch{"HAN2 DGD"}="2HAN DGD"; $resn_exch{"HAS1 DGD"}="1HAS DGD"; $resn_exch{"HAS2 DGD"}="2HAS DGD";
$resn_exch{"HAV1 DGD"}="1HAV DGD"; $resn_exch{"HAV2 DGD"}="2HAV DGD"; $resn_exch{"HAG1 DGD"}="1HAG DGD";
$resn_exch{"HAG2 DGD"}="2HAG DGD"; $resn_exch{"HAG3 DGD"}="3HAG DGD"; $resn_exch{"HB21 DGD"}="1HB2 DGD";
$resn_exch{"HB22 DGD"}="2HB2 DGD"; $resn_exch{"HB31 DGD"}="1HB3 DGD"; $resn_exch{"HB32 DGD"}="2HB3 DGD";
$resn_exch{"HB41 DGD"}="1HB4 DGD"; $resn_exch{"HB42 DGD"}="2HB4 DGD"; $resn_exch{"HB51 DGD"}="1HB5 DGD";
$resn_exch{"HB52 DGD"}="2HB5 DGD"; $resn_exch{"HB61 DGD"}="1HB6 DGD"; $resn_exch{"HB62 DGD"}="2HB6 DGD";
$resn_exch{"HB71 DGD"}="1HB7 DGD"; $resn_exch{"HB72 DGD"}="2HB7 DGD"; $resn_exch{"HB81 DGD"}="1HB8 DGD";
$resn_exch{"HB82 DGD"}="2HB8 DGD"; $resn_exch{"HB91 DGD"}="1HB9 DGD"; $resn_exch{"HB92 DGD"}="2HB9 DGD";
$resn_exch{"HBT1 DGD"}="1HBT DGD"; $resn_exch{"HBT2 DGD"}="2HBT DGD"; $resn_exch{"HBE1 DGD"}="1HBE DGD";
$resn_exch{"HBE2 DGD"}="2HBE DGD"; $resn_exch{"HBW1 DGD"}="1HBW DGD"; $resn_exch{"HBW2 DGD"}="2HBW DGD";
$resn_exch{"HBH1 DGD"}="1HBH DGD"; $resn_exch{"HBH2 DGD"}="2HBH DGD"; $resn_exch{"HBF1 DGD"}="1HBF DGD";
$resn_exch{"HBF2 DGD"}="2HBF DGD"; $resn_exch{"HBN1 DGD"}="1HBN DGD"; $resn_exch{"HBN2 DGD"}="2HBN DGD";
$resn_exch{"HBS1 DGD"}="1HBS DGD"; $resn_exch{"HBS2 DGD"}="2HBS DGD"; $resn_exch{"HBV1 DGD"}="1HBV DGD";
$resn_exch{"HBV2 DGD"}="2HBV DGD"; $resn_exch{"HBG1 DGD"}="1HBG DGD"; $resn_exch{"HBG2 DGD"}="2HBG DGD";
$resn_exch{"HBG3 DGD"}="3HBG DGD"; $resn_exch{"HG11 DGD"}="1HG1 DGD"; $resn_exch{"HG12 DGD"}="2HG1 DGD";
$resn_exch{"HG31 DGD"}="1HG3 DGD"; $resn_exch{"HG32 DGD"}="2HG3 DGD"; $resn_exch{"HO2D DGD"}="DHO2 DGD";
$resn_exch{"HO3D DGD"}="DHO3 DGD"; $resn_exch{"HO4D DGD"}="DHO4 DGD"; $resn_exch{"HD61 DGD"}="1HD6 DGD";
$resn_exch{"HD62 DGD"}="2HD6 DGD"; $resn_exch{"HO2E DGD"}="EHO2 DGD"; $resn_exch{"HO3E DGD"}="EHO3 DGD";
$resn_exch{"HO4E DGD"}="EHO4 DGD"; $resn_exch{"HE61 DGD"}="1HE6 DGD"; $resn_exch{"HE62 DGD"}="2HE6 DGD";
$resn_exch{"HO5E DGD"}="EHO5 DGD"; $resn_exch{"H161 DGG"}="1H16 DGG"; $resn_exch{"H162 DGG"}="2H16 DGG";
$resn_exch{"H163 DGG"}="3H16 DGG"; $resn_exch{"H151 DGG"}="1H15 DGG"; $resn_exch{"H152 DGG"}="2H15 DGG";
$resn_exch{"H141 DGG"}="1H14 DGG"; $resn_exch{"H142 DGG"}="2H14 DGG"; $resn_exch{"H131 DGG"}="1H13 DGG";
$resn_exch{"H132 DGG"}="2H13 DGG"; $resn_exch{"H121 DGG"}="1H12 DGG"; $resn_exch{"H122 DGG"}="2H12 DGG";
$resn_exch{"H111 DGG"}="1H11 DGG"; $resn_exch{"H112 DGG"}="2H11 DGG"; $resn_exch{"H101 DGG"}="1H10 DGG";
$resn_exch{"H102 DGG"}="2H10 DGG"; $resn_exch{"HC91 DGG"}="1HC9 DGG"; $resn_exch{"HC92 DGG"}="2HC9 DGG";
$resn_exch{"HC81 DGG"}="1HC8 DGG"; $resn_exch{"HC82 DGG"}="2HC8 DGG"; $resn_exch{"HC71 DGG"}="1HC7 DGG";
$resn_exch{"HC72 DGG"}="2HC7 DGG"; $resn_exch{"HC61 DGG"}="1HC6 DGG"; $resn_exch{"HC62 DGG"}="2HC6 DGG";
$resn_exch{"HC51 DGG"}="1HC5 DGG"; $resn_exch{"HC52 DGG"}="2HC5 DGG"; $resn_exch{"HC41 DGG"}="1HC4 DGG";
$resn_exch{"HC42 DGG"}="2HC4 DGG"; $resn_exch{"HC31 DGG"}="1HC3 DGG"; $resn_exch{"HC32 DGG"}="2HC3 DGG";
$resn_exch{"HC21 DGG"}="1HC2 DGG"; $resn_exch{"HC22 DGG"}="2HC2 DGG"; $resn_exch{"H171 DGG"}="1H17 DGG";
$resn_exch{"H172 DGG"}="2H17 DGG"; $resn_exch{"H191 DGG"}="1H19 DGG"; $resn_exch{"H221 DGG"}="1H22 DGG";
$resn_exch{"H222 DGG"}="2H22 DGG"; $resn_exch{"H231 DGG"}="1H23 DGG"; $resn_exch{"H232 DGG"}="2H23 DGG";
$resn_exch{"H241 DGG"}="1H24 DGG"; $resn_exch{"H242 DGG"}="2H24 DGG"; $resn_exch{"H251 DGG"}="1H25 DGG";
$resn_exch{"H252 DGG"}="2H25 DGG"; $resn_exch{"H261 DGG"}="1H26 DGG"; $resn_exch{"H262 DGG"}="2H26 DGG";
$resn_exch{"H271 DGG"}="1H27 DGG"; $resn_exch{"H272 DGG"}="2H27 DGG"; $resn_exch{"H281 DGG"}="1H28 DGG";
$resn_exch{"H282 DGG"}="2H28 DGG"; $resn_exch{"H291 DGG"}="1H29 DGG"; $resn_exch{"H301 DGG"}="1H30 DGG";
$resn_exch{"H391 DGG"}="1H39 DGG"; $resn_exch{"H392 DGG"}="2H39 DGG"; $resn_exch{"H311 DGG"}="1H31 DGG";
$resn_exch{"H312 DGG"}="2H31 DGG"; $resn_exch{"H321 DGG"}="1H32 DGG"; $resn_exch{"H322 DGG"}="2H32 DGG";
$resn_exch{"H331 DGG"}="1H33 DGG"; $resn_exch{"H332 DGG"}="2H33 DGG"; $resn_exch{"H341 DGG"}="1H34 DGG";
$resn_exch{"H342 DGG"}="2H34 DGG"; $resn_exch{"H351 DGG"}="1H35 DGG"; $resn_exch{"H352 DGG"}="2H35 DGG";
$resn_exch{"H361 DGG"}="1H36 DGG"; $resn_exch{"H362 DGG"}="2H36 DGG"; $resn_exch{"H363 DGG"}="3H36 DGG";
$resn_exch{"H181 DGG"}="1H18 DGG"; $resn_exch{"H182 DGG"}="2H18 DGG"; $resn_exch{"HOP1 DGG"}="1HOP DGG";
$resn_exch{"H401 DGG"}="1H40 DGG"; $resn_exch{"H402 DGG"}="2H40 DGG"; $resn_exch{"H411 DGG"}="1H41 DGG";
$resn_exch{"H421 DGG"}="1H42 DGG"; $resn_exch{"H431 DGG"}="1H43 DGG"; $resn_exch{"H432 DGG"}="2H43 DGG";
$resn_exch{"H441 DGG"}="1H44 DGG"; $resn_exch{" HN1 DGH"}="1HN  DGH"; $resn_exch{" HA2 DGH"}="2HA  DGH";
$resn_exch{"HOB2 DGI"}="2HOB DGI"; $resn_exch{"HOB3 DGI"}="3HOB DGI"; $resn_exch{"HOA2 DGI"}="2HOA DGI";
$resn_exch{" H5' DGI"}="1H5* DGI"; $resn_exch{"H5'' DGI"}="2H5* DGI"; $resn_exch{" H4' DGI"}=" H4* DGI";
$resn_exch{" H3' DGI"}=" H3* DGI"; $resn_exch{"HO3' DGI"}="*HO3 DGI"; $resn_exch{" H2' DGI"}="1H2* DGI";
$resn_exch{"H2'' DGI"}="2H2* DGI"; $resn_exch{" H1' DGI"}=" H1* DGI"; $resn_exch{"HN21 DGI"}="1HN2 DGI";
$resn_exch{"HN22 DGI"}="2HN2 DGI"; $resn_exch{" H2  DGL"}=" HN2 DGL"; $resn_exch{" HB2 DGL"}="1HB  DGL";
$resn_exch{" HB3 DGL"}="2HB  DGL"; $resn_exch{" HG2 DGL"}="1HG  DGL"; $resn_exch{" HG3 DGL"}="2HG  DGL";
$resn_exch{" H61 DGM"}="1H6  DGM"; $resn_exch{" H62 DGM"}="2H6  DGM"; $resn_exch{" H2  DGN"}=" HN2 DGN";
$resn_exch{" HB2 DGN"}="1HB  DGN"; $resn_exch{" HB3 DGN"}="2HB  DGN"; $resn_exch{" HG2 DGN"}="1HG  DGN";
$resn_exch{" HG3 DGN"}="2HG  DGN"; $resn_exch{"HE21 DGN"}="1HE2 DGN"; $resn_exch{"HE22 DGN"}="2HE2 DGN";
$resn_exch{"HOP2 DGP"}="2HOP DGP"; $resn_exch{"HOP3 DGP"}="3HOP DGP"; $resn_exch{" H5' DGP"}="1H5* DGP";
$resn_exch{"H5'' DGP"}="2H5* DGP"; $resn_exch{" H4' DGP"}=" H4* DGP"; $resn_exch{" H3' DGP"}=" H3* DGP";
$resn_exch{"HO3' DGP"}="*HO3 DGP"; $resn_exch{" H2' DGP"}="1H2* DGP"; $resn_exch{"H2'' DGP"}="2H2* DGP";
$resn_exch{" H1' DGP"}=" H1* DGP"; $resn_exch{"HN21 DGP"}="1HN2 DGP"; $resn_exch{"HN22 DGP"}="2HN2 DGP";
$resn_exch{" H61 DGS"}="1H6  DGS"; $resn_exch{" H62 DGS"}="2H6  DGS"; $resn_exch{"HOG2 DGT"}="2HOG DGT";
$resn_exch{"HOG3 DGT"}="3HOG DGT"; $resn_exch{"HOB2 DGT"}="2HOB DGT"; $resn_exch{"HOA2 DGT"}="2HOA DGT";
$resn_exch{"H5'1 DGT"}="1H5* DGT"; $resn_exch{"H5'2 DGT"}="2H5* DGT"; $resn_exch{" H4' DGT"}=" H4* DGT";
$resn_exch{" H3' DGT"}=" H3* DGT"; $resn_exch{"HO3' DGT"}="*HO3 DGT"; $resn_exch{"H2'1 DGT"}="1H2* DGT";
$resn_exch{"H2'2 DGT"}="2H2* DGT"; $resn_exch{" H1' DGT"}=" H1* DGT"; $resn_exch{"HN21 DGT"}="1HN2 DGT";
$resn_exch{"HN22 DGT"}="2HN2 DGT"; $resn_exch{" H11 DGX"}="1H1  DGX"; $resn_exch{"H12A DGX"}="2H1  DGX";
$resn_exch{" H21 DGX"}="1H2  DGX"; $resn_exch{"H22A DGX"}="2H2  DGX"; $resn_exch{" H41 DGX"}="1H4  DGX";
$resn_exch{" H42 DGX"}="2H4  DGX"; $resn_exch{" H61 DGX"}="1H6  DGX"; $resn_exch{" H62 DGX"}="2H6  DGX";
$resn_exch{" H71 DGX"}="1H7  DGX"; $resn_exch{" H72 DGX"}="2H7  DGX"; $resn_exch{"H111 DGX"}="1H11 DGX";
$resn_exch{"H112 DGX"}="2H11 DGX"; $resn_exch{"HO12 DGX"}="2HO1 DGX"; $resn_exch{"HO14 DGX"}="4HO1 DGX";
$resn_exch{"H151 DGX"}="1H15 DGX"; $resn_exch{"H152 DGX"}="2H15 DGX"; $resn_exch{"H161 DGX"}="1H16 DGX";
$resn_exch{"H162 DGX"}="2H16 DGX"; $resn_exch{"H181 DGX"}="1H18 DGX"; $resn_exch{"H182 DGX"}="2H18 DGX";
$resn_exch{"H183 DGX"}="3H18 DGX"; $resn_exch{"H191 DGX"}="1H19 DGX"; $resn_exch{"H192 DGX"}="2H19 DGX";
$resn_exch{"H193 DGX"}="3H19 DGX"; $resn_exch{"H211 DGX"}="1H21 DGX"; $resn_exch{"H212 DGX"}="2H21 DGX";
$resn_exch{"H2X1 DGX"}="1H2X DGX"; $resn_exch{"H2X2 DGX"}="2H2X DGX"; $resn_exch{"HOX3 DGX"}="3HOX DGX";
$resn_exch{"H6X1 DGX"}="1H6X DGX"; $resn_exch{"H6X2 DGX"}="2H6X DGX"; $resn_exch{"H6X3 DGX"}="3H6X DGX";
$resn_exch{"H2Y1 DGX"}="1H2Y DGX"; $resn_exch{"H2Y2 DGX"}="2H2Y DGX"; $resn_exch{"HOY3 DGX"}="3HOY DGX";
$resn_exch{"H6Y1 DGX"}="1H6Y DGX"; $resn_exch{"H6Y2 DGX"}="2H6Y DGX"; $resn_exch{"H6Y3 DGX"}="3H6Y DGX";
$resn_exch{"H2Z1 DGX"}="1H2Z DGX"; $resn_exch{"H2Z2 DGX"}="2H2Z DGX"; $resn_exch{"HOZ3 DGX"}="3HOZ DGX";
$resn_exch{"HOZ4 DGX"}="4HOZ DGX"; $resn_exch{"H6Z1 DGX"}="1H6Z DGX"; $resn_exch{"H6Z2 DGX"}="2H6Z DGX";
$resn_exch{"H6Z3 DGX"}="3H6Z DGX"; $resn_exch{"H3C1 DGY"}="1H3C DGY"; $resn_exch{"H3C2 DGY"}="2H3C DGY";
$resn_exch{"H161 DH1"}="1H16 DH1"; $resn_exch{"H162 DH1"}="2H16 DH1"; $resn_exch{"H163 DH1"}="3H16 DH1";
$resn_exch{"H521 DH1"}="1H52 DH1"; $resn_exch{"H522 DH1"}="2H52 DH1"; $resn_exch{" H11 DH1"}="1H1  DH1";
$resn_exch{" H12 DH1"}="2H1  DH1"; $resn_exch{" H31 DH1"}="1H3  DH1"; $resn_exch{" H32 DH1"}="2H3  DH1";
$resn_exch{" H71 DH1"}="1H7  DH1"; $resn_exch{" H72 DH1"}="2H7  DH1"; $resn_exch{"H511 DH1"}="1H51 DH1";
$resn_exch{"H512 DH1"}="2H51 DH1"; $resn_exch{"HN21 DH1"}="1HN2 DH1"; $resn_exch{"HN22 DH1"}="2HN2 DH1";
$resn_exch{"HN41 DH1"}="1HN4 DH1"; $resn_exch{"HN42 DH1"}="2HN4 DH1"; $resn_exch{"HN41 DH3"}="1HN4 DH3";
$resn_exch{"HN42 DH3"}="2HN4 DH3"; $resn_exch{"HN21 DH3"}="1HN2 DH3"; $resn_exch{"HN22 DH3"}="2HN2 DH3";
$resn_exch{"H511 DH3"}="1H51 DH3"; $resn_exch{"H512 DH3"}="2H51 DH3"; $resn_exch{" H11 DH3"}="1H1  DH3";
$resn_exch{" H12 DH3"}="2H1  DH3"; $resn_exch{" H31 DH3"}="1H3  DH3"; $resn_exch{" H32 DH3"}="2H3  DH3";
$resn_exch{" H71 DH3"}="1H7  DH3"; $resn_exch{" H72 DH3"}="2H7  DH3"; $resn_exch{"H5'1 DH3"}="1H5* DH3";
$resn_exch{"H5'2 DH3"}="2H5* DH3"; $resn_exch{"H5'3 DH3"}="3H5* DH3"; $resn_exch{"H4'1 DH3"}="1H4* DH3";
$resn_exch{"H4'2 DH3"}="2H4* DH3"; $resn_exch{"H4'3 DH3"}="3H4* DH3"; $resn_exch{"H221 DH9"}="1H22 DH9";
$resn_exch{"H222 DH9"}="2H22 DH9"; $resn_exch{"H223 DH9"}="3H22 DH9"; $resn_exch{"H211 DH9"}="1H21 DH9";
$resn_exch{"H212 DH9"}="2H21 DH9"; $resn_exch{"H201 DH9"}="1H20 DH9"; $resn_exch{"H202 DH9"}="2H20 DH9";
$resn_exch{"H191 DH9"}="1H19 DH9"; $resn_exch{"H192 DH9"}="2H19 DH9"; $resn_exch{"H181 DH9"}="1H18 DH9";
$resn_exch{"H182 DH9"}="2H18 DH9"; $resn_exch{"H171 DH9"}="1H17 DH9"; $resn_exch{"H172 DH9"}="2H17 DH9";
$resn_exch{" H2' DH9"}=" H2* DH9"; $resn_exch{"HO2' DH9"}="HO2* DH9"; $resn_exch{" H3' DH9"}=" H3* DH9";
$resn_exch{"HO3' DH9"}="HO3* DH9"; $resn_exch{"H4'1 DH9"}="1H4* DH9"; $resn_exch{"H4'2 DH9"}="2H4* DH9";
$resn_exch{" H5' DH9"}=" H5* DH9"; $resn_exch{"H261 DH9"}="1H26 DH9"; $resn_exch{"H262 DH9"}="2H26 DH9";
$resn_exch{"H281 DH9"}="1H28 DH9"; $resn_exch{"H282 DH9"}="2H28 DH9"; $resn_exch{"H283 DH9"}="3H28 DH9";
$resn_exch{"H291 DH9"}="1H29 DH9"; $resn_exch{"H292 DH9"}="2H29 DH9"; $resn_exch{"H293 DH9"}="3H29 DH9";
$resn_exch{"HN1' DH9"}="HN1* DH9"; $resn_exch{"H6'1 DH9"}="1H6* DH9"; $resn_exch{"H6'2 DH9"}="2H6* DH9";
$resn_exch{"H7'1 DH9"}="1H7* DH9"; $resn_exch{"H7'2 DH9"}="2H7* DH9"; $resn_exch{"H8'1 DH9"}="1H8* DH9";
$resn_exch{"H8'2 DH9"}="2H8* DH9"; $resn_exch{"H9'1 DH9"}="1H9* DH9"; $resn_exch{"H9'2 DH9"}="2H9* DH9";
$resn_exch{"H101 DH9"}="1H10 DH9"; $resn_exch{"H102 DH9"}="2H10 DH9"; $resn_exch{"H111 DH9"}="1H11 DH9";
$resn_exch{"H112 DH9"}="2H11 DH9"; $resn_exch{"H121 DH9"}="1H12 DH9"; $resn_exch{"H122 DH9"}="2H12 DH9";
$resn_exch{"H131 DH9"}="1H13 DH9"; $resn_exch{"H132 DH9"}="2H13 DH9"; $resn_exch{"H141 DH9"}="1H14 DH9";
$resn_exch{"H142 DH9"}="2H14 DH9"; $resn_exch{"H151 DH9"}="1H15 DH9"; $resn_exch{"H152 DH9"}="2H15 DH9";
$resn_exch{"H161 DH9"}="1H16 DH9"; $resn_exch{"H162 DH9"}="2H16 DH9"; $resn_exch{"H163 DH9"}="3H16 DH9";
$resn_exch{" H   DHA"}="1HN  DHA"; $resn_exch{" H2  DHA"}="2HN  DHA"; $resn_exch{" HB1 DHA"}="2HB  DHA";
$resn_exch{" HB2 DHA"}="3HB  DHA"; $resn_exch{" H31 DHD"}="1H3  DHD"; $resn_exch{" H32 DHD"}="2H3  DHD";
$resn_exch{"HMA1 DHE"}="1HMA DHE"; $resn_exch{"HMA2 DHE"}="2HMA DHE"; $resn_exch{"HMA3 DHE"}="3HMA DHE";
$resn_exch{"HAA1 DHE"}="1HAA DHE"; $resn_exch{"HAA2 DHE"}="2HAA DHE"; $resn_exch{"HBA1 DHE"}="1HBA DHE";
$resn_exch{"HBA2 DHE"}="2HBA DHE"; $resn_exch{"HGB1 DHE"}="1HGB DHE"; $resn_exch{"HGB2 DHE"}="2HGB DHE";
$resn_exch{"HGB3 DHE"}="3HGB DHE"; $resn_exch{"HAB1 DHE"}="1HAB DHE"; $resn_exch{"HAB2 DHE"}="2HAB DHE";
$resn_exch{"HGC1 DHE"}="1HGC DHE"; $resn_exch{"HGC2 DHE"}="2HGC DHE"; $resn_exch{"HGC3 DHE"}="3HGC DHE";
$resn_exch{"HAC1 DHE"}="1HAC DHE"; $resn_exch{"HAC2 DHE"}="2HAC DHE"; $resn_exch{"HMD1 DHE"}="1HMD DHE";
$resn_exch{"HMD2 DHE"}="2HMD DHE"; $resn_exch{"HMD3 DHE"}="3HMD DHE"; $resn_exch{"HAD1 DHE"}="1HAD DHE";
$resn_exch{"HAD2 DHE"}="2HAD DHE"; $resn_exch{"HBD1 DHE"}="1HBD DHE"; $resn_exch{"HBD2 DHE"}="2HBD DHE";
$resn_exch{"HN21 DHF"}="1HN2 DHF"; $resn_exch{"HN22 DHF"}="2HN2 DHF"; $resn_exch{" H71 DHF"}="1H7  DHF";
$resn_exch{" H72 DHF"}="2H7  DHF"; $resn_exch{" H91 DHF"}="1H9  DHF"; $resn_exch{" H92 DHF"}="2H9  DHF";
$resn_exch{" HB1 DHF"}="1HB  DHF"; $resn_exch{" HB2 DHF"}="2HB  DHF"; $resn_exch{" HG1 DHF"}="1HG  DHF";
$resn_exch{" HG2 DHF"}="2HG  DHF"; $resn_exch{"HOE2 DHF"}="2HOE DHF"; $resn_exch{" H21 DHG"}="1H2  DHG";
$resn_exch{" H22 DHG"}="2H2  DHG"; $resn_exch{" H31 DHG"}="1H3  DHG"; $resn_exch{" H32 DHG"}="2H3  DHG";
$resn_exch{" H41 DHG"}="1H4  DHG"; $resn_exch{" H42 DHG"}="2H4  DHG"; $resn_exch{" H51 DHG"}="1H5  DHG";
$resn_exch{" H52 DHG"}="2H5  DHG"; $resn_exch{" H61 DHG"}="1H6  DHG"; $resn_exch{" H62 DHG"}="2H6  DHG";
$resn_exch{" H71 DHG"}="1H7  DHG"; $resn_exch{" H72 DHG"}="2H7  DHG"; $resn_exch{" H81 DHG"}="1H8  DHG";
$resn_exch{" H82 DHG"}="2H8  DHG"; $resn_exch{" H91 DHG"}="1H9  DHG"; $resn_exch{" H92 DHG"}="2H9  DHG";
$resn_exch{"H101 DHG"}="1H10 DHG"; $resn_exch{"H102 DHG"}="2H10 DHG"; $resn_exch{"H111 DHG"}="1H11 DHG";
$resn_exch{"H112 DHG"}="2H11 DHG"; $resn_exch{"H121 DHG"}="1H12 DHG"; $resn_exch{"H122 DHG"}="2H12 DHG";
$resn_exch{"H123 DHG"}="3H12 DHG"; $resn_exch{"HH11 DHG"}="1HH1 DHG"; $resn_exch{"HH12 DHG"}="2HH1 DHG";
$resn_exch{"HH31 DHG"}="1HH3 DHG"; $resn_exch{"HH32 DHG"}="2HH3 DHG"; $resn_exch{"HH41 DHG"}="1HH4 DHG";
$resn_exch{"HH42 DHG"}="2HH4 DHG"; $resn_exch{"HH51 DHG"}="1HH5 DHG"; $resn_exch{"HH52 DHG"}="2HH5 DHG";
$resn_exch{"HH61 DHG"}="1HH6 DHG"; $resn_exch{"HH62 DHG"}="2HH6 DHG"; $resn_exch{"HH63 DHG"}="3HH6 DHG";
$resn_exch{"HOP2 DHG"}="2HOP DHG"; $resn_exch{"HG11 DHG"}="1HG1 DHG"; $resn_exch{"HG12 DHG"}="2HG1 DHG";
$resn_exch{"HG21 DHG"}="1HG2 DHG"; $resn_exch{"HG22 DHG"}="2HG2 DHG"; $resn_exch{"HOG2 DHG"}="2HOG DHG";
$resn_exch{"HP11 DHH"}="1HP1 DHH"; $resn_exch{"HP12 DHH"}="2HP1 DHH"; $resn_exch{"H061 DHH"}="1H06 DHH";
$resn_exch{"H062 DHH"}="2H06 DHH"; $resn_exch{"H071 DHH"}="1H07 DHH"; $resn_exch{"H072 DHH"}="2H07 DHH";
$resn_exch{"H081 DHH"}="1H08 DHH"; $resn_exch{"H082 DHH"}="2H08 DHH"; $resn_exch{"H091 DHH"}="1H09 DHH";
$resn_exch{"H092 DHH"}="2H09 DHH"; $resn_exch{" H2  DHI"}=" HN2 DHI"; $resn_exch{" HB2 DHI"}="1HB  DHI";
$resn_exch{" HB3 DHI"}="2HB  DHI"; $resn_exch{" H61 DHJ"}="1H6  DHJ"; $resn_exch{" H62 DHJ"}="2H6  DHJ";
$resn_exch{" H41 DHJ"}="1H4  DHJ"; $resn_exch{" H42 DHJ"}="2H4  DHJ"; $resn_exch{"H6C1 DHK"}="1H6C DHK";
$resn_exch{"H6C2 DHK"}="2H6C DHK"; $resn_exch{" HN1 DHL"}="1HN  DHL"; $resn_exch{" HN2 DHL"}="2HN  DHL";
$resn_exch{" HA2 DHL"}="2HA  DHL"; $resn_exch{" HA3 DHL"}="3HA  DHL"; $resn_exch{" HB2 DHL"}="2HB  DHL";
$resn_exch{" HB3 DHL"}="3HB  DHL"; $resn_exch{" H72 DHM"}="2H7  DHM"; $resn_exch{" H71 DHM"}="1H7  DHM";
$resn_exch{" H61 DHM"}="1H6  DHM"; $resn_exch{" H42 DHM"}="2H4  DHM"; $resn_exch{" H41 DHM"}="1H4  DHM";
$resn_exch{" H32 DHM"}="2H3  DHM"; $resn_exch{" H31 DHM"}="1H3  DHM"; $resn_exch{" H22 DHM"}="2H2  DHM";
$resn_exch{" H21 DHM"}="1H2  DHM"; $resn_exch{" H83 DHM"}="3H8  DHM"; $resn_exch{" H82 DHM"}="2H8  DHM";
$resn_exch{" H81 DHM"}="1H8  DHM"; $resn_exch{" H93 DHM"}="3H9  DHM"; $resn_exch{" H92 DHM"}="2H9  DHM";
$resn_exch{" H91 DHM"}="1H9  DHM"; $resn_exch{"H103 DHM"}="3H10 DHM"; $resn_exch{"H102 DHM"}="2H10 DHM";
$resn_exch{"H101 DHM"}="1H10 DHM"; $resn_exch{" HN1 DHN"}="1HN  DHN"; $resn_exch{" HN2 DHN"}="2HN  DHN";
$resn_exch{" HB1 DHN"}="1HB  DHN"; $resn_exch{" HB2 DHN"}="2HB  DHN"; $resn_exch{" HG1 DHN"}="1HG  DHN";
$resn_exch{" HG2 DHN"}="2HG  DHN"; $resn_exch{" HD1 DHN"}="1HD  DHN"; $resn_exch{" HD2 DHN"}="2HD  DHN";
$resn_exch{" H11 DHO"}="1H1  DHO"; $resn_exch{"H12A DHO"}="2H1  DHO"; $resn_exch{" H21 DHO"}="1H2  DHO";
$resn_exch{" H22 DHO"}="2H2  DHO"; $resn_exch{" H41 DHO"}="1H4  DHO"; $resn_exch{" H42 DHO"}="2H4  DHO";
$resn_exch{" H61 DHO"}="1H6  DHO"; $resn_exch{" H62 DHO"}="2H6  DHO"; $resn_exch{"H111 DHO"}="1H11 DHO";
$resn_exch{"H112 DHO"}="2H11 DHO"; $resn_exch{"H151 DHO"}="1H15 DHO"; $resn_exch{"H152 DHO"}="2H15 DHO";
$resn_exch{"H161 DHO"}="1H16 DHO"; $resn_exch{"H162 DHO"}="2H16 DHO"; $resn_exch{"H181 DHO"}="1H18 DHO";
$resn_exch{"H182 DHO"}="2H18 DHO"; $resn_exch{"H183 DHO"}="3H18 DHO"; $resn_exch{"H191 DHO"}="1H19 DHO";
$resn_exch{"H192 DHO"}="2H19 DHO"; $resn_exch{"H193 DHO"}="3H19 DHO"; $resn_exch{"H211 DHO"}="1H21 DHO";
$resn_exch{"H212 DHO"}="2H21 DHO"; $resn_exch{"H213 DHO"}="3H21 DHO"; $resn_exch{"H221 DHO"}="1H22 DHO";
$resn_exch{"H222 DHO"}="2H22 DHO"; $resn_exch{"H231 DHO"}="1H23 DHO"; $resn_exch{"H232 DHO"}="2H23 DHO";
$resn_exch{" H51 DHP"}="1H5  DHP"; $resn_exch{" H52 DHP"}="2H5  DHP"; $resn_exch{" H61 DHP"}="1H6  DHP";
$resn_exch{" H62 DHP"}="2H6  DHP"; $resn_exch{" H71 DHP"}="1H7  DHP"; $resn_exch{" H72 DHP"}="2H7  DHP";
$resn_exch{" H81 DHP"}="1H8  DHP"; $resn_exch{" H82 DHP"}="2H8  DHP"; $resn_exch{" H91 DHP"}="1H9  DHP";
$resn_exch{" H92 DHP"}="2H9  DHP"; $resn_exch{"H101 DHP"}="1H10 DHP"; $resn_exch{"H102 DHP"}="2H10 DHP";
$resn_exch{"H111 DHP"}="1H11 DHP"; $resn_exch{"H112 DHP"}="2H11 DHP"; $resn_exch{"H121 DHP"}="1H12 DHP";
$resn_exch{"H122 DHP"}="2H12 DHP"; $resn_exch{"H131 DHP"}="1H13 DHP"; $resn_exch{"H132 DHP"}="2H13 DHP";
$resn_exch{"H141 DHP"}="1H14 DHP"; $resn_exch{"H142 DHP"}="2H14 DHP"; $resn_exch{"H143 DHP"}="3H14 DHP";
$resn_exch{" H31 DHQ"}="1H3  DHQ"; $resn_exch{" H32 DHQ"}="2H3  DHQ"; $resn_exch{" H41 DHQ"}="1H4  DHQ";
$resn_exch{" H42 DHQ"}="2H4  DHQ"; $resn_exch{" H91 DHQ"}="1H9  DHQ"; $resn_exch{" H92 DHQ"}="2H9  DHQ";
$resn_exch{" H93 DHQ"}="3H9  DHQ"; $resn_exch{" HN1 DHS"}="1HN  DHS"; $resn_exch{" HN2 DHS"}="2HN  DHS";
$resn_exch{" H61 DHS"}="1H6  DHS"; $resn_exch{" H62 DHS"}="2H6  DHS"; $resn_exch{" H11 DHT"}="1H1  DHT";
$resn_exch{" H12 DHT"}="2H1  DHT"; $resn_exch{" H21 DHT"}="1H2  DHT"; $resn_exch{" H22 DHT"}="2H2  DHT";
$resn_exch{" H41 DHT"}="1H4  DHT"; $resn_exch{" H42 DHT"}="2H4  DHT"; $resn_exch{" H61 DHT"}="1H6  DHT";
$resn_exch{" H62 DHT"}="2H6  DHT"; $resn_exch{" H71 DHT"}="1H7  DHT"; $resn_exch{" H72 DHT"}="2H7  DHT";
$resn_exch{"H111 DHT"}="1H11 DHT"; $resn_exch{"H112 DHT"}="2H11 DHT"; $resn_exch{"H121 DHT"}="1H12 DHT";
$resn_exch{"H122 DHT"}="2H12 DHT"; $resn_exch{"H151 DHT"}="1H15 DHT"; $resn_exch{"H152 DHT"}="2H15 DHT";
$resn_exch{"H161 DHT"}="1H16 DHT"; $resn_exch{"H162 DHT"}="2H16 DHT"; $resn_exch{"H181 DHT"}="1H18 DHT";
$resn_exch{"H182 DHT"}="2H18 DHT"; $resn_exch{"H183 DHT"}="3H18 DHT"; $resn_exch{"H191 DHT"}="1H19 DHT";
$resn_exch{"H192 DHT"}="2H19 DHT"; $resn_exch{"H193 DHT"}="3H19 DHT"; $resn_exch{"HOP2 DHU"}="2HOP DHU";
$resn_exch{"HOP3 DHU"}="3HOP DHU"; $resn_exch{" H5' DHU"}="1H5* DHU"; $resn_exch{"H5'' DHU"}="2H5* DHU";
$resn_exch{" H4' DHU"}=" H4* DHU"; $resn_exch{" H3' DHU"}=" H3* DHU"; $resn_exch{"HO3' DHU"}="*HO3 DHU";
$resn_exch{" H2' DHU"}=" H2* DHU"; $resn_exch{"HO2' DHU"}="*HO2 DHU"; $resn_exch{" H1' DHU"}=" H1* DHU";
$resn_exch{" H51 DHU"}="1H5  DHU"; $resn_exch{" H52 DHU"}="2H5  DHU"; $resn_exch{" H61 DHU"}="1H6  DHU";
$resn_exch{" H62 DHU"}="2H6  DHU"; $resn_exch{" H71 DHY"}="1H7  DHY"; $resn_exch{" H72 DHY"}="2H7  DHY";
$resn_exch{" H41 DHZ"}="1H4  DHZ"; $resn_exch{" H42 DHZ"}="2H4  DHZ"; $resn_exch{"H5'1 DHZ"}="1H5' DHZ";
$resn_exch{"H5'2 DHZ"}="2H5' DHZ"; $resn_exch{"HOP3  DI"}="3HOP  DI"; $resn_exch{"HOP2  DI"}="2HOP  DI";
$resn_exch{" H5'  DI"}="1H5*  DI"; $resn_exch{"H5''  DI"}="2H5*  DI"; $resn_exch{" H4'  DI"}=" H4*  DI";
$resn_exch{" H3'  DI"}=" H3*  DI"; $resn_exch{"HO3'  DI"}="H3T   DI"; $resn_exch{" H2'  DI"}="1H2*  DI";
$resn_exch{"H2''  DI"}="2H2*  DI"; $resn_exch{" H1'  DI"}=" H1*  DI"; $resn_exch{"H21A DI2"}="1H2  DI2";
$resn_exch{" H22 DI2"}="2H2  DI2"; $resn_exch{" H31 DI2"}="1H3  DI2"; $resn_exch{" H32 DI2"}="2H3  DI2";
$resn_exch{" H41 DI2"}="1H4  DI2"; $resn_exch{" H42 DI2"}="2H4  DI2"; $resn_exch{" H51 DI2"}="1H5  DI2";
$resn_exch{" H52 DI2"}="2H5  DI2"; $resn_exch{" H81 DI2"}="1H8  DI2"; $resn_exch{" H82 DI2"}="2H8  DI2";
$resn_exch{" H91 DI2"}="1H9  DI2"; $resn_exch{" H92 DI2"}="2H9  DI2"; $resn_exch{"H101 DI2"}="1H10 DI2";
$resn_exch{"H102 DI2"}="2H10 DI2"; $resn_exch{"H121 DI2"}="1H12 DI2"; $resn_exch{"H122 DI2"}="2H12 DI2";
$resn_exch{"H151 DI2"}="1H15 DI2"; $resn_exch{"H152 DI2"}="2H15 DI2"; $resn_exch{"H153 DI2"}="3H15 DI2";
$resn_exch{"HN11 DI2"}="1HN1 DI2"; $resn_exch{"HN12 DI2"}="2HN1 DI2"; $resn_exch{" H21 DI3"}="1H2  DI3";
$resn_exch{"H22A DI3"}="2H2  DI3"; $resn_exch{" H31 DI3"}="1H3  DI3"; $resn_exch{" H32 DI3"}="2H3  DI3";
$resn_exch{" H41 DI3"}="1H4  DI3"; $resn_exch{" H42 DI3"}="2H4  DI3"; $resn_exch{" H51 DI3"}="1H5  DI3";
$resn_exch{" H52 DI3"}="2H5  DI3"; $resn_exch{" H91 DI3"}="1H9  DI3"; $resn_exch{" H92 DI3"}="2H9  DI3";
$resn_exch{"H101 DI3"}="1H10 DI3"; $resn_exch{"H102 DI3"}="2H10 DI3"; $resn_exch{"H111 DI3"}="1H11 DI3";
$resn_exch{"H112 DI3"}="2H11 DI3"; $resn_exch{"H131 DI3"}="1H13 DI3"; $resn_exch{"H132 DI3"}="2H13 DI3";
$resn_exch{"H161 DI3"}="1H16 DI3"; $resn_exch{"H162 DI3"}="2H16 DI3"; $resn_exch{"H163 DI3"}="3H16 DI3";
$resn_exch{"HN21 DI3"}="1HN2 DI3"; $resn_exch{"HN22 DI3"}="2HN2 DI3"; $resn_exch{" H21 DI4"}="1H2  DI4";
$resn_exch{"H22A DI4"}="2H2  DI4"; $resn_exch{" H31 DI4"}="1H3  DI4"; $resn_exch{" H32 DI4"}="2H3  DI4";
$resn_exch{" H41 DI4"}="1H4  DI4"; $resn_exch{" H42 DI4"}="2H4  DI4"; $resn_exch{" H51 DI4"}="1H5  DI4";
$resn_exch{" H52 DI4"}="2H5  DI4"; $resn_exch{" H61 DI4"}="1H6  DI4"; $resn_exch{" H62 DI4"}="2H6  DI4";
$resn_exch{" H91 DI4"}="1H9  DI4"; $resn_exch{" H92 DI4"}="2H9  DI4"; $resn_exch{"H101 DI4"}="1H10 DI4";
$resn_exch{"H102 DI4"}="2H10 DI4"; $resn_exch{"H111 DI4"}="1H11 DI4"; $resn_exch{"H112 DI4"}="2H11 DI4";
$resn_exch{"H131 DI4"}="1H13 DI4"; $resn_exch{"H132 DI4"}="2H13 DI4"; $resn_exch{"H161 DI4"}="1H16 DI4";
$resn_exch{"H162 DI4"}="2H16 DI4"; $resn_exch{"H163 DI4"}="3H16 DI4"; $resn_exch{"HN11 DI4"}="1HN1 DI4";
$resn_exch{"HN12 DI4"}="2HN1 DI4"; $resn_exch{" H21 DI5"}="1H2  DI5"; $resn_exch{" H22 DI5"}="2H2  DI5";
$resn_exch{" H31 DI5"}="1H3  DI5"; $resn_exch{" H32 DI5"}="2H3  DI5"; $resn_exch{" H41 DI5"}="1H4  DI5";
$resn_exch{" H42 DI5"}="2H4  DI5"; $resn_exch{" H71 DI5"}="1H7  DI5"; $resn_exch{" H72 DI5"}="2H7  DI5";
$resn_exch{" H81 DI5"}="1H8  DI5"; $resn_exch{" H82 DI5"}="2H8  DI5"; $resn_exch{" H91 DI5"}="1H9  DI5";
$resn_exch{" H92 DI5"}="2H9  DI5"; $resn_exch{"H111 DI5"}="1H11 DI5"; $resn_exch{"H112 DI5"}="2H11 DI5";
$resn_exch{"H141 DI5"}="1H14 DI5"; $resn_exch{"H142 DI5"}="2H14 DI5"; $resn_exch{"H143 DI5"}="3H14 DI5";
$resn_exch{"HN11 DI5"}="1HN1 DI5"; $resn_exch{"HN12 DI5"}="2HN1 DI5"; $resn_exch{" H61 DI6"}="1H6  DI6";
$resn_exch{" H62 DI6"}="2H6  DI6"; $resn_exch{" H71 DI6"}="1H7  DI6"; $resn_exch{" H72 DI6"}="2H7  DI6";
$resn_exch{"HAB1 DI9"}="1HAB DI9"; $resn_exch{"HAB2 DI9"}="2HAB DI9"; $resn_exch{"HAJ1 DI9"}="1HAJ DI9";
$resn_exch{"HAJ2 DI9"}="2HAJ DI9"; $resn_exch{"HAK1 DI9"}="1HAK DI9"; $resn_exch{"HAK2 DI9"}="2HAK DI9";
$resn_exch{"HAL1 DI9"}="1HAL DI9"; $resn_exch{"HAL2 DI9"}="2HAL DI9"; $resn_exch{"HAM1 DI9"}="1HAM DI9";
$resn_exch{"HAM2 DI9"}="2HAM DI9"; $resn_exch{"HAH1 DI9"}="1HAH DI9"; $resn_exch{"HAH2 DI9"}="2HAH DI9";
$resn_exch{"HAI1 DI9"}="1HAI DI9"; $resn_exch{"HAI2 DI9"}="2HAI DI9"; $resn_exch{"HAO1 DI9"}="1HAO DI9";
$resn_exch{"HAO2 DI9"}="2HAO DI9"; $resn_exch{"HAN1 DI9"}="1HAN DI9"; $resn_exch{"HAN2 DI9"}="2HAN DI9";
$resn_exch{"HN11 DIA"}="1HN1 DIA"; $resn_exch{"HN12 DIA"}="2HN1 DIA"; $resn_exch{"HC11 DIA"}="1HC1 DIA";
$resn_exch{"HC12 DIA"}="2HC1 DIA"; $resn_exch{"HC21 DIA"}="1HC2 DIA"; $resn_exch{"HC22 DIA"}="2HC2 DIA";
$resn_exch{"HC31 DIA"}="1HC3 DIA"; $resn_exch{"HC32 DIA"}="2HC3 DIA"; $resn_exch{"HC41 DIA"}="1HC4 DIA";
$resn_exch{"HC42 DIA"}="2HC4 DIA"; $resn_exch{"HC51 DIA"}="1HC5 DIA"; $resn_exch{"HC52 DIA"}="2HC5 DIA";
$resn_exch{"HC61 DIA"}="1HC6 DIA"; $resn_exch{"HC62 DIA"}="2HC6 DIA"; $resn_exch{"HC71 DIA"}="1HC7 DIA";
$resn_exch{"HC72 DIA"}="2HC7 DIA"; $resn_exch{"HC81 DIA"}="1HC8 DIA"; $resn_exch{"HC82 DIA"}="2HC8 DIA";
$resn_exch{"HN81 DIA"}="1HN8 DIA"; $resn_exch{"HN82 DIA"}="2HN8 DIA"; $resn_exch{" HA1 DIB"}="1HA  DIB";
$resn_exch{" HA2 DIB"}="2HA  DIB"; $resn_exch{" HB1 DIB"}="1HB  DIB"; $resn_exch{" HB2 DIB"}="2HB  DIB";
$resn_exch{" HG1 DIB"}="1HG  DIB"; $resn_exch{" HG2 DIB"}="2HG  DIB"; $resn_exch{"HE11 DIB"}="1HE1 DIB";
$resn_exch{"HE12 DIB"}="2HE1 DIB"; $resn_exch{"HE13 DIB"}="3HE1 DIB"; $resn_exch{"HE21 DIB"}="1HE2 DIB";
$resn_exch{"HE22 DIB"}="2HE2 DIB"; $resn_exch{"HE23 DIB"}="3HE2 DIB"; $resn_exch{"HN11 DID"}="1HN1 DID";
$resn_exch{"HN12 DID"}="2HN1 DID"; $resn_exch{" H71 DID"}="1H7  DID"; $resn_exch{" H72 DID"}="2H7  DID";
$resn_exch{" H81 DID"}="1H8  DID"; $resn_exch{" H82 DID"}="2H8  DID"; $resn_exch{"H101 DID"}="1H10 DID";
$resn_exch{"H102 DID"}="2H10 DID"; $resn_exch{"H111 DID"}="1H11 DID"; $resn_exch{"H112 DID"}="2H11 DID";
$resn_exch{"H8'1 DID"}="1H8' DID"; $resn_exch{"H8'2 DID"}="2H8' DID"; $resn_exch{"H7'1 DID"}="1H7' DID";
$resn_exch{"H7'2 DID"}="2H7' DID"; $resn_exch{"H1'1 DID"}="1H1' DID"; $resn_exch{"H1'2 DID"}="2H1' DID";
$resn_exch{"H131 DIF"}="1H13 DIF"; $resn_exch{"H132 DIF"}="2H13 DIF"; $resn_exch{" H11 DIG"}="1H1  DIG";
$resn_exch{" H12 DIG"}="2H1  DIG"; $resn_exch{" H61 DIG"}="1H6  DIG"; $resn_exch{" H62 DIG"}="2H6  DIG";
$resn_exch{"HO5' DIH"}="*HO5 DIH"; $resn_exch{"H5'1 DIH"}="1H5* DIH"; $resn_exch{"H5'2 DIH"}="2H5* DIH";
$resn_exch{" H4' DIH"}=" H4* DIH"; $resn_exch{"H6'1 DIH"}="1H6* DIH"; $resn_exch{"H6'2 DIH"}="2H6* DIH";
$resn_exch{" H3' DIH"}=" H3* DIH"; $resn_exch{"HO3' DIH"}="*HO3 DIH"; $resn_exch{"H2'1 DIH"}="1H2* DIH";
$resn_exch{"H2'2 DIH"}="2H2* DIH"; $resn_exch{" H1' DIH"}=" H1* DIH"; $resn_exch{"H101 DIH"}="1H10 DIH";
$resn_exch{"H102 DIH"}="2H10 DIH"; $resn_exch{" H2  DIL"}=" HN2 DIL"; $resn_exch{"HG12 DIL"}="1HG1 DIL";
$resn_exch{"HG13 DIL"}="2HG1 DIL"; $resn_exch{"HG21 DIL"}="1HG2 DIL"; $resn_exch{"HG22 DIL"}="2HG2 DIL";
$resn_exch{"HG23 DIL"}="3HG2 DIL"; $resn_exch{"HD11 DIL"}="1HD1 DIL"; $resn_exch{"HD12 DIL"}="2HD1 DIL";
$resn_exch{"HD13 DIL"}="3HD1 DIL"; $resn_exch{" H51 DIM"}="1H5  DIM"; $resn_exch{" H52 DIM"}="2H5  DIM";
$resn_exch{" H53 DIM"}="3H5  DIM"; $resn_exch{"H101 DIM"}="1H10 DIM"; $resn_exch{"H102 DIM"}="2H10 DIM";
$resn_exch{"H103 DIM"}="3H10 DIM"; $resn_exch{"H121 DIM"}="1H12 DIM"; $resn_exch{"H122 DIM"}="2H12 DIM";
$resn_exch{"H131 DIM"}="1H13 DIM"; $resn_exch{"H132 DIM"}="2H13 DIM"; $resn_exch{"HN91 DIM"}="1HN9 DIM";
$resn_exch{"HN92 DIM"}="2HN9 DIM"; $resn_exch{"HN81 DIM"}="1HN8 DIM"; $resn_exch{"HN82 DIM"}="2HN8 DIM";
$resn_exch{" H11 DIO"}="1H1  DIO"; $resn_exch{" H12 DIO"}="2H1  DIO"; $resn_exch{" H21 DIO"}="1H2  DIO";
$resn_exch{" H22 DIO"}="2H2  DIO"; $resn_exch{"H1'1 DIO"}="1H1' DIO"; $resn_exch{"H1'2 DIO"}="2H1' DIO";
$resn_exch{"H2'1 DIO"}="1H2' DIO"; $resn_exch{"H2'2 DIO"}="2H2' DIO"; $resn_exch{" H11 DIP"}="1H1  DIP";
$resn_exch{" H12 DIP"}="2H1  DIP"; $resn_exch{" H21 DIP"}="1H2  DIP"; $resn_exch{" H22 DIP"}="2H2  DIP";
$resn_exch{" H31 DIP"}="1H3  DIP"; $resn_exch{" H32 DIP"}="2H3  DIP"; $resn_exch{" H41 DIP"}="1H4  DIP";
$resn_exch{" H42 DIP"}="2H4  DIP"; $resn_exch{" H51 DIP"}="1H5  DIP"; $resn_exch{" H52 DIP"}="2H5  DIP";
$resn_exch{" H53 DIP"}="3H5  DIP"; $resn_exch{"H1'1 DIP"}="1H1' DIP"; $resn_exch{"H1'2 DIP"}="2H1' DIP";
$resn_exch{"H2'1 DIP"}="1H2' DIP"; $resn_exch{"H2'2 DIP"}="2H2' DIP"; $resn_exch{"H3'1 DIP"}="1H3' DIP";
$resn_exch{"H3'2 DIP"}="2H3' DIP"; $resn_exch{"H4'1 DIP"}="1H4' DIP"; $resn_exch{"H4'2 DIP"}="2H4' DIP";
$resn_exch{"H5'1 DIP"}="1H5' DIP"; $resn_exch{"H5'2 DIP"}="2H5' DIP"; $resn_exch{"H5'3 DIP"}="3H5' DIP";
$resn_exch{" HM1 DIQ"}="1HM  DIQ"; $resn_exch{" HM2 DIQ"}="2HM  DIQ"; $resn_exch{" HM3 DIQ"}="3HM  DIQ";
$resn_exch{" H31 DIQ"}="1H3  DIQ"; $resn_exch{" H32 DIQ"}="2H3  DIQ"; $resn_exch{" H41 DIQ"}="1H4  DIQ";
$resn_exch{" H42 DIQ"}="2H4  DIQ"; $resn_exch{" H51 DIQ"}="1H5  DIQ"; $resn_exch{" H52 DIQ"}="2H5  DIQ";
$resn_exch{" H61 DIQ"}="1H6  DIQ"; $resn_exch{" H62 DIQ"}="2H6  DIQ"; $resn_exch{" H71 DIQ"}="1H7  DIQ";
$resn_exch{" H72 DIQ"}="2H7  DIQ"; $resn_exch{" H81 DIQ"}="1H8  DIQ"; $resn_exch{" H82 DIQ"}="2H8  DIQ";
$resn_exch{" HB2 DIR"}="1HB  DIR"; $resn_exch{" HB3 DIR"}="2HB  DIR"; $resn_exch{" H   DIR"}="1HN  DIR";
$resn_exch{" H2  DIR"}="2HN  DIR"; $resn_exch{" HG  DIR"}=" HNG DIR"; $resn_exch{"HH21 DIR"}="1HH2 DIR";
$resn_exch{"HH22 DIR"}="2HH2 DIR"; $resn_exch{" H1  DIS"}="1H   DIS"; $resn_exch{" H2  DIS"}="2H   DIS";
$resn_exch{"H181 DIT"}="1H18 DIT"; $resn_exch{"H182 DIT"}="2H18 DIT"; $resn_exch{"H183 DIT"}="3H18 DIT";
$resn_exch{"H191 DIT"}="1H19 DIT"; $resn_exch{"H192 DIT"}="2H19 DIT"; $resn_exch{"H201 DIT"}="1H20 DIT";
$resn_exch{"H202 DIT"}="2H20 DIT"; $resn_exch{"H221 DIT"}="1H22 DIT"; $resn_exch{"H222 DIT"}="2H22 DIT";
$resn_exch{"H231 DIT"}="1H23 DIT"; $resn_exch{"H232 DIT"}="2H23 DIT"; $resn_exch{"H251 DIT"}="1H25 DIT";
$resn_exch{"H252 DIT"}="2H25 DIT"; $resn_exch{"H261 DIT"}="1H26 DIT"; $resn_exch{"H262 DIT"}="2H26 DIT";
$resn_exch{" H1X DIT"}=" XH1 DIT"; $resn_exch{" H3X DIT"}=" XH3 DIT"; $resn_exch{" H4X DIT"}=" XH4 DIT";
$resn_exch{" H5X DIT"}=" XH5 DIT"; $resn_exch{" H6X DIT"}=" XH6 DIT"; $resn_exch{"HN7X DIT"}="XHN7 DIT";
$resn_exch{" H8X DIT"}=" XH8 DIT"; $resn_exch{" H9X DIT"}=" XH9 DIT"; $resn_exch{"H11X DIT"}="XH11 DIT";
$resn_exch{"HX81 DIT"}="1HX8 DIT"; $resn_exch{"HX82 DIT"}="2HX8 DIT"; $resn_exch{"HX83 DIT"}="3HX8 DIT";
$resn_exch{"HX91 DIT"}="1HX9 DIT"; $resn_exch{"HX92 DIT"}="2HX9 DIT"; $resn_exch{"HX01 DIT"}="1HX0 DIT";
$resn_exch{"HX02 DIT"}="2HX0 DIT"; $resn_exch{"HX21 DIT"}="1HX2 DIT"; $resn_exch{"HX22 DIT"}="2HX2 DIT";
$resn_exch{"HX31 DIT"}="1HX3 DIT"; $resn_exch{"HX32 DIT"}="2HX3 DIT"; $resn_exch{"H24X DIT"}="XH24 DIT";
$resn_exch{"HX51 DIT"}="1HX5 DIT"; $resn_exch{"HX52 DIT"}="2HX5 DIT"; $resn_exch{"HX61 DIT"}="1HX6 DIT";
$resn_exch{"HX62 DIT"}="2HX6 DIT"; $resn_exch{" H   DIV"}="1HN  DIV"; $resn_exch{" H2  DIV"}="2HN  DIV";
$resn_exch{"HB11 DIV"}="1HB1 DIV"; $resn_exch{"HB12 DIV"}="2HB1 DIV"; $resn_exch{"HG11 DIV"}="1HG1 DIV";
$resn_exch{"HG12 DIV"}="2HG1 DIV"; $resn_exch{"HG13 DIV"}="3HG1 DIV"; $resn_exch{"HB21 DIV"}="1HB2 DIV";
$resn_exch{"HB22 DIV"}="2HB2 DIV"; $resn_exch{"HB23 DIV"}="3HB2 DIV"; $resn_exch{" HM1 DIX"}="1HM  DIX";
$resn_exch{" HM2 DIX"}="2HM  DIX"; $resn_exch{" HM3 DIX"}="3HM  DIX"; $resn_exch{" H31 DIX"}="1H3  DIX";
$resn_exch{" H32 DIX"}="2H3  DIX"; $resn_exch{" H21 DIX"}="1H2  DIX"; $resn_exch{" H22 DIX"}="2H2  DIX";
$resn_exch{" H11 DIX"}="1H1  DIX"; $resn_exch{" H12 DIX"}="2H1  DIX"; $resn_exch{"H2'1 DIX"}="1H2' DIX";
$resn_exch{"H2'2 DIX"}="2H2' DIX"; $resn_exch{"H5'1 DIX"}="1H5' DIX"; $resn_exch{"H5'2 DIX"}="2H5' DIX";
$resn_exch{"H4'1 DIX"}="1H4' DIX"; $resn_exch{"H4'2 DIX"}="2H4' DIX"; $resn_exch{"H3'1 DIX"}="1H3' DIX";
$resn_exch{"H3'2 DIX"}="2H3' DIX"; $resn_exch{" H21 DIY"}="1H2  DIY"; $resn_exch{" H22 DIY"}="2H2  DIY";
$resn_exch{" H61 DIY"}="1H6  DIY"; $resn_exch{" H62 DIY"}="2H6  DIY"; $resn_exch{" H31 DIY"}="1H3  DIY";
$resn_exch{" H32 DIY"}="2H3  DIY"; $resn_exch{" H41 DIY"}="1H4  DIY"; $resn_exch{" H42 DIY"}="2H4  DIY";
$resn_exch{"H1'1 DIY"}="1H1' DIY"; $resn_exch{"H1'2 DIY"}="2H1' DIY"; $resn_exch{"H2'1 DIY"}="1H2' DIY";
$resn_exch{"H2'2 DIY"}="2H2' DIY"; $resn_exch{"H3'1 DIY"}="1H3' DIY"; $resn_exch{"H3'2 DIY"}="2H3' DIY";
$resn_exch{"H4'1 DIY"}="1H4' DIY"; $resn_exch{"H4'2 DIY"}="2H4' DIY"; $resn_exch{"H4'3 DIY"}="3H4' DIY";
$resn_exch{"H511 DJK"}="1H51 DJK"; $resn_exch{"H512 DJK"}="2H51 DJK"; $resn_exch{"H111 DJK"}="1H11 DJK";
$resn_exch{" H21 DJR"}="1H2  DJR"; $resn_exch{"H22A DJR"}="2H2  DJR"; $resn_exch{"H11A DJR"}="1H1  DJR";
$resn_exch{"H12A DJR"}="2H1  DJR"; $resn_exch{" H31 DJR"}="1H3  DJR"; $resn_exch{" H32 DJR"}="2H3  DJR";
$resn_exch{" H81 DJR"}="1H8  DJR"; $resn_exch{" H82 DJR"}="2H8  DJR"; $resn_exch{"H151 DJR"}="1H15 DJR";
$resn_exch{"H152 DJR"}="2H15 DJR"; $resn_exch{"H261 DJR"}="1H26 DJR"; $resn_exch{"H262 DJR"}="2H26 DJR";
$resn_exch{"H263 DJR"}="3H26 DJR"; $resn_exch{"H161 DJR"}="1H16 DJR"; $resn_exch{"H162 DJR"}="2H16 DJR";
$resn_exch{"H251 DJR"}="1H25 DJR"; $resn_exch{"H252 DJR"}="2H25 DJR"; $resn_exch{"H253 DJR"}="3H25 DJR";
$resn_exch{"H241 DJR"}="1H24 DJR"; $resn_exch{"H242 DJR"}="2H24 DJR"; $resn_exch{"H243 DJR"}="3H24 DJR";
$resn_exch{"H161 DK2"}="1H16 DK2"; $resn_exch{"H162 DK2"}="2H16 DK2"; $resn_exch{" H21 DKA"}="1H2  DKA";
$resn_exch{" H22 DKA"}="2H2  DKA"; $resn_exch{" H31 DKA"}="1H3  DKA"; $resn_exch{" H32 DKA"}="2H3  DKA";
$resn_exch{" H41 DKA"}="1H4  DKA"; $resn_exch{" H42 DKA"}="2H4  DKA"; $resn_exch{" H51 DKA"}="1H5  DKA";
$resn_exch{" H52 DKA"}="2H5  DKA"; $resn_exch{" H61 DKA"}="1H6  DKA"; $resn_exch{" H62 DKA"}="2H6  DKA";
$resn_exch{" H71 DKA"}="1H7  DKA"; $resn_exch{" H72 DKA"}="2H7  DKA"; $resn_exch{" H81 DKA"}="1H8  DKA";
$resn_exch{" H82 DKA"}="2H8  DKA"; $resn_exch{" H91 DKA"}="1H9  DKA"; $resn_exch{" H92 DKA"}="2H9  DKA";
$resn_exch{"H101 DKA"}="1H10 DKA"; $resn_exch{"H102 DKA"}="2H10 DKA"; $resn_exch{"H103 DKA"}="3H10 DKA";
$resn_exch{"HAA1 DKI"}="1HAA DKI"; $resn_exch{"HAA2 DKI"}="2HAA DKI"; $resn_exch{"HAB1 DKI"}="1HAB DKI";
$resn_exch{"HAB2 DKI"}="2HAB DKI"; $resn_exch{"HC21 DKT"}="1HC2 DKT"; $resn_exch{"HC22 DKT"}="2HC2 DKT";
$resn_exch{"HB11 DKT"}="1HB1 DKT"; $resn_exch{"HB12 DKT"}="2HB1 DKT"; $resn_exch{"HD11 DKT"}="1HD1 DKT";
$resn_exch{"HD12 DKT"}="2HD1 DKT"; $resn_exch{"HD21 DKT"}="1HD2 DKT"; $resn_exch{"HD22 DKT"}="2HD2 DKT";
$resn_exch{"HE11 DKT"}="1HE1 DKT"; $resn_exch{"HE12 DKT"}="2HE1 DKT"; $resn_exch{"HE21 DKT"}="1HE2 DKT";
$resn_exch{"HE22 DKT"}="2HE2 DKT"; $resn_exch{"HZ11 DKT"}="1HZ1 DKT"; $resn_exch{"HZ12 DKT"}="2HZ1 DKT";
$resn_exch{"HB21 DKT"}="1HB2 DKT"; $resn_exch{"HB22 DKT"}="2HB2 DKT"; $resn_exch{"HG21 DKT"}="1HG2 DKT";
$resn_exch{"HG22 DKT"}="2HG2 DKT"; $resn_exch{"HD31 DKT"}="1HD3 DKT"; $resn_exch{"HD32 DKT"}="2HD3 DKT";
$resn_exch{"HH11 DKT"}="1HH1 DKT"; $resn_exch{"HH12 DKT"}="2HH1 DKT"; $resn_exch{"HH21 DKT"}="1HH2 DKT";
$resn_exch{"HB31 DKT"}="1HB3 DKT"; $resn_exch{"HB32 DKT"}="2HB3 DKT"; $resn_exch{"HG31 DKT"}="1HG3 DKT";
$resn_exch{"HG32 DKT"}="2HG3 DKT"; $resn_exch{"HB41 DKT"}="1HB4 DKT"; $resn_exch{"HB42 DKT"}="2HB4 DKT";
$resn_exch{"HD51 DKT"}="1HD5 DKT"; $resn_exch{"HD52 DKT"}="2HD5 DKT"; $resn_exch{"HD61 DKT"}="1HD6 DKT";
$resn_exch{"HD62 DKT"}="2HD6 DKT"; $resn_exch{"HE61 DKT"}="1HE6 DKT"; $resn_exch{"HE62 DKT"}="2HE6 DKT";
$resn_exch{"HE71 DKT"}="1HE7 DKT"; $resn_exch{"HE72 DKT"}="2HE7 DKT"; $resn_exch{"HZ31 DKT"}="1HZ3 DKT";
$resn_exch{"HZ32 DKT"}="2HZ3 DKT"; $resn_exch{" H61 DL6"}="1H6  DL6"; $resn_exch{" H62 DL6"}="2H6  DL6";
$resn_exch{" H81 DL6"}="1H8  DL6"; $resn_exch{" H82 DL6"}="2H8  DL6"; $resn_exch{" H81 DL8"}="1H8  DL8";
$resn_exch{" H82 DL8"}="2H8  DL8"; $resn_exch{" H61 DL8"}="1H6  DL8"; $resn_exch{" H62 DL8"}="2H6  DL8";
$resn_exch{" H31 DLA"}="1H3  DLA"; $resn_exch{" H32 DLA"}="2H3  DLA"; $resn_exch{" H33 DLA"}="3H3  DLA";
$resn_exch{" H2  DLE"}=" HN2 DLE"; $resn_exch{" HB2 DLE"}="1HB  DLE"; $resn_exch{" HB3 DLE"}="2HB  DLE";
$resn_exch{"HD11 DLE"}="1HD1 DLE"; $resn_exch{"HD12 DLE"}="2HD1 DLE"; $resn_exch{"HD13 DLE"}="3HD1 DLE";
$resn_exch{"HD21 DLE"}="1HD2 DLE"; $resn_exch{"HD22 DLE"}="2HD2 DLE"; $resn_exch{"HD23 DLE"}="3HD2 DLE";
$resn_exch{" H21 DLF"}="1H2  DLF"; $resn_exch{" H22 DLF"}="2H2  DLF"; $resn_exch{" H61 DLF"}="1H6  DLF";
$resn_exch{" H62 DLF"}="2H6  DLF"; $resn_exch{" H63 DLF"}="3H6  DLF"; $resn_exch{" H31 DLG"}="1H3  DLG";
$resn_exch{" H32 DLG"}="2H3  DLG"; $resn_exch{" H61 DLG"}="1H6  DLG"; $resn_exch{" H62 DLG"}="2H6  DLG";
$resn_exch{"H111 DLG"}="1H11 DLG"; $resn_exch{"H112 DLG"}="2H11 DLG"; $resn_exch{"H121 DLG"}="1H12 DLG";
$resn_exch{"H122 DLG"}="2H12 DLG"; $resn_exch{"H131 DLG"}="1H13 DLG"; $resn_exch{"H132 DLG"}="2H13 DLG";
$resn_exch{"H141 DLG"}="1H14 DLG"; $resn_exch{"H142 DLG"}="2H14 DLG"; $resn_exch{"H151 DLG"}="1H15 DLG";
$resn_exch{"H152 DLG"}="2H15 DLG"; $resn_exch{"H161 DLG"}="1H16 DLG"; $resn_exch{"H162 DLG"}="2H16 DLG";
$resn_exch{"H163 DLG"}="3H16 DLG"; $resn_exch{" H81 DLI"}="1H8  DLI"; $resn_exch{" H82 DLI"}="2H8  DLI";
$resn_exch{"H101 DLI"}="1H10 DLI"; $resn_exch{"H102 DLI"}="2H10 DLI"; $resn_exch{"H103 DLI"}="3H10 DLI";
$resn_exch{"H121 DLI"}="1H12 DLI"; $resn_exch{"H122 DLI"}="2H12 DLI"; $resn_exch{"H221 DLI"}="1H22 DLI";
$resn_exch{"H222 DLI"}="2H22 DLI"; $resn_exch{"HC11 DLP"}="1HC1 DLP"; $resn_exch{"HC12 DLP"}="2HC1 DLP";
$resn_exch{"HC31 DLP"}="1HC3 DLP"; $resn_exch{"HC32 DLP"}="2HC3 DLP"; $resn_exch{"HC41 DLP"}="1HC4 DLP";
$resn_exch{"HC42 DLP"}="2HC4 DLP"; $resn_exch{"HC51 DLP"}="1HC5 DLP"; $resn_exch{"HC52 DLP"}="2HC5 DLP";
$resn_exch{"HC61 DLP"}="1HC6 DLP"; $resn_exch{"HC62 DLP"}="2HC6 DLP"; $resn_exch{"HC63 DLP"}="3HC6 DLP";
$resn_exch{"HC71 DLP"}="1HC7 DLP"; $resn_exch{"HC72 DLP"}="2HC7 DLP"; $resn_exch{"HC73 DLP"}="3HC7 DLP";
$resn_exch{"HC81 DLP"}="1HC8 DLP"; $resn_exch{"HC82 DLP"}="2HC8 DLP"; $resn_exch{"HC83 DLP"}="3HC8 DLP";
$resn_exch{"H121 DLP"}="1H12 DLP"; $resn_exch{"H122 DLP"}="2H12 DLP"; $resn_exch{"H131 DLP"}="1H13 DLP";
$resn_exch{"H132 DLP"}="2H13 DLP"; $resn_exch{"H141 DLP"}="1H14 DLP"; $resn_exch{"H142 DLP"}="2H14 DLP";
$resn_exch{"H151 DLP"}="1H15 DLP"; $resn_exch{"H152 DLP"}="2H15 DLP"; $resn_exch{"H161 DLP"}="1H16 DLP";
$resn_exch{"H162 DLP"}="2H16 DLP"; $resn_exch{"H171 DLP"}="1H17 DLP"; $resn_exch{"H172 DLP"}="2H17 DLP";
$resn_exch{"H181 DLP"}="1H18 DLP"; $resn_exch{"H182 DLP"}="2H18 DLP"; $resn_exch{"H211 DLP"}="1H21 DLP";
$resn_exch{"H212 DLP"}="2H21 DLP"; $resn_exch{"H241 DLP"}="1H24 DLP"; $resn_exch{"H242 DLP"}="2H24 DLP";
$resn_exch{"H251 DLP"}="1H25 DLP"; $resn_exch{"H252 DLP"}="2H25 DLP"; $resn_exch{"H261 DLP"}="1H26 DLP";
$resn_exch{"H262 DLP"}="2H26 DLP"; $resn_exch{"H271 DLP"}="1H27 DLP"; $resn_exch{"H272 DLP"}="2H27 DLP";
$resn_exch{"H281 DLP"}="1H28 DLP"; $resn_exch{"H282 DLP"}="2H28 DLP"; $resn_exch{"H283 DLP"}="3H28 DLP";
$resn_exch{"H321 DLP"}="1H32 DLP"; $resn_exch{"H322 DLP"}="2H32 DLP"; $resn_exch{"H331 DLP"}="1H33 DLP";
$resn_exch{"H332 DLP"}="2H33 DLP"; $resn_exch{"H341 DLP"}="1H34 DLP"; $resn_exch{"H342 DLP"}="2H34 DLP";
$resn_exch{"H351 DLP"}="1H35 DLP"; $resn_exch{"H352 DLP"}="2H35 DLP"; $resn_exch{"H361 DLP"}="1H36 DLP";
$resn_exch{"H362 DLP"}="2H36 DLP"; $resn_exch{"H371 DLP"}="1H37 DLP"; $resn_exch{"H372 DLP"}="2H37 DLP";
$resn_exch{"H381 DLP"}="1H38 DLP"; $resn_exch{"H382 DLP"}="2H38 DLP"; $resn_exch{"H411 DLP"}="1H41 DLP";
$resn_exch{"H412 DLP"}="2H41 DLP"; $resn_exch{"H441 DLP"}="1H44 DLP"; $resn_exch{"H442 DLP"}="2H44 DLP";
$resn_exch{"H451 DLP"}="1H45 DLP"; $resn_exch{"H452 DLP"}="2H45 DLP"; $resn_exch{"H461 DLP"}="1H46 DLP";
$resn_exch{"H462 DLP"}="2H46 DLP"; $resn_exch{"H471 DLP"}="1H47 DLP"; $resn_exch{"H472 DLP"}="2H47 DLP";
$resn_exch{"H481 DLP"}="1H48 DLP"; $resn_exch{"H482 DLP"}="2H48 DLP"; $resn_exch{"H483 DLP"}="3H48 DLP";
$resn_exch{" H31 DLQ"}="1H3  DLQ"; $resn_exch{" H32 DLQ"}="2H3  DLQ"; $resn_exch{"H33A DLQ"}="3H3  DLQ";
$resn_exch{" H51 DLQ"}="1H5  DLQ"; $resn_exch{" H52 DLQ"}="2H5  DLQ"; $resn_exch{" H71 DLQ"}="1H7  DLQ";
$resn_exch{" H72 DLQ"}="2H7  DLQ"; $resn_exch{" H73 DLQ"}="3H7  DLQ"; $resn_exch{" H81 DLQ"}="1H8  DLQ";
$resn_exch{" H82 DLQ"}="2H8  DLQ"; $resn_exch{" H83 DLQ"}="3H8  DLQ"; $resn_exch{"H141 DLQ"}="1H14 DLQ";
$resn_exch{"H142 DLQ"}="2H14 DLQ"; $resn_exch{"H143 DLQ"}="3H14 DLQ"; $resn_exch{"H171 DLQ"}="1H17 DLQ";
$resn_exch{"H172 DLQ"}="2H17 DLQ"; $resn_exch{"H173 DLQ"}="3H17 DLQ"; $resn_exch{"H181 DLQ"}="1H18 DLQ";
$resn_exch{"H182 DLQ"}="2H18 DLQ"; $resn_exch{"H183 DLQ"}="3H18 DLQ"; $resn_exch{"H191 DLQ"}="1H19 DLQ";
$resn_exch{"H192 DLQ"}="2H19 DLQ"; $resn_exch{"H193 DLQ"}="3H19 DLQ"; $resn_exch{"H201 DLQ"}="1H20 DLQ";
$resn_exch{"H202 DLQ"}="2H20 DLQ"; $resn_exch{"H203 DLQ"}="3H20 DLQ"; $resn_exch{"H211 DLQ"}="1H21 DLQ";
$resn_exch{"H212 DLQ"}="2H21 DLQ"; $resn_exch{"H213 DLQ"}="3H21 DLQ"; $resn_exch{"H251 DLQ"}="1H25 DLQ";
$resn_exch{"H252 DLQ"}="2H25 DLQ"; $resn_exch{"H261 DLQ"}="1H26 DLQ"; $resn_exch{"H262 DLQ"}="2H26 DLQ";
$resn_exch{"H281 DLQ"}="1H28 DLQ"; $resn_exch{"H282 DLQ"}="2H28 DLQ"; $resn_exch{"H311 DLQ"}="1H31 DLQ";
$resn_exch{"H312 DLQ"}="2H31 DLQ"; $resn_exch{"H401 DLQ"}="1H40 DLQ"; $resn_exch{"H402 DLQ"}="2H40 DLQ";
$resn_exch{"H411 DLQ"}="1H41 DLQ"; $resn_exch{"H412 DLQ"}="2H41 DLQ"; $resn_exch{"H451 DLQ"}="1H45 DLQ";
$resn_exch{"H452 DLQ"}="2H45 DLQ"; $resn_exch{"H453 DLQ"}="3H45 DLQ"; $resn_exch{"H461 DLQ"}="1H46 DLQ";
$resn_exch{"H462 DLQ"}="2H46 DLQ"; $resn_exch{"H463 DLQ"}="3H46 DLQ"; $resn_exch{" H21 DLS"}="1H2  DLS";
$resn_exch{" H22 DLS"}="2H2  DLS"; $resn_exch{" H23 DLS"}="3H2  DLS"; $resn_exch{" H   DLS"}=" HN  DLS";
$resn_exch{" HB2 DLS"}="1HCB DLS"; $resn_exch{" HB3 DLS"}="2HCB DLS"; $resn_exch{" HG2 DLS"}="1HCG DLS";
$resn_exch{" HG3 DLS"}="2HCG DLS"; $resn_exch{" HD2 DLS"}="1HCD DLS"; $resn_exch{" HD3 DLS"}="2HCD DLS";
$resn_exch{" HE2 DLS"}="1HCE DLS"; $resn_exch{" HE3 DLS"}="2HCE DLS"; $resn_exch{" HZ  DLS"}=" HNZ DLS";
$resn_exch{"HH31 DLS"}="1HH3 DLS"; $resn_exch{"HH32 DLS"}="2HH3 DLS"; $resn_exch{"HH33 DLS"}="3HH3 DLS";
$resn_exch{" H2  DLY"}=" HN2 DLY"; $resn_exch{" HB2 DLY"}="1HB  DLY"; $resn_exch{" HB3 DLY"}="2HB  DLY";
$resn_exch{" HG2 DLY"}="1HG  DLY"; $resn_exch{" HG3 DLY"}="2HG  DLY"; $resn_exch{" HD2 DLY"}="1HD  DLY";
$resn_exch{" HD3 DLY"}="2HD  DLY"; $resn_exch{" HE2 DLY"}="1HE  DLY"; $resn_exch{" HE3 DLY"}="2HE  DLY";
$resn_exch{" HZ1 DLY"}="1HZ  DLY"; $resn_exch{" HZ2 DLY"}="2HZ  DLY"; $resn_exch{"H111 DM1"}="1H11 DM1";
$resn_exch{"H112 DM1"}="2H11 DM1"; $resn_exch{"HO12 DM1"}="2HO1 DM1"; $resn_exch{"H141 DM1"}="1H14 DM1";
$resn_exch{"H142 DM1"}="2H14 DM1"; $resn_exch{"H143 DM1"}="3H14 DM1"; $resn_exch{"H151 DM1"}="1H15 DM1";
$resn_exch{"H152 DM1"}="2H15 DM1"; $resn_exch{"HO17 DM1"}="7HO1 DM1"; $resn_exch{"H211 DM1"}="1H21 DM1";
$resn_exch{"H212 DM1"}="2H21 DM1"; $resn_exch{"H213 DM1"}="3H21 DM1"; $resn_exch{" H1' DM1"}=" H1* DM1";
$resn_exch{"H2'1 DM1"}="1H2* DM1"; $resn_exch{"H2'2 DM1"}="2H2* DM1"; $resn_exch{" H3' DM1"}=" H3* DM1";
$resn_exch{"HN'1 DM1"}="1HN* DM1"; $resn_exch{"HN'2 DM1"}="2HN* DM1"; $resn_exch{" H4' DM1"}=" H4* DM1";
$resn_exch{"HO4' DM1"}="*HO4 DM1"; $resn_exch{" H5' DM1"}=" H5* DM1"; $resn_exch{"H6'1 DM1"}="1H6* DM1";
$resn_exch{"H6'2 DM1"}="2H6* DM1"; $resn_exch{"H6'3 DM1"}="3H6* DM1"; $resn_exch{"H111 DM2"}="1H11 DM2";
$resn_exch{"H112 DM2"}="2H11 DM2"; $resn_exch{"HO12 DM2"}="2HO1 DM2"; $resn_exch{"H141 DM2"}="1H14 DM2";
$resn_exch{"H142 DM2"}="2H14 DM2"; $resn_exch{"HO14 DM2"}="4HO1 DM2"; $resn_exch{"H151 DM2"}="1H15 DM2";
$resn_exch{"H152 DM2"}="2H15 DM2"; $resn_exch{"HO17 DM2"}="7HO1 DM2"; $resn_exch{"H211 DM2"}="1H21 DM2";
$resn_exch{"H212 DM2"}="2H21 DM2"; $resn_exch{"H213 DM2"}="3H21 DM2"; $resn_exch{" H1' DM2"}=" H1* DM2";
$resn_exch{"H2'1 DM2"}="1H2* DM2"; $resn_exch{"H2'2 DM2"}="2H2* DM2"; $resn_exch{" H3' DM2"}=" H3* DM2";
$resn_exch{"HN'1 DM2"}="1HN* DM2"; $resn_exch{"HN'2 DM2"}="2HN* DM2"; $resn_exch{" H4' DM2"}=" H4* DM2";
$resn_exch{"HO4' DM2"}="*HO4 DM2"; $resn_exch{" H5' DM2"}=" H5* DM2"; $resn_exch{"H6'1 DM2"}="1H6* DM2";
$resn_exch{"H6'2 DM2"}="2H6* DM2"; $resn_exch{"H6'3 DM2"}="3H6* DM2"; $resn_exch{"H111 DM3"}="1H11 DM3";
$resn_exch{"H112 DM3"}="2H11 DM3"; $resn_exch{"HO12 DM3"}="2HO1 DM3"; $resn_exch{"H141 DM3"}="1H14 DM3";
$resn_exch{"H142 DM3"}="2H14 DM3"; $resn_exch{"H143 DM3"}="3H14 DM3"; $resn_exch{"H151 DM3"}="1H15 DM3";
$resn_exch{"H152 DM3"}="2H15 DM3"; $resn_exch{"H211 DM3"}="1H21 DM3"; $resn_exch{"H212 DM3"}="2H21 DM3";
$resn_exch{"H213 DM3"}="3H21 DM3"; $resn_exch{" H1' DM3"}=" H1* DM3"; $resn_exch{"H2'1 DM3"}="1H2* DM3";
$resn_exch{"H2'2 DM3"}="2H2* DM3"; $resn_exch{" H3' DM3"}=" H3* DM3"; $resn_exch{"HN'1 DM3"}="1HN* DM3";
$resn_exch{"HN'2 DM3"}="2HN* DM3"; $resn_exch{" H4' DM3"}=" H4* DM3"; $resn_exch{"HO4' DM3"}="*HO4 DM3";
$resn_exch{" H5' DM3"}=" H5* DM3"; $resn_exch{"H6'1 DM3"}="1H6* DM3"; $resn_exch{"H6'2 DM3"}="2H6* DM3";
$resn_exch{"H6'3 DM3"}="3H6* DM3"; $resn_exch{"H111 DM4"}="1H11 DM4"; $resn_exch{"H112 DM4"}="2H11 DM4";
$resn_exch{"HO12 DM4"}="2HO1 DM4"; $resn_exch{"H141 DM4"}="1H14 DM4"; $resn_exch{"H142 DM4"}="2H14 DM4";
$resn_exch{"HO14 DM4"}="4HO1 DM4"; $resn_exch{"H151 DM4"}="1H15 DM4"; $resn_exch{"H152 DM4"}="2H15 DM4";
$resn_exch{" H1' DM4"}=" H1* DM4"; $resn_exch{"H2'1 DM4"}="1H2* DM4"; $resn_exch{"H2'2 DM4"}="2H2* DM4";
$resn_exch{" H3' DM4"}=" H3* DM4"; $resn_exch{"HN'1 DM4"}="1HN* DM4"; $resn_exch{"HN'2 DM4"}="2HN* DM4";
$resn_exch{" H4' DM4"}=" H4* DM4"; $resn_exch{"HO4' DM4"}="*HO4 DM4"; $resn_exch{" H5' DM4"}=" H5* DM4";
$resn_exch{"H6'1 DM4"}="1H6* DM4"; $resn_exch{"H6'2 DM4"}="2H6* DM4"; $resn_exch{"H6'3 DM4"}="3H6* DM4";
$resn_exch{"H111 DM5"}="1H11 DM5"; $resn_exch{"H112 DM5"}="2H11 DM5"; $resn_exch{"HO12 DM5"}="2HO1 DM5";
$resn_exch{"H141 DM5"}="1H14 DM5"; $resn_exch{"H142 DM5"}="2H14 DM5"; $resn_exch{"H143 DM5"}="3H14 DM5";
$resn_exch{"H151 DM5"}="1H15 DM5"; $resn_exch{"H152 DM5"}="2H15 DM5"; $resn_exch{"HO17 DM5"}="7HO1 DM5";
$resn_exch{" H1' DM5"}=" H1* DM5"; $resn_exch{"H2'1 DM5"}="1H2* DM5"; $resn_exch{"H2'2 DM5"}="2H2* DM5";
$resn_exch{" H3' DM5"}=" H3* DM5"; $resn_exch{"HN'1 DM5"}="1HN* DM5"; $resn_exch{"HN'2 DM5"}="2HN* DM5";
$resn_exch{" H4' DM5"}=" H4* DM5"; $resn_exch{"HO4' DM5"}="*HO4 DM5"; $resn_exch{" H5' DM5"}=" H5* DM5";
$resn_exch{"H6'1 DM5"}="1H6* DM5"; $resn_exch{"H6'2 DM5"}="2H6* DM5"; $resn_exch{"H6'3 DM5"}="3H6* DM5";
$resn_exch{" H81 DM6"}="1H8  DM6"; $resn_exch{" H82 DM6"}="2H8  DM6"; $resn_exch{"H101 DM6"}="1H10 DM6";
$resn_exch{"H102 DM6"}="2H10 DM6"; $resn_exch{"H141 DM6"}="1H14 DM6"; $resn_exch{"H142 DM6"}="2H14 DM6";
$resn_exch{"H211 DM6"}="1H21 DM6"; $resn_exch{"H212 DM6"}="2H21 DM6"; $resn_exch{"H213 DM6"}="3H21 DM6";
$resn_exch{" H1' DM6"}=" H1* DM6"; $resn_exch{"H2'1 DM6"}="1H2* DM6"; $resn_exch{"H2'2 DM6"}="2H2* DM6";
$resn_exch{" H3' DM6"}=" H3* DM6"; $resn_exch{" H4' DM6"}=" H4* DM6"; $resn_exch{" H5' DM6"}=" H5* DM6";
$resn_exch{"H6'1 DM6"}="1H6* DM6"; $resn_exch{"H6'2 DM6"}="2H6* DM6"; $resn_exch{"H6'3 DM6"}="3H6* DM6";
$resn_exch{"HO'4 DM6"}="4HO* DM6"; $resn_exch{"HN'1 DM6"}="1HN* DM6"; $resn_exch{"HN'2 DM6"}="2HN* DM6";
$resn_exch{"HN'3 DM6"}="3HN* DM6"; $resn_exch{" H81 DM7"}="1H8  DM7"; $resn_exch{" H82 DM7"}="2H8  DM7";
$resn_exch{"H101 DM7"}="1H10 DM7"; $resn_exch{"H102 DM7"}="2H10 DM7"; $resn_exch{"H141 DM7"}="1H14 DM7";
$resn_exch{"H142 DM7"}="2H14 DM7"; $resn_exch{"H211 DM7"}="1H21 DM7"; $resn_exch{"H212 DM7"}="2H21 DM7";
$resn_exch{"H213 DM7"}="3H21 DM7"; $resn_exch{" H1' DM7"}=" H1* DM7"; $resn_exch{"H2'1 DM7"}="1H2* DM7";
$resn_exch{"H2'2 DM7"}="2H2* DM7"; $resn_exch{" H3' DM7"}=" H3* DM7"; $resn_exch{" H4' DM7"}=" H4* DM7";
$resn_exch{" H5' DM7"}=" H5* DM7"; $resn_exch{"H6'1 DM7"}="1H6* DM7"; $resn_exch{"H6'2 DM7"}="2H6* DM7";
$resn_exch{"H6'3 DM7"}="3H6* DM7"; $resn_exch{"HN'1 DM7"}="1HN* DM7"; $resn_exch{"HN'2 DM7"}="2HN* DM7";
$resn_exch{"HN'3 DM7"}="3HN* DM7"; $resn_exch{" H81 DM8"}="1H8  DM8"; $resn_exch{" H82 DM8"}="2H8  DM8";
$resn_exch{"H101 DM8"}="1H10 DM8"; $resn_exch{"H102 DM8"}="2H10 DM8"; $resn_exch{"H141 DM8"}="1H14 DM8";
$resn_exch{"H142 DM8"}="2H14 DM8"; $resn_exch{"H143 DM8"}="3H14 DM8"; $resn_exch{"H211 DM8"}="1H21 DM8";
$resn_exch{"H212 DM8"}="2H21 DM8"; $resn_exch{"H213 DM8"}="3H21 DM8"; $resn_exch{" H1' DM8"}=" H1* DM8";
$resn_exch{" H2' DM8"}=" H2* DM8"; $resn_exch{" H3' DM8"}=" H3* DM8"; $resn_exch{" H4' DM8"}=" H4* DM8";
$resn_exch{" H5' DM8"}=" H5* DM8"; $resn_exch{"H6'1 DM8"}="1H6* DM8"; $resn_exch{"H6'2 DM8"}="2H6* DM8";
$resn_exch{"H6'3 DM8"}="3H6* DM8"; $resn_exch{"HO4' DM8"}="*HO4 DM8"; $resn_exch{"HN'1 DM8"}="1HN* DM8";
$resn_exch{"HN'2 DM8"}="2HN* DM8"; $resn_exch{" H81 DM9"}="1H8  DM9"; $resn_exch{" H82 DM9"}="2H8  DM9";
$resn_exch{"H101 DM9"}="1H10 DM9"; $resn_exch{"H102 DM9"}="2H10 DM9"; $resn_exch{"H141 DM9"}="1H14 DM9";
$resn_exch{"H142 DM9"}="2H14 DM9"; $resn_exch{"H143 DM9"}="3H14 DM9"; $resn_exch{"H211 DM9"}="1H21 DM9";
$resn_exch{"H212 DM9"}="2H21 DM9"; $resn_exch{"H213 DM9"}="3H21 DM9"; $resn_exch{" H1' DM9"}=" H1* DM9";
$resn_exch{"H2'1 DM9"}="1H2* DM9"; $resn_exch{"H2'2 DM9"}="2H2* DM9"; $resn_exch{" H3' DM9"}=" H3* DM9";
$resn_exch{" H4' DM9"}=" H4* DM9"; $resn_exch{"HO4' DM9"}="HO4* DM9"; $resn_exch{" H5' DM9"}=" H5* DM9";
$resn_exch{"H6'1 DM9"}="1H6* DM9"; $resn_exch{"H6'2 DM9"}="2H6* DM9"; $resn_exch{"H6'3 DM9"}="3H6* DM9";
$resn_exch{"H311 DM9"}="1H31 DM9"; $resn_exch{"H312 DM9"}="2H31 DM9"; $resn_exch{"H411 DM9"}="1H41 DM9";
$resn_exch{"H412 DM9"}="2H41 DM9"; $resn_exch{"H421 DM9"}="1H42 DM9"; $resn_exch{"H422 DM9"}="2H42 DM9";
$resn_exch{"H431 DM9"}="1H43 DM9"; $resn_exch{"H432 DM9"}="2H43 DM9"; $resn_exch{"H433 DM9"}="3H43 DM9";
$resn_exch{" H11 DMA"}="1H1  DMA"; $resn_exch{" H12 DMA"}="2H1  DMA"; $resn_exch{" H41 DMA"}="1H4  DMA";
$resn_exch{" H42 DMA"}="2H4  DMA"; $resn_exch{" H43 DMA"}="3H4  DMA"; $resn_exch{" H51 DMA"}="1H5  DMA";
$resn_exch{" H52 DMA"}="2H5  DMA"; $resn_exch{" H53 DMA"}="3H5  DMA"; $resn_exch{"HOA2 DMA"}="2HOA DMA";
$resn_exch{"HOB2 DMA"}="2HOB DMA"; $resn_exch{"HOB3 DMA"}="3HOB DMA"; $resn_exch{"HM31 DMB"}="1HM3 DMB";
$resn_exch{"HM32 DMB"}="2HM3 DMB"; $resn_exch{"HM33 DMB"}="3HM3 DMB"; $resn_exch{"HM51 DMB"}="1HM5 DMB";
$resn_exch{"HM52 DMB"}="2HM5 DMB"; $resn_exch{"HM53 DMB"}="3HM5 DMB"; $resn_exch{"HO4' DMB"}="'HO4 DMB";
$resn_exch{"HB21 DMC"}="1HB2 DMC"; $resn_exch{"HB22 DMC"}="2HB2 DMC"; $resn_exch{"HB23 DMC"}="3HB2 DMC";
$resn_exch{"HH11 DMC"}="1HH1 DMC"; $resn_exch{"HH12 DMC"}="2HH1 DMC"; $resn_exch{"HH21 DMC"}="1HH2 DMC";
$resn_exch{"HH22 DMC"}="2HH2 DMC"; $resn_exch{"HM11 DMC"}="1HM1 DMC"; $resn_exch{"HM12 DMC"}="2HM1 DMC";
$resn_exch{"HM13 DMC"}="3HM1 DMC"; $resn_exch{"HM21 DMC"}="1HM2 DMC"; $resn_exch{"HM22 DMC"}="2HM2 DMC";
$resn_exch{"HM23 DMC"}="3HM2 DMC"; $resn_exch{"HN31 DMD"}="1HN3 DMD"; $resn_exch{" H21 DMD"}="1H2  DMD";
$resn_exch{" H41 DMD"}="1H4  DMD"; $resn_exch{" H71 DMD"}="1H7  DMD"; $resn_exch{" H81 DMD"}="1H8  DMD";
$resn_exch{" H82 DMD"}="2H8  DMD"; $resn_exch{" H83 DMD"}="3H8  DMD"; $resn_exch{" H91 DMD"}="1H9  DMD";
$resn_exch{" H92 DMD"}="2H9  DMD"; $resn_exch{" H93 DMD"}="3H9  DMD"; $resn_exch{" H21 DME"}="1H2  DME";
$resn_exch{" H22 DME"}="2H2  DME"; $resn_exch{" H31 DME"}="1H3  DME"; $resn_exch{" H32 DME"}="2H3  DME";
$resn_exch{" H41 DME"}="1H4  DME"; $resn_exch{" H42 DME"}="2H4  DME"; $resn_exch{" H51 DME"}="1H5  DME";
$resn_exch{" H52 DME"}="2H5  DME"; $resn_exch{" H61 DME"}="1H6  DME"; $resn_exch{" H62 DME"}="2H6  DME";
$resn_exch{" H71 DME"}="1H7  DME"; $resn_exch{" H72 DME"}="2H7  DME"; $resn_exch{" H81 DME"}="1H8  DME";
$resn_exch{" H82 DME"}="2H8  DME"; $resn_exch{" H91 DME"}="1H9  DME"; $resn_exch{" H92 DME"}="2H9  DME";
$resn_exch{"H101 DME"}="1H10 DME"; $resn_exch{"H102 DME"}="2H10 DME"; $resn_exch{"H111 DME"}="1H11 DME";
$resn_exch{"H112 DME"}="2H11 DME"; $resn_exch{"H131 DME"}="1H13 DME"; $resn_exch{"H132 DME"}="2H13 DME";
$resn_exch{"H133 DME"}="3H13 DME"; $resn_exch{"H141 DME"}="1H14 DME"; $resn_exch{"H142 DME"}="2H14 DME";
$resn_exch{"H143 DME"}="3H14 DME"; $resn_exch{"H151 DME"}="1H15 DME"; $resn_exch{"H152 DME"}="2H15 DME";
$resn_exch{"H153 DME"}="3H15 DME"; $resn_exch{"H161 DME"}="1H16 DME"; $resn_exch{"H162 DME"}="2H16 DME";
$resn_exch{"H163 DME"}="3H16 DME"; $resn_exch{"H171 DME"}="1H17 DME"; $resn_exch{"H172 DME"}="2H17 DME";
$resn_exch{"H173 DME"}="3H17 DME"; $resn_exch{"H181 DME"}="1H18 DME"; $resn_exch{"H182 DME"}="2H18 DME";
$resn_exch{"H183 DME"}="3H18 DME"; $resn_exch{" H11 DMF"}="1H1  DMF"; $resn_exch{" H12 DMF"}="2H1  DMF";
$resn_exch{" H13 DMF"}="3H1  DMF"; $resn_exch{" H21 DMF"}="1H2  DMF"; $resn_exch{" H22 DMF"}="2H2  DMF";
$resn_exch{" H23 DMF"}="3H2  DMF"; $resn_exch{" HA1 DMG"}="1HA  DMG"; $resn_exch{" HA2 DMG"}="2HA  DMG";
$resn_exch{" H41 DMG"}="1H4  DMG"; $resn_exch{" H42 DMG"}="2H4  DMG"; $resn_exch{" H43 DMG"}="3H4  DMG";
$resn_exch{" H51 DMG"}="1H5  DMG"; $resn_exch{" H52 DMG"}="2H5  DMG"; $resn_exch{" H53 DMG"}="3H5  DMG";
$resn_exch{" H2  DMH"}=" HN2 DMH"; $resn_exch{" HA1 DMH"}="1HA  DMH"; $resn_exch{" HB2 DMH"}="1HB  DMH";
$resn_exch{" HB3 DMH"}="2HB  DMH"; $resn_exch{"HE11 DMH"}="1HE1 DMH"; $resn_exch{"HE12 DMH"}="2HE1 DMH";
$resn_exch{"HE13 DMH"}="3HE1 DMH"; $resn_exch{"HE21 DMH"}="1HE2 DMH"; $resn_exch{"HE22 DMH"}="2HE2 DMH";
$resn_exch{"HE23 DMH"}="3HE2 DMH"; $resn_exch{"HM21 DMI"}="1HM2 DMI"; $resn_exch{"HM22 DMI"}="2HM2 DMI";
$resn_exch{"HM23 DMI"}="3HM2 DMI"; $resn_exch{"HM31 DMI"}="1HM3 DMI"; $resn_exch{"HM32 DMI"}="2HM3 DMI";
$resn_exch{"HM33 DMI"}="3HM3 DMI"; $resn_exch{" H11 DMJ"}="1H1  DMJ"; $resn_exch{" H12 DMJ"}="2H1  DMJ";
$resn_exch{" H61 DMJ"}="1H6  DMJ"; $resn_exch{" H62 DMJ"}="2H6  DMJ"; $resn_exch{" H   DMK"}="1HN  DMK";
$resn_exch{" H2  DMK"}="2HN  DMK"; $resn_exch{"HG21 DMK"}="1HG2 DMK"; $resn_exch{"HG22 DMK"}="2HG2 DMK";
$resn_exch{"HG23 DMK"}="3HG2 DMK"; $resn_exch{"HG31 DMK"}="1HG3 DMK"; $resn_exch{"HG32 DMK"}="2HG3 DMK";
$resn_exch{"HG33 DMK"}="3HG3 DMK"; $resn_exch{" HD1 DMK"}="OHD1 DMK"; $resn_exch{"HM11 DML"}="1HM1 DML";
$resn_exch{"HM12 DML"}="2HM1 DML"; $resn_exch{"HM13 DML"}="3HM1 DML"; $resn_exch{"HM21 DML"}="1HM2 DML";
$resn_exch{"HM22 DML"}="2HM2 DML"; $resn_exch{"HM23 DML"}="3HM2 DML"; $resn_exch{"H111 DMM"}="1H11 DMM";
$resn_exch{"H112 DMM"}="2H11 DMM"; $resn_exch{"HO12 DMM"}="2HO1 DMM"; $resn_exch{"H141 DMM"}="1H14 DMM";
$resn_exch{"H142 DMM"}="2H14 DMM"; $resn_exch{"HO14 DMM"}="4HO1 DMM"; $resn_exch{"H151 DMM"}="1H15 DMM";
$resn_exch{"H152 DMM"}="2H15 DMM"; $resn_exch{"HO17 DMM"}="7HO1 DMM"; $resn_exch{"H211 DMM"}="1H21 DMM";
$resn_exch{"H212 DMM"}="2H21 DMM"; $resn_exch{"H213 DMM"}="3H21 DMM"; $resn_exch{" H1' DMM"}=" H1* DMM";
$resn_exch{"H2'1 DMM"}="1H2* DMM"; $resn_exch{"H2'2 DMM"}="2H2* DMM"; $resn_exch{" H3' DMM"}=" H3* DMM";
$resn_exch{" H4' DMM"}=" H4* DMM"; $resn_exch{"HO4' DMM"}="*HO4 DMM"; $resn_exch{" H5' DMM"}=" H5* DMM";
$resn_exch{"H6'1 DMM"}="1H6* DMM"; $resn_exch{"H6'2 DMM"}="2H6* DMM"; $resn_exch{"H6'3 DMM"}="3H6* DMM";
$resn_exch{"H331 DMM"}="1H33 DMM"; $resn_exch{"H332 DMM"}="2H33 DMM"; $resn_exch{"H631 DMM"}="1H63 DMM";
$resn_exch{"H632 DMM"}="2H63 DMM"; $resn_exch{"H531 DMM"}="1H53 DMM"; $resn_exch{"H532 DMM"}="2H53 DMM";
$resn_exch{"H731 DMM"}="1H73 DMM"; $resn_exch{"H732 DMM"}="2H73 DMM"; $resn_exch{"H733 DMM"}="3H73 DMM";
$resn_exch{" H21 DMN"}="1H2  DMN"; $resn_exch{" H22 DMN"}="2H2  DMN"; $resn_exch{" H23 DMN"}="3H2  DMN";
$resn_exch{" H31 DMN"}="1H3  DMN"; $resn_exch{" H32 DMN"}="2H3  DMN"; $resn_exch{" H33 DMN"}="3H3  DMN";
$resn_exch{"HCO1 DMO"}="1HCO DMO"; $resn_exch{"HNO1 DMO"}="1HNO DMO"; $resn_exch{"HNO2 DMO"}="2HNO DMO";
$resn_exch{"HBO1 DMO"}="1HBO DMO"; $resn_exch{"HBO2 DMO"}="2HBO DMO"; $resn_exch{"HGO1 DMO"}="1HGO DMO";
$resn_exch{"HGO2 DMO"}="2HGO DMO"; $resn_exch{"HDO1 DMO"}="1HDO DMO"; $resn_exch{"HDO2 DMO"}="2HDO DMO";
$resn_exch{"HEO1 DMO"}="1HEO DMO"; $resn_exch{"HEO2 DMO"}="2HEO DMO"; $resn_exch{"H201 DMP"}="1H20 DMP";
$resn_exch{"H202 DMP"}="2H20 DMP"; $resn_exch{"H271 DMP"}="1H27 DMP"; $resn_exch{"H272 DMP"}="2H27 DMP";
$resn_exch{"H301 DMP"}="1H30 DMP"; $resn_exch{"H302 DMP"}="2H30 DMP"; $resn_exch{"H601 DMP"}="1H60 DMP";
$resn_exch{"H602 DMP"}="2H60 DMP"; $resn_exch{"H701 DMP"}="1H70 DMP"; $resn_exch{"H702 DMP"}="2H70 DMP";
$resn_exch{"H771 DMP"}="1H77 DMP"; $resn_exch{"H772 DMP"}="2H77 DMP"; $resn_exch{"HO27 DMP"}="7HO2 DMP";
$resn_exch{"HO77 DMP"}="7HO7 DMP"; $resn_exch{"H21A DMQ"}="1H2  DMQ"; $resn_exch{"H22A DMQ"}="2H2  DMQ";
$resn_exch{"H71A DMQ"}="1H7  DMQ"; $resn_exch{" H72 DMQ"}="2H7  DMQ"; $resn_exch{"HN21 DMQ"}="1HN2 DMQ";
$resn_exch{"HN22 DMQ"}="2HN2 DMQ"; $resn_exch{"H311 DMQ"}="1H31 DMQ"; $resn_exch{"H312 DMQ"}="2H31 DMQ";
$resn_exch{"H611 DMQ"}="1H61 DMQ"; $resn_exch{"H612 DMQ"}="2H61 DMQ"; $resn_exch{"HN71 DMQ"}="1HN7 DMQ";
$resn_exch{"HN72 DMQ"}="2HN7 DMQ"; $resn_exch{" H11 DMS"}="1H1  DMS"; $resn_exch{" H12 DMS"}="2H1  DMS";
$resn_exch{" H13 DMS"}="3H1  DMS"; $resn_exch{" H21 DMS"}="1H2  DMS"; $resn_exch{" H22 DMS"}="2H2  DMS";
$resn_exch{" H23 DMS"}="3H2  DMS"; $resn_exch{" H   DMT"}=" HN  DMT"; $resn_exch{"HCN1 DMT"}="1HCN DMT";
$resn_exch{"HCN2 DMT"}="2HCN DMT"; $resn_exch{"HCN3 DMT"}="3HCN DMT"; $resn_exch{"HD12 DMT"}="1HD1 DMT";
$resn_exch{"HD13 DMT"}="2HD1 DMT"; $resn_exch{"HD21 DMT"}="1HD2 DMT"; $resn_exch{"HD22 DMT"}="2HD2 DMT";
$resn_exch{"HD23 DMT"}="3HD2 DMT"; $resn_exch{"HD31 DMT"}="1HD3 DMT"; $resn_exch{"HD32 DMT"}="2HD3 DMT";
$resn_exch{"HD33 DMT"}="3HD3 DMT"; $resn_exch{" HH1 DMT"}="1HH  DMT"; $resn_exch{" HH2 DMT"}="2HH  DMT";
$resn_exch{" HH3 DMT"}="3HH  DMT"; $resn_exch{"H181 DMU"}="1H18 DMU"; $resn_exch{"H182 DMU"}="2H18 DMU";
$resn_exch{"H191 DMU"}="1H19 DMU"; $resn_exch{"H192 DMU"}="2H19 DMU"; $resn_exch{"H221 DMU"}="1H22 DMU";
$resn_exch{"H222 DMU"}="2H22 DMU"; $resn_exch{"H251 DMU"}="1H25 DMU"; $resn_exch{"H252 DMU"}="2H25 DMU";
$resn_exch{"H281 DMU"}="1H28 DMU"; $resn_exch{"H282 DMU"}="2H28 DMU"; $resn_exch{"H311 DMU"}="1H31 DMU";
$resn_exch{"H312 DMU"}="2H31 DMU"; $resn_exch{"H341 DMU"}="1H34 DMU"; $resn_exch{"H342 DMU"}="2H34 DMU";
$resn_exch{"H371 DMU"}="1H37 DMU"; $resn_exch{"H372 DMU"}="2H37 DMU"; $resn_exch{"H401 DMU"}="1H40 DMU";
$resn_exch{"H402 DMU"}="2H40 DMU"; $resn_exch{"H431 DMU"}="1H43 DMU"; $resn_exch{"H432 DMU"}="2H43 DMU";
$resn_exch{"H433 DMU"}="3H43 DMU"; $resn_exch{"H571 DMU"}="1H57 DMU"; $resn_exch{"H572 DMU"}="2H57 DMU";
$resn_exch{"H111 DMU"}="1H11 DMU"; $resn_exch{"H112 DMU"}="2H11 DMU"; $resn_exch{"H323 DMV"}="3H32 DMV";
$resn_exch{"H322 DMV"}="2H32 DMV"; $resn_exch{"H321 DMV"}="1H32 DMV"; $resn_exch{" H42 DMV"}="2H4  DMV";
$resn_exch{" H41 DMV"}="1H4  DMV"; $resn_exch{" H53 DMV"}="3H5  DMV"; $resn_exch{" H52 DMV"}="2H5  DMV";
$resn_exch{" H51 DMV"}="1H5  DMV"; $resn_exch{"H5M1 DMW"}="1H5M DMW"; $resn_exch{"H5M2 DMW"}="2H5M DMW";
$resn_exch{"H5M3 DMW"}="3H5M DMW"; $resn_exch{"H6M1 DMW"}="1H6M DMW"; $resn_exch{"H6M2 DMW"}="2H6M DMW";
$resn_exch{"H6M3 DMW"}="3H6M DMW"; $resn_exch{" H71 DMX"}="1H7  DMX"; $resn_exch{" H72 DMX"}="2H7  DMX";
$resn_exch{" H91 DMX"}="1H9  DMX"; $resn_exch{" H92 DMX"}="2H9  DMX"; $resn_exch{"H101 DMX"}="1H10 DMX";
$resn_exch{"H102 DMX"}="2H10 DMX"; $resn_exch{"H121 DMX"}="1H12 DMX"; $resn_exch{"H122 DMX"}="2H12 DMX";
$resn_exch{"H123 DMX"}="3H12 DMX"; $resn_exch{"H131 DMX"}="1H13 DMX"; $resn_exch{"H132 DMX"}="2H13 DMX";
$resn_exch{"H133 DMX"}="3H13 DMX"; $resn_exch{"H171 DMX"}="1H17 DMX"; $resn_exch{"H172 DMX"}="2H17 DMX";
$resn_exch{" H61 DMY"}="1H6  DMY"; $resn_exch{" H62 DMY"}="2H6  DMY"; $resn_exch{" H63 DMY"}="3H6  DMY";
$resn_exch{"H121 DMY"}="1H12 DMY"; $resn_exch{"H122 DMY"}="2H12 DMY"; $resn_exch{"H123 DMY"}="3H12 DMY";
$resn_exch{"H181 DMY"}="1H18 DMY"; $resn_exch{"H182 DMY"}="2H18 DMY"; $resn_exch{"H183 DMY"}="3H18 DMY";
$resn_exch{"H201 DMY"}="1H20 DMY"; $resn_exch{"H202 DMY"}="2H20 DMY"; $resn_exch{"H211 DMY"}="1H21 DMY";
$resn_exch{"H212 DMY"}="2H21 DMY"; $resn_exch{"HN91 DMY"}="1HN9 DMY"; $resn_exch{"HN92 DMY"}="2HN9 DMY";
$resn_exch{"H411 DMZ"}="1H41 DMZ"; $resn_exch{"H412 DMZ"}="2H41 DMZ"; $resn_exch{"H413 DMZ"}="3H41 DMZ";
$resn_exch{"H311 DMZ"}="1H31 DMZ"; $resn_exch{"H312 DMZ"}="2H31 DMZ"; $resn_exch{"H313 DMZ"}="3H31 DMZ";
$resn_exch{"H5C1 DN1"}="1H5C DN1"; $resn_exch{"H5C2 DN1"}="2H5C DN1"; $resn_exch{"H6C1 DN1"}="1H6C DN1";
$resn_exch{"H6C2 DN1"}="2H6C DN1"; $resn_exch{"H6C3 DN1"}="3H6C DN1"; $resn_exch{"H7C1 DN1"}="1H7C DN1";
$resn_exch{"H7C2 DN1"}="2H7C DN1"; $resn_exch{"H7C3 DN1"}="3H7C DN1"; $resn_exch{"H221 DN1"}="1H22 DN1";
$resn_exch{"H222 DN1"}="2H22 DN1"; $resn_exch{"H231 DN1"}="1H23 DN1"; $resn_exch{"H232 DN1"}="2H23 DN1";
$resn_exch{"H5C1 DN2"}="1H5C DN2"; $resn_exch{"H5C2 DN2"}="2H5C DN2"; $resn_exch{"H6C1 DN2"}="1H6C DN2";
$resn_exch{"H6C2 DN2"}="2H6C DN2"; $resn_exch{"H6C3 DN2"}="3H6C DN2"; $resn_exch{"H7C1 DN2"}="1H7C DN2";
$resn_exch{"H7C2 DN2"}="2H7C DN2"; $resn_exch{"H7C3 DN2"}="3H7C DN2"; $resn_exch{"H131 DN2"}="1H13 DN2";
$resn_exch{"H132 DN2"}="2H13 DN2"; $resn_exch{"H221 DN2"}="1H22 DN2"; $resn_exch{"H222 DN2"}="2H22 DN2";
$resn_exch{"H231 DN2"}="1H23 DN2"; $resn_exch{"H232 DN2"}="2H23 DN2"; $resn_exch{"HPNO DND"}="OHPN DND";
$resn_exch{"H5N1 DND"}="1H5N DND"; $resn_exch{"H5N2 DND"}="2H5N DND"; $resn_exch{"H4N1 DND"}="1H4N DND";
$resn_exch{"H3NC DND"}="CH3N DND"; $resn_exch{"H3NO DND"}="OH3N DND"; $resn_exch{" H2B DND"}="CH2' DND";
$resn_exch{"H2'O DND"}="OH2' DND"; $resn_exch{"H1NC DND"}="CH1N DND"; $resn_exch{"H6NC DND"}="CH6N DND";
$resn_exch{"H5NC DND"}="CH5N DND"; $resn_exch{"H4NC DND"}="CH4N DND"; $resn_exch{"H2NC DND"}="CH2N DND";
$resn_exch{"H7NO DND"}="OH7N DND"; $resn_exch{"HPAO DND"}="OHPA DND"; $resn_exch{"H5A1 DND"}="1H5A DND";
$resn_exch{"H5A2 DND"}="2H5A DND"; $resn_exch{" HB4 DND"}="4H'A DND"; $resn_exch{"H3AC DND"}="CH3A DND";
$resn_exch{" HB3 DND"}="3H'A DND"; $resn_exch{" HB2 DND"}="2H'A DND"; $resn_exch{" HBO DND"}="OH'A DND";
$resn_exch{" HB1 DND"}="1H'A DND"; $resn_exch{"H2AC DND"}="CH2A DND"; $resn_exch{"H8AC DND"}="CH8A DND";
$resn_exch{"HN61 DND"}="1HN6 DND"; $resn_exch{"HN62 DND"}="2HN6 DND"; $resn_exch{" H   DNE"}="1HN  DNE";
$resn_exch{" H2  DNE"}="2HN  DNE"; $resn_exch{" HB2 DNE"}="1HB  DNE"; $resn_exch{" HB3 DNE"}="2HB  DNE";
$resn_exch{" HG2 DNE"}="1HG  DNE"; $resn_exch{" HG3 DNE"}="2HG  DNE"; $resn_exch{" HD2 DNE"}="1HD  DNE";
$resn_exch{" HD3 DNE"}="2HD  DNE"; $resn_exch{" HE1 DNE"}="1HE  DNE"; $resn_exch{" HE2 DNE"}="2HE  DNE";
$resn_exch{" HE3 DNE"}="3HE  DNE"; $resn_exch{" H   DNG"}="1HN  DNG"; $resn_exch{" HB2 DNG"}="1HB  DNG";
$resn_exch{" HB3 DNG"}="2HB  DNG"; $resn_exch{" HG2 DNG"}="1HG  DNG"; $resn_exch{" HG3 DNG"}="2HG  DNG";
$resn_exch{" HD2 DNG"}="1HD  DNG"; $resn_exch{" HD3 DNG"}="2HD  DNG"; $resn_exch{" HE1 DNG"}="1HE  DNG";
$resn_exch{" HE2 DNG"}="2HE  DNG"; $resn_exch{" HE3 DNG"}="3HE  DNG"; $resn_exch{" H71 DNH"}="1H7  DNH";
$resn_exch{" H72 DNH"}="2H7  DNH"; $resn_exch{" H81 DNH"}="1H8  DNH"; $resn_exch{" H82 DNH"}="2H8  DNH";
$resn_exch{" H51 DNH"}="1H5  DNH"; $resn_exch{" H52 DNH"}="2H5  DNH"; $resn_exch{" H61 DNH"}="1H6  DNH";
$resn_exch{" H62 DNH"}="2H6  DNH"; $resn_exch{" H11 DNH"}="1H1  DNH"; $resn_exch{" H12 DNH"}="2H1  DNH";
$resn_exch{" H21 DNH"}="1H2  DNH"; $resn_exch{" H22 DNH"}="2H2  DNH"; $resn_exch{" H11 DNJ"}="1H1  DNJ";
$resn_exch{" H12 DNJ"}="2H1  DNJ"; $resn_exch{" H61 DNJ"}="1H6  DNJ"; $resn_exch{" H62 DNJ"}="2H6  DNJ";
$resn_exch{" HB1 DNL"}="1HB  DNL"; $resn_exch{" HB2 DNL"}="2HB  DNL"; $resn_exch{" HG1 DNL"}="1HG  DNL";
$resn_exch{" HG2 DNL"}="2HG  DNL"; $resn_exch{" HD1 DNL"}="1HD  DNL"; $resn_exch{" HD2 DNL"}="2HD  DNL";
$resn_exch{" HE1 DNL"}="1HE  DNL"; $resn_exch{" HE2 DNL"}="2HE  DNL"; $resn_exch{" HZ1 DNL"}="1HZ  DNL";
$resn_exch{" HZ2 DNL"}="2HZ  DNL"; $resn_exch{" H   DNM"}=" HN  DNM"; $resn_exch{" HB2 DNM"}="1HB  DNM";
$resn_exch{" HB3 DNM"}="2HB  DNM"; $resn_exch{" HG2 DNM"}="1HG  DNM"; $resn_exch{" HG3 DNM"}="2HG  DNM";
$resn_exch{" HD2 DNM"}="1HD  DNM"; $resn_exch{" HD3 DNM"}="2HD  DNM"; $resn_exch{" HE1 DNM"}="1HE  DNM";
$resn_exch{" HE2 DNM"}="2HE  DNM"; $resn_exch{" HE3 DNM"}="3HE  DNM"; $resn_exch{" H21 DNN"}="1H2  DNN";
$resn_exch{" H22 DNN"}="2H2  DNN"; $resn_exch{" H31 DNN"}="1H3  DNN"; $resn_exch{" H32 DNN"}="2H3  DNN";
$resn_exch{" H41 DNN"}="1H4  DNN"; $resn_exch{" H42 DNN"}="2H4  DNN"; $resn_exch{" H51 DNN"}="1H5  DNN";
$resn_exch{" H52 DNN"}="2H5  DNN"; $resn_exch{" H61 DNN"}="1H6  DNN"; $resn_exch{" H62 DNN"}="2H6  DNN";
$resn_exch{" H91 DNN"}="1H9  DNN"; $resn_exch{" H92 DNN"}="2H9  DNN"; $resn_exch{" H93 DNN"}="3H9  DNN";
$resn_exch{"HN71 DNN"}="1HN7 DNN"; $resn_exch{"HN72 DNN"}="2HN7 DNN"; $resn_exch{"HN81 DNN"}="1HN8 DNN";
$resn_exch{"HN82 DNN"}="2HN8 DNN"; $resn_exch{" H   DNP"}="1HN  DNP"; $resn_exch{" H2  DNP"}="2HN  DNP";
$resn_exch{" HB2 DNP"}="1HB  DNP"; $resn_exch{" HB3 DNP"}="2HB  DNP"; $resn_exch{" HG1 DNP"}="1HG  DNP";
$resn_exch{" HG2 DNP"}="2HG  DNP"; $resn_exch{" HG3 DNP"}="3HG  DNP"; $resn_exch{" H51 DNQ"}="1H5  DNQ";
$resn_exch{" H61 DNQ"}="1H6  DNQ"; $resn_exch{"HOP2 DNR"}="2HOP DNR"; $resn_exch{" H5' DNR"}="1H5* DNR";
$resn_exch{"H5'' DNR"}="2H5* DNR"; $resn_exch{" H4' DNR"}=" H4* DNR"; $resn_exch{" H1' DNR"}=" H1* DNR";
$resn_exch{" H2' DNR"}="1H2* DNR"; $resn_exch{"H2'' DNR"}="2H2* DNR"; $resn_exch{" H3' DNR"}=" H3* DNR";
$resn_exch{"HO3' DNR"}="H3T  DNR"; $resn_exch{"HOP3 DNR"}="3HOP DNR"; $resn_exch{"H11A DNS"}="1H1  DNS";
$resn_exch{" H12 DNS"}="2H1  DNS"; $resn_exch{" H13 DNS"}="3H1  DNS"; $resn_exch{" H21 DNS"}="1H2  DNS";
$resn_exch{" H22 DNS"}="2H2  DNS"; $resn_exch{" H23 DNS"}="3H2  DNS"; $resn_exch{" HZ  DNS"}=" HNZ DNS";
$resn_exch{" HE3 DNS"}="1HE  DNS"; $resn_exch{" HE2 DNS"}="2HE  DNS"; $resn_exch{" HD3 DNS"}="1HD  DNS";
$resn_exch{" HD2 DNS"}="2HD  DNS"; $resn_exch{" HG3 DNS"}="1HG  DNS"; $resn_exch{" HG2 DNS"}="2HG  DNS";
$resn_exch{" HB3 DNS"}="1HB  DNS"; $resn_exch{" HB2 DNS"}="2HB  DNS"; $resn_exch{" H   DNS"}="1HN  DNS";
$resn_exch{" H2  DNS"}="2HN  DNS"; $resn_exch{"H131 DNT"}="1H13 DNT"; $resn_exch{"H132 DNT"}="2H13 DNT";
$resn_exch{"H133 DNT"}="3H13 DNT"; $resn_exch{"H121 DNT"}="1H12 DNT"; $resn_exch{"H122 DNT"}="2H12 DNT";
$resn_exch{"H111 DNT"}="1H11 DNT"; $resn_exch{"H112 DNT"}="2H11 DNT"; $resn_exch{" H91 DNT"}="1H9  DNT";
$resn_exch{" H92 DNT"}="2H9  DNT"; $resn_exch{" H81 DNT"}="1H8  DNT"; $resn_exch{" H82 DNT"}="2H8  DNT";
$resn_exch{"H101 DNT"}="1H10 DNT"; $resn_exch{"H102 DNT"}="2H10 DNT"; $resn_exch{"H103 DNT"}="3H10 DNT";
$resn_exch{" HB1 DO2"}="1HB  DO2"; $resn_exch{" HB2 DO2"}="2HB  DO2"; $resn_exch{" HG1 DO2"}="1HG  DO2";
$resn_exch{" HG2 DO2"}="2HG  DO2"; $resn_exch{" HD1 DO2"}="1HD  DO2"; $resn_exch{" HD2 DO2"}="2HD  DO2";
$resn_exch{" HE1 DO2"}="1HE  DO2"; $resn_exch{" HO1 DO3"}="1HO  DO3"; $resn_exch{" HO3 DO3"}="3HO  DO3";
$resn_exch{" HO5 DO3"}="5HO  DO3"; $resn_exch{" HO7 DO3"}="7HO  DO3"; $resn_exch{"HC11 DO3"}="1HC1 DO3";
$resn_exch{"HC12 DO3"}="2HC1 DO3"; $resn_exch{"HC21 DO3"}="1HC2 DO3"; $resn_exch{"HC22 DO3"}="2HC2 DO3";
$resn_exch{"HC31 DO3"}="1HC3 DO3"; $resn_exch{"HC32 DO3"}="2HC3 DO3"; $resn_exch{"HC41 DO3"}="1HC4 DO3";
$resn_exch{"HC42 DO3"}="2HC4 DO3"; $resn_exch{"HC51 DO3"}="1HC5 DO3"; $resn_exch{"HC52 DO3"}="2HC5 DO3";
$resn_exch{"HC61 DO3"}="1HC6 DO3"; $resn_exch{"HC62 DO3"}="2HC6 DO3"; $resn_exch{"HC71 DO3"}="1HC7 DO3";
$resn_exch{"HC72 DO3"}="2HC7 DO3"; $resn_exch{"HC81 DO3"}="1HC8 DO3"; $resn_exch{"HC82 DO3"}="2HC8 DO3";
$resn_exch{"H101 DO3"}="1H10 DO3"; $resn_exch{"H102 DO3"}="2H10 DO3"; $resn_exch{"H121 DO3"}="1H12 DO3";
$resn_exch{"H122 DO3"}="2H12 DO3"; $resn_exch{"H141 DO3"}="1H14 DO3"; $resn_exch{"H142 DO3"}="2H14 DO3";
$resn_exch{"H161 DO3"}="1H16 DO3"; $resn_exch{"H162 DO3"}="2H16 DO3"; $resn_exch{"H171 DO3"}="1H17 DO3";
$resn_exch{"H172 DO3"}="2H17 DO3"; $resn_exch{"H173 DO3"}="3H17 DO3"; $resn_exch{" H41 DO4"}="1H4  DO4";
$resn_exch{" H42 DO4"}="2H4  DO4"; $resn_exch{" H31 DO4"}="1H3  DO4"; $resn_exch{" H32 DO4"}="2H3  DO4";
$resn_exch{" H31 DO8"}="1H3  DO8"; $resn_exch{" H32 DO8"}="2H3  DO8"; $resn_exch{" H81 DO8"}="1H8  DO8";
$resn_exch{" H82 DO8"}="2H8  DO8"; $resn_exch{"HOP2 DO8"}="2HOP DO8"; $resn_exch{"HOP3 DO8"}="3HOP DO8";
$resn_exch{" H2  DOA"}=" HN2 DOA"; $resn_exch{" HA1 DOA"}="1HA  DOA"; $resn_exch{" HA2 DOA"}="2HA  DOA";
$resn_exch{" H21 DOA"}="1H2  DOA"; $resn_exch{" H22 DOA"}="2H2  DOA"; $resn_exch{" H31 DOA"}="1H3  DOA";
$resn_exch{" H32 DOA"}="2H3  DOA"; $resn_exch{" H41 DOA"}="1H4  DOA"; $resn_exch{" H42 DOA"}="2H4  DOA";
$resn_exch{" H51 DOA"}="1H5  DOA"; $resn_exch{" H52 DOA"}="2H5  DOA"; $resn_exch{" H61 DOA"}="1H6  DOA";
$resn_exch{" H62 DOA"}="2H6  DOA"; $resn_exch{" H71 DOA"}="1H7  DOA"; $resn_exch{" H72 DOA"}="2H7  DOA";
$resn_exch{" H81 DOA"}="1H8  DOA"; $resn_exch{" H82 DOA"}="2H8  DOA"; $resn_exch{" H91 DOA"}="1H9  DOA";
$resn_exch{" H92 DOA"}="2H9  DOA"; $resn_exch{"H101 DOA"}="1H10 DOA"; $resn_exch{"H102 DOA"}="2H10 DOA";
$resn_exch{"HA21 DOA"}="1HA2 DOA"; $resn_exch{"HA22 DOA"}="2HA2 DOA"; $resn_exch{"HO2' DOB"}="'HO2 DOB";
$resn_exch{"HN41 DOC"}="1HN4 DOC"; $resn_exch{"HN42 DOC"}="2HN4 DOC"; $resn_exch{" H1' DOC"}=" H1* DOC";
$resn_exch{" H2' DOC"}="1H2* DOC"; $resn_exch{"H2'' DOC"}="2H2* DOC"; $resn_exch{"H3'1 DOC"}="1H3* DOC";
$resn_exch{"H3'2 DOC"}="2H3* DOC"; $resn_exch{" H4' DOC"}=" H4* DOC"; $resn_exch{" H5' DOC"}="1H5* DOC";
$resn_exch{"H5'' DOC"}="2H5* DOC"; $resn_exch{"HOP2 DOC"}="2HOP DOC"; $resn_exch{"HOP3 DOC"}="3HOP DOC";
$resn_exch{"HC21 DOE"}="1HC2 DOE"; $resn_exch{"HC22 DOE"}="2HC2 DOE"; $resn_exch{"HC31 DOE"}="1HC3 DOE";
$resn_exch{"HC32 DOE"}="2HC3 DOE"; $resn_exch{"HC41 DOE"}="1HC4 DOE"; $resn_exch{"HC42 DOE"}="2HC4 DOE";
$resn_exch{"HC51 DOE"}="1HC5 DOE"; $resn_exch{"HC52 DOE"}="2HC5 DOE"; $resn_exch{"HC61 DOE"}="1HC6 DOE";
$resn_exch{"HC62 DOE"}="2HC6 DOE"; $resn_exch{"HC71 DOE"}="1HC7 DOE"; $resn_exch{"HC72 DOE"}="2HC7 DOE";
$resn_exch{"HC81 DOE"}="1HC8 DOE"; $resn_exch{"HC82 DOE"}="2HC8 DOE"; $resn_exch{"HC91 DOE"}="1HC9 DOE";
$resn_exch{"HC92 DOE"}="2HC9 DOE"; $resn_exch{"H111 DOE"}="1H11 DOE"; $resn_exch{"H112 DOE"}="2H11 DOE";
$resn_exch{"H131 DOE"}="1H13 DOE"; $resn_exch{"H132 DOE"}="2H13 DOE"; $resn_exch{"H151 DOE"}="1H15 DOE";
$resn_exch{"H152 DOE"}="2H15 DOE"; $resn_exch{"H171 DOE"}="1H17 DOE"; $resn_exch{"H172 DOE"}="2H17 DOE";
$resn_exch{"H251 DOE"}="1H25 DOE"; $resn_exch{"H252 DOE"}="2H25 DOE"; $resn_exch{"H261 DOE"}="1H26 DOE";
$resn_exch{"H262 DOE"}="2H26 DOE"; $resn_exch{"H271 DOE"}="1H27 DOE"; $resn_exch{"H272 DOE"}="2H27 DOE";
$resn_exch{"HC21 DOF"}="1HC2 DOF"; $resn_exch{"HC22 DOF"}="2HC2 DOF"; $resn_exch{"HC31 DOF"}="1HC3 DOF";
$resn_exch{"HC32 DOF"}="2HC3 DOF"; $resn_exch{"HC41 DOF"}="1HC4 DOF"; $resn_exch{"HC42 DOF"}="2HC4 DOF";
$resn_exch{"HC51 DOF"}="1HC5 DOF"; $resn_exch{"HC52 DOF"}="2HC5 DOF"; $resn_exch{"HC61 DOF"}="1HC6 DOF";
$resn_exch{"HC62 DOF"}="2HC6 DOF"; $resn_exch{"HC71 DOF"}="1HC7 DOF"; $resn_exch{"HC72 DOF"}="2HC7 DOF";
$resn_exch{"HC81 DOF"}="1HC8 DOF"; $resn_exch{"HC82 DOF"}="2HC8 DOF"; $resn_exch{"HC91 DOF"}="1HC9 DOF";
$resn_exch{"HC92 DOF"}="2HC9 DOF"; $resn_exch{"H111 DOF"}="1H11 DOF"; $resn_exch{"H112 DOF"}="2H11 DOF";
$resn_exch{"H131 DOF"}="1H13 DOF"; $resn_exch{"H132 DOF"}="2H13 DOF"; $resn_exch{"H151 DOF"}="1H15 DOF";
$resn_exch{"H152 DOF"}="2H15 DOF"; $resn_exch{"H171 DOF"}="1H17 DOF"; $resn_exch{"H172 DOF"}="2H17 DOF";
$resn_exch{"H211 DOG"}="1H21 DOG"; $resn_exch{"H212 DOG"}="2H21 DOG"; $resn_exch{"H161 DOG"}="1H16 DOG";
$resn_exch{"H162 DOG"}="2H16 DOG"; $resn_exch{"H181 DOG"}="1H18 DOG"; $resn_exch{"H182 DOG"}="2H18 DOG";
$resn_exch{"H183 DOG"}="3H18 DOG"; $resn_exch{"H12O DOG"}="OH12 DOG"; $resn_exch{"H151 DOG"}="1H15 DOG";
$resn_exch{"H152 DOG"}="2H15 DOG"; $resn_exch{"H14O DOG"}="OH14 DOG"; $resn_exch{"HC71 DOG"}="1HC7 DOG";
$resn_exch{"HC72 DOG"}="2HC7 DOG"; $resn_exch{"H111 DOG"}="1H11 DOG"; $resn_exch{"H112 DOG"}="2H11 DOG";
$resn_exch{"H191 DOG"}="1H19 DOG"; $resn_exch{"H192 DOG"}="2H19 DOG"; $resn_exch{"H193 DOG"}="3H19 DOG";
$resn_exch{"HC61 DOG"}="1HC6 DOG"; $resn_exch{"HC62 DOG"}="2HC6 DOG"; $resn_exch{"HC41 DOG"}="1HC4 DOG";
$resn_exch{"HC42 DOG"}="2HC4 DOG"; $resn_exch{"H32O DOG"}="OH32 DOG"; $resn_exch{"HC21 DOG"}="1HC2 DOG";
$resn_exch{"HC22 DOG"}="2HC2 DOG"; $resn_exch{"HC11 DOG"}="1HC1 DOG"; $resn_exch{"HC12 DOG"}="2HC1 DOG";
$resn_exch{" HN1 DOH"}="1HN  DOH"; $resn_exch{" HN2 DOH"}="2HN  DOH"; $resn_exch{"H5'1 DOI"}="1H5* DOI";
$resn_exch{"H5'2 DOI"}="2H5* DOI"; $resn_exch{" H4' DOI"}=" H4* DOI"; $resn_exch{" H1' DOI"}=" H1* DOI";
$resn_exch{"H2'1 DOI"}="1H2* DOI"; $resn_exch{"H2'2 DOI"}="2H2* DOI"; $resn_exch{" H3' DOI"}=" H3* DOI";
$resn_exch{"HO3' DOI"}="*HO3 DOI"; $resn_exch{"HC41 DOL"}="1HC4 DOL"; $resn_exch{"HC42 DOL"}="2HC4 DOL";
$resn_exch{"HC31 DOL"}="1HC3 DOL"; $resn_exch{"HC3A DOL"}="2HC3 DOL"; $resn_exch{"H421 DOL"}="1H42 DOL";
$resn_exch{"H422 DOL"}="2H42 DOL"; $resn_exch{"H431 DOL"}="1H43 DOL"; $resn_exch{"H432 DOL"}="2H43 DOL";
$resn_exch{"H471 DOL"}="1H47 DOL"; $resn_exch{"H472 DOL"}="2H47 DOL"; $resn_exch{"H481 DOL"}="1H48 DOL";
$resn_exch{"H482 DOL"}="2H48 DOL"; $resn_exch{"H483 DOL"}="3H48 DOL"; $resn_exch{"H451 DOL"}="1H45 DOL";
$resn_exch{"H452 DOL"}="2H45 DOL"; $resn_exch{"H461 DOL"}="1H46 DOL"; $resn_exch{"H462 DOL"}="2H46 DOL";
$resn_exch{"H463 DOL"}="3H46 DOL"; $resn_exch{"H131 DOL"}="1H13 DOL"; $resn_exch{"H132 DOL"}="2H13 DOL";
$resn_exch{"H161 DOL"}="1H16 DOL"; $resn_exch{"H162 DOL"}="2H16 DOL"; $resn_exch{"H211 DOL"}="1H21 DOL";
$resn_exch{"H212 DOL"}="2H21 DOL"; $resn_exch{"H213 DOL"}="3H21 DOL"; $resn_exch{"H241 DOL"}="1H24 DOL";
$resn_exch{"H242 DOL"}="2H24 DOL"; $resn_exch{"H311 DOL"}="1H31 DOL"; $resn_exch{"H312 DOL"}="2H31 DOL";
$resn_exch{"H313 DOL"}="3H31 DOL"; $resn_exch{"H351 DOL"}="1H35 DOL"; $resn_exch{"H352 DOL"}="2H35 DOL";
$resn_exch{"H353 DOL"}="3H35 DOL"; $resn_exch{"H341 DOL"}="1H34 DOL"; $resn_exch{"H342 DOL"}="2H34 DOL";
$resn_exch{"H343 DOL"}="3H34 DOL"; $resn_exch{" H61 DOM"}="1H6  DOM"; $resn_exch{" H62 DOM"}="2H6  DOM";
$resn_exch{"H2'1 DOM"}="1H2' DOM"; $resn_exch{"H2'2 DOM"}="2H2' DOM"; $resn_exch{"H6'1 DOM"}="1H6' DOM";
$resn_exch{"H6'2 DOM"}="2H6' DOM"; $resn_exch{"HO1' DOM"}="'HO1 DOM"; $resn_exch{"HO3' DOM"}="'HO3 DOM";
$resn_exch{"HO6' DOM"}="'HO6 DOM"; $resn_exch{" HXT DON"}="HOXT DON"; $resn_exch{" H   DON"}="1HN  DON";
$resn_exch{" H2  DON"}="2HN  DON"; $resn_exch{" HB2 DON"}="1HB  DON"; $resn_exch{" HB3 DON"}="2HB  DON";
$resn_exch{"H1E1 DON"}="1H1E DON"; $resn_exch{"H1E2 DON"}="2H1E DON"; $resn_exch{"H1C1 DON"}="1H1C DON";
$resn_exch{"H1C2 DON"}="2H1C DON"; $resn_exch{"H1A1 DON"}="1H1A DON"; $resn_exch{" H11 DOP"}="1H1  DOP";
$resn_exch{" H12 DOP"}="2H1  DOP"; $resn_exch{" H21 DOP"}="1H2  DOP"; $resn_exch{" H22 DOP"}="2H2  DOP";
$resn_exch{" H31 DOP"}="1H3  DOP"; $resn_exch{" H32 DOP"}="2H3  DOP"; $resn_exch{" H41 DOP"}="1H4  DOP";
$resn_exch{" H42 DOP"}="2H4  DOP"; $resn_exch{" H51 DOP"}="1H5  DOP"; $resn_exch{" H52 DOP"}="2H5  DOP";
$resn_exch{" H61 DOP"}="1H6  DOP"; $resn_exch{" H62 DOP"}="2H6  DOP"; $resn_exch{" H71 DOP"}="1H7  DOP";
$resn_exch{" H72 DOP"}="2H7  DOP"; $resn_exch{" H81 DOP"}="1H8  DOP"; $resn_exch{" H82 DOP"}="2H8  DOP";
$resn_exch{" H83 DOP"}="3H8  DOP"; $resn_exch{"H1'1 DOP"}="1H1' DOP"; $resn_exch{"H1'2 DOP"}="2H1' DOP";
$resn_exch{"H2'1 DOP"}="1H2' DOP"; $resn_exch{"H2'2 DOP"}="2H2' DOP"; $resn_exch{"H3'1 DOP"}="1H3' DOP";
$resn_exch{"H3'2 DOP"}="2H3' DOP"; $resn_exch{"H4'1 DOP"}="1H4' DOP"; $resn_exch{"H4'2 DOP"}="2H4' DOP";
$resn_exch{"H5'1 DOP"}="1H5' DOP"; $resn_exch{"H5'2 DOP"}="2H5' DOP"; $resn_exch{"H6'1 DOP"}="1H6' DOP";
$resn_exch{"H6'2 DOP"}="2H6' DOP"; $resn_exch{"H7'1 DOP"}="1H7' DOP"; $resn_exch{"H7'2 DOP"}="2H7' DOP";
$resn_exch{"H8'1 DOP"}="1H8' DOP"; $resn_exch{"H8'2 DOP"}="2H8' DOP"; $resn_exch{"H8'3 DOP"}="3H8' DOP";
$resn_exch{"HAB1 DOQ"}="1HAB DOQ"; $resn_exch{"HAB2 DOQ"}="2HAB DOQ"; $resn_exch{"HAB3 DOQ"}="3HAB DOQ";
$resn_exch{"HAA1 DOQ"}="1HAA DOQ"; $resn_exch{"HAA2 DOQ"}="2HAA DOQ"; $resn_exch{"HAA3 DOQ"}="3HAA DOQ";
$resn_exch{"HAL1 DOQ"}="1HAL DOQ"; $resn_exch{"HAL2 DOQ"}="2HAL DOQ"; $resn_exch{" HG1 DOQ"}="1HG  DOQ";
$resn_exch{" HG2 DOQ"}="2HG  DOQ"; $resn_exch{" H51 DOR"}="1H5  DOR"; $resn_exch{" H52 DOR"}="2H5  DOR";
$resn_exch{" H31 DOS"}="1H3  DOS"; $resn_exch{" H32 DOS"}="2H3  DOS"; $resn_exch{" H41 DOS"}="1H4  DOS";
$resn_exch{" H42 DOS"}="2H4  DOS"; $resn_exch{" H51 DOS"}="1H5  DOS"; $resn_exch{" H61 DOS"}="1H6  DOS";
$resn_exch{" H77 DOS"}="7H7  DOS"; $resn_exch{" H88 DOS"}="8H8  DOS"; $resn_exch{" H91 DOS"}="1H9  DOS";
$resn_exch{" H92 DOS"}="2H9  DOS"; $resn_exch{"H101 DOS"}="1H10 DOS"; $resn_exch{"H102 DOS"}="2H10 DOS";
$resn_exch{"H111 DOS"}="1H11 DOS"; $resn_exch{"H112 DOS"}="2H11 DOS"; $resn_exch{"H121 DOS"}="1H12 DOS";
$resn_exch{"H122 DOS"}="2H12 DOS"; $resn_exch{"H271 DOS"}="1H27 DOS"; $resn_exch{"H272 DOS"}="2H27 DOS";
$resn_exch{"H281 DOS"}="1H28 DOS"; $resn_exch{"H282 DOS"}="2H28 DOS"; $resn_exch{"H291 DOS"}="1H29 DOS";
$resn_exch{"H292 DOS"}="2H29 DOS"; $resn_exch{"H301 DOS"}="1H30 DOS"; $resn_exch{"H302 DOS"}="2H30 DOS";
$resn_exch{"H331 DOS"}="31H3 DOS"; $resn_exch{"H332 DOS"}="32H3 DOS"; $resn_exch{"H33A DOS"}="1H33 DOS";
$resn_exch{"H33B DOS"}="2H33 DOS"; $resn_exch{"H341 DOS"}="1H34 DOS"; $resn_exch{"H342 DOS"}="2H34 DOS";
$resn_exch{"H351 DOS"}="1H35 DOS"; $resn_exch{"H352 DOS"}="2H35 DOS"; $resn_exch{"H361 DOS"}="1H36 DOS";
$resn_exch{"H362 DOS"}="2H36 DOS"; $resn_exch{" HG1 DOS"}="1HG  DOS"; $resn_exch{" HG2 DOS"}="2HG  DOS";
$resn_exch{"HD21 DOS"}="1HD2 DOS"; $resn_exch{"HD22 DOS"}="2HD2 DOS"; $resn_exch{"HE11 DOS"}="1HE1 DOS";
$resn_exch{"HE12 DOS"}="2HE1 DOS"; $resn_exch{"HE21 DOS"}="1HE2 DOS"; $resn_exch{"HOG1 DOT"}="1HOG DOT";
$resn_exch{"HOG3 DOT"}="3HOG DOT"; $resn_exch{"HOB2 DOT"}="2HOB DOT"; $resn_exch{"HOA2 DOT"}="2HOA DOT";
$resn_exch{"H5'1 DOT"}="1H5* DOT"; $resn_exch{"H5'2 DOT"}="2H5* DOT"; $resn_exch{" H4B DOT"}=" H4* DOT";
$resn_exch{" H3B DOT"}=" H3* DOT"; $resn_exch{"H2'1 DOT"}="1H2* DOT"; $resn_exch{"H2'2 DOT"}="2H2* DOT";
$resn_exch{" H1' DOT"}=" H1* DOT"; $resn_exch{"HN61 DOT"}="1HN6 DOT"; $resn_exch{"HN62 DOT"}="2HN6 DOT";
$resn_exch{" H2B DOT"}="1H2' DOT"; $resn_exch{" H2D DOT"}="2H2' DOT"; $resn_exch{" H11 DOX"}="1H1  DOX";
$resn_exch{" H12 DOX"}="2H1  DOX"; $resn_exch{" H21 DOX"}="1H2  DOX"; $resn_exch{" H22 DOX"}="2H2  DOX";
$resn_exch{" H31 DOX"}="1H3  DOX"; $resn_exch{" H32 DOX"}="2H3  DOX"; $resn_exch{" H41 DOX"}="1H4  DOX";
$resn_exch{" H42 DOX"}="2H4  DOX"; $resn_exch{" HD1 DP1"}="1HD  DP1"; $resn_exch{" HD2 DP1"}="2HD  DP1";
$resn_exch{" HG1 DP1"}="1HG  DP1"; $resn_exch{" HG2 DP1"}="2HG  DP1"; $resn_exch{" HB1 DP1"}="1HB  DP1";
$resn_exch{" HB2 DP1"}="2HB  DP1"; $resn_exch{" HN1 DP1"}="1HN  DP1"; $resn_exch{" HN2 DP1"}="2HN  DP1";
$resn_exch{"H1'1 DP1"}="1H1' DP1"; $resn_exch{"H1'2 DP1"}="2H1' DP1"; $resn_exch{"HB'1 DP1"}="1HB' DP1";
$resn_exch{"HB'2 DP1"}="2HB' DP1"; $resn_exch{" H11 DP1"}="1H1  DP1"; $resn_exch{" H12 DP1"}="2H1  DP1";
$resn_exch{"HN11 DP1"}="1HN1 DP1"; $resn_exch{"HN12 DP1"}="2HN1 DP1"; $resn_exch{"HN21 DP2"}="1HN2 DP2";
$resn_exch{"HN22 DP2"}="2HN2 DP2"; $resn_exch{" HB1 DP2"}="1HB  DP2"; $resn_exch{" HB2 DP2"}="2HB  DP2";
$resn_exch{" HG1 DP2"}="1HG  DP2"; $resn_exch{" HG2 DP2"}="2HG  DP2"; $resn_exch{" HD1 DP2"}="1HD  DP2";
$resn_exch{" HD2 DP2"}="2HD  DP2"; $resn_exch{"HN'1 DP2"}="1HN' DP2"; $resn_exch{"HN'2 DP2"}="2HN' DP2";
$resn_exch{"HB'1 DP2"}="1HB' DP2"; $resn_exch{"HB'2 DP2"}="2HB' DP2"; $resn_exch{"HG'1 DP2"}="1HG' DP2";
$resn_exch{"HG'2 DP2"}="2HG' DP2"; $resn_exch{"HD'1 DP2"}="1HD' DP2"; $resn_exch{"HD'2 DP2"}="2HD' DP2";
$resn_exch{"HE'1 DP2"}="1HE' DP2"; $resn_exch{"HE'2 DP2"}="2HE' DP2"; $resn_exch{"HZ'1 DP2"}="1HZ' DP2";
$resn_exch{"HZ'2 DP2"}="2HZ' DP2"; $resn_exch{"H2'1 DP3"}="1H2' DP3"; $resn_exch{"H2'2 DP3"}="2H2' DP3";
$resn_exch{" H11 DP3"}="1H1  DP3"; $resn_exch{" H12 DP3"}="2H1  DP3"; $resn_exch{"H1'1 DP3"}="1H1' DP3";
$resn_exch{"H1'2 DP3"}="2H1' DP3"; $resn_exch{" H1  DP3"}="1H   DP3"; $resn_exch{" H2  DP3"}="2H   DP3";
$resn_exch{" HN1 DP3"}="1HN  DP3"; $resn_exch{" HN2 DP3"}="2HN  DP3"; $resn_exch{" HB1 DP3"}="1HB  DP3";
$resn_exch{" HB2 DP3"}="2HB  DP3"; $resn_exch{" HG1 DP3"}="1HG  DP3"; $resn_exch{" HG2 DP3"}="2HG  DP3";
$resn_exch{" HD1 DP3"}="1HD  DP3"; $resn_exch{" HD2 DP3"}="2HD  DP3"; $resn_exch{"HC71 DP4"}="1HC7 DP4";
$resn_exch{"HC72 DP4"}="2HC7 DP4"; $resn_exch{"HC73 DP4"}="3HC7 DP4"; $resn_exch{"HC81 DP4"}="1HC8 DP4";
$resn_exch{"HC82 DP4"}="2HC8 DP4"; $resn_exch{"HC83 DP4"}="3HC8 DP4"; $resn_exch{"H101 DP4"}="1H10 DP4";
$resn_exch{"H102 DP4"}="2H10 DP4"; $resn_exch{"H111 DP4"}="1H11 DP4"; $resn_exch{"H112 DP4"}="2H11 DP4";
$resn_exch{"H121 DP4"}="1H12 DP4"; $resn_exch{"H122 DP4"}="2H12 DP4"; $resn_exch{"H131 DP4"}="1H13 DP4";
$resn_exch{"H132 DP4"}="2H13 DP4"; $resn_exch{"H141 DP4"}="1H14 DP4"; $resn_exch{"H142 DP4"}="2H14 DP4";
$resn_exch{"H143 DP4"}="3H14 DP4"; $resn_exch{"H6A1 DP5"}="1H6A DP5"; $resn_exch{"H6A2 DP5"}="2H6A DP5";
$resn_exch{"HO2A DP5"}="AHO2 DP5"; $resn_exch{"HO3A DP5"}="AHO3 DP5"; $resn_exch{"HS4A DP5"}="AHS4 DP5";
$resn_exch{"HO6A DP5"}="AHO6 DP5"; $resn_exch{"H6B1 DP5"}="1H6B DP5"; $resn_exch{"H6B2 DP5"}="2H6B DP5";
$resn_exch{"HO2B DP5"}="BHO2 DP5"; $resn_exch{"HO3B DP5"}="BHO3 DP5"; $resn_exch{"HO6B DP5"}="BHO6 DP5";
$resn_exch{"H6C1 DP5"}="1H6C DP5"; $resn_exch{"H6C2 DP5"}="2H6C DP5"; $resn_exch{"HS1C DP5"}="CHS1 DP5";
$resn_exch{"HO2C DP5"}="CHO2 DP5"; $resn_exch{"HO3C DP5"}="CHO3 DP5"; $resn_exch{"HO6C DP5"}="CHO6 DP5";
$resn_exch{"H3A1 DP6"}="1H3A DP6"; $resn_exch{"H3A2 DP6"}="2H3A DP6"; $resn_exch{"H3A3 DP6"}="3H3A DP6";
$resn_exch{" H51 DP6"}="1H5  DP6"; $resn_exch{" H52 DP6"}="2H5  DP6"; $resn_exch{" H41 DP6"}="1H4  DP6";
$resn_exch{" H42 DP6"}="2H4  DP6"; $resn_exch{" H21 DP6"}="1H2  DP6"; $resn_exch{" H22 DP6"}="2H2  DP6";
$resn_exch{" H31 DP7"}="1H3  DP7"; $resn_exch{" H32 DP7"}="2H3  DP7"; $resn_exch{" H41 DP7"}="1H4  DP7";
$resn_exch{" H42 DP7"}="2H4  DP7"; $resn_exch{" H51 DP7"}="1H5  DP7"; $resn_exch{" H52 DP7"}="2H5  DP7";
$resn_exch{" H91 DP7"}="1H9  DP7"; $resn_exch{" H92 DP7"}="2H9  DP7"; $resn_exch{"H101 DP7"}="1H10 DP7";
$resn_exch{"H102 DP7"}="2H10 DP7"; $resn_exch{"H111 DP7"}="1H11 DP7"; $resn_exch{"H112 DP7"}="2H11 DP7";
$resn_exch{"H131 DP7"}="1H13 DP7"; $resn_exch{"H132 DP7"}="2H13 DP7"; $resn_exch{"H161 DP7"}="1H16 DP7";
$resn_exch{"H162 DP7"}="2H16 DP7"; $resn_exch{"H163 DP7"}="3H16 DP7"; $resn_exch{"HN31 DP7"}="1HN3 DP7";
$resn_exch{"HN32 DP7"}="2HN3 DP7"; $resn_exch{" HD1 DP9"}="1HD  DP9"; $resn_exch{" HD2 DP9"}="2HD  DP9";
$resn_exch{" HG1 DP9"}="1HG  DP9"; $resn_exch{" HG2 DP9"}="2HG  DP9"; $resn_exch{" HB1 DP9"}="1HB  DP9";
$resn_exch{" HB2 DP9"}="2HB  DP9"; $resn_exch{" HN1 DP9"}="1HN  DP9"; $resn_exch{" HN2 DP9"}="2HN  DP9";
$resn_exch{"HD'1 DP9"}="1HD' DP9"; $resn_exch{"HD'2 DP9"}="2HD' DP9"; $resn_exch{"H1'1 DP9"}="1H1' DP9";
$resn_exch{"H1'2 DP9"}="2H1' DP9"; $resn_exch{"HB'1 DP9"}="1HB' DP9"; $resn_exch{"HB'2 DP9"}="2HB' DP9";
$resn_exch{"H101 DPA"}="1H10 DPA"; $resn_exch{"H102 DPA"}="2H10 DPA"; $resn_exch{"H111 DPA"}="1H11 DPA";
$resn_exch{"H112 DPA"}="2H11 DPA"; $resn_exch{"H113 DPA"}="3H11 DPA"; $resn_exch{"H121 DPA"}="1H12 DPA";
$resn_exch{"H122 DPA"}="2H12 DPA"; $resn_exch{"H123 DPA"}="3H12 DPA"; $resn_exch{"H131 DPA"}="1H13 DPA";
$resn_exch{"H132 DPA"}="2H13 DPA"; $resn_exch{"H133 DPA"}="3H13 DPA"; $resn_exch{" H71 DPB"}="1H5M DPB";
$resn_exch{" H72 DPB"}="2H5M DPB"; $resn_exch{" H73 DPB"}="3H5M DPB"; $resn_exch{" H1' DPB"}=" H1* DPB";
$resn_exch{" H4' DPB"}=" H4* DPB"; $resn_exch{" H3' DPB"}=" H3* DPB"; $resn_exch{" H2' DPB"}="1H2* DPB";
$resn_exch{"H2'' DPB"}="2H2* DPB"; $resn_exch{" H5' DPB"}="1H5* DPB"; $resn_exch{"H5'' DPB"}="2H5* DPB";
$resn_exch{"HOP2 DPB"}="2HOP DPB"; $resn_exch{"HOP3 DPB"}="3HOP DPB"; $resn_exch{"H111 DPC"}="1H11 DPC";
$resn_exch{"H112 DPC"}="2H11 DPC"; $resn_exch{"H113 DPC"}="3H11 DPC"; $resn_exch{"H811 DPC"}="1H81 DPC";
$resn_exch{"H812 DPC"}="2H81 DPC"; $resn_exch{"H821 DPC"}="1H82 DPC"; $resn_exch{"H822 DPC"}="2H82 DPC";
$resn_exch{" H91 DPC"}="1H9  DPC"; $resn_exch{" H92 DPC"}="2H9  DPC"; $resn_exch{"H911 DPC"}="1H91 DPC";
$resn_exch{"H912 DPC"}="2H91 DPC"; $resn_exch{"H921 DPC"}="1H92 DPC"; $resn_exch{"H922 DPC"}="2H92 DPC";
$resn_exch{"H923 DPC"}="3H92 DPC"; $resn_exch{"HNE1 DPC"}="1HNE DPC"; $resn_exch{"HNE2 DPC"}="2HNE DPC";
$resn_exch{"H22A DPD"}="2H2  DPD"; $resn_exch{" H21 DPD"}="1H2  DPD"; $resn_exch{"H32A DPD"}="2H3  DPD";
$resn_exch{" H31 DPD"}="1H3  DPD"; $resn_exch{" H52 DPD"}="2H5  DPD"; $resn_exch{" H51 DPD"}="1H5  DPD";
$resn_exch{" H62 DPD"}="2H6  DPD"; $resn_exch{" H61 DPD"}="1H6  DPD"; $resn_exch{" H72 DPD"}="2H7  DPD";
$resn_exch{" H71 DPD"}="1H7  DPD"; $resn_exch{"H193 DPD"}="3H19 DPD"; $resn_exch{"H192 DPD"}="2H19 DPD";
$resn_exch{"H191 DPD"}="1H19 DPD"; $resn_exch{"H412 DPD"}="2H41 DPD"; $resn_exch{"H411 DPD"}="1H41 DPD";
$resn_exch{"H422 DPD"}="2H42 DPD"; $resn_exch{"H421 DPD"}="1H42 DPD"; $resn_exch{"H432 DPD"}="2H43 DPD";
$resn_exch{"H431 DPD"}="1H43 DPD"; $resn_exch{"H443 DPD"}="3H44 DPD"; $resn_exch{"H442 DPD"}="2H44 DPD";
$resn_exch{"H441 DPD"}="1H44 DPD"; $resn_exch{"H512 DPD"}="2H51 DPD"; $resn_exch{"H511 DPD"}="1H51 DPD";
$resn_exch{"H522 DPD"}="2H52 DPD"; $resn_exch{"H521 DPD"}="1H52 DPD"; $resn_exch{"H532 DPD"}="2H53 DPD";
$resn_exch{"H531 DPD"}="1H53 DPD"; $resn_exch{"H543 DPD"}="3H54 DPD"; $resn_exch{"H542 DPD"}="2H54 DPD";
$resn_exch{"H541 DPD"}="1H54 DPD"; $resn_exch{" H11 DPE"}="1H1  DPE"; $resn_exch{" H12 DPE"}="2H1  DPE";
$resn_exch{" H21 DPE"}="1H2  DPE"; $resn_exch{" H22 DPE"}="2H2  DPE"; $resn_exch{" H31 DPE"}="1H3  DPE";
$resn_exch{" H32 DPE"}="2H3  DPE"; $resn_exch{" H41 DPE"}="1H4  DPE"; $resn_exch{" H42 DPE"}="2H4  DPE";
$resn_exch{" H51 DPE"}="1H5  DPE"; $resn_exch{" H52 DPE"}="2H5  DPE"; $resn_exch{" H61 DPE"}="1H6  DPE";
$resn_exch{" H62 DPE"}="2H6  DPE"; $resn_exch{" H71 DPE"}="1H7  DPE"; $resn_exch{" H72 DPE"}="2H7  DPE";
$resn_exch{" H81 DPE"}="1H8  DPE"; $resn_exch{" H82 DPE"}="2H8  DPE"; $resn_exch{" H91 DPE"}="1H9  DPE";
$resn_exch{" H92 DPE"}="2H9  DPE"; $resn_exch{"H101 DPE"}="1H10 DPE"; $resn_exch{"H102 DPE"}="2H10 DPE";
$resn_exch{"H111 DPE"}="1H11 DPE"; $resn_exch{"H112 DPE"}="2H11 DPE"; $resn_exch{"H113 DPE"}="3H11 DPE";
$resn_exch{" HM1 DPE"}="1HM  DPE"; $resn_exch{" HM2 DPE"}="2HM  DPE"; $resn_exch{" HM3 DPE"}="3HM  DPE";
$resn_exch{" H41 DPF"}="1H4  DPF"; $resn_exch{" H42 DPF"}="2H4  DPF"; $resn_exch{" H43 DPF"}="3H4  DPF";
$resn_exch{" H11 DPF"}="1H1  DPF"; $resn_exch{" H12 DPF"}="2H1  DPF"; $resn_exch{" H21 DPF"}="1H2  DPF";
$resn_exch{" H22 DPF"}="2H2  DPF"; $resn_exch{" H31 DPF"}="1H3  DPF"; $resn_exch{" H32 DPF"}="2H3  DPF";
$resn_exch{" H33 DPF"}="3H3  DPF"; $resn_exch{" H11 DPG"}="1H1  DPG"; $resn_exch{" H12 DPG"}="2H1  DPG";
$resn_exch{" H31 DPG"}="1H3  DPG"; $resn_exch{" H32 DPG"}="2H3  DPG"; $resn_exch{" H41 DPG"}="1H4  DPG";
$resn_exch{" H42 DPG"}="2H4  DPG"; $resn_exch{" H61 DPG"}="1H6  DPG"; $resn_exch{" H62 DPG"}="2H6  DPG";
$resn_exch{"H111 DPG"}="1H11 DPG"; $resn_exch{"H112 DPG"}="2H11 DPG"; $resn_exch{"H121 DPG"}="1H12 DPG";
$resn_exch{"H122 DPG"}="2H12 DPG"; $resn_exch{"H141 DPG"}="1H14 DPG"; $resn_exch{"H142 DPG"}="2H14 DPG";
$resn_exch{"H143 DPG"}="3H14 DPG"; $resn_exch{"H151 DPG"}="1H15 DPG"; $resn_exch{"H152 DPG"}="2H15 DPG";
$resn_exch{"H161 DPG"}="1H16 DPG"; $resn_exch{"H162 DPG"}="2H16 DPG"; $resn_exch{"H171 DPG"}="1H17 DPG";
$resn_exch{"H172 DPG"}="2H17 DPG"; $resn_exch{"H191 DPG"}="1H19 DPG"; $resn_exch{"H192 DPG"}="2H19 DPG";
$resn_exch{"H193 DPG"}="3H19 DPG"; $resn_exch{"H201 DPG"}="1H20 DPG"; $resn_exch{"H202 DPG"}="2H20 DPG";
$resn_exch{"H211 DPG"}="1H21 DPG"; $resn_exch{"H212 DPG"}="2H21 DPG"; $resn_exch{"H221 DPG"}="1H22 DPG";
$resn_exch{"H222 DPG"}="2H22 DPG"; $resn_exch{"H241 DPG"}="1H24 DPG"; $resn_exch{"H242 DPG"}="2H24 DPG";
$resn_exch{"H243 DPG"}="3H24 DPG"; $resn_exch{"H251 DPG"}="1H25 DPG"; $resn_exch{"H252 DPG"}="2H25 DPG";
$resn_exch{"H261 DPG"}="1H26 DPG"; $resn_exch{"H262 DPG"}="2H26 DPG"; $resn_exch{"H271 DPG"}="1H27 DPG";
$resn_exch{"H272 DPG"}="2H27 DPG"; $resn_exch{"H291 DPG"}="1H29 DPG"; $resn_exch{"H292 DPG"}="2H29 DPG";
$resn_exch{"H293 DPG"}="3H29 DPG"; $resn_exch{"H301 DPG"}="1H30 DPG"; $resn_exch{"H302 DPG"}="2H30 DPG";
$resn_exch{"H303 DPG"}="3H30 DPG"; $resn_exch{"H411 DPG"}="1H41 DPG"; $resn_exch{"H412 DPG"}="2H41 DPG";
$resn_exch{"H421 DPG"}="1H42 DPG"; $resn_exch{"H422 DPG"}="2H42 DPG"; $resn_exch{"H441 DPG"}="1H44 DPG";
$resn_exch{"H442 DPG"}="2H44 DPG"; $resn_exch{"H443 DPG"}="3H44 DPG"; $resn_exch{"H451 DPG"}="1H45 DPG";
$resn_exch{"H452 DPG"}="2H45 DPG"; $resn_exch{"H461 DPG"}="1H46 DPG"; $resn_exch{"H462 DPG"}="2H46 DPG";
$resn_exch{"H471 DPG"}="1H47 DPG"; $resn_exch{"H472 DPG"}="2H47 DPG"; $resn_exch{"H491 DPG"}="1H49 DPG";
$resn_exch{"H492 DPG"}="2H49 DPG"; $resn_exch{"H493 DPG"}="3H49 DPG"; $resn_exch{"H501 DPG"}="1H50 DPG";
$resn_exch{"H502 DPG"}="2H50 DPG"; $resn_exch{"H511 DPG"}="1H51 DPG"; $resn_exch{"H512 DPG"}="2H51 DPG";
$resn_exch{"H521 DPG"}="1H52 DPG"; $resn_exch{"H522 DPG"}="2H52 DPG"; $resn_exch{"H541 DPG"}="1H54 DPG";
$resn_exch{"H542 DPG"}="2H54 DPG"; $resn_exch{"H543 DPG"}="3H54 DPG"; $resn_exch{"H551 DPG"}="1H55 DPG";
$resn_exch{"H552 DPG"}="2H55 DPG"; $resn_exch{"H561 DPG"}="1H56 DPG"; $resn_exch{"H562 DPG"}="2H56 DPG";
$resn_exch{"H571 DPG"}="1H57 DPG"; $resn_exch{"H572 DPG"}="2H57 DPG"; $resn_exch{"H591 DPG"}="1H59 DPG";
$resn_exch{"H592 DPG"}="2H59 DPG"; $resn_exch{"H593 DPG"}="3H59 DPG"; $resn_exch{"H601 DPG"}="1H60 DPG";
$resn_exch{"H602 DPG"}="2H60 DPG"; $resn_exch{"H603 DPG"}="3H60 DPG"; $resn_exch{" HM1 DPH"}="1HM  DPH";
$resn_exch{" HM2 DPH"}="2HM  DPH"; $resn_exch{" HM3 DPH"}="3HM  DPH"; $resn_exch{" HB1 DPH"}="1HB  DPH";
$resn_exch{" HB2 DPH"}="2HB  DPH"; $resn_exch{" HC1 DPI"}="1HC  DPI"; $resn_exch{" HC2 DPI"}="2HC  DPI";
$resn_exch{" HC3 DPI"}="3HC  DPI"; $resn_exch{" HN2 DPI"}="2HN  DPI"; $resn_exch{"HC61 DPI"}="1HC6 DPI";
$resn_exch{"HC62 DPI"}="2HC6 DPI"; $resn_exch{" HC7 DPI"}="7HC  DPI"; $resn_exch{" HA1 DPI"}="1HA  DPI";
$resn_exch{" HB2 DPI"}="2HB  DPI"; $resn_exch{" HD3 DPI"}="3HD  DPI"; $resn_exch{" HB1 DPI"}="1HB  DPI";
$resn_exch{" HD2 DPI"}="2HD  DPI"; $resn_exch{" HE3 DPI"}="3HE  DPI"; $resn_exch{"HN01 DPI"}="1HN0 DPI";
$resn_exch{"HC11 DPI"}="1HC1 DPI"; $resn_exch{"HC41 DPI"}="1HC4 DPI"; $resn_exch{"HO51 DPI"}="1HO5 DPI";
$resn_exch{"HC71 DPI"}="1HC7 DPI"; $resn_exch{"HC81 DPI"}="1HC8 DPI"; $resn_exch{"HC02 DPI"}="2HC0 DPI";
$resn_exch{"HN42 DPI"}="2HN4 DPI"; $resn_exch{"HC52 DPI"}="2HC5 DPI"; $resn_exch{"HN23 DPI"}="3HN2 DPI";
$resn_exch{"HC33 DPI"}="3HC3 DPI"; $resn_exch{"HC73 DPI"}="3HC7 DPI"; $resn_exch{"HC83 DPI"}="3HC8 DPI";
$resn_exch{"HN34 DPI"}="4HN3 DPI"; $resn_exch{"HC44 DPI"}="4HC4 DPI"; $resn_exch{"HC84 DPI"}="4HC8 DPI";
$resn_exch{"HC94 DPI"}="4HC9 DPI"; $resn_exch{"HC15 DPI"}="5HC1 DPI"; $resn_exch{"HO35 DPI"}="5HO3 DPI";
$resn_exch{"HN45 DPI"}="5HN4 DPI"; $resn_exch{"HC55 DPI"}="5HC5 DPI"; $resn_exch{"HC85 DPI"}="5HC8 DPI";
$resn_exch{"HC16 DPI"}="6HC1 DPI"; $resn_exch{"HC26 DPI"}="6HC2 DPI"; $resn_exch{"HC36 DPI"}="6HC3 DPI";
$resn_exch{"HC46 DPI"}="6HC4 DPI"; $resn_exch{"HN86 DPI"}="6HN8 DPI"; $resn_exch{"HC96 DPI"}="6HC9 DPI";
$resn_exch{"HO17 DPI"}="7HO1 DPI"; $resn_exch{"HC47 DPI"}="7HC4 DPI"; $resn_exch{"HC67 DPI"}="7HC6 DPI";
$resn_exch{"HO87 DPI"}="7HO8 DPI"; $resn_exch{"HO97 DPI"}="7HO9 DPI"; $resn_exch{"HC'1 DPI"}="1HC' DPI";
$resn_exch{"H2'1 DPI"}="1H2' DPI"; $resn_exch{"H2'2 DPI"}="2H2' DPI"; $resn_exch{"HC'4 DPI"}="4HC' DPI";
$resn_exch{"HC'5 DPI"}="5HC' DPI"; $resn_exch{"H7'1 DPI"}="1H7' DPI"; $resn_exch{"H7'2 DPI"}="2H7' DPI";
$resn_exch{"H7'3 DPI"}="3H7' DPI"; $resn_exch{"H8'1 DPI"}="1H8' DPI"; $resn_exch{"H8'2 DPI"}="2H8' DPI";
$resn_exch{"HO'9 DPI"}="9HO' DPI"; $resn_exch{"H1'1 DPI"}="1H1\" DPI"; $resn_exch{"H1'2 DPI"}="2H1\" DPI";
$resn_exch{"H1'3 DPI"}="3H1\" DPI"; $resn_exch{"HC'2 DPI"}="2HC\" DPI"; $resn_exch{"HC'3 DPI"}="3HC\" DPI";
$resn_exch{"HCA4 DPI"}="4HC\" DPI"; $resn_exch{"HCA5 DPI"}="5HC\" DPI"; $resn_exch{"HC'6 DPI"}="6HC\" DPI";
$resn_exch{"HOA9 DPI"}="9HO\" DPI"; $resn_exch{"HOA1 DPI"}="1HOA DPI"; $resn_exch{"HCAA DPI"}="4HCA DPI";
$resn_exch{"HCAB DPI"}="5HCA DPI"; $resn_exch{"HCA6 DPI"}="6HCA DPI"; $resn_exch{"HCA7 DPI"}="7HCA DPI";
$resn_exch{"HCA8 DPI"}="8HCA DPI"; $resn_exch{"HOB1 DPI"}="1HOB DPI"; $resn_exch{"HOB2 DPI"}="2HOB DPI";
$resn_exch{"HOB3 DPI"}="3HOB DPI"; $resn_exch{"H4B1 DPI"}="1H4B DPI"; $resn_exch{"H4B2 DPI"}="2H4B DPI";
$resn_exch{"H4B3 DPI"}="3H4B DPI"; $resn_exch{"H281 DPI"}="1H28 DPI"; $resn_exch{"H282 DPI"}="2H28 DPI";
$resn_exch{"H311 DPI"}="1H31 DPI"; $resn_exch{"H312 DPI"}="2H31 DPI"; $resn_exch{"H2A1 DPI"}="1H2A DPI";
$resn_exch{"H2A2 DPI"}="2H2A DPI"; $resn_exch{"HOA3 DPI"}="3HOA DPI"; $resn_exch{" H91 DPJ"}="1H9  DPJ";
$resn_exch{" H92 DPJ"}="2H9  DPJ"; $resn_exch{" H93 DPJ"}="3H9  DPJ"; $resn_exch{" H71 DPJ"}="1H7  DPJ";
$resn_exch{" H72 DPJ"}="2H7  DPJ"; $resn_exch{" H61 DPJ"}="1H6  DPJ"; $resn_exch{" H62 DPJ"}="2H6  DPJ";
$resn_exch{" H81 DPJ"}="1H8  DPJ"; $resn_exch{" H82 DPJ"}="2H8  DPJ"; $resn_exch{" H83 DPJ"}="3H8  DPJ";
$resn_exch{"H121 DPK"}="1H12 DPK"; $resn_exch{"H122 DPK"}="2H12 DPK"; $resn_exch{"H101 DPK"}="1H10 DPK";
$resn_exch{"H102 DPK"}="2H10 DPK"; $resn_exch{"H9N1 DPK"}="1H9N DPK"; $resn_exch{"H9N2 DPK"}="2H9N DPK";
$resn_exch{"H9N3 DPK"}="3H9N DPK"; $resn_exch{"H8C1 DPK"}="1H8C DPK"; $resn_exch{"H8C2 DPK"}="2H8C DPK";
$resn_exch{"H8C3 DPK"}="3H8C DPK"; $resn_exch{"H7C1 DPK"}="1H7C DPK"; $resn_exch{"H7C2 DPK"}="2H7C DPK";
$resn_exch{" HB2 DPL"}="1HB  DPL"; $resn_exch{" HB3 DPL"}="2HB  DPL"; $resn_exch{" HD3 DPL"}="1HD  DPL";
$resn_exch{" HD2 DPL"}="2HD  DPL"; $resn_exch{"H5A1 DPM"}="1H5A DPM"; $resn_exch{"H5A2 DPM"}="2H5A DPM";
$resn_exch{"H7A1 DPM"}="1H7A DPM"; $resn_exch{"H7A2 DPM"}="2H7A DPM"; $resn_exch{"H8A1 DPM"}="1H8A DPM";
$resn_exch{"H8A2 DPM"}="2H8A DPM"; $resn_exch{"HCA1 DPM"}="1HCA DPM"; $resn_exch{"HCA2 DPM"}="2HCA DPM";
$resn_exch{"HCA3 DPM"}="3HCA DPM"; $resn_exch{"HOA2 DPM"}="2HOA DPM"; $resn_exch{"HOA4 DPM"}="4HOA DPM";
$resn_exch{"H4B1 DPM"}="1H4B DPM"; $resn_exch{"H4B2 DPM"}="2H4B DPM"; $resn_exch{"H5B1 DPM"}="1H5B DPM";
$resn_exch{"H5B2 DPM"}="2H5B DPM"; $resn_exch{"H7B1 DPM"}="1H7B DPM"; $resn_exch{"H7B2 DPM"}="2H7B DPM";
$resn_exch{"H8B1 DPM"}="1H8B DPM"; $resn_exch{"H8B2 DPM"}="2H8B DPM"; $resn_exch{"HCB1 DPM"}="1HCB DPM";
$resn_exch{"HCB2 DPM"}="2HCB DPM"; $resn_exch{"HOB2 DPM"}="2HOB DPM"; $resn_exch{"HOB4 DPM"}="4HOB DPM";
$resn_exch{" H2  DPN"}=" HN2 DPN"; $resn_exch{" HB2 DPN"}="1HB  DPN"; $resn_exch{" HB3 DPN"}="2HB  DPN";
$resn_exch{" H   DPP"}="1HN  DPP"; $resn_exch{" H2  DPP"}="2HN  DPP"; $resn_exch{" HB2 DPP"}="1HB  DPP";
$resn_exch{" HB3 DPP"}="2HB  DPP"; $resn_exch{" HG1 DPP"}="1HG  DPP"; $resn_exch{" HG2 DPP"}="2HG  DPP";
$resn_exch{" HB1 DPQ"}="1HB  DPQ"; $resn_exch{" HB2 DPQ"}="2HB  DPQ"; $resn_exch{"HD11 DPQ"}="1HD1 DPQ";
$resn_exch{"HD12 DPQ"}="2HD1 DPQ"; $resn_exch{" H   DPR"}=" HT1 DPR"; $resn_exch{" HB2 DPR"}="1HB  DPR";
$resn_exch{" HB3 DPR"}="2HB  DPR"; $resn_exch{" HG2 DPR"}="1HG  DPR"; $resn_exch{" HG3 DPR"}="2HG  DPR";
$resn_exch{" HD2 DPR"}="1HD  DPR"; $resn_exch{" HD3 DPR"}="2HD  DPR"; $resn_exch{" H71 DPS"}="1H7  DPS";
$resn_exch{" H72 DPS"}="2H7  DPS"; $resn_exch{"H131 DPS"}="1H13 DPS"; $resn_exch{"H132 DPS"}="2H13 DPS";
$resn_exch{"H151 DPS"}="1H15 DPS"; $resn_exch{"H152 DPS"}="2H15 DPS"; $resn_exch{"H181 DPS"}="1H18 DPS";
$resn_exch{"H182 DPS"}="2H18 DPS"; $resn_exch{"H201 DPS"}="1H20 DPS"; $resn_exch{"H202 DPS"}="2H20 DPS";
$resn_exch{"H111 DPT"}="1H11 DPT"; $resn_exch{"H112 DPT"}="2H11 DPT"; $resn_exch{"H113 DPT"}="3H11 DPT";
$resn_exch{"H121 DPT"}="1H12 DPT"; $resn_exch{"H122 DPT"}="2H12 DPT"; $resn_exch{"H123 DPT"}="3H12 DPT";
$resn_exch{" HA1 DPU"}="1HA  DPU"; $resn_exch{" HA2 DPU"}="2HA  DPU"; $resn_exch{" HB1 DPU"}="1HB  DPU";
$resn_exch{" HB2 DPU"}="2HB  DPU"; $resn_exch{" HG1 DPU"}="1HG  DPU"; $resn_exch{" HG2 DPU"}="2HG  DPU";
$resn_exch{" HD1 DPU"}="1HD  DPU"; $resn_exch{" HD2 DPU"}="2HD  DPU"; $resn_exch{" HE1 DPU"}="1HE  DPU";
$resn_exch{" HE2 DPU"}="2HE  DPU"; $resn_exch{" HS1 DPU"}="1HS  DPU"; $resn_exch{" HS2 DPU"}="2HS  DPU";
$resn_exch{" HS3 DPU"}="3HS  DPU"; $resn_exch{"HN21 DPU"}="1HN2 DPU"; $resn_exch{"HN22 DPU"}="2HN2 DPU";
$resn_exch{"H171 DPX"}="1H17 DPX"; $resn_exch{"H172 DPX"}="2H17 DPX"; $resn_exch{"H151 DPX"}="1H15 DPX";
$resn_exch{"H152 DPX"}="2H15 DPX"; $resn_exch{"H153 DPX"}="3H15 DPX"; $resn_exch{" H61 DPX"}="1H6  DPX";
$resn_exch{" H62 DPX"}="2H6  DPX"; $resn_exch{"HND1 DPX"}="1HND DPX"; $resn_exch{"HND2 DPX"}="2HND DPX";
$resn_exch{"H141 DPX"}="1H14 DPX"; $resn_exch{"H142 DPX"}="2H14 DPX"; $resn_exch{"H143 DPX"}="3H14 DPX";
$resn_exch{"HOP2 DPY"}="2HOP DPY"; $resn_exch{"H2'1 DPY"}="1H2* DPY"; $resn_exch{"H2'2 DPY"}="2H2* DPY";
$resn_exch{"H5'1 DPY"}="1H5* DPY"; $resn_exch{"H5'2 DPY"}="2H5* DPY"; $resn_exch{" H4' DPY"}=" H4* DPY";
$resn_exch{" H1' DPY"}=" H1* DPY"; $resn_exch{" H3' DPY"}=" H3* DPY"; $resn_exch{"HOP3 DPY"}="3HOP DPY";
$resn_exch{"H131 DPZ"}="1H13 DPZ"; $resn_exch{"H132 DPZ"}="2H13 DPZ"; $resn_exch{"H141 DPZ"}="1H14 DPZ";
$resn_exch{"H142 DPZ"}="2H14 DPZ"; $resn_exch{" H21 DQA"}="1H2  DQA"; $resn_exch{" H22 DQA"}="2H2  DQA";
$resn_exch{" H61 DQA"}="1H6  DQA"; $resn_exch{" H62 DQA"}="2H6  DQA"; $resn_exch{" HA1 DQB"}="1HA  DQB";
$resn_exch{" HA2 DQB"}="2HA  DQB"; $resn_exch{" HB1 DQB"}="1HB  DQB"; $resn_exch{" HB2 DQB"}="2HB  DQB";
$resn_exch{" HG1 DQB"}="1HG  DQB"; $resn_exch{" HG2 DQB"}="2HG  DQB"; $resn_exch{"HA21 DQB"}="1HA2 DQB";
$resn_exch{"HA22 DQB"}="2HA2 DQB"; $resn_exch{" H91 DQB"}="1H9  DQB"; $resn_exch{" H92 DQB"}="2H9  DQB";
$resn_exch{"H2M1 DQN"}="1H2M DQN"; $resn_exch{"H2M2 DQN"}="2H2M DQN"; $resn_exch{"H2M3 DQN"}="3H2M DQN";
$resn_exch{"H3M1 DQN"}="1H3M DQN"; $resn_exch{"H3M2 DQN"}="2H3M DQN"; $resn_exch{"H3M3 DQN"}="3H3M DQN";
$resn_exch{"H5M1 DQN"}="1H5M DQN"; $resn_exch{"H5M2 DQN"}="2H5M DQN"; $resn_exch{"H5M3 DQN"}="3H5M DQN";
$resn_exch{"H6M1 DQN"}="1H6M DQN"; $resn_exch{"H6M2 DQN"}="2H6M DQN"; $resn_exch{"H6M3 DQN"}="3H6M DQN";
$resn_exch{"H141 DQO"}="1H14 DQO"; $resn_exch{"H142 DQO"}="2H14 DQO"; $resn_exch{"H401 DQO"}="1H40 DQO";
$resn_exch{"H402 DQO"}="2H40 DQO"; $resn_exch{"H411 DQO"}="1H41 DQO"; $resn_exch{"H412 DQO"}="2H41 DQO";
$resn_exch{"H421 DQO"}="1H42 DQO"; $resn_exch{"H431 DQO"}="1H43 DQO"; $resn_exch{"H432 DQO"}="2H43 DQO";
$resn_exch{"H441 DQO"}="1H44 DQO"; $resn_exch{"H442 DQO"}="2H44 DQO"; $resn_exch{" H51 DQQ"}="1H5  DQQ";
$resn_exch{" H52 DQQ"}="2H5  DQQ"; $resn_exch{" H61 DQQ"}="1H6  DQQ"; $resn_exch{" H62 DQQ"}="2H6  DQQ";
$resn_exch{"H101 DQR"}="1H10 DQR"; $resn_exch{"H102 DQR"}="2H10 DQR"; $resn_exch{"H111 DQR"}="1H11 DQR";
$resn_exch{"H112 DQR"}="2H11 DQR"; $resn_exch{"H61A DQR"}="1H6  DQR"; $resn_exch{" H62 DQR"}="2H6  DQR";
$resn_exch{"H601 DQR"}="1H60 DQR"; $resn_exch{"H602 DQR"}="2H60 DQR"; $resn_exch{"H611 DQR"}="1H61 DQR";
$resn_exch{"H612 DQR"}="2H61 DQR"; $resn_exch{"HN41 DQU"}="1HN4 DQU"; $resn_exch{"HN42 DQU"}="2HN4 DQU";
$resn_exch{"HN21 DQU"}="1HN2 DQU"; $resn_exch{"HN22 DQU"}="2HN2 DQU"; $resn_exch{"HAJ1 DR0"}="1HAJ DR0";
$resn_exch{"HAJ2 DR0"}="2HAJ DR0"; $resn_exch{"HAJ3 DR0"}="3HAJ DR0"; $resn_exch{"HAI1 DR0"}="1HAI DR0";
$resn_exch{"HAI2 DR0"}="2HAI DR0"; $resn_exch{"HAH1 DR0"}="1HAH DR0"; $resn_exch{"HAH2 DR0"}="2HAH DR0";
$resn_exch{"HAG1 DR0"}="1HAG DR0"; $resn_exch{"HAG2 DR0"}="2HAG DR0"; $resn_exch{"HAF1 DR0"}="1HAF DR0";
$resn_exch{"HAF2 DR0"}="2HAF DR0"; $resn_exch{"HAE1 DR0"}="1HAE DR0"; $resn_exch{"HAE2 DR0"}="2HAE DR0";
$resn_exch{"HAC1 DR0"}="1HAC DR0"; $resn_exch{"HAC2 DR0"}="2HAC DR0"; $resn_exch{"HAC3 DR0"}="3HAC DR0";
$resn_exch{"HAK1 DR0"}="1HAK DR0"; $resn_exch{"HAK2 DR0"}="2HAK DR0"; $resn_exch{"HAK3 DR0"}="3HAK DR0";
$resn_exch{"HAB1 DR0"}="1HAB DR0"; $resn_exch{"HAB2 DR0"}="2HAB DR0"; $resn_exch{"H181 DR1"}="1H18 DR1";
$resn_exch{"H182 DR1"}="2H18 DR1"; $resn_exch{"H183 DR1"}="3H18 DR1"; $resn_exch{"HAB1 DR2"}="1HAB DR2";
$resn_exch{"HAB2 DR2"}="2HAB DR2"; $resn_exch{"HAB3 DR2"}="3HAB DR2"; $resn_exch{"HAV1 DR2"}="1HAV DR2";
$resn_exch{"HAV2 DR2"}="2HAV DR2"; $resn_exch{" H1' DR2"}=" H1* DR2"; $resn_exch{" H5' DR2"}=" H5* DR2";
$resn_exch{"H6'1 DR2"}="1H6* DR2"; $resn_exch{"H6'2 DR2"}="2H6* DR2"; $resn_exch{" H6' DR2"}=" H6* DR2";
$resn_exch{" H4' DR2"}=" H4* DR2"; $resn_exch{" H3' DR2"}=" H3* DR2"; $resn_exch{" H2' DR2"}=" H2* DR2";
$resn_exch{"HAC1 DR2"}="1HAC DR2"; $resn_exch{"HAC2 DR2"}="2HAC DR2"; $resn_exch{"HAC3 DR2"}="3HAC DR2";
$resn_exch{"HAC1 DR3"}="1HAC DR3"; $resn_exch{"HAC2 DR3"}="2HAC DR3"; $resn_exch{"HAC3 DR3"}="3HAC DR3";
$resn_exch{" H2' DR3"}=" H2* DR3"; $resn_exch{" H3' DR3"}=" H3* DR3"; $resn_exch{" H4' DR3"}=" H4* DR3";
$resn_exch{" H5' DR3"}=" H5* DR3"; $resn_exch{"H6'1 DR3"}="1H6* DR3"; $resn_exch{"H6'2 DR3"}="2H6* DR3";
$resn_exch{" H6' DR3"}=" H6* DR3"; $resn_exch{" H1' DR3"}=" H1* DR3"; $resn_exch{"HAV1 DR3"}="1HAV DR3";
$resn_exch{"HAV2 DR3"}="2HAV DR3"; $resn_exch{"HAB1 DR3"}="1HAB DR3"; $resn_exch{"HAB2 DR3"}="2HAB DR3";
$resn_exch{"HAB3 DR3"}="3HAB DR3"; $resn_exch{"HAA1 DR4"}="1HAA DR4"; $resn_exch{"HAA2 DR4"}="2HAA DR4";
$resn_exch{"HAA3 DR4"}="3HAA DR4"; $resn_exch{"HAO1 DR4"}="1HAO DR4"; $resn_exch{"HAO2 DR4"}="2HAO DR4";
$resn_exch{"HAL1 DR4"}="1HAL DR4"; $resn_exch{"HAL2 DR4"}="2HAL DR4"; $resn_exch{"HAJ1 DR4"}="1HAJ DR4";
$resn_exch{"HAJ2 DR4"}="2HAJ DR4"; $resn_exch{"HAH1 DR4"}="1HAH DR4"; $resn_exch{"HAH2 DR4"}="2HAH DR4";
$resn_exch{"HAG1 DR4"}="1HAG DR4"; $resn_exch{"HAG2 DR4"}="2HAG DR4"; $resn_exch{"HAI1 DR4"}="1HAI DR4";
$resn_exch{"HAI2 DR4"}="2HAI DR4"; $resn_exch{"HAK1 DR4"}="1HAK DR4"; $resn_exch{"HAK2 DR4"}="2HAK DR4";
$resn_exch{"HAM1 DR4"}="1HAM DR4"; $resn_exch{"HAM2 DR4"}="2HAM DR4"; $resn_exch{" H61 DR4"}="1H6  DR4";
$resn_exch{" H62 DR4"}="2H6  DR4"; $resn_exch{" H61 DR5"}="1H6  DR5"; $resn_exch{" H62 DR5"}="2H6  DR5";
$resn_exch{"HAA1 DR5"}="1HAA DR5"; $resn_exch{"HAA2 DR5"}="2HAA DR5"; $resn_exch{"HAA3 DR5"}="3HAA DR5";
$resn_exch{"HAJ1 DR5"}="1HAJ DR5"; $resn_exch{"HAJ2 DR5"}="2HAJ DR5"; $resn_exch{"HCY1 DR6"}="1HCY DR6";
$resn_exch{"HCY2 DR6"}="2HCY DR6"; $resn_exch{"HAA1 DR7"}="1HAA DR7"; $resn_exch{"HAA2 DR7"}="2HAA DR7";
$resn_exch{"HAA3 DR7"}="3HAA DR7"; $resn_exch{"HAE1 DR7"}="1HAE DR7"; $resn_exch{"HAE2 DR7"}="2HAE DR7";
$resn_exch{"HAE3 DR7"}="3HAE DR7"; $resn_exch{"HG21 DR7"}="1HG2 DR7"; $resn_exch{"HG22 DR7"}="2HG2 DR7";
$resn_exch{"HG23 DR7"}="3HG2 DR7"; $resn_exch{"HG11 DR7"}="1HG1 DR7"; $resn_exch{"HG12 DR7"}="2HG1 DR7";
$resn_exch{"HG13 DR7"}="3HG1 DR7"; $resn_exch{"HBA1 DR7"}="1HBA DR7"; $resn_exch{"HBA2 DR7"}="2HBA DR7";
$resn_exch{"HBC1 DR7"}="1HBC DR7"; $resn_exch{"HBC2 DR7"}="2HBC DR7"; $resn_exch{"HAH1 DR7"}="1HAH DR7";
$resn_exch{"HAH2 DR7"}="2HAH DR7"; $resn_exch{"HAH3 DR7"}="3HAH DR7"; $resn_exch{"HAF1 DR7"}="1HAF DR7";
$resn_exch{"HAF2 DR7"}="2HAF DR7"; $resn_exch{"HAF3 DR7"}="3HAF DR7"; $resn_exch{"HAG1 DR7"}="1HAG DR7";
$resn_exch{"HAG2 DR7"}="2HAG DR7"; $resn_exch{"HAG3 DR7"}="3HAG DR7"; $resn_exch{"HAB1 DR7"}="1HAB DR7";
$resn_exch{"HAB2 DR7"}="2HAB DR7"; $resn_exch{"HAB3 DR7"}="3HAB DR7"; $resn_exch{"HBB1 DR7"}="1HBB DR7";
$resn_exch{"HBB2 DR7"}="2HBB DR7"; $resn_exch{"HAJ1 DR8"}="1HAJ DR8"; $resn_exch{"HAJ2 DR8"}="2HAJ DR8";
$resn_exch{"HAJ3 DR8"}="3HAJ DR8"; $resn_exch{"HAK1 DR8"}="1HAK DR8"; $resn_exch{"HAK2 DR8"}="2HAK DR8";
$resn_exch{"HAK3 DR8"}="3HAK DR8"; $resn_exch{"HAA1 DR8"}="1HAA DR8"; $resn_exch{"HAA2 DR8"}="2HAA DR8";
$resn_exch{"HAA3 DR8"}="3HAA DR8"; $resn_exch{"HAB1 DR8"}="1HAB DR8"; $resn_exch{"HAB2 DR8"}="2HAB DR8";
$resn_exch{"HAB3 DR8"}="3HAB DR8"; $resn_exch{"HBX1 DR9"}="1HBX DR9"; $resn_exch{"HBX2 DR9"}="2HBX DR9";
$resn_exch{"HBX3 DR9"}="3HBX DR9"; $resn_exch{"HBV1 DR9"}="1HBV DR9"; $resn_exch{"HBV2 DR9"}="2HBV DR9";
$resn_exch{"HBS1 DR9"}="1HBS DR9"; $resn_exch{"HBS2 DR9"}="2HBS DR9"; $resn_exch{"HBH1 DR9"}="1HBH DR9";
$resn_exch{"HBH2 DR9"}="2HBH DR9"; $resn_exch{"HAQ1 DR9"}="1HAQ DR9"; $resn_exch{"HAQ2 DR9"}="2HAQ DR9";
$resn_exch{"HAP1 DR9"}="1HAP DR9"; $resn_exch{"HAP2 DR9"}="2HAP DR9"; $resn_exch{"HAO1 DR9"}="1HAO DR9";
$resn_exch{"HAO2 DR9"}="2HAO DR9"; $resn_exch{"HAN1 DR9"}="1HAN DR9"; $resn_exch{"HAN2 DR9"}="2HAN DR9";
$resn_exch{"HAK1 DR9"}="1HAK DR9"; $resn_exch{"HAK2 DR9"}="2HAK DR9"; $resn_exch{"HAJ1 DR9"}="1HAJ DR9";
$resn_exch{"HAJ2 DR9"}="2HAJ DR9"; $resn_exch{"HAI1 DR9"}="1HAI DR9"; $resn_exch{"HAI2 DR9"}="2HAI DR9";
$resn_exch{"HAH1 DR9"}="1HAH DR9"; $resn_exch{"HAH2 DR9"}="2HAH DR9"; $resn_exch{"HAG1 DR9"}="1HAG DR9";
$resn_exch{"HAG2 DR9"}="2HAG DR9"; $resn_exch{"HAF1 DR9"}="1HAF DR9"; $resn_exch{"HAF2 DR9"}="2HAF DR9";
$resn_exch{"HAE1 DR9"}="1HAE DR9"; $resn_exch{"HAE2 DR9"}="2HAE DR9"; $resn_exch{"HAB1 DR9"}="1HAB DR9";
$resn_exch{"HAB2 DR9"}="2HAB DR9"; $resn_exch{"HBK1 DR9"}="1HBK DR9"; $resn_exch{"HBK2 DR9"}="2HBK DR9";
$resn_exch{"HBO1 DR9"}="1HBO DR9"; $resn_exch{"HBO2 DR9"}="2HBO DR9"; $resn_exch{"HBQ1 DR9"}="1HBQ DR9";
$resn_exch{"HBQ2 DR9"}="2HBQ DR9"; $resn_exch{"HBB1 DR9"}="1HBB DR9"; $resn_exch{"HBB2 DR9"}="2HBB DR9";
$resn_exch{"HBA1 DR9"}="1HBA DR9"; $resn_exch{"HBA2 DR9"}="2HBA DR9"; $resn_exch{"HAZ1 DR9"}="1HAZ DR9";
$resn_exch{"HAZ2 DR9"}="2HAZ DR9"; $resn_exch{"HAY1 DR9"}="1HAY DR9"; $resn_exch{"HAY2 DR9"}="2HAY DR9";
$resn_exch{"HAX1 DR9"}="1HAX DR9"; $resn_exch{"HAX2 DR9"}="2HAX DR9"; $resn_exch{"HAW1 DR9"}="1HAW DR9";
$resn_exch{"HAW2 DR9"}="2HAW DR9"; $resn_exch{"HAV1 DR9"}="1HAV DR9"; $resn_exch{"HAV2 DR9"}="2HAV DR9";
$resn_exch{"HAS1 DR9"}="1HAS DR9"; $resn_exch{"HAS2 DR9"}="2HAS DR9"; $resn_exch{"HAR1 DR9"}="1HAR DR9";
$resn_exch{"HAR2 DR9"}="2HAR DR9"; $resn_exch{"HBI1 DR9"}="1HBI DR9"; $resn_exch{"HBI2 DR9"}="2HBI DR9";
$resn_exch{"HBT1 DR9"}="1HBT DR9"; $resn_exch{"HBT2 DR9"}="2HBT DR9"; $resn_exch{"HBW1 DR9"}="1HBW DR9";
$resn_exch{"HBW2 DR9"}="2HBW DR9"; $resn_exch{"HBY1 DR9"}="1HBY DR9"; $resn_exch{"HBY2 DR9"}="2HBY DR9";
$resn_exch{"HBY3 DR9"}="3HBY DR9"; $resn_exch{"H131 DRA"}="1H13 DRA"; $resn_exch{"H132 DRA"}="2H13 DRA";
$resn_exch{"H141 DRA"}="1H14 DRA"; $resn_exch{"H142 DRA"}="2H14 DRA"; $resn_exch{"H143 DRA"}="3H14 DRA";
$resn_exch{" H81 DRA"}="1H8  DRA"; $resn_exch{" H82 DRA"}="2H8  DRA"; $resn_exch{" H1' DRA"}=" H1* DRA";
$resn_exch{"H2'1 DRA"}="1H2* DRA"; $resn_exch{"H2'2 DRA"}="2H2* DRA"; $resn_exch{" H3' DRA"}=" H3* DRA";
$resn_exch{"H7'1 DRA"}="1H7* DRA"; $resn_exch{"H7'2 DRA"}="2H7* DRA"; $resn_exch{"H7'3 DRA"}="3H7* DRA";
$resn_exch{"H8'1 DRA"}="1H8* DRA"; $resn_exch{"H8'2 DRA"}="2H8* DRA"; $resn_exch{"H8'3 DRA"}="3H8* DRA";
$resn_exch{" H5' DRA"}=" H5* DRA"; $resn_exch{"H6'1 DRA"}="1H6* DRA"; $resn_exch{"H6'2 DRA"}="2H6* DRA";
$resn_exch{"H6'3 DRA"}="3H6* DRA"; $resn_exch{" H4' DRA"}=" H4* DRA"; $resn_exch{"H321 DRA"}="1H32 DRA";
$resn_exch{"H322 DRA"}="2H32 DRA"; $resn_exch{"HO16 DRA"}="6HO1 DRA"; $resn_exch{"H361 DRA"}="1H36 DRA";
$resn_exch{"H362 DRA"}="2H36 DRA"; $resn_exch{"H363 DRA"}="3H36 DRA"; $resn_exch{"H411 DRA"}="1H41 DRA";
$resn_exch{"H412 DRA"}="2H41 DRA"; $resn_exch{"H413 DRA"}="3H41 DRA"; $resn_exch{"H381 DRA"}="1H38 DRA";
$resn_exch{"H382 DRA"}="2H38 DRA"; $resn_exch{"H401 DRA"}="1H40 DRA"; $resn_exch{"H402 DRA"}="2H40 DRA";
$resn_exch{"H383 DRB"}="3H38 DRB"; $resn_exch{"H382 DRB"}="2H38 DRB"; $resn_exch{"H381 DRB"}="1H38 DRB";
$resn_exch{"H392 DRB"}="2H39 DRB"; $resn_exch{"H391 DRB"}="1H39 DRB"; $resn_exch{"H402 DRB"}="2H40 DRB";
$resn_exch{"H401 DRB"}="1H40 DRB"; $resn_exch{"H412 DRB"}="2H41 DRB"; $resn_exch{"H411 DRB"}="1H41 DRB";
$resn_exch{"H422 DRB"}="2H42 DRB"; $resn_exch{"H421 DRB"}="1H42 DRB"; $resn_exch{"H432 DRB"}="2H43 DRB";
$resn_exch{"H431 DRB"}="1H43 DRB"; $resn_exch{"H442 DRB"}="2H44 DRB"; $resn_exch{"H441 DRB"}="1H44 DRB";
$resn_exch{"H452 DRB"}="2H45 DRB"; $resn_exch{"H451 DRB"}="1H45 DRB"; $resn_exch{"H462 DRB"}="2H46 DRB";
$resn_exch{"H461 DRB"}="1H46 DRB"; $resn_exch{"H512 DRB"}="2H51 DRB"; $resn_exch{"H511 DRB"}="1H51 DRB";
$resn_exch{"H552 DRB"}="2H55 DRB"; $resn_exch{"H551 DRB"}="1H55 DRB"; $resn_exch{"H572 DRB"}="2H57 DRB";
$resn_exch{"H571 DRB"}="1H57 DRB"; $resn_exch{"H582 DRB"}="2H58 DRB"; $resn_exch{"H581 DRB"}="1H58 DRB";
$resn_exch{"H592 DRB"}="2H59 DRB"; $resn_exch{"H591 DRB"}="1H59 DRB"; $resn_exch{"H201 DRC"}="1H20 DRC";
$resn_exch{"H202 DRC"}="2H20 DRC"; $resn_exch{"H211 DRC"}="1H21 DRC"; $resn_exch{"H212 DRC"}="2H21 DRC";
$resn_exch{"H221 DRC"}="1H22 DRC"; $resn_exch{"H222 DRC"}="2H22 DRC"; $resn_exch{"H231 DRC"}="1H23 DRC";
$resn_exch{"H232 DRC"}="2H23 DRC"; $resn_exch{"H181 DRC"}="1H18 DRC"; $resn_exch{"H182 DRC"}="2H18 DRC";
$resn_exch{"H171 DRC"}="1H17 DRC"; $resn_exch{"H172 DRC"}="2H17 DRC"; $resn_exch{"HN91 DRC"}="1HN9 DRC";
$resn_exch{"HN92 DRC"}="2HN9 DRC"; $resn_exch{"HAA1 DRD"}="1HAA DRD"; $resn_exch{"HAA2 DRD"}="2HAA DRD";
$resn_exch{"HAB1 DRD"}="1HAB DRD"; $resn_exch{"HAB2 DRD"}="2HAB DRD"; $resn_exch{"HAC1 DRD"}="1HAC DRD";
$resn_exch{"HAC2 DRD"}="2HAC DRD"; $resn_exch{"HBL1 DRD"}="1HBL DRD"; $resn_exch{"HBL2 DRD"}="2HBL DRD";
$resn_exch{"HBM1 DRD"}="1HBM DRD"; $resn_exch{"HBM2 DRD"}="2HBM DRD"; $resn_exch{"HBN1 DRD"}="1HBN DRD";
$resn_exch{"HBN2 DRD"}="2HBN DRD"; $resn_exch{"HBN3 DRD"}="3HBN DRD"; $resn_exch{"HAL1 DRD"}="1HAL DRD";
$resn_exch{"HAL2 DRD"}="2HAL DRD"; $resn_exch{"HAL3 DRD"}="3HAL DRD"; $resn_exch{"HAK1 DRD"}="1HAK DRD";
$resn_exch{"HAK2 DRD"}="2HAK DRD"; $resn_exch{"HAK3 DRD"}="3HAK DRD"; $resn_exch{"HAA1 DRE"}="1HAA DRE";
$resn_exch{"HAA2 DRE"}="2HAA DRE"; $resn_exch{"HAB1 DRE"}="1HAB DRE"; $resn_exch{"HAB2 DRE"}="2HAB DRE";
$resn_exch{"HAC1 DRE"}="1HAC DRE"; $resn_exch{"HAC2 DRE"}="2HAC DRE"; $resn_exch{"HAD1 DRE"}="1HAD DRE";
$resn_exch{"HAD2 DRE"}="2HAD DRE"; $resn_exch{"HAE1 DRE"}="1HAE DRE"; $resn_exch{"HAE2 DRE"}="2HAE DRE";
$resn_exch{"HAF1 DRE"}="1HAF DRE"; $resn_exch{"HAF2 DRE"}="2HAF DRE"; $resn_exch{"HAG1 DRE"}="1HAG DRE";
$resn_exch{"HAG2 DRE"}="2HAG DRE"; $resn_exch{"HAI1 DRE"}="1HAI DRE"; $resn_exch{"HAI2 DRE"}="2HAI DRE";
$resn_exch{"HAJ1 DRE"}="1HAJ DRE"; $resn_exch{"HAJ2 DRE"}="2HAJ DRE"; $resn_exch{"HAK1 DRE"}="1HAK DRE";
$resn_exch{"HAK2 DRE"}="2HAK DRE"; $resn_exch{"HAL1 DRE"}="1HAL DRE"; $resn_exch{"HAL2 DRE"}="2HAL DRE";
$resn_exch{"HAM1 DRE"}="1HAM DRE"; $resn_exch{"HAM2 DRE"}="2HAM DRE"; $resn_exch{"HAN1 DRE"}="1HAN DRE";
$resn_exch{"HAN2 DRE"}="2HAN DRE"; $resn_exch{"HAO1 DRE"}="1HAO DRE"; $resn_exch{"HAO2 DRE"}="2HAO DRE";
$resn_exch{"H481 DRF"}="1H48 DRF"; $resn_exch{"H482 DRF"}="2H48 DRF"; $resn_exch{"H491 DRF"}="1H49 DRF";
$resn_exch{"H492 DRF"}="2H49 DRF"; $resn_exch{"H493 DRF"}="3H49 DRF"; $resn_exch{"H401 DRF"}="1H40 DRF";
$resn_exch{"H402 DRF"}="2H40 DRF"; $resn_exch{"H241 DRF"}="1H24 DRF"; $resn_exch{"H242 DRF"}="2H24 DRF";
$resn_exch{"H231 DRF"}="1H23 DRF"; $resn_exch{"H232 DRF"}="2H23 DRF"; $resn_exch{"HC51 DRG"}="1HC5 DRG";
$resn_exch{"HC52 DRG"}="2HC5 DRG"; $resn_exch{"HC61 DRG"}="1HC6 DRG"; $resn_exch{"HC62 DRG"}="2HC6 DRG";
$resn_exch{"H10C DRG"}="CH10 DRG"; $resn_exch{"HN31 DRG"}="1HN3 DRG"; $resn_exch{"HN32 DRG"}="2HN3 DRG";
$resn_exch{"HAB1 DRH"}="1HAB DRH"; $resn_exch{"HAB2 DRH"}="2HAB DRH"; $resn_exch{"HAB3 DRH"}="3HAB DRH";
$resn_exch{"HAO1 DRH"}="1HAO DRH"; $resn_exch{"HAO2 DRH"}="2HAO DRH"; $resn_exch{"HAC1 DRH"}="1HAC DRH";
$resn_exch{"HAC2 DRH"}="2HAC DRH"; $resn_exch{"HAC3 DRH"}="3HAC DRH"; $resn_exch{"HAT1 DRH"}="1HAT DRH";
$resn_exch{"HAT2 DRH"}="2HAT DRH"; $resn_exch{"HAV1 DRH"}="1HAV DRH"; $resn_exch{"HAV2 DRH"}="2HAV DRH";
$resn_exch{"HAU1 DRH"}="1HAU DRH"; $resn_exch{"HAU2 DRH"}="2HAU DRH"; $resn_exch{"HAS1 DRH"}="1HAS DRH";
$resn_exch{"HAS2 DRH"}="2HAS DRH"; $resn_exch{"HAR1 DRH"}="1HAR DRH"; $resn_exch{"HAR2 DRH"}="2HAR DRH";
$resn_exch{"HAQ1 DRH"}="1HAQ DRH"; $resn_exch{"HAQ2 DRH"}="2HAQ DRH"; $resn_exch{"HAP1 DRH"}="1HAP DRH";
$resn_exch{"HAP2 DRH"}="2HAP DRH"; $resn_exch{"HAN1 DRH"}="1HAN DRH"; $resn_exch{"HAN2 DRH"}="2HAN DRH";
$resn_exch{"HAA1 DRH"}="1HAA DRH"; $resn_exch{"HAA2 DRH"}="2HAA DRH"; $resn_exch{"HAA3 DRH"}="3HAA DRH";
$resn_exch{" H21 DRI"}="1H2  DRI"; $resn_exch{" H22 DRI"}="2H2  DRI"; $resn_exch{" H41 DRI"}="1H4  DRI";
$resn_exch{" H42 DRI"}="2H4  DRI"; $resn_exch{" H43 DRI"}="3H4  DRI"; $resn_exch{" H61 DRI"}="1H6  DRI";
$resn_exch{" H62 DRI"}="2H6  DRI"; $resn_exch{" H63 DRI"}="3H6  DRI"; $resn_exch{"HAB1 DRJ"}="1HAB DRJ";
$resn_exch{"HAB2 DRJ"}="2HAB DRJ"; $resn_exch{"HAB3 DRJ"}="3HAB DRJ"; $resn_exch{"HAO1 DRJ"}="1HAO DRJ";
$resn_exch{"HAO2 DRJ"}="2HAO DRJ"; $resn_exch{"HAC1 DRJ"}="1HAC DRJ"; $resn_exch{"HAC2 DRJ"}="2HAC DRJ";
$resn_exch{"HAC3 DRJ"}="3HAC DRJ"; $resn_exch{"HAT1 DRJ"}="1HAT DRJ"; $resn_exch{"HAT2 DRJ"}="2HAT DRJ";
$resn_exch{"HAV1 DRJ"}="1HAV DRJ"; $resn_exch{"HAV2 DRJ"}="2HAV DRJ"; $resn_exch{"HAU1 DRJ"}="1HAU DRJ";
$resn_exch{"HAU2 DRJ"}="2HAU DRJ"; $resn_exch{"HAS1 DRJ"}="1HAS DRJ"; $resn_exch{"HAS2 DRJ"}="2HAS DRJ";
$resn_exch{"HAR1 DRJ"}="1HAR DRJ"; $resn_exch{"HAR2 DRJ"}="2HAR DRJ"; $resn_exch{"HAQ1 DRJ"}="1HAQ DRJ";
$resn_exch{"HAQ2 DRJ"}="2HAQ DRJ"; $resn_exch{"HAP1 DRJ"}="1HAP DRJ"; $resn_exch{"HAP2 DRJ"}="2HAP DRJ";
$resn_exch{"HAN1 DRJ"}="1HAN DRJ"; $resn_exch{"HAN2 DRJ"}="2HAN DRJ"; $resn_exch{"HAA1 DRJ"}="1HAA DRJ";
$resn_exch{"HAA2 DRJ"}="2HAA DRJ"; $resn_exch{"HAA3 DRJ"}="3HAA DRJ"; $resn_exch{"HOP2 DRM"}="2HOP DRM";
$resn_exch{"HOP3 DRM"}="3HOP DRM"; $resn_exch{"H6'1 DRM"}="1H6* DRM"; $resn_exch{"H6'2 DRM"}="2H6* DRM";
$resn_exch{" H5' DRM"}=" H5* DRM"; $resn_exch{"H4'1 DRM"}="1H4* DRM"; $resn_exch{"H4'2 DRM"}="2H4* DRM";
$resn_exch{"H3'1 DRM"}="1H3* DRM"; $resn_exch{"H3'2 DRM"}="2H3* DRM"; $resn_exch{" H2' DRM"}="1H2* DRM";
$resn_exch{"H2'' DRM"}="2H2* DRM"; $resn_exch{" H1' DRM"}=" H1* DRM"; $resn_exch{"HAB1 DRN"}="1HAB DRN";
$resn_exch{"HAB2 DRN"}="2HAB DRN"; $resn_exch{"HBF1 DRN"}="1HBF DRN"; $resn_exch{"HBF2 DRN"}="2HBF DRN";
$resn_exch{"HBE1 DRN"}="1HBE DRN"; $resn_exch{"HBE2 DRN"}="2HBE DRN"; $resn_exch{"HA01 DRN"}="1HA0 DRN";
$resn_exch{"HA02 DRN"}="2HA0 DRN"; $resn_exch{"HAY1 DRN"}="1HAY DRN"; $resn_exch{"HAY2 DRN"}="2HAY DRN";
$resn_exch{"HAY3 DRN"}="3HAY DRN"; $resn_exch{"HOP2 DRP"}="2HOP DRP"; $resn_exch{"H2'1 DRP"}="1H2* DRP";
$resn_exch{"H2'2 DRP"}="2H2* DRP"; $resn_exch{"H5'1 DRP"}="1H5* DRP"; $resn_exch{"H5'2 DRP"}="2H5* DRP";
$resn_exch{" H4' DRP"}=" H4* DRP"; $resn_exch{" H1' DRP"}=" H1* DRP"; $resn_exch{" H3' DRP"}=" H3* DRP";
$resn_exch{"HOP3 DRP"}="3HOP DRP"; $resn_exch{"HAX1 DRQ"}="1HAX DRQ"; $resn_exch{"HAX2 DRQ"}="2HAX DRQ";
$resn_exch{"HAX3 DRQ"}="3HAX DRQ"; $resn_exch{"HAW1 DRQ"}="1HAW DRQ"; $resn_exch{"HAW2 DRQ"}="2HAW DRQ";
$resn_exch{"HAR1 DRQ"}="1HAR DRQ"; $resn_exch{"HAR2 DRQ"}="2HAR DRQ"; $resn_exch{"HAQ1 DRQ"}="1HAQ DRQ";
$resn_exch{"HAQ2 DRQ"}="2HAQ DRQ"; $resn_exch{"HAP1 DRQ"}="1HAP DRQ"; $resn_exch{"HAP2 DRQ"}="2HAP DRQ";
$resn_exch{"HAP3 DRQ"}="3HAP DRQ"; $resn_exch{"HAL1 DRQ"}="1HAL DRQ"; $resn_exch{"HAL2 DRQ"}="2HAL DRQ";
$resn_exch{"HAK1 DRQ"}="1HAK DRQ"; $resn_exch{"HAK2 DRQ"}="2HAK DRQ"; $resn_exch{"HAH1 DRQ"}="1HAH DRQ";
$resn_exch{"HAH2 DRQ"}="2HAH DRQ"; $resn_exch{"HAG1 DRQ"}="1HAG DRQ"; $resn_exch{"HAG2 DRQ"}="2HAG DRQ";
$resn_exch{"HOP3 DRT"}="3HOP DRT"; $resn_exch{"HOP2 DRT"}="2HOP DRT"; $resn_exch{" H5' DRT"}="1H5* DRT";
$resn_exch{"H5'' DRT"}="2H5* DRT"; $resn_exch{" H4' DRT"}=" H4* DRT"; $resn_exch{" H3' DRT"}=" H3* DRT";
$resn_exch{"HO3' DRT"}="H3T  DRT"; $resn_exch{" H2' DRT"}="1H2* DRT"; $resn_exch{"H2'' DRT"}="2H2* DRT";
$resn_exch{" H1' DRT"}=" H1* DRT"; $resn_exch{" H71 DRT"}="1H5M DRT"; $resn_exch{" H72 DRT"}="2H5M DRT";
$resn_exch{" H73 DRT"}="3H5M DRT"; $resn_exch{" H31 DRU"}="1H3  DRU"; $resn_exch{" H32 DRU"}="2H3  DRU";
$resn_exch{" H41 DRU"}="1H4  DRU"; $resn_exch{" H42 DRU"}="2H4  DRU"; $resn_exch{" H51 DRU"}="1H5  DRU";
$resn_exch{" H61 DRU"}="1H6  DRU"; $resn_exch{" H77 DRU"}="7H7  DRU"; $resn_exch{" H88 DRU"}="8H8  DRU";
$resn_exch{" H91 DRU"}="1H9  DRU"; $resn_exch{" H92 DRU"}="2H9  DRU"; $resn_exch{"H101 DRU"}="1H10 DRU";
$resn_exch{"H102 DRU"}="2H10 DRU"; $resn_exch{"H111 DRU"}="1H11 DRU"; $resn_exch{"H112 DRU"}="2H11 DRU";
$resn_exch{"H121 DRU"}="1H12 DRU"; $resn_exch{"H122 DRU"}="2H12 DRU"; $resn_exch{"H271 DRU"}="1H27 DRU";
$resn_exch{"H272 DRU"}="2H27 DRU"; $resn_exch{"H281 DRU"}="1H28 DRU"; $resn_exch{"H282 DRU"}="2H28 DRU";
$resn_exch{"H291 DRU"}="1H29 DRU"; $resn_exch{"H292 DRU"}="2H29 DRU"; $resn_exch{"H301 DRU"}="1H30 DRU";
$resn_exch{"H302 DRU"}="2H30 DRU"; $resn_exch{"H331 DRU"}="31H3 DRU"; $resn_exch{"H332 DRU"}="32H3 DRU";
$resn_exch{"H33A DRU"}="1H33 DRU"; $resn_exch{"H33B DRU"}="2H33 DRU"; $resn_exch{"H341 DRU"}="1H34 DRU";
$resn_exch{"H342 DRU"}="2H34 DRU"; $resn_exch{"H351 DRU"}="1H35 DRU"; $resn_exch{"H352 DRU"}="2H35 DRU";
$resn_exch{"H361 DRU"}="1H36 DRU"; $resn_exch{"H362 DRU"}="2H36 DRU"; $resn_exch{" HG1 DRU"}="1HG  DRU";
$resn_exch{" HG2 DRU"}="2HG  DRU"; $resn_exch{"HD21 DRU"}="1HD2 DRU"; $resn_exch{"HD22 DRU"}="2HD2 DRU";
$resn_exch{"HE11 DRU"}="1HE1 DRU"; $resn_exch{"HE12 DRU"}="2HE1 DRU"; $resn_exch{"HE21 DRU"}="1HE2 DRU";
$resn_exch{"HBJ1 DRY"}="1HBJ DRY"; $resn_exch{"HBJ2 DRY"}="2HBJ DRY"; $resn_exch{"HAA1 DRY"}="1HAA DRY";
$resn_exch{"HAA2 DRY"}="2HAA DRY"; $resn_exch{"HAB1 DRY"}="1HAB DRY"; $resn_exch{"HAB2 DRY"}="2HAB DRY";
$resn_exch{"HAC1 DRY"}="1HAC DRY"; $resn_exch{"HAC2 DRY"}="2HAC DRY"; $resn_exch{"HBE1 DRY"}="1HBE DRY";
$resn_exch{"HBE2 DRY"}="2HBE DRY"; $resn_exch{"HBF1 DRY"}="1HBF DRY"; $resn_exch{"HBF2 DRY"}="2HBF DRY";
$resn_exch{"HBG1 DRY"}="1HBG DRY"; $resn_exch{"HBG2 DRY"}="2HBG DRY"; $resn_exch{"HBG3 DRY"}="3HBG DRY";
$resn_exch{"HOP2 DRZ"}="2HOP DRZ"; $resn_exch{"HOP3 DRZ"}="3HOP DRZ"; $resn_exch{" H5' DRZ"}="1H5* DRZ";
$resn_exch{"H5'' DRZ"}="2H5* DRZ"; $resn_exch{" H4' DRZ"}=" H4* DRZ"; $resn_exch{"H4'O DRZ"}="OH4* DRZ";
$resn_exch{" H3' DRZ"}=" H3* DRZ"; $resn_exch{" H2' DRZ"}="1H2* DRZ"; $resn_exch{"H2'' DRZ"}="2H2* DRZ";
$resn_exch{" H1' DRZ"}=" H1* DRZ"; $resn_exch{"HO3' DRZ"}="H3T  DRZ"; $resn_exch{"H111 DSA"}="1H11 DSA";
$resn_exch{"H112 DSA"}="2H11 DSA"; $resn_exch{"H131 DSA"}="1H13 DSA"; $resn_exch{"H132 DSA"}="2H13 DSA";
$resn_exch{"H133 DSA"}="3H13 DSA"; $resn_exch{"H261 DSA"}="1H26 DSA"; $resn_exch{"H262 DSA"}="2H26 DSA";
$resn_exch{"H263 DSA"}="3H26 DSA"; $resn_exch{"H271 DSA"}="1H27 DSA"; $resn_exch{"H272 DSA"}="2H27 DSA";
$resn_exch{"H273 DSA"}="3H27 DSA"; $resn_exch{"H281 DSA"}="1H28 DSA"; $resn_exch{"H282 DSA"}="2H28 DSA";
$resn_exch{"H283 DSA"}="3H28 DSA"; $resn_exch{"H151 DSA"}="1H15 DSA"; $resn_exch{"H152 DSA"}="2H15 DSA";
$resn_exch{"H153 DSA"}="3H15 DSA"; $resn_exch{" H21 DSB"}="1H2  DSB"; $resn_exch{" H22 DSB"}="2H2  DSB";
$resn_exch{" H31 DSB"}="1H3  DSB"; $resn_exch{" H32 DSB"}="2H3  DSB"; $resn_exch{" H51 DSB"}="1H5  DSB";
$resn_exch{" H52 DSB"}="2H5  DSB"; $resn_exch{" H11 DSB"}="1H1  DSB"; $resn_exch{" H61 DSB"}="1H6  DSB";
$resn_exch{" H62 DSB"}="2H6  DSB"; $resn_exch{" H81 DSB"}="1H8  DSB"; $resn_exch{" H82 DSB"}="2H8  DSB";
$resn_exch{" H91 DSB"}="1H9  DSB"; $resn_exch{" H92 DSB"}="2H9  DSB"; $resn_exch{"H111 DSB"}="1H11 DSB";
$resn_exch{" H11 DSC"}="1H1  DSC"; $resn_exch{" H12 DSC"}="2H1  DSC"; $resn_exch{" H21 DSC"}="1H2  DSC";
$resn_exch{" H22 DSC"}="2H2  DSC"; $resn_exch{" H31 DSC"}="1H3  DSC"; $resn_exch{" H32 DSC"}="2H3  DSC";
$resn_exch{" H41 DSC"}="1H4  DSC"; $resn_exch{" H42 DSC"}="2H4  DSC"; $resn_exch{" H51 DSC"}="1H5  DSC";
$resn_exch{" H52 DSC"}="2H5  DSC"; $resn_exch{" H61 DSC"}="1H6  DSC"; $resn_exch{" H62 DSC"}="2H6  DSC";
$resn_exch{" H71 DSC"}="1H7  DSC"; $resn_exch{" H72 DSC"}="2H7  DSC"; $resn_exch{" H81 DSC"}="1H8  DSC";
$resn_exch{" H82 DSC"}="2H8  DSC"; $resn_exch{" H91 DSC"}="1H9  DSC"; $resn_exch{" H92 DSC"}="2H9  DSC";
$resn_exch{"H101 DSC"}="1H10 DSC"; $resn_exch{"H102 DSC"}="2H10 DSC"; $resn_exch{"H111 DSC"}="1H11 DSC";
$resn_exch{"H112 DSC"}="2H11 DSC"; $resn_exch{"H121 DSC"}="1H12 DSC"; $resn_exch{"H122 DSC"}="2H12 DSC";
$resn_exch{"H123 DSC"}="3H12 DSC"; $resn_exch{" H21 DSD"}="1H2  DSD"; $resn_exch{" H22 DSD"}="2H2  DSD";
$resn_exch{" H31 DSD"}="1H3  DSD"; $resn_exch{" H32 DSD"}="2H3  DSD"; $resn_exch{" H41 DSD"}="1H4  DSD";
$resn_exch{" H42 DSD"}="2H4  DSD"; $resn_exch{" H51 DSD"}="1H5  DSD"; $resn_exch{" H52 DSD"}="2H5  DSD";
$resn_exch{" H61 DSD"}="1H6  DSD"; $resn_exch{" H62 DSD"}="2H6  DSD"; $resn_exch{" H91 DSD"}="1H9  DSD";
$resn_exch{" H92 DSD"}="2H9  DSD"; $resn_exch{" H93 DSD"}="3H9  DSD"; $resn_exch{"HN81 DSD"}="1HN8 DSD";
$resn_exch{"HN82 DSD"}="2HN8 DSD"; $resn_exch{" H   DSE"}=" HN  DSE"; $resn_exch{" HB2 DSE"}="1HB  DSE";
$resn_exch{" HB3 DSE"}="2HB  DSE"; $resn_exch{" HN1 DSE"}="1HN  DSE"; $resn_exch{" HN2 DSE"}="2HN  DSE";
$resn_exch{" HN3 DSE"}="3HN  DSE"; $resn_exch{" HG  DSE"}=" HOG DSE"; $resn_exch{" H2  DSG"}=" HN2 DSG";
$resn_exch{" HB2 DSG"}="1HB  DSG"; $resn_exch{" HB3 DSG"}="2HB  DSG"; $resn_exch{"HD21 DSG"}="1HD2 DSG";
$resn_exch{"HD22 DSG"}="2HD2 DSG"; $resn_exch{"H111 DSI"}="1H11 DSI"; $resn_exch{"H112 DSI"}="2H11 DSI";
$resn_exch{"H131 DSI"}="1H13 DSI"; $resn_exch{"H132 DSI"}="2H13 DSI"; $resn_exch{"H133 DSI"}="3H13 DSI";
$resn_exch{"H151 DSI"}="1H15 DSI"; $resn_exch{"H152 DSI"}="2H15 DSI"; $resn_exch{"H153 DSI"}="3H15 DSI";
$resn_exch{"H181 DSM"}="1H18 DSM"; $resn_exch{"H182 DSM"}="2H18 DSM"; $resn_exch{"H183 DSM"}="3H18 DSM";
$resn_exch{"H171 DSM"}="1H17 DSM"; $resn_exch{"H172 DSM"}="2H17 DSM"; $resn_exch{"H161 DSM"}="1H16 DSM";
$resn_exch{"H162 DSM"}="2H16 DSM"; $resn_exch{"H151 DSM"}="1H15 DSM"; $resn_exch{"H152 DSM"}="2H15 DSM";
$resn_exch{" H51 DSM"}="1H5  DSM"; $resn_exch{" H52 DSM"}="2H5  DSM"; $resn_exch{" H61 DSM"}="1H6  DSM";
$resn_exch{" H62 DSM"}="2H6  DSM"; $resn_exch{" H2  DSN"}=" HN2 DSN"; $resn_exch{" HB2 DSN"}="1HB  DSN";
$resn_exch{" HB3 DSN"}="2HB  DSN"; $resn_exch{" HG  DSN"}=" HOG DSN"; $resn_exch{"H1C1 DSO"}="1H1C DSO";
$resn_exch{"H1C2 DSO"}="2H1C DSO"; $resn_exch{"H3C1 DSO"}="1H3C DSO"; $resn_exch{"H3C2 DSO"}="2H3C DSO";
$resn_exch{"H4C1 DSO"}="1H4C DSO"; $resn_exch{"H4C2 DSO"}="2H4C DSO"; $resn_exch{"H6C1 DSO"}="1H6C DSO";
$resn_exch{"H6C2 DSO"}="2H6C DSO"; $resn_exch{"H7C1 DSO"}="1H7C DSO"; $resn_exch{"H7C2 DSO"}="2H7C DSO";
$resn_exch{"H101 DSO"}="1H10 DSO"; $resn_exch{"H102 DSO"}="2H10 DSO"; $resn_exch{"H221 DSO"}="1H22 DSO";
$resn_exch{"H222 DSO"}="2H22 DSO"; $resn_exch{"H223 DSO"}="3H22 DSO"; $resn_exch{"H231 DSO"}="1H23 DSO";
$resn_exch{"H232 DSO"}="2H23 DSO"; $resn_exch{"H233 DSO"}="3H23 DSO"; $resn_exch{"H261 DSO"}="1H26 DSO";
$resn_exch{"H262 DSO"}="2H26 DSO"; $resn_exch{"H271 DSO"}="1H27 DSO"; $resn_exch{"H272 DSO"}="2H27 DSO";
$resn_exch{"H281 DSO"}="1H28 DSO"; $resn_exch{"H282 DSO"}="2H28 DSO"; $resn_exch{"H291 DSO"}="1H29 DSO";
$resn_exch{"H292 DSO"}="2H29 DSO"; $resn_exch{"H301 DSO"}="1H30 DSO"; $resn_exch{"H302 DSO"}="2H30 DSO";
$resn_exch{"H311 DSO"}="1H31 DSO"; $resn_exch{"H312 DSO"}="2H31 DSO"; $resn_exch{"H321 DSO"}="1H32 DSO";
$resn_exch{"H322 DSO"}="2H32 DSO"; $resn_exch{"H331 DSO"}="1H33 DSO"; $resn_exch{"H332 DSO"}="2H33 DSO";
$resn_exch{" HB1 DSP"}="1HB  DSP"; $resn_exch{" HB2 DSP"}="2HB  DSP"; $resn_exch{" H21 DSR"}="1H2  DSR";
$resn_exch{" H22 DSR"}="2H2  DSR"; $resn_exch{" H61 DSR"}="1H6  DSR"; $resn_exch{" H62 DSR"}="2H6  DSR";
$resn_exch{" H63 DSR"}="3H6  DSR"; $resn_exch{" H11 DSS"}="1H1  DSS"; $resn_exch{" H12 DSS"}="2H1  DSS";
$resn_exch{" H13 DSS"}="3H1  DSS"; $resn_exch{" H21 DSS"}="1H2  DSS"; $resn_exch{" H22 DSS"}="2H2  DSS";
$resn_exch{" H41 DSS"}="1H4  DSS"; $resn_exch{" H42 DSS"}="2H4  DSS"; $resn_exch{" H43 DSS"}="3H4  DSS";
$resn_exch{"H101 DST"}="1H10 DST"; $resn_exch{"H102 DST"}="2H10 DST"; $resn_exch{"H111 DST"}="1H11 DST";
$resn_exch{"H112 DST"}="2H11 DST"; $resn_exch{"H131 DST"}="1H13 DST"; $resn_exch{"H132 DST"}="2H13 DST";
$resn_exch{"H133 DST"}="3H13 DST"; $resn_exch{"H141 DST"}="1H14 DST"; $resn_exch{"H142 DST"}="2H14 DST";
$resn_exch{"H143 DST"}="3H14 DST"; $resn_exch{"H2N1 DSU"}="1H2N DSU"; $resn_exch{"H2N2 DSU"}="2H2N DSU";
$resn_exch{"H3N1 DSU"}="1H3N DSU"; $resn_exch{"H3N2 DSU"}="2H3N DSU"; $resn_exch{"H4N1 DSU"}="1H4N DSU";
$resn_exch{"H4N2 DSU"}="2H4N DSU"; $resn_exch{"H5N1 DSU"}="1H5N DSU"; $resn_exch{"H5N2 DSU"}="2H5N DSU";
$resn_exch{"H6N1 DSU"}="1H6N DSU"; $resn_exch{"H6N2 DSU"}="2H6N DSU"; $resn_exch{"H7N1 DSU"}="1H7N DSU";
$resn_exch{"H7N2 DSU"}="2H7N DSU"; $resn_exch{"H8N1 DSU"}="1H8N DSU"; $resn_exch{"H8N2 DSU"}="2H8N DSU";
$resn_exch{"H9N1 DSU"}="1H9N DSU"; $resn_exch{"H9N2 DSU"}="2H9N DSU"; $resn_exch{"H101 DSU"}="1H10 DSU";
$resn_exch{"H102 DSU"}="2H10 DSU"; $resn_exch{"H103 DSU"}="3H10 DSU"; $resn_exch{"H6'1 DSU"}="1H6* DSU";
$resn_exch{"H6'2 DSU"}="2H6* DSU"; $resn_exch{" H5' DSU"}=" H5* DSU"; $resn_exch{" H4' DSU"}=" H4* DSU";
$resn_exch{" H3' DSU"}=" H3* DSU"; $resn_exch{"H1'1 DSU"}="1H1* DSU"; $resn_exch{"H1'2 DSU"}="2H1* DSU";
$resn_exch{" H1' DSU"}=" H1* DSU"; $resn_exch{" H61 DSU"}="1H6  DSU"; $resn_exch{" H62 DSU"}="2H6  DSU";
$resn_exch{" H63 DSU"}="3H6  DSU"; $resn_exch{"H101 DSX"}="1H10 DSX"; $resn_exch{"H102 DSX"}="2H10 DSX";
$resn_exch{"H171 DSX"}="1H17 DSX"; $resn_exch{"H172 DSX"}="2H17 DSX"; $resn_exch{"H173 DSX"}="3H17 DSX";
$resn_exch{"H221 DSX"}="1H22 DSX"; $resn_exch{"H222 DSX"}="2H22 DSX"; $resn_exch{"H241 DSX"}="1H24 DSX";
$resn_exch{"H242 DSX"}="2H24 DSX"; $resn_exch{"H243 DSX"}="3H24 DSX"; $resn_exch{"H251 DSX"}="1H25 DSX";
$resn_exch{"H252 DSX"}="2H25 DSX"; $resn_exch{"H253 DSX"}="3H25 DSX"; $resn_exch{"H281 DSX"}="1H28 DSX";
$resn_exch{"H282 DSX"}="2H28 DSX"; $resn_exch{"HM11 DSY"}="1HM1 DSY"; $resn_exch{"HM12 DSY"}="2HM1 DSY";
$resn_exch{"HM13 DSY"}="3HM1 DSY"; $resn_exch{"HM21 DSY"}="1HM2 DSY"; $resn_exch{"HM22 DSY"}="2HM2 DSY";
$resn_exch{"HM23 DSY"}="3HM2 DSY"; $resn_exch{"HOS2 DSY"}="2HOS DSY"; $resn_exch{"HOP3  DT"}="3HOP  DT";
$resn_exch{"HOP2  DT"}="2HOP  DT"; $resn_exch{" H5'  DT"}="1H5*  DT"; $resn_exch{"H5''  DT"}="2H5*  DT";
$resn_exch{" H4'  DT"}=" H4*  DT"; $resn_exch{" H3'  DT"}=" H3*  DT"; $resn_exch{"HO3'  DT"}="H3T   DT";
$resn_exch{" H2'  DT"}="1H2*  DT"; $resn_exch{"H2''  DT"}="2H2*  DT"; $resn_exch{" H1'  DT"}=" H1*  DT";
$resn_exch{" H71  DT"}="1H5M  DT"; $resn_exch{" H72  DT"}="2H5M  DT"; $resn_exch{" H73  DT"}="3H5M  DT";
$resn_exch{"H6N1 DT1"}="1H6N DT1"; $resn_exch{"H6N2 DT1"}="2H6N DT1"; $resn_exch{"H201 DT1"}="1H20 DT1";
$resn_exch{"H202 DT1"}="2H20 DT1"; $resn_exch{"H2C1 DT1"}="1H2C DT1"; $resn_exch{"H2C2 DT1"}="2H2C DT1";
$resn_exch{"H3C1 DT1"}="1H3C DT1"; $resn_exch{"H3C2 DT1"}="2H3C DT1"; $resn_exch{"H4C1 DT1"}="1H4C DT1";
$resn_exch{"H4C2 DT1"}="2H4C DT1"; $resn_exch{"H5C1 DT1"}="1H5C DT1"; $resn_exch{"H5C2 DT1"}="2H5C DT1";
$resn_exch{"H6C1 DT1"}="1H6C DT1"; $resn_exch{"H6C2 DT1"}="2H6C DT1"; $resn_exch{"H6N1 DT2"}="1H6N DT2";
$resn_exch{"H6N2 DT2"}="2H6N DT2"; $resn_exch{"H2C1 DT2"}="1H2C DT2"; $resn_exch{"H2C2 DT2"}="2H2C DT2";
$resn_exch{"H3C1 DT2"}="1H3C DT2"; $resn_exch{"H3C2 DT2"}="2H3C DT2"; $resn_exch{"H4C1 DT2"}="1H4C DT2";
$resn_exch{"H4C2 DT2"}="2H4C DT2"; $resn_exch{"H5C1 DT2"}="1H5C DT2"; $resn_exch{"H5C2 DT2"}="2H5C DT2";
$resn_exch{"H6C1 DT2"}="1H6C DT2"; $resn_exch{"H6C2 DT2"}="2H6C DT2"; $resn_exch{" H41 DT3"}="1H4  DT3";
$resn_exch{" H42 DT3"}="2H4  DT3"; $resn_exch{" H11 DT3"}="1H1  DT3"; $resn_exch{" H12 DT3"}="2H1  DT3";
$resn_exch{"H6N1 DT4"}="1H6N DT4"; $resn_exch{"H6N2 DT4"}="2H6N DT4"; $resn_exch{"H2C1 DT4"}="1H2C DT4";
$resn_exch{"H2C2 DT4"}="2H2C DT4"; $resn_exch{"H3C1 DT4"}="1H3C DT4"; $resn_exch{"H3C2 DT4"}="2H3C DT4";
$resn_exch{"H8N1 DT4"}="1H8N DT4"; $resn_exch{"H8N2 DT4"}="2H8N DT4"; $resn_exch{"H5C1 DT4"}="1H5C DT4";
$resn_exch{"H5C2 DT4"}="2H5C DT4"; $resn_exch{"H6C1 DT4"}="1H6C DT4"; $resn_exch{"H6C2 DT4"}="2H6C DT4";
$resn_exch{"H2C1 DT5"}="1H2C DT5"; $resn_exch{"H2C2 DT5"}="2H2C DT5"; $resn_exch{"H3C1 DT5"}="1H3C DT5";
$resn_exch{"H3C2 DT5"}="2H3C DT5"; $resn_exch{"H4C1 DT5"}="1H4C DT5"; $resn_exch{"H4C2 DT5"}="2H4C DT5";
$resn_exch{"H5C1 DT5"}="1H5C DT5"; $resn_exch{"H5C2 DT5"}="2H5C DT5"; $resn_exch{"H6C1 DT5"}="1H6C DT5";
$resn_exch{"H6C2 DT5"}="2H6C DT5"; $resn_exch{" H11 DT6"}="1H1  DT6"; $resn_exch{" H12 DT6"}="2H1  DT6";
$resn_exch{" H81 DT6"}="1H8  DT6"; $resn_exch{" H82 DT6"}="2H8  DT6"; $resn_exch{" H83 DT6"}="3H8  DT6";
$resn_exch{"H101 DT6"}="1H10 DT6"; $resn_exch{"H102 DT6"}="2H10 DT6"; $resn_exch{"H103 DT6"}="3H10 DT6";
$resn_exch{" H61 DT6"}="1H6  DT6"; $resn_exch{" H62 DT6"}="2H6  DT6"; $resn_exch{"H371 DT8"}="1H37 DT8";
$resn_exch{"H372 DT8"}="2H37 DT8"; $resn_exch{"H373 DT8"}="3H37 DT8"; $resn_exch{"H361 DT8"}="1H36 DT8";
$resn_exch{"H362 DT8"}="2H36 DT8"; $resn_exch{"H363 DT8"}="3H36 DT8"; $resn_exch{"H181 DT8"}="1H18 DT8";
$resn_exch{"H182 DT8"}="2H18 DT8"; $resn_exch{"H191 DT8"}="1H19 DT8"; $resn_exch{"H192 DT8"}="2H19 DT8";
$resn_exch{"H161 DT8"}="1H16 DT8"; $resn_exch{"H162 DT8"}="2H16 DT8"; $resn_exch{"H281 DT8"}="1H28 DT8";
$resn_exch{"H282 DT8"}="2H28 DT8"; $resn_exch{"H331 DT8"}="1H33 DT8"; $resn_exch{"H332 DT8"}="2H33 DT8";
$resn_exch{"H333 DT8"}="3H33 DT8"; $resn_exch{"H301 DT8"}="1H30 DT8"; $resn_exch{"H302 DT8"}="2H30 DT8";
$resn_exch{"H311 DT8"}="1H31 DT8"; $resn_exch{"H312 DT8"}="2H31 DT8"; $resn_exch{"H621 DTA"}="1H62 DTA";
$resn_exch{"H622 DTA"}="2H62 DTA"; $resn_exch{" H1' DTA"}=" H1* DTA"; $resn_exch{" H2' DTA"}=" H2* DTA";
$resn_exch{"HO2' DTA"}="HO2* DTA"; $resn_exch{" H3' DTA"}=" H3* DTA"; $resn_exch{"HO3' DTA"}="HO3* DTA";
$resn_exch{" H4' DTA"}=" H4* DTA"; $resn_exch{"H5'1 DTA"}="1H5* DTA"; $resn_exch{"H5'2 DTA"}="2H5* DTA";
$resn_exch{"H5R1 DTA"}="1H5R DTA"; $resn_exch{"H5R2 DTA"}="2H5R DTA"; $resn_exch{"H611 DTA"}="1H61 DTA";
$resn_exch{"H612 DTA"}="2H61 DTA"; $resn_exch{"HCT1 DTB"}="1HCT DTB"; $resn_exch{"HCT2 DTB"}="2HCT DTB";
$resn_exch{"HCT3 DTB"}="3HCT DTB"; $resn_exch{"HCE1 DTB"}="1HCE DTB"; $resn_exch{"HCE2 DTB"}="2HCE DTB";
$resn_exch{"HCD1 DTB"}="1HCD DTB"; $resn_exch{"HCD2 DTB"}="2HCD DTB"; $resn_exch{"HCG1 DTB"}="1HCG DTB";
$resn_exch{"HCG2 DTB"}="2HCG DTB"; $resn_exch{"HCB1 DTB"}="1HCB DTB"; $resn_exch{"HCB2 DTB"}="2HCB DTB";
$resn_exch{"HCA1 DTB"}="1HCA DTB"; $resn_exch{"HCA2 DTB"}="2HCA DTB"; $resn_exch{"H151 DTC"}="1H15 DTC";
$resn_exch{"H152 DTC"}="2H15 DTC"; $resn_exch{" HN1 DTE"}="1HN  DTE"; $resn_exch{" HN2 DTE"}="2HN  DTE";
$resn_exch{"HBC1 DTE"}="1HBC DTE"; $resn_exch{"HBC2 DTE"}="2HBC DTE"; $resn_exch{"HAU1 DTF"}="1HAU DTF";
$resn_exch{"HAL1 DTF"}="1HAL DTF"; $resn_exch{"HAK1 DTF"}="1HAK DTF"; $resn_exch{"HAS1 DTF"}="1HAS DTF";
$resn_exch{"HAT1 DTF"}="1HAT DTF"; $resn_exch{"HAM1 DTF"}="1HAM DTF"; $resn_exch{"HAP1 DTF"}="1HAP DTF";
$resn_exch{"HBE1 DTF"}="1HBE DTF"; $resn_exch{"HBE2 DTF"}="2HBE DTF"; $resn_exch{"HBT1 DTF"}="1HBT DTF";
$resn_exch{"HAY1 DTF"}="1HAY DTF"; $resn_exch{"HAY2 DTF"}="2HAY DTF"; $resn_exch{"HAI1 DTF"}="1HAI DTF";
$resn_exch{"HAJ1 DTF"}="1HAJ DTF"; $resn_exch{"HBB1 DTF"}="1HBB DTF"; $resn_exch{"HBB2 DTF"}="2HBB DTF";
$resn_exch{" HA1 DTF"}="1HA  DTF"; $resn_exch{" HB1 DTF"}="1HB  DTF"; $resn_exch{" HB2 DTF"}="2HB  DTF";
$resn_exch{"HAZ1 DTF"}="1HAZ DTF"; $resn_exch{"HAZ2 DTF"}="2HAZ DTF"; $resn_exch{"HAW1 DTF"}="1HAW DTF";
$resn_exch{"HAW2 DTF"}="2HAW DTF"; $resn_exch{"HAV1 DTF"}="1HAV DTF"; $resn_exch{"HAV2 DTF"}="2HAV DTF";
$resn_exch{"HAX1 DTF"}="1HAX DTF"; $resn_exch{"HAX2 DTF"}="2HAX DTF"; $resn_exch{"HBA1 DTF"}="1HBA DTF";
$resn_exch{"HBA2 DTF"}="2HBA DTF"; $resn_exch{"HBC1 DTF"}="1HBC DTF"; $resn_exch{"HBC2 DTF"}="2HBC DTF";
$resn_exch{"HBS1 DTF"}="1HBS DTF"; $resn_exch{"HAR1 DTF"}="1HAR DTF"; $resn_exch{"HAO1 DTF"}="1HAO DTF";
$resn_exch{"HAQ1 DTF"}="1HAQ DTF"; $resn_exch{"HAN1 DTF"}="1HAN DTF"; $resn_exch{"HBF1 DTF"}="1HBF DTF";
$resn_exch{"HBF2 DTF"}="2HBF DTF"; $resn_exch{" HA1 DTG"}="1HA  DTG"; $resn_exch{" HA2 DTG"}="2HA  DTG";
$resn_exch{" H11 DTG"}="1H1  DTG"; $resn_exch{" H12 DTG"}="2H1  DTG"; $resn_exch{" H13 DTG"}="3H1  DTG";
$resn_exch{" H21 DTG"}="1H2  DTG"; $resn_exch{" H22 DTG"}="2H2  DTG"; $resn_exch{" H23 DTG"}="3H2  DTG";
$resn_exch{" H31 DTG"}="1H3  DTG"; $resn_exch{" H32 DTG"}="2H3  DTG"; $resn_exch{" H33 DTG"}="3H3  DTG";
$resn_exch{"HG21 DTH"}="1HG2 DTH"; $resn_exch{"HG22 DTH"}="2HG2 DTH"; $resn_exch{"HG23 DTH"}="3HG2 DTH";
$resn_exch{" HG1 DTH"}=" HOG DTH"; $resn_exch{"H11A DTI"}="1H1  DTI"; $resn_exch{" H12 DTI"}="2H1  DTI";
$resn_exch{" H13 DTI"}="3H1  DTI"; $resn_exch{" H51 DTI"}="1H5  DTI"; $resn_exch{" H52 DTI"}="2H5  DTI";
$resn_exch{" H53 DTI"}="3H5  DTI"; $resn_exch{"H1C1 DTL"}="1H1C DTL"; $resn_exch{"H1C2 DTL"}="2H1C DTL";
$resn_exch{"H4C1 DTL"}="1H4C DTL"; $resn_exch{"H4C2 DTL"}="2H4C DTL"; $resn_exch{" H5' DTM"}=" H5* DTM";
$resn_exch{" H7' DTM"}=" H7* DTM"; $resn_exch{"HC91 DTM"}="1HC9 DTM"; $resn_exch{"HC92 DTM"}="2HC9 DTM";
$resn_exch{"H311 DTM"}="1H31 DTM"; $resn_exch{"H312 DTM"}="2H31 DTM"; $resn_exch{"H313 DTM"}="3H31 DTM";
$resn_exch{"H411 DTM"}="1H41 DTM"; $resn_exch{"H412 DTM"}="2H41 DTM"; $resn_exch{"H413 DTM"}="3H41 DTM";
$resn_exch{"H511 DTM"}="1H51 DTM"; $resn_exch{"H512 DTM"}="2H51 DTM"; $resn_exch{"H513 DTM"}="3H51 DTM";
$resn_exch{"HCN1 DTM"}="1HCN DTM"; $resn_exch{"HCN2 DTM"}="2HCN DTM"; $resn_exch{"HCN3 DTM"}="3HCN DTM";
$resn_exch{"HN21 DTM"}="1HN2 DTM"; $resn_exch{"HN22 DTM"}="2HN2 DTM"; $resn_exch{"HN41 DTM"}="1HN4 DTM";
$resn_exch{"HN42 DTM"}="2HN4 DTM"; $resn_exch{" H12 DTO"}="2H1  DTO"; $resn_exch{" H11 DTO"}="1H1  DTO";
$resn_exch{"H2O1 DTO"}="1H2O DTO"; $resn_exch{"H3O1 DTO"}="1H3O DTO"; $resn_exch{" H42 DTO"}="2H4  DTO";
$resn_exch{" H41 DTO"}="1H4  DTO"; $resn_exch{"HOG2 DTP"}="2HOG DTP"; $resn_exch{"HOG3 DTP"}="3HOG DTP";
$resn_exch{"HOB2 DTP"}="2HOB DTP"; $resn_exch{"HOA2 DTP"}="2HOA DTP"; $resn_exch{"H5'1 DTP"}="1H5* DTP";
$resn_exch{"H5'2 DTP"}="2H5* DTP"; $resn_exch{" H4' DTP"}=" H4* DTP"; $resn_exch{" H3' DTP"}=" H3* DTP";
$resn_exch{"HO3' DTP"}="*HO3 DTP"; $resn_exch{"H2'1 DTP"}="1H2* DTP"; $resn_exch{"H2'2 DTP"}="2H2* DTP";
$resn_exch{" H1' DTP"}=" H1* DTP"; $resn_exch{"HN61 DTP"}="1HN6 DTP"; $resn_exch{"HN62 DTP"}="2HN6 DTP";
$resn_exch{" H21 DTQ"}="1H2  DTQ"; $resn_exch{" H71 DTQ"}="1H7  DTQ"; $resn_exch{"H101 DTQ"}="1H10 DTQ";
$resn_exch{"H121 DTQ"}="1H12 DTQ"; $resn_exch{"H151 DTQ"}="1H15 DTQ"; $resn_exch{"H171 DTQ"}="1H17 DTQ";
$resn_exch{"H181 DTQ"}="1H18 DTQ"; $resn_exch{"H191 DTQ"}="1H19 DTQ"; $resn_exch{"H211 DTQ"}="1H21 DTQ";
$resn_exch{"H271 DTQ"}="1H27 DTQ"; $resn_exch{"H272 DTQ"}="2H27 DTQ"; $resn_exch{"H273 DTQ"}="3H27 DTQ";
$resn_exch{"H321 DTQ"}="1H32 DTQ"; $resn_exch{"H322 DTQ"}="2H32 DTQ"; $resn_exch{"H323 DTQ"}="3H32 DTQ";
$resn_exch{" HB2 DTR"}="1HB  DTR"; $resn_exch{" HB3 DTR"}="2HB  DTR"; $resn_exch{" H11 DTT"}="1H1  DTT";
$resn_exch{" H12 DTT"}="2H1  DTT"; $resn_exch{" H41 DTT"}="1H4  DTT"; $resn_exch{" H42 DTT"}="2H4  DTT";
$resn_exch{"H1C1 DTU"}="1H1C DTU"; $resn_exch{"H1C2 DTU"}="2H1C DTU"; $resn_exch{"H4C1 DTU"}="1H4C DTU";
$resn_exch{"H4C2 DTU"}="2H4C DTU"; $resn_exch{"H1C1 DTV"}="1H1C DTV"; $resn_exch{"H1C2 DTV"}="2H1C DTV";
$resn_exch{"H4C1 DTV"}="1H4C DTV"; $resn_exch{"H4C2 DTV"}="2H4C DTV"; $resn_exch{"H211 DTX"}="1H21 DTX";
$resn_exch{"H212 DTX"}="2H21 DTX"; $resn_exch{"H161 DTX"}="1H16 DTX"; $resn_exch{"H162 DTX"}="2H16 DTX";
$resn_exch{"H121 DTX"}="1H12 DTX"; $resn_exch{"H122 DTX"}="2H12 DTX"; $resn_exch{"H111 DTX"}="1H11 DTX";
$resn_exch{"H112 DTX"}="2H11 DTX"; $resn_exch{"H181 DTX"}="1H18 DTX"; $resn_exch{"H182 DTX"}="2H18 DTX";
$resn_exch{"H183 DTX"}="3H18 DTX"; $resn_exch{"H151 DTX"}="1H15 DTX"; $resn_exch{"H152 DTX"}="2H15 DTX";
$resn_exch{"H14O DTX"}="OH14 DTX"; $resn_exch{"HC71 DTX"}="1HC7 DTX"; $resn_exch{"HC72 DTX"}="2HC7 DTX";
$resn_exch{"HC11 DTX"}="1HC1 DTX"; $resn_exch{"HC12 DTX"}="2HC1 DTX"; $resn_exch{"H191 DTX"}="1H19 DTX";
$resn_exch{"H192 DTX"}="2H19 DTX"; $resn_exch{"H193 DTX"}="3H19 DTX"; $resn_exch{"HC61 DTX"}="1HC6 DTX";
$resn_exch{"HC62 DTX"}="2HC6 DTX"; $resn_exch{"HC41 DTX"}="1HC4 DTX"; $resn_exch{"HC42 DTX"}="2HC4 DTX";
$resn_exch{"HC21 DTX"}="1HC2 DTX"; $resn_exch{"HC22 DTX"}="2HC2 DTX"; $resn_exch{"H32O DTX"}="OH32 DTX";
$resn_exch{" H2  DTY"}=" HN2 DTY"; $resn_exch{" HB2 DTY"}="1HB  DTY"; $resn_exch{" HB3 DTY"}="2HB  DTY";
$resn_exch{"HOP3  DU"}="3HOP  DU"; $resn_exch{"HOP2  DU"}="2HOP  DU"; $resn_exch{" H5'  DU"}="1H5*  DU";
$resn_exch{"H5''  DU"}="2H5*  DU"; $resn_exch{" H4'  DU"}=" H4*  DU"; $resn_exch{" H3'  DU"}=" H3*  DU";
$resn_exch{"HO3'  DU"}="H3T   DU"; $resn_exch{" H2'  DU"}="1H2*  DU"; $resn_exch{"H2''  DU"}="2H2*  DU";
$resn_exch{" H1'  DU"}=" H1*  DU"; $resn_exch{" H61 DUC"}="1H6  DUC"; $resn_exch{" H62 DUC"}="2H6  DUC";
$resn_exch{" H51 DUC"}="1H5  DUC"; $resn_exch{" H52 DUC"}="2H5  DUC"; $resn_exch{" H1' DUD"}=" H1* DUD";
$resn_exch{"H2'1 DUD"}="1H2* DUD"; $resn_exch{"H2'2 DUD"}="2H2* DUD"; $resn_exch{" H3' DUD"}=" H3* DUD";
$resn_exch{" H4' DUD"}=" H4* DUD"; $resn_exch{"HO3' DUD"}="*HO3 DUD"; $resn_exch{"H5'1 DUD"}="1H5* DUD";
$resn_exch{"H5'2 DUD"}="2H5* DUD"; $resn_exch{"HOA2 DUD"}="2HOA DUD"; $resn_exch{"HOB2 DUD"}="2HOB DUD";
$resn_exch{"HOB3 DUD"}="3HOB DUD"; $resn_exch{" H1' DUN"}=" H1* DUN"; $resn_exch{"H2'1 DUN"}="1H2* DUN";
$resn_exch{"H2'2 DUN"}="2H2* DUN"; $resn_exch{" H3' DUN"}=" H3* DUN"; $resn_exch{" H4' DUN"}=" H4* DUN";
$resn_exch{"H5'1 DUN"}="1H5* DUN"; $resn_exch{"H5'2 DUN"}="2H5* DUN"; $resn_exch{"H101 DUO"}="1H10 DUO";
$resn_exch{"H102 DUO"}="2H10 DUO"; $resn_exch{"H103 DUO"}="3H10 DUO"; $resn_exch{"H121 DUO"}="1H12 DUO";
$resn_exch{"H122 DUO"}="2H12 DUO"; $resn_exch{"H251 DUO"}="1H25 DUO"; $resn_exch{"H252 DUO"}="2H25 DUO";
$resn_exch{"H253 DUO"}="3H25 DUO"; $resn_exch{"H271 DUO"}="1H27 DUO"; $resn_exch{"H272 DUO"}="2H27 DUO";
$resn_exch{"H273 DUO"}="3H27 DUO"; $resn_exch{"H291 DUO"}="1H29 DUO"; $resn_exch{"H292 DUO"}="2H29 DUO";
$resn_exch{"H293 DUO"}="3H29 DUO"; $resn_exch{"H331 DUO"}="1H33 DUO"; $resn_exch{"H332 DUO"}="2H33 DUO";
$resn_exch{"H333 DUO"}="3H33 DUO"; $resn_exch{"H371 DUO"}="1H37 DUO"; $resn_exch{"H372 DUO"}="2H37 DUO";
$resn_exch{"H373 DUO"}="3H37 DUO"; $resn_exch{" H1' DUP"}=" H1* DUP"; $resn_exch{"H2'1 DUP"}="1H2* DUP";
$resn_exch{"H2'2 DUP"}="2H2* DUP"; $resn_exch{" H3' DUP"}=" H3* DUP"; $resn_exch{" H4' DUP"}=" H4* DUP";
$resn_exch{"H5'1 DUP"}="1H5* DUP"; $resn_exch{"H5'2 DUP"}="2H5* DUP"; $resn_exch{" H1' DUR"}=" H1* DUR";
$resn_exch{"H2'1 DUR"}="1H2* DUR"; $resn_exch{"H2'2 DUR"}="2H2* DUR"; $resn_exch{" H3' DUR"}=" H3* DUR";
$resn_exch{" H4' DUR"}=" H4* DUR"; $resn_exch{"HO3' DUR"}="*HO3 DUR"; $resn_exch{"H5'1 DUR"}="1H5* DUR";
$resn_exch{"H5'2 DUR"}="2H5* DUR"; $resn_exch{"HO5' DUR"}="*HO5 DUR"; $resn_exch{" H1' DUT"}=" H1* DUT";
$resn_exch{"H2'1 DUT"}="1H2* DUT"; $resn_exch{"H2'2 DUT"}="2H2* DUT"; $resn_exch{" H3' DUT"}=" H3* DUT";
$resn_exch{" H4' DUT"}=" H4* DUT"; $resn_exch{"HO3' DUT"}="*HO3 DUT"; $resn_exch{"H5'1 DUT"}="1H5* DUT";
$resn_exch{"H5'2 DUT"}="2H5* DUT"; $resn_exch{"HOA2 DUT"}="2HOA DUT"; $resn_exch{"HOB2 DUT"}="2HOB DUT";
$resn_exch{"HOG2 DUT"}="2HOG DUT"; $resn_exch{"HOG3 DUT"}="3HOG DUT"; $resn_exch{"H2'1 DUX"}="1H2' DUX";
$resn_exch{"H2'2 DUX"}="2H2' DUX"; $resn_exch{"H5'1 DUX"}="1H5' DUX"; $resn_exch{"H5'2 DUX"}="2H5' DUX";
$resn_exch{" H   DVA"}="1HN  DVA"; $resn_exch{" H2  DVA"}="2HN  DVA"; $resn_exch{"HG11 DVA"}="1HG1 DVA";
$resn_exch{"HG12 DVA"}="2HG1 DVA"; $resn_exch{"HG13 DVA"}="3HG1 DVA"; $resn_exch{"HG21 DVA"}="1HG2 DVA";
$resn_exch{"HG22 DVA"}="2HG2 DVA"; $resn_exch{"HG23 DVA"}="3HG2 DVA"; $resn_exch{"HC21 DVR"}="1HC2 DVR";
$resn_exch{"HC22 DVR"}="2HC2 DVR"; $resn_exch{"H121 DVR"}="1H12 DVR"; $resn_exch{"H122 DVR"}="2H12 DVR";
$resn_exch{"H11A DVV"}="1H1  DVV"; $resn_exch{" H12 DVV"}="2H1  DVV"; $resn_exch{" H13 DVV"}="3H1  DVV";
$resn_exch{"H61A DVV"}="1H6  DVV"; $resn_exch{"H62A DVV"}="2H6  DVV"; $resn_exch{" H81 DVV"}="1H8  DVV";
$resn_exch{" H82 DVV"}="2H8  DVV"; $resn_exch{" H83 DVV"}="3H8  DVV"; $resn_exch{" H91 DVV"}="1H9  DVV";
$resn_exch{" H92 DVV"}="2H9  DVV"; $resn_exch{" H93 DVV"}="3H9  DVV"; $resn_exch{"H6'1 DVV"}="1H6' DVV";
$resn_exch{"H6'2 DVV"}="2H6' DVV"; $resn_exch{"H281 DVV"}="1H28 DVV"; $resn_exch{"H282 DVV"}="2H28 DVV";
$resn_exch{"H311 DVV"}="1H31 DVV"; $resn_exch{"H312 DVV"}="2H31 DVV"; $resn_exch{"H121 DW2"}="1H12 DW2";
$resn_exch{"H131 DW2"}="1H13 DW2"; $resn_exch{"H141 DW2"}="1H14 DW2"; $resn_exch{"H161 DW2"}="1H16 DW2";
$resn_exch{" H61 DW2"}="1H6  DW2"; $resn_exch{" H51 DW2"}="1H5  DW2"; $resn_exch{"H223 DW2"}="23H2 DW2";
$resn_exch{"H228 DW2"}="28H2 DW2"; $resn_exch{"H271 DW2"}="1H27 DW2"; $resn_exch{"H226 DW2"}="26H2 DW2";
$resn_exch{"H251 DW2"}="1H25 DW2"; $resn_exch{"H241 DW2"}="1H24 DW2"; $resn_exch{" H11 DX5"}="1H1  DX5";
$resn_exch{" H12 DX5"}="2H1  DX5"; $resn_exch{" H51 DX5"}="1H5  DX5"; $resn_exch{" H52 DX5"}="2H5  DX5";
$resn_exch{"H227 DX9"}="2H27 DX9"; $resn_exch{"HN21 DX9"}="1HN2 DX9"; $resn_exch{"HN12 DX9"}="2HN1 DX9";
$resn_exch{"HN11 DX9"}="1HN1 DX9"; $resn_exch{"H141 DX9"}="1H14 DX9"; $resn_exch{"H142 DX9"}="2H14 DX9";
$resn_exch{"H127 DX9"}="1H27 DX9"; $resn_exch{"H130 DX9"}="1H30 DX9"; $resn_exch{"H129 DX9"}="1H29 DX9";
$resn_exch{"H229 DX9"}="2H29 DX9"; $resn_exch{"HN31 DX9"}="1HN3 DX9"; $resn_exch{"H230 DX9"}="2H30 DX9";
$resn_exch{"H331 DX9"}="1H33 DX9"; $resn_exch{"H332 DX9"}="2H33 DX9"; $resn_exch{"H333 DX9"}="3H33 DX9";
$resn_exch{" H21 DXA"}="1H2  DXA"; $resn_exch{" H22 DXA"}="2H2  DXA"; $resn_exch{" H11 DXA"}="1H1  DXA";
$resn_exch{" H12 DXA"}="2H1  DXA"; $resn_exch{" H13 DXA"}="3H1  DXA"; $resn_exch{" H51 DXA"}="1H5  DXA";
$resn_exch{" H52 DXA"}="2H5  DXA"; $resn_exch{" H53 DXA"}="3H5  DXA"; $resn_exch{" H71 DXA"}="1H7  DXA";
$resn_exch{" H72 DXA"}="2H7  DXA"; $resn_exch{" H73 DXA"}="3H7  DXA"; $resn_exch{" H81 DXA"}="1H8  DXA";
$resn_exch{" H82 DXA"}="2H8  DXA"; $resn_exch{" H91 DXA"}="1H9  DXA"; $resn_exch{" H92 DXA"}="2H9  DXA";
$resn_exch{" H93 DXA"}="3H9  DXA"; $resn_exch{" H21 DXB"}="1H2  DXB"; $resn_exch{" H22 DXB"}="2H2  DXB";
$resn_exch{" H22 DXC"}="2H2  DXC"; $resn_exch{" H21 DXC"}="1H2  DXC"; $resn_exch{" H52 DXC"}="2H5  DXC";
$resn_exch{" H51 DXC"}="1H5  DXC"; $resn_exch{" H62 DXC"}="2H6  DXC"; $resn_exch{" H61 DXC"}="1H6  DXC";
$resn_exch{" H72 DXC"}="2H7  DXC"; $resn_exch{" H71 DXC"}="1H7  DXC"; $resn_exch{" H82 DXC"}="2H8  DXC";
$resn_exch{" H81 DXC"}="1H8  DXC"; $resn_exch{"H142 DXC"}="2H14 DXC"; $resn_exch{"H141 DXC"}="1H14 DXC";
$resn_exch{"H152 DXC"}="2H15 DXC"; $resn_exch{"H151 DXC"}="1H15 DXC"; $resn_exch{"H162 DXC"}="2H16 DXC";
$resn_exch{"H161 DXC"}="1H16 DXC"; $resn_exch{"H183 DXC"}="3H18 DXC"; $resn_exch{"H182 DXC"}="2H18 DXC";
$resn_exch{"H181 DXC"}="1H18 DXC"; $resn_exch{"H1O1 DXC"}="1H1O DXC"; $resn_exch{"H203 DXC"}="3H20 DXC";
$resn_exch{"H202 DXC"}="2H20 DXC"; $resn_exch{"H201 DXC"}="1H20 DXC"; $resn_exch{"H212 DXC"}="2H21 DXC";
$resn_exch{"H211 DXC"}="1H21 DXC"; $resn_exch{"H222 DXC"}="2H22 DXC"; $resn_exch{"H221 DXC"}="1H22 DXC";
$resn_exch{"H243 DXC"}="3H24 DXC"; $resn_exch{"H242 DXC"}="2H24 DXC"; $resn_exch{"H241 DXC"}="1H24 DXC";
$resn_exch{"HOP2 DXD"}="2HOP DXD"; $resn_exch{"H5'1 DXD"}="1H5* DXD"; $resn_exch{"H5'2 DXD"}="2H5* DXD";
$resn_exch{" H4' DXD"}=" H4* DXD"; $resn_exch{" H1' DXD"}=" H1* DXD"; $resn_exch{"H6'1 DXD"}="1H6* DXD";
$resn_exch{"H6'2 DXD"}="2H6* DXD"; $resn_exch{" H7' DXD"}=" H7* DXD"; $resn_exch{"H2'1 DXD"}="1H2* DXD";
$resn_exch{"H2'2 DXD"}="2H2* DXD"; $resn_exch{" H3' DXD"}=" H3* DXD"; $resn_exch{"HO3' DXD"}="H3T  DXD";
$resn_exch{"HOP3 DXD"}="3HOP DXD"; $resn_exch{" H41 DXE"}="1H4  DXE"; $resn_exch{" H42 DXE"}="2H4  DXE";
$resn_exch{" H43 DXE"}="3H4  DXE"; $resn_exch{" H31 DXE"}="1H3  DXE"; $resn_exch{" H32 DXE"}="2H3  DXE";
$resn_exch{" H21 DXE"}="1H2  DXE"; $resn_exch{" H22 DXE"}="2H2  DXE"; $resn_exch{" H11 DXE"}="1H1  DXE";
$resn_exch{" H12 DXE"}="2H1  DXE"; $resn_exch{" H13 DXE"}="3H1  DXE"; $resn_exch{" H21 DXG"}="1H2  DXG";
$resn_exch{"HO21 DXG"}="1HO2 DXG"; $resn_exch{" H31 DXG"}="1H3  DXG"; $resn_exch{"HO31 DXG"}="1HO3 DXG";
$resn_exch{" H41 DXG"}="1H4  DXG"; $resn_exch{" H42 DXG"}="2H4  DXG"; $resn_exch{"HO51 DXG"}="1HO5 DXG";
$resn_exch{" H51 DXG"}="1H5  DXG"; $resn_exch{"HOP2 DXN"}="2HOP DXN"; $resn_exch{"H5'1 DXN"}="1H5* DXN";
$resn_exch{"H5'2 DXN"}="2H5* DXN"; $resn_exch{" H4' DXN"}=" H4* DXN"; $resn_exch{" H1' DXN"}=" H1* DXN";
$resn_exch{"H6'1 DXN"}="1H6* DXN"; $resn_exch{"H6'2 DXN"}="2H6* DXN"; $resn_exch{" H7' DXN"}=" H7* DXN";
$resn_exch{"H2'1 DXN"}="1H2* DXN"; $resn_exch{"H2'2 DXN"}="2H2* DXN"; $resn_exch{" H3' DXN"}=" H3* DXN";
$resn_exch{"HO3' DXN"}="H3T  DXN"; $resn_exch{"HOP3 DXN"}="3HOP DXN"; $resn_exch{"HC11 DXP"}="1HC1 DXP";
$resn_exch{"HC12 DXP"}="2HC1 DXP"; $resn_exch{"HC13 DXP"}="3HC1 DXP"; $resn_exch{"HOP2 DXP"}="2HOP DXP";
$resn_exch{"HOP1 DXP"}="1HOP DXP"; $resn_exch{"H211 DXT"}="1H21 DXT"; $resn_exch{"H212 DXT"}="2H21 DXT";
$resn_exch{"H411 DXT"}="1H41 DXT"; $resn_exch{"H412 DXT"}="2H41 DXT"; $resn_exch{"H413 DXT"}="3H41 DXT";
$resn_exch{"H421 DXT"}="1H42 DXT"; $resn_exch{"H422 DXT"}="2H42 DXT"; $resn_exch{"H423 DXT"}="3H42 DXT";
$resn_exch{"H611 DXT"}="1H61 DXT"; $resn_exch{"H612 DXT"}="2H61 DXT"; $resn_exch{"H613 DXT"}="3H61 DXT";
$resn_exch{" H1  DXX"}="1H   DXX"; $resn_exch{" HB1 DXX"}="1HB  DXX"; $resn_exch{" HB2 DXX"}="2HB  DXX";
$resn_exch{" HB3 DXX"}="3HB  DXX"; $resn_exch{"HZ22 DYB"}="2HZ2 DYB"; $resn_exch{"HZ23 DYB"}="3HZ2 DYB";
$resn_exch{"HZ32 DYB"}="2HZ3 DYB"; $resn_exch{"HZ33 DYB"}="3HZ3 DYB"; $resn_exch{"HZ12 DYB"}="2HZ1 DYB";
$resn_exch{"HZ13 DYB"}="3HZ1 DYB"; $resn_exch{" HH2 DYB"}="2HH  DYB"; $resn_exch{" HH3 DYB"}="3HH  DYB";
$resn_exch{" HQ2 DYB"}="2HQ  DYB"; $resn_exch{" HQ3 DYB"}="3HQ  DYB"; $resn_exch{" HI2 DYB"}="2HI  DYB";
$resn_exch{" HI3 DYB"}="3HI  DYB"; $resn_exch{" HK1 DYB"}="1HK  DYB"; $resn_exch{" HK2 DYB"}="2HK  DYB";
$resn_exch{" HK3 DYB"}="3HK  DYB"; $resn_exch{"HB11 DYG"}="1HB1 DYG"; $resn_exch{"HB12 DYG"}="2HB1 DYG";
$resn_exch{" HN1 DYG"}="1HN  DYG"; $resn_exch{" HN2 DYG"}="2HN  DYG"; $resn_exch{"HA31 DYG"}="1HA3 DYG";
$resn_exch{"HA32 DYG"}="2HA3 DYG"; $resn_exch{" H71 DYM"}="1H7  DYM"; $resn_exch{" H72 DYM"}="2H7  DYM";
$resn_exch{" H91 DYM"}="1H9  DYM"; $resn_exch{" H92 DYM"}="2H9  DYM"; $resn_exch{"H111 DYM"}="1H11 DYM";
$resn_exch{"H112 DYM"}="2H11 DYM"; $resn_exch{"H113 DYM"}="3H11 DYM"; $resn_exch{"HAY1 DYP"}="1HAY DYP";
$resn_exch{"HAY2 DYP"}="2HAY DYP"; $resn_exch{"HAY3 DYP"}="3HAY DYP"; $resn_exch{"HAX1 DYP"}="1HAX DYP";
$resn_exch{"HAX2 DYP"}="2HAX DYP"; $resn_exch{"HAU1 DYP"}="1HAU DYP"; $resn_exch{"HAU2 DYP"}="2HAU DYP";
$resn_exch{"HAT1 DYP"}="1HAT DYP"; $resn_exch{"HAT2 DYP"}="2HAT DYP"; $resn_exch{"HAI1 DYP"}="1HAI DYP";
$resn_exch{"HAI2 DYP"}="2HAI DYP"; $resn_exch{"HAH1 DYP"}="1HAH DYP"; $resn_exch{"HAH2 DYP"}="2HAH DYP";
$resn_exch{"HAF1 DYP"}="1HAF DYP"; $resn_exch{"HAF2 DYP"}="2HAF DYP"; $resn_exch{" H91 DYP"}="1H9  DYP";
$resn_exch{" H92 DYP"}="2H9  DYP"; $resn_exch{"HN21 DYP"}="1HN2 DYP"; $resn_exch{"HN22 DYP"}="2HN2 DYP";
$resn_exch{"HN41 DYP"}="1HN4 DYP"; $resn_exch{"HN42 DYP"}="2HN4 DYP"; $resn_exch{"H961 DZ8"}="1H96 DZ8";
$resn_exch{"H962 DZ8"}="2H96 DZ8"; $resn_exch{"H963 DZ8"}="3H96 DZ8"; $resn_exch{"H951 DZ8"}="1H95 DZ8";
$resn_exch{"H952 DZ8"}="2H95 DZ8"; $resn_exch{"H953 DZ8"}="3H95 DZ8"; $resn_exch{"H931 DZ8"}="1H93 DZ8";
$resn_exch{"H932 DZ8"}="2H93 DZ8"; $resn_exch{"H921 DZ8"}="1H92 DZ8"; $resn_exch{"H922 DZ8"}="2H92 DZ8";
$resn_exch{"H911 DZ8"}="1H91 DZ8"; $resn_exch{"H912 DZ8"}="2H91 DZ8"; $resn_exch{"HN61 DZ8"}="1HN6 DZ8";
$resn_exch{"HN62 DZ8"}="2HN6 DZ8"; $resn_exch{"H811 DZ8"}="1H81 DZ8"; $resn_exch{"H812 DZ8"}="2H81 DZ8";
$resn_exch{"H7'1 DZ8"}="1H7' DZ8"; $resn_exch{"H7'2 DZ8"}="2H7' DZ8"; $resn_exch{"HN21 DZF"}="1HN2 DZF";
$resn_exch{"HN22 DZF"}="2HN2 DZF"; $resn_exch{" H91 DZF"}="1H9  DZF"; $resn_exch{" H92 DZF"}="2H9  DZF";
$resn_exch{" HB1 DZF"}="1HB  DZF"; $resn_exch{" HB2 DZF"}="2HB  DZF"; $resn_exch{" HG1 DZF"}="1HG  DZF";
$resn_exch{" HG2 DZF"}="2HG  DZF"; $resn_exch{"HOE2 DZF"}="2HOE DZF"; $resn_exch{"H6C1 DZN"}="1H6C DZN";
$resn_exch{"H6C2 DZN"}="2H6C DZN"; $resn_exch{"H171 DZP"}="1H17 DZP"; $resn_exch{"H172 DZP"}="2H17 DZP";
$resn_exch{"H201 DZP"}="1H20 DZP"; $resn_exch{"H202 DZP"}="2H20 DZP"; $resn_exch{"H203 DZP"}="3H20 DZP";
$resn_exch{" H11 DZZ"}="1H1  DZZ"; $resn_exch{" H12 DZZ"}="2H1  DZZ"; $resn_exch{" H13 DZZ"}="3H1  DZZ";
$resn_exch{" H21 DZZ"}="1H2  DZZ"; $resn_exch{" H22 DZZ"}="2H2  DZZ"; $resn_exch{" H23 DZZ"}="3H2  DZZ";
$resn_exch{"HOP2   E"}="2HOP   E"; $resn_exch{"HOP3   E"}="3HOP   E"; $resn_exch{" H5'   E"}="1H5*   E";
$resn_exch{"H5''   E"}="2H5*   E"; $resn_exch{" H4'   E"}=" H4*   E"; $resn_exch{" H3'   E"}=" H3*   E";
$resn_exch{"HO3'   E"}="*HO3   E"; $resn_exch{" H2'   E"}="1H2*   E"; $resn_exch{"H2''   E"}="2H2*   E";
$resn_exch{" H1'   E"}=" H1*   E"; $resn_exch{"H251 E04"}="1H25 E04"; $resn_exch{"H252 E04"}="2H25 E04";
$resn_exch{"H253 E04"}="3H25 E04"; $resn_exch{"H241 E04"}="1H24 E04"; $resn_exch{"H242 E04"}="2H24 E04";
$resn_exch{"H243 E04"}="3H24 E04"; $resn_exch{"H221 E04"}="1H22 E04"; $resn_exch{"H222 E04"}="2H22 E04";
$resn_exch{"H231 E04"}="1H23 E04"; $resn_exch{"H232 E04"}="2H23 E04"; $resn_exch{"H181 E04"}="1H18 E04";
$resn_exch{"H182 E04"}="2H18 E04"; $resn_exch{"H183 E04"}="3H18 E04"; $resn_exch{"H151 E04"}="1H15 E04";
$resn_exch{"H152 E04"}="2H15 E04"; $resn_exch{"H161 E04"}="1H16 E04"; $resn_exch{"H162 E04"}="2H16 E04";
$resn_exch{"H111 E04"}="1H11 E04"; $resn_exch{"H112 E04"}="2H11 E04"; $resn_exch{"H121 E04"}="1H12 E04";
$resn_exch{"H122 E04"}="2H12 E04"; $resn_exch{"H281 E04"}="1H28 E04"; $resn_exch{"H282 E04"}="2H28 E04";
$resn_exch{" H61 E04"}="1H6  E04"; $resn_exch{" H62 E04"}="2H6  E04"; $resn_exch{" H71 E04"}="1H7  E04";
$resn_exch{" H72 E04"}="2H7  E04"; $resn_exch{" H51 E09"}="1H5  E09"; $resn_exch{"H111 E09"}="1H11 E09";
$resn_exch{"H112 E09"}="2H11 E09"; $resn_exch{"H121 E09"}="1H12 E09"; $resn_exch{"H122 E09"}="2H12 E09";
$resn_exch{"H201 E09"}="1H20 E09"; $resn_exch{"H202 E09"}="2H20 E09"; $resn_exch{"H251 E09"}="1H25 E09";
$resn_exch{"H252 E09"}="2H25 E09"; $resn_exch{"H261 E09"}="1H26 E09"; $resn_exch{"H262 E09"}="2H26 E09";
$resn_exch{" H71 E09"}="1H7  E09"; $resn_exch{" H72 E09"}="2H7  E09"; $resn_exch{"H311 E09"}="1H31 E09";
$resn_exch{"H312 E09"}="2H31 E09"; $resn_exch{"H313 E09"}="3H31 E09"; $resn_exch{"H9C1 E10"}="1H9C E10";
$resn_exch{"H9C2 E10"}="2H9C E10"; $resn_exch{"H131 E10"}="1H13 E10"; $resn_exch{"H132 E10"}="2H13 E10";
$resn_exch{"H141 E10"}="1H14 E10"; $resn_exch{"H142 E10"}="2H14 E10"; $resn_exch{"H241 E10"}="1H24 E10";
$resn_exch{"H242 E10"}="2H24 E10"; $resn_exch{"H361 E10"}="1H36 E10"; $resn_exch{"H362 E10"}="2H36 E10";
$resn_exch{"H381 E10"}="1H38 E10"; $resn_exch{"H382 E10"}="2H38 E10"; $resn_exch{"H391 E10"}="1H39 E10";
$resn_exch{"H392 E10"}="2H39 E10"; $resn_exch{"H411 E10"}="1H41 E10"; $resn_exch{"H412 E10"}="2H41 E10";
$resn_exch{"H631 E10"}="1H63 E10"; $resn_exch{"H632 E10"}="2H63 E10"; $resn_exch{"H641 E10"}="1H64 E10";
$resn_exch{"H642 E10"}="2H64 E10"; $resn_exch{"H651 E10"}="1H65 E10"; $resn_exch{"H652 E10"}="2H65 E10";
$resn_exch{"H661 E10"}="1H66 E10"; $resn_exch{"H662 E10"}="2H66 E10"; $resn_exch{"H671 E10"}="1H67 E10";
$resn_exch{"H672 E10"}="2H67 E10"; $resn_exch{"H681 E10"}="1H68 E10"; $resn_exch{"H682 E10"}="2H68 E10";
$resn_exch{"H691 E10"}="1H69 E10"; $resn_exch{"H692 E10"}="2H69 E10"; $resn_exch{"H701 E10"}="1H70 E10";
$resn_exch{"H702 E10"}="2H70 E10"; $resn_exch{"H141 E12"}="1H14 E12"; $resn_exch{"H142 E12"}="2H14 E12";
$resn_exch{"H241 E12"}="1H24 E12"; $resn_exch{"H242 E12"}="2H24 E12"; $resn_exch{"H9C1 E12"}="1H9C E12";
$resn_exch{"H9C2 E12"}="2H9C E12"; $resn_exch{"H131 E12"}="1H13 E12"; $resn_exch{"H132 E12"}="2H13 E12";
$resn_exch{"H631 E12"}="1H63 E12"; $resn_exch{"H632 E12"}="2H63 E12"; $resn_exch{"H361 E12"}="1H36 E12";
$resn_exch{"H362 E12"}="2H36 E12"; $resn_exch{"H381 E12"}="1H38 E12"; $resn_exch{"H382 E12"}="2H38 E12";
$resn_exch{"H391 E12"}="1H39 E12"; $resn_exch{"H392 E12"}="2H39 E12"; $resn_exch{"H411 E12"}="1H41 E12";
$resn_exch{"H412 E12"}="2H41 E12"; $resn_exch{"H641 E12"}="1H64 E12"; $resn_exch{"H642 E12"}="2H64 E12";
$resn_exch{"H651 E12"}="1H65 E12"; $resn_exch{"H652 E12"}="2H65 E12"; $resn_exch{"H661 E12"}="1H66 E12";
$resn_exch{"H662 E12"}="2H66 E12"; $resn_exch{"H671 E12"}="1H67 E12"; $resn_exch{"H672 E12"}="2H67 E12";
$resn_exch{"H681 E12"}="1H68 E12"; $resn_exch{"H682 E12"}="2H68 E12"; $resn_exch{"H691 E12"}="1H69 E12";
$resn_exch{"H692 E12"}="2H69 E12"; $resn_exch{"H701 E12"}="1H70 E12"; $resn_exch{"H702 E12"}="2H70 E12";
$resn_exch{"H711 E12"}="1H71 E12"; $resn_exch{"H712 E12"}="2H71 E12"; $resn_exch{"H721 E12"}="1H72 E12";
$resn_exch{"H722 E12"}="2H72 E12"; $resn_exch{"HN11 E1H"}="1HN1 E1H"; $resn_exch{"HA12 E1H"}="2HA1 E1H";
$resn_exch{"HB11 E1H"}="1HB1 E1H"; $resn_exch{"HB12 E1H"}="2HB1 E1H"; $resn_exch{"HB13 E1H"}="3HB1 E1H";
$resn_exch{"H3C1 E1P"}="1H3C E1P"; $resn_exch{"H3C2 E1P"}="2H3C E1P"; $resn_exch{"H2C1 E1P"}="1H2C E1P";
$resn_exch{"H2C2 E1P"}="2H2C E1P"; $resn_exch{"H1C1 E1P"}="1H1C E1P"; $resn_exch{"H1C2 E1P"}="2H1C E1P";
$resn_exch{"H5C1 E1P"}="1H5C E1P"; $resn_exch{"H5C2 E1P"}="2H5C E1P"; $resn_exch{"H4C1 E1P"}="1H4C E1P";
$resn_exch{"H4C2 E1P"}="2H4C E1P"; $resn_exch{"H8C1 E1P"}="1H8C E1P"; $resn_exch{"H8C2 E1P"}="2H8C E1P";
$resn_exch{"H9C1 E1P"}="1H9C E1P"; $resn_exch{"H9C2 E1P"}="2H9C E1P"; $resn_exch{"H9C3 E1P"}="3H9C E1P";
$resn_exch{"HOP2 E1X"}="2HOP E1X"; $resn_exch{"H101 E1X"}="1H10 E1X"; $resn_exch{"H102 E1X"}="2H10 E1X";
$resn_exch{"H111 E1X"}="1H11 E1X"; $resn_exch{"H112 E1X"}="2H11 E1X"; $resn_exch{"H113 E1X"}="3H11 E1X";
$resn_exch{" H2' E1X"}="1H2* E1X"; $resn_exch{"H2'' E1X"}="2H2* E1X"; $resn_exch{" H5' E1X"}="1H5* E1X";
$resn_exch{"H5'' E1X"}="2H5* E1X"; $resn_exch{" H4' E1X"}=" H4* E1X"; $resn_exch{" H1' E1X"}=" H1* E1X";
$resn_exch{"HO3' E1X"}="H3T  E1X"; $resn_exch{"HOP3 E1X"}="3HOP E1X"; $resn_exch{" H91 E20"}="1H9  E20";
$resn_exch{" H92 E20"}="2H9  E20"; $resn_exch{"H101 E20"}="1H10 E20"; $resn_exch{"H102 E20"}="2H10 E20";
$resn_exch{"H121 E20"}="1H12 E20"; $resn_exch{"H122 E20"}="2H12 E20"; $resn_exch{"H131 E20"}="1H13 E20";
$resn_exch{"H132 E20"}="2H13 E20"; $resn_exch{"H151 E20"}="1H15 E20"; $resn_exch{"H152 E20"}="2H15 E20";
$resn_exch{"H161 E20"}="1H16 E20"; $resn_exch{"H162 E20"}="2H16 E20"; $resn_exch{"H171 E20"}="1H17 E20";
$resn_exch{"H172 E20"}="2H17 E20"; $resn_exch{"H261 E20"}="1H26 E20"; $resn_exch{"H262 E20"}="2H26 E20";
$resn_exch{"H263 E20"}="3H26 E20"; $resn_exch{"H281 E20"}="1H28 E20"; $resn_exch{"H282 E20"}="2H28 E20";
$resn_exch{"H283 E20"}="3H28 E20"; $resn_exch{" H61 E3G"}="1H6  E3G"; $resn_exch{" H62 E3G"}="2H6  E3G";
$resn_exch{" H71 E3G"}="1H7  E3G"; $resn_exch{" H72 E3G"}="2H7  E3G"; $resn_exch{"H111 E3G"}="1H11 E3G";
$resn_exch{"H112 E3G"}="2H11 E3G"; $resn_exch{"H121 E3G"}="1H12 E3G"; $resn_exch{"H122 E3G"}="2H12 E3G";
$resn_exch{"H151 E3G"}="1H15 E3G"; $resn_exch{"H152 E3G"}="2H15 E3G"; $resn_exch{"H161 E3G"}="1H16 E3G";
$resn_exch{"H162 E3G"}="2H16 E3G"; $resn_exch{"H181 E3G"}="1H18 E3G"; $resn_exch{"H182 E3G"}="2H18 E3G";
$resn_exch{"H183 E3G"}="3H18 E3G"; $resn_exch{"H3C1 E3O"}="1H3C E3O"; $resn_exch{"H3C2 E3O"}="2H3C E3O";
$resn_exch{"H4C1 E3O"}="1H4C E3O"; $resn_exch{"H4C2 E3O"}="2H4C E3O"; $resn_exch{"H6C1 E3O"}="1H6C E3O";
$resn_exch{"H6C2 E3O"}="2H6C E3O"; $resn_exch{"H6C3 E3O"}="3H6C E3O"; $resn_exch{"H9C1 E3O"}="1H9C E3O";
$resn_exch{"H9C2 E3O"}="2H9C E3O"; $resn_exch{"H141 E3O"}="1H14 E3O"; $resn_exch{"H142 E3O"}="2H14 E3O";
$resn_exch{"H151 E3O"}="1H15 E3O"; $resn_exch{"H152 E3O"}="2H15 E3O"; $resn_exch{"H261 E4D"}="1H26 E4D";
$resn_exch{"H262 E4D"}="2H26 E4D"; $resn_exch{"H271 E4D"}="1H27 E4D"; $resn_exch{"H272 E4D"}="2H27 E4D";
$resn_exch{"H291 E4D"}="1H29 E4D"; $resn_exch{"H292 E4D"}="2H29 E4D"; $resn_exch{"H301 E4D"}="1H30 E4D";
$resn_exch{"H302 E4D"}="2H30 E4D"; $resn_exch{"H311 E4D"}="1H31 E4D"; $resn_exch{"H312 E4D"}="2H31 E4D";
$resn_exch{"H321 E4D"}="1H32 E4D"; $resn_exch{"H322 E4D"}="2H32 E4D"; $resn_exch{"H331 E4D"}="1H33 E4D";
$resn_exch{"H332 E4D"}="2H33 E4D"; $resn_exch{"HAD1 E4H"}="1HAD E4H"; $resn_exch{"HAD2 E4H"}="2HAD E4H";
$resn_exch{"HAD3 E4H"}="3HAD E4H"; $resn_exch{"HAL1 E4H"}="1HAL E4H"; $resn_exch{"HAL2 E4H"}="2HAL E4H";
$resn_exch{"HAC1 E4H"}="1HAC E4H"; $resn_exch{"HAC2 E4H"}="2HAC E4H"; $resn_exch{"HAC3 E4H"}="3HAC E4H";
$resn_exch{"HAB1 E4H"}="1HAB E4H"; $resn_exch{"HAB2 E4H"}="2HAB E4H"; $resn_exch{"HAB3 E4H"}="3HAB E4H";
$resn_exch{"HAK1 E4H"}="1HAK E4H"; $resn_exch{"HAK2 E4H"}="2HAK E4H"; $resn_exch{"HAA1 E4H"}="1HAA E4H";
$resn_exch{"HAA2 E4H"}="2HAA E4H"; $resn_exch{"HAA3 E4H"}="3HAA E4H"; $resn_exch{"HAE1 E4H"}="1HAE E4H";
$resn_exch{"HAE2 E4H"}="2HAE E4H"; $resn_exch{"HAE3 E4H"}="3HAE E4H"; $resn_exch{"H111 E4N"}="1H11 E4N";
$resn_exch{"H112 E4N"}="2H11 E4N"; $resn_exch{"H121 E4N"}="1H12 E4N"; $resn_exch{"H122 E4N"}="2H12 E4N";
$resn_exch{"H123 E4N"}="3H12 E4N"; $resn_exch{"H211 E4N"}="1H21 E4N"; $resn_exch{"H212 E4N"}="2H21 E4N";
$resn_exch{"H221 E4N"}="1H22 E4N"; $resn_exch{"H222 E4N"}="2H22 E4N"; $resn_exch{"H223 E4N"}="3H22 E4N";
$resn_exch{"H311 E4N"}="1H31 E4N"; $resn_exch{"H312 E4N"}="2H31 E4N"; $resn_exch{"H321 E4N"}="1H32 E4N";
$resn_exch{"H322 E4N"}="2H32 E4N"; $resn_exch{"H323 E4N"}="3H32 E4N"; $resn_exch{"H411 E4N"}="1H41 E4N";
$resn_exch{"H412 E4N"}="2H41 E4N"; $resn_exch{"H421 E4N"}="1H42 E4N"; $resn_exch{"H422 E4N"}="2H42 E4N";
$resn_exch{"H423 E4N"}="3H42 E4N"; $resn_exch{" H41 E4P"}="1H4  E4P"; $resn_exch{" H42 E4P"}="2H4  E4P";
$resn_exch{"HOP2 E4P"}="2HOP E4P"; $resn_exch{"HOP3 E4P"}="3HOP E4P"; $resn_exch{"HAA1 E55"}="1HAA E55";
$resn_exch{"HAA2 E55"}="2HAA E55"; $resn_exch{"HAA3 E55"}="3HAA E55"; $resn_exch{"HAS1 E55"}="1HAS E55";
$resn_exch{"HAS2 E55"}="2HAS E55"; $resn_exch{"HAY1 E55"}="1HAY E55"; $resn_exch{"HAY2 E55"}="2HAY E55";
$resn_exch{"HBE1 E55"}="1HBE E55"; $resn_exch{"HBE2 E55"}="2HBE E55"; $resn_exch{"HBC1 E55"}="1HBC E55";
$resn_exch{"HBC2 E55"}="2HBC E55"; $resn_exch{"HAW1 E55"}="1HAW E55"; $resn_exch{"HAW2 E55"}="2HAW E55";
$resn_exch{"HAX1 E55"}="1HAX E55"; $resn_exch{"HAX2 E55"}="2HAX E55"; $resn_exch{"HBD1 E55"}="1HBD E55";
$resn_exch{"HBD2 E55"}="2HBD E55"; $resn_exch{"HBI1 E55"}="1HBI E55"; $resn_exch{"HBI2 E55"}="2HBI E55";
$resn_exch{"HBL1 E55"}="1HBL E55"; $resn_exch{"HBL2 E55"}="2HBL E55"; $resn_exch{"HBO1 E55"}="1HBO E55";
$resn_exch{"HBO2 E55"}="2HBO E55"; $resn_exch{"HBS1 E55"}="1HBS E55"; $resn_exch{"HBS2 E55"}="2HBS E55";
$resn_exch{"HBV1 E55"}="1HBV E55"; $resn_exch{"HBV2 E55"}="2HBV E55"; $resn_exch{"HBZ1 E55"}="1HBZ E55";
$resn_exch{"HBZ2 E55"}="2HBZ E55"; $resn_exch{"HCE1 E55"}="1HCE E55"; $resn_exch{"HCE2 E55"}="2HCE E55";
$resn_exch{" H61 E55"}="1H6  E55"; $resn_exch{" H62 E55"}="2H6  E55"; $resn_exch{"HCH1 E55"}="1HCH E55";
$resn_exch{"HCH2 E55"}="2HCH E55"; $resn_exch{"HCD1 E55"}="1HCD E55"; $resn_exch{"HCD2 E55"}="2HCD E55";
$resn_exch{"HBY1 E55"}="1HBY E55"; $resn_exch{"HBY2 E55"}="2HBY E55"; $resn_exch{"HBU1 E55"}="1HBU E55";
$resn_exch{"HBU2 E55"}="2HBU E55"; $resn_exch{"HBR1 E55"}="1HBR E55"; $resn_exch{"HBR2 E55"}="2HBR E55";
$resn_exch{"HBP1 E55"}="1HBP E55"; $resn_exch{"HBP2 E55"}="2HBP E55"; $resn_exch{"HBM1 E55"}="1HBM E55";
$resn_exch{"HBM2 E55"}="2HBM E55"; $resn_exch{"HBJ1 E55"}="1HBJ E55"; $resn_exch{"HBJ2 E55"}="2HBJ E55";
$resn_exch{"HBF1 E55"}="1HBF E55"; $resn_exch{"HBF2 E55"}="2HBF E55"; $resn_exch{"HAZ1 E55"}="1HAZ E55";
$resn_exch{"HAZ2 E55"}="2HAZ E55"; $resn_exch{"HAT1 E55"}="1HAT E55"; $resn_exch{"HAT2 E55"}="2HAT E55";
$resn_exch{"HAB1 E55"}="1HAB E55"; $resn_exch{"HAB2 E55"}="2HAB E55"; $resn_exch{"HAB3 E55"}="3HAB E55";
$resn_exch{"HCB1 E55"}="1HCB E55"; $resn_exch{"HCB2 E55"}="2HCB E55"; $resn_exch{"HBX1 E55"}="1HBX E55";
$resn_exch{"HBX2 E55"}="2HBX E55"; $resn_exch{"HBT1 E55"}="1HBT E55"; $resn_exch{"HBT2 E55"}="2HBT E55";
$resn_exch{"HBQ1 E55"}="1HBQ E55"; $resn_exch{"HBQ2 E55"}="2HBQ E55"; $resn_exch{"HBN1 E55"}="1HBN E55";
$resn_exch{"HBN2 E55"}="2HBN E55"; $resn_exch{"HBK1 E55"}="1HBK E55"; $resn_exch{"HBK2 E55"}="2HBK E55";
$resn_exch{"HBG1 E55"}="1HBG E55"; $resn_exch{"HBG2 E55"}="2HBG E55"; $resn_exch{"HBA1 E55"}="1HBA E55";
$resn_exch{"HBA2 E55"}="2HBA E55"; $resn_exch{"HAU1 E55"}="1HAU E55"; $resn_exch{"HAU2 E55"}="2HAU E55";
$resn_exch{"HAC1 E55"}="1HAC E55"; $resn_exch{"HAC2 E55"}="2HAC E55"; $resn_exch{"HAC3 E55"}="3HAC E55";
$resn_exch{"HCI1 E55"}="1HCI E55"; $resn_exch{"HCI2 E55"}="2HCI E55"; $resn_exch{"HAE1 E55"}="1HAE E55";
$resn_exch{"HAE2 E55"}="2HAE E55"; $resn_exch{"HAE3 E55"}="3HAE E55"; $resn_exch{"HCC1 E55"}="1HCC E55";
$resn_exch{"HCC2 E55"}="2HCC E55"; $resn_exch{"HCG1 E55"}="1HCG E55"; $resn_exch{"HCG2 E55"}="2HCG E55";
$resn_exch{"HCF1 E55"}="1HCF E55"; $resn_exch{"HCF2 E55"}="2HCF E55"; $resn_exch{"HCA1 E55"}="1HCA E55";
$resn_exch{"HCA2 E55"}="2HCA E55"; $resn_exch{"HBW1 E55"}="1HBW E55"; $resn_exch{"HBW2 E55"}="2HBW E55";
$resn_exch{"HBH1 E55"}="1HBH E55"; $resn_exch{"HBH2 E55"}="2HBH E55"; $resn_exch{"HBB1 E55"}="1HBB E55";
$resn_exch{"HBB2 E55"}="2HBB E55"; $resn_exch{"HAV1 E55"}="1HAV E55"; $resn_exch{"HAV2 E55"}="2HAV E55";
$resn_exch{"HAD1 E55"}="1HAD E55"; $resn_exch{"HAD2 E55"}="2HAD E55"; $resn_exch{"HAD3 E55"}="3HAD E55";
$resn_exch{"HAF1 E55"}="1HAF E55"; $resn_exch{"HAF2 E55"}="2HAF E55"; $resn_exch{"HAF3 E55"}="3HAF E55";
$resn_exch{" H21 E64"}="1H2  E64"; $resn_exch{" H22 E64"}="2H2  E64"; $resn_exch{" H71 E64"}="1H7  E64";
$resn_exch{" H72 E64"}="2H7  E64"; $resn_exch{" H91 E64"}="1H9  E64"; $resn_exch{" H92 E64"}="2H9  E64";
$resn_exch{" H93 E64"}="3H9  E64"; $resn_exch{"H101 E64"}="1H10 E64"; $resn_exch{"H102 E64"}="2H10 E64";
$resn_exch{"H103 E64"}="3H10 E64"; $resn_exch{"H121 E64"}="1H12 E64"; $resn_exch{"H122 E64"}="2H12 E64";
$resn_exch{"H131 E64"}="1H13 E64"; $resn_exch{"H132 E64"}="2H13 E64"; $resn_exch{"H141 E64"}="1H14 E64";
$resn_exch{"H142 E64"}="2H14 E64"; $resn_exch{"H151 E64"}="1H15 E64"; $resn_exch{"H152 E64"}="2H15 E64";
$resn_exch{"HN41 E64"}="1HN4 E64"; $resn_exch{"HN42 E64"}="2HN4 E64"; $resn_exch{"HN51 E64"}="1HN5 E64";
$resn_exch{"HN52 E64"}="2HN5 E64"; $resn_exch{" H21 E6C"}="1H2  E6C"; $resn_exch{" H22 E6C"}="2H2  E6C";
$resn_exch{"H111 E6C"}="1H11 E6C"; $resn_exch{"H112 E6C"}="2H11 E6C"; $resn_exch{"H121 E6C"}="1H12 E6C";
$resn_exch{"H122 E6C"}="2H12 E6C"; $resn_exch{"H141 E6C"}="1H14 E6C"; $resn_exch{"H142 E6C"}="2H14 E6C";
$resn_exch{"H143 E6C"}="3H14 E6C"; $resn_exch{"H151 E6C"}="1H15 E6C"; $resn_exch{"H152 E6C"}="2H15 E6C";
$resn_exch{"H153 E6C"}="3H15 E6C"; $resn_exch{"H191 E6C"}="1H19 E6C"; $resn_exch{"H192 E6C"}="2H19 E6C";
$resn_exch{"H211 E6C"}="1H21 E6C"; $resn_exch{"H212 E6C"}="2H21 E6C"; $resn_exch{"H213 E6C"}="3H21 E6C";
$resn_exch{"H221 E6C"}="1H22 E6C"; $resn_exch{"H222 E6C"}="2H22 E6C"; $resn_exch{"H223 E6C"}="3H22 E6C";
$resn_exch{"H251 E89"}="1H25 E89"; $resn_exch{"H252 E89"}="2H25 E89"; $resn_exch{"H141 E89"}="1H14 E89";
$resn_exch{"H142 E89"}="2H14 E89"; $resn_exch{"H151 E89"}="1H15 E89"; $resn_exch{"H152 E89"}="2H15 E89";
$resn_exch{"H231 E89"}="1H23 E89"; $resn_exch{"H232 E89"}="2H23 E89"; $resn_exch{"H233 E89"}="3H23 E89";
$resn_exch{"H321 E97"}="1H32 E97"; $resn_exch{"H322 E97"}="2H32 E97"; $resn_exch{"H331 E97"}="1H33 E97";
$resn_exch{"H332 E97"}="2H33 E97"; $resn_exch{"H341 E97"}="1H34 E97"; $resn_exch{"H342 E97"}="2H34 E97";
$resn_exch{"H371 E97"}="1H37 E97"; $resn_exch{"H372 E97"}="2H37 E97"; $resn_exch{"H381 E97"}="1H38 E97";
$resn_exch{"H382 E97"}="2H38 E97"; $resn_exch{"H391 E97"}="1H39 E97"; $resn_exch{"H392 E97"}="2H39 E97";
$resn_exch{"H411 E97"}="1H41 E97"; $resn_exch{"H412 E97"}="2H41 E97"; $resn_exch{"H413 E97"}="3H41 E97";
$resn_exch{"H421 E97"}="1H42 E97"; $resn_exch{"H422 E97"}="2H42 E97"; $resn_exch{"H423 E97"}="3H42 E97";
$resn_exch{"H431 E97"}="1H43 E97"; $resn_exch{"H432 E97"}="2H43 E97"; $resn_exch{"H433 E97"}="3H43 E97";
$resn_exch{"H441 E97"}="1H44 E97"; $resn_exch{"H442 E97"}="2H44 E97"; $resn_exch{"H443 E97"}="3H44 E97";
$resn_exch{" H11 EA2"}="1H1  EA2"; $resn_exch{" H12 EA2"}="2H1  EA2"; $resn_exch{" H21 EA2"}="1H2  EA2";
$resn_exch{" H22 EA2"}="2H2  EA2"; $resn_exch{"HN31 EA2"}="1HN3 EA2"; $resn_exch{"HN32 EA2"}="2HN3 EA2";
$resn_exch{"H5A1 EA5"}="1H5A EA5"; $resn_exch{"H5A2 EA5"}="2H5A EA5"; $resn_exch{"H2A1 EA5"}="1H2A EA5";
$resn_exch{"H2A2 EA5"}="2H2A EA5"; $resn_exch{"H2A3 EA5"}="3H2A EA5"; $resn_exch{"H4A1 EA5"}="1H4A EA5";
$resn_exch{"H4A2 EA5"}="2H4A EA5"; $resn_exch{"HN41 EA5"}="1HN4 EA5"; $resn_exch{"HN42 EA5"}="2HN4 EA5";
$resn_exch{" H1  EA5"}="1H   EA5"; $resn_exch{" H2  EA5"}="2H   EA5"; $resn_exch{" HB1 EA5"}="1HB  EA5";
$resn_exch{" HB2 EA5"}="2HB  EA5"; $resn_exch{" HG1 EA5"}="1HG  EA5"; $resn_exch{" HG2 EA5"}="2HG  EA5";
$resn_exch{" H91 EAA"}="1H9  EAA"; $resn_exch{" H92 EAA"}="2H9  EAA"; $resn_exch{"H101 EAA"}="1H10 EAA";
$resn_exch{"H102 EAA"}="2H10 EAA"; $resn_exch{"H103 EAA"}="3H10 EAA"; $resn_exch{"H111 EAA"}="1H11 EAA";
$resn_exch{"H112 EAA"}="2H11 EAA"; $resn_exch{"H121 EAA"}="1H12 EAA"; $resn_exch{"H122 EAA"}="2H12 EAA";
$resn_exch{" HN1 EAB"}="1HN  EAB"; $resn_exch{" HN2 EAB"}="2HN  EAB"; $resn_exch{" HA2 EAB"}="2HA  EAB";
$resn_exch{" HA3 EAB"}="3HA  EAB"; $resn_exch{" HM3 EAB"}="3HM  EAB"; $resn_exch{" HM2 EAB"}="2HM  EAB";
$resn_exch{" H11 EAD"}="1H1  EAD"; $resn_exch{"H12A EAD"}="2H1  EAD"; $resn_exch{"HN31 EAD"}="1HN3 EAD";
$resn_exch{"HN32 EAD"}="2HN3 EAD"; $resn_exch{"H111 EAD"}="1H11 EAD"; $resn_exch{"H112 EAD"}="2H11 EAD";
$resn_exch{"HN71 EAD"}="1HN7 EAD"; $resn_exch{"HN72 EAD"}="2HN7 EAD"; $resn_exch{"H281 EAD"}="1H28 EAD";
$resn_exch{"H282 EAD"}="2H28 EAD"; $resn_exch{"H291 EAD"}="1H29 EAD"; $resn_exch{"H292 EAD"}="2H29 EAD";
$resn_exch{"H293 EAD"}="3H29 EAD"; $resn_exch{" H61 EAL"}="1H6  EAL"; $resn_exch{" H62 EAL"}="2H6  EAL";
$resn_exch{" H71 EAL"}="1H7  EAL"; $resn_exch{" H72 EAL"}="2H7  EAL"; $resn_exch{"H8C1 EAL"}="1H8C EAL";
$resn_exch{"H8C2 EAL"}="2H8C EAL"; $resn_exch{"H101 EAL"}="1H10 EAL"; $resn_exch{"H102 EAL"}="2H10 EAL";
$resn_exch{"H103 EAL"}="3H10 EAL"; $resn_exch{"H141 EAL"}="1H14 EAL"; $resn_exch{"H142 EAL"}="2H14 EAL";
$resn_exch{"H151 EAL"}="1H15 EAL"; $resn_exch{"H152 EAL"}="2H15 EAL"; $resn_exch{" H11 EAP"}="1H1  EAP";
$resn_exch{" H21 EAP"}="1H2  EAP"; $resn_exch{"HN31 EAP"}="1HN3 EAP"; $resn_exch{"HN32 EAP"}="2HN3 EAP";
$resn_exch{" H21 EB1"}="1H2  EB1"; $resn_exch{"H22A EB1"}="2H2  EB1"; $resn_exch{" H41 EB1"}="1H4  EB1";
$resn_exch{" H42 EB1"}="2H4  EB1"; $resn_exch{" H91 EB1"}="1H9  EB1"; $resn_exch{" H92 EB1"}="2H9  EB1";
$resn_exch{"H111 EB1"}="1H11 EB1"; $resn_exch{"H112 EB1"}="2H11 EB1"; $resn_exch{"H121 EB1"}="1H12 EB1";
$resn_exch{"H122 EB1"}="2H12 EB1"; $resn_exch{"H151 EB1"}="1H15 EB1"; $resn_exch{"H152 EB1"}="2H15 EB1";
$resn_exch{"H161 EB1"}="1H16 EB1"; $resn_exch{"H162 EB1"}="2H16 EB1"; $resn_exch{"H181 EB1"}="1H18 EB1";
$resn_exch{"H182 EB1"}="2H18 EB1"; $resn_exch{"H183 EB1"}="3H18 EB1"; $resn_exch{"H191 EB1"}="1H19 EB1";
$resn_exch{"H192 EB1"}="2H19 EB1"; $resn_exch{"H211 EB1"}="1H21 EB1"; $resn_exch{"H212 EB1"}="2H21 EB1";
$resn_exch{"H213 EB1"}="3H21 EB1"; $resn_exch{"H261 EB1"}="1H26 EB1"; $resn_exch{"H262 EB1"}="2H26 EB1";
$resn_exch{"H281 EB1"}="1H28 EB1"; $resn_exch{"H282 EB1"}="2H28 EB1"; $resn_exch{"H301 EB1"}="1H30 EB1";
$resn_exch{"H302 EB1"}="2H30 EB1"; $resn_exch{"H303 EB1"}="3H30 EB1"; $resn_exch{"H311 EB1"}="1H31 EB1";
$resn_exch{"H312 EB1"}="2H31 EB1"; $resn_exch{"H313 EB1"}="3H31 EB1"; $resn_exch{" H61 EBG"}="1H6  EBG";
$resn_exch{" H62 EBG"}="2H6  EBG"; $resn_exch{" H71 EBG"}="1H7  EBG"; $resn_exch{" H72 EBG"}="2H7  EBG";
$resn_exch{" H81 EBG"}="1H8  EBG"; $resn_exch{" H82 EBG"}="2H8  EBG"; $resn_exch{"H101 EBG"}="1H10 EBG";
$resn_exch{"H102 EBG"}="2H10 EBG"; $resn_exch{" H71 EBP"}="1H7  EBP"; $resn_exch{" H72 EBP"}="2H7  EBP";
$resn_exch{" H81 EBP"}="1H8  EBP"; $resn_exch{" H82 EBP"}="2H8  EBP"; $resn_exch{" H83 EBP"}="3H8  EBP";
$resn_exch{" H91 EBP"}="1H9  EBP"; $resn_exch{" H92 EBP"}="2H9  EBP"; $resn_exch{"H101 EBP"}="1H10 EBP";
$resn_exch{"H102 EBP"}="2H10 EBP"; $resn_exch{"H103 EBP"}="3H10 EBP"; $resn_exch{"H111 EBP"}="1H11 EBP";
$resn_exch{"H112 EBP"}="2H11 EBP"; $resn_exch{"H121 EBP"}="1H12 EBP"; $resn_exch{"H122 EBP"}="2H12 EBP";
$resn_exch{"H123 EBP"}="3H12 EBP"; $resn_exch{"H211 EBS"}="1H21 EBS"; $resn_exch{"H212 EBS"}="2H21 EBS";
$resn_exch{"H221 EBS"}="1H22 EBS"; $resn_exch{"H222 EBS"}="2H22 EBS"; $resn_exch{"H223 EBS"}="3H22 EBS";
$resn_exch{"H241 EBS"}="1H24 EBS"; $resn_exch{"H242 EBS"}="2H24 EBS"; $resn_exch{"H251 EBS"}="1H25 EBS";
$resn_exch{"H252 EBS"}="2H25 EBS"; $resn_exch{"H253 EBS"}="3H25 EBS"; $resn_exch{" H22 EBW"}="2H2  EBW";
$resn_exch{" H21 EBW"}="1H2  EBW"; $resn_exch{" H52 EBW"}="2H5  EBW"; $resn_exch{" H51 EBW"}="1H5  EBW";
$resn_exch{" H62 EBW"}="2H6  EBW"; $resn_exch{" H61 EBW"}="1H6  EBW"; $resn_exch{"H132 EBW"}="2H13 EBW";
$resn_exch{"H131 EBW"}="1H13 EBW"; $resn_exch{"H203 EBW"}="3H20 EBW"; $resn_exch{"H202 EBW"}="2H20 EBW";
$resn_exch{"H201 EBW"}="1H20 EBW"; $resn_exch{"H213 EBW"}="3H21 EBW"; $resn_exch{"H212 EBW"}="2H21 EBW";
$resn_exch{"H211 EBW"}="1H21 EBW"; $resn_exch{"H222 EBW"}="2H22 EBW"; $resn_exch{"H221 EBW"}="1H22 EBW";
$resn_exch{"H242 EBW"}="2H24 EBW"; $resn_exch{"H241 EBW"}="1H24 EBW"; $resn_exch{"H253 EBW"}="3H25 EBW";
$resn_exch{"H252 EBW"}="2H25 EBW"; $resn_exch{"H251 EBW"}="1H25 EBW"; $resn_exch{"H263 EBW"}="3H26 EBW";
$resn_exch{"H262 EBW"}="2H26 EBW"; $resn_exch{"H261 EBW"}="1H26 EBW"; $resn_exch{"H282 EBW"}="2H28 EBW";
$resn_exch{"H281 EBW"}="1H28 EBW"; $resn_exch{"H292 EBW"}="2H29 EBW"; $resn_exch{"H291 EBW"}="1H29 EBW";
$resn_exch{"H9C1 ECA"}="1H9C ECA"; $resn_exch{"H9C2 ECA"}="2H9C ECA"; $resn_exch{"H8C1 ECA"}="1H8C ECA";
$resn_exch{"H8C2 ECA"}="2H8C ECA"; $resn_exch{"H7C1 ECA"}="1H7C ECA"; $resn_exch{"H7C2 ECA"}="2H7C ECA";
$resn_exch{"H211 ECG"}="1H21 ECG"; $resn_exch{"H212 ECG"}="2H21 ECG"; $resn_exch{"H261 ECG"}="1H26 ECG";
$resn_exch{"H262 ECG"}="2H26 ECG"; $resn_exch{"H181 ECG"}="1H18 ECG"; $resn_exch{"H182 ECG"}="2H18 ECG";
$resn_exch{" H11 ECG"}="1H1  ECG"; $resn_exch{" H12 ECG"}="2H1  ECG"; $resn_exch{" H13 ECG"}="3H1  ECG";
$resn_exch{"H141 ECG"}="1H14 ECG"; $resn_exch{"H142 ECG"}="2H14 ECG"; $resn_exch{"H143 ECG"}="3H14 ECG";
$resn_exch{"H8C1 ECN"}="1H8C ECN"; $resn_exch{"H8C2 ECN"}="2H8C ECN"; $resn_exch{"H191 ECN"}="1H19 ECN";
$resn_exch{"H192 ECN"}="2H19 ECN"; $resn_exch{"HC71 ECO"}="1HC7 ECO"; $resn_exch{"HC72 ECO"}="2HC7 ECO";
$resn_exch{"H111 ECO"}="1H11 ECO"; $resn_exch{"H112 ECO"}="2H11 ECO"; $resn_exch{"H121 ECO"}="1H12 ECO";
$resn_exch{"H122 ECO"}="2H12 ECO"; $resn_exch{"H151 ECO"}="1H15 ECO"; $resn_exch{"H152 ECO"}="2H15 ECO";
$resn_exch{"H161 ECO"}="1H16 ECO"; $resn_exch{"H162 ECO"}="2H16 ECO"; $resn_exch{"H17O ECO"}="OH17 ECO";
$resn_exch{"H181 ECO"}="1H18 ECO"; $resn_exch{"H182 ECO"}="2H18 ECO"; $resn_exch{"H183 ECO"}="3H18 ECO";
$resn_exch{"H191 ECO"}="1H19 ECO"; $resn_exch{"H192 ECO"}="2H19 ECO"; $resn_exch{" HN1 ECQ"}="1HN  ECQ";
$resn_exch{" HN2 ECQ"}="2HN  ECQ"; $resn_exch{"HMK1 ECQ"}="1HMK ECQ"; $resn_exch{"HMK2 ECQ"}="2HMK ECQ";
$resn_exch{"HMK3 ECQ"}="3HMK ECQ"; $resn_exch{" HB1 ECQ"}="1HB  ECQ"; $resn_exch{" HB2 ECQ"}="2HB  ECQ";
$resn_exch{"HAE1 ECQ"}="1HAE ECQ"; $resn_exch{"HAE2 ECQ"}="2HAE ECQ"; $resn_exch{"HD11 ECQ"}="1HD1 ECQ";
$resn_exch{"HD12 ECQ"}="2HD1 ECQ"; $resn_exch{"H111 ECS"}="1H11 ECS"; $resn_exch{"H112 ECS"}="2H11 ECS";
$resn_exch{"H121 ECS"}="1H12 ECS"; $resn_exch{"H122 ECS"}="2H12 ECS"; $resn_exch{"H181 ECS"}="1H18 ECS";
$resn_exch{"H182 ECS"}="2H18 ECS"; $resn_exch{"H183 ECS"}="3H18 ECS"; $resn_exch{"H151 ECS"}="1H15 ECS";
$resn_exch{"H152 ECS"}="2H15 ECS"; $resn_exch{"H161 ECS"}="1H16 ECS"; $resn_exch{"H162 ECS"}="2H16 ECS";
$resn_exch{"H6C1 ECS"}="1H6C ECS"; $resn_exch{"H6C2 ECS"}="2H6C ECS"; $resn_exch{"H7C1 ECS"}="1H7C ECS";
$resn_exch{"H7C2 ECS"}="2H7C ECS"; $resn_exch{"H11A ECT"}="1H1  ECT"; $resn_exch{"H12A ECT"}="2H1  ECT";
$resn_exch{" H61 ECT"}="1H6  ECT"; $resn_exch{" H62 ECT"}="2H6  ECT"; $resn_exch{"H181 ECT"}="1H18 ECT";
$resn_exch{"H182 ECT"}="2H18 ECT"; $resn_exch{"H183 ECT"}="3H18 ECT"; $resn_exch{"H231 ECT"}="1H23 ECT";
$resn_exch{"H232 ECT"}="2H23 ECT"; $resn_exch{"H321 ECT"}="1H32 ECT"; $resn_exch{"H322 ECT"}="2H32 ECT";
$resn_exch{"H323 ECT"}="3H32 ECT"; $resn_exch{"H371 ECT"}="1H37 ECT"; $resn_exch{"H372 ECT"}="2H37 ECT";
$resn_exch{"H381 ECT"}="1H38 ECT"; $resn_exch{"H382 ECT"}="2H38 ECT"; $resn_exch{"H431 ECT"}="1H43 ECT";
$resn_exch{"H432 ECT"}="2H43 ECT"; $resn_exch{"H433 ECT"}="3H43 ECT"; $resn_exch{"H491 ECT"}="1H49 ECT";
$resn_exch{"H492 ECT"}="2H49 ECT"; $resn_exch{"H493 ECT"}="3H49 ECT"; $resn_exch{"H511 ECT"}="1H51 ECT";
$resn_exch{"H512 ECT"}="2H51 ECT"; $resn_exch{"H513 ECT"}="3H51 ECT"; $resn_exch{"H541 ECT"}="1H54 ECT";
$resn_exch{"H542 ECT"}="2H54 ECT"; $resn_exch{"HOP2 EDA"}="2HOP EDA"; $resn_exch{"HOP3 EDA"}="3HOP EDA";
$resn_exch{" H2' EDA"}="1H2* EDA"; $resn_exch{"H2'' EDA"}="2H2* EDA"; $resn_exch{" H5' EDA"}="1H5* EDA";
$resn_exch{"H5'' EDA"}="2H5* EDA"; $resn_exch{" H4' EDA"}=" H4* EDA"; $resn_exch{" H1' EDA"}=" H1* EDA";
$resn_exch{" H3' EDA"}=" H3* EDA"; $resn_exch{"HO3' EDA"}="H3T  EDA"; $resn_exch{"HOP2 EDC"}="2HOP EDC";
$resn_exch{"HOP3 EDC"}="3HOP EDC"; $resn_exch{" H5' EDC"}="1H5* EDC"; $resn_exch{"H5'' EDC"}="2H5* EDC";
$resn_exch{" H4' EDC"}=" H4* EDC"; $resn_exch{" H1' EDC"}=" H1* EDC"; $resn_exch{" H2' EDC"}="1H2* EDC";
$resn_exch{"H2'' EDC"}="2H2* EDC"; $resn_exch{" H3' EDC"}=" H3* EDC"; $resn_exch{"HO3' EDC"}="*HO3 EDC";
$resn_exch{" H21 EDE"}="1H2  EDE"; $resn_exch{" H22 EDE"}="2H2  EDE"; $resn_exch{"H101 EDE"}="1H10 EDE";
$resn_exch{"H102 EDE"}="2H10 EDE"; $resn_exch{"H111 EDE"}="1H11 EDE"; $resn_exch{"H112 EDE"}="2H11 EDE";
$resn_exch{"H121 EDE"}="1H12 EDE"; $resn_exch{"H122 EDE"}="2H12 EDE"; $resn_exch{"H151 EDE"}="1H15 EDE";
$resn_exch{"H152 EDE"}="2H15 EDE"; $resn_exch{"H181 EDE"}="1H18 EDE"; $resn_exch{"H182 EDE"}="2H18 EDE";
$resn_exch{"H211 EDE"}="1H21 EDE"; $resn_exch{"H212 EDE"}="2H21 EDE"; $resn_exch{"H221 EDE"}="1H22 EDE";
$resn_exch{"H222 EDE"}="2H22 EDE"; $resn_exch{"H231 EDE"}="1H23 EDE"; $resn_exch{"H232 EDE"}="2H23 EDE";
$resn_exch{"H251 EDE"}="1H25 EDE"; $resn_exch{"H252 EDE"}="2H25 EDE"; $resn_exch{"H261 EDE"}="1H26 EDE";
$resn_exch{"H262 EDE"}="2H26 EDE"; $resn_exch{"H271 EDE"}="1H27 EDE"; $resn_exch{"H272 EDE"}="2H27 EDE";
$resn_exch{"H281 EDE"}="1H28 EDE"; $resn_exch{"H282 EDE"}="2H28 EDE"; $resn_exch{"H311 EDE"}="1H31 EDE";
$resn_exch{"H312 EDE"}="2H31 EDE"; $resn_exch{"H331 EDE"}="1H33 EDE"; $resn_exch{"H332 EDE"}="2H33 EDE";
$resn_exch{"H451 EDE"}="1H45 EDE"; $resn_exch{"H452 EDE"}="2H45 EDE"; $resn_exch{"H461 EDE"}="1H46 EDE";
$resn_exch{"H462 EDE"}="2H46 EDE"; $resn_exch{"H501 EDE"}="1H50 EDE"; $resn_exch{"H502 EDE"}="2H50 EDE";
$resn_exch{"H551 EDE"}="1H55 EDE"; $resn_exch{"H552 EDE"}="2H55 EDE"; $resn_exch{" H11 EDO"}="1H1  EDO";
$resn_exch{" H12 EDO"}="2H1  EDO"; $resn_exch{" H21 EDO"}="1H2  EDO"; $resn_exch{" H22 EDO"}="2H2  EDO";
$resn_exch{" H71 EDR"}="1H7  EDR"; $resn_exch{" H72 EDR"}="2H7  EDR"; $resn_exch{" H73 EDR"}="3H7  EDR";
$resn_exch{" H81 EDR"}="1H8  EDR"; $resn_exch{" H82 EDR"}="2H8  EDR"; $resn_exch{" H83 EDR"}="3H8  EDR";
$resn_exch{" H91 EDR"}="1H9  EDR"; $resn_exch{" H92 EDR"}="2H9  EDR"; $resn_exch{"H101 EDR"}="1H10 EDR";
$resn_exch{"H102 EDR"}="2H10 EDR"; $resn_exch{"H103 EDR"}="3H10 EDR"; $resn_exch{"H041 EDT"}="1H04 EDT";
$resn_exch{"H042 EDT"}="2H04 EDT"; $resn_exch{"H021 EDT"}="1H02 EDT"; $resn_exch{"H022 EDT"}="2H02 EDT";
$resn_exch{"H061 EDT"}="1H06 EDT"; $resn_exch{"H062 EDT"}="2H06 EDT"; $resn_exch{"H071 EDT"}="1H07 EDT";
$resn_exch{"H072 EDT"}="2H07 EDT"; $resn_exch{"H091 EDT"}="1H09 EDT"; $resn_exch{"H092 EDT"}="2H09 EDT";
$resn_exch{"H111 EDT"}="1H11 EDT"; $resn_exch{"H112 EDT"}="2H11 EDT"; $resn_exch{"HN3U EEB"}="UHN3 EEB";
$resn_exch{" H5U EEB"}=" UH5 EEB"; $resn_exch{" H6U EEB"}=" UH6 EEB"; $resn_exch{" H1D EEB"}="UH1* EEB";
$resn_exch{" H2D EEB"}="UH2* EEB"; $resn_exch{"HO2U EEB"}="UHO2 EEB"; $resn_exch{" H3D EEB"}="UH3* EEB";
$resn_exch{" H4D EEB"}="UH4* EEB"; $resn_exch{"HO3U EEB"}="UHO3 EEB"; $resn_exch{"H5'1 EEB"}="1H5* EEB";
$resn_exch{"H5'2 EEB"}="2H5* EEB"; $resn_exch{"HOA2 EEB"}="2HOA EEB"; $resn_exch{"HOB2 EEB"}="2HOB EEB";
$resn_exch{" H61 EEB"}="1H6  EEB"; $resn_exch{" H62 EEB"}="2H6  EEB"; $resn_exch{" H81 EEB"}="1H8  EEB";
$resn_exch{" H82 EEB"}="2H8  EEB"; $resn_exch{" H83 EEB"}="3H8  EEB"; $resn_exch{" H2E EEB"}=" EH2 EEB";
$resn_exch{"HE31 EEB"}="1HE3 EEB"; $resn_exch{"HE32 EEB"}="2HE3 EEB"; $resn_exch{"HE41 EEB"}="1HE4 EEB";
$resn_exch{"HE42 EEB"}="2HE4 EEB"; $resn_exch{"HE43 EEB"}="3HE4 EEB"; $resn_exch{"HO2E EEB"}="EHO2 EEB";
$resn_exch{" H61 EED"}="1H6  EED"; $resn_exch{" H62 EED"}="2H6  EED"; $resn_exch{" H71 EED"}="1H7  EED";
$resn_exch{" H72 EED"}="2H7  EED"; $resn_exch{"H151 EED"}="1H15 EED"; $resn_exch{"H152 EED"}="2H15 EED";
$resn_exch{"H161 EED"}="1H16 EED"; $resn_exch{"H162 EED"}="2H16 EED"; $resn_exch{"H181 EED"}="1H18 EED";
$resn_exch{"H182 EED"}="2H18 EED"; $resn_exch{"H183 EED"}="3H18 EED"; $resn_exch{"H121 EED"}="1H12 EED";
$resn_exch{"H122 EED"}="2H12 EED"; $resn_exch{"H191 EED"}="1H19 EED"; $resn_exch{"H192 EED"}="2H19 EED";
$resn_exch{"H211 EED"}="1H21 EED"; $resn_exch{"H212 EED"}="2H21 EED"; $resn_exch{"H213 EED"}="3H21 EED";
$resn_exch{" H21 EEE"}="1H2  EEE"; $resn_exch{" H22 EEE"}="2H2  EEE"; $resn_exch{" H23 EEE"}="3H2  EEE";
$resn_exch{" H31 EEE"}="1H3  EEE"; $resn_exch{" H32 EEE"}="2H3  EEE"; $resn_exch{" H41 EEE"}="1H4  EEE";
$resn_exch{" H42 EEE"}="2H4  EEE"; $resn_exch{" H43 EEE"}="3H4  EEE"; $resn_exch{" H   EFC"}="1HN  EFC";
$resn_exch{" H2  EFC"}="2HN  EFC"; $resn_exch{" HB2 EFC"}="1HB  EFC"; $resn_exch{" HB3 EFC"}="2HB  EFC";
$resn_exch{" H11 EFC"}="1H1  EFC"; $resn_exch{" H12 EFC"}="2H1  EFC"; $resn_exch{" H21 EFC"}="1H2  EFC";
$resn_exch{" H22 EFC"}="2H2  EFC"; $resn_exch{"H121 EFS"}="1H12 EFS"; $resn_exch{"H122 EFS"}="2H12 EFS";
$resn_exch{"H123 EFS"}="3H12 EFS"; $resn_exch{"H111 EFS"}="1H11 EFS"; $resn_exch{"H112 EFS"}="2H11 EFS";
$resn_exch{"H101 EFZ"}="1H10 EFZ"; $resn_exch{"H111 EFZ"}="1H11 EFZ"; $resn_exch{"H112 EFZ"}="2H11 EFZ";
$resn_exch{"H121 EFZ"}="1H12 EFZ"; $resn_exch{"H122 EFZ"}="2H12 EFZ"; $resn_exch{"HN11 EG1"}="1HN1 EG1";
$resn_exch{"HN12 EG1"}="2HN1 EG1"; $resn_exch{" H81 EG1"}="1H8  EG1"; $resn_exch{" H82 EG1"}="2H8  EG1";
$resn_exch{" H91 EG1"}="1H9  EG1"; $resn_exch{" H92 EG1"}="2H9  EG1"; $resn_exch{"H101 EG1"}="1H10 EG1";
$resn_exch{"H102 EG1"}="2H10 EG1"; $resn_exch{"H111 EG1"}="1H11 EG1"; $resn_exch{"H112 EG1"}="2H11 EG1";
$resn_exch{"H121 EG1"}="1H12 EG1"; $resn_exch{"H122 EG1"}="2H12 EG1"; $resn_exch{"H131 EG1"}="1H13 EG1";
$resn_exch{"H132 EG1"}="2H13 EG1"; $resn_exch{"H151 EG1"}="1H15 EG1"; $resn_exch{"H152 EG1"}="2H15 EG1";
$resn_exch{"HN41 EG1"}="1HN4 EG1"; $resn_exch{"HN42 EG1"}="2HN4 EG1"; $resn_exch{"HN11 EG2"}="1HN1 EG2";
$resn_exch{"HN12 EG2"}="2HN1 EG2"; $resn_exch{" H81 EG2"}="1H8  EG2"; $resn_exch{" H82 EG2"}="2H8  EG2";
$resn_exch{" H91 EG2"}="1H9  EG2"; $resn_exch{" H92 EG2"}="2H9  EG2"; $resn_exch{"H101 EG2"}="1H10 EG2";
$resn_exch{"H102 EG2"}="2H10 EG2"; $resn_exch{"H111 EG2"}="1H11 EG2"; $resn_exch{"H112 EG2"}="2H11 EG2";
$resn_exch{"H121 EG2"}="1H12 EG2"; $resn_exch{"H122 EG2"}="2H12 EG2"; $resn_exch{"H131 EG2"}="1H13 EG2";
$resn_exch{"H132 EG2"}="2H13 EG2"; $resn_exch{"HN31 EG2"}="1HN3 EG2"; $resn_exch{"HN32 EG2"}="2HN3 EG2";
$resn_exch{"HN11 EG3"}="1HN1 EG3"; $resn_exch{"HN12 EG3"}="2HN1 EG3"; $resn_exch{" H81 EG3"}="1H8  EG3";
$resn_exch{" H82 EG3"}="2H8  EG3"; $resn_exch{" H91 EG3"}="1H9  EG3"; $resn_exch{" H92 EG3"}="2H9  EG3";
$resn_exch{"H101 EG3"}="1H10 EG3"; $resn_exch{"H102 EG3"}="2H10 EG3"; $resn_exch{"H111 EG3"}="1H11 EG3";
$resn_exch{"H112 EG3"}="2H11 EG3"; $resn_exch{"H121 EG3"}="1H12 EG3"; $resn_exch{"H122 EG3"}="2H12 EG3";
$resn_exch{"H131 EG3"}="1H13 EG3"; $resn_exch{"H132 EG3"}="2H13 EG3"; $resn_exch{"HN41 EG3"}="1HN4 EG3";
$resn_exch{"HN42 EG3"}="2HN4 EG3"; $resn_exch{"H161 EG3"}="1H16 EG3"; $resn_exch{"H162 EG3"}="2H16 EG3";
$resn_exch{" H21 EGC"}="1H2  EGC"; $resn_exch{" H22 EGC"}="2H2  EGC"; $resn_exch{"H31A EGC"}="1H3  EGC";
$resn_exch{" H32 EGC"}="2H3  EGC"; $resn_exch{" H51 EGC"}="1H5  EGC"; $resn_exch{" H52 EGC"}="2H5  EGC";
$resn_exch{" H61 EGC"}="1H6  EGC"; $resn_exch{" H62 EGC"}="2H6  EGC"; $resn_exch{" H81 EGC"}="1H8  EGC";
$resn_exch{" H82 EGC"}="2H8  EGC"; $resn_exch{" H91 EGC"}="1H9  EGC"; $resn_exch{" H92 EGC"}="2H9  EGC";
$resn_exch{"H111 EGC"}="1H11 EGC"; $resn_exch{"H112 EGC"}="2H11 EGC"; $resn_exch{"H121 EGC"}="1H12 EGC";
$resn_exch{"H122 EGC"}="2H12 EGC"; $resn_exch{"H141 EGC"}="1H14 EGC"; $resn_exch{"H142 EGC"}="2H14 EGC";
$resn_exch{"H151 EGC"}="1H15 EGC"; $resn_exch{"H152 EGC"}="2H15 EGC"; $resn_exch{"H171 EGC"}="1H17 EGC";
$resn_exch{"H172 EGC"}="2H17 EGC"; $resn_exch{"H181 EGC"}="1H18 EGC"; $resn_exch{"H182 EGC"}="2H18 EGC";
$resn_exch{"H201 EGC"}="1H20 EGC"; $resn_exch{"H202 EGC"}="2H20 EGC"; $resn_exch{"H211 EGC"}="1H21 EGC";
$resn_exch{"H212 EGC"}="2H21 EGC"; $resn_exch{"H231 EGC"}="1H23 EGC"; $resn_exch{"H232 EGC"}="2H23 EGC";
$resn_exch{"H241 EGC"}="1H24 EGC"; $resn_exch{"H242 EGC"}="2H24 EGC"; $resn_exch{"H261 EGC"}="1H26 EGC";
$resn_exch{"H262 EGC"}="2H26 EGC"; $resn_exch{"H271 EGC"}="1H27 EGC"; $resn_exch{"H272 EGC"}="2H27 EGC";
$resn_exch{"H361 EGC"}="1H36 EGC"; $resn_exch{"H362 EGC"}="2H36 EGC"; $resn_exch{"H363 EGC"}="3H36 EGC";
$resn_exch{"H371 EGC"}="1H37 EGC"; $resn_exch{"H372 EGC"}="2H37 EGC"; $resn_exch{"H373 EGC"}="3H37 EGC";
$resn_exch{"H381 EGC"}="1H38 EGC"; $resn_exch{"H382 EGC"}="2H38 EGC"; $resn_exch{"H401 EGC"}="1H40 EGC";
$resn_exch{"H402 EGC"}="2H40 EGC"; $resn_exch{"H403 EGC"}="3H40 EGC"; $resn_exch{"H411 EGC"}="1H41 EGC";
$resn_exch{"H412 EGC"}="2H41 EGC"; $resn_exch{"H413 EGC"}="3H41 EGC"; $resn_exch{"H421 EGC"}="1H42 EGC";
$resn_exch{"H422 EGC"}="2H42 EGC"; $resn_exch{"H423 EGC"}="3H42 EGC"; $resn_exch{" HG1 EGD"}="1HG  EGD";
$resn_exch{" HG2 EGD"}="2HG  EGD"; $resn_exch{" HG3 EGD"}="3HG  EGD"; $resn_exch{" HD1 EGD"}="1HD  EGD";
$resn_exch{" HD2 EGD"}="2HD  EGD"; $resn_exch{"HH21 EGD"}="1HH2 EGD"; $resn_exch{"HH22 EGD"}="2HH2 EGD";
$resn_exch{" H11 EGL"}="1H1  EGL"; $resn_exch{" H12 EGL"}="2H1  EGL"; $resn_exch{" H21 EGL"}="1H2  EGL";
$resn_exch{" H22 EGL"}="2H2  EGL"; $resn_exch{"HC41 EGT"}="1HC4 EGT"; $resn_exch{"HC42 EGT"}="2HC4 EGT";
$resn_exch{" HA1 EH5"}="1HA  EH5"; $resn_exch{" HB1 EH5"}="1HB  EH5"; $resn_exch{" HB2 EH5"}="2HB  EH5";
$resn_exch{"HD11 EH5"}="1HD1 EH5"; $resn_exch{"HD21 EH5"}="1HD2 EH5"; $resn_exch{"HE11 EH5"}="1HE1 EH5";
$resn_exch{"HE21 EH5"}="1HE2 EH5"; $resn_exch{" HZ1 EH5"}="1HZ  EH5"; $resn_exch{"HN21 EH5"}="1HN2 EH5";
$resn_exch{" H11 EH5"}="1H1  EH5"; $resn_exch{"HB11 EH5"}="1HB1 EH5"; $resn_exch{"HB12 EH5"}="2HB1 EH5";
$resn_exch{"HO11 EH5"}="1HO1 EH5"; $resn_exch{" H31 EH5"}="1H3  EH5"; $resn_exch{" H32 EH5"}="2H3  EH5";
$resn_exch{" H41 EH5"}="1H4  EH5"; $resn_exch{" H42 EH5"}="2H4  EH5"; $resn_exch{" H61 EH5"}="1H6  EH5";
$resn_exch{" H62 EH5"}="2H6  EH5"; $resn_exch{" H71 EH5"}="1H7  EH5"; $resn_exch{" H72 EH5"}="2H7  EH5";
$resn_exch{"H101 EH5"}="1H10 EH5"; $resn_exch{"H221 EH5"}="1H22 EH5"; $resn_exch{"H321 EH5"}="1H32 EH5";
$resn_exch{"H261 EH5"}="1H26 EH5"; $resn_exch{"H111 EH5"}="1H11 EH5"; $resn_exch{"H112 EH5"}="2H11 EH5";
$resn_exch{"H231 EH5"}="1H23 EH5"; $resn_exch{"H232 EH5"}="2H23 EH5"; $resn_exch{"H151 EH5"}="1H15 EH5";
$resn_exch{"H191 EH5"}="1H19 EH5"; $resn_exch{"H331 EH5"}="1H33 EH5"; $resn_exch{"H332 EH5"}="2H33 EH5";
$resn_exch{"H271 EH5"}="1H27 EH5"; $resn_exch{" H81 EH5"}="1H8  EH5"; $resn_exch{" H82 EH5"}="2H8  EH5";
$resn_exch{"H161 EH5"}="1H16 EH5"; $resn_exch{"H341 EH5"}="1H34 EH5"; $resn_exch{"H311 EH5"}="1H31 EH5";
$resn_exch{"H312 EH5"}="2H31 EH5"; $resn_exch{"H313 EH5"}="3H31 EH5"; $resn_exch{"H371 EH5"}="1H37 EH5";
$resn_exch{"H372 EH5"}="2H37 EH5"; $resn_exch{"H373 EH5"}="3H37 EH5"; $resn_exch{"H361 EH5"}="1H36 EH5";
$resn_exch{"H362 EH5"}="2H36 EH5"; $resn_exch{"H131 EH5"}="1H13 EH5"; $resn_exch{"H251 EH5"}="1H25 EH5";
$resn_exch{"H171 EH5"}="1H17 EH5"; $resn_exch{"H351 EH5"}="1H35 EH5"; $resn_exch{"H291 EH5"}="1H29 EH5";
$resn_exch{"HBK1 EHA"}="1HBK EHA"; $resn_exch{"HBK2 EHA"}="2HBK EHA"; $resn_exch{"HBL1 EHA"}="1HBL EHA";
$resn_exch{"HBL2 EHA"}="2HBL EHA"; $resn_exch{"HBM1 EHA"}="1HBM EHA"; $resn_exch{"HBM2 EHA"}="2HBM EHA";
$resn_exch{"HBM3 EHA"}="3HBM EHA"; $resn_exch{"HBI1 EHA"}="1HBI EHA"; $resn_exch{"HBI2 EHA"}="2HBI EHA";
$resn_exch{"HBH1 EHA"}="1HBH EHA"; $resn_exch{"HBH2 EHA"}="2HBH EHA"; $resn_exch{"HBG1 EHA"}="1HBG EHA";
$resn_exch{"HBG2 EHA"}="2HBG EHA"; $resn_exch{" HA1 EHA"}="1HA  EHA"; $resn_exch{" HA2 EHA"}="2HA  EHA";
$resn_exch{"H191 EHD"}="1H19 EHD"; $resn_exch{"H192 EHD"}="2H19 EHD"; $resn_exch{"H251 EHD"}="1H25 EHD";
$resn_exch{"H252 EHD"}="2H25 EHD"; $resn_exch{"H311 EHD"}="1H31 EHD"; $resn_exch{"H312 EHD"}="2H31 EHD";
$resn_exch{"H313 EHD"}="3H31 EHD"; $resn_exch{"H111 EHD"}="1H11 EHD"; $resn_exch{"H112 EHD"}="2H11 EHD";
$resn_exch{" H41 EHM"}="1H4  EHM"; $resn_exch{" H42 EHM"}="2H4  EHM"; $resn_exch{" HN1 EHP"}="1HN  EHP";
$resn_exch{" HN2 EHP"}="2HN  EHP"; $resn_exch{" HB1 EHP"}="1HB  EHP"; $resn_exch{" HB2 EHP"}="2HB  EHP";
$resn_exch{"HAA1 EI1"}="1HAA EI1"; $resn_exch{"HAA2 EI1"}="2HAA EI1"; $resn_exch{"HAA3 EI1"}="3HAA EI1";
$resn_exch{"HAK1 EI1"}="1HAK EI1"; $resn_exch{"HAK2 EI1"}="2HAK EI1"; $resn_exch{" H21 EIC"}="1H2  EIC";
$resn_exch{" H22 EIC"}="2H2  EIC"; $resn_exch{" H31 EIC"}="1H3  EIC"; $resn_exch{" H32 EIC"}="2H3  EIC";
$resn_exch{" H41 EIC"}="1H4  EIC"; $resn_exch{" H42 EIC"}="2H4  EIC"; $resn_exch{" H51 EIC"}="1H5  EIC";
$resn_exch{" H52 EIC"}="2H5  EIC"; $resn_exch{" H61 EIC"}="1H6  EIC"; $resn_exch{" H62 EIC"}="2H6  EIC";
$resn_exch{" H71 EIC"}="1H7  EIC"; $resn_exch{" H72 EIC"}="2H7  EIC"; $resn_exch{" H81 EIC"}="1H8  EIC";
$resn_exch{" H82 EIC"}="2H8  EIC"; $resn_exch{" H91 EIC"}="1H9  EIC"; $resn_exch{"H1O1 EIC"}="1H1O EIC";
$resn_exch{"H111 EIC"}="1H11 EIC"; $resn_exch{"H112 EIC"}="2H11 EIC"; $resn_exch{"H121 EIC"}="1H12 EIC";
$resn_exch{"H131 EIC"}="1H13 EIC"; $resn_exch{"H141 EIC"}="1H14 EIC"; $resn_exch{"H142 EIC"}="2H14 EIC";
$resn_exch{"H151 EIC"}="1H15 EIC"; $resn_exch{"H152 EIC"}="2H15 EIC"; $resn_exch{"H161 EIC"}="1H16 EIC";
$resn_exch{"H162 EIC"}="2H16 EIC"; $resn_exch{"H171 EIC"}="1H17 EIC"; $resn_exch{"H172 EIC"}="2H17 EIC";
$resn_exch{"H181 EIC"}="1H18 EIC"; $resn_exch{"H182 EIC"}="2H18 EIC"; $resn_exch{"H183 EIC"}="3H18 EIC";
$resn_exch{" H31 EIN"}="1H3  EIN"; $resn_exch{" H32 EIN"}="2H3  EIN"; $resn_exch{" H33 EIN"}="3H3  EIN";
$resn_exch{"H121 EIN"}="1H12 EIN"; $resn_exch{"H122 EIN"}="2H12 EIN"; $resn_exch{"H123 EIN"}="3H12 EIN";
$resn_exch{"H131 EIN"}="1H13 EIN"; $resn_exch{"H132 EIN"}="2H13 EIN"; $resn_exch{"H133 EIN"}="3H13 EIN";
$resn_exch{"H101 EIP"}="1H10 EIP"; $resn_exch{"H102 EIP"}="2H10 EIP"; $resn_exch{"H111 EIP"}="1H11 EIP";
$resn_exch{"H112 EIP"}="2H11 EIP"; $resn_exch{"H131 EIP"}="1H13 EIP"; $resn_exch{"H132 EIP"}="2H13 EIP";
$resn_exch{"H141 EIP"}="1H14 EIP"; $resn_exch{"H142 EIP"}="2H14 EIP"; $resn_exch{"H143 EIP"}="3H14 EIP";
$resn_exch{"HOP3 EIT"}="3HOP EIT"; $resn_exch{"HOP2 EIT"}="2HOP EIT"; $resn_exch{" H5' EIT"}="1H5* EIT";
$resn_exch{"H5'' EIT"}="2H5* EIT"; $resn_exch{" H4' EIT"}=" H4* EIT"; $resn_exch{" H1' EIT"}=" H1* EIT";
$resn_exch{" H71 EIT"}="1H5M EIT"; $resn_exch{" H72 EIT"}="2H5M EIT"; $resn_exch{" H73 EIT"}="3H5M EIT";
$resn_exch{" H2' EIT"}=" H2* EIT"; $resn_exch{"HB'1 EIT"}="1HB' EIT"; $resn_exch{"HB'2 EIT"}="2HB' EIT";
$resn_exch{"HC'1 EIT"}="1HC' EIT"; $resn_exch{"HC'2 EIT"}="2HC' EIT"; $resn_exch{" H3' EIT"}=" H3* EIT";
$resn_exch{"HO3' EIT"}="H3T  EIT"; $resn_exch{"HC61 EJT"}="1HC6 EJT"; $resn_exch{"HC62 EJT"}="2HC6 EJT";
$resn_exch{"HX61 EJT"}="1HX6 EJT"; $resn_exch{"HX62 EJT"}="2HX6 EJT"; $resn_exch{"HCB1 EJT"}="1HCB EJT";
$resn_exch{"HCB2 EJT"}="2HCB EJT"; $resn_exch{"HCG1 EJT"}="1HCG EJT"; $resn_exch{"HCG2 EJT"}="2HCG EJT";
$resn_exch{"HCDA EJT"}="1HCD EJT"; $resn_exch{"HCDB EJT"}="2HCD EJT"; $resn_exch{"HCE1 EJT"}="1HCE EJT";
$resn_exch{"HCE2 EJT"}="2HCE EJT"; $resn_exch{"HXG1 EJT"}="1HXG EJT"; $resn_exch{"HXG2 EJT"}="2HXG EJT";
$resn_exch{"HXD1 EJT"}="1HXD EJT"; $resn_exch{"HXD2 EJT"}="2HXD EJT"; $resn_exch{"HXE1 EJT"}="1HXE EJT";
$resn_exch{"HXE2 EJT"}="2HXE EJT"; $resn_exch{" HN1 EKE"}="1HN  EKE"; $resn_exch{" HN2 EKE"}="2HN  EKE";
$resn_exch{" HB1 EKE"}="1HB  EKE"; $resn_exch{" HB2 EKE"}="2HB  EKE"; $resn_exch{" HG1 EKE"}="1HG  EKE";
$resn_exch{" HG2 EKE"}="2HG  EKE"; $resn_exch{"HMK1 EKE"}="1HMK EKE"; $resn_exch{"HMK2 EKE"}="2HMK EKE";
$resn_exch{"HMK3 EKE"}="3HMK EKE"; $resn_exch{"H111  EL"}="1H11  EL"; $resn_exch{"H112  EL"}="2H11  EL";
$resn_exch{"H113  EL"}="3H11  EL"; $resn_exch{"H151  EL"}="1H15  EL"; $resn_exch{"H152  EL"}="2H15  EL";
$resn_exch{"H153  EL"}="3H15  EL"; $resn_exch{" HA1 ELA"}="1HA  ELA"; $resn_exch{" HA2 ELA"}="2HA  ELA";
$resn_exch{" H31 ELA"}="1H3  ELA"; $resn_exch{" H32 ELA"}="2H3  ELA"; $resn_exch{" H41 ELA"}="1H4  ELA";
$resn_exch{" H42 ELA"}="2H4  ELA"; $resn_exch{" H51 ELA"}="1H5  ELA"; $resn_exch{" H52 ELA"}="2H5  ELA";
$resn_exch{" H61 ELA"}="1H6  ELA"; $resn_exch{" H62 ELA"}="2H6  ELA"; $resn_exch{" H71 ELA"}="1H7  ELA";
$resn_exch{" H72 ELA"}="2H7  ELA"; $resn_exch{" H81 ELA"}="1H8  ELA"; $resn_exch{" H82 ELA"}="2H8  ELA";
$resn_exch{"H111 ELA"}="1H11 ELA"; $resn_exch{"H112 ELA"}="2H11 ELA"; $resn_exch{"H121 ELA"}="1H12 ELA";
$resn_exch{"H122 ELA"}="2H12 ELA"; $resn_exch{"H131 ELA"}="1H13 ELA"; $resn_exch{"H132 ELA"}="2H13 ELA";
$resn_exch{"H141 ELA"}="1H14 ELA"; $resn_exch{"H142 ELA"}="2H14 ELA"; $resn_exch{"H151 ELA"}="1H15 ELA";
$resn_exch{"H152 ELA"}="2H15 ELA"; $resn_exch{"H161 ELA"}="1H16 ELA"; $resn_exch{"H162 ELA"}="2H16 ELA";
$resn_exch{"H171 ELA"}="1H17 ELA"; $resn_exch{"H172 ELA"}="2H17 ELA"; $resn_exch{"H181 ELA"}="1H18 ELA";
$resn_exch{"H182 ELA"}="2H18 ELA"; $resn_exch{"H183 ELA"}="3H18 ELA"; $resn_exch{"HN11 ELD"}="1HN1 ELD";
$resn_exch{"HN12 ELD"}="2HN1 ELD"; $resn_exch{" H21 ELD"}="1H2  ELD"; $resn_exch{" H22 ELD"}="2H2  ELD";
$resn_exch{" H31 ELD"}="1H3  ELD"; $resn_exch{" H32 ELD"}="2H3  ELD"; $resn_exch{" H41 ELD"}="1H4  ELD";
$resn_exch{" H42 ELD"}="2H4  ELD"; $resn_exch{" H51 ELD"}="1H5  ELD"; $resn_exch{" H52 ELD"}="2H5  ELD";
$resn_exch{" H61 ELD"}="1H6  ELD"; $resn_exch{" H62 ELD"}="2H6  ELD"; $resn_exch{" H71 ELD"}="1H7  ELD";
$resn_exch{" H72 ELD"}="2H7  ELD"; $resn_exch{" H81 ELD"}="1H8  ELD"; $resn_exch{" H82 ELD"}="2H8  ELD";
$resn_exch{"H111 ELD"}="1H11 ELD"; $resn_exch{"H112 ELD"}="2H11 ELD"; $resn_exch{"H121 ELD"}="1H12 ELD";
$resn_exch{"H122 ELD"}="2H12 ELD"; $resn_exch{"H131 ELD"}="1H13 ELD"; $resn_exch{"H132 ELD"}="2H13 ELD";
$resn_exch{"H141 ELD"}="1H14 ELD"; $resn_exch{"H142 ELD"}="2H14 ELD"; $resn_exch{"H151 ELD"}="1H15 ELD";
$resn_exch{"H152 ELD"}="2H15 ELD"; $resn_exch{"H161 ELD"}="1H16 ELD"; $resn_exch{"H162 ELD"}="2H16 ELD";
$resn_exch{"H171 ELD"}="1H17 ELD"; $resn_exch{"H172 ELD"}="2H17 ELD"; $resn_exch{"H181 ELD"}="1H18 ELD";
$resn_exch{"H182 ELD"}="2H18 ELD"; $resn_exch{"H183 ELD"}="3H18 ELD"; $resn_exch{"H171 ELI"}="1H17 ELI";
$resn_exch{"H172 ELI"}="2H17 ELI"; $resn_exch{"H173 ELI"}="3H17 ELI"; $resn_exch{"H181 ELI"}="1H18 ELI";
$resn_exch{"H182 ELI"}="2H18 ELI"; $resn_exch{"H191 ELI"}="1H19 ELI"; $resn_exch{"H192 ELI"}="2H19 ELI";
$resn_exch{"H201 ELI"}="1H20 ELI"; $resn_exch{"H202 ELI"}="2H20 ELI"; $resn_exch{"H231 ELI"}="1H23 ELI";
$resn_exch{"H232 ELI"}="2H23 ELI"; $resn_exch{"H261 ELI"}="1H26 ELI"; $resn_exch{"H262 ELI"}="2H26 ELI";
$resn_exch{"HA11 ELP"}="1HA1 ELP"; $resn_exch{"HA12 ELP"}="2HA1 ELP"; $resn_exch{"HA13 ELP"}="3HA1 ELP";
$resn_exch{"H2A1 ELP"}="1H2A ELP"; $resn_exch{"H2A2 ELP"}="2H2A ELP"; $resn_exch{"H2A3 ELP"}="3H2A ELP";
$resn_exch{"H5A1 ELP"}="1H5A ELP"; $resn_exch{"H5A2 ELP"}="2H5A ELP"; $resn_exch{"H5'1 EMA"}="1H5* EMA";
$resn_exch{"H5'2 EMA"}="2H5* EMA"; $resn_exch{"H4'1 EMA"}="1H4* EMA"; $resn_exch{"H4'2 EMA"}="2H4* EMA";
$resn_exch{"H1'1 EMA"}="1H1* EMA"; $resn_exch{"H1'2 EMA"}="2H1* EMA"; $resn_exch{"HN61 EMA"}="1HN6 EMA";
$resn_exch{"HN62 EMA"}="2HN6 EMA"; $resn_exch{" H13 EMB"}="3H1  EMB"; $resn_exch{" H12 EMB"}="2H1  EMB";
$resn_exch{" H11 EMB"}="1H1  EMB"; $resn_exch{" H22 EMB"}="2H2  EMB"; $resn_exch{" H21 EMB"}="1H2  EMB";
$resn_exch{" H43 EMB"}="3H4  EMB"; $resn_exch{" H42 EMB"}="2H4  EMB"; $resn_exch{" H41 EMB"}="1H4  EMB";
$resn_exch{" H11 EMC"}="1H1  EMC"; $resn_exch{" H12 EMC"}="2H1  EMC"; $resn_exch{" H21 EMC"}="1H2  EMC";
$resn_exch{" H22 EMC"}="2H2  EMC"; $resn_exch{" H23 EMC"}="3H2  EMC"; $resn_exch{" H71 EMD"}="1H7  EMD";
$resn_exch{" H72 EMD"}="2H7  EMD"; $resn_exch{" H73 EMD"}="3H7  EMD"; $resn_exch{"H151 EMD"}="1H15 EMD";
$resn_exch{"H152 EMD"}="2H15 EMD"; $resn_exch{"H161 EMD"}="1H16 EMD"; $resn_exch{"H162 EMD"}="2H16 EMD";
$resn_exch{"H171 EMD"}="1H17 EMD"; $resn_exch{"H172 EMD"}="2H17 EMD"; $resn_exch{"H251 EMD"}="1H25 EMD";
$resn_exch{"H252 EMD"}="2H25 EMD"; $resn_exch{"H253 EMD"}="3H25 EMD"; $resn_exch{"H261 EMD"}="1H26 EMD";
$resn_exch{"H262 EMD"}="2H26 EMD"; $resn_exch{"H263 EMD"}="3H26 EMD"; $resn_exch{"H111 EMF"}="1H11 EMF";
$resn_exch{"H112 EMF"}="2H11 EMF"; $resn_exch{"H121 EMF"}="1H12 EMF"; $resn_exch{"H122 EMF"}="2H12 EMF";
$resn_exch{"H123 EMF"}="3H12 EMF"; $resn_exch{"H141 EMF"}="1H14 EMF"; $resn_exch{"H142 EMF"}="2H14 EMF";
$resn_exch{"H131 EMF"}="1H13 EMF"; $resn_exch{"H132 EMF"}="2H13 EMF"; $resn_exch{"H133 EMF"}="3H13 EMF";
$resn_exch{"H2C1 EMM"}="1H2C EMM"; $resn_exch{"H2C2 EMM"}="2H2C EMM"; $resn_exch{"H2C3 EMM"}="3H2C EMM";
$resn_exch{"H3C1 EMM"}="1H3C EMM"; $resn_exch{"H3C2 EMM"}="2H3C EMM"; $resn_exch{"H101 EMO"}="1H10 EMO";
$resn_exch{"H102 EMO"}="2H10 EMO"; $resn_exch{"H103 EMO"}="3H10 EMO"; $resn_exch{" H21 EMP"}="1H2  EMP";
$resn_exch{" H22 EMP"}="2H2  EMP"; $resn_exch{" H51 EMP"}="1H5  EMP"; $resn_exch{" H52 EMP"}="2H5  EMP";
$resn_exch{" H61 EMP"}="1H6  EMP"; $resn_exch{" H62 EMP"}="2H6  EMP"; $resn_exch{" H71 EMP"}="1H7  EMP";
$resn_exch{" H72 EMP"}="2H7  EMP"; $resn_exch{" H73 EMP"}="3H7  EMP"; $resn_exch{" H81 EMP"}="1H8  EMP";
$resn_exch{" H82 EMP"}="2H8  EMP"; $resn_exch{" H83 EMP"}="3H8  EMP"; $resn_exch{" H21 EMR"}="1H2  EMR";
$resn_exch{" H22 EMR"}="2H2  EMR"; $resn_exch{" H31 EMR"}="1H3  EMR"; $resn_exch{" H32 EMR"}="2H3  EMR";
$resn_exch{" H51 EMR"}="1H5  EMR"; $resn_exch{" H52 EMR"}="2H5  EMR"; $resn_exch{" H61 EMR"}="1H6  EMR";
$resn_exch{" H62 EMR"}="2H6  EMR"; $resn_exch{"H1'1 EMR"}="1H1' EMR"; $resn_exch{"H1'2 EMR"}="2H1' EMR";
$resn_exch{"H2'1 EMR"}="1H2' EMR"; $resn_exch{"H2'2 EMR"}="2H2' EMR"; $resn_exch{"HN21 EMR"}="1HN2 EMR";
$resn_exch{"HN22 EMR"}="2HN2 EMR"; $resn_exch{" H11 EMT"}="1H1  EMT"; $resn_exch{" H12 EMT"}="2H1  EMT";
$resn_exch{" H21 EMT"}="1H2  EMT"; $resn_exch{" H22 EMT"}="2H2  EMT"; $resn_exch{" H23 EMT"}="3H2  EMT";
$resn_exch{"H9C1 EMU"}="1H9C EMU"; $resn_exch{"H9C2 EMU"}="2H9C EMU"; $resn_exch{" H1' ENA"}=" H1* ENA";
$resn_exch{" H2' ENA"}=" H2* ENA"; $resn_exch{" H4' ENA"}=" H4* ENA"; $resn_exch{" H3' ENA"}=" H3* ENA";
$resn_exch{"HO'4 ENA"}="4HO* ENA"; $resn_exch{" H5' ENA"}="1H5* ENA"; $resn_exch{"H5'' ENA"}="2H5* ENA";
$resn_exch{"HO'3 ENA"}="3HO* ENA"; $resn_exch{"H6'1 ENA"}="1H6* ENA"; $resn_exch{"H6'2 ENA"}="2H6* ENA";
$resn_exch{" H7' ENA"}=" H7* ENA"; $resn_exch{" H8' ENA"}=" H8* ENA"; $resn_exch{" H9' ENA"}=" H9* ENA";
$resn_exch{" H0' ENA"}=" H0* ENA"; $resn_exch{"HO'7 ENA"}="7HO* ENA"; $resn_exch{"HO'8 ENA"}="8HO* ENA";
$resn_exch{"H151 ENB"}="1H15 ENB"; $resn_exch{"H152 ENB"}="2H15 ENB"; $resn_exch{"H171 ENB"}="1H17 ENB";
$resn_exch{"H172 ENB"}="2H17 ENB"; $resn_exch{"H181 ENB"}="1H18 ENB"; $resn_exch{"H182 ENB"}="2H18 ENB";
$resn_exch{"H211 ENB"}="1H21 ENB"; $resn_exch{"H212 ENB"}="2H21 ENB"; $resn_exch{"H213 ENB"}="3H21 ENB";
$resn_exch{"H221 ENB"}="1H22 ENB"; $resn_exch{"H222 ENB"}="2H22 ENB"; $resn_exch{"H223 ENB"}="3H22 ENB";
$resn_exch{"H231 ENB"}="1H23 ENB"; $resn_exch{"H232 ENB"}="2H23 ENB"; $resn_exch{"H233 ENB"}="3H23 ENB";
$resn_exch{"H241 ENB"}="1H24 ENB"; $resn_exch{"H242 ENB"}="2H24 ENB"; $resn_exch{"H243 ENB"}="3H24 ENB";
$resn_exch{"H251 ENB"}="1H25 ENB"; $resn_exch{"H252 ENB"}="2H25 ENB"; $resn_exch{"H253 ENB"}="3H25 ENB";
$resn_exch{"H281 ENB"}="1H28 ENB"; $resn_exch{"H282 ENB"}="2H28 ENB"; $resn_exch{"H291 ENB"}="1H29 ENB";
$resn_exch{"H292 ENB"}="2H29 ENB"; $resn_exch{"H293 ENB"}="3H29 ENB"; $resn_exch{" H11 ENC"}="1H1  ENC";
$resn_exch{" H12 ENC"}="2H1  ENC"; $resn_exch{" H21 ENC"}="1H2  ENC"; $resn_exch{" H22 ENC"}="2H2  ENC";
$resn_exch{" H23 ENC"}="3H2  ENC"; $resn_exch{"HAI1 END"}="1HAI END"; $resn_exch{"HAI2 END"}="2HAI END";
$resn_exch{"HAK1 END"}="1HAK END"; $resn_exch{"HAK2 END"}="2HAK END"; $resn_exch{"HAJ1 END"}="1HAJ END";
$resn_exch{"HAJ2 END"}="2HAJ END"; $resn_exch{"HAH1 END"}="1HAH END"; $resn_exch{"HAH2 END"}="2HAH END";
$resn_exch{"HAK1 ENE"}="1HAK ENE"; $resn_exch{"HAK2 ENE"}="2HAK ENE"; $resn_exch{"HAJ1 ENE"}="1HAJ ENE";
$resn_exch{"HAJ2 ENE"}="2HAJ ENE"; $resn_exch{"HAL1 ENE"}="1HAL ENE"; $resn_exch{"HAL2 ENE"}="2HAL ENE";
$resn_exch{"HAN1 ENE"}="1HAN ENE"; $resn_exch{"HAN2 ENE"}="2HAN ENE"; $resn_exch{"HAM1 ENE"}="1HAM ENE";
$resn_exch{"HAM2 ENE"}="2HAM ENE"; $resn_exch{" H1  ENH"}="1H   ENH"; $resn_exch{" H91 ENH"}="1H9  ENH";
$resn_exch{" H11 ENH"}="1H1  ENH"; $resn_exch{" H12 ENH"}="2H1  ENH"; $resn_exch{" H13 ENH"}="3H1  ENH";
$resn_exch{"H211 ENM"}="1H21 ENM"; $resn_exch{"H212 ENM"}="2H21 ENM"; $resn_exch{"H201 ENM"}="1H20 ENM";
$resn_exch{"H202 ENM"}="2H20 ENM"; $resn_exch{"H191 ENM"}="1H19 ENM"; $resn_exch{"H192 ENM"}="2H19 ENM";
$resn_exch{"H131 ENM"}="1H13 ENM"; $resn_exch{"H132 ENM"}="2H13 ENM"; $resn_exch{"H141 ENM"}="1H14 ENM";
$resn_exch{"H142 ENM"}="2H14 ENM"; $resn_exch{"H181 ENM"}="1H18 ENM"; $resn_exch{"H182 ENM"}="2H18 ENM";
$resn_exch{"H183 ENM"}="3H18 ENM"; $resn_exch{"H061 ENM"}="1H06 ENM"; $resn_exch{"H062 ENM"}="2H06 ENM";
$resn_exch{"H011 ENM"}="1H01 ENM"; $resn_exch{"H012 ENM"}="2H01 ENM"; $resn_exch{"H031 ENM"}="1H03 ENM";
$resn_exch{"H032 ENM"}="2H03 ENM"; $resn_exch{"H071 ENM"}="1H07 ENM"; $resn_exch{"H072 ENM"}="2H07 ENM";
$resn_exch{"H081 ENM"}="1H08 ENM"; $resn_exch{"H082 ENM"}="2H08 ENM"; $resn_exch{"H151 ENM"}="1H15 ENM";
$resn_exch{"H152 ENM"}="2H15 ENM"; $resn_exch{"H161 ENM"}="1H16 ENM"; $resn_exch{"H162 ENM"}="2H16 ENM";
$resn_exch{" H31 ENO"}="1H3  ENO"; $resn_exch{" H32 ENO"}="2H3  ENO"; $resn_exch{" H5' ENP"}="1H5* ENP";
$resn_exch{"H5'' ENP"}="2H5* ENP"; $resn_exch{" H4' ENP"}=" H4* ENP"; $resn_exch{" H3' ENP"}=" H3* ENP";
$resn_exch{" H2' ENP"}=" H2* ENP"; $resn_exch{" H1' ENP"}=" H1* ENP"; $resn_exch{"H0'1 ENP"}="1H0* ENP";
$resn_exch{"H0'2 ENP"}="2H0* ENP"; $resn_exch{" H9' ENP"}=" H9* ENP"; $resn_exch{" H8' ENP"}=" H8* ENP";
$resn_exch{"HO'7 ENP"}="7HO* ENP"; $resn_exch{" H7' ENP"}=" H7* ENP"; $resn_exch{"HO'6 ENP"}="6HO* ENP";
$resn_exch{" H6' ENP"}=" H6* ENP"; $resn_exch{" H81 ENT"}="1H8  ENT"; $resn_exch{" H82 ENT"}="2H8  ENT";
$resn_exch{"H161 ENT"}="1H16 ENT"; $resn_exch{"H162 ENT"}="2H16 ENT"; $resn_exch{"H163 ENT"}="3H16 ENT";
$resn_exch{" H71 ENT"}="1H7  ENT"; $resn_exch{" H72 ENT"}="2H7  ENT"; $resn_exch{"H151 ENT"}="1H15 ENT";
$resn_exch{"H152 ENT"}="2H15 ENT"; $resn_exch{"H153 ENT"}="3H15 ENT"; $resn_exch{"H141 ENT"}="1H14 ENT";
$resn_exch{"H142 ENT"}="2H14 ENT"; $resn_exch{"H143 ENT"}="3H14 ENT"; $resn_exch{"H131 ENT"}="1H13 ENT";
$resn_exch{"H132 ENT"}="2H13 ENT"; $resn_exch{"H133 ENT"}="3H13 ENT"; $resn_exch{"H1C1 ENX"}="1H1C ENX";
$resn_exch{"H1C2 ENX"}="2H1C ENX"; $resn_exch{"H1C3 ENX"}="3H1C ENX"; $resn_exch{"H2C1 ENX"}="1H2C ENX";
$resn_exch{"H2C2 ENX"}="2H2C ENX"; $resn_exch{"H8C1 ENX"}="1H8C ENX"; $resn_exch{"H8C2 ENX"}="2H8C ENX";
$resn_exch{"H261 ENX"}="1H26 ENX"; $resn_exch{"H262 ENX"}="2H26 ENX"; $resn_exch{"H341 ENX"}="1H34 ENX";
$resn_exch{"H342 ENX"}="2H34 ENX"; $resn_exch{"H343 ENX"}="3H34 ENX"; $resn_exch{"H361 ENX"}="1H36 ENX";
$resn_exch{"H362 ENX"}="2H36 ENX"; $resn_exch{"H363 ENX"}="3H36 ENX"; $resn_exch{"H391 ENX"}="1H39 ENX";
$resn_exch{"H392 ENX"}="2H39 ENX"; $resn_exch{"H401 ENX"}="1H40 ENX"; $resn_exch{"H402 ENX"}="2H40 ENX";
$resn_exch{"H431 ENX"}="1H43 ENX"; $resn_exch{"H432 ENX"}="2H43 ENX"; $resn_exch{" H71 EOA"}="1H7  EOA";
$resn_exch{" H72 EOA"}="2H7  EOA"; $resn_exch{" H81 EOA"}="1H8  EOA"; $resn_exch{" H82 EOA"}="2H8  EOA";
$resn_exch{"HAM1 EOB"}="1HAM EOB"; $resn_exch{"HAM2 EOB"}="2HAM EOB"; $resn_exch{"HAN1 EOB"}="1HAN EOB";
$resn_exch{"HAN2 EOB"}="2HAN EOB"; $resn_exch{" H21 EOD"}="1H2  EOD"; $resn_exch{" H22 EOD"}="2H2  EOD";
$resn_exch{" H31 EOD"}="1H3  EOD"; $resn_exch{" H32 EOD"}="2H3  EOD"; $resn_exch{" H41 EOD"}="1H4  EOD";
$resn_exch{" H42 EOD"}="2H4  EOD"; $resn_exch{" H51 EOD"}="1H5  EOD"; $resn_exch{" H52 EOD"}="2H5  EOD";
$resn_exch{" H61 EOD"}="1H6  EOD"; $resn_exch{" H62 EOD"}="2H6  EOD"; $resn_exch{" H71 EOD"}="1H7  EOD";
$resn_exch{" H72 EOD"}="2H7  EOD"; $resn_exch{" H81 EOD"}="1H8  EOD"; $resn_exch{" H82 EOD"}="2H8  EOD";
$resn_exch{" H91 EOD"}="1H9  EOD"; $resn_exch{" H92 EOD"}="2H9  EOD"; $resn_exch{"H101 EOD"}="1H10 EOD";
$resn_exch{"H102 EOD"}="2H10 EOD"; $resn_exch{"H111 EOD"}="1H11 EOD"; $resn_exch{"H112 EOD"}="2H11 EOD";
$resn_exch{"H141 EOD"}="1H14 EOD"; $resn_exch{"H142 EOD"}="2H14 EOD"; $resn_exch{"H151 EOD"}="1H15 EOD";
$resn_exch{"H152 EOD"}="2H15 EOD"; $resn_exch{"H161 EOD"}="1H16 EOD"; $resn_exch{"H162 EOD"}="2H16 EOD";
$resn_exch{"H171 EOD"}="1H17 EOD"; $resn_exch{"H172 EOD"}="2H17 EOD"; $resn_exch{"H181 EOD"}="1H18 EOD";
$resn_exch{"H182 EOD"}="2H18 EOD"; $resn_exch{"H183 EOD"}="3H18 EOD"; $resn_exch{" H11 EOH"}="1H1  EOH";
$resn_exch{" H12 EOH"}="2H1  EOH"; $resn_exch{" H21 EOH"}="1H2  EOH"; $resn_exch{" H22 EOH"}="2H2  EOH";
$resn_exch{" H23 EOH"}="3H2  EOH"; $resn_exch{"HA'1 EOM"}="1HA* EOM"; $resn_exch{"HA'2 EOM"}="2HA* EOM";
$resn_exch{"HC'1 EOM"}="1HC* EOM"; $resn_exch{"HC'2 EOM"}="2HC* EOM"; $resn_exch{"HD'1 EOM"}="1HD* EOM";
$resn_exch{"HD'2 EOM"}="2HD* EOM"; $resn_exch{"HD'3 EOM"}="3HD* EOM"; $resn_exch{"HAL1 EOP"}="1HAL EOP";
$resn_exch{"HAL2 EOP"}="2HAL EOP"; $resn_exch{"HAJ1 EOP"}="1HAJ EOP"; $resn_exch{"HAJ2 EOP"}="2HAJ EOP";
$resn_exch{"HAI1 EOP"}="1HAI EOP"; $resn_exch{"HAI2 EOP"}="2HAI EOP"; $resn_exch{"HAK1 EOP"}="1HAK EOP";
$resn_exch{"HAK2 EOP"}="2HAK EOP"; $resn_exch{"H191 EOT"}="1H19 EOT"; $resn_exch{"H192 EOT"}="2H19 EOT";
$resn_exch{"H201 EOT"}="1H20 EOT"; $resn_exch{"H202 EOT"}="2H20 EOT"; $resn_exch{"H111 EOT"}="1H11 EOT";
$resn_exch{"H112 EOT"}="2H11 EOT"; $resn_exch{" H21 EOT"}="1H2  EOT"; $resn_exch{" H22 EOT"}="2H2  EOT";
$resn_exch{" H91 EOT"}="1H9  EOT"; $resn_exch{" H92 EOT"}="2H9  EOT"; $resn_exch{" H71 EOT"}="1H7  EOT";
$resn_exch{" H72 EOT"}="2H7  EOT"; $resn_exch{" H51 EOT"}="1H5  EOT"; $resn_exch{" H52 EOT"}="2H5  EOT";
$resn_exch{" H31 EOT"}="1H3  EOT"; $resn_exch{" H32 EOT"}="2H3  EOT"; $resn_exch{" H11 EOX"}="1H1  EOX";
$resn_exch{" H12 EOX"}="2H1  EOX"; $resn_exch{" H21 EOX"}="1H2  EOX"; $resn_exch{" H22 EOX"}="2H2  EOX";
$resn_exch{" H23 EOX"}="3H2  EOX"; $resn_exch{"HAN1 EOZ"}="1HAN EOZ"; $resn_exch{"HAN2 EOZ"}="2HAN EOZ";
$resn_exch{"HAO1 EOZ"}="1HAO EOZ"; $resn_exch{"HAO2 EOZ"}="2HAO EOZ"; $resn_exch{"H161  EP"}="1H16  EP";
$resn_exch{"H162  EP"}="2H16  EP"; $resn_exch{"H163  EP"}="3H16  EP"; $resn_exch{" H61  EP"}="1H6   EP";
$resn_exch{" H62  EP"}="2H6   EP"; $resn_exch{" H63  EP"}="3H6   EP"; $resn_exch{"H211  EP"}="1H21  EP";
$resn_exch{"H212  EP"}="2H21  EP"; $resn_exch{"H321  EP"}="1H32  EP"; $resn_exch{"H322  EP"}="2H32  EP";
$resn_exch{"H351  EP"}="1H35  EP"; $resn_exch{"H352  EP"}="2H35  EP"; $resn_exch{"H381  EP"}="1H38  EP";
$resn_exch{"H382  EP"}="2H38  EP"; $resn_exch{"H431  EP"}="1H43  EP"; $resn_exch{"H432  EP"}="2H43  EP";
$resn_exch{"H433  EP"}="3H43  EP"; $resn_exch{"H531  EP"}="1H53  EP"; $resn_exch{"H532  EP"}="2H53  EP";
$resn_exch{"H533  EP"}="3H53  EP"; $resn_exch{"H641  EP"}="1H64  EP"; $resn_exch{"H642  EP"}="2H64  EP";
$resn_exch{"H643  EP"}="3H64  EP"; $resn_exch{"H601  EP"}="1H60  EP"; $resn_exch{"H602  EP"}="2H60  EP";
$resn_exch{"H603  EP"}="3H60  EP"; $resn_exch{"H721  EP"}="1H72  EP"; $resn_exch{"H722  EP"}="2H72  EP";
$resn_exch{"H111 EP1"}="1H11 EP1"; $resn_exch{"H112 EP1"}="2H11 EP1"; $resn_exch{"H101 EP1"}="1H10 EP1";
$resn_exch{"H102 EP1"}="2H10 EP1"; $resn_exch{" H91 EP1"}="1H9  EP1"; $resn_exch{" H92 EP1"}="2H9  EP1";
$resn_exch{" H21 EP1"}="1H2  EP1"; $resn_exch{" H22 EP1"}="2H2  EP1"; $resn_exch{" H31 EP1"}="1H3  EP1";
$resn_exch{" H32 EP1"}="2H3  EP1"; $resn_exch{" H51 EP1"}="1H5  EP1"; $resn_exch{" H52 EP1"}="2H5  EP1";
$resn_exch{" H61 EP1"}="1H6  EP1"; $resn_exch{" H62 EP1"}="2H6  EP1"; $resn_exch{" H71 EP1"}="1H7  EP1";
$resn_exch{" H72 EP1"}="2H7  EP1"; $resn_exch{" H81 EP1"}="1H8  EP1"; $resn_exch{" H82 EP1"}="2H8  EP1";
$resn_exch{" H21 EPA"}="1H2  EPA"; $resn_exch{" H22 EPA"}="2H2  EPA"; $resn_exch{" H31 EPA"}="1H3  EPA";
$resn_exch{" H32 EPA"}="2H3  EPA"; $resn_exch{" H41 EPA"}="1H4  EPA"; $resn_exch{" H42 EPA"}="2H4  EPA";
$resn_exch{" H51 EPA"}="1H5  EPA"; $resn_exch{" H61 EPA"}="1H6  EPA"; $resn_exch{" H71 EPA"}="1H7  EPA";
$resn_exch{" H72 EPA"}="2H7  EPA"; $resn_exch{" H81 EPA"}="1H8  EPA"; $resn_exch{" H91 EPA"}="1H9  EPA";
$resn_exch{"H101 EPA"}="1H10 EPA"; $resn_exch{"H102 EPA"}="2H10 EPA"; $resn_exch{"H111 EPA"}="1H11 EPA";
$resn_exch{"H121 EPA"}="1H12 EPA"; $resn_exch{"H131 EPA"}="1H13 EPA"; $resn_exch{"H132 EPA"}="2H13 EPA";
$resn_exch{"H141 EPA"}="1H14 EPA"; $resn_exch{"H151 EPA"}="1H15 EPA"; $resn_exch{"H161 EPA"}="1H16 EPA";
$resn_exch{"H162 EPA"}="2H16 EPA"; $resn_exch{"H171 EPA"}="1H17 EPA"; $resn_exch{"H181 EPA"}="1H18 EPA";
$resn_exch{"H191 EPA"}="1H19 EPA"; $resn_exch{"H192 EPA"}="2H19 EPA"; $resn_exch{"H201 EPA"}="1H20 EPA";
$resn_exch{"H202 EPA"}="2H20 EPA"; $resn_exch{"H203 EPA"}="3H20 EPA"; $resn_exch{"H511 EPB"}="1H51 EPB";
$resn_exch{"H512 EPB"}="2H51 EPB"; $resn_exch{"H513 EPB"}="3H51 EPB"; $resn_exch{" H31 EPB"}="1H3  EPB";
$resn_exch{" H32 EPB"}="2H3  EPB"; $resn_exch{"H211 EPB"}="1H21 EPB"; $resn_exch{"H212 EPB"}="2H21 EPB";
$resn_exch{"H213 EPB"}="3H21 EPB"; $resn_exch{"H271 EPB"}="1H27 EPB"; $resn_exch{"H272 EPB"}="2H27 EPB";
$resn_exch{"H273 EPB"}="3H27 EPB"; $resn_exch{"H151 EPB"}="1H15 EPB"; $resn_exch{"H152 EPB"}="2H15 EPB";
$resn_exch{"H291 EPB"}="1H29 EPB"; $resn_exch{"H292 EPB"}="2H29 EPB"; $resn_exch{"H293 EPB"}="3H29 EPB";
$resn_exch{"H301 EPB"}="1H30 EPB"; $resn_exch{"H302 EPB"}="2H30 EPB"; $resn_exch{"H303 EPB"}="3H30 EPB";
$resn_exch{"H281 EPB"}="1H28 EPB"; $resn_exch{"H282 EPB"}="2H28 EPB"; $resn_exch{"H283 EPB"}="3H28 EPB";
$resn_exch{"H911 EPB"}="1H91 EPB"; $resn_exch{"H912 EPB"}="2H91 EPB"; $resn_exch{"H913 EPB"}="3H91 EPB";
$resn_exch{" H81 EPB"}="1H8  EPB"; $resn_exch{" H82 EPB"}="2H8  EPB"; $resn_exch{" H71 EPB"}="1H7  EPB";
$resn_exch{" H72 EPB"}="2H7  EPB"; $resn_exch{" H61 EPB"}="1H6  EPB"; $resn_exch{" H62 EPB"}="2H6  EPB";
$resn_exch{"H1C1 EPC"}="1H1C EPC"; $resn_exch{"H2A1 EPC"}="1H2A EPC"; $resn_exch{"H2A2 EPC"}="2H2A EPC";
$resn_exch{"H2A3 EPC"}="3H2A EPC"; $resn_exch{"H4A1 EPC"}="1H4A EPC"; $resn_exch{"HC61 EPC"}="1HC6 EPC";
$resn_exch{"H5A1 EPC"}="1H5A EPC"; $resn_exch{"H5A2 EPC"}="2H5A EPC"; $resn_exch{"HCA1 EPC"}="1HCA EPC";
$resn_exch{"HCA2 EPC"}="2HCA EPC"; $resn_exch{"HCA3 EPC"}="3HCA EPC"; $resn_exch{"H511 EPD"}="1H51 EPD";
$resn_exch{"H512 EPD"}="2H51 EPD"; $resn_exch{"H513 EPD"}="3H51 EPD"; $resn_exch{" H31 EPD"}="1H3  EPD";
$resn_exch{" H32 EPD"}="2H3  EPD"; $resn_exch{"H211 EPD"}="1H21 EPD"; $resn_exch{"H212 EPD"}="2H21 EPD";
$resn_exch{"H213 EPD"}="3H21 EPD"; $resn_exch{"H271 EPD"}="1H27 EPD"; $resn_exch{"H272 EPD"}="2H27 EPD";
$resn_exch{"H273 EPD"}="3H27 EPD"; $resn_exch{" H61 EPD"}="1H6  EPD"; $resn_exch{" H62 EPD"}="2H6  EPD";
$resn_exch{" H71 EPD"}="1H7  EPD"; $resn_exch{" H72 EPD"}="2H7  EPD"; $resn_exch{" H81 EPD"}="1H8  EPD";
$resn_exch{" H82 EPD"}="2H8  EPD"; $resn_exch{"H911 EPD"}="1H91 EPD"; $resn_exch{"H912 EPD"}="2H91 EPD";
$resn_exch{"H913 EPD"}="3H91 EPD"; $resn_exch{"H281 EPD"}="1H28 EPD"; $resn_exch{"H282 EPD"}="2H28 EPD";
$resn_exch{"H283 EPD"}="3H28 EPD"; $resn_exch{"H301 EPD"}="1H30 EPD"; $resn_exch{"H302 EPD"}="2H30 EPD";
$resn_exch{"H303 EPD"}="3H30 EPD"; $resn_exch{"H291 EPD"}="1H29 EPD"; $resn_exch{"H292 EPD"}="2H29 EPD";
$resn_exch{"H293 EPD"}="3H29 EPD"; $resn_exch{"H151 EPD"}="1H15 EPD"; $resn_exch{"H152 EPD"}="2H15 EPD";
$resn_exch{" H21 EPE"}="1H2  EPE"; $resn_exch{" H22 EPE"}="2H2  EPE"; $resn_exch{" H31 EPE"}="1H3  EPE";
$resn_exch{" H32 EPE"}="2H3  EPE"; $resn_exch{" H51 EPE"}="1H5  EPE"; $resn_exch{" H52 EPE"}="2H5  EPE";
$resn_exch{" H61 EPE"}="1H6  EPE"; $resn_exch{" H62 EPE"}="2H6  EPE"; $resn_exch{" H71 EPE"}="1H7  EPE";
$resn_exch{" H72 EPE"}="2H7  EPE"; $resn_exch{" H81 EPE"}="1H8  EPE"; $resn_exch{" H82 EPE"}="2H8  EPE";
$resn_exch{" H91 EPE"}="1H9  EPE"; $resn_exch{" H92 EPE"}="2H9  EPE"; $resn_exch{"H101 EPE"}="1H10 EPE";
$resn_exch{"H102 EPE"}="2H10 EPE"; $resn_exch{"HOS3 EPE"}="3HOS EPE"; $resn_exch{" H61 EPG"}="1H6  EPG";
$resn_exch{" H62 EPG"}="2H6  EPG"; $resn_exch{" H71 EPG"}="1H7  EPG"; $resn_exch{" H72 EPG"}="2H7  EPG";
$resn_exch{" H91 EPG"}="1H9  EPG"; $resn_exch{" H92 EPG"}="2H9  EPG"; $resn_exch{" H11 EPH"}="1H1  EPH";
$resn_exch{"H12A EPH"}="2H1  EPH"; $resn_exch{"H181 EPH"}="1H18 EPH"; $resn_exch{"H182 EPH"}="2H18 EPH";
$resn_exch{"H191 EPH"}="1H19 EPH"; $resn_exch{"H192 EPH"}="2H19 EPH"; $resn_exch{"H201 EPH"}="1H20 EPH";
$resn_exch{"H202 EPH"}="2H20 EPH"; $resn_exch{"H211 EPH"}="1H21 EPH"; $resn_exch{"H212 EPH"}="2H21 EPH";
$resn_exch{"H221 EPH"}="1H22 EPH"; $resn_exch{"H222 EPH"}="2H22 EPH"; $resn_exch{"H231 EPH"}="1H23 EPH";
$resn_exch{"H232 EPH"}="2H23 EPH"; $resn_exch{"H261 EPH"}="1H26 EPH"; $resn_exch{"H262 EPH"}="2H26 EPH";
$resn_exch{"H271 EPH"}="1H27 EPH"; $resn_exch{"H272 EPH"}="2H27 EPH"; $resn_exch{"H301 EPH"}="1H30 EPH";
$resn_exch{"H302 EPH"}="2H30 EPH"; $resn_exch{"H311 EPH"}="1H31 EPH"; $resn_exch{"H312 EPH"}="2H31 EPH";
$resn_exch{"H341 EPH"}="1H34 EPH"; $resn_exch{"H342 EPH"}="2H34 EPH"; $resn_exch{"H343 EPH"}="3H34 EPH";
$resn_exch{"H371 EPH"}="1H37 EPH"; $resn_exch{"H372 EPH"}="2H37 EPH"; $resn_exch{" H51 EPH"}="1H5  EPH";
$resn_exch{" H52 EPH"}="2H5  EPH"; $resn_exch{" H61 EPH"}="1H6  EPH"; $resn_exch{" H62 EPH"}="2H6  EPH";
$resn_exch{" H71 EPH"}="1H7  EPH"; $resn_exch{" H72 EPH"}="2H7  EPH"; $resn_exch{" H81 EPH"}="1H8  EPH";
$resn_exch{" H82 EPH"}="2H8  EPH"; $resn_exch{" H91 EPH"}="1H9  EPH"; $resn_exch{" H92 EPH"}="2H9  EPH";
$resn_exch{"H101 EPH"}="1H10 EPH"; $resn_exch{"H102 EPH"}="2H10 EPH"; $resn_exch{"H111 EPH"}="1H11 EPH";
$resn_exch{"H112 EPH"}="2H11 EPH"; $resn_exch{"H141 EPH"}="1H14 EPH"; $resn_exch{"H142 EPH"}="2H14 EPH";
$resn_exch{"H171 EPH"}="1H17 EPH"; $resn_exch{"H172 EPH"}="2H17 EPH"; $resn_exch{"H351 EPH"}="1H35 EPH";
$resn_exch{"H352 EPH"}="2H35 EPH"; $resn_exch{"H361 EPH"}="1H36 EPH"; $resn_exch{"H362 EPH"}="2H36 EPH";
$resn_exch{"H363 EPH"}="3H36 EPH"; $resn_exch{"H381 EPH"}="1H38 EPH"; $resn_exch{"H382 EPH"}="2H38 EPH";
$resn_exch{"H391 EPH"}="1H39 EPH"; $resn_exch{"H392 EPH"}="2H39 EPH"; $resn_exch{"HN11 EPH"}="1HN1 EPH";
$resn_exch{"HN12 EPH"}="2HN1 EPH"; $resn_exch{" H21 EPI"}="1H2  EPI"; $resn_exch{" H22 EPI"}="2H2  EPI";
$resn_exch{" H31 EPI"}="1H3  EPI"; $resn_exch{" H32 EPI"}="2H3  EPI"; $resn_exch{" H51 EPI"}="1H5  EPI";
$resn_exch{" H52 EPI"}="2H5  EPI"; $resn_exch{" H61 EPI"}="1H6  EPI"; $resn_exch{" H62 EPI"}="2H6  EPI";
$resn_exch{"H1'1 EPI"}="1H1' EPI"; $resn_exch{"H1'2 EPI"}="2H1' EPI"; $resn_exch{"H2'1 EPI"}="1H2' EPI";
$resn_exch{"H2'2 EPI"}="2H2' EPI"; $resn_exch{"H2'3 EPI"}="3H2' EPI"; $resn_exch{"H2C1 EPJ"}="1H2C EPJ";
$resn_exch{"H2C2 EPJ"}="2H2C EPJ"; $resn_exch{"H4C1 EPJ"}="1H4C EPJ"; $resn_exch{"H4C2 EPJ"}="2H4C EPJ";
$resn_exch{"H5C1 EPJ"}="1H5C EPJ"; $resn_exch{"H5C2 EPJ"}="2H5C EPJ"; $resn_exch{" H91 EPL"}="1H9  EPL";
$resn_exch{" H92 EPL"}="2H9  EPL"; $resn_exch{" H93 EPL"}="3H9  EPL"; $resn_exch{" H81 EPL"}="1H8  EPL";
$resn_exch{" H82 EPL"}="2H8  EPL"; $resn_exch{"H101 EPL"}="1H10 EPL"; $resn_exch{"H102 EPL"}="2H10 EPL";
$resn_exch{"H111 EPL"}="1H11 EPL"; $resn_exch{"H112 EPL"}="2H11 EPL"; $resn_exch{"H113 EPL"}="3H11 EPL";
$resn_exch{" H71 EPL"}="1H7  EPL"; $resn_exch{" H72 EPL"}="2H7  EPL"; $resn_exch{" H73 EPL"}="3H7  EPL";
$resn_exch{" H31 EPM"}="1H3  EPM"; $resn_exch{" H32 EPM"}="2H3  EPM"; $resn_exch{" H41 EPM"}="1H4  EPM";
$resn_exch{" H42 EPM"}="2H4  EPM"; $resn_exch{" H51 EPM"}="1H5  EPM"; $resn_exch{" H52 EPM"}="2H5  EPM";
$resn_exch{" H53 EPM"}="3H5  EPM"; $resn_exch{" H71 EPM"}="1H7  EPM"; $resn_exch{" H72 EPM"}="2H7  EPM";
$resn_exch{" H81 EPM"}="1H8  EPM"; $resn_exch{" H82 EPM"}="2H8  EPM"; $resn_exch{" H91 EPM"}="1H9  EPM";
$resn_exch{" H92 EPM"}="2H9  EPM"; $resn_exch{"H101 EPM"}="1H10 EPM"; $resn_exch{"H102 EPM"}="2H10 EPM";
$resn_exch{"H111 EPM"}="1H11 EPM"; $resn_exch{"H112 EPM"}="2H11 EPM"; $resn_exch{"H121 EPM"}="1H12 EPM";
$resn_exch{"H122 EPM"}="2H12 EPM"; $resn_exch{"H131 EPM"}="1H13 EPM"; $resn_exch{"H132 EPM"}="2H13 EPM";
$resn_exch{"H141 EPM"}="1H14 EPM"; $resn_exch{"H142 EPM"}="2H14 EPM"; $resn_exch{"H151 EPM"}="1H15 EPM";
$resn_exch{"H152 EPM"}="2H15 EPM"; $resn_exch{"H161 EPM"}="1H16 EPM"; $resn_exch{"H162 EPM"}="2H16 EPM";
$resn_exch{"H171 EPM"}="1H17 EPM"; $resn_exch{"H172 EPM"}="2H17 EPM"; $resn_exch{"H181 EPM"}="1H18 EPM";
$resn_exch{"H182 EPM"}="2H18 EPM"; $resn_exch{"H191 EPM"}="1H19 EPM"; $resn_exch{"H192 EPM"}="2H19 EPM";
$resn_exch{"H201 EPM"}="1H20 EPM"; $resn_exch{"H202 EPM"}="2H20 EPM"; $resn_exch{"H211 EPM"}="1H21 EPM";
$resn_exch{"H212 EPM"}="2H21 EPM"; $resn_exch{"H213 EPM"}="3H21 EPM"; $resn_exch{" H11 EPN"}="1H1  EPN";
$resn_exch{" H12 EPN"}="2H1  EPN"; $resn_exch{" H21 EPN"}="1H2  EPN"; $resn_exch{" H22 EPN"}="2H2  EPN";
$resn_exch{" H31 EPN"}="1H3  EPN"; $resn_exch{" H32 EPN"}="2H3  EPN"; $resn_exch{" H11 EPO"}="1H1  EPO";
$resn_exch{" H31 EPO"}="1H3  EPO"; $resn_exch{" H32 EPO"}="2H3  EPO"; $resn_exch{" H44 EPO"}="4H4  EPO";
$resn_exch{"HNQ1 EPQ"}="1HNQ EPQ"; $resn_exch{"HNQ2 EPQ"}="2HNQ EPQ"; $resn_exch{"HQB1 EPQ"}="1HQB EPQ";
$resn_exch{"HQB2 EPQ"}="2HQB EPQ"; $resn_exch{"HMK1 EPQ"}="1HMK EPQ"; $resn_exch{"HMK2 EPQ"}="2HMK EPQ";
$resn_exch{"HMK3 EPQ"}="3HMK EPQ"; $resn_exch{" HG1 EPQ"}="1HG  EPQ"; $resn_exch{" HG2 EPQ"}="2HG  EPQ";
$resn_exch{"HH11 EPQ"}="1HH1 EPQ"; $resn_exch{"HH12 EPQ"}="2HH1 EPQ"; $resn_exch{"HH13 EPQ"}="3HH1 EPQ";
$resn_exch{"HH21 EPQ"}="1HH2 EPQ"; $resn_exch{"HH22 EPQ"}="2HH2 EPQ"; $resn_exch{"HH23 EPQ"}="3HH2 EPQ";
$resn_exch{" H61 EPS"}="1H6  EPS"; $resn_exch{" H62 EPS"}="2H6  EPS"; $resn_exch{" H81 EPS"}="1H8  EPS";
$resn_exch{" H82 EPS"}="2H8  EPS"; $resn_exch{" H71 EPT"}="1H7  EPT"; $resn_exch{" H72 EPT"}="2H7  EPT";
$resn_exch{" H81 EPT"}="1H8  EPT"; $resn_exch{" H82 EPT"}="2H8  EPT"; $resn_exch{" H91 EPT"}="1H9  EPT";
$resn_exch{" H92 EPT"}="2H9  EPT"; $resn_exch{"H101 EPT"}="1H10 EPT"; $resn_exch{"H102 EPT"}="2H10 EPT";
$resn_exch{"H111 EPT"}="1H11 EPT"; $resn_exch{"H112 EPT"}="2H11 EPT"; $resn_exch{"H121 EPT"}="1H12 EPT";
$resn_exch{"H122 EPT"}="2H12 EPT"; $resn_exch{"H131 EPT"}="1H13 EPT"; $resn_exch{"H132 EPT"}="2H13 EPT";
$resn_exch{"H133 EPT"}="3H13 EPT"; $resn_exch{"HN3U EPU"}="UHN3 EPU"; $resn_exch{" H5U EPU"}=" UH5 EPU";
$resn_exch{" H6U EPU"}=" UH6 EPU"; $resn_exch{" H1D EPU"}="UH1* EPU"; $resn_exch{" H2D EPU"}="UH2* EPU";
$resn_exch{"HO2U EPU"}="UHO2 EPU"; $resn_exch{" H3D EPU"}="UH3* EPU"; $resn_exch{" H4D EPU"}="UH4* EPU";
$resn_exch{"HO3U EPU"}="UHO3 EPU"; $resn_exch{"H5'1 EPU"}="1H5* EPU"; $resn_exch{"H5'2 EPU"}="2H5* EPU";
$resn_exch{"HOA2 EPU"}="2HOA EPU"; $resn_exch{"HOB2 EPU"}="2HOB EPU"; $resn_exch{" H61 EPU"}="1H6  EPU";
$resn_exch{" H62 EPU"}="2H6  EPU"; $resn_exch{" H81 EPU"}="1H8  EPU"; $resn_exch{" H82 EPU"}="2H8  EPU";
$resn_exch{" H83 EPU"}="3H8  EPU"; $resn_exch{"HO2E EPU"}="EHO2 EPU"; $resn_exch{"HE31 EPU"}="1HE3 EPU";
$resn_exch{"HE32 EPU"}="2HE3 EPU"; $resn_exch{" H11 EPX"}="1H1  EPX"; $resn_exch{"H31A EPX"}="1H3  EPX";
$resn_exch{"H32A EPX"}="2H3  EPX"; $resn_exch{" H33 EPX"}="3H3  EPX"; $resn_exch{" H51 EPX"}="1H5  EPX";
$resn_exch{" H61 EPX"}="1H6  EPX"; $resn_exch{" H62 EPX"}="2H6  EPX"; $resn_exch{" H63 EPX"}="3H6  EPX";
$resn_exch{" H71 EPX"}="1H7  EPX"; $resn_exch{" H72 EPX"}="2H7  EPX"; $resn_exch{" H81 EPX"}="1H8  EPX";
$resn_exch{" H82 EPX"}="2H8  EPX"; $resn_exch{" H83 EPX"}="3H8  EPX"; $resn_exch{" H91 EPX"}="1H9  EPX";
$resn_exch{" H92 EPX"}="2H9  EPX"; $resn_exch{" H93 EPX"}="3H9  EPX"; $resn_exch{"HN21 EPX"}="1HN2 EPX";
$resn_exch{"H131 EPX"}="1H13 EPX"; $resn_exch{"H132 EPX"}="2H13 EPX"; $resn_exch{"H133 EPX"}="3H13 EPX";
$resn_exch{"H141 EPX"}="1H14 EPX"; $resn_exch{"H142 EPX"}="2H14 EPX"; $resn_exch{"H151 EPX"}="1H15 EPX";
$resn_exch{"H152 EPX"}="2H15 EPX"; $resn_exch{"H153 EPX"}="3H15 EPX"; $resn_exch{"HN31 EPX"}="1HN3 EPX";
$resn_exch{"H191 EPX"}="1H19 EPX"; $resn_exch{"H192 EPX"}="2H19 EPX"; $resn_exch{"H193 EPX"}="3H19 EPX";
$resn_exch{"HN51 EPX"}="1HN5 EPX"; $resn_exch{"H261 EPX"}="1H26 EPX"; $resn_exch{"H262 EPX"}="2H26 EPX";
$resn_exch{"H251 EPX"}="1H25 EPX"; $resn_exch{"H252 EPX"}="2H25 EPX"; $resn_exch{"H253 EPX"}="3H25 EPX";
$resn_exch{"H241 EPX"}="1H24 EPX"; $resn_exch{"H242 EPX"}="2H24 EPX"; $resn_exch{"H243 EPX"}="3H24 EPX";
$resn_exch{"HO61 EPX"}="1HO6 EPX"; $resn_exch{"HN41 EPX"}="1HN4 EPX"; $resn_exch{"H301 EPX"}="1H30 EPX";
$resn_exch{"H302 EPX"}="2H30 EPX"; $resn_exch{"H303 EPX"}="3H30 EPX"; $resn_exch{"H281 EPX"}="1H28 EPX";
$resn_exch{"H282 EPX"}="2H28 EPX"; $resn_exch{"H283 EPX"}="3H28 EPX"; $resn_exch{"H271 EPX"}="1H27 EPX";
$resn_exch{"H272 EPX"}="2H27 EPX"; $resn_exch{"HO71 EPX"}="1HO7 EPX"; $resn_exch{"H3A1 EPY"}="1H3A EPY";
$resn_exch{"HNA4 EPY"}="4HNA EPY"; $resn_exch{"H1A1 EPY"}="1H1A EPY"; $resn_exch{"H1A2 EPY"}="2H1A EPY";
$resn_exch{"H5A1 EPY"}="1H5A EPY"; $resn_exch{"H5A2 EPY"}="2H5A EPY"; $resn_exch{"HOA6 EPY"}="6HOA EPY";
$resn_exch{"HOB6 EPY"}="6HOB EPY"; $resn_exch{"H5B1 EPY"}="1H5B EPY"; $resn_exch{"HN51 EPY"}="1HN5 EPY";
$resn_exch{"HN52 EPY"}="2HN5 EPY"; $resn_exch{"H4B1 EPY"}="1H4B EPY"; $resn_exch{"H4B2 EPY"}="2H4B EPY";
$resn_exch{"H3B1 EPY"}="1H3B EPY"; $resn_exch{"H3B2 EPY"}="2H3B EPY"; $resn_exch{"HNB1 EPY"}="1HNB EPY";
$resn_exch{"H1C1 EPY"}="1H1C EPY"; $resn_exch{"H3C1 EPY"}="1H3C EPY"; $resn_exch{"H3C2 EPY"}="2H3C EPY";
$resn_exch{"HOC2 EPY"}="2HOC EPY"; $resn_exch{"H2C1 EPY"}="1H2C EPY"; $resn_exch{"H2C2 EPY"}="2H2C EPY";
$resn_exch{"H9C1 EPY"}="1H9C EPY"; $resn_exch{"H8C1 EPY"}="1H8C EPY"; $resn_exch{"H6C1 EPY"}="1H6C EPY";
$resn_exch{"H5C1 EPY"}="1H5C EPY"; $resn_exch{"H102 EQI"}="2H10 EQI"; $resn_exch{"H131 EQI"}="1H13 EQI";
$resn_exch{"H181 EQI"}="1H18 EQI"; $resn_exch{"H182 EQI"}="2H18 EQI"; $resn_exch{"H191 EQI"}="1H19 EQI";
$resn_exch{"H192 EQI"}="2H19 EQI"; $resn_exch{"H241 EQI"}="1H24 EQI"; $resn_exch{"H242 EQI"}="2H24 EQI";
$resn_exch{"H251 EQI"}="1H25 EQI"; $resn_exch{"H252 EQI"}="2H25 EQI"; $resn_exch{"H271 EQI"}="1H27 EQI";
$resn_exch{"H272 EQI"}="2H27 EQI"; $resn_exch{"H273 EQI"}="3H27 EQI"; $resn_exch{" H31 EQP"}="1H3  EQP";
$resn_exch{" H32 EQP"}="2H3  EQP"; $resn_exch{" H91 EQP"}="1H9  EQP"; $resn_exch{" H92 EQP"}="2H9  EQP";
$resn_exch{"H111 EQP"}="1H11 EQP"; $resn_exch{"H112 EQP"}="2H11 EQP"; $resn_exch{"H113 EQP"}="3H11 EQP";
$resn_exch{"HOP2 EQP"}="2HOP EQP"; $resn_exch{"HOP3 EQP"}="3HOP EQP"; $resn_exch{"H181 EQU"}="1H18 EQU";
$resn_exch{"H182 EQU"}="2H18 EQU"; $resn_exch{"H191 EQU"}="1H19 EQU"; $resn_exch{"H192 EQU"}="2H19 EQU";
$resn_exch{"H241 EQU"}="1H24 EQU"; $resn_exch{"H242 EQU"}="2H24 EQU"; $resn_exch{"H251 EQU"}="1H25 EQU";
$resn_exch{"H252 EQU"}="2H25 EQU"; $resn_exch{"H271 EQU"}="1H27 EQU"; $resn_exch{"H272 EQU"}="2H27 EQU";
$resn_exch{"H273 EQU"}="3H27 EQU"; $resn_exch{"H101 ERD"}="1H10 ERD"; $resn_exch{"H102 ERD"}="2H10 ERD";
$resn_exch{" H11 ERG"}="1H1  ERG"; $resn_exch{" H12 ERG"}="2H1  ERG"; $resn_exch{" H21 ERG"}="1H2  ERG";
$resn_exch{"H22A ERG"}="2H2  ERG"; $resn_exch{" H41 ERG"}="1H4  ERG"; $resn_exch{" H42 ERG"}="2H4  ERG";
$resn_exch{"H111 ERG"}="1H11 ERG"; $resn_exch{"H112 ERG"}="2H11 ERG"; $resn_exch{"H121 ERG"}="1H12 ERG";
$resn_exch{"H122 ERG"}="2H12 ERG"; $resn_exch{"H151 ERG"}="1H15 ERG"; $resn_exch{"H152 ERG"}="2H15 ERG";
$resn_exch{"H161 ERG"}="1H16 ERG"; $resn_exch{"H162 ERG"}="2H16 ERG"; $resn_exch{"H181 ERG"}="1H18 ERG";
$resn_exch{"H182 ERG"}="2H18 ERG"; $resn_exch{"H183 ERG"}="3H18 ERG"; $resn_exch{"H191 ERG"}="1H19 ERG";
$resn_exch{"H192 ERG"}="2H19 ERG"; $resn_exch{"H193 ERG"}="3H19 ERG"; $resn_exch{"H211 ERG"}="1H21 ERG";
$resn_exch{"H212 ERG"}="2H21 ERG"; $resn_exch{"H213 ERG"}="3H21 ERG"; $resn_exch{"H261 ERG"}="1H26 ERG";
$resn_exch{"H262 ERG"}="2H26 ERG"; $resn_exch{"H263 ERG"}="3H26 ERG"; $resn_exch{"H271 ERG"}="1H27 ERG";
$resn_exch{"H272 ERG"}="2H27 ERG"; $resn_exch{"H273 ERG"}="3H27 ERG"; $resn_exch{"H281 ERG"}="1H28 ERG";
$resn_exch{"H282 ERG"}="2H28 ERG"; $resn_exch{"H283 ERG"}="3H28 ERG"; $resn_exch{" H21 ERI"}="1H2  ERI";
$resn_exch{" H22 ERI"}="2H2  ERI"; $resn_exch{" H31 ERI"}="1H3  ERI"; $resn_exch{" H32 ERI"}="2H3  ERI";
$resn_exch{" H33 ERI"}="3H3  ERI"; $resn_exch{" H41 ERI"}="1H4  ERI"; $resn_exch{" H42 ERI"}="2H4  ERI";
$resn_exch{" H43 ERI"}="3H4  ERI"; $resn_exch{" H61 ERI"}="1H6  ERI"; $resn_exch{" H62 ERI"}="2H6  ERI";
$resn_exch{" H63 ERI"}="3H6  ERI"; $resn_exch{"HN21 ERN"}="1HN2 ERN"; $resn_exch{"HN22 ERN"}="2HN2 ERN";
$resn_exch{"H341 ERN"}="1H34 ERN"; $resn_exch{"H342 ERN"}="2H34 ERN"; $resn_exch{"H343 ERN"}="3H34 ERN";
$resn_exch{"H361 ERN"}="1H36 ERN"; $resn_exch{"H362 ERN"}="2H36 ERN"; $resn_exch{"H371 ERN"}="1H37 ERN";
$resn_exch{"H372 ERN"}="2H37 ERN"; $resn_exch{"H373 ERN"}="3H37 ERN"; $resn_exch{"H301 ERN"}="1H30 ERN";
$resn_exch{"H302 ERN"}="2H30 ERN"; $resn_exch{"H303 ERN"}="3H30 ERN"; $resn_exch{"H321 ERN"}="1H32 ERN";
$resn_exch{"H322 ERN"}="2H32 ERN"; $resn_exch{"H323 ERN"}="3H32 ERN"; $resn_exch{" H71 ERN"}="1H7  ERN";
$resn_exch{" H72 ERN"}="2H7  ERN"; $resn_exch{"H331 ERN"}="1H33 ERN"; $resn_exch{"H332 ERN"}="2H33 ERN";
$resn_exch{"H333 ERN"}="3H33 ERN"; $resn_exch{"H351 ERN"}="1H35 ERN"; $resn_exch{"H352 ERN"}="2H35 ERN";
$resn_exch{"H353 ERN"}="3H35 ERN"; $resn_exch{"H311 ERN"}="1H31 ERN"; $resn_exch{"H312 ERN"}="2H31 ERN";
$resn_exch{"H313 ERN"}="3H31 ERN"; $resn_exch{"H251 ERN"}="1H25 ERN"; $resn_exch{"H252 ERN"}="2H25 ERN";
$resn_exch{"H271 ERN"}="1H27 ERN"; $resn_exch{"H272 ERN"}="2H27 ERN"; $resn_exch{"H273 ERN"}="3H27 ERN";
$resn_exch{"H281 ERN"}="1H28 ERN"; $resn_exch{"H282 ERN"}="2H28 ERN"; $resn_exch{"H283 ERN"}="3H28 ERN";
$resn_exch{"H291 ERN"}="1H29 ERN"; $resn_exch{"H292 ERN"}="2H29 ERN"; $resn_exch{"H293 ERN"}="3H29 ERN";
$resn_exch{"H151 ERN"}="1H15 ERN"; $resn_exch{"H152 ERN"}="2H15 ERN"; $resn_exch{"H201 ERN"}="1H20 ERN";
$resn_exch{"H202 ERN"}="2H20 ERN"; $resn_exch{"H203 ERN"}="3H20 ERN"; $resn_exch{"H191 ERN"}="1H19 ERN";
$resn_exch{"H192 ERN"}="2H19 ERN"; $resn_exch{"H193 ERN"}="3H19 ERN"; $resn_exch{"H211 ERN"}="1H21 ERN";
$resn_exch{"H212 ERN"}="2H21 ERN"; $resn_exch{"H213 ERN"}="3H21 ERN"; $resn_exch{"H401 ERT"}="1H40 ERT";
$resn_exch{"H402 ERT"}="2H40 ERT"; $resn_exch{"H403 ERT"}="3H40 ERT"; $resn_exch{"H221 ERT"}="1H22 ERT";
$resn_exch{"H222 ERT"}="2H22 ERT"; $resn_exch{"H223 ERT"}="3H22 ERT"; $resn_exch{" H81 ERT"}="1H8  ERT";
$resn_exch{" H82 ERT"}="2H8  ERT"; $resn_exch{"H131 ERT"}="1H13 ERT"; $resn_exch{"H132 ERT"}="2H13 ERT";
$resn_exch{"H141 ERT"}="1H14 ERT"; $resn_exch{"H142 ERT"}="2H14 ERT"; $resn_exch{"H143 ERT"}="3H14 ERT";
$resn_exch{" H1' ERT"}=" H1* ERT"; $resn_exch{"H2'1 ERT"}="1H2* ERT"; $resn_exch{"H2'2 ERT"}="2H2* ERT";
$resn_exch{" H3' ERT"}=" H3* ERT"; $resn_exch{"H7'1 ERT"}="1H7* ERT"; $resn_exch{"H7'2 ERT"}="2H7* ERT";
$resn_exch{"H7'3 ERT"}="3H7* ERT"; $resn_exch{"H8'1 ERT"}="1H8* ERT"; $resn_exch{"H8'2 ERT"}="2H8* ERT";
$resn_exch{"H8'3 ERT"}="3H8* ERT"; $resn_exch{" H4' ERT"}=" H4* ERT"; $resn_exch{" H5' ERT"}=" H5* ERT";
$resn_exch{"H6'1 ERT"}="1H6* ERT"; $resn_exch{"H6'2 ERT"}="2H6* ERT"; $resn_exch{"H6'3 ERT"}="3H6* ERT";
$resn_exch{" H71 ERY"}="1H7  ERY"; $resn_exch{" H72 ERY"}="2H7  ERY"; $resn_exch{"H341 ERY"}="1H34 ERY";
$resn_exch{"H342 ERY"}="2H34 ERY"; $resn_exch{"H343 ERY"}="3H34 ERY"; $resn_exch{"H331 ERY"}="1H33 ERY";
$resn_exch{"H332 ERY"}="2H33 ERY"; $resn_exch{"H333 ERY"}="3H33 ERY"; $resn_exch{"H351 ERY"}="1H35 ERY";
$resn_exch{"H352 ERY"}="2H35 ERY"; $resn_exch{"H353 ERY"}="3H35 ERY"; $resn_exch{"H361 ERY"}="1H36 ERY";
$resn_exch{"H362 ERY"}="2H36 ERY"; $resn_exch{"H301 ERY"}="1H30 ERY"; $resn_exch{"H302 ERY"}="2H30 ERY";
$resn_exch{"H303 ERY"}="3H30 ERY"; $resn_exch{"H321 ERY"}="1H32 ERY"; $resn_exch{"H322 ERY"}="2H32 ERY";
$resn_exch{"H323 ERY"}="3H32 ERY"; $resn_exch{"H251 ERY"}="1H25 ERY"; $resn_exch{"H252 ERY"}="2H25 ERY";
$resn_exch{"H271 ERY"}="1H27 ERY"; $resn_exch{"H272 ERY"}="2H27 ERY"; $resn_exch{"H273 ERY"}="3H27 ERY";
$resn_exch{"H281 ERY"}="1H28 ERY"; $resn_exch{"H282 ERY"}="2H28 ERY"; $resn_exch{"H283 ERY"}="3H28 ERY";
$resn_exch{"H151 ERY"}="1H15 ERY"; $resn_exch{"H152 ERY"}="2H15 ERY"; $resn_exch{"H201 ERY"}="1H20 ERY";
$resn_exch{"H202 ERY"}="2H20 ERY"; $resn_exch{"H203 ERY"}="3H20 ERY"; $resn_exch{"H291 ERY"}="1H29 ERY";
$resn_exch{"H292 ERY"}="2H29 ERY"; $resn_exch{"H293 ERY"}="3H29 ERY"; $resn_exch{"H211 ERY"}="1H21 ERY";
$resn_exch{"H212 ERY"}="2H21 ERY"; $resn_exch{"H213 ERY"}="3H21 ERY"; $resn_exch{"H371 ERY"}="1H37 ERY";
$resn_exch{"H372 ERY"}="2H37 ERY"; $resn_exch{"H373 ERY"}="3H37 ERY"; $resn_exch{"H311 ERY"}="1H31 ERY";
$resn_exch{"H312 ERY"}="2H31 ERY"; $resn_exch{"H313 ERY"}="3H31 ERY"; $resn_exch{"H191 ERY"}="1H19 ERY";
$resn_exch{"H192 ERY"}="2H19 ERY"; $resn_exch{"H193 ERY"}="3H19 ERY"; $resn_exch{" H11 ESA"}="1H1  ESA";
$resn_exch{" H12 ESA"}="2H1  ESA"; $resn_exch{" H13 ESA"}="3H1  ESA"; $resn_exch{" H21 ESA"}="1H2  ESA";
$resn_exch{" H22 ESA"}="2H2  ESA"; $resn_exch{" H   ESC"}="1HN  ESC"; $resn_exch{" H2  ESC"}="2HN  ESC";
$resn_exch{"HCA1 ESC"}="1HCA ESC"; $resn_exch{" HB2 ESC"}="1HCB ESC"; $resn_exch{" HB3 ESC"}="2HCB ESC";
$resn_exch{" HG2 ESC"}="1HCG ESC"; $resn_exch{" HG3 ESC"}="2HCG ESC"; $resn_exch{" HD2 ESC"}="1HCD ESC";
$resn_exch{" HD3 ESC"}="2HCD ESC"; $resn_exch{" HE1 ESC"}="1HCE ESC"; $resn_exch{" HE2 ESC"}="2HCE ESC";
$resn_exch{" HE3 ESC"}="3HCE ESC"; $resn_exch{" HG1 ESD"}="1HG  ESD"; $resn_exch{" HG2 ESD"}="2HG  ESD";
$resn_exch{" HD2 ESD"}="2HD  ESD"; $resn_exch{" HD1 ESD"}="1HD  ESD"; $resn_exch{" HA2 ESD"}="2HA  ESD";
$resn_exch{" HA1 ESD"}="1HA  ESD"; $resn_exch{"H121 ESH"}="1H12 ESH"; $resn_exch{"H122 ESH"}="2H12 ESH";
$resn_exch{"H132 ESH"}="2H13 ESH"; $resn_exch{"H131 ESH"}="1H13 ESH"; $resn_exch{"H141 ESH"}="1H14 ESH";
$resn_exch{"H142 ESH"}="2H14 ESH"; $resn_exch{"H231 ESH"}="1H23 ESH"; $resn_exch{"H232 ESH"}="2H23 ESH";
$resn_exch{"H323 ESH"}="3H32 ESH"; $resn_exch{"H322 ESH"}="2H32 ESH"; $resn_exch{"H321 ESH"}="1H32 ESH";
$resn_exch{"H333 ESH"}="3H33 ESH"; $resn_exch{"H332 ESH"}="2H33 ESH"; $resn_exch{"H331 ESH"}="1H33 ESH";
$resn_exch{"HH11 ESI"}="1HH1 ESI"; $resn_exch{"HH12 ESI"}="2HH1 ESI"; $resn_exch{"HH21 ESI"}="1HH2 ESI";
$resn_exch{"HH22 ESI"}="2HH2 ESI"; $resn_exch{" H31 ESL"}="1H3  ESL"; $resn_exch{" H32 ESL"}="2H3  ESL";
$resn_exch{" H41 ESL"}="1H4  ESL"; $resn_exch{" H42 ESL"}="2H4  ESL"; $resn_exch{" H61 ESL"}="1H6  ESL";
$resn_exch{" H62 ESL"}="2H6  ESL"; $resn_exch{" H63 ESL"}="3H6  ESL"; $resn_exch{" H91 ESL"}="1H9  ESL";
$resn_exch{" H92 ESL"}="2H9  ESL"; $resn_exch{"H141 ESL"}="1H14 ESL"; $resn_exch{"H142 ESL"}="2H14 ESL";
$resn_exch{"H151 ESL"}="1H15 ESL"; $resn_exch{"H152 ESL"}="2H15 ESL"; $resn_exch{"H111 ESM"}="1H11 ESM";
$resn_exch{"H112 ESM"}="2H11 ESM"; $resn_exch{"H121 ESM"}="1H12 ESM"; $resn_exch{"H122 ESM"}="2H12 ESM";
$resn_exch{"H14C ESM"}="CH14 ESM"; $resn_exch{"H151 ESM"}="1H15 ESM"; $resn_exch{"H152 ESM"}="2H15 ESM";
$resn_exch{"H161 ESM"}="1H16 ESM"; $resn_exch{"H162 ESM"}="2H16 ESM"; $resn_exch{"H17C ESM"}="CH17 ESM";
$resn_exch{"H181 ESM"}="1H18 ESM"; $resn_exch{"H182 ESM"}="2H18 ESM"; $resn_exch{"H183 ESM"}="3H18 ESM";
$resn_exch{"HC61 ESM"}="1HC6 ESM"; $resn_exch{"HC62 ESM"}="2HC6 ESM"; $resn_exch{"HC71 ESM"}="1HC7 ESM";
$resn_exch{"HC72 ESM"}="2HC7 ESM"; $resn_exch{"H17O ESM"}="OH17 ESM"; $resn_exch{"H191 ESM"}="1H19 ESM";
$resn_exch{"H192 ESM"}="2H19 ESM"; $resn_exch{"H193 ESM"}="3H19 ESM"; $resn_exch{"HOP2 ESO"}="2HOP ESO";
$resn_exch{"HOP3 ESO"}="3HOP ESO"; $resn_exch{" H61 ESO"}="1H6  ESO"; $resn_exch{" H62 ESO"}="2H6  ESO";
$resn_exch{" H71 ESO"}="1H7  ESO"; $resn_exch{" H72 ESO"}="2H7  ESO"; $resn_exch{"H111 ESO"}="1H11 ESO";
$resn_exch{"H112 ESO"}="2H11 ESO"; $resn_exch{"H121 ESO"}="1H12 ESO"; $resn_exch{"H122 ESO"}="2H12 ESO";
$resn_exch{"H151 ESO"}="1H15 ESO"; $resn_exch{"H152 ESO"}="2H15 ESO"; $resn_exch{"H161 ESO"}="1H16 ESO";
$resn_exch{"H162 ESO"}="2H16 ESO"; $resn_exch{"H181 ESO"}="1H18 ESO"; $resn_exch{"H182 ESO"}="2H18 ESO";
$resn_exch{"H183 ESO"}="3H18 ESO"; $resn_exch{"HH11 ESP"}="1HH1 ESP"; $resn_exch{"HH12 ESP"}="2HH1 ESP";
$resn_exch{"HH21 ESP"}="1HH2 ESP"; $resn_exch{"HH22 ESP"}="2HH2 ESP"; $resn_exch{"H2C1 ESR"}="1H2C ESR";
$resn_exch{"H2C2 ESR"}="2H2C ESR"; $resn_exch{"H5C1 ESR"}="1H5C ESR"; $resn_exch{"H5C2 ESR"}="2H5C ESR";
$resn_exch{"H6C1 ESR"}="1H6C ESR"; $resn_exch{"H6C2 ESR"}="2H6C ESR"; $resn_exch{"H101 ESR"}="1H10 ESR";
$resn_exch{"H102 ESR"}="2H10 ESR"; $resn_exch{"H111 ESR"}="1H11 ESR"; $resn_exch{"H112 ESR"}="2H11 ESR";
$resn_exch{"H191 ESR"}="1H19 ESR"; $resn_exch{"H192 ESR"}="2H19 ESR"; $resn_exch{"H241 ESR"}="1H24 ESR";
$resn_exch{"H242 ESR"}="2H24 ESR"; $resn_exch{"H251 ESR"}="1H25 ESR"; $resn_exch{"H252 ESR"}="2H25 ESR";
$resn_exch{"H271 ESR"}="1H27 ESR"; $resn_exch{"H272 ESR"}="2H27 ESR"; $resn_exch{"H273 ESR"}="3H27 ESR";
$resn_exch{"H181 ESR"}="1H18 ESR"; $resn_exch{"H182 ESR"}="2H18 ESR"; $resn_exch{" H61 EST"}="1H6  EST";
$resn_exch{" H62 EST"}="2H6  EST"; $resn_exch{" H71 EST"}="1H7  EST"; $resn_exch{" H72 EST"}="2H7  EST";
$resn_exch{"H111 EST"}="1H11 EST"; $resn_exch{"H112 EST"}="2H11 EST"; $resn_exch{"H121 EST"}="1H12 EST";
$resn_exch{"H122 EST"}="2H12 EST"; $resn_exch{"H151 EST"}="1H15 EST"; $resn_exch{"H152 EST"}="2H15 EST";
$resn_exch{"H161 EST"}="1H16 EST"; $resn_exch{"H162 EST"}="2H16 EST"; $resn_exch{"H181 EST"}="1H18 EST";
$resn_exch{"H182 EST"}="2H18 EST"; $resn_exch{"H183 EST"}="3H18 EST"; $resn_exch{"HH11 ESX"}="1HH1 ESX";
$resn_exch{"HH12 ESX"}="2HH1 ESX"; $resn_exch{"HH21 ESX"}="1HH2 ESX"; $resn_exch{"HH22 ESX"}="2HH2 ESX";
$resn_exch{" H51 ESY"}="1H5  ESY"; $resn_exch{" H52 ESY"}="2H5  ESY"; $resn_exch{"H131 ESY"}="1H13 ESY";
$resn_exch{"H132 ESY"}="2H13 ESY"; $resn_exch{"H141 ESY"}="1H14 ESY"; $resn_exch{"H142 ESY"}="2H14 ESY";
$resn_exch{"H151 ESY"}="1H15 ESY"; $resn_exch{"H152 ESY"}="2H15 ESY"; $resn_exch{"H161 ESY"}="1H16 ESY";
$resn_exch{"H162 ESY"}="2H16 ESY"; $resn_exch{"H241 ESY"}="1H24 ESY"; $resn_exch{"H242 ESY"}="2H24 ESY";
$resn_exch{"H251 ESY"}="1H25 ESY"; $resn_exch{"H252 ESY"}="2H25 ESY"; $resn_exch{"H253 ESY"}="3H25 ESY";
$resn_exch{"H261 ESY"}="1H26 ESY"; $resn_exch{"H262 ESY"}="2H26 ESY"; $resn_exch{"H263 ESY"}="3H26 ESY";
$resn_exch{"H311 ESY"}="1H31 ESY"; $resn_exch{"H312 ESY"}="2H31 ESY"; $resn_exch{"H401 ESY"}="1H40 ESY";
$resn_exch{"H402 ESY"}="2H40 ESY"; $resn_exch{"H403 ESY"}="3H40 ESY"; $resn_exch{"H211  ET"}="1H21  ET";
$resn_exch{"H212  ET"}="2H21  ET"; $resn_exch{"H221  ET"}="1H22  ET"; $resn_exch{"H222  ET"}="2H22  ET";
$resn_exch{"H223  ET"}="3H22  ET"; $resn_exch{"H231  ET"}="1H23  ET"; $resn_exch{"H232  ET"}="2H23  ET";
$resn_exch{"H241  ET"}="1H24  ET"; $resn_exch{"H242  ET"}="2H24  ET"; $resn_exch{" HA1 ETA"}="1HA  ETA";
$resn_exch{" HA2 ETA"}="2HA  ETA"; $resn_exch{" HN1 ETA"}="1HN  ETA"; $resn_exch{" HN2 ETA"}="2HN  ETA";
$resn_exch{" HB1 ETA"}="1HB  ETA"; $resn_exch{" HB2 ETA"}="2HB  ETA"; $resn_exch{"H6A1 ETB"}="1H6A ETB";
$resn_exch{"H6A2 ETB"}="2H6A ETB"; $resn_exch{"H5B1 ETB"}="1H5B ETB"; $resn_exch{"H5B2 ETB"}="2H5B ETB";
$resn_exch{"HCP1 ETB"}="1HCP ETB"; $resn_exch{"HCP2 ETB"}="2HCP ETB"; $resn_exch{"HDP1 ETB"}="1HDP ETB";
$resn_exch{"HDP2 ETB"}="2HDP ETB"; $resn_exch{"HDP3 ETB"}="3HDP ETB"; $resn_exch{"HEP1 ETB"}="1HEP ETB";
$resn_exch{"HEP2 ETB"}="2HEP ETB"; $resn_exch{"HEP3 ETB"}="3HEP ETB"; $resn_exch{"H7P1 ETB"}="1H7P ETB";
$resn_exch{"H7P2 ETB"}="2H7P ETB"; $resn_exch{"H6P1 ETB"}="1H6P ETB"; $resn_exch{"H6P2 ETB"}="2H6P ETB";
$resn_exch{"H3P1 ETB"}="1H3P ETB"; $resn_exch{"H3P2 ETB"}="2H3P ETB"; $resn_exch{"H3P3 ETB"}="3H3P ETB";
$resn_exch{"H221 ETC"}="1H22 ETC"; $resn_exch{"H222 ETC"}="2H22 ETC"; $resn_exch{"H223 ETC"}="3H22 ETC";
$resn_exch{"H211 ETC"}="1H21 ETC"; $resn_exch{"H212 ETC"}="2H21 ETC"; $resn_exch{"H121 ETC"}="1H12 ETC";
$resn_exch{"H122 ETC"}="2H12 ETC"; $resn_exch{"HC61 ETC"}="1HC6 ETC"; $resn_exch{"HC62 ETC"}="2HC6 ETC";
$resn_exch{"H201 ETC"}="1H20 ETC"; $resn_exch{"H202 ETC"}="2H20 ETC"; $resn_exch{"H191 ETC"}="1H19 ETC";
$resn_exch{"H192 ETC"}="2H19 ETC"; $resn_exch{"H193 ETC"}="3H19 ETC"; $resn_exch{"H121 ETE"}="1H12 ETE";
$resn_exch{"H122 ETE"}="2H12 ETE"; $resn_exch{"H221 ETE"}="1H22 ETE"; $resn_exch{"H222 ETE"}="2H22 ETE";
$resn_exch{"H131 ETE"}="1H13 ETE"; $resn_exch{"H132 ETE"}="2H13 ETE"; $resn_exch{"H231 ETE"}="1H23 ETE";
$resn_exch{"H232 ETE"}="2H23 ETE"; $resn_exch{"H141 ETE"}="1H14 ETE"; $resn_exch{"H142 ETE"}="2H14 ETE";
$resn_exch{"H241 ETE"}="1H24 ETE"; $resn_exch{"H242 ETE"}="2H24 ETE"; $resn_exch{"H151 ETE"}="1H15 ETE";
$resn_exch{"H152 ETE"}="2H15 ETE"; $resn_exch{"H251 ETE"}="1H25 ETE"; $resn_exch{"H252 ETE"}="2H25 ETE";
$resn_exch{"H261 ETE"}="1H26 ETE"; $resn_exch{"H262 ETE"}="2H26 ETE"; $resn_exch{"H263 ETE"}="3H26 ETE";
$resn_exch{" H21 ETF"}="1H2  ETF"; $resn_exch{" H22 ETF"}="2H2  ETF"; $resn_exch{" H11 ETH"}="1H1  ETH";
$resn_exch{" H12 ETH"}="2H1  ETH"; $resn_exch{" H21 ETH"}="1H2  ETH"; $resn_exch{" H22 ETH"}="2H2  ETH";
$resn_exch{" H23 ETH"}="3H2  ETH"; $resn_exch{" H11 ETI"}="1H1  ETI"; $resn_exch{" H12 ETI"}="2H1  ETI";
$resn_exch{" H21 ETI"}="1H2  ETI"; $resn_exch{" H22 ETI"}="2H2  ETI"; $resn_exch{" H23 ETI"}="3H2  ETI";
$resn_exch{" H11 ETM"}="1H1  ETM"; $resn_exch{" H12 ETM"}="2H1  ETM"; $resn_exch{" H21 ETM"}="1H2  ETM";
$resn_exch{" H22 ETM"}="2H2  ETM"; $resn_exch{" H31 ETM"}="1H3  ETM"; $resn_exch{" H32 ETM"}="2H3  ETM";
$resn_exch{" H33 ETM"}="3H3  ETM"; $resn_exch{" H41 ETM"}="1H4  ETM"; $resn_exch{" H42 ETM"}="2H4  ETM";
$resn_exch{" H43 ETM"}="3H4  ETM"; $resn_exch{" H51 ETM"}="1H5  ETM"; $resn_exch{" H52 ETM"}="2H5  ETM";
$resn_exch{" H53 ETM"}="3H5  ETM"; $resn_exch{" H1  ETN"}="1H   ETN"; $resn_exch{" H2  ETN"}="2H   ETN";
$resn_exch{" H3  ETN"}="3H   ETN"; $resn_exch{" H11 ETN"}="1H1  ETN"; $resn_exch{" H12 ETN"}="2H1  ETN";
$resn_exch{" H21 ETN"}="1H2  ETN"; $resn_exch{" H22 ETN"}="2H2  ETN"; $resn_exch{" H23 ETN"}="3H2  ETN";
$resn_exch{" HB1 ETO"}="1HB  ETO"; $resn_exch{" HB2 ETO"}="2HB  ETO"; $resn_exch{" HB3 ETO"}="3HB  ETO";
$resn_exch{" HA1 ETO"}="1HA  ETO"; $resn_exch{" HA2 ETO"}="2HA  ETO"; $resn_exch{" H21 ETR"}="1H2  ETR";
$resn_exch{" H22 ETR"}="2H2  ETR"; $resn_exch{" H31 ETR"}="1H3  ETR"; $resn_exch{" H32 ETR"}="2H3  ETR";
$resn_exch{" H41 ETR"}="1H4  ETR"; $resn_exch{" H42 ETR"}="2H4  ETR"; $resn_exch{"H161 ETR"}="1H16 ETR";
$resn_exch{"H162 ETR"}="2H16 ETR"; $resn_exch{"H163 ETR"}="3H16 ETR"; $resn_exch{"H171 ETR"}="1H17 ETR";
$resn_exch{"H172 ETR"}="2H17 ETR"; $resn_exch{"H173 ETR"}="3H17 ETR"; $resn_exch{"H181 ETR"}="1H18 ETR";
$resn_exch{"H182 ETR"}="2H18 ETR"; $resn_exch{"H183 ETR"}="3H18 ETR"; $resn_exch{"H191 ETR"}="1H19 ETR";
$resn_exch{"H192 ETR"}="2H19 ETR"; $resn_exch{"H193 ETR"}="3H19 ETR"; $resn_exch{"H201 ETR"}="1H20 ETR";
$resn_exch{"H202 ETR"}="2H20 ETR"; $resn_exch{"H203 ETR"}="3H20 ETR"; $resn_exch{"H211 ETR"}="1H21 ETR";
$resn_exch{"H212 ETR"}="2H21 ETR"; $resn_exch{"H221 ETR"}="1H22 ETR"; $resn_exch{"H222 ETR"}="2H22 ETR";
$resn_exch{"H223 ETR"}="3H22 ETR"; $resn_exch{"HN31 ETS"}="1HN3 ETS"; $resn_exch{"HN32 ETS"}="2HN3 ETS";
$resn_exch{"H151 ETS"}="1H15 ETS"; $resn_exch{"H152 ETS"}="2H15 ETS"; $resn_exch{"H153 ETS"}="3H15 ETS";
$resn_exch{"H181 ETS"}="1H18 ETS"; $resn_exch{"H182 ETS"}="2H18 ETS"; $resn_exch{"H191 ETS"}="1H19 ETS";
$resn_exch{"H192 ETS"}="2H19 ETS"; $resn_exch{"H193 ETS"}="3H19 ETS"; $resn_exch{"HN61 ETV"}="1HN6 ETV";
$resn_exch{"HN62 ETV"}="2HN6 ETV"; $resn_exch{"H161 ETV"}="1H16 ETV"; $resn_exch{"H162 ETV"}="2H16 ETV";
$resn_exch{"H141 ETV"}="1H14 ETV"; $resn_exch{"H142 ETV"}="2H14 ETV"; $resn_exch{" H21 ETX"}="1H2  ETX";
$resn_exch{" H22 ETX"}="2H2  ETX"; $resn_exch{" H31 ETX"}="1H3  ETX"; $resn_exch{" H32 ETX"}="2H3  ETX";
$resn_exch{" H41 ETX"}="1H4  ETX"; $resn_exch{" H42 ETX"}="2H4  ETX"; $resn_exch{" H43 ETX"}="3H4  ETX";
$resn_exch{" H11 ETX"}="1H1  ETX"; $resn_exch{" H12 ETX"}="2H1  ETX"; $resn_exch{" H81 ETY"}="1H8  ETY";
$resn_exch{" H82 ETY"}="2H8  ETY"; $resn_exch{" H83 ETY"}="3H8  ETY"; $resn_exch{" H71 ETY"}="1H7  ETY";
$resn_exch{" H72 ETY"}="2H7  ETY"; $resn_exch{" H11 EUD"}="1H1  EUD"; $resn_exch{" H12 EUD"}="2H1  EUD";
$resn_exch{" H33 EUD"}="3H3  EUD"; $resn_exch{" H41 EUD"}="1H4  EUD"; $resn_exch{" H42 EUD"}="2H4  EUD";
$resn_exch{" H43 EUD"}="3H4  EUD"; $resn_exch{" H51 EUD"}="1H5  EUD"; $resn_exch{" H52 EUD"}="2H5  EUD";
$resn_exch{" H61 EUD"}="1H6  EUD"; $resn_exch{" H62 EUD"}="2H6  EUD"; $resn_exch{" H71 EUD"}="1H7  EUD";
$resn_exch{" H72 EUD"}="2H7  EUD"; $resn_exch{" H91 EUD"}="1H9  EUD"; $resn_exch{" H92 EUD"}="2H9  EUD";
$resn_exch{" H93 EUD"}="3H9  EUD"; $resn_exch{"H101 EUD"}="1H10 EUD"; $resn_exch{"H102 EUD"}="2H10 EUD";
$resn_exch{"H111 EUD"}="1H11 EUD"; $resn_exch{"H112 EUD"}="2H11 EUD"; $resn_exch{"H121 EUD"}="1H12 EUD";
$resn_exch{"H141 EUD"}="1H14 EUD"; $resn_exch{"H142 EUD"}="2H14 EUD"; $resn_exch{"H143 EUD"}="3H14 EUD";
$resn_exch{"H151 EUD"}="1H15 EUD"; $resn_exch{"H152 EUD"}="2H15 EUD"; $resn_exch{" H81 EUG"}="1H8  EUG";
$resn_exch{" H82 EUG"}="2H8  EUG"; $resn_exch{" H91 EUG"}="1H9  EUG"; $resn_exch{" H92 EUG"}="2H9  EUG";
$resn_exch{" H93 EUG"}="3H9  EUG"; $resn_exch{"HBH1 EXO"}="1HBH EXO"; $resn_exch{"HBH2 EXO"}="2HBH EXO";
$resn_exch{"HBE1 EXO"}="1HBE EXO"; $resn_exch{"HBE2 EXO"}="2HBE EXO"; $resn_exch{" H3  EYS"}="3H   EYS";
$resn_exch{" H2  EYS"}="2H   EYS"; $resn_exch{" H   EYS"}="1H   EYS"; $resn_exch{" HB2 EYS"}="2HB  EYS";
$resn_exch{" HB3 EYS"}="1HB  EYS"; $resn_exch{" HC  EYS"}=" H   EYS"; $resn_exch{"H181 EZT"}="1H18 EZT";
$resn_exch{"H182 EZT"}="2H18 EZT"; $resn_exch{"H183 EZT"}="3H18 EZT"; $resn_exch{"H121 EZT"}="1H12 EZT";
$resn_exch{"H122 EZT"}="2H12 EZT"; $resn_exch{"H111 EZT"}="1H11 EZT"; $resn_exch{"H112 EZT"}="2H11 EZT";
$resn_exch{" H61 EZT"}="1H6  EZT"; $resn_exch{" H62 EZT"}="2H6  EZT"; $resn_exch{" H71 EZT"}="1H7  EZT";
$resn_exch{" H72 EZT"}="2H7  EZT"; $resn_exch{"H151 EZT"}="1H15 EZT"; $resn_exch{"H152 EZT"}="2H15 EZT";
$resn_exch{"H161 EZT"}="1H16 EZT"; $resn_exch{"H162 EZT"}="2H16 EZT"; $resn_exch{" H11 F09"}="1H1  F09";
$resn_exch{" H12 F09"}="2H1  F09"; $resn_exch{" H13 F09"}="3H1  F09"; $resn_exch{" H21 F09"}="1H2  F09";
$resn_exch{" H22 F09"}="2H2  F09"; $resn_exch{" H31 F09"}="1H3  F09"; $resn_exch{" H32 F09"}="2H3  F09";
$resn_exch{" H41 F09"}="1H4  F09"; $resn_exch{" H42 F09"}="2H4  F09"; $resn_exch{" H51 F09"}="1H5  F09";
$resn_exch{" H52 F09"}="2H5  F09"; $resn_exch{" H61 F09"}="1H6  F09"; $resn_exch{" H62 F09"}="2H6  F09";
$resn_exch{" H71 F09"}="1H7  F09"; $resn_exch{" H72 F09"}="2H7  F09"; $resn_exch{" H81 F09"}="1H8  F09";
$resn_exch{" H82 F09"}="2H8  F09"; $resn_exch{" H91 F09"}="1H9  F09"; $resn_exch{" H92 F09"}="2H9  F09";
$resn_exch{"H171 F10"}="1H17 F10"; $resn_exch{"H172 F10"}="2H17 F10"; $resn_exch{"H181 F10"}="1H18 F10";
$resn_exch{"H182 F10"}="2H18 F10"; $resn_exch{"H3C1 F11"}="1H3C F11"; $resn_exch{"H3C2 F11"}="2H3C F11";
$resn_exch{"H4C1 F11"}="1H4C F11"; $resn_exch{"H4C2 F11"}="2H4C F11"; $resn_exch{"H5C1 F11"}="1H5C F11";
$resn_exch{"H5C2 F11"}="2H5C F11"; $resn_exch{"H6C1 F11"}="1H6C F11"; $resn_exch{"H6C2 F11"}="2H6C F11";
$resn_exch{"H191 F11"}="1H19 F11"; $resn_exch{"H192 F11"}="2H19 F11"; $resn_exch{"H201 F11"}="1H20 F11";
$resn_exch{"H202 F11"}="2H20 F11"; $resn_exch{"H211 F11"}="1H21 F11"; $resn_exch{"H212 F11"}="2H21 F11";
$resn_exch{"H241 F11"}="1H24 F11"; $resn_exch{"H242 F11"}="2H24 F11"; $resn_exch{"H251 F11"}="1H25 F11";
$resn_exch{"H252 F11"}="2H25 F11"; $resn_exch{"H281 F11"}="1H28 F11"; $resn_exch{"H282 F11"}="2H28 F11";
$resn_exch{"H291 F11"}="1H29 F11"; $resn_exch{"H292 F11"}="2H29 F11"; $resn_exch{"H301 F11"}="1H30 F11";
$resn_exch{"H302 F11"}="2H30 F11"; $resn_exch{"H311 F11"}="1H31 F11"; $resn_exch{"H312 F11"}="2H31 F11";
$resn_exch{"H121 F12"}="1H12 F12"; $resn_exch{"H122 F12"}="2H12 F12"; $resn_exch{"H123 F12"}="3H12 F12";
$resn_exch{"H201 F17"}="1H20 F17"; $resn_exch{"H202 F17"}="2H20 F17"; $resn_exch{"H351 F17"}="1H35 F17";
$resn_exch{"H352 F17"}="2H35 F17"; $resn_exch{"H401 F17"}="1H40 F17"; $resn_exch{"H402 F17"}="2H40 F17";
$resn_exch{"H7C1 F17"}="1H7C F17"; $resn_exch{"H7C2 F17"}="2H7C F17"; $resn_exch{"H161 F18"}="1H16 F18";
$resn_exch{"H162 F18"}="2H16 F18"; $resn_exch{"H181 F18"}="1H18 F18"; $resn_exch{"H182 F18"}="2H18 F18";
$resn_exch{"H161 F19"}="1H16 F19"; $resn_exch{"H162 F19"}="2H16 F19"; $resn_exch{"H181 F19"}="1H18 F19";
$resn_exch{"H182 F19"}="2H18 F19"; $resn_exch{"H191 F1G"}="1H19 F1G"; $resn_exch{"H192 F1G"}="2H19 F1G";
$resn_exch{"H193 F1G"}="3H19 F1G"; $resn_exch{" H31 F1G"}="1H3  F1G"; $resn_exch{" H32 F1G"}="2H3  F1G";
$resn_exch{" H21 F1G"}="1H2  F1G"; $resn_exch{" H22 F1G"}="2H2  F1G"; $resn_exch{" H11 F1P"}="1H1  F1P";
$resn_exch{" H12 F1P"}="2H1  F1P"; $resn_exch{" H61 F1P"}="1H6  F1P"; $resn_exch{" H62 F1P"}="2H6  F1P";
$resn_exch{"H2C1 F20"}="1H2C F20"; $resn_exch{"H2C2 F20"}="2H2C F20"; $resn_exch{"H211 F20"}="1H21 F20";
$resn_exch{"H212 F20"}="2H21 F20"; $resn_exch{"H271 F20"}="1H27 F20"; $resn_exch{"H272 F20"}="2H27 F20";
$resn_exch{"H4C1 F20"}="1H4C F20"; $resn_exch{"H4C2 F20"}="2H4C F20"; $resn_exch{"H121 F21"}="1H12 F21";
$resn_exch{"H122 F21"}="2H12 F21"; $resn_exch{"H131 F21"}="1H13 F21"; $resn_exch{"H132 F21"}="2H13 F21";
$resn_exch{"H151 F21"}="1H15 F21"; $resn_exch{"H152 F21"}="2H15 F21"; $resn_exch{"H141 F21"}="1H14 F21";
$resn_exch{"H142 F21"}="2H14 F21"; $resn_exch{" H11 F21"}="1H1  F21"; $resn_exch{" H12 F21"}="2H1  F21";
$resn_exch{"H111 F21"}="1H11 F21"; $resn_exch{"H112 F21"}="2H11 F21"; $resn_exch{" H41 F21"}="1H4  F21";
$resn_exch{" H42 F21"}="2H4  F21"; $resn_exch{"H101 F23"}="1H10 F23"; $resn_exch{"H102 F23"}="2H10 F23";
$resn_exch{"H111 F23"}="1H11 F23"; $resn_exch{"H112 F23"}="2H11 F23"; $resn_exch{"H121 F23"}="1H12 F23";
$resn_exch{"H122 F23"}="2H12 F23"; $resn_exch{"H131 F23"}="1H13 F23"; $resn_exch{"H132 F23"}="2H13 F23";
$resn_exch{"H141 F23"}="1H14 F23"; $resn_exch{"H142 F23"}="2H14 F23"; $resn_exch{"H151 F23"}="1H15 F23";
$resn_exch{"H152 F23"}="2H15 F23"; $resn_exch{"H161 F23"}="1H16 F23"; $resn_exch{"H162 F23"}="2H16 F23";
$resn_exch{"H171 F23"}="1H17 F23"; $resn_exch{"H172 F23"}="2H17 F23"; $resn_exch{"H181 F23"}="1H18 F23";
$resn_exch{"H182 F23"}="2H18 F23"; $resn_exch{"H191 F23"}="1H19 F23"; $resn_exch{"H192 F23"}="2H19 F23";
$resn_exch{"H201 F23"}="1H20 F23"; $resn_exch{"H202 F23"}="2H20 F23"; $resn_exch{"H211 F23"}="1H21 F23";
$resn_exch{"H212 F23"}="2H21 F23"; $resn_exch{"H221 F23"}="1H22 F23"; $resn_exch{"H222 F23"}="2H22 F23";
$resn_exch{"H231 F23"}="1H23 F23"; $resn_exch{"H232 F23"}="2H23 F23"; $resn_exch{"H233 F23"}="3H23 F23";
$resn_exch{" H61 F24"}="1H6  F24"; $resn_exch{" H62 F24"}="2H6  F24"; $resn_exch{"H211 F24"}="1H21 F24";
$resn_exch{"H212 F24"}="2H21 F24"; $resn_exch{"H213 F24"}="3H21 F24"; $resn_exch{"H121 F24"}="1H12 F24";
$resn_exch{"H122 F24"}="2H12 F24"; $resn_exch{" H71 F24"}="1H7  F24"; $resn_exch{" H72 F24"}="2H7  F24";
$resn_exch{" H73 F24"}="3H7  F24"; $resn_exch{"H161 F24"}="1H16 F24"; $resn_exch{"H162 F24"}="2H16 F24";
$resn_exch{"H131 F24"}="1H13 F24"; $resn_exch{"H132 F24"}="2H13 F24"; $resn_exch{"H133 F24"}="3H13 F24";
$resn_exch{"H201 F24"}="1H20 F24"; $resn_exch{"H202 F24"}="2H20 F24"; $resn_exch{"H111 F24"}="1H11 F24";
$resn_exch{"H112 F24"}="2H11 F24"; $resn_exch{"H101 F24"}="1H10 F24"; $resn_exch{"H102 F24"}="2H10 F24";
$resn_exch{" H51 F24"}="1H5  F24"; $resn_exch{" H52 F24"}="2H5  F24"; $resn_exch{" H41 F24"}="1H4  F24";
$resn_exch{" H42 F24"}="2H4  F24"; $resn_exch{"H231 F25"}="1H23 F25"; $resn_exch{"H232 F25"}="2H23 F25";
$resn_exch{"H121 F25"}="1H12 F25"; $resn_exch{"H122 F25"}="2H12 F25"; $resn_exch{"H131 F25"}="1H13 F25";
$resn_exch{"H132 F25"}="2H13 F25"; $resn_exch{"H142 F25"}="2H14 F25"; $resn_exch{"H141 F25"}="1H14 F25";
$resn_exch{" H33 F25"}="3H3  F25"; $resn_exch{"H301 F29"}="1H30 F29"; $resn_exch{"H302 F29"}="2H30 F29";
$resn_exch{" H31 F29"}="1H3  F29"; $resn_exch{" H32 F29"}="2H3  F29"; $resn_exch{"H171 F29"}="1H17 F29";
$resn_exch{"H172 F29"}="2H17 F29"; $resn_exch{"H01C F2B"}="CH01 F2B"; $resn_exch{"H02C F2B"}="CH02 F2B";
$resn_exch{"H04C F2B"}="CH04 F2B"; $resn_exch{"H05C F2B"}="CH05 F2B"; $resn_exch{"H09N F2B"}="NH09 F2B";
$resn_exch{"H101 F2B"}="1H10 F2B"; $resn_exch{"H102 F2B"}="2H10 F2B"; $resn_exch{"HNP1 F2B"}="1HNP F2B";
$resn_exch{"HNP2 F2B"}="2HNP F2B"; $resn_exch{"H16C F2B"}="CH16 F2B"; $resn_exch{"H17C F2B"}="CH17 F2B";
$resn_exch{"H18C F2B"}="CH18 F2B"; $resn_exch{"H241 F2G"}="1H24 F2G"; $resn_exch{"H242 F2G"}="2H24 F2G";
$resn_exch{"H261 F2G"}="1H26 F2G"; $resn_exch{"H262 F2G"}="2H26 F2G"; $resn_exch{"H271 F2G"}="1H27 F2G";
$resn_exch{"H272 F2G"}="2H27 F2G"; $resn_exch{"H281 F2G"}="1H28 F2G"; $resn_exch{"H282 F2G"}="2H28 F2G";
$resn_exch{"H291 F2G"}="1H29 F2G"; $resn_exch{"H292 F2G"}="2H29 F2G"; $resn_exch{"H301 F2G"}="1H30 F2G";
$resn_exch{"H302 F2G"}="2H30 F2G"; $resn_exch{" H51 F2G"}="1H5  F2G"; $resn_exch{" H52 F2G"}="2H5  F2G";
$resn_exch{" H53 F2G"}="3H5  F2G"; $resn_exch{" H41 F2G"}="1H4  F2G"; $resn_exch{" H42 F2G"}="2H4  F2G";
$resn_exch{" H43 F2G"}="3H4  F2G"; $resn_exch{"H101 F2G"}="1H10 F2G"; $resn_exch{"H102 F2G"}="2H10 F2G";
$resn_exch{"H103 F2G"}="3H10 F2G"; $resn_exch{"H311 F2G"}="1H31 F2G"; $resn_exch{"H312 F2G"}="2H31 F2G";
$resn_exch{"H313 F2G"}="3H31 F2G"; $resn_exch{"H151 F2G"}="1H15 F2G"; $resn_exch{"H152 F2G"}="2H15 F2G";
$resn_exch{" H71 F2I"}="1H7  F2I"; $resn_exch{" H72 F2I"}="2H7  F2I"; $resn_exch{" H73 F2I"}="3H7  F2I";
$resn_exch{"H101 F2I"}="1H10 F2I"; $resn_exch{"H102 F2I"}="2H10 F2I"; $resn_exch{"H131 F2I"}="1H13 F2I";
$resn_exch{"H132 F2I"}="2H13 F2I"; $resn_exch{"H171 F2I"}="1H17 F2I"; $resn_exch{"H172 F2I"}="2H17 F2I";
$resn_exch{" H91 F2I"}="1H9  F2I"; $resn_exch{" H92 F2I"}="2H9  F2I"; $resn_exch{"H111 F2I"}="1H11 F2I";
$resn_exch{"H112 F2I"}="2H11 F2I"; $resn_exch{"H121 F2I"}="1H12 F2I"; $resn_exch{"H122 F2I"}="2H12 F2I";
$resn_exch{"H123 F2I"}="3H12 F2I"; $resn_exch{"H141 F2I"}="1H14 F2I"; $resn_exch{"H142 F2I"}="2H14 F2I";
$resn_exch{"H143 F2I"}="3H14 F2I"; $resn_exch{"H181 F2I"}="1H18 F2I"; $resn_exch{"H182 F2I"}="2H18 F2I";
$resn_exch{"H191 F2I"}="1H19 F2I"; $resn_exch{"H192 F2I"}="2H19 F2I"; $resn_exch{"HCD1 F2N"}="1HCD F2N";
$resn_exch{"HCD2 F2N"}="2HCD F2N"; $resn_exch{"HCE1 F2N"}="1HCE F2N"; $resn_exch{"HCE2 F2N"}="2HCE F2N";
$resn_exch{"HBJ1 F2N"}="1HBJ F2N"; $resn_exch{"HBJ2 F2N"}="2HBJ F2N"; $resn_exch{"HBJ3 F2N"}="3HBJ F2N";
$resn_exch{"HBH1 F2N"}="1HBH F2N"; $resn_exch{"HBH2 F2N"}="2HBH F2N"; $resn_exch{"HBH3 F2N"}="3HBH F2N";
$resn_exch{"HBO1 F2N"}="1HBO F2N"; $resn_exch{"HBO2 F2N"}="2HBO F2N"; $resn_exch{"HBV1 F2N"}="1HBV F2N";
$resn_exch{"HBV2 F2N"}="2HBV F2N"; $resn_exch{"H5'1 F2N"}="1H5* F2N"; $resn_exch{"H5'2 F2N"}="2H5* F2N";
$resn_exch{" H4' F2N"}=" H4* F2N"; $resn_exch{" H3' F2N"}=" H3* F2N"; $resn_exch{"HO3' F2N"}="HO3* F2N";
$resn_exch{" H2' F2N"}=" H2* F2N"; $resn_exch{"HO2' F2N"}="HO2* F2N"; $resn_exch{" H1' F2N"}=" H1* F2N";
$resn_exch{"HN61 F2N"}="1HN6 F2N"; $resn_exch{"HN62 F2N"}="2HN6 F2N"; $resn_exch{"H1C1 F2P"}="1H1C F2P";
$resn_exch{"H1C2 F2P"}="2H1C F2P"; $resn_exch{"H6C1 F2P"}="1H6C F2P"; $resn_exch{"H6C2 F2P"}="2H6C F2P";
$resn_exch{"H2C1 F32"}="1H2C F32"; $resn_exch{"H2C2 F32"}="2H2C F32"; $resn_exch{"H211 F32"}="1H21 F32";
$resn_exch{"H212 F32"}="2H21 F32"; $resn_exch{"H241 F3G"}="1H24 F3G"; $resn_exch{"H242 F3G"}="2H24 F3G";
$resn_exch{" H51 F3G"}="1H5  F3G"; $resn_exch{" H52 F3G"}="2H5  F3G"; $resn_exch{" H53 F3G"}="3H5  F3G";
$resn_exch{" H41 F3G"}="1H4  F3G"; $resn_exch{" H42 F3G"}="2H4  F3G"; $resn_exch{" H43 F3G"}="3H4  F3G";
$resn_exch{"H101 F3G"}="1H10 F3G"; $resn_exch{"H102 F3G"}="2H10 F3G"; $resn_exch{"H103 F3G"}="3H10 F3G";
$resn_exch{"H311 F3G"}="1H31 F3G"; $resn_exch{"H312 F3G"}="2H31 F3G"; $resn_exch{"H313 F3G"}="3H31 F3G";
$resn_exch{"H151 F3G"}="1H15 F3G"; $resn_exch{"H152 F3G"}="2H15 F3G"; $resn_exch{" H11 F3P"}="1H1  F3P";
$resn_exch{" H12 F3P"}="2H1  F3P"; $resn_exch{" H13 F3P"}="3H1  F3P"; $resn_exch{" H41 F3P"}="1H4  F3P";
$resn_exch{" H42 F3P"}="2H4  F3P"; $resn_exch{" H51 F3P"}="1H5  F3P"; $resn_exch{" H52 F3P"}="2H5  F3P";
$resn_exch{" H61 F3P"}="1H6  F3P"; $resn_exch{" H62 F3P"}="2H6  F3P"; $resn_exch{" H81 F3P"}="1H8  F3P";
$resn_exch{" H82 F3P"}="2H8  F3P"; $resn_exch{" H83 F3P"}="3H8  F3P"; $resn_exch{" H91 F3P"}="1H9  F3P";
$resn_exch{" H92 F3P"}="2H9  F3P"; $resn_exch{" H93 F3P"}="3H9  F3P"; $resn_exch{"H101 F3P"}="1H10 F3P";
$resn_exch{"H102 F3P"}="2H10 F3P"; $resn_exch{"H103 F3P"}="3H10 F3P"; $resn_exch{"HM81 F42"}="1HM8 F42";
$resn_exch{"H1'1 F42"}="1H1* F42"; $resn_exch{"H1'2 F42"}="2H1* F42"; $resn_exch{" H2' F42"}=" H2* F42";
$resn_exch{"HO2' F42"}="*HO2 F42"; $resn_exch{" H3' F42"}=" H3* F42"; $resn_exch{"HO3' F42"}="*HO3 F42";
$resn_exch{" H4' F42"}=" H4* F42"; $resn_exch{"HO4' F42"}="*HO4 F42"; $resn_exch{"H5'1 F42"}="1H5* F42";
$resn_exch{"H5'2 F42"}="2H5* F42"; $resn_exch{"HOP2 F42"}="2HOP F42"; $resn_exch{"H1I1 F42"}="1H1I F42";
$resn_exch{"H2I1 F42"}="1H2I F42"; $resn_exch{"H2I2 F42"}="2H2I F42"; $resn_exch{"H2I3 F42"}="3H2I F42";
$resn_exch{"HNH1 F42"}="1HNH F42"; $resn_exch{"H1H1 F42"}="1H1H F42"; $resn_exch{"H3H1 F42"}="1H3H F42";
$resn_exch{"H3H2 F42"}="2H3H F42"; $resn_exch{"H4H1 F42"}="1H4H F42"; $resn_exch{"H4H2 F42"}="2H4H F42";
$resn_exch{"HNG1 F42"}="1HNG F42"; $resn_exch{"H1G1 F42"}="1H1G F42"; $resn_exch{"H3G1 F42"}="1H3G F42";
$resn_exch{"H3G2 F42"}="2H3G F42"; $resn_exch{"H4G1 F42"}="1H4G F42"; $resn_exch{"H4G2 F42"}="2H4G F42";
$resn_exch{"HHA1 F43"}="1HHA F43"; $resn_exch{"HHA2 F43"}="2HHA F43"; $resn_exch{"H5A1 F43"}="1H5A F43";
$resn_exch{"H5A2 F43"}="2H5A F43"; $resn_exch{"HN81 F43"}="1HN8 F43"; $resn_exch{"HN82 F43"}="2HN8 F43";
$resn_exch{"H9A1 F43"}="1H9A F43"; $resn_exch{"H9A2 F43"}="2H9A F43"; $resn_exch{"H9A3 F43"}="3H9A F43";
$resn_exch{"HAA1 F43"}="1HAA F43"; $resn_exch{"HAA2 F43"}="2HAA F43"; $resn_exch{"HBA1 F43"}="1HBA F43";
$resn_exch{"HBA2 F43"}="2HBA F43"; $resn_exch{"HHB1 F43"}="1HHB F43"; $resn_exch{"HHB2 F43"}="2HHB F43";
$resn_exch{"H8B1 F43"}="1H8B F43"; $resn_exch{"H8B2 F43"}="2H8B F43"; $resn_exch{"H9B1 F43"}="1H9B F43";
$resn_exch{"H9B2 F43"}="2H9B F43"; $resn_exch{"H9B3 F43"}="3H9B F43"; $resn_exch{"HAB1 F43"}="1HAB F43";
$resn_exch{"HAB2 F43"}="2HAB F43"; $resn_exch{"HBB1 F43"}="1HBB F43"; $resn_exch{"HBB2 F43"}="2HBB F43";
$resn_exch{"H5C1 F43"}="1H5C F43"; $resn_exch{"H5C2 F43"}="2H5C F43"; $resn_exch{"H8C1 F43"}="1H8C F43";
$resn_exch{"H8C2 F43"}="2H8C F43"; $resn_exch{"H9C1 F43"}="1H9C F43"; $resn_exch{"H9C2 F43"}="2H9C F43";
$resn_exch{"H5D1 F43"}="1H5D F43"; $resn_exch{"H5D2 F43"}="2H5D F43"; $resn_exch{"H6D1 F43"}="1H6D F43";
$resn_exch{"H6D2 F43"}="2H6D F43"; $resn_exch{"H9D1 F43"}="1H9D F43"; $resn_exch{"H9D2 F43"}="2H9D F43";
$resn_exch{"HH31 F50"}="1HH3 F50"; $resn_exch{"HH32 F50"}="2HH3 F50"; $resn_exch{"HH33 F50"}="3HH3 F50";
$resn_exch{"H01C F6B"}="CH01 F6B"; $resn_exch{"H02C F6B"}="CH02 F6B"; $resn_exch{"H04C F6B"}="CH04 F6B";
$resn_exch{"H05C F6B"}="CH05 F6B"; $resn_exch{"H09N F6B"}="NH09 F6B"; $resn_exch{"H101 F6B"}="1H10 F6B";
$resn_exch{"H102 F6B"}="2H10 F6B"; $resn_exch{"HNP1 F6B"}="1HNP F6B"; $resn_exch{"HNP2 F6B"}="2HNP F6B";
$resn_exch{"H17C F6B"}="CH17 F6B"; $resn_exch{"H18C F6B"}="CH18 F6B"; $resn_exch{"H19C F6B"}="CH19 F6B";
$resn_exch{"H151 F6F"}="1H15 F6F"; $resn_exch{"H152 F6F"}="2H15 F6F"; $resn_exch{"H161 F6F"}="1H16 F6F";
$resn_exch{"H162 F6F"}="2H16 F6F"; $resn_exch{" H11 F6P"}="1H1  F6P"; $resn_exch{" H12 F6P"}="2H1  F6P";
$resn_exch{" H61 F6P"}="1H6  F6P"; $resn_exch{" H62 F6P"}="2H6  F6P"; $resn_exch{"HOP2 F6P"}="2HOP F6P";
$resn_exch{"HOP3 F6P"}="3HOP F6P"; $resn_exch{"H1C1 F6R"}="1H1C F6R"; $resn_exch{"H1C2 F6R"}="2H1C F6R";
$resn_exch{"H6C1 F6R"}="1H6C F6R"; $resn_exch{"H6C2 F6R"}="2H6C F6R"; $resn_exch{" H11 F77"}="1H1  F77";
$resn_exch{" H12 F77"}="2H1  F77"; $resn_exch{" H13 F77"}="3H1  F77"; $resn_exch{"H21A F77"}="1H2  F77";
$resn_exch{"H22A F77"}="2H2  F77"; $resn_exch{" H91 F77"}="1H9  F77"; $resn_exch{" H92 F77"}="2H9  F77";
$resn_exch{" H93 F77"}="3H9  F77"; $resn_exch{" H71 F79"}="1H7  F79"; $resn_exch{" H72 F79"}="2H7  F79";
$resn_exch{" H81 F79"}="1H8  F79"; $resn_exch{" H82 F79"}="2H8  F79"; $resn_exch{" H83 F79"}="3H8  F79";
$resn_exch{"H101 F79"}="1H10 F79"; $resn_exch{"H102 F79"}="2H10 F79"; $resn_exch{"H111 F79"}="1H11 F79";
$resn_exch{"H112 F79"}="2H11 F79"; $resn_exch{"H121 F79"}="1H12 F79"; $resn_exch{"H122 F79"}="2H12 F79";
$resn_exch{" H11 F83"}="1H1  F83"; $resn_exch{"H12A F83"}="2H1  F83"; $resn_exch{" H31 F83"}="1H3  F83";
$resn_exch{" H32 F83"}="2H3  F83"; $resn_exch{"H101 F83"}="1H10 F83"; $resn_exch{"H102 F83"}="2H10 F83";
$resn_exch{"H3M1 F89"}="1H3M F89"; $resn_exch{"H3M2 F89"}="2H3M F89"; $resn_exch{"H3M3 F89"}="3H3M F89";
$resn_exch{"H111 F89"}="1H11 F89"; $resn_exch{"H112 F89"}="2H11 F89"; $resn_exch{"HN12 F89"}="2HN1 F89";
$resn_exch{"H191 F89"}="1H19 F89"; $resn_exch{"H192 F89"}="2H19 F89"; $resn_exch{" HB1 F89"}="1HB  F89";
$resn_exch{" HB2 F89"}="2HB  F89"; $resn_exch{" HG1 F89"}="1HG  F89"; $resn_exch{" HG2 F89"}="2HG  F89";
$resn_exch{"HOE2 F89"}="2HOE F89"; $resn_exch{"H141 F9F"}="1H14 F9F"; $resn_exch{"H142 F9F"}="2H14 F9F";
$resn_exch{"H151 F9F"}="1H15 F9F"; $resn_exch{"H152 F9F"}="2H15 F9F"; $resn_exch{" H91  FA"}="1H9   FA";
$resn_exch{" H92  FA"}="2H9   FA"; $resn_exch{"HB11  FA"}="1HB1  FA"; $resn_exch{"HB12  FA"}="2HB1  FA";
$resn_exch{"HG11  FA"}="1HG1  FA"; $resn_exch{"HG12  FA"}="2HG1  FA"; $resn_exch{"HA21  FA"}="1HA2  FA";
$resn_exch{"HA22  FA"}="2HA2  FA"; $resn_exch{"HOE1  FA"}="1HOE  FA"; $resn_exch{"H2C1 FA1"}="1H2C FA1";
$resn_exch{"H2C2 FA1"}="2H2C FA1"; $resn_exch{"HOP2 FA2"}="2HOP FA2"; $resn_exch{" H2' FA2"}=" H2* FA2";
$resn_exch{"H4'1 FA2"}="1H4* FA2"; $resn_exch{"H4'2 FA2"}="2H4* FA2"; $resn_exch{" H1' FA2"}=" H1* FA2";
$resn_exch{" H3' FA2"}=" H3* FA2"; $resn_exch{"HO2' FA2"}="2HO* FA2"; $resn_exch{"HOP3 FA2"}="3HOP FA2";
$resn_exch{"H2C1 FA3"}="1H2C FA3"; $resn_exch{"H2C2 FA3"}="2H2C FA3"; $resn_exch{" H71 FA4"}="1H7  FA4";
$resn_exch{" H72 FA4"}="2H7  FA4"; $resn_exch{" H81 FA4"}="1H8  FA4"; $resn_exch{" H82 FA4"}="2H8  FA4";
$resn_exch{" H91 FA4"}="1H9  FA4"; $resn_exch{" H92 FA4"}="2H9  FA4"; $resn_exch{"H101 FA4"}="1H10 FA4";
$resn_exch{"H102 FA4"}="2H10 FA4"; $resn_exch{"H111 FA4"}="1H11 FA4"; $resn_exch{"H112 FA4"}="2H11 FA4";
$resn_exch{"H121 FA4"}="1H12 FA4"; $resn_exch{"H122 FA4"}="2H12 FA4"; $resn_exch{"H131 FA4"}="1H13 FA4";
$resn_exch{"H132 FA4"}="2H13 FA4"; $resn_exch{"H141 FA4"}="1H14 FA4"; $resn_exch{"H142 FA4"}="2H14 FA4";
$resn_exch{"H151 FA4"}="1H15 FA4"; $resn_exch{"H152 FA4"}="2H15 FA4"; $resn_exch{"H201 FA4"}="1H20 FA4";
$resn_exch{"H202 FA4"}="2H20 FA4"; $resn_exch{"H231 FA4"}="1H23 FA4"; $resn_exch{"H232 FA4"}="2H23 FA4";
$resn_exch{"H241 FA4"}="1H24 FA4"; $resn_exch{"H242 FA4"}="2H24 FA4"; $resn_exch{"H251 FA4"}="1H25 FA4";
$resn_exch{"H252 FA4"}="2H25 FA4"; $resn_exch{"H261 FA4"}="1H26 FA4"; $resn_exch{"H262 FA4"}="2H26 FA4";
$resn_exch{"H271 FA4"}="1H27 FA4"; $resn_exch{"H272 FA4"}="2H27 FA4"; $resn_exch{"H301 FA4"}="1H30 FA4";
$resn_exch{"H302 FA4"}="2H30 FA4"; $resn_exch{" HN1 FA5"}="1HN  FA5"; $resn_exch{" HN2 FA5"}="2HN  FA5";
$resn_exch{" HB1 FA5"}="1HB  FA5"; $resn_exch{" HB2 FA5"}="2HB  FA5"; $resn_exch{"HOP2 FA5"}="2HOP FA5";
$resn_exch{" H5' FA5"}="1H5* FA5"; $resn_exch{"H5'' FA5"}="2H5* FA5"; $resn_exch{" H4' FA5"}=" H4* FA5";
$resn_exch{" H3' FA5"}=" H3* FA5"; $resn_exch{"HO'3 FA5"}="3HO* FA5"; $resn_exch{" H2' FA5"}=" H2* FA5";
$resn_exch{"HO2' FA5"}="2HO* FA5"; $resn_exch{" H1' FA5"}=" H1* FA5"; $resn_exch{" H61 FA5"}="1H6  FA5";
$resn_exch{" H62 FA5"}="2H6  FA5"; $resn_exch{"H2C1 FA6"}="1H2C FA6"; $resn_exch{"H2C2 FA6"}="2H2C FA6";
$resn_exch{"H6C1 FA6"}="1H6C FA6"; $resn_exch{"H6C2 FA6"}="2H6C FA6"; $resn_exch{"HB1' FA9"}="1HB' FA9";
$resn_exch{"HB2' FA9"}="2HB' FA9"; $resn_exch{"H7M1 FA9"}="1H7M FA9"; $resn_exch{"H7M2 FA9"}="2H7M FA9";
$resn_exch{"H7M3 FA9"}="3H7M FA9"; $resn_exch{"H8M1 FA9"}="1H8M FA9"; $resn_exch{"H8M2 FA9"}="2H8M FA9";
$resn_exch{"H8M3 FA9"}="3H8M FA9"; $resn_exch{"H11' FA9"}="1H1* FA9"; $resn_exch{"H12' FA9"}="2H1* FA9";
$resn_exch{" H2' FA9"}=" H2* FA9"; $resn_exch{" H3' FA9"}=" H3* FA9"; $resn_exch{" H4' FA9"}=" H4* FA9";
$resn_exch{"H51' FA9"}="1H5* FA9"; $resn_exch{"H52' FA9"}="2H5* FA9"; $resn_exch{"HA1' FA9"}="1HA' FA9";
$resn_exch{"HA2' FA9"}="2HA' FA9"; $resn_exch{"HAA1 FA9"}="1HAA FA9"; $resn_exch{"HAA2 FA9"}="2HAA FA9";
$resn_exch{"HOA2 FAA"}="2HOA FAA"; $resn_exch{"H51A FAA"}="AH51 FAA"; $resn_exch{"H52A FAA"}="AH52 FAA";
$resn_exch{" H4B FAA"}="AH4* FAA"; $resn_exch{" H3B FAA"}="AH3* FAA"; $resn_exch{"HO3A FAA"}="AHO3 FAA";
$resn_exch{" H2B FAA"}="AH2* FAA"; $resn_exch{"HO2A FAA"}="AHO2 FAA"; $resn_exch{" H1B FAA"}="AH1* FAA";
$resn_exch{" H8A FAA"}=" AH8 FAA"; $resn_exch{"H61A FAA"}="AH61 FAA"; $resn_exch{"H62A FAA"}="AH62 FAA";
$resn_exch{" H2A FAA"}=" AH2 FAA"; $resn_exch{"HM71 FAA"}="1HM7 FAA"; $resn_exch{"HM72 FAA"}="2HM7 FAA";
$resn_exch{"HM73 FAA"}="3HM7 FAA"; $resn_exch{"HM81 FAA"}="1HM8 FAA"; $resn_exch{"HM82 FAA"}="2HM8 FAA";
$resn_exch{"HM83 FAA"}="3HM8 FAA"; $resn_exch{"H1'1 FAA"}="1H1* FAA"; $resn_exch{"H1'2 FAA"}="2H1* FAA";
$resn_exch{" H2' FAA"}=" H2* FAA"; $resn_exch{"HO2' FAA"}="*HO2 FAA"; $resn_exch{" H3' FAA"}=" H3* FAA";
$resn_exch{"HO3' FAA"}="*HO3 FAA"; $resn_exch{" H4' FAA"}=" H4* FAA"; $resn_exch{"HO4' FAA"}="*HO4 FAA";
$resn_exch{"H5'1 FAA"}="1H5* FAA"; $resn_exch{"H5'2 FAA"}="2H5* FAA"; $resn_exch{"H7P1 FAA"}="1H7P FAA";
$resn_exch{"H7P2 FAA"}="2H7P FAA"; $resn_exch{"HOP4 FAA"}="4HOP FAA"; $resn_exch{"HOA2 FAB"}="2HOA FAB";
$resn_exch{"H51A FAB"}="AH51 FAB"; $resn_exch{"H52A FAB"}="AH52 FAB"; $resn_exch{" H4B FAB"}="AH4* FAB";
$resn_exch{" H3B FAB"}="AH3* FAB"; $resn_exch{"HO3A FAB"}="AHO3 FAB"; $resn_exch{" H2B FAB"}="AH2* FAB";
$resn_exch{"HO2A FAB"}="AHO2 FAB"; $resn_exch{" H1B FAB"}="AH1* FAB"; $resn_exch{" H8A FAB"}=" AH8 FAB";
$resn_exch{"H61A FAB"}="AH61 FAB"; $resn_exch{"H62A FAB"}="AH62 FAB"; $resn_exch{" H2A FAB"}=" AH2 FAB";
$resn_exch{"HM71 FAB"}="1HM7 FAB"; $resn_exch{"HM72 FAB"}="2HM7 FAB"; $resn_exch{"HM73 FAB"}="3HM7 FAB";
$resn_exch{"HM81 FAB"}="1HM8 FAB"; $resn_exch{"HM82 FAB"}="2HM8 FAB"; $resn_exch{"HM83 FAB"}="3HM8 FAB";
$resn_exch{"H1'1 FAB"}="1H1* FAB"; $resn_exch{"H1'2 FAB"}="2H1* FAB"; $resn_exch{" H2' FAB"}=" H2* FAB";
$resn_exch{"HO2' FAB"}="*HO2 FAB"; $resn_exch{" H3D FAB"}=" H3* FAB"; $resn_exch{"HO3' FAB"}="*HO3 FAB";
$resn_exch{" H4' FAB"}=" H4* FAB"; $resn_exch{"HO4' FAB"}="*HO4 FAB"; $resn_exch{"H5'1 FAB"}="1H5* FAB";
$resn_exch{"H5'2 FAB"}="2H5* FAB"; $resn_exch{"H4'1 FAB"}="1H4' FAB"; $resn_exch{"H4'2 FAB"}="2H4' FAB";
$resn_exch{"H4'3 FAB"}="3H4' FAB"; $resn_exch{"H51X FAB"}="1H5' FAB"; $resn_exch{"H52X FAB"}="2H5' FAB";
$resn_exch{"H5'3 FAB"}="3H5' FAB"; $resn_exch{"HOA2 FAD"}="2HOA FAD"; $resn_exch{"H51A FAD"}="AH51 FAD";
$resn_exch{"H52A FAD"}="AH52 FAD"; $resn_exch{" H4B FAD"}="AH4* FAD"; $resn_exch{" H3B FAD"}="AH3* FAD";
$resn_exch{"HO3A FAD"}="AHO3 FAD"; $resn_exch{" H2B FAD"}="AH2* FAD"; $resn_exch{"HO2A FAD"}="AHO2 FAD";
$resn_exch{" H1B FAD"}="AH1* FAD"; $resn_exch{" H8A FAD"}=" AH8 FAD"; $resn_exch{"H61A FAD"}="AH61 FAD";
$resn_exch{"H62A FAD"}="AH62 FAD"; $resn_exch{" H2A FAD"}=" AH2 FAD"; $resn_exch{"HM71 FAD"}="1HM7 FAD";
$resn_exch{"HM72 FAD"}="2HM7 FAD"; $resn_exch{"HM73 FAD"}="3HM7 FAD"; $resn_exch{"HM81 FAD"}="1HM8 FAD";
$resn_exch{"HM82 FAD"}="2HM8 FAD"; $resn_exch{"HM83 FAD"}="3HM8 FAD"; $resn_exch{"H1'1 FAD"}="1H1* FAD";
$resn_exch{"H1'2 FAD"}="2H1* FAD"; $resn_exch{" H2' FAD"}=" H2* FAD"; $resn_exch{"HO2' FAD"}="*HO2 FAD";
$resn_exch{" H3' FAD"}=" H3* FAD"; $resn_exch{"HO3' FAD"}="*HO3 FAD"; $resn_exch{" H4' FAD"}=" H4* FAD";
$resn_exch{"HO4' FAD"}="*HO4 FAD"; $resn_exch{"H5'1 FAD"}="1H5* FAD"; $resn_exch{"H5'2 FAD"}="2H5* FAD";
$resn_exch{"HOP2 FAD"}="2HOP FAD"; $resn_exch{"HOA2 FAE"}="2HOA FAE"; $resn_exch{"H51A FAE"}="AH51 FAE";
$resn_exch{"H52A FAE"}="AH52 FAE"; $resn_exch{" H4B FAE"}="AH4* FAE"; $resn_exch{" H3B FAE"}="AH3* FAE";
$resn_exch{"HO3A FAE"}="AHO3 FAE"; $resn_exch{" H2B FAE"}="AH2* FAE"; $resn_exch{"HO2A FAE"}="AHO2 FAE";
$resn_exch{" H1B FAE"}="AH1* FAE"; $resn_exch{" H8A FAE"}=" AH8 FAE"; $resn_exch{" H7A FAE"}=" AH7 FAE";
$resn_exch{"H61A FAE"}="AH61 FAE"; $resn_exch{"H62A FAE"}="AH62 FAE"; $resn_exch{" H2A FAE"}=" AH2 FAE";
$resn_exch{"HM71 FAE"}="1HM7 FAE"; $resn_exch{"HM72 FAE"}="2HM7 FAE"; $resn_exch{"HM73 FAE"}="3HM7 FAE";
$resn_exch{"HM81 FAE"}="1HM8 FAE"; $resn_exch{"HM82 FAE"}="2HM8 FAE"; $resn_exch{"HM83 FAE"}="3HM8 FAE";
$resn_exch{"H1'1 FAE"}="1H1* FAE"; $resn_exch{"H1'2 FAE"}="2H1* FAE"; $resn_exch{" H2' FAE"}=" H2* FAE";
$resn_exch{"HO2' FAE"}="*HO2 FAE"; $resn_exch{" H3' FAE"}=" H3* FAE"; $resn_exch{"HO3' FAE"}="*HO3 FAE";
$resn_exch{" H4' FAE"}=" H4* FAE"; $resn_exch{"HO4' FAE"}="*HO4 FAE"; $resn_exch{"H5'1 FAE"}="1H5* FAE";
$resn_exch{"H5'2 FAE"}="2H5* FAE"; $resn_exch{"HOP2 FAE"}="2HOP FAE"; $resn_exch{"HAX1 FAF"}="1HAX FAF";
$resn_exch{"HAX2 FAF"}="2HAX FAF"; $resn_exch{"HAX3 FAF"}="3HAX FAF"; $resn_exch{"HBL1 FAF"}="1HBL FAF";
$resn_exch{"HBL2 FAF"}="2HBL FAF"; $resn_exch{"HDL1 FAF"}="1HDL FAF"; $resn_exch{"HDL2 FAF"}="2HDL FAF";
$resn_exch{"HDL3 FAF"}="3HDL FAF"; $resn_exch{"HDM1 FAF"}="1HDM FAF"; $resn_exch{"HDM2 FAF"}="2HDM FAF";
$resn_exch{"HDM3 FAF"}="3HDM FAF"; $resn_exch{" H32 FAG"}="2H3  FAG"; $resn_exch{"H2A1 FAG"}="1H2A FAG";
$resn_exch{"H2A2 FAG"}="2H2A FAG"; $resn_exch{"HOP2 FAG"}="2HOP FAG"; $resn_exch{"H5'1 FAG"}="1H5* FAG";
$resn_exch{"H5'2 FAG"}="2H5* FAG"; $resn_exch{" H4' FAG"}=" H4* FAG"; $resn_exch{" H1' FAG"}=" H1* FAG";
$resn_exch{"HN21 FAG"}="1HN2 FAG"; $resn_exch{"HN22 FAG"}="2HN2 FAG"; $resn_exch{"H2'1 FAG"}="1H2* FAG";
$resn_exch{"H2'2 FAG"}="2H2* FAG"; $resn_exch{" H3' FAG"}=" H3* FAG"; $resn_exch{"HO3' FAG"}="*HO3 FAG";
$resn_exch{" HM1 FAG"}="1HM  FAG"; $resn_exch{" HM2 FAG"}="2HM  FAG"; $resn_exch{" HM3 FAG"}="3HM  FAG";
$resn_exch{"HOP3 FAG"}="3HOP FAG"; $resn_exch{" H41 FAI"}="1H4  FAI"; $resn_exch{" H42 FAI"}="2H4  FAI";
$resn_exch{"HN21 FAI"}="1HN2 FAI"; $resn_exch{"HN22 FAI"}="2HN2 FAI"; $resn_exch{"H221 FAL"}="1H22 FAL";
$resn_exch{"H222 FAL"}="2H22 FAL"; $resn_exch{"H241 FAL"}="1H24 FAL"; $resn_exch{"H242 FAL"}="2H24 FAL";
$resn_exch{"H261 FAL"}="1H26 FAL"; $resn_exch{"H262 FAL"}="2H26 FAL"; $resn_exch{"H263 FAL"}="3H26 FAL";
$resn_exch{"H271 FAL"}="1H27 FAL"; $resn_exch{"H272 FAL"}="2H27 FAL"; $resn_exch{"H273 FAL"}="3H27 FAL";
$resn_exch{" H2A FAM"}=" AH2 FAM"; $resn_exch{"H61A FAM"}="AH61 FAM"; $resn_exch{"H62A FAM"}="AH62 FAM";
$resn_exch{" H8A FAM"}=" AH8 FAM"; $resn_exch{" H1B FAM"}="AH1* FAM"; $resn_exch{" H2B FAM"}="AH2* FAM";
$resn_exch{"HO2A FAM"}="AHO2 FAM"; $resn_exch{" H3B FAM"}="AH3* FAM"; $resn_exch{"HOA8 FAM"}="8HOA FAM";
$resn_exch{"HOA9 FAM"}="9HOA FAM"; $resn_exch{" H4B FAM"}="AH4* FAM"; $resn_exch{"H51A FAM"}="AH51 FAM";
$resn_exch{"H52A FAM"}="AH52 FAM"; $resn_exch{"HOA2 FAM"}="2HOA FAM"; $resn_exch{"HOA5 FAM"}="5HOA FAM";
$resn_exch{"H121 FAM"}="1H12 FAM"; $resn_exch{"H122 FAM"}="2H12 FAM"; $resn_exch{"H131 FAM"}="1H13 FAM";
$resn_exch{"H132 FAM"}="2H13 FAM"; $resn_exch{"H133 FAM"}="3H13 FAM"; $resn_exch{"H141 FAM"}="1H14 FAM";
$resn_exch{"H142 FAM"}="2H14 FAM"; $resn_exch{"H143 FAM"}="3H14 FAM"; $resn_exch{" H71 FAM"}="1H7  FAM";
$resn_exch{" H72 FAM"}="2H7  FAM"; $resn_exch{" H61 FAM"}="1H6  FAM"; $resn_exch{" H62 FAM"}="2H6  FAM";
$resn_exch{" H31 FAM"}="1H3  FAM"; $resn_exch{" H32 FAM"}="2H3  FAM"; $resn_exch{" H21 FAM"}="1H2  FAM";
$resn_exch{" H22 FAM"}="2H2  FAM"; $resn_exch{"HN21 FAM"}="1HN2 FAM"; $resn_exch{"HN22 FAM"}="2HN2 FAM";
$resn_exch{"HC1' FAO"}="1HC' FAO"; $resn_exch{"HC2' FAO"}="2HC' FAO"; $resn_exch{"H6A1 FAO"}="1H6A FAO";
$resn_exch{"H6A2 FAO"}="2H6A FAO"; $resn_exch{"H7M1 FAO"}="1H7M FAO"; $resn_exch{"H7M2 FAO"}="2H7M FAO";
$resn_exch{"H7M3 FAO"}="3H7M FAO"; $resn_exch{"H8M1 FAO"}="1H8M FAO"; $resn_exch{"H8M2 FAO"}="2H8M FAO";
$resn_exch{"H8M3 FAO"}="3H8M FAO"; $resn_exch{"H11' FAO"}="1H1' FAO"; $resn_exch{"H12' FAO"}="2H1' FAO";
$resn_exch{"H51' FAO"}="1H5' FAO"; $resn_exch{"H52' FAO"}="2H5' FAO"; $resn_exch{"H241 FAP"}="1H24 FAP";
$resn_exch{"H242 FAP"}="2H24 FAP"; $resn_exch{"H261 FAP"}="1H26 FAP"; $resn_exch{"H262 FAP"}="2H26 FAP";
$resn_exch{"H281 FAP"}="1H28 FAP"; $resn_exch{"H282 FAP"}="2H28 FAP"; $resn_exch{"H283 FAP"}="3H28 FAP";
$resn_exch{"H291 FAP"}="1H29 FAP"; $resn_exch{"H292 FAP"}="2H29 FAP"; $resn_exch{"H293 FAP"}="3H29 FAP";
$resn_exch{" H11 FAR"}="1H1  FAR"; $resn_exch{"H12A FAR"}="2H1  FAR"; $resn_exch{" H13 FAR"}="3H1  FAR";
$resn_exch{" H51 FAR"}="1H5  FAR"; $resn_exch{" H52 FAR"}="2H5  FAR"; $resn_exch{" H61 FAR"}="1H6  FAR";
$resn_exch{" H62 FAR"}="2H6  FAR"; $resn_exch{" H91 FAR"}="1H9  FAR"; $resn_exch{" H92 FAR"}="2H9  FAR";
$resn_exch{"H111 FAR"}="1H11 FAR"; $resn_exch{"H112 FAR"}="2H11 FAR"; $resn_exch{"H141 FAR"}="1H14 FAR";
$resn_exch{"H142 FAR"}="2H14 FAR"; $resn_exch{"H143 FAR"}="3H14 FAR"; $resn_exch{"H151 FAR"}="1H15 FAR";
$resn_exch{"H152 FAR"}="2H15 FAR"; $resn_exch{"H153 FAR"}="3H15 FAR"; $resn_exch{" H41 FAR"}="1H4  FAR";
$resn_exch{" H42 FAR"}="2H4  FAR"; $resn_exch{" H43 FAR"}="3H4  FAR"; $resn_exch{"H101 FAR"}="1H10 FAR";
$resn_exch{"H102 FAR"}="2H10 FAR"; $resn_exch{"H103 FAR"}="3H10 FAR"; $resn_exch{"HOA2 FAS"}="2HOA FAS";
$resn_exch{"H51A FAS"}="AH51 FAS"; $resn_exch{"H52A FAS"}="AH52 FAS"; $resn_exch{" H4B FAS"}="AH4* FAS";
$resn_exch{" H3B FAS"}="AH3* FAS"; $resn_exch{"HO3A FAS"}="AHO3 FAS"; $resn_exch{" H2B FAS"}="AH2* FAS";
$resn_exch{"HO2A FAS"}="AHO2 FAS"; $resn_exch{" H1B FAS"}="AH1* FAS"; $resn_exch{" H8A FAS"}=" AH8 FAS";
$resn_exch{"H61A FAS"}="AH61 FAS"; $resn_exch{"H62A FAS"}="AH62 FAS"; $resn_exch{" H2A FAS"}=" AH2 FAS";
$resn_exch{"HM71 FAS"}="1HM7 FAS"; $resn_exch{"HM72 FAS"}="2HM7 FAS"; $resn_exch{"HM73 FAS"}="3HM7 FAS";
$resn_exch{"HM81 FAS"}="1HM8 FAS"; $resn_exch{"HM82 FAS"}="2HM8 FAS"; $resn_exch{"HM83 FAS"}="3HM8 FAS";
$resn_exch{"H1'1 FAS"}="1H1* FAS"; $resn_exch{"H1'2 FAS"}="2H1* FAS"; $resn_exch{" H2' FAS"}=" H2* FAS";
$resn_exch{"HO2' FAS"}="*HO2 FAS"; $resn_exch{" H3' FAS"}=" H3* FAS"; $resn_exch{"HO3' FAS"}="*HO3 FAS";
$resn_exch{" H4' FAS"}=" H4* FAS"; $resn_exch{"HO4' FAS"}="*HO4 FAS"; $resn_exch{"H5'1 FAS"}="1H5* FAS";
$resn_exch{"H5'2 FAS"}="2H5* FAS"; $resn_exch{"HOP2 FAS"}="2HOP FAS"; $resn_exch{" H21 FAT"}="1H2  FAT";
$resn_exch{" H22 FAT"}="2H2  FAT"; $resn_exch{" H31 FAT"}="1H3  FAT"; $resn_exch{" H32 FAT"}="2H3  FAT";
$resn_exch{" H41 FAT"}="1H4  FAT"; $resn_exch{" H42 FAT"}="2H4  FAT"; $resn_exch{" H51 FAT"}="1H5  FAT";
$resn_exch{" H52 FAT"}="2H5  FAT"; $resn_exch{" H61 FAT"}="1H6  FAT"; $resn_exch{" H62 FAT"}="2H6  FAT";
$resn_exch{" H71 FAT"}="1H7  FAT"; $resn_exch{" H72 FAT"}="2H7  FAT"; $resn_exch{" H81 FAT"}="1H8  FAT";
$resn_exch{" H82 FAT"}="2H8  FAT"; $resn_exch{" H91 FAT"}="1H9  FAT"; $resn_exch{" H92 FAT"}="2H9  FAT";
$resn_exch{"H101 FAT"}="1H10 FAT"; $resn_exch{"H102 FAT"}="2H10 FAT"; $resn_exch{"H111 FAT"}="1H11 FAT";
$resn_exch{"H112 FAT"}="2H11 FAT"; $resn_exch{"H121 FAT"}="1H12 FAT"; $resn_exch{"H122 FAT"}="2H12 FAT";
$resn_exch{"H131 FAT"}="1H13 FAT"; $resn_exch{"H132 FAT"}="2H13 FAT"; $resn_exch{"H141 FAT"}="1H14 FAT";
$resn_exch{"H142 FAT"}="2H14 FAT"; $resn_exch{"H151 FAT"}="1H15 FAT"; $resn_exch{"H152 FAT"}="2H15 FAT";
$resn_exch{"H161 FAT"}="1H16 FAT"; $resn_exch{"H162 FAT"}="2H16 FAT"; $resn_exch{"H163 FAT"}="3H16 FAT";
$resn_exch{" H41  FB"}="1H4   FB"; $resn_exch{" H42  FB"}="2H4   FB"; $resn_exch{" H31  FB"}="1H3   FB";
$resn_exch{" H32  FB"}="2H3   FB"; $resn_exch{" HB1  FB"}="1HB   FB"; $resn_exch{" HB2  FB"}="2HB   FB";
$resn_exch{" HB3  FB"}="3HB   FB"; $resn_exch{" HA1  FB"}="1HA   FB"; $resn_exch{" HA2  FB"}="2HA   FB";
$resn_exch{" H1  FBA"}="1H   FBA"; $resn_exch{" H2A FBA"}="2H   FBA"; $resn_exch{" HN1 FBA"}="1HN  FBA";
$resn_exch{" HN2 FBA"}="2HN  FBA"; $resn_exch{" HN3 FBA"}="3HN  FBA"; $resn_exch{" H11 FBE"}="1H1  FBE";
$resn_exch{" H12 FBE"}="2H1  FBE"; $resn_exch{" H41 FBE"}="1H4  FBE"; $resn_exch{" H42 FBE"}="2H4  FBE";
$resn_exch{" HN1 FBE"}="1HN  FBE"; $resn_exch{" HN2 FBE"}="2HN  FBE"; $resn_exch{"H121 FBI"}="1H12 FBI";
$resn_exch{"H122 FBI"}="2H12 FBI"; $resn_exch{"H123 FBI"}="3H12 FBI"; $resn_exch{"H131 FBI"}="1H13 FBI";
$resn_exch{"H132 FBI"}="2H13 FBI"; $resn_exch{"H133 FBI"}="3H13 FBI"; $resn_exch{" H21 FBI"}="1H2  FBI";
$resn_exch{" H22 FBI"}="2H2  FBI"; $resn_exch{" H41 FBI"}="1H4  FBI"; $resn_exch{" H42 FBI"}="2H4  FBI";
$resn_exch{" H61 FBI"}="1H6  FBI"; $resn_exch{" H62 FBI"}="2H6  FBI"; $resn_exch{" H71 FBI"}="1H7  FBI";
$resn_exch{" H72 FBI"}="2H7  FBI"; $resn_exch{"H921 FBI"}="1H92 FBI"; $resn_exch{"H922 FBI"}="2H92 FBI";
$resn_exch{"H923 FBI"}="3H92 FBI"; $resn_exch{"H931 FBI"}="1H93 FBI"; $resn_exch{"H932 FBI"}="2H93 FBI";
$resn_exch{"H933 FBI"}="3H93 FBI"; $resn_exch{"H221 FBL"}="1H22 FBL"; $resn_exch{"H222 FBL"}="2H22 FBL";
$resn_exch{"H241 FBL"}="1H24 FBL"; $resn_exch{"H242 FBL"}="2H24 FBL"; $resn_exch{"H261 FBL"}="1H26 FBL";
$resn_exch{"H262 FBL"}="2H26 FBL"; $resn_exch{"H263 FBL"}="3H26 FBL"; $resn_exch{"H271 FBL"}="1H27 FBL";
$resn_exch{"H272 FBL"}="2H27 FBL"; $resn_exch{"H273 FBL"}="3H27 FBL"; $resn_exch{"HOP2 FBP"}="2HOP FBP";
$resn_exch{"HOP3 FBP"}="3HOP FBP"; $resn_exch{" H11 FBP"}="1H1  FBP"; $resn_exch{" H12 FBP"}="2H1  FBP";
$resn_exch{" H61 FBP"}="1H6  FBP"; $resn_exch{" H62 FBP"}="2H6  FBP"; $resn_exch{"HOP5 FBP"}="5HOP FBP";
$resn_exch{"HOP6 FBP"}="6HOP FBP"; $resn_exch{" HC8 FBQ"}="8HC  FBQ"; $resn_exch{" HC9 FBQ"}="9HC  FBQ";
$resn_exch{"HC01 FBQ"}="1HC0 FBQ"; $resn_exch{"H111 FBQ"}="1H11 FBQ"; $resn_exch{"H112 FBQ"}="2H11 FBQ";
$resn_exch{"H121 FBQ"}="1H12 FBQ"; $resn_exch{"H122 FBQ"}="2H12 FBQ"; $resn_exch{"H141 FBQ"}="1H14 FBQ";
$resn_exch{"H142 FBQ"}="2H14 FBQ"; $resn_exch{"HC61 FBQ"}="1HC6 FBQ"; $resn_exch{"HC81 FBQ"}="1HC8 FBQ";
$resn_exch{"HC91 FBQ"}="1HC9 FBQ"; $resn_exch{"HC02 FBQ"}="2HC0 FBQ"; $resn_exch{"HO62 FBQ"}="2HO6 FBQ";
$resn_exch{"H281 FBQ"}="1H28 FBQ"; $resn_exch{"H282 FBQ"}="2H28 FBQ"; $resn_exch{"H283 FBQ"}="3H28 FBQ";
$resn_exch{"H6SO FBQ"}="OH6S FBQ"; $resn_exch{" H21 FBR"}="1H2  FBR"; $resn_exch{" H22 FBR"}="2H2  FBR";
$resn_exch{" H31 FBR"}="1H3  FBR"; $resn_exch{" H32 FBR"}="2H3  FBR"; $resn_exch{"H161 FBR"}="1H16 FBR";
$resn_exch{"H162 FBR"}="2H16 FBR"; $resn_exch{"H171 FBR"}="1H17 FBR"; $resn_exch{"H172 FBR"}="2H17 FBR";
$resn_exch{"H181 FBR"}="1H18 FBR"; $resn_exch{"H182 FBR"}="2H18 FBR"; $resn_exch{"H191 FBR"}="1H19 FBR";
$resn_exch{"H192 FBR"}="2H19 FBR"; $resn_exch{"H201 FBR"}="1H20 FBR"; $resn_exch{"H202 FBR"}="2H20 FBR";
$resn_exch{"H203 FBR"}="3H20 FBR"; $resn_exch{"HN01 FBS"}="1HN0 FBS"; $resn_exch{"HN02 FBS"}="2HN0 FBS";
$resn_exch{"HN01 FBT"}="1HN0 FBT"; $resn_exch{"HN02 FBT"}="2HN0 FBT"; $resn_exch{"HN01 FBU"}="1HN0 FBU";
$resn_exch{"HN02 FBU"}="2HN0 FBU"; $resn_exch{"H171 FC1"}="1H17 FC1"; $resn_exch{"H172 FC1"}="2H17 FC1";
$resn_exch{"H173 FC1"}="3H17 FC1"; $resn_exch{"H181 FC1"}="1H18 FC1"; $resn_exch{"H182 FC1"}="2H18 FC1";
$resn_exch{"H183 FC1"}="3H18 FC1"; $resn_exch{"H111 FC1"}="1H11 FC1"; $resn_exch{"H112 FC1"}="2H11 FC1";
$resn_exch{"H141 FC1"}="1H14 FC1"; $resn_exch{"H142 FC1"}="2H14 FC1"; $resn_exch{"H151 FC1"}="1H15 FC1";
$resn_exch{"H152 FC1"}="2H15 FC1"; $resn_exch{" H61 FCA"}="1H6  FCA"; $resn_exch{" H62 FCA"}="2H6  FCA";
$resn_exch{" H63 FCA"}="3H6  FCA"; $resn_exch{" H61 FCB"}="1H6  FCB"; $resn_exch{" H62 FCB"}="2H6  FCB";
$resn_exch{" H63 FCB"}="3H6  FCB"; $resn_exch{" H11 FCG"}="1H1  FCG"; $resn_exch{" H12 FCG"}="2H1  FCG";
$resn_exch{" H1' FCG"}=" H1* FCG"; $resn_exch{"HN61 FCG"}="1HN6 FCG"; $resn_exch{"HN62 FCG"}="2HN6 FCG";
$resn_exch{"H111 FCG"}="H11  FCG"; $resn_exch{"H122 FCG"}="H12  FCG"; $resn_exch{"H5'1 FCG"}="1H5* FCG";
$resn_exch{"H5'2 FCG"}="2H5* FCG"; $resn_exch{" H4' FCG"}=" H4* FCG"; $resn_exch{" H3' FCG"}=" H3* FCG";
$resn_exch{" H2' FCG"}=" H2* FCG"; $resn_exch{"H1'1 FCG"}="1H1* FCG"; $resn_exch{"H1'2 FCG"}="2H1* FCG";
$resn_exch{"H7M1 FCG"}="1H7M FCG"; $resn_exch{"H7M2 FCG"}="2H7M FCG"; $resn_exch{"H7M3 FCG"}="3H7M FCG";
$resn_exch{"H141 FCG"}="1H11 FCG"; $resn_exch{"H112 FCG"}="2H11 FCG"; $resn_exch{"H121 FCG"}="1H12 FCG";
$resn_exch{"H142 FCG"}="2H12 FCG"; $resn_exch{"H8M1 FCG"}="1H8M FCG"; $resn_exch{"H8M2 FCG"}="2H8M FCG";
$resn_exch{"H8M3 FCG"}="3H8M FCG"; $resn_exch{"H141 FCI"}="1H14 FCI"; $resn_exch{"H142 FCI"}="2H14 FCI";
$resn_exch{"H171 FCI"}="1H17 FCI"; $resn_exch{"H172 FCI"}="2H17 FCI"; $resn_exch{"H181 FCI"}="1H18 FCI";
$resn_exch{"H182 FCI"}="2H18 FCI"; $resn_exch{"H211 FCI"}="1H21 FCI"; $resn_exch{"H212 FCI"}="2H21 FCI";
$resn_exch{"H241 FCI"}="1H24 FCI"; $resn_exch{"H242 FCI"}="2H24 FCI"; $resn_exch{"H251 FCI"}="1H25 FCI";
$resn_exch{"H252 FCI"}="2H25 FCI"; $resn_exch{"H271 FCI"}="1H27 FCI"; $resn_exch{"H272 FCI"}="2H27 FCI";
$resn_exch{"H291 FCI"}="1H29 FCI"; $resn_exch{"H292 FCI"}="2H29 FCI"; $resn_exch{"H301 FCI"}="1H30 FCI";
$resn_exch{"H302 FCI"}="2H30 FCI"; $resn_exch{"H321 FCI"}="1H32 FCI"; $resn_exch{"H322 FCI"}="2H32 FCI";
$resn_exch{"H341 FCI"}="1H34 FCI"; $resn_exch{"H342 FCI"}="2H34 FCI"; $resn_exch{"H351 FCI"}="1H35 FCI";
$resn_exch{"H352 FCI"}="2H35 FCI"; $resn_exch{"H353 FCI"}="3H35 FCI"; $resn_exch{"H361 FCI"}="1H36 FCI";
$resn_exch{"H362 FCI"}="2H36 FCI"; $resn_exch{"H363 FCI"}="3H36 FCI"; $resn_exch{"H371 FCI"}="1H37 FCI";
$resn_exch{"H372 FCI"}="2H37 FCI"; $resn_exch{"H373 FCI"}="3H37 FCI"; $resn_exch{"H381 FCI"}="1H38 FCI";
$resn_exch{"H382 FCI"}="2H38 FCI"; $resn_exch{" HN1 FCL"}="1HN  FCL"; $resn_exch{" HN2 FCL"}="2HN  FCL";
$resn_exch{"HBC1 FCL"}="1HBC FCL"; $resn_exch{"HBC2 FCL"}="2HBC FCL"; $resn_exch{" H31 FCN"}="1H3  FCN";
$resn_exch{" H32 FCN"}="2H3  FCN"; $resn_exch{" H33 FCN"}="3H3  FCN"; $resn_exch{"HOP2 FCN"}="2HOP FCN";
$resn_exch{"HOP3 FCN"}="3HOP FCN"; $resn_exch{" HA1 FCP"}="1HA  FCP"; $resn_exch{" HB2 FCP"}="2HB  FCP";
$resn_exch{" HB1 FCP"}="1HB  FCP"; $resn_exch{" HC2 FCP"}="2HC  FCP"; $resn_exch{" HC1 FCP"}="1HC  FCP";
$resn_exch{" HD2 FCP"}="2HD  FCP"; $resn_exch{" HE3 FCP"}="3HE  FCP"; $resn_exch{" HD1 FCP"}="1HD  FCP";
$resn_exch{" HE2 FCP"}="2HE  FCP"; $resn_exch{" HF3 FCP"}="3HF  FCP"; $resn_exch{"H6A1 FCT"}="1H6A FCT";
$resn_exch{"H6A2 FCT"}="2H6A FCT"; $resn_exch{"HOA3 FCT"}="3HOA FCT"; $resn_exch{"HOA6 FCT"}="6HOA FCT";
$resn_exch{"H6B1 FCT"}="1H6B FCT"; $resn_exch{"H6B2 FCT"}="2H6B FCT"; $resn_exch{"HOB2 FCT"}="2HOB FCT";
$resn_exch{"HOB3 FCT"}="3HOB FCT"; $resn_exch{"HOB6 FCT"}="6HOB FCT"; $resn_exch{"H6C1 FCT"}="1H6C FCT";
$resn_exch{"H6C2 FCT"}="2H6C FCT"; $resn_exch{"HOC2 FCT"}="2HOC FCT"; $resn_exch{"HOC3 FCT"}="3HOC FCT";
$resn_exch{"HOC4 FCT"}="4HOC FCT"; $resn_exch{"HOC6 FCT"}="6HOC FCT"; $resn_exch{" H2A FCX"}=" AH2 FCX";
$resn_exch{"H61A FCX"}="AH61 FCX"; $resn_exch{"H62A FCX"}="AH62 FCX"; $resn_exch{" H8A FCX"}=" AH8 FCX";
$resn_exch{" H1B FCX"}="AH1* FCX"; $resn_exch{" H2B FCX"}="AH2* FCX"; $resn_exch{"HO2A FCX"}="AHO2 FCX";
$resn_exch{" H3B FCX"}="AH3* FCX"; $resn_exch{"HOA8 FCX"}="8HOA FCX"; $resn_exch{"HOA9 FCX"}="9HOA FCX";
$resn_exch{" H4B FCX"}="AH4* FCX"; $resn_exch{"H51A FCX"}="AH51 FCX"; $resn_exch{"H52A FCX"}="AH52 FCX";
$resn_exch{"HOA2 FCX"}="2HOA FCX"; $resn_exch{"HOA5 FCX"}="5HOA FCX"; $resn_exch{"H121 FCX"}="1H12 FCX";
$resn_exch{"H122 FCX"}="2H12 FCX"; $resn_exch{"H131 FCX"}="1H13 FCX"; $resn_exch{"H132 FCX"}="2H13 FCX";
$resn_exch{"H133 FCX"}="3H13 FCX"; $resn_exch{"H141 FCX"}="1H14 FCX"; $resn_exch{"H142 FCX"}="2H14 FCX";
$resn_exch{"H143 FCX"}="3H14 FCX"; $resn_exch{" H71 FCX"}="1H7  FCX"; $resn_exch{" H72 FCX"}="2H7  FCX";
$resn_exch{" H61 FCX"}="1H6  FCX"; $resn_exch{" H62 FCX"}="2H6  FCX"; $resn_exch{" H31 FCX"}="1H3  FCX";
$resn_exch{" H32 FCX"}="2H3  FCX"; $resn_exch{" H21 FCX"}="1H2  FCX"; $resn_exch{" H22 FCX"}="2H2  FCX";
$resn_exch{" HB1 FCY"}="1HB  FCY"; $resn_exch{" HB2 FCY"}="2HB  FCY"; $resn_exch{"H661 FD1"}="1H66 FD1";
$resn_exch{"H641 FD1"}="1H64 FD1"; $resn_exch{"H631 FD1"}="1H63 FD1"; $resn_exch{"H621 FD1"}="1H62 FD1";
$resn_exch{"H431 FD1"}="1H43 FD1"; $resn_exch{"H432 FD1"}="2H43 FD1"; $resn_exch{"H541 FD1"}="1H54 FD1";
$resn_exch{"H341 FD1"}="1H34 FD1"; $resn_exch{"H342 FD1"}="2H34 FD1"; $resn_exch{"H311 FD1"}="1H31 FD1";
$resn_exch{"H711 FD1"}="1H71 FD1"; $resn_exch{"H721 FD1"}="1H72 FD1"; $resn_exch{"H722 FD1"}="2H72 FD1";
$resn_exch{"H741 FD1"}="1H74 FD1"; $resn_exch{"H742 FD1"}="2H74 FD1"; $resn_exch{"H751 FD1"}="1H75 FD1";
$resn_exch{"H752 FD1"}="2H75 FD1"; $resn_exch{"H401 FD1"}="1H40 FD1"; $resn_exch{"H402 FD1"}="2H40 FD1";
$resn_exch{"H921 FD1"}="1H92 FD1"; $resn_exch{"H291 FD1"}="1H29 FD1"; $resn_exch{" H31 FD1"}="1H3  FD1";
$resn_exch{" H41 FD1"}="1H4  FD1"; $resn_exch{" H11 FD1"}="1H1  FD1"; $resn_exch{" H51 FD1"}="1H5  FD1";
$resn_exch{" H61 FD1"}="1H6  FD1"; $resn_exch{" H71 FD1"}="1H7  FD1"; $resn_exch{" H81 FD1"}="1H8  FD1";
$resn_exch{"H661 FD2"}="1H66 FD2"; $resn_exch{"H641 FD2"}="1H64 FD2"; $resn_exch{"H631 FD2"}="1H63 FD2";
$resn_exch{"H621 FD2"}="1H62 FD2"; $resn_exch{"H431 FD2"}="1H43 FD2"; $resn_exch{"H432 FD2"}="2H43 FD2";
$resn_exch{"H541 FD2"}="1H54 FD2"; $resn_exch{"H341 FD2"}="1H34 FD2"; $resn_exch{"H342 FD2"}="2H34 FD2";
$resn_exch{"H311 FD2"}="1H31 FD2"; $resn_exch{"H711 FD2"}="1H71 FD2"; $resn_exch{"H712 FD2"}="2H71 FD2";
$resn_exch{"H721 FD2"}="1H72 FD2"; $resn_exch{"H722 FD2"}="2H72 FD2"; $resn_exch{"H741 FD2"}="1H74 FD2";
$resn_exch{"H742 FD2"}="2H74 FD2"; $resn_exch{"H751 FD2"}="1H75 FD2"; $resn_exch{"H752 FD2"}="2H75 FD2";
$resn_exch{"H401 FD2"}="1H40 FD2"; $resn_exch{"H991 FD2"}="1H99 FD2"; $resn_exch{"H992 FD2"}="2H99 FD2";
$resn_exch{"H993 FD2"}="3H99 FD2"; $resn_exch{"H291 FD2"}="1H29 FD2"; $resn_exch{" H31 FD2"}="1H3  FD2";
$resn_exch{" H41 FD2"}="1H4  FD2"; $resn_exch{" H11 FD2"}="1H1  FD2"; $resn_exch{" H51 FD2"}="1H5  FD2";
$resn_exch{" H61 FD2"}="1H6  FD2"; $resn_exch{" H71 FD2"}="1H7  FD2"; $resn_exch{" H81 FD2"}="1H8  FD2";
$resn_exch{"H661 FD3"}="1H66 FD3"; $resn_exch{"H641 FD3"}="1H64 FD3"; $resn_exch{"H631 FD3"}="1H63 FD3";
$resn_exch{"H621 FD3"}="1H62 FD3"; $resn_exch{"H431 FD3"}="1H43 FD3"; $resn_exch{"H432 FD3"}="2H43 FD3";
$resn_exch{"H541 FD3"}="1H54 FD3"; $resn_exch{"H341 FD3"}="1H34 FD3"; $resn_exch{"H342 FD3"}="2H34 FD3";
$resn_exch{"H311 FD3"}="1H31 FD3"; $resn_exch{"H711 FD3"}="1H71 FD3"; $resn_exch{"H712 FD3"}="2H71 FD3";
$resn_exch{"H721 FD3"}="1H72 FD3"; $resn_exch{"H722 FD3"}="2H72 FD3"; $resn_exch{"H741 FD3"}="1H74 FD3";
$resn_exch{"H742 FD3"}="2H74 FD3"; $resn_exch{"H751 FD3"}="1H75 FD3"; $resn_exch{"H752 FD3"}="2H75 FD3";
$resn_exch{"H401 FD3"}="1H40 FD3"; $resn_exch{"H291 FD3"}="1H29 FD3"; $resn_exch{" H31 FD3"}="1H3  FD3";
$resn_exch{" H41 FD3"}="1H4  FD3"; $resn_exch{" H11 FD3"}="1H1  FD3"; $resn_exch{" H51 FD3"}="1H5  FD3";
$resn_exch{" H61 FD3"}="1H6  FD3"; $resn_exch{" H71 FD3"}="1H7  FD3"; $resn_exch{" H81 FD3"}="1H8  FD3";
$resn_exch{"H661 FD4"}="1H66 FD4"; $resn_exch{"H641 FD4"}="1H64 FD4"; $resn_exch{"H631 FD4"}="1H63 FD4";
$resn_exch{"H621 FD4"}="1H62 FD4"; $resn_exch{"H431 FD4"}="1H43 FD4"; $resn_exch{"H432 FD4"}="2H43 FD4";
$resn_exch{"H541 FD4"}="1H54 FD4"; $resn_exch{"H341 FD4"}="1H34 FD4"; $resn_exch{"H342 FD4"}="2H34 FD4";
$resn_exch{"H311 FD4"}="1H31 FD4"; $resn_exch{"H711 FD4"}="1H71 FD4"; $resn_exch{"H712 FD4"}="2H71 FD4";
$resn_exch{"H721 FD4"}="1H72 FD4"; $resn_exch{"H722 FD4"}="2H72 FD4"; $resn_exch{"H741 FD4"}="1H74 FD4";
$resn_exch{"H742 FD4"}="2H74 FD4"; $resn_exch{"H751 FD4"}="1H75 FD4"; $resn_exch{"H752 FD4"}="2H75 FD4";
$resn_exch{"H291 FD4"}="1H29 FD4"; $resn_exch{" H31 FD4"}="1H3  FD4"; $resn_exch{" H41 FD4"}="1H4  FD4";
$resn_exch{" H11 FD4"}="1H1  FD4"; $resn_exch{" H51 FD4"}="1H5  FD4"; $resn_exch{" H61 FD4"}="1H6  FD4";
$resn_exch{" H71 FD4"}="1H7  FD4"; $resn_exch{" H81 FD4"}="1H8  FD4"; $resn_exch{"H551 FD4"}="1H55 FD4";
$resn_exch{"H552 FD4"}="2H55 FD4"; $resn_exch{"H553 FD4"}="3H55 FD4"; $resn_exch{"HOA2 FDA"}="2HOA FDA";
$resn_exch{"H51A FDA"}="AH51 FDA"; $resn_exch{"H52A FDA"}="AH52 FDA"; $resn_exch{" H4B FDA"}="AH4* FDA";
$resn_exch{" H3B FDA"}="AH3* FDA"; $resn_exch{"HO3A FDA"}="AHO3 FDA"; $resn_exch{" H2B FDA"}="AH2* FDA";
$resn_exch{"HO2A FDA"}="AHO2 FDA"; $resn_exch{" H1B FDA"}="AH1* FDA"; $resn_exch{" H8A FDA"}=" AH8 FDA";
$resn_exch{"H61A FDA"}="AH61 FDA"; $resn_exch{"H62A FDA"}="AH62 FDA"; $resn_exch{" H2A FDA"}=" AH2 FDA";
$resn_exch{"HM71 FDA"}="1HM7 FDA"; $resn_exch{"HM72 FDA"}="2HM7 FDA"; $resn_exch{"HM73 FDA"}="3HM7 FDA";
$resn_exch{"HM81 FDA"}="1HM8 FDA"; $resn_exch{"HM82 FDA"}="2HM8 FDA"; $resn_exch{"HM83 FDA"}="3HM8 FDA";
$resn_exch{"H1'1 FDA"}="1H1* FDA"; $resn_exch{"H1'2 FDA"}="2H1* FDA"; $resn_exch{" H2' FDA"}=" H2* FDA";
$resn_exch{"HO2' FDA"}="*HO2 FDA"; $resn_exch{" H3' FDA"}=" H3* FDA"; $resn_exch{"HO3' FDA"}="*HO3 FDA";
$resn_exch{" H4' FDA"}=" H4* FDA"; $resn_exch{"HO4' FDA"}="*HO4 FDA"; $resn_exch{"H5'1 FDA"}="1H5* FDA";
$resn_exch{"H5'2 FDA"}="2H5* FDA"; $resn_exch{"HOP2 FDA"}="2HOP FDA"; $resn_exch{" H21 FDC"}="1H2  FDC";
$resn_exch{" H22 FDC"}="2H2  FDC"; $resn_exch{" H41 FDC"}="1H4  FDC"; $resn_exch{" H42 FDC"}="2H4  FDC";
$resn_exch{" H81 FDC"}="1H8  FDC"; $resn_exch{" H82 FDC"}="2H8  FDC"; $resn_exch{"H10O FDC"}="OH10 FDC";
$resn_exch{"H11O FDC"}="OH11 FDC"; $resn_exch{"H101 FDC"}="1H10 FDC"; $resn_exch{"H102 FDC"}="2H10 FDC";
$resn_exch{"HBD1 FDD"}="1HBD FDD"; $resn_exch{"HBD2 FDD"}="2HBD FDD"; $resn_exch{"HAD1 FDD"}="1HAD FDD";
$resn_exch{"HAD2 FDD"}="2HAD FDD"; $resn_exch{"HMD1 FDD"}="1HMD FDD"; $resn_exch{"HMD2 FDD"}="2HMD FDD";
$resn_exch{"HMD3 FDD"}="3HMD FDD"; $resn_exch{"HMF1 FDD"}="1HMF FDD"; $resn_exch{"HMF2 FDD"}="2HMF FDD";
$resn_exch{"HMF3 FDD"}="3HMF FDD"; $resn_exch{"HMC1 FDD"}="1HMC FDD"; $resn_exch{"HMC2 FDD"}="2HMC FDD";
$resn_exch{"HMC3 FDD"}="3HMC FDD"; $resn_exch{"HME1 FDD"}="1HME FDD"; $resn_exch{"HME2 FDD"}="2HME FDD";
$resn_exch{"HME3 FDD"}="3HME FDD"; $resn_exch{"HMB1 FDD"}="1HMB FDD"; $resn_exch{"HMB2 FDD"}="2HMB FDD";
$resn_exch{"HMB3 FDD"}="3HMB FDD"; $resn_exch{"HMA1 FDD"}="1HMA FDD"; $resn_exch{"HMA2 FDD"}="2HMA FDD";
$resn_exch{"HMA3 FDD"}="3HMA FDD"; $resn_exch{"HAA1 FDD"}="1HAA FDD"; $resn_exch{"HAA2 FDD"}="2HAA FDD";
$resn_exch{"HBA1 FDD"}="1HBA FDD"; $resn_exch{"HBA2 FDD"}="2HBA FDD"; $resn_exch{"HAA1 FDE"}="1HAA FDE";
$resn_exch{"HAA2 FDE"}="2HAA FDE"; $resn_exch{"HBA1 FDE"}="1HBA FDE"; $resn_exch{"HBA2 FDE"}="2HBA FDE";
$resn_exch{"HMA1 FDE"}="1HMA FDE"; $resn_exch{"HMA2 FDE"}="2HMA FDE"; $resn_exch{"HMA3 FDE"}="3HMA FDE";
$resn_exch{"HME1 FDE"}="1HME FDE"; $resn_exch{"HME2 FDE"}="2HME FDE"; $resn_exch{"HME3 FDE"}="3HME FDE";
$resn_exch{"HMF1 FDE"}="1HMF FDE"; $resn_exch{"HMF2 FDE"}="2HMF FDE"; $resn_exch{"HMF3 FDE"}="3HMF FDE";
$resn_exch{"HMD1 FDE"}="1HMD FDE"; $resn_exch{"HMD2 FDE"}="2HMD FDE"; $resn_exch{"HMD3 FDE"}="3HMD FDE";
$resn_exch{"HAD1 FDE"}="1HAD FDE"; $resn_exch{"HAD2 FDE"}="2HAD FDE"; $resn_exch{"HBD1 FDE"}="1HBD FDE";
$resn_exch{"HBD2 FDE"}="2HBD FDE"; $resn_exch{"H171 FDI"}="1H17 FDI"; $resn_exch{"H172 FDI"}="2H17 FDI";
$resn_exch{"H181 FDI"}="1H18 FDI"; $resn_exch{"H182 FDI"}="2H18 FDI"; $resn_exch{"H183 FDI"}="3H18 FDI";
$resn_exch{"H241 FDI"}="1H24 FDI"; $resn_exch{"H242 FDI"}="2H24 FDI"; $resn_exch{"H261 FDI"}="1H26 FDI";
$resn_exch{"H262 FDI"}="2H26 FDI"; $resn_exch{"H263 FDI"}="3H26 FDI"; $resn_exch{"H341 FDI"}="1H34 FDI";
$resn_exch{"H342 FDI"}="2H34 FDI"; $resn_exch{"H343 FDI"}="3H34 FDI"; $resn_exch{"HC61 FDM"}="1HC6 FDM";
$resn_exch{"HN31 FDM"}="1HN3 FDM"; $resn_exch{"HC71 FDM"}="1HC7 FDM"; $resn_exch{"HC72 FDM"}="2HC7 FDM";
$resn_exch{"HC73 FDM"}="3HC7 FDM"; $resn_exch{"H1'1 FDM"}="1H1* FDM"; $resn_exch{"H2'1 FDM"}="1H2* FDM";
$resn_exch{"H2'2 FDM"}="2H2* FDM"; $resn_exch{"H3'1 FDM"}="1H3* FDM"; $resn_exch{"H4'1 FDM"}="1H4* FDM";
$resn_exch{"H5'1 FDM"}="1H5* FDM"; $resn_exch{"H5'2 FDM"}="2H5* FDM"; $resn_exch{"HP31 FDM"}="1HP3 FDM";
$resn_exch{"HP21 FDM"}="1HP2 FDM"; $resn_exch{" H71 FDN"}="1H7  FDN"; $resn_exch{" H72 FDN"}="2H7  FDN";
$resn_exch{" H73 FDN"}="3H7  FDN"; $resn_exch{"HOP2 FDP"}="2HOP FDP"; $resn_exch{"HOP3 FDP"}="3HOP FDP";
$resn_exch{" H11 FDP"}="1H1  FDP"; $resn_exch{" H12 FDP"}="2H1  FDP"; $resn_exch{" H61 FDP"}="1H6  FDP";
$resn_exch{" H62 FDP"}="2H6  FDP"; $resn_exch{"HOP5 FDP"}="5HOP FDP"; $resn_exch{"HOP6 FDP"}="6HOP FDP";
$resn_exch{"HOP2 FDQ"}="2HOP FDQ"; $resn_exch{"HOP3 FDQ"}="3HOP FDQ"; $resn_exch{"H211 FDQ"}="1H21 FDQ";
$resn_exch{"H212 FDQ"}="2H21 FDQ"; $resn_exch{" H61 FDQ"}="1H6  FDQ"; $resn_exch{" H62 FDQ"}="2H6  FDQ";
$resn_exch{"HOP5 FDQ"}="5HOP FDQ"; $resn_exch{"HOP6 FDQ"}="6HOP FDQ"; $resn_exch{"H191 FDT"}="1H19 FDT";
$resn_exch{"H192 FDT"}="2H19 FDT"; $resn_exch{"H193 FDT"}="3H19 FDT"; $resn_exch{"HAW1 FE1"}="1HAW FE1";
$resn_exch{"HAW2 FE1"}="2HAW FE1"; $resn_exch{"HAX1 FE1"}="1HAX FE1"; $resn_exch{"HAX2 FE1"}="2HAX FE1";
$resn_exch{"HBA1 FE1"}="1HBA FE1"; $resn_exch{"HBA2 FE1"}="2HBA FE1"; $resn_exch{"HBB1 FE1"}="1HBB FE1";
$resn_exch{"HBB2 FE1"}="2HBB FE1"; $resn_exch{"HBF1 FE1"}="1HBF FE1"; $resn_exch{"HBF2 FE1"}="2HBF FE1";
$resn_exch{"HBF3 FE1"}="3HBF FE1"; $resn_exch{"HAB1 FE1"}="1HAB FE1"; $resn_exch{"HAB2 FE1"}="2HAB FE1";
$resn_exch{"HAB3 FE1"}="3HAB FE1"; $resn_exch{" H91 FE1"}="1H9  FE1"; $resn_exch{" H92 FE1"}="2H9  FE1";
$resn_exch{"HN41 FE1"}="1HN4 FE1"; $resn_exch{"HN42 FE1"}="2HN4 FE1"; $resn_exch{"HN21 FE1"}="1HN2 FE1";
$resn_exch{"HN22 FE1"}="2HN2 FE1"; $resn_exch{"HBA1 FEC"}="1HBA FEC"; $resn_exch{"HBA2 FEC"}="2HBA FEC";
$resn_exch{"HAA1 FEC"}="1HAA FEC"; $resn_exch{"HAA2 FEC"}="2HAA FEC"; $resn_exch{"HMB1 FEC"}="1HMB FEC";
$resn_exch{"HMB2 FEC"}="2HMB FEC"; $resn_exch{"HMB3 FEC"}="3HMB FEC"; $resn_exch{"HAB1 FEC"}="1HAB FEC";
$resn_exch{"HAB2 FEC"}="2HAB FEC"; $resn_exch{"HBB1 FEC"}="1HBB FEC"; $resn_exch{"HBB2 FEC"}="2HBB FEC";
$resn_exch{"HAC1 FEC"}="1HAC FEC"; $resn_exch{"HAC2 FEC"}="2HAC FEC"; $resn_exch{"HBC1 FEC"}="1HBC FEC";
$resn_exch{"HBC2 FEC"}="2HBC FEC"; $resn_exch{"HMC1 FEC"}="1HMC FEC"; $resn_exch{"HMC2 FEC"}="2HMC FEC";
$resn_exch{"HMC3 FEC"}="3HMC FEC"; $resn_exch{"HAD1 FEC"}="1HAD FEC"; $resn_exch{"HAD2 FEC"}="2HAD FEC";
$resn_exch{"HBD1 FEC"}="1HBD FEC"; $resn_exch{"HBD2 FEC"}="2HBD FEC"; $resn_exch{"HMD1 FEC"}="1HMD FEC";
$resn_exch{"HMD2 FEC"}="2HMD FEC"; $resn_exch{"HMD3 FEC"}="3HMD FEC"; $resn_exch{"HMA1 FEC"}="1HMA FEC";
$resn_exch{"HMA2 FEC"}="2HMA FEC"; $resn_exch{"HMA3 FEC"}="3HMA FEC"; $resn_exch{"HO11 FEL"}="1HO1 FEL";
$resn_exch{"HO12 FEL"}="2HO1 FEL"; $resn_exch{"HO21 FEL"}="1HO2 FEL"; $resn_exch{"HO22 FEL"}="2HO2 FEL";
$resn_exch{"HO31 FEL"}="1HO3 FEL"; $resn_exch{"HO32 FEL"}="2HO3 FEL"; $resn_exch{"H111 FEM"}="1H11 FEM";
$resn_exch{"H112 FEM"}="2H11 FEM"; $resn_exch{"H121 FEM"}="1H12 FEM"; $resn_exch{"H122 FEM"}="2H12 FEM";
$resn_exch{"H151 FEM"}="1H15 FEM"; $resn_exch{"H152 FEM"}="2H15 FEM"; $resn_exch{"H161 FEM"}="1H16 FEM";
$resn_exch{"H162 FEM"}="2H16 FEM"; $resn_exch{" H21 FEN"}="1H2  FEN"; $resn_exch{" H22 FEN"}="2H2  FEN";
$resn_exch{" H31 FEN"}="1H3  FEN"; $resn_exch{" H32 FEN"}="2H3  FEN"; $resn_exch{" H41 FEN"}="1H4  FEN";
$resn_exch{" H42 FEN"}="2H4  FEN"; $resn_exch{"H161 FEN"}="1H16 FEN"; $resn_exch{"H162 FEN"}="2H16 FEN";
$resn_exch{"H163 FEN"}="3H16 FEN"; $resn_exch{"H171 FEN"}="1H17 FEN"; $resn_exch{"H172 FEN"}="2H17 FEN";
$resn_exch{"H173 FEN"}="3H17 FEN"; $resn_exch{"H181 FEN"}="1H18 FEN"; $resn_exch{"H182 FEN"}="2H18 FEN";
$resn_exch{"H183 FEN"}="3H18 FEN"; $resn_exch{"H191 FEN"}="1H19 FEN"; $resn_exch{"H192 FEN"}="2H19 FEN";
$resn_exch{"H193 FEN"}="3H19 FEN"; $resn_exch{"H201 FEN"}="1H20 FEN"; $resn_exch{"H202 FEN"}="2H20 FEN";
$resn_exch{"H203 FEN"}="3H20 FEN"; $resn_exch{"HO4' FEN"}="'HO4 FEN"; $resn_exch{"HC11 FEP"}="1HC1 FEP";
$resn_exch{"HC12 FEP"}="2HC1 FEP"; $resn_exch{"HC21 FEP"}="1HC2 FEP"; $resn_exch{"HC22 FEP"}="2HC2 FEP";
$resn_exch{"HC31 FEP"}="1HC3 FEP"; $resn_exch{"HC32 FEP"}="2HC3 FEP"; $resn_exch{"HC41 FEP"}="1HC4 FEP";
$resn_exch{"HC42 FEP"}="2HC4 FEP"; $resn_exch{"H101 FER"}="1H10 FER"; $resn_exch{"H102 FER"}="2H10 FER";
$resn_exch{"H103 FER"}="3H10 FER"; $resn_exch{"H261 FEX"}="1H26 FEX"; $resn_exch{"H262 FEX"}="2H26 FEX";
$resn_exch{"H263 FEX"}="3H26 FEX"; $resn_exch{"H241 FEX"}="1H24 FEX"; $resn_exch{"H242 FEX"}="2H24 FEX";
$resn_exch{"H231 FEX"}="1H23 FEX"; $resn_exch{"H232 FEX"}="2H23 FEX"; $resn_exch{"H151 FEX"}="1H15 FEX";
$resn_exch{"H152 FEX"}="2H15 FEX"; $resn_exch{" H21 FEX"}="1H2  FEX"; $resn_exch{"H22A FEX"}="2H2  FEX";
$resn_exch{" H23 FEX"}="3H2  FEX"; $resn_exch{"H31A FEX"}="1H3  FEX"; $resn_exch{"H32A FEX"}="2H3  FEX";
$resn_exch{" H33 FEX"}="3H3  FEX"; $resn_exch{"HN41 FF3"}="1HN4 FF3"; $resn_exch{"HN42 FF3"}="2HN4 FF3";
$resn_exch{"HN31 FF3"}="1HN3 FF3"; $resn_exch{"HN32 FF3"}="2HN3 FF3"; $resn_exch{" H11 FFA"}="1H1  FFA";
$resn_exch{" H12 FFA"}="2H1  FFA"; $resn_exch{" H21 FFA"}="1H2  FFA"; $resn_exch{" H22 FFA"}="2H2  FFA";
$resn_exch{" H91 FFA"}="1H9  FFA"; $resn_exch{" H92 FFA"}="2H9  FFA"; $resn_exch{"H101 FFA"}="1H10 FFA";
$resn_exch{"H102 FFA"}="2H10 FFA"; $resn_exch{"H111 FFA"}="1H11 FFA"; $resn_exch{"H112 FFA"}="2H11 FFA";
$resn_exch{"H121 FFA"}="1H12 FFA"; $resn_exch{"H122 FFA"}="2H12 FFA"; $resn_exch{"H151 FFA"}="1H15 FFA";
$resn_exch{"H152 FFA"}="2H15 FFA"; $resn_exch{"H161 FFA"}="1H16 FFA"; $resn_exch{"H162 FFA"}="2H16 FFA";
$resn_exch{"H181 FFA"}="1H18 FFA"; $resn_exch{"H182 FFA"}="2H18 FFA"; $resn_exch{"H183 FFA"}="3H18 FFA";
$resn_exch{"H191 FFA"}="1H19 FFA"; $resn_exch{"H192 FFA"}="2H19 FFA"; $resn_exch{"H193 FFA"}="3H19 FFA";
$resn_exch{"H01C FFB"}="CH01 FFB"; $resn_exch{"H02C FFB"}="CH02 FFB"; $resn_exch{"H04C FFB"}="CH04 FFB";
$resn_exch{"H05C FFB"}="CH05 FFB"; $resn_exch{"H09N FFB"}="NH09 FFB"; $resn_exch{"H101 FFB"}="1H10 FFB";
$resn_exch{"H102 FFB"}="2H10 FFB"; $resn_exch{"HNP1 FFB"}="1HNP FFB"; $resn_exch{"HNP2 FFB"}="2HNP FFB";
$resn_exch{"H1A1 FFC"}="1H1A FFC"; $resn_exch{"H1A2 FFC"}="2H1A FFC"; $resn_exch{"H6A1 FFC"}="1H6A FFC";
$resn_exch{"H6A2 FFC"}="2H6A FFC"; $resn_exch{"HO3A FFC"}="AHO3 FFC"; $resn_exch{"HO6A FFC"}="AHO6 FFC";
$resn_exch{"H6B1 FFC"}="1H6B FFC"; $resn_exch{"H6B2 FFC"}="2H6B FFC"; $resn_exch{"HO2B FFC"}="BHO2 FFC";
$resn_exch{"HO3B FFC"}="BHO3 FFC"; $resn_exch{"HO4B FFC"}="BHO4 FFC"; $resn_exch{"HO6B FFC"}="BHO6 FFC";
$resn_exch{"HOP3 FFD"}="3HOP FFD"; $resn_exch{" H1' FFD"}=" H1* FFD"; $resn_exch{"H2'1 FFD"}="1H2* FFD";
$resn_exch{"H2'2 FFD"}="2H2* FFD"; $resn_exch{" H3' FFD"}=" H3* FFD"; $resn_exch{" H4' FFD"}=" H4* FFD";
$resn_exch{"H5'1 FFD"}="1H5* FFD"; $resn_exch{"H5'2 FFD"}="2H5* FFD"; $resn_exch{"HOP2 FFD"}="2HOP FFD";
$resn_exch{"HOA2 FFF"}="2HOA FFF"; $resn_exch{"HOB2 FFF"}="2HOB FFF"; $resn_exch{"HOB3 FFF"}="3HOB FFF";
$resn_exch{" H11 FFF"}="1H1  FFF"; $resn_exch{" H12 FFF"}="2H1  FFF"; $resn_exch{" H41 FFF"}="1H4  FFF";
$resn_exch{" H42 FFF"}="2H4  FFF"; $resn_exch{" H51 FFF"}="1H5  FFF"; $resn_exch{" H52 FFF"}="2H5  FFF";
$resn_exch{" H81 FFF"}="1H8  FFF"; $resn_exch{" H82 FFF"}="2H8  FFF"; $resn_exch{" H91 FFF"}="1H9  FFF";
$resn_exch{" H92 FFF"}="2H9  FFF"; $resn_exch{"H121 FFF"}="1H12 FFF"; $resn_exch{"H122 FFF"}="2H12 FFF";
$resn_exch{"H123 FFF"}="3H12 FFF"; $resn_exch{"H151 FFF"}="1H15 FFF"; $resn_exch{"H152 FFF"}="2H15 FFF";
$resn_exch{"H153 FFF"}="3H15 FFF"; $resn_exch{"H131 FFF"}="1H13 FFF"; $resn_exch{"H132 FFF"}="2H13 FFF";
$resn_exch{"H133 FFF"}="3H13 FFF"; $resn_exch{"HN21 FFO"}="1HN2 FFO"; $resn_exch{"HN22 FFO"}="2HN2 FFO";
$resn_exch{" H91 FFO"}="1H9  FFO"; $resn_exch{" H92 FFO"}="2H9  FFO"; $resn_exch{" HB1 FFO"}="1HB  FFO";
$resn_exch{" HB2 FFO"}="2HB  FFO"; $resn_exch{" HG1 FFO"}="1HG  FFO"; $resn_exch{" HG2 FFO"}="2HG  FFO";
$resn_exch{"HOE2 FFO"}="2HOE FFO"; $resn_exch{"H181 FG1"}="1H18 FG1"; $resn_exch{"H182 FG1"}="2H18 FG1";
$resn_exch{"H171 FG1"}="1H17 FG1"; $resn_exch{"H172 FG1"}="2H17 FG1"; $resn_exch{" H71 FG1"}="1H7  FG1";
$resn_exch{" H72 FG1"}="2H7  FG1"; $resn_exch{" H61 FG1"}="1H6  FG1"; $resn_exch{" H62 FG1"}="2H6  FG1";
$resn_exch{"H21A FG1"}="1H2  FG1"; $resn_exch{" H22 FG1"}="2H2  FG1"; $resn_exch{"H141 FG1"}="1H14 FG1";
$resn_exch{"H142 FG1"}="2H14 FG1"; $resn_exch{" H2  FGL"}="2H   FGL"; $resn_exch{" H   FGL"}="1H   FGL";
$resn_exch{" HN1 FGP"}="1HN  FGP"; $resn_exch{" HN2 FGP"}="2HN  FGP"; $resn_exch{"H101 FGR"}="1H10 FGR";
$resn_exch{"H102 FGR"}="2H10 FGR"; $resn_exch{"H231 FGR"}="1H23 FGR"; $resn_exch{"H232 FGR"}="2H23 FGR";
$resn_exch{"H2C1 FHL"}="1H2C FHL"; $resn_exch{"H2C2 FHL"}="2H2C FHL"; $resn_exch{"H3C1 FHL"}="1H3C FHL";
$resn_exch{"H3C2 FHL"}="2H3C FHL"; $resn_exch{"H5C1 FHL"}="1H5C FHL"; $resn_exch{"H5C2 FHL"}="2H5C FHL";
$resn_exch{"HEC1 FHL"}="1HEC FHL"; $resn_exch{"HEC2 FHL"}="2HEC FHL"; $resn_exch{"HDC1 FHL"}="1HDC FHL";
$resn_exch{"HDC2 FHL"}="2HDC FHL"; $resn_exch{"HGC1 FHL"}="1HGC FHL"; $resn_exch{"HGC2 FHL"}="2HGC FHL";
$resn_exch{"HBC1 FHL"}="1HBC FHL"; $resn_exch{"HBC2 FHL"}="2HBC FHL"; $resn_exch{" HN1 FHL"}="1HN  FHL";
$resn_exch{" HN2 FHL"}="2HN  FHL"; $resn_exch{"H131 FHM"}="1H13 FHM"; $resn_exch{"H132 FHM"}="2H13 FHM";
$resn_exch{"H121 FHM"}="1H12 FHM"; $resn_exch{"H122 FHM"}="2H12 FHM"; $resn_exch{"H123 FHM"}="3H12 FHM";
$resn_exch{" H41 FHP"}="1H4  FHP"; $resn_exch{" H42 FHP"}="2H4  FHP"; $resn_exch{" H51 FHP"}="1H5  FHP";
$resn_exch{" H52 FHP"}="2H5  FHP"; $resn_exch{" H81 FHP"}="1H8  FHP"; $resn_exch{" H82 FHP"}="2H8  FHP";
$resn_exch{" H91 FHP"}="1H9  FHP"; $resn_exch{" H92 FHP"}="2H9  FHP"; $resn_exch{"H121 FHP"}="1H12 FHP";
$resn_exch{"H122 FHP"}="2H12 FHP"; $resn_exch{"H123 FHP"}="3H12 FHP"; $resn_exch{"H131 FHP"}="1H13 FHP";
$resn_exch{"H132 FHP"}="2H13 FHP"; $resn_exch{"H133 FHP"}="3H13 FHP"; $resn_exch{"H141 FHP"}="1H14 FHP";
$resn_exch{"H142 FHP"}="2H14 FHP"; $resn_exch{"H143 FHP"}="3H14 FHP"; $resn_exch{"H151 FHP"}="1H15 FHP";
$resn_exch{"H152 FHP"}="2H15 FHP"; $resn_exch{"H153 FHP"}="3H15 FHP"; $resn_exch{"HOA2 FHP"}="2HOA FHP";
$resn_exch{"HOA3 FHP"}="3HOA FHP"; $resn_exch{" H1' FHU"}=" H1* FHU"; $resn_exch{" H2' FHU"}=" H2* FHU";
$resn_exch{"HO2' FHU"}="2HO* FHU"; $resn_exch{" H3' FHU"}=" H3* FHU"; $resn_exch{" H4' FHU"}=" H4* FHU";
$resn_exch{"HO3' FHU"}="H3T  FHU"; $resn_exch{" H5' FHU"}="1H5* FHU"; $resn_exch{"H5'' FHU"}="2H5* FHU";
$resn_exch{"HOP2 FHU"}="2HOP FHU"; $resn_exch{"HOP3 FHU"}="3HOP FHU"; $resn_exch{"H211 FID"}="1H21 FID";
$resn_exch{"H212 FID"}="2H21 FID"; $resn_exch{"H8I1 FID"}="1H8I FID"; $resn_exch{"H8I2 FID"}="2H8I FID";
$resn_exch{"H251 FIH"}="1H25 FIH"; $resn_exch{"H252 FIH"}="2H25 FIH"; $resn_exch{"H253 FIH"}="3H25 FIH";
$resn_exch{" H11 FII"}="1H1  FII"; $resn_exch{" H12 FII"}="2H1  FII"; $resn_exch{" H13 FII"}="3H1  FII";
$resn_exch{" H61 FII"}="1H6  FII"; $resn_exch{" H62 FII"}="2H6  FII"; $resn_exch{" H63 FII"}="3H6  FII";
$resn_exch{"H101 FII"}="1H10 FII"; $resn_exch{"H111 FII"}="1H11 FII"; $resn_exch{"H112 FII"}="2H11 FII";
$resn_exch{"H121 FII"}="1H12 FII"; $resn_exch{"H122 FII"}="2H12 FII"; $resn_exch{"H181 FII"}="1H18 FII";
$resn_exch{"H182 FII"}="2H18 FII"; $resn_exch{"H183 FII"}="3H18 FII"; $resn_exch{"H221 FII"}="1H22 FII";
$resn_exch{"H231 FII"}="1H23 FII"; $resn_exch{"H232 FII"}="2H23 FII"; $resn_exch{"H241 FII"}="1H24 FII";
$resn_exch{"H242 FII"}="2H24 FII"; $resn_exch{"H301 FII"}="1H30 FII"; $resn_exch{"H302 FII"}="2H30 FII";
$resn_exch{"H303 FII"}="3H30 FII"; $resn_exch{"H341 FII"}="1H34 FII"; $resn_exch{"H351 FII"}="1H35 FII";
$resn_exch{"H352 FII"}="2H35 FII"; $resn_exch{" HN1 FII"}="1HN  FII"; $resn_exch{"H451 FII"}="1H45 FII";
$resn_exch{"H452 FII"}="2H45 FII"; $resn_exch{"H501 FII"}="1H50 FII"; $resn_exch{"H511 FII"}="1H51 FII";
$resn_exch{"H201 FIL"}="1H20 FIL"; $resn_exch{"H202 FIL"}="2H20 FIL"; $resn_exch{"H171 FIL"}="1H17 FIL";
$resn_exch{"H172 FIL"}="2H17 FIL"; $resn_exch{"H173 FIL"}="3H17 FIL"; $resn_exch{" H81 FIL"}="1H8  FIL";
$resn_exch{" H82 FIL"}="2H8  FIL"; $resn_exch{" H83 FIL"}="3H8  FIL"; $resn_exch{"H121 FIL"}="1H12 FIL";
$resn_exch{"H122 FIL"}="2H12 FIL"; $resn_exch{"H131 FIL"}="1H13 FIL"; $resn_exch{"H132 FIL"}="2H13 FIL";
$resn_exch{"H141 FIL"}="1H14 FIL"; $resn_exch{"H142 FIL"}="2H14 FIL"; $resn_exch{"H151 FIL"}="1H15 FIL";
$resn_exch{"H152 FIL"}="2H15 FIL"; $resn_exch{" H31 FIN"}="1H3  FIN"; $resn_exch{" H32 FIN"}="2H3  FIN";
$resn_exch{" H33 FIN"}="3H3  FIN"; $resn_exch{"H121 FIN"}="1H12 FIN"; $resn_exch{"H122 FIN"}="2H12 FIN";
$resn_exch{"H123 FIN"}="3H12 FIN"; $resn_exch{"H131 FIN"}="1H13 FIN"; $resn_exch{"H132 FIN"}="2H13 FIN";
$resn_exch{"H133 FIN"}="3H13 FIN"; $resn_exch{"H3P1 FIP"}="1H3P FIP"; $resn_exch{"H3P2 FIP"}="2H3P FIP";
$resn_exch{"H2P1 FIP"}="1H2P FIP"; $resn_exch{"H2P2 FIP"}="2H2P FIP"; $resn_exch{"H1P1 FIP"}="1H1P FIP";
$resn_exch{"H1P2 FIP"}="2H1P FIP"; $resn_exch{"H211 FIR"}="1H21 FIR"; $resn_exch{"H212 FIR"}="2H21 FIR";
$resn_exch{"H8I1 FIR"}="1H8I FIR"; $resn_exch{"H8I2 FIR"}="2H8I FIR"; $resn_exch{"H211 FIS"}="1H21 FIS";
$resn_exch{"H212 FIS"}="2H21 FIS"; $resn_exch{"H8I1 FIS"}="1H8I FIS"; $resn_exch{"H8I2 FIS"}="2H8I FIS";
$resn_exch{"H31A FK5"}="1H3  FK5"; $resn_exch{"H32A FK5"}="2H3  FK5"; $resn_exch{" H41 FK5"}="1H4  FK5";
$resn_exch{" H42 FK5"}="2H4  FK5"; $resn_exch{" H51 FK5"}="1H5  FK5"; $resn_exch{" H52 FK5"}="2H5  FK5";
$resn_exch{" H61 FK5"}="1H6  FK5"; $resn_exch{" H62 FK5"}="2H6  FK5"; $resn_exch{"H121 FK5"}="1H12 FK5";
$resn_exch{"H122 FK5"}="2H12 FK5"; $resn_exch{"H161 FK5"}="1H16 FK5"; $resn_exch{"H162 FK5"}="2H16 FK5";
$resn_exch{"H181 FK5"}="1H18 FK5"; $resn_exch{"H182 FK5"}="2H18 FK5"; $resn_exch{"H231 FK5"}="1H23 FK5";
$resn_exch{"H232 FK5"}="2H23 FK5"; $resn_exch{"H301 FK5"}="1H30 FK5"; $resn_exch{"H302 FK5"}="2H30 FK5";
$resn_exch{"H331 FK5"}="1H33 FK5"; $resn_exch{"H332 FK5"}="2H33 FK5"; $resn_exch{"H341 FK5"}="1H34 FK5";
$resn_exch{"H342 FK5"}="2H34 FK5"; $resn_exch{"H351 FK5"}="1H35 FK5"; $resn_exch{"H352 FK5"}="2H35 FK5";
$resn_exch{"H353 FK5"}="3H35 FK5"; $resn_exch{"H361 FK5"}="1H36 FK5"; $resn_exch{"H362 FK5"}="2H36 FK5";
$resn_exch{"H363 FK5"}="3H36 FK5"; $resn_exch{"H371 FK5"}="1H37 FK5"; $resn_exch{"H372 FK5"}="2H37 FK5";
$resn_exch{"H373 FK5"}="3H37 FK5"; $resn_exch{"H381 FK5"}="1H38 FK5"; $resn_exch{"H382 FK5"}="2H38 FK5";
$resn_exch{"H401 FK5"}="1H40 FK5"; $resn_exch{"H402 FK5"}="2H40 FK5"; $resn_exch{"H411 FK5"}="1H41 FK5";
$resn_exch{"H412 FK5"}="2H41 FK5"; $resn_exch{"H413 FK5"}="3H41 FK5"; $resn_exch{"H421 FK5"}="1H42 FK5";
$resn_exch{"H422 FK5"}="2H42 FK5"; $resn_exch{"H423 FK5"}="3H42 FK5"; $resn_exch{"H431 FK5"}="1H43 FK5";
$resn_exch{"H432 FK5"}="2H43 FK5"; $resn_exch{"H433 FK5"}="3H43 FK5"; $resn_exch{"H441 FK5"}="1H44 FK5";
$resn_exch{"H442 FK5"}="2H44 FK5"; $resn_exch{"H443 FK5"}="3H44 FK5"; $resn_exch{"H451 FK5"}="1H45 FK5";
$resn_exch{"H452 FK5"}="2H45 FK5"; $resn_exch{"H453 FK5"}="3H45 FK5"; $resn_exch{"HO10 FK5"}="0HO1 FK5";
$resn_exch{"HO12 FK5"}="2HO1 FK5"; $resn_exch{"H31A FKA"}="1H3  FKA"; $resn_exch{" H32 FKA"}="2H3  FKA";
$resn_exch{" H41 FKA"}="1H4  FKA"; $resn_exch{" H42 FKA"}="2H4  FKA"; $resn_exch{" H51 FKA"}="1H5  FKA";
$resn_exch{" H52 FKA"}="2H5  FKA"; $resn_exch{" H61 FKA"}="1H6  FKA"; $resn_exch{" H62 FKA"}="2H6  FKA";
$resn_exch{"H111 FKA"}="1H11 FKA"; $resn_exch{"H112 FKA"}="2H11 FKA"; $resn_exch{"H341 FKA"}="1H34 FKA";
$resn_exch{"H342 FKA"}="2H34 FKA"; $resn_exch{"H343 FKA"}="3H34 FKA"; $resn_exch{"H421 FKA"}="1H42 FKA";
$resn_exch{"H422 FKA"}="2H42 FKA"; $resn_exch{"H423 FKA"}="3H42 FKA"; $resn_exch{"H431 FKA"}="1H43 FKA";
$resn_exch{"H432 FKA"}="2H43 FKA"; $resn_exch{"H433 FKA"}="3H43 FKA"; $resn_exch{"H151 FKA"}="1H15 FKA";
$resn_exch{"H152 FKA"}="2H15 FKA"; $resn_exch{"H351 FKA"}="1H35 FKA"; $resn_exch{"H352 FKA"}="2H35 FKA";
$resn_exch{"H353 FKA"}="3H35 FKA"; $resn_exch{"H171 FKA"}="1H17 FKA"; $resn_exch{"H172 FKA"}="2H17 FKA";
$resn_exch{"H361 FKA"}="1H36 FKA"; $resn_exch{"H362 FKA"}="2H36 FKA"; $resn_exch{"H363 FKA"}="3H36 FKA";
$resn_exch{"H371 FKA"}="1H37 FKA"; $resn_exch{"H372 FKA"}="2H37 FKA"; $resn_exch{"H221 FKA"}="1H22 FKA";
$resn_exch{"H222 FKA"}="2H22 FKA"; $resn_exch{"H401 FKA"}="1H40 FKA"; $resn_exch{"H402 FKA"}="2H40 FKA";
$resn_exch{"H403 FKA"}="3H40 FKA"; $resn_exch{"H411 FKA"}="1H41 FKA"; $resn_exch{"H412 FKA"}="2H41 FKA";
$resn_exch{"H413 FKA"}="3H41 FKA"; $resn_exch{"H291 FKA"}="1H29 FKA"; $resn_exch{"H292 FKA"}="2H29 FKA";
$resn_exch{"H321 FKA"}="1H32 FKA"; $resn_exch{"H322 FKA"}="2H32 FKA"; $resn_exch{"H331 FKA"}="1H33 FKA";
$resn_exch{"H332 FKA"}="2H33 FKA"; $resn_exch{"H441 FKA"}="1H44 FKA"; $resn_exch{"H442 FKA"}="2H44 FKA";
$resn_exch{"H443 FKA"}="3H44 FKA"; $resn_exch{"H511 FKA"}="1H51 FKA"; $resn_exch{"H512 FKA"}="2H51 FKA";
$resn_exch{"H491 FKA"}="1H49 FKA"; $resn_exch{"H492 FKA"}="2H49 FKA"; $resn_exch{" H91 FKD"}="1H9  FKD";
$resn_exch{" H92 FKD"}="2H9  FKD"; $resn_exch{" H2  FKI"}="2H   FKI"; $resn_exch{" H1  FKI"}="1H   FKI";
$resn_exch{" HB2 FKI"}="2HB  FKI"; $resn_exch{" HB1 FKI"}="1HB  FKI"; $resn_exch{" H21 FKP"}="1H2  FKP";
$resn_exch{" H22 FKP"}="2H2  FKP"; $resn_exch{" H31 FKP"}="1H3  FKP"; $resn_exch{" H32 FKP"}="2H3  FKP";
$resn_exch{"H121 FKP"}="1H12 FKP"; $resn_exch{"H122 FKP"}="2H12 FKP"; $resn_exch{"H151 FKP"}="1H15 FKP";
$resn_exch{"H152 FKP"}="2H15 FKP"; $resn_exch{"H161 FKP"}="1H16 FKP"; $resn_exch{"H162 FKP"}="2H16 FKP";
$resn_exch{"H163 FKP"}="3H16 FKP"; $resn_exch{"H171 FKP"}="1H17 FKP"; $resn_exch{"H172 FKP"}="2H17 FKP";
$resn_exch{"H173 FKP"}="3H17 FKP"; $resn_exch{"H181 FKP"}="1H18 FKP"; $resn_exch{"H182 FKP"}="2H18 FKP";
$resn_exch{"H183 FKP"}="3H18 FKP"; $resn_exch{"H191 FKP"}="1H19 FKP"; $resn_exch{"H192 FKP"}="2H19 FKP";
$resn_exch{"H193 FKP"}="3H19 FKP"; $resn_exch{"H201 FKP"}="1H20 FKP"; $resn_exch{"H202 FKP"}="2H20 FKP";
$resn_exch{"H203 FKP"}="3H20 FKP"; $resn_exch{"H221 FKP"}="1H22 FKP"; $resn_exch{"H222 FKP"}="2H22 FKP";
$resn_exch{"H231 FKP"}="1H23 FKP"; $resn_exch{"H232 FKP"}="2H23 FKP"; $resn_exch{"H241 FKP"}="1H24 FKP";
$resn_exch{"H242 FKP"}="2H24 FKP"; $resn_exch{"H251 FKP"}="1H25 FKP"; $resn_exch{"H252 FKP"}="2H25 FKP";
$resn_exch{"H261 FKP"}="1H26 FKP"; $resn_exch{"H262 FKP"}="2H26 FKP"; $resn_exch{"H271 FKP"}="1H27 FKP";
$resn_exch{"H272 FKP"}="2H27 FKP"; $resn_exch{"H281 FKP"}="1H28 FKP"; $resn_exch{"H282 FKP"}="2H28 FKP";
$resn_exch{"H291 FKP"}="1H29 FKP"; $resn_exch{"H292 FKP"}="2H29 FKP"; $resn_exch{"H301 FKP"}="1H30 FKP";
$resn_exch{"H302 FKP"}="2H30 FKP"; $resn_exch{"H303 FKP"}="3H30 FKP"; $resn_exch{"HN11 FL1"}="1HN1 FL1";
$resn_exch{"HN12 FL1"}="2HN1 FL1"; $resn_exch{" H71 FL1"}="1H7  FL1"; $resn_exch{" H72 FL1"}="2H7  FL1";
$resn_exch{"H291 FL1"}="1H29 FL1"; $resn_exch{"H292 FL1"}="2H29 FL1"; $resn_exch{"H131 FL2"}="1H13 FL2";
$resn_exch{"H132 FL2"}="2H13 FL2"; $resn_exch{"H133 FL2"}="3H13 FL2"; $resn_exch{"H161 FL2"}="1H16 FL2";
$resn_exch{"H162 FL2"}="2H16 FL2"; $resn_exch{"H163 FL2"}="3H16 FL2"; $resn_exch{" H51 FL8"}="1H5  FL8";
$resn_exch{" H71 FL8"}="1H7  FL8"; $resn_exch{" H81 FL8"}="1H8  FL8"; $resn_exch{"H2'1 FL8"}="1H2' FL8";
$resn_exch{"H6'1 FL8"}="1H6' FL8"; $resn_exch{"H4'O FL8"}="OH4' FL8"; $resn_exch{"H3'1 FL8"}="1H3* FL8";
$resn_exch{"H3'2 FL8"}="2H3* FL8"; $resn_exch{"H3'3 FL8"}="3H3* FL8"; $resn_exch{"HO61 FL8"}="1HO6 FL8";
$resn_exch{" H6' FL9"}=" H6* FL9"; $resn_exch{"HO2' FL9"}="*HO2 FL9"; $resn_exch{"HO4' FL9"}="*HO4 FL9";
$resn_exch{" H2  FLA"}=" HN2 FLA"; $resn_exch{" HA1 FLC"}="1HA  FLC"; $resn_exch{" HA2 FLC"}="2HA  FLC";
$resn_exch{" HG1 FLC"}="1HG  FLC"; $resn_exch{" HG2 FLC"}="2HG  FLC"; $resn_exch{"H181 FLD"}="1H18 FLD";
$resn_exch{"H182 FLD"}="2H18 FLD"; $resn_exch{"H183 FLD"}="3H18 FLD"; $resn_exch{"H191 FLD"}="1H19 FLD";
$resn_exch{"H192 FLD"}="2H19 FLD"; $resn_exch{"H201 FLD"}="1H20 FLD"; $resn_exch{"H202 FLD"}="2H20 FLD";
$resn_exch{"HN21 FLD"}="1HN2 FLD"; $resn_exch{"HN22 FLD"}="2HN2 FLD"; $resn_exch{"H221 FLD"}="1H22 FLD";
$resn_exch{"H222 FLD"}="2H22 FLD"; $resn_exch{"H231 FLD"}="1H23 FLD"; $resn_exch{"H232 FLD"}="2H23 FLD";
$resn_exch{"H241 FLD"}="1H24 FLD"; $resn_exch{"H242 FLD"}="2H24 FLD"; $resn_exch{" H1X FLD"}=" XH1 FLD";
$resn_exch{" H3X FLD"}=" XH3 FLD"; $resn_exch{" H4X FLD"}=" XH4 FLD"; $resn_exch{" H5X FLD"}=" XH5 FLD";
$resn_exch{" H6X FLD"}=" XH6 FLD"; $resn_exch{"HN7X FLD"}="XHN7 FLD"; $resn_exch{" H8X FLD"}=" XH8 FLD";
$resn_exch{" H9X FLD"}=" XH9 FLD"; $resn_exch{"H11X FLD"}="XH11 FLD"; $resn_exch{"HX81 FLD"}="1HX8 FLD";
$resn_exch{"HX82 FLD"}="2HX8 FLD"; $resn_exch{"HX83 FLD"}="3HX8 FLD"; $resn_exch{"HX91 FLD"}="1HX9 FLD";
$resn_exch{"HX92 FLD"}="2HX9 FLD"; $resn_exch{"HX01 FLD"}="1HX0 FLD"; $resn_exch{"HX02 FLD"}="2HX0 FLD";
$resn_exch{"HN1X FLD"}="XHN1 FLD"; $resn_exch{"HN2X FLD"}="XHN2 FLD"; $resn_exch{"HX21 FLD"}="1HX2 FLD";
$resn_exch{"HX22 FLD"}="2HX2 FLD"; $resn_exch{"HX31 FLD"}="1HX3 FLD"; $resn_exch{"HX32 FLD"}="2HX3 FLD";
$resn_exch{"HX41 FLD"}="1HX4 FLD"; $resn_exch{"HX42 FLD"}="2HX4 FLD"; $resn_exch{" H   FLE"}=" HN  FLE";
$resn_exch{" HB2 FLE"}="1HB  FLE"; $resn_exch{" HB3 FLE"}="2HB  FLE"; $resn_exch{"HD11 FLE"}="1HD1 FLE";
$resn_exch{"HD12 FLE"}="2HD1 FLE"; $resn_exch{"HD13 FLE"}="3HD1 FLE"; $resn_exch{"HD21 FLE"}="1HD2 FLE";
$resn_exch{"HD22 FLE"}="2HD2 FLE"; $resn_exch{"HD23 FLE"}="3HD2 FLE"; $resn_exch{"H221 FLG"}="1H22 FLG";
$resn_exch{"H222 FLG"}="2H22 FLG"; $resn_exch{"H231 FLG"}="1H23 FLG"; $resn_exch{"H232 FLG"}="2H23 FLG";
$resn_exch{"H233 FLG"}="3H23 FLG"; $resn_exch{" H11 FLH"}="1H1  FLH"; $resn_exch{" H12 FLH"}="2H1  FLH";
$resn_exch{"HC41 FLM"}="1HC4 FLM"; $resn_exch{"HC42 FLM"}="2HC4 FLM"; $resn_exch{"HC43 FLM"}="3HC4 FLM";
$resn_exch{"HN61 FLM"}="1HN6 FLM"; $resn_exch{"HN62 FLM"}="2HN6 FLM"; $resn_exch{" H3  FLN"}="3H   FLN";
$resn_exch{" H2' FLN"}=" H2* FLN"; $resn_exch{" H3' FLN"}=" H3* FLN"; $resn_exch{" H4' FLN"}=" H4* FLN";
$resn_exch{" H5' FLN"}=" H5* FLN"; $resn_exch{" H6' FLN"}=" H6* FLN"; $resn_exch{" H5  FLN"}="5H   FLN";
$resn_exch{" H6  FLN"}="6H   FLN"; $resn_exch{" H7  FLN"}="7H   FLN"; $resn_exch{" H8  FLN"}="8H   FLN";
$resn_exch{"H131 FLP"}="1H13 FLP"; $resn_exch{"H132 FLP"}="2H13 FLP"; $resn_exch{"H133 FLP"}="3H13 FLP";
$resn_exch{"H161 FLQ"}="1H16 FLQ"; $resn_exch{"H162 FLQ"}="2H16 FLQ"; $resn_exch{"H171 FLQ"}="1H17 FLQ";
$resn_exch{"H172 FLQ"}="2H17 FLQ"; $resn_exch{"H181 FLQ"}="1H18 FLQ"; $resn_exch{"H182 FLQ"}="2H18 FLQ";
$resn_exch{"H191 FLQ"}="1H19 FLQ"; $resn_exch{"H192 FLQ"}="2H19 FLQ"; $resn_exch{" H2' FLQ"}=" H2* FLQ";
$resn_exch{" H3' FLQ"}=" H3* FLQ"; $resn_exch{" H6' FLQ"}=" H6* FLQ"; $resn_exch{"H201 FLQ"}="1H20 FLQ";
$resn_exch{"H202 FLQ"}="2H20 FLQ"; $resn_exch{"H211 FLQ"}="1H21 FLQ"; $resn_exch{"H212 FLQ"}="2H21 FLQ";
$resn_exch{"H231 FLQ"}="1H23 FLQ"; $resn_exch{"H232 FLQ"}="2H23 FLQ"; $resn_exch{"H233 FLQ"}="3H23 FLQ";
$resn_exch{" H   FLT"}="1HN  FLT"; $resn_exch{" H2  FLT"}="2HN  FLT"; $resn_exch{" HA1 FLT"}="1HA  FLT";
$resn_exch{" HB2 FLT"}="1HB  FLT"; $resn_exch{" HB3 FLT"}="2HB  FLT"; $resn_exch{" HD1 FLT"}="1HD1 FLT";
$resn_exch{" HE1 FLT"}="1HE1 FLT"; $resn_exch{" HD2 FLT"}="1HD2 FLT"; $resn_exch{" HE2 FLT"}="1HE2 FLT";
$resn_exch{"HO11 FLT"}="1HO1 FLT"; $resn_exch{"HO31 FLT"}="1HO3 FLT"; $resn_exch{" HXT FLT"}="1HOX FLT";
$resn_exch{" HB1 FLX"}="1HB  FLX"; $resn_exch{" HB2 FLX"}="2HB  FLX"; $resn_exch{"HD11 FLX"}="1HD1 FLX";
$resn_exch{"HD12 FLX"}="2HD1 FLX"; $resn_exch{"HD13 FLX"}="3HD1 FLX"; $resn_exch{"HD21 FLX"}="1HD2 FLX";
$resn_exch{"HD22 FLX"}="2HD2 FLX"; $resn_exch{"HD23 FLX"}="3HD2 FLX"; $resn_exch{"H21A FLX"}="1H2  FLX";
$resn_exch{" H22 FLX"}="2H2  FLX"; $resn_exch{"HOP2 FLX"}="2HOP FLX"; $resn_exch{"HOP3 FLX"}="3HOP FLX";
$resn_exch{"HN61 FM1"}="1HN6 FM1"; $resn_exch{"HO2' FM1"}="'HO2 FM1"; $resn_exch{"HO3' FM1"}="'HO3 FM1";
$resn_exch{"H5'1 FM1"}="1H5' FM1"; $resn_exch{"H5'2 FM1"}="2H5' FM1"; $resn_exch{"HO5' FM1"}="'HO5 FM1";
$resn_exch{"H661 FM1"}="1H66 FM1"; $resn_exch{"H662 FM1"}="2H66 FM1"; $resn_exch{"H663 FM1"}="3H66 FM1";
$resn_exch{"HN61 FM2"}="1HN6 FM2"; $resn_exch{"HN62 FM2"}="2HN6 FM2"; $resn_exch{"HO2' FM2"}="'HO2 FM2";
$resn_exch{"HO3' FM2"}="'HO3 FM2"; $resn_exch{"H5'1 FM2"}="1H5' FM2"; $resn_exch{"H5'2 FM2"}="2H5' FM2";
$resn_exch{"HO5' FM2"}="'HO5 FM2"; $resn_exch{"H111 FM2"}="1H11 FM2"; $resn_exch{"H112 FM2"}="2H11 FM2";
$resn_exch{"H113 FM2"}="3H11 FM2"; $resn_exch{"HM71 FMA"}="1HM7 FMA"; $resn_exch{"HM72 FMA"}="2HM7 FMA";
$resn_exch{"HM73 FMA"}="3HM7 FMA"; $resn_exch{"HM81 FMA"}="1HM8 FMA"; $resn_exch{"HM82 FMA"}="2HM8 FMA";
$resn_exch{"HM83 FMA"}="3HM8 FMA"; $resn_exch{"H1'1 FMA"}="1H1* FMA"; $resn_exch{"H1'2 FMA"}="2H1* FMA";
$resn_exch{" H2' FMA"}=" H2* FMA"; $resn_exch{"HO2' FMA"}="*HO2 FMA"; $resn_exch{" H3B FMA"}=" H3* FMA";
$resn_exch{"HO3' FMA"}="*HO3 FMA"; $resn_exch{" H4' FMA"}=" H4* FMA"; $resn_exch{"HO4' FMA"}="*HO4 FMA";
$resn_exch{"H5'1 FMA"}="1H5* FMA"; $resn_exch{"H5'2 FMA"}="2H5* FMA"; $resn_exch{"HOP2 FMA"}="2HOP FMA";
$resn_exch{"HOP3 FMA"}="3HOP FMA"; $resn_exch{"HO2A FMA"}="'HO2 FMA"; $resn_exch{"H2'1 FMA"}="1H2' FMA";
$resn_exch{"H2'2 FMA"}="2H2' FMA"; $resn_exch{"H4'1 FMA"}="1H4' FMA"; $resn_exch{"H4'2 FMA"}="2H4' FMA";
$resn_exch{" H5B FMA"}="1H5' FMA"; $resn_exch{" H5D FMA"}="2H5' FMA"; $resn_exch{"H6'1 FMA"}="1H6' FMA";
$resn_exch{"H6'2 FMA"}="2H6' FMA"; $resn_exch{"H7'1 FMA"}="1H7' FMA"; $resn_exch{"H7'2 FMA"}="2H7' FMA";
$resn_exch{"H8'1 FMA"}="1H8' FMA"; $resn_exch{"H8'2 FMA"}="2H8' FMA"; $resn_exch{"H9'1 FMA"}="1H9' FMA";
$resn_exch{"H9'2 FMA"}="2H9' FMA"; $resn_exch{"H111 FMA"}="1H11 FMA"; $resn_exch{"H112 FMA"}="2H11 FMA";
$resn_exch{"H121 FMA"}="1H12 FMA"; $resn_exch{"H122 FMA"}="2H12 FMA"; $resn_exch{"H131 FMA"}="1H13 FMA";
$resn_exch{"H132 FMA"}="2H13 FMA"; $resn_exch{"H141 FMA"}="1H14 FMA"; $resn_exch{"H142 FMA"}="2H14 FMA";
$resn_exch{"H151 FMA"}="1H15 FMA"; $resn_exch{"H152 FMA"}="2H15 FMA"; $resn_exch{"H153 FMA"}="3H15 FMA";
$resn_exch{"H5'1 FMB"}="1H5' FMB"; $resn_exch{"H5'2 FMB"}="2H5' FMB"; $resn_exch{"HN61 FMC"}="1HN6 FMC";
$resn_exch{"HN62 FMC"}="2HN6 FMC"; $resn_exch{"HO2' FMC"}="'HO2 FMC"; $resn_exch{"HO3' FMC"}="'HO3 FMC";
$resn_exch{"H5'1 FMC"}="1H5' FMC"; $resn_exch{"H5'2 FMC"}="2H5' FMC"; $resn_exch{"HO5' FMC"}="'HO5 FMC";
$resn_exch{"H281 FMD"}="1H28 FMD"; $resn_exch{"H282 FMD"}="2H28 FMD"; $resn_exch{"H251 FMD"}="1H25 FMD";
$resn_exch{"H252 FMD"}="2H25 FMD"; $resn_exch{"H253 FMD"}="3H25 FMD"; $resn_exch{"H221 FMD"}="1H22 FMD";
$resn_exch{"H222 FMD"}="2H22 FMD"; $resn_exch{"H223 FMD"}="3H22 FMD"; $resn_exch{" H   FME"}=" HN  FME";
$resn_exch{" HB2 FME"}="1HB  FME"; $resn_exch{" HB3 FME"}="2HB  FME"; $resn_exch{" HG2 FME"}="1HG  FME";
$resn_exch{" HG3 FME"}="2HG  FME"; $resn_exch{" HE1 FME"}="1HE  FME"; $resn_exch{" HE2 FME"}="2HE  FME";
$resn_exch{" HE3 FME"}="3HE  FME"; $resn_exch{" H61 FMF"}="1H6  FMF"; $resn_exch{" H62 FMF"}="2H6  FMF";
$resn_exch{"HAA1 FMI"}="1HAA FMI"; $resn_exch{"HAA2 FMI"}="2HAA FMI"; $resn_exch{"HMA1 FMI"}="1HMA FMI";
$resn_exch{"HMA2 FMI"}="2HMA FMI"; $resn_exch{"HMA3 FMI"}="3HMA FMI"; $resn_exch{"HBA1 FMI"}="1HBA FMI";
$resn_exch{"HBA2 FMI"}="2HBA FMI"; $resn_exch{"HLB1 FMI"}="1HLB FMI"; $resn_exch{"HLB2 FMI"}="2HLB FMI";
$resn_exch{"HLB3 FMI"}="3HLB FMI"; $resn_exch{"HMB1 FMI"}="1HMB FMI"; $resn_exch{"HMB2 FMI"}="2HMB FMI";
$resn_exch{"HNB1 FMI"}="1HNB FMI"; $resn_exch{"HNB2 FMI"}="2HNB FMI"; $resn_exch{"HNB3 FMI"}="3HNB FMI";
$resn_exch{"HMC1 FMI"}="1HMC FMI"; $resn_exch{"HMC2 FMI"}="2HMC FMI"; $resn_exch{"HMC3 FMI"}="3HMC FMI";
$resn_exch{"HAC1 FMI"}="1HAC FMI"; $resn_exch{"HAC2 FMI"}="2HAC FMI"; $resn_exch{"HBC1 FMI"}="1HBC FMI";
$resn_exch{"HBC2 FMI"}="2HBC FMI"; $resn_exch{"HBC3 FMI"}="3HBC FMI"; $resn_exch{"HMD1 FMI"}="1HMD FMI";
$resn_exch{"HMD2 FMI"}="2HMD FMI"; $resn_exch{"HMD3 FMI"}="3HMD FMI"; $resn_exch{"HAD1 FMI"}="1HAD FMI";
$resn_exch{"HAD2 FMI"}="2HAD FMI"; $resn_exch{"HBD1 FMI"}="1HBD FMI"; $resn_exch{"HBD2 FMI"}="2HBD FMI";
$resn_exch{"H11A FMM"}="1H1  FMM"; $resn_exch{" H12 FMM"}="2H1  FMM"; $resn_exch{" H13 FMM"}="3H1  FMM";
$resn_exch{" H51 FMM"}="1H5  FMM"; $resn_exch{" H52 FMM"}="2H5  FMM"; $resn_exch{" H61 FMM"}="1H6  FMM";
$resn_exch{" H62 FMM"}="2H6  FMM"; $resn_exch{" H81 FMM"}="1H8  FMM"; $resn_exch{" H82 FMM"}="2H8  FMM";
$resn_exch{"H281 FMM"}="1H28 FMM"; $resn_exch{"H282 FMM"}="2H28 FMM"; $resn_exch{"HM71 FMN"}="1HM7 FMN";
$resn_exch{"HM72 FMN"}="2HM7 FMN"; $resn_exch{"HM73 FMN"}="3HM7 FMN"; $resn_exch{"HM81 FMN"}="1HM8 FMN";
$resn_exch{"HM82 FMN"}="2HM8 FMN"; $resn_exch{"HM83 FMN"}="3HM8 FMN"; $resn_exch{"H1'1 FMN"}="1H1* FMN";
$resn_exch{"H1'2 FMN"}="2H1* FMN"; $resn_exch{" H2' FMN"}=" H2* FMN"; $resn_exch{"HO2' FMN"}="*HO2 FMN";
$resn_exch{" H3' FMN"}=" H3* FMN"; $resn_exch{"HO3' FMN"}="*HO3 FMN"; $resn_exch{" H4' FMN"}=" H4* FMN";
$resn_exch{"HO4' FMN"}="*HO4 FMN"; $resn_exch{"H5'1 FMN"}="1H5* FMN"; $resn_exch{"H5'2 FMN"}="2H5* FMN";
$resn_exch{"HOP2 FMN"}="2HOP FMN"; $resn_exch{"HOP3 FMN"}="3HOP FMN"; $resn_exch{"HOP2 FMP"}="2HOP FMP";
$resn_exch{"HOP3 FMP"}="3HOP FMP"; $resn_exch{"H5'1 FMP"}="1H5* FMP"; $resn_exch{"H5'2 FMP"}="2H5* FMP";
$resn_exch{" H4' FMP"}=" H4* FMP"; $resn_exch{" H3' FMP"}=" H3* FMP"; $resn_exch{"HO3' FMP"}="*HO3 FMP";
$resn_exch{" H2' FMP"}=" H2* FMP"; $resn_exch{"HO2' FMP"}="*HO2 FMP"; $resn_exch{" H1' FMP"}=" H1* FMP";
$resn_exch{"HN61 FMP"}="1HN6 FMP"; $resn_exch{"HN62 FMP"}="2HN6 FMP"; $resn_exch{" HN1 FMS"}="1HN  FMS";
$resn_exch{" HN2 FMS"}="2HN  FMS"; $resn_exch{"H5M1 FMU"}="1H5M FMU"; $resn_exch{"H5M2 FMU"}="2H5M FMU";
$resn_exch{"H5M3 FMU"}="3H5M FMU"; $resn_exch{"H6C1 FMU"}="1H6C FMU"; $resn_exch{"H6C2 FMU"}="2H6C FMU";
$resn_exch{" H1' FMU"}=" H1* FMU"; $resn_exch{" H2' FMU"}=" H2* FMU"; $resn_exch{" H3' FMU"}=" H3* FMU";
$resn_exch{" H4' FMU"}=" H4* FMU"; $resn_exch{"H5'1 FMU"}="1H5* FMU"; $resn_exch{"H5'2 FMU"}="2H5* FMU";
$resn_exch{"HN11 FMX"}="1HN1 FMX"; $resn_exch{"H221 FMX"}="1H22 FMX"; $resn_exch{"H231 FMX"}="1H23 FMX";
$resn_exch{"H241 FMX"}="1H24 FMX"; $resn_exch{"H251 FMX"}="1H25 FMX"; $resn_exch{"H261 FMX"}="1H26 FMX";
$resn_exch{" H71 FMX"}="1H7  FMX"; $resn_exch{" H72 FMX"}="2H7  FMX"; $resn_exch{" H73 FMX"}="3H7  FMX";
$resn_exch{" H91 FMX"}="1H9  FMX"; $resn_exch{"H101 FMX"}="1H10 FMX"; $resn_exch{"H121 FMX"}="1H12 FMX";
$resn_exch{"H131 FMX"}="1H13 FMX"; $resn_exch{"H161 FMX"}="1H16 FMX"; $resn_exch{"H171 FMX"}="1H17 FMX";
$resn_exch{"H181 FMX"}="1H18 FMX"; $resn_exch{"H191 FMX"}="1H19 FMX"; $resn_exch{"H201 FMX"}="1H20 FMX";
$resn_exch{" H11 FNG"}="1H1  FNG"; $resn_exch{" H21 FNG"}="1H2  FNG"; $resn_exch{"HO21 FNG"}="1HO2 FNG";
$resn_exch{" H31 FNG"}="1H3  FNG"; $resn_exch{"HO31 FNG"}="1HO3 FNG"; $resn_exch{" H41 FNG"}="1H4  FNG";
$resn_exch{"HO41 FNG"}="1HO4 FNG"; $resn_exch{" H51 FNG"}="1H5  FNG"; $resn_exch{" H61 FNG"}="1H6  FNG";
$resn_exch{" H62 FNG"}="2H6  FNG"; $resn_exch{"HO61 FNG"}="1HO6 FNG"; $resn_exch{"H121 FNG"}="1H12 FNG";
$resn_exch{"H101 FNG"}="1H10 FNG"; $resn_exch{" H81 FNG"}="1H8  FNG"; $resn_exch{"H151 FNG"}="1H15 FNG";
$resn_exch{"H152 FNG"}="2H15 FNG"; $resn_exch{"H31A FNH"}="1H3  FNH"; $resn_exch{"H32A FNH"}="2H3  FNH";
$resn_exch{" H41 FNH"}="1H4  FNH"; $resn_exch{" H42 FNH"}="2H4  FNH"; $resn_exch{"H51A FNH"}="1H5  FNH";
$resn_exch{" H52 FNH"}="2H5  FNH"; $resn_exch{" H61 FNH"}="1H6  FNH"; $resn_exch{" H62 FNH"}="2H6  FNH";
$resn_exch{" H71 FNH"}="1H7  FNH"; $resn_exch{" H72 FNH"}="2H7  FNH"; $resn_exch{" H81 FNH"}="1H8  FNH";
$resn_exch{" H82 FNH"}="2H8  FNH"; $resn_exch{" H91 FNH"}="1H9  FNH"; $resn_exch{" H92 FNH"}="2H9  FNH";
$resn_exch{"H131 FNH"}="1H13 FNH"; $resn_exch{"H132 FNH"}="2H13 FNH"; $resn_exch{"H171 FNH"}="1H17 FNH";
$resn_exch{"H172 FNH"}="2H17 FNH"; $resn_exch{"H141 FNH"}="1H14 FNH"; $resn_exch{"H142 FNH"}="2H14 FNH";
$resn_exch{"H161 FNH"}="1H16 FNH"; $resn_exch{"H162 FNH"}="2H16 FNH"; $resn_exch{"H151 FNH"}="1H15 FNH";
$resn_exch{"H152 FNH"}="2H15 FNH"; $resn_exch{"H221 FNH"}="1H22 FNH"; $resn_exch{"H222 FNH"}="2H22 FNH";
$resn_exch{"H331 FNH"}="1H33 FNH"; $resn_exch{"H332 FNH"}="2H33 FNH"; $resn_exch{"H341 FNH"}="1H34 FNH";
$resn_exch{"H342 FNH"}="2H34 FNH"; $resn_exch{"H351 FNH"}="1H35 FNH"; $resn_exch{"H352 FNH"}="2H35 FNH";
$resn_exch{"H353 FNH"}="3H35 FNH"; $resn_exch{"H401 FNH"}="1H40 FNH"; $resn_exch{"H402 FNH"}="2H40 FNH";
$resn_exch{"H7M1 FNR"}="1H7M FNR"; $resn_exch{"H7M2 FNR"}="2H7M FNR"; $resn_exch{"H7M3 FNR"}="3H7M FNR";
$resn_exch{"H8M1 FNR"}="1H8M FNR"; $resn_exch{"H8M2 FNR"}="2H8M FNR"; $resn_exch{"H8M3 FNR"}="3H8M FNR";
$resn_exch{"H1'1 FNR"}="1H1* FNR"; $resn_exch{"H1'2 FNR"}="2H1* FNR"; $resn_exch{" H2' FNR"}=" H2* FNR";
$resn_exch{" H3' FNR"}=" H3* FNR"; $resn_exch{" H4' FNR"}=" H4* FNR"; $resn_exch{"H5'1 FNR"}="1H5* FNR";
$resn_exch{"H5'2 FNR"}="2H5* FNR"; $resn_exch{"HM71 FNS"}="1HM7 FNS"; $resn_exch{"HM72 FNS"}="2HM7 FNS";
$resn_exch{"HM73 FNS"}="3HM7 FNS"; $resn_exch{"HM81 FNS"}="1HM8 FNS"; $resn_exch{"HM82 FNS"}="2HM8 FNS";
$resn_exch{"HM83 FNS"}="3HM8 FNS"; $resn_exch{"H1'1 FNS"}="1H1* FNS"; $resn_exch{"H1'2 FNS"}="2H1* FNS";
$resn_exch{" H2' FNS"}=" H2* FNS"; $resn_exch{"HO2' FNS"}="*HO2 FNS"; $resn_exch{" H3' FNS"}=" H3* FNS";
$resn_exch{"HO3' FNS"}="*HO3 FNS"; $resn_exch{" H4' FNS"}=" H4* FNS"; $resn_exch{"HO4' FNS"}="*HO4 FNS";
$resn_exch{"H5'1 FNS"}="1H5* FNS"; $resn_exch{"H5'2 FNS"}="2H5* FNS"; $resn_exch{"HOP2 FNS"}="2HOP FNS";
$resn_exch{"HOP3 FNS"}="3HOP FNS"; $resn_exch{"H3C1 FNZ"}="1H3C FNZ"; $resn_exch{"H3C2 FNZ"}="2H3C FNZ";
$resn_exch{" H11 FOC"}="1H1  FOC"; $resn_exch{" H12 FOC"}="2H1  FOC"; $resn_exch{" H61 FOC"}="1H6  FOC";
$resn_exch{" H62 FOC"}="2H6  FOC"; $resn_exch{" H63 FOC"}="3H6  FOC"; $resn_exch{" H   FOE"}="1HN  FOE";
$resn_exch{" H2  FOE"}="2HN  FOE"; $resn_exch{" HB2 FOE"}="1HB  FOE"; $resn_exch{" HB3 FOE"}="2HB  FOE";
$resn_exch{"H11A FOE"}="1H1  FOE"; $resn_exch{"H12A FOE"}="2H1  FOE"; $resn_exch{" H51 FOE"}="1H5  FOE";
$resn_exch{" H52 FOE"}="2H5  FOE"; $resn_exch{" H53 FOE"}="3H5  FOE"; $resn_exch{" H71 FOE"}="1H7  FOE";
$resn_exch{" H72 FOE"}="2H7  FOE"; $resn_exch{" H73 FOE"}="3H7  FOE"; $resn_exch{" HC1 FOG"}="1H   FOG";
$resn_exch{" HC2 FOG"}="2H   FOG"; $resn_exch{" H   FOG"}="1HN  FOG"; $resn_exch{" H2  FOG"}="2HN  FOG";
$resn_exch{" HB2 FOG"}="1HB  FOG"; $resn_exch{" HB3 FOG"}="2HB  FOG"; $resn_exch{"H101 FOH"}="1H10 FOH";
$resn_exch{"H102 FOH"}="2H10 FOH"; $resn_exch{"H111 FOH"}="1H11 FOH"; $resn_exch{"H112 FOH"}="2H11 FOH";
$resn_exch{"H133 FOH"}="3H13 FOH"; $resn_exch{"H132 FOH"}="2H13 FOH"; $resn_exch{"H131 FOH"}="1H13 FOH";
$resn_exch{"H151 FOH"}="1H15 FOH"; $resn_exch{"H152 FOH"}="2H15 FOH"; $resn_exch{" H21 FOK"}="1H2  FOK";
$resn_exch{" H22 FOK"}="2H2  FOK"; $resn_exch{" H31 FOK"}="1H3  FOK"; $resn_exch{" H32 FOK"}="2H3  FOK";
$resn_exch{"H121 FOK"}="1H12 FOK"; $resn_exch{"H122 FOK"}="2H12 FOK"; $resn_exch{"H151 FOK"}="1H15 FOK";
$resn_exch{"H152 FOK"}="2H15 FOK"; $resn_exch{"H161 FOK"}="1H16 FOK"; $resn_exch{"H162 FOK"}="2H16 FOK";
$resn_exch{"H163 FOK"}="3H16 FOK"; $resn_exch{"H171 FOK"}="1H17 FOK"; $resn_exch{"H172 FOK"}="2H17 FOK";
$resn_exch{"H173 FOK"}="3H17 FOK"; $resn_exch{"H181 FOK"}="1H18 FOK"; $resn_exch{"H182 FOK"}="2H18 FOK";
$resn_exch{"H183 FOK"}="3H18 FOK"; $resn_exch{"H191 FOK"}="1H19 FOK"; $resn_exch{"H192 FOK"}="2H19 FOK";
$resn_exch{"H193 FOK"}="3H19 FOK"; $resn_exch{"H201 FOK"}="1H20 FOK"; $resn_exch{"H202 FOK"}="2H20 FOK";
$resn_exch{"H203 FOK"}="3H20 FOK"; $resn_exch{"H221 FOK"}="1H22 FOK"; $resn_exch{"H222 FOK"}="2H22 FOK";
$resn_exch{"H223 FOK"}="3H22 FOK"; $resn_exch{"HN21 FOL"}="1HN2 FOL"; $resn_exch{"HN22 FOL"}="2HN2 FOL";
$resn_exch{" H91 FOL"}="1H9  FOL"; $resn_exch{" H92 FOL"}="2H9  FOL"; $resn_exch{" HB1 FOL"}="1HB  FOL";
$resn_exch{" HB2 FOL"}="2HB  FOL"; $resn_exch{" HG1 FOL"}="1HG  FOL"; $resn_exch{" HG2 FOL"}="2HG  FOL";
$resn_exch{"HOE2 FOL"}="2HOE FOL"; $resn_exch{" H21 FOM"}="1H2  FOM"; $resn_exch{" H22 FOM"}="2H2  FOM";
$resn_exch{" H31 FOM"}="1H3  FOM"; $resn_exch{" H32 FOM"}="2H3  FOM"; $resn_exch{" H41 FOM"}="1H4  FOM";
$resn_exch{" H42 FOM"}="2H4  FOM"; $resn_exch{"HN21 FON"}="1HN2 FON"; $resn_exch{"HN22 FON"}="2HN2 FON";
$resn_exch{" H61 FON"}="1H6  FON"; $resn_exch{"HC72 FON"}="2HC7 FON"; $resn_exch{" H91 FON"}="1H9  FON";
$resn_exch{" H92 FON"}="2H9  FON"; $resn_exch{" HB1 FON"}="1HB  FON"; $resn_exch{" HB2 FON"}="2HB  FON";
$resn_exch{" HG1 FON"}="1HG  FON"; $resn_exch{" HG2 FON"}="2HG  FON"; $resn_exch{"HOE2 FON"}="2HOE FON";
$resn_exch{"H2A1 FOO"}="1H2A FOO"; $resn_exch{"H2A2 FOO"}="2H2A FOO"; $resn_exch{"H2A3 FOO"}="3H2A FOO";
$resn_exch{"H5A1 FOO"}="1H5A FOO"; $resn_exch{"H5A2 FOO"}="2H5A FOO"; $resn_exch{" H1  FOR"}="1H   FOR";
$resn_exch{" H2  FOR"}="2H   FOR"; $resn_exch{" H1  FOS"}="1H   FOS"; $resn_exch{" H2A FOS"}="2H   FOS";
$resn_exch{" HM1 FOS"}="1HM  FOS"; $resn_exch{" HM2 FOS"}="2HM  FOS"; $resn_exch{"HOP2 FOS"}="2HOP FOS";
$resn_exch{"HOP3 FOS"}="3HOP FOS"; $resn_exch{"HOP2 FOX"}="2HOP FOX"; $resn_exch{"H5'1 FOX"}="1H5* FOX";
$resn_exch{"H5'2 FOX"}="2H5* FOX"; $resn_exch{" H4' FOX"}=" H4* FOX"; $resn_exch{"H6'1 FOX"}="1H6* FOX";
$resn_exch{"H6'2 FOX"}="2H6* FOX"; $resn_exch{" H3' FOX"}=" H3* FOX"; $resn_exch{"H2'1 FOX"}="1H2* FOX";
$resn_exch{"H2'2 FOX"}="2H2* FOX"; $resn_exch{" H1' FOX"}=" H1* FOX"; $resn_exch{" H21 FOX"}="1H2  FOX";
$resn_exch{" H22 FOX"}="2H2  FOX"; $resn_exch{"HOP3 FOX"}="3HOP FOX"; $resn_exch{"H7C1 FP1"}="1H7C FP1";
$resn_exch{"H7C2 FP1"}="2H7C FP1"; $resn_exch{"H7C3 FP1"}="3H7C FP1"; $resn_exch{" H31 FPA"}="1H3  FPA";
$resn_exch{" H32 FPA"}="2H3  FPA"; $resn_exch{" H41 FPA"}="1H4  FPA"; $resn_exch{" H42 FPA"}="2H4  FPA";
$resn_exch{"H111 FPB"}="1H11 FPB"; $resn_exch{"H112 FPB"}="2H11 FPB"; $resn_exch{"H181 FPB"}="1H18 FPB";
$resn_exch{"H182 FPB"}="2H18 FPB"; $resn_exch{"H9C1 FPB"}="1H9C FPB"; $resn_exch{"H9C2 FPB"}="2H9C FPB";
$resn_exch{"H211 FPB"}="1H21 FPB"; $resn_exch{"H212 FPB"}="2H21 FPB"; $resn_exch{"H201 FPB"}="1H20 FPB";
$resn_exch{"H202 FPB"}="2H20 FPB"; $resn_exch{"H191 FPB"}="1H19 FPB"; $resn_exch{"H192 FPB"}="2H19 FPB";
$resn_exch{"H151 FPB"}="1H15 FPB"; $resn_exch{"H152 FPB"}="2H15 FPB"; $resn_exch{" H61 FPC"}="1H6  FPC";
$resn_exch{" H62 FPC"}="2H6  FPC"; $resn_exch{" H11 FPC"}="1H1  FPC"; $resn_exch{" H12 FPC"}="2H1  FPC";
$resn_exch{"HOP2 FPC"}="2HOP FPC"; $resn_exch{"HOP3 FPC"}="3HOP FPC"; $resn_exch{" H31 FPD"}="1H3  FPD";
$resn_exch{" H32 FPD"}="2H3  FPD"; $resn_exch{" H33 FPD"}="3H3  FPD"; $resn_exch{" H21 FPD"}="1H2  FPD";
$resn_exch{" H22 FPD"}="2H2  FPD"; $resn_exch{" H71 FPD"}="1H7  FPD"; $resn_exch{" H72 FPD"}="2H7  FPD";
$resn_exch{" H81 FPD"}="1H8  FPD"; $resn_exch{" H82 FPD"}="2H8  FPD"; $resn_exch{"H121 FPD"}="1H12 FPD";
$resn_exch{"H122 FPD"}="2H12 FPD"; $resn_exch{" H31 FPE"}="1H3  FPE"; $resn_exch{" H32 FPE"}="2H3  FPE";
$resn_exch{" H91 FPG"}="1H9  FPG"; $resn_exch{" H92 FPG"}="2H9  FPG"; $resn_exch{" H93 FPG"}="3H9  FPG";
$resn_exch{" H61 FPG"}="1H6  FPG"; $resn_exch{" H62 FPG"}="2H6  FPG"; $resn_exch{" H81 FPG"}="1H8  FPG";
$resn_exch{" H82 FPG"}="2H8  FPG"; $resn_exch{" H83 FPG"}="3H8  FPG"; $resn_exch{" H11 FPG"}="1H1  FPG";
$resn_exch{" H12 FPG"}="2H1  FPG"; $resn_exch{" H13 FPG"}="3H1  FPG"; $resn_exch{"H101 FPG"}="1H10 FPG";
$resn_exch{"H102 FPG"}="2H10 FPG"; $resn_exch{"H103 FPG"}="3H10 FPG"; $resn_exch{" H51 FPG"}="1H5  FPG";
$resn_exch{" H52 FPG"}="2H5  FPG"; $resn_exch{" H41 FPG"}="1H4  FPG"; $resn_exch{" H42 FPG"}="2H4  FPG";
$resn_exch{" H21 FPI"}="1H2  FPI"; $resn_exch{" H22 FPI"}="2H2  FPI"; $resn_exch{" H31 FPI"}="1H3  FPI";
$resn_exch{" H32 FPI"}="2H3  FPI"; $resn_exch{" H41 FPI"}="1H4  FPI"; $resn_exch{" H42 FPI"}="2H4  FPI";
$resn_exch{" H51 FPI"}="1H5  FPI"; $resn_exch{" H52 FPI"}="2H5  FPI"; $resn_exch{" H61 FPI"}="1H6  FPI";
$resn_exch{" H62 FPI"}="2H6  FPI"; $resn_exch{" H11 FPP"}="1H1  FPP"; $resn_exch{"H12A FPP"}="2H1  FPP";
$resn_exch{" H41 FPP"}="1H4  FPP"; $resn_exch{" H42 FPP"}="2H4  FPP"; $resn_exch{" H43 FPP"}="3H4  FPP";
$resn_exch{" H51 FPP"}="1H5  FPP"; $resn_exch{" H52 FPP"}="2H5  FPP"; $resn_exch{" H61 FPP"}="1H6  FPP";
$resn_exch{" H62 FPP"}="2H6  FPP"; $resn_exch{"H101 FPP"}="1H10 FPP"; $resn_exch{"H102 FPP"}="2H10 FPP";
$resn_exch{"H103 FPP"}="3H10 FPP"; $resn_exch{" H91 FPP"}="1H9  FPP"; $resn_exch{" H92 FPP"}="2H9  FPP";
$resn_exch{"H111 FPP"}="1H11 FPP"; $resn_exch{"H112 FPP"}="2H11 FPP"; $resn_exch{"H141 FPP"}="1H14 FPP";
$resn_exch{"H142 FPP"}="2H14 FPP"; $resn_exch{"H143 FPP"}="3H14 FPP"; $resn_exch{"H151 FPP"}="1H15 FPP";
$resn_exch{"H152 FPP"}="2H15 FPP"; $resn_exch{"H153 FPP"}="3H15 FPP"; $resn_exch{"HOA2 FPP"}="2HOA FPP";
$resn_exch{"HOB2 FPP"}="2HOB FPP"; $resn_exch{"HOB3 FPP"}="3HOB FPP"; $resn_exch{" H11 FPR"}="1H1  FPR";
$resn_exch{" H12 FPR"}="2H1  FPR"; $resn_exch{" H13 FPR"}="3H1  FPR"; $resn_exch{" H21 FPR"}="1H2  FPR";
$resn_exch{" H22 FPR"}="2H2  FPR"; $resn_exch{" H31 FPR"}="1H3  FPR"; $resn_exch{" H32 FPR"}="2H3  FPR";
$resn_exch{" H11 FPS"}="1H1  FPS"; $resn_exch{"H12A FPS"}="2H1  FPS"; $resn_exch{" H41 FPS"}="1H4  FPS";
$resn_exch{" H42 FPS"}="2H4  FPS"; $resn_exch{" H43 FPS"}="3H4  FPS"; $resn_exch{" H51 FPS"}="1H5  FPS";
$resn_exch{" H52 FPS"}="2H5  FPS"; $resn_exch{" H61 FPS"}="1H6  FPS"; $resn_exch{" H62 FPS"}="2H6  FPS";
$resn_exch{"H101 FPS"}="1H10 FPS"; $resn_exch{"H102 FPS"}="2H10 FPS"; $resn_exch{" H91 FPS"}="1H9  FPS";
$resn_exch{" H92 FPS"}="2H9  FPS"; $resn_exch{" H93 FPS"}="3H9  FPS"; $resn_exch{"H111 FPS"}="1H11 FPS";
$resn_exch{"H112 FPS"}="2H11 FPS"; $resn_exch{"H141 FPS"}="1H14 FPS"; $resn_exch{"H142 FPS"}="2H14 FPS";
$resn_exch{"H143 FPS"}="3H14 FPS"; $resn_exch{"H151 FPS"}="1H15 FPS"; $resn_exch{"H152 FPS"}="2H15 FPS";
$resn_exch{"H153 FPS"}="3H15 FPS"; $resn_exch{"H111 FPT"}="1H11 FPT"; $resn_exch{"H112 FPT"}="2H11 FPT";
$resn_exch{"H121 FPT"}="1H12 FPT"; $resn_exch{"H122 FPT"}="2H12 FPT"; $resn_exch{" HA1 FPT"}="1HA  FPT";
$resn_exch{" HA2 FPT"}="2HA  FPT"; $resn_exch{" HB1 FPT"}="1HB  FPT"; $resn_exch{" HB2 FPT"}="2HB  FPT";
$resn_exch{" HB3 FPT"}="3HB  FPT"; $resn_exch{"H4A1 FQP"}="1H4A FQP"; $resn_exch{"H4A2 FQP"}="2H4A FQP";
$resn_exch{"H6A1 FQP"}="1H6A FQP"; $resn_exch{"H7A1 FQP"}="1H7A FQP"; $resn_exch{"H7A2 FQP"}="2H7A FQP";
$resn_exch{"HO21 FQP"}="1HO2 FQP"; $resn_exch{"HO51 FQP"}="1HO5 FQP"; $resn_exch{"HO61 FQP"}="1HO6 FQP";
$resn_exch{"H111 FR0"}="1H11 FR0"; $resn_exch{"H112 FR0"}="2H11 FR0"; $resn_exch{"H281 FR0"}="1H28 FR0";
$resn_exch{"H282 FR0"}="2H28 FR0"; $resn_exch{"H291 FR0"}="1H29 FR0"; $resn_exch{"H292 FR0"}="2H29 FR0";
$resn_exch{"H151 FR1"}="1H15 FR1"; $resn_exch{"H152 FR1"}="2H15 FR1"; $resn_exch{"H153 FR1"}="3H15 FR1";
$resn_exch{"H161 FR1"}="1H16 FR1"; $resn_exch{"H162 FR1"}="2H16 FR1"; $resn_exch{"H163 FR1"}="3H16 FR1";
$resn_exch{"H211 FR1"}="1H21 FR1"; $resn_exch{"H212 FR1"}="2H21 FR1"; $resn_exch{"H221 FR1"}="1H22 FR1";
$resn_exch{"H222 FR1"}="2H22 FR1"; $resn_exch{"H231 FR1"}="1H23 FR1"; $resn_exch{"H232 FR1"}="2H23 FR1";
$resn_exch{"H291 FR1"}="1H29 FR1"; $resn_exch{"H292 FR1"}="2H29 FR1"; $resn_exch{"H293 FR1"}="3H29 FR1";
$resn_exch{"H301 FR1"}="1H30 FR1"; $resn_exch{"H302 FR1"}="2H30 FR1"; $resn_exch{"H303 FR1"}="3H30 FR1";
$resn_exch{"H371 FR1"}="1H37 FR1"; $resn_exch{"H372 FR1"}="2H37 FR1"; $resn_exch{"H101 FR2"}="1H10 FR2";
$resn_exch{"H102 FR2"}="2H10 FR2"; $resn_exch{"H141 FR2"}="1H14 FR2"; $resn_exch{"H142 FR2"}="2H14 FR2";
$resn_exch{"H181 FR2"}="1H18 FR2"; $resn_exch{"H182 FR2"}="2H18 FR2"; $resn_exch{"H191 FR2"}="1H19 FR2";
$resn_exch{"H192 FR2"}="2H19 FR2"; $resn_exch{"H101 FR3"}="1H10 FR3"; $resn_exch{"H102 FR3"}="2H10 FR3";
$resn_exch{"H141 FR3"}="1H14 FR3"; $resn_exch{"H142 FR3"}="2H14 FR3"; $resn_exch{"H181 FR3"}="1H18 FR3";
$resn_exch{"H182 FR3"}="2H18 FR3"; $resn_exch{"H191 FR3"}="1H19 FR3"; $resn_exch{"H192 FR3"}="2H19 FR3";
$resn_exch{"H101 FR4"}="1H10 FR4"; $resn_exch{"H102 FR4"}="2H10 FR4"; $resn_exch{"H141 FR4"}="1H14 FR4";
$resn_exch{"H142 FR4"}="2H14 FR4"; $resn_exch{"H181 FR4"}="1H18 FR4"; $resn_exch{"H182 FR4"}="2H18 FR4";
$resn_exch{"H191 FR4"}="1H19 FR4"; $resn_exch{"H192 FR4"}="2H19 FR4"; $resn_exch{" H41 FR4"}="1H4  FR4";
$resn_exch{" H42 FR4"}="2H4  FR4"; $resn_exch{" H61 FR4"}="1H6  FR4"; $resn_exch{" H62 FR4"}="2H6  FR4";
$resn_exch{"H101 FR5"}="1H10 FR5"; $resn_exch{"H102 FR5"}="2H10 FR5"; $resn_exch{"H141 FR5"}="1H14 FR5";
$resn_exch{"H142 FR5"}="2H14 FR5"; $resn_exch{"H15O FR5"}="OH15 FR5"; $resn_exch{"H181 FR5"}="1H18 FR5";
$resn_exch{"H182 FR5"}="2H18 FR5"; $resn_exch{"H191 FR5"}="1H19 FR5"; $resn_exch{"H192 FR5"}="2H19 FR5";
$resn_exch{" H41 FR5"}="1H4  FR5"; $resn_exch{" H42 FR5"}="2H4  FR5"; $resn_exch{" H61 FR5"}="1H6  FR5";
$resn_exch{" H62 FR5"}="2H6  FR5"; $resn_exch{" H71 FR5"}="1H7  FR5"; $resn_exch{" H72 FR5"}="2H7  FR5";
$resn_exch{" H73 FR5"}="3H7  FR5"; $resn_exch{"H101 FR6"}="1H10 FR6"; $resn_exch{"H102 FR6"}="2H10 FR6";
$resn_exch{"H181 FR6"}="1H18 FR6"; $resn_exch{"H182 FR6"}="2H18 FR6"; $resn_exch{"H191 FR6"}="1H19 FR6";
$resn_exch{"H192 FR6"}="2H19 FR6"; $resn_exch{" H91 FR6"}="1H9  FR6"; $resn_exch{" H92 FR6"}="2H9  FR6";
$resn_exch{" H93 FR6"}="3H9  FR6"; $resn_exch{"H101 FR7"}="1H10 FR7"; $resn_exch{"H102 FR7"}="2H10 FR7";
$resn_exch{"H181 FR7"}="1H18 FR7"; $resn_exch{"H182 FR7"}="2H18 FR7"; $resn_exch{" H21 FR7"}="1H2  FR7";
$resn_exch{" H22 FR7"}="2H2  FR7"; $resn_exch{" H91 FR7"}="1H9  FR7"; $resn_exch{" H92 FR7"}="2H9  FR7";
$resn_exch{" H93 FR7"}="3H9  FR7"; $resn_exch{"H101 FR8"}="1H10 FR8"; $resn_exch{"H102 FR8"}="2H10 FR8";
$resn_exch{"H141 FR8"}="1H14 FR8"; $resn_exch{"H142 FR8"}="2H14 FR8"; $resn_exch{"H181 FR8"}="1H18 FR8";
$resn_exch{"H182 FR8"}="2H18 FR8"; $resn_exch{"H191 FR8"}="1H19 FR8"; $resn_exch{"H192 FR8"}="2H19 FR8";
$resn_exch{" H41 FR8"}="1H4  FR8"; $resn_exch{" H42 FR8"}="2H4  FR8"; $resn_exch{" H61 FR8"}="1H6  FR8";
$resn_exch{" H62 FR8"}="2H6  FR8"; $resn_exch{"H161 FR8"}="1H16 FR8"; $resn_exch{"H162 FR8"}="2H16 FR8";
$resn_exch{"H171 FR8"}="1H17 FR8"; $resn_exch{"H172 FR8"}="2H17 FR8"; $resn_exch{"H101 FR9"}="1H10 FR9";
$resn_exch{"H102 FR9"}="2H10 FR9"; $resn_exch{"H141 FR9"}="1H14 FR9"; $resn_exch{"H142 FR9"}="2H14 FR9";
$resn_exch{"H181 FR9"}="1H18 FR9"; $resn_exch{"H182 FR9"}="2H18 FR9"; $resn_exch{"H191 FR9"}="1H19 FR9";
$resn_exch{"H192 FR9"}="2H19 FR9"; $resn_exch{" H61 FR9"}="1H6  FR9"; $resn_exch{" H62 FR9"}="2H6  FR9";
$resn_exch{"H221 FRA"}="1H22 FRA"; $resn_exch{"H222 FRA"}="2H22 FRA"; $resn_exch{"H223 FRA"}="3H22 FRA";
$resn_exch{"H171 FRA"}="1H17 FRA"; $resn_exch{"H172 FRA"}="2H17 FRA"; $resn_exch{"H161 FRA"}="1H16 FRA";
$resn_exch{"H162 FRA"}="2H16 FRA"; $resn_exch{"H141 FRA"}="1H14 FRA"; $resn_exch{"H142 FRA"}="2H14 FRA";
$resn_exch{"H131 FRA"}="1H13 FRA"; $resn_exch{"H132 FRA"}="2H13 FRA"; $resn_exch{"H191 FRA"}="1H19 FRA";
$resn_exch{"H192 FRA"}="2H19 FRA"; $resn_exch{"HN31 FRB"}="1HN3 FRB"; $resn_exch{"HN32 FRB"}="2HN3 FRB";
$resn_exch{" H71 FRB"}="1H7  FRB"; $resn_exch{" H72 FRB"}="2H7  FRB"; $resn_exch{" H81 FRB"}="1H8  FRB";
$resn_exch{" H82 FRB"}="2H8  FRB"; $resn_exch{" H91 FRB"}="1H9  FRB"; $resn_exch{" H92 FRB"}="2H9  FRB";
$resn_exch{"H101 FRB"}="1H10 FRB"; $resn_exch{"H102 FRB"}="2H10 FRB"; $resn_exch{"H111 FRB"}="1H11 FRB";
$resn_exch{"H112 FRB"}="2H11 FRB"; $resn_exch{"H151 FRB"}="1H15 FRB"; $resn_exch{"H152 FRB"}="2H15 FRB";
$resn_exch{"H191 FRB"}="1H19 FRB"; $resn_exch{"H192 FRB"}="2H19 FRB"; $resn_exch{"H201 FRB"}="1H20 FRB";
$resn_exch{"H202 FRB"}="2H20 FRB"; $resn_exch{"H231 FRB"}="1H23 FRB"; $resn_exch{"H232 FRB"}="2H23 FRB";
$resn_exch{"H221 FRB"}="1H22 FRB"; $resn_exch{"H222 FRB"}="2H22 FRB"; $resn_exch{"H101 FRC"}="1H10 FRC";
$resn_exch{"H102 FRC"}="2H10 FRC"; $resn_exch{"H181 FRC"}="1H18 FRC"; $resn_exch{"H182 FRC"}="2H18 FRC";
$resn_exch{" H21 FRC"}="1H2  FRC"; $resn_exch{" H22 FRC"}="2H2  FRC"; $resn_exch{" H91 FRC"}="1H9  FRC";
$resn_exch{" H92 FRC"}="2H9  FRC"; $resn_exch{" H93 FRC"}="3H9  FRC"; $resn_exch{" H2  FRD"}=" HN2 FRD";
$resn_exch{" HC1 FRD"}="1H   FRD"; $resn_exch{" HC2 FRD"}="2H   FRD"; $resn_exch{" HC3 FRD"}="3H   FRD";
$resn_exch{" HB2 FRD"}="1HB  FRD"; $resn_exch{" HB3 FRD"}="2HB  FRD"; $resn_exch{"HN61 FRE"}="1HN6 FRE";
$resn_exch{"HN62 FRE"}="2HN6 FRE"; $resn_exch{" H1' FRE"}=" H1* FRE"; $resn_exch{" H2' FRE"}=" H2* FRE";
$resn_exch{" H3' FRE"}=" H3* FRE"; $resn_exch{" H4' FRE"}=" H4* FRE"; $resn_exch{"H5'1 FRE"}="1H5* FRE";
$resn_exch{"H5'2 FRE"}="2H5* FRE"; $resn_exch{"H121 FRE"}="1H12 FRE"; $resn_exch{"H122 FRE"}="2H12 FRE";
$resn_exch{" H11 FRE"}="1H1  FRE"; $resn_exch{" H12 FRE"}="2H1  FRE"; $resn_exch{" H13 FRE"}="3H1  FRE";
$resn_exch{"H141 FRE"}="1H14 FRE"; $resn_exch{"H142 FRE"}="2H14 FRE"; $resn_exch{"H143 FRE"}="3H14 FRE";
$resn_exch{" H71 FRE"}="1H7  FRE"; $resn_exch{" H72 FRE"}="2H7  FRE"; $resn_exch{" H61 FRE"}="1H6  FRE";
$resn_exch{" H62 FRE"}="2H6  FRE"; $resn_exch{" H31 FRE"}="1H3  FRE"; $resn_exch{" H32 FRE"}="2H3  FRE";
$resn_exch{" H21 FRE"}="1H2  FRE"; $resn_exch{" H22 FRE"}="2H2  FRE"; $resn_exch{"H111 FRE"}="H11  FRE";
$resn_exch{"H112 FRE"}="H12  FRE"; $resn_exch{"H131 FRE"}="1H13 FRE"; $resn_exch{"H132 FRE"}="2H13 FRE";
$resn_exch{"H133 FRE"}="3H13 FRE"; $resn_exch{" HN1 FRF"}="1HN  FRF"; $resn_exch{" HN2 FRF"}="2HN  FRF";
$resn_exch{"HTC1 FRF"}="1HTC FRF"; $resn_exch{"HTC2 FRF"}="2HTC FRF"; $resn_exch{"HBC1 FRF"}="1HBC FRF";
$resn_exch{"HBC2 FRF"}="2HBC FRF"; $resn_exch{"HB11 FRF"}="1HB1 FRF"; $resn_exch{"HB12 FRF"}="2HB1 FRF";
$resn_exch{"H321 FRG"}="1H32 FRG"; $resn_exch{"H322 FRG"}="2H32 FRG"; $resn_exch{"H301 FRG"}="1H30 FRG";
$resn_exch{"H302 FRG"}="2H30 FRG"; $resn_exch{"H281 FRG"}="1H28 FRG"; $resn_exch{"H282 FRG"}="2H28 FRG";
$resn_exch{"H271 FRG"}="1H27 FRG"; $resn_exch{"H272 FRG"}="2H27 FRG"; $resn_exch{"H261 FRG"}="1H26 FRG";
$resn_exch{"H262 FRG"}="2H26 FRG"; $resn_exch{"H231 FRG"}="1H23 FRG"; $resn_exch{"H232 FRG"}="2H23 FRG";
$resn_exch{"H201 FRG"}="1H20 FRG"; $resn_exch{"H202 FRG"}="2H20 FRG"; $resn_exch{"H203 FRG"}="3H20 FRG";
$resn_exch{"H151 FRG"}="1H15 FRG"; $resn_exch{"H152 FRG"}="2H15 FRG"; $resn_exch{"H321 FRH"}="1H32 FRH";
$resn_exch{"H322 FRH"}="2H32 FRH"; $resn_exch{"H381 FRH"}="1H38 FRH"; $resn_exch{"H382 FRH"}="2H38 FRH";
$resn_exch{"H383 FRH"}="3H38 FRH"; $resn_exch{"H371 FRH"}="1H37 FRH"; $resn_exch{"H372 FRH"}="2H37 FRH";
$resn_exch{"H373 FRH"}="3H37 FRH"; $resn_exch{"HN41 FRH"}="1HN4 FRH"; $resn_exch{"HN42 FRH"}="2HN4 FRH";
$resn_exch{"H281 FRH"}="1H28 FRH"; $resn_exch{"H282 FRH"}="2H28 FRH"; $resn_exch{"H221 FRH"}="1H22 FRH";
$resn_exch{"H222 FRH"}="2H22 FRH"; $resn_exch{"H151 FRH"}="1H15 FRH"; $resn_exch{"H152 FRH"}="2H15 FRH";
$resn_exch{"H271 FRH"}="1H27 FRH"; $resn_exch{"H272 FRH"}="2H27 FRH"; $resn_exch{"H291 FRH"}="1H29 FRH";
$resn_exch{"H292 FRH"}="2H29 FRH"; $resn_exch{"H341 FRH"}="1H34 FRH"; $resn_exch{"H342 FRH"}="2H34 FRH";
$resn_exch{"H343 FRH"}="3H34 FRH"; $resn_exch{"H351 FRH"}="1H35 FRH"; $resn_exch{"H352 FRH"}="2H35 FRH";
$resn_exch{"H291 FRI"}="1H29 FRI"; $resn_exch{"H292 FRI"}="2H29 FRI"; $resn_exch{"H411 FRI"}="1H41 FRI";
$resn_exch{"H412 FRI"}="2H41 FRI"; $resn_exch{"H371 FRI"}="1H37 FRI"; $resn_exch{"H372 FRI"}="2H37 FRI";
$resn_exch{"H373 FRI"}="3H37 FRI"; $resn_exch{"H361 FRI"}="1H36 FRI"; $resn_exch{"H362 FRI"}="2H36 FRI";
$resn_exch{"H363 FRI"}="3H36 FRI"; $resn_exch{"HN41 FRI"}="1HN4 FRI"; $resn_exch{"HN42 FRI"}="2HN4 FRI";
$resn_exch{"H311 FRI"}="1H31 FRI"; $resn_exch{"H312 FRI"}="2H31 FRI"; $resn_exch{"H391 FRI"}="1H39 FRI";
$resn_exch{"H392 FRI"}="2H39 FRI"; $resn_exch{"H211 FRI"}="1H21 FRI"; $resn_exch{"H212 FRI"}="2H21 FRI";
$resn_exch{"H431 FRI"}="1H43 FRI"; $resn_exch{"H432 FRI"}="2H43 FRI"; $resn_exch{"H381 FRI"}="1H38 FRI";
$resn_exch{"H382 FRI"}="2H38 FRI"; $resn_exch{"H383 FRI"}="3H38 FRI"; $resn_exch{"H401 FRI"}="1H40 FRI";
$resn_exch{"H402 FRI"}="2H40 FRI"; $resn_exch{"H421 FRI"}="1H42 FRI"; $resn_exch{"H422 FRI"}="2H42 FRI";
$resn_exch{"H423 FRI"}="3H42 FRI"; $resn_exch{"H131 FRJ"}="1H13 FRJ"; $resn_exch{"H132 FRJ"}="2H13 FRJ";
$resn_exch{"H141 FRJ"}="1H14 FRJ"; $resn_exch{"H142 FRJ"}="2H14 FRJ"; $resn_exch{"H143 FRJ"}="3H14 FRJ";
$resn_exch{" H91 FRK"}="1H9  FRK"; $resn_exch{" H92 FRK"}="2H9  FRK"; $resn_exch{"H111 FRK"}="1H11 FRK";
$resn_exch{"H112 FRK"}="2H11 FRK"; $resn_exch{"H121 FRK"}="1H12 FRK"; $resn_exch{"H122 FRK"}="2H12 FRK";
$resn_exch{"H151 FRK"}="1H15 FRK"; $resn_exch{"H152 FRK"}="2H15 FRK"; $resn_exch{"H181 FRK"}="1H18 FRK";
$resn_exch{"H182 FRK"}="2H18 FRK"; $resn_exch{"H183 FRK"}="3H18 FRK"; $resn_exch{"H171 FRL"}="1H17 FRL";
$resn_exch{"H172 FRL"}="2H17 FRL"; $resn_exch{"H181 FRL"}="1H18 FRL"; $resn_exch{"H182 FRL"}="2H18 FRL";
$resn_exch{"H231 FRL"}="1H23 FRL"; $resn_exch{"H232 FRL"}="2H23 FRL"; $resn_exch{"H233 FRL"}="3H23 FRL";
$resn_exch{"H301 FRL"}="1H30 FRL"; $resn_exch{"H302 FRL"}="2H30 FRL"; $resn_exch{"H171 FRM"}="1H17 FRM";
$resn_exch{"H172 FRM"}="2H17 FRM"; $resn_exch{"H201 FRM"}="1H20 FRM"; $resn_exch{"H202 FRM"}="2H20 FRM";
$resn_exch{"H231 FRM"}="1H23 FRM"; $resn_exch{"H232 FRM"}="2H23 FRM"; $resn_exch{"H291 FRM"}="1H29 FRM";
$resn_exch{"H292 FRM"}="2H29 FRM"; $resn_exch{"H321 FRM"}="1H32 FRM"; $resn_exch{"H322 FRM"}="2H32 FRM";
$resn_exch{"H331 FRM"}="1H33 FRM"; $resn_exch{"H332 FRM"}="2H33 FRM"; $resn_exch{" H71 FRM"}="1H7  FRM";
$resn_exch{" H72 FRM"}="2H7  FRM"; $resn_exch{" H73 FRM"}="3H7  FRM"; $resn_exch{"HN1A FRN"}="1HN  FRN";
$resn_exch{"HN2A FRN"}="2HN  FRN"; $resn_exch{"HN3A FRN"}="3HN  FRN"; $resn_exch{"HG21 FRN"}="1HG2 FRN";
$resn_exch{"HG22 FRN"}="2HG2 FRN"; $resn_exch{"HG23 FRN"}="3HG2 FRN"; $resn_exch{"H311 FRN"}="1H31 FRN";
$resn_exch{"H312 FRN"}="2H31 FRN"; $resn_exch{"H313 FRN"}="3H31 FRN"; $resn_exch{"H221 FRN"}="1H22 FRN";
$resn_exch{"H222 FRN"}="2H22 FRN"; $resn_exch{"HD11 FRN"}="1HD1 FRN"; $resn_exch{"HD12 FRN"}="2HD1 FRN";
$resn_exch{"HD13 FRN"}="3HD1 FRN"; $resn_exch{"HD21 FRN"}="1HD2 FRN"; $resn_exch{"HD22 FRN"}="2HD2 FRN";
$resn_exch{"HD23 FRN"}="3HD2 FRN"; $resn_exch{"H231 FRN"}="1H23 FRN"; $resn_exch{"H232 FRN"}="2H23 FRN";
$resn_exch{"H321 FRN"}="1H32 FRN"; $resn_exch{"H322 FRN"}="2H32 FRN"; $resn_exch{" HD1 FRN"}="1HD  FRN";
$resn_exch{" HD2 FRN"}="2HD  FRN"; $resn_exch{"HE1A FRN"}="1HE  FRN"; $resn_exch{"HE2A FRN"}="2HE  FRN";
$resn_exch{"H241 FRN"}="1H24 FRN"; $resn_exch{"H242 FRN"}="2H24 FRN"; $resn_exch{"H251 FRN"}="1H25 FRN";
$resn_exch{"H252 FRN"}="2H25 FRN"; $resn_exch{"H341 FRN"}="1H34 FRN"; $resn_exch{"H342 FRN"}="2H34 FRN";
$resn_exch{"H421 FRN"}="1H42 FRN"; $resn_exch{"H422 FRN"}="2H42 FRN"; $resn_exch{"H261 FRN"}="1H26 FRN";
$resn_exch{"H262 FRN"}="2H26 FRN"; $resn_exch{"H271 FRN"}="1H27 FRN"; $resn_exch{"H272 FRN"}="2H27 FRN";
$resn_exch{"H281 FRN"}="1H28 FRN"; $resn_exch{"H282 FRN"}="2H28 FRN"; $resn_exch{"H291 FRN"}="1H29 FRN";
$resn_exch{"H292 FRN"}="2H29 FRN"; $resn_exch{"H441 FRN"}="1H44 FRN"; $resn_exch{"H442 FRN"}="2H44 FRN";
$resn_exch{" H81 FRO"}="1H8  FRO"; $resn_exch{" H82 FRO"}="2H8  FRO"; $resn_exch{" H91 FRO"}="1H9  FRO";
$resn_exch{" H92 FRO"}="2H9  FRO"; $resn_exch{"H101 FRO"}="1H10 FRO"; $resn_exch{"H102 FRO"}="2H10 FRO";
$resn_exch{"H181 FRO"}="1H18 FRO"; $resn_exch{"H182 FRO"}="2H18 FRO"; $resn_exch{"H183 FRO"}="3H18 FRO";
$resn_exch{"H171 FRP"}="1H17 FRP"; $resn_exch{"H172 FRP"}="2H17 FRP"; $resn_exch{"H131 FRP"}="1H13 FRP";
$resn_exch{"H132 FRP"}="2H13 FRP"; $resn_exch{"H133 FRP"}="3H13 FRP"; $resn_exch{"H141 FRP"}="1H14 FRP";
$resn_exch{"H142 FRP"}="2H14 FRP"; $resn_exch{"H143 FRP"}="3H14 FRP"; $resn_exch{"H111 FRP"}="1H11 FRP";
$resn_exch{"H112 FRP"}="2H11 FRP"; $resn_exch{"H101 FRP"}="1H10 FRP"; $resn_exch{"H102 FRP"}="2H10 FRP";
$resn_exch{" H91 FRP"}="1H9  FRP"; $resn_exch{" H92 FRP"}="2H9  FRP"; $resn_exch{" H81 FRP"}="1H8  FRP";
$resn_exch{" H82 FRP"}="2H8  FRP"; $resn_exch{"H211 FRP"}="1H21 FRP"; $resn_exch{"H212 FRP"}="2H21 FRP";
$resn_exch{"H213 FRP"}="3H21 FRP"; $resn_exch{"H181 FRP"}="1H18 FRP"; $resn_exch{"H182 FRP"}="2H18 FRP";
$resn_exch{"HN41 FRP"}="1HN4 FRP"; $resn_exch{"HN42 FRP"}="2HN4 FRP"; $resn_exch{"H161 FRQ"}="1H16 FRQ";
$resn_exch{"H162 FRQ"}="2H16 FRQ"; $resn_exch{"H171 FRQ"}="1H17 FRQ"; $resn_exch{"H172 FRQ"}="2H17 FRQ";
$resn_exch{"H201 FRQ"}="1H20 FRQ"; $resn_exch{"H202 FRQ"}="2H20 FRQ"; $resn_exch{"H231 FRQ"}="1H23 FRQ";
$resn_exch{"H232 FRQ"}="2H23 FRQ"; $resn_exch{"H291 FRQ"}="1H29 FRQ"; $resn_exch{"H292 FRQ"}="2H29 FRQ";
$resn_exch{"H301 FRQ"}="1H30 FRQ"; $resn_exch{"H302 FRQ"}="2H30 FRQ"; $resn_exch{"H331 FRQ"}="1H33 FRQ";
$resn_exch{"H332 FRQ"}="2H33 FRQ"; $resn_exch{"H591 FRR"}="1H59 FRR"; $resn_exch{"H592 FRR"}="2H59 FRR";
$resn_exch{"H593 FRR"}="3H59 FRR"; $resn_exch{"H341 FRR"}="1H34 FRR"; $resn_exch{"H342 FRR"}="2H34 FRR";
$resn_exch{"H181 FRR"}="1H18 FRR"; $resn_exch{"H182 FRR"}="2H18 FRR"; $resn_exch{"H621 FRR"}="1H62 FRR";
$resn_exch{"H622 FRR"}="2H62 FRR"; $resn_exch{"H623 FRR"}="3H62 FRR"; $resn_exch{" H11 FRU"}="1H1  FRU";
$resn_exch{" H12 FRU"}="2H1  FRU"; $resn_exch{" H61 FRU"}="1H6  FRU"; $resn_exch{" H62 FRU"}="2H6  FRU";
$resn_exch{"H241 FRX"}="1H24 FRX"; $resn_exch{"H242 FRX"}="2H24 FRX"; $resn_exch{"H251 FRX"}="1H25 FRX";
$resn_exch{"H252 FRX"}="2H25 FRX"; $resn_exch{"H211 FRX"}="1H21 FRX"; $resn_exch{"H212 FRX"}="2H21 FRX";
$resn_exch{"H151 FRX"}="1H15 FRX"; $resn_exch{"H152 FRX"}="2H15 FRX"; $resn_exch{"H141 FRY"}="1H14 FRY";
$resn_exch{"H142 FRY"}="2H14 FRY"; $resn_exch{"H201 FRY"}="1H20 FRY"; $resn_exch{"H202 FRY"}="2H20 FRY";
$resn_exch{"H231 FRY"}="1H23 FRY"; $resn_exch{"H232 FRY"}="2H23 FRY"; $resn_exch{"H301 FRZ"}="1H30 FRZ";
$resn_exch{"H302 FRZ"}="2H30 FRZ"; $resn_exch{"H01C FSB"}="CH01 FSB"; $resn_exch{"H02C FSB"}="CH02 FSB";
$resn_exch{"H04C FSB"}="CH04 FSB"; $resn_exch{"H05C FSB"}="CH05 FSB"; $resn_exch{"H09N FSB"}="NH09 FSB";
$resn_exch{"H101 FSB"}="1H10 FSB"; $resn_exch{"H102 FSB"}="2H10 FSB"; $resn_exch{"HNP1 FSB"}="1HNP FSB";
$resn_exch{"HNP2 FSB"}="2HNP FSB"; $resn_exch{"H16C FSB"}="CH16 FSB"; $resn_exch{"H17C FSB"}="CH17 FSB";
$resn_exch{"H18C FSB"}="CH18 FSB"; $resn_exch{"H19C FSB"}="CH19 FSB"; $resn_exch{"H181 FSC"}="1H18 FSC";
$resn_exch{"H182 FSC"}="2H18 FSC"; $resn_exch{"H183 FSC"}="3H18 FSC"; $resn_exch{"H171 FSC"}="1H17 FSC";
$resn_exch{"H172 FSC"}="2H17 FSC"; $resn_exch{"H361 FSC"}="1H36 FSC"; $resn_exch{"H362 FSC"}="2H36 FSC";
$resn_exch{"H363 FSC"}="3H36 FSC"; $resn_exch{"H101 FSC"}="1H10 FSC"; $resn_exch{"H102 FSC"}="2H10 FSC";
$resn_exch{"H7C1 FSC"}="1H7C FSC"; $resn_exch{"H7C2 FSC"}="2H7C FSC"; $resn_exch{"H7C3 FSC"}="3H7C FSC";
$resn_exch{"H271 FSC"}="1H27 FSC"; $resn_exch{"H272 FSC"}="2H27 FSC"; $resn_exch{"H381 FSC"}="1H38 FSC";
$resn_exch{"H382 FSC"}="2H38 FSC"; $resn_exch{"H383 FSC"}="3H38 FSC"; $resn_exch{"H261 FSC"}="1H26 FSC";
$resn_exch{"H262 FSC"}="2H26 FSC"; $resn_exch{"H251 FSC"}="1H25 FSC"; $resn_exch{"H252 FSC"}="2H25 FSC";
$resn_exch{"H231 FSC"}="1H23 FSC"; $resn_exch{"H232 FSC"}="2H23 FSC"; $resn_exch{"H233 FSC"}="3H23 FSC";
$resn_exch{"H421 FSC"}="1H42 FSC"; $resn_exch{"H422 FSC"}="2H42 FSC"; $resn_exch{"H423 FSC"}="3H42 FSC";
$resn_exch{"H351 FSC"}="1H35 FSC"; $resn_exch{"H352 FSC"}="2H35 FSC"; $resn_exch{"H471 FSC"}="1H47 FSC";
$resn_exch{"H472 FSC"}="2H47 FSC"; $resn_exch{"H473 FSC"}="3H47 FSC"; $resn_exch{"H461 FSC"}="1H46 FSC";
$resn_exch{"H462 FSC"}="2H46 FSC"; $resn_exch{"H463 FSC"}="3H46 FSC"; $resn_exch{"H481 FSC"}="1H48 FSC";
$resn_exch{"H482 FSC"}="2H48 FSC"; $resn_exch{"H111 FSI"}="1H11 FSI"; $resn_exch{"H112 FSI"}="2H11 FSI";
$resn_exch{"H113 FSI"}="3H11 FSI"; $resn_exch{" H91 FSI"}="1H9  FSI"; $resn_exch{" H92 FSI"}="2H9  FSI";
$resn_exch{"HAA1 FSM"}="1HAA FSM"; $resn_exch{"HAA2 FSM"}="2HAA FSM"; $resn_exch{"HA'1 FSM"}="1HA' FSM";
$resn_exch{"HA'2 FSM"}="2HA' FSM"; $resn_exch{"HA'3 FSM"}="3HA' FSM"; $resn_exch{"HB'1 FSM"}="1HB' FSM";
$resn_exch{"HB'2 FSM"}="2HB' FSM"; $resn_exch{"HBA1 FSM"}="1HBA FSM"; $resn_exch{"HBA2 FSM"}="2HBA FSM";
$resn_exch{"HBA3 FSM"}="3HBA FSM"; $resn_exch{"H231 FSN"}="1H23 FSN"; $resn_exch{"H232 FSN"}="2H23 FSN";
$resn_exch{"H121 FSN"}="1H12 FSN"; $resn_exch{"H122 FSN"}="2H12 FSN"; $resn_exch{"H131 FSN"}="1H13 FSN";
$resn_exch{"H132 FSN"}="2H13 FSN"; $resn_exch{"H141 FSN"}="1H14 FSN"; $resn_exch{"H142 FSN"}="2H14 FSN";
$resn_exch{"HN11 FSN"}="1HN1 FSN"; $resn_exch{"HN12 FSN"}="2HN1 FSN"; $resn_exch{"HN21 FSN"}="1HN2 FSN";
$resn_exch{"HN22 FSN"}="2HN2 FSN"; $resn_exch{" H11 FSP"}="1H1  FSP"; $resn_exch{" H12 FSP"}="2H1  FSP";
$resn_exch{" H13 FSP"}="3H1  FSP"; $resn_exch{" H21 FSP"}="1H2  FSP"; $resn_exch{" H22 FSP"}="2H2  FSP";
$resn_exch{" H31 FSP"}="1H3  FSP"; $resn_exch{"H32A FSP"}="2H3  FSP"; $resn_exch{" H41 FSP"}="1H4  FSP";
$resn_exch{" H42 FSP"}="2H4  FSP"; $resn_exch{"H101 FSP"}="1H10 FSP"; $resn_exch{"H102 FSP"}="2H10 FSP";
$resn_exch{"H121 FSP"}="1H12 FSP"; $resn_exch{"H122 FSP"}="2H12 FSP"; $resn_exch{"H201 FSP"}="1H20 FSP";
$resn_exch{"H202 FSP"}="2H20 FSP"; $resn_exch{"H211 FSP"}="1H21 FSP"; $resn_exch{"H212 FSP"}="2H21 FSP";
$resn_exch{"H221 FSP"}="1H22 FSP"; $resn_exch{"H222 FSP"}="2H22 FSP"; $resn_exch{" H   FT6"}="1HN  FT6";
$resn_exch{" H2  FT6"}="2HN  FT6"; $resn_exch{" HB2 FT6"}="1HB  FT6"; $resn_exch{" HB3 FT6"}="2HB  FT6";
$resn_exch{" HE1 FT6"}="HNE1 FT6"; $resn_exch{"H341 FTB"}="1H34 FTB"; $resn_exch{"H342 FTB"}="2H34 FTB";
$resn_exch{"H343 FTB"}="3H34 FTB"; $resn_exch{"H241 FTB"}="1H24 FTB"; $resn_exch{"H242 FTB"}="2H24 FTB";
$resn_exch{"H121 FTB"}="1H12 FTB"; $resn_exch{"H122 FTB"}="2H12 FTB"; $resn_exch{"H131 FTB"}="1H13 FTB";
$resn_exch{"H132 FTB"}="2H13 FTB"; $resn_exch{"H141 FTB"}="1H14 FTB"; $resn_exch{"H142 FTB"}="2H14 FTB";
$resn_exch{"H151 FTB"}="1H15 FTB"; $resn_exch{"H152 FTB"}="2H15 FTB"; $resn_exch{"H153 FTB"}="3H15 FTB";
$resn_exch{"H161 FTB"}="1H16 FTB"; $resn_exch{"H162 FTB"}="2H16 FTB"; $resn_exch{"H111 FTC"}="1H11 FTC";
$resn_exch{"H112 FTC"}="2H11 FTC"; $resn_exch{"H121 FTC"}="1H12 FTC"; $resn_exch{"H122 FTC"}="2H12 FTC";
$resn_exch{" HA1 FTC"}="1HA  FTC"; $resn_exch{" HA2 FTC"}="2HA  FTC"; $resn_exch{" HB1 FTC"}="1HB  FTC";
$resn_exch{" HB2 FTC"}="2HB  FTC"; $resn_exch{" HB3 FTC"}="3HB  FTC"; $resn_exch{"HC11 FTH"}="1HC1 FTH";
$resn_exch{"HC1A FTH"}="2HC1 FTH"; $resn_exch{"HC1B FTH"}="3HC1 FTH"; $resn_exch{"HC71 FTH"}="1HC7 FTH";
$resn_exch{"HC72 FTH"}="2HC7 FTH"; $resn_exch{"H101 FTH"}="1H10 FTH"; $resn_exch{"H102 FTH"}="2H10 FTH";
$resn_exch{"H221 FTH"}="1H22 FTH"; $resn_exch{"H222 FTH"}="2H22 FTH"; $resn_exch{"H231 FTH"}="1H23 FTH";
$resn_exch{"H232 FTH"}="2H23 FTH"; $resn_exch{"H261 FTH"}="1H26 FTH"; $resn_exch{"H262 FTH"}="2H26 FTH";
$resn_exch{"HC1A FTI"}="1HC1 FTI"; $resn_exch{"HC12 FTI"}="2HC1 FTI"; $resn_exch{"HC2A FTI"}="1HC2 FTI";
$resn_exch{"HC22 FTI"}="2HC2 FTI"; $resn_exch{"HC31 FTI"}="1HC3 FTI"; $resn_exch{"HC32 FTI"}="2HC3 FTI";
$resn_exch{"H161 FTI"}="1H16 FTI"; $resn_exch{"H162 FTI"}="2H16 FTI"; $resn_exch{"H201 FTI"}="1H20 FTI";
$resn_exch{"H202 FTI"}="2H20 FTI"; $resn_exch{"HC81 FTL"}="1HC8 FTL"; $resn_exch{"HC82 FTL"}="2HC8 FTL";
$resn_exch{"H151 FTL"}="1H15 FTL"; $resn_exch{"H152 FTL"}="2H15 FTL"; $resn_exch{"H153 FTL"}="3H15 FTL";
$resn_exch{"H181 FTL"}="1H18 FTL"; $resn_exch{"H182 FTL"}="2H18 FTL"; $resn_exch{"H4A1 FTP"}="1H4A FTP";
$resn_exch{"H4A2 FTP"}="2H4A FTP"; $resn_exch{"H6A1 FTP"}="1H6A FTP"; $resn_exch{"H7A1 FTP"}="1H7A FTP";
$resn_exch{"H7A2 FTP"}="2H7A FTP"; $resn_exch{" H21 FTP"}="1H2  FTP"; $resn_exch{"HM41 FTP"}="1HM4 FTP";
$resn_exch{"HM42 FTP"}="2HM4 FTP"; $resn_exch{"HM43 FTP"}="3HM4 FTP"; $resn_exch{" H61 FTP"}="1H6  FTP";
$resn_exch{" H62 FTP"}="2H6  FTP"; $resn_exch{" H71 FTP"}="1H7  FTP"; $resn_exch{" H72 FTP"}="2H7  FTP";
$resn_exch{" H   FTR"}="1HN  FTR"; $resn_exch{" H2  FTR"}="2HN  FTR"; $resn_exch{" HB2 FTR"}="1HB  FTR";
$resn_exch{" HB3 FTR"}="2HB  FTR"; $resn_exch{" HE1 FTR"}=" HNE FTR"; $resn_exch{" H11 FTS"}="1H1  FTS";
$resn_exch{" H12 FTS"}="2H1  FTS"; $resn_exch{"H101 FTS"}="1H10 FTS"; $resn_exch{"H102 FTS"}="2H10 FTS";
$resn_exch{" H31 FTS"}="1H3  FTS"; $resn_exch{" H32 FTS"}="2H3  FTS"; $resn_exch{"H111 FTS"}="1H11 FTS";
$resn_exch{"H112 FTS"}="2H11 FTS"; $resn_exch{"H121 FTS"}="1H12 FTS"; $resn_exch{"H122 FTS"}="2H12 FTS";
$resn_exch{" H21 FTT"}="1H2  FTT"; $resn_exch{" H22 FTT"}="2H2  FTT"; $resn_exch{" H41 FTT"}="1H4  FTT";
$resn_exch{" H42 FTT"}="2H4  FTT"; $resn_exch{" H51 FTT"}="1H5  FTT"; $resn_exch{" H52 FTT"}="2H5  FTT";
$resn_exch{" H61 FTT"}="1H6  FTT"; $resn_exch{" H62 FTT"}="2H6  FTT"; $resn_exch{" H71 FTT"}="1H7  FTT";
$resn_exch{" H72 FTT"}="2H7  FTT"; $resn_exch{" H81 FTT"}="1H8  FTT"; $resn_exch{" H82 FTT"}="2H8  FTT";
$resn_exch{" H91 FTT"}="1H9  FTT"; $resn_exch{" H92 FTT"}="2H9  FTT"; $resn_exch{"H101 FTT"}="1H10 FTT";
$resn_exch{"H102 FTT"}="2H10 FTT"; $resn_exch{"H111 FTT"}="1H11 FTT"; $resn_exch{"H112 FTT"}="2H11 FTT";
$resn_exch{"H121 FTT"}="1H12 FTT"; $resn_exch{"H122 FTT"}="2H12 FTT"; $resn_exch{"H131 FTT"}="1H13 FTT";
$resn_exch{"H132 FTT"}="2H13 FTT"; $resn_exch{"H141 FTT"}="1H14 FTT"; $resn_exch{"H142 FTT"}="2H14 FTT";
$resn_exch{"H143 FTT"}="3H14 FTT"; $resn_exch{" H   FTY"}="1HN  FTY"; $resn_exch{" H2  FTY"}="2HN  FTY";
$resn_exch{" HB2 FTY"}="1HB  FTY"; $resn_exch{" HB3 FTY"}="2HB  FTY"; $resn_exch{"HO2P FTY"}="PHO2 FTY";
$resn_exch{"HO3P FTY"}="PHO3 FTY"; $resn_exch{"H1C1 FU4"}="1H1C FU4"; $resn_exch{"H1C2 FU4"}="2H1C FU4";
$resn_exch{"H6C1 FU4"}="1H6C FU4"; $resn_exch{"H6C2 FU4"}="2H6C FU4"; $resn_exch{"H6C3 FU4"}="3H6C FU4";
$resn_exch{"H11A FUA"}="1H1  FUA"; $resn_exch{" H12 FUA"}="2H1  FUA"; $resn_exch{" H21 FUA"}="1H2  FUA";
$resn_exch{" H22 FUA"}="2H2  FUA"; $resn_exch{" H61 FUA"}="1H6  FUA"; $resn_exch{" H62 FUA"}="2H6  FUA";
$resn_exch{" H71 FUA"}="1H7  FUA"; $resn_exch{" H72 FUA"}="2H7  FUA"; $resn_exch{"H121 FUA"}="1H12 FUA";
$resn_exch{"H122 FUA"}="2H12 FUA"; $resn_exch{"H151 FUA"}="1H15 FUA"; $resn_exch{"H152 FUA"}="2H15 FUA";
$resn_exch{"H181 FUA"}="1H18 FUA"; $resn_exch{"H182 FUA"}="2H18 FUA"; $resn_exch{"H183 FUA"}="3H18 FUA";
$resn_exch{"H191 FUA"}="1H19 FUA"; $resn_exch{"H192 FUA"}="2H19 FUA"; $resn_exch{"H193 FUA"}="3H19 FUA";
$resn_exch{"H201 FUA"}="1H20 FUA"; $resn_exch{"H202 FUA"}="2H20 FUA"; $resn_exch{"H203 FUA"}="3H20 FUA";
$resn_exch{"H211 FUA"}="1H21 FUA"; $resn_exch{"H212 FUA"}="2H21 FUA"; $resn_exch{"H213 FUA"}="3H21 FUA";
$resn_exch{"H231 FUA"}="1H23 FUA"; $resn_exch{"H232 FUA"}="2H23 FUA"; $resn_exch{"H241 FUA"}="1H24 FUA";
$resn_exch{"H242 FUA"}="2H24 FUA"; $resn_exch{"H271 FUA"}="1H27 FUA"; $resn_exch{"H272 FUA"}="2H27 FUA";
$resn_exch{"H273 FUA"}="3H27 FUA"; $resn_exch{"H281 FUA"}="1H28 FUA"; $resn_exch{"H282 FUA"}="2H28 FUA";
$resn_exch{"H283 FUA"}="3H28 FUA"; $resn_exch{"H321 FUA"}="1H32 FUA"; $resn_exch{"H322 FUA"}="2H32 FUA";
$resn_exch{"H323 FUA"}="3H32 FUA"; $resn_exch{" H61 FUC"}="1H6  FUC"; $resn_exch{" H62 FUC"}="2H6  FUC";
$resn_exch{" H63 FUC"}="3H6  FUC"; $resn_exch{" H11 FUD"}="1H1  FUD"; $resn_exch{" H12 FUD"}="2H1  FUD";
$resn_exch{" H61 FUD"}="1H6  FUD"; $resn_exch{" H62 FUD"}="2H6  FUD"; $resn_exch{"H6C1 FUF"}="1H6C FUF";
$resn_exch{"H6C2 FUF"}="2H6C FUF"; $resn_exch{"H6C3 FUF"}="3H6C FUF"; $resn_exch{" H51 FUG"}="1H5  FUG";
$resn_exch{" H52 FUG"}="2H5  FUG"; $resn_exch{" H61 FUG"}="1H6  FUG"; $resn_exch{" H62 FUG"}="2H6  FUG";
$resn_exch{"H111 FUG"}="1H11 FUG"; $resn_exch{"H112 FUG"}="2H11 FUG"; $resn_exch{"H113 FUG"}="3H11 FUG";
$resn_exch{"H231 FUG"}="1H23 FUG"; $resn_exch{"H232 FUG"}="2H23 FUG"; $resn_exch{"H2A1 FUG"}="1H2A FUG";
$resn_exch{"H2A2 FUG"}="2H2A FUG"; $resn_exch{"H2A3 FUG"}="3H2A FUG"; $resn_exch{"H2B1 FUG"}="1H2B FUG";
$resn_exch{"H2B2 FUG"}="2H2B FUG"; $resn_exch{"H2B3 FUG"}="3H2B FUG"; $resn_exch{"H2C1 FUG"}="1H2C FUG";
$resn_exch{"H2C2 FUG"}="2H2C FUG"; $resn_exch{"H2C3 FUG"}="3H2C FUG"; $resn_exch{"H311 FUG"}="1H31 FUG";
$resn_exch{"H312 FUG"}="2H31 FUG"; $resn_exch{"H313 FUG"}="3H31 FUG"; $resn_exch{"HOB4 FUG"}="4HOB FUG";
$resn_exch{" H61 FUL"}="1H6  FUL"; $resn_exch{" H62 FUL"}="2H6  FUL"; $resn_exch{" H63 FUL"}="3H6  FUL";
$resn_exch{"HN11 FUN"}="1HN1 FUN"; $resn_exch{"HN12 FUN"}="2HN1 FUN"; $resn_exch{" H51 FUN"}="1H5  FUN";
$resn_exch{" H52 FUN"}="2H5  FUN"; $resn_exch{"HOB2 FUP"}="2HOB FUP"; $resn_exch{"HOB3 FUP"}="3HOB FUP";
$resn_exch{"HOA2 FUP"}="2HOA FUP"; $resn_exch{"H5'1 FUP"}="1H5* FUP"; $resn_exch{"H5'2 FUP"}="2H5* FUP";
$resn_exch{" H4' FUP"}=" H4* FUP"; $resn_exch{" H3' FUP"}=" H3* FUP"; $resn_exch{"H2'1 FUP"}="1H2* FUP";
$resn_exch{"H2'2 FUP"}="2H2* FUP"; $resn_exch{" H1' FUP"}=" H1* FUP"; $resn_exch{" H71 FUX"}="1H7  FUX";
$resn_exch{" H72 FUX"}="2H7  FUX"; $resn_exch{"HB11 FVF"}="1HB1 FVF"; $resn_exch{"HB21 FVF"}="1HB2 FVF";
$resn_exch{"HD11 FVF"}="1HD1 FVF"; $resn_exch{"HD21 FVF"}="1HD2 FVF"; $resn_exch{"HE11 FVF"}="1HE1 FVF";
$resn_exch{"HE21 FVF"}="1HE2 FVF"; $resn_exch{" HZ1 FVF"}="1HZ  FVF"; $resn_exch{" HN2 FVF"}="2HN  FVF";
$resn_exch{" HA2 FVF"}="2HA  FVF"; $resn_exch{"HB12 FVF"}="2HB1 FVF"; $resn_exch{"HB22 FVF"}="2HB2 FVF";
$resn_exch{"HD12 FVF"}="2HD1 FVF"; $resn_exch{"HD22 FVF"}="2HD2 FVF"; $resn_exch{"HE12 FVF"}="2HE1 FVF";
$resn_exch{"HE22 FVF"}="2HE2 FVF"; $resn_exch{" HZ2 FVF"}="2HZ  FVF"; $resn_exch{" HN3 FVF"}="3HN  FVF";
$resn_exch{" HA3 FVF"}="3HA  FVF"; $resn_exch{" HB3 FVF"}="3HB  FVF"; $resn_exch{"H113 FVF"}="3H11 FVF";
$resn_exch{"H123 FVF"}="3H12 FVF"; $resn_exch{"H133 FVF"}="3H13 FVF"; $resn_exch{"H213 FVF"}="3H21 FVF";
$resn_exch{"H223 FVF"}="3H22 FVF"; $resn_exch{"H233 FVF"}="3H23 FVF"; $resn_exch{"HOP2 FVF"}="2HOP FVF";
$resn_exch{" HA4 FVF"}="4HA  FVF"; $resn_exch{"HXT4 FVF"}="4HXT FVF"; $resn_exch{"HB14 FVF"}="4HB1 FVF";
$resn_exch{"HB24 FVF"}="4HB2 FVF"; $resn_exch{"HD14 FVF"}="4HD1 FVF"; $resn_exch{"HD24 FVF"}="4HD2 FVF";
$resn_exch{"HE14 FVF"}="4HE1 FVF"; $resn_exch{"HE24 FVF"}="4HE2 FVF"; $resn_exch{" HZ4 FVF"}="4HZ  FVF";
$resn_exch{" H71 FWD"}="1H7  FWD"; $resn_exch{" H72 FWD"}="2H7  FWD"; $resn_exch{" H81 FWD"}="1H8  FWD";
$resn_exch{" H82 FWD"}="2H8  FWD"; $resn_exch{"H92O FWD"}="OH92 FWD"; $resn_exch{"H101 FX3"}="1H10 FX3";
$resn_exch{"H102 FX3"}="2H10 FX3"; $resn_exch{"H103 FX3"}="3H10 FX3"; $resn_exch{"H5'1 FX3"}="1H5* FX3";
$resn_exch{"H5'2 FX3"}="2H5* FX3"; $resn_exch{" H4' FX3"}=" H4* FX3"; $resn_exch{" H3' FX3"}=" H3* FX3";
$resn_exch{" H1' FX3"}=" H1* FX3"; $resn_exch{" H2' FX3"}=" H2* FX3"; $resn_exch{" H81 FXI"}="1H8  FXI";
$resn_exch{" H82 FXI"}="2H8  FXI"; $resn_exch{" H91 FXI"}="1H9  FXI"; $resn_exch{" H92 FXI"}="2H9  FXI";
$resn_exch{"H111 FXI"}="1H11 FXI"; $resn_exch{"H112 FXI"}="2H11 FXI"; $resn_exch{"H131 FXI"}="1H13 FXI";
$resn_exch{"H132 FXI"}="2H13 FXI"; $resn_exch{"H133 FXI"}="3H13 FXI"; $resn_exch{"H261 FXI"}="1H26 FXI";
$resn_exch{"H262 FXI"}="2H26 FXI"; $resn_exch{"H271 FXI"}="1H27 FXI"; $resn_exch{"H272 FXI"}="2H27 FXI";
$resn_exch{"H291 FXI"}="1H29 FXI"; $resn_exch{"H292 FXI"}="2H29 FXI"; $resn_exch{"H293 FXI"}="3H29 FXI";
$resn_exch{"H311 FXM"}="1H31 FXM"; $resn_exch{"H312 FXM"}="2H31 FXM"; $resn_exch{"H313 FXM"}="3H31 FXM";
$resn_exch{"H321 FXM"}="1H32 FXM"; $resn_exch{"H322 FXM"}="2H32 FXM"; $resn_exch{"H323 FXM"}="3H32 FXM";
$resn_exch{"H331 FXM"}="1H33 FXM"; $resn_exch{"H332 FXM"}="2H33 FXM"; $resn_exch{"H333 FXM"}="3H33 FXM";
$resn_exch{"H261 FXM"}="1H26 FXM"; $resn_exch{"H262 FXM"}="2H26 FXM"; $resn_exch{"H161 FXM"}="1H16 FXM";
$resn_exch{"H162 FXM"}="2H16 FXM"; $resn_exch{"H191 FXM"}="1H19 FXM"; $resn_exch{"H192 FXM"}="2H19 FXM";
$resn_exch{" H61 FXM"}="1H6  FXM"; $resn_exch{" H62 FXM"}="2H6  FXM"; $resn_exch{"H361 FXM"}="1H36 FXM";
$resn_exch{"H362 FXM"}="2H36 FXM"; $resn_exch{"H371 FXM"}="1H37 FXM"; $resn_exch{"H372 FXM"}="2H37 FXM";
$resn_exch{"H373 FXM"}="3H37 FXM"; $resn_exch{"H381 FXM"}="1H38 FXM"; $resn_exch{"H382 FXM"}="2H38 FXM";
$resn_exch{"H141 FXN"}="1H14 FXN"; $resn_exch{"H142 FXN"}="2H14 FXN"; $resn_exch{" H91 FXN"}="1H9  FXN";
$resn_exch{" H92 FXN"}="2H9  FXN"; $resn_exch{"H1A1 FXP"}="1H1A FXP"; $resn_exch{"H1A2 FXP"}="2H1A FXP";
$resn_exch{"H5A1 FXP"}="1H5A FXP"; $resn_exch{"H5A2 FXP"}="2H5A FXP"; $resn_exch{"H5B1 FXP"}="1H5B FXP";
$resn_exch{"H5B2 FXP"}="2H5B FXP"; $resn_exch{"H161 FXV"}="1H16 FXV"; $resn_exch{"H162 FXV"}="2H16 FXV";
$resn_exch{"H163 FXV"}="3H16 FXV"; $resn_exch{"H201 FXV"}="1H20 FXV"; $resn_exch{"H202 FXV"}="2H20 FXV";
$resn_exch{"H301 FXV"}="1H30 FXV"; $resn_exch{"H302 FXV"}="2H30 FXV"; $resn_exch{"H311 FXV"}="1H31 FXV";
$resn_exch{"H312 FXV"}="2H31 FXV"; $resn_exch{"H341 FXV"}="1H34 FXV"; $resn_exch{"H342 FXV"}="2H34 FXV";
$resn_exch{"H343 FXV"}="3H34 FXV"; $resn_exch{"H251 FXY"}="1H25 FXY"; $resn_exch{"H252 FXY"}="2H25 FXY";
$resn_exch{"H253 FXY"}="3H25 FXY"; $resn_exch{" H21 FXY"}="1H2  FXY"; $resn_exch{" H22 FXY"}="2H2  FXY";
$resn_exch{" H31 FXY"}="1H3  FXY"; $resn_exch{" H32 FXY"}="2H3  FXY"; $resn_exch{" H41 FXY"}="1H4  FXY";
$resn_exch{" H42 FXY"}="2H4  FXY"; $resn_exch{" H51 FXY"}="1H5  FXY"; $resn_exch{" H52 FXY"}="2H5  FXY";
$resn_exch{" H61 FXY"}="1H6  FXY"; $resn_exch{" H62 FXY"}="2H6  FXY"; $resn_exch{" H71 FXY"}="1H7  FXY";
$resn_exch{" H72 FXY"}="2H7  FXY"; $resn_exch{" H73 FXY"}="3H7  FXY"; $resn_exch{" H1  FYA"}="1H   FYA";
$resn_exch{" H2A FYA"}="2H   FYA"; $resn_exch{" HN1 FYA"}="1HN  FYA"; $resn_exch{" HN2 FYA"}="2HN  FYA";
$resn_exch{" HB1 FYA"}="1HB  FYA"; $resn_exch{" HB2 FYA"}="2HB  FYA"; $resn_exch{"HOP2 FYA"}="2HOP FYA";
$resn_exch{"H5'1 FYA"}="1H5* FYA"; $resn_exch{"H5'2 FYA"}="2H5* FYA"; $resn_exch{" H4' FYA"}=" H4* FYA";
$resn_exch{" H3' FYA"}=" H3* FYA"; $resn_exch{"HO'3 FYA"}="3HO* FYA"; $resn_exch{" H2' FYA"}=" H2* FYA";
$resn_exch{"HO'2 FYA"}="2HO* FYA"; $resn_exch{" H1' FYA"}=" H1* FYA"; $resn_exch{" H61 FYA"}="1H6  FYA";
$resn_exch{" H62 FYA"}="2H6  FYA"; $resn_exch{"H6N1 FYN"}="1H6N FYN"; $resn_exch{"H6N2 FYN"}="2H6N FYN";
$resn_exch{"H5'1 FYN"}="1H5' FYN"; $resn_exch{"H5'2 FYN"}="2H5' FYN"; $resn_exch{"HPB1 FYN"}="1HPB FYN";
$resn_exch{"HPB2 FYN"}="2HPB FYN"; $resn_exch{"HP91 FYN"}="1HP9 FYN"; $resn_exch{"HP92 FYN"}="2HP9 FYN";
$resn_exch{"HP93 FYN"}="3HP9 FYN"; $resn_exch{"HP81 FYN"}="1HP8 FYN"; $resn_exch{"HP82 FYN"}="2HP8 FYN";
$resn_exch{"HP83 FYN"}="3HP8 FYN"; $resn_exch{"HP51 FYN"}="1HP5 FYN"; $resn_exch{"HP52 FYN"}="2HP5 FYN";
$resn_exch{"HP41 FYN"}="1HP4 FYN"; $resn_exch{"HP42 FYN"}="2HP4 FYN"; $resn_exch{"HP21 FYN"}="1HP2 FYN";
$resn_exch{"HP22 FYN"}="2HP2 FYN"; $resn_exch{"HP11 FYN"}="1HP1 FYN"; $resn_exch{"HP12 FYN"}="2HP1 FYN";
$resn_exch{"HOP3   G"}="3HOP   G"; $resn_exch{"HOP2   G"}="2HOP   G"; $resn_exch{" H5'   G"}="1H5*   G";
$resn_exch{"H5''   G"}="2H5*   G"; $resn_exch{" H4'   G"}=" H4*   G"; $resn_exch{" H3'   G"}=" H3*   G";
$resn_exch{"HO3'   G"}="H3T    G"; $resn_exch{" H2'   G"}=" H2*   G"; $resn_exch{"HO2'   G"}="2HO*   G";
$resn_exch{" H1'   G"}=" H1*   G"; $resn_exch{" H21   G"}="1H2    G"; $resn_exch{" H22   G"}="2H2    G";
$resn_exch{" H71 G0G"}="1H7  G0G"; $resn_exch{" H72 G0G"}="2H7  G0G"; $resn_exch{"H211 G0G"}="1H21 G0G";
$resn_exch{"H212 G0G"}="2H21 G0G"; $resn_exch{"H251 G0G"}="1H25 G0G"; $resn_exch{"H252 G0G"}="2H25 G0G";
$resn_exch{"H331 G0G"}="1H33 G0G"; $resn_exch{"H332 G0G"}="2H33 G0G"; $resn_exch{"H131 G12"}="1H13 G12";
$resn_exch{"H132 G12"}="2H13 G12"; $resn_exch{"H133 G12"}="3H13 G12"; $resn_exch{"H271 G15"}="1H27 G15";
$resn_exch{"H272 G15"}="2H27 G15"; $resn_exch{"H281 G15"}="1H28 G15"; $resn_exch{"H282 G15"}="2H28 G15";
$resn_exch{"H291 G15"}="1H29 G15"; $resn_exch{"H292 G15"}="2H29 G15"; $resn_exch{"H301 G15"}="1H30 G15";
$resn_exch{"H302 G15"}="2H30 G15"; $resn_exch{"H241 G15"}="1H24 G15"; $resn_exch{"H242 G15"}="2H24 G15";
$resn_exch{"H243 G15"}="3H24 G15"; $resn_exch{"H211 G15"}="1H21 G15"; $resn_exch{"H212 G15"}="2H21 G15";
$resn_exch{"H181 G15"}="1H18 G15"; $resn_exch{"H182 G15"}="2H18 G15"; $resn_exch{" H61 G16"}="1H6  G16";
$resn_exch{" H62 G16"}="2H6  G16"; $resn_exch{"HOP2 G16"}="2HOP G16"; $resn_exch{"HOP3 G16"}="3HOP G16";
$resn_exch{"HO1' G16"}="HO1* G16"; $resn_exch{"HN41 G19"}="1HN4 G19"; $resn_exch{"HN42 G19"}="2HN4 G19";
$resn_exch{"H11A G19"}="1H1  G19"; $resn_exch{" H12 G19"}="2H1  G19"; $resn_exch{"H131 G19"}="1H13 G19";
$resn_exch{"H132 G19"}="2H13 G19"; $resn_exch{"H151 G19"}="1H15 G19"; $resn_exch{"H152 G19"}="2H15 G19";
$resn_exch{"H153 G19"}="3H15 G19"; $resn_exch{"H161 G19"}="1H16 G19"; $resn_exch{"H162 G19"}="2H16 G19";
$resn_exch{"H163 G19"}="3H16 G19"; $resn_exch{"H171 G19"}="1H17 G19"; $resn_exch{"H172 G19"}="2H17 G19";
$resn_exch{"H173 G19"}="3H17 G19"; $resn_exch{"H181 G19"}="1H18 G19"; $resn_exch{"H182 G19"}="2H18 G19";
$resn_exch{"H183 G19"}="3H18 G19"; $resn_exch{"H201 G19"}="1H20 G19"; $resn_exch{"H202 G19"}="2H20 G19";
$resn_exch{"H321 G1G"}="1H32 G1G"; $resn_exch{"H322 G1G"}="2H32 G1G"; $resn_exch{"H461 G1G"}="1H46 G1G";
$resn_exch{"H462 G1G"}="2H46 G1G"; $resn_exch{"H463 G1G"}="3H46 G1G"; $resn_exch{"H451 G1G"}="1H45 G1G";
$resn_exch{"H452 G1G"}="2H45 G1G"; $resn_exch{"H251 G1G"}="1H25 G1G"; $resn_exch{"H252 G1G"}="2H25 G1G";
$resn_exch{"H121 G1G"}="1H12 G1G"; $resn_exch{"H122 G1G"}="2H12 G1G"; $resn_exch{"H371 G1G"}="1H37 G1G";
$resn_exch{"H372 G1G"}="2H37 G1G"; $resn_exch{"H373 G1G"}="3H37 G1G"; $resn_exch{"H141 G1L"}="1H14 G1L";
$resn_exch{"H142 G1L"}="2H14 G1L"; $resn_exch{"H143 G1L"}="3H14 G1L"; $resn_exch{"H151 G1L"}="1H15 G1L";
$resn_exch{"H152 G1L"}="2H15 G1L"; $resn_exch{"H153 G1L"}="3H15 G1L"; $resn_exch{"H161 G1L"}="1H16 G1L";
$resn_exch{"H162 G1L"}="2H16 G1L"; $resn_exch{" H61 G1P"}="1H6  G1P"; $resn_exch{" H62 G1P"}="2H6  G1P";
$resn_exch{"HOP2 G1P"}="2HOP G1P"; $resn_exch{"HOP3 G1P"}="3HOP G1P"; $resn_exch{"HN21 G1R"}="1HN2 G1R";
$resn_exch{"HN22 G1R"}="2HN2 G1R"; $resn_exch{" H1' G1R"}=" H1* G1R"; $resn_exch{" H2' G1R"}=" H2* G1R";
$resn_exch{"HO2' G1R"}="HO2* G1R"; $resn_exch{" H3' G1R"}=" H3* G1R"; $resn_exch{"HO3' G1R"}="HO3* G1R";
$resn_exch{" H4' G1R"}=" H4* G1R"; $resn_exch{"H5'1 G1R"}="1H5* G1R"; $resn_exch{"H5'2 G1R"}="2H5* G1R";
$resn_exch{"H5X1 G1R"}="1H5X G1R"; $resn_exch{"H5X2 G1R"}="2H5X G1R"; $resn_exch{"H111 G20"}="1H11 G20";
$resn_exch{"H112 G20"}="2H11 G20"; $resn_exch{"H113 G20"}="3H11 G20"; $resn_exch{"H811 G20"}="1H81 G20";
$resn_exch{"H812 G20"}="2H81 G20"; $resn_exch{"H813 G20"}="3H81 G20"; $resn_exch{" H91 G20"}="1H9  G20";
$resn_exch{" H92 G20"}="2H9  G20"; $resn_exch{"H911 G20"}="1H91 G20"; $resn_exch{"H912 G20"}="2H91 G20";
$resn_exch{"H921 G20"}="1H92 G20"; $resn_exch{"H922 G20"}="2H92 G20"; $resn_exch{"H923 G20"}="3H92 G20";
$resn_exch{"HN21 G20"}="1HN2 G20"; $resn_exch{"HN22 G20"}="2HN2 G20"; $resn_exch{" H31 G21"}="1H3  G21";
$resn_exch{" H32 G21"}="2H3  G21"; $resn_exch{"H111 G21"}="1H11 G21"; $resn_exch{"H112 G21"}="2H11 G21";
$resn_exch{"H113 G21"}="3H11 G21"; $resn_exch{"H811 G21"}="1H81 G21"; $resn_exch{"H812 G21"}="2H81 G21";
$resn_exch{"H821 G21"}="1H82 G21"; $resn_exch{"H822 G21"}="2H82 G21"; $resn_exch{" H91 G21"}="1H9  G21";
$resn_exch{" H92 G21"}="2H9  G21"; $resn_exch{"H911 G21"}="1H91 G21"; $resn_exch{"H912 G21"}="2H91 G21";
$resn_exch{"H921 G21"}="1H92 G21"; $resn_exch{"H922 G21"}="2H92 G21"; $resn_exch{"H923 G21"}="3H92 G21";
$resn_exch{"HNE1 G21"}="1HNE G21"; $resn_exch{"HNE2 G21"}="2HNE G21"; $resn_exch{" H51 G23"}="1H5  G23";
$resn_exch{" H52 G23"}="2H5  G23"; $resn_exch{" H53 G23"}="3H5  G23"; $resn_exch{" H61 G23"}="1H6  G23";
$resn_exch{" H62 G23"}="2H6  G23"; $resn_exch{" H63 G23"}="3H6  G23"; $resn_exch{" H91 G23"}="1H9  G23";
$resn_exch{" H92 G23"}="2H9  G23"; $resn_exch{"H111 G23"}="1H11 G23"; $resn_exch{"H112 G23"}="2H11 G23";
$resn_exch{"HAQ1 G24"}="1HAQ G24"; $resn_exch{"HAQ2 G24"}="2HAQ G24"; $resn_exch{"HAM1 G24"}="1HAM G24";
$resn_exch{"HAM2 G24"}="2HAM G24"; $resn_exch{"HBB1 G24"}="1HBB G24"; $resn_exch{"HBB2 G24"}="2HBB G24";
$resn_exch{"HBB3 G24"}="3HBB G24"; $resn_exch{"HAE1 G24"}="1HAE G24"; $resn_exch{"HAE2 G24"}="2HAE G24";
$resn_exch{"HAE3 G24"}="3HAE G24"; $resn_exch{"HAT1 G24"}="1HAT G24"; $resn_exch{"HAT2 G24"}="2HAT G24";
$resn_exch{"HOP3 G25"}="3HOP G25"; $resn_exch{"HOP2 G25"}="2HOP G25"; $resn_exch{" H5' G25"}="1H5* G25";
$resn_exch{"H5'' G25"}="2H5* G25"; $resn_exch{" H4' G25"}=" H4* G25"; $resn_exch{" H3' G25"}=" H3* G25";
$resn_exch{"HO'3 G25"}="3HO* G25"; $resn_exch{" H2' G25"}=" H2* G25"; $resn_exch{"HO2' G25"}="2HO* G25";
$resn_exch{" H1' G25"}=" H1* G25"; $resn_exch{" H21 G25"}="1H2  G25"; $resn_exch{" H22 G25"}="2H2  G25";
$resn_exch{" H51 G26"}="1H5  G26"; $resn_exch{" H52 G26"}="2H5  G26"; $resn_exch{" H53 G26"}="3H5  G26";
$resn_exch{" H61 G26"}="1H6  G26"; $resn_exch{" H62 G26"}="2H6  G26"; $resn_exch{" H63 G26"}="3H6  G26";
$resn_exch{" H91 G26"}="1H9  G26"; $resn_exch{" H92 G26"}="2H9  G26"; $resn_exch{"H101 G26"}="1H10 G26";
$resn_exch{"H102 G26"}="2H10 G26"; $resn_exch{"H121 G26"}="1H12 G26"; $resn_exch{"H122 G26"}="2H12 G26";
$resn_exch{"H191 G26"}="1H19 G26"; $resn_exch{"H192 G26"}="2H19 G26"; $resn_exch{"H101 G27"}="1H10 G27";
$resn_exch{"H102 G27"}="2H10 G27"; $resn_exch{" H91 G27"}="1H9  G27"; $resn_exch{" H92 G27"}="2H9  G27";
$resn_exch{" H81 G27"}="1H8  G27"; $resn_exch{" H82 G27"}="2H8  G27"; $resn_exch{" H71 G27"}="1H7  G27";
$resn_exch{" H72 G27"}="2H7  G27"; $resn_exch{" H21 G27"}="1H2  G27"; $resn_exch{" H22 G27"}="2H2  G27";
$resn_exch{" H61 G27"}="1H6  G27"; $resn_exch{" H62 G27"}="2H6  G27"; $resn_exch{" H31 G28"}="1H3  G28";
$resn_exch{" H32 G28"}="2H3  G28"; $resn_exch{"H111 G28"}="1H11 G28"; $resn_exch{"H112 G28"}="2H11 G28";
$resn_exch{"H113 G28"}="3H11 G28"; $resn_exch{"H811 G28"}="1H81 G28"; $resn_exch{"H812 G28"}="2H81 G28";
$resn_exch{"H821 G28"}="1H82 G28"; $resn_exch{"H822 G28"}="2H82 G28"; $resn_exch{"H823 G28"}="3H82 G28";
$resn_exch{" H91 G28"}="1H9  G28"; $resn_exch{" H92 G28"}="2H9  G28"; $resn_exch{"H911 G28"}="1H91 G28";
$resn_exch{"H912 G28"}="2H91 G28"; $resn_exch{"H913 G28"}="3H91 G28"; $resn_exch{"HN41 G28"}="1HN4 G28";
$resn_exch{"HN42 G28"}="2HN4 G28"; $resn_exch{" H61 G2F"}="1H6  G2F"; $resn_exch{" H62 G2F"}="2H6  G2F";
$resn_exch{"H11A G2H"}="1H1  G2H"; $resn_exch{" H12 G2H"}="2H1  G2H"; $resn_exch{"H31A G2H"}="1H3  G2H";
$resn_exch{" H32 G2H"}="2H3  G2H"; $resn_exch{"HOP3 G2L"}="3HOP G2L"; $resn_exch{"HOP2 G2L"}="2HOP G2L";
$resn_exch{" H5' G2L"}="1H5* G2L"; $resn_exch{"H5'' G2L"}="2H5* G2L"; $resn_exch{" H4' G2L"}=" H4* G2L";
$resn_exch{" H3' G2L"}=" H3* G2L"; $resn_exch{" H2' G2L"}=" H2* G2L"; $resn_exch{"HO2' G2L"}="2HO* G2L";
$resn_exch{" H1' G2L"}=" H1* G2L"; $resn_exch{" H21 G2L"}="1H2  G2L"; $resn_exch{" H22 G2L"}="2H2  G2L";
$resn_exch{"HA'1 G2L"}="1HA* G2L"; $resn_exch{"HA'2 G2L"}="2HA* G2L"; $resn_exch{"HB'1 G2L"}="1HB* G2L";
$resn_exch{"HB'2 G2L"}="2HB* G2L"; $resn_exch{"HD'1 G2L"}="1HD* G2L"; $resn_exch{"HD'2 G2L"}="2HD* G2L";
$resn_exch{"HD'3 G2L"}="3HD* G2L"; $resn_exch{"H3A1 G2P"}="1H3A G2P"; $resn_exch{"H3A2 G2P"}="2H3A G2P";
$resn_exch{"H5'1 G2P"}="1H5* G2P"; $resn_exch{"H5'2 G2P"}="2H5* G2P"; $resn_exch{" H4' G2P"}=" H4* G2P";
$resn_exch{" H3' G2P"}=" H3* G2P"; $resn_exch{" H2' G2P"}=" H2* G2P"; $resn_exch{" H1' G2P"}=" H1* G2P";
$resn_exch{"H2N1 G2P"}="1H2N G2P"; $resn_exch{"H2N2 G2P"}="2H2N G2P"; $resn_exch{"HN21 G2R"}="1HN2 G2R";
$resn_exch{"HN22 G2R"}="2HN2 G2R"; $resn_exch{" H1' G2R"}=" H1* G2R"; $resn_exch{" H2' G2R"}=" H2* G2R";
$resn_exch{"HO2' G2R"}="HO2* G2R"; $resn_exch{" H3' G2R"}=" H3* G2R"; $resn_exch{"HO3' G2R"}="HO3* G2R";
$resn_exch{" H4' G2R"}=" H4* G2R"; $resn_exch{"H5'1 G2R"}="1H5* G2R"; $resn_exch{"H5'2 G2R"}="2H5* G2R";
$resn_exch{"H5X1 G2R"}="1H5X G2R"; $resn_exch{"H5X2 G2R"}="2H5X G2R"; $resn_exch{"H1X1 G2R"}="1H1X G2R";
$resn_exch{"H1X2 G2R"}="2H1X G2R"; $resn_exch{"HOP3 G2S"}="3HOP G2S"; $resn_exch{"HOP2 G2S"}="2HOP G2S";
$resn_exch{" H5' G2S"}="1H5* G2S"; $resn_exch{"H5'' G2S"}="2H5* G2S"; $resn_exch{" H4' G2S"}=" H4* G2S";
$resn_exch{" H3' G2S"}=" H3* G2S"; $resn_exch{"HO3' G2S"}="H3T  G2S"; $resn_exch{"H2'1 G2S"}="1H2* G2S";
$resn_exch{" H1' G2S"}=" H1* G2S"; $resn_exch{" H21 G2S"}="1H2  G2S"; $resn_exch{" H22 G2S"}="2H2  G2S";
$resn_exch{"H2'2 G2S"}="2H2* G2S"; $resn_exch{"H5'1 G31"}="1H5* G31"; $resn_exch{"H5'2 G31"}="2H5* G31";
$resn_exch{" H4' G31"}=" H4* G31"; $resn_exch{" H3' G31"}=" H3* G31"; $resn_exch{"H2'1 G31"}="1H2* G31";
$resn_exch{"H2'2 G31"}="2H2* G31"; $resn_exch{" H1' G31"}=" H1* G31"; $resn_exch{"H2N1 G31"}="1H2N G31";
$resn_exch{"H2N2 G31"}="2H2N G31"; $resn_exch{"H3M1 G31"}="1H3M G31"; $resn_exch{"H3M2 G31"}="2H3M G31";
$resn_exch{"H3M3 G31"}="3H3M G31"; $resn_exch{"H5'1 G32"}="1H5* G32"; $resn_exch{"H5'2 G32"}="2H5* G32";
$resn_exch{" H4' G32"}=" H4* G32"; $resn_exch{" H3' G32"}=" H3* G32"; $resn_exch{"H2'1 G32"}="1H2* G32";
$resn_exch{"H2'2 G32"}="2H2* G32"; $resn_exch{" H1' G32"}=" H1* G32"; $resn_exch{"H2N1 G32"}="1H2N G32";
$resn_exch{"H2N2 G32"}="2H2N G32"; $resn_exch{"H6M1 G32"}="1H6M G32"; $resn_exch{"H6M2 G32"}="2H6M G32";
$resn_exch{"H6M3 G32"}="3H6M G32"; $resn_exch{"H5'1 G33"}="1H5* G33"; $resn_exch{"H5'2 G33"}="2H5* G33";
$resn_exch{" H4' G33"}=" H4* G33"; $resn_exch{" H3' G33"}=" H3* G33"; $resn_exch{"H2'1 G33"}="1H2* G33";
$resn_exch{"H2'2 G33"}="2H2* G33"; $resn_exch{" H1' G33"}=" H1* G33"; $resn_exch{" H21 G33"}="1H2  G33";
$resn_exch{" H22 G33"}="2H2  G33"; $resn_exch{"H8M1 G33"}="1H8M G33"; $resn_exch{"H8M2 G33"}="2H8M G33";
$resn_exch{"H8M3 G33"}="3H8M G33"; $resn_exch{" H81 G34"}="1H8  G34"; $resn_exch{" H82 G34"}="2H8  G34";
$resn_exch{" H21 G34"}="1H2  G34"; $resn_exch{" H22 G34"}="2H2  G34"; $resn_exch{"H11A G34"}="1H1  G34";
$resn_exch{" H12 G34"}="2H1  G34"; $resn_exch{" H71 G34"}="1H7  G34"; $resn_exch{" H72 G34"}="2H7  G34";
$resn_exch{"H131 G34"}="1H13 G34"; $resn_exch{"H132 G34"}="2H13 G34"; $resn_exch{"H151 G34"}="1H15 G34";
$resn_exch{"H152 G34"}="2H15 G34"; $resn_exch{"H153 G34"}="3H15 G34"; $resn_exch{"H161 G34"}="1H16 G34";
$resn_exch{"H162 G34"}="2H16 G34"; $resn_exch{"H163 G34"}="3H16 G34"; $resn_exch{"H171 G34"}="1H17 G34";
$resn_exch{"H172 G34"}="2H17 G34"; $resn_exch{"H173 G34"}="3H17 G34"; $resn_exch{"H181 G34"}="1H18 G34";
$resn_exch{"H182 G34"}="2H18 G34"; $resn_exch{"H183 G34"}="3H18 G34"; $resn_exch{"H201 G34"}="1H20 G34";
$resn_exch{"H202 G34"}="2H20 G34"; $resn_exch{"H221 G34"}="1H22 G34"; $resn_exch{"H222 G34"}="2H22 G34";
$resn_exch{"H241 G34"}="1H24 G34"; $resn_exch{"H242 G34"}="2H24 G34"; $resn_exch{"H261 G34"}="1H26 G34";
$resn_exch{"H262 G34"}="2H26 G34"; $resn_exch{"H271 G34"}="1H27 G34"; $resn_exch{"H272 G34"}="2H27 G34";
$resn_exch{"H291 G34"}="1H29 G34"; $resn_exch{"H292 G34"}="2H29 G34"; $resn_exch{"H301 G34"}="1H30 G34";
$resn_exch{"H302 G34"}="2H30 G34"; $resn_exch{"H303 G34"}="3H30 G34"; $resn_exch{"H5'1 G36"}="1H5* G36";
$resn_exch{"H5'2 G36"}="2H5* G36"; $resn_exch{" H4' G36"}=" H4* G36"; $resn_exch{" H3' G36"}=" H3* G36";
$resn_exch{"H2'1 G36"}="1H2* G36"; $resn_exch{"H2'2 G36"}="2H2* G36"; $resn_exch{"HM21 G36"}="1HM2 G36";
$resn_exch{"HM22 G36"}="2HM2 G36"; $resn_exch{"HM23 G36"}="3HM2 G36"; $resn_exch{" H1' G36"}=" H1* G36";
$resn_exch{"H2N1 G36"}="1H2N G36"; $resn_exch{"H2N2 G36"}="2H2N G36"; $resn_exch{"H1M1 G36"}="1H1M G36";
$resn_exch{"H1M2 G36"}="2H1M G36"; $resn_exch{" H11 G37"}="1H1  G37"; $resn_exch{" H12 G37"}="2H1  G37";
$resn_exch{" H71 G37"}="1H7  G37"; $resn_exch{" H72 G37"}="2H7  G37"; $resn_exch{" H73 G37"}="3H7  G37";
$resn_exch{" H81 G37"}="1H8  G37"; $resn_exch{" H82 G37"}="2H8  G37"; $resn_exch{" H83 G37"}="3H8  G37";
$resn_exch{"H111 G37"}="1H11 G37"; $resn_exch{"H112 G37"}="2H11 G37"; $resn_exch{"H191 G37"}="1H19 G37";
$resn_exch{"H192 G37"}="2H19 G37"; $resn_exch{"H341 G37"}="1H34 G37"; $resn_exch{"H342 G37"}="2H34 G37";
$resn_exch{"H411 G37"}="1H41 G37"; $resn_exch{"H412 G37"}="2H41 G37"; $resn_exch{"H5'1 G38"}="1H5* G38";
$resn_exch{"H5'2 G38"}="2H5* G38"; $resn_exch{" H4' G38"}=" H4* G38"; $resn_exch{" H3' G38"}=" H3* G38";
$resn_exch{"H2'1 G38"}="1H2* G38"; $resn_exch{"H2'2 G38"}="2H2* G38"; $resn_exch{" H1' G38"}=" H1* G38";
$resn_exch{" HN3 G38"}="3HN  G38"; $resn_exch{"H2N1 G38"}="1H2N G38"; $resn_exch{"H2N2 G38"}="2H2N G38";
$resn_exch{" HN1 G38"}="1HN  G38"; $resn_exch{" HN2 G38"}="2HN  G38"; $resn_exch{" H31 G39"}="1H3  G39";
$resn_exch{" H32 G39"}="2H3  G39"; $resn_exch{"H111 G39"}="1H11 G39"; $resn_exch{"H112 G39"}="2H11 G39";
$resn_exch{"H113 G39"}="3H11 G39"; $resn_exch{" H91 G39"}="1H9  G39"; $resn_exch{" H92 G39"}="2H9  G39";
$resn_exch{"H811 G39"}="1H81 G39"; $resn_exch{"H812 G39"}="2H81 G39"; $resn_exch{"H821 G39"}="1H82 G39";
$resn_exch{"H822 G39"}="2H82 G39"; $resn_exch{"H823 G39"}="3H82 G39"; $resn_exch{"H911 G39"}="1H91 G39";
$resn_exch{"H912 G39"}="2H91 G39"; $resn_exch{"H913 G39"}="3H91 G39"; $resn_exch{"HN41 G39"}="1HN4 G39";
$resn_exch{"HN42 G39"}="2HN4 G39"; $resn_exch{"H321 G3A"}="1H32 G3A"; $resn_exch{"H322 G3A"}="2H32 G3A";
$resn_exch{"H451 G3A"}="1H45 G3A"; $resn_exch{"H452 G3A"}="2H45 G3A"; $resn_exch{"H251 G3A"}="1H25 G3A";
$resn_exch{"H252 G3A"}="2H25 G3A"; $resn_exch{"H161 G3A"}="1H16 G3A"; $resn_exch{"H162 G3A"}="2H16 G3A";
$resn_exch{"HOB2 G3D"}="2HOB G3D"; $resn_exch{"HOB3 G3D"}="3HOB G3D"; $resn_exch{"HOA2 G3D"}="2HOA G3D";
$resn_exch{"H5'1 G3D"}="1H5* G3D"; $resn_exch{"H5'2 G3D"}="2H5* G3D"; $resn_exch{" H4' G3D"}=" H4* G3D";
$resn_exch{" H3' G3D"}=" H3* G3D"; $resn_exch{" H2' G3D"}=" H2* G3D"; $resn_exch{"HO2' G3D"}="*HO2 G3D";
$resn_exch{" H1' G3D"}=" H1* G3D"; $resn_exch{"HN21 G3D"}="1HN2 G3D"; $resn_exch{"HN22 G3D"}="2HN2 G3D";
$resn_exch{"H331 G3G"}="1H33 G3G"; $resn_exch{"H332 G3G"}="2H33 G3G"; $resn_exch{"H251 G3G"}="1H25 G3G";
$resn_exch{"H252 G3G"}="2H25 G3G"; $resn_exch{"H211 G3G"}="1H21 G3G"; $resn_exch{"H212 G3G"}="2H21 G3G";
$resn_exch{" H71 G3G"}="1H7  G3G"; $resn_exch{" H72 G3G"}="2H7  G3G"; $resn_exch{"H421 G3G"}="1H42 G3G";
$resn_exch{"H422 G3G"}="2H42 G3G"; $resn_exch{"H431 G3G"}="1H43 G3G"; $resn_exch{"H432 G3G"}="2H43 G3G";
$resn_exch{" H11 G3H"}="1H1  G3H"; $resn_exch{" H31 G3H"}="1H3  G3H"; $resn_exch{" H32 G3H"}="2H3  G3H";
$resn_exch{"HOP3 G3H"}="3HOP G3H"; $resn_exch{"HOP4 G3H"}="4HOP G3H"; $resn_exch{" H11 G3P"}="1H1  G3P";
$resn_exch{" H12 G3P"}="2H1  G3P"; $resn_exch{" H31 G3P"}="1H3  G3P"; $resn_exch{" H32 G3P"}="2H3  G3P";
$resn_exch{"HOP3 G3P"}="3HOP G3P"; $resn_exch{"HOP4 G3P"}="4HOP G3P"; $resn_exch{"H5'1 G42"}="1H5* G42";
$resn_exch{"H5'2 G42"}="2H5* G42"; $resn_exch{" H4' G42"}=" H4* G42"; $resn_exch{" H3' G42"}=" H3* G42";
$resn_exch{"H2'1 G42"}="1H2* G42"; $resn_exch{"H2'2 G42"}="2H2* G42"; $resn_exch{" H1' G42"}=" H1* G42";
$resn_exch{"H2N1 G42"}="1H2N G42"; $resn_exch{"H2N2 G42"}="2H2N G42"; $resn_exch{"H11A G44"}="1H1  G44";
$resn_exch{"H12A G44"}="2H1  G44"; $resn_exch{" H13 G44"}="3H1  G44"; $resn_exch{"H161 G44"}="1H16 G44";
$resn_exch{"H162 G44"}="2H16 G44"; $resn_exch{"H241 G44"}="1H24 G44"; $resn_exch{"H242 G44"}="2H24 G44";
$resn_exch{"H5'1 G46"}="1H5* G46"; $resn_exch{"H5'2 G46"}="2H5* G46"; $resn_exch{" H4' G46"}=" H4* G46";
$resn_exch{" H3' G46"}=" H3* G46"; $resn_exch{" H2' G46"}=" H2* G46"; $resn_exch{" H1' G46"}=" H1* G46";
$resn_exch{"H2N1 G46"}="1H2N G46"; $resn_exch{"H2N2 G46"}="2H2N G46"; $resn_exch{"H5'1 G47"}="1H5* G47";
$resn_exch{"H5'2 G47"}="2H5* G47"; $resn_exch{" H4' G47"}=" H4* G47"; $resn_exch{" H3' G47"}=" H3* G47";
$resn_exch{"H2'1 G47"}="1H2* G47"; $resn_exch{"H2'2 G47"}="2H2* G47"; $resn_exch{" H1' G47"}=" H1* G47";
$resn_exch{"H6A1 G47"}="1H6A G47"; $resn_exch{"H6A2 G47"}="2H6A G47"; $resn_exch{"H7A1 G47"}="1H7A G47";
$resn_exch{"H7A2 G47"}="2H7A G47"; $resn_exch{"H5'1 G48"}="1H5* G48"; $resn_exch{"H5'2 G48"}="2H5* G48";
$resn_exch{" H4' G48"}=" H4* G48"; $resn_exch{" H3' G48"}=" H3* G48"; $resn_exch{" H2' G48"}=" H2* G48";
$resn_exch{" H1' G48"}=" H1* G48"; $resn_exch{"H2N1 G48"}="1H2N G48"; $resn_exch{"H2N2 G48"}="2H2N G48";
$resn_exch{"HA'1 G48"}="1HA* G48"; $resn_exch{"HA'2 G48"}="2HA* G48"; $resn_exch{"HB'1 G48"}="1HB* G48";
$resn_exch{"HB'2 G48"}="2HB* G48"; $resn_exch{"HD'1 G48"}="1HD* G48"; $resn_exch{"HD'2 G48"}="2HD* G48";
$resn_exch{"HD'3 G48"}="3HD* G48"; $resn_exch{"H5'1 G49"}="1H5* G49"; $resn_exch{"H5'2 G49"}="2H5* G49";
$resn_exch{" H4' G49"}=" H4* G49"; $resn_exch{" H3' G49"}=" H3* G49"; $resn_exch{"H2'1 G49"}="1H2* G49";
$resn_exch{"H2'2 G49"}="2H2* G49"; $resn_exch{" H1' G49"}=" H1* G49"; $resn_exch{"HM21 G49"}="1HM2 G49";
$resn_exch{"HM22 G49"}="2HM2 G49"; $resn_exch{"HM23 G49"}="3HM2 G49"; $resn_exch{" H41 G4D"}="1H4  G4D";
$resn_exch{" H42 G4D"}="2H4  G4D"; $resn_exch{" H61 G4D"}="1H6  G4D"; $resn_exch{" H62 G4D"}="2H6  G4D";
$resn_exch{"HN21 G4G"}="1HN2 G4G"; $resn_exch{"HN2A G4G"}="2HN2 G4G"; $resn_exch{"HN11 G4G"}="1HN1 G4G";
$resn_exch{"HN12 G4G"}="2HN1 G4G"; $resn_exch{" H71 G4G"}="1H7  G4G"; $resn_exch{" H72 G4G"}="2H7  G4G";
$resn_exch{"H211 G4G"}="1H21 G4G"; $resn_exch{"H212 G4G"}="2H21 G4G"; $resn_exch{"H251 G4G"}="1H25 G4G";
$resn_exch{"H252 G4G"}="2H25 G4G"; $resn_exch{"H331 G4G"}="1H33 G4G"; $resn_exch{"H332 G4G"}="2H33 G4G";
$resn_exch{"H5B1 G4M"}="1H5B G4M"; $resn_exch{"H5B2 G4M"}="2H5B G4M"; $resn_exch{"H2A1 G4M"}="1H2A G4M";
$resn_exch{"H2A2 G4M"}="2H2A G4M"; $resn_exch{"H2A3 G4M"}="3H2A G4M"; $resn_exch{"H4A1 G4M"}="1H4A G4M";
$resn_exch{"H4A2 G4M"}="2H4A G4M"; $resn_exch{"H6G1 G4M"}="1H6G G4M"; $resn_exch{"H6G2 G4M"}="2H6G G4M";
$resn_exch{"H6G3 G4M"}="3H6G G4M"; $resn_exch{"H5'1 G4M"}="1H5* G4M"; $resn_exch{"H5'2 G4M"}="2H5* G4M";
$resn_exch{" H4' G4M"}=" H4* G4M"; $resn_exch{" H3' G4M"}=" H3* G4M"; $resn_exch{"HO3' G4M"}="HO3* G4M";
$resn_exch{" H2' G4M"}=" H2* G4M"; $resn_exch{"HO2' G4M"}="HO2* G4M"; $resn_exch{" H1' G4M"}=" H1* G4M";
$resn_exch{"HN21 G4M"}="1HN2 G4M"; $resn_exch{"HN22 G4M"}="2HN2 G4M"; $resn_exch{"HOB2 G4P"}="2HOB G4P";
$resn_exch{"HOB3 G4P"}="3HOB G4P"; $resn_exch{"HOA2 G4P"}="2HOA G4P"; $resn_exch{" H5' G4P"}="1H5* G4P";
$resn_exch{"H5'' G4P"}="2H5* G4P"; $resn_exch{" H4' G4P"}=" H4* G4P"; $resn_exch{" H3' G4P"}=" H3* G4P";
$resn_exch{" H2' G4P"}=" H2* G4P"; $resn_exch{"HO2' G4P"}="*HO2 G4P"; $resn_exch{" H1' G4P"}=" H1* G4P";
$resn_exch{"HN21 G4P"}="1HN2 G4P"; $resn_exch{"HN22 G4P"}="2HN2 G4P"; $resn_exch{" H61 G4S"}="1H6  G4S";
$resn_exch{" H62 G4S"}="2H6  G4S"; $resn_exch{"H8C1 G50"}="1H8C G50"; $resn_exch{"H8C2 G50"}="2H8C G50";
$resn_exch{"H9C1 G50"}="1H9C G50"; $resn_exch{"H9C2 G50"}="2H9C G50"; $resn_exch{"H5'1 G5P"}="1H5* G5P";
$resn_exch{"H5'2 G5P"}="2H5* G5P"; $resn_exch{" H4' G5P"}=" H4* G5P"; $resn_exch{" H1' G5P"}=" H1* G5P";
$resn_exch{"HN21 G5P"}="1HN2 G5P"; $resn_exch{"HN22 G5P"}="2HN2 G5P"; $resn_exch{"H11X G5P"}="1H1  G5P";
$resn_exch{" H12 G5P"}="2H1  G5P"; $resn_exch{" H2' G5P"}=" H2* G5P"; $resn_exch{" H3' G5P"}=" H3* G5P";
$resn_exch{"HN61 G5P"}="1HN6 G5P"; $resn_exch{"HN62 G5P"}="2HN6 G5P"; $resn_exch{"HCA3 G6D"}="3HCA G6D";
$resn_exch{"HOA3 G6D"}="3HOA G6D"; $resn_exch{"HCA4 G6D"}="4HCA G6D"; $resn_exch{"HOA4 G6D"}="4HOA G6D";
$resn_exch{"HCA5 G6D"}="5HCA G6D"; $resn_exch{"HC61 G6D"}="1HC6 G6D"; $resn_exch{"HC62 G6D"}="2HC6 G6D";
$resn_exch{"HC63 G6D"}="3HC6 G6D"; $resn_exch{" H61 G6P"}="1H6  G6P"; $resn_exch{" H62 G6P"}="2H6  G6P";
$resn_exch{" H61 G6Q"}="1H6  G6Q"; $resn_exch{" H62 G6Q"}="2H6  G6Q"; $resn_exch{"HOP2 G6Q"}="2HOP G6Q";
$resn_exch{"HOP3 G6Q"}="3HOP G6Q"; $resn_exch{" H61 G6S"}="1H6  G6S"; $resn_exch{" H62 G6S"}="2H6  G6S";
$resn_exch{"HOP3 G7M"}="3HOP G7M"; $resn_exch{"HOP2 G7M"}="2HOP G7M"; $resn_exch{" H5' G7M"}="1H5* G7M";
$resn_exch{"H5'' G7M"}="2H5* G7M"; $resn_exch{" H4' G7M"}=" H4* G7M"; $resn_exch{" H3' G7M"}=" H3* G7M";
$resn_exch{"HO3' G7M"}="H3T  G7M"; $resn_exch{" H2' G7M"}=" H2* G7M"; $resn_exch{"HO2' G7M"}="2HO* G7M";
$resn_exch{" H1' G7M"}=" H1* G7M"; $resn_exch{"HN71 G7M"}="1HN7 G7M"; $resn_exch{"HN72 G7M"}="2HN7 G7M";
$resn_exch{"HN73 G7M"}="3HN7 G7M"; $resn_exch{" H21 G7M"}="1H2  G7M"; $resn_exch{" H22 G7M"}="2H2  G7M";
$resn_exch{"H11A G80"}="1H1  G80"; $resn_exch{" H12 G80"}="2H1  G80"; $resn_exch{" H21 G80"}="1H2  G80";
$resn_exch{" H22 G80"}="2H2  G80"; $resn_exch{" H71 G80"}="1H7  G80"; $resn_exch{" H72 G80"}="2H7  G80";
$resn_exch{" H81 G80"}="1H8  G80"; $resn_exch{" H82 G80"}="2H8  G80"; $resn_exch{"H131 G80"}="1H13 G80";
$resn_exch{"H132 G80"}="2H13 G80"; $resn_exch{"H151 G80"}="1H15 G80"; $resn_exch{"H152 G80"}="2H15 G80";
$resn_exch{"H153 G80"}="3H15 G80"; $resn_exch{"H161 G80"}="1H16 G80"; $resn_exch{"H162 G80"}="2H16 G80";
$resn_exch{"H163 G80"}="3H16 G80"; $resn_exch{"H171 G80"}="1H17 G80"; $resn_exch{"H172 G80"}="2H17 G80";
$resn_exch{"H173 G80"}="3H17 G80"; $resn_exch{"H181 G80"}="1H18 G80"; $resn_exch{"H182 G80"}="2H18 G80";
$resn_exch{"H183 G80"}="3H18 G80"; $resn_exch{"H201 G80"}="1H20 G80"; $resn_exch{"H202 G80"}="2H20 G80";
$resn_exch{"H221 G80"}="1H22 G80"; $resn_exch{"H222 G80"}="2H22 G80"; $resn_exch{"H241 G80"}="1H24 G80";
$resn_exch{"H242 G80"}="2H24 G80"; $resn_exch{"H251 G80"}="1H25 G80"; $resn_exch{"H252 G80"}="2H25 G80";
$resn_exch{"H271 G80"}="1H27 G80"; $resn_exch{"H272 G80"}="2H27 G80"; $resn_exch{"H261 G80"}="1H26 G80";
$resn_exch{"H262 G80"}="2H26 G80"; $resn_exch{"H5C1 G88"}="1H5C G88"; $resn_exch{"H5C2 G88"}="2H5C G88";
$resn_exch{"H4C1 G88"}="1H4C G88"; $resn_exch{"H4C2 G88"}="2H4C G88"; $resn_exch{"H1C1 G88"}="1H1C G88";
$resn_exch{"H1C2 G88"}="2H1C G88"; $resn_exch{"HN11 GA0"}="1HN1 GA0"; $resn_exch{"HN12 GA0"}="2HN1 GA0";
$resn_exch{"HA11 GA0"}="1HA1 GA0"; $resn_exch{"HA12 GA0"}="2HA1 GA0"; $resn_exch{"HB21 GA0"}="1HB2 GA0";
$resn_exch{"HB22 GA0"}="2HB2 GA0"; $resn_exch{"HB23 GA0"}="3HB2 GA0"; $resn_exch{"HD31 GA0"}="1HD3 GA0";
$resn_exch{"HD32 GA0"}="2HD3 GA0"; $resn_exch{"HB31 GA0"}="1HB3 GA0"; $resn_exch{"HB32 GA0"}="2HB3 GA0";
$resn_exch{"HG31 GA0"}="1HG3 GA0"; $resn_exch{"HG32 GA0"}="2HG3 GA0"; $resn_exch{"H4'1 GA2"}="1H4* GA2";
$resn_exch{"H4'2 GA2"}="2H4* GA2"; $resn_exch{"HO4' GA2"}="*HO4 GA2"; $resn_exch{"H3'1 GA2"}="1H3* GA2";
$resn_exch{"H3'2 GA2"}="2H3* GA2"; $resn_exch{"HO3' GA2"}="*HO3 GA2"; $resn_exch{" H2' GA2"}=" H2* GA2";
$resn_exch{"H1'1 GA2"}="1H1* GA2"; $resn_exch{"H1'2 GA2"}="2H1* GA2"; $resn_exch{"HN21 GA2"}="1HN2 GA2";
$resn_exch{"HN22 GA2"}="2HN2 GA2"; $resn_exch{" H11 GA4"}="1H1  GA4"; $resn_exch{" H12 GA4"}="2H1  GA4";
$resn_exch{" H21 GA4"}="1H2  GA4"; $resn_exch{" H22 GA4"}="2H2  GA4"; $resn_exch{"H111 GA4"}="1H11 GA4";
$resn_exch{"H112 GA4"}="2H11 GA4"; $resn_exch{"H121 GA4"}="1H12 GA4"; $resn_exch{"H122 GA4"}="2H12 GA4";
$resn_exch{"H141 GA4"}="1H14 GA4"; $resn_exch{"H142 GA4"}="2H14 GA4"; $resn_exch{"H151 GA4"}="1H15 GA4";
$resn_exch{"H152 GA4"}="2H15 GA4"; $resn_exch{"H171 GA4"}="1H17 GA4"; $resn_exch{"H172 GA4"}="2H17 GA4";
$resn_exch{"H181 GA4"}="1H18 GA4"; $resn_exch{"H182 GA4"}="2H18 GA4"; $resn_exch{"H183 GA4"}="3H18 GA4";
$resn_exch{" H61 GAA"}="1H6  GAA"; $resn_exch{" H62 GAA"}="2H6  GAA"; $resn_exch{"HO2' GAB"}="'HO2 GAB";
$resn_exch{"HN31 GAB"}="1HN3 GAB"; $resn_exch{"HN32 GAB"}="2HN3 GAB"; $resn_exch{"H6A1 GAC"}="1H6A GAC";
$resn_exch{"H6A2 GAC"}="2H6A GAC"; $resn_exch{"H7A1 GAC"}="1H7A GAC"; $resn_exch{"H7A2 GAC"}="2H7A GAC";
$resn_exch{"HOA2 GAC"}="2HOA GAC"; $resn_exch{"HOA3 GAC"}="3HOA GAC"; $resn_exch{"HOA4 GAC"}="4HOA GAC";
$resn_exch{"HOA6 GAC"}="6HOA GAC"; $resn_exch{"H6B1 GAC"}="1H6B GAC"; $resn_exch{"H6B2 GAC"}="2H6B GAC";
$resn_exch{"H6B3 GAC"}="3H6B GAC"; $resn_exch{"HOB2 GAC"}="2HOB GAC"; $resn_exch{"HOB3 GAC"}="3HOB GAC";
$resn_exch{"H6C1 GAC"}="1H6C GAC"; $resn_exch{"H6C2 GAC"}="2H6C GAC"; $resn_exch{"HOC2 GAC"}="2HOC GAC";
$resn_exch{"HOC3 GAC"}="3HOC GAC"; $resn_exch{"HOC6 GAC"}="6HOC GAC"; $resn_exch{"H6D1 GAC"}="1H6D GAC";
$resn_exch{"H6D2 GAC"}="2H6D GAC"; $resn_exch{"HOD1 GAC"}="1HOD GAC"; $resn_exch{"HOD2 GAC"}="2HOD GAC";
$resn_exch{"HOD3 GAC"}="3HOD GAC"; $resn_exch{"HOD6 GAC"}="6HOD GAC"; $resn_exch{" H11 GAD"}="1H1  GAD";
$resn_exch{" H12 GAD"}="2H1  GAD"; $resn_exch{"H1C1 GAF"}="1H1C GAF"; $resn_exch{"H6C1 GAF"}="1H6C GAF";
$resn_exch{"H6C2 GAF"}="2H6C GAF"; $resn_exch{"H11A GAH"}="1H1  GAH"; $resn_exch{" H12 GAH"}="2H1  GAH";
$resn_exch{" H71 GAH"}="1H7  GAH"; $resn_exch{" H72 GAH"}="2H7  GAH"; $resn_exch{"H101 GAH"}="1H10 GAH";
$resn_exch{"H102 GAH"}="2H10 GAH"; $resn_exch{" H51 GAH"}="1H5  GAH"; $resn_exch{"H52A GAH"}="2H5  GAH";
$resn_exch{" H41 GAH"}="1H4  GAH"; $resn_exch{" H42 GAH"}="2H4  GAH"; $resn_exch{"H481 GAH"}="1H48 GAH";
$resn_exch{"H482 GAH"}="2H48 GAH"; $resn_exch{"H491 GAH"}="1H49 GAH"; $resn_exch{"H492 GAH"}="2H49 GAH";
$resn_exch{"H501 GAH"}="1H50 GAH"; $resn_exch{"H502 GAH"}="2H50 GAH"; $resn_exch{"H511 GAH"}="1H51 GAH";
$resn_exch{"H512 GAH"}="2H51 GAH"; $resn_exch{"H541 GAH"}="1H54 GAH"; $resn_exch{"H542 GAH"}="2H54 GAH";
$resn_exch{"H471 GAH"}="1H47 GAH"; $resn_exch{"H472 GAH"}="2H47 GAH"; $resn_exch{"H331 GAH"}="1H33 GAH";
$resn_exch{"H332 GAH"}="2H33 GAH"; $resn_exch{"H271 GAH"}="1H27 GAH"; $resn_exch{"H272 GAH"}="2H27 GAH";
$resn_exch{"H261 GAH"}="1H26 GAH"; $resn_exch{"H262 GAH"}="2H26 GAH"; $resn_exch{"H161 GAH"}="1H16 GAH";
$resn_exch{"H162 GAH"}="2H16 GAH"; $resn_exch{" H21 GAH"}="1H2  GAH"; $resn_exch{" H22 GAH"}="2H2  GAH";
$resn_exch{"H231 GAH"}="1H23 GAH"; $resn_exch{"H232 GAH"}="2H23 GAH"; $resn_exch{"H221 GAH"}="1H22 GAH";
$resn_exch{"H222 GAH"}="2H22 GAH"; $resn_exch{"H211 GAH"}="1H21 GAH"; $resn_exch{"H212 GAH"}="2H21 GAH";
$resn_exch{"H201 GAH"}="1H20 GAH"; $resn_exch{"H202 GAH"}="2H20 GAH"; $resn_exch{"H191 GAH"}="1H19 GAH";
$resn_exch{"H192 GAH"}="2H19 GAH"; $resn_exch{"HN21 GAI"}="1HN2 GAI"; $resn_exch{"HN22 GAI"}="2HN2 GAI";
$resn_exch{"HN31 GAI"}="1HN3 GAI"; $resn_exch{"HN32 GAI"}="2HN3 GAI"; $resn_exch{" H61 GAL"}="1H6  GAL";
$resn_exch{" H62 GAL"}="2H6  GAL"; $resn_exch{" H21 GAM"}="1H2  GAM"; $resn_exch{" H22 GAM"}="2H2  GAM";
$resn_exch{" H23 GAM"}="3H2  GAM"; $resn_exch{" H11 GAM"}="1H1  GAM"; $resn_exch{" H12 GAM"}="2H1  GAM";
$resn_exch{" HG1 GAM"}="1HG  GAM"; $resn_exch{" HG2 GAM"}="2HG  GAM"; $resn_exch{" HN1 GAM"}="1HN  GAM";
$resn_exch{" HN2 GAM"}="2HN  GAM"; $resn_exch{"HN11 GAN"}="1HN1 GAN"; $resn_exch{"HN12 GAN"}="2HN1 GAN";
$resn_exch{" H31 GAN"}="1H3  GAN"; $resn_exch{" H32 GAN"}="2H3  GAN"; $resn_exch{" H33 GAN"}="3H3  GAN";
$resn_exch{" H81 GAN"}="1H8  GAN"; $resn_exch{" H82 GAN"}="2H8  GAN"; $resn_exch{" H83 GAN"}="3H8  GAN";
$resn_exch{"H121 GAN"}="1H12 GAN"; $resn_exch{"H122 GAN"}="2H12 GAN"; $resn_exch{"H191 GAN"}="1H19 GAN";
$resn_exch{"H192 GAN"}="2H19 GAN"; $resn_exch{"H201 GAN"}="1H20 GAN"; $resn_exch{"H202 GAN"}="2H20 GAN";
$resn_exch{"H211 GAN"}="1H21 GAN"; $resn_exch{"H212 GAN"}="2H21 GAN"; $resn_exch{"H241 GAN"}="1H24 GAN";
$resn_exch{"H242 GAN"}="2H24 GAN"; $resn_exch{"H243 GAN"}="3H24 GAN"; $resn_exch{"H301 GAN"}="1H30 GAN";
$resn_exch{"H302 GAN"}="2H30 GAN"; $resn_exch{"H303 GAN"}="3H30 GAN"; $resn_exch{"H311 GAN"}="1H31 GAN";
$resn_exch{"H312 GAN"}="2H31 GAN"; $resn_exch{"H313 GAN"}="3H31 GAN"; $resn_exch{"H381 GAN"}="1H38 GAN";
$resn_exch{"H382 GAN"}="2H38 GAN"; $resn_exch{"H383 GAN"}="3H38 GAN"; $resn_exch{"H391 GAN"}="1H39 GAN";
$resn_exch{"H392 GAN"}="2H39 GAN"; $resn_exch{"H393 GAN"}="3H39 GAN"; $resn_exch{"H421 GAN"}="1H42 GAN";
$resn_exch{"H422 GAN"}="2H42 GAN"; $resn_exch{"H423 GAN"}="3H42 GAN"; $resn_exch{"HOP3 GAO"}="3HOP GAO";
$resn_exch{"HOP2 GAO"}="2HOP GAO"; $resn_exch{" H5' GAO"}="1H5* GAO"; $resn_exch{"H5'' GAO"}="2H5* GAO";
$resn_exch{" H4' GAO"}=" H4* GAO"; $resn_exch{" H3' GAO"}=" H3* GAO"; $resn_exch{"HO3' GAO"}="H3T  GAO";
$resn_exch{" H2' GAO"}=" H2* GAO"; $resn_exch{"HO2' GAO"}="2HO* GAO"; $resn_exch{" H1' GAO"}=" H1* GAO";
$resn_exch{" H21 GAO"}="1H2  GAO"; $resn_exch{" H22 GAO"}="2H2  GAO"; $resn_exch{" H1  GAP"}="1H   GAP";
$resn_exch{" H2A GAP"}="2H   GAP"; $resn_exch{" H3  GAP"}="3H   GAP"; $resn_exch{" HA1 GAP"}="1HA  GAP";
$resn_exch{" HA2 GAP"}="2HA  GAP"; $resn_exch{"H5'1 GAP"}="1H5* GAP"; $resn_exch{"H5'2 GAP"}="2H5* GAP";
$resn_exch{" H4' GAP"}=" H4* GAP"; $resn_exch{" H3' GAP"}=" H3* GAP"; $resn_exch{"HO'3 GAP"}="3HO* GAP";
$resn_exch{" H2' GAP"}=" H2* GAP"; $resn_exch{"HO'2 GAP"}="2HO* GAP"; $resn_exch{" H1' GAP"}=" H1* GAP";
$resn_exch{" H55 GAQ"}="5H5  GAQ"; $resn_exch{" H41 GAQ"}="1H4  GAQ"; $resn_exch{" H42 GAQ"}="2H4  GAQ";
$resn_exch{" H31 GAQ"}="1H3  GAQ"; $resn_exch{" H32 GAQ"}="2H3  GAQ"; $resn_exch{" H21 GAQ"}="1H2  GAQ";
$resn_exch{" H22 GAQ"}="2H2  GAQ"; $resn_exch{" H11 GAQ"}="1H1  GAQ"; $resn_exch{"H101 GAR"}="1H10 GAR";
$resn_exch{"H102 GAR"}="2H10 GAR"; $resn_exch{"H231 GAR"}="1H23 GAR"; $resn_exch{"H232 GAR"}="2H23 GAR";
$resn_exch{"H241 GAR"}="1H24 GAR"; $resn_exch{"H242 GAR"}="2H24 GAR"; $resn_exch{" H91 GAS"}="1H9  GAS";
$resn_exch{" H92 GAS"}="2H9  GAS"; $resn_exch{" H61 GAT"}="1H6  GAT"; $resn_exch{" H62 GAT"}="2H6  GAT";
$resn_exch{"HN11 GAT"}="1HN1 GAT"; $resn_exch{"HN12 GAT"}="2HN1 GAT"; $resn_exch{" H   GAU"}="1HN  GAU";
$resn_exch{" H2  GAU"}="2HN  GAU"; $resn_exch{" HB2 GAU"}="1HB  GAU"; $resn_exch{" HB3 GAU"}="2HB  GAU";
$resn_exch{" HG2 GAU"}="1HG  GAU"; $resn_exch{" HG3 GAU"}="2HG  GAU"; $resn_exch{" HO1 GAU"}="1HO  GAU";
$resn_exch{" HO2 GAU"}="2HO  GAU"; $resn_exch{"H2N1 GAV"}="1H2N GAV"; $resn_exch{"H2N2 GAV"}="2H2N GAV";
$resn_exch{" H1' GAV"}=" H1* GAV"; $resn_exch{" H2' GAV"}=" H2* GAV"; $resn_exch{" H3' GAV"}=" H3* GAV";
$resn_exch{" H4' GAV"}=" H4* GAV"; $resn_exch{"H5'1 GAV"}="1H5* GAV"; $resn_exch{"H5'2 GAV"}="2H5* GAV";
$resn_exch{" H1   GB"}="1H    GB"; $resn_exch{" H2   GB"}="2H    GB"; $resn_exch{" H3   GB"}="3H    GB";
$resn_exch{" H91 GB1"}="1H9  GB1"; $resn_exch{" H92 GB1"}="2H9  GB1"; $resn_exch{" H61 GB1"}="1H6  GB1";
$resn_exch{" H62 GB1"}="2H6  GB1"; $resn_exch{" H21 GB1"}="1H2  GB1"; $resn_exch{" H22 GB1"}="2H2  GB1";
$resn_exch{" H61 GB2"}="1H6  GB2"; $resn_exch{" H62 GB2"}="2H6  GB2"; $resn_exch{" H21 GB2"}="1H2  GB2";
$resn_exch{" H22 GB2"}="2H2  GB2"; $resn_exch{" H91 GB2"}="1H9  GB2"; $resn_exch{" H92 GB2"}="2H9  GB2";
$resn_exch{" H91 GB3"}="1H9  GB3"; $resn_exch{" H92 GB3"}="2H9  GB3"; $resn_exch{" H61 GB3"}="1H6  GB3";
$resn_exch{" H62 GB3"}="2H6  GB3"; $resn_exch{"H211 GB3"}="1H21 GB3"; $resn_exch{"H212 GB3"}="2H21 GB3";
$resn_exch{"H213 GB3"}="3H21 GB3"; $resn_exch{" H41 GBC"}="1H4  GBC"; $resn_exch{" H42 GBC"}="2H4  GBC";
$resn_exch{" H51 GBC"}="1H5  GBC"; $resn_exch{" H52 GBC"}="2H5  GBC"; $resn_exch{" H61 GBC"}="1H6  GBC";
$resn_exch{" H62 GBC"}="2H6  GBC"; $resn_exch{"HN31 GBC"}="1HN3 GBC"; $resn_exch{"HN32 GBC"}="2HN3 GBC";
$resn_exch{"HN11 GBI"}="1HN1 GBI"; $resn_exch{"HN12 GBI"}="2HN1 GBI"; $resn_exch{"HB11 GBI"}="1HB1 GBI";
$resn_exch{"HB12 GBI"}="2HB1 GBI"; $resn_exch{"HG11 GBI"}="1HG1 GBI"; $resn_exch{"HG12 GBI"}="2HG1 GBI";
$resn_exch{"HB21 GBI"}="1HB2 GBI"; $resn_exch{"HB22 GBI"}="2HB2 GBI"; $resn_exch{"HA31 GBI"}="1HA3 GBI";
$resn_exch{"HA32 GBI"}="2HA3 GBI"; $resn_exch{" H'1 GBI"}="1H'  GBI"; $resn_exch{" H'2 GBI"}="2H'  GBI";
$resn_exch{"HAC1 GBL"}="1HAC GBL"; $resn_exch{"HAC2 GBL"}="2HAC GBL"; $resn_exch{"HBC1 GBL"}="1HBC GBL";
$resn_exch{"HBC2 GBL"}="2HBC GBL"; $resn_exch{"HGC1 GBL"}="1HGC GBL"; $resn_exch{"HGC2 GBL"}="2HGC GBL";
$resn_exch{"HN11 GBN"}="1HN1 GBN"; $resn_exch{"HN12 GBN"}="2HN1 GBN"; $resn_exch{" H21 GBN"}="1H2  GBN";
$resn_exch{" H22 GBN"}="2H2  GBN"; $resn_exch{" H31 GBN"}="1H3  GBN"; $resn_exch{" H32 GBN"}="2H3  GBN";
$resn_exch{"H2R1 GBN"}="1H2R GBN"; $resn_exch{"H2R2 GBN"}="2H2R GBN"; $resn_exch{"H3R1 GBN"}="1H3R GBN";
$resn_exch{"H3R2 GBN"}="2H3R GBN"; $resn_exch{"H4R1 GBN"}="1H4R GBN"; $resn_exch{"H4R2 GBN"}="2H4R GBN";
$resn_exch{"H5R1 GBN"}="1H5R GBN"; $resn_exch{"H5R2 GBN"}="2H5R GBN"; $resn_exch{"H6R1 GBN"}="1H6R GBN";
$resn_exch{"H6R2 GBN"}="2H6R GBN"; $resn_exch{"HOT1 GBP"}="1HOT GBP"; $resn_exch{"H11N GBP"}="1NH1 GBP";
$resn_exch{"H12N GBP"}="2NH1 GBP"; $resn_exch{" HA1 GBP"}="1HA  GBP"; $resn_exch{" HB1 GBP"}="1HB  GBP";
$resn_exch{" HB2 GBP"}="2HB  GBP"; $resn_exch{"HB21 GBP"}="1HB2 GBP"; $resn_exch{"HB22 GBP"}="2HB2 GBP";
$resn_exch{" HG1 GBP"}="1HG  GBP"; $resn_exch{" HG2 GBP"}="2HG  GBP"; $resn_exch{"HN21 GBP"}="1HN2 GBP";
$resn_exch{"HA31 GBP"}="1HA3 GBP"; $resn_exch{"HA32 GBP"}="2HA3 GBP"; $resn_exch{"HO31 GBP"}="1HO3 GBP";
$resn_exch{"HH11 GBS"}="1HH1 GBS"; $resn_exch{"HH12 GBS"}="2HH1 GBS"; $resn_exch{"HN11 GBX"}="1HN1 GBX";
$resn_exch{"HN12 GBX"}="2HN1 GBX"; $resn_exch{"HB11 GBX"}="1HB1 GBX"; $resn_exch{"HB12 GBX"}="2HB1 GBX";
$resn_exch{"HG11 GBX"}="1HG1 GBX"; $resn_exch{"HG12 GBX"}="2HG1 GBX"; $resn_exch{"HB21 GBX"}="1HB2 GBX";
$resn_exch{"HB22 GBX"}="2HB2 GBX"; $resn_exch{"HA31 GBX"}="1HA3 GBX"; $resn_exch{"HA32 GBX"}="2HA3 GBX";
$resn_exch{" H11 GC1"}="1H1  GC1"; $resn_exch{" H41 GC4"}="1H4  GC4"; $resn_exch{" H42 GC4"}="2H4  GC4";
$resn_exch{"HN11 GC7"}="1HN1 GC7"; $resn_exch{"HN12 GC7"}="2HN1 GC7"; $resn_exch{" H11 GC7"}="1H1  GC7";
$resn_exch{" H12 GC7"}="2H1  GC7"; $resn_exch{" H21 GC7"}="1H2  GC7"; $resn_exch{" H22 GC7"}="2H2  GC7";
$resn_exch{" H31 GC7"}="1H3  GC7"; $resn_exch{" H32 GC7"}="2H3  GC7"; $resn_exch{" H41 GC7"}="1H4  GC7";
$resn_exch{" H42 GC7"}="2H4  GC7"; $resn_exch{" H51 GC7"}="1H5  GC7"; $resn_exch{" H52 GC7"}="2H5  GC7";
$resn_exch{" H61 GC7"}="1H6  GC7"; $resn_exch{" H62 GC7"}="2H6  GC7"; $resn_exch{" H71 GC7"}="1H7  GC7";
$resn_exch{" H72 GC7"}="2H7  GC7"; $resn_exch{"HH11 GC7"}="1HH1 GC7"; $resn_exch{"HH12 GC7"}="2HH1 GC7";
$resn_exch{"HH21 GC7"}="1HH2 GC7"; $resn_exch{"HH22 GC7"}="2HH2 GC7"; $resn_exch{" H21 GCA"}="1H2  GCA";
$resn_exch{" H41 GCA"}="1H4  GCA"; $resn_exch{" H61 GCA"}="1H6  GCA"; $resn_exch{" HP1 GCA"}="1HP  GCA";
$resn_exch{" HP2 GCA"}="2HP  GCA"; $resn_exch{" HP3 GCA"}="3HP  GCA"; $resn_exch{" HQ1 GCA"}="1HQ  GCA";
$resn_exch{" HQ2 GCA"}="2HQ  GCA"; $resn_exch{" HQ3 GCA"}="3HQ  GCA"; $resn_exch{" HG1 GCA"}="1HG  GCA";
$resn_exch{" HG2 GCA"}="2HG  GCA"; $resn_exch{" HF1 GCA"}="1HF  GCA"; $resn_exch{" HH1 GCA"}="1HH  GCA";
$resn_exch{" HH2 GCA"}="2HH  GCA"; $resn_exch{" HH3 GCA"}="3HH  GCA"; $resn_exch{" HI1 GCA"}="1HI  GCA";
$resn_exch{" HI2 GCA"}="2HI  GCA"; $resn_exch{" HI3 GCA"}="3HI  GCA"; $resn_exch{" HN1 GCA"}="1HN  GCA";
$resn_exch{" HA1 GCA"}="1HA  GCA"; $resn_exch{" HA2 GCA"}="2HA  GCA"; $resn_exch{" HC1 GCA"}="1HC  GCA";
$resn_exch{" HC2 GCA"}="2HC  GCA"; $resn_exch{" HD1 GCA"}="1HD  GCA"; $resn_exch{" HD2 GCA"}="2HD  GCA";
$resn_exch{" HD3 GCA"}="3HD  GCA"; $resn_exch{"HB11 GCG"}="1HB1 GCG"; $resn_exch{"HB12 GCG"}="2HB1 GCG";
$resn_exch{"HN11 GCG"}="1HN1 GCG"; $resn_exch{"HN12 GCG"}="2HN1 GCG"; $resn_exch{"HG11 GCG"}="1HG1 GCG";
$resn_exch{"HG12 GCG"}="2HG1 GCG"; $resn_exch{"HB21 GCG"}="1HB2 GCG"; $resn_exch{"HB22 GCG"}="2HB2 GCG";
$resn_exch{"HA31 GCG"}="1HA3 GCG"; $resn_exch{"HA32 GCG"}="2HA3 GCG"; $resn_exch{"HNS1 GCG"}="1HNS GCG";
$resn_exch{"H2S1 GCG"}="1H2S GCG"; $resn_exch{"H2S2 GCG"}="2H2S GCG"; $resn_exch{"H3S1 GCG"}="1H3S GCG";
$resn_exch{"H3S2 GCG"}="2H3S GCG"; $resn_exch{"H4S1 GCG"}="1H4S GCG"; $resn_exch{"H4S2 GCG"}="2H4S GCG";
$resn_exch{"H5S1 GCG"}="1H5S GCG"; $resn_exch{"H5S2 GCG"}="2H5S GCG"; $resn_exch{"H7S1 GCG"}="1H7S GCG";
$resn_exch{"H7S2 GCG"}="2H7S GCG"; $resn_exch{"HNS6 GCG"}="6HNS GCG"; $resn_exch{"H8S1 GCG"}="1H8S GCG";
$resn_exch{"H8S2 GCG"}="2H8S GCG"; $resn_exch{"H9S1 GCG"}="1H9S GCG"; $resn_exch{"H9S2 GCG"}="2H9S GCG";
$resn_exch{"HA51 GCG"}="1HA5 GCG"; $resn_exch{"HA52 GCG"}="2HA5 GCG"; $resn_exch{"HB61 GCG"}="1HB6 GCG";
$resn_exch{"HB62 GCG"}="2HB6 GCG"; $resn_exch{"HB71 GCG"}="1HB7 GCG"; $resn_exch{"HB72 GCG"}="2HB7 GCG";
$resn_exch{"HN71 GCG"}="1HN7 GCG"; $resn_exch{"HN72 GCG"}="2HN7 GCG"; $resn_exch{"HG71 GCG"}="1HG7 GCG";
$resn_exch{"HG72 GCG"}="2HG7 GCG"; $resn_exch{"HOP2 GCK"}="2HOP GCK"; $resn_exch{"H161 GCK"}="1H16 GCK";
$resn_exch{"H162 GCK"}="2H16 GCK"; $resn_exch{"H171 GCK"}="1H17 GCK"; $resn_exch{"H172 GCK"}="2H17 GCK";
$resn_exch{"H201 GCK"}="1H20 GCK"; $resn_exch{"H202 GCK"}="2H20 GCK"; $resn_exch{"H211 GCK"}="1H21 GCK";
$resn_exch{"H212 GCK"}="2H21 GCK"; $resn_exch{"H2'1 GCK"}="1H2* GCK"; $resn_exch{"H2'2 GCK"}="2H2* GCK";
$resn_exch{"H5'1 GCK"}="1H5* GCK"; $resn_exch{"H5'2 GCK"}="2H5* GCK"; $resn_exch{" H4' GCK"}=" H4* GCK";
$resn_exch{" H1' GCK"}=" H1* GCK"; $resn_exch{" H3' GCK"}=" H3* GCK"; $resn_exch{"HOP3 GCK"}="3HOP GCK";
$resn_exch{" H11 GCL"}="1H1  GCL"; $resn_exch{" H12 GCL"}="2H1  GCL"; $resn_exch{" H51 GCL"}="1H5  GCL";
$resn_exch{" H52 GCL"}="2H5  GCL"; $resn_exch{" H71 GCL"}="1H7  GCL"; $resn_exch{" H72 GCL"}="2H7  GCL";
$resn_exch{" H73 GCL"}="3H7  GCL"; $resn_exch{" H81 GCL"}="1H8  GCL"; $resn_exch{" H82 GCL"}="2H8  GCL";
$resn_exch{" H83 GCL"}="3H8  GCL"; $resn_exch{"H2'1 GCL"}="1H2' GCL"; $resn_exch{"H2'2 GCL"}="2H2' GCL";
$resn_exch{"H3'1 GCL"}="1H3' GCL"; $resn_exch{"H3'2 GCL"}="2H3' GCL"; $resn_exch{"H4'1 GCL"}="1H4' GCL";
$resn_exch{"H4'2 GCL"}="2H4' GCL"; $resn_exch{"H5'1 GCL"}="1H5' GCL"; $resn_exch{"H5'2 GCL"}="2H5' GCL";
$resn_exch{"H6'1 GCL"}="1H6' GCL"; $resn_exch{"H6'2 GCL"}="2H6' GCL"; $resn_exch{"HN21 GCL"}="1HN2 GCL";
$resn_exch{"HN22 GCL"}="2HN2 GCL"; $resn_exch{" H   GCM"}="1HN  GCM"; $resn_exch{" H2  GCM"}="2HN  GCM";
$resn_exch{" HA1 GCM"}="1HA  GCM"; $resn_exch{" HA2 GCM"}="2HA  GCM"; $resn_exch{" HM1 GCM"}="1HM  GCM";
$resn_exch{" HM2 GCM"}="2HM  GCM"; $resn_exch{" HM3 GCM"}="3HM  GCM"; $resn_exch{" H31 GCN"}="1H3  GCN";
$resn_exch{" H32 GCN"}="2H3  GCN"; $resn_exch{" H61 GCN"}="1H6  GCN"; $resn_exch{" H62 GCN"}="2H6  GCN";
$resn_exch{"HN21 GCN"}="1HN2 GCN"; $resn_exch{"HN22 GCN"}="2HN2 GCN"; $resn_exch{" H61 GCO"}="1H6  GCO";
$resn_exch{" H62 GCO"}="2H6  GCO"; $resn_exch{"HOG2 GCP"}="2HOG GCP"; $resn_exch{"HOG3 GCP"}="3HOG GCP";
$resn_exch{"H3B1 GCP"}="1H3B GCP"; $resn_exch{"H3B2 GCP"}="2H3B GCP"; $resn_exch{"HOB2 GCP"}="2HOB GCP";
$resn_exch{"HOA2 GCP"}="2HOA GCP"; $resn_exch{"H5'2 GCP"}="2H5* GCP"; $resn_exch{"H5'1 GCP"}="1H5* GCP";
$resn_exch{" H4' GCP"}=" H4* GCP"; $resn_exch{" H3' GCP"}=" H3* GCP"; $resn_exch{"HO3' GCP"}="*HO3 GCP";
$resn_exch{" H2' GCP"}=" H2* GCP"; $resn_exch{"HO2' GCP"}="*HO2 GCP"; $resn_exch{" H1' GCP"}=" H1* GCP";
$resn_exch{"HN21 GCP"}="1HN2 GCP"; $resn_exch{"HN22 GCP"}="2HN2 GCP"; $resn_exch{"H5'1 GCQ"}="1H5* GCQ";
$resn_exch{"H5'2 GCQ"}="2H5* GCQ"; $resn_exch{" H4' GCQ"}=" H4* GCQ"; $resn_exch{" H3' GCQ"}=" H3* GCQ";
$resn_exch{" H1' GCQ"}=" H1* GCQ"; $resn_exch{"HN41 GCQ"}="1HN4 GCQ"; $resn_exch{"HN42 GCQ"}="2HN4 GCQ";
$resn_exch{"H141 GCR"}="1H14 GCR"; $resn_exch{"H142 GCR"}="2H14 GCR"; $resn_exch{"H171 GCR"}="1H17 GCR";
$resn_exch{"H172 GCR"}="2H17 GCR"; $resn_exch{"H181 GCR"}="1H18 GCR"; $resn_exch{"H182 GCR"}="2H18 GCR";
$resn_exch{"H211 GCR"}="1H21 GCR"; $resn_exch{"H212 GCR"}="2H21 GCR"; $resn_exch{"H241 GCR"}="1H24 GCR";
$resn_exch{"H242 GCR"}="2H24 GCR"; $resn_exch{"H251 GCR"}="1H25 GCR"; $resn_exch{"H252 GCR"}="2H25 GCR";
$resn_exch{"H271 GCR"}="1H27 GCR"; $resn_exch{"H272 GCR"}="2H27 GCR"; $resn_exch{"H291 GCR"}="1H29 GCR";
$resn_exch{"H292 GCR"}="2H29 GCR"; $resn_exch{"H301 GCR"}="1H30 GCR"; $resn_exch{"H302 GCR"}="2H30 GCR";
$resn_exch{"H321 GCR"}="1H32 GCR"; $resn_exch{"H322 GCR"}="2H32 GCR"; $resn_exch{"H341 GCR"}="1H34 GCR";
$resn_exch{"H342 GCR"}="2H34 GCR"; $resn_exch{"H351 GCR"}="1H35 GCR"; $resn_exch{"H352 GCR"}="2H35 GCR";
$resn_exch{"H353 GCR"}="3H35 GCR"; $resn_exch{"H361 GCR"}="1H36 GCR"; $resn_exch{"H362 GCR"}="2H36 GCR";
$resn_exch{"H363 GCR"}="3H36 GCR"; $resn_exch{"H371 GCR"}="1H37 GCR"; $resn_exch{"H372 GCR"}="2H37 GCR";
$resn_exch{"H373 GCR"}="3H37 GCR"; $resn_exch{"H381 GCR"}="1H38 GCR"; $resn_exch{"H382 GCR"}="2H38 GCR";
$resn_exch{" H31 GCS"}="1H3  GCS"; $resn_exch{" H61 GCS"}="1H6  GCS"; $resn_exch{" H62 GCS"}="2H6  GCS";
$resn_exch{"HN21 GCS"}="1HN2 GCS"; $resn_exch{"HN22 GCS"}="2HN2 GCS"; $resn_exch{" H32 GCS"}="2H3  GCS";
$resn_exch{"HC11 GCT"}="1HC1 GCT"; $resn_exch{"HC12 GCT"}="2HC1 GCT"; $resn_exch{"HC41 GCT"}="1HC4 GCT";
$resn_exch{"HC42 GCT"}="2HC4 GCT"; $resn_exch{" H71 GCV"}="1H7  GCV"; $resn_exch{" H72 GCV"}="2H7  GCV";
$resn_exch{" H73 GCV"}="3H7  GCV"; $resn_exch{" H71 GCW"}="1H7  GCW"; $resn_exch{" H72 GCW"}="2H7  GCW";
$resn_exch{" H73 GCW"}="3H7  GCW"; $resn_exch{" H11 GD7"}="1H1  GD7"; $resn_exch{" H12 GD7"}="2H1  GD7";
$resn_exch{" H13 GD7"}="3H1  GD7"; $resn_exch{" H21 GD7"}="1H2  GD7"; $resn_exch{" H22 GD7"}="2H2  GD7";
$resn_exch{" H23 GD7"}="3H2  GD7"; $resn_exch{" H31 GD7"}="1H3  GD7"; $resn_exch{" H32 GD7"}="2H3  GD7";
$resn_exch{" H33 GD7"}="3H3  GD7"; $resn_exch{" H41 GD7"}="1H4  GD7"; $resn_exch{" H42 GD7"}="2H4  GD7";
$resn_exch{" H43 GD7"}="3H4  GD7"; $resn_exch{" H71 GD7"}="1H7  GD7"; $resn_exch{" H72 GD7"}="2H7  GD7";
$resn_exch{" H73 GD7"}="3H7  GD7"; $resn_exch{"H6C1 GDA"}="1H6C GDA"; $resn_exch{"H6C2 GDA"}="2H6C GDA";
$resn_exch{"H4N1 GDA"}="1H4N GDA"; $resn_exch{"H4N2 GDA"}="2H4N GDA"; $resn_exch{"HN11 GDB"}="1HN1 GDB";
$resn_exch{"HN12 GDB"}="2HN1 GDB"; $resn_exch{"HB11 GDB"}="1HB1 GDB"; $resn_exch{"HB12 GDB"}="2HB1 GDB";
$resn_exch{"HG11 GDB"}="1HG1 GDB"; $resn_exch{"HG12 GDB"}="2HG1 GDB"; $resn_exch{"HB21 GDB"}="1HB2 GDB";
$resn_exch{"HB22 GDB"}="2HB2 GDB"; $resn_exch{"HA31 GDB"}="1HA3 GDB"; $resn_exch{"HA32 GDB"}="2HA3 GDB";
$resn_exch{"H5'1 GDC"}="1H5* GDC"; $resn_exch{"H5'2 GDC"}="2H5* GDC"; $resn_exch{" H4' GDC"}=" H4* GDC";
$resn_exch{" H3' GDC"}=" H3* GDC"; $resn_exch{" H2' GDC"}=" H2* GDC"; $resn_exch{" H1' GDC"}=" H1* GDC";
$resn_exch{"H2N1 GDC"}="1H2N GDC"; $resn_exch{"H2N2 GDC"}="2H2N GDC"; $resn_exch{"H6A1 GDC"}="1H6A GDC";
$resn_exch{"H6A2 GDC"}="2H6A GDC"; $resn_exch{"H2N1 GDD"}="1H2N GDD"; $resn_exch{"H2N2 GDD"}="2H2N GDD";
$resn_exch{" H1' GDD"}=" H1* GDD"; $resn_exch{" H2' GDD"}=" H2* GDD"; $resn_exch{" H3' GDD"}=" H3* GDD";
$resn_exch{" H4' GDD"}=" H4* GDD"; $resn_exch{"H5'1 GDD"}="1H5* GDD"; $resn_exch{"H5'2 GDD"}="2H5* GDD";
$resn_exch{"H611 GDD"}="1H61 GDD"; $resn_exch{"H612 GDD"}="2H61 GDD"; $resn_exch{"H6C1 GDL"}="1H6C GDL";
$resn_exch{"H6C2 GDL"}="2H6C GDL"; $resn_exch{"H8C1 GDL"}="1H8C GDL"; $resn_exch{"H8C2 GDL"}="2H8C GDL";
$resn_exch{"H8C3 GDL"}="3H8C GDL"; $resn_exch{"HN21 GDM"}="1HN2 GDM"; $resn_exch{"HN22 GDM"}="2HN2 GDM";
$resn_exch{"H131 GDM"}="1H13 GDM"; $resn_exch{"H132 GDM"}="2H13 GDM"; $resn_exch{"H151 GDM"}="1H15 GDM";
$resn_exch{"H152 GDM"}="2H15 GDM"; $resn_exch{"H221 GDM"}="1H22 GDM"; $resn_exch{"H222 GDM"}="2H22 GDM";
$resn_exch{"H223 GDM"}="3H22 GDM"; $resn_exch{"H231 GDM"}="1H23 GDM"; $resn_exch{"H232 GDM"}="2H23 GDM";
$resn_exch{"H233 GDM"}="3H23 GDM"; $resn_exch{"H251 GDM"}="1H25 GDM"; $resn_exch{"H252 GDM"}="2H25 GDM";
$resn_exch{"H253 GDM"}="3H25 GDM"; $resn_exch{"H261 GDM"}="1H26 GDM"; $resn_exch{"H262 GDM"}="2H26 GDM";
$resn_exch{"H263 GDM"}="3H26 GDM"; $resn_exch{"H271 GDM"}="1H27 GDM"; $resn_exch{"H272 GDM"}="2H27 GDM";
$resn_exch{"H273 GDM"}="3H27 GDM"; $resn_exch{"H281 GDM"}="1H28 GDM"; $resn_exch{"H282 GDM"}="2H28 GDM";
$resn_exch{"H283 GDM"}="3H28 GDM"; $resn_exch{"H291 GDM"}="1H29 GDM"; $resn_exch{"H292 GDM"}="2H29 GDM";
$resn_exch{"H293 GDM"}="3H29 GDM"; $resn_exch{" HN1 GDN"}="1HN  GDN"; $resn_exch{"HN2A GDN"}="2HN  GDN";
$resn_exch{"HB11 GDN"}="1HB1 GDN"; $resn_exch{"HB12 GDN"}="2HB1 GDN"; $resn_exch{"HG11 GDN"}="1HG1 GDN";
$resn_exch{"HG12 GDN"}="2HG1 GDN"; $resn_exch{"HB21 GDN"}="1HB2 GDN"; $resn_exch{"HB22 GDN"}="2HB2 GDN";
$resn_exch{"HA31 GDN"}="1HA3 GDN"; $resn_exch{"HA32 GDN"}="2HA3 GDN"; $resn_exch{"HOB2 GDP"}="2HOB GDP";
$resn_exch{"HOB3 GDP"}="3HOB GDP"; $resn_exch{"HOA2 GDP"}="2HOA GDP"; $resn_exch{" H5' GDP"}="1H5* GDP";
$resn_exch{"H5'' GDP"}="2H5* GDP"; $resn_exch{" H4' GDP"}=" H4* GDP"; $resn_exch{" H3' GDP"}=" H3* GDP";
$resn_exch{"HO3' GDP"}="*HO3 GDP"; $resn_exch{" H2' GDP"}=" H2* GDP"; $resn_exch{"HO2' GDP"}="*HO2 GDP";
$resn_exch{" H1' GDP"}=" H1* GDP"; $resn_exch{"HN21 GDP"}="1HN2 GDP"; $resn_exch{"HN22 GDP"}="2HN2 GDP";
$resn_exch{"HOB2 GDR"}="2HOB GDR"; $resn_exch{"HOA2 GDR"}="2HOA GDR"; $resn_exch{" H5' GDR"}="1H5* GDR";
$resn_exch{"H5'' GDR"}="2H5* GDR"; $resn_exch{" H4' GDR"}=" H4* GDR"; $resn_exch{" H1' GDR"}=" H1* GDR";
$resn_exch{"HN21 GDR"}="1HN2 GDR"; $resn_exch{"HN22 GDR"}="2HN2 GDR"; $resn_exch{" H2' GDR"}=" H2* GDR";
$resn_exch{"HO2A GDR"}="HO2* GDR"; $resn_exch{" H3' GDR"}=" H3* GDR"; $resn_exch{"HO3A GDR"}="HO3* GDR";
$resn_exch{"HC61 GDR"}="1HC6 GDR"; $resn_exch{"HC62 GDR"}="2HC6 GDR"; $resn_exch{"HC63 GDR"}="3HC6 GDR";
$resn_exch{"HN11 GDS"}="1HN1 GDS"; $resn_exch{"HN12 GDS"}="2HN1 GDS"; $resn_exch{"HB11 GDS"}="1HB1 GDS";
$resn_exch{"HB12 GDS"}="2HB1 GDS"; $resn_exch{"HG11 GDS"}="1HG1 GDS"; $resn_exch{"HG12 GDS"}="2HG1 GDS";
$resn_exch{"HB21 GDS"}="1HB2 GDS"; $resn_exch{"HB22 GDS"}="2HB2 GDS"; $resn_exch{"HA31 GDS"}="1HA3 GDS";
$resn_exch{"HA32 GDS"}="2HA3 GDS"; $resn_exch{"HA41 GDS"}="1HA4 GDS"; $resn_exch{"HA42 GDS"}="2HA4 GDS";
$resn_exch{"HB51 GDS"}="1HB5 GDS"; $resn_exch{"HB52 GDS"}="2HB5 GDS"; $resn_exch{"HN61 GDS"}="1HN6 GDS";
$resn_exch{"HN62 GDS"}="2HN6 GDS"; $resn_exch{"HB61 GDS"}="1HB6 GDS"; $resn_exch{"HB62 GDS"}="2HB6 GDS";
$resn_exch{"HG61 GDS"}="1HG6 GDS"; $resn_exch{"HG62 GDS"}="2HG6 GDS"; $resn_exch{" H1D GDU"}=" H1* GDU";
$resn_exch{" H2D GDU"}=" H2* GDU"; $resn_exch{"HO2' GDU"}="*HO2 GDU"; $resn_exch{" H3D GDU"}=" H3* GDU";
$resn_exch{" H4D GDU"}=" H4* GDU"; $resn_exch{"HO3' GDU"}="*HO3 GDU"; $resn_exch{"H5'1 GDU"}="1H5* GDU";
$resn_exch{"H5'2 GDU"}="2H5* GDU"; $resn_exch{"HOA2 GDU"}="2HOA GDU"; $resn_exch{"HOB2 GDU"}="2HOB GDU";
$resn_exch{"H6'1 GDU"}="1H6' GDU"; $resn_exch{"H6'2 GDU"}="2H6' GDU"; $resn_exch{"HO2A GDU"}="'HO2 GDU";
$resn_exch{"HO3A GDU"}="'HO3 GDU"; $resn_exch{"HO4' GDU"}="'HO4 GDU"; $resn_exch{"HO6' GDU"}="'HO6 GDU";
$resn_exch{"H121 GDV"}="1H12 GDV"; $resn_exch{"H122 GDV"}="2H12 GDV"; $resn_exch{"H123 GDV"}="3H12 GDV";
$resn_exch{"H6C1 GDV"}="1H6C GDV"; $resn_exch{"H6C2 GDV"}="2H6C GDV"; $resn_exch{"HN21 GDX"}="1HN2 GDX";
$resn_exch{"HN22 GDX"}="2HN2 GDX"; $resn_exch{" H1' GDX"}=" H1* GDX"; $resn_exch{" H2' GDX"}=" H2* GDX";
$resn_exch{"HO2A GDX"}="*HO2 GDX"; $resn_exch{" H3' GDX"}=" H3* GDX"; $resn_exch{"HO3A GDX"}="*HO3 GDX";
$resn_exch{" H4' GDX"}=" H4* GDX"; $resn_exch{"H5'2 GDX"}="2H5* GDX"; $resn_exch{"H5'1 GDX"}="1H5* GDX";
$resn_exch{"HOA2 GDX"}="2HOA GDX"; $resn_exch{"HOB2 GDX"}="2HOB GDX"; $resn_exch{"HN21 GE1"}="1HN2 GE1";
$resn_exch{"HN22 GE1"}="2HN2 GE1"; $resn_exch{" H31 GE1"}="1H3  GE1"; $resn_exch{" H32 GE1"}="2H3  GE1";
$resn_exch{" H41 GE1"}="1H4  GE1"; $resn_exch{" H42 GE1"}="2H4  GE1"; $resn_exch{" H61 GE1"}="1H6  GE1";
$resn_exch{" H62 GE1"}="2H6  GE1"; $resn_exch{"HN11 GE1"}="1HN1 GE1"; $resn_exch{"HN12 GE1"}="2HN1 GE1";
$resn_exch{" H1A GE2"}="1H   GE2"; $resn_exch{" H2  GE2"}="2H   GE2"; $resn_exch{"HN11 GE2"}="1HN1 GE2";
$resn_exch{"HN12 GE2"}="2HN1 GE2"; $resn_exch{" H21 GE2"}="1H2  GE2"; $resn_exch{" H22 GE2"}="2H2  GE2";
$resn_exch{" H41 GE2"}="1H4  GE2"; $resn_exch{" H42 GE2"}="2H4  GE2"; $resn_exch{"HN61 GE2"}="1HN6 GE2";
$resn_exch{"HN62 GE2"}="2HN6 GE2"; $resn_exch{" H1  GE3"}="1H   GE3"; $resn_exch{" H2A GE3"}="2H   GE3";
$resn_exch{"H311 GE3"}="1H31 GE3"; $resn_exch{"H312 GE3"}="2H31 GE3"; $resn_exch{"H313 GE3"}="3H31 GE3";
$resn_exch{"H411 GE3"}="1H41 GE3"; $resn_exch{"H412 GE3"}="2H41 GE3"; $resn_exch{"H413 GE3"}="3H41 GE3";
$resn_exch{"HN11 GEA"}="1HN1 GEA"; $resn_exch{"HN12 GEA"}="2HN1 GEA"; $resn_exch{"HC41 GEA"}="1HC4 GEA";
$resn_exch{"HC51 GEA"}="1HC5 GEA"; $resn_exch{"HC52 GEA"}="2HC5 GEA"; $resn_exch{"HC61 GEA"}="1HC6 GEA";
$resn_exch{"HC62 GEA"}="2HC6 GEA"; $resn_exch{"HC71 GEA"}="1HC7 GEA"; $resn_exch{"HC72 GEA"}="2HC7 GEA";
$resn_exch{"H111 GEA"}="1H11 GEA"; $resn_exch{"H121 GEA"}="1H12 GEA"; $resn_exch{"H122 GEA"}="2H12 GEA";
$resn_exch{"H171 GEA"}="1H17 GEA"; $resn_exch{"H221 GEA"}="1H22 GEA"; $resn_exch{"H231 GEA"}="1H23 GEA";
$resn_exch{"H281 GEA"}="1H28 GEA"; $resn_exch{"H331 GEA"}="1H33 GEA"; $resn_exch{"H371 GEA"}="1H37 GEA";
$resn_exch{"H381 GEA"}="1H38 GEA"; $resn_exch{"H411 GEA"}="1H41 GEA"; $resn_exch{"H421 GEA"}="1H42 GEA";
$resn_exch{"H431 GEA"}="1H43 GEA"; $resn_exch{"H441 GEA"}="1H44 GEA"; $resn_exch{"H451 GEA"}="1H45 GEA";
$resn_exch{"H491 GEA"}="1H49 GEA"; $resn_exch{"H492 GEA"}="2H49 GEA"; $resn_exch{"H501 GEA"}="1H50 GEA";
$resn_exch{"H581 GEA"}="1H58 GEA"; $resn_exch{"H582 GEA"}="2H58 GEA"; $resn_exch{"H601 GEA"}="1H60 GEA";
$resn_exch{"H602 GEA"}="2H60 GEA"; $resn_exch{"H603 GEA"}="3H60 GEA"; $resn_exch{"H611 GEA"}="1H61 GEA";
$resn_exch{"H621 GEA"}="1H62 GEA"; $resn_exch{"H631 GEA"}="1H63 GEA"; $resn_exch{"H632 GEA"}="2H63 GEA";
$resn_exch{"H633 GEA"}="3H63 GEA"; $resn_exch{"H641 GEA"}="1H64 GEA"; $resn_exch{"H642 GEA"}="2H64 GEA";
$resn_exch{"H643 GEA"}="3H64 GEA"; $resn_exch{"H651 GEA"}="1H65 GEA"; $resn_exch{"H731 GEA"}="1H73 GEA";
$resn_exch{"H732 GEA"}="2H73 GEA"; $resn_exch{"H733 GEA"}="3H73 GEA"; $resn_exch{"H741 GEA"}="1H74 GEA";
$resn_exch{"H751 GEA"}="1H75 GEA"; $resn_exch{"H752 GEA"}="2H75 GEA"; $resn_exch{"H781 GEA"}="1H78 GEA";
$resn_exch{"H791 GEA"}="1H79 GEA"; $resn_exch{"H792 GEA"}="2H79 GEA"; $resn_exch{"H793 GEA"}="3H79 GEA";
$resn_exch{"H801 GEA"}="1H80 GEA"; $resn_exch{"H841 GEA"}="1H84 GEA"; $resn_exch{"H1C1 GEG"}="1H1C GEG";
$resn_exch{"H1C2 GEG"}="2H1C GEG"; $resn_exch{"H2C1 GEG"}="1H2C GEG"; $resn_exch{"H2C2 GEG"}="2H2C GEG";
$resn_exch{"H5C1 GEG"}="1H5C GEG"; $resn_exch{"H5C2 GEG"}="2H5C GEG"; $resn_exch{"H5C3 GEG"}="3H5C GEG";
$resn_exch{"H3N1 GEG"}="1H3N GEG"; $resn_exch{"H3N2 GEG"}="2H3N GEG"; $resn_exch{" H11 GEL"}="1H1  GEL";
$resn_exch{" H12 GEL"}="2H1  GEL"; $resn_exch{"H111 GEL"}="1H11 GEL"; $resn_exch{"H112 GEL"}="2H11 GEL";
$resn_exch{"H121 GEL"}="1H12 GEL"; $resn_exch{"H122 GEL"}="2H12 GEL"; $resn_exch{"H131 GEL"}="1H13 GEL";
$resn_exch{"H132 GEL"}="2H13 GEL"; $resn_exch{"H141 GEL"}="1H14 GEL"; $resn_exch{"H142 GEL"}="2H14 GEL";
$resn_exch{"H151 GEL"}="1H15 GEL"; $resn_exch{"H152 GEL"}="2H15 GEL"; $resn_exch{"H161 GEL"}="1H16 GEL";
$resn_exch{"H162 GEL"}="2H16 GEL"; $resn_exch{"H171 GEL"}="1H17 GEL"; $resn_exch{"H172 GEL"}="2H17 GEL";
$resn_exch{"H181 GEL"}="1H18 GEL"; $resn_exch{"H182 GEL"}="2H18 GEL"; $resn_exch{"H183 GEL"}="3H18 GEL";
$resn_exch{"HOP2 GEL"}="2HOP GEL"; $resn_exch{"H221 GEL"}="1H22 GEL"; $resn_exch{"H222 GEL"}="2H22 GEL";
$resn_exch{"H231 GEL"}="1H23 GEL"; $resn_exch{"H232 GEL"}="2H23 GEL"; $resn_exch{"H241 GEL"}="1H24 GEL";
$resn_exch{"H242 GEL"}="2H24 GEL"; $resn_exch{"H251 GEL"}="1H25 GEL"; $resn_exch{"H252 GEL"}="2H25 GEL";
$resn_exch{"H261 GEL"}="1H26 GEL"; $resn_exch{"H262 GEL"}="2H26 GEL"; $resn_exch{"H271 GEL"}="1H27 GEL";
$resn_exch{"H272 GEL"}="2H27 GEL"; $resn_exch{"H281 GEL"}="1H28 GEL"; $resn_exch{"H282 GEL"}="2H28 GEL";
$resn_exch{"H283 GEL"}="3H28 GEL"; $resn_exch{" H31 GEL"}="1H3  GEL"; $resn_exch{" H32 GEL"}="2H3  GEL";
$resn_exch{"HOP4 GEL"}="4HOP GEL"; $resn_exch{"H311 GEL"}="1H31 GEL"; $resn_exch{"H312 GEL"}="2H31 GEL";
$resn_exch{"H321 GEL"}="1H32 GEL"; $resn_exch{"H322 GEL"}="2H32 GEL"; $resn_exch{"HN31 GEL"}="1HN3 GEL";
$resn_exch{"HN32 GEL"}="2HN3 GEL"; $resn_exch{" HC8 GEM"}="8HC  GEM"; $resn_exch{"HC61 GEM"}="1HC6 GEM";
$resn_exch{"HC62 GEM"}="2HC6 GEM"; $resn_exch{"HC51 GEM"}="1HC5 GEM"; $resn_exch{"HC52 GEM"}="2HC5 GEM";
$resn_exch{" HN3 GEM"}="3HN  GEM"; $resn_exch{"HN41 GEM"}="1HN4 GEM"; $resn_exch{"HN42 GEM"}="2HN4 GEM";
$resn_exch{"HN21 GEM"}="1HN2 GEM"; $resn_exch{"HO41 GEM"}="1HO4 GEM"; $resn_exch{"H101 GEM"}="1H10 GEM";
$resn_exch{"H102 GEM"}="2H10 GEM"; $resn_exch{"HO31 GEM"}="1HO3 GEM"; $resn_exch{"H5'1 GEO"}="1H5' GEO";
$resn_exch{"H5'2 GEO"}="2H5' GEO"; $resn_exch{"HN41 GEO"}="1HN4 GEO"; $resn_exch{"HN42 GEO"}="2HN4 GEO";
$resn_exch{" H21 GEP"}="1H2  GEP"; $resn_exch{" H22 GEP"}="2H2  GEP"; $resn_exch{" H31 GEP"}="1H3  GEP";
$resn_exch{" H32 GEP"}="2H3  GEP"; $resn_exch{" H41 GEP"}="1H4  GEP"; $resn_exch{" H42 GEP"}="2H4  GEP";
$resn_exch{" H51 GEP"}="1H5  GEP"; $resn_exch{" H52 GEP"}="2H5  GEP"; $resn_exch{" H61 GEP"}="1H6  GEP";
$resn_exch{" H62 GEP"}="2H6  GEP"; $resn_exch{" H71 GEP"}="1H7  GEP"; $resn_exch{" H72 GEP"}="2H7  GEP";
$resn_exch{" H73 GEP"}="3H7  GEP"; $resn_exch{" H81 GEP"}="1H8  GEP"; $resn_exch{" H82 GEP"}="2H8  GEP";
$resn_exch{" H91 GEP"}="1H9  GEP"; $resn_exch{" H92 GEP"}="2H9  GEP"; $resn_exch{"H191 GEP"}="1H19 GEP";
$resn_exch{"H192 GEP"}="2H19 GEP"; $resn_exch{"H201 GEP"}="1H20 GEP"; $resn_exch{"H202 GEP"}="2H20 GEP";
$resn_exch{"H211 GEP"}="1H21 GEP"; $resn_exch{"H212 GEP"}="2H21 GEP"; $resn_exch{"H2C1 GEQ"}="1H2C GEQ";
$resn_exch{"H2C2 GEQ"}="2H2C GEQ"; $resn_exch{"H3C1 GEQ"}="1H3C GEQ"; $resn_exch{"H3C2 GEQ"}="2H3C GEQ";
$resn_exch{"H5C1 GEQ"}="1H5C GEQ"; $resn_exch{"H5C2 GEQ"}="2H5C GEQ"; $resn_exch{"H6C1 GEQ"}="1H6C GEQ";
$resn_exch{"H6C2 GEQ"}="2H6C GEQ"; $resn_exch{" H11 GER"}="1H1  GER"; $resn_exch{" H12 GER"}="2H1  GER";
$resn_exch{" H13 GER"}="3H1  GER"; $resn_exch{" H21 GER"}="1H2  GER"; $resn_exch{" H41 GER"}="1H4  GER";
$resn_exch{" H42 GER"}="2H4  GER"; $resn_exch{" H43 GER"}="3H4  GER"; $resn_exch{" H51 GER"}="1H5  GER";
$resn_exch{" H52 GER"}="2H5  GER"; $resn_exch{" H61 GER"}="1H6  GER"; $resn_exch{" H62 GER"}="2H6  GER";
$resn_exch{" H71 GER"}="1H7  GER"; $resn_exch{" H91 GER"}="1H9  GER"; $resn_exch{" H92 GER"}="2H9  GER";
$resn_exch{" H93 GER"}="3H9  GER"; $resn_exch{"H101 GER"}="1H10 GER"; $resn_exch{"H102 GER"}="2H10 GER";
$resn_exch{"H111 GER"}="1H11 GER"; $resn_exch{"H112 GER"}="2H11 GER"; $resn_exch{"H121 GER"}="1H12 GER";
$resn_exch{"H141 GER"}="1H14 GER"; $resn_exch{"H142 GER"}="2H14 GER"; $resn_exch{"H143 GER"}="3H14 GER";
$resn_exch{"H151 GER"}="1H15 GER"; $resn_exch{"H152 GER"}="2H15 GER"; $resn_exch{"H161 GER"}="1H16 GER";
$resn_exch{"H162 GER"}="2H16 GER"; $resn_exch{"H171 GER"}="1H17 GER"; $resn_exch{"H191 GER"}="1H19 GER";
$resn_exch{"H192 GER"}="2H19 GER"; $resn_exch{"H193 GER"}="3H19 GER"; $resn_exch{"H201 GER"}="1H20 GER";
$resn_exch{"H202 GER"}="2H20 GER"; $resn_exch{"H203 GER"}="3H20 GER"; $resn_exch{"H111 GET"}="1H11 GET";
$resn_exch{"H211 GET"}="1H21 GET"; $resn_exch{"H212 GET"}="2H21 GET"; $resn_exch{"H311 GET"}="1H31 GET";
$resn_exch{"H411 GET"}="1H41 GET"; $resn_exch{"H511 GET"}="1H51 GET"; $resn_exch{"H611 GET"}="1H61 GET";
$resn_exch{"H711 GET"}="1H71 GET"; $resn_exch{"H712 GET"}="2H71 GET"; $resn_exch{"H713 GET"}="3H71 GET";
$resn_exch{"H121 GET"}="1H12 GET"; $resn_exch{"H122 GET"}="2H12 GET"; $resn_exch{"H221 GET"}="1H22 GET";
$resn_exch{"H222 GET"}="2H22 GET"; $resn_exch{"H321 GET"}="1H32 GET"; $resn_exch{"H322 GET"}="2H32 GET";
$resn_exch{"H421 GET"}="1H42 GET"; $resn_exch{"H521 GET"}="1H52 GET"; $resn_exch{"H621 GET"}="1H62 GET";
$resn_exch{"H131 GET"}="1H13 GET"; $resn_exch{"H231 GET"}="1H23 GET"; $resn_exch{"H331 GET"}="1H33 GET";
$resn_exch{"H931 GET"}="1H93 GET"; $resn_exch{"H932 GET"}="2H93 GET"; $resn_exch{"H933 GET"}="3H93 GET";
$resn_exch{"H831 GET"}="1H83 GET"; $resn_exch{"H832 GET"}="2H83 GET"; $resn_exch{"H833 GET"}="3H83 GET";
$resn_exch{"H531 GET"}="1H53 GET"; $resn_exch{"H532 GET"}="2H53 GET"; $resn_exch{"HN21 GFF"}="1HN2 GFF";
$resn_exch{"HN22 GFF"}="2HN2 GFF"; $resn_exch{"H2'1 GFF"}="1H2' GFF"; $resn_exch{"H2'2 GFF"}="2H2' GFF";
$resn_exch{"H5'1 GFF"}="1H5' GFF"; $resn_exch{"H5'2 GFF"}="2H5' GFF"; $resn_exch{"H631 GFG"}="1H63 GFG";
$resn_exch{"H632 GFG"}="2H63 GFG"; $resn_exch{"H621 GFG"}="1H62 GFG"; $resn_exch{"H622 GFG"}="2H62 GFG";
$resn_exch{"H121 GFG"}="1H12 GFG"; $resn_exch{"H122 GFG"}="2H12 GFG"; $resn_exch{" H61 GFG"}="1H6  GFG";
$resn_exch{" H62 GFG"}="2H6  GFG"; $resn_exch{"H5'1 GFH"}="1H5' GFH"; $resn_exch{"H5'2 GFH"}="2H5' GFH";
$resn_exch{"HN21 GFH"}="1HN2 GFH"; $resn_exch{"HN22 GFH"}="2HN2 GFH"; $resn_exch{"H2'1 GFH"}="1H2' GFH";
$resn_exch{"H2'2 GFH"}="2H2' GFH"; $resn_exch{"H5'1 GFL"}="1H5* GFL"; $resn_exch{"H5'2 GFL"}="2H5* GFL";
$resn_exch{" H4' GFL"}=" H4* GFL"; $resn_exch{" H3' GFL"}=" H3* GFL"; $resn_exch{"HO3' GFL"}="HO3* GFL";
$resn_exch{" H2' GFL"}=" H2* GFL"; $resn_exch{" H1' GFL"}=" H1* GFL"; $resn_exch{"HN21 GFL"}="1HN2 GFL";
$resn_exch{"HN22 GFL"}="2HN2 GFL"; $resn_exch{" H61 GFP"}="1H6  GFP"; $resn_exch{" H62 GFP"}="2H6  GFP";
$resn_exch{"HOP2 GFP"}="2HOP GFP"; $resn_exch{"HOP3 GFP"}="3HOP GFP"; $resn_exch{" H41 GG1"}="1H4  GG1";
$resn_exch{" H42 GG1"}="2H4  GG1"; $resn_exch{"H251 GG1"}="1H25 GG1"; $resn_exch{"H252 GG1"}="2H25 GG1";
$resn_exch{"H253 GG1"}="3H25 GG1"; $resn_exch{"H111 GG1"}="1H11 GG1"; $resn_exch{"H112 GG1"}="2H11 GG1";
$resn_exch{"H151 GG2"}="1H15 GG2"; $resn_exch{"H152 GG2"}="2H15 GG2"; $resn_exch{"H153 GG2"}="3H15 GG2";
$resn_exch{"H201 GG2"}="1H20 GG2"; $resn_exch{"H202 GG2"}="2H20 GG2"; $resn_exch{"H211 GG2"}="1H21 GG2";
$resn_exch{"H212 GG2"}="2H21 GG2"; $resn_exch{"H241 GG2"}="1H24 GG2"; $resn_exch{"H242 GG2"}="2H24 GG2";
$resn_exch{"H251 GG2"}="1H25 GG2"; $resn_exch{"H252 GG2"}="2H25 GG2"; $resn_exch{"H231 GG2"}="1H23 GG2";
$resn_exch{"H232 GG2"}="2H23 GG2"; $resn_exch{"HN31 GG2"}="1HN3 GG2"; $resn_exch{"HN32 GG2"}="2HN3 GG2";
$resn_exch{"H171 GG2"}="1H17 GG2"; $resn_exch{"H172 GG2"}="2H17 GG2"; $resn_exch{" H21 GG3"}="1H2  GG3";
$resn_exch{" H22 GG3"}="2H2  GG3"; $resn_exch{" H61 GG3"}="1H6  GG3"; $resn_exch{" H62 GG3"}="2H6  GG3";
$resn_exch{" H71 GG3"}="1H7  GG3"; $resn_exch{" H72 GG3"}="2H7  GG3"; $resn_exch{"H211 GG3"}="1H21 GG3";
$resn_exch{"H212 GG3"}="2H21 GG3"; $resn_exch{"H213 GG3"}="3H21 GG3"; $resn_exch{"H221 GG3"}="1H22 GG3";
$resn_exch{"H222 GG3"}="2H22 GG3"; $resn_exch{"H261 GG3"}="1H26 GG3"; $resn_exch{"H262 GG3"}="2H26 GG3";
$resn_exch{"H271 GG3"}="1H27 GG3"; $resn_exch{"H272 GG3"}="2H27 GG3"; $resn_exch{"H31A GG4"}="1H3  GG4";
$resn_exch{" H32 GG4"}="2H3  GG4"; $resn_exch{" H51 GG4"}="1H5  GG4"; $resn_exch{" H52 GG4"}="2H5  GG4";
$resn_exch{" H61 GG4"}="1H6  GG4"; $resn_exch{" H62 GG4"}="2H6  GG4"; $resn_exch{" H71 GG4"}="1H7  GG4";
$resn_exch{" H72 GG4"}="2H7  GG4"; $resn_exch{"H121 GG4"}="1H12 GG4"; $resn_exch{"H122 GG4"}="2H12 GG4";
$resn_exch{"H161 GG4"}="1H16 GG4"; $resn_exch{"H162 GG4"}="2H16 GG4"; $resn_exch{"H191 GG4"}="1H19 GG4";
$resn_exch{"H192 GG4"}="2H19 GG4"; $resn_exch{"H201 GG4"}="1H20 GG4"; $resn_exch{"H202 GG4"}="2H20 GG4";
$resn_exch{"H211 GG4"}="1H21 GG4"; $resn_exch{"H212 GG4"}="2H21 GG4"; $resn_exch{"H213 GG4"}="3H21 GG4";
$resn_exch{"H221 GG4"}="1H22 GG4"; $resn_exch{"H222 GG4"}="2H22 GG4"; $resn_exch{"H223 GG4"}="3H22 GG4";
$resn_exch{"H231 GG4"}="1H23 GG4"; $resn_exch{"H232 GG4"}="2H23 GG4"; $resn_exch{"H233 GG4"}="3H23 GG4";
$resn_exch{"H241 GG4"}="1H24 GG4"; $resn_exch{"H242 GG4"}="2H24 GG4"; $resn_exch{"H251 GG4"}="1H25 GG4";
$resn_exch{"H252 GG4"}="2H25 GG4"; $resn_exch{"H271 GG4"}="1H27 GG4"; $resn_exch{"H272 GG4"}="2H27 GG4";
$resn_exch{"H281 GG4"}="1H28 GG4"; $resn_exch{"H282 GG4"}="2H28 GG4"; $resn_exch{"H331 GG4"}="1H33 GG4";
$resn_exch{"H332 GG4"}="2H33 GG4"; $resn_exch{"H333 GG4"}="3H33 GG4"; $resn_exch{"H291 GG4"}="1H29 GG4";
$resn_exch{"H292 GG4"}="2H29 GG4"; $resn_exch{"H301 GG4"}="1H30 GG4"; $resn_exch{"H302 GG4"}="2H30 GG4";
$resn_exch{" H31 GG6"}="1H3  GG6"; $resn_exch{" H32 GG6"}="2H3  GG6"; $resn_exch{" H33 GG6"}="3H3  GG6";
$resn_exch{" HB1 GG7"}="1HB  GG7"; $resn_exch{" HB2 GG7"}="2HB  GG7"; $resn_exch{" HB3 GG7"}="3HB  GG7";
$resn_exch{" HN1 GG7"}="1HN  GG7"; $resn_exch{" HN2 GG7"}="2HN  GG7"; $resn_exch{" H61 GG9"}="1H6  GG9";
$resn_exch{" H62 GG9"}="2H6  GG9"; $resn_exch{"HN81 GGA"}="1HN8 GGA"; $resn_exch{"HN82 GGA"}="2HN8 GGA";
$resn_exch{"H101 GGA"}="1H10 GGA"; $resn_exch{"H102 GGA"}="2H10 GGA"; $resn_exch{"H121 GGA"}="1H12 GGA";
$resn_exch{"H122 GGA"}="2H12 GGA"; $resn_exch{"H221 GGA"}="1H22 GGA"; $resn_exch{"H222 GGA"}="2H22 GGA";
$resn_exch{"H223 GGA"}="3H22 GGA"; $resn_exch{"H291 GGA"}="1H29 GGA"; $resn_exch{"H292 GGA"}="2H29 GGA";
$resn_exch{"H331 GGA"}="1H33 GGA"; $resn_exch{"H332 GGA"}="2H33 GGA"; $resn_exch{"H351 GGA"}="1H35 GGA";
$resn_exch{"H352 GGA"}="2H35 GGA"; $resn_exch{"H371 GGA"}="1H37 GGA"; $resn_exch{"H372 GGA"}="2H37 GGA";
$resn_exch{"H391 GGA"}="1H39 GGA"; $resn_exch{"H392 GGA"}="2H39 GGA"; $resn_exch{"H431 GGA"}="1H43 GGA";
$resn_exch{"H432 GGA"}="2H43 GGA"; $resn_exch{"H451 GGA"}="1H45 GGA"; $resn_exch{"H452 GGA"}="2H45 GGA";
$resn_exch{"H471 GGA"}="1H47 GGA"; $resn_exch{"H472 GGA"}="2H47 GGA"; $resn_exch{"H491 GGA"}="1H49 GGA";
$resn_exch{"H492 GGA"}="2H49 GGA"; $resn_exch{"H541 GGA"}="1H54 GGA"; $resn_exch{"H542 GGA"}="2H54 GGA";
$resn_exch{"HN1A GGB"}="1HN  GGB"; $resn_exch{" HN2 GGB"}="2HN  GGB"; $resn_exch{"HCB1 GGB"}="1HCB GGB";
$resn_exch{"HCB2 GGB"}="2HCB GGB"; $resn_exch{"HCG1 GGB"}="1HCG GGB"; $resn_exch{"HCG2 GGB"}="2HCG GGB";
$resn_exch{"HN21 GGB"}="1HN2 GGB"; $resn_exch{"HN22 GGB"}="2HN2 GGB"; $resn_exch{"HB11 GGC"}="1HB1 GGC";
$resn_exch{"HB12 GGC"}="2HB1 GGC"; $resn_exch{"HG11 GGC"}="1HG1 GGC"; $resn_exch{"HG12 GGC"}="2HG1 GGC";
$resn_exch{"HN11 GGC"}="1HN1 GGC"; $resn_exch{"HN12 GGC"}="2HN1 GGC"; $resn_exch{"HB21 GGC"}="1HB2 GGC";
$resn_exch{"HB22 GGC"}="2HB2 GGC"; $resn_exch{"H101 GGC"}="1H10 GGC"; $resn_exch{"H102 GGC"}="2H10 GGC";
$resn_exch{"HA31 GGC"}="1HA3 GGC"; $resn_exch{"HA32 GGC"}="2HA3 GGC"; $resn_exch{"HA11 GGD"}="1HA1 GGD";
$resn_exch{"HA21 GGD"}="1HA2 GGD"; $resn_exch{"HA31 GGD"}="1HA3 GGD"; $resn_exch{"HA41 GGD"}="1HA4 GGD";
$resn_exch{"HA51 GGD"}="1HA5 GGD"; $resn_exch{"HA61 GGD"}="1HA6 GGD"; $resn_exch{"HA62 GGD"}="2HA6 GGD";
$resn_exch{"HO21 GGD"}="1HO2 GGD"; $resn_exch{"HO41 GGD"}="1HO4 GGD"; $resn_exch{"HO61 GGD"}="1HO6 GGD";
$resn_exch{"HB11 GGD"}="1HB1 GGD"; $resn_exch{"HB21 GGD"}="1HB2 GGD"; $resn_exch{"HB31 GGD"}="1HB3 GGD";
$resn_exch{"HB41 GGD"}="1HB4 GGD"; $resn_exch{"HB51 GGD"}="1HB5 GGD"; $resn_exch{"HB61 GGD"}="1HB6 GGD";
$resn_exch{"HB62 GGD"}="2HB6 GGD"; $resn_exch{"HC31 GGD"}="1HC3 GGD"; $resn_exch{"HC32 GGD"}="2HC3 GGD";
$resn_exch{"HC41 GGD"}="1HC4 GGD"; $resn_exch{"HC61 GGD"}="1HC6 GGD"; $resn_exch{"HC62 GGD"}="2HC6 GGD";
$resn_exch{"H141 GGD"}="1H14 GGD"; $resn_exch{"H142 GGD"}="2H14 GGD"; $resn_exch{"H151 GGD"}="1H15 GGD";
$resn_exch{"H152 GGD"}="2H15 GGD"; $resn_exch{"H161 GGD"}="1H16 GGD"; $resn_exch{"H162 GGD"}="2H16 GGD";
$resn_exch{"H171 GGD"}="1H17 GGD"; $resn_exch{"H172 GGD"}="2H17 GGD"; $resn_exch{"H181 GGD"}="1H18 GGD";
$resn_exch{"H182 GGD"}="2H18 GGD"; $resn_exch{"H191 GGD"}="1H19 GGD"; $resn_exch{"H192 GGD"}="2H19 GGD";
$resn_exch{"H201 GGD"}="1H20 GGD"; $resn_exch{"H202 GGD"}="2H20 GGD"; $resn_exch{"H211 GGD"}="1H21 GGD";
$resn_exch{"H212 GGD"}="2H21 GGD"; $resn_exch{"H221 GGD"}="1H22 GGD"; $resn_exch{"H231 GGD"}="1H23 GGD";
$resn_exch{"H241 GGD"}="1H24 GGD"; $resn_exch{"H242 GGD"}="2H24 GGD"; $resn_exch{"H311 GGD"}="1H31 GGD";
$resn_exch{"H312 GGD"}="2H31 GGD"; $resn_exch{"H321 GGD"}="1H32 GGD"; $resn_exch{"H322 GGD"}="2H32 GGD";
$resn_exch{"H331 GGD"}="1H33 GGD"; $resn_exch{"H332 GGD"}="2H33 GGD"; $resn_exch{"H341 GGD"}="1H34 GGD";
$resn_exch{"H342 GGD"}="2H34 GGD"; $resn_exch{"H351 GGD"}="1H35 GGD"; $resn_exch{"H352 GGD"}="2H35 GGD";
$resn_exch{"H361 GGD"}="1H36 GGD"; $resn_exch{"H362 GGD"}="2H36 GGD"; $resn_exch{"H371 GGD"}="1H37 GGD";
$resn_exch{"H372 GGD"}="2H37 GGD"; $resn_exch{"H381 GGD"}="1H38 GGD"; $resn_exch{"H391 GGD"}="1H39 GGD";
$resn_exch{"H401 GGD"}="1H40 GGD"; $resn_exch{"H402 GGD"}="2H40 GGD"; $resn_exch{"H411 GGD"}="1H41 GGD";
$resn_exch{"H412 GGD"}="2H41 GGD"; $resn_exch{"H421 GGD"}="1H42 GGD"; $resn_exch{"H422 GGD"}="2H42 GGD";
$resn_exch{"H431 GGD"}="1H43 GGD"; $resn_exch{"H432 GGD"}="2H43 GGD"; $resn_exch{"H441 GGD"}="1H44 GGD";
$resn_exch{"H442 GGD"}="2H44 GGD"; $resn_exch{"H251 GGD"}="1H25 GGD"; $resn_exch{"H252 GGD"}="2H25 GGD";
$resn_exch{"H261 GGD"}="1H26 GGD"; $resn_exch{"H262 GGD"}="2H26 GGD"; $resn_exch{"H271 GGD"}="1H27 GGD";
$resn_exch{"H272 GGD"}="2H27 GGD"; $resn_exch{"H281 GGD"}="1H28 GGD"; $resn_exch{"H282 GGD"}="2H28 GGD";
$resn_exch{"H291 GGD"}="1H29 GGD"; $resn_exch{"H292 GGD"}="2H29 GGD"; $resn_exch{"H301 GGD"}="1H30 GGD";
$resn_exch{"H302 GGD"}="2H30 GGD"; $resn_exch{"H481 GGD"}="1H48 GGD"; $resn_exch{"H482 GGD"}="2H48 GGD";
$resn_exch{"H483 GGD"}="3H48 GGD"; $resn_exch{"H451 GGD"}="1H45 GGD"; $resn_exch{"H452 GGD"}="2H45 GGD";
$resn_exch{"H461 GGD"}="1H46 GGD"; $resn_exch{"H462 GGD"}="2H46 GGD"; $resn_exch{"H471 GGD"}="1H47 GGD";
$resn_exch{"H472 GGD"}="2H47 GGD"; $resn_exch{"H473 GGD"}="3H47 GGD"; $resn_exch{"H3B1 GGH"}="1H3B GGH";
$resn_exch{"H3B2 GGH"}="2H3B GGH"; $resn_exch{"H5'1 GGH"}="1H5' GGH"; $resn_exch{"H5'2 GGH"}="2H5' GGH";
$resn_exch{"HN21 GGH"}="1HN2 GGH"; $resn_exch{"HN22 GGH"}="2HN2 GGH"; $resn_exch{"H2'1 GGH"}="1H2' GGH";
$resn_exch{"H2'2 GGH"}="2H2' GGH"; $resn_exch{" HN1 GGL"}="1HN  GGL"; $resn_exch{" HN2 GGL"}="2HN  GGL";
$resn_exch{" HB1 GGL"}="1HB  GGL"; $resn_exch{" HB2 GGL"}="2HB  GGL"; $resn_exch{" HG1 GGL"}="1HG  GGL";
$resn_exch{" HG2 GGL"}="2HG  GGL"; $resn_exch{" H51 GGO"}="1H5  GGO"; $resn_exch{" H52 GGO"}="2H5  GGO";
$resn_exch{" H61 GGO"}="1H6  GGO"; $resn_exch{" H62 GGO"}="2H6  GGO"; $resn_exch{" HN1 GGO"}="1HN  GGO";
$resn_exch{" HN2 GGO"}="2HN  GGO"; $resn_exch{" H31 GGO"}="1H3  GGO"; $resn_exch{" H32 GGO"}="2H3  GGO";
$resn_exch{"H121 GGO"}="1H12 GGO"; $resn_exch{"H122 GGO"}="2H12 GGO"; $resn_exch{"H181 GGO"}="1H18 GGO";
$resn_exch{"H182 GGO"}="2H18 GGO"; $resn_exch{"H301 GGP"}="1H30 GGP"; $resn_exch{"H302 GGP"}="2H30 GGP";
$resn_exch{" H3' GGZ"}=" H3* GGZ"; $resn_exch{" H4' GGZ"}=" H4* GGZ"; $resn_exch{"H5'1 GGZ"}="1H5* GGZ";
$resn_exch{"H5'2 GGZ"}="2H5* GGZ"; $resn_exch{" H2' GGZ"}=" H2* GGZ"; $resn_exch{" H1' GGZ"}=" H1* GGZ";
$resn_exch{"HN61 GGZ"}="1HN6 GGZ"; $resn_exch{"HN62 GGZ"}="2HN6 GGZ"; $resn_exch{"HOG2 GH3"}="2HOG GH3";
$resn_exch{"HOG3 GH3"}="3HOG GH3"; $resn_exch{"HOB2 GH3"}="2HOB GH3"; $resn_exch{"HOA2 GH3"}="2HOA GH3";
$resn_exch{" H5' GH3"}="1H5* GH3"; $resn_exch{"H5'' GH3"}="2H5* GH3"; $resn_exch{" H4' GH3"}=" H4* GH3";
$resn_exch{"H3'2 GH3"}="2H3* GH3"; $resn_exch{" H3' GH3"}=" H3* GH3"; $resn_exch{" H2' GH3"}=" H2* GH3";
$resn_exch{"HO2' GH3"}="*HO2 GH3"; $resn_exch{" H1' GH3"}=" H1* GH3"; $resn_exch{"HN21 GH3"}="1HN2 GH3";
$resn_exch{"HN22 GH3"}="2HN2 GH3"; $resn_exch{" H31 GHA"}="1H3  GHA"; $resn_exch{" H32 GHA"}="2H3  GHA";
$resn_exch{" H61 GHA"}="1H6  GHA"; $resn_exch{" H62 GHA"}="2H6  GHA"; $resn_exch{" H71 GHA"}="1H7  GHA";
$resn_exch{" H72 GHA"}="2H7  GHA"; $resn_exch{"H101 GHA"}="1H10 GHA"; $resn_exch{"H102 GHA"}="2H10 GHA";
$resn_exch{" H   GHG"}="1HN  GHG"; $resn_exch{" H2  GHG"}="2HN  GHG"; $resn_exch{" HB2 GHG"}="1HB  GHG";
$resn_exch{" HB3 GHG"}="2HB  GHG"; $resn_exch{"HE21 GHG"}="1HE2 GHG"; $resn_exch{"HE22 GHG"}="2HE2 GHG";
$resn_exch{" H   GHP"}="1HN  GHP"; $resn_exch{" H2  GHP"}="2HN  GHP"; $resn_exch{" HC2 GHP"}=" H2  GHP";
$resn_exch{"H6C1 GI1"}="1H6C GI1"; $resn_exch{"H6C2 GI1"}="2H6C GI1"; $resn_exch{"H9C1 GI1"}="1H9C GI1";
$resn_exch{"H9C2 GI1"}="2H9C GI1"; $resn_exch{"H9C3 GI1"}="3H9C GI1"; $resn_exch{"H1C1 GI2"}="1H1C GI2";
$resn_exch{"H1C2 GI2"}="2H1C GI2"; $resn_exch{"H4C1 GI2"}="1H4C GI2"; $resn_exch{"H4C2 GI2"}="2H4C GI2";
$resn_exch{"H4C3 GI2"}="3H4C GI2"; $resn_exch{"H6B1 GI2"}="1H6B GI2"; $resn_exch{"H6B2 GI2"}="2H6B GI2";
$resn_exch{"H6C1 GI3"}="1H6C GI3"; $resn_exch{"H6C2 GI3"}="2H6C GI3"; $resn_exch{"H1C1 GI4"}="1H1C GI4";
$resn_exch{"H1C2 GI4"}="2H1C GI4"; $resn_exch{"H6B1 GI4"}="1H6B GI4"; $resn_exch{"H6B2 GI4"}="2H6B GI4";
$resn_exch{"H361 GIG"}="1H36 GIG"; $resn_exch{"H362 GIG"}="2H36 GIG"; $resn_exch{"H363 GIG"}="3H36 GIG";
$resn_exch{"H1N1 GIL"}="1H1N GIL"; $resn_exch{"H1N2 GIL"}="2H1N GIL"; $resn_exch{"H171 GIL"}="1H17 GIL";
$resn_exch{"H172 GIL"}="2H17 GIL"; $resn_exch{"H173 GIL"}="3H17 GIL"; $resn_exch{"H181 GIL"}="1H18 GIL";
$resn_exch{"H182 GIL"}="2H18 GIL"; $resn_exch{"H183 GIL"}="3H18 GIL"; $resn_exch{"H191 GIL"}="1H19 GIL";
$resn_exch{"H192 GIL"}="2H19 GIL"; $resn_exch{"H193 GIL"}="3H19 GIL"; $resn_exch{"H201 GIL"}="1H20 GIL";
$resn_exch{"H202 GIL"}="2H20 GIL"; $resn_exch{"H203 GIL"}="3H20 GIL"; $resn_exch{"H9C1 GIL"}="1H9C GIL";
$resn_exch{"H9C2 GIL"}="2H9C GIL"; $resn_exch{"H281 GIL"}="1H28 GIL"; $resn_exch{"H282 GIL"}="2H28 GIL";
$resn_exch{"H283 GIL"}="3H28 GIL"; $resn_exch{"H301 GIL"}="1H30 GIL"; $resn_exch{"H302 GIL"}="2H30 GIL";
$resn_exch{"H303 GIL"}="3H30 GIL"; $resn_exch{"H6C1 GIM"}="1H6C GIM"; $resn_exch{"H6C2 GIM"}="2H6C GIM";
$resn_exch{"H241 GIN"}="1H24 GIN"; $resn_exch{"H242 GIN"}="2H24 GIN"; $resn_exch{"H301 GIN"}="1H30 GIN";
$resn_exch{"H302 GIN"}="2H30 GIN"; $resn_exch{"H311 GIN"}="1H31 GIN"; $resn_exch{"H312 GIN"}="2H31 GIN";
$resn_exch{"H313 GIN"}="3H31 GIN"; $resn_exch{"H321 GIN"}="1H32 GIN"; $resn_exch{"H322 GIN"}="2H32 GIN";
$resn_exch{"HBP1 GIO"}="1HBP GIO"; $resn_exch{"HBP2 GIO"}="2HBP GIO"; $resn_exch{"HGP1 GIO"}="1HGP GIO";
$resn_exch{"HGP2 GIO"}="2HGP GIO"; $resn_exch{"HDP1 GIO"}="1HDP GIO"; $resn_exch{"HDP2 GIO"}="2HDP GIO";
$resn_exch{"HAC1 GIO"}="1HAC GIO"; $resn_exch{"HAC2 GIO"}="2HAC GIO"; $resn_exch{"HOT1 GIP"}="1HOT GIP";
$resn_exch{"H11N GIP"}="1NH1 GIP"; $resn_exch{"H12N GIP"}="2NH1 GIP"; $resn_exch{" HA1 GIP"}="1HA  GIP";
$resn_exch{" HB1 GIP"}="1HB  GIP"; $resn_exch{" HB2 GIP"}="2HB  GIP"; $resn_exch{"HB21 GIP"}="1HB2 GIP";
$resn_exch{"HB22 GIP"}="2HB2 GIP"; $resn_exch{" HG1 GIP"}="1HG  GIP"; $resn_exch{" HG2 GIP"}="2HG  GIP";
$resn_exch{"HN21 GIP"}="1HN2 GIP"; $resn_exch{"HA31 GIP"}="1HA3 GIP"; $resn_exch{"HA32 GIP"}="2HA3 GIP";
$resn_exch{"HO31 GIP"}="1HO3 GIP"; $resn_exch{"H101 GIQ"}="1H10 GIQ"; $resn_exch{"H102 GIQ"}="2H10 GIQ";
$resn_exch{"HN11 GIR"}="1HN1 GIR"; $resn_exch{"HN12 GIR"}="2HN1 GIR"; $resn_exch{" H21 GIR"}="1H2  GIR";
$resn_exch{" H22 GIR"}="2H2  GIR"; $resn_exch{"H111 GIR"}="1H11 GIR"; $resn_exch{"H112 GIR"}="2H11 GIR";
$resn_exch{"H111 GIS"}="1H11 GIS"; $resn_exch{"H112 GIS"}="2H11 GIS"; $resn_exch{"H121 GIS"}="1H12 GIS";
$resn_exch{"H122 GIS"}="2H12 GIS"; $resn_exch{"H123 GIS"}="3H12 GIS"; $resn_exch{"HN31 GIS"}="1HN3 GIS";
$resn_exch{"HN32 GIS"}="2HN3 GIS"; $resn_exch{"H2N1 GKD"}="1H2N GKD"; $resn_exch{"H2N2 GKD"}="2H2N GKD";
$resn_exch{" H1' GKD"}=" H1* GKD"; $resn_exch{" H2' GKD"}=" H2* GKD"; $resn_exch{" H3' GKD"}=" H3* GKD";
$resn_exch{" H4' GKD"}=" H4* GKD"; $resn_exch{"H5'1 GKD"}="1H5* GKD"; $resn_exch{"H5'2 GKD"}="2H5* GKD";
$resn_exch{"H6G1 GKD"}="1H6G GKD"; $resn_exch{"H6G2 GKD"}="2H6G GKD"; $resn_exch{"H2N1 GKE"}="1H2N GKE";
$resn_exch{" HA2 GKE"}="2HA  GKE"; $resn_exch{" H2' GKE"}=" H2* GKE"; $resn_exch{" H3' GKE"}=" H3* GKE";
$resn_exch{" HA1 GKE"}="1HA  GKE"; $resn_exch{" HB2 GKE"}="2HB  GKE"; $resn_exch{" HB1 GKE"}="1HB  GKE";
$resn_exch{" HC2 GKE"}="2HC  GKE"; $resn_exch{" H21 GKR"}="1H2  GKR"; $resn_exch{"HO21 GKR"}="1HO2 GKR";
$resn_exch{" H31 GKR"}="1H3  GKR"; $resn_exch{"HO31 GKR"}="1HO3 GKR"; $resn_exch{" H42 GKR"}="2H4  GKR";
$resn_exch{" H41 GKR"}="1H4  GKR"; $resn_exch{"HO51 GKR"}="1HO5 GKR"; $resn_exch{" H51 GKR"}="1H5  GKR";
$resn_exch{" H61 GL1"}="1H6  GL1"; $resn_exch{" H62 GL1"}="2H6  GL1"; $resn_exch{" H61 GL2"}="1H6  GL2";
$resn_exch{" H62 GL2"}="2H6  GL2"; $resn_exch{"HN31 GL2"}="1HN3 GL2"; $resn_exch{"HN32 GL2"}="2HN3 GL2";
$resn_exch{" HA1 GL3"}="1HA  GL3"; $resn_exch{" HA2 GL3"}="2HA  GL3"; $resn_exch{" H   GL3"}="1HN  GL3";
$resn_exch{" H2  GL3"}="2HN  GL3"; $resn_exch{" H61 GL4"}="1H6  GL4"; $resn_exch{" H62 GL4"}="2H6  GL4";
$resn_exch{" H61 GL5"}="1H6  GL5"; $resn_exch{" H62 GL5"}="2H6  GL5"; $resn_exch{"H151 GL6"}="1H15 GL6";
$resn_exch{"H152 GL6"}="2H15 GL6"; $resn_exch{"H153 GL6"}="3H15 GL6"; $resn_exch{"H161 GL6"}="1H16 GL6";
$resn_exch{"H162 GL6"}="2H16 GL6"; $resn_exch{"H163 GL6"}="3H16 GL6"; $resn_exch{" H61 GL6"}="1H6  GL6";
$resn_exch{" H62 GL6"}="2H6  GL6"; $resn_exch{" H61 GL7"}="1H6  GL7"; $resn_exch{" H62 GL7"}="2H6  GL7";
$resn_exch{" H91 GL7"}="1H9  GL7"; $resn_exch{" H92 GL7"}="2H9  GL7"; $resn_exch{" H93 GL7"}="3H9  GL7";
$resn_exch{"H4C1 GL8"}="1H4C GL8"; $resn_exch{"H4C2 GL8"}="2H4C GL8"; $resn_exch{"H111 GL8"}="1H11 GL8";
$resn_exch{"H112 GL8"}="2H11 GL8"; $resn_exch{"H121 GL8"}="1H12 GL8"; $resn_exch{"H122 GL8"}="2H12 GL8";
$resn_exch{"H161 GL8"}="1H16 GL8"; $resn_exch{"H162 GL8"}="2H16 GL8"; $resn_exch{"H163 GL8"}="3H16 GL8";
$resn_exch{"H171 GL8"}="1H17 GL8"; $resn_exch{"H172 GL8"}="2H17 GL8"; $resn_exch{"H181 GL8"}="1H18 GL8";
$resn_exch{"H182 GL8"}="2H18 GL8"; $resn_exch{"H191 GL8"}="1H19 GL8"; $resn_exch{"H192 GL8"}="2H19 GL8";
$resn_exch{"H201 GL8"}="1H20 GL8"; $resn_exch{"H202 GL8"}="2H20 GL8"; $resn_exch{"H211 GL8"}="1H21 GL8";
$resn_exch{"H212 GL8"}="2H21 GL8"; $resn_exch{"H221 GL8"}="1H22 GL8"; $resn_exch{"H222 GL8"}="2H22 GL8";
$resn_exch{"H231 GL8"}="1H23 GL8"; $resn_exch{"H232 GL8"}="2H23 GL8"; $resn_exch{"H241 GL8"}="1H24 GL8";
$resn_exch{"H242 GL8"}="2H24 GL8"; $resn_exch{" H61 GL9"}="1H6  GL9"; $resn_exch{" H62 GL9"}="2H6  GL9";
$resn_exch{"H101 GL9"}="1H10 GL9"; $resn_exch{"H102 GL9"}="2H10 GL9"; $resn_exch{"H103 GL9"}="3H10 GL9";
$resn_exch{" H61 GLA"}="1H6  GLA"; $resn_exch{" H62 GLA"}="2H6  GLA"; $resn_exch{" H61 GLB"}="1H6  GLB";
$resn_exch{" H62 GLB"}="2H6  GLB"; $resn_exch{" H41 GLD"}="1H4  GLD"; $resn_exch{" H42 GLD"}="2H4  GLD";
$resn_exch{" H61 GLD"}="1H6  GLD"; $resn_exch{" H62 GLD"}="2H6  GLD"; $resn_exch{" H63 GLD"}="3H6  GLD";
$resn_exch{" H11 GLE"}="1H1  GLE"; $resn_exch{" H12 GLE"}="2H1  GLE"; $resn_exch{" H31 GLE"}="1H3  GLE";
$resn_exch{" H32 GLE"}="2H3  GLE"; $resn_exch{"H311 GLE"}="1H31 GLE"; $resn_exch{"H312 GLE"}="2H31 GLE";
$resn_exch{"H2P1 GLE"}="1H2P GLE"; $resn_exch{"H2P2 GLE"}="2H2P GLE"; $resn_exch{"H2P3 GLE"}="3H2P GLE";
$resn_exch{"H111 GLE"}="1H11 GLE"; $resn_exch{"H112 GLE"}="2H11 GLE"; $resn_exch{"H121 GLE"}="1H12 GLE";
$resn_exch{"H122 GLE"}="2H12 GLE"; $resn_exch{"H131 GLE"}="1H13 GLE"; $resn_exch{"H132 GLE"}="2H13 GLE";
$resn_exch{"H141 GLE"}="1H14 GLE"; $resn_exch{"H142 GLE"}="2H14 GLE"; $resn_exch{"H151 GLE"}="1H15 GLE";
$resn_exch{"H152 GLE"}="2H15 GLE"; $resn_exch{"H161 GLE"}="1H16 GLE"; $resn_exch{"H162 GLE"}="2H16 GLE";
$resn_exch{"H171 GLE"}="1H17 GLE"; $resn_exch{"H172 GLE"}="2H17 GLE"; $resn_exch{"H181 GLE"}="1H18 GLE";
$resn_exch{"H182 GLE"}="2H18 GLE"; $resn_exch{"H191 GLE"}="1H19 GLE"; $resn_exch{"H192 GLE"}="2H19 GLE";
$resn_exch{"H201 GLE"}="1H20 GLE"; $resn_exch{"H202 GLE"}="2H20 GLE"; $resn_exch{"H203 GLE"}="3H20 GLE";
$resn_exch{" H61 GLF"}="1H6  GLF"; $resn_exch{" H62 GLF"}="2H6  GLF"; $resn_exch{" H61 GLG"}="1H6  GLG";
$resn_exch{" H62 GLG"}="2H6  GLG"; $resn_exch{" HN1 GLG"}="1HN  GLG"; $resn_exch{" HN2 GLG"}="2HN  GLG";
$resn_exch{" H   GLH"}="1HN  GLH"; $resn_exch{" H2  GLH"}="2HN  GLH"; $resn_exch{" HB2 GLH"}="1HCB GLH";
$resn_exch{" HB3 GLH"}="2HCB GLH"; $resn_exch{" HG2 GLH"}="1HCG GLH"; $resn_exch{" HG3 GLH"}="2HCG GLH";
$resn_exch{"HC31 GLH"}="1HC3 GLH"; $resn_exch{"HC32 GLH"}="2HC3 GLH"; $resn_exch{"HC71 GLH"}="1HC7 GLH";
$resn_exch{"HC72 GLH"}="2HC7 GLH"; $resn_exch{"HC41 GLH"}="1HC4 GLH"; $resn_exch{"HC42 GLH"}="2HC4 GLH";
$resn_exch{"HC51 GLH"}="1HC5 GLH"; $resn_exch{"HC52 GLH"}="2HC5 GLH"; $resn_exch{"HC61 GLH"}="1HC6 GLH";
$resn_exch{"HC62 GLH"}="2HC6 GLH"; $resn_exch{"HC91 GLH"}="1HC9 GLH"; $resn_exch{"HC92 GLH"}="2HC9 GLH";
$resn_exch{"H131 GLH"}="1H13 GLH"; $resn_exch{"H132 GLH"}="2H13 GLH"; $resn_exch{"H101 GLH"}="1H10 GLH";
$resn_exch{"H102 GLH"}="2H10 GLH"; $resn_exch{"H111 GLH"}="1H11 GLH"; $resn_exch{"H112 GLH"}="2H11 GLH";
$resn_exch{"H121 GLH"}="1H12 GLH"; $resn_exch{"H122 GLH"}="2H12 GLH"; $resn_exch{" HN1 GLI"}="1HN  GLI";
$resn_exch{"HN2A GLI"}="2HN  GLI"; $resn_exch{"HB11 GLI"}="1HB1 GLI"; $resn_exch{"HB12 GLI"}="2HB1 GLI";
$resn_exch{"HB13 GLI"}="3HB1 GLI"; $resn_exch{"HB21 GLI"}="1HB2 GLI"; $resn_exch{"HB22 GLI"}="2HB2 GLI";
$resn_exch{"HB23 GLI"}="3HB2 GLI"; $resn_exch{"HB31 GLI"}="1HB3 GLI"; $resn_exch{"HB32 GLI"}="2HB3 GLI";
$resn_exch{"HD31 GLI"}="1HD3 GLI"; $resn_exch{"HD32 GLI"}="2HD3 GLI"; $resn_exch{"HE31 GLI"}="1HE3 GLI";
$resn_exch{"HE32 GLI"}="2HE3 GLI"; $resn_exch{"HX41 GLI"}="1HX4 GLI"; $resn_exch{"HX42 GLI"}="2HX4 GLI";
$resn_exch{"HA41 GLI"}="1HA4 GLI"; $resn_exch{"HA42 GLI"}="2HA4 GLI"; $resn_exch{"H511 GLI"}="1H51 GLI";
$resn_exch{"H512 GLI"}="2H51 GLI"; $resn_exch{"H513 GLI"}="3H51 GLI"; $resn_exch{"H521 GLI"}="1H52 GLI";
$resn_exch{"H522 GLI"}="2H52 GLI"; $resn_exch{"H523 GLI"}="3H52 GLI"; $resn_exch{"H611 GLI"}="1H61 GLI";
$resn_exch{"H612 GLI"}="2H61 GLI"; $resn_exch{"H613 GLI"}="3H61 GLI"; $resn_exch{"H621 GLI"}="1H62 GLI";
$resn_exch{"H622 GLI"}="2H62 GLI"; $resn_exch{"H623 GLI"}="3H62 GLI"; $resn_exch{"HM61 GLI"}="1HM6 GLI";
$resn_exch{"HM62 GLI"}="2HM6 GLI"; $resn_exch{"HM63 GLI"}="3HM6 GLI"; $resn_exch{"HN1' GLL"}="'HN1 GLL";
$resn_exch{"HN2' GLL"}="'HN2 GLL"; $resn_exch{" H   GLM"}="1HN  GLM"; $resn_exch{" H2  GLM"}="2HN  GLM";
$resn_exch{" HA1 GLM"}="1HA  GLM"; $resn_exch{" HA2 GLM"}="2HA  GLM"; $resn_exch{" HM1 GLM"}="1HM  GLM";
$resn_exch{" HM2 GLM"}="2HM  GLM"; $resn_exch{" HM3 GLM"}="3HM  GLM"; $resn_exch{" H2  GLN"}=" HN2 GLN";
$resn_exch{" HB2 GLN"}="1HB  GLN"; $resn_exch{" HB3 GLN"}="2HB  GLN"; $resn_exch{" HG2 GLN"}="1HG  GLN";
$resn_exch{" HG3 GLN"}="2HG  GLN"; $resn_exch{"HE21 GLN"}="1HE2 GLN"; $resn_exch{"HE22 GLN"}="2HE2 GLN";
$resn_exch{" H61 GLO"}="1H6  GLO"; $resn_exch{" H62 GLO"}="2H6  GLO"; $resn_exch{" H61 GLP"}="1H6  GLP";
$resn_exch{" H62 GLP"}="2H6  GLP"; $resn_exch{"HN21 GLP"}="1HN2 GLP"; $resn_exch{"HN22 GLP"}="2HN2 GLP";
$resn_exch{"HOP2 GLP"}="2HOP GLP"; $resn_exch{"HOP3 GLP"}="3HOP GLP"; $resn_exch{" H2  GLQ"}=" HN2 GLQ";
$resn_exch{" HB1 GLQ"}="1HB  GLQ"; $resn_exch{" HB2 GLQ"}="2HB  GLQ"; $resn_exch{" HG1 GLQ"}="1HG  GLQ";
$resn_exch{" HG2 GLQ"}="2HG  GLQ"; $resn_exch{" H21 GLR"}="1H2  GLR"; $resn_exch{"HO21 GLR"}="1HO2 GLR";
$resn_exch{" H31 GLR"}="1H3  GLR"; $resn_exch{"HO31 GLR"}="1HO3 GLR"; $resn_exch{" H41 GLR"}="1H4  GLR";
$resn_exch{" H42 GLR"}="2H4  GLR"; $resn_exch{" H61 GLS"}="1H6  GLS"; $resn_exch{" H62 GLS"}="2H6  GLS";
$resn_exch{" H61 GLT"}="1H6  GLT"; $resn_exch{" H62 GLT"}="2H6  GLT"; $resn_exch{" H2  GLU"}=" HN2 GLU";
$resn_exch{" HB2 GLU"}="1HB  GLU"; $resn_exch{" HB3 GLU"}="2HB  GLU"; $resn_exch{" HG2 GLU"}="1HG  GLU";
$resn_exch{" HG3 GLU"}="2HG  GLU"; $resn_exch{" H61 GLW"}="1H6  GLW"; $resn_exch{" H62 GLW"}="2H6  GLW";
$resn_exch{" H63 GLW"}="3H6  GLW"; $resn_exch{" HB1 GLX"}="1HB  GLX"; $resn_exch{" HB2 GLX"}="2HB  GLX";
$resn_exch{" HG1 GLX"}="1HG  GLX"; $resn_exch{" HG2 GLX"}="2HG  GLX"; $resn_exch{" H2  GLY"}=" HN2 GLY";
$resn_exch{" HA2 GLY"}="1HA  GLY"; $resn_exch{" HA3 GLY"}="2HA  GLY"; $resn_exch{" H2  GLZ"}=" HN2 GLZ";
$resn_exch{" HA1 GLZ"}="1HA  GLZ"; $resn_exch{" HA2 GLZ"}="2HA  GLZ"; $resn_exch{"HN11 GM1"}="1HN1 GM1";
$resn_exch{"HN12 GM1"}="2HN1 GM1"; $resn_exch{" H21 GM1"}="1H2  GM1"; $resn_exch{" H22 GM1"}="2H2  GM1";
$resn_exch{"HN21 GM1"}="1HN2 GM1"; $resn_exch{"HN22 GM1"}="2HN2 GM1"; $resn_exch{"HB11 GM2"}="1HB1 GM2";
$resn_exch{"HB12 GM2"}="2HB1 GM2"; $resn_exch{"H6C1 GM2"}="1H6C GM2"; $resn_exch{"H6C2 GM2"}="2H6C GM2";
$resn_exch{"HL61 GM2"}="1HL6 GM2"; $resn_exch{"HL62 GM2"}="2HL6 GM2"; $resn_exch{"HL51 GM2"}="1HL5 GM2";
$resn_exch{"HL52 GM2"}="2HL5 GM2"; $resn_exch{"HL71 GM2"}="1HL7 GM2"; $resn_exch{"HL72 GM2"}="2HL7 GM2";
$resn_exch{"HL41 GM2"}="1HL4 GM2"; $resn_exch{"HL42 GM2"}="2HL4 GM2"; $resn_exch{"HL31 GM2"}="1HL3 GM2";
$resn_exch{"HL32 GM2"}="2HL3 GM2"; $resn_exch{"HL21 GM2"}="1HL2 GM2"; $resn_exch{"HL22 GM2"}="2HL2 GM2";
$resn_exch{"HR21 GM2"}="1HR2 GM2"; $resn_exch{"HR22 GM2"}="2HR2 GM2"; $resn_exch{"HR31 GM2"}="1HR3 GM2";
$resn_exch{"HR32 GM2"}="2HR3 GM2"; $resn_exch{"HR41 GM2"}="1HR4 GM2"; $resn_exch{"HR42 GM2"}="2HR4 GM2";
$resn_exch{"HR51 GM2"}="1HR5 GM2"; $resn_exch{"HR52 GM2"}="2HR5 GM2"; $resn_exch{"HR61 GM2"}="1HR6 GM2";
$resn_exch{"HR62 GM2"}="2HR6 GM2"; $resn_exch{"HR71 GM2"}="1HR7 GM2"; $resn_exch{"HR72 GM2"}="2HR7 GM2";
$resn_exch{"HR81 GM2"}="1HR8 GM2"; $resn_exch{"HR82 GM2"}="2HR8 GM2"; $resn_exch{"HR91 GM2"}="1HR9 GM2";
$resn_exch{"HR92 GM2"}="2HR9 GM2"; $resn_exch{"HR11 GM2"}="1HR1 GM2"; $resn_exch{"HL81 GM2"}="1HL8 GM2";
$resn_exch{"HL82 GM2"}="2HL8 GM2"; $resn_exch{"HL91 GM2"}="1HL9 GM2"; $resn_exch{"HL92 GM2"}="2HL9 GM2";
$resn_exch{"HL11 GM2"}="1HL1 GM2"; $resn_exch{"HL12 GM2"}="2HL1 GM2"; $resn_exch{"HBC1 GM3"}="1HBC GM3";
$resn_exch{"HBC2 GM3"}="2HBC GM3"; $resn_exch{"HBD1 GM3"}="1HBD GM3"; $resn_exch{"HBD2 GM3"}="2HBD GM3";
$resn_exch{"HBE1 GM3"}="1HBE GM3"; $resn_exch{"HBE2 GM3"}="2HBE GM3"; $resn_exch{"HBF1 GM3"}="1HBF GM3";
$resn_exch{"HBF2 GM3"}="2HBF GM3"; $resn_exch{"HBG1 GM3"}="1HBG GM3"; $resn_exch{"HBG2 GM3"}="2HBG GM3";
$resn_exch{"HBH1 GM3"}="1HBH GM3"; $resn_exch{"HBH2 GM3"}="2HBH GM3"; $resn_exch{"HBI1 GM3"}="1HBI GM3";
$resn_exch{"HBI2 GM3"}="2HBI GM3"; $resn_exch{"HBJ1 GM3"}="1HBJ GM3"; $resn_exch{"HBJ2 GM3"}="2HBJ GM3";
$resn_exch{"HBK1 GM3"}="1HBK GM3"; $resn_exch{"HBK2 GM3"}="2HBK GM3"; $resn_exch{"HBL1 GM3"}="1HBL GM3";
$resn_exch{"HBL2 GM3"}="2HBL GM3"; $resn_exch{"HBM1 GM3"}="1HBM GM3"; $resn_exch{"HBM2 GM3"}="2HBM GM3";
$resn_exch{"HBN1 GM3"}="1HBN GM3"; $resn_exch{"HBN2 GM3"}="2HBN GM3"; $resn_exch{"HBO1 GM3"}="1HBO GM3";
$resn_exch{"HBO2 GM3"}="2HBO GM3"; $resn_exch{"HBP1 GM3"}="1HBP GM3"; $resn_exch{"HBP2 GM3"}="2HBP GM3";
$resn_exch{"HBQ1 GM3"}="1HBQ GM3"; $resn_exch{"HBQ2 GM3"}="2HBQ GM3"; $resn_exch{"HB11 GM3"}="1HB1 GM3";
$resn_exch{"HB12 GM3"}="2HB1 GM3"; $resn_exch{"HB21 GM3"}="1HB2 GM3"; $resn_exch{"HB22 GM3"}="2HB2 GM3";
$resn_exch{"HB23 GM3"}="3HB2 GM3"; $resn_exch{"HAX1 GM3"}="1HAX GM3"; $resn_exch{"HAX2 GM3"}="2HAX GM3";
$resn_exch{"HAV1 GM3"}="1HAV GM3"; $resn_exch{"HAV2 GM3"}="2HAV GM3"; $resn_exch{"HBS1 GM3"}="1HBS GM3";
$resn_exch{"HBS2 GM3"}="2HBS GM3"; $resn_exch{"HBU1 GM3"}="1HBU GM3"; $resn_exch{"HBU2 GM3"}="2HBU GM3";
$resn_exch{"HBV1 GM3"}="1HBV GM3"; $resn_exch{"HBV2 GM3"}="2HBV GM3"; $resn_exch{"HBW1 GM3"}="1HBW GM3";
$resn_exch{"HBW2 GM3"}="2HBW GM3"; $resn_exch{"HBX1 GM3"}="1HBX GM3"; $resn_exch{"HBX2 GM3"}="2HBX GM3";
$resn_exch{"HBY1 GM3"}="1HBY GM3"; $resn_exch{"HBY2 GM3"}="2HBY GM3"; $resn_exch{"HBZ1 GM3"}="1HBZ GM3";
$resn_exch{"HBZ2 GM3"}="2HBZ GM3"; $resn_exch{"HCA1 GM3"}="1HCA GM3"; $resn_exch{"HCA2 GM3"}="2HCA GM3";
$resn_exch{"HCB1 GM3"}="1HCB GM3"; $resn_exch{"HCB2 GM3"}="2HCB GM3"; $resn_exch{"HCC1 GM3"}="1HCC GM3";
$resn_exch{"HCC2 GM3"}="2HCC GM3"; $resn_exch{"HCD1 GM3"}="1HCD GM3"; $resn_exch{"HCD2 GM3"}="2HCD GM3";
$resn_exch{"HCE1 GM3"}="1HCE GM3"; $resn_exch{"HCE2 GM3"}="2HCE GM3"; $resn_exch{"HCF1 GM3"}="1HCF GM3";
$resn_exch{"HCF2 GM3"}="2HCF GM3"; $resn_exch{"HCG1 GM3"}="1HCG GM3"; $resn_exch{"HCG2 GM3"}="2HCG GM3";
$resn_exch{"HCH1 GM3"}="1HCH GM3"; $resn_exch{"HCH2 GM3"}="2HCH GM3"; $resn_exch{"HCI1 GM3"}="1HCI GM3";
$resn_exch{"HCI2 GM3"}="2HCI GM3"; $resn_exch{"HCJ1 GM3"}="1HCJ GM3"; $resn_exch{"HCJ2 GM3"}="2HCJ GM3";
$resn_exch{"HCK1 GM3"}="1HCK GM3"; $resn_exch{"HCK2 GM3"}="2HCK GM3"; $resn_exch{"HCL1 GM3"}="1HCL GM3";
$resn_exch{"HCL2 GM3"}="2HCL GM3"; $resn_exch{"HCM1 GM3"}="1HCM GM3"; $resn_exch{"HCM2 GM3"}="2HCM GM3";
$resn_exch{"HCN1 GM3"}="1HCN GM3"; $resn_exch{"HCN2 GM3"}="2HCN GM3"; $resn_exch{"HCO1 GM3"}="1HCO GM3";
$resn_exch{"HCO2 GM3"}="2HCO GM3"; $resn_exch{"HCP1 GM3"}="1HCP GM3"; $resn_exch{"HCP2 GM3"}="2HCP GM3";
$resn_exch{"HCP3 GM3"}="3HCP GM3"; $resn_exch{"HAC1 GM6"}="1HAC GM6"; $resn_exch{"HAC2 GM6"}="2HAC GM6";
$resn_exch{"HBA1 GM6"}="1HBA GM6"; $resn_exch{"HBA2 GM6"}="2HBA GM6"; $resn_exch{"HBA3 GM6"}="3HBA GM6";
$resn_exch{"HAO1 GM6"}="1HAO GM6"; $resn_exch{"HAO2 GM6"}="2HAO GM6"; $resn_exch{"HAA1 GM6"}="1HAA GM6";
$resn_exch{"HAA2 GM6"}="2HAA GM6"; $resn_exch{"HAJ1 GM6"}="1HAJ GM6"; $resn_exch{"HAJ2 GM6"}="2HAJ GM6";
$resn_exch{"HAJ3 GM6"}="3HAJ GM6"; $resn_exch{"HAI1 GM6"}="1HAI GM6"; $resn_exch{"HAI2 GM6"}="2HAI GM6";
$resn_exch{"HAI3 GM6"}="3HAI GM6"; $resn_exch{" H2  GMA"}=" HN2 GMA"; $resn_exch{" HB2 GMA"}="1HB  GMA";
$resn_exch{" HB3 GMA"}="2HB  GMA"; $resn_exch{" HG2 GMA"}="1HG  GMA"; $resn_exch{" HG3 GMA"}="2HG  GMA";
$resn_exch{" HE2 GMA"}=" HO2 GMA"; $resn_exch{"HN2A GMA"}="2HN  GMA"; $resn_exch{" HN1 GMA"}="1HN  GMA";
$resn_exch{"H101 GMC"}="1H10 GMC"; $resn_exch{"H102 GMC"}="2H10 GMC"; $resn_exch{"H103 GMC"}="3H10 GMC";
$resn_exch{" HC1 GMC"}="1HC  GMC"; $resn_exch{" HC2 GMC"}="2HC  GMC"; $resn_exch{" HC3 GMC"}="3HC  GMC";
$resn_exch{" H1' GMC"}=" H1* GMC"; $resn_exch{" H4' GMC"}=" H4* GMC"; $resn_exch{"H5'1 GMC"}="1H5* GMC";
$resn_exch{"H5'2 GMC"}="2H5* GMC"; $resn_exch{" H5' GMC"}=" H5* GMC"; $resn_exch{" H2' GMC"}=" H2* GMC";
$resn_exch{" H3' GMC"}=" H3* GMC"; $resn_exch{"H3'1 GMC"}="1H3* GMC"; $resn_exch{"H3'2 GMC"}="2H3* GMC";
$resn_exch{" H71 GMH"}="1H7  GMH"; $resn_exch{" H72 GMH"}="2H7  GMH"; $resn_exch{"HAH1 GMM"}="1HAH GMM";
$resn_exch{"HAH2 GMM"}="2HAH GMM"; $resn_exch{"HAH3 GMM"}="3HAH GMM"; $resn_exch{"HAI1 GMM"}="1HAI GMM";
$resn_exch{"HAI2 GMM"}="2HAI GMM"; $resn_exch{"HAJ1 GMM"}="1HAJ GMM"; $resn_exch{"HAJ2 GMM"}="2HAJ GMM";
$resn_exch{"HAK1 GMM"}="1HAK GMM"; $resn_exch{"HAK2 GMM"}="2HAK GMM"; $resn_exch{"HAL1 GMM"}="1HAL GMM";
$resn_exch{"HAL2 GMM"}="2HAL GMM"; $resn_exch{"HAM1 GMM"}="1HAM GMM"; $resn_exch{"HAM2 GMM"}="2HAM GMM";
$resn_exch{"HAN1 GMM"}="1HAN GMM"; $resn_exch{"HAN2 GMM"}="2HAN GMM"; $resn_exch{"HAO1 GMM"}="1HAO GMM";
$resn_exch{"HAO2 GMM"}="2HAO GMM"; $resn_exch{"HAP1 GMM"}="1HAP GMM"; $resn_exch{"HAP2 GMM"}="2HAP GMM";
$resn_exch{"HAQ1 GMM"}="1HAQ GMM"; $resn_exch{"HAQ2 GMM"}="2HAQ GMM"; $resn_exch{"HAR1 GMM"}="1HAR GMM";
$resn_exch{"HAR2 GMM"}="2HAR GMM"; $resn_exch{"HAS1 GMM"}="1HAS GMM"; $resn_exch{"HAS2 GMM"}="2HAS GMM";
$resn_exch{"HAT1 GMM"}="1HAT GMM"; $resn_exch{"HAT2 GMM"}="2HAT GMM"; $resn_exch{"HAU1 GMM"}="1HAU GMM";
$resn_exch{"HAU2 GMM"}="2HAU GMM"; $resn_exch{"HAV1 GMM"}="1HAV GMM"; $resn_exch{"HAV2 GMM"}="2HAV GMM";
$resn_exch{"HAW1 GMM"}="1HAW GMM"; $resn_exch{"HAW2 GMM"}="2HAW GMM"; $resn_exch{"HAX1 GMM"}="1HAX GMM";
$resn_exch{"HAX2 GMM"}="2HAX GMM"; $resn_exch{"HAY1 GMM"}="1HAY GMM"; $resn_exch{"HAY2 GMM"}="2HAY GMM";
$resn_exch{"HBB1 GMM"}="1HBB GMM"; $resn_exch{"HBB2 GMM"}="2HBB GMM"; $resn_exch{"HBC1 GMM"}="1HBC GMM";
$resn_exch{"HBC2 GMM"}="2HBC GMM"; $resn_exch{"HBD1 GMM"}="1HBD GMM"; $resn_exch{"HBD2 GMM"}="2HBD GMM";
$resn_exch{"HBE1 GMM"}="1HBE GMM"; $resn_exch{"HBE2 GMM"}="2HBE GMM"; $resn_exch{"HBF1 GMM"}="1HBF GMM";
$resn_exch{"HBF2 GMM"}="2HBF GMM"; $resn_exch{"HBG1 GMM"}="1HBG GMM"; $resn_exch{"HBG2 GMM"}="2HBG GMM";
$resn_exch{"HBH1 GMM"}="1HBH GMM"; $resn_exch{"HBH2 GMM"}="2HBH GMM"; $resn_exch{"HBI1 GMM"}="1HBI GMM";
$resn_exch{"HBI2 GMM"}="2HBI GMM"; $resn_exch{"HBL1 GMM"}="1HBL GMM"; $resn_exch{"HBL2 GMM"}="2HBL GMM";
$resn_exch{"HBM1 GMM"}="1HBM GMM"; $resn_exch{"HBM2 GMM"}="2HBM GMM"; $resn_exch{"HBN1 GMM"}="1HBN GMM";
$resn_exch{"HBN2 GMM"}="2HBN GMM"; $resn_exch{"HBO1 GMM"}="1HBO GMM"; $resn_exch{"HBO2 GMM"}="2HBO GMM";
$resn_exch{"HBP1 GMM"}="1HBP GMM"; $resn_exch{"HBP2 GMM"}="2HBP GMM"; $resn_exch{"HBQ1 GMM"}="1HBQ GMM";
$resn_exch{"HBQ2 GMM"}="2HBQ GMM"; $resn_exch{"HBR1 GMM"}="1HBR GMM"; $resn_exch{"HBR2 GMM"}="2HBR GMM";
$resn_exch{"HBS1 GMM"}="1HBS GMM"; $resn_exch{"HBS2 GMM"}="2HBS GMM"; $resn_exch{"HBT1 GMM"}="1HBT GMM";
$resn_exch{"HBT2 GMM"}="2HBT GMM"; $resn_exch{"HBU1 GMM"}="1HBU GMM"; $resn_exch{"HBU2 GMM"}="2HBU GMM";
$resn_exch{"HBV1 GMM"}="1HBV GMM"; $resn_exch{"HBV2 GMM"}="2HBV GMM"; $resn_exch{"HBW1 GMM"}="1HBW GMM";
$resn_exch{"HBW2 GMM"}="2HBW GMM"; $resn_exch{"HBX1 GMM"}="1HBX GMM"; $resn_exch{"HBX2 GMM"}="2HBX GMM";
$resn_exch{"HBY1 GMM"}="1HBY GMM"; $resn_exch{"HBY2 GMM"}="2HBY GMM"; $resn_exch{"HBZ1 GMM"}="1HBZ GMM";
$resn_exch{"HBZ2 GMM"}="2HBZ GMM"; $resn_exch{"HCA1 GMM"}="1HCA GMM"; $resn_exch{"HCA2 GMM"}="2HCA GMM";
$resn_exch{"HCB1 GMM"}="1HCB GMM"; $resn_exch{"HCB2 GMM"}="2HCB GMM"; $resn_exch{"HCC1 GMM"}="1HCC GMM";
$resn_exch{"HCC2 GMM"}="2HCC GMM"; $resn_exch{"HCD1 GMM"}="1HCD GMM"; $resn_exch{"HCD2 GMM"}="2HCD GMM";
$resn_exch{"HCH1 GMM"}="1HCH GMM"; $resn_exch{"HCH2 GMM"}="2HCH GMM"; $resn_exch{"HCI1 GMM"}="1HCI GMM";
$resn_exch{"HCI2 GMM"}="2HCI GMM"; $resn_exch{"HCJ1 GMM"}="1HCJ GMM"; $resn_exch{"HCJ2 GMM"}="2HCJ GMM";
$resn_exch{"HCK1 GMM"}="1HCK GMM"; $resn_exch{"HCK2 GMM"}="2HCK GMM"; $resn_exch{"HCL1 GMM"}="1HCL GMM";
$resn_exch{"HCL2 GMM"}="2HCL GMM"; $resn_exch{"HCM1 GMM"}="1HCM GMM"; $resn_exch{"HCM2 GMM"}="2HCM GMM";
$resn_exch{"HCN1 GMM"}="1HCN GMM"; $resn_exch{"HCN2 GMM"}="2HCN GMM"; $resn_exch{"HCO1 GMM"}="1HCO GMM";
$resn_exch{"HCO2 GMM"}="2HCO GMM"; $resn_exch{"HCO3 GMM"}="3HCO GMM"; $resn_exch{"HCT1 GMM"}="1HCT GMM";
$resn_exch{"HCT2 GMM"}="2HCT GMM"; $resn_exch{" H11 GMN"}="1H1  GMN"; $resn_exch{" H12 GMN"}="2H1  GMN";
$resn_exch{" H21 GMN"}="1H2  GMN"; $resn_exch{" H22 GMN"}="2H2  GMN"; $resn_exch{" H41 GMN"}="1H4  GMN";
$resn_exch{" H42 GMN"}="2H4  GMN"; $resn_exch{" H51 GMN"}="1H5  GMN"; $resn_exch{" H52 GMN"}="2H5  GMN";
$resn_exch{" H53 GMN"}="3H5  GMN"; $resn_exch{" H61 GMN"}="1H6  GMN"; $resn_exch{" H62 GMN"}="2H6  GMN";
$resn_exch{" H71 GMN"}="1H7  GMN"; $resn_exch{" H72 GMN"}="2H7  GMN"; $resn_exch{" H73 GMN"}="3H7  GMN";
$resn_exch{" H81 GMN"}="1H8  GMN"; $resn_exch{" H82 GMN"}="2H8  GMN"; $resn_exch{" H91 GMN"}="1H9  GMN";
$resn_exch{" H92 GMN"}="2H9  GMN"; $resn_exch{" H93 GMN"}="3H9  GMN"; $resn_exch{"H101 GMN"}="1H10 GMN";
$resn_exch{"H102 GMN"}="2H10 GMN"; $resn_exch{"H111 GMN"}="1H11 GMN"; $resn_exch{"H112 GMN"}="2H11 GMN";
$resn_exch{"H131 GMN"}="1H13 GMN"; $resn_exch{"H132 GMN"}="2H13 GMN"; $resn_exch{"H141 GMN"}="1H14 GMN";
$resn_exch{"H142 GMN"}="2H14 GMN"; $resn_exch{"H143 GMN"}="3H14 GMN"; $resn_exch{"H151 GMN"}="1H15 GMN";
$resn_exch{"H152 GMN"}="2H15 GMN"; $resn_exch{"H161 GMN"}="1H16 GMN"; $resn_exch{"H162 GMN"}="2H16 GMN";
$resn_exch{"H163 GMN"}="3H16 GMN"; $resn_exch{"H171 GMN"}="1H17 GMN"; $resn_exch{"H172 GMN"}="2H17 GMN";
$resn_exch{"H181 GMN"}="1H18 GMN"; $resn_exch{"H182 GMN"}="2H18 GMN"; $resn_exch{"H183 GMN"}="3H18 GMN";
$resn_exch{"H251 GMN"}="1H25 GMN"; $resn_exch{"H252 GMN"}="2H25 GMN"; $resn_exch{"H261 GMN"}="1H26 GMN";
$resn_exch{"H262 GMN"}="2H26 GMN"; $resn_exch{"H281 GMN"}="1H28 GMN"; $resn_exch{"H282 GMN"}="2H28 GMN";
$resn_exch{"H291 GMN"}="1H29 GMN"; $resn_exch{"H292 GMN"}="2H29 GMN"; $resn_exch{"H293 GMN"}="3H29 GMN";
$resn_exch{"H301 GMN"}="1H30 GMN"; $resn_exch{"H302 GMN"}="2H30 GMN"; $resn_exch{"H311 GMN"}="1H31 GMN";
$resn_exch{"H312 GMN"}="2H31 GMN"; $resn_exch{"H313 GMN"}="3H31 GMN"; $resn_exch{"H321 GMN"}="1H32 GMN";
$resn_exch{"H322 GMN"}="2H32 GMN"; $resn_exch{"H331 GMN"}="1H33 GMN"; $resn_exch{"H332 GMN"}="2H33 GMN";
$resn_exch{"H333 GMN"}="3H33 GMN"; $resn_exch{"HO5' GMP"}="*HO5 GMP"; $resn_exch{"H5'1 GMP"}="1H5* GMP";
$resn_exch{"H5'2 GMP"}="2H5* GMP"; $resn_exch{" H4' GMP"}=" H4* GMP"; $resn_exch{" H3' GMP"}=" H3* GMP";
$resn_exch{"HO3' GMP"}="*HO3 GMP"; $resn_exch{" H2' GMP"}=" H2* GMP"; $resn_exch{"HO2' GMP"}="*HO2 GMP";
$resn_exch{" H1' GMP"}=" H1* GMP"; $resn_exch{"HN21 GMP"}="1HN2 GMP"; $resn_exch{"HN22 GMP"}="2HN2 GMP";
$resn_exch{" H21 GMS"}="1H2  GMS"; $resn_exch{" H22 GMS"}="2H2  GMS"; $resn_exch{" H2' GMS"}="1H2* GMS";
$resn_exch{"H2'' GMS"}="2H2* GMS"; $resn_exch{" H5' GMS"}="1H5* GMS"; $resn_exch{"H5'' GMS"}="2H5* GMS";
$resn_exch{" H4' GMS"}=" H4* GMS"; $resn_exch{" H1' GMS"}=" H1* GMS"; $resn_exch{" H3' GMS"}=" H3* GMS";
$resn_exch{"HO3' GMS"}="H3T  GMS"; $resn_exch{"HOP2 GMS"}="2HOP GMS"; $resn_exch{" H5' GMU"}="1H5* GMU";
$resn_exch{"H5'' GMU"}="2H5* GMU"; $resn_exch{" H4' GMU"}=" H4* GMU"; $resn_exch{" H3' GMU"}=" H3* GMU";
$resn_exch{" H2' GMU"}=" H2* GMU"; $resn_exch{" H1' GMU"}=" H1* GMU"; $resn_exch{" H71 GMU"}="1H5M GMU";
$resn_exch{" H72 GMU"}="2H5M GMU"; $resn_exch{" H73 GMU"}="3H5M GMU"; $resn_exch{"HA'1 GMU"}="1HA* GMU";
$resn_exch{"HA'2 GMU"}="2HA* GMU"; $resn_exch{"HB'1 GMU"}="1HB* GMU"; $resn_exch{"HB'2 GMU"}="2HB* GMU";
$resn_exch{" HC' GMU"}=" HC* GMU"; $resn_exch{" HD' GMU"}=" HD* GMU"; $resn_exch{"HE'1 GMU"}="1HE* GMU";
$resn_exch{"HE'2 GMU"}="2HE* GMU"; $resn_exch{"HF'1 GMU"}="1HF* GMU"; $resn_exch{"HF'2 GMU"}="2HF* GMU";
$resn_exch{" H71 GMY"}="1H7  GMY"; $resn_exch{" H72 GMY"}="2H7  GMY"; $resn_exch{" H91 GMY"}="1H9  GMY";
$resn_exch{" H92 GMY"}="2H9  GMY"; $resn_exch{"H261 GMY"}="1H26 GMY"; $resn_exch{"H262 GMY"}="2H26 GMY";
$resn_exch{"H263 GMY"}="3H26 GMY"; $resn_exch{"H271 GMY"}="1H27 GMY"; $resn_exch{"H272 GMY"}="2H27 GMY";
$resn_exch{"H273 GMY"}="3H27 GMY"; $resn_exch{"H291 GMY"}="1H29 GMY"; $resn_exch{"H292 GMY"}="2H29 GMY";
$resn_exch{"H293 GMY"}="3H29 GMY"; $resn_exch{"H311 GMY"}="1H31 GMY"; $resn_exch{"H312 GMY"}="2H31 GMY";
$resn_exch{"H313 GMY"}="3H31 GMY"; $resn_exch{"H321 GMY"}="1H32 GMY"; $resn_exch{"H322 GMY"}="2H32 GMY";
$resn_exch{"H323 GMY"}="3H32 GMY"; $resn_exch{"HN61 GMY"}="1HN6 GMY"; $resn_exch{"HN62 GMY"}="2HN6 GMY";
$resn_exch{"H381 GMY"}="1H38 GMY"; $resn_exch{"H382 GMY"}="2H38 GMY"; $resn_exch{"H383 GMY"}="3H38 GMY";
$resn_exch{"H391 GMY"}="1H39 GMY"; $resn_exch{"H392 GMY"}="2H39 GMY"; $resn_exch{"H393 GMY"}="3H39 GMY";
$resn_exch{"H6'1 GN1"}="1H6' GN1"; $resn_exch{"H6'2 GN1"}="2H6' GN1"; $resn_exch{"H8'1 GN1"}="1H8' GN1";
$resn_exch{"H8'2 GN1"}="2H8' GN1"; $resn_exch{"H8'3 GN1"}="3H8' GN1"; $resn_exch{"HOP2 GN7"}="2HOP GN7";
$resn_exch{"HOP3 GN7"}="3HOP GN7"; $resn_exch{" H5' GN7"}="1H5* GN7"; $resn_exch{"H5'' GN7"}="2H5* GN7";
$resn_exch{" H4' GN7"}=" H4* GN7"; $resn_exch{" H3' GN7"}=" H3* GN7"; $resn_exch{"HO3' GN7"}="*HO3 GN7";
$resn_exch{" H2' GN7"}="1H2* GN7"; $resn_exch{"H2'' GN7"}="2H2* GN7"; $resn_exch{" H1' GN7"}=" H1* GN7";
$resn_exch{"HN21 GN7"}="1HN2 GN7"; $resn_exch{"HN22 GN7"}="2HN2 GN7"; $resn_exch{"HAO1 GN8"}="1HAO GN8";
$resn_exch{"HAO2 GN8"}="2HAO GN8"; $resn_exch{" H31 GNA"}="1H3  GNA"; $resn_exch{" H32 GNA"}="2H3  GNA";
$resn_exch{" H91 GNA"}="1H9  GNA"; $resn_exch{" H92 GNA"}="2H9  GNA"; $resn_exch{"H111 GNA"}="1H11 GNA";
$resn_exch{"H112 GNA"}="2H11 GNA"; $resn_exch{"H113 GNA"}="3H11 GNA"; $resn_exch{"HN71 GNA"}="1HN7 GNA";
$resn_exch{"HN72 GNA"}="2HN7 GNA"; $resn_exch{"HOB1 GNA"}="1HOB GNA"; $resn_exch{"HOT1 GNB"}="1HOT GNB";
$resn_exch{"H11N GNB"}="1NH1 GNB"; $resn_exch{"H12N GNB"}="2NH1 GNB"; $resn_exch{" HA1 GNB"}="1HA  GNB";
$resn_exch{" HB1 GNB"}="1HB  GNB"; $resn_exch{" HB2 GNB"}="2HB  GNB"; $resn_exch{"HB21 GNB"}="1HB2 GNB";
$resn_exch{"HB22 GNB"}="2HB2 GNB"; $resn_exch{" HG1 GNB"}="1HG  GNB"; $resn_exch{" HG2 GNB"}="2HG  GNB";
$resn_exch{"HN21 GNB"}="1HN2 GNB"; $resn_exch{"HA31 GNB"}="1HA3 GNB"; $resn_exch{"HA32 GNB"}="2HA3 GNB";
$resn_exch{"HO31 GNB"}="1HO3 GNB"; $resn_exch{" HC1 GNB"}="1HC  GNB"; $resn_exch{" HC2 GNB"}="2HC  GNB";
$resn_exch{" HB1 GND"}="1HB  GND"; $resn_exch{" HB2 GND"}="2HB  GND"; $resn_exch{" HG1 GND"}="1HG  GND";
$resn_exch{" HG2 GND"}="2HG  GND"; $resn_exch{" HD1 GND"}="1HD  GND"; $resn_exch{" HD2 GND"}="2HD  GND";
$resn_exch{"HH11 GND"}="1HH1 GND"; $resn_exch{"HH21 GND"}="1HH2 GND"; $resn_exch{"HH22 GND"}="2HH2 GND";
$resn_exch{" HN1 GND"}="1HN  GND"; $resn_exch{" HN2 GND"}="2HN  GND"; $resn_exch{" HN3 GND"}="3HN  GND";
$resn_exch{" H5' GNE"}="1H5* GNE"; $resn_exch{"H5'' GNE"}="2H5* GNE"; $resn_exch{" H4' GNE"}=" H4* GNE";
$resn_exch{" H3' GNE"}=" H3* GNE"; $resn_exch{" H2' GNE"}="1H2* GNE"; $resn_exch{"H2'' GNE"}="2H2* GNE";
$resn_exch{" H1' GNE"}=" H1* GNE"; $resn_exch{"H131 GNF"}="1H13 GNF"; $resn_exch{"H132 GNF"}="2H13 GNF";
$resn_exch{"H191 GNF"}="1H19 GNF"; $resn_exch{"H192 GNF"}="2H19 GNF"; $resn_exch{"H181 GNF"}="1H18 GNF";
$resn_exch{"H182 GNF"}="2H18 GNF"; $resn_exch{"H171 GNF"}="1H17 GNF"; $resn_exch{"H172 GNF"}="2H17 GNF";
$resn_exch{"H161 GNF"}="1H16 GNF"; $resn_exch{"H162 GNF"}="2H16 GNF"; $resn_exch{"H151 GNF"}="1H15 GNF";
$resn_exch{"H152 GNF"}="2H15 GNF"; $resn_exch{" H81 GNF"}="1H8  GNF"; $resn_exch{" H82 GNF"}="2H8  GNF";
$resn_exch{" H71 GNF"}="1H7  GNF"; $resn_exch{" H72 GNF"}="2H7  GNF"; $resn_exch{" H1  GNF"}="1H   GNF";
$resn_exch{" H2  GNF"}="2H   GNF"; $resn_exch{" H3A GNF"}="3H   GNF"; $resn_exch{" H5' GNG"}=" H5* GNG";
$resn_exch{"H5'1 GNG"}="1H5* GNG"; $resn_exch{"H5'2 GNG"}="2H5* GNG"; $resn_exch{" H4' GNG"}=" H4* GNG";
$resn_exch{" H1' GNG"}=" H1* GNG"; $resn_exch{"HN21 GNG"}="1HN2 GNG"; $resn_exch{"HN22 GNG"}="2HN2 GNG";
$resn_exch{"H2'1 GNG"}="1H2* GNG"; $resn_exch{"H2'2 GNG"}="2H2* GNG"; $resn_exch{" H3' GNG"}=" H3* GNG";
$resn_exch{"HN31 GNH"}="1HN3 GNH"; $resn_exch{"HN32 GNH"}="2HN3 GNH"; $resn_exch{"HOB2 GNH"}="2HOB GNH";
$resn_exch{"HOA2 GNH"}="2HOA GNH"; $resn_exch{"H5'2 GNH"}="2H5* GNH"; $resn_exch{"H5'1 GNH"}="1H5* GNH";
$resn_exch{" H4' GNH"}=" H4* GNH"; $resn_exch{" H3' GNH"}=" H3* GNH"; $resn_exch{"HO3' GNH"}="*HO3 GNH";
$resn_exch{" H2' GNH"}=" H2* GNH"; $resn_exch{"HO2' GNH"}="*HO2 GNH"; $resn_exch{" H1' GNH"}=" H1* GNH";
$resn_exch{"HN21 GNH"}="1HN2 GNH"; $resn_exch{"HN22 GNH"}="2HN2 GNH"; $resn_exch{"H8C1 GNI"}="1H8C GNI";
$resn_exch{"H8C2 GNI"}="2H8C GNI"; $resn_exch{"H7C1 GNI"}="1H7C GNI"; $resn_exch{"H7C2 GNI"}="2H7C GNI";
$resn_exch{"H2C1 GNI"}="1H2C GNI"; $resn_exch{"H2C2 GNI"}="2H2C GNI"; $resn_exch{"HOG2 GNP"}="2HOG GNP";
$resn_exch{"HOG3 GNP"}="3HOG GNP"; $resn_exch{"HNB3 GNP"}="3HNB GNP"; $resn_exch{"HOB2 GNP"}="2HOB GNP";
$resn_exch{"HOA2 GNP"}="2HOA GNP"; $resn_exch{"H5'2 GNP"}="2H5* GNP"; $resn_exch{"H5'1 GNP"}="1H5* GNP";
$resn_exch{" H4' GNP"}=" H4* GNP"; $resn_exch{" H3' GNP"}=" H3* GNP"; $resn_exch{"HO3' GNP"}="*HO3 GNP";
$resn_exch{" H2' GNP"}=" H2* GNP"; $resn_exch{"HO2' GNP"}="*HO2 GNP"; $resn_exch{" H1' GNP"}=" H1* GNP";
$resn_exch{"HN21 GNP"}="1HN2 GNP"; $resn_exch{"HN22 GNP"}="2HN2 GNP"; $resn_exch{"H171 GNQ"}="1H17 GNQ";
$resn_exch{"H172 GNQ"}="2H17 GNQ"; $resn_exch{"H161 GNQ"}="1H16 GNQ"; $resn_exch{"H162 GNQ"}="2H16 GNQ";
$resn_exch{"H151 GNQ"}="1H15 GNQ"; $resn_exch{"H152 GNQ"}="2H15 GNQ"; $resn_exch{"H141 GNQ"}="1H14 GNQ";
$resn_exch{"H142 GNQ"}="2H14 GNQ"; $resn_exch{"H131 GNQ"}="1H13 GNQ"; $resn_exch{"H132 GNQ"}="2H13 GNQ";
$resn_exch{"H121 GNQ"}="1H12 GNQ"; $resn_exch{"H122 GNQ"}="2H12 GNQ"; $resn_exch{"H101 GNQ"}="1H10 GNQ";
$resn_exch{"H102 GNQ"}="2H10 GNQ"; $resn_exch{"H103 GNQ"}="3H10 GNQ"; $resn_exch{" H81 GNQ"}="1H8  GNQ";
$resn_exch{" H82 GNQ"}="2H8  GNQ"; $resn_exch{"HAK1 GNR"}="1HAK GNR"; $resn_exch{"HAK2 GNR"}="2HAK GNR";
$resn_exch{" H61 GNS"}="1H6  GNS"; $resn_exch{" H62 GNS"}="2H6  GNS"; $resn_exch{"H41A GNT"}="1H4  GNT";
$resn_exch{" H42 GNT"}="2H4  GNT"; $resn_exch{" H91 GNT"}="1H9  GNT"; $resn_exch{" H92 GNT"}="2H9  GNT";
$resn_exch{"H111 GNT"}="1H11 GNT"; $resn_exch{"H112 GNT"}="2H11 GNT"; $resn_exch{"H121 GNT"}="1H12 GNT";
$resn_exch{"H122 GNT"}="2H12 GNT"; $resn_exch{"H161 GNT"}="1H16 GNT"; $resn_exch{"H162 GNT"}="2H16 GNT";
$resn_exch{"H163 GNT"}="3H16 GNT"; $resn_exch{"H191 GNT"}="1H19 GNT"; $resn_exch{"H192 GNT"}="2H19 GNT";
$resn_exch{"H193 GNT"}="3H19 GNT"; $resn_exch{" H22 GOA"}="2H2  GOA"; $resn_exch{" H21 GOA"}="1H2  GOA";
$resn_exch{" H11 GOL"}="1H1  GOL"; $resn_exch{" H12 GOL"}="2H1  GOL"; $resn_exch{" H31 GOL"}="1H3  GOL";
$resn_exch{" H32 GOL"}="2H3  GOL"; $resn_exch{" HC1 GOM"}="1HC  GOM"; $resn_exch{" HC2 GOM"}="2HC  GOM";
$resn_exch{" HB1 GOM"}="1HB  GOM"; $resn_exch{" HB2 GOM"}="2HB  GOM"; $resn_exch{" HG1 GOM"}="1HG  GOM";
$resn_exch{" HG2 GOM"}="2HG  GOM"; $resn_exch{" H5' GOM"}="1H5* GOM"; $resn_exch{"H5'' GOM"}="2H5* GOM";
$resn_exch{" H4' GOM"}=" H4* GOM"; $resn_exch{" H1' GOM"}=" H1* GOM"; $resn_exch{"HN61 GOM"}="1HN6 GOM";
$resn_exch{"HN62 GOM"}="2HN6 GOM"; $resn_exch{" H2' GOM"}=" H2* GOM"; $resn_exch{"HO2' GOM"}="*HO2 GOM";
$resn_exch{" H3' GOM"}=" H3* GOM"; $resn_exch{"HO3' GOM"}="*HO3 GOM"; $resn_exch{"HC61 GOX"}="1HC6 GOX";
$resn_exch{"HC62 GOX"}="2HC6 GOX"; $resn_exch{" H61 GP1"}="1H6  GP1"; $resn_exch{" H62 GP1"}="2H6  GP1";
$resn_exch{"HN21 GP1"}="1HN2 GP1"; $resn_exch{"HN22 GP1"}="2HN2 GP1"; $resn_exch{"HOP1 GP1"}="1HOP GP1";
$resn_exch{"HOP2 GP1"}="2HOP GP1"; $resn_exch{"HN21 GP2"}="1HN2 GP2"; $resn_exch{"HN22 GP2"}="2HN2 GP2";
$resn_exch{" H1' GP2"}=" H1* GP2"; $resn_exch{" H2' GP2"}=" H2* GP2"; $resn_exch{" H3' GP2"}=" H3* GP2";
$resn_exch{" H4' GP2"}=" H4* GP2"; $resn_exch{"H5'1 GP2"}="1H5* GP2"; $resn_exch{"H5'2 GP2"}="2H5* GP2";
$resn_exch{"H3A1 GP2"}="1H3A GP2"; $resn_exch{"H3A2 GP2"}="2H3A GP2"; $resn_exch{" H8A GP3"}=" AH8 GP3";
$resn_exch{" H1A GP3"}=" AH1 GP3"; $resn_exch{"H21A GP3"}="AH21 GP3"; $resn_exch{"H22A GP3"}="AH22 GP3";
$resn_exch{"H51A GP3"}="AH51 GP3"; $resn_exch{"H52A GP3"}="AH52 GP3"; $resn_exch{" H4D GP3"}="AH4* GP3";
$resn_exch{" H3D GP3"}="AH3* GP3"; $resn_exch{"HO3A GP3"}="AHO3 GP3"; $resn_exch{" H2D GP3"}="AH2* GP3";
$resn_exch{"HO2A GP3"}="AHO2 GP3"; $resn_exch{" H1D GP3"}="AH1* GP3"; $resn_exch{"HOA2 GP3"}="2HOA GP3";
$resn_exch{"HOB2 GP3"}="2HOB GP3"; $resn_exch{"HOG2 GP3"}="2HOG GP3"; $resn_exch{"H51B GP3"}="BH51 GP3";
$resn_exch{"H52B GP3"}="BH52 GP3"; $resn_exch{" H4E GP3"}="BH4* GP3"; $resn_exch{" H3E GP3"}="BH3* GP3";
$resn_exch{"HO3B GP3"}="BHO3 GP3"; $resn_exch{" H2E GP3"}="BH2* GP3"; $resn_exch{"HO2B GP3"}="BHO2 GP3";
$resn_exch{" H1E GP3"}="BH1* GP3"; $resn_exch{" H8B GP3"}=" BH8 GP3"; $resn_exch{" H1B GP3"}=" BH1 GP3";
$resn_exch{"H21B GP3"}="BH21 GP3"; $resn_exch{"H22B GP3"}="BH22 GP3"; $resn_exch{" H61 GP4"}="1H6  GP4";
$resn_exch{" H62 GP4"}="2H6  GP4"; $resn_exch{"HN21 GP4"}="1HN2 GP4"; $resn_exch{"HN22 GP4"}="2HN2 GP4";
$resn_exch{"HOP1 GP4"}="1HOP GP4"; $resn_exch{"HOP2 GP4"}="2HOP GP4"; $resn_exch{"HN21 GP5"}="1HN2 GP5";
$resn_exch{"HN22 GP5"}="2HN2 GP5"; $resn_exch{" H1' GP5"}=" H1* GP5"; $resn_exch{" H2' GP5"}=" H2* GP5";
$resn_exch{" H3' GP5"}=" H3* GP5"; $resn_exch{" H4' GP5"}=" H4* GP5"; $resn_exch{"H5'1 GP5"}="1H5* GP5";
$resn_exch{"H5'2 GP5"}="2H5* GP5"; $resn_exch{"HAV1 GP5"}="1HAV GP5"; $resn_exch{"HAV2 GP5"}="2HAV GP5";
$resn_exch{"HAB1 GP5"}="1HAB GP5"; $resn_exch{"HAB2 GP5"}="2HAB GP5"; $resn_exch{"HN21 GP6"}="1HN2 GP6";
$resn_exch{"HN22 GP6"}="2HN2 GP6"; $resn_exch{"HN11 GP8"}="1HN1 GP8"; $resn_exch{"HN12 GP8"}="2HN1 GP8";
$resn_exch{"HA22 GPB"}="2HA2 GPB"; $resn_exch{"HA21 GPB"}="1HA2 GPB"; $resn_exch{" H53 GPB"}="3H5  GPB";
$resn_exch{" H52 GPB"}="2H5  GPB"; $resn_exch{" H51 GPB"}="1H5  GPB"; $resn_exch{" H91 GPB"}="1H9  GPB";
$resn_exch{" H92 GPB"}="2H9  GPB"; $resn_exch{"H101 GPB"}="1H10 GPB"; $resn_exch{"H102 GPB"}="2H10 GPB";
$resn_exch{" HA1 GPB"}="1HA  GPB"; $resn_exch{" HB2 GPB"}="2HB  GPB"; $resn_exch{" HB1 GPB"}="1HB  GPB";
$resn_exch{" HG2 GPB"}="2HG  GPB"; $resn_exch{" HG1 GPB"}="1HG  GPB"; $resn_exch{"HE2O GPB"}="OHE2 GPB";
$resn_exch{" H1O GPB"}=" OH1 GPB"; $resn_exch{" H1G GPC"}=" GH1 GPC"; $resn_exch{"H21G GPC"}="GH21 GPC";
$resn_exch{"H22G GPC"}="GH22 GPC"; $resn_exch{" H8G GPC"}=" GH8 GPC"; $resn_exch{" H1D GPC"}="GH1* GPC";
$resn_exch{" H2D GPC"}="GH2* GPC"; $resn_exch{" H3D GPC"}="GH3* GPC"; $resn_exch{" H4D GPC"}="GH4* GPC";
$resn_exch{"H51G GPC"}="GH51 GPC"; $resn_exch{"H52G GPC"}="GH52 GPC"; $resn_exch{"HO5G GPC"}="GHO5 GPC";
$resn_exch{"HOC2 GPC"}="2HOC GPC"; $resn_exch{"H51C GPC"}="CH51 GPC"; $resn_exch{"H52C GPC"}="CH52 GPC";
$resn_exch{" H4B GPC"}="CH4* GPC"; $resn_exch{" H3B GPC"}="CH3* GPC"; $resn_exch{"HO3C GPC"}="CHO3 GPC";
$resn_exch{" H2B GPC"}="CH2* GPC"; $resn_exch{"HO2C GPC"}="CHO2 GPC"; $resn_exch{" H1B GPC"}="CH1* GPC";
$resn_exch{"H41C GPC"}="CH41 GPC"; $resn_exch{"H42C GPC"}="CH42 GPC"; $resn_exch{" H5C GPC"}=" CH5 GPC";
$resn_exch{" H6C GPC"}=" CH6 GPC"; $resn_exch{" H11 GPE"}="1H1  GPE"; $resn_exch{" H12 GPE"}="2H1  GPE";
$resn_exch{" H31 GPE"}="1H3  GPE"; $resn_exch{" H32 GPE"}="2H3  GPE"; $resn_exch{"H111 GPE"}="1H11 GPE";
$resn_exch{"H112 GPE"}="2H11 GPE"; $resn_exch{"H121 GPE"}="1H12 GPE"; $resn_exch{"H122 GPE"}="2H12 GPE";
$resn_exch{" HN1 GPE"}="1HN  GPE"; $resn_exch{" HN2 GPE"}="2HN  GPE"; $resn_exch{" H8A GPG"}=" AH8 GPG";
$resn_exch{" H1A GPG"}=" AH1 GPG"; $resn_exch{"H21A GPG"}="AH21 GPG"; $resn_exch{"H22A GPG"}="AH22 GPG";
$resn_exch{"HO5A GPG"}="AHO5 GPG"; $resn_exch{"H51A GPG"}="AH51 GPG"; $resn_exch{"H52A GPG"}="AH52 GPG";
$resn_exch{" H4D GPG"}="AH4* GPG"; $resn_exch{" H3D GPG"}="AH3* GPG"; $resn_exch{"HO3A GPG"}="AHO3 GPG";
$resn_exch{" H2D GPG"}="AH2* GPG"; $resn_exch{" H1D GPG"}="AH1* GPG"; $resn_exch{"HOP2 GPG"}="2HOP GPG";
$resn_exch{"H51B GPG"}="BH51 GPG"; $resn_exch{"H52B GPG"}="BH52 GPG"; $resn_exch{" H4E GPG"}="BH4* GPG";
$resn_exch{" H3E GPG"}="BH3* GPG"; $resn_exch{"HO3B GPG"}="BHO3 GPG"; $resn_exch{" H2E GPG"}="BH2* GPG";
$resn_exch{"HO2B GPG"}="BHO2 GPG"; $resn_exch{" H1E GPG"}="BH1* GPG"; $resn_exch{" H8B GPG"}=" BH8 GPG";
$resn_exch{" H1B GPG"}=" BH1 GPG"; $resn_exch{"H21B GPG"}="BH21 GPG"; $resn_exch{"H22B GPG"}="BH22 GPG";
$resn_exch{" H51 GPI"}="1H5  GPI"; $resn_exch{" H52 GPI"}="2H5  GPI"; $resn_exch{" H31 GPI"}="1H3  GPI";
$resn_exch{" H32 GPI"}="2H3  GPI"; $resn_exch{" H41 GPI"}="1H4  GPI"; $resn_exch{" H42 GPI"}="2H4  GPI";
$resn_exch{"H141 GPI"}="1H14 GPI"; $resn_exch{"H142 GPI"}="2H14 GPI"; $resn_exch{"H151 GPI"}="1H15 GPI";
$resn_exch{"H152 GPI"}="2H15 GPI"; $resn_exch{"H161 GPI"}="1H16 GPI"; $resn_exch{"H162 GPI"}="2H16 GPI";
$resn_exch{"H121 GPI"}="1H12 GPI"; $resn_exch{"H122 GPI"}="2H12 GPI"; $resn_exch{"H123 GPI"}="3H12 GPI";
$resn_exch{"H131 GPI"}="1H13 GPI"; $resn_exch{"H132 GPI"}="2H13 GPI"; $resn_exch{"H133 GPI"}="3H13 GPI";
$resn_exch{"H101 GPI"}="1H10 GPI"; $resn_exch{"H102 GPI"}="2H10 GPI"; $resn_exch{"H111 GPI"}="1H11 GPI";
$resn_exch{"H112 GPI"}="2H11 GPI"; $resn_exch{"H113 GPI"}="3H11 GPI"; $resn_exch{" H11 GPJ"}="1H1  GPJ";
$resn_exch{" H12 GPJ"}="2H1  GPJ"; $resn_exch{" H21 GPJ"}="1H2  GPJ"; $resn_exch{" H22 GPJ"}="2H2  GPJ";
$resn_exch{"HN11 GPJ"}="1HN1 GPJ"; $resn_exch{"HOP2 GPL"}="2HOP GPL"; $resn_exch{"H5'1 GPL"}="1H5* GPL";
$resn_exch{"H5'2 GPL"}="2H5* GPL"; $resn_exch{" H4' GPL"}=" H4* GPL"; $resn_exch{" H3' GPL"}=" H3* GPL";
$resn_exch{"HO3' GPL"}="*HO3 GPL"; $resn_exch{" H2' GPL"}=" H2* GPL"; $resn_exch{"HO2' GPL"}="*HO2 GPL";
$resn_exch{" H1' GPL"}=" H1* GPL"; $resn_exch{"HN21 GPL"}="1HN2 GPL"; $resn_exch{"HN22 GPL"}="2HN2 GPL";
$resn_exch{" H   GPL"}="1HN  GPL"; $resn_exch{" H2  GPL"}="2HN  GPL"; $resn_exch{" HB2 GPL"}="1HB  GPL";
$resn_exch{" HB3 GPL"}="2HB  GPL"; $resn_exch{" HG2 GPL"}="1HG  GPL"; $resn_exch{" HG3 GPL"}="2HG  GPL";
$resn_exch{" HD2 GPL"}="1HD  GPL"; $resn_exch{" HD3 GPL"}="2HD  GPL"; $resn_exch{" HE2 GPL"}="1HE  GPL";
$resn_exch{" HE3 GPL"}="2HE  GPL"; $resn_exch{" HZ  GPL"}=" HNZ GPL"; $resn_exch{" H61 GPM"}="1H6  GPM";
$resn_exch{" H62 GPM"}="2H6  GPM"; $resn_exch{" H71 GPM"}="1H7  GPM"; $resn_exch{" H72 GPM"}="2H7  GPM";
$resn_exch{"HOP2 GPM"}="2HOP GPM"; $resn_exch{"HOP3 GPM"}="3HOP GPM"; $resn_exch{"H8'1 GPN"}="1H8' GPN";
$resn_exch{"H8'2 GPN"}="2H8' GPN"; $resn_exch{"H5'1 GPN"}="1H5' GPN"; $resn_exch{"H5'2 GPN"}="2H5' GPN";
$resn_exch{"H3'1 GPN"}="1H3' GPN"; $resn_exch{"H3'2 GPN"}="2H3' GPN"; $resn_exch{"H2'1 GPN"}="1H2' GPN";
$resn_exch{"H2'2 GPN"}="2H2' GPN"; $resn_exch{"H1'1 GPN"}="1H1' GPN"; $resn_exch{"H1'2 GPN"}="2H1' GPN";
$resn_exch{"H1'3 GPN"}="3H1' GPN"; $resn_exch{"HN21 GPN"}="1HN2 GPN"; $resn_exch{"HN22 GPN"}="2HN2 GPN";
$resn_exch{" H11 GPP"}="1H1  GPP"; $resn_exch{" H12 GPP"}="2H1  GPP"; $resn_exch{" H41 GPP"}="1H4  GPP";
$resn_exch{" H42 GPP"}="2H4  GPP"; $resn_exch{" H43 GPP"}="3H4  GPP"; $resn_exch{" H51 GPP"}="1H5  GPP";
$resn_exch{" H52 GPP"}="2H5  GPP"; $resn_exch{" H61 GPP"}="1H6  GPP"; $resn_exch{" H62 GPP"}="2H6  GPP";
$resn_exch{" H91 GPP"}="1H9  GPP"; $resn_exch{" H92 GPP"}="2H9  GPP"; $resn_exch{" H93 GPP"}="3H9  GPP";
$resn_exch{"H101 GPP"}="1H10 GPP"; $resn_exch{"H102 GPP"}="2H10 GPP"; $resn_exch{"H103 GPP"}="3H10 GPP";
$resn_exch{"HOA2 GPP"}="2HOA GPP"; $resn_exch{"HOB2 GPP"}="2HOB GPP"; $resn_exch{"HOB3 GPP"}="3HOB GPP";
$resn_exch{"HN11 GPR"}="1HN1 GPR"; $resn_exch{"HN12 GPR"}="2HN1 GPR"; $resn_exch{"HB11 GPR"}="1HB1 GPR";
$resn_exch{"HB12 GPR"}="2HB1 GPR"; $resn_exch{"HG11 GPR"}="1HG1 GPR"; $resn_exch{"HG12 GPR"}="2HG1 GPR";
$resn_exch{"HB21 GPR"}="1HB2 GPR"; $resn_exch{"HB22 GPR"}="2HB2 GPR"; $resn_exch{"HA31 GPR"}="1HA3 GPR";
$resn_exch{"HA32 GPR"}="2HA3 GPR"; $resn_exch{"HN11 GPS"}="1HN1 GPS"; $resn_exch{"HN12 GPS"}="2HN1 GPS";
$resn_exch{"HB11 GPS"}="1HB1 GPS"; $resn_exch{"HB12 GPS"}="2HB1 GPS"; $resn_exch{"HG11 GPS"}="1HG1 GPS";
$resn_exch{"HG12 GPS"}="2HG1 GPS"; $resn_exch{"HB21 GPS"}="1HB2 GPS"; $resn_exch{"HB22 GPS"}="2HB2 GPS";
$resn_exch{"HA31 GPS"}="1HA3 GPS"; $resn_exch{"HA32 GPS"}="2HA3 GPS"; $resn_exch{"HN21 GPX"}="1HN2 GPX";
$resn_exch{"HN22 GPX"}="2HN2 GPX"; $resn_exch{"HOA2 GPX"}="2HOA GPX"; $resn_exch{"HOB2 GPX"}="2HOB GPX";
$resn_exch{"HOB3 GPX"}="3HOB GPX"; $resn_exch{"H5'1 GPX"}="1H5' GPX"; $resn_exch{"H5'2 GPX"}="2H5' GPX";
$resn_exch{" H72 GR1"}="2H7  GR1"; $resn_exch{" H71 GR1"}="1H7  GR1"; $resn_exch{"H172 GR1"}="2H17 GR1";
$resn_exch{"H171 GR1"}="1H17 GR1"; $resn_exch{"H182 GR1"}="2H18 GR1"; $resn_exch{"H181 GR1"}="1H18 GR1";
$resn_exch{"H192 GR1"}="2H19 GR1"; $resn_exch{"H191 GR1"}="1H19 GR1"; $resn_exch{"H202 GR1"}="2H20 GR1";
$resn_exch{"H201 GR1"}="1H20 GR1"; $resn_exch{" H32 GR1"}="2H3  GR1"; $resn_exch{" H31 GR1"}="1H3  GR1";
$resn_exch{"H142 GR1"}="2H14 GR1"; $resn_exch{"H141 GR1"}="1H14 GR1"; $resn_exch{"H3C1 GR1"}="1H3C GR1";
$resn_exch{"H273 GR1"}="3H27 GR1"; $resn_exch{"H272 GR1"}="2H27 GR1"; $resn_exch{"H271 GR1"}="1H27 GR1";
$resn_exch{" H12 GR1"}="2H1  GR1"; $resn_exch{" H11 GR1"}="1H1  GR1"; $resn_exch{"H23A GR1"}="3H2  GR1";
$resn_exch{"H22A GR1"}="2H2  GR1"; $resn_exch{" H21 GR1"}="1H2  GR1"; $resn_exch{" H11 GR3"}="1H1  GR3";
$resn_exch{" H12 GR3"}="2H1  GR3"; $resn_exch{"H21A GR3"}="1H2  GR3"; $resn_exch{" H22 GR3"}="2H2  GR3";
$resn_exch{" H61 GR3"}="1H6  GR3"; $resn_exch{" H62 GR3"}="2H6  GR3"; $resn_exch{"H111 GR3"}="1H11 GR3";
$resn_exch{"H112 GR3"}="2H11 GR3"; $resn_exch{"H121 GR3"}="1H12 GR3"; $resn_exch{"H122 GR3"}="2H12 GR3";
$resn_exch{"H151 GR3"}="1H15 GR3"; $resn_exch{"H152 GR3"}="2H15 GR3"; $resn_exch{"H181 GR3"}="1H18 GR3";
$resn_exch{"H182 GR3"}="2H18 GR3"; $resn_exch{"H183 GR3"}="3H18 GR3"; $resn_exch{"H191 GR3"}="1H19 GR3";
$resn_exch{"H192 GR3"}="2H19 GR3"; $resn_exch{"H193 GR3"}="3H19 GR3"; $resn_exch{"H221 GR3"}="1H22 GR3";
$resn_exch{"H222 GR3"}="2H22 GR3"; $resn_exch{"H261 GR3"}="1H26 GR3"; $resn_exch{"H262 GR3"}="2H26 GR3";
$resn_exch{"H263 GR3"}="3H26 GR3"; $resn_exch{"H271 GR3"}="1H27 GR3"; $resn_exch{"H272 GR3"}="2H27 GR3";
$resn_exch{"H273 GR3"}="3H27 GR3"; $resn_exch{"H281 GR3"}="1H28 GR3"; $resn_exch{"H282 GR3"}="2H28 GR3";
$resn_exch{"H283 GR3"}="3H28 GR3"; $resn_exch{"H301 GR3"}="1H30 GR3"; $resn_exch{"H302 GR3"}="2H30 GR3";
$resn_exch{"H303 GR3"}="3H30 GR3"; $resn_exch{"H381 GR3"}="1H38 GR3"; $resn_exch{"H382 GR3"}="2H38 GR3";
$resn_exch{"H383 GR3"}="3H38 GR3"; $resn_exch{" H11 GR4"}="1H1  GR4"; $resn_exch{"H12A GR4"}="2H1  GR4";
$resn_exch{"H21A GR4"}="1H2  GR4"; $resn_exch{" H22 GR4"}="2H2  GR4"; $resn_exch{" H61 GR4"}="1H6  GR4";
$resn_exch{" H62 GR4"}="2H6  GR4"; $resn_exch{"H111 GR4"}="1H11 GR4"; $resn_exch{"H112 GR4"}="2H11 GR4";
$resn_exch{"H151 GR4"}="1H15 GR4"; $resn_exch{"H152 GR4"}="2H15 GR4"; $resn_exch{"H181 GR4"}="1H18 GR4";
$resn_exch{"H182 GR4"}="2H18 GR4"; $resn_exch{"H183 GR4"}="3H18 GR4"; $resn_exch{"H191 GR4"}="1H19 GR4";
$resn_exch{"H192 GR4"}="2H19 GR4"; $resn_exch{"H193 GR4"}="3H19 GR4"; $resn_exch{"H221 GR4"}="1H22 GR4";
$resn_exch{"H222 GR4"}="2H22 GR4"; $resn_exch{"H261 GR4"}="1H26 GR4"; $resn_exch{"H262 GR4"}="2H26 GR4";
$resn_exch{"H263 GR4"}="3H26 GR4"; $resn_exch{"H271 GR4"}="1H27 GR4"; $resn_exch{"H272 GR4"}="2H27 GR4";
$resn_exch{"H273 GR4"}="3H27 GR4"; $resn_exch{"H281 GR4"}="1H28 GR4"; $resn_exch{"H282 GR4"}="2H28 GR4";
$resn_exch{"H301 GR4"}="1H30 GR4"; $resn_exch{"H302 GR4"}="2H30 GR4"; $resn_exch{"H303 GR4"}="3H30 GR4";
$resn_exch{"H381 GR4"}="1H38 GR4"; $resn_exch{"H382 GR4"}="2H38 GR4"; $resn_exch{"H383 GR4"}="3H38 GR4";
$resn_exch{"H311 GR4"}="1H31 GR4"; $resn_exch{"H312 GR4"}="2H31 GR4"; $resn_exch{"H101 GRF"}="1H10 GRF";
$resn_exch{"H102 GRF"}="2H10 GRF"; $resn_exch{"H191 GRF"}="1H19 GRF"; $resn_exch{"H192 GRF"}="2H19 GRF";
$resn_exch{"H201 GRG"}="1H20 GRG"; $resn_exch{"H202 GRG"}="2H20 GRG"; $resn_exch{"H203 GRG"}="3H20 GRG";
$resn_exch{"H191 GRG"}="1H19 GRG"; $resn_exch{"H192 GRG"}="2H19 GRG"; $resn_exch{"H193 GRG"}="3H19 GRG";
$resn_exch{"H161 GRG"}="1H16 GRG"; $resn_exch{"H162 GRG"}="2H16 GRG"; $resn_exch{"H141 GRG"}="1H14 GRG";
$resn_exch{"H142 GRG"}="2H14 GRG"; $resn_exch{"H143 GRG"}="3H14 GRG"; $resn_exch{"H151 GRG"}="1H15 GRG";
$resn_exch{"H152 GRG"}="2H15 GRG"; $resn_exch{"H111 GRG"}="1H11 GRG"; $resn_exch{"H112 GRG"}="2H11 GRG";
$resn_exch{"HC91 GRG"}="1HC9 GRG"; $resn_exch{"HC92 GRG"}="2HC9 GRG"; $resn_exch{"H101 GRG"}="1H10 GRG";
$resn_exch{"H102 GRG"}="2H10 GRG"; $resn_exch{"H103 GRG"}="3H10 GRG"; $resn_exch{"HC61 GRG"}="1HC6 GRG";
$resn_exch{"HC62 GRG"}="2HC6 GRG"; $resn_exch{"HC51 GRG"}="1HC5 GRG"; $resn_exch{"HC52 GRG"}="2HC5 GRG";
$resn_exch{"HC41 GRG"}="1HC4 GRG"; $resn_exch{"HC42 GRG"}="2HC4 GRG"; $resn_exch{"HC43 GRG"}="3HC4 GRG";
$resn_exch{"HC11 GRG"}="1HC1 GRG"; $resn_exch{"HC12 GRG"}="2HC1 GRG"; $resn_exch{"H1AO GRG"}="OH1A GRG";
$resn_exch{"H1BO GRG"}="OH1B GRG"; $resn_exch{"H2BO GRG"}="OH2B GRG"; $resn_exch{"H121 GRL"}="1H12 GRL";
$resn_exch{"H122 GRL"}="2H12 GRL"; $resn_exch{"H141 GRL"}="1H14 GRL"; $resn_exch{"H142 GRL"}="2H14 GRL";
$resn_exch{"H161 GRL"}="1H16 GRL"; $resn_exch{"H162 GRL"}="2H16 GRL"; $resn_exch{" H21 GRL"}="1H2  GRL";
$resn_exch{"H22A GRL"}="2H2  GRL"; $resn_exch{"H231 GRL"}="1H23 GRL"; $resn_exch{"H232 GRL"}="2H23 GRL";
$resn_exch{"H321 GRL"}="1H32 GRL"; $resn_exch{"H322 GRL"}="2H32 GRL"; $resn_exch{"H331 GRL"}="1H33 GRL";
$resn_exch{"H332 GRL"}="2H33 GRL"; $resn_exch{"H351 GRL"}="1H35 GRL"; $resn_exch{"H352 GRL"}="2H35 GRL";
$resn_exch{"H353 GRL"}="3H35 GRL"; $resn_exch{"H361 GRL"}="1H36 GRL"; $resn_exch{"H362 GRL"}="2H36 GRL";
$resn_exch{"H363 GRL"}="3H36 GRL"; $resn_exch{"H31A GRL"}="1H3  GRL"; $resn_exch{" H32 GRL"}="2H3  GRL";
$resn_exch{"HC31 GRO"}="1HC3 GRO"; $resn_exch{"HC32 GRO"}="2HC3 GRO"; $resn_exch{"HC33 GRO"}="3HC3 GRO";
$resn_exch{"HSP2  GS"}="2HSP  GS"; $resn_exch{"HOP3  GS"}="3HOP  GS"; $resn_exch{" H5'  GS"}="1H5*  GS";
$resn_exch{"H5''  GS"}="2H5*  GS"; $resn_exch{" H4'  GS"}=" H4*  GS"; $resn_exch{" H3'  GS"}=" H3*  GS";
$resn_exch{"HO3'  GS"}="*HO3  GS"; $resn_exch{" H2'  GS"}="1H2*  GS"; $resn_exch{"H2''  GS"}="2H2*  GS";
$resn_exch{" H1'  GS"}=" H1*  GS"; $resn_exch{"HN21  GS"}="1HN2  GS"; $resn_exch{"HN22  GS"}="2HN2  GS";
$resn_exch{" H61 GS1"}="1H6  GS1"; $resn_exch{" H62 GS1"}="2H6  GS1"; $resn_exch{" H61 GS4"}="1H6  GS4";
$resn_exch{" H62 GS4"}="2H6  GS4"; $resn_exch{"H161 GS5"}="1H16 GS5"; $resn_exch{"H162 GS5"}="2H16 GS5";
$resn_exch{"H201 GS5"}="1H20 GS5"; $resn_exch{"H202 GS5"}="2H20 GS5"; $resn_exch{"H231 GS5"}="1H23 GS5";
$resn_exch{"H232 GS5"}="2H23 GS5"; $resn_exch{"H233 GS5"}="3H23 GS5"; $resn_exch{"H261 GS5"}="1H26 GS5";
$resn_exch{"H262 GS5"}="2H26 GS5"; $resn_exch{"H271 GS5"}="1H27 GS5"; $resn_exch{"H272 GS5"}="2H27 GS5";
$resn_exch{"H281 GS5"}="1H28 GS5"; $resn_exch{"H282 GS5"}="2H28 GS5"; $resn_exch{"H291 GS5"}="1H29 GS5";
$resn_exch{"H292 GS5"}="2H29 GS5"; $resn_exch{"H131 GS6"}="1H13 GS6"; $resn_exch{"H132 GS6"}="2H13 GS6";
$resn_exch{"H141 GS6"}="1H14 GS6"; $resn_exch{"H142 GS6"}="2H14 GS6"; $resn_exch{"H171 GS6"}="1H17 GS6";
$resn_exch{"H172 GS6"}="2H17 GS6"; $resn_exch{"H173 GS6"}="3H17 GS6"; $resn_exch{"H191 GS6"}="1H19 GS6";
$resn_exch{"H192 GS6"}="2H19 GS6"; $resn_exch{"H231 GS6"}="1H23 GS6"; $resn_exch{"H232 GS6"}="2H23 GS6";
$resn_exch{"H201 GS6"}="1H20 GS6"; $resn_exch{"H202 GS6"}="2H20 GS6"; $resn_exch{"H221 GS6"}="1H22 GS6";
$resn_exch{"H222 GS6"}="2H22 GS6"; $resn_exch{" H61 GSA"}="1H6  GSA"; $resn_exch{" H62 GSA"}="2H6  GSA";
$resn_exch{"HOS3 GSA"}="3HOS GSA"; $resn_exch{"HN11 GSB"}="1HN1 GSB"; $resn_exch{"HN12 GSB"}="2HN1 GSB";
$resn_exch{"HB11 GSB"}="1HB1 GSB"; $resn_exch{"HB12 GSB"}="2HB1 GSB"; $resn_exch{"HG11 GSB"}="1HG1 GSB";
$resn_exch{"HG12 GSB"}="2HG1 GSB"; $resn_exch{"HA31 GSB"}="1HA3 GSB"; $resn_exch{"HA32 GSB"}="2HA3 GSB";
$resn_exch{"HB21 GSB"}="1HB2 GSB"; $resn_exch{"HB22 GSB"}="2HB2 GSB"; $resn_exch{" H'1 GSB"}="1H'  GSB";
$resn_exch{" H'2 GSB"}="2H'  GSB"; $resn_exch{" H   GSC"}="1HN  GSC"; $resn_exch{" H2  GSC"}="2HN  GSC";
$resn_exch{" H11 GSC"}="1H1  GSC"; $resn_exch{" H12 GSC"}="2H1  GSC"; $resn_exch{" H21 GSC"}="1H2  GSC";
$resn_exch{" H22 GSC"}="2H2  GSC"; $resn_exch{" H23 GSC"}="3H2  GSC"; $resn_exch{" H61 GSD"}="1H6  GSD";
$resn_exch{" H62 GSD"}="2H6  GSD"; $resn_exch{" H11 GSE"}="1H1  GSE"; $resn_exch{"H12A GSE"}="2H1  GSE";
$resn_exch{" H31 GSE"}="1H3  GSE"; $resn_exch{" H32 GSE"}="2H3  GSE"; $resn_exch{"H111 GSE"}="1H11 GSE";
$resn_exch{"H112 GSE"}="2H11 GSE"; $resn_exch{" HN1 GSE"}="1HN  GSE"; $resn_exch{" HN2 GSE"}="2HN  GSE";
$resn_exch{"HB11 GSF"}="1HB1 GSF"; $resn_exch{"HB12 GSF"}="2HB1 GSF"; $resn_exch{"HG11 GSF"}="1HG1 GSF";
$resn_exch{"HG12 GSF"}="2HG1 GSF"; $resn_exch{"HN11 GSF"}="1HN1 GSF"; $resn_exch{"HN12 GSF"}="2HN1 GSF";
$resn_exch{"HB21 GSF"}="1HB2 GSF"; $resn_exch{"HB22 GSF"}="2HB2 GSF"; $resn_exch{"HA31 GSF"}="1HA3 GSF";
$resn_exch{"HA32 GSF"}="2HA3 GSF"; $resn_exch{" H81 GSG"}="1H8  GSG"; $resn_exch{" H82 GSG"}="2H8  GSG";
$resn_exch{" H91 GSG"}="1H9  GSG"; $resn_exch{" H92 GSG"}="2H9  GSG"; $resn_exch{" H93 GSG"}="3H9  GSG";
$resn_exch{"H141 GSG"}="1H14 GSG"; $resn_exch{"H142 GSG"}="2H14 GSG"; $resn_exch{"H143 GSG"}="3H14 GSG";
$resn_exch{"H191 GSG"}="1H19 GSG"; $resn_exch{"H192 GSG"}="2H19 GSG"; $resn_exch{"H193 GSG"}="3H19 GSG";
$resn_exch{"H201 GSG"}="1H20 GSG"; $resn_exch{"H202 GSG"}="2H20 GSG"; $resn_exch{"H211 GSG"}="1H21 GSG";
$resn_exch{"H212 GSG"}="2H21 GSG"; $resn_exch{"H221 GSG"}="1H22 GSG"; $resn_exch{"H222 GSG"}="2H22 GSG";
$resn_exch{"H223 GSG"}="3H22 GSG"; $resn_exch{"HN11 GSH"}="1HN1 GSH"; $resn_exch{"HN12 GSH"}="2HN1 GSH";
$resn_exch{"HB12 GSH"}="2HB1 GSH"; $resn_exch{"HB13 GSH"}="3HB1 GSH"; $resn_exch{"HG12 GSH"}="2HG1 GSH";
$resn_exch{"HG13 GSH"}="3HG1 GSH"; $resn_exch{"HB22 GSH"}="2HB2 GSH"; $resn_exch{"HB23 GSH"}="3HB2 GSH";
$resn_exch{"HA31 GSH"}="1HA3 GSH"; $resn_exch{"HA32 GSH"}="2HA3 GSH"; $resn_exch{"H231 GSJ"}="1H23 GSJ";
$resn_exch{"H232 GSJ"}="2H23 GSJ"; $resn_exch{"H233 GSJ"}="3H23 GSJ"; $resn_exch{"H221 GSJ"}="1H22 GSJ";
$resn_exch{"H222 GSJ"}="2H22 GSJ"; $resn_exch{"H111 GSJ"}="1H11 GSJ"; $resn_exch{"H112 GSJ"}="2H11 GSJ";
$resn_exch{"H201 GSJ"}="1H20 GSJ"; $resn_exch{"H202 GSJ"}="2H20 GSJ"; $resn_exch{"H203 GSJ"}="3H20 GSJ";
$resn_exch{"H211 GSJ"}="1H21 GSJ"; $resn_exch{"H212 GSJ"}="2H21 GSJ"; $resn_exch{"H213 GSJ"}="3H21 GSJ";
$resn_exch{"H171 GSJ"}="1H17 GSJ"; $resn_exch{"H172 GSJ"}="2H17 GSJ"; $resn_exch{"H173 GSJ"}="3H17 GSJ";
$resn_exch{"H141 GSJ"}="1H14 GSJ"; $resn_exch{"H142 GSJ"}="2H14 GSJ"; $resn_exch{"H131 GSJ"}="1H13 GSJ";
$resn_exch{"H132 GSJ"}="2H13 GSJ"; $resn_exch{"H131 GSK"}="1H13 GSK"; $resn_exch{"H132 GSK"}="2H13 GSK";
$resn_exch{"H141 GSK"}="1H14 GSK"; $resn_exch{"H142 GSK"}="2H14 GSK"; $resn_exch{"H171 GSK"}="1H17 GSK";
$resn_exch{"H172 GSK"}="2H17 GSK"; $resn_exch{"H173 GSK"}="3H17 GSK"; $resn_exch{"H191 GSK"}="1H19 GSK";
$resn_exch{"H192 GSK"}="2H19 GSK"; $resn_exch{"H201 GSK"}="1H20 GSK"; $resn_exch{"H202 GSK"}="2H20 GSK";
$resn_exch{"H221 GSK"}="1H22 GSK"; $resn_exch{"H222 GSK"}="2H22 GSK"; $resn_exch{"H231 GSK"}="1H23 GSK";
$resn_exch{"H232 GSK"}="2H23 GSK"; $resn_exch{"H171 GSL"}="1H17 GSL"; $resn_exch{"H172 GSL"}="2H17 GSL";
$resn_exch{"H173 GSL"}="3H17 GSL"; $resn_exch{"H161 GSL"}="1H16 GSL"; $resn_exch{"H162 GSL"}="2H16 GSL";
$resn_exch{"H151 GSL"}="1H15 GSL"; $resn_exch{"H152 GSL"}="2H15 GSL"; $resn_exch{"H141 GSL"}="1H14 GSL";
$resn_exch{"H142 GSL"}="2H14 GSL"; $resn_exch{"H131 GSL"}="1H13 GSL"; $resn_exch{"H132 GSL"}="2H13 GSL";
$resn_exch{"H121 GSL"}="1H12 GSL"; $resn_exch{"H122 GSL"}="2H12 GSL"; $resn_exch{"H111 GSL"}="1H11 GSL";
$resn_exch{"H112 GSL"}="2H11 GSL"; $resn_exch{"H101 GSL"}="1H10 GSL"; $resn_exch{"H102 GSL"}="2H10 GSL";
$resn_exch{" H91 GSL"}="1H9  GSL"; $resn_exch{" H92 GSL"}="2H9  GSL"; $resn_exch{" H81 GSL"}="1H8  GSL";
$resn_exch{" H82 GSL"}="2H8  GSL"; $resn_exch{" H71 GSL"}="1H7  GSL"; $resn_exch{" H72 GSL"}="2H7  GSL";
$resn_exch{" H61 GSL"}="1H6  GSL"; $resn_exch{" H62 GSL"}="2H6  GSL"; $resn_exch{" H51 GSL"}="1H5  GSL";
$resn_exch{" H52 GSL"}="2H5  GSL"; $resn_exch{" H41 GSL"}="1H4  GSL"; $resn_exch{" H42 GSL"}="2H4  GSL";
$resn_exch{"H31A GSL"}="1H3  GSL"; $resn_exch{"H32A GSL"}="2H3  GSL"; $resn_exch{" H1A GSL"}="1H   GSL";
$resn_exch{" H2A GSL"}="2H   GSL"; $resn_exch{"H191 GSL"}="1H19 GSL"; $resn_exch{"H192 GSL"}="2H19 GSL";
$resn_exch{"H201 GSL"}="1H20 GSL"; $resn_exch{"H202 GSL"}="2H20 GSL"; $resn_exch{"H211 GSL"}="1H21 GSL";
$resn_exch{"H212 GSL"}="2H21 GSL"; $resn_exch{"H221 GSL"}="1H22 GSL"; $resn_exch{"H222 GSL"}="2H22 GSL";
$resn_exch{"H231 GSL"}="1H23 GSL"; $resn_exch{"H232 GSL"}="2H23 GSL"; $resn_exch{"H241 GSL"}="1H24 GSL";
$resn_exch{"H242 GSL"}="2H24 GSL"; $resn_exch{"H251 GSL"}="1H25 GSL"; $resn_exch{"H252 GSL"}="2H25 GSL";
$resn_exch{"H261 GSL"}="1H26 GSL"; $resn_exch{"H262 GSL"}="2H26 GSL"; $resn_exch{"H271 GSL"}="1H27 GSL";
$resn_exch{"H272 GSL"}="2H27 GSL"; $resn_exch{"H381 GSL"}="1H38 GSL"; $resn_exch{"H382 GSL"}="2H38 GSL";
$resn_exch{"H371 GSL"}="1H37 GSL"; $resn_exch{"H372 GSL"}="2H37 GSL"; $resn_exch{"H361 GSL"}="1H36 GSL";
$resn_exch{"H362 GSL"}="2H36 GSL"; $resn_exch{"H351 GSL"}="1H35 GSL"; $resn_exch{"H352 GSL"}="2H35 GSL";
$resn_exch{"H353 GSL"}="3H35 GSL"; $resn_exch{"HA31 GSM"}="1HA3 GSM"; $resn_exch{"HA32 GSM"}="2HA3 GSM";
$resn_exch{"HB21 GSM"}="1HB2 GSM"; $resn_exch{"HB22 GSM"}="2HB2 GSM"; $resn_exch{" H41 GSM"}="1H4  GSM";
$resn_exch{" H42 GSM"}="2H4  GSM"; $resn_exch{" H43 GSM"}="3H4  GSM"; $resn_exch{"HG11 GSM"}="1HG1 GSM";
$resn_exch{"HG12 GSM"}="2HG1 GSM"; $resn_exch{"HB11 GSM"}="1HB1 GSM"; $resn_exch{"HB12 GSM"}="2HB1 GSM";
$resn_exch{"HN11 GSM"}="1HN1 GSM"; $resn_exch{"HN12 GSM"}="2HN1 GSM"; $resn_exch{" HN1 GSN"}="1HN  GSN";
$resn_exch{" HN2 GSN"}="2HN  GSN"; $resn_exch{" HB1 GSN"}="1HB  GSN"; $resn_exch{" HB2 GSN"}="2HB  GSN";
$resn_exch{"HAG1 GSN"}="1HAG GSN"; $resn_exch{"HAG2 GSN"}="2HAG GSN"; $resn_exch{"HAL1 GSN"}="1HAL GSN";
$resn_exch{"HAL2 GSN"}="2HAL GSN"; $resn_exch{"HAR1 GSN"}="1HAR GSN"; $resn_exch{"HAR2 GSN"}="2HAR GSN";
$resn_exch{" HN1 GSO"}="1HN  GSO"; $resn_exch{" HN2 GSO"}="2HN  GSO"; $resn_exch{"HBC1 GSO"}="1HBC GSO";
$resn_exch{"HBC2 GSO"}="2HBC GSO"; $resn_exch{"HB21 GSO"}="1HB2 GSO"; $resn_exch{"HB22 GSO"}="2HB2 GSO";
$resn_exch{"HGC1 GSO"}="1HGC GSO"; $resn_exch{"HGC2 GSO"}="2HGC GSO"; $resn_exch{"HA31 GSO"}="1HA3 GSO";
$resn_exch{"HA32 GSO"}="2HA3 GSO"; $resn_exch{"HD21 GSO"}="1HD2 GSO"; $resn_exch{"HD22 GSO"}="2HD2 GSO";
$resn_exch{"HOG2 GSP"}="2HOG GSP"; $resn_exch{"HOG3 GSP"}="3HOG GSP"; $resn_exch{"HOB2 GSP"}="2HOB GSP";
$resn_exch{"HOA2 GSP"}="2HOA GSP"; $resn_exch{"H5'1 GSP"}="1H5* GSP"; $resn_exch{"H5'2 GSP"}="2H5* GSP";
$resn_exch{" H4' GSP"}=" H4* GSP"; $resn_exch{" H3' GSP"}=" H3* GSP"; $resn_exch{"HO3' GSP"}="*HO3 GSP";
$resn_exch{" H2' GSP"}=" H2* GSP"; $resn_exch{"HO2' GSP"}="*HO2 GSP"; $resn_exch{" H1' GSP"}=" H1* GSP";
$resn_exch{"HN21 GSP"}="1HN2 GSP"; $resn_exch{"HN22 GSP"}="2HN2 GSP"; $resn_exch{"H161 GSQ"}="1H16 GSQ";
$resn_exch{"H162 GSQ"}="2H16 GSQ"; $resn_exch{"H201 GSQ"}="1H20 GSQ"; $resn_exch{"H202 GSQ"}="2H20 GSQ";
$resn_exch{"H231 GSQ"}="1H23 GSQ"; $resn_exch{"H232 GSQ"}="2H23 GSQ"; $resn_exch{"H233 GSQ"}="3H23 GSQ";
$resn_exch{"H261 GSQ"}="1H26 GSQ"; $resn_exch{"H262 GSQ"}="2H26 GSQ"; $resn_exch{"H271 GSQ"}="1H27 GSQ";
$resn_exch{"H272 GSQ"}="2H27 GSQ"; $resn_exch{"H281 GSQ"}="1H28 GSQ"; $resn_exch{"H282 GSQ"}="2H28 GSQ";
$resn_exch{"H291 GSQ"}="1H29 GSQ"; $resn_exch{"H292 GSQ"}="2H29 GSQ"; $resn_exch{"HOP2 GSR"}="2HOP GSR";
$resn_exch{"HOP3 GSR"}="3HOP GSR"; $resn_exch{" H5' GSR"}="1H5* GSR"; $resn_exch{"H5'' GSR"}="2H5* GSR";
$resn_exch{" H4' GSR"}=" H4* GSR"; $resn_exch{" H3' GSR"}=" H3* GSR"; $resn_exch{"HO3' GSR"}="*HO3 GSR";
$resn_exch{" H2' GSR"}="1H2* GSR"; $resn_exch{"H2'' GSR"}="2H2* GSR"; $resn_exch{" H1' GSR"}=" H1* GSR";
$resn_exch{" HB1 GSR"}="1HB  GSR"; $resn_exch{" HB2 GSR"}="2HB  GSR"; $resn_exch{"HOP2 GSS"}="2HOP GSS";
$resn_exch{"HOP3 GSS"}="3HOP GSS"; $resn_exch{" H5' GSS"}="1H5* GSS"; $resn_exch{"H5'' GSS"}="2H5* GSS";
$resn_exch{" H4' GSS"}=" H4* GSS"; $resn_exch{" H3' GSS"}=" H3* GSS"; $resn_exch{"HO3' GSS"}="*HO3 GSS";
$resn_exch{" H2' GSS"}="1H2* GSS"; $resn_exch{"H2'' GSS"}="2H2* GSS"; $resn_exch{" H1' GSS"}=" H1* GSS";
$resn_exch{" HB1 GSS"}="1HB  GSS"; $resn_exch{" HB2 GSS"}="2HB  GSS"; $resn_exch{" H11 GST"}="1H1  GST";
$resn_exch{" H12 GST"}="2H1  GST"; $resn_exch{"H101 GST"}="1H10 GST"; $resn_exch{"H102 GST"}="2H10 GST";
$resn_exch{"H103 GST"}="3H10 GST"; $resn_exch{" H41 GST"}="1H4  GST"; $resn_exch{" H42 GST"}="2H4  GST";
$resn_exch{" H51 GST"}="1H5  GST"; $resn_exch{" H52 GST"}="2H5  GST"; $resn_exch{" H91 GST"}="1H9  GST";
$resn_exch{" H92 GST"}="2H9  GST"; $resn_exch{" H93 GST"}="3H9  GST"; $resn_exch{" H81 GST"}="1H8  GST";
$resn_exch{" H82 GST"}="2H8  GST"; $resn_exch{" H83 GST"}="3H8  GST"; $resn_exch{" HC2 GSU"}=" H2  GSU";
$resn_exch{" H   GSU"}="1HN  GSU"; $resn_exch{" H2  GSU"}="2HN  GSU"; $resn_exch{" HB2 GSU"}="1HB  GSU";
$resn_exch{" HB3 GSU"}="2HB  GSU"; $resn_exch{" HG2 GSU"}="1HG  GSU"; $resn_exch{" HG3 GSU"}="2HG  GSU";
$resn_exch{"H5'1 GSU"}="1H5* GSU"; $resn_exch{"H5'2 GSU"}="2H5* GSU"; $resn_exch{" H4' GSU"}=" H4* GSU";
$resn_exch{" H1' GSU"}=" H1* GSU"; $resn_exch{"HN61 GSU"}="1HN6 GSU"; $resn_exch{"HN62 GSU"}="2HN6 GSU";
$resn_exch{" H2' GSU"}=" H2* GSU"; $resn_exch{"HO2' GSU"}="*HO2 GSU"; $resn_exch{" H3' GSU"}=" H3* GSU";
$resn_exch{"HO3' GSU"}="*HO3 GSU"; $resn_exch{"H1N1 GSW"}="1H1N GSW"; $resn_exch{"H1N2 GSW"}="2H1N GSW";
$resn_exch{"HB11 GSW"}="1HB1 GSW"; $resn_exch{"HB12 GSW"}="2HB1 GSW"; $resn_exch{"HG11 GSW"}="1HG1 GSW";
$resn_exch{"HG12 GSW"}="2HG1 GSW"; $resn_exch{"HB21 GSW"}="1HB2 GSW"; $resn_exch{"HB22 GSW"}="2HB2 GSW";
$resn_exch{"HA31 GSW"}="1HA3 GSW"; $resn_exch{"HA32 GSW"}="2HA3 GSW"; $resn_exch{"H131 GSX"}="1H13 GSX";
$resn_exch{"H132 GSX"}="2H13 GSX"; $resn_exch{"H141 GSX"}="1H14 GSX"; $resn_exch{"H142 GSX"}="2H14 GSX";
$resn_exch{"H171 GSX"}="1H17 GSX"; $resn_exch{"H172 GSX"}="2H17 GSX"; $resn_exch{"H173 GSX"}="3H17 GSX";
$resn_exch{"H191 GSX"}="1H19 GSX"; $resn_exch{"H192 GSX"}="2H19 GSX"; $resn_exch{"H231 GSX"}="1H23 GSX";
$resn_exch{"H232 GSX"}="2H23 GSX"; $resn_exch{"H201 GSX"}="1H20 GSX"; $resn_exch{"H202 GSX"}="2H20 GSX";
$resn_exch{"H221 GSX"}="1H22 GSX"; $resn_exch{"H222 GSX"}="2H22 GSX"; $resn_exch{"HAZ1 GSZ"}="1HAZ GSZ";
$resn_exch{"HAZ2 GSZ"}="2HAZ GSZ"; $resn_exch{"HAZ3 GSZ"}="3HAZ GSZ"; $resn_exch{"HBA1 GSZ"}="1HBA GSZ";
$resn_exch{"HBA2 GSZ"}="2HBA GSZ"; $resn_exch{"HBA3 GSZ"}="3HBA GSZ"; $resn_exch{"HAI1 GSZ"}="1HAI GSZ";
$resn_exch{"HAI2 GSZ"}="2HAI GSZ"; $resn_exch{"HAR1 GSZ"}="1HAR GSZ"; $resn_exch{"HAR2 GSZ"}="2HAR GSZ";
$resn_exch{"HAS1 GSZ"}="1HAS GSZ"; $resn_exch{"HAS2 GSZ"}="2HAS GSZ"; $resn_exch{" H   GT9"}="1HN  GT9";
$resn_exch{" H2  GT9"}="2HN  GT9"; $resn_exch{" HB2 GT9"}="1HB  GT9"; $resn_exch{" HB3 GT9"}="2HB  GT9";
$resn_exch{" H11 GT9"}="1H1  GT9"; $resn_exch{" H12 GT9"}="2H1  GT9"; $resn_exch{" H21 GT9"}="1H2  GT9";
$resn_exch{" H22 GT9"}="2H2  GT9"; $resn_exch{" H31 GT9"}="1H3  GT9"; $resn_exch{" H32 GT9"}="2H3  GT9";
$resn_exch{" H41 GT9"}="1H4  GT9"; $resn_exch{" H42 GT9"}="2H4  GT9"; $resn_exch{" H51 GT9"}="1H5  GT9";
$resn_exch{" H52 GT9"}="2H5  GT9"; $resn_exch{" H61 GT9"}="1H6  GT9"; $resn_exch{" H62 GT9"}="2H6  GT9";
$resn_exch{" H71 GT9"}="1H7  GT9"; $resn_exch{" H72 GT9"}="2H7  GT9"; $resn_exch{" H81 GT9"}="1H8  GT9";
$resn_exch{" H82 GT9"}="2H8  GT9"; $resn_exch{" H91 GT9"}="1H9  GT9"; $resn_exch{" H92 GT9"}="2H9  GT9";
$resn_exch{" H93 GT9"}="3H9  GT9"; $resn_exch{"HN21 GTA"}="1HN2 GTA"; $resn_exch{"HN22 GTA"}="2HN2 GTA";
$resn_exch{" H71 GTA"}="1H7  GTA"; $resn_exch{" H72 GTA"}="2H7  GTA"; $resn_exch{" H73 GTA"}="3H7  GTA";
$resn_exch{"HO3A GTA"}="AHO3 GTA"; $resn_exch{"H5A1 GTA"}="1H5A GTA"; $resn_exch{"H5A2 GTA"}="2H5A GTA";
$resn_exch{"HO2A GTA"}="AHO2 GTA"; $resn_exch{"H122 GTA"}="2H12 GTA"; $resn_exch{"H222 GTA"}="2H22 GTA";
$resn_exch{"H322 GTA"}="2H32 GTA"; $resn_exch{"H5B1 GTA"}="1H5B GTA"; $resn_exch{"H5B2 GTA"}="2H5B GTA";
$resn_exch{"HO3B GTA"}="BHO3 GTA"; $resn_exch{"HO2B GTA"}="BHO2 GTA"; $resn_exch{"HN61 GTA"}="1HN6 GTA";
$resn_exch{"HN62 GTA"}="2HN6 GTA"; $resn_exch{"HN11 GTB"}="1HN1 GTB"; $resn_exch{"HN12 GTB"}="2HN1 GTB";
$resn_exch{"HB11 GTB"}="1HB1 GTB"; $resn_exch{"HB12 GTB"}="2HB1 GTB"; $resn_exch{"HG11 GTB"}="1HG1 GTB";
$resn_exch{"HG12 GTB"}="2HG1 GTB"; $resn_exch{"HB21 GTB"}="1HB2 GTB"; $resn_exch{"HB22 GTB"}="2HB2 GTB";
$resn_exch{"HA31 GTB"}="1HA3 GTB"; $resn_exch{"HA32 GTB"}="2HA3 GTB"; $resn_exch{" H'1 GTB"}="1H'  GTB";
$resn_exch{" H'2 GTB"}="2H'  GTB"; $resn_exch{"HN11 GTD"}="1HN1 GTD"; $resn_exch{"HN12 GTD"}="2HN1 GTD";
$resn_exch{"HB11 GTD"}="1HB1 GTD"; $resn_exch{"HB12 GTD"}="2HB1 GTD"; $resn_exch{"HG11 GTD"}="1HG1 GTD";
$resn_exch{"HG12 GTD"}="2HG1 GTD"; $resn_exch{"HB21 GTD"}="1HB2 GTD"; $resn_exch{"HB22 GTD"}="2HB2 GTD";
$resn_exch{"HA31 GTD"}="1HA3 GTD"; $resn_exch{"HA32 GTD"}="2HA3 GTD"; $resn_exch{"HOA2 GTG"}="2HOA GTG";
$resn_exch{"HOB2 GTG"}="2HOB GTG"; $resn_exch{"HC71 GTG"}="1HC7 GTG"; $resn_exch{"HC72 GTG"}="2HC7 GTG";
$resn_exch{"HC73 GTG"}="3HC7 GTG"; $resn_exch{"HN21 GTG"}="1HN2 GTG"; $resn_exch{"HN22 GTG"}="2HN2 GTG";
$resn_exch{"H5'1 GTG"}="1H5* GTG"; $resn_exch{"H5'2 GTG"}="2H5* GTG"; $resn_exch{" H4' GTG"}=" H4* GTG";
$resn_exch{" H3' GTG"}=" H3* GTG"; $resn_exch{"HO3' GTG"}="*HO3 GTG"; $resn_exch{" H2' GTG"}=" H2* GTG";
$resn_exch{"HO2' GTG"}="*HO2 GTG"; $resn_exch{" H1' GTG"}=" H1* GTG"; $resn_exch{"HOG2 GTG"}="2HOG GTG";
$resn_exch{"H5B1 GTG"}="1H5B GTG"; $resn_exch{"H5B2 GTG"}="2H5B GTG"; $resn_exch{"H3BO GTG"}="OH3B GTG";
$resn_exch{"H2BO GTG"}="OH2B GTG"; $resn_exch{"H1BN GTG"}="NH1B GTG"; $resn_exch{"H2B1 GTG"}="1H2B GTG";
$resn_exch{"H2B2 GTG"}="2H2B GTG"; $resn_exch{"HN11 GTH"}="1HN1 GTH"; $resn_exch{"HN12 GTH"}="2HN1 GTH";
$resn_exch{" HG1 GTH"}="1HG  GTH"; $resn_exch{" HG2 GTH"}="2HG  GTH"; $resn_exch{" HG3 GTH"}="3HG  GTH";
$resn_exch{" H61 GTH"}="1H6  GTH"; $resn_exch{" H62 GTH"}="2H6  GTH"; $resn_exch{"H121 GTH"}="1H12 GTH";
$resn_exch{"H122 GTH"}="2H12 GTH"; $resn_exch{"H141 GTH"}="1H14 GTH"; $resn_exch{"H142 GTH"}="2H14 GTH";
$resn_exch{"H143 GTH"}="3H14 GTH"; $resn_exch{"H6C1 GTL"}="1H6C GTL"; $resn_exch{"H6C2 GTL"}="2H6C GTL";
$resn_exch{"H6C1 GTM"}="1H6C GTM"; $resn_exch{"H6C2 GTM"}="2H6C GTM"; $resn_exch{"H7C1 GTM"}="1H7C GTM";
$resn_exch{"H7C2 GTM"}="2H7C GTM"; $resn_exch{"H7C3 GTM"}="3H7C GTM"; $resn_exch{"HOG2 GTN"}="2HOG GTN";
$resn_exch{"HOG3 GTN"}="3HOG GTN"; $resn_exch{"HOB2 GTN"}="2HOB GTN"; $resn_exch{"HOA2 GTN"}="2HOA GTN";
$resn_exch{"H5'1 GTN"}="1H5* GTN"; $resn_exch{"H5'2 GTN"}="2H5* GTN"; $resn_exch{" H4' GTN"}=" H4* GTN";
$resn_exch{"H3'1 GTN"}="1H3* GTN"; $resn_exch{"H3'2 GTN"}="2H3* GTN"; $resn_exch{"H2'1 GTN"}="1H2* GTN";
$resn_exch{"H2'2 GTN"}="2H2* GTN"; $resn_exch{" H1' GTN"}=" H1* GTN"; $resn_exch{" H21 GTN"}="1H2  GTN";
$resn_exch{" H22 GTN"}="2H2  GTN"; $resn_exch{"HOG2 GTO"}="2HOG GTO"; $resn_exch{"HOG3 GTO"}="3HOG GTO";
$resn_exch{"H3B1 GTO"}="1H3B GTO"; $resn_exch{"H3B2 GTO"}="2H3B GTO"; $resn_exch{"HOB2 GTO"}="2HOB GTO";
$resn_exch{"HOA2 GTO"}="2HOA GTO"; $resn_exch{"H5'1 GTO"}="1H5* GTO"; $resn_exch{"H5'2 GTO"}="2H5* GTO";
$resn_exch{" H4' GTO"}=" H4* GTO"; $resn_exch{" H3' GTO"}=" H3* GTO"; $resn_exch{"HO3' GTO"}="*HO3 GTO";
$resn_exch{" H2' GTO"}=" H2* GTO"; $resn_exch{"HO2' GTO"}="*HO2 GTO"; $resn_exch{" H1' GTO"}=" H1* GTO";
$resn_exch{"HN21 GTO"}="1HN2 GTO"; $resn_exch{"HN22 GTO"}="2HN2 GTO"; $resn_exch{"HOG2 GTP"}="2HOG GTP";
$resn_exch{"HOG3 GTP"}="3HOG GTP"; $resn_exch{"HOB2 GTP"}="2HOB GTP"; $resn_exch{"HOA2 GTP"}="2HOA GTP";
$resn_exch{" H5' GTP"}="1H5* GTP"; $resn_exch{"H5'' GTP"}="2H5* GTP"; $resn_exch{" H4' GTP"}=" H4* GTP";
$resn_exch{" H3' GTP"}=" H3* GTP"; $resn_exch{"HO3' GTP"}="*HO3 GTP"; $resn_exch{" H2' GTP"}=" H2* GTP";
$resn_exch{"HO2' GTP"}="*HO2 GTP"; $resn_exch{" H1' GTP"}=" H1* GTP"; $resn_exch{"HN21 GTP"}="1HN2 GTP";
$resn_exch{"HN22 GTP"}="2HN2 GTP"; $resn_exch{"HN11 GTS"}="1HN1 GTS"; $resn_exch{"HN12 GTS"}="2HN1 GTS";
$resn_exch{"HB11 GTS"}="1HB1 GTS"; $resn_exch{"HB12 GTS"}="2HB1 GTS"; $resn_exch{"HG11 GTS"}="1HG1 GTS";
$resn_exch{"HG12 GTS"}="2HG1 GTS"; $resn_exch{"HB21 GTS"}="1HB2 GTS"; $resn_exch{"HB22 GTS"}="2HB2 GTS";
$resn_exch{"HOS3 GTS"}="3HOS GTS"; $resn_exch{"HA31 GTS"}="1HA3 GTS"; $resn_exch{"HA32 GTS"}="2HA3 GTS";
$resn_exch{"HN11 GTT"}="1HN1 GTT"; $resn_exch{"HN12 GTT"}="2HN1 GTT"; $resn_exch{"HN13 GTT"}="3HN1 GTT";
$resn_exch{"HB11 GTT"}="1HB1 GTT"; $resn_exch{"HB12 GTT"}="2HB1 GTT"; $resn_exch{"HG11 GTT"}="1HG1 GTT";
$resn_exch{"HG12 GTT"}="2HG1 GTT"; $resn_exch{"HB21 GTT"}="1HB2 GTT"; $resn_exch{"HB22 GTT"}="2HB2 GTT";
$resn_exch{"HA31 GTT"}="1HA3 GTT"; $resn_exch{"HA32 GTT"}="2HA3 GTT"; $resn_exch{"HN11 GTX"}="1HN1 GTX";
$resn_exch{"HN12 GTX"}="2HN1 GTX"; $resn_exch{"HN13 GTX"}="3HN1 GTX"; $resn_exch{"HB11 GTX"}="1HB1 GTX";
$resn_exch{"HB12 GTX"}="2HB1 GTX"; $resn_exch{"HG11 GTX"}="1HG1 GTX"; $resn_exch{"HG12 GTX"}="2HG1 GTX";
$resn_exch{"HB21 GTX"}="1HB2 GTX"; $resn_exch{"HB22 GTX"}="2HB2 GTX"; $resn_exch{"HS11 GTX"}="1HS1 GTX";
$resn_exch{"HS12 GTX"}="2HS1 GTX"; $resn_exch{"HS21 GTX"}="1HS2 GTX"; $resn_exch{"HS22 GTX"}="2HS2 GTX";
$resn_exch{"HS31 GTX"}="1HS3 GTX"; $resn_exch{"HS32 GTX"}="2HS3 GTX"; $resn_exch{"HS41 GTX"}="1HS4 GTX";
$resn_exch{"HS42 GTX"}="2HS4 GTX"; $resn_exch{"HS51 GTX"}="1HS5 GTX"; $resn_exch{"HS52 GTX"}="2HS5 GTX";
$resn_exch{"HS61 GTX"}="1HS6 GTX"; $resn_exch{"HS62 GTX"}="2HS6 GTX"; $resn_exch{"HS63 GTX"}="3HS6 GTX";
$resn_exch{"HA31 GTX"}="1HA3 GTX"; $resn_exch{"HA32 GTX"}="2HA3 GTX"; $resn_exch{"HN11 GTY"}="1HN1 GTY";
$resn_exch{"HN12 GTY"}="2HN1 GTY"; $resn_exch{"HB11 GTY"}="1HB1 GTY"; $resn_exch{"HB12 GTY"}="2HB1 GTY";
$resn_exch{"HG11 GTY"}="1HG1 GTY"; $resn_exch{"HG12 GTY"}="2HG1 GTY"; $resn_exch{"HB21 GTY"}="1HB2 GTY";
$resn_exch{"HB22 GTY"}="2HB2 GTY"; $resn_exch{"H1S1 GTY"}="1H1S GTY"; $resn_exch{"H1S2 GTY"}="2H1S GTY";
$resn_exch{"H2S1 GTY"}="1H2S GTY"; $resn_exch{"H2S2 GTY"}="2H2S GTY"; $resn_exch{"H3S1 GTY"}="1H3S GTY";
$resn_exch{"H3S2 GTY"}="2H3S GTY"; $resn_exch{"H4S1 GTY"}="1H4S GTY"; $resn_exch{"H4S2 GTY"}="2H4S GTY";
$resn_exch{"H5S1 GTY"}="1H5S GTY"; $resn_exch{"H5S2 GTY"}="2H5S GTY"; $resn_exch{"H6S1 GTY"}="1H6S GTY";
$resn_exch{"H6S2 GTY"}="2H6S GTY"; $resn_exch{"H7S1 GTY"}="1H7S GTY"; $resn_exch{"H7S2 GTY"}="2H7S GTY";
$resn_exch{"H8S1 GTY"}="1H8S GTY"; $resn_exch{"H8S2 GTY"}="2H8S GTY"; $resn_exch{"H8S3 GTY"}="3H8S GTY";
$resn_exch{"HA31 GTY"}="1HA3 GTY"; $resn_exch{"HA32 GTY"}="2HA3 GTY"; $resn_exch{" H61 GTZ"}="1H6  GTZ";
$resn_exch{" H62 GTZ"}="2H6  GTZ"; $resn_exch{" H61 GU0"}="1H6  GU0"; $resn_exch{" H62 GU0"}="2H6  GU0";
$resn_exch{" H71 GU1"}="1H7  GU1"; $resn_exch{" H72 GU1"}="2H7  GU1"; $resn_exch{" H73 GU1"}="3H7  GU1";
$resn_exch{" H81 GU1"}="1H8  GU1"; $resn_exch{" H82 GU1"}="2H8  GU1"; $resn_exch{" H83 GU1"}="3H8  GU1";
$resn_exch{" H71 GU2"}="1H7  GU2"; $resn_exch{" H72 GU2"}="2H7  GU2"; $resn_exch{" H73 GU2"}="3H7  GU2";
$resn_exch{" H81 GU2"}="1H8  GU2"; $resn_exch{" H82 GU2"}="2H8  GU2"; $resn_exch{" H83 GU2"}="3H8  GU2";
$resn_exch{" H61 GU3"}="1H6  GU3"; $resn_exch{" H62 GU3"}="2H6  GU3"; $resn_exch{" H81 GU3"}="1H8  GU3";
$resn_exch{" H82 GU3"}="2H8  GU3"; $resn_exch{" H83 GU3"}="3H8  GU3"; $resn_exch{" H71 GU3"}="1H7  GU3";
$resn_exch{" H72 GU3"}="2H7  GU3"; $resn_exch{" H73 GU3"}="3H7  GU3"; $resn_exch{" H61 GU4"}="1H6  GU4";
$resn_exch{" H62 GU4"}="2H6  GU4"; $resn_exch{" H71 GU5"}="1H7  GU5"; $resn_exch{" H72 GU5"}="2H7  GU5";
$resn_exch{" H73 GU5"}="3H7  GU5"; $resn_exch{" H81 GU5"}="1H8  GU5"; $resn_exch{" H82 GU5"}="2H8  GU5";
$resn_exch{" H83 GU5"}="3H8  GU5"; $resn_exch{" H61 GU5"}="1H6  GU5"; $resn_exch{" H62 GU5"}="2H6  GU5";
$resn_exch{" H61 GU6"}="1H6  GU6"; $resn_exch{" H62 GU6"}="2H6  GU6"; $resn_exch{"HN21 GU7"}="1HN2 GU7";
$resn_exch{"HN22 GU7"}="2HN2 GU7"; $resn_exch{"H101 GU7"}="1H10 GU7"; $resn_exch{"H102 GU7"}="2H10 GU7";
$resn_exch{"H111 GU7"}="1H11 GU7"; $resn_exch{"H112 GU7"}="2H11 GU7"; $resn_exch{"H141 GU7"}="1H14 GU7";
$resn_exch{"H142 GU7"}="2H14 GU7"; $resn_exch{"H161 GU7"}="1H16 GU7"; $resn_exch{"H162 GU7"}="2H16 GU7";
$resn_exch{" H91 GU8"}="1H9  GU8"; $resn_exch{" H92 GU8"}="2H9  GU8"; $resn_exch{" H93 GU8"}="3H9  GU8";
$resn_exch{" H61 GU8"}="1H6  GU8"; $resn_exch{" H62 GU8"}="2H6  GU8"; $resn_exch{" H71 GU8"}="1H7  GU8";
$resn_exch{" H72 GU8"}="2H7  GU8"; $resn_exch{" H73 GU8"}="3H7  GU8"; $resn_exch{" H81 GU8"}="1H8  GU8";
$resn_exch{" H82 GU8"}="2H8  GU8"; $resn_exch{" H83 GU8"}="3H8  GU8"; $resn_exch{" H61 GU9"}="1H6  GU9";
$resn_exch{" H62 GU9"}="2H6  GU9"; $resn_exch{" H91 GU9"}="1H9  GU9"; $resn_exch{" H92 GU9"}="2H9  GU9";
$resn_exch{" H93 GU9"}="3H9  GU9"; $resn_exch{" H81 GU9"}="1H8  GU9"; $resn_exch{" H82 GU9"}="2H8  GU9";
$resn_exch{" H83 GU9"}="3H8  GU9"; $resn_exch{" H71 GU9"}="1H7  GU9"; $resn_exch{" H72 GU9"}="2H7  GU9";
$resn_exch{" H73 GU9"}="3H7  GU9"; $resn_exch{" H21 GUA"}="1H2  GUA"; $resn_exch{" H22 GUA"}="2H2  GUA";
$resn_exch{" H31 GUA"}="1H3  GUA"; $resn_exch{" H32 GUA"}="2H3  GUA"; $resn_exch{" H41 GUA"}="1H4  GUA";
$resn_exch{" H42 GUA"}="2H4  GUA"; $resn_exch{" H1D GUD"}=" H1* GUD"; $resn_exch{" H2D GUD"}=" H2* GUD";
$resn_exch{"HO2' GUD"}="*HO2 GUD"; $resn_exch{" H3D GUD"}=" H3* GUD"; $resn_exch{" H4D GUD"}=" H4* GUD";
$resn_exch{"HO3' GUD"}="*HO3 GUD"; $resn_exch{"H5'1 GUD"}="1H5* GUD"; $resn_exch{"H5'2 GUD"}="2H5* GUD";
$resn_exch{"HOA2 GUD"}="2HOA GUD"; $resn_exch{"HOB2 GUD"}="2HOB GUD"; $resn_exch{"H6'1 GUD"}="1H6' GUD";
$resn_exch{"H6'2 GUD"}="2H6' GUD"; $resn_exch{"HO2A GUD"}="'HO2 GUD"; $resn_exch{"HO3A GUD"}="'HO3 GUD";
$resn_exch{"HO4' GUD"}="'HO4 GUD"; $resn_exch{"HO6' GUD"}="'HO6 GUD"; $resn_exch{" H61 GUL"}="1H6  GUL";
$resn_exch{" H62 GUL"}="2H6  GUL"; $resn_exch{"H101 GUM"}="1H10 GUM"; $resn_exch{"H102 GUM"}="2H10 GUM";
$resn_exch{"H103 GUM"}="3H10 GUM"; $resn_exch{"H161 GUM"}="1H16 GUM"; $resn_exch{"H162 GUM"}="2H16 GUM";
$resn_exch{"H181 GUM"}="1H18 GUM"; $resn_exch{"H182 GUM"}="2H18 GUM"; $resn_exch{"H183 GUM"}="3H18 GUM";
$resn_exch{"H241 GUM"}="1H24 GUM"; $resn_exch{"H242 GUM"}="2H24 GUM"; $resn_exch{"H261 GUM"}="1H26 GUM";
$resn_exch{"H262 GUM"}="2H26 GUM"; $resn_exch{"H263 GUM"}="3H26 GUM"; $resn_exch{"HN21 GUN"}="1HN2 GUN";
$resn_exch{"HN22 GUN"}="2HN2 GUN"; $resn_exch{" H61 GUP"}="1H6  GUP"; $resn_exch{" H62 GUP"}="2H6  GUP";
$resn_exch{"HN1' GUR"}="'HN1 GUR"; $resn_exch{"HN2' GUR"}="'HN2 GUR"; $resn_exch{" HA1 GVA"}="1HA  GVA";
$resn_exch{" HA2 GVA"}="2HA  GVA"; $resn_exch{" HB1 GVA"}="1HB  GVA"; $resn_exch{" HB2 GVA"}="2HB  GVA";
$resn_exch{" HG1 GVA"}="1HG  GVA"; $resn_exch{" HG2 GVA"}="2HG  GVA"; $resn_exch{" HD1 GVA"}="1HD  GVA";
$resn_exch{" HD2 GVA"}="2HD  GVA"; $resn_exch{"HH21 GVA"}="1HH2 GVA"; $resn_exch{"HH22 GVA"}="2HH2 GVA";
$resn_exch{"H361 GVC"}="1H36 GVC"; $resn_exch{"H362 GVC"}="2H36 GVC"; $resn_exch{"H363 GVC"}="3H36 GVC";
$resn_exch{"HAL1 GVD"}="1HAL GVD"; $resn_exch{"HAL2 GVD"}="2HAL GVD"; $resn_exch{"HAM1 GVD"}="1HAM GVD";
$resn_exch{"HAM2 GVD"}="2HAM GVD"; $resn_exch{"HA01 GVD"}="1HA0 GVD"; $resn_exch{"HA02 GVD"}="2HA0 GVD";
$resn_exch{" HN1 GVE"}="1HN  GVE"; $resn_exch{" HN2 GVE"}="2HN  GVE"; $resn_exch{" HA1 GVE"}="1HA  GVE";
$resn_exch{" HA2 GVE"}="2HA  GVE"; $resn_exch{" HB1 GVE"}="1HB  GVE"; $resn_exch{" HB2 GVE"}="2HB  GVE";
$resn_exch{" HG1 GVE"}="1HG  GVE"; $resn_exch{" HG2 GVE"}="2HG  GVE"; $resn_exch{"HH31 GVE"}="1HH3 GVE";
$resn_exch{"HH32 GVE"}="2HH3 GVE"; $resn_exch{"HH33 GVE"}="3HH3 GVE"; $resn_exch{"H2C1 GVG"}="1H2C GVG";
$resn_exch{"H2C2 GVG"}="2H2C GVG"; $resn_exch{"H2C3 GVG"}="3H2C GVG"; $resn_exch{"H9C1 GVI"}="1H9C GVI";
$resn_exch{"H9C2 GVI"}="2H9C GVI"; $resn_exch{"H1C1 GVI"}="1H1C GVI"; $resn_exch{"H1C2 GVI"}="2H1C GVI";
$resn_exch{"H1C3 GVI"}="3H1C GVI"; $resn_exch{"H1N1 GVJ"}="1H1N GVJ"; $resn_exch{"H1N2 GVJ"}="2H1N GVJ";
$resn_exch{"H9C1 GVK"}="1H9C GVK"; $resn_exch{"H9C2 GVK"}="2H9C GVK"; $resn_exch{"H151 GVK"}="1H15 GVK";
$resn_exch{"H152 GVK"}="2H15 GVK"; $resn_exch{"H101 GVK"}="1H10 GVK"; $resn_exch{"H102 GVK"}="2H10 GVK";
$resn_exch{"H141 GVK"}="1H14 GVK"; $resn_exch{"H142 GVK"}="2H14 GVK"; $resn_exch{" HN1 GVL"}="1HN  GVL";
$resn_exch{" HN2 GVL"}="2HN  GVL"; $resn_exch{"HBC1 GVL"}="1HBC GVL"; $resn_exch{"HBC2 GVL"}="2HBC GVL";
$resn_exch{"H281 GVL"}="1H28 GVL"; $resn_exch{"H282 GVL"}="2H28 GVL"; $resn_exch{"H301 GVL"}="1H30 GVL";
$resn_exch{"H302 GVL"}="2H30 GVL"; $resn_exch{"H303 GVL"}="3H30 GVL"; $resn_exch{"H311 GVL"}="1H31 GVL";
$resn_exch{"H312 GVL"}="2H31 GVL"; $resn_exch{"H313 GVL"}="3H31 GVL"; $resn_exch{"H361 GVL"}="1H36 GVL";
$resn_exch{"H362 GVL"}="2H36 GVL"; $resn_exch{"H2C1 GVM"}="1H2C GVM"; $resn_exch{"H2C2 GVM"}="2H2C GVM";
$resn_exch{"H4C1 GVM"}="1H4C GVM"; $resn_exch{"H4C2 GVM"}="2H4C GVM"; $resn_exch{"H5A1 GVM"}="1H5A GVM";
$resn_exch{"H5A2 GVM"}="2H5A GVM"; $resn_exch{"H5A3 GVM"}="3H5A GVM"; $resn_exch{"H5B1 GVM"}="1H5B GVM";
$resn_exch{"H5B2 GVM"}="2H5B GVM"; $resn_exch{"H5B3 GVM"}="3H5B GVM"; $resn_exch{"H5C1 GVM"}="1H5C GVM";
$resn_exch{"H5C2 GVM"}="2H5C GVM"; $resn_exch{"H5C3 GVM"}="3H5C GVM"; $resn_exch{"H1N1 GVN"}="1H1N GVN";
$resn_exch{"H1N2 GVN"}="2H1N GVN"; $resn_exch{"H2C1 GVN"}="1H2C GVN"; $resn_exch{"H2C2 GVN"}="2H2C GVN";
$resn_exch{"H1N1 GVO"}="1H1N GVO"; $resn_exch{"H1N2 GVO"}="2H1N GVO"; $resn_exch{"H2C1 GVO"}="1H2C GVO";
$resn_exch{"H2C2 GVO"}="2H2C GVO"; $resn_exch{"H9C1 GVP"}="1H9C GVP"; $resn_exch{"H9C2 GVP"}="2H9C GVP";
$resn_exch{"H131 GVP"}="1H13 GVP"; $resn_exch{"H132 GVP"}="2H13 GVP"; $resn_exch{"H101 GVP"}="1H10 GVP";
$resn_exch{"H102 GVP"}="2H10 GVP"; $resn_exch{"H121 GVP"}="1H12 GVP"; $resn_exch{"H122 GVP"}="2H12 GVP";
$resn_exch{"H151 GVQ"}="1H15 GVQ"; $resn_exch{"H152 GVQ"}="2H15 GVQ"; $resn_exch{"H1N1 GVQ"}="1H1N GVQ";
$resn_exch{"H1N2 GVQ"}="2H1N GVQ"; $resn_exch{"H251 GVR"}="1H25 GVR"; $resn_exch{"H252 GVR"}="2H25 GVR";
$resn_exch{"HN71 GW3"}="1HN7 GW3"; $resn_exch{"HN72 GW3"}="2HN7 GW3"; $resn_exch{"HN91 GW3"}="1HN9 GW3";
$resn_exch{"HN92 GW3"}="2HN9 GW3"; $resn_exch{"H271 GW3"}="1H27 GW3"; $resn_exch{"H272 GW3"}="2H27 GW3";
$resn_exch{"H301 GW3"}="1H30 GW3"; $resn_exch{"H302 GW3"}="2H30 GW3"; $resn_exch{"H303 GW3"}="3H30 GW3";
$resn_exch{"H341 GW3"}="1H34 GW3"; $resn_exch{"H342 GW3"}="2H34 GW3"; $resn_exch{"H371 GW3"}="1H37 GW3";
$resn_exch{"H372 GW3"}="2H37 GW3"; $resn_exch{"H373 GW3"}="3H37 GW3"; $resn_exch{" H31 GW4"}="1H3  GW4";
$resn_exch{" H32 GW4"}="2H3  GW4"; $resn_exch{" H33 GW4"}="3H3  GW4"; $resn_exch{"H111 GW4"}="1H11 GW4";
$resn_exch{"H112 GW4"}="2H11 GW4"; $resn_exch{"H121 GW4"}="1H12 GW4"; $resn_exch{"H122 GW4"}="2H12 GW4";
$resn_exch{"H321 GW4"}="1H32 GW4"; $resn_exch{"H322 GW4"}="2H32 GW4"; $resn_exch{"H331 GW4"}="1H33 GW4";
$resn_exch{"H332 GW4"}="2H33 GW4"; $resn_exch{"H351 GW4"}="1H35 GW4"; $resn_exch{"H352 GW4"}="2H35 GW4";
$resn_exch{"H353 GW4"}="3H35 GW4"; $resn_exch{"H21A GW5"}="1H2  GW5"; $resn_exch{"H22A GW5"}="2H2  GW5";
$resn_exch{" H11 GW5"}="1H1  GW5"; $resn_exch{"H12A GW5"}="2H1  GW5"; $resn_exch{"H13A GW5"}="3H1  GW5";
$resn_exch{" HD1 GWB"}="1HD  GWB"; $resn_exch{" HD2 GWB"}="2HD  GWB"; $resn_exch{" HE1 GWB"}="1HE  GWB";
$resn_exch{" HE2 GWB"}="2HE  GWB"; $resn_exch{"H121 GWB"}="1H12 GWB"; $resn_exch{"H122 GWB"}="2H12 GWB";
$resn_exch{"H123 GWB"}="3H12 GWB"; $resn_exch{"H131 GWB"}="1H13 GWB"; $resn_exch{"H132 GWB"}="2H13 GWB";
$resn_exch{"H133 GWB"}="3H13 GWB"; $resn_exch{"H6C1 GXL"}="1H6C GXL"; $resn_exch{"H6C2 GXL"}="2H6C GXL";
$resn_exch{" HN1 GYC"}="1HN  GYC"; $resn_exch{" HN2 GYC"}="2HN  GYC"; $resn_exch{"HB11 GYC"}="1HB1 GYC";
$resn_exch{"HB12 GYC"}="2HB1 GYC"; $resn_exch{"HA31 GYC"}="1HA3 GYC"; $resn_exch{"HA32 GYC"}="2HA3 GYC";
$resn_exch{"H6C1 GYP"}="1H6C GYP"; $resn_exch{"H6C2 GYP"}="2H6C GYP"; $resn_exch{"H7C1 GYP"}="1H7C GYP";
$resn_exch{"H7C2 GYP"}="2H7C GYP"; $resn_exch{"H7C3 GYP"}="3H7C GYP"; $resn_exch{" HN1 GYS"}="1HN  GYS";
$resn_exch{" HN2 GYS"}="2HN  GYS"; $resn_exch{"HB11 GYS"}="1HB1 GYS"; $resn_exch{"HB12 GYS"}="2HB1 GYS";
$resn_exch{"HA31 GYS"}="1HA3 GYS"; $resn_exch{"HA32 GYS"}="2HA3 GYS"; $resn_exch{"HH21 GZZ"}="1HH2 GZZ";
$resn_exch{"HH22 GZZ"}="2HH2 GZZ"; $resn_exch{"HN1E GZZ"}="EHN1 GZZ"; $resn_exch{"HC11 GZZ"}="1HC1 GZZ";
$resn_exch{"HC12 GZZ"}="2HC1 GZZ"; $resn_exch{"HC21 GZZ"}="1HC2 GZZ"; $resn_exch{"HC22 GZZ"}="2HC2 GZZ";
$resn_exch{"HC31 GZZ"}="1HC3 GZZ"; $resn_exch{"HC32 GZZ"}="2HC3 GZZ"; $resn_exch{"HC41 GZZ"}="1HC4 GZZ";
$resn_exch{"HC42 GZZ"}="2HC4 GZZ"; $resn_exch{"HC51 GZZ"}="1HC5 GZZ"; $resn_exch{"HC52 GZZ"}="2HC5 GZZ";
$resn_exch{"HC61 GZZ"}="1HC6 GZZ"; $resn_exch{"HC62 GZZ"}="2HC6 GZZ"; $resn_exch{"HC71 GZZ"}="1HC7 GZZ";
$resn_exch{"HC72 GZZ"}="2HC7 GZZ"; $resn_exch{"HC81 GZZ"}="1HC8 GZZ"; $resn_exch{"HC82 GZZ"}="2HC8 GZZ";
$resn_exch{" HN9 GZZ"}="9HN  GZZ"; $resn_exch{"H101 GZZ"}="1H10 GZZ"; $resn_exch{"H102 GZZ"}="2H10 GZZ";
$resn_exch{"H111 GZZ"}="1H11 GZZ"; $resn_exch{"H112 GZZ"}="2H11 GZZ"; $resn_exch{"H121 GZZ"}="1H12 GZZ";
$resn_exch{"H122 GZZ"}="2H12 GZZ"; $resn_exch{"H131 GZZ"}="1H13 GZZ"; $resn_exch{"H132 GZZ"}="2H13 GZZ";
$resn_exch{"H141 GZZ"}="1H14 GZZ"; $resn_exch{"H142 GZZ"}="2H14 GZZ"; $resn_exch{"H151 GZZ"}="1H15 GZZ";
$resn_exch{"H152 GZZ"}="2H15 GZZ"; $resn_exch{"H161 GZZ"}="1H16 GZZ"; $resn_exch{"H162 GZZ"}="2H16 GZZ";
$resn_exch{"H171 GZZ"}="1H17 GZZ"; $resn_exch{"H172 GZZ"}="2H17 GZZ"; $resn_exch{"HN2E GZZ"}="EHN2 GZZ";
$resn_exch{"HH31 GZZ"}="1HH3 GZZ"; $resn_exch{"HH32 GZZ"}="2HH3 GZZ"; $resn_exch{"H161 H01"}="1H16 H01";
$resn_exch{"H162 H01"}="2H16 H01"; $resn_exch{"H163 H01"}="3H16 H01"; $resn_exch{"H261 H01"}="1H26 H01";
$resn_exch{"H262 H01"}="2H26 H01"; $resn_exch{"H263 H01"}="3H26 H01"; $resn_exch{"H481 H01"}="1H48 H01";
$resn_exch{"H482 H01"}="2H48 H01"; $resn_exch{"H471 H01"}="1H47 H01"; $resn_exch{"H472 H01"}="2H47 H01";
$resn_exch{"H461 H01"}="1H46 H01"; $resn_exch{"H462 H01"}="2H46 H01"; $resn_exch{"H463 H01"}="3H46 H01";
$resn_exch{"H371 H01"}="1H37 H01"; $resn_exch{"H372 H01"}="2H37 H01"; $resn_exch{"H381 H01"}="1H38 H01";
$resn_exch{"H382 H01"}="2H38 H01"; $resn_exch{"H361 H01"}="1H36 H01"; $resn_exch{"H362 H01"}="2H36 H01";
$resn_exch{"H363 H01"}="3H36 H01"; $resn_exch{"HBA1 H02"}="1HBA H02"; $resn_exch{"HBA2 H02"}="2HBA H02";
$resn_exch{"HBA3 H02"}="3HBA H02"; $resn_exch{"HAA1 H02"}="1HAA H02"; $resn_exch{"HAA2 H02"}="2HAA H02";
$resn_exch{"HMA1 H02"}="1HMA H02"; $resn_exch{"HMA2 H02"}="2HMA H02"; $resn_exch{"HMA3 H02"}="3HMA H02";
$resn_exch{" HNA H02"}="HN_A H02"; $resn_exch{"HCB1 H02"}="1HCB H02"; $resn_exch{"HCB2 H02"}="2HCB H02";
$resn_exch{"HCB3 H02"}="3HCB H02"; $resn_exch{"HMB1 H02"}="1HMB H02"; $resn_exch{"HMB2 H02"}="2HMB H02";
$resn_exch{"HMB3 H02"}="3HMB H02"; $resn_exch{"HAB1 H02"}="1HAB H02"; $resn_exch{"HAB2 H02"}="2HAB H02";
$resn_exch{"HBB1 H02"}="1HBB H02"; $resn_exch{"HBB2 H02"}="2HBB H02"; $resn_exch{"HBB3 H02"}="3HBB H02";
$resn_exch{" HNC H02"}="HN_C H02"; $resn_exch{"HMC1 H02"}="1HMC H02"; $resn_exch{"HMC2 H02"}="2HMC H02";
$resn_exch{"HMC3 H02"}="3HMC H02"; $resn_exch{"HAC1 H02"}="1HAC H02"; $resn_exch{"HAC2 H02"}="2HAC H02";
$resn_exch{"HBC1 H02"}="1HBC H02"; $resn_exch{"HBC2 H02"}="2HBC H02"; $resn_exch{" HND H02"}="HN_D H02";
$resn_exch{"HMD1 H02"}="1HMD H02"; $resn_exch{"HMD2 H02"}="2HMD H02"; $resn_exch{"HMD3 H02"}="3HMD H02";
$resn_exch{"HAD1 H02"}="1HAD H02"; $resn_exch{"HAD2 H02"}="2HAD H02"; $resn_exch{"HBD1 H02"}="1HBD H02";
$resn_exch{"HBD2 H02"}="2HBD H02"; $resn_exch{" HB1 H12"}="1HB  H12"; $resn_exch{" HB2 H12"}="2HB  H12";
$resn_exch{" HC1 H12"}="1HC  H12"; $resn_exch{" HC2 H12"}="2HC  H12"; $resn_exch{" HD1 H12"}="1HD  H12";
$resn_exch{" HD2 H12"}="2HD  H12"; $resn_exch{" HE1 H12"}="1HE  H12"; $resn_exch{" HE2 H12"}="2HE  H12";
$resn_exch{" HF1 H12"}="1HF  H12"; $resn_exch{" HF2 H12"}="2HF  H12"; $resn_exch{"H111 H12"}="1H11 H12";
$resn_exch{"H112 H12"}="2H11 H12"; $resn_exch{"H121 H12"}="1H12 H12"; $resn_exch{"H122 H12"}="2H12 H12";
$resn_exch{"H131 H12"}="1H13 H12"; $resn_exch{"H132 H12"}="2H13 H12"; $resn_exch{"H133 H12"}="3H13 H12";
$resn_exch{" HB1 H16"}="1HB  H16"; $resn_exch{" HB2 H16"}="2HB  H16"; $resn_exch{" HC1 H16"}="1HC  H16";
$resn_exch{" HC2 H16"}="2HC  H16"; $resn_exch{" HD1 H16"}="1HD  H16"; $resn_exch{" HD2 H16"}="2HD  H16";
$resn_exch{" HE1 H16"}="1HE  H16"; $resn_exch{" HE2 H16"}="2HE  H16"; $resn_exch{" HF1 H16"}="1HF  H16";
$resn_exch{" HF2 H16"}="2HF  H16"; $resn_exch{"H111 H16"}="1H11 H16"; $resn_exch{"H112 H16"}="2H11 H16";
$resn_exch{"H121 H16"}="1H12 H16"; $resn_exch{"H122 H16"}="2H12 H16"; $resn_exch{"H131 H16"}="1H13 H16";
$resn_exch{"H132 H16"}="2H13 H16"; $resn_exch{"H133 H16"}="3H13 H16"; $resn_exch{" HB1 H18"}="1HB  H18";
$resn_exch{" HB2 H18"}="2HB  H18"; $resn_exch{" HC1 H18"}="1HC  H18"; $resn_exch{" HC2 H18"}="2HC  H18";
$resn_exch{" HD1 H18"}="1HD  H18"; $resn_exch{" HD2 H18"}="2HD  H18"; $resn_exch{" HE1 H18"}="1HE  H18";
$resn_exch{" HE2 H18"}="2HE  H18"; $resn_exch{" HF1 H18"}="1HF  H18"; $resn_exch{" HF2 H18"}="2HF  H18";
$resn_exch{"H111 H18"}="1H11 H18"; $resn_exch{"H112 H18"}="2H11 H18"; $resn_exch{"H121 H18"}="1H12 H18";
$resn_exch{"H122 H18"}="2H12 H18"; $resn_exch{"H131 H18"}="1H13 H18"; $resn_exch{"H132 H18"}="2H13 H18";
$resn_exch{"H133 H18"}="3H13 H18"; $resn_exch{"HN1A H1D"}="1HN  H1D"; $resn_exch{" HN2 H1D"}="2HN  H1D";
$resn_exch{" HB1 H1D"}="1HB  H1D"; $resn_exch{" HB2 H1D"}="2HB  H1D"; $resn_exch{" HG1 H1D"}="1HG  H1D";
$resn_exch{" HG2 H1D"}="2HG  H1D"; $resn_exch{" H51 H1D"}="1H5  H1D"; $resn_exch{" H52 H1D"}="2H5  H1D";
$resn_exch{"H2A1 H1L"}="1H2A H1L"; $resn_exch{"H2A2 H1L"}="2H2A H1L"; $resn_exch{"H2A3 H1L"}="3H2A H1L";
$resn_exch{"HAJ1 H1N"}="1HAJ H1N"; $resn_exch{"HAJ2 H1N"}="2HAJ H1N"; $resn_exch{"HAL1 H1N"}="1HAL H1N";
$resn_exch{"HAL2 H1N"}="2HAL H1N"; $resn_exch{"HAW1 H1N"}="1HAW H1N"; $resn_exch{"HAW2 H1N"}="2HAW H1N";
$resn_exch{"HAV1 H1N"}="1HAV H1N"; $resn_exch{"HAV2 H1N"}="2HAV H1N"; $resn_exch{"HAK1 H1N"}="1HAK H1N";
$resn_exch{"HAK2 H1N"}="2HAK H1N"; $resn_exch{"HAH1 H1N"}="1HAH H1N"; $resn_exch{"HAH2 H1N"}="2HAH H1N";
$resn_exch{"HAX1 H1N"}="1HAX H1N"; $resn_exch{"HAX2 H1N"}="2HAX H1N"; $resn_exch{"HAT1 H1N"}="1HAT H1N";
$resn_exch{"HAT2 H1N"}="2HAT H1N"; $resn_exch{"HAT3 H1N"}="3HAT H1N"; $resn_exch{" H61 H1S"}="1H6  H1S";
$resn_exch{" H62 H1S"}="2H6  H1S"; $resn_exch{" HB1 H20"}="1HB  H20"; $resn_exch{" HB2 H20"}="2HB  H20";
$resn_exch{" HC1 H20"}="1HC  H20"; $resn_exch{" HC2 H20"}="2HC  H20"; $resn_exch{" HD1 H20"}="1HD  H20";
$resn_exch{" HD2 H20"}="2HD  H20"; $resn_exch{" HE1 H20"}="1HE  H20"; $resn_exch{" HE2 H20"}="2HE  H20";
$resn_exch{" HF1 H20"}="1HF  H20"; $resn_exch{" HF2 H20"}="2HF  H20"; $resn_exch{"H121 H20"}="1H12 H20";
$resn_exch{"H122 H20"}="2H12 H20"; $resn_exch{"H123 H20"}="3H12 H20"; $resn_exch{"H131 H20"}="1H13 H20";
$resn_exch{"H132 H20"}="2H13 H20"; $resn_exch{"H133 H20"}="3H13 H20"; $resn_exch{"H1C1 H24"}="1H1C H24";
$resn_exch{"H1C2 H24"}="2H1C H24"; $resn_exch{"H1C3 H24"}="3H1C H24"; $resn_exch{"H151 H24"}="1H15 H24";
$resn_exch{"H152 H24"}="2H15 H24"; $resn_exch{"H153 H24"}="3H15 H24"; $resn_exch{"H181 H24"}="1H18 H24";
$resn_exch{"H182 H24"}="2H18 H24"; $resn_exch{"H221 H24"}="1H22 H24"; $resn_exch{"H222 H24"}="2H22 H24";
$resn_exch{"H223 H24"}="3H22 H24"; $resn_exch{"H241 H24"}="1H24 H24"; $resn_exch{"H242 H24"}="2H24 H24";
$resn_exch{"H21A H2A"}="1H2  H2A"; $resn_exch{" H22 H2A"}="2H2  H2A"; $resn_exch{"H3'1 H2A"}="1H3' H2A";
$resn_exch{"H3'2 H2A"}="2H3' H2A"; $resn_exch{"H131 H2A"}="1H13 H2A"; $resn_exch{"H132 H2A"}="2H13 H2A";
$resn_exch{"H141 H2A"}="1H14 H2A"; $resn_exch{"H142 H2A"}="2H14 H2A"; $resn_exch{"H151 H2A"}="1H15 H2A";
$resn_exch{"H152 H2A"}="2H15 H2A"; $resn_exch{"H161 H2A"}="1H16 H2A"; $resn_exch{"H162 H2A"}="2H16 H2A";
$resn_exch{"H241 H2A"}="1H24 H2A"; $resn_exch{"H242 H2A"}="2H24 H2A"; $resn_exch{"H251 H2A"}="1H25 H2A";
$resn_exch{"H252 H2A"}="2H25 H2A"; $resn_exch{"H253 H2A"}="3H25 H2A"; $resn_exch{"H261 H2A"}="1H26 H2A";
$resn_exch{"H262 H2A"}="2H26 H2A"; $resn_exch{"H263 H2A"}="3H26 H2A"; $resn_exch{" H22 H2B"}="2H2  H2B";
$resn_exch{" H21 H2B"}="1H2  H2B"; $resn_exch{" H71 H2B"}="1H7  H2B"; $resn_exch{" H72 H2B"}="2H7  H2B";
$resn_exch{"H9O1 H2B"}="1H9O H2B"; $resn_exch{"H113 H2B"}="3H11 H2B"; $resn_exch{"H112 H2B"}="2H11 H2B";
$resn_exch{"H111 H2B"}="1H11 H2B"; $resn_exch{"H1O1 H2B"}="1H1O H2B"; $resn_exch{" H61 H2P"}="1H6  H2P";
$resn_exch{" H62 H2P"}="2H6  H2P"; $resn_exch{" H71 H2P"}="1H7  H2P"; $resn_exch{" H72 H2P"}="2H7  H2P";
$resn_exch{" H73 H2P"}="3H7  H2P"; $resn_exch{"HOP2 H2P"}="2HOP H2P"; $resn_exch{"HOP3 H2P"}="3HOP H2P";
$resn_exch{" HS1 H2S"}="1HS  H2S"; $resn_exch{" HS2 H2S"}="2HS  H2S"; $resn_exch{"HOP2 H2U"}="2HOP H2U";
$resn_exch{"HOP3 H2U"}="3HOP H2U"; $resn_exch{" H5' H2U"}="1H5* H2U"; $resn_exch{"H5'' H2U"}="2H5* H2U";
$resn_exch{" H4' H2U"}=" H4* H2U"; $resn_exch{" H3' H2U"}=" H3* H2U"; $resn_exch{"HO3' H2U"}="*HO3 H2U";
$resn_exch{" H1' H2U"}=" H1* H2U"; $resn_exch{" H2' H2U"}=" H2* H2U"; $resn_exch{"HO2' H2U"}="*HO2 H2U";
$resn_exch{" H51 H2U"}="1H5  H2U"; $resn_exch{" H52 H2U"}="2H5  H2U"; $resn_exch{" H61 H2U"}="1H6  H2U";
$resn_exch{" H62 H2U"}="2H6  H2U"; $resn_exch{"H151 H33"}="1H15 H33"; $resn_exch{"H152 H33"}="2H15 H33";
$resn_exch{"H153 H33"}="3H15 H33"; $resn_exch{"H111 H33"}="1H11 H33"; $resn_exch{"H112 H33"}="2H11 H33";
$resn_exch{"H113 H33"}="3H11 H33"; $resn_exch{"HAP1 H35"}="1HAP H35"; $resn_exch{"HAP2 H35"}="2HAP H35";
$resn_exch{" H61 H3S"}="1H6  H3S"; $resn_exch{" H62 H3S"}="2H6  H3S"; $resn_exch{"HN21 H4B"}="1HN2 H4B";
$resn_exch{"HN22 H4B"}="2HN2 H4B"; $resn_exch{" H71 H4B"}="1H7  H4B"; $resn_exch{" H72 H4B"}="2H7  H4B";
$resn_exch{"H111 H4B"}="1H11 H4B"; $resn_exch{"H112 H4B"}="2H11 H4B"; $resn_exch{"H113 H4B"}="3H11 H4B";
$resn_exch{"HA21 H4M"}="1HA2 H4M"; $resn_exch{"HA22 H4M"}="2HA2 H4M"; $resn_exch{"H7M1 H4M"}="1H7M H4M";
$resn_exch{"H7M2 H4M"}="2H7M H4M"; $resn_exch{"H7M3 H4M"}="3H7M H4M"; $resn_exch{"H9M1 H4M"}="1H9M H4M";
$resn_exch{"H9M2 H4M"}="2H9M H4M"; $resn_exch{"H9M3 H4M"}="3H9M H4M"; $resn_exch{"H101 H4M"}="1H10 H4M";
$resn_exch{"H102 H4M"}="2H10 H4M"; $resn_exch{"HX11 H4M"}="1HX1 H4M"; $resn_exch{"HX12 H4M"}="2HX1 H4M";
$resn_exch{"HX51 H4M"}="1HX5 H4M"; $resn_exch{"HX52 H4M"}="2HX5 H4M"; $resn_exch{"HOX2 H4M"}="2HOX H4M";
$resn_exch{"HOX3 H4M"}="3HOX H4M"; $resn_exch{"HOX4 H4M"}="4HOX H4M"; $resn_exch{"H5J1 H4M"}="1H5J H4M";
$resn_exch{"H5J2 H4M"}="2H5J H4M"; $resn_exch{"HOJ3 H4M"}="3HOJ H4M"; $resn_exch{"HOJ2 H4M"}="2HOJ H4M";
$resn_exch{"HP31 H4M"}="1HP3 H4M"; $resn_exch{"HP32 H4M"}="2HP3 H4M"; $resn_exch{"HP41 H4M"}="1HP4 H4M";
$resn_exch{"HP42 H4M"}="2HP4 H4M"; $resn_exch{"HA51 H4P"}="1HA5 H4P"; $resn_exch{"HA52 H4P"}="2HA5 H4P";
$resn_exch{" H31 H4P"}="1H3  H4P"; $resn_exch{" H32 H4P"}="2H3  H4P"; $resn_exch{" H41 H4P"}="1H4  H4P";
$resn_exch{" H42 H4P"}="2H4  H4P"; $resn_exch{" H21 H4Z"}="1H2  H4Z"; $resn_exch{" H22 H4Z"}="2H2  H4Z";
$resn_exch{"H31A H4Z"}="1H3  H4Z"; $resn_exch{" H32 H4Z"}="2H3  H4Z"; $resn_exch{" H51 H4Z"}="1H5  H4Z";
$resn_exch{" H52 H4Z"}="2H5  H4Z"; $resn_exch{" H81 H4Z"}="1H8  H4Z"; $resn_exch{" H82 H4Z"}="2H8  H4Z";
$resn_exch{"H121 H4Z"}="1H12 H4Z"; $resn_exch{"H122 H4Z"}="2H12 H4Z"; $resn_exch{"H181 H4Z"}="1H18 H4Z";
$resn_exch{"H182 H4Z"}="2H18 H4Z"; $resn_exch{"H183 H4Z"}="3H18 H4Z"; $resn_exch{"HN41 H4Z"}="1HN4 H4Z";
$resn_exch{"HN42 H4Z"}="2HN4 H4Z"; $resn_exch{"H251 H4Z"}="1H25 H4Z"; $resn_exch{"H252 H4Z"}="2H25 H4Z";
$resn_exch{"H253 H4Z"}="3H25 H4Z"; $resn_exch{"H2M1 H52"}="1H2M H52"; $resn_exch{"H2M2 H52"}="2H2M H52";
$resn_exch{"H2M3 H52"}="3H2M H52"; $resn_exch{"H231 H52"}="1H23 H52"; $resn_exch{"H232 H52"}="2H23 H52";
$resn_exch{"H251 H52"}="1H25 H52"; $resn_exch{"H252 H52"}="2H25 H52"; $resn_exch{"H261 H52"}="1H26 H52";
$resn_exch{"H262 H52"}="2H26 H52"; $resn_exch{"H271 H52"}="1H27 H52"; $resn_exch{"H272 H52"}="2H27 H52";
$resn_exch{" HM1 H52"}="1HM  H52"; $resn_exch{" HM2 H52"}="2HM  H52"; $resn_exch{" HM3 H52"}="3HM  H52";
$resn_exch{" H61 H53"}="1H6  H53"; $resn_exch{" H62 H53"}="2H6  H53"; $resn_exch{" H   H5M"}=" HN  H5M";
$resn_exch{" HG2 H5M"}="1HG  H5M"; $resn_exch{" HG3 H5M"}="2HG  H5M"; $resn_exch{"HD11 H5M"}="1HD1 H5M";
$resn_exch{"HD12 H5M"}="2HD1 H5M"; $resn_exch{"HD13 H5M"}="3HD1 H5M"; $resn_exch{"HO2' H5P"}="'HO2 H5P";
$resn_exch{"HO3' H5P"}="'HO3 H5P"; $resn_exch{"H5'1 H5P"}="1H5' H5P"; $resn_exch{"H5'2 H5P"}="2H5' H5P";
$resn_exch{"HOP2 H5P"}="2HOP H5P"; $resn_exch{"HOP3 H5P"}="3HOP H5P"; $resn_exch{"H971 H64"}="1H97 H64";
$resn_exch{"H972 H64"}="2H97 H64"; $resn_exch{"H973 H64"}="3H97 H64"; $resn_exch{"H961 H64"}="1H96 H64";
$resn_exch{"H962 H64"}="2H96 H64"; $resn_exch{"H963 H64"}="3H96 H64"; $resn_exch{"H931 H64"}="1H93 H64";
$resn_exch{"H932 H64"}="2H93 H64"; $resn_exch{"H921 H64"}="1H92 H64"; $resn_exch{"H922 H64"}="2H92 H64";
$resn_exch{"H911 H64"}="1H91 H64"; $resn_exch{"H912 H64"}="2H91 H64"; $resn_exch{"HN61 H64"}="1HN6 H64";
$resn_exch{"HN62 H64"}="2HN6 H64"; $resn_exch{"H'21 H64"}="1H'2 H64"; $resn_exch{"H'22 H64"}="2H'2 H64";
$resn_exch{"H961 H71"}="1H96 H71"; $resn_exch{"H962 H71"}="2H96 H71"; $resn_exch{"H963 H71"}="3H96 H71";
$resn_exch{"H971 H71"}="1H97 H71"; $resn_exch{"H972 H71"}="2H97 H71"; $resn_exch{"H973 H71"}="3H97 H71";
$resn_exch{"H931 H71"}="1H93 H71"; $resn_exch{"H932 H71"}="2H93 H71"; $resn_exch{"H921 H71"}="1H92 H71";
$resn_exch{"H922 H71"}="2H92 H71"; $resn_exch{"H911 H71"}="1H91 H71"; $resn_exch{"H912 H71"}="2H91 H71";
$resn_exch{"HN61 H71"}="1HN6 H71"; $resn_exch{"HN62 H71"}="2HN6 H71"; $resn_exch{"H'21 H71"}="1H'2 H71";
$resn_exch{"H'22 H71"}="2H'2 H71"; $resn_exch{"H161 H7J"}="1H16 H7J"; $resn_exch{"H162 H7J"}="2H16 H7J";
$resn_exch{"H163 H7J"}="3H16 H7J"; $resn_exch{"H271 H7J"}="1H27 H7J"; $resn_exch{"H272 H7J"}="2H27 H7J";
$resn_exch{"H273 H7J"}="3H27 H7J"; $resn_exch{"H261 H7J"}="1H26 H7J"; $resn_exch{"H262 H7J"}="2H26 H7J";
$resn_exch{"H263 H7J"}="3H26 H7J"; $resn_exch{" H1  H7J"}="1H   H7J"; $resn_exch{" H2  H7J"}="2H   H7J";
$resn_exch{" H3  H7J"}="3H   H7J"; $resn_exch{" H51 H7J"}="1H5  H7J"; $resn_exch{" H52 H7J"}="2H5  H7J";
$resn_exch{" H91 H7J"}="1H9  H7J"; $resn_exch{" H92 H7J"}="2H9  H7J"; $resn_exch{" H93 H7J"}="3H9  H7J";
$resn_exch{" H11 H8H"}="1H1  H8H"; $resn_exch{" H12 H8H"}="2H1  H8H"; $resn_exch{"H13A H8H"}="3H1  H8H";
$resn_exch{" H31 H8H"}="1H3  H8H"; $resn_exch{" H32 H8H"}="2H3  H8H"; $resn_exch{" H41 H8H"}="1H4  H8H";
$resn_exch{" H42 H8H"}="2H4  H8H"; $resn_exch{" H61 H8H"}="1H6  H8H"; $resn_exch{" H62 H8H"}="2H6  H8H";
$resn_exch{" H71 H8H"}="1H7  H8H"; $resn_exch{" H72 H8H"}="2H7  H8H"; $resn_exch{"H141 H8H"}="1H14 H8H";
$resn_exch{"H142 H8H"}="2H14 H8H"; $resn_exch{"H151 H8H"}="1H15 H8H"; $resn_exch{"H152 H8H"}="2H15 H8H";
$resn_exch{"H171 H8H"}="1H17 H8H"; $resn_exch{"H172 H8H"}="2H17 H8H"; $resn_exch{"H181 H8H"}="1H18 H8H";
$resn_exch{"H182 H8H"}="2H18 H8H"; $resn_exch{"H251 H8H"}="1H25 H8H"; $resn_exch{"H252 H8H"}="2H25 H8H";
$resn_exch{"H371 H8H"}="1H37 H8H"; $resn_exch{"H372 H8H"}="2H37 H8H"; $resn_exch{"H381 H8H"}="1H38 H8H";
$resn_exch{"H382 H8H"}="2H38 H8H"; $resn_exch{"HN21 HA1"}="1HN2 HA1"; $resn_exch{"HN22 HA1"}="2HN2 HA1";
$resn_exch{"H101 HA1"}="1H10 HA1"; $resn_exch{"H102 HA1"}="2H10 HA1"; $resn_exch{"H111 HA1"}="1H11 HA1";
$resn_exch{"H112 HA1"}="2H11 HA1"; $resn_exch{"H121 HA1"}="1H12 HA1"; $resn_exch{"H122 HA1"}="2H12 HA1";
$resn_exch{"H131 HA1"}="1H13 HA1"; $resn_exch{"H132 HA1"}="2H13 HA1"; $resn_exch{"H7C1 HA2"}="1H7C HA2";
$resn_exch{"H7C2 HA2"}="2H7C HA2"; $resn_exch{"H141 HA2"}="1H14 HA2"; $resn_exch{"H142 HA2"}="2H14 HA2";
$resn_exch{"H2N1 HA2"}="1H2N HA2"; $resn_exch{"H2N2 HA2"}="2H2N HA2"; $resn_exch{"H161 HA2"}="1H16 HA2";
$resn_exch{"H162 HA2"}="2H16 HA2"; $resn_exch{"H181 HA2"}="1H18 HA2"; $resn_exch{"H182 HA2"}="2H18 HA2";
$resn_exch{"H191 HA2"}="1H19 HA2"; $resn_exch{"H192 HA2"}="2H19 HA2"; $resn_exch{"H201 HA2"}="1H20 HA2";
$resn_exch{"H202 HA2"}="2H20 HA2"; $resn_exch{"H211 HA2"}="1H21 HA2"; $resn_exch{"H212 HA2"}="2H21 HA2";
$resn_exch{" H21 HAA"}="1H2  HAA"; $resn_exch{" H22 HAA"}="2H2  HAA"; $resn_exch{"HO4' HAB"}="'HO4 HAB";
$resn_exch{" H21 HAC"}="1H2  HAC"; $resn_exch{" H22 HAC"}="2H2  HAC"; $resn_exch{" H31 HAC"}="1H3  HAC";
$resn_exch{" H32 HAC"}="2H3  HAC"; $resn_exch{" H41 HAC"}="1H4  HAC"; $resn_exch{" H42 HAC"}="2H4  HAC";
$resn_exch{" H51 HAC"}="1H5  HAC"; $resn_exch{" H52 HAC"}="2H5  HAC"; $resn_exch{" H61 HAC"}="1H6  HAC";
$resn_exch{" H62 HAC"}="2H6  HAC"; $resn_exch{" H71 HAC"}="1H7  HAC"; $resn_exch{" H72 HAC"}="2H7  HAC";
$resn_exch{"HN81 HAC"}="1HN8 HAC"; $resn_exch{"HN82 HAC"}="2HN8 HAC"; $resn_exch{" H11 HAD"}="1H1  HAD";
$resn_exch{" H12 HAD"}="2H1  HAD"; $resn_exch{" H11 HAE"}="1H1  HAE"; $resn_exch{" H12 HAE"}="2H1  HAE";
$resn_exch{" H13 HAE"}="3H1  HAE"; $resn_exch{"HN11 HAG"}="1HN1 HAG"; $resn_exch{"HN12 HAG"}="2HN1 HAG";
$resn_exch{"HB11 HAG"}="1HB1 HAG"; $resn_exch{"HB12 HAG"}="2HB1 HAG"; $resn_exch{"HG12 HAG"}="2HG1 HAG";
$resn_exch{"HG11 HAG"}="1HG1 HAG"; $resn_exch{"HN21 HAG"}="1HN2 HAG"; $resn_exch{"HB21 HAG"}="1HB2 HAG";
$resn_exch{"HB22 HAG"}="2HB2 HAG"; $resn_exch{"HN31 HAG"}="1HN3 HAG"; $resn_exch{"HA31 HAG"}="1HA3 HAG";
$resn_exch{"HA32 HAG"}="2HA3 HAG"; $resn_exch{"H421 HAG"}="1H42 HAG"; $resn_exch{"H422 HAG"}="2H42 HAG";
$resn_exch{"H451 HAG"}="1H45 HAG"; $resn_exch{"H452 HAG"}="2H45 HAG"; $resn_exch{"H461 HAG"}="1H46 HAG";
$resn_exch{"H462 HAG"}="2H46 HAG"; $resn_exch{"H471 HAG"}="1H47 HAG"; $resn_exch{"H472 HAG"}="2H47 HAG";
$resn_exch{"H481 HAG"}="1H48 HAG"; $resn_exch{"H482 HAG"}="2H48 HAG"; $resn_exch{"H491 HAG"}="1H49 HAG";
$resn_exch{"H492 HAG"}="2H49 HAG"; $resn_exch{"H493 HAG"}="3H49 HAG"; $resn_exch{" H21 HAI"}="1H2  HAI";
$resn_exch{" H22 HAI"}="2H2  HAI"; $resn_exch{" H31 HAI"}="1H3  HAI"; $resn_exch{" H32 HAI"}="2H3  HAI";
$resn_exch{" H41 HAI"}="1H4  HAI"; $resn_exch{" H42 HAI"}="2H4  HAI"; $resn_exch{" H51 HAI"}="1H5  HAI";
$resn_exch{" H52 HAI"}="2H5  HAI"; $resn_exch{" H61 HAI"}="1H6  HAI"; $resn_exch{" H62 HAI"}="2H6  HAI";
$resn_exch{" HN1 HAI"}="1HN  HAI"; $resn_exch{" HN2 HAI"}="2HN  HAI"; $resn_exch{" HN3 HAI"}="3HN  HAI";
$resn_exch{"HC11 HAL"}="1HC1 HAL"; $resn_exch{"HC12 HAL"}="2HC1 HAL"; $resn_exch{"HC21 HAL"}="1HC2 HAL";
$resn_exch{"HC22 HAL"}="2HC2 HAL"; $resn_exch{"HC41 HAL"}="1HC4 HAL"; $resn_exch{"HC42 HAL"}="2HC4 HAL";
$resn_exch{"HC61 HAL"}="1HC6 HAL"; $resn_exch{"HC62 HAL"}="2HC6 HAL"; $resn_exch{"HC81 HAL"}="1HC8 HAL";
$resn_exch{"HC82 HAL"}="2HC8 HAL"; $resn_exch{"HC91 HAL"}="1HC9 HAL"; $resn_exch{"HC92 HAL"}="2HC9 HAL";
$resn_exch{"H101 HAL"}="1H10 HAL"; $resn_exch{"H102 HAL"}="2H10 HAL"; $resn_exch{"H5'1 HAM"}="1H5* HAM";
$resn_exch{"H5'2 HAM"}="2H5* HAM"; $resn_exch{" H4' HAM"}=" H4* HAM"; $resn_exch{" H3' HAM"}=" H3* HAM";
$resn_exch{"HO3' HAM"}="*HO3 HAM"; $resn_exch{" H2' HAM"}=" H2* HAM"; $resn_exch{"HO2' HAM"}="*HO2 HAM";
$resn_exch{" H1' HAM"}=" H1* HAM"; $resn_exch{"HN61 HAM"}="1HN6 HAM"; $resn_exch{"HN62 HAM"}="2HN6 HAM";
$resn_exch{" HN1 HAM"}="1HN  HAM"; $resn_exch{" HN2 HAM"}="2HN  HAM"; $resn_exch{" HN3 HAM"}="3HN  HAM";
$resn_exch{" HB1 HAM"}="1HB  HAM"; $resn_exch{" HB2 HAM"}="2HB  HAM"; $resn_exch{"H101 HAN"}="1H10 HAN";
$resn_exch{"H102 HAN"}="2H10 HAN"; $resn_exch{"H103 HAN"}="3H10 HAN"; $resn_exch{"H111 HAN"}="1H11 HAN";
$resn_exch{"H112 HAN"}="2H11 HAN"; $resn_exch{"H113 HAN"}="3H11 HAN"; $resn_exch{"H121 HAN"}="1H12 HAN";
$resn_exch{"H122 HAN"}="2H12 HAN"; $resn_exch{"H131 HAN"}="1H13 HAN"; $resn_exch{"H132 HAN"}="2H13 HAN";
$resn_exch{"H141 HAN"}="1H14 HAN"; $resn_exch{"H142 HAN"}="2H14 HAN"; $resn_exch{"H151 HAN"}="1H15 HAN";
$resn_exch{"H152 HAN"}="2H15 HAN"; $resn_exch{"H161 HAN"}="1H16 HAN"; $resn_exch{"H162 HAN"}="2H16 HAN";
$resn_exch{" H21 HAP"}="1H2  HAP"; $resn_exch{" H22 HAP"}="2H2  HAP"; $resn_exch{" H41 HAP"}="1H4  HAP";
$resn_exch{" H42 HAP"}="2H4  HAP"; $resn_exch{" H61 HAP"}="1H6  HAP"; $resn_exch{" H62 HAP"}="2H6  HAP";
$resn_exch{" H63 HAP"}="3H6  HAP"; $resn_exch{" H71 HAP"}="1H7  HAP"; $resn_exch{" H72 HAP"}="2H7  HAP";
$resn_exch{" H73 HAP"}="3H7  HAP"; $resn_exch{"H111 HAP"}="1H11 HAP"; $resn_exch{"H112 HAP"}="2H11 HAP";
$resn_exch{"H113 HAP"}="3H11 HAP"; $resn_exch{"H121 HAP"}="1H12 HAP"; $resn_exch{"H122 HAP"}="2H12 HAP";
$resn_exch{" H   HAQ"}="1HN  HAQ"; $resn_exch{" H2  HAQ"}="2HN  HAQ"; $resn_exch{" HA1 HAQ"}="1HA  HAQ";
$resn_exch{" HB2 HAQ"}="1HB  HAQ"; $resn_exch{" HB3 HAQ"}="2HB  HAQ"; $resn_exch{" HG2 HAQ"}="1HG  HAQ";
$resn_exch{" HG3 HAQ"}="2HG  HAQ"; $resn_exch{" HA2 HAQ"}="1HA2 HAQ"; $resn_exch{"HB23 HAQ"}="1HB2 HAQ";
$resn_exch{"HB22 HAQ"}="2HB2 HAQ"; $resn_exch{" HD2 HAQ"}="1HD2 HAQ"; $resn_exch{" HE2 HAQ"}="1HE2 HAQ";
$resn_exch{" HZ  HAQ"}="1HZ  HAQ"; $resn_exch{" H2  HAR"}=" HN2 HAR"; $resn_exch{" HB2 HAR"}="1HB  HAR";
$resn_exch{" HB3 HAR"}="2HB  HAR"; $resn_exch{" HG2 HAR"}="1HG  HAR"; $resn_exch{" HG3 HAR"}="2HG  HAR";
$resn_exch{" HD2 HAR"}="1HD  HAR"; $resn_exch{" HD3 HAR"}="2HD  HAR"; $resn_exch{"HMA1 HAS"}="1HMA HAS";
$resn_exch{"HMA2 HAS"}="2HMA HAS"; $resn_exch{"HMA3 HAS"}="3HMA HAS"; $resn_exch{"HAA1 HAS"}="1HAA HAS";
$resn_exch{"HAA2 HAS"}="2HAA HAS"; $resn_exch{"HBA1 HAS"}="1HBA HAS"; $resn_exch{"HBA2 HAS"}="2HBA HAS";
$resn_exch{"HMB1 HAS"}="1HMB HAS"; $resn_exch{"HMB2 HAS"}="2HMB HAS"; $resn_exch{"HMB3 HAS"}="3HMB HAS";
$resn_exch{"HMC1 HAS"}="1HMC HAS"; $resn_exch{"HMC2 HAS"}="2HMC HAS"; $resn_exch{"HMC3 HAS"}="3HMC HAS";
$resn_exch{"HBC1 HAS"}="1HBC HAS"; $resn_exch{"HBC2 HAS"}="2HBC HAS"; $resn_exch{"HAD1 HAS"}="1HAD HAS";
$resn_exch{"HAD2 HAS"}="2HAD HAS"; $resn_exch{"HBD1 HAS"}="1HBD HAS"; $resn_exch{"HBD2 HAS"}="2HBD HAS";
$resn_exch{"H121 HAS"}="1H12 HAS"; $resn_exch{"H122 HAS"}="2H12 HAS"; $resn_exch{"H131 HAS"}="1H13 HAS";
$resn_exch{"H132 HAS"}="2H13 HAS"; $resn_exch{"H161 HAS"}="1H16 HAS"; $resn_exch{"H162 HAS"}="2H16 HAS";
$resn_exch{"H171 HAS"}="1H17 HAS"; $resn_exch{"H172 HAS"}="2H17 HAS"; $resn_exch{"H201 HAS"}="1H20 HAS";
$resn_exch{"H202 HAS"}="2H20 HAS"; $resn_exch{"H211 HAS"}="1H21 HAS"; $resn_exch{"H212 HAS"}="2H21 HAS";
$resn_exch{"H241 HAS"}="1H24 HAS"; $resn_exch{"H242 HAS"}="2H24 HAS"; $resn_exch{"H251 HAS"}="1H25 HAS";
$resn_exch{"H252 HAS"}="2H25 HAS"; $resn_exch{"H253 HAS"}="3H25 HAS"; $resn_exch{"H261 HAS"}="1H26 HAS";
$resn_exch{"H262 HAS"}="2H26 HAS"; $resn_exch{"H263 HAS"}="3H26 HAS"; $resn_exch{"H271 HAS"}="1H27 HAS";
$resn_exch{"H272 HAS"}="2H27 HAS"; $resn_exch{"H273 HAS"}="3H27 HAS"; $resn_exch{"H281 HAS"}="1H28 HAS";
$resn_exch{"H282 HAS"}="2H28 HAS"; $resn_exch{"H311 HAS"}="1H31 HAS"; $resn_exch{"H312 HAS"}="2H31 HAS";
$resn_exch{"H313 HAS"}="3H31 HAS"; $resn_exch{"H321 HAS"}="1H32 HAS"; $resn_exch{"H322 HAS"}="2H32 HAS";
$resn_exch{"H323 HAS"}="3H32 HAS"; $resn_exch{"HNA1 HAV"}="1HNA HAV"; $resn_exch{"HNA2 HAV"}="2HNA HAV";
$resn_exch{"HG11 HAV"}="1HG1 HAV"; $resn_exch{"HG12 HAV"}="2HG1 HAV"; $resn_exch{"HG13 HAV"}="3HG1 HAV";
$resn_exch{"HG21 HAV"}="1HG2 HAV"; $resn_exch{"HG22 HAV"}="2HG2 HAV"; $resn_exch{"HG23 HAV"}="3HG2 HAV";
$resn_exch{" H2A HAX"}=" AH2 HAX"; $resn_exch{"H61A HAX"}="AH61 HAX"; $resn_exch{"H62A HAX"}="AH62 HAX";
$resn_exch{" H8A HAX"}=" AH8 HAX"; $resn_exch{" H1B HAX"}="AH1* HAX"; $resn_exch{" H2B HAX"}="AH2* HAX";
$resn_exch{"HO2A HAX"}="AHO2 HAX"; $resn_exch{" H3B HAX"}="AH3* HAX"; $resn_exch{"HOA8 HAX"}="8HOA HAX";
$resn_exch{"HOA9 HAX"}="9HOA HAX"; $resn_exch{" H4B HAX"}="AH4* HAX"; $resn_exch{"H51A HAX"}="AH51 HAX";
$resn_exch{"H52A HAX"}="AH52 HAX"; $resn_exch{"HOA2 HAX"}="2HOA HAX"; $resn_exch{"HOA5 HAX"}="5HOA HAX";
$resn_exch{"H121 HAX"}="1H12 HAX"; $resn_exch{"H122 HAX"}="2H12 HAX"; $resn_exch{"H131 HAX"}="1H13 HAX";
$resn_exch{"H132 HAX"}="2H13 HAX"; $resn_exch{"H133 HAX"}="3H13 HAX"; $resn_exch{"H141 HAX"}="1H14 HAX";
$resn_exch{"H142 HAX"}="2H14 HAX"; $resn_exch{"H143 HAX"}="3H14 HAX"; $resn_exch{" H71 HAX"}="1H7  HAX";
$resn_exch{" H72 HAX"}="2H7  HAX"; $resn_exch{" H61 HAX"}="1H6  HAX"; $resn_exch{" H62 HAX"}="2H6  HAX";
$resn_exch{" H31 HAX"}="1H3  HAX"; $resn_exch{" H32 HAX"}="2H3  HAX"; $resn_exch{" H21 HAX"}="1H2  HAX";
$resn_exch{" H22 HAX"}="2H2  HAX"; $resn_exch{" H11 HAX"}="1H1  HAX"; $resn_exch{" H12 HAX"}="2H1  HAX";
$resn_exch{"H221 HAZ"}="1H22 HAZ"; $resn_exch{"H222 HAZ"}="2H22 HAZ"; $resn_exch{"H231 HAZ"}="1H23 HAZ";
$resn_exch{"H232 HAZ"}="2H23 HAZ"; $resn_exch{"H241 HAZ"}="1H24 HAZ"; $resn_exch{"H242 HAZ"}="2H24 HAZ";
$resn_exch{" H11 HAZ"}="1H1  HAZ"; $resn_exch{" H12 HAZ"}="2H1  HAZ"; $resn_exch{"H111 HAZ"}="1H11 HAZ";
$resn_exch{"H112 HAZ"}="2H11 HAZ"; $resn_exch{"H113 HAZ"}="3H11 HAZ"; $resn_exch{"H121 HAZ"}="1H12 HAZ";
$resn_exch{"H122 HAZ"}="2H12 HAZ"; $resn_exch{"H123 HAZ"}="3H12 HAZ"; $resn_exch{" H21 HAZ"}="1H2  HAZ";
$resn_exch{" H22 HAZ"}="2H2  HAZ"; $resn_exch{" H31 HAZ"}="1H3  HAZ"; $resn_exch{" H32 HAZ"}="2H3  HAZ";
$resn_exch{"H131 HAZ"}="1H13 HAZ"; $resn_exch{"H132 HAZ"}="2H13 HAZ"; $resn_exch{" H61 HAZ"}="1H6  HAZ";
$resn_exch{" H62 HAZ"}="2H6  HAZ"; $resn_exch{" H71 HAZ"}="1H7  HAZ"; $resn_exch{" H72 HAZ"}="2H7  HAZ";
$resn_exch{"H6C1 HB1"}="1H6C HB1"; $resn_exch{"H7C1 HB1"}="1H7C HB1"; $resn_exch{"H8C1 HB1"}="1H8C HB1";
$resn_exch{"H9C1 HB1"}="1H9C HB1"; $resn_exch{" H21 HBB"}="1H2  HBB"; $resn_exch{"H22A HBB"}="2H2  HBB";
$resn_exch{"H101 HBB"}="1H10 HBB"; $resn_exch{"H102 HBB"}="2H10 HBB"; $resn_exch{"H111 HBB"}="1H11 HBB";
$resn_exch{"H112 HBB"}="2H11 HBB"; $resn_exch{"H121 HBB"}="1H12 HBB"; $resn_exch{"H122 HBB"}="2H12 HBB";
$resn_exch{"H191 HBB"}="1H19 HBB"; $resn_exch{"H192 HBB"}="2H19 HBB"; $resn_exch{"H193 HBB"}="3H19 HBB";
$resn_exch{"H271 HBB"}="1H27 HBB"; $resn_exch{"H272 HBB"}="2H27 HBB"; $resn_exch{"H301 HBB"}="1H30 HBB";
$resn_exch{"H302 HBB"}="2H30 HBB"; $resn_exch{"H303 HBB"}="3H30 HBB"; $resn_exch{"H311 HBB"}="1H31 HBB";
$resn_exch{"H312 HBB"}="2H31 HBB"; $resn_exch{"H313 HBB"}="3H31 HBB"; $resn_exch{"H181 HBB"}="1H18 HBB";
$resn_exch{"H182 HBB"}="2H18 HBB"; $resn_exch{"H183 HBB"}="3H18 HBB"; $resn_exch{"H251 HBB"}="1H25 HBB";
$resn_exch{"H252 HBB"}="2H25 HBB"; $resn_exch{"H281 HBB"}="1H28 HBB"; $resn_exch{"H282 HBB"}="2H28 HBB";
$resn_exch{" H21 HBC"}="1H2  HBC"; $resn_exch{" H31 HBC"}="1H3  HBC"; $resn_exch{" H41 HBC"}="1H4  HBC";
$resn_exch{" H51 HBC"}="1H5  HBC"; $resn_exch{" H61 HBC"}="1H6  HBC"; $resn_exch{" H71 HBC"}="1H7  HBC";
$resn_exch{" H81 HBC"}="1H8  HBC"; $resn_exch{" H82 HBC"}="2H8  HBC"; $resn_exch{" H91 HBC"}="1H9  HBC";
$resn_exch{" H92 HBC"}="2H9  HBC"; $resn_exch{"H111 HBC"}="1H11 HBC"; $resn_exch{"H112 HBC"}="2H11 HBC";
$resn_exch{"HN11 HBC"}="1HN1 HBC"; $resn_exch{"HN12 HBC"}="2HN1 HBC"; $resn_exch{"HN'1 HBD"}="1HN' HBD";
$resn_exch{"HN'2 HBD"}="2HN' HBD"; $resn_exch{" H21 HBH"}="1H2  HBH"; $resn_exch{"H22A HBH"}="2H2  HBH";
$resn_exch{"H101 HBH"}="1H10 HBH"; $resn_exch{"H102 HBH"}="2H10 HBH"; $resn_exch{"H111 HBH"}="1H11 HBH";
$resn_exch{"H112 HBH"}="2H11 HBH"; $resn_exch{"H121 HBH"}="1H12 HBH"; $resn_exch{"H122 HBH"}="2H12 HBH";
$resn_exch{"H271 HBH"}="1H27 HBH"; $resn_exch{"H272 HBH"}="2H27 HBH"; $resn_exch{"H301 HBH"}="1H30 HBH";
$resn_exch{"H302 HBH"}="2H30 HBH"; $resn_exch{"H303 HBH"}="3H30 HBH"; $resn_exch{"H311 HBH"}="1H31 HBH";
$resn_exch{"H312 HBH"}="2H31 HBH"; $resn_exch{"H313 HBH"}="3H31 HBH"; $resn_exch{"H171 HBH"}="1H17 HBH";
$resn_exch{"H172 HBH"}="2H17 HBH"; $resn_exch{"H191 HBH"}="1H19 HBH"; $resn_exch{"H192 HBH"}="2H19 HBH";
$resn_exch{"H251 HBH"}="1H25 HBH"; $resn_exch{"H252 HBH"}="2H25 HBH"; $resn_exch{"H281 HBH"}="1H28 HBH";
$resn_exch{"H282 HBH"}="2H28 HBH"; $resn_exch{"HN21 HBI"}="1HN2 HBI"; $resn_exch{"HN22 HBI"}="2HN2 HBI";
$resn_exch{" H71 HBI"}="1H7  HBI"; $resn_exch{" H72 HBI"}="2H7  HBI"; $resn_exch{"H111 HBI"}="1H11 HBI";
$resn_exch{"H112 HBI"}="2H11 HBI"; $resn_exch{"H113 HBI"}="3H11 HBI"; $resn_exch{"HN21 HBL"}="1HN2 HBL";
$resn_exch{"HN22 HBL"}="2HN2 HBL"; $resn_exch{"HC71 HBL"}="1HC7 HBL"; $resn_exch{"HC72 HBL"}="2HC7 HBL";
$resn_exch{"H101 HBL"}="1H10 HBL"; $resn_exch{"H102 HBL"}="2H10 HBL"; $resn_exch{"H103 HBL"}="3H10 HBL";
$resn_exch{"H10O HBL"}="OH10 HBL"; $resn_exch{"H201 HBM"}="1H20 HBM"; $resn_exch{"H202 HBM"}="2H20 HBM";
$resn_exch{"H211 HBM"}="1H21 HBM"; $resn_exch{"H212 HBM"}="2H21 HBM"; $resn_exch{"H301 HBM"}="1H30 HBM";
$resn_exch{"H302 HBM"}="2H30 HBM"; $resn_exch{"H303 HBM"}="3H30 HBM"; $resn_exch{" HC2 HBN"}=" H2  HBN";
$resn_exch{" H   HBN"}="1HN  HBN"; $resn_exch{" H2  HBN"}="2HN  HBN"; $resn_exch{" HB2 HBN"}="1HB  HBN";
$resn_exch{" HB3 HBN"}="2HB  HBN"; $resn_exch{"H9B3 HBO"}="3H9B HBO"; $resn_exch{"H9B2 HBO"}="2H9B HBO";
$resn_exch{"H9B1 HBO"}="1H9B HBO"; $resn_exch{"H10A HBP"}="AH10 HBP"; $resn_exch{" H91 HBQ"}="1H9  HBQ";
$resn_exch{" H92 HBQ"}="2H9  HBQ"; $resn_exch{"H101 HBQ"}="1H10 HBQ"; $resn_exch{"H102 HBQ"}="2H10 HBQ";
$resn_exch{"H103 HBQ"}="3H10 HBQ"; $resn_exch{"H131 HBQ"}="1H13 HBQ"; $resn_exch{"H132 HBQ"}="2H13 HBQ";
$resn_exch{"H133 HBQ"}="3H13 HBQ"; $resn_exch{"H141 HBQ"}="1H14 HBQ"; $resn_exch{"H142 HBQ"}="2H14 HBQ";
$resn_exch{"H143 HBQ"}="3H14 HBQ"; $resn_exch{" H41 HBR"}="1H4  HBR"; $resn_exch{" H42 HBR"}="2H4  HBR";
$resn_exch{" H43 HBR"}="3H4  HBR"; $resn_exch{" H11 HBR"}="1H1  HBR"; $resn_exch{" H12 HBR"}="2H1  HBR";
$resn_exch{" H13 HBR"}="3H1  HBR"; $resn_exch{" H41 HBS"}="1H4  HBS"; $resn_exch{" H42 HBS"}="2H4  HBS";
$resn_exch{" H43 HBS"}="3H4  HBS"; $resn_exch{" H11 HBS"}="1H1  HBS"; $resn_exch{" H12 HBS"}="2H1  HBS";
$resn_exch{" H13 HBS"}="3H1  HBS"; $resn_exch{"HO11 HBU"}="1HO1 HBU"; $resn_exch{"HC41 HBU"}="1HC4 HBU";
$resn_exch{"HC42 HBU"}="2HC4 HBU"; $resn_exch{"HC91 HBU"}="1HC9 HBU"; $resn_exch{"HC92 HBU"}="2HC9 HBU";
$resn_exch{"H141 HBU"}="1H14 HBU"; $resn_exch{"H151 HBU"}="1H15 HBU"; $resn_exch{"H152 HBU"}="2H15 HBU";
$resn_exch{"H153 HBU"}="3H15 HBU"; $resn_exch{"H121 HBY"}="1H12 HBY"; $resn_exch{"H122 HBY"}="2H12 HBY";
$resn_exch{"H123 HBY"}="3H12 HBY"; $resn_exch{"H111 HBY"}="1H11 HBY"; $resn_exch{"H112 HBY"}="2H11 HBY";
$resn_exch{"H113 HBY"}="3H11 HBY"; $resn_exch{"H131 HBY"}="1H13 HBY"; $resn_exch{"H132 HBY"}="2H13 HBY";
$resn_exch{"H141 HBY"}="1H14 HBY"; $resn_exch{"H142 HBY"}="2H14 HBY"; $resn_exch{"H143 HBY"}="3H14 HBY";
$resn_exch{"H151 HBY"}="1H15 HBY"; $resn_exch{"H152 HBY"}="2H15 HBY"; $resn_exch{"H153 HBY"}="3H15 HBY";
$resn_exch{"HO11 HC0"}="1HO1 HC0"; $resn_exch{"HO12 HC0"}="2HO1 HC0"; $resn_exch{" H33 HC0"}="3H3  HC0";
$resn_exch{" H44 HC0"}="4H4  HC0"; $resn_exch{" H66 HC0"}="6H6  HC0"; $resn_exch{" H77 HC0"}="7H7  HC0";
$resn_exch{" H88 HC0"}="8H8  HC0"; $resn_exch{"HO11 HC1"}="1HO1 HC1"; $resn_exch{"HO12 HC1"}="2HO1 HC1";
$resn_exch{"HO21 HC1"}="1HO2 HC1"; $resn_exch{"HO22 HC1"}="2HO2 HC1"; $resn_exch{" H33 HC1"}="3H3  HC1";
$resn_exch{" H44 HC1"}="4H4  HC1"; $resn_exch{" H66 HC1"}="6H6  HC1"; $resn_exch{" H77 HC1"}="7H7  HC1";
$resn_exch{" H11 HC2"}="1H1  HC2"; $resn_exch{" H12 HC2"}="2H1  HC2"; $resn_exch{" H21 HC2"}="1H2  HC2";
$resn_exch{" H22 HC2"}="2H2  HC2"; $resn_exch{" H41 HC2"}="1H4  HC2"; $resn_exch{" H42 HC2"}="2H4  HC2";
$resn_exch{" H71 HC2"}="1H7  HC2"; $resn_exch{" H72 HC2"}="2H7  HC2"; $resn_exch{"H111 HC2"}="1H11 HC2";
$resn_exch{"H112 HC2"}="2H11 HC2"; $resn_exch{"H121 HC2"}="1H12 HC2"; $resn_exch{"H122 HC2"}="2H12 HC2";
$resn_exch{"H151 HC2"}="1H15 HC2"; $resn_exch{"H152 HC2"}="2H15 HC2"; $resn_exch{"H161 HC2"}="1H16 HC2";
$resn_exch{"H162 HC2"}="2H16 HC2"; $resn_exch{"H181 HC2"}="1H18 HC2"; $resn_exch{"H182 HC2"}="2H18 HC2";
$resn_exch{"H183 HC2"}="3H18 HC2"; $resn_exch{"H191 HC2"}="1H19 HC2"; $resn_exch{"H192 HC2"}="2H19 HC2";
$resn_exch{"H193 HC2"}="3H19 HC2"; $resn_exch{"H211 HC2"}="1H21 HC2"; $resn_exch{"H212 HC2"}="2H21 HC2";
$resn_exch{"H213 HC2"}="3H21 HC2"; $resn_exch{"H221 HC2"}="1H22 HC2"; $resn_exch{"H222 HC2"}="2H22 HC2";
$resn_exch{"H231 HC2"}="1H23 HC2"; $resn_exch{"H232 HC2"}="2H23 HC2"; $resn_exch{"H241 HC2"}="1H24 HC2";
$resn_exch{"H242 HC2"}="2H24 HC2"; $resn_exch{"H261 HC2"}="1H26 HC2"; $resn_exch{"H262 HC2"}="2H26 HC2";
$resn_exch{"H263 HC2"}="3H26 HC2"; $resn_exch{"H271 HC2"}="1H27 HC2"; $resn_exch{"H272 HC2"}="2H27 HC2";
$resn_exch{"H273 HC2"}="3H27 HC2"; $resn_exch{"H181 HC3"}="1H18 HC3"; $resn_exch{"H182 HC3"}="2H18 HC3";
$resn_exch{"H183 HC3"}="3H18 HC3"; $resn_exch{"H121 HC3"}="1H12 HC3"; $resn_exch{"H122 HC3"}="2H12 HC3";
$resn_exch{"H111 HC3"}="1H11 HC3"; $resn_exch{"H112 HC3"}="2H11 HC3"; $resn_exch{"H191 HC3"}="1H19 HC3";
$resn_exch{"H192 HC3"}="2H19 HC3"; $resn_exch{"H193 HC3"}="3H19 HC3"; $resn_exch{" H11 HC3"}="1H1  HC3";
$resn_exch{" H12 HC3"}="2H1  HC3"; $resn_exch{" H21 HC3"}="1H2  HC3"; $resn_exch{" H22 HC3"}="2H2  HC3";
$resn_exch{" H41 HC3"}="1H4  HC3"; $resn_exch{" H42 HC3"}="2H4  HC3"; $resn_exch{" H71 HC3"}="1H7  HC3";
$resn_exch{" H72 HC3"}="2H7  HC3"; $resn_exch{"H151 HC3"}="1H15 HC3"; $resn_exch{"H152 HC3"}="2H15 HC3";
$resn_exch{"H161 HC3"}="1H16 HC3"; $resn_exch{"H162 HC3"}="2H16 HC3"; $resn_exch{"H211 HC3"}="1H21 HC3";
$resn_exch{"H212 HC3"}="2H21 HC3"; $resn_exch{"H213 HC3"}="3H21 HC3"; $resn_exch{"H221 HC3"}="1H22 HC3";
$resn_exch{"H222 HC3"}="2H22 HC3"; $resn_exch{"H231 HC3"}="1H23 HC3"; $resn_exch{"H232 HC3"}="2H23 HC3";
$resn_exch{"H241 HC3"}="1H24 HC3"; $resn_exch{"H242 HC3"}="2H24 HC3"; $resn_exch{"H261 HC3"}="1H26 HC3";
$resn_exch{"H262 HC3"}="2H26 HC3"; $resn_exch{"H263 HC3"}="3H26 HC3"; $resn_exch{"H271 HC3"}="1H27 HC3";
$resn_exch{"H272 HC3"}="2H27 HC3"; $resn_exch{"H273 HC3"}="3H27 HC3"; $resn_exch{"HO4' HC4"}="'HO4 HC4";
$resn_exch{"HAC1 HC5"}="1HAC HC5"; $resn_exch{"HAC2 HC5"}="2HAC HC5"; $resn_exch{"HAC3 HC5"}="3HAC HC5";
$resn_exch{"HAD1 HC5"}="1HAD HC5"; $resn_exch{"HAD2 HC5"}="2HAD HC5"; $resn_exch{"HAE1 HC5"}="1HAE HC5";
$resn_exch{"HAE2 HC5"}="2HAE HC5"; $resn_exch{"HAF1 HC5"}="1HAF HC5"; $resn_exch{"HAF2 HC5"}="2HAF HC5";
$resn_exch{"HAG1 HC5"}="1HAG HC5"; $resn_exch{"HAG2 HC5"}="2HAG HC5"; $resn_exch{"HAQ1 HC5"}="1HAQ HC5";
$resn_exch{"HAQ2 HC5"}="2HAQ HC5"; $resn_exch{"HAO1 HC5"}="1HAO HC5"; $resn_exch{"HAO2 HC5"}="2HAO HC5";
$resn_exch{"HAT1 HC5"}="1HAT HC5"; $resn_exch{"HAT2 HC5"}="2HAT HC5"; $resn_exch{"HAT3 HC5"}="3HAT HC5";
$resn_exch{"HAJ1 HC5"}="1HAJ HC5"; $resn_exch{"HAJ2 HC5"}="2HAJ HC5"; $resn_exch{"HAJ3 HC5"}="3HAJ HC5";
$resn_exch{"HAM1 HC5"}="1HAM HC5"; $resn_exch{"HAM2 HC5"}="2HAM HC5"; $resn_exch{"HAM3 HC5"}="3HAM HC5";
$resn_exch{" H21 HCA"}="1H2  HCA"; $resn_exch{" H22 HCA"}="2H2  HCA"; $resn_exch{" H41 HCA"}="1H4  HCA";
$resn_exch{" H42 HCA"}="2H4  HCA"; $resn_exch{" H51 HCA"}="1H5  HCA"; $resn_exch{" H52 HCA"}="2H5  HCA";
$resn_exch{"H3C1 HCG"}="1H3C HCG"; $resn_exch{"H3C2 HCG"}="2H3C HCG"; $resn_exch{"H4C1 HCG"}="1H4C HCG";
$resn_exch{"H4C2 HCG"}="2H4C HCG"; $resn_exch{"H7C1 HCG"}="1H7C HCG"; $resn_exch{"H7C2 HCG"}="2H7C HCG";
$resn_exch{"H141 HCG"}="1H14 HCG"; $resn_exch{"H142 HCG"}="2H14 HCG"; $resn_exch{"H161 HCG"}="1H16 HCG";
$resn_exch{"H162 HCG"}="2H16 HCG"; $resn_exch{" H21 HCI"}="1H2  HCI"; $resn_exch{" H22 HCI"}="2H2  HCI";
$resn_exch{" H31 HCI"}="1H3  HCI"; $resn_exch{" H32 HCI"}="2H3  HCI"; $resn_exch{"HMA1 HCO"}="1HMA HCO";
$resn_exch{"HMA2 HCO"}="2HMA HCO"; $resn_exch{"HMA3 HCO"}="3HMA HCO"; $resn_exch{"HAA1 HCO"}="1HAA HCO";
$resn_exch{"HAA2 HCO"}="2HAA HCO"; $resn_exch{"HBA1 HCO"}="1HBA HCO"; $resn_exch{"HBA2 HCO"}="2HBA HCO";
$resn_exch{"H3B3 HCO"}="3H3B HCO"; $resn_exch{"HMB1 HCO"}="1HMB HCO"; $resn_exch{"HMB2 HCO"}="2HMB HCO";
$resn_exch{"HMB3 HCO"}="3HMB HCO"; $resn_exch{"HBB1 HCO"}="1HBB HCO"; $resn_exch{"HBB2 HCO"}="2HBB HCO";
$resn_exch{"H3C3 HCO"}="3H3C HCO"; $resn_exch{"HMC1 HCO"}="1HMC HCO"; $resn_exch{"HMC2 HCO"}="2HMC HCO";
$resn_exch{"HMC3 HCO"}="3HMC HCO"; $resn_exch{"HBC1 HCO"}="1HBC HCO"; $resn_exch{"HBC2 HCO"}="2HBC HCO";
$resn_exch{"HMD1 HCO"}="1HMD HCO"; $resn_exch{"HMD2 HCO"}="2HMD HCO"; $resn_exch{"HMD3 HCO"}="3HMD HCO";
$resn_exch{"HAD1 HCO"}="1HAD HCO"; $resn_exch{"HAD2 HCO"}="2HAD HCO"; $resn_exch{"HBD1 HCO"}="1HBD HCO";
$resn_exch{"HBD2 HCO"}="2HBD HCO"; $resn_exch{"HA11 HCP"}="1HA1 HCP"; $resn_exch{"HA12 HCP"}="2HA1 HCP";
$resn_exch{"HB11 HCP"}="1HB1 HCP"; $resn_exch{"HB12 HCP"}="2HB1 HCP"; $resn_exch{"HB21 HCP"}="1HB2 HCP";
$resn_exch{"HB22 HCP"}="2HB2 HCP"; $resn_exch{"HA21 HCP"}="1HA2 HCP"; $resn_exch{"HA22 HCP"}="2HA2 HCP";
$resn_exch{"HC21 HCP"}="1HC2 HCP"; $resn_exch{"HC22 HCP"}="2HC2 HCP"; $resn_exch{"HC23 HCP"}="3HC2 HCP";
$resn_exch{"HC51 HCP"}="1HC5 HCP"; $resn_exch{"HC52 HCP"}="2HC5 HCP"; $resn_exch{"H181 HCR"}="1H18 HCR";
$resn_exch{"H182 HCR"}="2H18 HCR"; $resn_exch{"H183 HCR"}="3H18 HCR"; $resn_exch{"H121 HCR"}="1H12 HCR";
$resn_exch{"H122 HCR"}="2H12 HCR"; $resn_exch{"H111 HCR"}="1H11 HCR"; $resn_exch{"H112 HCR"}="2H11 HCR";
$resn_exch{"H191 HCR"}="1H19 HCR"; $resn_exch{"H192 HCR"}="2H19 HCR"; $resn_exch{"H193 HCR"}="3H19 HCR";
$resn_exch{" H11 HCR"}="1H1  HCR"; $resn_exch{" H12 HCR"}="2H1  HCR"; $resn_exch{" H21 HCR"}="1H2  HCR";
$resn_exch{" H22 HCR"}="2H2  HCR"; $resn_exch{" H41 HCR"}="1H4  HCR"; $resn_exch{" H42 HCR"}="2H4  HCR";
$resn_exch{"H151 HCR"}="1H15 HCR"; $resn_exch{"H152 HCR"}="2H15 HCR"; $resn_exch{"H161 HCR"}="1H16 HCR";
$resn_exch{"H162 HCR"}="2H16 HCR"; $resn_exch{"H211 HCR"}="1H21 HCR"; $resn_exch{"H212 HCR"}="2H21 HCR";
$resn_exch{"H213 HCR"}="3H21 HCR"; $resn_exch{"H221 HCR"}="1H22 HCR"; $resn_exch{"H222 HCR"}="2H22 HCR";
$resn_exch{"H231 HCR"}="1H23 HCR"; $resn_exch{"H232 HCR"}="2H23 HCR"; $resn_exch{"H241 HCR"}="1H24 HCR";
$resn_exch{"H242 HCR"}="2H24 HCR"; $resn_exch{"H271 HCR"}="1H27 HCR"; $resn_exch{"H272 HCR"}="2H27 HCR";
$resn_exch{"H273 HCR"}="3H27 HCR"; $resn_exch{"H261 HCR"}="1H26 HCR"; $resn_exch{"H262 HCR"}="2H26 HCR";
$resn_exch{"H263 HCR"}="3H26 HCR"; $resn_exch{" H   HCS"}=" HN  HCS"; $resn_exch{" H2  HCS"}="2HN  HCS";
$resn_exch{" HB2 HCS"}="1HCB HCS"; $resn_exch{" HB3 HCS"}="2HCB HCS"; $resn_exch{" HG2 HCS"}="1HCG HCS";
$resn_exch{" HG3 HCS"}="2HCG HCS"; $resn_exch{" HD  HCS"}=" HSD HCS"; $resn_exch{" H71 HCX"}="1H7  HCX";
$resn_exch{" H72 HCX"}="2H7  HCX"; $resn_exch{"H2'1 HCX"}="1H2* HCX"; $resn_exch{"H2'2 HCX"}="2H2* HCX";
$resn_exch{"H5'1 HCX"}="1H5* HCX"; $resn_exch{"H5'2 HCX"}="2H5* HCX"; $resn_exch{" H4' HCX"}=" H4* HCX";
$resn_exch{" H1' HCX"}=" H1* HCX"; $resn_exch{" H3' HCX"}=" H3* HCX"; $resn_exch{" HB1 HDA"}="1HB  HDA";
$resn_exch{" HB2 HDA"}="2HB  HDA"; $resn_exch{"H071 HDB"}="1H07 HDB"; $resn_exch{"H072 HDB"}="2H07 HDB";
$resn_exch{"H111 HDB"}="1H11 HDB"; $resn_exch{"H112 HDB"}="2H11 HDB"; $resn_exch{"H161 HDB"}="1H16 HDB";
$resn_exch{"H162 HDB"}="2H16 HDB"; $resn_exch{"H171 HDB"}="1H17 HDB"; $resn_exch{"H172 HDB"}="2H17 HDB";
$resn_exch{" H71 HDC"}="1H7  HDC"; $resn_exch{" H72 HDC"}="2H7  HDC"; $resn_exch{" H61 HDC"}="1H6  HDC";
$resn_exch{" H62 HDC"}="2H6  HDC"; $resn_exch{" H31 HDC"}="1H3  HDC"; $resn_exch{" H32 HDC"}="2H3  HDC";
$resn_exch{" H21 HDC"}="1H2  HDC"; $resn_exch{" H22 HDC"}="2H2  HDC"; $resn_exch{"H2'1 HDC"}="1H2* HDC";
$resn_exch{"H2'2 HDC"}="2H2* HDC"; $resn_exch{" H3' HDC"}=" H3* HDC"; $resn_exch{" H2' HDC"}=" H2* HDC";
$resn_exch{"H4'1 HDC"}="1H4* HDC"; $resn_exch{"H4'2 HDC"}="2H4* HDC"; $resn_exch{"H5'1 HDC"}="1H5* HDC";
$resn_exch{"H5'2 HDC"}="2H5* HDC"; $resn_exch{"H6'1 HDC"}="1H6* HDC"; $resn_exch{"H6'2 HDC"}="2H6* HDC";
$resn_exch{"H7'1 HDC"}="1H7* HDC"; $resn_exch{"H7'2 HDC"}="2H7* HDC"; $resn_exch{"H8'1 HDC"}="1H8* HDC";
$resn_exch{"H8'2 HDC"}="2H8* HDC"; $resn_exch{"H9'1 HDC"}="1H9* HDC"; $resn_exch{"H9'2 HDC"}="2H9* HDC";
$resn_exch{"H101 HDC"}="1H10 HDC"; $resn_exch{"H102 HDC"}="2H10 HDC"; $resn_exch{"H103 HDC"}="3H10 HDC";
$resn_exch{"H131 HDC"}="1H13 HDC"; $resn_exch{"H132 HDC"}="2H13 HDC"; $resn_exch{"H133 HDC"}="3H13 HDC";
$resn_exch{"H141 HDC"}="1H14 HDC"; $resn_exch{"H142 HDC"}="2H14 HDC"; $resn_exch{"H143 HDC"}="3H14 HDC";
$resn_exch{"H121 HDC"}="1H12 HDC"; $resn_exch{"H122 HDC"}="2H12 HDC"; $resn_exch{" H11 HDC"}="1H1  HDC";
$resn_exch{" H12 HDC"}="2H1  HDC"; $resn_exch{" H4' HDC"}=" H4* HDC"; $resn_exch{" H1' HDC"}=" H1* HDC";
$resn_exch{"HN61 HDC"}="1HN6 HDC"; $resn_exch{"HN62 HDC"}="2HN6 HDC"; $resn_exch{"HMA1 HDD"}="1HMA HDD";
$resn_exch{"HMA2 HDD"}="2HMA HDD"; $resn_exch{"HMA3 HDD"}="3HMA HDD"; $resn_exch{"HAA1 HDD"}="1HAA HDD";
$resn_exch{"HAA2 HDD"}="2HAA HDD"; $resn_exch{"HBA1 HDD"}="1HBA HDD"; $resn_exch{"HBA2 HDD"}="2HBA HDD";
$resn_exch{"HMB1 HDD"}="1HMB HDD"; $resn_exch{"HMB2 HDD"}="2HMB HDD"; $resn_exch{"HMB3 HDD"}="3HMB HDD";
$resn_exch{"HBB1 HDD"}="1HBB HDD"; $resn_exch{"HBB2 HDD"}="2HBB HDD"; $resn_exch{"HMC1 HDD"}="1HMC HDD";
$resn_exch{"HMC2 HDD"}="2HMC HDD"; $resn_exch{"HMC3 HDD"}="3HMC HDD"; $resn_exch{"HBC1 HDD"}="1HBC HDD";
$resn_exch{"HBC2 HDD"}="2HBC HDD"; $resn_exch{"HMD1 HDD"}="1HMD HDD"; $resn_exch{"HMD2 HDD"}="2HMD HDD";
$resn_exch{"HMD3 HDD"}="3HMD HDD"; $resn_exch{"HAD1 HDD"}="1HAD HDD"; $resn_exch{"HAD2 HDD"}="2HAD HDD";
$resn_exch{"HBD1 HDD"}="1HBD HDD"; $resn_exch{"HBD2 HDD"}="2HBD HDD"; $resn_exch{"H1'1 HDF"}="1H1* HDF";
$resn_exch{"H1'2 HDF"}="2H1* HDF"; $resn_exch{" H2' HDF"}=" H2* HDF"; $resn_exch{"HO2' HDF"}="*HO2 HDF";
$resn_exch{" H3' HDF"}=" H3* HDF"; $resn_exch{"HO3' HDF"}="*HO3 HDF"; $resn_exch{" H4' HDF"}=" H4* HDF";
$resn_exch{"HO4' HDF"}="*HO4 HDF"; $resn_exch{"H5'1 HDF"}="1H5* HDF"; $resn_exch{"H5'2 HDF"}="2H5* HDF";
$resn_exch{"HO5' HDF"}="*HO5 HDF"; $resn_exch{"H121 HDI"}="1H12 HDI"; $resn_exch{"H122 HDI"}="2H12 HDI";
$resn_exch{"H123 HDI"}="3H12 HDI"; $resn_exch{"H101 HDI"}="1H10 HDI"; $resn_exch{"H102 HDI"}="2H10 HDI";
$resn_exch{"HC81 HDI"}="1HC8 HDI"; $resn_exch{"HC82 HDI"}="2HC8 HDI"; $resn_exch{"HOP2 HDP"}="2HOP HDP";
$resn_exch{"H5'1 HDP"}="1H5* HDP"; $resn_exch{"H5'' HDP"}="2H5* HDP"; $resn_exch{"HO3' HDP"}="H3T  HDP";
$resn_exch{"HOP3 HDP"}="3HOP HDP"; $resn_exch{" H11 HDS"}="1H1  HDS"; $resn_exch{" H12 HDS"}="2H1  HDS";
$resn_exch{" H21 HDS"}="1H2  HDS"; $resn_exch{" H22 HDS"}="2H2  HDS"; $resn_exch{" H31 HDS"}="1H3  HDS";
$resn_exch{" H32 HDS"}="2H3  HDS"; $resn_exch{" H41 HDS"}="1H4  HDS"; $resn_exch{" H42 HDS"}="2H4  HDS";
$resn_exch{" H51 HDS"}="1H5  HDS"; $resn_exch{" H52 HDS"}="2H5  HDS"; $resn_exch{" H61 HDS"}="1H6  HDS";
$resn_exch{" H62 HDS"}="2H6  HDS"; $resn_exch{" H71 HDS"}="1H7  HDS"; $resn_exch{" H72 HDS"}="2H7  HDS";
$resn_exch{" H81 HDS"}="1H8  HDS"; $resn_exch{" H82 HDS"}="2H8  HDS"; $resn_exch{" H91 HDS"}="1H9  HDS";
$resn_exch{" H92 HDS"}="2H9  HDS"; $resn_exch{"H101 HDS"}="1H10 HDS"; $resn_exch{"H102 HDS"}="2H10 HDS";
$resn_exch{"H111 HDS"}="1H11 HDS"; $resn_exch{"H112 HDS"}="2H11 HDS"; $resn_exch{"H121 HDS"}="1H12 HDS";
$resn_exch{"H122 HDS"}="2H12 HDS"; $resn_exch{"H131 HDS"}="1H13 HDS"; $resn_exch{"H132 HDS"}="2H13 HDS";
$resn_exch{"H141 HDS"}="1H14 HDS"; $resn_exch{"H142 HDS"}="2H14 HDS"; $resn_exch{"H151 HDS"}="1H15 HDS";
$resn_exch{"H152 HDS"}="2H15 HDS"; $resn_exch{"H161 HDS"}="1H16 HDS"; $resn_exch{"H162 HDS"}="2H16 HDS";
$resn_exch{"H163 HDS"}="3H16 HDS"; $resn_exch{"HOS3 HDS"}="3HOS HDS"; $resn_exch{"H241 HDT"}="1H24 HDT";
$resn_exch{"H242 HDT"}="2H24 HDT"; $resn_exch{"H191 HDU"}="1H19 HDU"; $resn_exch{"H192 HDU"}="2H19 HDU";
$resn_exch{"H193 HDU"}="3H19 HDU"; $resn_exch{"H101 HDU"}="1H10 HDU"; $resn_exch{"H102 HDU"}="2H10 HDU";
$resn_exch{"H103 HDU"}="3H10 HDU"; $resn_exch{"H101 HDY"}="1H10 HDY"; $resn_exch{"H102 HDY"}="2H10 HDY";
$resn_exch{"H103 HDY"}="3H10 HDY"; $resn_exch{"H251 HDY"}="1H25 HDY"; $resn_exch{"H252 HDY"}="2H25 HDY";
$resn_exch{"H271 HDY"}="1H27 HDY"; $resn_exch{"H272 HDY"}="2H27 HDY"; $resn_exch{"H301 HDY"}="1H30 HDY";
$resn_exch{"H302 HDY"}="2H30 HDY"; $resn_exch{"H303 HDY"}="3H30 HDY"; $resn_exch{"H291 HDY"}="1H29 HDY";
$resn_exch{"H292 HDY"}="2H29 HDY"; $resn_exch{"H293 HDY"}="3H29 HDY"; $resn_exch{" H31 HE1"}="1H3  HE1";
$resn_exch{" H32 HE1"}="2H3  HE1"; $resn_exch{" H41 HE1"}="1H4  HE1"; $resn_exch{" H42 HE1"}="2H4  HE1";
$resn_exch{"HAA1 HE2"}="1HAA HE2"; $resn_exch{"HAA2 HE2"}="2HAA HE2"; $resn_exch{"HAA3 HE2"}="3HAA HE2";
$resn_exch{"HAC1 HE2"}="1HAC HE2"; $resn_exch{"HAC2 HE2"}="2HAC HE2"; $resn_exch{"HAE1 HE2"}="1HAE HE2";
$resn_exch{"HAE2 HE2"}="2HAE HE2"; $resn_exch{"HAG1 HE2"}="1HAG HE2"; $resn_exch{"HAG2 HE2"}="2HAG HE2";
$resn_exch{"HAF1 HE2"}="1HAF HE2"; $resn_exch{"HAF2 HE2"}="2HAF HE2"; $resn_exch{"HAD1 HE2"}="1HAD HE2";
$resn_exch{"HAD2 HE2"}="2HAD HE2"; $resn_exch{" H91 HE3"}="1H9  HE3"; $resn_exch{" H92 HE3"}="2H9  HE3";
$resn_exch{" H93 HE3"}="3H9  HE3"; $resn_exch{"HM21 HE3"}="1HM2 HE3"; $resn_exch{"HM22 HE3"}="2HM2 HE3";
$resn_exch{"HM23 HE3"}="3HM2 HE3"; $resn_exch{"H7'1 HE3"}="1H7' HE3"; $resn_exch{"H7'2 HE3"}="2H7' HE3";
$resn_exch{"HM41 HE3"}="1HM4 HE3"; $resn_exch{"HM42 HE3"}="2HM4 HE3"; $resn_exch{"HM43 HE3"}="3HM4 HE3";
$resn_exch{" H61 HE3"}="1H6  HE3"; $resn_exch{" H62 HE3"}="2H6  HE3"; $resn_exch{" H71 HE3"}="1H7  HE3";
$resn_exch{" H72 HE3"}="2H7  HE3"; $resn_exch{"HAA1 HE4"}="1HAA HE4"; $resn_exch{"HAA2 HE4"}="2HAA HE4";
$resn_exch{"HAA3 HE4"}="3HAA HE4"; $resn_exch{"HAC1 HE4"}="1HAC HE4"; $resn_exch{"HAC2 HE4"}="2HAC HE4";
$resn_exch{"HAE1 HE4"}="1HAE HE4"; $resn_exch{"HAE2 HE4"}="2HAE HE4"; $resn_exch{"HAG1 HE4"}="1HAG HE4";
$resn_exch{"HAG2 HE4"}="2HAG HE4"; $resn_exch{"HAH1 HE4"}="1HAH HE4"; $resn_exch{"HAH2 HE4"}="2HAH HE4";
$resn_exch{"HAF1 HE4"}="1HAF HE4"; $resn_exch{"HAF2 HE4"}="2HAF HE4"; $resn_exch{"HAD1 HE4"}="1HAD HE4";
$resn_exch{"HAD2 HE4"}="2HAD HE4"; $resn_exch{"H401 HE5"}="1H40 HE5"; $resn_exch{"H402 HE5"}="2H40 HE5";
$resn_exch{"H403 HE5"}="3H40 HE5"; $resn_exch{"HBA1 HE5"}="1HBA HE5"; $resn_exch{"HBA2 HE5"}="2HBA HE5";
$resn_exch{"HAA1 HE5"}="1HAA HE5"; $resn_exch{"HAA2 HE5"}="2HAA HE5"; $resn_exch{"HMA1 HE5"}="1HMA HE5";
$resn_exch{"HMA2 HE5"}="2HMA HE5"; $resn_exch{"HMA3 HE5"}="3HMA HE5"; $resn_exch{"H2B1 HE5"}="1H2B HE5";
$resn_exch{"H2B2 HE5"}="2H2B HE5"; $resn_exch{"HAB1 HE5"}="1HAB HE5"; $resn_exch{"HAB2 HE5"}="2HAB HE5";
$resn_exch{"HAB3 HE5"}="3HAB HE5"; $resn_exch{"HAC1 HE5"}="1HAC HE5"; $resn_exch{"HAC2 HE5"}="2HAC HE5";
$resn_exch{"HAC3 HE5"}="3HAC HE5"; $resn_exch{"HMD1 HE5"}="1HMD HE5"; $resn_exch{"HMD2 HE5"}="2HMD HE5";
$resn_exch{"HMD3 HE5"}="3HMD HE5"; $resn_exch{"HAD1 HE5"}="1HAD HE5"; $resn_exch{"HAD2 HE5"}="2HAD HE5";
$resn_exch{"HBD1 HE5"}="1HBD HE5"; $resn_exch{"HBD2 HE5"}="2HBD HE5"; $resn_exch{"H411 HE5"}="1H41 HE5";
$resn_exch{"H412 HE5"}="2H41 HE5"; $resn_exch{"H413 HE5"}="3H41 HE5"; $resn_exch{"HMA1 HE6"}="1HMA HE6";
$resn_exch{"HMA2 HE6"}="2HMA HE6"; $resn_exch{"HMA3 HE6"}="3HMA HE6"; $resn_exch{"HMB1 HE6"}="1HMB HE6";
$resn_exch{"HMB2 HE6"}="2HMB HE6"; $resn_exch{"HMB3 HE6"}="3HMB HE6"; $resn_exch{"HNB1 HE6"}="1HNB HE6";
$resn_exch{"HNB2 HE6"}="2HNB HE6"; $resn_exch{"HNB3 HE6"}="3HNB HE6"; $resn_exch{"HMC1 HE6"}="1HMC HE6";
$resn_exch{"HMC2 HE6"}="2HMC HE6"; $resn_exch{"HMC3 HE6"}="3HMC HE6"; $resn_exch{"HNC1 HE6"}="1HNC HE6";
$resn_exch{"HNC2 HE6"}="2HNC HE6"; $resn_exch{"HNC3 HE6"}="3HNC HE6"; $resn_exch{"HMD1 HE6"}="1HMD HE6";
$resn_exch{"HMD2 HE6"}="2HMD HE6"; $resn_exch{"HMD3 HE6"}="3HMD HE6"; $resn_exch{"HAI1 HE7"}="1HAI HE7";
$resn_exch{"HAI2 HE7"}="2HAI HE7"; $resn_exch{"HAK1 HE7"}="1HAK HE7"; $resn_exch{"HAK2 HE7"}="2HAK HE7";
$resn_exch{"HAA1 HE7"}="1HAA HE7"; $resn_exch{"HAA2 HE7"}="2HAA HE7"; $resn_exch{"HAA3 HE7"}="3HAA HE7";
$resn_exch{"HAM1 HE7"}="1HAM HE7"; $resn_exch{"HAM2 HE7"}="2HAM HE7"; $resn_exch{"HAO1 HE7"}="1HAO HE7";
$resn_exch{"HAO2 HE7"}="2HAO HE7"; $resn_exch{"HAN1 HE7"}="1HAN HE7"; $resn_exch{"HAN2 HE7"}="2HAN HE7";
$resn_exch{"HAP1 HE7"}="1HAP HE7"; $resn_exch{"HAP2 HE7"}="2HAP HE7"; $resn_exch{"HAL1 HE7"}="1HAL HE7";
$resn_exch{"HAL2 HE7"}="2HAL HE7"; $resn_exch{"HAJ1 HE7"}="1HAJ HE7"; $resn_exch{"HAJ2 HE7"}="2HAJ HE7";
$resn_exch{"HAA1 HEA"}="1HAA HEA"; $resn_exch{"HAA2 HEA"}="2HAA HEA"; $resn_exch{"HBA1 HEA"}="1HBA HEA";
$resn_exch{"HBA2 HEA"}="2HBA HEA"; $resn_exch{"HMB1 HEA"}="1HMB HEA"; $resn_exch{"HMB2 HEA"}="2HMB HEA";
$resn_exch{"HMB3 HEA"}="3HMB HEA"; $resn_exch{"HMC1 HEA"}="1HMC HEA"; $resn_exch{"HMC2 HEA"}="2HMC HEA";
$resn_exch{"HMC3 HEA"}="3HMC HEA"; $resn_exch{"HBC1 HEA"}="1HBC HEA"; $resn_exch{"HBC2 HEA"}="2HBC HEA";
$resn_exch{"HMD1 HEA"}="1HMD HEA"; $resn_exch{"HMD2 HEA"}="2HMD HEA"; $resn_exch{"HMD3 HEA"}="3HMD HEA";
$resn_exch{"HAD1 HEA"}="1HAD HEA"; $resn_exch{"HAD2 HEA"}="2HAD HEA"; $resn_exch{"HBD1 HEA"}="1HBD HEA";
$resn_exch{"HBD2 HEA"}="2HBD HEA"; $resn_exch{"H121 HEA"}="1H12 HEA"; $resn_exch{"H122 HEA"}="2H12 HEA";
$resn_exch{"H131 HEA"}="1H13 HEA"; $resn_exch{"H132 HEA"}="2H13 HEA"; $resn_exch{"H161 HEA"}="1H16 HEA";
$resn_exch{"H162 HEA"}="2H16 HEA"; $resn_exch{"H171 HEA"}="1H17 HEA"; $resn_exch{"H172 HEA"}="2H17 HEA";
$resn_exch{"H201 HEA"}="1H20 HEA"; $resn_exch{"H202 HEA"}="2H20 HEA"; $resn_exch{"H211 HEA"}="1H21 HEA";
$resn_exch{"H212 HEA"}="2H21 HEA"; $resn_exch{"H241 HEA"}="1H24 HEA"; $resn_exch{"H242 HEA"}="2H24 HEA";
$resn_exch{"H243 HEA"}="3H24 HEA"; $resn_exch{"H251 HEA"}="1H25 HEA"; $resn_exch{"H252 HEA"}="2H25 HEA";
$resn_exch{"H253 HEA"}="3H25 HEA"; $resn_exch{"H261 HEA"}="1H26 HEA"; $resn_exch{"H262 HEA"}="2H26 HEA";
$resn_exch{"H263 HEA"}="3H26 HEA"; $resn_exch{"H271 HEA"}="1H27 HEA"; $resn_exch{"H272 HEA"}="2H27 HEA";
$resn_exch{"H273 HEA"}="3H27 HEA"; $resn_exch{"HMA1 HEB"}="1HMA HEB"; $resn_exch{"HMA2 HEB"}="2HMA HEB";
$resn_exch{"HMA3 HEB"}="3HMA HEB"; $resn_exch{"HAA1 HEB"}="1HAA HEB"; $resn_exch{"HAA2 HEB"}="2HAA HEB";
$resn_exch{"HBA1 HEB"}="1HBA HEB"; $resn_exch{"HBA2 HEB"}="2HBA HEB"; $resn_exch{"HMB1 HEB"}="1HMB HEB";
$resn_exch{"HMB2 HEB"}="2HMB HEB"; $resn_exch{"HMB3 HEB"}="3HMB HEB"; $resn_exch{"HAB2 HEB"}="2HAB HEB";
$resn_exch{"HBB1 HEB"}="1HBB HEB"; $resn_exch{"HBB2 HEB"}="2HBB HEB"; $resn_exch{"HBB3 HEB"}="3HBB HEB";
$resn_exch{"HMC1 HEB"}="1HMC HEB"; $resn_exch{"HMC2 HEB"}="2HMC HEB"; $resn_exch{"HMC3 HEB"}="3HMC HEB";
$resn_exch{"HBC1 HEB"}="1HBC HEB"; $resn_exch{"HBC2 HEB"}="2HBC HEB"; $resn_exch{"HMD1 HEB"}="1HMD HEB";
$resn_exch{"HMD2 HEB"}="2HMD HEB"; $resn_exch{"HMD3 HEB"}="3HMD HEB"; $resn_exch{"HAD1 HEB"}="1HAD HEB";
$resn_exch{"HAD2 HEB"}="2HAD HEB"; $resn_exch{"HBD1 HEB"}="1HBD HEB"; $resn_exch{"HBD2 HEB"}="2HBD HEB";
$resn_exch{"HMA1 HEC"}="1HMA HEC"; $resn_exch{"HMA2 HEC"}="2HMA HEC"; $resn_exch{"HMA3 HEC"}="3HMA HEC";
$resn_exch{"HAA1 HEC"}="1HAA HEC"; $resn_exch{"HAA2 HEC"}="2HAA HEC"; $resn_exch{"HBA1 HEC"}="1HBA HEC";
$resn_exch{"HBA2 HEC"}="2HBA HEC"; $resn_exch{"HMB1 HEC"}="1HMB HEC"; $resn_exch{"HMB2 HEC"}="2HMB HEC";
$resn_exch{"HMB3 HEC"}="3HMB HEC"; $resn_exch{"HBB1 HEC"}="1HBB HEC"; $resn_exch{"HBB2 HEC"}="2HBB HEC";
$resn_exch{"HBB3 HEC"}="3HBB HEC"; $resn_exch{"HMC1 HEC"}="1HMC HEC"; $resn_exch{"HMC2 HEC"}="2HMC HEC";
$resn_exch{"HMC3 HEC"}="3HMC HEC"; $resn_exch{"HBC1 HEC"}="1HBC HEC"; $resn_exch{"HBC2 HEC"}="2HBC HEC";
$resn_exch{"HBC3 HEC"}="3HBC HEC"; $resn_exch{"HMD1 HEC"}="1HMD HEC"; $resn_exch{"HMD2 HEC"}="2HMD HEC";
$resn_exch{"HMD3 HEC"}="3HMD HEC"; $resn_exch{"HAD1 HEC"}="1HAD HEC"; $resn_exch{"HAD2 HEC"}="2HAD HEC";
$resn_exch{"HBD1 HEC"}="1HBD HEC"; $resn_exch{"HBD2 HEC"}="2HBD HEC"; $resn_exch{" H11 HED"}="1H1  HED";
$resn_exch{" H12 HED"}="2H1  HED"; $resn_exch{" H21 HED"}="1H2  HED"; $resn_exch{" H22 HED"}="2H2  HED";
$resn_exch{" H51 HED"}="1H5  HED"; $resn_exch{" H52 HED"}="2H5  HED"; $resn_exch{" H61 HED"}="1H6  HED";
$resn_exch{" H62 HED"}="2H6  HED"; $resn_exch{" H11 HEE"}="1H1  HEE"; $resn_exch{" H12 HEE"}="2H1  HEE";
$resn_exch{" H21 HEE"}="1H2  HEE"; $resn_exch{" H22 HEE"}="2H2  HEE"; $resn_exch{" H31 HEE"}="1H3  HEE";
$resn_exch{" H32 HEE"}="2H3  HEE"; $resn_exch{" H41 HEE"}="1H4  HEE"; $resn_exch{" H42 HEE"}="2H4  HEE";
$resn_exch{" H51 HEE"}="1H5  HEE"; $resn_exch{" H52 HEE"}="2H5  HEE"; $resn_exch{" H61 HEE"}="1H6  HEE";
$resn_exch{" H62 HEE"}="2H6  HEE"; $resn_exch{" H63 HEE"}="3H6  HEE"; $resn_exch{"HOP3 HEE"}="3HOP HEE";
$resn_exch{"H1'1 HEE"}="1H1' HEE"; $resn_exch{"H1'2 HEE"}="2H1' HEE"; $resn_exch{"H2'1 HEE"}="1H2' HEE";
$resn_exch{"H2'2 HEE"}="2H2' HEE"; $resn_exch{"H2'3 HEE"}="3H2' HEE"; $resn_exch{"HM51 HEF"}="1HM5 HEF";
$resn_exch{"HM52 HEF"}="2HM5 HEF"; $resn_exch{"HM53 HEF"}="3HM5 HEF"; $resn_exch{" HM1 HEF"}="1HM  HEF";
$resn_exch{" HM2 HEF"}="2HM  HEF"; $resn_exch{"HE11 HEF"}="1HE1 HEF"; $resn_exch{"HE12 HEF"}="2HE1 HEF";
$resn_exch{"HE21 HEF"}="1HE2 HEF"; $resn_exch{"HE22 HEF"}="2HE2 HEF"; $resn_exch{"HMA1 HEG"}="1HMA HEG";
$resn_exch{"HMA2 HEG"}="2HMA HEG"; $resn_exch{"HMA3 HEG"}="3HMA HEG"; $resn_exch{"HAA1 HEG"}="1HAA HEG";
$resn_exch{"HAA2 HEG"}="2HAA HEG"; $resn_exch{"HBA1 HEG"}="1HBA HEG"; $resn_exch{"HBA2 HEG"}="2HBA HEG";
$resn_exch{"HMB1 HEG"}="1HMB HEG"; $resn_exch{"HMB2 HEG"}="2HMB HEG"; $resn_exch{"HMB3 HEG"}="3HMB HEG";
$resn_exch{"HBB1 HEG"}="1HBB HEG"; $resn_exch{"HBB2 HEG"}="2HBB HEG"; $resn_exch{"HMC1 HEG"}="1HMC HEG";
$resn_exch{"HMC2 HEG"}="2HMC HEG"; $resn_exch{"HMC3 HEG"}="3HMC HEG"; $resn_exch{"HBC1 HEG"}="1HBC HEG";
$resn_exch{"HBC2 HEG"}="2HBC HEG"; $resn_exch{"HMD1 HEG"}="1HMD HEG"; $resn_exch{"HMD2 HEG"}="2HMD HEG";
$resn_exch{"HMD3 HEG"}="3HMD HEG"; $resn_exch{"HAD1 HEG"}="1HAD HEG"; $resn_exch{"HAD2 HEG"}="2HAD HEG";
$resn_exch{"HBD1 HEG"}="1HBD HEG"; $resn_exch{"HBD2 HEG"}="2HBD HEG"; $resn_exch{"HMA1 HEH"}="1HMA HEH";
$resn_exch{"HMA2 HEH"}="2HMA HEH"; $resn_exch{"HMA3 HEH"}="3HMA HEH"; $resn_exch{"HE6' HEH"}="HE6* HEH";
$resn_exch{"HE2' HEH"}="HE2* HEH"; $resn_exch{"HM11 HEH"}="1HM1 HEH"; $resn_exch{"HM12 HEH"}="2HM1 HEH";
$resn_exch{"HM13 HEH"}="3HM1 HEH"; $resn_exch{"HE3' HEH"}="HE3* HEH"; $resn_exch{"HE4' HEH"}="HE4* HEH";
$resn_exch{"HG21 HEH"}="1HG2 HEH"; $resn_exch{"HG22 HEH"}="2HG2 HEH"; $resn_exch{"HG23 HEH"}="3HG2 HEH";
$resn_exch{"HG31 HEH"}="1HG3 HEH"; $resn_exch{"HG32 HEH"}="2HG3 HEH"; $resn_exch{"HG33 HEH"}="3HG3 HEH";
$resn_exch{"HE5' HEH"}="HE5* HEH"; $resn_exch{"HF6' HEH"}="HF6* HEH"; $resn_exch{"HF2' HEH"}="HF2* HEH";
$resn_exch{"HF3' HEH"}="HF3* HEH"; $resn_exch{"HK51 HEH"}="1HK5 HEH"; $resn_exch{"HK52 HEH"}="2HK5 HEH";
$resn_exch{"HK53 HEH"}="3HK5 HEH"; $resn_exch{"HK61 HEH"}="1HK6 HEH"; $resn_exch{"HK62 HEH"}="2HK6 HEH";
$resn_exch{"HK63 HEH"}="3HK6 HEH"; $resn_exch{"HM41 HEH"}="1HM4 HEH"; $resn_exch{"HM42 HEH"}="2HM4 HEH";
$resn_exch{"HM43 HEH"}="3HM4 HEH"; $resn_exch{"HF5' HEH"}="HF5* HEH"; $resn_exch{"HMB1 HEH"}="1HMB HEH";
$resn_exch{"HMB2 HEH"}="2HMB HEH"; $resn_exch{"HMB3 HEH"}="3HMB HEH"; $resn_exch{"HMC1 HEH"}="1HMC HEH";
$resn_exch{"HMC2 HEH"}="2HMC HEH"; $resn_exch{"HMC3 HEH"}="3HMC HEH"; $resn_exch{"H111 HEL"}="1H11 HEL";
$resn_exch{"H112 HEL"}="2H11 HEL"; $resn_exch{"H121 HEL"}="1H12 HEL"; $resn_exch{"H122 HEL"}="2H12 HEL";
$resn_exch{"H131 HEL"}="1H13 HEL"; $resn_exch{"H132 HEL"}="2H13 HEL"; $resn_exch{"H141 HEL"}="1H14 HEL";
$resn_exch{"H142 HEL"}="2H14 HEL"; $resn_exch{"H221 HEL"}="1H22 HEL"; $resn_exch{"H222 HEL"}="2H22 HEL";
$resn_exch{"H231 HEL"}="1H23 HEL"; $resn_exch{"H232 HEL"}="2H23 HEL"; $resn_exch{"H233 HEL"}="3H23 HEL";
$resn_exch{"H241 HEL"}="1H24 HEL"; $resn_exch{"H242 HEL"}="2H24 HEL"; $resn_exch{"H243 HEL"}="3H24 HEL";
$resn_exch{"H251 HEL"}="1H25 HEL"; $resn_exch{"H252 HEL"}="2H25 HEL"; $resn_exch{"H253 HEL"}="3H25 HEL";
$resn_exch{"HMA1 HEM"}="1HMA HEM"; $resn_exch{"HMA2 HEM"}="2HMA HEM"; $resn_exch{"HMA3 HEM"}="3HMA HEM";
$resn_exch{"HAA1 HEM"}="1HAA HEM"; $resn_exch{"HAA2 HEM"}="2HAA HEM"; $resn_exch{"HBA1 HEM"}="1HBA HEM";
$resn_exch{"HBA2 HEM"}="2HBA HEM"; $resn_exch{"HMB1 HEM"}="1HMB HEM"; $resn_exch{"HMB2 HEM"}="2HMB HEM";
$resn_exch{"HMB3 HEM"}="3HMB HEM"; $resn_exch{"HBB1 HEM"}="1HBB HEM"; $resn_exch{"HBB2 HEM"}="2HBB HEM";
$resn_exch{"HMC1 HEM"}="1HMC HEM"; $resn_exch{"HMC2 HEM"}="2HMC HEM"; $resn_exch{"HMC3 HEM"}="3HMC HEM";
$resn_exch{"HBC1 HEM"}="1HBC HEM"; $resn_exch{"HBC2 HEM"}="2HBC HEM"; $resn_exch{"HMD1 HEM"}="1HMD HEM";
$resn_exch{"HMD2 HEM"}="2HMD HEM"; $resn_exch{"HMD3 HEM"}="3HMD HEM"; $resn_exch{"HAD1 HEM"}="1HAD HEM";
$resn_exch{"HAD2 HEM"}="2HAD HEM"; $resn_exch{"HBD1 HEM"}="1HBD HEM"; $resn_exch{"HBD2 HEM"}="2HBD HEM";
$resn_exch{"H2A1 HEN"}="1H2A HEN"; $resn_exch{"H2A2 HEN"}="2H2A HEN"; $resn_exch{"H2A3 HEN"}="3H2A HEN";
$resn_exch{"H4A1 HEN"}="1H4A HEN"; $resn_exch{"H4A2 HEN"}="2H4A HEN"; $resn_exch{"H5A1 HEN"}="1H5A HEN";
$resn_exch{"H5A2 HEN"}="2H5A HEN"; $resn_exch{"HP1O HEN"}="OHP1 HEN"; $resn_exch{"HP3O HEN"}="OHP3 HEN";
$resn_exch{"H2BO HEN"}="OH2B HEN"; $resn_exch{"HBIC HEN"}="CHBI HEN"; $resn_exch{"HGIC HEN"}="CHGI HEN";
$resn_exch{"HCE1 HEN"}="1HCE HEN"; $resn_exch{"HCEC HEN"}="CHCE HEN"; $resn_exch{"HG1O HEN"}="OHG1 HEN";
$resn_exch{"HG2O HEN"}="OHG2 HEN"; $resn_exch{" HA1 HEO"}="1HA  HEO"; $resn_exch{" HA2 HEO"}="2HA  HEO";
$resn_exch{" HA3 HEO"}="3HA  HEO"; $resn_exch{"HAA1 HEO"}="1HAA HEO"; $resn_exch{"HAA2 HEO"}="2HAA HEO";
$resn_exch{"HBA1 HEO"}="1HBA HEO"; $resn_exch{"HBA2 HEO"}="2HBA HEO"; $resn_exch{"HMB1 HEO"}="1HMB HEO";
$resn_exch{"HMB2 HEO"}="2HMB HEO"; $resn_exch{"HMB3 HEO"}="3HMB HEO"; $resn_exch{"HMC1 HEO"}="1HMC HEO";
$resn_exch{"HMC2 HEO"}="2HMC HEO"; $resn_exch{"HMC3 HEO"}="3HMC HEO"; $resn_exch{"HBC1 HEO"}="1HBC HEO";
$resn_exch{"HBC2 HEO"}="2HBC HEO"; $resn_exch{"HMD1 HEO"}="1HMD HEO"; $resn_exch{"HMD2 HEO"}="2HMD HEO";
$resn_exch{"HMD3 HEO"}="3HMD HEO"; $resn_exch{"HAD1 HEO"}="1HAD HEO"; $resn_exch{"HAD2 HEO"}="2HAD HEO";
$resn_exch{"HBD1 HEO"}="1HBD HEO"; $resn_exch{"HBD2 HEO"}="2HBD HEO"; $resn_exch{"H121 HEO"}="1H12 HEO";
$resn_exch{"H122 HEO"}="2H12 HEO"; $resn_exch{"H131 HEO"}="1H13 HEO"; $resn_exch{"H132 HEO"}="2H13 HEO";
$resn_exch{"H161 HEO"}="1H16 HEO"; $resn_exch{"H162 HEO"}="2H16 HEO"; $resn_exch{"H171 HEO"}="1H17 HEO";
$resn_exch{"H172 HEO"}="2H17 HEO"; $resn_exch{"H201 HEO"}="1H20 HEO"; $resn_exch{"H202 HEO"}="2H20 HEO";
$resn_exch{"H211 HEO"}="1H21 HEO"; $resn_exch{"H212 HEO"}="2H21 HEO"; $resn_exch{"H241 HEO"}="1H24 HEO";
$resn_exch{"H242 HEO"}="2H24 HEO"; $resn_exch{"H243 HEO"}="3H24 HEO"; $resn_exch{"H251 HEO"}="1H25 HEO";
$resn_exch{"H252 HEO"}="2H25 HEO"; $resn_exch{"H253 HEO"}="3H25 HEO"; $resn_exch{"H261 HEO"}="1H26 HEO";
$resn_exch{"H262 HEO"}="2H26 HEO"; $resn_exch{"H263 HEO"}="3H26 HEO"; $resn_exch{"H271 HEO"}="1H27 HEO";
$resn_exch{"H272 HEO"}="2H27 HEO"; $resn_exch{"H273 HEO"}="3H27 HEO"; $resn_exch{" H21 HEP"}="1H2  HEP";
$resn_exch{" H22 HEP"}="2H2  HEP"; $resn_exch{" H31 HEP"}="1H3  HEP"; $resn_exch{" H32 HEP"}="2H3  HEP";
$resn_exch{" H41 HEP"}="1H4  HEP"; $resn_exch{" H42 HEP"}="2H4  HEP"; $resn_exch{" H51 HEP"}="1H5  HEP";
$resn_exch{" H52 HEP"}="2H5  HEP"; $resn_exch{" H53 HEP"}="3H5  HEP"; $resn_exch{"H131 HEP"}="1H13 HEP";
$resn_exch{"H132 HEP"}="2H13 HEP"; $resn_exch{"H141 HEP"}="1H14 HEP"; $resn_exch{"H142 HEP"}="2H14 HEP";
$resn_exch{"HOP2 HEP"}="2HOP HEP"; $resn_exch{"H321 HEQ"}="1H32 HEQ"; $resn_exch{"H322 HEQ"}="2H32 HEQ";
$resn_exch{"H323 HEQ"}="3H32 HEQ"; $resn_exch{"H311 HEQ"}="1H31 HEQ"; $resn_exch{"H312 HEQ"}="2H31 HEQ";
$resn_exch{"H313 HEQ"}="3H31 HEQ"; $resn_exch{"HC21 HEQ"}="1HC2 HEQ"; $resn_exch{"HC22 HEQ"}="2HC2 HEQ";
$resn_exch{"HC31 HEQ"}="1HC3 HEQ"; $resn_exch{"HO31 HEQ"}="1HO3 HEQ"; $resn_exch{"HC41 HEQ"}="1HC4 HEQ";
$resn_exch{"HC42 HEQ"}="2HC4 HEQ"; $resn_exch{"H331 HEQ"}="1H33 HEQ"; $resn_exch{"H332 HEQ"}="2H33 HEQ";
$resn_exch{"H333 HEQ"}="3H33 HEQ"; $resn_exch{"HC71 HEQ"}="1HC7 HEQ"; $resn_exch{"HC81 HEQ"}="1HC8 HEQ";
$resn_exch{"H341 HEQ"}="1H34 HEQ"; $resn_exch{"H342 HEQ"}="2H34 HEQ"; $resn_exch{"H343 HEQ"}="3H34 HEQ";
$resn_exch{"H101 HEQ"}="1H10 HEQ"; $resn_exch{"H111 HEQ"}="1H11 HEQ"; $resn_exch{"H121 HEQ"}="1H12 HEQ";
$resn_exch{"H351 HEQ"}="1H35 HEQ"; $resn_exch{"H352 HEQ"}="2H35 HEQ"; $resn_exch{"H353 HEQ"}="3H35 HEQ";
$resn_exch{"H141 HEQ"}="1H14 HEQ"; $resn_exch{"H151 HEQ"}="1H15 HEQ"; $resn_exch{"H161 HEQ"}="1H16 HEQ";
$resn_exch{"H171 HEQ"}="1H17 HEQ"; $resn_exch{"H361 HEQ"}="1H36 HEQ"; $resn_exch{"H362 HEQ"}="2H36 HEQ";
$resn_exch{"H363 HEQ"}="3H36 HEQ"; $resn_exch{"H191 HEQ"}="1H19 HEQ"; $resn_exch{"H201 HEQ"}="1H20 HEQ";
$resn_exch{"H211 HEQ"}="1H21 HEQ"; $resn_exch{"H371 HEQ"}="1H37 HEQ"; $resn_exch{"H372 HEQ"}="2H37 HEQ";
$resn_exch{"H373 HEQ"}="3H37 HEQ"; $resn_exch{"H231 HEQ"}="1H23 HEQ"; $resn_exch{"H242 HEQ"}="2H24 HEQ";
$resn_exch{"H381 HEQ"}="1H38 HEQ"; $resn_exch{"H382 HEQ"}="2H38 HEQ"; $resn_exch{"H383 HEQ"}="3H38 HEQ";
$resn_exch{"H281 HEQ"}="1H28 HEQ"; $resn_exch{"H282 HEQ"}="2H28 HEQ"; $resn_exch{"H291 HEQ"}="1H29 HEQ";
$resn_exch{"H292 HEQ"}="2H29 HEQ"; $resn_exch{"H401 HEQ"}="1H40 HEQ"; $resn_exch{"H402 HEQ"}="2H40 HEQ";
$resn_exch{"H403 HEQ"}="3H40 HEQ"; $resn_exch{"H391 HEQ"}="1H39 HEQ"; $resn_exch{"H392 HEQ"}="2H39 HEQ";
$resn_exch{"H393 HEQ"}="3H39 HEQ"; $resn_exch{"HMA1 HES"}="1HMA HES"; $resn_exch{"HMA2 HES"}="2HMA HES";
$resn_exch{"HMA3 HES"}="3HMA HES"; $resn_exch{"HAA1 HES"}="1HAA HES"; $resn_exch{"HAA2 HES"}="2HAA HES";
$resn_exch{"HBA1 HES"}="1HBA HES"; $resn_exch{"HBA2 HES"}="2HBA HES"; $resn_exch{"HMB1 HES"}="1HMB HES";
$resn_exch{"HMB2 HES"}="2HMB HES"; $resn_exch{"HMB3 HES"}="3HMB HES"; $resn_exch{"HBB1 HES"}="1HBB HES";
$resn_exch{"HBB2 HES"}="2HBB HES"; $resn_exch{"HBB3 HES"}="3HBB HES"; $resn_exch{"HMC1 HES"}="1HMC HES";
$resn_exch{"HMC2 HES"}="2HMC HES"; $resn_exch{"HMC3 HES"}="3HMC HES"; $resn_exch{"HBC1 HES"}="1HBC HES";
$resn_exch{"HBC2 HES"}="2HBC HES"; $resn_exch{"HBC3 HES"}="3HBC HES"; $resn_exch{"HMD1 HES"}="1HMD HES";
$resn_exch{"HMD2 HES"}="2HMD HES"; $resn_exch{"HMD3 HES"}="3HMD HES"; $resn_exch{"HAD1 HES"}="1HAD HES";
$resn_exch{"HAD2 HES"}="2HAD HES"; $resn_exch{"HBD1 HES"}="1HBD HES"; $resn_exch{"HBD2 HES"}="2HBD HES";
$resn_exch{"H1A1 HET"}="1H1A HET"; $resn_exch{"H1A2 HET"}="2H1A HET"; $resn_exch{"HT01 HET"}="1HT0 HET";
$resn_exch{"HT02 HET"}="2HT0 HET"; $resn_exch{"HA21 HET"}="1HA2 HET"; $resn_exch{"HA22 HET"}="2HA2 HET";
$resn_exch{"HA23 HET"}="3HA2 HET"; $resn_exch{"HA31 HET"}="1HA3 HET"; $resn_exch{"HA32 HET"}="2HA3 HET";
$resn_exch{"HA33 HET"}="3HA3 HET"; $resn_exch{"HS21 HET"}="1HS2 HET"; $resn_exch{"HS22 HET"}="2HS2 HET";
$resn_exch{"HS31 HET"}="1HS3 HET"; $resn_exch{"HS32 HET"}="2HS3 HET"; $resn_exch{"HS41 HET"}="1HS4 HET";
$resn_exch{"HS42 HET"}="2HS4 HET"; $resn_exch{"HS51 HET"}="1HS5 HET"; $resn_exch{"HS52 HET"}="2HS5 HET";
$resn_exch{"HOP2 HEU"}="2HOP HEU"; $resn_exch{" H5' HEU"}="1H5* HEU"; $resn_exch{"H5'' HEU"}="2H5* HEU";
$resn_exch{" H4' HEU"}=" H4* HEU"; $resn_exch{" H1' HEU"}=" H1* HEU"; $resn_exch{" H71 HEU"}="1H7  HEU";
$resn_exch{" H72 HEU"}="2H7  HEU"; $resn_exch{" H81 HEU"}="1H8  HEU"; $resn_exch{" H82 HEU"}="2H8  HEU";
$resn_exch{" H3' HEU"}=" H3* HEU"; $resn_exch{" H2' HEU"}="1H2* HEU"; $resn_exch{"H2'' HEU"}="2H2* HEU";
$resn_exch{"HO3' HEU"}="H3T  HEU"; $resn_exch{"HOP3 HEU"}="3HOP HEU"; $resn_exch{"HMA1 HEV"}="1HMA HEV";
$resn_exch{"HMA2 HEV"}="2HMA HEV"; $resn_exch{"HMA3 HEV"}="3HMA HEV"; $resn_exch{"HAA1 HEV"}="1HAA HEV";
$resn_exch{"HAA2 HEV"}="2HAA HEV"; $resn_exch{"HBA1 HEV"}="1HBA HEV"; $resn_exch{"HBA2 HEV"}="2HBA HEV";
$resn_exch{"HM11 HEV"}="1HM1 HEV"; $resn_exch{"HM12 HEV"}="2HM1 HEV"; $resn_exch{"HBB1 HEV"}="1HBB HEV";
$resn_exch{"HBB2 HEV"}="2HBB HEV"; $resn_exch{"HM21 HEV"}="1HM2 HEV"; $resn_exch{"HM22 HEV"}="2HM2 HEV";
$resn_exch{"HBC1 HEV"}="1HBC HEV"; $resn_exch{"HBC2 HEV"}="2HBC HEV"; $resn_exch{"HMD1 HEV"}="1HMD HEV";
$resn_exch{"HMD2 HEV"}="2HMD HEV"; $resn_exch{"HMD3 HEV"}="3HMD HEV"; $resn_exch{"HAD1 HEV"}="1HAD HEV";
$resn_exch{"HAD2 HEV"}="2HAD HEV"; $resn_exch{"HBD1 HEV"}="1HBD HEV"; $resn_exch{"HBD2 HEV"}="2HBD HEV";
$resn_exch{" H11 HEX"}="1H1  HEX"; $resn_exch{" H12 HEX"}="2H1  HEX"; $resn_exch{" H13 HEX"}="3H1  HEX";
$resn_exch{" H21 HEX"}="1H2  HEX"; $resn_exch{" H22 HEX"}="2H2  HEX"; $resn_exch{" H31 HEX"}="1H3  HEX";
$resn_exch{" H32 HEX"}="2H3  HEX"; $resn_exch{" H41 HEX"}="1H4  HEX"; $resn_exch{" H42 HEX"}="2H4  HEX";
$resn_exch{" H51 HEX"}="1H5  HEX"; $resn_exch{" H52 HEX"}="2H5  HEX"; $resn_exch{" H61 HEX"}="1H6  HEX";
$resn_exch{" H62 HEX"}="2H6  HEX"; $resn_exch{" H63 HEX"}="3H6  HEX"; $resn_exch{"H2A1 HEY"}="1H2A HEY";
$resn_exch{"H2A2 HEY"}="2H2A HEY"; $resn_exch{"H2A3 HEY"}="3H2A HEY"; $resn_exch{"H4A1 HEY"}="1H4A HEY";
$resn_exch{"H4A2 HEY"}="2H4A HEY"; $resn_exch{"H5A1 HEY"}="1H5A HEY"; $resn_exch{"H5A2 HEY"}="2H5A HEY";
$resn_exch{"HGI1 HEY"}="1HGI HEY"; $resn_exch{"HGI2 HEY"}="2HGI HEY"; $resn_exch{"HEI1 HEY"}="1HEI HEY";
$resn_exch{"HEI2 HEY"}="2HEI HEY"; $resn_exch{" H11 HEZ"}="1H1  HEZ"; $resn_exch{" H12 HEZ"}="2H1  HEZ";
$resn_exch{" H21 HEZ"}="1H2  HEZ"; $resn_exch{" H22 HEZ"}="2H2  HEZ"; $resn_exch{" H31 HEZ"}="1H3  HEZ";
$resn_exch{" H32 HEZ"}="2H3  HEZ"; $resn_exch{" H41 HEZ"}="1H4  HEZ"; $resn_exch{" H42 HEZ"}="2H4  HEZ";
$resn_exch{" H51 HEZ"}="1H5  HEZ"; $resn_exch{" H52 HEZ"}="2H5  HEZ"; $resn_exch{" H61 HEZ"}="1H6  HEZ";
$resn_exch{" H62 HEZ"}="2H6  HEZ"; $resn_exch{" H31 HF1"}="1H3  HF1"; $resn_exch{" H32 HF1"}="2H3  HF1";
$resn_exch{" H41 HF1"}="1H4  HF1"; $resn_exch{" H42 HF1"}="2H4  HF1"; $resn_exch{"HB31 HF3"}="1HB3 HF3";
$resn_exch{"HB11 HF3"}="1HB1 HF3"; $resn_exch{"HB21 HF3"}="1HB2 HF3"; $resn_exch{"HA21 HF3"}="1HA2 HF3";
$resn_exch{"HA11 HF3"}="1HA1 HF3"; $resn_exch{"HC11 HF3"}="1HC1 HF3"; $resn_exch{"HC21 HF3"}="1HC2 HF3";
$resn_exch{"HA11 HF5"}="1HA1 HF5"; $resn_exch{"HB11 HF5"}="1HB1 HF5"; $resn_exch{"HB21 HF5"}="1HB2 HF5";
$resn_exch{"HB31 HF5"}="1HB3 HF5"; $resn_exch{"HC11 HF5"}="1HC1 HF5"; $resn_exch{"HC21 HF5"}="1HC2 HF5";
$resn_exch{"HD11 HF5"}="1HD1 HF5"; $resn_exch{"HD21 HF5"}="1HD2 HF5"; $resn_exch{"HE11 HF5"}="1HE1 HF5";
$resn_exch{"HE21 HF5"}="1HE2 HF5"; $resn_exch{"HE31 HF5"}="1HE3 HF5"; $resn_exch{"HE41 HF5"}="1HE4 HF5";
$resn_exch{" HB1 HFA"}="1HB  HFA"; $resn_exch{" HB2 HFA"}="2HB  HFA"; $resn_exch{"H151 HFL"}="1H15 HFL";
$resn_exch{"H152 HFL"}="2H15 HFL"; $resn_exch{"H153 HFL"}="3H15 HFL"; $resn_exch{"HMA1 HFM"}="1HMA HFM";
$resn_exch{"HMA2 HFM"}="2HMA HFM"; $resn_exch{"HMA3 HFM"}="3HMA HFM"; $resn_exch{"HAA1 HFM"}="1HAA HFM";
$resn_exch{"HAA2 HFM"}="2HAA HFM"; $resn_exch{"HBA1 HFM"}="1HBA HFM"; $resn_exch{"HBA2 HFM"}="2HBA HFM";
$resn_exch{"HMB1 HFM"}="1HMB HFM"; $resn_exch{"HMB2 HFM"}="2HMB HFM"; $resn_exch{"HMB3 HFM"}="3HMB HFM";
$resn_exch{"H3C3 HFM"}="3H3C HFM"; $resn_exch{"HMC1 HFM"}="1HMC HFM"; $resn_exch{"HMC2 HFM"}="2HMC HFM";
$resn_exch{"HMC3 HFM"}="3HMC HFM"; $resn_exch{"HBC1 HFM"}="1HBC HFM"; $resn_exch{"HBC2 HFM"}="2HBC HFM";
$resn_exch{"HMD1 HFM"}="1HMD HFM"; $resn_exch{"HMD2 HFM"}="2HMD HFM"; $resn_exch{"HMD3 HFM"}="3HMD HFM";
$resn_exch{"HAD1 HFM"}="1HAD HFM"; $resn_exch{"HAD2 HFM"}="2HAD HFM"; $resn_exch{"HBD1 HFM"}="1HBD HFM";
$resn_exch{"HBD2 HFM"}="2HBD HFM"; $resn_exch{" H21 HFP"}="1H2  HFP"; $resn_exch{" H22 HFP"}="2H2  HFP";
$resn_exch{" H41 HFP"}="1H4  HFP"; $resn_exch{" H42 HFP"}="2H4  HFP"; $resn_exch{" H43 HFP"}="3H4  HFP";
$resn_exch{" H51 HFP"}="1H5  HFP"; $resn_exch{" H52 HFP"}="2H5  HFP"; $resn_exch{" H61 HFP"}="1H6  HFP";
$resn_exch{" H62 HFP"}="2H6  HFP"; $resn_exch{" H71 HFP"}="1H7  HFP"; $resn_exch{" H72 HFP"}="2H7  HFP";
$resn_exch{" H91 HFP"}="1H9  HFP"; $resn_exch{" H92 HFP"}="2H9  HFP"; $resn_exch{" H93 HFP"}="3H9  HFP";
$resn_exch{"H101 HFP"}="1H10 HFP"; $resn_exch{"H102 HFP"}="2H10 HFP"; $resn_exch{"H111 HFP"}="1H11 HFP";
$resn_exch{"H112 HFP"}="2H11 HFP"; $resn_exch{"H121 HFP"}="1H12 HFP"; $resn_exch{"H122 HFP"}="2H12 HFP";
$resn_exch{"H141 HFP"}="1H14 HFP"; $resn_exch{"H142 HFP"}="2H14 HFP"; $resn_exch{"H143 HFP"}="3H14 HFP";
$resn_exch{"H151 HFP"}="1H15 HFP"; $resn_exch{"H152 HFP"}="2H15 HFP"; $resn_exch{"H153 HFP"}="3H15 HFP";
$resn_exch{"H101 HFS"}="1H10 HFS"; $resn_exch{"H102 HFS"}="2H10 HFS"; $resn_exch{"H111 HFS"}="1H11 HFS";
$resn_exch{"H112 HFS"}="2H11 HFS"; $resn_exch{"H121 HFS"}="1H12 HFS"; $resn_exch{"H122 HFS"}="2H12 HFS";
$resn_exch{"H131 HFS"}="1H13 HFS"; $resn_exch{"H132 HFS"}="2H13 HFS"; $resn_exch{"H141 HFS"}="1H14 HFS";
$resn_exch{"H142 HFS"}="2H14 HFS"; $resn_exch{"H121 HFT"}="1H12 HFT"; $resn_exch{"H122 HFT"}="2H12 HFT";
$resn_exch{"H123 HFT"}="3H12 HFT"; $resn_exch{"H131 HFT"}="1H13 HFT"; $resn_exch{"H132 HFT"}="2H13 HFT";
$resn_exch{"H133 HFT"}="3H13 HFT"; $resn_exch{"H141 HFV"}="1H14 HFV"; $resn_exch{"H142 HFV"}="2H14 HFV";
$resn_exch{"H3C1 HFV"}="1H3C HFV"; $resn_exch{"H3C2 HFV"}="2H3C HFV"; $resn_exch{"H4C1 HFV"}="1H4C HFV";
$resn_exch{"H4C2 HFV"}="2H4C HFV"; $resn_exch{"H7C1 HFV"}="1H7C HFV"; $resn_exch{"H7C2 HFV"}="2H7C HFV";
$resn_exch{"H161 HFV"}="1H16 HFV"; $resn_exch{"H162 HFV"}="2H16 HFV"; $resn_exch{" H91 HG9"}="1H9  HG9";
$resn_exch{" H92 HG9"}="2H9  HG9"; $resn_exch{"H121 HG9"}="1H12 HG9"; $resn_exch{"H122 HG9"}="2H12 HG9";
$resn_exch{"H151 HG9"}="1H15 HG9"; $resn_exch{"H152 HG9"}="2H15 HG9"; $resn_exch{"H181 HG9"}="1H18 HG9";
$resn_exch{"H182 HG9"}="2H18 HG9"; $resn_exch{"H211 HG9"}="1H21 HG9"; $resn_exch{"H212 HG9"}="2H21 HG9";
$resn_exch{"H241 HG9"}="1H24 HG9"; $resn_exch{"H242 HG9"}="2H24 HG9"; $resn_exch{"H271 HG9"}="1H27 HG9";
$resn_exch{"H272 HG9"}="2H27 HG9"; $resn_exch{"H351 HG9"}="1H35 HG9"; $resn_exch{"H352 HG9"}="2H35 HG9";
$resn_exch{"H361 HG9"}="1H36 HG9"; $resn_exch{"H362 HG9"}="2H36 HG9"; $resn_exch{"H431 HG9"}="1H43 HG9";
$resn_exch{"H432 HG9"}="2H43 HG9"; $resn_exch{"H601 HG9"}="1H60 HG9"; $resn_exch{"H602 HG9"}="2H60 HG9";
$resn_exch{" H11 HG9"}="1H1  HG9"; $resn_exch{" H12 HG9"}="2H1  HG9"; $resn_exch{" H13 HG9"}="3H1  HG9";
$resn_exch{" HN1 HGA"}="1HN  HGA"; $resn_exch{" HN2 HGA"}="2HN  HGA"; $resn_exch{" HB1 HGA"}="1HB  HGA";
$resn_exch{" HB2 HGA"}="2HB  HGA"; $resn_exch{" HG1 HGA"}="1HG  HGA"; $resn_exch{" HG2 HGA"}="2HG  HGA";
$resn_exch{"HO11 HGB"}="1HO1 HGB"; $resn_exch{"HO31 HGB"}="1HO3 HGB"; $resn_exch{"H121 HGP"}="1H12 HGP";
$resn_exch{"H122 HGP"}="2H12 HGP"; $resn_exch{"H131 HGP"}="1H13 HGP"; $resn_exch{"H132 HGP"}="2H13 HGP";
$resn_exch{"H141 HGP"}="1H14 HGP"; $resn_exch{"H142 HGP"}="2H14 HGP"; $resn_exch{"H151 HGP"}="1H15 HGP";
$resn_exch{"H152 HGP"}="2H15 HGP"; $resn_exch{"H161 HGP"}="1H16 HGP"; $resn_exch{"H162 HGP"}="2H16 HGP";
$resn_exch{"H171 HGP"}="1H17 HGP"; $resn_exch{"H172 HGP"}="2H17 HGP"; $resn_exch{"H181 HGP"}="1H18 HGP";
$resn_exch{"H182 HGP"}="2H18 HGP"; $resn_exch{"H191 HGP"}="1H19 HGP"; $resn_exch{"H192 HGP"}="2H19 HGP";
$resn_exch{"H201 HGP"}="1H20 HGP"; $resn_exch{"H202 HGP"}="2H20 HGP"; $resn_exch{"H211 HGP"}="1H21 HGP";
$resn_exch{"H212 HGP"}="2H21 HGP"; $resn_exch{"H221 HGP"}="1H22 HGP"; $resn_exch{"H222 HGP"}="2H22 HGP";
$resn_exch{"H231 HGP"}="1H23 HGP"; $resn_exch{"H232 HGP"}="2H23 HGP"; $resn_exch{"H241 HGP"}="1H24 HGP";
$resn_exch{"H242 HGP"}="2H24 HGP"; $resn_exch{"H251 HGP"}="1H25 HGP"; $resn_exch{"H252 HGP"}="2H25 HGP";
$resn_exch{"H261 HGP"}="1H26 HGP"; $resn_exch{"H262 HGP"}="2H26 HGP"; $resn_exch{"H263 HGP"}="3H26 HGP";
$resn_exch{" H31 HGP"}="1H3  HGP"; $resn_exch{" H32 HGP"}="2H3  HGP"; $resn_exch{" H11 HGP"}="1H1  HGP";
$resn_exch{" H12 HGP"}="2H1  HGP"; $resn_exch{"H321 HGP"}="1H32 HGP"; $resn_exch{"H322 HGP"}="2H32 HGP";
$resn_exch{"H331 HGP"}="1H33 HGP"; $resn_exch{"H332 HGP"}="2H33 HGP"; $resn_exch{"H341 HGP"}="1H34 HGP";
$resn_exch{"H342 HGP"}="2H34 HGP"; $resn_exch{"H351 HGP"}="1H35 HGP"; $resn_exch{"H352 HGP"}="2H35 HGP";
$resn_exch{"H361 HGP"}="1H36 HGP"; $resn_exch{"H362 HGP"}="2H36 HGP"; $resn_exch{"H371 HGP"}="1H37 HGP";
$resn_exch{"H372 HGP"}="2H37 HGP"; $resn_exch{"H381 HGP"}="1H38 HGP"; $resn_exch{"H382 HGP"}="2H38 HGP";
$resn_exch{"H391 HGP"}="1H39 HGP"; $resn_exch{"H392 HGP"}="2H39 HGP"; $resn_exch{"H401 HGP"}="1H40 HGP";
$resn_exch{"H402 HGP"}="2H40 HGP"; $resn_exch{"H421 HGP"}="1H42 HGP"; $resn_exch{"H422 HGP"}="2H42 HGP";
$resn_exch{"H431 HGP"}="1H43 HGP"; $resn_exch{"H432 HGP"}="2H43 HGP"; $resn_exch{"H441 HGP"}="1H44 HGP";
$resn_exch{"H442 HGP"}="2H44 HGP"; $resn_exch{" H41 HGP"}="1H4  HGP"; $resn_exch{" H42 HGP"}="2H4  HGP";
$resn_exch{" H51 HGP"}="1H5  HGP"; $resn_exch{" H52 HGP"}="2H5  HGP"; $resn_exch{" H61 HGP"}="1H6  HGP";
$resn_exch{" H62 HGP"}="2H6  HGP"; $resn_exch{" H63 HGP"}="3H6  HGP"; $resn_exch{" H71 HGP"}="1H7  HGP";
$resn_exch{" H72 HGP"}="2H7  HGP"; $resn_exch{" H73 HGP"}="3H7  HGP"; $resn_exch{" H81 HGP"}="1H8  HGP";
$resn_exch{" H82 HGP"}="2H8  HGP"; $resn_exch{" H83 HGP"}="3H8  HGP"; $resn_exch{" H91 HGP"}="1H9  HGP";
$resn_exch{" H92 HGP"}="2H9  HGP"; $resn_exch{" H93 HGP"}="3H9  HGP"; $resn_exch{"HN21 HGT"}="1HN2 HGT";
$resn_exch{"HN22 HGT"}="2HN2 HGT"; $resn_exch{"H261 HGT"}="1H26 HGT"; $resn_exch{"H262 HGT"}="2H26 HGT";
$resn_exch{"H221 HGT"}="1H22 HGT"; $resn_exch{"H222 HGT"}="2H22 HGT"; $resn_exch{"H211 HGT"}="1H21 HGT";
$resn_exch{"H212 HGT"}="2H21 HGT"; $resn_exch{"HN21 HGU"}="1HN2 HGU"; $resn_exch{"HN22 HGU"}="2HN2 HGU";
$resn_exch{"H121 HGX"}="1H12 HGX"; $resn_exch{"H122 HGX"}="2H12 HGX"; $resn_exch{"H131 HGX"}="1H13 HGX";
$resn_exch{"H132 HGX"}="2H13 HGX"; $resn_exch{"H141 HGX"}="1H14 HGX"; $resn_exch{"H142 HGX"}="2H14 HGX";
$resn_exch{"H151 HGX"}="1H15 HGX"; $resn_exch{"H152 HGX"}="2H15 HGX"; $resn_exch{"H161 HGX"}="1H16 HGX";
$resn_exch{"H162 HGX"}="2H16 HGX"; $resn_exch{"H171 HGX"}="1H17 HGX"; $resn_exch{"H172 HGX"}="2H17 HGX";
$resn_exch{"H181 HGX"}="1H18 HGX"; $resn_exch{"H182 HGX"}="2H18 HGX"; $resn_exch{"H191 HGX"}="1H19 HGX";
$resn_exch{"H192 HGX"}="2H19 HGX"; $resn_exch{"H201 HGX"}="1H20 HGX"; $resn_exch{"H202 HGX"}="2H20 HGX";
$resn_exch{"H211 HGX"}="1H21 HGX"; $resn_exch{"H212 HGX"}="2H21 HGX"; $resn_exch{"H221 HGX"}="1H22 HGX";
$resn_exch{"H222 HGX"}="2H22 HGX"; $resn_exch{"H231 HGX"}="1H23 HGX"; $resn_exch{"H232 HGX"}="2H23 HGX";
$resn_exch{"H233 HGX"}="3H23 HGX"; $resn_exch{" H31 HGX"}="1H3  HGX"; $resn_exch{" H32 HGX"}="2H3  HGX";
$resn_exch{" H11 HGX"}="1H1  HGX"; $resn_exch{" H12 HGX"}="2H1  HGX"; $resn_exch{"H321 HGX"}="1H32 HGX";
$resn_exch{"H322 HGX"}="2H32 HGX"; $resn_exch{"H331 HGX"}="1H33 HGX"; $resn_exch{"H332 HGX"}="2H33 HGX";
$resn_exch{"H341 HGX"}="1H34 HGX"; $resn_exch{"H342 HGX"}="2H34 HGX"; $resn_exch{"H351 HGX"}="1H35 HGX";
$resn_exch{"H352 HGX"}="2H35 HGX"; $resn_exch{"H361 HGX"}="1H36 HGX"; $resn_exch{"H362 HGX"}="2H36 HGX";
$resn_exch{"H371 HGX"}="1H37 HGX"; $resn_exch{"H372 HGX"}="2H37 HGX"; $resn_exch{"H381 HGX"}="1H38 HGX";
$resn_exch{"H382 HGX"}="2H38 HGX"; $resn_exch{"H391 HGX"}="1H39 HGX"; $resn_exch{"H392 HGX"}="2H39 HGX";
$resn_exch{"H401 HGX"}="1H40 HGX"; $resn_exch{"H402 HGX"}="2H40 HGX"; $resn_exch{"H421 HGX"}="1H42 HGX";
$resn_exch{"H422 HGX"}="2H42 HGX"; $resn_exch{"H431 HGX"}="1H43 HGX"; $resn_exch{"H432 HGX"}="2H43 HGX";
$resn_exch{"H441 HGX"}="1H44 HGX"; $resn_exch{"H442 HGX"}="2H44 HGX"; $resn_exch{"H451 HGX"}="1H45 HGX";
$resn_exch{"H452 HGX"}="2H45 HGX"; $resn_exch{"H461 HGX"}="1H46 HGX"; $resn_exch{"H462 HGX"}="2H46 HGX";
$resn_exch{"H471 HGX"}="1H47 HGX"; $resn_exch{"H472 HGX"}="2H47 HGX"; $resn_exch{"H481 HGX"}="1H48 HGX";
$resn_exch{"H482 HGX"}="2H48 HGX"; $resn_exch{"H483 HGX"}="3H48 HGX"; $resn_exch{" H41 HGX"}="1H4  HGX";
$resn_exch{" H42 HGX"}="2H4  HGX"; $resn_exch{" H51 HGX"}="1H5  HGX"; $resn_exch{" H52 HGX"}="2H5  HGX";
$resn_exch{" H61 HGX"}="1H6  HGX"; $resn_exch{" H62 HGX"}="2H6  HGX"; $resn_exch{" H63 HGX"}="3H6  HGX";
$resn_exch{" H71 HGX"}="1H7  HGX"; $resn_exch{" H72 HGX"}="2H7  HGX"; $resn_exch{" H73 HGX"}="3H7  HGX";
$resn_exch{" H81 HGX"}="1H8  HGX"; $resn_exch{" H82 HGX"}="2H8  HGX"; $resn_exch{" H83 HGX"}="3H8  HGX";
$resn_exch{"H081 HH1"}="1H08 HH1"; $resn_exch{"H082 HH1"}="2H08 HH1"; $resn_exch{"H101 HH1"}="1H10 HH1";
$resn_exch{"H102 HH1"}="2H10 HH1"; $resn_exch{"H201 HH1"}="1H20 HH1"; $resn_exch{"H202 HH1"}="2H20 HH1";
$resn_exch{"H491 HH1"}="1H49 HH1"; $resn_exch{"H492 HH1"}="2H49 HH1"; $resn_exch{"H361 HH1"}="1H36 HH1";
$resn_exch{"H362 HH1"}="2H36 HH1"; $resn_exch{"H451 HH1"}="1H45 HH1"; $resn_exch{"H452 HH1"}="2H45 HH1";
$resn_exch{"H471 HH1"}="1H47 HH1"; $resn_exch{"H472 HH1"}="2H47 HH1"; $resn_exch{"H481 HH1"}="1H48 HH1";
$resn_exch{"H482 HH1"}="2H48 HH1"; $resn_exch{"HN11 HH1"}="1HN1 HH1"; $resn_exch{"HN12 HH1"}="2HN1 HH1";
$resn_exch{" H31 HH2"}="1H3  HH2"; $resn_exch{"HN61 HH2"}="1HN6 HH2"; $resn_exch{"HN62 HH2"}="2HN6 HH2";
$resn_exch{"H111 HH2"}="1H11 HH2"; $resn_exch{"H112 HH2"}="2H11 HH2"; $resn_exch{"HOP2 HH2"}="2HOP HH2";
$resn_exch{"HOP5 HH2"}="5HOP HH2"; $resn_exch{"HOP6 HH2"}="6HOP HH2"; $resn_exch{"HN21 HHA"}="1HN2 HHA";
$resn_exch{"HN22 HHA"}="2HN2 HHA"; $resn_exch{" H11 HHG"}="1H1  HHG"; $resn_exch{" H12 HHG"}="2H1  HHG";
$resn_exch{" H31 HHG"}="1H3  HHG"; $resn_exch{" H32 HHG"}="2H3  HHG"; $resn_exch{" H51 HHG"}="1H5  HHG";
$resn_exch{" H52 HHG"}="2H5  HHG"; $resn_exch{" H61 HHG"}="1H6  HHG"; $resn_exch{" H62 HHG"}="2H6  HHG";
$resn_exch{" H71 HHG"}="1H7  HHG"; $resn_exch{" H72 HHG"}="2H7  HHG"; $resn_exch{" H81 HHG"}="1H8  HHG";
$resn_exch{" H82 HHG"}="2H8  HHG"; $resn_exch{" H91 HHG"}="1H9  HHG"; $resn_exch{" H92 HHG"}="2H9  HHG";
$resn_exch{" H93 HHG"}="3H9  HHG"; $resn_exch{" H2  HHK"}=" HN2 HHK"; $resn_exch{" HB2 HHK"}="2HB  HHK";
$resn_exch{" HB3 HHK"}="1HB  HHK"; $resn_exch{" HG2 HHK"}="2HG  HHK"; $resn_exch{" HG3 HHK"}="1HG  HHK";
$resn_exch{" HD2 HHK"}="2HD  HHK"; $resn_exch{" HD3 HHK"}="1HD  HHK"; $resn_exch{" HE2 HHK"}="2HE  HHK";
$resn_exch{" HE3 HHK"}="3HE  HHK"; $resn_exch{" HZ2 HHK"}="2HZ  HHK"; $resn_exch{" HZ3 HHK"}="1HZ  HHK";
$resn_exch{" HT2 HHK"}="2HT  HHK"; $resn_exch{" HT1 HHK"}="1HT  HHK"; $resn_exch{" HH1 HHK"}="1HH  HHK";
$resn_exch{" HH2 HHK"}="2HH  HHK"; $resn_exch{" H1  HHO"}="1H   HHO"; $resn_exch{" H2  HHO"}="2H   HHO";
$resn_exch{" H11 HHO"}="1H1  HHO"; $resn_exch{" H12 HHO"}="2H1  HHO"; $resn_exch{" H21 HHO"}="1H2  HHO";
$resn_exch{" H22 HHO"}="2H2  HHO"; $resn_exch{" H31 HHO"}="1H3  HHO"; $resn_exch{" H32 HHO"}="2H3  HHO";
$resn_exch{" H41 HHO"}="1H4  HHO"; $resn_exch{" H42 HHO"}="2H4  HHO"; $resn_exch{" H51 HHO"}="1H5  HHO";
$resn_exch{" H52 HHO"}="2H5  HHO"; $resn_exch{" H61 HHO"}="1H6  HHO"; $resn_exch{" H62 HHO"}="2H6  HHO";
$resn_exch{" H71 HHO"}="1H7  HHO"; $resn_exch{" H72 HHO"}="2H7  HHO"; $resn_exch{" H81 HHO"}="1H8  HHO";
$resn_exch{" H82 HHO"}="2H8  HHO"; $resn_exch{" H71 HHP"}="1H7  HHP"; $resn_exch{" H72 HHP"}="2H7  HHP";
$resn_exch{"HN21 HHP"}="1HN2 HHP"; $resn_exch{"HN22 HHP"}="2HN2 HHP"; $resn_exch{"H6A1 HHP"}="1H6A HHP";
$resn_exch{"H6A2 HHP"}="2H6A HHP"; $resn_exch{" H71 HHR"}="1H7  HHR"; $resn_exch{"HN21 HHR"}="1HN2 HHR";
$resn_exch{"HN22 HHR"}="2HN2 HHR"; $resn_exch{"H6A1 HHR"}="1H6A HHR"; $resn_exch{"H6A2 HHR"}="2H6A HHR";
$resn_exch{"H1C1 HI5"}="1H1C HI5"; $resn_exch{"H1C2 HI5"}="2H1C HI5"; $resn_exch{"H3C1 HI5"}="1H3C HI5";
$resn_exch{"H3C2 HI5"}="2H3C HI5"; $resn_exch{"H121 HI5"}="1H12 HI5"; $resn_exch{"H122 HI5"}="2H12 HI5";
$resn_exch{"H131 HI5"}="1H13 HI5"; $resn_exch{"H132 HI5"}="2H13 HI5"; $resn_exch{"H141 HI5"}="1H14 HI5";
$resn_exch{"H142 HI5"}="2H14 HI5"; $resn_exch{"H143 HI5"}="3H14 HI5"; $resn_exch{"H321 HI5"}="1H32 HI5";
$resn_exch{"H322 HI5"}="2H32 HI5"; $resn_exch{"H331 HI5"}="1H33 HI5"; $resn_exch{"H332 HI5"}="2H33 HI5";
$resn_exch{"H341 HI5"}="1H34 HI5"; $resn_exch{"H342 HI5"}="2H34 HI5"; $resn_exch{"H343 HI5"}="3H34 HI5";
$resn_exch{"HN41 HI6"}="1HN4 HI6"; $resn_exch{"HN42 HI6"}="2HN4 HI6"; $resn_exch{" H81 HI6"}="1H8  HI6";
$resn_exch{" H82 HI6"}="2H8  HI6"; $resn_exch{" H71 HI6"}="1H7  HI6"; $resn_exch{" H72 HI6"}="2H7  HI6";
$resn_exch{" H   HIA"}="1HN  HIA"; $resn_exch{" H2  HIA"}="2HN  HIA"; $resn_exch{" HB2 HIA"}="1HBC HIA";
$resn_exch{" HB3 HIA"}="2HBC HIA"; $resn_exch{"HXT1 HIA"}="1HXT HIA"; $resn_exch{"HXT2 HIA"}="2HXT HIA";
$resn_exch{" H81 HIB"}="1H8  HIB"; $resn_exch{" H82 HIB"}="2H8  HIB"; $resn_exch{" H83 HIB"}="3H8  HIB";
$resn_exch{" H91 HIB"}="1H9  HIB"; $resn_exch{" H92 HIB"}="2H9  HIB"; $resn_exch{" H93 HIB"}="3H9  HIB";
$resn_exch{" HX1 HIB"}="1HX  HIB"; $resn_exch{" HX2 HIB"}="2HX  HIB"; $resn_exch{" HX3 HIB"}="3HX  HIB";
$resn_exch{" H2  HIC"}=" HN2 HIC"; $resn_exch{" HB2 HIC"}="1HB  HIC"; $resn_exch{" HB3 HIC"}="2HB  HIC";
$resn_exch{" HZ1 HIC"}="1HZ  HIC"; $resn_exch{" HZ2 HIC"}="2HZ  HIC"; $resn_exch{" HZ3 HIC"}="3HZ  HIC";
$resn_exch{"HAA1 HIF"}="1HAA HIF"; $resn_exch{"HAA2 HIF"}="2HAA HIF"; $resn_exch{"HMA1 HIF"}="1HMA HIF";
$resn_exch{"HMA2 HIF"}="2HMA HIF"; $resn_exch{"HMA3 HIF"}="3HMA HIF"; $resn_exch{"HBA1 HIF"}="1HBA HIF";
$resn_exch{"HBA2 HIF"}="2HBA HIF"; $resn_exch{"HMB1 HIF"}="1HMB HIF"; $resn_exch{"HMB2 HIF"}="2HMB HIF";
$resn_exch{"HMB3 HIF"}="3HMB HIF"; $resn_exch{"HAB1 HIF"}="1HAB HIF"; $resn_exch{"HAB2 HIF"}="2HAB HIF";
$resn_exch{"HBB1 HIF"}="1HBB HIF"; $resn_exch{"HBB2 HIF"}="2HBB HIF"; $resn_exch{"HBB3 HIF"}="3HBB HIF";
$resn_exch{"HAC1 HIF"}="1HAC HIF"; $resn_exch{"HAC2 HIF"}="2HAC HIF"; $resn_exch{"HBC1 HIF"}="1HBC HIF";
$resn_exch{"HBC2 HIF"}="2HBC HIF"; $resn_exch{"HBC3 HIF"}="3HBC HIF"; $resn_exch{"HIF1 HIF"}="1HIF HIF";
$resn_exch{"HIF2 HIF"}="2HIF HIF"; $resn_exch{"HIF3 HIF"}="3HIF HIF"; $resn_exch{"HMD1 HIF"}="1HMD HIF";
$resn_exch{"HMD2 HIF"}="2HMD HIF"; $resn_exch{"HMD3 HIF"}="3HMD HIF"; $resn_exch{"HAD1 HIF"}="1HAD HIF";
$resn_exch{"HAD2 HIF"}="2HAD HIF"; $resn_exch{"HBD1 HIF"}="1HBD HIF"; $resn_exch{"HBD2 HIF"}="2HBD HIF";
$resn_exch{" HN1 HII"}="1HN  HII"; $resn_exch{" HN2 HII"}="2HN  HII"; $resn_exch{" HN3 HII"}="3HN  HII";
$resn_exch{" HB1 HII"}="1HB  HII"; $resn_exch{" HB2 HII"}="2HB  HII"; $resn_exch{"HNZ1 HII"}="1HNZ HII";
$resn_exch{"HNZ2 HII"}="2HNZ HII"; $resn_exch{" H11 HIN"}="1H1  HIN"; $resn_exch{" H12 HIN"}="2H1  HIN";
$resn_exch{" H21 HIN"}="1H2  HIN"; $resn_exch{" H22 HIN"}="2H2  HIN"; $resn_exch{"H101 HIN"}="1H10 HIN";
$resn_exch{"H102 HIN"}="2H10 HIN"; $resn_exch{"H103 HIN"}="3H10 HIN"; $resn_exch{"H111 HIN"}="1H11 HIN";
$resn_exch{"H112 HIN"}="2H11 HIN"; $resn_exch{"H113 HIN"}="3H11 HIN"; $resn_exch{"H121 HIN"}="1H12 HIN";
$resn_exch{"H122 HIN"}="2H12 HIN"; $resn_exch{"H411 HIO"}="1H41 HIO"; $resn_exch{"H412 HIO"}="2H41 HIO";
$resn_exch{"H413 HIO"}="3H41 HIO"; $resn_exch{"H421 HIO"}="1H42 HIO"; $resn_exch{"H422 HIO"}="2H42 HIO";
$resn_exch{"H423 HIO"}="3H42 HIO"; $resn_exch{" H   HIP"}="1HN  HIP"; $resn_exch{" H2  HIP"}="2HN  HIP";
$resn_exch{" HB2 HIP"}="1HB  HIP"; $resn_exch{" HB3 HIP"}="2HB  HIP"; $resn_exch{"HOP2 HIP"}="2HOP HIP";
$resn_exch{"HOP3 HIP"}="3HOP HIP"; $resn_exch{" HN1 HIQ"}="1HN  HIQ"; $resn_exch{" HN2 HIQ"}="2HN  HIQ";
$resn_exch{" HB1 HIQ"}="1HB  HIQ"; $resn_exch{" HB2 HIQ"}="2HB  HIQ"; $resn_exch{"H1C1 HIQ"}="1H1C HIQ";
$resn_exch{"H1C2 HIQ"}="2H1C HIQ"; $resn_exch{"H3C1 HIQ"}="1H3C HIQ"; $resn_exch{"H3C2 HIQ"}="2H3C HIQ";
$resn_exch{" H2  HIS"}=" HN2 HIS"; $resn_exch{" HB2 HIS"}="1HB  HIS"; $resn_exch{" HB3 HIS"}="2HB  HIS";
$resn_exch{" H51 HIT"}="1H5  HIT"; $resn_exch{" H52 HIT"}="2H5  HIT"; $resn_exch{" H53 HIT"}="3H5  HIT";
$resn_exch{" H91 HIT"}="1H9  HIT"; $resn_exch{" H92 HIT"}="2H9  HIT"; $resn_exch{" H93 HIT"}="3H9  HIT";
$resn_exch{"H171 HIT"}="1H17 HIT"; $resn_exch{"H172 HIT"}="2H17 HIT"; $resn_exch{"H173 HIT"}="3H17 HIT";
$resn_exch{" H31 HIU"}="1H3  HIU"; $resn_exch{" H32 HIU"}="2H3  HIU"; $resn_exch{" H33 HIU"}="3H3  HIU";
$resn_exch{" H41 HIU"}="1H4  HIU"; $resn_exch{" H42 HIU"}="2H4  HIU"; $resn_exch{"H3C1 HKV"}="1H3C HKV";
$resn_exch{"H3C2 HKV"}="2H3C HKV"; $resn_exch{" H41 HKV"}="1H4  HKV"; $resn_exch{" HB1 HLE"}="1HB  HLE";
$resn_exch{" HB2 HLE"}="2HB  HLE"; $resn_exch{"HD11 HLE"}="1HD1 HLE"; $resn_exch{"HD12 HLE"}="2HD1 HLE";
$resn_exch{"HD13 HLE"}="3HD1 HLE"; $resn_exch{"HD21 HLE"}="1HD2 HLE"; $resn_exch{"HD22 HLE"}="2HD2 HLE";
$resn_exch{"HD23 HLE"}="3HD2 HLE"; $resn_exch{"H4N1 HLO"}="1H4N HLO"; $resn_exch{"H4N2 HLO"}="2H4N HLO";
$resn_exch{"H8C1 HLO"}="1H8C HLO"; $resn_exch{"H8C2 HLO"}="2H8C HLO"; $resn_exch{"H7C1 HLO"}="1H7C HLO";
$resn_exch{"H7C2 HLO"}="2H7C HLO"; $resn_exch{" H   HLU"}="1HN  HLU"; $resn_exch{" H2  HLU"}="2HN  HLU";
$resn_exch{"HD11 HLU"}="1HD1 HLU"; $resn_exch{"HD12 HLU"}="2HD1 HLU"; $resn_exch{"HD13 HLU"}="3HD1 HLU";
$resn_exch{"HD21 HLU"}="1HD2 HLU"; $resn_exch{"HD22 HLU"}="2HD2 HLU"; $resn_exch{"HD23 HLU"}="3HD2 HLU";
$resn_exch{" HH  HLU"}=" HO  HLU"; $resn_exch{"H101 HM2"}="1H10 HM2"; $resn_exch{"H102 HM2"}="2H10 HM2";
$resn_exch{"H111 HM2"}="1H11 HM2"; $resn_exch{"H112 HM2"}="2H11 HM2"; $resn_exch{"H181 HM2"}="1H18 HM2";
$resn_exch{"H182 HM2"}="2H18 HM2"; $resn_exch{"H183 HM2"}="3H18 HM2"; $resn_exch{"H121 HM4"}="1H12 HM4";
$resn_exch{"H122 HM4"}="2H12 HM4"; $resn_exch{"H123 HM4"}="3H12 HM4"; $resn_exch{"H131 HM4"}="1H13 HM4";
$resn_exch{"H132 HM4"}="2H13 HM4"; $resn_exch{"H133 HM4"}="3H13 HM4"; $resn_exch{"H141 HM4"}="1H14 HM4";
$resn_exch{"H142 HM4"}="2H14 HM4"; $resn_exch{"H143 HM4"}="3H14 HM4"; $resn_exch{"H121 HM5"}="1H12 HM5";
$resn_exch{"H122 HM5"}="2H12 HM5"; $resn_exch{"H123 HM5"}="3H12 HM5"; $resn_exch{"H131 HM5"}="1H13 HM5";
$resn_exch{"H132 HM5"}="2H13 HM5"; $resn_exch{"H133 HM5"}="3H13 HM5"; $resn_exch{"H141 HM5"}="1H14 HM5";
$resn_exch{"H142 HM5"}="2H14 HM5"; $resn_exch{"H143 HM5"}="3H14 HM5"; $resn_exch{" HN1 HMA"}="1HN  HMA";
$resn_exch{"HN2A HMA"}="2HN  HMA"; $resn_exch{" HB1 HMA"}="1HB  HMA"; $resn_exch{" HB2 HMA"}="2HB  HMA";
$resn_exch{" HB3 HMA"}="3HB  HMA"; $resn_exch{" HA1 HMB"}="1HA  HMB"; $resn_exch{" HA2 HMB"}="2HA  HMB";
$resn_exch{" HB1 HMB"}="1HB  HMB"; $resn_exch{" HB2 HMB"}="2HB  HMB"; $resn_exch{" HE1 HMB"}="1HE  HMB";
$resn_exch{" HE2 HMB"}="2HE  HMB"; $resn_exch{" HE3 HMB"}="3HE  HMB"; $resn_exch{" H71 HMC"}="1H7  HMC";
$resn_exch{" H72 HMC"}="2H7  HMC"; $resn_exch{"HN51 HMD"}="1HN5 HMD"; $resn_exch{"HN52 HMD"}="2HN5 HMD";
$resn_exch{"HAC1 HME"}="1HAC HME"; $resn_exch{"HAC2 HME"}="2HAC HME"; $resn_exch{"HBC1 HME"}="1HBC HME";
$resn_exch{"HBC2 HME"}="2HBC HME"; $resn_exch{"HBC3 HME"}="3HBC HME"; $resn_exch{"HMC1 HME"}="1HMC HME";
$resn_exch{"HMC2 HME"}="2HMC HME"; $resn_exch{"HMC3 HME"}="3HMC HME"; $resn_exch{"HMB1 HME"}="1HMB HME";
$resn_exch{"HMB2 HME"}="2HMB HME"; $resn_exch{"HMB3 HME"}="3HMB HME"; $resn_exch{"HAB1 HME"}="1HAB HME";
$resn_exch{"HAB2 HME"}="2HAB HME"; $resn_exch{"HBB1 HME"}="1HBB HME"; $resn_exch{"HBB2 HME"}="2HBB HME";
$resn_exch{"HBB3 HME"}="3HBB HME"; $resn_exch{"HMA1 HME"}="1HMA HME"; $resn_exch{"HMA2 HME"}="2HMA HME";
$resn_exch{"HMA3 HME"}="3HMA HME"; $resn_exch{"HAA1 HME"}="1HAA HME"; $resn_exch{"HAA2 HME"}="2HAA HME";
$resn_exch{"HBA1 HME"}="1HBA HME"; $resn_exch{"HBA2 HME"}="2HBA HME"; $resn_exch{"HAD1 HME"}="1HAD HME";
$resn_exch{"HAD2 HME"}="2HAD HME"; $resn_exch{"HBD1 HME"}="1HBD HME"; $resn_exch{"HBD2 HME"}="2HBD HME";
$resn_exch{"HMD1 HME"}="1HMD HME"; $resn_exch{"HMD2 HME"}="2HMD HME"; $resn_exch{"HMD3 HME"}="3HMD HME";
$resn_exch{" H   HMF"}="1HN  HMF"; $resn_exch{" H2  HMF"}="2HN  HMF"; $resn_exch{" HB2 HMF"}="1HB  HMF";
$resn_exch{" HB3 HMF"}="2HB  HMF"; $resn_exch{" HG2 HMF"}="1HG  HMF"; $resn_exch{" HG3 HMF"}="2HG  HMF";
$resn_exch{" HC2 HMF"}=" H2  HMF"; $resn_exch{"H5'1 HMG"}="1H5* HMG"; $resn_exch{"H5'2 HMG"}="2H5* HMG";
$resn_exch{" H4B HMG"}="H4*A HMG"; $resn_exch{" H3B HMG"}="H3*A HMG"; $resn_exch{" H2' HMG"}=" H2* HMG";
$resn_exch{" H2B HMG"}="H2*A HMG"; $resn_exch{" H1B HMG"}="H1*A HMG"; $resn_exch{"H2A1 HMG"}="1H2A HMG";
$resn_exch{"H8A1 HMG"}="1H8A HMG"; $resn_exch{"H121 HMG"}="1H12 HMG"; $resn_exch{"H122 HMG"}="2H12 HMG";
$resn_exch{"H131 HMG"}="1H13 HMG"; $resn_exch{"H132 HMG"}="2H13 HMG"; $resn_exch{"H133 HMG"}="3H13 HMG";
$resn_exch{"H141 HMG"}="1H14 HMG"; $resn_exch{"H142 HMG"}="2H14 HMG"; $resn_exch{"H143 HMG"}="3H14 HMG";
$resn_exch{"H101 HMG"}="1H10 HMG"; $resn_exch{"H7P1 HMG"}="1H7P HMG"; $resn_exch{"H7P2 HMG"}="2H7P HMG";
$resn_exch{"H6P1 HMG"}="1H6P HMG"; $resn_exch{"H6P2 HMG"}="2H6P HMG"; $resn_exch{"H3P1 HMG"}="1H3P HMG";
$resn_exch{"H3P2 HMG"}="2H3P HMG"; $resn_exch{"H2P1 HMG"}="1H2P HMG"; $resn_exch{"H2P2 HMG"}="2H2P HMG";
$resn_exch{" H21 HMG"}="1H2  HMG"; $resn_exch{" H22 HMG"}="2H2  HMG"; $resn_exch{" H41 HMG"}="1H4  HMG";
$resn_exch{" H42 HMG"}="2H4  HMG"; $resn_exch{" H61 HMG"}="1H6  HMG"; $resn_exch{" H62 HMG"}="2H6  HMG";
$resn_exch{" H63 HMG"}="3H6  HMG"; $resn_exch{"HCM1 HMH"}="1HCM HMH"; $resn_exch{"HCM2 HMH"}="2HCM HMH";
$resn_exch{"HCM3 HMH"}="3HCM HMH"; $resn_exch{"HN41 HMH"}="1HN4 HMH"; $resn_exch{"HN42 HMH"}="2HN4 HMH";
$resn_exch{"HC71 HMH"}="1HC7 HMH"; $resn_exch{"HC72 HMH"}="2HC7 HMH"; $resn_exch{" HB1 HMI"}="1HB  HMI";
$resn_exch{" HB2 HMI"}="2HB  HMI"; $resn_exch{"HD11 HMI"}="1HD1 HMI"; $resn_exch{"HD12 HMI"}="2HD1 HMI";
$resn_exch{"HD13 HMI"}="3HD1 HMI"; $resn_exch{"HD21 HMI"}="1HD2 HMI"; $resn_exch{"HD22 HMI"}="2HD2 HMI";
$resn_exch{"HD23 HMI"}="3HD2 HMI"; $resn_exch{" H11 HMK"}="1H1  HMK"; $resn_exch{"H12A HMK"}="2H1  HMK";
$resn_exch{"H161 HMK"}="1H16 HMK"; $resn_exch{"H162 HMK"}="2H16 HMK"; $resn_exch{"H311 HMM"}="1H31 HMM";
$resn_exch{"H312 HMM"}="2H31 HMM"; $resn_exch{"H313 HMM"}="3H31 HMM"; $resn_exch{"H141 HMM"}="1H14 HMM";
$resn_exch{"H142 HMM"}="2H14 HMM"; $resn_exch{"H151 HMM"}="1H15 HMM"; $resn_exch{"H161 HMM"}="1H16 HMM";
$resn_exch{"H171 HMM"}="1H17 HMM"; $resn_exch{"H181 HMM"}="1H18 HMM"; $resn_exch{"H182 HMM"}="2H18 HMM";
$resn_exch{"HC31 HMN"}="1HC3 HMN"; $resn_exch{"HC32 HMN"}="2HC3 HMN"; $resn_exch{"H111 HMN"}="1H11 HMN";
$resn_exch{"H112 HMN"}="2H11 HMN"; $resn_exch{"H113 HMN"}="3H11 HMN"; $resn_exch{"HC91 HMN"}="1HC9 HMN";
$resn_exch{"HC92 HMN"}="2HC9 HMN"; $resn_exch{"H201 HMO"}="1H20 HMO"; $resn_exch{"H202 HMO"}="2H20 HMO";
$resn_exch{"H203 HMO"}="3H20 HMO"; $resn_exch{"H1'1 HMP"}="1H1' HMP"; $resn_exch{"H1'2 HMP"}="2H1' HMP";
$resn_exch{"H3'1 HMP"}="1H3' HMP"; $resn_exch{"H3'2 HMP"}="2H3' HMP"; $resn_exch{"H3'3 HMP"}="3H3' HMP";
$resn_exch{"H4'1 HMP"}="1H4' HMP"; $resn_exch{"H4'2 HMP"}="2H4' HMP"; $resn_exch{"H4'3 HMP"}="3H4' HMP";
$resn_exch{" H   HMR"}="1HN  HMR"; $resn_exch{" H2  HMR"}="2HN  HMR"; $resn_exch{" HB2 HMR"}="1HB  HMR";
$resn_exch{" HB3 HMR"}="2HB  HMR"; $resn_exch{" HG2 HMR"}="1HG  HMR"; $resn_exch{" HG3 HMR"}="2HG  HMR";
$resn_exch{" HD2 HMR"}="1HD  HMR"; $resn_exch{" HD3 HMR"}="2HD  HMR"; $resn_exch{" HE  HMR"}=" HNE HMR";
$resn_exch{" HH1 HMR"}=" HN1 HMR"; $resn_exch{"HH21 HMR"}="1HN2 HMR"; $resn_exch{"HH22 HMR"}="2HN2 HMR";
$resn_exch{" HC1 HMR"}="1HC  HMR"; $resn_exch{" HC2 HMR"}="2HC  HMR"; $resn_exch{"HM51 HMU"}="1HM5 HMU";
$resn_exch{"HM52 HMU"}="2HM5 HMU"; $resn_exch{"H5C1 HNE"}="1H5C HNE"; $resn_exch{"H5C2 HNE"}="2H5C HNE";
$resn_exch{"H6C1 HNE"}="1H6C HNE"; $resn_exch{"H6C2 HNE"}="2H6C HNE"; $resn_exch{"H7C1 HNE"}="1H7C HNE";
$resn_exch{"H7C2 HNE"}="2H7C HNE"; $resn_exch{"H8C1 HNE"}="1H8C HNE"; $resn_exch{"H8C2 HNE"}="2H8C HNE";
$resn_exch{"H9C1 HNE"}="1H9C HNE"; $resn_exch{"H9C2 HNE"}="2H9C HNE"; $resn_exch{"H9C3 HNE"}="3H9C HNE";
$resn_exch{"HMA1 HNI"}="1HMA HNI"; $resn_exch{"HMA2 HNI"}="2HMA HNI"; $resn_exch{"HMA3 HNI"}="3HMA HNI";
$resn_exch{"HAA1 HNI"}="1HAA HNI"; $resn_exch{"HAA2 HNI"}="2HAA HNI"; $resn_exch{"HBA1 HNI"}="1HBA HNI";
$resn_exch{"HBA2 HNI"}="2HBA HNI"; $resn_exch{"HMB1 HNI"}="1HMB HNI"; $resn_exch{"HMB2 HNI"}="2HMB HNI";
$resn_exch{"HMB3 HNI"}="3HMB HNI"; $resn_exch{"HBB1 HNI"}="1HBB HNI"; $resn_exch{"HBB2 HNI"}="2HBB HNI";
$resn_exch{"HMC1 HNI"}="1HMC HNI"; $resn_exch{"HMC2 HNI"}="2HMC HNI"; $resn_exch{"HMC3 HNI"}="3HMC HNI";
$resn_exch{"HBC1 HNI"}="1HBC HNI"; $resn_exch{"HBC2 HNI"}="2HBC HNI"; $resn_exch{"HMD1 HNI"}="1HMD HNI";
$resn_exch{"HMD2 HNI"}="2HMD HNI"; $resn_exch{"HMD3 HNI"}="3HMD HNI"; $resn_exch{"HAD1 HNI"}="1HAD HNI";
$resn_exch{"HAD2 HNI"}="2HAD HNI"; $resn_exch{"HBD1 HNI"}="1HBD HNI"; $resn_exch{"HBD2 HNI"}="2HBD HNI";
$resn_exch{" H11 HNP"}="1H1  HNP"; $resn_exch{" H12 HNP"}="2H1  HNP"; $resn_exch{" H11 HNT"}="1H1  HNT";
$resn_exch{" H12 HNT"}="2H1  HNT"; $resn_exch{"H41A HNT"}="1H4  HNT"; $resn_exch{" H42 HNT"}="2H4  HNT";
$resn_exch{"H221 HNT"}="1H22 HNT"; $resn_exch{"H222 HNT"}="2H22 HNT"; $resn_exch{" HN1 HOA"}="1HN  HOA";
$resn_exch{" HN2 HOA"}="2HN  HOA"; $resn_exch{"HOP2 HOB"}="2HOP HOB"; $resn_exch{" H5' HOB"}="1H5* HOB";
$resn_exch{"H5'' HOB"}="2H5* HOB"; $resn_exch{" H4' HOB"}=" H4* HOB"; $resn_exch{" H3' HOB"}=" H3* HOB";
$resn_exch{" H2' HOB"}="1H2* HOB"; $resn_exch{"H2'' HOB"}="2H2* HOB"; $resn_exch{" H1' HOB"}=" H1* HOB";
$resn_exch{" H21 HOB"}="1H2  HOB"; $resn_exch{" H22 HOB"}="2H2  HOB"; $resn_exch{" H31 HOB"}="1H3  HOB";
$resn_exch{" H32 HOB"}="2H3  HOB"; $resn_exch{" H41 HOB"}="1H4  HOB"; $resn_exch{" H42 HOB"}="2H4  HOB";
$resn_exch{" H9  HOB"}="9H   HOB"; $resn_exch{"H141 HOB"}="1H14 HOB"; $resn_exch{"H142 HOB"}="2H14 HOB";
$resn_exch{"H131 HOB"}="1H13 HOB"; $resn_exch{"H132 HOB"}="2H13 HOB"; $resn_exch{"H101 HOB"}="1H10 HOB";
$resn_exch{"H102 HOB"}="2H10 HOB"; $resn_exch{" H16 HOB"}="6H1  HOB"; $resn_exch{"H171 HOB"}="1H17 HOB";
$resn_exch{"H172 HOB"}="2H17 HOB"; $resn_exch{" H18 HOB"}="8H1  HOB"; $resn_exch{" H19 HOB"}="9H1  HOB";
$resn_exch{"H201 HOB"}="1H20 HOB"; $resn_exch{"H202 HOB"}="2H20 HOB"; $resn_exch{"H211 HOB"}="1H21 HOB";
$resn_exch{"H212 HOB"}="2H21 HOB"; $resn_exch{" H23 HOB"}="3H2  HOB"; $resn_exch{"H251 HOB"}="1H25 HOB";
$resn_exch{"H252 HOB"}="2H25 HOB"; $resn_exch{"H261 HOB"}="1H26 HOB"; $resn_exch{"H262 HOB"}="2H26 HOB";
$resn_exch{" H27 HOB"}="7H2  HOB"; $resn_exch{" H28 HOB"}="8H2  HOB"; $resn_exch{"H291 HOB"}="1H29 HOB";
$resn_exch{"H292 HOB"}="2H29 HOB"; $resn_exch{"H293 HOB"}="3H29 HOB"; $resn_exch{"H301 HOB"}="1H30 HOB";
$resn_exch{"H302 HOB"}="2H30 HOB"; $resn_exch{"H311 HOB"}="1H31 HOB"; $resn_exch{"H312 HOB"}="2H31 HOB";
$resn_exch{"H321 HOB"}="1H32 HOB"; $resn_exch{"H322 HOB"}="2H32 HOB"; $resn_exch{" H33 HOB"}="3H3  HOB";
$resn_exch{"H341 HOB"}="1H34 HOB"; $resn_exch{"H342 HOB"}="2H34 HOB"; $resn_exch{"H343 HOB"}="3H34 HOB";
$resn_exch{"H351 HOB"}="1H35 HOB"; $resn_exch{"H352 HOB"}="2H35 HOB"; $resn_exch{"H353 HOB"}="3H35 HOB";
$resn_exch{"H241 HOB"}="1H24 HOB"; $resn_exch{"H242 HOB"}="2H24 HOB"; $resn_exch{"H243 HOB"}="3H24 HOB";
$resn_exch{"H151 HOB"}="1H15 HOB"; $resn_exch{"H152 HOB"}="2H15 HOB"; $resn_exch{"H153 HOB"}="3H15 HOB";
$resn_exch{"HO3' HOB"}="H3T  HOB"; $resn_exch{"HOP3 HOB"}="3HOP HOB"; $resn_exch{" H81 HOC"}="1H8  HOC";
$resn_exch{" H82 HOC"}="2H8  HOC"; $resn_exch{" H83 HOC"}="3H8  HOC"; $resn_exch{" H71 HOC"}="1H7  HOC";
$resn_exch{" H72 HOC"}="2H7  HOC"; $resn_exch{" H31 HOC"}="1H3  HOC"; $resn_exch{" H32 HOC"}="2H3  HOC";
$resn_exch{" H41 HOC"}="1H4  HOC"; $resn_exch{" H42 HOC"}="2H4  HOC"; $resn_exch{" H51 HOC"}="1H5  HOC";
$resn_exch{" H52 HOC"}="2H5  HOC"; $resn_exch{" H61 HOC"}="1H6  HOC"; $resn_exch{" H62 HOC"}="2H6  HOC";
$resn_exch{"H211 HOE"}="1H21 HOE"; $resn_exch{"H212 HOE"}="2H21 HOE"; $resn_exch{"H221 HOE"}="1H22 HOE";
$resn_exch{"H222 HOE"}="2H22 HOE"; $resn_exch{"H231 HOE"}="1H23 HOE"; $resn_exch{"H232 HOE"}="2H23 HOE";
$resn_exch{"H241 HOE"}="1H24 HOE"; $resn_exch{"H242 HOE"}="2H24 HOE"; $resn_exch{"H251 HOE"}="1H25 HOE";
$resn_exch{"H252 HOE"}="2H25 HOE"; $resn_exch{"H253 HOE"}="3H25 HOE"; $resn_exch{" H1  HOH"}="1H   HOH";
$resn_exch{" H2  HOH"}="2H   HOH"; $resn_exch{"HOP2 HOL"}="2HOP HOL"; $resn_exch{" H5' HOL"}="1H5* HOL";
$resn_exch{"H5'' HOL"}="2H5* HOL"; $resn_exch{" H4' HOL"}=" H4* HOL"; $resn_exch{" H3' HOL"}=" H3* HOL";
$resn_exch{" H2' HOL"}="1H2* HOL"; $resn_exch{"H2'' HOL"}="2H2* HOL"; $resn_exch{" H1' HOL"}=" H1* HOL";
$resn_exch{" H21 HOL"}="1H2  HOL"; $resn_exch{" H22 HOL"}="2H2  HOL"; $resn_exch{" H31 HOL"}="1H3  HOL";
$resn_exch{" H32 HOL"}="2H3  HOL"; $resn_exch{" H41 HOL"}="1H4  HOL"; $resn_exch{" H42 HOL"}="2H4  HOL";
$resn_exch{" H9  HOL"}="9H   HOL"; $resn_exch{"H141 HOL"}="1H14 HOL"; $resn_exch{"H142 HOL"}="2H14 HOL";
$resn_exch{"H131 HOL"}="1H13 HOL"; $resn_exch{"H132 HOL"}="2H13 HOL"; $resn_exch{"H101 HOL"}="1H10 HOL";
$resn_exch{"H102 HOL"}="2H10 HOL"; $resn_exch{" H16 HOL"}="6H1  HOL"; $resn_exch{"H171 HOL"}="1H17 HOL";
$resn_exch{"H172 HOL"}="2H17 HOL"; $resn_exch{" H18 HOL"}="8H1  HOL"; $resn_exch{" H19 HOL"}="9H1  HOL";
$resn_exch{"H201 HOL"}="1H20 HOL"; $resn_exch{"H202 HOL"}="2H20 HOL"; $resn_exch{"H211 HOL"}="1H21 HOL";
$resn_exch{"H212 HOL"}="2H21 HOL"; $resn_exch{" H23 HOL"}="3H2  HOL"; $resn_exch{"H251 HOL"}="1H25 HOL";
$resn_exch{"H252 HOL"}="2H25 HOL"; $resn_exch{"H261 HOL"}="1H26 HOL"; $resn_exch{"H262 HOL"}="2H26 HOL";
$resn_exch{" H27 HOL"}="7H2  HOL"; $resn_exch{" H28 HOL"}="8H2  HOL"; $resn_exch{"H291 HOL"}="1H29 HOL";
$resn_exch{"H292 HOL"}="2H29 HOL"; $resn_exch{"H293 HOL"}="3H29 HOL"; $resn_exch{"H301 HOL"}="1H30 HOL";
$resn_exch{"H302 HOL"}="2H30 HOL"; $resn_exch{"H311 HOL"}="1H31 HOL"; $resn_exch{"H312 HOL"}="2H31 HOL";
$resn_exch{"H321 HOL"}="1H32 HOL"; $resn_exch{"H322 HOL"}="2H32 HOL"; $resn_exch{" H33 HOL"}="3H3  HOL";
$resn_exch{"H341 HOL"}="1H34 HOL"; $resn_exch{"H342 HOL"}="2H34 HOL"; $resn_exch{"H343 HOL"}="3H34 HOL";
$resn_exch{"H351 HOL"}="1H35 HOL"; $resn_exch{"H352 HOL"}="2H35 HOL"; $resn_exch{"H353 HOL"}="3H35 HOL";
$resn_exch{"H241 HOL"}="1H24 HOL"; $resn_exch{"H242 HOL"}="2H24 HOL"; $resn_exch{"H243 HOL"}="3H24 HOL";
$resn_exch{"H151 HOL"}="1H15 HOL"; $resn_exch{"H152 HOL"}="2H15 HOL"; $resn_exch{"H153 HOL"}="3H15 HOL";
$resn_exch{"HO3' HOL"}="H3T  HOL"; $resn_exch{"HOP3 HOL"}="3HOP HOL"; $resn_exch{"HA41 HOM"}="1HA4 HOM";
$resn_exch{"HA61 HOM"}="1HA6 HOM"; $resn_exch{"HA71 HOM"}="1HA7 HOM"; $resn_exch{" HB1 HOM"}="1HB  HOM";
$resn_exch{" HB2 HOM"}="2HB  HOM"; $resn_exch{"HB71 HOM"}="1HB7 HOM"; $resn_exch{"HB72 HOM"}="2HB7 HOM";
$resn_exch{"HB81 HOM"}="1HB8 HOM"; $resn_exch{"HB82 HOM"}="2HB8 HOM"; $resn_exch{"HB83 HOM"}="3HB8 HOM";
$resn_exch{"HO41 HOM"}="1HO4 HOM"; $resn_exch{"HO11 HOM"}="1HO1 HOM"; $resn_exch{"H121 HOP"}="1H12 HOP";
$resn_exch{"H122 HOP"}="2H12 HOP"; $resn_exch{" H91 HOP"}="1H9  HOP"; $resn_exch{" H92 HOP"}="2H9  HOP";
$resn_exch{" H81 HOP"}="1H8  HOP"; $resn_exch{" H82 HOP"}="2H8  HOP"; $resn_exch{"H231 HOP"}="1H23 HOP";
$resn_exch{"H232 HOP"}="2H23 HOP"; $resn_exch{"H241 HOP"}="1H24 HOP"; $resn_exch{"H242 HOP"}="2H24 HOP";
$resn_exch{"H251 HOP"}="1H25 HOP"; $resn_exch{"H252 HOP"}="2H25 HOP"; $resn_exch{"H511 HOQ"}="1H51 HOQ";
$resn_exch{"H512 HOQ"}="2H51 HOQ"; $resn_exch{"H513 HOQ"}="3H51 HOQ"; $resn_exch{"H521 HOQ"}="1H52 HOQ";
$resn_exch{"H522 HOQ"}="2H52 HOQ"; $resn_exch{"H523 HOQ"}="3H52 HOQ"; $resn_exch{" H91 HOQ"}="1H9  HOQ";
$resn_exch{" H92 HOQ"}="2H9  HOQ"; $resn_exch{" H93 HOQ"}="3H9  HOQ"; $resn_exch{"H101 HOQ"}="1H10 HOQ";
$resn_exch{"H102 HOQ"}="2H10 HOQ"; $resn_exch{"H103 HOQ"}="3H10 HOQ"; $resn_exch{" HN1 HOZ"}="1HN  HOZ";
$resn_exch{" HN2 HOZ"}="2HN  HOZ"; $resn_exch{" H1  HOZ"}="1H   HOZ"; $resn_exch{" H2  HOZ"}="2H   HOZ";
$resn_exch{"HN11 HOZ"}="1HN1 HOZ"; $resn_exch{"HN12 HOZ"}="2HN1 HOZ"; $resn_exch{" HB1 HOZ"}="1HB  HOZ";
$resn_exch{" HB2 HOZ"}="2HB  HOZ"; $resn_exch{" HG1 HOZ"}="1HG  HOZ"; $resn_exch{" HG2 HOZ"}="2HG  HOZ";
$resn_exch{" H31 HP1"}="1H3  HP1"; $resn_exch{" H32 HP1"}="2H3  HP1"; $resn_exch{" H33 HP1"}="3H3  HP1";
$resn_exch{" H91 HP1"}="1H9  HP1"; $resn_exch{" H92 HP1"}="2H9  HP1"; $resn_exch{" H93 HP1"}="3H9  HP1";
$resn_exch{"H151 HP1"}="1H15 HP1"; $resn_exch{"H152 HP1"}="2H15 HP1"; $resn_exch{"H153 HP1"}="3H15 HP1";
$resn_exch{"H181 HP1"}="1H18 HP1"; $resn_exch{"H182 HP1"}="2H18 HP1"; $resn_exch{"H191 HP1"}="1H19 HP1";
$resn_exch{"H192 HP1"}="2H19 HP1"; $resn_exch{"H201 HP1"}="1H20 HP1"; $resn_exch{"H202 HP1"}="2H20 HP1";
$resn_exch{"H211 HP1"}="1H21 HP1"; $resn_exch{"H212 HP1"}="2H21 HP1"; $resn_exch{"H213 HP1"}="3H21 HP1";
$resn_exch{"H221 HP1"}="1H22 HP1"; $resn_exch{"H222 HP1"}="2H22 HP1"; $resn_exch{"H223 HP1"}="3H22 HP1";
$resn_exch{"H281 HP1"}="1H28 HP1"; $resn_exch{"H282 HP1"}="2H28 HP1"; $resn_exch{"H283 HP1"}="3H28 HP1";
$resn_exch{"H291 HP1"}="1H29 HP1"; $resn_exch{"H292 HP1"}="2H29 HP1"; $resn_exch{"H301 HP1"}="1H30 HP1";
$resn_exch{"H302 HP1"}="2H30 HP1"; $resn_exch{" H91 HP2"}="1H9  HP2"; $resn_exch{" H92 HP2"}="2H9  HP2";
$resn_exch{" H93 HP2"}="3H9  HP2"; $resn_exch{"H151 HP2"}="1H15 HP2"; $resn_exch{"H152 HP2"}="2H15 HP2";
$resn_exch{"H153 HP2"}="3H15 HP2"; $resn_exch{"H211 HP2"}="1H21 HP2"; $resn_exch{"H212 HP2"}="2H21 HP2";
$resn_exch{"H213 HP2"}="3H21 HP2"; $resn_exch{"H221 HP2"}="1H22 HP2"; $resn_exch{"H222 HP2"}="2H22 HP2";
$resn_exch{"H223 HP2"}="3H22 HP2"; $resn_exch{"H201 HP2"}="1H20 HP2"; $resn_exch{"H202 HP2"}="2H20 HP2";
$resn_exch{"H181 HP2"}="1H18 HP2"; $resn_exch{"H182 HP2"}="2H18 HP2"; $resn_exch{"H191 HP2"}="1H19 HP2";
$resn_exch{"H192 HP2"}="2H19 HP2"; $resn_exch{" H31 HP2"}="1H3  HP2"; $resn_exch{"H32A HP2"}="2H3  HP2";
$resn_exch{" H33 HP2"}="3H3  HP2"; $resn_exch{"H281 HP2"}="1H28 HP2"; $resn_exch{"H282 HP2"}="2H28 HP2";
$resn_exch{"H283 HP2"}="3H28 HP2"; $resn_exch{"H301 HP2"}="1H30 HP2"; $resn_exch{"H302 HP2"}="2H30 HP2";
$resn_exch{"H291 HP2"}="1H29 HP2"; $resn_exch{"H292 HP2"}="2H29 HP2"; $resn_exch{" H11 HP3"}="1H1  HP3";
$resn_exch{" H12 HP3"}="2H1  HP3"; $resn_exch{" H13 HP3"}="3H1  HP3"; $resn_exch{" H31 HP3"}="1H3  HP3";
$resn_exch{" H32 HP3"}="2H3  HP3"; $resn_exch{"H201 HP6"}="1H20 HP6"; $resn_exch{"H202 HP6"}="2H20 HP6";
$resn_exch{"H203 HP6"}="3H20 HP6"; $resn_exch{"H211 HP6"}="1H21 HP6"; $resn_exch{"H212 HP6"}="2H21 HP6";
$resn_exch{"H221 HP6"}="1H22 HP6"; $resn_exch{"H222 HP6"}="2H22 HP6"; $resn_exch{"H231 HP6"}="1H23 HP6";
$resn_exch{"H232 HP6"}="2H23 HP6"; $resn_exch{"H241 HP6"}="1H24 HP6"; $resn_exch{"H242 HP6"}="2H24 HP6";
$resn_exch{"H251 HP6"}="1H25 HP6"; $resn_exch{"H252 HP6"}="2H25 HP6"; $resn_exch{"H261 HP6"}="1H26 HP6";
$resn_exch{"H262 HP6"}="2H26 HP6"; $resn_exch{"H263 HP6"}="3H26 HP6"; $resn_exch{" H11 HPB"}="1H1  HPB";
$resn_exch{" H12 HPB"}="2H1  HPB"; $resn_exch{" H13 HPB"}="3H1  HPB"; $resn_exch{" H41 HPB"}="1H4  HPB";
$resn_exch{" H42 HPB"}="2H4  HPB"; $resn_exch{"HN31 HPB"}="1HN3 HPB"; $resn_exch{"HN32 HPB"}="2HN3 HPB";
$resn_exch{" HB1 HPC"}="1HB  HPC"; $resn_exch{" HB2 HPC"}="2HB  HPC"; $resn_exch{"HCH1 HPC"}="1HCH HPC";
$resn_exch{"HCH2 HPC"}="2HCH HPC"; $resn_exch{"HCH3 HPC"}="3HCH HPC"; $resn_exch{"HOP2 HPD"}="2HOP HPD";
$resn_exch{"HOP3 HPD"}="3HOP HPD"; $resn_exch{"H5'1 HPD"}="1H5* HPD"; $resn_exch{"H5'2 HPD"}="2H5* HPD";
$resn_exch{"HO'4 HPD"}="4HO* HPD"; $resn_exch{"HO'3 HPD"}="3HO* HPD"; $resn_exch{"H1'1 HPD"}="1H1* HPD";
$resn_exch{"H1'2 HPD"}="2H1* HPD"; $resn_exch{"HO'1 HPD"}="1HO* HPD"; $resn_exch{" H3' HPD"}=" H3* HPD";
$resn_exch{" H4' HPD"}=" H4* HPD"; $resn_exch{"H2'1 HPD"}="1H2* HPD"; $resn_exch{"H2'2 HPD"}="2H2* HPD";
$resn_exch{" H   HPE"}="1HN  HPE"; $resn_exch{" H2  HPE"}="2HN  HPE"; $resn_exch{" HB2 HPE"}="1HB  HPE";
$resn_exch{" HB3 HPE"}="2HB  HPE"; $resn_exch{" HG2 HPE"}="1HG  HPE"; $resn_exch{" HG3 HPE"}="2HG  HPE";
$resn_exch{" HE1 HPE"}="1HE  HPE"; $resn_exch{" HE2 HPE"}="2HE  HPE"; $resn_exch{" HZ1 HPE"}="1HZ  HPE";
$resn_exch{" HZ2 HPE"}="2HZ  HPE"; $resn_exch{" HH  HPE"}="1HH  HPE"; $resn_exch{"H111 HPF"}="1H11 HPF";
$resn_exch{"H112 HPF"}="2H11 HPF"; $resn_exch{" H11 HPG"}="1H1  HPG"; $resn_exch{" H12 HPG"}="2H1  HPG";
$resn_exch{" H21 HPG"}="1H2  HPG"; $resn_exch{" H22 HPG"}="2H2  HPG"; $resn_exch{" H31 HPG"}="1H3  HPG";
$resn_exch{" H32 HPG"}="2H3  HPG"; $resn_exch{" HC  HPH"}=" H   HPH"; $resn_exch{" H   HPH"}="1HN  HPH";
$resn_exch{" H2  HPH"}="2HN  HPH"; $resn_exch{" HB2 HPH"}="1HB  HPH"; $resn_exch{" HB3 HPH"}="2HB  HPH";
$resn_exch{"H071 HPI"}="1H07 HPI"; $resn_exch{"H072 HPI"}="2H07 HPI"; $resn_exch{"H111 HPI"}="1H11 HPI";
$resn_exch{"H112 HPI"}="2H11 HPI"; $resn_exch{"H121 HPI"}="1H12 HPI"; $resn_exch{"H122 HPI"}="2H12 HPI";
$resn_exch{"H281 HPI"}="1H28 HPI"; $resn_exch{"H282 HPI"}="2H28 HPI"; $resn_exch{"H291 HPI"}="1H29 HPI";
$resn_exch{"H292 HPI"}="2H29 HPI"; $resn_exch{"HA51 HPK"}="1HA5 HPK"; $resn_exch{"HA52 HPK"}="2HA5 HPK";
$resn_exch{" H11 HPL"}="1H1  HPL"; $resn_exch{" H12 HPL"}="2H1  HPL"; $resn_exch{" H21 HPL"}="1H2  HPL";
$resn_exch{" H22 HPL"}="2H2  HPL"; $resn_exch{" H31 HPL"}="1H3  HPL"; $resn_exch{" H32 HPL"}="2H3  HPL";
$resn_exch{" H41 HPL"}="1H4  HPL"; $resn_exch{" H42 HPL"}="2H4  HPL"; $resn_exch{" H51 HPL"}="1H5  HPL";
$resn_exch{" H52 HPL"}="2H5  HPL"; $resn_exch{" H61 HPL"}="1H6  HPL"; $resn_exch{" H62 HPL"}="2H6  HPL";
$resn_exch{" H63 HPL"}="3H6  HPL"; $resn_exch{"H121 HPL"}="1H12 HPL"; $resn_exch{"H122 HPL"}="2H12 HPL";
$resn_exch{"H1C1 HPM"}="1H1C HPM"; $resn_exch{"H1C2 HPM"}="2H1C HPM"; $resn_exch{"H2C1 HPM"}="1H2C HPM";
$resn_exch{"H2C2 HPM"}="2H2C HPM"; $resn_exch{"H3C1 HPM"}="1H3C HPM"; $resn_exch{"H3C2 HPM"}="2H3C HPM";
$resn_exch{"H4C1 HPM"}="1H4C HPM"; $resn_exch{"H4C2 HPM"}="2H4C HPM"; $resn_exch{"H5C1 HPM"}="1H5C HPM";
$resn_exch{"H5C2 HPM"}="2H5C HPM"; $resn_exch{"H7C1 HPM"}="1H7C HPM"; $resn_exch{"H7C2 HPM"}="2H7C HPM";
$resn_exch{"H9C1 HPM"}="1H9C HPM"; $resn_exch{"H9C2 HPM"}="2H9C HPM"; $resn_exch{"H381 HPM"}="1H38 HPM";
$resn_exch{"H382 HPM"}="2H38 HPM"; $resn_exch{"H383 HPM"}="3H38 HPM"; $resn_exch{"HN11 HPN"}="1HN1 HPN";
$resn_exch{"HN12 HPN"}="2HN1 HPN"; $resn_exch{" H21 HPN"}="1H2  HPN"; $resn_exch{" H22 HPN"}="2H2  HPN";
$resn_exch{" H31 HPN"}="1H3  HPN"; $resn_exch{" H32 HPN"}="2H3  HPN"; $resn_exch{" H41 HPN"}="1H4  HPN";
$resn_exch{" H42 HPN"}="2H4  HPN"; $resn_exch{" H51 HPN"}="1H5  HPN"; $resn_exch{" H52 HPN"}="2H5  HPN";
$resn_exch{" H61 HPN"}="1H6  HPN"; $resn_exch{" H62 HPN"}="2H6  HPN"; $resn_exch{" H71 HPN"}="1H7  HPN";
$resn_exch{" H72 HPN"}="2H7  HPN"; $resn_exch{" H73 HPN"}="3H7  HPN"; $resn_exch{" H11 HPO"}="1H1  HPO";
$resn_exch{" H12 HPO"}="2H1  HPO"; $resn_exch{" H13 HPO"}="3H1  HPO"; $resn_exch{" H21 HPO"}="1H2  HPO";
$resn_exch{" H22 HPO"}="2H2  HPO"; $resn_exch{" H31 HPO"}="1H3  HPO"; $resn_exch{" H32 HPO"}="2H3  HPO";
$resn_exch{" H41 HPO"}="1H4  HPO"; $resn_exch{" H42 HPO"}="2H4  HPO"; $resn_exch{" H51 HPO"}="1H5  HPO";
$resn_exch{" H52 HPO"}="2H5  HPO"; $resn_exch{" H71 HPO"}="1H7  HPO"; $resn_exch{" H72 HPO"}="2H7  HPO";
$resn_exch{" H71 HPP"}="1H7  HPP"; $resn_exch{" H72 HPP"}="2H7  HPP"; $resn_exch{" H81 HPP"}="1H8  HPP";
$resn_exch{" H82 HPP"}="2H8  HPP"; $resn_exch{" H   HPQ"}="1HN  HPQ"; $resn_exch{" H2  HPQ"}="2HN  HPQ";
$resn_exch{" HB1 HPQ"}="1HB  HPQ"; $resn_exch{" HB2 HPQ"}="2HB  HPQ"; $resn_exch{" HG1 HPQ"}="1HG  HPQ";
$resn_exch{" HG2 HPQ"}="2HG  HPQ"; $resn_exch{" HE1 HPQ"}="1HE  HPQ"; $resn_exch{" HE2 HPQ"}="2HE  HPQ";
$resn_exch{" HZ1 HPQ"}="1HZ  HPQ"; $resn_exch{" HZ2 HPQ"}="2HZ  HPQ"; $resn_exch{" HH1 HPQ"}="1HH  HPQ";
$resn_exch{" HM1 HPQ"}="1HM  HPQ"; $resn_exch{" HM2 HPQ"}="2HM  HPQ"; $resn_exch{" HM3 HPQ"}="3HM  HPQ";
$resn_exch{" H81 HPR"}="1H8  HPR"; $resn_exch{" H82 HPR"}="2H8  HPR"; $resn_exch{" H1' HPR"}=" H1* HPR";
$resn_exch{" H2' HPR"}=" H2* HPR"; $resn_exch{"HO2' HPR"}="*HO2 HPR"; $resn_exch{" H3' HPR"}=" H3* HPR";
$resn_exch{"HO3' HPR"}="*HO3 HPR"; $resn_exch{" H4' HPR"}=" H4* HPR"; $resn_exch{"H5'1 HPR"}="1H5* HPR";
$resn_exch{"H5'2 HPR"}="2H5* HPR"; $resn_exch{"HO5' HPR"}="*HO5 HPR"; $resn_exch{" H43 HPT"}="3H4  HPT";
$resn_exch{" H42 HPT"}="2H4  HPT"; $resn_exch{" H41 HPT"}="1H4  HPT"; $resn_exch{"H112 HPT"}="2H11 HPT";
$resn_exch{"H111 HPT"}="1H11 HPT"; $resn_exch{"H122 HPT"}="2H12 HPT"; $resn_exch{"H121 HPT"}="1H12 HPT";
$resn_exch{"H132 HPT"}="2H13 HPT"; $resn_exch{"H131 HPT"}="1H13 HPT"; $resn_exch{"H111  HQ"}="1H11  HQ";
$resn_exch{"H112  HQ"}="2H11  HQ"; $resn_exch{"H121  HQ"}="1H12  HQ"; $resn_exch{"H122  HQ"}="2H12  HQ";
$resn_exch{"H131  HQ"}="1H13  HQ"; $resn_exch{"H132  HQ"}="2H13  HQ"; $resn_exch{"H141  HQ"}="1H14  HQ";
$resn_exch{"H142  HQ"}="2H14  HQ"; $resn_exch{"H151  HQ"}="1H15  HQ"; $resn_exch{"H152  HQ"}="2H15  HQ";
$resn_exch{"H161  HQ"}="1H16  HQ"; $resn_exch{"H162  HQ"}="2H16  HQ"; $resn_exch{"H171  HQ"}="1H17  HQ";
$resn_exch{"H172  HQ"}="2H17  HQ"; $resn_exch{"H173  HQ"}="3H17  HQ"; $resn_exch{" H91 HQC"}="1H9  HQC";
$resn_exch{" H92 HQC"}="2H9  HQC"; $resn_exch{"H121 HQC"}="1H12 HQC"; $resn_exch{"H122 HQC"}="2H12 HQC";
$resn_exch{"H131 HQC"}="1H13 HQC"; $resn_exch{"H132 HQC"}="2H13 HQC"; $resn_exch{" H61 HQC"}="1H6  HQC";
$resn_exch{" H62 HQC"}="2H6  HQC"; $resn_exch{"H161 HQC"}="1H16 HQC"; $resn_exch{"H162 HQC"}="2H16 HQC";
$resn_exch{" H71 HQL"}="1H7  HQL"; $resn_exch{" H72 HQL"}="2H7  HQL"; $resn_exch{" H81 HQL"}="1H8  HQL";
$resn_exch{" H82 HQL"}="2H8  HQL"; $resn_exch{" H91 HQL"}="1H9  HQL"; $resn_exch{" H92 HQL"}="2H9  HQL";
$resn_exch{"H101 HQL"}="1H10 HQL"; $resn_exch{"H102 HQL"}="2H10 HQL"; $resn_exch{"H111 HQL"}="1H11 HQL";
$resn_exch{"H112 HQL"}="2H11 HQL"; $resn_exch{"H131 HQL"}="1H13 HQL"; $resn_exch{"H132 HQL"}="2H13 HQL";
$resn_exch{"H141 HQL"}="1H14 HQL"; $resn_exch{"H142 HQL"}="2H14 HQL"; $resn_exch{"HO11 HQO"}="1HO1 HQO";
$resn_exch{"H111 HQO"}="1H11 HQO"; $resn_exch{"H112 HQO"}="2H11 HQO"; $resn_exch{"H121 HQO"}="1H12 HQO";
$resn_exch{"H122 HQO"}="2H12 HQO"; $resn_exch{"H131 HQO"}="1H13 HQO"; $resn_exch{"H132 HQO"}="2H13 HQO";
$resn_exch{"H141 HQO"}="1H14 HQO"; $resn_exch{"H142 HQO"}="2H14 HQO"; $resn_exch{"H151 HQO"}="1H15 HQO";
$resn_exch{"H152 HQO"}="2H15 HQO"; $resn_exch{"H161 HQO"}="1H16 HQO"; $resn_exch{"H162 HQO"}="2H16 HQO";
$resn_exch{"H171 HQO"}="1H17 HQO"; $resn_exch{"H172 HQO"}="2H17 HQO"; $resn_exch{"H173 HQO"}="3H17 HQO";
$resn_exch{"HAL1 HQP"}="1HAL HQP"; $resn_exch{"HAL2 HQP"}="2HAL HQP"; $resn_exch{"HAM1 HQP"}="1HAM HQP";
$resn_exch{"HAM2 HQP"}="2HAM HQP"; $resn_exch{" H71 HQQ"}="1H7  HQQ"; $resn_exch{" H72 HQQ"}="2H7  HQQ";
$resn_exch{" H73 HQQ"}="3H7  HQQ"; $resn_exch{" H71 HR1"}="1H7  HR1"; $resn_exch{" H72 HR1"}="2H7  HR1";
$resn_exch{" H81 HR1"}="1H8  HR1"; $resn_exch{" H82 HR1"}="2H8  HR1"; $resn_exch{"H101 HR1"}="1H10 HR1";
$resn_exch{"H102 HR1"}="2H10 HR1"; $resn_exch{"H131 HR1"}="1H13 HR1"; $resn_exch{"H132 HR1"}="2H13 HR1";
$resn_exch{"H133 HR1"}="3H13 HR1"; $resn_exch{"H141 HR1"}="1H14 HR1"; $resn_exch{"H142 HR1"}="2H14 HR1";
$resn_exch{"H143 HR1"}="3H14 HR1"; $resn_exch{"H351 HR1"}="1H35 HR1"; $resn_exch{"H352 HR1"}="2H35 HR1";
$resn_exch{" H61 HR2"}="1H6  HR2"; $resn_exch{" H62 HR2"}="2H6  HR2"; $resn_exch{"H201 HR2"}="1H20 HR2";
$resn_exch{"H202 HR2"}="2H20 HR2"; $resn_exch{"H221 HR2"}="1H22 HR2"; $resn_exch{"H222 HR2"}="2H22 HR2";
$resn_exch{"H241 HR2"}="1H24 HR2"; $resn_exch{"H242 HR2"}="2H24 HR2"; $resn_exch{"H191 HR2"}="1H19 HR2";
$resn_exch{"H192 HR2"}="2H19 HR2"; $resn_exch{"H121 HR2"}="1H12 HR2"; $resn_exch{"H122 HR2"}="2H12 HR2";
$resn_exch{"H123 HR2"}="3H12 HR2"; $resn_exch{"H111 HR2"}="1H11 HR2"; $resn_exch{"H112 HR2"}="2H11 HR2";
$resn_exch{"H113 HR2"}="3H11 HR2"; $resn_exch{"H21A HR2"}="1H2  HR2"; $resn_exch{" H22 HR2"}="2H2  HR2";
$resn_exch{"H161 HR2"}="1H16 HR2"; $resn_exch{"H162 HR2"}="2H16 HR2"; $resn_exch{"HP51 HRB"}="1HP5 HRB";
$resn_exch{"HP52 HRB"}="2HP5 HRB"; $resn_exch{"HP53 HRB"}="3HP5 HRB"; $resn_exch{"HP41 HRB"}="1HP4 HRB";
$resn_exch{"HP42 HRB"}="2HP4 HRB"; $resn_exch{"HP31 HRB"}="1HP3 HRB"; $resn_exch{"HP32 HRB"}="2HP3 HRB";
$resn_exch{"HP21 HRB"}="1HP2 HRB"; $resn_exch{"HP22 HRB"}="2HP2 HRB"; $resn_exch{"HP11 HRB"}="1HP1 HRB";
$resn_exch{"HP12 HRB"}="2HP1 HRB"; $resn_exch{" H31 HRB"}="1H3  HRB"; $resn_exch{" H32 HRB"}="2H3  HRB";
$resn_exch{" HB1 HRB"}="1HB  HRB"; $resn_exch{" HB2 HRB"}="2HB  HRB"; $resn_exch{"HA21 HRB"}="1HA2 HRB";
$resn_exch{"HA22 HRB"}="2HA2 HRB"; $resn_exch{"HA23 HRB"}="3HA2 HRB"; $resn_exch{"HN1A HRG"}="1HN  HRG";
$resn_exch{" HN2 HRG"}="2HN  HRG"; $resn_exch{" HB1 HRG"}="1HB  HRG"; $resn_exch{" HB2 HRG"}="2HB  HRG";
$resn_exch{" HG1 HRG"}="1HG  HRG"; $resn_exch{" HG2 HRG"}="2HG  HRG"; $resn_exch{"HG'1 HRG"}="1HG' HRG";
$resn_exch{"HG'2 HRG"}="2HG' HRG"; $resn_exch{" HD1 HRG"}="1HD  HRG"; $resn_exch{" HD2 HRG"}="2HD  HRG";
$resn_exch{"HN21 HRG"}="1HN2 HRG"; $resn_exch{"HN22 HRG"}="2HN2 HRG"; $resn_exch{"HCH1 HRI"}="1HCH HRI";
$resn_exch{"HCH2 HRI"}="2HCH HRI"; $resn_exch{"HCF1 HRI"}="1HCF HRI"; $resn_exch{"HI11 HRI"}="1HI1 HRI";
$resn_exch{"HI12 HRI"}="2HI1 HRI"; $resn_exch{"HI13 HRI"}="3HI1 HRI"; $resn_exch{"HI21 HRI"}="1HI2 HRI";
$resn_exch{"HI22 HRI"}="2HI2 HRI"; $resn_exch{"HI23 HRI"}="3HI2 HRI"; $resn_exch{"HAB1 HRM"}="1HAB HRM";
$resn_exch{"HAB2 HRM"}="2HAB HRM"; $resn_exch{"HAB3 HRM"}="3HAB HRM"; $resn_exch{"HAA1 HRM"}="1HAA HRM";
$resn_exch{"HAA2 HRM"}="2HAA HRM"; $resn_exch{"HAA3 HRM"}="3HAA HRM"; $resn_exch{" H   HRP"}="1HN  HRP";
$resn_exch{" H2  HRP"}="2HN  HRP"; $resn_exch{" HB2 HRP"}="1HB  HRP"; $resn_exch{" HB3 HRP"}="2HB  HRP";
$resn_exch{" HXT HRP"}=" HO1 HRP"; $resn_exch{" HB1 HSA"}="1HB  HSA"; $resn_exch{" HB2 HSA"}="2HB  HSA";
$resn_exch{" H41 HSA"}="1H4  HSA"; $resn_exch{" H42 HSA"}="2H4  HSA"; $resn_exch{" HO2 HSA"}="2HO  HSA";
$resn_exch{" HO3 HSA"}="3HO  HSA"; $resn_exch{" HN1 HSA"}="1HN  HSA"; $resn_exch{" HN2 HSA"}="2HN  HSA";
$resn_exch{" H31 HSE"}="1H3  HSE"; $resn_exch{" H32 HSE"}="2H3  HSE"; $resn_exch{" H41 HSE"}="1H4  HSE";
$resn_exch{" H42 HSE"}="2H4  HSE"; $resn_exch{" H11 HSF"}="1H1  HSF"; $resn_exch{" H12 HSF"}="2H1  HSF";
$resn_exch{" H21 HSF"}="1H2  HSF"; $resn_exch{" H22 HSF"}="2H2  HSF"; $resn_exch{" H31 HSF"}="1H3  HSF";
$resn_exch{" H32 HSF"}="2H3  HSF"; $resn_exch{" H41 HSF"}="1H4  HSF"; $resn_exch{" H42 HSF"}="2H4  HSF";
$resn_exch{" H51 HSF"}="1H5  HSF"; $resn_exch{" H52 HSF"}="2H5  HSF"; $resn_exch{" H61 HSF"}="1H6  HSF";
$resn_exch{" H62 HSF"}="2H6  HSF"; $resn_exch{" H71 HSF"}="1H7  HSF"; $resn_exch{" H72 HSF"}="2H7  HSF";
$resn_exch{" H81 HSF"}="1H8  HSF"; $resn_exch{" H82 HSF"}="2H8  HSF"; $resn_exch{" H91 HSF"}="1H9  HSF";
$resn_exch{" H92 HSF"}="2H9  HSF"; $resn_exch{"H101 HSF"}="1H10 HSF"; $resn_exch{"H102 HSF"}="2H10 HSF";
$resn_exch{"H111 HSF"}="1H11 HSF"; $resn_exch{"H112 HSF"}="2H11 HSF"; $resn_exch{"H121 HSF"}="1H12 HSF";
$resn_exch{"H122 HSF"}="2H12 HSF"; $resn_exch{"H131 HSF"}="1H13 HSF"; $resn_exch{"H132 HSF"}="2H13 HSF";
$resn_exch{"H141 HSF"}="1H14 HSF"; $resn_exch{"H142 HSF"}="2H14 HSF"; $resn_exch{"H151 HSF"}="1H15 HSF";
$resn_exch{"H152 HSF"}="2H15 HSF"; $resn_exch{"H161 HSF"}="1H16 HSF"; $resn_exch{"H162 HSF"}="2H16 HSF";
$resn_exch{"H163 HSF"}="3H16 HSF"; $resn_exch{"H601 HSI"}="1H60 HSI"; $resn_exch{"H602 HSI"}="2H60 HSI";
$resn_exch{"H603 HSI"}="3H60 HSI"; $resn_exch{"H561 HSI"}="1H56 HSI"; $resn_exch{"H562 HSI"}="2H56 HSI";
$resn_exch{"H521 HSI"}="1H52 HSI"; $resn_exch{"H522 HSI"}="2H52 HSI"; $resn_exch{"H481 HSI"}="1H48 HSI";
$resn_exch{"H482 HSI"}="2H48 HSI"; $resn_exch{"H441 HSI"}="1H44 HSI"; $resn_exch{"H442 HSI"}="2H44 HSI";
$resn_exch{"H401 HSI"}="1H40 HSI"; $resn_exch{"H402 HSI"}="2H40 HSI"; $resn_exch{" H51 HSI"}="1H5  HSI";
$resn_exch{" H52 HSI"}="2H5  HSI"; $resn_exch{" H2  HSL"}=" HN2 HSL"; $resn_exch{" HB2 HSL"}="1HB  HSL";
$resn_exch{" HB3 HSL"}="2HB  HSL"; $resn_exch{" HG2 HSL"}="1HG  HSL"; $resn_exch{" HG3 HSL"}="2HG  HSL";
$resn_exch{" H1  HSM"}="1H   HSM"; $resn_exch{" H2  HSM"}="2H   HSM"; $resn_exch{" HA1 HSM"}="1HA  HSM";
$resn_exch{" HA2 HSM"}="2HA  HSM"; $resn_exch{" HB1 HSM"}="1HB  HSM"; $resn_exch{" HB2 HSM"}="2HB  HSM";
$resn_exch{" HN1 HSO"}="1HN  HSO"; $resn_exch{" HN2 HSO"}="2HN  HSO"; $resn_exch{" HB1 HSO"}="1HB  HSO";
$resn_exch{" HB2 HSO"}="2HB  HSO"; $resn_exch{" H71 HSP"}="1H7  HSP"; $resn_exch{" H72 HSP"}="2H7  HSP";
$resn_exch{" H81 HSP"}="1H8  HSP"; $resn_exch{" H82 HSP"}="2H8  HSP"; $resn_exch{" H91 HSP"}="1H9  HSP";
$resn_exch{" H92 HSP"}="2H9  HSP"; $resn_exch{"H101 HSP"}="1H10 HSP"; $resn_exch{"H102 HSP"}="2H10 HSP";
$resn_exch{" HB1 HSS"}="1HB  HSS"; $resn_exch{" HB2 HSS"}="2HB  HSS"; $resn_exch{" HN1 HSS"}="1HN  HSS";
$resn_exch{" HN2 HSS"}="2HN  HSS"; $resn_exch{"H5'1 HSS"}="1H5* HSS"; $resn_exch{"H5'2 HSS"}="2H5* HSS";
$resn_exch{" H4' HSS"}=" H4* HSS"; $resn_exch{" H3' HSS"}=" H3* HSS"; $resn_exch{"HO3' HSS"}="HO3* HSS";
$resn_exch{" H2' HSS"}=" H2* HSS"; $resn_exch{"HO2' HSS"}="HO2* HSS"; $resn_exch{" H1' HSS"}=" H1* HSS";
$resn_exch{"HN61 HSS"}="1HN6 HSS"; $resn_exch{"HN62 HSS"}="2HN6 HSS"; $resn_exch{" H11 HST"}="1H1  HST";
$resn_exch{" H12 HST"}="2H1  HST"; $resn_exch{" H13 HST"}="3H1  HST"; $resn_exch{" H21 HST"}="1H2  HST";
$resn_exch{" H22 HST"}="2H2  HST"; $resn_exch{" H31 HST"}="1H3  HST"; $resn_exch{" H32 HST"}="2H3  HST";
$resn_exch{" H41 HST"}="1H4  HST"; $resn_exch{" H42 HST"}="2H4  HST"; $resn_exch{" H51 HST"}="1H5  HST";
$resn_exch{" H52 HST"}="2H5  HST"; $resn_exch{" H71 HST"}="1H7  HST"; $resn_exch{" H72 HST"}="2H7  HST";
$resn_exch{" H81 HST"}="1H8  HST"; $resn_exch{" H82 HST"}="2H8  HST"; $resn_exch{" H91 HST"}="1H9  HST";
$resn_exch{" H92 HST"}="2H9  HST"; $resn_exch{"H101 HST"}="1H10 HST"; $resn_exch{"H102 HST"}="2H10 HST";
$resn_exch{"H111 HST"}="1H11 HST"; $resn_exch{"H112 HST"}="2H11 HST"; $resn_exch{"H113 HST"}="3H11 HST";
$resn_exch{"HN11 HSW"}="1HN1 HSW"; $resn_exch{"HN12 HSW"}="2HN1 HSW"; $resn_exch{"H211  HT"}="1H21  HT";
$resn_exch{"H212  HT"}="2H21  HT"; $resn_exch{"H221  HT"}="1H22  HT"; $resn_exch{"H222  HT"}="2H22  HT";
$resn_exch{"H231  HT"}="1H23  HT"; $resn_exch{"H232  HT"}="2H23  HT"; $resn_exch{"H241  HT"}="1H24  HT";
$resn_exch{"H242  HT"}="2H24  HT"; $resn_exch{"H253  HT"}="3H25  HT"; $resn_exch{"H252  HT"}="2H25  HT";
$resn_exch{"H251  HT"}="1H25  HT"; $resn_exch{"H211 HT1"}="1H21 HT1"; $resn_exch{"H212 HT1"}="2H21 HT1";
$resn_exch{"H221 HT1"}="1H22 HT1"; $resn_exch{"H222 HT1"}="2H22 HT1"; $resn_exch{"H231 HT1"}="1H23 HT1";
$resn_exch{"H232 HT1"}="2H23 HT1"; $resn_exch{"H241 HT1"}="1H24 HT1"; $resn_exch{"H242 HT1"}="2H24 HT1";
$resn_exch{"H251 HT1"}="1H25 HT1"; $resn_exch{"H252 HT1"}="2H25 HT1"; $resn_exch{"H253 HT1"}="3H25 HT1";
$resn_exch{"H261 HT1"}="1H26 HT1"; $resn_exch{"H262 HT1"}="2H26 HT1"; $resn_exch{"H271 HT1"}="1H27 HT1";
$resn_exch{"H272 HT1"}="2H27 HT1"; $resn_exch{"H273 HT1"}="3H27 HT1"; $resn_exch{"HN11 HT2"}="1HN1 HT2";
$resn_exch{"HN12 HT2"}="2HN1 HT2"; $resn_exch{"HN71 HT2"}="1HN7 HT2"; $resn_exch{"HN72 HT2"}="2HN7 HT2";
$resn_exch{"H141 HT5"}="1H14 HT5"; $resn_exch{"H142 HT5"}="2H14 HT5"; $resn_exch{"H131 HT5"}="1H13 HT5";
$resn_exch{"H132 HT5"}="2H13 HT5"; $resn_exch{"H9C1 HT5"}="1H9C HT5"; $resn_exch{"H9C2 HT5"}="2H9C HT5";
$resn_exch{"H2C1 HT5"}="1H2C HT5"; $resn_exch{"H2C2 HT5"}="2H2C HT5"; $resn_exch{"H1C1 HT5"}="1H1C HT5";
$resn_exch{"H1C2 HT5"}="2H1C HT5"; $resn_exch{"H501 HT5"}="1H50 HT5"; $resn_exch{"H502 HT5"}="2H50 HT5";
$resn_exch{"H3C1 HT5"}="1H3C HT5"; $resn_exch{"H3C2 HT5"}="2H3C HT5"; $resn_exch{"H4C1 HT5"}="1H4C HT5";
$resn_exch{"H4C2 HT5"}="2H4C HT5"; $resn_exch{"H5C1 HT5"}="1H5C HT5"; $resn_exch{"H5C2 HT5"}="2H5C HT5";
$resn_exch{"H6C1 HT5"}="1H6C HT5"; $resn_exch{"H6C2 HT5"}="2H6C HT5"; $resn_exch{"H8C1 HT5"}="1H8C HT5";
$resn_exch{"H8C2 HT5"}="2H8C HT5"; $resn_exch{"H8C3 HT5"}="3H8C HT5"; $resn_exch{" H21 HTA"}="1H2  HTA";
$resn_exch{" H22 HTA"}="2H2  HTA"; $resn_exch{" H51 HTA"}="1H5  HTA"; $resn_exch{" H52 HTA"}="2H5  HTA";
$resn_exch{" H71 HTA"}="1H7  HTA"; $resn_exch{" H72 HTA"}="2H7  HTA"; $resn_exch{" H73 HTA"}="3H7  HTA";
$resn_exch{" H81 HTA"}="1H8  HTA"; $resn_exch{" H82 HTA"}="2H8  HTA"; $resn_exch{" H83 HTA"}="3H8  HTA";
$resn_exch{"H111 HTA"}="1H11 HTA"; $resn_exch{"H112 HTA"}="2H11 HTA"; $resn_exch{"H181 HTA"}="1H18 HTA";
$resn_exch{"H182 HTA"}="2H18 HTA"; $resn_exch{"H183 HTA"}="3H18 HTA"; $resn_exch{"H191 HTA"}="1H19 HTA";
$resn_exch{"H192 HTA"}="2H19 HTA"; $resn_exch{"H193 HTA"}="3H19 HTA"; $resn_exch{" H81 HTF"}="1H8  HTF";
$resn_exch{" H82 HTF"}="2H8  HTF"; $resn_exch{" H83 HTF"}="3H8  HTF"; $resn_exch{" H72 HTF"}="2H7  HTF";
$resn_exch{" H73 HTF"}="3H7  HTF"; $resn_exch{" H62 HTF"}="2H6  HTF"; $resn_exch{" H63 HTF"}="3H6  HTF";
$resn_exch{" H52 HTF"}="2H5  HTF"; $resn_exch{" H53 HTF"}="3H5  HTF"; $resn_exch{" H42 HTF"}="2H4  HTF";
$resn_exch{" H43 HTF"}="3H4  HTF"; $resn_exch{" H32 HTF"}="2H3  HTF"; $resn_exch{" H33 HTF"}="3H3  HTF";
$resn_exch{" H22 HTF"}="2H2  HTF"; $resn_exch{" H23 HTF"}="3H2  HTF"; $resn_exch{" HG2 HTF"}="2HG  HTF";
$resn_exch{" HG3 HTF"}="3HG  HTF"; $resn_exch{" HB2 HTF"}="2HB  HTF"; $resn_exch{" HB3 HTF"}="3HB  HTF";
$resn_exch{" H61 HTG"}="1H6  HTG"; $resn_exch{" H62 HTG"}="2H6  HTG"; $resn_exch{"H1'1 HTG"}="1H1' HTG";
$resn_exch{"H1'2 HTG"}="2H1' HTG"; $resn_exch{"H2'1 HTG"}="1H2' HTG"; $resn_exch{"H2'2 HTG"}="2H2' HTG";
$resn_exch{"H3'1 HTG"}="1H3' HTG"; $resn_exch{"H3'2 HTG"}="2H3' HTG"; $resn_exch{"H4'1 HTG"}="1H4' HTG";
$resn_exch{"H4'2 HTG"}="2H4' HTG"; $resn_exch{"H5'1 HTG"}="1H5' HTG"; $resn_exch{"H5'2 HTG"}="2H5' HTG";
$resn_exch{"H6'1 HTG"}="1H6' HTG"; $resn_exch{"H6'2 HTG"}="2H6' HTG"; $resn_exch{"H7'1 HTG"}="1H7' HTG";
$resn_exch{"H7'2 HTG"}="2H7' HTG"; $resn_exch{"H7'3 HTG"}="3H7' HTG"; $resn_exch{" H   HTI"}="1HN  HTI";
$resn_exch{" H2  HTI"}="2HN  HTI"; $resn_exch{" HB2 HTI"}="1HB  HTI"; $resn_exch{" HB3 HTI"}="2HB  HTI";
$resn_exch{"HAJ1 HTI"}="1HAJ HTI"; $resn_exch{"HAJ2 HTI"}="2HAJ HTI"; $resn_exch{"HAL1 HTI"}="1HAL HTI";
$resn_exch{"HAL2 HTI"}="2HAL HTI"; $resn_exch{"H2A1 HTL"}="1H2A HTL"; $resn_exch{"H2A2 HTL"}="2H2A HTL";
$resn_exch{"H2A3 HTL"}="3H2A HTL"; $resn_exch{"H4'1 HTL"}="1H4* HTL"; $resn_exch{"H4'2 HTL"}="2H4* HTL";
$resn_exch{" H6' HTL"}=" H6* HTL"; $resn_exch{"H351 HTL"}="1H35 HTL"; $resn_exch{"H352 HTL"}="2H35 HTL";
$resn_exch{"H4A1 HTL"}="1H4A HTL"; $resn_exch{"H4A2 HTL"}="2H4A HTL"; $resn_exch{"H4A3 HTL"}="3H4A HTL";
$resn_exch{"H5A1 HTL"}="1H5A HTL"; $resn_exch{"H5A2 HTL"}="2H5A HTL"; $resn_exch{"H5B1 HTL"}="1H5B HTL";
$resn_exch{"H5B2 HTL"}="2H5B HTL"; $resn_exch{"H22O HTL"}="OH22 HTL"; $resn_exch{"H23O HTL"}="OH23 HTL";
$resn_exch{"H3'1 HTL"}="1H3' HTL"; $resn_exch{"H3'2 HTL"}="2H3' HTL"; $resn_exch{"H3'3 HTL"}="3H3' HTL";
$resn_exch{" H71 HTM"}="1H7  HTM"; $resn_exch{" H72 HTM"}="2H7  HTM"; $resn_exch{" H81 HTM"}="1H8  HTM";
$resn_exch{" H82 HTM"}="2H8  HTM"; $resn_exch{" H91 HTM"}="1H9  HTM"; $resn_exch{" H92 HTM"}="2H9  HTM";
$resn_exch{"H111 HTM"}="1H11 HTM"; $resn_exch{"H112 HTM"}="2H11 HTM"; $resn_exch{"H113 HTM"}="3H11 HTM";
$resn_exch{" H11 HTO"}="1H1  HTO"; $resn_exch{" H12 HTO"}="2H1  HTO"; $resn_exch{" H41 HTO"}="1H4  HTO";
$resn_exch{" H42 HTO"}="2H4  HTO"; $resn_exch{" H51 HTO"}="1H5  HTO"; $resn_exch{" H52 HTO"}="2H5  HTO";
$resn_exch{" H61 HTO"}="1H6  HTO"; $resn_exch{" H62 HTO"}="2H6  HTO"; $resn_exch{" H71 HTO"}="1H7  HTO";
$resn_exch{" H72 HTO"}="2H7  HTO"; $resn_exch{" H73 HTO"}="3H7  HTO"; $resn_exch{" H61 HTP"}="1H6  HTP";
$resn_exch{" H62 HTP"}="2H6  HTP"; $resn_exch{" H11 HTQ"}="1H1  HTQ"; $resn_exch{" H21 HTQ"}="1H2  HTQ";
$resn_exch{" H22 HTQ"}="2H2  HTQ"; $resn_exch{" H31 HTQ"}="1H3  HTQ"; $resn_exch{" H41 HTQ"}="1H4  HTQ";
$resn_exch{" H42 HTQ"}="2H4  HTQ"; $resn_exch{" H51 HTQ"}="1H5  HTQ"; $resn_exch{" H61 HTQ"}="1H6  HTQ";
$resn_exch{" H62 HTQ"}="2H6  HTQ"; $resn_exch{" H71 HTQ"}="1H7  HTQ"; $resn_exch{" H72 HTQ"}="2H7  HTQ";
$resn_exch{" H91 HTQ"}="1H9  HTQ"; $resn_exch{" H92 HTQ"}="2H9  HTQ"; $resn_exch{" H93 HTQ"}="3H9  HTQ";
$resn_exch{"H131 HTQ"}="1H13 HTQ"; $resn_exch{"H151 HTQ"}="1H15 HTQ"; $resn_exch{"H161 HTQ"}="1H16 HTQ";
$resn_exch{"H171 HTQ"}="1H17 HTQ"; $resn_exch{"H181 HTQ"}="1H18 HTQ"; $resn_exch{"H191 HTQ"}="1H19 HTQ";
$resn_exch{"H201 HTQ"}="1H20 HTQ"; $resn_exch{" H   HTR"}="1HN  HTR"; $resn_exch{" H2  HTR"}="2HN  HTR";
$resn_exch{" HH  HTR"}=" HO  HTR"; $resn_exch{" H51 HU1"}="1H5  HU1"; $resn_exch{" H52 HU1"}="2H5  HU1";
$resn_exch{" H53 HU1"}="3H5  HU1"; $resn_exch{" H61 HU1"}="1H6  HU1"; $resn_exch{" H62 HU1"}="2H6  HU1";
$resn_exch{" H63 HU1"}="3H6  HU1"; $resn_exch{" H71 HU1"}="1H7  HU1"; $resn_exch{" H72 HU1"}="2H7  HU1";
$resn_exch{" H73 HU1"}="3H7  HU1"; $resn_exch{"H111 HU1"}="1H11 HU1"; $resn_exch{"H112 HU1"}="2H11 HU1";
$resn_exch{"H121 HU1"}="1H12 HU1"; $resn_exch{"H122 HU1"}="2H12 HU1"; $resn_exch{"H131 HU1"}="1H13 HU1";
$resn_exch{"H132 HU1"}="2H13 HU1"; $resn_exch{"H141 HU1"}="1H14 HU1"; $resn_exch{"H142 HU1"}="2H14 HU1";
$resn_exch{"H151 HU1"}="1H15 HU1"; $resn_exch{"H152 HU1"}="2H15 HU1"; $resn_exch{"H191 HU1"}="1H19 HU1";
$resn_exch{"H192 HU1"}="2H19 HU1"; $resn_exch{"H241 HU1"}="1H24 HU1"; $resn_exch{"H242 HU1"}="2H24 HU1";
$resn_exch{"H243 HU1"}="3H24 HU1"; $resn_exch{"H251 HU1"}="1H25 HU1"; $resn_exch{"H252 HU1"}="2H25 HU1";
$resn_exch{"H253 HU1"}="3H25 HU1"; $resn_exch{"H301 HU1"}="1H30 HU1"; $resn_exch{"H302 HU1"}="2H30 HU1";
$resn_exch{"H321 HU1"}="1H32 HU1"; $resn_exch{"H322 HU1"}="2H32 HU1"; $resn_exch{"H331 HU1"}="1H33 HU1";
$resn_exch{"H332 HU1"}="2H33 HU1"; $resn_exch{"H341 HU1"}="1H34 HU1"; $resn_exch{"H342 HU1"}="2H34 HU1";
$resn_exch{"H381 HU1"}="1H38 HU1"; $resn_exch{"H382 HU1"}="2H38 HU1"; $resn_exch{" H31 HU2"}="1H3  HU2";
$resn_exch{" H32 HU2"}="2H3  HU2"; $resn_exch{"H41A HU2"}="1H4  HU2"; $resn_exch{" H51 HU2"}="1H5  HU2";
$resn_exch{" H61 HU2"}="1H6  HU2"; $resn_exch{" H62 HU2"}="2H6  HU2"; $resn_exch{" H81 HU2"}="1H8  HU2";
$resn_exch{" H82 HU2"}="2H8  HU2"; $resn_exch{" H83 HU2"}="3H8  HU2"; $resn_exch{" H91 HU2"}="1H9  HU2";
$resn_exch{" H92 HU2"}="2H9  HU2"; $resn_exch{" H93 HU2"}="3H9  HU2"; $resn_exch{"H111 HU2"}="1H11 HU2";
$resn_exch{"H112 HU2"}="2H11 HU2"; $resn_exch{"H113 HU2"}="3H11 HU2"; $resn_exch{"H121 HU2"}="1H12 HU2";
$resn_exch{"H122 HU2"}="2H12 HU2"; $resn_exch{"H123 HU2"}="3H12 HU2"; $resn_exch{"H161 HU2"}="1H16 HU2";
$resn_exch{"H162 HU2"}="2H16 HU2"; $resn_exch{"H181 HU2"}="1H18 HU2"; $resn_exch{"H182 HU2"}="2H18 HU2";
$resn_exch{"H183 HU2"}="3H18 HU2"; $resn_exch{"H191 HU2"}="1H19 HU2"; $resn_exch{"H192 HU2"}="2H19 HU2";
$resn_exch{"H193 HU2"}="3H19 HU2"; $resn_exch{"H241 HU2"}="1H24 HU2"; $resn_exch{"H242 HU2"}="2H24 HU2";
$resn_exch{"H251 HU2"}="1H25 HU2"; $resn_exch{"H252 HU2"}="2H25 HU2"; $resn_exch{"H261 HU2"}="1H26 HU2";
$resn_exch{"H262 HU2"}="2H26 HU2"; $resn_exch{"H263 HU2"}="3H26 HU2"; $resn_exch{"H311 HU2"}="1H31 HU2";
$resn_exch{"H312 HU2"}="2H31 HU2"; $resn_exch{" H51 HU4"}="1H5  HU4"; $resn_exch{" H52 HU4"}="2H5  HU4";
$resn_exch{" H53 HU4"}="3H5  HU4"; $resn_exch{" H61 HU4"}="1H6  HU4"; $resn_exch{" H62 HU4"}="2H6  HU4";
$resn_exch{" H63 HU4"}="3H6  HU4"; $resn_exch{" H71 HU4"}="1H7  HU4"; $resn_exch{" H72 HU4"}="2H7  HU4";
$resn_exch{" H73 HU4"}="3H7  HU4"; $resn_exch{"H111 HU4"}="1H11 HU4"; $resn_exch{"H112 HU4"}="2H11 HU4";
$resn_exch{"H121 HU4"}="1H12 HU4"; $resn_exch{"H122 HU4"}="2H12 HU4"; $resn_exch{"H131 HU4"}="1H13 HU4";
$resn_exch{"H132 HU4"}="2H13 HU4"; $resn_exch{"H141 HU4"}="1H14 HU4"; $resn_exch{"H142 HU4"}="2H14 HU4";
$resn_exch{"H151 HU4"}="1H15 HU4"; $resn_exch{"H152 HU4"}="2H15 HU4"; $resn_exch{"H191 HU4"}="1H19 HU4";
$resn_exch{"H192 HU4"}="2H19 HU4"; $resn_exch{"H251 HU4"}="1H25 HU4"; $resn_exch{"H252 HU4"}="2H25 HU4";
$resn_exch{"H253 HU4"}="3H25 HU4"; $resn_exch{"H261 HU4"}="1H26 HU4"; $resn_exch{"H262 HU4"}="2H26 HU4";
$resn_exch{"H263 HU4"}="3H26 HU4"; $resn_exch{"H311 HU4"}="1H31 HU4"; $resn_exch{"H312 HU4"}="2H31 HU4";
$resn_exch{"H331 HU4"}="1H33 HU4"; $resn_exch{"H332 HU4"}="2H33 HU4"; $resn_exch{"H341 HU4"}="1H34 HU4";
$resn_exch{"H342 HU4"}="2H34 HU4"; $resn_exch{"H351 HU4"}="1H35 HU4"; $resn_exch{"H352 HU4"}="2H35 HU4";
$resn_exch{"H391 HU4"}="1H39 HU4"; $resn_exch{"H392 HU4"}="2H39 HU4"; $resn_exch{" H51 HU5"}="1H5  HU5";
$resn_exch{" H52 HU5"}="2H5  HU5"; $resn_exch{" H53 HU5"}="3H5  HU5"; $resn_exch{" H61 HU5"}="1H6  HU5";
$resn_exch{" H62 HU5"}="2H6  HU5"; $resn_exch{" H63 HU5"}="3H6  HU5"; $resn_exch{" H71 HU5"}="1H7  HU5";
$resn_exch{" H72 HU5"}="2H7  HU5"; $resn_exch{" H73 HU5"}="3H7  HU5"; $resn_exch{"H111 HU5"}="1H11 HU5";
$resn_exch{"H112 HU5"}="2H11 HU5"; $resn_exch{"H113 HU5"}="3H11 HU5"; $resn_exch{"H121 HU5"}="1H12 HU5";
$resn_exch{"H122 HU5"}="2H12 HU5"; $resn_exch{"H123 HU5"}="3H12 HU5"; $resn_exch{"H131 HU5"}="1H13 HU5";
$resn_exch{"H132 HU5"}="2H13 HU5"; $resn_exch{"H133 HU5"}="3H13 HU5"; $resn_exch{"H171 HU5"}="1H17 HU5";
$resn_exch{"H172 HU5"}="2H17 HU5"; $resn_exch{"H221 HU5"}="1H22 HU5"; $resn_exch{"H222 HU5"}="2H22 HU5";
$resn_exch{"H223 HU5"}="3H22 HU5"; $resn_exch{"H231 HU5"}="1H23 HU5"; $resn_exch{"H232 HU5"}="2H23 HU5";
$resn_exch{"H233 HU5"}="3H23 HU5"; $resn_exch{"H281 HU5"}="1H28 HU5"; $resn_exch{"H282 HU5"}="2H28 HU5";
$resn_exch{"H301 HU5"}="1H30 HU5"; $resn_exch{"H302 HU5"}="2H30 HU5"; $resn_exch{"H311 HU5"}="1H31 HU5";
$resn_exch{"H312 HU5"}="2H31 HU5"; $resn_exch{"H321 HU5"}="1H32 HU5"; $resn_exch{"H322 HU5"}="2H32 HU5";
$resn_exch{"H361 HU5"}="1H36 HU5"; $resn_exch{"H362 HU5"}="2H36 HU5"; $resn_exch{"H6C1 HUB"}="1H6C HUB";
$resn_exch{"H6C2 HUB"}="2H6C HUB"; $resn_exch{"H9C1 HUB"}="1H9C HUB"; $resn_exch{"H9C2 HUB"}="2H9C HUB";
$resn_exch{"H101 HUB"}="1H10 HUB"; $resn_exch{"H102 HUB"}="2H10 HUB"; $resn_exch{"H131 HUB"}="1H13 HUB";
$resn_exch{"H132 HUB"}="2H13 HUB"; $resn_exch{"H151 HUB"}="1H15 HUB"; $resn_exch{"H152 HUB"}="2H15 HUB";
$resn_exch{"H153 HUB"}="3H15 HUB"; $resn_exch{"H901 HUB"}="1H90 HUB"; $resn_exch{"H902 HUB"}="2H90 HUB";
$resn_exch{" H51 HUD"}="1H5  HUD"; $resn_exch{" H52 HUD"}="2H5  HUD"; $resn_exch{" H53 HUD"}="3H5  HUD";
$resn_exch{" H61 HUD"}="1H6  HUD"; $resn_exch{" H62 HUD"}="2H6  HUD"; $resn_exch{" H63 HUD"}="3H6  HUD";
$resn_exch{" H71 HUD"}="1H7  HUD"; $resn_exch{" H72 HUD"}="2H7  HUD"; $resn_exch{" H73 HUD"}="3H7  HUD";
$resn_exch{"H111 HUD"}="1H11 HUD"; $resn_exch{"H112 HUD"}="2H11 HUD"; $resn_exch{"H113 HUD"}="3H11 HUD";
$resn_exch{"H121 HUD"}="1H12 HUD"; $resn_exch{"H122 HUD"}="2H12 HUD"; $resn_exch{"H123 HUD"}="3H12 HUD";
$resn_exch{"H131 HUD"}="1H13 HUD"; $resn_exch{"H132 HUD"}="2H13 HUD"; $resn_exch{"H133 HUD"}="3H13 HUD";
$resn_exch{"H171 HUD"}="1H17 HUD"; $resn_exch{"H172 HUD"}="2H17 HUD"; $resn_exch{"H221 HUD"}="1H22 HUD";
$resn_exch{"H222 HUD"}="2H22 HUD"; $resn_exch{"H223 HUD"}="3H22 HUD"; $resn_exch{"H231 HUD"}="1H23 HUD";
$resn_exch{"H232 HUD"}="2H23 HUD"; $resn_exch{"H233 HUD"}="3H23 HUD"; $resn_exch{"H281 HUD"}="1H28 HUD";
$resn_exch{"H282 HUD"}="2H28 HUD"; $resn_exch{"H301 HUD"}="1H30 HUD"; $resn_exch{"H302 HUD"}="2H30 HUD";
$resn_exch{"H311 HUD"}="1H31 HUD"; $resn_exch{"H312 HUD"}="2H31 HUD"; $resn_exch{"H351 HUD"}="1H35 HUD";
$resn_exch{"H352 HUD"}="2H35 HUD"; $resn_exch{" H61 HUP"}="1H6  HUP"; $resn_exch{" H62 HUP"}="2H6  HUP";
$resn_exch{" H81 HUP"}="1H8  HUP"; $resn_exch{" H82 HUP"}="2H8  HUP"; $resn_exch{" H91 HUP"}="1H9  HUP";
$resn_exch{" H92 HUP"}="2H9  HUP"; $resn_exch{" H93 HUP"}="3H9  HUP"; $resn_exch{"HN21 HUP"}="1HN2 HUP";
$resn_exch{"HN22 HUP"}="2HN2 HUP"; $resn_exch{"H131 HUP"}="1H13 HUP"; $resn_exch{"H132 HUP"}="2H13 HUP";
$resn_exch{"H151 HUP"}="1H15 HUP"; $resn_exch{"H152 HUP"}="2H15 HUP"; $resn_exch{"H153 HUP"}="3H15 HUP";
$resn_exch{"HN21 HUX"}="1HN2 HUX"; $resn_exch{"HN22 HUX"}="2HN2 HUX"; $resn_exch{"HC51 HUX"}="1HC5 HUX";
$resn_exch{"HC52 HUX"}="2HC5 HUX"; $resn_exch{"H121 HUX"}="1H12 HUX"; $resn_exch{"H122 HUX"}="2H12 HUX";
$resn_exch{"HC91 HUX"}="1HC9 HUX"; $resn_exch{"HC92 HUX"}="2HC9 HUX"; $resn_exch{"H181 HUX"}="1H18 HUX";
$resn_exch{"H182 HUX"}="2H18 HUX"; $resn_exch{"H101 HUX"}="1H10 HUX"; $resn_exch{"H102 HUX"}="2H10 HUX";
$resn_exch{"H103 HUX"}="3H10 HUX"; $resn_exch{" HN1 HV5"}="1HN  HV5"; $resn_exch{" HN2 HV5"}="2HN  HV5";
$resn_exch{"HG11 HV5"}="1HG1 HV5"; $resn_exch{"HG12 HV5"}="2HG1 HV5"; $resn_exch{"HG13 HV5"}="3HG1 HV5";
$resn_exch{"HG21 HV5"}="1HG2 HV5"; $resn_exch{"HG22 HV5"}="2HG2 HV5"; $resn_exch{"HG23 HV5"}="3HG2 HV5";
$resn_exch{"HG31 HV5"}="1HG3 HV5"; $resn_exch{"HG32 HV5"}="2HG3 HV5"; $resn_exch{"HG33 HV5"}="3HG3 HV5";
$resn_exch{" HN1 HV7"}="1HN  HV7"; $resn_exch{" HN2 HV7"}="2HN  HV7"; $resn_exch{" HB1 HV7"}="1HB  HV7";
$resn_exch{" HB2 HV7"}="2HB  HV7"; $resn_exch{" HM1 HV7"}="1HM  HV7"; $resn_exch{" HM2 HV7"}="2HM  HV7";
$resn_exch{" HM3 HV7"}="3HM  HV7"; $resn_exch{" HN1 HV8"}="1HN  HV8"; $resn_exch{"HN2A HV8"}="2HN  HV8";
$resn_exch{"HB11 HV8"}="1HB1 HV8"; $resn_exch{"HB12 HV8"}="2HB1 HV8"; $resn_exch{"HB21 HV8"}="1HB2 HV8";
$resn_exch{"HB22 HV8"}="2HB2 HV8"; $resn_exch{"HM21 HV8"}="1HM2 HV8"; $resn_exch{"HM22 HV8"}="2HM2 HV8";
$resn_exch{"HM23 HV8"}="3HM2 HV8"; $resn_exch{" H71 HWD"}="1H7  HWD"; $resn_exch{" H72 HWD"}="2H7  HWD";
$resn_exch{" H81 HWD"}="1H8  HWD"; $resn_exch{" H82 HWD"}="2H8  HWD"; $resn_exch{"H92O HWD"}="OH92 HWD";
$resn_exch{"H141 HWG"}="1H14 HWG"; $resn_exch{"H142 HWG"}="2H14 HWG"; $resn_exch{"H151 HWG"}="1H15 HWG";
$resn_exch{"H152 HWG"}="2H15 HWG"; $resn_exch{"H261 HWG"}="1H26 HWG"; $resn_exch{"H262 HWG"}="2H26 HWG";
$resn_exch{"H263 HWG"}="3H26 HWG"; $resn_exch{"H301 HWG"}="1H30 HWG"; $resn_exch{"H302 HWG"}="2H30 HWG";
$resn_exch{"H303 HWG"}="3H30 HWG"; $resn_exch{"H341 HWG"}="1H34 HWG"; $resn_exch{"H342 HWG"}="2H34 HWG";
$resn_exch{"H343 HWG"}="3H34 HWG"; $resn_exch{"H461 HWG"}="1H46 HWG"; $resn_exch{"H462 HWG"}="2H46 HWG";
$resn_exch{"H463 HWG"}="3H46 HWG"; $resn_exch{"H501 HWG"}="1H50 HWG"; $resn_exch{"H502 HWG"}="2H50 HWG";
$resn_exch{"H503 HWG"}="3H50 HWG"; $resn_exch{"H541 HWG"}="1H54 HWG"; $resn_exch{"H542 HWG"}="2H54 HWG";
$resn_exch{"H543 HWG"}="3H54 HWG"; $resn_exch{" H21 HXA"}="1H2  HXA"; $resn_exch{" H22 HXA"}="2H2  HXA";
$resn_exch{" H31 HXA"}="1H3  HXA"; $resn_exch{" H32 HXA"}="2H3  HXA"; $resn_exch{" H61 HXA"}="1H6  HXA";
$resn_exch{" H62 HXA"}="2H6  HXA"; $resn_exch{" H91 HXA"}="1H9  HXA"; $resn_exch{" H92 HXA"}="2H9  HXA";
$resn_exch{"H121 HXA"}="1H12 HXA"; $resn_exch{"H122 HXA"}="2H12 HXA"; $resn_exch{"H151 HXA"}="1H15 HXA";
$resn_exch{"H152 HXA"}="2H15 HXA"; $resn_exch{"H181 HXA"}="1H18 HXA"; $resn_exch{"H182 HXA"}="2H18 HXA";
$resn_exch{"H211 HXA"}="1H21 HXA"; $resn_exch{"H212 HXA"}="2H21 HXA"; $resn_exch{"H221 HXA"}="1H22 HXA";
$resn_exch{"H222 HXA"}="2H22 HXA"; $resn_exch{"H223 HXA"}="3H22 HXA"; $resn_exch{"HN61 HXC"}="1HN6 HXC";
$resn_exch{"HN62 HXC"}="2HN6 HXC"; $resn_exch{"H5'1 HXC"}="1H5' HXC"; $resn_exch{"H5'2 HXC"}="2H5' HXC";
$resn_exch{"HPB1 HXC"}="1HPB HXC"; $resn_exch{"HPB2 HXC"}="2HPB HXC"; $resn_exch{"HP91 HXC"}="1HP9 HXC";
$resn_exch{"HP92 HXC"}="2HP9 HXC"; $resn_exch{"HP93 HXC"}="3HP9 HXC"; $resn_exch{"HP81 HXC"}="1HP8 HXC";
$resn_exch{"HP82 HXC"}="2HP8 HXC"; $resn_exch{"HP83 HXC"}="3HP8 HXC"; $resn_exch{"HP51 HXC"}="1HP5 HXC";
$resn_exch{"HP52 HXC"}="2HP5 HXC"; $resn_exch{"HP41 HXC"}="1HP4 HXC"; $resn_exch{"HP42 HXC"}="2HP4 HXC";
$resn_exch{"HP21 HXC"}="1HP2 HXC"; $resn_exch{"HP22 HXC"}="2HP2 HXC"; $resn_exch{"HP11 HXC"}="1HP1 HXC";
$resn_exch{"HP12 HXC"}="2HP1 HXC"; $resn_exch{"HM21 HXC"}="1HM2 HXC"; $resn_exch{"HM22 HXC"}="2HM2 HXC";
$resn_exch{"HM31 HXC"}="1HM3 HXC"; $resn_exch{"HM32 HXC"}="2HM3 HXC"; $resn_exch{"HM41 HXC"}="1HM4 HXC";
$resn_exch{"HM42 HXC"}="2HM4 HXC"; $resn_exch{"HM51 HXC"}="1HM5 HXC"; $resn_exch{"HM52 HXC"}="2HM5 HXC";
$resn_exch{"HM61 HXC"}="1HM6 HXC"; $resn_exch{"HM62 HXC"}="2HM6 HXC"; $resn_exch{"HM63 HXC"}="3HM6 HXC";
$resn_exch{"H111 HXD"}="1H11 HXD"; $resn_exch{"H112 HXD"}="2H11 HXD"; $resn_exch{"H101 HXD"}="1H10 HXD";
$resn_exch{"H102 HXD"}="2H10 HXD"; $resn_exch{"H121 HXD"}="1H12 HXD"; $resn_exch{"H122 HXD"}="2H12 HXD";
$resn_exch{"H123 HXD"}="3H12 HXD"; $resn_exch{" H91 HXD"}="1H9  HXD"; $resn_exch{" H92 HXD"}="2H9  HXD";
$resn_exch{" H81 HXD"}="1H8  HXD"; $resn_exch{" H82 HXD"}="2H8  HXD"; $resn_exch{" H71 HXD"}="1H7  HXD";
$resn_exch{" H72 HXD"}="2H7  HXD"; $resn_exch{" H61 HXD"}="1H6  HXD"; $resn_exch{" H62 HXD"}="2H6  HXD";
$resn_exch{" H51 HXD"}="1H5  HXD"; $resn_exch{" H52 HXD"}="2H5  HXD"; $resn_exch{" H41 HXD"}="1H4  HXD";
$resn_exch{" H42 HXD"}="2H4  HXD"; $resn_exch{" H21 HXD"}="1H2  HXD"; $resn_exch{" H22 HXD"}="2H2  HXD";
$resn_exch{"HN21 HXL"}="1HN2 HXL"; $resn_exch{"HN22 HXL"}="2HN2 HXL"; $resn_exch{"HN61 HXL"}="1HN6 HXL";
$resn_exch{"HN62 HXL"}="2HN6 HXL"; $resn_exch{"H1'1 HXP"}="1H1' HXP"; $resn_exch{"H1'2 HXP"}="2H1' HXP";
$resn_exch{"H2'1 HXP"}="1H2' HXP"; $resn_exch{"H2'2 HXP"}="2H2' HXP"; $resn_exch{"HO2' HXP"}="'HO2 HXP";
$resn_exch{" HC  HY3"}=" H   HY3"; $resn_exch{" H   HY3"}=" HN  HY3"; $resn_exch{" HG2 HY3"}="1HG  HY3";
$resn_exch{" HG3 HY3"}="2HG  HY3"; $resn_exch{" HD3 HY3"}="1HD  HY3"; $resn_exch{" HD2 HY3"}="2HD  HY3";
$resn_exch{" H41 HYA"}="1H4  HYA"; $resn_exch{" H42 HYA"}="2H4  HYA"; $resn_exch{"H151 HYB"}="1H15 HYB";
$resn_exch{"H152 HYB"}="2H15 HYB"; $resn_exch{"H181 HYB"}="1H18 HYB"; $resn_exch{"H182 HYB"}="2H18 HYB";
$resn_exch{"H301 HYB"}="1H30 HYB"; $resn_exch{"H302 HYB"}="2H30 HYB"; $resn_exch{"H303 HYB"}="3H30 HYB";
$resn_exch{"H311 HYB"}="1H31 HYB"; $resn_exch{"H312 HYB"}="2H31 HYB"; $resn_exch{"H313 HYB"}="3H31 HYB";
$resn_exch{"H351 HYB"}="1H35 HYB"; $resn_exch{"H352 HYB"}="2H35 HYB"; $resn_exch{"H361 HYB"}="1H36 HYB";
$resn_exch{"H362 HYB"}="2H36 HYB"; $resn_exch{"H371 HYB"}="1H37 HYB"; $resn_exch{"H372 HYB"}="2H37 HYB";
$resn_exch{"H373 HYB"}="3H37 HYB"; $resn_exch{"H391 HYB"}="1H39 HYB"; $resn_exch{"H392 HYB"}="2H39 HYB";
$resn_exch{"H393 HYB"}="3H39 HYB"; $resn_exch{"H401 HYB"}="1H40 HYB"; $resn_exch{"H402 HYB"}="2H40 HYB";
$resn_exch{"H403 HYB"}="3H40 HYB"; $resn_exch{" H61 HYC"}="1H6  HYC"; $resn_exch{" H62 HYC"}="2H6  HYC";
$resn_exch{" H71 HYC"}="1H7  HYC"; $resn_exch{" H72 HYC"}="2H7  HYC"; $resn_exch{"H111 HYC"}="1H11 HYC";
$resn_exch{"H112 HYC"}="2H11 HYC"; $resn_exch{"H121 HYC"}="1H12 HYC"; $resn_exch{"H122 HYC"}="2H12 HYC";
$resn_exch{"H151 HYC"}="1H15 HYC"; $resn_exch{"H152 HYC"}="2H15 HYC"; $resn_exch{"H161 HYC"}="1H16 HYC";
$resn_exch{"H181 HYC"}="1H18 HYC"; $resn_exch{"H182 HYC"}="2H18 HYC"; $resn_exch{"H183 HYC"}="3H18 HYC";
$resn_exch{"H211 HYC"}="1H21 HYC"; $resn_exch{"H212 HYC"}="2H21 HYC"; $resn_exch{"H221 HYC"}="1H22 HYC";
$resn_exch{"H222 HYC"}="2H22 HYC"; $resn_exch{"H231 HYC"}="1H23 HYC"; $resn_exch{"H232 HYC"}="2H23 HYC";
$resn_exch{"H241 HYC"}="1H24 HYC"; $resn_exch{"H242 HYC"}="2H24 HYC"; $resn_exch{"H251 HYC"}="1H25 HYC";
$resn_exch{"H252 HYC"}="2H25 HYC"; $resn_exch{"H261 HYC"}="1H26 HYC"; $resn_exch{"H262 HYC"}="2H26 HYC";
$resn_exch{"H271 HYC"}="1H27 HYC"; $resn_exch{"H272 HYC"}="2H27 HYC"; $resn_exch{"H281 HYC"}="1H28 HYC";
$resn_exch{"H282 HYC"}="2H28 HYC"; $resn_exch{"HO21 HYC"}="1HO2 HYC"; $resn_exch{"HO31 HYC"}="1HO3 HYC";
$resn_exch{"H351 HYC"}="1H35 HYC"; $resn_exch{"H352 HYC"}="2H35 HYC"; $resn_exch{"HN41 HYC"}="1HN4 HYC";
$resn_exch{"HN42 HYC"}="2HN4 HYC"; $resn_exch{"H021 HYF"}="1H02 HYF"; $resn_exch{"H022 HYF"}="2H02 HYF";
$resn_exch{"H023 HYF"}="3H02 HYF"; $resn_exch{"H031 HYF"}="1H03 HYF"; $resn_exch{"H032 HYF"}="2H03 HYF";
$resn_exch{"H033 HYF"}="3H03 HYF"; $resn_exch{"H041 HYF"}="1H04 HYF"; $resn_exch{"H042 HYF"}="2H04 HYF";
$resn_exch{"H043 HYF"}="3H04 HYF"; $resn_exch{"H051 HYF"}="1H05 HYF"; $resn_exch{"H061 HYF"}="1H06 HYF";
$resn_exch{"H062 HYF"}="2H06 HYF"; $resn_exch{"H063 HYF"}="3H06 HYF"; $resn_exch{"H091 HYF"}="1H09 HYF";
$resn_exch{"H111 HYF"}="1H11 HYF"; $resn_exch{"H112 HYF"}="2H11 HYF"; $resn_exch{"H121 HYF"}="1H12 HYF";
$resn_exch{"H122 HYF"}="2H12 HYF"; $resn_exch{"H123 HYF"}="3H12 HYF"; $resn_exch{"H131 HYF"}="1H13 HYF";
$resn_exch{"H132 HYF"}="2H13 HYF"; $resn_exch{"H133 HYF"}="3H13 HYF"; $resn_exch{"H141 HYF"}="1H14 HYF";
$resn_exch{"H142 HYF"}="2H14 HYF"; $resn_exch{"H143 HYF"}="3H14 HYF"; $resn_exch{"H161 HYF"}="1H16 HYF";
$resn_exch{"H171 HYF"}="1H17 HYF"; $resn_exch{"H172 HYF"}="2H17 HYF"; $resn_exch{"H211 HYF"}="1H21 HYF";
$resn_exch{"H221 HYF"}="1H22 HYF"; $resn_exch{"H222 HYF"}="2H22 HYF"; $resn_exch{"H223 HYF"}="3H22 HYF";
$resn_exch{"H231 HYF"}="1H23 HYF"; $resn_exch{"H232 HYF"}="2H23 HYF"; $resn_exch{"H233 HYF"}="3H23 HYF";
$resn_exch{"H251 HYF"}="1H25 HYF"; $resn_exch{"H261 HYF"}="1H26 HYF"; $resn_exch{"H262 HYF"}="2H26 HYF";
$resn_exch{"H321 HYF"}="1H32 HYF"; $resn_exch{"H322 HYF"}="2H32 HYF"; $resn_exch{"H331 HYF"}="1H33 HYF";
$resn_exch{"H351 HYF"}="1H35 HYF"; $resn_exch{"H352 HYF"}="2H35 HYF"; $resn_exch{"H361 HYF"}="1H36 HYF";
$resn_exch{"H362 HYF"}="2H36 HYF"; $resn_exch{"H371 HYF"}="1H37 HYF"; $resn_exch{"H381 HYF"}="1H38 HYF";
$resn_exch{"H382 HYF"}="2H38 HYF"; $resn_exch{"H383 HYF"}="3H38 HYF"; $resn_exch{"H391 HYF"}="1H39 HYF";
$resn_exch{"H392 HYF"}="2H39 HYF"; $resn_exch{"H393 HYF"}="3H39 HYF"; $resn_exch{"H31A HYG"}="1H3  HYG";
$resn_exch{"H32A HYG"}="2H3  HYG"; $resn_exch{" H71 HYG"}="1H7  HYG"; $resn_exch{" H72 HYG"}="2H7  HYG";
$resn_exch{"H101 HYG"}="1H10 HYG"; $resn_exch{"H102 HYG"}="2H10 HYG"; $resn_exch{"H103 HYG"}="3H10 HYG";
$resn_exch{"H191 HYG"}="1H19 HYG"; $resn_exch{"H192 HYG"}="2H19 HYG"; $resn_exch{"H341 HYG"}="1H34 HYG";
$resn_exch{"H342 HYG"}="2H34 HYG"; $resn_exch{"H361 HYG"}="1H36 HYG"; $resn_exch{"H362 HYG"}="2H36 HYG";
$resn_exch{"HN1A HYI"}="1HN  HYI"; $resn_exch{" HN2 HYI"}="2HN  HYI"; $resn_exch{" HB1 HYI"}="1HB  HYI";
$resn_exch{" HB2 HYI"}="2HB  HYI"; $resn_exch{" HG1 HYI"}="1HG  HYI"; $resn_exch{" HG2 HYI"}="2HG  HYI";
$resn_exch{" H51 HYI"}="1H5  HYI"; $resn_exch{" H52 HYI"}="2H5  HYI"; $resn_exch{"HAI1 HYM"}="1HAI HYM";
$resn_exch{"HAI2 HYM"}="2HAI HYM"; $resn_exch{"HAH1 HYM"}="1HAH HYM"; $resn_exch{"HAH2 HYM"}="2HAH HYM";
$resn_exch{"HAQ1 HYM"}="1HAQ HYM"; $resn_exch{"HAQ2 HYM"}="2HAQ HYM"; $resn_exch{" HB2 HYP"}="1HB  HYP";
$resn_exch{" HB3 HYP"}="2HB  HYP"; $resn_exch{"HD22 HYP"}="1HD  HYP"; $resn_exch{"HD23 HYP"}="2HD  HYP";
$resn_exch{" HD1 HYP"}=" HOD HYP"; $resn_exch{"H121 HYQ"}="1H12 HYQ"; $resn_exch{"H122 HYQ"}="2H12 HYQ";
$resn_exch{" H81 HYQ"}="1H8  HYQ"; $resn_exch{" H82 HYQ"}="2H8  HYQ"; $resn_exch{" H71 HYQ"}="1H7  HYQ";
$resn_exch{" H72 HYQ"}="2H7  HYQ"; $resn_exch{"H111 HYQ"}="1H11 HYQ"; $resn_exch{"H112 HYQ"}="2H11 HYQ";
$resn_exch{"HAA1 HZ3"}="1HAA HZ3"; $resn_exch{"HAA2 HZ3"}="2HAA HZ3"; $resn_exch{"HAA3 HZ3"}="3HAA HZ3";
$resn_exch{"HAB1 HZ3"}="1HAB HZ3"; $resn_exch{"HAB2 HZ3"}="2HAB HZ3"; $resn_exch{"HAB3 HZ3"}="3HAB HZ3";
$resn_exch{"HOP3   I"}="3HOP   I"; $resn_exch{"HOP2   I"}="2HOP   I"; $resn_exch{" H5'   I"}="1H5*   I";
$resn_exch{"H5''   I"}="2H5*   I"; $resn_exch{" H4'   I"}=" H4*   I"; $resn_exch{" H3'   I"}=" H3*   I";
$resn_exch{"HO3'   I"}="H3T    I"; $resn_exch{" H2'   I"}=" H2*   I"; $resn_exch{"HO2'   I"}="2HO*   I";
$resn_exch{" H1'   I"}=" H1*   I"; $resn_exch{"H981 I01"}="1H98 I01"; $resn_exch{"H982 I01"}="2H98 I01";
$resn_exch{"H983 I01"}="3H98 I01"; $resn_exch{"H361 I01"}="1H36 I01"; $resn_exch{"H362 I01"}="2H36 I01";
$resn_exch{"H351 I01"}="1H35 I01"; $resn_exch{"H352 I01"}="2H35 I01"; $resn_exch{"H341 I01"}="1H34 I01";
$resn_exch{"H342 I01"}="2H34 I01"; $resn_exch{"H321 I01"}="1H32 I01"; $resn_exch{"H322 I01"}="2H32 I01";
$resn_exch{"H151 I02"}="1H15 I02"; $resn_exch{"H152 I02"}="2H15 I02"; $resn_exch{"H153 I02"}="3H15 I02";
$resn_exch{"H161 I02"}="1H16 I02"; $resn_exch{"H162 I02"}="2H16 I02"; $resn_exch{"H163 I02"}="3H16 I02";
$resn_exch{"H171 I02"}="1H17 I02"; $resn_exch{"H172 I02"}="2H17 I02"; $resn_exch{"H173 I02"}="3H17 I02";
$resn_exch{"HN31 I02"}="1HN3 I02"; $resn_exch{"HN32 I02"}="2HN3 I02"; $resn_exch{"H211 I02"}="1H21 I02";
$resn_exch{"H212 I02"}="2H21 I02"; $resn_exch{"H213 I02"}="3H21 I02"; $resn_exch{"H221 I02"}="1H22 I02";
$resn_exch{"H222 I02"}="2H22 I02"; $resn_exch{"H151 I03"}="1H15 I03"; $resn_exch{"H152 I03"}="2H15 I03";
$resn_exch{"H153 I03"}="3H15 I03"; $resn_exch{"H161 I03"}="1H16 I03"; $resn_exch{"H162 I03"}="2H16 I03";
$resn_exch{"H163 I03"}="3H16 I03"; $resn_exch{"H171 I03"}="1H17 I03"; $resn_exch{"H172 I03"}="2H17 I03";
$resn_exch{"H173 I03"}="3H17 I03"; $resn_exch{"H191 I03"}="1H19 I03"; $resn_exch{"H192 I03"}="2H19 I03";
$resn_exch{"HN31 I03"}="1HN3 I03"; $resn_exch{"HN32 I03"}="2HN3 I03"; $resn_exch{"H211 I03"}="1H21 I03";
$resn_exch{"H212 I03"}="2H21 I03"; $resn_exch{"H221 I03"}="1H22 I03"; $resn_exch{"H222 I03"}="2H22 I03";
$resn_exch{"H981 I04"}="1H98 I04"; $resn_exch{"H982 I04"}="2H98 I04"; $resn_exch{"H983 I04"}="3H98 I04";
$resn_exch{"H361 I04"}="1H36 I04"; $resn_exch{"H362 I04"}="2H36 I04"; $resn_exch{"H351 I04"}="1H35 I04";
$resn_exch{"H352 I04"}="2H35 I04"; $resn_exch{"H341 I04"}="1H34 I04"; $resn_exch{"H342 I04"}="2H34 I04";
$resn_exch{"H321 I04"}="1H32 I04"; $resn_exch{"H322 I04"}="2H32 I04"; $resn_exch{"H971 I05"}="1H97 I05";
$resn_exch{"H972 I05"}="2H97 I05"; $resn_exch{"H973 I05"}="3H97 I05"; $resn_exch{"H421 I05"}="1H42 I05";
$resn_exch{"H422 I05"}="2H42 I05"; $resn_exch{"H361 I05"}="1H36 I05"; $resn_exch{"H362 I05"}="2H36 I05";
$resn_exch{"H351 I05"}="1H35 I05"; $resn_exch{"H352 I05"}="2H35 I05"; $resn_exch{"H341 I05"}="1H34 I05";
$resn_exch{"H342 I05"}="2H34 I05"; $resn_exch{"H321 I05"}="1H32 I05"; $resn_exch{"H322 I05"}="2H32 I05";
$resn_exch{" H71 I06"}="1H7  I06"; $resn_exch{" H81 I06"}="1H8  I06"; $resn_exch{"H111 I06"}="1H11 I06";
$resn_exch{"H121 I06"}="1H12 I06"; $resn_exch{" HA1 I06"}="1HA  I06"; $resn_exch{" HA2 I06"}="2HA  I06";
$resn_exch{" HB1 I06"}="1HB  I06"; $resn_exch{" HB2 I06"}="2HB  I06"; $resn_exch{"HD11 I06"}="1HD1 I06";
$resn_exch{"HD21 I06"}="1HD2 I06"; $resn_exch{"HE11 I06"}="1HE1 I06"; $resn_exch{"HE21 I06"}="1HE2 I06";
$resn_exch{" HZ1 I06"}="1HZ  I06"; $resn_exch{"H981 I08"}="1H98 I08"; $resn_exch{"H982 I08"}="2H98 I08";
$resn_exch{"H983 I08"}="3H98 I08"; $resn_exch{"H361 I08"}="1H36 I08"; $resn_exch{"H362 I08"}="2H36 I08";
$resn_exch{"H351 I08"}="1H35 I08"; $resn_exch{"H352 I08"}="2H35 I08"; $resn_exch{"H341 I08"}="1H34 I08";
$resn_exch{"H342 I08"}="2H34 I08"; $resn_exch{"H321 I08"}="1H32 I08"; $resn_exch{"H322 I08"}="2H32 I08";
$resn_exch{"H241 I0G"}="1H24 I0G"; $resn_exch{"H242 I0G"}="2H24 I0G"; $resn_exch{"H11A I0G"}="1H1  I0G";
$resn_exch{" H12 I0G"}="2H1  I0G"; $resn_exch{"H231 I0G"}="1H23 I0G"; $resn_exch{"H232 I0G"}="2H23 I0G";
$resn_exch{"H161 I10"}="1H16 I10"; $resn_exch{"H162 I10"}="2H16 I10"; $resn_exch{"H201 I10"}="1H20 I10";
$resn_exch{"H202 I10"}="2H20 I10"; $resn_exch{"H203 I10"}="3H20 I10"; $resn_exch{"H241 I10"}="1H24 I10";
$resn_exch{"H242 I10"}="2H24 I10"; $resn_exch{"H243 I10"}="3H24 I10"; $resn_exch{"H301 I10"}="1H30 I10";
$resn_exch{"H302 I10"}="2H30 I10"; $resn_exch{"H351 I10"}="1H35 I10"; $resn_exch{"H352 I10"}="2H35 I10";
$resn_exch{" H11 I11"}="1H1  I11"; $resn_exch{" H12 I11"}="2H1  I11"; $resn_exch{" H21 I11"}="1H2  I11";
$resn_exch{" H22 I11"}="2H2  I11"; $resn_exch{" H41 I11"}="1H4  I11"; $resn_exch{" H42 I11"}="2H4  I11";
$resn_exch{" H51 I11"}="1H5  I11"; $resn_exch{" H52 I11"}="2H5  I11"; $resn_exch{" H61 I11"}="1H6  I11";
$resn_exch{" H62 I11"}="2H6  I11"; $resn_exch{"H211 I11"}="1H21 I11"; $resn_exch{"H212 I11"}="2H21 I11";
$resn_exch{"H251 I11"}="1H25 I11"; $resn_exch{"H252 I11"}="2H25 I11"; $resn_exch{"H261 I11"}="1H26 I11";
$resn_exch{"H262 I11"}="2H26 I11"; $resn_exch{"H361 I11"}="1H36 I11"; $resn_exch{"H362 I11"}="2H36 I11";
$resn_exch{"H371 I11"}="1H37 I11"; $resn_exch{"H372 I11"}="2H37 I11"; $resn_exch{"H381 I11"}="1H38 I11";
$resn_exch{"H382 I11"}="2H38 I11"; $resn_exch{"H391 I11"}="1H39 I11"; $resn_exch{"H392 I11"}="2H39 I11";
$resn_exch{"H401 I11"}="1H40 I11"; $resn_exch{"H402 I11"}="2H40 I11"; $resn_exch{" H41 I12"}="1H4  I12";
$resn_exch{" H42 I12"}="2H4  I12"; $resn_exch{" H43 I12"}="3H4  I12"; $resn_exch{" H81 I12"}="1H8  I12";
$resn_exch{" H82 I12"}="2H8  I12"; $resn_exch{" H83 I12"}="3H8  I12"; $resn_exch{"H111 I12"}="1H11 I12";
$resn_exch{"H112 I12"}="2H11 I12"; $resn_exch{"H113 I12"}="3H11 I12"; $resn_exch{"H121 I12"}="1H12 I12";
$resn_exch{"H122 I12"}="2H12 I12"; $resn_exch{"H123 I12"}="3H12 I12"; $resn_exch{"H151 I12"}="1H15 I12";
$resn_exch{"H152 I12"}="2H15 I12"; $resn_exch{"H171 I12"}="1H17 I12"; $resn_exch{"H172 I12"}="2H17 I12";
$resn_exch{"H173 I12"}="3H17 I12"; $resn_exch{"H181 I12"}="1H18 I12"; $resn_exch{"H182 I12"}="2H18 I12";
$resn_exch{"H183 I12"}="3H18 I12"; $resn_exch{"H241 I12"}="1H24 I12"; $resn_exch{"H242 I12"}="2H24 I12";
$resn_exch{"H251 I12"}="1H25 I12"; $resn_exch{"H252 I12"}="2H25 I12"; $resn_exch{"H253 I12"}="3H25 I12";
$resn_exch{"H261 I12"}="1H26 I12"; $resn_exch{"H262 I12"}="2H26 I12"; $resn_exch{"H281 I12"}="1H28 I12";
$resn_exch{"H282 I12"}="2H28 I12"; $resn_exch{"H291 I12"}="1H29 I12"; $resn_exch{"H292 I12"}="2H29 I12";
$resn_exch{"H261 I17"}="1H26 I17"; $resn_exch{" H81 I17"}="1H8  I17"; $resn_exch{"H181 I17"}="1H18 I17";
$resn_exch{"H191 I17"}="1H19 I17"; $resn_exch{"H192 I17"}="2H19 I17"; $resn_exch{"H251 I17"}="1H25 I17";
$resn_exch{"H241 I17"}="1H24 I17"; $resn_exch{"H231 I17"}="1H23 I17"; $resn_exch{"H221 I17"}="1H22 I17";
$resn_exch{"H211 I17"}="1H21 I17"; $resn_exch{"H101 I17"}="1H10 I17"; $resn_exch{"H111 I17"}="1H11 I17";
$resn_exch{"H141 I17"}="1H14 I17"; $resn_exch{"H151 I17"}="1H15 I17"; $resn_exch{"H152 I17"}="2H15 I17";
$resn_exch{"H161 I17"}="1H16 I17"; $resn_exch{"H162 I17"}="2H16 I17"; $resn_exch{"H121 I17"}="1H12 I17";
$resn_exch{"H122 I17"}="2H12 I17"; $resn_exch{"H131 I17"}="1H13 I17"; $resn_exch{"H132 I17"}="2H13 I17";
$resn_exch{"H171 I17"}="1H17 I17"; $resn_exch{"H172 I17"}="2H17 I17"; $resn_exch{"H301 I17"}="1H30 I17";
$resn_exch{"H302 I17"}="2H30 I17"; $resn_exch{"H291 I17"}="1H29 I17"; $resn_exch{"H292 I17"}="2H29 I17";
$resn_exch{"H271 I17"}="1H27 I17"; $resn_exch{"H272 I17"}="2H27 I17"; $resn_exch{"H281 I17"}="1H28 I17";
$resn_exch{"H282 I17"}="2H28 I17"; $resn_exch{"H9C1 I1H"}="1H9C I1H"; $resn_exch{"H9C2 I1H"}="2H9C I1H";
$resn_exch{"H101 I1H"}="1H10 I1H"; $resn_exch{"H102 I1H"}="2H10 I1H"; $resn_exch{"H251 I1H"}="1H25 I1H";
$resn_exch{"H252 I1H"}="2H25 I1H"; $resn_exch{"H291 I1H"}="1H29 I1H"; $resn_exch{"H292 I1H"}="2H29 I1H";
$resn_exch{"H301 I1H"}="1H30 I1H"; $resn_exch{"H302 I1H"}="2H30 I1H"; $resn_exch{"H321 I1H"}="1H32 I1H";
$resn_exch{"H322 I1H"}="2H32 I1H"; $resn_exch{"H331 I1H"}="1H33 I1H"; $resn_exch{"H332 I1H"}="2H33 I1H";
$resn_exch{"H7C1 I1H"}="1H7C I1H"; $resn_exch{"H7C2 I1H"}="2H7C I1H"; $resn_exch{"H7C3 I1H"}="3H7C I1H";
$resn_exch{"H251 I1P"}="1H25 I1P"; $resn_exch{"H252 I1P"}="2H25 I1P"; $resn_exch{"H261 I1P"}="1H26 I1P";
$resn_exch{"H262 I1P"}="2H26 I1P"; $resn_exch{"H271 I1P"}="1H27 I1P"; $resn_exch{"H272 I1P"}="2H27 I1P";
$resn_exch{"H291 I1P"}="1H29 I1P"; $resn_exch{"H292 I1P"}="2H29 I1P"; $resn_exch{"H293 I1P"}="3H29 I1P";
$resn_exch{"H301 I1P"}="1H30 I1P"; $resn_exch{"H302 I1P"}="2H30 I1P"; $resn_exch{"H303 I1P"}="3H30 I1P";
$resn_exch{" H11 I20"}="1H1  I20"; $resn_exch{"H12A I20"}="2H1  I20"; $resn_exch{"H13A I20"}="3H1  I20";
$resn_exch{" H71 I20"}="1H7  I20"; $resn_exch{" H72 I20"}="2H7  I20"; $resn_exch{" H91 I20"}="1H9  I20";
$resn_exch{" H92 I20"}="2H9  I20"; $resn_exch{"H271 I20"}="1H27 I20"; $resn_exch{"H272 I20"}="2H27 I20";
$resn_exch{"H291 I20"}="1H29 I20"; $resn_exch{"H292 I20"}="2H29 I20"; $resn_exch{"H321 I20"}="1H32 I20";
$resn_exch{"H322 I20"}="2H32 I20"; $resn_exch{" HG1 I20"}="1HG  I20"; $resn_exch{" HG2 I20"}="2HG  I20";
$resn_exch{" HB1 I20"}="1HB  I20"; $resn_exch{" HB2 I20"}="2HB  I20"; $resn_exch{" HA1 I20"}="1HA  I20";
$resn_exch{" HA2 I20"}="2HA  I20"; $resn_exch{"H361 I20"}="1H36 I20"; $resn_exch{"H362 I20"}="2H36 I20";
$resn_exch{"H363 I20"}="3H36 I20"; $resn_exch{"H101 I21"}="1H10 I21"; $resn_exch{"H102 I21"}="2H10 I21";
$resn_exch{"HN31 I21"}="1HN3 I21"; $resn_exch{"HN32 I21"}="2HN3 I21"; $resn_exch{"H141 I21"}="1H14 I21";
$resn_exch{"H142 I21"}="2H14 I21"; $resn_exch{"H301 I21"}="1H30 I21"; $resn_exch{"H302 I21"}="2H30 I21";
$resn_exch{"H303 I21"}="3H30 I21"; $resn_exch{" H71 I22"}="1H7  I22"; $resn_exch{" H72 I22"}="2H7  I22";
$resn_exch{" H21 I22"}="1H2  I22"; $resn_exch{" H22 I22"}="2H2  I22"; $resn_exch{"HO22 I2P"}="2HO2 I2P";
$resn_exch{"HO23 I2P"}="3HO2 I2P"; $resn_exch{"HO42 I2P"}="2HO4 I2P"; $resn_exch{"HO43 I2P"}="3HO4 I2P";
$resn_exch{"HO52 I2P"}="2HO5 I2P"; $resn_exch{"HO53 I2P"}="3HO5 I2P"; $resn_exch{" H11 I38"}="1H1  I38";
$resn_exch{" H12 I38"}="2H1  I38"; $resn_exch{" H21 I38"}="1H2  I38"; $resn_exch{" H22 I38"}="2H2  I38";
$resn_exch{"H511 I38"}="1H51 I38"; $resn_exch{"H512 I38"}="2H51 I38"; $resn_exch{"H513 I38"}="3H51 I38";
$resn_exch{" H3' I3A"}=" H3* I3A"; $resn_exch{" H11 I3N"}="1H1  I3N"; $resn_exch{" H12 I3N"}="2H1  I3N";
$resn_exch{" H13 I3N"}="3H1  I3N"; $resn_exch{"H121 I3N"}="1H12 I3N"; $resn_exch{"H122 I3N"}="2H12 I3N";
$resn_exch{"H161 I3N"}="1H16 I3N"; $resn_exch{"H162 I3N"}="2H16 I3N"; $resn_exch{"H163 I3N"}="3H16 I3N";
$resn_exch{"H171 I3N"}="1H17 I3N"; $resn_exch{"H172 I3N"}="2H17 I3N"; $resn_exch{"HO12 I3P"}="2HO1 I3P";
$resn_exch{"HO13 I3P"}="3HO1 I3P"; $resn_exch{"HO42 I3P"}="2HO4 I3P"; $resn_exch{"HO43 I3P"}="3HO4 I3P";
$resn_exch{"HO52 I3P"}="2HO5 I3P"; $resn_exch{"HO53 I3P"}="3HO5 I3P"; $resn_exch{" H12 I40"}="2H1  I40";
$resn_exch{" H11 I40"}="1H1  I40"; $resn_exch{" H22 I40"}="2H2  I40"; $resn_exch{" H21 I40"}="1H2  I40";
$resn_exch{" H32 I40"}="2H3  I40"; $resn_exch{" H31 I40"}="1H3  I40"; $resn_exch{" H42 I40"}="2H4  I40";
$resn_exch{" H41 I40"}="1H4  I40"; $resn_exch{"H2N1 I40"}="1H2N I40"; $resn_exch{"H142 I40"}="2H14 I40";
$resn_exch{"H141 I40"}="1H14 I40"; $resn_exch{"HA21 I48"}="1HA2 I48"; $resn_exch{"HA22 I48"}="2HA2 I48";
$resn_exch{"HA31 I48"}="1HA3 I48"; $resn_exch{"HA32 I48"}="2HA3 I48"; $resn_exch{"H271 I48"}="1H27 I48";
$resn_exch{"H272 I48"}="2H27 I48"; $resn_exch{"H273 I48"}="3H27 I48"; $resn_exch{"H1'1 I4B"}="1H1' I4B";
$resn_exch{"H1'2 I4B"}="2H1' I4B"; $resn_exch{"H3'1 I4B"}="1H3' I4B"; $resn_exch{"H3'2 I4B"}="2H3' I4B";
$resn_exch{"H3'3 I4B"}="3H3' I4B"; $resn_exch{"H4'1 I4B"}="1H4' I4B"; $resn_exch{"H4'2 I4B"}="2H4' I4B";
$resn_exch{"H4'3 I4B"}="3H4' I4B"; $resn_exch{"H381 I52"}="1H38 I52"; $resn_exch{"H382 I52"}="2H38 I52";
$resn_exch{"H391 I52"}="1H39 I52"; $resn_exch{"H392 I52"}="2H39 I52"; $resn_exch{"H393 I52"}="3H39 I52";
$resn_exch{"H371 I52"}="1H37 I52"; $resn_exch{"H372 I52"}="2H37 I52"; $resn_exch{"H351 I52"}="1H35 I52";
$resn_exch{"H352 I52"}="2H35 I52"; $resn_exch{"H361 I52"}="1H36 I52"; $resn_exch{"H362 I52"}="2H36 I52";
$resn_exch{"H251 I52"}="1H25 I52"; $resn_exch{"H252 I52"}="2H25 I52"; $resn_exch{"H253 I52"}="3H25 I52";
$resn_exch{"H241 I52"}="1H24 I52"; $resn_exch{"H242 I52"}="2H24 I52"; $resn_exch{"H243 I52"}="3H24 I52";
$resn_exch{" H71 I52"}="1H7  I52"; $resn_exch{" H72 I52"}="2H7  I52"; $resn_exch{" H91 I52"}="1H9  I52";
$resn_exch{" H92 I52"}="2H9  I52"; $resn_exch{"H131 I52"}="1H13 I52"; $resn_exch{"H132 I52"}="2H13 I52";
$resn_exch{"H121 I52"}="1H12 I52"; $resn_exch{"H122 I52"}="2H12 I52"; $resn_exch{"H101 I52"}="1H10 I52";
$resn_exch{"H102 I52"}="2H10 I52"; $resn_exch{" H61 I52"}="1H6  I52"; $resn_exch{" H62 I52"}="2H6  I52";
$resn_exch{" H   I58"}="1HN  I58"; $resn_exch{" H2  I58"}="2HN  I58"; $resn_exch{" HB2 I58"}="1HB  I58";
$resn_exch{" HB3 I58"}="2HB  I58"; $resn_exch{" HD2 I58"}="1HD  I58"; $resn_exch{" HD3 I58"}="2HD  I58";
$resn_exch{" HE2 I58"}="1HE  I58"; $resn_exch{" HE3 I58"}="2HE  I58"; $resn_exch{" HZ  I58"}="1HZ  I58";
$resn_exch{" HT1 I58"}="1HT  I58"; $resn_exch{" HT2 I58"}="2HT  I58"; $resn_exch{" HT3 I58"}="3HT  I58";
$resn_exch{"HOP2 I5C"}="2HOP I5C"; $resn_exch{" H41 I5C"}="1H4  I5C"; $resn_exch{" H42 I5C"}="2H4  I5C";
$resn_exch{" H2' I5C"}="1H2* I5C"; $resn_exch{"H2'' I5C"}="2H2* I5C"; $resn_exch{" H5' I5C"}="1H5* I5C";
$resn_exch{"H5'' I5C"}="2H5* I5C"; $resn_exch{" H4' I5C"}=" H4* I5C"; $resn_exch{" H1' I5C"}=" H1* I5C";
$resn_exch{" H3' I5C"}=" H3* I5C"; $resn_exch{"HO3' I5C"}="H3T  I5C"; $resn_exch{"HOP3 I5C"}="3HOP I5C";
$resn_exch{"H6C1 I5S"}="1H6C I5S"; $resn_exch{"H6C2 I5S"}="2H6C I5S"; $resn_exch{"H8C1 I5S"}="1H8C I5S";
$resn_exch{"H8C2 I5S"}="2H8C I5S"; $resn_exch{"H9C1 I5S"}="1H9C I5S"; $resn_exch{"H9C2 I5S"}="2H9C I5S";
$resn_exch{"H131 I5S"}="1H13 I5S"; $resn_exch{"H132 I5S"}="2H13 I5S"; $resn_exch{"H141 I5S"}="1H14 I5S";
$resn_exch{"H142 I5S"}="2H14 I5S"; $resn_exch{"HN11 I7A"}="1HN1 I7A"; $resn_exch{"HN12 I7A"}="2HN1 I7A";
$resn_exch{"HN21 I7A"}="1HN2 I7A"; $resn_exch{"HN22 I7A"}="2HN2 I7A"; $resn_exch{"HN11 I7B"}="1HN1 I7B";
$resn_exch{"HN12 I7B"}="2HN1 I7B"; $resn_exch{"HN21 I7B"}="1HN2 I7B"; $resn_exch{"HN22 I7B"}="2HN2 I7B";
$resn_exch{"HN31 I7B"}="1HN3 I7B"; $resn_exch{"HN32 I7B"}="2HN3 I7B"; $resn_exch{"HN11 I7C"}="1HN1 I7C";
$resn_exch{"HN12 I7C"}="2HN1 I7C"; $resn_exch{"HN21 I7C"}="1HN2 I7C"; $resn_exch{"HN22 I7C"}="2HN2 I7C";
$resn_exch{"HN31 I7C"}="1HN3 I7C"; $resn_exch{"HN32 I7C"}="2HN3 I7C"; $resn_exch{"H111 I84"}="1H11 I84";
$resn_exch{"H112 I84"}="2H11 I84"; $resn_exch{"H113 I84"}="3H11 I84"; $resn_exch{"H151 I84"}="1H15 I84";
$resn_exch{"H152 I84"}="2H15 I84"; $resn_exch{"H291 I84"}="1H29 I84"; $resn_exch{"H292 I84"}="2H29 I84";
$resn_exch{"H293 I84"}="3H29 I84"; $resn_exch{"H331 I84"}="1H33 I84"; $resn_exch{"H332 I84"}="2H33 I84";
$resn_exch{"H333 I84"}="3H33 I84"; $resn_exch{"H391 I84"}="1H39 I84"; $resn_exch{"H392 I84"}="2H39 I84";
$resn_exch{"H101 I96"}="1H10 I96"; $resn_exch{"H102 I96"}="2H10 I96"; $resn_exch{"H103 I96"}="3H10 I96";
$resn_exch{"H111 I96"}="1H11 I96"; $resn_exch{"H112 I96"}="2H11 I96"; $resn_exch{"H113 I96"}="3H11 I96";
$resn_exch{"H171 I96"}="1H17 I96"; $resn_exch{"H172 I96"}="2H17 I96"; $resn_exch{"H201  IA"}="1H20  IA";
$resn_exch{"H202  IA"}="2H20  IA"; $resn_exch{"H211  IA"}="1H21  IA"; $resn_exch{"H212  IA"}="2H21  IA";
$resn_exch{"H221  IA"}="1H22  IA"; $resn_exch{"H222  IA"}="2H22  IA"; $resn_exch{"H231  IA"}="1H23  IA";
$resn_exch{"H232  IA"}="2H23  IA"; $resn_exch{"H241  IA"}="1H24  IA"; $resn_exch{"H242  IA"}="2H24  IA";
$resn_exch{"H243  IA"}="3H24  IA"; $resn_exch{"H171 IAA"}="1H17 IAA"; $resn_exch{"H172 IAA"}="2H17 IAA";
$resn_exch{" H21 IAA"}="1H2  IAA"; $resn_exch{" H22 IAA"}="2H2  IAA"; $resn_exch{"H161 IAA"}="1H16 IAA";
$resn_exch{"H162 IAA"}="2H16 IAA"; $resn_exch{"H6X1 IAB"}="1H6X IAB"; $resn_exch{"H6X2 IAB"}="2H6X IAB";
$resn_exch{"H6Y1 IAB"}="1H6Y IAB"; $resn_exch{"H6Y2 IAB"}="2H6Y IAB"; $resn_exch{"H6Y3 IAB"}="3H6Y IAB";
$resn_exch{"H7Z1 IAB"}="1H7Z IAB"; $resn_exch{"H7Z2 IAB"}="2H7Z IAB"; $resn_exch{"H6W1 IAB"}="1H6W IAB";
$resn_exch{"H6W2 IAB"}="2H6W IAB"; $resn_exch{"H171 IAC"}="1H17 IAC"; $resn_exch{"H172 IAC"}="2H17 IAC";
$resn_exch{" HB1 IAD"}="1HB  IAD"; $resn_exch{" HB2 IAD"}="2HB  IAD"; $resn_exch{" H1  IAD"}="1H   IAD";
$resn_exch{" H2A IAD"}="2H   IAD"; $resn_exch{"H111 IAG"}="1H11 IAG"; $resn_exch{"H112 IAG"}="2H11 IAG";
$resn_exch{"HC91 IAG"}="1HC9 IAG"; $resn_exch{"HC92 IAG"}="2HC9 IAG"; $resn_exch{" HB1 IAM"}="1HB  IAM";
$resn_exch{" HT1 IAM"}="1HT  IAM"; $resn_exch{" HT2 IAM"}="2HT  IAM"; $resn_exch{"HK11 IAM"}="1HK1 IAM";
$resn_exch{"HK12 IAM"}="2HK1 IAM"; $resn_exch{"HK13 IAM"}="3HK1 IAM"; $resn_exch{"HK21 IAM"}="1HK2 IAM";
$resn_exch{"HK22 IAM"}="2HK2 IAM"; $resn_exch{"HK23 IAM"}="3HK2 IAM"; $resn_exch{"HO11 IAP"}="1HO1 IAP";
$resn_exch{"HO21 IAP"}="1HO2 IAP"; $resn_exch{" H21 IAP"}="1H2  IAP"; $resn_exch{" H31 IAP"}="1H3  IAP";
$resn_exch{" H41 IAP"}="1H4  IAP"; $resn_exch{" H61 IAP"}="1H6  IAP"; $resn_exch{"HN11 IAP"}="1HN1 IAP";
$resn_exch{" H81 IAP"}="1H8  IAP"; $resn_exch{" H82 IAP"}="2H8  IAP"; $resn_exch{" H   IAS"}="1HN  IAS";
$resn_exch{" H2  IAS"}="2HN  IAS"; $resn_exch{" HB2 IAS"}="1HB  IAS"; $resn_exch{" HB3 IAS"}="2HB  IAS";
$resn_exch{"H141 IAV"}="1H14 IAV"; $resn_exch{"H142 IAV"}="2H14 IAV"; $resn_exch{"H143 IAV"}="3H14 IAV";
$resn_exch{"H151 IAV"}="1H15 IAV"; $resn_exch{"H152 IAV"}="2H15 IAV"; $resn_exch{"H153 IAV"}="3H15 IAV";
$resn_exch{"HC91 IAV"}="1HC9 IAV"; $resn_exch{"HC92 IAV"}="2HC9 IAV"; $resn_exch{"H201  IB"}="1H20  IB";
$resn_exch{"H202  IB"}="2H20  IB"; $resn_exch{"H211  IB"}="1H21  IB"; $resn_exch{"H212  IB"}="2H21  IB";
$resn_exch{"H221  IB"}="1H22  IB"; $resn_exch{"H222  IB"}="2H22  IB"; $resn_exch{"H231  IB"}="1H23  IB";
$resn_exch{"H232  IB"}="2H23  IB"; $resn_exch{"H241  IB"}="1H24  IB"; $resn_exch{"H242  IB"}="2H24  IB";
$resn_exch{"H243  IB"}="3H24  IB"; $resn_exch{"HN21  IB"}="1HN2  IB"; $resn_exch{"HN22  IB"}="2HN2  IB";
$resn_exch{"HN23  IB"}="3HN2  IB"; $resn_exch{"HOA2 IB2"}="2HOA IB2"; $resn_exch{"H3A1 IB2"}="1H3A IB2";
$resn_exch{"H3A2 IB2"}="2H3A IB2"; $resn_exch{"HOB2 IB2"}="2HOB IB2"; $resn_exch{"HSG2 IB2"}="2HSG IB2";
$resn_exch{"H5'1 IB2"}="1H5* IB2"; $resn_exch{"H5'2 IB2"}="2H5* IB2"; $resn_exch{" H4D IB2"}=" H4* IB2";
$resn_exch{" H3D IB2"}=" H3* IB2"; $resn_exch{"HO3' IB2"}="*HO3 IB2"; $resn_exch{" H2D IB2"}=" H2* IB2";
$resn_exch{"HO2' IB2"}="*HO2 IB2"; $resn_exch{" H1D IB2"}=" H1* IB2"; $resn_exch{"H51X IB2"}="1H5' IB2";
$resn_exch{"H52X IB2"}="2H5' IB2"; $resn_exch{"HO3X IB2"}="'HO3 IB2"; $resn_exch{"HO2X IB2"}="'HO2 IB2";
$resn_exch{"HN61 IB2"}="1HN6 IB2"; $resn_exch{"HN62 IB2"}="2HN6 IB2"; $resn_exch{" H8A IB2"}=" AH8 IB2";
$resn_exch{"HNA1 IB2"}="1HNA IB2"; $resn_exch{"HNA2 IB2"}="2HNA IB2"; $resn_exch{" H2A IB2"}=" AH2 IB2";
$resn_exch{"H151 IBA"}="1H15 IBA"; $resn_exch{"H152 IBA"}="2H15 IBA"; $resn_exch{"H141 IBA"}="1H14 IBA";
$resn_exch{"H142 IBA"}="2H14 IBA"; $resn_exch{"H181 IBA"}="1H18 IBA"; $resn_exch{"H182 IBA"}="2H18 IBA";
$resn_exch{"H171 IBA"}="1H17 IBA"; $resn_exch{"H172 IBA"}="2H17 IBA"; $resn_exch{" H11 IBA"}="1H1  IBA";
$resn_exch{" H12 IBA"}="2H1  IBA"; $resn_exch{"H111 IBA"}="1H11 IBA"; $resn_exch{"H112 IBA"}="2H11 IBA";
$resn_exch{"H121 IBA"}="1H12 IBA"; $resn_exch{"H122 IBA"}="2H12 IBA"; $resn_exch{"H123 IBA"}="3H12 IBA";
$resn_exch{" H91 IBA"}="1H9  IBA"; $resn_exch{" H92 IBA"}="2H9  IBA"; $resn_exch{"H101 IBA"}="1H10 IBA";
$resn_exch{"H102 IBA"}="2H10 IBA"; $resn_exch{"H103 IBA"}="3H10 IBA"; $resn_exch{"H21A IBB"}="1H2  IBB";
$resn_exch{" H22 IBB"}="2H2  IBB"; $resn_exch{" H31 IBB"}="1H3  IBB"; $resn_exch{" H32 IBB"}="2H3  IBB";
$resn_exch{"HN21 IBC"}="1HN2 IBC"; $resn_exch{"HN22 IBC"}="2HN2 IBC"; $resn_exch{" H91 IBC"}="1H9  IBC";
$resn_exch{" H92 IBC"}="2H9  IBC"; $resn_exch{" H61 IBC"}="1H6  IBC"; $resn_exch{" H62 IBC"}="2H6  IBC";
$resn_exch{" H71 IBC"}="1H7  IBC"; $resn_exch{" H72 IBC"}="2H7  IBC"; $resn_exch{" H81 IBC"}="1H8  IBC";
$resn_exch{" H82 IBC"}="2H8  IBC"; $resn_exch{"HN11 IBG"}="1HN1 IBG"; $resn_exch{"HN12 IBG"}="2HN1 IBG";
$resn_exch{"HA1C IBG"}="CHA1 IBG"; $resn_exch{"HCB1 IBG"}="1HCB IBG"; $resn_exch{"HCB2 IBG"}="2HCB IBG";
$resn_exch{"HCG1 IBG"}="1HCG IBG"; $resn_exch{"HCG2 IBG"}="2HCG IBG"; $resn_exch{"HA2C IBG"}="CHA2 IBG";
$resn_exch{"HB21 IBG"}="1HB2 IBG"; $resn_exch{"HB22 IBG"}="2HB2 IBG"; $resn_exch{"HC31 IBG"}="1HC3 IBG";
$resn_exch{"HC32 IBG"}="2HC3 IBG"; $resn_exch{"HCS1 IBG"}="1HCS IBG"; $resn_exch{"HCS2 IBG"}="2HCS IBG";
$resn_exch{"H101 IBM"}="1H10 IBM"; $resn_exch{"H102 IBM"}="2H10 IBM"; $resn_exch{"H103 IBM"}="3H10 IBM";
$resn_exch{"H111 IBM"}="1H11 IBM"; $resn_exch{"H112 IBM"}="2H11 IBM"; $resn_exch{"H131 IBM"}="1H13 IBM";
$resn_exch{"H132 IBM"}="2H13 IBM"; $resn_exch{"H133 IBM"}="3H13 IBM"; $resn_exch{"H141 IBM"}="1H14 IBM";
$resn_exch{"H142 IBM"}="2H14 IBM"; $resn_exch{"H143 IBM"}="3H14 IBM"; $resn_exch{"HAN1 IBO"}="1HAN IBO";
$resn_exch{"HAN2 IBO"}="2HAN IBO"; $resn_exch{"HB21 IBO"}="1HB2 IBO"; $resn_exch{"HB22 IBO"}="2HB2 IBO";
$resn_exch{"HB23 IBO"}="3HB2 IBO"; $resn_exch{"HB11 IBO"}="1HB1 IBO"; $resn_exch{"HB12 IBO"}="2HB1 IBO";
$resn_exch{"HB13 IBO"}="3HB1 IBO"; $resn_exch{" H21 IBP"}="1H2  IBP"; $resn_exch{" H22 IBP"}="2H2  IBP";
$resn_exch{" H31 IBP"}="1H3  IBP"; $resn_exch{" H41 IBP"}="1H4  IBP"; $resn_exch{" H42 IBP"}="2H4  IBP";
$resn_exch{" H43 IBP"}="3H4  IBP"; $resn_exch{" H51 IBP"}="1H5  IBP"; $resn_exch{" H52 IBP"}="2H5  IBP";
$resn_exch{" H53 IBP"}="3H5  IBP"; $resn_exch{" H61 IBP"}="1H6  IBP"; $resn_exch{" H71 IBP"}="1H7  IBP";
$resn_exch{" H72 IBP"}="2H7  IBP"; $resn_exch{" H73 IBP"}="3H7  IBP"; $resn_exch{" H91 IBP"}="1H9  IBP";
$resn_exch{"H101 IBP"}="1H10 IBP"; $resn_exch{"H121 IBP"}="1H12 IBP"; $resn_exch{"H131 IBP"}="1H13 IBP";
$resn_exch{" HN1 IBR"}="1HN  IBR"; $resn_exch{" HN2 IBR"}="2HN  IBR"; $resn_exch{" H21 IBR"}="1H2  IBR";
$resn_exch{" H22 IBR"}="2H2  IBR"; $resn_exch{"HE11 IBR"}="1HE1 IBR"; $resn_exch{"HE12 IBR"}="2HE1 IBR";
$resn_exch{"HE21 IBR"}="1HE2 IBR"; $resn_exch{"HE22 IBR"}="2HE2 IBR"; $resn_exch{"H211 IBS"}="1H21 IBS";
$resn_exch{"H212 IBS"}="2H21 IBS"; $resn_exch{"H251 IBS"}="1H25 IBS"; $resn_exch{"HO13 IBS"}="3HO1 IBS";
$resn_exch{"HO42 IBS"}="2HO4 IBS"; $resn_exch{"HO43 IBS"}="3HO4 IBS"; $resn_exch{"HO52 IBS"}="2HO5 IBS";
$resn_exch{"HO53 IBS"}="3HO5 IBS"; $resn_exch{" H1  IBZ"}="1H   IBZ"; $resn_exch{" H2  IBZ"}="2H   IBZ";
$resn_exch{"HOP2  IC"}="2HOP  IC"; $resn_exch{"HOP3  IC"}="3HOP  IC"; $resn_exch{"HO3'  IC"}="H3T   IC";
$resn_exch{" H1'  IC"}=" H1*  IC"; $resn_exch{" H2'  IC"}=" H2*  IC"; $resn_exch{" H3'  IC"}=" H3*  IC";
$resn_exch{" H4'  IC"}=" H4*  IC"; $resn_exch{" H5'  IC"}="1H5*  IC"; $resn_exch{"H5''  IC"}="2H5*  IC";
$resn_exch{"HO2'  IC"}="2HO*  IC"; $resn_exch{" H21  IC"}="1H2   IC"; $resn_exch{" H22  IC"}="2H2   IC";
$resn_exch{" H41 IC1"}="1H4  IC1"; $resn_exch{" H51 IC1"}="1H5  IC1"; $resn_exch{" H61 IC1"}="1H6  IC1";
$resn_exch{" H71 IC1"}="1H7  IC1"; $resn_exch{"H101 IC1"}="1H10 IC1"; $resn_exch{"H3'1 IC1"}="1H3' IC1";
$resn_exch{"H5'1 IC1"}="1H5' IC1"; $resn_exch{"H111 IC1"}="1H11 IC1"; $resn_exch{"H112 IC1"}="2H11 IC1";
$resn_exch{"H113 IC1"}="3H11 IC1"; $resn_exch{"H121 IC1"}="1H12 IC1"; $resn_exch{"H122 IC1"}="2H12 IC1";
$resn_exch{"H123 IC1"}="3H12 IC1"; $resn_exch{"H131 IC1"}="1H13 IC1"; $resn_exch{"H132 IC1"}="2H13 IC1";
$resn_exch{"H133 IC1"}="3H13 IC1"; $resn_exch{" H41 ICA"}="1H4  ICA"; $resn_exch{" H42 ICA"}="2H4  ICA";
$resn_exch{" H41 ICI"}="1H4  ICI"; $resn_exch{" H42 ICI"}="2H4  ICI"; $resn_exch{"H111 ICL"}="1H11 ICL";
$resn_exch{"H112 ICL"}="2H11 ICL"; $resn_exch{"H113 ICL"}="3H11 ICL"; $resn_exch{"H121 ICL"}="1H12 ICL";
$resn_exch{"H122 ICL"}="2H12 ICL"; $resn_exch{"H123 ICL"}="3H12 ICL"; $resn_exch{"H151 ICL"}="1H15 ICL";
$resn_exch{"H152 ICL"}="2H15 ICL"; $resn_exch{"H153 ICL"}="3H15 ICL"; $resn_exch{"H161 ICL"}="1H16 ICL";
$resn_exch{"H162 ICL"}="2H16 ICL"; $resn_exch{"H163 ICL"}="3H16 ICL"; $resn_exch{"H171 ICL"}="1H17 ICL";
$resn_exch{"H172 ICL"}="2H17 ICL"; $resn_exch{"H173 ICL"}="3H17 ICL"; $resn_exch{" H11 ICN"}="1H1  ICN";
$resn_exch{" H12 ICN"}="2H1  ICN"; $resn_exch{" H13 ICN"}="3H1  ICN"; $resn_exch{" H31 ICN"}="1H3  ICN";
$resn_exch{" H32 ICN"}="2H3  ICN"; $resn_exch{" H33 ICN"}="3H3  ICN"; $resn_exch{" H51 ICN"}="1H5  ICN";
$resn_exch{" H52 ICN"}="2H5  ICN"; $resn_exch{" H53 ICN"}="3H5  ICN"; $resn_exch{"HM21 ICP"}="1HM2 ICP";
$resn_exch{"HM22 ICP"}="2HM2 ICP"; $resn_exch{"HM23 ICP"}="3HM2 ICP"; $resn_exch{"H6A1 ICP"}="1H6A ICP";
$resn_exch{"H7A1 ICP"}="1H7A ICP"; $resn_exch{"H7A2 ICP"}="2H7A ICP"; $resn_exch{"H5'1 ICR"}="1H5' ICR";
$resn_exch{"H5'2 ICR"}="2H5' ICR"; $resn_exch{" H41 ICT"}="1H4  ICT"; $resn_exch{" H42 ICT"}="2H4  ICT";
$resn_exch{"HN11 ICU"}="1HN1 ICU"; $resn_exch{"HN12 ICU"}="2HN1 ICU"; $resn_exch{"H101 ICU"}="1H10 ICU";
$resn_exch{"H102 ICU"}="2H10 ICU"; $resn_exch{"H103 ICU"}="3H10 ICU"; $resn_exch{"H121 ICU"}="1H12 ICU";
$resn_exch{"H122 ICU"}="2H12 ICU"; $resn_exch{"H123 ICU"}="3H12 ICU"; $resn_exch{"HO5' ID2"}="*HO5 ID2";
$resn_exch{"H5'1 ID2"}="1H5* ID2"; $resn_exch{"H5'2 ID2"}="2H5* ID2"; $resn_exch{" H4' ID2"}=" H4* ID2";
$resn_exch{" H1' ID2"}=" H1* ID2"; $resn_exch{"H2'1 ID2"}="1H2* ID2"; $resn_exch{"H2'2 ID2"}="2H2* ID2";
$resn_exch{" H3' ID2"}=" H3* ID2"; $resn_exch{"HO3' ID2"}="*HO3 ID2"; $resn_exch{"H181 ID5"}="1H18 ID5";
$resn_exch{"H182 ID5"}="2H18 ID5"; $resn_exch{"H201 ID5"}="1H20 ID5"; $resn_exch{"H202 ID5"}="2H20 ID5";
$resn_exch{"HN21 IDA"}="1HN2 IDA"; $resn_exch{"HN22 IDA"}="2HN2 IDA"; $resn_exch{"HC91 IDA"}="1HC9 IDA";
$resn_exch{"HC92 IDA"}="2HC9 IDA"; $resn_exch{"H121 IDA"}="1H12 IDA"; $resn_exch{"H122 IDA"}="2H12 IDA";
$resn_exch{"H131 IDA"}="1H13 IDA"; $resn_exch{"H132 IDA"}="2H13 IDA"; $resn_exch{"H141 IDA"}="1H14 IDA";
$resn_exch{"H142 IDA"}="2H14 IDA"; $resn_exch{"H143 IDA"}="3H14 IDA"; $resn_exch{"H111 IDA"}="1H11 IDA";
$resn_exch{"H112 IDA"}="2H11 IDA"; $resn_exch{"H8C1 IDB"}="1H8C IDB"; $resn_exch{"H8C2 IDB"}="2H8C IDB";
$resn_exch{"H101 IDB"}="1H10 IDB"; $resn_exch{"H102 IDB"}="2H10 IDB"; $resn_exch{"H111 IDB"}="1H11 IDB";
$resn_exch{"H112 IDB"}="2H11 IDB"; $resn_exch{"H191 IDB"}="1H19 IDB"; $resn_exch{"H192 IDB"}="2H19 IDB";
$resn_exch{"HCA1 IDC"}="1HCA IDC"; $resn_exch{"HCA2 IDC"}="2HCA IDC"; $resn_exch{"HCB1 IDC"}="1HCB IDC";
$resn_exch{"HCB2 IDC"}="2HCB IDC"; $resn_exch{" H21 IDD"}="1H2  IDD"; $resn_exch{" H31 IDD"}="1H3  IDD";
$resn_exch{" H41 IDD"}="1H4  IDD"; $resn_exch{" H51 IDD"}="1H5  IDD"; $resn_exch{" H61 IDD"}="1H6  IDD";
$resn_exch{"H2B1 IDD"}="1H2B IDD"; $resn_exch{"H3B1 IDD"}="1H3B IDD"; $resn_exch{"H4B1 IDD"}="1H4B IDD";
$resn_exch{"H5B1 IDD"}="1H5B IDD"; $resn_exch{"H6B1 IDD"}="1H6B IDD"; $resn_exch{"H6B2 IDD"}="2H6B IDD";
$resn_exch{"HO21 IDD"}="1HO2 IDD"; $resn_exch{"HO31 IDD"}="1HO3 IDD"; $resn_exch{"HO41 IDD"}="1HO4 IDD";
$resn_exch{"HO61 IDD"}="1HO6 IDD"; $resn_exch{"H8B1 IDD"}="1H8B IDD"; $resn_exch{"H6B1 IDE"}="1H6B IDE";
$resn_exch{"H6B2 IDE"}="2H6B IDE"; $resn_exch{" H71 IDE"}="1H7  IDE"; $resn_exch{" H72 IDE"}="2H7  IDE";
$resn_exch{"HN21 IDG"}="1HN2 IDG"; $resn_exch{"HN22 IDG"}="2HN2 IDG"; $resn_exch{" H61 IDG"}="1H6  IDG";
$resn_exch{" H62 IDG"}="2H6  IDG"; $resn_exch{"HN61 IDG"}="1HN6 IDG"; $resn_exch{"HN62 IDG"}="2HN6 IDG";
$resn_exch{"H6C1 IDH"}="1H6C IDH"; $resn_exch{"H6C2 IDH"}="2H6C IDH"; $resn_exch{"HC41 IDI"}="1HC4 IDI";
$resn_exch{"HC51 IDI"}="1HC5 IDI"; $resn_exch{"HC71 IDI"}="1HC7 IDI"; $resn_exch{"HC11 IDI"}="1HC1 IDI";
$resn_exch{"HC12 IDI"}="2HC1 IDI"; $resn_exch{"HC21 IDI"}="1HC2 IDI"; $resn_exch{"HC22 IDI"}="2HC2 IDI";
$resn_exch{"HC31 IDI"}="1HC3 IDI"; $resn_exch{"HC32 IDI"}="2HC3 IDI"; $resn_exch{"HN11 IDI"}="1HN1 IDI";
$resn_exch{" H21 IDM"}="1H2  IDM"; $resn_exch{" H22 IDM"}="2H2  IDM"; $resn_exch{" H31 IDM"}="1H3  IDM";
$resn_exch{" H32 IDM"}="2H3  IDM"; $resn_exch{"H121 IDN"}="1H12 IDN"; $resn_exch{"H122 IDN"}="2H12 IDN";
$resn_exch{"H123 IDN"}="3H12 IDN"; $resn_exch{"H131 IDN"}="1H13 IDN"; $resn_exch{"H132 IDN"}="2H13 IDN";
$resn_exch{"H201 IDN"}="1H20 IDN"; $resn_exch{"H202 IDN"}="2H20 IDN"; $resn_exch{"H203 IDN"}="3H20 IDN";
$resn_exch{"H251 IDN"}="1H25 IDN"; $resn_exch{"H252 IDN"}="2H25 IDN"; $resn_exch{"H261 IDN"}="1H26 IDN";
$resn_exch{"H262 IDN"}="2H26 IDN"; $resn_exch{"HOB2 IDP"}="2HOB IDP"; $resn_exch{"HOB3 IDP"}="3HOB IDP";
$resn_exch{"HOA2 IDP"}="2HOA IDP"; $resn_exch{"H5'1 IDP"}="1H5* IDP"; $resn_exch{"H5'2 IDP"}="2H5* IDP";
$resn_exch{" H4' IDP"}=" H4* IDP"; $resn_exch{" H3' IDP"}=" H3* IDP"; $resn_exch{"HO3' IDP"}="*HO3 IDP";
$resn_exch{" H2' IDP"}=" H2* IDP"; $resn_exch{"HO2' IDP"}="*HO2 IDP"; $resn_exch{" H1' IDP"}=" H1* IDP";
$resn_exch{"HOS3 IDS"}="3HOS IDS"; $resn_exch{" H11 IDU"}="1H1  IDU"; $resn_exch{" H12 IDU"}="2H1  IDU";
$resn_exch{" H41 IDU"}="1H4  IDU"; $resn_exch{" H42 IDU"}="2H4  IDU"; $resn_exch{"HOS3 IDU"}="3HOS IDU";
$resn_exch{" H71 IEM"}="1H7  IEM"; $resn_exch{" H72 IEM"}="2H7  IEM"; $resn_exch{" H91 IEM"}="1H9  IEM";
$resn_exch{" H92 IEM"}="2H9  IEM"; $resn_exch{" H93 IEM"}="3H9  IEM"; $resn_exch{"H101 IEM"}="1H10 IEM";
$resn_exch{"H102 IEM"}="2H10 IEM"; $resn_exch{"H103 IEM"}="3H10 IEM"; $resn_exch{"H121 IEM"}="1H12 IEM";
$resn_exch{"H122 IEM"}="2H12 IEM"; $resn_exch{"H123 IEM"}="3H12 IEM"; $resn_exch{" H71 IET"}="1H7  IET";
$resn_exch{" H72 IET"}="2H7  IET"; $resn_exch{"HA31 IEY"}="1HA3 IEY"; $resn_exch{"HA32 IEY"}="2HA3 IEY";
$resn_exch{"HB21 IEY"}="1HB2 IEY"; $resn_exch{"HB22 IEY"}="2HB2 IEY"; $resn_exch{"HC61 IFG"}="1HC6 IFG";
$resn_exch{"HC62 IFG"}="2HC6 IFG"; $resn_exch{"HC81 IFG"}="1HC8 IFG"; $resn_exch{"HC82 IFG"}="2HC8 IFG";
$resn_exch{"HC83 IFG"}="3HC8 IFG"; $resn_exch{"HC91 IFG"}="1HC9 IFG"; $resn_exch{"HC92 IFG"}="2HC9 IFG";
$resn_exch{"H7C1 IFL"}="1H7C IFL"; $resn_exch{"H7C2 IFL"}="2H7C IFL"; $resn_exch{"H6C1 IFL"}="1H6C IFL";
$resn_exch{"H6C2 IFL"}="2H6C IFL"; $resn_exch{"H2C1 IFM"}="1H2C IFM"; $resn_exch{"H2C2 IFM"}="2H2C IFM";
$resn_exch{"H1C1 IFM"}="1H1C IFM"; $resn_exch{"H1C2 IFM"}="2H1C IFM"; $resn_exch{"H6C1 IFM"}="1H6C IFM";
$resn_exch{"H6C2 IFM"}="2H6C IFM"; $resn_exch{"H6A1 IFP"}="1H6A IFP"; $resn_exch{"H7A1 IFP"}="1H7A IFP";
$resn_exch{"H7A2 IFP"}="2H7A IFP"; $resn_exch{"HOP2  IG"}="2HOP  IG"; $resn_exch{"HOP3  IG"}="3HOP  IG";
$resn_exch{"HO3'  IG"}="H3T   IG"; $resn_exch{" H1'  IG"}=" H1*  IG"; $resn_exch{" H2'  IG"}=" H2*  IG";
$resn_exch{" H3'  IG"}=" H3*  IG"; $resn_exch{" H4'  IG"}=" H4*  IG"; $resn_exch{" H5'  IG"}="1H5*  IG";
$resn_exch{"H5''  IG"}="2H5*  IG"; $resn_exch{"HO2'  IG"}="2HO*  IG"; $resn_exch{" H61  IG"}="1H6   IG";
$resn_exch{" H62  IG"}="2H6   IG"; $resn_exch{"H11A IGC"}="1H1  IGC"; $resn_exch{"H12A IGC"}="2H1  IGC";
$resn_exch{"H13A IGC"}="3H1  IGC"; $resn_exch{" H21 IGC"}="1H2  IGC"; $resn_exch{"H22A IGC"}="2H2  IGC";
$resn_exch{"H31A IGC"}="1H3  IGC"; $resn_exch{"H32A IGC"}="2H3  IGC"; $resn_exch{" H41 IGC"}="1H4  IGC";
$resn_exch{" H42 IGC"}="2H4  IGC"; $resn_exch{" H51 IGC"}="1H5  IGC"; $resn_exch{" H52 IGC"}="2H5  IGC";
$resn_exch{" H61 IGC"}="1H6  IGC"; $resn_exch{" H62 IGC"}="2H6  IGC"; $resn_exch{" H71 IGC"}="1H7  IGC";
$resn_exch{" H72 IGC"}="2H7  IGC"; $resn_exch{" H81 IGC"}="1H8  IGC"; $resn_exch{" H82 IGC"}="2H8  IGC";
$resn_exch{" H91 IGC"}="1H9  IGC"; $resn_exch{" H92 IGC"}="2H9  IGC"; $resn_exch{"H101 IGC"}="1H10 IGC";
$resn_exch{"H102 IGC"}="2H10 IGC"; $resn_exch{"H111 IGC"}="1H11 IGC"; $resn_exch{"H112 IGC"}="2H11 IGC";
$resn_exch{"H121 IGC"}="1H12 IGC"; $resn_exch{"H122 IGC"}="2H12 IGC"; $resn_exch{"H131 IGC"}="1H13 IGC";
$resn_exch{"H132 IGC"}="2H13 IGC"; $resn_exch{"H181 IGC"}="1H18 IGC"; $resn_exch{"H182 IGC"}="2H18 IGC";
$resn_exch{"H211 IGC"}="1H21 IGC"; $resn_exch{"H212 IGC"}="2H21 IGC"; $resn_exch{"H251 IGC"}="1H25 IGC";
$resn_exch{"H252 IGC"}="2H25 IGC"; $resn_exch{"H301 IGC"}="1H30 IGC"; $resn_exch{"H302 IGC"}="2H30 IGC";
$resn_exch{"H381 IGC"}="1H38 IGC"; $resn_exch{"H382 IGC"}="2H38 IGC"; $resn_exch{"H391 IGC"}="1H39 IGC";
$resn_exch{"H392 IGC"}="2H39 IGC"; $resn_exch{"H401 IGC"}="1H40 IGC"; $resn_exch{"H402 IGC"}="2H40 IGC";
$resn_exch{"H411 IGC"}="1H41 IGC"; $resn_exch{"H412 IGC"}="2H41 IGC"; $resn_exch{"H421 IGC"}="1H42 IGC";
$resn_exch{"H422 IGC"}="2H42 IGC"; $resn_exch{"H431 IGC"}="1H43 IGC"; $resn_exch{"H432 IGC"}="2H43 IGC";
$resn_exch{"H441 IGC"}="1H44 IGC"; $resn_exch{"H442 IGC"}="2H44 IGC"; $resn_exch{"H443 IGC"}="3H44 IGC";
$resn_exch{" H   IGL"}="1HN  IGL"; $resn_exch{" H2  IGL"}="2HN  IGL"; $resn_exch{"HG12 IGL"}="1HG1 IGL";
$resn_exch{"HG13 IGL"}="2HG1 IGL"; $resn_exch{"HG22 IGL"}="1HG2 IGL"; $resn_exch{"HG23 IGL"}="2HG2 IGL";
$resn_exch{" H31 IGN"}="1H3  IGN"; $resn_exch{" H41 IGN"}="1H4  IGN"; $resn_exch{" H42 IGN"}="2H4  IGN";
$resn_exch{" H51 IGN"}="1H5  IGN"; $resn_exch{" H52 IGN"}="2H5  IGN"; $resn_exch{" H61 IGN"}="1H6  IGN";
$resn_exch{" H62 IGN"}="2H6  IGN"; $resn_exch{" H71 IGN"}="1H7  IGN"; $resn_exch{" H91 IGN"}="1H9  IGN";
$resn_exch{"H101 IGN"}="1H10 IGN"; $resn_exch{"H102 IGN"}="2H10 IGN"; $resn_exch{"H121 IGN"}="1H12 IGN";
$resn_exch{"H131 IGN"}="1H13 IGN"; $resn_exch{"H132 IGN"}="2H13 IGN"; $resn_exch{"H141 IGN"}="1H14 IGN";
$resn_exch{"H142 IGN"}="2H14 IGN"; $resn_exch{"H151 IGN"}="1H15 IGN"; $resn_exch{"H152 IGN"}="2H15 IGN";
$resn_exch{"H161 IGN"}="1H16 IGN"; $resn_exch{"H162 IGN"}="2H16 IGN"; $resn_exch{"H331 IGN"}="1H33 IGN";
$resn_exch{"H341 IGN"}="1H34 IGN"; $resn_exch{"H342 IGN"}="2H34 IGN"; $resn_exch{"H511 IGN"}="1H51 IGN";
$resn_exch{"H512 IGN"}="2H51 IGN"; $resn_exch{"H521 IGN"}="1H52 IGN"; $resn_exch{"H531 IGN"}="1H53 IGN";
$resn_exch{"H532 IGN"}="2H53 IGN"; $resn_exch{"H541 IGN"}="1H54 IGN"; $resn_exch{"H542 IGN"}="2H54 IGN";
$resn_exch{"H551 IGN"}="1H55 IGN"; $resn_exch{"H552 IGN"}="2H55 IGN"; $resn_exch{"H561 IGN"}="1H56 IGN";
$resn_exch{"H562 IGN"}="2H56 IGN"; $resn_exch{"H211 IGN"}="1H21 IGN"; $resn_exch{"H221 IGN"}="1H22 IGN";
$resn_exch{"H222 IGN"}="2H22 IGN"; $resn_exch{"H251 IGN"}="1H25 IGN"; $resn_exch{" H11 IGP"}="1H1  IGP";
$resn_exch{" H12 IGP"}="2H1  IGP"; $resn_exch{"HOP1 IGP"}="1HOP IGP"; $resn_exch{"HOP2 IGP"}="2HOP IGP";
$resn_exch{"HOP2 IGU"}="2HOP IGU"; $resn_exch{"HOP3 IGU"}="3HOP IGU"; $resn_exch{" H5' IGU"}="1H5* IGU";
$resn_exch{"H5'' IGU"}="2H5* IGU"; $resn_exch{" H4' IGU"}=" H4* IGU"; $resn_exch{" H3' IGU"}=" H3* IGU";
$resn_exch{"HO3' IGU"}="*HO3 IGU"; $resn_exch{" H2' IGU"}="1H2* IGU"; $resn_exch{"H2'' IGU"}="2H2* IGU";
$resn_exch{" H1' IGU"}=" H1* IGU"; $resn_exch{"HN61 IGU"}="1HN6 IGU"; $resn_exch{"HN62 IGU"}="2HN6 IGU";
$resn_exch{"H231 IH1"}="1H23 IH1"; $resn_exch{"H232 IH1"}="2H23 IH1"; $resn_exch{"H221 IH1"}="1H22 IH1";
$resn_exch{"H222 IH1"}="2H22 IH1"; $resn_exch{"H161 IH1"}="1H16 IH1"; $resn_exch{"H162 IH1"}="2H16 IH1";
$resn_exch{"HC11 IH1"}="1HC1 IH1"; $resn_exch{"HC12 IH1"}="2HC1 IH1"; $resn_exch{"HC31 IH1"}="1HC3 IH1";
$resn_exch{"HC32 IH1"}="2HC3 IH1"; $resn_exch{"HC41 IH1"}="1HC4 IH1"; $resn_exch{"HC42 IH1"}="2HC4 IH1";
$resn_exch{"HC61 IH1"}="1HC6 IH1"; $resn_exch{"HC62 IH1"}="2HC6 IH1"; $resn_exch{"HC71 IH1"}="1HC7 IH1";
$resn_exch{"HC72 IH1"}="2HC7 IH1"; $resn_exch{"H111 IH1"}="1H11 IH1"; $resn_exch{"H112 IH1"}="2H11 IH1";
$resn_exch{" H11 IH2"}="1H1  IH2"; $resn_exch{" H12 IH2"}="2H1  IH2"; $resn_exch{"H101 IH2"}="1H10 IH2";
$resn_exch{"H102 IH2"}="2H10 IH2"; $resn_exch{" H71 IH2"}="1H7  IH2"; $resn_exch{" H72 IH2"}="2H7  IH2";
$resn_exch{" H61 IH2"}="1H6  IH2"; $resn_exch{" H62 IH2"}="2H6  IH2"; $resn_exch{" H41 IH2"}="1H4  IH2";
$resn_exch{" H42 IH2"}="2H4  IH2"; $resn_exch{" H31 IH2"}="1H3  IH2"; $resn_exch{"H32A IH2"}="2H3  IH2";
$resn_exch{"H171 IH2"}="1H17 IH2"; $resn_exch{"H172 IH2"}="2H17 IH2"; $resn_exch{"H161 IH2"}="1H16 IH2";
$resn_exch{"H162 IH2"}="2H16 IH2"; $resn_exch{"H163 IH2"}="3H16 IH2"; $resn_exch{"H231 IH2"}="1H23 IH2";
$resn_exch{"H232 IH2"}="2H23 IH2"; $resn_exch{"HNB1 IH3"}="1HNB IH3"; $resn_exch{"HNB2 IH3"}="2HNB IH3";
$resn_exch{"HC71 IH3"}="1HC7 IH3"; $resn_exch{"HC72 IH3"}="2HC7 IH3"; $resn_exch{"HC91 IH3"}="1HC9 IH3";
$resn_exch{"HC92 IH3"}="2HC9 IH3"; $resn_exch{"HC41 IH3"}="1HC4 IH3"; $resn_exch{"HC42 IH3"}="2HC4 IH3";
$resn_exch{"HC61 IH3"}="1HC6 IH3"; $resn_exch{"HC62 IH3"}="2HC6 IH3"; $resn_exch{"H161 IH3"}="1H16 IH3";
$resn_exch{"H162 IH3"}="2H16 IH3"; $resn_exch{"H151 IH3"}="1H15 IH3"; $resn_exch{"H152 IH3"}="2H15 IH3";
$resn_exch{"H153 IH3"}="3H15 IH3"; $resn_exch{"H221 IH3"}="1H22 IH3"; $resn_exch{"H222 IH3"}="2H22 IH3";
$resn_exch{"H291 IH4"}="1H29 IH4"; $resn_exch{"H292 IH4"}="2H29 IH4"; $resn_exch{"H293 IH4"}="3H29 IH4";
$resn_exch{"H251 IH4"}="1H25 IH4"; $resn_exch{"H252 IH4"}="2H25 IH4"; $resn_exch{"H161 IH4"}="1H16 IH4";
$resn_exch{"H162 IH4"}="2H16 IH4"; $resn_exch{"H381 IH4"}="1H38 IH4"; $resn_exch{"H382 IH4"}="2H38 IH4";
$resn_exch{"H391 IH4"}="1H39 IH4"; $resn_exch{"H392 IH4"}="2H39 IH4"; $resn_exch{"H401 IH4"}="1H40 IH4";
$resn_exch{"H402 IH4"}="2H40 IH4"; $resn_exch{"H411 IH4"}="1H41 IH4"; $resn_exch{"H412 IH4"}="2H41 IH4";
$resn_exch{"H421 IH4"}="1H42 IH4"; $resn_exch{"H422 IH4"}="2H42 IH4"; $resn_exch{"H423 IH4"}="3H42 IH4";
$resn_exch{"H5C1 IH4"}="1H5C IH4"; $resn_exch{"H5C2 IH4"}="2H5C IH4"; $resn_exch{"H6C1 IH4"}="1H6C IH4";
$resn_exch{"H6C2 IH4"}="2H6C IH4"; $resn_exch{"H141 IH4"}="1H14 IH4"; $resn_exch{"H142 IH4"}="2H14 IH4";
$resn_exch{"H151 IH4"}="1H15 IH4"; $resn_exch{"H152 IH4"}="2H15 IH4"; $resn_exch{"H8C1 IH4"}="1H8C IH4";
$resn_exch{"H8C2 IH4"}="2H8C IH4"; $resn_exch{"H131 IH5"}="1H13 IH5"; $resn_exch{"H132 IH5"}="2H13 IH5";
$resn_exch{"H161 IH5"}="1H16 IH5"; $resn_exch{"H162 IH5"}="2H16 IH5"; $resn_exch{"H163 IH5"}="3H16 IH5";
$resn_exch{"H181 IH5"}="1H18 IH5"; $resn_exch{"H182 IH5"}="2H18 IH5"; $resn_exch{"H183 IH5"}="3H18 IH5";
$resn_exch{"HL51 IH5"}="1HL5 IH5"; $resn_exch{"HL52 IH5"}="2HL5 IH5"; $resn_exch{"HL53 IH5"}="3HL5 IH5";
$resn_exch{"HL61 IH5"}="1HL6 IH5"; $resn_exch{"HL62 IH5"}="2HL6 IH5"; $resn_exch{"HL63 IH5"}="3HL6 IH5";
$resn_exch{" H11 IH6"}="1H1  IH6"; $resn_exch{" H12 IH6"}="2H1  IH6"; $resn_exch{" H13 IH6"}="3H1  IH6";
$resn_exch{" H41 IH6"}="1H4  IH6"; $resn_exch{" H42 IH6"}="2H4  IH6"; $resn_exch{"H111 IH6"}="1H11 IH6";
$resn_exch{"H112 IH6"}="2H11 IH6"; $resn_exch{"H121 IH6"}="1H12 IH6"; $resn_exch{"H122 IH6"}="2H12 IH6";
$resn_exch{"H161 IH6"}="1H16 IH6"; $resn_exch{"H162 IH6"}="2H16 IH6"; $resn_exch{"H163 IH6"}="3H16 IH6";
$resn_exch{"H171 IH6"}="1H17 IH6"; $resn_exch{"H172 IH6"}="2H17 IH6"; $resn_exch{"H173 IH6"}="3H17 IH6";
$resn_exch{"H171 IHD"}="1H17 IHD"; $resn_exch{"H172 IHD"}="2H17 IHD"; $resn_exch{"H173 IHD"}="3H17 IHD";
$resn_exch{"H161 IHD"}="1H16 IHD"; $resn_exch{"H162 IHD"}="2H16 IHD"; $resn_exch{"H151 IHD"}="1H15 IHD";
$resn_exch{"H152 IHD"}="2H15 IHD"; $resn_exch{"H141 IHD"}="1H14 IHD"; $resn_exch{"H142 IHD"}="2H14 IHD";
$resn_exch{"H131 IHD"}="1H13 IHD"; $resn_exch{"H132 IHD"}="2H13 IHD"; $resn_exch{"H121 IHD"}="1H12 IHD";
$resn_exch{"H122 IHD"}="2H12 IHD"; $resn_exch{"H111 IHD"}="1H11 IHD"; $resn_exch{"H112 IHD"}="2H11 IHD";
$resn_exch{"H101 IHD"}="1H10 IHD"; $resn_exch{"H102 IHD"}="2H10 IHD"; $resn_exch{" H91 IHD"}="1H9  IHD";
$resn_exch{" H92 IHD"}="2H9  IHD"; $resn_exch{" H81 IHD"}="1H8  IHD"; $resn_exch{" H82 IHD"}="2H8  IHD";
$resn_exch{" H71 IHD"}="1H7  IHD"; $resn_exch{" H72 IHD"}="2H7  IHD"; $resn_exch{" H61 IHD"}="1H6  IHD";
$resn_exch{" H62 IHD"}="2H6  IHD"; $resn_exch{" H51 IHD"}="1H5  IHD"; $resn_exch{" H52 IHD"}="2H5  IHD";
$resn_exch{" H41 IHD"}="1H4  IHD"; $resn_exch{" H42 IHD"}="2H4  IHD"; $resn_exch{" H31 IHD"}="1H3  IHD";
$resn_exch{" H32 IHD"}="2H3  IHD"; $resn_exch{" H21 IHD"}="1H2  IHD"; $resn_exch{" H22 IHD"}="2H2  IHD";
$resn_exch{"H011 IHE"}="1H01 IHE"; $resn_exch{"H012 IHE"}="2H01 IHE"; $resn_exch{"H021 IHE"}="1H02 IHE";
$resn_exch{"H022 IHE"}="2H02 IHE"; $resn_exch{"H041 IHE"}="1H04 IHE"; $resn_exch{"H042 IHE"}="2H04 IHE";
$resn_exch{"H051 IHE"}="1H05 IHE"; $resn_exch{"H052 IHE"}="2H05 IHE"; $resn_exch{"H151 IHE"}="1H15 IHE";
$resn_exch{"H152 IHE"}="2H15 IHE"; $resn_exch{"H153 IHE"}="3H15 IHE"; $resn_exch{"H161 IHE"}="1H16 IHE";
$resn_exch{"H162 IHE"}="2H16 IHE"; $resn_exch{"H171 IHE"}="1H17 IHE"; $resn_exch{"H172 IHE"}="2H17 IHE";
$resn_exch{"H301 IHE"}="1H30 IHE"; $resn_exch{"H302 IHE"}="2H30 IHE"; $resn_exch{"H311 IHE"}="1H31 IHE";
$resn_exch{"H312 IHE"}="2H31 IHE"; $resn_exch{"H321 IHE"}="1H32 IHE"; $resn_exch{"H322 IHE"}="2H32 IHE";
$resn_exch{"H331 IHE"}="1H33 IHE"; $resn_exch{"H332 IHE"}="2H33 IHE"; $resn_exch{"H341 IHE"}="1H34 IHE";
$resn_exch{"H342 IHE"}="2H34 IHE"; $resn_exch{" H11 IHG"}="1H1  IHG"; $resn_exch{" H12 IHG"}="2H1  IHG";
$resn_exch{" H13 IHG"}="3H1  IHG"; $resn_exch{" H21 IHG"}="1H2  IHG"; $resn_exch{" H31 IHG"}="1H3  IHG";
$resn_exch{" H32 IHG"}="2H3  IHG"; $resn_exch{" H33 IHG"}="3H3  IHG"; $resn_exch{" HE1 IHG"}="1HE  IHG";
$resn_exch{"HH11 IHG"}="1HH1 IHG"; $resn_exch{"HH21 IHG"}="1HH2 IHG"; $resn_exch{" HH1 IHG"}="1HH  IHG";
$resn_exch{"H131 IHI"}="1H13 IHI"; $resn_exch{"H132 IHI"}="2H13 IHI"; $resn_exch{"H141 IHI"}="1H14 IHI";
$resn_exch{"H142 IHI"}="2H14 IHI"; $resn_exch{"H151 IHI"}="1H15 IHI"; $resn_exch{"H152 IHI"}="2H15 IHI";
$resn_exch{"H161 IHI"}="1H16 IHI"; $resn_exch{"H162 IHI"}="2H16 IHI"; $resn_exch{"H391 IHI"}="1H39 IHI";
$resn_exch{"H392 IHI"}="2H39 IHI"; $resn_exch{"H401 IHI"}="1H40 IHI"; $resn_exch{"H402 IHI"}="2H40 IHI";
$resn_exch{"H411 IHI"}="1H41 IHI"; $resn_exch{"H412 IHI"}="2H41 IHI"; $resn_exch{"H171 IHJ"}="1H17 IHJ";
$resn_exch{"H172 IHJ"}="2H17 IHJ"; $resn_exch{"H181 IHJ"}="1H18 IHJ"; $resn_exch{"H182 IHJ"}="2H18 IHJ";
$resn_exch{"H191 IHJ"}="1H19 IHJ"; $resn_exch{"H192 IHJ"}="2H19 IHJ"; $resn_exch{"H201 IHJ"}="1H20 IHJ";
$resn_exch{"H202 IHJ"}="2H20 IHJ"; $resn_exch{"H411 IHJ"}="1H41 IHJ"; $resn_exch{"H412 IHJ"}="2H41 IHJ";
$resn_exch{"H421 IHJ"}="1H42 IHJ"; $resn_exch{"H422 IHJ"}="2H42 IHJ"; $resn_exch{"H431 IHJ"}="1H43 IHJ";
$resn_exch{"H432 IHJ"}="2H43 IHJ"; $resn_exch{"H521 IHJ"}="1H52 IHJ"; $resn_exch{"H522 IHJ"}="2H52 IHJ";
$resn_exch{"H531 IHJ"}="1H53 IHJ"; $resn_exch{"H532 IHJ"}="2H53 IHJ"; $resn_exch{"H551 IHJ"}="1H55 IHJ";
$resn_exch{"H552 IHJ"}="2H55 IHJ"; $resn_exch{"H561 IHJ"}="1H56 IHJ"; $resn_exch{"H562 IHJ"}="2H56 IHJ";
$resn_exch{"H611 IHJ"}="1H61 IHJ"; $resn_exch{"H612 IHJ"}="2H61 IHJ"; $resn_exch{"H613 IHJ"}="3H61 IHJ";
$resn_exch{" H21 IHN"}="1H2  IHN"; $resn_exch{" H22 IHN"}="2H2  IHN"; $resn_exch{"H101 IHN"}="1H10 IHN";
$resn_exch{"H102 IHN"}="2H10 IHN"; $resn_exch{"H131 IHN"}="1H13 IHN"; $resn_exch{"H132 IHN"}="2H13 IHN";
$resn_exch{"H221 IHN"}="1H22 IHN"; $resn_exch{"H222 IHN"}="2H22 IHN"; $resn_exch{"H241 IHN"}="1H24 IHN";
$resn_exch{"H242 IHN"}="2H24 IHN"; $resn_exch{"H243 IHN"}="3H24 IHN"; $resn_exch{"H251 IHN"}="1H25 IHN";
$resn_exch{"H252 IHN"}="2H25 IHN"; $resn_exch{"H271 IHN"}="1H27 IHN"; $resn_exch{"H272 IHN"}="2H27 IHN";
$resn_exch{"H273 IHN"}="3H27 IHN"; $resn_exch{"H281 IHN"}="1H28 IHN"; $resn_exch{"H282 IHN"}="2H28 IHN";
$resn_exch{"H283 IHN"}="3H28 IHN"; $resn_exch{"H291 IHN"}="1H29 IHN"; $resn_exch{"H292 IHN"}="2H29 IHN";
$resn_exch{"H293 IHN"}="3H29 IHN"; $resn_exch{"H311 IHN"}="1H31 IHN"; $resn_exch{"H312 IHN"}="2H31 IHN";
$resn_exch{"H313 IHN"}="3H31 IHN"; $resn_exch{"H321 IHN"}="1H32 IHN"; $resn_exch{"H322 IHN"}="2H32 IHN";
$resn_exch{"H323 IHN"}="3H32 IHN"; $resn_exch{"H341 IHN"}="1H34 IHN"; $resn_exch{"H342 IHN"}="2H34 IHN";
$resn_exch{"H343 IHN"}="3H34 IHN"; $resn_exch{"H351 IHN"}="1H35 IHN"; $resn_exch{"H352 IHN"}="2H35 IHN";
$resn_exch{"H353 IHN"}="3H35 IHN"; $resn_exch{"H371 IHN"}="1H37 IHN"; $resn_exch{"H372 IHN"}="2H37 IHN";
$resn_exch{"H373 IHN"}="3H37 IHN"; $resn_exch{"H381 IHN"}="1H38 IHN"; $resn_exch{"H382 IHN"}="2H38 IHN";
$resn_exch{"H383 IHN"}="3H38 IHN"; $resn_exch{"H391 IHN"}="1H39 IHN"; $resn_exch{"H392 IHN"}="2H39 IHN";
$resn_exch{"H393 IHN"}="3H39 IHN"; $resn_exch{"HO42 IHS"}="2HO4 IHS"; $resn_exch{"HO43 IHS"}="3HO4 IHS";
$resn_exch{"HO44 IHS"}="4HO4 IHS"; $resn_exch{"HO45 IHS"}="5HO4 IHS"; $resn_exch{"HO46 IHS"}="6HO4 IHS";
$resn_exch{"H511 IHU"}="1H51 IHU"; $resn_exch{"H512 IHU"}="2H51 IHU"; $resn_exch{"H513 IHU"}="3H51 IHU";
$resn_exch{"H131 IIA"}="1H13 IIA"; $resn_exch{"H132 IIA"}="2H13 IIA"; $resn_exch{"H141 IIA"}="1H14 IIA";
$resn_exch{"H142 IIA"}="2H14 IIA"; $resn_exch{"H161 IIA"}="1H16 IIA"; $resn_exch{"H162 IIA"}="2H16 IIA";
$resn_exch{"H171 IIA"}="1H17 IIA"; $resn_exch{"H172 IIA"}="2H17 IIA"; $resn_exch{"H281 IIA"}="1H28 IIA";
$resn_exch{"H282 IIA"}="2H28 IIA"; $resn_exch{"H283 IIA"}="3H28 IIA"; $resn_exch{"H321 IIA"}="1H32 IIA";
$resn_exch{"H322 IIA"}="2H32 IIA"; $resn_exch{"H323 IIA"}="3H32 IIA"; $resn_exch{"H371 IIA"}="1H37 IIA";
$resn_exch{"H372 IIA"}="2H37 IIA"; $resn_exch{"H131 IIB"}="1H13 IIB"; $resn_exch{"H132 IIB"}="2H13 IIB";
$resn_exch{"H141 IIB"}="1H14 IIB"; $resn_exch{"H142 IIB"}="2H14 IIB"; $resn_exch{"H161 IIB"}="1H16 IIB";
$resn_exch{"H162 IIB"}="2H16 IIB"; $resn_exch{"H171 IIB"}="1H17 IIB"; $resn_exch{"H172 IIB"}="2H17 IIB";
$resn_exch{"H281 IIB"}="1H28 IIB"; $resn_exch{"H282 IIB"}="2H28 IIB"; $resn_exch{"H283 IIB"}="3H28 IIB";
$resn_exch{"H321 IIB"}="1H32 IIB"; $resn_exch{"H322 IIB"}="2H32 IIB"; $resn_exch{"H323 IIB"}="3H32 IIB";
$resn_exch{"H371 IIB"}="1H37 IIB"; $resn_exch{"H372 IIB"}="2H37 IIB"; $resn_exch{"H111 IIB"}="1H11 IIB";
$resn_exch{"H112 IIB"}="2H11 IIB"; $resn_exch{"H113 IIB"}="3H11 IIB"; $resn_exch{"HB21 IIC"}="H21  IIC";
$resn_exch{"H141 IID"}="1H14 IID"; $resn_exch{"H142 IID"}="2H14 IID"; $resn_exch{"H171 IID"}="1H17 IID";
$resn_exch{"H172 IID"}="2H17 IID"; $resn_exch{"H261 IID"}="1H26 IID"; $resn_exch{"H262 IID"}="2H26 IID";
$resn_exch{"H7C1 IID"}="1H7C IID"; $resn_exch{"H7C2 IID"}="2H7C IID"; $resn_exch{"H7C3 IID"}="3H7C IID";
$resn_exch{"H101 IID"}="1H10 IID"; $resn_exch{"H102 IID"}="2H10 IID"; $resn_exch{"H121 IID"}="1H12 IID";
$resn_exch{"H122 IID"}="2H12 IID"; $resn_exch{"H191 IID"}="1H19 IID"; $resn_exch{"H192 IID"}="2H19 IID";
$resn_exch{"H193 IID"}="3H19 IID"; $resn_exch{"H201 IID"}="1H20 IID"; $resn_exch{"H202 IID"}="2H20 IID";
$resn_exch{"H203 IID"}="3H20 IID"; $resn_exch{"H131 IIE"}="1H13 IIE"; $resn_exch{"H132 IIE"}="2H13 IIE";
$resn_exch{"H141 IIE"}="1H14 IIE"; $resn_exch{"H142 IIE"}="2H14 IIE"; $resn_exch{"H161 IIE"}="1H16 IIE";
$resn_exch{"H162 IIE"}="2H16 IIE"; $resn_exch{"H171 IIE"}="1H17 IIE"; $resn_exch{"H172 IIE"}="2H17 IIE";
$resn_exch{"H281 IIE"}="1H28 IIE"; $resn_exch{"H282 IIE"}="2H28 IIE"; $resn_exch{"H283 IIE"}="3H28 IIE";
$resn_exch{"H321 IIE"}="1H32 IIE"; $resn_exch{"H322 IIE"}="2H32 IIE"; $resn_exch{"H323 IIE"}="3H32 IIE";
$resn_exch{"H371 IIE"}="1H37 IIE"; $resn_exch{"H372 IIE"}="2H37 IIE"; $resn_exch{" H2  IIL"}=" HN2 IIL";
$resn_exch{"HG21 IIL"}="1HG2 IIL"; $resn_exch{"HG22 IIL"}="2HG2 IIL"; $resn_exch{"HG23 IIL"}="3HG2 IIL";
$resn_exch{"HG12 IIL"}="1HG1 IIL"; $resn_exch{"HG13 IIL"}="2HG1 IIL"; $resn_exch{"HD11 IIL"}="1HD1 IIL";
$resn_exch{"HD12 IIL"}="2HD1 IIL"; $resn_exch{"HD13 IIL"}="3HD1 IIL"; $resn_exch{"HM21 IIN"}="1HM2 IIN";
$resn_exch{"HM22 IIN"}="2HM2 IIN"; $resn_exch{"HM23 IIN"}="3HM2 IIN"; $resn_exch{"HN31 IIN"}="1HN3 IIN";
$resn_exch{"HN32 IIN"}="2HN3 IIN"; $resn_exch{" H31 IIP"}="1H3  IIP"; $resn_exch{" H32 IIP"}="2H3  IIP";
$resn_exch{" H33 IIP"}="3H3  IIP"; $resn_exch{" H91 IIP"}="1H9  IIP"; $resn_exch{" H92 IIP"}="2H9  IIP";
$resn_exch{" H93 IIP"}="3H9  IIP"; $resn_exch{"H151 IIP"}="1H15 IIP"; $resn_exch{"H152 IIP"}="2H15 IIP";
$resn_exch{"H153 IIP"}="3H15 IIP"; $resn_exch{"H181 IIP"}="1H18 IIP"; $resn_exch{"H182 IIP"}="2H18 IIP";
$resn_exch{"H191 IIP"}="1H19 IIP"; $resn_exch{"H192 IIP"}="2H19 IIP"; $resn_exch{"H201 IIP"}="1H20 IIP";
$resn_exch{"H202 IIP"}="2H20 IIP"; $resn_exch{"H211 IIP"}="1H21 IIP"; $resn_exch{"H212 IIP"}="2H21 IIP";
$resn_exch{"H213 IIP"}="3H21 IIP"; $resn_exch{"H221 IIP"}="1H22 IIP"; $resn_exch{"H222 IIP"}="2H22 IIP";
$resn_exch{"H223 IIP"}="3H22 IIP"; $resn_exch{"H281 IIP"}="1H28 IIP"; $resn_exch{"H282 IIP"}="2H28 IIP";
$resn_exch{"H283 IIP"}="3H28 IIP"; $resn_exch{"H291 IIP"}="1H29 IIP"; $resn_exch{"H292 IIP"}="2H29 IIP";
$resn_exch{"H301 IIP"}="1H30 IIP"; $resn_exch{"H302 IIP"}="2H30 IIP"; $resn_exch{"H2A1 IK2"}="1H2A IK2";
$resn_exch{"H2A2 IK2"}="2H2A IK2"; $resn_exch{"H2A3 IK2"}="3H2A IK2"; $resn_exch{"H4A1 IK2"}="1H4A IK2";
$resn_exch{"H4A2 IK2"}="2H4A IK2"; $resn_exch{"H1'1 IK2"}="1H1' IK2"; $resn_exch{"H1'2 IK2"}="2H1' IK2";
$resn_exch{"H5A1 IK2"}="1H5A IK2"; $resn_exch{"H5A2 IK2"}="2H5A IK2"; $resn_exch{"HOP2 IK2"}="2HOP IK2";
$resn_exch{"HOP3 IK2"}="3HOP IK2"; $resn_exch{"HN11 IK8"}="1HN1 IK8"; $resn_exch{"HN12 IK8"}="2HN1 IK8";
$resn_exch{"H111 IK8"}="1H11 IK8"; $resn_exch{"H112 IK8"}="2H11 IK8"; $resn_exch{"H201 IK8"}="1H20 IK8";
$resn_exch{"H202 IK8"}="2H20 IK8"; $resn_exch{"H203 IK8"}="3H20 IK8"; $resn_exch{"H231 IK8"}="1H23 IK8";
$resn_exch{"H232 IK8"}="2H23 IK8"; $resn_exch{"H233 IK8"}="3H23 IK8"; $resn_exch{" H21 IKT"}="1H2  IKT";
$resn_exch{" H22 IKT"}="2H2  IKT"; $resn_exch{" H41 IKT"}="1H4  IKT"; $resn_exch{" H42 IKT"}="2H4  IKT";
$resn_exch{" H51 IKT"}="1H5  IKT"; $resn_exch{" H52 IKT"}="2H5  IKT"; $resn_exch{" H61 IKT"}="1H6  IKT";
$resn_exch{" H62 IKT"}="2H6  IKT"; $resn_exch{" H71 IKT"}="1H7  IKT"; $resn_exch{" H72 IKT"}="2H7  IKT";
$resn_exch{" H81 IKT"}="1H8  IKT"; $resn_exch{" H82 IKT"}="2H8  IKT"; $resn_exch{"H2'1 IKT"}="1H2' IKT";
$resn_exch{"H2'2 IKT"}="2H2' IKT"; $resn_exch{"H2'3 IKT"}="3H2' IKT"; $resn_exch{"HN11 IKT"}="1HN1 IKT";
$resn_exch{"HN12 IKT"}="2HN1 IKT"; $resn_exch{" HN1 ILA"}="1HN  ILA"; $resn_exch{" HN2 ILA"}="2HN  ILA";
$resn_exch{" HA1 ILA"}="1HA  ILA"; $resn_exch{"HG21 ILA"}="1HG2 ILA"; $resn_exch{"HG22 ILA"}="2HG2 ILA";
$resn_exch{"HG23 ILA"}="3HG2 ILA"; $resn_exch{"HG11 ILA"}="1HG1 ILA"; $resn_exch{"HG12 ILA"}="2HG1 ILA";
$resn_exch{" HD1 ILA"}="1HD  ILA"; $resn_exch{" HD2 ILA"}="2HD  ILA"; $resn_exch{" HD3 ILA"}="3HD  ILA";
$resn_exch{" H51 ILA"}="1H5  ILA"; $resn_exch{" H52 ILA"}="2H5  ILA"; $resn_exch{" H61 ILA"}="1H6  ILA";
$resn_exch{" H62 ILA"}="2H6  ILA"; $resn_exch{"H201 ILB"}="1H20 ILB"; $resn_exch{"H202 ILB"}="2H20 ILB";
$resn_exch{"H211 ILB"}="1H21 ILB"; $resn_exch{"H212 ILB"}="2H21 ILB"; $resn_exch{"H221 ILB"}="1H22 ILB";
$resn_exch{"H222 ILB"}="2H22 ILB"; $resn_exch{" H41 ILC"}="1H4  ILC"; $resn_exch{" H42 ILC"}="2H4  ILC";
$resn_exch{" H43 ILC"}="3H4  ILC"; $resn_exch{" H2  ILE"}=" HN2 ILE"; $resn_exch{"HG12 ILE"}="1HG1 ILE";
$resn_exch{"HG13 ILE"}="2HG1 ILE"; $resn_exch{"HG21 ILE"}="1HG2 ILE"; $resn_exch{"HG22 ILE"}="2HG2 ILE";
$resn_exch{"HG23 ILE"}="3HG2 ILE"; $resn_exch{"HD11 ILE"}="1HD1 ILE"; $resn_exch{"HD12 ILE"}="2HD1 ILE";
$resn_exch{"HD13 ILE"}="3HD1 ILE"; $resn_exch{" H   ILG"}="1HN  ILG"; $resn_exch{" H2  ILG"}="2HN  ILG";
$resn_exch{" H3  ILG"}="3HN  ILG"; $resn_exch{" HB2 ILG"}="1HB  ILG"; $resn_exch{" HB3 ILG"}="2HB  ILG";
$resn_exch{" HG2 ILG"}="1HG  ILG"; $resn_exch{" HG3 ILG"}="2HG  ILG"; $resn_exch{" H61 ILH"}="1H6  ILH";
$resn_exch{" H62 ILH"}="2H6  ILH"; $resn_exch{"H261 ILI"}="1H26 ILI"; $resn_exch{"H262 ILI"}="2H26 ILI";
$resn_exch{"H263 ILI"}="3H26 ILI"; $resn_exch{"H111 ILI"}="1H11 ILI"; $resn_exch{"H112 ILI"}="2H11 ILI";
$resn_exch{"H181 ILI"}="1H18 ILI"; $resn_exch{"H182 ILI"}="2H18 ILI"; $resn_exch{"H183 ILI"}="3H18 ILI";
$resn_exch{"H231 ILI"}="1H23 ILI"; $resn_exch{"H232 ILI"}="2H23 ILI"; $resn_exch{"H251 ILI"}="1H25 ILI";
$resn_exch{"H252 ILI"}="2H25 ILI"; $resn_exch{"H253 ILI"}="3H25 ILI"; $resn_exch{" HN1 ILO"}="1HN  ILO";
$resn_exch{"HN2A ILO"}="2HN  ILO"; $resn_exch{"HCB1 ILO"}="1HCB ILO"; $resn_exch{"HCB2 ILO"}="2HCB ILO";
$resn_exch{"HCG1 ILO"}="1HCG ILO"; $resn_exch{"HCG2 ILO"}="2HCG ILO"; $resn_exch{"HCD1 ILO"}="1HCD ILO";
$resn_exch{"HCD2 ILO"}="2HCD ILO"; $resn_exch{"HC11 ILO"}="1HC1 ILO"; $resn_exch{"HC12 ILO"}="2HC1 ILO";
$resn_exch{"HC13 ILO"}="3HC1 ILO"; $resn_exch{"HOP2 ILP"}="2HOP ILP"; $resn_exch{"HOP3 ILP"}="3HOP ILP";
$resn_exch{"H5A1 ILP"}="1H5A ILP"; $resn_exch{"H5A2 ILP"}="2H5A ILP"; $resn_exch{"H2A1 ILP"}="1H2A ILP";
$resn_exch{"H2A2 ILP"}="2H2A ILP"; $resn_exch{"H2A3 ILP"}="3H2A ILP"; $resn_exch{"H4A1 ILP"}="1H4A ILP";
$resn_exch{"H4A2 ILP"}="2H4A ILP"; $resn_exch{"HG21 ILP"}="1HG2 ILP"; $resn_exch{"HG22 ILP"}="2HG2 ILP";
$resn_exch{"HG23 ILP"}="3HG2 ILP"; $resn_exch{"HG11 ILP"}="1HG1 ILP"; $resn_exch{"HG12 ILP"}="2HG1 ILP";
$resn_exch{"HD11 ILP"}="1HD1 ILP"; $resn_exch{"HD12 ILP"}="2HD1 ILP"; $resn_exch{"HD13 ILP"}="3HD1 ILP";
$resn_exch{"HN11 ILT"}="1HN1 ILT"; $resn_exch{"HN12 ILT"}="2HN1 ILT"; $resn_exch{"HN21 ILT"}="1HN2 ILT";
$resn_exch{"HN22 ILT"}="2HN2 ILT"; $resn_exch{" H21 ILT"}="1H2  ILT"; $resn_exch{" H22 ILT"}="2H2  ILT";
$resn_exch{" H81 ILT"}="1H8  ILT"; $resn_exch{" H82 ILT"}="2H8  ILT"; $resn_exch{" H83 ILT"}="3H8  ILT";
$resn_exch{"H141 ILT"}="1H14 ILT"; $resn_exch{"H142 ILT"}="2H14 ILT"; $resn_exch{"H143 ILT"}="3H14 ILT";
$resn_exch{"H161 ILT"}="1H16 ILT"; $resn_exch{"H162 ILT"}="2H16 ILT"; $resn_exch{"H171 ILT"}="1H17 ILT";
$resn_exch{"H172 ILT"}="2H17 ILT"; $resn_exch{"HN91 ILT"}="1HN9 ILT"; $resn_exch{"HN92 ILT"}="2HN9 ILT";
$resn_exch{"HN01 ILT"}="1HN0 ILT"; $resn_exch{"HN02 ILT"}="2HN0 ILT"; $resn_exch{" H2  ILX"}=" HN2 ILX";
$resn_exch{"HG11 ILX"}="1HG1 ILX"; $resn_exch{"HG21 ILX"}="1HG2 ILX"; $resn_exch{"HG22 ILX"}="2HG2 ILX";
$resn_exch{"HG23 ILX"}="3HG2 ILX"; $resn_exch{"HD12 ILX"}="1HD1 ILX"; $resn_exch{"HD13 ILX"}="2HD1 ILX";
$resn_exch{" H71 IM2"}="1H7  IM2"; $resn_exch{"H61A IM2"}="1H6  IM2"; $resn_exch{" H51 IM2"}="1H5  IM2";
$resn_exch{"H621 IM2"}="1H62 IM2"; $resn_exch{"H622 IM2"}="2H62 IM2"; $resn_exch{"H623 IM2"}="3H62 IM2";
$resn_exch{" H41 IM2"}="1H4  IM2"; $resn_exch{"H321 IM2"}="1H32 IM2"; $resn_exch{"H221 IM2"}="1H22 IM2";
$resn_exch{"H222 IM2"}="2H22 IM2"; $resn_exch{"H231 IM2"}="1H23 IM2"; $resn_exch{"H232 IM2"}="2H23 IM2";
$resn_exch{"H241 IM2"}="1H24 IM2"; $resn_exch{" H11 IM2"}="1H1  IM2"; $resn_exch{" H12 IM2"}="2H1  IM2";
$resn_exch{" HB1 IM3"}="1HB  IM3"; $resn_exch{" HB2 IM3"}="2HB  IM3"; $resn_exch{" H61 IM8"}="1H6  IM8";
$resn_exch{" H62 IM8"}="2H6  IM8"; $resn_exch{" H63 IM8"}="3H6  IM8"; $resn_exch{"H161 IM8"}="1H16 IM8";
$resn_exch{"H162 IM8"}="2H16 IM8"; $resn_exch{"H163 IM8"}="3H16 IM8"; $resn_exch{"H171 IM8"}="1H17 IM8";
$resn_exch{"H172 IM8"}="2H17 IM8"; $resn_exch{"H201 IM8"}="1H20 IM8"; $resn_exch{"H202 IM8"}="2H20 IM8";
$resn_exch{"H211 IM8"}="1H21 IM8"; $resn_exch{"H212 IM8"}="2H21 IM8"; $resn_exch{"H221 IM8"}="1H22 IM8";
$resn_exch{"H222 IM8"}="2H22 IM8"; $resn_exch{"H223 IM8"}="3H22 IM8"; $resn_exch{"H201 IMA"}="1H20 IMA";
$resn_exch{"H202 IMA"}="2H20 IMA"; $resn_exch{"H231 IMA"}="1H23 IMA"; $resn_exch{"H232 IMA"}="2H23 IMA";
$resn_exch{"H371 IMA"}="1H37 IMA"; $resn_exch{"H372 IMA"}="2H37 IMA"; $resn_exch{"H451 IMA"}="1H45 IMA";
$resn_exch{"H452 IMA"}="2H45 IMA"; $resn_exch{"H453 IMA"}="3H45 IMA"; $resn_exch{"H491 IMA"}="1H49 IMA";
$resn_exch{"H492 IMA"}="2H49 IMA"; $resn_exch{"H493 IMA"}="3H49 IMA"; $resn_exch{"H531 IMA"}="1H53 IMA";
$resn_exch{"H532 IMA"}="2H53 IMA"; $resn_exch{"H533 IMA"}="3H53 IMA"; $resn_exch{"HC11 IMA"}="1HC1 IMA";
$resn_exch{"HC12 IMA"}="2HC1 IMA"; $resn_exch{"HOP2 IMC"}="2HOP IMC"; $resn_exch{"HOP3 IMC"}="3HOP IMC";
$resn_exch{" H5' IMC"}="1H5* IMC"; $resn_exch{"H5'' IMC"}="2H5* IMC"; $resn_exch{" H4' IMC"}=" H4* IMC";
$resn_exch{" H1' IMC"}=" H1* IMC"; $resn_exch{"HN21 IMC"}="1HN2 IMC"; $resn_exch{"HN22 IMC"}="2HN2 IMC";
$resn_exch{" H71 IMC"}="1H5M IMC"; $resn_exch{" H72 IMC"}="2H5M IMC"; $resn_exch{" H73 IMC"}="3H5M IMC";
$resn_exch{" H2' IMC"}="1H2* IMC"; $resn_exch{"H2'' IMC"}="2H2* IMC"; $resn_exch{" H3' IMC"}=" H3* IMC";
$resn_exch{"HO3' IMC"}="*HO3 IMC"; $resn_exch{" HG1 IME"}="1HG  IME"; $resn_exch{"HD21 IME"}="1HD2 IME";
$resn_exch{"HE11 IME"}="1HE1 IME"; $resn_exch{" HG2 IME"}="2HG  IME"; $resn_exch{"HD22 IME"}="2HD2 IME";
$resn_exch{"HE12 IME"}="2HE1 IME"; $resn_exch{" HG3 IME"}="3HG  IME"; $resn_exch{"HD23 IME"}="3HD2 IME";
$resn_exch{"HE13 IME"}="3HE1 IME"; $resn_exch{" HG4 IME"}="4HG  IME"; $resn_exch{"HD24 IME"}="4HD2 IME";
$resn_exch{"HE14 IME"}="4HE1 IME"; $resn_exch{"HO11 IMF"}="1HO1 IMF"; $resn_exch{"HO12 IMF"}="2HO1 IMF";
$resn_exch{"HO21 IMF"}="1HO2 IMF"; $resn_exch{"HO22 IMF"}="2HO2 IMF"; $resn_exch{" HG1 IMF"}="1HG  IMF";
$resn_exch{"HD21 IMF"}="1HD2 IMF"; $resn_exch{"HD11 IMF"}="1HD1 IMF"; $resn_exch{"H1E2 IMF"}="2H1E IMF";
$resn_exch{" HG2 IMF"}="2HG  IMF"; $resn_exch{"HD22 IMF"}="2HD2 IMF"; $resn_exch{"HD12 IMF"}="2HD1 IMF";
$resn_exch{"H2E1 IMF"}="1H2E IMF"; $resn_exch{" HG3 IMF"}="3HG  IMF"; $resn_exch{"HD23 IMF"}="3HD2 IMF";
$resn_exch{"HD13 IMF"}="3HD1 IMF"; $resn_exch{" HG4 IMF"}="4HG  IMF"; $resn_exch{"HD24 IMF"}="4HD2 IMF";
$resn_exch{"HD14 IMF"}="4HD1 IMF"; $resn_exch{"H4E1 IMF"}="1H4E IMF"; $resn_exch{"HO'5 IMG"}="5HO* IMG";
$resn_exch{"H5'1 IMG"}="1H5* IMG"; $resn_exch{"H5'2 IMG"}="2H5* IMG"; $resn_exch{" H4' IMG"}=" H4* IMG";
$resn_exch{"HN'4 IMG"}="4HN* IMG"; $resn_exch{" H3' IMG"}=" H3* IMG"; $resn_exch{"HO'3 IMG"}="3HO* IMG";
$resn_exch{" H2' IMG"}=" H2* IMG"; $resn_exch{"HO'2 IMG"}="2HO* IMG"; $resn_exch{" H1' IMG"}=" H1* IMG";
$resn_exch{" H21 IMG"}="1H2  IMG"; $resn_exch{" H22 IMG"}="2H2  IMG"; $resn_exch{"HO'5 IMH"}="5HO* IMH";
$resn_exch{"H5'1 IMH"}="1H5* IMH"; $resn_exch{"H5'2 IMH"}="2H5* IMH"; $resn_exch{" H4' IMH"}=" H4* IMH";
$resn_exch{"HN'4 IMH"}="4HN* IMH"; $resn_exch{" H3' IMH"}=" H3* IMH"; $resn_exch{"HO'3 IMH"}="3HO* IMH";
$resn_exch{" H2' IMH"}=" H2* IMH"; $resn_exch{"HO'2 IMH"}="2HO* IMH"; $resn_exch{" H1' IMH"}=" H1* IMH";
$resn_exch{"H101 IMI"}="1H10 IMI"; $resn_exch{"H102 IMI"}="2H10 IMI"; $resn_exch{" H91 IMI"}="1H9  IMI";
$resn_exch{" H92 IMI"}="2H9  IMI"; $resn_exch{" H81 IMI"}="1H8  IMI"; $resn_exch{" H82 IMI"}="2H8  IMI";
$resn_exch{" H71 IMI"}="1H7  IMI"; $resn_exch{" H72 IMI"}="2H7  IMI"; $resn_exch{" H61 IMI"}="1H6  IMI";
$resn_exch{" H62 IMI"}="2H6  IMI"; $resn_exch{" H61 IMK"}="1H6  IMK"; $resn_exch{" H62 IMK"}="2H6  IMK";
$resn_exch{"H141 IMK"}="1H14 IMK"; $resn_exch{"H142 IMK"}="2H14 IMK"; $resn_exch{"H143 IMK"}="3H14 IMK";
$resn_exch{" H   IML"}=" HN  IML"; $resn_exch{" HN1 IML"}="1HN  IML"; $resn_exch{" HN2 IML"}="2HN  IML";
$resn_exch{" HN3 IML"}="3HN  IML"; $resn_exch{"HG21 IML"}="1HG2 IML"; $resn_exch{"HG22 IML"}="2HG2 IML";
$resn_exch{"HG23 IML"}="3HG2 IML"; $resn_exch{"HG12 IML"}="1HG1 IML"; $resn_exch{"HG13 IML"}="2HG1 IML";
$resn_exch{"HD11 IML"}="1HD1 IML"; $resn_exch{"HD12 IML"}="2HD1 IML"; $resn_exch{"HD13 IML"}="3HD1 IML";
$resn_exch{"H171 IMM"}="1H17 IMM"; $resn_exch{"H172 IMM"}="2H17 IMM"; $resn_exch{"H173 IMM"}="3H17 IMM";
$resn_exch{"H181 IMM"}="1H18 IMM"; $resn_exch{"H182 IMM"}="2H18 IMM"; $resn_exch{"H201 IMM"}="1H20 IMM";
$resn_exch{"H202 IMM"}="2H20 IMM"; $resn_exch{"H203 IMM"}="3H20 IMM"; $resn_exch{" H61 IMN"}="1H6  IMN";
$resn_exch{" H62 IMN"}="2H6  IMN"; $resn_exch{" H63 IMN"}="3H6  IMN"; $resn_exch{"H161 IMN"}="1H16 IMN";
$resn_exch{"H162 IMN"}="2H16 IMN"; $resn_exch{"H163 IMN"}="3H16 IMN"; $resn_exch{"H171 IMN"}="1H17 IMN";
$resn_exch{"H172 IMN"}="2H17 IMN"; $resn_exch{"H5'1 IMO"}="1H5' IMO"; $resn_exch{"H5'2 IMO"}="2H5' IMO";
$resn_exch{"HOA1 IMO"}="1HOA IMO"; $resn_exch{"HOA2 IMO"}="2HOA IMO"; $resn_exch{" H81 IMO"}="1H8  IMO";
$resn_exch{"HO'2 IMO"}="2HO' IMO"; $resn_exch{"HO'3 IMO"}="3HO' IMO"; $resn_exch{"HOP2 IMP"}="2HOP IMP";
$resn_exch{"HOP3 IMP"}="3HOP IMP"; $resn_exch{"H5'1 IMP"}="1H5* IMP"; $resn_exch{"H5'2 IMP"}="2H5* IMP";
$resn_exch{" H4' IMP"}=" H4* IMP"; $resn_exch{" H3' IMP"}=" H3* IMP"; $resn_exch{"HO3' IMP"}="*HO3 IMP";
$resn_exch{" H2' IMP"}=" H2* IMP"; $resn_exch{"HO2' IMP"}="*HO2 IMP"; $resn_exch{" H1' IMP"}=" H1* IMP";
$resn_exch{"HO'5 IMR"}="5HO* IMR"; $resn_exch{"H5'1 IMR"}="1H5* IMR"; $resn_exch{"H5'2 IMR"}="2H5* IMR";
$resn_exch{" H4' IMR"}=" H4* IMR"; $resn_exch{"HN'4 IMR"}="4HN* IMR"; $resn_exch{" H3' IMR"}=" H3* IMR";
$resn_exch{"HO'3 IMR"}="3HO* IMR"; $resn_exch{" H2' IMR"}=" H2* IMR"; $resn_exch{"HO'2 IMR"}="2HO* IMR";
$resn_exch{"H1'1 IMR"}="1H1* IMR"; $resn_exch{"H1'2 IMR"}="2H1* IMR"; $resn_exch{" H61 IMS"}="1H6  IMS";
$resn_exch{" H62 IMS"}="2H6  IMS"; $resn_exch{" H63 IMS"}="3H6  IMS"; $resn_exch{"H211 IMS"}="1H21 IMS";
$resn_exch{"H212 IMS"}="2H21 IMS"; $resn_exch{"H221 IMS"}="1H22 IMS"; $resn_exch{"H222 IMS"}="2H22 IMS";
$resn_exch{"H223 IMS"}="3H22 IMS"; $resn_exch{"H201 IMS"}="1H20 IMS"; $resn_exch{"H202 IMS"}="2H20 IMS";
$resn_exch{"H171 IMS"}="1H17 IMS"; $resn_exch{"H172 IMS"}="2H17 IMS"; $resn_exch{"H161 IMS"}="1H16 IMS";
$resn_exch{"H162 IMS"}="2H16 IMS"; $resn_exch{"H163 IMS"}="3H16 IMS"; $resn_exch{" HD1 IMT"}="1HD  IMT";
$resn_exch{" HD2 IMT"}="2HD  IMT"; $resn_exch{" HD3 IMT"}="3HD  IMT"; $resn_exch{"HO1P IMU"}="PHO1 IMU";
$resn_exch{"HO3P IMU"}="PHO3 IMU"; $resn_exch{"H5'1 IMU"}="1H5* IMU"; $resn_exch{"H5'2 IMU"}="2H5* IMU";
$resn_exch{" H4' IMU"}=" H4* IMU"; $resn_exch{"HN4' IMU"}="*HN4 IMU"; $resn_exch{" H3' IMU"}=" H3* IMU";
$resn_exch{"HO3' IMU"}="*HO3 IMU"; $resn_exch{" H2' IMU"}=" H2* IMU"; $resn_exch{"HO2' IMU"}="*HO2 IMU";
$resn_exch{" H1' IMU"}=" H1* IMU"; $resn_exch{"HN21 IMU"}="1HN2 IMU"; $resn_exch{"HN22 IMU"}="2HN2 IMU";
$resn_exch{"H321 IMY"}="1H32 IMY"; $resn_exch{"H322 IMY"}="2H32 IMY"; $resn_exch{"H323 IMY"}="3H32 IMY";
$resn_exch{"H331 IMY"}="1H33 IMY"; $resn_exch{"H332 IMY"}="2H33 IMY"; $resn_exch{"H333 IMY"}="3H33 IMY";
$resn_exch{"H301 IMY"}="1H30 IMY"; $resn_exch{"H302 IMY"}="2H30 IMY"; $resn_exch{" H51 IMY"}="1H5  IMY";
$resn_exch{" H52 IMY"}="2H5  IMY"; $resn_exch{" H61 IMY"}="1H6  IMY"; $resn_exch{" H62 IMY"}="2H6  IMY";
$resn_exch{" H71 IMY"}="1H7  IMY"; $resn_exch{" H72 IMY"}="2H7  IMY"; $resn_exch{" H81 IMY"}="1H8  IMY";
$resn_exch{" H82 IMY"}="2H8  IMY"; $resn_exch{"H271 IMZ"}="1H27 IMZ"; $resn_exch{"H272 IMZ"}="2H27 IMZ";
$resn_exch{"H281 IMZ"}="1H28 IMZ"; $resn_exch{"H282 IMZ"}="2H28 IMZ"; $resn_exch{"H283 IMZ"}="3H28 IMZ";
$resn_exch{"H241 IMZ"}="1H24 IMZ"; $resn_exch{"H242 IMZ"}="2H24 IMZ"; $resn_exch{"H243 IMZ"}="3H24 IMZ";
$resn_exch{"H151 IMZ"}="1H15 IMZ"; $resn_exch{"H152 IMZ"}="2H15 IMZ"; $resn_exch{"H141 IMZ"}="1H14 IMZ";
$resn_exch{"H142 IMZ"}="2H14 IMZ"; $resn_exch{"H131 IMZ"}="1H13 IMZ"; $resn_exch{"H132 IMZ"}="2H13 IMZ";
$resn_exch{"H161 IMZ"}="1H16 IMZ"; $resn_exch{"H162 IMZ"}="2H16 IMZ"; $resn_exch{"H111 IMZ"}="1H11 IMZ";
$resn_exch{"H112 IMZ"}="2H11 IMZ"; $resn_exch{"H121 IMZ"}="1H12 IMZ"; $resn_exch{"H122 IMZ"}="2H12 IMZ";
$resn_exch{"H141 IN0"}="1H14 IN0"; $resn_exch{"H142 IN0"}="2H14 IN0"; $resn_exch{"H611 IN0"}="1H61 IN0";
$resn_exch{"H612 IN0"}="2H61 IN0"; $resn_exch{"H431 IN0"}="1H43 IN0"; $resn_exch{"H432 IN0"}="2H43 IN0";
$resn_exch{"H321 IN0"}="1H32 IN0"; $resn_exch{"H322 IN0"}="2H32 IN0"; $resn_exch{"H323 IN0"}="3H32 IN0";
$resn_exch{"H251 IN0"}="1H25 IN0"; $resn_exch{"H252 IN0"}="2H25 IN0"; $resn_exch{"H253 IN0"}="3H25 IN0";
$resn_exch{"H241 IN0"}="1H24 IN0"; $resn_exch{"H242 IN0"}="2H24 IN0"; $resn_exch{"H243 IN0"}="3H24 IN0";
$resn_exch{" H11 IN1"}="1H1  IN1"; $resn_exch{" H12 IN1"}="2H1  IN1"; $resn_exch{" H21 IN1"}="1H2  IN1";
$resn_exch{" H22 IN1"}="2H2  IN1"; $resn_exch{" H31 IN1"}="1H3  IN1"; $resn_exch{" H32 IN1"}="2H3  IN1";
$resn_exch{" H71 IN1"}="1H7  IN1"; $resn_exch{" H72 IN1"}="2H7  IN1"; $resn_exch{" H73 IN1"}="3H7  IN1";
$resn_exch{"H6B1 IN1"}="1H6B IN1"; $resn_exch{"H6B2 IN1"}="2H6B IN1"; $resn_exch{" H51 IN1"}="1H5  IN1";
$resn_exch{" H52 IN1"}="2H5  IN1"; $resn_exch{"HOS4 IN1"}="4HOS IN1"; $resn_exch{"HN11 IN2"}="1HN1 IN2";
$resn_exch{"HN12 IN2"}="2HN1 IN2"; $resn_exch{" H21 IN2"}="1H2  IN2"; $resn_exch{" H22 IN2"}="2H2  IN2";
$resn_exch{" H31 IN2"}="1H3  IN2"; $resn_exch{" H32 IN2"}="2H3  IN2"; $resn_exch{" H41 IN2"}="1H4  IN2";
$resn_exch{" H42 IN2"}="2H4  IN2"; $resn_exch{" H51 IN2"}="1H5  IN2"; $resn_exch{" H52 IN2"}="2H5  IN2";
$resn_exch{"H101 IN2"}="1H10 IN2"; $resn_exch{"H102 IN2"}="2H10 IN2"; $resn_exch{"H103 IN2"}="3H10 IN2";
$resn_exch{"H111 IN2"}="1H11 IN2"; $resn_exch{"H112 IN2"}="2H11 IN2"; $resn_exch{"H113 IN2"}="3H11 IN2";
$resn_exch{" H71 IN3"}="1H7  IN3"; $resn_exch{" H72 IN3"}="2H7  IN3"; $resn_exch{"H7'1 IN3"}="1H7' IN3";
$resn_exch{"H7'2 IN3"}="2H7' IN3"; $resn_exch{"H11' IN3"}="'H11 IN3"; $resn_exch{"H12A IN3"}="AH12 IN3";
$resn_exch{"H12B IN3"}="BH12 IN3"; $resn_exch{"H14' IN3"}="'H14 IN3"; $resn_exch{"H15A IN3"}="AH15 IN3";
$resn_exch{"H15B IN3"}="BH15 IN3"; $resn_exch{"H15C IN3"}="CH15 IN3"; $resn_exch{"H16A IN3"}="AH16 IN3";
$resn_exch{"H16B IN3"}="BH16 IN3"; $resn_exch{"H16C IN3"}="CH16 IN3"; $resn_exch{" H91 IN4"}="1H9  IN4";
$resn_exch{" H92 IN4"}="2H9  IN4"; $resn_exch{"H111 IN4"}="1H11 IN4"; $resn_exch{"H112 IN4"}="2H11 IN4";
$resn_exch{"HN41 IN4"}="1HN4 IN4"; $resn_exch{"HN42 IN4"}="2HN4 IN4"; $resn_exch{"HN11 IN4"}="1HN1 IN4";
$resn_exch{"HN12 IN4"}="2HN1 IN4"; $resn_exch{"H141 IN4"}="1H14 IN4"; $resn_exch{"H142 IN4"}="2H14 IN4";
$resn_exch{" H81 IN4"}="1H8  IN4"; $resn_exch{" H82 IN4"}="2H8  IN4"; $resn_exch{" H83 IN4"}="3H8  IN4";
$resn_exch{" H71 IN5"}="1H7  IN5"; $resn_exch{" H72 IN5"}="2H7  IN5"; $resn_exch{" H81 IN5"}="1H8  IN5";
$resn_exch{" H82 IN5"}="2H8  IN5"; $resn_exch{"H101 IN5"}="1H10 IN5"; $resn_exch{"H102 IN5"}="2H10 IN5";
$resn_exch{"H103 IN5"}="3H10 IN5"; $resn_exch{" H61 IN5"}="1H6  IN5"; $resn_exch{" H62 IN5"}="2H6  IN5";
$resn_exch{" H63 IN5"}="3H6  IN5"; $resn_exch{" H71 IN6"}="1H7  IN6"; $resn_exch{" H72 IN6"}="2H7  IN6";
$resn_exch{"H121 IN6"}="1H12 IN6"; $resn_exch{"H122 IN6"}="2H12 IN6"; $resn_exch{"H151 IN6"}="1H15 IN6";
$resn_exch{"H152 IN6"}="2H15 IN6"; $resn_exch{"H153 IN6"}="3H15 IN6"; $resn_exch{"H161 IN6"}="1H16 IN6";
$resn_exch{"H162 IN6"}="2H16 IN6"; $resn_exch{"H163 IN6"}="3H16 IN6"; $resn_exch{"H7'1 IN6"}="1H7' IN6";
$resn_exch{"H7'2 IN6"}="2H7' IN6"; $resn_exch{"H11' IN6"}="'H11 IN6"; $resn_exch{"H12A IN6"}="AH12 IN6";
$resn_exch{"H12B IN6"}="BH12 IN6"; $resn_exch{"H14' IN6"}="'H14 IN6"; $resn_exch{"H15A IN6"}="AH15 IN6";
$resn_exch{"H15B IN6"}="BH15 IN6"; $resn_exch{"H15C IN6"}="CH15 IN6"; $resn_exch{"H16A IN6"}="AH16 IN6";
$resn_exch{"H16B IN6"}="BH16 IN6"; $resn_exch{"H16C IN6"}="CH16 IN6"; $resn_exch{" H71 IN7"}="1H7  IN7";
$resn_exch{" H72 IN7"}="2H7  IN7"; $resn_exch{"H131 IN7"}="1H13 IN7"; $resn_exch{"H132 IN7"}="2H13 IN7";
$resn_exch{"H151 IN7"}="1H15 IN7"; $resn_exch{"H152 IN7"}="2H15 IN7"; $resn_exch{"H181 IN7"}="1H18 IN7";
$resn_exch{"H182 IN7"}="2H18 IN7"; $resn_exch{" H81 IN7"}="1H8  IN7"; $resn_exch{" H82 IN7"}="2H8  IN7";
$resn_exch{"H131 IN8"}="1H13 IN8"; $resn_exch{"H132 IN8"}="2H13 IN8"; $resn_exch{" H41 IN8"}="1H4  IN8";
$resn_exch{" H42 IN8"}="2H4  IN8"; $resn_exch{" H71 IN9"}="1H7  IN9"; $resn_exch{" H72 IN9"}="2H7  IN9";
$resn_exch{"H141 IN9"}="1H14 IN9"; $resn_exch{"H142 IN9"}="2H14 IN9"; $resn_exch{"H143 IN9"}="3H14 IN9";
$resn_exch{" H71 INA"}="1H7  INA"; $resn_exch{" H72 INA"}="2H7  INA"; $resn_exch{"H121 INA"}="1H12 INA";
$resn_exch{"H122 INA"}="2H12 INA"; $resn_exch{"H141 INA"}="1H14 INA"; $resn_exch{"H142 INA"}="2H14 INA";
$resn_exch{"H143 INA"}="3H14 INA"; $resn_exch{"H151 INA"}="1H15 INA"; $resn_exch{"H152 INA"}="2H15 INA";
$resn_exch{"H153 INA"}="3H15 INA"; $resn_exch{"H291 INA"}="1H29 INA"; $resn_exch{"H292 INA"}="2H29 INA";
$resn_exch{"H341 INA"}="1H34 INA"; $resn_exch{"H342 INA"}="2H34 INA"; $resn_exch{"H361 INA"}="1H36 INA";
$resn_exch{"H362 INA"}="2H36 INA"; $resn_exch{"H363 INA"}="3H36 INA"; $resn_exch{"H371 INA"}="1H37 INA";
$resn_exch{"H372 INA"}="2H37 INA"; $resn_exch{"H373 INA"}="3H37 INA"; $resn_exch{"H1'1 INB"}="1H1' INB";
$resn_exch{"H1'2 INB"}="2H1' INB"; $resn_exch{"H3'1 INB"}="1H3' INB"; $resn_exch{"H3'2 INB"}="2H3' INB";
$resn_exch{"HN2' INB"}="'HN2 INB"; $resn_exch{"HM'1 INB"}="1HM' INB"; $resn_exch{"HM'2 INB"}="2HM' INB";
$resn_exch{"HM'3 INB"}="3HM' INB"; $resn_exch{"HOP2 INB"}="2HOP INB"; $resn_exch{"H1A1 INB"}="1H1\" INB";
$resn_exch{"H1A2 INB"}="2H1\" INB"; $resn_exch{"H2'1 INB"}="1H2\" INB"; $resn_exch{"H2'2 INB"}="2H2\" INB";
$resn_exch{"HMA1 INB"}="1HM\" INB"; $resn_exch{"HMA2 INB"}="2HM\" INB"; $resn_exch{"HMA3 INB"}="3HM\" INB";
$resn_exch{" H11 INB"}="1H1  INB"; $resn_exch{" H12 INB"}="2H1  INB"; $resn_exch{" H21 INB"}="1H2  INB";
$resn_exch{" H22 INB"}="2H2  INB"; $resn_exch{" H31 INB"}="1H3  INB"; $resn_exch{" H32 INB"}="2H3  INB";
$resn_exch{" H41 INB"}="1H4  INB"; $resn_exch{" H42 INB"}="2H4  INB"; $resn_exch{" H51 INB"}="1H5  INB";
$resn_exch{" H52 INB"}="2H5  INB"; $resn_exch{" H61 INB"}="1H6  INB"; $resn_exch{" H62 INB"}="2H6  INB";
$resn_exch{" H71 INB"}="1H7  INB"; $resn_exch{" H72 INB"}="2H7  INB"; $resn_exch{" H81 INB"}="1H8  INB";
$resn_exch{" H82 INB"}="2H8  INB"; $resn_exch{" H91 INB"}="1H9  INB"; $resn_exch{" H92 INB"}="2H9  INB";
$resn_exch{"H101 INB"}="1H10 INB"; $resn_exch{"H102 INB"}="2H10 INB"; $resn_exch{"H111 INB"}="1H11 INB";
$resn_exch{"H112 INB"}="2H11 INB"; $resn_exch{"H121 INB"}="1H12 INB"; $resn_exch{"H122 INB"}="2H12 INB";
$resn_exch{"H131 INB"}="1H13 INB"; $resn_exch{"H132 INB"}="2H13 INB"; $resn_exch{"H141 INB"}="1H14 INB";
$resn_exch{"H142 INB"}="2H14 INB"; $resn_exch{"H151 INB"}="1H15 INB"; $resn_exch{"H152 INB"}="2H15 INB";
$resn_exch{"H161 INB"}="1H16 INB"; $resn_exch{"H162 INB"}="2H16 INB"; $resn_exch{"H171 INB"}="1H17 INB";
$resn_exch{"H172 INB"}="2H17 INB"; $resn_exch{"H181 INB"}="1H18 INB"; $resn_exch{"H182 INB"}="2H18 INB";
$resn_exch{"H183 INB"}="3H18 INB"; $resn_exch{"HA11 INC"}="1HA1 INC"; $resn_exch{"HA12 INC"}="2HA1 INC";
$resn_exch{"HA13 INC"}="3HA1 INC"; $resn_exch{"HB21 INC"}="1HB2 INC"; $resn_exch{"HB22 INC"}="2HB2 INC";
$resn_exch{"HD11 INC"}="1HD1 INC"; $resn_exch{"HD12 INC"}="2HD1 INC"; $resn_exch{"HD13 INC"}="3HD1 INC";
$resn_exch{"HD21 INC"}="1HD2 INC"; $resn_exch{"HD22 INC"}="2HD2 INC"; $resn_exch{"HD23 INC"}="3HD2 INC";
$resn_exch{" H31 INC"}="1H3  INC"; $resn_exch{" H32 INC"}="2H3  INC"; $resn_exch{" H33 INC"}="3H3  INC";
$resn_exch{" H11 INE"}="1H1  INE"; $resn_exch{" H41 INE"}="1H4  INE"; $resn_exch{" H51 INE"}="1H5  INE";
$resn_exch{" H61 INE"}="1H6  INE"; $resn_exch{" HB1 INF"}="1HB  INF"; $resn_exch{" HB2 INF"}="2HB  INF";
$resn_exch{" HB1 ING"}="1HB  ING"; $resn_exch{" HB2 ING"}="2HB  ING"; $resn_exch{"HN11 ING"}="1HN1 ING";
$resn_exch{"HN12 ING"}="2HN1 ING"; $resn_exch{" H81 INI"}="1H8  INI"; $resn_exch{" H82 INI"}="2H8  INI";
$resn_exch{"H121 INI"}="1H12 INI"; $resn_exch{"H122 INI"}="2H12 INI"; $resn_exch{" H81 INJ"}="1H8  INJ";
$resn_exch{" H82 INJ"}="2H8  INJ"; $resn_exch{" H91 INJ"}="1H9  INJ"; $resn_exch{"H101 INJ"}="1H10 INJ";
$resn_exch{"H111 INJ"}="1H11 INJ"; $resn_exch{"H121 INJ"}="1H12 INJ"; $resn_exch{"H122 INJ"}="2H12 INJ";
$resn_exch{"H131 INJ"}="1H13 INJ"; $resn_exch{"H132 INJ"}="2H13 INJ"; $resn_exch{"H141 INJ"}="1H14 INJ";
$resn_exch{"H142 INJ"}="2H14 INJ"; $resn_exch{"H151 INJ"}="1H15 INJ"; $resn_exch{"H152 INJ"}="2H15 INJ";
$resn_exch{"H161 INJ"}="1H16 INJ"; $resn_exch{"H162 INJ"}="2H16 INJ"; $resn_exch{"H171 INJ"}="1H17 INJ";
$resn_exch{"H172 INJ"}="2H17 INJ"; $resn_exch{"HOP2 INJ"}="2HOP INJ"; $resn_exch{"HOP3 INJ"}="3HOP INJ";
$resn_exch{"HC11 INK"}="1HC1 INK"; $resn_exch{"HC12 INK"}="2HC1 INK"; $resn_exch{"HC13 INK"}="3HC1 INK";
$resn_exch{"HC31 INK"}="1HC3 INK"; $resn_exch{"HC32 INK"}="2HC3 INK"; $resn_exch{"HC41 INK"}="1HC4 INK";
$resn_exch{"HC42 INK"}="2HC4 INK"; $resn_exch{"HC43 INK"}="3HC4 INK"; $resn_exch{"HC51 INK"}="1HC5 INK";
$resn_exch{"HC52 INK"}="2HC5 INK"; $resn_exch{"HN41 INL"}="1HN4 INL"; $resn_exch{"HN42 INL"}="2HN4 INL";
$resn_exch{" H61 INL"}="1H6  INL"; $resn_exch{"H101 INL"}="1H10 INL"; $resn_exch{"H161 INL"}="1H16 INL";
$resn_exch{"H162 INL"}="2H16 INL"; $resn_exch{"H171 INL"}="1H17 INL"; $resn_exch{"H172 INL"}="2H17 INL";
$resn_exch{"H181 INL"}="1H18 INL"; $resn_exch{"H182 INL"}="2H18 INL"; $resn_exch{"H201 INL"}="1H20 INL";
$resn_exch{"H202 INL"}="2H20 INL"; $resn_exch{"H211 INL"}="1H21 INL"; $resn_exch{"H212 INL"}="2H21 INL";
$resn_exch{"H231 INL"}="1H23 INL"; $resn_exch{"H251 INL"}="1H25 INL"; $resn_exch{"H261 INL"}="1H26 INL";
$resn_exch{"H271 INL"}="1H27 INL"; $resn_exch{"H301 INL"}="1H30 INL"; $resn_exch{"H302 INL"}="2H30 INL";
$resn_exch{"H303 INL"}="3H30 INL"; $resn_exch{"HC21 INM"}="1HC2 INM"; $resn_exch{" H71 INM"}="1H7  INM";
$resn_exch{" H72 INM"}="2H7  INM"; $resn_exch{" H81 INM"}="1H8  INM"; $resn_exch{" H82 INM"}="2H8  INM";
$resn_exch{" H91 INM"}="1H9  INM"; $resn_exch{" H92 INM"}="2H9  INM"; $resn_exch{"H101 INM"}="1H10 INM";
$resn_exch{"H102 INM"}="2H10 INM"; $resn_exch{"H121 INM"}="1H12 INM"; $resn_exch{"H122 INM"}="2H12 INM";
$resn_exch{"H123 INM"}="3H12 INM"; $resn_exch{"H131 INM"}="1H13 INM"; $resn_exch{"H191 INM"}="1H19 INM";
$resn_exch{"H192 INM"}="2H19 INM"; $resn_exch{" H21 INN"}="1H2  INN"; $resn_exch{" H22 INN"}="2H2  INN";
$resn_exch{" H23 INN"}="3H2  INN"; $resn_exch{" H81 INN"}="1H8  INN"; $resn_exch{" H82 INN"}="2H8  INN";
$resn_exch{" H83 INN"}="3H8  INN"; $resn_exch{" H91 INN"}="1H9  INN"; $resn_exch{" H92 INN"}="2H9  INN";
$resn_exch{" H93 INN"}="3H9  INN"; $resn_exch{"H121 INN"}="1H12 INN"; $resn_exch{"H122 INN"}="2H12 INN";
$resn_exch{"H123 INN"}="3H12 INN"; $resn_exch{" H01 INN"}="1H0  INN"; $resn_exch{" H02 INN"}="2H0  INN";
$resn_exch{" HB1 INN"}="1HB  INN"; $resn_exch{" HB2 INN"}="2HB  INN"; $resn_exch{" H31 INN"}="1H3  INN";
$resn_exch{" H32 INN"}="2H3  INN"; $resn_exch{" H33 INN"}="3H3  INN"; $resn_exch{" H71 INN"}="1H7  INN";
$resn_exch{" H72 INN"}="2H7  INN"; $resn_exch{" H73 INN"}="3H7  INN"; $resn_exch{"H141 INN"}="1H14 INN";
$resn_exch{"H142 INN"}="2H14 INN"; $resn_exch{"H151 INN"}="1H15 INN"; $resn_exch{"H152 INN"}="2H15 INN";
$resn_exch{"HN41 INN"}="1HN4 INN"; $resn_exch{"HN42 INN"}="2HN4 INN"; $resn_exch{" H71 INP"}="1H7  INP";
$resn_exch{" H72 INP"}="2H7  INP"; $resn_exch{"H131 INP"}="1H13 INP"; $resn_exch{"H132 INP"}="2H13 INP";
$resn_exch{"H151 INP"}="1H15 INP"; $resn_exch{"H152 INP"}="2H15 INP"; $resn_exch{"H161 INP"}="1H16 INP";
$resn_exch{"H162 INP"}="2H16 INP"; $resn_exch{"H191 INP"}="1H19 INP"; $resn_exch{"H192 INP"}="2H19 INP";
$resn_exch{"H211 INP"}="1H21 INP"; $resn_exch{"H212 INP"}="2H21 INP"; $resn_exch{"H213 INP"}="3H21 INP";
$resn_exch{"H221 INP"}="1H22 INP"; $resn_exch{"H222 INP"}="2H22 INP"; $resn_exch{"H223 INP"}="3H22 INP";
$resn_exch{"H271 INP"}="1H27 INP"; $resn_exch{"H272 INP"}="2H27 INP"; $resn_exch{"H291 INP"}="1H29 INP";
$resn_exch{"H292 INP"}="2H29 INP"; $resn_exch{"H293 INP"}="3H29 INP"; $resn_exch{"H301 INP"}="1H30 INP";
$resn_exch{"H302 INP"}="2H30 INP"; $resn_exch{"H303 INP"}="3H30 INP"; $resn_exch{"H351 INP"}="1H35 INP";
$resn_exch{"H352 INP"}="2H35 INP"; $resn_exch{"H031 INQ"}="1H03 INQ"; $resn_exch{"H032 INQ"}="2H03 INQ";
$resn_exch{"H051 INQ"}="1H05 INQ"; $resn_exch{"H091 INQ"}="1H09 INQ"; $resn_exch{"H151 INQ"}="1H15 INQ";
$resn_exch{"H152 INQ"}="2H15 INQ"; $resn_exch{"H161 INQ"}="1H16 INQ"; $resn_exch{"H162 INQ"}="2H16 INQ";
$resn_exch{"H171 INQ"}="1H17 INQ"; $resn_exch{"H172 INQ"}="2H17 INQ"; $resn_exch{"H191 INQ"}="1H19 INQ";
$resn_exch{"H192 INQ"}="2H19 INQ"; $resn_exch{"H201 INQ"}="1H20 INQ"; $resn_exch{"H202 INQ"}="2H20 INQ";
$resn_exch{"H221 INQ"}="1H22 INQ"; $resn_exch{"H241 INQ"}="1H24 INQ"; $resn_exch{"H251 INQ"}="1H25 INQ";
$resn_exch{"H261 INQ"}="1H26 INQ"; $resn_exch{"H281 INQ"}="1H28 INQ"; $resn_exch{" H21 INT"}="1H2  INT";
$resn_exch{" H22 INT"}="2H2  INT"; $resn_exch{" H31 INT"}="1H3  INT"; $resn_exch{" H32 INT"}="2H3  INT";
$resn_exch{"HG11 INT"}="1HG1 INT"; $resn_exch{"HG12 INT"}="2HG1 INT"; $resn_exch{"HG13 INT"}="3HG1 INT";
$resn_exch{"HG21 INT"}="1HG2 INT"; $resn_exch{"HG22 INT"}="2HG2 INT"; $resn_exch{"HG23 INT"}="3HG2 INT";
$resn_exch{"H201 INT"}="1H20 INT"; $resn_exch{"H202 INT"}="2H20 INT"; $resn_exch{"H203 INT"}="3H20 INT";
$resn_exch{" HA1 INT"}="1HA  INT"; $resn_exch{" HA2 INT"}="2HA  INT"; $resn_exch{"HG31 INU"}="1HG3 INU";
$resn_exch{"HG32 INU"}="2HG3 INU"; $resn_exch{"HG41 INU"}="1HG4 INU"; $resn_exch{"HG42 INU"}="2HG4 INU";
$resn_exch{"HD11 INU"}="1HD1 INU"; $resn_exch{"HD12 INU"}="2HD1 INU"; $resn_exch{"HD21 INU"}="1HD2 INU";
$resn_exch{"HD22 INU"}="2HD2 INU"; $resn_exch{"HD31 INU"}="1HD3 INU"; $resn_exch{"HD32 INU"}="2HD3 INU";
$resn_exch{"HD41 INU"}="1HD4 INU"; $resn_exch{"HD42 INU"}="2HD4 INU"; $resn_exch{"HD51 INU"}="1HD5 INU";
$resn_exch{"HD52 INU"}="2HD5 INU"; $resn_exch{"HD61 INU"}="1HD6 INU"; $resn_exch{"HD62 INU"}="2HD6 INU";
$resn_exch{"H011 INV"}="1H01 INV"; $resn_exch{"H021 INV"}="1H02 INV"; $resn_exch{"H041 INV"}="1H04 INV";
$resn_exch{"H051 INV"}="1H05 INV"; $resn_exch{"H091 INV"}="1H09 INV"; $resn_exch{"H101 INV"}="1H10 INV";
$resn_exch{"H102 INV"}="2H10 INV"; $resn_exch{"HP21 INV"}="1HP2 INV"; $resn_exch{"HP22 INV"}="2HP2 INV";
$resn_exch{"H161 INV"}="1H16 INV"; $resn_exch{"H171 INV"}="1H17 INV"; $resn_exch{"H191 INV"}="1H19 INV";
$resn_exch{"H201 INV"}="1H20 INV"; $resn_exch{"H011 INW"}="1H01 INW"; $resn_exch{"H021 INW"}="1H02 INW";
$resn_exch{"H041 INW"}="1H04 INW"; $resn_exch{"H051 INW"}="1H05 INW"; $resn_exch{"H091 INW"}="1H09 INW";
$resn_exch{"H101 INW"}="1H10 INW"; $resn_exch{"H102 INW"}="2H10 INW"; $resn_exch{"HP21 INW"}="1HP2 INW";
$resn_exch{"HP22 INW"}="2HP2 INW"; $resn_exch{"H171 INW"}="1H17 INW"; $resn_exch{"H191 INW"}="1H19 INW";
$resn_exch{"H201 INW"}="1H20 INW"; $resn_exch{"HC71 INX"}="1HC7 INX"; $resn_exch{"HC72 INX"}="2HC7 INX";
$resn_exch{"H151 INX"}="1H15 INX"; $resn_exch{"H152 INX"}="2H15 INX"; $resn_exch{"H161 INX"}="1H16 INX";
$resn_exch{"H162 INX"}="2H16 INX"; $resn_exch{"H171 INX"}="1H17 INX"; $resn_exch{"H172 INX"}="2H17 INX";
$resn_exch{"H181 INX"}="1H18 INX"; $resn_exch{"H182 INX"}="2H18 INX"; $resn_exch{"H191 INX"}="1H19 INX";
$resn_exch{"H192 INX"}="2H19 INX"; $resn_exch{"H193 INX"}="3H19 INX"; $resn_exch{"H231 INX"}="1H23 INX";
$resn_exch{"H232 INX"}="2H23 INX"; $resn_exch{"H301 INX"}="1H30 INX"; $resn_exch{"H302 INX"}="2H30 INX";
$resn_exch{"H381 INX"}="1H38 INX"; $resn_exch{"H382 INX"}="2H38 INX"; $resn_exch{"H391 INX"}="1H39 INX";
$resn_exch{"H392 INX"}="2H39 INX"; $resn_exch{" H61 INY"}="1H6  INY"; $resn_exch{" H62 INY"}="2H6  INY";
$resn_exch{" H63 INY"}="3H6  INY"; $resn_exch{"H171 INY"}="1H17 INY"; $resn_exch{"H172 INY"}="2H17 INY";
$resn_exch{"H121 INY"}="1H12 INY"; $resn_exch{"H122 INY"}="2H12 INY"; $resn_exch{"H123 INY"}="3H12 INY";
$resn_exch{"H111 INZ"}="1H11 INZ"; $resn_exch{"H112 INZ"}="2H11 INZ"; $resn_exch{"H221 INZ"}="1H22 INZ";
$resn_exch{"H222 INZ"}="2H22 INZ"; $resn_exch{"H231 INZ"}="1H23 INZ"; $resn_exch{"H232 INZ"}="2H23 INZ";
$resn_exch{"H241 INZ"}="1H24 INZ"; $resn_exch{"H242 INZ"}="2H24 INZ"; $resn_exch{"H251 INZ"}="1H25 INZ";
$resn_exch{"H252 INZ"}="2H25 INZ"; $resn_exch{"H261 INZ"}="1H26 INZ"; $resn_exch{"H262 INZ"}="2H26 INZ";
$resn_exch{"H263 INZ"}="3H26 INZ"; $resn_exch{"H301 INZ"}="1H30 INZ"; $resn_exch{"H302 INZ"}="2H30 INZ";
$resn_exch{"H011 IOA"}="1H01 IOA"; $resn_exch{"H021 IOA"}="1H02 IOA"; $resn_exch{"H041 IOA"}="1H04 IOA";
$resn_exch{"H051 IOA"}="1H05 IOA"; $resn_exch{"H091 IOA"}="1H09 IOA"; $resn_exch{"H101 IOA"}="1H10 IOA";
$resn_exch{"H102 IOA"}="2H10 IOA"; $resn_exch{"HP21 IOA"}="1HP2 IOA"; $resn_exch{"HP22 IOA"}="2HP2 IOA";
$resn_exch{"H161 IOA"}="1H16 IOA"; $resn_exch{"H181 IOA"}="1H18 IOA"; $resn_exch{"H191 IOA"}="1H19 IOA";
$resn_exch{" HB2 IOB"}="2HB  IOB"; $resn_exch{" HB1 IOB"}="1HB  IOB"; $resn_exch{"H011 IOC"}="1H01 IOC";
$resn_exch{"H021 IOC"}="1H02 IOC"; $resn_exch{"H041 IOC"}="1H04 IOC"; $resn_exch{"H051 IOC"}="1H05 IOC";
$resn_exch{"H091 IOC"}="1H09 IOC"; $resn_exch{"H101 IOC"}="1H10 IOC"; $resn_exch{"H102 IOC"}="2H10 IOC";
$resn_exch{"HP21 IOC"}="1HP2 IOC"; $resn_exch{"HP22 IOC"}="2HP2 IOC"; $resn_exch{"H191 IOC"}="1H19 IOC";
$resn_exch{"H201 IOC"}="1H20 IOC"; $resn_exch{"H011 IOE"}="1H01 IOE"; $resn_exch{"H021 IOE"}="1H02 IOE";
$resn_exch{"H041 IOE"}="1H04 IOE"; $resn_exch{"H051 IOE"}="1H05 IOE"; $resn_exch{"H091 IOE"}="1H09 IOE";
$resn_exch{"H101 IOE"}="1H10 IOE"; $resn_exch{"H102 IOE"}="2H10 IOE"; $resn_exch{"HP21 IOE"}="1HP2 IOE";
$resn_exch{"HP22 IOE"}="2HP2 IOE"; $resn_exch{"H171 IOE"}="1H17 IOE"; $resn_exch{"H191 IOE"}="1H19 IOE";
$resn_exch{"H011 IOF"}="1H01 IOF"; $resn_exch{"H021 IOF"}="1H02 IOF"; $resn_exch{"H041 IOF"}="1H04 IOF";
$resn_exch{"H051 IOF"}="1H05 IOF"; $resn_exch{"H091 IOF"}="1H09 IOF"; $resn_exch{"H101 IOF"}="1H10 IOF";
$resn_exch{"H102 IOF"}="2H10 IOF"; $resn_exch{"HP21 IOF"}="1HP2 IOF"; $resn_exch{"HP22 IOF"}="2HP2 IOF";
$resn_exch{"H161 IOF"}="1H16 IOF"; $resn_exch{"H201 IOF"}="1H20 IOF"; $resn_exch{" H81 IOG"}="1H8  IOG";
$resn_exch{" H82 IOG"}="2H8  IOG"; $resn_exch{" H91 IOG"}="1H9  IOG"; $resn_exch{" H92 IOG"}="2H9  IOG";
$resn_exch{"H111 IOG"}="1H11 IOG"; $resn_exch{"H112 IOG"}="2H11 IOG"; $resn_exch{"H113 IOG"}="3H11 IOG";
$resn_exch{"H151 IOG"}="1H15 IOG"; $resn_exch{"H152 IOG"}="2H15 IOG"; $resn_exch{"H321 IOG"}="1H32 IOG";
$resn_exch{"H322 IOG"}="2H32 IOG"; $resn_exch{"H331 IOG"}="1H33 IOG"; $resn_exch{"H332 IOG"}="2H33 IOG";
$resn_exch{"H351 IOG"}="1H35 IOG"; $resn_exch{"H352 IOG"}="2H35 IOG"; $resn_exch{"H361 IOG"}="1H36 IOG";
$resn_exch{"H362 IOG"}="2H36 IOG"; $resn_exch{"H371 IOG"}="1H37 IOG"; $resn_exch{"H372 IOG"}="2H37 IOG";
$resn_exch{"H381 IOG"}="1H38 IOG"; $resn_exch{"H382 IOG"}="2H38 IOG"; $resn_exch{"H391 IOG"}="1H39 IOG";
$resn_exch{"H392 IOG"}="2H39 IOG"; $resn_exch{"HB11 IOH"}="1HB1 IOH"; $resn_exch{"HB12 IOH"}="2HB1 IOH";
$resn_exch{"HB13 IOH"}="3HB1 IOH"; $resn_exch{"HB21 IOH"}="1HB2 IOH"; $resn_exch{"HB22 IOH"}="2HB2 IOH";
$resn_exch{"HB23 IOH"}="3HB2 IOH"; $resn_exch{" H81 IOK"}="1H8  IOK"; $resn_exch{" H82 IOK"}="2H8  IOK";
$resn_exch{" H91 IOK"}="1H9  IOK"; $resn_exch{" H92 IOK"}="2H9  IOK"; $resn_exch{"H111 IOK"}="1H11 IOK";
$resn_exch{"H112 IOK"}="2H11 IOK"; $resn_exch{"H113 IOK"}="3H11 IOK"; $resn_exch{"H151 IOK"}="1H15 IOK";
$resn_exch{"H152 IOK"}="2H15 IOK"; $resn_exch{"HNA1 IOM"}="1HNA IOM"; $resn_exch{"HNA2 IOM"}="2HNA IOM";
$resn_exch{"HNB1 IOM"}="1HNB IOM"; $resn_exch{"HNB2 IOM"}="2HNB IOM"; $resn_exch{"HCB1 IOM"}="1HCB IOM";
$resn_exch{"HCB2 IOM"}="2HCB IOM"; $resn_exch{"HCB3 IOM"}="3HCB IOM"; $resn_exch{"HCC1 IOM"}="1HCC IOM";
$resn_exch{"HCC2 IOM"}="2HCC IOM"; $resn_exch{" H11 ION"}="1H1  ION"; $resn_exch{" H12 ION"}="2H1  ION";
$resn_exch{" H41 ION"}="1H4  ION"; $resn_exch{" H42 ION"}="2H4  ION"; $resn_exch{" H43 ION"}="3H4  ION";
$resn_exch{" H51 ION"}="1H5  ION"; $resn_exch{" H52 ION"}="2H5  ION"; $resn_exch{" H61 ION"}="1H6  ION";
$resn_exch{" H62 ION"}="2H6  ION"; $resn_exch{" H71 ION"}="1H7  ION"; $resn_exch{" H72 ION"}="2H7  ION";
$resn_exch{" H91 ION"}="1H9  ION"; $resn_exch{" H92 ION"}="2H9  ION"; $resn_exch{" H93 ION"}="3H9  ION";
$resn_exch{"H111 ION"}="1H11 ION"; $resn_exch{"H112 ION"}="2H11 ION"; $resn_exch{"H121 ION"}="1H12 ION";
$resn_exch{"H141 ION"}="1H14 ION"; $resn_exch{"H142 ION"}="2H14 ION"; $resn_exch{"H143 ION"}="3H14 ION";
$resn_exch{"H151 ION"}="1H15 ION"; $resn_exch{"H152 ION"}="2H15 ION"; $resn_exch{"H2'1 IOP"}="1H2' IOP";
$resn_exch{"H2'2 IOP"}="2H2' IOP"; $resn_exch{"H3'1 IOP"}="1H3' IOP"; $resn_exch{"H3'2 IOP"}="2H3' IOP";
$resn_exch{"H5'1 IOT"}="1H5* IOT"; $resn_exch{"H5'2 IOT"}="2H5* IOT"; $resn_exch{" H4' IOT"}=" H4* IOT";
$resn_exch{" H3' IOT"}=" H3* IOT"; $resn_exch{"HO'3 IOT"}="3HO* IOT"; $resn_exch{" H2' IOT"}=" H2* IOT";
$resn_exch{"HO'2 IOT"}="2HO* IOT"; $resn_exch{" H1' IOT"}=" H1* IOT"; $resn_exch{" H61 IOT"}="1H6  IOT";
$resn_exch{" H62 IOT"}="2H6  IOT"; $resn_exch{"HCX1 IOT"}="1HCX IOT"; $resn_exch{"HCX2 IOT"}="2HCX IOT";
$resn_exch{" HB1 IOT"}="1HB  IOT"; $resn_exch{" HB2 IOT"}="2HB  IOT"; $resn_exch{" HG1 IOT"}="1HG  IOT";
$resn_exch{" HG2 IOT"}="2HG  IOT"; $resn_exch{" HD1 IOT"}="1HD  IOT"; $resn_exch{" HD2 IOT"}="2HD  IOT";
$resn_exch{"HH11 IOT"}="1HH1 IOT"; $resn_exch{"HH21 IOT"}="1HH2 IOT"; $resn_exch{"HH22 IOT"}="2HH2 IOT";
$resn_exch{" HB3 IOY"}="1HB  IOY"; $resn_exch{" HB2 IOY"}="2HB  IOY"; $resn_exch{" H   IOY"}="1HN  IOY";
$resn_exch{" H2  IOY"}="2HN  IOY"; $resn_exch{" H32 IP1"}="2H3  IP1"; $resn_exch{" H31 IP1"}="1H3  IP1";
$resn_exch{"H42A IP1"}="2H4  IP1"; $resn_exch{" H41 IP1"}="1H4  IP1"; $resn_exch{" H72 IP1"}="2H7  IP1";
$resn_exch{" H71 IP1"}="1H7  IP1"; $resn_exch{"H142 IP1"}="2H14 IP1"; $resn_exch{"H141 IP1"}="1H14 IP1";
$resn_exch{"H333 IP1"}="3H33 IP1"; $resn_exch{"H332 IP1"}="2H33 IP1"; $resn_exch{"H331 IP1"}="1H33 IP1";
$resn_exch{"H373 IP1"}="3H37 IP1"; $resn_exch{"H372 IP1"}="2H37 IP1"; $resn_exch{"H371 IP1"}="1H37 IP1";
$resn_exch{"HO42 IP2"}="2HO4 IP2"; $resn_exch{"HO43 IP2"}="3HO4 IP2"; $resn_exch{"HO52 IP2"}="2HO5 IP2";
$resn_exch{"HO53 IP2"}="3HO5 IP2"; $resn_exch{" H11 IP3"}="1H1  IP3"; $resn_exch{" H12 IP3"}="2H1  IP3";
$resn_exch{" H21 IP3"}="1H2  IP3"; $resn_exch{" H22 IP3"}="2H2  IP3"; $resn_exch{" H41 IP3"}="1H4  IP3";
$resn_exch{" H42 IP3"}="2H4  IP3"; $resn_exch{" H43 IP3"}="3H4  IP3"; $resn_exch{" H51 IP3"}="1H5  IP3";
$resn_exch{" H52 IP3"}="2H5  IP3"; $resn_exch{" H53 IP3"}="3H5  IP3"; $resn_exch{" H31 IP4"}="1H3  IP4";
$resn_exch{" H32 IP4"}="2H3  IP4"; $resn_exch{" H41 IP4"}="1H4  IP4"; $resn_exch{" H42 IP4"}="2H4  IP4";
$resn_exch{"HM11 IP4"}="1HM1 IP4"; $resn_exch{"HM12 IP4"}="2HM1 IP4"; $resn_exch{"HM13 IP4"}="3HM1 IP4";
$resn_exch{"HM21 IP4"}="1HM2 IP4"; $resn_exch{"HM22 IP4"}="2HM2 IP4"; $resn_exch{"HM23 IP4"}="3HM2 IP4";
$resn_exch{"HM31 IP4"}="1HM3 IP4"; $resn_exch{"HM32 IP4"}="2HM3 IP4"; $resn_exch{"HM33 IP4"}="3HM3 IP4";
$resn_exch{"HN11 IP6"}="1HN1 IP6"; $resn_exch{"HN12 IP6"}="2HN1 IP6"; $resn_exch{" H91 IP6"}="1H9  IP6";
$resn_exch{" H92 IP6"}="2H9  IP6"; $resn_exch{"HN11 IP7"}="1HN1 IP7"; $resn_exch{"HN12 IP7"}="2HN1 IP7";
$resn_exch{" H91 IP7"}="1H9  IP7"; $resn_exch{" H92 IP7"}="2H9  IP7"; $resn_exch{"H171 IP7"}="1H17 IP7";
$resn_exch{"H172 IP7"}="2H17 IP7"; $resn_exch{" H11 IPA"}="1H1  IPA"; $resn_exch{" H12 IPA"}="2H1  IPA";
$resn_exch{" H13 IPA"}="3H1  IPA"; $resn_exch{" H31 IPA"}="1H3  IPA"; $resn_exch{" H32 IPA"}="2H3  IPA";
$resn_exch{" H33 IPA"}="3H3  IPA"; $resn_exch{" H73 IPB"}="3H7  IPB"; $resn_exch{" H72 IPB"}="2H7  IPB";
$resn_exch{" H71 IPB"}="1H7  IPB"; $resn_exch{" H93 IPB"}="3H9  IPB"; $resn_exch{" H92 IPB"}="2H9  IPB";
$resn_exch{" H91 IPB"}="1H9  IPB"; $resn_exch{"H103 IPB"}="3H10 IPB"; $resn_exch{"H102 IPB"}="2H10 IPB";
$resn_exch{"H101 IPB"}="1H10 IPB"; $resn_exch{" H31 IPC"}="1H3  IPC"; $resn_exch{" H32 IPC"}="2H3  IPC";
$resn_exch{" H33 IPC"}="3H3  IPC"; $resn_exch{" H41 IPC"}="1H4  IPC"; $resn_exch{" H42 IPC"}="2H4  IPC";
$resn_exch{" H43 IPC"}="3H4  IPC"; $resn_exch{"H131 IPC"}="1H13 IPC"; $resn_exch{"H132 IPC"}="2H13 IPC";
$resn_exch{"H133 IPC"}="3H13 IPC"; $resn_exch{" H11 IPE"}="1H1  IPE"; $resn_exch{" H12 IPE"}="2H1  IPE";
$resn_exch{" H21 IPE"}="1H2  IPE"; $resn_exch{" H22 IPE"}="2H2  IPE"; $resn_exch{" H41 IPE"}="1H4  IPE";
$resn_exch{" H42 IPE"}="2H4  IPE"; $resn_exch{" H51 IPE"}="1H5  IPE"; $resn_exch{" H52 IPE"}="2H5  IPE";
$resn_exch{" H53 IPE"}="3H5  IPE"; $resn_exch{" H21 IPF"}="1H2  IPF"; $resn_exch{"H22A IPF"}="2H2  IPF";
$resn_exch{"H23A IPF"}="3H2  IPF"; $resn_exch{"H41A IPF"}="1H4  IPF"; $resn_exch{" H42 IPF"}="2H4  IPF";
$resn_exch{"H161 IPF"}="1H16 IPF"; $resn_exch{"H162 IPF"}="2H16 IPF"; $resn_exch{"H163 IPF"}="3H16 IPF";
$resn_exch{"H171 IPF"}="1H17 IPF"; $resn_exch{"H172 IPF"}="2H17 IPF"; $resn_exch{"H173 IPF"}="3H17 IPF";
$resn_exch{"H201 IPF"}="1H20 IPF"; $resn_exch{"H202 IPF"}="2H20 IPF"; $resn_exch{"H291 IPF"}="1H29 IPF";
$resn_exch{"H292 IPF"}="2H29 IPF"; $resn_exch{"H301 IPF"}="1H30 IPF"; $resn_exch{"H302 IPF"}="2H30 IPF";
$resn_exch{"H311 IPF"}="1H31 IPF"; $resn_exch{"H312 IPF"}="2H31 IPF"; $resn_exch{"H351 IPF"}="1H35 IPF";
$resn_exch{"H352 IPF"}="2H35 IPF"; $resn_exch{"H353 IPF"}="3H35 IPF"; $resn_exch{"H361 IPF"}="1H36 IPF";
$resn_exch{"H362 IPF"}="2H36 IPF"; $resn_exch{"H363 IPF"}="3H36 IPF"; $resn_exch{"H391 IPF"}="1H39 IPF";
$resn_exch{"H392 IPF"}="2H39 IPF"; $resn_exch{"H501 IPF"}="1H50 IPF"; $resn_exch{"H502 IPF"}="2H50 IPF";
$resn_exch{"H503 IPF"}="3H50 IPF"; $resn_exch{" HA1 IPG"}="1HA  IPG"; $resn_exch{" HA2 IPG"}="2HA  IPG";
$resn_exch{"HG11 IPG"}="1HG1 IPG"; $resn_exch{"HG12 IPG"}="2HG1 IPG"; $resn_exch{"HG13 IPG"}="3HG1 IPG";
$resn_exch{"HG21 IPG"}="1HG2 IPG"; $resn_exch{"HG22 IPG"}="2HG2 IPG"; $resn_exch{"HG23 IPG"}="3HG2 IPG";
$resn_exch{"H1'1 IPL"}="1H1' IPL"; $resn_exch{"H1'2 IPL"}="2H1' IPL"; $resn_exch{"H2'1 IPL"}="1H2' IPL";
$resn_exch{"H2'2 IPL"}="2H2' IPL"; $resn_exch{"H3'1 IPL"}="1H3' IPL"; $resn_exch{"H3'2 IPL"}="2H3' IPL";
$resn_exch{"HOP2 IPL"}="2HOP IPL"; $resn_exch{"HOP3 IPL"}="3HOP IPL"; $resn_exch{" H61 IPM"}="1H6  IPM";
$resn_exch{" H62 IPM"}="2H6  IPM"; $resn_exch{" H63 IPM"}="3H6  IPM"; $resn_exch{" H71 IPM"}="1H7  IPM";
$resn_exch{" H72 IPM"}="2H7  IPM"; $resn_exch{" H73 IPM"}="3H7  IPM"; $resn_exch{"H8'1 IPN"}="1H8' IPN";
$resn_exch{"H8'2 IPN"}="2H8' IPN"; $resn_exch{"H5'1 IPN"}="1H5' IPN"; $resn_exch{"H5'2 IPN"}="2H5' IPN";
$resn_exch{"HO2' IPN"}="'HO2 IPN"; $resn_exch{"H3'1 IPN"}="1H3' IPN"; $resn_exch{"H3'2 IPN"}="2H3' IPN";
$resn_exch{"H2'1 IPN"}="1H2' IPN"; $resn_exch{"H2'2 IPN"}="2H2' IPN"; $resn_exch{"H1'1 IPN"}="1H1' IPN";
$resn_exch{"H1'2 IPN"}="2H1' IPN"; $resn_exch{"H1'3 IPN"}="3H1' IPN"; $resn_exch{"H1'1 IPO"}="1H1' IPO";
$resn_exch{"H1'2 IPO"}="2H1' IPO"; $resn_exch{"HN21 IPO"}="1HN2 IPO"; $resn_exch{"HN22 IPO"}="2HN2 IPO";
$resn_exch{"HOP1 IPP"}="1HOP IPP"; $resn_exch{" H1  IPP"}="1H   IPP"; $resn_exch{" H2  IPP"}="2H   IPP";
$resn_exch{" H21 IPP"}="1H2  IPP"; $resn_exch{" H22 IPP"}="2H2  IPP"; $resn_exch{"HC51 IPR"}="1HC5 IPR";
$resn_exch{"HC52 IPR"}="2HC5 IPR"; $resn_exch{"HC41 IPR"}="1HC4 IPR"; $resn_exch{"HC42 IPR"}="2HC4 IPR";
$resn_exch{"HC11 IPR"}="1HC1 IPR"; $resn_exch{"HC12 IPR"}="2HC1 IPR"; $resn_exch{"HC13 IPR"}="3HC1 IPR";
$resn_exch{"HC31 IPR"}="1HC3 IPR"; $resn_exch{"HC32 IPR"}="2HC3 IPR"; $resn_exch{"HC33 IPR"}="3HC3 IPR";
$resn_exch{" H61 IPT"}="1H6  IPT"; $resn_exch{" H62 IPT"}="2H6  IPT"; $resn_exch{"H2'1 IPT"}="1H2' IPT";
$resn_exch{"H2'2 IPT"}="2H2' IPT"; $resn_exch{"H2'3 IPT"}="3H2' IPT"; $resn_exch{"H3'1 IPT"}="1H3' IPT";
$resn_exch{"H3'2 IPT"}="2H3' IPT"; $resn_exch{"H3'3 IPT"}="3H3' IPT"; $resn_exch{" H11 IPU"}="1H1  IPU";
$resn_exch{" H12 IPU"}="2H1  IPU"; $resn_exch{" H13 IPU"}="3H1  IPU"; $resn_exch{" H31 IPU"}="1H3  IPU";
$resn_exch{" H32 IPU"}="2H3  IPU"; $resn_exch{" H33 IPU"}="3H3  IPU"; $resn_exch{"HN21 IPU"}="1HN2 IPU";
$resn_exch{"HN22 IPU"}="2HN2 IPU"; $resn_exch{" H31 IPY"}="1H3  IPY"; $resn_exch{"H32A IPY"}="2H3  IPY";
$resn_exch{" H33 IPY"}="3H3  IPY"; $resn_exch{" H91 IPY"}="1H9  IPY"; $resn_exch{" H92 IPY"}="2H9  IPY";
$resn_exch{" H93 IPY"}="3H9  IPY"; $resn_exch{"H151 IPY"}="1H15 IPY"; $resn_exch{"H152 IPY"}="2H15 IPY";
$resn_exch{"H153 IPY"}="3H15 IPY"; $resn_exch{"H181 IPY"}="1H18 IPY"; $resn_exch{"H182 IPY"}="2H18 IPY";
$resn_exch{"H191 IPY"}="1H19 IPY"; $resn_exch{"H192 IPY"}="2H19 IPY"; $resn_exch{"H201 IPY"}="1H20 IPY";
$resn_exch{"H202 IPY"}="2H20 IPY"; $resn_exch{"H211 IPY"}="1H21 IPY"; $resn_exch{"H212 IPY"}="2H21 IPY";
$resn_exch{"H213 IPY"}="3H21 IPY"; $resn_exch{"H221 IPY"}="1H22 IPY"; $resn_exch{"H222 IPY"}="2H22 IPY";
$resn_exch{"H223 IPY"}="3H22 IPY"; $resn_exch{"H281 IPY"}="1H28 IPY"; $resn_exch{"H282 IPY"}="2H28 IPY";
$resn_exch{"H283 IPY"}="3H28 IPY"; $resn_exch{"H291 IPY"}="1H29 IPY"; $resn_exch{"H292 IPY"}="2H29 IPY";
$resn_exch{"H301 IPY"}="1H30 IPY"; $resn_exch{"H302 IPY"}="2H30 IPY"; $resn_exch{" H91 IPZ"}="1H9  IPZ";
$resn_exch{" H92 IPZ"}="2H9  IPZ"; $resn_exch{" H93 IPZ"}="3H9  IPZ"; $resn_exch{"H101 IPZ"}="1H10 IPZ";
$resn_exch{"H102 IPZ"}="2H10 IPZ"; $resn_exch{"H103 IPZ"}="3H10 IPZ"; $resn_exch{"H111 IPZ"}="1H11 IPZ";
$resn_exch{"H112 IPZ"}="2H11 IPZ"; $resn_exch{"H113 IPZ"}="3H11 IPZ"; $resn_exch{"H181 IQA"}="1H18 IQA";
$resn_exch{"H182 IQA"}="2H18 IQA"; $resn_exch{"H2'1 IQB"}="1H2' IQB"; $resn_exch{"H2'2 IQB"}="2H2' IQB";
$resn_exch{"H3'1 IQB"}="1H3' IQB"; $resn_exch{"H3'2 IQB"}="2H3' IQB"; $resn_exch{"H5'1 IQB"}="1H5' IQB";
$resn_exch{"H5'2 IQB"}="2H5' IQB"; $resn_exch{" H2' IQB"}=" H2* IQB"; $resn_exch{" H3' IQB"}=" H3* IQB";
$resn_exch{" H5' IQB"}=" H5* IQB"; $resn_exch{" H6B IQB"}=" H6* IQB"; $resn_exch{"HM21 IQP"}="1HM2 IQP";
$resn_exch{"HM22 IQP"}="2HM2 IQP"; $resn_exch{"HM23 IQP"}="3HM2 IQP"; $resn_exch{"H3'1 IQP"}="1H3' IQP";
$resn_exch{"H3'2 IQP"}="2H3' IQP"; $resn_exch{"H5'1 IQP"}="1H5' IQP"; $resn_exch{"H5'2 IQP"}="2H5' IQP";
$resn_exch{"H6'1 IQP"}="1H6' IQP"; $resn_exch{"H6'2 IQP"}="2H6' IQP"; $resn_exch{"H2'1 IQS"}="1H2' IQS";
$resn_exch{"H2'2 IQS"}="2H2' IQS"; $resn_exch{"H3'1 IQS"}="1H3' IQS"; $resn_exch{"H3'2 IQS"}="2H3' IQS";
$resn_exch{"H5'1 IQS"}="1H5' IQS"; $resn_exch{"H5'2 IQS"}="2H5' IQS"; $resn_exch{"H5'3 IQS"}="3H5' IQS";
$resn_exch{"H2'1 IQU"}="1H2' IQU"; $resn_exch{"H2'2 IQU"}="2H2' IQU"; $resn_exch{"H3'1 IQU"}="1H3' IQU";
$resn_exch{"H3'2 IQU"}="2H3' IQU"; $resn_exch{"H4'1 IQU"}="1H4' IQU"; $resn_exch{"H4'2 IQU"}="2H4' IQU";
$resn_exch{"H121 IQZ"}="1H12 IQZ"; $resn_exch{"H122 IQZ"}="2H12 IQZ"; $resn_exch{"H161 IQZ"}="1H16 IQZ";
$resn_exch{"H162 IQZ"}="2H16 IQZ"; $resn_exch{"H171 IQZ"}="1H17 IQZ"; $resn_exch{"H172 IQZ"}="2H17 IQZ";
$resn_exch{"H181 IQZ"}="1H18 IQZ"; $resn_exch{"H182 IQZ"}="2H18 IQZ"; $resn_exch{"H191 IQZ"}="1H19 IQZ";
$resn_exch{"H192 IQZ"}="2H19 IQZ"; $resn_exch{"HAO1 IRE"}="1HAO IRE"; $resn_exch{"HAO2 IRE"}="2HAO IRE";
$resn_exch{"HAL1 IRE"}="1HAL IRE"; $resn_exch{"HAL2 IRE"}="2HAL IRE"; $resn_exch{"HAM1 IRE"}="1HAM IRE";
$resn_exch{"HAM2 IRE"}="2HAM IRE"; $resn_exch{"HAP1 IRE"}="1HAP IRE"; $resn_exch{"HAP2 IRE"}="2HAP IRE";
$resn_exch{"HAN1 IRE"}="1HAN IRE"; $resn_exch{"HAN2 IRE"}="2HAN IRE"; $resn_exch{"HAJ1 IRE"}="1HAJ IRE";
$resn_exch{"HAJ2 IRE"}="2HAJ IRE"; $resn_exch{"HAK1 IRE"}="1HAK IRE"; $resn_exch{"HAK2 IRE"}="2HAK IRE";
$resn_exch{"HAA1 IRE"}="1HAA IRE"; $resn_exch{"HAA2 IRE"}="2HAA IRE"; $resn_exch{"HAA3 IRE"}="3HAA IRE";
$resn_exch{"H7M1 IRF"}="1H7M IRF"; $resn_exch{"H7M2 IRF"}="2H7M IRF"; $resn_exch{"H7M3 IRF"}="3H7M IRF";
$resn_exch{"H1'1 IRF"}="1H1* IRF"; $resn_exch{"H1'2 IRF"}="2H1* IRF"; $resn_exch{" H2' IRF"}=" H2* IRF";
$resn_exch{" H3' IRF"}=" H3* IRF"; $resn_exch{" H4' IRF"}=" H4* IRF"; $resn_exch{"H5'1 IRF"}="1H5* IRF";
$resn_exch{"H5'2 IRF"}="2H5* IRF"; $resn_exch{" H5' IRF"}=" H5* IRF"; $resn_exch{"HN11 IRI"}="1HN1 IRI";
$resn_exch{"HN12 IRI"}="2HN1 IRI"; $resn_exch{"HN13 IRI"}="3HN1 IRI"; $resn_exch{"HN21 IRI"}="1HN2 IRI";
$resn_exch{"HN22 IRI"}="2HN2 IRI"; $resn_exch{"HN23 IRI"}="3HN2 IRI"; $resn_exch{"HN31 IRI"}="1HN3 IRI";
$resn_exch{"HN32 IRI"}="2HN3 IRI"; $resn_exch{"HN33 IRI"}="3HN3 IRI"; $resn_exch{"HN41 IRI"}="1HN4 IRI";
$resn_exch{"HN42 IRI"}="2HN4 IRI"; $resn_exch{"HN43 IRI"}="3HN4 IRI"; $resn_exch{"HN51 IRI"}="1HN5 IRI";
$resn_exch{"HN52 IRI"}="2HN5 IRI"; $resn_exch{"HN53 IRI"}="3HN5 IRI"; $resn_exch{"HN61 IRI"}="1HN6 IRI";
$resn_exch{"HN62 IRI"}="2HN6 IRI"; $resn_exch{"HN63 IRI"}="3HN6 IRI"; $resn_exch{"H5'1 IRP"}="1H5* IRP";
$resn_exch{"H5'2 IRP"}="2H5* IRP"; $resn_exch{" H4' IRP"}=" H4* IRP"; $resn_exch{"HN'4 IRP"}="4HN* IRP";
$resn_exch{" H3' IRP"}=" H3* IRP"; $resn_exch{"HO'3 IRP"}="3HO* IRP"; $resn_exch{" H2' IRP"}=" H2* IRP";
$resn_exch{"HO'2 IRP"}="2HO* IRP"; $resn_exch{" H1' IRP"}=" H1* IRP"; $resn_exch{"H181 IS2"}="1H18 IS2";
$resn_exch{"H182 IS2"}="2H18 IS2"; $resn_exch{"H191 IS2"}="1H19 IS2"; $resn_exch{"H192 IS2"}="2H19 IS2";
$resn_exch{"H193 IS2"}="3H19 IS2"; $resn_exch{" H41 ISA"}="1H4  ISA"; $resn_exch{" H42 ISA"}="2H4  ISA";
$resn_exch{"HB11 ISB"}="1HB1 ISB"; $resn_exch{"HB12 ISB"}="2HB1 ISB"; $resn_exch{"HB13 ISB"}="3HB1 ISB";
$resn_exch{"HB21 ISB"}="1HB2 ISB"; $resn_exch{"HB22 ISB"}="2HB2 ISB"; $resn_exch{"HB23 ISB"}="3HB2 ISB";
$resn_exch{"H3'1 ISC"}="1H3' ISC"; $resn_exch{"H3'2 ISC"}="2H3' ISC"; $resn_exch{"H131 ISD"}="1H13 ISD";
$resn_exch{"H132 ISD"}="2H13 ISD"; $resn_exch{"H151 ISF"}="1H15 ISF"; $resn_exch{"H152 ISF"}="2H15 ISF";
$resn_exch{"H153 ISF"}="3H15 ISF"; $resn_exch{"HN11 ISO"}="1HN1 ISO"; $resn_exch{"HN12 ISO"}="2HN1 ISO";
$resn_exch{"H2'1 ISO"}="1H2' ISO"; $resn_exch{"H2'2 ISO"}="2H2' ISO"; $resn_exch{"H2'3 ISO"}="3H2' ISO";
$resn_exch{"H3'1 ISO"}="1H3' ISO"; $resn_exch{"H3'2 ISO"}="2H3' ISO"; $resn_exch{"H3'3 ISO"}="3H3' ISO";
$resn_exch{" H11 ISP"}="1H1  ISP"; $resn_exch{" H12 ISP"}="2H1  ISP"; $resn_exch{" H13 ISP"}="3H1  ISP";
$resn_exch{" H31 ISP"}="1H3  ISP"; $resn_exch{" H32 ISP"}="2H3  ISP"; $resn_exch{" H33 ISP"}="3H3  ISP";
$resn_exch{"HOP2 ISP"}="2HOP ISP"; $resn_exch{"HOP3 ISP"}="3HOP ISP"; $resn_exch{" H21 IST"}="1H2  IST";
$resn_exch{" H23 IST"}="3H2  IST"; $resn_exch{" H22 IST"}="2H2  IST"; $resn_exch{" H51 IST"}="1H5  IST";
$resn_exch{" H52 IST"}="2H5  IST"; $resn_exch{"H101 IST"}="1H10 IST"; $resn_exch{"H102 IST"}="2H10 IST";
$resn_exch{"H103 IST"}="3H10 IST"; $resn_exch{"HA21 IST"}="1HA2 IST"; $resn_exch{"HA22 IST"}="2HA2 IST";
$resn_exch{" H11 IST"}="1H1  IST"; $resn_exch{" H12 IST"}="2H1  IST"; $resn_exch{"HC11 ISU"}="1HC1 ISU";
$resn_exch{"HC12 ISU"}="2HC1 ISU"; $resn_exch{"HC13 ISU"}="3HC1 ISU"; $resn_exch{"HC21 ISU"}="1HC2 ISU";
$resn_exch{"HC22 ISU"}="2HC2 ISU"; $resn_exch{"HN11 ISU"}="1HN1 ISU"; $resn_exch{"HN12 ISU"}="2HN1 ISU";
$resn_exch{"H5C1 ISX"}="1H5C ISX"; $resn_exch{"H5C2 ISX"}="2H5C ISX"; $resn_exch{"H1C1 ISX"}="1H1C ISX";
$resn_exch{"H1C2 ISX"}="2H1C ISX"; $resn_exch{"H6C1 ISX"}="1H6C ISX"; $resn_exch{"H6C2 ISX"}="2H6C ISX";
$resn_exch{"H6A1 ISX"}="1H6A ISX"; $resn_exch{"H6A2 ISX"}="2H6A ISX"; $resn_exch{"H211 ITA"}="1H21 ITA";
$resn_exch{"H212 ITA"}="2H21 ITA"; $resn_exch{"H101 ITA"}="1H10 ITA"; $resn_exch{"H102 ITA"}="2H10 ITA";
$resn_exch{"H201 ITB"}="1H20 ITB"; $resn_exch{"H202 ITB"}="2H20 ITB"; $resn_exch{" H41 ITM"}="1H4  ITM";
$resn_exch{" H42 ITM"}="2H4  ITM"; $resn_exch{" HB1 ITR"}="1HB  ITR"; $resn_exch{" HB2 ITR"}="2HB  ITR";
$resn_exch{"H5'1 ITT"}="1H5* ITT"; $resn_exch{"H5'2 ITT"}="2H5* ITT"; $resn_exch{" H4' ITT"}=" H4* ITT";
$resn_exch{" H3' ITT"}=" H3* ITT"; $resn_exch{" H2' ITT"}=" H2* ITT"; $resn_exch{" H1' ITT"}=" H1* ITT";
$resn_exch{" H11 ITU"}="1H1  ITU"; $resn_exch{" H12 ITU"}="2H1  ITU"; $resn_exch{" H13 ITU"}="3H1  ITU";
$resn_exch{" H21 ITU"}="1H2  ITU"; $resn_exch{" H22 ITU"}="2H2  ITU"; $resn_exch{"HN11 ITU"}="1HN1 ITU";
$resn_exch{"HN12 ITU"}="2HN1 ITU"; $resn_exch{"HOP2  IU"}="2HOP  IU"; $resn_exch{"HOP3  IU"}="3HOP  IU";
$resn_exch{" H5'  IU"}="1H5*  IU"; $resn_exch{"H5''  IU"}="2H5*  IU"; $resn_exch{" H4'  IU"}=" H4*  IU";
$resn_exch{" H3'  IU"}=" H3*  IU"; $resn_exch{"HO3'  IU"}="H3T   IU"; $resn_exch{" H2'  IU"}=" H2*  IU";
$resn_exch{"HO2'  IU"}="2HO*  IU"; $resn_exch{" H1'  IU"}=" H1*  IU"; $resn_exch{"HC11 IU5"}="1HC1 IU5";
$resn_exch{"HC12 IU5"}="2HC1 IU5"; $resn_exch{"H111 IU5"}="1H11 IU5"; $resn_exch{"H112 IU5"}="2H11 IU5";
$resn_exch{"H121 IU5"}="1H12 IU5"; $resn_exch{"H122 IU5"}="2H12 IU5"; $resn_exch{"H141 IU5"}="1H14 IU5";
$resn_exch{"H151 IU5"}="1H15 IU5"; $resn_exch{"H152 IU5"}="2H15 IU5"; $resn_exch{"H161 IU5"}="1H16 IU5";
$resn_exch{"H162 IU5"}="2H16 IU5"; $resn_exch{"H171 IU5"}="1H17 IU5"; $resn_exch{"H181 IU5"}="1H18 IU5";
$resn_exch{"H182 IU5"}="2H18 IU5"; $resn_exch{"H183 IU5"}="3H18 IU5"; $resn_exch{"H191 IU5"}="1H19 IU5";
$resn_exch{"H192 IU5"}="2H19 IU5"; $resn_exch{"H193 IU5"}="3H19 IU5"; $resn_exch{"HOA1 IU5"}="1HOA IU5";
$resn_exch{"HOB1 IU5"}="1HOB IU5"; $resn_exch{"HC21 IU5"}="1HC2 IU5"; $resn_exch{"HC22 IU5"}="2HC2 IU5";
$resn_exch{"H201 IU5"}="1H20 IU5"; $resn_exch{"H211 IU5"}="1H21 IU5"; $resn_exch{"H212 IU5"}="2H21 IU5";
$resn_exch{"H213 IU5"}="3H21 IU5"; $resn_exch{"H221 IU5"}="1H22 IU5"; $resn_exch{"H222 IU5"}="2H22 IU5";
$resn_exch{"H231 IU5"}="1H23 IU5"; $resn_exch{"H232 IU5"}="2H23 IU5"; $resn_exch{"HC31 IU5"}="1HC3 IU5";
$resn_exch{"HC41 IU5"}="1HC4 IU5"; $resn_exch{"HC42 IU5"}="2HC4 IU5"; $resn_exch{"HOA4 IU5"}="4HOA IU5";
$resn_exch{"HC51 IU5"}="1HC5 IU5"; $resn_exch{"HC61 IU5"}="1HC6 IU5"; $resn_exch{"HC62 IU5"}="2HC6 IU5";
$resn_exch{"HC71 IU5"}="1HC7 IU5"; $resn_exch{"HC81 IU5"}="1HC8 IU5"; $resn_exch{"HC91 IU5"}="1HC9 IU5";
$resn_exch{"HC11 IU6"}="1HC1 IU6"; $resn_exch{"HC12 IU6"}="2HC1 IU6"; $resn_exch{"H111 IU6"}="1H11 IU6";
$resn_exch{"H112 IU6"}="2H11 IU6"; $resn_exch{"H121 IU6"}="1H12 IU6"; $resn_exch{"H122 IU6"}="2H12 IU6";
$resn_exch{"H141 IU6"}="1H14 IU6"; $resn_exch{"H151 IU6"}="1H15 IU6"; $resn_exch{"H152 IU6"}="2H15 IU6";
$resn_exch{"H161 IU6"}="1H16 IU6"; $resn_exch{"H162 IU6"}="2H16 IU6"; $resn_exch{"H171 IU6"}="1H17 IU6";
$resn_exch{"H181 IU6"}="1H18 IU6"; $resn_exch{"H182 IU6"}="2H18 IU6"; $resn_exch{"H183 IU6"}="3H18 IU6";
$resn_exch{"H191 IU6"}="1H19 IU6"; $resn_exch{"H192 IU6"}="2H19 IU6"; $resn_exch{"H193 IU6"}="3H19 IU6";
$resn_exch{"HOA1 IU6"}="1HOA IU6"; $resn_exch{"HOB1 IU6"}="1HOB IU6"; $resn_exch{"HC21 IU6"}="1HC2 IU6";
$resn_exch{"HC22 IU6"}="2HC2 IU6"; $resn_exch{"H201 IU6"}="1H20 IU6"; $resn_exch{"H211 IU6"}="1H21 IU6";
$resn_exch{"H212 IU6"}="2H21 IU6"; $resn_exch{"H213 IU6"}="3H21 IU6"; $resn_exch{"H221 IU6"}="1H22 IU6";
$resn_exch{"H222 IU6"}="2H22 IU6"; $resn_exch{"H231 IU6"}="1H23 IU6"; $resn_exch{"H232 IU6"}="2H23 IU6";
$resn_exch{"HC31 IU6"}="1HC3 IU6"; $resn_exch{"HC41 IU6"}="1HC4 IU6"; $resn_exch{"HC42 IU6"}="2HC4 IU6";
$resn_exch{"HOA4 IU6"}="4HOA IU6"; $resn_exch{"HC51 IU6"}="1HC5 IU6"; $resn_exch{"HC61 IU6"}="1HC6 IU6";
$resn_exch{"H261 IU6"}="1H26 IU6"; $resn_exch{"H262 IU6"}="2H26 IU6"; $resn_exch{"H251 IU6"}="1H25 IU6";
$resn_exch{"H252 IU6"}="2H25 IU6"; $resn_exch{"H253 IU6"}="3H25 IU6"; $resn_exch{"HC71 IU6"}="1HC7 IU6";
$resn_exch{"HC81 IU6"}="1HC8 IU6"; $resn_exch{"HC91 IU6"}="1HC9 IU6"; $resn_exch{" HA1 IVA"}="1HA  IVA";
$resn_exch{" HA2 IVA"}="2HA  IVA"; $resn_exch{"HG11 IVA"}="1HG1 IVA"; $resn_exch{"HG12 IVA"}="2HG1 IVA";
$resn_exch{"HG13 IVA"}="3HG1 IVA"; $resn_exch{"HG21 IVA"}="1HG2 IVA"; $resn_exch{"HG22 IVA"}="2HG2 IVA";
$resn_exch{"HG23 IVA"}="3HG2 IVA"; $resn_exch{"H571 IVS"}="1H57 IVS"; $resn_exch{"H572 IVS"}="2H57 IVS";
$resn_exch{"H521 IVS"}="1H52 IVS"; $resn_exch{"H522 IVS"}="2H52 IVS"; $resn_exch{"H511 IVS"}="1H51 IVS";
$resn_exch{"H231 IVS"}="1H23 IVS"; $resn_exch{"H581 IVS"}="1H58 IVS"; $resn_exch{"H582 IVS"}="2H58 IVS";
$resn_exch{"H221 IVS"}="1H22 IVS"; $resn_exch{"H201 IVS"}="1H20 IVS"; $resn_exch{"H191 IVS"}="1H19 IVS";
$resn_exch{"H171 IVS"}="1H17 IVS"; $resn_exch{"H161 IVS"}="1H16 IVS"; $resn_exch{"H101 IVS"}="1H10 IVS";
$resn_exch{"H102 IVS"}="2H10 IVS"; $resn_exch{"HC21 IVS"}="1HC2 IVS"; $resn_exch{"HC61 IVS"}="1HC6 IVS";
$resn_exch{"HC62 IVS"}="2HC6 IVS"; $resn_exch{"HC63 IVS"}="3HC6 IVS"; $resn_exch{"H761 IVS"}="1H76 IVS";
$resn_exch{"H762 IVS"}="2H76 IVS"; $resn_exch{"H763 IVS"}="3H76 IVS"; $resn_exch{"H601 IVS"}="1H60 IVS";
$resn_exch{"H671 IVS"}="1H67 IVS"; $resn_exch{"H672 IVS"}="2H67 IVS"; $resn_exch{"H673 IVS"}="3H67 IVS";
$resn_exch{"H631 IVS"}="1H63 IVS"; $resn_exch{"H632 IVS"}="2H63 IVS"; $resn_exch{"H633 IVS"}="3H63 IVS";
$resn_exch{"H381 IVS"}="1H38 IVS"; $resn_exch{"H401 IVS"}="1H40 IVS"; $resn_exch{"H402 IVS"}="2H40 IVS";
$resn_exch{"H403 IVS"}="3H40 IVS"; $resn_exch{"H441 IVS"}="1H44 IVS"; $resn_exch{"H442 IVS"}="2H44 IVS";
$resn_exch{"H443 IVS"}="3H44 IVS"; $resn_exch{"H251 IVS"}="1H25 IVS"; $resn_exch{"H271 IVS"}="1H27 IVS";
$resn_exch{"H272 IVS"}="2H27 IVS"; $resn_exch{"H273 IVS"}="3H27 IVS"; $resn_exch{"H311 IVS"}="1H31 IVS";
$resn_exch{"H312 IVS"}="2H31 IVS"; $resn_exch{"H313 IVS"}="3H31 IVS"; $resn_exch{"HC11 IVS"}="1HC1 IVS";
$resn_exch{"HC12 IVS"}="2HC1 IVS"; $resn_exch{"HC13 IVS"}="3HC1 IVS"; $resn_exch{" H71 IWD"}="1H7  IWD";
$resn_exch{" H72 IWD"}="2H7  IWD"; $resn_exch{" H81 IWD"}="1H8  IWD"; $resn_exch{" H82 IWD"}="2H8  IWD";
$resn_exch{"H92O IWD"}="OH92 IWD"; $resn_exch{"H391 IX1"}="1H39 IX1"; $resn_exch{"H392 IX1"}="2H39 IX1";
$resn_exch{"H393 IX1"}="3H39 IX1"; $resn_exch{"H401 IX1"}="1H40 IX1"; $resn_exch{"H402 IX1"}="2H40 IX1";
$resn_exch{" H91 IX1"}="1H9  IX1"; $resn_exch{" H92 IX1"}="2H9  IX1"; $resn_exch{"H221 IXS"}="1H22 IXS";
$resn_exch{"H222 IXS"}="2H22 IXS"; $resn_exch{"H223 IXS"}="3H22 IXS"; $resn_exch{"H291 IXS"}="1H29 IXS";
$resn_exch{"H292 IXS"}="2H29 IXS"; $resn_exch{"H293 IXS"}="3H29 IXS"; $resn_exch{"H501 IXS"}="1H50 IXS";
$resn_exch{"H502 IXS"}="2H50 IXS"; $resn_exch{"H503 IXS"}="3H50 IXS"; $resn_exch{"H561 IXS"}="1H56 IXS";
$resn_exch{"H562 IXS"}="2H56 IXS"; $resn_exch{"H371 IXS"}="1H37 IXS"; $resn_exch{"H372 IXS"}="2H37 IXS";
$resn_exch{" H81 IXS"}="1H8  IXS"; $resn_exch{" H82 IXS"}="2H8  IXS"; $resn_exch{" H83 IXS"}="3H8  IXS";
$resn_exch{"H101 IXS"}="1H10 IXS"; $resn_exch{"H102 IXS"}="2H10 IXS"; $resn_exch{"H521 IXS"}="1H52 IXS";
$resn_exch{"H522 IXS"}="2H52 IXS"; $resn_exch{"H523 IXS"}="3H52 IXS"; $resn_exch{"H531 IXS"}="1H53 IXS";
$resn_exch{"H532 IXS"}="2H53 IXS"; $resn_exch{"H533 IXS"}="3H53 IXS"; $resn_exch{" H61 IXX"}="1H6  IXX";
$resn_exch{" H51 IXX"}="1H5  IXX"; $resn_exch{"H151 IXX"}="1H15 IXX"; $resn_exch{"H152 IXX"}="2H15 IXX";
$resn_exch{"H161 IXX"}="1H16 IXX"; $resn_exch{"H162 IXX"}="2H16 IXX"; $resn_exch{"H171 IXX"}="1H17 IXX";
$resn_exch{"H172 IXX"}="2H17 IXX"; $resn_exch{"H181 IXX"}="1H18 IXX"; $resn_exch{"H182 IXX"}="2H18 IXX";
$resn_exch{"H183 IXX"}="3H18 IXX"; $resn_exch{"H191 IXX"}="1H19 IXX"; $resn_exch{"H192 IXX"}="2H19 IXX";
$resn_exch{"H193 IXX"}="3H19 IXX"; $resn_exch{" H52 IXX"}="2H5  IXX"; $resn_exch{" H62 IXX"}="2H6  IXX";
$resn_exch{"HAN1 IYG"}="1HAN IYG"; $resn_exch{"HAN2 IYG"}="2HAN IYG"; $resn_exch{"HAN3 IYG"}="3HAN IYG";
$resn_exch{" H1O IYG"}="1OH  IYG"; $resn_exch{" HT1 IYG"}="1HT  IYG"; $resn_exch{"HAT1 IYG"}="1HAT IYG";
$resn_exch{"HAT2 IYG"}="2HAT IYG"; $resn_exch{" H2  IYR"}=" HN2 IYR"; $resn_exch{" HB2 IYR"}="1HB  IYR";
$resn_exch{" HB3 IYR"}="2HB  IYR"; $resn_exch{"HAN1 IYT"}="1HAN IYT"; $resn_exch{"HAN2 IYT"}="2HAN IYT";
$resn_exch{"HAN3 IYT"}="3HAN IYT"; $resn_exch{" H   IYT"}=" HN  IYT"; $resn_exch{" HB2 IYT"}="1HB  IYT";
$resn_exch{" HB3 IYT"}="2HB  IYT"; $resn_exch{" HH  IYT"}=" HOH IYT"; $resn_exch{"HGT1 IYT"}="1HGT IYT";
$resn_exch{"HGT2 IYT"}="2HGT IYT"; $resn_exch{"HGT3 IYT"}="3HGT IYT"; $resn_exch{"HAJ1 IYZ"}="1HAJ IYZ";
$resn_exch{"HAJ2 IYZ"}="2HAJ IYZ"; $resn_exch{"HAK1 IYZ"}="1HAK IYZ"; $resn_exch{"HAK2 IYZ"}="2HAK IYZ";
$resn_exch{"HAL1 IYZ"}="1HAL IYZ"; $resn_exch{"HAL2 IYZ"}="2HAL IYZ"; $resn_exch{"HAA1 IYZ"}="1HAA IYZ";
$resn_exch{"HAA2 IYZ"}="2HAA IYZ"; $resn_exch{"HAA3 IYZ"}="3HAA IYZ"; $resn_exch{"H2C1 IZ1"}="1H2C IZ1";
$resn_exch{"H2C2 IZ1"}="2H2C IZ1"; $resn_exch{"H211 IZ1"}="1H21 IZ1"; $resn_exch{"H212 IZ1"}="2H21 IZ1";
$resn_exch{"H2C1 IZ2"}="1H2C IZ2"; $resn_exch{"H2C2 IZ2"}="2H2C IZ2"; $resn_exch{"H211 IZ2"}="1H21 IZ2";
$resn_exch{"H212 IZ2"}="2H21 IZ2"; $resn_exch{"H5C1 IZ2"}="1H5C IZ2"; $resn_exch{"H5C2 IZ2"}="2H5C IZ2";
$resn_exch{"H2C1 IZ3"}="1H2C IZ3"; $resn_exch{"H2C2 IZ3"}="2H2C IZ3"; $resn_exch{"H211 IZ3"}="1H21 IZ3";
$resn_exch{"H212 IZ3"}="2H21 IZ3"; $resn_exch{"H181 IZ3"}="1H18 IZ3"; $resn_exch{"H182 IZ3"}="2H18 IZ3";
$resn_exch{"H2C1 IZ4"}="1H2C IZ4"; $resn_exch{"H2C2 IZ4"}="2H2C IZ4"; $resn_exch{"H211 IZ4"}="1H21 IZ4";
$resn_exch{"H212 IZ4"}="2H21 IZ4"; $resn_exch{"H4C1 IZ4"}="1H4C IZ4"; $resn_exch{"H4C2 IZ4"}="2H4C IZ4";
$resn_exch{"H181 IZ4"}="1H18 IZ4"; $resn_exch{"H182 IZ4"}="2H18 IZ4"; $resn_exch{"H271 IZ4"}="1H27 IZ4";
$resn_exch{"H272 IZ4"}="2H27 IZ4"; $resn_exch{"H2C1 IZ5"}="1H2C IZ5"; $resn_exch{"H2C2 IZ5"}="2H2C IZ5";
$resn_exch{"H211 IZ5"}="1H21 IZ5"; $resn_exch{"H212 IZ5"}="2H21 IZ5"; $resn_exch{"H5C1 IZ5"}="1H5C IZ5";
$resn_exch{"H5C2 IZ5"}="2H5C IZ5"; $resn_exch{"H301 IZ5"}="1H30 IZ5"; $resn_exch{"H302 IZ5"}="2H30 IZ5";
$resn_exch{"H291 IZ5"}="1H29 IZ5"; $resn_exch{"H292 IZ5"}="2H29 IZ5"; $resn_exch{"H171 IZA"}="1H17 IZA";
$resn_exch{"H172 IZA"}="2H17 IZA"; $resn_exch{"H173 IZA"}="3H17 IZA"; $resn_exch{"H161 IZA"}="1H16 IZA";
$resn_exch{"H162 IZA"}="2H16 IZA"; $resn_exch{"H163 IZA"}="3H16 IZA"; $resn_exch{"H151 IZA"}="1H15 IZA";
$resn_exch{"H152 IZA"}="2H15 IZA"; $resn_exch{"H153 IZA"}="3H15 IZA"; $resn_exch{"H2C1 IZB"}="1H2C IZB";
$resn_exch{"H2C2 IZB"}="2H2C IZB"; $resn_exch{"H211 IZB"}="1H21 IZB"; $resn_exch{"H212 IZB"}="2H21 IZB";
$resn_exch{"H181 IZB"}="1H18 IZB"; $resn_exch{"H182 IZB"}="2H18 IZB"; $resn_exch{"H271 IZB"}="1H27 IZB";
$resn_exch{"H272 IZB"}="2H27 IZB"; $resn_exch{"H201 IZB"}="1H20 IZB"; $resn_exch{"H202 IZB"}="2H20 IZB";
$resn_exch{"H203 IZB"}="3H20 IZB"; $resn_exch{" H21 IZC"}="1H2  IZC"; $resn_exch{" H22 IZC"}="2H2  IZC";
$resn_exch{" H61 IZC"}="1H6  IZC"; $resn_exch{" H62 IZC"}="2H6  IZC"; $resn_exch{"H2C1 IZD"}="1H2C IZD";
$resn_exch{"H2C2 IZD"}="2H2C IZD"; $resn_exch{"H211 IZD"}="1H21 IZD"; $resn_exch{"H212 IZD"}="2H21 IZD";
$resn_exch{"H271 IZD"}="1H27 IZD"; $resn_exch{"H272 IZD"}="2H27 IZD"; $resn_exch{"H4C1 IZD"}="1H4C IZD";
$resn_exch{"H4C2 IZD"}="2H4C IZD"; $resn_exch{"H711 IZD"}="1H71 IZD"; $resn_exch{"H712 IZD"}="2H71 IZD";
$resn_exch{"H713 IZD"}="3H71 IZD"; $resn_exch{"H2C1 IZE"}="1H2C IZE"; $resn_exch{"H2C2 IZE"}="2H2C IZE";
$resn_exch{"H211 IZE"}="1H21 IZE"; $resn_exch{"H212 IZE"}="2H21 IZE"; $resn_exch{"H2C1 IZF"}="1H2C IZF";
$resn_exch{"H2C2 IZF"}="2H2C IZF"; $resn_exch{"H211 IZF"}="1H21 IZF"; $resn_exch{"H212 IZF"}="2H21 IZF";
$resn_exch{"H231 IZF"}="1H23 IZF"; $resn_exch{"H232 IZF"}="2H23 IZF"; $resn_exch{"H271 IZF"}="1H27 IZF";
$resn_exch{"H272 IZF"}="2H27 IZF"; $resn_exch{"H181 IZF"}="1H18 IZF"; $resn_exch{"H182 IZF"}="2H18 IZF";
$resn_exch{"H4C1 IZF"}="1H4C IZF"; $resn_exch{"H4C2 IZF"}="2H4C IZF"; $resn_exch{"H5C1 IZF"}="1H5C IZF";
$resn_exch{"H5C2 IZF"}="2H5C IZF"; $resn_exch{"H391 IZF"}="1H39 IZF"; $resn_exch{"H392 IZF"}="2H39 IZF";
$resn_exch{"H393 IZF"}="3H39 IZF"; $resn_exch{"H4C1 J01"}="1H4C J01"; $resn_exch{"H4C2 J01"}="2H4C J01";
$resn_exch{"H7C1 J01"}="1H7C J01"; $resn_exch{"H7C2 J01"}="2H7C J01"; $resn_exch{"H011 J07"}="1H01 J07";
$resn_exch{"H012 J07"}="2H01 J07"; $resn_exch{"H013 J07"}="3H01 J07"; $resn_exch{"H021 J07"}="1H02 J07";
$resn_exch{"H022 J07"}="2H02 J07"; $resn_exch{"H071 J07"}="1H07 J07"; $resn_exch{"H072 J07"}="2H07 J07";
$resn_exch{"H081 J07"}="1H08 J07"; $resn_exch{"H082 J07"}="2H08 J07"; $resn_exch{"H101 J07"}="1H10 J07";
$resn_exch{"H102 J07"}="2H10 J07"; $resn_exch{"H111 J07"}="1H11 J07"; $resn_exch{"H112 J07"}="2H11 J07";
$resn_exch{"H191 J12"}="1H19 J12"; $resn_exch{"H192 J12"}="2H19 J12"; $resn_exch{"H201 J12"}="1H20 J12";
$resn_exch{"H202 J12"}="2H20 J12"; $resn_exch{"H211 J12"}="1H21 J12"; $resn_exch{"H212 J12"}="2H21 J12";
$resn_exch{"H231 J12"}="1H23 J12"; $resn_exch{"H232 J12"}="2H23 J12"; $resn_exch{"H241 J12"}="1H24 J12";
$resn_exch{"H242 J12"}="2H24 J12"; $resn_exch{"H261 J12"}="1H26 J12"; $resn_exch{"H262 J12"}="2H26 J12";
$resn_exch{"H271 J12"}="1H27 J12"; $resn_exch{"H272 J12"}="2H27 J12"; $resn_exch{"H281 J12"}="1H28 J12";
$resn_exch{"H282 J12"}="2H28 J12"; $resn_exch{"H351 J12"}="1H35 J12"; $resn_exch{"H352 J12"}="2H35 J12";
$resn_exch{"H361 J12"}="1H36 J12"; $resn_exch{"H362 J12"}="2H36 J12"; $resn_exch{"H371 J12"}="1H37 J12";
$resn_exch{"H372 J12"}="2H37 J12"; $resn_exch{" H61 J12"}="1H6  J12"; $resn_exch{" H62 J12"}="2H6  J12";
$resn_exch{"H421 J12"}="1H42 J12"; $resn_exch{"H422 J12"}="2H42 J12"; $resn_exch{"H191 J15"}="1H19 J15";
$resn_exch{"H192 J15"}="2H19 J15"; $resn_exch{"H201 J15"}="1H20 J15"; $resn_exch{"H202 J15"}="2H20 J15";
$resn_exch{"H211 J15"}="1H21 J15"; $resn_exch{"H212 J15"}="2H21 J15"; $resn_exch{"H231 J15"}="1H23 J15";
$resn_exch{"H232 J15"}="2H23 J15"; $resn_exch{"H241 J15"}="1H24 J15"; $resn_exch{"H242 J15"}="2H24 J15";
$resn_exch{"H281 J15"}="1H28 J15"; $resn_exch{"H282 J15"}="2H28 J15"; $resn_exch{"H351 J15"}="1H35 J15";
$resn_exch{"H352 J15"}="2H35 J15"; $resn_exch{"H361 J15"}="1H36 J15"; $resn_exch{"H362 J15"}="2H36 J15";
$resn_exch{"H371 J15"}="1H37 J15"; $resn_exch{"H372 J15"}="2H37 J15"; $resn_exch{"H261 J15"}="1H26 J15";
$resn_exch{"H262 J15"}="2H26 J15"; $resn_exch{"H271 J15"}="1H27 J15"; $resn_exch{"H272 J15"}="2H27 J15";
$resn_exch{" H61 J15"}="1H6  J15"; $resn_exch{" H62 J15"}="2H6  J15"; $resn_exch{"HBA1 J60"}="1HBA J60";
$resn_exch{"HBA2 J60"}="2HBA J60"; $resn_exch{"HBA3 J60"}="3HBA J60"; $resn_exch{"HAZ1 J60"}="1HAZ J60";
$resn_exch{"HAZ2 J60"}="2HAZ J60"; $resn_exch{"HAY1 J60"}="1HAY J60"; $resn_exch{"HAY2 J60"}="2HAY J60";
$resn_exch{"HAW1 J60"}="1HAW J60"; $resn_exch{"HAW2 J60"}="2HAW J60"; $resn_exch{"HBB1 J60"}="1HBB J60";
$resn_exch{"HBB2 J60"}="2HBB J60"; $resn_exch{"HBB3 J60"}="3HBB J60"; $resn_exch{"HAV1 J60"}="1HAV J60";
$resn_exch{"HAV2 J60"}="2HAV J60"; $resn_exch{"HAQ1 J60"}="1HAQ J60"; $resn_exch{"HAQ2 J60"}="2HAQ J60";
$resn_exch{"HAQ3 J60"}="3HAQ J60"; $resn_exch{"HAR1 J60"}="1HAR J60"; $resn_exch{"HAR2 J60"}="2HAR J60";
$resn_exch{"HAR3 J60"}="3HAR J60"; $resn_exch{" H11 J77"}="1H1  J77"; $resn_exch{" H12 J77"}="2H1  J77";
$resn_exch{" H13 J77"}="3H1  J77"; $resn_exch{" H71 J77"}="1H7  J77"; $resn_exch{" H72 J77"}="2H7  J77";
$resn_exch{" H81 J77"}="1H8  J77"; $resn_exch{" H82 J77"}="2H8  J77"; $resn_exch{"H101 J77"}="1H10 J77";
$resn_exch{"H102 J77"}="2H10 J77"; $resn_exch{"H111 J77"}="1H11 J77"; $resn_exch{"H112 J77"}="2H11 J77";
$resn_exch{"H121 J77"}="1H12 J77"; $resn_exch{"H122 J77"}="2H12 J77"; $resn_exch{"H131 J77"}="1H13 J77";
$resn_exch{"H132 J77"}="2H13 J77"; $resn_exch{"H241 J77"}="1H24 J77"; $resn_exch{"H242 J77"}="2H24 J77";
$resn_exch{"H251 J77"}="1H25 J77"; $resn_exch{"H252 J77"}="2H25 J77"; $resn_exch{"H253 J77"}="3H25 J77";
$resn_exch{" H11 J78"}="1H1  J78"; $resn_exch{" H12 J78"}="2H1  J78"; $resn_exch{" H13 J78"}="3H1  J78";
$resn_exch{" H71 J78"}="1H7  J78"; $resn_exch{" H72 J78"}="2H7  J78"; $resn_exch{" H81 J78"}="1H8  J78";
$resn_exch{" H82 J78"}="2H8  J78"; $resn_exch{"H101 J78"}="1H10 J78"; $resn_exch{"H102 J78"}="2H10 J78";
$resn_exch{"H111 J78"}="1H11 J78"; $resn_exch{"H112 J78"}="2H11 J78"; $resn_exch{"H121 J78"}="1H12 J78";
$resn_exch{"H122 J78"}="2H12 J78"; $resn_exch{"H131 J78"}="1H13 J78"; $resn_exch{"H132 J78"}="2H13 J78";
$resn_exch{"H141 J78"}="1H14 J78"; $resn_exch{"H142 J78"}="2H14 J78"; $resn_exch{"H251 J78"}="1H25 J78";
$resn_exch{"H252 J78"}="2H25 J78"; $resn_exch{"H261 J78"}="1H26 J78"; $resn_exch{"H262 J78"}="2H26 J78";
$resn_exch{"H263 J78"}="3H26 J78"; $resn_exch{" H11 J80"}="1H1  J80"; $resn_exch{" H12 J80"}="2H1  J80";
$resn_exch{" H13 J80"}="3H1  J80"; $resn_exch{" H71 J80"}="1H7  J80"; $resn_exch{" H72 J80"}="2H7  J80";
$resn_exch{" H81 J80"}="1H8  J80"; $resn_exch{" H82 J80"}="2H8  J80"; $resn_exch{"H101 J80"}="1H10 J80";
$resn_exch{"H102 J80"}="2H10 J80"; $resn_exch{"H111 J80"}="1H11 J80"; $resn_exch{"H112 J80"}="2H11 J80";
$resn_exch{"H121 J80"}="1H12 J80"; $resn_exch{"H122 J80"}="2H12 J80"; $resn_exch{"H131 J80"}="1H13 J80";
$resn_exch{"H132 J80"}="2H13 J80"; $resn_exch{"H141 J80"}="1H14 J80"; $resn_exch{"H142 J80"}="2H14 J80";
$resn_exch{"H151 J80"}="1H15 J80"; $resn_exch{"H152 J80"}="2H15 J80"; $resn_exch{"H261 J80"}="1H26 J80";
$resn_exch{"H262 J80"}="2H26 J80"; $resn_exch{"H271 J80"}="1H27 J80"; $resn_exch{"H272 J80"}="2H27 J80";
$resn_exch{"H273 J80"}="3H27 J80"; $resn_exch{"H11A JAN"}="1H1  JAN"; $resn_exch{" H12 JAN"}="2H1  JAN";
$resn_exch{"H13A JAN"}="3H1  JAN"; $resn_exch{"HN31 JAN"}="1HN3 JAN"; $resn_exch{"HN32 JAN"}="2HN3 JAN";
$resn_exch{"H181 JAN"}="1H18 JAN"; $resn_exch{"H182 JAN"}="2H18 JAN"; $resn_exch{"H183 JAN"}="3H18 JAN";
$resn_exch{"H401 JAS"}="1H40 JAS"; $resn_exch{"H402 JAS"}="2H40 JAS"; $resn_exch{"H411 JAS"}="1H41 JAS";
$resn_exch{"H412 JAS"}="2H41 JAS"; $resn_exch{"H413 JAS"}="3H41 JAS"; $resn_exch{"H331 JAS"}="1H33 JAS";
$resn_exch{"H332 JAS"}="2H33 JAS"; $resn_exch{"H421 JAS"}="1H42 JAS"; $resn_exch{"H422 JAS"}="2H42 JAS";
$resn_exch{"H423 JAS"}="3H42 JAS"; $resn_exch{"H391 JAS"}="1H39 JAS"; $resn_exch{"H392 JAS"}="2H39 JAS";
$resn_exch{"H393 JAS"}="3H39 JAS"; $resn_exch{"H291 JAS"}="1H29 JAS"; $resn_exch{"H292 JAS"}="2H29 JAS";
$resn_exch{"H281 JAS"}="1H28 JAS"; $resn_exch{"H282 JAS"}="2H28 JAS"; $resn_exch{"H381 JAS"}="1H38 JAS";
$resn_exch{"H382 JAS"}="2H38 JAS"; $resn_exch{"H383 JAS"}="3H38 JAS"; $resn_exch{"H431 JAS"}="1H43 JAS";
$resn_exch{"H432 JAS"}="2H43 JAS"; $resn_exch{"H433 JAS"}="3H43 JAS"; $resn_exch{"H251 JAS"}="1H25 JAS";
$resn_exch{"H252 JAS"}="2H25 JAS"; $resn_exch{" H21 JAS"}="1H2  JAS"; $resn_exch{" H22 JAS"}="2H2  JAS";
$resn_exch{" H41 JAS"}="1H4  JAS"; $resn_exch{" H42 JAS"}="2H4  JAS"; $resn_exch{" H61 JAS"}="1H6  JAS";
$resn_exch{" H62 JAS"}="2H6  JAS"; $resn_exch{"H371 JAS"}="1H37 JAS"; $resn_exch{"H372 JAS"}="2H37 JAS";
$resn_exch{"H373 JAS"}="3H37 JAS"; $resn_exch{"H221 JAS"}="1H22 JAS"; $resn_exch{"H222 JAS"}="2H22 JAS";
$resn_exch{"H211 JAS"}="1H21 JAS"; $resn_exch{"H212 JAS"}="2H21 JAS"; $resn_exch{"H441 JAS"}="1H44 JAS";
$resn_exch{"H442 JAS"}="2H44 JAS"; $resn_exch{"H443 JAS"}="3H44 JAS"; $resn_exch{"H361 JAS"}="1H36 JAS";
$resn_exch{"H362 JAS"}="2H36 JAS"; $resn_exch{"H363 JAS"}="3H36 JAS"; $resn_exch{"H7C1 JCL"}="1H7C JCL";
$resn_exch{"H7C2 JCL"}="2H7C JCL"; $resn_exch{"H8C1 JCL"}="1H8C JCL"; $resn_exch{"H8C2 JCL"}="2H8C JCL";
$resn_exch{"H9C1 JCL"}="1H9C JCL"; $resn_exch{"H9C2 JCL"}="2H9C JCL"; $resn_exch{"H101 JCL"}="1H10 JCL";
$resn_exch{"H102 JCL"}="2H10 JCL"; $resn_exch{"HC81 JE2"}="1HC8 JE2"; $resn_exch{"HC82 JE2"}="2HC8 JE2";
$resn_exch{"HC83 JE2"}="3HC8 JE2"; $resn_exch{"H131 JE2"}="1H13 JE2"; $resn_exch{"H132 JE2"}="2H13 JE2";
$resn_exch{"H251 JE2"}="1H25 JE2"; $resn_exch{"H252 JE2"}="2H25 JE2"; $resn_exch{"H281 JE2"}="1H28 JE2";
$resn_exch{"H282 JE2"}="2H28 JE2"; $resn_exch{"H283 JE2"}="3H28 JE2"; $resn_exch{"H291 JE2"}="1H29 JE2";
$resn_exch{"H292 JE2"}="2H29 JE2"; $resn_exch{"H293 JE2"}="3H29 JE2"; $resn_exch{"H341 JE2"}="1H34 JE2";
$resn_exch{"H342 JE2"}="2H34 JE2"; $resn_exch{"H411 JE2"}="1H41 JE2"; $resn_exch{"H412 JE2"}="2H41 JE2";
$resn_exch{"H413 JE2"}="3H41 JE2"; $resn_exch{"H381 JEF"}="1H38 JEF"; $resn_exch{"H382 JEF"}="2H38 JEF";
$resn_exch{"H383 JEF"}="3H38 JEF"; $resn_exch{"H371 JEF"}="1H37 JEF"; $resn_exch{"H372 JEF"}="2H37 JEF";
$resn_exch{"H361 JEF"}="1H36 JEF"; $resn_exch{"H362 JEF"}="2H36 JEF"; $resn_exch{"H351 JEF"}="1H35 JEF";
$resn_exch{"H352 JEF"}="2H35 JEF"; $resn_exch{"H341 JEF"}="1H34 JEF"; $resn_exch{"H342 JEF"}="2H34 JEF";
$resn_exch{"H343 JEF"}="3H34 JEF"; $resn_exch{"H321 JEF"}="1H32 JEF"; $resn_exch{"H322 JEF"}="2H32 JEF";
$resn_exch{"H401 JEF"}="1H40 JEF"; $resn_exch{"H402 JEF"}="2H40 JEF"; $resn_exch{"H403 JEF"}="3H40 JEF";
$resn_exch{"H201 JEF"}="1H20 JEF"; $resn_exch{"H202 JEF"}="2H20 JEF"; $resn_exch{"H181 JEF"}="1H18 JEF";
$resn_exch{"H182 JEF"}="2H18 JEF"; $resn_exch{"H183 JEF"}="3H18 JEF"; $resn_exch{"H171 JEF"}="1H17 JEF";
$resn_exch{"H172 JEF"}="2H17 JEF"; $resn_exch{"H11A JEF"}="1H1  JEF"; $resn_exch{" H12 JEF"}="2H1  JEF";
$resn_exch{"H13A JEF"}="3H1  JEF"; $resn_exch{" H31 JEF"}="1H3  JEF"; $resn_exch{" H32 JEF"}="2H3  JEF";
$resn_exch{" H61 JEF"}="1H6  JEF"; $resn_exch{" H62 JEF"}="2H6  JEF"; $resn_exch{" H63 JEF"}="3H6  JEF";
$resn_exch{" H41 JEF"}="1H4  JEF"; $resn_exch{" H42 JEF"}="2H4  JEF"; $resn_exch{" H91 JEF"}="1H9  JEF";
$resn_exch{" H92 JEF"}="2H9  JEF"; $resn_exch{" H93 JEF"}="3H9  JEF"; $resn_exch{" H81 JEF"}="1H8  JEF";
$resn_exch{" H82 JEF"}="2H8  JEF"; $resn_exch{"H121 JEF"}="1H12 JEF"; $resn_exch{"H122 JEF"}="2H12 JEF";
$resn_exch{"H123 JEF"}="3H12 JEF"; $resn_exch{"H101 JEF"}="1H10 JEF"; $resn_exch{"H102 JEF"}="2H10 JEF";
$resn_exch{"H151 JEF"}="1H15 JEF"; $resn_exch{"H152 JEF"}="2H15 JEF"; $resn_exch{"H153 JEF"}="3H15 JEF";
$resn_exch{"H141 JEF"}="1H14 JEF"; $resn_exch{"H142 JEF"}="2H14 JEF"; $resn_exch{"H161 JEF"}="1H16 JEF";
$resn_exch{"H162 JEF"}="2H16 JEF"; $resn_exch{"H311 JEF"}="1H31 JEF"; $resn_exch{"H312 JEF"}="2H31 JEF";
$resn_exch{"H313 JEF"}="3H31 JEF"; $resn_exch{"HN11 JEF"}="1HN1 JEF"; $resn_exch{"HN12 JEF"}="2HN1 JEF";
$resn_exch{" H11 JEN"}="1H1  JEN"; $resn_exch{" H12 JEN"}="2H1  JEN"; $resn_exch{" H13 JEN"}="3H1  JEN";
$resn_exch{"H101 JEN"}="1H10 JEN"; $resn_exch{"H102 JEN"}="2H10 JEN"; $resn_exch{"H111 JEN"}="1H11 JEN";
$resn_exch{"H112 JEN"}="2H11 JEN"; $resn_exch{"H131 JEN"}="1H13 JEN"; $resn_exch{"H132 JEN"}="2H13 JEN";
$resn_exch{"H141 JEN"}="1H14 JEN"; $resn_exch{"H142 JEN"}="2H14 JEN"; $resn_exch{"H211 JEN"}="1H21 JEN";
$resn_exch{"H212 JEN"}="2H21 JEN"; $resn_exch{"H213 JEN"}="3H21 JEN"; $resn_exch{"H121 JH0"}="1H12 JH0";
$resn_exch{"H122 JH0"}="2H12 JH0"; $resn_exch{"H131 JH0"}="1H13 JH0"; $resn_exch{"H132 JH0"}="2H13 JH0";
$resn_exch{"H141 JH0"}="1H14 JH0"; $resn_exch{"H142 JH0"}="2H14 JH0"; $resn_exch{"H151 JH0"}="1H15 JH0";
$resn_exch{"H152 JH0"}="2H15 JH0"; $resn_exch{"H161 JH0"}="1H16 JH0"; $resn_exch{"H162 JH0"}="2H16 JH0";
$resn_exch{"H171 JH0"}="1H17 JH0"; $resn_exch{"H172 JH0"}="2H17 JH0"; $resn_exch{"H181 JH0"}="1H18 JH0";
$resn_exch{"H182 JH0"}="2H18 JH0"; $resn_exch{"H191 JH0"}="1H19 JH0"; $resn_exch{"H192 JH0"}="2H19 JH0";
$resn_exch{"H201 JH0"}="1H20 JH0"; $resn_exch{"H202 JH0"}="2H20 JH0"; $resn_exch{"H211 JH0"}="1H21 JH0";
$resn_exch{"H212 JH0"}="2H21 JH0"; $resn_exch{"H221 JH0"}="1H22 JH0"; $resn_exch{"H222 JH0"}="2H22 JH0";
$resn_exch{"H231 JH0"}="1H23 JH0"; $resn_exch{"H232 JH0"}="2H23 JH0"; $resn_exch{"H241 JH0"}="1H24 JH0";
$resn_exch{"H242 JH0"}="2H24 JH0"; $resn_exch{"H251 JH0"}="1H25 JH0"; $resn_exch{"H252 JH0"}="2H25 JH0";
$resn_exch{"H261 JH0"}="1H26 JH0"; $resn_exch{"H262 JH0"}="2H26 JH0"; $resn_exch{"H263 JH0"}="3H26 JH0";
$resn_exch{" H11 JH0"}="1H1  JH0"; $resn_exch{" H12 JH0"}="2H1  JH0"; $resn_exch{" H21 JH0"}="1H2  JH0";
$resn_exch{" H22 JH0"}="2H2  JH0"; $resn_exch{" H31 JH0"}="1H3  JH0"; $resn_exch{" H32 JH0"}="2H3  JH0";
$resn_exch{"H41A JH0"}="1H4  JH0"; $resn_exch{"H42A JH0"}="2H4  JH0"; $resn_exch{"H371 JH0"}="1H37 JH0";
$resn_exch{"H372 JH0"}="2H37 JH0"; $resn_exch{" H91 JH0"}="1H9  JH0"; $resn_exch{" H92 JH0"}="2H9  JH0";
$resn_exch{" H93 JH0"}="3H9  JH0"; $resn_exch{" H81 JH0"}="1H8  JH0"; $resn_exch{" H82 JH0"}="2H8  JH0";
$resn_exch{"H341 JH0"}="1H34 JH0"; $resn_exch{"H342 JH0"}="2H34 JH0"; $resn_exch{"H331 JH0"}="1H33 JH0";
$resn_exch{"H332 JH0"}="2H33 JH0"; $resn_exch{"H321 JH0"}="1H32 JH0"; $resn_exch{"H322 JH0"}="2H32 JH0";
$resn_exch{"H311 JH0"}="1H31 JH0"; $resn_exch{"H312 JH0"}="2H31 JH0"; $resn_exch{"H281 JH0"}="1H28 JH0";
$resn_exch{"H282 JH0"}="2H28 JH0"; $resn_exch{"H283 JH0"}="3H28 JH0"; $resn_exch{"H291 JH0"}="1H29 JH0";
$resn_exch{"H292 JH0"}="2H29 JH0"; $resn_exch{"H293 JH0"}="3H29 JH0"; $resn_exch{"H301 JH0"}="1H30 JH0";
$resn_exch{"H302 JH0"}="2H30 JH0"; $resn_exch{"H303 JH0"}="3H30 JH0"; $resn_exch{"H131 JIN"}="1H13 JIN";
$resn_exch{"H132 JIN"}="2H13 JIN"; $resn_exch{"H133 JIN"}="3H13 JIN"; $resn_exch{"H341 JIN"}="1H34 JIN";
$resn_exch{"H342 JIN"}="2H34 JIN"; $resn_exch{"H343 JIN"}="3H34 JIN"; $resn_exch{"H241 JJ3"}="1H24 JJ3";
$resn_exch{"H242 JJ3"}="2H24 JJ3"; $resn_exch{"H101 JJ3"}="1H10 JJ3"; $resn_exch{"H102 JJ3"}="2H10 JJ3";
$resn_exch{"H111 JJ3"}="1H11 JJ3"; $resn_exch{"H112 JJ3"}="2H11 JJ3"; $resn_exch{"H121 JJ3"}="1H12 JJ3";
$resn_exch{"H122 JJ3"}="2H12 JJ3"; $resn_exch{"H261 JJ3"}="1H26 JJ3"; $resn_exch{"H262 JJ3"}="2H26 JJ3";
$resn_exch{"H263 JJ3"}="3H26 JJ3"; $resn_exch{" H11 JN3"}="1H1  JN3"; $resn_exch{" H12 JN3"}="2H1  JN3";
$resn_exch{" H21 JN3"}="1H2  JN3"; $resn_exch{" H22 JN3"}="2H2  JN3"; $resn_exch{" H41 JN3"}="1H4  JN3";
$resn_exch{" H42 JN3"}="2H4  JN3"; $resn_exch{" H61 JN3"}="1H6  JN3"; $resn_exch{" H62 JN3"}="2H6  JN3";
$resn_exch{"H111 JN3"}="1H11 JN3"; $resn_exch{"H112 JN3"}="2H11 JN3"; $resn_exch{"H121 JN3"}="1H12 JN3";
$resn_exch{"H122 JN3"}="2H12 JN3"; $resn_exch{"H151 JN3"}="1H15 JN3"; $resn_exch{"H152 JN3"}="2H15 JN3";
$resn_exch{"H161 JN3"}="1H16 JN3"; $resn_exch{"H162 JN3"}="2H16 JN3"; $resn_exch{"H181 JN3"}="1H18 JN3";
$resn_exch{"H182 JN3"}="2H18 JN3"; $resn_exch{"H183 JN3"}="3H18 JN3"; $resn_exch{"H191 JN3"}="1H19 JN3";
$resn_exch{"H192 JN3"}="2H19 JN3"; $resn_exch{"H193 JN3"}="3H19 JN3"; $resn_exch{"H211 JN3"}="1H21 JN3";
$resn_exch{"H212 JN3"}="2H21 JN3"; $resn_exch{"H213 JN3"}="3H21 JN3"; $resn_exch{"H221 JN3"}="1H22 JN3";
$resn_exch{"H222 JN3"}="2H22 JN3"; $resn_exch{"H231 JN3"}="1H23 JN3"; $resn_exch{"H232 JN3"}="2H23 JN3";
$resn_exch{" H21 JNH"}="1H2  JNH"; $resn_exch{" H22 JNH"}="2H2  JNH"; $resn_exch{" H31 JNH"}="1H3  JNH";
$resn_exch{" H32 JNH"}="2H3  JNH"; $resn_exch{" H41 JNH"}="1H4  JNH"; $resn_exch{" H42 JNH"}="2H4  JNH";
$resn_exch{"HN21 JNH"}="1HN2 JNH"; $resn_exch{"HN22 JNH"}="2HN2 JNH"; $resn_exch{" H71 JNH"}="1H7  JNH";
$resn_exch{" H72 JNH"}="2H7  JNH"; $resn_exch{"H201 JNH"}="1H20 JNH"; $resn_exch{"H202 JNH"}="2H20 JNH";
$resn_exch{"HN31 JNH"}="1HN3 JNH"; $resn_exch{"HN32 JNH"}="2HN3 JNH"; $resn_exch{"H231 JNK"}="1H23 JNK";
$resn_exch{"H232 JNK"}="2H23 JNK"; $resn_exch{"H241 JNK"}="1H24 JNK"; $resn_exch{"H242 JNK"}="2H24 JNK";
$resn_exch{"H261 JNK"}="1H26 JNK"; $resn_exch{"H262 JNK"}="2H26 JNK"; $resn_exch{"H271 JNK"}="1H27 JNK";
$resn_exch{"H272 JNK"}="2H27 JNK"; $resn_exch{"H301 JNK"}="1H30 JNK"; $resn_exch{"H302 JNK"}="2H30 JNK";
$resn_exch{"H303 JNK"}="3H30 JNK"; $resn_exch{"H101 JOS"}="1H10 JOS"; $resn_exch{"H102 JOS"}="2H10 JOS";
$resn_exch{"H161 JOS"}="1H16 JOS"; $resn_exch{"H162 JOS"}="2H16 JOS"; $resn_exch{"H163 JOS"}="3H16 JOS";
$resn_exch{"H21A JOS"}="1H2  JOS"; $resn_exch{" H22 JOS"}="2H2  JOS"; $resn_exch{"H181 JOS"}="1H18 JOS";
$resn_exch{"H182 JOS"}="2H18 JOS"; $resn_exch{"H183 JOS"}="3H18 JOS"; $resn_exch{"H191 JOS"}="1H19 JOS";
$resn_exch{"H192 JOS"}="2H19 JOS"; $resn_exch{"H193 JOS"}="3H19 JOS"; $resn_exch{"H201 JOS"}="1H20 JOS";
$resn_exch{"H202 JOS"}="2H20 JOS"; $resn_exch{" H71 JOS"}="1H7  JOS"; $resn_exch{" H72 JOS"}="2H7  JOS";
$resn_exch{"H221 JOS"}="1H22 JOS"; $resn_exch{"H222 JOS"}="2H22 JOS"; $resn_exch{"H223 JOS"}="3H22 JOS";
$resn_exch{"H6A1 JOS"}="1H6A JOS"; $resn_exch{"H6A2 JOS"}="2H6A JOS"; $resn_exch{"H6A3 JOS"}="3H6A JOS";
$resn_exch{"H7A1 JOS"}="1H7A JOS"; $resn_exch{"H7A2 JOS"}="2H7A JOS"; $resn_exch{"H7A3 JOS"}="3H7A JOS";
$resn_exch{"H2B1 JOS"}="1H2B JOS"; $resn_exch{"H2B2 JOS"}="2H2B JOS"; $resn_exch{"H6B1 JOS"}="1H6B JOS";
$resn_exch{"H6B2 JOS"}="2H6B JOS"; $resn_exch{"H6B3 JOS"}="3H6B JOS"; $resn_exch{"H7B1 JOS"}="1H7B JOS";
$resn_exch{"H7B2 JOS"}="2H7B JOS"; $resn_exch{"H7B3 JOS"}="3H7B JOS"; $resn_exch{"H2C1 JOS"}="1H2C JOS";
$resn_exch{"H2C2 JOS"}="2H2C JOS"; $resn_exch{"H4C1 JOS"}="1H4C JOS"; $resn_exch{"H4C2 JOS"}="2H4C JOS";
$resn_exch{"H4C3 JOS"}="3H4C JOS"; $resn_exch{"H5C1 JOS"}="1H5C JOS"; $resn_exch{"H5C2 JOS"}="2H5C JOS";
$resn_exch{"H5C3 JOS"}="3H5C JOS"; $resn_exch{"H8A1 JOS"}="1H8A JOS"; $resn_exch{"H8A2 JOS"}="2H8A JOS";
$resn_exch{"H8A3 JOS"}="3H8A JOS"; $resn_exch{"H231 JP1"}="1H23 JP1"; $resn_exch{"H232 JP1"}="2H23 JP1";
$resn_exch{"H251 JP1"}="1H25 JP1"; $resn_exch{"H252 JP1"}="2H25 JP1"; $resn_exch{"H261 JP1"}="1H26 JP1";
$resn_exch{"H262 JP1"}="2H26 JP1"; $resn_exch{"H221 JP1"}="1H22 JP1"; $resn_exch{"H222 JP1"}="2H22 JP1";
$resn_exch{"H231 JPA"}="1H23 JPA"; $resn_exch{"H232 JPA"}="2H23 JPA"; $resn_exch{"H233 JPA"}="3H23 JPA";
$resn_exch{" H31 JPC"}="1H3  JPC"; $resn_exch{" H32 JPC"}="2H3  JPC"; $resn_exch{" H33 JPC"}="3H3  JPC";
$resn_exch{" H41 JPC"}="1H4  JPC"; $resn_exch{" H42 JPC"}="2H4  JPC"; $resn_exch{" H43 JPC"}="3H4  JPC";
$resn_exch{" H71 JPJ"}="1H7  JPJ"; $resn_exch{" H72 JPJ"}="2H7  JPJ"; $resn_exch{"H181 JPJ"}="1H18 JPJ";
$resn_exch{"H182 JPJ"}="2H18 JPJ"; $resn_exch{" H91 JPN"}="1H9  JPN"; $resn_exch{" H92 JPN"}="2H9  JPN";
$resn_exch{" H81 JPN"}="1H8  JPN"; $resn_exch{" H82 JPN"}="2H8  JPN"; $resn_exch{" H71 JPN"}="1H7  JPN";
$resn_exch{" H72 JPN"}="2H7  JPN"; $resn_exch{" H11 JPR"}="1H1  JPR"; $resn_exch{" H12 JPR"}="2H1  JPR";
$resn_exch{"H13A JPR"}="3H1  JPR"; $resn_exch{" H91 JPR"}="1H9  JPR"; $resn_exch{" H92 JPR"}="2H9  JPR";
$resn_exch{"H101 JPR"}="1H10 JPR"; $resn_exch{"H102 JPR"}="2H10 JPR"; $resn_exch{"H111 JPR"}="1H11 JPR";
$resn_exch{"H112 JPR"}="2H11 JPR"; $resn_exch{"H161 JPR"}="1H16 JPR"; $resn_exch{"H162 JPR"}="2H16 JPR";
$resn_exch{"H171 JPR"}="1H17 JPR"; $resn_exch{"H172 JPR"}="2H17 JPR"; $resn_exch{"H173 JPR"}="3H17 JPR";
$resn_exch{"H181 JPR"}="1H18 JPR"; $resn_exch{"H182 JPR"}="2H18 JPR"; $resn_exch{"H183 JPR"}="3H18 JPR";
$resn_exch{"H211 JS4"}="1H21 JS4"; $resn_exch{"H212 JS4"}="2H21 JS4"; $resn_exch{"H611 JS4"}="1H61 JS4";
$resn_exch{"H612 JS4"}="2H61 JS4"; $resn_exch{"H121 JS4"}="1H12 JS4"; $resn_exch{"H122 JS4"}="2H12 JS4";
$resn_exch{"H221 JS4"}="1H22 JS4"; $resn_exch{"H222 JS4"}="2H22 JS4"; $resn_exch{"H321 JS4"}="1H32 JS4";
$resn_exch{"H322 JS4"}="2H32 JS4"; $resn_exch{"H531 JS4"}="1H53 JS4"; $resn_exch{"H532 JS4"}="2H53 JS4";
$resn_exch{"H631 JS4"}="1H63 JS4"; $resn_exch{"H632 JS4"}="2H63 JS4"; $resn_exch{"H731 JS4"}="1H73 JS4";
$resn_exch{"H732 JS4"}="2H73 JS4"; $resn_exch{"H831 JS4"}="1H83 JS4"; $resn_exch{"H832 JS4"}="2H83 JS4";
$resn_exch{"H151 JS4"}="1H15 JS4"; $resn_exch{"H152 JS4"}="2H15 JS4"; $resn_exch{"H11A JS4"}="1H1  JS4";
$resn_exch{"H12A JS4"}="2H1  JS4"; $resn_exch{"H251 JS4"}="1H25 JS4"; $resn_exch{"H252 JS4"}="2H25 JS4";
$resn_exch{"H241 JS4"}="1H24 JS4"; $resn_exch{"H242 JS4"}="2H24 JS4"; $resn_exch{"H641 JS4"}="1H64 JS4";
$resn_exch{"H642 JS4"}="2H64 JS4"; $resn_exch{"H21A JS4"}="1H2  JS4"; $resn_exch{" H22 JS4"}="2H2  JS4";
$resn_exch{"H211 JS5"}="1H21 JS5"; $resn_exch{"H212 JS5"}="2H21 JS5"; $resn_exch{"H611 JS5"}="1H61 JS5";
$resn_exch{"H612 JS5"}="2H61 JS5"; $resn_exch{"H121 JS5"}="1H12 JS5"; $resn_exch{"H122 JS5"}="2H12 JS5";
$resn_exch{"H221 JS5"}="1H22 JS5"; $resn_exch{"H222 JS5"}="2H22 JS5"; $resn_exch{"H321 JS5"}="1H32 JS5";
$resn_exch{"H322 JS5"}="2H32 JS5"; $resn_exch{"H531 JS5"}="1H53 JS5"; $resn_exch{"H532 JS5"}="2H53 JS5";
$resn_exch{"H631 JS5"}="1H63 JS5"; $resn_exch{"H632 JS5"}="2H63 JS5"; $resn_exch{"H731 JS5"}="1H73 JS5";
$resn_exch{"H732 JS5"}="2H73 JS5"; $resn_exch{"H831 JS5"}="1H83 JS5"; $resn_exch{"H832 JS5"}="2H83 JS5";
$resn_exch{"H251 JS5"}="1H25 JS5"; $resn_exch{"H252 JS5"}="2H25 JS5"; $resn_exch{"H351 JS5"}="1H35 JS5";
$resn_exch{"H352 JS5"}="2H35 JS5"; $resn_exch{"H451 JS5"}="1H45 JS5"; $resn_exch{"H452 JS5"}="2H45 JS5";
$resn_exch{"H551 JS5"}="1H55 JS5"; $resn_exch{"H552 JS5"}="2H55 JS5"; $resn_exch{"H241 JS5"}="1H24 JS5";
$resn_exch{"H242 JS5"}="2H24 JS5"; $resn_exch{"H11A JS5"}="1H1  JS5"; $resn_exch{"H12A JS5"}="2H1  JS5";
$resn_exch{"H641 JS5"}="1H64 JS5"; $resn_exch{"H642 JS5"}="2H64 JS5"; $resn_exch{" H82 JST"}="2H8  JST";
$resn_exch{" H81 JST"}="1H8  JST"; $resn_exch{"H101 JST"}="1H10 JST"; $resn_exch{"H102 JST"}="2H10 JST";
$resn_exch{"H111 JST"}="1H11 JST"; $resn_exch{"H112 JST"}="2H11 JST"; $resn_exch{"H131 JST"}="1H13 JST";
$resn_exch{"H132 JST"}="2H13 JST"; $resn_exch{" H11 JTP"}="1H1  JTP"; $resn_exch{" H12 JTP"}="2H1  JTP";
$resn_exch{" H13 JTP"}="3H1  JTP"; $resn_exch{"H331 JTP"}="1H33 JTP"; $resn_exch{"H332 JTP"}="2H33 JTP";
$resn_exch{"H333 JTP"}="3H33 JTP"; $resn_exch{"H201 JTP"}="1H20 JTP"; $resn_exch{"H202 JTP"}="2H20 JTP";
$resn_exch{"H191 JTP"}="1H19 JTP"; $resn_exch{"H192 JTP"}="2H19 JTP"; $resn_exch{"H271 JTP"}="1H27 JTP";
$resn_exch{"H272 JTP"}="2H27 JTP"; $resn_exch{"H261 JTP"}="1H26 JTP"; $resn_exch{"H262 JTP"}="2H26 JTP";
$resn_exch{"H251 JTP"}="1H25 JTP"; $resn_exch{"H252 JTP"}="2H25 JTP"; $resn_exch{"H241 JTP"}="1H24 JTP";
$resn_exch{"H242 JTP"}="2H24 JTP"; $resn_exch{"H231 JTP"}="1H23 JTP"; $resn_exch{"H232 JTP"}="2H23 JTP";
$resn_exch{" H71 JW5"}="1H7  JW5"; $resn_exch{" H72 JW5"}="2H7  JW5"; $resn_exch{" H1' JW5"}=" H1* JW5";
$resn_exch{" H2' JW5"}=" H2* JW5"; $resn_exch{"HO'2 JW5"}="2HO* JW5"; $resn_exch{" H3' JW5"}=" H3* JW5";
$resn_exch{" H4' JW5"}=" H4* JW5"; $resn_exch{"H5'1 JW5"}="1H5* JW5"; $resn_exch{"H5'2 JW5"}="2H5* JW5";
$resn_exch{"HOP3 JW5"}="3HOP JW5"; $resn_exch{"HOP2 JW5"}="2HOP JW5"; $resn_exch{"HO21  JY"}="1HO2  JY";
$resn_exch{" H11  JY"}="1H1   JY"; $resn_exch{" H12  JY"}="2H1   JY"; $resn_exch{" H21  JY"}="1H2   JY";
$resn_exch{" H22  JY"}="2H2   JY"; $resn_exch{" H31  JY"}="1H3   JY"; $resn_exch{" H41  JY"}="1H4   JY";
$resn_exch{" H42  JY"}="2H4   JY"; $resn_exch{" H61  JY"}="1H6   JY"; $resn_exch{" H71  JY"}="1H7   JY";
$resn_exch{" H91  JY"}="1H9   JY"; $resn_exch{" H92  JY"}="2H9   JY"; $resn_exch{"H111  JY"}="1H11  JY";
$resn_exch{"H112  JY"}="2H11  JY"; $resn_exch{"H121  JY"}="1H12  JY"; $resn_exch{"H122  JY"}="2H12  JY";
$resn_exch{"H141  JY"}="1H14  JY"; $resn_exch{"H151  JY"}="1H15  JY"; $resn_exch{"H152  JY"}="2H15  JY";
$resn_exch{"H161  JY"}="1H16  JY"; $resn_exch{"H162  JY"}="2H16  JY"; $resn_exch{"H171  JY"}="1H17  JY";
$resn_exch{"H181  JY"}="1H18  JY"; $resn_exch{"H182  JY"}="2H18  JY"; $resn_exch{"H183  JY"}="3H18  JY";
$resn_exch{"H191  JY"}="1H19  JY"; $resn_exch{"H192  JY"}="2H19  JY"; $resn_exch{"H201  JY"}="1H20  JY";
$resn_exch{"H211  JY"}="1H21  JY"; $resn_exch{"H212  JY"}="2H21  JY"; $resn_exch{"H213  JY"}="3H21  JY";
$resn_exch{"H221  JY"}="1H22  JY"; $resn_exch{"H222  JY"}="2H22  JY"; $resn_exch{"H231  JY"}="1H23  JY";
$resn_exch{"H241  JY"}="1H24  JY"; $resn_exch{"H261  JY"}="1H26  JY"; $resn_exch{"H281  JY"}="1H28  JY";
$resn_exch{"HO11  JY"}="1HO1  JY"; $resn_exch{"H111 K01"}="1H11 K01"; $resn_exch{"H112 K01"}="2H11 K01";
$resn_exch{"H101 K01"}="1H10 K01"; $resn_exch{"H102 K01"}="2H10 K01"; $resn_exch{" H91 K01"}="1H9  K01";
$resn_exch{" H92 K01"}="2H9  K01"; $resn_exch{" H81 K01"}="1H8  K01"; $resn_exch{" H82 K01"}="2H8  K01";
$resn_exch{" H21 K01"}="1H2  K01"; $resn_exch{" H22 K01"}="2H2  K01"; $resn_exch{" H11 K01"}="1H1  K01";
$resn_exch{" H12 K01"}="2H1  K01"; $resn_exch{" H13 K01"}="3H1  K01"; $resn_exch{"H231 K01"}="1H23 K01";
$resn_exch{"H232 K01"}="2H23 K01"; $resn_exch{"H241 K01"}="1H24 K01"; $resn_exch{"H242 K01"}="2H24 K01";
$resn_exch{"H243 K01"}="3H24 K01"; $resn_exch{"H7C1 K02"}="1H7C K02"; $resn_exch{"H7C2 K02"}="2H7C K02";
$resn_exch{"H111 K02"}="1H11 K02"; $resn_exch{"H112 K02"}="2H11 K02"; $resn_exch{"H101 K02"}="1H10 K02";
$resn_exch{"H102 K02"}="2H10 K02"; $resn_exch{"H103 K02"}="3H10 K02"; $resn_exch{"H8C1 K02"}="1H8C K02";
$resn_exch{"H8C2 K02"}="2H8C K02"; $resn_exch{"H2C1 K02"}="1H2C K02"; $resn_exch{"H2C2 K02"}="2H2C K02";
$resn_exch{"H181 K02"}="1H18 K02"; $resn_exch{"H182 K02"}="2H18 K02"; $resn_exch{"H1C1 K02"}="1H1C K02";
$resn_exch{"H1C2 K02"}="2H1C K02"; $resn_exch{"H1C3 K02"}="3H1C K02"; $resn_exch{"H191 K02"}="1H19 K02";
$resn_exch{"H192 K02"}="2H19 K02"; $resn_exch{"H193 K02"}="3H19 K02"; $resn_exch{"H201 K03"}="1H20 K03";
$resn_exch{"H202 K03"}="2H20 K03"; $resn_exch{"H251 K03"}="1H25 K03"; $resn_exch{"H252 K03"}="2H25 K03";
$resn_exch{"H241 K03"}="1H24 K03"; $resn_exch{"H242 K03"}="2H24 K03"; $resn_exch{"H243 K03"}="3H24 K03";
$resn_exch{"H231 K03"}="1H23 K03"; $resn_exch{"H232 K03"}="2H23 K03"; $resn_exch{"H233 K03"}="3H23 K03";
$resn_exch{"H211 K03"}="1H21 K03"; $resn_exch{"H212 K03"}="2H21 K03"; $resn_exch{"H4C1 K03"}="1H4C K03";
$resn_exch{"H4C2 K03"}="2H4C K03"; $resn_exch{"H2C1 K03"}="1H2C K03"; $resn_exch{"H2C2 K03"}="2H2C K03";
$resn_exch{"H5C1 K03"}="1H5C K03"; $resn_exch{"H5C2 K03"}="2H5C K03"; $resn_exch{"H5C3 K03"}="3H5C K03";
$resn_exch{"H1C1 K03"}="1H1C K03"; $resn_exch{"H1C2 K03"}="2H1C K03"; $resn_exch{"H1C3 K03"}="3H1C K03";
$resn_exch{" H2' K05"}=" H2* K05"; $resn_exch{" H5' K05"}=" H5* K05"; $resn_exch{" H6' K05"}=" H6* K05";
$resn_exch{"H112 K05"}="2H11 K05"; $resn_exch{"H101 K05"}="1H10 K05"; $resn_exch{"H102 K05"}="2H10 K05";
$resn_exch{"H103 K05"}="3H10 K05"; $resn_exch{"HN11 K10"}="1HN1 K10"; $resn_exch{"HN12 K10"}="2HN1 K10";
$resn_exch{" H71 K10"}="1H7  K10"; $resn_exch{" H72 K10"}="2H7  K10"; $resn_exch{" H73 K10"}="3H7  K10";
$resn_exch{" H81 K10"}="1H8  K10"; $resn_exch{" H82 K10"}="2H8  K10"; $resn_exch{" H83 K10"}="3H8  K10";
$resn_exch{" H91 K10"}="1H9  K10"; $resn_exch{" H92 K10"}="2H9  K10"; $resn_exch{" H93 K10"}="3H9  K10";
$resn_exch{"H101 K10"}="1H10 K10"; $resn_exch{"H102 K10"}="2H10 K10"; $resn_exch{"H171 K10"}="1H17 K10";
$resn_exch{"H172 K10"}="2H17 K10"; $resn_exch{"H173 K10"}="3H17 K10"; $resn_exch{"H181 K10"}="1H18 K10";
$resn_exch{"H182 K10"}="2H18 K10"; $resn_exch{"H183 K10"}="3H18 K10"; $resn_exch{"H1N1 K1R"}="1H1N K1R";
$resn_exch{"H1N2 K1R"}="2H1N K1R"; $resn_exch{"HB11 K1R"}="1HB1 K1R"; $resn_exch{"HB12 K1R"}="2HB1 K1R";
$resn_exch{"HG11 K1R"}="1HG1 K1R"; $resn_exch{"HG12 K1R"}="2HG1 K1R"; $resn_exch{"H2N1 K1R"}="1H2N K1R";
$resn_exch{"H2N2 K1R"}="2H2N K1R"; $resn_exch{"HB21 K1R"}="1HB2 K1R"; $resn_exch{"HB22 K1R"}="2HB2 K1R";
$resn_exch{"H101 K21"}="1H10 K21"; $resn_exch{"H102 K21"}="2H10 K21"; $resn_exch{"H121 K21"}="1H12 K21";
$resn_exch{"H122 K21"}="2H12 K21"; $resn_exch{"H131 K21"}="1H13 K21"; $resn_exch{"H132 K21"}="2H13 K21";
$resn_exch{"H221 K21"}="1H22 K21"; $resn_exch{"H222 K21"}="2H22 K21"; $resn_exch{"H241 K21"}="1H24 K21";
$resn_exch{"H242 K21"}="2H24 K21"; $resn_exch{"H301 K21"}="1H30 K21"; $resn_exch{"H302 K21"}="2H30 K21";
$resn_exch{"H311 K21"}="1H31 K21"; $resn_exch{"H312 K21"}="2H31 K21"; $resn_exch{"H331 K21"}="1H33 K21";
$resn_exch{"H332 K21"}="2H33 K21"; $resn_exch{"H341 K21"}="1H34 K21"; $resn_exch{"H342 K21"}="2H34 K21";
$resn_exch{"H431 K21"}="1H43 K21"; $resn_exch{"H432 K21"}="2H43 K21"; $resn_exch{"H591 K21"}="1H59 K21";
$resn_exch{"H592 K21"}="2H59 K21"; $resn_exch{"H593 K21"}="3H59 K21"; $resn_exch{"H161 K25"}="1H16 K25";
$resn_exch{"H162 K25"}="2H16 K25"; $resn_exch{"H163 K25"}="3H16 K25"; $resn_exch{"H171 K25"}="1H17 K25";
$resn_exch{"H172 K25"}="2H17 K25"; $resn_exch{"H173 K25"}="3H17 K25"; $resn_exch{" H71 K2C"}="1H7  K2C";
$resn_exch{" H72 K2C"}="2H7  K2C"; $resn_exch{"H111 K37"}="1H11 K37"; $resn_exch{"H112 K37"}="2H11 K37";
$resn_exch{"H113 K37"}="3H11 K37"; $resn_exch{"H111 K44"}="1H11 K44"; $resn_exch{"H112 K44"}="2H11 K44";
$resn_exch{"H113 K44"}="3H11 K44"; $resn_exch{"H121 K44"}="1H12 K44"; $resn_exch{"H122 K44"}="2H12 K44";
$resn_exch{"H141 K44"}="1H14 K44"; $resn_exch{"H142 K44"}="2H14 K44"; $resn_exch{"H151 K51"}="1H15 K51";
$resn_exch{"H152 K51"}="2H15 K51"; $resn_exch{"H171 K51"}="1H17 K51"; $resn_exch{"H172 K51"}="2H17 K51";
$resn_exch{"H173 K51"}="3H17 K51"; $resn_exch{"H181 K51"}="1H18 K51"; $resn_exch{"H182 K51"}="2H18 K51";
$resn_exch{"H183 K51"}="3H18 K51"; $resn_exch{" H61 KAB"}="1H6  KAB"; $resn_exch{" H62 KAB"}="2H6  KAB";
$resn_exch{"H361 KAB"}="1H36 KAB"; $resn_exch{"H362 KAB"}="2H36 KAB"; $resn_exch{"H363 KAB"}="3H36 KAB";
$resn_exch{" H41 KAB"}="1H4  KAB"; $resn_exch{" H42 KAB"}="2H4  KAB"; $resn_exch{"HN11 KAB"}="1HN1 KAB";
$resn_exch{"HN12 KAB"}="2HN1 KAB"; $resn_exch{" H21 KAB"}="1H2  KAB"; $resn_exch{"H22A KAB"}="2H2  KAB";
$resn_exch{"H371 KAB"}="1H37 KAB"; $resn_exch{"H372 KAB"}="2H37 KAB"; $resn_exch{"H373 KAB"}="3H37 KAB";
$resn_exch{"H471 KAB"}="1H47 KAB"; $resn_exch{"H472 KAB"}="2H47 KAB"; $resn_exch{"H473 KAB"}="3H47 KAB";
$resn_exch{"H111 KAB"}="11H1 KAB"; $resn_exch{"H211 KAB"}="1H21 KAB"; $resn_exch{"H212 KAB"}="2H21 KAB";
$resn_exch{"H461 KAB"}="1H46 KAB"; $resn_exch{"H462 KAB"}="2H46 KAB"; $resn_exch{"H463 KAB"}="3H46 KAB";
$resn_exch{"H381 KAB"}="1H38 KAB"; $resn_exch{"H382 KAB"}="2H38 KAB"; $resn_exch{"H383 KAB"}="3H38 KAB";
$resn_exch{"H251 KAB"}="1H25 KAB"; $resn_exch{"H252 KAB"}="2H25 KAB"; $resn_exch{"H451 KAB"}="1H45 KAB";
$resn_exch{"H452 KAB"}="2H45 KAB"; $resn_exch{"H453 KAB"}="3H45 KAB"; $resn_exch{"H391 KAB"}="1H39 KAB";
$resn_exch{"H392 KAB"}="2H39 KAB"; $resn_exch{"H393 KAB"}="3H39 KAB"; $resn_exch{"H281 KAB"}="1H28 KAB";
$resn_exch{"H282 KAB"}="2H28 KAB"; $resn_exch{"H291 KAB"}="1H29 KAB"; $resn_exch{"H292 KAB"}="2H29 KAB";
$resn_exch{"H401 KAB"}="1H40 KAB"; $resn_exch{"H402 KAB"}="2H40 KAB"; $resn_exch{"H403 KAB"}="3H40 KAB";
$resn_exch{"H441 KAB"}="1H44 KAB"; $resn_exch{"H442 KAB"}="2H44 KAB"; $resn_exch{"H443 KAB"}="3H44 KAB";
$resn_exch{"H411 KAB"}="1H41 KAB"; $resn_exch{"H412 KAB"}="2H41 KAB"; $resn_exch{"H413 KAB"}="3H41 KAB";
$resn_exch{"H431 KAB"}="1H43 KAB"; $resn_exch{"H432 KAB"}="2H43 KAB"; $resn_exch{"H433 KAB"}="3H43 KAB";
$resn_exch{"H421 KAB"}="1H42 KAB"; $resn_exch{"H422 KAB"}="2H42 KAB"; $resn_exch{" H5' KAG"}="1H5* KAG";
$resn_exch{"H5'' KAG"}="2H5* KAG"; $resn_exch{" H4' KAG"}=" H4* KAG"; $resn_exch{" H3' KAG"}=" H3* KAG";
$resn_exch{"H2'' KAG"}="2H2* KAG"; $resn_exch{" H2' KAG"}="1H2* KAG"; $resn_exch{" H1' KAG"}=" H1* KAG";
$resn_exch{"HN51 KAH"}="1HN5 KAH"; $resn_exch{"HN52 KAH"}="2HN5 KAH"; $resn_exch{" HC1 KAH"}="1HC  KAH";
$resn_exch{" HC2 KAH"}="2HC  KAH"; $resn_exch{" HC3 KAH"}="3HC  KAH"; $resn_exch{"HC31 KAH"}="1HC3 KAH";
$resn_exch{"HC32 KAH"}="2HC3 KAH"; $resn_exch{"HC21 KAH"}="1HC2 KAH"; $resn_exch{"HC22 KAH"}="2HC2 KAH";
$resn_exch{"HA21 KAM"}="1HA2 KAM"; $resn_exch{"HA22 KAM"}="2HA2 KAM"; $resn_exch{"HA23 KAM"}="3HA2 KAM";
$resn_exch{"H4A1 KAM"}="1H4A KAM"; $resn_exch{"H4A2 KAM"}="2H4A KAM"; $resn_exch{"HA51 KAM"}="1HA5 KAM";
$resn_exch{"HA52 KAM"}="2HA5 KAM"; $resn_exch{"HOP2 KAM"}="2HOP KAM"; $resn_exch{"HOP3 KAM"}="3HOP KAM";
$resn_exch{"HNC9 KAM"}="9HNC KAM"; $resn_exch{"HC91 KAM"}="1HC9 KAM"; $resn_exch{"HC92 KAM"}="2HC9 KAM";
$resn_exch{"HC93 KAM"}="3HC9 KAM"; $resn_exch{"HC61 KAM"}="1HC6 KAM"; $resn_exch{"HC62 KAM"}="2HC6 KAM";
$resn_exch{"HC51 KAM"}="1HC5 KAM"; $resn_exch{"HC52 KAM"}="2HC5 KAM"; $resn_exch{"HC41 KAM"}="1HC4 KAM";
$resn_exch{"HC42 KAM"}="2HC4 KAM"; $resn_exch{"HC31 KAM"}="1HC3 KAM"; $resn_exch{"HC32 KAM"}="2HC3 KAM";
$resn_exch{"HC21 KAM"}="1HC2 KAM"; $resn_exch{"HC22 KAM"}="2HC2 KAM"; $resn_exch{"HOC1 KAM"}="1HOC KAM";
$resn_exch{" H61 KAN"}="1H6  KAN"; $resn_exch{" H62 KAN"}="2H6  KAN"; $resn_exch{"H121 KAN"}="1H12 KAN";
$resn_exch{"H122 KAN"}="2H12 KAN"; $resn_exch{"H181 KAN"}="1H18 KAN"; $resn_exch{"H182 KAN"}="2H18 KAN";
$resn_exch{"HN11 KAN"}="1HN1 KAN"; $resn_exch{"HN12 KAN"}="2HN1 KAN"; $resn_exch{"HN21 KAN"}="1HN2 KAN";
$resn_exch{"HN22 KAN"}="2HN2 KAN"; $resn_exch{"HN31 KAN"}="1HN3 KAN"; $resn_exch{"HN32 KAN"}="2HN3 KAN";
$resn_exch{"HN41 KAN"}="1HN4 KAN"; $resn_exch{"HN42 KAN"}="2HN4 KAN"; $resn_exch{" HA2 KAP"}="2HA  KAP";
$resn_exch{" HA1 KAP"}="1HA  KAP"; $resn_exch{" HB2 KAP"}="2HB  KAP"; $resn_exch{" HB1 KAP"}="1HB  KAP";
$resn_exch{" HG2 KAP"}="2HG  KAP"; $resn_exch{" HG1 KAP"}="1HG  KAP"; $resn_exch{" HD2 KAP"}="2HD  KAP";
$resn_exch{" HD1 KAP"}="1HD  KAP"; $resn_exch{" HE2 KAP"}="2HE  KAP"; $resn_exch{" HE1 KAP"}="1HE  KAP";
$resn_exch{" HS3 KAP"}="3HS  KAP"; $resn_exch{" HS2 KAP"}="2HS  KAP"; $resn_exch{" HS1 KAP"}="1HS  KAP";
$resn_exch{" H1  KAP"}="1H   KAP"; $resn_exch{" H2  KAP"}="2H   KAP"; $resn_exch{" H91 KAR"}="1H9  KAR";
$resn_exch{" H92 KAR"}="2H9  KAR"; $resn_exch{" H93 KAR"}="3H9  KAR"; $resn_exch{"H201 KAR"}="1H20 KAR";
$resn_exch{"H202 KAR"}="2H20 KAR"; $resn_exch{"H203 KAR"}="3H20 KAR"; $resn_exch{"H251 KAR"}="1H25 KAR";
$resn_exch{"H252 KAR"}="2H25 KAR"; $resn_exch{"H253 KAR"}="3H25 KAR"; $resn_exch{"H231 KAR"}="1H23 KAR";
$resn_exch{"H232 KAR"}="2H23 KAR"; $resn_exch{"H341 KAR"}="1H34 KAR"; $resn_exch{"H342 KAR"}="2H34 KAR";
$resn_exch{"H311 KAR"}="1H31 KAR"; $resn_exch{"H312 KAR"}="2H31 KAR"; $resn_exch{"H351 KAR"}="1H35 KAR";
$resn_exch{"H352 KAR"}="2H35 KAR"; $resn_exch{"H361 KAR"}="1H36 KAR"; $resn_exch{"H362 KAR"}="2H36 KAR";
$resn_exch{"H363 KAR"}="3H36 KAR"; $resn_exch{"H281 KAR"}="1H28 KAR"; $resn_exch{"H282 KAR"}="2H28 KAR";
$resn_exch{"H191 KAR"}="1H19 KAR"; $resn_exch{"H192 KAR"}="2H19 KAR"; $resn_exch{"H301 KAR"}="1H30 KAR";
$resn_exch{"H302 KAR"}="2H30 KAR"; $resn_exch{"H131 KAR"}="1H13 KAR"; $resn_exch{"H132 KAR"}="2H13 KAR";
$resn_exch{"H911 KAR"}="1H91 KAR"; $resn_exch{"H912 KAR"}="2H91 KAR"; $resn_exch{"H921 KAR"}="1H92 KAR";
$resn_exch{"H922 KAR"}="2H92 KAR"; $resn_exch{"H941 KAR"}="1H94 KAR"; $resn_exch{"H942 KAR"}="2H94 KAR";
$resn_exch{"H451 KAR"}="1H45 KAR"; $resn_exch{"H452 KAR"}="2H45 KAR"; $resn_exch{"H453 KAR"}="3H45 KAR";
$resn_exch{"H951 KAR"}="1H95 KAR"; $resn_exch{"H952 KAR"}="2H95 KAR"; $resn_exch{"H901 KAR"}="1H90 KAR";
$resn_exch{"H902 KAR"}="2H90 KAR"; $resn_exch{"H891 KAR"}="1H89 KAR"; $resn_exch{"H892 KAR"}="2H89 KAR";
$resn_exch{"HAE1 KBG"}="1HAE KBG"; $resn_exch{"HAE2 KBG"}="2HAE KBG"; $resn_exch{" H21 KCP"}="1H2  KCP";
$resn_exch{" H22 KCP"}="2H2  KCP"; $resn_exch{"H3'1 KCP"}="1H3' KCP"; $resn_exch{"H3'2 KCP"}="2H3' KCP";
$resn_exch{"H3'3 KCP"}="3H3' KCP"; $resn_exch{"H131 KCP"}="1H13 KCP"; $resn_exch{"H132 KCP"}="2H13 KCP";
$resn_exch{" HN1 KCQ"}="1HN  KCQ"; $resn_exch{" HN2 KCQ"}="2HN  KCQ"; $resn_exch{"HMK1 KCQ"}="1HMK KCQ";
$resn_exch{"HMK2 KCQ"}="2HMK KCQ"; $resn_exch{"HMK3 KCQ"}="3HMK KCQ"; $resn_exch{" HB1 KCQ"}="1HB  KCQ";
$resn_exch{" HB2 KCQ"}="2HB  KCQ"; $resn_exch{"HAE1 KCQ"}="1HAE KCQ"; $resn_exch{"HAE2 KCQ"}="2HAE KCQ";
$resn_exch{"HD11 KCQ"}="1HD1 KCQ"; $resn_exch{"HD12 KCQ"}="2HD1 KCQ"; $resn_exch{" H   KCX"}="1HN  KCX";
$resn_exch{" HN2 KCX"}="2HN  KCX"; $resn_exch{" HB2 KCX"}="1HB  KCX"; $resn_exch{" HB3 KCX"}="2HB  KCX";
$resn_exch{" HG2 KCX"}="1HG  KCX"; $resn_exch{" HG3 KCX"}="2HG  KCX"; $resn_exch{" HD2 KCX"}="1HD  KCX";
$resn_exch{" HD3 KCX"}="2HD  KCX"; $resn_exch{" HE2 KCX"}="1HE  KCX"; $resn_exch{" HE3 KCX"}="2HE  KCX";
$resn_exch{" HZ  KCX"}="1HZ  KCX"; $resn_exch{" HQ2 KCX"}=" HX2 KCX"; $resn_exch{" H31 KDA"}="1H3  KDA";
$resn_exch{" H32 KDA"}="2H3  KDA"; $resn_exch{" H81 KDA"}="1H8  KDA"; $resn_exch{" H82 KDA"}="2H8  KDA";
$resn_exch{" H91 KDA"}="1H9  KDA"; $resn_exch{" H92 KDA"}="2H9  KDA"; $resn_exch{"H111 KDA"}="1H11 KDA";
$resn_exch{"H112 KDA"}="2H11 KDA"; $resn_exch{"H1C1 KDF"}="1H1C KDF"; $resn_exch{"H1C2 KDF"}="2H1C KDF";
$resn_exch{"H4C1 KDF"}="1H4C KDF"; $resn_exch{"H4C2 KDF"}="2H4C KDF"; $resn_exch{"H1C1 KDG"}="1H1C KDG";
$resn_exch{"H1C2 KDG"}="2H1C KDG"; $resn_exch{"H4C1 KDG"}="1H4C KDG"; $resn_exch{"H4C2 KDG"}="2H4C KDG";
$resn_exch{" H31 KDO"}="1H3  KDO"; $resn_exch{" H32 KDO"}="2H3  KDO"; $resn_exch{" H81 KDO"}="1H8  KDO";
$resn_exch{" H82 KDO"}="2H8  KDO"; $resn_exch{"HAI1 KDP"}="1HAI KDP"; $resn_exch{"HAI2 KDP"}="2HAI KDP";
$resn_exch{"HAJ1 KDP"}="1HAJ KDP"; $resn_exch{"HAJ2 KDP"}="2HAJ KDP"; $resn_exch{"H5A1 KE4"}="1H5A KE4";
$resn_exch{"H5A2 KE4"}="2H5A KE4"; $resn_exch{"H2A1 KE4"}="1H2A KE4"; $resn_exch{"H2A2 KE4"}="2H2A KE4";
$resn_exch{"H2A3 KE4"}="3H2A KE4"; $resn_exch{"H4A1 KE4"}="1H4A KE4"; $resn_exch{"H4A2 KE4"}="2H4A KE4";
$resn_exch{" H1  KE4"}="1H   KE4"; $resn_exch{" H2  KE4"}="2H   KE4"; $resn_exch{"HN51 KE4"}="1HN5 KE4";
$resn_exch{"HN52 KE4"}="2HN5 KE4"; $resn_exch{" HB1 KE4"}="1HB  KE4"; $resn_exch{" HB2 KE4"}="2HB  KE4";
$resn_exch{" HG1 KE4"}="1HG  KE4"; $resn_exch{" HG2 KE4"}="2HG  KE4"; $resn_exch{"H183 KEF"}="3H18 KEF";
$resn_exch{"H182 KEF"}="2H18 KEF"; $resn_exch{"H181 KEF"}="1H18 KEF"; $resn_exch{"H3'2 KEF"}="2H3' KEF";
$resn_exch{"H3'1 KEF"}="1H3' KEF"; $resn_exch{"HN31 KEF"}="1HN3 KEF"; $resn_exch{"HN32 KEF"}="2HN3 KEF";
$resn_exch{"H2A1 KET"}="1H2A KET"; $resn_exch{"H2A2 KET"}="2H2A KET"; $resn_exch{"H2A3 KET"}="3H2A KET";
$resn_exch{"H5A1 KET"}="1H5A KET"; $resn_exch{"H5A2 KET"}="2H5A KET"; $resn_exch{"HOP2 KET"}="2HOP KET";
$resn_exch{"HOP3 KET"}="3HOP KET"; $resn_exch{" HB1 KET"}="1HB  KET"; $resn_exch{" HB2 KET"}="2HB  KET";
$resn_exch{"H201 KEU"}="1H20 KEU"; $resn_exch{"H202 KEU"}="2H20 KEU"; $resn_exch{"H191 KEU"}="1H19 KEU";
$resn_exch{"H192 KEU"}="2H19 KEU"; $resn_exch{" H11 KEU"}="1H1  KEU"; $resn_exch{"H12A KEU"}="2H1  KEU";
$resn_exch{" H21 KEU"}="1H2  KEU"; $resn_exch{" H22 KEU"}="2H2  KEU"; $resn_exch{" H31 KEU"}="1H3  KEU";
$resn_exch{" H32 KEU"}="2H3  KEU"; $resn_exch{"HN21 KEU"}="1HN2 KEU"; $resn_exch{"HN22 KEU"}="2HN2 KEU";
$resn_exch{"HN61 KEU"}="1HN6 KEU"; $resn_exch{"HN62 KEU"}="2HN6 KEU"; $resn_exch{" H   KGC"}="1HN  KGC";
$resn_exch{" H2  KGC"}="2HN  KGC"; $resn_exch{" HB2 KGC"}="1HB  KGC"; $resn_exch{" HB3 KGC"}="2HB  KGC";
$resn_exch{" HG2 KGC"}="1HG  KGC"; $resn_exch{" HG3 KGC"}="2HG  KGC"; $resn_exch{" HD2 KGC"}="1HD  KGC";
$resn_exch{" HD3 KGC"}="2HD  KGC"; $resn_exch{" HE2 KGC"}="1HE  KGC"; $resn_exch{" HE3 KGC"}="2HE  KGC";
$resn_exch{" HZ  KGC"}=" HNZ KGC"; $resn_exch{"H271 KGC"}="1H27 KGC"; $resn_exch{"H272 KGC"}="2H27 KGC";
$resn_exch{"H121 KGC"}="1H12 KGC"; $resn_exch{"H122 KGC"}="2H12 KGC"; $resn_exch{" H81 KGQ"}="1H8  KGQ";
$resn_exch{" H82 KGQ"}="2H8  KGQ"; $resn_exch{"H151 KGQ"}="1H15 KGQ"; $resn_exch{"H152 KGQ"}="2H15 KGQ";
$resn_exch{"H181 KGQ"}="1H18 KGQ"; $resn_exch{"H182 KGQ"}="2H18 KGQ"; $resn_exch{"H191 KGQ"}="1H19 KGQ";
$resn_exch{"H192 KGQ"}="2H19 KGQ"; $resn_exch{"H193 KGQ"}="3H19 KGQ"; $resn_exch{"H161 KGQ"}="1H16 KGQ";
$resn_exch{"H162 KGQ"}="2H16 KGQ"; $resn_exch{"H171 KGQ"}="1H17 KGQ"; $resn_exch{"H172 KGQ"}="2H17 KGQ";
$resn_exch{"H173 KGQ"}="3H17 KGQ"; $resn_exch{"H201 KGQ"}="1H20 KGQ"; $resn_exch{"H202 KGQ"}="2H20 KGQ";
$resn_exch{"H211 KGQ"}="1H21 KGQ"; $resn_exch{"H212 KGQ"}="2H21 KGQ"; $resn_exch{"H301 KGQ"}="1H30 KGQ";
$resn_exch{"H302 KGQ"}="2H30 KGQ"; $resn_exch{"H303 KGQ"}="3H30 KGQ"; $resn_exch{"H291 KGQ"}="1H29 KGQ";
$resn_exch{"H292 KGQ"}="2H29 KGQ"; $resn_exch{"H293 KGQ"}="3H29 KGQ"; $resn_exch{"H311 KGQ"}="1H31 KGQ";
$resn_exch{"H312 KGQ"}="2H31 KGQ"; $resn_exch{"H313 KGQ"}="3H31 KGQ"; $resn_exch{" H31 KGQ"}="1H3  KGQ";
$resn_exch{"H32A KGQ"}="2H3  KGQ"; $resn_exch{" H21 KGQ"}="1H2  KGQ"; $resn_exch{"H22A KGQ"}="2H2  KGQ";
$resn_exch{"H11A KGQ"}="1H1  KGQ"; $resn_exch{"H12A KGQ"}="2H1  KGQ"; $resn_exch{" H21 KGR"}="1H2  KGR";
$resn_exch{"HO21 KGR"}="1HO2 KGR"; $resn_exch{" H31 KGR"}="1H3  KGR"; $resn_exch{"HO31 KGR"}="1HO3 KGR";
$resn_exch{" H41 KGR"}="1H4  KGR"; $resn_exch{" H42 KGR"}="2H4  KGR"; $resn_exch{" H81 KGY"}="1H8  KGY";
$resn_exch{" H82 KGY"}="2H8  KGY"; $resn_exch{" H83 KGY"}="3H8  KGY"; $resn_exch{"H101 KGY"}="1H10 KGY";
$resn_exch{"H102 KGY"}="2H10 KGY"; $resn_exch{"H103 KGY"}="3H10 KGY"; $resn_exch{"H121 KGY"}="1H12 KGY";
$resn_exch{"H122 KGY"}="2H12 KGY"; $resn_exch{"H131 KGY"}="1H13 KGY"; $resn_exch{"H132 KGY"}="2H13 KGY";
$resn_exch{"H151 KGY"}="1H15 KGY"; $resn_exch{"H152 KGY"}="2H15 KGY"; $resn_exch{"H161 KGY"}="1H16 KGY";
$resn_exch{"H162 KGY"}="2H16 KGY"; $resn_exch{"H181 KGY"}="1H18 KGY"; $resn_exch{"H182 KGY"}="2H18 KGY";
$resn_exch{"H311 KGY"}="1H31 KGY"; $resn_exch{"H312 KGY"}="2H31 KGY"; $resn_exch{"H341 KGY"}="1H34 KGY";
$resn_exch{"H342 KGY"}="2H34 KGY"; $resn_exch{"H343 KGY"}="3H34 KGY"; $resn_exch{"HC21 KH1"}="1HC2 KH1";
$resn_exch{"HC22 KH1"}="2HC2 KH1"; $resn_exch{"HC41 KH1"}="1HC4 KH1"; $resn_exch{"HC42 KH1"}="2HC4 KH1";
$resn_exch{"HC91 KH1"}="1HC9 KH1"; $resn_exch{"HC92 KH1"}="2HC9 KH1"; $resn_exch{"H111 KH1"}="1H11 KH1";
$resn_exch{"H112 KH1"}="2H11 KH1"; $resn_exch{"H121 KH1"}="1H12 KH1"; $resn_exch{"H122 KH1"}="2H12 KH1";
$resn_exch{"H151 KH1"}="1H15 KH1"; $resn_exch{"H152 KH1"}="2H15 KH1"; $resn_exch{"H161 KH1"}="1H16 KH1";
$resn_exch{"H162 KH1"}="2H16 KH1"; $resn_exch{"H181 KH1"}="1H18 KH1"; $resn_exch{"H182 KH1"}="2H18 KH1";
$resn_exch{"H183 KH1"}="3H18 KH1"; $resn_exch{"H191 KH1"}="1H19 KH1"; $resn_exch{"H192 KH1"}="2H19 KH1";
$resn_exch{"H221 KH1"}="1H22 KH1"; $resn_exch{"H222 KH1"}="2H22 KH1"; $resn_exch{"H231 KH1"}="1H23 KH1";
$resn_exch{"H232 KH1"}="2H23 KH1"; $resn_exch{"H261 KH1"}="1H26 KH1"; $resn_exch{"H262 KH1"}="2H26 KH1";
$resn_exch{"H271 KH1"}="1H27 KH1"; $resn_exch{"H272 KH1"}="2H27 KH1"; $resn_exch{"H211 KH1"}="1H21 KH1";
$resn_exch{"H212 KH1"}="2H21 KH1"; $resn_exch{"H213 KH1"}="3H21 KH1"; $resn_exch{"H241 KH1"}="1H24 KH1";
$resn_exch{"H242 KH1"}="2H24 KH1"; $resn_exch{"H281 KH1"}="1H28 KH1"; $resn_exch{"H282 KH1"}="2H28 KH1";
$resn_exch{"H283 KH1"}="3H28 KH1"; $resn_exch{"H291 KH1"}="1H29 KH1"; $resn_exch{"H292 KH1"}="2H29 KH1";
$resn_exch{"H293 KH1"}="3H29 KH1"; $resn_exch{"H9C1 KHA"}="1H9C KHA"; $resn_exch{"H9C2 KHA"}="2H9C KHA";
$resn_exch{"H101 KHA"}="1H10 KHA"; $resn_exch{"H102 KHA"}="2H10 KHA"; $resn_exch{"H7C1 KHA"}="1H7C KHA";
$resn_exch{"H7C2 KHA"}="2H7C KHA"; $resn_exch{"H111 KHA"}="1H11 KHA"; $resn_exch{"H112 KHA"}="2H11 KHA";
$resn_exch{"H113 KHA"}="3H11 KHA"; $resn_exch{"H161 KHA"}="1H16 KHA"; $resn_exch{"H162 KHA"}="2H16 KHA";
$resn_exch{"H151 KHA"}="1H15 KHA"; $resn_exch{"H152 KHA"}="2H15 KHA"; $resn_exch{"H131 KHA"}="1H13 KHA";
$resn_exch{"H132 KHA"}="2H13 KHA"; $resn_exch{"H141 KHA"}="1H14 KHA"; $resn_exch{"H142 KHA"}="2H14 KHA";
$resn_exch{"H121 KHA"}="1H12 KHA"; $resn_exch{"H122 KHA"}="2H12 KHA"; $resn_exch{" H1' KHO"}=" H1* KHO";
$resn_exch{" H2' KHO"}=" H2* KHO"; $resn_exch{" H3' KHO"}=" H3* KHO"; $resn_exch{" H4' KHO"}=" H4* KHO";
$resn_exch{"H5'1 KHO"}="1H5* KHO"; $resn_exch{"H5'2 KHO"}="2H5* KHO"; $resn_exch{" H5' KHO"}=" H5* KHO";
$resn_exch{" H51 KHO"}="1H5  KHO"; $resn_exch{" H52 KHO"}="2H5  KHO"; $resn_exch{" H5B KHP"}=" H5* KHP";
$resn_exch{"H5'1 KHP"}="1H5* KHP"; $resn_exch{"H5'2 KHP"}="2H5* KHP"; $resn_exch{" H4' KHP"}=" H4* KHP";
$resn_exch{" H1' KHP"}=" H1* KHP"; $resn_exch{" H2B KHP"}=" H2* KHP"; $resn_exch{" H3B KHP"}=" H3* KHP";
$resn_exch{"H121 KI2"}="1H12 KI2"; $resn_exch{"H122 KI2"}="2H12 KI2"; $resn_exch{"H191 KI2"}="1H19 KI2";
$resn_exch{"H192 KI2"}="2H19 KI2"; $resn_exch{" H61 KIF"}="1H6  KIF"; $resn_exch{" H62 KIF"}="2H6  KIF";
$resn_exch{" H81 KIQ"}="1H8  KIQ"; $resn_exch{" H82 KIQ"}="2H8  KIQ"; $resn_exch{"H211 KIQ"}="1H21 KIQ";
$resn_exch{"H212 KIQ"}="2H21 KIQ"; $resn_exch{" HN1 KIQ"}="1HN  KIQ"; $resn_exch{" HN2 KIQ"}="2HN  KIQ";
$resn_exch{" H31 KIQ"}="1H3  KIQ"; $resn_exch{" H32 KIQ"}="2H3  KIQ"; $resn_exch{"H121 KIQ"}="1H12 KIQ";
$resn_exch{"H122 KIQ"}="2H12 KIQ"; $resn_exch{" H91 KIQ"}="1H9  KIQ"; $resn_exch{" H92 KIQ"}="2H9  KIQ";
$resn_exch{" H51 KIR"}="1H5  KIR"; $resn_exch{" H61 KIR"}="1H6  KIR"; $resn_exch{"H201 KIR"}="1H20 KIR";
$resn_exch{"H241 KIR"}="1H24 KIR"; $resn_exch{"H251 KIR"}="1H25 KIR"; $resn_exch{"H252 KIR"}="2H25 KIR";
$resn_exch{" H41 KIR"}="1H4  KIR"; $resn_exch{" H91 KIR"}="1H9  KIR"; $resn_exch{"H101 KIR"}="1H10 KIR";
$resn_exch{"H111 KIR"}="1H11 KIR"; $resn_exch{"H121 KIR"}="1H12 KIR"; $resn_exch{"H131 KIR"}="1H13 KIR";
$resn_exch{"H141 KIR"}="1H14 KIR"; $resn_exch{"H151 KIR"}="1H15 KIR"; $resn_exch{"H161 KIR"}="1H16 KIR";
$resn_exch{"H171 KIR"}="1H17 KIR"; $resn_exch{"H191 KIR"}="1H19 KIR"; $resn_exch{"H221 KIR"}="1H22 KIR";
$resn_exch{"H231 KIR"}="1H23 KIR"; $resn_exch{"HO51 KIR"}="1HO5 KIR"; $resn_exch{" H11 KIR"}="1H1  KIR";
$resn_exch{"H411 KIR"}="1H41 KIR"; $resn_exch{"H412 KIR"}="2H41 KIR"; $resn_exch{"H413 KIR"}="3H41 KIR";
$resn_exch{"HO61 KIR"}="1HO6 KIR"; $resn_exch{"H421 KIR"}="1H42 KIR"; $resn_exch{"H422 KIR"}="2H42 KIR";
$resn_exch{"H423 KIR"}="3H42 KIR"; $resn_exch{"H431 KIR"}="1H43 KIR"; $resn_exch{"H432 KIR"}="2H43 KIR";
$resn_exch{"H433 KIR"}="3H43 KIR"; $resn_exch{"H441 KIR"}="1H44 KIR"; $resn_exch{"H442 KIR"}="2H44 KIR";
$resn_exch{"H443 KIR"}="3H44 KIR"; $resn_exch{"H261 KIR"}="1H26 KIR"; $resn_exch{"H451 KIR"}="1H45 KIR";
$resn_exch{"H452 KIR"}="2H45 KIR"; $resn_exch{"H461 KIR"}="1H46 KIR"; $resn_exch{"H462 KIR"}="2H46 KIR";
$resn_exch{"H463 KIR"}="3H46 KIR"; $resn_exch{"H281 KIR"}="1H28 KIR"; $resn_exch{"H291 KIR"}="1H29 KIR";
$resn_exch{"H301 KIR"}="1H30 KIR"; $resn_exch{"HO01 KIR"}="1HO0 KIR"; $resn_exch{"H311 KIR"}="1H31 KIR";
$resn_exch{"HO11 KIR"}="1HO1 KIR"; $resn_exch{"H471 KIR"}="1H47 KIR"; $resn_exch{"H472 KIR"}="2H47 KIR";
$resn_exch{"H473 KIR"}="3H47 KIR"; $resn_exch{"H481 KIR"}="1H48 KIR"; $resn_exch{"H482 KIR"}="2H48 KIR";
$resn_exch{"H483 KIR"}="3H48 KIR"; $resn_exch{"H331 KIR"}="1H33 KIR"; $resn_exch{"H351 KIR"}="1H35 KIR";
$resn_exch{"H361 KIR"}="1H36 KIR"; $resn_exch{"H371 KIR"}="1H37 KIR"; $resn_exch{"H381 KIR"}="1H38 KIR";
$resn_exch{"H391 KIR"}="1H39 KIR"; $resn_exch{"H392 KIR"}="2H39 KIR"; $resn_exch{"H393 KIR"}="3H39 KIR";
$resn_exch{"HO11 KIV"}="1HO1 KIV"; $resn_exch{"HC31 KIV"}="1HC3 KIV"; $resn_exch{"HC41 KIV"}="1HC4 KIV";
$resn_exch{"HC42 KIV"}="2HC4 KIV"; $resn_exch{"HC43 KIV"}="3HC4 KIV"; $resn_exch{"HC51 KIV"}="1HC5 KIV";
$resn_exch{"HC52 KIV"}="2HC5 KIV"; $resn_exch{"HC53 KIV"}="3HC5 KIV"; $resn_exch{"H221 KJ2"}="1H22 KJ2";
$resn_exch{"H222 KJ2"}="2H22 KJ2"; $resn_exch{"H231 KJ2"}="1H23 KJ2"; $resn_exch{"H232 KJ2"}="2H23 KJ2";
$resn_exch{"H241 KJ2"}="1H24 KJ2"; $resn_exch{"H242 KJ2"}="2H24 KJ2"; $resn_exch{"H251 KJ2"}="1H25 KJ2";
$resn_exch{"H252 KJ2"}="2H25 KJ2"; $resn_exch{"H4C1 KLN"}="1H4C KLN"; $resn_exch{"H4C2 KLN"}="2H4C KLN";
$resn_exch{"H7C1 KLN"}="1H7C KLN"; $resn_exch{"H7C2 KLN"}="2H7C KLN"; $resn_exch{"H141 KLN"}="1H14 KLN";
$resn_exch{"H142 KLN"}="2H14 KLN"; $resn_exch{"H211 KLN"}="1H21 KLN"; $resn_exch{"H212 KLN"}="2H21 KLN";
$resn_exch{"H241 KLN"}="1H24 KLN"; $resn_exch{"H242 KLN"}="2H24 KLN"; $resn_exch{"H221 KLN"}="1H22 KLN";
$resn_exch{"H222 KLN"}="2H22 KLN"; $resn_exch{"H231 KLN"}="1H23 KLN"; $resn_exch{"H232 KLN"}="2H23 KLN";
$resn_exch{"H261 KLN"}="1H26 KLN"; $resn_exch{"H262 KLN"}="2H26 KLN"; $resn_exch{"H263 KLN"}="3H26 KLN";
$resn_exch{" H41 KMB"}="1H4  KMB"; $resn_exch{" H42 KMB"}="2H4  KMB"; $resn_exch{"HN41 KMB"}="1HN4 KMB";
$resn_exch{"HN42 KMB"}="2HN4 KMB"; $resn_exch{"HN21 KMB"}="1HN2 KMB"; $resn_exch{"HN22 KMB"}="2HN2 KMB";
$resn_exch{" H91 KMB"}="1H9  KMB"; $resn_exch{" H92 KMB"}="2H9  KMB"; $resn_exch{" H31 KMT"}="1H3  KMT";
$resn_exch{" H32 KMT"}="2H3  KMT"; $resn_exch{" H41 KMT"}="1H4  KMT"; $resn_exch{" H42 KMT"}="2H4  KMT";
$resn_exch{" H51 KMT"}="1H5  KMT"; $resn_exch{" H52 KMT"}="2H5  KMT"; $resn_exch{" H53 KMT"}="3H5  KMT";
$resn_exch{"H101 KNI"}="1H10 KNI"; $resn_exch{"H102 KNI"}="2H10 KNI"; $resn_exch{"H121 KNI"}="1H12 KNI";
$resn_exch{"H122 KNI"}="2H12 KNI"; $resn_exch{"H161 KNI"}="1H16 KNI"; $resn_exch{"H162 KNI"}="2H16 KNI";
$resn_exch{"H191 KNI"}="1H19 KNI"; $resn_exch{"H192 KNI"}="2H19 KNI"; $resn_exch{"H201 KNI"}="1H20 KNI";
$resn_exch{"H202 KNI"}="2H20 KNI"; $resn_exch{"H231 KNI"}="1H23 KNI"; $resn_exch{"H232 KNI"}="2H23 KNI";
$resn_exch{"H233 KNI"}="3H23 KNI"; $resn_exch{"H241 KNI"}="1H24 KNI"; $resn_exch{"H242 KNI"}="2H24 KNI";
$resn_exch{"H243 KNI"}="3H24 KNI"; $resn_exch{"H251 KNI"}="1H25 KNI"; $resn_exch{"H252 KNI"}="2H25 KNI";
$resn_exch{"H253 KNI"}="3H25 KNI"; $resn_exch{"H271 KNI"}="1H27 KNI"; $resn_exch{"H272 KNI"}="2H27 KNI";
$resn_exch{"H273 KNI"}="3H27 KNI"; $resn_exch{" H11 KO4"}="1H1  KO4"; $resn_exch{" H12 KO4"}="2H1  KO4";
$resn_exch{" H21 KO4"}="1H2  KO4"; $resn_exch{" H22 KO4"}="2H2  KO4"; $resn_exch{" H31 KO4"}="1H3  KO4";
$resn_exch{" H32 KO4"}="2H3  KO4"; $resn_exch{" H41 KO4"}="1H4  KO4"; $resn_exch{" H42 KO4"}="2H4  KO4";
$resn_exch{"H6C1 KOJ"}="1H6C KOJ"; $resn_exch{"H6C2 KOJ"}="2H6C KOJ"; $resn_exch{"HAK1 KOR"}="1HAK KOR";
$resn_exch{"HAK2 KOR"}="2HAK KOR"; $resn_exch{"HAI3 KOR"}="1HAI KOR"; $resn_exch{"HAI2 KOR"}="2HAI KOR";
$resn_exch{" H   KOR"}="1HN  KOR"; $resn_exch{" H2  KOR"}="2HN  KOR"; $resn_exch{"HAH3 KOR"}="1HAH KOR";
$resn_exch{"HAH2 KOR"}="2HAH KOR"; $resn_exch{" HB3 KOR"}="1HBC KOR"; $resn_exch{" HB2 KOR"}="2HBC KOR";
$resn_exch{"H271 KOS"}="1H27 KOS"; $resn_exch{"H272 KOS"}="2H27 KOS"; $resn_exch{"H273 KOS"}="3H27 KOS";
$resn_exch{"H131 KOS"}="1H13 KOS"; $resn_exch{"H132 KOS"}="2H13 KOS"; $resn_exch{"H281 KOS"}="1H28 KOS";
$resn_exch{"H282 KOS"}="2H28 KOS"; $resn_exch{"H283 KOS"}="3H28 KOS"; $resn_exch{"H151 KOS"}="1H15 KOS";
$resn_exch{"H152 KOS"}="2H15 KOS"; $resn_exch{"H301 KOS"}="1H30 KOS"; $resn_exch{"H302 KOS"}="2H30 KOS";
$resn_exch{"H311 KOS"}="1H31 KOS"; $resn_exch{"H312 KOS"}="2H31 KOS"; $resn_exch{"H331 KOS"}="1H33 KOS";
$resn_exch{"H332 KOS"}="2H33 KOS"; $resn_exch{"H333 KOS"}="3H33 KOS"; $resn_exch{"H341 KOS"}="1H34 KOS";
$resn_exch{"H342 KOS"}="2H34 KOS"; $resn_exch{"H343 KOS"}="3H34 KOS"; $resn_exch{"H221 KOS"}="1H22 KOS";
$resn_exch{"H222 KOS"}="2H22 KOS"; $resn_exch{"H223 KOS"}="3H22 KOS"; $resn_exch{"H231 KOS"}="1H23 KOS";
$resn_exch{"H232 KOS"}="2H23 KOS"; $resn_exch{"H233 KOS"}="3H23 KOS"; $resn_exch{"HN21 KOS"}="1HN2 KOS";
$resn_exch{"HN22 KOS"}="2HN2 KOS"; $resn_exch{"H251 KOS"}="1H25 KOS"; $resn_exch{"H252 KOS"}="2H25 KOS";
$resn_exch{"H253 KOS"}="3H25 KOS"; $resn_exch{"H261 KOS"}="1H26 KOS"; $resn_exch{"H262 KOS"}="2H26 KOS";
$resn_exch{"H263 KOS"}="3H26 KOS"; $resn_exch{" HB1 KPA"}="1HB  KPA"; $resn_exch{" HB2 KPA"}="2HB  KPA";
$resn_exch{" HG1 KPA"}="1HG  KPA"; $resn_exch{" HG2 KPA"}="2HG  KPA"; $resn_exch{" HP1 KPA"}="1HP  KPA";
$resn_exch{" HP2 KPA"}="2HP  KPA"; $resn_exch{"HAK1 KPC"}="1HAK KPC"; $resn_exch{"HAK2 KPC"}="2HAK KPC";
$resn_exch{"HAK3 KPC"}="3HAK KPC"; $resn_exch{"HAI1 KPC"}="1HAI KPC"; $resn_exch{"HAI2 KPC"}="2HAI KPC";
$resn_exch{"HAG1 KPC"}="1HAG KPC"; $resn_exch{"HAG2 KPC"}="2HAG KPC"; $resn_exch{"HAF1 KPC"}="1HAF KPC";
$resn_exch{"HAF2 KPC"}="2HAF KPC"; $resn_exch{"HAL1 KPC"}="1HAL KPC"; $resn_exch{" HN1 KPH"}="1HN  KPH";
$resn_exch{" HN2 KPH"}="2HN  KPH"; $resn_exch{"HOP2 KPH"}="2HOP KPH"; $resn_exch{" HB1 KPH"}="1HB  KPH";
$resn_exch{" HB2 KPH"}="2HB  KPH"; $resn_exch{" HG1 KPH"}="1HG  KPH"; $resn_exch{" HG2 KPH"}="2HG  KPH";
$resn_exch{" HD1 KPH"}="1HD  KPH"; $resn_exch{" HD2 KPH"}="2HD  KPH"; $resn_exch{" HE1 KPH"}="1HE  KPH";
$resn_exch{" HE2 KPH"}="2HE  KPH"; $resn_exch{"HNZ1 KPH"}="1HNZ KPH"; $resn_exch{"HNZ2 KPH"}="2HNZ KPH";
$resn_exch{" H31 KPL"}="1H3  KPL"; $resn_exch{" H32 KPL"}="2H3  KPL"; $resn_exch{" H33 KPL"}="3H3  KPL";
$resn_exch{" H11 KPL"}="1H1  KPL"; $resn_exch{" H12 KPL"}="2H1  KPL"; $resn_exch{" H13 KPL"}="3H1  KPL";
$resn_exch{" H41 KPL"}="1H4  KPL"; $resn_exch{" H42 KPL"}="2H4  KPL"; $resn_exch{" H71 KPV"}="1H7  KPV";
$resn_exch{" H72 KPV"}="2H7  KPV"; $resn_exch{" H81 KPV"}="1H8  KPV"; $resn_exch{" H82 KPV"}="2H8  KPV";
$resn_exch{" H91 KPV"}="1H9  KPV"; $resn_exch{" H92 KPV"}="2H9  KPV"; $resn_exch{"HAM1 KR2"}="1HAM KR2";
$resn_exch{"HAM2 KR2"}="2HAM KR2"; $resn_exch{"HAN1 KR2"}="1HAN KR2"; $resn_exch{"HAN2 KR2"}="2HAN KR2";
$resn_exch{"HAV1 KR2"}="1HAV KR2"; $resn_exch{"HAV2 KR2"}="2HAV KR2"; $resn_exch{"HDD1 KR2"}="1HDD KR2";
$resn_exch{"HDD2 KR2"}="2HDD KR2"; $resn_exch{"HNN1 KR2"}="1HNN KR2"; $resn_exch{"HNN2 KR2"}="2HNN KR2";
$resn_exch{"HAT1 KR2"}="1HAT KR2"; $resn_exch{"HAT2 KR2"}="2HAT KR2"; $resn_exch{"HAS1 KR2"}="1HAS KR2";
$resn_exch{"HAS2 KR2"}="2HAS KR2"; $resn_exch{"HAK1 KR2"}="1HAK KR2"; $resn_exch{"HAK2 KR2"}="2HAK KR2";
$resn_exch{" H71 KRA"}="1H7  KRA"; $resn_exch{" H72 KRA"}="2H7  KRA"; $resn_exch{"H101 KRA"}="1H10 KRA";
$resn_exch{"H102 KRA"}="2H10 KRA"; $resn_exch{"H161 KRB"}="1H16 KRB"; $resn_exch{"H162 KRB"}="2H16 KRB";
$resn_exch{"H251 KRB"}="1H25 KRB"; $resn_exch{"H252 KRB"}="2H25 KRB"; $resn_exch{" HN1 KRI"}="1HN  KRI";
$resn_exch{" HN2 KRI"}="2HN  KRI"; $resn_exch{" HB1 KRI"}="1HB  KRI"; $resn_exch{" HB2 KRI"}="2HB  KRI";
$resn_exch{" HG1 KRI"}="1HG  KRI"; $resn_exch{" HG2 KRI"}="2HG  KRI"; $resn_exch{" H51 KRI"}="1H5  KRI";
$resn_exch{" H52 KRI"}="2H5  KRI"; $resn_exch{" H11 KRI"}="1H1  KRI"; $resn_exch{" H12 KRI"}="2H1  KRI";
$resn_exch{"H261 KSA"}="1H26 KSA"; $resn_exch{"H262 KSA"}="2H26 KSA"; $resn_exch{"H263 KSA"}="3H26 KSA";
$resn_exch{"H251 KSA"}="1H25 KSA"; $resn_exch{"H252 KSA"}="2H25 KSA"; $resn_exch{"H253 KSA"}="3H25 KSA";
$resn_exch{"H271 KSA"}="1H27 KSA"; $resn_exch{"H272 KSA"}="2H27 KSA"; $resn_exch{"H231 KSA"}="1H23 KSA";
$resn_exch{"H232 KSA"}="2H23 KSA"; $resn_exch{" H91 KSG"}="1H9  KSG"; $resn_exch{" H92 KSG"}="2H9  KSG";
$resn_exch{" H93 KSG"}="3H9  KSG"; $resn_exch{"H111 KSG"}="1H11 KSG"; $resn_exch{"H112 KSG"}="2H11 KSG";
$resn_exch{"HN11 KSG"}="1HN1 KSG"; $resn_exch{"HN12 KSG"}="2HN1 KSG"; $resn_exch{" H41 KST"}="1H4  KST";
$resn_exch{" H42 KST"}="2H4  KST"; $resn_exch{" H31 KST"}="1H3  KST"; $resn_exch{" H32 KST"}="2H3  KST";
$resn_exch{" H21 KST"}="1H2  KST"; $resn_exch{" H22 KST"}="2H2  KST"; $resn_exch{" H11 KST"}="1H1  KST";
$resn_exch{" H12 KST"}="2H1  KST"; $resn_exch{"HN1A KST"}="1HN  KST"; $resn_exch{" HN2 KST"}="2HN  KST";
$resn_exch{" H2  KT3"}="2H   KT3"; $resn_exch{"HG11 KT3"}="1HG1 KT3"; $resn_exch{"HG21 KT3"}="1HG2 KT3";
$resn_exch{"HB11 KT3"}="1HB1 KT3"; $resn_exch{"HB21 KT3"}="1HB2 KT3"; $resn_exch{"HA1A KT3"}="1HA  KT3";
$resn_exch{"HXT1 KT3"}="1HXT KT3"; $resn_exch{" H1  KT3"}="1H   KT3"; $resn_exch{" H11 KT3"}="1H1  KT3";
$resn_exch{"H12A KT3"}="2H1  KT3"; $resn_exch{" H21 KT3"}="1H2  KT3"; $resn_exch{" H22 KT3"}="2H2  KT3";
$resn_exch{" H31 KT3"}="1H3  KT3"; $resn_exch{" H32 KT3"}="2H3  KT3"; $resn_exch{"HN81 KT3"}="1HN8 KT3";
$resn_exch{"HN82 KT3"}="2HN8 KT3"; $resn_exch{"HN21 KT3"}="1HN2 KT3"; $resn_exch{"HN22 KT3"}="2HN2 KT3";
$resn_exch{"HE23 KT3"}="3HE2 KT3"; $resn_exch{"HG13 KT3"}="3HG1 KT3"; $resn_exch{"HG23 KT3"}="3HG2 KT3";
$resn_exch{"HB13 KT3"}="3HB1 KT3"; $resn_exch{"HB23 KT3"}="3HB2 KT3"; $resn_exch{" HA3 KT3"}="3HA  KT3";
$resn_exch{"HXT3 KT3"}="3HXT KT3"; $resn_exch{" H3  KT3"}="3H   KT3"; $resn_exch{"HG12 KT3"}="2HG1 KT3";
$resn_exch{"HG22 KT3"}="2HG2 KT3"; $resn_exch{"HB12 KT3"}="2HB1 KT3"; $resn_exch{"HB22 KT3"}="2HB2 KT3";
$resn_exch{"HA2A KT3"}="2HA  KT3"; $resn_exch{"HXT2 KT3"}="2HXT KT3"; $resn_exch{" H2  KT5"}="2H   KT5";
$resn_exch{"HG11 KT5"}="1HG1 KT5"; $resn_exch{"HG21 KT5"}="1HG2 KT5"; $resn_exch{"HB11 KT5"}="1HB1 KT5";
$resn_exch{"HB21 KT5"}="1HB2 KT5"; $resn_exch{"HA1A KT5"}="1HA  KT5"; $resn_exch{"HXT1 KT5"}="1HXT KT5";
$resn_exch{" H1  KT5"}="1H   KT5"; $resn_exch{" H11 KT5"}="1H1  KT5"; $resn_exch{"H12A KT5"}="2H1  KT5";
$resn_exch{" H21 KT5"}="1H2  KT5"; $resn_exch{" H22 KT5"}="2H2  KT5"; $resn_exch{" H31 KT5"}="1H3  KT5";
$resn_exch{" H32 KT5"}="2H3  KT5"; $resn_exch{"HN81 KT5"}="1HN8 KT5"; $resn_exch{"HN82 KT5"}="2HN8 KT5";
$resn_exch{"HN21 KT5"}="1HN2 KT5"; $resn_exch{"HN22 KT5"}="2HN2 KT5"; $resn_exch{"HE25 KT5"}="5HE2 KT5";
$resn_exch{"HG15 KT5"}="5HG1 KT5"; $resn_exch{"HG25 KT5"}="5HG2 KT5"; $resn_exch{"HB15 KT5"}="5HB1 KT5";
$resn_exch{"HB25 KT5"}="5HB2 KT5"; $resn_exch{" HA5 KT5"}="5HA  KT5"; $resn_exch{"HXT5 KT5"}="5HXT KT5";
$resn_exch{" H5  KT5"}="5H   KT5"; $resn_exch{"HG14 KT5"}="4HG1 KT5"; $resn_exch{"HG24 KT5"}="4HG2 KT5";
$resn_exch{"HB14 KT5"}="4HB1 KT5"; $resn_exch{"HB24 KT5"}="4HB2 KT5"; $resn_exch{" HA4 KT5"}="4HA  KT5";
$resn_exch{"HXT4 KT5"}="4HXT KT5"; $resn_exch{" H4  KT5"}="4H   KT5"; $resn_exch{"HG13 KT5"}="3HG1 KT5";
$resn_exch{"HG23 KT5"}="3HG2 KT5"; $resn_exch{"HB13 KT5"}="3HB1 KT5"; $resn_exch{"HB23 KT5"}="3HB2 KT5";
$resn_exch{" HA3 KT5"}="3HA  KT5"; $resn_exch{"HXT3 KT5"}="3HXT KT5"; $resn_exch{" H3  KT5"}="3H   KT5";
$resn_exch{"HG12 KT5"}="2HG1 KT5"; $resn_exch{"HG22 KT5"}="2HG2 KT5"; $resn_exch{"HB12 KT5"}="2HB1 KT5";
$resn_exch{"HB22 KT5"}="2HB2 KT5"; $resn_exch{"HA2A KT5"}="2HA  KT5"; $resn_exch{"HXT2 KT5"}="2HXT KT5";
$resn_exch{"H6C1 KTA"}="1H6C KTA"; $resn_exch{"H6C2 KTA"}="2H6C KTA"; $resn_exch{"H5C1 KTA"}="1H5C KTA";
$resn_exch{"H5C2 KTA"}="2H5C KTA"; $resn_exch{"H4C1 KTA"}="1H4C KTA"; $resn_exch{"H4C2 KTA"}="2H4C KTA";
$resn_exch{"H2C1 KTA"}="1H2C KTA"; $resn_exch{"H2C2 KTA"}="2H2C KTA"; $resn_exch{"H7C1 KTA"}="1H7C KTA";
$resn_exch{"H7C2 KTA"}="2H7C KTA"; $resn_exch{"HC41 KTN"}="1HC4 KTN"; $resn_exch{"HC42 KTN"}="2HC4 KTN";
$resn_exch{"HC71 KTN"}="1HC7 KTN"; $resn_exch{"HC72 KTN"}="2HC7 KTN"; $resn_exch{"H141 KTN"}="1H14 KTN";
$resn_exch{"H142 KTN"}="2H14 KTN"; $resn_exch{"H211 KTN"}="1H21 KTN"; $resn_exch{"H212 KTN"}="2H21 KTN";
$resn_exch{"H221 KTN"}="1H22 KTN"; $resn_exch{"H222 KTN"}="2H22 KTN"; $resn_exch{"H231 KTN"}="1H23 KTN";
$resn_exch{"H232 KTN"}="2H23 KTN"; $resn_exch{"H241 KTN"}="1H24 KTN"; $resn_exch{"H242 KTN"}="2H24 KTN";
$resn_exch{"H261 KTN"}="1H26 KTN"; $resn_exch{"H262 KTN"}="2H26 KTN"; $resn_exch{"H263 KTN"}="3H26 KTN";
$resn_exch{"H231 KWT"}="1H23 KWT"; $resn_exch{"H232 KWT"}="2H23 KWT"; $resn_exch{"H101 KWT"}="1H10 KWT";
$resn_exch{"H102 KWT"}="2H10 KWT"; $resn_exch{"H103 KWT"}="3H10 KWT"; $resn_exch{"H121 KWT"}="1H12 KWT";
$resn_exch{"H122 KWT"}="2H12 KWT"; $resn_exch{"H141 KWT"}="1H14 KWT"; $resn_exch{"H142 KWT"}="2H14 KWT";
$resn_exch{"H143 KWT"}="3H14 KWT"; $resn_exch{"H161 KWT"}="1H16 KWT"; $resn_exch{"H162 KWT"}="2H16 KWT";
$resn_exch{"H171 KWT"}="1H17 KWT"; $resn_exch{"H172 KWT"}="2H17 KWT"; $resn_exch{"H221 KWT"}="1H22 KWT";
$resn_exch{"H222 KWT"}="2H22 KWT"; $resn_exch{"H223 KWT"}="3H22 KWT"; $resn_exch{"H241 KWT"}="1H24 KWT";
$resn_exch{"H242 KWT"}="2H24 KWT"; $resn_exch{"H243 KWT"}="3H24 KWT"; $resn_exch{"HAC1 KY1"}="1HAC KY1";
$resn_exch{"HAC2 KY1"}="2HAC KY1"; $resn_exch{"HBC1 KY1"}="1HBC KY1"; $resn_exch{"HBC2 KY1"}="2HBC KY1";
$resn_exch{"H1N1 KY1"}="1H1N KY1"; $resn_exch{"H1N2 KY1"}="2H1N KY1"; $resn_exch{" H2  KYN"}=" HN2 KYN";
$resn_exch{" HB2 KYN"}="1HB  KYN"; $resn_exch{" HB3 KYN"}="2HB  KYN"; $resn_exch{" HD1 KYN"}="1HD  KYN";
$resn_exch{" HE1 KYN"}="1HE  KYN"; $resn_exch{" HE2 KYN"}="2HE  KYN"; $resn_exch{" H11 KYN"}="1H1  KYN";
$resn_exch{" H12 KYN"}="2H1  KYN"; $resn_exch{"H511 KYS"}="1H51 KYS"; $resn_exch{"H512 KYS"}="2H51 KYS";
$resn_exch{"H471 KYS"}="1H47 KYS"; $resn_exch{"H472 KYS"}="2H47 KYS"; $resn_exch{"H461 KYS"}="1H46 KYS";
$resn_exch{"H462 KYS"}="2H46 KYS"; $resn_exch{"H451 KYS"}="1H45 KYS"; $resn_exch{"H452 KYS"}="2H45 KYS";
$resn_exch{"H401 KYS"}="1H40 KYS"; $resn_exch{"H402 KYS"}="2H40 KYS"; $resn_exch{"H281 KYS"}="1H28 KYS";
$resn_exch{"H282 KYS"}="2H28 KYS"; $resn_exch{"H291 KYS"}="1H29 KYS"; $resn_exch{"H292 KYS"}="2H29 KYS";
$resn_exch{"H151 L00"}="1H15 L00"; $resn_exch{"H152 L00"}="2H15 L00"; $resn_exch{"H153 L00"}="3H15 L00";
$resn_exch{"H161 L00"}="1H16 L00"; $resn_exch{"H162 L00"}="2H16 L00"; $resn_exch{"H163 L00"}="3H16 L00";
$resn_exch{"HN61 L00"}="1HN6 L00"; $resn_exch{"HN62 L00"}="2HN6 L00"; $resn_exch{"H211 L00"}="1H21 L00";
$resn_exch{"H212 L00"}="2H21 L00"; $resn_exch{"H213 L00"}="3H21 L00"; $resn_exch{"H221 L00"}="1H22 L00";
$resn_exch{"H222 L00"}="2H22 L00"; $resn_exch{" H11 L00"}="1H1  L00"; $resn_exch{" H12 L00"}="2H1  L00";
$resn_exch{" H21 L00"}="1H2  L00"; $resn_exch{" H22 L00"}="2H2  L00"; $resn_exch{" H31 L00"}="1H3  L00";
$resn_exch{" H32 L00"}="2H3  L00"; $resn_exch{" H41 L00"}="1H4  L00"; $resn_exch{" H42 L00"}="2H4  L00";
$resn_exch{" H43 L00"}="3H4  L00"; $resn_exch{" H71 L00"}="1H7  L00"; $resn_exch{" H72 L00"}="2H7  L00";
$resn_exch{" H81 L00"}="1H8  L00"; $resn_exch{" H82 L00"}="2H8  L00"; $resn_exch{" H83 L00"}="3H8  L00";
$resn_exch{"H1C1 L01"}="1H1C L01"; $resn_exch{"H1C2 L01"}="2H1C L01"; $resn_exch{"H1C3 L01"}="3H1C L01";
$resn_exch{"H2C1 L01"}="1H2C L01"; $resn_exch{"H2C2 L01"}="2H2C L01"; $resn_exch{"H3C1 L01"}="1H3C L01";
$resn_exch{"H3C2 L01"}="2H3C L01"; $resn_exch{"H5C1 L01"}="1H5C L01"; $resn_exch{"H5C2 L01"}="2H5C L01";
$resn_exch{"H6C1 L01"}="1H6C L01"; $resn_exch{"H6C2 L01"}="2H6C L01"; $resn_exch{"H7C1 L01"}="1H7C L01";
$resn_exch{"H7C2 L01"}="2H7C L01"; $resn_exch{"H7C3 L01"}="3H7C L01"; $resn_exch{"H211 L01"}="1H21 L01";
$resn_exch{"H212 L01"}="2H21 L01"; $resn_exch{"H311 L01"}="1H31 L01"; $resn_exch{"H312 L01"}="2H31 L01";
$resn_exch{"H351 L01"}="1H35 L01"; $resn_exch{"H352 L01"}="2H35 L01"; $resn_exch{"H421 L01"}="1H42 L01";
$resn_exch{"H422 L01"}="2H42 L01"; $resn_exch{"H423 L01"}="3H42 L01"; $resn_exch{"H8C1 L02"}="1H8C L02";
$resn_exch{"H8C2 L02"}="2H8C L02"; $resn_exch{"H101 L02"}="1H10 L02"; $resn_exch{"H102 L02"}="2H10 L02";
$resn_exch{"H131 L02"}="1H13 L02"; $resn_exch{"H132 L02"}="2H13 L02"; $resn_exch{"H151 L02"}="1H15 L02";
$resn_exch{"H152 L02"}="2H15 L02"; $resn_exch{"H1C1 L03"}="1H1C L03"; $resn_exch{"H1C2 L03"}="2H1C L03";
$resn_exch{"H1C3 L03"}="3H1C L03"; $resn_exch{"H11A L04"}="1H1  L04"; $resn_exch{"H12A L04"}="2H1  L04";
$resn_exch{" H41 L04"}="1H4  L04"; $resn_exch{"H42A L04"}="2H4  L04"; $resn_exch{"H43A L04"}="3H4  L04";
$resn_exch{" H81 L04"}="1H8  L04"; $resn_exch{" H82 L04"}="2H8  L04"; $resn_exch{" H91 L04"}="1H9  L04";
$resn_exch{" H92 L04"}="2H9  L04"; $resn_exch{"H181 L04"}="1H18 L04"; $resn_exch{"H182 L04"}="2H18 L04";
$resn_exch{"H201 L04"}="1H20 L04"; $resn_exch{"H202 L04"}="2H20 L04"; $resn_exch{"H221 L04"}="1H22 L04";
$resn_exch{"H222 L04"}="2H22 L04"; $resn_exch{"H223 L04"}="3H22 L04"; $resn_exch{"H231 L04"}="1H23 L04";
$resn_exch{"H232 L04"}="2H23 L04"; $resn_exch{"H241 L04"}="1H24 L04"; $resn_exch{"H242 L04"}="2H24 L04";
$resn_exch{"H251 L04"}="1H25 L04"; $resn_exch{"H252 L04"}="2H25 L04"; $resn_exch{"H281 L04"}="1H28 L04";
$resn_exch{"H282 L04"}="2H28 L04"; $resn_exch{" H11 L05"}="1H1  L05"; $resn_exch{" H12 L05"}="2H1  L05";
$resn_exch{" H13 L05"}="3H1  L05"; $resn_exch{"H131 L05"}="1H13 L05"; $resn_exch{"H132 L05"}="2H13 L05";
$resn_exch{" H11 L06"}="1H1  L06"; $resn_exch{"H12A L06"}="2H1  L06"; $resn_exch{" H31 L06"}="1H3  L06";
$resn_exch{" H32 L06"}="2H3  L06"; $resn_exch{" H41 L06"}="1H4  L06"; $resn_exch{" H42 L06"}="2H4  L06";
$resn_exch{" H51 L06"}="1H5  L06"; $resn_exch{" H52 L06"}="2H5  L06"; $resn_exch{"HN91 L06"}="1HN9 L06";
$resn_exch{"HN92 L06"}="2HN9 L06"; $resn_exch{"H191 L08"}="1H19 L08"; $resn_exch{"H192 L08"}="2H19 L08";
$resn_exch{"H201 L08"}="1H20 L08"; $resn_exch{"H202 L08"}="2H20 L08"; $resn_exch{"H211 L08"}="1H21 L08";
$resn_exch{"H212 L08"}="2H21 L08"; $resn_exch{"H221 L08"}="1H22 L08"; $resn_exch{"H222 L08"}="2H22 L08";
$resn_exch{"H241 L08"}="1H24 L08"; $resn_exch{"H242 L08"}="2H24 L08"; $resn_exch{"H243 L08"}="3H24 L08";
$resn_exch{" H91 L08"}="1H9  L08"; $resn_exch{" H92 L08"}="2H9  L08"; $resn_exch{"H331 L08"}="1H33 L08";
$resn_exch{"H332 L08"}="2H33 L08"; $resn_exch{"H333 L08"}="3H33 L08"; $resn_exch{"H1C1 L09"}="1H1C L09";
$resn_exch{"H1C2 L09"}="2H1C L09"; $resn_exch{"H1C3 L09"}="3H1C L09"; $resn_exch{"H3C1 L09"}="1H3C L09";
$resn_exch{"H3C2 L09"}="2H3C L09"; $resn_exch{"H3C3 L09"}="3H3C L09"; $resn_exch{"H4C1 L09"}="1H4C L09";
$resn_exch{"H4C2 L09"}="2H4C L09"; $resn_exch{"H4C3 L09"}="3H4C L09"; $resn_exch{"H181 L09"}="1H18 L09";
$resn_exch{"H182 L09"}="2H18 L09"; $resn_exch{"H4C1 L10"}="1H4C L10"; $resn_exch{"H4C2 L10"}="2H4C L10";
$resn_exch{"H4C3 L10"}="3H4C L10"; $resn_exch{"H3C1 L10"}="1H3C L10"; $resn_exch{"H3C2 L10"}="2H3C L10";
$resn_exch{"H3C3 L10"}="3H3C L10"; $resn_exch{"H1C1 L10"}="1H1C L10"; $resn_exch{"H1C2 L10"}="2H1C L10";
$resn_exch{"H1C3 L10"}="3H1C L10"; $resn_exch{"H171 L11"}="1H17 L11"; $resn_exch{"H172 L11"}="2H17 L11";
$resn_exch{"H3C1 L11"}="1H3C L11"; $resn_exch{"H3C2 L11"}="2H3C L11"; $resn_exch{"H311 L11"}="1H31 L11";
$resn_exch{"H312 L11"}="2H31 L11"; $resn_exch{"H301 L11"}="1H30 L11"; $resn_exch{"H302 L11"}="2H30 L11";
$resn_exch{"H281 L11"}="1H28 L11"; $resn_exch{"H282 L11"}="2H28 L11"; $resn_exch{"H271 L11"}="1H27 L11";
$resn_exch{"H272 L11"}="2H27 L11"; $resn_exch{"H5C1 L12"}="1H5C L12"; $resn_exch{"H5C2 L12"}="2H5C L12";
$resn_exch{"H6C1 L12"}="1H6C L12"; $resn_exch{"H6C2 L12"}="2H6C L12"; $resn_exch{"H141 L13"}="1H14 L13";
$resn_exch{"H142 L13"}="2H14 L13"; $resn_exch{"H131 L13"}="1H13 L13"; $resn_exch{"H132 L13"}="2H13 L13";
$resn_exch{"H133 L13"}="3H13 L13"; $resn_exch{"H1C1 L15"}="1H1C L15"; $resn_exch{"H1C2 L15"}="2H1C L15";
$resn_exch{"H1C3 L15"}="3H1C L15"; $resn_exch{"H111 L15"}="1H11 L15"; $resn_exch{"H112 L15"}="2H11 L15";
$resn_exch{"H121 L15"}="1H12 L15"; $resn_exch{"H122 L15"}="2H12 L15"; $resn_exch{"H131 L15"}="1H13 L15";
$resn_exch{"H132 L15"}="2H13 L15"; $resn_exch{" H11 L17"}="1H1  L17"; $resn_exch{" H12 L17"}="2H1  L17";
$resn_exch{"H331 L17"}="1H33 L17"; $resn_exch{"H332 L17"}="2H33 L17"; $resn_exch{" H71 L17"}="1H7  L17";
$resn_exch{" H72 L17"}="2H7  L17"; $resn_exch{" H73 L17"}="3H7  L17"; $resn_exch{" H91 L1A"}="1H9  L1A";
$resn_exch{" H92 L1A"}="2H9  L1A"; $resn_exch{"H131 L1A"}="1H13 L1A"; $resn_exch{"H132 L1A"}="2H13 L1A";
$resn_exch{"HN41 L1A"}="1HN4 L1A"; $resn_exch{"HN42 L1A"}="2HN4 L1A"; $resn_exch{"H171 L1A"}="1H17 L1A";
$resn_exch{"H172 L1A"}="2H17 L1A"; $resn_exch{"H201 L1A"}="1H20 L1A"; $resn_exch{"H202 L1A"}="2H20 L1A";
$resn_exch{"H221 L1A"}="1H22 L1A"; $resn_exch{"H222 L1A"}="2H22 L1A"; $resn_exch{"H223 L1A"}="3H22 L1A";
$resn_exch{"H231 L1A"}="1H23 L1A"; $resn_exch{"H232 L1A"}="2H23 L1A"; $resn_exch{"H233 L1A"}="3H23 L1A";
$resn_exch{"H241 L1A"}="1H24 L1A"; $resn_exch{"H242 L1A"}="2H24 L1A"; $resn_exch{"H251 L1A"}="1H25 L1A";
$resn_exch{"H252 L1A"}="2H25 L1A"; $resn_exch{"H261 L1A"}="1H26 L1A"; $resn_exch{"H262 L1A"}="2H26 L1A";
$resn_exch{"H271 L1A"}="1H27 L1A"; $resn_exch{"H272 L1A"}="2H27 L1A"; $resn_exch{"H281 L1A"}="1H28 L1A";
$resn_exch{"H282 L1A"}="2H28 L1A"; $resn_exch{"H301 L1A"}="1H30 L1A"; $resn_exch{"H302 L1A"}="2H30 L1A";
$resn_exch{"H331 L1A"}="1H33 L1A"; $resn_exch{"H332 L1A"}="2H33 L1A"; $resn_exch{"HN31 L1B"}="1HN3 L1B";
$resn_exch{"HN32 L1B"}="2HN3 L1B"; $resn_exch{"HN41 L1B"}="1HN4 L1B"; $resn_exch{"HN42 L1B"}="2HN4 L1B";
$resn_exch{" H51 L1B"}="1H5  L1B"; $resn_exch{" H52 L1B"}="2H5  L1B"; $resn_exch{" H61 L1B"}="1H6  L1B";
$resn_exch{" H62 L1B"}="2H6  L1B"; $resn_exch{" H63 L1B"}="3H6  L1B"; $resn_exch{"H161 L1B"}="1H16 L1B";
$resn_exch{"H162 L1B"}="2H16 L1B"; $resn_exch{"H171 L1B"}="1H17 L1B"; $resn_exch{"H172 L1B"}="2H17 L1B";
$resn_exch{"H181 L1B"}="1H18 L1B"; $resn_exch{"H182 L1B"}="2H18 L1B"; $resn_exch{"H191 L1B"}="1H19 L1B";
$resn_exch{"H192 L1B"}="2H19 L1B"; $resn_exch{"H193 L1B"}="3H19 L1B"; $resn_exch{"H5N1 L1G"}="1H5N L1G";
$resn_exch{"H5N2 L1G"}="2H5N L1G"; $resn_exch{"H131 L1G"}="1H13 L1G"; $resn_exch{"H132 L1G"}="2H13 L1G";
$resn_exch{"H141 L1G"}="1H14 L1G"; $resn_exch{"H142 L1G"}="2H14 L1G"; $resn_exch{"H151 L1G"}="1H15 L1G";
$resn_exch{"H152 L1G"}="2H15 L1G"; $resn_exch{"H153 L1G"}="3H15 L1G"; $resn_exch{"H161 L1G"}="1H16 L1G";
$resn_exch{"H162 L1G"}="2H16 L1G"; $resn_exch{"H171 L1G"}="1H17 L1G"; $resn_exch{"H172 L1G"}="2H17 L1G";
$resn_exch{"H221 L1G"}="1H22 L1G"; $resn_exch{"H222 L1G"}="2H22 L1G"; $resn_exch{"H231 L1G"}="1H23 L1G";
$resn_exch{"H232 L1G"}="2H23 L1G"; $resn_exch{"H251 L1G"}="1H25 L1G"; $resn_exch{"H252 L1G"}="2H25 L1G";
$resn_exch{"H253 L1G"}="3H25 L1G"; $resn_exch{"H271 L1G"}="1H27 L1G"; $resn_exch{"H272 L1G"}="2H27 L1G";
$resn_exch{"H281 L1G"}="1H28 L1G"; $resn_exch{"H282 L1G"}="2H28 L1G"; $resn_exch{"H331 L1G"}="1H33 L1G";
$resn_exch{"H332 L1G"}="2H33 L1G"; $resn_exch{"H333 L1G"}="3H33 L1G"; $resn_exch{"H1N1 L1O"}="1H1N L1O";
$resn_exch{"H1N2 L1O"}="2H1N L1O"; $resn_exch{"H2C1 L1O"}="1H2C L1O"; $resn_exch{"H2C2 L1O"}="2H2C L1O";
$resn_exch{"H3C1 L1O"}="1H3C L1O"; $resn_exch{"H3C2 L1O"}="2H3C L1O"; $resn_exch{" H12 L1P"}="2H1  L1P";
$resn_exch{" H11 L1P"}="1H1  L1P"; $resn_exch{" H32 L1P"}="2H3  L1P"; $resn_exch{" H31 L1P"}="1H3  L1P";
$resn_exch{"HOP2 L1P"}="2HOP L1P"; $resn_exch{"HOP3 L1P"}="3HOP L1P"; $resn_exch{"H112 L1P"}="2H11 L1P";
$resn_exch{"H111 L1P"}="1H11 L1P"; $resn_exch{"H122 L1P"}="2H12 L1P"; $resn_exch{"H121 L1P"}="1H12 L1P";
$resn_exch{"H143 L1P"}="3H14 L1P"; $resn_exch{"H142 L1P"}="2H14 L1P"; $resn_exch{"H141 L1P"}="1H14 L1P";
$resn_exch{"H152 L1P"}="2H15 L1P"; $resn_exch{"H151 L1P"}="1H15 L1P"; $resn_exch{"H162 L1P"}="2H16 L1P";
$resn_exch{"H161 L1P"}="1H16 L1P"; $resn_exch{"H172 L1P"}="2H17 L1P"; $resn_exch{"H171 L1P"}="1H17 L1P";
$resn_exch{"H193 L1P"}="3H19 L1P"; $resn_exch{"H192 L1P"}="2H19 L1P"; $resn_exch{"H191 L1P"}="1H19 L1P";
$resn_exch{"H202 L1P"}="2H20 L1P"; $resn_exch{"H201 L1P"}="1H20 L1P"; $resn_exch{"H212 L1P"}="2H21 L1P";
$resn_exch{"H211 L1P"}="1H21 L1P"; $resn_exch{"H222 L1P"}="2H22 L1P"; $resn_exch{"H221 L1P"}="1H22 L1P";
$resn_exch{"H243 L1P"}="3H24 L1P"; $resn_exch{"H242 L1P"}="2H24 L1P"; $resn_exch{"H241 L1P"}="1H24 L1P";
$resn_exch{"H252 L1P"}="2H25 L1P"; $resn_exch{"H251 L1P"}="1H25 L1P"; $resn_exch{"H262 L1P"}="2H26 L1P";
$resn_exch{"H261 L1P"}="1H26 L1P"; $resn_exch{"H272 L1P"}="2H27 L1P"; $resn_exch{"H271 L1P"}="1H27 L1P";
$resn_exch{"H293 L1P"}="3H29 L1P"; $resn_exch{"H292 L1P"}="2H29 L1P"; $resn_exch{"H291 L1P"}="1H29 L1P";
$resn_exch{"H303 L1P"}="3H30 L1P"; $resn_exch{"H302 L1P"}="2H30 L1P"; $resn_exch{"H301 L1P"}="1H30 L1P";
$resn_exch{"H412 L1P"}="2H41 L1P"; $resn_exch{"H411 L1P"}="1H41 L1P"; $resn_exch{"H422 L1P"}="2H42 L1P";
$resn_exch{"H421 L1P"}="1H42 L1P"; $resn_exch{"H443 L1P"}="3H44 L1P"; $resn_exch{"H442 L1P"}="2H44 L1P";
$resn_exch{"H441 L1P"}="1H44 L1P"; $resn_exch{"H452 L1P"}="2H45 L1P"; $resn_exch{"H451 L1P"}="1H45 L1P";
$resn_exch{"H462 L1P"}="2H46 L1P"; $resn_exch{"H461 L1P"}="1H46 L1P"; $resn_exch{"H472 L1P"}="2H47 L1P";
$resn_exch{"H471 L1P"}="1H47 L1P"; $resn_exch{"H493 L1P"}="3H49 L1P"; $resn_exch{"H492 L1P"}="2H49 L1P";
$resn_exch{"H491 L1P"}="1H49 L1P"; $resn_exch{"H502 L1P"}="2H50 L1P"; $resn_exch{"H501 L1P"}="1H50 L1P";
$resn_exch{"H512 L1P"}="2H51 L1P"; $resn_exch{"H511 L1P"}="1H51 L1P"; $resn_exch{"H522 L1P"}="2H52 L1P";
$resn_exch{"H521 L1P"}="1H52 L1P"; $resn_exch{"H543 L1P"}="3H54 L1P"; $resn_exch{"H542 L1P"}="2H54 L1P";
$resn_exch{"H541 L1P"}="1H54 L1P"; $resn_exch{"H552 L1P"}="2H55 L1P"; $resn_exch{"H551 L1P"}="1H55 L1P";
$resn_exch{"H562 L1P"}="2H56 L1P"; $resn_exch{"H561 L1P"}="1H56 L1P"; $resn_exch{"H572 L1P"}="2H57 L1P";
$resn_exch{"H571 L1P"}="1H57 L1P"; $resn_exch{"H593 L1P"}="3H59 L1P"; $resn_exch{"H592 L1P"}="2H59 L1P";
$resn_exch{"H591 L1P"}="1H59 L1P"; $resn_exch{"H603 L1P"}="3H60 L1P"; $resn_exch{"H602 L1P"}="2H60 L1P";
$resn_exch{"H601 L1P"}="1H60 L1P"; $resn_exch{"H1C1 L1R"}="1H1C L1R"; $resn_exch{"H1C2 L1R"}="2H1C L1R";
$resn_exch{"H1C3 L1R"}="3H1C L1R"; $resn_exch{"H3C1 L1R"}="1H3C L1R"; $resn_exch{"H3C2 L1R"}="2H3C L1R";
$resn_exch{"H3C3 L1R"}="3H3C L1R"; $resn_exch{"H191 L1R"}="1H19 L1R"; $resn_exch{"H192 L1R"}="2H19 L1R";
$resn_exch{"H201 L1R"}="1H20 L1R"; $resn_exch{"H202 L1R"}="2H20 L1R"; $resn_exch{"H211 L1R"}="1H21 L1R";
$resn_exch{"H212 L1R"}="2H21 L1R"; $resn_exch{"H1C1 L20"}="1H1C L20"; $resn_exch{"H1C2 L20"}="2H1C L20";
$resn_exch{"H1C3 L20"}="3H1C L20"; $resn_exch{"H181 L20"}="1H18 L20"; $resn_exch{"H182 L20"}="2H18 L20";
$resn_exch{"H211 L20"}="1H21 L20"; $resn_exch{"H212 L20"}="2H21 L20"; $resn_exch{"H251 L20"}="1H25 L20";
$resn_exch{"H252 L20"}="2H25 L20"; $resn_exch{"HN21 L24"}="1HN2 L24"; $resn_exch{"HN22 L24"}="2HN2 L24";
$resn_exch{" H81 L24"}="1H8  L24"; $resn_exch{" H82 L24"}="2H8  L24"; $resn_exch{"H101 L24"}="1H10 L24";
$resn_exch{"H102 L24"}="2H10 L24"; $resn_exch{"H111 L24"}="1H11 L24"; $resn_exch{"H112 L24"}="2H11 L24";
$resn_exch{"H121 L24"}="1H12 L24"; $resn_exch{"H122 L24"}="2H12 L24"; $resn_exch{" HB1 L24"}="1HB  L24";
$resn_exch{" HB2 L24"}="2HB  L24"; $resn_exch{" HG1 L24"}="1HG  L24"; $resn_exch{" HG2 L24"}="2HG  L24";
$resn_exch{"H121 L2C"}="1H12 L2C"; $resn_exch{"H122 L2C"}="2H12 L2C"; $resn_exch{"H131 L2C"}="1H13 L2C";
$resn_exch{"H132 L2C"}="2H13 L2C"; $resn_exch{"H141 L2C"}="1H14 L2C"; $resn_exch{"H142 L2C"}="2H14 L2C";
$resn_exch{"H151 L2C"}="1H15 L2C"; $resn_exch{"H152 L2C"}="2H15 L2C"; $resn_exch{"H161 L2C"}="1H16 L2C";
$resn_exch{"H162 L2C"}="2H16 L2C"; $resn_exch{"H171 L2C"}="1H17 L2C"; $resn_exch{"H172 L2C"}="2H17 L2C";
$resn_exch{"H181 L2C"}="1H18 L2C"; $resn_exch{"H182 L2C"}="2H18 L2C"; $resn_exch{"H191 L2C"}="1H19 L2C";
$resn_exch{"H192 L2C"}="2H19 L2C"; $resn_exch{"H193 L2C"}="3H19 L2C"; $resn_exch{"H221 L2C"}="1H22 L2C";
$resn_exch{"H222 L2C"}="2H22 L2C"; $resn_exch{"H231 L2C"}="1H23 L2C"; $resn_exch{"H232 L2C"}="2H23 L2C";
$resn_exch{"H241 L2C"}="1H24 L2C"; $resn_exch{"H242 L2C"}="2H24 L2C"; $resn_exch{"H251 L2C"}="1H25 L2C";
$resn_exch{"H252 L2C"}="2H25 L2C"; $resn_exch{"H261 L2C"}="1H26 L2C"; $resn_exch{"H262 L2C"}="2H26 L2C";
$resn_exch{"H271 L2C"}="1H27 L2C"; $resn_exch{"H272 L2C"}="2H27 L2C"; $resn_exch{"H281 L2C"}="1H28 L2C";
$resn_exch{"H282 L2C"}="2H28 L2C"; $resn_exch{"H291 L2C"}="1H29 L2C"; $resn_exch{"H292 L2C"}="2H29 L2C";
$resn_exch{"H301 L2C"}="1H30 L2C"; $resn_exch{"H302 L2C"}="2H30 L2C"; $resn_exch{"H311 L2C"}="1H31 L2C";
$resn_exch{"H312 L2C"}="2H31 L2C"; $resn_exch{"H321 L2C"}="1H32 L2C"; $resn_exch{"H322 L2C"}="2H32 L2C";
$resn_exch{"H323 L2C"}="3H32 L2C"; $resn_exch{"H411 L2C"}="1H41 L2C"; $resn_exch{"H412 L2C"}="2H41 L2C";
$resn_exch{"H431 L2C"}="1H43 L2C"; $resn_exch{"H432 L2C"}="2H43 L2C"; $resn_exch{"H5N1 L2G"}="1H5N L2G";
$resn_exch{"H5N2 L2G"}="2H5N L2G"; $resn_exch{"H131 L2G"}="1H13 L2G"; $resn_exch{"H132 L2G"}="2H13 L2G";
$resn_exch{"H141 L2G"}="1H14 L2G"; $resn_exch{"H142 L2G"}="2H14 L2G"; $resn_exch{"H151 L2G"}="1H15 L2G";
$resn_exch{"H152 L2G"}="2H15 L2G"; $resn_exch{"H153 L2G"}="3H15 L2G"; $resn_exch{"H161 L2G"}="1H16 L2G";
$resn_exch{"H162 L2G"}="2H16 L2G"; $resn_exch{"H171 L2G"}="1H17 L2G"; $resn_exch{"H172 L2G"}="2H17 L2G";
$resn_exch{"H221 L2G"}="1H22 L2G"; $resn_exch{"H222 L2G"}="2H22 L2G"; $resn_exch{"H231 L2G"}="1H23 L2G";
$resn_exch{"H232 L2G"}="2H23 L2G"; $resn_exch{"H251 L2G"}="1H25 L2G"; $resn_exch{"H252 L2G"}="2H25 L2G";
$resn_exch{"H253 L2G"}="3H25 L2G"; $resn_exch{"H271 L2G"}="1H27 L2G"; $resn_exch{"H272 L2G"}="2H27 L2G";
$resn_exch{"H281 L2G"}="1H28 L2G"; $resn_exch{"H282 L2G"}="2H28 L2G"; $resn_exch{"H341 L2G"}="1H34 L2G";
$resn_exch{"H342 L2G"}="2H34 L2G"; $resn_exch{"H343 L2G"}="3H34 L2G"; $resn_exch{" H12 L2P"}="2H1  L2P";
$resn_exch{" H11 L2P"}="1H1  L2P"; $resn_exch{" H32 L2P"}="2H3  L2P"; $resn_exch{" H31 L2P"}="1H3  L2P";
$resn_exch{"H112 L2P"}="2H11 L2P"; $resn_exch{"H111 L2P"}="1H11 L2P"; $resn_exch{"H122 L2P"}="2H12 L2P";
$resn_exch{"H121 L2P"}="1H12 L2P"; $resn_exch{"H143 L2P"}="3H14 L2P"; $resn_exch{"H142 L2P"}="2H14 L2P";
$resn_exch{"H141 L2P"}="1H14 L2P"; $resn_exch{"H152 L2P"}="2H15 L2P"; $resn_exch{"H151 L2P"}="1H15 L2P";
$resn_exch{"H162 L2P"}="2H16 L2P"; $resn_exch{"H161 L2P"}="1H16 L2P"; $resn_exch{"H172 L2P"}="2H17 L2P";
$resn_exch{"H171 L2P"}="1H17 L2P"; $resn_exch{"H193 L2P"}="3H19 L2P"; $resn_exch{"H192 L2P"}="2H19 L2P";
$resn_exch{"H191 L2P"}="1H19 L2P"; $resn_exch{"H202 L2P"}="2H20 L2P"; $resn_exch{"H201 L2P"}="1H20 L2P";
$resn_exch{"H212 L2P"}="2H21 L2P"; $resn_exch{"H211 L2P"}="1H21 L2P"; $resn_exch{"H222 L2P"}="2H22 L2P";
$resn_exch{"H221 L2P"}="1H22 L2P"; $resn_exch{"H243 L2P"}="3H24 L2P"; $resn_exch{"H242 L2P"}="2H24 L2P";
$resn_exch{"H241 L2P"}="1H24 L2P"; $resn_exch{"H252 L2P"}="2H25 L2P"; $resn_exch{"H251 L2P"}="1H25 L2P";
$resn_exch{"H262 L2P"}="2H26 L2P"; $resn_exch{"H261 L2P"}="1H26 L2P"; $resn_exch{"H272 L2P"}="2H27 L2P";
$resn_exch{"H271 L2P"}="1H27 L2P"; $resn_exch{"H293 L2P"}="3H29 L2P"; $resn_exch{"H292 L2P"}="2H29 L2P";
$resn_exch{"H291 L2P"}="1H29 L2P"; $resn_exch{"H303 L2P"}="3H30 L2P"; $resn_exch{"H302 L2P"}="2H30 L2P";
$resn_exch{"H301 L2P"}="1H30 L2P"; $resn_exch{"H412 L2P"}="2H41 L2P"; $resn_exch{"H411 L2P"}="1H41 L2P";
$resn_exch{"H422 L2P"}="2H42 L2P"; $resn_exch{"H421 L2P"}="1H42 L2P"; $resn_exch{"H443 L2P"}="3H44 L2P";
$resn_exch{"H442 L2P"}="2H44 L2P"; $resn_exch{"H441 L2P"}="1H44 L2P"; $resn_exch{"H452 L2P"}="2H45 L2P";
$resn_exch{"H451 L2P"}="1H45 L2P"; $resn_exch{"H462 L2P"}="2H46 L2P"; $resn_exch{"H461 L2P"}="1H46 L2P";
$resn_exch{"H472 L2P"}="2H47 L2P"; $resn_exch{"H471 L2P"}="1H47 L2P"; $resn_exch{"H493 L2P"}="3H49 L2P";
$resn_exch{"H492 L2P"}="2H49 L2P"; $resn_exch{"H491 L2P"}="1H49 L2P"; $resn_exch{"H502 L2P"}="2H50 L2P";
$resn_exch{"H501 L2P"}="1H50 L2P"; $resn_exch{"H512 L2P"}="2H51 L2P"; $resn_exch{"H511 L2P"}="1H51 L2P";
$resn_exch{"H522 L2P"}="2H52 L2P"; $resn_exch{"H521 L2P"}="1H52 L2P"; $resn_exch{"H543 L2P"}="3H54 L2P";
$resn_exch{"H542 L2P"}="2H54 L2P"; $resn_exch{"H541 L2P"}="1H54 L2P"; $resn_exch{"H552 L2P"}="2H55 L2P";
$resn_exch{"H551 L2P"}="1H55 L2P"; $resn_exch{"H562 L2P"}="2H56 L2P"; $resn_exch{"H561 L2P"}="1H56 L2P";
$resn_exch{"H572 L2P"}="2H57 L2P"; $resn_exch{"H571 L2P"}="1H57 L2P"; $resn_exch{"H593 L2P"}="3H59 L2P";
$resn_exch{"H592 L2P"}="2H59 L2P"; $resn_exch{"H591 L2P"}="1H59 L2P"; $resn_exch{"H603 L2P"}="3H60 L2P";
$resn_exch{"H602 L2P"}="2H60 L2P"; $resn_exch{"H601 L2P"}="1H60 L2P"; $resn_exch{"HN21 L34"}="1HN2 L34";
$resn_exch{"HN22 L34"}="2HN2 L34"; $resn_exch{" H71 L34"}="1H7  L34"; $resn_exch{" H72 L34"}="2H7  L34";
$resn_exch{" H91 L34"}="1H9  L34"; $resn_exch{" H92 L34"}="2H9  L34"; $resn_exch{" HB1 L34"}="1HB  L34";
$resn_exch{" HB2 L34"}="2HB  L34"; $resn_exch{" HG1 L34"}="1HG  L34"; $resn_exch{" HG2 L34"}="2HG  L34";
$resn_exch{"H151 L35"}="1H15 L35"; $resn_exch{"H152 L35"}="2H15 L35"; $resn_exch{"H153 L35"}="3H15 L35";
$resn_exch{"H161 L35"}="1H16 L35"; $resn_exch{"H162 L35"}="2H16 L35"; $resn_exch{"H163 L35"}="3H16 L35";
$resn_exch{"HN21 L37"}="1HN2 L37"; $resn_exch{"HN22 L37"}="2HN2 L37"; $resn_exch{"HN51 L37"}="1HN5 L37";
$resn_exch{"HN52 L37"}="2HN5 L37"; $resn_exch{" HB1 L37"}="1HB  L37"; $resn_exch{" HB2 L37"}="2HB  L37";
$resn_exch{" HG1 L37"}="1HG  L37"; $resn_exch{" HG2 L37"}="2HG  L37"; $resn_exch{"H5N1 L3G"}="1H5N L3G";
$resn_exch{"H5N2 L3G"}="2H5N L3G"; $resn_exch{"H131 L3G"}="1H13 L3G"; $resn_exch{"H132 L3G"}="2H13 L3G";
$resn_exch{"H141 L3G"}="1H14 L3G"; $resn_exch{"H142 L3G"}="2H14 L3G"; $resn_exch{"H151 L3G"}="1H15 L3G";
$resn_exch{"H152 L3G"}="2H15 L3G"; $resn_exch{"H153 L3G"}="3H15 L3G"; $resn_exch{"H161 L3G"}="1H16 L3G";
$resn_exch{"H162 L3G"}="2H16 L3G"; $resn_exch{"H171 L3G"}="1H17 L3G"; $resn_exch{"H172 L3G"}="2H17 L3G";
$resn_exch{"H221 L3G"}="1H22 L3G"; $resn_exch{"H222 L3G"}="2H22 L3G"; $resn_exch{"H231 L3G"}="1H23 L3G";
$resn_exch{"H232 L3G"}="2H23 L3G"; $resn_exch{"H251 L3G"}="1H25 L3G"; $resn_exch{"H252 L3G"}="2H25 L3G";
$resn_exch{"H253 L3G"}="3H25 L3G"; $resn_exch{"H271 L3G"}="1H27 L3G"; $resn_exch{"H272 L3G"}="2H27 L3G";
$resn_exch{"H281 L3G"}="1H28 L3G"; $resn_exch{"H282 L3G"}="2H28 L3G"; $resn_exch{" H12 L3P"}="2H1  L3P";
$resn_exch{" H11 L3P"}="1H1  L3P"; $resn_exch{" H32 L3P"}="2H3  L3P"; $resn_exch{" H31 L3P"}="1H3  L3P";
$resn_exch{" H42 L3P"}="2H4  L3P"; $resn_exch{" H41 L3P"}="1H4  L3P"; $resn_exch{" H62 L3P"}="2H6  L3P";
$resn_exch{" H61 L3P"}="1H6  L3P"; $resn_exch{"HOP6 L3P"}="6HOP L3P"; $resn_exch{"H112 L3P"}="2H11 L3P";
$resn_exch{"H111 L3P"}="1H11 L3P"; $resn_exch{"H122 L3P"}="2H12 L3P"; $resn_exch{"H121 L3P"}="1H12 L3P";
$resn_exch{"H143 L3P"}="3H14 L3P"; $resn_exch{"H142 L3P"}="2H14 L3P"; $resn_exch{"H141 L3P"}="1H14 L3P";
$resn_exch{"H152 L3P"}="2H15 L3P"; $resn_exch{"H151 L3P"}="1H15 L3P"; $resn_exch{"H162 L3P"}="2H16 L3P";
$resn_exch{"H161 L3P"}="1H16 L3P"; $resn_exch{"H172 L3P"}="2H17 L3P"; $resn_exch{"H171 L3P"}="1H17 L3P";
$resn_exch{"H193 L3P"}="3H19 L3P"; $resn_exch{"H192 L3P"}="2H19 L3P"; $resn_exch{"H191 L3P"}="1H19 L3P";
$resn_exch{"H202 L3P"}="2H20 L3P"; $resn_exch{"H201 L3P"}="1H20 L3P"; $resn_exch{"H212 L3P"}="2H21 L3P";
$resn_exch{"H211 L3P"}="1H21 L3P"; $resn_exch{"H222 L3P"}="2H22 L3P"; $resn_exch{"H221 L3P"}="1H22 L3P";
$resn_exch{"H243 L3P"}="3H24 L3P"; $resn_exch{"H242 L3P"}="2H24 L3P"; $resn_exch{"H241 L3P"}="1H24 L3P";
$resn_exch{"H252 L3P"}="2H25 L3P"; $resn_exch{"H251 L3P"}="1H25 L3P"; $resn_exch{"H262 L3P"}="2H26 L3P";
$resn_exch{"H261 L3P"}="1H26 L3P"; $resn_exch{"H272 L3P"}="2H27 L3P"; $resn_exch{"H271 L3P"}="1H27 L3P";
$resn_exch{"H293 L3P"}="3H29 L3P"; $resn_exch{"H292 L3P"}="2H29 L3P"; $resn_exch{"H291 L3P"}="1H29 L3P";
$resn_exch{"H303 L3P"}="3H30 L3P"; $resn_exch{"H302 L3P"}="2H30 L3P"; $resn_exch{"H301 L3P"}="1H30 L3P";
$resn_exch{"H412 L3P"}="2H41 L3P"; $resn_exch{"H411 L3P"}="1H41 L3P"; $resn_exch{"H422 L3P"}="2H42 L3P";
$resn_exch{"H421 L3P"}="1H42 L3P"; $resn_exch{"H443 L3P"}="3H44 L3P"; $resn_exch{"H442 L3P"}="2H44 L3P";
$resn_exch{"H441 L3P"}="1H44 L3P"; $resn_exch{"H452 L3P"}="2H45 L3P"; $resn_exch{"H451 L3P"}="1H45 L3P";
$resn_exch{"H462 L3P"}="2H46 L3P"; $resn_exch{"H461 L3P"}="1H46 L3P"; $resn_exch{"H472 L3P"}="2H47 L3P";
$resn_exch{"H471 L3P"}="1H47 L3P"; $resn_exch{"H493 L3P"}="3H49 L3P"; $resn_exch{"H492 L3P"}="2H49 L3P";
$resn_exch{"H491 L3P"}="1H49 L3P"; $resn_exch{"H502 L3P"}="2H50 L3P"; $resn_exch{"H501 L3P"}="1H50 L3P";
$resn_exch{"H512 L3P"}="2H51 L3P"; $resn_exch{"H511 L3P"}="1H51 L3P"; $resn_exch{"H522 L3P"}="2H52 L3P";
$resn_exch{"H521 L3P"}="1H52 L3P"; $resn_exch{"H543 L3P"}="3H54 L3P"; $resn_exch{"H542 L3P"}="2H54 L3P";
$resn_exch{"H541 L3P"}="1H54 L3P"; $resn_exch{"H552 L3P"}="2H55 L3P"; $resn_exch{"H551 L3P"}="1H55 L3P";
$resn_exch{"H562 L3P"}="2H56 L3P"; $resn_exch{"H561 L3P"}="1H56 L3P"; $resn_exch{"H572 L3P"}="2H57 L3P";
$resn_exch{"H571 L3P"}="1H57 L3P"; $resn_exch{"H593 L3P"}="3H59 L3P"; $resn_exch{"H592 L3P"}="2H59 L3P";
$resn_exch{"H591 L3P"}="1H59 L3P"; $resn_exch{"H603 L3P"}="3H60 L3P"; $resn_exch{"H602 L3P"}="2H60 L3P";
$resn_exch{"H601 L3P"}="1H60 L3P"; $resn_exch{"H151 L47"}="1H15 L47"; $resn_exch{"H152 L47"}="2H15 L47";
$resn_exch{"H211 L47"}="1H21 L47"; $resn_exch{"H212 L47"}="2H21 L47"; $resn_exch{"H213 L47"}="3H21 L47";
$resn_exch{"H101 L4C"}="1H10 L4C"; $resn_exch{"H102 L4C"}="2H10 L4C"; $resn_exch{"H111 L4C"}="1H11 L4C";
$resn_exch{"H112 L4C"}="2H11 L4C"; $resn_exch{"H5C1 L4C"}="1H5C L4C"; $resn_exch{"H5C2 L4C"}="2H5C L4C";
$resn_exch{"H7C1 L4C"}="1H7C L4C"; $resn_exch{"H7C2 L4C"}="2H7C L4C"; $resn_exch{"H7C3 L4C"}="3H7C L4C";
$resn_exch{"H271 L4G"}="1H27 L4G"; $resn_exch{"H272 L4G"}="2H27 L4G"; $resn_exch{"H273 L4G"}="3H27 L4G";
$resn_exch{"H301 L4G"}="1H30 L4G"; $resn_exch{"H302 L4G"}="2H30 L4G"; $resn_exch{"H311 L4G"}="1H31 L4G";
$resn_exch{"H312 L4G"}="2H31 L4G"; $resn_exch{"H331 L4G"}="1H33 L4G"; $resn_exch{"H332 L4G"}="2H33 L4G";
$resn_exch{"H341 L4G"}="1H34 L4G"; $resn_exch{"H342 L4G"}="2H34 L4G"; $resn_exch{"H351 L4G"}="1H35 L4G";
$resn_exch{"H352 L4G"}="2H35 L4G"; $resn_exch{"H361 L4G"}="1H36 L4G"; $resn_exch{"H362 L4G"}="2H36 L4G";
$resn_exch{"H371 L4G"}="1H37 L4G"; $resn_exch{"H372 L4G"}="2H37 L4G"; $resn_exch{" H12 L4P"}="2H1  L4P";
$resn_exch{" H11 L4P"}="1H1  L4P"; $resn_exch{" H32 L4P"}="2H3  L4P"; $resn_exch{" H31 L4P"}="1H3  L4P";
$resn_exch{" H42 L4P"}="2H4  L4P"; $resn_exch{" H41 L4P"}="1H4  L4P"; $resn_exch{" H62 L4P"}="2H6  L4P";
$resn_exch{" H61 L4P"}="1H6  L4P"; $resn_exch{"HOP2 L4P"}="2HOP L4P"; $resn_exch{"H112 L4P"}="2H11 L4P";
$resn_exch{"H111 L4P"}="1H11 L4P"; $resn_exch{"H122 L4P"}="2H12 L4P"; $resn_exch{"H121 L4P"}="1H12 L4P";
$resn_exch{"H143 L4P"}="3H14 L4P"; $resn_exch{"H142 L4P"}="2H14 L4P"; $resn_exch{"H141 L4P"}="1H14 L4P";
$resn_exch{"H152 L4P"}="2H15 L4P"; $resn_exch{"H151 L4P"}="1H15 L4P"; $resn_exch{"H162 L4P"}="2H16 L4P";
$resn_exch{"H161 L4P"}="1H16 L4P"; $resn_exch{"H172 L4P"}="2H17 L4P"; $resn_exch{"H171 L4P"}="1H17 L4P";
$resn_exch{"H193 L4P"}="3H19 L4P"; $resn_exch{"H192 L4P"}="2H19 L4P"; $resn_exch{"H191 L4P"}="1H19 L4P";
$resn_exch{"H202 L4P"}="2H20 L4P"; $resn_exch{"H201 L4P"}="1H20 L4P"; $resn_exch{"H212 L4P"}="2H21 L4P";
$resn_exch{"H211 L4P"}="1H21 L4P"; $resn_exch{"H222 L4P"}="2H22 L4P"; $resn_exch{"H221 L4P"}="1H22 L4P";
$resn_exch{"H243 L4P"}="3H24 L4P"; $resn_exch{"H242 L4P"}="2H24 L4P"; $resn_exch{"H241 L4P"}="1H24 L4P";
$resn_exch{"H252 L4P"}="2H25 L4P"; $resn_exch{"H251 L4P"}="1H25 L4P"; $resn_exch{"H262 L4P"}="2H26 L4P";
$resn_exch{"H261 L4P"}="1H26 L4P"; $resn_exch{"H272 L4P"}="2H27 L4P"; $resn_exch{"H271 L4P"}="1H27 L4P";
$resn_exch{"H293 L4P"}="3H29 L4P"; $resn_exch{"H292 L4P"}="2H29 L4P"; $resn_exch{"H291 L4P"}="1H29 L4P";
$resn_exch{"H303 L4P"}="3H30 L4P"; $resn_exch{"H302 L4P"}="2H30 L4P"; $resn_exch{"H301 L4P"}="1H30 L4P";
$resn_exch{"H412 L4P"}="2H41 L4P"; $resn_exch{"H411 L4P"}="1H41 L4P"; $resn_exch{"H422 L4P"}="2H42 L4P";
$resn_exch{"H421 L4P"}="1H42 L4P"; $resn_exch{"H443 L4P"}="3H44 L4P"; $resn_exch{"H442 L4P"}="2H44 L4P";
$resn_exch{"H441 L4P"}="1H44 L4P"; $resn_exch{"H452 L4P"}="2H45 L4P"; $resn_exch{"H451 L4P"}="1H45 L4P";
$resn_exch{"H462 L4P"}="2H46 L4P"; $resn_exch{"H461 L4P"}="1H46 L4P"; $resn_exch{"H472 L4P"}="2H47 L4P";
$resn_exch{"H471 L4P"}="1H47 L4P"; $resn_exch{"H493 L4P"}="3H49 L4P"; $resn_exch{"H492 L4P"}="2H49 L4P";
$resn_exch{"H491 L4P"}="1H49 L4P"; $resn_exch{"H502 L4P"}="2H50 L4P"; $resn_exch{"H501 L4P"}="1H50 L4P";
$resn_exch{"H512 L4P"}="2H51 L4P"; $resn_exch{"H511 L4P"}="1H51 L4P"; $resn_exch{"H522 L4P"}="2H52 L4P";
$resn_exch{"H521 L4P"}="1H52 L4P"; $resn_exch{"H543 L4P"}="3H54 L4P"; $resn_exch{"H542 L4P"}="2H54 L4P";
$resn_exch{"H541 L4P"}="1H54 L4P"; $resn_exch{"H552 L4P"}="2H55 L4P"; $resn_exch{"H551 L4P"}="1H55 L4P";
$resn_exch{"H562 L4P"}="2H56 L4P"; $resn_exch{"H561 L4P"}="1H56 L4P"; $resn_exch{"H572 L4P"}="2H57 L4P";
$resn_exch{"H571 L4P"}="1H57 L4P"; $resn_exch{"H593 L4P"}="3H59 L4P"; $resn_exch{"H592 L4P"}="2H59 L4P";
$resn_exch{"H591 L4P"}="1H59 L4P"; $resn_exch{"H603 L4P"}="3H60 L4P"; $resn_exch{"H602 L4P"}="2H60 L4P";
$resn_exch{"H601 L4P"}="1H60 L4P"; $resn_exch{" H11 L75"}="1H1  L75"; $resn_exch{" H12 L75"}="2H1  L75";
$resn_exch{" H21 L75"}="1H2  L75"; $resn_exch{" H51 L75"}="1H5  L75"; $resn_exch{" H52 L75"}="2H5  L75";
$resn_exch{" H53 L75"}="3H5  L75"; $resn_exch{" H61 L75"}="1H6  L75"; $resn_exch{" H62 L75"}="2H6  L75";
$resn_exch{" H63 L75"}="3H6  L75"; $resn_exch{" H71 L75"}="1H7  L75"; $resn_exch{" H72 L75"}="2H7  L75";
$resn_exch{" H73 L75"}="3H7  L75"; $resn_exch{" H81 L75"}="1H8  L75"; $resn_exch{" H82 L75"}="2H8  L75";
$resn_exch{" H91 L75"}="1H9  L75"; $resn_exch{" H92 L75"}="2H9  L75"; $resn_exch{"H101 L75"}="1H10 L75";
$resn_exch{"H102 L75"}="2H10 L75"; $resn_exch{"H111 L75"}="1H11 L75"; $resn_exch{"H121 L75"}="1H12 L75";
$resn_exch{"H122 L75"}="2H12 L75"; $resn_exch{"H131 L75"}="1H13 L75"; $resn_exch{"H141 L75"}="1H14 L75";
$resn_exch{"H142 L75"}="2H14 L75"; $resn_exch{"H161 L75"}="1H16 L75"; $resn_exch{"H171 L75"}="1H17 L75";
$resn_exch{"H181 L75"}="1H18 L75"; $resn_exch{"H191 L75"}="1H19 L75"; $resn_exch{"H201 L75"}="1H20 L75";
$resn_exch{"H221 L75"}="1H22 L75"; $resn_exch{"H231 L75"}="1H23 L75"; $resn_exch{"H241 L75"}="1H24 L75";
$resn_exch{"H242 L75"}="2H24 L75"; $resn_exch{"H261 L75"}="1H26 L75"; $resn_exch{"H271 L75"}="1H27 L75";
$resn_exch{"H281 L75"}="1H28 L75"; $resn_exch{"H291 L75"}="1H29 L75"; $resn_exch{"H311 L75"}="1H31 L75";
$resn_exch{"H312 L75"}="2H31 L75"; $resn_exch{"H321 L75"}="1H32 L75"; $resn_exch{"H331 L75"}="1H33 L75";
$resn_exch{"H361 L75"}="1H36 L75"; $resn_exch{"H371 L75"}="1H37 L75"; $resn_exch{"H391 L75"}="1H39 L75";
$resn_exch{" H1A L79"}="1H   L79"; $resn_exch{" H2A L79"}="2H   L79"; $resn_exch{"H161 L79"}="1H16 L79";
$resn_exch{"H162 L79"}="2H16 L79"; $resn_exch{"H171 L79"}="1H17 L79"; $resn_exch{"H172 L79"}="2H17 L79";
$resn_exch{"H173 L79"}="3H17 L79"; $resn_exch{"H181 L79"}="1H18 L79"; $resn_exch{"H182 L79"}="2H18 L79";
$resn_exch{"H183 L79"}="3H18 L79"; $resn_exch{"H211 L79"}="1H21 L79"; $resn_exch{"H212 L79"}="2H21 L79";
$resn_exch{"H213 L79"}="3H21 L79"; $resn_exch{"H221 L79"}="1H22 L79"; $resn_exch{"H222 L79"}="2H22 L79";
$resn_exch{"H231 L79"}="1H23 L79"; $resn_exch{"H232 L79"}="2H23 L79"; $resn_exch{"H233 L79"}="3H23 L79";
$resn_exch{"H111 L86"}="1H11 L86"; $resn_exch{"H112 L86"}="2H11 L86"; $resn_exch{"H331 L86"}="1H33 L86";
$resn_exch{"H332 L86"}="2H33 L86"; $resn_exch{"H391 L86"}="1H39 L86"; $resn_exch{"H392 L86"}="2H39 L86";
$resn_exch{" H91 L86"}="1H9  L86"; $resn_exch{" H92 L86"}="2H9  L86"; $resn_exch{" H31 L86"}="1H3  L86";
$resn_exch{" H32 L86"}="2H3  L86"; $resn_exch{" H71 L86"}="1H7  L86"; $resn_exch{" H72 L86"}="2H7  L86";
$resn_exch{"H121 L86"}="1H12 L86"; $resn_exch{"H122 L86"}="2H12 L86"; $resn_exch{"H131 L86"}="1H13 L86";
$resn_exch{"H132 L86"}="2H13 L86"; $resn_exch{"H141 L86"}="1H14 L86"; $resn_exch{"H142 L86"}="2H14 L86";
$resn_exch{"H11A L98"}="1H1  L98"; $resn_exch{" H12 L98"}="2H1  L98"; $resn_exch{"H121 L98"}="1H12 L98";
$resn_exch{"H122 L98"}="2H12 L98"; $resn_exch{"H731 LA1"}="1H73 LA1"; $resn_exch{"H732 LA1"}="2H73 LA1";
$resn_exch{"H733 LA1"}="3H73 LA1"; $resn_exch{"H491 LA1"}="1H49 LA1"; $resn_exch{"H492 LA1"}="2H49 LA1";
$resn_exch{" H31 LA1"}="1H3  LA1"; $resn_exch{" H32 LA1"}="2H3  LA1"; $resn_exch{"H61A LA1"}="1H6  LA1";
$resn_exch{" H62 LA1"}="2H6  LA1"; $resn_exch{"H141 LA1"}="1H14 LA1"; $resn_exch{"H142 LA1"}="2H14 LA1";
$resn_exch{"H231 LA1"}="1H23 LA1"; $resn_exch{"H232 LA1"}="2H23 LA1"; $resn_exch{"H233 LA1"}="3H23 LA1";
$resn_exch{"H191 LA1"}="1H19 LA1"; $resn_exch{"H192 LA1"}="2H19 LA1"; $resn_exch{"H193 LA1"}="3H19 LA1";
$resn_exch{"H271 LA1"}="1H27 LA1"; $resn_exch{"H272 LA1"}="2H27 LA1"; $resn_exch{"H171 LAB"}="1H17 LAB";
$resn_exch{"H172 LAB"}="2H17 LAB"; $resn_exch{"H141 LAB"}="1H14 LAB"; $resn_exch{"H142 LAB"}="2H14 LAB";
$resn_exch{"H121 LAB"}="1H12 LAB"; $resn_exch{"H122 LAB"}="2H12 LAB"; $resn_exch{"H101 LAB"}="1H10 LAB";
$resn_exch{"H102 LAB"}="2H10 LAB"; $resn_exch{" H91 LAB"}="1H9  LAB"; $resn_exch{" H92 LAB"}="2H9  LAB";
$resn_exch{"H201 LAB"}="1H20 LAB"; $resn_exch{"H202 LAB"}="2H20 LAB"; $resn_exch{"H203 LAB"}="3H20 LAB";
$resn_exch{" H51 LAB"}="1H5  LAB"; $resn_exch{" H52 LAB"}="2H5  LAB"; $resn_exch{" H41 LAB"}="1H4  LAB";
$resn_exch{" H42 LAB"}="2H4  LAB"; $resn_exch{"H191 LAB"}="1H19 LAB"; $resn_exch{"H192 LAB"}="2H19 LAB";
$resn_exch{"H193 LAB"}="3H19 LAB"; $resn_exch{" H31 LAC"}="1H3  LAC"; $resn_exch{" H32 LAC"}="2H3  LAC";
$resn_exch{" H33 LAC"}="3H3  LAC"; $resn_exch{" H2A LAD"}="2H   LAD"; $resn_exch{" H1  LAD"}="1H   LAD";
$resn_exch{" HB2 LAD"}="2HB  LAD"; $resn_exch{" HB1 LAD"}="1HB  LAD"; $resn_exch{" HG2 LAD"}="2HG  LAD";
$resn_exch{" HG1 LAD"}="1HG  LAD"; $resn_exch{" HD2 LAD"}="2HD  LAD"; $resn_exch{" HD1 LAD"}="1HD  LAD";
$resn_exch{" HE2 LAD"}="2HE  LAD"; $resn_exch{" HE1 LAD"}="1HE  LAD"; $resn_exch{" HZ2 LAD"}="2HZ  LAD";
$resn_exch{" HZ1 LAD"}="1HZ  LAD"; $resn_exch{"H5'2 LAD"}="2H5' LAD"; $resn_exch{"H5'1 LAD"}="1H5' LAD";
$resn_exch{" H62 LAD"}="2H6  LAD"; $resn_exch{" H61 LAD"}="1H6  LAD"; $resn_exch{"H2O1 LAD"}="1H2O LAD";
$resn_exch{"H3O1 LAD"}="1H3O LAD"; $resn_exch{"H2AO LAD"}="OH2A LAD"; $resn_exch{" H11 LAE"}="1H1  LAE";
$resn_exch{" H41 LAE"}="1H4  LAE"; $resn_exch{" H42 LAE"}="2H4  LAE"; $resn_exch{" H51 LAE"}="1H5  LAE";
$resn_exch{" H52 LAE"}="2H5  LAE"; $resn_exch{"H111 LAE"}="1H11 LAE"; $resn_exch{"H141 LAE"}="1H14 LAE";
$resn_exch{"H142 LAE"}="2H14 LAE"; $resn_exch{"H181 LAE"}="1H18 LAE"; $resn_exch{"H182 LAE"}="2H18 LAE";
$resn_exch{"H191 LAE"}="1H19 LAE"; $resn_exch{"H192 LAE"}="2H19 LAE"; $resn_exch{"H221 LAE"}="1H22 LAE";
$resn_exch{"H222 LAE"}="2H22 LAE"; $resn_exch{"H251 LAE"}="1H25 LAE"; $resn_exch{"H252 LAE"}="2H25 LAE";
$resn_exch{"H281 LAE"}="1H28 LAE"; $resn_exch{"H282 LAE"}="2H28 LAE"; $resn_exch{"H283 LAE"}="3H28 LAE";
$resn_exch{"H5C1 LAF"}="1H5C LAF"; $resn_exch{"H5C2 LAF"}="2H5C LAF"; $resn_exch{"H3C1 LAF"}="1H3C LAF";
$resn_exch{"H3C2 LAF"}="2H3C LAF"; $resn_exch{"H2C1 LAF"}="1H2C LAF"; $resn_exch{"H2C2 LAF"}="2H2C LAF";
$resn_exch{"HA61 LAG"}="1HA6 LAG"; $resn_exch{"HA62 LAG"}="2HA6 LAG"; $resn_exch{"HB61 LAG"}="1HB6 LAG";
$resn_exch{"HB62 LAG"}="2HB6 LAG"; $resn_exch{"HD61 LAG"}="1HD6 LAG"; $resn_exch{"HD62 LAG"}="2HD6 LAG";
$resn_exch{" H61 LAK"}="1H6  LAK"; $resn_exch{" H62 LAK"}="2H6  LAK"; $resn_exch{"H6'1 LAK"}="1H6' LAK";
$resn_exch{"H6'2 LAK"}="2H6' LAK"; $resn_exch{"HO1' LAK"}="'HO1 LAK"; $resn_exch{"HO2' LAK"}="'HO2 LAK";
$resn_exch{"HO3' LAK"}="'HO3 LAK"; $resn_exch{"HO4' LAK"}="'HO4 LAK"; $resn_exch{" HB1 LAL"}="1HB  LAL";
$resn_exch{" HB2 LAL"}="2HB  LAL"; $resn_exch{" HB3 LAL"}="3HB  LAL"; $resn_exch{"HH11 LAL"}="1HH1 LAL";
$resn_exch{"HH12 LAL"}="2HH1 LAL"; $resn_exch{"HH13 LAL"}="3HH1 LAL"; $resn_exch{"HH21 LAL"}="1HH2 LAL";
$resn_exch{"HH22 LAL"}="2HH2 LAL"; $resn_exch{"HH23 LAL"}="3HH2 LAL"; $resn_exch{" H11 LAM"}="1H1  LAM";
$resn_exch{" H21 LAM"}="1H2  LAM"; $resn_exch{"HO21 LAM"}="1HO2 LAM"; $resn_exch{" H31 LAM"}="1H3  LAM";
$resn_exch{"HO31 LAM"}="1HO3 LAM"; $resn_exch{" H41 LAM"}="1H4  LAM"; $resn_exch{"HO41 LAM"}="1HO4 LAM";
$resn_exch{" H51 LAM"}="1H5  LAM"; $resn_exch{" H61 LAM"}="1H6  LAM"; $resn_exch{" H62 LAM"}="2H6  LAM";
$resn_exch{"HO61 LAM"}="1HO6 LAM"; $resn_exch{"H1A1 LAM"}="1H1A LAM"; $resn_exch{"H2A1 LAM"}="1H2A LAM";
$resn_exch{"H3A1 LAM"}="1H3A LAM"; $resn_exch{"H4A1 LAM"}="1H4A LAM"; $resn_exch{"H5A1 LAM"}="1H5A LAM";
$resn_exch{"H6A1 LAM"}="1H6A LAM"; $resn_exch{"H6A2 LAM"}="2H6A LAM"; $resn_exch{"HA21 LAM"}="1HA2 LAM";
$resn_exch{"HA41 LAM"}="1HA4 LAM"; $resn_exch{"HA61 LAM"}="1HA6 LAM"; $resn_exch{"H1B1 LAM"}="1H1B LAM";
$resn_exch{"H2B1 LAM"}="1H2B LAM"; $resn_exch{"H3B1 LAM"}="1H3B LAM"; $resn_exch{"H4B1 LAM"}="1H4B LAM";
$resn_exch{"H5B1 LAM"}="1H5B LAM"; $resn_exch{"H6B1 LAM"}="1H6B LAM"; $resn_exch{"H6B2 LAM"}="2H6B LAM";
$resn_exch{"HB21 LAM"}="1HB2 LAM"; $resn_exch{"HB41 LAM"}="1HB4 LAM"; $resn_exch{"HB61 LAM"}="1HB6 LAM";
$resn_exch{"H2'1 LAM"}="1H2' LAM"; $resn_exch{"H3'1 LAM"}="1H3' LAM"; $resn_exch{"H5'1 LAM"}="1H5' LAM";
$resn_exch{"H6'1 LAM"}="1H6' LAM"; $resn_exch{"H5C1 LAN"}="1H5C LAN"; $resn_exch{"H5C2 LAN"}="2H5C LAN";
$resn_exch{"H4C1 LAN"}="1H4C LAN"; $resn_exch{"H4C2 LAN"}="2H4C LAN"; $resn_exch{"H261 LAN"}="1H26 LAN";
$resn_exch{"H262 LAN"}="2H26 LAN"; $resn_exch{"H263 LAN"}="3H26 LAN"; $resn_exch{"H271 LAN"}="1H27 LAN";
$resn_exch{"H272 LAN"}="2H27 LAN"; $resn_exch{"H273 LAN"}="3H27 LAN"; $resn_exch{"H301 LAN"}="1H30 LAN";
$resn_exch{"H302 LAN"}="2H30 LAN"; $resn_exch{"H311 LAN"}="1H31 LAN"; $resn_exch{"H312 LAN"}="2H31 LAN";
$resn_exch{"H2C1 LAN"}="1H2C LAN"; $resn_exch{"H2C2 LAN"}="2H2C LAN"; $resn_exch{"H2C3 LAN"}="3H2C LAN";
$resn_exch{"H231 LAN"}="1H23 LAN"; $resn_exch{"H232 LAN"}="2H23 LAN"; $resn_exch{"H221 LAN"}="1H22 LAN";
$resn_exch{"H222 LAN"}="2H22 LAN"; $resn_exch{"H111 LAN"}="1H11 LAN"; $resn_exch{"H112 LAN"}="2H11 LAN";
$resn_exch{"H113 LAN"}="3H11 LAN"; $resn_exch{"H8C1 LAN"}="1H8C LAN"; $resn_exch{"H8C2 LAN"}="2H8C LAN";
$resn_exch{"H8C3 LAN"}="3H8C LAN"; $resn_exch{"H9C1 LAN"}="1H9C LAN"; $resn_exch{"H9C2 LAN"}="2H9C LAN";
$resn_exch{"H211 LAN"}="1H21 LAN"; $resn_exch{"H212 LAN"}="2H21 LAN"; $resn_exch{"H201 LAN"}="1H20 LAN";
$resn_exch{"H202 LAN"}="2H20 LAN"; $resn_exch{"H203 LAN"}="3H20 LAN"; $resn_exch{"H141 LAN"}="1H14 LAN";
$resn_exch{"H142 LAN"}="2H14 LAN"; $resn_exch{"H151 LAN"}="1H15 LAN"; $resn_exch{"H152 LAN"}="2H15 LAN";
$resn_exch{"H191 LAN"}="1H19 LAN"; $resn_exch{"H192 LAN"}="2H19 LAN"; $resn_exch{"H193 LAN"}="3H19 LAN";
$resn_exch{"H181 LAN"}="1H18 LAN"; $resn_exch{"H182 LAN"}="2H18 LAN"; $resn_exch{"H183 LAN"}="3H18 LAN";
$resn_exch{" H21 LAP"}="1H2  LAP"; $resn_exch{" H22 LAP"}="2H2  LAP"; $resn_exch{" H31 LAP"}="1H3  LAP";
$resn_exch{" H32 LAP"}="2H3  LAP"; $resn_exch{" H41 LAP"}="1H4  LAP"; $resn_exch{" H42 LAP"}="2H4  LAP";
$resn_exch{" H51 LAP"}="1H5  LAP"; $resn_exch{" H52 LAP"}="2H5  LAP"; $resn_exch{" H61 LAP"}="1H6  LAP";
$resn_exch{" H62 LAP"}="2H6  LAP"; $resn_exch{" H71 LAP"}="1H7  LAP"; $resn_exch{" H72 LAP"}="2H7  LAP";
$resn_exch{" H81 LAP"}="1H8  LAP"; $resn_exch{" H82 LAP"}="2H8  LAP"; $resn_exch{" H91 LAP"}="1H9  LAP";
$resn_exch{" H92 LAP"}="2H9  LAP"; $resn_exch{"H101 LAP"}="1H10 LAP"; $resn_exch{"H102 LAP"}="2H10 LAP";
$resn_exch{"H111 LAP"}="1H11 LAP"; $resn_exch{"H112 LAP"}="2H11 LAP"; $resn_exch{"H121 LAP"}="1H12 LAP";
$resn_exch{"H122 LAP"}="2H12 LAP"; $resn_exch{"H123 LAP"}="3H12 LAP"; $resn_exch{"H131 LAP"}="1H13 LAP";
$resn_exch{"H132 LAP"}="2H13 LAP"; $resn_exch{"H151 LAP"}="1H15 LAP"; $resn_exch{"H152 LAP"}="2H15 LAP";
$resn_exch{"H161 LAP"}="1H16 LAP"; $resn_exch{"H162 LAP"}="2H16 LAP"; $resn_exch{"H171 LAP"}="1H17 LAP";
$resn_exch{"H172 LAP"}="2H17 LAP"; $resn_exch{"H181 LAP"}="1H18 LAP"; $resn_exch{"H182 LAP"}="2H18 LAP";
$resn_exch{"H183 LAP"}="3H18 LAP"; $resn_exch{"H191 LAP"}="1H19 LAP"; $resn_exch{"H192 LAP"}="2H19 LAP";
$resn_exch{"H193 LAP"}="3H19 LAP"; $resn_exch{"H201 LAP"}="1H20 LAP"; $resn_exch{"H202 LAP"}="2H20 LAP";
$resn_exch{"H203 LAP"}="3H20 LAP"; $resn_exch{"H111 LAQ"}="1H11 LAQ"; $resn_exch{"H112 LAQ"}="2H11 LAQ";
$resn_exch{"H121 LAQ"}="1H12 LAQ"; $resn_exch{"H122 LAQ"}="2H12 LAQ"; $resn_exch{"H131 LAQ"}="1H13 LAQ";
$resn_exch{"H132 LAQ"}="2H13 LAQ"; $resn_exch{"H141 LAQ"}="1H14 LAQ"; $resn_exch{"H142 LAQ"}="2H14 LAQ";
$resn_exch{"H171 LAQ"}="1H17 LAQ"; $resn_exch{"H172 LAQ"}="2H17 LAQ"; $resn_exch{"H161 LAQ"}="1H16 LAQ";
$resn_exch{"H162 LAQ"}="2H16 LAQ"; $resn_exch{"H5'1 LAQ"}="1H5* LAQ"; $resn_exch{"H5'2 LAQ"}="2H5* LAQ";
$resn_exch{" H4' LAQ"}=" H4* LAQ"; $resn_exch{" H3' LAQ"}=" H3* LAQ"; $resn_exch{"HO3' LAQ"}="HO3* LAQ";
$resn_exch{" H2' LAQ"}=" H2* LAQ"; $resn_exch{"HO2' LAQ"}="HO2* LAQ"; $resn_exch{" H1' LAQ"}=" H1* LAQ";
$resn_exch{"HN61 LAQ"}="1HN6 LAQ"; $resn_exch{"HN62 LAQ"}="2HN6 LAQ"; $resn_exch{" H21 LAR"}="1H2  LAR";
$resn_exch{" H41 LAR"}="1H4  LAR"; $resn_exch{" H42 LAR"}="2H4  LAR"; $resn_exch{" H51 LAR"}="1H5  LAR";
$resn_exch{" H52 LAR"}="2H5  LAR"; $resn_exch{" H61 LAR"}="1H6  LAR"; $resn_exch{" H71 LAR"}="1H7  LAR";
$resn_exch{" H81 LAR"}="1H8  LAR"; $resn_exch{" H91 LAR"}="1H9  LAR"; $resn_exch{"H101 LAR"}="1H10 LAR";
$resn_exch{"H111 LAR"}="1H11 LAR"; $resn_exch{"H112 LAR"}="2H11 LAR"; $resn_exch{"H121 LAR"}="1H12 LAR";
$resn_exch{"H122 LAR"}="2H12 LAR"; $resn_exch{"H131 LAR"}="1H13 LAR"; $resn_exch{"H141 LAR"}="1H14 LAR";
$resn_exch{"H142 LAR"}="2H14 LAR"; $resn_exch{"H151 LAR"}="1H15 LAR"; $resn_exch{"H161 LAR"}="1H16 LAR";
$resn_exch{"H162 LAR"}="2H16 LAR"; $resn_exch{"H181 LAR"}="1H18 LAR"; $resn_exch{"H191 LAR"}="1H19 LAR";
$resn_exch{"H192 LAR"}="2H19 LAR"; $resn_exch{"H211 LAR"}="1H21 LAR"; $resn_exch{"H212 LAR"}="2H21 LAR";
$resn_exch{"H213 LAR"}="3H21 LAR"; $resn_exch{"H221 LAR"}="1H22 LAR"; $resn_exch{"H222 LAR"}="2H22 LAR";
$resn_exch{"H223 LAR"}="3H22 LAR"; $resn_exch{" H61 LAT"}="1H6  LAT"; $resn_exch{" H62 LAT"}="2H6  LAT";
$resn_exch{"H6'1 LAT"}="1H6' LAT"; $resn_exch{"H6'2 LAT"}="2H6' LAT"; $resn_exch{"HO1' LAT"}="'HO1 LAT";
$resn_exch{"HO2' LAT"}="'HO2 LAT"; $resn_exch{"HO3' LAT"}="'HO3 LAT"; $resn_exch{"HO6' LAT"}="'HO6 LAT";
$resn_exch{" H21 LAU"}="1H2  LAU"; $resn_exch{" H22 LAU"}="2H2  LAU"; $resn_exch{" H31 LAU"}="1H3  LAU";
$resn_exch{" H32 LAU"}="2H3  LAU"; $resn_exch{" H41 LAU"}="1H4  LAU"; $resn_exch{" H42 LAU"}="2H4  LAU";
$resn_exch{" H51 LAU"}="1H5  LAU"; $resn_exch{" H52 LAU"}="2H5  LAU"; $resn_exch{" H61 LAU"}="1H6  LAU";
$resn_exch{" H62 LAU"}="2H6  LAU"; $resn_exch{" H71 LAU"}="1H7  LAU"; $resn_exch{" H72 LAU"}="2H7  LAU";
$resn_exch{" H81 LAU"}="1H8  LAU"; $resn_exch{" H82 LAU"}="2H8  LAU"; $resn_exch{" H91 LAU"}="1H9  LAU";
$resn_exch{" H92 LAU"}="2H9  LAU"; $resn_exch{"H101 LAU"}="1H10 LAU"; $resn_exch{"H102 LAU"}="2H10 LAU";
$resn_exch{"H111 LAU"}="1H11 LAU"; $resn_exch{"H112 LAU"}="2H11 LAU"; $resn_exch{"H121 LAU"}="1H12 LAU";
$resn_exch{"H122 LAU"}="2H12 LAU"; $resn_exch{"H123 LAU"}="3H12 LAU"; $resn_exch{" H21 LAX"}="1H2  LAX";
$resn_exch{" H22 LAX"}="2H2  LAX"; $resn_exch{" H31 LAX"}="1H3  LAX"; $resn_exch{" H32 LAX"}="2H3  LAX";
$resn_exch{" H41 LAX"}="1H4  LAX"; $resn_exch{" H42 LAX"}="2H4  LAX"; $resn_exch{" H51 LAX"}="1H5  LAX";
$resn_exch{" H52 LAX"}="2H5  LAX"; $resn_exch{" H61 LAX"}="1H6  LAX"; $resn_exch{" H62 LAX"}="2H6  LAX";
$resn_exch{" H71 LAX"}="1H7  LAX"; $resn_exch{" H72 LAX"}="2H7  LAX"; $resn_exch{"H101 LAX"}="1H10 LAX";
$resn_exch{"H102 LAX"}="2H10 LAX"; $resn_exch{"H131 LAX"}="1H13 LAX"; $resn_exch{"H132 LAX"}="2H13 LAX";
$resn_exch{"H161 LAX"}="1H16 LAX"; $resn_exch{"H162 LAX"}="2H16 LAX"; $resn_exch{"H171 LAX"}="1H17 LAX";
$resn_exch{"H172 LAX"}="2H17 LAX"; $resn_exch{"H181 LAX"}="1H18 LAX"; $resn_exch{"H182 LAX"}="2H18 LAX";
$resn_exch{"H191 LAX"}="1H19 LAX"; $resn_exch{"H192 LAX"}="2H19 LAX"; $resn_exch{"H201 LAX"}="1H20 LAX";
$resn_exch{"H202 LAX"}="2H20 LAX"; $resn_exch{"H203 LAX"}="3H20 LAX"; $resn_exch{"H9C1 LAZ"}="1H9C LAZ";
$resn_exch{"H9C2 LAZ"}="2H9C LAZ"; $resn_exch{"H101 LAZ"}="1H10 LAZ"; $resn_exch{"H102 LAZ"}="2H10 LAZ";
$resn_exch{"H6C1 LBT"}="1H6C LBT"; $resn_exch{"H6C2 LBT"}="2H6C LBT"; $resn_exch{"H6'1 LBT"}="1H6' LBT";
$resn_exch{"H6'2 LBT"}="2H6' LBT"; $resn_exch{"HBC1 LBV"}="1HBC LBV"; $resn_exch{"HBC2 LBV"}="2HBC LBV";
$resn_exch{"HBC3 LBV"}="3HBC LBV"; $resn_exch{"HAC1 LBV"}="1HAC LBV"; $resn_exch{"HMC1 LBV"}="1HMC LBV";
$resn_exch{"HMC2 LBV"}="2HMC LBV"; $resn_exch{"HMC3 LBV"}="3HMC LBV"; $resn_exch{"HHD1 LBV"}="1HHD LBV";
$resn_exch{"HAD1 LBV"}="1HAD LBV"; $resn_exch{"HAD2 LBV"}="2HAD LBV"; $resn_exch{"HBD1 LBV"}="1HBD LBV";
$resn_exch{"HBD2 LBV"}="2HBD LBV"; $resn_exch{"HMD1 LBV"}="1HMD LBV"; $resn_exch{"HMD2 LBV"}="2HMD LBV";
$resn_exch{"HMD3 LBV"}="3HMD LBV"; $resn_exch{"HHA1 LBV"}="1HHA LBV"; $resn_exch{"HAA1 LBV"}="1HAA LBV";
$resn_exch{"HAA2 LBV"}="2HAA LBV"; $resn_exch{"HBA1 LBV"}="1HBA LBV"; $resn_exch{"HBA2 LBV"}="2HBA LBV";
$resn_exch{"HMA1 LBV"}="1HMA LBV"; $resn_exch{"HMA2 LBV"}="2HMA LBV"; $resn_exch{"HMA3 LBV"}="3HMA LBV";
$resn_exch{"HHB1 LBV"}="1HHB LBV"; $resn_exch{"HMB1 LBV"}="1HMB LBV"; $resn_exch{"HMB2 LBV"}="2HMB LBV";
$resn_exch{"HMB3 LBV"}="3HMB LBV"; $resn_exch{"HBB1 LBV"}="1HBB LBV"; $resn_exch{"HBB2 LBV"}="2HBB LBV";
$resn_exch{"HOP3  LC"}="3HOP  LC"; $resn_exch{"HOP2  LC"}="2HOP  LC"; $resn_exch{" H5'  LC"}="1H5*  LC";
$resn_exch{"H5''  LC"}="2H5*  LC"; $resn_exch{" H4'  LC"}=" H4*  LC"; $resn_exch{" H3'  LC"}=" H3*  LC";
$resn_exch{"HO3'  LC"}="H3T   LC"; $resn_exch{" H2'  LC"}=" H2*  LC"; $resn_exch{"HO2'  LC"}="2HO*  LC";
$resn_exch{" H1'  LC"}=" H1*  LC"; $resn_exch{" H41  LC"}="1H4   LC"; $resn_exch{" H42  LC"}="2H4   LC";
$resn_exch{"H5'1 LCA"}="1H5* LCA"; $resn_exch{"H5'2 LCA"}="2H5* LCA"; $resn_exch{" H3' LCA"}=" H3* LCA";
$resn_exch{"H6'1 LCA"}="1H6* LCA"; $resn_exch{"H6'2 LCA"}="2H6* LCA"; $resn_exch{"H2'1 LCA"}="1H2* LCA";
$resn_exch{" H61 LCA"}="1H6  LCA"; $resn_exch{" H62 LCA"}="2H6  LCA"; $resn_exch{" H1' LCA"}=" H1* LCA";
$resn_exch{"H5'1 LCC"}="1H5* LCC"; $resn_exch{"H5'2 LCC"}="2H5* LCC"; $resn_exch{" H1' LCC"}=" H1* LCC";
$resn_exch{"H5M1 LCC"}="1H5M LCC"; $resn_exch{"H5M2 LCC"}="2H5M LCC"; $resn_exch{"H5M3 LCC"}="3H5M LCC";
$resn_exch{" H41 LCC"}="1H4  LCC"; $resn_exch{" H42 LCC"}="2H4  LCC"; $resn_exch{" H3' LCC"}=" H3* LCC";
$resn_exch{"H2'1 LCC"}="1H2* LCC"; $resn_exch{"H6'1 LCC"}="1H6* LCC"; $resn_exch{"H6'2 LCC"}="2H6* LCC";
$resn_exch{" H21 LCF"}="1H2  LCF"; $resn_exch{"H22A LCF"}="2H2  LCF"; $resn_exch{" H41 LCF"}="1H4  LCF";
$resn_exch{" H42 LCF"}="2H4  LCF"; $resn_exch{" H81 LCF"}="1H8  LCF"; $resn_exch{" H82 LCF"}="2H8  LCF";
$resn_exch{"H101 LCF"}="1H10 LCF"; $resn_exch{"H102 LCF"}="2H10 LCF"; $resn_exch{"H103 LCF"}="3H10 LCF";
$resn_exch{"H111 LCF"}="1H11 LCF"; $resn_exch{"H112 LCF"}="2H11 LCF"; $resn_exch{"H113 LCF"}="3H11 LCF";
$resn_exch{" H5' LCG"}="1H5* LCG"; $resn_exch{"H5'' LCG"}="2H5* LCG"; $resn_exch{" H3' LCG"}=" H3* LCG";
$resn_exch{"H6'1 LCG"}="1H6* LCG"; $resn_exch{"H6'2 LCG"}="2H6* LCG"; $resn_exch{" H2' LCG"}="1H2* LCG";
$resn_exch{" H1' LCG"}=" H1* LCG"; $resn_exch{"HOP2 LCG"}="2HOP LCG"; $resn_exch{" H21 LCG"}="1H2  LCG";
$resn_exch{" H22 LCG"}="2H2  LCG"; $resn_exch{"HO3' LCG"}="H3T  LCG"; $resn_exch{"HOP3 LCG"}="3HOP LCG";
$resn_exch{"H5'1 LCH"}="1H5* LCH"; $resn_exch{"H5'2 LCH"}="2H5* LCH"; $resn_exch{" H1' LCH"}=" H1* LCH";
$resn_exch{"H5M1 LCH"}="1H5M LCH"; $resn_exch{"H5M2 LCH"}="2H5M LCH"; $resn_exch{"H5M3 LCH"}="3H5M LCH";
$resn_exch{" H41 LCH"}="1H4  LCH"; $resn_exch{" H42 LCH"}="2H4  LCH"; $resn_exch{" H3' LCH"}=" H3* LCH";
$resn_exch{"H2'1 LCH"}="1H2* LCH"; $resn_exch{"H6'1 LCH"}="1H6* LCH"; $resn_exch{"H6'2 LCH"}="2H6* LCH";
$resn_exch{"H2A1 LCS"}="1H2A LCS"; $resn_exch{"H2A2 LCS"}="2H2A LCS"; $resn_exch{"H2A3 LCS"}="3H2A LCS";
$resn_exch{"H4A1 LCS"}="1H4A LCS"; $resn_exch{"H4A2 LCS"}="2H4A LCS"; $resn_exch{"H5A1 LCS"}="1H5A LCS";
$resn_exch{"H5A2 LCS"}="2H5A LCS"; $resn_exch{"HOP2 LCS"}="2HOP LCS"; $resn_exch{"HOP3 LCS"}="3HOP LCS";
$resn_exch{" HB1 LCS"}="1HB  LCS"; $resn_exch{" HB2 LCS"}="2HB  LCS"; $resn_exch{" H   LCX"}="1HN  LCX";
$resn_exch{" H2  LCX"}="2HN  LCX"; $resn_exch{" HC1 LCX"}="1H   LCX"; $resn_exch{" HC2 LCX"}="2H   LCX";
$resn_exch{" HB2 LCX"}="1HB  LCX"; $resn_exch{" HB3 LCX"}="2HB  LCX"; $resn_exch{" HG2 LCX"}="1HG  LCX";
$resn_exch{" HG3 LCX"}="2HG  LCX"; $resn_exch{" HD2 LCX"}="1HD  LCX"; $resn_exch{" HD3 LCX"}="2HD  LCX";
$resn_exch{" HE2 LCX"}="1HE  LCX"; $resn_exch{" HE3 LCX"}="2HE  LCX"; $resn_exch{" HZ  LCX"}=" HNZ LCX";
$resn_exch{"HM11 LDA"}="1HM1 LDA"; $resn_exch{"HM12 LDA"}="2HM1 LDA"; $resn_exch{"HM13 LDA"}="3HM1 LDA";
$resn_exch{"HM21 LDA"}="1HM2 LDA"; $resn_exch{"HM22 LDA"}="2HM2 LDA"; $resn_exch{"HM23 LDA"}="3HM2 LDA";
$resn_exch{" H11 LDA"}="1H1  LDA"; $resn_exch{" H12 LDA"}="2H1  LDA"; $resn_exch{" H21 LDA"}="1H2  LDA";
$resn_exch{" H22 LDA"}="2H2  LDA"; $resn_exch{" H31 LDA"}="1H3  LDA"; $resn_exch{" H32 LDA"}="2H3  LDA";
$resn_exch{" H41 LDA"}="1H4  LDA"; $resn_exch{" H42 LDA"}="2H4  LDA"; $resn_exch{" H51 LDA"}="1H5  LDA";
$resn_exch{" H52 LDA"}="2H5  LDA"; $resn_exch{" H61 LDA"}="1H6  LDA"; $resn_exch{" H62 LDA"}="2H6  LDA";
$resn_exch{" H71 LDA"}="1H7  LDA"; $resn_exch{" H72 LDA"}="2H7  LDA"; $resn_exch{" H81 LDA"}="1H8  LDA";
$resn_exch{" H82 LDA"}="2H8  LDA"; $resn_exch{" H91 LDA"}="1H9  LDA"; $resn_exch{" H92 LDA"}="2H9  LDA";
$resn_exch{"H101 LDA"}="1H10 LDA"; $resn_exch{"H102 LDA"}="2H10 LDA"; $resn_exch{"H111 LDA"}="1H11 LDA";
$resn_exch{"H112 LDA"}="2H11 LDA"; $resn_exch{"H121 LDA"}="1H12 LDA"; $resn_exch{"H122 LDA"}="2H12 LDA";
$resn_exch{"H123 LDA"}="3H12 LDA"; $resn_exch{"HN41 LDC"}="1HN4 LDC"; $resn_exch{"HN42 LDC"}="2HN4 LDC";
$resn_exch{" H91 LDC"}="1H9  LDC"; $resn_exch{" H92 LDC"}="2H9  LDC"; $resn_exch{"H121 LDC"}="1H12 LDC";
$resn_exch{"H122 LDC"}="2H12 LDC"; $resn_exch{" HN1 LDH"}="1HN  LDH"; $resn_exch{" HN2 LDH"}="2HN  LDH";
$resn_exch{" HB1 LDH"}="1HB  LDH"; $resn_exch{" HB2 LDH"}="2HB  LDH"; $resn_exch{" HG1 LDH"}="1HG  LDH";
$resn_exch{" HG2 LDH"}="2HG  LDH"; $resn_exch{" HD1 LDH"}="1HD  LDH"; $resn_exch{" HD2 LDH"}="2HD  LDH";
$resn_exch{" HE1 LDH"}="1HE  LDH"; $resn_exch{" HE2 LDH"}="2HE  LDH"; $resn_exch{" HH1 LDH"}="1HH  LDH";
$resn_exch{" HH2 LDH"}="2HH  LDH"; $resn_exch{" H11 LDH"}="1H1  LDH"; $resn_exch{" H12 LDH"}="2H1  LDH";
$resn_exch{" H13 LDH"}="3H1  LDH"; $resn_exch{"H1C1 LDM"}="1H1C LDM"; $resn_exch{"H1C2 LDM"}="2H1C LDM";
$resn_exch{"H3C1 LDM"}="1H3C LDM"; $resn_exch{"H3C2 LDM"}="2H3C LDM"; $resn_exch{"H5C1 LDM"}="1H5C LDM";
$resn_exch{"H5C2 LDM"}="2H5C LDM"; $resn_exch{"H7C1 LDM"}="1H7C LDM"; $resn_exch{"H7C2 LDM"}="2H7C LDM";
$resn_exch{"H8C1 LDM"}="1H8C LDM"; $resn_exch{"H8C2 LDM"}="2H8C LDM"; $resn_exch{"H101 LDM"}="1H10 LDM";
$resn_exch{"H102 LDM"}="2H10 LDM"; $resn_exch{"H111 LDM"}="1H11 LDM"; $resn_exch{"H112 LDM"}="2H11 LDM";
$resn_exch{"H121 LDM"}="1H12 LDM"; $resn_exch{"H122 LDM"}="2H12 LDM"; $resn_exch{"HM11 LDM"}="1HM1 LDM";
$resn_exch{"HM12 LDM"}="2HM1 LDM"; $resn_exch{"HM13 LDM"}="3HM1 LDM"; $resn_exch{"HM21 LDM"}="1HM2 LDM";
$resn_exch{"HM22 LDM"}="2HM2 LDM"; $resn_exch{"HM23 LDM"}="3HM2 LDM"; $resn_exch{"H2C1 LDM"}="1H2C LDM";
$resn_exch{"H2C2 LDM"}="2H2C LDM"; $resn_exch{"H9C1 LDM"}="1H9C LDM"; $resn_exch{"H9C2 LDM"}="2H9C LDM";
$resn_exch{"H141 LDM"}="1H14 LDM"; $resn_exch{"H142 LDM"}="2H14 LDM"; $resn_exch{"H151 LDM"}="1H15 LDM";
$resn_exch{"H152 LDM"}="2H15 LDM"; $resn_exch{"H153 LDM"}="3H15 LDM"; $resn_exch{"H6C1 LDM"}="1H6C LDM";
$resn_exch{"H6C2 LDM"}="2H6C LDM"; $resn_exch{"H131 LDM"}="1H13 LDM"; $resn_exch{"H132 LDM"}="2H13 LDM";
$resn_exch{" HN1 LDO"}="1HN  LDO"; $resn_exch{" HN2 LDO"}="2HN  LDO"; $resn_exch{" HA1 LDO"}="1HA  LDO";
$resn_exch{" HB1 LDO"}="1HB  LDO"; $resn_exch{" HB2 LDO"}="2HB  LDO"; $resn_exch{" HG1 LDO"}="1HG  LDO";
$resn_exch{" HG2 LDO"}="2HG  LDO"; $resn_exch{" HD1 LDO"}="1HD  LDO"; $resn_exch{" HD2 LDO"}="2HD  LDO";
$resn_exch{" HE1 LDO"}="1HE  LDO"; $resn_exch{" HE2 LDO"}="2HE  LDO"; $resn_exch{" H71 LDP"}="1H7  LDP";
$resn_exch{" H72 LDP"}="2H7  LDP"; $resn_exch{" H81 LDP"}="1H8  LDP"; $resn_exch{" H82 LDP"}="2H8  LDP";
$resn_exch{"HN11 LDP"}="1HN1 LDP"; $resn_exch{"HN12 LDP"}="2HN1 LDP"; $resn_exch{"H131 LDT"}="1H13 LDT";
$resn_exch{"H132 LDT"}="2H13 LDT"; $resn_exch{"H201 LDT"}="1H20 LDT"; $resn_exch{"H202 LDT"}="2H20 LDT";
$resn_exch{" H31 LEA"}="1H3  LEA"; $resn_exch{" H32 LEA"}="2H3  LEA"; $resn_exch{" H41 LEA"}="1H4  LEA";
$resn_exch{" H42 LEA"}="2H4  LEA"; $resn_exch{" H51 LEA"}="1H5  LEA"; $resn_exch{" H52 LEA"}="2H5  LEA";
$resn_exch{" H61 LEA"}="1H6  LEA"; $resn_exch{" H62 LEA"}="2H6  LEA"; $resn_exch{" H63 LEA"}="3H6  LEA";
$resn_exch{" HN1 LED"}="1HN  LED"; $resn_exch{" HN2 LED"}="2HN  LED"; $resn_exch{"HBC1 LED"}="1HBC LED";
$resn_exch{"HBC2 LED"}="2HBC LED"; $resn_exch{"HD11 LED"}="1HD1 LED"; $resn_exch{"HD12 LED"}="2HD1 LED";
$resn_exch{"HD13 LED"}="3HD1 LED"; $resn_exch{"HD12 LEF"}="1HD1 LEF"; $resn_exch{"HD13 LEF"}="2HD1 LEF";
$resn_exch{"HD21 LEF"}="1HD2 LEF"; $resn_exch{"HD22 LEF"}="2HD2 LEF"; $resn_exch{"HD23 LEF"}="3HD2 LEF";
$resn_exch{" HB3 LEF"}="1HBC LEF"; $resn_exch{" HB2 LEF"}="2HBC LEF"; $resn_exch{" H   LEF"}="1HN  LEF";
$resn_exch{" H2  LEF"}="2HN  LEF"; $resn_exch{"H271 LEG"}="1H27 LEG"; $resn_exch{"H272 LEG"}="2H27 LEG";
$resn_exch{"H273 LEG"}="3H27 LEG"; $resn_exch{"H241 LEG"}="1H24 LEG"; $resn_exch{"H242 LEG"}="2H24 LEG";
$resn_exch{"H211 LEG"}="1H21 LEG"; $resn_exch{"H212 LEG"}="2H21 LEG"; $resn_exch{"H181 LEG"}="1H18 LEG";
$resn_exch{"H182 LEG"}="2H18 LEG"; $resn_exch{"H151 LEG"}="1H15 LEG"; $resn_exch{"H152 LEG"}="2H15 LEG";
$resn_exch{"H141 LEG"}="1H14 LEG"; $resn_exch{"H142 LEG"}="2H14 LEG"; $resn_exch{"H331 LEG"}="1H33 LEG";
$resn_exch{"H332 LEG"}="2H33 LEG"; $resn_exch{"HD21 LEN"}="1HD2 LEN"; $resn_exch{"HD22 LEN"}="2HD2 LEN";
$resn_exch{"HD23 LEN"}="3HD2 LEN"; $resn_exch{"HD11 LEN"}="1HD1 LEN"; $resn_exch{"HD12 LEN"}="2HD1 LEN";
$resn_exch{"HD13 LEN"}="3HD1 LEN"; $resn_exch{" HB1 LEN"}="1HB  LEN"; $resn_exch{" HB2 LEN"}="2HB  LEN";
$resn_exch{" HA1 LEN"}="1HA  LEN"; $resn_exch{" HA2 LEN"}="2HA  LEN"; $resn_exch{" HN1 LEN"}="1HN  LEN";
$resn_exch{" HN2 LEN"}="2HN  LEN"; $resn_exch{" H21 LEO"}="1H2  LEO"; $resn_exch{" H22 LEO"}="2H2  LEO";
$resn_exch{" H41 LEO"}="1H4  LEO"; $resn_exch{" H42 LEO"}="2H4  LEO"; $resn_exch{" H51 LEO"}="1H5  LEO";
$resn_exch{" H52 LEO"}="2H5  LEO"; $resn_exch{" H81 LEO"}="1H8  LEO"; $resn_exch{" H82 LEO"}="2H8  LEO";
$resn_exch{" H83 LEO"}="3H8  LEO"; $resn_exch{"H211 LEO"}="1H21 LEO"; $resn_exch{"H212 LEO"}="2H21 LEO";
$resn_exch{"H221 LEO"}="1H22 LEO"; $resn_exch{"H222 LEO"}="2H22 LEO"; $resn_exch{"H223 LEO"}="3H22 LEO";
$resn_exch{"H101 LET"}="1H10 LET"; $resn_exch{"H102 LET"}="2H10 LET"; $resn_exch{"H9C1 LET"}="1H9C LET";
$resn_exch{"H9C2 LET"}="2H9C LET"; $resn_exch{"H7C1 LET"}="1H7C LET"; $resn_exch{"H7C2 LET"}="2H7C LET";
$resn_exch{"H5C1 LET"}="1H5C LET"; $resn_exch{"H5C2 LET"}="2H5C LET"; $resn_exch{"H3C1 LET"}="1H3C LET";
$resn_exch{"H3C2 LET"}="2H3C LET"; $resn_exch{"H2C1 LET"}="1H2C LET"; $resn_exch{"H2C2 LET"}="2H2C LET";
$resn_exch{"HEC1 LET"}="1HEC LET"; $resn_exch{"HEC2 LET"}="2HEC LET"; $resn_exch{"HDC1 LET"}="1HDC LET";
$resn_exch{"HDC2 LET"}="2HDC LET"; $resn_exch{"HGC1 LET"}="1HGC LET"; $resn_exch{"HGC2 LET"}="2HGC LET";
$resn_exch{"HBC1 LET"}="1HBC LET"; $resn_exch{"HBC2 LET"}="2HBC LET"; $resn_exch{" HN1 LET"}="1HN  LET";
$resn_exch{" HN2 LET"}="2HN  LET"; $resn_exch{" H2  LEU"}=" HN2 LEU"; $resn_exch{" HB2 LEU"}="1HB  LEU";
$resn_exch{" HB3 LEU"}="2HB  LEU"; $resn_exch{"HD11 LEU"}="1HD1 LEU"; $resn_exch{"HD12 LEU"}="2HD1 LEU";
$resn_exch{"HD13 LEU"}="3HD1 LEU"; $resn_exch{"HD21 LEU"}="1HD2 LEU"; $resn_exch{"HD22 LEU"}="2HD2 LEU";
$resn_exch{"HD23 LEU"}="3HD2 LEU"; $resn_exch{" H11 LFA"}="1H1  LFA"; $resn_exch{" H12 LFA"}="2H1  LFA";
$resn_exch{" H13 LFA"}="3H1  LFA"; $resn_exch{" H21 LFA"}="1H2  LFA"; $resn_exch{" H22 LFA"}="2H2  LFA";
$resn_exch{" H31 LFA"}="1H3  LFA"; $resn_exch{" H32 LFA"}="2H3  LFA"; $resn_exch{" H41 LFA"}="1H4  LFA";
$resn_exch{" H42 LFA"}="2H4  LFA"; $resn_exch{" H51 LFA"}="1H5  LFA"; $resn_exch{" H52 LFA"}="2H5  LFA";
$resn_exch{" H61 LFA"}="1H6  LFA"; $resn_exch{" H62 LFA"}="2H6  LFA"; $resn_exch{" H71 LFA"}="1H7  LFA";
$resn_exch{" H72 LFA"}="2H7  LFA"; $resn_exch{" H81 LFA"}="1H8  LFA"; $resn_exch{" H82 LFA"}="2H8  LFA";
$resn_exch{" H91 LFA"}="1H9  LFA"; $resn_exch{" H92 LFA"}="2H9  LFA"; $resn_exch{"H101 LFA"}="1H10 LFA";
$resn_exch{"H102 LFA"}="2H10 LFA"; $resn_exch{"H111 LFA"}="1H11 LFA"; $resn_exch{"H112 LFA"}="2H11 LFA";
$resn_exch{"H121 LFA"}="1H12 LFA"; $resn_exch{"H122 LFA"}="2H12 LFA"; $resn_exch{"H131 LFA"}="1H13 LFA";
$resn_exch{"H132 LFA"}="2H13 LFA"; $resn_exch{"H141 LFA"}="1H14 LFA"; $resn_exch{"H142 LFA"}="2H14 LFA";
$resn_exch{"H151 LFA"}="1H15 LFA"; $resn_exch{"H152 LFA"}="2H15 LFA"; $resn_exch{"H161 LFA"}="1H16 LFA";
$resn_exch{"H162 LFA"}="2H16 LFA"; $resn_exch{"H171 LFA"}="1H17 LFA"; $resn_exch{"H172 LFA"}="2H17 LFA";
$resn_exch{"H181 LFA"}="1H18 LFA"; $resn_exch{"H182 LFA"}="2H18 LFA"; $resn_exch{"H191 LFA"}="1H19 LFA";
$resn_exch{"H192 LFA"}="2H19 LFA"; $resn_exch{"H201 LFA"}="1H20 LFA"; $resn_exch{"H202 LFA"}="2H20 LFA";
$resn_exch{"H203 LFA"}="3H20 LFA"; $resn_exch{" H61 LFC"}="1H6  LFC"; $resn_exch{" H62 LFC"}="2H6  LFC";
$resn_exch{" H63 LFC"}="3H6  LFC"; $resn_exch{"H7M1 LFN"}="1H7M LFN"; $resn_exch{"H7M2 LFN"}="2H7M LFN";
$resn_exch{"H7M3 LFN"}="3H7M LFN"; $resn_exch{"H8M1 LFN"}="1H8M LFN"; $resn_exch{"H8M2 LFN"}="2H8M LFN";
$resn_exch{"H8M3 LFN"}="3H8M LFN"; $resn_exch{"H1'1 LFN"}="1H1* LFN"; $resn_exch{"H1'2 LFN"}="2H1* LFN";
$resn_exch{"H1'3 LFN"}="3H1* LFN"; $resn_exch{"H1C1 LFR"}="1H1C LFR"; $resn_exch{"H1C2 LFR"}="2H1C LFR";
$resn_exch{"H6C1 LFR"}="1H6C LFR"; $resn_exch{"H6C2 LFR"}="2H6C LFR"; $resn_exch{"HOP3  LG"}="3HOP  LG";
$resn_exch{"HOP2  LG"}="2HOP  LG"; $resn_exch{" H5'  LG"}="1H5*  LG"; $resn_exch{"H5''  LG"}="2H5*  LG";
$resn_exch{" H4'  LG"}=" H4*  LG"; $resn_exch{" H3'  LG"}=" H3*  LG"; $resn_exch{"HO3'  LG"}="H3T   LG";
$resn_exch{" H2'  LG"}=" H2*  LG"; $resn_exch{"HO2'  LG"}="2HO*  LG"; $resn_exch{" H1'  LG"}=" H1*  LG";
$resn_exch{" H21  LG"}="1H2   LG"; $resn_exch{" H22  LG"}="2H2   LG"; $resn_exch{" HC2 LG2"}="2HC  LG2";
$resn_exch{" HC5 LG2"}="5HC  LG2"; $resn_exch{"HC81 LG2"}="1HC8 LG2"; $resn_exch{"HC82 LG2"}="2HC8 LG2";
$resn_exch{"HC91 LG2"}="1HC9 LG2"; $resn_exch{"HC92 LG2"}="2HC9 LG2"; $resn_exch{"HC41 LG2"}="1HC4 LG2";
$resn_exch{"HC61 LG2"}="1HC6 LG2"; $resn_exch{"HC71 LG2"}="1HC7 LG2"; $resn_exch{"HO02 LG2"}="2HO0 LG2";
$resn_exch{"H211 LG2"}="1H21 LG2"; $resn_exch{"H212 LG2"}="2H21 LG2"; $resn_exch{"H221 LG2"}="1H22 LG2";
$resn_exch{"H222 LG2"}="2H22 LG2"; $resn_exch{"H231 LG2"}="1H23 LG2"; $resn_exch{"H232 LG2"}="2H23 LG2";
$resn_exch{"H233 LG2"}="3H23 LG2"; $resn_exch{"H241 LG2"}="1H24 LG2"; $resn_exch{"H242 LG2"}="2H24 LG2";
$resn_exch{"H243 LG2"}="3H24 LG2"; $resn_exch{"H251 LG2"}="1H25 LG2"; $resn_exch{"H252 LG2"}="2H25 LG2";
$resn_exch{"H253 LG2"}="3H25 LG2"; $resn_exch{"H261 LG2"}="1H26 LG2"; $resn_exch{"H262 LG2"}="2H26 LG2";
$resn_exch{"H263 LG2"}="3H26 LG2"; $resn_exch{"H271 LG2"}="1H27 LG2"; $resn_exch{"H272 LG2"}="2H27 LG2";
$resn_exch{"H273 LG2"}="3H27 LG2"; $resn_exch{"HN71 LG3"}="1HN7 LG3"; $resn_exch{"HN72 LG3"}="2HN7 LG3";
$resn_exch{"HN81 LG3"}="1HN8 LG3"; $resn_exch{"HN82 LG3"}="2HN8 LG3"; $resn_exch{"HN71 LG4"}="1HN7 LG4";
$resn_exch{"HN72 LG4"}="2HN7 LG4"; $resn_exch{" H81 LG4"}="1H8  LG4"; $resn_exch{" H82 LG4"}="2H8  LG4";
$resn_exch{" H83 LG4"}="3H8  LG4"; $resn_exch{" H21 LG5"}="1H2  LG5"; $resn_exch{" H22 LG5"}="2H2  LG5";
$resn_exch{" H71 LG5"}="1H7  LG5"; $resn_exch{" H72 LG5"}="2H7  LG5"; $resn_exch{" H73 LG5"}="3H7  LG5";
$resn_exch{"HN81 LG5"}="1HN8 LG5"; $resn_exch{"HN82 LG5"}="2HN8 LG5"; $resn_exch{" H61 LG6"}="1H6  LG6";
$resn_exch{" H62 LG6"}="2H6  LG6"; $resn_exch{"HOP2 LG6"}="2HOP LG6"; $resn_exch{"HOP3 LG6"}="3HOP LG6";
$resn_exch{" H31 LG7"}="1H3  LG7"; $resn_exch{" H32 LG7"}="2H3  LG7"; $resn_exch{" H41 LG7"}="1H4  LG7";
$resn_exch{" H42 LG7"}="2H4  LG7"; $resn_exch{"HBD1 LG8"}="1HBD LG8"; $resn_exch{"HBD2 LG8"}="2HBD LG8";
$resn_exch{"HBB1 LG8"}="1HBB LG8"; $resn_exch{"HBB2 LG8"}="2HBB LG8"; $resn_exch{"H6C1 LG8"}="1H6C LG8";
$resn_exch{"H6C2 LG8"}="2H6C LG8"; $resn_exch{"HBA1 LG8"}="1HBA LG8"; $resn_exch{"HBA2 LG8"}="2HBA LG8";
$resn_exch{"HBA3 LG8"}="3HBA LG8"; $resn_exch{"HBG1 LG8"}="1HBG LG8"; $resn_exch{"HBG2 LG8"}="2HBG LG8";
$resn_exch{"HBJ1 LG8"}="1HBJ LG8"; $resn_exch{"HBJ2 LG8"}="2HBJ LG8"; $resn_exch{"HBJ3 LG8"}="3HBJ LG8";
$resn_exch{"HBI1 LG8"}="1HBI LG8"; $resn_exch{"HBI2 LG8"}="2HBI LG8"; $resn_exch{"HBI3 LG8"}="3HBI LG8";
$resn_exch{"HC61 LGC"}="1HC6 LGC"; $resn_exch{"HC62 LGC"}="2HC6 LGC"; $resn_exch{"H121 LGD"}="1H12 LGD";
$resn_exch{"H122 LGD"}="2H12 LGD"; $resn_exch{"H141 LGD"}="1H14 LGD"; $resn_exch{"H142 LGD"}="2H14 LGD";
$resn_exch{" H41 LGF"}="1H4  LGF"; $resn_exch{" H42 LGF"}="2H4  LGF"; $resn_exch{" H21 LGF"}="1H2  LGF";
$resn_exch{" H22 LGF"}="2H2  LGF"; $resn_exch{" H11 LGF"}="1H1  LGF"; $resn_exch{"H12A LGF"}="2H1  LGF";
$resn_exch{"H351 LGF"}="1H35 LGF"; $resn_exch{"H352 LGF"}="2H35 LGF"; $resn_exch{"H353 LGF"}="3H35 LGF";
$resn_exch{"H361 LGF"}="1H36 LGF"; $resn_exch{"H362 LGF"}="2H36 LGF"; $resn_exch{"H381 LGF"}="1H38 LGF";
$resn_exch{"H382 LGF"}="2H38 LGF"; $resn_exch{"H391 LGF"}="1H39 LGF"; $resn_exch{"H392 LGF"}="2H39 LGF";
$resn_exch{"H171 LGG"}="1H17 LGG"; $resn_exch{"H172 LGG"}="2H17 LGG"; $resn_exch{"H201 LGG"}="1H20 LGG";
$resn_exch{"H202 LGG"}="2H20 LGG"; $resn_exch{"H203 LGG"}="3H20 LGG"; $resn_exch{"H211 LGG"}="1H21 LGG";
$resn_exch{"H212 LGG"}="2H21 LGG"; $resn_exch{"H213 LGG"}="3H21 LGG"; $resn_exch{"H281 LGG"}="1H28 LGG";
$resn_exch{"H282 LGG"}="2H28 LGG"; $resn_exch{"H121 LGG"}="1H12 LGG"; $resn_exch{"H122 LGG"}="2H12 LGG";
$resn_exch{" H31 LGG"}="1H3  LGG"; $resn_exch{" H32 LGG"}="2H3  LGG"; $resn_exch{" H21 LGG"}="1H2  LGG";
$resn_exch{"H22A LGG"}="2H2  LGG"; $resn_exch{"H341 LGG"}="1H34 LGG"; $resn_exch{"H342 LGG"}="2H34 LGG";
$resn_exch{"H343 LGG"}="3H34 LGG"; $resn_exch{"H161 LGG"}="1H16 LGG"; $resn_exch{"H162 LGG"}="2H16 LGG";
$resn_exch{"HN41 LGG"}="1HN4 LGG"; $resn_exch{"HN42 LGG"}="2HN4 LGG"; $resn_exch{"H231 LGG"}="1H23 LGG";
$resn_exch{"H232 LGG"}="2H23 LGG"; $resn_exch{"H301 LGG"}="1H30 LGG"; $resn_exch{"H302 LGG"}="2H30 LGG";
$resn_exch{"H311 LGG"}="1H31 LGG"; $resn_exch{"H312 LGG"}="2H31 LGG"; $resn_exch{"H271 LGG"}="1H27 LGG";
$resn_exch{"H272 LGG"}="2H27 LGG"; $resn_exch{"HN71 LGG"}="1HN7 LGG"; $resn_exch{"HN72 LGG"}="2HN7 LGG";
$resn_exch{"HOP2 LGP"}="2HOP LGP"; $resn_exch{"HOP3 LGP"}="3HOP LGP"; $resn_exch{" H5' LGP"}="1H5* LGP";
$resn_exch{"H5'' LGP"}="2H5* LGP"; $resn_exch{" H4' LGP"}=" H4* LGP"; $resn_exch{"H3'1 LGP"}="1H3* LGP";
$resn_exch{"H3'2 LGP"}="2H3* LGP"; $resn_exch{"HO3' LGP"}="*HO3 LGP"; $resn_exch{"H1'1 LGP"}="1H1* LGP";
$resn_exch{"H1'2 LGP"}="2H1* LGP"; $resn_exch{"HN21 LGP"}="1HN2 LGP"; $resn_exch{"HN22 LGP"}="2HN2 LGP";
$resn_exch{"H211 LHA"}="1H21 LHA"; $resn_exch{"H212 LHA"}="2H21 LHA"; $resn_exch{"H611 LHA"}="1H61 LHA";
$resn_exch{"H612 LHA"}="2H61 LHA"; $resn_exch{"H221 LHA"}="1H22 LHA"; $resn_exch{"H222 LHA"}="2H22 LHA";
$resn_exch{"H321 LHA"}="1H32 LHA"; $resn_exch{"H322 LHA"}="2H32 LHA"; $resn_exch{"H531 LHA"}="1H53 LHA";
$resn_exch{"H532 LHA"}="2H53 LHA"; $resn_exch{"H241 LHA"}="1H24 LHA"; $resn_exch{"H242 LHA"}="2H24 LHA";
$resn_exch{"H641 LHA"}="1H64 LHA"; $resn_exch{"H642 LHA"}="2H64 LHA"; $resn_exch{"HN41 LHA"}="1HN4 LHA";
$resn_exch{"HN42 LHA"}="2HN4 LHA"; $resn_exch{"H151 LHA"}="1H15 LHA"; $resn_exch{"H152 LHA"}="2H15 LHA";
$resn_exch{"H251 LHA"}="1H25 LHA"; $resn_exch{"H252 LHA"}="2H25 LHA"; $resn_exch{"H351 LHA"}="1H35 LHA";
$resn_exch{"H352 LHA"}="2H35 LHA"; $resn_exch{"H451 LHA"}="1H45 LHA"; $resn_exch{"H452 LHA"}="2H45 LHA";
$resn_exch{"H371 LHA"}="1H37 LHA"; $resn_exch{"H372 LHA"}="2H37 LHA"; $resn_exch{"H471 LHA"}="1H47 LHA";
$resn_exch{"H472 LHA"}="2H47 LHA"; $resn_exch{"HN71 LHA"}="1HN7 LHA"; $resn_exch{"HN72 LHA"}="2HN7 LHA";
$resn_exch{"HC11 LHG"}="1HC1 LHG"; $resn_exch{"HC12 LHG"}="2HC1 LHG"; $resn_exch{"HC31 LHG"}="1HC3 LHG";
$resn_exch{"HC32 LHG"}="2HC3 LHG"; $resn_exch{"HC41 LHG"}="1HC4 LHG"; $resn_exch{"HC42 LHG"}="2HC4 LHG";
$resn_exch{"HC61 LHG"}="1HC6 LHG"; $resn_exch{"HC62 LHG"}="2HC6 LHG"; $resn_exch{"HC81 LHG"}="1HC8 LHG";
$resn_exch{"HC82 LHG"}="2HC8 LHG"; $resn_exch{"HC91 LHG"}="1HC9 LHG"; $resn_exch{"HC92 LHG"}="2HC9 LHG";
$resn_exch{"H101 LHG"}="1H10 LHG"; $resn_exch{"H102 LHG"}="2H10 LHG"; $resn_exch{"H241 LHG"}="1H24 LHG";
$resn_exch{"H242 LHG"}="2H24 LHG"; $resn_exch{"H111 LHG"}="1H11 LHG"; $resn_exch{"H112 LHG"}="2H11 LHG";
$resn_exch{"H121 LHG"}="1H12 LHG"; $resn_exch{"H122 LHG"}="2H12 LHG"; $resn_exch{"H131 LHG"}="1H13 LHG";
$resn_exch{"H132 LHG"}="2H13 LHG"; $resn_exch{"H141 LHG"}="1H14 LHG"; $resn_exch{"H142 LHG"}="2H14 LHG";
$resn_exch{"H151 LHG"}="1H15 LHG"; $resn_exch{"H152 LHG"}="2H15 LHG"; $resn_exch{"H161 LHG"}="1H16 LHG";
$resn_exch{"H162 LHG"}="2H16 LHG"; $resn_exch{"H171 LHG"}="1H17 LHG"; $resn_exch{"H172 LHG"}="2H17 LHG";
$resn_exch{"H181 LHG"}="1H18 LHG"; $resn_exch{"H182 LHG"}="2H18 LHG"; $resn_exch{"H191 LHG"}="1H19 LHG";
$resn_exch{"H192 LHG"}="2H19 LHG"; $resn_exch{"H201 LHG"}="1H20 LHG"; $resn_exch{"H202 LHG"}="2H20 LHG";
$resn_exch{"H211 LHG"}="1H21 LHG"; $resn_exch{"H212 LHG"}="2H21 LHG"; $resn_exch{"H221 LHG"}="1H22 LHG";
$resn_exch{"H222 LHG"}="2H22 LHG"; $resn_exch{"H223 LHG"}="3H22 LHG"; $resn_exch{"H251 LHG"}="1H25 LHG";
$resn_exch{"H252 LHG"}="2H25 LHG"; $resn_exch{"H261 LHG"}="1H26 LHG"; $resn_exch{"H262 LHG"}="2H26 LHG";
$resn_exch{"H271 LHG"}="1H27 LHG"; $resn_exch{"H272 LHG"}="2H27 LHG"; $resn_exch{"H281 LHG"}="1H28 LHG";
$resn_exch{"H282 LHG"}="2H28 LHG"; $resn_exch{"H291 LHG"}="1H29 LHG"; $resn_exch{"H292 LHG"}="2H29 LHG";
$resn_exch{"H301 LHG"}="1H30 LHG"; $resn_exch{"H302 LHG"}="2H30 LHG"; $resn_exch{"H311 LHG"}="1H31 LHG";
$resn_exch{"H312 LHG"}="2H31 LHG"; $resn_exch{"H321 LHG"}="1H32 LHG"; $resn_exch{"H322 LHG"}="2H32 LHG";
$resn_exch{"H331 LHG"}="1H33 LHG"; $resn_exch{"H332 LHG"}="2H33 LHG"; $resn_exch{"H341 LHG"}="1H34 LHG";
$resn_exch{"H342 LHG"}="2H34 LHG"; $resn_exch{"H351 LHG"}="1H35 LHG"; $resn_exch{"H352 LHG"}="2H35 LHG";
$resn_exch{"H361 LHG"}="1H36 LHG"; $resn_exch{"H362 LHG"}="2H36 LHG"; $resn_exch{"H371 LHG"}="1H37 LHG";
$resn_exch{"H372 LHG"}="2H37 LHG"; $resn_exch{"H381 LHG"}="1H38 LHG"; $resn_exch{"H382 LHG"}="2H38 LHG";
$resn_exch{"H383 LHG"}="3H38 LHG"; $resn_exch{"HOP3 LHU"}="3HOP LHU"; $resn_exch{"HOP2 LHU"}="2HOP LHU";
$resn_exch{" H5' LHU"}="1H5* LHU"; $resn_exch{"H5'' LHU"}="2H5* LHU"; $resn_exch{" H4' LHU"}=" H4* LHU";
$resn_exch{" H3' LHU"}=" H3* LHU"; $resn_exch{"HO3' LHU"}="H3T  LHU"; $resn_exch{" H2' LHU"}=" H2* LHU";
$resn_exch{"HO2' LHU"}="2HO* LHU"; $resn_exch{" H1' LHU"}=" H1* LHU"; $resn_exch{" HB1 LHY"}="1HB  LHY";
$resn_exch{" HB2 LHY"}="2HB  LHY"; $resn_exch{" H11 LI0"}="1H1  LI0"; $resn_exch{"H12A LI0"}="2H1  LI0";
$resn_exch{"H31A LI0"}="1H3  LI0"; $resn_exch{"H32A LI0"}="2H3  LI0"; $resn_exch{" H41 LI0"}="1H4  LI0";
$resn_exch{" H42 LI0"}="2H4  LI0"; $resn_exch{" H51 LI0"}="1H5  LI0"; $resn_exch{" H52 LI0"}="2H5  LI0";
$resn_exch{"H181 LI0"}="1H18 LI0"; $resn_exch{"H182 LI0"}="2H18 LI0"; $resn_exch{"H251 LI0"}="1H25 LI0";
$resn_exch{"H252 LI0"}="2H25 LI0"; $resn_exch{"H253 LI0"}="3H25 LI0"; $resn_exch{"H261 LI0"}="1H26 LI0";
$resn_exch{"H262 LI0"}="2H26 LI0"; $resn_exch{"H263 LI0"}="3H26 LI0"; $resn_exch{"H291 LI0"}="1H29 LI0";
$resn_exch{"H292 LI0"}="2H29 LI0"; $resn_exch{"H293 LI0"}="3H29 LI0"; $resn_exch{"H301 LI0"}="1H30 LI0";
$resn_exch{"H302 LI0"}="2H30 LI0"; $resn_exch{" H11 LI1"}="1H1  LI1"; $resn_exch{" H12 LI1"}="2H1  LI1";
$resn_exch{" H31 LI1"}="1H3  LI1"; $resn_exch{" H32 LI1"}="2H3  LI1"; $resn_exch{"H411 LI1"}="1H41 LI1";
$resn_exch{"H412 LI1"}="2H41 LI1"; $resn_exch{"H421 LI1"}="1H42 LI1"; $resn_exch{"H422 LI1"}="2H42 LI1";
$resn_exch{"H441 LI1"}="1H44 LI1"; $resn_exch{"H442 LI1"}="2H44 LI1"; $resn_exch{"H443 LI1"}="3H44 LI1";
$resn_exch{"H451 LI1"}="1H45 LI1"; $resn_exch{"H452 LI1"}="2H45 LI1"; $resn_exch{"H461 LI1"}="1H46 LI1";
$resn_exch{"H462 LI1"}="2H46 LI1"; $resn_exch{"H471 LI1"}="1H47 LI1"; $resn_exch{"H472 LI1"}="2H47 LI1";
$resn_exch{"H491 LI1"}="1H49 LI1"; $resn_exch{"H492 LI1"}="2H49 LI1"; $resn_exch{"H493 LI1"}="3H49 LI1";
$resn_exch{"H501 LI1"}="1H50 LI1"; $resn_exch{"H502 LI1"}="2H50 LI1"; $resn_exch{"H511 LI1"}="1H51 LI1";
$resn_exch{"H512 LI1"}="2H51 LI1"; $resn_exch{"H521 LI1"}="1H52 LI1"; $resn_exch{"H522 LI1"}="2H52 LI1";
$resn_exch{"H531 LI1"}="1H53 LI1"; $resn_exch{"H532 LI1"}="2H53 LI1"; $resn_exch{"H551 LI1"}="1H55 LI1";
$resn_exch{"H552 LI1"}="2H55 LI1"; $resn_exch{"H561 LI1"}="1H56 LI1"; $resn_exch{"H562 LI1"}="2H56 LI1";
$resn_exch{"H571 LI1"}="1H57 LI1"; $resn_exch{"H572 LI1"}="2H57 LI1"; $resn_exch{"H591 LI1"}="1H59 LI1";
$resn_exch{"H592 LI1"}="2H59 LI1"; $resn_exch{"H593 LI1"}="3H59 LI1"; $resn_exch{"H601 LI1"}="1H60 LI1";
$resn_exch{"H602 LI1"}="2H60 LI1"; $resn_exch{"H603 LI1"}="3H60 LI1"; $resn_exch{"H111 LI1"}="1H11 LI1";
$resn_exch{"H112 LI1"}="2H11 LI1"; $resn_exch{"H121 LI1"}="1H12 LI1"; $resn_exch{"H122 LI1"}="2H12 LI1";
$resn_exch{"H141 LI1"}="1H14 LI1"; $resn_exch{"H142 LI1"}="2H14 LI1"; $resn_exch{"H143 LI1"}="3H14 LI1";
$resn_exch{"H151 LI1"}="1H15 LI1"; $resn_exch{"H152 LI1"}="2H15 LI1"; $resn_exch{"H161 LI1"}="1H16 LI1";
$resn_exch{"H162 LI1"}="2H16 LI1"; $resn_exch{"H171 LI1"}="1H17 LI1"; $resn_exch{"H172 LI1"}="2H17 LI1";
$resn_exch{"H191 LI1"}="1H19 LI1"; $resn_exch{"H192 LI1"}="2H19 LI1"; $resn_exch{"H193 LI1"}="3H19 LI1";
$resn_exch{"H201 LI1"}="1H20 LI1"; $resn_exch{"H202 LI1"}="2H20 LI1"; $resn_exch{"H211 LI1"}="1H21 LI1";
$resn_exch{"H212 LI1"}="2H21 LI1"; $resn_exch{"H221 LI1"}="1H22 LI1"; $resn_exch{"H222 LI1"}="2H22 LI1";
$resn_exch{"H241 LI1"}="1H24 LI1"; $resn_exch{"H242 LI1"}="2H24 LI1"; $resn_exch{"H243 LI1"}="3H24 LI1";
$resn_exch{"H251 LI1"}="1H25 LI1"; $resn_exch{"H252 LI1"}="2H25 LI1"; $resn_exch{"H261 LI1"}="1H26 LI1";
$resn_exch{"H262 LI1"}="2H26 LI1"; $resn_exch{"H271 LI1"}="1H27 LI1"; $resn_exch{"H272 LI1"}="2H27 LI1";
$resn_exch{"H291 LI1"}="1H29 LI1"; $resn_exch{"H292 LI1"}="2H29 LI1"; $resn_exch{"H293 LI1"}="3H29 LI1";
$resn_exch{"H301 LI1"}="1H30 LI1"; $resn_exch{"H302 LI1"}="2H30 LI1"; $resn_exch{"H303 LI1"}="3H30 LI1";
$resn_exch{"H191 LI2"}="1H19 LI2"; $resn_exch{"H192 LI2"}="2H19 LI2"; $resn_exch{"H201 LI2"}="1H20 LI2";
$resn_exch{"H202 LI2"}="2H20 LI2"; $resn_exch{"H301 LI2"}="1H30 LI2"; $resn_exch{"H302 LI2"}="2H30 LI2";
$resn_exch{"H311 LI2"}="1H31 LI2"; $resn_exch{"H312 LI2"}="2H31 LI2"; $resn_exch{"H331 LI2"}="1H33 LI2";
$resn_exch{"H332 LI2"}="2H33 LI2"; $resn_exch{"H341 LI2"}="1H34 LI2"; $resn_exch{"H342 LI2"}="2H34 LI2";
$resn_exch{"H221 LI3"}="1H22 LI3"; $resn_exch{"H222 LI3"}="2H22 LI3"; $resn_exch{"H231 LI3"}="1H23 LI3";
$resn_exch{"H232 LI3"}="2H23 LI3"; $resn_exch{"H251 LI3"}="1H25 LI3"; $resn_exch{"H252 LI3"}="2H25 LI3";
$resn_exch{"H261 LI3"}="1H26 LI3"; $resn_exch{"H262 LI3"}="2H26 LI3"; $resn_exch{"H121 LI4"}="1H12 LI4";
$resn_exch{"H122 LI4"}="2H12 LI4"; $resn_exch{"H9C1 LI4"}="1H9C LI4"; $resn_exch{"H9C2 LI4"}="2H9C LI4";
$resn_exch{" H11 LI5"}="1H1  LI5"; $resn_exch{" H12 LI5"}="2H1  LI5"; $resn_exch{" H31 LI5"}="1H3  LI5";
$resn_exch{" H32 LI5"}="2H3  LI5"; $resn_exch{"H311 LI5"}="1H31 LI5"; $resn_exch{"H312 LI5"}="2H31 LI5";
$resn_exch{"H321 LI5"}="1H32 LI5"; $resn_exch{"H322 LI5"}="2H32 LI5"; $resn_exch{"H323 LI5"}="3H32 LI5";
$resn_exch{"H111 LI6"}="1H11 LI6"; $resn_exch{"H112 LI6"}="2H11 LI6"; $resn_exch{"H113 LI6"}="3H11 LI6";
$resn_exch{"H211 LI8"}="1H21 LI8"; $resn_exch{"H212 LI8"}="2H21 LI8"; $resn_exch{"H221 LI8"}="1H22 LI8";
$resn_exch{"H222 LI8"}="2H22 LI8"; $resn_exch{"H231 LI8"}="1H23 LI8"; $resn_exch{"H232 LI8"}="2H23 LI8";
$resn_exch{"H241 LI8"}="1H24 LI8"; $resn_exch{"H242 LI8"}="2H24 LI8"; $resn_exch{"H271 LI8"}="1H27 LI8";
$resn_exch{"H272 LI8"}="2H27 LI8"; $resn_exch{"H281 LI8"}="1H28 LI8"; $resn_exch{"H282 LI8"}="2H28 LI8";
$resn_exch{"H331 LI8"}="1H33 LI8"; $resn_exch{"H332 LI8"}="2H33 LI8"; $resn_exch{"H351 LI8"}="1H35 LI8";
$resn_exch{"H352 LI8"}="2H35 LI8"; $resn_exch{"H353 LI8"}="3H35 LI8"; $resn_exch{"H361 LI8"}="1H36 LI8";
$resn_exch{"H362 LI8"}="2H36 LI8"; $resn_exch{"H363 LI8"}="3H36 LI8"; $resn_exch{"H391 LI8"}="1H39 LI8";
$resn_exch{"H392 LI8"}="2H39 LI8"; $resn_exch{"H121 LI9"}="1H12 LI9"; $resn_exch{"H122 LI9"}="2H12 LI9";
$resn_exch{"H123 LI9"}="3H12 LI9"; $resn_exch{"H181 LI9"}="1H18 LI9"; $resn_exch{"H182 LI9"}="2H18 LI9";
$resn_exch{"H191 LI9"}="1H19 LI9"; $resn_exch{"H192 LI9"}="2H19 LI9"; $resn_exch{"H201 LI9"}="1H20 LI9";
$resn_exch{"H202 LI9"}="2H20 LI9"; $resn_exch{"H211 LI9"}="1H21 LI9"; $resn_exch{"H212 LI9"}="2H21 LI9";
$resn_exch{"H311 LI9"}="1H31 LI9"; $resn_exch{"H312 LI9"}="2H31 LI9"; $resn_exch{"H313 LI9"}="3H31 LI9";
$resn_exch{" H21 LIA"}="1H2  LIA"; $resn_exch{" H22 LIA"}="2H2  LIA"; $resn_exch{" H31 LIA"}="1H3  LIA";
$resn_exch{" H32 LIA"}="2H3  LIA"; $resn_exch{" H51 LIA"}="1H5  LIA"; $resn_exch{" H52 LIA"}="2H5  LIA";
$resn_exch{" H61 LIA"}="1H6  LIA"; $resn_exch{" H62 LIA"}="2H6  LIA"; $resn_exch{"H231 LIA"}="1H23 LIA";
$resn_exch{"H232 LIA"}="2H23 LIA"; $resn_exch{"H233 LIA"}="3H23 LIA"; $resn_exch{"H261 LIA"}="1H26 LIA";
$resn_exch{"H262 LIA"}="2H26 LIA"; $resn_exch{"H281 LIA"}="1H28 LIA"; $resn_exch{"H282 LIA"}="2H28 LIA";
$resn_exch{"H283 LIA"}="3H28 LIA"; $resn_exch{"H141 LIB"}="1H14 LIB"; $resn_exch{"H142 LIB"}="2H14 LIB";
$resn_exch{"H143 LIB"}="3H14 LIB"; $resn_exch{"H431 LIB"}="1H43 LIB"; $resn_exch{"H432 LIB"}="2H43 LIB";
$resn_exch{"H433 LIB"}="3H43 LIB"; $resn_exch{"H471 LIB"}="1H47 LIB"; $resn_exch{"H472 LIB"}="2H47 LIB";
$resn_exch{"H473 LIB"}="3H47 LIB"; $resn_exch{"H461 LIB"}="1H46 LIB"; $resn_exch{"H462 LIB"}="2H46 LIB";
$resn_exch{"H463 LIB"}="3H46 LIB"; $resn_exch{" H71 LIC"}="1H7  LIC"; $resn_exch{" H72 LIC"}="2H7  LIC";
$resn_exch{"H311 LIC"}="1H31 LIC"; $resn_exch{"H312 LIC"}="2H31 LIC"; $resn_exch{"H313 LIC"}="3H31 LIC";
$resn_exch{"H451 LIC"}="1H45 LIC"; $resn_exch{"H452 LIC"}="2H45 LIC"; $resn_exch{"H453 LIC"}="3H45 LIC";
$resn_exch{"H531 LIC"}="1H53 LIC"; $resn_exch{"H532 LIC"}="2H53 LIC"; $resn_exch{"H533 LIC"}="3H53 LIC";
$resn_exch{"H511 LIC"}="1H51 LIC"; $resn_exch{"H512 LIC"}="2H51 LIC"; $resn_exch{"H513 LIC"}="3H51 LIC";
$resn_exch{"H101 LID"}="1H10 LID"; $resn_exch{"H102 LID"}="2H10 LID"; $resn_exch{"H111 LID"}="1H11 LID";
$resn_exch{"H112 LID"}="2H11 LID"; $resn_exch{"H113 LID"}="3H11 LID"; $resn_exch{"H261 LIE"}="1H26 LIE";
$resn_exch{"H262 LIE"}="2H26 LIE"; $resn_exch{"H263 LIE"}="3H26 LIE"; $resn_exch{"H251 LIE"}="1H25 LIE";
$resn_exch{"H252 LIE"}="2H25 LIE"; $resn_exch{"H253 LIE"}="3H25 LIE"; $resn_exch{"H391 LIF"}="1H39 LIF";
$resn_exch{"H392 LIF"}="2H39 LIF"; $resn_exch{"H393 LIF"}="3H39 LIF"; $resn_exch{"H311 LIF"}="1H31 LIF";
$resn_exch{"H312 LIF"}="2H31 LIF"; $resn_exch{"H171 LIG"}="1H17 LIG"; $resn_exch{"H172 LIG"}="2H17 LIG";
$resn_exch{" H71 LIH"}="1H7  LIH"; $resn_exch{"H5A1 LIH"}="1H5A LIH"; $resn_exch{"H5A2 LIH"}="2H5A LIH";
$resn_exch{"H5A3 LIH"}="3H5A LIH"; $resn_exch{"H6'1 LIH"}="1H6' LIH"; $resn_exch{"H7'1 LIH"}="1H7' LIH";
$resn_exch{"H8'1 LIH"}="1H8' LIH"; $resn_exch{" H91 LIH"}="1H9  LIH"; $resn_exch{" H92 LIH"}="2H9  LIH";
$resn_exch{"H4'1 LIH"}="1H4' LIH"; $resn_exch{"H2'1 LIH"}="1H2' LIH"; $resn_exch{"H3'1 LIH"}="1H3' LIH";
$resn_exch{"H101 LIH"}="1H10 LIH"; $resn_exch{" H41 LIH"}="1H4  LIH"; $resn_exch{" H42 LIH"}="2H4  LIH";
$resn_exch{" H21 LIH"}="1H2  LIH"; $resn_exch{" H22 LIH"}="2H2  LIH"; $resn_exch{" H71 LII"}="1H7  LII";
$resn_exch{" H51 LII"}="1H5  LII"; $resn_exch{" H52 LII"}="2H5  LII"; $resn_exch{" H53 LII"}="3H5  LII";
$resn_exch{" H91 LII"}="1H9  LII"; $resn_exch{"H3'1 LII"}="1H3' LII"; $resn_exch{"H4'1 LII"}="1H4' LII";
$resn_exch{"H6'1 LII"}="1H6' LII"; $resn_exch{"H101 LII"}="1H10 LII"; $resn_exch{"H2A1 LII"}="1H2A LII";
$resn_exch{"H2A2 LII"}="2H2A LII"; $resn_exch{"H2A3 LII"}="3H2A LII"; $resn_exch{"H5'1 LII"}="1H5' LII";
$resn_exch{"H5'2 LII"}="2H5' LII"; $resn_exch{"H5'3 LII"}="3H5' LII"; $resn_exch{" H41 LII"}="1H4  LII";
$resn_exch{" H42 LII"}="2H4  LII"; $resn_exch{"H161 LII"}="1H16 LII"; $resn_exch{"H162 LII"}="2H16 LII";
$resn_exch{" H41 LIJ"}="1H4  LIJ"; $resn_exch{" H42 LIJ"}="2H4  LIJ"; $resn_exch{" H43 LIJ"}="3H4  LIJ";
$resn_exch{" H51 LIJ"}="1H5  LIJ"; $resn_exch{" H52 LIJ"}="2H5  LIJ"; $resn_exch{" H61 LIJ"}="1H6  LIJ";
$resn_exch{" H62 LIJ"}="2H6  LIJ"; $resn_exch{"H221 LIJ"}="1H22 LIJ"; $resn_exch{"H222 LIJ"}="2H22 LIJ";
$resn_exch{"H241 LIJ"}="1H24 LIJ"; $resn_exch{"H242 LIJ"}="2H24 LIJ"; $resn_exch{" H11 LIJ"}="1H1  LIJ";
$resn_exch{" H12 LIJ"}="2H1  LIJ"; $resn_exch{"H13A LIJ"}="3H1  LIJ"; $resn_exch{"H21A LIJ"}="1H2  LIJ";
$resn_exch{" H22 LIJ"}="2H2  LIJ"; $resn_exch{" H31 LIJ"}="1H3  LIJ"; $resn_exch{" H32 LIJ"}="2H3  LIJ";
$resn_exch{" H91 LIJ"}="1H9  LIJ"; $resn_exch{" H92 LIJ"}="2H9  LIJ"; $resn_exch{"H141 LIJ"}="1H14 LIJ";
$resn_exch{"H142 LIJ"}="2H14 LIJ"; $resn_exch{"H311 LIJ"}="1H31 LIJ"; $resn_exch{"H312 LIJ"}="2H31 LIJ";
$resn_exch{"H313 LIJ"}="3H31 LIJ"; $resn_exch{"H441 LIJ"}="1H44 LIJ"; $resn_exch{"H442 LIJ"}="2H44 LIJ";
$resn_exch{"H141 LIK"}="1H14 LIK"; $resn_exch{"H142 LIK"}="2H14 LIK"; $resn_exch{"HC11 LIK"}="1HC1 LIK";
$resn_exch{"HC12 LIK"}="2HC1 LIK"; $resn_exch{" H71 LIK"}="1H7  LIK"; $resn_exch{" H72 LIK"}="2H7  LIK";
$resn_exch{" H11 LIK"}="1H1  LIK"; $resn_exch{" H12 LIK"}="2H1  LIK"; $resn_exch{"HC31 LIK"}="1HC3 LIK";
$resn_exch{"HC32 LIK"}="2HC3 LIK"; $resn_exch{" H31 LIK"}="1H3  LIK"; $resn_exch{" H32 LIK"}="2H3  LIK";
$resn_exch{" H41 LIK"}="1H4  LIK"; $resn_exch{" H42 LIK"}="2H4  LIK"; $resn_exch{" H51 LIK"}="1H5  LIK";
$resn_exch{" H52 LIK"}="2H5  LIK"; $resn_exch{" H61 LIK"}="1H6  LIK"; $resn_exch{" H62 LIK"}="2H6  LIK";
$resn_exch{" H63 LIK"}="3H6  LIK"; $resn_exch{"H2E1 LIL"}="1H2E LIL"; $resn_exch{"H2E2 LIL"}="2H2E LIL";
$resn_exch{"H3E1 LIL"}="1H3E LIL"; $resn_exch{"H4E1 LIL"}="1H4E LIL"; $resn_exch{"H4E2 LIL"}="2H4E LIL";
$resn_exch{"H5E1 LIL"}="1H5E LIL"; $resn_exch{"H5E2 LIL"}="2H5E LIL"; $resn_exch{"H6E1 LIL"}="1H6E LIL";
$resn_exch{"H6E2 LIL"}="2H6E LIL"; $resn_exch{"H7E1 LIL"}="1H7E LIL"; $resn_exch{"H7E2 LIL"}="2H7E LIL";
$resn_exch{"H8E1 LIL"}="1H8E LIL"; $resn_exch{"H8E2 LIL"}="2H8E LIL"; $resn_exch{"H9E1 LIL"}="1H9E LIL";
$resn_exch{"H9E2 LIL"}="2H9E LIL"; $resn_exch{"H0S1 LIL"}="1H0S LIL"; $resn_exch{"H0S2 LIL"}="2H0S LIL";
$resn_exch{"H1S1 LIL"}="1H1S LIL"; $resn_exch{"H1S2 LIL"}="2H1S LIL"; $resn_exch{"H2S1 LIL"}="1H2S LIL";
$resn_exch{"H2S2 LIL"}="2H2S LIL"; $resn_exch{"H3S1 LIL"}="1H3S LIL"; $resn_exch{"H3S2 LIL"}="2H3S LIL";
$resn_exch{"H4S1 LIL"}="1H4S LIL"; $resn_exch{"H4S2 LIL"}="2H4S LIL"; $resn_exch{"H5S1 LIL"}="1H5S LIL";
$resn_exch{"H5S2 LIL"}="2H5S LIL"; $resn_exch{"H5S3 LIL"}="3H5S LIL"; $resn_exch{"H2F1 LIL"}="1H2F LIL";
$resn_exch{"H2F2 LIL"}="2H2F LIL"; $resn_exch{"H3F1 LIL"}="1H3F LIL"; $resn_exch{"H3F2 LIL"}="2H3F LIL";
$resn_exch{"H4F1 LIL"}="1H4F LIL"; $resn_exch{"H4F2 LIL"}="2H4F LIL"; $resn_exch{"H5F1 LIL"}="1H5F LIL";
$resn_exch{"H5F2 LIL"}="2H5F LIL"; $resn_exch{"H6F1 LIL"}="1H6F LIL"; $resn_exch{"H6F2 LIL"}="2H6F LIL";
$resn_exch{"H7F1 LIL"}="1H7F LIL"; $resn_exch{"H7F2 LIL"}="2H7F LIL"; $resn_exch{"H8F1 LIL"}="1H8F LIL";
$resn_exch{"H8F2 LIL"}="2H8F LIL"; $resn_exch{"H9F1 LIL"}="1H9F LIL"; $resn_exch{"H9F2 LIL"}="2H9F LIL";
$resn_exch{"H0T1 LIL"}="1H0T LIL"; $resn_exch{"H0T2 LIL"}="2H0T LIL"; $resn_exch{"H1T1 LIL"}="1H1T LIL";
$resn_exch{"H1T2 LIL"}="2H1T LIL"; $resn_exch{"H2T1 LIL"}="1H2T LIL"; $resn_exch{"H2T2 LIL"}="2H2T LIL";
$resn_exch{"H3T1 LIL"}="1H3T LIL"; $resn_exch{"H3T2 LIL"}="2H3T LIL"; $resn_exch{"H3T3 LIL"}="3H3T LIL";
$resn_exch{"H2J1 LIM"}="1H2J LIM"; $resn_exch{"H2J2 LIM"}="2H2J LIM"; $resn_exch{"H4J1 LIM"}="1H4J LIM";
$resn_exch{"H4J2 LIM"}="2H4J LIM"; $resn_exch{"H5J1 LIM"}="1H5J LIM"; $resn_exch{"H5J2 LIM"}="2H5J LIM";
$resn_exch{"H6J1 LIM"}="1H6J LIM"; $resn_exch{"H6J2 LIM"}="2H6J LIM"; $resn_exch{"H7J1 LIM"}="1H7J LIM";
$resn_exch{"H7J2 LIM"}="2H7J LIM"; $resn_exch{"H8J1 LIM"}="1H8J LIM"; $resn_exch{"H8J2 LIM"}="2H8J LIM";
$resn_exch{"H9J1 LIM"}="1H9J LIM"; $resn_exch{"H9J2 LIM"}="2H9J LIM"; $resn_exch{"H0X1 LIM"}="1H0X LIM";
$resn_exch{"H0X2 LIM"}="2H0X LIM"; $resn_exch{"H1X1 LIM"}="1H1X LIM"; $resn_exch{"H1X2 LIM"}="2H1X LIM";
$resn_exch{"H2X1 LIM"}="1H2X LIM"; $resn_exch{"H2X2 LIM"}="2H2X LIM"; $resn_exch{"H3X1 LIM"}="1H3X LIM";
$resn_exch{"H3X2 LIM"}="2H3X LIM"; $resn_exch{"H4X1 LIM"}="1H4X LIM"; $resn_exch{"H4X2 LIM"}="2H4X LIM";
$resn_exch{"H5X1 LIM"}="1H5X LIM"; $resn_exch{"H5X2 LIM"}="2H5X LIM"; $resn_exch{"H5X3 LIM"}="3H5X LIM";
$resn_exch{"H2I1 LIN"}="1H2I LIN"; $resn_exch{"H2I2 LIN"}="2H2I LIN"; $resn_exch{"H4I1 LIN"}="1H4I LIN";
$resn_exch{"H4I2 LIN"}="2H4I LIN"; $resn_exch{"H4I3 LIN"}="3H4I LIN"; $resn_exch{" H11 LIO"}="1H1  LIO";
$resn_exch{" H12 LIO"}="2H1  LIO"; $resn_exch{" H21 LIO"}="1H2  LIO"; $resn_exch{" H31 LIO"}="1H3  LIO";
$resn_exch{" H32 LIO"}="2H3  LIO"; $resn_exch{" H41 LIO"}="1H4  LIO"; $resn_exch{" H42 LIO"}="2H4  LIO";
$resn_exch{" H51 LIO"}="1H5  LIO"; $resn_exch{" H52 LIO"}="2H5  LIO"; $resn_exch{" H61 LIO"}="1H6  LIO";
$resn_exch{" H62 LIO"}="2H6  LIO"; $resn_exch{" H63 LIO"}="3H6  LIO"; $resn_exch{" H71 LIO"}="1H7  LIO";
$resn_exch{" H72 LIO"}="2H7  LIO"; $resn_exch{" H73 LIO"}="3H7  LIO"; $resn_exch{" H81 LIO"}="1H8  LIO";
$resn_exch{" H82 LIO"}="2H8  LIO"; $resn_exch{" H83 LIO"}="3H8  LIO"; $resn_exch{"H121 LIO"}="1H12 LIO";
$resn_exch{"H122 LIO"}="2H12 LIO"; $resn_exch{"H131 LIO"}="1H13 LIO"; $resn_exch{"H132 LIO"}="2H13 LIO";
$resn_exch{"H141 LIO"}="1H14 LIO"; $resn_exch{"H142 LIO"}="2H14 LIO"; $resn_exch{"H151 LIO"}="1H15 LIO";
$resn_exch{"H152 LIO"}="2H15 LIO"; $resn_exch{"H161 LIO"}="1H16 LIO"; $resn_exch{"H162 LIO"}="2H16 LIO";
$resn_exch{"H171 LIO"}="1H17 LIO"; $resn_exch{"H172 LIO"}="2H17 LIO"; $resn_exch{"H181 LIO"}="1H18 LIO";
$resn_exch{"H182 LIO"}="2H18 LIO"; $resn_exch{"H191 LIO"}="1H19 LIO"; $resn_exch{"H192 LIO"}="2H19 LIO";
$resn_exch{"H201 LIO"}="1H20 LIO"; $resn_exch{"H202 LIO"}="2H20 LIO"; $resn_exch{"H211 LIO"}="1H21 LIO";
$resn_exch{"H212 LIO"}="2H21 LIO"; $resn_exch{"H221 LIO"}="1H22 LIO"; $resn_exch{"H222 LIO"}="2H22 LIO";
$resn_exch{"H231 LIO"}="1H23 LIO"; $resn_exch{"H232 LIO"}="2H23 LIO"; $resn_exch{"H241 LIO"}="1H24 LIO";
$resn_exch{"H242 LIO"}="2H24 LIO"; $resn_exch{"H251 LIO"}="1H25 LIO"; $resn_exch{"H252 LIO"}="2H25 LIO";
$resn_exch{"H253 LIO"}="3H25 LIO"; $resn_exch{"H321 LIO"}="1H32 LIO"; $resn_exch{"H322 LIO"}="2H32 LIO";
$resn_exch{"H331 LIO"}="1H33 LIO"; $resn_exch{"H332 LIO"}="2H33 LIO"; $resn_exch{"H341 LIO"}="1H34 LIO";
$resn_exch{"H342 LIO"}="2H34 LIO"; $resn_exch{"H351 LIO"}="1H35 LIO"; $resn_exch{"H352 LIO"}="2H35 LIO";
$resn_exch{"H361 LIO"}="1H36 LIO"; $resn_exch{"H362 LIO"}="2H36 LIO"; $resn_exch{"H371 LIO"}="1H37 LIO";
$resn_exch{"H372 LIO"}="2H37 LIO"; $resn_exch{"H381 LIO"}="1H38 LIO"; $resn_exch{"H382 LIO"}="2H38 LIO";
$resn_exch{"H391 LIO"}="1H39 LIO"; $resn_exch{"H392 LIO"}="2H39 LIO"; $resn_exch{"H401 LIO"}="1H40 LIO";
$resn_exch{"H402 LIO"}="2H40 LIO"; $resn_exch{"H403 LIO"}="3H40 LIO"; $resn_exch{"HOP1 LIO"}="1HOP LIO";
$resn_exch{" H71 LIQ"}="1H7  LIQ"; $resn_exch{" H72 LIQ"}="2H7  LIQ"; $resn_exch{"H221 LIQ"}="1H22 LIQ";
$resn_exch{"H222 LIQ"}="2H22 LIQ"; $resn_exch{"H241 LIQ"}="1H24 LIQ"; $resn_exch{"H242 LIQ"}="2H24 LIQ";
$resn_exch{"H281 LIQ"}="1H28 LIQ"; $resn_exch{"H282 LIQ"}="2H28 LIQ"; $resn_exch{" H11 LIQ"}="1H1  LIQ";
$resn_exch{" H12 LIQ"}="2H1  LIQ"; $resn_exch{"H1A1 LIQ"}="1H1A LIQ"; $resn_exch{"H1A2 LIQ"}="2H1A LIQ";
$resn_exch{"H1A3 LIQ"}="3H1A LIQ"; $resn_exch{"H21A LIQ"}="1H2  LIQ"; $resn_exch{" H22 LIQ"}="2H2  LIQ";
$resn_exch{"H31A LIQ"}="1H3  LIQ"; $resn_exch{" H32 LIQ"}="2H3  LIQ"; $resn_exch{" H91 LIQ"}="1H9  LIQ";
$resn_exch{" H92 LIQ"}="2H9  LIQ"; $resn_exch{"H141 LIQ"}="1H14 LIQ"; $resn_exch{"H142 LIQ"}="2H14 LIQ";
$resn_exch{"H321 LIQ"}="1H32 LIQ"; $resn_exch{"H322 LIQ"}="2H32 LIQ"; $resn_exch{"H323 LIQ"}="3H32 LIQ";
$resn_exch{"H131 LIR"}="1H13 LIR"; $resn_exch{"H132 LIR"}="2H13 LIR"; $resn_exch{"H151 LIR"}="1H15 LIR";
$resn_exch{"H152 LIR"}="2H15 LIR"; $resn_exch{"H171 LIR"}="1H17 LIR"; $resn_exch{"H172 LIR"}="2H17 LIR";
$resn_exch{"H111 LIR"}="1H11 LIR"; $resn_exch{"H112 LIR"}="2H11 LIR"; $resn_exch{"H113 LIR"}="3H11 LIR";
$resn_exch{"H161 LIR"}="1H16 LIR"; $resn_exch{"H162 LIR"}="2H16 LIR"; $resn_exch{"H181 LIR"}="1H18 LIR";
$resn_exch{"H182 LIR"}="2H18 LIR"; $resn_exch{"H191 LIR"}="1H19 LIR"; $resn_exch{"H192 LIR"}="2H19 LIR";
$resn_exch{" HN1 LIS"}="1HN  LIS"; $resn_exch{" HN2 LIS"}="2HN  LIS"; $resn_exch{"HBC1 LIS"}="1HBC LIS";
$resn_exch{"HBC2 LIS"}="2HBC LIS"; $resn_exch{"HDC1 LIS"}="1HDC LIS"; $resn_exch{"HDC2 LIS"}="2HDC LIS";
$resn_exch{"H101 LIT"}="1H10 LIT"; $resn_exch{"H102 LIT"}="2H10 LIT"; $resn_exch{"H11A LIU"}="1H1  LIU";
$resn_exch{" H12 LIU"}="2H1  LIU"; $resn_exch{"H31A LIU"}="1H3  LIU"; $resn_exch{"H32A LIU"}="2H3  LIU";
$resn_exch{" H41 LIU"}="1H4  LIU"; $resn_exch{" H42 LIU"}="2H4  LIU"; $resn_exch{" H61 LIU"}="1H6  LIU";
$resn_exch{" H62 LIU"}="2H6  LIU"; $resn_exch{" H71 LIU"}="1H7  LIU"; $resn_exch{" H72 LIU"}="2H7  LIU";
$resn_exch{" H73 LIU"}="3H7  LIU"; $resn_exch{" H81 LIU"}="1H8  LIU"; $resn_exch{" H82 LIU"}="2H8  LIU";
$resn_exch{" H83 LIU"}="3H8  LIU"; $resn_exch{"H271 LIU"}="1H27 LIU"; $resn_exch{"H272 LIU"}="2H27 LIU";
$resn_exch{"H401 LIU"}="1H40 LIU"; $resn_exch{"H402 LIU"}="2H40 LIU"; $resn_exch{"H403 LIU"}="3H40 LIU";
$resn_exch{"H411 LIU"}="1H41 LIU"; $resn_exch{"H412 LIU"}="2H41 LIU"; $resn_exch{"H413 LIU"}="3H41 LIU";
$resn_exch{"H211 LIV"}="1H21 LIV"; $resn_exch{"H212 LIV"}="2H21 LIV"; $resn_exch{"H311 LIV"}="1H31 LIV";
$resn_exch{"H312 LIV"}="2H31 LIV"; $resn_exch{"H611 LIV"}="1H61 LIV"; $resn_exch{"H612 LIV"}="2H61 LIV";
$resn_exch{"H121 LIV"}="1H12 LIV"; $resn_exch{"H122 LIV"}="2H12 LIV"; $resn_exch{"H321 LIV"}="1H32 LIV";
$resn_exch{"H322 LIV"}="2H32 LIV"; $resn_exch{"H221 LIV"}="1H22 LIV"; $resn_exch{"H222 LIV"}="2H22 LIV";
$resn_exch{"H531 LIV"}="1H53 LIV"; $resn_exch{"H532 LIV"}="2H53 LIV"; $resn_exch{"H241 LIV"}="1H24 LIV";
$resn_exch{"H242 LIV"}="2H24 LIV"; $resn_exch{"H11A LIV"}="1H1  LIV"; $resn_exch{"H12A LIV"}="2H1  LIV";
$resn_exch{"H641 LIV"}="1H64 LIV"; $resn_exch{"H642 LIV"}="2H64 LIV"; $resn_exch{"H651 LIV"}="1H65 LIV";
$resn_exch{"H652 LIV"}="2H65 LIV"; $resn_exch{" H81 LIW"}="1H8  LIW"; $resn_exch{" H82 LIW"}="2H8  LIW";
$resn_exch{" H91 LIW"}="1H9  LIW"; $resn_exch{" H92 LIW"}="2H9  LIW"; $resn_exch{"H101 LIW"}="1H10 LIW";
$resn_exch{"H102 LIW"}="2H10 LIW"; $resn_exch{"H111 LIW"}="1H11 LIW"; $resn_exch{"H112 LIW"}="2H11 LIW";
$resn_exch{"H121 LIW"}="1H12 LIW"; $resn_exch{"H122 LIW"}="2H12 LIW"; $resn_exch{"H191 LIW"}="1H19 LIW";
$resn_exch{"H192 LIW"}="2H19 LIW"; $resn_exch{"H193 LIW"}="3H19 LIW"; $resn_exch{"H201 LIW"}="1H20 LIW";
$resn_exch{"H202 LIW"}="2H20 LIW"; $resn_exch{"H271 LIW"}="1H27 LIW"; $resn_exch{"H272 LIW"}="2H27 LIW";
$resn_exch{"H273 LIW"}="3H27 LIW"; $resn_exch{" H51 LIX"}="1H5  LIX"; $resn_exch{" H52 LIX"}="2H5  LIX";
$resn_exch{" H61 LIX"}="1H6  LIX"; $resn_exch{" H62 LIX"}="2H6  LIX"; $resn_exch{" H63 LIX"}="3H6  LIX";
$resn_exch{"H131 LIX"}="1H13 LIX"; $resn_exch{"H132 LIX"}="2H13 LIX"; $resn_exch{"H133 LIX"}="3H13 LIX";
$resn_exch{"H171 LIX"}="1H17 LIX"; $resn_exch{"H172 LIX"}="2H17 LIX"; $resn_exch{"H201 LIX"}="1H20 LIX";
$resn_exch{"H202 LIX"}="2H20 LIX"; $resn_exch{"H211 LIX"}="1H21 LIX"; $resn_exch{"H212 LIX"}="2H21 LIX";
$resn_exch{"H221 LIX"}="1H22 LIX"; $resn_exch{"H222 LIX"}="2H22 LIX"; $resn_exch{"H223 LIX"}="3H22 LIX";
$resn_exch{"HN41 LIX"}="1HN4 LIX"; $resn_exch{"HN42 LIX"}="2HN4 LIX"; $resn_exch{"H161 LIX"}="1H16 LIX";
$resn_exch{"H162 LIX"}="2H16 LIX"; $resn_exch{"H181 LIX"}="1H18 LIX"; $resn_exch{"H182 LIX"}="2H18 LIX";
$resn_exch{"H191 LIX"}="1H19 LIX"; $resn_exch{"H192 LIX"}="2H19 LIX"; $resn_exch{"H193 LIX"}="3H19 LIX";
$resn_exch{"HN31 LIY"}="1HN3 LIY"; $resn_exch{"HN32 LIY"}="2HN3 LIY"; $resn_exch{" H61 LIY"}="1H6  LIY";
$resn_exch{" H62 LIY"}="2H6  LIY"; $resn_exch{"H131 LIY"}="1H13 LIY"; $resn_exch{"H132 LIY"}="2H13 LIY";
$resn_exch{"H171 LIY"}="1H17 LIY"; $resn_exch{"H172 LIY"}="2H17 LIY"; $resn_exch{"H201 LIY"}="1H20 LIY";
$resn_exch{"H202 LIY"}="2H20 LIY"; $resn_exch{"H203 LIY"}="3H20 LIY"; $resn_exch{"HN41 LIY"}="1HN4 LIY";
$resn_exch{"HN42 LIY"}="2HN4 LIY"; $resn_exch{"H141 LIY"}="1H14 LIY"; $resn_exch{"H142 LIY"}="2H14 LIY";
$resn_exch{"H161 LIY"}="1H16 LIY"; $resn_exch{"H162 LIY"}="2H16 LIY"; $resn_exch{"H181 LIY"}="1H18 LIY";
$resn_exch{"H182 LIY"}="2H18 LIY"; $resn_exch{"H191 LIY"}="1H19 LIY"; $resn_exch{"H192 LIY"}="2H19 LIY";
$resn_exch{"H193 LIY"}="3H19 LIY"; $resn_exch{"HBC1 LK1"}="1HBC LK1"; $resn_exch{"HBC2 LK1"}="2HBC LK1";
$resn_exch{"HGC1 LK1"}="1HGC LK1"; $resn_exch{"HGC2 LK1"}="2HGC LK1"; $resn_exch{"H231 LK1"}="1H23 LK1";
$resn_exch{"H232 LK1"}="2H23 LK1"; $resn_exch{"H241 LK1"}="1H24 LK1"; $resn_exch{"H242 LK1"}="2H24 LK1";
$resn_exch{"H251 LK1"}="1H25 LK1"; $resn_exch{"H252 LK1"}="2H25 LK1"; $resn_exch{"H261 LK1"}="1H26 LK1";
$resn_exch{"H262 LK1"}="2H26 LK1"; $resn_exch{"H263 LK1"}="3H26 LK1"; $resn_exch{"HBC1 LK2"}="1HBC LK2";
$resn_exch{"HBC2 LK2"}="2HBC LK2"; $resn_exch{"HGC1 LK2"}="1HGC LK2"; $resn_exch{"HGC2 LK2"}="2HGC LK2";
$resn_exch{"H231 LK2"}="1H23 LK2"; $resn_exch{"H232 LK2"}="2H23 LK2"; $resn_exch{"H241 LK2"}="1H24 LK2";
$resn_exch{"H242 LK2"}="2H24 LK2"; $resn_exch{"H291 LK2"}="1H29 LK2"; $resn_exch{"H292 LK2"}="2H29 LK2";
$resn_exch{"H441 LK2"}="1H44 LK2"; $resn_exch{"H442 LK2"}="2H44 LK2"; $resn_exch{"H443 LK2"}="3H44 LK2";
$resn_exch{"HBC1 LK3"}="1HBC LK3"; $resn_exch{"HBC2 LK3"}="2HBC LK3"; $resn_exch{"HGC1 LK3"}="1HGC LK3";
$resn_exch{"HGC2 LK3"}="2HGC LK3"; $resn_exch{"H231 LK3"}="1H23 LK3"; $resn_exch{"H232 LK3"}="2H23 LK3";
$resn_exch{"H241 LK3"}="1H24 LK3"; $resn_exch{"H242 LK3"}="2H24 LK3"; $resn_exch{"H251 LK3"}="1H25 LK3";
$resn_exch{"H252 LK3"}="2H25 LK3"; $resn_exch{"H261 LK3"}="1H26 LK3"; $resn_exch{"H262 LK3"}="2H26 LK3";
$resn_exch{"H301 LK3"}="1H30 LK3"; $resn_exch{"H302 LK3"}="2H30 LK3"; $resn_exch{"H303 LK3"}="3H30 LK3";
$resn_exch{"H231 LK4"}="1H23 LK4"; $resn_exch{"H232 LK4"}="2H23 LK4"; $resn_exch{"HBC1 LK4"}="1HBC LK4";
$resn_exch{"HBC2 LK4"}="2HBC LK4"; $resn_exch{"HGC1 LK4"}="1HGC LK4"; $resn_exch{"HGC2 LK4"}="2HGC LK4";
$resn_exch{" H11 LK5"}="1H1  LK5"; $resn_exch{" H12 LK5"}="2H1  LK5"; $resn_exch{" H13 LK5"}="3H1  LK5";
$resn_exch{" H31 LK5"}="1H3  LK5"; $resn_exch{" H32 LK5"}="2H3  LK5"; $resn_exch{" H41 LK5"}="1H4  LK5";
$resn_exch{" H42 LK5"}="2H4  LK5"; $resn_exch{" H51 LK5"}="1H5  LK5"; $resn_exch{" H52 LK5"}="2H5  LK5";
$resn_exch{" H71 LK5"}="1H7  LK5"; $resn_exch{" H72 LK5"}="2H7  LK5"; $resn_exch{" H81 LK5"}="1H8  LK5";
$resn_exch{" H82 LK5"}="2H8  LK5"; $resn_exch{" H91 LK5"}="1H9  LK5"; $resn_exch{" H92 LK5"}="2H9  LK5";
$resn_exch{"H161 LK5"}="1H16 LK5"; $resn_exch{"H162 LK5"}="2H16 LK5"; $resn_exch{" H21 LK5"}="1H2  LK5";
$resn_exch{" H22 LK5"}="2H2  LK5"; $resn_exch{" H23 LK5"}="3H2  LK5"; $resn_exch{" H11 LK7"}="1H1  LK7";
$resn_exch{" H12 LK7"}="2H1  LK7"; $resn_exch{"H13A LK7"}="3H1  LK7"; $resn_exch{"H101 LK7"}="1H10 LK7";
$resn_exch{"H102 LK7"}="2H10 LK7"; $resn_exch{"H111 LK7"}="1H11 LK7"; $resn_exch{"H112 LK7"}="2H11 LK7";
$resn_exch{"H121 LK7"}="1H12 LK7"; $resn_exch{"H122 LK7"}="2H12 LK7"; $resn_exch{"H141 LK7"}="1H14 LK7";
$resn_exch{"H142 LK7"}="2H14 LK7"; $resn_exch{"H143 LK7"}="3H14 LK7"; $resn_exch{"HN11 LKA"}="1HN1 LKA";
$resn_exch{"HN12 LKA"}="2HN1 LKA"; $resn_exch{" H61 LKA"}="1H6  LKA"; $resn_exch{" H62 LKA"}="2H6  LKA";
$resn_exch{" H41 LKC"}="1H4  LKC"; $resn_exch{" H42 LKC"}="2H4  LKC"; $resn_exch{" H1' LKC"}=" H1* LKC";
$resn_exch{"H2'1 LKC"}="1H2* LKC"; $resn_exch{" H3' LKC"}=" H3* LKC"; $resn_exch{"H5'1 LKC"}="1H5* LKC";
$resn_exch{"H5'2 LKC"}="2H5* LKC"; $resn_exch{"H5M1 LKC"}="1H5M LKC"; $resn_exch{"H5M2 LKC"}="2H5M LKC";
$resn_exch{"H5M3 LKC"}="3H5M LKC"; $resn_exch{"H6'1 LKC"}="1H6* LKC"; $resn_exch{"H6'2 LKC"}="2H6* LKC";
$resn_exch{" H61 LKS"}="1H6  LKS"; $resn_exch{" H62 LKS"}="2H6  LKS"; $resn_exch{" H81 LKS"}="1H8  LKS";
$resn_exch{" H82 LKS"}="2H8  LKS"; $resn_exch{" H71 LLB"}="1H7  LLB"; $resn_exch{" H72 LLB"}="2H7  LLB";
$resn_exch{" H73 LLB"}="3H7  LLB"; $resn_exch{"H281 LLB"}="1H28 LLB"; $resn_exch{"H282 LLB"}="2H28 LLB";
$resn_exch{"H291 LLB"}="1H29 LLB"; $resn_exch{"H292 LLB"}="2H29 LLB"; $resn_exch{"H661 LLB"}="1H66 LLB";
$resn_exch{"H662 LLB"}="2H66 LLB"; $resn_exch{"H651 LLB"}="1H65 LLB"; $resn_exch{"H652 LLB"}="2H65 LLB";
$resn_exch{"H631 LLB"}="1H63 LLB"; $resn_exch{"H632 LLB"}="2H63 LLB"; $resn_exch{"H621 LLB"}="1H62 LLB";
$resn_exch{"H622 LLB"}="2H62 LLB"; $resn_exch{"H261 LLC"}="1H26 LLC"; $resn_exch{"H262 LLC"}="2H26 LLC";
$resn_exch{"H281 LLC"}="1H28 LLC"; $resn_exch{"H282 LLC"}="2H28 LLC"; $resn_exch{"H251 LLC"}="1H25 LLC";
$resn_exch{"H252 LLC"}="2H25 LLC"; $resn_exch{"H311 LLC"}="1H31 LLC"; $resn_exch{"H312 LLC"}="2H31 LLC";
$resn_exch{"H301 LLC"}="1H30 LLC"; $resn_exch{"H302 LLC"}="2H30 LLC"; $resn_exch{"H291 LLC"}="1H29 LLC";
$resn_exch{"H292 LLC"}="2H29 LLC"; $resn_exch{"HAF1 LLF"}="1HAF LLF"; $resn_exch{"HAF2 LLF"}="2HAF LLF";
$resn_exch{"H2A1 LLF"}="1H2A LLF"; $resn_exch{"H2A2 LLF"}="2H2A LLF"; $resn_exch{"H2A3 LLF"}="3H2A LLF";
$resn_exch{"H5M1 LLF"}="1H5M LLF"; $resn_exch{"H5M2 LLF"}="2H5M LLF"; $resn_exch{"H211 LLL"}="1H21 LLL";
$resn_exch{"H212 LLL"}="2H21 LLL"; $resn_exch{"H311 LLL"}="1H31 LLL"; $resn_exch{"H312 LLL"}="2H31 LLL";
$resn_exch{"H411 LLL"}="1H41 LLL"; $resn_exch{"H412 LLL"}="2H41 LLL"; $resn_exch{"H611 LLL"}="1H61 LLL";
$resn_exch{"H612 LLL"}="2H61 LLL"; $resn_exch{"H11A LLL"}="1H1  LLL"; $resn_exch{"H12A LLL"}="2H1  LLL";
$resn_exch{"H121 LLL"}="1H12 LLL"; $resn_exch{"H122 LLL"}="2H12 LLL"; $resn_exch{"H221 LLL"}="1H22 LLL";
$resn_exch{"H222 LLL"}="2H22 LLL"; $resn_exch{"H321 LLL"}="1H32 LLL"; $resn_exch{"H322 LLL"}="2H32 LLL";
$resn_exch{"H531 LLL"}="1H53 LLL"; $resn_exch{"H532 LLL"}="2H53 LLL"; $resn_exch{"H831 LLL"}="1H83 LLL";
$resn_exch{"H832 LLL"}="2H83 LLL"; $resn_exch{"H833 LLL"}="3H83 LLL"; $resn_exch{"H931 LLL"}="1H93 LLL";
$resn_exch{"H932 LLL"}="2H93 LLL"; $resn_exch{"H933 LLL"}="3H93 LLL"; $resn_exch{"H2'1 LLP"}="1H2A LLP";
$resn_exch{"H2'2 LLP"}="2H2A LLP"; $resn_exch{"H2'3 LLP"}="3H2A LLP"; $resn_exch{"H4'1 LLP"}="1H4A LLP";
$resn_exch{"H4'2 LLP"}="2H4A LLP"; $resn_exch{"H5'1 LLP"}="1H5A LLP"; $resn_exch{"H5'2 LLP"}="2H5A LLP";
$resn_exch{"HOP2 LLP"}="2HOP LLP"; $resn_exch{"HOP3 LLP"}="3HOP LLP"; $resn_exch{" H   LLP"}="1HN  LLP";
$resn_exch{" HN2 LLP"}="2HN  LLP"; $resn_exch{" HB2 LLP"}="1HB  LLP"; $resn_exch{" HB3 LLP"}="2HB  LLP";
$resn_exch{" HG2 LLP"}="1HG  LLP"; $resn_exch{" HG3 LLP"}="2HG  LLP"; $resn_exch{" HD2 LLP"}="1HD  LLP";
$resn_exch{" HD3 LLP"}="2HD  LLP"; $resn_exch{" HE2 LLP"}="1HE  LLP"; $resn_exch{" HE3 LLP"}="2HE  LLP";
$resn_exch{" HZ1 LLP"}=" HNZ LLP"; $resn_exch{" H   LLY"}="1HN  LLY"; $resn_exch{" H2  LLY"}="2HN  LLY";
$resn_exch{" HB2 LLY"}="1HB  LLY"; $resn_exch{" HB3 LLY"}="2HB  LLY"; $resn_exch{" HG2 LLY"}="1HG  LLY";
$resn_exch{" HG3 LLY"}="2HG  LLY"; $resn_exch{" HD2 LLY"}="1HD  LLY"; $resn_exch{" HD3 LLY"}="2HD  LLY";
$resn_exch{" HE2 LLY"}="1HE  LLY"; $resn_exch{" HE3 LLY"}="2HE  LLY"; $resn_exch{" HZ  LLY"}=" HNZ LLY";
$resn_exch{"HA61 LM2"}="1HA6 LM2"; $resn_exch{"HA62 LM2"}="2HA6 LM2"; $resn_exch{"HB61 LM2"}="1HB6 LM2";
$resn_exch{"HB62 LM2"}="2HB6 LM2"; $resn_exch{"HD61 LM2"}="1HD6 LM2"; $resn_exch{"HD62 LM2"}="2HD6 LM2";
$resn_exch{"HD71 LM2"}="1HD7 LM2"; $resn_exch{"HD72 LM2"}="2HD7 LM2"; $resn_exch{"HD73 LM2"}="3HD7 LM2";
$resn_exch{" H2  LME"}=" HN2 LME"; $resn_exch{" H   LME"}=" HN  LME"; $resn_exch{" H11 LME"}="1H1  LME";
$resn_exch{" H12 LME"}="2H1  LME"; $resn_exch{" H13 LME"}="3H1  LME"; $resn_exch{" HG2 LME"}="1HG  LME";
$resn_exch{" HG3 LME"}="2HG  LME"; $resn_exch{"HC61 LMG"}="1HC6 LMG"; $resn_exch{"HC62 LMG"}="2HC6 LMG";
$resn_exch{"HC71 LMG"}="1HC7 LMG"; $resn_exch{"HC72 LMG"}="2HC7 LMG"; $resn_exch{"HC91 LMG"}="1HC9 LMG";
$resn_exch{"HC92 LMG"}="2HC9 LMG"; $resn_exch{"H111 LMG"}="1H11 LMG"; $resn_exch{"H112 LMG"}="2H11 LMG";
$resn_exch{"H121 LMG"}="1H12 LMG"; $resn_exch{"H122 LMG"}="2H12 LMG"; $resn_exch{"H131 LMG"}="1H13 LMG";
$resn_exch{"H132 LMG"}="2H13 LMG"; $resn_exch{"H141 LMG"}="1H14 LMG"; $resn_exch{"H142 LMG"}="2H14 LMG";
$resn_exch{"H151 LMG"}="1H15 LMG"; $resn_exch{"H152 LMG"}="2H15 LMG"; $resn_exch{"H161 LMG"}="1H16 LMG";
$resn_exch{"H162 LMG"}="2H16 LMG"; $resn_exch{"H171 LMG"}="1H17 LMG"; $resn_exch{"H172 LMG"}="2H17 LMG";
$resn_exch{"H181 LMG"}="1H18 LMG"; $resn_exch{"H182 LMG"}="2H18 LMG"; $resn_exch{"H191 LMG"}="1H19 LMG";
$resn_exch{"H192 LMG"}="2H19 LMG"; $resn_exch{"H201 LMG"}="1H20 LMG"; $resn_exch{"H202 LMG"}="2H20 LMG";
$resn_exch{"H211 LMG"}="1H21 LMG"; $resn_exch{"H212 LMG"}="2H21 LMG"; $resn_exch{"H221 LMG"}="1H22 LMG";
$resn_exch{"H222 LMG"}="2H22 LMG"; $resn_exch{"H231 LMG"}="1H23 LMG"; $resn_exch{"H232 LMG"}="2H23 LMG";
$resn_exch{"H241 LMG"}="1H24 LMG"; $resn_exch{"H242 LMG"}="2H24 LMG"; $resn_exch{"H251 LMG"}="1H25 LMG";
$resn_exch{"H252 LMG"}="2H25 LMG"; $resn_exch{"H261 LMG"}="1H26 LMG"; $resn_exch{"H262 LMG"}="2H26 LMG";
$resn_exch{"H271 LMG"}="1H27 LMG"; $resn_exch{"H272 LMG"}="2H27 LMG"; $resn_exch{"H273 LMG"}="3H27 LMG";
$resn_exch{"H291 LMG"}="1H29 LMG"; $resn_exch{"H292 LMG"}="2H29 LMG"; $resn_exch{"H301 LMG"}="1H30 LMG";
$resn_exch{"H302 LMG"}="2H30 LMG"; $resn_exch{"H311 LMG"}="1H31 LMG"; $resn_exch{"H312 LMG"}="2H31 LMG";
$resn_exch{"H321 LMG"}="1H32 LMG"; $resn_exch{"H322 LMG"}="2H32 LMG"; $resn_exch{"H331 LMG"}="1H33 LMG";
$resn_exch{"H332 LMG"}="2H33 LMG"; $resn_exch{"H341 LMG"}="1H34 LMG"; $resn_exch{"H342 LMG"}="2H34 LMG";
$resn_exch{"H351 LMG"}="1H35 LMG"; $resn_exch{"H352 LMG"}="2H35 LMG"; $resn_exch{"H361 LMG"}="1H36 LMG";
$resn_exch{"H362 LMG"}="2H36 LMG"; $resn_exch{"H371 LMG"}="1H37 LMG"; $resn_exch{"H372 LMG"}="2H37 LMG";
$resn_exch{"H381 LMG"}="1H38 LMG"; $resn_exch{"H382 LMG"}="2H38 LMG"; $resn_exch{"H391 LMG"}="1H39 LMG";
$resn_exch{"H392 LMG"}="2H39 LMG"; $resn_exch{"H401 LMG"}="1H40 LMG"; $resn_exch{"H402 LMG"}="2H40 LMG";
$resn_exch{"H411 LMG"}="1H41 LMG"; $resn_exch{"H412 LMG"}="2H41 LMG"; $resn_exch{"H421 LMG"}="1H42 LMG";
$resn_exch{"H422 LMG"}="2H42 LMG"; $resn_exch{"H431 LMG"}="1H43 LMG"; $resn_exch{"H432 LMG"}="2H43 LMG";
$resn_exch{"H441 LMG"}="1H44 LMG"; $resn_exch{"H442 LMG"}="2H44 LMG"; $resn_exch{"H451 LMG"}="1H45 LMG";
$resn_exch{"H452 LMG"}="2H45 LMG"; $resn_exch{"H453 LMG"}="3H45 LMG"; $resn_exch{" HB1 LML"}="1HB  LML";
$resn_exch{" HB2 LML"}="2HB  LML"; $resn_exch{"HD11 LML"}="1HD1 LML"; $resn_exch{"HD12 LML"}="2HD1 LML";
$resn_exch{"HD13 LML"}="3HD1 LML"; $resn_exch{"HD21 LML"}="1HD2 LML"; $resn_exch{"HD22 LML"}="2HD2 LML";
$resn_exch{"HD23 LML"}="3HD2 LML"; $resn_exch{" O3' LMS"}=" O3* LMS"; $resn_exch{" C2' LMS"}=" C2* LMS";
$resn_exch{" O2' LMS"}=" O2* LMS"; $resn_exch{" C1' LMS"}=" C1* LMS"; $resn_exch{"H5'1 LMS"}="1H5* LMS";
$resn_exch{"H5'2 LMS"}="2H5* LMS"; $resn_exch{" H4' LMS"}=" H4* LMS"; $resn_exch{" H3' LMS"}=" H3* LMS";
$resn_exch{" H2' LMS"}=" H2* LMS"; $resn_exch{" H1' LMS"}=" H1* LMS"; $resn_exch{"H6N1 LMS"}="1H6N LMS";
$resn_exch{"H6N2 LMS"}="2H6N LMS"; $resn_exch{" HN1 LMS"}="1HN  LMS"; $resn_exch{" HN2 LMS"}="2HN  LMS";
$resn_exch{" H1B LMT"}=" H1* LMT"; $resn_exch{" H2B LMT"}=" H2* LMT"; $resn_exch{" H3B LMT"}=" H3* LMT";
$resn_exch{" H4B LMT"}=" H4* LMT"; $resn_exch{" H5B LMT"}=" H5* LMT"; $resn_exch{"H6'2 LMT"}="2H6* LMT";
$resn_exch{"H6'1 LMT"}="1H6* LMT"; $resn_exch{"H2O1 LMT"}="1H2O LMT"; $resn_exch{"H3O1 LMT"}="1H3O LMT";
$resn_exch{"H4O1 LMT"}="1H4O LMT"; $resn_exch{" H6B LMT"}=" H6* LMT"; $resn_exch{" H6D LMT"}="2H6' LMT";
$resn_exch{" H6E LMT"}="1H6' LMT"; $resn_exch{"H2O2 LMT"}="2H2O LMT"; $resn_exch{"H3O2 LMT"}="2H3O LMT";
$resn_exch{" H12 LMT"}="2H1  LMT"; $resn_exch{" H11 LMT"}="1H1  LMT"; $resn_exch{" H22 LMT"}="2H2  LMT";
$resn_exch{" H21 LMT"}="1H2  LMT"; $resn_exch{" H32 LMT"}="2H3  LMT"; $resn_exch{" H31 LMT"}="1H3  LMT";
$resn_exch{" H42 LMT"}="2H4  LMT"; $resn_exch{" H41 LMT"}="1H4  LMT"; $resn_exch{" H52 LMT"}="2H5  LMT";
$resn_exch{" H51 LMT"}="1H5  LMT"; $resn_exch{" H62 LMT"}="2H6  LMT"; $resn_exch{" H61 LMT"}="1H6  LMT";
$resn_exch{" H72 LMT"}="2H7  LMT"; $resn_exch{" H71 LMT"}="1H7  LMT"; $resn_exch{" H82 LMT"}="2H8  LMT";
$resn_exch{" H81 LMT"}="1H8  LMT"; $resn_exch{" H92 LMT"}="2H9  LMT"; $resn_exch{" H91 LMT"}="1H9  LMT";
$resn_exch{"H102 LMT"}="2H10 LMT"; $resn_exch{"H101 LMT"}="1H10 LMT"; $resn_exch{"H112 LMT"}="2H11 LMT";
$resn_exch{"H111 LMT"}="1H11 LMT"; $resn_exch{"H123 LMT"}="3H12 LMT"; $resn_exch{"H122 LMT"}="2H12 LMT";
$resn_exch{"H121 LMT"}="1H12 LMT"; $resn_exch{" H1B LMU"}=" H1* LMU"; $resn_exch{" H2B LMU"}=" H2* LMU";
$resn_exch{" H3B LMU"}=" H3* LMU"; $resn_exch{" H4B LMU"}=" H4* LMU"; $resn_exch{" H5B LMU"}=" H5* LMU";
$resn_exch{"H6'2 LMU"}="2H6* LMU"; $resn_exch{"H6'1 LMU"}="1H6* LMU"; $resn_exch{"H2O1 LMU"}="1H2O LMU";
$resn_exch{"H3O1 LMU"}="1H3O LMU"; $resn_exch{"H4O1 LMU"}="1H4O LMU"; $resn_exch{" H6B LMU"}=" H6* LMU";
$resn_exch{" H6D LMU"}="2H6' LMU"; $resn_exch{" H6E LMU"}="1H6' LMU"; $resn_exch{"H2O2 LMU"}="2H2O LMU";
$resn_exch{"H3O2 LMU"}="2H3O LMU"; $resn_exch{" H12 LMU"}="2H1  LMU"; $resn_exch{" H11 LMU"}="1H1  LMU";
$resn_exch{" H22 LMU"}="2H2  LMU"; $resn_exch{" H21 LMU"}="1H2  LMU"; $resn_exch{" H32 LMU"}="2H3  LMU";
$resn_exch{" H31 LMU"}="1H3  LMU"; $resn_exch{" H42 LMU"}="2H4  LMU"; $resn_exch{" H41 LMU"}="1H4  LMU";
$resn_exch{" H52 LMU"}="2H5  LMU"; $resn_exch{" H51 LMU"}="1H5  LMU"; $resn_exch{" H62 LMU"}="2H6  LMU";
$resn_exch{" H61 LMU"}="1H6  LMU"; $resn_exch{" H72 LMU"}="2H7  LMU"; $resn_exch{" H71 LMU"}="1H7  LMU";
$resn_exch{" H82 LMU"}="2H8  LMU"; $resn_exch{" H81 LMU"}="1H8  LMU"; $resn_exch{" H92 LMU"}="2H9  LMU";
$resn_exch{" H91 LMU"}="1H9  LMU"; $resn_exch{"H102 LMU"}="2H10 LMU"; $resn_exch{"H101 LMU"}="1H10 LMU";
$resn_exch{"H112 LMU"}="2H11 LMU"; $resn_exch{"H111 LMU"}="1H11 LMU"; $resn_exch{"H123 LMU"}="3H12 LMU";
$resn_exch{"H122 LMU"}="2H12 LMU"; $resn_exch{"H121 LMU"}="1H12 LMU"; $resn_exch{" H81 LMZ"}="1H8  LMZ";
$resn_exch{" H82 LMZ"}="2H8  LMZ"; $resn_exch{"H121 LMZ"}="1H12 LMZ"; $resn_exch{"H122 LMZ"}="2H12 LMZ";
$resn_exch{"HN71 LNC"}="1HN7 LNC"; $resn_exch{"HN72 LNC"}="2HN7 LNC"; $resn_exch{"HGS1 LNC"}="1HGS LNC";
$resn_exch{"HGS2 LNC"}="2HGS LNC"; $resn_exch{"HBS1 LNC"}="1HBS LNC"; $resn_exch{"HBS2 LNC"}="2HBS LNC";
$resn_exch{" H1' LNC"}=" H1* LNC"; $resn_exch{" H4' LNC"}=" H4* LNC"; $resn_exch{"H5'1 LNC"}="1H5* LNC";
$resn_exch{"H5'2 LNC"}="2H5* LNC"; $resn_exch{" H11 LNC"}="1H1  LNC"; $resn_exch{" H12 LNC"}="2H1  LNC";
$resn_exch{" H2' LNC"}=" H2* LNC"; $resn_exch{" H3' LNC"}=" H3* LNC"; $resn_exch{"HN61 LNC"}="1HN6 LNC";
$resn_exch{"HN62 LNC"}="2HN6 LNC"; $resn_exch{"H111 LNC"}="H11  LNC"; $resn_exch{"H122 LNC"}="H12  LNC";
$resn_exch{"H2C1 LNG"}="1H2C LNG"; $resn_exch{"H2C2 LNG"}="2H2C LNG"; $resn_exch{"H3C1 LNG"}="1H3C LNG";
$resn_exch{"H3C2 LNG"}="2H3C LNG"; $resn_exch{"H4C1 LNG"}="1H4C LNG"; $resn_exch{"H4C2 LNG"}="2H4C LNG";
$resn_exch{"H5C1 LNG"}="1H5C LNG"; $resn_exch{"H5C2 LNG"}="2H5C LNG"; $resn_exch{"H6C1 LNG"}="1H6C LNG";
$resn_exch{"H6C2 LNG"}="2H6C LNG"; $resn_exch{"H7C1 LNG"}="1H7C LNG"; $resn_exch{"H7C2 LNG"}="2H7C LNG";
$resn_exch{"H8C1 LNG"}="1H8C LNG"; $resn_exch{"H8C2 LNG"}="2H8C LNG"; $resn_exch{"H9C1 LNG"}="1H9C LNG";
$resn_exch{"H9C2 LNG"}="2H9C LNG"; $resn_exch{"H101 LNG"}="1H10 LNG"; $resn_exch{"H102 LNG"}="2H10 LNG";
$resn_exch{"H111 LNG"}="1H11 LNG"; $resn_exch{"H112 LNG"}="2H11 LNG"; $resn_exch{"H131 LNG"}="1H13 LNG";
$resn_exch{"H132 LNG"}="2H13 LNG"; $resn_exch{"H133 LNG"}="3H13 LNG"; $resn_exch{"H141 LNG"}="1H14 LNG";
$resn_exch{"H142 LNG"}="2H14 LNG"; $resn_exch{"H143 LNG"}="3H14 LNG"; $resn_exch{" H21 LNL"}="1H2  LNL";
$resn_exch{" H22 LNL"}="2H2  LNL"; $resn_exch{" H31 LNL"}="1H3  LNL"; $resn_exch{" H32 LNL"}="2H3  LNL";
$resn_exch{" H41 LNL"}="1H4  LNL"; $resn_exch{" H42 LNL"}="2H4  LNL"; $resn_exch{" H51 LNL"}="1H5  LNL";
$resn_exch{" H52 LNL"}="2H5  LNL"; $resn_exch{" H61 LNL"}="1H6  LNL"; $resn_exch{" H62 LNL"}="2H6  LNL";
$resn_exch{" H71 LNL"}="1H7  LNL"; $resn_exch{" H72 LNL"}="2H7  LNL"; $resn_exch{" H81 LNL"}="1H8  LNL";
$resn_exch{" H82 LNL"}="2H8  LNL"; $resn_exch{"H111 LNL"}="1H11 LNL"; $resn_exch{"H112 LNL"}="2H11 LNL";
$resn_exch{"H141 LNL"}="1H14 LNL"; $resn_exch{"H142 LNL"}="2H14 LNL"; $resn_exch{"H171 LNL"}="1H17 LNL";
$resn_exch{"H172 LNL"}="2H17 LNL"; $resn_exch{"H181 LNL"}="1H18 LNL"; $resn_exch{"H182 LNL"}="2H18 LNL";
$resn_exch{"H183 LNL"}="3H18 LNL"; $resn_exch{" HN1 LNO"}="1HN  LNO"; $resn_exch{"HN2A LNO"}="2HN  LNO";
$resn_exch{" HB1 LNO"}="1HB  LNO"; $resn_exch{" HB2 LNO"}="2HB  LNO"; $resn_exch{"HD11 LNO"}="1HD1 LNO";
$resn_exch{"HD12 LNO"}="2HD1 LNO"; $resn_exch{"HD13 LNO"}="3HD1 LNO"; $resn_exch{"HD21 LNO"}="1HD2 LNO";
$resn_exch{"HD22 LNO"}="2HD2 LNO"; $resn_exch{"HD23 LNO"}="3HD2 LNO"; $resn_exch{" H11 LNQ"}="1H1  LNQ";
$resn_exch{" H12 LNQ"}="2H1  LNQ"; $resn_exch{" H13 LNQ"}="3H1  LNQ"; $resn_exch{" H21 LNQ"}="1H2  LNQ";
$resn_exch{" H22 LNQ"}="2H2  LNQ"; $resn_exch{" H31 LNQ"}="1H3  LNQ"; $resn_exch{" H32 LNQ"}="2H3  LNQ";
$resn_exch{" H51 LNQ"}="1H5  LNQ"; $resn_exch{" H52 LNQ"}="2H5  LNQ"; $resn_exch{" H61 LNQ"}="1H6  LNQ";
$resn_exch{" H62 LNQ"}="2H6  LNQ"; $resn_exch{" H81 LNQ"}="1H8  LNQ"; $resn_exch{" H82 LNQ"}="2H8  LNQ";
$resn_exch{" H91 LNQ"}="1H9  LNQ"; $resn_exch{" H92 LNQ"}="2H9  LNQ"; $resn_exch{"H111 LNQ"}="1H11 LNQ";
$resn_exch{"H112 LNQ"}="2H11 LNQ"; $resn_exch{"H121 LNQ"}="1H12 LNQ"; $resn_exch{"H122 LNQ"}="2H12 LNQ";
$resn_exch{"H131 LNQ"}="1H13 LNQ"; $resn_exch{"H132 LNQ"}="2H13 LNQ"; $resn_exch{"H141 LNQ"}="1H14 LNQ";
$resn_exch{"H211 LNQ"}="1H21 LNQ"; $resn_exch{"H212 LNQ"}="2H21 LNQ"; $resn_exch{" H81 LNR"}="1H8  LNR";
$resn_exch{" H82 LNR"}="2H8  LNR"; $resn_exch{"HN11 LNR"}="1HN1 LNR"; $resn_exch{"HN12 LNR"}="2HN1 LNR";
$resn_exch{" HC1 LO1"}="1HC  LO1"; $resn_exch{" HC2 LO1"}="2HC  LO1"; $resn_exch{"H1N1 LO1"}="1H1N LO1";
$resn_exch{"H1N2 LO1"}="2H1N LO1"; $resn_exch{"H8C1 LOB"}="1H8C LOB"; $resn_exch{"H8C2 LOB"}="2H8C LOB";
$resn_exch{"H121 LOB"}="1H12 LOB"; $resn_exch{"H122 LOB"}="2H12 LOB"; $resn_exch{"H151 LOB"}="1H15 LOB";
$resn_exch{"H152 LOB"}="2H15 LOB"; $resn_exch{"H131 LOB"}="1H13 LOB"; $resn_exch{"H132 LOB"}="2H13 LOB";
$resn_exch{"H221 LOB"}="1H22 LOB"; $resn_exch{"H222 LOB"}="2H22 LOB"; $resn_exch{"H223 LOB"}="3H22 LOB";
$resn_exch{"H141 LOB"}="1H14 LOB"; $resn_exch{"H142 LOB"}="2H14 LOB"; $resn_exch{" H31 LOF"}="1H3  LOF";
$resn_exch{" H32 LOF"}="2H3  LOF"; $resn_exch{"HD11 LOL"}="1HD1 LOL"; $resn_exch{"HD12 LOL"}="2HD1 LOL";
$resn_exch{"HD13 LOL"}="3HD1 LOL"; $resn_exch{"HD21 LOL"}="1HD2 LOL"; $resn_exch{"HD22 LOL"}="2HD2 LOL";
$resn_exch{"HD23 LOL"}="3HD2 LOL"; $resn_exch{" HB3 LOL"}="1HB  LOL"; $resn_exch{" HB2 LOL"}="2HB  LOL";
$resn_exch{" H   LOL"}="1HN  LOL"; $resn_exch{" H2  LOL"}="2HN  LOL"; $resn_exch{" HC  LOL"}=" H   LOL";
$resn_exch{"HN11 LOP"}="1HN1 LOP"; $resn_exch{"HN12 LOP"}="2HN1 LOP"; $resn_exch{" H11 LOP"}="1H1  LOP";
$resn_exch{" H12 LOP"}="2H1  LOP"; $resn_exch{" H21 LOP"}="1H2  LOP"; $resn_exch{" H22 LOP"}="2H2  LOP";
$resn_exch{" H31 LOP"}="1H3  LOP"; $resn_exch{" H32 LOP"}="2H3  LOP"; $resn_exch{" H51 LOP"}="1H5  LOP";
$resn_exch{" H52 LOP"}="2H5  LOP"; $resn_exch{" H71 LOP"}="1H7  LOP"; $resn_exch{" H72 LOP"}="2H7  LOP";
$resn_exch{" H81 LOP"}="1H8  LOP"; $resn_exch{" H82 LOP"}="2H8  LOP"; $resn_exch{" H91 LOP"}="1H9  LOP";
$resn_exch{" H92 LOP"}="2H9  LOP"; $resn_exch{"H101 LOP"}="1H10 LOP"; $resn_exch{"H102 LOP"}="2H10 LOP";
$resn_exch{"H111 LOP"}="1H11 LOP"; $resn_exch{"H112 LOP"}="2H11 LOP"; $resn_exch{"H121 LOP"}="1H12 LOP";
$resn_exch{"H122 LOP"}="2H12 LOP"; $resn_exch{"H131 LOP"}="1H13 LOP"; $resn_exch{"H132 LOP"}="2H13 LOP";
$resn_exch{"H161 LOP"}="1H16 LOP"; $resn_exch{"H162 LOP"}="2H16 LOP"; $resn_exch{"H171 LOP"}="1H17 LOP";
$resn_exch{"H172 LOP"}="2H17 LOP"; $resn_exch{"H181 LOP"}="1H18 LOP"; $resn_exch{"H182 LOP"}="2H18 LOP";
$resn_exch{"H191 LOP"}="1H19 LOP"; $resn_exch{"H192 LOP"}="2H19 LOP"; $resn_exch{"H201 LOP"}="1H20 LOP";
$resn_exch{"H202 LOP"}="2H20 LOP"; $resn_exch{"H211 LOP"}="1H21 LOP"; $resn_exch{"H212 LOP"}="2H21 LOP";
$resn_exch{"H221 LOP"}="1H22 LOP"; $resn_exch{"H222 LOP"}="2H22 LOP"; $resn_exch{"H231 LOP"}="1H23 LOP";
$resn_exch{"H232 LOP"}="2H23 LOP"; $resn_exch{"H233 LOP"}="3H23 LOP"; $resn_exch{"H251 LOP"}="1H25 LOP";
$resn_exch{"H252 LOP"}="2H25 LOP"; $resn_exch{"H261 LOP"}="1H26 LOP"; $resn_exch{"H262 LOP"}="2H26 LOP";
$resn_exch{"H271 LOP"}="1H27 LOP"; $resn_exch{"H272 LOP"}="2H27 LOP"; $resn_exch{"H281 LOP"}="1H28 LOP";
$resn_exch{"H282 LOP"}="2H28 LOP"; $resn_exch{"H291 LOP"}="1H29 LOP"; $resn_exch{"H292 LOP"}="2H29 LOP";
$resn_exch{"H301 LOP"}="1H30 LOP"; $resn_exch{"H302 LOP"}="2H30 LOP"; $resn_exch{"H311 LOP"}="1H31 LOP";
$resn_exch{"H312 LOP"}="2H31 LOP"; $resn_exch{"H321 LOP"}="1H32 LOP"; $resn_exch{"H322 LOP"}="2H32 LOP";
$resn_exch{"H331 LOP"}="1H33 LOP"; $resn_exch{"H332 LOP"}="2H33 LOP"; $resn_exch{"H341 LOP"}="1H34 LOP";
$resn_exch{"H342 LOP"}="2H34 LOP"; $resn_exch{"H351 LOP"}="1H35 LOP"; $resn_exch{"H352 LOP"}="2H35 LOP";
$resn_exch{"H353 LOP"}="3H35 LOP"; $resn_exch{" H51 LOR"}="1H5  LOR"; $resn_exch{" H52 LOR"}="2H5  LOR";
$resn_exch{" H41 LOR"}="1H4  LOR"; $resn_exch{" H42 LOR"}="2H4  LOR"; $resn_exch{"H221 LOR"}="1H22 LOR";
$resn_exch{"H222 LOR"}="2H22 LOR"; $resn_exch{" H31 LOS"}="1H3  LOS"; $resn_exch{" H32 LOS"}="2H3  LOS";
$resn_exch{" H41 LOS"}="1H4  LOS"; $resn_exch{" H42 LOS"}="2H4  LOS"; $resn_exch{" H51 LOS"}="1H5  LOS";
$resn_exch{" H61 LOS"}="1H6  LOS"; $resn_exch{" H77 LOS"}="7H7  LOS"; $resn_exch{" H88 LOS"}="8H8  LOS";
$resn_exch{" H91 LOS"}="1H9  LOS"; $resn_exch{" H92 LOS"}="2H9  LOS"; $resn_exch{"H101 LOS"}="1H10 LOS";
$resn_exch{"H102 LOS"}="2H10 LOS"; $resn_exch{"H111 LOS"}="1H11 LOS"; $resn_exch{"H112 LOS"}="2H11 LOS";
$resn_exch{"H121 LOS"}="1H12 LOS"; $resn_exch{"H122 LOS"}="2H12 LOS"; $resn_exch{"H271 LOS"}="1H27 LOS";
$resn_exch{"H272 LOS"}="2H27 LOS"; $resn_exch{"H281 LOS"}="1H28 LOS"; $resn_exch{"H282 LOS"}="2H28 LOS";
$resn_exch{"H291 LOS"}="1H29 LOS"; $resn_exch{"H292 LOS"}="2H29 LOS"; $resn_exch{"H301 LOS"}="1H30 LOS";
$resn_exch{"H302 LOS"}="2H30 LOS"; $resn_exch{"H331 LOS"}="31H3 LOS"; $resn_exch{"H332 LOS"}="32H3 LOS";
$resn_exch{"H33A LOS"}="1H33 LOS"; $resn_exch{"H33B LOS"}="2H33 LOS"; $resn_exch{"H341 LOS"}="1H34 LOS";
$resn_exch{"H342 LOS"}="2H34 LOS"; $resn_exch{"H351 LOS"}="1H35 LOS"; $resn_exch{"H352 LOS"}="2H35 LOS";
$resn_exch{"H361 LOS"}="1H36 LOS"; $resn_exch{"H362 LOS"}="2H36 LOS"; $resn_exch{" HG1 LOS"}="1HG  LOS";
$resn_exch{" HG2 LOS"}="2HG  LOS"; $resn_exch{"HD21 LOS"}="1HD2 LOS"; $resn_exch{"HD22 LOS"}="2HD2 LOS";
$resn_exch{"HE11 LOS"}="1HE1 LOS"; $resn_exch{"HE12 LOS"}="2HE1 LOS"; $resn_exch{"HE21 LOS"}="1HE2 LOS";
$resn_exch{"HD11 LOV"}="1HD1 LOV"; $resn_exch{"HD12 LOV"}="2HD1 LOV"; $resn_exch{"HD13 LOV"}="3HD1 LOV";
$resn_exch{"HD21 LOV"}="1HD2 LOV"; $resn_exch{"HD22 LOV"}="2HD2 LOV"; $resn_exch{"HD23 LOV"}="3HD2 LOV";
$resn_exch{"H1B1 LOV"}="1H1B LOV"; $resn_exch{"H1B2 LOV"}="2H1B LOV"; $resn_exch{" HS1 LOV"}="1HS  LOV";
$resn_exch{" H   LOV"}="1HN  LOV"; $resn_exch{" H2  LOV"}="2HN  LOV"; $resn_exch{" HT1 LOV"}="1HT  LOV";
$resn_exch{" HT2 LOV"}="2HT  LOV"; $resn_exch{"HG11 LOV"}="1HG1 LOV"; $resn_exch{"HG12 LOV"}="2HG1 LOV";
$resn_exch{"HG13 LOV"}="3HG1 LOV"; $resn_exch{"HG21 LOV"}="1HG2 LOV"; $resn_exch{"HG22 LOV"}="2HG2 LOV";
$resn_exch{"HG23 LOV"}="3HG2 LOV"; $resn_exch{" H51 LOX"}="1H5  LOX"; $resn_exch{" H52 LOX"}="2H5  LOX";
$resn_exch{" H11 LP1"}="1H1  LP1"; $resn_exch{" H12 LP1"}="2H1  LP1"; $resn_exch{"H13A LP1"}="3H1  LP1";
$resn_exch{"H241 LP1"}="1H24 LP1"; $resn_exch{"H242 LP1"}="2H24 LP1"; $resn_exch{"H243 LP1"}="3H24 LP1";
$resn_exch{"H301 LP1"}="1H30 LP1"; $resn_exch{"H302 LP1"}="2H30 LP1"; $resn_exch{"H303 LP1"}="3H30 LP1";
$resn_exch{"H311 LP1"}="1H31 LP1"; $resn_exch{"H312 LP1"}="2H31 LP1"; $resn_exch{"H313 LP1"}="3H31 LP1";
$resn_exch{"H331 LP1"}="1H33 LP1"; $resn_exch{"H332 LP1"}="2H33 LP1"; $resn_exch{" H61 LP1"}="1H6  LP1";
$resn_exch{" H62 LP1"}="2H6  LP1"; $resn_exch{"H251 LP1"}="1H25 LP1"; $resn_exch{"H252 LP1"}="2H25 LP1";
$resn_exch{"H253 LP1"}="3H25 LP1"; $resn_exch{"H281 LP1"}="1H28 LP1"; $resn_exch{"H282 LP1"}="2H28 LP1";
$resn_exch{"H391 LP1"}="1H39 LP1"; $resn_exch{"H392 LP1"}="2H39 LP1"; $resn_exch{"H401 LP1"}="1H40 LP1";
$resn_exch{"H402 LP1"}="2H40 LP1"; $resn_exch{"H403 LP1"}="3H40 LP1"; $resn_exch{"H451 LP1"}="1H45 LP1";
$resn_exch{"H452 LP1"}="2H45 LP1"; $resn_exch{" HN1 LP1"}="1HN  LP1"; $resn_exch{" HN2 LP1"}="2HN  LP1";
$resn_exch{" H11 LP2"}="1H1  LP2"; $resn_exch{" H12 LP2"}="2H1  LP2"; $resn_exch{"H13A LP2"}="3H1  LP2";
$resn_exch{" H61 LP2"}="1H6  LP2"; $resn_exch{" H62 LP2"}="2H6  LP2"; $resn_exch{"H241 LP2"}="1H24 LP2";
$resn_exch{"H242 LP2"}="2H24 LP2"; $resn_exch{"H243 LP2"}="3H24 LP2"; $resn_exch{"H251 LP2"}="1H25 LP2";
$resn_exch{"H252 LP2"}="2H25 LP2"; $resn_exch{"H253 LP2"}="3H25 LP2"; $resn_exch{"H281 LP2"}="1H28 LP2";
$resn_exch{"H282 LP2"}="2H28 LP2"; $resn_exch{"H301 LP2"}="1H30 LP2"; $resn_exch{"H302 LP2"}="2H30 LP2";
$resn_exch{"H303 LP2"}="3H30 LP2"; $resn_exch{"H311 LP2"}="1H31 LP2"; $resn_exch{"H312 LP2"}="2H31 LP2";
$resn_exch{"H313 LP2"}="3H31 LP2"; $resn_exch{"H331 LP2"}="1H33 LP2"; $resn_exch{"H332 LP2"}="2H33 LP2";
$resn_exch{"H391 LP2"}="1H39 LP2"; $resn_exch{"H392 LP2"}="2H39 LP2"; $resn_exch{"H401 LP2"}="1H40 LP2";
$resn_exch{"H402 LP2"}="2H40 LP2"; $resn_exch{"H403 LP2"}="3H40 LP2"; $resn_exch{"H451 LP2"}="1H45 LP2";
$resn_exch{"H452 LP2"}="2H45 LP2"; $resn_exch{" HN1 LP2"}="1HN  LP2"; $resn_exch{" HN2 LP2"}="2HN  LP2";
$resn_exch{" H11 LP3"}="1H1  LP3"; $resn_exch{" H12 LP3"}="2H1  LP3"; $resn_exch{" H31 LP3"}="1H3  LP3";
$resn_exch{" H32 LP3"}="2H3  LP3"; $resn_exch{" H41 LP3"}="1H4  LP3"; $resn_exch{" H42 LP3"}="2H4  LP3";
$resn_exch{" H51 LP3"}="1H5  LP3"; $resn_exch{" H52 LP3"}="2H5  LP3"; $resn_exch{" H61 LP3"}="1H6  LP3";
$resn_exch{" H62 LP3"}="2H6  LP3"; $resn_exch{" H63 LP3"}="3H6  LP3"; $resn_exch{" H71 LP3"}="1H7  LP3";
$resn_exch{" H72 LP3"}="2H7  LP3"; $resn_exch{" H73 LP3"}="3H7  LP3"; $resn_exch{" H81 LP3"}="1H8  LP3";
$resn_exch{" H82 LP3"}="2H8  LP3"; $resn_exch{" H83 LP3"}="3H8  LP3"; $resn_exch{"H121 LP3"}="1H12 LP3";
$resn_exch{"H122 LP3"}="2H12 LP3"; $resn_exch{"H131 LP3"}="1H13 LP3"; $resn_exch{"H132 LP3"}="2H13 LP3";
$resn_exch{"H141 LP3"}="1H14 LP3"; $resn_exch{"H142 LP3"}="2H14 LP3"; $resn_exch{"H151 LP3"}="1H15 LP3";
$resn_exch{"H152 LP3"}="2H15 LP3"; $resn_exch{"H161 LP3"}="1H16 LP3"; $resn_exch{"H162 LP3"}="2H16 LP3";
$resn_exch{"H171 LP3"}="1H17 LP3"; $resn_exch{"H172 LP3"}="2H17 LP3"; $resn_exch{"H181 LP3"}="1H18 LP3";
$resn_exch{"H182 LP3"}="2H18 LP3"; $resn_exch{"H191 LP3"}="1H19 LP3"; $resn_exch{"H192 LP3"}="2H19 LP3";
$resn_exch{"H201 LP3"}="1H20 LP3"; $resn_exch{"H202 LP3"}="2H20 LP3"; $resn_exch{"H211 LP3"}="1H21 LP3";
$resn_exch{"H212 LP3"}="2H21 LP3"; $resn_exch{"H221 LP3"}="1H22 LP3"; $resn_exch{"H222 LP3"}="2H22 LP3";
$resn_exch{"H231 LP3"}="1H23 LP3"; $resn_exch{"H232 LP3"}="2H23 LP3"; $resn_exch{"H241 LP3"}="1H24 LP3";
$resn_exch{"H242 LP3"}="2H24 LP3"; $resn_exch{"H251 LP3"}="1H25 LP3"; $resn_exch{"H252 LP3"}="2H25 LP3";
$resn_exch{"H261 LP3"}="1H26 LP3"; $resn_exch{"H262 LP3"}="2H26 LP3"; $resn_exch{"H271 LP3"}="1H27 LP3";
$resn_exch{"H272 LP3"}="2H27 LP3"; $resn_exch{"H281 LP3"}="1H28 LP3"; $resn_exch{"H282 LP3"}="2H28 LP3";
$resn_exch{"H283 LP3"}="3H28 LP3"; $resn_exch{" H61 LP4"}="1H6  LP4"; $resn_exch{" H62 LP4"}="2H6  LP4";
$resn_exch{"H291 LP4"}="1H29 LP4"; $resn_exch{"H292 LP4"}="2H29 LP4"; $resn_exch{"H311 LP4"}="1H31 LP4";
$resn_exch{"H312 LP4"}="2H31 LP4"; $resn_exch{"H321 LP4"}="1H32 LP4"; $resn_exch{"H322 LP4"}="2H32 LP4";
$resn_exch{"H331 LP4"}="1H33 LP4"; $resn_exch{"H332 LP4"}="2H33 LP4"; $resn_exch{"H341 LP4"}="1H34 LP4";
$resn_exch{"H342 LP4"}="2H34 LP4"; $resn_exch{"H351 LP4"}="1H35 LP4"; $resn_exch{"H352 LP4"}="2H35 LP4";
$resn_exch{"H361 LP4"}="1H36 LP4"; $resn_exch{"H362 LP4"}="2H36 LP4"; $resn_exch{"H371 LP4"}="1H37 LP4";
$resn_exch{"H372 LP4"}="2H37 LP4"; $resn_exch{"H381 LP4"}="1H38 LP4"; $resn_exch{"H382 LP4"}="2H38 LP4";
$resn_exch{"H391 LP4"}="1H39 LP4"; $resn_exch{"H392 LP4"}="2H39 LP4"; $resn_exch{"H401 LP4"}="1H40 LP4";
$resn_exch{"H402 LP4"}="2H40 LP4"; $resn_exch{"H411 LP4"}="1H41 LP4"; $resn_exch{"H412 LP4"}="2H41 LP4";
$resn_exch{"H413 LP4"}="3H41 LP4"; $resn_exch{" H11 LP4"}="1H1  LP4"; $resn_exch{" H12 LP4"}="2H1  LP4";
$resn_exch{" H81 LP4"}="1H8  LP4"; $resn_exch{" H82 LP4"}="2H8  LP4"; $resn_exch{"H171 LP4"}="1H17 LP4";
$resn_exch{"H172 LP4"}="2H17 LP4"; $resn_exch{"H181 LP4"}="1H18 LP4"; $resn_exch{"H182 LP4"}="2H18 LP4";
$resn_exch{"H191 LP4"}="1H19 LP4"; $resn_exch{"H192 LP4"}="2H19 LP4"; $resn_exch{"H201 LP4"}="1H20 LP4";
$resn_exch{"H202 LP4"}="2H20 LP4"; $resn_exch{"H211 LP4"}="1H21 LP4"; $resn_exch{"H212 LP4"}="2H21 LP4";
$resn_exch{"H221 LP4"}="1H22 LP4"; $resn_exch{"H222 LP4"}="2H22 LP4"; $resn_exch{"H231 LP4"}="1H23 LP4";
$resn_exch{"H232 LP4"}="2H23 LP4"; $resn_exch{"H241 LP4"}="1H24 LP4"; $resn_exch{"H242 LP4"}="2H24 LP4";
$resn_exch{"H251 LP4"}="1H25 LP4"; $resn_exch{"H252 LP4"}="2H25 LP4"; $resn_exch{"H261 LP4"}="1H26 LP4";
$resn_exch{"H262 LP4"}="2H26 LP4"; $resn_exch{"H271 LP4"}="1H27 LP4"; $resn_exch{"H272 LP4"}="2H27 LP4";
$resn_exch{"H273 LP4"}="3H27 LP4"; $resn_exch{" H81 LP5"}="1H8  LP5"; $resn_exch{" H82 LP5"}="2H8  LP5";
$resn_exch{"H171 LP5"}="1H17 LP5"; $resn_exch{"H172 LP5"}="2H17 LP5"; $resn_exch{"H181 LP5"}="1H18 LP5";
$resn_exch{"H182 LP5"}="2H18 LP5"; $resn_exch{"H191 LP5"}="1H19 LP5"; $resn_exch{"H192 LP5"}="2H19 LP5";
$resn_exch{"H201 LP5"}="1H20 LP5"; $resn_exch{"H202 LP5"}="2H20 LP5"; $resn_exch{"H211 LP5"}="1H21 LP5";
$resn_exch{"H212 LP5"}="2H21 LP5"; $resn_exch{"H221 LP5"}="1H22 LP5"; $resn_exch{"H222 LP5"}="2H22 LP5";
$resn_exch{"H231 LP5"}="1H23 LP5"; $resn_exch{"H232 LP5"}="2H23 LP5"; $resn_exch{"H241 LP5"}="1H24 LP5";
$resn_exch{"H242 LP5"}="2H24 LP5"; $resn_exch{"H251 LP5"}="1H25 LP5"; $resn_exch{"H252 LP5"}="2H25 LP5";
$resn_exch{"H261 LP5"}="1H26 LP5"; $resn_exch{"H262 LP5"}="2H26 LP5"; $resn_exch{"H271 LP5"}="1H27 LP5";
$resn_exch{"H272 LP5"}="2H27 LP5"; $resn_exch{"H273 LP5"}="3H27 LP5"; $resn_exch{" H61 LP5"}="1H6  LP5";
$resn_exch{" H62 LP5"}="2H6  LP5"; $resn_exch{"H291 LP5"}="1H29 LP5"; $resn_exch{"H292 LP5"}="2H29 LP5";
$resn_exch{"H311 LP5"}="1H31 LP5"; $resn_exch{"H312 LP5"}="2H31 LP5"; $resn_exch{"H321 LP5"}="1H32 LP5";
$resn_exch{"H322 LP5"}="2H32 LP5"; $resn_exch{"H331 LP5"}="1H33 LP5"; $resn_exch{"H332 LP5"}="2H33 LP5";
$resn_exch{"H341 LP5"}="1H34 LP5"; $resn_exch{"H342 LP5"}="2H34 LP5"; $resn_exch{"H351 LP5"}="1H35 LP5";
$resn_exch{"H352 LP5"}="2H35 LP5"; $resn_exch{"H361 LP5"}="1H36 LP5"; $resn_exch{"H362 LP5"}="2H36 LP5";
$resn_exch{"H371 LP5"}="1H37 LP5"; $resn_exch{"H372 LP5"}="2H37 LP5"; $resn_exch{"H381 LP5"}="1H38 LP5";
$resn_exch{"H382 LP5"}="2H38 LP5"; $resn_exch{"H391 LP5"}="1H39 LP5"; $resn_exch{"H392 LP5"}="2H39 LP5";
$resn_exch{"H401 LP5"}="1H40 LP5"; $resn_exch{"H402 LP5"}="2H40 LP5"; $resn_exch{"H411 LP5"}="1H41 LP5";
$resn_exch{"H412 LP5"}="2H41 LP5"; $resn_exch{"H413 LP5"}="3H41 LP5"; $resn_exch{" H21 LPA"}="1H2  LPA";
$resn_exch{" H22 LPA"}="2H2  LPA"; $resn_exch{" H31 LPA"}="1H3  LPA"; $resn_exch{" H32 LPA"}="2H3  LPA";
$resn_exch{" H41 LPA"}="1H4  LPA"; $resn_exch{" H42 LPA"}="2H4  LPA"; $resn_exch{" H51 LPA"}="1H5  LPA";
$resn_exch{" H52 LPA"}="2H5  LPA"; $resn_exch{" H71 LPA"}="1H7  LPA"; $resn_exch{" H72 LPA"}="2H7  LPA";
$resn_exch{" H81 LPA"}="1H8  LPA"; $resn_exch{" H82 LPA"}="2H8  LPA"; $resn_exch{"H0A1 LPC"}="1H0A LPC";
$resn_exch{"H0A2 LPC"}="2H0A LPC"; $resn_exch{"H0A3 LPC"}="3H0A LPC"; $resn_exch{"H0B1 LPC"}="1H0B LPC";
$resn_exch{"H0B2 LPC"}="2H0B LPC"; $resn_exch{"H0B3 LPC"}="3H0B LPC"; $resn_exch{"H0C1 LPC"}="1H0C LPC";
$resn_exch{"H0C2 LPC"}="2H0C LPC"; $resn_exch{"H0C3 LPC"}="3H0C LPC"; $resn_exch{" H21 LPC"}="1H2  LPC";
$resn_exch{" H22 LPC"}="2H2  LPC"; $resn_exch{" H31 LPC"}="1H3  LPC"; $resn_exch{" H32 LPC"}="2H3  LPC";
$resn_exch{"HO5A LPC"}="AHO5 LPC"; $resn_exch{" H71 LPC"}="1H7  LPC"; $resn_exch{" H72 LPC"}="2H7  LPC";
$resn_exch{" H91 LPC"}="1H9  LPC"; $resn_exch{" H92 LPC"}="2H9  LPC"; $resn_exch{" HB1 LPC"}="1HB  LPC";
$resn_exch{" HB2 LPC"}="2HB  LPC"; $resn_exch{" HC1 LPC"}="1HC  LPC"; $resn_exch{" HC2 LPC"}="2HC  LPC";
$resn_exch{" HD1 LPC"}="1HD  LPC"; $resn_exch{" HD2 LPC"}="2HD  LPC"; $resn_exch{" HE1 LPC"}="1HE  LPC";
$resn_exch{" HE2 LPC"}="2HE  LPC"; $resn_exch{" HF1 LPC"}="1HF  LPC"; $resn_exch{" HF2 LPC"}="2HF  LPC";
$resn_exch{" HG1 LPC"}="1HG  LPC"; $resn_exch{" HG2 LPC"}="2HG  LPC"; $resn_exch{" HH1 LPC"}="1HH  LPC";
$resn_exch{" HH2 LPC"}="2HH  LPC"; $resn_exch{" HI1 LPC"}="1HI  LPC"; $resn_exch{" HI2 LPC"}="2HI  LPC";
$resn_exch{" HJ1 LPC"}="1HJ  LPC"; $resn_exch{" HJ2 LPC"}="2HJ  LPC"; $resn_exch{" HK1 LPC"}="1HK  LPC";
$resn_exch{" HK2 LPC"}="2HK  LPC"; $resn_exch{" HL1 LPC"}="1HL  LPC"; $resn_exch{" HL2 LPC"}="2HL  LPC";
$resn_exch{" HM1 LPC"}="1HM  LPC"; $resn_exch{" HM2 LPC"}="2HM  LPC"; $resn_exch{" HN1 LPC"}="1HN  LPC";
$resn_exch{" HN2 LPC"}="2HN  LPC"; $resn_exch{" HN3 LPC"}="3HN  LPC"; $resn_exch{"HN21 LPD"}="1HN2 LPD";
$resn_exch{"HN22 LPD"}="2HN2 LPD"; $resn_exch{" H   LPD"}=" HN  LPD"; $resn_exch{" HD3 LPD"}="1HD  LPD";
$resn_exch{" HD2 LPD"}="2HD  LPD"; $resn_exch{" HG3 LPD"}="1HG  LPD"; $resn_exch{" HG2 LPD"}="2HG  LPD";
$resn_exch{" HB2 LPD"}="1HB  LPD"; $resn_exch{" HB3 LPD"}="2HB  LPD"; $resn_exch{" H11 LPE"}="1H1  LPE";
$resn_exch{" H12 LPE"}="2H1  LPE"; $resn_exch{" H21 LPE"}="1H2  LPE"; $resn_exch{"HO21 LPE"}="1HO2 LPE";
$resn_exch{" H31 LPE"}="1H3  LPE"; $resn_exch{" H32 LPE"}="2H3  LPE"; $resn_exch{"H111 LPE"}="1H11 LPE";
$resn_exch{"H112 LPE"}="2H11 LPE"; $resn_exch{"H121 LPE"}="1H12 LPE"; $resn_exch{"H122 LPE"}="2H12 LPE";
$resn_exch{"HO31 LPE"}="1HO3 LPE"; $resn_exch{"H311 LPE"}="1H31 LPE"; $resn_exch{"H312 LPE"}="2H31 LPE";
$resn_exch{"H321 LPE"}="1H32 LPE"; $resn_exch{"H322 LPE"}="2H32 LPE"; $resn_exch{"H1N1 LPE"}="1H1N LPE";
$resn_exch{"H1N2 LPE"}="2H1N LPE"; $resn_exch{"H1N3 LPE"}="3H1N LPE"; $resn_exch{"H2N1 LPE"}="1H2N LPE";
$resn_exch{"H2N2 LPE"}="2H2N LPE"; $resn_exch{"H2N3 LPE"}="3H2N LPE"; $resn_exch{"H3N1 LPE"}="1H3N LPE";
$resn_exch{"H3N2 LPE"}="2H3N LPE"; $resn_exch{"H3N3 LPE"}="3H3N LPE"; $resn_exch{"H131 LPE"}="1H13 LPE";
$resn_exch{"H132 LPE"}="2H13 LPE"; $resn_exch{"H141 LPE"}="1H14 LPE"; $resn_exch{"H142 LPE"}="2H14 LPE";
$resn_exch{"H151 LPE"}="1H15 LPE"; $resn_exch{"H152 LPE"}="2H15 LPE"; $resn_exch{"H161 LPE"}="1H16 LPE";
$resn_exch{"H162 LPE"}="2H16 LPE"; $resn_exch{"H171 LPE"}="1H17 LPE"; $resn_exch{"H172 LPE"}="2H17 LPE";
$resn_exch{"H181 LPE"}="1H18 LPE"; $resn_exch{"H182 LPE"}="2H18 LPE"; $resn_exch{"H191 LPE"}="1H19 LPE";
$resn_exch{"H192 LPE"}="2H19 LPE"; $resn_exch{"H201 LPE"}="1H20 LPE"; $resn_exch{"H202 LPE"}="2H20 LPE";
$resn_exch{"H211 LPE"}="1H21 LPE"; $resn_exch{"H212 LPE"}="2H21 LPE"; $resn_exch{"H221 LPE"}="1H22 LPE";
$resn_exch{"H222 LPE"}="2H22 LPE"; $resn_exch{"H231 LPE"}="1H23 LPE"; $resn_exch{"H232 LPE"}="2H23 LPE";
$resn_exch{"H241 LPE"}="1H24 LPE"; $resn_exch{"H242 LPE"}="2H24 LPE"; $resn_exch{"H251 LPE"}="1H25 LPE";
$resn_exch{"H252 LPE"}="2H25 LPE"; $resn_exch{"H261 LPE"}="1H26 LPE"; $resn_exch{"H262 LPE"}="2H26 LPE";
$resn_exch{"H271 LPE"}="1H27 LPE"; $resn_exch{"H272 LPE"}="2H27 LPE"; $resn_exch{"H281 LPE"}="1H28 LPE";
$resn_exch{"H282 LPE"}="2H28 LPE"; $resn_exch{"H283 LPE"}="3H28 LPE"; $resn_exch{" H41 LPF"}="1H4  LPF";
$resn_exch{" H42 LPF"}="2H4  LPF"; $resn_exch{"HL21 LPF"}="1HL2 LPF"; $resn_exch{"HL22 LPF"}="2HL2 LPF";
$resn_exch{"HL23 LPF"}="3HL2 LPF"; $resn_exch{"HA11 LPF"}="1HA1 LPF"; $resn_exch{"HA12 LPF"}="2HA1 LPF";
$resn_exch{"HA31 LPF"}="1HA3 LPF"; $resn_exch{"HA32 LPF"}="2HA3 LPF"; $resn_exch{"HA33 LPF"}="3HA3 LPF";
$resn_exch{"HA41 LPF"}="1HA4 LPF"; $resn_exch{"HA42 LPF"}="2HA4 LPF"; $resn_exch{"HA43 LPF"}="3HA4 LPF";
$resn_exch{" H   LPG"}="1HN  LPG"; $resn_exch{" H2  LPG"}="2HN  LPG"; $resn_exch{" H   LPL"}="1HN  LPL";
$resn_exch{" H2  LPL"}="2HN  LPL"; $resn_exch{" HB2 LPL"}="1HB  LPL"; $resn_exch{" HB3 LPL"}="2HB  LPL";
$resn_exch{"HD11 LPL"}="1HD1 LPL"; $resn_exch{"HD12 LPL"}="2HD1 LPL"; $resn_exch{"HD13 LPL"}="3HD1 LPL";
$resn_exch{"HD21 LPL"}="1HD2 LPL"; $resn_exch{"HD22 LPL"}="2HD2 LPL"; $resn_exch{"HD23 LPL"}="3HD2 LPL";
$resn_exch{" HM1 LPL"}="1HM  LPL"; $resn_exch{" HM2 LPL"}="2HM  LPL"; $resn_exch{"HB11 LPL"}="1HB1 LPL";
$resn_exch{"HB12 LPL"}="2HB1 LPL"; $resn_exch{"HD31 LPL"}="1HD3 LPL"; $resn_exch{"HD32 LPL"}="2HD3 LPL";
$resn_exch{"HD33 LPL"}="3HD3 LPL"; $resn_exch{"HD41 LPL"}="1HD4 LPL"; $resn_exch{"HD42 LPL"}="2HD4 LPL";
$resn_exch{"HD43 LPL"}="3HD4 LPL"; $resn_exch{" HN1 LPM"}="1HN  LPM"; $resn_exch{" HN2 LPM"}="2HN  LPM";
$resn_exch{" H21 LPM"}="1H2  LPM"; $resn_exch{" H22 LPM"}="2H2  LPM"; $resn_exch{" H31 LPM"}="1H3  LPM";
$resn_exch{" H32 LPM"}="2H3  LPM"; $resn_exch{" H41 LPM"}="1H4  LPM"; $resn_exch{" H42 LPM"}="2H4  LPM";
$resn_exch{" H51 LPM"}="1H5  LPM"; $resn_exch{" H52 LPM"}="2H5  LPM"; $resn_exch{" H71 LPM"}="1H7  LPM";
$resn_exch{" H72 LPM"}="2H7  LPM"; $resn_exch{" H81 LPM"}="1H8  LPM"; $resn_exch{" H82 LPM"}="2H8  LPM";
$resn_exch{"HC61 LPP"}="1HC6 LPP"; $resn_exch{"HC62 LPP"}="2HC6 LPP"; $resn_exch{"HC81 LPP"}="1HC8 LPP";
$resn_exch{"HC82 LPP"}="2HC8 LPP"; $resn_exch{"H121 LPP"}="1H12 LPP"; $resn_exch{"H122 LPP"}="2H12 LPP";
$resn_exch{"H131 LPP"}="1H13 LPP"; $resn_exch{"H132 LPP"}="2H13 LPP"; $resn_exch{"H141 LPP"}="1H14 LPP";
$resn_exch{"H142 LPP"}="2H14 LPP"; $resn_exch{"H151 LPP"}="1H15 LPP"; $resn_exch{"H152 LPP"}="2H15 LPP";
$resn_exch{"H161 LPP"}="1H16 LPP"; $resn_exch{"H162 LPP"}="2H16 LPP"; $resn_exch{"H171 LPP"}="1H17 LPP";
$resn_exch{"H172 LPP"}="2H17 LPP"; $resn_exch{"H181 LPP"}="1H18 LPP"; $resn_exch{"H182 LPP"}="2H18 LPP";
$resn_exch{"H191 LPP"}="1H19 LPP"; $resn_exch{"H192 LPP"}="2H19 LPP"; $resn_exch{"H201 LPP"}="1H20 LPP";
$resn_exch{"H202 LPP"}="2H20 LPP"; $resn_exch{"H211 LPP"}="1H21 LPP"; $resn_exch{"H212 LPP"}="2H21 LPP";
$resn_exch{"H221 LPP"}="1H22 LPP"; $resn_exch{"H222 LPP"}="2H22 LPP"; $resn_exch{"H231 LPP"}="1H23 LPP";
$resn_exch{"H232 LPP"}="2H23 LPP"; $resn_exch{"H241 LPP"}="1H24 LPP"; $resn_exch{"H242 LPP"}="2H24 LPP";
$resn_exch{"H251 LPP"}="1H25 LPP"; $resn_exch{"H252 LPP"}="2H25 LPP"; $resn_exch{"H261 LPP"}="1H26 LPP";
$resn_exch{"H262 LPP"}="2H26 LPP"; $resn_exch{"H263 LPP"}="3H26 LPP"; $resn_exch{"H301 LPP"}="1H30 LPP";
$resn_exch{"H302 LPP"}="2H30 LPP"; $resn_exch{"H311 LPP"}="1H31 LPP"; $resn_exch{"H312 LPP"}="2H31 LPP";
$resn_exch{"H321 LPP"}="1H32 LPP"; $resn_exch{"H322 LPP"}="2H32 LPP"; $resn_exch{"H331 LPP"}="1H33 LPP";
$resn_exch{"H332 LPP"}="2H33 LPP"; $resn_exch{"H341 LPP"}="1H34 LPP"; $resn_exch{"H342 LPP"}="2H34 LPP";
$resn_exch{"H351 LPP"}="1H35 LPP"; $resn_exch{"H352 LPP"}="2H35 LPP"; $resn_exch{"H361 LPP"}="1H36 LPP";
$resn_exch{"H362 LPP"}="2H36 LPP"; $resn_exch{"H371 LPP"}="1H37 LPP"; $resn_exch{"H372 LPP"}="2H37 LPP";
$resn_exch{"H381 LPP"}="1H38 LPP"; $resn_exch{"H382 LPP"}="2H38 LPP"; $resn_exch{"H391 LPP"}="1H39 LPP";
$resn_exch{"H392 LPP"}="2H39 LPP"; $resn_exch{"H401 LPP"}="1H40 LPP"; $resn_exch{"H402 LPP"}="2H40 LPP";
$resn_exch{"H411 LPP"}="1H41 LPP"; $resn_exch{"H412 LPP"}="2H41 LPP"; $resn_exch{"H421 LPP"}="1H42 LPP";
$resn_exch{"H422 LPP"}="2H42 LPP"; $resn_exch{"H431 LPP"}="1H43 LPP"; $resn_exch{"H432 LPP"}="2H43 LPP";
$resn_exch{"H441 LPP"}="1H44 LPP"; $resn_exch{"H442 LPP"}="2H44 LPP"; $resn_exch{"H443 LPP"}="3H44 LPP";
$resn_exch{"H3N1 LPR"}="1H3N LPR"; $resn_exch{"H3N2 LPR"}="2H3N LPR"; $resn_exch{" H62 LPR"}="2H6  LPR";
$resn_exch{" H72 LPR"}="2H7  LPR"; $resn_exch{"H8C1 LPR"}="1H8C LPR"; $resn_exch{"H8C2 LPR"}="2H8C LPR";
$resn_exch{"H101 LPR"}="1H10 LPR"; $resn_exch{"H102 LPR"}="2H10 LPR"; $resn_exch{"H111 LPR"}="1H11 LPR";
$resn_exch{"H112 LPR"}="2H11 LPR"; $resn_exch{"H121 LPR"}="1H12 LPR"; $resn_exch{"H122 LPR"}="2H12 LPR";
$resn_exch{"H131 LPR"}="1H13 LPR"; $resn_exch{"H132 LPR"}="2H13 LPR"; $resn_exch{"H141 LPR"}="1H14 LPR";
$resn_exch{"H142 LPR"}="2H14 LPR"; $resn_exch{"H151 LPR"}="1H15 LPR"; $resn_exch{"H152 LPR"}="2H15 LPR";
$resn_exch{" HN1 LPS"}="1HN  LPS"; $resn_exch{" HN2 LPS"}="2HN  LPS"; $resn_exch{"H111 LPS"}="1H11 LPS";
$resn_exch{"H112 LPS"}="2H11 LPS"; $resn_exch{" H11 LPS"}="1H1  LPS"; $resn_exch{"H12A LPS"}="2H1  LPS";
$resn_exch{"H31A LPS"}="1H3  LPS"; $resn_exch{"H32A LPS"}="2H3  LPS"; $resn_exch{"H321 LPS"}="1H32 LPS";
$resn_exch{"H322 LPS"}="2H32 LPS"; $resn_exch{"H331 LPS"}="1H33 LPS"; $resn_exch{"H332 LPS"}="2H33 LPS";
$resn_exch{"H341 LPS"}="1H34 LPS"; $resn_exch{"H342 LPS"}="2H34 LPS"; $resn_exch{"H351 LPS"}="1H35 LPS";
$resn_exch{"H352 LPS"}="2H35 LPS"; $resn_exch{"H361 LPS"}="1H36 LPS"; $resn_exch{"H362 LPS"}="2H36 LPS";
$resn_exch{"H371 LPS"}="1H37 LPS"; $resn_exch{"H372 LPS"}="2H37 LPS"; $resn_exch{"H381 LPS"}="1H38 LPS";
$resn_exch{"H382 LPS"}="2H38 LPS"; $resn_exch{"H391 LPS"}="1H39 LPS"; $resn_exch{"H392 LPS"}="2H39 LPS";
$resn_exch{"H401 LPS"}="1H40 LPS"; $resn_exch{"H402 LPS"}="2H40 LPS"; $resn_exch{"H411 LPS"}="1H41 LPS";
$resn_exch{"H412 LPS"}="2H41 LPS"; $resn_exch{"H421 LPS"}="1H42 LPS"; $resn_exch{"H422 LPS"}="2H42 LPS";
$resn_exch{"H431 LPS"}="1H43 LPS"; $resn_exch{"H432 LPS"}="2H43 LPS"; $resn_exch{"H441 LPS"}="1H44 LPS";
$resn_exch{"H442 LPS"}="2H44 LPS"; $resn_exch{"H451 LPS"}="1H45 LPS"; $resn_exch{"H452 LPS"}="2H45 LPS";
$resn_exch{"H453 LPS"}="3H45 LPS"; $resn_exch{" H81 LPT"}="1H8  LPT"; $resn_exch{" H82 LPT"}="2H8  LPT";
$resn_exch{" H71 LPT"}="1H7  LPT"; $resn_exch{" H72 LPT"}="2H7  LPT"; $resn_exch{" H61 LPT"}="1H6  LPT";
$resn_exch{" H62 LPT"}="2H6  LPT"; $resn_exch{" H51 LPT"}="1H5  LPT"; $resn_exch{" H52 LPT"}="2H5  LPT";
$resn_exch{" H41 LPT"}="1H4  LPT"; $resn_exch{" H42 LPT"}="2H4  LPT"; $resn_exch{"HN21 LPT"}="1HN2 LPT";
$resn_exch{"HN22 LPT"}="2HN2 LPT"; $resn_exch{"HN11 LPT"}="1HN1 LPT"; $resn_exch{"HN12 LPT"}="2HN1 LPT";
$resn_exch{"HN13 LPT"}="3HN1 LPT"; $resn_exch{"H011 LQQ"}="1H01 LQQ"; $resn_exch{"H012 LQQ"}="2H01 LQQ";
$resn_exch{"H013 LQQ"}="3H01 LQQ"; $resn_exch{"H051 LQQ"}="1H05 LQQ"; $resn_exch{"H052 LQQ"}="2H05 LQQ";
$resn_exch{"H053 LQQ"}="3H05 LQQ"; $resn_exch{"H111 LQQ"}="1H11 LQQ"; $resn_exch{"H112 LQQ"}="2H11 LQQ";
$resn_exch{"H121 LQQ"}="1H12 LQQ"; $resn_exch{"H122 LQQ"}="2H12 LQQ"; $resn_exch{"H131 LQQ"}="1H13 LQQ";
$resn_exch{"H132 LQQ"}="2H13 LQQ"; $resn_exch{"H141 LQQ"}="1H14 LQQ"; $resn_exch{"H142 LQQ"}="2H14 LQQ";
$resn_exch{"H211 LQQ"}="1H21 LQQ"; $resn_exch{"H212 LQQ"}="2H21 LQQ"; $resn_exch{"H221 LQQ"}="1H22 LQQ";
$resn_exch{"H222 LQQ"}="2H22 LQQ"; $resn_exch{"H231 LQQ"}="1H23 LQQ"; $resn_exch{"H232 LQQ"}="2H23 LQQ";
$resn_exch{"H241 LQQ"}="1H24 LQQ"; $resn_exch{"H242 LQQ"}="2H24 LQQ"; $resn_exch{"H383 LRB"}="3H38 LRB";
$resn_exch{"H382 LRB"}="2H38 LRB"; $resn_exch{"H381 LRB"}="1H38 LRB"; $resn_exch{"H392 LRB"}="2H39 LRB";
$resn_exch{"H391 LRB"}="1H39 LRB"; $resn_exch{"H402 LRB"}="2H40 LRB"; $resn_exch{"H401 LRB"}="1H40 LRB";
$resn_exch{"H412 LRB"}="2H41 LRB"; $resn_exch{"H411 LRB"}="1H41 LRB"; $resn_exch{"H422 LRB"}="2H42 LRB";
$resn_exch{"H421 LRB"}="1H42 LRB"; $resn_exch{"H432 LRB"}="2H43 LRB"; $resn_exch{"H431 LRB"}="1H43 LRB";
$resn_exch{"H442 LRB"}="2H44 LRB"; $resn_exch{"H441 LRB"}="1H44 LRB"; $resn_exch{"H452 LRB"}="2H45 LRB";
$resn_exch{"H451 LRB"}="1H45 LRB"; $resn_exch{"H462 LRB"}="2H46 LRB"; $resn_exch{"H461 LRB"}="1H46 LRB";
$resn_exch{"H512 LRB"}="2H51 LRB"; $resn_exch{"H511 LRB"}="1H51 LRB"; $resn_exch{"H552 LRB"}="2H55 LRB";
$resn_exch{"H551 LRB"}="1H55 LRB"; $resn_exch{"H572 LRB"}="2H57 LRB"; $resn_exch{"H571 LRB"}="1H57 LRB";
$resn_exch{"H582 LRB"}="2H58 LRB"; $resn_exch{"H581 LRB"}="1H58 LRB"; $resn_exch{"H592 LRB"}="2H59 LRB";
$resn_exch{"H591 LRB"}="1H59 LRB"; $resn_exch{"H1C1 LRH"}="1H1C LRH"; $resn_exch{"H1C2 LRH"}="2H1C LRH";
$resn_exch{"H6C1 LRH"}="1H6C LRH"; $resn_exch{"H6C2 LRH"}="2H6C LRH"; $resn_exch{"H6C3 LRH"}="3H6C LRH";
$resn_exch{" H31 LRU"}="1H3  LRU"; $resn_exch{" H32 LRU"}="2H3  LRU"; $resn_exch{" H41 LRU"}="1H4  LRU";
$resn_exch{" H42 LRU"}="2H4  LRU"; $resn_exch{" H51 LRU"}="1H5  LRU"; $resn_exch{" H61 LRU"}="1H6  LRU";
$resn_exch{" H77 LRU"}="7H7  LRU"; $resn_exch{" H88 LRU"}="8H8  LRU"; $resn_exch{" H91 LRU"}="1H9  LRU";
$resn_exch{" H92 LRU"}="2H9  LRU"; $resn_exch{"H101 LRU"}="1H10 LRU"; $resn_exch{"H102 LRU"}="2H10 LRU";
$resn_exch{"H111 LRU"}="1H11 LRU"; $resn_exch{"H112 LRU"}="2H11 LRU"; $resn_exch{"H121 LRU"}="1H12 LRU";
$resn_exch{"H122 LRU"}="2H12 LRU"; $resn_exch{"H271 LRU"}="1H27 LRU"; $resn_exch{"H272 LRU"}="2H27 LRU";
$resn_exch{"H281 LRU"}="1H28 LRU"; $resn_exch{"H282 LRU"}="2H28 LRU"; $resn_exch{"H291 LRU"}="1H29 LRU";
$resn_exch{"H292 LRU"}="2H29 LRU"; $resn_exch{"H301 LRU"}="1H30 LRU"; $resn_exch{"H302 LRU"}="2H30 LRU";
$resn_exch{"H331 LRU"}="31H3 LRU"; $resn_exch{"H332 LRU"}="32H3 LRU"; $resn_exch{"H33A LRU"}="1H33 LRU";
$resn_exch{"H33B LRU"}="2H33 LRU"; $resn_exch{"H341 LRU"}="1H34 LRU"; $resn_exch{"H342 LRU"}="2H34 LRU";
$resn_exch{"H351 LRU"}="1H35 LRU"; $resn_exch{"H352 LRU"}="2H35 LRU"; $resn_exch{"H361 LRU"}="1H36 LRU";
$resn_exch{"H362 LRU"}="2H36 LRU"; $resn_exch{" HG1 LRU"}="1HG  LRU"; $resn_exch{" HG2 LRU"}="2HG  LRU";
$resn_exch{"HD21 LRU"}="1HD2 LRU"; $resn_exch{"HD22 LRU"}="2HD2 LRU"; $resn_exch{"HE11 LRU"}="1HE1 LRU";
$resn_exch{"HE12 LRU"}="2HE1 LRU"; $resn_exch{"HE21 LRU"}="1HE2 LRU"; $resn_exch{" H1N LS1"}=" NH1 LS1";
$resn_exch{" H41 LS1"}="1H4  LS1"; $resn_exch{" H51 LS1"}="1H5  LS1"; $resn_exch{" H61 LS1"}="1H6  LS1";
$resn_exch{" H71 LS1"}="1H7  LS1"; $resn_exch{"H121 LS1"}="1H12 LS1"; $resn_exch{"H17N LS1"}="NH17 LS1";
$resn_exch{"H211 LS1"}="1H21 LS1"; $resn_exch{"H221 LS1"}="1H22 LS1"; $resn_exch{"H241 LS1"}="1H24 LS1";
$resn_exch{"H251 LS1"}="1H25 LS1"; $resn_exch{"H33N LS1"}="NH33 LS1"; $resn_exch{"H351 LS1"}="1H35 LS1";
$resn_exch{"H352 LS1"}="2H35 LS1"; $resn_exch{"H353 LS1"}="3H35 LS1"; $resn_exch{" H1N LS2"}=" NH1 LS2";
$resn_exch{" H61 LS2"}="1H6  LS2"; $resn_exch{" H71 LS2"}="1H7  LS2"; $resn_exch{"H141 LS2"}="1H14 LS2";
$resn_exch{"H19N LS2"}="NH19 LS2"; $resn_exch{"H221 LS2"}="1H22 LS2"; $resn_exch{"H231 LS2"}="1H23 LS2";
$resn_exch{"H251 LS2"}="1H25 LS2"; $resn_exch{"H261 LS2"}="1H26 LS2"; $resn_exch{"H291 LS2"}="1H29 LS2";
$resn_exch{"H292 LS2"}="2H29 LS2"; $resn_exch{"H37N LS2"}="NH37 LS2"; $resn_exch{"H391 LS2"}="1H39 LS2";
$resn_exch{"H392 LS2"}="2H39 LS2"; $resn_exch{"H393 LS2"}="3H39 LS2"; $resn_exch{" H7N LS3"}=" NH7 LS3";
$resn_exch{" H91 LS3"}="1H9  LS3"; $resn_exch{"H101 LS3"}="1H10 LS3"; $resn_exch{"H141 LS3"}="1H14 LS3";
$resn_exch{"H161 LS3"}="1H16 LS3"; $resn_exch{"H171 LS3"}="1H17 LS3"; $resn_exch{"H201 LS3"}="1H20 LS3";
$resn_exch{"H21N LS3"}="NH21 LS3"; $resn_exch{"H231 LS3"}="1H23 LS3"; $resn_exch{"H251 LS3"}="1H25 LS3";
$resn_exch{"H252 LS3"}="2H25 LS3"; $resn_exch{"H291 LS3"}="1H29 LS3"; $resn_exch{"H292 LS3"}="2H29 LS3";
$resn_exch{"H311 LS3"}="1H31 LS3"; $resn_exch{"H321 LS3"}="1H32 LS3"; $resn_exch{" H71 LS4"}="1H7  LS4";
$resn_exch{" H91 LS4"}="1H9  LS4"; $resn_exch{"H101 LS4"}="1H10 LS4"; $resn_exch{"H111 LS4"}="1H11 LS4";
$resn_exch{"H121 LS4"}="1H12 LS4"; $resn_exch{"H151 LS4"}="1H15 LS4"; $resn_exch{"H16N LS4"}="NH16 LS4";
$resn_exch{"H181 LS4"}="1H18 LS4"; $resn_exch{"H191 LS4"}="1H19 LS4"; $resn_exch{"H211 LS4"}="1H21 LS4";
$resn_exch{"H221 LS4"}="1H22 LS4"; $resn_exch{"H26N LS4"}="NH26 LS4"; $resn_exch{"H291 LS4"}="1H29 LS4";
$resn_exch{"H301 LS4"}="1H30 LS4"; $resn_exch{" H1N LS5"}=" NH1 LS5"; $resn_exch{" H41 LS5"}="1H4  LS5";
$resn_exch{" H51 LS5"}="1H5  LS5"; $resn_exch{" H61 LS5"}="1H6  LS5"; $resn_exch{" H71 LS5"}="1H7  LS5";
$resn_exch{"H121 LS5"}="1H12 LS5"; $resn_exch{"H17N LS5"}="NH17 LS5"; $resn_exch{"H211 LS5"}="1H21 LS5";
$resn_exch{"H221 LS5"}="1H22 LS5"; $resn_exch{"H241 LS5"}="1H24 LS5"; $resn_exch{"H251 LS5"}="1H25 LS5";
$resn_exch{"H33N LS5"}="NH33 LS5"; $resn_exch{"H36N LS5"}="NH36 LS5"; $resn_exch{"H371 LS5"}="1H37 LS5";
$resn_exch{"H372 LS5"}="2H37 LS5"; $resn_exch{" HN1 LSO"}="1HN  LSO"; $resn_exch{" HN2 LSO"}="2HN  LSO";
$resn_exch{"HBC1 LSO"}="1HBC LSO"; $resn_exch{"HBC2 LSO"}="2HBC LSO"; $resn_exch{"HGC1 LSO"}="1HGC LSO";
$resn_exch{"HGC2 LSO"}="2HGC LSO"; $resn_exch{"HDC1 LSO"}="1HDC LSO"; $resn_exch{"HDC2 LSO"}="2HDC LSO";
$resn_exch{"HEC1 LSO"}="1HEC LSO"; $resn_exch{"HEC2 LSO"}="2HEC LSO"; $resn_exch{"H3C1 LSO"}="1H3C LSO";
$resn_exch{"H3C2 LSO"}="2H3C LSO"; $resn_exch{"H2C1 LSO"}="1H2C LSO"; $resn_exch{"H2C2 LSO"}="2H2C LSO";
$resn_exch{"H5C1 LSO"}="1H5C LSO"; $resn_exch{"H5C2 LSO"}="2H5C LSO"; $resn_exch{"H7C1 LSO"}="1H7C LSO";
$resn_exch{"H7C2 LSO"}="2H7C LSO"; $resn_exch{"H9C1 LSO"}="1H9C LSO"; $resn_exch{"H9C2 LSO"}="2H9C LSO";
$resn_exch{"H101 LSO"}="1H10 LSO"; $resn_exch{"H102 LSO"}="2H10 LSO"; $resn_exch{" HN1 LSP"}="1HN  LSP";
$resn_exch{" HN2 LSP"}="2HN  LSP"; $resn_exch{" HN3 LSP"}="3HN  LSP"; $resn_exch{"H121 LSP"}="1H12 LSP";
$resn_exch{"H122 LSP"}="2H12 LSP"; $resn_exch{"H111 LSP"}="1H11 LSP"; $resn_exch{"H112 LSP"}="2H11 LSP";
$resn_exch{"H321 LSP"}="1H32 LSP"; $resn_exch{"H322 LSP"}="2H32 LSP"; $resn_exch{"H323 LSP"}="3H32 LSP";
$resn_exch{" H   LTA"}="1HN  LTA"; $resn_exch{" H2  LTA"}="2HN  LTA"; $resn_exch{" HB2 LTA"}="1HB  LTA";
$resn_exch{" HB3 LTA"}="2HB  LTA"; $resn_exch{" HG2 LTA"}="1HG  LTA"; $resn_exch{" HG3 LTA"}="2HG  LTA";
$resn_exch{" HD2 LTA"}="1HD  LTA"; $resn_exch{" HD3 LTA"}="2HD  LTA"; $resn_exch{" HE2 LTA"}="1HE  LTA";
$resn_exch{" HE3 LTA"}="2HE  LTA"; $resn_exch{" HZ1 LTA"}="1HZ  LTA"; $resn_exch{" HZ2 LTA"}="2HZ  LTA";
$resn_exch{" HC  LTA"}=" H   LTA"; $resn_exch{" H21 LTA"}="1H2  LTA"; $resn_exch{" H22 LTA"}="2H2  LTA";
$resn_exch{" H91 LTA"}="1H9  LTA"; $resn_exch{" H92 LTA"}="2H9  LTA"; $resn_exch{"H101 LTA"}="1H10 LTA";
$resn_exch{"H102 LTA"}="2H10 LTA"; $resn_exch{"H103 LTA"}="3H10 LTA"; $resn_exch{"HC11 LTL"}="1HC1 LTL";
$resn_exch{"HC12 LTL"}="2HC1 LTL"; $resn_exch{"HC13 LTL"}="3HC1 LTL"; $resn_exch{"HC61 LTL"}="1HC6 LTL";
$resn_exch{"HC62 LTL"}="2HC6 LTL"; $resn_exch{"HC71 LTL"}="1HC7 LTL"; $resn_exch{"HC72 LTL"}="2HC7 LTL";
$resn_exch{"H131 LTL"}="1H13 LTL"; $resn_exch{"H132 LTL"}="2H13 LTL"; $resn_exch{"H141 LTL"}="1H14 LTL";
$resn_exch{"H142 LTL"}="2H14 LTL"; $resn_exch{"H143 LTL"}="3H14 LTL"; $resn_exch{"H221 LTL"}="1H22 LTL";
$resn_exch{"H222 LTL"}="2H22 LTL"; $resn_exch{"H223 LTL"}="3H22 LTL"; $resn_exch{"HH31 LTN"}="1HH3 LTN";
$resn_exch{"HH32 LTN"}="2HH3 LTN"; $resn_exch{" HB1 LTN"}="1HB  LTN"; $resn_exch{" HB2 LTN"}="2HB  LTN";
$resn_exch{" HN1 LTN"}="1HN  LTN"; $resn_exch{" HN2 LTN"}="2HN  LTN"; $resn_exch{" HB1 LTR"}="1HB  LTR";
$resn_exch{" HB2 LTR"}="2HB  LTR"; $resn_exch{"HN31 LTT"}="1HN3 LTT"; $resn_exch{"HN32 LTT"}="2HN3 LTT";
$resn_exch{" H81 LTT"}="1H8  LTT"; $resn_exch{" H82 LTT"}="2H8  LTT"; $resn_exch{" H61 LTT"}="1H6  LTT";
$resn_exch{" H62 LTT"}="2H6  LTT"; $resn_exch{"H7M1 LUM"}="1H7M LUM"; $resn_exch{"H7M2 LUM"}="2H7M LUM";
$resn_exch{"H7M3 LUM"}="3H7M LUM"; $resn_exch{"H8M1 LUM"}="1H8M LUM"; $resn_exch{"H8M2 LUM"}="2H8M LUM";
$resn_exch{"H8M3 LUM"}="3H8M LUM"; $resn_exch{" H21 LUT"}="1H2  LUT"; $resn_exch{" H22 LUT"}="2H2  LUT";
$resn_exch{" H41 LUT"}="1H4  LUT"; $resn_exch{" H42 LUT"}="2H4  LUT"; $resn_exch{"H161 LUT"}="1H16 LUT";
$resn_exch{"H162 LUT"}="2H16 LUT"; $resn_exch{"H163 LUT"}="3H16 LUT"; $resn_exch{"H171 LUT"}="1H17 LUT";
$resn_exch{"H172 LUT"}="2H17 LUT"; $resn_exch{"H173 LUT"}="3H17 LUT"; $resn_exch{"H181 LUT"}="1H18 LUT";
$resn_exch{"H182 LUT"}="2H18 LUT"; $resn_exch{"H183 LUT"}="3H18 LUT"; $resn_exch{"H191 LUT"}="1H19 LUT";
$resn_exch{"H192 LUT"}="2H19 LUT"; $resn_exch{"H193 LUT"}="3H19 LUT"; $resn_exch{"H201 LUT"}="1H20 LUT";
$resn_exch{"H202 LUT"}="2H20 LUT"; $resn_exch{"H203 LUT"}="3H20 LUT"; $resn_exch{"H221 LUT"}="1H22 LUT";
$resn_exch{"H222 LUT"}="2H22 LUT"; $resn_exch{"H361 LUT"}="1H36 LUT"; $resn_exch{"H362 LUT"}="2H36 LUT";
$resn_exch{"H363 LUT"}="3H36 LUT"; $resn_exch{"H371 LUT"}="1H37 LUT"; $resn_exch{"H372 LUT"}="2H37 LUT";
$resn_exch{"H373 LUT"}="3H37 LUT"; $resn_exch{"H381 LUT"}="1H38 LUT"; $resn_exch{"H382 LUT"}="2H38 LUT";
$resn_exch{"H383 LUT"}="3H38 LUT"; $resn_exch{"H391 LUT"}="1H39 LUT"; $resn_exch{"H392 LUT"}="2H39 LUT";
$resn_exch{"H393 LUT"}="3H39 LUT"; $resn_exch{"H401 LUT"}="1H40 LUT"; $resn_exch{"H402 LUT"}="2H40 LUT";
$resn_exch{"H403 LUT"}="3H40 LUT"; $resn_exch{"H2C1 LUX"}="1H2C LUX"; $resn_exch{"H2C2 LUX"}="2H2C LUX";
$resn_exch{"H4C1 LUX"}="1H4C LUX"; $resn_exch{"H4C2 LUX"}="2H4C LUX"; $resn_exch{"H101 LUX"}="1H10 LUX";
$resn_exch{"H102 LUX"}="2H10 LUX"; $resn_exch{"H111 LUX"}="1H11 LUX"; $resn_exch{"H112 LUX"}="2H11 LUX";
$resn_exch{"H121 LUX"}="1H12 LUX"; $resn_exch{"H122 LUX"}="2H12 LUX"; $resn_exch{"H141 LUX"}="1H14 LUX";
$resn_exch{"H142 LUX"}="2H14 LUX"; $resn_exch{"H151 LUX"}="1H15 LUX"; $resn_exch{"H152 LUX"}="2H15 LUX";
$resn_exch{"H161 LUX"}="1H16 LUX"; $resn_exch{"H162 LUX"}="2H16 LUX"; $resn_exch{"H163 LUX"}="3H16 LUX";
$resn_exch{"H171 LUX"}="1H17 LUX"; $resn_exch{"H172 LUX"}="2H17 LUX"; $resn_exch{"H173 LUX"}="3H17 LUX";
$resn_exch{"H181 LUX"}="1H18 LUX"; $resn_exch{"H182 LUX"}="2H18 LUX"; $resn_exch{"H191 LUX"}="1H19 LUX";
$resn_exch{"H192 LUX"}="2H19 LUX"; $resn_exch{"H193 LUX"}="3H19 LUX"; $resn_exch{"H201 LUX"}="1H20 LUX";
$resn_exch{"H202 LUX"}="2H20 LUX"; $resn_exch{"H203 LUX"}="3H20 LUX"; $resn_exch{"H221 LUX"}="1H22 LUX";
$resn_exch{"H222 LUX"}="2H22 LUX"; $resn_exch{"H241 LUX"}="1H24 LUX"; $resn_exch{"H271 LUX"}="1H27 LUX";
$resn_exch{"H281 LUX"}="1H28 LUX"; $resn_exch{"H282 LUX"}="2H28 LUX"; $resn_exch{"H301 LUX"}="1H30 LUX";
$resn_exch{"H302 LUX"}="2H30 LUX"; $resn_exch{"H311 LUX"}="1H31 LUX"; $resn_exch{"H312 LUX"}="2H31 LUX";
$resn_exch{"H321 LUX"}="1H32 LUX"; $resn_exch{"H322 LUX"}="2H32 LUX"; $resn_exch{"H341 LUX"}="1H34 LUX";
$resn_exch{"H342 LUX"}="2H34 LUX"; $resn_exch{"H351 LUX"}="1H35 LUX"; $resn_exch{"H352 LUX"}="2H35 LUX";
$resn_exch{"H361 LUX"}="1H36 LUX"; $resn_exch{"H362 LUX"}="2H36 LUX"; $resn_exch{"H363 LUX"}="3H36 LUX";
$resn_exch{"H371 LUX"}="1H37 LUX"; $resn_exch{"H372 LUX"}="2H37 LUX"; $resn_exch{"H373 LUX"}="3H37 LUX";
$resn_exch{"H381 LUX"}="1H38 LUX"; $resn_exch{"H382 LUX"}="2H38 LUX"; $resn_exch{"H383 LUX"}="3H38 LUX";
$resn_exch{"H391 LUX"}="1H39 LUX"; $resn_exch{"H392 LUX"}="2H39 LUX"; $resn_exch{"H393 LUX"}="3H39 LUX";
$resn_exch{"H401 LUX"}="1H40 LUX"; $resn_exch{"H402 LUX"}="2H40 LUX"; $resn_exch{"H403 LUX"}="3H40 LUX";
$resn_exch{"H161 LVA"}="1H16 LVA"; $resn_exch{"H162 LVA"}="2H16 LVA"; $resn_exch{"H171 LVA"}="1H17 LVA";
$resn_exch{"H172 LVA"}="2H17 LVA"; $resn_exch{"H181 LVA"}="1H18 LVA"; $resn_exch{"H182 LVA"}="2H18 LVA";
$resn_exch{"H191 LVA"}="1H19 LVA"; $resn_exch{"H192 LVA"}="2H19 LVA"; $resn_exch{"H201 LVA"}="1H20 LVA";
$resn_exch{"H202 LVA"}="2H20 LVA"; $resn_exch{"H203 LVA"}="3H20 LVA"; $resn_exch{"H211 LVA"}="1H21 LVA";
$resn_exch{"H212 LVA"}="2H21 LVA"; $resn_exch{"H231 LVA"}="1H23 LVA"; $resn_exch{"H232 LVA"}="2H23 LVA";
$resn_exch{"H233 LVA"}="3H23 LVA"; $resn_exch{"H271 LVA"}="1H27 LVA"; $resn_exch{"H272 LVA"}="2H27 LVA";
$resn_exch{"H281 LVA"}="1H28 LVA"; $resn_exch{"H282 LVA"}="2H28 LVA"; $resn_exch{"H283 LVA"}="3H28 LVA";
$resn_exch{"H291 LVA"}="1H29 LVA"; $resn_exch{"H292 LVA"}="2H29 LVA"; $resn_exch{"H293 LVA"}="3H29 LVA";
$resn_exch{" H   LVG"}="1HN  LVG"; $resn_exch{" H2  LVG"}="2HN  LVG"; $resn_exch{"HG11 LVG"}="1HG1 LVG";
$resn_exch{"HG12 LVG"}="2HG1 LVG"; $resn_exch{"HB11 LVS"}="1HB1 LVS"; $resn_exch{"HB12 LVS"}="2HB1 LVS";
$resn_exch{"HD11 LVS"}="1HD1 LVS"; $resn_exch{"HD12 LVS"}="2HD1 LVS"; $resn_exch{"HD13 LVS"}="3HD1 LVS";
$resn_exch{"HD21 LVS"}="1HD2 LVS"; $resn_exch{"HD22 LVS"}="2HD2 LVS"; $resn_exch{"HD23 LVS"}="3HD2 LVS";
$resn_exch{"HB21 LVS"}="1HB2 LVS"; $resn_exch{"HB22 LVS"}="2HB2 LVS"; $resn_exch{"HD31 LVS"}="1HD3 LVS";
$resn_exch{"HD32 LVS"}="2HD3 LVS"; $resn_exch{"HD33 LVS"}="3HD3 LVS"; $resn_exch{"HD41 LVS"}="1HD4 LVS";
$resn_exch{"HD42 LVS"}="2HD4 LVS"; $resn_exch{"HD43 LVS"}="3HD4 LVS"; $resn_exch{"HB31 LVS"}="1HB3 LVS";
$resn_exch{"HB32 LVS"}="2HB3 LVS"; $resn_exch{"HD51 LVS"}="1HD5 LVS"; $resn_exch{"HD52 LVS"}="2HD5 LVS";
$resn_exch{"HD53 LVS"}="3HD5 LVS"; $resn_exch{"HD61 LVS"}="1HD6 LVS"; $resn_exch{"HD62 LVS"}="2HD6 LVS";
$resn_exch{"HD63 LVS"}="3HD6 LVS"; $resn_exch{"H1'1 LVS"}="1H1* LVS"; $resn_exch{"H1'2 LVS"}="2H1* LVS";
$resn_exch{"H1'3 LVS"}="3H1* LVS"; $resn_exch{" H2' LVS"}=" H2* LVS"; $resn_exch{"HC41 LVS"}="1HC4 LVS";
$resn_exch{"HC42 LVS"}="2HC4 LVS"; $resn_exch{" H51 LX1"}="1H5  LX1"; $resn_exch{" H52 LX1"}="2H5  LX1";
$resn_exch{" H11 LX1"}="1H1  LX1"; $resn_exch{" H12 LX1"}="2H1  LX1"; $resn_exch{" H13 LX1"}="3H1  LX1";
$resn_exch{"H5C1 LXC"}="1H5C LXC"; $resn_exch{"H5C2 LXC"}="2H5C LXC"; $resn_exch{" H51 LXP"}="1H5  LXP";
$resn_exch{" H52 LXP"}="2H5  LXP"; $resn_exch{" H11 LXP"}="1H1  LXP"; $resn_exch{" H12 LXP"}="2H1  LXP";
$resn_exch{"HC11 LY1"}="1HC1 LY1"; $resn_exch{"HC12 LY1"}="2HC1 LY1"; $resn_exch{"HN11 LY1"}="1HN1 LY1";
$resn_exch{"HC21 LY1"}="1HC2 LY1"; $resn_exch{"HC22 LY1"}="2HC2 LY1"; $resn_exch{"HC31 LY1"}="1HC3 LY1";
$resn_exch{"HC32 LY1"}="2HC3 LY1"; $resn_exch{"HC41 LY1"}="1HC4 LY1"; $resn_exch{"HC42 LY1"}="2HC4 LY1";
$resn_exch{"HC51 LY1"}="1HC5 LY1"; $resn_exch{"HC61 LY1"}="1HC6 LY1"; $resn_exch{"H151 LY2"}="1H15 LY2";
$resn_exch{"H152 LY2"}="2H15 LY2"; $resn_exch{"H191 LY2"}="1H19 LY2"; $resn_exch{"H192 LY2"}="2H19 LY2";
$resn_exch{"H161 LY2"}="1H16 LY2"; $resn_exch{"H162 LY2"}="2H16 LY2"; $resn_exch{"H181 LY2"}="1H18 LY2";
$resn_exch{"H182 LY2"}="2H18 LY2"; $resn_exch{"HC71 LY3"}="1HC7 LY3"; $resn_exch{"HC72 LY3"}="2HC7 LY3";
$resn_exch{"HC81 LY3"}="1HC8 LY3"; $resn_exch{"HC82 LY3"}="2HC8 LY3"; $resn_exch{"HN41 LY3"}="1HN4 LY3";
$resn_exch{"HN42 LY3"}="2HN4 LY3"; $resn_exch{"H171 LY3"}="1H17 LY3"; $resn_exch{"H172 LY3"}="2H17 LY3";
$resn_exch{"H191 LY3"}="1H19 LY3"; $resn_exch{"H192 LY3"}="2H19 LY3"; $resn_exch{"HBD1 LY4"}="1HBD LY4";
$resn_exch{"HBD2 LY4"}="2HBD LY4"; $resn_exch{"HBB1 LY4"}="1HBB LY4"; $resn_exch{"HBB2 LY4"}="2HBB LY4";
$resn_exch{"HBA1 LY4"}="1HBA LY4"; $resn_exch{"HBA2 LY4"}="2HBA LY4"; $resn_exch{"HBC1 LY4"}="1HBC LY4";
$resn_exch{"HBC2 LY4"}="2HBC LY4"; $resn_exch{"HBG1 LY4"}="1HBG LY4"; $resn_exch{"HBG2 LY4"}="2HBG LY4";
$resn_exch{"HBI1 LY4"}="1HBI LY4"; $resn_exch{"HBI2 LY4"}="2HBI LY4"; $resn_exch{"HBI3 LY4"}="3HBI LY4";
$resn_exch{"HBJ1 LY4"}="1HBJ LY4"; $resn_exch{"HBJ2 LY4"}="2HBJ LY4"; $resn_exch{"HBJ3 LY4"}="3HBJ LY4";
$resn_exch{"HC71 LYA"}="1HC7 LYA"; $resn_exch{"HC72 LYA"}="2HC7 LYA"; $resn_exch{"HC81 LYA"}="1HC8 LYA";
$resn_exch{"HC82 LYA"}="2HC8 LYA"; $resn_exch{"H191 LYA"}="1H19 LYA"; $resn_exch{"H192 LYA"}="2H19 LYA";
$resn_exch{"H241 LYA"}="1H24 LYA"; $resn_exch{"H242 LYA"}="2H24 LYA"; $resn_exch{"H251 LYA"}="1H25 LYA";
$resn_exch{"H252 LYA"}="2H25 LYA"; $resn_exch{"HC81 LYB"}="1HC8 LYB"; $resn_exch{"HC82 LYB"}="2HC8 LYB";
$resn_exch{"HC91 LYB"}="1HC9 LYB"; $resn_exch{"HC92 LYB"}="2HC9 LYB"; $resn_exch{"HN41 LYB"}="1HN4 LYB";
$resn_exch{"HN42 LYB"}="2HN4 LYB"; $resn_exch{"H181 LYB"}="1H18 LYB"; $resn_exch{"H182 LYB"}="2H18 LYB";
$resn_exch{"H191 LYB"}="1H19 LYB"; $resn_exch{"H192 LYB"}="2H19 LYB"; $resn_exch{"H231 LYB"}="1H23 LYB";
$resn_exch{"H232 LYB"}="2H23 LYB"; $resn_exch{"H241 LYB"}="1H24 LYB"; $resn_exch{"H242 LYB"}="2H24 LYB";
$resn_exch{"H281 LYB"}="1H28 LYB"; $resn_exch{"H282 LYB"}="2H28 LYB"; $resn_exch{"H291 LYB"}="1H29 LYB";
$resn_exch{"H292 LYB"}="2H29 LYB"; $resn_exch{"H331 LYB"}="1H33 LYB"; $resn_exch{"H332 LYB"}="2H33 LYB";
$resn_exch{"H341 LYB"}="1H34 LYB"; $resn_exch{"H342 LYB"}="2H34 LYB"; $resn_exch{"H11A LYC"}="1H1  LYC";
$resn_exch{" H12 LYC"}="2H1  LYC"; $resn_exch{" H13 LYC"}="3H1  LYC"; $resn_exch{" H31 LYC"}="1H3  LYC";
$resn_exch{" H32 LYC"}="2H3  LYC"; $resn_exch{" H33 LYC"}="3H3  LYC"; $resn_exch{" H51 LYC"}="1H5  LYC";
$resn_exch{" H52 LYC"}="2H5  LYC"; $resn_exch{" H61 LYC"}="1H6  LYC"; $resn_exch{" H62 LYC"}="2H6  LYC";
$resn_exch{" H81 LYC"}="1H8  LYC"; $resn_exch{" H82 LYC"}="2H8  LYC"; $resn_exch{" H83 LYC"}="3H8  LYC";
$resn_exch{"H131 LYC"}="1H13 LYC"; $resn_exch{"H132 LYC"}="2H13 LYC"; $resn_exch{"H133 LYC"}="3H13 LYC";
$resn_exch{"H181 LYC"}="1H18 LYC"; $resn_exch{"H182 LYC"}="2H18 LYC"; $resn_exch{"H183 LYC"}="3H18 LYC";
$resn_exch{"H521 LYC"}="1H52 LYC"; $resn_exch{"H522 LYC"}="2H52 LYC"; $resn_exch{"H523 LYC"}="3H52 LYC";
$resn_exch{"H571 LYC"}="1H57 LYC"; $resn_exch{"H572 LYC"}="2H57 LYC"; $resn_exch{"H573 LYC"}="3H57 LYC";
$resn_exch{"H621 LYC"}="1H62 LYC"; $resn_exch{"H622 LYC"}="2H62 LYC"; $resn_exch{"H623 LYC"}="3H62 LYC";
$resn_exch{"H631 LYC"}="1H63 LYC"; $resn_exch{"H632 LYC"}="2H63 LYC"; $resn_exch{"H641 LYC"}="1H64 LYC";
$resn_exch{"H642 LYC"}="2H64 LYC"; $resn_exch{"H671 LYC"}="1H67 LYC"; $resn_exch{"H672 LYC"}="2H67 LYC";
$resn_exch{"H673 LYC"}="3H67 LYC"; $resn_exch{"H681 LYC"}="1H68 LYC"; $resn_exch{"H682 LYC"}="2H68 LYC";
$resn_exch{"H683 LYC"}="3H68 LYC"; $resn_exch{"HG11 LYD"}="1HG1 LYD"; $resn_exch{"HG12 LYD"}="2HG1 LYD";
$resn_exch{"HG13 LYD"}="3HG1 LYD"; $resn_exch{"HG21 LYD"}="1HG2 LYD"; $resn_exch{"HG22 LYD"}="2HG2 LYD";
$resn_exch{"HG23 LYD"}="3HG2 LYD"; $resn_exch{"HC81 LYD"}="1HC8 LYD"; $resn_exch{"HC82 LYD"}="2HC8 LYD";
$resn_exch{"HC71 LYD"}="1HC7 LYD"; $resn_exch{"HC72 LYD"}="2HC7 LYD"; $resn_exch{"HN21 LYD"}="1HN2 LYD";
$resn_exch{"HN22 LYD"}="2HN2 LYD"; $resn_exch{"HC41 LYL"}="1HC4 LYL"; $resn_exch{"HC42 LYL"}="2HC4 LYL";
$resn_exch{"H151 LYL"}="1H15 LYL"; $resn_exch{"H152 LYL"}="2H15 LYL"; $resn_exch{"H153 LYL"}="3H15 LYL";
$resn_exch{"H141 LYL"}="1H14 LYL"; $resn_exch{"H142 LYL"}="2H14 LYL"; $resn_exch{" H   LYM"}="1HN  LYM";
$resn_exch{" H2  LYM"}="2HN  LYM"; $resn_exch{" HB2 LYM"}="1HB  LYM"; $resn_exch{" HB3 LYM"}="2HB  LYM";
$resn_exch{" HG2 LYM"}="1HG  LYM"; $resn_exch{" HG3 LYM"}="2HG  LYM"; $resn_exch{" HD2 LYM"}="1HD  LYM";
$resn_exch{" HD3 LYM"}="2HD  LYM"; $resn_exch{" HE2 LYM"}="1HE  LYM"; $resn_exch{" HE3 LYM"}="2HE  LYM";
$resn_exch{" HZ1 LYM"}="1HZ  LYM"; $resn_exch{" HZ2 LYM"}="2HZ  LYM"; $resn_exch{" HM1 LYM"}="1HM  LYM";
$resn_exch{" HM2 LYM"}="2HM  LYM"; $resn_exch{" HM3 LYM"}="3HM  LYM"; $resn_exch{" H   LYN"}=" HN  LYN";
$resn_exch{" H2  LYN"}=" HN2 LYN"; $resn_exch{" HB2 LYN"}=" HB1 LYN"; $resn_exch{" HB3 LYN"}=" HB2 LYN";
$resn_exch{" HG2 LYN"}=" HG1 LYN"; $resn_exch{" HG3 LYN"}=" HG2 LYN"; $resn_exch{" HD2 LYN"}=" HD1 LYN";
$resn_exch{" HD3 LYN"}=" HD2 LYN"; $resn_exch{" HE2 LYN"}=" HE1 LYN"; $resn_exch{" HE3 LYN"}=" HE2 LYN";
$resn_exch{"HNT1 LYN"}="H1   LYN"; $resn_exch{"HNT2 LYN"}="H2   LYN"; $resn_exch{" HN1 LYP"}="1HN  LYP";
$resn_exch{" HN2 LYP"}="2HN  LYP"; $resn_exch{"HBC1 LYP"}="1HBC LYP"; $resn_exch{"HBC2 LYP"}="2HBC LYP";
$resn_exch{"HGC1 LYP"}="1HGC LYP"; $resn_exch{"HGC2 LYP"}="2HGC LYP"; $resn_exch{"HDC1 LYP"}="1HDC LYP";
$resn_exch{"HDC2 LYP"}="2HDC LYP"; $resn_exch{"HEC1 LYP"}="1HEC LYP"; $resn_exch{"HEC2 LYP"}="2HEC LYP";
$resn_exch{"HMC1 LYP"}="1HMC LYP"; $resn_exch{"HMC2 LYP"}="2HMC LYP"; $resn_exch{"HMC3 LYP"}="3HMC LYP";
$resn_exch{"HCD1 LYP"}="1HCD LYP"; $resn_exch{"HCD2 LYP"}="2HCD LYP"; $resn_exch{"HCA1 LYP"}="1HCA LYP";
$resn_exch{"HCA2 LYP"}="2HCA LYP"; $resn_exch{"HCA3 LYP"}="3HCA LYP"; $resn_exch{"HCC1 LYP"}="1HCC LYP";
$resn_exch{"HCC2 LYP"}="2HCC LYP"; $resn_exch{" H2  LYR"}=" HN2 LYR"; $resn_exch{" HB2 LYR"}=" HB1 LYR";
$resn_exch{" HB3 LYR"}=" HB2 LYR"; $resn_exch{" HG2 LYR"}=" HG1 LYR"; $resn_exch{" HG3 LYR"}=" HG2 LYR";
$resn_exch{" HD2 LYR"}=" HD1 LYR"; $resn_exch{" HD3 LYR"}=" HD2 LYR"; $resn_exch{" HE2 LYR"}=" HE1 LYR";
$resn_exch{" HE3 LYR"}=" HE2 LYR"; $resn_exch{" H11 LYR"}="1H1  LYR"; $resn_exch{" HC2 LYR"}=" H2  LYR";
$resn_exch{" H41 LYR"}="1H4  LYR"; $resn_exch{" H42 LYR"}="2H4  LYR"; $resn_exch{" H43 LYR"}="3H4  LYR";
$resn_exch{" H81 LYR"}="1H8  LYR"; $resn_exch{" H82 LYR"}="2H8  LYR"; $resn_exch{" H83 LYR"}="3H8  LYR";
$resn_exch{"H131 LYR"}="1H13 LYR"; $resn_exch{"H132 LYR"}="2H13 LYR"; $resn_exch{"H133 LYR"}="3H13 LYR";
$resn_exch{"H141 LYR"}="1H14 LYR"; $resn_exch{"H142 LYR"}="2H14 LYR"; $resn_exch{"H151 LYR"}="1H15 LYR";
$resn_exch{"H152 LYR"}="2H15 LYR"; $resn_exch{"H161 LYR"}="1H16 LYR"; $resn_exch{"H162 LYR"}="2H16 LYR";
$resn_exch{"H181 LYR"}="1H18 LYR"; $resn_exch{"H182 LYR"}="2H18 LYR"; $resn_exch{"H183 LYR"}="3H18 LYR";
$resn_exch{"H191 LYR"}="1H19 LYR"; $resn_exch{"H192 LYR"}="2H19 LYR"; $resn_exch{"H193 LYR"}="3H19 LYR";
$resn_exch{" H2  LYS"}=" HN2 LYS"; $resn_exch{" HB2 LYS"}="1HB  LYS"; $resn_exch{" HB3 LYS"}="2HB  LYS";
$resn_exch{" HG2 LYS"}="1HG  LYS"; $resn_exch{" HG3 LYS"}="2HG  LYS"; $resn_exch{" HD2 LYS"}="1HD  LYS";
$resn_exch{" HD3 LYS"}="2HD  LYS"; $resn_exch{" HE2 LYS"}="1HE  LYS"; $resn_exch{" HE3 LYS"}="2HE  LYS";
$resn_exch{" HZ1 LYS"}="1HZ  LYS"; $resn_exch{" HZ2 LYS"}="2HZ  LYS"; $resn_exch{" HZ3 LYS"}="3HZ  LYS";
$resn_exch{" H11 LYT"}="1H1  LYT"; $resn_exch{" H12 LYT"}="2H1  LYT"; $resn_exch{" HN1 LYT"}="1HN  LYT";
$resn_exch{" HN2 LYT"}="2HN  LYT"; $resn_exch{" H21 LYT"}="1H2  LYT"; $resn_exch{" H22 LYT"}="2H2  LYT";
$resn_exch{" H31 LYT"}="1H3  LYT"; $resn_exch{" H32 LYT"}="2H3  LYT"; $resn_exch{" H41 LYT"}="1H4  LYT";
$resn_exch{" H42 LYT"}="2H4  LYT"; $resn_exch{" H43 LYT"}="3H4  LYT"; $resn_exch{" HN1 LYW"}="1HN  LYW";
$resn_exch{" HN2 LYW"}="2HN  LYW"; $resn_exch{" H21 LYW"}="1H2  LYW"; $resn_exch{" H22 LYW"}="2H2  LYW";
$resn_exch{" H31 LYW"}="1H3  LYW"; $resn_exch{" H32 LYW"}="2H3  LYW"; $resn_exch{" H41 LYW"}="1H4  LYW";
$resn_exch{" H42 LYW"}="2H4  LYW"; $resn_exch{" H51 LYW"}="1H5  LYW"; $resn_exch{" H52 LYW"}="2H5  LYW";
$resn_exch{" H53 LYW"}="3H5  LYW"; $resn_exch{" H61 LYX"}="1H6  LYX"; $resn_exch{" H62 LYX"}="2H6  LYX";
$resn_exch{" H81 LYX"}="1H8  LYX"; $resn_exch{" H82 LYX"}="2H8  LYX"; $resn_exch{" H83 LYX"}="3H8  LYX";
$resn_exch{" H91 LYX"}="1H9  LYX"; $resn_exch{" H92 LYX"}="2H9  LYX"; $resn_exch{" H93 LYX"}="3H9  LYX";
$resn_exch{"H141 LYX"}="1H14 LYX"; $resn_exch{"H142 LYX"}="2H14 LYX"; $resn_exch{"H151 LYX"}="1H15 LYX";
$resn_exch{"H152 LYX"}="2H15 LYX"; $resn_exch{"H181 LYX"}="1H18 LYX"; $resn_exch{"H182 LYX"}="2H18 LYX";
$resn_exch{"H191 LYX"}="1H19 LYX"; $resn_exch{"H192 LYX"}="2H19 LYX"; $resn_exch{"H211 LYX"}="1H21 LYX";
$resn_exch{"H221 LYX"}="1H22 LYX"; $resn_exch{"H222 LYX"}="2H22 LYX"; $resn_exch{"H223 LYX"}="3H22 LYX";
$resn_exch{" HE3 LYX"}="1HE  LYX"; $resn_exch{" HE2 LYX"}="2HE  LYX"; $resn_exch{" HD3 LYX"}="1HD  LYX";
$resn_exch{" HD2 LYX"}="2HD  LYX"; $resn_exch{" HG3 LYX"}="1HG  LYX"; $resn_exch{" HG2 LYX"}="2HG  LYX";
$resn_exch{" HB3 LYX"}="1HB  LYX"; $resn_exch{" HB2 LYX"}="2HB  LYX"; $resn_exch{" H2  LYX"}=" HN2 LYX";
$resn_exch{"H401 LYX"}="1H40 LYX"; $resn_exch{"H402 LYX"}="2H40 LYX"; $resn_exch{"H601 LYX"}="1H60 LYX";
$resn_exch{"H602 LYX"}="2H60 LYX"; $resn_exch{" H2  LYZ"}=" HN2 LYZ"; $resn_exch{" HB2 LYZ"}="1HB  LYZ";
$resn_exch{" HB3 LYZ"}="2HB  LYZ"; $resn_exch{" HG2 LYZ"}="1HG  LYZ"; $resn_exch{" HG3 LYZ"}="2HG  LYZ";
$resn_exch{" HE2 LYZ"}="1HE  LYZ"; $resn_exch{" HE3 LYZ"}="2HE  LYZ"; $resn_exch{" HZ1 LYZ"}="1HZ  LYZ";
$resn_exch{" HZ2 LYZ"}="2HZ  LYZ"; $resn_exch{" HH  LYZ"}=" HO  LYZ"; $resn_exch{"H9C1 M01"}="1H9C M01";
$resn_exch{"H9C2 M01"}="2H9C M01"; $resn_exch{"H101 M01"}="1H10 M01"; $resn_exch{"H102 M01"}="2H10 M01";
$resn_exch{"H111 M01"}="1H11 M01"; $resn_exch{"H112 M01"}="2H11 M01"; $resn_exch{"H151 M01"}="1H15 M01";
$resn_exch{"H152 M01"}="2H15 M01"; $resn_exch{"H121 M01"}="1H12 M01"; $resn_exch{"H122 M01"}="2H12 M01";
$resn_exch{"H141 M01"}="1H14 M01"; $resn_exch{"H142 M01"}="2H14 M01"; $resn_exch{"HAO1 M02"}="1HAO M02";
$resn_exch{"HAO2 M02"}="2HAO M02"; $resn_exch{"HAK1 M02"}="1HAK M02"; $resn_exch{"HAK2 M02"}="2HAK M02";
$resn_exch{"H101 M02"}="1H10 M02"; $resn_exch{"H102 M02"}="2H10 M02"; $resn_exch{"H7C1 M02"}="1H7C M02";
$resn_exch{"H7C2 M02"}="2H7C M02"; $resn_exch{"HAP1 M02"}="1HAP M02"; $resn_exch{"HAP2 M02"}="2HAP M02";
$resn_exch{"H2C1 M03"}="1H2C M03"; $resn_exch{" HA2 M03"}="2HA  M03"; $resn_exch{"H8C1 M03"}="1H8C M03";
$resn_exch{"H8C2 M03"}="2H8C M03"; $resn_exch{" HA1 M03"}="1HA  M03"; $resn_exch{" HB2 M03"}="2HB  M03";
$resn_exch{" HB1 M03"}="1HB  M03"; $resn_exch{" HC2 M03"}="2HC  M03"; $resn_exch{"H7C1 M03"}="1H7C M03";
$resn_exch{"H7C2 M03"}="2H7C M03"; $resn_exch{" HC1 M03"}="1HC  M03"; $resn_exch{" HD2 M03"}="2HD  M03";
$resn_exch{"H2C1 M04"}="1H2C M04"; $resn_exch{"H2C2 M04"}="2H2C M04"; $resn_exch{"H8C1 M04"}="1H8C M04";
$resn_exch{"H8C2 M04"}="2H8C M04"; $resn_exch{"H3C1 M04"}="1H3C M04"; $resn_exch{"H3C2 M04"}="2H3C M04";
$resn_exch{"H5C1 M04"}="1H5C M04"; $resn_exch{"H5C2 M04"}="2H5C M04"; $resn_exch{"H6N1 M04"}="1H6N M04";
$resn_exch{"H6N2 M04"}="2H6N M04"; $resn_exch{"H7C1 M04"}="1H7C M04"; $resn_exch{"H7C2 M04"}="2H7C M04";
$resn_exch{"H131 M04"}="1H13 M04"; $resn_exch{"H132 M04"}="2H13 M04"; $resn_exch{"H8C1 M05"}="1H8C M05";
$resn_exch{"H8C2 M05"}="2H8C M05"; $resn_exch{"H2C1 M05"}="1H2C M05"; $resn_exch{"H2C2 M05"}="2H2C M05";
$resn_exch{"H3C1 M05"}="1H3C M05"; $resn_exch{"H3C2 M05"}="2H3C M05"; $resn_exch{"H4C1 M05"}="1H4C M05";
$resn_exch{"H4C2 M05"}="2H4C M05"; $resn_exch{"H6N1 M05"}="1H6N M05"; $resn_exch{"H6N2 M05"}="2H6N M05";
$resn_exch{"H171 M05"}="1H17 M05"; $resn_exch{"H172 M05"}="2H17 M05"; $resn_exch{"HDK1 M0E"}="1HDK M0E";
$resn_exch{"HDK2 M0E"}="2HDK M0E"; $resn_exch{"HAU1 M0E"}="1HAU M0E"; $resn_exch{"HAU2 M0E"}="2HAU M0E";
$resn_exch{"HAS1 M0E"}="1HAS M0E"; $resn_exch{"HAS2 M0E"}="2HAS M0E"; $resn_exch{"HAS3 M0E"}="3HAS M0E";
$resn_exch{"HAT1 M0E"}="1HAT M0E"; $resn_exch{"HAT2 M0E"}="2HAT M0E"; $resn_exch{"HAH1 M0E"}="1HAH M0E";
$resn_exch{"HAH2 M0E"}="2HAH M0E"; $resn_exch{"HAH3 M0E"}="3HAH M0E"; $resn_exch{" H61 M0E"}="1H6  M0E";
$resn_exch{" H62 M0E"}="2H6  M0E"; $resn_exch{"HBO1 M0E"}="1HBO M0E"; $resn_exch{"HBO2 M0E"}="2HBO M0E";
$resn_exch{"HCB1 M0E"}="1HCB M0E"; $resn_exch{"HCB2 M0E"}="2HCB M0E"; $resn_exch{"HCB3 M0E"}="3HCB M0E";
$resn_exch{"HBZ1 M0E"}="1HBZ M0E"; $resn_exch{"HBZ2 M0E"}="2HBZ M0E"; $resn_exch{"HBZ3 M0E"}="3HBZ M0E";
$resn_exch{" H11 M0E"}="1H1  M0E"; $resn_exch{" H41 M0E"}="1H4  M0E"; $resn_exch{" H42 M0E"}="2H4  M0E";
$resn_exch{" H43 M0E"}="3H4  M0E"; $resn_exch{" H51 M0E"}="1H5  M0E"; $resn_exch{" H52 M0E"}="2H5  M0E";
$resn_exch{"H361 M0E"}="1H36 M0E"; $resn_exch{"H362 M0E"}="2H36 M0E"; $resn_exch{"H101 M0E"}="1H10 M0E";
$resn_exch{"H102 M0E"}="2H10 M0E"; $resn_exch{"H103 M0E"}="3H10 M0E"; $resn_exch{"H111 M0E"}="1H11 M0E";
$resn_exch{"H112 M0E"}="2H11 M0E"; $resn_exch{"H113 M0E"}="3H11 M0E"; $resn_exch{"H121 M0E"}="1H12 M0E";
$resn_exch{"H122 M0E"}="2H12 M0E"; $resn_exch{"H131 M0E"}="1H13 M0E"; $resn_exch{"H132 M0E"}="2H13 M0E";
$resn_exch{"H151 M0E"}="1H15 M0E"; $resn_exch{"H152 M0E"}="2H15 M0E"; $resn_exch{"H161 M0E"}="1H16 M0E";
$resn_exch{"H162 M0E"}="2H16 M0E"; $resn_exch{"H191 M0E"}="1H19 M0E"; $resn_exch{"H192 M0E"}="2H19 M0E";
$resn_exch{"H193 M0E"}="3H19 M0E"; $resn_exch{"H201 M0E"}="1H20 M0E"; $resn_exch{"H202 M0E"}="2H20 M0E";
$resn_exch{"H211 M0E"}="1H21 M0E"; $resn_exch{"H212 M0E"}="2H21 M0E"; $resn_exch{"H241 M0E"}="1H24 M0E";
$resn_exch{"H242 M0E"}="2H24 M0E"; $resn_exch{"H243 M0E"}="3H24 M0E"; $resn_exch{"H251 M0E"}="1H25 M0E";
$resn_exch{"H252 M0E"}="2H25 M0E"; $resn_exch{"H253 M0E"}="3H25 M0E"; $resn_exch{" H21 M0E"}="1H2  M0E";
$resn_exch{" HN1 M0H"}="1HN  M0H"; $resn_exch{" HN2 M0H"}="2HN  M0H"; $resn_exch{" HB1 M0H"}="1HB  M0H";
$resn_exch{" HB2 M0H"}="2HB  M0H"; $resn_exch{" HD1 M0H"}="1HD  M0H"; $resn_exch{" HD2 M0H"}="2HD  M0H";
$resn_exch{" H81 M0N"}="1H8  M0N"; $resn_exch{" H82 M0N"}="2H8  M0N"; $resn_exch{"H121 M13"}="1H12 M13";
$resn_exch{"H122 M13"}="2H12 M13"; $resn_exch{" HM1 M13"}="1HM  M13"; $resn_exch{" HM2 M13"}="2HM  M13";
$resn_exch{" HM3 M13"}="3HM  M13"; $resn_exch{" H61 M13"}="1H6  M13"; $resn_exch{" H62 M13"}="2H6  M13";
$resn_exch{"HN61 M1A"}="1HN6 M1A"; $resn_exch{"HN62 M1A"}="2HN6 M1A"; $resn_exch{" H11 M1A"}="1H1  M1A";
$resn_exch{" H12 M1A"}="2H1  M1A"; $resn_exch{" H13 M1A"}="3H1  M1A"; $resn_exch{"H161 M1B"}="1H16 M1B";
$resn_exch{"H162 M1B"}="2H16 M1B"; $resn_exch{"H261 M1B"}="1H26 M1B"; $resn_exch{"H262 M1B"}="2H26 M1B";
$resn_exch{"H131 M1C"}="1H13 M1C"; $resn_exch{"H132 M1C"}="2H13 M1C"; $resn_exch{"H121 M1C"}="1H12 M1C";
$resn_exch{"H122 M1C"}="2H12 M1C"; $resn_exch{"HN31 M1C"}="1HN3 M1C"; $resn_exch{"HN32 M1C"}="2HN3 M1C";
$resn_exch{" H91 M1C"}="1H9  M1C"; $resn_exch{" H92 M1C"}="2H9  M1C"; $resn_exch{" H41 M1C"}="1H4  M1C";
$resn_exch{" H42 M1C"}="2H4  M1C"; $resn_exch{" H51 M1C"}="1H5  M1C"; $resn_exch{" H52 M1C"}="2H5  M1C";
$resn_exch{" H61 M1C"}="1H6  M1C"; $resn_exch{" H62 M1C"}="2H6  M1C"; $resn_exch{" H71 M1C"}="1H7  M1C";
$resn_exch{" H72 M1C"}="2H7  M1C"; $resn_exch{" H73 M1C"}="3H7  M1C"; $resn_exch{"HOP2 M1G"}="2HOP M1G";
$resn_exch{"HOP3 M1G"}="3HOP M1G"; $resn_exch{" H5' M1G"}="1H5* M1G"; $resn_exch{"H5'' M1G"}="2H5* M1G";
$resn_exch{" H4' M1G"}=" H4* M1G"; $resn_exch{" H3' M1G"}=" H3* M1G"; $resn_exch{"HO3' M1G"}="H3T  M1G";
$resn_exch{" H2' M1G"}="1H2* M1G"; $resn_exch{"H2'' M1G"}="2H2* M1G"; $resn_exch{" H1' M1G"}=" H1* M1G";
$resn_exch{" H11 M1L"}="1H1  M1L"; $resn_exch{" H12 M1L"}="2H1  M1L"; $resn_exch{" H81 M1L"}="1H8  M1L";
$resn_exch{" H82 M1L"}="2H8  M1L"; $resn_exch{"H101 M1L"}="1H10 M1L"; $resn_exch{"H102 M1L"}="2H10 M1L";
$resn_exch{"H131 M1L"}="1H13 M1L"; $resn_exch{"H132 M1L"}="2H13 M1L"; $resn_exch{"H133 M1L"}="3H13 M1L";
$resn_exch{"H201 M1L"}="1H20 M1L"; $resn_exch{"H202 M1L"}="2H20 M1L"; $resn_exch{"H203 M1L"}="3H20 M1L";
$resn_exch{"H211 M1L"}="1H21 M1L"; $resn_exch{"H212 M1L"}="2H21 M1L"; $resn_exch{"H213 M1L"}="3H21 M1L";
$resn_exch{"H231 M1L"}="1H23 M1L"; $resn_exch{"H232 M1L"}="2H23 M1L"; $resn_exch{"H241 M1L"}="1H24 M1L";
$resn_exch{"H242 M1L"}="2H24 M1L"; $resn_exch{"H221 M1N"}="1H22 M1N"; $resn_exch{"H222 M1N"}="2H22 M1N";
$resn_exch{"H251 M1N"}="1H25 M1N"; $resn_exch{"H252 M1N"}="2H25 M1N"; $resn_exch{"H253 M1N"}="3H25 M1N";
$resn_exch{"H241 M1N"}="1H24 M1N"; $resn_exch{"H242 M1N"}="2H24 M1N"; $resn_exch{"H243 M1N"}="3H24 M1N";
$resn_exch{"H141 M1N"}="1H14 M1N"; $resn_exch{"H142 M1N"}="2H14 M1N"; $resn_exch{"H131 M1N"}="1H13 M1N";
$resn_exch{"H132 M1N"}="2H13 M1N"; $resn_exch{"H111 M1N"}="1H11 M1N"; $resn_exch{"H112 M1N"}="2H11 M1N";
$resn_exch{"H101 M1N"}="1H10 M1N"; $resn_exch{"H102 M1N"}="2H10 M1N"; $resn_exch{" H51 M1N"}="1H5  M1N";
$resn_exch{" H52 M1N"}="2H5  M1N"; $resn_exch{" H61 M1P"}="1H6  M1P"; $resn_exch{" H62 M1P"}="2H6  M1P";
$resn_exch{"H8C1 M1S"}="1H8C M1S"; $resn_exch{"H8C2 M1S"}="2H8C M1S"; $resn_exch{"H101 M1S"}="1H10 M1S";
$resn_exch{"H102 M1S"}="2H10 M1S"; $resn_exch{"H111 M1S"}="1H11 M1S"; $resn_exch{"H112 M1S"}="2H11 M1S";
$resn_exch{"H121 M1S"}="1H12 M1S"; $resn_exch{"H122 M1S"}="2H12 M1S"; $resn_exch{"H131 M1S"}="1H13 M1S";
$resn_exch{"H132 M1S"}="2H13 M1S"; $resn_exch{"H141 M1S"}="1H14 M1S"; $resn_exch{"H142 M1S"}="2H14 M1S";
$resn_exch{"H171 M1S"}="1H17 M1S"; $resn_exch{"H172 M1S"}="2H17 M1S"; $resn_exch{"H181 M1S"}="1H18 M1S";
$resn_exch{"H182 M1S"}="2H18 M1S"; $resn_exch{"HN11 M25"}="1HN1 M25"; $resn_exch{"HN12 M25"}="2HN1 M25";
$resn_exch{" H71 M25"}="1H7  M25"; $resn_exch{" H72 M25"}="2H7  M25"; $resn_exch{" H81 M25"}="1H8  M25";
$resn_exch{" H82 M25"}="2H8  M25"; $resn_exch{"H101 M25"}="1H10 M25"; $resn_exch{"H102 M25"}="2H10 M25";
$resn_exch{"H103 M25"}="3H10 M25"; $resn_exch{"HN31 M28"}="1HN3 M28"; $resn_exch{"HN32 M28"}="2HN3 M28";
$resn_exch{" H71 M28"}="1H7  M28"; $resn_exch{" H72 M28"}="2H7  M28"; $resn_exch{" H81 M28"}="1H8  M28";
$resn_exch{" H82 M28"}="2H8  M28"; $resn_exch{" HN1 M29"}="1HN  M29"; $resn_exch{" HN2 M29"}="2HN  M29";
$resn_exch{" H71 M29"}="1H7  M29"; $resn_exch{" H72 M29"}="2H7  M29"; $resn_exch{" H81 M29"}="1H8  M29";
$resn_exch{" H82 M29"}="2H8  M29"; $resn_exch{"H101 M29"}="1H10 M29"; $resn_exch{"H102 M29"}="2H10 M29";
$resn_exch{"H111 M29"}="1H11 M29"; $resn_exch{"H112 M29"}="2H11 M29"; $resn_exch{"H113 M29"}="3H11 M29";
$resn_exch{"H111 M2C"}="1H11 M2C"; $resn_exch{"H112 M2C"}="2H11 M2C"; $resn_exch{"H121 M2C"}="1H12 M2C";
$resn_exch{"H122 M2C"}="2H12 M2C"; $resn_exch{"H141 M2C"}="1H14 M2C"; $resn_exch{"H142 M2C"}="2H14 M2C";
$resn_exch{"H143 M2C"}="3H14 M2C"; $resn_exch{"H101 M2C"}="1H10 M2C"; $resn_exch{"H102 M2C"}="2H10 M2C";
$resn_exch{"HOP2 M2G"}="2HOP M2G"; $resn_exch{"HOP3 M2G"}="3HOP M2G"; $resn_exch{" H5' M2G"}="1H5* M2G";
$resn_exch{"H5'' M2G"}="2H5* M2G"; $resn_exch{" H4' M2G"}=" H4* M2G"; $resn_exch{" H3' M2G"}=" H3* M2G";
$resn_exch{"HO3' M2G"}="*HO3 M2G"; $resn_exch{" H2' M2G"}=" H2* M2G"; $resn_exch{"HO2' M2G"}="*HO2 M2G";
$resn_exch{" H1' M2G"}=" H1* M2G"; $resn_exch{"HM11 M2G"}="1HM1 M2G"; $resn_exch{"HM12 M2G"}="2HM1 M2G";
$resn_exch{"HM13 M2G"}="3HM1 M2G"; $resn_exch{"HM21 M2G"}="1HM2 M2G"; $resn_exch{"HM22 M2G"}="2HM2 M2G";
$resn_exch{"HM23 M2G"}="3HM2 M2G"; $resn_exch{" H11 M2M"}="1H1  M2M"; $resn_exch{" H12 M2M"}="2H1  M2M";
$resn_exch{" H13 M2M"}="3H1  M2M"; $resn_exch{" H21 M2M"}="1H2  M2M"; $resn_exch{" H22 M2M"}="2H2  M2M";
$resn_exch{" H31 M2M"}="1H3  M2M"; $resn_exch{" H32 M2M"}="2H3  M2M"; $resn_exch{" H41 M2M"}="1H4  M2M";
$resn_exch{" H42 M2M"}="2H4  M2M"; $resn_exch{" H51 M2M"}="1H5  M2M"; $resn_exch{" H52 M2M"}="2H5  M2M";
$resn_exch{" H61 M2M"}="1H6  M2M"; $resn_exch{" H62 M2M"}="2H6  M2M"; $resn_exch{" H63 M2M"}="3H6  M2M";
$resn_exch{"H11A M2P"}="1H1  M2P"; $resn_exch{" H12 M2P"}="2H1  M2P"; $resn_exch{" H61 M2P"}="1H6  M2P";
$resn_exch{"H62A M2P"}="2H6  M2P"; $resn_exch{"H3B1 M33"}="1H3B M33"; $resn_exch{"H3B2 M33"}="2H3B M33";
$resn_exch{"H3B3 M33"}="3H3B M33"; $resn_exch{"H5'1 M33"}="1H5' M33"; $resn_exch{"H5'2 M33"}="2H5' M33";
$resn_exch{"HN61 M33"}="1HN6 M33"; $resn_exch{"HN62 M33"}="2HN6 M33"; $resn_exch{"H221 M38"}="1H22 M38";
$resn_exch{"H222 M38"}="2H22 M38"; $resn_exch{"H231 M38"}="1H23 M38"; $resn_exch{"H232 M38"}="2H23 M38";
$resn_exch{"H251 M38"}="1H25 M38"; $resn_exch{"H252 M38"}="2H25 M38"; $resn_exch{"HN11 M3C"}="1HN1 M3C";
$resn_exch{"HN12 M3C"}="2HN1 M3C"; $resn_exch{"H101 M3C"}="1H10 M3C"; $resn_exch{"H102 M3C"}="2H10 M3C";
$resn_exch{"H111 M3C"}="1H11 M3C"; $resn_exch{"H112 M3C"}="2H11 M3C"; $resn_exch{"H131 M3C"}="1H13 M3C";
$resn_exch{"H132 M3C"}="2H13 M3C"; $resn_exch{"H133 M3C"}="3H13 M3C"; $resn_exch{" H   M3L"}="1HN  M3L";
$resn_exch{" H2  M3L"}="2HN  M3L"; $resn_exch{" HB2 M3L"}="1HB  M3L"; $resn_exch{" HB3 M3L"}="2HB  M3L";
$resn_exch{" HG2 M3L"}="1HG  M3L"; $resn_exch{" HG3 M3L"}="2HG  M3L"; $resn_exch{" HD2 M3L"}="1HD  M3L";
$resn_exch{" HD3 M3L"}="2HD  M3L"; $resn_exch{" HE2 M3L"}="1HE  M3L"; $resn_exch{" HE3 M3L"}="2HE  M3L";
$resn_exch{"HM11 M3L"}="1HM1 M3L"; $resn_exch{"HM12 M3L"}="2HM1 M3L"; $resn_exch{"HM13 M3L"}="3HM1 M3L";
$resn_exch{"HM21 M3L"}="1HM2 M3L"; $resn_exch{"HM22 M3L"}="2HM2 M3L"; $resn_exch{"HM23 M3L"}="3HM2 M3L";
$resn_exch{"HM31 M3L"}="1HM3 M3L"; $resn_exch{"HM32 M3L"}="2HM3 M3L"; $resn_exch{"HM33 M3L"}="3HM3 M3L";
$resn_exch{"H181 M54"}="1H18 M54"; $resn_exch{"H182 M54"}="2H18 M54"; $resn_exch{"H191 M54"}="1H19 M54";
$resn_exch{"H192 M54"}="2H19 M54"; $resn_exch{"H201 M54"}="1H20 M54"; $resn_exch{"H202 M54"}="2H20 M54";
$resn_exch{"H211 M54"}="1H21 M54"; $resn_exch{"H212 M54"}="2H21 M54"; $resn_exch{"HN'2 M5M"}="2HN* M5M";
$resn_exch{"H7'1 M5M"}="1H7* M5M"; $resn_exch{"H7'2 M5M"}="2H7* M5M"; $resn_exch{"H7'3 M5M"}="3H7* M5M";
$resn_exch{"HOP2 M5M"}="2HOP M5M"; $resn_exch{" H5' M5M"}="1H5* M5M"; $resn_exch{"H5'' M5M"}="2H5* M5M";
$resn_exch{" H4' M5M"}=" H4* M5M"; $resn_exch{" H1' M5M"}=" H1* M5M"; $resn_exch{"HN41 M5M"}="1HN4 M5M";
$resn_exch{"HN42 M5M"}="2HN4 M5M"; $resn_exch{" H2' M5M"}=" H2* M5M"; $resn_exch{" H3' M5M"}=" H3* M5M";
$resn_exch{"HO3' M5M"}="H3T  M5M"; $resn_exch{"HOP3 M5M"}="3HOP M5M"; $resn_exch{" H71 M5P"}="1H7  M5P";
$resn_exch{" H72 M5P"}="2H7  M5P"; $resn_exch{" H81 M5P"}="1H8  M5P"; $resn_exch{" H82 M5P"}="2H8  M5P";
$resn_exch{" H91 M5P"}="1H9  M5P"; $resn_exch{" H92 M5P"}="2H9  M5P"; $resn_exch{"H111 M5P"}="1H11 M5P";
$resn_exch{"H112 M5P"}="2H11 M5P"; $resn_exch{" H61 M6P"}="1H6  M6P"; $resn_exch{" H62 M6P"}="2H6  M6P";
$resn_exch{"HOP2 M6P"}="2HOP M6P"; $resn_exch{"HOP3 M6P"}="3HOP M6P"; $resn_exch{" H61 M6R"}="1H6  M6R";
$resn_exch{" H62 M6R"}="2H6  M6R"; $resn_exch{"HN21 M6R"}="1HN2 M6R"; $resn_exch{"HN22 M6R"}="2HN2 M6R";
$resn_exch{" H11 M6R"}="1H1  M6R"; $resn_exch{" H12 M6R"}="2H1  M6R"; $resn_exch{"HM21 M6T"}="1HM2 M6T";
$resn_exch{"HM22 M6T"}="2HM2 M6T"; $resn_exch{"HM23 M6T"}="3HM2 M6T"; $resn_exch{"HN41 M6T"}="1HN4 M6T";
$resn_exch{"HN42 M6T"}="2HN4 M6T"; $resn_exch{"H7'1 M6T"}="1H7' M6T"; $resn_exch{"H7'2 M6T"}="2H7' M6T";
$resn_exch{"HM41 M6T"}="1HM4 M6T"; $resn_exch{"HM42 M6T"}="2HM4 M6T"; $resn_exch{"HM43 M6T"}="3HM4 M6T";
$resn_exch{"HM61 M6T"}="1HM6 M6T"; $resn_exch{"HM62 M6T"}="2HM6 M6T"; $resn_exch{"HM63 M6T"}="3HM6 M6T";
$resn_exch{" H61 M6T"}="1H6  M6T"; $resn_exch{" H62 M6T"}="2H6  M6T"; $resn_exch{" H71 M6T"}="1H7  M6T";
$resn_exch{" H72 M6T"}="2H7  M6T"; $resn_exch{"HOA2 M6T"}="2HOA M6T"; $resn_exch{"HOB2 M6T"}="2HOB M6T";
$resn_exch{"H221 M77"}="1H22 M77"; $resn_exch{"H222 M77"}="2H22 M77"; $resn_exch{"H211 M77"}="1H21 M77";
$resn_exch{"H212 M77"}="2H21 M77"; $resn_exch{"H161 M77"}="1H16 M77"; $resn_exch{"H162 M77"}="2H16 M77";
$resn_exch{"H201 M77"}="1H20 M77"; $resn_exch{"H202 M77"}="2H20 M77"; $resn_exch{"H151 M77"}="1H15 M77";
$resn_exch{"H152 M77"}="2H15 M77"; $resn_exch{"HOA2 M7G"}="2HOA M7G"; $resn_exch{"HOB2 M7G"}="2HOB M7G";
$resn_exch{"HOB3 M7G"}="3HOB M7G"; $resn_exch{"H5'1 M7G"}="1H5* M7G"; $resn_exch{"H5'2 M7G"}="2H5* M7G";
$resn_exch{" H4' M7G"}=" H4* M7G"; $resn_exch{" H3' M7G"}=" H3* M7G"; $resn_exch{"HO3' M7G"}="*HO3 M7G";
$resn_exch{" H2' M7G"}=" H2* M7G"; $resn_exch{"HO2' M7G"}="*HO2 M7G"; $resn_exch{" H1' M7G"}=" H1* M7G";
$resn_exch{" H81 M7G"}="1H8  M7G"; $resn_exch{" H82 M7G"}="2H8  M7G"; $resn_exch{"HM71 M7G"}="1HM7 M7G";
$resn_exch{"HM72 M7G"}="2HM7 M7G"; $resn_exch{"HM73 M7G"}="3HM7 M7G"; $resn_exch{"HN21 M7G"}="1HN2 M7G";
$resn_exch{"HN22 M7G"}="2HN2 M7G"; $resn_exch{" H71 M7P"}="1H7  M7P"; $resn_exch{" H72 M7P"}="2H7  M7P";
$resn_exch{"HH31 M8C"}="1HH3 M8C"; $resn_exch{"HH32 M8C"}="2HH3 M8C"; $resn_exch{"HH33 M8C"}="3HH3 M8C";
$resn_exch{"H121 M98"}="1H12 M98"; $resn_exch{"H122 M98"}="2H12 M98"; $resn_exch{"H131 M98"}="1H13 M98";
$resn_exch{"H132 M98"}="2H13 M98"; $resn_exch{"H181 M98"}="1H18 M98"; $resn_exch{"H182 M98"}="2H18 M98";
$resn_exch{" H11 M99"}="1H1  M99"; $resn_exch{"H121 M99"}="1H12 M99"; $resn_exch{"H122 M99"}="2H12 M99";
$resn_exch{"H131 M99"}="1H13 M99"; $resn_exch{"H132 M99"}="2H13 M99"; $resn_exch{" H88 M99"}="8H8  M99";
$resn_exch{" H44 M99"}="4H4  M99"; $resn_exch{" H66 M99"}="6H6  M99"; $resn_exch{" H77 M99"}="7H7  M99";
$resn_exch{" H99 M99"}="9H9  M99"; $resn_exch{"H111 M99"}="11H1 M99"; $resn_exch{"H181 M99"}="1H18 M99";
$resn_exch{"H182 M99"}="2H18 M99"; $resn_exch{"H220 M99"}="20H2 M99"; $resn_exch{"H221 M99"}="21H2 M99";
$resn_exch{"H222 M99"}="22H2 M99"; $resn_exch{"H223 M99"}="23H2 M99"; $resn_exch{"H131 MA0"}="1H13 MA0";
$resn_exch{"H132 MA0"}="2H13 MA0"; $resn_exch{"H1C1 MA0"}="1H1C MA0"; $resn_exch{"H1C2 MA0"}="2H1C MA0";
$resn_exch{"H2C1 MA0"}="1H2C MA0"; $resn_exch{"H2C2 MA0"}="2H2C MA0"; $resn_exch{"H101 MA0"}="1H10 MA0";
$resn_exch{"H102 MA0"}="2H10 MA0"; $resn_exch{"H103 MA0"}="3H10 MA0"; $resn_exch{" H61 MA1"}="1H6  MA1";
$resn_exch{" H62 MA1"}="2H6  MA1"; $resn_exch{" H61 MA2"}="1H6  MA2"; $resn_exch{" H62 MA2"}="2H6  MA2";
$resn_exch{" H71 MA2"}="1H7  MA2"; $resn_exch{" H72 MA2"}="2H7  MA2"; $resn_exch{" H73 MA2"}="3H7  MA2";
$resn_exch{" H61 MA3"}="1H6  MA3"; $resn_exch{" H62 MA3"}="2H6  MA3"; $resn_exch{" H71 MA3"}="1H7  MA3";
$resn_exch{" H72 MA3"}="2H7  MA3"; $resn_exch{" H73 MA3"}="3H7  MA3"; $resn_exch{" H11 MA4"}="1H1  MA4";
$resn_exch{" H21 MA4"}="1H2  MA4"; $resn_exch{" H31 MA4"}="1H3  MA4"; $resn_exch{" H41 MA4"}="1H4  MA4";
$resn_exch{" H51 MA4"}="1H5  MA4"; $resn_exch{" H61 MA4"}="1H6  MA4"; $resn_exch{" H62 MA4"}="2H6  MA4";
$resn_exch{"H101 MA4"}="1H10 MA4"; $resn_exch{"H201 MA4"}="1H20 MA4"; $resn_exch{"H301 MA4"}="1H30 MA4";
$resn_exch{"H401 MA4"}="1H40 MA4"; $resn_exch{"H501 MA4"}="1H50 MA4"; $resn_exch{"H601 MA4"}="1H60 MA4";
$resn_exch{"H602 MA4"}="2H60 MA4"; $resn_exch{"H111 MA4"}="1H11 MA4"; $resn_exch{"H112 MA4"}="2H11 MA4";
$resn_exch{"H211 MA4"}="1H21 MA4"; $resn_exch{"H212 MA4"}="2H21 MA4"; $resn_exch{"H311 MA4"}="1H31 MA4";
$resn_exch{"H312 MA4"}="2H31 MA4"; $resn_exch{"H411 MA4"}="1H41 MA4"; $resn_exch{"H412 MA4"}="2H41 MA4";
$resn_exch{"H511 MA4"}="1H51 MA4"; $resn_exch{"H512 MA4"}="2H51 MA4"; $resn_exch{"H611 MA4"}="1H61 MA4";
$resn_exch{"H612 MA4"}="2H61 MA4"; $resn_exch{"H121 MA4"}="1H12 MA4"; $resn_exch{"H221 MA4"}="1H22 MA4";
$resn_exch{"H222 MA4"}="2H22 MA4"; $resn_exch{"H321 MA4"}="1H32 MA4"; $resn_exch{"H322 MA4"}="2H32 MA4";
$resn_exch{"H421 MA4"}="1H42 MA4"; $resn_exch{"H422 MA4"}="2H42 MA4"; $resn_exch{"H521 MA4"}="1H52 MA4";
$resn_exch{"H522 MA4"}="2H52 MA4"; $resn_exch{"H621 MA4"}="1H62 MA4"; $resn_exch{"H622 MA4"}="2H62 MA4";
$resn_exch{"H421 MA5"}="1H42 MA5"; $resn_exch{"H422 MA5"}="2H42 MA5"; $resn_exch{"H321 MA5"}="1H32 MA5";
$resn_exch{"H322 MA5"}="2H32 MA5"; $resn_exch{"H221 MA5"}="1H22 MA5"; $resn_exch{"H222 MA5"}="2H22 MA5";
$resn_exch{"H121 MA5"}="1H12 MA5"; $resn_exch{"H122 MA5"}="2H12 MA5"; $resn_exch{"H621 MA5"}="1H62 MA5";
$resn_exch{"H622 MA5"}="2H62 MA5"; $resn_exch{"H611 MA5"}="1H61 MA5"; $resn_exch{"H612 MA5"}="2H61 MA5";
$resn_exch{"H111 MA5"}="1H11 MA5"; $resn_exch{"H112 MA5"}="2H11 MA5"; $resn_exch{"H601 MA5"}="1H60 MA5";
$resn_exch{"H602 MA5"}="2H60 MA5"; $resn_exch{" H61 MA5"}="1H6  MA5"; $resn_exch{" H62 MA5"}="2H6  MA5";
$resn_exch{"HOP1 MA6"}="1HOP MA6"; $resn_exch{"HOP2 MA6"}="2HOP MA6"; $resn_exch{" H5' MA6"}="1H5* MA6";
$resn_exch{"H5'' MA6"}="2H5* MA6"; $resn_exch{" H4' MA6"}=" H4* MA6"; $resn_exch{" H1' MA6"}=" H1* MA6";
$resn_exch{" H91 MA6"}="1H9  MA6"; $resn_exch{" H92 MA6"}="2H9  MA6"; $resn_exch{" H93 MA6"}="3H9  MA6";
$resn_exch{"H101 MA6"}="1H10 MA6"; $resn_exch{"H102 MA6"}="2H10 MA6"; $resn_exch{"H103 MA6"}="3H10 MA6";
$resn_exch{" H2' MA6"}=" H2* MA6"; $resn_exch{"HO2' MA6"}="*HO2 MA6"; $resn_exch{" H3' MA6"}=" H3* MA6";
$resn_exch{"HO3' MA6"}="*HO3 MA6"; $resn_exch{"HOP2 MA7"}="2HOP MA7"; $resn_exch{" H61 MA7"}="1H6  MA7";
$resn_exch{" H62 MA7"}="2H6  MA7"; $resn_exch{" HN1 MA7"}="1HN  MA7"; $resn_exch{" HN2 MA7"}="2HN  MA7";
$resn_exch{" HN3 MA7"}="3HN  MA7"; $resn_exch{" H2' MA7"}="1H2* MA7"; $resn_exch{"H2'' MA7"}="2H2* MA7";
$resn_exch{" H5' MA7"}="1H5* MA7"; $resn_exch{"H5'' MA7"}="2H5* MA7"; $resn_exch{" H4' MA7"}=" H4* MA7";
$resn_exch{" H1' MA7"}=" H1* MA7"; $resn_exch{" H3' MA7"}=" H3* MA7"; $resn_exch{"HO3' MA7"}="H3T  MA7";
$resn_exch{"HOP3 MA7"}="3HOP MA7"; $resn_exch{"H4C1 MA8"}="1H4C MA8"; $resn_exch{"H4C2 MA8"}="2H4C MA8";
$resn_exch{"H2N1 MA8"}="1H2N MA8"; $resn_exch{"H2N2 MA8"}="2H2N MA8"; $resn_exch{"H151 MA8"}="1H15 MA8";
$resn_exch{"H152 MA8"}="2H15 MA8"; $resn_exch{"H151 MA9"}="1H15 MA9"; $resn_exch{"H152 MA9"}="2H15 MA9";
$resn_exch{"H281 MA9"}="1H28 MA9"; $resn_exch{"H282 MA9"}="2H28 MA9"; $resn_exch{"H141 MA9"}="1H14 MA9";
$resn_exch{"H142 MA9"}="2H14 MA9"; $resn_exch{"H271 MA9"}="1H27 MA9"; $resn_exch{"H272 MA9"}="2H27 MA9";
$resn_exch{" H1  MA9"}="1H   MA9"; $resn_exch{" H2  MA9"}="2H   MA9"; $resn_exch{" H3  MA9"}="3H   MA9";
$resn_exch{" H   MAA"}=" HN  MAA"; $resn_exch{" HM1 MAA"}="1HM  MAA"; $resn_exch{" HM2 MAA"}="2HM  MAA";
$resn_exch{" HM3 MAA"}="3HM  MAA"; $resn_exch{" HB1 MAA"}="1HB  MAA"; $resn_exch{" HB2 MAA"}="2HB  MAA";
$resn_exch{" HB3 MAA"}="3HB  MAA"; $resn_exch{"H6A2 MAB"}="2H6A MAB"; $resn_exch{"H6A1 MAB"}="1H6A MAB";
$resn_exch{"H2O1 MAB"}="1H2O MAB"; $resn_exch{"H3O1 MAB"}="1H3O MAB"; $resn_exch{"H6B2 MAB"}="2H6B MAB";
$resn_exch{"H6B1 MAB"}="1H6B MAB"; $resn_exch{"H2O2 MAB"}="2H2O MAB"; $resn_exch{"H3O2 MAB"}="2H3O MAB";
$resn_exch{"H4O1 MAB"}="1H4O MAB"; $resn_exch{"H1O1 MAB"}="1H1O MAB"; $resn_exch{" H21 MAC"}="1H2  MAC";
$resn_exch{" H22 MAC"}="2H2  MAC"; $resn_exch{" H23 MAC"}="3H2  MAC"; $resn_exch{" H5' MAD"}="1H5* MAD";
$resn_exch{"H5'' MAD"}="2H5* MAD"; $resn_exch{" H4' MAD"}=" H4* MAD"; $resn_exch{" H3' MAD"}=" H3* MAD";
$resn_exch{" H2' MAD"}=" H2* MAD"; $resn_exch{" H1' MAD"}=" H1* MAD"; $resn_exch{"HM11 MAD"}="1HM1 MAD";
$resn_exch{"HM12 MAD"}="2HM1 MAD"; $resn_exch{"HM13 MAD"}="3HM1 MAD"; $resn_exch{"H6C1 MAF"}="1H6C MAF";
$resn_exch{"H6C2 MAF"}="2H6C MAF"; $resn_exch{" H61 MAG"}="1H6  MAG"; $resn_exch{" H62 MAG"}="2H6  MAG";
$resn_exch{" H81 MAG"}="1H8  MAG"; $resn_exch{" H82 MAG"}="2H8  MAG"; $resn_exch{" H83 MAG"}="3H8  MAG";
$resn_exch{" HM1 MAG"}="1HM  MAG"; $resn_exch{" HM2 MAG"}="2HM  MAG"; $resn_exch{" HM3 MAG"}="3HM  MAG";
$resn_exch{" H21 MAH"}="1H2  MAH"; $resn_exch{" H22 MAH"}="2H2  MAH"; $resn_exch{" H41 MAH"}="1H4  MAH";
$resn_exch{" H42 MAH"}="2H4  MAH"; $resn_exch{" H61 MAH"}="1H6  MAH"; $resn_exch{" H62 MAH"}="2H6  MAH";
$resn_exch{" H63 MAH"}="3H6  MAH"; $resn_exch{"HN1A MAI"}="1HN  MAI"; $resn_exch{" HN2 MAI"}="2HN  MAI";
$resn_exch{" HJ1 MAI"}="1HJ  MAI"; $resn_exch{" HJ2 MAI"}="2HJ  MAI"; $resn_exch{" HJ3 MAI"}="3HJ  MAI";
$resn_exch{" HB1 MAI"}="1HB  MAI"; $resn_exch{" HB2 MAI"}="2HB  MAI"; $resn_exch{" HG1 MAI"}="1HG  MAI";
$resn_exch{" HG2 MAI"}="2HG  MAI"; $resn_exch{" HD1 MAI"}="1HD  MAI"; $resn_exch{" HD2 MAI"}="2HD  MAI";
$resn_exch{"HN21 MAI"}="1HN2 MAI"; $resn_exch{"HN22 MAI"}="2HN2 MAI"; $resn_exch{" H71 MAJ"}="1H7  MAJ";
$resn_exch{" H72 MAJ"}="2H7  MAJ"; $resn_exch{" H81 MAJ"}="1H8  MAJ"; $resn_exch{" H82 MAJ"}="2H8  MAJ";
$resn_exch{" H91 MAJ"}="1H9  MAJ"; $resn_exch{" H92 MAJ"}="2H9  MAJ"; $resn_exch{"HN11 MAJ"}="1HN1 MAJ";
$resn_exch{"HN12 MAJ"}="2HN1 MAJ"; $resn_exch{" H61 MAL"}="1H6  MAL"; $resn_exch{" H62 MAL"}="2H6  MAL";
$resn_exch{"H6'1 MAL"}="1H6' MAL"; $resn_exch{"H6'2 MAL"}="2H6' MAL"; $resn_exch{"HO1' MAL"}="'HO1 MAL";
$resn_exch{"HO2' MAL"}="'HO2 MAL"; $resn_exch{"HO3' MAL"}="'HO3 MAL"; $resn_exch{"HO6' MAL"}="'HO6 MAL";
$resn_exch{" H61 MAM"}="1H6  MAM"; $resn_exch{" H62 MAM"}="2H6  MAM"; $resn_exch{" H71 MAM"}="1H7  MAM";
$resn_exch{" H72 MAM"}="2H7  MAM"; $resn_exch{" H73 MAM"}="3H7  MAM"; $resn_exch{" H61 MAN"}="1H6  MAN";
$resn_exch{" H62 MAN"}="2H6  MAN"; $resn_exch{" HN1 MAO"}="1HN  MAO"; $resn_exch{" HN2 MAO"}="2HN  MAO";
$resn_exch{" HB1 MAO"}="1HB  MAO"; $resn_exch{" HB2 MAO"}="2HB  MAO"; $resn_exch{" HG1 MAO"}="1HG  MAO";
$resn_exch{" HG2 MAO"}="2HG  MAO"; $resn_exch{" HE1 MAO"}="1HE  MAO"; $resn_exch{" HE2 MAO"}="2HE  MAO";
$resn_exch{" HE3 MAO"}="3HE  MAO"; $resn_exch{"H5'1 MAO"}="1H5* MAO"; $resn_exch{"H5'2 MAO"}="2H5* MAO";
$resn_exch{" H4' MAO"}=" H4* MAO"; $resn_exch{" H1' MAO"}=" H1* MAO"; $resn_exch{"HN61 MAO"}="1HN6 MAO";
$resn_exch{"HN62 MAO"}="2HN6 MAO"; $resn_exch{" H2' MAO"}=" H2* MAO"; $resn_exch{"HO2' MAO"}="*HO2 MAO";
$resn_exch{" H3' MAO"}=" H3* MAO"; $resn_exch{"HO3' MAO"}="*HO3 MAO"; $resn_exch{"HOG2 MAP"}="2HOG MAP";
$resn_exch{"HOG3 MAP"}="3HOG MAP"; $resn_exch{"HOB2 MAP"}="2HOB MAP"; $resn_exch{"HNB1 MAP"}="1HNB MAP";
$resn_exch{"H5'1 MAP"}="1H5* MAP"; $resn_exch{"H5'2 MAP"}="2H5* MAP"; $resn_exch{" H4' MAP"}=" H4* MAP";
$resn_exch{" H3' MAP"}=" H3* MAP"; $resn_exch{"HO3' MAP"}="*HO3 MAP"; $resn_exch{" H2' MAP"}=" H2* MAP";
$resn_exch{"HO2' MAP"}="*HO2 MAP"; $resn_exch{" H1' MAP"}=" H1* MAP"; $resn_exch{"HN61 MAP"}="1HN6 MAP";
$resn_exch{"HN62 MAP"}="2HN6 MAP"; $resn_exch{" H91 MAQ"}="1H9  MAQ"; $resn_exch{" H92 MAQ"}="2H9  MAQ";
$resn_exch{" H93 MAQ"}="3H9  MAQ"; $resn_exch{" H21 MAQ"}="1H2  MAQ"; $resn_exch{" H22 MAQ"}="2H2  MAQ";
$resn_exch{"H111 MAR"}="1H11 MAR"; $resn_exch{"H112 MAR"}="2H11 MAR"; $resn_exch{"HO12 MAR"}="2HO1 MAR";
$resn_exch{"H141 MAR"}="1H14 MAR"; $resn_exch{"H142 MAR"}="2H14 MAR"; $resn_exch{"H143 MAR"}="3H14 MAR";
$resn_exch{"H151 MAR"}="1H15 MAR"; $resn_exch{"H152 MAR"}="2H15 MAR"; $resn_exch{"HO17 MAR"}="7HO1 MAR";
$resn_exch{"H211 MAR"}="1H21 MAR"; $resn_exch{"H212 MAR"}="2H21 MAR"; $resn_exch{"H213 MAR"}="3H21 MAR";
$resn_exch{" H1' MAR"}=" H1* MAR"; $resn_exch{"H2'1 MAR"}="1H2* MAR"; $resn_exch{"H2'2 MAR"}="2H2* MAR";
$resn_exch{" H3' MAR"}=" H3* MAR"; $resn_exch{"HN'1 MAR"}="1HN* MAR"; $resn_exch{"HN'2 MAR"}="2HN* MAR";
$resn_exch{" H4' MAR"}=" H4* MAR"; $resn_exch{" H5' MAR"}=" H5* MAR"; $resn_exch{"H6'1 MAR"}="1H6* MAR";
$resn_exch{"H6'2 MAR"}="2H6* MAR"; $resn_exch{"H6'3 MAR"}="3H6* MAR"; $resn_exch{"HB21 MAR"}="1HB2 MAR";
$resn_exch{"HB22 MAR"}="2HB2 MAR"; $resn_exch{"HOB3 MAR"}="3HOB MAR"; $resn_exch{"HOB4 MAR"}="4HOB MAR";
$resn_exch{"HB61 MAR"}="1HB6 MAR"; $resn_exch{"HB62 MAR"}="2HB6 MAR"; $resn_exch{"HB63 MAR"}="3HB6 MAR";
$resn_exch{" H11 MAS"}="1H1  MAS"; $resn_exch{" H12 MAS"}="2H1  MAS"; $resn_exch{" H13 MAS"}="3H1  MAS";
$resn_exch{" H21 MAS"}="1H2  MAS"; $resn_exch{" H22 MAS"}="2H2  MAS"; $resn_exch{" H23 MAS"}="3H2  MAS";
$resn_exch{" H21 MAT"}="1H2  MAT"; $resn_exch{" H22 MAT"}="2H2  MAT"; $resn_exch{" H51 MAT"}="1H5  MAT";
$resn_exch{" H52 MAT"}="2H5  MAT"; $resn_exch{" H71 MAT"}="1H7  MAT"; $resn_exch{" H72 MAT"}="2H7  MAT";
$resn_exch{" H73 MAT"}="3H7  MAT"; $resn_exch{" H81 MAT"}="1H8  MAT"; $resn_exch{" H82 MAT"}="2H8  MAT";
$resn_exch{" H83 MAT"}="3H8  MAT"; $resn_exch{" H91 MAT"}="1H9  MAT"; $resn_exch{" H92 MAT"}="2H9  MAT";
$resn_exch{" H93 MAT"}="3H9  MAT"; $resn_exch{" HC5 MAU"}="5HC  MAU"; $resn_exch{" HC6 MAU"}="6HC  MAU";
$resn_exch{"HC02 MAU"}="2HC0 MAU"; $resn_exch{"HC42 MAU"}="2HC4 MAU"; $resn_exch{"H251 MAU"}="1H25 MAU";
$resn_exch{"H252 MAU"}="2H25 MAU"; $resn_exch{" HO4 MAU"}="4HO  MAU"; $resn_exch{" HC9 MAU"}="9HC  MAU";
$resn_exch{"HC01 MAU"}="1HC0 MAU"; $resn_exch{"HC11 MAU"}="1HC1 MAU"; $resn_exch{"HC21 MAU"}="1HC2 MAU";
$resn_exch{"HC31 MAU"}="1HC3 MAU"; $resn_exch{"HC41 MAU"}="1HC4 MAU"; $resn_exch{"HC51 MAU"}="1HC5 MAU";
$resn_exch{"HC61 MAU"}="1HC6 MAU"; $resn_exch{"HC71 MAU"}="1HC7 MAU"; $resn_exch{"HC91 MAU"}="1HC9 MAU";
$resn_exch{"HC22 MAU"}="2HC2 MAU"; $resn_exch{"HC32 MAU"}="2HC3 MAU"; $resn_exch{"HO51 MAU"}="1HO5 MAU";
$resn_exch{"H401 MAU"}="1H40 MAU"; $resn_exch{"H402 MAU"}="2H40 MAU"; $resn_exch{"H403 MAU"}="3H40 MAU";
$resn_exch{"H411 MAU"}="1H41 MAU"; $resn_exch{"H412 MAU"}="2H41 MAU"; $resn_exch{"H413 MAU"}="3H41 MAU";
$resn_exch{"HO61 MAU"}="1HO6 MAU"; $resn_exch{"H421 MAU"}="1H42 MAU"; $resn_exch{"H422 MAU"}="2H42 MAU";
$resn_exch{"H423 MAU"}="3H42 MAU"; $resn_exch{"H431 MAU"}="1H43 MAU"; $resn_exch{"H432 MAU"}="2H43 MAU";
$resn_exch{"H433 MAU"}="3H43 MAU"; $resn_exch{"H441 MAU"}="1H44 MAU"; $resn_exch{"H442 MAU"}="2H44 MAU";
$resn_exch{"H443 MAU"}="3H44 MAU"; $resn_exch{"HN62 MAU"}="2HN6 MAU"; $resn_exch{"H451 MAU"}="1H45 MAU";
$resn_exch{"H452 MAU"}="2H45 MAU"; $resn_exch{"H461 MAU"}="1H46 MAU"; $resn_exch{"H462 MAU"}="2H46 MAU";
$resn_exch{"H463 MAU"}="3H46 MAU"; $resn_exch{"HC82 MAU"}="2HC8 MAU"; $resn_exch{"HO92 MAU"}="2HO9 MAU";
$resn_exch{" HO3 MAU"}="3HO  MAU"; $resn_exch{"HC13 MAU"}="3HC1 MAU"; $resn_exch{"HO13 MAU"}="3HO1 MAU";
$resn_exch{"H471 MAU"}="1H47 MAU"; $resn_exch{"H472 MAU"}="2H47 MAU"; $resn_exch{"H473 MAU"}="3H47 MAU";
$resn_exch{"H481 MAU"}="1H48 MAU"; $resn_exch{"H482 MAU"}="2H48 MAU"; $resn_exch{"H483 MAU"}="3H48 MAU";
$resn_exch{"HC33 MAU"}="3HC3 MAU"; $resn_exch{"HC53 MAU"}="3HC5 MAU"; $resn_exch{"HC63 MAU"}="3HC6 MAU";
$resn_exch{"HC73 MAU"}="3HC7 MAU"; $resn_exch{"HC83 MAU"}="3HC8 MAU"; $resn_exch{"H391 MAU"}="1H39 MAU";
$resn_exch{"H392 MAU"}="2H39 MAU"; $resn_exch{"H393 MAU"}="3H39 MAU"; $resn_exch{"H7C1 MAX"}="1H7C MAX";
$resn_exch{"H7C2 MAX"}="2H7C MAX"; $resn_exch{"H9C1 MAX"}="1H9C MAX"; $resn_exch{"H9C2 MAX"}="2H9C MAX";
$resn_exch{"H141 MAX"}="1H14 MAX"; $resn_exch{"H142 MAX"}="2H14 MAX"; $resn_exch{"H221 MAX"}="1H22 MAX";
$resn_exch{"H222 MAX"}="2H22 MAX"; $resn_exch{"H223 MAX"}="3H22 MAX"; $resn_exch{"H261 MAX"}="1H26 MAX";
$resn_exch{"H262 MAX"}="2H26 MAX"; $resn_exch{"H263 MAX"}="3H26 MAX"; $resn_exch{"HCM1 MAY"}="1HCM MAY";
$resn_exch{"HCM2 MAY"}="2HCM MAY"; $resn_exch{"HCM3 MAY"}="3HCM MAY"; $resn_exch{" H11 MAY"}="1H1  MAY";
$resn_exch{" H12 MAY"}="2H1  MAY"; $resn_exch{" H21 MAY"}="1H2  MAY"; $resn_exch{" H22 MAY"}="2H2  MAY";
$resn_exch{" H31 MAY"}="1H3  MAY"; $resn_exch{" H32 MAY"}="2H3  MAY"; $resn_exch{" H41 MAY"}="1H4  MAY";
$resn_exch{" H42 MAY"}="2H4  MAY"; $resn_exch{" H51 MAY"}="1H5  MAY"; $resn_exch{" H61 MAY"}="1H6  MAY";
$resn_exch{" H71 MAY"}="1H7  MAY"; $resn_exch{" H72 MAY"}="2H7  MAY"; $resn_exch{" H81 MAY"}="1H8  MAY";
$resn_exch{" H91 MAY"}="1H9  MAY"; $resn_exch{"H101 MAY"}="1H10 MAY"; $resn_exch{"H102 MAY"}="2H10 MAY";
$resn_exch{"H111 MAY"}="1H11 MAY"; $resn_exch{"H121 MAY"}="1H12 MAY"; $resn_exch{"H131 MAY"}="1H13 MAY";
$resn_exch{"H132 MAY"}="2H13 MAY"; $resn_exch{"H141 MAY"}="1H14 MAY"; $resn_exch{"H151 MAY"}="1H15 MAY";
$resn_exch{"H161 MAY"}="1H16 MAY"; $resn_exch{"H162 MAY"}="2H16 MAY"; $resn_exch{"H171 MAY"}="1H17 MAY";
$resn_exch{"H172 MAY"}="2H17 MAY"; $resn_exch{"H181 MAY"}="1H18 MAY"; $resn_exch{"H182 MAY"}="2H18 MAY";
$resn_exch{"H191 MAY"}="1H19 MAY"; $resn_exch{"H192 MAY"}="2H19 MAY"; $resn_exch{"H201 MAY"}="1H20 MAY";
$resn_exch{"H202 MAY"}="2H20 MAY"; $resn_exch{"H203 MAY"}="3H20 MAY"; $resn_exch{"H121 MAZ"}="1H12 MAZ";
$resn_exch{"H122 MAZ"}="2H12 MAZ"; $resn_exch{"H111 MAZ"}="1H11 MAZ"; $resn_exch{"H112 MAZ"}="2H11 MAZ";
$resn_exch{" HN1 MBA"}="1HN  MBA"; $resn_exch{" HN2 MBA"}="2HN  MBA"; $resn_exch{" H11 MBA"}="1H1  MBA";
$resn_exch{" H12 MBA"}="2H1  MBA"; $resn_exch{"H2'1 MBA"}="1H2' MBA"; $resn_exch{"H2'2 MBA"}="2H2' MBA";
$resn_exch{"H2'3 MBA"}="3H2' MBA"; $resn_exch{" H31 MBA"}="1H3  MBA"; $resn_exch{" H32 MBA"}="2H3  MBA";
$resn_exch{" H41 MBA"}="1H4  MBA"; $resn_exch{" H42 MBA"}="2H4  MBA"; $resn_exch{" H43 MBA"}="3H4  MBA";
$resn_exch{"H201 MBB"}="1H20 MBB"; $resn_exch{"H202 MBB"}="2H20 MBB"; $resn_exch{"H211 MBB"}="1H21 MBB";
$resn_exch{"H212 MBB"}="2H21 MBB"; $resn_exch{"H221 MBB"}="1H22 MBB"; $resn_exch{"H222 MBB"}="2H22 MBB";
$resn_exch{"H231 MBB"}="1H23 MBB"; $resn_exch{"H232 MBB"}="2H23 MBB"; $resn_exch{"H241 MBB"}="1H24 MBB";
$resn_exch{"H242 MBB"}="2H24 MBB"; $resn_exch{"H243 MBB"}="3H24 MBB"; $resn_exch{"HN11 MBB"}="1HN1 MBB";
$resn_exch{"HN12 MBB"}="2HN1 MBB"; $resn_exch{"HN13 MBB"}="3HN1 MBB"; $resn_exch{"HN21 MBB"}="1HN2 MBB";
$resn_exch{"HN22 MBB"}="2HN2 MBB"; $resn_exch{"HN23 MBB"}="3HN2 MBB"; $resn_exch{"H251 MBB"}="1H25 MBB";
$resn_exch{"H252 MBB"}="2H25 MBB"; $resn_exch{"H253 MBB"}="3H25 MBB"; $resn_exch{"H241 MBC"}="1H24 MBC";
$resn_exch{"H242 MBC"}="2H24 MBC"; $resn_exch{"H261 MBC"}="1H26 MBC"; $resn_exch{"H262 MBC"}="2H26 MBC";
$resn_exch{"HB31 MBD"}="1HB3 MBD"; $resn_exch{"HB32 MBD"}="2HB3 MBD"; $resn_exch{"HB33 MBD"}="3HB3 MBD";
$resn_exch{"H6C1 MBF"}="1H6C MBF"; $resn_exch{"H6C2 MBF"}="2H6C MBF"; $resn_exch{" H61 MBG"}="1H6  MBG";
$resn_exch{" H62 MBG"}="2H6  MBG"; $resn_exch{" H71 MBG"}="1H7  MBG"; $resn_exch{" H72 MBG"}="2H7  MBG";
$resn_exch{" H73 MBG"}="3H7  MBG"; $resn_exch{"HN21 MBH"}="1HN2 MBH"; $resn_exch{"HN22 MBH"}="2HN2 MBH";
$resn_exch{" H1  MBH"}="1H   MBH"; $resn_exch{" H2A MBH"}="2H   MBH"; $resn_exch{"H1M1 MBH"}="1H1M MBH";
$resn_exch{"H1M2 MBH"}="2H1M MBH"; $resn_exch{"H1M3 MBH"}="3H1M MBH"; $resn_exch{"HA21 MBH"}="1HA2 MBH";
$resn_exch{"HA22 MBH"}="2HA2 MBH"; $resn_exch{" H1  MBN"}="1H   MBN"; $resn_exch{" H2A MBN"}="2H   MBN";
$resn_exch{" H3A MBN"}="3H   MBN"; $resn_exch{"H3'1 MBP"}="1H3* MBP"; $resn_exch{"H3'2 MBP"}="2H3* MBP";
$resn_exch{"H3'3 MBP"}="3H3* MBP"; $resn_exch{" H2' MBP"}=" H2* MBP"; $resn_exch{"H121 MBP"}="1H12 MBP";
$resn_exch{"H122 MBP"}="2H12 MBP"; $resn_exch{"H123 MBP"}="3H12 MBP"; $resn_exch{"H2N1 MBP"}="1H2N MBP";
$resn_exch{"H2N2 MBP"}="2H2N MBP"; $resn_exch{"H601 MBP"}="1H60 MBP"; $resn_exch{"H602 MBP"}="2H60 MBP";
$resn_exch{" H2  MBQ"}=" HN  MBQ"; $resn_exch{" HB2 MBQ"}=" HB1 MBQ"; $resn_exch{" HB3 MBQ"}=" HB2 MBQ";
$resn_exch{"HAE1 MBQ"}="1HAE MBQ"; $resn_exch{"HAE2 MBQ"}="2HAE MBQ"; $resn_exch{"HAK3 MBQ"}="3HAK MBQ";
$resn_exch{"HAK2 MBQ"}="2HAK MBQ"; $resn_exch{"HAK1 MBQ"}="1HAK MBQ"; $resn_exch{" H51 MBS"}="1H5  MBS";
$resn_exch{" H52 MBS"}="2H5  MBS"; $resn_exch{" H53 MBS"}="3H5  MBS"; $resn_exch{"H161 MBS"}="1H16 MBS";
$resn_exch{"H162 MBS"}="2H16 MBS"; $resn_exch{" H31 MBS"}="1H3  MBS"; $resn_exch{" H32 MBS"}="2H3  MBS";
$resn_exch{" H21 MBS"}="1H2  MBS"; $resn_exch{"HO11 MBS"}="1HO1 MBS"; $resn_exch{"H221 MBS"}="1H22 MBS";
$resn_exch{"H231 MBS"}="1H23 MBS"; $resn_exch{"H251 MBS"}="1H25 MBS"; $resn_exch{"H261 MBS"}="1H26 MBS";
$resn_exch{"H321 MBS"}="1H32 MBS"; $resn_exch{"H331 MBS"}="1H33 MBS"; $resn_exch{"H351 MBS"}="1H35 MBS";
$resn_exch{"H361 MBS"}="1H36 MBS"; $resn_exch{" H71 MBS"}="1H7  MBS"; $resn_exch{" H72 MBS"}="2H7  MBS";
$resn_exch{" H73 MBS"}="3H7  MBS"; $resn_exch{"H191 MBT"}="1H19 MBT"; $resn_exch{"H192 MBT"}="2H19 MBT";
$resn_exch{"H193 MBT"}="3H19 MBT"; $resn_exch{"H201 MBT"}="1H20 MBT"; $resn_exch{"H202 MBT"}="2H20 MBT";
$resn_exch{"H203 MBT"}="3H20 MBT"; $resn_exch{"H161 MBT"}="1H16 MBT"; $resn_exch{"H162 MBT"}="2H16 MBT";
$resn_exch{"H163 MBT"}="3H16 MBT"; $resn_exch{"H171 MBT"}="1H17 MBT"; $resn_exch{"H172 MBT"}="2H17 MBT";
$resn_exch{"H173 MBT"}="3H17 MBT"; $resn_exch{"H1A1 MBV"}="1H1A MBV"; $resn_exch{"H1A2 MBV"}="2H1A MBV";
$resn_exch{"H1A3 MBV"}="3H1A MBV"; $resn_exch{"H2A1 MBV"}="1H2A MBV"; $resn_exch{"H2A2 MBV"}="2H2A MBV";
$resn_exch{"H1B1 MBV"}="1H1B MBV"; $resn_exch{"H1B2 MBV"}="2H1B MBV"; $resn_exch{"H6B1 MBV"}="1H6B MBV";
$resn_exch{"H6B2 MBV"}="2H6B MBV"; $resn_exch{"H6B3 MBV"}="3H6B MBV"; $resn_exch{"H6C1 MBV"}="1H6C MBV";
$resn_exch{"H6C2 MBV"}="2H6C MBV"; $resn_exch{"H6C3 MBV"}="3H6C MBV"; $resn_exch{"H1C1 MBV"}="1H1C MBV";
$resn_exch{"H1C2 MBV"}="2H1C MBV"; $resn_exch{"H2D1 MBV"}="1H2D MBV"; $resn_exch{"H2D2 MBV"}="2H2D MBV";
$resn_exch{"H1D1 MBV"}="1H1D MBV"; $resn_exch{"H1D2 MBV"}="2H1D MBV"; $resn_exch{"H1D3 MBV"}="3H1D MBV";
$resn_exch{"H3A1 MBV"}="1H3A MBV"; $resn_exch{"H3A2 MBV"}="2H3A MBV"; $resn_exch{"H3A3 MBV"}="3H3A MBV";
$resn_exch{"H2B1 MBV"}="1H2B MBV"; $resn_exch{"H2B2 MBV"}="2H2B MBV"; $resn_exch{"H2C1 MBV"}="1H2C MBV";
$resn_exch{"H2C2 MBV"}="2H2C MBV"; $resn_exch{"H3D1 MBV"}="1H3D MBV"; $resn_exch{"H3D2 MBV"}="2H3D MBV";
$resn_exch{"H3D3 MBV"}="3H3D MBV"; $resn_exch{"HOP2 MBZ"}="2HOP MBZ"; $resn_exch{"HOP3 MBZ"}="3HOP MBZ";
$resn_exch{"HO'3 MBZ"}="3HO* MBZ"; $resn_exch{" H1' MBZ"}=" H1* MBZ"; $resn_exch{" H2' MBZ"}="1H2* MBZ";
$resn_exch{"H2'' MBZ"}="2H2* MBZ"; $resn_exch{" H3' MBZ"}=" H3* MBZ"; $resn_exch{" H4' MBZ"}=" H4* MBZ";
$resn_exch{" H5' MBZ"}="1H5* MBZ"; $resn_exch{"H5'' MBZ"}="2H5* MBZ"; $resn_exch{" H71 MBZ"}="1H5M MBZ";
$resn_exch{" H72 MBZ"}="2H5M MBZ"; $resn_exch{" H73 MBZ"}="3H5M MBZ"; $resn_exch{"H111  MC"}="1H11  MC";
$resn_exch{"H112  MC"}="2H11  MC"; $resn_exch{"H113  MC"}="3H11  MC"; $resn_exch{" H71  MC"}="1H7   MC";
$resn_exch{" H72  MC"}="2H7   MC"; $resn_exch{"H101  MC"}="1H10  MC"; $resn_exch{"H102  MC"}="2H10  MC";
$resn_exch{"H141  MC"}="1H14  MC"; $resn_exch{"H142  MC"}="2H14  MC"; $resn_exch{" H31  MC"}="1H3   MC";
$resn_exch{" H32  MC"}="2H3   MC"; $resn_exch{" H11  MC"}="1H1   MC"; $resn_exch{" H12  MC"}="2H1   MC";
$resn_exch{"H241 MC1"}="H1   MC1"; $resn_exch{"H242 MC1"}="H2   MC1"; $resn_exch{"H243 MC1"}="H3   MC1";
$resn_exch{" H   MC1"}=" H25 MC1"; $resn_exch{" H2  MC1"}=" H26 MC1"; $resn_exch{" HB2 MC1"}=" H22 MC1";
$resn_exch{" HB3 MC1"}=" H23 MC1"; $resn_exch{"H121 MC2"}="1H12 MC2"; $resn_exch{"H122 MC2"}="2H12 MC2";
$resn_exch{"H131 MC2"}="1H13 MC2"; $resn_exch{"H132 MC2"}="2H13 MC2"; $resn_exch{"H141 MC2"}="1H14 MC2";
$resn_exch{"H142 MC2"}="2H14 MC2"; $resn_exch{"HN41 MC2"}="1HN4 MC2"; $resn_exch{"HN42 MC2"}="2HN4 MC2";
$resn_exch{"HN11 MC2"}="1HN1 MC2"; $resn_exch{"HN12 MC2"}="2HN1 MC2"; $resn_exch{" H31 MC2"}="1H3  MC2";
$resn_exch{" H32 MC2"}="2H3  MC2"; $resn_exch{" H11 MC3"}="1H1  MC3"; $resn_exch{" H12 MC3"}="2H1  MC3";
$resn_exch{" H31 MC3"}="1H3  MC3"; $resn_exch{" H32 MC3"}="2H3  MC3"; $resn_exch{" H41 MC3"}="1H4  MC3";
$resn_exch{" H42 MC3"}="2H4  MC3"; $resn_exch{" H51 MC3"}="1H5  MC3"; $resn_exch{" H52 MC3"}="2H5  MC3";
$resn_exch{" H61 MC3"}="1H6  MC3"; $resn_exch{" H62 MC3"}="2H6  MC3"; $resn_exch{" H63 MC3"}="3H6  MC3";
$resn_exch{" H71 MC3"}="1H7  MC3"; $resn_exch{" H72 MC3"}="2H7  MC3"; $resn_exch{" H73 MC3"}="3H7  MC3";
$resn_exch{" H81 MC3"}="1H8  MC3"; $resn_exch{" H82 MC3"}="2H8  MC3"; $resn_exch{" H83 MC3"}="3H8  MC3";
$resn_exch{"H121 MC3"}="1H12 MC3"; $resn_exch{"H122 MC3"}="2H12 MC3"; $resn_exch{"H131 MC3"}="1H13 MC3";
$resn_exch{"H132 MC3"}="2H13 MC3"; $resn_exch{"H141 MC3"}="1H14 MC3"; $resn_exch{"H142 MC3"}="2H14 MC3";
$resn_exch{"H151 MC3"}="1H15 MC3"; $resn_exch{"H152 MC3"}="2H15 MC3"; $resn_exch{"H161 MC3"}="1H16 MC3";
$resn_exch{"H162 MC3"}="2H16 MC3"; $resn_exch{"H171 MC3"}="1H17 MC3"; $resn_exch{"H172 MC3"}="2H17 MC3";
$resn_exch{"H181 MC3"}="1H18 MC3"; $resn_exch{"H182 MC3"}="2H18 MC3"; $resn_exch{"H191 MC3"}="1H19 MC3";
$resn_exch{"H192 MC3"}="2H19 MC3"; $resn_exch{"H201 MC3"}="1H20 MC3"; $resn_exch{"H202 MC3"}="2H20 MC3";
$resn_exch{"H211 MC3"}="1H21 MC3"; $resn_exch{"H212 MC3"}="2H21 MC3"; $resn_exch{"H221 MC3"}="1H22 MC3";
$resn_exch{"H222 MC3"}="2H22 MC3"; $resn_exch{"H231 MC3"}="1H23 MC3"; $resn_exch{"H232 MC3"}="2H23 MC3";
$resn_exch{"H241 MC3"}="1H24 MC3"; $resn_exch{"H242 MC3"}="2H24 MC3"; $resn_exch{"H243 MC3"}="3H24 MC3";
$resn_exch{"H321 MC3"}="1H32 MC3"; $resn_exch{"H322 MC3"}="2H32 MC3"; $resn_exch{"H331 MC3"}="1H33 MC3";
$resn_exch{"H332 MC3"}="2H33 MC3"; $resn_exch{"H341 MC3"}="1H34 MC3"; $resn_exch{"H342 MC3"}="2H34 MC3";
$resn_exch{"H351 MC3"}="1H35 MC3"; $resn_exch{"H352 MC3"}="2H35 MC3"; $resn_exch{"H361 MC3"}="1H36 MC3";
$resn_exch{"H362 MC3"}="2H36 MC3"; $resn_exch{"H371 MC3"}="1H37 MC3"; $resn_exch{"H372 MC3"}="2H37 MC3";
$resn_exch{"H381 MC3"}="1H38 MC3"; $resn_exch{"H382 MC3"}="2H38 MC3"; $resn_exch{"H391 MC3"}="1H39 MC3";
$resn_exch{"H392 MC3"}="2H39 MC3"; $resn_exch{"H401 MC3"}="1H40 MC3"; $resn_exch{"H402 MC3"}="2H40 MC3";
$resn_exch{"H411 MC3"}="1H41 MC3"; $resn_exch{"H412 MC3"}="2H41 MC3"; $resn_exch{"H421 MC3"}="1H42 MC3";
$resn_exch{"H422 MC3"}="2H42 MC3"; $resn_exch{"H431 MC3"}="1H43 MC3"; $resn_exch{"H432 MC3"}="2H43 MC3";
$resn_exch{"H441 MC3"}="1H44 MC3"; $resn_exch{"H442 MC3"}="2H44 MC3"; $resn_exch{"H443 MC3"}="3H44 MC3";
$resn_exch{" H21 MC9"}="1H2  MC9"; $resn_exch{"H22A MC9"}="2H2  MC9"; $resn_exch{" H41 MC9"}="1H4  MC9";
$resn_exch{" H42 MC9"}="2H4  MC9"; $resn_exch{" H91 MC9"}="1H9  MC9"; $resn_exch{" H92 MC9"}="2H9  MC9";
$resn_exch{"H111 MC9"}="1H11 MC9"; $resn_exch{"H112 MC9"}="2H11 MC9"; $resn_exch{"H121 MC9"}="1H12 MC9";
$resn_exch{"H122 MC9"}="2H12 MC9"; $resn_exch{"H151 MC9"}="1H15 MC9"; $resn_exch{"H152 MC9"}="2H15 MC9";
$resn_exch{"H161 MC9"}="1H16 MC9"; $resn_exch{"H162 MC9"}="2H16 MC9"; $resn_exch{"H181 MC9"}="1H18 MC9";
$resn_exch{"H182 MC9"}="2H18 MC9"; $resn_exch{"H183 MC9"}="3H18 MC9"; $resn_exch{"H191 MC9"}="1H19 MC9";
$resn_exch{"H192 MC9"}="2H19 MC9"; $resn_exch{"H211 MC9"}="1H21 MC9"; $resn_exch{"H212 MC9"}="2H21 MC9";
$resn_exch{"H213 MC9"}="3H21 MC9"; $resn_exch{"H261 MC9"}="1H26 MC9"; $resn_exch{"H262 MC9"}="2H26 MC9";
$resn_exch{"H271 MC9"}="1H27 MC9"; $resn_exch{"H272 MC9"}="2H27 MC9"; $resn_exch{"HN61 MCA"}="1HN6 MCA";
$resn_exch{"HN62 MCA"}="2HN6 MCA"; $resn_exch{"HO2' MCA"}="'HO2 MCA"; $resn_exch{"H5'1 MCA"}="1H5' MCA";
$resn_exch{"H5'2 MCA"}="2H5' MCA"; $resn_exch{"HPB1 MCA"}="1HPB MCA"; $resn_exch{"HPB2 MCA"}="2HPB MCA";
$resn_exch{" H91 MCA"}="1H9  MCA"; $resn_exch{" H92 MCA"}="2H9  MCA"; $resn_exch{" H93 MCA"}="3H9  MCA";
$resn_exch{" H81 MCA"}="1H8  MCA"; $resn_exch{" H82 MCA"}="2H8  MCA"; $resn_exch{" H83 MCA"}="3H8  MCA";
$resn_exch{" H51 MCA"}="1H5  MCA"; $resn_exch{" H52 MCA"}="2H5  MCA"; $resn_exch{" H41 MCA"}="1H4  MCA";
$resn_exch{" H42 MCA"}="2H4  MCA"; $resn_exch{" H21 MCA"}="1H2  MCA"; $resn_exch{" H22 MCA"}="2H2  MCA";
$resn_exch{" H11 MCA"}="1H1  MCA"; $resn_exch{" H12 MCA"}="2H1  MCA"; $resn_exch{" H31 MCA"}="1H3  MCA";
$resn_exch{" H32 MCA"}="2H3  MCA"; $resn_exch{" H33 MCA"}="3H3  MCA"; $resn_exch{"HA1A MCB"}="1HA  MCB";
$resn_exch{"HN61 MCD"}="1HN6 MCD"; $resn_exch{"HN62 MCD"}="2HN6 MCD"; $resn_exch{"HO2' MCD"}="'HO2 MCD";
$resn_exch{"H5'1 MCD"}="1H5' MCD"; $resn_exch{"H5'2 MCD"}="2H5' MCD"; $resn_exch{"HPB1 MCD"}="1HPB MCD";
$resn_exch{"HPB2 MCD"}="2HPB MCD"; $resn_exch{"HP91 MCD"}="1HP9 MCD"; $resn_exch{"HP92 MCD"}="2HP9 MCD";
$resn_exch{"HP93 MCD"}="3HP9 MCD"; $resn_exch{"HP81 MCD"}="1HP8 MCD"; $resn_exch{"HP82 MCD"}="2HP8 MCD";
$resn_exch{"HP83 MCD"}="3HP8 MCD"; $resn_exch{"HOP3 MCD"}="3HOP MCD"; $resn_exch{"HNP2 MCD"}="2HNP MCD";
$resn_exch{"HP51 MCD"}="1HP5 MCD"; $resn_exch{"HP52 MCD"}="2HP5 MCD"; $resn_exch{"HP41 MCD"}="1HP4 MCD";
$resn_exch{"HP42 MCD"}="2HP4 MCD"; $resn_exch{"HNP1 MCD"}="1HNP MCD"; $resn_exch{"HP21 MCD"}="1HP2 MCD";
$resn_exch{"HP22 MCD"}="2HP2 MCD"; $resn_exch{"HP11 MCD"}="1HP1 MCD"; $resn_exch{"HP12 MCD"}="2HP1 MCD";
$resn_exch{"HPS1 MCD"}="1HPS MCD"; $resn_exch{"HPS2 MCD"}="2HPS MCD"; $resn_exch{"HS31 MCD"}="1HS3 MCD";
$resn_exch{"HS32 MCD"}="2HS3 MCD"; $resn_exch{"HS33 MCD"}="3HS3 MCD"; $resn_exch{"HOS5 MCD"}="5HOS MCD";
$resn_exch{" H11 MCE"}="1H1  MCE"; $resn_exch{" H12 MCE"}="2H1  MCE"; $resn_exch{" H13 MCE"}="3H1  MCE";
$resn_exch{" H21 MCE"}="1H2  MCE"; $resn_exch{" H22 MCE"}="2H2  MCE"; $resn_exch{" H   MCG"}="1HN  MCG";
$resn_exch{" H2  MCG"}="2HN  MCG"; $resn_exch{" HH2 MCG"}=" HO2 MCG"; $resn_exch{" HM1 MCG"}="1HM  MCG";
$resn_exch{" HM2 MCG"}="2HM  MCG"; $resn_exch{" HM3 MCG"}="3HM  MCG"; $resn_exch{" H11 MCI"}="1H1  MCI";
$resn_exch{" H31 MCI"}="1H3  MCI"; $resn_exch{" H41 MCI"}="1H4  MCI"; $resn_exch{" H51 MCI"}="1H5  MCI";
$resn_exch{" H61 MCI"}="1H6  MCI"; $resn_exch{"H121 MCI"}="1H12 MCI"; $resn_exch{"H122 MCI"}="2H12 MCI";
$resn_exch{"H131 MCI"}="1H13 MCI"; $resn_exch{"H132 MCI"}="2H13 MCI"; $resn_exch{"H161 MCI"}="1H16 MCI";
$resn_exch{"H191 MCI"}="1H19 MCI"; $resn_exch{"H192 MCI"}="2H19 MCI"; $resn_exch{"H331 MCI"}="1H33 MCI";
$resn_exch{"H341 MCI"}="1H34 MCI"; $resn_exch{"H371 MCI"}="1H37 MCI"; $resn_exch{"H372 MCI"}="2H37 MCI";
$resn_exch{"H411 MCI"}="1H41 MCI"; $resn_exch{" H   MCL"}="1HN  MCL"; $resn_exch{" H2  MCL"}="2HN  MCL";
$resn_exch{" HB2 MCL"}="1HB  MCL"; $resn_exch{" HB3 MCL"}="2HB  MCL"; $resn_exch{" HG2 MCL"}="1HG  MCL";
$resn_exch{" HG3 MCL"}="2HG  MCL"; $resn_exch{" HD2 MCL"}="1HD  MCL"; $resn_exch{" HD3 MCL"}="2HD  MCL";
$resn_exch{" HE2 MCL"}="1HE  MCL"; $resn_exch{" HE3 MCL"}="2HE  MCL"; $resn_exch{" HZ  MCL"}=" HNZ MCL";
$resn_exch{" H11 MCL"}="1H1  MCL"; $resn_exch{" H12 MCL"}="2H1  MCL"; $resn_exch{" H13 MCL"}="3H1  MCL";
$resn_exch{" HN1 MCM"}="1HN  MCM"; $resn_exch{" HN2 MCM"}="2HN  MCM"; $resn_exch{"H101 MCM"}="1H10 MCM";
$resn_exch{"H102 MCM"}="2H10 MCM"; $resn_exch{"H103 MCM"}="3H10 MCM"; $resn_exch{"HN41 MCN"}="1HN4 MCN";
$resn_exch{"HN42 MCN"}="2HN4 MCN"; $resn_exch{" H1' MCN"}=" H1* MCN"; $resn_exch{" H2' MCN"}=" H2* MCN";
$resn_exch{"HO2' MCN"}="*HO2 MCN"; $resn_exch{" H3' MCN"}=" H3* MCN"; $resn_exch{" H4' MCN"}=" H4* MCN";
$resn_exch{"HO3' MCN"}="*HO3 MCN"; $resn_exch{"H5'1 MCN"}="1H5* MCN"; $resn_exch{"H5'2 MCN"}="2H5* MCN";
$resn_exch{"HOA2 MCN"}="2HOA MCN"; $resn_exch{"HOB2 MCN"}="2HOB MCN"; $resn_exch{"H101 MCN"}="1H10 MCN";
$resn_exch{"H102 MCN"}="2H10 MCN"; $resn_exch{"HN21 MCN"}="1HN2 MCN"; $resn_exch{"HN22 MCN"}="2HN2 MCN";
$resn_exch{"HO31 MCO"}="1HO3 MCO"; $resn_exch{"HC81 MCO"}="1HC8 MCO"; $resn_exch{"HC71 MCO"}="1HC7 MCO";
$resn_exch{"HC72 MCO"}="2HC7 MCO"; $resn_exch{"HC61 MCO"}="1HC6 MCO"; $resn_exch{"HC62 MCO"}="2HC6 MCO";
$resn_exch{"HC51 MCO"}="1HC5 MCO"; $resn_exch{"HC52 MCO"}="2HC5 MCO"; $resn_exch{"HC21 MCO"}="1HC2 MCO";
$resn_exch{"HC31 MCO"}="1HC3 MCO"; $resn_exch{"HC32 MCO"}="2HC3 MCO"; $resn_exch{"HC33 MCO"}="3HC3 MCO";
$resn_exch{"HC11 MCO"}="1HC1 MCO"; $resn_exch{"HC12 MCO"}="2HC1 MCO"; $resn_exch{" HS1 MCO"}="1HS  MCO";
$resn_exch{" H21 MCP"}="1H2  MCP"; $resn_exch{" H22 MCP"}="2H2  MCP"; $resn_exch{" H41 MCP"}="1H4  MCP";
$resn_exch{" H42 MCP"}="2H4  MCP"; $resn_exch{" H51 MCP"}="1H5  MCP"; $resn_exch{" H52 MCP"}="2H5  MCP";
$resn_exch{" H71 MCP"}="1H7  MCP"; $resn_exch{" H72 MCP"}="2H7  MCP"; $resn_exch{" H73 MCP"}="3H7  MCP";
$resn_exch{" H21 MCR"}="1H2  MCR"; $resn_exch{" H22 MCR"}="2H2  MCR"; $resn_exch{" HN1 MCS"}="1HN  MCS";
$resn_exch{" HN2 MCS"}="2HN  MCS"; $resn_exch{"HBC1 MCS"}="1HBC MCS"; $resn_exch{"HBC2 MCS"}="2HBC MCS";
$resn_exch{"HAG1 MCS"}="1HAG MCS"; $resn_exch{"HAG2 MCS"}="2HAG MCS"; $resn_exch{" H1  MCT"}="1H   MCT";
$resn_exch{" H2A MCT"}="2H   MCT"; $resn_exch{" H3  MCT"}="3H   MCT"; $resn_exch{"HN41 MCY"}="1HN4 MCY";
$resn_exch{"HN42 MCY"}="2HN4 MCY"; $resn_exch{" H1' MCY"}=" H1* MCY"; $resn_exch{" H2' MCY"}="1H2* MCY";
$resn_exch{"H2'' MCY"}="2H2* MCY"; $resn_exch{" H3' MCY"}=" H3* MCY"; $resn_exch{" H4' MCY"}=" H4* MCY";
$resn_exch{"HO3' MCY"}="*HO3 MCY"; $resn_exch{" H5' MCY"}="1H5* MCY"; $resn_exch{"H5'' MCY"}="2H5* MCY";
$resn_exch{"HO5' MCY"}="*HO5 MCY"; $resn_exch{"H5A1 MCY"}="1H5A MCY"; $resn_exch{"H5A2 MCY"}="2H5A MCY";
$resn_exch{"H5A3 MCY"}="3H5A MCY"; $resn_exch{"H101 MD1"}="1H10 MD1"; $resn_exch{"H102 MD1"}="2H10 MD1";
$resn_exch{"H161 MD1"}="1H16 MD1"; $resn_exch{"H162 MD1"}="2H16 MD1"; $resn_exch{"H5'1 MD1"}="1H5* MD1";
$resn_exch{"H5'2 MD1"}="2H5* MD1"; $resn_exch{" H4' MD1"}=" H4* MD1"; $resn_exch{" H1' MD1"}=" H1* MD1";
$resn_exch{"HN21 MD1"}="1HN2 MD1"; $resn_exch{"HN22 MD1"}="2HN2 MD1"; $resn_exch{" H2' MD1"}=" H2* MD1";
$resn_exch{" H3' MD1"}=" H3* MD1"; $resn_exch{"HC11 MD2"}="1HC1 MD2"; $resn_exch{"HC1A MD2"}="2HC1 MD2";
$resn_exch{"HC61 MD2"}="1HC6 MD2"; $resn_exch{"HC62 MD2"}="2HC6 MD2"; $resn_exch{"HC71 MD2"}="1HC7 MD2";
$resn_exch{"HC72 MD2"}="2HC7 MD2"; $resn_exch{"H111 MD2"}="1H11 MD2"; $resn_exch{"H112 MD2"}="2H11 MD2";
$resn_exch{"HC41 MD2"}="1HC4 MD2"; $resn_exch{"HC42 MD2"}="2HC4 MD2"; $resn_exch{"HC81 MD2"}="1HC8 MD2";
$resn_exch{"HC82 MD2"}="2HC8 MD2"; $resn_exch{"HC91 MD2"}="1HC9 MD2"; $resn_exch{"HC92 MD2"}="2HC9 MD2";
$resn_exch{"H141 MD2"}="1H14 MD2"; $resn_exch{"H142 MD2"}="2H14 MD2"; $resn_exch{" H21 MDA"}="1H2  MDA";
$resn_exch{" H22 MDA"}="2H2  MDA"; $resn_exch{" H61 MDA"}="1H6  MDA"; $resn_exch{" H62 MDA"}="2H6  MDA";
$resn_exch{" H63 MDA"}="3H6  MDA"; $resn_exch{"H3'1 MDA"}="1H3' MDA"; $resn_exch{"H3'2 MDA"}="2H3' MDA";
$resn_exch{"H3'3 MDA"}="3H3' MDA"; $resn_exch{" H11 MDC"}="1H1  MDC"; $resn_exch{" H12 MDC"}="2H1  MDC";
$resn_exch{" H21 MDC"}="1H2  MDC"; $resn_exch{" H22 MDC"}="2H2  MDC"; $resn_exch{"H191 MDC"}="1H19 MDC";
$resn_exch{"H192 MDC"}="2H19 MDC"; $resn_exch{"H201 MDC"}="1H20 MDC"; $resn_exch{"H202 MDC"}="2H20 MDC";
$resn_exch{"H203 MDC"}="3H20 MDC"; $resn_exch{"H211 MDC"}="1H21 MDC"; $resn_exch{"H212 MDC"}="2H21 MDC";
$resn_exch{"H221 MDC"}="1H22 MDC"; $resn_exch{"H222 MDC"}="2H22 MDC"; $resn_exch{"H223 MDC"}="3H22 MDC";
$resn_exch{"HA61 MDD"}="1HA6 MDD"; $resn_exch{"HA62 MDD"}="2HA6 MDD"; $resn_exch{"H5'1 MDE"}="1H5* MDE";
$resn_exch{"H5'2 MDE"}="2H5* MDE"; $resn_exch{" H4' MDE"}=" H4* MDE"; $resn_exch{" H1' MDE"}=" H1* MDE";
$resn_exch{"HN61 MDE"}="1HN6 MDE"; $resn_exch{"HN62 MDE"}="2HN6 MDE"; $resn_exch{" H7' MDE"}=" H7* MDE";
$resn_exch{" H8' MDE"}=" H8* MDE"; $resn_exch{"H121 MDE"}="1H12 MDE"; $resn_exch{"H122 MDE"}="2H12 MDE";
$resn_exch{"H131 MDE"}="1H13 MDE"; $resn_exch{"H132 MDE"}="2H13 MDE"; $resn_exch{"H133 MDE"}="3H13 MDE";
$resn_exch{"H141 MDE"}="1H14 MDE"; $resn_exch{"H142 MDE"}="2H14 MDE"; $resn_exch{"H143 MDE"}="3H14 MDE";
$resn_exch{" H71 MDE"}="1H7  MDE"; $resn_exch{" H72 MDE"}="2H7  MDE"; $resn_exch{" H61 MDE"}="1H6  MDE";
$resn_exch{" H62 MDE"}="2H6  MDE"; $resn_exch{" H31 MDE"}="1H3  MDE"; $resn_exch{" H32 MDE"}="2H3  MDE";
$resn_exch{" H21 MDE"}="1H2  MDE"; $resn_exch{" H22 MDE"}="2H2  MDE"; $resn_exch{" H2' MDE"}=" H2* MDE";
$resn_exch{" H3' MDE"}=" H3* MDE"; $resn_exch{"H4'1 MDE"}="1H4* MDE"; $resn_exch{"H4'2 MDE"}="2H4* MDE";
$resn_exch{" H5' MDE"}=" H5* MDE"; $resn_exch{" HB1 MDE"}="1HB  MDE"; $resn_exch{" HB2 MDE"}="2HB  MDE";
$resn_exch{"HB11 MDE"}="HB1  MDE"; $resn_exch{"H6'1 MDE"}="1H6* MDE"; $resn_exch{"H6'2 MDE"}="2H6* MDE";
$resn_exch{"H7'1 MDE"}="1H7* MDE"; $resn_exch{"H7'2 MDE"}="2H7* MDE"; $resn_exch{"H8'1 MDE"}="1H8* MDE";
$resn_exch{"H8'2 MDE"}="2H8* MDE"; $resn_exch{"H9'1 MDE"}="1H9* MDE"; $resn_exch{"H9'2 MDE"}="2H9* MDE";
$resn_exch{"H101 MDE"}="1H10 MDE"; $resn_exch{"H102 MDE"}="2H10 MDE"; $resn_exch{"H103 MDE"}="3H10 MDE";
$resn_exch{"HOB2 MDG"}="2HOB MDG"; $resn_exch{"HOB3 MDG"}="3HOB MDG"; $resn_exch{"HOA2 MDG"}="2HOA MDG";
$resn_exch{"H5'1 MDG"}="1H5* MDG"; $resn_exch{"H5'2 MDG"}="2H5* MDG"; $resn_exch{" H4' MDG"}=" H4* MDG";
$resn_exch{" H3' MDG"}=" H3* MDG"; $resn_exch{"HO3' MDG"}="*HO3 MDG"; $resn_exch{" H2' MDG"}=" H2* MDG";
$resn_exch{"HO2' MDG"}="*HO2 MDG"; $resn_exch{" H1' MDG"}=" H1* MDG"; $resn_exch{" H81 MDG"}="1H8  MDG";
$resn_exch{" H82 MDG"}="2H8  MDG"; $resn_exch{"HM71 MDG"}="1HM7 MDG"; $resn_exch{"HM72 MDG"}="2HM7 MDG";
$resn_exch{"HM73 MDG"}="3HM7 MDG"; $resn_exch{"HN21 MDG"}="1HN2 MDG"; $resn_exch{"HN22 MDG"}="2HN2 MDG";
$resn_exch{" H   MDH"}=" HN  MDH"; $resn_exch{" HM1 MDH"}="1HM  MDH"; $resn_exch{" HM2 MDH"}="2HM  MDH";
$resn_exch{" HM3 MDH"}="3HM  MDH"; $resn_exch{" HG1 MDH"}="1HG  MDH"; $resn_exch{" HG2 MDH"}="2HG  MDH";
$resn_exch{" HG3 MDH"}="3HG  MDH"; $resn_exch{" H11 MDL"}="1H1  MDL"; $resn_exch{" H12 MDL"}="2H1  MDL";
$resn_exch{" H13 MDL"}="3H1  MDL"; $resn_exch{"H111 MDL"}="1H11 MDL"; $resn_exch{"H112 MDL"}="2H11 MDL";
$resn_exch{"H121 MDL"}="1H12 MDL"; $resn_exch{"H122 MDL"}="2H12 MDL"; $resn_exch{"H151 MDL"}="1H15 MDL";
$resn_exch{"H152 MDL"}="2H15 MDL"; $resn_exch{"H261 MDL"}="1H26 MDL"; $resn_exch{"H262 MDL"}="2H26 MDL";
$resn_exch{"HN33 MDL"}="3HN3 MDL"; $resn_exch{"H551 MDL"}="1H55 MDL"; $resn_exch{"H552 MDL"}="2H55 MDL";
$resn_exch{" H61 MDM"}="1H6  MDM"; $resn_exch{" H62 MDM"}="2H6  MDM"; $resn_exch{"H6'1 MDM"}="1H6' MDM";
$resn_exch{"H6'2 MDM"}="2H6' MDM"; $resn_exch{"H7'1 MDM"}="1H7' MDM"; $resn_exch{"H7'2 MDM"}="2H7' MDM";
$resn_exch{"H7'3 MDM"}="3H7' MDM"; $resn_exch{"HO2' MDM"}="'HO2 MDM"; $resn_exch{"HO4' MDM"}="'HO4 MDM";
$resn_exch{"HO6' MDM"}="'HO6 MDM"; $resn_exch{" HO3 MDN"}="3HO  MDN"; $resn_exch{" HO2 MDN"}="2HO  MDN";
$resn_exch{" H41 MDN"}="1H4  MDN"; $resn_exch{" H42 MDN"}="2H4  MDN"; $resn_exch{" HO7 MDN"}="7HO  MDN";
$resn_exch{" HO8 MDN"}="8HO  MDN"; $resn_exch{" H   MDO"}="1HN  MDO"; $resn_exch{" H2  MDO"}="2HN  MDO";
$resn_exch{" HB1 MDO"}="1HC  MDO"; $resn_exch{" HB2 MDO"}="2HC  MDO"; $resn_exch{" HB3 MDO"}="3HC  MDO";
$resn_exch{"HB21 MDO"}="1HB2 MDO"; $resn_exch{"HB22 MDO"}="2HB2 MDO"; $resn_exch{"HB23 MDO"}="3HB2 MDO";
$resn_exch{"HA31 MDO"}="1HA3 MDO"; $resn_exch{"HA32 MDO"}="2HA3 MDO"; $resn_exch{" H61 MDP"}="1H6  MDP";
$resn_exch{" H62 MDP"}="2H6  MDP"; $resn_exch{" H91 MDP"}="1H9  MDP"; $resn_exch{" H92 MDP"}="2H9  MDP";
$resn_exch{" H93 MDP"}="3H9  MDP"; $resn_exch{"H111 MDP"}="1H11 MDP"; $resn_exch{"H112 MDP"}="2H11 MDP";
$resn_exch{"H113 MDP"}="3H11 MDP"; $resn_exch{"H6'1 MDR"}="1H6' MDR"; $resn_exch{"H6'2 MDR"}="2H6' MDR";
$resn_exch{"H6'3 MDR"}="3H6' MDR"; $resn_exch{"H2'1 MDR"}="1H2' MDR"; $resn_exch{"H2'2 MDR"}="2H2' MDR";
$resn_exch{"H5'1 MDR"}="1H5' MDR"; $resn_exch{"H5'2 MDR"}="2H5' MDR"; $resn_exch{" H28 MDW"}="8H2  MDW";
$resn_exch{" H27 MDW"}="7H2  MDW"; $resn_exch{" H35 MDW"}="5H3  MDW"; $resn_exch{" H36 MDW"}="6H3  MDW";
$resn_exch{" H37 MDW"}="7H3  MDW"; $resn_exch{" H29 MDW"}="9H2  MDW"; $resn_exch{" H30 MDW"}="0H3  MDW";
$resn_exch{" H38 MDW"}="8H3  MDW"; $resn_exch{" H39 MDW"}="9H3  MDW"; $resn_exch{" H31 MDW"}="1H3  MDW";
$resn_exch{" H32 MDW"}="2H3  MDW"; $resn_exch{" H33 MDW"}="3H3  MDW"; $resn_exch{" H34 MDW"}="4H3  MDW";
$resn_exch{" H48 MDW"}="8H4  MDW"; $resn_exch{" H42 MDW"}="2H4  MDW"; $resn_exch{" H45 MDW"}="5H4  MDW";
$resn_exch{" H46 MDW"}="6H4  MDW"; $resn_exch{" H47 MDW"}="7H4  MDW"; $resn_exch{" H43 MDW"}="3H4  MDW";
$resn_exch{" H44 MDW"}="4H4  MDW"; $resn_exch{"H101 MDX"}="1H10 MDX"; $resn_exch{"H102 MDX"}="2H10 MDX";
$resn_exch{" H91 MDX"}="1H9  MDX"; $resn_exch{" H92 MDX"}="2H9  MDX"; $resn_exch{" H81 MDX"}="1H8  MDX";
$resn_exch{" H82 MDX"}="2H8  MDX"; $resn_exch{" H71 MDX"}="1H7  MDX"; $resn_exch{" H72 MDX"}="2H7  MDX";
$resn_exch{" H61 MDX"}="1H6  MDX"; $resn_exch{" H62 MDX"}="2H6  MDX"; $resn_exch{" H51 MDX"}="1H5  MDX";
$resn_exch{" H52 MDX"}="2H5  MDX"; $resn_exch{" H41 MDX"}="1H4  MDX"; $resn_exch{" H42 MDX"}="2H4  MDX";
$resn_exch{" H31 MDX"}="1H3  MDX"; $resn_exch{" H32 MDX"}="2H3  MDX"; $resn_exch{" H21 MDX"}="1H2  MDX";
$resn_exch{" H22 MDX"}="2H2  MDX"; $resn_exch{" H11 MDX"}="1H1  MDX"; $resn_exch{" H12 MDX"}="2H1  MDX";
$resn_exch{"H251 MDZ"}="1H25 MDZ"; $resn_exch{"H252 MDZ"}="2H25 MDZ"; $resn_exch{"H253 MDZ"}="3H25 MDZ";
$resn_exch{"H7C1 MDZ"}="1H7C MDZ"; $resn_exch{"H7C2 MDZ"}="2H7C MDZ"; $resn_exch{"H4C1 MDZ"}="1H4C MDZ";
$resn_exch{"H4C2 MDZ"}="2H4C MDZ"; $resn_exch{"H3C1 MDZ"}="1H3C MDZ"; $resn_exch{"H3C2 MDZ"}="2H3C MDZ";
$resn_exch{"H141 MDZ"}="1H14 MDZ"; $resn_exch{"H142 MDZ"}="2H14 MDZ"; $resn_exch{"H371 MDZ"}="1H37 MDZ";
$resn_exch{"H372 MDZ"}="2H37 MDZ"; $resn_exch{"H373 MDZ"}="3H37 MDZ"; $resn_exch{"H331 MDZ"}="1H33 MDZ";
$resn_exch{"H332 MDZ"}="2H33 MDZ"; $resn_exch{"H333 MDZ"}="3H33 MDZ"; $resn_exch{" H91 ME1"}="1H9  ME1";
$resn_exch{" H92 ME1"}="2H9  ME1"; $resn_exch{"H101 ME1"}="1H10 ME1"; $resn_exch{"H102 ME1"}="2H10 ME1";
$resn_exch{" H71 ME2"}="1H7  ME2"; $resn_exch{" H72 ME2"}="2H7  ME2"; $resn_exch{" H73 ME2"}="3H7  ME2";
$resn_exch{" H61 ME2"}="1H6  ME2"; $resn_exch{" H62 ME2"}="2H6  ME2"; $resn_exch{" H51 ME2"}="1H5  ME2";
$resn_exch{" H52 ME2"}="2H5  ME2"; $resn_exch{" H41 ME2"}="1H4  ME2"; $resn_exch{" H42 ME2"}="2H4  ME2";
$resn_exch{" H31 ME2"}="1H3  ME2"; $resn_exch{" H32 ME2"}="2H3  ME2"; $resn_exch{" H21 ME2"}="1H2  ME2";
$resn_exch{" H22 ME2"}="2H2  ME2"; $resn_exch{" H11 ME2"}="1H1  ME2"; $resn_exch{" H12 ME2"}="2H1  ME2";
$resn_exch{" H13 ME2"}="3H1  ME2"; $resn_exch{"H251 ME3"}="1H25 ME3"; $resn_exch{"H252 ME3"}="2H25 ME3";
$resn_exch{"H261 ME3"}="1H26 ME3"; $resn_exch{"H262 ME3"}="2H26 ME3"; $resn_exch{"H263 ME3"}="3H26 ME3";
$resn_exch{"H8C1 ME3"}="1H8C ME3"; $resn_exch{"H9C1 ME3"}="1H9C ME3"; $resn_exch{"H5C1 ME3"}="1H5C ME3";
$resn_exch{"H4C1 ME3"}="1H4C ME3"; $resn_exch{" H21 ME4"}="1H2  ME4"; $resn_exch{" H22 ME4"}="2H2  ME4";
$resn_exch{"H31A ME4"}="1H3  ME4"; $resn_exch{"H32A ME4"}="2H3  ME4"; $resn_exch{" H41 ME4"}="1H4  ME4";
$resn_exch{" H42 ME4"}="2H4  ME4"; $resn_exch{" H51 ME4"}="1H5  ME4"; $resn_exch{" H52 ME4"}="2H5  ME4";
$resn_exch{" H81 ME5"}="1H8  ME5"; $resn_exch{" H82 ME5"}="2H8  ME5"; $resn_exch{"H101 ME5"}="1H10 ME5";
$resn_exch{"H102 ME5"}="2H10 ME5"; $resn_exch{"H121 ME5"}="1H12 ME5"; $resn_exch{"H122 ME5"}="2H12 ME5";
$resn_exch{"H1C1 MEA"}="1H1C MEA"; $resn_exch{"H1C2 MEA"}="2H1C MEA"; $resn_exch{"H1C3 MEA"}="3H1C MEA";
$resn_exch{"H'C1 MEA"}="1H'C MEA"; $resn_exch{"H'C2 MEA"}="2H'C MEA"; $resn_exch{" H21 MEB"}="1H2  MEB";
$resn_exch{" H31 MEB"}="1H3  MEB"; $resn_exch{" H32 MEB"}="2H3  MEB"; $resn_exch{"H1'1 MEB"}="1H1' MEB";
$resn_exch{"H1'2 MEB"}="2H1' MEB"; $resn_exch{"H1'3 MEB"}="3H1' MEB"; $resn_exch{" H23 MEC"}="3H2  MEC";
$resn_exch{" H22 MEC"}="2H2  MEC"; $resn_exch{" H21 MEC"}="1H2  MEC"; $resn_exch{" H42 MEC"}="2H4  MEC";
$resn_exch{" H41 MEC"}="1H4  MEC"; $resn_exch{" H53 MEC"}="3H5  MEC"; $resn_exch{" H52 MEC"}="2H5  MEC";
$resn_exch{" H51 MEC"}="1H5  MEC"; $resn_exch{" H2  MED"}=" HN2 MED"; $resn_exch{" HB2 MED"}="1HB  MED";
$resn_exch{" HB3 MED"}="2HB  MED"; $resn_exch{" HG2 MED"}="1HG  MED"; $resn_exch{" HG3 MED"}="2HG  MED";
$resn_exch{" HE1 MED"}="1HE  MED"; $resn_exch{" HE2 MED"}="2HE  MED"; $resn_exch{" HE3 MED"}="3HE  MED";
$resn_exch{" H1  MEE"}="1H   MEE"; $resn_exch{" H2  MEE"}="2H   MEE"; $resn_exch{" H3  MEE"}="3H   MEE";
$resn_exch{" H2  MEG"}=" HN2 MEG"; $resn_exch{" HG2 MEG"}="1HG  MEG"; $resn_exch{" HG3 MEG"}="2HG  MEG";
$resn_exch{" H11 MEG"}="1H1  MEG"; $resn_exch{" H12 MEG"}="2H1  MEG"; $resn_exch{" H13 MEG"}="3H1  MEG";
$resn_exch{" HE2 MEG"}=" HO1 MEG"; $resn_exch{"H11A MEI"}="1H1  MEI"; $resn_exch{"H12A MEI"}="2H1  MEI";
$resn_exch{" H13 MEI"}="3H1  MEI"; $resn_exch{" H31 MEI"}="1H3  MEI"; $resn_exch{" H32 MEI"}="2H3  MEI";
$resn_exch{" H33 MEI"}="3H3  MEI"; $resn_exch{" H41 MEI"}="1H4  MEI"; $resn_exch{" H42 MEI"}="2H4  MEI";
$resn_exch{" H43 MEI"}="3H4  MEI"; $resn_exch{" H51 MEI"}="1H5  MEI"; $resn_exch{" H52 MEI"}="2H5  MEI";
$resn_exch{" H61 MEI"}="1H6  MEI"; $resn_exch{" H62 MEI"}="2H6  MEI"; $resn_exch{" H71 MEI"}="1H7  MEI";
$resn_exch{" H72 MEI"}="2H7  MEI"; $resn_exch{" H91 MEI"}="1H9  MEI"; $resn_exch{" H92 MEI"}="2H9  MEI";
$resn_exch{" H93 MEI"}="3H9  MEI"; $resn_exch{"H101 MEI"}="1H10 MEI"; $resn_exch{"H102 MEI"}="2H10 MEI";
$resn_exch{"H141 MEI"}="1H14 MEI"; $resn_exch{"H142 MEI"}="2H14 MEI"; $resn_exch{"H143 MEI"}="3H14 MEI";
$resn_exch{" H11 MEL"}="1H1  MEL"; $resn_exch{" H21 MEL"}="1H2  MEL"; $resn_exch{" H22 MEL"}="2H2  MEL";
$resn_exch{" H31 MEL"}="1H3  MEL"; $resn_exch{" H32 MEL"}="2H3  MEL"; $resn_exch{" H41 MEL"}="1H4  MEL";
$resn_exch{" H42 MEL"}="2H4  MEL"; $resn_exch{" H51 MEL"}="1H5  MEL"; $resn_exch{" H52 MEL"}="2H5  MEL";
$resn_exch{" H61 MEL"}="1H6  MEL"; $resn_exch{" H62 MEL"}="2H6  MEL"; $resn_exch{" H71 MEL"}="1H7  MEL";
$resn_exch{" H81 MEL"}="1H8  MEL"; $resn_exch{"H121 MEL"}="1H12 MEL"; $resn_exch{"H151 MEL"}="1H15 MEL";
$resn_exch{"H161 MEL"}="1H16 MEL"; $resn_exch{"H162 MEL"}="2H16 MEL"; $resn_exch{"H181 MEL"}="1H18 MEL";
$resn_exch{"H191 MEL"}="1H19 MEL"; $resn_exch{"H211 MEL"}="1H21 MEL"; $resn_exch{"H221 MEL"}="1H22 MEL";
$resn_exch{"H241 MEL"}="1H24 MEL"; $resn_exch{"H251 MEL"}="1H25 MEL"; $resn_exch{"H252 MEL"}="2H25 MEL";
$resn_exch{"H261 MEL"}="1H26 MEL"; $resn_exch{"H262 MEL"}="2H26 MEL"; $resn_exch{"H271 MEL"}="1H27 MEL";
$resn_exch{"H272 MEL"}="2H27 MEL"; $resn_exch{"H281 MEL"}="1H28 MEL"; $resn_exch{"H282 MEL"}="2H28 MEL";
$resn_exch{"H301 MEL"}="1H30 MEL"; $resn_exch{" H2  MEN"}=" HN2 MEN"; $resn_exch{" HB2 MEN"}="1HB  MEN";
$resn_exch{" HB3 MEN"}="2HB  MEN"; $resn_exch{"HE21 MEN"}="1HE2 MEN"; $resn_exch{"HE22 MEN"}="2HE2 MEN";
$resn_exch{"HE23 MEN"}="3HE2 MEN"; $resn_exch{"HOP2 MEP"}="2HOP MEP"; $resn_exch{"HOP3 MEP"}="3HOP MEP";
$resn_exch{" H5' MEP"}="1H5* MEP"; $resn_exch{"H5'' MEP"}="2H5* MEP"; $resn_exch{" H4' MEP"}=" H4* MEP";
$resn_exch{" H3' MEP"}=" H3* MEP"; $resn_exch{"HO3' MEP"}="H3T  MEP"; $resn_exch{"H3T2 MEP"}="2H3T MEP";
$resn_exch{"H3T3 MEP"}="3H3T MEP"; $resn_exch{" H2' MEP"}=" H2* MEP"; $resn_exch{" H1' MEP"}=" H1* MEP";
$resn_exch{" H71 MEP"}="1H5M MEP"; $resn_exch{" H72 MEP"}="2H5M MEP"; $resn_exch{" H73 MEP"}="3H5M MEP";
$resn_exch{"HA'1 MEP"}="1HA* MEP"; $resn_exch{"HA'2 MEP"}="2HA* MEP"; $resn_exch{"HA'3 MEP"}="3HA* MEP";
$resn_exch{" H2  MEQ"}=" HN2 MEQ"; $resn_exch{" HB2 MEQ"}="1HB  MEQ"; $resn_exch{" HB3 MEQ"}="2HB  MEQ";
$resn_exch{" HG2 MEQ"}="1HG  MEQ"; $resn_exch{" HG3 MEQ"}="2HG  MEQ"; $resn_exch{"HE21 MEQ"}="1HE2 MEQ";
$resn_exch{" HE1 MEQ"}="1HE  MEQ"; $resn_exch{" HE2 MEQ"}="2HE  MEQ"; $resn_exch{" HE3 MEQ"}="3HE  MEQ";
$resn_exch{" HC2 MER"}="2HC  MER"; $resn_exch{" HC6 MER"}="6HC  MER"; $resn_exch{" HC5 MER"}="5HC  MER";
$resn_exch{"HC16 MER"}="6HC1 MER"; $resn_exch{"HO26 MER"}="6HO2 MER"; $resn_exch{"H621 MER"}="1H62 MER";
$resn_exch{"H622 MER"}="2H62 MER"; $resn_exch{"H623 MER"}="3H62 MER"; $resn_exch{"HO23 MER"}="3HO2 MER";
$resn_exch{"HCA1 MER"}="1HCA MER"; $resn_exch{"H4A1 MER"}="1H4A MER"; $resn_exch{"H4A2 MER"}="2H4A MER";
$resn_exch{"HCA5 MER"}="5HCA MER"; $resn_exch{" HC1 MER"}="1HC  MER"; $resn_exch{"HNA3 MER"}="3HNA MER";
$resn_exch{"H111 MER"}="1H11 MER"; $resn_exch{"H112 MER"}="2H11 MER"; $resn_exch{"H113 MER"}="3H11 MER";
$resn_exch{"H2A1 MER"}="1H2A MER"; $resn_exch{"H2A2 MER"}="2H2A MER"; $resn_exch{"H8A1 MER"}="1H8A MER";
$resn_exch{"H8A2 MER"}="2H8A MER"; $resn_exch{"H8A3 MER"}="3H8A MER"; $resn_exch{"H9A1 MER"}="1H9A MER";
$resn_exch{"H9A2 MER"}="2H9A MER"; $resn_exch{"H9A3 MER"}="3H9A MER"; $resn_exch{" H21 MES"}="1H2  MES";
$resn_exch{" H22 MES"}="2H2  MES"; $resn_exch{" H31 MES"}="1H3  MES"; $resn_exch{" H32 MES"}="2H3  MES";
$resn_exch{" H51 MES"}="1H5  MES"; $resn_exch{" H52 MES"}="2H5  MES"; $resn_exch{" H61 MES"}="1H6  MES";
$resn_exch{" H62 MES"}="2H6  MES"; $resn_exch{" H71 MES"}="1H7  MES"; $resn_exch{" H72 MES"}="2H7  MES";
$resn_exch{" H81 MES"}="1H8  MES"; $resn_exch{" H82 MES"}="2H8  MES"; $resn_exch{" H2  MET"}=" HN2 MET";
$resn_exch{" HB2 MET"}="1HB  MET"; $resn_exch{" HB3 MET"}="2HB  MET"; $resn_exch{" HG2 MET"}="1HG  MET";
$resn_exch{" HG3 MET"}="2HG  MET"; $resn_exch{" HE1 MET"}="1HE  MET"; $resn_exch{" HE2 MET"}="2HE  MET";
$resn_exch{" HE3 MET"}="3HE  MET"; $resn_exch{" HB1 MEU"}="1HB  MEU"; $resn_exch{" HB2 MEU"}="2HB  MEU";
$resn_exch{" HB3 MEU"}="3HB  MEU"; $resn_exch{" H   MEU"}="1HN  MEU"; $resn_exch{" H2  MEU"}="2HN  MEU";
$resn_exch{" HA1 MEU"}="1HA  MEU"; $resn_exch{" HA2 MEU"}="2HA  MEU"; $resn_exch{" H81 MEV"}="1H8  MEV";
$resn_exch{" H82 MEV"}="2H8  MEV"; $resn_exch{" H21 MEV"}="1H2  MEV"; $resn_exch{" H22 MEV"}="2H2  MEV";
$resn_exch{" H41 MEV"}="1H4  MEV"; $resn_exch{" H42 MEV"}="2H4  MEV"; $resn_exch{" H61 MEV"}="1H6  MEV";
$resn_exch{" H62 MEV"}="2H6  MEV"; $resn_exch{" H63 MEV"}="3H6  MEV"; $resn_exch{" H41 MEZ"}="1H4  MEZ";
$resn_exch{" H42 MEZ"}="2H4  MEZ"; $resn_exch{" H43 MEZ"}="3H4  MEZ"; $resn_exch{" H21 MF2"}="1H2  MF2";
$resn_exch{" H22 MF2"}="2H2  MF2"; $resn_exch{" H71 MF2"}="1H7  MF2"; $resn_exch{" H72 MF2"}="2H7  MF2";
$resn_exch{" H73 MF2"}="3H7  MF2"; $resn_exch{" H81 MF2"}="1H8  MF2"; $resn_exch{" H82 MF2"}="2H8  MF2";
$resn_exch{" H83 MF2"}="3H8  MF2"; $resn_exch{" H91 MF2"}="1H9  MF2"; $resn_exch{" H92 MF2"}="2H9  MF2";
$resn_exch{"H101 MF2"}="1H10 MF2"; $resn_exch{"H102 MF2"}="2H10 MF2"; $resn_exch{"H111 MF2"}="1H11 MF2";
$resn_exch{"H112 MF2"}="2H11 MF2"; $resn_exch{"H121 MF2"}="1H12 MF2"; $resn_exch{"H122 MF2"}="2H12 MF2";
$resn_exch{"H131 MF2"}="1H13 MF2"; $resn_exch{"H132 MF2"}="2H13 MF2"; $resn_exch{"H141 MF2"}="1H14 MF2";
$resn_exch{"H142 MF2"}="2H14 MF2"; $resn_exch{"H151 MF2"}="1H15 MF2"; $resn_exch{"H152 MF2"}="2H15 MF2";
$resn_exch{"H161 MF2"}="1H16 MF2"; $resn_exch{"H162 MF2"}="2H16 MF2"; $resn_exch{" H61 MF2"}="1H6  MF2";
$resn_exch{" H62 MF2"}="2H6  MF2"; $resn_exch{" H2  MF3"}=" HN2 MF3"; $resn_exch{" HB2 MF3"}="1HB  MF3";
$resn_exch{" HB3 MF3"}="2HB  MF3"; $resn_exch{" HG2 MF3"}="1HG  MF3"; $resn_exch{" HG3 MF3"}="2HG  MF3";
$resn_exch{" HM1 MFA"}="1HM  MFA"; $resn_exch{" HM2 MFA"}="2HM  MFA"; $resn_exch{" HM3 MFA"}="3HM  MFA";
$resn_exch{" H61 MFA"}="1H6  MFA"; $resn_exch{" H62 MFA"}="2H6  MFA"; $resn_exch{" H63 MFA"}="3H6  MFA";
$resn_exch{" H61 MFB"}="1H6  MFB"; $resn_exch{" H62 MFB"}="2H6  MFB"; $resn_exch{" H63 MFB"}="3H6  MFB";
$resn_exch{" HM1 MFB"}="1HM  MFB"; $resn_exch{" HM2 MFB"}="2HM  MFB"; $resn_exch{" HM3 MFB"}="3HM  MFB";
$resn_exch{"HG11 MFC"}="1HG1 MFC"; $resn_exch{"HG12 MFC"}="2HG1 MFC"; $resn_exch{"HG13 MFC"}="3HG1 MFC";
$resn_exch{"HA31 MFC"}="1HA3 MFC"; $resn_exch{"HA32 MFC"}="2HA3 MFC"; $resn_exch{"HN31 MFD"}="1HN3 MFD";
$resn_exch{"HN32 MFD"}="2HN3 MFD"; $resn_exch{"HM21 MFD"}="1HM2 MFD"; $resn_exch{"HM22 MFD"}="2HM2 MFD";
$resn_exch{"HM23 MFD"}="3HM2 MFD"; $resn_exch{" H'1 MFD"}="1H'  MFD"; $resn_exch{" H'2 MFD"}="2H'  MFD";
$resn_exch{"HM91 MFD"}="1HM9 MFD"; $resn_exch{"HM92 MFD"}="2HM9 MFD"; $resn_exch{"HM93 MFD"}="3HM9 MFD";
$resn_exch{"HM81 MFD"}="1HM8 MFD"; $resn_exch{"HM82 MFD"}="2HM8 MFD"; $resn_exch{"HM83 MFD"}="3HM8 MFD";
$resn_exch{"HM61 MFD"}="1HM6 MFD"; $resn_exch{"HM62 MFD"}="2HM6 MFD"; $resn_exch{"HM63 MFD"}="3HM6 MFD";
$resn_exch{"H151 MFM"}="1H15 MFM"; $resn_exch{"H152 MFM"}="2H15 MFM"; $resn_exch{"H161 MFM"}="1H16 MFM";
$resn_exch{"H162 MFM"}="2H16 MFM"; $resn_exch{"H11A MFN"}="1H1  MFN"; $resn_exch{"H12A MFN"}="2H1  MFN";
$resn_exch{"H221 MFN"}="1H22 MFN"; $resn_exch{"H222 MFN"}="2H22 MFN"; $resn_exch{"H141 MFN"}="1H14 MFN";
$resn_exch{"H142 MFN"}="2H14 MFN"; $resn_exch{"H261 MFN"}="1H26 MFN"; $resn_exch{"H262 MFN"}="2H26 MFN";
$resn_exch{"H271 MFN"}="1H27 MFN"; $resn_exch{"H272 MFN"}="2H27 MFN"; $resn_exch{"H282 MFN"}="2H28 MFN";
$resn_exch{"H171 MFN"}="1H17 MFN"; $resn_exch{"H172 MFN"}="2H17 MFN"; $resn_exch{"H181 MFN"}="1H18 MFN";
$resn_exch{"H182 MFN"}="2H18 MFN"; $resn_exch{"H151 MFN"}="1H15 MFN"; $resn_exch{"H152 MFN"}="2H15 MFN";
$resn_exch{"H121 MFN"}="1H12 MFN"; $resn_exch{"H122 MFN"}="2H12 MFN"; $resn_exch{"H131 MFN"}="1H13 MFN";
$resn_exch{"H132 MFN"}="2H13 MFN"; $resn_exch{"HN11 MFN"}="1HN1 MFN"; $resn_exch{"H311 MFN"}="1H31 MFN";
$resn_exch{"H312 MFN"}="2H31 MFN"; $resn_exch{"H321 MFN"}="1H32 MFN"; $resn_exch{"H322 MFN"}="2H32 MFN";
$resn_exch{"H161 MFP"}="1H16 MFP"; $resn_exch{"H162 MFP"}="2H16 MFP"; $resn_exch{"H151 MFP"}="1H15 MFP";
$resn_exch{"H152 MFP"}="2H15 MFP"; $resn_exch{"H153 MFP"}="3H15 MFP"; $resn_exch{"H271 MFP"}="1H27 MFP";
$resn_exch{"H272 MFP"}="2H27 MFP"; $resn_exch{"H291 MFP"}="1H29 MFP"; $resn_exch{"H292 MFP"}="2H29 MFP";
$resn_exch{"H311 MFP"}="1H31 MFP"; $resn_exch{"H312 MFP"}="2H31 MFP"; $resn_exch{"H313 MFP"}="3H31 MFP";
$resn_exch{"H321 MFP"}="1H32 MFP"; $resn_exch{"H322 MFP"}="2H32 MFP"; $resn_exch{"H323 MFP"}="3H32 MFP";
$resn_exch{"H161 MFQ"}="1H16 MFQ"; $resn_exch{"H162 MFQ"}="2H16 MFQ"; $resn_exch{"H151 MFQ"}="1H15 MFQ";
$resn_exch{"H152 MFQ"}="2H15 MFQ"; $resn_exch{"H153 MFQ"}="3H15 MFQ"; $resn_exch{"H271 MFQ"}="1H27 MFQ";
$resn_exch{"H272 MFQ"}="2H27 MFQ"; $resn_exch{"H291 MFQ"}="1H29 MFQ"; $resn_exch{"H292 MFQ"}="2H29 MFQ";
$resn_exch{"H311 MFQ"}="1H31 MFQ"; $resn_exch{"H312 MFQ"}="2H31 MFQ"; $resn_exch{"H313 MFQ"}="3H31 MFQ";
$resn_exch{"H321 MFQ"}="1H32 MFQ"; $resn_exch{"H322 MFQ"}="2H32 MFQ"; $resn_exch{"H323 MFQ"}="3H32 MFQ";
$resn_exch{" H61 MFU"}="1H6  MFU"; $resn_exch{" H62 MFU"}="2H6  MFU"; $resn_exch{" H63 MFU"}="3H6  MFU";
$resn_exch{" HM1 MFU"}="1HM  MFU"; $resn_exch{" HM2 MFU"}="2HM  MFU"; $resn_exch{" HM3 MFU"}="3HM  MFU";
$resn_exch{"HOP3 MG1"}="3HOP MG1"; $resn_exch{"HOP2 MG1"}="2HOP MG1"; $resn_exch{" H5' MG1"}="1H5* MG1";
$resn_exch{"H5'' MG1"}="2H5* MG1"; $resn_exch{" H4' MG1"}=" H4* MG1"; $resn_exch{" H3' MG1"}=" H3* MG1";
$resn_exch{"HO3' MG1"}="H3T  MG1"; $resn_exch{" H2' MG1"}="1H2* MG1"; $resn_exch{"H2'' MG1"}="2H2* MG1";
$resn_exch{" H1' MG1"}=" H1* MG1"; $resn_exch{"HM11 MG1"}="1HM1 MG1"; $resn_exch{"HM12 MG1"}="2HM1 MG1";
$resn_exch{"HM13 MG1"}="3HM1 MG1"; $resn_exch{" H21 MG1"}="1H2  MG1"; $resn_exch{" H22 MG1"}="2H2  MG1";
$resn_exch{"HN21 MG7"}="1HN2 MG7"; $resn_exch{"HN22 MG7"}="2HN2 MG7"; $resn_exch{"HO5' MG7"}="'HO5 MG7";
$resn_exch{"H5'1 MG7"}="1H5' MG7"; $resn_exch{"H5'2 MG7"}="2H5' MG7"; $resn_exch{" H71 MG7"}="1H7  MG7";
$resn_exch{" H72 MG7"}="2H7  MG7"; $resn_exch{" H73 MG7"}="3H7  MG7"; $resn_exch{"HO2' MG7"}="'HO2 MG7";
$resn_exch{"HO3' MG7"}="'HO3 MG7"; $resn_exch{"H11A MG8"}="1H1  MG8"; $resn_exch{"H12A MG8"}="2H1  MG8";
$resn_exch{"H13A MG8"}="3H1  MG8"; $resn_exch{" H21 MG8"}="1H2  MG8"; $resn_exch{" H22 MG8"}="2H2  MG8";
$resn_exch{" H31 MG8"}="1H3  MG8"; $resn_exch{" H32 MG8"}="2H3  MG8"; $resn_exch{" H41 MG8"}="1H4  MG8";
$resn_exch{" H42 MG8"}="2H4  MG8"; $resn_exch{" H51 MG8"}="1H5  MG8"; $resn_exch{" H52 MG8"}="2H5  MG8";
$resn_exch{" H61 MG8"}="1H6  MG8"; $resn_exch{" H62 MG8"}="2H6  MG8"; $resn_exch{" H71 MG8"}="1H7  MG8";
$resn_exch{" H72 MG8"}="2H7  MG8"; $resn_exch{" H91 MG8"}="1H9  MG8"; $resn_exch{" H92 MG8"}="2H9  MG8";
$resn_exch{"H141 MG8"}="1H14 MG8"; $resn_exch{"H142 MG8"}="2H14 MG8"; $resn_exch{"H151 MG8"}="1H15 MG8";
$resn_exch{"H152 MG8"}="2H15 MG8"; $resn_exch{"H153 MG8"}="3H15 MG8"; $resn_exch{" H61 MGA"}="1H6  MGA";
$resn_exch{" H62 MGA"}="2H6  MGA"; $resn_exch{" H71 MGA"}="1H7  MGA"; $resn_exch{" H72 MGA"}="2H7  MGA";
$resn_exch{" H73 MGA"}="3H7  MGA"; $resn_exch{" H11 MGB"}="1H1  MGB"; $resn_exch{" H12 MGB"}="2H1  MGB";
$resn_exch{" H21 MGB"}="1H2  MGB"; $resn_exch{" H41 MGB"}="1H4  MGB"; $resn_exch{" H61 MGB"}="1H6  MGB";
$resn_exch{" H81 MGB"}="1H8  MGB"; $resn_exch{" H82 MGB"}="2H8  MGB"; $resn_exch{" H83 MGB"}="3H8  MGB";
$resn_exch{"H101 MGB"}="1H10 MGB"; $resn_exch{"H121 MGB"}="1H12 MGB"; $resn_exch{"H131 MGB"}="1H13 MGB";
$resn_exch{"H132 MGB"}="2H13 MGB"; $resn_exch{" H61 MGC"}="1H6  MGC"; $resn_exch{" H62 MGC"}="2H6  MGC";
$resn_exch{" HM1 MGC"}="1HM  MGC"; $resn_exch{" HM2 MGC"}="2HM  MGC"; $resn_exch{" HM3 MGC"}="3HM  MGC";
$resn_exch{" H81 MGC"}="1H8  MGC"; $resn_exch{" H82 MGC"}="2H8  MGC"; $resn_exch{" H83 MGC"}="3H8  MGC";
$resn_exch{"HOB2 MGD"}="2HOB MGD"; $resn_exch{"HOA2 MGD"}="2HOA MGD"; $resn_exch{"H5'1 MGD"}="1H5' MGD";
$resn_exch{"H5'2 MGD"}="2H5' MGD"; $resn_exch{"HO3' MGD"}="'HO3 MGD"; $resn_exch{"HO2' MGD"}="'HO2 MGD";
$resn_exch{"HN21 MGD"}="1HN2 MGD"; $resn_exch{"HN22 MGD"}="2HN2 MGD"; $resn_exch{"H101 MGD"}="1H10 MGD";
$resn_exch{"H102 MGD"}="2H10 MGD"; $resn_exch{"H191 MGD"}="1H19 MGD"; $resn_exch{"H192 MGD"}="2H19 MGD";
$resn_exch{"H2A1 MGE"}="1H2A MGE"; $resn_exch{"H2A2 MGE"}="2H2A MGE"; $resn_exch{"H3A1 MGE"}="1H3A MGE";
$resn_exch{"H3A2 MGE"}="2H3A MGE"; $resn_exch{"H4A1 MGE"}="1H4A MGE"; $resn_exch{"H4A2 MGE"}="2H4A MGE";
$resn_exch{"H5A1 MGE"}="1H5A MGE"; $resn_exch{"H5A2 MGE"}="2H5A MGE"; $resn_exch{"H6A1 MGE"}="1H6A MGE";
$resn_exch{"H6A2 MGE"}="2H6A MGE"; $resn_exch{"H7A1 MGE"}="1H7A MGE"; $resn_exch{"H7A2 MGE"}="2H7A MGE";
$resn_exch{"H8A1 MGE"}="1H8A MGE"; $resn_exch{"H8A2 MGE"}="2H8A MGE"; $resn_exch{"H9A1 MGE"}="1H9A MGE";
$resn_exch{"H9A2 MGE"}="2H9A MGE"; $resn_exch{"H101 MGE"}="1H10 MGE"; $resn_exch{"H102 MGE"}="2H10 MGE";
$resn_exch{"H111 MGE"}="1H11 MGE"; $resn_exch{"H112 MGE"}="2H11 MGE"; $resn_exch{"H121 MGE"}="1H12 MGE";
$resn_exch{"H122 MGE"}="2H12 MGE"; $resn_exch{"H131 MGE"}="1H13 MGE"; $resn_exch{"H132 MGE"}="2H13 MGE";
$resn_exch{"H133 MGE"}="3H13 MGE"; $resn_exch{"H2B1 MGE"}="1H2B MGE"; $resn_exch{"H2B2 MGE"}="2H2B MGE";
$resn_exch{"H3B1 MGE"}="1H3B MGE"; $resn_exch{"H3B2 MGE"}="2H3B MGE"; $resn_exch{"H4B1 MGE"}="1H4B MGE";
$resn_exch{"H4B2 MGE"}="2H4B MGE"; $resn_exch{"H5B1 MGE"}="1H5B MGE"; $resn_exch{"H5B2 MGE"}="2H5B MGE";
$resn_exch{"H6B1 MGE"}="1H6B MGE"; $resn_exch{"H6B2 MGE"}="2H6B MGE"; $resn_exch{"H7B1 MGE"}="1H7B MGE";
$resn_exch{"H7B2 MGE"}="2H7B MGE"; $resn_exch{"H8B1 MGE"}="1H8B MGE"; $resn_exch{"H8B2 MGE"}="2H8B MGE";
$resn_exch{"H9B1 MGE"}="1H9B MGE"; $resn_exch{"H9B2 MGE"}="2H9B MGE"; $resn_exch{"H201 MGE"}="1H20 MGE";
$resn_exch{"H202 MGE"}="2H20 MGE"; $resn_exch{"H211 MGE"}="1H21 MGE"; $resn_exch{"H212 MGE"}="2H21 MGE";
$resn_exch{"H221 MGE"}="1H22 MGE"; $resn_exch{"H222 MGE"}="2H22 MGE"; $resn_exch{"H231 MGE"}="1H23 MGE";
$resn_exch{"H232 MGE"}="2H23 MGE"; $resn_exch{"H241 MGE"}="1H24 MGE"; $resn_exch{"H242 MGE"}="2H24 MGE";
$resn_exch{"H251 MGE"}="1H25 MGE"; $resn_exch{"H252 MGE"}="2H25 MGE"; $resn_exch{"H261 MGE"}="1H26 MGE";
$resn_exch{"H262 MGE"}="2H26 MGE"; $resn_exch{"H263 MGE"}="3H26 MGE"; $resn_exch{"H1G1 MGE"}="1H1G MGE";
$resn_exch{"H1G2 MGE"}="2H1G MGE"; $resn_exch{"H3G1 MGE"}="1H3G MGE"; $resn_exch{"H3G2 MGE"}="2H3G MGE";
$resn_exch{"H6D1 MGE"}="1H6D MGE"; $resn_exch{"H6D2 MGE"}="2H6D MGE"; $resn_exch{" HB2 MGG"}="1HB  MGG";
$resn_exch{" HB3 MGG"}="2HB  MGG"; $resn_exch{" HG2 MGG"}="1HG  MGG"; $resn_exch{" HG3 MGG"}="2HG  MGG";
$resn_exch{" HD2 MGG"}="1HD  MGG"; $resn_exch{" HD3 MGG"}="2HD  MGG"; $resn_exch{" HE  MGG"}=" HNE MGG";
$resn_exch{"HH11 MGG"}="1HH1 MGG"; $resn_exch{"HH12 MGG"}="2HH1 MGG"; $resn_exch{"HH21 MGG"}="1HH2 MGG";
$resn_exch{"HH22 MGG"}="2HH2 MGG"; $resn_exch{" H21 MGG"}="1H2  MGG"; $resn_exch{" H22 MGG"}="2H2  MGG";
$resn_exch{" H1  MGG"}="1H   MGG"; $resn_exch{" H61 MGL"}="1H6  MGL"; $resn_exch{" H62 MGL"}="2H6  MGL";
$resn_exch{" H71 MGL"}="1H7  MGL"; $resn_exch{" H72 MGL"}="2H7  MGL"; $resn_exch{" H73 MGL"}="3H7  MGL";
$resn_exch{"H201 MGM"}="1H20 MGM"; $resn_exch{"H202 MGM"}="2H20 MGM"; $resn_exch{"H203 MGM"}="3H20 MGM";
$resn_exch{"H191 MGM"}="1H19 MGM"; $resn_exch{"H192 MGM"}="2H19 MGM"; $resn_exch{"H193 MGM"}="3H19 MGM";
$resn_exch{"H161 MGM"}="1H16 MGM"; $resn_exch{"H162 MGM"}="2H16 MGM"; $resn_exch{"H141 MGM"}="1H14 MGM";
$resn_exch{"H142 MGM"}="2H14 MGM"; $resn_exch{"H143 MGM"}="3H14 MGM"; $resn_exch{"H151 MGM"}="1H15 MGM";
$resn_exch{"H152 MGM"}="2H15 MGM"; $resn_exch{"H111 MGM"}="1H11 MGM"; $resn_exch{"H112 MGM"}="2H11 MGM";
$resn_exch{"HC91 MGM"}="1HC9 MGM"; $resn_exch{"HC92 MGM"}="2HC9 MGM"; $resn_exch{"H101 MGM"}="1H10 MGM";
$resn_exch{"H102 MGM"}="2H10 MGM"; $resn_exch{"H103 MGM"}="3H10 MGM"; $resn_exch{"HC61 MGM"}="1HC6 MGM";
$resn_exch{"HC62 MGM"}="2HC6 MGM"; $resn_exch{"HC51 MGM"}="1HC5 MGM"; $resn_exch{"HC52 MGM"}="2HC5 MGM";
$resn_exch{"HC41 MGM"}="1HC4 MGM"; $resn_exch{"HC42 MGM"}="2HC4 MGM"; $resn_exch{"HC43 MGM"}="3HC4 MGM";
$resn_exch{"HC21 MGM"}="1HC2 MGM"; $resn_exch{"HC22 MGM"}="2HC2 MGM"; $resn_exch{"HC11 MGM"}="1HC1 MGM";
$resn_exch{"HC12 MGM"}="2HC1 MGM"; $resn_exch{"H1AO MGM"}="OH1A MGM"; $resn_exch{"H1BO MGM"}="OH1B MGM";
$resn_exch{"H2BO MGM"}="OH2B MGM"; $resn_exch{" H   MGN"}="1HN  MGN"; $resn_exch{" H2  MGN"}="2HN  MGN";
$resn_exch{"HB11 MGN"}="1HB1 MGN"; $resn_exch{"HB12 MGN"}="2HB1 MGN"; $resn_exch{"HB21 MGN"}="1HB2 MGN";
$resn_exch{"HB22 MGN"}="2HB2 MGN"; $resn_exch{"HB23 MGN"}="3HB2 MGN"; $resn_exch{" HG1 MGN"}="1HG  MGN";
$resn_exch{" HG2 MGN"}="2HG  MGN"; $resn_exch{"HN21 MGN"}="1HN2 MGN"; $resn_exch{"HN22 MGN"}="2HN2 MGN";
$resn_exch{" H51 MGO"}="1H5* MGO"; $resn_exch{" H52 MGO"}="2H5* MGO"; $resn_exch{" H4  MGO"}=" H4* MGO";
$resn_exch{" H3  MGO"}=" H3* MGO"; $resn_exch{" H1  MGO"}=" H1* MGO"; $resn_exch{" H2  MGO"}=" H2* MGO";
$resn_exch{"HAA1 MGO"}="1HAA MGO"; $resn_exch{"HAA2 MGO"}="2HAA MGO"; $resn_exch{"HAA3 MGO"}="3HAA MGO";
$resn_exch{"HAB1 MGO"}="1HAB MGO"; $resn_exch{"HAB2 MGO"}="2HAB MGO"; $resn_exch{" H2' MGQ"}=" H2  MGQ";
$resn_exch{" H4' MGQ"}=" H4  MGQ"; $resn_exch{" H3' MGQ"}=" H3  MGQ"; $resn_exch{"H5'1 MGQ"}="H51  MGQ";
$resn_exch{"H5'2 MGQ"}="H52  MGQ"; $resn_exch{" H31 MGR"}="1H3  MGR"; $resn_exch{" H41 MGR"}="1H4  MGR";
$resn_exch{" H51 MGR"}="1H5  MGR"; $resn_exch{" H61 MGR"}="1H6  MGR"; $resn_exch{" H71 MGR"}="1H7  MGR";
$resn_exch{" H91 MGR"}="1H9  MGR"; $resn_exch{"H101 MGR"}="1H10 MGR"; $resn_exch{"H121 MGR"}="1H12 MGR";
$resn_exch{"H131 MGR"}="1H13 MGR"; $resn_exch{"H151 MGR"}="1H15 MGR"; $resn_exch{"H161 MGR"}="1H16 MGR";
$resn_exch{"H181 MGR"}="1H18 MGR"; $resn_exch{"H191 MGR"}="1H19 MGR"; $resn_exch{"H221 MGR"}="1H22 MGR";
$resn_exch{"H222 MGR"}="2H22 MGR"; $resn_exch{"H223 MGR"}="3H22 MGR"; $resn_exch{"H231 MGR"}="1H23 MGR";
$resn_exch{"H232 MGR"}="2H23 MGR"; $resn_exch{"H233 MGR"}="3H23 MGR"; $resn_exch{"H241 MGR"}="1H24 MGR";
$resn_exch{"H242 MGR"}="2H24 MGR"; $resn_exch{"H243 MGR"}="3H24 MGR"; $resn_exch{"H251 MGR"}="1H25 MGR";
$resn_exch{"H252 MGR"}="2H25 MGR"; $resn_exch{"H253 MGR"}="3H25 MGR"; $resn_exch{"HC71 MGS"}="1HC7 MGS";
$resn_exch{"HC72 MGS"}="2HC7 MGS"; $resn_exch{"HC73 MGS"}="3HC7 MGS"; $resn_exch{"HC81 MGS"}="1HC8 MGS";
$resn_exch{"HC82 MGS"}="2HC8 MGS"; $resn_exch{"HC83 MGS"}="3HC8 MGS"; $resn_exch{"HC61 MGS"}="1HC6 MGS";
$resn_exch{"HC62 MGS"}="2HC6 MGS"; $resn_exch{"HC63 MGS"}="3HC6 MGS"; $resn_exch{"H121 MGS"}="1H12 MGS";
$resn_exch{"H122 MGS"}="2H12 MGS"; $resn_exch{"H111 MGS"}="1H11 MGS"; $resn_exch{"H112 MGS"}="2H11 MGS";
$resn_exch{"HOG2 MGT"}="2HOG MGT"; $resn_exch{"HOG3 MGT"}="3HOG MGT"; $resn_exch{"HOB2 MGT"}="2HOB MGT";
$resn_exch{"HOA2 MGT"}="2HOA MGT"; $resn_exch{"H5'1 MGT"}="1H5* MGT"; $resn_exch{"H5'2 MGT"}="2H5* MGT";
$resn_exch{" H4' MGT"}=" H4* MGT"; $resn_exch{" H3' MGT"}=" H3* MGT"; $resn_exch{"HO3' MGT"}="*HO3 MGT";
$resn_exch{" H2' MGT"}=" H2* MGT"; $resn_exch{"HO2' MGT"}="*HO2 MGT"; $resn_exch{" H1' MGT"}=" H1* MGT";
$resn_exch{" H81 MGT"}="1H8  MGT"; $resn_exch{" H82 MGT"}="2H8  MGT"; $resn_exch{"HM71 MGT"}="1HM7 MGT";
$resn_exch{"HM72 MGT"}="2HM7 MGT"; $resn_exch{"HM73 MGT"}="3HM7 MGT"; $resn_exch{"HN21 MGT"}="1HN2 MGT";
$resn_exch{"HN22 MGT"}="2HN2 MGT"; $resn_exch{"HC81 MGU"}="1HC8 MGU"; $resn_exch{"HC82 MGU"}="2HC8 MGU";
$resn_exch{"HC83 MGU"}="3HC8 MGU"; $resn_exch{"HC61 MGU"}="1HC6 MGU"; $resn_exch{"HC62 MGU"}="2HC6 MGU";
$resn_exch{"HC63 MGU"}="3HC6 MGU"; $resn_exch{"H121 MGU"}="1H12 MGU"; $resn_exch{"H122 MGU"}="2H12 MGU";
$resn_exch{"H111 MGU"}="1H11 MGU"; $resn_exch{"H112 MGU"}="2H11 MGU"; $resn_exch{"H271 MGU"}="1H27 MGU";
$resn_exch{"H272 MGU"}="2H27 MGU"; $resn_exch{"H273 MGU"}="3H27 MGU"; $resn_exch{"H261 MGU"}="1H26 MGU";
$resn_exch{"H262 MGU"}="2H26 MGU"; $resn_exch{"H263 MGU"}="3H26 MGU"; $resn_exch{"H321 MGU"}="1H32 MGU";
$resn_exch{"H322 MGU"}="2H32 MGU"; $resn_exch{"H311 MGU"}="1H31 MGU"; $resn_exch{"H312 MGU"}="2H31 MGU";
$resn_exch{"H5'1 MGV"}="H51  MGV"; $resn_exch{"H5'2 MGV"}="H52  MGV"; $resn_exch{" HD1 MGX"}="1HD  MGX";
$resn_exch{" HD2 MGX"}="2HD  MGX"; $resn_exch{" HD3 MGX"}="3HD  MGX"; $resn_exch{"HH11 MGX"}="1HH1 MGX";
$resn_exch{"HH12 MGX"}="2HH1 MGX"; $resn_exch{" HA1 MGY"}="1HA  MGY"; $resn_exch{" HA2 MGY"}="2HA  MGY";
$resn_exch{" HN1 MGY"}="1HN  MGY"; $resn_exch{" HN2 MGY"}="2HN  MGY"; $resn_exch{" HN3 MGY"}="3HN  MGY";
$resn_exch{"HC11 MHA"}="1HC1 MHA"; $resn_exch{"HC12 MHA"}="2HC1 MHA"; $resn_exch{"HC31 MHA"}="1HC3 MHA";
$resn_exch{"HC32 MHA"}="2HC3 MHA"; $resn_exch{"HC51 MHA"}="1HC5 MHA"; $resn_exch{"HC52 MHA"}="2HC5 MHA";
$resn_exch{"HN21 MHA"}="1HN2 MHA"; $resn_exch{"HN22 MHA"}="2HN2 MHA"; $resn_exch{"HM31 MHB"}="1HM3 MHB";
$resn_exch{"HM32 MHB"}="2HM3 MHB"; $resn_exch{"HM33 MHB"}="3HM3 MHB"; $resn_exch{"HO4' MHB"}="'HO4 MHB";
$resn_exch{" H21 MHC"}="1H2  MHC"; $resn_exch{" H22 MHC"}="2H2  MHC"; $resn_exch{" H31 MHC"}="1H3  MHC";
$resn_exch{" H32 MHC"}="2H3  MHC"; $resn_exch{" H71 MHC"}="1H7  MHC"; $resn_exch{" H72 MHC"}="2H7  MHC";
$resn_exch{" H81 MHC"}="1H8  MHC"; $resn_exch{" H82 MHC"}="2H8  MHC"; $resn_exch{"H111 MHC"}="1H11 MHC";
$resn_exch{"H112 MHC"}="2H11 MHC"; $resn_exch{"HN21 MHF"}="1HN2 MHF"; $resn_exch{"HN22 MHF"}="2HN2 MHF";
$resn_exch{" H71 MHF"}="1H7  MHF"; $resn_exch{" H72 MHF"}="2H7  MHF"; $resn_exch{" H91 MHF"}="1H9  MHF";
$resn_exch{" H92 MHF"}="2H9  MHF"; $resn_exch{"H111 MHF"}="1H11 MHF"; $resn_exch{"H112 MHF"}="2H11 MHF";
$resn_exch{" HB1 MHF"}="1HB  MHF"; $resn_exch{" HB2 MHF"}="2HB  MHF"; $resn_exch{" HG1 MHF"}="1HG  MHF";
$resn_exch{" HG2 MHF"}="2HG  MHF"; $resn_exch{"HOE2 MHF"}="2HOE MHF"; $resn_exch{" H   MHL"}=" HN  MHL";
$resn_exch{" HB2 MHL"}="1HB  MHL"; $resn_exch{" HB3 MHL"}="2HB  MHL"; $resn_exch{"HD11 MHL"}="1HD1 MHL";
$resn_exch{"HD12 MHL"}="2HD1 MHL"; $resn_exch{"HD13 MHL"}="3HD1 MHL"; $resn_exch{"HD21 MHL"}="1HD2 MHL";
$resn_exch{"HD22 MHL"}="2HD2 MHL"; $resn_exch{"HD23 MHL"}="3HD2 MHL"; $resn_exch{" HN1 MHL"}="1HN  MHL";
$resn_exch{" HN2 MHL"}="2HN  MHL"; $resn_exch{" HN3 MHL"}="3HN  MHL"; $resn_exch{" HD3 MHL"}=" HO3 MHL";
$resn_exch{"HAA2 MHM"}="2HAA MHM"; $resn_exch{"HMA2 MHM"}="2HMA MHM"; $resn_exch{"HMA3 MHM"}="3HMA MHM";
$resn_exch{"HBA2 MHM"}="2HBA MHM"; $resn_exch{"HO1A MHM"}="AHO1 MHM"; $resn_exch{"HMB2 MHM"}="2HMB MHM";
$resn_exch{"HMB3 MHM"}="3HMB MHM"; $resn_exch{"HBB2 MHM"}="2HBB MHM"; $resn_exch{"HMC2 MHM"}="2HMC MHM";
$resn_exch{"HMC3 MHM"}="3HMC MHM"; $resn_exch{"HBC2 MHM"}="2HBC MHM"; $resn_exch{"HMD2 MHM"}="2HMD MHM";
$resn_exch{"HMD3 MHM"}="3HMD MHM"; $resn_exch{"HAD2 MHM"}="2HAD MHM"; $resn_exch{"HBD2 MHM"}="2HBD MHM";
$resn_exch{"HO2D MHM"}="DHO2 MHM"; $resn_exch{" H81 MHN"}="1H8  MHN"; $resn_exch{" H82 MHN"}="2H8  MHN";
$resn_exch{" H71 MHN"}="1H7  MHN"; $resn_exch{" H72 MHN"}="2H7  MHN"; $resn_exch{" H61 MHN"}="1H6  MHN";
$resn_exch{" H62 MHN"}="2H6  MHN"; $resn_exch{" H51 MHN"}="1H5  MHN"; $resn_exch{" H52 MHN"}="2H5  MHN";
$resn_exch{" H41 MHN"}="1H4  MHN"; $resn_exch{" H42 MHN"}="2H4  MHN"; $resn_exch{" H11 MHN"}="1H1  MHN";
$resn_exch{" H12 MHN"}="2H1  MHN"; $resn_exch{" H13 MHN"}="3H1  MHN"; $resn_exch{" H31 MHN"}="1H3  MHN";
$resn_exch{" H32 MHN"}="2H3  MHN"; $resn_exch{" H33 MHN"}="3H3  MHN"; $resn_exch{" H   MHO"}="1HN  MHO";
$resn_exch{" H2  MHO"}="2HN  MHO"; $resn_exch{" HB2 MHO"}="1HB  MHO"; $resn_exch{" HB3 MHO"}="2HB  MHO";
$resn_exch{" HG2 MHO"}="1HG  MHO"; $resn_exch{" HG3 MHO"}="2HG  MHO"; $resn_exch{" HE1 MHO"}="1HE  MHO";
$resn_exch{" HE2 MHO"}="2HE  MHO"; $resn_exch{" HE3 MHO"}="3HE  MHO"; $resn_exch{" H2  MHS"}=" HN2 MHS";
$resn_exch{" HB2 MHS"}="1HB  MHS"; $resn_exch{" HB3 MHS"}="2HB  MHS"; $resn_exch{" HM1 MHS"}="1HM  MHS";
$resn_exch{" HM2 MHS"}="2HM  MHS"; $resn_exch{" HM3 MHS"}="3HM  MHS"; $resn_exch{" HN1 MHZ"}="1HN  MHZ";
$resn_exch{" HN2 MHZ"}="2HN  MHZ"; $resn_exch{"HNA1 MHZ"}="1HNA MHZ"; $resn_exch{" HB1 MHZ"}="1HB  MHZ";
$resn_exch{" HB2 MHZ"}="2HB  MHZ"; $resn_exch{"HCC1 MHZ"}="1HCC MHZ"; $resn_exch{"HCC2 MHZ"}="2HCC MHZ";
$resn_exch{" HG1 MHZ"}="1HG  MHZ"; $resn_exch{" HG2 MHZ"}="2HG  MHZ"; $resn_exch{" HE1 MHZ"}="1HE  MHZ";
$resn_exch{" HE2 MHZ"}="2HE  MHZ"; $resn_exch{" HE3 MHZ"}="3HE  MHZ"; $resn_exch{"H5'1 MHZ"}="1H5* MHZ";
$resn_exch{"H5'2 MHZ"}="2H5* MHZ"; $resn_exch{" H4' MHZ"}=" H4* MHZ"; $resn_exch{" H1' MHZ"}=" H1* MHZ";
$resn_exch{"HN61 MHZ"}="1HN6 MHZ"; $resn_exch{"HN62 MHZ"}="2HN6 MHZ"; $resn_exch{" H2' MHZ"}=" H2* MHZ";
$resn_exch{"HO2' MHZ"}="*HO2 MHZ"; $resn_exch{" H3' MHZ"}=" H3* MHZ"; $resn_exch{"HO3' MHZ"}="*HO3 MHZ";
$resn_exch{"H121 MI2"}="1H12 MI2"; $resn_exch{"H122 MI2"}="2H12 MI2"; $resn_exch{"H123 MI2"}="3H12 MI2";
$resn_exch{"H191 MI2"}="1H19 MI2"; $resn_exch{"H192 MI2"}="2H19 MI2"; $resn_exch{"HOP3 MIA"}="3HOP MIA";
$resn_exch{"HOP2 MIA"}="2HOP MIA"; $resn_exch{" H5' MIA"}="1H5* MIA"; $resn_exch{"H5'' MIA"}="2H5* MIA";
$resn_exch{" H4' MIA"}=" H4* MIA"; $resn_exch{" H3' MIA"}=" H3* MIA"; $resn_exch{"HO3' MIA"}="H3T  MIA";
$resn_exch{" H2' MIA"}=" H2* MIA"; $resn_exch{"HO2' MIA"}="2HO* MIA"; $resn_exch{" H1' MIA"}=" H1* MIA";
$resn_exch{"H111 MIA"}="1H11 MIA"; $resn_exch{"H112 MIA"}="2H11 MIA"; $resn_exch{"H113 MIA"}="3H11 MIA";
$resn_exch{"H121 MIA"}="1H12 MIA"; $resn_exch{"H122 MIA"}="2H12 MIA"; $resn_exch{"H131 MIA"}="1H13 MIA";
$resn_exch{"H132 MIA"}="2H13 MIA"; $resn_exch{"H151 MIA"}="1H15 MIA"; $resn_exch{"H152 MIA"}="2H15 MIA";
$resn_exch{"H153 MIA"}="3H15 MIA"; $resn_exch{"H161 MIA"}="1H16 MIA"; $resn_exch{"H162 MIA"}="2H16 MIA";
$resn_exch{"H163 MIA"}="3H16 MIA"; $resn_exch{"HM21 MIC"}="1HM2 MIC"; $resn_exch{"HM22 MIC"}="2HM2 MIC";
$resn_exch{"HM23 MIC"}="3HM2 MIC"; $resn_exch{" H41 MIC"}="1H4  MIC"; $resn_exch{" H42 MIC"}="2H4  MIC";
$resn_exch{" H21 MID"}="1H2  MID"; $resn_exch{" H22 MID"}="2H2  MID"; $resn_exch{"H31A MID"}="1H3  MID";
$resn_exch{"H32A MID"}="2H3  MID"; $resn_exch{" H41 MID"}="1H4  MID"; $resn_exch{" H42 MID"}="2H4  MID";
$resn_exch{" H51 MID"}="1H5  MID"; $resn_exch{" H52 MID"}="2H5  MID"; $resn_exch{" H61 MID"}="1H6  MID";
$resn_exch{" H62 MID"}="2H6  MID"; $resn_exch{"H111 MID"}="1H11 MID"; $resn_exch{"H112 MID"}="2H11 MID";
$resn_exch{"H191 MID"}="1H19 MID"; $resn_exch{"H192 MID"}="2H19 MID"; $resn_exch{"H371 MID"}="1H37 MID";
$resn_exch{"H372 MID"}="2H37 MID"; $resn_exch{"HBC1 MIK"}="1HBC MIK"; $resn_exch{"HBC2 MIK"}="2HBC MIK";
$resn_exch{"HGC1 MIK"}="1HGC MIK"; $resn_exch{"HGC2 MIK"}="2HGC MIK"; $resn_exch{"HDC1 MIK"}="1HDC MIK";
$resn_exch{"HDC2 MIK"}="2HDC MIK"; $resn_exch{"HEC1 MIK"}="1HEC MIK"; $resn_exch{"HEC2 MIK"}="2HEC MIK";
$resn_exch{"H2A1 MIL"}="1H2A MIL"; $resn_exch{"H2A2 MIL"}="2H2A MIL"; $resn_exch{"H2A3 MIL"}="3H2A MIL";
$resn_exch{"HIB1 MIM"}="1HIB MIM"; $resn_exch{"HIB2 MIM"}="2HIB MIM"; $resn_exch{"HIB3 MIM"}="3HIB MIM";
$resn_exch{"H9B1 MIM"}="1H9B MIM"; $resn_exch{"H9B2 MIM"}="2H9B MIM"; $resn_exch{"H8B1 MIM"}="1H8B MIM";
$resn_exch{"H8B2 MIM"}="2H8B MIM"; $resn_exch{"H7B1 MIM"}="1H7B MIM"; $resn_exch{"H7B2 MIM"}="2H7B MIM";
$resn_exch{"H6B1 MIM"}="1H6B MIM"; $resn_exch{"H6B2 MIM"}="2H6B MIM"; $resn_exch{"HAB1 MIM"}="1HAB MIM";
$resn_exch{"HAB2 MIM"}="2HAB MIM"; $resn_exch{"HBS1 MIM"}="1HBS MIM"; $resn_exch{"HBS2 MIM"}="2HBS MIM";
$resn_exch{"HOGS MIM"}="SHOG MIM"; $resn_exch{"HBK1 MIM"}="1HBK MIM"; $resn_exch{"HBK2 MIM"}="2HBK MIM";
$resn_exch{"HGK1 MIM"}="1HGK MIM"; $resn_exch{"HGK2 MIM"}="2HGK MIM"; $resn_exch{"HDK1 MIM"}="1HDK MIM";
$resn_exch{"HDK2 MIM"}="2HDK MIM"; $resn_exch{"HEK1 MIM"}="1HEK MIM"; $resn_exch{"HEK2 MIM"}="2HEK MIM";
$resn_exch{"HNZ1 MIM"}="1HNZ MIM"; $resn_exch{"HNZ2 MIM"}="2HNZ MIM"; $resn_exch{"HAY1 MIM"}="1HAY MIM";
$resn_exch{"HAY2 MIM"}="2HAY MIM"; $resn_exch{"HBY1 MIM"}="1HBY MIM"; $resn_exch{"HBY2 MIM"}="2HBY MIM";
$resn_exch{"HKY1 MIM"}="1HKY MIM"; $resn_exch{"HKY2 MIM"}="2HKY MIM"; $resn_exch{"HHY1 MIM"}="1HHY MIM";
$resn_exch{"HHY2 MIM"}="2HHY MIM"; $resn_exch{"HZY1 MIM"}="1HZY MIM"; $resn_exch{"HZY2 MIM"}="2HZY MIM";
$resn_exch{"HEY1 MIM"}="1HEY MIM"; $resn_exch{"HEY2 MIM"}="2HEY MIM"; $resn_exch{"HDY1 MIM"}="1HDY MIM";
$resn_exch{"HDY2 MIM"}="2HDY MIM"; $resn_exch{"HN11 MIN"}="1HN1 MIN"; $resn_exch{"HN12 MIN"}="2HN1 MIN";
$resn_exch{"H21A MIN"}="1H2  MIN"; $resn_exch{"H22A MIN"}="2H2  MIN"; $resn_exch{" H31 MIN"}="1H3  MIN";
$resn_exch{" H32 MIN"}="2H3  MIN"; $resn_exch{" H51 MIN"}="1H5  MIN"; $resn_exch{" H52 MIN"}="2H5  MIN";
$resn_exch{" H61 MIN"}="1H6  MIN"; $resn_exch{" H62 MIN"}="2H6  MIN"; $resn_exch{" H71 MIN"}="1H7  MIN";
$resn_exch{" H72 MIN"}="2H7  MIN"; $resn_exch{"H101 MIN"}="1H10 MIN"; $resn_exch{"H102 MIN"}="2H10 MIN";
$resn_exch{"H111 MIN"}="1H11 MIN"; $resn_exch{"H112 MIN"}="2H11 MIN"; $resn_exch{"H121 MIN"}="1H12 MIN";
$resn_exch{"H122 MIN"}="2H12 MIN"; $resn_exch{"H141 MIN"}="1H14 MIN"; $resn_exch{"H142 MIN"}="2H14 MIN";
$resn_exch{"H143 MIN"}="3H14 MIN"; $resn_exch{"H161 MIN"}="1H16 MIN"; $resn_exch{"H162 MIN"}="2H16 MIN";
$resn_exch{"H1B1 MIP"}="1H1B MIP"; $resn_exch{"H1B2 MIP"}="2H1B MIP"; $resn_exch{"H1B3 MIP"}="3H1B MIP";
$resn_exch{"H3B1 MIP"}="1H3B MIP"; $resn_exch{"H3B2 MIP"}="2H3B MIP"; $resn_exch{"H3B3 MIP"}="3H3B MIP";
$resn_exch{" H   MIS"}="1HN  MIS"; $resn_exch{" H2  MIS"}="2HN  MIS"; $resn_exch{" HB2 MIS"}="1HB  MIS";
$resn_exch{" HB3 MIS"}="2HB  MIS"; $resn_exch{"HOP1 MIS"}="1HOP MIS"; $resn_exch{" H21 MIS"}="1H2  MIS";
$resn_exch{" H22 MIS"}="2H2  MIS"; $resn_exch{" H23 MIS"}="3H2  MIS"; $resn_exch{" H31 MIS"}="1H3  MIS";
$resn_exch{" H32 MIS"}="2H3  MIS"; $resn_exch{" H33 MIS"}="3H3  MIS"; $resn_exch{" H61 MIT"}="1H6  MIT";
$resn_exch{" H62 MIT"}="2H6  MIT"; $resn_exch{" H81 MIT"}="1H8  MIT"; $resn_exch{" H82 MIT"}="2H8  MIT";
$resn_exch{"H161 MIT"}="1H16 MIT"; $resn_exch{"H162 MIT"}="2H16 MIT"; $resn_exch{"H171 MIT"}="1H17 MIT";
$resn_exch{"H172 MIT"}="2H17 MIT"; $resn_exch{"H181 MIT"}="1H18 MIT"; $resn_exch{"H182 MIT"}="2H18 MIT";
$resn_exch{"HN19 MIT"}="9HN1 MIT"; $resn_exch{"HN21 MIT"}="1HN2 MIT"; $resn_exch{"HN22 MIT"}="2HN2 MIT";
$resn_exch{"H231 MIT"}="1H23 MIT"; $resn_exch{"H232 MIT"}="2H23 MIT"; $resn_exch{"H233 MIT"}="3H23 MIT";
$resn_exch{"H281 MIT"}="1H28 MIT"; $resn_exch{"H282 MIT"}="2H28 MIT"; $resn_exch{"H301 MIT"}="1H30 MIT";
$resn_exch{"H302 MIT"}="2H30 MIT"; $resn_exch{"H311 MIT"}="1H31 MIT"; $resn_exch{"H312 MIT"}="2H31 MIT";
$resn_exch{"H351 MIT"}="1H35 MIT"; $resn_exch{"H352 MIT"}="2H35 MIT"; $resn_exch{"H353 MIT"}="3H35 MIT";
$resn_exch{"H271 MIU"}="1H27 MIU"; $resn_exch{"H272 MIU"}="2H27 MIU"; $resn_exch{"H281 MIU"}="1H28 MIU";
$resn_exch{"H282 MIU"}="2H28 MIU"; $resn_exch{"H301 MIU"}="1H30 MIU"; $resn_exch{"H302 MIU"}="2H30 MIU";
$resn_exch{"H311 MIU"}="1H31 MIU"; $resn_exch{"H312 MIU"}="2H31 MIU"; $resn_exch{"H321 MIU"}="1H32 MIU";
$resn_exch{"H322 MIU"}="2H32 MIU"; $resn_exch{"H161 MIU"}="1H16 MIU"; $resn_exch{"H162 MIU"}="2H16 MIU";
$resn_exch{"H171 MIU"}="1H17 MIU"; $resn_exch{"H172 MIU"}="2H17 MIU"; $resn_exch{"H181 MIU"}="1H18 MIU";
$resn_exch{"H182 MIU"}="2H18 MIU"; $resn_exch{"H211 MIU"}="1H21 MIU"; $resn_exch{"H212 MIU"}="2H21 MIU";
$resn_exch{"H221 MIU"}="1H22 MIU"; $resn_exch{"H222 MIU"}="2H22 MIU"; $resn_exch{"H331 MIU"}="1H33 MIU";
$resn_exch{"H332 MIU"}="2H33 MIU"; $resn_exch{"H8C1 MIU"}="1H8C MIU"; $resn_exch{"H8C2 MIU"}="2H8C MIU";
$resn_exch{"H9N1 MIU"}="1H9N MIU"; $resn_exch{"H9N2 MIU"}="2H9N MIU"; $resn_exch{"HAK1 MIX"}="1HAK MIX";
$resn_exch{"HAK2 MIX"}="2HAK MIX"; $resn_exch{"HAM1 MIX"}="1HAM MIX"; $resn_exch{"HAM2 MIX"}="2HAM MIX";
$resn_exch{"HAO1 MIX"}="1HAO MIX"; $resn_exch{"HAO2 MIX"}="2HAO MIX"; $resn_exch{"HAQ1 MIX"}="1HAQ MIX";
$resn_exch{"HAQ2 MIX"}="2HAQ MIX"; $resn_exch{"HAR1 MIX"}="1HAR MIX"; $resn_exch{"HAR2 MIX"}="2HAR MIX";
$resn_exch{"HAP1 MIX"}="1HAP MIX"; $resn_exch{"HAP2 MIX"}="2HAP MIX"; $resn_exch{"HAN1 MIX"}="1HAN MIX";
$resn_exch{"HAN2 MIX"}="2HAN MIX"; $resn_exch{"HAL1 MIX"}="1HAL MIX"; $resn_exch{"HAL2 MIX"}="2HAL MIX";
$resn_exch{"HN21 MIY"}="1HN2 MIY"; $resn_exch{"HN22 MIY"}="2HN2 MIY"; $resn_exch{"H201 MIY"}="1H20 MIY";
$resn_exch{"H202 MIY"}="2H20 MIY"; $resn_exch{"H203 MIY"}="3H20 MIY"; $resn_exch{"H191 MIY"}="1H19 MIY";
$resn_exch{"H192 MIY"}="2H19 MIY"; $resn_exch{"H193 MIY"}="3H19 MIY"; $resn_exch{" H61 MIY"}="1H6  MIY";
$resn_exch{" H62 MIY"}="2H6  MIY"; $resn_exch{" H81 MIY"}="1H8  MIY"; $resn_exch{" H82 MIY"}="2H8  MIY";
$resn_exch{"HN71 MIY"}="1HN7 MIY"; $resn_exch{"HN72 MIY"}="2HN7 MIY"; $resn_exch{"HN73 MIY"}="3HN7 MIY";
$resn_exch{"H711 MIY"}="1H71 MIY"; $resn_exch{"H712 MIY"}="2H71 MIY"; $resn_exch{"H713 MIY"}="3H71 MIY";
$resn_exch{" H11 MJI"}="1H1  MJI"; $resn_exch{" H12 MJI"}="2H1  MJI"; $resn_exch{" H21 MJI"}="1H2  MJI";
$resn_exch{" H31 MJI"}="1H3  MJI"; $resn_exch{" H32 MJI"}="2H3  MJI"; $resn_exch{"H311 MJI"}="1H31 MJI";
$resn_exch{"H312 MJI"}="2H31 MJI"; $resn_exch{"HO21 MJI"}="1HO2 MJI"; $resn_exch{"H2P1 MJI"}="1H2P MJI";
$resn_exch{"H2P2 MJI"}="2H2P MJI"; $resn_exch{"H2P3 MJI"}="3H2P MJI"; $resn_exch{"H111 MJI"}="1H11 MJI";
$resn_exch{"H112 MJI"}="2H11 MJI"; $resn_exch{"H121 MJI"}="1H12 MJI"; $resn_exch{"H122 MJI"}="2H12 MJI";
$resn_exch{"H131 MJI"}="1H13 MJI"; $resn_exch{"H132 MJI"}="2H13 MJI"; $resn_exch{"H141 MJI"}="1H14 MJI";
$resn_exch{"H142 MJI"}="2H14 MJI"; $resn_exch{"H151 MJI"}="1H15 MJI"; $resn_exch{"H152 MJI"}="2H15 MJI";
$resn_exch{"H161 MJI"}="1H16 MJI"; $resn_exch{"H162 MJI"}="2H16 MJI"; $resn_exch{"H171 MJI"}="1H17 MJI";
$resn_exch{"H172 MJI"}="2H17 MJI"; $resn_exch{"H181 MJI"}="1H18 MJI"; $resn_exch{"H182 MJI"}="2H18 MJI";
$resn_exch{"H191 MJI"}="1H19 MJI"; $resn_exch{"H192 MJI"}="2H19 MJI"; $resn_exch{"H201 MJI"}="1H20 MJI";
$resn_exch{"H202 MJI"}="2H20 MJI"; $resn_exch{"H211 MJI"}="1H21 MJI"; $resn_exch{"H212 MJI"}="2H21 MJI";
$resn_exch{"H221 MJI"}="1H22 MJI"; $resn_exch{"H222 MJI"}="2H22 MJI"; $resn_exch{"H231 MJI"}="1H23 MJI";
$resn_exch{"H232 MJI"}="2H23 MJI"; $resn_exch{"H241 MJI"}="1H24 MJI"; $resn_exch{"H242 MJI"}="2H24 MJI";
$resn_exch{"H251 MJI"}="1H25 MJI"; $resn_exch{"H252 MJI"}="2H25 MJI"; $resn_exch{"H261 MJI"}="1H26 MJI";
$resn_exch{"H262 MJI"}="2H26 MJI"; $resn_exch{"H263 MJI"}="3H26 MJI"; $resn_exch{"H11A MK1"}="1H1  MK1";
$resn_exch{" H12 MK1"}="2H1  MK1"; $resn_exch{" H51 MK1"}="1H5  MK1"; $resn_exch{" H52 MK1"}="2H5  MK1";
$resn_exch{" H53 MK1"}="3H5  MK1"; $resn_exch{" H61 MK1"}="1H6  MK1"; $resn_exch{" H62 MK1"}="2H6  MK1";
$resn_exch{" H63 MK1"}="3H6  MK1"; $resn_exch{" H71 MK1"}="1H7  MK1"; $resn_exch{" H72 MK1"}="2H7  MK1";
$resn_exch{" H73 MK1"}="3H7  MK1"; $resn_exch{" H81 MK1"}="1H8  MK1"; $resn_exch{" H82 MK1"}="2H8  MK1";
$resn_exch{" H91 MK1"}="1H9  MK1"; $resn_exch{" H92 MK1"}="2H9  MK1"; $resn_exch{"H101 MK1"}="1H10 MK1";
$resn_exch{"H102 MK1"}="2H10 MK1"; $resn_exch{"H121 MK1"}="1H12 MK1"; $resn_exch{"H122 MK1"}="2H12 MK1";
$resn_exch{"H141 MK1"}="1H14 MK1"; $resn_exch{"H142 MK1"}="2H14 MK1"; $resn_exch{"H241 MK1"}="1H24 MK1";
$resn_exch{"H242 MK1"}="2H24 MK1"; $resn_exch{"H311 MK1"}="1H31 MK1"; $resn_exch{"H312 MK1"}="2H31 MK1";
$resn_exch{"H131 MKC"}="1H13 MKC"; $resn_exch{"H132 MKC"}="2H13 MKC"; $resn_exch{"H151 MKC"}="1H15 MKC";
$resn_exch{"H152 MKC"}="2H15 MKC"; $resn_exch{"H153 MKC"}="3H15 MKC"; $resn_exch{"H161 MKC"}="1H16 MKC";
$resn_exch{"H162 MKC"}="2H16 MKC"; $resn_exch{"H163 MKC"}="3H16 MKC"; $resn_exch{"H171 MKC"}="1H17 MKC";
$resn_exch{"H172 MKC"}="2H17 MKC"; $resn_exch{"H181 MKC"}="1H18 MKC"; $resn_exch{"H182 MKC"}="2H18 MKC";
$resn_exch{"H191 MKC"}="1H19 MKC"; $resn_exch{"H192 MKC"}="2H19 MKC"; $resn_exch{"H193 MKC"}="3H19 MKC";
$resn_exch{" HN1 MKE"}="1HN  MKE"; $resn_exch{" HN2 MKE"}="2HN  MKE"; $resn_exch{" HB1 MKE"}="1HB  MKE";
$resn_exch{" HB2 MKE"}="2HB  MKE"; $resn_exch{" HG1 MKE"}="1HG  MKE"; $resn_exch{" HG2 MKE"}="2HG  MKE";
$resn_exch{"HMK1 MKE"}="1HMK MKE"; $resn_exch{"HMK2 MKE"}="2HMK MKE"; $resn_exch{"HMK3 MKE"}="3HMK MKE";
$resn_exch{"H141 MKK"}="1H14 MKK"; $resn_exch{"H142 MKK"}="2H14 MKK"; $resn_exch{" H91 MKK"}="1H9  MKK";
$resn_exch{" H92 MKK"}="2H9  MKK"; $resn_exch{" H93 MKK"}="3H9  MKK"; $resn_exch{"H101 MKK"}="1H10 MKK";
$resn_exch{"H102 MKK"}="2H10 MKK"; $resn_exch{"H103 MKK"}="3H10 MKK"; $resn_exch{" H71 MKK"}="1H7  MKK";
$resn_exch{" H72 MKK"}="2H7  MKK"; $resn_exch{" H81 MKK"}="1H8  MKK"; $resn_exch{" H82 MKK"}="2H8  MKK";
$resn_exch{"H111 MKK"}="1H11 MKK"; $resn_exch{"H112 MKK"}="2H11 MKK"; $resn_exch{"H121 MKK"}="1H12 MKK";
$resn_exch{"H122 MKK"}="2H12 MKK"; $resn_exch{"H151 MKK"}="1H15 MKK"; $resn_exch{"H152 MKK"}="2H15 MKK";
$resn_exch{"H161 MKK"}="1H16 MKK"; $resn_exch{"H162 MKK"}="2H16 MKK"; $resn_exch{"H181 MKK"}="1H18 MKK";
$resn_exch{"H182 MKK"}="2H18 MKK"; $resn_exch{"H261 MKK"}="1H26 MKK"; $resn_exch{"H262 MKK"}="2H26 MKK";
$resn_exch{"H263 MKK"}="3H26 MKK"; $resn_exch{" H91 MKR"}="1H9  MKR"; $resn_exch{" H92 MKR"}="2H9  MKR";
$resn_exch{" H93 MKR"}="3H9  MKR"; $resn_exch{" H71 MKR"}="1H7  MKR"; $resn_exch{" H72 MKR"}="2H7  MKR";
$resn_exch{"H101 MKR"}="1H10 MKR"; $resn_exch{"H102 MKR"}="2H10 MKR"; $resn_exch{"H111 MKR"}="1H11 MKR";
$resn_exch{"H112 MKR"}="2H11 MKR"; $resn_exch{"H121 MKR"}="1H12 MKR"; $resn_exch{"H122 MKR"}="2H12 MKR";
$resn_exch{"H161 MKR"}="1H16 MKR"; $resn_exch{"H162 MKR"}="2H16 MKR"; $resn_exch{"H171 MKR"}="1H17 MKR";
$resn_exch{"H172 MKR"}="2H17 MKR"; $resn_exch{"H191 MKR"}="1H19 MKR"; $resn_exch{"H192 MKR"}="2H19 MKR";
$resn_exch{"H211 MKR"}="1H21 MKR"; $resn_exch{"H212 MKR"}="2H21 MKR"; $resn_exch{"H241 MKR"}="1H24 MKR";
$resn_exch{"H242 MKR"}="2H24 MKR"; $resn_exch{"H243 MKR"}="3H24 MKR"; $resn_exch{" H11 ML1"}="1H1  ML1";
$resn_exch{"H12A ML1"}="2H1  ML1"; $resn_exch{" H31 ML1"}="1H3  ML1"; $resn_exch{" H32 ML1"}="2H3  ML1";
$resn_exch{" H33 ML1"}="3H3  ML1"; $resn_exch{" H51 ML1"}="1H5  ML1"; $resn_exch{" H52 ML1"}="2H5  ML1";
$resn_exch{" H53 ML1"}="3H5  ML1"; $resn_exch{" H61 ML1"}="1H6  ML1"; $resn_exch{" H62 ML1"}="2H6  ML1";
$resn_exch{"H111 ML1"}="1H11 ML1"; $resn_exch{"H112 ML1"}="2H11 ML1"; $resn_exch{"HC21 MLA"}="1HC2 MLA";
$resn_exch{"HC22 MLA"}="2HC2 MLA"; $resn_exch{"HN61 MLC"}="1HN6 MLC"; $resn_exch{"HN62 MLC"}="2HN6 MLC";
$resn_exch{"H5'1 MLC"}="1H5' MLC"; $resn_exch{"H5'2 MLC"}="2H5' MLC"; $resn_exch{"HPB1 MLC"}="1HPB MLC";
$resn_exch{"HPB2 MLC"}="2HPB MLC"; $resn_exch{"HP91 MLC"}="1HP9 MLC"; $resn_exch{"HP92 MLC"}="2HP9 MLC";
$resn_exch{"HP93 MLC"}="3HP9 MLC"; $resn_exch{"HP81 MLC"}="1HP8 MLC"; $resn_exch{"HP82 MLC"}="2HP8 MLC";
$resn_exch{"HP83 MLC"}="3HP8 MLC"; $resn_exch{"HP51 MLC"}="1HP5 MLC"; $resn_exch{"HP52 MLC"}="2HP5 MLC";
$resn_exch{"HP41 MLC"}="1HP4 MLC"; $resn_exch{"HP42 MLC"}="2HP4 MLC"; $resn_exch{"HP21 MLC"}="1HP2 MLC";
$resn_exch{"HP22 MLC"}="2HP2 MLC"; $resn_exch{"HP11 MLC"}="1HP1 MLC"; $resn_exch{"HP12 MLC"}="2HP1 MLC";
$resn_exch{"HM21 MLC"}="1HM2 MLC"; $resn_exch{"HM22 MLC"}="2HM2 MLC"; $resn_exch{"HAA1 MLD"}="1HAA MLD";
$resn_exch{"HAA2 MLD"}="2HAA MLD"; $resn_exch{"HAA3 MLD"}="3HAA MLD"; $resn_exch{" H61 MLD"}="1H6  MLD";
$resn_exch{" H62 MLD"}="2H6  MLD"; $resn_exch{"HA61 MLD"}="1HA6 MLD"; $resn_exch{"HA62 MLD"}="2HA6 MLD";
$resn_exch{"HAF1 MLD"}="1HAF MLD"; $resn_exch{"HAF2 MLD"}="2HAF MLD"; $resn_exch{"HAF3 MLD"}="3HAF MLD";
$resn_exch{"HAJ1 MLD"}="1HAJ MLD"; $resn_exch{"HAJ2 MLD"}="2HAJ MLD"; $resn_exch{"HAJ3 MLD"}="3HAJ MLD";
$resn_exch{" HB1 MLD"}="1HB  MLD"; $resn_exch{" HB2 MLD"}="2HB  MLD"; $resn_exch{" HB3 MLD"}="3HB  MLD";
$resn_exch{"HBB1 MLD"}="1HBB MLD"; $resn_exch{"HBB2 MLD"}="2HBB MLD"; $resn_exch{"HBC1 MLD"}="1HBC MLD";
$resn_exch{"HBC2 MLD"}="2HBC MLD"; $resn_exch{"HBL1 MLD"}="1HBL MLD"; $resn_exch{"HBL2 MLD"}="2HBL MLD";
$resn_exch{"HBL3 MLD"}="3HBL MLD"; $resn_exch{"HBP1 MLD"}="1HBP MLD"; $resn_exch{"HBP2 MLD"}="2HBP MLD";
$resn_exch{"HBQ1 MLD"}="1HBQ MLD"; $resn_exch{"HBQ2 MLD"}="2HBQ MLD"; $resn_exch{"HBR1 MLD"}="1HBR MLD";
$resn_exch{"HBR2 MLD"}="2HBR MLD"; $resn_exch{"HBW1 MLD"}="1HBW MLD"; $resn_exch{"HBW2 MLD"}="2HBW MLD";
$resn_exch{" H   MLE"}=" HN  MLE"; $resn_exch{" HN1 MLE"}="1HN  MLE"; $resn_exch{" HN2 MLE"}="2HN  MLE";
$resn_exch{" HN3 MLE"}="3HN  MLE"; $resn_exch{" HB2 MLE"}="1HB  MLE"; $resn_exch{" HB3 MLE"}="2HB  MLE";
$resn_exch{"HD11 MLE"}="1HD1 MLE"; $resn_exch{"HD12 MLE"}="2HD1 MLE"; $resn_exch{"HD13 MLE"}="3HD1 MLE";
$resn_exch{"HD21 MLE"}="1HD2 MLE"; $resn_exch{"HD22 MLE"}="2HD2 MLE"; $resn_exch{"HD23 MLE"}="3HD2 MLE";
$resn_exch{"H111 MLG"}="1H11 MLG"; $resn_exch{"H112 MLG"}="2H11 MLG"; $resn_exch{"H101 MLG"}="1H10 MLG";
$resn_exch{"H102 MLG"}="2H10 MLG"; $resn_exch{"H121 MLG"}="1H12 MLG"; $resn_exch{"H122 MLG"}="2H12 MLG";
$resn_exch{"H171 MLG"}="1H17 MLG"; $resn_exch{"H172 MLG"}="2H17 MLG"; $resn_exch{"H173 MLG"}="3H17 MLG";
$resn_exch{"H141 MLG"}="1H14 MLG"; $resn_exch{"H142 MLG"}="2H14 MLG"; $resn_exch{" H11 MLI"}="1H1  MLI";
$resn_exch{" H12 MLI"}="2H1  MLI"; $resn_exch{"H9C1 MLK"}="1H9C MLK"; $resn_exch{"H9C2 MLK"}="2H9C MLK";
$resn_exch{"H121 MLK"}="1H12 MLK"; $resn_exch{"H122 MLK"}="2H12 MLK"; $resn_exch{"H123 MLK"}="3H12 MLK";
$resn_exch{"H151 MLK"}="1H15 MLK"; $resn_exch{"H152 MLK"}="2H15 MLK"; $resn_exch{"H211 MLK"}="1H21 MLK";
$resn_exch{"H212 MLK"}="2H21 MLK"; $resn_exch{"H201 MLK"}="1H20 MLK"; $resn_exch{"H202 MLK"}="2H20 MLK";
$resn_exch{"H221 MLK"}="1H22 MLK"; $resn_exch{"H222 MLK"}="2H22 MLK"; $resn_exch{"H223 MLK"}="3H22 MLK";
$resn_exch{"H241 MLK"}="1H24 MLK"; $resn_exch{"H242 MLK"}="2H24 MLK"; $resn_exch{"H251 MLK"}="1H25 MLK";
$resn_exch{"H252 MLK"}="2H25 MLK"; $resn_exch{"H253 MLK"}="3H25 MLK"; $resn_exch{"H231 MLK"}="1H23 MLK";
$resn_exch{"H232 MLK"}="2H23 MLK"; $resn_exch{"H331 MLK"}="1H33 MLK"; $resn_exch{"H332 MLK"}="2H33 MLK";
$resn_exch{"H371 MLK"}="1H37 MLK"; $resn_exch{"H372 MLK"}="2H37 MLK"; $resn_exch{"H373 MLK"}="3H37 MLK";
$resn_exch{"H361 MLK"}="1H36 MLK"; $resn_exch{"H362 MLK"}="2H36 MLK"; $resn_exch{"H391 MLK"}="1H39 MLK";
$resn_exch{"H392 MLK"}="2H39 MLK"; $resn_exch{"H393 MLK"}="3H39 MLK"; $resn_exch{"H291 MLK"}="1H29 MLK";
$resn_exch{"H292 MLK"}="2H29 MLK"; $resn_exch{"H293 MLK"}="3H29 MLK"; $resn_exch{" H   MLL"}="1HN  MLL";
$resn_exch{" H2  MLL"}="2HN  MLL"; $resn_exch{" HB2 MLL"}="1HB  MLL"; $resn_exch{" HB3 MLL"}="2HB  MLL";
$resn_exch{"HD21 MLL"}="1HD2 MLL"; $resn_exch{"HD22 MLL"}="2HD2 MLL"; $resn_exch{"HD23 MLL"}="3HD2 MLL";
$resn_exch{"HD11 MLL"}="1HD1 MLL"; $resn_exch{"HD12 MLL"}="2HD1 MLL"; $resn_exch{"HD13 MLL"}="3HD1 MLL";
$resn_exch{"H101 MLL"}="1H10 MLL"; $resn_exch{"H102 MLL"}="2H10 MLL"; $resn_exch{"H103 MLL"}="3H10 MLL";
$resn_exch{"H2C1 MLM"}="1H2C MLM"; $resn_exch{"H2C2 MLM"}="2H2C MLM"; $resn_exch{"H6N1 MLM"}="1H6N MLM";
$resn_exch{"H6N2 MLM"}="2H6N MLM"; $resn_exch{"HOP3 MLN"}="3HOP MLN"; $resn_exch{"HOP4 MLN"}="4HOP MLN";
$resn_exch{"HC51 MLN"}="1HC5 MLN"; $resn_exch{"HC71 MLN"}="1HC7 MLN"; $resn_exch{"HC72 MLN"}="2HC7 MLN";
$resn_exch{"HC81 MLN"}="1HC8 MLN"; $resn_exch{"HC82 MLN"}="2HC8 MLN"; $resn_exch{"HC91 MLN"}="1HC9 MLN";
$resn_exch{"HC92 MLN"}="2HC9 MLN"; $resn_exch{"H101 MLN"}="1H10 MLN"; $resn_exch{"H102 MLN"}="2H10 MLN";
$resn_exch{"H103 MLN"}="3H10 MLN"; $resn_exch{"H141 MLN"}="1H14 MLN"; $resn_exch{"H142 MLN"}="2H14 MLN";
$resn_exch{"H161 MLN"}="1H16 MLN"; $resn_exch{"H162 MLN"}="2H16 MLN"; $resn_exch{"H163 MLN"}="3H16 MLN";
$resn_exch{"H171 MLN"}="1H17 MLN"; $resn_exch{"H172 MLN"}="2H17 MLN"; $resn_exch{"H173 MLN"}="3H17 MLN";
$resn_exch{" H11 MLP"}="1H1  MLP"; $resn_exch{" H12 MLP"}="2H1  MLP"; $resn_exch{" H21 MLP"}="1H2  MLP";
$resn_exch{" H22 MLP"}="2H2  MLP"; $resn_exch{"HN41 MLP"}="1HN4 MLP"; $resn_exch{"HN42 MLP"}="2HN4 MLP";
$resn_exch{"H6A1 MLR"}="1H6A MLR"; $resn_exch{"H6A2 MLR"}="2H6A MLR"; $resn_exch{"HOA2 MLR"}="2HOA MLR";
$resn_exch{"HOA3 MLR"}="3HOA MLR"; $resn_exch{"HOA6 MLR"}="6HOA MLR"; $resn_exch{"H6B1 MLR"}="1H6B MLR";
$resn_exch{"H6B2 MLR"}="2H6B MLR"; $resn_exch{"HOB2 MLR"}="2HOB MLR"; $resn_exch{"HOB3 MLR"}="3HOB MLR";
$resn_exch{"HOB6 MLR"}="6HOB MLR"; $resn_exch{"H6C1 MLR"}="1H6C MLR"; $resn_exch{"H6C2 MLR"}="2H6C MLR";
$resn_exch{"HOC2 MLR"}="2HOC MLR"; $resn_exch{"HOC3 MLR"}="3HOC MLR"; $resn_exch{"HOC4 MLR"}="4HOC MLR";
$resn_exch{"HOC6 MLR"}="6HOC MLR"; $resn_exch{" H31 MLT"}="1H3  MLT"; $resn_exch{" H32 MLT"}="2H3  MLT";
$resn_exch{" H   MLY"}="1HN  MLY"; $resn_exch{" H2  MLY"}="2HN  MLY"; $resn_exch{" HB2 MLY"}="1HB  MLY";
$resn_exch{" HB3 MLY"}="2HB  MLY"; $resn_exch{" HG2 MLY"}="1HG  MLY"; $resn_exch{" HG3 MLY"}="2HG  MLY";
$resn_exch{" HD2 MLY"}="1HD  MLY"; $resn_exch{" HD3 MLY"}="2HD  MLY"; $resn_exch{" HE2 MLY"}="1HE  MLY";
$resn_exch{" HE3 MLY"}="2HE  MLY"; $resn_exch{"HH11 MLY"}="1HC1 MLY"; $resn_exch{"HH12 MLY"}="2HC1 MLY";
$resn_exch{"HH13 MLY"}="3HC1 MLY"; $resn_exch{"HH21 MLY"}="1HC2 MLY"; $resn_exch{"HH22 MLY"}="2HC2 MLY";
$resn_exch{"HH23 MLY"}="3HC2 MLY"; $resn_exch{" H   MLZ"}="1HN  MLZ"; $resn_exch{" H2  MLZ"}="2HN  MLZ";
$resn_exch{" HB2 MLZ"}="1HB  MLZ"; $resn_exch{" HB3 MLZ"}="2HB  MLZ"; $resn_exch{" HG2 MLZ"}="1HG  MLZ";
$resn_exch{" HG3 MLZ"}="2HG  MLZ"; $resn_exch{" HD2 MLZ"}="1HD  MLZ"; $resn_exch{" HD3 MLZ"}="2HD  MLZ";
$resn_exch{" HE2 MLZ"}="1HE  MLZ"; $resn_exch{" HE3 MLZ"}="2HE  MLZ"; $resn_exch{"HCM1 MLZ"}="1HCM MLZ";
$resn_exch{"HCM2 MLZ"}="2HCM MLZ"; $resn_exch{"HCM3 MLZ"}="3HCM MLZ"; $resn_exch{"H11A MM1"}="1H1  MM1";
$resn_exch{" H12 MM1"}="2H1  MM1"; $resn_exch{" H21 MM1"}="1H2  MM1"; $resn_exch{" H22 MM1"}="2H2  MM1";
$resn_exch{" H31 MM1"}="1H3  MM1"; $resn_exch{" H32 MM1"}="2H3  MM1"; $resn_exch{" H51 MM1"}="1H5  MM1";
$resn_exch{" H52 MM1"}="2H5  MM1"; $resn_exch{" H61 MM1"}="1H6  MM1"; $resn_exch{" H62 MM1"}="2H6  MM1";
$resn_exch{" H81 MM1"}="1H8  MM1"; $resn_exch{" H82 MM1"}="2H8  MM1"; $resn_exch{" H91 MM1"}="1H9  MM1";
$resn_exch{" H92 MM1"}="2H9  MM1"; $resn_exch{"H101 MM1"}="1H10 MM1"; $resn_exch{"H102 MM1"}="2H10 MM1";
$resn_exch{"H121 MM1"}="1H12 MM1"; $resn_exch{"H122 MM1"}="2H12 MM1"; $resn_exch{"H131 MM1"}="1H13 MM1";
$resn_exch{"H132 MM1"}="2H13 MM1"; $resn_exch{"H11A MM2"}="1H1  MM2"; $resn_exch{" H12 MM2"}="2H1  MM2";
$resn_exch{"H21A MM2"}="1H2  MM2"; $resn_exch{" H22 MM2"}="2H2  MM2"; $resn_exch{" H31 MM2"}="1H3  MM2";
$resn_exch{" H32 MM2"}="2H3  MM2"; $resn_exch{" H51 MM2"}="1H5  MM2"; $resn_exch{" H52 MM2"}="2H5  MM2";
$resn_exch{" H61 MM2"}="1H6  MM2"; $resn_exch{" H62 MM2"}="2H6  MM2"; $resn_exch{" H81 MM2"}="1H8  MM2";
$resn_exch{" H82 MM2"}="2H8  MM2"; $resn_exch{" H91 MM2"}="1H9  MM2"; $resn_exch{" H92 MM2"}="2H9  MM2";
$resn_exch{"H101 MM2"}="1H10 MM2"; $resn_exch{"H102 MM2"}="2H10 MM2"; $resn_exch{"H121 MM2"}="1H12 MM2";
$resn_exch{"H122 MM2"}="2H12 MM2"; $resn_exch{"H131 MM2"}="1H13 MM2"; $resn_exch{"H132 MM2"}="2H13 MM2";
$resn_exch{" H41 MM2"}="1H4  MM2"; $resn_exch{" H42 MM2"}="2H4  MM2"; $resn_exch{" H71 MM2"}="1H7  MM2";
$resn_exch{" H72 MM2"}="2H7  MM2"; $resn_exch{"H111 MM2"}="1H11 MM2"; $resn_exch{"H112 MM2"}="2H11 MM2";
$resn_exch{"H141 MM2"}="1H14 MM2"; $resn_exch{"H142 MM2"}="2H14 MM2"; $resn_exch{"H151 MM2"}="1H15 MM2";
$resn_exch{"H152 MM2"}="2H15 MM2"; $resn_exch{"H161 MM2"}="1H16 MM2"; $resn_exch{"H162 MM2"}="2H16 MM2";
$resn_exch{"H171 MM2"}="1H17 MM2"; $resn_exch{"H172 MM2"}="2H17 MM2"; $resn_exch{"H181 MM2"}="1H18 MM2";
$resn_exch{"H182 MM2"}="2H18 MM2"; $resn_exch{"H191 MM2"}="1H19 MM2"; $resn_exch{"H192 MM2"}="2H19 MM2";
$resn_exch{"H201 MM2"}="1H20 MM2"; $resn_exch{"H202 MM2"}="2H20 MM2"; $resn_exch{"H271 MM2"}="1H27 MM2";
$resn_exch{"H272 MM2"}="2H27 MM2"; $resn_exch{"H281 MM2"}="1H28 MM2"; $resn_exch{"H282 MM2"}="2H28 MM2";
$resn_exch{" H21 MM3"}="1H2  MM3"; $resn_exch{" H31 MM3"}="1H3  MM3"; $resn_exch{" H51 MM3"}="1H5  MM3";
$resn_exch{" H61 MM3"}="1H6  MM3"; $resn_exch{" H81 MM3"}="1H8  MM3"; $resn_exch{" H82 MM3"}="2H8  MM3";
$resn_exch{" H83 MM3"}="3H8  MM3"; $resn_exch{"H131 MM3"}="1H13 MM3"; $resn_exch{"H161 MM3"}="1H16 MM3";
$resn_exch{"H162 MM3"}="2H16 MM3"; $resn_exch{"H171 MM3"}="1H17 MM3"; $resn_exch{"H172 MM3"}="2H17 MM3";
$resn_exch{"H181 MM3"}="1H18 MM3"; $resn_exch{"H182 MM3"}="2H18 MM3"; $resn_exch{"H191 MM3"}="1H19 MM3";
$resn_exch{"H192 MM3"}="2H19 MM3"; $resn_exch{"H193 MM3"}="3H19 MM3"; $resn_exch{"H201 MM3"}="1H20 MM3";
$resn_exch{"H202 MM3"}="2H20 MM3"; $resn_exch{"H203 MM3"}="3H20 MM3"; $resn_exch{"H171 MM5"}="1H17 MM5";
$resn_exch{"H172 MM5"}="2H17 MM5"; $resn_exch{"H181 MM5"}="1H18 MM5"; $resn_exch{"H182 MM5"}="2H18 MM5";
$resn_exch{"H191 MM5"}="1H19 MM5"; $resn_exch{"H192 MM5"}="2H19 MM5"; $resn_exch{"H201 MM5"}="1H20 MM5";
$resn_exch{"H202 MM5"}="2H20 MM5"; $resn_exch{"H211 MM5"}="1H21 MM5"; $resn_exch{"H212 MM5"}="2H21 MM5";
$resn_exch{"H221 MM5"}="1H22 MM5"; $resn_exch{"H222 MM5"}="2H22 MM5"; $resn_exch{"H231 MM5"}="1H23 MM5";
$resn_exch{"H232 MM5"}="2H23 MM5"; $resn_exch{"H241 MM5"}="1H24 MM5"; $resn_exch{"H242 MM5"}="2H24 MM5";
$resn_exch{"H251 MM5"}="1H25 MM5"; $resn_exch{"H252 MM5"}="2H25 MM5"; $resn_exch{"H261 MM5"}="1H26 MM5";
$resn_exch{"H262 MM5"}="2H26 MM5"; $resn_exch{"H271 MM5"}="1H27 MM5"; $resn_exch{"H272 MM5"}="2H27 MM5";
$resn_exch{"H281 MM5"}="1H28 MM5"; $resn_exch{"H282 MM5"}="2H28 MM5"; $resn_exch{" H51 MM5"}="1H5  MM5";
$resn_exch{" H52 MM5"}="2H5  MM5"; $resn_exch{" H61 MM5"}="1H6  MM5"; $resn_exch{" H62 MM5"}="2H6  MM5";
$resn_exch{" H81 MM5"}="1H8  MM5"; $resn_exch{" H82 MM5"}="2H8  MM5"; $resn_exch{" H91 MM5"}="1H9  MM5";
$resn_exch{" H92 MM5"}="2H9  MM5"; $resn_exch{"H101 MM5"}="1H10 MM5"; $resn_exch{"H102 MM5"}="2H10 MM5";
$resn_exch{"H121 MM5"}="1H12 MM5"; $resn_exch{"H122 MM5"}="2H12 MM5"; $resn_exch{"H131 MM5"}="1H13 MM5";
$resn_exch{"H132 MM5"}="2H13 MM5"; $resn_exch{" H11 MM5"}="1H1  MM5"; $resn_exch{" H12 MM5"}="2H1  MM5";
$resn_exch{" H21 MM5"}="1H2  MM5"; $resn_exch{" H22 MM5"}="2H2  MM5"; $resn_exch{" H31 MM5"}="1H3  MM5";
$resn_exch{" H32 MM5"}="2H3  MM5"; $resn_exch{"H131 MM6"}="1H13 MM6"; $resn_exch{"H132 MM6"}="2H13 MM6";
$resn_exch{" H11 MM6"}="1H1  MM6"; $resn_exch{" H12 MM6"}="2H1  MM6"; $resn_exch{"H121 MM6"}="1H12 MM6";
$resn_exch{"H122 MM6"}="2H12 MM6"; $resn_exch{"H101 MM6"}="1H10 MM6"; $resn_exch{"H102 MM6"}="2H10 MM6";
$resn_exch{" H91 MM6"}="1H9  MM6"; $resn_exch{" H92 MM6"}="2H9  MM6"; $resn_exch{" H81 MM6"}="1H8  MM6";
$resn_exch{" H82 MM6"}="2H8  MM6"; $resn_exch{" H61 MM6"}="1H6  MM6"; $resn_exch{" H62 MM6"}="2H6  MM6";
$resn_exch{" H51 MM6"}="1H5  MM6"; $resn_exch{" H52 MM6"}="2H5  MM6"; $resn_exch{" H31 MM6"}="1H3  MM6";
$resn_exch{" H32 MM6"}="2H3  MM6"; $resn_exch{" H21 MM6"}="1H2  MM6"; $resn_exch{" H22 MM6"}="2H2  MM6";
$resn_exch{" H61 MMA"}="1H6  MMA"; $resn_exch{" H62 MMA"}="2H6  MMA"; $resn_exch{" H71 MMA"}="1H7  MMA";
$resn_exch{" H72 MMA"}="2H7  MMA"; $resn_exch{" H73 MMA"}="3H7  MMA"; $resn_exch{"H11A MMB"}="1H1  MMB";
$resn_exch{"H12A MMB"}="2H1  MMB"; $resn_exch{"H131 MMB"}="1H13 MMB"; $resn_exch{"H132 MMB"}="2H13 MMB";
$resn_exch{"H261 MMB"}="1H26 MMB"; $resn_exch{"H262 MMB"}="2H26 MMB"; $resn_exch{"H281 MMB"}="1H28 MMB";
$resn_exch{"H282 MMB"}="2H28 MMB"; $resn_exch{"H283 MMB"}="3H28 MMB"; $resn_exch{"H291 MMB"}="1H29 MMB";
$resn_exch{"H292 MMB"}="2H29 MMB"; $resn_exch{"H321 MMB"}="1H32 MMB"; $resn_exch{"H322 MMB"}="2H32 MMB";
$resn_exch{"H323 MMB"}="3H32 MMB"; $resn_exch{" H51 MMB"}="1H5  MMB"; $resn_exch{" H52 MMB"}="2H5  MMB";
$resn_exch{" H53 MMB"}="3H5  MMB"; $resn_exch{" H1  MMC"}="1H   MMC"; $resn_exch{" H2  MMC"}="2H   MMC";
$resn_exch{" H3  MMC"}="3H   MMC"; $resn_exch{" HB2 MME"}="1HB  MME"; $resn_exch{" HB3 MME"}="2HB  MME";
$resn_exch{" HG2 MME"}="1HG  MME"; $resn_exch{" HG3 MME"}="2HG  MME"; $resn_exch{" HE1 MME"}="1HE  MME";
$resn_exch{" HE2 MME"}="2HE  MME"; $resn_exch{" HE3 MME"}="3HE  MME"; $resn_exch{" HM1 MME"}="1HM  MME";
$resn_exch{" HM2 MME"}="2HM  MME"; $resn_exch{" HM3 MME"}="3HM  MME"; $resn_exch{"H131 MMI"}="1H13 MMI";
$resn_exch{"H132 MMI"}="2H13 MMI"; $resn_exch{"H133 MMI"}="3H13 MMI"; $resn_exch{"H141 MMI"}="1H14 MMI";
$resn_exch{"H142 MMI"}="2H14 MMI"; $resn_exch{"H143 MMI"}="3H14 MMI"; $resn_exch{"H171 MMI"}="1H17 MMI";
$resn_exch{"H172 MMI"}="2H17 MMI"; $resn_exch{"H221 MMI"}="1H22 MMI"; $resn_exch{"H222 MMI"}="2H22 MMI";
$resn_exch{"H281 MMI"}="1H28 MMI"; $resn_exch{"H282 MMI"}="2H28 MMI"; $resn_exch{"H271 MMI"}="1H27 MMI";
$resn_exch{"H272 MMI"}="2H27 MMI"; $resn_exch{"H261 MMI"}="1H26 MMI"; $resn_exch{"H262 MMI"}="2H26 MMI";
$resn_exch{"H251 MMI"}="1H25 MMI"; $resn_exch{"H252 MMI"}="2H25 MMI"; $resn_exch{"H241 MMI"}="1H24 MMI";
$resn_exch{"H242 MMI"}="2H24 MMI"; $resn_exch{"H331 MMI"}="1H33 MMI"; $resn_exch{"H332 MMI"}="2H33 MMI";
$resn_exch{"H351 MMI"}="1H35 MMI"; $resn_exch{"H352 MMI"}="2H35 MMI"; $resn_exch{"H353 MMI"}="3H35 MMI";
$resn_exch{"H361 MMI"}="1H36 MMI"; $resn_exch{"H362 MMI"}="2H36 MMI"; $resn_exch{"H363 MMI"}="3H36 MMI";
$resn_exch{"H371 MMI"}="1H37 MMI"; $resn_exch{"H372 MMI"}="2H37 MMI"; $resn_exch{"H391 MMI"}="1H39 MMI";
$resn_exch{"H392 MMI"}="2H39 MMI"; $resn_exch{"H393 MMI"}="3H39 MMI"; $resn_exch{"H441 MMI"}="1H44 MMI";
$resn_exch{"H442 MMI"}="2H44 MMI"; $resn_exch{"H443 MMI"}="3H44 MMI"; $resn_exch{"H431 MMI"}="1H43 MMI";
$resn_exch{"H432 MMI"}="2H43 MMI"; $resn_exch{"H433 MMI"}="3H43 MMI"; $resn_exch{"H511 MMI"}="1H51 MMI";
$resn_exch{"H512 MMI"}="2H51 MMI"; $resn_exch{"H2A1 MMM"}="1H2A MMM"; $resn_exch{"H2A2 MMM"}="2H2A MMM";
$resn_exch{"H2A3 MMM"}="3H2A MMM"; $resn_exch{"H5A1 MMM"}="1H5A MMM"; $resn_exch{"H5A2 MMM"}="2H5A MMM";
$resn_exch{"HAB1 MMM"}="1HAB MMM"; $resn_exch{"HAB2 MMM"}="2HAB MMM"; $resn_exch{"HAB3 MMM"}="3HAB MMM";
$resn_exch{"HBA1 MMM"}="1HBA MMM"; $resn_exch{"HBA2 MMM"}="2HBA MMM"; $resn_exch{" H81 MMN"}="1H8  MMN";
$resn_exch{" H82 MMN"}="2H8  MMN"; $resn_exch{" H83 MMN"}="3H8  MMN"; $resn_exch{" H11 MMN"}="1H1  MMN";
$resn_exch{" H12 MMN"}="2H1  MMN"; $resn_exch{" H61 MMN"}="1H6  MMN"; $resn_exch{" H62 MMN"}="2H6  MMN";
$resn_exch{"HCB1 MMO"}="1HCB MMO"; $resn_exch{"HCB2 MMO"}="2HCB MMO"; $resn_exch{"HCG1 MMO"}="1HCG MMO";
$resn_exch{"HCG2 MMO"}="2HCG MMO"; $resn_exch{"HCD1 MMO"}="1HCD MMO"; $resn_exch{"HCD2 MMO"}="2HCD MMO";
$resn_exch{"HH21 MMO"}="1HH2 MMO"; $resn_exch{"HH22 MMO"}="2HH2 MMO"; $resn_exch{"HH11 MMO"}="1HH1 MMO";
$resn_exch{"HH12 MMO"}="2HH1 MMO"; $resn_exch{" HC1 MMO"}="1HC  MMO"; $resn_exch{" HC2 MMO"}="2HC  MMO";
$resn_exch{" HC3 MMO"}="3HC  MMO"; $resn_exch{" H11 MMP"}="1H1  MMP"; $resn_exch{" H12 MMP"}="2H1  MMP";
$resn_exch{" H13 MMP"}="3H1  MMP"; $resn_exch{"H161 MMP"}="1H16 MMP"; $resn_exch{"H162 MMP"}="2H16 MMP";
$resn_exch{"H163 MMP"}="3H16 MMP"; $resn_exch{"H171 MMP"}="1H17 MMP"; $resn_exch{"H172 MMP"}="2H17 MMP";
$resn_exch{"H181 MMP"}="1H18 MMP"; $resn_exch{"H182 MMP"}="2H18 MMP"; $resn_exch{"H183 MMP"}="3H18 MMP";
$resn_exch{"H261 MMP"}="1H26 MMP"; $resn_exch{"H262 MMP"}="2H26 MMP"; $resn_exch{"H263 MMP"}="3H26 MMP";
$resn_exch{"H271 MMP"}="1H27 MMP"; $resn_exch{"H272 MMP"}="2H27 MMP"; $resn_exch{"H281 MMP"}="1H28 MMP";
$resn_exch{"H282 MMP"}="2H28 MMP"; $resn_exch{"H283 MMP"}="3H28 MMP"; $resn_exch{"H361 MMP"}="1H36 MMP";
$resn_exch{"H362 MMP"}="2H36 MMP"; $resn_exch{"H363 MMP"}="3H36 MMP"; $resn_exch{"H371 MMP"}="1H37 MMP";
$resn_exch{"H372 MMP"}="2H37 MMP"; $resn_exch{"H381 MMP"}="1H38 MMP"; $resn_exch{"H382 MMP"}="2H38 MMP";
$resn_exch{"H461 MMP"}="1H46 MMP"; $resn_exch{"H462 MMP"}="2H46 MMP"; $resn_exch{"H463 MMP"}="3H46 MMP";
$resn_exch{"H471 MMP"}="1H47 MMP"; $resn_exch{"H472 MMP"}="2H47 MMP"; $resn_exch{"H481 MMP"}="1H48 MMP";
$resn_exch{"H482 MMP"}="2H48 MMP"; $resn_exch{"HC11 MMQ"}="1HC1 MMQ"; $resn_exch{"HC12 MMQ"}="2HC1 MMQ";
$resn_exch{" H41 MMR"}="1H4  MMR"; $resn_exch{" H42 MMR"}="2H4  MMR"; $resn_exch{" H61 MMR"}="1H6  MMR";
$resn_exch{" H62 MMR"}="2H6  MMR"; $resn_exch{" H71 MMR"}="1H7  MMR"; $resn_exch{" H72 MMR"}="2H7  MMR";
$resn_exch{" H73 MMR"}="3H7  MMR"; $resn_exch{" H91 MMR"}="1H9  MMR"; $resn_exch{" H92 MMR"}="2H9  MMR";
$resn_exch{" H93 MMR"}="3H9  MMR"; $resn_exch{"H121 MMR"}="1H12 MMR"; $resn_exch{"H122 MMR"}="2H12 MMR";
$resn_exch{"H123 MMR"}="3H12 MMR"; $resn_exch{"HC31 MMT"}="1HC3 MMT"; $resn_exch{"HC32 MMT"}="2HC3 MMT";
$resn_exch{"HC33 MMT"}="3HC3 MMT"; $resn_exch{"HNP1 MMT"}="1HNP MMT"; $resn_exch{"HNP2 MMT"}="2HNP MMT";
$resn_exch{"HNP3 MMT"}="3HNP MMT"; $resn_exch{" H5' MMT"}="1H5* MMT"; $resn_exch{"H5'' MMT"}="2H5* MMT";
$resn_exch{" H4' MMT"}=" H4* MMT"; $resn_exch{" H1' MMT"}=" H1* MMT"; $resn_exch{" H2' MMT"}="1H2* MMT";
$resn_exch{"H2'' MMT"}="2H2* MMT"; $resn_exch{" H3' MMT"}=" H3* MMT"; $resn_exch{"HO3' MMT"}="H3T  MMT";
$resn_exch{" H71 MMT"}="1H5M MMT"; $resn_exch{" H72 MMT"}="2H5M MMT"; $resn_exch{" H73 MMT"}="3H5M MMT";
$resn_exch{" H41 MMZ"}="1H4  MMZ"; $resn_exch{" H42 MMZ"}="2H4  MMZ"; $resn_exch{" H43 MMZ"}="3H4  MMZ";
$resn_exch{" H21 MN1"}="1H2  MN1"; $resn_exch{" H22 MN1"}="2H2  MN1"; $resn_exch{" H31 MN1"}="1H3  MN1";
$resn_exch{" H32 MN1"}="2H3  MN1"; $resn_exch{" H51 MN1"}="1H5  MN1"; $resn_exch{" H52 MN1"}="2H5  MN1";
$resn_exch{" H61 MN1"}="1H6  MN1"; $resn_exch{" H62 MN1"}="2H6  MN1"; $resn_exch{" H71 MN2"}="1H7  MN2";
$resn_exch{" H72 MN2"}="2H7  MN2"; $resn_exch{" H91 MN2"}="1H9  MN2"; $resn_exch{" H92 MN2"}="2H9  MN2";
$resn_exch{"H101 MN2"}="1H10 MN2"; $resn_exch{"H102 MN2"}="2H10 MN2"; $resn_exch{"H111 MN2"}="1H11 MN2";
$resn_exch{"H112 MN2"}="2H11 MN2"; $resn_exch{"HN21 MN2"}="1HN2 MN2"; $resn_exch{"HN22 MN2"}="2HN2 MN2";
$resn_exch{"HN23 MN2"}="3HN2 MN2"; $resn_exch{"HO11 MN5"}="1HO1 MN5"; $resn_exch{"HO12 MN5"}="2HO1 MN5";
$resn_exch{"HO21 MN5"}="1HO2 MN5"; $resn_exch{"HO22 MN5"}="2HO2 MN5"; $resn_exch{"HO31 MN5"}="1HO3 MN5";
$resn_exch{"HO32 MN5"}="2HO3 MN5"; $resn_exch{"HO41 MN5"}="1HO4 MN5"; $resn_exch{"HO42 MN5"}="2HO4 MN5";
$resn_exch{"HO51 MN5"}="1HO5 MN5"; $resn_exch{"HO52 MN5"}="2HO5 MN5"; $resn_exch{"HO11 MN6"}="1HO1 MN6";
$resn_exch{"HO12 MN6"}="2HO1 MN6"; $resn_exch{"HO21 MN6"}="1HO2 MN6"; $resn_exch{"HO22 MN6"}="2HO2 MN6";
$resn_exch{"HO31 MN6"}="1HO3 MN6"; $resn_exch{"HO32 MN6"}="2HO3 MN6"; $resn_exch{"HO41 MN6"}="1HO4 MN6";
$resn_exch{"HO42 MN6"}="2HO4 MN6"; $resn_exch{"HO51 MN6"}="1HO5 MN6"; $resn_exch{"HO52 MN6"}="2HO5 MN6";
$resn_exch{"HO61 MN6"}="1HO6 MN6"; $resn_exch{"HO62 MN6"}="2HO6 MN6"; $resn_exch{" H71 MN7"}="1H7  MN7";
$resn_exch{" H72 MN7"}="2H7  MN7"; $resn_exch{" H91 MN7"}="1H9  MN7"; $resn_exch{" H92 MN7"}="2H9  MN7";
$resn_exch{"H111 MN7"}="1H11 MN7"; $resn_exch{"H112 MN7"}="2H11 MN7"; $resn_exch{"H113 MN7"}="3H11 MN7";
$resn_exch{"H121 MN7"}="1H12 MN7"; $resn_exch{"H122 MN7"}="2H12 MN7"; $resn_exch{"H123 MN7"}="3H12 MN7";
$resn_exch{" H51 MN8"}="1H5  MN8"; $resn_exch{" H52 MN8"}="2H5  MN8"; $resn_exch{" H53 MN8"}="3H5  MN8";
$resn_exch{" H61 MN8"}="1H6  MN8"; $resn_exch{" H62 MN8"}="2H6  MN8"; $resn_exch{" H63 MN8"}="3H6  MN8";
$resn_exch{"H121 MN8"}="1H12 MN8"; $resn_exch{"H122 MN8"}="2H12 MN8"; $resn_exch{"H151 MN8"}="1H15 MN8";
$resn_exch{"H152 MN8"}="2H15 MN8"; $resn_exch{" H31 MNA"}="1H3  MNA"; $resn_exch{" H32 MNA"}="2H3  MNA";
$resn_exch{" H91 MNA"}="1H9  MNA"; $resn_exch{" H92 MNA"}="2H9  MNA"; $resn_exch{"H111 MNA"}="1H11 MNA";
$resn_exch{"H112 MNA"}="2H11 MNA"; $resn_exch{"H113 MNA"}="3H11 MNA"; $resn_exch{"H121 MNA"}="1H12 MNA";
$resn_exch{"H122 MNA"}="2H12 MNA"; $resn_exch{"H123 MNA"}="3H12 MNA"; $resn_exch{"HOB1 MNA"}="1HOB MNA";
$resn_exch{" H11 MNC"}="1H1  MNC"; $resn_exch{" H12 MNC"}="2H1  MNC"; $resn_exch{" H13 MNC"}="3H1  MNC";
$resn_exch{" H81 MNG"}="1H8  MNG"; $resn_exch{" H82 MNG"}="2H8  MNG"; $resn_exch{"H101 MNG"}="1H10 MNG";
$resn_exch{"H102 MNG"}="2H10 MNG"; $resn_exch{" H1' MNG"}=" H1* MNG"; $resn_exch{" H2' MNG"}=" H2* MNG";
$resn_exch{" H3' MNG"}=" H3* MNG"; $resn_exch{" H4' MNG"}=" H4* MNG"; $resn_exch{"H5M1 MNG"}="1H5M MNG";
$resn_exch{"H5M2 MNG"}="2H5M MNG"; $resn_exch{"H5M3 MNG"}="3H5M MNG"; $resn_exch{"H1M1 MNG"}="1H1M MNG";
$resn_exch{"H1M2 MNG"}="2H1M MNG"; $resn_exch{"H1M3 MNG"}="3H1M MNG"; $resn_exch{"H2M1 MNG"}="1H2M MNG";
$resn_exch{"H2M2 MNG"}="2H2M MNG"; $resn_exch{"H2M3 MNG"}="3H2M MNG"; $resn_exch{"H7M1 MNG"}="1H7M MNG";
$resn_exch{"H7M2 MNG"}="2H7M MNG"; $resn_exch{"H7M3 MNG"}="3H7M MNG"; $resn_exch{"H9M1 MNG"}="1H9M MNG";
$resn_exch{"H9M2 MNG"}="2H9M MNG"; $resn_exch{"H9M3 MNG"}="3H9M MNG"; $resn_exch{"HO2' MNG"}="*HO2 MNG";
$resn_exch{"HO4' MNG"}="*HO4 MNG"; $resn_exch{"HMA1 MNH"}="1HMA MNH"; $resn_exch{"HMA2 MNH"}="2HMA MNH";
$resn_exch{"HMA3 MNH"}="3HMA MNH"; $resn_exch{"HAA1 MNH"}="1HAA MNH"; $resn_exch{"HAA2 MNH"}="2HAA MNH";
$resn_exch{"HBA1 MNH"}="1HBA MNH"; $resn_exch{"HBA2 MNH"}="2HBA MNH"; $resn_exch{"HMB1 MNH"}="1HMB MNH";
$resn_exch{"HMB2 MNH"}="2HMB MNH"; $resn_exch{"HMB3 MNH"}="3HMB MNH"; $resn_exch{"HBB1 MNH"}="1HBB MNH";
$resn_exch{"HBB2 MNH"}="2HBB MNH"; $resn_exch{"HMC1 MNH"}="1HMC MNH"; $resn_exch{"HMC2 MNH"}="2HMC MNH";
$resn_exch{"HMC3 MNH"}="3HMC MNH"; $resn_exch{"HBC1 MNH"}="1HBC MNH"; $resn_exch{"HBC2 MNH"}="2HBC MNH";
$resn_exch{"HMD1 MNH"}="1HMD MNH"; $resn_exch{"HMD2 MNH"}="2HMD MNH"; $resn_exch{"HMD3 MNH"}="3HMD MNH";
$resn_exch{"HAD1 MNH"}="1HAD MNH"; $resn_exch{"HAD2 MNH"}="2HAD MNH"; $resn_exch{"HBD1 MNH"}="1HBD MNH";
$resn_exch{"HBD2 MNH"}="2HBD MNH"; $resn_exch{"H8B1 MNI"}="1H8B MNI"; $resn_exch{"H8B2 MNI"}="2H8B MNI";
$resn_exch{"H8B3 MNI"}="3H8B MNI"; $resn_exch{"H9B1 MNI"}="1H9B MNI"; $resn_exch{"H9B2 MNI"}="2H9B MNI";
$resn_exch{"H9B3 MNI"}="3H9B MNI"; $resn_exch{"H1A1 MNI"}="1H1A MNI"; $resn_exch{"H1A2 MNI"}="2H1A MNI";
$resn_exch{"H6A1 MNI"}="1H6A MNI"; $resn_exch{"H6A2 MNI"}="2H6A MNI"; $resn_exch{" H   MNL"}=" HN  MNL";
$resn_exch{" HB2 MNL"}="1HB  MNL"; $resn_exch{" HB3 MNL"}="2HB  MNL"; $resn_exch{" HD2 MNL"}="1HD  MNL";
$resn_exch{" HD3 MNL"}="2HD  MNL"; $resn_exch{" HE1 MNL"}="1HE  MNL"; $resn_exch{" HE2 MNL"}="2HE  MNL";
$resn_exch{" HE3 MNL"}="3HE  MNL"; $resn_exch{"HM11 MNL"}="1HM1 MNL"; $resn_exch{"HM12 MNL"}="2HM1 MNL";
$resn_exch{"HM13 MNL"}="3HM1 MNL"; $resn_exch{"HM41 MNL"}="1HM4 MNL"; $resn_exch{"HM42 MNL"}="2HM4 MNL";
$resn_exch{"HM43 MNL"}="3HM4 MNL"; $resn_exch{"HC91 MNM"}="1HC9 MNM"; $resn_exch{"HC92 MNM"}="2HC9 MNM";
$resn_exch{"H161 MNO"}="1H16 MNO"; $resn_exch{"H162 MNO"}="2H16 MNO"; $resn_exch{"H163 MNO"}="3H16 MNO";
$resn_exch{"H171 MNO"}="1H17 MNO"; $resn_exch{"H172 MNO"}="2H17 MNO"; $resn_exch{"H173 MNO"}="3H17 MNO";
$resn_exch{"H201 MNO"}="1H20 MNO"; $resn_exch{"H202 MNO"}="2H20 MNO"; $resn_exch{"H211 MNO"}="1H21 MNO";
$resn_exch{"H212 MNO"}="2H21 MNO"; $resn_exch{"H221 MNO"}="1H22 MNO"; $resn_exch{"H222 MNO"}="2H22 MNO";
$resn_exch{"H241 MNO"}="1H24 MNO"; $resn_exch{"H242 MNO"}="2H24 MNO"; $resn_exch{"H151 MNO"}="1H15 MNO";
$resn_exch{"H152 MNO"}="2H15 MNO"; $resn_exch{"H153 MNO"}="3H15 MNO"; $resn_exch{" H21 MNP"}="1H2  MNP";
$resn_exch{" H22 MNP"}="2H2  MNP"; $resn_exch{"HOB2 MNQ"}="2HOB MNQ"; $resn_exch{"HOA2 MNQ"}="2HOA MNQ";
$resn_exch{"HNA1 MNQ"}="1HNA MNQ"; $resn_exch{"HA21 MNQ"}="1HA2 MNQ"; $resn_exch{"HA22 MNQ"}="2HA2 MNQ";
$resn_exch{"HA11 MNQ"}="1HA1 MNQ"; $resn_exch{"HA12 MNQ"}="2HA1 MNQ"; $resn_exch{"HMA1 MNR"}="1HMA MNR";
$resn_exch{"HMA2 MNR"}="2HMA MNR"; $resn_exch{"HMA3 MNR"}="3HMA MNR"; $resn_exch{"HAA1 MNR"}="1HAA MNR";
$resn_exch{"HAA2 MNR"}="2HAA MNR"; $resn_exch{"HBA1 MNR"}="1HBA MNR"; $resn_exch{"HBA2 MNR"}="2HBA MNR";
$resn_exch{"HMB1 MNR"}="1HMB MNR"; $resn_exch{"HMB2 MNR"}="2HMB MNR"; $resn_exch{"HMB3 MNR"}="3HMB MNR";
$resn_exch{"HBB1 MNR"}="1HBB MNR"; $resn_exch{"HBB2 MNR"}="2HBB MNR"; $resn_exch{"HMC1 MNR"}="1HMC MNR";
$resn_exch{"HMC2 MNR"}="2HMC MNR"; $resn_exch{"HMC3 MNR"}="3HMC MNR"; $resn_exch{"HBC1 MNR"}="1HBC MNR";
$resn_exch{"HBC2 MNR"}="2HBC MNR"; $resn_exch{"HMD1 MNR"}="1HMD MNR"; $resn_exch{"HMD2 MNR"}="2HMD MNR";
$resn_exch{"HMD3 MNR"}="3HMD MNR"; $resn_exch{"HAD1 MNR"}="1HAD MNR"; $resn_exch{"HAD2 MNR"}="2HAD MNR";
$resn_exch{"HBD1 MNR"}="1HBD MNR"; $resn_exch{"HBD2 MNR"}="2HBD MNR"; $resn_exch{"HM11 MNS"}="1HM1 MNS";
$resn_exch{"HM12 MNS"}="2HM1 MNS"; $resn_exch{"HM13 MNS"}="3HM1 MNS"; $resn_exch{"HM21 MNS"}="1HM2 MNS";
$resn_exch{"HM22 MNS"}="2HM2 MNS"; $resn_exch{"HM23 MNS"}="3HM2 MNS"; $resn_exch{"HN31 MNS"}="1HN3 MNS";
$resn_exch{"HN32 MNS"}="2HN3 MNS"; $resn_exch{"HOB2 MNT"}="2HOB MNT"; $resn_exch{"HOB3 MNT"}="3HOB MNT";
$resn_exch{"HOA2 MNT"}="2HOA MNT"; $resn_exch{"H5'1 MNT"}="1H5* MNT"; $resn_exch{"H5'2 MNT"}="2H5* MNT";
$resn_exch{" H4B MNT"}=" H4* MNT"; $resn_exch{" H3B MNT"}=" H3* MNT"; $resn_exch{"H2'1 MNT"}="1H2* MNT";
$resn_exch{"H2'2 MNT"}="2H2* MNT"; $resn_exch{" H1' MNT"}=" H1* MNT"; $resn_exch{"HN61 MNT"}="1HN6 MNT";
$resn_exch{"HN62 MNT"}="2HN6 MNT"; $resn_exch{"HN2' MNT"}="'HN2 MNT"; $resn_exch{"HM'1 MNT"}="1HM' MNT";
$resn_exch{"HM'2 MNT"}="2HM' MNT"; $resn_exch{"HM'3 MNT"}="3HM' MNT"; $resn_exch{"HOP2 MNU"}="2HOP MNU";
$resn_exch{" H5' MNU"}="1H5* MNU"; $resn_exch{"H5'' MNU"}="2H5* MNU"; $resn_exch{" H4' MNU"}=" H4* MNU";
$resn_exch{" H1' MNU"}=" H1* MNU"; $resn_exch{" H71 MNU"}="1H7  MNU"; $resn_exch{" H72 MNU"}="2H7  MNU";
$resn_exch{" H91 MNU"}="1H9  MNU"; $resn_exch{" H92 MNU"}="2H9  MNU"; $resn_exch{" H93 MNU"}="3H9  MNU";
$resn_exch{" H2' MNU"}=" H2* MNU"; $resn_exch{"HO2' MNU"}="2HO* MNU"; $resn_exch{" H3' MNU"}=" H3* MNU";
$resn_exch{"HO3' MNU"}="H3T  MNU"; $resn_exch{"HOP3 MNU"}="3HOP MNU"; $resn_exch{" H   MNV"}=" HN  MNV";
$resn_exch{" HN1 MNV"}="1HN  MNV"; $resn_exch{" HN2 MNV"}="2HN  MNV"; $resn_exch{" HN3 MNV"}="3HN  MNV";
$resn_exch{"HG11 MNV"}="1HG1 MNV"; $resn_exch{"HG12 MNV"}="2HG1 MNV"; $resn_exch{"HG13 MNV"}="3HG1 MNV";
$resn_exch{"HG21 MNV"}="1HG2 MNV"; $resn_exch{"HG22 MNV"}="2HG2 MNV"; $resn_exch{"HG23 MNV"}="3HG2 MNV";
$resn_exch{"HNT1 MNV"}="1HNT MNV"; $resn_exch{"HNT2 MNV"}="2HNT MNV"; $resn_exch{"H221 MNY"}="1H22 MNY";
$resn_exch{"H222 MNY"}="2H22 MNY"; $resn_exch{"H251 MNY"}="1H25 MNY"; $resn_exch{"H252 MNY"}="2H25 MNY";
$resn_exch{"HO11 MO1"}="1HO1 MO1"; $resn_exch{"HO12 MO1"}="2HO1 MO1"; $resn_exch{"HO11 MO2"}="1HO1 MO2";
$resn_exch{"HO12 MO2"}="2HO1 MO2"; $resn_exch{"HO21 MO2"}="1HO2 MO2"; $resn_exch{"HO22 MO2"}="2HO2 MO2";
$resn_exch{"HO11 MO3"}="1HO1 MO3"; $resn_exch{"HO12 MO3"}="2HO1 MO3"; $resn_exch{"HO21 MO3"}="1HO2 MO3";
$resn_exch{"HO22 MO3"}="2HO2 MO3"; $resn_exch{"HO31 MO3"}="1HO3 MO3"; $resn_exch{"HO32 MO3"}="2HO3 MO3";
$resn_exch{"HO11 MO4"}="1HO1 MO4"; $resn_exch{"HO12 MO4"}="2HO1 MO4"; $resn_exch{"HO21 MO4"}="1HO2 MO4";
$resn_exch{"HO22 MO4"}="2HO2 MO4"; $resn_exch{"HO31 MO4"}="1HO3 MO4"; $resn_exch{"HO32 MO4"}="2HO3 MO4";
$resn_exch{"HO41 MO4"}="1HO4 MO4"; $resn_exch{"HO42 MO4"}="2HO4 MO4"; $resn_exch{"HO11 MO5"}="1HO1 MO5";
$resn_exch{"HO12 MO5"}="2HO1 MO5"; $resn_exch{"HO21 MO5"}="1HO2 MO5"; $resn_exch{"HO22 MO5"}="2HO2 MO5";
$resn_exch{"HO31 MO5"}="1HO3 MO5"; $resn_exch{"HO32 MO5"}="2HO3 MO5"; $resn_exch{"HO41 MO5"}="1HO4 MO5";
$resn_exch{"HO42 MO5"}="2HO4 MO5"; $resn_exch{"HO51 MO5"}="1HO5 MO5"; $resn_exch{"HO52 MO5"}="2HO5 MO5";
$resn_exch{"HO11 MO6"}="1HO1 MO6"; $resn_exch{"HO12 MO6"}="2HO1 MO6"; $resn_exch{"HO21 MO6"}="1HO2 MO6";
$resn_exch{"HO22 MO6"}="2HO2 MO6"; $resn_exch{"HO31 MO6"}="1HO3 MO6"; $resn_exch{"HO32 MO6"}="2HO3 MO6";
$resn_exch{"HO41 MO6"}="1HO4 MO6"; $resn_exch{"HO42 MO6"}="2HO4 MO6"; $resn_exch{"HO51 MO6"}="1HO5 MO6";
$resn_exch{"HO52 MO6"}="2HO5 MO6"; $resn_exch{"HO61 MO6"}="1HO6 MO6"; $resn_exch{"HO62 MO6"}="2HO6 MO6";
$resn_exch{" H71 MO9"}="1H7  MO9"; $resn_exch{" H72 MO9"}="2H7  MO9"; $resn_exch{" H91 MO9"}="1H9  MO9";
$resn_exch{" H92 MO9"}="2H9  MO9"; $resn_exch{"H171 MO9"}="1H17 MO9"; $resn_exch{"H172 MO9"}="2H17 MO9";
$resn_exch{"H221 MO9"}="1H22 MO9"; $resn_exch{"H222 MO9"}="2H22 MO9"; $resn_exch{"H231 MO9"}="1H23 MO9";
$resn_exch{"H232 MO9"}="2H23 MO9"; $resn_exch{"H251 MO9"}="1H25 MO9"; $resn_exch{"H252 MO9"}="2H25 MO9";
$resn_exch{"H261 MO9"}="1H26 MO9"; $resn_exch{"H262 MO9"}="2H26 MO9"; $resn_exch{" H21 MOA"}="1H2  MOA";
$resn_exch{" H41 MOA"}="1H4  MOA"; $resn_exch{" H42 MOA"}="2H4  MOA"; $resn_exch{" H51 MOA"}="1H5  MOA";
$resn_exch{" H52 MOA"}="2H5  MOA"; $resn_exch{" H71 MOA"}="1H7  MOA"; $resn_exch{" H72 MOA"}="2H7  MOA";
$resn_exch{" H73 MOA"}="3H7  MOA"; $resn_exch{" H81 MOA"}="1H8  MOA"; $resn_exch{" H82 MOA"}="2H8  MOA";
$resn_exch{" H83 MOA"}="3H8  MOA"; $resn_exch{" H91 MOA"}="1H9  MOA"; $resn_exch{" H92 MOA"}="2H9  MOA";
$resn_exch{" H93 MOA"}="3H9  MOA"; $resn_exch{"H1O1 MOA"}="1H1O MOA"; $resn_exch{"H102 MOA"}="2H10 MOA";
$resn_exch{"H171 MOA"}="1H17 MOA"; $resn_exch{"H172 MOA"}="2H17 MOA"; $resn_exch{"HO41 MOA"}="1HO4 MOA";
$resn_exch{"HO51 MOA"}="1HO5 MOA"; $resn_exch{" H31 MOB"}="1H3  MOB"; $resn_exch{" H32 MOB"}="2H3  MOB";
$resn_exch{" H33 MOB"}="3H3  MOB"; $resn_exch{" H51 MOB"}="1H5  MOB"; $resn_exch{" H52 MOB"}="2H5  MOB";
$resn_exch{" H53 MOB"}="3H5  MOB"; $resn_exch{"HO4' MOB"}="'HO4 MOB"; $resn_exch{" H11 MOC"}="1H1  MOC";
$resn_exch{" H12 MOC"}="2H1  MOC"; $resn_exch{"HN21 MOC"}="1HN2 MOC"; $resn_exch{"HN22 MOC"}="2HN2 MOC";
$resn_exch{" H31 MOC"}="1H3  MOC"; $resn_exch{" H32 MOC"}="2H3  MOC"; $resn_exch{"HM61 MOC"}="1HM6 MOC";
$resn_exch{"HM62 MOC"}="2HM6 MOC"; $resn_exch{"HM63 MOC"}="3HM6 MOC"; $resn_exch{"HN71 MOC"}="1HN7 MOC";
$resn_exch{"HN72 MOC"}="2HN7 MOC"; $resn_exch{"H101 MOC"}="1H10 MOC"; $resn_exch{"H102 MOC"}="2H10 MOC";
$resn_exch{"H121 MOC"}="1H12 MOC"; $resn_exch{"H122 MOC"}="2H12 MOC"; $resn_exch{"HN21 MOD"}="1HN2 MOD";
$resn_exch{"HN22 MOD"}="2HN2 MOD"; $resn_exch{" HB1 MOD"}="1HB  MOD"; $resn_exch{" HB2 MOD"}="2HB  MOD";
$resn_exch{" HG1 MOD"}="1HG  MOD"; $resn_exch{" HG2 MOD"}="2HG  MOD"; $resn_exch{" HE1 MOD"}="1HE  MOD";
$resn_exch{" HE2 MOD"}="2HE  MOD"; $resn_exch{" HE3 MOD"}="3HE  MOD"; $resn_exch{" H91 MOD"}="1H9  MOD";
$resn_exch{" H92 MOD"}="2H9  MOD"; $resn_exch{"HOS1 MOD"}="1HOS MOD"; $resn_exch{"H5'1 MOD"}="1H5* MOD";
$resn_exch{"H5'2 MOD"}="2H5* MOD"; $resn_exch{" H4' MOD"}=" H4* MOD"; $resn_exch{" H3' MOD"}=" H3* MOD";
$resn_exch{" H2' MOD"}=" H2* MOD"; $resn_exch{" H1' MOD"}=" H1* MOD"; $resn_exch{"HN61 MOD"}="1HN6 MOD";
$resn_exch{"HN62 MOD"}="2HN6 MOD"; $resn_exch{"HA'1 MOE"}="1HA* MOE"; $resn_exch{"HA'2 MOE"}="2HA* MOE";
$resn_exch{"HB'1 MOE"}="1HB* MOE"; $resn_exch{"HB'2 MOE"}="2HB* MOE"; $resn_exch{"HD'1 MOE"}="1HD* MOE";
$resn_exch{"HD'2 MOE"}="2HD* MOE"; $resn_exch{"HD'3 MOE"}="3HD* MOE"; $resn_exch{"H261 MOF"}="1H26 MOF";
$resn_exch{"H262 MOF"}="2H26 MOF"; $resn_exch{"H311 MOF"}="1H31 MOF"; $resn_exch{"H312 MOF"}="2H31 MOF";
$resn_exch{"H313 MOF"}="3H31 MOF"; $resn_exch{"H201 MOF"}="1H20 MOF"; $resn_exch{"H202 MOF"}="2H20 MOF";
$resn_exch{" H91 MOF"}="1H9  MOF"; $resn_exch{" H92 MOF"}="2H9  MOF"; $resn_exch{"H101 MOF"}="1H10 MOF";
$resn_exch{"H102 MOF"}="2H10 MOF"; $resn_exch{"H103 MOF"}="3H10 MOF"; $resn_exch{" H71 MOF"}="1H7  MOF";
$resn_exch{" H72 MOF"}="2H7  MOF"; $resn_exch{" H81 MOF"}="1H8  MOF"; $resn_exch{" H82 MOF"}="2H8  MOF";
$resn_exch{" H83 MOF"}="3H8  MOF"; $resn_exch{"H121 MOF"}="1H12 MOF"; $resn_exch{"H122 MOF"}="2H12 MOF";
$resn_exch{"H101 MOG"}="1H10 MOG"; $resn_exch{"H102 MOG"}="2H10 MOG"; $resn_exch{"H103 MOG"}="3H10 MOG";
$resn_exch{"HN21 MOG"}="1HN2 MOG"; $resn_exch{"HN22 MOG"}="2HN2 MOG"; $resn_exch{" H1  MOH"}="1H   MOH";
$resn_exch{" H2  MOH"}="2H   MOH"; $resn_exch{" H3  MOH"}="3H   MOH"; $resn_exch{" H21 MOI"}="1H2  MOI";
$resn_exch{" H22 MOI"}="2H2  MOI"; $resn_exch{" H71 MOI"}="1H7  MOI"; $resn_exch{" H72 MOI"}="2H7  MOI";
$resn_exch{" H81 MOI"}="1H8  MOI"; $resn_exch{" H82 MOI"}="2H8  MOI"; $resn_exch{" H91 MOI"}="1H9  MOI";
$resn_exch{" H92 MOI"}="2H9  MOI"; $resn_exch{" H93 MOI"}="3H9  MOI"; $resn_exch{"HN11 MOL"}="1HN1 MOL";
$resn_exch{"HN12 MOL"}="2HN1 MOL"; $resn_exch{" H71 MOL"}="1H7  MOL"; $resn_exch{" H72 MOL"}="2H7  MOL";
$resn_exch{"H101 MOL"}="1H10 MOL"; $resn_exch{"H102 MOL"}="2H10 MOL"; $resn_exch{"H111 MOL"}="1H11 MOL";
$resn_exch{"H112 MOL"}="2H11 MOL"; $resn_exch{"H121 MOL"}="1H12 MOL"; $resn_exch{"H122 MOL"}="2H12 MOL";
$resn_exch{"H131 MOL"}="1H13 MOL"; $resn_exch{"H132 MOL"}="2H13 MOL"; $resn_exch{" H51 MON"}="1H5  MON";
$resn_exch{" H52 MON"}="2H5  MON"; $resn_exch{" H61 MON"}="1H6  MON"; $resn_exch{" H62 MON"}="2H6  MON";
$resn_exch{" H91 MON"}="1H9  MON"; $resn_exch{" H92 MON"}="2H9  MON"; $resn_exch{"H101 MON"}="1H10 MON";
$resn_exch{"H102 MON"}="2H10 MON"; $resn_exch{"H151 MON"}="1H15 MON"; $resn_exch{"H152 MON"}="2H15 MON";
$resn_exch{"H161 MON"}="1H16 MON"; $resn_exch{"H162 MON"}="2H16 MON"; $resn_exch{"H171 MON"}="1H17 MON";
$resn_exch{"H172 MON"}="2H17 MON"; $resn_exch{"H181 MON"}="1H18 MON"; $resn_exch{"H182 MON"}="2H18 MON";
$resn_exch{"H201 MON"}="1H20 MON"; $resn_exch{"H202 MON"}="2H20 MON"; $resn_exch{" H21 MOP"}="1H2  MOP";
$resn_exch{" H22 MOP"}="2H2  MOP"; $resn_exch{" H31 MOP"}="1H3  MOP"; $resn_exch{" H32 MOP"}="2H3  MOP";
$resn_exch{" H51 MOP"}="1H5  MOP"; $resn_exch{" H52 MOP"}="2H5  MOP"; $resn_exch{" H53 MOP"}="3H5  MOP";
$resn_exch{"HM41 MOP"}="1HM4 MOP"; $resn_exch{"HM42 MOP"}="2HM4 MOP"; $resn_exch{"HM43 MOP"}="3HM4 MOP";
$resn_exch{" H31 MOQ"}="1H3  MOQ"; $resn_exch{"H32A MOQ"}="2H3  MOQ"; $resn_exch{" H41 MOQ"}="1H4  MOQ";
$resn_exch{" H42 MOQ"}="2H4  MOQ"; $resn_exch{" H61 MOQ"}="1H6  MOQ"; $resn_exch{" H62 MOQ"}="2H6  MOQ";
$resn_exch{" H63 MOQ"}="3H6  MOQ"; $resn_exch{" H91 MOQ"}="1H9  MOQ"; $resn_exch{" H92 MOQ"}="2H9  MOQ";
$resn_exch{"H101 MOQ"}="1H10 MOQ"; $resn_exch{"H102 MOQ"}="2H10 MOQ"; $resn_exch{"H103 MOQ"}="3H10 MOQ";
$resn_exch{"H151 MOQ"}="1H15 MOQ"; $resn_exch{"H152 MOQ"}="2H15 MOQ"; $resn_exch{"H153 MOQ"}="3H15 MOQ";
$resn_exch{"H191 MOQ"}="1H19 MOQ"; $resn_exch{"H192 MOQ"}="2H19 MOQ"; $resn_exch{"H193 MOQ"}="3H19 MOQ";
$resn_exch{"H201 MOQ"}="1H20 MOQ"; $resn_exch{"H202 MOQ"}="2H20 MOQ"; $resn_exch{"H203 MOQ"}="3H20 MOQ";
$resn_exch{"H231 MOQ"}="1H23 MOQ"; $resn_exch{"H232 MOQ"}="2H23 MOQ"; $resn_exch{"H233 MOQ"}="3H23 MOQ";
$resn_exch{"H251 MOQ"}="1H25 MOQ"; $resn_exch{"H252 MOQ"}="2H25 MOQ"; $resn_exch{"H261 MOQ"}="1H26 MOQ";
$resn_exch{"H262 MOQ"}="2H26 MOQ"; $resn_exch{"H281 MOQ"}="1H28 MOQ"; $resn_exch{"H282 MOQ"}="2H28 MOQ";
$resn_exch{"H283 MOQ"}="3H28 MOQ"; $resn_exch{"H291 MOQ"}="1H29 MOQ"; $resn_exch{"H292 MOQ"}="2H29 MOQ";
$resn_exch{"H311 MOQ"}="1H31 MOQ"; $resn_exch{"H312 MOQ"}="2H31 MOQ"; $resn_exch{"H313 MOQ"}="3H31 MOQ";
$resn_exch{"H331 MOQ"}="1H33 MOQ"; $resn_exch{"H332 MOQ"}="2H33 MOQ"; $resn_exch{"H333 MOQ"}="3H33 MOQ";
$resn_exch{"H341 MOQ"}="1H34 MOQ"; $resn_exch{"H342 MOQ"}="2H34 MOQ"; $resn_exch{" H21 MOR"}="1H2  MOR";
$resn_exch{" H22 MOR"}="2H2  MOR"; $resn_exch{" H31 MOR"}="1H3  MOR"; $resn_exch{" H32 MOR"}="2H3  MOR";
$resn_exch{" H51 MOR"}="1H5  MOR"; $resn_exch{" H52 MOR"}="2H5  MOR"; $resn_exch{" H61 MOR"}="1H6  MOR";
$resn_exch{" H62 MOR"}="2H6  MOR"; $resn_exch{"HN21 MOT"}="1HN2 MOT"; $resn_exch{"HN22 MOT"}="2HN2 MOT";
$resn_exch{"HN41 MOT"}="1HN4 MOT"; $resn_exch{"HN42 MOT"}="2HN4 MOT"; $resn_exch{" H91 MOT"}="1H9  MOT";
$resn_exch{" H92 MOT"}="2H9  MOT"; $resn_exch{" HM1 MOT"}="1HM  MOT"; $resn_exch{" HM2 MOT"}="2HM  MOT";
$resn_exch{" HM3 MOT"}="3HM  MOT"; $resn_exch{" HB1 MOT"}="1HB  MOT"; $resn_exch{" HB2 MOT"}="2HB  MOT";
$resn_exch{" HG1 MOT"}="1HG  MOT"; $resn_exch{" HG2 MOT"}="2HG  MOT"; $resn_exch{"HOE2 MOT"}="2HOE MOT";
$resn_exch{"H441 MOU"}="1H44 MOU"; $resn_exch{"H442 MOU"}="2H44 MOU"; $resn_exch{"H431 MOU"}="1H43 MOU";
$resn_exch{"H432 MOU"}="2H43 MOU"; $resn_exch{"H421 MOU"}="1H42 MOU"; $resn_exch{"H422 MOU"}="2H42 MOU";
$resn_exch{"H411 MOU"}="1H41 MOU"; $resn_exch{"H412 MOU"}="2H41 MOU"; $resn_exch{"H401 MOU"}="1H40 MOU";
$resn_exch{"H402 MOU"}="2H40 MOU"; $resn_exch{"H391 MOU"}="1H39 MOU"; $resn_exch{"H392 MOU"}="2H39 MOU";
$resn_exch{"H381 MOU"}="1H38 MOU"; $resn_exch{"H382 MOU"}="2H38 MOU"; $resn_exch{"H261 MOU"}="1H26 MOU";
$resn_exch{"H262 MOU"}="2H26 MOU"; $resn_exch{"H131 MOU"}="1H13 MOU"; $resn_exch{"H132 MOU"}="2H13 MOU";
$resn_exch{"H191 MOU"}="1H19 MOU"; $resn_exch{"H192 MOU"}="2H19 MOU"; $resn_exch{"H193 MOU"}="3H19 MOU";
$resn_exch{"H171 MOU"}="1H17 MOU"; $resn_exch{"H172 MOU"}="2H17 MOU"; $resn_exch{"H181 MOU"}="1H18 MOU";
$resn_exch{"H182 MOU"}="2H18 MOU"; $resn_exch{" H71 MOU"}="1H7  MOU"; $resn_exch{" H72 MOU"}="2H7  MOU";
$resn_exch{"H211 MOU"}="1H21 MOU"; $resn_exch{"H212 MOU"}="2H21 MOU"; $resn_exch{"H213 MOU"}="3H21 MOU";
$resn_exch{"H221 MOU"}="1H22 MOU"; $resn_exch{"H222 MOU"}="2H22 MOU"; $resn_exch{"H223 MOU"}="3H22 MOU";
$resn_exch{"H341 MOU"}="1H34 MOU"; $resn_exch{"H342 MOU"}="2H34 MOU"; $resn_exch{"H343 MOU"}="3H34 MOU";
$resn_exch{"H361 MOU"}="1H36 MOU"; $resn_exch{"H362 MOU"}="2H36 MOU"; $resn_exch{"H363 MOU"}="3H36 MOU";
$resn_exch{"H621 MOX"}="1H62 MOX"; $resn_exch{"H622 MOX"}="2H62 MOX"; $resn_exch{"H623 MOX"}="3H62 MOX";
$resn_exch{" H51 MOX"}="1H5  MOX"; $resn_exch{" H52 MOX"}="2H5  MOX"; $resn_exch{" H91 MOX"}="1H9  MOX";
$resn_exch{" H92 MOX"}="2H9  MOX"; $resn_exch{" H93 MOX"}="3H9  MOX"; $resn_exch{" H51 MOY"}="1H5  MOY";
$resn_exch{" H52 MOY"}="2H5  MOY"; $resn_exch{" H53 MOY"}="3H5  MOY"; $resn_exch{" H61 MOY"}="1H6  MOY";
$resn_exch{" H62 MOY"}="2H6  MOY"; $resn_exch{" H63 MOY"}="3H6  MOY"; $resn_exch{" H11 MP1"}="1H1  MP1";
$resn_exch{" H12 MP1"}="2H1  MP1"; $resn_exch{" H13 MP1"}="3H1  MP1"; $resn_exch{"H161 MP1"}="1H16 MP1";
$resn_exch{"H162 MP1"}="2H16 MP1"; $resn_exch{"H163 MP1"}="3H16 MP1"; $resn_exch{"H171 MP1"}="1H17 MP1";
$resn_exch{"H172 MP1"}="2H17 MP1"; $resn_exch{"H181 MP1"}="1H18 MP1"; $resn_exch{"H182 MP1"}="2H18 MP1";
$resn_exch{"H183 MP1"}="3H18 MP1"; $resn_exch{"H261 MP1"}="1H26 MP1"; $resn_exch{"H262 MP1"}="2H26 MP1";
$resn_exch{"H263 MP1"}="3H26 MP1"; $resn_exch{"H271 MP1"}="1H27 MP1"; $resn_exch{"H272 MP1"}="2H27 MP1";
$resn_exch{"H281 MP1"}="1H28 MP1"; $resn_exch{"H282 MP1"}="2H28 MP1"; $resn_exch{"H283 MP1"}="3H28 MP1";
$resn_exch{"H361 MP1"}="1H36 MP1"; $resn_exch{"H362 MP1"}="2H36 MP1"; $resn_exch{"H363 MP1"}="3H36 MP1";
$resn_exch{"H371 MP1"}="1H37 MP1"; $resn_exch{"H372 MP1"}="2H37 MP1"; $resn_exch{"H381 MP1"}="1H38 MP1";
$resn_exch{"H382 MP1"}="2H38 MP1"; $resn_exch{"H461 MP1"}="1H46 MP1"; $resn_exch{"H462 MP1"}="2H46 MP1";
$resn_exch{"H463 MP1"}="3H46 MP1"; $resn_exch{"H471 MP1"}="1H47 MP1"; $resn_exch{"H472 MP1"}="2H47 MP1";
$resn_exch{"H481 MP1"}="1H48 MP1"; $resn_exch{"H482 MP1"}="2H48 MP1"; $resn_exch{"H171 MP2"}="1H17 MP2";
$resn_exch{"H172 MP2"}="2H17 MP2"; $resn_exch{" HA1 MP2"}="1HA  MP2"; $resn_exch{" HA2 MP2"}="2HA  MP2";
$resn_exch{"H231 MP2"}="1H23 MP2"; $resn_exch{"H232 MP2"}="2H23 MP2"; $resn_exch{" H31 MP3"}="1H3  MP3";
$resn_exch{" H32 MP3"}="2H3  MP3"; $resn_exch{"HOC2 MP3"}="2HOC MP3"; $resn_exch{"HOC3 MP3"}="3HOC MP3";
$resn_exch{" H31 MP5"}="1H3  MP5"; $resn_exch{" H32 MP5"}="2H3  MP5"; $resn_exch{"HN11 MP5"}="1HN1 MP5";
$resn_exch{"HN12 MP5"}="2HN1 MP5"; $resn_exch{" H51 MP5"}="1H5  MP5"; $resn_exch{" H52 MP5"}="2H5  MP5";
$resn_exch{" H53 MP5"}="3H5  MP5"; $resn_exch{"H111 MP9"}="1H11 MP9"; $resn_exch{"H112 MP9"}="2H11 MP9";
$resn_exch{"H241 MP9"}="1H24 MP9"; $resn_exch{"H242 MP9"}="2H24 MP9"; $resn_exch{"H291 MP9"}="1H29 MP9";
$resn_exch{"H292 MP9"}="2H29 MP9"; $resn_exch{"H12A MPA"}="2H1  MPA"; $resn_exch{" H21 MPA"}="1H2  MPA";
$resn_exch{" H22 MPA"}="2H2  MPA"; $resn_exch{" H31 MPA"}="1H3  MPA"; $resn_exch{" H32 MPA"}="2H3  MPA";
$resn_exch{" H41 MPA"}="1H4  MPA"; $resn_exch{" H42 MPA"}="2H4  MPA"; $resn_exch{" H51 MPA"}="1H5  MPA";
$resn_exch{" H52 MPA"}="2H5  MPA"; $resn_exch{" H61 MPA"}="1H6  MPA"; $resn_exch{" H62 MPA"}="2H6  MPA";
$resn_exch{" H63 MPA"}="3H6  MPA"; $resn_exch{" H81 MPA"}="1H8  MPA"; $resn_exch{" H82 MPA"}="2H8  MPA";
$resn_exch{"H101 MPA"}="1H10 MPA"; $resn_exch{"H102 MPA"}="2H10 MPA"; $resn_exch{"H111 MPA"}="1H11 MPA";
$resn_exch{"H112 MPA"}="2H11 MPA"; $resn_exch{"H131 MPA"}="1H13 MPA"; $resn_exch{"H132 MPA"}="2H13 MPA";
$resn_exch{"H133 MPA"}="3H13 MPA"; $resn_exch{"H151 MPA"}="1H15 MPA"; $resn_exch{"H152 MPA"}="2H15 MPA";
$resn_exch{"H153 MPA"}="3H15 MPA"; $resn_exch{"H161 MPA"}="1H16 MPA"; $resn_exch{"H162 MPA"}="2H16 MPA";
$resn_exch{"H163 MPA"}="3H16 MPA"; $resn_exch{" HM1 MPB"}="1HM  MPB"; $resn_exch{" HM2 MPB"}="2HM  MPB";
$resn_exch{" HM3 MPB"}="3HM  MPB"; $resn_exch{"H12A MPC"}="2H1  MPC"; $resn_exch{" H21 MPC"}="1H2  MPC";
$resn_exch{" H22 MPC"}="2H2  MPC"; $resn_exch{" H31 MPC"}="1H3  MPC"; $resn_exch{" H32 MPC"}="2H3  MPC";
$resn_exch{" H41 MPC"}="1H4  MPC"; $resn_exch{" H42 MPC"}="2H4  MPC"; $resn_exch{" H51 MPC"}="1H5  MPC";
$resn_exch{" H52 MPC"}="2H5  MPC"; $resn_exch{" H61 MPC"}="1H6  MPC"; $resn_exch{" H62 MPC"}="2H6  MPC";
$resn_exch{" H63 MPC"}="3H6  MPC"; $resn_exch{" H81 MPC"}="1H8  MPC"; $resn_exch{" H82 MPC"}="2H8  MPC";
$resn_exch{"H101 MPC"}="1H10 MPC"; $resn_exch{"H102 MPC"}="2H10 MPC"; $resn_exch{"H111 MPC"}="1H11 MPC";
$resn_exch{"H112 MPC"}="2H11 MPC"; $resn_exch{"H131 MPC"}="1H13 MPC"; $resn_exch{"H132 MPC"}="2H13 MPC";
$resn_exch{"H133 MPC"}="3H13 MPC"; $resn_exch{"H151 MPC"}="1H15 MPC"; $resn_exch{"H152 MPC"}="2H15 MPC";
$resn_exch{"H153 MPC"}="3H15 MPC"; $resn_exch{"H161 MPC"}="1H16 MPC"; $resn_exch{"H162 MPC"}="2H16 MPC";
$resn_exch{"H163 MPC"}="3H16 MPC"; $resn_exch{" H11 MPD"}="1H1  MPD"; $resn_exch{" H12 MPD"}="2H1  MPD";
$resn_exch{" H13 MPD"}="3H1  MPD"; $resn_exch{" HM1 MPD"}="1HM  MPD"; $resn_exch{" HM2 MPD"}="2HM  MPD";
$resn_exch{" HM3 MPD"}="3HM  MPD"; $resn_exch{" H31 MPD"}="1H3  MPD"; $resn_exch{" H32 MPD"}="2H3  MPD";
$resn_exch{" H51 MPD"}="1H5  MPD"; $resn_exch{" H52 MPD"}="2H5  MPD"; $resn_exch{" H53 MPD"}="3H5  MPD";
$resn_exch{" H31 MPE"}="1H3  MPE"; $resn_exch{" H32 MPE"}="2H3  MPE"; $resn_exch{" H21 MPE"}="1H2  MPE";
$resn_exch{" H22 MPE"}="2H2  MPE"; $resn_exch{" H22 MPG"}="2H2  MPG"; $resn_exch{" H21 MPG"}="1H2  MPG";
$resn_exch{" H32 MPG"}="2H3  MPG"; $resn_exch{" H31 MPG"}="1H3  MPG"; $resn_exch{" H42 MPG"}="2H4  MPG";
$resn_exch{" H41 MPG"}="1H4  MPG"; $resn_exch{" H52 MPG"}="2H5  MPG"; $resn_exch{" H51 MPG"}="1H5  MPG";
$resn_exch{" H62 MPG"}="2H6  MPG"; $resn_exch{" H61 MPG"}="1H6  MPG"; $resn_exch{" H72 MPG"}="2H7  MPG";
$resn_exch{" H71 MPG"}="1H7  MPG"; $resn_exch{" H82 MPG"}="2H8  MPG"; $resn_exch{" H81 MPG"}="1H8  MPG";
$resn_exch{"H112 MPG"}="2H11 MPG"; $resn_exch{"H111 MPG"}="1H11 MPG"; $resn_exch{"H122 MPG"}="2H12 MPG";
$resn_exch{"H121 MPG"}="1H12 MPG"; $resn_exch{"H132 MPG"}="2H13 MPG"; $resn_exch{"H131 MPG"}="1H13 MPG";
$resn_exch{"H142 MPG"}="2H14 MPG"; $resn_exch{"H141 MPG"}="1H14 MPG"; $resn_exch{"H152 MPG"}="2H15 MPG";
$resn_exch{"H151 MPG"}="1H15 MPG"; $resn_exch{"H162 MPG"}="2H16 MPG"; $resn_exch{"H161 MPG"}="1H16 MPG";
$resn_exch{"H172 MPG"}="2H17 MPG"; $resn_exch{"H171 MPG"}="1H17 MPG"; $resn_exch{"H183 MPG"}="3H18 MPG";
$resn_exch{"H182 MPG"}="2H18 MPG"; $resn_exch{"H181 MPG"}="1H18 MPG"; $resn_exch{"H12A MPG"}="2H1  MPG";
$resn_exch{" H11 MPG"}="1H1  MPG"; $resn_exch{"H212 MPG"}="2H21 MPG"; $resn_exch{"H211 MPG"}="1H21 MPG";
$resn_exch{" HB1 MPH"}="1HB  MPH"; $resn_exch{" HB2 MPH"}="2HB  MPH"; $resn_exch{" HG1 MPH"}="1HG  MPH";
$resn_exch{" HG2 MPH"}="2HG  MPH"; $resn_exch{" HE1 MPH"}="1HE  MPH"; $resn_exch{" HE2 MPH"}="2HE  MPH";
$resn_exch{" HE3 MPH"}="3HE  MPH"; $resn_exch{" HB1 MPJ"}="1HB  MPJ"; $resn_exch{" HB2 MPJ"}="2HB  MPJ";
$resn_exch{" HG1 MPJ"}="1HG  MPJ"; $resn_exch{" HG2 MPJ"}="2HG  MPJ"; $resn_exch{" HE1 MPJ"}="1HE  MPJ";
$resn_exch{" HE2 MPJ"}="2HE  MPJ"; $resn_exch{" HE3 MPJ"}="3HE  MPJ"; $resn_exch{"HOP2 MPL"}="2HOP MPL";
$resn_exch{"HOP3 MPL"}="3HOP MPL"; $resn_exch{" H51 MPL"}="1H5  MPL"; $resn_exch{" H52 MPL"}="2H5  MPL";
$resn_exch{" H11 MPL"}="1H1  MPL"; $resn_exch{" H12 MPL"}="2H1  MPL"; $resn_exch{" H13 MPL"}="3H1  MPL";
$resn_exch{" H21 MPL"}="1H2  MPL"; $resn_exch{" H22 MPL"}="2H2  MPL"; $resn_exch{" H23 MPL"}="3H2  MPL";
$resn_exch{"H4AC MPM"}="CH4A MPM"; $resn_exch{"H5A1 MPM"}="1H5A MPM"; $resn_exch{"H5A2 MPM"}="2H5A MPM";
$resn_exch{"H2A1 MPM"}="1H2A MPM"; $resn_exch{"H2A2 MPM"}="2H2A MPM"; $resn_exch{"H2A3 MPM"}="3H2A MPM";
$resn_exch{"HA21 MPM"}="1HA2 MPM"; $resn_exch{"HA22 MPM"}="2HA2 MPM"; $resn_exch{"HA11 MPM"}="1HA1 MPM";
$resn_exch{"HA12 MPM"}="2HA1 MPM"; $resn_exch{"HA13 MPM"}="3HA1 MPM"; $resn_exch{"HA31 MPM"}="1HA3 MPM";
$resn_exch{"HA32 MPM"}="2HA3 MPM"; $resn_exch{"HA33 MPM"}="3HA3 MPM"; $resn_exch{"HOP3 MPN"}="3HOP MPN";
$resn_exch{"HOP2 MPN"}="2HOP MPN"; $resn_exch{"H4'1 MPN"}="1H4' MPN"; $resn_exch{"H4'2 MPN"}="2H4' MPN";
$resn_exch{" H21 MPN"}="1H2  MPN"; $resn_exch{" H22 MPN"}="2H2  MPN"; $resn_exch{" H11 MPO"}="1H1  MPO";
$resn_exch{" H12 MPO"}="2H1  MPO"; $resn_exch{" H21 MPO"}="1H2  MPO"; $resn_exch{" H22 MPO"}="2H2  MPO";
$resn_exch{" H31 MPO"}="1H3  MPO"; $resn_exch{" H32 MPO"}="2H3  MPO"; $resn_exch{" H41 MPO"}="1H4  MPO";
$resn_exch{" H42 MPO"}="2H4  MPO"; $resn_exch{" H51 MPO"}="1H5  MPO"; $resn_exch{" H52 MPO"}="2H5  MPO";
$resn_exch{" H61 MPO"}="1H6  MPO"; $resn_exch{" H62 MPO"}="2H6  MPO"; $resn_exch{" H71 MPO"}="1H7  MPO";
$resn_exch{" H72 MPO"}="2H7  MPO"; $resn_exch{" HA1 MPP"}="1HA  MPP"; $resn_exch{" HA2 MPP"}="2HA  MPP";
$resn_exch{" HB1 MPP"}="1HB  MPP"; $resn_exch{" HB2 MPP"}="2HB  MPP"; $resn_exch{"HEA1 MPP"}="1HEA MPP";
$resn_exch{"HEA2 MPP"}="2HEA MPP"; $resn_exch{"HEA3 MPP"}="3HEA MPP"; $resn_exch{"HZA1 MPP"}="1HZA MPP";
$resn_exch{"HZA2 MPP"}="2HZA MPP"; $resn_exch{"HZA3 MPP"}="3HZA MPP"; $resn_exch{" HM1 MPQ"}="1HM  MPQ";
$resn_exch{" HM2 MPQ"}="2HM  MPQ"; $resn_exch{" HM3 MPQ"}="3HM  MPQ"; $resn_exch{" HA1 MPQ"}="1HA  MPQ";
$resn_exch{"HD11 MPQ"}="1HD1 MPQ"; $resn_exch{"HD21 MPQ"}="1HD2 MPQ"; $resn_exch{"HE11 MPQ"}="1HE1 MPQ";
$resn_exch{"HE21 MPQ"}="1HE2 MPQ"; $resn_exch{" HZ1 MPQ"}="1HZ  MPQ"; $resn_exch{" H21 MPR"}="1H2  MPR";
$resn_exch{" H22 MPR"}="2H2  MPR"; $resn_exch{" H31 MPR"}="1H3  MPR"; $resn_exch{" H32 MPR"}="2H3  MPR";
$resn_exch{" HM1 MPS"}="1HM  MPS"; $resn_exch{" HM2 MPS"}="2HM  MPS"; $resn_exch{" HM3 MPS"}="3HM  MPS";
$resn_exch{" HA1 MPT"}="1HA  MPT"; $resn_exch{" HA2 MPT"}="2HA  MPT"; $resn_exch{" HB1 MPT"}="1HB  MPT";
$resn_exch{" HB2 MPT"}="2HB  MPT"; $resn_exch{"H131 MPU"}="1H13 MPU"; $resn_exch{"H132 MPU"}="2H13 MPU";
$resn_exch{"H281 MPU"}="1H28 MPU"; $resn_exch{"H282 MPU"}="2H28 MPU"; $resn_exch{"H231 MPX"}="1H23 MPX";
$resn_exch{"H232 MPX"}="2H23 MPX"; $resn_exch{"H221 MPX"}="1H22 MPX"; $resn_exch{"H222 MPX"}="2H22 MPX";
$resn_exch{"H241 MPX"}="1H24 MPX"; $resn_exch{"H242 MPX"}="2H24 MPX"; $resn_exch{"H243 MPX"}="3H24 MPX";
$resn_exch{"H201 MPX"}="1H20 MPX"; $resn_exch{"H202 MPX"}="2H20 MPX"; $resn_exch{"H191 MPX"}="1H19 MPX";
$resn_exch{"H192 MPX"}="2H19 MPX"; $resn_exch{"H151 MPX"}="1H15 MPX"; $resn_exch{"H152 MPX"}="2H15 MPX";
$resn_exch{"H181 MPY"}="1H18 MPY"; $resn_exch{"H182 MPY"}="2H18 MPY"; $resn_exch{"H191 MPY"}="1H19 MPY";
$resn_exch{"H192 MPY"}="2H19 MPY"; $resn_exch{"H211 MPY"}="1H21 MPY"; $resn_exch{"H212 MPY"}="2H21 MPY";
$resn_exch{"H221 MPY"}="1H22 MPY"; $resn_exch{"H222 MPY"}="2H22 MPY"; $resn_exch{"H231 MPY"}="1H23 MPY";
$resn_exch{"H232 MPY"}="2H23 MPY"; $resn_exch{"H233 MPY"}="3H23 MPY"; $resn_exch{"H321 MPY"}="1H32 MPY";
$resn_exch{"H322 MPY"}="2H32 MPY"; $resn_exch{"H341 MPY"}="1H34 MPY"; $resn_exch{"H342 MPY"}="2H34 MPY";
$resn_exch{"H361 MPY"}="1H36 MPY"; $resn_exch{"H362 MPY"}="2H36 MPY"; $resn_exch{"H363 MPY"}="3H36 MPY";
$resn_exch{"H371 MPY"}="1H37 MPY"; $resn_exch{"H372 MPY"}="2H37 MPY"; $resn_exch{"H373 MPY"}="3H37 MPY";
$resn_exch{"H081 MPZ"}="1H08 MPZ"; $resn_exch{"H082 MPZ"}="2H08 MPZ"; $resn_exch{"H061 MPZ"}="1H06 MPZ";
$resn_exch{"H062 MPZ"}="2H06 MPZ"; $resn_exch{"H051 MPZ"}="1H05 MPZ"; $resn_exch{"H052 MPZ"}="2H05 MPZ";
$resn_exch{"H041 MPZ"}="1H04 MPZ"; $resn_exch{"H042 MPZ"}="2H04 MPZ"; $resn_exch{"H031 MPZ"}="1H03 MPZ";
$resn_exch{"H032 MPZ"}="2H03 MPZ"; $resn_exch{"H011 MPZ"}="1H01 MPZ"; $resn_exch{"H012 MPZ"}="2H01 MPZ";
$resn_exch{"H013 MPZ"}="3H01 MPZ"; $resn_exch{"H201 MQ0"}="1H20 MQ0"; $resn_exch{"H202 MQ0"}="2H20 MQ0";
$resn_exch{"HN91 MQ0"}="1HN9 MQ0"; $resn_exch{"HN92 MQ0"}="2HN9 MQ0"; $resn_exch{" H71 MQ1"}="1H7  MQ1";
$resn_exch{" H72 MQ1"}="2H7  MQ1"; $resn_exch{"H141 MQ1"}="1H14 MQ1"; $resn_exch{"H142 MQ1"}="2H14 MQ1";
$resn_exch{"H331 MQ1"}="1H33 MQ1"; $resn_exch{"H332 MQ1"}="2H33 MQ1"; $resn_exch{"H333 MQ1"}="3H33 MQ1";
$resn_exch{"H2M1 MQ7"}="1H2M MQ7"; $resn_exch{"H2M2 MQ7"}="2H2M MQ7"; $resn_exch{"H2M3 MQ7"}="3H2M MQ7";
$resn_exch{"H111 MQ7"}="1H11 MQ7"; $resn_exch{"H112 MQ7"}="2H11 MQ7"; $resn_exch{"H141 MQ7"}="1H14 MQ7";
$resn_exch{"H142 MQ7"}="2H14 MQ7"; $resn_exch{"H143 MQ7"}="3H14 MQ7"; $resn_exch{"H151 MQ7"}="1H15 MQ7";
$resn_exch{"H152 MQ7"}="2H15 MQ7"; $resn_exch{"H161 MQ7"}="1H16 MQ7"; $resn_exch{"H162 MQ7"}="2H16 MQ7";
$resn_exch{"H191 MQ7"}="1H19 MQ7"; $resn_exch{"H192 MQ7"}="2H19 MQ7"; $resn_exch{"H193 MQ7"}="3H19 MQ7";
$resn_exch{"H201 MQ7"}="1H20 MQ7"; $resn_exch{"H202 MQ7"}="2H20 MQ7"; $resn_exch{"H211 MQ7"}="1H21 MQ7";
$resn_exch{"H212 MQ7"}="2H21 MQ7"; $resn_exch{"H241 MQ7"}="1H24 MQ7"; $resn_exch{"H242 MQ7"}="2H24 MQ7";
$resn_exch{"H243 MQ7"}="3H24 MQ7"; $resn_exch{"H251 MQ7"}="1H25 MQ7"; $resn_exch{"H252 MQ7"}="2H25 MQ7";
$resn_exch{"H261 MQ7"}="1H26 MQ7"; $resn_exch{"H262 MQ7"}="2H26 MQ7"; $resn_exch{"H291 MQ7"}="1H29 MQ7";
$resn_exch{"H292 MQ7"}="2H29 MQ7"; $resn_exch{"H293 MQ7"}="3H29 MQ7"; $resn_exch{"H301 MQ7"}="1H30 MQ7";
$resn_exch{"H302 MQ7"}="2H30 MQ7"; $resn_exch{"H311 MQ7"}="1H31 MQ7"; $resn_exch{"H312 MQ7"}="2H31 MQ7";
$resn_exch{"H341 MQ7"}="1H34 MQ7"; $resn_exch{"H342 MQ7"}="2H34 MQ7"; $resn_exch{"H343 MQ7"}="3H34 MQ7";
$resn_exch{"H351 MQ7"}="1H35 MQ7"; $resn_exch{"H352 MQ7"}="2H35 MQ7"; $resn_exch{"H361 MQ7"}="1H36 MQ7";
$resn_exch{"H362 MQ7"}="2H36 MQ7"; $resn_exch{"H391 MQ7"}="1H39 MQ7"; $resn_exch{"H392 MQ7"}="2H39 MQ7";
$resn_exch{"H393 MQ7"}="3H39 MQ7"; $resn_exch{"H401 MQ7"}="1H40 MQ7"; $resn_exch{"H402 MQ7"}="2H40 MQ7";
$resn_exch{"H411 MQ7"}="1H41 MQ7"; $resn_exch{"H412 MQ7"}="2H41 MQ7"; $resn_exch{"H441 MQ7"}="1H44 MQ7";
$resn_exch{"H442 MQ7"}="2H44 MQ7"; $resn_exch{"H443 MQ7"}="3H44 MQ7"; $resn_exch{"H451 MQ7"}="1H45 MQ7";
$resn_exch{"H452 MQ7"}="2H45 MQ7"; $resn_exch{"H453 MQ7"}="3H45 MQ7"; $resn_exch{"H2M1 MQ8"}="1H2M MQ8";
$resn_exch{"H2M2 MQ8"}="2H2M MQ8"; $resn_exch{"H2M3 MQ8"}="3H2M MQ8"; $resn_exch{" H61 MQ8"}="1H6  MQ8";
$resn_exch{" H71 MQ8"}="1H7  MQ8"; $resn_exch{" H81 MQ8"}="1H8  MQ8"; $resn_exch{" H91 MQ8"}="1H9  MQ8";
$resn_exch{"H111 MQ8"}="1H11 MQ8"; $resn_exch{"H112 MQ8"}="2H11 MQ8"; $resn_exch{"H121 MQ8"}="1H12 MQ8";
$resn_exch{"H141 MQ8"}="1H14 MQ8"; $resn_exch{"H142 MQ8"}="2H14 MQ8"; $resn_exch{"H143 MQ8"}="3H14 MQ8";
$resn_exch{"H151 MQ8"}="1H15 MQ8"; $resn_exch{"H152 MQ8"}="2H15 MQ8"; $resn_exch{"H161 MQ8"}="1H16 MQ8";
$resn_exch{"H162 MQ8"}="2H16 MQ8"; $resn_exch{"H171 MQ8"}="1H17 MQ8"; $resn_exch{"H191 MQ8"}="1H19 MQ8";
$resn_exch{"H192 MQ8"}="2H19 MQ8"; $resn_exch{"H193 MQ8"}="3H19 MQ8"; $resn_exch{"H201 MQ8"}="1H20 MQ8";
$resn_exch{"H202 MQ8"}="2H20 MQ8"; $resn_exch{"H211 MQ8"}="1H21 MQ8"; $resn_exch{"H212 MQ8"}="2H21 MQ8";
$resn_exch{"H221 MQ8"}="1H22 MQ8"; $resn_exch{"H241 MQ8"}="1H24 MQ8"; $resn_exch{"H242 MQ8"}="2H24 MQ8";
$resn_exch{"H243 MQ8"}="3H24 MQ8"; $resn_exch{"H251 MQ8"}="1H25 MQ8"; $resn_exch{"H252 MQ8"}="2H25 MQ8";
$resn_exch{"H261 MQ8"}="1H26 MQ8"; $resn_exch{"H262 MQ8"}="2H26 MQ8"; $resn_exch{"H271 MQ8"}="1H27 MQ8";
$resn_exch{"H291 MQ8"}="1H29 MQ8"; $resn_exch{"H292 MQ8"}="2H29 MQ8"; $resn_exch{"H293 MQ8"}="3H29 MQ8";
$resn_exch{"H301 MQ8"}="1H30 MQ8"; $resn_exch{"H302 MQ8"}="2H30 MQ8"; $resn_exch{"H311 MQ8"}="1H31 MQ8";
$resn_exch{"H312 MQ8"}="2H31 MQ8"; $resn_exch{"H321 MQ8"}="1H32 MQ8"; $resn_exch{"H341 MQ8"}="1H34 MQ8";
$resn_exch{"H342 MQ8"}="2H34 MQ8"; $resn_exch{"H343 MQ8"}="3H34 MQ8"; $resn_exch{"H351 MQ8"}="1H35 MQ8";
$resn_exch{"H352 MQ8"}="2H35 MQ8"; $resn_exch{"H361 MQ8"}="1H36 MQ8"; $resn_exch{"H362 MQ8"}="2H36 MQ8";
$resn_exch{"H371 MQ8"}="1H37 MQ8"; $resn_exch{"H391 MQ8"}="1H39 MQ8"; $resn_exch{"H392 MQ8"}="2H39 MQ8";
$resn_exch{"H393 MQ8"}="3H39 MQ8"; $resn_exch{"H401 MQ8"}="1H40 MQ8"; $resn_exch{"H402 MQ8"}="2H40 MQ8";
$resn_exch{"H411 MQ8"}="1H41 MQ8"; $resn_exch{"H412 MQ8"}="2H41 MQ8"; $resn_exch{"H421 MQ8"}="1H42 MQ8";
$resn_exch{"H441 MQ8"}="1H44 MQ8"; $resn_exch{"H442 MQ8"}="2H44 MQ8"; $resn_exch{"H451 MQ8"}="1H45 MQ8";
$resn_exch{"H452 MQ8"}="2H45 MQ8"; $resn_exch{"H453 MQ8"}="3H45 MQ8"; $resn_exch{"H461 MQ8"}="1H46 MQ8";
$resn_exch{"H462 MQ8"}="2H46 MQ8"; $resn_exch{"H471 MQ8"}="1H47 MQ8"; $resn_exch{"H491 MQ8"}="1H49 MQ8";
$resn_exch{"H492 MQ8"}="2H49 MQ8"; $resn_exch{"H493 MQ8"}="3H49 MQ8"; $resn_exch{"H501 MQ8"}="1H50 MQ8";
$resn_exch{"H502 MQ8"}="2H50 MQ8"; $resn_exch{"H503 MQ8"}="3H50 MQ8"; $resn_exch{"H5M3 MQ9"}="3H5M MQ9";
$resn_exch{"H5M2 MQ9"}="2H5M MQ9"; $resn_exch{"H5M1 MQ9"}="1H5M MQ9"; $resn_exch{" H72 MQ9"}="2H7  MQ9";
$resn_exch{" H71 MQ9"}="1H7  MQ9"; $resn_exch{"H103 MQ9"}="3H10 MQ9"; $resn_exch{"H102 MQ9"}="2H10 MQ9";
$resn_exch{"H101 MQ9"}="1H10 MQ9"; $resn_exch{"H112 MQ9"}="2H11 MQ9"; $resn_exch{"H111 MQ9"}="1H11 MQ9";
$resn_exch{"H121 MQ9"}="1H12 MQ9"; $resn_exch{"H122 MQ9"}="2H12 MQ9"; $resn_exch{"H153 MQ9"}="3H15 MQ9";
$resn_exch{"H152 MQ9"}="2H15 MQ9"; $resn_exch{"H151 MQ9"}="1H15 MQ9"; $resn_exch{"H162 MQ9"}="2H16 MQ9";
$resn_exch{"H161 MQ9"}="1H16 MQ9"; $resn_exch{"H172 MQ9"}="2H17 MQ9"; $resn_exch{"H171 MQ9"}="1H17 MQ9";
$resn_exch{"H203 MQ9"}="3H20 MQ9"; $resn_exch{"H202 MQ9"}="2H20 MQ9"; $resn_exch{"H201 MQ9"}="1H20 MQ9";
$resn_exch{"H212 MQ9"}="2H21 MQ9"; $resn_exch{"H211 MQ9"}="1H21 MQ9"; $resn_exch{"H222 MQ9"}="2H22 MQ9";
$resn_exch{"H221 MQ9"}="1H22 MQ9"; $resn_exch{"H253 MQ9"}="3H25 MQ9"; $resn_exch{"H252 MQ9"}="2H25 MQ9";
$resn_exch{"H251 MQ9"}="1H25 MQ9"; $resn_exch{"H262 MQ9"}="2H26 MQ9"; $resn_exch{"H261 MQ9"}="1H26 MQ9";
$resn_exch{"H272 MQ9"}="2H27 MQ9"; $resn_exch{"H271 MQ9"}="1H27 MQ9"; $resn_exch{"H303 MQ9"}="3H30 MQ9";
$resn_exch{"H302 MQ9"}="2H30 MQ9"; $resn_exch{"H301 MQ9"}="1H30 MQ9"; $resn_exch{"H312 MQ9"}="2H31 MQ9";
$resn_exch{"H311 MQ9"}="1H31 MQ9"; $resn_exch{"H321 MQ9"}="1H32 MQ9"; $resn_exch{"H322 MQ9"}="2H32 MQ9";
$resn_exch{"H353 MQ9"}="3H35 MQ9"; $resn_exch{"H352 MQ9"}="2H35 MQ9"; $resn_exch{"H351 MQ9"}="1H35 MQ9";
$resn_exch{"H362 MQ9"}="2H36 MQ9"; $resn_exch{"H361 MQ9"}="1H36 MQ9"; $resn_exch{"H372 MQ9"}="2H37 MQ9";
$resn_exch{"H371 MQ9"}="1H37 MQ9"; $resn_exch{"H403 MQ9"}="3H40 MQ9"; $resn_exch{"H402 MQ9"}="2H40 MQ9";
$resn_exch{"H401 MQ9"}="1H40 MQ9"; $resn_exch{"H412 MQ9"}="2H41 MQ9"; $resn_exch{"H411 MQ9"}="1H41 MQ9";
$resn_exch{"H422 MQ9"}="2H42 MQ9"; $resn_exch{"H421 MQ9"}="1H42 MQ9"; $resn_exch{"H453 MQ9"}="3H45 MQ9";
$resn_exch{"H452 MQ9"}="2H45 MQ9"; $resn_exch{"H451 MQ9"}="1H45 MQ9"; $resn_exch{"H462 MQ9"}="2H46 MQ9";
$resn_exch{"H461 MQ9"}="1H46 MQ9"; $resn_exch{"H472 MQ9"}="2H47 MQ9"; $resn_exch{"H471 MQ9"}="1H47 MQ9";
$resn_exch{"H503 MQ9"}="3H50 MQ9"; $resn_exch{"H502 MQ9"}="2H50 MQ9"; $resn_exch{"H501 MQ9"}="1H50 MQ9";
$resn_exch{"H513 MQ9"}="3H51 MQ9"; $resn_exch{"H512 MQ9"}="2H51 MQ9"; $resn_exch{"H511 MQ9"}="1H51 MQ9";
$resn_exch{"H1C1 MQD"}="1H1C MQD"; $resn_exch{"H1C2 MQD"}="2H1C MQD"; $resn_exch{"H1C3 MQD"}="3H1C MQD";
$resn_exch{"HMC1 MQD"}="1HMC MQD"; $resn_exch{"HMC2 MQD"}="2HMC MQD"; $resn_exch{"H3C1 MQD"}="1H3C MQD";
$resn_exch{"H3C2 MQD"}="2H3C MQD"; $resn_exch{"H5C1 MQD"}="1H5C MQD"; $resn_exch{"H5C2 MQD"}="2H5C MQD";
$resn_exch{"H5C3 MQD"}="3H5C MQD"; $resn_exch{"H101 MQI"}="1H10 MQI"; $resn_exch{"H102 MQI"}="2H10 MQI";
$resn_exch{"H103 MQI"}="3H10 MQI"; $resn_exch{"H21A MQQ"}="1H2  MQQ"; $resn_exch{" H22 MQQ"}="2H2  MQQ";
$resn_exch{"H31A MQQ"}="1H3  MQQ"; $resn_exch{" H32 MQQ"}="2H3  MQQ"; $resn_exch{" H41 MQQ"}="1H4  MQQ";
$resn_exch{" H42 MQQ"}="2H4  MQQ"; $resn_exch{"H51A MQQ"}="1H5  MQQ"; $resn_exch{" H52 MQQ"}="2H5  MQQ";
$resn_exch{" HB1 MQQ"}="1HB  MQQ"; $resn_exch{" HB2 MQQ"}="2HB  MQQ"; $resn_exch{" HG1 MQQ"}="1HG  MQQ";
$resn_exch{" HG2 MQQ"}="2HG  MQQ"; $resn_exch{" HM1 MQQ"}="1HM  MQQ"; $resn_exch{" HM2 MQQ"}="2HM  MQQ";
$resn_exch{"HB31 MQQ"}="1HB3 MQQ"; $resn_exch{"HB32 MQQ"}="2HB3 MQQ"; $resn_exch{"HB21 MQQ"}="1HB2 MQQ";
$resn_exch{"HB22 MQQ"}="2HB2 MQQ"; $resn_exch{" HT1 MQQ"}="1HT  MQQ"; $resn_exch{" HT2 MQQ"}="2HT  MQQ";
$resn_exch{" H21 MQU"}="1H2  MQU"; $resn_exch{" H22 MQU"}="2H2  MQU"; $resn_exch{"H141 MQU"}="1H14 MQU";
$resn_exch{"H142 MQU"}="2H14 MQU"; $resn_exch{"H143 MQU"}="3H14 MQU"; $resn_exch{" H51 MQU"}="1H5  MQU";
$resn_exch{" H52 MQU"}="2H5  MQU"; $resn_exch{"H181 MQU"}="1H18 MQU"; $resn_exch{"H182 MQU"}="2H18 MQU";
$resn_exch{"H183 MQU"}="3H18 MQU"; $resn_exch{"H251 MQU"}="1H25 MQU"; $resn_exch{"H252 MQU"}="2H25 MQU";
$resn_exch{"H261 MQU"}="1H26 MQU"; $resn_exch{"H262 MQU"}="2H26 MQU"; $resn_exch{"H101 MR0"}="1H10 MR0";
$resn_exch{"H102 MR0"}="2H10 MR0"; $resn_exch{"H131 MR0"}="1H13 MR0"; $resn_exch{"H132 MR0"}="2H13 MR0";
$resn_exch{"H171 MR0"}="1H17 MR0"; $resn_exch{"H172 MR0"}="2H17 MR0"; $resn_exch{" H91 MR0"}="1H9  MR0";
$resn_exch{" H92 MR0"}="2H9  MR0"; $resn_exch{"H111 MR0"}="1H11 MR0"; $resn_exch{"H112 MR0"}="2H11 MR0";
$resn_exch{"H121 MR0"}="1H12 MR0"; $resn_exch{"H122 MR0"}="2H12 MR0"; $resn_exch{"H123 MR0"}="3H12 MR0";
$resn_exch{"H141 MR0"}="1H14 MR0"; $resn_exch{"H142 MR0"}="2H14 MR0"; $resn_exch{"H143 MR0"}="3H14 MR0";
$resn_exch{"H181 MR0"}="1H18 MR0"; $resn_exch{"H182 MR0"}="2H18 MR0"; $resn_exch{"H191 MR0"}="1H19 MR0";
$resn_exch{"H192 MR0"}="2H19 MR0"; $resn_exch{" HO1 MR0"}="1HO  MR0"; $resn_exch{"HO2A MR0"}="2HO  MR0";
$resn_exch{" HO3 MR0"}="3HO  MR0"; $resn_exch{"H7A1 MR0"}="1H7A MR0"; $resn_exch{"H7A2 MR0"}="2H7A MR0";
$resn_exch{"H381 MR1"}="1H38 MR1"; $resn_exch{"H382 MR1"}="2H38 MR1"; $resn_exch{"H301 MR1"}="1H30 MR1";
$resn_exch{"H302 MR1"}="2H30 MR1"; $resn_exch{" HN1 MR1"}="1HN  MR1"; $resn_exch{" HN2 MR1"}="2HN  MR1";
$resn_exch{"HAA1 MR3"}="1HAA MR3"; $resn_exch{"HAA2 MR3"}="2HAA MR3"; $resn_exch{"HAA3 MR3"}="3HAA MR3";
$resn_exch{"HAA1 MR4"}="1HAA MR4"; $resn_exch{"HAA2 MR4"}="2HAA MR4"; $resn_exch{"HAA3 MR4"}="3HAA MR4";
$resn_exch{"HAB1 MR4"}="1HAB MR4"; $resn_exch{"HAB2 MR4"}="2HAB MR4"; $resn_exch{"HAB3 MR4"}="3HAB MR4";
$resn_exch{"HAA1 MR6"}="1HAA MR6"; $resn_exch{"HAA2 MR6"}="2HAA MR6"; $resn_exch{"HAA3 MR6"}="3HAA MR6";
$resn_exch{"HAB1 MR6"}="1HAB MR6"; $resn_exch{"HAB2 MR6"}="2HAB MR6"; $resn_exch{"HAB3 MR6"}="3HAB MR6";
$resn_exch{"H261 MR9"}="1H26 MR9"; $resn_exch{"H262 MR9"}="2H26 MR9"; $resn_exch{"H271 MR9"}="1H27 MR9";
$resn_exch{"H272 MR9"}="2H27 MR9"; $resn_exch{"H281 MR9"}="1H28 MR9"; $resn_exch{"H282 MR9"}="2H28 MR9";
$resn_exch{"H291 MR9"}="1H29 MR9"; $resn_exch{"H292 MR9"}="2H29 MR9"; $resn_exch{"H161 MR9"}="1H16 MR9";
$resn_exch{"H162 MR9"}="2H16 MR9"; $resn_exch{"H163 MR9"}="3H16 MR9"; $resn_exch{"H181 MR9"}="1H18 MR9";
$resn_exch{"H182 MR9"}="2H18 MR9"; $resn_exch{"H183 MR9"}="3H18 MR9"; $resn_exch{"H161 MRA"}="1H16 MRA";
$resn_exch{"H162 MRA"}="2H16 MRA"; $resn_exch{"H171 MRA"}="1H17 MRA"; $resn_exch{"H172 MRA"}="2H17 MRA";
$resn_exch{" H41 MRC"}="1H4  MRC"; $resn_exch{" H42 MRC"}="2H4  MRC"; $resn_exch{" H91 MRC"}="1H9  MRC";
$resn_exch{" H92 MRC"}="2H9  MRC"; $resn_exch{"H141 MRC"}="1H14 MRC"; $resn_exch{"H142 MRC"}="2H14 MRC";
$resn_exch{"H143 MRC"}="3H14 MRC"; $resn_exch{"H151 MRC"}="1H15 MRC"; $resn_exch{"H152 MRC"}="2H15 MRC";
$resn_exch{"H153 MRC"}="3H15 MRC"; $resn_exch{"H161 MRC"}="1H16 MRC"; $resn_exch{"H162 MRC"}="2H16 MRC";
$resn_exch{"HOP1 MRC"}="1HOP MRC"; $resn_exch{"H2'1 MRC"}="1H2* MRC"; $resn_exch{"H2'2 MRC"}="2H2* MRC";
$resn_exch{"H3'1 MRC"}="1H3* MRC"; $resn_exch{"H3'2 MRC"}="2H3* MRC"; $resn_exch{"H4'1 MRC"}="1H4* MRC";
$resn_exch{"H4'2 MRC"}="2H4* MRC"; $resn_exch{"H5'1 MRC"}="1H5* MRC"; $resn_exch{"H5'2 MRC"}="2H5* MRC";
$resn_exch{"H6'1 MRC"}="1H6* MRC"; $resn_exch{"H6'2 MRC"}="2H6* MRC"; $resn_exch{"H7'1 MRC"}="1H7* MRC";
$resn_exch{"H7'2 MRC"}="2H7* MRC"; $resn_exch{"H8'1 MRC"}="1H8* MRC"; $resn_exch{"H8'2 MRC"}="2H8* MRC";
$resn_exch{"H9'1 MRC"}="1H9* MRC"; $resn_exch{"H9'2 MRC"}="2H9* MRC"; $resn_exch{"H171 MRC"}="1H17 MRC";
$resn_exch{"H172 MRC"}="2H17 MRC"; $resn_exch{"H173 MRC"}="3H17 MRC"; $resn_exch{"H1C1 MRD"}="1H1C MRD";
$resn_exch{"H1C2 MRD"}="2H1C MRD"; $resn_exch{"H1C3 MRD"}="3H1C MRD"; $resn_exch{"HMC1 MRD"}="1HMC MRD";
$resn_exch{"HMC2 MRD"}="2HMC MRD"; $resn_exch{"HMC3 MRD"}="3HMC MRD"; $resn_exch{"H3C1 MRD"}="1H3C MRD";
$resn_exch{"H3C2 MRD"}="2H3C MRD"; $resn_exch{"H5C1 MRD"}="1H5C MRD"; $resn_exch{"H5C2 MRD"}="2H5C MRD";
$resn_exch{"H5C3 MRD"}="3H5C MRD"; $resn_exch{"HHX1 MRE"}="1HHX MRE"; $resn_exch{"HHX2 MRE"}="2HHX MRE";
$resn_exch{"HHX3 MRE"}="3HHX MRE"; $resn_exch{"HHZ1 MRE"}="1HHZ MRE"; $resn_exch{"HHZ2 MRE"}="2HHZ MRE";
$resn_exch{"HHZ3 MRE"}="3HHZ MRE"; $resn_exch{"HOP2 MRG"}="2HOP MRG"; $resn_exch{" H21 MRG"}="1H2  MRG";
$resn_exch{"H211 MRG"}="1H21 MRG"; $resn_exch{"H212 MRG"}="2H21 MRG"; $resn_exch{"H221 MRG"}="1H22 MRG";
$resn_exch{"H222 MRG"}="2H22 MRG"; $resn_exch{"H231 MRG"}="1H23 MRG"; $resn_exch{"H232 MRG"}="2H23 MRG";
$resn_exch{" H2' MRG"}="1H2* MRG"; $resn_exch{"H2'' MRG"}="2H2* MRG"; $resn_exch{" H5' MRG"}="1H5* MRG";
$resn_exch{"H5'' MRG"}="2H5* MRG"; $resn_exch{" H4' MRG"}=" H4* MRG"; $resn_exch{" H1' MRG"}=" H1* MRG";
$resn_exch{" H3' MRG"}=" H3* MRG"; $resn_exch{"HO3' MRG"}="H3T  MRG"; $resn_exch{"HOP3 MRG"}="3HOP MRG";
$resn_exch{"H221 MRK"}="1H22 MRK"; $resn_exch{"H222 MRK"}="2H22 MRK"; $resn_exch{"H223 MRK"}="3H22 MRK";
$resn_exch{"H231 MRK"}="1H23 MRK"; $resn_exch{"H232 MRK"}="2H23 MRK"; $resn_exch{"HNB1 MRM"}="1HNB MRM";
$resn_exch{"HNB2 MRM"}="2HNB MRM"; $resn_exch{"HM21 MRM"}="1HM2 MRM"; $resn_exch{"HM22 MRM"}="2HM2 MRM";
$resn_exch{"HM51 MRM"}="1HM5 MRM"; $resn_exch{"HM52 MRM"}="2HM5 MRM"; $resn_exch{"HM31 MRM"}="1HM3 MRM";
$resn_exch{"HM32 MRM"}="2HM3 MRM"; $resn_exch{"HM41 MRM"}="1HM4 MRM"; $resn_exch{"HM42 MRM"}="2HM4 MRM";
$resn_exch{" H61 MRP"}="1H6  MRP"; $resn_exch{" H62 MRP"}="2H6  MRP"; $resn_exch{" H63 MRP"}="3H6  MRP";
$resn_exch{" H71 MRP"}="1H7  MRP"; $resn_exch{" H72 MRP"}="2H7  MRP"; $resn_exch{" H73 MRP"}="3H7  MRP";
$resn_exch{" H3' MRR"}=" H3* MRR"; $resn_exch{" H2' MRR"}=" H2* MRR"; $resn_exch{" H1' MRR"}=" H1* MRR";
$resn_exch{"HN61 MRR"}="1HN6 MRR"; $resn_exch{"HN62 MRR"}="2HN6 MRR"; $resn_exch{" H4' MRR"}=" H4* MRR";
$resn_exch{"H5'1 MRR"}="1H5* MRR"; $resn_exch{"H5'2 MRR"}="2H5* MRR"; $resn_exch{"H201 MRR"}="1H20 MRR";
$resn_exch{"H202 MRR"}="2H20 MRR"; $resn_exch{"H181 MRR"}="1H18 MRR"; $resn_exch{"H182 MRR"}="2H18 MRR";
$resn_exch{"H183 MRR"}="3H18 MRR"; $resn_exch{"H191 MRR"}="1H19 MRR"; $resn_exch{"H192 MRR"}="2H19 MRR";
$resn_exch{"H193 MRR"}="3H19 MRR"; $resn_exch{"H171 MRR"}="1H17 MRR"; $resn_exch{"H172 MRR"}="2H17 MRR";
$resn_exch{"H161 MRR"}="1H16 MRR"; $resn_exch{"H162 MRR"}="2H16 MRR"; $resn_exch{" H11 MRR"}="1H1  MRR";
$resn_exch{" H12 MRR"}="2H1  MRR"; $resn_exch{" H21 MRR"}="1H2  MRR"; $resn_exch{" H22 MRR"}="2H2  MRR";
$resn_exch{"H131 MRR"}="1H13 MRR"; $resn_exch{"H132 MRR"}="2H13 MRR"; $resn_exch{"H133 MRR"}="3H13 MRR";
$resn_exch{" H31 MRR"}="1H3  MRR"; $resn_exch{" H32 MRR"}="2H3  MRR"; $resn_exch{" H41 MRR"}="1H4  MRR";
$resn_exch{" H42 MRR"}="2H4  MRR"; $resn_exch{" H51 MRR"}="1H5  MRR"; $resn_exch{" H52 MRR"}="2H5  MRR";
$resn_exch{" H61 MRR"}="1H6  MRR"; $resn_exch{" H62 MRR"}="2H6  MRR"; $resn_exch{" H71 MRR"}="1H7  MRR";
$resn_exch{" H72 MRR"}="2H7  MRR"; $resn_exch{" H81 MRR"}="1H8  MRR"; $resn_exch{" H82 MRR"}="2H8  MRR";
$resn_exch{" H91 MRR"}="1H9  MRR"; $resn_exch{" H92 MRR"}="2H9  MRR"; $resn_exch{"H101 MRR"}="1H10 MRR";
$resn_exch{"H102 MRR"}="2H10 MRR"; $resn_exch{"H111 MRR"}="1H11 MRR"; $resn_exch{"H112 MRR"}="2H11 MRR";
$resn_exch{"H121 MRR"}="1H12 MRR"; $resn_exch{"H122 MRR"}="2H12 MRR"; $resn_exch{"H141 MRR"}="1H14 MRR";
$resn_exch{"H142 MRR"}="2H14 MRR"; $resn_exch{"H151 MRR"}="1H15 MRR"; $resn_exch{"H152 MRR"}="2H15 MRR";
$resn_exch{"H153 MRR"}="3H15 MRR"; $resn_exch{" H3' MRS"}=" H3* MRS"; $resn_exch{" H2' MRS"}=" H2* MRS";
$resn_exch{" H1' MRS"}=" H1* MRS"; $resn_exch{"HN61 MRS"}="1HN6 MRS"; $resn_exch{"HN62 MRS"}="2HN6 MRS";
$resn_exch{" H4' MRS"}=" H4* MRS"; $resn_exch{"H5'1 MRS"}="1H5* MRS"; $resn_exch{"H5'2 MRS"}="2H5* MRS";
$resn_exch{"H201 MRS"}="1H20 MRS"; $resn_exch{"H202 MRS"}="2H20 MRS"; $resn_exch{"H181 MRS"}="1H18 MRS";
$resn_exch{"H182 MRS"}="2H18 MRS"; $resn_exch{"H183 MRS"}="3H18 MRS"; $resn_exch{"H191 MRS"}="1H19 MRS";
$resn_exch{"H192 MRS"}="2H19 MRS"; $resn_exch{"H193 MRS"}="3H19 MRS"; $resn_exch{"H171 MRS"}="1H17 MRS";
$resn_exch{"H172 MRS"}="2H17 MRS"; $resn_exch{"H161 MRS"}="1H16 MRS"; $resn_exch{"H162 MRS"}="2H16 MRS";
$resn_exch{" H11 MRS"}="1H1  MRS"; $resn_exch{" H12 MRS"}="2H1  MRS"; $resn_exch{" H21 MRS"}="1H2  MRS";
$resn_exch{" H22 MRS"}="2H2  MRS"; $resn_exch{"H131 MRS"}="1H13 MRS"; $resn_exch{"H132 MRS"}="2H13 MRS";
$resn_exch{"H133 MRS"}="3H13 MRS"; $resn_exch{" H31 MRS"}="1H3  MRS"; $resn_exch{" H32 MRS"}="2H3  MRS";
$resn_exch{" H41 MRS"}="1H4  MRS"; $resn_exch{" H42 MRS"}="2H4  MRS"; $resn_exch{" H51 MRS"}="1H5  MRS";
$resn_exch{" H52 MRS"}="2H5  MRS"; $resn_exch{" H61 MRS"}="1H6  MRS"; $resn_exch{" H62 MRS"}="2H6  MRS";
$resn_exch{" H71 MRS"}="1H7  MRS"; $resn_exch{" H72 MRS"}="2H7  MRS"; $resn_exch{" H81 MRS"}="1H8  MRS";
$resn_exch{" H82 MRS"}="2H8  MRS"; $resn_exch{" H91 MRS"}="1H9  MRS"; $resn_exch{" H92 MRS"}="2H9  MRS";
$resn_exch{"H101 MRS"}="1H10 MRS"; $resn_exch{"H102 MRS"}="2H10 MRS"; $resn_exch{"H111 MRS"}="1H11 MRS";
$resn_exch{"H112 MRS"}="2H11 MRS"; $resn_exch{"H121 MRS"}="1H12 MRS"; $resn_exch{"H122 MRS"}="2H12 MRS";
$resn_exch{"H141 MRS"}="1H14 MRS"; $resn_exch{"H142 MRS"}="2H14 MRS"; $resn_exch{"H151 MRS"}="1H15 MRS";
$resn_exch{"H152 MRS"}="2H15 MRS"; $resn_exch{"H153 MRS"}="3H15 MRS"; $resn_exch{" H11 MRU"}="1H1  MRU";
$resn_exch{" H12 MRU"}="2H1  MRU"; $resn_exch{" H51 MRU"}="1H5  MRU"; $resn_exch{" H52 MRU"}="2H5  MRU";
$resn_exch{" H61 MRU"}="1H6  MRU"; $resn_exch{" H62 MRU"}="2H6  MRU"; $resn_exch{" H63 MRU"}="3H6  MRU";
$resn_exch{"H201 MRX"}="1H20 MRX"; $resn_exch{"H202 MRX"}="2H20 MRX"; $resn_exch{"H151 MRX"}="1H15 MRX";
$resn_exch{"H152 MRX"}="2H15 MRX"; $resn_exch{"H141 MRX"}="1H14 MRX"; $resn_exch{"H142 MRX"}="2H14 MRX";
$resn_exch{"H131 MRX"}="1H13 MRX"; $resn_exch{"H132 MRX"}="2H13 MRX"; $resn_exch{"H121 MRX"}="1H12 MRX";
$resn_exch{"H122 MRX"}="2H12 MRX"; $resn_exch{"HAA1 MRY"}="1HAA MRY"; $resn_exch{"HAA2 MRY"}="2HAA MRY";
$resn_exch{"HAG1 MRY"}="1HAG MRY"; $resn_exch{"HAG2 MRY"}="2HAG MRY"; $resn_exch{" HG1 MS1"}="1HG  MS1";
$resn_exch{" HG2 MS1"}="2HG  MS1"; $resn_exch{" HB1 MS1"}="1HB  MS1"; $resn_exch{" HB2 MS1"}="2HB  MS1";
$resn_exch{" H51 MS1"}="1H5  MS1"; $resn_exch{" H52 MS1"}="2H5  MS1"; $resn_exch{"HN11 MS1"}="1HN1 MS1";
$resn_exch{"HN12 MS1"}="2HN1 MS1"; $resn_exch{" H91 MS1"}="1H9  MS1"; $resn_exch{" H92 MS1"}="2H9  MS1";
$resn_exch{"H2F1 MS1"}="1H2F MS1"; $resn_exch{"H2F2 MS1"}="2H2F MS1"; $resn_exch{" H61 MS2"}="1H6  MS2";
$resn_exch{" H62 MS2"}="2H6  MS2"; $resn_exch{" H63 MS2"}="3H6  MS2"; $resn_exch{"H121 MS2"}="1H12 MS2";
$resn_exch{"H122 MS2"}="2H12 MS2"; $resn_exch{"H123 MS2"}="3H12 MS2"; $resn_exch{"H131 MS3"}="1H13 MS3";
$resn_exch{"H132 MS3"}="2H13 MS3"; $resn_exch{"H241 MS3"}="1H24 MS3"; $resn_exch{"H242 MS3"}="2H24 MS3";
$resn_exch{"H243 MS3"}="3H24 MS3"; $resn_exch{"H251 MS3"}="1H25 MS3"; $resn_exch{"H252 MS3"}="2H25 MS3";
$resn_exch{"H253 MS3"}="3H25 MS3"; $resn_exch{"H291 MS3"}="1H29 MS3"; $resn_exch{"H292 MS3"}="2H29 MS3";
$resn_exch{"H311 MS3"}="1H31 MS3"; $resn_exch{"H312 MS3"}="2H31 MS3"; $resn_exch{"H391 MS3"}="1H39 MS3";
$resn_exch{"H392 MS3"}="2H39 MS3"; $resn_exch{"H393 MS3"}="3H39 MS3"; $resn_exch{"H401 MS3"}="1H40 MS3";
$resn_exch{"H402 MS3"}="2H40 MS3"; $resn_exch{"H403 MS3"}="3H40 MS3"; $resn_exch{"H441 MS3"}="1H44 MS3";
$resn_exch{"H442 MS3"}="2H44 MS3"; $resn_exch{" H   MSA"}=" HN  MSA"; $resn_exch{" HN1 MSA"}="1HN  MSA";
$resn_exch{" HN2 MSA"}="2HN  MSA"; $resn_exch{" HN3 MSA"}="3HN  MSA"; $resn_exch{" HG1 MSA"}="1HG  MSA";
$resn_exch{" HG2 MSA"}="2HG  MSA"; $resn_exch{" HG3 MSA"}="3HG  MSA"; $resn_exch{" HH1 MSB"}="1HH  MSB";
$resn_exch{" HH2 MSB"}="2HH  MSB"; $resn_exch{" HH3 MSB"}="3HH  MSB"; $resn_exch{"H011 MSC"}="1H01 MSC";
$resn_exch{"H021 MSC"}="1H02 MSC"; $resn_exch{"H031 MSC"}="1H03 MSC"; $resn_exch{"H051 MSC"}="1H05 MSC";
$resn_exch{"H052 MSC"}="2H05 MSC"; $resn_exch{"H071 MSC"}="1H07 MSC"; $resn_exch{"H081 MSC"}="1H08 MSC";
$resn_exch{"H091 MSC"}="1H09 MSC"; $resn_exch{"H101 MSC"}="1H10 MSC"; $resn_exch{"H111 MSC"}="1H11 MSC";
$resn_exch{"H141 MSC"}="1H14 MSC"; $resn_exch{"H151 MSC"}="1H15 MSC"; $resn_exch{"H171 MSC"}="1H17 MSC";
$resn_exch{"H181 MSC"}="1H18 MSC"; $resn_exch{"H191 MSC"}="1H19 MSC"; $resn_exch{"H201 MSC"}="1H20 MSC";
$resn_exch{"H221 MSC"}="1H22 MSC"; $resn_exch{"H222 MSC"}="2H22 MSC"; $resn_exch{"H231 MSC"}="1H23 MSC";
$resn_exch{"H241 MSC"}="1H24 MSC"; $resn_exch{"H311 MSC"}="1H31 MSC"; $resn_exch{"H321 MSC"}="1H32 MSC";
$resn_exch{"H331 MSC"}="1H33 MSC"; $resn_exch{"H351 MSC"}="1H35 MSC"; $resn_exch{"H352 MSC"}="2H35 MSC";
$resn_exch{"H371 MSC"}="1H37 MSC"; $resn_exch{"H381 MSC"}="1H38 MSC"; $resn_exch{"H391 MSC"}="1H39 MSC";
$resn_exch{"H401 MSC"}="1H40 MSC"; $resn_exch{"H411 MSC"}="1H41 MSC"; $resn_exch{"H441 MSC"}="1H44 MSC";
$resn_exch{"H451 MSC"}="1H45 MSC"; $resn_exch{"H452 MSC"}="2H45 MSC"; $resn_exch{"H481 MSC"}="1H48 MSC";
$resn_exch{"H491 MSC"}="1H49 MSC"; $resn_exch{"H501 MSC"}="1H50 MSC"; $resn_exch{"H11N MSD"}="NH11 MSD";
$resn_exch{"H12C MSD"}="CH12 MSD"; $resn_exch{"H131 MSD"}="1H13 MSD"; $resn_exch{"H132 MSD"}="2H13 MSD";
$resn_exch{"H14C MSD"}="CH14 MSD"; $resn_exch{"H18C MSD"}="CH18 MSD"; $resn_exch{"H19C MSD"}="CH19 MSD";
$resn_exch{"H22O MSD"}="OH22 MSD"; $resn_exch{"H251 MSD"}="1H25 MSD"; $resn_exch{"H252 MSD"}="2H25 MSD";
$resn_exch{"H253 MSD"}="3H25 MSD"; $resn_exch{"H441 MSD"}="1H44 MSD"; $resn_exch{"H442 MSD"}="2H44 MSD";
$resn_exch{"H443 MSD"}="3H44 MSD"; $resn_exch{" H   MSE"}="1HN  MSE"; $resn_exch{" HN2 MSE"}="2HN  MSE";
$resn_exch{" HB2 MSE"}="1HB  MSE"; $resn_exch{" HB3 MSE"}="2HB  MSE"; $resn_exch{" HG2 MSE"}="1HG  MSE";
$resn_exch{" HG3 MSE"}="2HG  MSE"; $resn_exch{" HE1 MSE"}="1HE  MSE"; $resn_exch{" HE2 MSE"}="2HE  MSE";
$resn_exch{" HE3 MSE"}="3HE  MSE"; $resn_exch{" H21 MSF"}="1H2  MSF"; $resn_exch{" H22 MSF"}="2H2  MSF";
$resn_exch{" H41 MSF"}="1H4  MSF"; $resn_exch{" H42 MSF"}="2H4  MSF"; $resn_exch{" H43 MSF"}="3H4  MSF";
$resn_exch{" H5' MSG"}=" H5* MSG"; $resn_exch{"H5'1 MSG"}="1H5* MSG"; $resn_exch{"H5'2 MSG"}="2H5* MSG";
$resn_exch{" H4' MSG"}=" H4* MSG"; $resn_exch{" H1' MSG"}=" H1* MSG"; $resn_exch{" H11 MSG"}="1H1  MSG";
$resn_exch{" H12 MSG"}="2H1  MSG"; $resn_exch{" H13 MSG"}="3H1  MSG"; $resn_exch{"HN21 MSG"}="1HN2 MSG";
$resn_exch{"HN22 MSG"}="2HN2 MSG"; $resn_exch{" H2' MSG"}=" H2* MSG"; $resn_exch{" H3' MSG"}=" H3* MSG";
$resn_exch{"HC41 MSH"}="1HC4 MSH"; $resn_exch{"HC42 MSH"}="2HC4 MSH"; $resn_exch{"HC43 MSH"}="3HC4 MSH";
$resn_exch{"HC51 MSH"}="1HC5 MSH"; $resn_exch{"HC52 MSH"}="2HC5 MSH"; $resn_exch{"HC53 MSH"}="3HC5 MSH";
$resn_exch{"HC61 MSH"}="1HC6 MSH"; $resn_exch{"HC62 MSH"}="2HC6 MSH"; $resn_exch{"HC71 MSH"}="1HC7 MSH";
$resn_exch{"HC81 MSH"}="1HC8 MSH"; $resn_exch{"HC82 MSH"}="2HC8 MSH"; $resn_exch{"HC91 MSH"}="1HC9 MSH";
$resn_exch{"HC92 MSH"}="2HC9 MSH"; $resn_exch{"H101 MSH"}="1H10 MSH"; $resn_exch{"H102 MSH"}="2H10 MSH";
$resn_exch{"H103 MSH"}="3H10 MSH"; $resn_exch{"H101 MSI"}="1H10 MSI"; $resn_exch{"H102 MSI"}="2H10 MSI";
$resn_exch{"H111 MSI"}="1H11 MSI"; $resn_exch{"H112 MSI"}="2H11 MSI"; $resn_exch{"H121 MSI"}="1H12 MSI";
$resn_exch{"H122 MSI"}="2H12 MSI"; $resn_exch{"H131 MSI"}="1H13 MSI"; $resn_exch{"H132 MSI"}="2H13 MSI";
$resn_exch{"HC11 MSI"}="1HC1 MSI"; $resn_exch{"HC12 MSI"}="2HC1 MSI"; $resn_exch{"HC13 MSI"}="3HC1 MSI";
$resn_exch{" H   MSL"}="1HN  MSL"; $resn_exch{" H2  MSL"}="2HN  MSL"; $resn_exch{" HB2 MSL"}="1HB  MSL";
$resn_exch{" HB3 MSL"}="2HB  MSL"; $resn_exch{" HG2 MSL"}="1HG  MSL"; $resn_exch{" HG3 MSL"}="2HG  MSL";
$resn_exch{" HE  MSL"}=" HNE MSL"; $resn_exch{" HE1 MSL"}="1HE  MSL"; $resn_exch{" HE2 MSL"}="2HE  MSL";
$resn_exch{" HE3 MSL"}="3HE  MSL"; $resn_exch{"HB11 MSM"}="1HB1 MSM"; $resn_exch{"HB12 MSM"}="2HB1 MSM";
$resn_exch{"HB13 MSM"}="3HB1 MSM"; $resn_exch{"HD11 MSM"}="1HD1 MSM"; $resn_exch{"HD12 MSM"}="2HD1 MSM";
$resn_exch{"HD13 MSM"}="3HD1 MSM"; $resn_exch{"HN51 MSN"}="1HN5 MSN"; $resn_exch{"HN52 MSN"}="2HN5 MSN";
$resn_exch{" H71 MSN"}="1H7  MSN"; $resn_exch{" H72 MSN"}="2H7  MSN"; $resn_exch{" H73 MSN"}="3H7  MSN";
$resn_exch{" H   MSO"}="1HN  MSO"; $resn_exch{" H2  MSO"}="2HN  MSO"; $resn_exch{" HB2 MSO"}="1HCB MSO";
$resn_exch{" HB3 MSO"}="2HCB MSO"; $resn_exch{" HG2 MSO"}="1HCG MSO"; $resn_exch{" HG3 MSO"}="2HCG MSO";
$resn_exch{" HE1 MSO"}="1HC  MSO"; $resn_exch{" HE2 MSO"}="2HC  MSO"; $resn_exch{" HE3 MSO"}="3HC  MSO";
$resn_exch{"HN21 MSP"}="1HN2 MSP"; $resn_exch{"HN22 MSP"}="2HN2 MSP"; $resn_exch{" HB1 MSP"}="1HB  MSP";
$resn_exch{" HB2 MSP"}="2HB  MSP"; $resn_exch{" HG1 MSP"}="1HG  MSP"; $resn_exch{" HG2 MSP"}="2HG  MSP";
$resn_exch{" HE1 MSP"}="1HE  MSP"; $resn_exch{" HE2 MSP"}="2HE  MSP"; $resn_exch{" HE3 MSP"}="3HE  MSP";
$resn_exch{"H5'1 MSP"}="1H5* MSP"; $resn_exch{"H5'2 MSP"}="2H5* MSP"; $resn_exch{" H4' MSP"}=" H4* MSP";
$resn_exch{" H3' MSP"}=" H3* MSP"; $resn_exch{" H2' MSP"}=" H2* MSP"; $resn_exch{" H1' MSP"}=" H1* MSP";
$resn_exch{"HN61 MSP"}="1HN6 MSP"; $resn_exch{"HN62 MSP"}="2HN6 MSP"; $resn_exch{" H21 MSQ"}="1H2  MSQ";
$resn_exch{" H71 MSQ"}="1H7  MSQ"; $resn_exch{"H101 MSQ"}="1H10 MSQ"; $resn_exch{"H121 MSQ"}="1H12 MSQ";
$resn_exch{"H151 MSQ"}="1H15 MSQ"; $resn_exch{"H171 MSQ"}="1H17 MSQ"; $resn_exch{"H181 MSQ"}="1H18 MSQ";
$resn_exch{"H191 MSQ"}="1H19 MSQ"; $resn_exch{"H221 MSQ"}="1H22 MSQ"; $resn_exch{"H222 MSQ"}="2H22 MSQ";
$resn_exch{"H223 MSQ"}="3H22 MSQ"; $resn_exch{"H271 MSQ"}="1H27 MSQ"; $resn_exch{"H272 MSQ"}="2H27 MSQ";
$resn_exch{"H273 MSQ"}="3H27 MSQ"; $resn_exch{"H321 MSQ"}="1H32 MSQ"; $resn_exch{"H322 MSQ"}="2H32 MSQ";
$resn_exch{"H323 MSQ"}="3H32 MSQ"; $resn_exch{"H2N1 MSS"}="1H2N MSS"; $resn_exch{"H2N2 MSS"}="2H2N MSS";
$resn_exch{"H4'1 MSS"}="1H4' MSS"; $resn_exch{"H4'2 MSS"}="2H4' MSS"; $resn_exch{" HP1 MSS"}="1HP  MSS";
$resn_exch{" HP2 MSS"}="2HP  MSS"; $resn_exch{" H83 MST"}="3H8  MST"; $resn_exch{" H82 MST"}="2H8  MST";
$resn_exch{" H81 MST"}="1H8  MST"; $resn_exch{"H113 MST"}="3H11 MST"; $resn_exch{"H112 MST"}="2H11 MST";
$resn_exch{"H111 MST"}="1H11 MST"; $resn_exch{"H123 MST"}="3H12 MST"; $resn_exch{"H122 MST"}="2H12 MST";
$resn_exch{"H121 MST"}="1H12 MST"; $resn_exch{"H133 MST"}="3H13 MST"; $resn_exch{"H132 MST"}="2H13 MST";
$resn_exch{"H131 MST"}="1H13 MST"; $resn_exch{"H152 MST"}="2H15 MST"; $resn_exch{"H151 MST"}="1H15 MST";
$resn_exch{"H163 MST"}="3H16 MST"; $resn_exch{"H162 MST"}="2H16 MST"; $resn_exch{"H161 MST"}="1H16 MST";
$resn_exch{" H21 MSU"}="1H2  MSU"; $resn_exch{" H22 MSU"}="2H2  MSU"; $resn_exch{" H31 MSU"}="1H3  MSU";
$resn_exch{" H32 MSU"}="2H3  MSU"; $resn_exch{" HT1 MSU"}="1HT  MSU"; $resn_exch{" HT2 MSU"}="2HT  MSU";
$resn_exch{" HT3 MSU"}="3HT  MSU"; $resn_exch{"HA21 MT1"}="1HA2 MT1"; $resn_exch{"HA22 MT1"}="2HA2 MT1";
$resn_exch{"HA41 MT1"}="1HA4 MT1"; $resn_exch{"HA42 MT1"}="2HA4 MT1"; $resn_exch{" H91 MT1"}="1H9  MT1";
$resn_exch{" H92 MT1"}="2H9  MT1"; $resn_exch{" HM1 MT1"}="1HM  MT1"; $resn_exch{" HM2 MT1"}="2HM  MT1";
$resn_exch{" HM3 MT1"}="3HM  MT1"; $resn_exch{" HB1 MT1"}="1HB  MT1"; $resn_exch{" HB2 MT1"}="2HB  MT1";
$resn_exch{" HG1 MT1"}="1HG  MT1"; $resn_exch{" HG2 MT1"}="2HG  MT1"; $resn_exch{"HCS1 MTA"}="1HCS MTA";
$resn_exch{"HCS2 MTA"}="2HCS MTA"; $resn_exch{"HCS3 MTA"}="3HCS MTA"; $resn_exch{"H5'1 MTA"}="1H5* MTA";
$resn_exch{"H5'2 MTA"}="2H5* MTA"; $resn_exch{" H4' MTA"}=" H4* MTA"; $resn_exch{" H2' MTA"}=" H2* MTA";
$resn_exch{"HO2' MTA"}="HO2* MTA"; $resn_exch{" H3' MTA"}=" H3* MTA"; $resn_exch{" H1' MTA"}=" H1* MTA";
$resn_exch{" H61 MTA"}="1H6  MTA"; $resn_exch{" H62 MTA"}="2H6  MTA"; $resn_exch{" H11 MTB"}="1H1  MTB";
$resn_exch{" H12 MTB"}="2H1  MTB"; $resn_exch{" H13 MTB"}="3H1  MTB"; $resn_exch{" H21 MTB"}="1H2  MTB";
$resn_exch{" H22 MTB"}="2H2  MTB"; $resn_exch{" H23 MTB"}="3H2  MTB"; $resn_exch{" H31 MTB"}="1H3  MTB";
$resn_exch{" H32 MTB"}="2H3  MTB"; $resn_exch{" H33 MTB"}="3H3  MTB"; $resn_exch{"HO4' MTB"}="'HO4 MTB";
$resn_exch{"H11A MTC"}="1H1  MTC"; $resn_exch{" H12 MTC"}="2H1  MTC"; $resn_exch{" H91 MTC"}="1H9  MTC";
$resn_exch{" H92 MTC"}="2H9  MTC"; $resn_exch{"H151 MTC"}="1H15 MTC"; $resn_exch{"H152 MTC"}="2H15 MTC";
$resn_exch{"H161 MTC"}="1H16 MTC"; $resn_exch{"H162 MTC"}="2H16 MTC"; $resn_exch{"H163 MTC"}="3H16 MTC";
$resn_exch{"H181 MTC"}="1H18 MTC"; $resn_exch{"H182 MTC"}="2H18 MTC"; $resn_exch{"H183 MTC"}="3H18 MTC";
$resn_exch{" H21 MTD"}="1H2  MTD"; $resn_exch{" H22 MTD"}="2H2  MTD"; $resn_exch{" H41 MTD"}="1H4  MTD";
$resn_exch{" H42 MTD"}="2H4  MTD"; $resn_exch{" H43 MTD"}="3H4  MTD"; $resn_exch{"HN21 MTE"}="1HN2 MTE";
$resn_exch{"HN22 MTE"}="2HN2 MTE"; $resn_exch{"H4'1 MTE"}="1H4' MTE"; $resn_exch{"H4'2 MTE"}="2H4' MTE";
$resn_exch{"HOP2 MTE"}="2HOP MTE"; $resn_exch{"HOP3 MTE"}="3HOP MTE"; $resn_exch{" HM1 MTF"}="1HM  MTF";
$resn_exch{" HM2 MTF"}="2HM  MTF"; $resn_exch{" HM3 MTF"}="3HM  MTF"; $resn_exch{" H21 MTG"}="1H2  MTG";
$resn_exch{" H22 MTG"}="2H2  MTG"; $resn_exch{" H41 MTG"}="1H4  MTG"; $resn_exch{" H42 MTG"}="2H4  MTG";
$resn_exch{" H43 MTG"}="3H4  MTG"; $resn_exch{"HCS1 MTH"}="1HCS MTH"; $resn_exch{"HCS2 MTH"}="2HCS MTH";
$resn_exch{"HCS3 MTH"}="3HCS MTH"; $resn_exch{"H5'1 MTH"}="1H5* MTH"; $resn_exch{"H5'2 MTH"}="2H5* MTH";
$resn_exch{"H4'1 MTH"}="1H4* MTH"; $resn_exch{"H2'1 MTH"}="1H2* MTH"; $resn_exch{"HO21 MTH"}="1HO2 MTH";
$resn_exch{"H3'1 MTH"}="1H3* MTH"; $resn_exch{"HO31 MTH"}="1HO3 MTH"; $resn_exch{"H1'1 MTH"}="1H1* MTH";
$resn_exch{"HC81 MTH"}="1HC8 MTH"; $resn_exch{"HC71 MTH"}="1HC7 MTH"; $resn_exch{"HN61 MTH"}="1HN6 MTH";
$resn_exch{"HN62 MTH"}="2HN6 MTH"; $resn_exch{"HC21 MTH"}="1HC2 MTH"; $resn_exch{" HS1 MTI"}="1HS  MTI";
$resn_exch{" HS2 MTI"}="2HS  MTI"; $resn_exch{" HS3 MTI"}="3HS  MTI"; $resn_exch{"H5'1 MTI"}="1H5* MTI";
$resn_exch{"H5'2 MTI"}="2H5* MTI"; $resn_exch{" H4' MTI"}=" H4* MTI"; $resn_exch{"H4'1 MTI"}="1H4* MTI";
$resn_exch{"H4'2 MTI"}="2H4* MTI"; $resn_exch{" H2' MTI"}=" H2* MTI"; $resn_exch{" H3' MTI"}=" H3* MTI";
$resn_exch{" H1' MTI"}=" H1* MTI"; $resn_exch{"HN1A MTJ"}="1HN  MTJ"; $resn_exch{" HN2 MTJ"}="2HN  MTJ";
$resn_exch{" HN3 MTJ"}="3HN  MTJ"; $resn_exch{" HB1 MTJ"}="1HB  MTJ"; $resn_exch{" HB2 MTJ"}="2HB  MTJ";
$resn_exch{" HM1 MTJ"}="1HM  MTJ"; $resn_exch{" HM2 MTJ"}="2HM  MTJ"; $resn_exch{" HM3 MTJ"}="3HM  MTJ";
$resn_exch{"HAH1 MTK"}="1HAH MTK"; $resn_exch{"HAH2 MTK"}="2HAH MTK"; $resn_exch{"HAG1 MTK"}="1HAG MTK";
$resn_exch{"HAG2 MTK"}="2HAG MTK"; $resn_exch{"HAZ1 MTK"}="1HAZ MTK"; $resn_exch{"HAZ2 MTK"}="2HAZ MTK";
$resn_exch{"HAX1 MTK"}="1HAX MTK"; $resn_exch{"HAX2 MTK"}="2HAX MTK"; $resn_exch{"HAY1 MTK"}="1HAY MTK";
$resn_exch{"HAY2 MTK"}="2HAY MTK"; $resn_exch{"HBA1 MTK"}="1HBA MTK"; $resn_exch{"HBA2 MTK"}="2HBA MTK";
$resn_exch{"HAW1 MTK"}="1HAW MTK"; $resn_exch{"HAW2 MTK"}="2HAW MTK"; $resn_exch{"HAV1 MTK"}="1HAV MTK";
$resn_exch{"HAV2 MTK"}="2HAV MTK"; $resn_exch{"HAA1 MTK"}="1HAA MTK"; $resn_exch{"HAA2 MTK"}="2HAA MTK";
$resn_exch{"HAA3 MTK"}="3HAA MTK"; $resn_exch{"HAB1 MTK"}="1HAB MTK"; $resn_exch{"HAB2 MTK"}="2HAB MTK";
$resn_exch{"HAB3 MTK"}="3HAB MTK"; $resn_exch{" H11 MTL"}="1H1  MTL"; $resn_exch{" H12 MTL"}="2H1  MTL";
$resn_exch{" H61 MTL"}="1H6  MTL"; $resn_exch{" H62 MTL"}="2H6  MTL"; $resn_exch{"HCS1 MTM"}="1HCS MTM";
$resn_exch{"HCS2 MTM"}="2HCS MTM"; $resn_exch{"HCS3 MTM"}="3HCS MTM"; $resn_exch{"H5'1 MTM"}="1H5* MTM";
$resn_exch{"H5'2 MTM"}="2H5* MTM"; $resn_exch{" H4' MTM"}=" H4* MTM"; $resn_exch{" H2' MTM"}=" H2* MTM";
$resn_exch{"HO'2 MTM"}="2HO* MTM"; $resn_exch{" H3' MTM"}=" H3* MTM"; $resn_exch{" H1' MTM"}=" H1* MTM";
$resn_exch{" H61 MTM"}="1H6  MTM"; $resn_exch{" H62 MTM"}="2H6  MTM"; $resn_exch{" H41 MTN"}="1H4  MTN";
$resn_exch{" H42 MTN"}="2H4  MTN"; $resn_exch{" H61 MTN"}="1H6  MTN"; $resn_exch{" H62 MTN"}="2H6  MTN";
$resn_exch{" H63 MTN"}="3H6  MTN"; $resn_exch{" H71 MTN"}="1H7  MTN"; $resn_exch{" H72 MTN"}="2H7  MTN";
$resn_exch{" H73 MTN"}="3H7  MTN"; $resn_exch{" H81 MTN"}="1H8  MTN"; $resn_exch{" H82 MTN"}="2H8  MTN";
$resn_exch{" H83 MTN"}="3H8  MTN"; $resn_exch{" H91 MTN"}="1H9  MTN"; $resn_exch{" H92 MTN"}="2H9  MTN";
$resn_exch{" H93 MTN"}="3H9  MTN"; $resn_exch{" H1  MTO"}="1H   MTO"; $resn_exch{" H2  MTO"}="2H   MTO";
$resn_exch{"H5'1 MTP"}="1H5' MTP"; $resn_exch{"H5'2 MTP"}="2H5' MTP"; $resn_exch{" HS1 MTP"}="1HS  MTP";
$resn_exch{" HS2 MTP"}="2HS  MTP"; $resn_exch{" HS3 MTP"}="3HS  MTP"; $resn_exch{"HN21 MTQ"}="1HN2 MTQ";
$resn_exch{"HN22 MTQ"}="2HN2 MTQ"; $resn_exch{"H4'1 MTQ"}="1H4' MTQ"; $resn_exch{"H4'2 MTQ"}="2H4' MTQ";
$resn_exch{"HOP2 MTQ"}="2HOP MTQ"; $resn_exch{"HOP3 MTQ"}="3HOP MTQ"; $resn_exch{"HOP2 MTR"}="2HOP MTR";
$resn_exch{" H71 MTR"}="1H5M MTR"; $resn_exch{" H72 MTR"}="2H5M MTR"; $resn_exch{" H73 MTR"}="3H5M MTR";
$resn_exch{" H2' MTR"}="1H2* MTR"; $resn_exch{"H2'' MTR"}="2H2* MTR"; $resn_exch{" H5' MTR"}="1H5* MTR";
$resn_exch{"H5'' MTR"}="2H5* MTR"; $resn_exch{" H4' MTR"}=" H4* MTR"; $resn_exch{" H1' MTR"}=" H1* MTR";
$resn_exch{" H3' MTR"}=" H3* MTR"; $resn_exch{"HO3' MTR"}="H3T  MTR"; $resn_exch{"HOP3 MTR"}="3HOP MTR";
$resn_exch{"HN31 MTS"}="1HN3 MTS"; $resn_exch{"HN32 MTS"}="2HN3 MTS"; $resn_exch{"H151 MTS"}="1H15 MTS";
$resn_exch{"H152 MTS"}="2H15 MTS"; $resn_exch{"H153 MTS"}="3H15 MTS"; $resn_exch{"H181 MTS"}="1H18 MTS";
$resn_exch{"H182 MTS"}="2H18 MTS"; $resn_exch{"H183 MTS"}="3H18 MTS"; $resn_exch{"H141 MTT"}="1H14 MTT";
$resn_exch{"H131 MTT"}="1H13 MTT"; $resn_exch{"H121 MTT"}="1H12 MTT"; $resn_exch{"H111 MTT"}="1H11 MTT";
$resn_exch{"H151 MTT"}="1H15 MTT"; $resn_exch{"H161 MTT"}="1H16 MTT"; $resn_exch{"H162 MTT"}="2H16 MTT";
$resn_exch{"H241 MTT"}="1H24 MTT"; $resn_exch{"H231 MTT"}="1H23 MTT"; $resn_exch{"H221 MTT"}="1H22 MTT";
$resn_exch{"H211 MTT"}="1H21 MTT"; $resn_exch{"H251 MTT"}="1H25 MTT"; $resn_exch{"H261 MTT"}="1H26 MTT";
$resn_exch{"H262 MTT"}="2H26 MTT"; $resn_exch{"H341 MTT"}="1H34 MTT"; $resn_exch{"H331 MTT"}="1H33 MTT";
$resn_exch{"H321 MTT"}="1H32 MTT"; $resn_exch{"H311 MTT"}="1H31 MTT"; $resn_exch{"H351 MTT"}="1H35 MTT";
$resn_exch{"H361 MTT"}="1H36 MTT"; $resn_exch{"H362 MTT"}="2H36 MTT"; $resn_exch{"H441 MTT"}="1H44 MTT";
$resn_exch{"H431 MTT"}="1H43 MTT"; $resn_exch{"H421 MTT"}="1H42 MTT"; $resn_exch{"H411 MTT"}="1H41 MTT";
$resn_exch{"H451 MTT"}="1H45 MTT"; $resn_exch{"H461 MTT"}="1H46 MTT"; $resn_exch{"H462 MTT"}="2H46 MTT";
$resn_exch{" H5' MTU"}="1H5* MTU"; $resn_exch{"H5'' MTU"}="2H5* MTU"; $resn_exch{" H4' MTU"}=" H4* MTU";
$resn_exch{" H1' MTU"}=" H1* MTU"; $resn_exch{"HN21 MTU"}="1HN2 MTU"; $resn_exch{"HN22 MTU"}="2HN2 MTU";
$resn_exch{" H2' MTU"}=" H2* MTU"; $resn_exch{"HO2' MTU"}="HO2* MTU"; $resn_exch{" H3' MTU"}=" H3* MTU";
$resn_exch{"HO3' MTU"}="HO3* MTU"; $resn_exch{"H2N1 MTV"}="1H2N MTV"; $resn_exch{"H2N2 MTV"}="2H2N MTV";
$resn_exch{"H3N1 MTV"}="1H3N MTV"; $resn_exch{"H5N1 MTV"}="1H5N MTV"; $resn_exch{"H6C1 MTV"}="1H6C MTV";
$resn_exch{"H3'1 MTV"}="1H3' MTV"; $resn_exch{"H4'1 MTV"}="1H4' MTV"; $resn_exch{"H4'2 MTV"}="2H4' MTV";
$resn_exch{"H2P1 MTV"}="1H2P MTV"; $resn_exch{"H3P1 MTV"}="1H3P MTV"; $resn_exch{"HM21 MTV"}="1HM2 MTV";
$resn_exch{"H2C1 MTW"}="1H2C MTW"; $resn_exch{"H2C2 MTW"}="2H2C MTW"; $resn_exch{"H8C1 MTW"}="1H8C MTW";
$resn_exch{"H8C2 MTW"}="2H8C MTW"; $resn_exch{"HN21 MTX"}="1HN2 MTX"; $resn_exch{"HN22 MTX"}="2HN2 MTX";
$resn_exch{"HN41 MTX"}="1HN4 MTX"; $resn_exch{"HN42 MTX"}="2HN4 MTX"; $resn_exch{" H91 MTX"}="1H9  MTX";
$resn_exch{" H92 MTX"}="2H9  MTX"; $resn_exch{" HM1 MTX"}="1HM  MTX"; $resn_exch{" HM2 MTX"}="2HM  MTX";
$resn_exch{" HM3 MTX"}="3HM  MTX"; $resn_exch{" HB1 MTX"}="1HB  MTX"; $resn_exch{" HB2 MTX"}="2HB  MTX";
$resn_exch{" HG1 MTX"}="1HG  MTX"; $resn_exch{" HG2 MTX"}="2HG  MTX"; $resn_exch{"HOE2 MTX"}="2HOE MTX";
$resn_exch{" H   MTY"}="1HN  MTY"; $resn_exch{" H2  MTY"}="2HN  MTY"; $resn_exch{" HB2 MTY"}="1HB  MTY";
$resn_exch{" HB3 MTY"}="2HB  MTY"; $resn_exch{" HH  MTY"}=" HOH MTY"; $resn_exch{"H141 MTZ"}="1H14 MTZ";
$resn_exch{"H142 MTZ"}="2H14 MTZ"; $resn_exch{"H151 MTZ"}="1H15 MTZ"; $resn_exch{"H152 MTZ"}="2H15 MTZ";
$resn_exch{"H161 MTZ"}="1H16 MTZ"; $resn_exch{"H162 MTZ"}="2H16 MTZ"; $resn_exch{"H171 MTZ"}="1H17 MTZ";
$resn_exch{"H172 MTZ"}="2H17 MTZ"; $resn_exch{"H7C1 MTZ"}="1H7C MTZ"; $resn_exch{"H7C2 MTZ"}="2H7C MTZ";
$resn_exch{"H2C1 MTZ"}="1H2C MTZ"; $resn_exch{"H2C2 MTZ"}="2H2C MTZ"; $resn_exch{"H2C3 MTZ"}="3H2C MTZ";
$resn_exch{" HN1 MTZ"}="1HN  MTZ"; $resn_exch{" HN2 MTZ"}="2HN  MTZ"; $resn_exch{"H321 MU0"}="1H32 MU0";
$resn_exch{"H322 MU0"}="2H32 MU0"; $resn_exch{"H291 MU0"}="1H29 MU0"; $resn_exch{"H292 MU0"}="2H29 MU0";
$resn_exch{"H231 MU0"}="1H23 MU0"; $resn_exch{"H232 MU0"}="2H23 MU0"; $resn_exch{"H141 MU0"}="1H14 MU0";
$resn_exch{"H142 MU0"}="2H14 MU0"; $resn_exch{"H143 MU0"}="3H14 MU0"; $resn_exch{"H151 MU0"}="1H15 MU0";
$resn_exch{"H152 MU0"}="2H15 MU0"; $resn_exch{"H153 MU0"}="3H15 MU0"; $resn_exch{"H21A MU0"}="1H2  MU0";
$resn_exch{" H22 MU0"}="2H2  MU0"; $resn_exch{"H391 MU0"}="1H39 MU0"; $resn_exch{"H392 MU0"}="2H39 MU0";
$resn_exch{" H11 MU0"}="1H1  MU0"; $resn_exch{"H12A MU0"}="2H1  MU0"; $resn_exch{"H221 MU0"}="1H22 MU0";
$resn_exch{"H222 MU0"}="2H22 MU0"; $resn_exch{"H21A MU1"}="1H2  MU1"; $resn_exch{" H22 MU1"}="2H2  MU1";
$resn_exch{" H23 MU1"}="3H2  MU1"; $resn_exch{"H141 MU1"}="1H14 MU1"; $resn_exch{"H142 MU1"}="2H14 MU1";
$resn_exch{"H143 MU1"}="3H14 MU1"; $resn_exch{"H151 MU1"}="1H15 MU1"; $resn_exch{"H152 MU1"}="2H15 MU1";
$resn_exch{"H153 MU1"}="3H15 MU1"; $resn_exch{"H221 MU1"}="1H22 MU1"; $resn_exch{"H222 MU1"}="2H22 MU1";
$resn_exch{"H231 MU1"}="1H23 MU1"; $resn_exch{"H232 MU1"}="2H23 MU1"; $resn_exch{"H291 MU1"}="1H29 MU1";
$resn_exch{"H292 MU1"}="2H29 MU1"; $resn_exch{"H321 MU1"}="1H32 MU1"; $resn_exch{"H322 MU1"}="2H32 MU1";
$resn_exch{"H401 MU1"}="1H40 MU1"; $resn_exch{"H402 MU1"}="2H40 MU1"; $resn_exch{"H403 MU1"}="3H40 MU1";
$resn_exch{"H8C1 MU2"}="1H8C MU2"; $resn_exch{"H8C2 MU2"}="2H8C MU2"; $resn_exch{"H8C3 MU2"}="3H8C MU2";
$resn_exch{"H6C1 MU2"}="1H6C MU2"; $resn_exch{"H6C2 MU2"}="2H6C MU2"; $resn_exch{"H201 MU2"}="1H20 MU2";
$resn_exch{"H202 MU2"}="2H20 MU2"; $resn_exch{"H203 MU2"}="3H20 MU2"; $resn_exch{"H231 MU2"}="1H23 MU2";
$resn_exch{"H232 MU2"}="2H23 MU2"; $resn_exch{"H233 MU2"}="3H23 MU2"; $resn_exch{"H251 MU2"}="1H25 MU2";
$resn_exch{"H252 MU2"}="2H25 MU2"; $resn_exch{"H261 MU2"}="1H26 MU2"; $resn_exch{"H262 MU2"}="2H26 MU2";
$resn_exch{" HA1 MU2"}="1HA  MU2"; $resn_exch{" HB2 MU2"}="2HB  MU2"; $resn_exch{"H101 MUA"}="1H10 MUA";
$resn_exch{"H102 MUA"}="2H10 MUA"; $resn_exch{"H103 MUA"}="3H10 MUA"; $resn_exch{" H1  MUB"}="1H   MUB";
$resn_exch{" H2  MUB"}="2H   MUB"; $resn_exch{" HN2 MUB"}="2HN  MUB"; $resn_exch{" H81 MUB"}="1H8  MUB";
$resn_exch{" H82 MUB"}="2H8  MUB"; $resn_exch{" H83 MUB"}="3H8  MUB"; $resn_exch{" H3  MUB"}="3H   MUB";
$resn_exch{" H9  MUB"}="9H   MUB"; $resn_exch{"H111 MUB"}="1H11 MUB"; $resn_exch{"H112 MUB"}="2H11 MUB";
$resn_exch{"H113 MUB"}="3H11 MUB"; $resn_exch{" H4A MUB"}="4H   MUB"; $resn_exch{" H5  MUB"}="5H   MUB";
$resn_exch{" H61 MUB"}="1H6  MUB"; $resn_exch{" H62 MUB"}="2H6  MUB"; $resn_exch{" HO6 MUB"}="6HO  MUB";
$resn_exch{"HM41 MUG"}="1HM4 MUG"; $resn_exch{"HM42 MUG"}="2HM4 MUG"; $resn_exch{"HM43 MUG"}="3HM4 MUG";
$resn_exch{"HO2' MUG"}="'HO2 MUG"; $resn_exch{"HO3' MUG"}="'HO3 MUG"; $resn_exch{"HO4' MUG"}="'HO4 MUG";
$resn_exch{"H6'1 MUG"}="1H6' MUG"; $resn_exch{"H6'2 MUG"}="2H6' MUG"; $resn_exch{"HO6' MUG"}="'HO6 MUG";
$resn_exch{"H151 MUH"}="1H15 MUH"; $resn_exch{"H152 MUH"}="2H15 MUH"; $resn_exch{"H153 MUH"}="3H15 MUH";
$resn_exch{"H471 MUI"}="1H47 MUI"; $resn_exch{"H472 MUI"}="2H47 MUI"; $resn_exch{"H473 MUI"}="3H47 MUI";
$resn_exch{"H251 MUI"}="1H25 MUI"; $resn_exch{"H252 MUI"}="2H25 MUI"; $resn_exch{"H321 MUI"}="1H32 MUI";
$resn_exch{"H322 MUI"}="2H32 MUI"; $resn_exch{"H161 MUI"}="1H16 MUI"; $resn_exch{"H162 MUI"}="2H16 MUI";
$resn_exch{"H121 MUI"}="1H12 MUI"; $resn_exch{"H122 MUI"}="2H12 MUI"; $resn_exch{"H151 MUI"}="1H15 MUI";
$resn_exch{"H152 MUI"}="2H15 MUI"; $resn_exch{"H153 MUI"}="3H15 MUI"; $resn_exch{"H141 MUI"}="1H14 MUI";
$resn_exch{"H142 MUI"}="2H14 MUI"; $resn_exch{"H143 MUI"}="3H14 MUI"; $resn_exch{"H181 MUI"}="1H18 MUI";
$resn_exch{"H182 MUI"}="2H18 MUI"; $resn_exch{"H183 MUI"}="3H18 MUI"; $resn_exch{"H11A MUL"}="1H1  MUL";
$resn_exch{" H12 MUL"}="2H1  MUL"; $resn_exch{" H21 MUL"}="1H2  MUL"; $resn_exch{" H22 MUL"}="2H2  MUL";
$resn_exch{" H71 MUL"}="1H7  MUL"; $resn_exch{" H72 MUL"}="2H7  MUL"; $resn_exch{" H81 MUL"}="1H8  MUL";
$resn_exch{" H82 MUL"}="2H8  MUL"; $resn_exch{"H131 MUL"}="1H13 MUL"; $resn_exch{"H132 MUL"}="2H13 MUL";
$resn_exch{"H151 MUL"}="1H15 MUL"; $resn_exch{"H152 MUL"}="2H15 MUL"; $resn_exch{"H153 MUL"}="3H15 MUL";
$resn_exch{"H161 MUL"}="1H16 MUL"; $resn_exch{"H162 MUL"}="2H16 MUL"; $resn_exch{"H163 MUL"}="3H16 MUL";
$resn_exch{"H171 MUL"}="1H17 MUL"; $resn_exch{"H172 MUL"}="2H17 MUL"; $resn_exch{"H173 MUL"}="3H17 MUL";
$resn_exch{"H181 MUL"}="1H18 MUL"; $resn_exch{"H182 MUL"}="2H18 MUL"; $resn_exch{"H183 MUL"}="3H18 MUL";
$resn_exch{"H201 MUL"}="1H20 MUL"; $resn_exch{"H202 MUL"}="2H20 MUL"; $resn_exch{"H221 MUL"}="1H22 MUL";
$resn_exch{"H222 MUL"}="2H22 MUL"; $resn_exch{"H231 MUL"}="1H23 MUL"; $resn_exch{"H232 MUL"}="2H23 MUL";
$resn_exch{"H241 MUL"}="1H24 MUL"; $resn_exch{"H242 MUL"}="2H24 MUL"; $resn_exch{"H251 MUL"}="1H25 MUL";
$resn_exch{"H252 MUL"}="2H25 MUL"; $resn_exch{"H261 MUL"}="1H26 MUL"; $resn_exch{"H262 MUL"}="2H26 MUL";
$resn_exch{"H263 MUL"}="3H26 MUL"; $resn_exch{"H271 MUL"}="1H27 MUL"; $resn_exch{"H272 MUL"}="2H27 MUL";
$resn_exch{"H281 MUL"}="1H28 MUL"; $resn_exch{"H282 MUL"}="2H28 MUL"; $resn_exch{"H283 MUL"}="3H28 MUL";
$resn_exch{" H11 MUP"}="1H1  MUP"; $resn_exch{" H12 MUP"}="2H1  MUP"; $resn_exch{" H21 MUP"}="1H2  MUP";
$resn_exch{" H22 MUP"}="2H2  MUP"; $resn_exch{" H31 MUP"}="1H3  MUP"; $resn_exch{" H32 MUP"}="2H3  MUP";
$resn_exch{" H41 MUP"}="1H4  MUP"; $resn_exch{" H42 MUP"}="2H4  MUP"; $resn_exch{" H51 MUP"}="1H5  MUP";
$resn_exch{" H52 MUP"}="2H5  MUP"; $resn_exch{" H61 MUP"}="1H6  MUP"; $resn_exch{" H62 MUP"}="2H6  MUP";
$resn_exch{" H71 MUP"}="1H7  MUP"; $resn_exch{" H72 MUP"}="2H7  MUP"; $resn_exch{" H81 MUP"}="1H8  MUP";
$resn_exch{" H82 MUP"}="2H8  MUP"; $resn_exch{" H91 MUP"}="1H9  MUP"; $resn_exch{" H92 MUP"}="2H9  MUP";
$resn_exch{"H101 MUP"}="1H10 MUP"; $resn_exch{"H102 MUP"}="2H10 MUP"; $resn_exch{"H111 MUP"}="1H11 MUP";
$resn_exch{"H112 MUP"}="2H11 MUP"; $resn_exch{"H113 MUP"}="3H11 MUP"; $resn_exch{" HM1 MUP"}="1HM  MUP";
$resn_exch{" HM2 MUP"}="2HM  MUP"; $resn_exch{" HM3 MUP"}="3HM  MUP"; $resn_exch{"HOP3 MUP"}="3HOP MUP";
$resn_exch{"HN21 MUR"}="1HN2 MUR"; $resn_exch{"HN22 MUR"}="2HN2 MUR"; $resn_exch{" H61 MUR"}="1H6  MUR";
$resn_exch{" H62 MUR"}="2H6  MUR"; $resn_exch{" H91 MUR"}="1H9  MUR"; $resn_exch{" H92 MUR"}="2H9  MUR";
$resn_exch{" H93 MUR"}="3H9  MUR"; $resn_exch{"H161 MUS"}="1H16 MUS"; $resn_exch{"H162 MUS"}="2H16 MUS";
$resn_exch{"H181 MUS"}="1H18 MUS"; $resn_exch{"H182 MUS"}="2H18 MUS"; $resn_exch{"H183 MUS"}="3H18 MUS";
$resn_exch{"H101 MUS"}="1H10 MUS"; $resn_exch{"H102 MUS"}="2H10 MUS"; $resn_exch{"HM41 MUS"}="1HM4 MUS";
$resn_exch{"HM42 MUS"}="2HM4 MUS"; $resn_exch{"HM43 MUS"}="3HM4 MUS"; $resn_exch{"H181 MUT"}="1H18 MUT";
$resn_exch{"H182 MUT"}="2H18 MUT"; $resn_exch{"H183 MUT"}="3H18 MUT"; $resn_exch{"H121 MUT"}="1H12 MUT";
$resn_exch{"H122 MUT"}="2H12 MUT"; $resn_exch{"H141 MUT"}="1H14 MUT"; $resn_exch{"H142 MUT"}="2H14 MUT";
$resn_exch{"H143 MUT"}="3H14 MUT"; $resn_exch{"H151 MUT"}="1H15 MUT"; $resn_exch{"H152 MUT"}="2H15 MUT";
$resn_exch{"H153 MUT"}="3H15 MUT"; $resn_exch{"H161 MUT"}="1H16 MUT"; $resn_exch{"H162 MUT"}="2H16 MUT";
$resn_exch{"H321 MUT"}="1H32 MUT"; $resn_exch{"H322 MUT"}="2H32 MUT"; $resn_exch{"H251 MUT"}="1H25 MUT";
$resn_exch{"H252 MUT"}="2H25 MUT"; $resn_exch{"H471 MUT"}="1H47 MUT"; $resn_exch{"H472 MUT"}="2H47 MUT";
$resn_exch{"H473 MUT"}="3H47 MUT"; $resn_exch{"H181 MUU"}="1H18 MUU"; $resn_exch{"H182 MUU"}="2H18 MUU";
$resn_exch{"H183 MUU"}="3H18 MUU"; $resn_exch{"H121 MUU"}="1H12 MUU"; $resn_exch{"H122 MUU"}="2H12 MUU";
$resn_exch{"H151 MUU"}="1H15 MUU"; $resn_exch{"H152 MUU"}="2H15 MUU"; $resn_exch{"H141 MUU"}="1H14 MUU";
$resn_exch{"H142 MUU"}="2H14 MUU"; $resn_exch{"H161 MUU"}="1H16 MUU"; $resn_exch{"H162 MUU"}="2H16 MUU";
$resn_exch{"H321 MUU"}="1H32 MUU"; $resn_exch{"H322 MUU"}="2H32 MUU"; $resn_exch{"H231 MUU"}="1H23 MUU";
$resn_exch{"H232 MUU"}="2H23 MUU"; $resn_exch{"H241 MUU"}="1H24 MUU"; $resn_exch{"H242 MUU"}="2H24 MUU";
$resn_exch{"H281 MUU"}="1H28 MUU"; $resn_exch{"H282 MUU"}="2H28 MUU"; $resn_exch{"H283 MUU"}="3H28 MUU";
$resn_exch{"H281 MUV"}="1H28 MUV"; $resn_exch{"H282 MUV"}="2H28 MUV"; $resn_exch{"H283 MUV"}="3H28 MUV";
$resn_exch{"H241 MUV"}="1H24 MUV"; $resn_exch{"H242 MUV"}="2H24 MUV"; $resn_exch{"H231 MUV"}="1H23 MUV";
$resn_exch{"H232 MUV"}="2H23 MUV"; $resn_exch{"H321 MUV"}="1H32 MUV"; $resn_exch{"H322 MUV"}="2H32 MUV";
$resn_exch{"H161 MUV"}="1H16 MUV"; $resn_exch{"H162 MUV"}="2H16 MUV"; $resn_exch{"H121 MUV"}="1H12 MUV";
$resn_exch{"H122 MUV"}="2H12 MUV"; $resn_exch{"H141 MUV"}="1H14 MUV"; $resn_exch{"H142 MUV"}="2H14 MUV";
$resn_exch{"H151 MUV"}="1H15 MUV"; $resn_exch{"H152 MUV"}="2H15 MUV"; $resn_exch{"H151 MUW"}="1H15 MUW";
$resn_exch{"H152 MUW"}="2H15 MUW"; $resn_exch{"H451 MUW"}="1H45 MUW"; $resn_exch{"H452 MUW"}="2H45 MUW";
$resn_exch{"H111 MUW"}="1H11 MUW"; $resn_exch{"H112 MUW"}="2H11 MUW"; $resn_exch{"H131 MUW"}="1H13 MUW";
$resn_exch{"H132 MUW"}="2H13 MUW"; $resn_exch{"H133 MUW"}="3H13 MUW"; $resn_exch{"H141 MUW"}="1H14 MUW";
$resn_exch{"H142 MUW"}="2H14 MUW"; $resn_exch{"H143 MUW"}="3H14 MUW"; $resn_exch{"H221 MUW"}="1H22 MUW";
$resn_exch{"H222 MUW"}="2H22 MUW"; $resn_exch{"H171 MUW"}="1H17 MUW"; $resn_exch{"H172 MUW"}="2H17 MUW";
$resn_exch{"H181 MUW"}="1H18 MUW"; $resn_exch{"H182 MUW"}="2H18 MUW"; $resn_exch{"H191 MUW"}="1H19 MUW";
$resn_exch{"H192 MUW"}="2H19 MUW"; $resn_exch{"H201 MUW"}="1H20 MUW"; $resn_exch{"H202 MUW"}="2H20 MUW";
$resn_exch{"H441 MUW"}="1H44 MUW"; $resn_exch{"H442 MUW"}="2H44 MUW"; $resn_exch{"H443 MUW"}="3H44 MUW";
$resn_exch{" H   MVA"}=" HN  MVA"; $resn_exch{" HN1 MVA"}="1HN  MVA"; $resn_exch{" HN2 MVA"}="2HN  MVA";
$resn_exch{" HN3 MVA"}="3HN  MVA"; $resn_exch{"HG11 MVA"}="1HG1 MVA"; $resn_exch{"HG12 MVA"}="2HG1 MVA";
$resn_exch{"HG13 MVA"}="3HG1 MVA"; $resn_exch{"HG21 MVA"}="1HG2 MVA"; $resn_exch{"HG22 MVA"}="2HG2 MVA";
$resn_exch{"HG23 MVA"}="3HG2 MVA"; $resn_exch{" H81 MVB"}="1H8  MVB"; $resn_exch{" H82 MVB"}="2H8  MVB";
$resn_exch{"H101 MVB"}="1H10 MVB"; $resn_exch{"H102 MVB"}="2H10 MVB"; $resn_exch{"H111 MVB"}="1H11 MVB";
$resn_exch{"H112 MVB"}="2H11 MVB"; $resn_exch{"H113 MVB"}="3H11 MVB"; $resn_exch{"H121 MVB"}="1H12 MVB";
$resn_exch{"H122 MVB"}="2H12 MVB"; $resn_exch{"H131 MVB"}="1H13 MVB"; $resn_exch{"H132 MVB"}="2H13 MVB";
$resn_exch{"H211 MVB"}="1H21 MVB"; $resn_exch{"H212 MVB"}="2H21 MVB"; $resn_exch{"H221 MVB"}="1H22 MVB";
$resn_exch{"H222 MVB"}="2H22 MVB"; $resn_exch{" H41 MVD"}="1H4  MVD"; $resn_exch{" H42 MVD"}="2H4  MVD";
$resn_exch{" H91 MVD"}="1H9  MVD"; $resn_exch{" H92 MVD"}="2H9  MVD"; $resn_exch{"H111 MVD"}="1H11 MVD";
$resn_exch{"H112 MVD"}="2H11 MVD"; $resn_exch{"H121 MVD"}="1H12 MVD"; $resn_exch{"H122 MVD"}="2H12 MVD";
$resn_exch{"H151 MVD"}="1H15 MVD"; $resn_exch{"H152 MVD"}="2H15 MVD"; $resn_exch{"H161 MVD"}="1H16 MVD";
$resn_exch{"H162 MVD"}="2H16 MVD"; $resn_exch{"H181 MVD"}="1H18 MVD"; $resn_exch{"H182 MVD"}="2H18 MVD";
$resn_exch{"H183 MVD"}="3H18 MVD"; $resn_exch{"H191 MVD"}="1H19 MVD"; $resn_exch{"H192 MVD"}="2H19 MVD";
$resn_exch{"H211 MVD"}="1H21 MVD"; $resn_exch{"H212 MVD"}="2H21 MVD"; $resn_exch{"H213 MVD"}="3H21 MVD";
$resn_exch{"H221 MVD"}="1H22 MVD"; $resn_exch{"H222 MVD"}="2H22 MVD"; $resn_exch{"H231 MVD"}="1H23 MVD";
$resn_exch{"H232 MVD"}="2H23 MVD"; $resn_exch{"H241 MVD"}="1H24 MVD"; $resn_exch{"H242 MVD"}="2H24 MVD";
$resn_exch{"H261 MVD"}="1H26 MVD"; $resn_exch{"H262 MVD"}="2H26 MVD"; $resn_exch{"H263 MVD"}="3H26 MVD";
$resn_exch{"H271 MVD"}="1H27 MVD"; $resn_exch{"H272 MVD"}="2H27 MVD"; $resn_exch{"H273 MVD"}="3H27 MVD";
$resn_exch{"H281 MVD"}="1H28 MVD"; $resn_exch{"H282 MVD"}="2H28 MVD"; $resn_exch{"H283 MVD"}="3H28 MVD";
$resn_exch{"H6C1 MVL"}="1H6C MVL"; $resn_exch{"H6C2 MVL"}="2H6C MVL"; $resn_exch{"HO11 MW1"}="1HO1 MW1";
$resn_exch{"HO12 MW1"}="2HO1 MW1"; $resn_exch{"HO11 MW2"}="1HO1 MW2"; $resn_exch{"HO12 MW2"}="2HO1 MW2";
$resn_exch{"HO21 MW2"}="1HO2 MW2"; $resn_exch{"HO22 MW2"}="2HO2 MW2"; $resn_exch{"HO11 MW3"}="1HO1 MW3";
$resn_exch{"HO12 MW3"}="2HO1 MW3"; $resn_exch{"HO21 MW3"}="1HO2 MW3"; $resn_exch{"HO22 MW3"}="2HO2 MW3";
$resn_exch{"HO31 MW3"}="1HO3 MW3"; $resn_exch{"HO32 MW3"}="2HO3 MW3"; $resn_exch{" H81 MX1"}="1H8  MX1";
$resn_exch{" H82 MX1"}="2H8  MX1"; $resn_exch{" H61 MX1"}="1H6  MX1"; $resn_exch{" H62 MX1"}="2H6  MX1";
$resn_exch{" H91 MX1"}="1H9  MX1"; $resn_exch{" H92 MX1"}="2H9  MX1"; $resn_exch{" H93 MX1"}="3H9  MX1";
$resn_exch{"H201 MX3"}="1H20 MX3"; $resn_exch{"H202 MX3"}="2H20 MX3"; $resn_exch{"H7C1 MX3"}="1H7C MX3";
$resn_exch{"H7C2 MX3"}="2H7C MX3"; $resn_exch{"H9C1 MX3"}="1H9C MX3"; $resn_exch{"H9C2 MX3"}="2H9C MX3";
$resn_exch{"H101 MX3"}="1H10 MX3"; $resn_exch{"H102 MX3"}="2H10 MX3"; $resn_exch{"H121 MX3"}="1H12 MX3";
$resn_exch{"H122 MX3"}="2H12 MX3"; $resn_exch{"H141 MX3"}="1H14 MX3"; $resn_exch{"H142 MX3"}="2H14 MX3";
$resn_exch{"H143 MX3"}="3H14 MX3"; $resn_exch{"H2C1 MX4"}="1H2C MX4"; $resn_exch{"H2C2 MX4"}="2H2C MX4";
$resn_exch{"H3C1 MX4"}="1H3C MX4"; $resn_exch{"H3C2 MX4"}="2H3C MX4"; $resn_exch{"H101 MX4"}="1H10 MX4";
$resn_exch{"H102 MX4"}="2H10 MX4"; $resn_exch{"H9C1 MX4"}="1H9C MX4"; $resn_exch{"H9C2 MX4"}="2H9C MX4";
$resn_exch{"H161 MX4"}="1H16 MX4"; $resn_exch{"H162 MX4"}="2H16 MX4"; $resn_exch{"H2C1 MX5"}="1H2C MX5";
$resn_exch{"H2C2 MX5"}="2H2C MX5"; $resn_exch{"H201 MX5"}="1H20 MX5"; $resn_exch{"H202 MX5"}="2H20 MX5";
$resn_exch{"H9C1 MX5"}="1H9C MX5"; $resn_exch{"H9C2 MX5"}="2H9C MX5"; $resn_exch{"H101 MX5"}="1H10 MX5";
$resn_exch{"H102 MX5"}="2H10 MX5"; $resn_exch{"H121 MX5"}="1H12 MX5"; $resn_exch{"H122 MX5"}="2H12 MX5";
$resn_exch{"H141 MX5"}="1H14 MX5"; $resn_exch{"H142 MX5"}="2H14 MX5"; $resn_exch{"H191 MX5"}="1H19 MX5";
$resn_exch{"H192 MX5"}="2H19 MX5"; $resn_exch{"HN21 MXA"}="1HN2 MXA"; $resn_exch{"HN22 MXA"}="2HN2 MXA";
$resn_exch{"HN41 MXA"}="1HN4 MXA"; $resn_exch{"HN42 MXA"}="2HN4 MXA"; $resn_exch{"H5M1 MXA"}="1H5M MXA";
$resn_exch{"H5M2 MXA"}="2H5M MXA"; $resn_exch{"H5M3 MXA"}="3H5M MXA"; $resn_exch{" H91 MXA"}="1H9  MXA";
$resn_exch{" H92 MXA"}="2H9  MXA"; $resn_exch{"H211 MXA"}="1H21 MXA"; $resn_exch{"H212 MXA"}="2H21 MXA";
$resn_exch{"H213 MXA"}="3H21 MXA"; $resn_exch{"H511 MXA"}="1H51 MXA"; $resn_exch{"H512 MXA"}="2H51 MXA";
$resn_exch{"H513 MXA"}="3H51 MXA"; $resn_exch{"HAO1 MXD"}="1HAO MXD"; $resn_exch{"HAO2 MXD"}="2HAO MXD";
$resn_exch{"HAM1 MXD"}="1HAM MXD"; $resn_exch{"HAM2 MXD"}="2HAM MXD"; $resn_exch{"HAG1 MXD"}="1HAG MXD";
$resn_exch{"HAG2 MXD"}="2HAG MXD"; $resn_exch{"HAK1 MXD"}="1HAK MXD"; $resn_exch{"HAK2 MXD"}="2HAK MXD";
$resn_exch{"HAL1 MXD"}="1HAL MXD"; $resn_exch{"HAL2 MXD"}="2HAL MXD"; $resn_exch{"HAJ1 MXD"}="1HAJ MXD";
$resn_exch{"HAJ2 MXD"}="2HAJ MXD"; $resn_exch{"HAH1 MXD"}="1HAH MXD"; $resn_exch{"HAH2 MXD"}="2HAH MXD";
$resn_exch{" H11 MXE"}="1H1  MXE"; $resn_exch{" H12 MXE"}="2H1  MXE"; $resn_exch{" H21 MXE"}="1H2  MXE";
$resn_exch{" H22 MXE"}="2H2  MXE"; $resn_exch{" H31 MXE"}="1H3  MXE"; $resn_exch{" H32 MXE"}="2H3  MXE";
$resn_exch{" H33 MXE"}="3H3  MXE"; $resn_exch{"H351 MXG"}="1H35 MXG"; $resn_exch{"H352 MXG"}="2H35 MXG";
$resn_exch{"H353 MXG"}="3H35 MXG"; $resn_exch{"H311 MXG"}="1H31 MXG"; $resn_exch{"H312 MXG"}="2H31 MXG";
$resn_exch{"H313 MXG"}="3H31 MXG"; $resn_exch{"H221 MXG"}="1H22 MXG"; $resn_exch{"H222 MXG"}="2H22 MXG";
$resn_exch{"H211 MXG"}="1H21 MXG"; $resn_exch{"H212 MXG"}="2H21 MXG"; $resn_exch{"H131 MXG"}="1H13 MXG";
$resn_exch{"H132 MXG"}="2H13 MXG"; $resn_exch{"H141 MXG"}="1H14 MXG"; $resn_exch{"H142 MXG"}="2H14 MXG";
$resn_exch{"H151 MXG"}="1H15 MXG"; $resn_exch{"H152 MXG"}="2H15 MXG"; $resn_exch{"H161 MXG"}="1H16 MXG";
$resn_exch{"H162 MXG"}="2H16 MXG"; $resn_exch{"H171 MXG"}="1H17 MXG"; $resn_exch{"H172 MXG"}="2H17 MXG";
$resn_exch{"HC31 MXG"}="1HC3 MXG"; $resn_exch{"HC32 MXG"}="2HC3 MXG"; $resn_exch{"HC21 MXG"}="1HC2 MXG";
$resn_exch{"HC22 MXG"}="2HC2 MXG"; $resn_exch{" H31 MXL"}="1H3  MXL"; $resn_exch{" H61 MXL"}="1H6  MXL";
$resn_exch{" H62 MXL"}="2H6  MXL"; $resn_exch{"HO41 MXL"}="1HO4 MXL"; $resn_exch{" H81 MXL"}="1H8  MXL";
$resn_exch{" H82 MXL"}="2H8  MXL"; $resn_exch{" H91 MXL"}="1H9  MXL"; $resn_exch{" H92 MXL"}="2H9  MXL";
$resn_exch{" H93 MXL"}="3H9  MXL"; $resn_exch{"HN21 MXL"}="1HN2 MXL"; $resn_exch{"H111 MXL"}="1H11 MXL";
$resn_exch{"HO71 MXL"}="1HO7 MXL"; $resn_exch{"H141 MXL"}="1H14 MXL"; $resn_exch{"H151 MXL"}="1H15 MXL";
$resn_exch{"H171 MXL"}="1H17 MXL"; $resn_exch{"H181 MXL"}="1H18 MXL"; $resn_exch{"HO91 MXL"}="1HO9 MXL";
$resn_exch{"H201 MXL"}="1H20 MXL"; $resn_exch{"H202 MXL"}="2H20 MXL"; $resn_exch{"H203 MXL"}="3H20 MXL";
$resn_exch{"H121 MXT"}="1H12 MXT"; $resn_exch{"H122 MXT"}="2H12 MXT"; $resn_exch{"H221 MXT"}="1H22 MXT";
$resn_exch{"H222 MXT"}="2H22 MXT"; $resn_exch{"H231 MXT"}="1H23 MXT"; $resn_exch{"H232 MXT"}="2H23 MXT";
$resn_exch{"H251 MXT"}="1H25 MXT"; $resn_exch{"H252 MXT"}="2H25 MXT"; $resn_exch{"H253 MXT"}="3H25 MXT";
$resn_exch{"H261 MXT"}="1H26 MXT"; $resn_exch{"H262 MXT"}="2H26 MXT"; $resn_exch{"H263 MXT"}="3H26 MXT";
$resn_exch{"H281 MXT"}="1H28 MXT"; $resn_exch{"H282 MXT"}="2H28 MXT"; $resn_exch{"H283 MXT"}="3H28 MXT";
$resn_exch{"H291 MXT"}="1H29 MXT"; $resn_exch{"H292 MXT"}="2H29 MXT"; $resn_exch{"H293 MXT"}="3H29 MXT";
$resn_exch{"H111 MXT"}="1H11 MXT"; $resn_exch{"H112 MXT"}="2H11 MXT"; $resn_exch{"HO21 MXT"}="1HO2 MXT";
$resn_exch{"HCM1 MXY"}="1HCM MXY"; $resn_exch{"HCM2 MXY"}="2HCM MXY"; $resn_exch{"HCM3 MXY"}="3HCM MXY";
$resn_exch{" H61 MXY"}="1H6  MXY"; $resn_exch{" H62 MXY"}="2H6  MXY"; $resn_exch{" H63 MXY"}="3H6  MXY";
$resn_exch{"H1C1 MXZ"}="1H1C MXZ"; $resn_exch{"HMC1 MXZ"}="1HMC MXZ"; $resn_exch{"HMC2 MXZ"}="2HMC MXZ";
$resn_exch{"HMC3 MXZ"}="3HMC MXZ"; $resn_exch{"H6C1 MXZ"}="1H6C MXZ"; $resn_exch{"H6C2 MXZ"}="2H6C MXZ";
$resn_exch{"H6C3 MXZ"}="3H6C MXZ"; $resn_exch{"H4N1 MY1"}="1H4N MY1"; $resn_exch{"H4N2 MY1"}="2H4N MY1";
$resn_exch{"H5C1 MY1"}="1H5C MY1"; $resn_exch{"H5C2 MY1"}="2H5C MY1"; $resn_exch{"H9C1 MY1"}="1H9C MY1";
$resn_exch{"H9C2 MY1"}="2H9C MY1"; $resn_exch{"H151 MY1"}="1H15 MY1"; $resn_exch{"H152 MY1"}="2H15 MY1";
$resn_exch{"H161 MY1"}="1H16 MY1"; $resn_exch{"H162 MY1"}="2H16 MY1"; $resn_exch{"H2C1 MY2"}="1H2C MY2";
$resn_exch{"H2C2 MY2"}="2H2C MY2"; $resn_exch{"H101 MY2"}="1H10 MY2"; $resn_exch{"H102 MY2"}="2H10 MY2";
$resn_exch{"H9C1 MY2"}="1H9C MY2"; $resn_exch{"H9C2 MY2"}="2H9C MY2"; $resn_exch{"H161 MY2"}="1H16 MY2";
$resn_exch{"H162 MY2"}="2H16 MY2"; $resn_exch{"H201 MY3"}="1H20 MY3"; $resn_exch{"H202 MY3"}="2H20 MY3";
$resn_exch{"H7C1 MY3"}="1H7C MY3"; $resn_exch{"H7C2 MY3"}="2H7C MY3"; $resn_exch{"H101 MY3"}="1H10 MY3";
$resn_exch{"H102 MY3"}="2H10 MY3"; $resn_exch{"H121 MY3"}="1H12 MY3"; $resn_exch{"H122 MY3"}="2H12 MY3";
$resn_exch{"H201 MY4"}="1H20 MY4"; $resn_exch{"H202 MY4"}="2H20 MY4"; $resn_exch{"H101 MY4"}="1H10 MY4";
$resn_exch{"H102 MY4"}="2H10 MY4"; $resn_exch{"H121 MY4"}="1H12 MY4"; $resn_exch{"H122 MY4"}="2H12 MY4";
$resn_exch{"H141 MY4"}="1H14 MY4"; $resn_exch{"H142 MY4"}="2H14 MY4"; $resn_exch{"H161 MY4"}="1H16 MY4";
$resn_exch{"H162 MY4"}="2H16 MY4"; $resn_exch{"H2N1 MY5"}="1H2N MY5"; $resn_exch{"H2N2 MY5"}="2H2N MY5";
$resn_exch{"H3C1 MY5"}="1H3C MY5"; $resn_exch{"H3C2 MY5"}="2H3C MY5"; $resn_exch{"H9C1 MY5"}="1H9C MY5";
$resn_exch{"H9C2 MY5"}="2H9C MY5"; $resn_exch{"H151 MY5"}="1H15 MY5"; $resn_exch{"H152 MY5"}="2H15 MY5";
$resn_exch{"H153 MY5"}="3H15 MY5"; $resn_exch{"H191 MY5"}="1H19 MY5"; $resn_exch{"H192 MY5"}="2H19 MY5";
$resn_exch{"H151 MY9"}="1H15 MY9"; $resn_exch{"H152 MY9"}="2H15 MY9"; $resn_exch{"H153 MY9"}="3H15 MY9";
$resn_exch{"H161 MY9"}="1H16 MY9"; $resn_exch{"H162 MY9"}="2H16 MY9"; $resn_exch{"H163 MY9"}="3H16 MY9";
$resn_exch{"H181 MY9"}="1H18 MY9"; $resn_exch{"H182 MY9"}="2H18 MY9"; $resn_exch{"H183 MY9"}="3H18 MY9";
$resn_exch{"H201 MY9"}="1H20 MY9"; $resn_exch{"H202 MY9"}="2H20 MY9"; $resn_exch{"H311 MY9"}="1H31 MY9";
$resn_exch{"H312 MY9"}="2H31 MY9"; $resn_exch{"H313 MY9"}="3H31 MY9"; $resn_exch{"H321 MY9"}="1H32 MY9";
$resn_exch{"H322 MY9"}="2H32 MY9"; $resn_exch{"H323 MY9"}="3H32 MY9"; $resn_exch{"H331 MY9"}="1H33 MY9";
$resn_exch{"H332 MY9"}="2H33 MY9"; $resn_exch{"H333 MY9"}="3H33 MY9"; $resn_exch{"H6A1 MYA"}="1H6A MYA";
$resn_exch{"H6A2 MYA"}="2H6A MYA"; $resn_exch{"H2AC MYA"}="CH2A MYA"; $resn_exch{"H8AC MYA"}="CH8A MYA";
$resn_exch{"HFM1 MYA"}="1HFM MYA"; $resn_exch{"HFM2 MYA"}="2HFM MYA"; $resn_exch{"HFM3 MYA"}="3HFM MYA";
$resn_exch{"HBM1 MYA"}="1HBM MYA"; $resn_exch{"HBM2 MYA"}="2HBM MYA"; $resn_exch{"HCM1 MYA"}="1HCM MYA";
$resn_exch{"HCM2 MYA"}="2HCM MYA"; $resn_exch{"HDM1 MYA"}="1HDM MYA"; $resn_exch{"HDM2 MYA"}="2HDM MYA";
$resn_exch{"HEM1 MYA"}="1HEM MYA"; $resn_exch{"HEM2 MYA"}="2HEM MYA"; $resn_exch{"HAM1 MYA"}="1HAM MYA";
$resn_exch{"HAM2 MYA"}="2HAM MYA"; $resn_exch{"H9M1 MYA"}="1H9M MYA"; $resn_exch{"H9M2 MYA"}="2H9M MYA";
$resn_exch{"H8M1 MYA"}="1H8M MYA"; $resn_exch{"H8M2 MYA"}="2H8M MYA"; $resn_exch{"H7M1 MYA"}="1H7M MYA";
$resn_exch{"H7M2 MYA"}="2H7M MYA"; $resn_exch{"H3M1 MYA"}="1H3M MYA"; $resn_exch{"H3M2 MYA"}="2H3M MYA";
$resn_exch{"H4M1 MYA"}="1H4M MYA"; $resn_exch{"H4M2 MYA"}="2H4M MYA"; $resn_exch{"H5M1 MYA"}="1H5M MYA";
$resn_exch{"H5M2 MYA"}="2H5M MYA"; $resn_exch{"H6M1 MYA"}="1H6M MYA"; $resn_exch{"H6M2 MYA"}="2H6M MYA";
$resn_exch{"H7AO MYA"}="OH7A MYA"; $resn_exch{"H8AO MYA"}="OH8A MYA"; $resn_exch{"HN41 MYA"}="1HN4 MYA";
$resn_exch{"HC61 MYA"}="1HC6 MYA"; $resn_exch{"HC62 MYA"}="2HC6 MYA"; $resn_exch{"HC71 MYA"}="1HC7 MYA";
$resn_exch{"HC72 MYA"}="2HC7 MYA"; $resn_exch{"HN81 MYA"}="1HN8 MYA"; $resn_exch{"H131 MYA"}="1H13 MYA";
$resn_exch{"H132 MYA"}="2H13 MYA"; $resn_exch{"H133 MYA"}="3H13 MYA"; $resn_exch{"H141 MYA"}="1H14 MYA";
$resn_exch{"H142 MYA"}="2H14 MYA"; $resn_exch{"H143 MYA"}="3H14 MYA"; $resn_exch{"H121 MYA"}="1H12 MYA";
$resn_exch{"H122 MYA"}="2H12 MYA"; $resn_exch{"H2AO MYA"}="OH2A MYA"; $resn_exch{"H4X1 MYA"}="1H4X MYA";
$resn_exch{"H1XC MYA"}="CH1X MYA"; $resn_exch{"H2XC MYA"}="CH2X MYA"; $resn_exch{"H2XO MYA"}="OH2X MYA";
$resn_exch{"H3X1 MYA"}="1H3X MYA"; $resn_exch{"H5X1 MYA"}="1H5X MYA"; $resn_exch{"H5X2 MYA"}="2H5X MYA";
$resn_exch{"H4AO MYA"}="OH4A MYA"; $resn_exch{"H10C MYA"}="CH10 MYA"; $resn_exch{"H10O MYA"}="OH10 MYA";
$resn_exch{"HC31 MYA"}="1HC3 MYA"; $resn_exch{"HC32 MYA"}="2HC3 MYA"; $resn_exch{"HC21 MYA"}="1HC2 MYA";
$resn_exch{"HC22 MYA"}="2HC2 MYA"; $resn_exch{"H531 MYD"}="1H53 MYD"; $resn_exch{"H532 MYD"}="2H53 MYD";
$resn_exch{"H533 MYD"}="3H53 MYD"; $resn_exch{"H521 MYD"}="1H52 MYD"; $resn_exch{"H522 MYD"}="2H52 MYD";
$resn_exch{"H523 MYD"}="3H52 MYD"; $resn_exch{"H491 MYD"}="1H49 MYD"; $resn_exch{"H492 MYD"}="2H49 MYD";
$resn_exch{"H381 MYD"}="1H38 MYD"; $resn_exch{"H382 MYD"}="2H38 MYD"; $resn_exch{"H371 MYD"}="1H37 MYD";
$resn_exch{"H372 MYD"}="2H37 MYD"; $resn_exch{"H341 MYD"}="1H34 MYD"; $resn_exch{"H342 MYD"}="2H34 MYD";
$resn_exch{"H281 MYD"}="1H28 MYD"; $resn_exch{"H282 MYD"}="2H28 MYD"; $resn_exch{"H111 MYD"}="1H11 MYD";
$resn_exch{"H112 MYD"}="2H11 MYD"; $resn_exch{"H21A MYE"}="1H2  MYE"; $resn_exch{" H22 MYE"}="2H2  MYE";
$resn_exch{" H31 MYE"}="1H3  MYE"; $resn_exch{" H32 MYE"}="2H3  MYE"; $resn_exch{" H41 MYE"}="1H4  MYE";
$resn_exch{" H42 MYE"}="2H4  MYE"; $resn_exch{"H101 MYE"}="1H10 MYE"; $resn_exch{"H102 MYE"}="2H10 MYE";
$resn_exch{"H121 MYE"}="1H12 MYE"; $resn_exch{"H122 MYE"}="2H12 MYE"; $resn_exch{"H123 MYE"}="3H12 MYE";
$resn_exch{"H131 MYE"}="1H13 MYE"; $resn_exch{"H132 MYE"}="2H13 MYE"; $resn_exch{"H133 MYE"}="3H13 MYE";
$resn_exch{"H181 MYE"}="1H18 MYE"; $resn_exch{"H182 MYE"}="2H18 MYE"; $resn_exch{"H11A MYE"}="1H1  MYE";
$resn_exch{" H12 MYE"}="2H1  MYE"; $resn_exch{" H13 MYE"}="3H1  MYE"; $resn_exch{"H121 MYG"}="1H12 MYG";
$resn_exch{"H122 MYG"}="2H12 MYG"; $resn_exch{"HN81 MYG"}="1HN8 MYG"; $resn_exch{"HN82 MYG"}="2HN8 MYG";
$resn_exch{"HO10 MYG"}="0HO1 MYG"; $resn_exch{"HO12 MYG"}="2HO1 MYG"; $resn_exch{"H231 MYP"}="1H23 MYP";
$resn_exch{"H211 MYP"}="1H21 MYP"; $resn_exch{"H212 MYP"}="2H21 MYP"; $resn_exch{"H221 MYP"}="1H22 MYP";
$resn_exch{"H222 MYP"}="2H22 MYP"; $resn_exch{"H251 MYP"}="1H25 MYP"; $resn_exch{"H252 MYP"}="2H25 MYP";
$resn_exch{"H241 MYP"}="1H24 MYP"; $resn_exch{"H242 MYP"}="2H24 MYP"; $resn_exch{"H291 MYP"}="1H29 MYP";
$resn_exch{"H292 MYP"}="2H29 MYP"; $resn_exch{"H281 MYP"}="1H28 MYP"; $resn_exch{"H282 MYP"}="2H28 MYP";
$resn_exch{"H261 MYP"}="1H26 MYP"; $resn_exch{"H262 MYP"}="2H26 MYP"; $resn_exch{"H271 MYP"}="1H27 MYP";
$resn_exch{"H272 MYP"}="2H27 MYP"; $resn_exch{"H112 MYP"}="12H1 MYP"; $resn_exch{"H131 MYP"}="1H13 MYP";
$resn_exch{"H132 MYP"}="2H13 MYP"; $resn_exch{"HC81 MYP"}="1HC8 MYP"; $resn_exch{"HC82 MYP"}="2HC8 MYP";
$resn_exch{"H101 MYP"}="1H10 MYP"; $resn_exch{"H102 MYP"}="2H10 MYP"; $resn_exch{"H103 MYP"}="3H10 MYP";
$resn_exch{"HC91 MYP"}="1HC9 MYP"; $resn_exch{"HC92 MYP"}="2HC9 MYP"; $resn_exch{" H31 MYQ"}="1H3  MYQ";
$resn_exch{" H32 MYQ"}="2H3  MYQ"; $resn_exch{" H21 MYR"}="1H2  MYR"; $resn_exch{" H22 MYR"}="2H2  MYR";
$resn_exch{" H31 MYR"}="1H3  MYR"; $resn_exch{" H32 MYR"}="2H3  MYR"; $resn_exch{" H41 MYR"}="1H4  MYR";
$resn_exch{" H42 MYR"}="2H4  MYR"; $resn_exch{" H51 MYR"}="1H5  MYR"; $resn_exch{" H52 MYR"}="2H5  MYR";
$resn_exch{" H61 MYR"}="1H6  MYR"; $resn_exch{" H62 MYR"}="2H6  MYR"; $resn_exch{" H71 MYR"}="1H7  MYR";
$resn_exch{" H72 MYR"}="2H7  MYR"; $resn_exch{" H81 MYR"}="1H8  MYR"; $resn_exch{" H82 MYR"}="2H8  MYR";
$resn_exch{" H91 MYR"}="1H9  MYR"; $resn_exch{" H92 MYR"}="2H9  MYR"; $resn_exch{"H101 MYR"}="1H10 MYR";
$resn_exch{"H102 MYR"}="2H10 MYR"; $resn_exch{"H111 MYR"}="1H11 MYR"; $resn_exch{"H112 MYR"}="2H11 MYR";
$resn_exch{"H121 MYR"}="1H12 MYR"; $resn_exch{"H122 MYR"}="2H12 MYR"; $resn_exch{"H131 MYR"}="1H13 MYR";
$resn_exch{"H132 MYR"}="2H13 MYR"; $resn_exch{"H141 MYR"}="1H14 MYR"; $resn_exch{"H142 MYR"}="2H14 MYR";
$resn_exch{"H143 MYR"}="3H14 MYR"; $resn_exch{" H11 MYS"}="1H1  MYS"; $resn_exch{" H12 MYS"}="2H1  MYS";
$resn_exch{" H13 MYS"}="3H1  MYS"; $resn_exch{" H21 MYS"}="1H2  MYS"; $resn_exch{" H22 MYS"}="2H2  MYS";
$resn_exch{" H31 MYS"}="1H3  MYS"; $resn_exch{" H32 MYS"}="2H3  MYS"; $resn_exch{" H41 MYS"}="1H4  MYS";
$resn_exch{" H42 MYS"}="2H4  MYS"; $resn_exch{" H51 MYS"}="1H5  MYS"; $resn_exch{" H52 MYS"}="2H5  MYS";
$resn_exch{" H61 MYS"}="1H6  MYS"; $resn_exch{" H62 MYS"}="2H6  MYS"; $resn_exch{" H71 MYS"}="1H7  MYS";
$resn_exch{" H72 MYS"}="2H7  MYS"; $resn_exch{" H81 MYS"}="1H8  MYS"; $resn_exch{" H82 MYS"}="2H8  MYS";
$resn_exch{" H91 MYS"}="1H9  MYS"; $resn_exch{" H92 MYS"}="2H9  MYS"; $resn_exch{"H101 MYS"}="1H10 MYS";
$resn_exch{"H102 MYS"}="2H10 MYS"; $resn_exch{"H111 MYS"}="1H11 MYS"; $resn_exch{"H112 MYS"}="2H11 MYS";
$resn_exch{"H121 MYS"}="1H12 MYS"; $resn_exch{"H122 MYS"}="2H12 MYS"; $resn_exch{"H131 MYS"}="1H13 MYS";
$resn_exch{"H132 MYS"}="2H13 MYS"; $resn_exch{"H141 MYS"}="1H14 MYS"; $resn_exch{"H142 MYS"}="2H14 MYS";
$resn_exch{"H151 MYS"}="1H15 MYS"; $resn_exch{"H152 MYS"}="2H15 MYS"; $resn_exch{"H153 MYS"}="3H15 MYS";
$resn_exch{" H91 MYT"}="1H9  MYT"; $resn_exch{" H92 MYT"}="2H9  MYT"; $resn_exch{" H93 MYT"}="3H9  MYT";
$resn_exch{"H111 MYT"}="1H11 MYT"; $resn_exch{"H112 MYT"}="2H11 MYT"; $resn_exch{"H113 MYT"}="3H11 MYT";
$resn_exch{"HN11 MYX"}="1HN1 MYX"; $resn_exch{"HN12 MYX"}="2HN1 MYX"; $resn_exch{" H41 MYX"}="1H4  MYX";
$resn_exch{" H42 MYX"}="2H4  MYX"; $resn_exch{" H43 MYX"}="3H4  MYX"; $resn_exch{" H61 MYX"}="1H6  MYX";
$resn_exch{" H62 MYX"}="2H6  MYX"; $resn_exch{" H63 MYX"}="3H6  MYX"; $resn_exch{" H81 MYX"}="1H8  MYX";
$resn_exch{" H82 MYX"}="2H8  MYX"; $resn_exch{" H83 MYX"}="3H8  MYX"; $resn_exch{"H181 MYX"}="1H18 MYX";
$resn_exch{"H182 MYX"}="2H18 MYX"; $resn_exch{"H183 MYX"}="3H18 MYX"; $resn_exch{"H241 MYX"}="1H24 MYX";
$resn_exch{"H242 MYX"}="2H24 MYX"; $resn_exch{"H243 MYX"}="3H24 MYX"; $resn_exch{"H251 MYX"}="1H25 MYX";
$resn_exch{"H252 MYX"}="2H25 MYX"; $resn_exch{"H253 MYX"}="3H25 MYX"; $resn_exch{"H381 MYY"}="1H38 MYY";
$resn_exch{"H382 MYY"}="2H38 MYY"; $resn_exch{"H383 MYY"}="3H38 MYY"; $resn_exch{"H371 MYY"}="1H37 MYY";
$resn_exch{"H372 MYY"}="2H37 MYY"; $resn_exch{"H361 MYY"}="1H36 MYY"; $resn_exch{"H362 MYY"}="2H36 MYY";
$resn_exch{"H351 MYY"}="1H35 MYY"; $resn_exch{"H352 MYY"}="2H35 MYY"; $resn_exch{"H341 MYY"}="1H34 MYY";
$resn_exch{"H342 MYY"}="2H34 MYY"; $resn_exch{"H331 MYY"}="1H33 MYY"; $resn_exch{"H332 MYY"}="2H33 MYY";
$resn_exch{"H321 MYY"}="1H32 MYY"; $resn_exch{"H322 MYY"}="2H32 MYY"; $resn_exch{"H311 MYY"}="1H31 MYY";
$resn_exch{"H312 MYY"}="2H31 MYY"; $resn_exch{"H301 MYY"}="1H30 MYY"; $resn_exch{"H302 MYY"}="2H30 MYY";
$resn_exch{"H291 MYY"}="1H29 MYY"; $resn_exch{"H292 MYY"}="2H29 MYY"; $resn_exch{"H281 MYY"}="1H28 MYY";
$resn_exch{"H282 MYY"}="2H28 MYY"; $resn_exch{"H271 MYY"}="1H27 MYY"; $resn_exch{"H272 MYY"}="2H27 MYY";
$resn_exch{"H261 MYY"}="1H26 MYY"; $resn_exch{"H262 MYY"}="2H26 MYY"; $resn_exch{"H251 MYY"}="1H25 MYY";
$resn_exch{"H252 MYY"}="2H25 MYY"; $resn_exch{"H241 MYY"}="1H24 MYY"; $resn_exch{"H242 MYY"}="2H24 MYY";
$resn_exch{"H6C1 MYY"}="1H6C MYY"; $resn_exch{"H6C2 MYY"}="2H6C MYY"; $resn_exch{"H4C1 MYY"}="1H4C MYY";
$resn_exch{"H4C2 MYY"}="2H4C MYY"; $resn_exch{"H8C1 MYY"}="1H8C MYY"; $resn_exch{"H8C2 MYY"}="2H8C MYY";
$resn_exch{"H9C1 MYY"}="1H9C MYY"; $resn_exch{"H9C2 MYY"}="2H9C MYY"; $resn_exch{"H101 MYY"}="1H10 MYY";
$resn_exch{"H102 MYY"}="2H10 MYY"; $resn_exch{"H111 MYY"}="1H11 MYY"; $resn_exch{"H112 MYY"}="2H11 MYY";
$resn_exch{"H121 MYY"}="1H12 MYY"; $resn_exch{"H122 MYY"}="2H12 MYY"; $resn_exch{"H131 MYY"}="1H13 MYY";
$resn_exch{"H132 MYY"}="2H13 MYY"; $resn_exch{"H141 MYY"}="1H14 MYY"; $resn_exch{"H142 MYY"}="2H14 MYY";
$resn_exch{"H151 MYY"}="1H15 MYY"; $resn_exch{"H152 MYY"}="2H15 MYY"; $resn_exch{"H161 MYY"}="1H16 MYY";
$resn_exch{"H162 MYY"}="2H16 MYY"; $resn_exch{"H171 MYY"}="1H17 MYY"; $resn_exch{"H172 MYY"}="2H17 MYY";
$resn_exch{"H181 MYY"}="1H18 MYY"; $resn_exch{"H182 MYY"}="2H18 MYY"; $resn_exch{"H191 MYY"}="1H19 MYY";
$resn_exch{"H192 MYY"}="2H19 MYY"; $resn_exch{"H201 MYY"}="1H20 MYY"; $resn_exch{"H202 MYY"}="2H20 MYY";
$resn_exch{"H203 MYY"}="3H20 MYY"; $resn_exch{"H271 MZ1"}="1H27 MZ1"; $resn_exch{"H272 MZ1"}="2H27 MZ1";
$resn_exch{"H273 MZ1"}="3H27 MZ1"; $resn_exch{"H231 MZ1"}="1H23 MZ1"; $resn_exch{"H232 MZ1"}="2H23 MZ1";
$resn_exch{"H233 MZ1"}="3H23 MZ1"; $resn_exch{"H321 MZ1"}="1H32 MZ1"; $resn_exch{"H322 MZ1"}="2H32 MZ1";
$resn_exch{"H161 MZ1"}="1H16 MZ1"; $resn_exch{"H162 MZ1"}="2H16 MZ1"; $resn_exch{"H121 MZ1"}="1H12 MZ1";
$resn_exch{"H122 MZ1"}="2H12 MZ1"; $resn_exch{"H241 MZ2"}="1H24 MZ2"; $resn_exch{"H242 MZ2"}="2H24 MZ2";
$resn_exch{"H243 MZ2"}="3H24 MZ2"; $resn_exch{"H321 MZ2"}="1H32 MZ2"; $resn_exch{"H322 MZ2"}="2H32 MZ2";
$resn_exch{"H161 MZ2"}="1H16 MZ2"; $resn_exch{"H162 MZ2"}="2H16 MZ2"; $resn_exch{"H121 MZ2"}="1H12 MZ2";
$resn_exch{"H122 MZ2"}="2H12 MZ2"; $resn_exch{"H281 MZ3"}="1H28 MZ3"; $resn_exch{"H282 MZ3"}="2H28 MZ3";
$resn_exch{"H283 MZ3"}="3H28 MZ3"; $resn_exch{"H321 MZ3"}="1H32 MZ3"; $resn_exch{"H322 MZ3"}="2H32 MZ3";
$resn_exch{"H161 MZ3"}="1H16 MZ3"; $resn_exch{"H162 MZ3"}="2H16 MZ3"; $resn_exch{"H121 MZ3"}="1H12 MZ3";
$resn_exch{"H122 MZ3"}="2H12 MZ3"; $resn_exch{"H201 MZ3"}="1H20 MZ3"; $resn_exch{"H202 MZ3"}="2H20 MZ3";
$resn_exch{"H203 MZ3"}="3H20 MZ3"; $resn_exch{"H201 MZ4"}="1H20 MZ4"; $resn_exch{"H202 MZ4"}="2H20 MZ4";
$resn_exch{"H203 MZ4"}="3H20 MZ4"; $resn_exch{"H121 MZ4"}="1H12 MZ4"; $resn_exch{"H122 MZ4"}="2H12 MZ4";
$resn_exch{"H161 MZ4"}="1H16 MZ4"; $resn_exch{"H162 MZ4"}="2H16 MZ4"; $resn_exch{"H321 MZ4"}="1H32 MZ4";
$resn_exch{"H322 MZ4"}="2H32 MZ4"; $resn_exch{"H261 MZ5"}="1H26 MZ5"; $resn_exch{"H262 MZ5"}="2H26 MZ5";
$resn_exch{"H263 MZ5"}="3H26 MZ5"; $resn_exch{"H251 MZ5"}="1H25 MZ5"; $resn_exch{"H252 MZ5"}="2H25 MZ5";
$resn_exch{"H253 MZ5"}="3H25 MZ5"; $resn_exch{"H321 MZ5"}="1H32 MZ5"; $resn_exch{"H322 MZ5"}="2H32 MZ5";
$resn_exch{"H161 MZ5"}="1H16 MZ5"; $resn_exch{"H162 MZ5"}="2H16 MZ5"; $resn_exch{"H121 MZ5"}="1H12 MZ5";
$resn_exch{"H122 MZ5"}="2H12 MZ5"; $resn_exch{"H141 MZ5"}="1H14 MZ5"; $resn_exch{"H142 MZ5"}="2H14 MZ5";
$resn_exch{"H143 MZ5"}="3H14 MZ5"; $resn_exch{"H151 MZ5"}="1H15 MZ5"; $resn_exch{"H152 MZ5"}="2H15 MZ5";
$resn_exch{"H181 MZ5"}="1H18 MZ5"; $resn_exch{"H182 MZ5"}="2H18 MZ5"; $resn_exch{"H183 MZ5"}="3H18 MZ5";
$resn_exch{"H321 MZ6"}="1H32 MZ6"; $resn_exch{"H322 MZ6"}="2H32 MZ6"; $resn_exch{"H161 MZ6"}="1H16 MZ6";
$resn_exch{"H162 MZ6"}="2H16 MZ6"; $resn_exch{"H121 MZ6"}="1H12 MZ6"; $resn_exch{"H122 MZ6"}="2H12 MZ6";
$resn_exch{"H151 MZ6"}="1H15 MZ6"; $resn_exch{"H152 MZ6"}="2H15 MZ6"; $resn_exch{"H181 MZ6"}="1H18 MZ6";
$resn_exch{"H182 MZ6"}="2H18 MZ6"; $resn_exch{"H183 MZ6"}="3H18 MZ6"; $resn_exch{"H141 MZ6"}="1H14 MZ6";
$resn_exch{"H142 MZ6"}="2H14 MZ6"; $resn_exch{"H143 MZ6"}="3H14 MZ6"; $resn_exch{"H121 MZ7"}="1H12 MZ7";
$resn_exch{"H122 MZ7"}="2H12 MZ7"; $resn_exch{"H131 MZ7"}="1H13 MZ7"; $resn_exch{"H132 MZ7"}="2H13 MZ7";
$resn_exch{"H141 MZ7"}="1H14 MZ7"; $resn_exch{"H142 MZ7"}="2H14 MZ7"; $resn_exch{"H151 MZ7"}="1H15 MZ7";
$resn_exch{"H152 MZ7"}="2H15 MZ7"; $resn_exch{"H181 MZ7"}="1H18 MZ7"; $resn_exch{"H182 MZ7"}="2H18 MZ7";
$resn_exch{"H183 MZ7"}="3H18 MZ7"; $resn_exch{"H161 MZ7"}="1H16 MZ7"; $resn_exch{"H162 MZ7"}="2H16 MZ7";
$resn_exch{"H251 MZ7"}="1H25 MZ7"; $resn_exch{"H252 MZ7"}="2H25 MZ7"; $resn_exch{"H253 MZ7"}="3H25 MZ7";
$resn_exch{"H241 MZ7"}="1H24 MZ7"; $resn_exch{"H242 MZ7"}="2H24 MZ7"; $resn_exch{"H243 MZ7"}="3H24 MZ7";
$resn_exch{"H291 MZ7"}="1H29 MZ7"; $resn_exch{"H292 MZ7"}="2H29 MZ7"; $resn_exch{"H293 MZ7"}="3H29 MZ7";
$resn_exch{"H321 MZ7"}="1H32 MZ7"; $resn_exch{"H322 MZ7"}="2H32 MZ7"; $resn_exch{"H121 MZ8"}="1H12 MZ8";
$resn_exch{"H122 MZ8"}="2H12 MZ8"; $resn_exch{"H141 MZ8"}="1H14 MZ8"; $resn_exch{"H142 MZ8"}="2H14 MZ8";
$resn_exch{"H143 MZ8"}="3H14 MZ8"; $resn_exch{"H151 MZ8"}="1H15 MZ8"; $resn_exch{"H152 MZ8"}="2H15 MZ8";
$resn_exch{"H153 MZ8"}="3H15 MZ8"; $resn_exch{"H161 MZ8"}="1H16 MZ8"; $resn_exch{"H162 MZ8"}="2H16 MZ8";
$resn_exch{"H321 MZ8"}="1H32 MZ8"; $resn_exch{"H322 MZ8"}="2H32 MZ8"; $resn_exch{"H221 MZ9"}="1H22 MZ9";
$resn_exch{"H222 MZ9"}="2H22 MZ9"; $resn_exch{"H231 MZ9"}="1H23 MZ9"; $resn_exch{"H232 MZ9"}="2H23 MZ9";
$resn_exch{"H261 MZ9"}="1H26 MZ9"; $resn_exch{"H262 MZ9"}="2H26 MZ9"; $resn_exch{"H271 MZ9"}="1H27 MZ9";
$resn_exch{"H272 MZ9"}="2H27 MZ9"; $resn_exch{"H273 MZ9"}="3H27 MZ9"; $resn_exch{"H321 MZ9"}="1H32 MZ9";
$resn_exch{"H322 MZ9"}="2H32 MZ9"; $resn_exch{"H161 MZ9"}="1H16 MZ9"; $resn_exch{"H162 MZ9"}="2H16 MZ9";
$resn_exch{"H121 MZ9"}="1H12 MZ9"; $resn_exch{"H122 MZ9"}="2H12 MZ9"; $resn_exch{"H141 MZ9"}="1H14 MZ9";
$resn_exch{"H142 MZ9"}="2H14 MZ9"; $resn_exch{"H143 MZ9"}="3H14 MZ9"; $resn_exch{"H151 MZ9"}="1H15 MZ9";
$resn_exch{"H152 MZ9"}="2H15 MZ9"; $resn_exch{"H181 MZ9"}="1H18 MZ9"; $resn_exch{"H182 MZ9"}="2H18 MZ9";
$resn_exch{"H183 MZ9"}="3H18 MZ9"; $resn_exch{"H201 MZ9"}="1H20 MZ9"; $resn_exch{"H202 MZ9"}="2H20 MZ9";
$resn_exch{"H203 MZ9"}="3H20 MZ9"; $resn_exch{"HN11 MZM"}="1HN1 MZM"; $resn_exch{"HN12 MZM"}="2HN1 MZM";
$resn_exch{" H51 MZM"}="1H5  MZM"; $resn_exch{" H52 MZM"}="2H5  MZM"; $resn_exch{" H53 MZM"}="3H5  MZM";
$resn_exch{" H41 MZM"}="1H4  MZM"; $resn_exch{" H42 MZM"}="2H4  MZM"; $resn_exch{" H43 MZM"}="3H4  MZM";
$resn_exch{"H5'1 MZP"}="1H5* MZP"; $resn_exch{"H5'2 MZP"}="2H5* MZP"; $resn_exch{"H4'C MZP"}="CH4* MZP";
$resn_exch{"H3'C MZP"}="CH3* MZP"; $resn_exch{"H3'O MZP"}="OH3* MZP"; $resn_exch{"H2'C MZP"}="CH2* MZP";
$resn_exch{"H2'O MZP"}="OH2* MZP"; $resn_exch{"H1'C MZP"}="CH1* MZP"; $resn_exch{"HN61 MZP"}="1HN6 MZP";
$resn_exch{"HN62 MZP"}="2HN6 MZP"; $resn_exch{"HOP3   N"}="3HOP   N"; $resn_exch{"HOP2   N"}="2HOP   N";
$resn_exch{" H5'   N"}="1H5*   N"; $resn_exch{"H5''   N"}="2H5*   N"; $resn_exch{" H4'   N"}=" H4*   N";
$resn_exch{" H3'   N"}=" H3*   N"; $resn_exch{"HO3'   N"}="H3T    N"; $resn_exch{" H2'   N"}=" H2*   N";
$resn_exch{"HO2'   N"}="2HO*   N"; $resn_exch{" H1'   N"}=" H1*   N"; $resn_exch{"H1'2   N"}="2H1*   N";
$resn_exch{"HN1A N10"}="1HN  N10"; $resn_exch{" HN2 N10"}="2HN  N10"; $resn_exch{" HB1 N10"}="1HB  N10";
$resn_exch{" HB2 N10"}="2HB  N10"; $resn_exch{"HA11 N10"}="1HA1 N10"; $resn_exch{"HA12 N10"}="2HA1 N10";
$resn_exch{"HB21 N10"}="1HB2 N10"; $resn_exch{"HB22 N10"}="2HB2 N10"; $resn_exch{"HG31 N10"}="1HG3 N10";
$resn_exch{"HG32 N10"}="2HG3 N10"; $resn_exch{"HD41 N10"}="1HD4 N10"; $resn_exch{"HD42 N10"}="2HD4 N10";
$resn_exch{"HG51 N10"}="1HG5 N10"; $resn_exch{"HG52 N10"}="2HG5 N10"; $resn_exch{"HD61 N10"}="1HD6 N10";
$resn_exch{"HD62 N10"}="2HD6 N10"; $resn_exch{"HD63 N10"}="3HD6 N10"; $resn_exch{"H21A N12"}="1H2  N12";
$resn_exch{"H22A N12"}="2H2  N12"; $resn_exch{" H61 N12"}="1H6  N12"; $resn_exch{" H62 N12"}="2H6  N12";
$resn_exch{" H81 N12"}="1H8  N12"; $resn_exch{" H82 N12"}="2H8  N12"; $resn_exch{" H91 N12"}="1H9  N12";
$resn_exch{" H92 N12"}="2H9  N12"; $resn_exch{"H101 N12"}="1H10 N12"; $resn_exch{"H102 N12"}="2H10 N12";
$resn_exch{"H111 N12"}="1H11 N12"; $resn_exch{"H112 N12"}="2H11 N12"; $resn_exch{"H121 N12"}="1H12 N12";
$resn_exch{"H122 N12"}="2H12 N12"; $resn_exch{"H191 N12"}="1H19 N12"; $resn_exch{"H192 N12"}="2H19 N12";
$resn_exch{"H301 N12"}="1H30 N12"; $resn_exch{"H302 N12"}="2H30 N12"; $resn_exch{"H311 N12"}="1H31 N12";
$resn_exch{"H312 N12"}="2H31 N12"; $resn_exch{"H321 N12"}="1H32 N12"; $resn_exch{"H322 N12"}="2H32 N12";
$resn_exch{"H331 N12"}="1H33 N12"; $resn_exch{"H332 N12"}="2H33 N12"; $resn_exch{"H341 N12"}="1H34 N12";
$resn_exch{"H342 N12"}="2H34 N12"; $resn_exch{"H5D1 N1C"}="1H5D N1C"; $resn_exch{"H5D2 N1C"}="2H5D N1C";
$resn_exch{" H1' N1C"}=" H1* N1C"; $resn_exch{" H2' N1C"}=" H2* N1C"; $resn_exch{"HO2' N1C"}="HO2* N1C";
$resn_exch{"HO3' N1C"}="HO3* N1C"; $resn_exch{"H5'1 N1C"}="1H5* N1C"; $resn_exch{"H5'2 N1C"}="2H5* N1C";
$resn_exch{"HN71 N1H"}="1HN7 N1H"; $resn_exch{"HN72 N1H"}="2HN7 N1H"; $resn_exch{"H311 N1H"}="1H31 N1H";
$resn_exch{"H312 N1H"}="2H31 N1H"; $resn_exch{" H2B N1H"}="H2'' N1H"; $resn_exch{" H3B N1H"}="H3'' N1H";
$resn_exch{" H4B N1H"}="H4'' N1H"; $resn_exch{" H6B N1H"}="H6'' N1H"; $resn_exch{"HM21 N1T"}="1HM2 N1T";
$resn_exch{"HM22 N1T"}="2HM2 N1T"; $resn_exch{"HM23 N1T"}="3HM2 N1T"; $resn_exch{"HN41 N1T"}="1HN4 N1T";
$resn_exch{"HN42 N1T"}="2HN4 N1T"; $resn_exch{"H7'1 N1T"}="1H7' N1T"; $resn_exch{"H7'2 N1T"}="2H7' N1T";
$resn_exch{"HM41 N1T"}="1HM4 N1T"; $resn_exch{"HM42 N1T"}="2HM4 N1T"; $resn_exch{"HM43 N1T"}="3HM4 N1T";
$resn_exch{" H61 N1T"}="1H6  N1T"; $resn_exch{" H62 N1T"}="2H6  N1T"; $resn_exch{" H71 N1T"}="1H7  N1T";
$resn_exch{" H72 N1T"}="2H7  N1T"; $resn_exch{"HOA2 N1T"}="2HOA N1T"; $resn_exch{"HOB3 N1T"}="3HOB N1T";
$resn_exch{"H101 N20"}="1H10 N20"; $resn_exch{"H102 N20"}="2H10 N20"; $resn_exch{"H121 N20"}="1H12 N20";
$resn_exch{"H122 N20"}="2H12 N20"; $resn_exch{"H131 N20"}="1H13 N20"; $resn_exch{"H132 N20"}="2H13 N20";
$resn_exch{"H141 N20"}="1H14 N20"; $resn_exch{"H142 N20"}="2H14 N20"; $resn_exch{"H151 N20"}="1H15 N20";
$resn_exch{"H152 N20"}="2H15 N20"; $resn_exch{"H161 N20"}="1H16 N20"; $resn_exch{"H162 N20"}="2H16 N20";
$resn_exch{" H81 N25"}="1H8  N25"; $resn_exch{" H82 N25"}="2H8  N25"; $resn_exch{" H83 N25"}="3H8  N25";
$resn_exch{"H151 N25"}="1H15 N25"; $resn_exch{"H152 N25"}="2H15 N25"; $resn_exch{"H171 N25"}="1H17 N25";
$resn_exch{"H172 N25"}="2H17 N25"; $resn_exch{"H181 N25"}="1H18 N25"; $resn_exch{"H182 N25"}="2H18 N25";
$resn_exch{"H183 N25"}="3H18 N25"; $resn_exch{"H191 N25"}="1H19 N25"; $resn_exch{"H192 N25"}="2H19 N25";
$resn_exch{"H193 N25"}="3H19 N25"; $resn_exch{"H341 N25"}="1H34 N25"; $resn_exch{"H342 N25"}="2H34 N25";
$resn_exch{"H343 N25"}="3H34 N25"; $resn_exch{"H1'1 N2B"}="1H1' N2B"; $resn_exch{"H1'2 N2B"}="2H1' N2B";
$resn_exch{"H2'1 N2B"}="1H2' N2B"; $resn_exch{"H2'2 N2B"}="2H2' N2B"; $resn_exch{"H2'3 N2B"}="3H2' N2B";
$resn_exch{" H   N2C"}=" HN  N2C"; $resn_exch{" HB2 N2C"}="1HB  N2C"; $resn_exch{" HB3 N2C"}="2HB  N2C";
$resn_exch{" HD1 N2C"}="1HD  N2C"; $resn_exch{" HD2 N2C"}="2HD  N2C"; $resn_exch{" HD3 N2C"}="3HD  N2C";
$resn_exch{" HN1 N2C"}="1HN  N2C"; $resn_exch{" HN2 N2C"}="2HN  N2C"; $resn_exch{" HN3 N2C"}="3HN  N2C";
$resn_exch{" HN1 N2M"}="1HN  N2M"; $resn_exch{" HN2 N2M"}="2HN  N2M"; $resn_exch{"H1C1 N2P"}="1H1C N2P";
$resn_exch{"H1C2 N2P"}="2H1C N2P"; $resn_exch{"H2C1 N2P"}="1H2C N2P"; $resn_exch{"H2C2 N2P"}="2H2C N2P";
$resn_exch{"H3C1 N2P"}="1H3C N2P"; $resn_exch{"H3C2 N2P"}="2H3C N2P"; $resn_exch{"H4C1 N2P"}="1H4C N2P";
$resn_exch{"H4C2 N2P"}="2H4C N2P"; $resn_exch{"HE21 N2P"}="1HE2 N2P"; $resn_exch{"HE22 N2P"}="2HE2 N2P";
$resn_exch{"H5C1 N2P"}="1H5C N2P"; $resn_exch{"H5C2 N2P"}="2H5C N2P"; $resn_exch{"H1N1 N2P"}="1H1N N2P";
$resn_exch{"H1N2 N2P"}="2H1N N2P"; $resn_exch{"H111 N2T"}="1H11 N2T"; $resn_exch{"H112 N2T"}="2H11 N2T";
$resn_exch{"H151 N2T"}="1H15 N2T"; $resn_exch{"H152 N2T"}="2H15 N2T"; $resn_exch{"H261 N2T"}="1H26 N2T";
$resn_exch{"H262 N2T"}="2H26 N2T"; $resn_exch{"H263 N2T"}="3H26 N2T"; $resn_exch{"H121 N2T"}="1H12 N2T";
$resn_exch{"H122 N2T"}="2H12 N2T"; $resn_exch{" H91 N2T"}="1H9  N2T"; $resn_exch{" H92 N2T"}="2H9  N2T";
$resn_exch{"H161 N2T"}="1H16 N2T"; $resn_exch{"H162 N2T"}="2H16 N2T"; $resn_exch{"H211 N30"}="1H21 N30";
$resn_exch{"H212 N30"}="2H21 N30"; $resn_exch{"H611 N30"}="1H61 N30"; $resn_exch{"H612 N30"}="2H61 N30";
$resn_exch{"H121 N30"}="1H12 N30"; $resn_exch{"H122 N30"}="2H12 N30"; $resn_exch{"H221 N30"}="1H22 N30";
$resn_exch{"H222 N30"}="2H22 N30"; $resn_exch{"H321 N30"}="1H32 N30"; $resn_exch{"H322 N30"}="2H32 N30";
$resn_exch{"H531 N30"}="1H53 N30"; $resn_exch{"H532 N30"}="2H53 N30"; $resn_exch{"H211 N33"}="1H21 N33";
$resn_exch{"H212 N33"}="2H21 N33"; $resn_exch{"H611 N33"}="1H61 N33"; $resn_exch{"H612 N33"}="2H61 N33";
$resn_exch{"H121 N33"}="1H12 N33"; $resn_exch{"H122 N33"}="2H12 N33"; $resn_exch{"H221 N33"}="1H22 N33";
$resn_exch{"H222 N33"}="2H22 N33"; $resn_exch{"H321 N33"}="1H32 N33"; $resn_exch{"H322 N33"}="2H32 N33";
$resn_exch{"H231 N33"}="1H23 N33"; $resn_exch{"H232 N33"}="2H23 N33"; $resn_exch{"H631 N33"}="1H63 N33";
$resn_exch{"H632 N33"}="2H63 N33"; $resn_exch{"H771 N33"}="1H77 N33"; $resn_exch{"H772 N33"}="2H77 N33";
$resn_exch{"H411 N3C"}="1H41 N3C"; $resn_exch{"H412 N3C"}="2H41 N3C"; $resn_exch{"H371 N3C"}="1H37 N3C";
$resn_exch{"H372 N3C"}="2H37 N3C"; $resn_exch{"H381 N3C"}="1H38 N3C"; $resn_exch{"H382 N3C"}="2H38 N3C";
$resn_exch{"H401 N3C"}="1H40 N3C"; $resn_exch{"H402 N3C"}="2H40 N3C"; $resn_exch{"H403 N3C"}="3H40 N3C";
$resn_exch{"H391 N3C"}="1H39 N3C"; $resn_exch{"H392 N3C"}="2H39 N3C"; $resn_exch{"H393 N3C"}="3H39 N3C";
$resn_exch{"H161 N3C"}="1H16 N3C"; $resn_exch{"H162 N3C"}="2H16 N3C"; $resn_exch{"H151 N3C"}="1H15 N3C";
$resn_exch{"H152 N3C"}="2H15 N3C"; $resn_exch{"H181 N3C"}="1H18 N3C"; $resn_exch{"H182 N3C"}="2H18 N3C";
$resn_exch{"H191 N3C"}="1H19 N3C"; $resn_exch{"H192 N3C"}="2H19 N3C"; $resn_exch{"H131 N3C"}="1H13 N3C";
$resn_exch{"H132 N3C"}="2H13 N3C"; $resn_exch{"H5'1 N3E"}="1H5' N3E"; $resn_exch{"H5'2 N3E"}="2H5' N3E";
$resn_exch{"H131 N3E"}="1H13 N3E"; $resn_exch{"H132 N3E"}="2H13 N3E"; $resn_exch{" H81 N3E"}="1H8  N3E";
$resn_exch{" H82 N3E"}="2H8  N3E"; $resn_exch{" H91 N3E"}="1H9  N3E"; $resn_exch{" H92 N3E"}="2H9  N3E";
$resn_exch{"H121 N3E"}="1H12 N3E"; $resn_exch{"H122 N3E"}="2H12 N3E"; $resn_exch{"HM21 N3T"}="1HM2 N3T";
$resn_exch{"HM22 N3T"}="2HM2 N3T"; $resn_exch{"HM23 N3T"}="3HM2 N3T"; $resn_exch{"HN41 N3T"}="1HN4 N3T";
$resn_exch{"HN42 N3T"}="2HN4 N3T"; $resn_exch{"H7'1 N3T"}="1H7' N3T"; $resn_exch{"H7'2 N3T"}="2H7' N3T";
$resn_exch{"HM41 N3T"}="1HM4 N3T"; $resn_exch{"HM42 N3T"}="2HM4 N3T"; $resn_exch{"HM43 N3T"}="3HM4 N3T";
$resn_exch{" H61 N3T"}="1H6  N3T"; $resn_exch{" H62 N3T"}="2H6  N3T"; $resn_exch{" H71 N3T"}="1H7  N3T";
$resn_exch{" H72 N3T"}="2H7  N3T"; $resn_exch{"HOA2 N3T"}="2HOA N3T"; $resn_exch{"HOB3 N3T"}="3HOB N3T";
$resn_exch{"H101 N41"}="1H10 N41"; $resn_exch{"H102 N41"}="2H10 N41"; $resn_exch{"H121 N41"}="1H12 N41";
$resn_exch{"H122 N41"}="2H12 N41"; $resn_exch{"H131 N41"}="1H13 N41"; $resn_exch{"H132 N41"}="2H13 N41";
$resn_exch{"H141 N41"}="1H14 N41"; $resn_exch{"H142 N41"}="2H14 N41"; $resn_exch{"H151 N41"}="1H15 N41";
$resn_exch{"H152 N41"}="2H15 N41"; $resn_exch{"H161 N41"}="1H16 N41"; $resn_exch{"H162 N41"}="2H16 N41";
$resn_exch{"H251 N41"}="1H25 N41"; $resn_exch{"H252 N41"}="2H25 N41"; $resn_exch{"H1'1 N4B"}="1H1' N4B";
$resn_exch{"H1'2 N4B"}="2H1' N4B"; $resn_exch{"H2'1 N4B"}="1H2' N4B"; $resn_exch{"H2'2 N4B"}="2H2' N4B";
$resn_exch{"H3'1 N4B"}="1H3' N4B"; $resn_exch{"H3'2 N4B"}="2H3' N4B"; $resn_exch{"H4'1 N4B"}="1H4' N4B";
$resn_exch{"H4'2 N4B"}="2H4' N4B"; $resn_exch{"H4'3 N4B"}="3H4' N4B"; $resn_exch{"H111 N4T"}="1H11 N4T";
$resn_exch{"H112 N4T"}="2H11 N4T"; $resn_exch{"H151 N4T"}="1H15 N4T"; $resn_exch{"H152 N4T"}="2H15 N4T";
$resn_exch{"HN11 N4T"}="1HN1 N4T"; $resn_exch{"HN12 N4T"}="2HN1 N4T"; $resn_exch{"H121 N4T"}="1H12 N4T";
$resn_exch{"H122 N4T"}="2H12 N4T"; $resn_exch{"H531 N5A"}="1H53 N5A"; $resn_exch{"H532 N5A"}="2H53 N5A";
$resn_exch{"H521 N5A"}="1H52 N5A"; $resn_exch{"H522 N5A"}="2H52 N5A"; $resn_exch{"H511 N5A"}="1H51 N5A";
$resn_exch{"H512 N5A"}="2H51 N5A"; $resn_exch{"HN5' N5A"}="HN5* N5A"; $resn_exch{" H4' N5A"}=" H4* N5A";
$resn_exch{" H3' N5A"}=" H3* N5A"; $resn_exch{"HO3' N5A"}="HO3* N5A"; $resn_exch{" H2' N5A"}=" H2* N5A";
$resn_exch{"HO2' N5A"}="HO2* N5A"; $resn_exch{" H1' N5A"}=" H1* N5A"; $resn_exch{"HN61 N5A"}="1HN6 N5A";
$resn_exch{"HN62 N5A"}="2HN6 N5A"; $resn_exch{"H5C1 N5B"}="1H5C N5B"; $resn_exch{"H5C2 N5B"}="2H5C N5B";
$resn_exch{"H6C1 N5B"}="1H6C N5B"; $resn_exch{"H6C2 N5B"}="2H6C N5B"; $resn_exch{"HOP3 N5I"}="3HOP N5I";
$resn_exch{" H1' N5I"}=" H1* N5I"; $resn_exch{" H4' N5I"}=" H4* N5I"; $resn_exch{" H3' N5I"}=" H3* N5I";
$resn_exch{"H2'1 N5I"}="1H2* N5I"; $resn_exch{"H2'2 N5I"}="2H2* N5I"; $resn_exch{"H5'1 N5I"}="1H5* N5I";
$resn_exch{"H5'2 N5I"}="2H5* N5I"; $resn_exch{"HOP2 N5I"}="2HOP N5I"; $resn_exch{"H521 N5O"}="1H52 N5O";
$resn_exch{"H522 N5O"}="2H52 N5O"; $resn_exch{"H511 N5O"}="1H51 N5O"; $resn_exch{"H512 N5O"}="2H51 N5O";
$resn_exch{"HN5' N5O"}="HN5* N5O"; $resn_exch{" H4' N5O"}=" H4* N5O"; $resn_exch{" H3' N5O"}=" H3* N5O";
$resn_exch{"HO3' N5O"}="HO3* N5O"; $resn_exch{" H2' N5O"}=" H2* N5O"; $resn_exch{"HO2' N5O"}="HO2* N5O";
$resn_exch{" H1' N5O"}=" H1* N5O"; $resn_exch{"HN61 N5O"}="1HN6 N5O"; $resn_exch{"HN62 N5O"}="2HN6 N5O";
$resn_exch{" H1' N5P"}=" H1* N5P"; $resn_exch{" H4' N5P"}=" H4* N5P"; $resn_exch{" H3' N5P"}=" H3* N5P";
$resn_exch{"HO3' N5P"}="HO3* N5P"; $resn_exch{"H2'1 N5P"}="1H2* N5P"; $resn_exch{"H2'2 N5P"}="2H2* N5P";
$resn_exch{"H5'1 N5P"}="1H5* N5P"; $resn_exch{"H5'2 N5P"}="2H5* N5P"; $resn_exch{"HOG3 N5P"}="3HOG N5P";
$resn_exch{"HOG2 N5P"}="2HOG N5P"; $resn_exch{"H111 N5T"}="1H11 N5T"; $resn_exch{"H112 N5T"}="2H11 N5T";
$resn_exch{"H311 N5T"}="1H31 N5T"; $resn_exch{"H312 N5T"}="2H31 N5T"; $resn_exch{"H313 N5T"}="3H31 N5T";
$resn_exch{"H351 N5T"}="1H35 N5T"; $resn_exch{"H352 N5T"}="2H35 N5T"; $resn_exch{"H353 N5T"}="3H35 N5T";
$resn_exch{"H2N1 N69"}="1H2N N69"; $resn_exch{"H2N2 N69"}="2H2N N69"; $resn_exch{"H191 N69"}="1H19 N69";
$resn_exch{"H192 N69"}="2H19 N69"; $resn_exch{"H193 N69"}="3H19 N69"; $resn_exch{"H181 N69"}="1H18 N69";
$resn_exch{"H182 N69"}="2H18 N69"; $resn_exch{"H183 N69"}="3H18 N69"; $resn_exch{"H101 N69"}="1H10 N69";
$resn_exch{"H102 N69"}="2H10 N69"; $resn_exch{"H121 N69"}="1H12 N69"; $resn_exch{"H122 N69"}="2H12 N69";
$resn_exch{"H131 N69"}="1H13 N69"; $resn_exch{"H132 N69"}="2H13 N69"; $resn_exch{"H141 N69"}="1H14 N69";
$resn_exch{"H142 N69"}="2H14 N69"; $resn_exch{"H151 N69"}="1H15 N69"; $resn_exch{"H152 N69"}="2H15 N69";
$resn_exch{"H161 N69"}="1H16 N69"; $resn_exch{"H162 N69"}="2H16 N69"; $resn_exch{"HOP2 N6G"}="2HOP N6G";
$resn_exch{" H5' N6G"}="1H5* N6G"; $resn_exch{"H5'' N6G"}="2H5* N6G"; $resn_exch{" H4' N6G"}=" H4* N6G";
$resn_exch{" H1' N6G"}=" H1* N6G"; $resn_exch{"HN21 N6G"}="1HN2 N6G"; $resn_exch{"HN22 N6G"}="2HN2 N6G";
$resn_exch{"HN61 N6G"}="1HN6 N6G"; $resn_exch{"HN62 N6G"}="2HN6 N6G"; $resn_exch{" H2' N6G"}=" H2* N6G";
$resn_exch{"HO2' N6G"}="2HO* N6G"; $resn_exch{" H3' N6G"}=" H3* N6G"; $resn_exch{"HO3' N6G"}="H3T  N6G";
$resn_exch{"HOP3 N6G"}="3HOP N6G"; $resn_exch{"H101 N76"}="1H10 N76"; $resn_exch{"H102 N76"}="2H10 N76";
$resn_exch{"H121 N76"}="1H12 N76"; $resn_exch{"H122 N76"}="2H12 N76"; $resn_exch{"H131 N76"}="1H13 N76";
$resn_exch{"H132 N76"}="2H13 N76"; $resn_exch{"H141 N76"}="1H14 N76"; $resn_exch{"H142 N76"}="2H14 N76";
$resn_exch{"H151 N76"}="1H15 N76"; $resn_exch{"H152 N76"}="2H15 N76"; $resn_exch{"H161 N76"}="1H16 N76";
$resn_exch{"H162 N76"}="2H16 N76"; $resn_exch{"HN21 N76"}="1HN2 N76"; $resn_exch{"H261 N76"}="1H26 N76";
$resn_exch{"H262 N76"}="2H26 N76"; $resn_exch{" H81 N7P"}="1H8  N7P"; $resn_exch{" H82 N7P"}="2H8  N7P";
$resn_exch{" H83 N7P"}="3H8  N7P"; $resn_exch{" H51 N7P"}="1H5  N7P"; $resn_exch{" H52 N7P"}="2H5  N7P";
$resn_exch{" H41 N7P"}="1H4  N7P"; $resn_exch{" H42 N7P"}="2H4  N7P"; $resn_exch{" H31 N7P"}="1H3  N7P";
$resn_exch{" H32 N7P"}="2H3  N7P"; $resn_exch{"H011 N8E"}="1H01 N8E"; $resn_exch{"H012 N8E"}="2H01 N8E";
$resn_exch{"H013 N8E"}="3H01 N8E"; $resn_exch{"H021 N8E"}="1H02 N8E"; $resn_exch{"H022 N8E"}="2H02 N8E";
$resn_exch{"H031 N8E"}="1H03 N8E"; $resn_exch{"H032 N8E"}="2H03 N8E"; $resn_exch{"H041 N8E"}="1H04 N8E";
$resn_exch{"H042 N8E"}="2H04 N8E"; $resn_exch{"H051 N8E"}="1H05 N8E"; $resn_exch{"H052 N8E"}="2H05 N8E";
$resn_exch{"H061 N8E"}="1H06 N8E"; $resn_exch{"H062 N8E"}="2H06 N8E"; $resn_exch{"H071 N8E"}="1H07 N8E";
$resn_exch{"H072 N8E"}="2H07 N8E"; $resn_exch{"H081 N8E"}="1H08 N8E"; $resn_exch{"H082 N8E"}="2H08 N8E";
$resn_exch{"H101 N8E"}="1H10 N8E"; $resn_exch{"H102 N8E"}="2H10 N8E"; $resn_exch{"H111 N8E"}="1H11 N8E";
$resn_exch{"H112 N8E"}="2H11 N8E"; $resn_exch{"H131 N8E"}="1H13 N8E"; $resn_exch{"H132 N8E"}="2H13 N8E";
$resn_exch{"H141 N8E"}="1H14 N8E"; $resn_exch{"H142 N8E"}="2H14 N8E"; $resn_exch{"H161 N8E"}="1H16 N8E";
$resn_exch{"H162 N8E"}="2H16 N8E"; $resn_exch{"H171 N8E"}="1H17 N8E"; $resn_exch{"H172 N8E"}="2H17 N8E";
$resn_exch{"H191 N8E"}="1H19 N8E"; $resn_exch{"H192 N8E"}="2H19 N8E"; $resn_exch{"H201 N8E"}="1H20 N8E";
$resn_exch{"H202 N8E"}="2H20 N8E"; $resn_exch{"H221 N8E"}="1H22 N8E"; $resn_exch{"H222 N8E"}="2H22 N8E";
$resn_exch{"H231 N8E"}="1H23 N8E"; $resn_exch{"H232 N8E"}="2H23 N8E"; $resn_exch{"HBD1 N8T"}="1HBD N8T";
$resn_exch{"HBD2 N8T"}="2HBD N8T"; $resn_exch{"HBE1 N8T"}="1HBE N8T"; $resn_exch{"HBE2 N8T"}="2HBE N8T";
$resn_exch{"HAQ1 N8T"}="1HAQ N8T"; $resn_exch{"HAQ2 N8T"}="2HAQ N8T"; $resn_exch{"HAP1 N8T"}="1HAP N8T";
$resn_exch{"HAP2 N8T"}="2HAP N8T"; $resn_exch{"HAB1 N8T"}="1HAB N8T"; $resn_exch{"HAB2 N8T"}="2HAB N8T";
$resn_exch{"HAC1 N8T"}="1HAC N8T"; $resn_exch{"HAC2 N8T"}="2HAC N8T"; $resn_exch{"HAD1 N8T"}="1HAD N8T";
$resn_exch{"HAD2 N8T"}="2HAD N8T"; $resn_exch{"HAE1 N8T"}="1HAE N8T"; $resn_exch{"HAE2 N8T"}="2HAE N8T";
$resn_exch{"HAF1 N8T"}="1HAF N8T"; $resn_exch{"HAF2 N8T"}="2HAF N8T"; $resn_exch{"HAG1 N8T"}="1HAG N8T";
$resn_exch{"HAG2 N8T"}="2HAG N8T"; $resn_exch{"HAH1 N8T"}="1HAH N8T"; $resn_exch{"HAH2 N8T"}="2HAH N8T";
$resn_exch{"HAI1 N8T"}="1HAI N8T"; $resn_exch{"HAI2 N8T"}="2HAI N8T"; $resn_exch{"HAW1 N8T"}="1HAW N8T";
$resn_exch{"HAW2 N8T"}="2HAW N8T"; $resn_exch{"HAX1 N8T"}="1HAX N8T"; $resn_exch{"HAX2 N8T"}="2HAX N8T";
$resn_exch{"HBL1 N8T"}="1HBL N8T"; $resn_exch{"HBL2 N8T"}="2HBL N8T"; $resn_exch{"HBK1 N8T"}="1HBK N8T";
$resn_exch{"HBK2 N8T"}="2HBK N8T"; $resn_exch{"H141 N9H"}="1H14 N9H"; $resn_exch{"H142 N9H"}="2H14 N9H";
$resn_exch{"H311 N9H"}="1H31 N9H"; $resn_exch{"H312 N9H"}="2H31 N9H"; $resn_exch{" H71 N9H"}="1H7  N9H";
$resn_exch{" H72 N9H"}="2H7  N9H"; $resn_exch{" H81 N9H"}="1H8  N9H"; $resn_exch{" H82 N9H"}="2H8  N9H";
$resn_exch{" H91 N9H"}="1H9  N9H"; $resn_exch{" H92 N9H"}="2H9  N9H"; $resn_exch{" H93 N9H"}="3H9  N9H";
$resn_exch{"H111 N9H"}="1H11 N9H"; $resn_exch{"H112 N9H"}="2H11 N9H"; $resn_exch{"HN21 N9H"}="1HN2 N9H";
$resn_exch{"HN22 N9H"}="2HN2 N9H"; $resn_exch{"H1C1 NA1"}="1H1C NA1"; $resn_exch{"H6C1 NA1"}="1H6C NA1";
$resn_exch{"H6C2 NA1"}="2H6C NA1"; $resn_exch{"H8C1 NA1"}="1H8C NA1"; $resn_exch{"H8C2 NA1"}="2H8C NA1";
$resn_exch{"H8C3 NA1"}="3H8C NA1"; $resn_exch{" HO1 NA1"}="1HO  NA1"; $resn_exch{"H9C1 NA1"}="1H9C NA1";
$resn_exch{"H9C2 NA1"}="2H9C NA1"; $resn_exch{"H9C3 NA1"}="3H9C NA1"; $resn_exch{"HO11 NA2"}="1HO1 NA2";
$resn_exch{"HO12 NA2"}="2HO1 NA2"; $resn_exch{"HO21 NA2"}="1HO2 NA2"; $resn_exch{"HO22 NA2"}="2HO2 NA2";
$resn_exch{" H21 NA3"}="1H2  NA3"; $resn_exch{" H22 NA3"}="2H2  NA3"; $resn_exch{" H21 NA4"}="1H2  NA4";
$resn_exch{" H22 NA4"}="2H2  NA4"; $resn_exch{"HO11 NA5"}="1HO1 NA5"; $resn_exch{"HO12 NA5"}="2HO1 NA5";
$resn_exch{"HO21 NA5"}="1HO2 NA5"; $resn_exch{"HO22 NA5"}="2HO2 NA5"; $resn_exch{"HO31 NA5"}="1HO3 NA5";
$resn_exch{"HO32 NA5"}="2HO3 NA5"; $resn_exch{"HO41 NA5"}="1HO4 NA5"; $resn_exch{"HO42 NA5"}="2HO4 NA5";
$resn_exch{"HO51 NA5"}="1HO5 NA5"; $resn_exch{"HO52 NA5"}="2HO5 NA5"; $resn_exch{"HO11 NA6"}="1HO1 NA6";
$resn_exch{"HO12 NA6"}="2HO1 NA6"; $resn_exch{"HO21 NA6"}="1HO2 NA6"; $resn_exch{"HO22 NA6"}="2HO2 NA6";
$resn_exch{"HO31 NA6"}="1HO3 NA6"; $resn_exch{"HO32 NA6"}="2HO3 NA6"; $resn_exch{"HO41 NA6"}="1HO4 NA6";
$resn_exch{"HO42 NA6"}="2HO4 NA6"; $resn_exch{"HO51 NA6"}="1HO5 NA6"; $resn_exch{"HO52 NA6"}="2HO5 NA6";
$resn_exch{"HO61 NA6"}="1HO6 NA6"; $resn_exch{"HO62 NA6"}="2HO6 NA6"; $resn_exch{"H5'1 NA7"}="1H5* NA7";
$resn_exch{"H5'2 NA7"}="2H5* NA7"; $resn_exch{"HC4' NA7"}="HC4* NA7"; $resn_exch{"HC3' NA7"}="HC3* NA7";
$resn_exch{"HO3' NA7"}="HO3* NA7"; $resn_exch{"HC2' NA7"}="HC2* NA7"; $resn_exch{"HC1' NA7"}="HC1* NA7";
$resn_exch{"HN61 NA7"}="1HN6 NA7"; $resn_exch{"HN62 NA7"}="2HN6 NA7"; $resn_exch{"H51' NA7"}="H51* NA7";
$resn_exch{"H52' NA7"}="H52* NA7"; $resn_exch{"HN4' NA7"}="HN4* NA7"; $resn_exch{"HN3' NA7"}="HN3* NA7";
$resn_exch{"HN5' NA7"}="HN5* NA7"; $resn_exch{"HN6' NA7"}="HN6* NA7"; $resn_exch{"HN7' NA7"}="HN7* NA7";
$resn_exch{"H1'1 NA7"}="1H1* NA7"; $resn_exch{"H1'2 NA7"}="2H1* NA7"; $resn_exch{" H71 NA9"}="1H7  NA9";
$resn_exch{" H72 NA9"}="2H7  NA9"; $resn_exch{"HN91 NA9"}="1HN9 NA9"; $resn_exch{"HN92 NA9"}="2HN9 NA9";
$resn_exch{"H151 NA9"}="1H15 NA9"; $resn_exch{"H152 NA9"}="2H15 NA9"; $resn_exch{"H153 NA9"}="3H15 NA9";
$resn_exch{"H161 NA9"}="1H16 NA9"; $resn_exch{"H162 NA9"}="2H16 NA9"; $resn_exch{"H163 NA9"}="3H16 NA9";
$resn_exch{"H221 NA9"}="1H22 NA9"; $resn_exch{"H222 NA9"}="2H22 NA9"; $resn_exch{"H261 NA9"}="1H26 NA9";
$resn_exch{"H262 NA9"}="2H26 NA9"; $resn_exch{"H281 NA9"}="1H28 NA9"; $resn_exch{"H282 NA9"}="2H28 NA9";
$resn_exch{"H291 NA9"}="1H29 NA9"; $resn_exch{"H292 NA9"}="2H29 NA9"; $resn_exch{"H301 NA9"}="1H30 NA9";
$resn_exch{"H302 NA9"}="2H30 NA9"; $resn_exch{"H311 NA9"}="1H31 NA9"; $resn_exch{"H312 NA9"}="2H31 NA9";
$resn_exch{"H321 NA9"}="1H32 NA9"; $resn_exch{"H322 NA9"}="2H32 NA9"; $resn_exch{" H61 NAA"}="1H6  NAA";
$resn_exch{" H62 NAA"}="2H6  NAA"; $resn_exch{" H81 NAA"}="1H8  NAA"; $resn_exch{" H82 NAA"}="2H8  NAA";
$resn_exch{" H83 NAA"}="3H8  NAA"; $resn_exch{"HO4' NAB"}="'HO4 NAB"; $resn_exch{"HOA2 NAC"}="2HOA NAC";
$resn_exch{"H51A NAC"}="AH51 NAC"; $resn_exch{"H52A NAC"}="AH52 NAC"; $resn_exch{" H4B NAC"}="AH4* NAC";
$resn_exch{" H3B NAC"}="AH3* NAC"; $resn_exch{"HO3A NAC"}="AHO3 NAC"; $resn_exch{" H2B NAC"}="AH2* NAC";
$resn_exch{"HO2A NAC"}="AHO2 NAC"; $resn_exch{" H1B NAC"}="AH1* NAC"; $resn_exch{" H8A NAC"}=" AH8 NAC";
$resn_exch{"H61A NAC"}="AH61 NAC"; $resn_exch{"H62A NAC"}="AH62 NAC"; $resn_exch{" H2A NAC"}=" AH2 NAC";
$resn_exch{"H51N NAC"}="NH51 NAC"; $resn_exch{"H52N NAC"}="NH52 NAC"; $resn_exch{" H4D NAC"}="NH4* NAC";
$resn_exch{" H3D NAC"}="NH3* NAC"; $resn_exch{"HO3N NAC"}="NHO3 NAC"; $resn_exch{" H2D NAC"}="NH2* NAC";
$resn_exch{"HO2N NAC"}="NHO2 NAC"; $resn_exch{" H1D NAC"}="NH1* NAC"; $resn_exch{" H2N NAC"}=" NH2 NAC";
$resn_exch{"H81N NAC"}="NH81 NAC"; $resn_exch{"H82N NAC"}="NH82 NAC"; $resn_exch{"H83N NAC"}="NH83 NAC";
$resn_exch{" H4N NAC"}=" NH4 NAC"; $resn_exch{" H5N NAC"}=" NH5 NAC"; $resn_exch{" H6N NAC"}=" NH6 NAC";
$resn_exch{"HOA2 NAD"}="2HOA NAD"; $resn_exch{"H51A NAD"}="AH51 NAD"; $resn_exch{"H52A NAD"}="AH52 NAD";
$resn_exch{" H4B NAD"}="AH4* NAD"; $resn_exch{" H3B NAD"}="AH3* NAD"; $resn_exch{"HO3A NAD"}="AHO3 NAD";
$resn_exch{" H2B NAD"}="AH2* NAD"; $resn_exch{"HO2A NAD"}="AHO2 NAD"; $resn_exch{" H1B NAD"}="AH1* NAD";
$resn_exch{" H8A NAD"}=" AH8 NAD"; $resn_exch{"H61A NAD"}="AH61 NAD"; $resn_exch{"H62A NAD"}="AH62 NAD";
$resn_exch{" H2A NAD"}=" AH2 NAD"; $resn_exch{"H51N NAD"}="NH51 NAD"; $resn_exch{"H52N NAD"}="NH52 NAD";
$resn_exch{" H4D NAD"}="NH4* NAD"; $resn_exch{" H3D NAD"}="NH3* NAD"; $resn_exch{"HO3N NAD"}="NHO3 NAD";
$resn_exch{" H2D NAD"}="NH2* NAD"; $resn_exch{"HO2N NAD"}="NHO2 NAD"; $resn_exch{" H1D NAD"}="NH1* NAD";
$resn_exch{" H2N NAD"}=" NH2 NAD"; $resn_exch{"H71N NAD"}="NH71 NAD"; $resn_exch{"H72N NAD"}="NH72 NAD";
$resn_exch{" H4N NAD"}=" NH4 NAD"; $resn_exch{" H5N NAD"}=" NH5 NAD"; $resn_exch{" H6N NAD"}=" NH6 NAD";
$resn_exch{"HOA2 NAE"}="2HOA NAE"; $resn_exch{"H5'1 NAE"}="1H5' NAE"; $resn_exch{"H5'2 NAE"}="2H5' NAE";
$resn_exch{"HO'3 NAE"}="3HO' NAE"; $resn_exch{"HO'2 NAE"}="2HO' NAE"; $resn_exch{"H6A1 NAE"}="1H6A NAE";
$resn_exch{"H6A2 NAE"}="2H6A NAE"; $resn_exch{"H5A1 NAE"}="1H5* NAE"; $resn_exch{"H5A2 NAE"}="2H5* NAE";
$resn_exch{" H4B NAE"}=" H4* NAE"; $resn_exch{" H3B NAE"}=" H3* NAE"; $resn_exch{"HOA3 NAE"}="3HO* NAE";
$resn_exch{" H2B NAE"}=" H2* NAE"; $resn_exch{"HOB2 NAE"}="2HO* NAE"; $resn_exch{" H1B NAE"}=" H1* NAE";
$resn_exch{"H7N1 NAE"}="1H7N NAE"; $resn_exch{"H7N2 NAE"}="2H7N NAE"; $resn_exch{"H4P1 NAE"}="1H4P NAE";
$resn_exch{"H4P2 NAE"}="2H4P NAE"; $resn_exch{"H2P1 NAE"}="1H2P NAE"; $resn_exch{"H2P2 NAE"}="2H2P NAE";
$resn_exch{"H2P3 NAE"}="3H2P NAE"; $resn_exch{"HM11 NAF"}="1HM1 NAF"; $resn_exch{"HM12 NAF"}="2HM1 NAF";
$resn_exch{"HM13 NAF"}="3HM1 NAF"; $resn_exch{"HM21 NAF"}="1HM2 NAF"; $resn_exch{"HM22 NAF"}="2HM2 NAF";
$resn_exch{"HM23 NAF"}="3HM2 NAF"; $resn_exch{"HM31 NAF"}="1HM3 NAF"; $resn_exch{"HM32 NAF"}="2HM3 NAF";
$resn_exch{"HM33 NAF"}="3HM3 NAF"; $resn_exch{" H61 NAG"}="1H6  NAG"; $resn_exch{" H62 NAG"}="2H6  NAG";
$resn_exch{" H81 NAG"}="1H8  NAG"; $resn_exch{" H82 NAG"}="2H8  NAG"; $resn_exch{" H83 NAG"}="3H8  NAG";
$resn_exch{"HOA2 NAH"}="2HOA NAH"; $resn_exch{"H51A NAH"}="AH51 NAH"; $resn_exch{"H52A NAH"}="AH52 NAH";
$resn_exch{" H4B NAH"}="AH4* NAH"; $resn_exch{" H3B NAH"}="AH3* NAH"; $resn_exch{"HO3A NAH"}="AHO3 NAH";
$resn_exch{" H2B NAH"}="AH2* NAH"; $resn_exch{"HO2A NAH"}="AHO2 NAH"; $resn_exch{" H1B NAH"}="AH1* NAH";
$resn_exch{" H8A NAH"}=" AH8 NAH"; $resn_exch{"H61A NAH"}="AH61 NAH"; $resn_exch{"H62A NAH"}="AH62 NAH";
$resn_exch{" H2A NAH"}=" AH2 NAH"; $resn_exch{"H51N NAH"}="NH51 NAH"; $resn_exch{"H52N NAH"}="NH52 NAH";
$resn_exch{" H4D NAH"}="NH4* NAH"; $resn_exch{" H3D NAH"}="NH3* NAH"; $resn_exch{"HO3N NAH"}="NHO3 NAH";
$resn_exch{" H2D NAH"}="NH2* NAH"; $resn_exch{"HO2N NAH"}="NHO2 NAH"; $resn_exch{" H1D NAH"}="NH1* NAH";
$resn_exch{" H2N NAH"}=" NH2 NAH"; $resn_exch{"H71N NAH"}="NH71 NAH"; $resn_exch{"H72N NAH"}="NH72 NAH";
$resn_exch{" H4N NAH"}=" NH4 NAH"; $resn_exch{" H5N NAH"}=" NH5 NAH"; $resn_exch{" H6N NAH"}=" NH6 NAH";
$resn_exch{"HOA2 NAI"}="2HOA NAI"; $resn_exch{"H51A NAI"}="AH51 NAI"; $resn_exch{"H52A NAI"}="AH52 NAI";
$resn_exch{" H4B NAI"}="AH4* NAI"; $resn_exch{" H3B NAI"}="AH3* NAI"; $resn_exch{"HO3A NAI"}="AHO3 NAI";
$resn_exch{" H2B NAI"}="AH2* NAI"; $resn_exch{"HO2A NAI"}="AHO2 NAI"; $resn_exch{" H1B NAI"}="AH1* NAI";
$resn_exch{" H8A NAI"}=" AH8 NAI"; $resn_exch{"H61A NAI"}="AH61 NAI"; $resn_exch{"H62A NAI"}="AH62 NAI";
$resn_exch{" H2A NAI"}=" AH2 NAI"; $resn_exch{"HO1N NAI"}="1NHO NAI"; $resn_exch{"H51N NAI"}="NH51 NAI";
$resn_exch{"H52N NAI"}="NH52 NAI"; $resn_exch{" H4D NAI"}="NH4* NAI"; $resn_exch{" H3D NAI"}="NH3* NAI";
$resn_exch{"HO3N NAI"}="NHO3 NAI"; $resn_exch{" H2D NAI"}="NH2* NAI"; $resn_exch{"HO2N NAI"}="NHO2 NAI";
$resn_exch{" H1D NAI"}="NH1* NAI"; $resn_exch{" H2N NAI"}=" NH2 NAI"; $resn_exch{"H71N NAI"}="NH71 NAI";
$resn_exch{"H72N NAI"}="NH72 NAI"; $resn_exch{" H4N NAI"}=" NH4 NAI"; $resn_exch{"H42N NAI"}="2NH4 NAI";
$resn_exch{" H5N NAI"}=" NH5 NAI"; $resn_exch{" H6N NAI"}=" NH6 NAI"; $resn_exch{"H51A NAJ"}="AH51 NAJ";
$resn_exch{"H52A NAJ"}="AH52 NAJ"; $resn_exch{" H4B NAJ"}="AH4* NAJ"; $resn_exch{" H3B NAJ"}="AH3* NAJ";
$resn_exch{"HO3A NAJ"}="AHO3 NAJ"; $resn_exch{" H2B NAJ"}="AH2* NAJ"; $resn_exch{"HO2A NAJ"}="AHO2 NAJ";
$resn_exch{" H1B NAJ"}="AH1* NAJ"; $resn_exch{" H8A NAJ"}=" AH8 NAJ"; $resn_exch{"H61A NAJ"}="AH61 NAJ";
$resn_exch{"H62A NAJ"}="AH62 NAJ"; $resn_exch{" H1A NAJ"}=" AH1 NAJ"; $resn_exch{" H2A NAJ"}=" AH2 NAJ";
$resn_exch{"H51N NAJ"}="NH51 NAJ"; $resn_exch{"H52N NAJ"}="NH52 NAJ"; $resn_exch{" H4D NAJ"}="NH4* NAJ";
$resn_exch{" H3D NAJ"}="NH3* NAJ"; $resn_exch{"HO3N NAJ"}="NHO3 NAJ"; $resn_exch{" H2D NAJ"}="NH2* NAJ";
$resn_exch{"HO2N NAJ"}="NHO2 NAJ"; $resn_exch{" H1D NAJ"}="NH1* NAJ"; $resn_exch{" H2N NAJ"}=" NH2 NAJ";
$resn_exch{"H71N NAJ"}="NH71 NAJ"; $resn_exch{"H72N NAJ"}="NH72 NAJ"; $resn_exch{" H4N NAJ"}=" NH4 NAJ";
$resn_exch{" H5N NAJ"}=" NH5 NAJ"; $resn_exch{" H6N NAJ"}=" NH6 NAJ"; $resn_exch{"HAE1 NAK"}="1HAE NAK";
$resn_exch{"HAE2 NAK"}="2HAE NAK"; $resn_exch{"HAE3 NAK"}="3HAE NAK"; $resn_exch{"HAB1 NAK"}="1HAB NAK";
$resn_exch{"HAB2 NAK"}="2HAB NAK"; $resn_exch{" H91 NAL"}="1H9  NAL"; $resn_exch{" H92 NAL"}="2H9  NAL";
$resn_exch{" HN1 NAL"}="1HN  NAL"; $resn_exch{" HN2 NAL"}="2HN  NAL"; $resn_exch{" H   NAM"}="1HN  NAM";
$resn_exch{" H2  NAM"}="2HN  NAM"; $resn_exch{" HB2 NAM"}="1HB  NAM"; $resn_exch{" HB3 NAM"}="2HB  NAM";
$resn_exch{"HNX1 NAM"}="1HNX NAM"; $resn_exch{"HNX2 NAM"}="2HNX NAM"; $resn_exch{" H31 NAN"}="1H3  NAN";
$resn_exch{" H32 NAN"}="2H3  NAN"; $resn_exch{" H91 NAN"}="1H9  NAN"; $resn_exch{" H92 NAN"}="2H9  NAN";
$resn_exch{"H111 NAN"}="1H11 NAN"; $resn_exch{"H112 NAN"}="2H11 NAN"; $resn_exch{"H113 NAN"}="3H11 NAN";
$resn_exch{"HOB1 NAN"}="1HOB NAN"; $resn_exch{"HO11 NAO"}="1HO1 NAO"; $resn_exch{"HO12 NAO"}="2HO1 NAO";
$resn_exch{"HOA2 NAP"}="2HOA NAP"; $resn_exch{"H51A NAP"}="AH51 NAP"; $resn_exch{"H52A NAP"}="AH52 NAP";
$resn_exch{" H4B NAP"}="AH4* NAP"; $resn_exch{" H3B NAP"}="AH3* NAP"; $resn_exch{"HO3A NAP"}="AHO3 NAP";
$resn_exch{" H2B NAP"}="AH2* NAP"; $resn_exch{" H1B NAP"}="AH1* NAP"; $resn_exch{" H8A NAP"}=" AH8 NAP";
$resn_exch{"H61A NAP"}="AH61 NAP"; $resn_exch{"H62A NAP"}="AH62 NAP"; $resn_exch{" H2A NAP"}=" AH2 NAP";
$resn_exch{"H51N NAP"}="NH51 NAP"; $resn_exch{"H52N NAP"}="NH52 NAP"; $resn_exch{" H4D NAP"}="NH4* NAP";
$resn_exch{" H3D NAP"}="NH3* NAP"; $resn_exch{"HO3N NAP"}="NHO3 NAP"; $resn_exch{" H2D NAP"}="NH2* NAP";
$resn_exch{"HO2N NAP"}="NHO2 NAP"; $resn_exch{" H1D NAP"}="NH1* NAP"; $resn_exch{" H2N NAP"}=" NH2 NAP";
$resn_exch{"H71N NAP"}="NH71 NAP"; $resn_exch{"H72N NAP"}="NH72 NAP"; $resn_exch{" H4N NAP"}=" NH4 NAP";
$resn_exch{" H5N NAP"}=" NH5 NAP"; $resn_exch{" H6N NAP"}=" NH6 NAP"; $resn_exch{"HOP2 NAP"}="2HOP NAP";
$resn_exch{"HOP3 NAP"}="3HOP NAP"; $resn_exch{"HOA2 NAQ"}="2HOA NAQ"; $resn_exch{"H5'1 NAQ"}="1H5' NAQ";
$resn_exch{"H5'2 NAQ"}="2H5' NAQ"; $resn_exch{"HO'3 NAQ"}="3HO' NAQ"; $resn_exch{"HO'2 NAQ"}="2HO' NAQ";
$resn_exch{"H6A1 NAQ"}="1H6A NAQ"; $resn_exch{"H6A2 NAQ"}="2H6A NAQ"; $resn_exch{"H5A1 NAQ"}="1H5* NAQ";
$resn_exch{"H5A2 NAQ"}="2H5* NAQ"; $resn_exch{" H4B NAQ"}=" H4* NAQ"; $resn_exch{" H3B NAQ"}=" H3* NAQ";
$resn_exch{"HOA3 NAQ"}="3HO* NAQ"; $resn_exch{" H2B NAQ"}=" H2* NAQ"; $resn_exch{"HOB2 NAQ"}="2HO* NAQ";
$resn_exch{" H1B NAQ"}=" H1* NAQ"; $resn_exch{"H7N1 NAQ"}="1H7N NAQ"; $resn_exch{"H7N2 NAQ"}="2H7N NAQ";
$resn_exch{"H5P1 NAQ"}="1H5P NAQ"; $resn_exch{"H5P2 NAQ"}="2H5P NAQ"; $resn_exch{"H5P3 NAQ"}="3H5P NAQ";
$resn_exch{"H2P1 NAQ"}="1H2P NAQ"; $resn_exch{"H2P2 NAQ"}="2H2P NAQ"; $resn_exch{"H1P1 NAQ"}="1H1P NAQ";
$resn_exch{"H1P2 NAQ"}="2H1P NAQ"; $resn_exch{"H1P3 NAQ"}="3H1P NAQ"; $resn_exch{" H11 NAR"}="1H1  NAR";
$resn_exch{" H31 NAR"}="1H3  NAR"; $resn_exch{" H81 NAR"}="1H8  NAR"; $resn_exch{" H82 NAR"}="2H8  NAR";
$resn_exch{"H111 NAR"}="1H11 NAR"; $resn_exch{"H121 NAR"}="1H12 NAR"; $resn_exch{"H141 NAR"}="1H14 NAR";
$resn_exch{"H151 NAR"}="1H15 NAR"; $resn_exch{" H41 NAR"}="1H4  NAR"; $resn_exch{" H51 NAR"}="1H5  NAR";
$resn_exch{"HOS3 NAS"}="3HOS NAS"; $resn_exch{"H161 NAT"}="1H16 NAT"; $resn_exch{"H162 NAT"}="2H16 NAT";
$resn_exch{"H171 NAT"}="1H17 NAT"; $resn_exch{"H172 NAT"}="2H17 NAT"; $resn_exch{"H173 NAT"}="3H17 NAT";
$resn_exch{"H181 NAT"}="1H18 NAT"; $resn_exch{"H182 NAT"}="2H18 NAT"; $resn_exch{"H183 NAT"}="3H18 NAT";
$resn_exch{"HC21 NAU"}="1HC2 NAU"; $resn_exch{"HC22 NAU"}="2HC2 NAU"; $resn_exch{"HC31 NAU"}="1HC3 NAU";
$resn_exch{"HC32 NAU"}="2HC3 NAU"; $resn_exch{"H111 NAU"}="1H11 NAU"; $resn_exch{"H112 NAU"}="2H11 NAU";
$resn_exch{"H113 NAU"}="3H11 NAU"; $resn_exch{"HC91 NAU"}="1HC9 NAU"; $resn_exch{"HC92 NAU"}="2HC9 NAU";
$resn_exch{"HC21 NAV"}="1HC2 NAV"; $resn_exch{"HC22 NAV"}="2HC2 NAV"; $resn_exch{"HC31 NAV"}="1HC3 NAV";
$resn_exch{"HC32 NAV"}="2HC3 NAV"; $resn_exch{"H111 NAV"}="1H11 NAV"; $resn_exch{"H112 NAV"}="2H11 NAV";
$resn_exch{"H113 NAV"}="3H11 NAV"; $resn_exch{"HC91 NAV"}="1HC9 NAV"; $resn_exch{"HC92 NAV"}="2HC9 NAV";
$resn_exch{"HO11 NAW"}="1HO1 NAW"; $resn_exch{"HO12 NAW"}="2HO1 NAW"; $resn_exch{"HO21 NAW"}="1HO2 NAW";
$resn_exch{"HO22 NAW"}="2HO2 NAW"; $resn_exch{"HO31 NAW"}="1HO3 NAW"; $resn_exch{"HO32 NAW"}="2HO3 NAW";
$resn_exch{"HOA2 NAX"}="2HOA NAX"; $resn_exch{"H51A NAX"}="AH51 NAX"; $resn_exch{"H52A NAX"}="AH52 NAX";
$resn_exch{" H4B NAX"}="AH4* NAX"; $resn_exch{" H3B NAX"}="AH3* NAX"; $resn_exch{"HO3A NAX"}="AHO3 NAX";
$resn_exch{" H2B NAX"}="AH2* NAX"; $resn_exch{"HO2A NAX"}="AHO2 NAX"; $resn_exch{" H1B NAX"}="AH1* NAX";
$resn_exch{" H8A NAX"}=" AH8 NAX"; $resn_exch{"H61A NAX"}="AH61 NAX"; $resn_exch{"H62A NAX"}="AH62 NAX";
$resn_exch{" H2A NAX"}=" AH2 NAX"; $resn_exch{"HNO2 NAX"}="2HNO NAX"; $resn_exch{"H5'1 NAX"}="1H5* NAX";
$resn_exch{"H5'2 NAX"}="2H5* NAX"; $resn_exch{" H4D NAX"}="NH4* NAX"; $resn_exch{" H3D NAX"}="NH3* NAX";
$resn_exch{"HO3N NAX"}="NHO3 NAX"; $resn_exch{" H2D NAX"}="NH2* NAX"; $resn_exch{"HO2N NAX"}="NHO2 NAX";
$resn_exch{" H1D NAX"}="NH1* NAX"; $resn_exch{" H2N NAX"}=" NH2 NAX"; $resn_exch{"H71N NAX"}="NH71 NAX";
$resn_exch{"H72N NAX"}="NH72 NAX"; $resn_exch{"H41N NAX"}="NH41 NAX"; $resn_exch{"H42N NAX"}="NH42 NAX";
$resn_exch{"H51N NAX"}="NH51 NAX"; $resn_exch{"H52N NAX"}="NH52 NAX"; $resn_exch{" H6N NAX"}=" NH6 NAX";
$resn_exch{"HO6N NAX"}="NHO6 NAX"; $resn_exch{"HC31 NAY"}="1HC3 NAY"; $resn_exch{"HC32 NAY"}="2HC3 NAY";
$resn_exch{"HC41 NAY"}="1HC4 NAY"; $resn_exch{"HC42 NAY"}="2HC4 NAY"; $resn_exch{"H111 NAY"}="1H11 NAY";
$resn_exch{"H112 NAY"}="2H11 NAY"; $resn_exch{"H113 NAY"}="3H11 NAY"; $resn_exch{"HC91 NAY"}="1HC9 NAY";
$resn_exch{"HC92 NAY"}="2HC9 NAY"; $resn_exch{"H6C1 NB1"}="1H6C NB1"; $resn_exch{"H6C2 NB1"}="2H6C NB1";
$resn_exch{"H8C1 NB1"}="1H8C NB1"; $resn_exch{"H8C2 NB1"}="2H8C NB1"; $resn_exch{"H9C1 NB1"}="1H9C NB1";
$resn_exch{"H9C2 NB1"}="2H9C NB1"; $resn_exch{"HMC1 NB1"}="1HMC NB1"; $resn_exch{"HMC2 NB1"}="2HMC NB1";
$resn_exch{"HMC3 NB1"}="3HMC NB1"; $resn_exch{" HO4 NBA"}="4HO  NBA"; $resn_exch{" HC4 NBA"}="4HC  NBA";
$resn_exch{"HC51 NBA"}="1HC5 NBA"; $resn_exch{"HC52 NBA"}="2HC5 NBA"; $resn_exch{"HC61 NBA"}="1HC6 NBA";
$resn_exch{"HC62 NBA"}="2HC6 NBA"; $resn_exch{" HN7 NBA"}="7HN  NBA"; $resn_exch{"HC81 NBA"}="1HC8 NBA";
$resn_exch{"HC82 NBA"}="2HC8 NBA"; $resn_exch{"HC91 NBA"}="1HC9 NBA"; $resn_exch{"HC92 NBA"}="2HC9 NBA";
$resn_exch{"H101 NBA"}="1H10 NBA"; $resn_exch{"H102 NBA"}="2H10 NBA"; $resn_exch{"HN11 NBA"}="1HN1 NBA";
$resn_exch{"H121 NBA"}="1H12 NBA"; $resn_exch{"H122 NBA"}="2H12 NBA"; $resn_exch{"H131 NBA"}="1H13 NBA";
$resn_exch{"H132 NBA"}="2H13 NBA"; $resn_exch{"H141 NBA"}="1H14 NBA"; $resn_exch{"H142 NBA"}="2H14 NBA";
$resn_exch{"HN51 NBA"}="1HN5 NBA"; $resn_exch{"H161 NBA"}="1H16 NBA"; $resn_exch{"H162 NBA"}="2H16 NBA";
$resn_exch{"H163 NBA"}="3H16 NBA"; $resn_exch{"HOL4 NBA"}="4HOL NBA"; $resn_exch{"H111 NBB"}="1H11 NBB";
$resn_exch{"H112 NBB"}="2H11 NBB"; $resn_exch{"H121 NBB"}="1H12 NBB"; $resn_exch{"H122 NBB"}="2H12 NBB";
$resn_exch{"H131 NBB"}="1H13 NBB"; $resn_exch{"H132 NBB"}="2H13 NBB"; $resn_exch{"H141 NBB"}="1H14 NBB";
$resn_exch{"H142 NBB"}="2H14 NBB"; $resn_exch{"H143 NBB"}="3H14 NBB"; $resn_exch{" H91 NBC"}="1H9  NBC";
$resn_exch{" H71 NBC"}="1H7  NBC"; $resn_exch{" H81 NBC"}="1H8  NBC"; $resn_exch{" H51 NBC"}="1H5  NBC";
$resn_exch{" H61 NBC"}="1H6  NBC"; $resn_exch{"H31A NBC"}="1H3  NBC"; $resn_exch{"H41A NBC"}="1H4  NBC";
$resn_exch{"H11A NBC"}="1H1  NBC"; $resn_exch{"H21A NBC"}="1H2  NBC"; $resn_exch{"HOA2 NBD"}="2HOA NBD";
$resn_exch{"H51A NBD"}="AH51 NBD"; $resn_exch{"H52A NBD"}="AH52 NBD"; $resn_exch{" H4B NBD"}="AH4* NBD";
$resn_exch{" H3B NBD"}="AH3* NBD"; $resn_exch{"HO3A NBD"}="AHO3 NBD"; $resn_exch{" H2B NBD"}="AH2* NBD";
$resn_exch{"HO2A NBD"}="AHO2 NBD"; $resn_exch{" H1B NBD"}="AH1* NBD"; $resn_exch{" H8A NBD"}=" AH8 NBD";
$resn_exch{" H6A NBD"}=" AH6 NBD"; $resn_exch{" H2A NBD"}=" AH2 NBD"; $resn_exch{"H51N NBD"}="NH51 NBD";
$resn_exch{"H52N NBD"}="NH52 NBD"; $resn_exch{" H4D NBD"}="NH4* NBD"; $resn_exch{" H3D NBD"}="NH3* NBD";
$resn_exch{"HO3N NBD"}="NHO3 NBD"; $resn_exch{" H2D NBD"}="NH2* NBD"; $resn_exch{"HO2N NBD"}="NHO2 NBD";
$resn_exch{" H1D NBD"}="NH1* NBD"; $resn_exch{" H2N NBD"}=" NH2 NBD"; $resn_exch{"H71N NBD"}="NH71 NBD";
$resn_exch{"H72N NBD"}="NH72 NBD"; $resn_exch{" H4N NBD"}=" NH4 NBD"; $resn_exch{" H5N NBD"}=" NH5 NBD";
$resn_exch{" H6N NBD"}=" NH6 NBD"; $resn_exch{"HB11 NBD"}="1HB1 NBD"; $resn_exch{"HB12 NBD"}="2HB1 NBD";
$resn_exch{"H251 NBF"}="1H25 NBF"; $resn_exch{"H252 NBF"}="2H25 NBF"; $resn_exch{"H253 NBF"}="3H25 NBF";
$resn_exch{"H241 NBF"}="1H24 NBF"; $resn_exch{"H242 NBF"}="2H24 NBF"; $resn_exch{" H71 NBF"}="1H7  NBF";
$resn_exch{" H72 NBF"}="2H7  NBF"; $resn_exch{" H61 NBG"}="1H6  NBG"; $resn_exch{" H62 NBG"}="2H6  NBG";
$resn_exch{" H81 NBG"}="1H8  NBG"; $resn_exch{" H82 NBG"}="2H8  NBG"; $resn_exch{" H83 NBG"}="3H8  NBG";
$resn_exch{" H11 NBL"}="1H1  NBL"; $resn_exch{"H12A NBL"}="2H1  NBL"; $resn_exch{"H13A NBL"}="3H1  NBL";
$resn_exch{" H31 NBL"}="1H3  NBL"; $resn_exch{" H32 NBL"}="2H3  NBL"; $resn_exch{" H33 NBL"}="3H3  NBL";
$resn_exch{" H41 NBL"}="1H4  NBL"; $resn_exch{" H42 NBL"}="2H4  NBL"; $resn_exch{"H101 NBL"}="1H10 NBL";
$resn_exch{"H102 NBL"}="2H10 NBL"; $resn_exch{"H211 NBL"}="1H21 NBL"; $resn_exch{"H212 NBL"}="2H21 NBL";
$resn_exch{"H221 NBL"}="1H22 NBL"; $resn_exch{"H222 NBL"}="2H22 NBL"; $resn_exch{"H241 NBL"}="1H24 NBL";
$resn_exch{"H242 NBL"}="2H24 NBL"; $resn_exch{" H11 NBN"}="1H1  NBN"; $resn_exch{" H12 NBN"}="2H1  NBN";
$resn_exch{" H21 NBN"}="1H2  NBN"; $resn_exch{" H22 NBN"}="2H2  NBN"; $resn_exch{" H31 NBN"}="1H3  NBN";
$resn_exch{" H32 NBN"}="2H3  NBN"; $resn_exch{" H41 NBN"}="1H4  NBN"; $resn_exch{" H42 NBN"}="2H4  NBN";
$resn_exch{" H43 NBN"}="3H4  NBN"; $resn_exch{"HOA2 NBP"}="2HOA NBP"; $resn_exch{"H51A NBP"}="AH51 NBP";
$resn_exch{"H52A NBP"}="AH52 NBP"; $resn_exch{" H4B NBP"}="AH4* NBP"; $resn_exch{" H3B NBP"}="AH3* NBP";
$resn_exch{"HO3A NBP"}="AHO3 NBP"; $resn_exch{" H2B NBP"}="AH2* NBP"; $resn_exch{" H1B NBP"}="AH1* NBP";
$resn_exch{"H61A NBP"}="AH61 NBP"; $resn_exch{"H62A NBP"}="AH62 NBP"; $resn_exch{" H2A NBP"}=" AH2 NBP";
$resn_exch{"H51N NBP"}="NH51 NBP"; $resn_exch{"H52N NBP"}="NH52 NBP"; $resn_exch{" H4D NBP"}="NH4* NBP";
$resn_exch{" H3D NBP"}="NH3* NBP"; $resn_exch{"HO3N NBP"}="NHO3 NBP"; $resn_exch{" H2D NBP"}="NH2* NBP";
$resn_exch{"HO2N NBP"}="NHO2 NBP"; $resn_exch{" H1D NBP"}="NH1* NBP"; $resn_exch{" H2N NBP"}=" NH2 NBP";
$resn_exch{"H71N NBP"}="NH71 NBP"; $resn_exch{"H72N NBP"}="NH72 NBP"; $resn_exch{" H4N NBP"}=" NH4 NBP";
$resn_exch{" H5N NBP"}=" NH5 NBP"; $resn_exch{" H6N NBP"}=" NH6 NBP"; $resn_exch{"HOP2 NBP"}="2HOP NBP";
$resn_exch{"HOP3 NBP"}="3HOP NBP"; $resn_exch{" H2  NBQ"}=" HN  NBQ"; $resn_exch{" HB2 NBQ"}=" HB1 NBQ";
$resn_exch{" HB3 NBQ"}=" HB2 NBQ"; $resn_exch{"HAE1 NBQ"}="1HAE NBQ"; $resn_exch{"HAE2 NBQ"}="2HAE NBQ";
$resn_exch{"HOP4 NBS"}="4HOP NBS"; $resn_exch{"HOP5 NBS"}="5HOP NBS"; $resn_exch{"HOP1 NBS"}="1HOP NBS";
$resn_exch{"H5'1 NBS"}="1H5* NBS"; $resn_exch{"H5'2 NBS"}="2H5* NBS"; $resn_exch{" H4' NBS"}=" H4* NBS";
$resn_exch{" H1' NBS"}=" H1* NBS"; $resn_exch{"HB11 NBS"}="1HB1 NBS"; $resn_exch{"HB12 NBS"}="2HB1 NBS";
$resn_exch{" H2' NBS"}=" H2* NBS"; $resn_exch{"HO'2 NBS"}="2HO* NBS"; $resn_exch{" H3' NBS"}=" H3* NBS";
$resn_exch{"HO'3 NBS"}="3HO* NBS"; $resn_exch{"H1C1 NBT"}="1H1C NBT"; $resn_exch{"H1C2 NBT"}="2H1C NBT";
$resn_exch{"H1C3 NBT"}="3H1C NBT"; $resn_exch{"H3C1 NBT"}="1H3C NBT"; $resn_exch{"H3C2 NBT"}="2H3C NBT";
$resn_exch{"H4C1 NBT"}="1H4C NBT"; $resn_exch{"H4C2 NBT"}="2H4C NBT"; $resn_exch{"H4C3 NBT"}="3H4C NBT";
$resn_exch{"H121 NBV"}="1H12 NBV"; $resn_exch{"H122 NBV"}="2H12 NBV"; $resn_exch{"H7C1 NBV"}="1H7C NBV";
$resn_exch{"H7C2 NBV"}="2H7C NBV"; $resn_exch{"H131 NBV"}="1H13 NBV"; $resn_exch{"H132 NBV"}="2H13 NBV";
$resn_exch{"H141 NBV"}="1H14 NBV"; $resn_exch{"H142 NBV"}="2H14 NBV"; $resn_exch{"H151 NBV"}="1H15 NBV";
$resn_exch{"H152 NBV"}="2H15 NBV"; $resn_exch{"H161 NBV"}="1H16 NBV"; $resn_exch{"H162 NBV"}="2H16 NBV";
$resn_exch{"H163 NBV"}="3H16 NBV"; $resn_exch{" H11  NC"}="1H1   NC"; $resn_exch{" H12  NC"}="2H1   NC";
$resn_exch{" H13  NC"}="3H1   NC"; $resn_exch{"HC28 NC1"}="H2   NC1"; $resn_exch{" H   NC1"}=" H20 NC1";
$resn_exch{" H2  NC1"}=" H21 NC1"; $resn_exch{" HB2 NC1"}=" H17 NC1"; $resn_exch{" HB3 NC1"}=" H18 NC1";
$resn_exch{"H6C1 NC2"}="1H6C NC2"; $resn_exch{"H6C2 NC2"}="2H6C NC2"; $resn_exch{"H131 NC2"}="1H13 NC2";
$resn_exch{"H132 NC2"}="2H13 NC2"; $resn_exch{"H221 NC2"}="1H22 NC2"; $resn_exch{"H231 NC2"}="1H23 NC2";
$resn_exch{" HN1 NC2"}="1HN  NC2"; $resn_exch{" HN2 NC2"}="2HN  NC2"; $resn_exch{"HBC1 NC2"}="1HBC NC2";
$resn_exch{"HBC2 NC2"}="2HBC NC2"; $resn_exch{" H51 NC3"}="1H5  NC3"; $resn_exch{" H52 NC3"}="2H5  NC3";
$resn_exch{"H101 NC3"}="1H10 NC3"; $resn_exch{"H102 NC3"}="2H10 NC3"; $resn_exch{"H111 NC3"}="1H11 NC3";
$resn_exch{"H112 NC3"}="2H11 NC3"; $resn_exch{"H121 NC3"}="1H12 NC3"; $resn_exch{"H122 NC3"}="2H12 NC3";
$resn_exch{"H131 NC3"}="1H13 NC3"; $resn_exch{"H132 NC3"}="2H13 NC3"; $resn_exch{"H141 NC3"}="1H14 NC3";
$resn_exch{"H142 NC3"}="2H14 NC3"; $resn_exch{" H51 NC4"}="1H5  NC4"; $resn_exch{" H52 NC4"}="2H5  NC4";
$resn_exch{" H61 NC4"}="1H6  NC4"; $resn_exch{" H62 NC4"}="2H6  NC4"; $resn_exch{" H71 NC4"}="1H7  NC4";
$resn_exch{" H72 NC4"}="2H7  NC4"; $resn_exch{"H121 NC4"}="1H12 NC4"; $resn_exch{"H122 NC4"}="2H12 NC4";
$resn_exch{"H131 NC4"}="1H13 NC4"; $resn_exch{"H132 NC4"}="2H13 NC4"; $resn_exch{"H141 NC4"}="1H14 NC4";
$resn_exch{"H142 NC4"}="2H14 NC4"; $resn_exch{"H151 NC4"}="1H15 NC4"; $resn_exch{"H152 NC4"}="2H15 NC4";
$resn_exch{"H161 NC4"}="1H16 NC4"; $resn_exch{"H162 NC4"}="2H16 NC4"; $resn_exch{" H51 NC6"}="1H5  NC6";
$resn_exch{" H52 NC6"}="2H5  NC6"; $resn_exch{" H61 NC6"}="1H6  NC6"; $resn_exch{" H62 NC6"}="2H6  NC6";
$resn_exch{" H71 NC6"}="1H7  NC6"; $resn_exch{" H72 NC6"}="2H7  NC6"; $resn_exch{" H81 NC6"}="1H8  NC6";
$resn_exch{" H82 NC6"}="2H8  NC6"; $resn_exch{" H91 NC6"}="1H9  NC6"; $resn_exch{" H92 NC6"}="2H9  NC6";
$resn_exch{"H141 NC6"}="1H14 NC6"; $resn_exch{"H142 NC6"}="2H14 NC6"; $resn_exch{"H151 NC6"}="1H15 NC6";
$resn_exch{"H152 NC6"}="2H15 NC6"; $resn_exch{"H161 NC6"}="1H16 NC6"; $resn_exch{"H162 NC6"}="2H16 NC6";
$resn_exch{"H171 NC6"}="1H17 NC6"; $resn_exch{"H172 NC6"}="2H17 NC6"; $resn_exch{"H181 NC6"}="1H18 NC6";
$resn_exch{"H182 NC6"}="2H18 NC6"; $resn_exch{" H51 NC7"}="1H5  NC7"; $resn_exch{" H52 NC7"}="2H5  NC7";
$resn_exch{" H61 NC7"}="1H6  NC7"; $resn_exch{" H62 NC7"}="2H6  NC7"; $resn_exch{" H71 NC7"}="1H7  NC7";
$resn_exch{" H72 NC7"}="2H7  NC7"; $resn_exch{" H81 NC7"}="1H8  NC7"; $resn_exch{" H82 NC7"}="2H8  NC7";
$resn_exch{" H91 NC7"}="1H9  NC7"; $resn_exch{" H92 NC7"}="2H9  NC7"; $resn_exch{"H111 NC7"}="1H11 NC7";
$resn_exch{"H112 NC7"}="2H11 NC7"; $resn_exch{"H151 NC7"}="1H15 NC7"; $resn_exch{"H152 NC7"}="2H15 NC7";
$resn_exch{"H161 NC7"}="1H16 NC7"; $resn_exch{"H162 NC7"}="2H16 NC7"; $resn_exch{"H171 NC7"}="1H17 NC7";
$resn_exch{"H172 NC7"}="2H17 NC7"; $resn_exch{"H181 NC7"}="1H18 NC7"; $resn_exch{"H182 NC7"}="2H18 NC7";
$resn_exch{"H191 NC7"}="1H19 NC7"; $resn_exch{"H192 NC7"}="2H19 NC7"; $resn_exch{"HN71 NCA"}="1HN7 NCA";
$resn_exch{"HN72 NCA"}="2HN7 NCA"; $resn_exch{" HXT NCB"}=" HO  NCB"; $resn_exch{" HB3 NCB"}="3HB  NCB";
$resn_exch{" HB2 NCB"}="2HB  NCB"; $resn_exch{" HB1 NCB"}="1HB  NCB"; $resn_exch{" H22 NCB"}="2H2  NCB";
$resn_exch{" H21 NCB"}="1H2  NCB"; $resn_exch{"H5'1 NCC"}="1H5' NCC"; $resn_exch{"H5'2 NCC"}="2H5' NCC";
$resn_exch{"HN41 NCC"}="1HN4 NCC"; $resn_exch{"HN42 NCC"}="2HN4 NCC"; $resn_exch{"H151 NCC"}="1H15 NCC";
$resn_exch{"H152 NCC"}="2H15 NCC"; $resn_exch{"H181 NCC"}="1H18 NCC"; $resn_exch{"H182 NCC"}="2H18 NCC";
$resn_exch{"H183 NCC"}="3H18 NCC"; $resn_exch{"H201 NCC"}="1H20 NCC"; $resn_exch{"H202 NCC"}="2H20 NCC";
$resn_exch{" H61 NCD"}="1H6  NCD"; $resn_exch{"H611 NCD"}="1H61 NCD"; $resn_exch{" H11 NCD"}="1H1  NCD";
$resn_exch{" H31 NCD"}="1H3  NCD"; $resn_exch{" H32 NCD"}="2H3  NCD"; $resn_exch{" H51 NCD"}="1H5  NCD";
$resn_exch{" H52 NCD"}="2H5  NCD"; $resn_exch{"HO51 NCD"}="1HO5 NCD"; $resn_exch{"H2C1 NCF"}="1H2C NCF";
$resn_exch{"H2C2 NCF"}="2H2C NCF"; $resn_exch{"H3C1 NCF"}="1H3C NCF"; $resn_exch{"H3C2 NCF"}="2H3C NCF";
$resn_exch{"H131 NCF"}="1H13 NCF"; $resn_exch{"H132 NCF"}="2H13 NCF"; $resn_exch{" H8B NCG"}="H8'' NCG";
$resn_exch{" H6B NCG"}="H6'' NCG"; $resn_exch{" H3B NCG"}="H3'' NCG"; $resn_exch{" H4B NCG"}="H4'' NCG";
$resn_exch{" H11 NCH"}="1H1  NCH"; $resn_exch{" H12 NCH"}="2H1  NCH"; $resn_exch{" H21 NCH"}="1H2  NCH";
$resn_exch{" H22 NCH"}="2H2  NCH"; $resn_exch{" H31 NCH"}="1H3  NCH"; $resn_exch{" H32 NCH"}="2H3  NCH";
$resn_exch{" H33 NCH"}="3H3  NCH"; $resn_exch{" H51 NCH"}="1H5  NCH"; $resn_exch{" H52 NCH"}="2H5  NCH";
$resn_exch{" H53 NCH"}="3H5  NCH"; $resn_exch{" H41 NCH"}="1H4  NCH"; $resn_exch{" H42 NCH"}="2H4  NCH";
$resn_exch{" H43 NCH"}="3H4  NCH"; $resn_exch{" H31 NCM"}="1H3  NCM"; $resn_exch{" H32 NCM"}="2H3  NCM";
$resn_exch{" H51 NCM"}="1H5  NCM"; $resn_exch{" H52 NCM"}="2H5  NCM"; $resn_exch{" H61 NCM"}="1H6  NCM";
$resn_exch{" H62 NCM"}="2H6  NCM"; $resn_exch{" H71 NCM"}="1H7  NCM"; $resn_exch{" H72 NCM"}="2H7  NCM";
$resn_exch{"HOP1 NCN"}="1HOP NCN"; $resn_exch{"H5'1 NCN"}="1H5* NCN"; $resn_exch{"H5'2 NCN"}="2H5* NCN";
$resn_exch{" H4' NCN"}=" H4* NCN"; $resn_exch{" H3' NCN"}=" H3* NCN"; $resn_exch{"HO'3 NCN"}="3HO* NCN";
$resn_exch{" H2' NCN"}=" H2* NCN"; $resn_exch{"HO'2 NCN"}="2HO* NCN"; $resn_exch{" H1' NCN"}=" H1* NCN";
$resn_exch{"HN11 NCO"}="1HN1 NCO"; $resn_exch{"HN12 NCO"}="2HN1 NCO"; $resn_exch{"HN13 NCO"}="3HN1 NCO";
$resn_exch{"HN21 NCO"}="1HN2 NCO"; $resn_exch{"HN22 NCO"}="2HN2 NCO"; $resn_exch{"HN23 NCO"}="3HN2 NCO";
$resn_exch{"HN31 NCO"}="1HN3 NCO"; $resn_exch{"HN32 NCO"}="2HN3 NCO"; $resn_exch{"HN33 NCO"}="3HN3 NCO";
$resn_exch{"HN41 NCO"}="1HN4 NCO"; $resn_exch{"HN42 NCO"}="2HN4 NCO"; $resn_exch{"HN43 NCO"}="3HN4 NCO";
$resn_exch{"HN51 NCO"}="1HN5 NCO"; $resn_exch{"HN52 NCO"}="2HN5 NCO"; $resn_exch{"HN53 NCO"}="3HN5 NCO";
$resn_exch{"HN61 NCO"}="1HN6 NCO"; $resn_exch{"HN62 NCO"}="2HN6 NCO"; $resn_exch{"HN63 NCO"}="3HN6 NCO";
$resn_exch{"HN11 NCP"}="1HN1 NCP"; $resn_exch{"HN12 NCP"}="2HN1 NCP"; $resn_exch{"HN21 NCP"}="1HN2 NCP";
$resn_exch{"HN22 NCP"}="2HN2 NCP"; $resn_exch{" H71 NCR"}="1H7  NCR"; $resn_exch{" H72 NCR"}="2H7  NCR";
$resn_exch{" H73 NCR"}="3H7  NCR"; $resn_exch{"H171 NCS"}="1H17 NCS"; $resn_exch{"H172 NCS"}="2H17 NCS";
$resn_exch{"H301 NCS"}="1H30 NCS"; $resn_exch{"H302 NCS"}="2H30 NCS"; $resn_exch{"H303 NCS"}="3H30 NCS";
$resn_exch{"H311 NCS"}="1H31 NCS"; $resn_exch{"H312 NCS"}="2H31 NCS"; $resn_exch{"H313 NCS"}="3H31 NCS";
$resn_exch{" H1' NCS"}=" H1* NCS"; $resn_exch{" H2' NCS"}=" H2* NCS"; $resn_exch{"HN21 NCS"}="1HN2 NCS";
$resn_exch{"HN22 NCS"}="2HN2 NCS"; $resn_exch{"HM21 NCS"}="1HM2 NCS"; $resn_exch{"HM22 NCS"}="2HM2 NCS";
$resn_exch{"HM23 NCS"}="3HM2 NCS"; $resn_exch{" H3' NCS"}=" H3* NCS"; $resn_exch{"HO3' NCS"}="*HO3 NCS";
$resn_exch{" H4' NCS"}=" H4* NCS"; $resn_exch{"HO4' NCS"}="*HO4 NCS"; $resn_exch{" H5' NCS"}=" H5* NCS";
$resn_exch{"HM51 NCS"}="1HM5 NCS"; $resn_exch{"HM52 NCS"}="2HM5 NCS"; $resn_exch{"HM53 NCS"}="3HM5 NCS";
$resn_exch{"HC71 NCT"}="1HC7 NCT"; $resn_exch{"HC72 NCT"}="2HC7 NCT"; $resn_exch{"HC81 NCT"}="1HC8 NCT";
$resn_exch{"HC82 NCT"}="2HC8 NCT"; $resn_exch{"HC91 NCT"}="1HC9 NCT"; $resn_exch{"HC92 NCT"}="2HC9 NCT";
$resn_exch{"H101 NCT"}="1H10 NCT"; $resn_exch{"H102 NCT"}="2H10 NCT"; $resn_exch{"H103 NCT"}="3H10 NCT";
$resn_exch{"HOP3 NCX"}="3HOP NCX"; $resn_exch{"HOP2 NCX"}="2HOP NCX"; $resn_exch{" H5' NCX"}="1H5* NCX";
$resn_exch{"H5'' NCX"}="2H5* NCX"; $resn_exch{" H4' NCX"}=" H4* NCX"; $resn_exch{" H1' NCX"}=" H1* NCX";
$resn_exch{"HH71 NCX"}="1HH7 NCX"; $resn_exch{"HH72 NCX"}="2HH7 NCX"; $resn_exch{" H3' NCX"}=" H3* NCX";
$resn_exch{" H2' NCX"}="1H2* NCX"; $resn_exch{"H2'' NCX"}="2H2* NCX"; $resn_exch{"HO3' NCX"}="H3T  NCX";
$resn_exch{" H   NCY"}=" HN  NCY"; $resn_exch{" HB2 NCY"}="1HB  NCY"; $resn_exch{" HB3 NCY"}="2HB  NCY";
$resn_exch{"HCN1 NCY"}="1HCN NCY"; $resn_exch{"HCN2 NCY"}="2HCN NCY"; $resn_exch{"HCN3 NCY"}="3HCN NCY";
$resn_exch{" H1  NCZ"}="1H   NCZ"; $resn_exch{" H7  NCZ"}="7H   NCZ"; $resn_exch{" H8  NCZ"}="8H   NCZ";
$resn_exch{" H12 NCZ"}="2H1  NCZ"; $resn_exch{" H13 NCZ"}="3H1  NCZ"; $resn_exch{" H14 NCZ"}="4H1  NCZ";
$resn_exch{" H15 NCZ"}="5H1  NCZ"; $resn_exch{" H16 NCZ"}="6H1  NCZ"; $resn_exch{" H6  NCZ"}="6H   NCZ";
$resn_exch{" H5  NCZ"}="5H   NCZ"; $resn_exch{" H2  NCZ"}="2H   NCZ"; $resn_exch{" H3  NCZ"}="3H   NCZ";
$resn_exch{"H171 NCZ"}="1H17 NCZ"; $resn_exch{"H172 NCZ"}="2H17 NCZ"; $resn_exch{"H173 NCZ"}="3H17 NCZ";
$resn_exch{"H181 NCZ"}="1H18 NCZ"; $resn_exch{"H182 NCZ"}="2H18 NCZ"; $resn_exch{"H183 NCZ"}="3H18 NCZ";
$resn_exch{"HOA2 NDA"}="2HOA NDA"; $resn_exch{"H51A NDA"}="AH51 NDA"; $resn_exch{"H52A NDA"}="AH52 NDA";
$resn_exch{" H4B NDA"}="AH4* NDA"; $resn_exch{" H3B NDA"}="AH3* NDA"; $resn_exch{"HO3A NDA"}="AHO3 NDA";
$resn_exch{" H2B NDA"}="AH2* NDA"; $resn_exch{"HO2A NDA"}="AHO2 NDA"; $resn_exch{" H1B NDA"}="AH1* NDA";
$resn_exch{" H8A NDA"}=" AH8 NDA"; $resn_exch{"H61A NDA"}="AH61 NDA"; $resn_exch{"H62A NDA"}="AH62 NDA";
$resn_exch{" H2A NDA"}=" AH2 NDA"; $resn_exch{"H51N NDA"}="NH51 NDA"; $resn_exch{"H52N NDA"}="NH52 NDA";
$resn_exch{" H4D NDA"}="NH4* NDA"; $resn_exch{" H3D NDA"}="NH3* NDA"; $resn_exch{"HO3N NDA"}="NHO3 NDA";
$resn_exch{" H2D NDA"}="NH2* NDA"; $resn_exch{"HO2N NDA"}="NHO2 NDA"; $resn_exch{" H1D NDA"}="NH1* NDA";
$resn_exch{" H2N NDA"}=" NH2 NDA"; $resn_exch{"HC71 NDA"}="1HC7 NDA"; $resn_exch{"HC72 NDA"}="2HC7 NDA";
$resn_exch{"H71N NDA"}="NH71 NDA"; $resn_exch{"H72N NDA"}="NH72 NDA"; $resn_exch{" H4N NDA"}=" NH4 NDA";
$resn_exch{" H5N NDA"}=" NH5 NDA"; $resn_exch{" H6N NDA"}=" NH6 NDA"; $resn_exch{"HOA2 NDC"}="2HOA NDC";
$resn_exch{"H5'1 NDC"}="1H5' NDC"; $resn_exch{"H5'2 NDC"}="2H5' NDC"; $resn_exch{"HO'3 NDC"}="3HO' NDC";
$resn_exch{"HO'2 NDC"}="2HO' NDC"; $resn_exch{"H6A1 NDC"}="1H6A NDC"; $resn_exch{"H6A2 NDC"}="2H6A NDC";
$resn_exch{"H5A1 NDC"}="1H5* NDC"; $resn_exch{"H5A2 NDC"}="2H5* NDC"; $resn_exch{" H4B NDC"}=" H4* NDC";
$resn_exch{" H3B NDC"}=" H3* NDC"; $resn_exch{"HOA3 NDC"}="3HO* NDC"; $resn_exch{" H2B NDC"}=" H2* NDC";
$resn_exch{"HOB2 NDC"}="2HO* NDC"; $resn_exch{" H1B NDC"}=" H1* NDC"; $resn_exch{"H7N1 NDC"}="1H7N NDC";
$resn_exch{"H7N2 NDC"}="2H7N NDC"; $resn_exch{"HC21 NDC"}="1HC2 NDC"; $resn_exch{"HC22 NDC"}="2HC2 NDC";
$resn_exch{"HC31 NDC"}="1HC3 NDC"; $resn_exch{"HC32 NDC"}="2HC3 NDC"; $resn_exch{"HC41 NDC"}="1HC4 NDC";
$resn_exch{"HC42 NDC"}="2HC4 NDC"; $resn_exch{"HC51 NDC"}="1HC5 NDC"; $resn_exch{"HC52 NDC"}="2HC5 NDC";
$resn_exch{"H71N NDE"}="NH71 NDE"; $resn_exch{"H72N NDE"}="NH72 NDE"; $resn_exch{" H5N NDE"}=" NH5 NDE";
$resn_exch{" H6N NDE"}=" NH6 NDE"; $resn_exch{" H2N NDE"}=" NH2 NDE"; $resn_exch{" H1D NDE"}="NH1* NDE";
$resn_exch{" H2D NDE"}="NH2* NDE"; $resn_exch{"HO2N NDE"}="NHO2 NDE"; $resn_exch{" H3D NDE"}="NH3* NDE";
$resn_exch{"HO3N NDE"}="NHO3 NDE"; $resn_exch{" H4D NDE"}="NH4* NDE"; $resn_exch{"H51N NDE"}="NH51 NDE";
$resn_exch{"H52N NDE"}="NH52 NDE"; $resn_exch{"HOA2 NDE"}="2HOA NDE"; $resn_exch{"H51A NDE"}="AH51 NDE";
$resn_exch{"H52A NDE"}="AH52 NDE"; $resn_exch{" H4B NDE"}="AH4* NDE"; $resn_exch{" H1B NDE"}="AH1* NDE";
$resn_exch{" H2B NDE"}="AH2* NDE"; $resn_exch{"HO2A NDE"}="AHO2 NDE"; $resn_exch{" H3B NDE"}="AH3* NDE";
$resn_exch{"HO3A NDE"}="AHO3 NDE"; $resn_exch{" H8A NDE"}=" AH8 NDE"; $resn_exch{"H61A NDE"}="AH61 NDE";
$resn_exch{"H62A NDE"}="AH62 NDE"; $resn_exch{" H2A NDE"}=" AH2 NDE"; $resn_exch{"HOP2 NDE"}="2HOP NDE";
$resn_exch{"HOP3 NDE"}="3HOP NDE"; $resn_exch{" H11 NDE"}="1H1  NDE"; $resn_exch{" H12 NDE"}="2H1  NDE";
$resn_exch{" H31 NDE"}="1H3  NDE"; $resn_exch{"H311 NDE"}="1H31 NDE"; $resn_exch{" HB3 NDF"}="1HB  NDF";
$resn_exch{" HB2 NDF"}="2HB  NDF"; $resn_exch{" H   NDF"}=" HN  NDF"; $resn_exch{"H121 NDL"}="1H12 NDL";
$resn_exch{"H122 NDL"}="2H12 NDL"; $resn_exch{"H111 NDL"}="1H11 NDL"; $resn_exch{"H112 NDL"}="2H11 NDL";
$resn_exch{"H141 NDL"}="1H14 NDL"; $resn_exch{"H142 NDL"}="2H14 NDL"; $resn_exch{"H131 NDL"}="1H13 NDL";
$resn_exch{"H132 NDL"}="2H13 NDL"; $resn_exch{"H133 NDL"}="3H13 NDL"; $resn_exch{"H191 NDL"}="1H19 NDL";
$resn_exch{"H192 NDL"}="2H19 NDL"; $resn_exch{"H201 NDL"}="1H20 NDL"; $resn_exch{"H202 NDL"}="2H20 NDL";
$resn_exch{"H211 NDL"}="1H21 NDL"; $resn_exch{"H212 NDL"}="2H21 NDL"; $resn_exch{"H221 NDL"}="1H22 NDL";
$resn_exch{"H222 NDL"}="2H22 NDL"; $resn_exch{"H231 NDL"}="1H23 NDL"; $resn_exch{"H232 NDL"}="2H23 NDL";
$resn_exch{"H281 NDL"}="1H28 NDL"; $resn_exch{"H282 NDL"}="2H28 NDL"; $resn_exch{"H291 NDL"}="1H29 NDL";
$resn_exch{"H292 NDL"}="2H29 NDL"; $resn_exch{"H301 NDL"}="1H30 NDL"; $resn_exch{"H302 NDL"}="2H30 NDL";
$resn_exch{"H331 NDL"}="1H33 NDL"; $resn_exch{"H332 NDL"}="2H33 NDL"; $resn_exch{"H391 NDL"}="1H39 NDL";
$resn_exch{"H392 NDL"}="2H39 NDL"; $resn_exch{"H401 NDL"}="1H40 NDL"; $resn_exch{"H402 NDL"}="2H40 NDL";
$resn_exch{"H411 NDL"}="1H41 NDL"; $resn_exch{"H412 NDL"}="2H41 NDL"; $resn_exch{"H451 NDL"}="1H45 NDL";
$resn_exch{"H452 NDL"}="2H45 NDL"; $resn_exch{"HN91 NDM"}="1HN9 NDM"; $resn_exch{"HN92 NDM"}="2HN9 NDM";
$resn_exch{"HN93 NDM"}="3HN9 NDM"; $resn_exch{" H22 NDM"}="2H2  NDM"; $resn_exch{"HN21 NDM"}="1HN2 NDM";
$resn_exch{"HN22 NDM"}="2HN2 NDM"; $resn_exch{"HN11 NDM"}="1HN1 NDM"; $resn_exch{" H66 NDM"}="6H6  NDM";
$resn_exch{"HN71 NDM"}="1HN7 NDM"; $resn_exch{"HN72 NDM"}="2HN7 NDM"; $resn_exch{"HN73 NDM"}="3HN7 NDM";
$resn_exch{"HC81 NDM"}="1HC8 NDM"; $resn_exch{"HOA2 NDO"}="2HOA NDO"; $resn_exch{"H51A NDO"}="AH51 NDO";
$resn_exch{"H52A NDO"}="AH52 NDO"; $resn_exch{" H4B NDO"}="AH4* NDO"; $resn_exch{" H3B NDO"}="AH3* NDO";
$resn_exch{"HO3A NDO"}="AHO3 NDO"; $resn_exch{" H2B NDO"}="AH2* NDO"; $resn_exch{" H1B NDO"}="AH1* NDO";
$resn_exch{" H8A NDO"}=" AH8 NDO"; $resn_exch{"HO6A NDO"}="AHO6 NDO"; $resn_exch{" H2A NDO"}=" AH2 NDO";
$resn_exch{"H51N NDO"}="NH51 NDO"; $resn_exch{"H52N NDO"}="NH52 NDO"; $resn_exch{" H4D NDO"}="NH4* NDO";
$resn_exch{" H3D NDO"}="NH3* NDO"; $resn_exch{"HO3N NDO"}="NHO3 NDO"; $resn_exch{" H2D NDO"}="NH2* NDO";
$resn_exch{"HO2N NDO"}="NHO2 NDO"; $resn_exch{" H1D NDO"}="NH1* NDO"; $resn_exch{" H2N NDO"}=" NH2 NDO";
$resn_exch{"H71N NDO"}="NH71 NDO"; $resn_exch{"H72N NDO"}="NH72 NDO"; $resn_exch{" H4N NDO"}=" NH4 NDO";
$resn_exch{" H5N NDO"}=" NH5 NDO"; $resn_exch{" H6N NDO"}=" NH6 NDO"; $resn_exch{"HOP2 NDO"}="2HOP NDO";
$resn_exch{"HOP3 NDO"}="3HOP NDO"; $resn_exch{"HOA2 NDP"}="2HOA NDP"; $resn_exch{"H51A NDP"}="AH51 NDP";
$resn_exch{"H52A NDP"}="AH52 NDP"; $resn_exch{" H4B NDP"}="AH4* NDP"; $resn_exch{" H3B NDP"}="AH3* NDP";
$resn_exch{"HO3A NDP"}="AHO3 NDP"; $resn_exch{" H2B NDP"}="AH2* NDP"; $resn_exch{" H1B NDP"}="AH1* NDP";
$resn_exch{" H8A NDP"}=" AH8 NDP"; $resn_exch{"H61A NDP"}="AH61 NDP"; $resn_exch{"H62A NDP"}="AH62 NDP";
$resn_exch{" H2A NDP"}=" AH2 NDP"; $resn_exch{"H21N NDP"}="NH21 NDP"; $resn_exch{"H51N NDP"}="NH51 NDP";
$resn_exch{"H52N NDP"}="NH52 NDP"; $resn_exch{" H4D NDP"}="NH4* NDP"; $resn_exch{" H3D NDP"}="NH3* NDP";
$resn_exch{"HO3N NDP"}="NHO3 NDP"; $resn_exch{" H2D NDP"}="NH2* NDP"; $resn_exch{"HO2N NDP"}="NHO2 NDP";
$resn_exch{" H1D NDP"}="NH1* NDP"; $resn_exch{" H2N NDP"}=" NH2 NDP"; $resn_exch{"H71N NDP"}="NH71 NDP";
$resn_exch{"H72N NDP"}="NH72 NDP"; $resn_exch{"H41N NDP"}="NH41 NDP"; $resn_exch{"H42N NDP"}="NH42 NDP";
$resn_exch{" H5N NDP"}=" NH5 NDP"; $resn_exch{" H6N NDP"}=" NH6 NDP"; $resn_exch{"HOP2 NDP"}="2HOP NDP";
$resn_exch{"HOP3 NDP"}="3HOP NDP"; $resn_exch{"H201 NDR"}="1H20 NDR"; $resn_exch{"H202 NDR"}="2H20 NDR";
$resn_exch{"H151 NDR"}="1H15 NDR"; $resn_exch{"H152 NDR"}="2H15 NDR"; $resn_exch{"H181 NDR"}="1H18 NDR";
$resn_exch{"H182 NDR"}="2H18 NDR"; $resn_exch{" H91 NDR"}="1H9  NDR"; $resn_exch{" H92 NDR"}="2H9  NDR";
$resn_exch{"H131 NDR"}="1H13 NDR"; $resn_exch{"H132 NDR"}="2H13 NDR"; $resn_exch{" H61 NDR"}="1H6  NDR";
$resn_exch{" H62 NDR"}="2H6  NDR"; $resn_exch{" H71 NDR"}="1H7  NDR"; $resn_exch{" H72 NDR"}="2H7  NDR";
$resn_exch{" H73 NDR"}="3H7  NDR"; $resn_exch{"H101 NDR"}="1H10 NDR"; $resn_exch{"H102 NDR"}="2H10 NDR";
$resn_exch{" H41 NDR"}="1H4  NDR"; $resn_exch{" H42 NDR"}="2H4  NDR"; $resn_exch{" H41 NDS"}="1H4  NDS";
$resn_exch{" H42 NDS"}="2H4  NDS"; $resn_exch{" H31 NDS"}="1H3  NDS"; $resn_exch{" H32 NDS"}="2H3  NDS";
$resn_exch{" H21 NDS"}="1H2  NDS"; $resn_exch{" H22 NDS"}="2H2  NDS"; $resn_exch{" H11 NDS"}="1H1  NDS";
$resn_exch{" H12 NDS"}="2H1  NDS"; $resn_exch{" H13 NDS"}="3H1  NDS"; $resn_exch{" H71 NDS"}="1H7  NDS";
$resn_exch{" H72 NDS"}="2H7  NDS"; $resn_exch{" H73 NDS"}="3H7  NDS"; $resn_exch{" H51 NDS"}="1H5  NDS";
$resn_exch{" H52 NDS"}="2H5  NDS"; $resn_exch{" H61 NDS"}="1H6  NDS"; $resn_exch{" H62 NDS"}="2H6  NDS";
$resn_exch{" H63 NDS"}="3H6  NDS"; $resn_exch{"H211 NDT"}="1H21 NDT"; $resn_exch{"H212 NDT"}="2H21 NDT";
$resn_exch{"H213 NDT"}="3H21 NDT"; $resn_exch{"H51' NDU"}="1H5' NDU"; $resn_exch{"H52' NDU"}="2H5' NDU";
$resn_exch{"H21' NDU"}="1H2' NDU"; $resn_exch{"H22' NDU"}="2H2' NDU"; $resn_exch{"H611 NDU"}="1H61 NDU";
$resn_exch{"H612 NDU"}="2H61 NDU"; $resn_exch{"H151 NE8"}="1H15 NE8"; $resn_exch{"H152 NE8"}="2H15 NE8";
$resn_exch{"H141 NE8"}="1H14 NE8"; $resn_exch{"H142 NE8"}="2H14 NE8"; $resn_exch{"H231 NE8"}="1H23 NE8";
$resn_exch{"H232 NE8"}="2H23 NE8"; $resn_exch{" HN1 NEA"}="1HN  NEA"; $resn_exch{" HN2 NEA"}="2HN  NEA";
$resn_exch{" HB1 NEA"}="1HB  NEA"; $resn_exch{" HB2 NEA"}="2HB  NEA"; $resn_exch{" HG1 NEA"}="1HG  NEA";
$resn_exch{" HG2 NEA"}="2HG  NEA"; $resn_exch{"H5'1 NEA"}="1H5* NEA"; $resn_exch{"H5'2 NEA"}="2H5* NEA";
$resn_exch{" H4' NEA"}=" H4* NEA"; $resn_exch{" H3' NEA"}=" H3* NEA"; $resn_exch{" H2' NEA"}=" H2* NEA";
$resn_exch{"HO'2 NEA"}="2HO* NEA"; $resn_exch{" H1' NEA"}=" H1* NEA"; $resn_exch{" H61 NEA"}="1H6  NEA";
$resn_exch{" H62 NEA"}="2H6  NEA"; $resn_exch{" H11 NEB"}="1H1  NEB"; $resn_exch{" H12 NEB"}="2H1  NEB";
$resn_exch{" H13 NEB"}="3H1  NEB"; $resn_exch{" H21 NEB"}="1H2  NEB"; $resn_exch{" H22 NEB"}="2H2  NEB";
$resn_exch{" H31 NEB"}="1H3  NEB"; $resn_exch{" H32 NEB"}="2H3  NEB"; $resn_exch{"H511 NEC"}="1H51 NEC";
$resn_exch{"H512 NEC"}="2H51 NEC"; $resn_exch{"H521 NEC"}="1H52 NEC"; $resn_exch{"H522 NEC"}="2H52 NEC";
$resn_exch{"H523 NEC"}="3H52 NEC"; $resn_exch{" H4' NEC"}=" H4* NEC"; $resn_exch{" H3' NEC"}=" H3* NEC";
$resn_exch{"HO3' NEC"}="*HO3 NEC"; $resn_exch{" H2' NEC"}=" H2* NEC"; $resn_exch{"HO2' NEC"}="*HO2 NEC";
$resn_exch{" H1' NEC"}=" H1* NEC"; $resn_exch{"HN61 NEC"}="1HN6 NEC"; $resn_exch{"HN62 NEC"}="2HN6 NEC";
$resn_exch{" H21 NED"}="1H2  NED"; $resn_exch{" H22 NED"}="2H2  NED"; $resn_exch{" H23 NED"}="3H2  NED";
$resn_exch{" H61 NED"}="1H6  NED"; $resn_exch{" H62 NED"}="2H6  NED"; $resn_exch{"HN61 NED"}="1HN6 NED";
$resn_exch{"HN62 NED"}="2HN6 NED"; $resn_exch{" H63 NED"}="3H6  NED"; $resn_exch{" HA2 NEH"}="2HA  NEH";
$resn_exch{" HA3 NEH"}="3HA  NEH"; $resn_exch{" HB1 NEH"}="1HB  NEH"; $resn_exch{" HB2 NEH"}="2HB  NEH";
$resn_exch{" HB3 NEH"}="3HB  NEH"; $resn_exch{"H521 NEI"}="1H52 NEI"; $resn_exch{"H522 NEI"}="2H52 NEI";
$resn_exch{"H523 NEI"}="3H52 NEI"; $resn_exch{"H511 NEI"}="1H51 NEI"; $resn_exch{"H512 NEI"}="2H51 NEI";
$resn_exch{"HN5' NEI"}="HN5* NEI"; $resn_exch{" H4' NEI"}=" H4* NEI"; $resn_exch{" H3' NEI"}=" H3* NEI";
$resn_exch{"HO3' NEI"}="HO3* NEI"; $resn_exch{" H2' NEI"}=" H2* NEI"; $resn_exch{"HO2' NEI"}="HO2* NEI";
$resn_exch{" H1' NEI"}=" H1* NEI"; $resn_exch{"HN61 NEI"}="1HN6 NEI"; $resn_exch{"HN62 NEI"}="2HN6 NEI";
$resn_exch{" HB1 NEM"}="1HB  NEM"; $resn_exch{" HB2 NEM"}="2HB  NEM"; $resn_exch{" HM1 NEM"}="1HM  NEM";
$resn_exch{" HM2 NEM"}="2HM  NEM"; $resn_exch{" HM3 NEM"}="3HM  NEM"; $resn_exch{" H11 NEN"}="1H1  NEN";
$resn_exch{" H12 NEN"}="2H1  NEN"; $resn_exch{" H41 NEN"}="1H4  NEN"; $resn_exch{" H42 NEN"}="2H4  NEN";
$resn_exch{" H51 NEN"}="1H5  NEN"; $resn_exch{" H52 NEN"}="2H5  NEN"; $resn_exch{" H61 NEN"}="1H6  NEN";
$resn_exch{" H62 NEN"}="2H6  NEN"; $resn_exch{" H63 NEN"}="3H6  NEN"; $resn_exch{"H131 NEO"}="1H13 NEO";
$resn_exch{"H132 NEO"}="2H13 NEO"; $resn_exch{"H281 NEO"}="1H28 NEO"; $resn_exch{"H282 NEO"}="2H28 NEO";
$resn_exch{" H2  NEP"}=" HN2 NEP"; $resn_exch{" HB2 NEP"}="1HB  NEP"; $resn_exch{" HB3 NEP"}="2HB  NEP";
$resn_exch{"HOP1 NEP"}="1HOP NEP"; $resn_exch{"HOP2 NEP"}="2HOP NEP"; $resn_exch{" H21 NEQ"}="1H2  NEQ";
$resn_exch{" H31 NEQ"}="1H3  NEQ"; $resn_exch{" H51 NEQ"}="1H5  NEQ"; $resn_exch{" H52 NEQ"}="2H5  NEQ";
$resn_exch{" H61 NEQ"}="1H6  NEQ"; $resn_exch{" H62 NEQ"}="2H6  NEQ"; $resn_exch{" H63 NEQ"}="3H6  NEQ";
$resn_exch{" H21 NER"}="1H2  NER"; $resn_exch{" H22 NER"}="2H2  NER"; $resn_exch{" H31 NER"}="1H3  NER";
$resn_exch{" H32 NER"}="2H3  NER"; $resn_exch{" H41 NER"}="1H4  NER"; $resn_exch{" H42 NER"}="2H4  NER";
$resn_exch{" H51 NER"}="1H5  NER"; $resn_exch{" H52 NER"}="2H5  NER"; $resn_exch{" H61 NER"}="1H6  NER";
$resn_exch{" H62 NER"}="2H6  NER"; $resn_exch{" H71 NER"}="1H7  NER"; $resn_exch{" H72 NER"}="2H7  NER";
$resn_exch{" H81 NER"}="1H8  NER"; $resn_exch{" H82 NER"}="2H8  NER"; $resn_exch{" H91 NER"}="1H9  NER";
$resn_exch{" H92 NER"}="2H9  NER"; $resn_exch{"H101 NER"}="1H10 NER"; $resn_exch{"H102 NER"}="2H10 NER";
$resn_exch{"H111 NER"}="1H11 NER"; $resn_exch{"H112 NER"}="2H11 NER"; $resn_exch{"H121 NER"}="1H12 NER";
$resn_exch{"H122 NER"}="2H12 NER"; $resn_exch{"H131 NER"}="1H13 NER"; $resn_exch{"H132 NER"}="2H13 NER";
$resn_exch{"H141 NER"}="1H14 NER"; $resn_exch{"H142 NER"}="2H14 NER"; $resn_exch{"H171 NER"}="1H17 NER";
$resn_exch{"H172 NER"}="2H17 NER"; $resn_exch{"H181 NER"}="1H18 NER"; $resn_exch{"H182 NER"}="2H18 NER";
$resn_exch{"H191 NER"}="1H19 NER"; $resn_exch{"H192 NER"}="2H19 NER"; $resn_exch{"H201 NER"}="1H20 NER";
$resn_exch{"H202 NER"}="2H20 NER"; $resn_exch{"H211 NER"}="1H21 NER"; $resn_exch{"H212 NER"}="2H21 NER";
$resn_exch{"H221 NER"}="1H22 NER"; $resn_exch{"H222 NER"}="2H22 NER"; $resn_exch{"H231 NER"}="1H23 NER";
$resn_exch{"H232 NER"}="2H23 NER"; $resn_exch{"H241 NER"}="1H24 NER"; $resn_exch{"H242 NER"}="2H24 NER";
$resn_exch{"H243 NER"}="3H24 NER"; $resn_exch{"HC71 NES"}="1HC7 NES"; $resn_exch{"HC72 NES"}="2HC7 NES";
$resn_exch{"HC81 NES"}="1HC8 NES"; $resn_exch{"HC82 NES"}="2HC8 NES"; $resn_exch{"HC21 NES"}="1HC2 NES";
$resn_exch{"HC22 NES"}="2HC2 NES"; $resn_exch{"HC41 NES"}="1HC4 NES"; $resn_exch{"HC42 NES"}="2HC4 NES";
$resn_exch{"HC61 NES"}="1HC6 NES"; $resn_exch{"HC62 NES"}="2HC6 NES"; $resn_exch{" H11 NET"}="1H1  NET";
$resn_exch{" H12 NET"}="2H1  NET"; $resn_exch{" H21 NET"}="1H2  NET"; $resn_exch{" H22 NET"}="2H2  NET";
$resn_exch{" H23 NET"}="3H2  NET"; $resn_exch{" H31 NET"}="1H3  NET"; $resn_exch{" H32 NET"}="2H3  NET";
$resn_exch{" H41 NET"}="1H4  NET"; $resn_exch{" H42 NET"}="2H4  NET"; $resn_exch{" H43 NET"}="3H4  NET";
$resn_exch{" H51 NET"}="1H5  NET"; $resn_exch{" H52 NET"}="2H5  NET"; $resn_exch{" H61 NET"}="1H6  NET";
$resn_exch{" H62 NET"}="2H6  NET"; $resn_exch{" H63 NET"}="3H6  NET"; $resn_exch{" H71 NET"}="1H7  NET";
$resn_exch{" H72 NET"}="2H7  NET"; $resn_exch{" H81 NET"}="1H8  NET"; $resn_exch{" H82 NET"}="2H8  NET";
$resn_exch{" H83 NET"}="3H8  NET"; $resn_exch{"H131 NEU"}="1H13 NEU"; $resn_exch{"H132 NEU"}="2H13 NEU";
$resn_exch{"H281 NEU"}="1H28 NEU"; $resn_exch{"H282 NEU"}="2H28 NEU"; $resn_exch{"H121 NEV"}="1H12 NEV";
$resn_exch{"H122 NEV"}="2H12 NEV"; $resn_exch{"H123 NEV"}="3H12 NEV"; $resn_exch{"H151 NEV"}="1H15 NEV";
$resn_exch{"H152 NEV"}="2H15 NEV"; $resn_exch{"H161 NEV"}="1H16 NEV"; $resn_exch{"H162 NEV"}="2H16 NEV";
$resn_exch{" H21 NEX"}="1H2  NEX"; $resn_exch{" H22 NEX"}="2H2  NEX"; $resn_exch{" H41 NEX"}="1H4  NEX";
$resn_exch{" H42 NEX"}="2H4  NEX"; $resn_exch{"H161 NEX"}="1H16 NEX"; $resn_exch{"H162 NEX"}="2H16 NEX";
$resn_exch{"H163 NEX"}="3H16 NEX"; $resn_exch{"H171 NEX"}="1H17 NEX"; $resn_exch{"H172 NEX"}="2H17 NEX";
$resn_exch{"H173 NEX"}="3H17 NEX"; $resn_exch{"H181 NEX"}="1H18 NEX"; $resn_exch{"H182 NEX"}="2H18 NEX";
$resn_exch{"H183 NEX"}="3H18 NEX"; $resn_exch{"H191 NEX"}="1H19 NEX"; $resn_exch{"H192 NEX"}="2H19 NEX";
$resn_exch{"H193 NEX"}="3H19 NEX"; $resn_exch{"H201 NEX"}="1H20 NEX"; $resn_exch{"H202 NEX"}="2H20 NEX";
$resn_exch{"H203 NEX"}="3H20 NEX"; $resn_exch{"H221 NEX"}="1H22 NEX"; $resn_exch{"H222 NEX"}="2H22 NEX";
$resn_exch{"H241 NEX"}="1H24 NEX"; $resn_exch{"H242 NEX"}="2H24 NEX"; $resn_exch{"H361 NEX"}="1H36 NEX";
$resn_exch{"H362 NEX"}="2H36 NEX"; $resn_exch{"H363 NEX"}="3H36 NEX"; $resn_exch{"H371 NEX"}="1H37 NEX";
$resn_exch{"H372 NEX"}="2H37 NEX"; $resn_exch{"H373 NEX"}="3H37 NEX"; $resn_exch{"H381 NEX"}="1H38 NEX";
$resn_exch{"H382 NEX"}="2H38 NEX"; $resn_exch{"H383 NEX"}="3H38 NEX"; $resn_exch{"H391 NEX"}="1H39 NEX";
$resn_exch{"H392 NEX"}="2H39 NEX"; $resn_exch{"H393 NEX"}="3H39 NEX"; $resn_exch{"H401 NEX"}="1H40 NEX";
$resn_exch{"H402 NEX"}="2H40 NEX"; $resn_exch{"H403 NEX"}="3H40 NEX"; $resn_exch{"H141 NEZ"}="1H14 NEZ";
$resn_exch{"H142 NEZ"}="2H14 NEZ"; $resn_exch{"H231 NEZ"}="1H23 NEZ"; $resn_exch{"H232 NEZ"}="2H23 NEZ";
$resn_exch{"H151 NEZ"}="1H15 NEZ"; $resn_exch{"H152 NEZ"}="2H15 NEZ"; $resn_exch{"H241 NEZ"}="1H24 NEZ";
$resn_exch{"H242 NEZ"}="2H24 NEZ"; $resn_exch{"H243 NEZ"}="3H24 NEZ"; $resn_exch{"H112  NF"}="2H11  NF";
$resn_exch{"H122  NF"}="2H12  NF"; $resn_exch{" H91  NF"}="1H9   NF"; $resn_exch{" H81  NF"}="1H8   NF";
$resn_exch{" H71  NF"}="1H7   NF"; $resn_exch{" H61  NF"}="1H6   NF"; $resn_exch{" H51  NF"}="1H5   NF";
$resn_exch{" H41  NF"}="1H4   NF"; $resn_exch{"H212  NF"}="2H21  NF"; $resn_exch{"H222  NF"}="2H22  NF";
$resn_exch{"H312  NF"}="2H31  NF"; $resn_exch{"H322  NF"}="2H32  NF"; $resn_exch{"H412  NF"}="2H41  NF";
$resn_exch{"H422  NF"}="2H42  NF"; $resn_exch{"H512  NF"}="2H51  NF"; $resn_exch{"H522  NF"}="2H52  NF";
$resn_exch{"H612  NF"}="2H61  NF"; $resn_exch{"H622  NF"}="2H62  NF"; $resn_exch{"H712  NF"}="2H71  NF";
$resn_exch{"H722  NF"}="2H72  NF"; $resn_exch{"H812  NF"}="2H81  NF"; $resn_exch{"H822  NF"}="2H82  NF";
$resn_exch{"H912  NF"}="2H91  NF"; $resn_exch{"H922  NF"}="2H92  NF"; $resn_exch{" H93  NF"}="3H9   NF";
$resn_exch{" H83  NF"}="3H8   NF"; $resn_exch{" H73  NF"}="3H7   NF"; $resn_exch{" H63  NF"}="3H6   NF";
$resn_exch{" H53  NF"}="3H5   NF"; $resn_exch{" H43  NF"}="3H4   NF"; $resn_exch{" HM1 NF2"}="1HM  NF2";
$resn_exch{" HM2 NF2"}="2HM  NF2"; $resn_exch{" HM3 NF2"}="3HM  NF2"; $resn_exch{" H1' NF2"}=" H1* NF2";
$resn_exch{" H2' NF2"}=" H2* NF2"; $resn_exch{"HO2' NF2"}="2HO* NF2"; $resn_exch{" H3' NF2"}=" H3* NF2";
$resn_exch{"HO3' NF2"}="H3T  NF2"; $resn_exch{" H4' NF2"}=" H4* NF2"; $resn_exch{" H5' NF2"}="1H5* NF2";
$resn_exch{"H5'' NF2"}="2H5* NF2"; $resn_exch{"HOP2 NF2"}="2HOP NF2"; $resn_exch{"HOP3 NF2"}="3HOP NF2";
$resn_exch{" H2  NFA"}=" HN2 NFA"; $resn_exch{" HB2 NFA"}="1HB  NFA"; $resn_exch{" HB3 NFA"}="2HB  NFA";
$resn_exch{"HXT1 NFA"}="1HXT NFA"; $resn_exch{"HXT2 NFA"}="2HXT NFA"; $resn_exch{"H3O1 NFG"}="1H3O NFG";
$resn_exch{"H4O1 NFG"}="1H4O NFG"; $resn_exch{" H62 NFG"}="2H6  NFG"; $resn_exch{" H61 NFG"}="1H6  NFG";
$resn_exch{"H191 NFN"}="1H19 NFN"; $resn_exch{"H192 NFN"}="2H19 NFN"; $resn_exch{"H193 NFN"}="3H19 NFN";
$resn_exch{"H201 NFN"}="1H20 NFN"; $resn_exch{"H202 NFN"}="2H20 NFN"; $resn_exch{"H203 NFN"}="3H20 NFN";
$resn_exch{"H171 NFN"}="1H17 NFN"; $resn_exch{"H172 NFN"}="2H17 NFN"; $resn_exch{"H181 NFN"}="1H18 NFN";
$resn_exch{"H182 NFN"}="2H18 NFN"; $resn_exch{"H183 NFN"}="3H18 NFN"; $resn_exch{" H11 NFP"}="1H1  NFP";
$resn_exch{" H12 NFP"}="2H1  NFP"; $resn_exch{" H13 NFP"}="3H1  NFP"; $resn_exch{" H21 NFP"}="1H2  NFP";
$resn_exch{" H22 NFP"}="2H2  NFP"; $resn_exch{" HN1 NFP"}="1HN  NFP"; $resn_exch{" HN2 NFP"}="2HN  NFP";
$resn_exch{" H41 NFP"}="1H4  NFP"; $resn_exch{" H42 NFP"}="2H4  NFP"; $resn_exch{" H51 NFP"}="1H5  NFP";
$resn_exch{" H52 NFP"}="2H5  NFP"; $resn_exch{"H411 NFT"}="1H41 NFT"; $resn_exch{"H412 NFT"}="2H41 NFT";
$resn_exch{"H181 NFT"}="1H18 NFT"; $resn_exch{"H182 NFT"}="2H18 NFT"; $resn_exch{"H201 NFT"}="1H20 NFT";
$resn_exch{"H202 NFT"}="2H20 NFT"; $resn_exch{"H203 NFT"}="3H20 NFT"; $resn_exch{"H211 NFT"}="1H21 NFT";
$resn_exch{"H212 NFT"}="2H21 NFT"; $resn_exch{"H213 NFT"}="3H21 NFT"; $resn_exch{"H631 NFT"}="1H63 NFT";
$resn_exch{"H632 NFT"}="2H63 NFT"; $resn_exch{"H441 NFT"}="1H44 NFT"; $resn_exch{"H442 NFT"}="2H44 NFT";
$resn_exch{"H451 NFT"}="1H45 NFT"; $resn_exch{"H452 NFT"}="2H45 NFT"; $resn_exch{"HN41 NFZ"}="1HN4 NFZ";
$resn_exch{"HN42 NFZ"}="2HN4 NFZ"; $resn_exch{" H61 NG1"}="1H6  NG1"; $resn_exch{" H62 NG1"}="2H6  NG1";
$resn_exch{" H81 NG1"}="1H8  NG1"; $resn_exch{" H82 NG1"}="2H8  NG1"; $resn_exch{" H83 NG1"}="3H8  NG1";
$resn_exch{" H61 NG6"}="1H6  NG6"; $resn_exch{" H62 NG6"}="2H6  NG6"; $resn_exch{" H81 NG6"}="1H8  NG6";
$resn_exch{" H82 NG6"}="2H8  NG6"; $resn_exch{" H83 NG6"}="3H8  NG6"; $resn_exch{"HOS3 NG6"}="3HOS NG6";
$resn_exch{" H61 NGA"}="1H6  NGA"; $resn_exch{" H62 NGA"}="2H6  NGA"; $resn_exch{" H81 NGA"}="1H8  NGA";
$resn_exch{" H82 NGA"}="2H8  NGA"; $resn_exch{" H83 NGA"}="3H8  NGA"; $resn_exch{"H2A1 NGD"}="1H2A NGD";
$resn_exch{"H2A2 NGD"}="2H2A NGD"; $resn_exch{" H1' NGD"}=" H1* NGD"; $resn_exch{" H2' NGD"}=" H2* NGD";
$resn_exch{" H3' NGD"}=" H3* NGD"; $resn_exch{" H4' NGD"}=" H4* NGD"; $resn_exch{" H51 NGD"}="1H5  NGD";
$resn_exch{" H52 NGD"}="2H5  NGD"; $resn_exch{"H5'1 NGD"}="1H5* NGD"; $resn_exch{"H5'2 NGD"}="2H5* NGD";
$resn_exch{"HC4' NGD"}="HC4* NGD"; $resn_exch{"HC3' NGD"}="HC3* NGD"; $resn_exch{"HO3' NGD"}="HO3* NGD";
$resn_exch{"HC2' NGD"}="HC2* NGD"; $resn_exch{"HO2' NGD"}="HO2* NGD"; $resn_exch{"HC1' NGD"}="HC1* NGD";
$resn_exch{"HN71 NGD"}="1HN7 NGD"; $resn_exch{"HN72 NGD"}="2HN7 NGD"; $resn_exch{" H71 NGH"}="1H7  NGH";
$resn_exch{" H72 NGH"}="2H7  NGH"; $resn_exch{" H73 NGH"}="3H7  NGH"; $resn_exch{" H91 NGH"}="1H9  NGH";
$resn_exch{" H92 NGH"}="2H9  NGH"; $resn_exch{"H101 NGH"}="1H10 NGH"; $resn_exch{"H102 NGH"}="2H10 NGH";
$resn_exch{"H131 NGH"}="1H13 NGH"; $resn_exch{"H132 NGH"}="2H13 NGH"; $resn_exch{"H133 NGH"}="3H13 NGH";
$resn_exch{"H141 NGH"}="1H14 NGH"; $resn_exch{"H142 NGH"}="2H14 NGH"; $resn_exch{"H143 NGH"}="3H14 NGH";
$resn_exch{"H6C1 NGK"}="1H6C NGK"; $resn_exch{"H6C2 NGK"}="2H6C NGK"; $resn_exch{"H8C1 NGK"}="1H8C NGK";
$resn_exch{"H8C2 NGK"}="2H8C NGK"; $resn_exch{"H8C3 NGK"}="3H8C NGK"; $resn_exch{" H61 NGL"}="1H6  NGL";
$resn_exch{" H62 NGL"}="2H6  NGL"; $resn_exch{" H81 NGL"}="1H8  NGL"; $resn_exch{" H82 NGL"}="2H8  NGL";
$resn_exch{" H83 NGL"}="3H8  NGL"; $resn_exch{"HOS3 NGL"}="3HOS NGL"; $resn_exch{" H81 NGM"}="1H8  NGM";
$resn_exch{" H82 NGM"}="2H8  NGM"; $resn_exch{"H131 NGM"}="1H13 NGM"; $resn_exch{"H132 NGM"}="2H13 NGM";
$resn_exch{"H133 NGM"}="3H13 NGM"; $resn_exch{"H151 NGM"}="1H15 NGM"; $resn_exch{"H152 NGM"}="2H15 NGM";
$resn_exch{"H153 NGM"}="3H15 NGM"; $resn_exch{"H221 NGM"}="1H22 NGM"; $resn_exch{"H222 NGM"}="2H22 NGM";
$resn_exch{"H223 NGM"}="3H22 NGM"; $resn_exch{"H231 NGM"}="1H23 NGM"; $resn_exch{"H232 NGM"}="2H23 NGM";
$resn_exch{"H233 NGM"}="3H23 NGM"; $resn_exch{"H241 NGM"}="1H24 NGM"; $resn_exch{"H242 NGM"}="2H24 NGM";
$resn_exch{"H243 NGM"}="3H24 NGM"; $resn_exch{"H251 NGM"}="1H25 NGM"; $resn_exch{"H252 NGM"}="2H25 NGM";
$resn_exch{"H253 NGM"}="3H25 NGM"; $resn_exch{"H261 NGM"}="1H26 NGM"; $resn_exch{"H262 NGM"}="2H26 NGM";
$resn_exch{"H263 NGM"}="3H26 NGM"; $resn_exch{"H271 NGM"}="1H27 NGM"; $resn_exch{"H272 NGM"}="2H27 NGM";
$resn_exch{"H273 NGM"}="3H27 NGM"; $resn_exch{"H281 NGM"}="1H28 NGM"; $resn_exch{"H282 NGM"}="2H28 NGM";
$resn_exch{"H283 NGM"}="3H28 NGM"; $resn_exch{"H291 NGM"}="1H29 NGM"; $resn_exch{"H292 NGM"}="2H29 NGM";
$resn_exch{"H293 NGM"}="3H29 NGM"; $resn_exch{" H1' NGM"}=" H1* NGM"; $resn_exch{" H2' NGM"}=" H2* NGM";
$resn_exch{" H4' NGM"}=" H4* NGM"; $resn_exch{" H5' NGM"}=" H5* NGM"; $resn_exch{"HC61 NGO"}="1HC6 NGO";
$resn_exch{"HC62 NGO"}="2HC6 NGO"; $resn_exch{"HC81 NGO"}="1HC8 NGO"; $resn_exch{"HC82 NGO"}="2HC8 NGO";
$resn_exch{"HC83 NGO"}="3HC8 NGO"; $resn_exch{" H81 NGP"}="1H8  NGP"; $resn_exch{" H82 NGP"}="2H8  NGP";
$resn_exch{"H131 NGP"}="1H13 NGP"; $resn_exch{"H132 NGP"}="2H13 NGP"; $resn_exch{"H133 NGP"}="3H13 NGP";
$resn_exch{"H151 NGP"}="1H15 NGP"; $resn_exch{"H152 NGP"}="2H15 NGP"; $resn_exch{"H153 NGP"}="3H15 NGP";
$resn_exch{"H119 NGP"}="19H1 NGP"; $resn_exch{"H221 NGP"}="21H2 NGP"; $resn_exch{"H22A NGP"}="1H22 NGP";
$resn_exch{"H222 NGP"}="2H22 NGP"; $resn_exch{"H223 NGP"}="3H22 NGP"; $resn_exch{"H231 NGP"}="1H23 NGP";
$resn_exch{"H232 NGP"}="2H23 NGP"; $resn_exch{"H233 NGP"}="3H23 NGP"; $resn_exch{"H241 NGP"}="1H24 NGP";
$resn_exch{"H242 NGP"}="2H24 NGP"; $resn_exch{"H243 NGP"}="3H24 NGP"; $resn_exch{"H251 NGP"}="1H25 NGP";
$resn_exch{"H252 NGP"}="2H25 NGP"; $resn_exch{"H253 NGP"}="3H25 NGP"; $resn_exch{"H261 NGP"}="1H26 NGP";
$resn_exch{"H262 NGP"}="2H26 NGP"; $resn_exch{"H263 NGP"}="3H26 NGP"; $resn_exch{"H271 NGP"}="1H27 NGP";
$resn_exch{"H272 NGP"}="2H27 NGP"; $resn_exch{"H273 NGP"}="3H27 NGP"; $resn_exch{"H281 NGP"}="1H28 NGP";
$resn_exch{"H282 NGP"}="2H28 NGP"; $resn_exch{"H283 NGP"}="3H28 NGP"; $resn_exch{"H291 NGP"}="1H29 NGP";
$resn_exch{"H292 NGP"}="2H29 NGP"; $resn_exch{"H293 NGP"}="3H29 NGP"; $resn_exch{" H1' NGP"}=" H1* NGP";
$resn_exch{" H2' NGP"}=" H2* NGP"; $resn_exch{" H4' NGP"}=" H4* NGP"; $resn_exch{" H5' NGP"}=" H5* NGP";
$resn_exch{" H61 NGS"}="1H6  NGS"; $resn_exch{" H62 NGS"}="2H6  NGS"; $resn_exch{"HH31 NGS"}="1HH3 NGS";
$resn_exch{"HH32 NGS"}="2HH3 NGS"; $resn_exch{"HH33 NGS"}="3HH3 NGS"; $resn_exch{"HC61 NGT"}="1HC6 NGT";
$resn_exch{"HC62 NGT"}="2HC6 NGT"; $resn_exch{"HC81 NGT"}="1HC8 NGT"; $resn_exch{"HC82 NGT"}="2HC8 NGT";
$resn_exch{"HC83 NGT"}="3HC8 NGT"; $resn_exch{" H81 NGU"}="1H8  NGU"; $resn_exch{" H82 NGU"}="2H8  NGU";
$resn_exch{"H131 NGU"}="1H13 NGU"; $resn_exch{"H132 NGU"}="2H13 NGU"; $resn_exch{"H133 NGU"}="3H13 NGU";
$resn_exch{"H151 NGU"}="1H15 NGU"; $resn_exch{"H152 NGU"}="2H15 NGU"; $resn_exch{"H153 NGU"}="3H15 NGU";
$resn_exch{" H1' NGU"}=" H1* NGU"; $resn_exch{" H2' NGU"}=" H2* NGU"; $resn_exch{" H4' NGU"}=" H4* NGU";
$resn_exch{" H5' NGU"}=" H5* NGU"; $resn_exch{"H251 NGU"}="1H25 NGU"; $resn_exch{"H252 NGU"}="2H25 NGU";
$resn_exch{"H253 NGU"}="3H25 NGU"; $resn_exch{"H261 NGU"}="1H26 NGU"; $resn_exch{"H262 NGU"}="2H26 NGU";
$resn_exch{"H263 NGU"}="3H26 NGU"; $resn_exch{"H271 NGU"}="1H27 NGU"; $resn_exch{"H272 NGU"}="2H27 NGU";
$resn_exch{"H273 NGU"}="3H27 NGU"; $resn_exch{"H281 NGU"}="1H28 NGU"; $resn_exch{"H282 NGU"}="2H28 NGU";
$resn_exch{"H283 NGU"}="3H28 NGU"; $resn_exch{"H291 NGU"}="1H29 NGU"; $resn_exch{"H292 NGU"}="2H29 NGU";
$resn_exch{"H293 NGU"}="3H29 NGU"; $resn_exch{"H221 NGU"}="1H22 NGU"; $resn_exch{"H222 NGU"}="2H22 NGU";
$resn_exch{"H223 NGU"}="3H22 NGU"; $resn_exch{"H231 NGU"}="1H23 NGU"; $resn_exch{"H232 NGU"}="2H23 NGU";
$resn_exch{"H241 NGU"}="1H24 NGU"; $resn_exch{"H242 NGU"}="2H24 NGU"; $resn_exch{"H243 NGU"}="3H24 NGU";
$resn_exch{"H151 NGV"}="1H15 NGV"; $resn_exch{"H152 NGV"}="2H15 NGV"; $resn_exch{"H153 NGV"}="3H15 NGV";
$resn_exch{"H131 NGV"}="1H13 NGV"; $resn_exch{"H132 NGV"}="2H13 NGV"; $resn_exch{"H133 NGV"}="3H13 NGV";
$resn_exch{" H81 NGV"}="1H8  NGV"; $resn_exch{" H82 NGV"}="2H8  NGV"; $resn_exch{" H61 NGZ"}="1H6  NGZ";
$resn_exch{" H62 NGZ"}="2H6  NGZ"; $resn_exch{" H81 NGZ"}="1H8  NGZ"; $resn_exch{" H82 NGZ"}="2H8  NGZ";
$resn_exch{" H83 NGZ"}="3H8  NGZ"; $resn_exch{" HN1 NH2"}="1HN  NH2"; $resn_exch{" HN2 NH2"}="2HN  NH2";
$resn_exch{" HN1 NH3"}="1HN  NH3"; $resn_exch{" HN2 NH3"}="2HN  NH3"; $resn_exch{" HN3 NH3"}="3HN  NH3";
$resn_exch{" HN1 NH4"}="1HN  NH4"; $resn_exch{" HN2 NH4"}="2HN  NH4"; $resn_exch{" HN3 NH4"}="3HN  NH4";
$resn_exch{" HN4 NH4"}="4HN  NH4"; $resn_exch{"H161 NHB"}="1H16 NHB"; $resn_exch{"H162 NHB"}="2H16 NHB";
$resn_exch{"H163 NHB"}="3H16 NHB"; $resn_exch{"HOA2 NHD"}="2HOA NHD"; $resn_exch{"H51A NHD"}="AH51 NHD";
$resn_exch{"H52A NHD"}="AH52 NHD"; $resn_exch{" H4B NHD"}="AH4* NHD"; $resn_exch{" H3B NHD"}="AH3* NHD";
$resn_exch{"HO3A NHD"}="AHO3 NHD"; $resn_exch{" H2B NHD"}="AH2* NHD"; $resn_exch{"HO2A NHD"}="AHO2 NHD";
$resn_exch{" H1B NHD"}="AH1* NHD"; $resn_exch{" H8A NHD"}=" AH8 NHD"; $resn_exch{" H6A NHD"}=" AH6 NHD";
$resn_exch{" H2A NHD"}=" AH2 NHD"; $resn_exch{"H51N NHD"}="NH51 NHD"; $resn_exch{"H52N NHD"}="NH52 NHD";
$resn_exch{" H4D NHD"}="NH4* NHD"; $resn_exch{" H3D NHD"}="NH3* NHD"; $resn_exch{"HO3N NHD"}="NHO3 NHD";
$resn_exch{" H2D NHD"}="NH2* NHD"; $resn_exch{"HO2N NHD"}="NHO2 NHD"; $resn_exch{" H1D NHD"}="NH1* NHD";
$resn_exch{" H2N NHD"}=" NH2 NHD"; $resn_exch{"H71N NHD"}="NH71 NHD"; $resn_exch{"H72N NHD"}="NH72 NHD";
$resn_exch{" H4N NHD"}=" NH4 NHD"; $resn_exch{" H5N NHD"}=" NH5 NHD"; $resn_exch{" H6N NHD"}=" NH6 NHD";
$resn_exch{"H3'1 NHE"}="1H3' NHE"; $resn_exch{"H3'2 NHE"}="2H3' NHE"; $resn_exch{"H2'1 NHE"}="1H2' NHE";
$resn_exch{"H2'2 NHE"}="2H2' NHE"; $resn_exch{"HC'1 NHE"}="1HC' NHE"; $resn_exch{"H6'1 NHE"}="1H6' NHE";
$resn_exch{"H6'2 NHE"}="2H6' NHE"; $resn_exch{"HC11 NHE"}="1HC1 NHE"; $resn_exch{"HC12 NHE"}="2HC1 NHE";
$resn_exch{"HC21 NHE"}="1HC2 NHE"; $resn_exch{"HC22 NHE"}="2HC2 NHE"; $resn_exch{" HO3 NHE"}="3HO  NHE";
$resn_exch{"H5'1 NHE"}="1H5' NHE"; $resn_exch{"H5'2 NHE"}="2H5' NHE"; $resn_exch{"H4'1 NHE"}="1H4' NHE";
$resn_exch{"H4'2 NHE"}="2H4' NHE"; $resn_exch{" HC2 NHL"}=" H2  NHL"; $resn_exch{"H111 NHL"}="1H11 NHL";
$resn_exch{"H112 NHL"}="2H11 NHL"; $resn_exch{" HG  NHL"}="1HG  NHL"; $resn_exch{" HB3 NHL"}="1HB  NHL";
$resn_exch{" HB2 NHL"}="2HB  NHL"; $resn_exch{" H2  NHL"}=" NH2 NHL"; $resn_exch{"HN61 NHM"}="1HN6 NHM";
$resn_exch{"HN62 NHM"}="2HN6 NHM"; $resn_exch{"HEM1 NHM"}="1HEM NHM"; $resn_exch{"HEM2 NHM"}="2HEM NHM";
$resn_exch{"HEM3 NHM"}="3HEM NHM"; $resn_exch{"HAM1 NHM"}="1HAM NHM"; $resn_exch{"HAM2 NHM"}="2HAM NHM";
$resn_exch{"HBM1 NHM"}="1HBM NHM"; $resn_exch{"HBM2 NHM"}="2HBM NHM"; $resn_exch{"HCM1 NHM"}="1HCM NHM";
$resn_exch{"HCM2 NHM"}="2HCM NHM"; $resn_exch{"HDM1 NHM"}="1HDM NHM"; $resn_exch{"HDM2 NHM"}="2HDM NHM";
$resn_exch{"H9M1 NHM"}="1H9M NHM"; $resn_exch{"H9M2 NHM"}="2H9M NHM"; $resn_exch{"H8M1 NHM"}="1H8M NHM";
$resn_exch{"H8M2 NHM"}="2H8M NHM"; $resn_exch{"H7M1 NHM"}="1H7M NHM"; $resn_exch{"H7M2 NHM"}="2H7M NHM";
$resn_exch{"H6M1 NHM"}="1H6M NHM"; $resn_exch{"H6M2 NHM"}="2H6M NHM"; $resn_exch{"H2M1 NHM"}="1H2M NHM";
$resn_exch{"H2M2 NHM"}="2H2M NHM"; $resn_exch{"H3M1 NHM"}="1H3M NHM"; $resn_exch{"H3M2 NHM"}="2H3M NHM";
$resn_exch{"H4M1 NHM"}="1H4M NHM"; $resn_exch{"H4M2 NHM"}="2H4M NHM"; $resn_exch{"H5M1 NHM"}="1H5M NHM";
$resn_exch{"H5M2 NHM"}="2H5M NHM"; $resn_exch{" HP1 NHM"}="1HP  NHM"; $resn_exch{" HP2 NHM"}="2HP  NHM";
$resn_exch{"HO7A NHM"}="AHO7 NHM"; $resn_exch{"HO8A NHM"}="AHO8 NHM"; $resn_exch{" H61 NHM"}="1H6  NHM";
$resn_exch{" H62 NHM"}="2H6  NHM"; $resn_exch{" H71 NHM"}="1H7  NHM"; $resn_exch{" H72 NHM"}="2H7  NHM";
$resn_exch{"H131 NHM"}="1H13 NHM"; $resn_exch{"H132 NHM"}="2H13 NHM"; $resn_exch{"H133 NHM"}="3H13 NHM";
$resn_exch{"H141 NHM"}="1H14 NHM"; $resn_exch{"H142 NHM"}="2H14 NHM"; $resn_exch{"H143 NHM"}="3H14 NHM";
$resn_exch{"H121 NHM"}="1H12 NHM"; $resn_exch{"H122 NHM"}="2H12 NHM"; $resn_exch{"HO1A NHM"}="AHO1 NHM";
$resn_exch{"HO2X NHM"}="XHO2 NHM"; $resn_exch{"H5X1 NHM"}="1H5X NHM"; $resn_exch{"H5X2 NHM"}="2H5X NHM";
$resn_exch{"HO5A NHM"}="AHO5 NHM"; $resn_exch{" H31 NHM"}="1H3  NHM"; $resn_exch{" H32 NHM"}="2H3  NHM";
$resn_exch{" H21 NHM"}="1H2  NHM"; $resn_exch{" H22 NHM"}="2H2  NHM"; $resn_exch{" H11 NHN"}="1H1  NHN";
$resn_exch{" H12 NHN"}="2H1  NHN"; $resn_exch{" H21 NHN"}="1H2  NHN"; $resn_exch{" H22 NHN"}="2H2  NHN";
$resn_exch{"H121 NHN"}="1H12 NHN"; $resn_exch{"H122 NHN"}="2H12 NHN"; $resn_exch{"H131 NHN"}="1H13 NHN";
$resn_exch{"H132 NHN"}="2H13 NHN"; $resn_exch{"H181 NHN"}="1H18 NHN"; $resn_exch{"H182 NHN"}="2H18 NHN";
$resn_exch{"H191 NHN"}="1H19 NHN"; $resn_exch{"H192 NHN"}="2H19 NHN"; $resn_exch{"H201 NHN"}="1H20 NHN";
$resn_exch{"H202 NHN"}="2H20 NHN"; $resn_exch{"H211 NHN"}="1H21 NHN"; $resn_exch{"H212 NHN"}="2H21 NHN";
$resn_exch{"H221 NHN"}="1H22 NHN"; $resn_exch{"H222 NHN"}="2H22 NHN"; $resn_exch{"H261 NHN"}="1H26 NHN";
$resn_exch{"H262 NHN"}="2H26 NHN"; $resn_exch{"H281 NHN"}="1H28 NHN"; $resn_exch{"H282 NHN"}="2H28 NHN";
$resn_exch{"H341 NHN"}="1H34 NHN"; $resn_exch{"H342 NHN"}="2H34 NHN"; $resn_exch{"H351 NHN"}="1H35 NHN";
$resn_exch{"H352 NHN"}="2H35 NHN"; $resn_exch{"H361 NHN"}="1H36 NHN"; $resn_exch{"H362 NHN"}="2H36 NHN";
$resn_exch{"H363 NHN"}="3H36 NHN"; $resn_exch{"H411 NHN"}="1H41 NHN"; $resn_exch{"H412 NHN"}="2H41 NHN";
$resn_exch{"H561 NHN"}="1H56 NHN"; $resn_exch{"H562 NHN"}="2H56 NHN"; $resn_exch{"H563 NHN"}="3H56 NHN";
$resn_exch{"H571 NHN"}="1H57 NHN"; $resn_exch{"H572 NHN"}="2H57 NHN"; $resn_exch{"H573 NHN"}="3H57 NHN";
$resn_exch{"H5'1 NHO"}="1H5' NHO"; $resn_exch{"H5'2 NHO"}="2H5' NHO"; $resn_exch{"H6A1 NHO"}="1H6A NHO";
$resn_exch{"H6A2 NHO"}="2H6A NHO"; $resn_exch{"H5A1 NHO"}="1H5* NHO"; $resn_exch{"H5A2 NHO"}="2H5* NHO";
$resn_exch{" H4B NHO"}=" H4* NHO"; $resn_exch{" H3B NHO"}=" H3* NHO"; $resn_exch{" H2B NHO"}=" H2* NHO";
$resn_exch{" H1B NHO"}=" H1* NHO"; $resn_exch{"H7N1 NHO"}="1H7N NHO"; $resn_exch{"H7N2 NHO"}="2H7N NHO";
$resn_exch{"H3R1 NHO"}="1H3R NHO"; $resn_exch{"H3R2 NHO"}="2H3R NHO"; $resn_exch{"H1R1 NHO"}="1H1R NHO";
$resn_exch{"H1R2 NHO"}="2H1R NHO"; $resn_exch{"HN11 NHP"}="1HN1 NHP"; $resn_exch{"HN12 NHP"}="2HN1 NHP";
$resn_exch{" H71 NHP"}="1H7  NHP"; $resn_exch{" H72 NHP"}="2H7  NHP"; $resn_exch{" H81 NHP"}="1H8  NHP";
$resn_exch{" H82 NHP"}="2H8  NHP"; $resn_exch{" H91 NHP"}="1H9  NHP"; $resn_exch{" H92 NHP"}="2H9  NHP";
$resn_exch{"H101 NHP"}="1H10 NHP"; $resn_exch{"H102 NHP"}="2H10 NHP"; $resn_exch{"H6A1 NHQ"}="1H6A NHQ";
$resn_exch{"H6A2 NHQ"}="2H6A NHQ"; $resn_exch{"HEM1 NHQ"}="1HEM NHQ"; $resn_exch{"HEM2 NHQ"}="2HEM NHQ";
$resn_exch{"HDM1 NHQ"}="1HDM NHQ"; $resn_exch{"HDM2 NHQ"}="2HDM NHQ"; $resn_exch{"HBM1 NHQ"}="1HBM NHQ";
$resn_exch{"HBM2 NHQ"}="2HBM NHQ"; $resn_exch{"HCM1 NHQ"}="1HCM NHQ"; $resn_exch{"HCM2 NHQ"}="2HCM NHQ";
$resn_exch{"H9M1 NHQ"}="1H9M NHQ"; $resn_exch{"H9M2 NHQ"}="2H9M NHQ"; $resn_exch{"H8M1 NHQ"}="1H8M NHQ";
$resn_exch{"H8M2 NHQ"}="2H8M NHQ"; $resn_exch{"H7M1 NHQ"}="1H7M NHQ"; $resn_exch{"H7M2 NHQ"}="2H7M NHQ";
$resn_exch{"H531 NHQ"}="1H53 NHQ"; $resn_exch{"H532 NHQ"}="2H53 NHQ"; $resn_exch{"H3M1 NHQ"}="1H3M NHQ";
$resn_exch{"H3M2 NHQ"}="2H3M NHQ"; $resn_exch{"H4M1 NHQ"}="1H4M NHQ"; $resn_exch{"H4M2 NHQ"}="2H4M NHQ";
$resn_exch{"H5M1 NHQ"}="1H5M NHQ"; $resn_exch{"H5M2 NHQ"}="2H5M NHQ"; $resn_exch{"HPC1 NHQ"}="1HPC NHQ";
$resn_exch{"HPC2 NHQ"}="2HPC NHQ"; $resn_exch{"H2C1 NHQ"}="1H2C NHQ"; $resn_exch{"H2C2 NHQ"}="2H2C NHQ";
$resn_exch{"H3C1 NHQ"}="1H3C NHQ"; $resn_exch{"H3C2 NHQ"}="2H3C NHQ"; $resn_exch{"H6C1 NHQ"}="1H6C NHQ";
$resn_exch{"H6C2 NHQ"}="2H6C NHQ"; $resn_exch{"H7C1 NHQ"}="1H7C NHQ"; $resn_exch{"H7C2 NHQ"}="2H7C NHQ";
$resn_exch{"H131 NHQ"}="1H13 NHQ"; $resn_exch{"H132 NHQ"}="2H13 NHQ"; $resn_exch{"H133 NHQ"}="3H13 NHQ";
$resn_exch{"H141 NHQ"}="1H14 NHQ"; $resn_exch{"H142 NHQ"}="2H14 NHQ"; $resn_exch{"H143 NHQ"}="3H14 NHQ";
$resn_exch{"H121 NHQ"}="1H12 NHQ"; $resn_exch{"H122 NHQ"}="2H12 NHQ"; $resn_exch{"H5X1 NHQ"}="1H5X NHQ";
$resn_exch{"H5X2 NHQ"}="2H5X NHQ"; $resn_exch{"HNA1 NHR"}="1HNA NHR"; $resn_exch{"HNA2 NHR"}="2HNA NHR";
$resn_exch{" H4O NHR"}=" OH4 NHR"; $resn_exch{" H91 NHR"}="1H9  NHR"; $resn_exch{" H92 NHR"}="2H9  NHR";
$resn_exch{"H101 NHR"}="1H10 NHR"; $resn_exch{"H161 NHR"}="1H16 NHR"; $resn_exch{"H121 NHR"}="1H12 NHR";
$resn_exch{"H131 NHR"}="1H13 NHR"; $resn_exch{"H151 NHR"}="1H15 NHR"; $resn_exch{" HB1 NHR"}="1HB  NHR";
$resn_exch{" HB2 NHR"}="2HB  NHR"; $resn_exch{" HG1 NHR"}="1HG  NHR"; $resn_exch{" HG2 NHR"}="2HG  NHR";
$resn_exch{"HOE1 NHR"}="1HOE NHR"; $resn_exch{"HOA1 NHR"}="1HOA NHR"; $resn_exch{"HN21 NHS"}="1HN2 NHS";
$resn_exch{"HN22 NHS"}="2HN2 NHS"; $resn_exch{" H91 NHS"}="1H9  NHS"; $resn_exch{" H92 NHS"}="2H9  NHS";
$resn_exch{" HB1 NHS"}="1HB  NHS"; $resn_exch{" HB2 NHS"}="2HB  NHS"; $resn_exch{" HG1 NHS"}="1HG  NHS";
$resn_exch{" HG2 NHS"}="2HG  NHS"; $resn_exch{"HOE2 NHS"}="2HOE NHS"; $resn_exch{"H6B1 NHV"}="1H6B NHV";
$resn_exch{"H6B2 NHV"}="2H6B NHV"; $resn_exch{"H7C1 NHV"}="1H7C NHV"; $resn_exch{"H7C2 NHV"}="2H7C NHV";
$resn_exch{"HN21 NHY"}="1HN2 NHY"; $resn_exch{"HN22 NHY"}="2HN2 NHY"; $resn_exch{"HO11 NI1"}="1HO1 NI1";
$resn_exch{"HO12 NI1"}="2HO1 NI1"; $resn_exch{"HO11 NI2"}="1HO1 NI2"; $resn_exch{"HO12 NI2"}="2HO1 NI2";
$resn_exch{"HO13 NI2"}="3HO1 NI2"; $resn_exch{"HO14 NI2"}="4HO1 NI2"; $resn_exch{"HO11 NI3"}="1HO1 NI3";
$resn_exch{"HO12 NI3"}="2HO1 NI3"; $resn_exch{"HO21 NI3"}="1HO2 NI3"; $resn_exch{"HO22 NI3"}="2HO2 NI3";
$resn_exch{"HO31 NI3"}="1HO3 NI3"; $resn_exch{"HO32 NI3"}="2HO3 NI3"; $resn_exch{" H21 NI9"}="1H2  NI9";
$resn_exch{" H22 NI9"}="2H2  NI9"; $resn_exch{"H5'1 NIA"}="1H5' NIA"; $resn_exch{"H5'2 NIA"}="2H5' NIA";
$resn_exch{"HN61 NIA"}="1HN6 NIA"; $resn_exch{"HN62 NIA"}="2HN6 NIA"; $resn_exch{" H41 NIC"}="1H4  NIC";
$resn_exch{" H42 NIC"}="2H4  NIC"; $resn_exch{" HB1 NIG"}="1HB  NIG"; $resn_exch{" HB2 NIG"}="2HB  NIG";
$resn_exch{" HG1 NIG"}="1HG  NIG"; $resn_exch{" HG2 NIG"}="2HG  NIG"; $resn_exch{" H11 NIK"}="1H1  NIK";
$resn_exch{" H12 NIK"}="2H1  NIK"; $resn_exch{" H21 NIK"}="1H2  NIK"; $resn_exch{" H22 NIK"}="2H2  NIK";
$resn_exch{" H31 NIK"}="1H3  NIK"; $resn_exch{" H32 NIK"}="2H3  NIK"; $resn_exch{" H41 NIK"}="1H4  NIK";
$resn_exch{" H42 NIK"}="2H4  NIK"; $resn_exch{" H51 NIK"}="1H5  NIK"; $resn_exch{" H52 NIK"}="2H5  NIK";
$resn_exch{"H11A NIM"}="1H1  NIM"; $resn_exch{"H12A NIM"}="2H1  NIM"; $resn_exch{"H13A NIM"}="3H1  NIM";
$resn_exch{" H71 NIP"}="1H7  NIP"; $resn_exch{" H72 NIP"}="2H7  NIP"; $resn_exch{"H101 NIP"}="1H10 NIP";
$resn_exch{"H102 NIP"}="2H10 NIP"; $resn_exch{"H111 NIP"}="1H11 NIP"; $resn_exch{"H112 NIP"}="2H11 NIP";
$resn_exch{"H121 NIP"}="1H12 NIP"; $resn_exch{"H122 NIP"}="2H12 NIP"; $resn_exch{"H131 NIP"}="1H13 NIP";
$resn_exch{"H132 NIP"}="2H13 NIP"; $resn_exch{"H141 NIP"}="1H14 NIP"; $resn_exch{"H142 NIP"}="2H14 NIP";
$resn_exch{"H1N1 NIR"}="1H1N NIR"; $resn_exch{"H1N2 NIR"}="2H1N NIR"; $resn_exch{" H41 NIR"}="1H4  NIR";
$resn_exch{" H55 NIR"}="5H5  NIR"; $resn_exch{" H81 NIR"}="1H8  NIR"; $resn_exch{" H1' NIR"}=" H1* NIR";
$resn_exch{" H2' NIR"}=" H2* NIR"; $resn_exch{" H3' NIR"}=" H3* NIR"; $resn_exch{" H4' NIR"}=" H4* NIR";
$resn_exch{"H5'1 NIR"}="1H5* NIR"; $resn_exch{"H5'2 NIR"}="2H5* NIR"; $resn_exch{"H5'3 NIR"}="3H5* NIR";
$resn_exch{" H11 NIS"}="1H1  NIS"; $resn_exch{" H12 NIS"}="2H1  NIS"; $resn_exch{" H13 NIS"}="3H1  NIS";
$resn_exch{" H41 NIS"}="1H4  NIS"; $resn_exch{" H42 NIS"}="2H4  NIS"; $resn_exch{" H43 NIS"}="3H4  NIS";
$resn_exch{"HN11 NIT"}="1HN1 NIT"; $resn_exch{"HN12 NIT"}="2HN1 NIT"; $resn_exch{"HG21 NIU"}="1HG2 NIU";
$resn_exch{"HG22 NIU"}="2HG2 NIU"; $resn_exch{"HG31 NIU"}="1HG3 NIU"; $resn_exch{"HG32 NIU"}="2HG3 NIU";
$resn_exch{"HG33 NIU"}="3HG3 NIU"; $resn_exch{"HD11 NIU"}="1HD1 NIU"; $resn_exch{"HD12 NIU"}="2HD1 NIU";
$resn_exch{"HD21 NIU"}="1HD2 NIU"; $resn_exch{"HD22 NIU"}="2HD2 NIU"; $resn_exch{"HD31 NIU"}="1HD3 NIU";
$resn_exch{"HD32 NIU"}="2HD3 NIU"; $resn_exch{"HD41 NIU"}="1HD4 NIU"; $resn_exch{"HD42 NIU"}="2HD4 NIU";
$resn_exch{"HD51 NIU"}="1HD5 NIU"; $resn_exch{"HD52 NIU"}="2HD5 NIU"; $resn_exch{"H121 NIV"}="1H12 NIV";
$resn_exch{"H122 NIV"}="2H12 NIV"; $resn_exch{"H123 NIV"}="3H12 NIV"; $resn_exch{"H141 NIV"}="1H14 NIV";
$resn_exch{"H142 NIV"}="2H14 NIV"; $resn_exch{"H151 NIV"}="1H15 NIV"; $resn_exch{"H152 NIV"}="2H15 NIV";
$resn_exch{"HE11 NIX"}="1HE1 NIX"; $resn_exch{"HE21 NIX"}="1HE2 NIX"; $resn_exch{"HE12 NIX"}="2HE1 NIX";
$resn_exch{"HE22 NIX"}="2HE2 NIX"; $resn_exch{"HE32 NIX"}="2HE3 NIX"; $resn_exch{"H2L1 NIX"}="1H2L NIX";
$resn_exch{" H2  NIY"}=" HN2 NIY"; $resn_exch{" HB2 NIY"}="1HB  NIY"; $resn_exch{" HB3 NIY"}="2HB  NIY";
$resn_exch{" H41 NK1"}="1H4  NK1"; $resn_exch{" H42 NK1"}="2H4  NK1"; $resn_exch{" H31 NK1"}="1H3  NK1";
$resn_exch{" H32 NK1"}="2H3  NK1"; $resn_exch{" H21 NK1"}="1H2  NK1"; $resn_exch{" H22 NK1"}="2H2  NK1";
$resn_exch{" H91 NK1"}="1H9  NK1"; $resn_exch{" H92 NK1"}="2H9  NK1"; $resn_exch{" H51 NK2"}="1H5  NK2";
$resn_exch{" H52 NK2"}="2H5  NK2"; $resn_exch{" H41 NK2"}="1H4  NK2"; $resn_exch{" H42 NK2"}="2H4  NK2";
$resn_exch{" H31 NK2"}="1H3  NK2"; $resn_exch{" H32 NK2"}="2H3  NK2"; $resn_exch{" H21 NK2"}="1H2  NK2";
$resn_exch{" H22 NK2"}="2H2  NK2"; $resn_exch{" H91 NK2"}="1H9  NK2"; $resn_exch{" H92 NK2"}="2H9  NK2";
$resn_exch{" H31 NLA"}="1H3  NLA"; $resn_exch{" H41 NLA"}="1H4  NLA"; $resn_exch{" H51 NLA"}="1H5  NLA";
$resn_exch{" H71 NLA"}="1H7  NLA"; $resn_exch{" H81 NLA"}="1H8  NLA"; $resn_exch{" H91 NLA"}="1H9  NLA";
$resn_exch{"H101 NLA"}="1H10 NLA"; $resn_exch{"H111 NLA"}="1H11 NLA"; $resn_exch{"H112 NLA"}="2H11 NLA";
$resn_exch{"H8C1 NLC"}="1H8C NLC"; $resn_exch{"H8C2 NLC"}="2H8C NLC"; $resn_exch{"H8C3 NLC"}="3H8C NLC";
$resn_exch{"H6C1 NLC"}="1H6C NLC"; $resn_exch{"H6C2 NLC"}="2H6C NLC"; $resn_exch{"H161 NLC"}="1H16 NLC";
$resn_exch{"H162 NLC"}="2H16 NLC"; $resn_exch{" H   NLE"}="1HN  NLE"; $resn_exch{" HN2 NLE"}="2HN  NLE";
$resn_exch{" HB2 NLE"}="1HB  NLE"; $resn_exch{" HB3 NLE"}="2HB  NLE"; $resn_exch{" HG2 NLE"}="1HG  NLE";
$resn_exch{" HG3 NLE"}="2HG  NLE"; $resn_exch{" HD2 NLE"}="1HD  NLE"; $resn_exch{" HD3 NLE"}="2HD  NLE";
$resn_exch{" HE1 NLE"}="1HE  NLE"; $resn_exch{" HE2 NLE"}="2HE  NLE"; $resn_exch{" HE3 NLE"}="3HE  NLE";
$resn_exch{"HBC1 NLG"}="1HBC NLG"; $resn_exch{"HBC2 NLG"}="2HBC NLG"; $resn_exch{"HGC1 NLG"}="1HGC NLG";
$resn_exch{"HGC2 NLG"}="2HGC NLG"; $resn_exch{"H8C1 NLG"}="1H8C NLG"; $resn_exch{"H8C2 NLG"}="2H8C NLG";
$resn_exch{"H8C3 NLG"}="3H8C NLG"; $resn_exch{" H   NLN"}="1HN  NLN"; $resn_exch{" H2  NLN"}="2HN  NLN";
$resn_exch{" HB2 NLN"}="1HB  NLN"; $resn_exch{" HB3 NLN"}="2HB  NLN"; $resn_exch{" HG2 NLN"}="1HG  NLN";
$resn_exch{" HG3 NLN"}="2HG  NLN"; $resn_exch{" HD2 NLN"}="1HD  NLN"; $resn_exch{" HD3 NLN"}="2HD  NLN";
$resn_exch{" HE1 NLN"}="1HE  NLN"; $resn_exch{" HE2 NLN"}="2HE  NLN"; $resn_exch{" HE3 NLN"}="3HE  NLN";
$resn_exch{"HH21 NLN"}="1HH2 NLN"; $resn_exch{"HH22 NLN"}="2HH2 NLN"; $resn_exch{" H   NLO"}="1HN  NLO";
$resn_exch{" H2  NLO"}="2HN  NLO"; $resn_exch{" HB2 NLO"}="1HB  NLO"; $resn_exch{" HB3 NLO"}="2HB  NLO";
$resn_exch{" HG2 NLO"}="1HG  NLO"; $resn_exch{" HG3 NLO"}="2HG  NLO"; $resn_exch{" HD2 NLO"}="1HD  NLO";
$resn_exch{" HD3 NLO"}="2HD  NLO"; $resn_exch{" HE1 NLO"}="1HE  NLO"; $resn_exch{" HE2 NLO"}="2HE  NLO";
$resn_exch{" HE3 NLO"}="3HE  NLO"; $resn_exch{" HN1 NLP"}="1HN  NLP"; $resn_exch{" HN2 NLP"}="2HN  NLP";
$resn_exch{" HB1 NLP"}="1HB  NLP"; $resn_exch{" HB2 NLP"}="2HB  NLP"; $resn_exch{" HG1 NLP"}="1HG  NLP";
$resn_exch{" HG2 NLP"}="2HG  NLP"; $resn_exch{" HD1 NLP"}="1HD  NLP"; $resn_exch{" HD2 NLP"}="2HD  NLP";
$resn_exch{" HE1 NLP"}="1HE  NLP"; $resn_exch{" HE2 NLP"}="2HE  NLP"; $resn_exch{" HE3 NLP"}="3HE  NLP";
$resn_exch{" H   NLQ"}=" HN  NLQ"; $resn_exch{" H71 NLQ"}="1H7  NLQ"; $resn_exch{" H72 NLQ"}="2H7  NLQ";
$resn_exch{" H73 NLQ"}="3H7  NLQ"; $resn_exch{" HB2 NLQ"}="1HB  NLQ"; $resn_exch{" HB3 NLQ"}="2HB  NLQ";
$resn_exch{" HG2 NLQ"}="1HG  NLQ"; $resn_exch{" HG3 NLQ"}="2HG  NLQ"; $resn_exch{"HE21 NLQ"}="1HE2 NLQ";
$resn_exch{"HE22 NLQ"}="2HE2 NLQ"; $resn_exch{" H21 NLT"}="1H2  NLT"; $resn_exch{" H22 NLT"}="2H2  NLT";
$resn_exch{" H31 NLT"}="1H3  NLT"; $resn_exch{" H32 NLT"}="2H3  NLT"; $resn_exch{" H41 NLT"}="1H4  NLT";
$resn_exch{" H42 NLT"}="2H4  NLT"; $resn_exch{" H51 NLT"}="1H5  NLT"; $resn_exch{" H52 NLT"}="2H5  NLT";
$resn_exch{" H61 NLT"}="1H6  NLT"; $resn_exch{" H62 NLT"}="2H6  NLT"; $resn_exch{" H71 NLT"}="1H7  NLT";
$resn_exch{" H72 NLT"}="2H7  NLT"; $resn_exch{" H81 NLT"}="1H8  NLT"; $resn_exch{" H82 NLT"}="2H8  NLT";
$resn_exch{" H91 NLT"}="1H9  NLT"; $resn_exch{" H92 NLT"}="2H9  NLT"; $resn_exch{"H101 NLT"}="1H10 NLT";
$resn_exch{"H102 NLT"}="2H10 NLT"; $resn_exch{"H111 NLT"}="1H11 NLT"; $resn_exch{"H112 NLT"}="2H11 NLT";
$resn_exch{"H121 NLT"}="1H12 NLT"; $resn_exch{"H122 NLT"}="2H12 NLT"; $resn_exch{"H123 NLT"}="3H12 NLT";
$resn_exch{" HB1 NLT"}="1HB  NLT"; $resn_exch{" HB2 NLT"}="2HB  NLT"; $resn_exch{"HO11 NLX"}="1HO1 NLX";
$resn_exch{"HO41 NLX"}="1HO4 NLX"; $resn_exch{" H11 NLX"}="1H1  NLX"; $resn_exch{" H21 NLX"}="1H2  NLX";
$resn_exch{" H51 NLX"}="1H5  NLX"; $resn_exch{" H71 NLX"}="1H7  NLX"; $resn_exch{" H72 NLX"}="2H7  NLX";
$resn_exch{" H81 NLX"}="1H8  NLX"; $resn_exch{" H82 NLX"}="2H8  NLX"; $resn_exch{" H91 NLX"}="1H9  NLX";
$resn_exch{"H101 NLX"}="1H10 NLX"; $resn_exch{"H102 NLX"}="2H10 NLX"; $resn_exch{"H151 NLX"}="1H15 NLX";
$resn_exch{"H152 NLX"}="2H15 NLX"; $resn_exch{"H161 NLX"}="1H16 NLX"; $resn_exch{"H162 NLX"}="2H16 NLX";
$resn_exch{"H171 NLX"}="1H17 NLX"; $resn_exch{"H172 NLX"}="2H17 NLX"; $resn_exch{"H181 NLX"}="1H18 NLX";
$resn_exch{"H191 NLX"}="1H19 NLX"; $resn_exch{"H192 NLX"}="2H19 NLX"; $resn_exch{"H201 NLX"}="1H20 NLX";
$resn_exch{"H202 NLX"}="2H20 NLX"; $resn_exch{"H203 NLX"}="3H20 NLX"; $resn_exch{"H081 NM1"}="1H08 NM1";
$resn_exch{"H082 NM1"}="2H08 NM1"; $resn_exch{"H231 NM1"}="1H23 NM1"; $resn_exch{"H232 NM1"}="2H23 NM1";
$resn_exch{"H321 NM1"}="1H32 NM1"; $resn_exch{"H322 NM1"}="2H32 NM1"; $resn_exch{"H411 NM1"}="1H41 NM1";
$resn_exch{"H412 NM1"}="2H41 NM1"; $resn_exch{"H541 NM1"}="1H54 NM1"; $resn_exch{"H542 NM1"}="2H54 NM1";
$resn_exch{"H543 NM1"}="3H54 NM1"; $resn_exch{"H4A1 NMA"}="1H4A NMA"; $resn_exch{"H4A2 NMA"}="2H4A NMA";
$resn_exch{"HOP2 NMA"}="2HOP NMA"; $resn_exch{"HOP3 NMA"}="3HOP NMA"; $resn_exch{"H5A1 NMA"}="1H5A NMA";
$resn_exch{"H5A2 NMA"}="2H5A NMA"; $resn_exch{"H2A1 NMA"}="1H2A NMA"; $resn_exch{"H2A2 NMA"}="2H2A NMA";
$resn_exch{"H2A3 NMA"}="3H2A NMA"; $resn_exch{" H91 NMA"}="1H9  NMA"; $resn_exch{" H92 NMA"}="2H9  NMA";
$resn_exch{" H93 NMA"}="3H9  NMA"; $resn_exch{"H101 NMA"}="1H10 NMA"; $resn_exch{"H102 NMA"}="2H10 NMA";
$resn_exch{"H103 NMA"}="3H10 NMA"; $resn_exch{"H081 NMB"}="1H08 NMB"; $resn_exch{"H082 NMB"}="2H08 NMB";
$resn_exch{"H231 NMB"}="1H23 NMB"; $resn_exch{"H232 NMB"}="2H23 NMB"; $resn_exch{"H321 NMB"}="1H32 NMB";
$resn_exch{"H322 NMB"}="2H32 NMB"; $resn_exch{"H411 NMB"}="1H41 NMB"; $resn_exch{"H412 NMB"}="2H41 NMB";
$resn_exch{" HA1 NMC"}="1HA  NMC"; $resn_exch{" HA2 NMC"}="2HA  NMC"; $resn_exch{"HCN1 NMC"}="1HCN NMC";
$resn_exch{"HCN2 NMC"}="2HCN NMC"; $resn_exch{"HC21 NMC"}="1HC2 NMC"; $resn_exch{"HC22 NMC"}="2HC2 NMC";
$resn_exch{"HC31 NMC"}="1HC3 NMC"; $resn_exch{"HC32 NMC"}="2HC3 NMC"; $resn_exch{"H5'1 NMD"}="1H5* NMD";
$resn_exch{"H5'2 NMD"}="2H5* NMD"; $resn_exch{" H4' NMD"}=" H4* NMD"; $resn_exch{" H3' NMD"}=" H3* NMD";
$resn_exch{"HO3' NMD"}="*HO3 NMD"; $resn_exch{" H2' NMD"}=" H2* NMD"; $resn_exch{"HN2' NMD"}="*HN2 NMD";
$resn_exch{" H1' NMD"}=" H1* NMD"; $resn_exch{"H111 NMD"}="1H11 NMD"; $resn_exch{"H112 NMD"}="2H11 NMD";
$resn_exch{"H2M1 NMD"}="1H2M NMD"; $resn_exch{"H2M2 NMD"}="2H2M NMD"; $resn_exch{"H2M3 NMD"}="3H2M NMD";
$resn_exch{"H1M1 NMD"}="1H1M NMD"; $resn_exch{"H1M2 NMD"}="2H1M NMD"; $resn_exch{"H1M3 NMD"}="3H1M NMD";
$resn_exch{"HO5' NMD"}="*HO5 NMD"; $resn_exch{" HN1 NME"}="1HN  NME"; $resn_exch{" HN2 NME"}="2HN  NME";
$resn_exch{" H1  NME"}="1H   NME"; $resn_exch{" H2  NME"}="2H   NME"; $resn_exch{" H3  NME"}="3H   NME";
$resn_exch{" HD1 NMG"}="1HD  NMG"; $resn_exch{" HD2 NMG"}="2HD  NMG"; $resn_exch{"HH11 NMG"}="1HH1 NMG";
$resn_exch{"HH12 NMG"}="2HH1 NMG"; $resn_exch{"HC71 NMH"}="1HC7 NMH"; $resn_exch{"HC72 NMH"}="2HC7 NMH";
$resn_exch{"HC73 NMH"}="3HC7 NMH"; $resn_exch{"HC61 NMH"}="1HC6 NMH"; $resn_exch{"HC62 NMH"}="2HC6 NMH";
$resn_exch{"HC51 NMH"}="1HC5 NMH"; $resn_exch{"HC52 NMH"}="2HC5 NMH"; $resn_exch{"HC41 NMH"}="1HC4 NMH";
$resn_exch{"HC42 NMH"}="2HC4 NMH"; $resn_exch{"HC31 NMH"}="1HC3 NMH"; $resn_exch{"HC32 NMH"}="2HC3 NMH";
$resn_exch{"HC11 NMH"}="1HC1 NMH"; $resn_exch{"HC12 NMH"}="2HC1 NMH"; $resn_exch{"HC13 NMH"}="3HC1 NMH";
$resn_exch{" HN' NMH"}=" HN* NMH"; $resn_exch{" HC' NMH"}=" HC* NMH"; $resn_exch{" H11 NML"}="1H1  NML";
$resn_exch{" H12 NML"}="2H1  NML"; $resn_exch{" H13 NML"}="3H1  NML"; $resn_exch{" H31 NML"}="1H3  NML";
$resn_exch{" H32 NML"}="2H3  NML"; $resn_exch{" H33 NML"}="3H3  NML"; $resn_exch{"HAA1 NMM"}="1HAA NMM";
$resn_exch{"HAA2 NMM"}="2HAA NMM"; $resn_exch{"HAA3 NMM"}="3HAA NMM"; $resn_exch{" HE  NMM"}=" HNE NMM";
$resn_exch{" HD3 NMM"}="1HD  NMM"; $resn_exch{" HD2 NMM"}="2HD  NMM"; $resn_exch{" HG3 NMM"}="1HG  NMM";
$resn_exch{" HG2 NMM"}="2HG  NMM"; $resn_exch{" HB3 NMM"}="1HB  NMM"; $resn_exch{" HB2 NMM"}="2HB  NMM";
$resn_exch{" H   NMM"}="1HN  NMM"; $resn_exch{" H2  NMM"}="2HN  NMM"; $resn_exch{"H1PO NMN"}="OH1P NMN";
$resn_exch{"H2PO NMN"}="OH2P NMN"; $resn_exch{"H5R1 NMN"}="1H5R NMN"; $resn_exch{"H5R2 NMN"}="2H5R NMN";
$resn_exch{"H4RC NMN"}="CH4R NMN"; $resn_exch{"H3RC NMN"}="CH3R NMN"; $resn_exch{"H3RO NMN"}="OH3R NMN";
$resn_exch{"H2RC NMN"}="CH2R NMN"; $resn_exch{"H2RO NMN"}="OH2R NMN"; $resn_exch{"H1RC NMN"}="CH1R NMN";
$resn_exch{"HN71 NMN"}="1HN7 NMN"; $resn_exch{"HN72 NMN"}="2HN7 NMN"; $resn_exch{"H261 NMP"}="1H26 NMP";
$resn_exch{"H262 NMP"}="2H26 NMP"; $resn_exch{"H251 NMP"}="1H25 NMP"; $resn_exch{"H252 NMP"}="2H25 NMP";
$resn_exch{"H241 NMP"}="1H24 NMP"; $resn_exch{"H242 NMP"}="2H24 NMP"; $resn_exch{"H221 NMP"}="1H22 NMP";
$resn_exch{"H222 NMP"}="2H22 NMP"; $resn_exch{"H223 NMP"}="3H22 NMP"; $resn_exch{"H151 NMP"}="1H15 NMP";
$resn_exch{"H152 NMP"}="2H15 NMP"; $resn_exch{" H31 NMP"}="1H3  NMP"; $resn_exch{" H32 NMP"}="2H3  NMP";
$resn_exch{" H91 NMP"}="1H9  NMP"; $resn_exch{" H92 NMP"}="2H9  NMP"; $resn_exch{"H101 NMP"}="1H10 NMP";
$resn_exch{"H102 NMP"}="2H10 NMP"; $resn_exch{"H111 NMP"}="1H11 NMP"; $resn_exch{"H112 NMP"}="2H11 NMP";
$resn_exch{"H121 NMP"}="1H12 NMP"; $resn_exch{"H122 NMP"}="2H12 NMP"; $resn_exch{"H131 NMP"}="1H13 NMP";
$resn_exch{"H132 NMP"}="2H13 NMP"; $resn_exch{"HOB2 NMQ"}="2HOB NMQ"; $resn_exch{"HOA2 NMQ"}="2HOA NMQ";
$resn_exch{"HA21 NMQ"}="1HA2 NMQ"; $resn_exch{"HA22 NMQ"}="2HA2 NMQ"; $resn_exch{"HA11 NMQ"}="1HA1 NMQ";
$resn_exch{"HA12 NMQ"}="2HA1 NMQ"; $resn_exch{"HA31 NMQ"}="1HA3 NMQ"; $resn_exch{"HA32 NMQ"}="2HA3 NMQ";
$resn_exch{"HA33 NMQ"}="3HA3 NMQ"; $resn_exch{"HOP2 NMS"}="2HOP NMS"; $resn_exch{" H71 NMS"}="1H5M NMS";
$resn_exch{" H72 NMS"}="2H5M NMS"; $resn_exch{" H73 NMS"}="3H5M NMS"; $resn_exch{" H2' NMS"}=" H2* NMS";
$resn_exch{" H5' NMS"}="1H5* NMS"; $resn_exch{"H5'' NMS"}="2H5* NMS"; $resn_exch{" H4' NMS"}=" H4* NMS";
$resn_exch{" H1' NMS"}=" H1* NMS"; $resn_exch{" H3' NMS"}=" H3* NMS"; $resn_exch{"HO3' NMS"}="H3T  NMS";
$resn_exch{"H7'1 NMS"}="1H7' NMS"; $resn_exch{"H7'2 NMS"}="2H7' NMS"; $resn_exch{"H101 NMS"}="1H10 NMS";
$resn_exch{"H102 NMS"}="2H10 NMS"; $resn_exch{"H103 NMS"}="3H10 NMS"; $resn_exch{"HOP3 NMS"}="3HOP NMS";
$resn_exch{"HOP3 NMT"}="3HOP NMT"; $resn_exch{"HOP2 NMT"}="2HOP NMT"; $resn_exch{" H71 NMT"}="1H5M NMT";
$resn_exch{" H72 NMT"}="2H5M NMT"; $resn_exch{" H73 NMT"}="3H5M NMT"; $resn_exch{" H2' NMT"}=" H2* NMT";
$resn_exch{" H5' NMT"}="1H5* NMT"; $resn_exch{"H5'' NMT"}="2H5* NMT"; $resn_exch{" H4' NMT"}=" H4* NMT";
$resn_exch{" H1' NMT"}=" H1* NMT"; $resn_exch{" H3' NMT"}=" H3* NMT"; $resn_exch{"HO3' NMT"}="H3T  NMT";
$resn_exch{"H9'1 NMT"}="1H9' NMT"; $resn_exch{"H9'2 NMT"}="2H9' NMT"; $resn_exch{"H9'3 NMT"}="3H9' NMT";
$resn_exch{"HN31 NMU"}="1HN3 NMU"; $resn_exch{"HN32 NMU"}="2HN3 NMU"; $resn_exch{" H51 NMU"}="1H5  NMU";
$resn_exch{" H52 NMU"}="2H5  NMU"; $resn_exch{" H53 NMU"}="3H5  NMU"; $resn_exch{" H2A NMX"}=" AH2 NMX";
$resn_exch{"H61A NMX"}="AH61 NMX"; $resn_exch{"H62A NMX"}="AH62 NMX"; $resn_exch{" H8A NMX"}=" AH8 NMX";
$resn_exch{" H1B NMX"}="AH1* NMX"; $resn_exch{" H2B NMX"}="AH2* NMX"; $resn_exch{"HO2A NMX"}="AHO2 NMX";
$resn_exch{" H3B NMX"}="AH3* NMX"; $resn_exch{"HOA8 NMX"}="8HOA NMX"; $resn_exch{"HOA9 NMX"}="9HOA NMX";
$resn_exch{" H4B NMX"}="AH4* NMX"; $resn_exch{"H51A NMX"}="AH51 NMX"; $resn_exch{"H52A NMX"}="AH52 NMX";
$resn_exch{"HOA2 NMX"}="2HOA NMX"; $resn_exch{"HOA5 NMX"}="5HOA NMX"; $resn_exch{"H121 NMX"}="1H12 NMX";
$resn_exch{"H122 NMX"}="2H12 NMX"; $resn_exch{"H131 NMX"}="1H13 NMX"; $resn_exch{"H132 NMX"}="2H13 NMX";
$resn_exch{"H133 NMX"}="3H13 NMX"; $resn_exch{"H141 NMX"}="1H14 NMX"; $resn_exch{"H142 NMX"}="2H14 NMX";
$resn_exch{"H143 NMX"}="3H14 NMX"; $resn_exch{" H71 NMX"}="1H7  NMX"; $resn_exch{" H72 NMX"}="2H7  NMX";
$resn_exch{" H61 NMX"}="1H6  NMX"; $resn_exch{" H62 NMX"}="2H6  NMX"; $resn_exch{" H31 NMX"}="1H3  NMX";
$resn_exch{" H32 NMX"}="2H3  NMX"; $resn_exch{" H21 NMX"}="1H2  NMX"; $resn_exch{" H22 NMX"}="2H2  NMX";
$resn_exch{"HI11 NMX"}="1HI1 NMX"; $resn_exch{"HI12 NMX"}="2HI1 NMX"; $resn_exch{"HN21 NMY"}="1HN2 NMY";
$resn_exch{"HN22 NMY"}="2HN2 NMY"; $resn_exch{" H61 NMY"}="1H6  NMY"; $resn_exch{" H62 NMY"}="2H6  NMY";
$resn_exch{" H71 NMY"}="1H7  NMY"; $resn_exch{" H72 NMY"}="2H7  NMY"; $resn_exch{" H91 NMY"}="1H9  NMY";
$resn_exch{" H92 NMY"}="2H9  NMY"; $resn_exch{" H81 NMY"}="1H8  NMY"; $resn_exch{" H82 NMY"}="2H8  NMY";
$resn_exch{"H171 NMY"}="1H17 NMY"; $resn_exch{"H172 NMY"}="2H17 NMY"; $resn_exch{"H231 NMY"}="1H23 NMY";
$resn_exch{"H232 NMY"}="2H23 NMY"; $resn_exch{"HN61 NMY"}="1HN6 NMY"; $resn_exch{"HN62 NMY"}="2HN6 NMY";
$resn_exch{"HO41 NMY"}="1HO4 NMY"; $resn_exch{"HO22 NMY"}="2HO2 NMY"; $resn_exch{"HO21 NMY"}="1HO2 NMY";
$resn_exch{"HO12 NMY"}="2HO1 NMY"; $resn_exch{"H191 NMY"}="1H19 NMY"; $resn_exch{"H192 NMY"}="2H19 NMY";
$resn_exch{"HN31 NMY"}="1HN3 NMY"; $resn_exch{"HN32 NMY"}="2HN3 NMY"; $resn_exch{" H51 NN1"}="1H5  NN1";
$resn_exch{" H52 NN1"}="2H5  NN1"; $resn_exch{" H53 NN1"}="3H5  NN1"; $resn_exch{"H171 NN1"}="1H17 NN1";
$resn_exch{"H172 NN1"}="2H17 NN1"; $resn_exch{"H211 NN1"}="1H21 NN1"; $resn_exch{"H212 NN1"}="2H21 NN1";
$resn_exch{"H241 NN1"}="1H24 NN1"; $resn_exch{"H242 NN1"}="2H24 NN1"; $resn_exch{" H11 NN1"}="1H1  NN1";
$resn_exch{" H12 NN1"}="2H1  NN1"; $resn_exch{" H13 NN1"}="3H1  NN1"; $resn_exch{" H31 NN1"}="1H3  NN1";
$resn_exch{" H32 NN1"}="2H3  NN1"; $resn_exch{" H33 NN1"}="3H3  NN1"; $resn_exch{"H191 NN1"}="1H19 NN1";
$resn_exch{"H192 NN1"}="2H19 NN1"; $resn_exch{"H231 NN1"}="1H23 NN1"; $resn_exch{"H232 NN1"}="2H23 NN1";
$resn_exch{"H241 NN2"}="1H24 NN2"; $resn_exch{"H242 NN2"}="2H24 NN2"; $resn_exch{"H251 NN2"}="1H25 NN2";
$resn_exch{"H252 NN2"}="2H25 NN2"; $resn_exch{"H301 NN2"}="1H30 NN2"; $resn_exch{"H302 NN2"}="2H30 NN2";
$resn_exch{"H291 NN2"}="1H29 NN2"; $resn_exch{"H292 NN2"}="2H29 NN2"; $resn_exch{" H51 NN3"}="1H5  NN3";
$resn_exch{" H52 NN3"}="2H5  NN3"; $resn_exch{" H61 NN3"}="1H6  NN3"; $resn_exch{" H62 NN3"}="2H6  NN3";
$resn_exch{" H71 NN3"}="1H7  NN3"; $resn_exch{" H72 NN3"}="2H7  NN3"; $resn_exch{" H73 NN3"}="3H7  NN3";
$resn_exch{" H31 NN3"}="1H3  NN3"; $resn_exch{" H32 NN3"}="2H3  NN3"; $resn_exch{"H21A NN3"}="1H2  NN3";
$resn_exch{"H22A NN3"}="2H2  NN3"; $resn_exch{"H241 NN3"}="1H24 NN3"; $resn_exch{"H242 NN3"}="2H24 NN3";
$resn_exch{"H243 NN3"}="3H24 NN3"; $resn_exch{"H231 NN3"}="1H23 NN3"; $resn_exch{"H232 NN3"}="2H23 NN3";
$resn_exch{"H233 NN3"}="3H23 NN3"; $resn_exch{" H41 NN4"}="1H4  NN4"; $resn_exch{" H42 NN4"}="2H4  NN4";
$resn_exch{" H61 NN4"}="1H6  NN4"; $resn_exch{" H62 NN4"}="2H6  NN4"; $resn_exch{" H71 NN4"}="1H7  NN4";
$resn_exch{" H72 NN4"}="2H7  NN4"; $resn_exch{" H81 NN4"}="1H8  NN4"; $resn_exch{" H82 NN4"}="2H8  NN4";
$resn_exch{"H201 NN4"}="1H20 NN4"; $resn_exch{"H202 NN4"}="2H20 NN4"; $resn_exch{"H203 NN4"}="3H20 NN4";
$resn_exch{"H281 NN4"}="1H28 NN4"; $resn_exch{"H282 NN4"}="2H28 NN4"; $resn_exch{"H283 NN4"}="3H28 NN4";
$resn_exch{" H21 NN4"}="1H2  NN4"; $resn_exch{"H22A NN4"}="2H2  NN4"; $resn_exch{"H131 NN4"}="1H13 NN4";
$resn_exch{"H132 NN4"}="2H13 NN4"; $resn_exch{"H191 NN4"}="1H19 NN4"; $resn_exch{"H192 NN4"}="2H19 NN4";
$resn_exch{"H193 NN4"}="3H19 NN4"; $resn_exch{"H101 NNB"}="1H10 NNB"; $resn_exch{"H102 NNB"}="2H10 NNB";
$resn_exch{"H9C1 NNB"}="1H9C NNB"; $resn_exch{"H9C2 NNB"}="2H9C NNB"; $resn_exch{"H101 NNC"}="1H10 NNC";
$resn_exch{"H102 NNC"}="2H10 NNC"; $resn_exch{"H9C1 NNC"}="1H9C NNC"; $resn_exch{"H9C2 NNC"}="2H9C NNC";
$resn_exch{"H6C1 NND"}="1H6C NND"; $resn_exch{"H6C2 NND"}="2H6C NND"; $resn_exch{"H1C1 NND"}="1H1C NND";
$resn_exch{"H1C2 NND"}="2H1C NND"; $resn_exch{"H8C1 NND"}="1H8C NND"; $resn_exch{"H8C2 NND"}="2H8C NND";
$resn_exch{"H7C1 NND"}="1H7C NND"; $resn_exch{"H7C2 NND"}="2H7C NND"; $resn_exch{"H9C1 NND"}="1H9C NND";
$resn_exch{"H9C2 NND"}="2H9C NND"; $resn_exch{"H101 NND"}="1H10 NND"; $resn_exch{"H102 NND"}="2H10 NND";
$resn_exch{"H141 NND"}="1H14 NND"; $resn_exch{"H142 NND"}="2H14 NND"; $resn_exch{"H131 NND"}="1H13 NND";
$resn_exch{"H132 NND"}="2H13 NND"; $resn_exch{"H121 NND"}="1H12 NND"; $resn_exch{"H122 NND"}="2H12 NND";
$resn_exch{"H111 NND"}="1H11 NND"; $resn_exch{"H112 NND"}="2H11 NND"; $resn_exch{"H161 NND"}="1H16 NND";
$resn_exch{"H162 NND"}="2H16 NND"; $resn_exch{"H163 NND"}="3H16 NND"; $resn_exch{" H61 NNG"}="1H6  NNG";
$resn_exch{" H62 NNG"}="2H6  NNG"; $resn_exch{" H81 NNG"}="1H8  NNG"; $resn_exch{" H82 NNG"}="2H8  NNG";
$resn_exch{" H83 NNG"}="3H8  NNG"; $resn_exch{" H2  NNH"}=" HN2 NNH"; $resn_exch{" HB2 NNH"}="1HB  NNH";
$resn_exch{" HB3 NNH"}="2HB  NNH"; $resn_exch{" HG2 NNH"}="1HG  NNH"; $resn_exch{" HG3 NNH"}="2HG  NNH";
$resn_exch{"H101 NNI"}="1H10 NNI"; $resn_exch{"H102 NNI"}="2H10 NNI"; $resn_exch{"H9C1 NNI"}="1H9C NNI";
$resn_exch{"H9C2 NNI"}="2H9C NNI"; $resn_exch{"H5R1 NNR"}="1H5R NNR"; $resn_exch{"H5R2 NNR"}="2H5R NNR";
$resn_exch{"HN71 NNR"}="1HN7 NNR"; $resn_exch{"HN72 NNR"}="2HN7 NNR"; $resn_exch{" H11 NNS"}="1H1  NNS";
$resn_exch{" H12 NNS"}="2H1  NNS"; $resn_exch{" H13 NNS"}="3H1  NNS"; $resn_exch{" HA1 NOA"}="1HA  NOA";
$resn_exch{" HA2 NOA"}="2HA  NOA"; $resn_exch{"H6'1 NOC"}="1H6* NOC"; $resn_exch{"H6'2 NOC"}="2H6* NOC";
$resn_exch{"HO6' NOC"}="HO6* NOC"; $resn_exch{"H5'1 NOC"}="1H5* NOC"; $resn_exch{"H5'2 NOC"}="2H5* NOC";
$resn_exch{"HC4' NOC"}="HC4* NOC"; $resn_exch{"HC3' NOC"}="HC3* NOC"; $resn_exch{"HO3' NOC"}="HO3* NOC";
$resn_exch{"HC2' NOC"}="HC2* NOC"; $resn_exch{"HO2' NOC"}="HO2* NOC"; $resn_exch{"HC1' NOC"}="HC1* NOC";
$resn_exch{"HN61 NOC"}="1HN6 NOC"; $resn_exch{"HN62 NOC"}="2HN6 NOC"; $resn_exch{"H111 NOD"}="1H11 NOD";
$resn_exch{"H112 NOD"}="2H11 NOD"; $resn_exch{"HO12 NOD"}="2HO1 NOD"; $resn_exch{"H141 NOD"}="1H14 NOD";
$resn_exch{"H142 NOD"}="2H14 NOD"; $resn_exch{"HO14 NOD"}="4HO1 NOD"; $resn_exch{"H151 NOD"}="1H15 NOD";
$resn_exch{"H152 NOD"}="2H15 NOD"; $resn_exch{"HO17 NOD"}="7HO1 NOD"; $resn_exch{"H211 NOD"}="1H21 NOD";
$resn_exch{"H212 NOD"}="2H21 NOD"; $resn_exch{"H213 NOD"}="3H21 NOD"; $resn_exch{" H1' NOD"}=" H1* NOD";
$resn_exch{"H2'1 NOD"}="1H2* NOD"; $resn_exch{"H2'2 NOD"}="2H2* NOD"; $resn_exch{" H3' NOD"}=" H3* NOD";
$resn_exch{" H4' NOD"}=" H4* NOD"; $resn_exch{"HO4' NOD"}="*HO4 NOD"; $resn_exch{" H5' NOD"}=" H5* NOD";
$resn_exch{"H6'1 NOD"}="1H6* NOD"; $resn_exch{"H6'2 NOD"}="2H6* NOD"; $resn_exch{"H6'3 NOD"}="3H6* NOD";
$resn_exch{"HN3' NOD"}="HN3* NOD"; $resn_exch{"H631 NOD"}="1H63 NOD"; $resn_exch{"H632 NOD"}="2H63 NOD";
$resn_exch{"H531 NOD"}="1H53 NOD"; $resn_exch{"H532 NOD"}="2H53 NOD"; $resn_exch{"HC11 NOE"}="1HC1 NOE";
$resn_exch{"HC12 NOE"}="2HC1 NOE"; $resn_exch{"HC21 NOE"}="1HC2 NOE"; $resn_exch{"HC22 NOE"}="2HC2 NOE";
$resn_exch{"HC23 NOE"}="3HC2 NOE"; $resn_exch{"HC11 NOG"}="1HC1 NOG"; $resn_exch{"HC12 NOG"}="2HC1 NOG";
$resn_exch{"HC21 NOG"}="1HC2 NOG"; $resn_exch{"HC22 NOG"}="2HC2 NOG"; $resn_exch{"HC61 NOG"}="1HC6 NOG";
$resn_exch{"HC62 NOG"}="2HC6 NOG"; $resn_exch{"HC71 NOG"}="1HC7 NOG"; $resn_exch{"HC72 NOG"}="2HC7 NOG";
$resn_exch{"H10C NOG"}="CH10 NOG"; $resn_exch{"H111 NOG"}="1H11 NOG"; $resn_exch{"H112 NOG"}="2H11 NOG";
$resn_exch{"H121 NOG"}="1H12 NOG"; $resn_exch{"H122 NOG"}="2H12 NOG"; $resn_exch{"H14C NOG"}="CH14 NOG";
$resn_exch{"H151 NOG"}="1H15 NOG"; $resn_exch{"H152 NOG"}="2H15 NOG"; $resn_exch{"H161 NOG"}="1H16 NOG";
$resn_exch{"H162 NOG"}="2H16 NOG"; $resn_exch{"H181 NOG"}="1H18 NOG"; $resn_exch{"H182 NOG"}="2H18 NOG";
$resn_exch{"H191 NOG"}="1H19 NOG"; $resn_exch{"H192 NOG"}="2H19 NOG"; $resn_exch{"H193 NOG"}="3H19 NOG";
$resn_exch{"H21C NOG"}="CH21 NOG"; $resn_exch{"H17O NOG"}="OH17 NOG"; $resn_exch{" H11 NOJ"}="1H1  NOJ";
$resn_exch{" H12 NOJ"}="2H1  NOJ"; $resn_exch{" H61 NOJ"}="1H6  NOJ"; $resn_exch{" H62 NOJ"}="2H6  NOJ";
$resn_exch{"H8C1 NOK"}="1H8C NOK"; $resn_exch{"H8C2 NOK"}="2H8C NOK"; $resn_exch{"H8C3 NOK"}="3H8C NOK";
$resn_exch{"H1C1 NOK"}="1H1C NOK"; $resn_exch{"H1C2 NOK"}="2H1C NOK"; $resn_exch{"H6C1 NOK"}="1H6C NOK";
$resn_exch{"H6C2 NOK"}="2H6C NOK"; $resn_exch{"H131 NOL"}="1H13 NOL"; $resn_exch{"H132 NOL"}="2H13 NOL";
$resn_exch{"H351 NOL"}="1H35 NOL"; $resn_exch{"H352 NOL"}="2H35 NOL"; $resn_exch{"H411 NOL"}="1H41 NOL";
$resn_exch{"H412 NOL"}="2H41 NOL"; $resn_exch{"H451 NOL"}="1H45 NOL"; $resn_exch{"H452 NOL"}="2H45 NOL";
$resn_exch{"H471 NOL"}="1H47 NOL"; $resn_exch{"H472 NOL"}="2H47 NOL"; $resn_exch{"H581 NOL"}="1H58 NOL";
$resn_exch{"H582 NOL"}="2H58 NOL"; $resn_exch{"H621 NOL"}="1H62 NOL"; $resn_exch{"H622 NOL"}="2H62 NOL";
$resn_exch{"H631 NOL"}="1H63 NOL"; $resn_exch{"H632 NOL"}="2H63 NOL"; $resn_exch{"H641 NOL"}="1H64 NOL";
$resn_exch{"H642 NOL"}="2H64 NOL"; $resn_exch{"H651 NOL"}="1H65 NOL"; $resn_exch{"H652 NOL"}="2H65 NOL";
$resn_exch{"H661 NOL"}="1H66 NOL"; $resn_exch{"H662 NOL"}="2H66 NOL"; $resn_exch{"H771 NOL"}="1H77 NOL";
$resn_exch{"H772 NOL"}="2H77 NOL"; $resn_exch{"H773 NOL"}="3H77 NOL"; $resn_exch{"H831 NOL"}="1H83 NOL";
$resn_exch{"H832 NOL"}="2H83 NOL"; $resn_exch{"H833 NOL"}="3H83 NOL"; $resn_exch{"H851 NOL"}="1H85 NOL";
$resn_exch{"H852 NOL"}="2H85 NOL"; $resn_exch{"H853 NOL"}="3H85 NOL"; $resn_exch{"H871 NOL"}="1H87 NOL";
$resn_exch{"H872 NOL"}="2H87 NOL"; $resn_exch{"H873 NOL"}="3H87 NOL"; $resn_exch{"H121 NOM"}="1H12 NOM";
$resn_exch{"H122 NOM"}="2H12 NOM"; $resn_exch{"H123 NOM"}="3H12 NOM"; $resn_exch{"H151 NOM"}="1H15 NOM";
$resn_exch{"H152 NOM"}="2H15 NOM"; $resn_exch{" H21 NON"}="1H2  NON"; $resn_exch{" H22 NON"}="2H2  NON";
$resn_exch{" H31 NON"}="1H3  NON"; $resn_exch{" H32 NON"}="2H3  NON"; $resn_exch{" H41 NON"}="1H4  NON";
$resn_exch{" H42 NON"}="2H4  NON"; $resn_exch{" H51 NON"}="1H5  NON"; $resn_exch{" H52 NON"}="2H5  NON";
$resn_exch{" H61 NON"}="1H6  NON"; $resn_exch{" H62 NON"}="2H6  NON"; $resn_exch{" H71 NON"}="1H7  NON";
$resn_exch{" H72 NON"}="2H7  NON"; $resn_exch{" H81 NON"}="1H8  NON"; $resn_exch{" H82 NON"}="2H8  NON";
$resn_exch{" H91 NON"}="1H9  NON"; $resn_exch{" H92 NON"}="2H9  NON"; $resn_exch{" H93 NON"}="3H9  NON";
$resn_exch{" HM1 NON"}="1HM  NON"; $resn_exch{" HM2 NON"}="2HM  NON"; $resn_exch{" HM3 NON"}="3HM  NON";
$resn_exch{"H2A1 NOP"}="1H2A NOP"; $resn_exch{"H2A2 NOP"}="2H2A NOP"; $resn_exch{"H2A3 NOP"}="3H2A NOP";
$resn_exch{"H5A1 NOP"}="1H5A NOP"; $resn_exch{"H5A2 NOP"}="2H5A NOP"; $resn_exch{"HOP2 NOP"}="2HOP NOP";
$resn_exch{"HOP3 NOP"}="3HOP NOP"; $resn_exch{"H111 NOQ"}="1H11 NOQ"; $resn_exch{"H112 NOQ"}="2H11 NOQ";
$resn_exch{"H161 NOQ"}="1H16 NOQ"; $resn_exch{"H162 NOQ"}="2H16 NOQ"; $resn_exch{"H171 NOQ"}="1H17 NOQ";
$resn_exch{"H172 NOQ"}="2H17 NOQ"; $resn_exch{"H151 NOQ"}="1H15 NOQ"; $resn_exch{"H152 NOQ"}="2H15 NOQ";
$resn_exch{"H141 NOQ"}="1H14 NOQ"; $resn_exch{"H142 NOQ"}="2H14 NOQ"; $resn_exch{"H301 NOQ"}="1H30 NOQ";
$resn_exch{"H302 NOQ"}="2H30 NOQ"; $resn_exch{"H441 NOQ"}="1H44 NOQ"; $resn_exch{"H442 NOQ"}="2H44 NOQ";
$resn_exch{"H443 NOQ"}="3H44 NOQ"; $resn_exch{"H201 NOQ"}="1H20 NOQ"; $resn_exch{"H202 NOQ"}="2H20 NOQ";
$resn_exch{"H211 NOQ"}="1H21 NOQ"; $resn_exch{"H212 NOQ"}="2H21 NOQ"; $resn_exch{" HN1 NOR"}="1HN  NOR";
$resn_exch{" HN2 NOR"}="2HN  NOR"; $resn_exch{" HB1 NOR"}="1HB  NOR"; $resn_exch{" HB2 NOR"}="2HB  NOR";
$resn_exch{"HD11 NOR"}="1HD1 NOR"; $resn_exch{"HD12 NOR"}="2HD1 NOR"; $resn_exch{"HD21 NOR"}="1HD2 NOR";
$resn_exch{"HD22 NOR"}="2HD2 NOR"; $resn_exch{"HE11 NOR"}="1HE1 NOR"; $resn_exch{"HE12 NOR"}="2HE1 NOR";
$resn_exch{"HE21 NOR"}="1HE2 NOR"; $resn_exch{"HE22 NOR"}="2HE2 NOR"; $resn_exch{" HZ1 NOR"}="1HZ  NOR";
$resn_exch{" HZ2 NOR"}="2HZ  NOR"; $resn_exch{"HM11 NOR"}="1HM1 NOR"; $resn_exch{"HM12 NOR"}="2HM1 NOR";
$resn_exch{"HM13 NOR"}="3HM1 NOR"; $resn_exch{"HM21 NOR"}="1HM2 NOR"; $resn_exch{"HM22 NOR"}="2HM2 NOR";
$resn_exch{"HM23 NOR"}="3HM2 NOR"; $resn_exch{"H5'1 NOS"}="1H5* NOS"; $resn_exch{"H5'2 NOS"}="2H5* NOS";
$resn_exch{"HO5' NOS"}="*HO5 NOS"; $resn_exch{" H4' NOS"}=" H4* NOS"; $resn_exch{" H1' NOS"}=" H1* NOS";
$resn_exch{" H2' NOS"}=" H2* NOS"; $resn_exch{"HO2' NOS"}="*HO2 NOS"; $resn_exch{" H3' NOS"}=" H3* NOS";
$resn_exch{"HO3' NOS"}="*HO3 NOS"; $resn_exch{"H11A NOV"}="1H1  NOV"; $resn_exch{" H12 NOV"}="2H1  NOV";
$resn_exch{" H13 NOV"}="3H1  NOV"; $resn_exch{"HN11 NOV"}="1HN1 NOV"; $resn_exch{"HN12 NOV"}="2HN1 NOV";
$resn_exch{"H231 NOV"}="1H23 NOV"; $resn_exch{"H232 NOV"}="2H23 NOV"; $resn_exch{"H233 NOV"}="3H23 NOV";
$resn_exch{"H261 NOV"}="1H26 NOV"; $resn_exch{"H262 NOV"}="2H26 NOV"; $resn_exch{"H263 NOV"}="3H26 NOV";
$resn_exch{"H21A NOV"}="1H2  NOV"; $resn_exch{" H22 NOV"}="2H2  NOV"; $resn_exch{" H23 NOV"}="3H2  NOV";
$resn_exch{"H201 NOV"}="1H20 NOV"; $resn_exch{"H202 NOV"}="2H20 NOV"; $resn_exch{"H241 NOV"}="1H24 NOV";
$resn_exch{"H242 NOV"}="2H24 NOV"; $resn_exch{"H243 NOV"}="3H24 NOV"; $resn_exch{"H251 NOV"}="1H25 NOV";
$resn_exch{"H252 NOV"}="2H25 NOV"; $resn_exch{"H253 NOV"}="3H25 NOV"; $resn_exch{" H21 NOX"}="1H2  NOX";
$resn_exch{" H22 NOX"}="2H2  NOX"; $resn_exch{" H31 NOX"}="1H3  NOX"; $resn_exch{" H32 NOX"}="2H3  NOX";
$resn_exch{" H41 NOX"}="1H4  NOX"; $resn_exch{" H42 NOX"}="2H4  NOX"; $resn_exch{" H51 NOX"}="1H5  NOX";
$resn_exch{" H52 NOX"}="2H5  NOX"; $resn_exch{" H61 NOX"}="1H6  NOX"; $resn_exch{" H62 NOX"}="2H6  NOX";
$resn_exch{" H81 NOX"}="1H8  NOX"; $resn_exch{" H82 NOX"}="2H8  NOX"; $resn_exch{" H91 NOX"}="1H9  NOX";
$resn_exch{" H92 NOX"}="2H9  NOX"; $resn_exch{"H191 NOX"}="1H19 NOX"; $resn_exch{"H192 NOX"}="2H19 NOX";
$resn_exch{"H201 NOX"}="1H20 NOX"; $resn_exch{"H202 NOX"}="2H20 NOX"; $resn_exch{"H211 NOX"}="1H21 NOX";
$resn_exch{"H212 NOX"}="2H21 NOX"; $resn_exch{"H7C1 NOY"}="1H7C NOY"; $resn_exch{"H7C2 NOY"}="2H7C NOY";
$resn_exch{"H6C1 NOY"}="1H6C NOY"; $resn_exch{"H6C2 NOY"}="2H6C NOY"; $resn_exch{"H6C3 NOY"}="3H6C NOY";
$resn_exch{" H12 NOY"}="2H1  NOY"; $resn_exch{" H22 NOY"}="2H2  NOY"; $resn_exch{"H7C1 NOZ"}="1H7C NOZ";
$resn_exch{"H7C2 NOZ"}="2H7C NOZ"; $resn_exch{"H6C1 NOZ"}="1H6C NOZ"; $resn_exch{"H6C2 NOZ"}="2H6C NOZ";
$resn_exch{"HO4'  NP"}="'HO4  NP"; $resn_exch{" H11  NP"}="1H1   NP"; $resn_exch{" H12  NP"}="2H1   NP";
$resn_exch{" H41  NP"}="1H4   NP"; $resn_exch{" H42  NP"}="2H4   NP"; $resn_exch{" H51  NP"}="1H5   NP";
$resn_exch{" H52  NP"}="2H5   NP"; $resn_exch{" H61  NP"}="1H6   NP"; $resn_exch{" H62  NP"}="2H6   NP";
$resn_exch{" H71  NP"}="1H7   NP"; $resn_exch{" H72  NP"}="2H7   NP"; $resn_exch{" H81  NP"}="1H8   NP";
$resn_exch{" H82  NP"}="2H8   NP"; $resn_exch{"HNZ1 NP1"}="1HNZ NP1"; $resn_exch{"HNZ2 NP1"}="2HNZ NP1";
$resn_exch{" HA1 NP1"}="1HA  NP1"; $resn_exch{" HA2 NP1"}="2HA  NP1"; $resn_exch{" HB1 NP1"}="1HB  NP1";
$resn_exch{" HB2 NP1"}="2HB  NP1"; $resn_exch{" HG1 NP1"}="1HG  NP1"; $resn_exch{" HG2 NP1"}="2HG  NP1";
$resn_exch{" HM1 NP1"}="1HM  NP1"; $resn_exch{" HM2 NP1"}="2HM  NP1"; $resn_exch{" HM3 NP1"}="3HM  NP1";
$resn_exch{"HNZ1 NP2"}="1HNZ NP2"; $resn_exch{"HNZ2 NP2"}="2HNZ NP2"; $resn_exch{" HA1 NP2"}="1HA  NP2";
$resn_exch{" HA2 NP2"}="2HA  NP2"; $resn_exch{" HB1 NP2"}="1HB  NP2"; $resn_exch{" HB2 NP2"}="2HB  NP2";
$resn_exch{" HG1 NP2"}="1HG  NP2"; $resn_exch{" HG2 NP2"}="2HG  NP2"; $resn_exch{"HOP2 NP3"}="2HOP NP3";
$resn_exch{"HOP3 NP3"}="3HOP NP3"; $resn_exch{"H5'1 NP3"}="1H5* NP3"; $resn_exch{"H5'2 NP3"}="2H5* NP3";
$resn_exch{" H4' NP3"}=" H4* NP3"; $resn_exch{" H1' NP3"}=" H1* NP3"; $resn_exch{"H2'1 NP3"}="1H2* NP3";
$resn_exch{"H2'2 NP3"}="2H2* NP3"; $resn_exch{" H3' NP3"}=" H3* NP3"; $resn_exch{"H8C1 NP4"}="1H8C NP4";
$resn_exch{"H8C2 NP4"}="2H8C NP4"; $resn_exch{"H101 NP4"}="1H10 NP4"; $resn_exch{"H102 NP4"}="2H10 NP4";
$resn_exch{"H111 NP4"}="1H11 NP4"; $resn_exch{"H112 NP4"}="2H11 NP4"; $resn_exch{"H121 NP4"}="1H12 NP4";
$resn_exch{"H122 NP4"}="2H12 NP4"; $resn_exch{"H151 NP4"}="1H15 NP4"; $resn_exch{"H152 NP4"}="2H15 NP4";
$resn_exch{"H161 NP4"}="1H16 NP4"; $resn_exch{"H162 NP4"}="2H16 NP4"; $resn_exch{"H8C1 NP5"}="1H8C NP5";
$resn_exch{"H8C2 NP5"}="2H8C NP5"; $resn_exch{"H101 NP5"}="1H10 NP5"; $resn_exch{"H102 NP5"}="2H10 NP5";
$resn_exch{"H111 NP5"}="1H11 NP5"; $resn_exch{"H112 NP5"}="2H11 NP5"; $resn_exch{"H121 NP5"}="1H12 NP5";
$resn_exch{"H122 NP5"}="2H12 NP5"; $resn_exch{"H151 NP5"}="1H15 NP5"; $resn_exch{"H152 NP5"}="2H15 NP5";
$resn_exch{"H161 NP5"}="1H16 NP5"; $resn_exch{"H162 NP5"}="2H16 NP5"; $resn_exch{" H71 NPA"}="1H7  NPA";
$resn_exch{" H72 NPA"}="2H7  NPA"; $resn_exch{"HO11 NPB"}="1HO1 NPB"; $resn_exch{"HO21 NPB"}="1HO2 NPB";
$resn_exch{"HC21 NPB"}="1HC2 NPB"; $resn_exch{"HC41 NPB"}="1HC4 NPB"; $resn_exch{"HC51 NPB"}="1HC5 NPB";
$resn_exch{"HC61 NPB"}="1HC6 NPB"; $resn_exch{" H71 NPC"}="1H7  NPC"; $resn_exch{" H72 NPC"}="2H7  NPC";
$resn_exch{"H101 NPC"}="1H10 NPC"; $resn_exch{"H102 NPC"}="2H10 NPC"; $resn_exch{"H111 NPC"}="1H11 NPC";
$resn_exch{"H112 NPC"}="2H11 NPC"; $resn_exch{"H121 NPC"}="1H12 NPC"; $resn_exch{"H122 NPC"}="2H12 NPC";
$resn_exch{"H131 NPC"}="1H13 NPC"; $resn_exch{"H132 NPC"}="2H13 NPC"; $resn_exch{"H141 NPC"}="1H14 NPC";
$resn_exch{"H142 NPC"}="2H14 NPC"; $resn_exch{"HCN1 NPD"}="1HCN NPD"; $resn_exch{"HCN2 NPD"}="2HCN NPD";
$resn_exch{"HCN3 NPD"}="3HCN NPD"; $resn_exch{" HB1 NPD"}="1HB  NPD"; $resn_exch{" HB2 NPD"}="2HB  NPD";
$resn_exch{"HD11 NPD"}="1HD1 NPD"; $resn_exch{"HE11 NPD"}="1HE1 NPD"; $resn_exch{"HZ11 NPD"}="1HZ1 NPD";
$resn_exch{"HZ21 NPD"}="1HZ2 NPD"; $resn_exch{"HH21 NPD"}="1HH2 NPD"; $resn_exch{"HZ31 NPD"}="1HZ3 NPD";
$resn_exch{"HE31 NPD"}="1HE3 NPD"; $resn_exch{" H71 NPE"}="1H7  NPE"; $resn_exch{" H72 NPE"}="2H7  NPE";
$resn_exch{" H81 NPE"}="1H8  NPE"; $resn_exch{" H82 NPE"}="2H8  NPE"; $resn_exch{" H91 NPE"}="1H9  NPE";
$resn_exch{" H92 NPE"}="2H9  NPE"; $resn_exch{"H101 NPE"}="1H10 NPE"; $resn_exch{"H102 NPE"}="2H10 NPE";
$resn_exch{"HO'2 NPF"}="2HO' NPF"; $resn_exch{"HO'3 NPF"}="3HO' NPF"; $resn_exch{"HO'4 NPF"}="4HO' NPF";
$resn_exch{"H6'1 NPF"}="1H6' NPF"; $resn_exch{"H6'2 NPF"}="2H6' NPF"; $resn_exch{"H6'3 NPF"}="3H6' NPF";
$resn_exch{" H51 NPG"}="1H5  NPG"; $resn_exch{" H52 NPG"}="2H5  NPG"; $resn_exch{" H41 NPG"}="1H4  NPG";
$resn_exch{" H42 NPG"}="2H4  NPG"; $resn_exch{" H   NPH"}="1HN  NPH"; $resn_exch{" H2  NPH"}="2HN  NPH";
$resn_exch{" HB2 NPH"}="1HB  NPH"; $resn_exch{" HB3 NPH"}="2HB  NPH"; $resn_exch{" HD2 NPH"}="1HD  NPH";
$resn_exch{" HD3 NPH"}="2HD  NPH"; $resn_exch{" HZ  NPH"}=" HNZ NPH"; $resn_exch{" HC2 NPH"}=" H2  NPH";
$resn_exch{" H31 NPI"}="1H3  NPI"; $resn_exch{" H32 NPI"}="2H3  NPI"; $resn_exch{" H41 NPI"}="1H4  NPI";
$resn_exch{" H42 NPI"}="2H4  NPI"; $resn_exch{" H51 NPI"}="1H5  NPI"; $resn_exch{" H52 NPI"}="2H5  NPI";
$resn_exch{" H61 NPI"}="1H6  NPI"; $resn_exch{" H62 NPI"}="2H6  NPI"; $resn_exch{"HN21 NPI"}="1HN2 NPI";
$resn_exch{"HN22 NPI"}="2HN2 NPI"; $resn_exch{"H6A1 NPJ"}="1H6A NPJ"; $resn_exch{"H6A2 NPJ"}="2H6A NPJ";
$resn_exch{"H6B1 NPJ"}="1H6B NPJ"; $resn_exch{"H6B2 NPJ"}="2H6B NPJ"; $resn_exch{" H11 NPL"}="1H1  NPL";
$resn_exch{" H12 NPL"}="2H1  NPL"; $resn_exch{" H13 NPL"}="3H1  NPL"; $resn_exch{" H21 NPL"}="1H2  NPL";
$resn_exch{" H22 NPL"}="2H2  NPL"; $resn_exch{" H23 NPL"}="3H2  NPL"; $resn_exch{" H41 NPL"}="1H4  NPL";
$resn_exch{" H42 NPL"}="2H4  NPL"; $resn_exch{"HN41 NPL"}="1HN4 NPL"; $resn_exch{"HN42 NPL"}="2HN4 NPL";
$resn_exch{" H51 NPL"}="1H5  NPL"; $resn_exch{" H52 NPL"}="2H5  NPL"; $resn_exch{"HOP2 NPL"}="2HOP NPL";
$resn_exch{"HOP3 NPL"}="3HOP NPL"; $resn_exch{"H1M1 NPM"}="1H1M NPM"; $resn_exch{"H1M2 NPM"}="2H1M NPM";
$resn_exch{" H11 NPN"}="1H1  NPN"; $resn_exch{" H12 NPN"}="2H1  NPN"; $resn_exch{" H21 NPN"}="1H2  NPN";
$resn_exch{" H22 NPN"}="2H2  NPN"; $resn_exch{" H31 NPN"}="1H3  NPN"; $resn_exch{" H32 NPN"}="2H3  NPN";
$resn_exch{" H33 NPN"}="3H3  NPN"; $resn_exch{"HM11 NPP"}="1HM1 NPP"; $resn_exch{"HM12 NPP"}="2HM1 NPP";
$resn_exch{"HM13 NPP"}="3HM1 NPP"; $resn_exch{"HM21 NPP"}="1HM2 NPP"; $resn_exch{"HM22 NPP"}="2HM2 NPP";
$resn_exch{"HM23 NPP"}="3HM2 NPP"; $resn_exch{" H31 NPP"}="1H3  NPP"; $resn_exch{" H32 NPP"}="2H3  NPP";
$resn_exch{" H51 NPP"}="1H5  NPP"; $resn_exch{" H52 NPP"}="2H5  NPP"; $resn_exch{"HM31 NPP"}="1HM3 NPP";
$resn_exch{"HM32 NPP"}="2HM3 NPP"; $resn_exch{"HM33 NPP"}="3HM3 NPP"; $resn_exch{"HM41 NPP"}="1HM4 NPP";
$resn_exch{"HM42 NPP"}="2HM4 NPP"; $resn_exch{"HM43 NPP"}="3HM4 NPP"; $resn_exch{"H131 NPP"}="1H13 NPP";
$resn_exch{"H132 NPP"}="2H13 NPP"; $resn_exch{"H141 NPP"}="1H14 NPP"; $resn_exch{"H142 NPP"}="2H14 NPP";
$resn_exch{"HN61 NPP"}="1HN6 NPP"; $resn_exch{"HN62 NPP"}="2HN6 NPP"; $resn_exch{"HN11 NPR"}="1HN1 NPR";
$resn_exch{"HN12 NPR"}="2HN1 NPR"; $resn_exch{" H91 NPR"}="1H9  NPR"; $resn_exch{" H92 NPR"}="2H9  NPR";
$resn_exch{"H171 NPR"}="1H17 NPR"; $resn_exch{"H172 NPR"}="2H17 NPR"; $resn_exch{"H101 NPS"}="1H10 NPS";
$resn_exch{"H102 NPS"}="2H10 NPS"; $resn_exch{"H103 NPS"}="3H10 NPS"; $resn_exch{"H121 NPS"}="1H12 NPS";
$resn_exch{"H122 NPS"}="2H12 NPS"; $resn_exch{"H123 NPS"}="3H12 NPS"; $resn_exch{" HN1 NPT"}="1HN  NPT";
$resn_exch{" HN2 NPT"}="2HN  NPT"; $resn_exch{" H11 NPT"}="1H1  NPT"; $resn_exch{" H12 NPT"}="2H1  NPT";
$resn_exch{" H31 NPT"}="1H3  NPT"; $resn_exch{" H32 NPT"}="2H3  NPT"; $resn_exch{" H33 NPT"}="3H3  NPT";
$resn_exch{" H41 NPT"}="1H4  NPT"; $resn_exch{" H42 NPT"}="2H4  NPT"; $resn_exch{" H43 NPT"}="3H4  NPT";
$resn_exch{" H51 NPT"}="1H5  NPT"; $resn_exch{" H52 NPT"}="2H5  NPT"; $resn_exch{" H53 NPT"}="3H5  NPT";
$resn_exch{" H11 NPY"}="1H1  NPY"; $resn_exch{"HAA1 NQG"}="1HAA NQG"; $resn_exch{"HAA2 NQG"}="2HAA NQG";
$resn_exch{"HAA3 NQG"}="3HAA NQG"; $resn_exch{"HAS1 NQG"}="1HAS NQG"; $resn_exch{"HAS2 NQG"}="2HAS NQG";
$resn_exch{"HBT1 NQG"}="1HBT NQG"; $resn_exch{"HBT2 NQG"}="2HBT NQG"; $resn_exch{" HB1 NQG"}="1HB  NQG";
$resn_exch{" HB2 NQG"}="2HB  NQG"; $resn_exch{"HAT1 NQG"}="1HAT NQG"; $resn_exch{"HAT2 NQG"}="2HAT NQG";
$resn_exch{"HAX1 NQG"}="1HAX NQG"; $resn_exch{"HAX2 NQG"}="2HAX NQG"; $resn_exch{"HAU1 NQG"}="1HAU NQG";
$resn_exch{"HAU2 NQG"}="2HAU NQG"; $resn_exch{"HAV1 NQG"}="1HAV NQG"; $resn_exch{"HAV2 NQG"}="2HAV NQG";
$resn_exch{"HAE1 NQG"}="1HAE NQG"; $resn_exch{"HAE2 NQG"}="2HAE NQG"; $resn_exch{"HAF1 NQG"}="1HAF NQG";
$resn_exch{"HAF2 NQG"}="2HAF NQG"; $resn_exch{"HAY1 NQG"}="1HAY NQG"; $resn_exch{"HAY2 NQG"}="2HAY NQG";
$resn_exch{"HAB1 NQI"}="1HAB NQI"; $resn_exch{"HAB2 NQI"}="2HAB NQI"; $resn_exch{"HAB3 NQI"}="3HAB NQI";
$resn_exch{"HAS1 NQI"}="1HAS NQI"; $resn_exch{"HAS2 NQI"}="2HAS NQI"; $resn_exch{"HBS1 NQI"}="1HBS NQI";
$resn_exch{"HBS2 NQI"}="2HBS NQI"; $resn_exch{" HB1 NQI"}="1HB  NQI"; $resn_exch{" HB2 NQI"}="2HB  NQI";
$resn_exch{"HAA1 NQI"}="1HAA NQI"; $resn_exch{"HAA2 NQI"}="2HAA NQI"; $resn_exch{"HAW1 NQI"}="1HAW NQI";
$resn_exch{"HAW2 NQI"}="2HAW NQI"; $resn_exch{"HAT1 NQI"}="1HAT NQI"; $resn_exch{"HAT2 NQI"}="2HAT NQI";
$resn_exch{"HAU1 NQI"}="1HAU NQI"; $resn_exch{"HAU2 NQI"}="2HAU NQI"; $resn_exch{"HAD1 NQI"}="1HAD NQI";
$resn_exch{"HAD2 NQI"}="2HAD NQI"; $resn_exch{"HAF1 NQI"}="1HAF NQI"; $resn_exch{"HAF2 NQI"}="2HAF NQI";
$resn_exch{"HAX1 NQI"}="1HAX NQI"; $resn_exch{"HAX2 NQI"}="2HAX NQI"; $resn_exch{"H241 NR2"}="1H24 NR2";
$resn_exch{"H242 NR2"}="2H24 NR2"; $resn_exch{"H211 NR2"}="1H21 NR2"; $resn_exch{"H212 NR2"}="2H21 NR2";
$resn_exch{"H213 NR2"}="3H21 NR2"; $resn_exch{"H1C1 NRB"}="1H1C NRB"; $resn_exch{"H1C2 NRB"}="2H1C NRB";
$resn_exch{"H1C3 NRB"}="3H1C NRB"; $resn_exch{"H2C1 NRB"}="1H2C NRB"; $resn_exch{"H2C2 NRB"}="2H2C NRB";
$resn_exch{"H6C1 NRB"}="1H6C NRB"; $resn_exch{"H6C2 NRB"}="2H6C NRB"; $resn_exch{"H6C3 NRB"}="3H6C NRB";
$resn_exch{"H9C1 NRB"}="1H9C NRB"; $resn_exch{"H9C2 NRB"}="2H9C NRB"; $resn_exch{"H9C3 NRB"}="3H9C NRB";
$resn_exch{"H131 NRB"}="1H13 NRB"; $resn_exch{"H132 NRB"}="2H13 NRB"; $resn_exch{"H133 NRB"}="3H13 NRB";
$resn_exch{"H141 NRB"}="1H14 NRB"; $resn_exch{"H142 NRB"}="2H14 NRB"; $resn_exch{"H161 NRB"}="1H16 NRB";
$resn_exch{"H162 NRB"}="2H16 NRB"; $resn_exch{"H163 NRB"}="3H16 NRB"; $resn_exch{"H171 NRB"}="1H17 NRB";
$resn_exch{"H172 NRB"}="2H17 NRB"; $resn_exch{"H173 NRB"}="3H17 NRB"; $resn_exch{"H201 NRB"}="1H20 NRB";
$resn_exch{"H202 NRB"}="2H20 NRB"; $resn_exch{"H203 NRB"}="3H20 NRB"; $resn_exch{"H211 NRB"}="1H21 NRB";
$resn_exch{"H212 NRB"}="2H21 NRB"; $resn_exch{"H231 NRB"}="1H23 NRB"; $resn_exch{"H232 NRB"}="2H23 NRB";
$resn_exch{"H233 NRB"}="3H23 NRB"; $resn_exch{"H281 NRB"}="1H28 NRB"; $resn_exch{"H282 NRB"}="2H28 NRB";
$resn_exch{"H283 NRB"}="3H28 NRB"; $resn_exch{" HN1 NRG"}="1HN  NRG"; $resn_exch{" HN2 NRG"}="2HN  NRG";
$resn_exch{"HCB1 NRG"}="1HCB NRG"; $resn_exch{"HCB2 NRG"}="2HCB NRG"; $resn_exch{"HCG1 NRG"}="1HCG NRG";
$resn_exch{"HCG2 NRG"}="2HCG NRG"; $resn_exch{"HCD1 NRG"}="1HCD NRG"; $resn_exch{"HCD2 NRG"}="2HCD NRG";
$resn_exch{"HOP2 NRI"}="2HOP NRI"; $resn_exch{"HOP3 NRI"}="3HOP NRI"; $resn_exch{"H2'1 NRI"}="1H2* NRI";
$resn_exch{"H2'2 NRI"}="2H2* NRI"; $resn_exch{"H5'1 NRI"}="1H5* NRI"; $resn_exch{"H5'2 NRI"}="2H5* NRI";
$resn_exch{" H4' NRI"}=" H4* NRI"; $resn_exch{" H3' NRI"}=" H3* NRI"; $resn_exch{"H6'1 NRI"}="1H6* NRI";
$resn_exch{"H6'2 NRI"}="2H6* NRI"; $resn_exch{"HN'1 NRI"}="1HN* NRI"; $resn_exch{"HO'3 NRI"}="3HO* NRI";
$resn_exch{" H2  NRQ"}=" HN2 NRQ"; $resn_exch{"HE1A NRQ"}="1HE  NRQ"; $resn_exch{"HE2A NRQ"}="2HE  NRQ";
$resn_exch{" HE3 NRQ"}="3HE  NRQ"; $resn_exch{"HG11 NRQ"}="1HG1 NRQ"; $resn_exch{"HG12 NRQ"}="2HG1 NRQ";
$resn_exch{"HB11 NRQ"}="1HB1 NRQ"; $resn_exch{"HB12 NRQ"}="2HB1 NRQ"; $resn_exch{"HA31 NRQ"}="1HA3 NRQ";
$resn_exch{"HA32 NRQ"}="2HA3 NRQ"; $resn_exch{" H4' NRT"}=" H4* NRT"; $resn_exch{"H3'1 NRT"}="1H3* NRT";
$resn_exch{"H3'2 NRT"}="2H3* NRT"; $resn_exch{"H2'1 NRT"}="1H2* NRT"; $resn_exch{"H2'2 NRT"}="2H2* NRT";
$resn_exch{"H2N1 NRT"}="1H2N NRT"; $resn_exch{"H2N2 NRT"}="2H2N NRT"; $resn_exch{"H601 NRT"}="1H60 NRT";
$resn_exch{"H602 NRT"}="2H60 NRT"; $resn_exch{"H141 NRT"}="1H14 NRT"; $resn_exch{"H142 NRT"}="2H14 NRT";
$resn_exch{"HN11 NRU"}="1HN1 NRU"; $resn_exch{"HN12 NRU"}="2HN1 NRU"; $resn_exch{"HN41 NRU"}="1HN4 NRU";
$resn_exch{"HN42 NRU"}="2HN4 NRU"; $resn_exch{"HN51 NRU"}="1HN5 NRU"; $resn_exch{"HN52 NRU"}="2HN5 NRU";
$resn_exch{"HN21 NRU"}="1HN2 NRU"; $resn_exch{"HN22 NRU"}="2HN2 NRU"; $resn_exch{"HN31 NRU"}="1HN3 NRU";
$resn_exch{"HN32 NRU"}="2HN3 NRU"; $resn_exch{"HN61 NRU"}="1HN6 NRU"; $resn_exch{"HN62 NRU"}="2HN6 NRU";
$resn_exch{"HM11 NS1"}="1HM1 NS1"; $resn_exch{"HM12 NS1"}="2HM1 NS1"; $resn_exch{"HM13 NS1"}="3HM1 NS1";
$resn_exch{"HM21 NS1"}="1HM2 NS1"; $resn_exch{"HM22 NS1"}="2HM2 NS1"; $resn_exch{"HM23 NS1"}="3HM2 NS1";
$resn_exch{" H21 NS1"}="1H2  NS1"; $resn_exch{" H22 NS1"}="2H2  NS1"; $resn_exch{" H31 NS1"}="1H3  NS1";
$resn_exch{" H32 NS1"}="2H3  NS1"; $resn_exch{" H41 NS1"}="1H4  NS1"; $resn_exch{" H42 NS1"}="2H4  NS1";
$resn_exch{" H61 NS1"}="1H6  NS1"; $resn_exch{" H62 NS1"}="2H6  NS1"; $resn_exch{" H63 NS1"}="3H6  NS1";
$resn_exch{" H81 NS1"}="1H8  NS1"; $resn_exch{" H82 NS1"}="2H8  NS1"; $resn_exch{" H91 NS1"}="1H9  NS1";
$resn_exch{" H92 NS1"}="2H9  NS1"; $resn_exch{"H111 NS1"}="1H11 NS1"; $resn_exch{"H112 NS1"}="2H11 NS1";
$resn_exch{"H113 NS1"}="3H11 NS1"; $resn_exch{"H161 NS1"}="1H16 NS1"; $resn_exch{"H162 NS1"}="2H16 NS1";
$resn_exch{"H163 NS1"}="3H16 NS1"; $resn_exch{"H221 NS1"}="1H22 NS1"; $resn_exch{"H222 NS1"}="2H22 NS1";
$resn_exch{"H223 NS1"}="3H22 NS1"; $resn_exch{"H271 NS1"}="1H27 NS1"; $resn_exch{"H272 NS1"}="2H27 NS1";
$resn_exch{"H273 NS1"}="3H27 NS1"; $resn_exch{"H321 NS1"}="1H32 NS1"; $resn_exch{"H322 NS1"}="2H32 NS1";
$resn_exch{"H323 NS1"}="3H32 NS1"; $resn_exch{"H331 NS1"}="1H33 NS1"; $resn_exch{"H332 NS1"}="2H33 NS1";
$resn_exch{"H341 NS1"}="1H34 NS1"; $resn_exch{"H342 NS1"}="2H34 NS1"; $resn_exch{"H351 NS1"}="1H35 NS1";
$resn_exch{"H352 NS1"}="2H35 NS1"; $resn_exch{"HM31 NS1"}="1HM3 NS1"; $resn_exch{"HM32 NS1"}="2HM3 NS1";
$resn_exch{"HM33 NS1"}="3HM3 NS1"; $resn_exch{"HM41 NS1"}="1HM4 NS1"; $resn_exch{"HM42 NS1"}="2HM4 NS1";
$resn_exch{"HM43 NS1"}="3HM4 NS1"; $resn_exch{"HM11 NS5"}="1HM1 NS5"; $resn_exch{"HM12 NS5"}="2HM1 NS5";
$resn_exch{"HM13 NS5"}="3HM1 NS5"; $resn_exch{"HM21 NS5"}="1HM2 NS5"; $resn_exch{"HM22 NS5"}="2HM2 NS5";
$resn_exch{"HM23 NS5"}="3HM2 NS5"; $resn_exch{" H21 NS5"}="1H2  NS5"; $resn_exch{" H22 NS5"}="2H2  NS5";
$resn_exch{" H31 NS5"}="1H3  NS5"; $resn_exch{" H32 NS5"}="2H3  NS5"; $resn_exch{" H41 NS5"}="1H4  NS5";
$resn_exch{" H42 NS5"}="2H4  NS5"; $resn_exch{" H61 NS5"}="1H6  NS5"; $resn_exch{" H62 NS5"}="2H6  NS5";
$resn_exch{" H63 NS5"}="3H6  NS5"; $resn_exch{" H81 NS5"}="1H8  NS5"; $resn_exch{" H82 NS5"}="2H8  NS5";
$resn_exch{" H91 NS5"}="1H9  NS5"; $resn_exch{" H92 NS5"}="2H9  NS5"; $resn_exch{"H111 NS5"}="1H11 NS5";
$resn_exch{"H112 NS5"}="2H11 NS5"; $resn_exch{"H113 NS5"}="3H11 NS5"; $resn_exch{"H161 NS5"}="1H16 NS5";
$resn_exch{"H162 NS5"}="2H16 NS5"; $resn_exch{"H163 NS5"}="3H16 NS5"; $resn_exch{"H221 NS5"}="1H22 NS5";
$resn_exch{"H222 NS5"}="2H22 NS5"; $resn_exch{"H223 NS5"}="3H22 NS5"; $resn_exch{"H271 NS5"}="1H27 NS5";
$resn_exch{"H272 NS5"}="2H27 NS5"; $resn_exch{"H273 NS5"}="3H27 NS5"; $resn_exch{"H321 NS5"}="1H32 NS5";
$resn_exch{"H322 NS5"}="2H32 NS5"; $resn_exch{"H323 NS5"}="3H32 NS5"; $resn_exch{"H331 NS5"}="1H33 NS5";
$resn_exch{"H332 NS5"}="2H33 NS5"; $resn_exch{"H341 NS5"}="1H34 NS5"; $resn_exch{"H342 NS5"}="2H34 NS5";
$resn_exch{"H351 NS5"}="1H35 NS5"; $resn_exch{"H352 NS5"}="2H35 NS5"; $resn_exch{"HM31 NS5"}="1HM3 NS5";
$resn_exch{"HM32 NS5"}="2HM3 NS5"; $resn_exch{"HM33 NS5"}="3HM3 NS5"; $resn_exch{"HM41 NS5"}="1HM4 NS5";
$resn_exch{"HM42 NS5"}="2HM4 NS5"; $resn_exch{"HM43 NS5"}="3HM4 NS5"; $resn_exch{" H61 NSB"}="1H6  NSB";
$resn_exch{" H62 NSB"}="2H6  NSB"; $resn_exch{" H41 NSB"}="1H4  NSB"; $resn_exch{" H42 NSB"}="2H4  NSB";
$resn_exch{" H21 NSB"}="1H2  NSB"; $resn_exch{" H22 NSB"}="2H2  NSB"; $resn_exch{"H201 NSB"}="1H20 NSB";
$resn_exch{"H202 NSB"}="2H20 NSB"; $resn_exch{"H203 NSB"}="3H20 NSB"; $resn_exch{"H181 NSB"}="1H18 NSB";
$resn_exch{"H182 NSB"}="2H18 NSB"; $resn_exch{"H183 NSB"}="3H18 NSB"; $resn_exch{"H161 NSB"}="1H16 NSB";
$resn_exch{"H162 NSB"}="2H16 NSB"; $resn_exch{"H163 NSB"}="3H16 NSB"; $resn_exch{"HN11 NSC"}="1HN1 NSC";
$resn_exch{"HN12 NSC"}="2HN1 NSC"; $resn_exch{" H81 NSC"}="1H8  NSC"; $resn_exch{" H82 NSC"}="2H8  NSC";
$resn_exch{" H83 NSC"}="3H8  NSC"; $resn_exch{"H251 NSC"}="1H25 NSC"; $resn_exch{"H252 NSC"}="2H25 NSC";
$resn_exch{"H201 NSC"}="1H20 NSC"; $resn_exch{"H202 NSC"}="2H20 NSC"; $resn_exch{"H203 NSC"}="3H20 NSC";
$resn_exch{"H121 NSI"}="1H12 NSI"; $resn_exch{"H122 NSI"}="2H12 NSI"; $resn_exch{"H281 NSI"}="1H28 NSI";
$resn_exch{"H282 NSI"}="2H28 NSI"; $resn_exch{"H283 NSI"}="3H28 NSI"; $resn_exch{" H21 NSK"}="1H2  NSK";
$resn_exch{" H22 NSK"}="2H2  NSK"; $resn_exch{" H31 NSK"}="1H3  NSK"; $resn_exch{" H32 NSK"}="2H3  NSK";
$resn_exch{" H71 NSK"}="1H7  NSK"; $resn_exch{" H72 NSK"}="2H7  NSK"; $resn_exch{" H81 NSK"}="1H8  NSK";
$resn_exch{" H82 NSK"}="2H8  NSK"; $resn_exch{" H91 NSK"}="1H9  NSK"; $resn_exch{" H92 NSK"}="2H9  NSK";
$resn_exch{"H101 NSK"}="1H10 NSK"; $resn_exch{"H102 NSK"}="2H10 NSK"; $resn_exch{"HN21 NSK"}="1HN2 NSK";
$resn_exch{"HN22 NSK"}="2HN2 NSK"; $resn_exch{" H11 NSM"}="1H1  NSM"; $resn_exch{" H12 NSM"}="2H1  NSM";
$resn_exch{" H13 NSM"}="3H1  NSM"; $resn_exch{" H11 NSN"}="1H1  NSN"; $resn_exch{" H12 NSN"}="2H1  NSN";
$resn_exch{" H13 NSN"}="3H1  NSN"; $resn_exch{" H51 NSN"}="1H5  NSN"; $resn_exch{" H52 NSN"}="2H5  NSN";
$resn_exch{"HN21 NSN"}="1HN2 NSN"; $resn_exch{"HN22 NSN"}="2HN2 NSN"; $resn_exch{"HN31 NSN"}="1HN3 NSN";
$resn_exch{"HN32 NSN"}="2HN3 NSN"; $resn_exch{" HN1 NSP"}="1HN  NSP"; $resn_exch{" HN2 NSP"}="2HN  NSP";
$resn_exch{"HCA1 NSP"}="1HCA NSP"; $resn_exch{"HCA2 NSP"}="2HCA NSP"; $resn_exch{"HQ21 NSP"}="1HQ2 NSP";
$resn_exch{"HQ22 NSP"}="2HQ2 NSP"; $resn_exch{"HS11 NSP"}="1HS1 NSP"; $resn_exch{"HS12 NSP"}="2HS1 NSP";
$resn_exch{"HS13 NSP"}="3HS1 NSP"; $resn_exch{" HN1 NSS"}="1HN  NSS"; $resn_exch{" HN2 NSS"}="2HN  NSS";
$resn_exch{" HN3 NSS"}="3HN  NSS"; $resn_exch{" HB1 NSS"}="1HB  NSS"; $resn_exch{" HB2 NSS"}="2HB  NSS";
$resn_exch{"HD21 NSS"}="1HD2 NSS"; $resn_exch{"HD22 NSS"}="2HD2 NSS"; $resn_exch{"H5'1 NSS"}="1H5* NSS";
$resn_exch{"H5'2 NSS"}="2H5* NSS"; $resn_exch{" H4' NSS"}=" H4* NSS"; $resn_exch{" H3' NSS"}=" H3* NSS";
$resn_exch{" H2' NSS"}=" H2* NSS"; $resn_exch{" H1' NSS"}=" H1* NSS"; $resn_exch{"HN61 NSS"}="1HN6 NSS";
$resn_exch{"HN62 NSS"}="2HN6 NSS"; $resn_exch{"HN21  NT"}="1HN2  NT"; $resn_exch{"HN22  NT"}="2HN2  NT";
$resn_exch{" H21  NT"}="1H2   NT"; $resn_exch{" H22  NT"}="2H2   NT"; $resn_exch{" H81  NT"}="1H8   NT";
$resn_exch{" H82  NT"}="2H8   NT"; $resn_exch{" H83  NT"}="3H8   NT"; $resn_exch{"H141  NT"}="1H14  NT";
$resn_exch{"H142  NT"}="2H14  NT"; $resn_exch{"H143  NT"}="3H14  NT"; $resn_exch{"H161  NT"}="1H16  NT";
$resn_exch{"H162  NT"}="2H16  NT"; $resn_exch{"H171  NT"}="1H17  NT"; $resn_exch{"H172  NT"}="2H17  NT";
$resn_exch{"HN01  NT"}="1HN0  NT"; $resn_exch{"HN02  NT"}="2HN0  NT"; $resn_exch{"H101 NT1"}="1H10 NT1";
$resn_exch{"H102 NT1"}="2H10 NT1"; $resn_exch{"H131 NT1"}="1H13 NT1"; $resn_exch{"H132 NT1"}="2H13 NT1";
$resn_exch{"H151 NT1"}="1H15 NT1"; $resn_exch{"H152 NT1"}="2H15 NT1"; $resn_exch{"H153 NT1"}="3H15 NT1";
$resn_exch{"H181 NT1"}="1H18 NT1"; $resn_exch{"H182 NT1"}="2H18 NT1"; $resn_exch{"H183 NT1"}="3H18 NT1";
$resn_exch{"H1B1 NT1"}="1H1B NT1"; $resn_exch{"H1B2 NT1"}="2H1B NT1"; $resn_exch{"H1B3 NT1"}="3H1B NT1";
$resn_exch{"H231 NT1"}="1H23 NT1"; $resn_exch{"H232 NT1"}="2H23 NT1"; $resn_exch{"H281 NT1"}="1H28 NT1";
$resn_exch{"H282 NT1"}="2H28 NT1"; $resn_exch{"H283 NT1"}="3H28 NT1"; $resn_exch{"H2B1 NT1"}="1H2B NT1";
$resn_exch{"H2B2 NT1"}="2H2B NT1"; $resn_exch{"H2B3 NT1"}="3H2B NT1"; $resn_exch{"H251 NT1"}="1H25 NT1";
$resn_exch{"H252 NT1"}="2H25 NT1"; $resn_exch{"H253 NT1"}="3H25 NT1"; $resn_exch{"H351 NT1"}="1H35 NT1";
$resn_exch{"H352 NT1"}="2H35 NT1"; $resn_exch{"H3D1 NT1"}="1H3D NT1"; $resn_exch{"H3D2 NT1"}="2H3D NT1";
$resn_exch{"H3D3 NT1"}="3H3D NT1"; $resn_exch{"H3G1 NT1"}="1H3G NT1"; $resn_exch{"H3G2 NT1"}="2H3G NT1";
$resn_exch{"H3G3 NT1"}="3H3G NT1"; $resn_exch{"H481 NT1"}="1H48 NT1"; $resn_exch{"H482 NT1"}="2H48 NT1";
$resn_exch{"H483 NT1"}="3H48 NT1"; $resn_exch{"H4B1 NT1"}="1H4B NT1"; $resn_exch{"H4B2 NT1"}="2H4B NT1";
$resn_exch{"H4B3 NT1"}="3H4B NT1"; $resn_exch{"H531 NT1"}="1H53 NT1"; $resn_exch{"H532 NT1"}="2H53 NT1";
$resn_exch{"H681 NT1"}="1H68 NT1"; $resn_exch{"H682 NT1"}="2H68 NT1"; $resn_exch{"H683 NT1"}="3H68 NT1";
$resn_exch{"H6B1 NT1"}="1H6B NT1"; $resn_exch{"H6B2 NT1"}="2H6B NT1"; $resn_exch{"H6B3 NT1"}="3H6B NT1";
$resn_exch{"H731 NT1"}="1H73 NT1"; $resn_exch{"H732 NT1"}="2H73 NT1"; $resn_exch{"H7B1 NT1"}="1H7B NT1";
$resn_exch{"H7B2 NT1"}="2H7B NT1"; $resn_exch{"H7B3 NT1"}="3H7B NT1"; $resn_exch{"H7I1 NT1"}="1H7I NT1";
$resn_exch{"H7I2 NT1"}="2H7I NT1"; $resn_exch{"H861 NT2"}="1H86 NT2"; $resn_exch{"H862 NT2"}="2H86 NT2";
$resn_exch{" H61 NTA"}="1H6  NTA"; $resn_exch{" H62 NTA"}="2H6  NTA"; $resn_exch{"H101 NTA"}="1H10 NTA";
$resn_exch{"H102 NTA"}="2H10 NTA"; $resn_exch{" H21 NTA"}="1H2  NTA"; $resn_exch{" H22 NTA"}="2H2  NTA";
$resn_exch{" HN1 NTB"}="1HN  NTB"; $resn_exch{" HN2 NTB"}="2HN  NTB"; $resn_exch{" H11 NTB"}="1H1  NTB";
$resn_exch{" H12 NTB"}="2H1  NTB"; $resn_exch{" H13 NTB"}="3H1  NTB"; $resn_exch{" H21 NTB"}="1H2  NTB";
$resn_exch{" H22 NTB"}="2H2  NTB"; $resn_exch{" H23 NTB"}="3H2  NTB"; $resn_exch{" H31 NTB"}="1H3  NTB";
$resn_exch{" H32 NTB"}="2H3  NTB"; $resn_exch{" H33 NTB"}="3H3  NTB"; $resn_exch{" H21 NTC"}="1H2  NTC";
$resn_exch{" H22 NTC"}="2H2  NTC"; $resn_exch{" H41 NTC"}="1H4  NTC"; $resn_exch{" H42 NTC"}="2H4  NTC";
$resn_exch{" H21 NTD"}="1H2  NTD"; $resn_exch{" H22 NTD"}="2H2  NTD"; $resn_exch{" H31 NTD"}="1H3  NTD";
$resn_exch{" H32 NTD"}="2H3  NTD"; $resn_exch{" H41 NTD"}="1H4  NTD"; $resn_exch{" H42 NTD"}="2H4  NTD";
$resn_exch{" H61 NTF"}="1H6  NTF"; $resn_exch{" H62 NTF"}="2H6  NTF"; $resn_exch{"H111 NTH"}="1H11 NTH";
$resn_exch{"H112 NTH"}="2H11 NTH"; $resn_exch{"H121 NTH"}="1H12 NTH"; $resn_exch{"H122 NTH"}="2H12 NTH";
$resn_exch{"H151 NTH"}="1H15 NTH"; $resn_exch{"H152 NTH"}="2H15 NTH"; $resn_exch{"H161 NTH"}="1H16 NTH";
$resn_exch{"H162 NTH"}="2H16 NTH"; $resn_exch{"H181 NTH"}="1H18 NTH"; $resn_exch{"H182 NTH"}="2H18 NTH";
$resn_exch{"H183 NTH"}="3H18 NTH"; $resn_exch{"H221 NTH"}="1H22 NTH"; $resn_exch{"H222 NTH"}="2H22 NTH";
$resn_exch{"H211 NTH"}="1H21 NTH"; $resn_exch{"H212 NTH"}="2H21 NTH"; $resn_exch{"H2C1 NTJ"}="1H2C NTJ";
$resn_exch{"H2C2 NTJ"}="2H2C NTJ"; $resn_exch{"H2C3 NTJ"}="3H2C NTJ"; $resn_exch{"H1C1 NTJ"}="1H1C NTJ";
$resn_exch{"H1C2 NTJ"}="2H1C NTJ"; $resn_exch{"H1C3 NTJ"}="3H1C NTJ"; $resn_exch{"H3C1 NTJ"}="1H3C NTJ";
$resn_exch{"H3C2 NTJ"}="2H3C NTJ"; $resn_exch{"H4C1 NTJ"}="1H4C NTJ"; $resn_exch{"H4C2 NTJ"}="2H4C NTJ";
$resn_exch{"H4C3 NTJ"}="3H4C NTJ"; $resn_exch{" HZ1 NTN"}="1HZ  NTN"; $resn_exch{" HZ2 NTN"}="2HZ  NTN";
$resn_exch{"H151 NTO"}="1H15 NTO"; $resn_exch{"H152 NTO"}="2H15 NTO"; $resn_exch{"H331 NTO"}="1H33 NTO";
$resn_exch{"H332 NTO"}="2H33 NTO"; $resn_exch{"H531 NTO"}="1H53 NTO"; $resn_exch{"H532 NTO"}="2H53 NTO";
$resn_exch{"H5N1 NTO"}="1H5N NTO"; $resn_exch{"H5N2 NTO"}="2H5N NTO"; $resn_exch{"H5N3 NTO"}="3H5N NTO";
$resn_exch{"H111 NTP"}="1H11 NTP"; $resn_exch{"H112 NTP"}="2H11 NTP"; $resn_exch{"H113 NTP"}="3H11 NTP";
$resn_exch{"H151 NTP"}="1H15 NTP"; $resn_exch{"H152 NTP"}="2H15 NTP"; $resn_exch{"H1D1 NTP"}="1H1D NTP";
$resn_exch{"H1D2 NTP"}="2H1D NTP"; $resn_exch{"H1D3 NTP"}="3H1D NTP"; $resn_exch{"H281 NTP"}="1H28 NTP";
$resn_exch{"H282 NTP"}="2H28 NTP"; $resn_exch{"H283 NTP"}="3H28 NTP"; $resn_exch{"H2B1 NTP"}="1H2B NTP";
$resn_exch{"H2B2 NTP"}="2H2B NTP"; $resn_exch{"H2B3 NTP"}="3H2B NTP"; $resn_exch{"H331 NTP"}="1H33 NTP";
$resn_exch{"H332 NTP"}="2H33 NTP"; $resn_exch{"HO3D NTP"}="DHO3 NTP"; $resn_exch{"HO3I NTP"}="IHO3 NTP";
$resn_exch{"H481 NTP"}="1H48 NTP"; $resn_exch{"H482 NTP"}="2H48 NTP"; $resn_exch{"H483 NTP"}="3H48 NTP";
$resn_exch{"H531 NTP"}="1H53 NTP"; $resn_exch{"H532 NTP"}="2H53 NTP"; $resn_exch{"HO5D NTP"}="DHO5 NTP";
$resn_exch{"HO5I NTP"}="IHO5 NTP"; $resn_exch{"H5N1 NTP"}="1H5N NTP"; $resn_exch{"H5N2 NTP"}="2H5N NTP";
$resn_exch{"H5N3 NTP"}="3H5N NTP"; $resn_exch{"H2'1 NTU"}="1H2* NTU"; $resn_exch{"H2'2 NTU"}="2H2* NTU";
$resn_exch{"H2'3 NTU"}="3H2* NTU"; $resn_exch{"HN21 NTU"}="1HN2 NTU"; $resn_exch{"HN22 NTU"}="2HN2 NTU";
$resn_exch{"H101 NTU"}="1H10 NTU"; $resn_exch{"H102 NTU"}="2H10 NTU"; $resn_exch{"H111 NTU"}="1H11 NTU";
$resn_exch{"H112 NTU"}="2H11 NTU"; $resn_exch{"H121 NTU"}="1H12 NTU"; $resn_exch{"H122 NTU"}="2H12 NTU";
$resn_exch{"H131 NTU"}="1H13 NTU"; $resn_exch{"H132 NTU"}="2H13 NTU"; $resn_exch{"H141 NTU"}="1H14 NTU";
$resn_exch{"H142 NTU"}="2H14 NTU"; $resn_exch{"H4'1 NTU"}="1H4* NTU"; $resn_exch{"H4'2 NTU"}="2H4* NTU";
$resn_exch{"H4'3 NTU"}="3H4* NTU"; $resn_exch{"HN41 NTU"}="1HN4 NTU"; $resn_exch{"HN42 NTU"}="2HN4 NTU";
$resn_exch{" H61 NTZ"}="1H6  NTZ"; $resn_exch{" H62 NTZ"}="2H6  NTZ"; $resn_exch{" H91 NU1"}="1H9  NU1";
$resn_exch{" H92 NU1"}="2H9  NU1"; $resn_exch{" H93 NU1"}="3H9  NU1"; $resn_exch{"H221 NU5"}="1H22 NU5";
$resn_exch{"H222 NU5"}="2H22 NU5"; $resn_exch{"H231 NU5"}="1H23 NU5"; $resn_exch{"H232 NU5"}="2H23 NU5";
$resn_exch{"H241 NU5"}="1H24 NU5"; $resn_exch{"H242 NU5"}="2H24 NU5"; $resn_exch{"H251 NU5"}="1H25 NU5";
$resn_exch{"H252 NU5"}="2H25 NU5"; $resn_exch{"H261 NU5"}="1H26 NU5"; $resn_exch{"H262 NU5"}="2H26 NU5";
$resn_exch{" H91 NU5"}="1H9  NU5"; $resn_exch{" H92 NU5"}="2H9  NU5"; $resn_exch{"H151 NU5"}="1H15 NU5";
$resn_exch{"H152 NU5"}="2H15 NU5"; $resn_exch{"H141 NU5"}="1H14 NU5"; $resn_exch{"H142 NU5"}="2H14 NU5";
$resn_exch{"H131 NU5"}="1H13 NU5"; $resn_exch{"H132 NU5"}="2H13 NU5"; $resn_exch{"H121 NU5"}="1H12 NU5";
$resn_exch{"H122 NU5"}="2H12 NU5"; $resn_exch{"H111 NU5"}="1H11 NU5"; $resn_exch{"H112 NU5"}="2H11 NU5";
$resn_exch{"HN31 NUP"}="1HN3 NUP"; $resn_exch{"HN32 NUP"}="2HN3 NUP"; $resn_exch{" H1' NUP"}=" H1* NUP";
$resn_exch{" H2' NUP"}=" H2* NUP"; $resn_exch{"HO2' NUP"}="HO2* NUP"; $resn_exch{" H3' NUP"}=" H3* NUP";
$resn_exch{"HO3' NUP"}="HO3* NUP"; $resn_exch{" H4' NUP"}=" H4* NUP"; $resn_exch{"H5'1 NUP"}="1H5* NUP";
$resn_exch{"H5'2 NUP"}="2H5* NUP"; $resn_exch{" H   NVA"}="1HN  NVA"; $resn_exch{" H2  NVA"}="2HN  NVA";
$resn_exch{" HB2 NVA"}="1HB  NVA"; $resn_exch{" HB3 NVA"}="2HB  NVA"; $resn_exch{" HG2 NVA"}="1HG  NVA";
$resn_exch{" HG3 NVA"}="2HG  NVA"; $resn_exch{" HD1 NVA"}="1HD  NVA"; $resn_exch{" HD2 NVA"}="2HD  NVA";
$resn_exch{" HD3 NVA"}="3HD  NVA"; $resn_exch{"HAA1 NVG"}="1HAA NVG"; $resn_exch{"HAA2 NVG"}="2HAA NVG";
$resn_exch{"HAN1 NVG"}="1HAN NVG"; $resn_exch{"HAN2 NVG"}="2HAN NVG"; $resn_exch{"HAL1 NVG"}="1HAL NVG";
$resn_exch{"HAL2 NVG"}="2HAL NVG"; $resn_exch{"HAM1 NVG"}="1HAM NVG"; $resn_exch{"HAM2 NVG"}="2HAM NVG";
$resn_exch{" H71 NVI"}="1H7  NVI"; $resn_exch{" H72 NVI"}="2H7  NVI"; $resn_exch{"HCB1 NVP"}="1HCB NVP";
$resn_exch{"HCB2 NVP"}="2HCB NVP"; $resn_exch{"HCC1 NVP"}="1HCC NVP"; $resn_exch{"HCC2 NVP"}="2HCC NVP";
$resn_exch{"HCD1 NVP"}="1HCD NVP"; $resn_exch{"HCD2 NVP"}="2HCD NVP"; $resn_exch{"HCD3 NVP"}="3HCD NVP";
$resn_exch{" H12 NW1"}="2H1  NW1"; $resn_exch{" H11 NW1"}="1H1  NW1"; $resn_exch{"H112 NW1"}="2H11 NW1";
$resn_exch{"H111 NW1"}="1H11 NW1"; $resn_exch{"H102 NW1"}="2H10 NW1"; $resn_exch{"H101 NW1"}="1H10 NW1";
$resn_exch{" H92 NW1"}="2H9  NW1"; $resn_exch{" H91 NW1"}="1H9  NW1"; $resn_exch{" H82 NW1"}="2H8  NW1";
$resn_exch{" H81 NW1"}="1H8  NW1"; $resn_exch{" H72 NW1"}="2H7  NW1"; $resn_exch{" H71 NW1"}="1H7  NW1";
$resn_exch{"H9N3 NW1"}="3H9N NW1"; $resn_exch{"H9N2 NW1"}="2H9N NW1"; $resn_exch{" H22 NW1"}="2H2  NW1";
$resn_exch{" H21 NW1"}="1H2  NW1"; $resn_exch{"H2C1 NWA"}="1H2C NWA"; $resn_exch{"H2C2 NWA"}="2H2C NWA";
$resn_exch{"H3C1 NWA"}="1H3C NWA"; $resn_exch{"H3C2 NWA"}="2H3C NWA"; $resn_exch{"H6C1 NWA"}="1H6C NWA";
$resn_exch{"H6C2 NWA"}="2H6C NWA"; $resn_exch{"H6C3 NWA"}="3H6C NWA"; $resn_exch{"H8C1 NWA"}="1H8C NWA";
$resn_exch{"H8C2 NWA"}="2H8C NWA"; $resn_exch{"H8C3 NWA"}="3H8C NWA"; $resn_exch{"H9C1 NWA"}="1H9C NWA";
$resn_exch{"H9C2 NWA"}="2H9C NWA"; $resn_exch{"H9C3 NWA"}="3H9C NWA"; $resn_exch{"H101 NWA"}="1H10 NWA";
$resn_exch{"H102 NWA"}="2H10 NWA"; $resn_exch{"H103 NWA"}="3H10 NWA"; $resn_exch{"H4C1 NWA"}="1H4C NWA";
$resn_exch{"H4C2 NWA"}="2H4C NWA"; $resn_exch{"H2C1 NWB"}="1H2C NWB"; $resn_exch{" HA2 NWB"}="2HA  NWB";
$resn_exch{" HA1 NWB"}="1HA  NWB"; $resn_exch{" HB2 NWB"}="2HB  NWB"; $resn_exch{"H6C1 NWB"}="1H6C NWB";
$resn_exch{"H6C2 NWB"}="2H6C NWB"; $resn_exch{" H63 NWB"}="3H6  NWB"; $resn_exch{"H101 NWB"}="1H10 NWB";
$resn_exch{"H102 NWB"}="2H10 NWB"; $resn_exch{"H103 NWB"}="3H10 NWB"; $resn_exch{"H8C1 NWB"}="1H8C NWB";
$resn_exch{"H8C2 NWB"}="2H8C NWB"; $resn_exch{"H8C3 NWB"}="3H8C NWB"; $resn_exch{"H141 NWB"}="1H14 NWB";
$resn_exch{"H142 NWB"}="2H14 NWB"; $resn_exch{"H9C1 NWB"}="1H9C NWB"; $resn_exch{"H9C2 NWB"}="2H9C NWB";
$resn_exch{"H9C3 NWB"}="3H9C NWB"; $resn_exch{"HBC1 NXA"}="1HBC NXA"; $resn_exch{"HBC2 NXA"}="2HBC NXA";
$resn_exch{"HBC3 NXA"}="3HBC NXA"; $resn_exch{" HD1 NXA"}="1HD  NXA"; $resn_exch{"H111 NXD"}="1H11 NXD";
$resn_exch{"H112 NXD"}="2H11 NXD"; $resn_exch{"H113 NXD"}="3H11 NXD"; $resn_exch{" H31 NXD"}="1H3  NXD";
$resn_exch{" H32 NXD"}="2H3  NXD"; $resn_exch{" HB1 NXD"}="1HB  NXD"; $resn_exch{" HB2 NXD"}="2HB  NXD";
$resn_exch{" HB3 NXD"}="3HB  NXD"; $resn_exch{" H91 NXD"}="1H9  NXD"; $resn_exch{" H92 NXD"}="2H9  NXD";
$resn_exch{"HAK1 NXD"}="1HAK NXD"; $resn_exch{"HAK2 NXD"}="2HAK NXD"; $resn_exch{"H141 NXN"}="1H14 NXN";
$resn_exch{"H142 NXN"}="2H14 NXN"; $resn_exch{"H151 NXN"}="1H15 NXN"; $resn_exch{"H152 NXN"}="2H15 NXN";
$resn_exch{" H91 NXN"}="1H9  NXN"; $resn_exch{" H92 NXN"}="2H9  NXN"; $resn_exch{"H251 NYC"}="1H25 NYC";
$resn_exch{"H252 NYC"}="2H25 NYC"; $resn_exch{"H253 NYC"}="3H25 NYC"; $resn_exch{"H261 NYC"}="1H26 NYC";
$resn_exch{"H262 NYC"}="2H26 NYC"; $resn_exch{"H181 NYC"}="1H18 NYC"; $resn_exch{"H182 NYC"}="2H18 NYC";
$resn_exch{"HD11 NYG"}="1HD1 NYG"; $resn_exch{"HD12 NYG"}="2HD1 NYG"; $resn_exch{"HB11 NYG"}="1HB1 NYG";
$resn_exch{"HB12 NYG"}="2HB1 NYG"; $resn_exch{" HN1 NYG"}="1HN  NYG"; $resn_exch{" HN2 NYG"}="2HN  NYG";
$resn_exch{"HA31 NYG"}="1HA3 NYG"; $resn_exch{"HA32 NYG"}="2HA3 NYG"; $resn_exch{"H131 NYL"}="1H13 NYL";
$resn_exch{"H132 NYL"}="2H13 NYL"; $resn_exch{"H161 NYL"}="1H16 NYL"; $resn_exch{"H162 NYL"}="2H16 NYL";
$resn_exch{"HC61 NYM"}="1HC6 NYM"; $resn_exch{"HN31 NYM"}="1HN3 NYM"; $resn_exch{"HC71 NYM"}="1HC7 NYM";
$resn_exch{"HC72 NYM"}="2HC7 NYM"; $resn_exch{"HC73 NYM"}="3HC7 NYM"; $resn_exch{"H3'1 NYM"}="1H3* NYM";
$resn_exch{"H3'2 NYM"}="2H3* NYM"; $resn_exch{"H1'1 NYM"}="1H1* NYM"; $resn_exch{"H2'1 NYM"}="1H2* NYM";
$resn_exch{"H2'2 NYM"}="2H2* NYM"; $resn_exch{"HC31 NYM"}="1HC3 NYM"; $resn_exch{"H4'1 NYM"}="1H4* NYM";
$resn_exch{"H5'1 NYM"}="1H5* NYM"; $resn_exch{"H5'2 NYM"}="2H5* NYM"; $resn_exch{"HP31 NYM"}="1HP3 NYM";
$resn_exch{"HP21 NYM"}="1HP2 NYM"; $resn_exch{"H8N1 NYP"}="1H8N NYP"; $resn_exch{"H8N2 NYP"}="2H8N NYP";
$resn_exch{"H8N3 NYP"}="3H8N NYP"; $resn_exch{"H111 NYP"}="1H11 NYP"; $resn_exch{"HC71 NYP"}="1HC7 NYP";
$resn_exch{"HC72 NYP"}="2HC7 NYP"; $resn_exch{" H21 NYP"}="1H2  NYP"; $resn_exch{" H31 NYP"}="1H3  NYP";
$resn_exch{" H61 NYP"}="1H6  NYP"; $resn_exch{"HAQ1 NZA"}="1HAQ NZA"; $resn_exch{"HAQ2 NZA"}="2HAQ NZA";
$resn_exch{" H   NZH"}="1HN  NZH"; $resn_exch{" H2  NZH"}="2HN  NZH"; $resn_exch{" HB2 NZH"}="1HB  NZH";
$resn_exch{" HB3 NZH"}="2HB  NZH"; $resn_exch{" HC  NZH"}=" H   NZH"; $resn_exch{"HA'1 NZQ"}="1HA' NZQ";
$resn_exch{"HA'2 NZQ"}="2HA' NZQ"; $resn_exch{"HAA1 NZQ"}="1HAA NZQ"; $resn_exch{"HAA2 NZQ"}="2HAA NZQ";
$resn_exch{"HN'1 NZQ"}="1HN' NZQ"; $resn_exch{"HN'2 NZQ"}="2HN' NZQ"; $resn_exch{"HNB1 NZQ"}="1HNB NZQ";
$resn_exch{"HNB2 NZQ"}="2HNB NZQ"; $resn_exch{"HNA1 NZQ"}="1HNA NZQ"; $resn_exch{"HNA2 NZQ"}="2HNA NZQ";
$resn_exch{" H41 O1C"}="1H4  O1C"; $resn_exch{" H42 O1C"}="2H4  O1C"; $resn_exch{"H121 O1C"}="1H12 O1C";
$resn_exch{"H122 O1C"}="2H12 O1C"; $resn_exch{"H111 O1C"}="1H11 O1C"; $resn_exch{"H112 O1C"}="2H11 O1C";
$resn_exch{" H91 O1C"}="1H9  O1C"; $resn_exch{" H92 O1C"}="2H9  O1C"; $resn_exch{"H151 O1C"}="1H15 O1C";
$resn_exch{"H152 O1C"}="2H15 O1C"; $resn_exch{"H161 O1C"}="1H16 O1C"; $resn_exch{"H162 O1C"}="2H16 O1C";
$resn_exch{"H191 O1C"}="1H19 O1C"; $resn_exch{"H192 O1C"}="2H19 O1C"; $resn_exch{"H221 O1C"}="1H22 O1C";
$resn_exch{"H222 O1C"}="2H22 O1C"; $resn_exch{"H181 O1C"}="1H18 O1C"; $resn_exch{"H182 O1C"}="2H18 O1C";
$resn_exch{"H183 O1C"}="3H18 O1C"; $resn_exch{"H211 O1C"}="1H21 O1C"; $resn_exch{"H212 O1C"}="2H21 O1C";
$resn_exch{"H213 O1C"}="3H21 O1C"; $resn_exch{"H231 O1C"}="1H23 O1C"; $resn_exch{"H232 O1C"}="2H23 O1C";
$resn_exch{"H241 O1C"}="1H24 O1C"; $resn_exch{"H242 O1C"}="2H24 O1C"; $resn_exch{"H271 O1C"}="1H27 O1C";
$resn_exch{"H272 O1C"}="2H27 O1C"; $resn_exch{"H273 O1C"}="3H27 O1C"; $resn_exch{"H261 O1C"}="1H26 O1C";
$resn_exch{"H262 O1C"}="2H26 O1C"; $resn_exch{"H263 O1C"}="3H26 O1C"; $resn_exch{"H281 O1C"}="1H28 O1C";
$resn_exch{"H282 O1C"}="2H28 O1C"; $resn_exch{"H291 O1C"}="1H29 O1C"; $resn_exch{"H292 O1C"}="2H29 O1C";
$resn_exch{"H301 O1C"}="1H30 O1C"; $resn_exch{"H302 O1C"}="2H30 O1C"; $resn_exch{"HOP2 O2C"}="2HOP O2C";
$resn_exch{"H5'1 O2C"}="1H5* O2C"; $resn_exch{"H5'2 O2C"}="2H5* O2C"; $resn_exch{" H4' O2C"}=" H4* O2C";
$resn_exch{" H1' O2C"}=" H1* O2C"; $resn_exch{"HN41 O2C"}="1HN4 O2C"; $resn_exch{"HN42 O2C"}="2HN4 O2C";
$resn_exch{" H2' O2C"}=" H2* O2C"; $resn_exch{"HO'2 O2C"}="2HO* O2C"; $resn_exch{"H3'1 O2C"}="1H3* O2C";
$resn_exch{"H3'2 O2C"}="2H3* O2C"; $resn_exch{"HOP3 O2C"}="3HOP O2C"; $resn_exch{" H71 O33"}="1H7  O33";
$resn_exch{" H72 O33"}="2H7  O33"; $resn_exch{"H141 O33"}="1H14 O33"; $resn_exch{"H142 O33"}="2H14 O33";
$resn_exch{"H181 O33"}="1H18 O33"; $resn_exch{"H182 O33"}="2H18 O33"; $resn_exch{"H183 O33"}="3H18 O33";
$resn_exch{"H191 O33"}="1H19 O33"; $resn_exch{"H192 O33"}="2H19 O33"; $resn_exch{"H193 O33"}="3H19 O33";
$resn_exch{"H201 O33"}="1H20 O33"; $resn_exch{"H202 O33"}="2H20 O33"; $resn_exch{"H211 O33"}="1H21 O33";
$resn_exch{"H212 O33"}="2H21 O33"; $resn_exch{"H221 O33"}="1H22 O33"; $resn_exch{"H222 O33"}="2H22 O33";
$resn_exch{"H223 O33"}="3H22 O33"; $resn_exch{"H351 O33"}="1H35 O33"; $resn_exch{"H352 O33"}="2H35 O33";
$resn_exch{"HO41 O4M"}="1HO4 O4M"; $resn_exch{"HO42 O4M"}="2HO4 O4M"; $resn_exch{"HO31 O4M"}="1HO3 O4M";
$resn_exch{"HO32 O4M"}="2HO3 O4M"; $resn_exch{"HO21 O4M"}="1HO2 O4M"; $resn_exch{"HO22 O4M"}="2HO2 O4M";
$resn_exch{"HO11 O4M"}="1HO1 O4M"; $resn_exch{"HO12 O4M"}="2HO1 O4M"; $resn_exch{" H21 OAA"}="1H2  OAA";
$resn_exch{" H22 OAA"}="2H2  OAA"; $resn_exch{"HB11 OAC"}="1HB1 OAC"; $resn_exch{"HB12 OAC"}="2HB1 OAC";
$resn_exch{"HB21 OAC"}="1HB2 OAC"; $resn_exch{"HB22 OAC"}="2HB2 OAC"; $resn_exch{"HB23 OAC"}="3HB2 OAC";
$resn_exch{"HOA2 OAD"}="2HOA OAD"; $resn_exch{"H5'1 OAD"}="1H5* OAD"; $resn_exch{"H5'2 OAD"}="2H5* OAD";
$resn_exch{" H'4 OAD"}="4H*  OAD"; $resn_exch{" H'3 OAD"}="3H*  OAD"; $resn_exch{"HO'3 OAD"}="3HO* OAD";
$resn_exch{" H'2 OAD"}="2H*  OAD"; $resn_exch{"HO'2 OAD"}="2HO* OAD"; $resn_exch{" H'1 OAD"}="1H*  OAD";
$resn_exch{" H61 OAD"}="1H6  OAD"; $resn_exch{" H62 OAD"}="2H6  OAD"; $resn_exch{"HOB2 OAD"}="2HOB OAD";
$resn_exch{"H5R1 OAD"}="1H5R OAD"; $resn_exch{"H5R2 OAD"}="2H5R OAD"; $resn_exch{"HR'4 OAD"}="4HR* OAD";
$resn_exch{"HR'3 OAD"}="3HR* OAD"; $resn_exch{"HOR3 OAD"}="3HOR OAD"; $resn_exch{"HR'2 OAD"}="2HR* OAD";
$resn_exch{"HR'1 OAD"}="1HR* OAD"; $resn_exch{"HOR1 OAD"}="1HOR OAD"; $resn_exch{"H7R1 OAD"}="1H7R OAD";
$resn_exch{"H7R2 OAD"}="2H7R OAD"; $resn_exch{"H7R3 OAD"}="3H7R OAD"; $resn_exch{"H151 OAL"}="1H15 OAL";
$resn_exch{"H152 OAL"}="2H15 OAL"; $resn_exch{"HAH1 OAN"}="1HAH OAN"; $resn_exch{"HAH2 OAN"}="2HAH OAN";
$resn_exch{"HAH3 OAN"}="3HAH OAN"; $resn_exch{"HAF1 OAN"}="1HAF OAN"; $resn_exch{"HAF2 OAN"}="2HAF OAN";
$resn_exch{" H11 OAP"}="1H1  OAP"; $resn_exch{"H12A OAP"}="2H1  OAP"; $resn_exch{" H31 OAP"}="1H3  OAP";
$resn_exch{" H32 OAP"}="2H3  OAP"; $resn_exch{"H311 OAP"}="1H31 OAP"; $resn_exch{"H312 OAP"}="2H31 OAP";
$resn_exch{"H171 OAP"}="1H17 OAP"; $resn_exch{"H172 OAP"}="2H17 OAP"; $resn_exch{"H221 OAP"}="1H22 OAP";
$resn_exch{"H222 OAP"}="2H22 OAP"; $resn_exch{"H231 OAP"}="1H23 OAP"; $resn_exch{"H232 OAP"}="2H23 OAP";
$resn_exch{"H241 OAP"}="1H24 OAP"; $resn_exch{"H242 OAP"}="2H24 OAP"; $resn_exch{"H251 OAP"}="1H25 OAP";
$resn_exch{"H252 OAP"}="2H25 OAP"; $resn_exch{"H261 OAP"}="1H26 OAP"; $resn_exch{"H262 OAP"}="2H26 OAP";
$resn_exch{"H271 OAP"}="1H27 OAP"; $resn_exch{"H272 OAP"}="2H27 OAP"; $resn_exch{"HN11 OAR"}="1HN1 OAR";
$resn_exch{"HN12 OAR"}="2HN1 OAR"; $resn_exch{"HN41 OAR"}="1HN4 OAR"; $resn_exch{"HN42 OAR"}="2HN4 OAR";
$resn_exch{" H21 OAR"}="1H2  OAR"; $resn_exch{" H22 OAR"}="2H2  OAR"; $resn_exch{" H31 OAR"}="1H3  OAR";
$resn_exch{" H32 OAR"}="2H3  OAR"; $resn_exch{" H41 OAR"}="1H4  OAR"; $resn_exch{" H42 OAR"}="2H4  OAR";
$resn_exch{" H61 OAR"}="1H6  OAR"; $resn_exch{" H62 OAR"}="2H6  OAR"; $resn_exch{" H2  OAS"}=" HN2 OAS";
$resn_exch{" HB2 OAS"}="1HB  OAS"; $resn_exch{" HB3 OAS"}="2HB  OAS"; $resn_exch{"HC21 OAS"}="1HC2 OAS";
$resn_exch{"HC22 OAS"}="2HC2 OAS"; $resn_exch{"HC23 OAS"}="3HC2 OAS"; $resn_exch{" H41 OBE"}="1H4  OBE";
$resn_exch{" H42 OBE"}="2H4  OBE"; $resn_exch{" H61 OBE"}="1H6  OBE"; $resn_exch{" H62 OBE"}="2H6  OBE";
$resn_exch{" H71 OBE"}="1H7  OBE"; $resn_exch{" H72 OBE"}="2H7  OBE"; $resn_exch{"H161 OBE"}="1H16 OBE";
$resn_exch{"H162 OBE"}="2H16 OBE"; $resn_exch{"H163 OBE"}="3H16 OBE"; $resn_exch{" H81 OBI"}="1H8  OBI";
$resn_exch{" H82 OBI"}="2H8  OBI"; $resn_exch{" H71 OBI"}="1H7  OBI"; $resn_exch{" H72 OBI"}="2H7  OBI";
$resn_exch{" H21 OBN"}="1H2  OBN"; $resn_exch{"H22A OBN"}="2H2  OBN"; $resn_exch{" H41 OBN"}="1H4  OBN";
$resn_exch{" H42 OBN"}="2H4  OBN"; $resn_exch{" H61 OBN"}="1H6  OBN"; $resn_exch{" H62 OBN"}="2H6  OBN";
$resn_exch{" H71 OBN"}="1H7  OBN"; $resn_exch{" H72 OBN"}="2H7  OBN"; $resn_exch{"H121 OBN"}="1H12 OBN";
$resn_exch{"H122 OBN"}="2H12 OBN"; $resn_exch{"H151 OBN"}="1H15 OBN"; $resn_exch{"H152 OBN"}="2H15 OBN";
$resn_exch{"H161 OBN"}="1H16 OBN"; $resn_exch{"H162 OBN"}="2H16 OBN"; $resn_exch{"H181 OBN"}="1H18 OBN";
$resn_exch{"H182 OBN"}="2H18 OBN"; $resn_exch{"H183 OBN"}="3H18 OBN"; $resn_exch{"H191 OBN"}="1H19 OBN";
$resn_exch{"H192 OBN"}="2H19 OBN"; $resn_exch{"H211 OBN"}="1H21 OBN"; $resn_exch{"H212 OBN"}="2H21 OBN";
$resn_exch{"H6'1 OBN"}="1H6' OBN"; $resn_exch{"H6'2 OBN"}="2H6' OBN"; $resn_exch{"H6'3 OBN"}="3H6' OBN";
$resn_exch{"HO2' OBN"}="'HO2 OBN"; $resn_exch{"HO3' OBN"}="'HO3 OBN"; $resn_exch{"HO4' OBN"}="'HO4 OBN";
$resn_exch{"H101 OBS"}="1H10 OBS"; $resn_exch{"H102 OBS"}="2H10 OBS"; $resn_exch{"H9C1 OBS"}="1H9C OBS";
$resn_exch{"H9C2 OBS"}="2H9C OBS"; $resn_exch{"H7C1 OBS"}="1H7C OBS"; $resn_exch{"H7C2 OBS"}="2H7C OBS";
$resn_exch{"H5C1 OBS"}="1H5C OBS"; $resn_exch{"H5C2 OBS"}="2H5C OBS"; $resn_exch{"H2C1 OBS"}="1H2C OBS";
$resn_exch{"H2C2 OBS"}="2H2C OBS"; $resn_exch{"HEC1 OBS"}="1HEC OBS"; $resn_exch{"HEC2 OBS"}="2HEC OBS";
$resn_exch{"HDC1 OBS"}="1HDC OBS"; $resn_exch{"HDC2 OBS"}="2HDC OBS"; $resn_exch{"HGC1 OBS"}="1HGC OBS";
$resn_exch{"HGC2 OBS"}="2HGC OBS"; $resn_exch{"HBC1 OBS"}="1HBC OBS"; $resn_exch{"HBC2 OBS"}="2HBC OBS";
$resn_exch{" HN1 OBS"}="1HN  OBS"; $resn_exch{" HN2 OBS"}="2HN  OBS"; $resn_exch{"HO11 OC1"}="1HO1 OC1";
$resn_exch{"HO12 OC1"}="2HO1 OC1"; $resn_exch{"HO11 OC2"}="1HO1 OC2"; $resn_exch{"HO12 OC2"}="2HO1 OC2";
$resn_exch{"HO13 OC2"}="3HO1 OC2"; $resn_exch{"HO14 OC2"}="4HO1 OC2"; $resn_exch{"HO11 OC3"}="1HO1 OC3";
$resn_exch{"HO12 OC3"}="2HO1 OC3"; $resn_exch{"HO21 OC3"}="1HO2 OC3"; $resn_exch{"HO22 OC3"}="2HO2 OC3";
$resn_exch{"HO31 OC3"}="1HO3 OC3"; $resn_exch{"HO32 OC3"}="2HO3 OC3"; $resn_exch{"HO11 OC4"}="1HO1 OC4";
$resn_exch{"HO12 OC4"}="2HO1 OC4"; $resn_exch{"HO21 OC4"}="1HO2 OC4"; $resn_exch{"HO22 OC4"}="2HO2 OC4";
$resn_exch{"HO31 OC4"}="1HO3 OC4"; $resn_exch{"HO32 OC4"}="2HO3 OC4"; $resn_exch{"HO41 OC4"}="1HO4 OC4";
$resn_exch{"HO42 OC4"}="2HO4 OC4"; $resn_exch{"HO11 OC5"}="1HO1 OC5"; $resn_exch{"HO12 OC5"}="2HO1 OC5";
$resn_exch{"HO21 OC5"}="1HO2 OC5"; $resn_exch{"HO22 OC5"}="2HO2 OC5"; $resn_exch{"HO31 OC5"}="1HO3 OC5";
$resn_exch{"HO32 OC5"}="2HO3 OC5"; $resn_exch{"HO41 OC5"}="1HO4 OC5"; $resn_exch{"HO42 OC5"}="2HO4 OC5";
$resn_exch{"HO51 OC5"}="1HO5 OC5"; $resn_exch{"HO52 OC5"}="2HO5 OC5"; $resn_exch{"HO11 OC6"}="1HO1 OC6";
$resn_exch{"HO12 OC6"}="2HO1 OC6"; $resn_exch{"HO21 OC6"}="1HO2 OC6"; $resn_exch{"HO22 OC6"}="2HO2 OC6";
$resn_exch{"HO31 OC6"}="1HO3 OC6"; $resn_exch{"HO32 OC6"}="2HO3 OC6"; $resn_exch{"HO41 OC6"}="1HO4 OC6";
$resn_exch{"HO42 OC6"}="2HO4 OC6"; $resn_exch{"HO51 OC6"}="1HO5 OC6"; $resn_exch{"HO52 OC6"}="2HO5 OC6";
$resn_exch{"HO61 OC6"}="1HO6 OC6"; $resn_exch{"HO62 OC6"}="2HO6 OC6"; $resn_exch{"HO11 OC7"}="1HO1 OC7";
$resn_exch{"HO12 OC7"}="2HO1 OC7"; $resn_exch{"HO21 OC7"}="1HO2 OC7"; $resn_exch{"HO22 OC7"}="2HO2 OC7";
$resn_exch{"HO31 OC7"}="1HO3 OC7"; $resn_exch{"HO32 OC7"}="2HO3 OC7"; $resn_exch{"HO41 OC7"}="1HO4 OC7";
$resn_exch{"HO42 OC7"}="2HO4 OC7"; $resn_exch{"HO51 OC7"}="1HO5 OC7"; $resn_exch{"HO52 OC7"}="2HO5 OC7";
$resn_exch{"HO61 OC7"}="1HO6 OC7"; $resn_exch{"HO62 OC7"}="2HO6 OC7"; $resn_exch{"HO71 OC7"}="1HO7 OC7";
$resn_exch{"HO72 OC7"}="2HO7 OC7"; $resn_exch{"HO11 OC8"}="1HO1 OC8"; $resn_exch{"HO12 OC8"}="2HO1 OC8";
$resn_exch{"HO21 OC8"}="1HO2 OC8"; $resn_exch{"HO22 OC8"}="2HO2 OC8"; $resn_exch{"HO31 OC8"}="1HO3 OC8";
$resn_exch{"HO32 OC8"}="2HO3 OC8"; $resn_exch{"HO41 OC8"}="1HO4 OC8"; $resn_exch{"HO42 OC8"}="2HO4 OC8";
$resn_exch{"HO51 OC8"}="1HO5 OC8"; $resn_exch{"HO52 OC8"}="2HO5 OC8"; $resn_exch{"HO61 OC8"}="1HO6 OC8";
$resn_exch{"HO62 OC8"}="2HO6 OC8"; $resn_exch{"HO71 OC8"}="1HO7 OC8"; $resn_exch{"HO72 OC8"}="2HO7 OC8";
$resn_exch{"HO81 OC8"}="1HO8 OC8"; $resn_exch{"HO82 OC8"}="2HO8 OC8"; $resn_exch{"HAA1 OC9"}="1HAA OC9";
$resn_exch{"HAA2 OC9"}="2HAA OC9"; $resn_exch{"HAA3 OC9"}="3HAA OC9"; $resn_exch{"HAC1 OC9"}="1HAC OC9";
$resn_exch{"HAC2 OC9"}="2HAC OC9"; $resn_exch{"HAE1 OC9"}="1HAE OC9"; $resn_exch{"HAE2 OC9"}="2HAE OC9";
$resn_exch{"HAG1 OC9"}="1HAG OC9"; $resn_exch{"HAG2 OC9"}="2HAG OC9"; $resn_exch{"HAI1 OC9"}="1HAI OC9";
$resn_exch{"HAI2 OC9"}="2HAI OC9"; $resn_exch{"HAH1 OC9"}="1HAH OC9"; $resn_exch{"HAH2 OC9"}="2HAH OC9";
$resn_exch{"HAF1 OC9"}="1HAF OC9"; $resn_exch{"HAF2 OC9"}="2HAF OC9"; $resn_exch{"HAD1 OC9"}="1HAD OC9";
$resn_exch{"HAD2 OC9"}="2HAD OC9"; $resn_exch{" H21 OCA"}="1H2  OCA"; $resn_exch{" H22 OCA"}="2H2  OCA";
$resn_exch{" H31 OCA"}="1H3  OCA"; $resn_exch{" H32 OCA"}="2H3  OCA"; $resn_exch{" H41 OCA"}="1H4  OCA";
$resn_exch{" H42 OCA"}="2H4  OCA"; $resn_exch{" H51 OCA"}="1H5  OCA"; $resn_exch{" H52 OCA"}="2H5  OCA";
$resn_exch{" H61 OCA"}="1H6  OCA"; $resn_exch{" H62 OCA"}="2H6  OCA"; $resn_exch{" H71 OCA"}="1H7  OCA";
$resn_exch{" H72 OCA"}="2H7  OCA"; $resn_exch{" H81 OCA"}="1H8  OCA"; $resn_exch{" H82 OCA"}="2H8  OCA";
$resn_exch{" H83 OCA"}="3H8  OCA"; $resn_exch{"HAA1 OCB"}="1HAA OCB"; $resn_exch{"HAA2 OCB"}="2HAA OCB";
$resn_exch{"HAA3 OCB"}="3HAA OCB"; $resn_exch{"HAB1 OCB"}="1HAB OCB"; $resn_exch{"HAB2 OCB"}="2HAB OCB";
$resn_exch{"HAC1 OCB"}="1HAC OCB"; $resn_exch{"HAC2 OCB"}="2HAC OCB"; $resn_exch{"HAD1 OCB"}="1HAD OCB";
$resn_exch{"HAD2 OCB"}="2HAD OCB"; $resn_exch{"HAE1 OCB"}="1HAE OCB"; $resn_exch{"HAE2 OCB"}="2HAE OCB";
$resn_exch{"HAF1 OCB"}="1HAF OCB"; $resn_exch{"HAF2 OCB"}="2HAF OCB"; $resn_exch{"HAG1 OCB"}="1HAG OCB";
$resn_exch{"HAG2 OCB"}="2HAG OCB"; $resn_exch{"HAQ1 OCB"}="1HAQ OCB"; $resn_exch{"HAQ2 OCB"}="2HAQ OCB";
$resn_exch{"HAO1 OCB"}="1HAO OCB"; $resn_exch{"HAO2 OCB"}="2HAO OCB"; $resn_exch{"HAT1 OCB"}="1HAT OCB";
$resn_exch{"HAT2 OCB"}="2HAT OCB"; $resn_exch{"HAT3 OCB"}="3HAT OCB"; $resn_exch{"HAJ1 OCB"}="1HAJ OCB";
$resn_exch{"HAJ2 OCB"}="2HAJ OCB"; $resn_exch{"HAJ3 OCB"}="3HAJ OCB"; $resn_exch{"HAM1 OCB"}="1HAM OCB";
$resn_exch{"HAM2 OCB"}="2HAM OCB"; $resn_exch{"HAM3 OCB"}="3HAM OCB"; $resn_exch{" H41 OCC"}="1H4  OCC";
$resn_exch{" H42 OCC"}="2H4  OCC"; $resn_exch{"H121 OCC"}="1H12 OCC"; $resn_exch{"H122 OCC"}="2H12 OCC";
$resn_exch{"H111 OCC"}="1H11 OCC"; $resn_exch{"H112 OCC"}="2H11 OCC"; $resn_exch{" H91 OCC"}="1H9  OCC";
$resn_exch{" H92 OCC"}="2H9  OCC"; $resn_exch{"H151 OCC"}="1H15 OCC"; $resn_exch{"H152 OCC"}="2H15 OCC";
$resn_exch{"H161 OCC"}="1H16 OCC"; $resn_exch{"H162 OCC"}="2H16 OCC"; $resn_exch{"H191 OCC"}="1H19 OCC";
$resn_exch{"H192 OCC"}="2H19 OCC"; $resn_exch{"H221 OCC"}="1H22 OCC"; $resn_exch{"H222 OCC"}="2H22 OCC";
$resn_exch{"H181 OCC"}="1H18 OCC"; $resn_exch{"H182 OCC"}="2H18 OCC"; $resn_exch{"H183 OCC"}="3H18 OCC";
$resn_exch{"H211 OCC"}="1H21 OCC"; $resn_exch{"H212 OCC"}="2H21 OCC"; $resn_exch{"H213 OCC"}="3H21 OCC";
$resn_exch{"H231 OCC"}="1H23 OCC"; $resn_exch{"H232 OCC"}="2H23 OCC"; $resn_exch{"H241 OCC"}="1H24 OCC";
$resn_exch{"H242 OCC"}="2H24 OCC"; $resn_exch{"H271 OCC"}="1H27 OCC"; $resn_exch{"H272 OCC"}="2H27 OCC";
$resn_exch{"H273 OCC"}="3H27 OCC"; $resn_exch{"H261 OCC"}="1H26 OCC"; $resn_exch{"H262 OCC"}="2H26 OCC";
$resn_exch{"H263 OCC"}="3H26 OCC"; $resn_exch{"H291 OCC"}="1H29 OCC"; $resn_exch{"H292 OCC"}="2H29 OCC";
$resn_exch{"H301 OCC"}="1H30 OCC"; $resn_exch{"H302 OCC"}="2H30 OCC"; $resn_exch{"H311 OCC"}="1H31 OCC";
$resn_exch{"H312 OCC"}="2H31 OCC"; $resn_exch{"HO11 OCL"}="1HO1 OCL"; $resn_exch{"HO12 OCL"}="2HO1 OCL";
$resn_exch{"HO11 OCM"}="1HO1 OCM"; $resn_exch{"HO12 OCM"}="2HO1 OCM"; $resn_exch{"HO21 OCM"}="1HO2 OCM";
$resn_exch{"HO22 OCM"}="2HO2 OCM"; $resn_exch{"HO31 OCM"}="1HO3 OCM"; $resn_exch{"HO32 OCM"}="2HO3 OCM";
$resn_exch{"HO11 OCN"}="1HO1 OCN"; $resn_exch{"HO12 OCN"}="2HO1 OCN"; $resn_exch{"HO13 OCN"}="3HO1 OCN";
$resn_exch{"HO14 OCN"}="4HO1 OCN"; $resn_exch{"HO11 OCO"}="1HO1 OCO"; $resn_exch{"HO12 OCO"}="2HO1 OCO";
$resn_exch{"HO21 OCO"}="1HO2 OCO"; $resn_exch{"HO22 OCO"}="2HO2 OCO"; $resn_exch{"HO31 OCO"}="1HO3 OCO";
$resn_exch{"HO32 OCO"}="2HO3 OCO"; $resn_exch{"HO41 OCO"}="1HO4 OCO"; $resn_exch{"HO42 OCO"}="2HO4 OCO";
$resn_exch{"HO51 OCO"}="1HO5 OCO"; $resn_exch{"HO52 OCO"}="2HO5 OCO"; $resn_exch{"HO61 OCO"}="1HO6 OCO";
$resn_exch{"HO62 OCO"}="2HO6 OCO"; $resn_exch{" H11 OCP"}="1H1  OCP"; $resn_exch{" H12 OCP"}="2H1  OCP";
$resn_exch{" H31 OCP"}="1H3  OCP"; $resn_exch{" H32 OCP"}="2H3  OCP"; $resn_exch{" H41 OCP"}="1H4  OCP";
$resn_exch{" H42 OCP"}="2H4  OCP"; $resn_exch{" H51 OCP"}="1H5  OCP"; $resn_exch{" H52 OCP"}="2H5  OCP";
$resn_exch{" H61 OCP"}="1H6  OCP"; $resn_exch{" H62 OCP"}="2H6  OCP"; $resn_exch{" H71 OCP"}="1H7  OCP";
$resn_exch{" H72 OCP"}="2H7  OCP"; $resn_exch{" H81 OCP"}="1H8  OCP"; $resn_exch{" H82 OCP"}="2H8  OCP";
$resn_exch{" H91 OCP"}="1H9  OCP"; $resn_exch{" H92 OCP"}="2H9  OCP"; $resn_exch{"H101 OCP"}="1H10 OCP";
$resn_exch{"H102 OCP"}="2H10 OCP"; $resn_exch{"H111 OCP"}="1H11 OCP"; $resn_exch{"H112 OCP"}="2H11 OCP";
$resn_exch{"H113 OCP"}="3H11 OCP"; $resn_exch{"H131 OCP"}="1H13 OCP"; $resn_exch{"H132 OCP"}="2H13 OCP";
$resn_exch{"H141 OCP"}="1H14 OCP"; $resn_exch{"H142 OCP"}="2H14 OCP"; $resn_exch{"H151 OCP"}="1H15 OCP";
$resn_exch{"H152 OCP"}="2H15 OCP"; $resn_exch{"H161 OCP"}="1H16 OCP"; $resn_exch{"H162 OCP"}="2H16 OCP";
$resn_exch{"H171 OCP"}="1H17 OCP"; $resn_exch{"H172 OCP"}="2H17 OCP"; $resn_exch{"H181 OCP"}="1H18 OCP";
$resn_exch{"H182 OCP"}="2H18 OCP"; $resn_exch{"H191 OCP"}="1H19 OCP"; $resn_exch{"H192 OCP"}="2H19 OCP";
$resn_exch{"H201 OCP"}="1H20 OCP"; $resn_exch{"H202 OCP"}="2H20 OCP"; $resn_exch{"H203 OCP"}="3H20 OCP";
$resn_exch{"H231 OCP"}="1H23 OCP"; $resn_exch{"H232 OCP"}="2H23 OCP"; $resn_exch{"H241 OCP"}="1H24 OCP";
$resn_exch{"H242 OCP"}="2H24 OCP"; $resn_exch{"H251 OCP"}="1H25 OCP"; $resn_exch{"H252 OCP"}="2H25 OCP";
$resn_exch{"H261 OCP"}="1H26 OCP"; $resn_exch{"H262 OCP"}="2H26 OCP"; $resn_exch{"H271 OCP"}="1H27 OCP";
$resn_exch{"H272 OCP"}="2H27 OCP"; $resn_exch{"H281 OCP"}="1H28 OCP"; $resn_exch{"H282 OCP"}="2H28 OCP";
$resn_exch{"H291 OCP"}="1H29 OCP"; $resn_exch{"H292 OCP"}="2H29 OCP"; $resn_exch{"H301 OCP"}="1H30 OCP";
$resn_exch{"H302 OCP"}="2H30 OCP"; $resn_exch{"H303 OCP"}="3H30 OCP"; $resn_exch{"HMK1 OCQ"}="1HMK OCQ";
$resn_exch{"HMK2 OCQ"}="2HMK OCQ"; $resn_exch{" HN1 OCQ"}="1HN  OCQ"; $resn_exch{" HN2 OCQ"}="2HN  OCQ";
$resn_exch{" HB1 OCQ"}="1HB  OCQ"; $resn_exch{" HB2 OCQ"}="2HB  OCQ"; $resn_exch{"HAE1 OCQ"}="1HAE OCQ";
$resn_exch{"HAE2 OCQ"}="2HAE OCQ"; $resn_exch{"HD11 OCQ"}="1HD1 OCQ"; $resn_exch{"HD12 OCQ"}="2HD1 OCQ";
$resn_exch{" H   OCS"}="1HN  OCS"; $resn_exch{" HN2 OCS"}="2HN  OCS"; $resn_exch{" HB2 OCS"}="1HB  OCS";
$resn_exch{" HB3 OCS"}="2HB  OCS"; $resn_exch{" H11 OCT"}="1H1  OCT"; $resn_exch{" H12 OCT"}="2H1  OCT";
$resn_exch{" H13 OCT"}="3H1  OCT"; $resn_exch{" H21 OCT"}="1H2  OCT"; $resn_exch{" H22 OCT"}="2H2  OCT";
$resn_exch{" H31 OCT"}="1H3  OCT"; $resn_exch{" H32 OCT"}="2H3  OCT"; $resn_exch{" H41 OCT"}="1H4  OCT";
$resn_exch{" H42 OCT"}="2H4  OCT"; $resn_exch{" H51 OCT"}="1H5  OCT"; $resn_exch{" H52 OCT"}="2H5  OCT";
$resn_exch{" H61 OCT"}="1H6  OCT"; $resn_exch{" H62 OCT"}="2H6  OCT"; $resn_exch{" H71 OCT"}="1H7  OCT";
$resn_exch{" H72 OCT"}="2H7  OCT"; $resn_exch{" H81 OCT"}="1H8  OCT"; $resn_exch{" H82 OCT"}="2H8  OCT";
$resn_exch{" H83 OCT"}="3H8  OCT"; $resn_exch{" HC2 OCV"}="2HC  OCV"; $resn_exch{"HC31 OCV"}="1HC3 OCV";
$resn_exch{"HC32 OCV"}="2HC3 OCV"; $resn_exch{"HC41 OCV"}="1HC4 OCV"; $resn_exch{"HC42 OCV"}="2HC4 OCV";
$resn_exch{"HC71 OCV"}="1HC7 OCV"; $resn_exch{"HC72 OCV"}="2HC7 OCV"; $resn_exch{"HN11 OCV"}="1HN1 OCV";
$resn_exch{"HC21 OCV"}="1HC2 OCV"; $resn_exch{"H141 OCV"}="1H14 OCV"; $resn_exch{"H142 OCV"}="2H14 OCV";
$resn_exch{"H161 OCV"}="1H16 OCV"; $resn_exch{"H162 OCV"}="2H16 OCV"; $resn_exch{"HS17 OCV"}="17HS OCV";
$resn_exch{"HO91 OCV"}="1HO9 OCV"; $resn_exch{"HC03 OCV"}="3HC0 OCV"; $resn_exch{"HC23 OCV"}="3HC2 OCV";
$resn_exch{"H331 OCV"}="1H33 OCV"; $resn_exch{"H332 OCV"}="2H33 OCV"; $resn_exch{"H333 OCV"}="3H33 OCV";
$resn_exch{"H371 OCV"}="1H37 OCV"; $resn_exch{"H372 OCV"}="2H37 OCV"; $resn_exch{"H373 OCV"}="3H37 OCV";
$resn_exch{"HO34 OCV"}="4HO3 OCV"; $resn_exch{" H   OCY"}="1HN  OCY"; $resn_exch{" H2  OCY"}="2HN  OCY";
$resn_exch{" HB2 OCY"}="1HB  OCY"; $resn_exch{" HB3 OCY"}="2HB  OCY"; $resn_exch{" HD2 OCY"}="1HD  OCY";
$resn_exch{" HD3 OCY"}="2HD  OCY"; $resn_exch{" HE2 OCY"}="1HE  OCY"; $resn_exch{" HE3 OCY"}="2HE  OCY";
$resn_exch{" HZ  OCY"}=" HOZ OCY"; $resn_exch{" H   ODA"}="1HN  ODA"; $resn_exch{" H2  ODA"}="2HN  ODA";
$resn_exch{" HA1 ODA"}="1HA  ODA"; $resn_exch{" HB2 ODA"}="1HB  ODA"; $resn_exch{" HB3 ODA"}="2HB  ODA";
$resn_exch{" HG2 ODA"}="1HG  ODA"; $resn_exch{" HG3 ODA"}="2HG  ODA"; $resn_exch{" HA2 ODA"}="1HA2 ODA";
$resn_exch{"HB23 ODA"}="1HB2 ODA"; $resn_exch{"HB22 ODA"}="2HB2 ODA"; $resn_exch{"HG23 ODA"}="1HG2 ODA";
$resn_exch{"HG22 ODA"}="2HG2 ODA"; $resn_exch{"HD22 ODA"}="1HD2 ODA"; $resn_exch{"HD23 ODA"}="2HD2 ODA";
$resn_exch{" H21 ODD"}="1H2  ODD"; $resn_exch{" H22 ODD"}="2H2  ODD"; $resn_exch{" H31 ODD"}="1H3  ODD";
$resn_exch{" H32 ODD"}="2H3  ODD"; $resn_exch{" H41 ODD"}="1H4  ODD"; $resn_exch{" H42 ODD"}="2H4  ODD";
$resn_exch{" H51 ODD"}="1H5  ODD"; $resn_exch{" H52 ODD"}="2H5  ODD"; $resn_exch{" H61 ODD"}="1H6  ODD";
$resn_exch{" H62 ODD"}="2H6  ODD"; $resn_exch{" H71 ODD"}="1H7  ODD"; $resn_exch{" H72 ODD"}="2H7  ODD";
$resn_exch{" H81 ODD"}="1H8  ODD"; $resn_exch{" H82 ODD"}="2H8  ODD"; $resn_exch{" H91 ODD"}="1H9  ODD";
$resn_exch{" H92 ODD"}="2H9  ODD"; $resn_exch{"H141 ODD"}="1H14 ODD"; $resn_exch{"H142 ODD"}="2H14 ODD";
$resn_exch{"H151 ODD"}="1H15 ODD"; $resn_exch{"H152 ODD"}="2H15 ODD"; $resn_exch{"H161 ODD"}="1H16 ODD";
$resn_exch{"H162 ODD"}="2H16 ODD"; $resn_exch{"H171 ODD"}="1H17 ODD"; $resn_exch{"H172 ODD"}="2H17 ODD";
$resn_exch{"H181 ODD"}="1H18 ODD"; $resn_exch{"H182 ODD"}="2H18 ODD"; $resn_exch{"H183 ODD"}="3H18 ODD";
$resn_exch{"HAJ1 ODE"}="1HAJ ODE"; $resn_exch{"HAJ2 ODE"}="2HAJ ODE"; $resn_exch{"HAU1 ODE"}="1HAU ODE";
$resn_exch{"HAU2 ODE"}="2HAU ODE"; $resn_exch{"HAU3 ODE"}="3HAU ODE"; $resn_exch{"HAS1 ODE"}="1HAS ODE";
$resn_exch{"HAS2 ODE"}="2HAS ODE"; $resn_exch{"HAG1 ODE"}="1HAG ODE"; $resn_exch{"HAG2 ODE"}="2HAG ODE";
$resn_exch{"HAG3 ODE"}="3HAG ODE"; $resn_exch{"HAC1 ODI"}="1HAC ODI"; $resn_exch{"HAC2 ODI"}="2HAC ODI";
$resn_exch{"HAE1 ODI"}="1HAE ODI"; $resn_exch{"HAE2 ODI"}="2HAE ODI"; $resn_exch{"HAG1 ODI"}="1HAG ODI";
$resn_exch{"HAG2 ODI"}="2HAG ODI"; $resn_exch{"HAI1 ODI"}="1HAI ODI"; $resn_exch{"HAI2 ODI"}="2HAI ODI";
$resn_exch{"HAJ1 ODI"}="1HAJ ODI"; $resn_exch{"HAJ2 ODI"}="2HAJ ODI"; $resn_exch{"HAH1 ODI"}="1HAH ODI";
$resn_exch{"HAH2 ODI"}="2HAH ODI"; $resn_exch{"HAF1 ODI"}="1HAF ODI"; $resn_exch{"HAF2 ODI"}="2HAF ODI";
$resn_exch{"HAD1 ODI"}="1HAD ODI"; $resn_exch{"HAD2 ODI"}="2HAD ODI"; $resn_exch{"HOP2 ODP"}="2HOP ODP";
$resn_exch{"HOP3 ODP"}="3HOP ODP"; $resn_exch{" H2B ODP"}="AH2* ODP"; $resn_exch{" H3B ODP"}="AH3* ODP";
$resn_exch{"HO3A ODP"}="AHO3 ODP"; $resn_exch{" H1B ODP"}="AH1* ODP"; $resn_exch{" H8A ODP"}=" AH8 ODP";
$resn_exch{" H2A ODP"}=" AH2 ODP"; $resn_exch{"H62A ODP"}="AH62 ODP"; $resn_exch{"H61A ODP"}="AH61 ODP";
$resn_exch{" H4B ODP"}="AH4* ODP"; $resn_exch{"H51A ODP"}="AH51 ODP"; $resn_exch{"H52A ODP"}="AH52 ODP";
$resn_exch{"HOA2 ODP"}="2HOA ODP"; $resn_exch{"H51N ODP"}="NH51 ODP"; $resn_exch{"H52N ODP"}="NH52 ODP";
$resn_exch{" H4D ODP"}="NH4* ODP"; $resn_exch{" H1D ODP"}="NH1* ODP"; $resn_exch{" H2D ODP"}="NH2* ODP";
$resn_exch{"HO2N ODP"}="NHO2 ODP"; $resn_exch{" H3D ODP"}="NH3* ODP"; $resn_exch{"HO3N ODP"}="NHO3 ODP";
$resn_exch{" H2N ODP"}=" NH2 ODP"; $resn_exch{" H3N ODP"}=" NH3 ODP"; $resn_exch{"H72N ODP"}="NH72 ODP";
$resn_exch{"H71N ODP"}="NH71 ODP"; $resn_exch{" H5N ODP"}=" NH5 ODP"; $resn_exch{" H6N ODP"}=" NH6 ODP";
$resn_exch{" H21 ODS"}="1H2  ODS"; $resn_exch{" H22 ODS"}="2H2  ODS"; $resn_exch{" H31 ODS"}="1H3  ODS";
$resn_exch{" H32 ODS"}="2H3  ODS"; $resn_exch{" HM1 ODS"}="1HM  ODS"; $resn_exch{" HM2 ODS"}="2HM  ODS";
$resn_exch{" HM3 ODS"}="3HM  ODS"; $resn_exch{" H51 ODS"}="1H5  ODS"; $resn_exch{" H52 ODS"}="2H5  ODS";
$resn_exch{" H61 ODS"}="1H6  ODS"; $resn_exch{" H62 ODS"}="2H6  ODS"; $resn_exch{" H21 ODT"}="1H2  ODT";
$resn_exch{" H22 ODT"}="2H2  ODT"; $resn_exch{" H31 ODT"}="1H3  ODT"; $resn_exch{" H32 ODT"}="2H3  ODT";
$resn_exch{" H41 ODT"}="1H4  ODT"; $resn_exch{" H42 ODT"}="2H4  ODT"; $resn_exch{" H51 ODT"}="1H5  ODT";
$resn_exch{" H52 ODT"}="2H5  ODT"; $resn_exch{" H61 ODT"}="1H6  ODT"; $resn_exch{" H62 ODT"}="2H6  ODT";
$resn_exch{" H71 ODT"}="1H7  ODT"; $resn_exch{" H72 ODT"}="2H7  ODT"; $resn_exch{" H81 ODT"}="1H8  ODT";
$resn_exch{" H82 ODT"}="2H8  ODT"; $resn_exch{" H91 ODT"}="1H9  ODT"; $resn_exch{" H92 ODT"}="2H9  ODT";
$resn_exch{"H101 ODT"}="1H10 ODT"; $resn_exch{"H102 ODT"}="2H10 ODT"; $resn_exch{"H171 ODT"}="1H17 ODT";
$resn_exch{"H172 ODT"}="2H17 ODT"; $resn_exch{"H181 ODT"}="1H18 ODT"; $resn_exch{"H182 ODT"}="2H18 ODT";
$resn_exch{"H183 ODT"}="3H18 ODT"; $resn_exch{"H131 OEF"}="1H13 OEF"; $resn_exch{"H132 OEF"}="2H13 OEF";
$resn_exch{"H141 OEF"}="1H14 OEF"; $resn_exch{"H142 OEF"}="2H14 OEF"; $resn_exch{"H161 OEF"}="1H16 OEF";
$resn_exch{"H162 OEF"}="2H16 OEF"; $resn_exch{"H163 OEF"}="3H16 OEF"; $resn_exch{"H171 OEF"}="1H17 OEF";
$resn_exch{"H172 OEF"}="2H17 OEF"; $resn_exch{"H173 OEF"}="3H17 OEF"; $resn_exch{"HC11 OES"}="1HC1 OES";
$resn_exch{"HC12 OES"}="2HC1 OES"; $resn_exch{"HC13 OES"}="3HC1 OES"; $resn_exch{"HC21 OES"}="1HC2 OES";
$resn_exch{"HC22 OES"}="2HC2 OES"; $resn_exch{"HC31 OES"}="1HC3 OES"; $resn_exch{"HC32 OES"}="2HC3 OES";
$resn_exch{"HC41 OES"}="1HC4 OES"; $resn_exch{"HC42 OES"}="2HC4 OES"; $resn_exch{"HC51 OES"}="1HC5 OES";
$resn_exch{"HC52 OES"}="2HC5 OES"; $resn_exch{"HC61 OES"}="1HC6 OES"; $resn_exch{"HC62 OES"}="2HC6 OES";
$resn_exch{"HC71 OES"}="1HC7 OES"; $resn_exch{"HC72 OES"}="2HC7 OES"; $resn_exch{"HC81 OES"}="1HC8 OES";
$resn_exch{"HC82 OES"}="2HC8 OES"; $resn_exch{"H221 OES"}="1H22 OES"; $resn_exch{"H222 OES"}="2H22 OES";
$resn_exch{"H231 OES"}="1H23 OES"; $resn_exch{"H232 OES"}="2H23 OES"; $resn_exch{"HO31 OES"}="1HO3 OES";
$resn_exch{" H11 OET"}="1H1  OET"; $resn_exch{" H12 OET"}="2H1  OET"; $resn_exch{" H21 OET"}="1H2  OET";
$resn_exch{" H22 OET"}="2H2  OET"; $resn_exch{" H23 OET"}="3H2  OET"; $resn_exch{"HO11 OF1"}="1HO1 OF1";
$resn_exch{"HO12 OF1"}="2HO1 OF1"; $resn_exch{"HO11 OF3"}="1HO1 OF3"; $resn_exch{"HO12 OF3"}="2HO1 OF3";
$resn_exch{"H171 OFF"}="1H17 OFF"; $resn_exch{"H161 OFF"}="1H16 OFF"; $resn_exch{"H131 OFF"}="1H13 OFF";
$resn_exch{"H132 OFF"}="2H13 OFF"; $resn_exch{" H81 OFF"}="1H8  OFF"; $resn_exch{" H71 OFF"}="1H7  OFF";
$resn_exch{"H4C1 OGA"}="1H4C OGA"; $resn_exch{"H4C2 OGA"}="2H4C OGA"; $resn_exch{"HNB1 OGG"}="1HNB OGG";
$resn_exch{"HNB2 OGG"}="2HNB OGG"; $resn_exch{" HD1 OGG"}="1HD  OGG"; $resn_exch{" HD2 OGG"}="2HD  OGG";
$resn_exch{" HF1 OGG"}="1HF  OGG"; $resn_exch{" HF2 OGG"}="2HF  OGG"; $resn_exch{" HG1 OGG"}="1HG  OGG";
$resn_exch{" HG2 OGG"}="2HG  OGG"; $resn_exch{" HI1 OGG"}="1HI  OGG"; $resn_exch{" HI2 OGG"}="2HI  OGG";
$resn_exch{"HNK1 OGG"}="1HNK OGG"; $resn_exch{"HNK2 OGG"}="2HNK OGG"; $resn_exch{"H151 OHA"}="1H15 OHA";
$resn_exch{"H152 OHA"}="2H15 OHA"; $resn_exch{"H141 OHA"}="1H14 OHA"; $resn_exch{"H142 OHA"}="2H14 OHA";
$resn_exch{"H131 OHA"}="1H13 OHA"; $resn_exch{"H132 OHA"}="2H13 OHA"; $resn_exch{"H121 OHA"}="1H12 OHA";
$resn_exch{"H122 OHA"}="2H12 OHA"; $resn_exch{"H111 OHA"}="1H11 OHA"; $resn_exch{"H112 OHA"}="2H11 OHA";
$resn_exch{"H101 OHA"}="1H10 OHA"; $resn_exch{"H102 OHA"}="2H10 OHA"; $resn_exch{" H91 OHA"}="1H9  OHA";
$resn_exch{" H92 OHA"}="2H9  OHA"; $resn_exch{" H81 OHA"}="1H8  OHA"; $resn_exch{" H82 OHA"}="2H8  OHA";
$resn_exch{" H61 OHA"}="1H6  OHA"; $resn_exch{" H62 OHA"}="2H6  OHA"; $resn_exch{" H51 OHA"}="1H5  OHA";
$resn_exch{" H52 OHA"}="2H5  OHA"; $resn_exch{" H41 OHA"}="1H4  OHA"; $resn_exch{" H42 OHA"}="2H4  OHA";
$resn_exch{" H31 OHA"}="1H3  OHA"; $resn_exch{" H32 OHA"}="2H3  OHA"; $resn_exch{" H21 OHA"}="1H2  OHA";
$resn_exch{" H22 OHA"}="2H2  OHA"; $resn_exch{" H11 OHA"}="1H1  OHA"; $resn_exch{" H12 OHA"}="2H1  OHA";
$resn_exch{" H13 OHA"}="3H1  OHA"; $resn_exch{" H11 OHE"}="1H1  OHE"; $resn_exch{" H12 OHE"}="2H1  OHE";
$resn_exch{" H13 OHE"}="3H1  OHE"; $resn_exch{" H21 OHE"}="1H2  OHE"; $resn_exch{" H22 OHE"}="2H2  OHE";
$resn_exch{"H201 OHH"}="1H20 OHH"; $resn_exch{"H202 OHH"}="2H20 OHH"; $resn_exch{"H203 OHH"}="3H20 OHH";
$resn_exch{"H241 OHH"}="1H24 OHH"; $resn_exch{"H242 OHH"}="2H24 OHH"; $resn_exch{"H231 OHH"}="1H23 OHH";
$resn_exch{"H232 OHH"}="2H23 OHH"; $resn_exch{"H221 OHH"}="1H22 OHH"; $resn_exch{"H222 OHH"}="2H22 OHH";
$resn_exch{"H251 OHH"}="1H25 OHH"; $resn_exch{"H252 OHH"}="2H25 OHH"; $resn_exch{" HB1 OHI"}="1HB  OHI";
$resn_exch{" HB2 OHI"}="2HB  OHI"; $resn_exch{" HN1 OHI"}="1HN  OHI"; $resn_exch{" HN2 OHI"}="2HN  OHI";
$resn_exch{"H211 OHM"}="1H21 OHM"; $resn_exch{"H212 OHM"}="2H21 OHM"; $resn_exch{"H213 OHM"}="3H21 OHM";
$resn_exch{"H201 OHM"}="1H20 OHM"; $resn_exch{"H202 OHM"}="2H20 OHM"; $resn_exch{"H191 OHM"}="1H19 OHM";
$resn_exch{"H192 OHM"}="2H19 OHM"; $resn_exch{"H181 OHM"}="1H18 OHM"; $resn_exch{"H182 OHM"}="2H18 OHM";
$resn_exch{"H171 OHM"}="1H17 OHM"; $resn_exch{"H172 OHM"}="2H17 OHM"; $resn_exch{"H161 OHM"}="1H16 OHM";
$resn_exch{"H162 OHM"}="2H16 OHM"; $resn_exch{"H151 OHM"}="1H15 OHM"; $resn_exch{"H152 OHM"}="2H15 OHM";
$resn_exch{"H141 OHM"}="1H14 OHM"; $resn_exch{"H142 OHM"}="2H14 OHM"; $resn_exch{"H131 OHM"}="1H13 OHM";
$resn_exch{"H132 OHM"}="2H13 OHM"; $resn_exch{"H101 OHM"}="1H10 OHM"; $resn_exch{"H102 OHM"}="2H10 OHM";
$resn_exch{" H51 OHM"}="1H5  OHM"; $resn_exch{" H52 OHM"}="2H5  OHM"; $resn_exch{" H41 OHM"}="1H4  OHM";
$resn_exch{" H42 OHM"}="2H4  OHM"; $resn_exch{"H211 OHN"}="1H21 OHN"; $resn_exch{"H212 OHN"}="2H21 OHN";
$resn_exch{"H213 OHN"}="3H21 OHN"; $resn_exch{"H201 OHN"}="1H20 OHN"; $resn_exch{"H202 OHN"}="2H20 OHN";
$resn_exch{"H191 OHN"}="1H19 OHN"; $resn_exch{"H192 OHN"}="2H19 OHN"; $resn_exch{"H181 OHN"}="1H18 OHN";
$resn_exch{"H182 OHN"}="2H18 OHN"; $resn_exch{"H171 OHN"}="1H17 OHN"; $resn_exch{"H172 OHN"}="2H17 OHN";
$resn_exch{"H161 OHN"}="1H16 OHN"; $resn_exch{"H162 OHN"}="2H16 OHN"; $resn_exch{"H151 OHN"}="1H15 OHN";
$resn_exch{"H152 OHN"}="2H15 OHN"; $resn_exch{"H141 OHN"}="1H14 OHN"; $resn_exch{"H142 OHN"}="2H14 OHN";
$resn_exch{"H131 OHN"}="1H13 OHN"; $resn_exch{"H132 OHN"}="2H13 OHN"; $resn_exch{"H101 OHN"}="1H10 OHN";
$resn_exch{"H102 OHN"}="2H10 OHN"; $resn_exch{"H5C1 OHN"}="1H5C OHN"; $resn_exch{"H5C2 OHN"}="2H5C OHN";
$resn_exch{"H4C1 OHN"}="1H4C OHN"; $resn_exch{"H4C2 OHN"}="2H4C OHN"; $resn_exch{" H21 OHO"}="1H2  OHO";
$resn_exch{" H22 OHO"}="2H2  OHO"; $resn_exch{" H31 OHO"}="1H3  OHO"; $resn_exch{" H32 OHO"}="2H3  OHO";
$resn_exch{" H41 OHO"}="1H4  OHO"; $resn_exch{" H42 OHO"}="2H4  OHO"; $resn_exch{" H51 OHO"}="1H5  OHO";
$resn_exch{" H52 OHO"}="2H5  OHO"; $resn_exch{" H61 OHO"}="1H6  OHO"; $resn_exch{" H62 OHO"}="2H6  OHO";
$resn_exch{" H71 OHP"}="1H7  OHP"; $resn_exch{" H72 OHP"}="2H7  OHP"; $resn_exch{" HA1 OHS"}="1HA  OHS";
$resn_exch{" H2  OHS"}=" HAC OHS"; $resn_exch{" H   OHS"}=" HAA OHS"; $resn_exch{" HB2 OHS"}="1HB  OHS";
$resn_exch{" HB3 OHS"}="2HB  OHS"; $resn_exch{" HXT OHS"}=" HO1 OHS"; $resn_exch{" HZ  OHS"}=" HO2 OHS";
$resn_exch{"H101 OHT"}="1H10 OHT"; $resn_exch{"H102 OHT"}="2H10 OHT"; $resn_exch{"H103 OHT"}="3H10 OHT";
$resn_exch{" H91 OHT"}="1H9  OHT"; $resn_exch{" H92 OHT"}="2H9  OHT"; $resn_exch{"H231 OHT"}="1H23 OHT";
$resn_exch{"H232 OHT"}="2H23 OHT"; $resn_exch{"H241 OHT"}="1H24 OHT"; $resn_exch{"H242 OHT"}="2H24 OHT";
$resn_exch{"H251 OHT"}="1H25 OHT"; $resn_exch{"H252 OHT"}="2H25 OHT"; $resn_exch{"H253 OHT"}="3H25 OHT";
$resn_exch{"H261 OHT"}="1H26 OHT"; $resn_exch{"H262 OHT"}="2H26 OHT"; $resn_exch{"H263 OHT"}="3H26 OHT";
$resn_exch{" H71 OI1"}="1H7  OI1"; $resn_exch{" H72 OI1"}="2H7  OI1"; $resn_exch{"H141 OI1"}="1H14 OI1";
$resn_exch{"H142 OI1"}="2H14 OI1"; $resn_exch{"H151 OI1"}="1H15 OI1"; $resn_exch{"H152 OI1"}="2H15 OI1";
$resn_exch{"H221 OI1"}="1H22 OI1"; $resn_exch{"H222 OI1"}="2H22 OI1"; $resn_exch{"H231 OI1"}="1H23 OI1";
$resn_exch{"H232 OI1"}="2H23 OI1"; $resn_exch{"H241 OI1"}="1H24 OI1"; $resn_exch{"H242 OI1"}="2H24 OI1";
$resn_exch{"H243 OI1"}="3H24 OI1"; $resn_exch{"H251 OI1"}="1H25 OI1"; $resn_exch{"H252 OI1"}="2H25 OI1";
$resn_exch{"H321 OI1"}="1H32 OI1"; $resn_exch{"H322 OI1"}="2H32 OI1"; $resn_exch{" H   OIC"}=" HN  OIC";
$resn_exch{" HB2 OIC"}="1HB  OIC"; $resn_exch{" HB3 OIC"}="2HB  OIC"; $resn_exch{" H41 OIC"}="1H4  OIC";
$resn_exch{" H42 OIC"}="2H4  OIC"; $resn_exch{" H51 OIC"}="1H5  OIC"; $resn_exch{" H52 OIC"}="2H5  OIC";
$resn_exch{" H61 OIC"}="1H6  OIC"; $resn_exch{" H62 OIC"}="2H6  OIC"; $resn_exch{" H71 OIC"}="1H7  OIC";
$resn_exch{" H72 OIC"}="2H7  OIC"; $resn_exch{"H101 OIN"}="1H10 OIN"; $resn_exch{"H102 OIN"}="2H10 OIN";
$resn_exch{"H103 OIN"}="3H10 OIN"; $resn_exch{"H171 OIN"}="1H17 OIN"; $resn_exch{"H172 OIN"}="2H17 OIN";
$resn_exch{"H161 OIN"}="1H16 OIN"; $resn_exch{"H162 OIN"}="2H16 OIN"; $resn_exch{"H141 OIN"}="1H14 OIN";
$resn_exch{"H142 OIN"}="2H14 OIN"; $resn_exch{"H121 OIN"}="1H12 OIN"; $resn_exch{"H122 OIN"}="2H12 OIN";
$resn_exch{" H81 OIN"}="1H8  OIN"; $resn_exch{" H82 OIN"}="2H8  OIN"; $resn_exch{"HOP2 OIP"}="2HOP OIP";
$resn_exch{"HOP3 OIP"}="3HOP OIP"; $resn_exch{" H5' OIP"}="1H5* OIP"; $resn_exch{"H5'' OIP"}="2H5* OIP";
$resn_exch{" H4' OIP"}=" H4* OIP"; $resn_exch{" H3' OIP"}=" H3* OIP"; $resn_exch{"HO3' OIP"}="*HO3 OIP";
$resn_exch{" H2' OIP"}="1H2* OIP"; $resn_exch{"H2'' OIP"}="2H2* OIP"; $resn_exch{" H1' OIP"}=" H1* OIP";
$resn_exch{"H211 OIR"}="1H21 OIR"; $resn_exch{"H212 OIR"}="2H21 OIR"; $resn_exch{"H191 OIR"}="1H19 OIR";
$resn_exch{"H192 OIR"}="2H19 OIR"; $resn_exch{"H193 OIR"}="3H19 OIR"; $resn_exch{" H11 OIR"}="1H1  OIR";
$resn_exch{" H12 OIR"}="2H1  OIR"; $resn_exch{" H41 OIS"}="1H4  OIS"; $resn_exch{" H42 OIS"}="2H4  OIS";
$resn_exch{" H51 OIS"}="1H5  OIS"; $resn_exch{" H52 OIS"}="2H5  OIS"; $resn_exch{" H81 OIS"}="1H8  OIS";
$resn_exch{" H82 OIS"}="2H8  OIS"; $resn_exch{"H171 OIS"}="1H17 OIS"; $resn_exch{"H172 OIS"}="2H17 OIS";
$resn_exch{"H173 OIS"}="3H17 OIS"; $resn_exch{"H181 OIS"}="1H18 OIS"; $resn_exch{"H182 OIS"}="2H18 OIS";
$resn_exch{"H183 OIS"}="3H18 OIS"; $resn_exch{"H221 OIS"}="1H22 OIS"; $resn_exch{"H222 OIS"}="2H22 OIS";
$resn_exch{"H223 OIS"}="3H22 OIS"; $resn_exch{"H231 OIS"}="1H23 OIS"; $resn_exch{"H232 OIS"}="2H23 OIS";
$resn_exch{"H233 OIS"}="3H23 OIS"; $resn_exch{"H402 OIS"}="2H40 OIS"; $resn_exch{"H101 OIS"}="1H10 OIS";
$resn_exch{"H102 OIS"}="2H10 OIS"; $resn_exch{"H251 OIS"}="1H25 OIS"; $resn_exch{"H252 OIS"}="2H25 OIS";
$resn_exch{"H253 OIS"}="3H25 OIS"; $resn_exch{" H31 OKA"}="1H3  OKA"; $resn_exch{" H32 OKA"}="2H3  OKA";
$resn_exch{" H41 OKA"}="1H4  OKA"; $resn_exch{" H51 OKA"}="1H5  OKA"; $resn_exch{" H52 OKA"}="2H5  OKA";
$resn_exch{" H61 OKA"}="1H6  OKA"; $resn_exch{" H62 OKA"}="2H6  OKA"; $resn_exch{" H71 OKA"}="1H7  OKA";
$resn_exch{" H91 OKA"}="1H9  OKA"; $resn_exch{"H111 OKA"}="1H11 OKA"; $resn_exch{"H112 OKA"}="2H11 OKA";
$resn_exch{"H121 OKA"}="1H12 OKA"; $resn_exch{"H131 OKA"}="1H13 OKA"; $resn_exch{"H141 OKA"}="1H14 OKA";
$resn_exch{"H151 OKA"}="1H15 OKA"; $resn_exch{"H161 OKA"}="1H16 OKA"; $resn_exch{"H171 OKA"}="1H17 OKA";
$resn_exch{"H172 OKA"}="2H17 OKA"; $resn_exch{"H181 OKA"}="1H18 OKA"; $resn_exch{"H182 OKA"}="2H18 OKA";
$resn_exch{"H201 OKA"}="1H20 OKA"; $resn_exch{"H202 OKA"}="2H20 OKA"; $resn_exch{"H211 OKA"}="1H21 OKA";
$resn_exch{"H212 OKA"}="2H21 OKA"; $resn_exch{"H221 OKA"}="1H22 OKA"; $resn_exch{"H231 OKA"}="1H23 OKA";
$resn_exch{"H241 OKA"}="1H24 OKA"; $resn_exch{"H261 OKA"}="1H26 OKA"; $resn_exch{"H271 OKA"}="1H27 OKA";
$resn_exch{"H281 OKA"}="1H28 OKA"; $resn_exch{"H282 OKA"}="2H28 OKA"; $resn_exch{"H291 OKA"}="1H29 OKA";
$resn_exch{"H301 OKA"}="1H30 OKA"; $resn_exch{"H311 OKA"}="1H31 OKA"; $resn_exch{"H321 OKA"}="1H32 OKA";
$resn_exch{"H322 OKA"}="2H32 OKA"; $resn_exch{"H331 OKA"}="1H33 OKA"; $resn_exch{"H332 OKA"}="2H33 OKA";
$resn_exch{"H351 OKA"}="1H35 OKA"; $resn_exch{"H352 OKA"}="2H35 OKA"; $resn_exch{"H361 OKA"}="1H36 OKA";
$resn_exch{"H362 OKA"}="2H36 OKA"; $resn_exch{"H371 OKA"}="1H37 OKA"; $resn_exch{"H372 OKA"}="2H37 OKA";
$resn_exch{"H381 OKA"}="1H38 OKA"; $resn_exch{"H382 OKA"}="2H38 OKA"; $resn_exch{"H391 OKA"}="1H39 OKA";
$resn_exch{"H392 OKA"}="2H39 OKA"; $resn_exch{"H393 OKA"}="3H39 OKA"; $resn_exch{"H401 OKA"}="1H40 OKA";
$resn_exch{"H402 OKA"}="2H40 OKA"; $resn_exch{"H403 OKA"}="3H40 OKA"; $resn_exch{"H411 OKA"}="1H41 OKA";
$resn_exch{"H412 OKA"}="2H41 OKA"; $resn_exch{"H421 OKA"}="1H42 OKA"; $resn_exch{"H422 OKA"}="2H42 OKA";
$resn_exch{"H423 OKA"}="3H42 OKA"; $resn_exch{"H431 OKA"}="1H43 OKA"; $resn_exch{"H432 OKA"}="2H43 OKA";
$resn_exch{"H433 OKA"}="3H43 OKA"; $resn_exch{"H441 OKA"}="1H44 OKA"; $resn_exch{"H442 OKA"}="2H44 OKA";
$resn_exch{"H443 OKA"}="3H44 OKA"; $resn_exch{" H21 OLA"}="1H2  OLA"; $resn_exch{" H22 OLA"}="2H2  OLA";
$resn_exch{" H31 OLA"}="1H3  OLA"; $resn_exch{" H32 OLA"}="2H3  OLA"; $resn_exch{" H41 OLA"}="1H4  OLA";
$resn_exch{" H42 OLA"}="2H4  OLA"; $resn_exch{" H51 OLA"}="1H5  OLA"; $resn_exch{" H52 OLA"}="2H5  OLA";
$resn_exch{" H61 OLA"}="1H6  OLA"; $resn_exch{" H62 OLA"}="2H6  OLA"; $resn_exch{" H71 OLA"}="1H7  OLA";
$resn_exch{" H72 OLA"}="2H7  OLA"; $resn_exch{" H81 OLA"}="1H8  OLA"; $resn_exch{" H82 OLA"}="2H8  OLA";
$resn_exch{"H111 OLA"}="1H11 OLA"; $resn_exch{"H112 OLA"}="2H11 OLA"; $resn_exch{"H121 OLA"}="1H12 OLA";
$resn_exch{"H122 OLA"}="2H12 OLA"; $resn_exch{"H131 OLA"}="1H13 OLA"; $resn_exch{"H132 OLA"}="2H13 OLA";
$resn_exch{"H141 OLA"}="1H14 OLA"; $resn_exch{"H142 OLA"}="2H14 OLA"; $resn_exch{"H151 OLA"}="1H15 OLA";
$resn_exch{"H152 OLA"}="2H15 OLA"; $resn_exch{"H161 OLA"}="1H16 OLA"; $resn_exch{"H162 OLA"}="2H16 OLA";
$resn_exch{"H171 OLA"}="1H17 OLA"; $resn_exch{"H172 OLA"}="2H17 OLA"; $resn_exch{"H181 OLA"}="1H18 OLA";
$resn_exch{"H182 OLA"}="2H18 OLA"; $resn_exch{"H183 OLA"}="3H18 OLA"; $resn_exch{" HB1 OLE"}="1HB  OLE";
$resn_exch{" HB2 OLE"}="2HB  OLE"; $resn_exch{"HD11 OLE"}="1HD1 OLE"; $resn_exch{"HD12 OLE"}="2HD1 OLE";
$resn_exch{"HD13 OLE"}="3HD1 OLE"; $resn_exch{"HD21 OLE"}="1HD2 OLE"; $resn_exch{"HD22 OLE"}="2HD2 OLE";
$resn_exch{"HD23 OLE"}="3HD2 OLE"; $resn_exch{" H21 OLI"}="1H2  OLI"; $resn_exch{" H22 OLI"}="2H2  OLI";
$resn_exch{" H31 OLI"}="1H3  OLI"; $resn_exch{" H32 OLI"}="2H3  OLI"; $resn_exch{" H41 OLI"}="1H4  OLI";
$resn_exch{" H42 OLI"}="2H4  OLI"; $resn_exch{" H51 OLI"}="1H5  OLI"; $resn_exch{" H52 OLI"}="2H5  OLI";
$resn_exch{" H61 OLI"}="1H6  OLI"; $resn_exch{" H62 OLI"}="2H6  OLI"; $resn_exch{" H71 OLI"}="1H7  OLI";
$resn_exch{" H72 OLI"}="2H7  OLI"; $resn_exch{" H81 OLI"}="1H8  OLI"; $resn_exch{" H82 OLI"}="2H8  OLI";
$resn_exch{"H111 OLI"}="1H11 OLI"; $resn_exch{"H112 OLI"}="2H11 OLI"; $resn_exch{"H121 OLI"}="1H12 OLI";
$resn_exch{"H122 OLI"}="2H12 OLI"; $resn_exch{"H131 OLI"}="1H13 OLI"; $resn_exch{"H132 OLI"}="2H13 OLI";
$resn_exch{"H141 OLI"}="1H14 OLI"; $resn_exch{"H142 OLI"}="2H14 OLI"; $resn_exch{"H151 OLI"}="1H15 OLI";
$resn_exch{"H152 OLI"}="2H15 OLI"; $resn_exch{"H161 OLI"}="1H16 OLI"; $resn_exch{"H162 OLI"}="2H16 OLI";
$resn_exch{"H171 OLI"}="1H17 OLI"; $resn_exch{"H172 OLI"}="2H17 OLI"; $resn_exch{"H181 OLI"}="1H18 OLI";
$resn_exch{"H182 OLI"}="2H18 OLI"; $resn_exch{"H183 OLI"}="3H18 OLI"; $resn_exch{" HB1 OLN"}="1HB  OLN";
$resn_exch{" HB2 OLN"}="2HB  OLN"; $resn_exch{" HG1 OLN"}="1HG  OLN"; $resn_exch{" HG2 OLN"}="2HG  OLN";
$resn_exch{" HD1 OLN"}="1HD  OLN"; $resn_exch{" HD2 OLN"}="2HD  OLN"; $resn_exch{"HH21 OLN"}="1HH2 OLN";
$resn_exch{"HH22 OLN"}="2HH2 OLN"; $resn_exch{" H21 OLN"}="1H2  OLN"; $resn_exch{" H22 OLN"}="2H2  OLN";
$resn_exch{" H23 OLN"}="3H2  OLN"; $resn_exch{" H31 OLO"}="1H3  OLO"; $resn_exch{" H32 OLO"}="2H3  OLO";
$resn_exch{" H33 OLO"}="3H3  OLO"; $resn_exch{" H41 OLO"}="1H4  OLO"; $resn_exch{" H42 OLO"}="2H4  OLO";
$resn_exch{" H51 OLO"}="1H5  OLO"; $resn_exch{" H52 OLO"}="2H5  OLO"; $resn_exch{" H61 OLO"}="1H6  OLO";
$resn_exch{" H62 OLO"}="2H6  OLO"; $resn_exch{"HAB1 OLP"}="1HAB OLP"; $resn_exch{"HAB2 OLP"}="2HAB OLP";
$resn_exch{"HAB3 OLP"}="3HAB OLP"; $resn_exch{"HAA1 OLP"}="1HAA OLP"; $resn_exch{"HAA2 OLP"}="2HAA OLP";
$resn_exch{"HAA3 OLP"}="3HAA OLP"; $resn_exch{"HAK1 OLP"}="1HAK OLP"; $resn_exch{"HAK2 OLP"}="2HAK OLP";
$resn_exch{"HAJ1 OLP"}="1HAJ OLP"; $resn_exch{"HAJ2 OLP"}="2HAJ OLP"; $resn_exch{" H   OLT"}="1HN  OLT";
$resn_exch{" H2  OLT"}="2HN  OLT"; $resn_exch{" HB  OLT"}=" HCB OLT"; $resn_exch{"HG21 OLT"}="1HG2 OLT";
$resn_exch{"HG22 OLT"}="2HG2 OLT"; $resn_exch{"HG23 OLT"}="3HG2 OLT"; $resn_exch{"HD11 OLT"}="1HD1 OLT";
$resn_exch{"HD12 OLT"}="2HD1 OLT"; $resn_exch{"HD13 OLT"}="3HD1 OLT"; $resn_exch{"H5'1 OLU"}="1H5* OLU";
$resn_exch{"H5'2 OLU"}="2H5* OLU"; $resn_exch{" HC1 OMB"}="1HC  OMB"; $resn_exch{" HC2 OMB"}="2HC  OMB";
$resn_exch{"HN41 OMC"}="1HN4 OMC"; $resn_exch{"HN42 OMC"}="2HN4 OMC"; $resn_exch{" H1' OMC"}=" H1* OMC";
$resn_exch{" H2' OMC"}=" H2* OMC"; $resn_exch{"HM21 OMC"}="1HM2 OMC"; $resn_exch{"HM22 OMC"}="2HM2 OMC";
$resn_exch{"HM23 OMC"}="3HM2 OMC"; $resn_exch{" H3' OMC"}=" H3* OMC"; $resn_exch{" H4' OMC"}=" H4* OMC";
$resn_exch{"HO3' OMC"}="*HO3 OMC"; $resn_exch{" H5' OMC"}="1H5* OMC"; $resn_exch{"H5'' OMC"}="2H5* OMC";
$resn_exch{"HOP2 OMC"}="2HOP OMC"; $resn_exch{"HOP3 OMC"}="3HOP OMC"; $resn_exch{" H21 OMD"}="1H2  OMD";
$resn_exch{" H22 OMD"}="2H2  OMD"; $resn_exch{" H11 OME"}="1H1  OME"; $resn_exch{" H12 OME"}="2H1  OME";
$resn_exch{" H13 OME"}="3H1  OME"; $resn_exch{"HOP2 OMG"}="2HOP OMG"; $resn_exch{"HOP3 OMG"}="3HOP OMG";
$resn_exch{" H5' OMG"}="1H5* OMG"; $resn_exch{"H5'' OMG"}="2H5* OMG"; $resn_exch{" H4' OMG"}=" H4* OMG";
$resn_exch{" H3' OMG"}=" H3* OMG"; $resn_exch{"HO3' OMG"}="*HO3 OMG"; $resn_exch{" H2' OMG"}=" H2* OMG";
$resn_exch{"HM21 OMG"}="1HM2 OMG"; $resn_exch{"HM22 OMG"}="2HM2 OMG"; $resn_exch{"HM23 OMG"}="3HM2 OMG";
$resn_exch{" H1' OMG"}=" H1* OMG"; $resn_exch{"HN21 OMG"}="1HN2 OMG"; $resn_exch{"HN22 OMG"}="2HN2 OMG";
$resn_exch{"H11A OMN"}="1H1  OMN"; $resn_exch{" H12 OMN"}="2H1  OMN"; $resn_exch{" H13 OMN"}="3H1  OMN";
$resn_exch{"HOP2 OMP"}="2HOP OMP"; $resn_exch{"HOP3 OMP"}="3HOP OMP"; $resn_exch{" H1' OMP"}=" H1* OMP";
$resn_exch{" H2' OMP"}=" H2* OMP"; $resn_exch{" H3' OMP"}=" H3* OMP"; $resn_exch{" H4' OMP"}=" H4* OMP";
$resn_exch{"HO2' OMP"}="*HO2 OMP"; $resn_exch{"HO3' OMP"}="*HO3 OMP"; $resn_exch{"H5'1 OMP"}="1H5* OMP";
$resn_exch{"H5'2 OMP"}="2H5* OMP"; $resn_exch{" H   OMT"}="1HN  OMT"; $resn_exch{" H2  OMT"}="2HN  OMT";
$resn_exch{" HB2 OMT"}="1HB  OMT"; $resn_exch{" HB3 OMT"}="2HB  OMT"; $resn_exch{" HG2 OMT"}="1HG  OMT";
$resn_exch{" HG3 OMT"}="2HG  OMT"; $resn_exch{" HE1 OMT"}="1HE  OMT"; $resn_exch{" HE2 OMT"}="2HE  OMT";
$resn_exch{" HE3 OMT"}="3HE  OMT"; $resn_exch{" H1' OMU"}=" H1* OMU"; $resn_exch{" H2' OMU"}=" H2* OMU";
$resn_exch{"HM21 OMU"}="1HM2 OMU"; $resn_exch{"HM22 OMU"}="2HM2 OMU"; $resn_exch{"HM23 OMU"}="3HM2 OMU";
$resn_exch{" H3' OMU"}=" H3* OMU"; $resn_exch{" H4' OMU"}=" H4* OMU"; $resn_exch{"HO3' OMU"}="*HO3 OMU";
$resn_exch{" H5' OMU"}="1H5* OMU"; $resn_exch{"H5'' OMU"}="2H5* OMU"; $resn_exch{"HOP2 OMU"}="2HOP OMU";
$resn_exch{"HOP3 OMU"}="3HOP OMU"; $resn_exch{"HA71 ONA"}="1HA7 ONA"; $resn_exch{"HA72 ONA"}="2HA7 ONA";
$resn_exch{"HA73 ONA"}="3HA7 ONA"; $resn_exch{" H3' ONA"}=" H3* ONA"; $resn_exch{" H2' ONA"}=" H2* ONA";
$resn_exch{" H4' ONA"}=" H4* ONA"; $resn_exch{"H5'1 ONA"}="1H5* ONA"; $resn_exch{"H5'2 ONA"}="2H5* ONA";
$resn_exch{" H1' ONA"}=" H1* ONA"; $resn_exch{"HN61 ONA"}="1HN6 ONA"; $resn_exch{"HN62 ONA"}="2HN6 ONA";
$resn_exch{"HOP3 ONE"}="3HOP ONE"; $resn_exch{"HOP2 ONE"}="2HOP ONE"; $resn_exch{"H5'' ONE"}="2H5* ONE";
$resn_exch{" H5' ONE"}="1H5* ONE"; $resn_exch{" H4' ONE"}=" H4* ONE"; $resn_exch{" H3' ONE"}=" H3* ONE";
$resn_exch{" H2' ONE"}=" H2* ONE"; $resn_exch{" HO2 ONE"}="2HO  ONE"; $resn_exch{" HO3 ONE"}="3HO  ONE";
$resn_exch{" H1' ONE"}=" H1* ONE"; $resn_exch{"HA71 ONM"}="1HA7 ONM"; $resn_exch{"HA72 ONM"}="2HA7 ONM";
$resn_exch{"HA73 ONM"}="3HA7 ONM"; $resn_exch{" H3' ONM"}=" H3* ONM"; $resn_exch{" H2' ONM"}=" H2* ONM";
$resn_exch{" H4' ONM"}=" H4* ONM"; $resn_exch{"H5'1 ONM"}="1H5* ONM"; $resn_exch{"H5'2 ONM"}="2H5* ONM";
$resn_exch{" H1' ONM"}=" H1* ONM"; $resn_exch{"HN21 ONM"}="1HN2 ONM"; $resn_exch{"HN22 ONM"}="2HN2 ONM";
$resn_exch{"H621 ONO"}="1H62 ONO"; $resn_exch{"H622 ONO"}="2H62 ONO"; $resn_exch{"H623 ONO"}="3H62 ONO";
$resn_exch{"H371 ONO"}="1H37 ONO"; $resn_exch{"H372 ONO"}="2H37 ONO"; $resn_exch{"H373 ONO"}="3H37 ONO";
$resn_exch{"H361 ONO"}="1H36 ONO"; $resn_exch{"H362 ONO"}="2H36 ONO"; $resn_exch{"H363 ONO"}="3H36 ONO";
$resn_exch{"H581 ONO"}="1H58 ONO"; $resn_exch{"H582 ONO"}="2H58 ONO"; $resn_exch{"H583 ONO"}="3H58 ONO";
$resn_exch{"H101 ONO"}="1H10 ONO"; $resn_exch{"H102 ONO"}="2H10 ONO"; $resn_exch{"HN91 ONO"}="1HN9 ONO";
$resn_exch{"HN92 ONO"}="2HN9 ONO"; $resn_exch{"HOB2 ONP"}="2HOB ONP"; $resn_exch{"HOA2 ONP"}="2HOA ONP";
$resn_exch{"HNA1 ONP"}="1HNA ONP"; $resn_exch{"HA21 ONP"}="1HA2 ONP"; $resn_exch{"HA22 ONP"}="2HA2 ONP";
$resn_exch{"HA11 ONP"}="1HA1 ONP"; $resn_exch{"HA12 ONP"}="2HA1 ONP"; $resn_exch{"H2C1 OOA"}="1H2C OOA";
$resn_exch{"H2C2 OOA"}="2H2C OOA"; $resn_exch{"H4C1 OOA"}="1H4C OOA"; $resn_exch{"H4C2 OOA"}="2H4C OOA";
$resn_exch{"H5C1 OOA"}="1H5C OOA"; $resn_exch{"H5C2 OOA"}="2H5C OOA"; $resn_exch{"H6C1 OOA"}="1H6C OOA";
$resn_exch{"H6C2 OOA"}="2H6C OOA"; $resn_exch{"H7C1 OOA"}="1H7C OOA"; $resn_exch{"H7C2 OOA"}="2H7C OOA";
$resn_exch{"H8C1 OOA"}="1H8C OOA"; $resn_exch{"H8C2 OOA"}="2H8C OOA"; $resn_exch{"H8C3 OOA"}="3H8C OOA";
$resn_exch{" H21 OPA"}="1H2  OPA"; $resn_exch{"H22A OPA"}="2H2  OPA"; $resn_exch{" H51 OPA"}="1H5  OPA";
$resn_exch{" H52 OPA"}="2H5  OPA"; $resn_exch{" H61 OPA"}="1H6  OPA"; $resn_exch{" H62 OPA"}="2H6  OPA";
$resn_exch{"H211 OPB"}="1H21 OPB"; $resn_exch{"H212 OPB"}="2H21 OPB"; $resn_exch{"H213 OPB"}="3H21 OPB";
$resn_exch{"H201 OPB"}="1H20 OPB"; $resn_exch{"H202 OPB"}="2H20 OPB"; $resn_exch{"H191 OPB"}="1H19 OPB";
$resn_exch{"H192 OPB"}="2H19 OPB"; $resn_exch{"H181 OPB"}="1H18 OPB"; $resn_exch{"H182 OPB"}="2H18 OPB";
$resn_exch{"HBF1 OPC"}="1HBF OPC"; $resn_exch{"HBF2 OPC"}="2HBF OPC"; $resn_exch{"HBF3 OPC"}="3HBF OPC";
$resn_exch{"HBE1 OPC"}="1HBE OPC"; $resn_exch{"HBE2 OPC"}="2HBE OPC"; $resn_exch{"HBD1 OPC"}="1HBD OPC";
$resn_exch{"HBD2 OPC"}="2HBD OPC"; $resn_exch{"HBC1 OPC"}="1HBC OPC"; $resn_exch{"HBC2 OPC"}="2HBC OPC";
$resn_exch{"HBB1 OPC"}="1HBB OPC"; $resn_exch{"HBB2 OPC"}="2HBB OPC"; $resn_exch{"HBA1 OPC"}="1HBA OPC";
$resn_exch{"HBA2 OPC"}="2HBA OPC"; $resn_exch{"HAZ1 OPC"}="1HAZ OPC"; $resn_exch{"HAZ2 OPC"}="2HAZ OPC";
$resn_exch{"HAY1 OPC"}="1HAY OPC"; $resn_exch{"HAY2 OPC"}="2HAY OPC"; $resn_exch{"HAX1 OPC"}="1HAX OPC";
$resn_exch{"HAX2 OPC"}="2HAX OPC"; $resn_exch{"HAU1 OPC"}="1HAU OPC"; $resn_exch{"HAU2 OPC"}="2HAU OPC";
$resn_exch{"HAT1 OPC"}="1HAT OPC"; $resn_exch{"HAT2 OPC"}="2HAT OPC"; $resn_exch{"HAS1 OPC"}="1HAS OPC";
$resn_exch{"HAS2 OPC"}="2HAS OPC"; $resn_exch{"HAR1 OPC"}="1HAR OPC"; $resn_exch{"HAR2 OPC"}="2HAR OPC";
$resn_exch{"HAQ1 OPC"}="1HAQ OPC"; $resn_exch{"HAQ2 OPC"}="2HAQ OPC"; $resn_exch{"HAP1 OPC"}="1HAP OPC";
$resn_exch{"HAP2 OPC"}="2HAP OPC"; $resn_exch{"HAL1 OPC"}="1HAL OPC"; $resn_exch{"HAL2 OPC"}="2HAL OPC";
$resn_exch{"HAH1 OPC"}="1HAH OPC"; $resn_exch{"HAH2 OPC"}="2HAH OPC"; $resn_exch{"HAG1 OPC"}="1HAG OPC";
$resn_exch{"HAG2 OPC"}="2HAG OPC"; $resn_exch{"HAE1 OPC"}="1HAE OPC"; $resn_exch{"HAE2 OPC"}="2HAE OPC";
$resn_exch{"HAE3 OPC"}="3HAE OPC"; $resn_exch{"HBG1 OPC"}="1HBG OPC"; $resn_exch{"HBG2 OPC"}="2HBG OPC";
$resn_exch{"HBG3 OPC"}="3HBG OPC"; $resn_exch{"HAA1 OPC"}="1HAA OPC"; $resn_exch{"HAA2 OPC"}="2HAA OPC";
$resn_exch{"HAA3 OPC"}="3HAA OPC"; $resn_exch{"HAC1 OPC"}="1HAC OPC"; $resn_exch{"HAC2 OPC"}="2HAC OPC";
$resn_exch{"HAC3 OPC"}="3HAC OPC"; $resn_exch{"HBI1 OPC"}="1HBI OPC"; $resn_exch{"HBI2 OPC"}="2HBI OPC";
$resn_exch{"HBL1 OPC"}="1HBL OPC"; $resn_exch{"HBL2 OPC"}="2HBL OPC"; $resn_exch{"HBM1 OPC"}="1HBM OPC";
$resn_exch{"HBM2 OPC"}="2HBM OPC"; $resn_exch{"HBN1 OPC"}="1HBN OPC"; $resn_exch{"HBN2 OPC"}="2HBN OPC";
$resn_exch{"HBO1 OPC"}="1HBO OPC"; $resn_exch{"HBO2 OPC"}="2HBO OPC"; $resn_exch{"HBP1 OPC"}="1HBP OPC";
$resn_exch{"HBP2 OPC"}="2HBP OPC"; $resn_exch{"HBQ1 OPC"}="1HBQ OPC"; $resn_exch{"HBQ2 OPC"}="2HBQ OPC";
$resn_exch{"HBT1 OPC"}="1HBT OPC"; $resn_exch{"HBT2 OPC"}="2HBT OPC"; $resn_exch{"HBU1 OPC"}="1HBU OPC";
$resn_exch{"HBU2 OPC"}="2HBU OPC"; $resn_exch{"HBV1 OPC"}="1HBV OPC"; $resn_exch{"HBV2 OPC"}="2HBV OPC";
$resn_exch{"HBW1 OPC"}="1HBW OPC"; $resn_exch{"HBW2 OPC"}="2HBW OPC"; $resn_exch{"HBX1 OPC"}="1HBX OPC";
$resn_exch{"HBX2 OPC"}="2HBX OPC"; $resn_exch{"HBY1 OPC"}="1HBY OPC"; $resn_exch{"HBY2 OPC"}="2HBY OPC";
$resn_exch{"HBZ1 OPC"}="1HBZ OPC"; $resn_exch{"HBZ2 OPC"}="2HBZ OPC"; $resn_exch{"HCA1 OPC"}="1HCA OPC";
$resn_exch{"HCA2 OPC"}="2HCA OPC"; $resn_exch{"HCB1 OPC"}="1HCB OPC"; $resn_exch{"HCB2 OPC"}="2HCB OPC";
$resn_exch{"HCB3 OPC"}="3HCB OPC"; $resn_exch{" H81 OPD"}="1H8  OPD"; $resn_exch{" H71 OPD"}="1H7  OPD";
$resn_exch{" H51 OPD"}="1H5  OPD"; $resn_exch{" H21 OPD"}="1H2  OPD"; $resn_exch{" H31 OPD"}="1H3  OPD";
$resn_exch{" H41 OPD"}="1H4  OPD"; $resn_exch{"H111 OPD"}="1H11 OPD"; $resn_exch{"H112 OPD"}="2H11 OPD";
$resn_exch{"H121 OPD"}="1H12 OPD"; $resn_exch{"H122 OPD"}="2H12 OPD"; $resn_exch{"H131 OPD"}="1H13 OPD";
$resn_exch{"H132 OPD"}="2H13 OPD"; $resn_exch{"H141 OPD"}="1H14 OPD"; $resn_exch{"H142 OPD"}="2H14 OPD";
$resn_exch{"H151 OPD"}="1H15 OPD"; $resn_exch{"H152 OPD"}="2H15 OPD"; $resn_exch{"H161 OPD"}="1H16 OPD";
$resn_exch{"H162 OPD"}="2H16 OPD"; $resn_exch{"H171 OPD"}="1H17 OPD"; $resn_exch{"H172 OPD"}="2H17 OPD";
$resn_exch{"H201 OPD"}="1H20 OPD"; $resn_exch{" H61 OPD"}="1H6  OPD"; $resn_exch{" H62 OPD"}="2H6  OPD";
$resn_exch{" H91 OPD"}="1H9  OPD"; $resn_exch{" H92 OPD"}="2H9  OPD"; $resn_exch{"H101 OPD"}="1H10 OPD";
$resn_exch{"H102 OPD"}="2H10 OPD"; $resn_exch{"H103 OPD"}="3H10 OPD"; $resn_exch{" HA1 OPE"}="1HA  OPE";
$resn_exch{" HA2 OPE"}="2HA  OPE"; $resn_exch{" HB1 OPE"}="1HB  OPE"; $resn_exch{" HB2 OPE"}="2HB  OPE";
$resn_exch{" HN1 OPE"}="1HN  OPE"; $resn_exch{" HN2 OPE"}="2HN  OPE"; $resn_exch{" H11 OPG"}="1H1  OPG";
$resn_exch{" H12 OPG"}="2H1  OPG"; $resn_exch{" H71 OPG"}="1H7  OPG"; $resn_exch{" H72 OPG"}="2H7  OPG";
$resn_exch{" HB1 OPH"}="1HB  OPH"; $resn_exch{" HB2 OPH"}="2HB  OPH"; $resn_exch{" HS1 OPH"}="1HS  OPH";
$resn_exch{" HS2 OPH"}="2HS  OPH"; $resn_exch{" HS3 OPH"}="3HS  OPH"; $resn_exch{"H3'1 OPM"}="1H3' OPM";
$resn_exch{"H3'2 OPM"}="2H3' OPM"; $resn_exch{"H4'1 OPM"}="1H4' OPM"; $resn_exch{"H4'2 OPM"}="2H4' OPM";
$resn_exch{"H5'1 OPM"}="1H5' OPM"; $resn_exch{"H5'2 OPM"}="2H5' OPM"; $resn_exch{"H5'3 OPM"}="3H5' OPM";
$resn_exch{"H1'1 OPM"}="1H1' OPM"; $resn_exch{"H1'2 OPM"}="2H1' OPM"; $resn_exch{"H2'1 OPM"}="1H2' OPM";
$resn_exch{"H2'2 OPM"}="2H2' OPM"; $resn_exch{" H61 OPM"}="1H6  OPM"; $resn_exch{" H62 OPM"}="2H6  OPM";
$resn_exch{"H1'1 OPN"}="1H1' OPN"; $resn_exch{"H1'2 OPN"}="2H1' OPN"; $resn_exch{"H1'3 OPN"}="3H1' OPN";
$resn_exch{"H2'1 OPN"}="1H2' OPN"; $resn_exch{"H2'2 OPN"}="2H2' OPN"; $resn_exch{"H3'1 OPN"}="1H3' OPN";
$resn_exch{"H3'2 OPN"}="2H3' OPN"; $resn_exch{"H5'1 OPN"}="1H5' OPN"; $resn_exch{"H5'2 OPN"}="2H5' OPN";
$resn_exch{"H8'1 OPN"}="1H8' OPN"; $resn_exch{"H8'2 OPN"}="2H8' OPN"; $resn_exch{" H51 OPP"}="1H5  OPP";
$resn_exch{" H52 OPP"}="2H5  OPP"; $resn_exch{"H5'1 OPP"}="1H5' OPP"; $resn_exch{"H5'2 OPP"}="2H5' OPP";
$resn_exch{" H   OPR"}="1HN  OPR"; $resn_exch{" H2  OPR"}="2HN  OPR"; $resn_exch{" HB2 OPR"}="1HB  OPR";
$resn_exch{" HB3 OPR"}="2HB  OPR"; $resn_exch{" HG2 OPR"}="1HG  OPR"; $resn_exch{" HG3 OPR"}="2HG  OPR";
$resn_exch{" HD2 OPR"}="1HD  OPR"; $resn_exch{" HD3 OPR"}="2HD  OPR"; $resn_exch{"HH21 OPR"}="1HH2 OPR";
$resn_exch{"HH22 OPR"}="2HH2 OPR"; $resn_exch{" H11 OPR"}="1H1  OPR"; $resn_exch{" H12 OPR"}="2H1  OPR";
$resn_exch{" H21 OPR"}="1H2  OPR"; $resn_exch{" H22 OPR"}="2H2  OPR"; $resn_exch{" HN1 OPS"}="1HN  OPS";
$resn_exch{" HN2 OPS"}="2HN  OPS"; $resn_exch{" HO1 OPS"}="1HO  OPS"; $resn_exch{" HO2 OPS"}="2HO  OPS";
$resn_exch{" HU1 OPS"}="1HU  OPS"; $resn_exch{" HU2 OPS"}="2HU  OPS"; $resn_exch{" HV1 OPS"}="1HV  OPS";
$resn_exch{" HV2 OPS"}="2HV  OPS"; $resn_exch{"H101 OQB"}="1H10 OQB"; $resn_exch{"H102 OQB"}="2H10 OQB";
$resn_exch{"H191 OQB"}="1H19 OQB"; $resn_exch{"H192 OQB"}="2H19 OQB"; $resn_exch{"H1'1 OR5"}="1H1* OR5";
$resn_exch{"H1'2 OR5"}="2H1* OR5"; $resn_exch{"HO'1 OR5"}="1HO* OR5"; $resn_exch{" H2' OR5"}=" H2* OR5";
$resn_exch{"HO'2 OR5"}="2HO* OR5"; $resn_exch{" H3' OR5"}=" H3* OR5"; $resn_exch{"HO'3 OR5"}="3HO* OR5";
$resn_exch{" H4' OR5"}=" H4* OR5"; $resn_exch{"HO'4 OR5"}="4HO* OR5"; $resn_exch{"H5'1 OR5"}="1H5* OR5";
$resn_exch{"H5'2 OR5"}="2H5* OR5"; $resn_exch{"HOP2 OR5"}="2HOP OR5"; $resn_exch{"HOP3 OR5"}="3HOP OR5";
$resn_exch{" H   ORN"}="1HN  ORN"; $resn_exch{" HN2 ORN"}="2HN  ORN"; $resn_exch{" HN3 ORN"}="3HN  ORN";
$resn_exch{" HB2 ORN"}="1HB  ORN"; $resn_exch{" HB3 ORN"}="2HB  ORN"; $resn_exch{" HG2 ORN"}="1HG  ORN";
$resn_exch{" HG3 ORN"}="2HG  ORN"; $resn_exch{" HD2 ORN"}="1HD  ORN"; $resn_exch{" HD3 ORN"}="2HD  ORN";
$resn_exch{" HE1 ORN"}="1HNE ORN"; $resn_exch{" HE2 ORN"}="2HNE ORN"; $resn_exch{" HE3 ORN"}="3HNE ORN";
$resn_exch{" H21 ORP"}="1H2  ORP"; $resn_exch{" H22 ORP"}="2H2  ORP"; $resn_exch{" H51 ORP"}="1H5  ORP";
$resn_exch{" H52 ORP"}="2H5  ORP"; $resn_exch{"HOP2 ORP"}="2HOP ORP"; $resn_exch{"HOP3 ORP"}="3HOP ORP";
$resn_exch{" H   ORQ"}="1HN  ORQ"; $resn_exch{" H2  ORQ"}="2HN  ORQ"; $resn_exch{" HB2 ORQ"}="1HB  ORQ";
$resn_exch{" HB3 ORQ"}="2HB  ORQ"; $resn_exch{" HG2 ORQ"}="1HG  ORQ"; $resn_exch{" HG3 ORQ"}="2HG  ORQ";
$resn_exch{" HD2 ORQ"}="1HD  ORQ"; $resn_exch{" HD3 ORQ"}="2HD  ORQ"; $resn_exch{" HE  ORQ"}="1HNE ORQ";
$resn_exch{" H21 ORQ"}="1H2  ORQ"; $resn_exch{" H22 ORQ"}="2H2  ORQ"; $resn_exch{" H23 ORQ"}="3H2  ORQ";
$resn_exch{" HXT ORQ"}="HOXT ORQ"; $resn_exch{"H4A1 ORX"}="1H4A ORX"; $resn_exch{"H4A2 ORX"}="2H4A ORX";
$resn_exch{"H2A1 ORX"}="1H2A ORX"; $resn_exch{"H2A2 ORX"}="2H2A ORX"; $resn_exch{"H2A3 ORX"}="3H2A ORX";
$resn_exch{"H5A1 ORX"}="1H5A ORX"; $resn_exch{"H5A2 ORX"}="2H5A ORX"; $resn_exch{"HBA1 ORX"}="1HBA ORX";
$resn_exch{"HBA2 ORX"}="2HBA ORX"; $resn_exch{"HGA1 ORX"}="1HGA ORX"; $resn_exch{"HGA2 ORX"}="2HGA ORX";
$resn_exch{"HDA1 ORX"}="1HDA ORX"; $resn_exch{"HDA2 ORX"}="2HDA ORX"; $resn_exch{"HNE1 ORX"}="1HNE ORX";
$resn_exch{"HNE2 ORX"}="2HNE ORX"; $resn_exch{" H61 OSB"}="1H6  OSB"; $resn_exch{" H51 OSB"}="1H5  OSB";
$resn_exch{" H41 OSB"}="1H4  OSB"; $resn_exch{" H31 OSB"}="1H3  OSB"; $resn_exch{" H91 OSB"}="1H9  OSB";
$resn_exch{" H92 OSB"}="2H9  OSB"; $resn_exch{"H101 OSB"}="1H10 OSB"; $resn_exch{"H102 OSB"}="2H10 OSB";
$resn_exch{" H61 OSC"}="1H6  OSC"; $resn_exch{" H62 OSC"}="2H6  OSC"; $resn_exch{" H71 OSC"}="1H7  OSC";
$resn_exch{" H72 OSC"}="2H7  OSC"; $resn_exch{"H111 OSC"}="1H11 OSC"; $resn_exch{"H112 OSC"}="2H11 OSC";
$resn_exch{"H151 OSC"}="1H15 OSC"; $resn_exch{"H152 OSC"}="2H15 OSC"; $resn_exch{"H171 OSC"}="1H17 OSC";
$resn_exch{"H172 OSC"}="2H17 OSC"; $resn_exch{"H191 OSC"}="1H19 OSC"; $resn_exch{"H192 OSC"}="2H19 OSC";
$resn_exch{"H221 OSC"}="1H22 OSC"; $resn_exch{"H222 OSC"}="2H22 OSC"; $resn_exch{"H301 OSC"}="1H30 OSC";
$resn_exch{"H302 OSC"}="2H30 OSC"; $resn_exch{"H321 OSC"}="1H32 OSC"; $resn_exch{"H322 OSC"}="2H32 OSC";
$resn_exch{"H371 OSC"}="1H37 OSC"; $resn_exch{"H372 OSC"}="2H37 OSC"; $resn_exch{"H441 OSC"}="1H44 OSC";
$resn_exch{"H442 OSC"}="2H44 OSC"; $resn_exch{" H2  OSE"}=" HN2 OSE"; $resn_exch{" HB2 OSE"}="1HB  OSE";
$resn_exch{" HB3 OSE"}="2HB  OSE"; $resn_exch{" H1  OSM"}="1H   OSM"; $resn_exch{" H2  OSM"}="2H   OSM";
$resn_exch{" HN1 OSM"}="1HN  OSM"; $resn_exch{" HN2 OSM"}="2HN  OSM"; $resn_exch{" H71 OSP"}="1H7  OSP";
$resn_exch{" H72 OSP"}="2H7  OSP"; $resn_exch{" H81 OSP"}="1H8  OSP"; $resn_exch{" H82 OSP"}="2H8  OSP";
$resn_exch{" H91 OSP"}="1H9  OSP"; $resn_exch{" H92 OSP"}="2H9  OSP"; $resn_exch{"H101 OSP"}="1H10 OSP";
$resn_exch{"H102 OSP"}="2H10 OSP"; $resn_exch{"HN11 OSP"}="1HN1 OSP"; $resn_exch{"HN12 OSP"}="2HN1 OSP";
$resn_exch{" H21 OSS"}="1H2  OSS"; $resn_exch{" H22 OSS"}="2H2  OSS"; $resn_exch{" H31 OSS"}="1H3  OSS";
$resn_exch{" H32 OSS"}="2H3  OSS"; $resn_exch{" H41 OSS"}="1H4  OSS"; $resn_exch{" H42 OSS"}="2H4  OSS";
$resn_exch{" H51 OSS"}="1H5  OSS"; $resn_exch{" H52 OSS"}="2H5  OSS"; $resn_exch{" H71 OSS"}="1H7  OSS";
$resn_exch{" H72 OSS"}="2H7  OSS"; $resn_exch{" H81 OSS"}="1H8  OSS"; $resn_exch{" H82 OSS"}="2H8  OSS";
$resn_exch{"H101 OSS"}="1H10 OSS"; $resn_exch{"H102 OSS"}="2H10 OSS"; $resn_exch{"HN11 OSS"}="1HN1 OSS";
$resn_exch{"HN12 OSS"}="2HN1 OSS"; $resn_exch{"H141 OSS"}="1H14 OSS"; $resn_exch{"H142 OSS"}="2H14 OSS";
$resn_exch{"H151 OSS"}="1H15 OSS"; $resn_exch{"H152 OSS"}="2H15 OSS"; $resn_exch{" H51 OST"}="1H5  OST";
$resn_exch{" H52 OST"}="2H5  OST"; $resn_exch{" H53 OST"}="3H5  OST"; $resn_exch{" H41 OST"}="1H4  OST";
$resn_exch{" H42 OST"}="2H4  OST"; $resn_exch{" H43 OST"}="3H4  OST"; $resn_exch{" H61 OSU"}="1H6  OSU";
$resn_exch{" H62 OSU"}="2H6  OSU"; $resn_exch{"H1'1 OSU"}="1H1' OSU"; $resn_exch{"H1'2 OSU"}="2H1' OSU";
$resn_exch{"H6'1 OSU"}="1H6' OSU"; $resn_exch{"H6'2 OSU"}="2H6' OSU"; $resn_exch{"H2N1 OSU"}="1H2N OSU";
$resn_exch{"H2N2 OSU"}="2H2N OSU"; $resn_exch{"H3N1 OSU"}="1H3N OSU"; $resn_exch{"H3N2 OSU"}="2H3N OSU";
$resn_exch{"H4N1 OSU"}="1H4N OSU"; $resn_exch{"H4N2 OSU"}="2H4N OSU"; $resn_exch{"H5N1 OSU"}="1H5N OSU";
$resn_exch{"H5N2 OSU"}="2H5N OSU"; $resn_exch{"H6N1 OSU"}="1H6N OSU"; $resn_exch{"H6N2 OSU"}="2H6N OSU";
$resn_exch{"H7N1 OSU"}="1H7N OSU"; $resn_exch{"H7N2 OSU"}="2H7N OSU"; $resn_exch{"H8N1 OSU"}="1H8N OSU";
$resn_exch{"H8N2 OSU"}="2H8N OSU"; $resn_exch{"H8N3 OSU"}="3H8N OSU"; $resn_exch{" H21 OTA"}="1H2  OTA";
$resn_exch{"H22A OTA"}="2H2  OTA"; $resn_exch{" H51 OTA"}="1H5  OTA"; $resn_exch{" H52 OTA"}="2H5  OTA";
$resn_exch{" H61 OTA"}="1H6  OTA"; $resn_exch{" H62 OTA"}="2H6  OTA"; $resn_exch{" H11 OTB"}="1H1  OTB";
$resn_exch{" H12 OTB"}="2H1  OTB"; $resn_exch{" H13 OTB"}="3H1  OTB"; $resn_exch{" H21 OTB"}="1H2  OTB";
$resn_exch{" H22 OTB"}="2H2  OTB"; $resn_exch{" H23 OTB"}="3H2  OTB"; $resn_exch{" H31 OTB"}="1H3  OTB";
$resn_exch{" H32 OTB"}="2H3  OTB"; $resn_exch{" H33 OTB"}="3H3  OTB"; $resn_exch{" H11 OTE"}="1H1  OTE";
$resn_exch{" H12 OTE"}="2H1  OTE"; $resn_exch{" H13 OTE"}="3H1  OTE"; $resn_exch{"H21A OTE"}="1H2  OTE";
$resn_exch{" H22 OTE"}="2H2  OTE"; $resn_exch{" H31 OTE"}="1H3  OTE"; $resn_exch{" H32 OTE"}="2H3  OTE";
$resn_exch{" H41 OTE"}="1H4  OTE"; $resn_exch{" H42 OTE"}="2H4  OTE"; $resn_exch{" H51 OTE"}="1H5  OTE";
$resn_exch{" H52 OTE"}="2H5  OTE"; $resn_exch{" H61 OTE"}="1H6  OTE"; $resn_exch{" H62 OTE"}="2H6  OTE";
$resn_exch{" H71 OTE"}="1H7  OTE"; $resn_exch{" H72 OTE"}="2H7  OTE"; $resn_exch{" H81 OTE"}="1H8  OTE";
$resn_exch{" H82 OTE"}="2H8  OTE"; $resn_exch{"H101 OTE"}="1H10 OTE"; $resn_exch{"H102 OTE"}="2H10 OTE";
$resn_exch{"H111 OTE"}="1H11 OTE"; $resn_exch{"H112 OTE"}="2H11 OTE"; $resn_exch{"H131 OTE"}="1H13 OTE";
$resn_exch{"H132 OTE"}="2H13 OTE"; $resn_exch{"H141 OTE"}="1H14 OTE"; $resn_exch{"H142 OTE"}="2H14 OTE";
$resn_exch{"H161 OTE"}="1H16 OTE"; $resn_exch{"H162 OTE"}="2H16 OTE"; $resn_exch{"H171 OTE"}="1H17 OTE";
$resn_exch{"H172 OTE"}="2H17 OTE"; $resn_exch{"H191 OTE"}="1H19 OTE"; $resn_exch{"H192 OTE"}="2H19 OTE";
$resn_exch{"H201 OTE"}="1H20 OTE"; $resn_exch{"H202 OTE"}="2H20 OTE"; $resn_exch{" H61 OTG"}="1H6  OTG";
$resn_exch{" H62 OTG"}="2H6  OTG"; $resn_exch{" H3' OTG"}=" H3* OTG"; $resn_exch{" H4' OTG"}=" H4* OTG";
$resn_exch{" H5' OTG"}=" H5* OTG"; $resn_exch{" H6' OTG"}=" H6* OTG"; $resn_exch{"H7'1 OTG"}="1H7* OTG";
$resn_exch{"H7'2 OTG"}="2H7* OTG"; $resn_exch{"H7'3 OTG"}="3H7* OTG"; $resn_exch{"H401 OTP"}="1H40 OTP";
$resn_exch{"H402 OTP"}="2H40 OTP"; $resn_exch{"H381 OTP"}="1H38 OTP"; $resn_exch{"H382 OTP"}="2H38 OTP";
$resn_exch{"H383 OTP"}="3H38 OTP"; $resn_exch{"H361 OTP"}="1H36 OTP"; $resn_exch{"H362 OTP"}="2H36 OTP";
$resn_exch{"H351 OTP"}="1H35 OTP"; $resn_exch{"H352 OTP"}="2H35 OTP"; $resn_exch{"H331 OTP"}="1H33 OTP";
$resn_exch{"H332 OTP"}="2H33 OTP"; $resn_exch{"H333 OTP"}="3H33 OTP"; $resn_exch{"H311 OTP"}="1H31 OTP";
$resn_exch{"H312 OTP"}="2H31 OTP"; $resn_exch{"H301 OTP"}="1H30 OTP"; $resn_exch{"H302 OTP"}="2H30 OTP";
$resn_exch{"H281 OTP"}="1H28 OTP"; $resn_exch{"H282 OTP"}="2H28 OTP"; $resn_exch{"H283 OTP"}="3H28 OTP";
$resn_exch{"H261 OTP"}="1H26 OTP"; $resn_exch{"H262 OTP"}="2H26 OTP"; $resn_exch{"H251 OTP"}="1H25 OTP";
$resn_exch{"H252 OTP"}="2H25 OTP"; $resn_exch{"H231 OTP"}="1H23 OTP"; $resn_exch{"H232 OTP"}="2H23 OTP";
$resn_exch{"H233 OTP"}="3H23 OTP"; $resn_exch{"H211 OTP"}="1H21 OTP"; $resn_exch{"H212 OTP"}="2H21 OTP";
$resn_exch{"H201 OTP"}="1H20 OTP"; $resn_exch{"H202 OTP"}="2H20 OTP"; $resn_exch{"H181 OTP"}="1H18 OTP";
$resn_exch{"H182 OTP"}="2H18 OTP"; $resn_exch{"H183 OTP"}="3H18 OTP"; $resn_exch{"H161 OTP"}="1H16 OTP";
$resn_exch{"H162 OTP"}="2H16 OTP"; $resn_exch{"H151 OTP"}="1H15 OTP"; $resn_exch{"H152 OTP"}="2H15 OTP";
$resn_exch{"H131 OTP"}="1H13 OTP"; $resn_exch{"H132 OTP"}="2H13 OTP"; $resn_exch{"H133 OTP"}="3H13 OTP";
$resn_exch{"H111 OTP"}="1H11 OTP"; $resn_exch{"H112 OTP"}="2H11 OTP"; $resn_exch{"H101 OTP"}="1H10 OTP";
$resn_exch{"H102 OTP"}="2H10 OTP"; $resn_exch{" H81 OTP"}="1H8  OTP"; $resn_exch{" H82 OTP"}="2H8  OTP";
$resn_exch{" H83 OTP"}="3H8  OTP"; $resn_exch{" H61 OTP"}="1H6  OTP"; $resn_exch{" H62 OTP"}="2H6  OTP";
$resn_exch{" H51 OTP"}="1H5  OTP"; $resn_exch{" H52 OTP"}="2H5  OTP"; $resn_exch{" H31 OTP"}="1H3  OTP";
$resn_exch{" H32 OTP"}="2H3  OTP"; $resn_exch{" H33 OTP"}="3H3  OTP"; $resn_exch{" H11 OTP"}="1H1  OTP";
$resn_exch{" H12 OTP"}="2H1  OTP"; $resn_exch{" H13 OTP"}="3H1  OTP"; $resn_exch{"HN81 OTR"}="1HN8 OTR";
$resn_exch{"HN82 OTR"}="2HN8 OTR"; $resn_exch{" H81 OTR"}="1H8  OTR"; $resn_exch{" H82 OTR"}="2H8  OTR";
$resn_exch{" H81 OTS"}="1H8  OTS"; $resn_exch{" H82 OTS"}="2H8  OTS"; $resn_exch{"HN81 OTS"}="1HN8 OTS";
$resn_exch{"HN82 OTS"}="2HN8 OTS"; $resn_exch{" H2  OTY"}=" HN2 OTY"; $resn_exch{" HB2 OTY"}="1HB  OTY";
$resn_exch{" HB3 OTY"}="2HB  OTY"; $resn_exch{"HE23 OTY"}="1HE2 OTY"; $resn_exch{"HE22 OTY"}="2HE2 OTY";
$resn_exch{" HN1 OUT"}="1HN  OUT"; $resn_exch{" HN2 OUT"}="2HN  OUT"; $resn_exch{" H51 OVA"}="1H5  OVA";
$resn_exch{" H52 OVA"}="2H5  OVA"; $resn_exch{" H61 OVA"}="1H6  OVA"; $resn_exch{" H62 OVA"}="2H6  OVA";
$resn_exch{"H111 OVA"}="1H11 OVA"; $resn_exch{"H112 OVA"}="2H11 OVA"; $resn_exch{"H113 OVA"}="3H11 OVA";
$resn_exch{"H231 OVA"}="1H23 OVA"; $resn_exch{"H232 OVA"}="2H23 OVA"; $resn_exch{" HA1 OVA"}="1HA  OVA";
$resn_exch{" HA2 OVA"}="2HA  OVA"; $resn_exch{" HA3 OVA"}="3HA  OVA"; $resn_exch{" HB1 OVA"}="1HB  OVA";
$resn_exch{" HB2 OVA"}="2HB  OVA"; $resn_exch{" HB3 OVA"}="3HB  OVA"; $resn_exch{" HC1 OVA"}="1HC  OVA";
$resn_exch{" HC2 OVA"}="2HC  OVA"; $resn_exch{" HC3 OVA"}="3HC  OVA"; $resn_exch{" H11 OVA"}="1H1  OVA";
$resn_exch{"H12A OVA"}="2H1  OVA"; $resn_exch{" H13 OVA"}="3H1  OVA"; $resn_exch{" H61 OX2"}="1H6  OX2";
$resn_exch{" H62 OX2"}="2H6  OX2"; $resn_exch{" H91 OX2"}="1H9  OX2"; $resn_exch{" H92 OX2"}="2H9  OX2";
$resn_exch{" H93 OX2"}="3H9  OX2"; $resn_exch{"H121 OX3"}="1H12 OX3"; $resn_exch{"H122 OX3"}="2H12 OX3";
$resn_exch{"H131 OX3"}="1H13 OX3"; $resn_exch{"H132 OX3"}="2H13 OX3"; $resn_exch{"H141 OX3"}="1H14 OX3";
$resn_exch{"H142 OX3"}="2H14 OX3"; $resn_exch{"H151 OX3"}="1H15 OX3"; $resn_exch{"H152 OX3"}="2H15 OX3";
$resn_exch{"H161 OX3"}="1H16 OX3"; $resn_exch{"H162 OX3"}="2H16 OX3"; $resn_exch{"H121 OX4"}="1H12 OX4";
$resn_exch{"H122 OX4"}="2H12 OX4"; $resn_exch{"H131 OX4"}="1H13 OX4"; $resn_exch{"H132 OX4"}="2H13 OX4";
$resn_exch{"H141 OX4"}="1H14 OX4"; $resn_exch{"H142 OX4"}="2H14 OX4"; $resn_exch{"H151 OX4"}="1H15 OX4";
$resn_exch{"H152 OX4"}="2H15 OX4"; $resn_exch{"H161 OX4"}="1H16 OX4"; $resn_exch{"H162 OX4"}="2H16 OX4";
$resn_exch{" H81 OX5"}="1H8  OX5"; $resn_exch{" H82 OX5"}="2H8  OX5"; $resn_exch{" H91 OX5"}="1H9  OX5";
$resn_exch{" H92 OX5"}="2H9  OX5"; $resn_exch{" H93 OX5"}="3H9  OX5"; $resn_exch{"H111 OX5"}="1H11 OX5";
$resn_exch{"H112 OX5"}="2H11 OX5"; $resn_exch{"H113 OX5"}="3H11 OX5"; $resn_exch{"H131 OX5"}="1H13 OX5";
$resn_exch{"H132 OX5"}="2H13 OX5"; $resn_exch{"H133 OX5"}="3H13 OX5"; $resn_exch{"H141 OX6"}="1H14 OX6";
$resn_exch{"H142 OX6"}="2H14 OX6"; $resn_exch{"H143 OX6"}="3H14 OX6"; $resn_exch{"H131 OX6"}="1H13 OX6";
$resn_exch{"H132 OX6"}="2H13 OX6"; $resn_exch{"H133 OX6"}="3H13 OX6"; $resn_exch{"H111 OX6"}="1H11 OX6";
$resn_exch{"H112 OX6"}="2H11 OX6"; $resn_exch{"H113 OX6"}="3H11 OX6"; $resn_exch{"H171 OX6"}="1H17 OX6";
$resn_exch{"H172 OX6"}="2H17 OX6"; $resn_exch{"H173 OX6"}="3H17 OX6"; $resn_exch{"H161 OX6"}="1H16 OX6";
$resn_exch{"H162 OX6"}="2H16 OX6"; $resn_exch{"H163 OX6"}="3H16 OX6"; $resn_exch{"H141 OX7"}="1H14 OX7";
$resn_exch{"H142 OX7"}="2H14 OX7"; $resn_exch{"H143 OX7"}="3H14 OX7"; $resn_exch{"H131 OX7"}="1H13 OX7";
$resn_exch{"H132 OX7"}="2H13 OX7"; $resn_exch{"H133 OX7"}="3H13 OX7"; $resn_exch{"H111 OX7"}="1H11 OX7";
$resn_exch{"H112 OX7"}="2H11 OX7"; $resn_exch{"H113 OX7"}="3H11 OX7"; $resn_exch{"H171 OX7"}="1H17 OX7";
$resn_exch{"H172 OX7"}="2H17 OX7"; $resn_exch{"H173 OX7"}="3H17 OX7"; $resn_exch{"H161 OX7"}="1H16 OX7";
$resn_exch{"H162 OX7"}="2H16 OX7"; $resn_exch{"H163 OX7"}="3H16 OX7"; $resn_exch{" H11 OXA"}="1H1  OXA";
$resn_exch{" H12 OXA"}="2H1  OXA"; $resn_exch{" H21 OXA"}="1H2  OXA"; $resn_exch{" H22 OXA"}="2H2  OXA";
$resn_exch{" H23 OXA"}="3H2  OXA"; $resn_exch{"H1'1 OXE"}="1H1' OXE"; $resn_exch{"H1'2 OXE"}="2H1' OXE";
$resn_exch{"H1'3 OXE"}="3H1' OXE"; $resn_exch{"H2'1 OXE"}="1H2' OXE"; $resn_exch{"H2'2 OXE"}="2H2' OXE";
$resn_exch{"H2'3 OXE"}="3H2' OXE"; $resn_exch{" H21 OXG"}="1H2  OXG"; $resn_exch{" H22 OXG"}="2H2  OXG";
$resn_exch{"H111 OXI"}="1H11 OXI"; $resn_exch{"H112 OXI"}="2H11 OXI"; $resn_exch{"H121 OXI"}="1H12 OXI";
$resn_exch{"H122 OXI"}="2H12 OXI"; $resn_exch{"H131 OXI"}="1H13 OXI"; $resn_exch{"H132 OXI"}="2H13 OXI";
$resn_exch{"H133 OXI"}="3H13 OXI"; $resn_exch{"H6N1 OXK"}="1H6N OXK"; $resn_exch{"H6N2 OXK"}="2H6N OXK";
$resn_exch{"H5'1 OXK"}="1H5' OXK"; $resn_exch{"H5'2 OXK"}="2H5' OXK"; $resn_exch{"HPB1 OXK"}="1HPB OXK";
$resn_exch{"HPB2 OXK"}="2HPB OXK"; $resn_exch{"HP91 OXK"}="1HP9 OXK"; $resn_exch{"HP92 OXK"}="2HP9 OXK";
$resn_exch{"HP93 OXK"}="3HP9 OXK"; $resn_exch{"HP81 OXK"}="1HP8 OXK"; $resn_exch{"HP82 OXK"}="2HP8 OXK";
$resn_exch{"HP83 OXK"}="3HP8 OXK"; $resn_exch{"HP51 OXK"}="1HP5 OXK"; $resn_exch{"HP52 OXK"}="2HP5 OXK";
$resn_exch{"HP41 OXK"}="1HP4 OXK"; $resn_exch{"HP42 OXK"}="2HP4 OXK"; $resn_exch{"HP21 OXK"}="1HP2 OXK";
$resn_exch{"HP22 OXK"}="2HP2 OXK"; $resn_exch{"HP11 OXK"}="1HP1 OXK"; $resn_exch{"HP12 OXK"}="2HP1 OXK";
$resn_exch{" HN1 OXM"}="1HN  OXM"; $resn_exch{" HN2 OXM"}="2HN  OXM"; $resn_exch{" H51 OXN"}="1H5  OXN";
$resn_exch{" H52 OXN"}="2H5  OXN"; $resn_exch{" H31 OXN"}="1H3  OXN"; $resn_exch{" H32 OXN"}="2H3  OXN";
$resn_exch{" H33 OXN"}="3H3  OXN"; $resn_exch{" H41 OXN"}="1H4  OXN"; $resn_exch{" H42 OXN"}="2H4  OXN";
$resn_exch{" H43 OXN"}="3H4  OXN"; $resn_exch{" H21 OXN"}="1H2  OXN"; $resn_exch{" H22 OXN"}="2H2  OXN";
$resn_exch{" H23 OXN"}="3H2  OXN"; $resn_exch{" H81 OXN"}="1H8  OXN"; $resn_exch{" H82 OXN"}="2H8  OXN";
$resn_exch{" H83 OXN"}="3H8  OXN"; $resn_exch{" H71 OXN"}="1H7  OXN"; $resn_exch{" H72 OXN"}="2H7  OXN";
$resn_exch{" H73 OXN"}="3H7  OXN"; $resn_exch{"H161 OXN"}="1H16 OXN"; $resn_exch{"H162 OXN"}="2H16 OXN";
$resn_exch{"H171 OXN"}="1H17 OXN"; $resn_exch{"H172 OXN"}="2H17 OXN"; $resn_exch{"H191 OXN"}="1H19 OXN";
$resn_exch{"H192 OXN"}="2H19 OXN"; $resn_exch{"H201 OXN"}="1H20 OXN"; $resn_exch{"H202 OXN"}="2H20 OXN";
$resn_exch{"H221 OXN"}="1H22 OXN"; $resn_exch{"H222 OXN"}="2H22 OXN"; $resn_exch{"H231 OXN"}="1H23 OXN";
$resn_exch{"H232 OXN"}="2H23 OXN"; $resn_exch{"H251 OXN"}="1H25 OXN"; $resn_exch{"H252 OXN"}="2H25 OXN";
$resn_exch{"H261 OXN"}="1H26 OXN"; $resn_exch{"H262 OXN"}="2H26 OXN"; $resn_exch{"H281 OXN"}="1H28 OXN";
$resn_exch{"H282 OXN"}="2H28 OXN"; $resn_exch{"H291 OXN"}="1H29 OXN"; $resn_exch{"H292 OXN"}="2H29 OXN";
$resn_exch{"H311 OXN"}="1H31 OXN"; $resn_exch{"H312 OXN"}="2H31 OXN"; $resn_exch{"H321 OXN"}="1H32 OXN";
$resn_exch{"H322 OXN"}="2H32 OXN"; $resn_exch{"H341 OXN"}="1H34 OXN"; $resn_exch{"H342 OXN"}="2H34 OXN";
$resn_exch{"H351 OXN"}="1H35 OXN"; $resn_exch{"H352 OXN"}="2H35 OXN"; $resn_exch{"H371 OXN"}="1H37 OXN";
$resn_exch{"H372 OXN"}="2H37 OXN"; $resn_exch{"H381 OXN"}="1H38 OXN"; $resn_exch{"H382 OXN"}="2H38 OXN";
$resn_exch{"H401 OXN"}="1H40 OXN"; $resn_exch{"H402 OXN"}="2H40 OXN"; $resn_exch{"H411 OXN"}="1H41 OXN";
$resn_exch{"H412 OXN"}="2H41 OXN"; $resn_exch{"H431 OXN"}="1H43 OXN"; $resn_exch{"H432 OXN"}="2H43 OXN";
$resn_exch{"H441 OXN"}="1H44 OXN"; $resn_exch{"H442 OXN"}="2H44 OXN"; $resn_exch{" H51 OXP"}="1H5  OXP";
$resn_exch{" H52 OXP"}="2H5  OXP"; $resn_exch{" H53 OXP"}="3H5  OXP"; $resn_exch{" H31 OXR"}="1H3  OXR";
$resn_exch{" H32 OXR"}="2H3  OXR"; $resn_exch{" H21 OXR"}="1H2  OXR"; $resn_exch{" H22 OXR"}="2H2  OXR";
$resn_exch{"H171 OXR"}="1H17 OXR"; $resn_exch{"H172 OXR"}="2H17 OXR"; $resn_exch{"H173 OXR"}="3H17 OXR";
$resn_exch{"H161 OXR"}="1H16 OXR"; $resn_exch{"H162 OXR"}="2H16 OXR"; $resn_exch{"H163 OXR"}="3H16 OXR";
$resn_exch{"H181 OXR"}="1H18 OXR"; $resn_exch{"H182 OXR"}="2H18 OXR"; $resn_exch{"H183 OXR"}="3H18 OXR";
$resn_exch{"H191 OXR"}="1H19 OXR"; $resn_exch{"H192 OXR"}="2H19 OXR"; $resn_exch{"H193 OXR"}="3H19 OXR";
$resn_exch{"H201 OXR"}="1H20 OXR"; $resn_exch{"H202 OXR"}="2H20 OXR"; $resn_exch{"H203 OXR"}="3H20 OXR";
$resn_exch{"H151 OXR"}="1H15 OXR"; $resn_exch{"H152 OXR"}="2H15 OXR"; $resn_exch{" H31 OXS"}="1H3  OXS";
$resn_exch{" H32 OXS"}="2H3  OXS"; $resn_exch{" H3' OXT"}=" H3* OXT"; $resn_exch{" H2' OXT"}=" H2* OXT";
$resn_exch{" H4' OXT"}=" H4* OXT"; $resn_exch{" H1' OXT"}=" H1* OXT"; $resn_exch{"H621 OXT"}="1H62 OXT";
$resn_exch{"H622 OXT"}="2H62 OXT"; $resn_exch{"H5'1 OXT"}="1H5* OXT"; $resn_exch{"H5'2 OXT"}="2H5* OXT";
$resn_exch{"H321 OXT"}="1H32 OXT"; $resn_exch{"H322 OXT"}="2H32 OXT"; $resn_exch{"H331 OXT"}="1H33 OXT";
$resn_exch{"H332 OXT"}="2H33 OXT"; $resn_exch{"H333 OXT"}="3H33 OXT"; $resn_exch{"H341 OXT"}="1H34 OXT";
$resn_exch{"H342 OXT"}="2H34 OXT"; $resn_exch{"H343 OXT"}="3H34 OXT"; $resn_exch{"H741 OXT"}="1H74 OXT";
$resn_exch{"H742 OXT"}="2H74 OXT"; $resn_exch{"H641 OXT"}="1H64 OXT"; $resn_exch{"H642 OXT"}="2H64 OXT";
$resn_exch{"H351 OXT"}="1H35 OXT"; $resn_exch{"H352 OXT"}="2H35 OXT"; $resn_exch{"H251 OXT"}="1H25 OXT";
$resn_exch{"H252 OXT"}="2H25 OXT"; $resn_exch{"H711 OXT"}="1H71 OXT"; $resn_exch{"H712 OXT"}="2H71 OXT";
$resn_exch{"H411 OXT"}="1H41 OXT"; $resn_exch{"H412 OXT"}="2H41 OXT"; $resn_exch{"HM21 OXT"}="1HM2 OXT";
$resn_exch{"HM22 OXT"}="2HM2 OXT"; $resn_exch{"HM23 OXT"}="3HM2 OXT"; $resn_exch{"HM41 OXT"}="1HM4 OXT";
$resn_exch{"HM42 OXT"}="2HM4 OXT"; $resn_exch{"HM43 OXT"}="3HM4 OXT"; $resn_exch{"H661 OXT"}="1H66 OXT";
$resn_exch{"H662 OXT"}="2H66 OXT"; $resn_exch{"H761 OXT"}="1H76 OXT"; $resn_exch{"H762 OXT"}="2H76 OXT";
$resn_exch{" H   OXX"}="1HN  OXX"; $resn_exch{" H2  OXX"}="2HN  OXX"; $resn_exch{" HB2 OXX"}="1HB  OXX";
$resn_exch{" HB3 OXX"}="2HB  OXX"; $resn_exch{"H1C1 OXZ"}="1H1C OXZ"; $resn_exch{"H1C2 OXZ"}="2H1C OXZ";
$resn_exch{"H5C1 OXZ"}="1H5C OXZ"; $resn_exch{"H5C2 OXZ"}="2H5C OXZ"; $resn_exch{"H141 OYA"}="1H14 OYA";
$resn_exch{"H142 OYA"}="2H14 OYA"; $resn_exch{"H143 OYA"}="3H14 OYA"; $resn_exch{" H51 OYA"}="1H5  OYA";
$resn_exch{" H52 OYA"}="2H5  OYA"; $resn_exch{" H61 OYA"}="1H6  OYA"; $resn_exch{" H62 OYA"}="2H6  OYA";
$resn_exch{"H111 OYA"}="1H11 OYA"; $resn_exch{"H112 OYA"}="2H11 OYA"; $resn_exch{" H71 OYA"}="1H7  OYA";
$resn_exch{" H72 OYA"}="2H7  OYA"; $resn_exch{" H81 OYA"}="1H8  OYA"; $resn_exch{" H82 OYA"}="2H8  OYA";
$resn_exch{" H91 OYA"}="1H9  OYA"; $resn_exch{" H92 OYA"}="2H9  OYA"; $resn_exch{" H5'   P"}="H5'1   P";
$resn_exch{"H5''   P"}="H5'2   P"; $resn_exch{" H2'   P"}="H2'1   P"; $resn_exch{"H2''   P"}="H2'2   P";
$resn_exch{"H111 P01"}="1H11 P01"; $resn_exch{"H112 P01"}="2H11 P01"; $resn_exch{"H113 P01"}="3H11 P01";
$resn_exch{"H121 P01"}="1H12 P01"; $resn_exch{"H122 P01"}="2H12 P01"; $resn_exch{"H123 P01"}="3H12 P01";
$resn_exch{"H261 P01"}="1H26 P01"; $resn_exch{"H262 P01"}="2H26 P01"; $resn_exch{"H263 P01"}="3H26 P01";
$resn_exch{"H271 P01"}="1H27 P01"; $resn_exch{"H272 P01"}="2H27 P01"; $resn_exch{"H273 P01"}="3H27 P01";
$resn_exch{"H231 P01"}="1H23 P01"; $resn_exch{"H232 P01"}="2H23 P01"; $resn_exch{"H31A P0E"}="1H3  P0E";
$resn_exch{"H32A P0E"}="2H3  P0E"; $resn_exch{" H41 P0E"}="1H4  P0E"; $resn_exch{" H42 P0E"}="2H4  P0E";
$resn_exch{"H151 P0E"}="1H15 P0E"; $resn_exch{"H152 P0E"}="2H15 P0E"; $resn_exch{"H161 P0E"}="1H16 P0E";
$resn_exch{"H162 P0E"}="2H16 P0E"; $resn_exch{"H171 P0E"}="1H17 P0E"; $resn_exch{"H172 P0E"}="2H17 P0E";
$resn_exch{"H181 P0E"}="1H18 P0E"; $resn_exch{"H182 P0E"}="2H18 P0E"; $resn_exch{"H191 P0E"}="1H19 P0E";
$resn_exch{"H192 P0E"}="2H19 P0E"; $resn_exch{"H201 P0E"}="1H20 P0E"; $resn_exch{"H202 P0E"}="2H20 P0E";
$resn_exch{"H211 P0E"}="1H21 P0E"; $resn_exch{"H212 P0E"}="2H21 P0E"; $resn_exch{"H241 P0E"}="1H24 P0E";
$resn_exch{"H242 P0E"}="2H24 P0E"; $resn_exch{"H251 P0E"}="1H25 P0E"; $resn_exch{"H252 P0E"}="2H25 P0E";
$resn_exch{"H261 P0E"}="1H26 P0E"; $resn_exch{"H262 P0E"}="2H26 P0E"; $resn_exch{"H271 P0E"}="1H27 P0E";
$resn_exch{"H272 P0E"}="2H27 P0E"; $resn_exch{"H281 P0E"}="1H28 P0E"; $resn_exch{"H282 P0E"}="2H28 P0E";
$resn_exch{"H291 P0E"}="1H29 P0E"; $resn_exch{"H292 P0E"}="2H29 P0E"; $resn_exch{"H301 P0E"}="1H30 P0E";
$resn_exch{"H302 P0E"}="2H30 P0E"; $resn_exch{"H331 P0E"}="1H33 P0E"; $resn_exch{"H332 P0E"}="2H33 P0E";
$resn_exch{"H341 P0E"}="1H34 P0E"; $resn_exch{"H342 P0E"}="2H34 P0E"; $resn_exch{"H351 P0E"}="1H35 P0E";
$resn_exch{"H352 P0E"}="2H35 P0E"; $resn_exch{"H361 P0E"}="1H36 P0E"; $resn_exch{"H362 P0E"}="2H36 P0E";
$resn_exch{"H371 P0E"}="1H37 P0E"; $resn_exch{"H372 P0E"}="2H37 P0E"; $resn_exch{"H381 P0E"}="1H38 P0E";
$resn_exch{"H382 P0E"}="2H38 P0E"; $resn_exch{"H391 P0E"}="1H39 P0E"; $resn_exch{"H392 P0E"}="2H39 P0E";
$resn_exch{"H401 P0E"}="1H40 P0E"; $resn_exch{"H402 P0E"}="2H40 P0E"; $resn_exch{"H411 P0E"}="1H41 P0E";
$resn_exch{"H412 P0E"}="2H41 P0E"; $resn_exch{"H413 P0E"}="3H41 P0E"; $resn_exch{"H421 P0E"}="1H42 P0E";
$resn_exch{"H422 P0E"}="2H42 P0E"; $resn_exch{"H431 P0E"}="1H43 P0E"; $resn_exch{"H432 P0E"}="2H43 P0E";
$resn_exch{"H441 P0E"}="1H44 P0E"; $resn_exch{"H442 P0E"}="2H44 P0E"; $resn_exch{"H451 P0E"}="1H45 P0E";
$resn_exch{"H452 P0E"}="2H45 P0E"; $resn_exch{"H461 P0E"}="1H46 P0E"; $resn_exch{"H462 P0E"}="2H46 P0E";
$resn_exch{"H463 P0E"}="3H46 P0E"; $resn_exch{"H471 P0E"}="1H47 P0E"; $resn_exch{"H472 P0E"}="2H47 P0E";
$resn_exch{"H481 P0E"}="1H48 P0E"; $resn_exch{"H482 P0E"}="2H48 P0E"; $resn_exch{"H2A1 P0P"}="1H2A P0P";
$resn_exch{"H2A2 P0P"}="2H2A P0P"; $resn_exch{"H2A3 P0P"}="3H2A P0P"; $resn_exch{"H5A1 P0P"}="1H5A P0P";
$resn_exch{"H5A2 P0P"}="2H5A P0P"; $resn_exch{"H021 P10"}="1H02 P10"; $resn_exch{"H022 P10"}="2H02 P10";
$resn_exch{"H151 P10"}="1H15 P10"; $resn_exch{"H152 P10"}="2H15 P10"; $resn_exch{"H161 P10"}="1H16 P10";
$resn_exch{"H162 P10"}="2H16 P10"; $resn_exch{"H271 P10"}="1H27 P10"; $resn_exch{"H272 P10"}="2H27 P10";
$resn_exch{"H381 P10"}="1H38 P10"; $resn_exch{"H382 P10"}="2H38 P10"; $resn_exch{" H31 P11"}="1H3  P11";
$resn_exch{"H32A P11"}="2H3  P11"; $resn_exch{" H33 P11"}="3H3  P11"; $resn_exch{"H111 P11"}="1H11 P11";
$resn_exch{"H112 P11"}="2H11 P11"; $resn_exch{"H113 P11"}="3H11 P11"; $resn_exch{"H131 P11"}="1H13 P11";
$resn_exch{"H132 P11"}="2H13 P11"; $resn_exch{"H141 P11"}="1H14 P11"; $resn_exch{"H142 P11"}="2H14 P11";
$resn_exch{"H171 P11"}="1H17 P11"; $resn_exch{"H172 P11"}="2H17 P11"; $resn_exch{"H173 P11"}="3H17 P11";
$resn_exch{"H211 P11"}="1H21 P11"; $resn_exch{"H212 P11"}="2H21 P11"; $resn_exch{"H221 P11"}="1H22 P11";
$resn_exch{"H222 P11"}="2H22 P11"; $resn_exch{"H10N P11"}="NH10 P11"; $resn_exch{"H251 P11"}="1H25 P11";
$resn_exch{"H252 P11"}="2H25 P11"; $resn_exch{"H253 P11"}="3H25 P11"; $resn_exch{"H11N P11"}="NH11 P11";
$resn_exch{"H331 P11"}="1H33 P11"; $resn_exch{"H332 P11"}="2H33 P11"; $resn_exch{"H333 P11"}="3H33 P11";
$resn_exch{"H14N P11"}="NH14 P11"; $resn_exch{"H351 P11"}="1H35 P11"; $resn_exch{"H352 P11"}="2H35 P11";
$resn_exch{"H361 P11"}="1H36 P11"; $resn_exch{"H362 P11"}="2H36 P11"; $resn_exch{"H16N P11"}="NH16 P11";
$resn_exch{"H371 P11"}="1H37 P11"; $resn_exch{"H372 P11"}="2H37 P11"; $resn_exch{"H381 P11"}="1H38 P11";
$resn_exch{"H382 P11"}="2H38 P11"; $resn_exch{"H391 P11"}="1H39 P11"; $resn_exch{"H392 P11"}="2H39 P11";
$resn_exch{"H17N P11"}="NH17 P11"; $resn_exch{"H401 P11"}="1H40 P11"; $resn_exch{"H402 P11"}="2H40 P11";
$resn_exch{"H403 P11"}="3H40 P11"; $resn_exch{"H411 P11"}="1H41 P11"; $resn_exch{"H412 P11"}="2H41 P11";
$resn_exch{"H413 P11"}="3H41 P11"; $resn_exch{" HB1 P12"}="1HB  P12"; $resn_exch{" HB2 P12"}="2HB  P12";
$resn_exch{" HG1 P12"}="1HG  P12"; $resn_exch{" HG2 P12"}="2HG  P12"; $resn_exch{" HD1 P12"}="1HD  P12";
$resn_exch{" HD2 P12"}="2HD  P12"; $resn_exch{" HE1 P12"}="1HE  P12"; $resn_exch{" HE2 P12"}="2HE  P12";
$resn_exch{" HE3 P12"}="3HE  P12"; $resn_exch{"HG11 P12"}="1HG1 P12"; $resn_exch{"HG12 P12"}="2HG1 P12";
$resn_exch{"HD11 P12"}="1HD1 P12"; $resn_exch{"HD12 P12"}="2HD1 P12"; $resn_exch{"HD21 P12"}="1HD2 P12";
$resn_exch{"HD22 P12"}="2HD2 P12"; $resn_exch{"HG21 P12"}="1HG2 P12"; $resn_exch{"HG22 P12"}="2HG2 P12";
$resn_exch{" HA1 P12"}="1HA  P12"; $resn_exch{" HA2 P12"}="2HA  P12"; $resn_exch{"HH11 P12"}="1HH1 P12";
$resn_exch{"HH21 P12"}="1HH2 P12"; $resn_exch{"HH22 P12"}="2HH2 P12"; $resn_exch{"HB11 P12"}="1HB1 P12";
$resn_exch{"HB12 P12"}="2HB1 P12"; $resn_exch{" HB3 P12"}="3HB  P12"; $resn_exch{"HN1A P13"}="1HN  P13";
$resn_exch{" HN2 P13"}="2HN  P13"; $resn_exch{" HB1 P13"}="1HB  P13"; $resn_exch{" HB2 P13"}="2HB  P13";
$resn_exch{" HA1 P13"}="1HA  P13"; $resn_exch{" HA2 P13"}="2HA  P13"; $resn_exch{" HC1 P13"}="1HC  P13";
$resn_exch{" HC2 P13"}="2HC  P13"; $resn_exch{"HA11 P13"}="1HA1 P13"; $resn_exch{"HA12 P13"}="2HA1 P13";
$resn_exch{"HB11 P13"}="1HB1 P13"; $resn_exch{" HG1 P13"}="1HG  P13"; $resn_exch{" HG2 P13"}="2HG  P13";
$resn_exch{" HD1 P13"}="1HD  P13"; $resn_exch{" HD2 P13"}="2HD  P13"; $resn_exch{"HH11 P13"}="1HH1 P13";
$resn_exch{"HH12 P13"}="2HH1 P13"; $resn_exch{"HI11 P13"}="1HI1 P13"; $resn_exch{"HI12 P13"}="2HI1 P13";
$resn_exch{" HJ1 P13"}="1HJ  P13"; $resn_exch{"HI21 P13"}="1HI2 P13"; $resn_exch{"HI22 P13"}="2HI2 P13";
$resn_exch{"HH21 P13"}="1HH2 P13"; $resn_exch{"HH22 P13"}="2HH2 P13"; $resn_exch{" HB1 P14"}="1HB  P14";
$resn_exch{" HB2 P14"}="2HB  P14"; $resn_exch{" HG1 P14"}="1HG  P14"; $resn_exch{" HG2 P14"}="2HG  P14";
$resn_exch{"HZ11 P14"}="1HZ1 P14"; $resn_exch{"HZ12 P14"}="2HZ1 P14"; $resn_exch{"HZ21 P14"}="1HZ2 P14";
$resn_exch{"HZ22 P14"}="2HZ2 P14"; $resn_exch{" HA1 P14"}="1HA  P14"; $resn_exch{" HA2 P14"}="2HA  P14";
$resn_exch{"HB11 P14"}="1HB1 P14"; $resn_exch{"HG11 P14"}="1HG1 P14"; $resn_exch{"HG12 P14"}="2HG1 P14";
$resn_exch{" HD1 P14"}="1HD  P14"; $resn_exch{" HD2 P14"}="2HD  P14"; $resn_exch{" HE1 P14"}="1HE  P14";
$resn_exch{" HE2 P14"}="2HE  P14"; $resn_exch{" HZ1 P14"}="1HZ  P14"; $resn_exch{" HZ2 P14"}="2HZ  P14";
$resn_exch{"HJ11 P14"}="1HJ1 P14"; $resn_exch{"HJ12 P14"}="2HJ1 P14"; $resn_exch{"HJ21 P14"}="1HJ2 P14";
$resn_exch{"HJ22 P14"}="2HJ2 P14"; $resn_exch{" H11 P14"}="1H1  P14"; $resn_exch{" H12 P14"}="2H1  P14";
$resn_exch{" H13 P14"}="3H1  P14"; $resn_exch{"H131 P15"}="1H13 P15"; $resn_exch{"H132 P15"}="2H13 P15";
$resn_exch{"H133 P15"}="3H13 P15"; $resn_exch{"H121 P15"}="1H12 P15"; $resn_exch{"H122 P15"}="2H12 P15";
$resn_exch{"H111 P15"}="1H11 P15"; $resn_exch{"H112 P15"}="2H11 P15"; $resn_exch{"H101 P15"}="1H10 P15";
$resn_exch{"H102 P15"}="2H10 P15"; $resn_exch{" H91 P15"}="1H9  P15"; $resn_exch{" H92 P15"}="2H9  P15";
$resn_exch{" H81 P15"}="1H8  P15"; $resn_exch{" H82 P15"}="2H8  P15"; $resn_exch{" H71 P15"}="1H7  P15";
$resn_exch{" H72 P15"}="2H7  P15"; $resn_exch{" H61 P15"}="1H6  P15"; $resn_exch{" H62 P15"}="2H6  P15";
$resn_exch{" H51 P15"}="1H5  P15"; $resn_exch{" H52 P15"}="2H5  P15"; $resn_exch{" H41 P15"}="1H4  P15";
$resn_exch{" H42 P15"}="2H4  P15"; $resn_exch{" H31 P15"}="1H3  P15"; $resn_exch{" H32 P15"}="2H3  P15";
$resn_exch{" H21 P15"}="1H2  P15"; $resn_exch{" H22 P15"}="2H2  P15"; $resn_exch{" H11 P15"}="1H1  P15";
$resn_exch{" H12 P15"}="2H1  P15"; $resn_exch{" H21 P16"}="1H2  P16"; $resn_exch{" H22 P16"}="2H2  P16";
$resn_exch{"H201 P16"}="1H20 P16"; $resn_exch{"H202 P16"}="2H20 P16"; $resn_exch{"H203 P16"}="3H20 P16";
$resn_exch{" H11 P1A"}="1H1  P1A"; $resn_exch{" H12 P1A"}="2H1  P1A"; $resn_exch{" H41 P1A"}="1H4  P1A";
$resn_exch{" H42 P1A"}="2H4  P1A"; $resn_exch{"H191 P1A"}="1H19 P1A"; $resn_exch{"H192 P1A"}="2H19 P1A";
$resn_exch{"H193 P1A"}="3H19 P1A"; $resn_exch{"H111 P1A"}="1H11 P1A"; $resn_exch{"H112 P1A"}="2H11 P1A";
$resn_exch{"H121 P1A"}="1H12 P1A"; $resn_exch{"H122 P1A"}="2H12 P1A"; $resn_exch{"H181 P1A"}="1H18 P1A";
$resn_exch{"H182 P1A"}="2H18 P1A"; $resn_exch{"H183 P1A"}="3H18 P1A"; $resn_exch{"H151 P1A"}="1H15 P1A";
$resn_exch{"H152 P1A"}="2H15 P1A"; $resn_exch{"H161 P1A"}="1H16 P1A"; $resn_exch{"H162 P1A"}="2H16 P1A";
$resn_exch{"H211 P1A"}="1H21 P1A"; $resn_exch{"H212 P1A"}="2H21 P1A"; $resn_exch{"H213 P1A"}="3H21 P1A";
$resn_exch{"H231 P1A"}="1H23 P1A"; $resn_exch{"H232 P1A"}="2H23 P1A"; $resn_exch{"H241 P1A"}="1H24 P1A";
$resn_exch{"H242 P1A"}="2H24 P1A"; $resn_exch{"H261 P1A"}="1H26 P1A"; $resn_exch{"H262 P1A"}="2H26 P1A";
$resn_exch{"H263 P1A"}="3H26 P1A"; $resn_exch{"H271 P1A"}="1H27 P1A"; $resn_exch{"H272 P1A"}="2H27 P1A";
$resn_exch{"H273 P1A"}="3H27 P1A"; $resn_exch{"H3C1 P1C"}="1H3C P1C"; $resn_exch{"H3C2 P1C"}="2H3C P1C";
$resn_exch{"H4C1 P1C"}="1H4C P1C"; $resn_exch{"H4C2 P1C"}="2H4C P1C"; $resn_exch{"H7C1 P1C"}="1H7C P1C";
$resn_exch{"H7C2 P1C"}="2H7C P1C"; $resn_exch{"H141 P1C"}="1H14 P1C"; $resn_exch{"H142 P1C"}="2H14 P1C";
$resn_exch{"H331 P1C"}="1H33 P1C"; $resn_exch{"H332 P1C"}="2H33 P1C"; $resn_exch{"H333 P1C"}="3H33 P1C";
$resn_exch{"H371 P1C"}="1H37 P1C"; $resn_exch{"H372 P1C"}="2H37 P1C"; $resn_exch{"H421 P1C"}="1H42 P1C";
$resn_exch{"H1P1 P1D"}="1H1P P1D"; $resn_exch{"H1P2 P1D"}="2H1P P1D"; $resn_exch{"H5'1 P1D"}="1H5' P1D";
$resn_exch{"H5'2 P1D"}="2H5' P1D"; $resn_exch{"H1P1 P1G"}="1H1P P1G"; $resn_exch{"H1P2 P1G"}="2H1P P1G";
$resn_exch{"H5'1 P1G"}="1H5' P1G"; $resn_exch{"H5'2 P1G"}="2H5' P1G"; $resn_exch{"HN21 P1G"}="1HN2 P1G";
$resn_exch{"HN22 P1G"}="2HN2 P1G"; $resn_exch{" H11 P1H"}="1H1  P1H"; $resn_exch{"H12A P1H"}="2H1  P1H";
$resn_exch{"HN31 P1H"}="1HN3 P1H"; $resn_exch{"HN32 P1H"}="2HN3 P1H"; $resn_exch{" H61 P1H"}="1H6  P1H";
$resn_exch{"H111 P1H"}="1H11 P1H"; $resn_exch{"H112 P1H"}="2H11 P1H"; $resn_exch{"HN71 P1H"}="1HN7 P1H";
$resn_exch{"HN72 P1H"}="2HN7 P1H"; $resn_exch{"H161 P1H"}="1H16 P1H"; $resn_exch{"H231 P1H"}="1H23 P1H";
$resn_exch{"H261 P1H"}="1H26 P1H"; $resn_exch{"H291 P1H"}="1H29 P1H"; $resn_exch{"H292 P1H"}="2H29 P1H";
$resn_exch{"H281 P1H"}="1H28 P1H"; $resn_exch{"H282 P1H"}="2H28 P1H"; $resn_exch{"H301 P1H"}="1H30 P1H";
$resn_exch{"H302 P1H"}="2H30 P1H"; $resn_exch{"H303 P1H"}="3H30 P1H"; $resn_exch{" H   P1L"}="1H3N P1L";
$resn_exch{" H2  P1L"}="2H3N P1L"; $resn_exch{" HB2 P1L"}="1H5C P1L"; $resn_exch{" HB3 P1L"}="2H5C P1L";
$resn_exch{"H8C1 P1L"}="1H8C P1L"; $resn_exch{"H8C2 P1L"}="2H8C P1L"; $resn_exch{"H9C1 P1L"}="1H9C P1L";
$resn_exch{"H9C2 P1L"}="2H9C P1L"; $resn_exch{"H101 P1L"}="1H10 P1L"; $resn_exch{"H102 P1L"}="2H10 P1L";
$resn_exch{"H111 P1L"}="1H11 P1L"; $resn_exch{"H112 P1L"}="2H11 P1L"; $resn_exch{"H121 P1L"}="1H12 P1L";
$resn_exch{"H122 P1L"}="2H12 P1L"; $resn_exch{"H131 P1L"}="1H13 P1L"; $resn_exch{"H132 P1L"}="2H13 P1L";
$resn_exch{"H141 P1L"}="1H14 P1L"; $resn_exch{"H142 P1L"}="2H14 P1L"; $resn_exch{"H151 P1L"}="1H15 P1L";
$resn_exch{"H152 P1L"}="2H15 P1L"; $resn_exch{"H161 P1L"}="1H16 P1L"; $resn_exch{"H162 P1L"}="2H16 P1L";
$resn_exch{"H171 P1L"}="1H17 P1L"; $resn_exch{"H172 P1L"}="2H17 P1L"; $resn_exch{"H181 P1L"}="1H18 P1L";
$resn_exch{"H182 P1L"}="2H18 P1L"; $resn_exch{"H191 P1L"}="1H19 P1L"; $resn_exch{"H192 P1L"}="2H19 P1L";
$resn_exch{"H201 P1L"}="1H20 P1L"; $resn_exch{"H202 P1L"}="2H20 P1L"; $resn_exch{"H211 P1L"}="1H21 P1L";
$resn_exch{"H212 P1L"}="2H21 P1L"; $resn_exch{"H221 P1L"}="1H22 P1L"; $resn_exch{"H222 P1L"}="2H22 P1L";
$resn_exch{"H223 P1L"}="3H22 P1L"; $resn_exch{" HXT P1L"}=" HO' P1L"; $resn_exch{" H5' P1P"}="1H5* P1P";
$resn_exch{"H5'' P1P"}="2H5* P1P"; $resn_exch{"H4'1 P1P"}="1H4* P1P"; $resn_exch{"H4'2 P1P"}="2H4* P1P";
$resn_exch{"H3'1 P1P"}="1H3* P1P"; $resn_exch{"H3'2 P1P"}="2H3* P1P"; $resn_exch{" H11 P1S"}="1H1  P1S";
$resn_exch{"H12A P1S"}="2H1  P1S"; $resn_exch{"H161 P1S"}="1H16 P1S"; $resn_exch{"H162 P1S"}="2H16 P1S";
$resn_exch{"H171 P1S"}="1H17 P1S"; $resn_exch{"H172 P1S"}="2H17 P1S"; $resn_exch{"H5A1 P1T"}="1H5A P1T";
$resn_exch{"H5A2 P1T"}="2H5A P1T"; $resn_exch{"H2A1 P1T"}="1H2A P1T"; $resn_exch{"H2A2 P1T"}="2H2A P1T";
$resn_exch{"H2A3 P1T"}="3H2A P1T"; $resn_exch{"H4A1 P1T"}="1H4A P1T"; $resn_exch{"H4A2 P1T"}="2H4A P1T";
$resn_exch{" HB1 P1T"}="1HB  P1T"; $resn_exch{" HB2 P1T"}="2HB  P1T"; $resn_exch{"H211 P1Z"}="1H21 P1Z";
$resn_exch{"H212 P1Z"}="2H21 P1Z"; $resn_exch{"H213 P1Z"}="3H21 P1Z"; $resn_exch{"H201 P1Z"}="1H20 P1Z";
$resn_exch{"H202 P1Z"}="2H20 P1Z"; $resn_exch{"H191 P1Z"}="1H19 P1Z"; $resn_exch{"H192 P1Z"}="2H19 P1Z";
$resn_exch{"H181 P1Z"}="1H18 P1Z"; $resn_exch{"H182 P1Z"}="2H18 P1Z"; $resn_exch{"H11A P20"}="1H1  P20";
$resn_exch{" H12 P20"}="2H1  P20"; $resn_exch{"H121 P20"}="1H12 P20"; $resn_exch{"H122 P20"}="2H12 P20";
$resn_exch{"H121 P21"}="1H12 P21"; $resn_exch{"H122 P21"}="2H12 P21"; $resn_exch{"H191 P21"}="1H19 P21";
$resn_exch{"H192 P21"}="2H19 P21"; $resn_exch{"H231 P21"}="1H23 P21"; $resn_exch{"H232 P21"}="2H23 P21";
$resn_exch{"H6C1 P21"}="1H6C P21"; $resn_exch{"H6C2 P21"}="2H6C P21"; $resn_exch{"H201 P21"}="1H20 P21";
$resn_exch{"H202 P21"}="2H20 P21"; $resn_exch{"H181 P21"}="1H18 P21"; $resn_exch{"H182 P21"}="2H18 P21";
$resn_exch{"H101 P21"}="1H10 P21"; $resn_exch{"H102 P21"}="2H10 P21"; $resn_exch{"H211 P21"}="1H21 P21";
$resn_exch{"H212 P21"}="2H21 P21"; $resn_exch{"H161 P21"}="1H16 P21"; $resn_exch{"H162 P21"}="2H16 P21";
$resn_exch{"H261 P21"}="1H26 P21"; $resn_exch{"H262 P21"}="2H26 P21"; $resn_exch{"H263 P21"}="3H26 P21";
$resn_exch{" H61 P22"}="1H6  P22"; $resn_exch{" H62 P22"}="2H6  P22"; $resn_exch{" H63 P22"}="3H6  P22";
$resn_exch{" H71 P22"}="1H7  P22"; $resn_exch{" H72 P22"}="2H7  P22"; $resn_exch{" H51 P23"}="1H5  P23";
$resn_exch{" H52 P23"}="2H5  P23"; $resn_exch{" H53 P23"}="3H5  P23"; $resn_exch{" H61 P23"}="1H6  P23";
$resn_exch{" H62 P23"}="2H6  P23"; $resn_exch{" H71 P23"}="1H7  P23"; $resn_exch{" H72 P23"}="2H7  P23";
$resn_exch{"HN11 P24"}="1HN1 P24"; $resn_exch{"HN12 P24"}="2HN1 P24"; $resn_exch{" H21 P24"}="1H2  P24";
$resn_exch{" H22 P24"}="2H2  P24"; $resn_exch{" H31 P24"}="1H3  P24"; $resn_exch{" H32 P24"}="2H3  P24";
$resn_exch{" H51 P24"}="1H5  P24"; $resn_exch{" H52 P24"}="2H5  P24"; $resn_exch{" H61 P24"}="1H6  P24";
$resn_exch{" H62 P24"}="2H6  P24"; $resn_exch{" H71 P24"}="1H7  P24"; $resn_exch{" H72 P24"}="2H7  P24";
$resn_exch{" H81 P24"}="1H8  P24"; $resn_exch{" H82 P24"}="2H8  P24"; $resn_exch{"HN91 P24"}="1HN9 P24";
$resn_exch{"HN92 P24"}="2HN9 P24"; $resn_exch{" H41 P25"}="1H4  P25"; $resn_exch{" H42 P25"}="2H4  P25";
$resn_exch{"HM41 P25"}="1HM4 P25"; $resn_exch{"HM42 P25"}="2HM4 P25"; $resn_exch{"HM43 P25"}="3HM4 P25";
$resn_exch{" H51 P25"}="1H5  P25"; $resn_exch{" H52 P25"}="2H5  P25"; $resn_exch{" H61 P25"}="1H6  P25";
$resn_exch{" H62 P25"}="2H6  P25"; $resn_exch{" H71 P25"}="1H7  P25"; $resn_exch{" H72 P25"}="2H7  P25";
$resn_exch{"H321 P27"}="1H32 P27"; $resn_exch{"H322 P27"}="2H32 P27"; $resn_exch{"H121 P27"}="1H12 P27";
$resn_exch{"H122 P27"}="2H12 P27"; $resn_exch{" H71 P28"}="1H7  P28"; $resn_exch{" H72 P28"}="2H7  P28";
$resn_exch{"H821 P28"}="1H82 P28"; $resn_exch{"H822 P28"}="2H82 P28"; $resn_exch{"H823 P28"}="3H82 P28";
$resn_exch{"HN21 P2G"}="1HN2 P2G"; $resn_exch{"HN22 P2G"}="2HN2 P2G"; $resn_exch{"H5'1 P2G"}="1H5' P2G";
$resn_exch{"H5'2 P2G"}="2H5' P2G"; $resn_exch{" H81 P2K"}="1H8  P2K"; $resn_exch{" H82 P2K"}="2H8  P2K";
$resn_exch{" H91 P2K"}="1H9  P2K"; $resn_exch{" H92 P2K"}="2H9  P2K"; $resn_exch{"H111 P2K"}="1H11 P2K";
$resn_exch{"H112 P2K"}="2H11 P2K"; $resn_exch{"H121 P2K"}="1H12 P2K"; $resn_exch{"H122 P2K"}="2H12 P2K";
$resn_exch{"H141 P2K"}="1H14 P2K"; $resn_exch{"H142 P2K"}="2H14 P2K"; $resn_exch{"H151 P2K"}="1H15 P2K";
$resn_exch{"H152 P2K"}="2H15 P2K"; $resn_exch{"H171 P2K"}="1H17 P2K"; $resn_exch{"H172 P2K"}="2H17 P2K";
$resn_exch{"H181 P2K"}="1H18 P2K"; $resn_exch{"H182 P2K"}="2H18 P2K"; $resn_exch{"H201 P2K"}="1H20 P2K";
$resn_exch{"H202 P2K"}="2H20 P2K"; $resn_exch{"H211 P2K"}="1H21 P2K"; $resn_exch{"H212 P2K"}="2H21 P2K";
$resn_exch{"H231 P2K"}="1H23 P2K"; $resn_exch{"H232 P2K"}="2H23 P2K"; $resn_exch{"H241 P2K"}="1H24 P2K";
$resn_exch{"H242 P2K"}="2H24 P2K"; $resn_exch{"H8C1 P2N"}="1H8C P2N"; $resn_exch{"H8C2 P2N"}="2H8C P2N";
$resn_exch{"H101 P2N"}="1H10 P2N"; $resn_exch{"H102 P2N"}="2H10 P2N"; $resn_exch{"H111 P2N"}="1H11 P2N";
$resn_exch{"H112 P2N"}="2H11 P2N"; $resn_exch{"H121 P2N"}="1H12 P2N"; $resn_exch{"H122 P2N"}="2H12 P2N";
$resn_exch{"H131 P2N"}="1H13 P2N"; $resn_exch{"H132 P2N"}="2H13 P2N"; $resn_exch{"H161 P2N"}="1H16 P2N";
$resn_exch{"H162 P2N"}="2H16 P2N"; $resn_exch{"H171 P2N"}="1H17 P2N"; $resn_exch{"H172 P2N"}="2H17 P2N";
$resn_exch{"H131 P2O"}="1H13 P2O"; $resn_exch{"H132 P2O"}="2H13 P2O"; $resn_exch{" H91 P2O"}="1H9  P2O";
$resn_exch{" H92 P2O"}="2H9  P2O"; $resn_exch{"H141 P2O"}="1H14 P2O"; $resn_exch{"H142 P2O"}="2H14 P2O";
$resn_exch{"HOP2 P2P"}="2HOP P2P"; $resn_exch{"HOP3 P2P"}="3HOP P2P"; $resn_exch{"H5'1 P2P"}="1H5* P2P";
$resn_exch{"H5'2 P2P"}="2H5* P2P"; $resn_exch{" H4' P2P"}=" H4* P2P"; $resn_exch{" H3' P2P"}=" H3* P2P";
$resn_exch{"HO3' P2P"}="*HO3 P2P"; $resn_exch{" H2' P2P"}=" H2* P2P"; $resn_exch{"HO2' P2P"}="*HO2 P2P";
$resn_exch{" H1' P2P"}=" H1* P2P"; $resn_exch{" HD1 P2S"}="1HD  P2S"; $resn_exch{" HD2 P2S"}="2HD  P2S";
$resn_exch{" HF1 P2S"}="1HF  P2S"; $resn_exch{" HF2 P2S"}="2HF  P2S"; $resn_exch{" HH1 P2S"}="1HH  P2S";
$resn_exch{" HH2 P2S"}="2HH  P2S"; $resn_exch{" HH3 P2S"}="3HH  P2S"; $resn_exch{" HG1 P2S"}="1HG  P2S";
$resn_exch{" HG2 P2S"}="2HG  P2S"; $resn_exch{" HB1 P2S"}="1HB  P2S"; $resn_exch{" HB2 P2S"}="2HB  P2S";
$resn_exch{" HN1 P2S"}="1HN  P2S"; $resn_exch{" HN2 P2S"}="2HN  P2S"; $resn_exch{"HOP2 P2T"}="2HOP P2T";
$resn_exch{" H5' P2T"}="1H5* P2T"; $resn_exch{"H5'' P2T"}="2H5* P2T"; $resn_exch{" H4' P2T"}=" H4* P2T";
$resn_exch{" H1' P2T"}=" H1* P2T"; $resn_exch{" H71 P2T"}="1H5M P2T"; $resn_exch{" H72 P2T"}="2H5M P2T";
$resn_exch{" H73 P2T"}="3H5M P2T"; $resn_exch{" H2' P2T"}=" H2* P2T"; $resn_exch{"HB'1 P2T"}="1HB' P2T";
$resn_exch{"HB'2 P2T"}="2HB' P2T"; $resn_exch{"HC'1 P2T"}="1HC' P2T"; $resn_exch{"HC'2 P2T"}="2HC' P2T";
$resn_exch{"HD'1 P2T"}="1HD' P2T"; $resn_exch{"HD'2 P2T"}="2HD' P2T"; $resn_exch{"HD'3 P2T"}="3HD' P2T";
$resn_exch{" H3' P2T"}=" H3* P2T"; $resn_exch{"HO3' P2T"}="H3T  P2T"; $resn_exch{"HOP3 P2T"}="3HOP P2T";
$resn_exch{" H1' P2U"}=" H1* P2U"; $resn_exch{" H2' P2U"}="1H2* P2U"; $resn_exch{"H2'' P2U"}="2H2* P2U";
$resn_exch{" H3' P2U"}=" H3* P2U"; $resn_exch{" H4' P2U"}=" H4* P2U"; $resn_exch{"HO3' P2U"}="H3T  P2U";
$resn_exch{" H5' P2U"}="1H5* P2U"; $resn_exch{"H5'' P2U"}="2H5* P2U"; $resn_exch{"HOP2 P2U"}="2HOP P2U";
$resn_exch{"HOP3 P2U"}="3HOP P2U"; $resn_exch{"HNA1 P2Y"}="1HNA P2Y"; $resn_exch{"HNA2 P2Y"}="2HNA P2Y";
$resn_exch{" H1  P2Y"}="1H   P2Y"; $resn_exch{" H2  P2Y"}="2H   P2Y"; $resn_exch{" H   P2Y"}=" HN  P2Y";
$resn_exch{" HD3 P2Y"}="1HD  P2Y"; $resn_exch{" HD2 P2Y"}="2HD  P2Y"; $resn_exch{" HG3 P2Y"}="1HG  P2Y";
$resn_exch{" HG2 P2Y"}="2HG  P2Y"; $resn_exch{" HB2 P2Y"}="1HB  P2Y"; $resn_exch{" HB3 P2Y"}="2HB  P2Y";
$resn_exch{"H211 P33"}="1H21 P33"; $resn_exch{"H212 P33"}="2H21 P33"; $resn_exch{"H201 P33"}="1H20 P33";
$resn_exch{"H202 P33"}="2H20 P33"; $resn_exch{"H181 P33"}="1H18 P33"; $resn_exch{"H182 P33"}="2H18 P33";
$resn_exch{"H171 P33"}="1H17 P33"; $resn_exch{"H172 P33"}="2H17 P33"; $resn_exch{"H151 P33"}="1H15 P33";
$resn_exch{"H152 P33"}="2H15 P33"; $resn_exch{"H141 P33"}="1H14 P33"; $resn_exch{"H142 P33"}="2H14 P33";
$resn_exch{"H121 P33"}="1H12 P33"; $resn_exch{"H122 P33"}="2H12 P33"; $resn_exch{"H111 P33"}="1H11 P33";
$resn_exch{"H112 P33"}="2H11 P33"; $resn_exch{" H91 P33"}="1H9  P33"; $resn_exch{" H92 P33"}="2H9  P33";
$resn_exch{" H81 P33"}="1H8  P33"; $resn_exch{" H82 P33"}="2H8  P33"; $resn_exch{" H61 P33"}="1H6  P33";
$resn_exch{" H62 P33"}="2H6  P33"; $resn_exch{" H51 P33"}="1H5  P33"; $resn_exch{" H52 P33"}="2H5  P33";
$resn_exch{" H31 P33"}="1H3  P33"; $resn_exch{" H32 P33"}="2H3  P33"; $resn_exch{" H21 P33"}="1H2  P33";
$resn_exch{"H22A P33"}="2H2  P33"; $resn_exch{"HAA1 P34"}="1HAA P34"; $resn_exch{"HAA2 P34"}="2HAA P34";
$resn_exch{"HAA3 P34"}="3HAA P34"; $resn_exch{"HAB1 P34"}="1HAB P34"; $resn_exch{"HAB2 P34"}="2HAB P34";
$resn_exch{"HAB3 P34"}="3HAB P34"; $resn_exch{" HA1 P34"}="1HA  P34"; $resn_exch{" HA2 P34"}="2HA  P34";
$resn_exch{"H441 P3A"}="1H44 P3A"; $resn_exch{"H442 P3A"}="2H44 P3A"; $resn_exch{"H451 P3A"}="1H45 P3A";
$resn_exch{"H452 P3A"}="2H45 P3A"; $resn_exch{"H461 P3A"}="1H46 P3A"; $resn_exch{"H462 P3A"}="2H46 P3A";
$resn_exch{"H471 P3A"}="1H47 P3A"; $resn_exch{"H472 P3A"}="2H47 P3A"; $resn_exch{"H481 P3A"}="1H48 P3A";
$resn_exch{"H482 P3A"}="2H48 P3A"; $resn_exch{"H491 P3A"}="1H49 P3A"; $resn_exch{"H492 P3A"}="2H49 P3A";
$resn_exch{"H501 P3A"}="1H50 P3A"; $resn_exch{"H502 P3A"}="2H50 P3A"; $resn_exch{"H511 P3A"}="1H51 P3A";
$resn_exch{"H512 P3A"}="2H51 P3A"; $resn_exch{"H521 P3A"}="1H52 P3A"; $resn_exch{"H522 P3A"}="2H52 P3A";
$resn_exch{"H531 P3A"}="1H53 P3A"; $resn_exch{"H532 P3A"}="2H53 P3A"; $resn_exch{"H541 P3A"}="1H54 P3A";
$resn_exch{"H542 P3A"}="2H54 P3A"; $resn_exch{"H551 P3A"}="1H55 P3A"; $resn_exch{"H552 P3A"}="2H55 P3A";
$resn_exch{"H561 P3A"}="1H56 P3A"; $resn_exch{"H562 P3A"}="2H56 P3A"; $resn_exch{"H571 P3A"}="1H57 P3A";
$resn_exch{"H572 P3A"}="2H57 P3A"; $resn_exch{"H581 P3A"}="1H58 P3A"; $resn_exch{"H582 P3A"}="2H58 P3A";
$resn_exch{"H583 P3A"}="3H58 P3A"; $resn_exch{"H401 P3A"}="1H40 P3A"; $resn_exch{"H402 P3A"}="2H40 P3A";
$resn_exch{"H391 P3A"}="1H39 P3A"; $resn_exch{"H392 P3A"}="2H39 P3A"; $resn_exch{"H141 P3A"}="1H14 P3A";
$resn_exch{"H142 P3A"}="2H14 P3A"; $resn_exch{" H81 P3A"}="1H8  P3A"; $resn_exch{" H82 P3A"}="2H8  P3A";
$resn_exch{" H11 P3A"}="1H1  P3A"; $resn_exch{" H12 P3A"}="2H1  P3A"; $resn_exch{" H41 P3A"}="1H4  P3A";
$resn_exch{" H42 P3A"}="2H4  P3A"; $resn_exch{"H221 P3A"}="1H22 P3A"; $resn_exch{"H222 P3A"}="2H22 P3A";
$resn_exch{"H231 P3A"}="1H23 P3A"; $resn_exch{"H232 P3A"}="2H23 P3A"; $resn_exch{"H241 P3A"}="1H24 P3A";
$resn_exch{"H242 P3A"}="2H24 P3A"; $resn_exch{"H251 P3A"}="1H25 P3A"; $resn_exch{"H252 P3A"}="2H25 P3A";
$resn_exch{"H261 P3A"}="1H26 P3A"; $resn_exch{"H262 P3A"}="2H26 P3A"; $resn_exch{"H271 P3A"}="1H27 P3A";
$resn_exch{"H272 P3A"}="2H27 P3A"; $resn_exch{"H281 P3A"}="1H28 P3A"; $resn_exch{"H282 P3A"}="2H28 P3A";
$resn_exch{"H311 P3A"}="1H31 P3A"; $resn_exch{"H312 P3A"}="2H31 P3A"; $resn_exch{"H321 P3A"}="1H32 P3A";
$resn_exch{"H322 P3A"}="2H32 P3A"; $resn_exch{"H331 P3A"}="1H33 P3A"; $resn_exch{"H332 P3A"}="2H33 P3A";
$resn_exch{"H341 P3A"}="1H34 P3A"; $resn_exch{"H342 P3A"}="2H34 P3A"; $resn_exch{"H351 P3A"}="1H35 P3A";
$resn_exch{"H352 P3A"}="2H35 P3A"; $resn_exch{"H361 P3A"}="1H36 P3A"; $resn_exch{"H362 P3A"}="2H36 P3A";
$resn_exch{"H363 P3A"}="3H36 P3A"; $resn_exch{"HAN1 P3D"}="1HAN P3D"; $resn_exch{"HAN2 P3D"}="2HAN P3D";
$resn_exch{"HAA1 P3D"}="1HAA P3D"; $resn_exch{"HAA2 P3D"}="2HAA P3D"; $resn_exch{"HAA3 P3D"}="3HAA P3D";
$resn_exch{"HAG1 P3D"}="1HAG P3D"; $resn_exch{"HAM1 P3D"}="1HAM P3D"; $resn_exch{"HAM2 P3D"}="2HAM P3D";
$resn_exch{"HAL1 P3D"}="1HAL P3D"; $resn_exch{"HAL2 P3D"}="2HAL P3D"; $resn_exch{"HAK1 P3D"}="1HAK P3D";
$resn_exch{"HAK2 P3D"}="2HAK P3D"; $resn_exch{"HAJ1 P3D"}="1HAJ P3D"; $resn_exch{"HAJ2 P3D"}="2HAJ P3D";
$resn_exch{"HAI1 P3D"}="1HAI P3D"; $resn_exch{"HAI2 P3D"}="2HAI P3D"; $resn_exch{"HAB1 P3D"}="1HAB P3D";
$resn_exch{"HAB2 P3D"}="2HAB P3D"; $resn_exch{" H51 P3F"}="1H5  P3F"; $resn_exch{" H52 P3F"}="2H5  P3F";
$resn_exch{" H11 P3F"}="1H1  P3F"; $resn_exch{"H12A P3F"}="2H1  P3F"; $resn_exch{"H13A P3F"}="3H1  P3F";
$resn_exch{" H91 P3F"}="1H9  P3F"; $resn_exch{" H92 P3F"}="2H9  P3F"; $resn_exch{"H101 P3G"}="1H10 P3G";
$resn_exch{"H102 P3G"}="2H10 P3G"; $resn_exch{"H111 P3G"}="1H11 P3G"; $resn_exch{"H112 P3G"}="2H11 P3G";
$resn_exch{"H121 P3G"}="1H12 P3G"; $resn_exch{"H122 P3G"}="2H12 P3G"; $resn_exch{"H131 P3G"}="1H13 P3G";
$resn_exch{"H132 P3G"}="2H13 P3G"; $resn_exch{"H141 P3G"}="1H14 P3G"; $resn_exch{"H142 P3G"}="2H14 P3G";
$resn_exch{"H151 P3G"}="1H15 P3G"; $resn_exch{"H152 P3G"}="2H15 P3G"; $resn_exch{"H162 P3G"}="2H16 P3G";
$resn_exch{"H161 P3G"}="1H16 P3G"; $resn_exch{"HOP1 P3M"}="1HOP P3M"; $resn_exch{"HOP2 P3M"}="2HOP P3M";
$resn_exch{"HOA2 P3M"}="2HOA P3M"; $resn_exch{"HOA3 P3M"}="3HOA P3M"; $resn_exch{"HOA4 P3M"}="4HOA P3M";
$resn_exch{"H6A1 P3M"}="1H6A P3M"; $resn_exch{"H6A2 P3M"}="2H6A P3M"; $resn_exch{"HOB2 P3M"}="2HOB P3M";
$resn_exch{"HOB4 P3M"}="4HOB P3M"; $resn_exch{"H6B1 P3M"}="1H6B P3M"; $resn_exch{"H6B2 P3M"}="2H6B P3M";
$resn_exch{"HOB6 P3M"}="6HOB P3M"; $resn_exch{"HOC1 P3M"}="1HOC P3M"; $resn_exch{"HOC2 P3M"}="2HOC P3M";
$resn_exch{"HOC4 P3M"}="4HOC P3M"; $resn_exch{"H6C1 P3M"}="1H6C P3M"; $resn_exch{"H6C2 P3M"}="2H6C P3M";
$resn_exch{"HOC6 P3M"}="6HOC P3M"; $resn_exch{"HNP1 P3P"}="1HNP P3P"; $resn_exch{"HNP2 P3P"}="2HNP P3P";
$resn_exch{"HBP1 P3P"}="1HBP P3P"; $resn_exch{"HBP2 P3P"}="2HBP P3P"; $resn_exch{"HGP1 P3P"}="1HGP P3P";
$resn_exch{"HGP2 P3P"}="2HGP P3P"; $resn_exch{"HEP1 P3P"}="1HEP P3P"; $resn_exch{"HEP2 P3P"}="2HEP P3P";
$resn_exch{"HEP3 P3P"}="3HEP P3P"; $resn_exch{" HO3 P3S"}="3HO  P3S"; $resn_exch{" HO2 P3S"}="2HO  P3S";
$resn_exch{"HEC1 P3S"}="1HEC P3S"; $resn_exch{"HEC2 P3S"}="2HEC P3S"; $resn_exch{"HEC3 P3S"}="3HEC P3S";
$resn_exch{"HGC1 P3S"}="1HGC P3S"; $resn_exch{"HGC2 P3S"}="2HGC P3S"; $resn_exch{"HBC1 P3S"}="1HBC P3S";
$resn_exch{"HBC2 P3S"}="2HBC P3S"; $resn_exch{" HN1 P3S"}="1HN  P3S"; $resn_exch{" HN2 P3S"}="2HN  P3S";
$resn_exch{"HAM1 P3T"}="1HAM P3T"; $resn_exch{"HAM2 P3T"}="2HAM P3T"; $resn_exch{"HAA1 P3T"}="1HAA P3T";
$resn_exch{"HAA2 P3T"}="2HAA P3T"; $resn_exch{"HAA3 P3T"}="3HAA P3T"; $resn_exch{"HAG1 P3T"}="1HAG P3T";
$resn_exch{"HAG2 P3T"}="2HAG P3T"; $resn_exch{"HAL1 P3T"}="1HAL P3T"; $resn_exch{"HAL2 P3T"}="2HAL P3T";
$resn_exch{"HAK1 P3T"}="1HAK P3T"; $resn_exch{"HAK2 P3T"}="2HAK P3T"; $resn_exch{"HAJ1 P3T"}="1HAJ P3T";
$resn_exch{"HAJ2 P3T"}="2HAJ P3T"; $resn_exch{"HAI1 P3T"}="1HAI P3T"; $resn_exch{"HAI2 P3T"}="2HAI P3T";
$resn_exch{"HAB1 P3T"}="1HAB P3T"; $resn_exch{"HAB2 P3T"}="2HAB P3T"; $resn_exch{"H11A P3Y"}="1H1  P3Y";
$resn_exch{" H12 P3Y"}="2H1  P3Y"; $resn_exch{" H13 P3Y"}="3H1  P3Y"; $resn_exch{"H411 P3Y"}="1H41 P3Y";
$resn_exch{"H412 P3Y"}="2H41 P3Y"; $resn_exch{"H413 P3Y"}="3H41 P3Y"; $resn_exch{"H451 P3Y"}="1H45 P3Y";
$resn_exch{"H452 P3Y"}="2H45 P3Y"; $resn_exch{"H453 P3Y"}="3H45 P3Y"; $resn_exch{"H21A P4C"}="1H2  P4C";
$resn_exch{" H22 P4C"}="2H2  P4C"; $resn_exch{" H31 P4C"}="1H3  P4C"; $resn_exch{" H32 P4C"}="2H3  P4C";
$resn_exch{" H51 P4C"}="1H5  P4C"; $resn_exch{" H52 P4C"}="2H5  P4C"; $resn_exch{" H61 P4C"}="1H6  P4C";
$resn_exch{" H62 P4C"}="2H6  P4C"; $resn_exch{" H81 P4C"}="1H8  P4C"; $resn_exch{" H82 P4C"}="2H8  P4C";
$resn_exch{" H91 P4C"}="1H9  P4C"; $resn_exch{" H92 P4C"}="2H9  P4C"; $resn_exch{"H111 P4C"}="1H11 P4C";
$resn_exch{"H112 P4C"}="2H11 P4C"; $resn_exch{"H121 P4C"}="1H12 P4C"; $resn_exch{"H122 P4C"}="2H12 P4C";
$resn_exch{"H141 P4C"}="1H14 P4C"; $resn_exch{"H142 P4C"}="2H14 P4C"; $resn_exch{"H151 P4C"}="1H15 P4C";
$resn_exch{"H152 P4C"}="2H15 P4C"; $resn_exch{"H171 P4C"}="1H17 P4C"; $resn_exch{"H172 P4C"}="2H17 P4C";
$resn_exch{"H181 P4C"}="1H18 P4C"; $resn_exch{"H182 P4C"}="2H18 P4C"; $resn_exch{"H201 P4C"}="1H20 P4C";
$resn_exch{"H202 P4C"}="2H20 P4C"; $resn_exch{" H81 P4G"}="1H8  P4G"; $resn_exch{" H82 P4G"}="2H8  P4G";
$resn_exch{" H83 P4G"}="3H8  P4G"; $resn_exch{" H71 P4G"}="1H7  P4G"; $resn_exch{" H72 P4G"}="2H7  P4G";
$resn_exch{" H61 P4G"}="1H6  P4G"; $resn_exch{" H62 P4G"}="2H6  P4G"; $resn_exch{" H51 P4G"}="1H5  P4G";
$resn_exch{" H52 P4G"}="2H5  P4G"; $resn_exch{" H41 P4G"}="1H4  P4G"; $resn_exch{" H42 P4G"}="2H4  P4G";
$resn_exch{" H31 P4G"}="1H3  P4G"; $resn_exch{" H32 P4G"}="2H3  P4G"; $resn_exch{" H21 P4G"}="1H2  P4G";
$resn_exch{" H22 P4G"}="2H2  P4G"; $resn_exch{" H11 P4G"}="1H1  P4G"; $resn_exch{" H12 P4G"}="2H1  P4G";
$resn_exch{" H13 P4G"}="3H1  P4G"; $resn_exch{"H8C1 P4O"}="1H8C P4O"; $resn_exch{"H8C2 P4O"}="2H8C P4O";
$resn_exch{"H9C1 P4O"}="1H9C P4O"; $resn_exch{"H9C2 P4O"}="2H9C P4O"; $resn_exch{"H6C1 P4T"}="1H6C P4T";
$resn_exch{"H6C2 P4T"}="2H6C P4T"; $resn_exch{"H6C3 P4T"}="3H6C P4T"; $resn_exch{"H2C1 P4T"}="1H2C P4T";
$resn_exch{"H2C2 P4T"}="2H2C P4T"; $resn_exch{"H2C3 P4T"}="3H2C P4T"; $resn_exch{" HB1 P5A"}="1HB  P5A";
$resn_exch{" HB2 P5A"}="2HB  P5A"; $resn_exch{" HG1 P5A"}="1HG  P5A"; $resn_exch{" HG2 P5A"}="2HG  P5A";
$resn_exch{" HD1 P5A"}="1HD  P5A"; $resn_exch{" HD2 P5A"}="2HD  P5A"; $resn_exch{"H1'1 P5A"}="1H1* P5A";
$resn_exch{"H1'2 P5A"}="2H1* P5A"; $resn_exch{" H4' P5A"}=" H4* P5A"; $resn_exch{" H1' P5A"}=" H1* P5A";
$resn_exch{" H61 P5A"}="1H6  P5A"; $resn_exch{" H62 P5A"}="2H6  P5A"; $resn_exch{" H2' P5A"}=" H2* P5A";
$resn_exch{"HO'2 P5A"}="2HO* P5A"; $resn_exch{" H3' P5A"}=" H3* P5A"; $resn_exch{"H181 P5B"}="1H18 P5B";
$resn_exch{"H182 P5B"}="2H18 P5B"; $resn_exch{"H191 P5B"}="1H19 P5B"; $resn_exch{"H192 P5B"}="2H19 P5B";
$resn_exch{"H211 P5B"}="1H21 P5B"; $resn_exch{"H212 P5B"}="2H21 P5B"; $resn_exch{"HN31 P5B"}="1HN3 P5B";
$resn_exch{"HN32 P5B"}="2HN3 P5B"; $resn_exch{"H11A P5B"}="1H1  P5B"; $resn_exch{" H12 P5B"}="2H1  P5B";
$resn_exch{" H91 P5B"}="1H9  P5B"; $resn_exch{" H92 P5B"}="2H9  P5B"; $resn_exch{" H81 P5B"}="1H8  P5B";
$resn_exch{" H82 P5B"}="2H8  P5B"; $resn_exch{" H83 P5B"}="3H8  P5B"; $resn_exch{"HN21 P5B"}="1HN2 P5B";
$resn_exch{"HN22 P5B"}="2HN2 P5B"; $resn_exch{"HO2' P5P"}="'HO2 P5P"; $resn_exch{"HO3' P5P"}="'HO3 P5P";
$resn_exch{"H5'1 P5P"}="1H5' P5P"; $resn_exch{"H5'2 P5P"}="2H5' P5P"; $resn_exch{"HOP2 P5P"}="2HOP P5P";
$resn_exch{"HOP3 P5P"}="3HOP P5P"; $resn_exch{"H171 P6C"}="1H17 P6C"; $resn_exch{"H172 P6C"}="2H17 P6C";
$resn_exch{" H61 P6C"}="1H6  P6C"; $resn_exch{" H62 P6C"}="2H6  P6C"; $resn_exch{" H63 P6C"}="3H6  P6C";
$resn_exch{" H41 P6C"}="1H4  P6C"; $resn_exch{" H42 P6C"}="2H4  P6C"; $resn_exch{" H43 P6C"}="3H4  P6C";
$resn_exch{" H21 P6G"}="1H2  P6G"; $resn_exch{" H22 P6G"}="2H2  P6G"; $resn_exch{" H31 P6G"}="1H3  P6G";
$resn_exch{" H32 P6G"}="2H3  P6G"; $resn_exch{" H51 P6G"}="1H5  P6G"; $resn_exch{" H52 P6G"}="2H5  P6G";
$resn_exch{" H61 P6G"}="1H6  P6G"; $resn_exch{" H62 P6G"}="2H6  P6G"; $resn_exch{" H81 P6G"}="1H8  P6G";
$resn_exch{" H82 P6G"}="2H8  P6G"; $resn_exch{" H91 P6G"}="1H9  P6G"; $resn_exch{" H92 P6G"}="2H9  P6G";
$resn_exch{"H111 P6G"}="1H11 P6G"; $resn_exch{"H112 P6G"}="2H11 P6G"; $resn_exch{"H121 P6G"}="1H12 P6G";
$resn_exch{"H122 P6G"}="2H12 P6G"; $resn_exch{"H141 P6G"}="1H14 P6G"; $resn_exch{"H142 P6G"}="2H14 P6G";
$resn_exch{"H151 P6G"}="1H15 P6G"; $resn_exch{"H152 P6G"}="2H15 P6G"; $resn_exch{"H171 P6G"}="1H17 P6G";
$resn_exch{"H172 P6G"}="2H17 P6G"; $resn_exch{"H181 P6G"}="1H18 P6G"; $resn_exch{"H182 P6G"}="2H18 P6G";
$resn_exch{" H11 P6L"}="1H1  P6L"; $resn_exch{" H12 P6L"}="2H1  P6L"; $resn_exch{" H61 P6L"}="1H6  P6L";
$resn_exch{" H62 P6L"}="2H6  P6L"; $resn_exch{" H71 P6L"}="1H7  P6L"; $resn_exch{" H72 P6L"}="2H7  P6L";
$resn_exch{"H181 P6L"}="1H18 P6L"; $resn_exch{"H182 P6L"}="2H18 P6L"; $resn_exch{"H191 P6L"}="1H19 P6L";
$resn_exch{"H192 P6L"}="2H19 P6L"; $resn_exch{"H201 P6L"}="1H20 P6L"; $resn_exch{"H202 P6L"}="2H20 P6L";
$resn_exch{"H211 P6L"}="1H21 P6L"; $resn_exch{"H212 P6L"}="2H21 P6L"; $resn_exch{"H241 P6L"}="1H24 P6L";
$resn_exch{"H242 P6L"}="2H24 P6L"; $resn_exch{"H251 P6L"}="1H25 P6L"; $resn_exch{"H252 P6L"}="2H25 P6L";
$resn_exch{"H261 P6L"}="1H26 P6L"; $resn_exch{"H262 P6L"}="2H26 P6L"; $resn_exch{"H271 P6L"}="1H27 P6L";
$resn_exch{"H272 P6L"}="2H27 P6L"; $resn_exch{"H281 P6L"}="1H28 P6L"; $resn_exch{"H282 P6L"}="2H28 P6L";
$resn_exch{"H291 P6L"}="1H29 P6L"; $resn_exch{"H292 P6L"}="2H29 P6L"; $resn_exch{"H301 P6L"}="1H30 P6L";
$resn_exch{"H302 P6L"}="2H30 P6L"; $resn_exch{"H311 P6L"}="1H31 P6L"; $resn_exch{"H312 P6L"}="2H31 P6L";
$resn_exch{"H321 P6L"}="1H32 P6L"; $resn_exch{"H322 P6L"}="2H32 P6L"; $resn_exch{"H351 P6L"}="1H35 P6L";
$resn_exch{"H352 P6L"}="2H35 P6L"; $resn_exch{"H361 P6L"}="1H36 P6L"; $resn_exch{"H362 P6L"}="2H36 P6L";
$resn_exch{"H371 P6L"}="1H37 P6L"; $resn_exch{"H372 P6L"}="2H37 P6L"; $resn_exch{"H381 P6L"}="1H38 P6L";
$resn_exch{"H382 P6L"}="2H38 P6L"; $resn_exch{"H391 P6L"}="1H39 P6L"; $resn_exch{"H392 P6L"}="2H39 P6L";
$resn_exch{"H401 P6L"}="1H40 P6L"; $resn_exch{"H402 P6L"}="2H40 P6L"; $resn_exch{"H411 P6L"}="1H41 P6L";
$resn_exch{"H412 P6L"}="2H41 P6L"; $resn_exch{"H421 P6L"}="1H42 P6L"; $resn_exch{"H422 P6L"}="2H42 P6L";
$resn_exch{"H431 P6L"}="1H43 P6L"; $resn_exch{"H432 P6L"}="2H43 P6L"; $resn_exch{"H441 P6L"}="1H44 P6L";
$resn_exch{"H442 P6L"}="2H44 P6L"; $resn_exch{"H443 P6L"}="3H44 P6L"; $resn_exch{"H451 P6L"}="1H45 P6L";
$resn_exch{"H452 P6L"}="2H45 P6L"; $resn_exch{"H461 P6L"}="1H46 P6L"; $resn_exch{"H462 P6L"}="2H46 P6L";
$resn_exch{"H471 P6L"}="1H47 P6L"; $resn_exch{"H472 P6L"}="2H47 P6L"; $resn_exch{"H481 P6L"}="1H48 P6L";
$resn_exch{"H482 P6L"}="2H48 P6L"; $resn_exch{"H491 P6L"}="1H49 P6L"; $resn_exch{"H492 P6L"}="2H49 P6L";
$resn_exch{"H493 P6L"}="3H49 P6L"; $resn_exch{"H501 P6L"}="1H50 P6L"; $resn_exch{"H502 P6L"}="2H50 P6L";
$resn_exch{"H321 P90"}="1H32 P90"; $resn_exch{"H322 P90"}="2H32 P90"; $resn_exch{"H121 P90"}="1H12 P90";
$resn_exch{"H122 P90"}="2H12 P90"; $resn_exch{"HN21 PA1"}="1HN2 PA1"; $resn_exch{"HN22 PA1"}="2HN2 PA1";
$resn_exch{" H61 PA1"}="1H6  PA1"; $resn_exch{" H62 PA1"}="2H6  PA1"; $resn_exch{"HN11 PA2"}="1HN1 PA2";
$resn_exch{"HN12 PA2"}="2HN1 PA2"; $resn_exch{" H41 PA2"}="1H4  PA2"; $resn_exch{" H42 PA2"}="2H4  PA2";
$resn_exch{"HN51 PA2"}="1HN5 PA2"; $resn_exch{"HN52 PA2"}="2HN5 PA2"; $resn_exch{" H61 PA2"}="1H6  PA2";
$resn_exch{" H62 PA2"}="2H6  PA2"; $resn_exch{" H11 PA3"}="1H1  PA3"; $resn_exch{" H12 PA3"}="2H1  PA3";
$resn_exch{" H31 PA3"}="1H3  PA3"; $resn_exch{" H32 PA3"}="2H3  PA3"; $resn_exch{" H51 PA3"}="1H5  PA3";
$resn_exch{" H52 PA3"}="2H5  PA3"; $resn_exch{"HN31 PA4"}="1HN3 PA4"; $resn_exch{"HN32 PA4"}="2HN3 PA4";
$resn_exch{" H61 PA4"}="1H6  PA4"; $resn_exch{" H62 PA4"}="2H6  PA4"; $resn_exch{"HN61 PA4"}="1HN6 PA4";
$resn_exch{"HN62 PA4"}="2HN6 PA4"; $resn_exch{"HOP2 PA5"}="2HOP PA5"; $resn_exch{"HOP3 PA5"}="3HOP PA5";
$resn_exch{"H321 PA6"}="1H32 PA6"; $resn_exch{"H322 PA6"}="2H32 PA6"; $resn_exch{"H331 PA6"}="1H33 PA6";
$resn_exch{"H332 PA6"}="2H33 PA6"; $resn_exch{"H341 PA6"}="1H34 PA6"; $resn_exch{"H342 PA6"}="2H34 PA6";
$resn_exch{"H351 PA6"}="1H35 PA6"; $resn_exch{"H352 PA6"}="2H35 PA6"; $resn_exch{"H353 PA6"}="3H35 PA6";
$resn_exch{" H31 PA6"}="1H3  PA6"; $resn_exch{" H32 PA6"}="2H3  PA6"; $resn_exch{" H11 PA6"}="1H1  PA6";
$resn_exch{" H12 PA6"}="2H1  PA6"; $resn_exch{" H5' PA7"}=" H5* PA7"; $resn_exch{"H511 PA7"}="1H51 PA7";
$resn_exch{"H512 PA7"}="2H51 PA7"; $resn_exch{"H521 PA7"}="1H52 PA7"; $resn_exch{"H522 PA7"}="2H52 PA7";
$resn_exch{"H531 PA7"}="1H53 PA7"; $resn_exch{"H532 PA7"}="2H53 PA7"; $resn_exch{"H533 PA7"}="3H53 PA7";
$resn_exch{" H4' PA7"}=" H4* PA7"; $resn_exch{" H3' PA7"}=" H3* PA7"; $resn_exch{" H2' PA7"}=" H2* PA7";
$resn_exch{" H1' PA7"}=" H1* PA7"; $resn_exch{"HN61 PA7"}="1HN6 PA7"; $resn_exch{"HN62 PA7"}="2HN6 PA7";
$resn_exch{" HM1 PAA"}="1HM  PAA"; $resn_exch{" HM2 PAA"}="2HM  PAA"; $resn_exch{" HM3 PAA"}="3HM  PAA";
$resn_exch{"H2'1 PAA"}="1H2' PAA"; $resn_exch{"H2'2 PAA"}="2H2' PAA"; $resn_exch{"HO2' PAB"}="'HO2 PAB";
$resn_exch{"HN41 PAB"}="1HN4 PAB"; $resn_exch{"HN42 PAB"}="2HN4 PAB"; $resn_exch{" H21 PAC"}="1H2  PAC";
$resn_exch{" H22 PAC"}="2H2  PAC"; $resn_exch{"HOA2 PAD"}="2HOA PAD"; $resn_exch{"H51A PAD"}="AH51 PAD";
$resn_exch{"H52A PAD"}="AH52 PAD"; $resn_exch{" H4B PAD"}="AH4* PAD"; $resn_exch{" H3B PAD"}="AH3* PAD";
$resn_exch{"HO3A PAD"}="AHO3 PAD"; $resn_exch{" H2B PAD"}="AH2* PAD"; $resn_exch{"HO2A PAD"}="AHO2 PAD";
$resn_exch{" H1B PAD"}="AH1* PAD"; $resn_exch{" H8A PAD"}=" AH8 PAD"; $resn_exch{"H61A PAD"}="AH61 PAD";
$resn_exch{"H62A PAD"}="AH62 PAD"; $resn_exch{" H2A PAD"}=" AH2 PAD"; $resn_exch{"H51N PAD"}="NH51 PAD";
$resn_exch{"H52N PAD"}="NH52 PAD"; $resn_exch{" H4D PAD"}="NH4* PAD"; $resn_exch{" H3D PAD"}="NH3* PAD";
$resn_exch{"HO3N PAD"}="NHO3 PAD"; $resn_exch{" H2D PAD"}="NH2* PAD"; $resn_exch{"HO2N PAD"}="NHO2 PAD";
$resn_exch{" H1D PAD"}="NH1* PAD"; $resn_exch{" H2N PAD"}=" NH2 PAD"; $resn_exch{"H71N PAD"}="NH71 PAD";
$resn_exch{"H72N PAD"}="NH72 PAD"; $resn_exch{" H4N PAD"}=" NH4 PAD"; $resn_exch{" H5N PAD"}=" NH5 PAD";
$resn_exch{" H6N PAD"}=" NH6 PAD"; $resn_exch{"HOP1 PAE"}="1HOP PAE"; $resn_exch{"HOP2 PAE"}="2HOP PAE";
$resn_exch{" H11 PAE"}="1H1  PAE"; $resn_exch{" H12 PAE"}="2H1  PAE"; $resn_exch{" H21 PAF"}="1H2  PAF";
$resn_exch{"HO31 PAF"}="1HO3 PAF"; $resn_exch{" H41 PAF"}="1H4  PAF"; $resn_exch{" H42 PAF"}="2H4  PAF";
$resn_exch{" H43 PAF"}="3H4  PAF"; $resn_exch{" H51 PAF"}="1H5  PAF"; $resn_exch{" H52 PAF"}="2H5  PAF";
$resn_exch{" H53 PAF"}="3H5  PAF"; $resn_exch{" H61 PAF"}="1H6  PAF"; $resn_exch{" H62 PAF"}="2H6  PAF";
$resn_exch{"HO41 PAF"}="1HO4 PAF"; $resn_exch{" H31 PAG"}="1H3  PAG"; $resn_exch{" H32 PAG"}="2H3  PAG";
$resn_exch{"HOP2 PAG"}="2HOP PAG"; $resn_exch{"HOP3 PAG"}="3HOP PAG"; $resn_exch{" H11 PAH"}="1H1  PAH";
$resn_exch{" H12 PAH"}="2H1  PAH"; $resn_exch{"HOP2 PAH"}="2HOP PAH"; $resn_exch{"HOP3 PAH"}="3HOP PAH";
$resn_exch{"HC61 PAI"}="1HC6 PAI"; $resn_exch{"HC62 PAI"}="2HC6 PAI"; $resn_exch{"HC11 PAI"}="1HC1 PAI";
$resn_exch{"HC12 PAI"}="2HC1 PAI"; $resn_exch{"HC71 PAI"}="1HC7 PAI"; $resn_exch{"HC72 PAI"}="2HC7 PAI";
$resn_exch{"HC91 PAI"}="1HC9 PAI"; $resn_exch{"HC92 PAI"}="2HC9 PAI"; $resn_exch{"HOP2 PAJ"}="2HOP PAJ";
$resn_exch{"H5'1 PAJ"}="1H5* PAJ"; $resn_exch{"H5'2 PAJ"}="2H5* PAJ"; $resn_exch{" H4' PAJ"}=" H4* PAJ";
$resn_exch{" H3' PAJ"}=" H3* PAJ"; $resn_exch{" H2' PAJ"}=" H2* PAJ"; $resn_exch{"HO'2 PAJ"}="2HO* PAJ";
$resn_exch{" H1' PAJ"}=" H1* PAJ"; $resn_exch{" H61 PAJ"}="1H6  PAJ"; $resn_exch{" H62 PAJ"}="2H6  PAJ";
$resn_exch{"H141 PAJ"}="1H14 PAJ"; $resn_exch{"H142 PAJ"}="2H14 PAJ"; $resn_exch{"H143 PAJ"}="3H14 PAJ";
$resn_exch{"H151 PAJ"}="1H15 PAJ"; $resn_exch{"H152 PAJ"}="2H15 PAJ"; $resn_exch{"H153 PAJ"}="3H15 PAJ";
$resn_exch{"H161 PAJ"}="1H16 PAJ"; $resn_exch{"H162 PAJ"}="2H16 PAJ"; $resn_exch{"HOP2 PAL"}="2HOP PAL";
$resn_exch{"HOP3 PAL"}="3HOP PAL"; $resn_exch{"H1P1 PAL"}="1H1P PAL"; $resn_exch{"H1P2 PAL"}="2H1P PAL";
$resn_exch{" H31 PAL"}="1H3  PAL"; $resn_exch{" H32 PAL"}="2H3  PAL"; $resn_exch{" H21 PAM"}="1H2  PAM";
$resn_exch{" H22 PAM"}="2H2  PAM"; $resn_exch{" H31 PAM"}="1H3  PAM"; $resn_exch{" H32 PAM"}="2H3  PAM";
$resn_exch{" H41 PAM"}="1H4  PAM"; $resn_exch{" H42 PAM"}="2H4  PAM"; $resn_exch{" H51 PAM"}="1H5  PAM";
$resn_exch{" H52 PAM"}="2H5  PAM"; $resn_exch{" H61 PAM"}="1H6  PAM"; $resn_exch{" H62 PAM"}="2H6  PAM";
$resn_exch{" H71 PAM"}="1H7  PAM"; $resn_exch{" H72 PAM"}="2H7  PAM"; $resn_exch{" H81 PAM"}="1H8  PAM";
$resn_exch{" H82 PAM"}="2H8  PAM"; $resn_exch{"H111 PAM"}="1H11 PAM"; $resn_exch{"H112 PAM"}="2H11 PAM";
$resn_exch{"H121 PAM"}="1H12 PAM"; $resn_exch{"H122 PAM"}="2H12 PAM"; $resn_exch{"H131 PAM"}="1H13 PAM";
$resn_exch{"H132 PAM"}="2H13 PAM"; $resn_exch{"H141 PAM"}="1H14 PAM"; $resn_exch{"H142 PAM"}="2H14 PAM";
$resn_exch{"H151 PAM"}="1H15 PAM"; $resn_exch{"H152 PAM"}="2H15 PAM"; $resn_exch{"H161 PAM"}="1H16 PAM";
$resn_exch{"H162 PAM"}="2H16 PAM"; $resn_exch{"H163 PAM"}="3H16 PAM"; $resn_exch{" H51 PAN"}="1H5  PAN";
$resn_exch{" H52 PAN"}="2H5  PAN"; $resn_exch{"HOP2 PAN"}="2HOP PAN"; $resn_exch{"HOP3 PAN"}="3HOP PAN";
$resn_exch{" HN1 PAO"}="1HN  PAO"; $resn_exch{" HN2 PAO"}="2HN  PAO"; $resn_exch{" HB1 PAO"}="1HB  PAO";
$resn_exch{" HB2 PAO"}="2HB  PAO"; $resn_exch{" HG1 PAO"}="1HG  PAO"; $resn_exch{" HG2 PAO"}="2HG  PAO";
$resn_exch{" HD1 PAO"}="1HD  PAO"; $resn_exch{" HD2 PAO"}="2HD  PAO"; $resn_exch{" HP1 PAO"}="1HP  PAO";
$resn_exch{" HP2 PAO"}="2HP  PAO"; $resn_exch{"HOP2 PAO"}="2HOP PAO"; $resn_exch{"HOP3 PAO"}="3HOP PAO";
$resn_exch{"HOB2 PAP"}="2HOB PAP"; $resn_exch{"HOB3 PAP"}="3HOB PAP"; $resn_exch{"HOA2 PAP"}="2HOA PAP";
$resn_exch{"H5'1 PAP"}="1H5* PAP"; $resn_exch{"H5'2 PAP"}="2H5* PAP"; $resn_exch{" H4' PAP"}=" H4* PAP";
$resn_exch{" H3' PAP"}=" H3* PAP"; $resn_exch{" H2' PAP"}=" H2* PAP"; $resn_exch{"HO2' PAP"}="*HO2 PAP";
$resn_exch{" H1' PAP"}=" H1* PAP"; $resn_exch{"HN61 PAP"}="1HN6 PAP"; $resn_exch{"HN62 PAP"}="2HN6 PAP";
$resn_exch{" H   PAQ"}="1HN  PAQ"; $resn_exch{" H2  PAQ"}="2HN  PAQ"; $resn_exch{" HB2 PAQ"}="1HB  PAQ";
$resn_exch{" HB3 PAQ"}="2HB  PAQ"; $resn_exch{" HH  PAQ"}=" HOH PAQ"; $resn_exch{" HC2 PAQ"}=" H2  PAQ";
$resn_exch{"HN21 PAR"}="1HN2 PAR"; $resn_exch{"HN22 PAR"}="2HN2 PAR"; $resn_exch{"HO31 PAR"}="1HO3 PAR";
$resn_exch{"HO41 PAR"}="1HO4 PAR"; $resn_exch{"H611 PAR"}="1H61 PAR"; $resn_exch{"H612 PAR"}="2H61 PAR";
$resn_exch{"HO61 PAR"}="1HO6 PAR"; $resn_exch{"H121 PAR"}="1H12 PAR"; $resn_exch{"H122 PAR"}="2H12 PAR";
$resn_exch{"H221 PAR"}="1H22 PAR"; $resn_exch{"H222 PAR"}="2H22 PAR"; $resn_exch{"H321 PAR"}="1H32 PAR";
$resn_exch{"H322 PAR"}="2H32 PAR"; $resn_exch{"HO62 PAR"}="2HO6 PAR"; $resn_exch{"HO23 PAR"}="3HO2 PAR";
$resn_exch{"H531 PAR"}="1H53 PAR"; $resn_exch{"H532 PAR"}="2H53 PAR"; $resn_exch{"HO53 PAR"}="3HO5 PAR";
$resn_exch{"H241 PAR"}="1H24 PAR"; $resn_exch{"H242 PAR"}="2H24 PAR"; $resn_exch{"HO34 PAR"}="4HO3 PAR";
$resn_exch{"HO44 PAR"}="4HO4 PAR"; $resn_exch{"H641 PAR"}="1H64 PAR"; $resn_exch{"H642 PAR"}="2H64 PAR";
$resn_exch{"HN61 PAR"}="1HN6 PAR"; $resn_exch{"HN62 PAR"}="2HN6 PAR"; $resn_exch{" HB1 PAS"}="1HB  PAS";
$resn_exch{" HB2 PAS"}="2HB  PAS"; $resn_exch{" HO1 PAS"}="1HO  PAS"; $resn_exch{" HO2 PAS"}="2HO  PAS";
$resn_exch{" H2  PAT"}=" HN2 PAT"; $resn_exch{" HB2 PAT"}="1HB  PAT"; $resn_exch{" HB3 PAT"}="2HB  PAT";
$resn_exch{"HOP2 PAT"}="2HOP PAT"; $resn_exch{"HOP3 PAT"}="3HOP PAT"; $resn_exch{" H21 PAU"}="1H2  PAU";
$resn_exch{" H22 PAU"}="2H2  PAU"; $resn_exch{" H31 PAU"}="1H3  PAU"; $resn_exch{" H32 PAU"}="2H3  PAU";
$resn_exch{" H33 PAU"}="3H3  PAU"; $resn_exch{" H41 PAU"}="1H4  PAU"; $resn_exch{" H42 PAU"}="2H4  PAU";
$resn_exch{" H43 PAU"}="3H4  PAU"; $resn_exch{" H5' PAU"}=" H5* PAU"; $resn_exch{" H81 PAU"}="1H8  PAU";
$resn_exch{" H82 PAU"}="2H8  PAU"; $resn_exch{" H91 PAU"}="1H9  PAU"; $resn_exch{" H92 PAU"}="2H9  PAU";
$resn_exch{"H111 PAV"}="1H11 PAV"; $resn_exch{"H112 PAV"}="2H11 PAV"; $resn_exch{"H113 PAV"}="3H11 PAV";
$resn_exch{" H61 PAV"}="1H6  PAV"; $resn_exch{" H62 PAV"}="2H6  PAV"; $resn_exch{"HN11 PAW"}="1HN1 PAW";
$resn_exch{"HN12 PAW"}="2HN1 PAW"; $resn_exch{" H21 PAW"}="1H2  PAW"; $resn_exch{" H22 PAW"}="2H2  PAW";
$resn_exch{" H31 PAW"}="1H3  PAW"; $resn_exch{" H32 PAW"}="2H3  PAW"; $resn_exch{" H51 PAW"}="1H5  PAW";
$resn_exch{" H52 PAW"}="2H5  PAW"; $resn_exch{" H61 PAW"}="1H6  PAW"; $resn_exch{" H62 PAW"}="2H6  PAW";
$resn_exch{" H81 PAW"}="1H8  PAW"; $resn_exch{" H82 PAW"}="2H8  PAW"; $resn_exch{" H91 PAW"}="1H9  PAW";
$resn_exch{" H92 PAW"}="2H9  PAW"; $resn_exch{"H111 PAW"}="1H11 PAW"; $resn_exch{"H112 PAW"}="2H11 PAW";
$resn_exch{"H121 PAW"}="1H12 PAW"; $resn_exch{"H122 PAW"}="2H12 PAW"; $resn_exch{"H131 PAW"}="1H13 PAW";
$resn_exch{"H132 PAW"}="2H13 PAW"; $resn_exch{"H6A1 PAX"}="1H6A PAX"; $resn_exch{"H6A2 PAX"}="2H6A PAX";
$resn_exch{"H5'1 PAX"}="1H5* PAX"; $resn_exch{"H5'2 PAX"}="2H5* PAX"; $resn_exch{" H4' PAX"}=" H4* PAX";
$resn_exch{" H1' PAX"}=" H1* PAX"; $resn_exch{" H2' PAX"}=" H2* PAX"; $resn_exch{" H3' PAX"}=" H3* PAX";
$resn_exch{" H51 PAX"}="1H5  PAX"; $resn_exch{" H52 PAX"}="2H5  PAX"; $resn_exch{" H21 PAX"}="1H2  PAX";
$resn_exch{" H22 PAX"}="2H2  PAX"; $resn_exch{"HM51 PAX"}="1HM5 PAX"; $resn_exch{"HM52 PAX"}="2HM5 PAX";
$resn_exch{"HM53 PAX"}="3HM5 PAX"; $resn_exch{" H71 PB1"}="1H7  PB1"; $resn_exch{" H72 PB1"}="2H7  PB1";
$resn_exch{" H61 PB1"}="1H6  PB1"; $resn_exch{" H62 PB1"}="2H6  PB1"; $resn_exch{" H21 PB1"}="1H2  PB1";
$resn_exch{" H22 PB1"}="2H2  PB1"; $resn_exch{"HN11 PB1"}="1HN1 PB1"; $resn_exch{"HN12 PB1"}="2HN1 PB1";
$resn_exch{" H91 PB1"}="1H9  PB1"; $resn_exch{" H92 PB1"}="2H9  PB1"; $resn_exch{" H51 PB1"}="1H5  PB1";
$resn_exch{" H52 PB1"}="2H5  PB1"; $resn_exch{"H111 PB1"}="1H11 PB1"; $resn_exch{"H112 PB1"}="2H11 PB1";
$resn_exch{"H121 PB1"}="1H12 PB1"; $resn_exch{"H122 PB1"}="2H12 PB1"; $resn_exch{" H71 PB3"}="1H7  PB3";
$resn_exch{" H72 PB3"}="2H7  PB3"; $resn_exch{" H73 PB3"}="3H7  PB3"; $resn_exch{" H81 PB3"}="1H8  PB3";
$resn_exch{" H82 PB3"}="2H8  PB3"; $resn_exch{" H83 PB3"}="3H8  PB3"; $resn_exch{"H111 PB3"}="1H11 PB3";
$resn_exch{"H112 PB3"}="2H11 PB3"; $resn_exch{"H171 PB3"}="1H17 PB3"; $resn_exch{"H172 PB3"}="2H17 PB3";
$resn_exch{" H71 PB4"}="1H7  PB4"; $resn_exch{" H72 PB4"}="2H7  PB4"; $resn_exch{" H73 PB4"}="3H7  PB4";
$resn_exch{" H81 PB4"}="1H8  PB4"; $resn_exch{" H82 PB4"}="2H8  PB4"; $resn_exch{" H83 PB4"}="3H8  PB4";
$resn_exch{"H101 PB4"}="1H10 PB4"; $resn_exch{"H102 PB4"}="2H10 PB4"; $resn_exch{"H131 PB4"}="1H13 PB4";
$resn_exch{"H132 PB4"}="2H13 PB4"; $resn_exch{"H111 PB5"}="1H11 PB5"; $resn_exch{"H112 PB5"}="2H11 PB5";
$resn_exch{"H171 PB5"}="1H17 PB5"; $resn_exch{"H172 PB5"}="2H17 PB5"; $resn_exch{" HA1 PBA"}="1HA  PBA";
$resn_exch{" HA2 PBA"}="2HA  PBA"; $resn_exch{" HB1 PBA"}="1HB  PBA"; $resn_exch{" HB2 PBA"}="2HB  PBA";
$resn_exch{" H   PBB"}="1HN  PBB"; $resn_exch{" H2  PBB"}="2HN  PBB"; $resn_exch{" HB2 PBB"}="1HB  PBB";
$resn_exch{" HB3 PBB"}="2HB  PBB"; $resn_exch{" HC2 PBB"}=" H2  PBB"; $resn_exch{" H71 PBB"}="1H7  PBB";
$resn_exch{" H72 PBB"}="2H7  PBB"; $resn_exch{" HD1 PBE"}="1HD  PBE"; $resn_exch{" HD2 PBE"}="2HD  PBE";
$resn_exch{" HB1 PBE"}="1HB  PBE"; $resn_exch{" HB2 PBE"}="2HB  PBE"; $resn_exch{" HG1 PBE"}="1HG  PBE";
$resn_exch{" HG2 PBE"}="2HG  PBE"; $resn_exch{" HE1 PBE"}="1HE  PBE"; $resn_exch{" HE2 PBE"}="2HE  PBE";
$resn_exch{" HE3 PBE"}="3HE  PBE"; $resn_exch{" HF1 PBE"}="1HF  PBE"; $resn_exch{" HF2 PBE"}="2HF  PBE";
$resn_exch{" HF3 PBE"}="3HF  PBE"; $resn_exch{" H   PBF"}="1HN  PBF"; $resn_exch{" H2  PBF"}="2HN  PBF";
$resn_exch{" HB2 PBF"}="1HB  PBF"; $resn_exch{" HB3 PBF"}="2HB  PBF"; $resn_exch{"HHA2 PBG"}="2HHA PBG";
$resn_exch{"HHA1 PBG"}="1HHA PBG"; $resn_exch{"H4A1 PBG"}="1H4A PBG"; $resn_exch{"H5A2 PBG"}="2H5A PBG";
$resn_exch{"H5A1 PBG"}="1H5A PBG"; $resn_exch{"H2AO PBG"}="OH2A PBG"; $resn_exch{"H7A2 PBG"}="2H7A PBG";
$resn_exch{"H7A1 PBG"}="1H7A PBG"; $resn_exch{"H8A2 PBG"}="2H8A PBG"; $resn_exch{"H8A1 PBG"}="1H8A PBG";
$resn_exch{"H4AO PBG"}="OH4A PBG"; $resn_exch{" H12 PBG"}="2H1  PBG"; $resn_exch{" H11 PBG"}="1H1  PBG";
$resn_exch{"HN11 PBI"}="1HN1 PBI"; $resn_exch{"HN12 PBI"}="2HN1 PBI"; $resn_exch{"HA11 PBI"}="1HA1 PBI";
$resn_exch{"HA12 PBI"}="2HA1 PBI"; $resn_exch{"HA31 PBI"}="1HA3 PBI"; $resn_exch{"HA32 PBI"}="2HA3 PBI";
$resn_exch{"HB21 PBI"}="1HB2 PBI"; $resn_exch{"HB22 PBI"}="2HB2 PBI"; $resn_exch{" H11 PBM"}="1H1  PBM";
$resn_exch{" H12 PBM"}="2H1  PBM"; $resn_exch{" H13 PBM"}="3H1  PBM"; $resn_exch{" H21 PBM"}="1H2  PBM";
$resn_exch{" H22 PBM"}="2H2  PBM"; $resn_exch{" H23 PBM"}="3H2  PBM"; $resn_exch{" H31 PBM"}="1H3  PBM";
$resn_exch{" H32 PBM"}="2H3  PBM"; $resn_exch{" H33 PBM"}="3H3  PBM"; $resn_exch{" H11 PBN"}="1H1  PBN";
$resn_exch{" H12 PBN"}="2H1  PBN"; $resn_exch{" H21 PBN"}="1H2  PBN"; $resn_exch{" H22 PBN"}="2H2  PBN";
$resn_exch{" H31 PBN"}="1H3  PBN"; $resn_exch{" H32 PBN"}="2H3  PBN"; $resn_exch{" H41 PBN"}="1H4  PBN";
$resn_exch{" H42 PBN"}="2H4  PBN"; $resn_exch{" HN1 PBN"}="1HN  PBN"; $resn_exch{" HN2 PBN"}="2HN  PBN";
$resn_exch{"H103 PBO"}="3H10 PBO"; $resn_exch{"H101 PBO"}="1H10 PBO"; $resn_exch{"H102 PBO"}="2H10 PBO";
$resn_exch{"H111 PBO"}="1H11 PBO"; $resn_exch{"H112 PBO"}="2H11 PBO"; $resn_exch{"H121 PBO"}="1H12 PBO";
$resn_exch{"H122 PBO"}="2H12 PBO"; $resn_exch{" HH1 PBP"}="1HH  PBP"; $resn_exch{" HH2 PBP"}="2HH  PBP";
$resn_exch{" HH3 PBP"}="3HH  PBP"; $resn_exch{"H171 PBS"}="1H17 PBS"; $resn_exch{"H172 PBS"}="2H17 PBS";
$resn_exch{"H173 PBS"}="3H17 PBS"; $resn_exch{"H161 PBS"}="1H16 PBS"; $resn_exch{"H162 PBS"}="2H16 PBS";
$resn_exch{"H151 PBS"}="1H15 PBS"; $resn_exch{"H152 PBS"}="2H15 PBS"; $resn_exch{"H141 PBS"}="1H14 PBS";
$resn_exch{"H142 PBS"}="2H14 PBS"; $resn_exch{"H131 PBS"}="1H13 PBS"; $resn_exch{"H132 PBS"}="2H13 PBS";
$resn_exch{"H121 PBS"}="1H12 PBS"; $resn_exch{"H122 PBS"}="2H12 PBS"; $resn_exch{"H111 PBS"}="1H11 PBS";
$resn_exch{"H112 PBS"}="2H11 PBS"; $resn_exch{"H101 PBS"}="1H10 PBS"; $resn_exch{"H102 PBS"}="2H10 PBS";
$resn_exch{" H91 PBS"}="1H9  PBS"; $resn_exch{" H92 PBS"}="2H9  PBS"; $resn_exch{" H81 PBS"}="1H8  PBS";
$resn_exch{" H82 PBS"}="2H8  PBS"; $resn_exch{" H71 PBS"}="1H7  PBS"; $resn_exch{" H72 PBS"}="2H7  PBS";
$resn_exch{" H61 PBS"}="1H6  PBS"; $resn_exch{" H62 PBS"}="2H6  PBS"; $resn_exch{" H51 PBS"}="1H5  PBS";
$resn_exch{" H52 PBS"}="2H5  PBS"; $resn_exch{" H41 PBS"}="1H4  PBS"; $resn_exch{" H42 PBS"}="2H4  PBS";
$resn_exch{"H191 PBS"}="1H19 PBS"; $resn_exch{"H192 PBS"}="2H19 PBS"; $resn_exch{"H201 PBS"}="1H20 PBS";
$resn_exch{"H202 PBS"}="2H20 PBS"; $resn_exch{"H211 PBS"}="1H21 PBS"; $resn_exch{"H212 PBS"}="2H21 PBS";
$resn_exch{"H221 PBS"}="1H22 PBS"; $resn_exch{"H222 PBS"}="2H22 PBS"; $resn_exch{"H231 PBS"}="1H23 PBS";
$resn_exch{"H232 PBS"}="2H23 PBS"; $resn_exch{"H241 PBS"}="1H24 PBS"; $resn_exch{"H242 PBS"}="2H24 PBS";
$resn_exch{"H251 PBS"}="1H25 PBS"; $resn_exch{"H252 PBS"}="2H25 PBS"; $resn_exch{"H253 PBS"}="3H25 PBS";
$resn_exch{" H1A PBS"}="1H   PBS"; $resn_exch{" H2A PBS"}="2H   PBS"; $resn_exch{"HG51 PBS"}="1HG5 PBS";
$resn_exch{"HG52 PBS"}="2HG5 PBS"; $resn_exch{"H5'1 PBT"}="1H5* PBT"; $resn_exch{"H5'2 PBT"}="2H5* PBT";
$resn_exch{" H4' PBT"}=" H4* PBT"; $resn_exch{" H3' PBT"}=" H3* PBT"; $resn_exch{"H2'1 PBT"}="1H2* PBT";
$resn_exch{"H2'2 PBT"}="2H2* PBT"; $resn_exch{" H1' PBT"}=" H1* PBT"; $resn_exch{"H5M1 PBT"}="1H5M PBT";
$resn_exch{"H5M2 PBT"}="2H5M PBT"; $resn_exch{"H5M3 PBT"}="3H5M PBT"; $resn_exch{"H6C1 PBT"}="1H6C PBT";
$resn_exch{"H6C2 PBT"}="2H6C PBT"; $resn_exch{"H3'1 PBU"}="1H3' PBU"; $resn_exch{"H3'2 PBU"}="2H3' PBU";
$resn_exch{"H1'1 PBU"}="1H1' PBU"; $resn_exch{"H1'2 PBU"}="2H1' PBU"; $resn_exch{" H02 PBU"}="2H0  PBU";
$resn_exch{" H03 PBU"}="3H0  PBU"; $resn_exch{" H06 PBU"}="6H0  PBU"; $resn_exch{" H81 PBU"}="1H8  PBU";
$resn_exch{" H82 PBU"}="2H8  PBU"; $resn_exch{" H91 PBU"}="1H9  PBU"; $resn_exch{" H92 PBU"}="2H9  PBU";
$resn_exch{"H121 PBU"}="1H12 PBU"; $resn_exch{"H122 PBU"}="2H12 PBU"; $resn_exch{"H131 PBU"}="1H13 PBU";
$resn_exch{"H132 PBU"}="2H13 PBU"; $resn_exch{"H141 PBU"}="1H14 PBU"; $resn_exch{"H142 PBU"}="2H14 PBU";
$resn_exch{"H143 PBU"}="3H14 PBU"; $resn_exch{"HN21 PBZ"}="1HN2 PBZ"; $resn_exch{"HN22 PBZ"}="2HN2 PBZ";
$resn_exch{"HN31 PBZ"}="1HN3 PBZ"; $resn_exch{"HN32 PBZ"}="2HN3 PBZ"; $resn_exch{"HN11 PBZ"}="1HN1 PBZ";
$resn_exch{"HN12 PBZ"}="2HN1 PBZ"; $resn_exch{" H11  PC"}="1H1   PC"; $resn_exch{" H12  PC"}="2H1   PC";
$resn_exch{" H21  PC"}="1H2   PC"; $resn_exch{" H22  PC"}="2H2   PC"; $resn_exch{" H31  PC"}="1H3   PC";
$resn_exch{" H32  PC"}="2H3   PC"; $resn_exch{" H33  PC"}="3H3   PC"; $resn_exch{" H41  PC"}="1H4   PC";
$resn_exch{" H42  PC"}="2H4   PC"; $resn_exch{" H43  PC"}="3H4   PC"; $resn_exch{" H51  PC"}="1H5   PC";
$resn_exch{" H52  PC"}="2H5   PC"; $resn_exch{" H53  PC"}="3H5   PC"; $resn_exch{"HL71 PC0"}="1HL7 PC0";
$resn_exch{"HL72 PC0"}="2HL7 PC0"; $resn_exch{"HL73 PC0"}="3HL7 PC0"; $resn_exch{"H111 PC0"}="1H11 PC0";
$resn_exch{"H112 PC0"}="2H11 PC0"; $resn_exch{"H121 PC0"}="1H12 PC0"; $resn_exch{"H122 PC0"}="2H12 PC0";
$resn_exch{" HA1 PC0"}="1HA  PC0"; $resn_exch{" HA2 PC0"}="2HA  PC0"; $resn_exch{" HB1 PC0"}="1HB  PC0";
$resn_exch{" HB2 PC0"}="2HB  PC0"; $resn_exch{" HB3 PC0"}="3HB  PC0"; $resn_exch{"H111 PC1"}="1H11 PC1";
$resn_exch{"H112 PC1"}="2H11 PC1"; $resn_exch{"H121 PC1"}="1H12 PC1"; $resn_exch{"H122 PC1"}="2H12 PC1";
$resn_exch{"H131 PC1"}="1H13 PC1"; $resn_exch{"H132 PC1"}="2H13 PC1"; $resn_exch{"H133 PC1"}="3H13 PC1";
$resn_exch{"H141 PC1"}="1H14 PC1"; $resn_exch{"H142 PC1"}="2H14 PC1"; $resn_exch{"H143 PC1"}="3H14 PC1";
$resn_exch{"H151 PC1"}="1H15 PC1"; $resn_exch{"H152 PC1"}="2H15 PC1"; $resn_exch{"H153 PC1"}="3H15 PC1";
$resn_exch{" H11 PC1"}="1H1  PC1"; $resn_exch{" H12 PC1"}="2H1  PC1"; $resn_exch{"H221 PC1"}="1H22 PC1";
$resn_exch{"H222 PC1"}="2H22 PC1"; $resn_exch{"H231 PC1"}="1H23 PC1"; $resn_exch{"H232 PC1"}="2H23 PC1";
$resn_exch{"H241 PC1"}="1H24 PC1"; $resn_exch{"H242 PC1"}="2H24 PC1"; $resn_exch{"H251 PC1"}="1H25 PC1";
$resn_exch{"H252 PC1"}="2H25 PC1"; $resn_exch{"H261 PC1"}="1H26 PC1"; $resn_exch{"H262 PC1"}="2H26 PC1";
$resn_exch{"H271 PC1"}="1H27 PC1"; $resn_exch{"H272 PC1"}="2H27 PC1"; $resn_exch{"H281 PC1"}="1H28 PC1";
$resn_exch{"H282 PC1"}="2H28 PC1"; $resn_exch{"H291 PC1"}="1H29 PC1"; $resn_exch{"H292 PC1"}="2H29 PC1";
$resn_exch{"H2A1 PC1"}="1H2A PC1"; $resn_exch{"H2A2 PC1"}="2H2A PC1"; $resn_exch{"H2B1 PC1"}="1H2B PC1";
$resn_exch{"H2B2 PC1"}="2H2B PC1"; $resn_exch{"H2C1 PC1"}="1H2C PC1"; $resn_exch{"H2C2 PC1"}="2H2C PC1";
$resn_exch{"H2D1 PC1"}="1H2D PC1"; $resn_exch{"H2D2 PC1"}="2H2D PC1"; $resn_exch{"H2E1 PC1"}="1H2E PC1";
$resn_exch{"H2E2 PC1"}="2H2E PC1"; $resn_exch{"H2F1 PC1"}="1H2F PC1"; $resn_exch{"H2F2 PC1"}="2H2F PC1";
$resn_exch{"H2G1 PC1"}="1H2G PC1"; $resn_exch{"H2G2 PC1"}="2H2G PC1"; $resn_exch{"H2H1 PC1"}="1H2H PC1";
$resn_exch{"H2H2 PC1"}="2H2H PC1"; $resn_exch{"H2I1 PC1"}="1H2I PC1"; $resn_exch{"H2I2 PC1"}="2H2I PC1";
$resn_exch{"H2I3 PC1"}="3H2I PC1"; $resn_exch{" H31 PC1"}="1H3  PC1"; $resn_exch{" H32 PC1"}="2H3  PC1";
$resn_exch{"H321 PC1"}="1H32 PC1"; $resn_exch{"H322 PC1"}="2H32 PC1"; $resn_exch{"H331 PC1"}="1H33 PC1";
$resn_exch{"H332 PC1"}="2H33 PC1"; $resn_exch{"H341 PC1"}="1H34 PC1"; $resn_exch{"H342 PC1"}="2H34 PC1";
$resn_exch{"H351 PC1"}="1H35 PC1"; $resn_exch{"H352 PC1"}="2H35 PC1"; $resn_exch{"H361 PC1"}="1H36 PC1";
$resn_exch{"H362 PC1"}="2H36 PC1"; $resn_exch{"H371 PC1"}="1H37 PC1"; $resn_exch{"H372 PC1"}="2H37 PC1";
$resn_exch{"H381 PC1"}="1H38 PC1"; $resn_exch{"H382 PC1"}="2H38 PC1"; $resn_exch{"H391 PC1"}="1H39 PC1";
$resn_exch{"H392 PC1"}="2H39 PC1"; $resn_exch{"H3A1 PC1"}="1H3A PC1"; $resn_exch{"H3A2 PC1"}="2H3A PC1";
$resn_exch{"H3B1 PC1"}="1H3B PC1"; $resn_exch{"H3B2 PC1"}="2H3B PC1"; $resn_exch{"H3C1 PC1"}="1H3C PC1";
$resn_exch{"H3C2 PC1"}="2H3C PC1"; $resn_exch{"H3D1 PC1"}="1H3D PC1"; $resn_exch{"H3D2 PC1"}="2H3D PC1";
$resn_exch{"H3E1 PC1"}="1H3E PC1"; $resn_exch{"H3E2 PC1"}="2H3E PC1"; $resn_exch{"H3F1 PC1"}="1H3F PC1";
$resn_exch{"H3F2 PC1"}="2H3F PC1"; $resn_exch{"H3G1 PC1"}="1H3G PC1"; $resn_exch{"H3G2 PC1"}="2H3G PC1";
$resn_exch{"H3H1 PC1"}="1H3H PC1"; $resn_exch{"H3H2 PC1"}="2H3H PC1"; $resn_exch{"H3I1 PC1"}="1H3I PC1";
$resn_exch{"H3I2 PC1"}="2H3I PC1"; $resn_exch{"H3I3 PC1"}="3H3I PC1"; $resn_exch{" H12 PC2"}="2H1  PC2";
$resn_exch{" H11 PC2"}="1H1  PC2"; $resn_exch{" H32 PC2"}="2H3  PC2"; $resn_exch{" H31 PC2"}="1H3  PC2";
$resn_exch{" H42 PC2"}="2H4  PC2"; $resn_exch{" H41 PC2"}="1H4  PC2"; $resn_exch{" H52 PC2"}="2H5  PC2";
$resn_exch{" H51 PC2"}="1H5  PC2"; $resn_exch{" H63 PC2"}="3H6  PC2"; $resn_exch{" H62 PC2"}="2H6  PC2";
$resn_exch{" H61 PC2"}="1H6  PC2"; $resn_exch{" H73 PC2"}="3H7  PC2"; $resn_exch{" H72 PC2"}="2H7  PC2";
$resn_exch{" H71 PC2"}="1H7  PC2"; $resn_exch{" H83 PC2"}="3H8  PC2"; $resn_exch{" H82 PC2"}="2H8  PC2";
$resn_exch{" H81 PC2"}="1H8  PC2"; $resn_exch{"H122 PC2"}="2H12 PC2"; $resn_exch{"H121 PC2"}="1H12 PC2";
$resn_exch{"H132 PC2"}="2H13 PC2"; $resn_exch{"H131 PC2"}="1H13 PC2"; $resn_exch{"H142 PC2"}="2H14 PC2";
$resn_exch{"H141 PC2"}="1H14 PC2"; $resn_exch{"H152 PC2"}="2H15 PC2"; $resn_exch{"H151 PC2"}="1H15 PC2";
$resn_exch{"H162 PC2"}="2H16 PC2"; $resn_exch{"H161 PC2"}="1H16 PC2"; $resn_exch{"H172 PC2"}="2H17 PC2";
$resn_exch{"H171 PC2"}="1H17 PC2"; $resn_exch{"H182 PC2"}="2H18 PC2"; $resn_exch{"H181 PC2"}="1H18 PC2";
$resn_exch{"H192 PC2"}="2H19 PC2"; $resn_exch{"H191 PC2"}="1H19 PC2"; $resn_exch{"H202 PC2"}="2H20 PC2";
$resn_exch{"H201 PC2"}="1H20 PC2"; $resn_exch{"H212 PC2"}="2H21 PC2"; $resn_exch{"H211 PC2"}="1H21 PC2";
$resn_exch{"H222 PC2"}="2H22 PC2"; $resn_exch{"H221 PC2"}="1H22 PC2"; $resn_exch{"H232 PC2"}="2H23 PC2";
$resn_exch{"H231 PC2"}="1H23 PC2"; $resn_exch{"H242 PC2"}="2H24 PC2"; $resn_exch{"H241 PC2"}="1H24 PC2";
$resn_exch{"H252 PC2"}="2H25 PC2"; $resn_exch{"H251 PC2"}="1H25 PC2"; $resn_exch{"H262 PC2"}="2H26 PC2";
$resn_exch{"H261 PC2"}="1H26 PC2"; $resn_exch{"H272 PC2"}="2H27 PC2"; $resn_exch{"H271 PC2"}="1H27 PC2";
$resn_exch{"H283 PC2"}="3H28 PC2"; $resn_exch{"H282 PC2"}="2H28 PC2"; $resn_exch{"H281 PC2"}="1H28 PC2";
$resn_exch{"H322 PC2"}="2H32 PC2"; $resn_exch{"H321 PC2"}="1H32 PC2"; $resn_exch{"H332 PC2"}="2H33 PC2";
$resn_exch{"H331 PC2"}="1H33 PC2"; $resn_exch{"H342 PC2"}="2H34 PC2"; $resn_exch{"H341 PC2"}="1H34 PC2";
$resn_exch{"H352 PC2"}="2H35 PC2"; $resn_exch{"H351 PC2"}="1H35 PC2"; $resn_exch{"H362 PC2"}="2H36 PC2";
$resn_exch{"H361 PC2"}="1H36 PC2"; $resn_exch{"H372 PC2"}="2H37 PC2"; $resn_exch{"H371 PC2"}="1H37 PC2";
$resn_exch{"H382 PC2"}="2H38 PC2"; $resn_exch{"H381 PC2"}="1H38 PC2"; $resn_exch{"H392 PC2"}="2H39 PC2";
$resn_exch{"H391 PC2"}="1H39 PC2"; $resn_exch{"H402 PC2"}="2H40 PC2"; $resn_exch{"H401 PC2"}="1H40 PC2";
$resn_exch{"H412 PC2"}="2H41 PC2"; $resn_exch{"H411 PC2"}="1H41 PC2"; $resn_exch{"H422 PC2"}="2H42 PC2";
$resn_exch{"H421 PC2"}="1H42 PC2"; $resn_exch{"H432 PC2"}="2H43 PC2"; $resn_exch{"H431 PC2"}="1H43 PC2";
$resn_exch{"H442 PC2"}="2H44 PC2"; $resn_exch{"H441 PC2"}="1H44 PC2"; $resn_exch{"H452 PC2"}="2H45 PC2";
$resn_exch{"H451 PC2"}="1H45 PC2"; $resn_exch{"H462 PC2"}="2H46 PC2"; $resn_exch{"H461 PC2"}="1H46 PC2";
$resn_exch{"H472 PC2"}="2H47 PC2"; $resn_exch{"H471 PC2"}="1H47 PC2"; $resn_exch{"H483 PC2"}="3H48 PC2";
$resn_exch{"H482 PC2"}="2H48 PC2"; $resn_exch{"H481 PC2"}="1H48 PC2"; $resn_exch{"HOP2 PC2"}="2HOP PC2";
$resn_exch{" H81 PC3"}="1H8  PC3"; $resn_exch{" H82 PC3"}="2H8  PC3"; $resn_exch{" H83 PC3"}="3H8  PC3";
$resn_exch{" H91 PC3"}="1H9  PC3"; $resn_exch{" H92 PC3"}="2H9  PC3"; $resn_exch{" H93 PC3"}="3H9  PC3";
$resn_exch{"H101 PC3"}="1H10 PC3"; $resn_exch{"H102 PC3"}="2H10 PC3"; $resn_exch{"H103 PC3"}="3H10 PC3";
$resn_exch{"H111 PC3"}="1H11 PC3"; $resn_exch{"H112 PC3"}="2H11 PC3"; $resn_exch{"H113 PC3"}="3H11 PC3";
$resn_exch{"H121 PC3"}="1H12 PC3"; $resn_exch{"H122 PC3"}="2H12 PC3"; $resn_exch{"H221 PC3"}="1H22 PC3";
$resn_exch{"H222 PC3"}="2H22 PC3"; $resn_exch{"H141 PC3"}="1H14 PC3"; $resn_exch{"H142 PC3"}="2H14 PC3";
$resn_exch{"H151 PC3"}="1H15 PC3"; $resn_exch{"H152 PC3"}="2H15 PC3"; $resn_exch{"H161 PC3"}="1H16 PC3";
$resn_exch{"H162 PC3"}="2H16 PC3"; $resn_exch{"H171 PC3"}="1H17 PC3"; $resn_exch{"H172 PC3"}="2H17 PC3";
$resn_exch{"H181 PC3"}="1H18 PC3"; $resn_exch{"H182 PC3"}="2H18 PC3"; $resn_exch{"H192 PC3"}="2H19 PC3";
$resn_exch{"H191 PC3"}="1H19 PC3"; $resn_exch{"HO22 PC3"}="2HO2 PC3"; $resn_exch{"HO32 PC3"}="2HO3 PC3";
$resn_exch{"H141 PC5"}="1H14 PC5"; $resn_exch{"H142 PC5"}="2H14 PC5"; $resn_exch{"H143 PC5"}="3H14 PC5";
$resn_exch{"H131 PC5"}="1H13 PC5"; $resn_exch{"H132 PC5"}="2H13 PC5"; $resn_exch{"H121 PC5"}="1H12 PC5";
$resn_exch{"H122 PC5"}="2H12 PC5"; $resn_exch{"H111 PC5"}="1H11 PC5"; $resn_exch{"H112 PC5"}="2H11 PC5";
$resn_exch{" H91 PC5"}="1H9  PC5"; $resn_exch{" H92 PC5"}="2H9  PC5"; $resn_exch{"H161 PC5"}="1H16 PC5";
$resn_exch{"H162 PC5"}="2H16 PC5"; $resn_exch{"H171 PC5"}="1H17 PC5"; $resn_exch{"H172 PC5"}="2H17 PC5";
$resn_exch{"H181 PC5"}="1H18 PC5"; $resn_exch{"H182 PC5"}="2H18 PC5"; $resn_exch{"H191 PC5"}="1H19 PC5";
$resn_exch{"H192 PC5"}="2H19 PC5"; $resn_exch{"H193 PC5"}="3H19 PC5"; $resn_exch{" H71 PC5"}="1H7  PC5";
$resn_exch{" H72 PC5"}="2H7  PC5"; $resn_exch{" H51 PC5"}="1H5  PC5"; $resn_exch{" H52 PC5"}="2H5  PC5";
$resn_exch{" H41 PC5"}="1H4  PC5"; $resn_exch{" H42 PC5"}="2H4  PC5"; $resn_exch{" H11 PC5"}="1H1  PC5";
$resn_exch{" H12 PC5"}="2H1  PC5"; $resn_exch{" H13 PC5"}="3H1  PC5"; $resn_exch{" H21 PC5"}="1H2  PC5";
$resn_exch{" H22 PC5"}="2H2  PC5"; $resn_exch{" H23 PC5"}="3H2  PC5"; $resn_exch{" H31 PC5"}="1H3  PC5";
$resn_exch{" H32 PC5"}="2H3  PC5"; $resn_exch{" H33 PC5"}="3H3  PC5"; $resn_exch{"HE11 PC6"}="1HE1 PC6";
$resn_exch{"HE12 PC6"}="2HE1 PC6"; $resn_exch{"HE21 PC6"}="1HE2 PC6"; $resn_exch{"HE22 PC6"}="2HE2 PC6";
$resn_exch{"HM31 PC6"}="1HM3 PC6"; $resn_exch{"HM32 PC6"}="2HM3 PC6"; $resn_exch{"HM33 PC6"}="3HM3 PC6";
$resn_exch{"HM21 PC6"}="1HM2 PC6"; $resn_exch{"HM22 PC6"}="2HM2 PC6"; $resn_exch{"HM23 PC6"}="3HM2 PC6";
$resn_exch{"HM11 PC6"}="1HM1 PC6"; $resn_exch{"HM12 PC6"}="2HM1 PC6"; $resn_exch{"HM13 PC6"}="3HM1 PC6";
$resn_exch{"H391 PC6"}="1H39 PC6"; $resn_exch{"H392 PC6"}="2H39 PC6"; $resn_exch{"H401 PC6"}="1H40 PC6";
$resn_exch{"H402 PC6"}="2H40 PC6"; $resn_exch{"H271 PC6"}="1H27 PC6"; $resn_exch{"H272 PC6"}="2H27 PC6";
$resn_exch{"H281 PC6"}="1H28 PC6"; $resn_exch{"H282 PC6"}="2H28 PC6"; $resn_exch{"H291 PC6"}="1H29 PC6";
$resn_exch{"H292 PC6"}="2H29 PC6"; $resn_exch{"H301 PC6"}="1H30 PC6"; $resn_exch{"H302 PC6"}="2H30 PC6";
$resn_exch{"H311 PC6"}="1H31 PC6"; $resn_exch{"H312 PC6"}="2H31 PC6"; $resn_exch{"H321 PC6"}="1H32 PC6";
$resn_exch{"H322 PC6"}="2H32 PC6"; $resn_exch{"H331 PC6"}="1H33 PC6"; $resn_exch{"H332 PC6"}="2H33 PC6";
$resn_exch{"H341 PC6"}="1H34 PC6"; $resn_exch{"H342 PC6"}="2H34 PC6"; $resn_exch{"H351 PC6"}="1H35 PC6";
$resn_exch{"H352 PC6"}="2H35 PC6"; $resn_exch{"H361 PC6"}="1H36 PC6"; $resn_exch{"H362 PC6"}="2H36 PC6";
$resn_exch{"H371 PC6"}="1H37 PC6"; $resn_exch{"H372 PC6"}="2H37 PC6"; $resn_exch{"H373 PC6"}="3H37 PC6";
$resn_exch{" H21 PC6"}="1H2  PC6"; $resn_exch{" H22 PC6"}="2H2  PC6"; $resn_exch{" H31 PC6"}="1H3  PC6";
$resn_exch{" H32 PC6"}="2H3  PC6"; $resn_exch{" H41 PC6"}="1H4  PC6"; $resn_exch{" H42 PC6"}="2H4  PC6";
$resn_exch{" H51 PC6"}="1H5  PC6"; $resn_exch{" H52 PC6"}="2H5  PC6"; $resn_exch{" H61 PC6"}="1H6  PC6";
$resn_exch{" H62 PC6"}="2H6  PC6"; $resn_exch{" H71 PC6"}="1H7  PC6"; $resn_exch{" H72 PC6"}="2H7  PC6";
$resn_exch{" H81 PC6"}="1H8  PC6"; $resn_exch{" H82 PC6"}="2H8  PC6"; $resn_exch{" H91 PC6"}="1H9  PC6";
$resn_exch{" H92 PC6"}="2H9  PC6"; $resn_exch{"H101 PC6"}="1H10 PC6"; $resn_exch{"H102 PC6"}="2H10 PC6";
$resn_exch{"H111 PC6"}="1H11 PC6"; $resn_exch{"H112 PC6"}="2H11 PC6"; $resn_exch{"H121 PC6"}="1H12 PC6";
$resn_exch{"H122 PC6"}="2H12 PC6"; $resn_exch{"H131 PC6"}="1H13 PC6"; $resn_exch{"H132 PC6"}="2H13 PC6";
$resn_exch{"H141 PC6"}="1H14 PC6"; $resn_exch{"H142 PC6"}="2H14 PC6"; $resn_exch{"H151 PC6"}="1H15 PC6";
$resn_exch{"H152 PC6"}="2H15 PC6"; $resn_exch{"H161 PC6"}="1H16 PC6"; $resn_exch{"H162 PC6"}="2H16 PC6";
$resn_exch{"H171 PC6"}="1H17 PC6"; $resn_exch{"H172 PC6"}="2H17 PC6"; $resn_exch{"H181 PC6"}="1H18 PC6";
$resn_exch{"H182 PC6"}="2H18 PC6"; $resn_exch{"H191 PC6"}="1H19 PC6"; $resn_exch{"H192 PC6"}="2H19 PC6";
$resn_exch{"H201 PC6"}="1H20 PC6"; $resn_exch{"H202 PC6"}="2H20 PC6"; $resn_exch{"H211 PC6"}="1H21 PC6";
$resn_exch{"H212 PC6"}="2H21 PC6"; $resn_exch{"H221 PC6"}="1H22 PC6"; $resn_exch{"H222 PC6"}="2H22 PC6";
$resn_exch{"H231 PC6"}="1H23 PC6"; $resn_exch{"H232 PC6"}="2H23 PC6"; $resn_exch{"H241 PC6"}="1H24 PC6";
$resn_exch{"H242 PC6"}="2H24 PC6"; $resn_exch{"H251 PC6"}="1H25 PC6"; $resn_exch{"H252 PC6"}="2H25 PC6";
$resn_exch{"H253 PC6"}="3H25 PC6"; $resn_exch{"H481 PC7"}="1H48 PC7"; $resn_exch{"H482 PC7"}="2H48 PC7";
$resn_exch{"H483 PC7"}="3H48 PC7"; $resn_exch{"H471 PC7"}="1H47 PC7"; $resn_exch{"H472 PC7"}="2H47 PC7";
$resn_exch{"H461 PC7"}="1H46 PC7"; $resn_exch{"H462 PC7"}="2H46 PC7"; $resn_exch{"H451 PC7"}="1H45 PC7";
$resn_exch{"H452 PC7"}="2H45 PC7"; $resn_exch{"H441 PC7"}="1H44 PC7"; $resn_exch{"H442 PC7"}="2H44 PC7";
$resn_exch{"H431 PC7"}="1H43 PC7"; $resn_exch{"H432 PC7"}="2H43 PC7"; $resn_exch{"H421 PC7"}="1H42 PC7";
$resn_exch{"H422 PC7"}="2H42 PC7"; $resn_exch{"H411 PC7"}="1H41 PC7"; $resn_exch{"H412 PC7"}="2H41 PC7";
$resn_exch{"H401 PC7"}="1H40 PC7"; $resn_exch{"H402 PC7"}="2H40 PC7"; $resn_exch{"H391 PC7"}="1H39 PC7";
$resn_exch{"H392 PC7"}="2H39 PC7"; $resn_exch{"H381 PC7"}="1H38 PC7"; $resn_exch{"H382 PC7"}="2H38 PC7";
$resn_exch{"H371 PC7"}="1H37 PC7"; $resn_exch{"H372 PC7"}="2H37 PC7"; $resn_exch{"H361 PC7"}="1H36 PC7";
$resn_exch{"H362 PC7"}="2H36 PC7"; $resn_exch{"H351 PC7"}="1H35 PC7"; $resn_exch{"H352 PC7"}="2H35 PC7";
$resn_exch{"H341 PC7"}="1H34 PC7"; $resn_exch{"H342 PC7"}="2H34 PC7"; $resn_exch{"H331 PC7"}="1H33 PC7";
$resn_exch{"H332 PC7"}="2H33 PC7"; $resn_exch{"H321 PC7"}="1H32 PC7"; $resn_exch{"H322 PC7"}="2H32 PC7";
$resn_exch{" H11 PC7"}="1H1  PC7"; $resn_exch{" H12 PC7"}="2H1  PC7"; $resn_exch{" H41 PC7"}="1H4  PC7";
$resn_exch{" H42 PC7"}="2H4  PC7"; $resn_exch{" H51 PC7"}="1H5  PC7"; $resn_exch{" H52 PC7"}="2H5  PC7";
$resn_exch{" H71 PC7"}="1H7  PC7"; $resn_exch{" H72 PC7"}="2H7  PC7"; $resn_exch{" H73 PC7"}="3H7  PC7";
$resn_exch{" H81 PC7"}="1H8  PC7"; $resn_exch{" H82 PC7"}="2H8  PC7"; $resn_exch{" H83 PC7"}="3H8  PC7";
$resn_exch{" H61 PC7"}="1H6  PC7"; $resn_exch{" H62 PC7"}="2H6  PC7"; $resn_exch{" H63 PC7"}="3H6  PC7";
$resn_exch{" H31 PC7"}="1H3  PC7"; $resn_exch{" H32 PC7"}="2H3  PC7"; $resn_exch{"H121 PC7"}="1H12 PC7";
$resn_exch{"H122 PC7"}="2H12 PC7"; $resn_exch{"H131 PC7"}="1H13 PC7"; $resn_exch{"H132 PC7"}="2H13 PC7";
$resn_exch{"H141 PC7"}="1H14 PC7"; $resn_exch{"H142 PC7"}="2H14 PC7"; $resn_exch{"H151 PC7"}="1H15 PC7";
$resn_exch{"H152 PC7"}="2H15 PC7"; $resn_exch{"H161 PC7"}="1H16 PC7"; $resn_exch{"H162 PC7"}="2H16 PC7";
$resn_exch{"H171 PC7"}="1H17 PC7"; $resn_exch{"H172 PC7"}="2H17 PC7"; $resn_exch{"H181 PC7"}="1H18 PC7";
$resn_exch{"H182 PC7"}="2H18 PC7"; $resn_exch{"H191 PC7"}="1H19 PC7"; $resn_exch{"H192 PC7"}="2H19 PC7";
$resn_exch{"H201 PC7"}="1H20 PC7"; $resn_exch{"H202 PC7"}="2H20 PC7"; $resn_exch{"H211 PC7"}="1H21 PC7";
$resn_exch{"H212 PC7"}="2H21 PC7"; $resn_exch{"H221 PC7"}="1H22 PC7"; $resn_exch{"H222 PC7"}="2H22 PC7";
$resn_exch{"H231 PC7"}="1H23 PC7"; $resn_exch{"H232 PC7"}="2H23 PC7"; $resn_exch{"H241 PC7"}="1H24 PC7";
$resn_exch{"H242 PC7"}="2H24 PC7"; $resn_exch{"H251 PC7"}="1H25 PC7"; $resn_exch{"H252 PC7"}="2H25 PC7";
$resn_exch{"H261 PC7"}="1H26 PC7"; $resn_exch{"H262 PC7"}="2H26 PC7"; $resn_exch{"H263 PC7"}="3H26 PC7";
$resn_exch{"H481 PC9"}="1H48 PC9"; $resn_exch{"H482 PC9"}="2H48 PC9"; $resn_exch{"H483 PC9"}="3H48 PC9";
$resn_exch{"H471 PC9"}="1H47 PC9"; $resn_exch{"H472 PC9"}="2H47 PC9"; $resn_exch{"H461 PC9"}="1H46 PC9";
$resn_exch{"H462 PC9"}="2H46 PC9"; $resn_exch{"H451 PC9"}="1H45 PC9"; $resn_exch{"H452 PC9"}="2H45 PC9";
$resn_exch{"H441 PC9"}="1H44 PC9"; $resn_exch{"H442 PC9"}="2H44 PC9"; $resn_exch{"H431 PC9"}="1H43 PC9";
$resn_exch{"H432 PC9"}="2H43 PC9"; $resn_exch{"H421 PC9"}="1H42 PC9"; $resn_exch{"H422 PC9"}="2H42 PC9";
$resn_exch{"H411 PC9"}="1H41 PC9"; $resn_exch{"H412 PC9"}="2H41 PC9"; $resn_exch{"H401 PC9"}="1H40 PC9";
$resn_exch{"H402 PC9"}="2H40 PC9"; $resn_exch{"H391 PC9"}="1H39 PC9"; $resn_exch{"H392 PC9"}="2H39 PC9";
$resn_exch{"H381 PC9"}="1H38 PC9"; $resn_exch{"H382 PC9"}="2H38 PC9"; $resn_exch{"H351 PC9"}="1H35 PC9";
$resn_exch{"H352 PC9"}="2H35 PC9"; $resn_exch{"H341 PC9"}="1H34 PC9"; $resn_exch{"H342 PC9"}="2H34 PC9";
$resn_exch{"H331 PC9"}="1H33 PC9"; $resn_exch{"H332 PC9"}="2H33 PC9"; $resn_exch{"H321 PC9"}="1H32 PC9";
$resn_exch{"H322 PC9"}="2H32 PC9"; $resn_exch{" H11 PC9"}="1H1  PC9"; $resn_exch{" H12 PC9"}="2H1  PC9";
$resn_exch{" H41 PC9"}="1H4  PC9"; $resn_exch{" H42 PC9"}="2H4  PC9"; $resn_exch{" H51 PC9"}="1H5  PC9";
$resn_exch{" H52 PC9"}="2H5  PC9"; $resn_exch{" H71 PC9"}="1H7  PC9"; $resn_exch{" H72 PC9"}="2H7  PC9";
$resn_exch{" H73 PC9"}="3H7  PC9"; $resn_exch{" H81 PC9"}="1H8  PC9"; $resn_exch{" H82 PC9"}="2H8  PC9";
$resn_exch{" H83 PC9"}="3H8  PC9"; $resn_exch{" H61 PC9"}="1H6  PC9"; $resn_exch{" H62 PC9"}="2H6  PC9";
$resn_exch{" H63 PC9"}="3H6  PC9"; $resn_exch{" H31 PC9"}="1H3  PC9"; $resn_exch{" H32 PC9"}="2H3  PC9";
$resn_exch{"H121 PC9"}="1H12 PC9"; $resn_exch{"H122 PC9"}="2H12 PC9"; $resn_exch{"H131 PC9"}="1H13 PC9";
$resn_exch{"H132 PC9"}="2H13 PC9"; $resn_exch{"H141 PC9"}="1H14 PC9"; $resn_exch{"H142 PC9"}="2H14 PC9";
$resn_exch{"H151 PC9"}="1H15 PC9"; $resn_exch{"H152 PC9"}="2H15 PC9"; $resn_exch{"H161 PC9"}="1H16 PC9";
$resn_exch{"H162 PC9"}="2H16 PC9"; $resn_exch{"H171 PC9"}="1H17 PC9"; $resn_exch{"H172 PC9"}="2H17 PC9";
$resn_exch{"H181 PC9"}="1H18 PC9"; $resn_exch{"H182 PC9"}="2H18 PC9"; $resn_exch{"H191 PC9"}="1H19 PC9";
$resn_exch{"H192 PC9"}="2H19 PC9"; $resn_exch{"H201 PC9"}="1H20 PC9"; $resn_exch{"H202 PC9"}="2H20 PC9";
$resn_exch{"H211 PC9"}="1H21 PC9"; $resn_exch{"H212 PC9"}="2H21 PC9"; $resn_exch{"H221 PC9"}="1H22 PC9";
$resn_exch{"H222 PC9"}="2H22 PC9"; $resn_exch{"H231 PC9"}="1H23 PC9"; $resn_exch{"H232 PC9"}="2H23 PC9";
$resn_exch{"H241 PC9"}="1H24 PC9"; $resn_exch{"H242 PC9"}="2H24 PC9"; $resn_exch{"H251 PC9"}="1H25 PC9";
$resn_exch{"H252 PC9"}="2H25 PC9"; $resn_exch{"H261 PC9"}="1H26 PC9"; $resn_exch{"H262 PC9"}="2H26 PC9";
$resn_exch{"H263 PC9"}="3H26 PC9"; $resn_exch{" H   PCA"}=" HN  PCA"; $resn_exch{" HB2 PCA"}="1HB  PCA";
$resn_exch{" HB3 PCA"}="2HB  PCA"; $resn_exch{" HG2 PCA"}="1HG  PCA"; $resn_exch{" HG3 PCA"}="2HG  PCA";
$resn_exch{" HM1 PCB"}="1HM  PCB"; $resn_exch{" HM2 PCB"}="2HM  PCB"; $resn_exch{" HM3 PCB"}="3HM  PCB";
$resn_exch{"HM'1 PCB"}="1HM' PCB"; $resn_exch{"HM'2 PCB"}="2HM' PCB"; $resn_exch{"HM'3 PCB"}="3HM' PCB";
$resn_exch{" HB2 PCC"}="1HB  PCC"; $resn_exch{" HB3 PCC"}="2HB  PCC"; $resn_exch{" HG2 PCC"}="1HG  PCC";
$resn_exch{" HG3 PCC"}="2HG  PCC"; $resn_exch{" H42 PCD"}="2H4  PCD"; $resn_exch{" H41 PCD"}="1H4  PCD";
$resn_exch{" H1' PCD"}=" H1* PCD"; $resn_exch{" H2' PCD"}=" H2* PCD"; $resn_exch{"H2O1 PCD"}="1H2O PCD";
$resn_exch{" H3' PCD"}=" H3* PCD"; $resn_exch{" H4D PCD"}=" H4* PCD"; $resn_exch{"H3O1 PCD"}="1H3O PCD";
$resn_exch{"H5'2 PCD"}="2H5* PCD"; $resn_exch{"H5'1 PCD"}="1H5* PCD"; $resn_exch{"HOA2 PCD"}="2HOA PCD";
$resn_exch{"HOB2 PCD"}="2HOB PCD"; $resn_exch{"H102 PCD"}="2H10 PCD"; $resn_exch{"H101 PCD"}="1H10 PCD";
$resn_exch{"H2'2 PCD"}="2H2' PCD"; $resn_exch{"H2'1 PCD"}="1H2' PCD"; $resn_exch{"HR11 PCD"}="1HR1 PCD";
$resn_exch{"HR12 PCD"}="2HR1 PCD"; $resn_exch{" H   PCE"}="1HN  PCE"; $resn_exch{" H2  PCE"}="2HN  PCE";
$resn_exch{" HB2 PCE"}="1HB  PCE"; $resn_exch{" HB3 PCE"}="2HB  PCE"; $resn_exch{"HD12 PCE"}="1HD1 PCE";
$resn_exch{"HD13 PCE"}="2HD1 PCE"; $resn_exch{"HD23 PCE"}="1HD2 PCE"; $resn_exch{"HD22 PCE"}="2HD2 PCE";
$resn_exch{"HE12 PCE"}="1HE1 PCE"; $resn_exch{"HE13 PCE"}="2HE1 PCE"; $resn_exch{"HE23 PCE"}="1HE2 PCE";
$resn_exch{"HE22 PCE"}="2HE2 PCE"; $resn_exch{" HZ2 PCE"}="1HZ  PCE"; $resn_exch{" HZ3 PCE"}="2HZ  PCE";
$resn_exch{" HM1 PCE"}="1HM  PCE"; $resn_exch{" HM2 PCE"}="2HM  PCE"; $resn_exch{"HB21 PCE"}="1HB2 PCE";
$resn_exch{"HB22 PCE"}="2HB2 PCE"; $resn_exch{"HD31 PCE"}="1HD3 PCE"; $resn_exch{"H111 PCF"}="1H11 PCF";
$resn_exch{"H112 PCF"}="2H11 PCF"; $resn_exch{"H121 PCF"}="1H12 PCF"; $resn_exch{"H122 PCF"}="2H12 PCF";
$resn_exch{"H131 PCF"}="1H13 PCF"; $resn_exch{"H132 PCF"}="2H13 PCF"; $resn_exch{"H133 PCF"}="3H13 PCF";
$resn_exch{"H141 PCF"}="1H14 PCF"; $resn_exch{"H142 PCF"}="2H14 PCF"; $resn_exch{"H143 PCF"}="3H14 PCF";
$resn_exch{"H151 PCF"}="1H15 PCF"; $resn_exch{"H152 PCF"}="2H15 PCF"; $resn_exch{"H153 PCF"}="3H15 PCF";
$resn_exch{" H11 PCF"}="1H1  PCF"; $resn_exch{" H12 PCF"}="2H1  PCF"; $resn_exch{" H21 PCF"}="1H2  PCF";
$resn_exch{" H31 PCF"}="1H3  PCF"; $resn_exch{" H32 PCF"}="2H3  PCF"; $resn_exch{"H321 PCF"}="1H32 PCF";
$resn_exch{"H322 PCF"}="2H32 PCF"; $resn_exch{"H331 PCF"}="1H33 PCF"; $resn_exch{"H332 PCF"}="2H33 PCF";
$resn_exch{"H341 PCF"}="1H34 PCF"; $resn_exch{"H342 PCF"}="2H34 PCF"; $resn_exch{"H351 PCF"}="1H35 PCF";
$resn_exch{"H352 PCF"}="2H35 PCF"; $resn_exch{"H361 PCF"}="1H36 PCF"; $resn_exch{"H362 PCF"}="2H36 PCF";
$resn_exch{"H371 PCF"}="1H37 PCF"; $resn_exch{"H372 PCF"}="2H37 PCF"; $resn_exch{"H381 PCF"}="1H38 PCF";
$resn_exch{"H382 PCF"}="2H38 PCF"; $resn_exch{"H391 PCF"}="1H39 PCF"; $resn_exch{"H392 PCF"}="2H39 PCF";
$resn_exch{"H401 PCF"}="1H40 PCF"; $resn_exch{"H402 PCF"}="2H40 PCF"; $resn_exch{"H411 PCF"}="1H41 PCF";
$resn_exch{"H412 PCF"}="2H41 PCF"; $resn_exch{"H421 PCF"}="1H42 PCF"; $resn_exch{"H422 PCF"}="2H42 PCF";
$resn_exch{"H431 PCF"}="1H43 PCF"; $resn_exch{"H432 PCF"}="2H43 PCF"; $resn_exch{"H221 PCF"}="1H22 PCF";
$resn_exch{"H222 PCF"}="2H22 PCF"; $resn_exch{"H231 PCF"}="1H23 PCF"; $resn_exch{"H232 PCF"}="2H23 PCF";
$resn_exch{"H241 PCF"}="1H24 PCF"; $resn_exch{"H242 PCF"}="2H24 PCF"; $resn_exch{"H251 PCF"}="1H25 PCF";
$resn_exch{"H252 PCF"}="2H25 PCF"; $resn_exch{"H261 PCF"}="1H26 PCF"; $resn_exch{"H262 PCF"}="2H26 PCF";
$resn_exch{"H271 PCF"}="1H27 PCF"; $resn_exch{"H272 PCF"}="2H27 PCF"; $resn_exch{"H281 PCF"}="1H28 PCF";
$resn_exch{"H282 PCF"}="2H28 PCF"; $resn_exch{"H291 PCF"}="1H29 PCF"; $resn_exch{"H292 PCF"}="2H29 PCF";
$resn_exch{"H301 PCF"}="1H30 PCF"; $resn_exch{"H302 PCF"}="2H30 PCF"; $resn_exch{"H471 PCF"}="1H47 PCF";
$resn_exch{"H472 PCF"}="2H47 PCF"; $resn_exch{"H481 PCF"}="1H48 PCF"; $resn_exch{"H482 PCF"}="2H48 PCF";
$resn_exch{"H491 PCF"}="1H49 PCF"; $resn_exch{"H492 PCF"}="2H49 PCF"; $resn_exch{"H501 PCF"}="1H50 PCF";
$resn_exch{"H502 PCF"}="2H50 PCF"; $resn_exch{"H511 PCF"}="1H51 PCF"; $resn_exch{"H512 PCF"}="2H51 PCF";
$resn_exch{"H521 PCF"}="1H52 PCF"; $resn_exch{"H522 PCF"}="2H52 PCF"; $resn_exch{"H523 PCF"}="3H52 PCF";
$resn_exch{"H441 PCF"}="1H44 PCF"; $resn_exch{"H442 PCF"}="2H44 PCF"; $resn_exch{"H451 PCF"}="1H45 PCF";
$resn_exch{"H452 PCF"}="2H45 PCF"; $resn_exch{"H461 PCF"}="1H46 PCF"; $resn_exch{"H462 PCF"}="2H46 PCF";
$resn_exch{"H463 PCF"}="3H46 PCF"; $resn_exch{"HO1A PCG"}="AHO1 PCG"; $resn_exch{"H5'1 PCG"}="1H5* PCG";
$resn_exch{"H5'2 PCG"}="2H5* PCG"; $resn_exch{" H4' PCG"}=" H4* PCG"; $resn_exch{" H3' PCG"}=" H3* PCG";
$resn_exch{" H2' PCG"}=" H2* PCG"; $resn_exch{"HO2' PCG"}="*HO2 PCG"; $resn_exch{" H1' PCG"}=" H1* PCG";
$resn_exch{"HN21 PCG"}="1HN2 PCG"; $resn_exch{"HN22 PCG"}="2HN2 PCG"; $resn_exch{" HN1 PCH"}="1HN  PCH";
$resn_exch{" HN2 PCH"}="2HN  PCH"; $resn_exch{" HB1 PCH"}="1HB  PCH"; $resn_exch{" HB2 PCH"}="2HB  PCH";
$resn_exch{"HD11 PCH"}="1HD1 PCH"; $resn_exch{"HD12 PCH"}="2HD1 PCH"; $resn_exch{"HD21 PCH"}="1HD2 PCH";
$resn_exch{"HD22 PCH"}="2HD2 PCH"; $resn_exch{"HE11 PCH"}="1HE1 PCH"; $resn_exch{"HE12 PCH"}="2HE1 PCH";
$resn_exch{"HE21 PCH"}="1HE2 PCH"; $resn_exch{"HE22 PCH"}="2HE2 PCH"; $resn_exch{"HCZ1 PCH"}="1HCZ PCH";
$resn_exch{"HCZ2 PCH"}="2HCZ PCH"; $resn_exch{" HM1 PCH"}="1HM  PCH"; $resn_exch{" HM2 PCH"}="2HM  PCH";
$resn_exch{"H511 PCJ"}="1H51 PCJ"; $resn_exch{"H512 PCJ"}="2H51 PCJ"; $resn_exch{"H513 PCJ"}="3H51 PCJ";
$resn_exch{"H521 PCJ"}="1H52 PCJ"; $resn_exch{"H522 PCJ"}="2H52 PCJ"; $resn_exch{"H531 PCJ"}="1H53 PCJ";
$resn_exch{"H532 PCJ"}="2H53 PCJ"; $resn_exch{"H541 PCJ"}="1H54 PCJ"; $resn_exch{"H542 PCJ"}="2H54 PCJ";
$resn_exch{"H551 PCJ"}="1H55 PCJ"; $resn_exch{"H552 PCJ"}="2H55 PCJ"; $resn_exch{"H561 PCJ"}="1H56 PCJ";
$resn_exch{"H562 PCJ"}="2H56 PCJ"; $resn_exch{"H571 PCJ"}="1H57 PCJ"; $resn_exch{"H572 PCJ"}="2H57 PCJ";
$resn_exch{"H581 PCJ"}="1H58 PCJ"; $resn_exch{"H582 PCJ"}="2H58 PCJ"; $resn_exch{"H591 PCJ"}="1H59 PCJ";
$resn_exch{"H592 PCJ"}="2H59 PCJ"; $resn_exch{"H601 PCJ"}="1H60 PCJ"; $resn_exch{"H602 PCJ"}="2H60 PCJ";
$resn_exch{"H611 PCJ"}="1H61 PCJ"; $resn_exch{"H612 PCJ"}="2H61 PCJ"; $resn_exch{"H621 PCJ"}="1H62 PCJ";
$resn_exch{"H622 PCJ"}="2H62 PCJ"; $resn_exch{"H631 PCJ"}="1H63 PCJ"; $resn_exch{"H632 PCJ"}="2H63 PCJ";
$resn_exch{"H641 PCJ"}="1H64 PCJ"; $resn_exch{"H642 PCJ"}="2H64 PCJ"; $resn_exch{"H651 PCJ"}="1H65 PCJ";
$resn_exch{"H652 PCJ"}="2H65 PCJ"; $resn_exch{" H1  PCJ"}="1H   PCJ"; $resn_exch{" H2A PCJ"}="2H   PCJ";
$resn_exch{" HB1 PCJ"}="1HB  PCJ"; $resn_exch{" HB2 PCJ"}="2HB  PCJ"; $resn_exch{" H11 PCJ"}="1H1  PCJ";
$resn_exch{" H12 PCJ"}="2H1  PCJ"; $resn_exch{" H31 PCJ"}="1H3  PCJ"; $resn_exch{" H32 PCJ"}="2H3  PCJ";
$resn_exch{"H251 PCJ"}="1H25 PCJ"; $resn_exch{"H252 PCJ"}="2H25 PCJ"; $resn_exch{"H241 PCJ"}="1H24 PCJ";
$resn_exch{"H242 PCJ"}="2H24 PCJ"; $resn_exch{"H231 PCJ"}="1H23 PCJ"; $resn_exch{"H232 PCJ"}="2H23 PCJ";
$resn_exch{"H221 PCJ"}="1H22 PCJ"; $resn_exch{"H222 PCJ"}="2H22 PCJ"; $resn_exch{"H211 PCJ"}="1H21 PCJ";
$resn_exch{"H212 PCJ"}="2H21 PCJ"; $resn_exch{"H201 PCJ"}="1H20 PCJ"; $resn_exch{"H202 PCJ"}="2H20 PCJ";
$resn_exch{"H191 PCJ"}="1H19 PCJ"; $resn_exch{"H192 PCJ"}="2H19 PCJ"; $resn_exch{"H181 PCJ"}="1H18 PCJ";
$resn_exch{"H182 PCJ"}="2H18 PCJ"; $resn_exch{"H171 PCJ"}="1H17 PCJ"; $resn_exch{"H172 PCJ"}="2H17 PCJ";
$resn_exch{"H161 PCJ"}="1H16 PCJ"; $resn_exch{"H162 PCJ"}="2H16 PCJ"; $resn_exch{"H151 PCJ"}="1H15 PCJ";
$resn_exch{"H152 PCJ"}="2H15 PCJ"; $resn_exch{"H141 PCJ"}="1H14 PCJ"; $resn_exch{"H142 PCJ"}="2H14 PCJ";
$resn_exch{"H131 PCJ"}="1H13 PCJ"; $resn_exch{"H132 PCJ"}="2H13 PCJ"; $resn_exch{"H121 PCJ"}="1H12 PCJ";
$resn_exch{"H122 PCJ"}="2H12 PCJ"; $resn_exch{"H111 PCJ"}="1H11 PCJ"; $resn_exch{"H112 PCJ"}="2H11 PCJ";
$resn_exch{"H113 PCJ"}="3H11 PCJ"; $resn_exch{"H451 PCJ"}="1H45 PCJ"; $resn_exch{"H452 PCJ"}="2H45 PCJ";
$resn_exch{"H441 PCJ"}="1H44 PCJ"; $resn_exch{"H442 PCJ"}="2H44 PCJ"; $resn_exch{"H431 PCJ"}="1H43 PCJ";
$resn_exch{"H432 PCJ"}="2H43 PCJ"; $resn_exch{"H421 PCJ"}="1H42 PCJ"; $resn_exch{"H422 PCJ"}="2H42 PCJ";
$resn_exch{"H411 PCJ"}="1H41 PCJ"; $resn_exch{"H412 PCJ"}="2H41 PCJ"; $resn_exch{"H401 PCJ"}="1H40 PCJ";
$resn_exch{"H402 PCJ"}="2H40 PCJ"; $resn_exch{"H391 PCJ"}="1H39 PCJ"; $resn_exch{"H392 PCJ"}="2H39 PCJ";
$resn_exch{"H381 PCJ"}="1H38 PCJ"; $resn_exch{"H382 PCJ"}="2H38 PCJ"; $resn_exch{"H371 PCJ"}="1H37 PCJ";
$resn_exch{"H372 PCJ"}="2H37 PCJ"; $resn_exch{"H361 PCJ"}="1H36 PCJ"; $resn_exch{"H362 PCJ"}="2H36 PCJ";
$resn_exch{"H351 PCJ"}="1H35 PCJ"; $resn_exch{"H352 PCJ"}="2H35 PCJ"; $resn_exch{"H341 PCJ"}="1H34 PCJ";
$resn_exch{"H342 PCJ"}="2H34 PCJ"; $resn_exch{"H331 PCJ"}="1H33 PCJ"; $resn_exch{"H332 PCJ"}="2H33 PCJ";
$resn_exch{"H321 PCJ"}="1H32 PCJ"; $resn_exch{"H322 PCJ"}="2H32 PCJ"; $resn_exch{"H311 PCJ"}="1H31 PCJ";
$resn_exch{"H312 PCJ"}="2H31 PCJ"; $resn_exch{"H313 PCJ"}="3H31 PCJ"; $resn_exch{"H481 PCK"}="1H48 PCK";
$resn_exch{"H482 PCK"}="2H48 PCK"; $resn_exch{"H483 PCK"}="3H48 PCK"; $resn_exch{"H471 PCK"}="1H47 PCK";
$resn_exch{"H472 PCK"}="2H47 PCK"; $resn_exch{"H461 PCK"}="1H46 PCK"; $resn_exch{"H462 PCK"}="2H46 PCK";
$resn_exch{"H451 PCK"}="1H45 PCK"; $resn_exch{"H452 PCK"}="2H45 PCK"; $resn_exch{"H441 PCK"}="1H44 PCK";
$resn_exch{"H442 PCK"}="2H44 PCK"; $resn_exch{"H431 PCK"}="1H43 PCK"; $resn_exch{"H432 PCK"}="2H43 PCK";
$resn_exch{"H421 PCK"}="1H42 PCK"; $resn_exch{"H422 PCK"}="2H42 PCK"; $resn_exch{"H411 PCK"}="1H41 PCK";
$resn_exch{"H412 PCK"}="2H41 PCK"; $resn_exch{"H381 PCK"}="1H38 PCK"; $resn_exch{"H382 PCK"}="2H38 PCK";
$resn_exch{"H371 PCK"}="1H37 PCK"; $resn_exch{"H372 PCK"}="2H37 PCK"; $resn_exch{"H361 PCK"}="1H36 PCK";
$resn_exch{"H362 PCK"}="2H36 PCK"; $resn_exch{"H351 PCK"}="1H35 PCK"; $resn_exch{"H352 PCK"}="2H35 PCK";
$resn_exch{"H341 PCK"}="1H34 PCK"; $resn_exch{"H342 PCK"}="2H34 PCK"; $resn_exch{"H331 PCK"}="1H33 PCK";
$resn_exch{"H332 PCK"}="2H33 PCK"; $resn_exch{"H321 PCK"}="1H32 PCK"; $resn_exch{"H322 PCK"}="2H32 PCK";
$resn_exch{" H11 PCK"}="1H1  PCK"; $resn_exch{" H12 PCK"}="2H1  PCK"; $resn_exch{" H41 PCK"}="1H4  PCK";
$resn_exch{" H42 PCK"}="2H4  PCK"; $resn_exch{" H51 PCK"}="1H5  PCK"; $resn_exch{" H52 PCK"}="2H5  PCK";
$resn_exch{" H71 PCK"}="1H7  PCK"; $resn_exch{" H72 PCK"}="2H7  PCK"; $resn_exch{" H73 PCK"}="3H7  PCK";
$resn_exch{" H81 PCK"}="1H8  PCK"; $resn_exch{" H82 PCK"}="2H8  PCK"; $resn_exch{" H83 PCK"}="3H8  PCK";
$resn_exch{" H61 PCK"}="1H6  PCK"; $resn_exch{" H62 PCK"}="2H6  PCK"; $resn_exch{" H63 PCK"}="3H6  PCK";
$resn_exch{" H31 PCK"}="1H3  PCK"; $resn_exch{" H32 PCK"}="2H3  PCK"; $resn_exch{"H121 PCK"}="1H12 PCK";
$resn_exch{"H122 PCK"}="2H12 PCK"; $resn_exch{"H131 PCK"}="1H13 PCK"; $resn_exch{"H132 PCK"}="2H13 PCK";
$resn_exch{"H141 PCK"}="1H14 PCK"; $resn_exch{"H142 PCK"}="2H14 PCK"; $resn_exch{"H151 PCK"}="1H15 PCK";
$resn_exch{"H152 PCK"}="2H15 PCK"; $resn_exch{"H161 PCK"}="1H16 PCK"; $resn_exch{"H162 PCK"}="2H16 PCK";
$resn_exch{"H171 PCK"}="1H17 PCK"; $resn_exch{"H172 PCK"}="2H17 PCK"; $resn_exch{"H181 PCK"}="1H18 PCK";
$resn_exch{"H182 PCK"}="2H18 PCK"; $resn_exch{"H211 PCK"}="1H21 PCK"; $resn_exch{"H212 PCK"}="2H21 PCK";
$resn_exch{"H221 PCK"}="1H22 PCK"; $resn_exch{"H222 PCK"}="2H22 PCK"; $resn_exch{"H231 PCK"}="1H23 PCK";
$resn_exch{"H232 PCK"}="2H23 PCK"; $resn_exch{"H241 PCK"}="1H24 PCK"; $resn_exch{"H242 PCK"}="2H24 PCK";
$resn_exch{"H251 PCK"}="1H25 PCK"; $resn_exch{"H252 PCK"}="2H25 PCK"; $resn_exch{"H261 PCK"}="1H26 PCK";
$resn_exch{"H262 PCK"}="2H26 PCK"; $resn_exch{"H271 PCK"}="1H27 PCK"; $resn_exch{"H272 PCK"}="2H27 PCK";
$resn_exch{"H281 PCK"}="1H28 PCK"; $resn_exch{"H282 PCK"}="2H28 PCK"; $resn_exch{"H283 PCK"}="3H28 PCK";
$resn_exch{" H71 PCM"}="1H7  PCM"; $resn_exch{" H72 PCM"}="2H7  PCM"; $resn_exch{"H121 PCM"}="1H12 PCM";
$resn_exch{"H122 PCM"}="2H12 PCM"; $resn_exch{"H141 PCM"}="1H14 PCM"; $resn_exch{"H142 PCM"}="2H14 PCM";
$resn_exch{"H143 PCM"}="3H14 PCM"; $resn_exch{"H151 PCM"}="1H15 PCM"; $resn_exch{"H152 PCM"}="2H15 PCM";
$resn_exch{"H153 PCM"}="3H15 PCM"; $resn_exch{"H211 PCM"}="1H21 PCM"; $resn_exch{"H212 PCM"}="2H21 PCM";
$resn_exch{"H301 PCM"}="1H30 PCM"; $resn_exch{"H302 PCM"}="2H30 PCM"; $resn_exch{"H351 PCM"}="1H35 PCM";
$resn_exch{"H352 PCM"}="2H35 PCM"; $resn_exch{"H371 PCM"}="1H37 PCM"; $resn_exch{"H372 PCM"}="2H37 PCM";
$resn_exch{"H373 PCM"}="3H37 PCM"; $resn_exch{"H381 PCM"}="1H38 PCM"; $resn_exch{"H382 PCM"}="2H38 PCM";
$resn_exch{"H383 PCM"}="3H38 PCM"; $resn_exch{"H421 PCM"}="1H42 PCM"; $resn_exch{"H422 PCM"}="2H42 PCM";
$resn_exch{"H511 PCN"}="1H51 PCN"; $resn_exch{"H512 PCN"}="2H51 PCN"; $resn_exch{"H513 PCN"}="3H51 PCN";
$resn_exch{"H521 PCN"}="1H52 PCN"; $resn_exch{"H522 PCN"}="2H52 PCN"; $resn_exch{"H523 PCN"}="3H52 PCN";
$resn_exch{"H63O PCN"}="OH63 PCN"; $resn_exch{"H131 PCN"}="1H13 PCN"; $resn_exch{"H132 PCN"}="2H13 PCN";
$resn_exch{"H133 PCN"}="3H13 PCN"; $resn_exch{"H191 PCN"}="1H19 PCN"; $resn_exch{"H192 PCN"}="2H19 PCN";
$resn_exch{"H161 PCO"}="1H16 PCO"; $resn_exch{"H162 PCO"}="2H16 PCO"; $resn_exch{"H151 PCO"}="1H15 PCO";
$resn_exch{"H152 PCO"}="2H15 PCO"; $resn_exch{"HC61 PCO"}="1HC6 PCO"; $resn_exch{"HC62 PCO"}="2HC6 PCO";
$resn_exch{"HC81 PCO"}="1HC8 PCO"; $resn_exch{"HC82 PCO"}="2HC8 PCO"; $resn_exch{"HC83 PCO"}="3HC8 PCO";
$resn_exch{"HC91 PCO"}="1HC9 PCO"; $resn_exch{"HC92 PCO"}="2HC9 PCO"; $resn_exch{"HC93 PCO"}="3HC9 PCO";
$resn_exch{" H51 PCP"}="1H5  PCP"; $resn_exch{" H52 PCP"}="2H5  PCP"; $resn_exch{" HP1 PCP"}="1HP  PCP";
$resn_exch{" HP2 PCP"}="2HP  PCP"; $resn_exch{"HOP2 PCP"}="2HOP PCP"; $resn_exch{"HOP3 PCP"}="3HOP PCP";
$resn_exch{"HOA2 PCP"}="2HOA PCP"; $resn_exch{"HOB2 PCP"}="2HOB PCP"; $resn_exch{"HOB3 PCP"}="3HOB PCP";
$resn_exch{" H21 PCQ"}="1H2  PCQ"; $resn_exch{" H61 PCQ"}="1H6  PCQ"; $resn_exch{"H2'1 PCQ"}="1H2' PCQ";
$resn_exch{"H6'1 PCQ"}="1H6' PCQ"; $resn_exch{" HB1 PCR"}="1HB  PCR"; $resn_exch{" HB2 PCR"}="2HB  PCR";
$resn_exch{" HB3 PCR"}="3HB  PCR"; $resn_exch{"HD11 PCR"}="1HD1 PCR"; $resn_exch{"HE11 PCR"}="1HE1 PCR";
$resn_exch{"HD21 PCR"}="1HD2 PCR"; $resn_exch{"HE21 PCR"}="1HE2 PCR"; $resn_exch{" HO1 PCR"}="1HO  PCR";
$resn_exch{" H2  PCS"}=" HN2 PCS"; $resn_exch{" HB2 PCS"}="1HB  PCS"; $resn_exch{" HB3 PCS"}="2HB  PCS";
$resn_exch{" HT1 PCS"}="1HT  PCS"; $resn_exch{" HT2 PCS"}="2HT  PCS"; $resn_exch{"HOP2 PCT"}="2HOP PCT";
$resn_exch{"HOP3 PCT"}="3HOP PCT"; $resn_exch{"H1P1 PCT"}="1H1P PCT"; $resn_exch{"H1P2 PCT"}="2H1P PCT";
$resn_exch{"HN11 PCT"}="1HN1 PCT"; $resn_exch{"HN12 PCT"}="2HN1 PCT"; $resn_exch{"HMA1 PCU"}="1HMA PCU";
$resn_exch{"HMA2 PCU"}="2HMA PCU"; $resn_exch{"HMA3 PCU"}="3HMA PCU"; $resn_exch{"HMB1 PCU"}="1HMB PCU";
$resn_exch{"HMB2 PCU"}="2HMB PCU"; $resn_exch{"HMB3 PCU"}="3HMB PCU"; $resn_exch{"HMC1 PCU"}="1HMC PCU";
$resn_exch{"HMC2 PCU"}="2HMC PCU"; $resn_exch{"HMC3 PCU"}="3HMC PCU"; $resn_exch{"HMD1 PCU"}="1HMD PCU";
$resn_exch{"HMD2 PCU"}="2HMD PCU"; $resn_exch{"HMD3 PCU"}="3HMD PCU"; $resn_exch{"HC91 PCV"}="1HC9 PCV";
$resn_exch{"HC92 PCV"}="2HC9 PCV"; $resn_exch{"HC71 PCV"}="1HC7 PCV"; $resn_exch{"HC72 PCV"}="2HC7 PCV";
$resn_exch{" H22 PCV"}="2H2  PCV"; $resn_exch{"HC41 PCV"}="1HC4 PCV"; $resn_exch{"HC42 PCV"}="2HC4 PCV";
$resn_exch{"HC51 PCV"}="1HC5 PCV"; $resn_exch{"HC52 PCV"}="2HC5 PCV"; $resn_exch{"HN21 PCV"}="1HN2 PCV";
$resn_exch{"HN22 PCV"}="2HN2 PCV"; $resn_exch{" H11 PCW"}="1H1  PCW"; $resn_exch{" H12 PCW"}="2H1  PCW";
$resn_exch{" H31 PCW"}="1H3  PCW"; $resn_exch{" H32 PCW"}="2H3  PCW"; $resn_exch{" H41 PCW"}="1H4  PCW";
$resn_exch{" H42 PCW"}="2H4  PCW"; $resn_exch{" H51 PCW"}="1H5  PCW"; $resn_exch{" H52 PCW"}="2H5  PCW";
$resn_exch{" H61 PCW"}="1H6  PCW"; $resn_exch{" H62 PCW"}="2H6  PCW"; $resn_exch{" H63 PCW"}="3H6  PCW";
$resn_exch{" H71 PCW"}="1H7  PCW"; $resn_exch{" H72 PCW"}="2H7  PCW"; $resn_exch{" H73 PCW"}="3H7  PCW";
$resn_exch{" H81 PCW"}="1H8  PCW"; $resn_exch{" H82 PCW"}="2H8  PCW"; $resn_exch{" H83 PCW"}="3H8  PCW";
$resn_exch{"H121 PCW"}="1H12 PCW"; $resn_exch{"H122 PCW"}="2H12 PCW"; $resn_exch{"H131 PCW"}="1H13 PCW";
$resn_exch{"H132 PCW"}="2H13 PCW"; $resn_exch{"H141 PCW"}="1H14 PCW"; $resn_exch{"H142 PCW"}="2H14 PCW";
$resn_exch{"H151 PCW"}="1H15 PCW"; $resn_exch{"H152 PCW"}="2H15 PCW"; $resn_exch{"H161 PCW"}="1H16 PCW";
$resn_exch{"H162 PCW"}="2H16 PCW"; $resn_exch{"H171 PCW"}="1H17 PCW"; $resn_exch{"H172 PCW"}="2H17 PCW";
$resn_exch{"H181 PCW"}="1H18 PCW"; $resn_exch{"H182 PCW"}="2H18 PCW"; $resn_exch{"H211 PCW"}="1H21 PCW";
$resn_exch{"H212 PCW"}="2H21 PCW"; $resn_exch{"H221 PCW"}="1H22 PCW"; $resn_exch{"H222 PCW"}="2H22 PCW";
$resn_exch{"H231 PCW"}="1H23 PCW"; $resn_exch{"H232 PCW"}="2H23 PCW"; $resn_exch{"H241 PCW"}="1H24 PCW";
$resn_exch{"H242 PCW"}="2H24 PCW"; $resn_exch{"H251 PCW"}="1H25 PCW"; $resn_exch{"H252 PCW"}="2H25 PCW";
$resn_exch{"H261 PCW"}="1H26 PCW"; $resn_exch{"H262 PCW"}="2H26 PCW"; $resn_exch{"H271 PCW"}="1H27 PCW";
$resn_exch{"H272 PCW"}="2H27 PCW"; $resn_exch{"H281 PCW"}="1H28 PCW"; $resn_exch{"H282 PCW"}="2H28 PCW";
$resn_exch{"H283 PCW"}="3H28 PCW"; $resn_exch{"H321 PCW"}="1H32 PCW"; $resn_exch{"H322 PCW"}="2H32 PCW";
$resn_exch{"H331 PCW"}="1H33 PCW"; $resn_exch{"H332 PCW"}="2H33 PCW"; $resn_exch{"H341 PCW"}="1H34 PCW";
$resn_exch{"H342 PCW"}="2H34 PCW"; $resn_exch{"H351 PCW"}="1H35 PCW"; $resn_exch{"H352 PCW"}="2H35 PCW";
$resn_exch{"H361 PCW"}="1H36 PCW"; $resn_exch{"H362 PCW"}="2H36 PCW"; $resn_exch{"H371 PCW"}="1H37 PCW";
$resn_exch{"H372 PCW"}="2H37 PCW"; $resn_exch{"H381 PCW"}="1H38 PCW"; $resn_exch{"H382 PCW"}="2H38 PCW";
$resn_exch{"H411 PCW"}="1H41 PCW"; $resn_exch{"H412 PCW"}="2H41 PCW"; $resn_exch{"H421 PCW"}="1H42 PCW";
$resn_exch{"H422 PCW"}="2H42 PCW"; $resn_exch{"H431 PCW"}="1H43 PCW"; $resn_exch{"H432 PCW"}="2H43 PCW";
$resn_exch{"H441 PCW"}="1H44 PCW"; $resn_exch{"H442 PCW"}="2H44 PCW"; $resn_exch{"H451 PCW"}="1H45 PCW";
$resn_exch{"H452 PCW"}="2H45 PCW"; $resn_exch{"H461 PCW"}="1H46 PCW"; $resn_exch{"H462 PCW"}="2H46 PCW";
$resn_exch{"H471 PCW"}="1H47 PCW"; $resn_exch{"H472 PCW"}="2H47 PCW"; $resn_exch{"H481 PCW"}="1H48 PCW";
$resn_exch{"H482 PCW"}="2H48 PCW"; $resn_exch{"H483 PCW"}="3H48 PCW"; $resn_exch{"H2C1 PCX"}="1H2C PCX";
$resn_exch{"H2C2 PCX"}="2H2C PCX"; $resn_exch{"H4C1 PCX"}="1H4C PCX"; $resn_exch{"H4C2 PCX"}="2H4C PCX";
$resn_exch{"H5C1 PCX"}="1H5C PCX"; $resn_exch{"H5C2 PCX"}="2H5C PCX"; $resn_exch{"H3N1 PCX"}="1H3N PCX";
$resn_exch{"H3N2 PCX"}="2H3N PCX"; $resn_exch{"H9C1 PCX"}="1H9C PCX"; $resn_exch{"H9C2 PCX"}="2H9C PCX";
$resn_exch{"H7C1 PCX"}="1H7C PCX"; $resn_exch{"H7C2 PCX"}="2H7C PCX"; $resn_exch{" H91 PCY"}="1H9  PCY";
$resn_exch{" H92 PCY"}="2H9  PCY"; $resn_exch{" H93 PCY"}="3H9  PCY"; $resn_exch{"H101 PCY"}="1H10 PCY";
$resn_exch{"H102 PCY"}="2H10 PCY"; $resn_exch{"H103 PCY"}="3H10 PCY"; $resn_exch{"H111 PCY"}="1H11 PCY";
$resn_exch{"H112 PCY"}="2H11 PCY"; $resn_exch{"H113 PCY"}="3H11 PCY"; $resn_exch{"H131 PCY"}="1H13 PCY";
$resn_exch{"H132 PCY"}="2H13 PCY"; $resn_exch{"H133 PCY"}="3H13 PCY"; $resn_exch{"H161 PCY"}="1H16 PCY";
$resn_exch{"H162 PCY"}="2H16 PCY"; $resn_exch{"H181 PCY"}="1H18 PCY"; $resn_exch{"H182 PCY"}="2H18 PCY";
$resn_exch{"H341 PCY"}="1H34 PCY"; $resn_exch{"H342 PCY"}="2H34 PCY"; $resn_exch{"H343 PCY"}="3H34 PCY";
$resn_exch{"H381 PCY"}="1H38 PCY"; $resn_exch{"H382 PCY"}="2H38 PCY"; $resn_exch{"H383 PCY"}="3H38 PCY";
$resn_exch{"H511 PCZ"}="1H51 PCZ"; $resn_exch{"H512 PCZ"}="2H51 PCZ"; $resn_exch{"H521 PCZ"}="1H52 PCZ";
$resn_exch{"H522 PCZ"}="2H52 PCZ"; $resn_exch{"H131 PCZ"}="1H13 PCZ"; $resn_exch{"H132 PCZ"}="2H13 PCZ";
$resn_exch{"H133 PCZ"}="3H13 PCZ"; $resn_exch{"H191 PCZ"}="1H19 PCZ"; $resn_exch{"H192 PCZ"}="2H19 PCZ";
$resn_exch{" H11 PD1"}="1H1  PD1"; $resn_exch{" H12 PD1"}="2H1  PD1"; $resn_exch{"H13A PD1"}="3H1  PD1";
$resn_exch{"H21A PD1"}="1H2  PD1"; $resn_exch{" H22 PD1"}="2H2  PD1"; $resn_exch{" H31 PD1"}="1H3  PD1";
$resn_exch{" H32 PD1"}="2H3  PD1"; $resn_exch{" H33 PD1"}="3H3  PD1"; $resn_exch{" H41 PD1"}="1H4  PD1";
$resn_exch{" H42 PD1"}="2H4  PD1"; $resn_exch{" H51 PD1"}="1H5  PD1"; $resn_exch{" H52 PD1"}="2H5  PD1";
$resn_exch{" H61 PD1"}="1H6  PD1"; $resn_exch{" H62 PD1"}="2H6  PD1"; $resn_exch{" H71 PD1"}="1H7  PD1";
$resn_exch{" H72 PD1"}="2H7  PD1"; $resn_exch{" H81 PD1"}="1H8  PD1"; $resn_exch{" H82 PD1"}="2H8  PD1";
$resn_exch{"H221 PD1"}="1H22 PD1"; $resn_exch{"H222 PD1"}="2H22 PD1"; $resn_exch{"H223 PD1"}="3H22 PD1";
$resn_exch{"H231 PD1"}="1H23 PD1"; $resn_exch{"H232 PD1"}="2H23 PD1"; $resn_exch{"H233 PD1"}="3H23 PD1";
$resn_exch{"H261 PD1"}="1H26 PD1"; $resn_exch{"H262 PD1"}="2H26 PD1"; $resn_exch{"H263 PD1"}="3H26 PD1";
$resn_exch{"H271 PD1"}="1H27 PD1"; $resn_exch{"H272 PD1"}="2H27 PD1"; $resn_exch{"H273 PD1"}="3H27 PD1";
$resn_exch{"H281 PD1"}="1H28 PD1"; $resn_exch{"H282 PD1"}="2H28 PD1"; $resn_exch{"H283 PD1"}="3H28 PD1";
$resn_exch{"H1'' PD7"}="H1\"  PD7"; $resn_exch{"H1AA PD7"}="H1\"A PD7"; $resn_exch{"H2'' PD7"}="H2\"  PD7";
$resn_exch{"H2AA PD7"}="H2\"A PD7"; $resn_exch{"H3'' PD7"}="H3\"  PD7"; $resn_exch{"H3AA PD7"}="H3\"A PD7";
$resn_exch{"H4'' PD7"}="H4\"  PD7"; $resn_exch{"H4AA PD7"}="H4\"A PD7"; $resn_exch{"H5'' PD7"}="H5\"  PD7";
$resn_exch{"H5AA PD7"}="H5\"A PD7"; $resn_exch{"H6'' PD7"}="H6\"  PD7"; $resn_exch{"H6AA PD7"}="H6\"A PD7";
$resn_exch{"H6AB PD7"}="H6\"B PD7"; $resn_exch{"H5N1 PD8"}="1H5N PD8"; $resn_exch{"H5N2 PD8"}="2H5N PD8";
$resn_exch{"H111 PD8"}="1H11 PD8"; $resn_exch{"H112 PD8"}="2H11 PD8"; $resn_exch{"H2A1 PDA"}="1H2A PDA";
$resn_exch{"H2A2 PDA"}="2H2A PDA"; $resn_exch{"H2A3 PDA"}="3H2A PDA"; $resn_exch{"H4A1 PDA"}="1H4A PDA";
$resn_exch{"H4A2 PDA"}="2H4A PDA"; $resn_exch{"H5A1 PDA"}="1H5A PDA"; $resn_exch{"H5A2 PDA"}="2H5A PDA";
$resn_exch{" HB1 PDA"}="1HB  PDA"; $resn_exch{" HB2 PDA"}="2HB  PDA"; $resn_exch{" HB3 PDA"}="3HB  PDA";
$resn_exch{" H   PDD"}=" HN  PDD"; $resn_exch{"H2A1 PDD"}="1H2A PDD"; $resn_exch{"H2A2 PDD"}="2H2A PDD";
$resn_exch{"H2A3 PDD"}="3H2A PDD"; $resn_exch{"H4A1 PDD"}="1H4A PDD"; $resn_exch{"H4A2 PDD"}="2H4A PDD";
$resn_exch{"H5A1 PDD"}="1H5A PDD"; $resn_exch{"H5A2 PDD"}="2H5A PDD"; $resn_exch{"HOP2 PDD"}="2HOP PDD";
$resn_exch{"HOP3 PDD"}="3HOP PDD"; $resn_exch{" HB1 PDD"}="1HB  PDD"; $resn_exch{" HB2 PDD"}="2HB  PDD";
$resn_exch{" HB3 PDD"}="3HB  PDD"; $resn_exch{"HC81 PDE"}="1HC8 PDE"; $resn_exch{"HC82 PDE"}="2HC8 PDE";
$resn_exch{"HC91 PDE"}="1HC9 PDE"; $resn_exch{"HC92 PDE"}="2HC9 PDE"; $resn_exch{"H101 PDE"}="1H10 PDE";
$resn_exch{"H102 PDE"}="2H10 PDE"; $resn_exch{"H1D1 PDE"}="1H1D PDE"; $resn_exch{"H1D2 PDE"}="2H1D PDE";
$resn_exch{"H1D3 PDE"}="3H1D PDE"; $resn_exch{"H2A1 PDG"}="1H2A PDG"; $resn_exch{"H2A2 PDG"}="2H2A PDG";
$resn_exch{"H2A3 PDG"}="3H2A PDG"; $resn_exch{"H4A1 PDG"}="1H4A PDG"; $resn_exch{"H4A2 PDG"}="2H4A PDG";
$resn_exch{"H5A1 PDG"}="1H5A PDG"; $resn_exch{"H5A2 PDG"}="2H5A PDG"; $resn_exch{"HBA1 PDG"}="1HBA PDG";
$resn_exch{"HBA2 PDG"}="2HBA PDG"; $resn_exch{"HGA1 PDG"}="1HGA PDG"; $resn_exch{"HGA2 PDG"}="2HGA PDG";
$resn_exch{" HB1 PDH"}="1HB  PDH"; $resn_exch{" HB2 PDH"}="2HB  PDH"; $resn_exch{"HOP3 PDI"}="3HOP PDI";
$resn_exch{"HOP2 PDI"}="2HOP PDI"; $resn_exch{"H111 PDJ"}="1H11 PDJ"; $resn_exch{"H112 PDJ"}="2H11 PDJ";
$resn_exch{"H113 PDJ"}="3H11 PDJ"; $resn_exch{"H121 PDJ"}="1H12 PDJ"; $resn_exch{"H122 PDJ"}="2H12 PDJ";
$resn_exch{"H131 PDJ"}="1H13 PDJ"; $resn_exch{"H132 PDJ"}="2H13 PDJ"; $resn_exch{"H141 PDJ"}="1H14 PDJ";
$resn_exch{"H142 PDJ"}="2H14 PDJ"; $resn_exch{"H151 PDJ"}="1H15 PDJ"; $resn_exch{"H152 PDJ"}="2H15 PDJ";
$resn_exch{"H161 PDJ"}="1H16 PDJ"; $resn_exch{"H162 PDJ"}="2H16 PDJ"; $resn_exch{"H171 PDJ"}="1H17 PDJ";
$resn_exch{"H172 PDJ"}="2H17 PDJ"; $resn_exch{"H181 PDJ"}="1H18 PDJ"; $resn_exch{"H182 PDJ"}="2H18 PDJ";
$resn_exch{"H191 PDJ"}="1H19 PDJ"; $resn_exch{"H192 PDJ"}="2H19 PDJ"; $resn_exch{"H201 PDJ"}="1H20 PDJ";
$resn_exch{"H202 PDJ"}="2H20 PDJ"; $resn_exch{"H211 PDJ"}="1H21 PDJ"; $resn_exch{"H212 PDJ"}="2H21 PDJ";
$resn_exch{"H221 PDJ"}="1H22 PDJ"; $resn_exch{"H222 PDJ"}="2H22 PDJ"; $resn_exch{"H231 PDJ"}="1H23 PDJ";
$resn_exch{"H232 PDJ"}="2H23 PDJ"; $resn_exch{"H241 PDJ"}="1H24 PDJ"; $resn_exch{"H242 PDJ"}="2H24 PDJ";
$resn_exch{"H251 PDJ"}="1H25 PDJ"; $resn_exch{"H252 PDJ"}="2H25 PDJ"; $resn_exch{" H31 PDJ"}="1H3  PDJ";
$resn_exch{" H32 PDJ"}="2H3  PDJ"; $resn_exch{" H11 PDJ"}="1H1  PDJ"; $resn_exch{" H12 PDJ"}="2H1  PDJ";
$resn_exch{" HB1 PDJ"}="1HB  PDJ"; $resn_exch{" HB2 PDJ"}="2HB  PDJ"; $resn_exch{" HN1 PDJ"}="1HN  PDJ";
$resn_exch{" HN2 PDJ"}="2HN  PDJ"; $resn_exch{" H1  PDJ"}="1H   PDJ"; $resn_exch{" H2A PDJ"}="2H   PDJ";
$resn_exch{"H451 PDJ"}="1H45 PDJ"; $resn_exch{"H452 PDJ"}="2H45 PDJ"; $resn_exch{"H441 PDJ"}="1H44 PDJ";
$resn_exch{"H442 PDJ"}="2H44 PDJ"; $resn_exch{"H431 PDJ"}="1H43 PDJ"; $resn_exch{"H432 PDJ"}="2H43 PDJ";
$resn_exch{"H421 PDJ"}="1H42 PDJ"; $resn_exch{"H422 PDJ"}="2H42 PDJ"; $resn_exch{"H411 PDJ"}="1H41 PDJ";
$resn_exch{"H412 PDJ"}="2H41 PDJ"; $resn_exch{"H401 PDJ"}="1H40 PDJ"; $resn_exch{"H402 PDJ"}="2H40 PDJ";
$resn_exch{"H391 PDJ"}="1H39 PDJ"; $resn_exch{"H392 PDJ"}="2H39 PDJ"; $resn_exch{"H381 PDJ"}="1H38 PDJ";
$resn_exch{"H382 PDJ"}="2H38 PDJ"; $resn_exch{"H371 PDJ"}="1H37 PDJ"; $resn_exch{"H372 PDJ"}="2H37 PDJ";
$resn_exch{"H361 PDJ"}="1H36 PDJ"; $resn_exch{"H362 PDJ"}="2H36 PDJ"; $resn_exch{"H351 PDJ"}="1H35 PDJ";
$resn_exch{"H352 PDJ"}="2H35 PDJ"; $resn_exch{"H341 PDJ"}="1H34 PDJ"; $resn_exch{"H342 PDJ"}="2H34 PDJ";
$resn_exch{"H331 PDJ"}="1H33 PDJ"; $resn_exch{"H332 PDJ"}="2H33 PDJ"; $resn_exch{"H321 PDJ"}="1H32 PDJ";
$resn_exch{"H322 PDJ"}="2H32 PDJ"; $resn_exch{"H311 PDJ"}="1H31 PDJ"; $resn_exch{"H312 PDJ"}="2H31 PDJ";
$resn_exch{"H313 PDJ"}="3H31 PDJ"; $resn_exch{" H   PDL"}=" HN  PDL"; $resn_exch{"H2A1 PDL"}="1H2A PDL";
$resn_exch{"H2A2 PDL"}="2H2A PDL"; $resn_exch{"H2A3 PDL"}="3H2A PDL"; $resn_exch{"H4A1 PDL"}="1H4A PDL";
$resn_exch{"H4A2 PDL"}="2H4A PDL"; $resn_exch{"H5A1 PDL"}="1H5A PDL"; $resn_exch{"H5A2 PDL"}="2H5A PDL";
$resn_exch{"HOP2 PDL"}="2HOP PDL"; $resn_exch{"HOP3 PDL"}="3HOP PDL"; $resn_exch{" HB1 PDL"}="1HB  PDL";
$resn_exch{" HB2 PDL"}="2HB  PDL"; $resn_exch{" HB3 PDL"}="3HB  PDL"; $resn_exch{"HP81 PDM"}="1HP8 PDM";
$resn_exch{"HP82 PDM"}="2HP8 PDM"; $resn_exch{"HP91 PDM"}="1HP9 PDM"; $resn_exch{"HP92 PDM"}="2HP9 PDM";
$resn_exch{"HP2N PDM"}="NHP2 PDM"; $resn_exch{"HP3O PDM"}="OHP3 PDM"; $resn_exch{"HP4O PDM"}="OHP4 PDM";
$resn_exch{"H121 PDM"}="1H12 PDM"; $resn_exch{"H122 PDM"}="2H12 PDM"; $resn_exch{"H131 PDM"}="1H13 PDM";
$resn_exch{"H132 PDM"}="2H13 PDM"; $resn_exch{" HC1 PDN"}="1HC  PDN"; $resn_exch{" HC2 PDN"}="2HC  PDN";
$resn_exch{" HC4 PDN"}="4HC  PDN"; $resn_exch{"HC61 PDN"}="1HC6 PDN"; $resn_exch{"HC62 PDN"}="2HC6 PDN";
$resn_exch{"HC71 PDN"}="1HC7 PDN"; $resn_exch{"HC72 PDN"}="2HC7 PDN"; $resn_exch{" HC8 PDN"}="8HC  PDN";
$resn_exch{" HC9 PDN"}="9HC  PDN"; $resn_exch{"H121 PDN"}="1H12 PDN"; $resn_exch{"H122 PDN"}="2H12 PDN";
$resn_exch{"HC41 PDN"}="1HC4 PDN"; $resn_exch{"H151 PDN"}="1H15 PDN"; $resn_exch{"H152 PDN"}="2H15 PDN";
$resn_exch{"H161 PDN"}="1H16 PDN"; $resn_exch{"H162 PDN"}="2H16 PDN"; $resn_exch{"H181 PDN"}="1H18 PDN";
$resn_exch{"H182 PDN"}="2H18 PDN"; $resn_exch{"H183 PDN"}="3H18 PDN"; $resn_exch{"H191 PDN"}="1H19 PDN";
$resn_exch{"H192 PDN"}="2H19 PDN"; $resn_exch{"H193 PDN"}="3H19 PDN"; $resn_exch{"HC12 PDN"}="2HC1 PDN";
$resn_exch{" HO3 PDN"}="3HO  PDN"; $resn_exch{" HO4 PDN"}="4HO  PDN"; $resn_exch{" HO5 PDN"}="5HO  PDN";
$resn_exch{" H11 PDO"}="1H1  PDO"; $resn_exch{" H12 PDO"}="2H1  PDO"; $resn_exch{" H21 PDO"}="1H2  PDO";
$resn_exch{" H22 PDO"}="2H2  PDO"; $resn_exch{" H31 PDO"}="1H3  PDO"; $resn_exch{" H32 PDO"}="2H3  PDO";
$resn_exch{"H2A1 PDP"}="1H2A PDP"; $resn_exch{"H2A2 PDP"}="2H2A PDP"; $resn_exch{"H2A3 PDP"}="3H2A PDP";
$resn_exch{"H5A1 PDP"}="1H5A PDP"; $resn_exch{"H5A2 PDP"}="2H5A PDP"; $resn_exch{"HOA2 PDP"}="2HOA PDP";
$resn_exch{"HOB2 PDP"}="2HOB PDP"; $resn_exch{"HOB3 PDP"}="3HOB PDP"; $resn_exch{"H261 PDS"}="1H26 PDS";
$resn_exch{"H262 PDS"}="2H26 PDS"; $resn_exch{"H263 PDS"}="3H26 PDS"; $resn_exch{"H101 PDS"}="1H10 PDS";
$resn_exch{"H102 PDS"}="2H10 PDS"; $resn_exch{"H103 PDS"}="3H10 PDS"; $resn_exch{"H171 PDS"}="1H17 PDS";
$resn_exch{"H172 PDS"}="2H17 PDS"; $resn_exch{"H221 PDS"}="1H22 PDS"; $resn_exch{"H222 PDS"}="2H22 PDS";
$resn_exch{"H271 PDS"}="1H27 PDS"; $resn_exch{"H272 PDS"}="2H27 PDS"; $resn_exch{"H321 PDS"}="1H32 PDS";
$resn_exch{"H322 PDS"}="2H32 PDS"; $resn_exch{"H323 PDS"}="3H32 PDS"; $resn_exch{"H331 PDS"}="1H33 PDS";
$resn_exch{"H332 PDS"}="2H33 PDS"; $resn_exch{"H333 PDS"}="3H33 PDS"; $resn_exch{" H21 PDT"}="1H2  PDT";
$resn_exch{" H22 PDT"}="2H2  PDT"; $resn_exch{" H11 PDT"}="1H1  PDT"; $resn_exch{" H12 PDT"}="2H1  PDT";
$resn_exch{" H31 PDT"}="1H3  PDT"; $resn_exch{" H32 PDT"}="2H3  PDT"; $resn_exch{"HOP3 PDU"}="3HOP PDU";
$resn_exch{"HOP2 PDU"}="2HOP PDU"; $resn_exch{" H5' PDU"}="1H5* PDU"; $resn_exch{"H5'' PDU"}="2H5* PDU";
$resn_exch{" H4' PDU"}=" H4* PDU"; $resn_exch{" H3' PDU"}=" H3* PDU"; $resn_exch{"HO3' PDU"}="H3T  PDU";
$resn_exch{"H2'' PDU"}="2H2* PDU"; $resn_exch{" H2' PDU"}="1H2* PDU"; $resn_exch{" H1' PDU"}=" H1* PDU";
$resn_exch{"H5A1 PDU"}="1H5A PDU"; $resn_exch{"H5A2 PDU"}="2H5A PDU"; $resn_exch{"H5B1 PDU"}="1H5B PDU";
$resn_exch{"H5B2 PDU"}="2H5B PDU"; $resn_exch{" H71 PDU"}="1H5M PDU"; $resn_exch{" H72 PDU"}="2H5M PDU";
$resn_exch{" H73 PDU"}="3H5M PDU"; $resn_exch{"H6C1 PDX"}="1H6C PDX"; $resn_exch{"H6C2 PDX"}="2H6C PDX";
$resn_exch{"HN61 PE0"}="1HN6 PE0"; $resn_exch{"HN62 PE0"}="2HN6 PE0"; $resn_exch{"H2A1 PE1"}="1H2A PE1";
$resn_exch{"H2A2 PE1"}="2H2A PE1"; $resn_exch{"H2A3 PE1"}="3H2A PE1"; $resn_exch{"H4A1 PE1"}="1H4A PE1";
$resn_exch{"H4A2 PE1"}="2H4A PE1"; $resn_exch{"H5A1 PE1"}="1H5A PE1"; $resn_exch{"H5A2 PE1"}="2H5A PE1";
$resn_exch{"HBC1 PE1"}="1HBC PE1"; $resn_exch{"HBC2 PE1"}="2HBC PE1"; $resn_exch{"HGC1 PE1"}="1HGC PE1";
$resn_exch{"HGC2 PE1"}="2HGC PE1"; $resn_exch{"HDC1 PE1"}="1HDC PE1"; $resn_exch{"HDC2 PE1"}="2HDC PE1";
$resn_exch{"HEC1 PE1"}="1HEC PE1"; $resn_exch{"HEC2 PE1"}="2HEC PE1"; $resn_exch{"HZN1 PE1"}="1HZN PE1";
$resn_exch{"HZN2 PE1"}="2HZN PE1"; $resn_exch{"H4'1 PE2"}="1H4* PE2"; $resn_exch{"H4'2 PE2"}="2H4* PE2";
$resn_exch{"HO4' PE2"}="*HO4 PE2"; $resn_exch{"H3'1 PE2"}="1H3* PE2"; $resn_exch{"H3'2 PE2"}="2H3* PE2";
$resn_exch{"HO3' PE2"}="*HO3 PE2"; $resn_exch{" H2' PE2"}=" H2* PE2"; $resn_exch{"H121 PE2"}="1H12 PE2";
$resn_exch{"H122 PE2"}="2H12 PE2"; $resn_exch{"H1'1 PE2"}="1H1* PE2"; $resn_exch{"H1'2 PE2"}="2H1* PE2";
$resn_exch{"HN21 PE2"}="1HN2 PE2"; $resn_exch{"HN22 PE2"}="2HN2 PE2"; $resn_exch{"H421 PE3"}="1H42 PE3";
$resn_exch{"H422 PE3"}="2H42 PE3"; $resn_exch{"H411 PE3"}="1H41 PE3"; $resn_exch{"H412 PE3"}="2H41 PE3";
$resn_exch{"H391 PE3"}="1H39 PE3"; $resn_exch{"H392 PE3"}="2H39 PE3"; $resn_exch{"H381 PE3"}="1H38 PE3";
$resn_exch{"H382 PE3"}="2H38 PE3"; $resn_exch{"H361 PE3"}="1H36 PE3"; $resn_exch{"H362 PE3"}="2H36 PE3";
$resn_exch{"H351 PE3"}="1H35 PE3"; $resn_exch{"H352 PE3"}="2H35 PE3"; $resn_exch{"H331 PE3"}="1H33 PE3";
$resn_exch{"H332 PE3"}="2H33 PE3"; $resn_exch{"H321 PE3"}="1H32 PE3"; $resn_exch{"H322 PE3"}="2H32 PE3";
$resn_exch{"H301 PE3"}="1H30 PE3"; $resn_exch{"H302 PE3"}="2H30 PE3"; $resn_exch{"H291 PE3"}="1H29 PE3";
$resn_exch{"H292 PE3"}="2H29 PE3"; $resn_exch{"H271 PE3"}="1H27 PE3"; $resn_exch{"H272 PE3"}="2H27 PE3";
$resn_exch{"H261 PE3"}="1H26 PE3"; $resn_exch{"H262 PE3"}="2H26 PE3"; $resn_exch{"H241 PE3"}="1H24 PE3";
$resn_exch{"H242 PE3"}="2H24 PE3"; $resn_exch{"H231 PE3"}="1H23 PE3"; $resn_exch{"H232 PE3"}="2H23 PE3";
$resn_exch{"H211 PE3"}="1H21 PE3"; $resn_exch{"H212 PE3"}="2H21 PE3"; $resn_exch{"H201 PE3"}="1H20 PE3";
$resn_exch{"H202 PE3"}="2H20 PE3"; $resn_exch{"H181 PE3"}="1H18 PE3"; $resn_exch{"H182 PE3"}="2H18 PE3";
$resn_exch{"H171 PE3"}="1H17 PE3"; $resn_exch{"H172 PE3"}="2H17 PE3"; $resn_exch{"H151 PE3"}="1H15 PE3";
$resn_exch{"H152 PE3"}="2H15 PE3"; $resn_exch{"H141 PE3"}="1H14 PE3"; $resn_exch{"H142 PE3"}="2H14 PE3";
$resn_exch{"H121 PE3"}="1H12 PE3"; $resn_exch{"H122 PE3"}="2H12 PE3"; $resn_exch{"H111 PE3"}="1H11 PE3";
$resn_exch{"H112 PE3"}="2H11 PE3"; $resn_exch{" H91 PE3"}="1H9  PE3"; $resn_exch{" H92 PE3"}="2H9  PE3";
$resn_exch{" H81 PE3"}="1H8  PE3"; $resn_exch{" H82 PE3"}="2H8  PE3"; $resn_exch{" H61 PE3"}="1H6  PE3";
$resn_exch{" H62 PE3"}="2H6  PE3"; $resn_exch{" H51 PE3"}="1H5  PE3"; $resn_exch{" H52 PE3"}="2H5  PE3";
$resn_exch{" H31 PE3"}="1H3  PE3"; $resn_exch{" H32 PE3"}="2H3  PE3"; $resn_exch{" H21 PE3"}="1H2  PE3";
$resn_exch{" H22 PE3"}="2H2  PE3"; $resn_exch{" H11 PE4"}="1H1  PE4"; $resn_exch{" H12 PE4"}="2H1  PE4";
$resn_exch{" H21 PE4"}="1H2  PE4"; $resn_exch{" H22 PE4"}="2H2  PE4"; $resn_exch{" H31 PE4"}="1H3  PE4";
$resn_exch{" H32 PE4"}="2H3  PE4"; $resn_exch{" H41 PE4"}="1H4  PE4"; $resn_exch{" H42 PE4"}="2H4  PE4";
$resn_exch{" H51 PE4"}="1H5  PE4"; $resn_exch{" H52 PE4"}="2H5  PE4"; $resn_exch{" H61 PE4"}="1H6  PE4";
$resn_exch{" H62 PE4"}="2H6  PE4"; $resn_exch{" H71 PE4"}="1H7  PE4"; $resn_exch{" H72 PE4"}="2H7  PE4";
$resn_exch{" H81 PE4"}="1H8  PE4"; $resn_exch{" H82 PE4"}="2H8  PE4"; $resn_exch{" H91 PE4"}="1H9  PE4";
$resn_exch{" H92 PE4"}="2H9  PE4"; $resn_exch{"H101 PE4"}="1H10 PE4"; $resn_exch{"H102 PE4"}="2H10 PE4";
$resn_exch{"H111 PE4"}="1H11 PE4"; $resn_exch{"H112 PE4"}="2H11 PE4"; $resn_exch{"H121 PE4"}="1H12 PE4";
$resn_exch{"H122 PE4"}="2H12 PE4"; $resn_exch{"H131 PE4"}="1H13 PE4"; $resn_exch{"H132 PE4"}="2H13 PE4";
$resn_exch{"H141 PE4"}="1H14 PE4"; $resn_exch{"H142 PE4"}="2H14 PE4"; $resn_exch{"H151 PE4"}="1H15 PE4";
$resn_exch{"H152 PE4"}="2H15 PE4"; $resn_exch{"H161 PE4"}="1H16 PE4"; $resn_exch{"H162 PE4"}="2H16 PE4";
$resn_exch{"H163 PE4"}="3H16 PE4"; $resn_exch{"H481 PE5"}="1H48 PE5"; $resn_exch{"H482 PE5"}="2H48 PE5";
$resn_exch{"H483 PE5"}="3H48 PE5"; $resn_exch{"H501 PE5"}="1H50 PE5"; $resn_exch{"H502 PE5"}="2H50 PE5";
$resn_exch{" H11 PE5"}="1H1  PE5"; $resn_exch{" H12 PE5"}="2H1  PE5"; $resn_exch{" H21 PE5"}="1H2  PE5";
$resn_exch{" H22 PE5"}="2H2  PE5"; $resn_exch{" H31 PE5"}="1H3  PE5"; $resn_exch{" H32 PE5"}="2H3  PE5";
$resn_exch{" H41 PE5"}="1H4  PE5"; $resn_exch{" H42 PE5"}="2H4  PE5"; $resn_exch{" H51 PE5"}="1H5  PE5";
$resn_exch{"H52A PE5"}="2H5  PE5"; $resn_exch{" H61 PE5"}="1H6  PE5"; $resn_exch{" H62 PE5"}="2H6  PE5";
$resn_exch{" H71 PE5"}="1H7  PE5"; $resn_exch{" H72 PE5"}="2H7  PE5"; $resn_exch{" H81 PE5"}="1H8  PE5";
$resn_exch{" H82 PE5"}="2H8  PE5"; $resn_exch{" H91 PE5"}="1H9  PE5"; $resn_exch{" H92 PE5"}="2H9  PE5";
$resn_exch{"H101 PE5"}="1H10 PE5"; $resn_exch{"H102 PE5"}="2H10 PE5"; $resn_exch{"H111 PE5"}="1H11 PE5";
$resn_exch{"H112 PE5"}="2H11 PE5"; $resn_exch{"H121 PE5"}="1H12 PE5"; $resn_exch{"H122 PE5"}="2H12 PE5";
$resn_exch{"H131 PE5"}="1H13 PE5"; $resn_exch{"H132 PE5"}="2H13 PE5"; $resn_exch{"H141 PE5"}="1H14 PE5";
$resn_exch{"H142 PE5"}="2H14 PE5"; $resn_exch{"H151 PE5"}="1H15 PE5"; $resn_exch{"H152 PE5"}="2H15 PE5";
$resn_exch{"H161 PE5"}="1H16 PE5"; $resn_exch{"H162 PE5"}="2H16 PE5"; $resn_exch{" H31 PE6"}="1H3  PE6";
$resn_exch{" H32 PE6"}="2H3  PE6"; $resn_exch{"HOP2 PE6"}="2HOP PE6"; $resn_exch{" H41 PE6"}="1H4  PE6";
$resn_exch{" H42 PE6"}="2H4  PE6"; $resn_exch{" H61 PE6"}="1H6  PE6"; $resn_exch{" H62 PE6"}="2H6  PE6";
$resn_exch{" H71 PE6"}="1H7  PE6"; $resn_exch{" H72 PE6"}="2H7  PE6"; $resn_exch{" H91 PE6"}="1H9  PE6";
$resn_exch{" H92 PE6"}="2H9  PE6"; $resn_exch{"H101 PE6"}="1H10 PE6"; $resn_exch{"H102 PE6"}="2H10 PE6";
$resn_exch{"H191 PE6"}="1H19 PE6"; $resn_exch{"H192 PE6"}="2H19 PE6"; $resn_exch{"H181 PE6"}="1H18 PE6";
$resn_exch{"H182 PE6"}="2H18 PE6"; $resn_exch{"H161 PE6"}="1H16 PE6"; $resn_exch{"H162 PE6"}="2H16 PE6";
$resn_exch{"H151 PE6"}="1H15 PE6"; $resn_exch{"H152 PE6"}="2H15 PE6"; $resn_exch{"H131 PE6"}="1H13 PE6";
$resn_exch{"H132 PE6"}="2H13 PE6"; $resn_exch{"H121 PE6"}="1H12 PE6"; $resn_exch{"H122 PE6"}="2H12 PE6";
$resn_exch{"HOP3 PE6"}="3HOP PE6"; $resn_exch{" H21 PE7"}="1H2  PE7"; $resn_exch{" H22 PE7"}="2H2  PE7";
$resn_exch{" H31 PE7"}="1H3  PE7"; $resn_exch{" H32 PE7"}="2H3  PE7"; $resn_exch{" H51 PE7"}="1H5  PE7";
$resn_exch{" H52 PE7"}="2H5  PE7"; $resn_exch{" H61 PE7"}="1H6  PE7"; $resn_exch{" H62 PE7"}="2H6  PE7";
$resn_exch{" H81 PE7"}="1H8  PE7"; $resn_exch{" H82 PE7"}="2H8  PE7"; $resn_exch{" H91 PE7"}="1H9  PE7";
$resn_exch{" H92 PE7"}="2H9  PE7"; $resn_exch{"H111 PE7"}="1H11 PE7"; $resn_exch{"H112 PE7"}="2H11 PE7";
$resn_exch{"H121 PE7"}="1H12 PE7"; $resn_exch{"H122 PE7"}="2H12 PE7"; $resn_exch{"H141 PE7"}="1H14 PE7";
$resn_exch{"H142 PE7"}="2H14 PE7"; $resn_exch{"H151 PE7"}="1H15 PE7"; $resn_exch{"H152 PE7"}="2H15 PE7";
$resn_exch{"H171 PE7"}="1H17 PE7"; $resn_exch{"H172 PE7"}="2H17 PE7"; $resn_exch{"H181 PE7"}="1H18 PE7";
$resn_exch{"H182 PE7"}="2H18 PE7"; $resn_exch{"H201 PE7"}="1H20 PE7"; $resn_exch{"H202 PE7"}="2H20 PE7";
$resn_exch{"H211 PE7"}="1H21 PE7"; $resn_exch{"H212 PE7"}="2H21 PE7"; $resn_exch{" HO2 PE7"}="2HO  PE7";
$resn_exch{" H21 PE8"}="1H2  PE8"; $resn_exch{" H22 PE8"}="2H2  PE8"; $resn_exch{" H31 PE8"}="1H3  PE8";
$resn_exch{" H32 PE8"}="2H3  PE8"; $resn_exch{" H51 PE8"}="1H5  PE8"; $resn_exch{" H52 PE8"}="2H5  PE8";
$resn_exch{" H61 PE8"}="1H6  PE8"; $resn_exch{" H62 PE8"}="2H6  PE8"; $resn_exch{" H81 PE8"}="1H8  PE8";
$resn_exch{" H82 PE8"}="2H8  PE8"; $resn_exch{" H91 PE8"}="1H9  PE8"; $resn_exch{" H92 PE8"}="2H9  PE8";
$resn_exch{"H111 PE8"}="1H11 PE8"; $resn_exch{"H112 PE8"}="2H11 PE8"; $resn_exch{"H121 PE8"}="1H12 PE8";
$resn_exch{"H122 PE8"}="2H12 PE8"; $resn_exch{"H141 PE8"}="1H14 PE8"; $resn_exch{"H142 PE8"}="2H14 PE8";
$resn_exch{"H151 PE8"}="1H15 PE8"; $resn_exch{"H152 PE8"}="2H15 PE8"; $resn_exch{"H171 PE8"}="1H17 PE8";
$resn_exch{"H172 PE8"}="2H17 PE8"; $resn_exch{"H181 PE8"}="1H18 PE8"; $resn_exch{"H182 PE8"}="2H18 PE8";
$resn_exch{"H201 PE8"}="1H20 PE8"; $resn_exch{"H202 PE8"}="2H20 PE8"; $resn_exch{"H211 PE8"}="1H21 PE8";
$resn_exch{"H212 PE8"}="2H21 PE8"; $resn_exch{"H231 PE8"}="1H23 PE8"; $resn_exch{"H232 PE8"}="2H23 PE8";
$resn_exch{"H241 PE8"}="1H24 PE8"; $resn_exch{"H242 PE8"}="2H24 PE8"; $resn_exch{"HAA1 PE9"}="1HAA PE9";
$resn_exch{"HAA2 PE9"}="2HAA PE9"; $resn_exch{"HAA3 PE9"}="3HAA PE9"; $resn_exch{"HAC1 PE9"}="1HAC PE9";
$resn_exch{"HAC2 PE9"}="2HAC PE9"; $resn_exch{"HAE1 PE9"}="1HAE PE9"; $resn_exch{"HAE2 PE9"}="2HAE PE9";
$resn_exch{"HAF1 PE9"}="1HAF PE9"; $resn_exch{"HAF2 PE9"}="2HAF PE9"; $resn_exch{"HAD1 PE9"}="1HAD PE9";
$resn_exch{"HAD2 PE9"}="2HAD PE9"; $resn_exch{" H21 PEA"}="1H2  PEA"; $resn_exch{" H22 PEA"}="2H2  PEA";
$resn_exch{" H11 PEA"}="1H1  PEA"; $resn_exch{" H12 PEA"}="2H1  PEA"; $resn_exch{" HN1 PEA"}="1HN  PEA";
$resn_exch{" HN2 PEA"}="2HN  PEA"; $resn_exch{" HN3 PEA"}="3HN  PEA"; $resn_exch{"HHC1 PEB"}="1HHC PEB";
$resn_exch{"HHC2 PEB"}="2HHC PEB"; $resn_exch{"HMC1 PEB"}="1HMC PEB"; $resn_exch{"HMC2 PEB"}="2HMC PEB";
$resn_exch{"HMC3 PEB"}="3HMC PEB"; $resn_exch{"HAC1 PEB"}="1HAC PEB"; $resn_exch{"HAC2 PEB"}="2HAC PEB";
$resn_exch{"HBC1 PEB"}="1HBC PEB"; $resn_exch{"HBC2 PEB"}="2HBC PEB"; $resn_exch{"HOC2 PEB"}="2HOC PEB";
$resn_exch{"H1D1 PEB"}="1H1D PEB"; $resn_exch{"HMD1 PEB"}="1HMD PEB"; $resn_exch{"HMD2 PEB"}="2HMD PEB";
$resn_exch{"HMD3 PEB"}="3HMD PEB"; $resn_exch{"HAD1 PEB"}="1HAD PEB"; $resn_exch{"HBD1 PEB"}="1HBD PEB";
$resn_exch{"HBD2 PEB"}="2HBD PEB"; $resn_exch{"H2A1 PEB"}="1H2A PEB"; $resn_exch{"H3A1 PEB"}="1H3A PEB";
$resn_exch{"HMA1 PEB"}="1HMA PEB"; $resn_exch{"HMA2 PEB"}="2HMA PEB"; $resn_exch{"HMA3 PEB"}="3HMA PEB";
$resn_exch{"HBA1 PEB"}="1HBA PEB"; $resn_exch{"HBA2 PEB"}="2HBA PEB"; $resn_exch{"HBA3 PEB"}="3HBA PEB";
$resn_exch{"HHA1 PEB"}="1HHA PEB"; $resn_exch{"HAA1 PEB"}="1HAA PEB"; $resn_exch{"HAA2 PEB"}="2HAA PEB";
$resn_exch{"HHB1 PEB"}="1HHB PEB"; $resn_exch{"HMB1 PEB"}="1HMB PEB"; $resn_exch{"HMB2 PEB"}="2HMB PEB";
$resn_exch{"HMB3 PEB"}="3HMB PEB"; $resn_exch{"HAB1 PEB"}="1HAB PEB"; $resn_exch{"HAB2 PEB"}="2HAB PEB";
$resn_exch{"HBB1 PEB"}="1HBB PEB"; $resn_exch{"HBB2 PEB"}="2HBB PEB"; $resn_exch{"HOB2 PEB"}="2HOB PEB";
$resn_exch{" H   PEC"}="1HN  PEC"; $resn_exch{" H2  PEC"}="2HN  PEC"; $resn_exch{" HB2 PEC"}="1HB  PEC";
$resn_exch{" HB3 PEC"}="2HB  PEC"; $resn_exch{" H11 PEC"}="1H1  PEC"; $resn_exch{" H12 PEC"}="2H1  PEC";
$resn_exch{" H21 PEC"}="1H2  PEC"; $resn_exch{" H22 PEC"}="2H2  PEC"; $resn_exch{" H31 PEC"}="1H3  PEC";
$resn_exch{" H32 PEC"}="2H3  PEC"; $resn_exch{" H41 PEC"}="1H4  PEC"; $resn_exch{" H42 PEC"}="2H4  PEC";
$resn_exch{" H51 PEC"}="1H5  PEC"; $resn_exch{" H52 PEC"}="2H5  PEC"; $resn_exch{" H53 PEC"}="3H5  PEC";
$resn_exch{"HOP2 PED"}="2HOP PED"; $resn_exch{"HOP3 PED"}="3HOP PED"; $resn_exch{"H2'1 PED"}="1H2* PED";
$resn_exch{"H2'2 PED"}="2H2* PED"; $resn_exch{"H5'1 PED"}="1H5* PED"; $resn_exch{"H5'2 PED"}="2H5* PED";
$resn_exch{"H4'1 PED"}="1H4* PED"; $resn_exch{"HO'4 PED"}="4HO* PED"; $resn_exch{" H3' PED"}=" H3* PED";
$resn_exch{"H1'1 PED"}="1H1* PED"; $resn_exch{"H1'2 PED"}="2H1* PED"; $resn_exch{"H1'3 PED"}="3H1* PED";
$resn_exch{" H11 PEE"}="1H1  PEE"; $resn_exch{" H12 PEE"}="2H1  PEE"; $resn_exch{" HN1 PEE"}="1HN  PEE";
$resn_exch{" HN2 PEE"}="2HN  PEE"; $resn_exch{" HN3 PEE"}="3HN  PEE"; $resn_exch{" H31 PEE"}="1H3  PEE";
$resn_exch{" H32 PEE"}="2H3  PEE"; $resn_exch{" H41 PEE"}="1H4  PEE"; $resn_exch{" H42 PEE"}="2H4  PEE";
$resn_exch{" H51 PEE"}="1H5  PEE"; $resn_exch{" H52 PEE"}="2H5  PEE"; $resn_exch{"H111 PEE"}="1H11 PEE";
$resn_exch{"H112 PEE"}="2H11 PEE"; $resn_exch{"H121 PEE"}="1H12 PEE"; $resn_exch{"H122 PEE"}="2H12 PEE";
$resn_exch{"H131 PEE"}="1H13 PEE"; $resn_exch{"H132 PEE"}="2H13 PEE"; $resn_exch{"H141 PEE"}="1H14 PEE";
$resn_exch{"H142 PEE"}="2H14 PEE"; $resn_exch{"H151 PEE"}="1H15 PEE"; $resn_exch{"H152 PEE"}="2H15 PEE";
$resn_exch{"H161 PEE"}="1H16 PEE"; $resn_exch{"H162 PEE"}="2H16 PEE"; $resn_exch{"H171 PEE"}="1H17 PEE";
$resn_exch{"H172 PEE"}="2H17 PEE"; $resn_exch{"H181 PEE"}="1H18 PEE"; $resn_exch{"H182 PEE"}="2H18 PEE";
$resn_exch{"H191 PEE"}="1H19 PEE"; $resn_exch{"H192 PEE"}="2H19 PEE"; $resn_exch{"H201 PEE"}="1H20 PEE";
$resn_exch{"H202 PEE"}="2H20 PEE"; $resn_exch{"H211 PEE"}="1H21 PEE"; $resn_exch{"H212 PEE"}="2H21 PEE";
$resn_exch{"H221 PEE"}="1H22 PEE"; $resn_exch{"H222 PEE"}="2H22 PEE"; $resn_exch{"H231 PEE"}="1H23 PEE";
$resn_exch{"H232 PEE"}="2H23 PEE"; $resn_exch{"H241 PEE"}="1H24 PEE"; $resn_exch{"H242 PEE"}="2H24 PEE";
$resn_exch{"H251 PEE"}="1H25 PEE"; $resn_exch{"H252 PEE"}="2H25 PEE"; $resn_exch{"H261 PEE"}="1H26 PEE";
$resn_exch{"H262 PEE"}="2H26 PEE"; $resn_exch{"H271 PEE"}="1H27 PEE"; $resn_exch{"H272 PEE"}="2H27 PEE";
$resn_exch{"H273 PEE"}="3H27 PEE"; $resn_exch{"H311 PEE"}="1H31 PEE"; $resn_exch{"H312 PEE"}="2H31 PEE";
$resn_exch{"H321 PEE"}="1H32 PEE"; $resn_exch{"H322 PEE"}="2H32 PEE"; $resn_exch{"H331 PEE"}="1H33 PEE";
$resn_exch{"H332 PEE"}="2H33 PEE"; $resn_exch{"H341 PEE"}="1H34 PEE"; $resn_exch{"H342 PEE"}="2H34 PEE";
$resn_exch{"H351 PEE"}="1H35 PEE"; $resn_exch{"H352 PEE"}="2H35 PEE"; $resn_exch{"H361 PEE"}="1H36 PEE";
$resn_exch{"H362 PEE"}="2H36 PEE"; $resn_exch{"H371 PEE"}="1H37 PEE"; $resn_exch{"H372 PEE"}="2H37 PEE";
$resn_exch{"H381 PEE"}="1H38 PEE"; $resn_exch{"H382 PEE"}="2H38 PEE"; $resn_exch{"H391 PEE"}="1H39 PEE";
$resn_exch{"H392 PEE"}="2H39 PEE"; $resn_exch{"H401 PEE"}="1H40 PEE"; $resn_exch{"H402 PEE"}="2H40 PEE";
$resn_exch{"H411 PEE"}="1H41 PEE"; $resn_exch{"H412 PEE"}="2H41 PEE"; $resn_exch{"H421 PEE"}="1H42 PEE";
$resn_exch{"H422 PEE"}="2H42 PEE"; $resn_exch{"H431 PEE"}="1H43 PEE"; $resn_exch{"H432 PEE"}="2H43 PEE";
$resn_exch{"H441 PEE"}="1H44 PEE"; $resn_exch{"H442 PEE"}="2H44 PEE"; $resn_exch{"H451 PEE"}="1H45 PEE";
$resn_exch{"H452 PEE"}="2H45 PEE"; $resn_exch{"H461 PEE"}="1H46 PEE"; $resn_exch{"H462 PEE"}="2H46 PEE";
$resn_exch{"H471 PEE"}="1H47 PEE"; $resn_exch{"H472 PEE"}="2H47 PEE"; $resn_exch{"H473 PEE"}="3H47 PEE";
$resn_exch{" H21 PEF"}="1H2  PEF"; $resn_exch{" H11 PEF"}="1H1  PEF"; $resn_exch{" H12 PEF"}="2H1  PEF";
$resn_exch{" HN1 PEF"}="1HN  PEF"; $resn_exch{" HN2 PEF"}="2HN  PEF"; $resn_exch{" H31 PEF"}="1H3  PEF";
$resn_exch{" H32 PEF"}="2H3  PEF"; $resn_exch{" H41 PEF"}="1H4  PEF"; $resn_exch{" H42 PEF"}="2H4  PEF";
$resn_exch{" H51 PEF"}="1H5  PEF"; $resn_exch{" H52 PEF"}="2H5  PEF"; $resn_exch{"H112 PEF"}="2H11 PEF";
$resn_exch{"H111 PEF"}="1H11 PEF"; $resn_exch{"H122 PEF"}="2H12 PEF"; $resn_exch{"H121 PEF"}="1H12 PEF";
$resn_exch{"H132 PEF"}="2H13 PEF"; $resn_exch{"H131 PEF"}="1H13 PEF"; $resn_exch{"H142 PEF"}="2H14 PEF";
$resn_exch{"H141 PEF"}="1H14 PEF"; $resn_exch{"H152 PEF"}="2H15 PEF"; $resn_exch{"H151 PEF"}="1H15 PEF";
$resn_exch{"H162 PEF"}="2H16 PEF"; $resn_exch{"H161 PEF"}="1H16 PEF"; $resn_exch{"H172 PEF"}="2H17 PEF";
$resn_exch{"H171 PEF"}="1H17 PEF"; $resn_exch{"H182 PEF"}="2H18 PEF"; $resn_exch{"H181 PEF"}="1H18 PEF";
$resn_exch{"H192 PEF"}="2H19 PEF"; $resn_exch{"H191 PEF"}="1H19 PEF"; $resn_exch{"H202 PEF"}="2H20 PEF";
$resn_exch{"H201 PEF"}="1H20 PEF"; $resn_exch{"H212 PEF"}="2H21 PEF"; $resn_exch{"H211 PEF"}="1H21 PEF";
$resn_exch{"H222 PEF"}="2H22 PEF"; $resn_exch{"H221 PEF"}="1H22 PEF"; $resn_exch{"H232 PEF"}="2H23 PEF";
$resn_exch{"H231 PEF"}="1H23 PEF"; $resn_exch{"H242 PEF"}="2H24 PEF"; $resn_exch{"H241 PEF"}="1H24 PEF";
$resn_exch{"H252 PEF"}="2H25 PEF"; $resn_exch{"H251 PEF"}="1H25 PEF"; $resn_exch{"H253 PEF"}="3H25 PEF";
$resn_exch{"H312 PEF"}="2H31 PEF"; $resn_exch{"H311 PEF"}="1H31 PEF"; $resn_exch{"H322 PEF"}="2H32 PEF";
$resn_exch{"H321 PEF"}="1H32 PEF"; $resn_exch{"H332 PEF"}="2H33 PEF"; $resn_exch{"H331 PEF"}="1H33 PEF";
$resn_exch{"H342 PEF"}="2H34 PEF"; $resn_exch{"H341 PEF"}="1H34 PEF"; $resn_exch{"H352 PEF"}="2H35 PEF";
$resn_exch{"H351 PEF"}="1H35 PEF"; $resn_exch{"H362 PEF"}="2H36 PEF"; $resn_exch{"H361 PEF"}="1H36 PEF";
$resn_exch{"H372 PEF"}="2H37 PEF"; $resn_exch{"H371 PEF"}="1H37 PEF"; $resn_exch{"H382 PEF"}="2H38 PEF";
$resn_exch{"H381 PEF"}="1H38 PEF"; $resn_exch{"H392 PEF"}="2H39 PEF"; $resn_exch{"H391 PEF"}="1H39 PEF";
$resn_exch{"H402 PEF"}="2H40 PEF"; $resn_exch{"H401 PEF"}="1H40 PEF"; $resn_exch{"H412 PEF"}="2H41 PEF";
$resn_exch{"H411 PEF"}="1H41 PEF"; $resn_exch{"H422 PEF"}="2H42 PEF"; $resn_exch{"H421 PEF"}="1H42 PEF";
$resn_exch{"H432 PEF"}="2H43 PEF"; $resn_exch{"H431 PEF"}="1H43 PEF"; $resn_exch{"H442 PEF"}="2H44 PEF";
$resn_exch{"H441 PEF"}="1H44 PEF"; $resn_exch{"H452 PEF"}="2H45 PEF"; $resn_exch{"H451 PEF"}="1H45 PEF";
$resn_exch{"H453 PEF"}="3H45 PEF"; $resn_exch{"HOP2 PEF"}="2HOP PEF"; $resn_exch{" H11 PEG"}="1H1  PEG";
$resn_exch{" H12 PEG"}="2H1  PEG"; $resn_exch{" H21 PEG"}="1H2  PEG"; $resn_exch{" H22 PEG"}="2H2  PEG";
$resn_exch{" H31 PEG"}="1H3  PEG"; $resn_exch{" H32 PEG"}="2H3  PEG"; $resn_exch{" H41 PEG"}="1H4  PEG";
$resn_exch{" H42 PEG"}="2H4  PEG"; $resn_exch{" HN1 PEH"}="1HN  PEH"; $resn_exch{" HN2 PEH"}="2HN  PEH";
$resn_exch{"H111 PEH"}="1H11 PEH"; $resn_exch{"H112 PEH"}="2H11 PEH"; $resn_exch{"H121 PEH"}="1H12 PEH";
$resn_exch{"H122 PEH"}="2H12 PEH"; $resn_exch{" H11 PEH"}="1H1  PEH"; $resn_exch{" H12 PEH"}="2H1  PEH";
$resn_exch{" H31 PEH"}="1H3  PEH"; $resn_exch{" H32 PEH"}="2H3  PEH"; $resn_exch{"H321 PEH"}="1H32 PEH";
$resn_exch{"H322 PEH"}="2H32 PEH"; $resn_exch{"H331 PEH"}="1H33 PEH"; $resn_exch{"H332 PEH"}="2H33 PEH";
$resn_exch{"H341 PEH"}="1H34 PEH"; $resn_exch{"H342 PEH"}="2H34 PEH"; $resn_exch{"H351 PEH"}="1H35 PEH";
$resn_exch{"H352 PEH"}="2H35 PEH"; $resn_exch{"H361 PEH"}="1H36 PEH"; $resn_exch{"H362 PEH"}="2H36 PEH";
$resn_exch{"H371 PEH"}="1H37 PEH"; $resn_exch{"H372 PEH"}="2H37 PEH"; $resn_exch{"H381 PEH"}="1H38 PEH";
$resn_exch{"H382 PEH"}="2H38 PEH"; $resn_exch{"H391 PEH"}="1H39 PEH"; $resn_exch{"H392 PEH"}="2H39 PEH";
$resn_exch{"H3A1 PEH"}="1H3A PEH"; $resn_exch{"H3A2 PEH"}="2H3A PEH"; $resn_exch{"H3B1 PEH"}="1H3B PEH";
$resn_exch{"H3B2 PEH"}="2H3B PEH"; $resn_exch{"H3C1 PEH"}="1H3C PEH"; $resn_exch{"H3C2 PEH"}="2H3C PEH";
$resn_exch{"H3D1 PEH"}="1H3D PEH"; $resn_exch{"H3D2 PEH"}="2H3D PEH"; $resn_exch{"H3E1 PEH"}="1H3E PEH";
$resn_exch{"H3E2 PEH"}="2H3E PEH"; $resn_exch{"H3F1 PEH"}="1H3F PEH"; $resn_exch{"H3F2 PEH"}="2H3F PEH";
$resn_exch{"H3G1 PEH"}="1H3G PEH"; $resn_exch{"H3G2 PEH"}="2H3G PEH"; $resn_exch{"H3H1 PEH"}="1H3H PEH";
$resn_exch{"H3H2 PEH"}="2H3H PEH"; $resn_exch{"H3I1 PEH"}="1H3I PEH"; $resn_exch{"H3I2 PEH"}="2H3I PEH";
$resn_exch{"H3I3 PEH"}="3H3I PEH"; $resn_exch{"H221 PEH"}="1H22 PEH"; $resn_exch{"H222 PEH"}="2H22 PEH";
$resn_exch{"H231 PEH"}="1H23 PEH"; $resn_exch{"H232 PEH"}="2H23 PEH"; $resn_exch{"H241 PEH"}="1H24 PEH";
$resn_exch{"H242 PEH"}="2H24 PEH"; $resn_exch{"H251 PEH"}="1H25 PEH"; $resn_exch{"H252 PEH"}="2H25 PEH";
$resn_exch{"H261 PEH"}="1H26 PEH"; $resn_exch{"H262 PEH"}="2H26 PEH"; $resn_exch{"H271 PEH"}="1H27 PEH";
$resn_exch{"H272 PEH"}="2H27 PEH"; $resn_exch{"H281 PEH"}="1H28 PEH"; $resn_exch{"H282 PEH"}="2H28 PEH";
$resn_exch{"H291 PEH"}="1H29 PEH"; $resn_exch{"H292 PEH"}="2H29 PEH"; $resn_exch{"H2A1 PEH"}="1H2A PEH";
$resn_exch{"H2A2 PEH"}="2H2A PEH"; $resn_exch{"H2B1 PEH"}="1H2B PEH"; $resn_exch{"H2B2 PEH"}="2H2B PEH";
$resn_exch{"H2C1 PEH"}="1H2C PEH"; $resn_exch{"H2C2 PEH"}="2H2C PEH"; $resn_exch{"H2D1 PEH"}="1H2D PEH";
$resn_exch{"H2D2 PEH"}="2H2D PEH"; $resn_exch{"H2E1 PEH"}="1H2E PEH"; $resn_exch{"H2E2 PEH"}="2H2E PEH";
$resn_exch{"H2F1 PEH"}="1H2F PEH"; $resn_exch{"H2F2 PEH"}="2H2F PEH"; $resn_exch{"H2G1 PEH"}="1H2G PEH";
$resn_exch{"H2G2 PEH"}="2H2G PEH"; $resn_exch{"H2H1 PEH"}="1H2H PEH"; $resn_exch{"H2H2 PEH"}="2H2H PEH";
$resn_exch{"H2I1 PEH"}="1H2I PEH"; $resn_exch{"H2I2 PEH"}="2H2I PEH"; $resn_exch{"H2I3 PEH"}="3H2I PEH";
$resn_exch{" H11 PEI"}="1H1  PEI"; $resn_exch{" H12 PEI"}="2H1  PEI"; $resn_exch{" H13 PEI"}="3H1  PEI";
$resn_exch{" H21 PEI"}="1H2  PEI"; $resn_exch{" H22 PEI"}="2H2  PEI"; $resn_exch{" H31 PEI"}="1H3  PEI";
$resn_exch{" H32 PEI"}="2H3  PEI"; $resn_exch{" H41 PEI"}="1H4  PEI"; $resn_exch{" H42 PEI"}="2H4  PEI";
$resn_exch{" HN1 PEK"}="1HN  PEK"; $resn_exch{" HN2 PEK"}="2HN  PEK"; $resn_exch{"H011 PEK"}="1H01 PEK";
$resn_exch{"H012 PEK"}="2H01 PEK"; $resn_exch{"H031 PEK"}="1H03 PEK"; $resn_exch{"H032 PEK"}="2H03 PEK";
$resn_exch{"H041 PEK"}="1H04 PEK"; $resn_exch{"H042 PEK"}="2H04 PEK"; $resn_exch{"H051 PEK"}="1H05 PEK";
$resn_exch{"H052 PEK"}="2H05 PEK"; $resn_exch{" H21 PEK"}="1H2  PEK"; $resn_exch{" H22 PEK"}="2H2  PEK";
$resn_exch{" H31 PEK"}="1H3  PEK"; $resn_exch{" H32 PEK"}="2H3  PEK"; $resn_exch{" H41 PEK"}="1H4  PEK";
$resn_exch{" H42 PEK"}="2H4  PEK"; $resn_exch{" H71 PEK"}="1H7  PEK"; $resn_exch{" H72 PEK"}="2H7  PEK";
$resn_exch{"H101 PEK"}="1H10 PEK"; $resn_exch{"H102 PEK"}="2H10 PEK"; $resn_exch{"H131 PEK"}="1H13 PEK";
$resn_exch{"H132 PEK"}="2H13 PEK"; $resn_exch{"H161 PEK"}="1H16 PEK"; $resn_exch{"H162 PEK"}="2H16 PEK";
$resn_exch{"H171 PEK"}="1H17 PEK"; $resn_exch{"H172 PEK"}="2H17 PEK"; $resn_exch{"H181 PEK"}="1H18 PEK";
$resn_exch{"H182 PEK"}="2H18 PEK"; $resn_exch{"H191 PEK"}="1H19 PEK"; $resn_exch{"H192 PEK"}="2H19 PEK";
$resn_exch{"H201 PEK"}="1H20 PEK"; $resn_exch{"H202 PEK"}="2H20 PEK"; $resn_exch{"H203 PEK"}="3H20 PEK";
$resn_exch{"H221 PEK"}="1H22 PEK"; $resn_exch{"H222 PEK"}="2H22 PEK"; $resn_exch{"H231 PEK"}="1H23 PEK";
$resn_exch{"H232 PEK"}="2H23 PEK"; $resn_exch{"H241 PEK"}="1H24 PEK"; $resn_exch{"H242 PEK"}="2H24 PEK";
$resn_exch{"H251 PEK"}="1H25 PEK"; $resn_exch{"H252 PEK"}="2H25 PEK"; $resn_exch{"H261 PEK"}="1H26 PEK";
$resn_exch{"H262 PEK"}="2H26 PEK"; $resn_exch{"H271 PEK"}="1H27 PEK"; $resn_exch{"H272 PEK"}="2H27 PEK";
$resn_exch{"H281 PEK"}="1H28 PEK"; $resn_exch{"H282 PEK"}="2H28 PEK"; $resn_exch{"H291 PEK"}="1H29 PEK";
$resn_exch{"H292 PEK"}="2H29 PEK"; $resn_exch{"H301 PEK"}="1H30 PEK"; $resn_exch{"H302 PEK"}="2H30 PEK";
$resn_exch{"H311 PEK"}="1H31 PEK"; $resn_exch{"H312 PEK"}="2H31 PEK"; $resn_exch{"H321 PEK"}="1H32 PEK";
$resn_exch{"H322 PEK"}="2H32 PEK"; $resn_exch{"H331 PEK"}="1H33 PEK"; $resn_exch{"H332 PEK"}="2H33 PEK";
$resn_exch{"H341 PEK"}="1H34 PEK"; $resn_exch{"H342 PEK"}="2H34 PEK"; $resn_exch{"H351 PEK"}="1H35 PEK";
$resn_exch{"H352 PEK"}="2H35 PEK"; $resn_exch{"H361 PEK"}="1H36 PEK"; $resn_exch{"H362 PEK"}="2H36 PEK";
$resn_exch{"H371 PEK"}="1H37 PEK"; $resn_exch{"H372 PEK"}="2H37 PEK"; $resn_exch{"H381 PEK"}="1H38 PEK";
$resn_exch{"H382 PEK"}="2H38 PEK"; $resn_exch{"H383 PEK"}="3H38 PEK"; $resn_exch{" H21 PEM"}="1H2  PEM";
$resn_exch{" H31 PEM"}="1H3  PEM"; $resn_exch{" H51 PEM"}="1H5  PEM"; $resn_exch{"HC81 PEM"}="1HC8 PEM";
$resn_exch{"HC82 PEM"}="2HC8 PEM"; $resn_exch{"HC91 PEM"}="1HC9 PEM"; $resn_exch{"HC92 PEM"}="2HC9 PEM";
$resn_exch{"H171 PEM"}="1H17 PEM"; $resn_exch{"H172 PEM"}="2H17 PEM"; $resn_exch{"H173 PEM"}="3H17 PEM";
$resn_exch{"H181 PEM"}="1H18 PEM"; $resn_exch{"H182 PEM"}="2H18 PEM"; $resn_exch{"H183 PEM"}="3H18 PEM";
$resn_exch{" H31 PEP"}="1H3  PEP"; $resn_exch{" H32 PEP"}="2H3  PEP"; $resn_exch{"HOP2 PEP"}="2HOP PEP";
$resn_exch{"HOP3 PEP"}="3HOP PEP"; $resn_exch{" H21 PEQ"}="1H2  PEQ"; $resn_exch{" H31 PEQ"}="1H3  PEQ";
$resn_exch{" H32 PEQ"}="2H3  PEQ"; $resn_exch{" H33 PEQ"}="3H3  PEQ"; $resn_exch{"HOP2 PEQ"}="2HOP PEQ";
$resn_exch{"HOP3 PEQ"}="3HOP PEQ"; $resn_exch{" HA1 PET"}="1HA  PET"; $resn_exch{" HA2 PET"}="2HA  PET";
$resn_exch{" HB1 PET"}="1HB  PET"; $resn_exch{" HB2 PET"}="2HB  PET"; $resn_exch{"HA'1 PET"}="1HA' PET";
$resn_exch{"HA'2 PET"}="2HA' PET"; $resn_exch{"HB'1 PET"}="1HB' PET"; $resn_exch{"HB'2 PET"}="2HB' PET";
$resn_exch{"HN21 PET"}="1HN2 PET"; $resn_exch{"HN22 PET"}="2HN2 PET"; $resn_exch{"HN1' PET"}="'HN1 PET";
$resn_exch{"HN'1 PET"}="1HN' PET"; $resn_exch{"HN'2 PET"}="2HN' PET"; $resn_exch{"HAA1 PEU"}="1HAA PEU";
$resn_exch{"HAA2 PEU"}="2HAA PEU"; $resn_exch{"HAA3 PEU"}="3HAA PEU"; $resn_exch{"HAC1 PEU"}="1HAC PEU";
$resn_exch{"HAC2 PEU"}="2HAC PEU"; $resn_exch{"HAD1 PEU"}="1HAD PEU"; $resn_exch{"HAD2 PEU"}="2HAD PEU";
$resn_exch{"HAF1 PEU"}="1HAF PEU"; $resn_exch{"HAF2 PEU"}="2HAF PEU"; $resn_exch{"HAG1 PEU"}="1HAG PEU";
$resn_exch{"HAG2 PEU"}="2HAG PEU"; $resn_exch{"HAI1 PEU"}="1HAI PEU"; $resn_exch{"HAI2 PEU"}="2HAI PEU";
$resn_exch{"HAJ1 PEU"}="1HAJ PEU"; $resn_exch{"HAJ2 PEU"}="2HAJ PEU"; $resn_exch{"HAL1 PEU"}="1HAL PEU";
$resn_exch{"HAL2 PEU"}="2HAL PEU"; $resn_exch{"HAM1 PEU"}="1HAM PEU"; $resn_exch{"HAM2 PEU"}="2HAM PEU";
$resn_exch{"HAO1 PEU"}="1HAO PEU"; $resn_exch{"HAO2 PEU"}="2HAO PEU"; $resn_exch{"HAP1 PEU"}="1HAP PEU";
$resn_exch{"HAP2 PEU"}="2HAP PEU"; $resn_exch{"HAR1 PEU"}="1HAR PEU"; $resn_exch{"HAR2 PEU"}="2HAR PEU";
$resn_exch{"HAS1 PEU"}="1HAS PEU"; $resn_exch{"HAS2 PEU"}="2HAS PEU"; $resn_exch{"HAU1 PEU"}="1HAU PEU";
$resn_exch{"HAU2 PEU"}="2HAU PEU"; $resn_exch{"HAV1 PEU"}="1HAV PEU"; $resn_exch{"HAV2 PEU"}="2HAV PEU";
$resn_exch{"HAX1 PEU"}="1HAX PEU"; $resn_exch{"HAX2 PEU"}="2HAX PEU"; $resn_exch{"HAY1 PEU"}="1HAY PEU";
$resn_exch{"HAY2 PEU"}="2HAY PEU"; $resn_exch{"HBA1 PEU"}="1HBA PEU"; $resn_exch{"HBA2 PEU"}="2HBA PEU";
$resn_exch{"HBB1 PEU"}="1HBB PEU"; $resn_exch{"HBB2 PEU"}="2HBB PEU"; $resn_exch{"HCG1 PEU"}="1HCG PEU";
$resn_exch{"HCG2 PEU"}="2HCG PEU"; $resn_exch{"HCF1 PEU"}="1HCF PEU"; $resn_exch{"HCF2 PEU"}="2HCF PEU";
$resn_exch{"HCD1 PEU"}="1HCD PEU"; $resn_exch{"HCD2 PEU"}="2HCD PEU"; $resn_exch{"HCC1 PEU"}="1HCC PEU";
$resn_exch{"HCC2 PEU"}="2HCC PEU"; $resn_exch{"HCA1 PEU"}="1HCA PEU"; $resn_exch{"HCA2 PEU"}="2HCA PEU";
$resn_exch{"HBZ1 PEU"}="1HBZ PEU"; $resn_exch{"HBZ2 PEU"}="2HBZ PEU"; $resn_exch{"HBX1 PEU"}="1HBX PEU";
$resn_exch{"HBX2 PEU"}="2HBX PEU"; $resn_exch{"HBW1 PEU"}="1HBW PEU"; $resn_exch{"HBW2 PEU"}="2HBW PEU";
$resn_exch{"HBU1 PEU"}="1HBU PEU"; $resn_exch{"HBU2 PEU"}="2HBU PEU"; $resn_exch{"HBT1 PEU"}="1HBT PEU";
$resn_exch{"HBT2 PEU"}="2HBT PEU"; $resn_exch{"HBR1 PEU"}="1HBR PEU"; $resn_exch{"HBR2 PEU"}="2HBR PEU";
$resn_exch{"HBQ1 PEU"}="1HBQ PEU"; $resn_exch{"HBQ2 PEU"}="2HBQ PEU"; $resn_exch{"HBO1 PEU"}="1HBO PEU";
$resn_exch{"HBO2 PEU"}="2HBO PEU"; $resn_exch{"HBN1 PEU"}="1HBN PEU"; $resn_exch{"HBN2 PEU"}="2HBN PEU";
$resn_exch{"HBL1 PEU"}="1HBL PEU"; $resn_exch{"HBL2 PEU"}="2HBL PEU"; $resn_exch{"HBK1 PEU"}="1HBK PEU";
$resn_exch{"HBK2 PEU"}="2HBK PEU"; $resn_exch{"HBI1 PEU"}="1HBI PEU"; $resn_exch{"HBI2 PEU"}="2HBI PEU";
$resn_exch{"HBH1 PEU"}="1HBH PEU"; $resn_exch{"HBH2 PEU"}="2HBH PEU"; $resn_exch{"HBF1 PEU"}="1HBF PEU";
$resn_exch{"HBF2 PEU"}="2HBF PEU"; $resn_exch{"HBE1 PEU"}="1HBE PEU"; $resn_exch{"HBE2 PEU"}="2HBE PEU";
$resn_exch{"HCH1 PEU"}="1HCH PEU"; $resn_exch{"HCH2 PEU"}="2HCH PEU"; $resn_exch{"HCI1 PEU"}="1HCI PEU";
$resn_exch{"HCI2 PEU"}="2HCI PEU"; $resn_exch{"HCK1 PEU"}="1HCK PEU"; $resn_exch{"HCK2 PEU"}="2HCK PEU";
$resn_exch{"HCL1 PEU"}="1HCL PEU"; $resn_exch{"HCL2 PEU"}="2HCL PEU"; $resn_exch{"HCN1 PEU"}="1HCN PEU";
$resn_exch{"HCN2 PEU"}="2HCN PEU"; $resn_exch{"HCO1 PEU"}="1HCO PEU"; $resn_exch{"HCO2 PEU"}="2HCO PEU";
$resn_exch{"HCQ1 PEU"}="1HCQ PEU"; $resn_exch{"HCQ2 PEU"}="2HCQ PEU"; $resn_exch{"HCR1 PEU"}="1HCR PEU";
$resn_exch{"HCR2 PEU"}="2HCR PEU"; $resn_exch{"HCT1 PEU"}="1HCT PEU"; $resn_exch{"HCT2 PEU"}="2HCT PEU";
$resn_exch{"HCU1 PEU"}="1HCU PEU"; $resn_exch{"HCU2 PEU"}="2HCU PEU"; $resn_exch{"HCW1 PEU"}="1HCW PEU";
$resn_exch{"HCW2 PEU"}="2HCW PEU"; $resn_exch{"HCX1 PEU"}="1HCX PEU"; $resn_exch{"HCX2 PEU"}="2HCX PEU";
$resn_exch{"HCZ1 PEU"}="1HCZ PEU"; $resn_exch{"HCZ2 PEU"}="2HCZ PEU"; $resn_exch{"HDA1 PEU"}="1HDA PEU";
$resn_exch{"HDA2 PEU"}="2HDA PEU"; $resn_exch{"HDC1 PEU"}="1HDC PEU"; $resn_exch{"HDC2 PEU"}="2HDC PEU";
$resn_exch{"HDD1 PEU"}="1HDD PEU"; $resn_exch{"HDD2 PEU"}="2HDD PEU"; $resn_exch{"H481 PEV"}="1H48 PEV";
$resn_exch{"H482 PEV"}="2H48 PEV"; $resn_exch{"H483 PEV"}="3H48 PEV"; $resn_exch{"H471 PEV"}="1H47 PEV";
$resn_exch{"H472 PEV"}="2H47 PEV"; $resn_exch{"H461 PEV"}="1H46 PEV"; $resn_exch{"H462 PEV"}="2H46 PEV";
$resn_exch{"H451 PEV"}="1H45 PEV"; $resn_exch{"H452 PEV"}="2H45 PEV"; $resn_exch{"H441 PEV"}="1H44 PEV";
$resn_exch{"H442 PEV"}="2H44 PEV"; $resn_exch{"H431 PEV"}="1H43 PEV"; $resn_exch{"H432 PEV"}="2H43 PEV";
$resn_exch{"H421 PEV"}="1H42 PEV"; $resn_exch{"H422 PEV"}="2H42 PEV"; $resn_exch{"H411 PEV"}="1H41 PEV";
$resn_exch{"H412 PEV"}="2H41 PEV"; $resn_exch{"H401 PEV"}="1H40 PEV"; $resn_exch{"H402 PEV"}="2H40 PEV";
$resn_exch{"H391 PEV"}="1H39 PEV"; $resn_exch{"H392 PEV"}="2H39 PEV"; $resn_exch{"H381 PEV"}="1H38 PEV";
$resn_exch{"H382 PEV"}="2H38 PEV"; $resn_exch{"H371 PEV"}="1H37 PEV"; $resn_exch{"H372 PEV"}="2H37 PEV";
$resn_exch{"H361 PEV"}="1H36 PEV"; $resn_exch{"H362 PEV"}="2H36 PEV"; $resn_exch{"H351 PEV"}="1H35 PEV";
$resn_exch{"H352 PEV"}="2H35 PEV"; $resn_exch{"H341 PEV"}="1H34 PEV"; $resn_exch{"H342 PEV"}="2H34 PEV";
$resn_exch{"H331 PEV"}="1H33 PEV"; $resn_exch{"H332 PEV"}="2H33 PEV"; $resn_exch{"H321 PEV"}="1H32 PEV";
$resn_exch{"H322 PEV"}="2H32 PEV"; $resn_exch{" H11 PEV"}="1H1  PEV"; $resn_exch{" H12 PEV"}="2H1  PEV";
$resn_exch{" H41 PEV"}="1H4  PEV"; $resn_exch{" H42 PEV"}="2H4  PEV"; $resn_exch{" H51 PEV"}="1H5  PEV";
$resn_exch{" H52 PEV"}="2H5  PEV"; $resn_exch{"HN61 PEV"}="1HN6 PEV"; $resn_exch{"HN62 PEV"}="2HN6 PEV";
$resn_exch{" H31 PEV"}="1H3  PEV"; $resn_exch{" H32 PEV"}="2H3  PEV"; $resn_exch{"H121 PEV"}="1H12 PEV";
$resn_exch{"H122 PEV"}="2H12 PEV"; $resn_exch{"H131 PEV"}="1H13 PEV"; $resn_exch{"H132 PEV"}="2H13 PEV";
$resn_exch{"H141 PEV"}="1H14 PEV"; $resn_exch{"H142 PEV"}="2H14 PEV"; $resn_exch{"H151 PEV"}="1H15 PEV";
$resn_exch{"H152 PEV"}="2H15 PEV"; $resn_exch{"H161 PEV"}="1H16 PEV"; $resn_exch{"H162 PEV"}="2H16 PEV";
$resn_exch{"H171 PEV"}="1H17 PEV"; $resn_exch{"H172 PEV"}="2H17 PEV"; $resn_exch{"H181 PEV"}="1H18 PEV";
$resn_exch{"H182 PEV"}="2H18 PEV"; $resn_exch{"H191 PEV"}="1H19 PEV"; $resn_exch{"H192 PEV"}="2H19 PEV";
$resn_exch{"H201 PEV"}="1H20 PEV"; $resn_exch{"H202 PEV"}="2H20 PEV"; $resn_exch{"H211 PEV"}="1H21 PEV";
$resn_exch{"H212 PEV"}="2H21 PEV"; $resn_exch{"H221 PEV"}="1H22 PEV"; $resn_exch{"H222 PEV"}="2H22 PEV";
$resn_exch{"H231 PEV"}="1H23 PEV"; $resn_exch{"H232 PEV"}="2H23 PEV"; $resn_exch{"H241 PEV"}="1H24 PEV";
$resn_exch{"H242 PEV"}="2H24 PEV"; $resn_exch{"H251 PEV"}="1H25 PEV"; $resn_exch{"H252 PEV"}="2H25 PEV";
$resn_exch{"H261 PEV"}="1H26 PEV"; $resn_exch{"H262 PEV"}="2H26 PEV"; $resn_exch{"H263 PEV"}="3H26 PEV";
$resn_exch{"H481 PEW"}="1H48 PEW"; $resn_exch{"H482 PEW"}="2H48 PEW"; $resn_exch{"H483 PEW"}="3H48 PEW";
$resn_exch{"H471 PEW"}="1H47 PEW"; $resn_exch{"H472 PEW"}="2H47 PEW"; $resn_exch{"H461 PEW"}="1H46 PEW";
$resn_exch{"H462 PEW"}="2H46 PEW"; $resn_exch{"H451 PEW"}="1H45 PEW"; $resn_exch{"H452 PEW"}="2H45 PEW";
$resn_exch{"H441 PEW"}="1H44 PEW"; $resn_exch{"H442 PEW"}="2H44 PEW"; $resn_exch{"H431 PEW"}="1H43 PEW";
$resn_exch{"H432 PEW"}="2H43 PEW"; $resn_exch{"H421 PEW"}="1H42 PEW"; $resn_exch{"H422 PEW"}="2H42 PEW";
$resn_exch{"H411 PEW"}="1H41 PEW"; $resn_exch{"H412 PEW"}="2H41 PEW"; $resn_exch{"H381 PEW"}="1H38 PEW";
$resn_exch{"H382 PEW"}="2H38 PEW"; $resn_exch{"H371 PEW"}="1H37 PEW"; $resn_exch{"H372 PEW"}="2H37 PEW";
$resn_exch{"H361 PEW"}="1H36 PEW"; $resn_exch{"H362 PEW"}="2H36 PEW"; $resn_exch{"H351 PEW"}="1H35 PEW";
$resn_exch{"H352 PEW"}="2H35 PEW"; $resn_exch{"H341 PEW"}="1H34 PEW"; $resn_exch{"H342 PEW"}="2H34 PEW";
$resn_exch{"H331 PEW"}="1H33 PEW"; $resn_exch{"H332 PEW"}="2H33 PEW"; $resn_exch{"H321 PEW"}="1H32 PEW";
$resn_exch{"H322 PEW"}="2H32 PEW"; $resn_exch{" H11 PEW"}="1H1  PEW"; $resn_exch{" H12 PEW"}="2H1  PEW";
$resn_exch{" H41 PEW"}="1H4  PEW"; $resn_exch{" H42 PEW"}="2H4  PEW"; $resn_exch{" H51 PEW"}="1H5  PEW";
$resn_exch{" H52 PEW"}="2H5  PEW"; $resn_exch{"HN61 PEW"}="1HN6 PEW"; $resn_exch{"HN62 PEW"}="2HN6 PEW";
$resn_exch{" H31 PEW"}="1H3  PEW"; $resn_exch{" H32 PEW"}="2H3  PEW"; $resn_exch{"H121 PEW"}="1H12 PEW";
$resn_exch{"H122 PEW"}="2H12 PEW"; $resn_exch{"H131 PEW"}="1H13 PEW"; $resn_exch{"H132 PEW"}="2H13 PEW";
$resn_exch{"H141 PEW"}="1H14 PEW"; $resn_exch{"H142 PEW"}="2H14 PEW"; $resn_exch{"H151 PEW"}="1H15 PEW";
$resn_exch{"H152 PEW"}="2H15 PEW"; $resn_exch{"H161 PEW"}="1H16 PEW"; $resn_exch{"H162 PEW"}="2H16 PEW";
$resn_exch{"H171 PEW"}="1H17 PEW"; $resn_exch{"H172 PEW"}="2H17 PEW"; $resn_exch{"H181 PEW"}="1H18 PEW";
$resn_exch{"H182 PEW"}="2H18 PEW"; $resn_exch{"H191 PEW"}="1H19 PEW"; $resn_exch{"H192 PEW"}="2H19 PEW";
$resn_exch{"H201 PEW"}="1H20 PEW"; $resn_exch{"H202 PEW"}="2H20 PEW"; $resn_exch{"H211 PEW"}="1H21 PEW";
$resn_exch{"H212 PEW"}="2H21 PEW"; $resn_exch{"H221 PEW"}="1H22 PEW"; $resn_exch{"H222 PEW"}="2H22 PEW";
$resn_exch{"H231 PEW"}="1H23 PEW"; $resn_exch{"H232 PEW"}="2H23 PEW"; $resn_exch{"H241 PEW"}="1H24 PEW";
$resn_exch{"H242 PEW"}="2H24 PEW"; $resn_exch{"H251 PEW"}="1H25 PEW"; $resn_exch{"H252 PEW"}="2H25 PEW";
$resn_exch{"H261 PEW"}="1H26 PEW"; $resn_exch{"H262 PEW"}="2H26 PEW"; $resn_exch{"H263 PEW"}="3H26 PEW";
$resn_exch{" H31 PEZ"}="1H3  PEZ"; $resn_exch{" H32 PEZ"}="2H3  PEZ"; $resn_exch{" H41 PEZ"}="1H4  PEZ";
$resn_exch{" H42 PEZ"}="2H4  PEZ"; $resn_exch{" H43 PEZ"}="3H4  PEZ"; $resn_exch{"H151  PF"}="1H15  PF";
$resn_exch{"H152  PF"}="2H15  PF"; $resn_exch{"H161  PF"}="1H16  PF"; $resn_exch{"H162  PF"}="2H16  PF";
$resn_exch{"HAG1 PF1"}="1HAG PF1"; $resn_exch{"HAG2 PF1"}="2HAG PF1"; $resn_exch{"HAH1 PF1"}="1HAH PF1";
$resn_exch{"HAH2 PF1"}="2HAH PF1"; $resn_exch{"HAI1 PF1"}="1HAI PF1"; $resn_exch{"HAI2 PF1"}="2HAI PF1";
$resn_exch{"HAA1 PF1"}="1HAA PF1"; $resn_exch{"HAA2 PF1"}="2HAA PF1"; $resn_exch{"HAA3 PF1"}="3HAA PF1";
$resn_exch{"HAB1 PF1"}="1HAB PF1"; $resn_exch{"HAB2 PF1"}="2HAB PF1"; $resn_exch{"H181 PF3"}="1H18 PF3";
$resn_exch{"H182 PF3"}="2H18 PF3"; $resn_exch{" H   PF5"}="1HN  PF5"; $resn_exch{" H2  PF5"}="2HN  PF5";
$resn_exch{" HB2 PF5"}="1HB  PF5"; $resn_exch{" HB3 PF5"}="2HB  PF5"; $resn_exch{" HXT PF5"}="HOXT PF5";
$resn_exch{"H17C PFA"}="CH17 PFA"; $resn_exch{"HC71 PFA"}="1HC7 PFA"; $resn_exch{"HC72 PFA"}="2HC7 PFA";
$resn_exch{"HC73 PFA"}="3HC7 PFA"; $resn_exch{"H191 PFA"}="1H19 PFA"; $resn_exch{"H192 PFA"}="2H19 PFA";
$resn_exch{"H193 PFA"}="3H19 PFA"; $resn_exch{"H31C PFA"}="CH31 PFA"; $resn_exch{"H22C PFA"}="CH22 PFA";
$resn_exch{"H25C PFA"}="CH25 PFA"; $resn_exch{"HC81 PFA"}="1HC8 PFA"; $resn_exch{"HC82 PFA"}="2HC8 PFA";
$resn_exch{"HC83 PFA"}="3HC8 PFA"; $resn_exch{"HC91 PFA"}="1HC9 PFA"; $resn_exch{"HC92 PFA"}="2HC9 PFA";
$resn_exch{"HC93 PFA"}="3HC9 PFA"; $resn_exch{"H29C PFA"}="CH29 PFA"; $resn_exch{"H27O PFA"}="OH27 PFA";
$resn_exch{" H71 PFB"}="1H7  PFB"; $resn_exch{" H72 PFB"}="2H7  PFB"; $resn_exch{"H142 PFC"}="2H14 PFC";
$resn_exch{"H141 PFC"}="1H14 PFC"; $resn_exch{"H172 PFC"}="2H17 PFC"; $resn_exch{"H171 PFC"}="1H17 PFC";
$resn_exch{"H182 PFC"}="2H18 PFC"; $resn_exch{"H181 PFC"}="1H18 PFC"; $resn_exch{"H212 PFC"}="2H21 PFC";
$resn_exch{"H211 PFC"}="1H21 PFC"; $resn_exch{"H242 PFC"}="2H24 PFC"; $resn_exch{"H241 PFC"}="1H24 PFC";
$resn_exch{"H252 PFC"}="2H25 PFC"; $resn_exch{"H251 PFC"}="1H25 PFC"; $resn_exch{"H272 PFC"}="2H27 PFC";
$resn_exch{"H271 PFC"}="1H27 PFC"; $resn_exch{"H292 PFC"}="2H29 PFC"; $resn_exch{"H291 PFC"}="1H29 PFC";
$resn_exch{"H302 PFC"}="2H30 PFC"; $resn_exch{"H301 PFC"}="1H30 PFC"; $resn_exch{"H322 PFC"}="2H32 PFC";
$resn_exch{"H321 PFC"}="1H32 PFC"; $resn_exch{"H342 PFC"}="2H34 PFC"; $resn_exch{"H341 PFC"}="1H34 PFC";
$resn_exch{"H353 PFC"}="3H35 PFC"; $resn_exch{"H352 PFC"}="2H35 PFC"; $resn_exch{"H351 PFC"}="1H35 PFC";
$resn_exch{"H363 PFC"}="3H36 PFC"; $resn_exch{"H362 PFC"}="2H36 PFC"; $resn_exch{"H361 PFC"}="1H36 PFC";
$resn_exch{"H373 PFC"}="3H37 PFC"; $resn_exch{"H372 PFC"}="2H37 PFC"; $resn_exch{"H371 PFC"}="1H37 PFC";
$resn_exch{"H382 PFC"}="2H38 PFC"; $resn_exch{"H381 PFC"}="1H38 PFC"; $resn_exch{" H51 PFD"}="1H5  PFD";
$resn_exch{" H52 PFD"}="2H5  PFD"; $resn_exch{"H111 PFD"}="1H11 PFD"; $resn_exch{"H112 PFD"}="2H11 PFD";
$resn_exch{"H191 PFD"}="1H19 PFD"; $resn_exch{"H192 PFD"}="2H19 PFD"; $resn_exch{"H211 PFD"}="1H21 PFD";
$resn_exch{"H212 PFD"}="2H21 PFD"; $resn_exch{"H213 PFD"}="3H21 PFD"; $resn_exch{"H471 PFE"}="1H47 PFE";
$resn_exch{"H472 PFE"}="2H47 PFE"; $resn_exch{"H431 PFE"}="1H43 PFE"; $resn_exch{"H351 PFE"}="1H35 PFE";
$resn_exch{"H331 PFE"}="1H33 PFE"; $resn_exch{"H371 PFE"}="1H37 PFE"; $resn_exch{"H341 PFE"}="1H34 PFE";
$resn_exch{" H31 PFE"}="1H3  PFE"; $resn_exch{" H71 PFE"}="1H7  PFE"; $resn_exch{" H91 PFE"}="1H9  PFE";
$resn_exch{"H201 PFE"}="1H20 PFE"; $resn_exch{"H202 PFE"}="2H20 PFE"; $resn_exch{"H211 PFE"}="1H21 PFE";
$resn_exch{"H212 PFE"}="2H21 PFE"; $resn_exch{"H213 PFE"}="3H21 PFE"; $resn_exch{"H181 PFE"}="1H18 PFE";
$resn_exch{"H191 PFE"}="1H19 PFE"; $resn_exch{"H192 PFE"}="2H19 PFE"; $resn_exch{"H261 PFE"}="1H26 PFE";
$resn_exch{"H262 PFE"}="2H26 PFE"; $resn_exch{"H263 PFE"}="3H26 PFE"; $resn_exch{" H   PFF"}="1HN  PFF";
$resn_exch{" H2  PFF"}="2HN  PFF"; $resn_exch{" HB2 PFF"}="1HBC PFF"; $resn_exch{" HB3 PFF"}="2HBC PFF";
$resn_exch{"HN21 PFG"}="1HN2 PFG"; $resn_exch{"HN22 PFG"}="2HN2 PFG"; $resn_exch{" H91 PFG"}="1H9  PFG";
$resn_exch{" H92 PFG"}="2H9  PFG"; $resn_exch{"HP11 PFG"}="1HP1 PFG"; $resn_exch{"HP12 PFG"}="2HP1 PFG";
$resn_exch{"HNP1 PFG"}="1HNP PFG"; $resn_exch{"HB11 PFG"}="1HB1 PFG"; $resn_exch{"HB12 PFG"}="2HB1 PFG";
$resn_exch{"HG11 PFG"}="1HG1 PFG"; $resn_exch{"HG12 PFG"}="2HG1 PFG"; $resn_exch{"HO21 PFG"}="1HO2 PFG";
$resn_exch{"HNP2 PFG"}="2HNP PFG"; $resn_exch{"HB21 PFG"}="1HB2 PFG"; $resn_exch{"HB22 PFG"}="2HB2 PFG";
$resn_exch{"HG21 PFG"}="1HG2 PFG"; $resn_exch{"HG22 PFG"}="2HG2 PFG"; $resn_exch{"HO22 PFG"}="2HO2 PFG";
$resn_exch{"HNP3 PFG"}="3HNP PFG"; $resn_exch{"HB31 PFG"}="1HB3 PFG"; $resn_exch{"HB32 PFG"}="2HB3 PFG";
$resn_exch{"HG31 PFG"}="1HG3 PFG"; $resn_exch{"HG32 PFG"}="2HG3 PFG"; $resn_exch{"HO23 PFG"}="3HO2 PFG";
$resn_exch{"HNP4 PFG"}="4HNP PFG"; $resn_exch{"HB41 PFG"}="1HB4 PFG"; $resn_exch{"HB42 PFG"}="2HB4 PFG";
$resn_exch{"HG41 PFG"}="1HG4 PFG"; $resn_exch{"HG42 PFG"}="2HG4 PFG"; $resn_exch{"HO24 PFG"}="4HO2 PFG";
$resn_exch{"H181 PFH"}="1H18 PFH"; $resn_exch{"H182 PFH"}="2H18 PFH"; $resn_exch{"H161 PFH"}="1H16 PFH";
$resn_exch{"H162 PFH"}="2H16 PFH"; $resn_exch{"H191 PFH"}="1H19 PFH"; $resn_exch{"H192 PFH"}="2H19 PFH";
$resn_exch{"H141 PFH"}="1H14 PFH"; $resn_exch{"H142 PFH"}="2H14 PFH"; $resn_exch{"H143 PFH"}="3H14 PFH";
$resn_exch{"H121 PFH"}="1H12 PFH"; $resn_exch{"H122 PFH"}="2H12 PFH"; $resn_exch{"H123 PFH"}="3H12 PFH";
$resn_exch{"H21A PFI"}="1H2  PFI"; $resn_exch{" H22 PFI"}="2H2  PFI"; $resn_exch{" H23 PFI"}="3H2  PFI";
$resn_exch{" H11 PFI"}="1H1  PFI"; $resn_exch{" H12 PFI"}="2H1  PFI"; $resn_exch{" H13 PFI"}="3H1  PFI";
$resn_exch{"H121 PFI"}="1H12 PFI"; $resn_exch{"H122 PFI"}="2H12 PFI"; $resn_exch{"H111 PFI"}="1H11 PFI";
$resn_exch{"H112 PFI"}="2H11 PFI"; $resn_exch{" H61 PFI"}="1H6  PFI"; $resn_exch{" H62 PFI"}="2H6  PFI";
$resn_exch{"H131 PFI"}="1H13 PFI"; $resn_exch{"H132 PFI"}="2H13 PFI"; $resn_exch{"H161 PFI"}="1H16 PFI";
$resn_exch{"H162 PFI"}="2H16 PFI"; $resn_exch{"H151 PFI"}="1H15 PFI"; $resn_exch{"H152 PFI"}="2H15 PFI";
$resn_exch{"H141 PFI"}="1H14 PFI"; $resn_exch{"H142 PFI"}="2H14 PFI"; $resn_exch{"H181 PFJ"}="1H18 PFJ";
$resn_exch{"H182 PFJ"}="2H18 PFJ"; $resn_exch{"H161 PFJ"}="1H16 PFJ"; $resn_exch{"H162 PFJ"}="2H16 PFJ";
$resn_exch{"H191 PFJ"}="1H19 PFJ"; $resn_exch{"H192 PFJ"}="2H19 PFJ"; $resn_exch{"H141 PFJ"}="1H14 PFJ";
$resn_exch{"H142 PFJ"}="2H14 PFJ"; $resn_exch{"H143 PFJ"}="3H14 PFJ"; $resn_exch{"H121 PFJ"}="1H12 PFJ";
$resn_exch{"H122 PFJ"}="2H12 PFJ"; $resn_exch{"H123 PFJ"}="3H12 PFJ"; $resn_exch{" H81 PFL"}="1H8  PFL";
$resn_exch{" H82 PFL"}="2H8  PFL"; $resn_exch{" H83 PFL"}="3H8  PFL"; $resn_exch{" H91 PFL"}="1H9  PFL";
$resn_exch{" H92 PFL"}="2H9  PFL"; $resn_exch{" H93 PFL"}="3H9  PFL"; $resn_exch{"H111 PFL"}="1H11 PFL";
$resn_exch{"H112 PFL"}="2H11 PFL"; $resn_exch{"H113 PFL"}="3H11 PFL"; $resn_exch{"H121 PFL"}="1H12 PFL";
$resn_exch{"H122 PFL"}="2H12 PFL"; $resn_exch{"H123 PFL"}="3H12 PFL"; $resn_exch{" HN1 PFM"}="1HN  PFM";
$resn_exch{" HN2 PFM"}="2HN  PFM"; $resn_exch{" HB1 PFM"}="1HB  PFM"; $resn_exch{" HB2 PFM"}="2HB  PFM";
$resn_exch{"H2A1 PFM"}="1H2A PFM"; $resn_exch{"H2A2 PFM"}="2H2A PFM"; $resn_exch{"H2A3 PFM"}="3H2A PFM";
$resn_exch{"H5A1 PFM"}="1H5A PFM"; $resn_exch{"H5A2 PFM"}="2H5A PFM"; $resn_exch{"HOP2 PFM"}="2HOP PFM";
$resn_exch{"HOP3 PFM"}="3HOP PFM"; $resn_exch{"HAB1 PFP"}="1HAB PFP"; $resn_exch{"HAB2 PFP"}="2HAB PFP";
$resn_exch{"HAB3 PFP"}="3HAB PFP"; $resn_exch{"HAN1 PFP"}="1HAN PFP"; $resn_exch{"HAN2 PFP"}="2HAN PFP";
$resn_exch{"HAM1 PFP"}="1HAM PFP"; $resn_exch{"HAM2 PFP"}="2HAM PFP"; $resn_exch{"HAA1 PFP"}="1HAA PFP";
$resn_exch{"HAA2 PFP"}="2HAA PFP"; $resn_exch{"HAA3 PFP"}="3HAA PFP"; $resn_exch{"H231 PFQ"}="1H23 PFQ";
$resn_exch{"H232 PFQ"}="2H23 PFQ"; $resn_exch{"H8C1 PFQ"}="1H8C PFQ"; $resn_exch{"H8C2 PFQ"}="2H8C PFQ";
$resn_exch{" H11 PFS"}="1H1  PFS"; $resn_exch{" H12 PFS"}="2H1  PFS"; $resn_exch{"HH31 PFS"}="1HH3 PFS";
$resn_exch{"HH32 PFS"}="2HH3 PFS"; $resn_exch{"HH33 PFS"}="3HH3 PFS"; $resn_exch{" H31 PFS"}="1H3  PFS";
$resn_exch{" H32 PFS"}="2H3  PFS"; $resn_exch{"H111 PFS"}="1H11 PFS"; $resn_exch{"H112 PFS"}="2H11 PFS";
$resn_exch{"H121 PFS"}="1H12 PFS"; $resn_exch{"H122 PFS"}="2H12 PFS"; $resn_exch{"H311 PFS"}="1H31 PFS";
$resn_exch{"H312 PFS"}="2H31 PFS"; $resn_exch{"H321 PFS"}="1H32 PFS"; $resn_exch{"H322 PFS"}="2H32 PFS";
$resn_exch{"H1N1 PFS"}="1H1N PFS"; $resn_exch{"H1N2 PFS"}="2H1N PFS"; $resn_exch{"H1N3 PFS"}="3H1N PFS";
$resn_exch{"H2N1 PFS"}="1H2N PFS"; $resn_exch{"H2N2 PFS"}="2H2N PFS"; $resn_exch{"H2N3 PFS"}="3H2N PFS";
$resn_exch{"H3N1 PFS"}="1H3N PFS"; $resn_exch{"H3N2 PFS"}="2H3N PFS"; $resn_exch{"H3N3 PFS"}="3H3N PFS";
$resn_exch{"H131 PFS"}="1H13 PFS"; $resn_exch{"H132 PFS"}="2H13 PFS"; $resn_exch{"H141 PFS"}="1H14 PFS";
$resn_exch{"H142 PFS"}="2H14 PFS"; $resn_exch{"H151 PFS"}="1H15 PFS"; $resn_exch{"H152 PFS"}="2H15 PFS";
$resn_exch{"H161 PFS"}="1H16 PFS"; $resn_exch{"H162 PFS"}="2H16 PFS"; $resn_exch{"H171 PFS"}="1H17 PFS";
$resn_exch{"H172 PFS"}="2H17 PFS"; $resn_exch{"H181 PFS"}="1H18 PFS"; $resn_exch{"H182 PFS"}="2H18 PFS";
$resn_exch{"H191 PFS"}="1H19 PFS"; $resn_exch{"H192 PFS"}="2H19 PFS"; $resn_exch{"H201 PFS"}="1H20 PFS";
$resn_exch{"H202 PFS"}="2H20 PFS"; $resn_exch{"H211 PFS"}="1H21 PFS"; $resn_exch{"H212 PFS"}="2H21 PFS";
$resn_exch{"H221 PFS"}="1H22 PFS"; $resn_exch{"H222 PFS"}="2H22 PFS"; $resn_exch{"H231 PFS"}="1H23 PFS";
$resn_exch{"H232 PFS"}="2H23 PFS"; $resn_exch{"H241 PFS"}="1H24 PFS"; $resn_exch{"H242 PFS"}="2H24 PFS";
$resn_exch{"H251 PFS"}="1H25 PFS"; $resn_exch{"H252 PFS"}="2H25 PFS"; $resn_exch{"H261 PFS"}="1H26 PFS";
$resn_exch{"H262 PFS"}="2H26 PFS"; $resn_exch{"H263 PFS"}="3H26 PFS"; $resn_exch{" H41 PFZ"}="1H4  PFZ";
$resn_exch{" H42 PFZ"}="2H4  PFZ"; $resn_exch{" H51 PFZ"}="1H5  PFZ"; $resn_exch{" H52 PFZ"}="2H5  PFZ";
$resn_exch{" H61 PFZ"}="1H6  PFZ"; $resn_exch{" H62 PFZ"}="2H6  PFZ"; $resn_exch{" H71 PFZ"}="1H7  PFZ";
$resn_exch{" H72 PFZ"}="2H7  PFZ"; $resn_exch{" H81 PFZ"}="1H8  PFZ"; $resn_exch{" H82 PFZ"}="2H8  PFZ";
$resn_exch{" H91 PFZ"}="1H9  PFZ"; $resn_exch{" H92 PFZ"}="2H9  PFZ"; $resn_exch{"H101 PFZ"}="1H10 PFZ";
$resn_exch{"H102 PFZ"}="2H10 PFZ"; $resn_exch{"H103 PFZ"}="3H10 PFZ"; $resn_exch{" H51 PG0"}="1H5  PG0";
$resn_exch{" H52 PG0"}="2H5  PG0"; $resn_exch{" H53 PG0"}="3H5  PG0"; $resn_exch{" H41 PG0"}="1H4  PG0";
$resn_exch{" H42 PG0"}="2H4  PG0"; $resn_exch{" H31 PG0"}="1H3  PG0"; $resn_exch{" H32 PG0"}="2H3  PG0";
$resn_exch{" H21 PG0"}="1H2  PG0"; $resn_exch{" H22 PG0"}="2H2  PG0"; $resn_exch{" H11 PG0"}="1H1  PG0";
$resn_exch{" H12 PG0"}="2H1  PG0"; $resn_exch{"HC22 PG1"}="H2   PG1"; $resn_exch{" H   PG1"}=" H23 PG1";
$resn_exch{" H2  PG1"}=" H24 PG1"; $resn_exch{" HB2 PG1"}=" H20 PG1"; $resn_exch{" HB3 PG1"}=" H21 PG1";
$resn_exch{" H21 PG2"}="1H2  PG2"; $resn_exch{" H22 PG2"}="2H2  PG2"; $resn_exch{" H31 PG2"}="1H3  PG2";
$resn_exch{" H32 PG2"}="2H3  PG2"; $resn_exch{" H41 PG2"}="1H4  PG2"; $resn_exch{" H42 PG2"}="2H4  PG2";
$resn_exch{" H71 PG2"}="1H7  PG2"; $resn_exch{" H72 PG2"}="2H7  PG2"; $resn_exch{"H101 PG2"}="1H10 PG2";
$resn_exch{"H102 PG2"}="2H10 PG2"; $resn_exch{"H161 PG2"}="1H16 PG2"; $resn_exch{"H162 PG2"}="2H16 PG2";
$resn_exch{"H171 PG2"}="1H17 PG2"; $resn_exch{"H172 PG2"}="2H17 PG2"; $resn_exch{"H181 PG2"}="1H18 PG2";
$resn_exch{"H182 PG2"}="2H18 PG2"; $resn_exch{"H191 PG2"}="1H19 PG2"; $resn_exch{"H192 PG2"}="2H19 PG2";
$resn_exch{"H201 PG2"}="1H20 PG2"; $resn_exch{"H202 PG2"}="2H20 PG2"; $resn_exch{"H203 PG2"}="3H20 PG2";
$resn_exch{" HB1 PG3"}="1HB  PG3"; $resn_exch{" HB2 PG3"}="2HB  PG3"; $resn_exch{" HG1 PG3"}="1HG  PG3";
$resn_exch{" HG2 PG3"}="2HG  PG3"; $resn_exch{" HD1 PG3"}="1HD  PG3"; $resn_exch{" HD2 PG3"}="2HD  PG3";
$resn_exch{"HH11 PG3"}="1HH1 PG3"; $resn_exch{"HH12 PG3"}="2HH1 PG3"; $resn_exch{"HH21 PG3"}="1HH2 PG3";
$resn_exch{"HH22 PG3"}="2HH2 PG3"; $resn_exch{" H11 PG5"}="1H1  PG5"; $resn_exch{" H12 PG5"}="2H1  PG5";
$resn_exch{" H13 PG5"}="3H1  PG5"; $resn_exch{" H21 PG5"}="1H2  PG5"; $resn_exch{" H22 PG5"}="2H2  PG5";
$resn_exch{" H31 PG5"}="1H3  PG5"; $resn_exch{" H32 PG5"}="2H3  PG5"; $resn_exch{" H41 PG5"}="1H4  PG5";
$resn_exch{" H42 PG5"}="2H4  PG5"; $resn_exch{" H51 PG5"}="1H5  PG5"; $resn_exch{" H52 PG5"}="2H5  PG5";
$resn_exch{" H61 PG5"}="1H6  PG5"; $resn_exch{" H62 PG5"}="2H6  PG5"; $resn_exch{" H71 PG5"}="1H7  PG5";
$resn_exch{" H72 PG5"}="2H7  PG5"; $resn_exch{" H81 PG5"}="1H8  PG5"; $resn_exch{" H82 PG5"}="2H8  PG5";
$resn_exch{" H83 PG5"}="3H8  PG5"; $resn_exch{" H11 PG6"}="1H1  PG6"; $resn_exch{" H12 PG6"}="2H1  PG6";
$resn_exch{" H13 PG6"}="3H1  PG6"; $resn_exch{" H21 PG6"}="1H2  PG6"; $resn_exch{" H22 PG6"}="2H2  PG6";
$resn_exch{" H31 PG6"}="1H3  PG6"; $resn_exch{" H32 PG6"}="2H3  PG6"; $resn_exch{" H41 PG6"}="1H4  PG6";
$resn_exch{" H42 PG6"}="2H4  PG6"; $resn_exch{" H51 PG6"}="1H5  PG6"; $resn_exch{" H52 PG6"}="2H5  PG6";
$resn_exch{" H61 PG6"}="1H6  PG6"; $resn_exch{" H62 PG6"}="2H6  PG6"; $resn_exch{" H71 PG6"}="1H7  PG6";
$resn_exch{" H72 PG6"}="2H7  PG6"; $resn_exch{" H81 PG6"}="1H8  PG6"; $resn_exch{" H82 PG6"}="2H8  PG6";
$resn_exch{" H91 PG6"}="1H9  PG6"; $resn_exch{" H92 PG6"}="2H9  PG6"; $resn_exch{"H101 PG6"}="1H10 PG6";
$resn_exch{"H102 PG6"}="2H10 PG6"; $resn_exch{"H111 PG6"}="1H11 PG6"; $resn_exch{"H112 PG6"}="2H11 PG6";
$resn_exch{"H121 PG6"}="1H12 PG6"; $resn_exch{"H122 PG6"}="2H12 PG6"; $resn_exch{"H123 PG6"}="3H12 PG6";
$resn_exch{"HOP2 PG7"}="2HOP PG7"; $resn_exch{"HO3' PG7"}="H3T  PG7"; $resn_exch{" H1' PG7"}=" H1* PG7";
$resn_exch{" H2' PG7"}="1H2* PG7"; $resn_exch{"H2'' PG7"}="2H2* PG7"; $resn_exch{" H3' PG7"}=" H3* PG7";
$resn_exch{" H4' PG7"}=" H4* PG7"; $resn_exch{" H5' PG7"}="1H5* PG7"; $resn_exch{"H5'' PG7"}="2H5* PG7";
$resn_exch{" H21 PG7"}="1H2  PG7"; $resn_exch{"HOP3 PG7"}="3HOP PG7"; $resn_exch{" H   PG9"}="1HN  PG9";
$resn_exch{" H2  PG9"}="2HN  PG9"; $resn_exch{"HOP3 PGA"}="3HOP PGA"; $resn_exch{"HOP4 PGA"}="4HOP PGA";
$resn_exch{" H21 PGA"}="1H2  PGA"; $resn_exch{" H22 PGA"}="2H2  PGA"; $resn_exch{" H21 PGC"}="1H2  PGC";
$resn_exch{" H22 PGC"}="2H2  PGC"; $resn_exch{"HOP3 PGC"}="3HOP PGC"; $resn_exch{"HOP4 PGC"}="4HOP PGC";
$resn_exch{"HOB2 PGD"}="2HOB PGD"; $resn_exch{"HOA2 PGD"}="2HOA PGD"; $resn_exch{"H5'1 PGD"}="1H5' PGD";
$resn_exch{"H5'2 PGD"}="2H5' PGD"; $resn_exch{"HO3' PGD"}="'HO3 PGD"; $resn_exch{"HO2' PGD"}="'HO2 PGD";
$resn_exch{"HN21 PGD"}="1HN2 PGD"; $resn_exch{"HN22 PGD"}="2HN2 PGD"; $resn_exch{"H101 PGD"}="1H10 PGD";
$resn_exch{"H102 PGD"}="2H10 PGD"; $resn_exch{"H191 PGD"}="1H19 PGD"; $resn_exch{"H192 PGD"}="2H19 PGD";
$resn_exch{" H12 PGE"}="2H1  PGE"; $resn_exch{" H22 PGE"}="2H2  PGE"; $resn_exch{" H32 PGE"}="2H3  PGE";
$resn_exch{" H42 PGE"}="2H4  PGE"; $resn_exch{" H62 PGE"}="2H6  PGE"; $resn_exch{" H52 PGE"}="2H5  PGE";
$resn_exch{" H71 PGF"}="1H7  PGF"; $resn_exch{" H72 PGF"}="2H7  PGF"; $resn_exch{" H73 PGF"}="3H7  PGF";
$resn_exch{" H81 PGF"}="1H8  PGF"; $resn_exch{" H82 PGF"}="2H8  PGF"; $resn_exch{" H91 PGF"}="1H9  PGF";
$resn_exch{" H92 PGF"}="2H9  PGF"; $resn_exch{"H101 PGF"}="1H10 PGF"; $resn_exch{"H102 PGF"}="2H10 PGF";
$resn_exch{"H111 PGF"}="1H11 PGF"; $resn_exch{"H112 PGF"}="2H11 PGF"; $resn_exch{"H121 PGF"}="1H12 PGF";
$resn_exch{"H122 PGF"}="2H12 PGF"; $resn_exch{"H131 PGF"}="1H13 PGF"; $resn_exch{"H132 PGF"}="2H13 PGF";
$resn_exch{"H141 PGF"}="1H14 PGF"; $resn_exch{"H142 PGF"}="2H14 PGF"; $resn_exch{"H151 PGF"}="1H15 PGF";
$resn_exch{"H152 PGF"}="2H15 PGF"; $resn_exch{"H153 PGF"}="3H15 PGF"; $resn_exch{" H21 PGG"}="1H2  PGG";
$resn_exch{" H31 PGG"}="1H3  PGG"; $resn_exch{" H51 PGG"}="1H5  PGG"; $resn_exch{" H61 PGG"}="1H6  PGG";
$resn_exch{"HO21 PGG"}="1HO2 PGG"; $resn_exch{" H81 PGG"}="1H8  PGG"; $resn_exch{" H82 PGG"}="2H8  PGG";
$resn_exch{" H91 PGG"}="1H9  PGG"; $resn_exch{" H92 PGG"}="2H9  PGG"; $resn_exch{"H101 PGG"}="1H10 PGG";
$resn_exch{"H102 PGG"}="2H10 PGG"; $resn_exch{"HN21 PGG"}="1HN2 PGG"; $resn_exch{"H121 PGG"}="1H12 PGG";
$resn_exch{"H122 PGG"}="2H12 PGG"; $resn_exch{"HO71 PGG"}="1HO7 PGG"; $resn_exch{" H21 PGH"}="1H2  PGH";
$resn_exch{" H22 PGH"}="2H2  PGH"; $resn_exch{"HOP3 PGH"}="3HOP PGH"; $resn_exch{"HOP4 PGH"}="4HOP PGH";
$resn_exch{"H6C1 PGI"}="1H6C PGI"; $resn_exch{"H6C2 PGI"}="2H6C PGI"; $resn_exch{"H9C1 PGI"}="1H9C PGI";
$resn_exch{"H9C2 PGI"}="2H9C PGI"; $resn_exch{"H101 PGI"}="1H10 PGI"; $resn_exch{"H102 PGI"}="2H10 PGI";
$resn_exch{" H61 PGJ"}="1H6  PGJ"; $resn_exch{" H62 PGJ"}="2H6  PGJ"; $resn_exch{"H101 PGJ"}="1H10 PGJ";
$resn_exch{"H102 PGJ"}="2H10 PGJ"; $resn_exch{"H111 PGJ"}="1H11 PGJ"; $resn_exch{"H112 PGJ"}="2H11 PGJ";
$resn_exch{"H113 PGJ"}="3H11 PGJ"; $resn_exch{"H481 PGK"}="1H48 PGK"; $resn_exch{"H482 PGK"}="2H48 PGK";
$resn_exch{"H483 PGK"}="3H48 PGK"; $resn_exch{"H471 PGK"}="1H47 PGK"; $resn_exch{"H472 PGK"}="2H47 PGK";
$resn_exch{"H461 PGK"}="1H46 PGK"; $resn_exch{"H462 PGK"}="2H46 PGK"; $resn_exch{"H451 PGK"}="1H45 PGK";
$resn_exch{"H452 PGK"}="2H45 PGK"; $resn_exch{"H441 PGK"}="1H44 PGK"; $resn_exch{"H442 PGK"}="2H44 PGK";
$resn_exch{"H431 PGK"}="1H43 PGK"; $resn_exch{"H432 PGK"}="2H43 PGK"; $resn_exch{"H421 PGK"}="1H42 PGK";
$resn_exch{"H422 PGK"}="2H42 PGK"; $resn_exch{"H411 PGK"}="1H41 PGK"; $resn_exch{"H412 PGK"}="2H41 PGK";
$resn_exch{"H381 PGK"}="1H38 PGK"; $resn_exch{"H382 PGK"}="2H38 PGK"; $resn_exch{"H371 PGK"}="1H37 PGK";
$resn_exch{"H372 PGK"}="2H37 PGK"; $resn_exch{"H361 PGK"}="1H36 PGK"; $resn_exch{"H362 PGK"}="2H36 PGK";
$resn_exch{"H351 PGK"}="1H35 PGK"; $resn_exch{"H352 PGK"}="2H35 PGK"; $resn_exch{"H341 PGK"}="1H34 PGK";
$resn_exch{"H342 PGK"}="2H34 PGK"; $resn_exch{"H331 PGK"}="1H33 PGK"; $resn_exch{"H332 PGK"}="2H33 PGK";
$resn_exch{"H321 PGK"}="1H32 PGK"; $resn_exch{"H322 PGK"}="2H32 PGK"; $resn_exch{" H11 PGK"}="1H1  PGK";
$resn_exch{" H12 PGK"}="2H1  PGK"; $resn_exch{" H41 PGK"}="1H4  PGK"; $resn_exch{" H42 PGK"}="2H4  PGK";
$resn_exch{" H61 PGK"}="1H6  PGK"; $resn_exch{" H62 PGK"}="2H6  PGK"; $resn_exch{" H31 PGK"}="1H3  PGK";
$resn_exch{" H32 PGK"}="2H3  PGK"; $resn_exch{"H121 PGK"}="1H12 PGK"; $resn_exch{"H122 PGK"}="2H12 PGK";
$resn_exch{"H131 PGK"}="1H13 PGK"; $resn_exch{"H132 PGK"}="2H13 PGK"; $resn_exch{"H141 PGK"}="1H14 PGK";
$resn_exch{"H142 PGK"}="2H14 PGK"; $resn_exch{"H151 PGK"}="1H15 PGK"; $resn_exch{"H152 PGK"}="2H15 PGK";
$resn_exch{"H161 PGK"}="1H16 PGK"; $resn_exch{"H162 PGK"}="2H16 PGK"; $resn_exch{"H171 PGK"}="1H17 PGK";
$resn_exch{"H172 PGK"}="2H17 PGK"; $resn_exch{"H181 PGK"}="1H18 PGK"; $resn_exch{"H182 PGK"}="2H18 PGK";
$resn_exch{"H191 PGK"}="1H19 PGK"; $resn_exch{"H192 PGK"}="2H19 PGK"; $resn_exch{"H201 PGK"}="1H20 PGK";
$resn_exch{"H202 PGK"}="2H20 PGK"; $resn_exch{"H211 PGK"}="1H21 PGK"; $resn_exch{"H212 PGK"}="2H21 PGK";
$resn_exch{"H221 PGK"}="1H22 PGK"; $resn_exch{"H222 PGK"}="2H22 PGK"; $resn_exch{"H231 PGK"}="1H23 PGK";
$resn_exch{"H232 PGK"}="2H23 PGK"; $resn_exch{"H241 PGK"}="1H24 PGK"; $resn_exch{"H242 PGK"}="2H24 PGK";
$resn_exch{"H251 PGK"}="1H25 PGK"; $resn_exch{"H252 PGK"}="2H25 PGK"; $resn_exch{"H261 PGK"}="1H26 PGK";
$resn_exch{"H262 PGK"}="2H26 PGK"; $resn_exch{"H263 PGK"}="3H26 PGK"; $resn_exch{" HN1 PGL"}="1HN  PGL";
$resn_exch{" HN2 PGL"}="2HN  PGL"; $resn_exch{" H1  PGL"}="1H   PGL"; $resn_exch{" H2  PGL"}="2H   PGL";
$resn_exch{"HC11 PGM"}="1HC1 PGM"; $resn_exch{"HC12 PGM"}="2HC1 PGM"; $resn_exch{"HC31 PGM"}="1HC3 PGM";
$resn_exch{"HC32 PGM"}="2HC3 PGM"; $resn_exch{"HC71 PGM"}="1HC7 PGM"; $resn_exch{"HC72 PGM"}="2HC7 PGM";
$resn_exch{"HC91 PGM"}="1HC9 PGM"; $resn_exch{"HC92 PGM"}="2HC9 PGM"; $resn_exch{"HCB1 PGM"}="1HCB PGM";
$resn_exch{"HCB2 PGM"}="2HCB PGM"; $resn_exch{"HCC1 PGM"}="1HCC PGM"; $resn_exch{"HCC2 PGM"}="2HCC PGM";
$resn_exch{"HCD1 PGM"}="1HCD PGM"; $resn_exch{"HCD2 PGM"}="2HCD PGM"; $resn_exch{"HCE1 PGM"}="1HCE PGM";
$resn_exch{"HCE2 PGM"}="2HCE PGM"; $resn_exch{"HCF1 PGM"}="1HCF PGM"; $resn_exch{"HCF2 PGM"}="2HCF PGM";
$resn_exch{"HCG1 PGM"}="1HCG PGM"; $resn_exch{"HCG2 PGM"}="2HCG PGM"; $resn_exch{"HCH1 PGM"}="1HCH PGM";
$resn_exch{"HCH2 PGM"}="2HCH PGM"; $resn_exch{"HCI1 PGM"}="1HCI PGM"; $resn_exch{"HCI2 PGM"}="2HCI PGM";
$resn_exch{"HCJ1 PGM"}="1HCJ PGM"; $resn_exch{"HCJ2 PGM"}="2HCJ PGM"; $resn_exch{"HCK1 PGM"}="1HCK PGM";
$resn_exch{"HCK2 PGM"}="2HCK PGM"; $resn_exch{"HCL1 PGM"}="1HCL PGM"; $resn_exch{"HCL2 PGM"}="2HCL PGM";
$resn_exch{"HCM1 PGM"}="1HCM PGM"; $resn_exch{"HCM2 PGM"}="2HCM PGM"; $resn_exch{"HCN1 PGM"}="1HCN PGM";
$resn_exch{"HCN2 PGM"}="2HCN PGM"; $resn_exch{"HCO1 PGM"}="1HCO PGM"; $resn_exch{"HCO2 PGM"}="2HCO PGM";
$resn_exch{"HCP1 PGM"}="1HCP PGM"; $resn_exch{"HCP2 PGM"}="2HCP PGM"; $resn_exch{"HCP3 PGM"}="3HCP PGM";
$resn_exch{"HOP5 PGN"}="5HOP PGN"; $resn_exch{"HOP6 PGN"}="6HOP PGN"; $resn_exch{"HOP2 PGN"}="2HOP PGN";
$resn_exch{"HOP3 PGN"}="3HOP PGN"; $resn_exch{" H5' PGN"}="1H5* PGN"; $resn_exch{"H5'' PGN"}="2H5* PGN";
$resn_exch{" H4' PGN"}=" H4* PGN"; $resn_exch{" H3' PGN"}=" H3* PGN"; $resn_exch{" H2' PGN"}="1H2* PGN";
$resn_exch{"H2'' PGN"}="2H2* PGN"; $resn_exch{" H1' PGN"}=" H1* PGN"; $resn_exch{"HN21 PGN"}="1HN2 PGN";
$resn_exch{"HN22 PGN"}="2HN2 PGN"; $resn_exch{"HOP5 PGP"}="5HOP PGP"; $resn_exch{"HOP6 PGP"}="6HOP PGP";
$resn_exch{"HOP2 PGP"}="2HOP PGP"; $resn_exch{"HOP3 PGP"}="3HOP PGP"; $resn_exch{" H5' PGP"}="1H5* PGP";
$resn_exch{"H5'' PGP"}="2H5* PGP"; $resn_exch{" H4' PGP"}=" H4* PGP"; $resn_exch{" H3' PGP"}=" H3* PGP";
$resn_exch{" H2' PGP"}=" H2* PGP"; $resn_exch{"HO2' PGP"}="*HO2 PGP"; $resn_exch{" H1' PGP"}=" H1* PGP";
$resn_exch{"HN21 PGP"}="1HN2 PGP"; $resn_exch{"HN22 PGP"}="2HN2 PGP"; $resn_exch{" H11 PGQ"}="1H1  PGQ";
$resn_exch{" H12 PGQ"}="2H1  PGQ"; $resn_exch{" H31 PGQ"}="1H3  PGQ"; $resn_exch{" H32 PGQ"}="2H3  PGQ";
$resn_exch{" H33 PGQ"}="3H3  PGQ"; $resn_exch{" H11 PGR"}="1H1  PGR"; $resn_exch{" H12 PGR"}="2H1  PGR";
$resn_exch{" H31 PGR"}="1H3  PGR"; $resn_exch{" H32 PGR"}="2H3  PGR"; $resn_exch{" H33 PGR"}="3H3  PGR";
$resn_exch{"HOA2 PGS"}="2HOA PGS"; $resn_exch{"HOA3 PGS"}="3HOA PGS"; $resn_exch{"H5'1 PGS"}="1H5* PGS";
$resn_exch{"H5'2 PGS"}="2H5* PGS"; $resn_exch{" H4' PGS"}=" H4* PGS"; $resn_exch{" H3' PGS"}=" H3* PGS";
$resn_exch{"HO3' PGS"}="*HO3 PGS"; $resn_exch{" H2' PGS"}=" H2* PGS"; $resn_exch{"HO2' PGS"}="*HO2 PGS";
$resn_exch{" H1' PGS"}=" H1* PGS"; $resn_exch{"HOP2 PGS"}="2HOP PGS"; $resn_exch{"HOP3 PGS"}="3HOP PGS";
$resn_exch{"H481 PGT"}="1H48 PGT"; $resn_exch{"H482 PGT"}="2H48 PGT"; $resn_exch{"H483 PGT"}="3H48 PGT";
$resn_exch{"H471 PGT"}="1H47 PGT"; $resn_exch{"H472 PGT"}="2H47 PGT"; $resn_exch{"H461 PGT"}="1H46 PGT";
$resn_exch{"H462 PGT"}="2H46 PGT"; $resn_exch{"H451 PGT"}="1H45 PGT"; $resn_exch{"H452 PGT"}="2H45 PGT";
$resn_exch{"H441 PGT"}="1H44 PGT"; $resn_exch{"H442 PGT"}="2H44 PGT"; $resn_exch{"H431 PGT"}="1H43 PGT";
$resn_exch{"H432 PGT"}="2H43 PGT"; $resn_exch{"H421 PGT"}="1H42 PGT"; $resn_exch{"H422 PGT"}="2H42 PGT";
$resn_exch{"H411 PGT"}="1H41 PGT"; $resn_exch{"H412 PGT"}="2H41 PGT"; $resn_exch{"H401 PGT"}="1H40 PGT";
$resn_exch{"H402 PGT"}="2H40 PGT"; $resn_exch{"H391 PGT"}="1H39 PGT"; $resn_exch{"H392 PGT"}="2H39 PGT";
$resn_exch{"H381 PGT"}="1H38 PGT"; $resn_exch{"H382 PGT"}="2H38 PGT"; $resn_exch{"H371 PGT"}="1H37 PGT";
$resn_exch{"H372 PGT"}="2H37 PGT"; $resn_exch{"H361 PGT"}="1H36 PGT"; $resn_exch{"H362 PGT"}="2H36 PGT";
$resn_exch{"H351 PGT"}="1H35 PGT"; $resn_exch{"H352 PGT"}="2H35 PGT"; $resn_exch{"H341 PGT"}="1H34 PGT";
$resn_exch{"H342 PGT"}="2H34 PGT"; $resn_exch{"H331 PGT"}="1H33 PGT"; $resn_exch{"H332 PGT"}="2H33 PGT";
$resn_exch{"H321 PGT"}="1H32 PGT"; $resn_exch{"H322 PGT"}="2H32 PGT"; $resn_exch{" H11 PGT"}="1H1  PGT";
$resn_exch{" H12 PGT"}="2H1  PGT"; $resn_exch{" H41 PGT"}="1H4  PGT"; $resn_exch{" H42 PGT"}="2H4  PGT";
$resn_exch{" H61 PGT"}="1H6  PGT"; $resn_exch{" H62 PGT"}="2H6  PGT"; $resn_exch{" H31 PGT"}="1H3  PGT";
$resn_exch{" H32 PGT"}="2H3  PGT"; $resn_exch{"H121 PGT"}="1H12 PGT"; $resn_exch{"H122 PGT"}="2H12 PGT";
$resn_exch{"H131 PGT"}="1H13 PGT"; $resn_exch{"H132 PGT"}="2H13 PGT"; $resn_exch{"H141 PGT"}="1H14 PGT";
$resn_exch{"H142 PGT"}="2H14 PGT"; $resn_exch{"H151 PGT"}="1H15 PGT"; $resn_exch{"H152 PGT"}="2H15 PGT";
$resn_exch{"H161 PGT"}="1H16 PGT"; $resn_exch{"H162 PGT"}="2H16 PGT"; $resn_exch{"H171 PGT"}="1H17 PGT";
$resn_exch{"H172 PGT"}="2H17 PGT"; $resn_exch{"H181 PGT"}="1H18 PGT"; $resn_exch{"H182 PGT"}="2H18 PGT";
$resn_exch{"H191 PGT"}="1H19 PGT"; $resn_exch{"H192 PGT"}="2H19 PGT"; $resn_exch{"H201 PGT"}="1H20 PGT";
$resn_exch{"H202 PGT"}="2H20 PGT"; $resn_exch{"H211 PGT"}="1H21 PGT"; $resn_exch{"H212 PGT"}="2H21 PGT";
$resn_exch{"H221 PGT"}="1H22 PGT"; $resn_exch{"H222 PGT"}="2H22 PGT"; $resn_exch{"H231 PGT"}="1H23 PGT";
$resn_exch{"H232 PGT"}="2H23 PGT"; $resn_exch{"H241 PGT"}="1H24 PGT"; $resn_exch{"H242 PGT"}="2H24 PGT";
$resn_exch{"H251 PGT"}="1H25 PGT"; $resn_exch{"H252 PGT"}="2H25 PGT"; $resn_exch{"H261 PGT"}="1H26 PGT";
$resn_exch{"H262 PGT"}="2H26 PGT"; $resn_exch{"H263 PGT"}="3H26 PGT"; $resn_exch{"H2A1 PGU"}="1H2A PGU";
$resn_exch{"H2A2 PGU"}="2H2A PGU"; $resn_exch{"H2A3 PGU"}="3H2A PGU"; $resn_exch{"H4A1 PGU"}="1H4A PGU";
$resn_exch{"H4A2 PGU"}="2H4A PGU"; $resn_exch{"H5A1 PGU"}="1H5A PGU"; $resn_exch{"H5A2 PGU"}="2H5A PGU";
$resn_exch{" HB1 PGU"}="1HB  PGU"; $resn_exch{" HB2 PGU"}="2HB  PGU"; $resn_exch{" HG1 PGU"}="1HG  PGU";
$resn_exch{" HG2 PGU"}="2HG  PGU"; $resn_exch{"H011 PGV"}="1H01 PGV"; $resn_exch{"H012 PGV"}="2H01 PGV";
$resn_exch{"H031 PGV"}="1H03 PGV"; $resn_exch{"H032 PGV"}="2H03 PGV"; $resn_exch{"H041 PGV"}="1H04 PGV";
$resn_exch{"H042 PGV"}="2H04 PGV"; $resn_exch{"H061 PGV"}="1H06 PGV"; $resn_exch{"H062 PGV"}="2H06 PGV";
$resn_exch{" H21 PGV"}="1H2  PGV"; $resn_exch{" H22 PGV"}="2H2  PGV"; $resn_exch{" H31 PGV"}="1H3  PGV";
$resn_exch{" H32 PGV"}="2H3  PGV"; $resn_exch{" H41 PGV"}="1H4  PGV"; $resn_exch{" H42 PGV"}="2H4  PGV";
$resn_exch{" H51 PGV"}="1H5  PGV"; $resn_exch{" H52 PGV"}="2H5  PGV"; $resn_exch{" H61 PGV"}="1H6  PGV";
$resn_exch{" H62 PGV"}="2H6  PGV"; $resn_exch{" H71 PGV"}="1H7  PGV"; $resn_exch{" H72 PGV"}="2H7  PGV";
$resn_exch{" H81 PGV"}="1H8  PGV"; $resn_exch{" H82 PGV"}="2H8  PGV"; $resn_exch{" H91 PGV"}="1H9  PGV";
$resn_exch{" H92 PGV"}="2H9  PGV"; $resn_exch{"H101 PGV"}="1H10 PGV"; $resn_exch{"H102 PGV"}="2H10 PGV";
$resn_exch{"H131 PGV"}="1H13 PGV"; $resn_exch{"H132 PGV"}="2H13 PGV"; $resn_exch{"H141 PGV"}="1H14 PGV";
$resn_exch{"H142 PGV"}="2H14 PGV"; $resn_exch{"H151 PGV"}="1H15 PGV"; $resn_exch{"H152 PGV"}="2H15 PGV";
$resn_exch{"H161 PGV"}="1H16 PGV"; $resn_exch{"H162 PGV"}="2H16 PGV"; $resn_exch{"H171 PGV"}="1H17 PGV";
$resn_exch{"H172 PGV"}="2H17 PGV"; $resn_exch{"H181 PGV"}="1H18 PGV"; $resn_exch{"H182 PGV"}="2H18 PGV";
$resn_exch{"H183 PGV"}="3H18 PGV"; $resn_exch{"H201 PGV"}="1H20 PGV"; $resn_exch{"H202 PGV"}="2H20 PGV";
$resn_exch{"H211 PGV"}="1H21 PGV"; $resn_exch{"H212 PGV"}="2H21 PGV"; $resn_exch{"H221 PGV"}="1H22 PGV";
$resn_exch{"H222 PGV"}="2H22 PGV"; $resn_exch{"H231 PGV"}="1H23 PGV"; $resn_exch{"H232 PGV"}="2H23 PGV";
$resn_exch{"H241 PGV"}="1H24 PGV"; $resn_exch{"H242 PGV"}="2H24 PGV"; $resn_exch{"H251 PGV"}="1H25 PGV";
$resn_exch{"H252 PGV"}="2H25 PGV"; $resn_exch{"H261 PGV"}="1H26 PGV"; $resn_exch{"H262 PGV"}="2H26 PGV";
$resn_exch{"H271 PGV"}="1H27 PGV"; $resn_exch{"H272 PGV"}="2H27 PGV"; $resn_exch{"H281 PGV"}="1H28 PGV";
$resn_exch{"H282 PGV"}="2H28 PGV"; $resn_exch{"H291 PGV"}="1H29 PGV"; $resn_exch{"H292 PGV"}="2H29 PGV";
$resn_exch{"H301 PGV"}="1H30 PGV"; $resn_exch{"H302 PGV"}="2H30 PGV"; $resn_exch{"H311 PGV"}="1H31 PGV";
$resn_exch{"H312 PGV"}="2H31 PGV"; $resn_exch{"H321 PGV"}="1H32 PGV"; $resn_exch{"H322 PGV"}="2H32 PGV";
$resn_exch{"H331 PGV"}="1H33 PGV"; $resn_exch{"H332 PGV"}="2H33 PGV"; $resn_exch{"H341 PGV"}="1H34 PGV";
$resn_exch{"H342 PGV"}="2H34 PGV"; $resn_exch{"H343 PGV"}="3H34 PGV"; $resn_exch{" HAD PGW"}=" H1  PGW";
$resn_exch{"HADA PGW"}="H2   PGW"; $resn_exch{"HOAE PGW"}="H3   PGW"; $resn_exch{"HOAF PGW"}="H4   PGW";
$resn_exch{" H01 PGW"}=" H5  PGW"; $resn_exch{"H01A PGW"}="H6   PGW"; $resn_exch{" H02 PGW"}=" H7  PGW";
$resn_exch{" H2  PGW"}=" H8  PGW"; $resn_exch{" H2A PGW"}=" H9  PGW"; $resn_exch{" H03 PGW"}=" H10 PGW";
$resn_exch{"H03A PGW"}="H11  PGW"; $resn_exch{" H3  PGW"}=" H12 PGW"; $resn_exch{" H3A PGW"}=" H13 PGW";
$resn_exch{" H04 PGW"}=" H14 PGW"; $resn_exch{"H04A PGW"}="H15  PGW"; $resn_exch{" H4  PGW"}=" H16 PGW";
$resn_exch{" H4A PGW"}=" H17 PGW"; $resn_exch{" H05 PGW"}=" H18 PGW"; $resn_exch{" H5  PGW"}=" H19 PGW";
$resn_exch{" H5A PGW"}=" H20 PGW"; $resn_exch{" H06 PGW"}=" H21 PGW"; $resn_exch{"H06A PGW"}="H22  PGW";
$resn_exch{" H6  PGW"}=" H23 PGW"; $resn_exch{" H6A PGW"}=" H24 PGW"; $resn_exch{" H07 PGW"}=" H25 PGW";
$resn_exch{"H07A PGW"}="H26  PGW"; $resn_exch{" H7  PGW"}=" H27 PGW"; $resn_exch{" H7A PGW"}=" H28 PGW";
$resn_exch{" H08 PGW"}=" H29 PGW"; $resn_exch{"H08A PGW"}="H30  PGW"; $resn_exch{" H8  PGW"}=" H31 PGW";
$resn_exch{" H8A PGW"}=" H32 PGW"; $resn_exch{" H09 PGW"}=" H33 PGW"; $resn_exch{"H09A PGW"}="H34  PGW";
$resn_exch{" H11 PGW"}=" H35 PGW"; $resn_exch{"H11A PGW"}="H36  PGW"; $resn_exch{" H12 PGW"}=" H37 PGW";
$resn_exch{"H12A PGW"}="H38  PGW"; $resn_exch{" H13 PGW"}=" H39 PGW"; $resn_exch{"H13A PGW"}="H40  PGW";
$resn_exch{"HO13 PGW"}="H41  PGW"; $resn_exch{" H14 PGW"}=" H42 PGW"; $resn_exch{"H14A PGW"}="H43  PGW";
$resn_exch{"H14B PGW"}="H44  PGW"; $resn_exch{" H15 PGW"}=" H45 PGW"; $resn_exch{"H15A PGW"}="H46  PGW";
$resn_exch{" H16 PGW"}=" H47 PGW"; $resn_exch{"H16A PGW"}="H48  PGW"; $resn_exch{" H17 PGW"}=" H49 PGW";
$resn_exch{"H17A PGW"}="H50  PGW"; $resn_exch{" H18 PGW"}=" H51 PGW"; $resn_exch{"H18A PGW"}="H52  PGW";
$resn_exch{" H20 PGW"}=" H53 PGW"; $resn_exch{"H20A PGW"}="H54  PGW"; $resn_exch{" H21 PGW"}=" H55 PGW";
$resn_exch{"H21A PGW"}="H56  PGW"; $resn_exch{" H22 PGW"}=" H57 PGW"; $resn_exch{"H22A PGW"}="H58  PGW";
$resn_exch{" H23 PGW"}=" H59 PGW"; $resn_exch{"H23A PGW"}="H60  PGW"; $resn_exch{" H24 PGW"}=" H61 PGW";
$resn_exch{"H24A PGW"}="H62  PGW"; $resn_exch{" H25 PGW"}=" H63 PGW"; $resn_exch{"H25A PGW"}="H64  PGW";
$resn_exch{" H26 PGW"}=" H65 PGW"; $resn_exch{"H26A PGW"}="H66  PGW"; $resn_exch{" H27 PGW"}=" H67 PGW";
$resn_exch{"H27A PGW"}="H68  PGW"; $resn_exch{" H28 PGW"}=" H69 PGW"; $resn_exch{"H28A PGW"}="H70  PGW";
$resn_exch{" H29 PGW"}=" H71 PGW"; $resn_exch{"H29A PGW"}="H72  PGW"; $resn_exch{"H29B PGW"}="H73  PGW";
$resn_exch{" H30 PGW"}=" H74 PGW"; $resn_exch{"H30A PGW"}="H75  PGW"; $resn_exch{" H9  PGW"}=" H76 PGW";
$resn_exch{" H10 PGW"}=" H77 PGW"; $resn_exch{" H21 PGX"}="1H2  PGX"; $resn_exch{" H22 PGX"}="2H2  PGX";
$resn_exch{" H31 PGX"}="1H3  PGX"; $resn_exch{" H32 PGX"}="2H3  PGX"; $resn_exch{" H41 PGX"}="1H4  PGX";
$resn_exch{" H42 PGX"}="2H4  PGX"; $resn_exch{" H51 PGX"}="1H5  PGX"; $resn_exch{" H61 PGX"}="1H6  PGX";
$resn_exch{" H71 PGX"}="1H7  PGX"; $resn_exch{" H72 PGX"}="2H7  PGX"; $resn_exch{" H81 PGX"}="1H8  PGX";
$resn_exch{" H91 PGX"}="1H9  PGX"; $resn_exch{"H101 PGX"}="1H10 PGX"; $resn_exch{"H102 PGX"}="2H10 PGX";
$resn_exch{"H111 PGX"}="1H11 PGX"; $resn_exch{"H121 PGX"}="1H12 PGX"; $resn_exch{"H131 PGX"}="1H13 PGX";
$resn_exch{"H141 PGX"}="1H14 PGX"; $resn_exch{"H151 PGX"}="1H15 PGX"; $resn_exch{"H161 PGX"}="1H16 PGX";
$resn_exch{"H162 PGX"}="2H16 PGX"; $resn_exch{"H171 PGX"}="1H17 PGX"; $resn_exch{"H172 PGX"}="2H17 PGX";
$resn_exch{"H181 PGX"}="1H18 PGX"; $resn_exch{"H182 PGX"}="2H18 PGX"; $resn_exch{"H191 PGX"}="1H19 PGX";
$resn_exch{"H192 PGX"}="2H19 PGX"; $resn_exch{"H201 PGX"}="1H20 PGX"; $resn_exch{"H202 PGX"}="2H20 PGX";
$resn_exch{"H203 PGX"}="3H20 PGX"; $resn_exch{"HO61 PGX"}="1HO6 PGX"; $resn_exch{" H   PGY"}="1HN  PGY";
$resn_exch{" H2  PGY"}="2HN  PGY"; $resn_exch{" H11 PH1"}="1H1  PH1"; $resn_exch{" H12 PH1"}="2H1  PH1";
$resn_exch{" H21 PH1"}="1H2  PH1"; $resn_exch{" H22 PH1"}="2H2  PH1"; $resn_exch{" H41 PH1"}="1H4  PH1";
$resn_exch{" H42 PH1"}="2H4  PH1"; $resn_exch{"H43A PH1"}="3H4  PH1"; $resn_exch{" H51 PH1"}="1H5  PH1";
$resn_exch{"H52A PH1"}="2H5  PH1"; $resn_exch{" H61 PH1"}="1H6  PH1"; $resn_exch{" H62 PH1"}="2H6  PH1";
$resn_exch{" H71 PH1"}="1H7  PH1"; $resn_exch{" H72 PH1"}="2H7  PH1"; $resn_exch{" H91 PH1"}="1H9  PH1";
$resn_exch{" H92 PH1"}="2H9  PH1"; $resn_exch{" H93 PH1"}="3H9  PH1"; $resn_exch{"H101 PH1"}="1H10 PH1";
$resn_exch{"H102 PH1"}="2H10 PH1"; $resn_exch{"H111 PH1"}="1H11 PH1"; $resn_exch{"H112 PH1"}="2H11 PH1";
$resn_exch{"H121 PH1"}="1H12 PH1"; $resn_exch{"H122 PH1"}="2H12 PH1"; $resn_exch{"H141 PH1"}="1H14 PH1";
$resn_exch{"H142 PH1"}="2H14 PH1"; $resn_exch{"H143 PH1"}="3H14 PH1"; $resn_exch{"H151 PH1"}="1H15 PH1";
$resn_exch{"H152 PH1"}="2H15 PH1"; $resn_exch{"H161 PH1"}="1H16 PH1"; $resn_exch{"H162 PH1"}="2H16 PH1";
$resn_exch{"H171 PH1"}="1H17 PH1"; $resn_exch{"H172 PH1"}="2H17 PH1"; $resn_exch{"H191 PH1"}="1H19 PH1";
$resn_exch{"H192 PH1"}="2H19 PH1"; $resn_exch{"H193 PH1"}="3H19 PH1"; $resn_exch{"H201 PH1"}="1H20 PH1";
$resn_exch{"H202 PH1"}="2H20 PH1"; $resn_exch{"H203 PH1"}="3H20 PH1"; $resn_exch{"H311 PH1"}="1H31 PH1";
$resn_exch{"H312 PH1"}="2H31 PH1"; $resn_exch{"H321 PH1"}="1H32 PH1"; $resn_exch{"H322 PH1"}="2H32 PH1";
$resn_exch{"H341 PH1"}="1H34 PH1"; $resn_exch{"H342 PH1"}="2H34 PH1"; $resn_exch{"H343 PH1"}="3H34 PH1";
$resn_exch{"H351 PH1"}="1H35 PH1"; $resn_exch{"H352 PH1"}="2H35 PH1"; $resn_exch{"H361 PH1"}="1H36 PH1";
$resn_exch{"H362 PH1"}="2H36 PH1"; $resn_exch{"H371 PH1"}="1H37 PH1"; $resn_exch{"H372 PH1"}="2H37 PH1";
$resn_exch{"H391 PH1"}="1H39 PH1"; $resn_exch{"H392 PH1"}="2H39 PH1"; $resn_exch{"H393 PH1"}="3H39 PH1";
$resn_exch{"H401 PH1"}="1H40 PH1"; $resn_exch{"H402 PH1"}="2H40 PH1"; $resn_exch{"H411 PH1"}="1H41 PH1";
$resn_exch{"H412 PH1"}="2H41 PH1"; $resn_exch{"H421 PH1"}="1H42 PH1"; $resn_exch{"H422 PH1"}="2H42 PH1";
$resn_exch{"H441 PH1"}="1H44 PH1"; $resn_exch{"H442 PH1"}="2H44 PH1"; $resn_exch{"H443 PH1"}="3H44 PH1";
$resn_exch{"H451 PH1"}="1H45 PH1"; $resn_exch{"H452 PH1"}="2H45 PH1"; $resn_exch{"H461 PH1"}="1H46 PH1";
$resn_exch{"H462 PH1"}="2H46 PH1"; $resn_exch{"H471 PH1"}="1H47 PH1"; $resn_exch{"H472 PH1"}="2H47 PH1";
$resn_exch{"H491 PH1"}="1H49 PH1"; $resn_exch{"H492 PH1"}="2H49 PH1"; $resn_exch{"H493 PH1"}="3H49 PH1";
$resn_exch{"H501 PH1"}="1H50 PH1"; $resn_exch{"H502 PH1"}="2H50 PH1"; $resn_exch{"H503 PH1"}="3H50 PH1";
$resn_exch{"H511 PH1"}="1H51 PH1"; $resn_exch{"H512 PH1"}="2H51 PH1"; $resn_exch{"H531 PH1"}="1H53 PH1";
$resn_exch{"H532 PH1"}="2H53 PH1"; $resn_exch{"H533 PH1"}="3H53 PH1"; $resn_exch{" H31 PH2"}="1H3  PH2";
$resn_exch{" H32 PH2"}="2H3  PH2"; $resn_exch{"HN61 PH2"}="1HN6 PH2"; $resn_exch{"HN62 PH2"}="2HN6 PH2";
$resn_exch{"H111 PH2"}="1H11 PH2"; $resn_exch{"H112 PH2"}="2H11 PH2"; $resn_exch{"H3C1 PH5"}="1H3C PH5";
$resn_exch{"H3C2 PH5"}="2H3C PH5"; $resn_exch{"H111 PH5"}="1H11 PH5"; $resn_exch{"H112 PH5"}="2H11 PH5";
$resn_exch{"H121 PH5"}="1H12 PH5"; $resn_exch{"H122 PH5"}="2H12 PH5"; $resn_exch{"H123 PH5"}="3H12 PH5";
$resn_exch{"H9C1 PH5"}="1H9C PH5"; $resn_exch{"H9C2 PH5"}="2H9C PH5"; $resn_exch{"HBC1 PH5"}="1HBC PH5";
$resn_exch{"HBC2 PH5"}="2HBC PH5"; $resn_exch{" H11 PH9"}="1H1  PH9"; $resn_exch{" H12 PH9"}="2H1  PH9";
$resn_exch{"HC41 PH9"}="1HC4 PH9"; $resn_exch{"HC42 PH9"}="2HC4 PH9"; $resn_exch{"HC21 PH9"}="1HC2 PH9";
$resn_exch{"HC22 PH9"}="2HC2 PH9"; $resn_exch{"HC61 PH9"}="1HC6 PH9"; $resn_exch{"HC62 PH9"}="2HC6 PH9";
$resn_exch{"H101 PH9"}="1H10 PH9"; $resn_exch{"H102 PH9"}="2H10 PH9"; $resn_exch{"HC91 PH9"}="1HC9 PH9";
$resn_exch{"HC92 PH9"}="2HC9 PH9"; $resn_exch{"HC71 PH9"}="1HC7 PH9"; $resn_exch{"HC72 PH9"}="2HC7 PH9";
$resn_exch{" H   PHA"}="1HN  PHA"; $resn_exch{" H2  PHA"}="2HN  PHA"; $resn_exch{" HC  PHA"}=" H   PHA";
$resn_exch{" HB2 PHA"}="1HB  PHA"; $resn_exch{" HB3 PHA"}="2HB  PHA"; $resn_exch{"HO2' PHB"}="'HO2 PHB";
$resn_exch{" HA1 PHC"}="1HA  PHC"; $resn_exch{" HA2 PHC"}="2HA  PHC"; $resn_exch{" HA3 PHC"}="3HA  PHC";
$resn_exch{"HG21 PHC"}="1HG2 PHC"; $resn_exch{"HG22 PHC"}="2HG2 PHC"; $resn_exch{"HG23 PHC"}="3HG2 PHC";
$resn_exch{" H2  PHD"}="2H   PHD"; $resn_exch{" H   PHD"}="1H   PHD"; $resn_exch{" HB2 PHD"}="2HB  PHD";
$resn_exch{" HB3 PHD"}="1HB  PHD"; $resn_exch{" HXT PHD"}=" HO  PHD"; $resn_exch{" H2  PHE"}=" HN2 PHE";
$resn_exch{" HB2 PHE"}="1HB  PHE"; $resn_exch{" HB3 PHE"}="2HB  PHE"; $resn_exch{"HA11 PHF"}="1HA1 PHF";
$resn_exch{"HB11 PHF"}="1HB1 PHF"; $resn_exch{"HB21 PHF"}="1HB2 PHF"; $resn_exch{"HB31 PHF"}="1HB3 PHF";
$resn_exch{"HC11 PHF"}="1HC1 PHF"; $resn_exch{"HC21 PHF"}="1HC2 PHF"; $resn_exch{"HC31 PHF"}="1HC3 PHF";
$resn_exch{"HD11 PHF"}="1HD1 PHF"; $resn_exch{"HE11 PHF"}="1HE1 PHF"; $resn_exch{"HE21 PHF"}="1HE2 PHF";
$resn_exch{"HE31 PHF"}="1HE3 PHF"; $resn_exch{" H31 PHH"}="1H3  PHH"; $resn_exch{" H32 PHH"}="2H3  PHH";
$resn_exch{" H   PHI"}="1HN  PHI"; $resn_exch{" H2  PHI"}="2HN  PHI"; $resn_exch{" HB2 PHI"}="1HB  PHI";
$resn_exch{" HB3 PHI"}="2HB  PHI"; $resn_exch{" H21 PHJ"}="1H2  PHJ"; $resn_exch{" H22 PHJ"}="2H2  PHJ";
$resn_exch{" H   PHL"}="1HN  PHL"; $resn_exch{" H2  PHL"}="2HN  PHL"; $resn_exch{" HC1 PHL"}=" H1  PHL";
$resn_exch{" HC2 PHL"}=" H2  PHL"; $resn_exch{" HB2 PHL"}="1HB  PHL"; $resn_exch{" HB3 PHL"}="2HB  PHL";
$resn_exch{" HM1 PHM"}="1HM  PHM"; $resn_exch{" HM2 PHM"}="2HM  PHM"; $resn_exch{" HM3 PHM"}="3HM  PHM";
$resn_exch{" H   PHM"}="1HN  PHM"; $resn_exch{" H2  PHM"}="2HN  PHM"; $resn_exch{" HB2 PHM"}="1HB  PHM";
$resn_exch{" HB3 PHM"}="2HB  PHM"; $resn_exch{"HMA1 PHO"}="1HMA PHO"; $resn_exch{"HMA2 PHO"}="2HMA PHO";
$resn_exch{"HMA3 PHO"}="3HMA PHO"; $resn_exch{"HAA1 PHO"}="1HAA PHO"; $resn_exch{"HAA2 PHO"}="2HAA PHO";
$resn_exch{"HBA1 PHO"}="1HBA PHO"; $resn_exch{"HBA2 PHO"}="2HBA PHO"; $resn_exch{"HMB1 PHO"}="1HMB PHO";
$resn_exch{"HMB2 PHO"}="2HMB PHO"; $resn_exch{"HMB3 PHO"}="3HMB PHO"; $resn_exch{"HBB1 PHO"}="1HBB PHO";
$resn_exch{"HBB2 PHO"}="2HBB PHO"; $resn_exch{"HMC1 PHO"}="1HMC PHO"; $resn_exch{"HMC2 PHO"}="2HMC PHO";
$resn_exch{"HMC3 PHO"}="3HMC PHO"; $resn_exch{"HAC1 PHO"}="1HAC PHO"; $resn_exch{"HAC2 PHO"}="2HAC PHO";
$resn_exch{"HBC1 PHO"}="1HBC PHO"; $resn_exch{"HBC2 PHO"}="2HBC PHO"; $resn_exch{"HBC3 PHO"}="3HBC PHO";
$resn_exch{"HMD1 PHO"}="1HMD PHO"; $resn_exch{"HMD2 PHO"}="2HMD PHO"; $resn_exch{"HMD3 PHO"}="3HMD PHO";
$resn_exch{"HED1 PHO"}="1HED PHO"; $resn_exch{"HED2 PHO"}="2HED PHO"; $resn_exch{"HED3 PHO"}="3HED PHO";
$resn_exch{" H11 PHO"}="1H1  PHO"; $resn_exch{" H12 PHO"}="2H1  PHO"; $resn_exch{" H41 PHO"}="1H4  PHO";
$resn_exch{" H42 PHO"}="2H4  PHO"; $resn_exch{" H43 PHO"}="3H4  PHO"; $resn_exch{" H51 PHO"}="1H5  PHO";
$resn_exch{" H52 PHO"}="2H5  PHO"; $resn_exch{" H61 PHO"}="1H6  PHO"; $resn_exch{" H62 PHO"}="2H6  PHO";
$resn_exch{" H71 PHO"}="1H7  PHO"; $resn_exch{" H72 PHO"}="2H7  PHO"; $resn_exch{" H91 PHO"}="1H9  PHO";
$resn_exch{" H92 PHO"}="2H9  PHO"; $resn_exch{" H93 PHO"}="3H9  PHO"; $resn_exch{"H101 PHO"}="1H10 PHO";
$resn_exch{"H102 PHO"}="2H10 PHO"; $resn_exch{"H111 PHO"}="1H11 PHO"; $resn_exch{"H112 PHO"}="2H11 PHO";
$resn_exch{"H121 PHO"}="1H12 PHO"; $resn_exch{"H122 PHO"}="2H12 PHO"; $resn_exch{"H141 PHO"}="1H14 PHO";
$resn_exch{"H142 PHO"}="2H14 PHO"; $resn_exch{"H143 PHO"}="3H14 PHO"; $resn_exch{"H151 PHO"}="1H15 PHO";
$resn_exch{"H152 PHO"}="2H15 PHO"; $resn_exch{"H161 PHO"}="1H16 PHO"; $resn_exch{"H162 PHO"}="2H16 PHO";
$resn_exch{"H171 PHO"}="1H17 PHO"; $resn_exch{"H172 PHO"}="2H17 PHO"; $resn_exch{"H191 PHO"}="1H19 PHO";
$resn_exch{"H192 PHO"}="2H19 PHO"; $resn_exch{"H193 PHO"}="3H19 PHO"; $resn_exch{"H201 PHO"}="1H20 PHO";
$resn_exch{"H202 PHO"}="2H20 PHO"; $resn_exch{"H203 PHO"}="3H20 PHO"; $resn_exch{"HB11 PHP"}="1HB1 PHP";
$resn_exch{"HB12 PHP"}="2HB1 PHP"; $resn_exch{"HD11 PHP"}="1HD1 PHP"; $resn_exch{"HD12 PHP"}="2HD1 PHP";
$resn_exch{"HE11 PHP"}="1HE1 PHP"; $resn_exch{"HE12 PHP"}="2HE1 PHP"; $resn_exch{"HG21 PHP"}="1HG2 PHP";
$resn_exch{"HG22 PHP"}="2HG2 PHP"; $resn_exch{"HG23 PHP"}="3HG2 PHP"; $resn_exch{"HG24 PHP"}="4HG2 PHP";
$resn_exch{"HG25 PHP"}="5HG2 PHP"; $resn_exch{"HG26 PHP"}="6HG2 PHP"; $resn_exch{"HB31 PHP"}="1HB3 PHP";
$resn_exch{"HB32 PHP"}="2HB3 PHP"; $resn_exch{"HD31 PHP"}="1HD3 PHP"; $resn_exch{"HD32 PHP"}="2HD3 PHP";
$resn_exch{"HE31 PHP"}="1HE3 PHP"; $resn_exch{"HE32 PHP"}="2HE3 PHP"; $resn_exch{"HO3' PHP"}="'HO3 PHP";
$resn_exch{"HB41 PHP"}="1HB4 PHP"; $resn_exch{"HB42 PHP"}="2HB4 PHP"; $resn_exch{"HD41 PHP"}="1HD4 PHP";
$resn_exch{"HD42 PHP"}="2HD4 PHP"; $resn_exch{"HE41 PHP"}="1HE4 PHP"; $resn_exch{"HE42 PHP"}="2HE4 PHP";
$resn_exch{"HG51 PHP"}="1HG5 PHP"; $resn_exch{"HG52 PHP"}="2HG5 PHP"; $resn_exch{"HG53 PHP"}="3HG5 PHP";
$resn_exch{"HG54 PHP"}="4HG5 PHP"; $resn_exch{"HG55 PHP"}="5HG5 PHP"; $resn_exch{"HG56 PHP"}="6HG5 PHP";
$resn_exch{"HB61 PHP"}="1HB6 PHP"; $resn_exch{"HB62 PHP"}="2HB6 PHP"; $resn_exch{"HD61 PHP"}="1HD6 PHP";
$resn_exch{"HE61 PHP"}="1HE6 PHP"; $resn_exch{"HE62 PHP"}="2HE6 PHP"; $resn_exch{"HD62 PHP"}="2HD6 PHP";
$resn_exch{" H11 PHQ"}="1H1  PHQ"; $resn_exch{" H21 PHQ"}="1H2  PHQ"; $resn_exch{" H22 PHQ"}="2H2  PHQ";
$resn_exch{" H41 PHQ"}="1H4  PHQ"; $resn_exch{" H51 PHQ"}="1H5  PHQ"; $resn_exch{" H61 PHQ"}="1H6  PHQ";
$resn_exch{" H71 PHQ"}="1H7  PHQ"; $resn_exch{" H81 PHQ"}="1H8  PHQ"; $resn_exch{"H201 PHR"}="1H20 PHR";
$resn_exch{"H202 PHR"}="2H20 PHR"; $resn_exch{"H191 PHR"}="1H19 PHR"; $resn_exch{"H192 PHR"}="2H19 PHR";
$resn_exch{"H181 PHR"}="1H18 PHR"; $resn_exch{"H182 PHR"}="2H18 PHR"; $resn_exch{"H171 PHR"}="1H17 PHR";
$resn_exch{"H172 PHR"}="2H17 PHR"; $resn_exch{"H161 PHR"}="1H16 PHR"; $resn_exch{"H162 PHR"}="2H16 PHR";
$resn_exch{"H151 PHR"}="1H15 PHR"; $resn_exch{"H152 PHR"}="2H15 PHR"; $resn_exch{"H101 PHR"}="1H10 PHR";
$resn_exch{"H102 PHR"}="2H10 PHR"; $resn_exch{"H141 PHR"}="1H14 PHR"; $resn_exch{"H142 PHR"}="2H14 PHR";
$resn_exch{"HO2P PHS"}="PHO2 PHS"; $resn_exch{"HO3P PHS"}="PHO3 PHS"; $resn_exch{"HOA1 PHW"}="1HOA PHW";
$resn_exch{"HC11 PHW"}="1HC1 PHW"; $resn_exch{"HC12 PHW"}="2HC1 PHW"; $resn_exch{"HC13 PHW"}="3HC1 PHW";
$resn_exch{"HC41 PHW"}="1HC4 PHW"; $resn_exch{"HC42 PHW"}="2HC4 PHW"; $resn_exch{"H161 PHW"}="1H16 PHW";
$resn_exch{"H162 PHW"}="2H16 PHW"; $resn_exch{"HOC1 PHW"}="1HOC PHW"; $resn_exch{"H611 PHW"}="1H61 PHW";
$resn_exch{"H612 PHW"}="2H61 PHW"; $resn_exch{"H613 PHW"}="3H61 PHW"; $resn_exch{"H8A1 PHW"}="1H8A PHW";
$resn_exch{"H8A2 PHW"}="2H8A PHW"; $resn_exch{"H8B1 PHW"}="1H8B PHW"; $resn_exch{"H8B2 PHW"}="2H8B PHW";
$resn_exch{"H8C1 PHW"}="1H8C PHW"; $resn_exch{"H8C2 PHW"}="2H8C PHW"; $resn_exch{"HN31 PHW"}="1HN3 PHW";
$resn_exch{"HAH1 PHX"}="1HAH PHX"; $resn_exch{"HAH2 PHX"}="2HAH PHX"; $resn_exch{" H21 PHY"}="1H2  PHY";
$resn_exch{" H22 PHY"}="2H2  PHY"; $resn_exch{" H23 PHY"}="3H2  PHY"; $resn_exch{"H1'1 PHY"}="1H1' PHY";
$resn_exch{"H1'2 PHY"}="2H1' PHY"; $resn_exch{"H3'1 PHY"}="1H3' PHY"; $resn_exch{"H3'2 PHY"}="2H3' PHY";
$resn_exch{"H3'3 PHY"}="3H3' PHY"; $resn_exch{"HN11 PHY"}="1HN1 PHY"; $resn_exch{"HN12 PHY"}="2HN1 PHY";
$resn_exch{"HO4P PHY"}="PHO4 PHY"; $resn_exch{"HO5P PHY"}="PHO5 PHY"; $resn_exch{"HN11 PHZ"}="1HN1 PHZ";
$resn_exch{"HN12 PHZ"}="2HN1 PHZ"; $resn_exch{" H11 PI1"}="1H1  PI1"; $resn_exch{" H12 PI1"}="2H1  PI1";
$resn_exch{" H41 PI1"}="1H4  PI1"; $resn_exch{"H42A PI1"}="2H4  PI1"; $resn_exch{" H91 PI1"}="1H9  PI1";
$resn_exch{" H92 PI1"}="2H9  PI1"; $resn_exch{"H121 PI1"}="1H12 PI1"; $resn_exch{"H122 PI1"}="2H12 PI1";
$resn_exch{"H151 PI1"}="1H15 PI1"; $resn_exch{"H152 PI1"}="2H15 PI1"; $resn_exch{"H231 PI1"}="1H23 PI1";
$resn_exch{"H232 PI1"}="2H23 PI1"; $resn_exch{"H261 PI1"}="1H26 PI1"; $resn_exch{"H262 PI1"}="2H26 PI1";
$resn_exch{"H311 PI1"}="1H31 PI1"; $resn_exch{"H312 PI1"}="2H31 PI1"; $resn_exch{"H321 PI1"}="1H32 PI1";
$resn_exch{"H322 PI1"}="2H32 PI1"; $resn_exch{"H331 PI1"}="1H33 PI1"; $resn_exch{"H332 PI1"}="2H33 PI1";
$resn_exch{"H391 PI1"}="1H39 PI1"; $resn_exch{"H392 PI1"}="2H39 PI1"; $resn_exch{"H401 PI1"}="1H40 PI1";
$resn_exch{"H402 PI1"}="2H40 PI1"; $resn_exch{"H403 PI1"}="3H40 PI1"; $resn_exch{"H411 PI1"}="1H41 PI1";
$resn_exch{"H412 PI1"}="2H41 PI1"; $resn_exch{"H413 PI1"}="3H41 PI1"; $resn_exch{"H471 PI1"}="1H47 PI1";
$resn_exch{"H472 PI1"}="2H47 PI1"; $resn_exch{"H473 PI1"}="3H47 PI1"; $resn_exch{"H481 PI1"}="1H48 PI1";
$resn_exch{"H482 PI1"}="2H48 PI1"; $resn_exch{"H483 PI1"}="3H48 PI1"; $resn_exch{"H491 PI1"}="1H49 PI1";
$resn_exch{"H492 PI1"}="2H49 PI1"; $resn_exch{" H11 PI2"}="1H1  PI2"; $resn_exch{" H12 PI2"}="2H1  PI2";
$resn_exch{" H41 PI2"}="1H4  PI2"; $resn_exch{" H42 PI2"}="2H4  PI2"; $resn_exch{" H91 PI2"}="1H9  PI2";
$resn_exch{" H92 PI2"}="2H9  PI2"; $resn_exch{"H121 PI2"}="1H12 PI2"; $resn_exch{"H122 PI2"}="2H12 PI2";
$resn_exch{"H151 PI2"}="1H15 PI2"; $resn_exch{"H152 PI2"}="2H15 PI2"; $resn_exch{"H231 PI2"}="1H23 PI2";
$resn_exch{"H232 PI2"}="2H23 PI2"; $resn_exch{"H261 PI2"}="1H26 PI2"; $resn_exch{"H262 PI2"}="2H26 PI2";
$resn_exch{"H281 PI2"}="1H28 PI2"; $resn_exch{"H282 PI2"}="2H28 PI2"; $resn_exch{"H291 PI2"}="1H29 PI2";
$resn_exch{"H292 PI2"}="2H29 PI2"; $resn_exch{"H311 PI2"}="1H31 PI2"; $resn_exch{"H312 PI2"}="2H31 PI2";
$resn_exch{"H313 PI2"}="3H31 PI2"; $resn_exch{"H321 PI2"}="1H32 PI2"; $resn_exch{"H322 PI2"}="2H32 PI2";
$resn_exch{"H323 PI2"}="3H32 PI2"; $resn_exch{" H11 PI3"}="1H1  PI3"; $resn_exch{"H12A PI3"}="2H1  PI3";
$resn_exch{" H41 PI3"}="1H4  PI3"; $resn_exch{" H42 PI3"}="2H4  PI3"; $resn_exch{"H101 PI3"}="1H10 PI3";
$resn_exch{"H102 PI3"}="2H10 PI3"; $resn_exch{"H103 PI3"}="3H10 PI3"; $resn_exch{"H111 PI3"}="1H11 PI3";
$resn_exch{"H112 PI3"}="2H11 PI3"; $resn_exch{"H113 PI3"}="3H11 PI3"; $resn_exch{"H141 PI3"}="1H14 PI3";
$resn_exch{"H142 PI3"}="2H14 PI3"; $resn_exch{"H221 PI3"}="1H22 PI3"; $resn_exch{"H222 PI3"}="2H22 PI3";
$resn_exch{"H251 PI3"}="1H25 PI3"; $resn_exch{"H252 PI3"}="2H25 PI3"; $resn_exch{"H271 PI3"}="1H27 PI3";
$resn_exch{"H272 PI3"}="2H27 PI3"; $resn_exch{"H281 PI3"}="1H28 PI3"; $resn_exch{"H282 PI3"}="2H28 PI3";
$resn_exch{"H301 PI3"}="1H30 PI3"; $resn_exch{"H302 PI3"}="2H30 PI3"; $resn_exch{"H303 PI3"}="3H30 PI3";
$resn_exch{"H311 PI3"}="1H31 PI3"; $resn_exch{"H312 PI3"}="2H31 PI3"; $resn_exch{"H313 PI3"}="3H31 PI3";
$resn_exch{" H11 PI4"}="1H1  PI4"; $resn_exch{" H12 PI4"}="2H1  PI4"; $resn_exch{" H41 PI4"}="1H4  PI4";
$resn_exch{" H42 PI4"}="2H4  PI4"; $resn_exch{" H91 PI4"}="1H9  PI4"; $resn_exch{" H92 PI4"}="2H9  PI4";
$resn_exch{"H121 PI4"}="1H12 PI4"; $resn_exch{"H122 PI4"}="2H12 PI4"; $resn_exch{"H151 PI4"}="1H15 PI4";
$resn_exch{"H152 PI4"}="2H15 PI4"; $resn_exch{"H231 PI4"}="1H23 PI4"; $resn_exch{"H232 PI4"}="2H23 PI4";
$resn_exch{"H261 PI4"}="1H26 PI4"; $resn_exch{"H262 PI4"}="2H26 PI4"; $resn_exch{"H291 PI4"}="1H29 PI4";
$resn_exch{"H292 PI4"}="2H29 PI4"; $resn_exch{"H301 PI4"}="1H30 PI4"; $resn_exch{"H302 PI4"}="2H30 PI4";
$resn_exch{"H311 PI4"}="1H31 PI4"; $resn_exch{"H312 PI4"}="2H31 PI4"; $resn_exch{"H321 PI4"}="1H32 PI4";
$resn_exch{"H322 PI4"}="2H32 PI4"; $resn_exch{"H361 PI4"}="1H36 PI4"; $resn_exch{"H362 PI4"}="2H36 PI4";
$resn_exch{"H363 PI4"}="3H36 PI4"; $resn_exch{"H371 PI4"}="1H37 PI4"; $resn_exch{"H372 PI4"}="2H37 PI4";
$resn_exch{"H373 PI4"}="3H37 PI4"; $resn_exch{"H381 PI4"}="1H38 PI4"; $resn_exch{"H382 PI4"}="2H38 PI4";
$resn_exch{"H383 PI4"}="3H38 PI4"; $resn_exch{" H11 PI5"}="1H1  PI5"; $resn_exch{" H12 PI5"}="2H1  PI5";
$resn_exch{" H21 PI5"}="1H2  PI5"; $resn_exch{"H22A PI5"}="2H2  PI5"; $resn_exch{" H51 PI5"}="1H5  PI5";
$resn_exch{" H52 PI5"}="2H5  PI5"; $resn_exch{" H91 PI5"}="1H9  PI5"; $resn_exch{" H92 PI5"}="2H9  PI5";
$resn_exch{" H93 PI5"}="3H9  PI5"; $resn_exch{"H101 PI5"}="1H10 PI5"; $resn_exch{"H102 PI5"}="2H10 PI5";
$resn_exch{"H103 PI5"}="3H10 PI5"; $resn_exch{"H151 PI5"}="1H15 PI5"; $resn_exch{"H152 PI5"}="2H15 PI5";
$resn_exch{"H231 PI5"}="1H23 PI5"; $resn_exch{"H232 PI5"}="2H23 PI5"; $resn_exch{"H281 PI5"}="1H28 PI5";
$resn_exch{"H282 PI5"}="2H28 PI5"; $resn_exch{"H361 PI5"}="1H36 PI5"; $resn_exch{"H362 PI5"}="2H36 PI5";
$resn_exch{"H441 PI5"}="1H44 PI5"; $resn_exch{"H442 PI5"}="2H44 PI5"; $resn_exch{"H451 PI5"}="1H45 PI5";
$resn_exch{"H452 PI5"}="2H45 PI5"; $resn_exch{"H453 PI5"}="3H45 PI5"; $resn_exch{"H461 PI5"}="1H46 PI5";
$resn_exch{"H462 PI5"}="2H46 PI5"; $resn_exch{"H463 PI5"}="3H46 PI5"; $resn_exch{"H481 PI5"}="1H48 PI5";
$resn_exch{"H482 PI5"}="2H48 PI5"; $resn_exch{"H491 PI5"}="1H49 PI5"; $resn_exch{"H492 PI5"}="2H49 PI5";
$resn_exch{" H31 PI6"}="1H3  PI6"; $resn_exch{" H32 PI6"}="2H3  PI6"; $resn_exch{"H33A PI6"}="3H3  PI6";
$resn_exch{"H41A PI6"}="1H4  PI6"; $resn_exch{" H42 PI6"}="2H4  PI6"; $resn_exch{" H43 PI6"}="3H4  PI6";
$resn_exch{" H51 PI6"}="1H5  PI6"; $resn_exch{" H52 PI6"}="2H5  PI6"; $resn_exch{" H53 PI6"}="3H5  PI6";
$resn_exch{"H101 PI6"}="1H10 PI6"; $resn_exch{"H102 PI6"}="2H10 PI6"; $resn_exch{"H191 PI6"}="1H19 PI6";
$resn_exch{"H192 PI6"}="2H19 PI6"; $resn_exch{"H221 PI6"}="1H22 PI6"; $resn_exch{"H222 PI6"}="2H22 PI6";
$resn_exch{"H301 PI6"}="1H30 PI6"; $resn_exch{"H302 PI6"}="2H30 PI6"; $resn_exch{"H381 PI6"}="1H38 PI6";
$resn_exch{"H382 PI6"}="2H38 PI6"; $resn_exch{"H391 PI6"}="1H39 PI6"; $resn_exch{"H392 PI6"}="2H39 PI6";
$resn_exch{"H393 PI6"}="3H39 PI6"; $resn_exch{"H401 PI6"}="1H40 PI6"; $resn_exch{"H402 PI6"}="2H40 PI6";
$resn_exch{"H403 PI6"}="3H40 PI6"; $resn_exch{"H421 PI6"}="1H42 PI6"; $resn_exch{"H422 PI6"}="2H42 PI6";
$resn_exch{"H431 PI6"}="1H43 PI6"; $resn_exch{"H432 PI6"}="2H43 PI6"; $resn_exch{" H11 PI7"}="1H1  PI7";
$resn_exch{"H12A PI7"}="2H1  PI7"; $resn_exch{"H13A PI7"}="3H1  PI7"; $resn_exch{"H21A PI7"}="1H2  PI7";
$resn_exch{" H22 PI7"}="2H2  PI7"; $resn_exch{"H101 PI7"}="1H10 PI7"; $resn_exch{"H102 PI7"}="2H10 PI7";
$resn_exch{"H103 PI7"}="3H10 PI7"; $resn_exch{"H111 PI7"}="1H11 PI7"; $resn_exch{"H112 PI7"}="2H11 PI7";
$resn_exch{"H113 PI7"}="3H11 PI7"; $resn_exch{"H141 PI7"}="1H14 PI7"; $resn_exch{"H142 PI7"}="2H14 PI7";
$resn_exch{"H221 PI7"}="1H22 PI7"; $resn_exch{"H222 PI7"}="2H22 PI7"; $resn_exch{"H271 PI7"}="1H27 PI7";
$resn_exch{"H272 PI7"}="2H27 PI7"; $resn_exch{"H351 PI7"}="1H35 PI7"; $resn_exch{"H352 PI7"}="2H35 PI7";
$resn_exch{"H431 PI7"}="1H43 PI7"; $resn_exch{"H432 PI7"}="2H43 PI7"; $resn_exch{"H441 PI7"}="1H44 PI7";
$resn_exch{"H442 PI7"}="2H44 PI7"; $resn_exch{"H443 PI7"}="3H44 PI7"; $resn_exch{"H451 PI7"}="1H45 PI7";
$resn_exch{"H452 PI7"}="2H45 PI7"; $resn_exch{"H453 PI7"}="3H45 PI7"; $resn_exch{"H471 PI7"}="1H47 PI7";
$resn_exch{"H472 PI7"}="2H47 PI7"; $resn_exch{"H481 PI7"}="1H48 PI7"; $resn_exch{"H482 PI7"}="2H48 PI7";
$resn_exch{"HN11 PI8"}="1HN1 PI8"; $resn_exch{" H21 PI8"}="1H2  PI8"; $resn_exch{" H11 PI8"}="1H1  PI8";
$resn_exch{"HO11 PI8"}="1HO1 PI8"; $resn_exch{" H91 PI8"}="1H9  PI8"; $resn_exch{" H92 PI8"}="2H9  PI8";
$resn_exch{" H41 PI8"}="1H4  PI8"; $resn_exch{" H71 PI8"}="1H7  PI8"; $resn_exch{" H51 PI8"}="1H5  PI8";
$resn_exch{" H61 PI8"}="1H6  PI8"; $resn_exch{"H111 PI8"}="1H11 PI8"; $resn_exch{"H121 PI8"}="1H12 PI8";
$resn_exch{"H122 PI8"}="2H12 PI8"; $resn_exch{"H181 PI8"}="1H18 PI8"; $resn_exch{"H141 PI8"}="1H14 PI8";
$resn_exch{"H171 PI8"}="1H17 PI8"; $resn_exch{"H151 PI8"}="1H15 PI8"; $resn_exch{"H161 PI8"}="1H16 PI8";
$resn_exch{"H191 PI8"}="1H19 PI8"; $resn_exch{"H192 PI8"}="2H19 PI8"; $resn_exch{"HO31 PI8"}="1HO3 PI8";
$resn_exch{"H201 PI8"}="1H20 PI8"; $resn_exch{"H211 PI8"}="1H21 PI8"; $resn_exch{"H212 PI8"}="2H21 PI8";
$resn_exch{"HN21 PI8"}="1HN2 PI8"; $resn_exch{"H221 PI8"}="1H22 PI8"; $resn_exch{"H231 PI8"}="1H23 PI8";
$resn_exch{"H232 PI8"}="2H23 PI8"; $resn_exch{"H251 PI8"}="1H25 PI8"; $resn_exch{"H261 PI8"}="1H26 PI8";
$resn_exch{"H291 PI8"}="1H29 PI8"; $resn_exch{"H281 PI8"}="1H28 PI8"; $resn_exch{"HN31 PI8"}="1HN3 PI8";
$resn_exch{"H311 PI8"}="1H31 PI8"; $resn_exch{"H321 PI8"}="1H32 PI8"; $resn_exch{"H331 PI8"}="1H33 PI8";
$resn_exch{"H332 PI8"}="2H33 PI8"; $resn_exch{"H333 PI8"}="3H33 PI8"; $resn_exch{"H341 PI8"}="1H34 PI8";
$resn_exch{"H342 PI8"}="2H34 PI8"; $resn_exch{"H351 PI8"}="1H35 PI8"; $resn_exch{"H352 PI8"}="2H35 PI8";
$resn_exch{"H353 PI8"}="3H35 PI8"; $resn_exch{"H41N PI8"}="1NH4 PI8"; $resn_exch{"H371 PI8"}="1H37 PI8";
$resn_exch{"H372 PI8"}="2H37 PI8"; $resn_exch{"H381 PI8"}="1H38 PI8"; $resn_exch{"H382 PI8"}="2H38 PI8";
$resn_exch{"H391 PI8"}="1H39 PI8"; $resn_exch{"H392 PI8"}="2H39 PI8"; $resn_exch{"H401 PI8"}="1H40 PI8";
$resn_exch{"H402 PI8"}="2H40 PI8"; $resn_exch{"H411 PI8"}="1H41 PI8"; $resn_exch{"H412 PI8"}="2H41 PI8";
$resn_exch{" H41 PI9"}="1H4  PI9"; $resn_exch{" H42 PI9"}="2H4  PI9"; $resn_exch{" H51 PI9"}="1H5  PI9";
$resn_exch{" H52 PI9"}="2H5  PI9"; $resn_exch{" H31 PI9"}="1H3  PI9"; $resn_exch{" H32 PI9"}="2H3  PI9";
$resn_exch{" H21 PI9"}="1H2  PI9"; $resn_exch{" H22 PI9"}="2H2  PI9"; $resn_exch{"HN11 PI9"}="1HN1 PI9";
$resn_exch{" H71 PI9"}="1H7  PI9"; $resn_exch{" H81 PI9"}="1H8  PI9"; $resn_exch{"H101 PI9"}="1H10 PI9";
$resn_exch{"H102 PI9"}="2H10 PI9"; $resn_exch{"H103 PI9"}="3H10 PI9"; $resn_exch{" H91 PI9"}="1H9  PI9";
$resn_exch{" H92 PI9"}="2H9  PI9"; $resn_exch{" H93 PI9"}="3H9  PI9"; $resn_exch{"HN21 PI9"}="1HN2 PI9";
$resn_exch{"H121 PI9"}="1H12 PI9"; $resn_exch{"H131 PI9"}="1H13 PI9"; $resn_exch{"H132 PI9"}="2H13 PI9";
$resn_exch{"H151 PI9"}="1H15 PI9"; $resn_exch{"H161 PI9"}="1H16 PI9"; $resn_exch{"H191 PI9"}="1H19 PI9";
$resn_exch{"H181 PI9"}="1H18 PI9"; $resn_exch{" H11 PI9"}="1H1  PI9"; $resn_exch{" H12 PI9"}="2H1  PI9";
$resn_exch{"H211 PI9"}="1H21 PI9"; $resn_exch{"H212 PI9"}="2H21 PI9"; $resn_exch{"H221 PI9"}="1H22 PI9";
$resn_exch{"H222 PI9"}="2H22 PI9"; $resn_exch{"H231 PI9"}="1H23 PI9"; $resn_exch{"H232 PI9"}="2H23 PI9";
$resn_exch{"H241 PI9"}="1H24 PI9"; $resn_exch{"H261 PI9"}="1H26 PI9"; $resn_exch{"H262 PI9"}="2H26 PI9";
$resn_exch{"H263 PI9"}="3H26 PI9"; $resn_exch{"H251 PI9"}="1H25 PI9"; $resn_exch{"H252 PI9"}="2H25 PI9";
$resn_exch{"H253 PI9"}="3H25 PI9"; $resn_exch{"H321 PI9"}="1H32 PI9"; $resn_exch{"H281 PI9"}="1H28 PI9";
$resn_exch{"H311 PI9"}="1H31 PI9"; $resn_exch{"H291 PI9"}="1H29 PI9"; $resn_exch{"HN41 PI9"}="1HN4 PI9";
$resn_exch{"HN42 PI9"}="2HN4 PI9"; $resn_exch{"HN11 PIA"}="1HN1 PIA"; $resn_exch{"HN12 PIA"}="2HN1 PIA";
$resn_exch{"HB11 PIA"}="1HB1 PIA"; $resn_exch{"HB12 PIA"}="2HB1 PIA"; $resn_exch{"HB13 PIA"}="3HB1 PIA";
$resn_exch{"HA31 PIA"}="1HA3 PIA"; $resn_exch{"HA32 PIA"}="2HA3 PIA"; $resn_exch{" HC1 PIB"}="1HC  PIB";
$resn_exch{" HC2 PIB"}="2HC  PIB"; $resn_exch{" HC3 PIB"}="3HC  PIB"; $resn_exch{" HC4 PIB"}="4HC  PIB";
$resn_exch{" HC5 PIB"}="5HC  PIB"; $resn_exch{" HC6 PIB"}="6HC  PIB"; $resn_exch{" HO2 PIB"}="2HO  PIB";
$resn_exch{" HO4 PIB"}="4HO  PIB"; $resn_exch{" HO5 PIB"}="5HO  PIB"; $resn_exch{" HO6 PIB"}="6HO  PIB";
$resn_exch{"HO11 PIB"}="11HO PIB"; $resn_exch{"HO31 PIB"}="31HO PIB"; $resn_exch{"HO33 PIB"}="33HO PIB";
$resn_exch{"HC71 PIB"}="1HC7 PIB"; $resn_exch{"HC72 PIB"}="2HC7 PIB"; $resn_exch{" HC8 PIB"}="8HC  PIB";
$resn_exch{"HC91 PIB"}="1HC9 PIB"; $resn_exch{"HC92 PIB"}="2HC9 PIB"; $resn_exch{"H111 PIB"}="1H11 PIB";
$resn_exch{"H112 PIB"}="2H11 PIB"; $resn_exch{"H121 PIB"}="1H12 PIB"; $resn_exch{"H122 PIB"}="2H12 PIB";
$resn_exch{"H131 PIB"}="1H13 PIB"; $resn_exch{"H132 PIB"}="2H13 PIB"; $resn_exch{"H133 PIB"}="3H13 PIB";
$resn_exch{"H151 PIB"}="1H15 PIB"; $resn_exch{"H152 PIB"}="2H15 PIB"; $resn_exch{"H161 PIB"}="1H16 PIB";
$resn_exch{"H162 PIB"}="2H16 PIB"; $resn_exch{"H171 PIB"}="1H17 PIB"; $resn_exch{"H172 PIB"}="2H17 PIB";
$resn_exch{"H173 PIB"}="3H17 PIB"; $resn_exch{" H21 PID"}="1H2  PID"; $resn_exch{"H22A PID"}="2H2  PID";
$resn_exch{" H41 PID"}="1H4  PID"; $resn_exch{" H42 PID"}="2H4  PID"; $resn_exch{"H261 PID"}="1H26 PID";
$resn_exch{"H262 PID"}="2H26 PID"; $resn_exch{"H281 PID"}="1H28 PID"; $resn_exch{"H282 PID"}="2H28 PID";
$resn_exch{"H311 PID"}="1H31 PID"; $resn_exch{"H312 PID"}="2H31 PID"; $resn_exch{"H313 PID"}="3H31 PID";
$resn_exch{"HM11 PID"}="1HM1 PID"; $resn_exch{"HM12 PID"}="2HM1 PID"; $resn_exch{"HM13 PID"}="3HM1 PID";
$resn_exch{"HM21 PID"}="1HM2 PID"; $resn_exch{"HM22 PID"}="2HM2 PID"; $resn_exch{"HM23 PID"}="3HM2 PID";
$resn_exch{"HM31 PID"}="1HM3 PID"; $resn_exch{"HM32 PID"}="2HM3 PID"; $resn_exch{"HM33 PID"}="3HM3 PID";
$resn_exch{"HM41 PID"}="1HM4 PID"; $resn_exch{"HM42 PID"}="2HM4 PID"; $resn_exch{"HM43 PID"}="3HM4 PID";
$resn_exch{"HM51 PID"}="1HM5 PID"; $resn_exch{"HM52 PID"}="2HM5 PID"; $resn_exch{"HM53 PID"}="3HM5 PID";
$resn_exch{"HM61 PID"}="1HM6 PID"; $resn_exch{"HM62 PID"}="2HM6 PID"; $resn_exch{"HM63 PID"}="3HM6 PID";
$resn_exch{"HM71 PID"}="1HM7 PID"; $resn_exch{"HM72 PID"}="2HM7 PID"; $resn_exch{"HM73 PID"}="3HM7 PID";
$resn_exch{"HM81 PID"}="1HM8 PID"; $resn_exch{"HM82 PID"}="2HM8 PID"; $resn_exch{"HM83 PID"}="3HM8 PID";
$resn_exch{" H11 PIE"}="1H1  PIE"; $resn_exch{" H12 PIE"}="2H1  PIE"; $resn_exch{" H21 PIE"}="1H2  PIE";
$resn_exch{" H31 PIE"}="1H3  PIE"; $resn_exch{" H32 PIE"}="2H3  PIE"; $resn_exch{"H321 PIE"}="1H32 PIE";
$resn_exch{"H322 PIE"}="2H32 PIE"; $resn_exch{"H331 PIE"}="1H33 PIE"; $resn_exch{"H332 PIE"}="2H33 PIE";
$resn_exch{"H341 PIE"}="1H34 PIE"; $resn_exch{"H342 PIE"}="2H34 PIE"; $resn_exch{"H351 PIE"}="1H35 PIE";
$resn_exch{"H352 PIE"}="2H35 PIE"; $resn_exch{"H361 PIE"}="1H36 PIE"; $resn_exch{"H362 PIE"}="2H36 PIE";
$resn_exch{"H371 PIE"}="1H37 PIE"; $resn_exch{"H372 PIE"}="2H37 PIE"; $resn_exch{"H381 PIE"}="1H38 PIE";
$resn_exch{"H382 PIE"}="2H38 PIE"; $resn_exch{"H391 PIE"}="1H39 PIE"; $resn_exch{"H392 PIE"}="2H39 PIE";
$resn_exch{"H401 PIE"}="1H40 PIE"; $resn_exch{"H402 PIE"}="2H40 PIE"; $resn_exch{"H411 PIE"}="1H41 PIE";
$resn_exch{"H412 PIE"}="2H41 PIE"; $resn_exch{"H421 PIE"}="1H42 PIE"; $resn_exch{"H422 PIE"}="2H42 PIE";
$resn_exch{"H221 PIE"}="1H22 PIE"; $resn_exch{"H222 PIE"}="2H22 PIE"; $resn_exch{"H231 PIE"}="1H23 PIE";
$resn_exch{"H232 PIE"}="2H23 PIE"; $resn_exch{"H241 PIE"}="1H24 PIE"; $resn_exch{"H242 PIE"}="2H24 PIE";
$resn_exch{"H251 PIE"}="1H25 PIE"; $resn_exch{"H252 PIE"}="2H25 PIE"; $resn_exch{"H261 PIE"}="1H26 PIE";
$resn_exch{"H262 PIE"}="2H26 PIE"; $resn_exch{"H271 PIE"}="1H27 PIE"; $resn_exch{"H272 PIE"}="2H27 PIE";
$resn_exch{"H281 PIE"}="1H28 PIE"; $resn_exch{"H282 PIE"}="2H28 PIE"; $resn_exch{"H291 PIE"}="1H29 PIE";
$resn_exch{"H471 PIE"}="1H47 PIE"; $resn_exch{"H481 PIE"}="1H48 PIE"; $resn_exch{"H482 PIE"}="2H48 PIE";
$resn_exch{"H491 PIE"}="1H49 PIE"; $resn_exch{"H492 PIE"}="2H49 PIE"; $resn_exch{"H501 PIE"}="1H50 PIE";
$resn_exch{"H502 PIE"}="2H50 PIE"; $resn_exch{"H511 PIE"}="1H51 PIE"; $resn_exch{"H512 PIE"}="2H51 PIE";
$resn_exch{"H1'1 PIE"}="1H1' PIE"; $resn_exch{"H2'1 PIE"}="1H2' PIE"; $resn_exch{"H3'1 PIE"}="1H3' PIE";
$resn_exch{"H4'1 PIE"}="1H4' PIE"; $resn_exch{"H5'1 PIE"}="1H5' PIE"; $resn_exch{"H6'1 PIE"}="1H6' PIE";
$resn_exch{"HO11 PIE"}="1HO1 PIE"; $resn_exch{"HO21 PIE"}="1HO2 PIE"; $resn_exch{"HO31 PIE"}="1HO3 PIE";
$resn_exch{"HO41 PIE"}="1HO4 PIE"; $resn_exch{"HO61 PIE"}="1HO6 PIE"; $resn_exch{"H521 PIE"}="1H52 PIE";
$resn_exch{"H522 PIE"}="2H52 PIE"; $resn_exch{"H531 PIE"}="1H53 PIE"; $resn_exch{"H532 PIE"}="2H53 PIE";
$resn_exch{"H541 PIE"}="1H54 PIE"; $resn_exch{"H542 PIE"}="2H54 PIE"; $resn_exch{"H551 PIE"}="1H55 PIE";
$resn_exch{"H552 PIE"}="2H55 PIE"; $resn_exch{"H553 PIE"}="3H55 PIE"; $resn_exch{"H431 PIE"}="1H43 PIE";
$resn_exch{"H432 PIE"}="2H43 PIE"; $resn_exch{"H441 PIE"}="1H44 PIE"; $resn_exch{"H442 PIE"}="2H44 PIE";
$resn_exch{"H451 PIE"}="1H45 PIE"; $resn_exch{"H452 PIE"}="2H45 PIE"; $resn_exch{"H461 PIE"}="1H46 PIE";
$resn_exch{"H462 PIE"}="2H46 PIE"; $resn_exch{"H453 PIE"}="3H45 PIE"; $resn_exch{" H11 PIG"}="1H1  PIG";
$resn_exch{" H12 PIG"}="2H1  PIG"; $resn_exch{" H21 PIG"}="1H2  PIG"; $resn_exch{" H22 PIG"}="2H2  PIG";
$resn_exch{" H31 PIG"}="1H3  PIG"; $resn_exch{" H32 PIG"}="2H3  PIG"; $resn_exch{" H41 PIG"}="1H4  PIG";
$resn_exch{" H42 PIG"}="2H4  PIG"; $resn_exch{" H51 PIG"}="1H5  PIG"; $resn_exch{" H52 PIG"}="2H5  PIG";
$resn_exch{" H61 PIG"}="1H6  PIG"; $resn_exch{" H62 PIG"}="2H6  PIG"; $resn_exch{"H211 PII"}="1H21 PII";
$resn_exch{"H212 PII"}="2H21 PII"; $resn_exch{"H251 PII"}="1H25 PII"; $resn_exch{"H252 PII"}="2H25 PII";
$resn_exch{"HR11 PII"}="1HR1 PII"; $resn_exch{"HR12 PII"}="2HR1 PII"; $resn_exch{"HR21 PII"}="1HR2 PII";
$resn_exch{"HR22 PII"}="2HR2 PII"; $resn_exch{"HL11 PII"}="1HL1 PII"; $resn_exch{"HL12 PII"}="2HL1 PII";
$resn_exch{"HL21 PII"}="1HL2 PII"; $resn_exch{"HL22 PII"}="2HL2 PII"; $resn_exch{"HR31 PII"}="1HR3 PII";
$resn_exch{"HR32 PII"}="2HR3 PII"; $resn_exch{"HR41 PII"}="1HR4 PII"; $resn_exch{"HR42 PII"}="2HR4 PII";
$resn_exch{"HR51 PII"}="1HR5 PII"; $resn_exch{"HR52 PII"}="2HR5 PII"; $resn_exch{"HR61 PII"}="1HR6 PII";
$resn_exch{"HR62 PII"}="2HR6 PII"; $resn_exch{"HR71 PII"}="1HR7 PII"; $resn_exch{"HR72 PII"}="2HR7 PII";
$resn_exch{"HR81 PII"}="1HR8 PII"; $resn_exch{"HR82 PII"}="2HR8 PII"; $resn_exch{"HR91 PII"}="1HR9 PII";
$resn_exch{"HR92 PII"}="2HR9 PII"; $resn_exch{" HG1 PII"}="1HG  PII"; $resn_exch{" HH1 PII"}="1HH  PII";
$resn_exch{"HL31 PII"}="1HL3 PII"; $resn_exch{"HL32 PII"}="2HL3 PII"; $resn_exch{"HL41 PII"}="1HL4 PII";
$resn_exch{"HL42 PII"}="2HL4 PII"; $resn_exch{"HL51 PII"}="1HL5 PII"; $resn_exch{"HL52 PII"}="2HL5 PII";
$resn_exch{"HL61 PII"}="1HL6 PII"; $resn_exch{"HL62 PII"}="2HL6 PII"; $resn_exch{"HL71 PII"}="1HL7 PII";
$resn_exch{"HL72 PII"}="2HL7 PII"; $resn_exch{"HL81 PII"}="1HL8 PII"; $resn_exch{"HL82 PII"}="2HL8 PII";
$resn_exch{"HL91 PII"}="1HL9 PII"; $resn_exch{"HL92 PII"}="2HL9 PII"; $resn_exch{"H111 PIL"}="1H11 PIL";
$resn_exch{"H112 PIL"}="2H11 PIL"; $resn_exch{"H113 PIL"}="3H11 PIL"; $resn_exch{" H51 PIL"}="1H5  PIL";
$resn_exch{" H52 PIL"}="2H5  PIL"; $resn_exch{" H61 PIL"}="1H6  PIL"; $resn_exch{" H62 PIL"}="2H6  PIL";
$resn_exch{" H71 PIL"}="1H7  PIL"; $resn_exch{" H72 PIL"}="2H7  PIL"; $resn_exch{" H81 PIL"}="1H8  PIL";
$resn_exch{" H82 PIL"}="2H8  PIL"; $resn_exch{" H11 PIN"}="1H1  PIN"; $resn_exch{" H12 PIN"}="2H1  PIN";
$resn_exch{" H21 PIN"}="1H2  PIN"; $resn_exch{" H22 PIN"}="2H2  PIN"; $resn_exch{" H31 PIN"}="1H3  PIN";
$resn_exch{" H32 PIN"}="2H3  PIN"; $resn_exch{" H41 PIN"}="1H4  PIN"; $resn_exch{" H42 PIN"}="2H4  PIN";
$resn_exch{"H1'1 PIN"}="1H1' PIN"; $resn_exch{"H1'2 PIN"}="2H1' PIN"; $resn_exch{"H2'1 PIN"}="1H2' PIN";
$resn_exch{"H2'2 PIN"}="2H2' PIN"; $resn_exch{"H3'1 PIN"}="1H3' PIN"; $resn_exch{"H3'2 PIN"}="2H3' PIN";
$resn_exch{"H4'1 PIN"}="1H4' PIN"; $resn_exch{"H4'2 PIN"}="2H4' PIN"; $resn_exch{"HO3' PIN"}="'HO3 PIN";
$resn_exch{"HP12 PIO"}="2HP1 PIO"; $resn_exch{"HP42 PIO"}="2HP4 PIO"; $resn_exch{"HP53 PIO"}="3HP5 PIO";
$resn_exch{"HP52 PIO"}="2HP5 PIO"; $resn_exch{"HC11 PIO"}="1HC1 PIO"; $resn_exch{"HC12 PIO"}="2HC1 PIO";
$resn_exch{"HC31 PIO"}="1HC3 PIO"; $resn_exch{"HC32 PIO"}="2HC3 PIO"; $resn_exch{"H2A1 PIO"}="1H2A PIO";
$resn_exch{"H2A2 PIO"}="2H2A PIO"; $resn_exch{"H3A1 PIO"}="1H3A PIO"; $resn_exch{"H3A2 PIO"}="2H3A PIO";
$resn_exch{"H4A1 PIO"}="1H4A PIO"; $resn_exch{"H4A2 PIO"}="2H4A PIO"; $resn_exch{"H5A1 PIO"}="1H5A PIO";
$resn_exch{"H5A2 PIO"}="2H5A PIO"; $resn_exch{"H6A1 PIO"}="1H6A PIO"; $resn_exch{"H6A2 PIO"}="2H6A PIO";
$resn_exch{"H7A1 PIO"}="1H7A PIO"; $resn_exch{"H7A2 PIO"}="2H7A PIO"; $resn_exch{"H8A1 PIO"}="1H8A PIO";
$resn_exch{"H8A2 PIO"}="2H8A PIO"; $resn_exch{"H8A3 PIO"}="3H8A PIO"; $resn_exch{"H2B1 PIO"}="1H2B PIO";
$resn_exch{"H2B2 PIO"}="2H2B PIO"; $resn_exch{"H3B1 PIO"}="1H3B PIO"; $resn_exch{"H3B2 PIO"}="2H3B PIO";
$resn_exch{"H4B1 PIO"}="1H4B PIO"; $resn_exch{"H4B2 PIO"}="2H4B PIO"; $resn_exch{"H5B1 PIO"}="1H5B PIO";
$resn_exch{"H5B2 PIO"}="2H5B PIO"; $resn_exch{"H6B1 PIO"}="1H6B PIO"; $resn_exch{"H6B2 PIO"}="2H6B PIO";
$resn_exch{"H7B1 PIO"}="1H7B PIO"; $resn_exch{"H7B2 PIO"}="2H7B PIO"; $resn_exch{"H8B1 PIO"}="1H8B PIO";
$resn_exch{"H8B2 PIO"}="2H8B PIO"; $resn_exch{"H8B3 PIO"}="3H8B PIO"; $resn_exch{" H21 PIP"}="1H2  PIP";
$resn_exch{" H22 PIP"}="2H2  PIP"; $resn_exch{" H31 PIP"}="1H3  PIP"; $resn_exch{" H32 PIP"}="2H3  PIP";
$resn_exch{" H41 PIP"}="1H4  PIP"; $resn_exch{" H42 PIP"}="2H4  PIP"; $resn_exch{" H51 PIP"}="1H5  PIP";
$resn_exch{" H52 PIP"}="2H5  PIP"; $resn_exch{" H61 PIP"}="1H6  PIP"; $resn_exch{" H62 PIP"}="2H6  PIP";
$resn_exch{" HM1 PIQ"}="1HM  PIQ"; $resn_exch{" HM2 PIQ"}="2HM  PIQ"; $resn_exch{" HM3 PIQ"}="3HM  PIQ";
$resn_exch{"HN51 PIR"}="1HN5 PIR"; $resn_exch{"HN52 PIR"}="2HN5 PIR"; $resn_exch{"HO2' PIR"}="'HO2 PIR";
$resn_exch{"HO3' PIR"}="'HO3 PIR"; $resn_exch{"HN4' PIR"}="'HN4 PIR"; $resn_exch{"H5'1 PIR"}="1H5' PIR";
$resn_exch{"H5'2 PIR"}="2H5' PIR"; $resn_exch{"HO5' PIR"}="'HO5 PIR"; $resn_exch{"HN11 PIU"}="1HN1 PIU";
$resn_exch{"HN12 PIU"}="2HN1 PIU"; $resn_exch{"H151 PIU"}="1H15 PIU"; $resn_exch{"H152 PIU"}="2H15 PIU";
$resn_exch{"H161 PIU"}="1H16 PIU"; $resn_exch{"H162 PIU"}="2H16 PIU"; $resn_exch{"H121 PIU"}="1H12 PIU";
$resn_exch{"H122 PIU"}="2H12 PIU"; $resn_exch{"H123 PIU"}="3H12 PIU"; $resn_exch{"H131 PIU"}="1H13 PIU";
$resn_exch{"H132 PIU"}="2H13 PIU"; $resn_exch{"H133 PIU"}="3H13 PIU"; $resn_exch{"H141 PIU"}="1H14 PIU";
$resn_exch{"H142 PIU"}="2H14 PIU"; $resn_exch{"H143 PIU"}="3H14 PIU"; $resn_exch{" H11 PIV"}="1H1  PIV";
$resn_exch{" H12 PIV"}="2H1  PIV"; $resn_exch{" H13 PIV"}="3H1  PIV"; $resn_exch{" H21 PIV"}="1H2  PIV";
$resn_exch{" H22 PIV"}="2H2  PIV"; $resn_exch{" H23 PIV"}="3H2  PIV"; $resn_exch{" H31 PIV"}="1H3  PIV";
$resn_exch{" H32 PIV"}="2H3  PIV"; $resn_exch{" H33 PIV"}="3H3  PIV"; $resn_exch{"H22A PKF"}="2H2  PKF";
$resn_exch{" H21 PKF"}="1H2  PKF"; $resn_exch{" H92 PKF"}="2H9  PKF"; $resn_exch{" H91 PKF"}="1H9  PKF";
$resn_exch{"H112 PKF"}="2H11 PKF"; $resn_exch{"H111 PKF"}="1H11 PKF"; $resn_exch{"H122 PKF"}="2H12 PKF";
$resn_exch{"H121 PKF"}="1H12 PKF"; $resn_exch{"H142 PKF"}="2H14 PKF"; $resn_exch{"H141 PKF"}="1H14 PKF";
$resn_exch{"H162 PKF"}="2H16 PKF"; $resn_exch{"H161 PKF"}="1H16 PKF"; $resn_exch{"H172 PKF"}="2H17 PKF";
$resn_exch{"H171 PKF"}="1H17 PKF"; $resn_exch{"H192 PKF"}="2H19 PKF"; $resn_exch{"H191 PKF"}="1H19 PKF";
$resn_exch{" H33 PKF"}="3H3  PKF"; $resn_exch{" H32 PKF"}="2H3  PKF"; $resn_exch{"H31A PKF"}="1H3  PKF";
$resn_exch{"H4N1 PKF"}="1H4N PKF"; $resn_exch{"H262 PKF"}="2H26 PKF"; $resn_exch{"H261 PKF"}="1H26 PKF";
$resn_exch{"H293 PKF"}="3H29 PKF"; $resn_exch{"H292 PKF"}="2H29 PKF"; $resn_exch{"H291 PKF"}="1H29 PKF";
$resn_exch{"H323 PKF"}="3H32 PKF"; $resn_exch{"H322 PKF"}="2H32 PKF"; $resn_exch{"H321 PKF"}="1H32 PKF";
$resn_exch{"H332 PKF"}="2H33 PKF"; $resn_exch{"H331 PKF"}="1H33 PKF"; $resn_exch{"H352 PKF"}="2H35 PKF";
$resn_exch{"H351 PKF"}="1H35 PKF"; $resn_exch{"H362 PKF"}="2H36 PKF"; $resn_exch{"H361 PKF"}="1H36 PKF";
$resn_exch{"HN21 PL0"}="1HN2 PL0"; $resn_exch{"HN22 PL0"}="2HN2 PL0"; $resn_exch{"H2A1 PL2"}="1H2A PL2";
$resn_exch{"H2A2 PL2"}="2H2A PL2"; $resn_exch{"H2A3 PL2"}="3H2A PL2"; $resn_exch{"H4A1 PL2"}="1H4A PL2";
$resn_exch{"H4A2 PL2"}="2H4A PL2"; $resn_exch{" HA1 PL2"}="1HA  PL2"; $resn_exch{" HA2 PL2"}="2HA  PL2";
$resn_exch{" HG1 PL2"}="1HG  PL2"; $resn_exch{" HG2 PL2"}="2HG  PL2"; $resn_exch{"H5M1 PL2"}="1H5M PL2";
$resn_exch{"H5M2 PL2"}="2H5M PL2"; $resn_exch{" HB1 PL2"}="1HB  PL2"; $resn_exch{" HB2 PL2"}="2HB  PL2";
$resn_exch{" HD1 PL2"}="1HD  PL2"; $resn_exch{" HD2 PL2"}="2HD  PL2"; $resn_exch{"HH11 PL2"}="1HH1 PL2";
$resn_exch{"HH12 PL2"}="2HH1 PL2"; $resn_exch{"H1C1 PL3"}="1H1C PL3"; $resn_exch{"H1C2 PL3"}="2H1C PL3";
$resn_exch{"H2C1 PL3"}="1H2C PL3"; $resn_exch{"H2C2 PL3"}="2H2C PL3"; $resn_exch{"H3C1 PL3"}="1H3C PL3";
$resn_exch{"H3C2 PL3"}="2H3C PL3"; $resn_exch{"H4C1 PL3"}="1H4C PL3"; $resn_exch{"H4C2 PL3"}="2H4C PL3";
$resn_exch{"H5C1 PL3"}="1H5C PL3"; $resn_exch{"H5C2 PL3"}="2H5C PL3"; $resn_exch{"H6C1 PL3"}="1H6C PL3";
$resn_exch{"H6C2 PL3"}="2H6C PL3"; $resn_exch{"H7C1 PL3"}="1H7C PL3"; $resn_exch{"H7C2 PL3"}="2H7C PL3";
$resn_exch{"H8C1 PL3"}="1H8C PL3"; $resn_exch{"H8C2 PL3"}="2H8C PL3"; $resn_exch{"H9C1 PL3"}="1H9C PL3";
$resn_exch{"H9C2 PL3"}="2H9C PL3"; $resn_exch{"HAC1 PL3"}="1HAC PL3"; $resn_exch{"HAC2 PL3"}="2HAC PL3";
$resn_exch{"HBC1 PL3"}="1HBC PL3"; $resn_exch{"HBC2 PL3"}="2HBC PL3"; $resn_exch{"HCC1 PL3"}="1HCC PL3";
$resn_exch{"HCC2 PL3"}="2HCC PL3"; $resn_exch{"HDC1 PL3"}="1HDC PL3"; $resn_exch{"HDC2 PL3"}="2HDC PL3";
$resn_exch{"HEC1 PL3"}="1HEC PL3"; $resn_exch{"HEC2 PL3"}="2HEC PL3"; $resn_exch{"HFC1 PL3"}="1HFC PL3";
$resn_exch{"HFC2 PL3"}="2HFC PL3"; $resn_exch{"HGC1 PL3"}="1HGC PL3"; $resn_exch{"HGC2 PL3"}="2HGC PL3";
$resn_exch{"HGC3 PL3"}="3HGC PL3"; $resn_exch{"H101 PL7"}="1H10 PL7"; $resn_exch{"H102 PL7"}="2H10 PL7";
$resn_exch{"H111 PL7"}="1H11 PL7"; $resn_exch{"H112 PL7"}="2H11 PL7"; $resn_exch{"H5C1 PL7"}="1H5C PL7";
$resn_exch{"H5C2 PL7"}="2H5C PL7"; $resn_exch{"H7C1 PL7"}="1H7C PL7"; $resn_exch{"H7C2 PL7"}="2H7C PL7";
$resn_exch{"HC71 PL9"}="1HC7 PL9"; $resn_exch{"HC72 PL9"}="2HC7 PL9"; $resn_exch{"H101 PL9"}="1H10 PL9";
$resn_exch{"H102 PL9"}="2H10 PL9"; $resn_exch{"H103 PL9"}="3H10 PL9"; $resn_exch{"H111 PL9"}="1H11 PL9";
$resn_exch{"H112 PL9"}="2H11 PL9"; $resn_exch{"H121 PL9"}="1H12 PL9"; $resn_exch{"H122 PL9"}="2H12 PL9";
$resn_exch{"H151 PL9"}="1H15 PL9"; $resn_exch{"H152 PL9"}="2H15 PL9"; $resn_exch{"H153 PL9"}="3H15 PL9";
$resn_exch{"H161 PL9"}="1H16 PL9"; $resn_exch{"H162 PL9"}="2H16 PL9"; $resn_exch{"H171 PL9"}="1H17 PL9";
$resn_exch{"H172 PL9"}="2H17 PL9"; $resn_exch{"H201 PL9"}="1H20 PL9"; $resn_exch{"H202 PL9"}="2H20 PL9";
$resn_exch{"H203 PL9"}="3H20 PL9"; $resn_exch{"H211 PL9"}="1H21 PL9"; $resn_exch{"H212 PL9"}="2H21 PL9";
$resn_exch{"H221 PL9"}="1H22 PL9"; $resn_exch{"H222 PL9"}="2H22 PL9"; $resn_exch{"H251 PL9"}="1H25 PL9";
$resn_exch{"H252 PL9"}="2H25 PL9"; $resn_exch{"H253 PL9"}="3H25 PL9"; $resn_exch{"H261 PL9"}="1H26 PL9";
$resn_exch{"H262 PL9"}="2H26 PL9"; $resn_exch{"H271 PL9"}="1H27 PL9"; $resn_exch{"H272 PL9"}="2H27 PL9";
$resn_exch{"H301 PL9"}="1H30 PL9"; $resn_exch{"H302 PL9"}="2H30 PL9"; $resn_exch{"H303 PL9"}="3H30 PL9";
$resn_exch{"H311 PL9"}="1H31 PL9"; $resn_exch{"H312 PL9"}="2H31 PL9"; $resn_exch{"H321 PL9"}="1H32 PL9";
$resn_exch{"H322 PL9"}="2H32 PL9"; $resn_exch{"H351 PL9"}="1H35 PL9"; $resn_exch{"H352 PL9"}="2H35 PL9";
$resn_exch{"H353 PL9"}="3H35 PL9"; $resn_exch{"H361 PL9"}="1H36 PL9"; $resn_exch{"H362 PL9"}="2H36 PL9";
$resn_exch{"H371 PL9"}="1H37 PL9"; $resn_exch{"H372 PL9"}="2H37 PL9"; $resn_exch{"H401 PL9"}="1H40 PL9";
$resn_exch{"H402 PL9"}="2H40 PL9"; $resn_exch{"H403 PL9"}="3H40 PL9"; $resn_exch{"H411 PL9"}="1H41 PL9";
$resn_exch{"H412 PL9"}="2H41 PL9"; $resn_exch{"H421 PL9"}="1H42 PL9"; $resn_exch{"H422 PL9"}="2H42 PL9";
$resn_exch{"H451 PL9"}="1H45 PL9"; $resn_exch{"H452 PL9"}="2H45 PL9"; $resn_exch{"H453 PL9"}="3H45 PL9";
$resn_exch{"H461 PL9"}="1H46 PL9"; $resn_exch{"H462 PL9"}="2H46 PL9"; $resn_exch{"H471 PL9"}="1H47 PL9";
$resn_exch{"H472 PL9"}="2H47 PL9"; $resn_exch{"H501 PL9"}="1H50 PL9"; $resn_exch{"H502 PL9"}="2H50 PL9";
$resn_exch{"H503 PL9"}="3H50 PL9"; $resn_exch{"H511 PL9"}="1H51 PL9"; $resn_exch{"H512 PL9"}="2H51 PL9";
$resn_exch{"H513 PL9"}="3H51 PL9"; $resn_exch{"H521 PL9"}="1H52 PL9"; $resn_exch{"H522 PL9"}="2H52 PL9";
$resn_exch{"H523 PL9"}="3H52 PL9"; $resn_exch{"H531 PL9"}="1H53 PL9"; $resn_exch{"H532 PL9"}="2H53 PL9";
$resn_exch{"H533 PL9"}="3H53 PL9"; $resn_exch{"H2A1 PLA"}="1H2A PLA"; $resn_exch{"H2A2 PLA"}="2H2A PLA";
$resn_exch{"H2A3 PLA"}="3H2A PLA"; $resn_exch{"H4A1 PLA"}="1H4A PLA"; $resn_exch{"H4A2 PLA"}="2H4A PLA";
$resn_exch{"H5A1 PLA"}="1H5A PLA"; $resn_exch{"H5A2 PLA"}="2H5A PLA"; $resn_exch{"HOP2 PLA"}="2HOP PLA";
$resn_exch{"HOP3 PLA"}="3HOP PLA"; $resn_exch{"HB11 PLA"}="1HB1 PLA"; $resn_exch{"HB12 PLA"}="2HB1 PLA";
$resn_exch{"HB21 PLA"}="1HB2 PLA"; $resn_exch{"HB22 PLA"}="2HB2 PLA"; $resn_exch{"HB23 PLA"}="3HB2 PLA";
$resn_exch{" H11 PLC"}="1H1  PLC"; $resn_exch{" H12 PLC"}="2H1  PLC"; $resn_exch{" H31 PLC"}="1H3  PLC";
$resn_exch{" H32 PLC"}="2H3  PLC"; $resn_exch{" H41 PLC"}="1H4  PLC"; $resn_exch{" H42 PLC"}="2H4  PLC";
$resn_exch{" H51 PLC"}="1H5  PLC"; $resn_exch{" H52 PLC"}="2H5  PLC"; $resn_exch{" H61 PLC"}="1H6  PLC";
$resn_exch{" H62 PLC"}="2H6  PLC"; $resn_exch{" H63 PLC"}="3H6  PLC"; $resn_exch{" H71 PLC"}="1H7  PLC";
$resn_exch{" H72 PLC"}="2H7  PLC"; $resn_exch{" H73 PLC"}="3H7  PLC"; $resn_exch{" H81 PLC"}="1H8  PLC";
$resn_exch{" H82 PLC"}="2H8  PLC"; $resn_exch{" H83 PLC"}="3H8  PLC"; $resn_exch{"H1'1 PLC"}="1H1' PLC";
$resn_exch{"H1'2 PLC"}="2H1' PLC"; $resn_exch{"H2'1 PLC"}="1H2' PLC"; $resn_exch{"H2'2 PLC"}="2H2' PLC";
$resn_exch{"H3'1 PLC"}="1H3' PLC"; $resn_exch{"H3'2 PLC"}="2H3' PLC"; $resn_exch{"H4'1 PLC"}="1H4' PLC";
$resn_exch{"H4'2 PLC"}="2H4' PLC"; $resn_exch{"H5'1 PLC"}="1H5' PLC"; $resn_exch{"H5'2 PLC"}="2H5' PLC";
$resn_exch{"H6'1 PLC"}="1H6' PLC"; $resn_exch{"H6'2 PLC"}="2H6' PLC"; $resn_exch{"H7'1 PLC"}="1H7' PLC";
$resn_exch{"H7'2 PLC"}="2H7' PLC"; $resn_exch{"H8'1 PLC"}="1H8' PLC"; $resn_exch{"H8'2 PLC"}="2H8' PLC";
$resn_exch{"H9'1 PLC"}="1H9' PLC"; $resn_exch{"H9'2 PLC"}="2H9' PLC"; $resn_exch{"HT'1 PLC"}="1HT' PLC";
$resn_exch{"HT'2 PLC"}="2HT' PLC"; $resn_exch{"HE'1 PLC"}="1HE' PLC"; $resn_exch{"HE'2 PLC"}="2HE' PLC";
$resn_exch{"HE'3 PLC"}="3HE' PLC"; $resn_exch{"H1A1 PLC"}="1H1\" PLC"; $resn_exch{"H1A2 PLC"}="2H1\" PLC";
$resn_exch{"H2A1 PLC"}="1H2\" PLC"; $resn_exch{"H2A2 PLC"}="2H2\" PLC"; $resn_exch{"H3A1 PLC"}="1H3\" PLC";
$resn_exch{"H3A2 PLC"}="2H3\" PLC"; $resn_exch{"H4A1 PLC"}="1H4\" PLC"; $resn_exch{"H4A2 PLC"}="2H4\" PLC";
$resn_exch{"H5A1 PLC"}="1H5\" PLC"; $resn_exch{"H5A2 PLC"}="2H5\" PLC"; $resn_exch{"H6A1 PLC"}="1H6\" PLC";
$resn_exch{"H6A2 PLC"}="2H6\" PLC"; $resn_exch{"H7A1 PLC"}="1H7\" PLC"; $resn_exch{"H7A2 PLC"}="2H7\" PLC";
$resn_exch{"H8A1 PLC"}="1H8\" PLC"; $resn_exch{"H8A2 PLC"}="2H8\" PLC"; $resn_exch{"H9A1 PLC"}="1H9\" PLC";
$resn_exch{"H9A2 PLC"}="2H9\" PLC"; $resn_exch{"HTA1 PLC"}="1HT\" PLC"; $resn_exch{"HTA2 PLC"}="2HT\" PLC";
$resn_exch{"HEA1 PLC"}="1HE\" PLC"; $resn_exch{"HEA2 PLC"}="2HE\" PLC"; $resn_exch{"HEA3 PLC"}="3HE\" PLC";
$resn_exch{"HOP2 PLC"}="2HOP PLC"; $resn_exch{" H11 PLD"}="1H1  PLD"; $resn_exch{" H12 PLD"}="2H1  PLD";
$resn_exch{" H31 PLD"}="1H3  PLD"; $resn_exch{" H32 PLD"}="2H3  PLD"; $resn_exch{" H41 PLD"}="1H4  PLD";
$resn_exch{" H42 PLD"}="2H4  PLD"; $resn_exch{" H51 PLD"}="1H5  PLD"; $resn_exch{" H52 PLD"}="2H5  PLD";
$resn_exch{" H61 PLD"}="1H6  PLD"; $resn_exch{" H62 PLD"}="2H6  PLD"; $resn_exch{" H63 PLD"}="3H6  PLD";
$resn_exch{" H71 PLD"}="1H7  PLD"; $resn_exch{" H72 PLD"}="2H7  PLD"; $resn_exch{" H73 PLD"}="3H7  PLD";
$resn_exch{" H81 PLD"}="1H8  PLD"; $resn_exch{" H82 PLD"}="2H8  PLD"; $resn_exch{" H83 PLD"}="3H8  PLD";
$resn_exch{"H1'1 PLD"}="1H1' PLD"; $resn_exch{"H1'2 PLD"}="2H1' PLD"; $resn_exch{"H2'1 PLD"}="1H2' PLD";
$resn_exch{"H2'2 PLD"}="2H2' PLD"; $resn_exch{"H3'1 PLD"}="1H3' PLD"; $resn_exch{"H3'2 PLD"}="2H3' PLD";
$resn_exch{"H4'1 PLD"}="1H4' PLD"; $resn_exch{"H4'2 PLD"}="2H4' PLD"; $resn_exch{"H5'1 PLD"}="1H5' PLD";
$resn_exch{"H5'2 PLD"}="2H5' PLD"; $resn_exch{"H6'1 PLD"}="1H6' PLD"; $resn_exch{"H6'2 PLD"}="2H6' PLD";
$resn_exch{"H7'1 PLD"}="1H7' PLD"; $resn_exch{"H7'2 PLD"}="2H7' PLD"; $resn_exch{"H8'1 PLD"}="1H8' PLD";
$resn_exch{"H8'2 PLD"}="2H8' PLD"; $resn_exch{"H9'1 PLD"}="1H9' PLD"; $resn_exch{"H9'2 PLD"}="2H9' PLD";
$resn_exch{"HT'1 PLD"}="1HT' PLD"; $resn_exch{"HT'2 PLD"}="2HT' PLD"; $resn_exch{"HE'1 PLD"}="1HE' PLD";
$resn_exch{"HE'2 PLD"}="2HE' PLD"; $resn_exch{"H1A1 PLD"}="1H1\" PLD"; $resn_exch{"H1A2 PLD"}="2H1\" PLD";
$resn_exch{"H2A1 PLD"}="1H2\" PLD"; $resn_exch{"H2A2 PLD"}="2H2\" PLD"; $resn_exch{"H3A1 PLD"}="1H3\" PLD";
$resn_exch{"H3A2 PLD"}="2H3\" PLD"; $resn_exch{"H4A1 PLD"}="1H4\" PLD"; $resn_exch{"H4A2 PLD"}="2H4\" PLD";
$resn_exch{"H5A1 PLD"}="1H5\" PLD"; $resn_exch{"H5A2 PLD"}="2H5\" PLD"; $resn_exch{"H6A1 PLD"}="1H6\" PLD";
$resn_exch{"H6A2 PLD"}="2H6\" PLD"; $resn_exch{"H7A1 PLD"}="1H7\" PLD"; $resn_exch{"H7A2 PLD"}="2H7\" PLD";
$resn_exch{"H8A1 PLD"}="1H8\" PLD"; $resn_exch{"H8A2 PLD"}="2H8\" PLD"; $resn_exch{"H9A1 PLD"}="1H9\" PLD";
$resn_exch{"H9A2 PLD"}="2H9\" PLD"; $resn_exch{"HTA1 PLD"}="1HT\" PLD"; $resn_exch{"HTA2 PLD"}="2HT\" PLD";
$resn_exch{"HEA1 PLD"}="1HE\" PLD"; $resn_exch{"HEA2 PLD"}="2HE\" PLD"; $resn_exch{"HOP2 PLD"}="2HOP PLD";
$resn_exch{"HA'1 PLD"}="1HA' PLD"; $resn_exch{"HA'2 PLD"}="2HA' PLD"; $resn_exch{"HB'1 PLD"}="1HB' PLD";
$resn_exch{"HB'2 PLD"}="2HB' PLD"; $resn_exch{"HC'1 PLD"}="1HC' PLD"; $resn_exch{"HC'2 PLD"}="2HC' PLD";
$resn_exch{"HD'1 PLD"}="1HD' PLD"; $resn_exch{"HD'2 PLD"}="2HD' PLD"; $resn_exch{"HF'1 PLD"}="1HF' PLD";
$resn_exch{"HF'2 PLD"}="2HF' PLD"; $resn_exch{"HG'1 PLD"}="1HG' PLD"; $resn_exch{"HG'2 PLD"}="2HG' PLD";
$resn_exch{"HH'1 PLD"}="1HH' PLD"; $resn_exch{"HH'2 PLD"}="2HH' PLD"; $resn_exch{"HI'1 PLD"}="1HI' PLD";
$resn_exch{"HI'2 PLD"}="2HI' PLD"; $resn_exch{"HJ'1 PLD"}="1HJ' PLD"; $resn_exch{"HJ'2 PLD"}="2HJ' PLD";
$resn_exch{"HJ'3 PLD"}="3HJ' PLD"; $resn_exch{"HAA1 PLD"}="1HA\" PLD"; $resn_exch{"HAA2 PLD"}="2HA\" PLD";
$resn_exch{"HBA1 PLD"}="1HB\" PLD"; $resn_exch{"HBA2 PLD"}="2HB\" PLD"; $resn_exch{"HCA1 PLD"}="1HC\" PLD";
$resn_exch{"HCA2 PLD"}="2HC\" PLD"; $resn_exch{"HDA1 PLD"}="1HD\" PLD"; $resn_exch{"HDA2 PLD"}="2HD\" PLD";
$resn_exch{"HFA1 PLD"}="1HF\" PLD"; $resn_exch{"HFA2 PLD"}="2HF\" PLD"; $resn_exch{"HGA1 PLD"}="1HG\" PLD";
$resn_exch{"HGA2 PLD"}="2HG\" PLD"; $resn_exch{"HHA1 PLD"}="1HH\" PLD"; $resn_exch{"HHA2 PLD"}="2HH\" PLD";
$resn_exch{"HIA1 PLD"}="1HI\" PLD"; $resn_exch{"HIA2 PLD"}="2HI\" PLD"; $resn_exch{"HJA1 PLD"}="1HJ\" PLD";
$resn_exch{"HJA2 PLD"}="2HJ\" PLD"; $resn_exch{"HJA3 PLD"}="3HJ\" PLD"; $resn_exch{" HB1 PLE"}="1HB  PLE";
$resn_exch{" HB2 PLE"}="2HB  PLE"; $resn_exch{"HD11 PLE"}="1HD1 PLE"; $resn_exch{"HD12 PLE"}="2HD1 PLE";
$resn_exch{"HD13 PLE"}="3HD1 PLE"; $resn_exch{"HD21 PLE"}="1HD2 PLE"; $resn_exch{"HD22 PLE"}="2HD2 PLE";
$resn_exch{"HD23 PLE"}="3HD2 PLE"; $resn_exch{" HN1 PLE"}="1HN  PLE"; $resn_exch{" HN2 PLE"}="2HN  PLE";
$resn_exch{"H2A1 PLG"}="1H2A PLG"; $resn_exch{"H2A2 PLG"}="2H2A PLG"; $resn_exch{"H2A3 PLG"}="3H2A PLG";
$resn_exch{"H4A1 PLG"}="1H4A PLG"; $resn_exch{"H4A2 PLG"}="2H4A PLG"; $resn_exch{"H5A1 PLG"}="1H5A PLG";
$resn_exch{"H5A2 PLG"}="2H5A PLG"; $resn_exch{"HOP2 PLG"}="2HOP PLG"; $resn_exch{"HOP3 PLG"}="3HOP PLG";
$resn_exch{" H21 PLH"}="1H2  PLH"; $resn_exch{" H22 PLH"}="2H2  PLH"; $resn_exch{" H41 PLH"}="1H4  PLH";
$resn_exch{" H42 PLH"}="2H4  PLH"; $resn_exch{" H61 PLH"}="1H6  PLH"; $resn_exch{" H62 PLH"}="2H6  PLH";
$resn_exch{" H63 PLH"}="3H6  PLH"; $resn_exch{" H71 PLH"}="1H7  PLH"; $resn_exch{" H72 PLH"}="2H7  PLH";
$resn_exch{" H73 PLH"}="3H7  PLH"; $resn_exch{"H111 PLH"}="1H11 PLH"; $resn_exch{"H112 PLH"}="2H11 PLH";
$resn_exch{"H113 PLH"}="3H11 PLH"; $resn_exch{"H121 PLH"}="1H12 PLH"; $resn_exch{"H122 PLH"}="2H12 PLH";
$resn_exch{" H21 PLM"}="1H2  PLM"; $resn_exch{" H22 PLM"}="2H2  PLM"; $resn_exch{" H31 PLM"}="1H3  PLM";
$resn_exch{" H32 PLM"}="2H3  PLM"; $resn_exch{" H41 PLM"}="1H4  PLM"; $resn_exch{" H42 PLM"}="2H4  PLM";
$resn_exch{" H51 PLM"}="1H5  PLM"; $resn_exch{" H52 PLM"}="2H5  PLM"; $resn_exch{" H61 PLM"}="1H6  PLM";
$resn_exch{" H62 PLM"}="2H6  PLM"; $resn_exch{" H71 PLM"}="1H7  PLM"; $resn_exch{" H72 PLM"}="2H7  PLM";
$resn_exch{" H81 PLM"}="1H8  PLM"; $resn_exch{" H82 PLM"}="2H8  PLM"; $resn_exch{" H91 PLM"}="1H9  PLM";
$resn_exch{" H92 PLM"}="2H9  PLM"; $resn_exch{" HA1 PLM"}="1HA  PLM"; $resn_exch{" HA2 PLM"}="2HA  PLM";
$resn_exch{" HB1 PLM"}="1HB  PLM"; $resn_exch{" HB2 PLM"}="2HB  PLM"; $resn_exch{" HC1 PLM"}="1HC  PLM";
$resn_exch{" HC2 PLM"}="2HC  PLM"; $resn_exch{" HD1 PLM"}="1HD  PLM"; $resn_exch{" HD2 PLM"}="2HD  PLM";
$resn_exch{" HE1 PLM"}="1HE  PLM"; $resn_exch{" HE2 PLM"}="2HE  PLM"; $resn_exch{" HF1 PLM"}="1HF  PLM";
$resn_exch{" HF2 PLM"}="2HF  PLM"; $resn_exch{" HG1 PLM"}="1HG  PLM"; $resn_exch{" HG2 PLM"}="2HG  PLM";
$resn_exch{" HG3 PLM"}="3HG  PLM"; $resn_exch{" H11 PLO"}="1H1  PLO"; $resn_exch{" H12 PLO"}="2H1  PLO";
$resn_exch{" H21 PLO"}="1H2  PLO"; $resn_exch{" H22 PLO"}="2H2  PLO"; $resn_exch{" H41 PLO"}="1H4  PLO";
$resn_exch{" H42 PLO"}="2H4  PLO"; $resn_exch{" H71 PLO"}="1H7  PLO"; $resn_exch{" H72 PLO"}="2H7  PLO";
$resn_exch{"H111 PLO"}="1H11 PLO"; $resn_exch{"H112 PLO"}="2H11 PLO"; $resn_exch{"H121 PLO"}="1H12 PLO";
$resn_exch{"H122 PLO"}="2H12 PLO"; $resn_exch{"H151 PLO"}="1H15 PLO"; $resn_exch{"H152 PLO"}="2H15 PLO";
$resn_exch{"H161 PLO"}="1H16 PLO"; $resn_exch{"H162 PLO"}="2H16 PLO"; $resn_exch{"H181 PLO"}="1H18 PLO";
$resn_exch{"H182 PLO"}="2H18 PLO"; $resn_exch{"H183 PLO"}="3H18 PLO"; $resn_exch{"H191 PLO"}="1H19 PLO";
$resn_exch{"H192 PLO"}="2H19 PLO"; $resn_exch{"H193 PLO"}="3H19 PLO"; $resn_exch{"H211 PLO"}="1H21 PLO";
$resn_exch{"H212 PLO"}="2H21 PLO"; $resn_exch{"H213 PLO"}="3H21 PLO"; $resn_exch{"H2A1 PLP"}="1H2A PLP";
$resn_exch{"H2A2 PLP"}="2H2A PLP"; $resn_exch{"H2A3 PLP"}="3H2A PLP"; $resn_exch{"H5A1 PLP"}="1H5A PLP";
$resn_exch{"H5A2 PLP"}="2H5A PLP"; $resn_exch{"HOP2 PLP"}="2HOP PLP"; $resn_exch{"HOP3 PLP"}="3HOP PLP";
$resn_exch{"H2A1 PLR"}="1H2A PLR"; $resn_exch{"H2A2 PLR"}="2H2A PLR"; $resn_exch{"H2A3 PLR"}="3H2A PLR";
$resn_exch{"H4A1 PLR"}="1H4A PLR"; $resn_exch{"H4A2 PLR"}="2H4A PLR"; $resn_exch{"H4A3 PLR"}="3H4A PLR";
$resn_exch{"H5A1 PLR"}="1H5A PLR"; $resn_exch{"H5A2 PLR"}="2H5A PLR"; $resn_exch{" HB1 PLS"}="1HB  PLS";
$resn_exch{" HB2 PLS"}="2HB  PLS"; $resn_exch{"H2A1 PLS"}="1H2A PLS"; $resn_exch{"H2A2 PLS"}="2H2A PLS";
$resn_exch{"H2A3 PLS"}="3H2A PLS"; $resn_exch{"H4A1 PLS"}="1H4A PLS"; $resn_exch{"H4A2 PLS"}="2H4A PLS";
$resn_exch{"H5A1 PLS"}="1H5A PLS"; $resn_exch{"H5A2 PLS"}="2H5A PLS"; $resn_exch{"HOP2 PLS"}="2HOP PLS";
$resn_exch{"HOP3 PLS"}="3HOP PLS"; $resn_exch{"H2A1 PLT"}="1H2A PLT"; $resn_exch{"H2A2 PLT"}="2H2A PLT";
$resn_exch{"H2A3 PLT"}="3H2A PLT"; $resn_exch{"H5A1 PLT"}="1H5A PLT"; $resn_exch{"H5A2 PLT"}="2H5A PLT";
$resn_exch{"HOP2 PLT"}="2HOP PLT"; $resn_exch{"HOP3 PLT"}="3HOP PLT"; $resn_exch{" HB1 PLT"}="1HB  PLT";
$resn_exch{" HB2 PLT"}="2HB  PLT"; $resn_exch{" HB1 PLU"}="1HB  PLU"; $resn_exch{" HB2 PLU"}="2HB  PLU";
$resn_exch{"HD11 PLU"}="1HD1 PLU"; $resn_exch{"HD12 PLU"}="2HD1 PLU"; $resn_exch{"HD13 PLU"}="3HD1 PLU";
$resn_exch{"HD21 PLU"}="1HD2 PLU"; $resn_exch{"HD22 PLU"}="2HD2 PLU"; $resn_exch{"HD23 PLU"}="3HD2 PLU";
$resn_exch{" HN1 PLU"}="1HN  PLU"; $resn_exch{" HN2 PLU"}="2HN  PLU"; $resn_exch{" HB1 PLV"}="1HB  PLV";
$resn_exch{" HB2 PLV"}="2HB  PLV"; $resn_exch{"H2A1 PLV"}="1H2A PLV"; $resn_exch{"H2A2 PLV"}="2H2A PLV";
$resn_exch{"H2A3 PLV"}="3H2A PLV"; $resn_exch{"H4A1 PLV"}="1H4A PLV"; $resn_exch{"H4A2 PLV"}="2H4A PLV";
$resn_exch{"H5A1 PLV"}="1H5A PLV"; $resn_exch{"H5A2 PLV"}="2H5A PLV"; $resn_exch{" H71 PLV"}="1H7  PLV";
$resn_exch{" H72 PLV"}="2H7  PLV"; $resn_exch{" H73 PLV"}="3H7  PLV"; $resn_exch{"H231 PLX"}="1H23 PLX";
$resn_exch{"H232 PLX"}="2H23 PLX"; $resn_exch{"H233 PLX"}="3H23 PLX"; $resn_exch{"H221 PLX"}="1H22 PLX";
$resn_exch{"H222 PLX"}="2H22 PLX"; $resn_exch{"H211 PLX"}="1H21 PLX"; $resn_exch{"H212 PLX"}="2H21 PLX";
$resn_exch{"H201 PLX"}="1H20 PLX"; $resn_exch{"H202 PLX"}="2H20 PLX"; $resn_exch{"H191 PLX"}="1H19 PLX";
$resn_exch{"H192 PLX"}="2H19 PLX"; $resn_exch{"H181 PLX"}="1H18 PLX"; $resn_exch{"H182 PLX"}="2H18 PLX";
$resn_exch{"H171 PLX"}="1H17 PLX"; $resn_exch{"H172 PLX"}="2H17 PLX"; $resn_exch{"H161 PLX"}="1H16 PLX";
$resn_exch{"H162 PLX"}="2H16 PLX"; $resn_exch{"H151 PLX"}="1H15 PLX"; $resn_exch{"H152 PLX"}="2H15 PLX";
$resn_exch{"H141 PLX"}="1H14 PLX"; $resn_exch{"H142 PLX"}="2H14 PLX"; $resn_exch{"H131 PLX"}="1H13 PLX";
$resn_exch{"H132 PLX"}="2H13 PLX"; $resn_exch{"H121 PLX"}="1H12 PLX"; $resn_exch{"H122 PLX"}="2H12 PLX";
$resn_exch{"H111 PLX"}="1H11 PLX"; $resn_exch{"H112 PLX"}="2H11 PLX"; $resn_exch{"H101 PLX"}="1H10 PLX";
$resn_exch{"H102 PLX"}="2H10 PLX"; $resn_exch{" H91 PLX"}="1H9  PLX"; $resn_exch{" H92 PLX"}="2H9  PLX";
$resn_exch{" H81 PLX"}="1H8  PLX"; $resn_exch{" H82 PLX"}="2H8  PLX"; $resn_exch{" H71 PLX"}="1H7  PLX";
$resn_exch{" H72 PLX"}="2H7  PLX"; $resn_exch{" H31 PLX"}="1H3  PLX"; $resn_exch{" H32 PLX"}="2H3  PLX";
$resn_exch{" H21 PLX"}="1H2  PLX"; $resn_exch{" H22 PLX"}="2H2  PLX"; $resn_exch{" H11 PLX"}="1H1  PLX";
$resn_exch{" H12 PLX"}="2H1  PLX"; $resn_exch{"H1C1 PLX"}="1H1C PLX"; $resn_exch{"H1C2 PLX"}="2H1C PLX";
$resn_exch{"H1C3 PLX"}="3H1C PLX"; $resn_exch{"H1B1 PLX"}="1H1B PLX"; $resn_exch{"H1B2 PLX"}="2H1B PLX";
$resn_exch{"H1B3 PLX"}="3H1B PLX"; $resn_exch{"H1A1 PLX"}="1H1A PLX"; $resn_exch{"H1A2 PLX"}="2H1A PLX";
$resn_exch{"H1A3 PLX"}="3H1A PLX"; $resn_exch{" H51 PLX"}="1H5  PLX"; $resn_exch{" H52 PLX"}="2H5  PLX";
$resn_exch{"H251 PLX"}="1H25 PLX"; $resn_exch{"H252 PLX"}="2H25 PLX"; $resn_exch{"H261 PLX"}="1H26 PLX";
$resn_exch{"H262 PLX"}="2H26 PLX"; $resn_exch{"H271 PLX"}="1H27 PLX"; $resn_exch{"H272 PLX"}="2H27 PLX";
$resn_exch{"H281 PLX"}="1H28 PLX"; $resn_exch{"H282 PLX"}="2H28 PLX"; $resn_exch{"H291 PLX"}="1H29 PLX";
$resn_exch{"H292 PLX"}="2H29 PLX"; $resn_exch{"H301 PLX"}="1H30 PLX"; $resn_exch{"H302 PLX"}="2H30 PLX";
$resn_exch{"H311 PLX"}="1H31 PLX"; $resn_exch{"H312 PLX"}="2H31 PLX"; $resn_exch{"H321 PLX"}="1H32 PLX";
$resn_exch{"H322 PLX"}="2H32 PLX"; $resn_exch{"H331 PLX"}="1H33 PLX"; $resn_exch{"H332 PLX"}="2H33 PLX";
$resn_exch{"H341 PLX"}="1H34 PLX"; $resn_exch{"H342 PLX"}="2H34 PLX"; $resn_exch{"H351 PLX"}="1H35 PLX";
$resn_exch{"H352 PLX"}="2H35 PLX"; $resn_exch{"H361 PLX"}="1H36 PLX"; $resn_exch{"H362 PLX"}="2H36 PLX";
$resn_exch{"H371 PLX"}="1H37 PLX"; $resn_exch{"H372 PLX"}="2H37 PLX"; $resn_exch{"H381 PLX"}="1H38 PLX";
$resn_exch{"H382 PLX"}="2H38 PLX"; $resn_exch{"H391 PLX"}="1H39 PLX"; $resn_exch{"H392 PLX"}="2H39 PLX";
$resn_exch{"H393 PLX"}="3H39 PLX"; $resn_exch{" H11 PLY"}="1H1  PLY"; $resn_exch{" H21 PLY"}="1H2  PLY";
$resn_exch{" H22 PLY"}="2H2  PLY"; $resn_exch{" H31 PLY"}="1H3  PLY"; $resn_exch{" H32 PLY"}="2H3  PLY";
$resn_exch{" H41 PLY"}="1H4  PLY"; $resn_exch{" H42 PLY"}="2H4  PLY"; $resn_exch{" H51 PLY"}="1H5  PLY";
$resn_exch{" H52 PLY"}="2H5  PLY"; $resn_exch{" H61 PLY"}="1H6  PLY"; $resn_exch{" H62 PLY"}="2H6  PLY";
$resn_exch{" H71 PLY"}="1H7  PLY"; $resn_exch{" H72 PLY"}="2H7  PLY"; $resn_exch{" H81 PLY"}="1H8  PLY";
$resn_exch{" H82 PLY"}="2H8  PLY"; $resn_exch{" H91 PLY"}="1H9  PLY"; $resn_exch{" H92 PLY"}="2H9  PLY";
$resn_exch{" HA1 PLY"}="1HA  PLY"; $resn_exch{" HA2 PLY"}="2HA  PLY"; $resn_exch{" HB1 PLY"}="1HB  PLY";
$resn_exch{" HB2 PLY"}="2HB  PLY"; $resn_exch{" HC1 PLY"}="1HC  PLY"; $resn_exch{" HC2 PLY"}="2HC  PLY";
$resn_exch{" HD1 PLY"}="1HD  PLY"; $resn_exch{" HD2 PLY"}="2HD  PLY"; $resn_exch{" HE1 PLY"}="1HE  PLY";
$resn_exch{" HE2 PLY"}="2HE  PLY"; $resn_exch{" HF1 PLY"}="1HF  PLY"; $resn_exch{" HF2 PLY"}="2HF  PLY";
$resn_exch{" HG1 PLY"}="1HG  PLY"; $resn_exch{" HG2 PLY"}="2HG  PLY"; $resn_exch{" HG3 PLY"}="3HG  PLY";
$resn_exch{"H2A1 PLZ"}="1H2A PLZ"; $resn_exch{"H2A2 PLZ"}="2H2A PLZ"; $resn_exch{"H2A3 PLZ"}="3H2A PLZ";
$resn_exch{"H4A1 PLZ"}="1H4A PLZ"; $resn_exch{"H4A2 PLZ"}="2H4A PLZ"; $resn_exch{" H51 PLZ"}="1H5  PLZ";
$resn_exch{" H52 PLZ"}="2H5  PLZ"; $resn_exch{" H61 PLZ"}="1H6  PLZ"; $resn_exch{" H62 PLZ"}="2H6  PLZ";
$resn_exch{" H81 PLZ"}="1H8  PLZ"; $resn_exch{" H82 PLZ"}="2H8  PLZ"; $resn_exch{"H5M1 PLZ"}="1H5M PLZ";
$resn_exch{"H5M2 PLZ"}="2H5M PLZ"; $resn_exch{"H351 PM1"}="1H35 PM1"; $resn_exch{"H352 PM1"}="2H35 PM1";
$resn_exch{"H7C1 PM2"}="1H7C PM2"; $resn_exch{"H7C2 PM2"}="2H7C PM2"; $resn_exch{"H8N1 PM2"}="1H8N PM2";
$resn_exch{"H8N2 PM2"}="2H8N PM2"; $resn_exch{"H9C1 PM2"}="1H9C PM2"; $resn_exch{"H9C2 PM2"}="2H9C PM2";
$resn_exch{"H101 PM2"}="1H10 PM2"; $resn_exch{"H102 PM2"}="2H10 PM2"; $resn_exch{"H151 PM2"}="1H15 PM2";
$resn_exch{"H152 PM2"}="2H15 PM2"; $resn_exch{"H161 PM2"}="1H16 PM2"; $resn_exch{"H162 PM2"}="2H16 PM2";
$resn_exch{"H221 PM2"}="1H22 PM2"; $resn_exch{"H222 PM2"}="2H22 PM2"; $resn_exch{"H241 PM2"}="1H24 PM2";
$resn_exch{"H242 PM2"}="2H24 PM2"; $resn_exch{" H   PM3"}="1HN  PM3"; $resn_exch{" H2  PM3"}="2HN  PM3";
$resn_exch{" HB2 PM3"}="1HB  PM3"; $resn_exch{" HB3 PM3"}="2HB  PM3"; $resn_exch{"HH42 PM3"}="1HH4 PM3";
$resn_exch{"HH43 PM3"}="2HH4 PM3"; $resn_exch{"H281 PM4"}="1H28 PM4"; $resn_exch{"H282 PM4"}="2H28 PM4";
$resn_exch{"H301 PM4"}="1H30 PM4"; $resn_exch{"H302 PM4"}="2H30 PM4"; $resn_exch{"H303 PM4"}="3H30 PM4";
$resn_exch{"H311 PM4"}="1H31 PM4"; $resn_exch{"H312 PM4"}="2H31 PM4"; $resn_exch{"H313 PM4"}="3H31 PM4";
$resn_exch{"H371 PM4"}="1H37 PM4"; $resn_exch{"H372 PM4"}="2H37 PM4"; $resn_exch{"H381 PM4"}="1H38 PM4";
$resn_exch{"H382 PM4"}="2H38 PM4"; $resn_exch{"H421 PM4"}="1H42 PM4"; $resn_exch{"H422 PM4"}="2H42 PM4";
$resn_exch{"H431 PM4"}="1H43 PM4"; $resn_exch{"H432 PM4"}="2H43 PM4"; $resn_exch{" H21 PM4"}="1H2  PM4";
$resn_exch{" H22 PM4"}="2H2  PM4"; $resn_exch{" H31 PM4"}="1H3  PM4"; $resn_exch{"H32A PM4"}="2H3  PM4";
$resn_exch{"H41A PM4"}="1H4  PM4"; $resn_exch{" H42 PM4"}="2H4  PM4"; $resn_exch{" H51 PM4"}="1H5  PM4";
$resn_exch{" H52 PM4"}="2H5  PM4"; $resn_exch{" H61 PM4"}="1H6  PM4"; $resn_exch{" H62 PM4"}="2H6  PM4";
$resn_exch{" H63 PM4"}="3H6  PM4"; $resn_exch{"H281 PM5"}="1H28 PM5"; $resn_exch{"H282 PM5"}="2H28 PM5";
$resn_exch{"H301 PM5"}="1H30 PM5"; $resn_exch{"H302 PM5"}="2H30 PM5"; $resn_exch{"H303 PM5"}="3H30 PM5";
$resn_exch{"H311 PM5"}="1H31 PM5"; $resn_exch{"H312 PM5"}="2H31 PM5"; $resn_exch{"H313 PM5"}="3H31 PM5";
$resn_exch{"H371 PM5"}="1H37 PM5"; $resn_exch{"H372 PM5"}="2H37 PM5"; $resn_exch{"H381 PM5"}="1H38 PM5";
$resn_exch{"H382 PM5"}="2H38 PM5"; $resn_exch{"H421 PM5"}="1H42 PM5"; $resn_exch{"H422 PM5"}="2H42 PM5";
$resn_exch{"H431 PM5"}="1H43 PM5"; $resn_exch{"H432 PM5"}="2H43 PM5"; $resn_exch{" H21 PM5"}="1H2  PM5";
$resn_exch{" H22 PM5"}="2H2  PM5"; $resn_exch{" H31 PM5"}="1H3  PM5"; $resn_exch{"H32A PM5"}="2H3  PM5";
$resn_exch{"H41A PM5"}="1H4  PM5"; $resn_exch{" H42 PM5"}="2H4  PM5"; $resn_exch{" H51 PM5"}="1H5  PM5";
$resn_exch{" H52 PM5"}="2H5  PM5"; $resn_exch{" H61 PM5"}="1H6  PM5"; $resn_exch{" H62 PM5"}="2H6  PM5";
$resn_exch{" H71 PM5"}="1H7  PM5"; $resn_exch{" H72 PM5"}="2H7  PM5"; $resn_exch{" H73 PM5"}="3H7  PM5";
$resn_exch{"H281 PM8"}="1H28 PM8"; $resn_exch{"H282 PM8"}="2H28 PM8"; $resn_exch{"H301 PM8"}="1H30 PM8";
$resn_exch{"H302 PM8"}="2H30 PM8"; $resn_exch{"H303 PM8"}="3H30 PM8"; $resn_exch{"H311 PM8"}="1H31 PM8";
$resn_exch{"H312 PM8"}="2H31 PM8"; $resn_exch{"H313 PM8"}="3H31 PM8"; $resn_exch{"H371 PM8"}="1H37 PM8";
$resn_exch{"H372 PM8"}="2H37 PM8"; $resn_exch{"H381 PM8"}="1H38 PM8"; $resn_exch{"H382 PM8"}="2H38 PM8";
$resn_exch{"H421 PM8"}="1H42 PM8"; $resn_exch{"H422 PM8"}="2H42 PM8"; $resn_exch{"H431 PM8"}="1H43 PM8";
$resn_exch{"H432 PM8"}="2H43 PM8"; $resn_exch{" H21 PM8"}="1H2  PM8"; $resn_exch{" H22 PM8"}="2H2  PM8";
$resn_exch{" H31 PM8"}="1H3  PM8"; $resn_exch{"H32A PM8"}="2H3  PM8"; $resn_exch{"H41A PM8"}="1H4  PM8";
$resn_exch{" H42 PM8"}="2H4  PM8"; $resn_exch{" H51 PM8"}="1H5  PM8"; $resn_exch{" H52 PM8"}="2H5  PM8";
$resn_exch{" H61 PM8"}="1H6  PM8"; $resn_exch{" H62 PM8"}="2H6  PM8"; $resn_exch{" H71 PM8"}="1H7  PM8";
$resn_exch{" H72 PM8"}="2H7  PM8"; $resn_exch{" H81 PM8"}="1H8  PM8"; $resn_exch{" H82 PM8"}="2H8  PM8";
$resn_exch{" H91 PM8"}="1H9  PM8"; $resn_exch{" H92 PM8"}="2H9  PM8"; $resn_exch{"H101 PM8"}="1H10 PM8";
$resn_exch{"H102 PM8"}="2H10 PM8"; $resn_exch{"H103 PM8"}="3H10 PM8"; $resn_exch{"HCA1 PMC"}="1HCA PMC";
$resn_exch{"HCA2 PMC"}="2HCA PMC"; $resn_exch{"HC2O PMC"}="OHC2 PMC"; $resn_exch{"H7'1 PMD"}="1H7* PMD";
$resn_exch{"H9'1 PMD"}="1H9* PMD"; $resn_exch{"H9'2 PMD"}="2H9* PMD"; $resn_exch{"H1'1 PMD"}="1H1' PMD";
$resn_exch{"H2'1 PMD"}="1H2' PMD"; $resn_exch{"H3'1 PMD"}="1H3' PMD"; $resn_exch{"H4'1 PMD"}="1H4' PMD";
$resn_exch{"H6'1 PMD"}="1H6' PMD"; $resn_exch{" H61 PMD"}="1H6  PMD"; $resn_exch{"H8'1 PMD"}="1H8' PMD";
$resn_exch{" H9B PMD"}="1H9' PMD"; $resn_exch{"H10' PMD"}="'H10 PMD"; $resn_exch{"H11' PMD"}="'H11 PMD";
$resn_exch{"HN21 PMD"}="1HN2 PMD"; $resn_exch{"HN22 PMD"}="2HN2 PMD"; $resn_exch{"HN41 PMD"}="1HN4 PMD";
$resn_exch{"HN42 PMD"}="2HN4 PMD"; $resn_exch{"HN11 PME"}="1HN1 PME"; $resn_exch{"HN12 PME"}="2HN1 PME";
$resn_exch{" H31 PME"}="1H3  PME"; $resn_exch{" H32 PME"}="2H3  PME"; $resn_exch{"H111 PME"}="1H11 PME";
$resn_exch{"H112 PME"}="2H11 PME"; $resn_exch{"H211 PME"}="1H21 PME"; $resn_exch{"H212 PME"}="2H21 PME";
$resn_exch{"H213 PME"}="3H21 PME"; $resn_exch{"H2A1 PMG"}="1H2A PMG"; $resn_exch{"H2A2 PMG"}="2H2A PMG";
$resn_exch{"H2A3 PMG"}="3H2A PMG"; $resn_exch{"H4A1 PMG"}="1H4A PMG"; $resn_exch{"H4A2 PMG"}="2H4A PMG";
$resn_exch{"H5A1 PMG"}="1H5A PMG"; $resn_exch{"H5A2 PMG"}="2H5A PMG"; $resn_exch{"HB21 PMG"}="1HB2 PMG";
$resn_exch{"HB22 PMG"}="2HB2 PMG"; $resn_exch{"HB23 PMG"}="3HB2 PMG"; $resn_exch{"HB11 PMG"}="1HB1 PMG";
$resn_exch{"HB12 PMG"}="2HB1 PMG"; $resn_exch{" HG1 PMG"}="1HG  PMG"; $resn_exch{" HG2 PMG"}="2HG  PMG";
$resn_exch{"H2A1 PMH"}="1H2A PMH"; $resn_exch{"H2A2 PMH"}="2H2A PMH"; $resn_exch{"H2A3 PMH"}="3H2A PMH";
$resn_exch{"H4A1 PMH"}="1H4A PMH"; $resn_exch{"H4A2 PMH"}="2H4A PMH"; $resn_exch{"H5A1 PMH"}="1H5A PMH";
$resn_exch{"H5A2 PMH"}="2H5A PMH"; $resn_exch{"H9'1 PMI"}="1H9' PMI"; $resn_exch{"H9'2 PMI"}="2H9' PMI";
$resn_exch{"H111 PMI"}="1H11 PMI"; $resn_exch{"H112 PMI"}="2H11 PMI"; $resn_exch{"H7'1 PMI"}="1H7' PMI";
$resn_exch{"H7'2 PMI"}="2H7' PMI"; $resn_exch{"HC21 PML"}="1HC2 PML"; $resn_exch{"HC22 PML"}="2HC2 PML";
$resn_exch{"HC31 PML"}="1HC3 PML"; $resn_exch{"HC32 PML"}="2HC3 PML"; $resn_exch{"HC41 PML"}="1HC4 PML";
$resn_exch{"HC42 PML"}="2HC4 PML"; $resn_exch{"HC51 PML"}="1HC5 PML"; $resn_exch{"HC52 PML"}="2HC5 PML";
$resn_exch{"HC61 PML"}="1HC6 PML"; $resn_exch{"HC62 PML"}="2HC6 PML"; $resn_exch{"HN21 PMM"}="1HN2 PMM";
$resn_exch{"HN22 PMM"}="2HN2 PMM"; $resn_exch{" H91 PMM"}="1H9  PMM"; $resn_exch{" H92 PMM"}="2H9  PMM";
$resn_exch{"HOP2 PMM"}="2HOP PMM"; $resn_exch{"HOP3 PMM"}="3HOP PMM"; $resn_exch{" H21 PMN"}="1H2  PMN";
$resn_exch{" H22 PMN"}="2H2  PMN"; $resn_exch{" H31 PMN"}="1H3  PMN"; $resn_exch{" H32 PMN"}="2H3  PMN";
$resn_exch{"H111 PMN"}="1H11 PMN"; $resn_exch{"H112 PMN"}="2H11 PMN"; $resn_exch{"H131 PMN"}="1H13 PMN";
$resn_exch{"H132 PMN"}="2H13 PMN"; $resn_exch{"H141 PMN"}="1H14 PMN"; $resn_exch{"H142 PMN"}="2H14 PMN";
$resn_exch{"H171 PMN"}="1H17 PMN"; $resn_exch{"H172 PMN"}="2H17 PMN"; $resn_exch{"H173 PMN"}="3H17 PMN";
$resn_exch{"H181 PMN"}="1H18 PMN"; $resn_exch{"H182 PMN"}="2H18 PMN"; $resn_exch{"H183 PMN"}="3H18 PMN";
$resn_exch{" H21 PMO"}="1H2  PMO"; $resn_exch{" H41 PMO"}="1H4  PMO"; $resn_exch{" H61 PMO"}="1H6  PMO";
$resn_exch{" H71 PMO"}="1H7  PMO"; $resn_exch{"H111 PMO"}="1H11 PMO"; $resn_exch{"H112 PMO"}="2H11 PMO";
$resn_exch{"H113 PMO"}="3H11 PMO"; $resn_exch{"HOP2 PMO"}="2HOP PMO"; $resn_exch{"HOP3 PMO"}="3HOP PMO";
$resn_exch{"H5'1 PMO"}="1H5* PMO"; $resn_exch{"H5'2 PMO"}="2H5* PMO"; $resn_exch{" H4' PMO"}=" H4* PMO";
$resn_exch{" H3' PMO"}=" H3* PMO"; $resn_exch{"HO3' PMO"}="*HO3 PMO"; $resn_exch{" H2' PMO"}=" H2* PMO";
$resn_exch{"HO2' PMO"}="*HO2 PMO"; $resn_exch{" H1' PMO"}=" H1* PMO"; $resn_exch{"H2A1 PMP"}="1H2A PMP";
$resn_exch{"H2A2 PMP"}="2H2A PMP"; $resn_exch{"H2A3 PMP"}="3H2A PMP"; $resn_exch{"H4A1 PMP"}="1H4A PMP";
$resn_exch{"H4A2 PMP"}="2H4A PMP"; $resn_exch{"HNA1 PMP"}="1HNA PMP"; $resn_exch{"HNA2 PMP"}="2HNA PMP";
$resn_exch{"H5A1 PMP"}="1H5A PMP"; $resn_exch{"H5A2 PMP"}="2H5A PMP"; $resn_exch{"HOP2 PMP"}="2HOP PMP";
$resn_exch{"HOP3 PMP"}="3HOP PMP"; $resn_exch{" H1  PMS"}="1H   PMS"; $resn_exch{" H2A PMS"}="2H   PMS";
$resn_exch{"HOS2 PMS"}="2HOS PMS"; $resn_exch{"H7'1 PMT"}="1H7' PMT"; $resn_exch{"H7'2 PMT"}="2H7' PMT";
$resn_exch{"H9'1 PMT"}="1H9' PMT"; $resn_exch{"H9'2 PMT"}="2H9' PMT"; $resn_exch{"H9'3 PMT"}="3H9' PMT";
$resn_exch{"H8'1 PMT"}="1H8' PMT"; $resn_exch{"H8'2 PMT"}="2H8' PMT"; $resn_exch{"H8'3 PMT"}="3H8' PMT";
$resn_exch{"H3'1 PMT"}="1H3' PMT"; $resn_exch{"H3'2 PMT"}="2H3' PMT"; $resn_exch{"H2'1 PMT"}="1H2' PMT";
$resn_exch{"H2'2 PMT"}="2H2' PMT"; $resn_exch{"H5'1 PMT"}="1H5* PMT"; $resn_exch{"H5'' PMT"}="2H5* PMT";
$resn_exch{" H4' PMT"}=" H4* PMT"; $resn_exch{" H3B PMT"}=" H3* PMT"; $resn_exch{" H2' PMT"}=" H2* PMT";
$resn_exch{" H1' PMT"}=" H1* PMT"; $resn_exch{"HN41 PMT"}="1HN4 PMT"; $resn_exch{"HN42 PMT"}="2HN4 PMT";
$resn_exch{" H51 PMV"}="1H5  PMV"; $resn_exch{" H52 PMV"}="2H5  PMV"; $resn_exch{" H41 PMV"}="1H4  PMV";
$resn_exch{" H42 PMV"}="2H4  PMV"; $resn_exch{"H3A1 PMV"}="1H3A PMV"; $resn_exch{"H3A2 PMV"}="2H3A PMV";
$resn_exch{"H3A3 PMV"}="3H3A PMV"; $resn_exch{" H21 PMV"}="1H2  PMV"; $resn_exch{" H22 PMV"}="2H2  PMV";
$resn_exch{"HA11 PMZ"}="1HA1 PMZ"; $resn_exch{"HA12 PMZ"}="2HA1 PMZ"; $resn_exch{"HB11 PMZ"}="1HB1 PMZ";
$resn_exch{"HB12 PMZ"}="2HB1 PMZ"; $resn_exch{"HE11 PMZ"}="1HE1 PMZ"; $resn_exch{"HE12 PMZ"}="2HE1 PMZ";
$resn_exch{"HE13 PMZ"}="3HE1 PMZ"; $resn_exch{"HE21 PMZ"}="1HE2 PMZ"; $resn_exch{"HE22 PMZ"}="2HE2 PMZ";
$resn_exch{"HE23 PMZ"}="3HE2 PMZ"; $resn_exch{"HB21 PMZ"}="1HB2 PMZ"; $resn_exch{"HB22 PMZ"}="2HB2 PMZ";
$resn_exch{"HB23 PMZ"}="3HB2 PMZ"; $resn_exch{"H1C1 PN1"}="1H1C PN1"; $resn_exch{"H1C2 PN1"}="2H1C PN1";
$resn_exch{"H1C3 PN1"}="3H1C PN1"; $resn_exch{"H161 PN1"}="1H16 PN1"; $resn_exch{"H162 PN1"}="2H16 PN1";
$resn_exch{"H163 PN1"}="3H16 PN1"; $resn_exch{"H2N1 PN1"}="1H2N PN1"; $resn_exch{"H2N2 PN1"}="2H2N PN1";
$resn_exch{" HN1 PN2"}="1HN  PN2"; $resn_exch{" HN2 PN2"}="2HN  PN2"; $resn_exch{"HCB1 PN2"}="1HCB PN2";
$resn_exch{"HCB2 PN2"}="2HCB PN2"; $resn_exch{"HOP6 PN2"}="6HOP PN2"; $resn_exch{"H121 PN2"}="1H12 PN2";
$resn_exch{"H122 PN2"}="2H12 PN2"; $resn_exch{"H131 PN2"}="1H13 PN2"; $resn_exch{"H132 PN2"}="2H13 PN2";
$resn_exch{"H133 PN2"}="3H13 PN2"; $resn_exch{"H141 PN2"}="1H14 PN2"; $resn_exch{"H142 PN2"}="2H14 PN2";
$resn_exch{"H143 PN2"}="3H14 PN2"; $resn_exch{" H10 PN2"}="10H  PN2"; $resn_exch{"H10O PN2"}="10OH PN2";
$resn_exch{" HN8 PN2"}="8HN  PN2"; $resn_exch{" H71 PN2"}="1H7  PN2"; $resn_exch{" H72 PN2"}="2H7  PN2";
$resn_exch{"H421 PN2"}="1H42 PN2"; $resn_exch{"H422 PN2"}="2H42 PN2"; $resn_exch{" HN4 PN2"}="4HN  PN2";
$resn_exch{" H31 PN2"}="1H3  PN2"; $resn_exch{" H2  PN2"}="2H   PN2"; $resn_exch{"H471 PN2"}="1H47 PN2";
$resn_exch{"H472 PN2"}="2H47 PN2"; $resn_exch{" H61 PNA"}="1H6  PNA"; $resn_exch{" H62 PNA"}="2H6  PNA";
$resn_exch{" H71 PNB"}="1H7  PNB"; $resn_exch{" H72 PNB"}="2H7  PNB"; $resn_exch{" H81 PNB"}="1H8  PNB";
$resn_exch{" H82 PNB"}="2H8  PNB"; $resn_exch{" H91 PNB"}="1H9  PNB"; $resn_exch{" H92 PNB"}="2H9  PNB";
$resn_exch{"H101 PNB"}="1H10 PNB"; $resn_exch{"H102 PNB"}="2H10 PNB"; $resn_exch{"H121 PNB"}="1H12 PNB";
$resn_exch{"H122 PNB"}="2H12 PNB"; $resn_exch{" H71 PNC"}="1H7  PNC"; $resn_exch{" H72 PNC"}="2H7  PNC";
$resn_exch{" H81 PNC"}="1H8  PNC"; $resn_exch{" H82 PNC"}="2H8  PNC"; $resn_exch{" H91 PNC"}="1H9  PNC";
$resn_exch{" H92 PNC"}="2H9  PNC"; $resn_exch{"H101 PNC"}="1H10 PNC"; $resn_exch{"H102 PNC"}="2H10 PNC";
$resn_exch{"H121 PNC"}="1H12 PNC"; $resn_exch{"H122 PNC"}="2H12 PNC"; $resn_exch{"HN11 PND"}="1HN1 PND";
$resn_exch{"HN12 PND"}="2HN1 PND"; $resn_exch{"HC81 PNE"}="1HC8 PNE"; $resn_exch{"HC82 PNE"}="2HC8 PNE";
$resn_exch{"HC91 PNE"}="1HC9 PNE"; $resn_exch{"HC92 PNE"}="2HC9 PNE"; $resn_exch{"H101 PNE"}="1H10 PNE";
$resn_exch{"H102 PNE"}="2H10 PNE"; $resn_exch{"H1L1 PNE"}="1H1L PNE"; $resn_exch{"H1L2 PNE"}="2H1L PNE";
$resn_exch{"H1L3 PNE"}="3H1L PNE"; $resn_exch{" H21 PNF"}="1H2  PNF"; $resn_exch{" H31 PNF"}="1H3  PNF";
$resn_exch{" H51 PNF"}="1H5  PNF"; $resn_exch{" H61 PNF"}="1H6  PNF"; $resn_exch{"HO21 PNF"}="1HO2 PNF";
$resn_exch{" H81 PNF"}="1H8  PNF"; $resn_exch{" H82 PNF"}="2H8  PNF"; $resn_exch{" H91 PNF"}="1H9  PNF";
$resn_exch{" H92 PNF"}="2H9  PNF"; $resn_exch{"H101 PNF"}="1H10 PNF"; $resn_exch{"H102 PNF"}="2H10 PNF";
$resn_exch{"HN21 PNF"}="1HN2 PNF"; $resn_exch{"H121 PNF"}="1H12 PNF"; $resn_exch{"H122 PNF"}="2H12 PNF";
$resn_exch{"H131 PNF"}="1H13 PNF"; $resn_exch{"H132 PNF"}="2H13 PNF"; $resn_exch{"H141 PNF"}="1H14 PNF";
$resn_exch{"H142 PNF"}="2H14 PNF"; $resn_exch{"H151 PNF"}="1H15 PNF"; $resn_exch{"H152 PNF"}="2H15 PNF";
$resn_exch{"H161 PNF"}="1H16 PNF"; $resn_exch{"H162 PNF"}="2H16 PNF"; $resn_exch{"HO81 PNF"}="1HO8 PNF";
$resn_exch{" H61 PNG"}="1H6  PNG"; $resn_exch{" H62 PNG"}="2H6  PNG"; $resn_exch{"HMA1 PNI"}="1HMA PNI";
$resn_exch{"HMA2 PNI"}="2HMA PNI"; $resn_exch{"HMA3 PNI"}="3HMA PNI"; $resn_exch{"HMB1 PNI"}="1HMB PNI";
$resn_exch{"HMB2 PNI"}="2HMB PNI"; $resn_exch{"HMB3 PNI"}="3HMB PNI"; $resn_exch{"HMC1 PNI"}="1HMC PNI";
$resn_exch{"HMC2 PNI"}="2HMC PNI"; $resn_exch{"HMC3 PNI"}="3HMC PNI"; $resn_exch{"HMD1 PNI"}="1HMD PNI";
$resn_exch{"HMD2 PNI"}="2HMD PNI"; $resn_exch{"HMD3 PNI"}="3HMD PNI"; $resn_exch{" H21 PNL"}="1H2  PNL";
$resn_exch{" H22 PNL"}="2H2  PNL"; $resn_exch{" H31 PNL"}="1H3  PNL"; $resn_exch{" H32 PNL"}="2H3  PNL";
$resn_exch{" H33 PNL"}="3H3  PNL"; $resn_exch{"H101 PNM"}="1H10 PNM"; $resn_exch{"H102 PNM"}="2H10 PNM";
$resn_exch{"H103 PNM"}="3H10 PNM"; $resn_exch{"HC91 PNM"}="1HC9 PNM"; $resn_exch{"HC92 PNM"}="2HC9 PNM";
$resn_exch{"HC93 PNM"}="3HC9 PNM"; $resn_exch{"H171 PNM"}="1H17 PNM"; $resn_exch{"H172 PNM"}="2H17 PNM";
$resn_exch{"H101 PNN"}="1H10 PNN"; $resn_exch{"H102 PNN"}="2H10 PNN"; $resn_exch{"H103 PNN"}="3H10 PNN";
$resn_exch{"HC91 PNN"}="1HC9 PNN"; $resn_exch{"HC92 PNN"}="2HC9 PNN"; $resn_exch{"HC93 PNN"}="3HC9 PNN";
$resn_exch{"H171 PNN"}="1H17 PNN"; $resn_exch{"H172 PNN"}="2H17 PNN"; $resn_exch{" HN1 PNO"}="1HN  PNO";
$resn_exch{" HN2 PNO"}="2HN  PNO"; $resn_exch{" H1  PNO"}="1H   PNO"; $resn_exch{" H2  PNO"}="2H   PNO";
$resn_exch{" H3  PNO"}="3H   PNO"; $resn_exch{" HM1 PNP"}="1HM  PNP"; $resn_exch{" HM2 PNP"}="2HM  PNP";
$resn_exch{" HM3 PNP"}="3HM  PNP"; $resn_exch{"HOP3 PNP"}="3HOP PNP"; $resn_exch{"HOB2 PNQ"}="2HOB PNQ";
$resn_exch{"HOA2 PNQ"}="2HOA PNQ"; $resn_exch{"HNA1 PNQ"}="1HNA PNQ"; $resn_exch{"HA21 PNQ"}="1HA2 PNQ";
$resn_exch{"HA22 PNQ"}="2HA2 PNQ"; $resn_exch{"HA11 PNQ"}="1HA1 PNQ"; $resn_exch{"HA12 PNQ"}="2HA1 PNQ";
$resn_exch{"HOP1 PNS"}="1HOP PNS"; $resn_exch{"HOP2 PNS"}="2HOP PNS"; $resn_exch{"H282 PNS"}="2H28 PNS";
$resn_exch{"H281 PNS"}="1H28 PNS"; $resn_exch{"H303 PNS"}="3H30 PNS"; $resn_exch{"H302 PNS"}="2H30 PNS";
$resn_exch{"H301 PNS"}="1H30 PNS"; $resn_exch{"H313 PNS"}="3H31 PNS"; $resn_exch{"H312 PNS"}="2H31 PNS";
$resn_exch{"H311 PNS"}="1H31 PNS"; $resn_exch{"H372 PNS"}="2H37 PNS"; $resn_exch{"H371 PNS"}="1H37 PNS";
$resn_exch{"H382 PNS"}="2H38 PNS"; $resn_exch{"H381 PNS"}="1H38 PNS"; $resn_exch{"H422 PNS"}="2H42 PNS";
$resn_exch{"H421 PNS"}="1H42 PNS"; $resn_exch{"H431 PNS"}="1H43 PNS"; $resn_exch{"H432 PNS"}="2H43 PNS";
$resn_exch{" H71 PNT"}="1H7  PNT"; $resn_exch{" H72 PNT"}="2H7  PNT"; $resn_exch{" H81 PNT"}="1H8  PNT";
$resn_exch{" H82 PNT"}="2H8  PNT"; $resn_exch{"H101 PNT"}="1H10 PNT"; $resn_exch{"H102 PNT"}="2H10 PNT";
$resn_exch{"H7'1 PNT"}="1H7' PNT"; $resn_exch{"H7'2 PNT"}="2H7' PNT"; $resn_exch{"H8'1 PNT"}="1H8' PNT";
$resn_exch{"H8'2 PNT"}="2H8' PNT"; $resn_exch{" HN1 PNT"}="1HN  PNT"; $resn_exch{" HN2 PNT"}="2HN  PNT";
$resn_exch{" HN3 PNT"}="3HN  PNT"; $resn_exch{"HN'1 PNT"}="1HN' PNT"; $resn_exch{"HN'2 PNT"}="2HN' PNT";
$resn_exch{"HN'3 PNT"}="3HN' PNT"; $resn_exch{"H11C PNU"}="CH11 PNU"; $resn_exch{"H13C PNU"}="CH13 PNU";
$resn_exch{"H15C PNU"}="CH15 PNU"; $resn_exch{"H16C PNU"}="CH16 PNU"; $resn_exch{"H181 PNU"}="1H18 PNU";
$resn_exch{"H182 PNU"}="2H18 PNU"; $resn_exch{"H211 PNU"}="1H21 PNU"; $resn_exch{"H212 PNU"}="2H21 PNU";
$resn_exch{"H213 PNU"}="3H21 PNU"; $resn_exch{"H21A PNV"}="1H2  PNV"; $resn_exch{"H22A PNV"}="2H2  PNV";
$resn_exch{"H121 PNV"}="1H12 PNV"; $resn_exch{"H122 PNV"}="2H12 PNV"; $resn_exch{"H123 PNV"}="3H12 PNV";
$resn_exch{"H151 PNV"}="1H15 PNV"; $resn_exch{"H152 PNV"}="2H15 PNV"; $resn_exch{"H153 PNV"}="3H15 PNV";
$resn_exch{"HAA1 PNX"}="1HAA PNX"; $resn_exch{"HAA2 PNX"}="2HAA PNX"; $resn_exch{"HAA3 PNX"}="3HAA PNX";
$resn_exch{"HAJ1 PNX"}="1HAJ PNX"; $resn_exch{"HAJ2 PNX"}="2HAJ PNX"; $resn_exch{"HAH1 PNX"}="1HAH PNX";
$resn_exch{"HAH2 PNX"}="2HAH PNX"; $resn_exch{"HAI1 PNX"}="1HAI PNX"; $resn_exch{"HAI2 PNX"}="2HAI PNX";
$resn_exch{"HAK1 PNX"}="1HAK PNX"; $resn_exch{"HAK2 PNX"}="2HAK PNX"; $resn_exch{"HAB1 PNX"}="1HAB PNX";
$resn_exch{"HAB2 PNX"}="2HAB PNX"; $resn_exch{"HAB3 PNX"}="3HAB PNX"; $resn_exch{"HAC1 PNX"}="1HAC PNX";
$resn_exch{"HAC2 PNX"}="2HAC PNX"; $resn_exch{"HAC3 PNX"}="3HAC PNX"; $resn_exch{"H2N1 PNZ"}="1H2N PNZ";
$resn_exch{"H2N2 PNZ"}="2H2N PNZ"; $resn_exch{"H3C1 PNZ"}="1H3C PNZ"; $resn_exch{"H3C2 PNZ"}="2H3C PNZ";
$resn_exch{" HC1 PO0"}="1HC  PO0"; $resn_exch{" HC2 PO0"}="2HC  PO0"; $resn_exch{" HB1 PO0"}="1HB  PO0";
$resn_exch{" HB2 PO0"}="2HB  PO0"; $resn_exch{"HN11 PO1"}="1HN1 PO1"; $resn_exch{"HN12 PO1"}="2HN1 PO1";
$resn_exch{" H71 PO1"}="1H7  PO1"; $resn_exch{" H72 PO1"}="2H7  PO1"; $resn_exch{" H73 PO1"}="3H7  PO1";
$resn_exch{"H101 PO1"}="1H10 PO1"; $resn_exch{"H102 PO1"}="2H10 PO1"; $resn_exch{"H111 PO1"}="1H11 PO1";
$resn_exch{"H112 PO1"}="2H11 PO1"; $resn_exch{"H121 PO1"}="1H12 PO1"; $resn_exch{"H122 PO1"}="2H12 PO1";
$resn_exch{"H131 PO1"}="1H13 PO1"; $resn_exch{"H132 PO1"}="2H13 PO1"; $resn_exch{"H161 PO1"}="1H16 PO1";
$resn_exch{"H162 PO1"}="2H16 PO1"; $resn_exch{"H163 PO1"}="3H16 PO1"; $resn_exch{"H181 PO1"}="1H18 PO1";
$resn_exch{"H182 PO1"}="2H18 PO1"; $resn_exch{"H191 PO1"}="1H19 PO1"; $resn_exch{"H192 PO1"}="2H19 PO1";
$resn_exch{"HN21 PO1"}="1HN2 PO1"; $resn_exch{"HN22 PO1"}="2HN2 PO1"; $resn_exch{" H11 POA"}="1H1  POA";
$resn_exch{" H12 POA"}="2H1  POA"; $resn_exch{" H31 POB"}="1H3  POB"; $resn_exch{" H32 POB"}="2H3  POB";
$resn_exch{" H41 POB"}="1H4  POB"; $resn_exch{" H42 POB"}="2H4  POB"; $resn_exch{" H43 POB"}="3H4  POB";
$resn_exch{"HOP5 POB"}="5HOP POB"; $resn_exch{"HOP6 POB"}="6HOP POB"; $resn_exch{"HN11 POB"}="1HN1 POB";
$resn_exch{"HN12 POB"}="2HN1 POB"; $resn_exch{"H2'1 POB"}="1H2' POB"; $resn_exch{"H2'2 POB"}="2H2' POB";
$resn_exch{"H2'3 POB"}="3H2' POB"; $resn_exch{" H41 POC"}="1H4  POC"; $resn_exch{" H42 POC"}="2H4  POC";
$resn_exch{" H51 POC"}="1H5  POC"; $resn_exch{" H52 POC"}="2H5  POC"; $resn_exch{" H61 POC"}="1H6  POC";
$resn_exch{" H62 POC"}="2H6  POC"; $resn_exch{" H63 POC"}="3H6  POC"; $resn_exch{" H71 POC"}="1H7  POC";
$resn_exch{" H72 POC"}="2H7  POC"; $resn_exch{" H73 POC"}="3H7  POC"; $resn_exch{" H81 POC"}="1H8  POC";
$resn_exch{" H82 POC"}="2H8  POC"; $resn_exch{" H83 POC"}="3H8  POC"; $resn_exch{"H121 POD"}="1H12 POD";
$resn_exch{"H122 POD"}="2H12 POD"; $resn_exch{"H11A POD"}="1H1  POD"; $resn_exch{" H12 POD"}="2H1  POD";
$resn_exch{"H201 POD"}="1H20 POD"; $resn_exch{"H202 POD"}="2H20 POD"; $resn_exch{"H203 POD"}="3H20 POD";
$resn_exch{"H211 POD"}="1H21 POD"; $resn_exch{"H212 POD"}="2H21 POD"; $resn_exch{"H213 POD"}="3H21 POD";
$resn_exch{"H221 POD"}="1H22 POD"; $resn_exch{"H222 POD"}="2H22 POD"; $resn_exch{"H223 POD"}="3H22 POD";
$resn_exch{" H11 POG"}="1H1  POG"; $resn_exch{"H12A POG"}="2H1  POG"; $resn_exch{" H13 POG"}="3H1  POG";
$resn_exch{" H31 POG"}="1H3  POG"; $resn_exch{" H32 POG"}="2H3  POG"; $resn_exch{" H41 POG"}="1H4  POG";
$resn_exch{" H42 POG"}="2H4  POG"; $resn_exch{" H71 POG"}="1H7  POG"; $resn_exch{" H72 POG"}="2H7  POG";
$resn_exch{" H91 POG"}="1H9  POG"; $resn_exch{" H92 POG"}="2H9  POG"; $resn_exch{"H111 POG"}="1H11 POG";
$resn_exch{"H112 POG"}="2H11 POG"; $resn_exch{"H131 POG"}="1H13 POG"; $resn_exch{"H132 POG"}="2H13 POG";
$resn_exch{"H151 POG"}="1H15 POG"; $resn_exch{"H152 POG"}="2H15 POG"; $resn_exch{"H161 POG"}="1H16 POG";
$resn_exch{"H162 POG"}="2H16 POG"; $resn_exch{"H163 POG"}="3H16 POG"; $resn_exch{"H171 POG"}="1H17 POG";
$resn_exch{"H172 POG"}="2H17 POG"; $resn_exch{"H173 POG"}="3H17 POG"; $resn_exch{"H181 POG"}="1H18 POG";
$resn_exch{"H182 POG"}="2H18 POG"; $resn_exch{"H183 POG"}="3H18 POG"; $resn_exch{"H191 POG"}="1H19 POG";
$resn_exch{"H192 POG"}="2H19 POG"; $resn_exch{"H193 POG"}="3H19 POG"; $resn_exch{"H201 POG"}="1H20 POG";
$resn_exch{"H202 POG"}="2H20 POG"; $resn_exch{"H203 POG"}="3H20 POG"; $resn_exch{"H211 POG"}="1H21 POG";
$resn_exch{"H212 POG"}="2H21 POG"; $resn_exch{"H213 POG"}="3H21 POG"; $resn_exch{" H51 POH"}="1H5  POH";
$resn_exch{"H112 POH"}="2H11 POH"; $resn_exch{" H11 POH"}="1H1  POH"; $resn_exch{" H31 POH"}="1H3  POH";
$resn_exch{" H41 POH"}="1H4  POH"; $resn_exch{"H192 POH"}="2H19 POH"; $resn_exch{"H811 POH"}="1H81 POH";
$resn_exch{"H821 POH"}="1H82 POH"; $resn_exch{"H831 POH"}="1H83 POH"; $resn_exch{" H21 POH"}="1H2  POH";
$resn_exch{" H61 POH"}="1H6  POH"; $resn_exch{" H71 POH"}="1H7  POH"; $resn_exch{" H73 POH"}="3H7  POH";
$resn_exch{" H52 POH"}="2H5  POH"; $resn_exch{"H211 POH"}="1H21 POH"; $resn_exch{" H12 POH"}="2H1  POH";
$resn_exch{"H272 POH"}="2H27 POH"; $resn_exch{" H32 POH"}="2H3  POH"; $resn_exch{"H281 POH"}="1H28 POH";
$resn_exch{" H42 POH"}="2H4  POH"; $resn_exch{"H291 POH"}="1H29 POH"; $resn_exch{"H812 POH"}="2H81 POH";
$resn_exch{"H822 POH"}="2H82 POH"; $resn_exch{"H832 POH"}="2H83 POH"; $resn_exch{" H22 POH"}="2H2  POH";
$resn_exch{" H62 POH"}="2H6  POH"; $resn_exch{" H53 POH"}="3H5  POH"; $resn_exch{"H311 POH"}="1H31 POH";
$resn_exch{" H13 POH"}="3H1  POH"; $resn_exch{"H373 POH"}="3H37 POH"; $resn_exch{" H33 POH"}="3H3  POH";
$resn_exch{"H381 POH"}="1H38 POH"; $resn_exch{" H43 POH"}="3H4  POH"; $resn_exch{"H391 POH"}="1H39 POH";
$resn_exch{"H813 POH"}="3H81 POH"; $resn_exch{"H823 POH"}="3H82 POH"; $resn_exch{"H833 POH"}="3H83 POH";
$resn_exch{" H23 POH"}="3H2  POH"; $resn_exch{" H63 POH"}="3H6  POH"; $resn_exch{" H54 POH"}="4H5  POH";
$resn_exch{"H411 POH"}="1H41 POH"; $resn_exch{" H14 POH"}="4H1  POH"; $resn_exch{"H474 POH"}="4H47 POH";
$resn_exch{" H34 POH"}="4H3  POH"; $resn_exch{"H481 POH"}="1H48 POH"; $resn_exch{" H44 POH"}="4H4  POH";
$resn_exch{"H491 POH"}="1H49 POH"; $resn_exch{"H814 POH"}="4H81 POH"; $resn_exch{"H824 POH"}="4H82 POH";
$resn_exch{"H834 POH"}="4H83 POH"; $resn_exch{" H24 POH"}="4H2  POH"; $resn_exch{" H64 POH"}="4H6  POH";
$resn_exch{" HM1 POI"}="1HM  POI"; $resn_exch{" HM2 POI"}="2HM  POI"; $resn_exch{" HM3 POI"}="3HM  POI";
$resn_exch{" HB1 POI"}="1HB  POI"; $resn_exch{" HB2 POI"}="2HB  POI"; $resn_exch{" HG1 POI"}="1HG  POI";
$resn_exch{" HG2 POI"}="2HG  POI"; $resn_exch{" HD1 POI"}="1HD  POI"; $resn_exch{" HD2 POI"}="2HD  POI";
$resn_exch{"H4A1 POI"}="1H4A POI"; $resn_exch{"H4A2 POI"}="2H4A POI"; $resn_exch{"H2A1 POI"}="1H2A POI";
$resn_exch{"H2A2 POI"}="2H2A POI"; $resn_exch{"H2A3 POI"}="3H2A POI"; $resn_exch{"H5A1 POI"}="1H5A POI";
$resn_exch{"H5A2 POI"}="2H5A POI"; $resn_exch{" H11 POL"}="1H1  POL"; $resn_exch{" H12 POL"}="2H1  POL";
$resn_exch{" H21 POL"}="1H2  POL"; $resn_exch{" H22 POL"}="2H2  POL"; $resn_exch{" H31 POL"}="1H3  POL";
$resn_exch{" H32 POL"}="2H3  POL"; $resn_exch{" H33 POL"}="3H3  POL"; $resn_exch{" H   POM"}=" HN  POM";
$resn_exch{" HB2 POM"}="1HB  POM"; $resn_exch{" HB3 POM"}="2HB  POM"; $resn_exch{"HD11 POM"}="1HD1 POM";
$resn_exch{"HD12 POM"}="2HD1 POM"; $resn_exch{"HD13 POM"}="3HD1 POM"; $resn_exch{"H281 POO"}="1H28 POO";
$resn_exch{"H282 POO"}="2H28 POO"; $resn_exch{"H291 POO"}="1H29 POO"; $resn_exch{"H292 POO"}="2H29 POO";
$resn_exch{"H301 POO"}="1H30 POO"; $resn_exch{"H302 POO"}="2H30 POO"; $resn_exch{"H311 POO"}="1H31 POO";
$resn_exch{"H312 POO"}="2H31 POO"; $resn_exch{"H313 POO"}="3H31 POO"; $resn_exch{"H271 POO"}="1H27 POO";
$resn_exch{"H272 POO"}="2H27 POO"; $resn_exch{"H251 POO"}="1H25 POO"; $resn_exch{"H252 POO"}="2H25 POO";
$resn_exch{"H241 POO"}="1H24 POO"; $resn_exch{"H242 POO"}="2H24 POO"; $resn_exch{"H243 POO"}="3H24 POO";
$resn_exch{"H221 POO"}="1H22 POO"; $resn_exch{"H222 POO"}="2H22 POO"; $resn_exch{"H171 POO"}="1H17 POO";
$resn_exch{"H172 POO"}="2H17 POO"; $resn_exch{"H201 POO"}="1H20 POO"; $resn_exch{"H202 POO"}="2H20 POO";
$resn_exch{"H211 POO"}="1H21 POO"; $resn_exch{"H212 POO"}="2H21 POO"; $resn_exch{"H191 POO"}="1H19 POO";
$resn_exch{"H192 POO"}="2H19 POO"; $resn_exch{"H181 POO"}="1H18 POO"; $resn_exch{"H182 POO"}="2H18 POO";
$resn_exch{" H61 POQ"}="1H6  POQ"; $resn_exch{" H62 POQ"}="2H6  POQ"; $resn_exch{"H151 POQ"}="1H15 POQ";
$resn_exch{"H152 POQ"}="2H15 POQ"; $resn_exch{"H161 POQ"}="1H16 POQ"; $resn_exch{"H162 POQ"}="2H16 POQ";
$resn_exch{"H163 POQ"}="3H16 POQ"; $resn_exch{"H6B1 POQ"}="1H6B POQ"; $resn_exch{"H6B2 POQ"}="2H6B POQ";
$resn_exch{"H251 POQ"}="1H25 POQ"; $resn_exch{"H252 POQ"}="2H25 POQ"; $resn_exch{"H261 POQ"}="1H26 POQ";
$resn_exch{"H262 POQ"}="2H26 POQ"; $resn_exch{"H263 POQ"}="3H26 POQ"; $resn_exch{"H181 POS"}="1H18 POS";
$resn_exch{"H182 POS"}="2H18 POS"; $resn_exch{"H211 POS"}="1H21 POS"; $resn_exch{"H212 POS"}="2H21 POS";
$resn_exch{"H261 POS"}="1H26 POS"; $resn_exch{"H262 POS"}="2H26 POS"; $resn_exch{"H281 POS"}="1H28 POS";
$resn_exch{"H282 POS"}="2H28 POS"; $resn_exch{"H283 POS"}="3H28 POS"; $resn_exch{"H291 POS"}="1H29 POS";
$resn_exch{"H292 POS"}="2H29 POS"; $resn_exch{"H293 POS"}="3H29 POS"; $resn_exch{" H41 POT"}="1H4  POT";
$resn_exch{" H42 POT"}="2H4  POT"; $resn_exch{" H43 POT"}="3H4  POT"; $resn_exch{" H31 POT"}="1H3  POT";
$resn_exch{" H32 POT"}="2H3  POT"; $resn_exch{" H51 POT"}="1H5  POT"; $resn_exch{" H52 POT"}="2H5  POT";
$resn_exch{"H11A POT"}="1H1  POT"; $resn_exch{" H12 POT"}="2H1  POT"; $resn_exch{" H13 POT"}="3H1  POT";
$resn_exch{"H101 PP1"}="1H10 PP1"; $resn_exch{"H102 PP1"}="2H10 PP1"; $resn_exch{"H241 PP1"}="1H24 PP1";
$resn_exch{"H242 PP1"}="2H24 PP1"; $resn_exch{"H243 PP1"}="3H24 PP1"; $resn_exch{"H291 PP1"}="1H29 PP1";
$resn_exch{"H292 PP1"}="2H29 PP1"; $resn_exch{"H293 PP1"}="3H29 PP1"; $resn_exch{"H331 PP1"}="1H33 PP1";
$resn_exch{"H332 PP1"}="2H33 PP1"; $resn_exch{"H333 PP1"}="3H33 PP1"; $resn_exch{"H371 PP1"}="1H37 PP1";
$resn_exch{"H372 PP1"}="2H37 PP1"; $resn_exch{"H373 PP1"}="3H37 PP1"; $resn_exch{"H2A1 PP3"}="1H2A PP3";
$resn_exch{"H2A2 PP3"}="2H2A PP3"; $resn_exch{"H2A3 PP3"}="3H2A PP3"; $resn_exch{"H4A1 PP3"}="1H4A PP3";
$resn_exch{"H4A2 PP3"}="2H4A PP3"; $resn_exch{"H5A1 PP3"}="1H5A PP3"; $resn_exch{"H5A2 PP3"}="2H5A PP3";
$resn_exch{"HOP2 PP3"}="2HOP PP3"; $resn_exch{"HOP3 PP3"}="3HOP PP3"; $resn_exch{" HB1 PP3"}="1HB  PP3";
$resn_exch{" HB2 PP3"}="2HB  PP3"; $resn_exch{" HB3 PP3"}="3HB  PP3"; $resn_exch{"H131 PP4"}="1H13 PP4";
$resn_exch{"H132 PP4"}="2H13 PP4"; $resn_exch{"HV11 PP4"}="1HV1 PP4"; $resn_exch{"HV12 PP4"}="2HV1 PP4";
$resn_exch{"HV13 PP4"}="3HV1 PP4"; $resn_exch{"HV21 PP4"}="1HV2 PP4"; $resn_exch{"HV22 PP4"}="2HV2 PP4";
$resn_exch{"HV23 PP4"}="3HV2 PP4"; $resn_exch{" HB1 PP4"}="1HB  PP4"; $resn_exch{" HB2 PP4"}="2HB  PP4";
$resn_exch{" HS1 PP4"}="1HS  PP4"; $resn_exch{" HS2 PP4"}="2HS  PP4"; $resn_exch{" HS3 PP4"}="3HS  PP4";
$resn_exch{"H111 PP5"}="1H11 PP5"; $resn_exch{"H112 PP5"}="2H11 PP5"; $resn_exch{"H141 PP5"}="1H14 PP5";
$resn_exch{"H142 PP5"}="2H14 PP5"; $resn_exch{"HV11 PP5"}="1HV1 PP5"; $resn_exch{"HV12 PP5"}="2HV1 PP5";
$resn_exch{"HV13 PP5"}="3HV1 PP5"; $resn_exch{"HV21 PP5"}="1HV2 PP5"; $resn_exch{"HV22 PP5"}="2HV2 PP5";
$resn_exch{"HV23 PP5"}="3HV2 PP5"; $resn_exch{" HB1 PP5"}="1HB  PP5"; $resn_exch{" HB2 PP5"}="2HB  PP5";
$resn_exch{" HS1 PP5"}="1HS  PP5"; $resn_exch{" HS2 PP5"}="2HS  PP5"; $resn_exch{" HS3 PP5"}="3HS  PP5";
$resn_exch{"H111 PP6"}="1H11 PP6"; $resn_exch{"H112 PP6"}="2H11 PP6"; $resn_exch{"H131 PP6"}="1H13 PP6";
$resn_exch{"H132 PP6"}="2H13 PP6"; $resn_exch{"HV11 PP6"}="1HV1 PP6"; $resn_exch{"HV12 PP6"}="2HV1 PP6";
$resn_exch{"HV13 PP6"}="3HV1 PP6"; $resn_exch{"HV21 PP6"}="1HV2 PP6"; $resn_exch{"HV22 PP6"}="2HV2 PP6";
$resn_exch{"HV23 PP6"}="3HV2 PP6"; $resn_exch{" HB1 PP6"}="1HB  PP6"; $resn_exch{" HB2 PP6"}="2HB  PP6";
$resn_exch{" HS1 PP6"}="1HS  PP6"; $resn_exch{" HS2 PP6"}="2HS  PP6"; $resn_exch{" HS3 PP6"}="3HS  PP6";
$resn_exch{"HV11 PP7"}="1HV1 PP7"; $resn_exch{"HV12 PP7"}="2HV1 PP7"; $resn_exch{"HV31 PP7"}="1HV3 PP7";
$resn_exch{"HV32 PP7"}="2HV3 PP7"; $resn_exch{"HV33 PP7"}="3HV3 PP7"; $resn_exch{"HV81 PP7"}="1HV8 PP7";
$resn_exch{"HV82 PP7"}="2HV8 PP7"; $resn_exch{"HV83 PP7"}="3HV8 PP7"; $resn_exch{"HV41 PP7"}="1HV4 PP7";
$resn_exch{"HV43 PP7"}="3HV4 PP7"; $resn_exch{"HV42 PP7"}="2HV4 PP7"; $resn_exch{"HV91 PP7"}="1HV9 PP7";
$resn_exch{"HV92 PP7"}="2HV9 PP7"; $resn_exch{"HV93 PP7"}="3HV9 PP7"; $resn_exch{"HC21 PP7"}="1HC2 PP7";
$resn_exch{"HC22 PP7"}="2HC2 PP7"; $resn_exch{"HCB1 PP7"}="1HCB PP7"; $resn_exch{"HCB2 PP7"}="2HCB PP7";
$resn_exch{"HBL1 PP7"}="1HBL PP7"; $resn_exch{"HBL2 PP7"}="2HBL PP7"; $resn_exch{"HL11 PP7"}="1HL1 PP7";
$resn_exch{"HL13 PP7"}="3HL1 PP7"; $resn_exch{"HL12 PP7"}="2HL1 PP7"; $resn_exch{"HL21 PP7"}="1HL2 PP7";
$resn_exch{"HL23 PP7"}="3HL2 PP7"; $resn_exch{"HL22 PP7"}="2HL2 PP7"; $resn_exch{"HC61 PP7"}="1HC6 PP7";
$resn_exch{"HC62 PP7"}="2HC6 PP7"; $resn_exch{"HNV2 PP8"}="2HNV PP8"; $resn_exch{"HV11 PP8"}="1HV1 PP8";
$resn_exch{"HV12 PP8"}="2HV1 PP8"; $resn_exch{"HV31 PP8"}="1HV3 PP8"; $resn_exch{"HV32 PP8"}="2HV3 PP8";
$resn_exch{"HV33 PP8"}="3HV3 PP8"; $resn_exch{"HV81 PP8"}="1HV8 PP8"; $resn_exch{"HV82 PP8"}="2HV8 PP8";
$resn_exch{"HV83 PP8"}="3HV8 PP8"; $resn_exch{"HV41 PP8"}="1HV4 PP8"; $resn_exch{"HV42 PP8"}="2HV4 PP8";
$resn_exch{"HV43 PP8"}="3HV4 PP8"; $resn_exch{"HV91 PP8"}="1HV9 PP8"; $resn_exch{"HV92 PP8"}="2HV9 PP8";
$resn_exch{"HV93 PP8"}="3HV9 PP8"; $resn_exch{" H21 PP8"}="1H2  PP8"; $resn_exch{" H22 PP8"}="2H2  PP8";
$resn_exch{"HN21 PP8"}="1HN2 PP8"; $resn_exch{"HN22 PP8"}="2HN2 PP8"; $resn_exch{"HBL1 PP8"}="1HBL PP8";
$resn_exch{"HBL2 PP8"}="2HBL PP8"; $resn_exch{"HL11 PP8"}="1HL1 PP8"; $resn_exch{"HL12 PP8"}="2HL1 PP8";
$resn_exch{"HL13 PP8"}="3HL1 PP8"; $resn_exch{"HL21 PP8"}="1HL2 PP8"; $resn_exch{"HL22 PP8"}="2HL2 PP8";
$resn_exch{"HL23 PP8"}="3HL2 PP8"; $resn_exch{" H61 PP8"}="1H6  PP8"; $resn_exch{" H62 PP8"}="2H6  PP8";
$resn_exch{" H71 PP8"}="1H7  PP8"; $resn_exch{" H72 PP8"}="2H7  PP8"; $resn_exch{" H73 PP8"}="3H7  PP8";
$resn_exch{"HMA1 PP9"}="1HMA PP9"; $resn_exch{"HMA2 PP9"}="2HMA PP9"; $resn_exch{"HMA3 PP9"}="3HMA PP9";
$resn_exch{"HAA1 PP9"}="1HAA PP9"; $resn_exch{"HAA2 PP9"}="2HAA PP9"; $resn_exch{"HBA1 PP9"}="1HBA PP9";
$resn_exch{"HBA2 PP9"}="2HBA PP9"; $resn_exch{"HMB1 PP9"}="1HMB PP9"; $resn_exch{"HMB2 PP9"}="2HMB PP9";
$resn_exch{"HMB3 PP9"}="3HMB PP9"; $resn_exch{"HBB1 PP9"}="1HBB PP9"; $resn_exch{"HBB2 PP9"}="2HBB PP9";
$resn_exch{"HMC1 PP9"}="1HMC PP9"; $resn_exch{"HMC2 PP9"}="2HMC PP9"; $resn_exch{"HMC3 PP9"}="3HMC PP9";
$resn_exch{"HBC1 PP9"}="1HBC PP9"; $resn_exch{"HBC2 PP9"}="2HBC PP9"; $resn_exch{"HMD1 PP9"}="1HMD PP9";
$resn_exch{"HMD2 PP9"}="2HMD PP9"; $resn_exch{"HMD3 PP9"}="3HMD PP9"; $resn_exch{"HAD1 PP9"}="1HAD PP9";
$resn_exch{"HAD2 PP9"}="2HAD PP9"; $resn_exch{"HBD1 PP9"}="1HBD PP9"; $resn_exch{"HBD2 PP9"}="2HBD PP9";
$resn_exch{" H31 PPB"}="1H3  PPB"; $resn_exch{" H32 PPB"}="2H3  PPB"; $resn_exch{" H91 PPB"}="1H9  PPB";
$resn_exch{" H92 PPB"}="2H9  PPB"; $resn_exch{"H101 PPB"}="1H10 PPB"; $resn_exch{"H102 PPB"}="2H10 PPB";
$resn_exch{"H111 PPB"}="1H11 PPB"; $resn_exch{"H112 PPB"}="2H11 PPB"; $resn_exch{"H131 PPB"}="1H13 PPB";
$resn_exch{"H132 PPB"}="2H13 PPB"; $resn_exch{"HN61 PPB"}="1HN6 PPB"; $resn_exch{"HN62 PPB"}="2HN6 PPB";
$resn_exch{" H51 PPC"}="1H5  PPC"; $resn_exch{" H52 PPC"}="2H5  PPC"; $resn_exch{"HO1P PPC"}="PHO1 PPC";
$resn_exch{"HO2P PPC"}="PHO2 PPC"; $resn_exch{"HO1A PPC"}="AHO1 PPC"; $resn_exch{"HO1B PPC"}="BHO1 PPC";
$resn_exch{"HO3B PPC"}="BHO3 PPC"; $resn_exch{"H3A1 PPC"}="1H3A PPC"; $resn_exch{"H3A2 PPC"}="2H3A PPC";
$resn_exch{"H2A1 PPD"}="1H2A PPD"; $resn_exch{"H2A2 PPD"}="2H2A PPD"; $resn_exch{"H2A3 PPD"}="3H2A PPD";
$resn_exch{"H4A1 PPD"}="1H4A PPD"; $resn_exch{"H4A2 PPD"}="2H4A PPD"; $resn_exch{"H5A1 PPD"}="1H5A PPD";
$resn_exch{"H5A2 PPD"}="2H5A PPD"; $resn_exch{"HOP2 PPD"}="2HOP PPD"; $resn_exch{"HOP3 PPD"}="3HOP PPD";
$resn_exch{" HB1 PPD"}="1HB  PPD"; $resn_exch{" HB2 PPD"}="2HB  PPD"; $resn_exch{"H2A1 PPE"}="1H2A PPE";
$resn_exch{"H2A2 PPE"}="2H2A PPE"; $resn_exch{"H2A3 PPE"}="3H2A PPE"; $resn_exch{"H4A1 PPE"}="1H4A PPE";
$resn_exch{"H4A2 PPE"}="2H4A PPE"; $resn_exch{"H5A1 PPE"}="1H5A PPE"; $resn_exch{"H5A2 PPE"}="2H5A PPE";
$resn_exch{"HOP2 PPE"}="2HOP PPE"; $resn_exch{"HOP3 PPE"}="3HOP PPE"; $resn_exch{"HBA1 PPE"}="1HBA PPE";
$resn_exch{"HBA2 PPE"}="2HBA PPE"; $resn_exch{"HGA1 PPE"}="1HGA PPE"; $resn_exch{"HGA2 PPE"}="2HGA PPE";
$resn_exch{"HOE2 PPE"}="2HOE PPE"; $resn_exch{"H2A1 PPG"}="1H2A PPG"; $resn_exch{"H2A2 PPG"}="2H2A PPG";
$resn_exch{"H2A3 PPG"}="3H2A PPG"; $resn_exch{"H4A1 PPG"}="1H4A PPG"; $resn_exch{"H4A2 PPG"}="2H4A PPG";
$resn_exch{"H5A1 PPG"}="1H5A PPG"; $resn_exch{"H5A2 PPG"}="2H5A PPG"; $resn_exch{"HOP2 PPG"}="2HOP PPG";
$resn_exch{"HOP3 PPG"}="3HOP PPG"; $resn_exch{"HOB3 PPG"}="3HOB PPG"; $resn_exch{"HEI1 PPG"}="1HEI PPG";
$resn_exch{"HEI2 PPG"}="2HEI PPG"; $resn_exch{"HFI1 PPG"}="1HFI PPG"; $resn_exch{"HFI2 PPG"}="2HFI PPG";
$resn_exch{"HNI1 PPG"}="1HNI PPG"; $resn_exch{"HNI2 PPG"}="2HNI PPG"; $resn_exch{" HN1 PPH"}="1HN  PPH";
$resn_exch{" HN2 PPH"}="2HN  PPH"; $resn_exch{"HOP2 PPH"}="2HOP PPH"; $resn_exch{" HB1 PPH"}="1HB  PPH";
$resn_exch{" HB2 PPH"}="2HB  PPH"; $resn_exch{" H21 PPI"}="1H2  PPI"; $resn_exch{" H22 PPI"}="2H2  PPI";
$resn_exch{" H31 PPI"}="1H3  PPI"; $resn_exch{" H32 PPI"}="2H3  PPI"; $resn_exch{" H33 PPI"}="3H3  PPI";
$resn_exch{"H2A1 PPJ"}="1H2A PPJ"; $resn_exch{"H2A2 PPJ"}="2H2A PPJ"; $resn_exch{"H2A3 PPJ"}="3H2A PPJ";
$resn_exch{"H5A1 PPJ"}="1H5A PPJ"; $resn_exch{"H5A2 PPJ"}="2H5A PPJ"; $resn_exch{" HB1 PPJ"}="1HB  PPJ";
$resn_exch{" HC1 PPJ"}="1HC  PPJ"; $resn_exch{" HC2 PPJ"}="2HC  PPJ"; $resn_exch{" HC3 PPJ"}="3HC  PPJ";
$resn_exch{" H31 PPL"}="1H3  PPL"; $resn_exch{" H32 PPL"}="2H3  PPL"; $resn_exch{" H41 PPL"}="1H4  PPL";
$resn_exch{" H42 PPL"}="2H4  PPL"; $resn_exch{" H51 PPL"}="1H5  PPL"; $resn_exch{" H52 PPL"}="2H5  PPL";
$resn_exch{" H61 PPL"}="1H6  PPL"; $resn_exch{" H62 PPL"}="2H6  PPL"; $resn_exch{"H2'1 PPL"}="1H2' PPL";
$resn_exch{"H2'2 PPL"}="2H2' PPL"; $resn_exch{"H2'3 PPL"}="3H2' PPL"; $resn_exch{"H3'1 PPL"}="1H3' PPL";
$resn_exch{"H3'2 PPL"}="2H3' PPL"; $resn_exch{"H3'3 PPL"}="3H3' PPL"; $resn_exch{"H4'1 PPL"}="1H4' PPL";
$resn_exch{"H4'2 PPL"}="2H4' PPL"; $resn_exch{"H4'3 PPL"}="3H4' PPL"; $resn_exch{" HM1 PPM"}="1HM  PPM";
$resn_exch{" HM2 PPM"}="2HM  PPM"; $resn_exch{" HM3 PPM"}="3HM  PPM"; $resn_exch{"HOP2 PPM"}="2HOP PPM";
$resn_exch{"HOP3 PPM"}="3HOP PPM"; $resn_exch{" H   PPN"}="1HN  PPN"; $resn_exch{" H2  PPN"}="2HN  PPN";
$resn_exch{" HB2 PPN"}="1HB  PPN"; $resn_exch{" HB3 PPN"}="2HB  PPN"; $resn_exch{"HC91 PPO"}="1HC9 PPO";
$resn_exch{"HC92 PPO"}="2HC9 PPO"; $resn_exch{" H11 PPP"}="1H1  PPP"; $resn_exch{" H12 PPP"}="2H1  PPP";
$resn_exch{" H31 PPP"}="1H3  PPP"; $resn_exch{" H32 PPP"}="2H3  PPP"; $resn_exch{"HNP1 PPQ"}="1HNP PPQ";
$resn_exch{"HNP2 PPQ"}="2HNP PPQ"; $resn_exch{"HBP1 PPQ"}="1HBP PPQ"; $resn_exch{"HBP2 PPQ"}="2HBP PPQ";
$resn_exch{"HGP1 PPQ"}="1HGP PPQ"; $resn_exch{"HGP2 PPQ"}="2HGP PPQ"; $resn_exch{"HEP1 PPQ"}="1HEP PPQ";
$resn_exch{"HEP2 PPQ"}="2HEP PPQ"; $resn_exch{"HEP3 PPQ"}="3HEP PPQ"; $resn_exch{"H2'O PPR"}="OH2' PPR";
$resn_exch{" H31 PPR"}="1H3  PPR"; $resn_exch{" H32 PPR"}="2H3  PPR"; $resn_exch{"HO5P PPS"}="PHO5 PPS";
$resn_exch{"H5'1 PPS"}="1H5* PPS"; $resn_exch{"H5'2 PPS"}="2H5* PPS"; $resn_exch{" H4' PPS"}=" H4* PPS";
$resn_exch{" H1' PPS"}=" H1* PPS"; $resn_exch{"HN61 PPS"}="1HN6 PPS"; $resn_exch{"HN62 PPS"}="2HN6 PPS";
$resn_exch{" H2' PPS"}=" H2* PPS"; $resn_exch{"HO2' PPS"}="*HO2 PPS"; $resn_exch{" H3' PPS"}=" H3* PPS";
$resn_exch{"HO1P PPS"}="PHO1 PPS"; $resn_exch{"HO2P PPS"}="PHO2 PPS"; $resn_exch{" HA1 PPT"}="1HA  PPT";
$resn_exch{" HA2 PPT"}="2HA  PPT"; $resn_exch{" HB1 PPT"}="1HB  PPT"; $resn_exch{" HB2 PPT"}="2HB  PPT";
$resn_exch{" HH1 PPT"}="1HH  PPT"; $resn_exch{" HH2 PPT"}="2HH  PPT"; $resn_exch{" HH3 PPT"}="3HH  PPT";
$resn_exch{"HOP2 PPU"}="2HOP PPU"; $resn_exch{"HOP3 PPU"}="3HOP PPU"; $resn_exch{" HN1 PPU"}="1HN  PPU";
$resn_exch{" HN2 PPU"}="2HN  PPU"; $resn_exch{" HB1 PPU"}="1HB  PPU"; $resn_exch{" HB2 PPU"}="2HB  PPU";
$resn_exch{" HM1 PPU"}="1HM  PPU"; $resn_exch{" HM2 PPU"}="2HM  PPU"; $resn_exch{" HM3 PPU"}="3HM  PPU";
$resn_exch{" H5' PPU"}="1H5* PPU"; $resn_exch{"H5'' PPU"}="2H5* PPU"; $resn_exch{" H4' PPU"}=" H4* PPU";
$resn_exch{" H3' PPU"}=" H3* PPU"; $resn_exch{"HN'3 PPU"}="3HN* PPU"; $resn_exch{" H2' PPU"}=" H2* PPU";
$resn_exch{"HO2' PPU"}="*HO2 PPU"; $resn_exch{" H1' PPU"}=" H1* PPU"; $resn_exch{" H91 PPU"}="1H9  PPU";
$resn_exch{" H92 PPU"}="2H9  PPU"; $resn_exch{" H93 PPU"}="3H9  PPU"; $resn_exch{"H101 PPU"}="1H10 PPU";
$resn_exch{"H102 PPU"}="2H10 PPU"; $resn_exch{"H103 PPU"}="3H10 PPU"; $resn_exch{"HOP2 PPW"}="2HOP PPW";
$resn_exch{"HN21 PPW"}="1HN2 PPW"; $resn_exch{"HN22 PPW"}="2HN2 PPW"; $resn_exch{" H2' PPW"}="1H2* PPW";
$resn_exch{"H2'' PPW"}="2H2* PPW"; $resn_exch{" H5' PPW"}="1H5* PPW"; $resn_exch{"H5'' PPW"}="2H5* PPW";
$resn_exch{" H4' PPW"}=" H4* PPW"; $resn_exch{" H1' PPW"}=" H1* PPW"; $resn_exch{" H3' PPW"}=" H3* PPW";
$resn_exch{"HO3' PPW"}="H3T  PPW"; $resn_exch{"HOP3 PPW"}="3HOP PPW"; $resn_exch{"HN11 PPX"}="1HN1 PPX";
$resn_exch{"HN12 PPX"}="2HN1 PPX"; $resn_exch{" H31 PPX"}="1H3  PPX"; $resn_exch{" H32 PPX"}="2H3  PPX";
$resn_exch{"H131 PPX"}="1H13 PPX"; $resn_exch{"H132 PPX"}="2H13 PPX"; $resn_exch{"H151 PPX"}="1H15 PPX";
$resn_exch{"H152 PPX"}="2H15 PPX"; $resn_exch{"H161 PPX"}="1H16 PPX"; $resn_exch{"H162 PPX"}="2H16 PPX";
$resn_exch{"H201 PPX"}="1H20 PPX"; $resn_exch{"H202 PPX"}="2H20 PPX"; $resn_exch{"H301 PPX"}="1H30 PPX";
$resn_exch{"H302 PPX"}="2H30 PPX"; $resn_exch{" H31 PPY"}="1H3  PPY"; $resn_exch{" H32 PPY"}="2H3  PPY";
$resn_exch{" H61 PPZ"}="1H6  PPZ"; $resn_exch{" H62 PPZ"}="2H6  PPZ"; $resn_exch{" H71 PPZ"}="1H7  PPZ";
$resn_exch{" H72 PPZ"}="2H7  PPZ"; $resn_exch{"HN21 PQ0"}="1HN2 PQ0"; $resn_exch{"HN22 PQ0"}="2HN2 PQ0";
$resn_exch{"HOP3 PQ1"}="3HOP PQ1"; $resn_exch{"HOP2 PQ1"}="2HOP PQ1"; $resn_exch{" H5' PQ1"}="1H5* PQ1";
$resn_exch{"H5'' PQ1"}="2H5* PQ1"; $resn_exch{" H4' PQ1"}=" H4* PQ1"; $resn_exch{" H1' PQ1"}=" H1* PQ1";
$resn_exch{"HN21 PQ1"}="1HN2 PQ1"; $resn_exch{"HN22 PQ1"}="2HN2 PQ1"; $resn_exch{"H101 PQ1"}="1H10 PQ1";
$resn_exch{"H102 PQ1"}="2H10 PQ1"; $resn_exch{"H111 PQ1"}="1H11 PQ1"; $resn_exch{"H112 PQ1"}="2H11 PQ1";
$resn_exch{" H2' PQ1"}=" H2* PQ1"; $resn_exch{" H3' PQ1"}=" H3* PQ1"; $resn_exch{" H61 PQ9"}="1H6  PQ9";
$resn_exch{" H62 PQ9"}="2H6  PQ9"; $resn_exch{" H63 PQ9"}="3H6  PQ9"; $resn_exch{" H91 PQ9"}="1H9  PQ9";
$resn_exch{" H92 PQ9"}="2H9  PQ9"; $resn_exch{" H93 PQ9"}="3H9  PQ9"; $resn_exch{"H111 PQ9"}="1H11 PQ9";
$resn_exch{"H112 PQ9"}="2H11 PQ9"; $resn_exch{"H141 PQ9"}="1H14 PQ9"; $resn_exch{"H142 PQ9"}="2H14 PQ9";
$resn_exch{"H143 PQ9"}="3H14 PQ9"; $resn_exch{"H151 PQ9"}="1H15 PQ9"; $resn_exch{"H152 PQ9"}="2H15 PQ9";
$resn_exch{"H161 PQ9"}="1H16 PQ9"; $resn_exch{"H162 PQ9"}="2H16 PQ9"; $resn_exch{"H191 PQ9"}="1H19 PQ9";
$resn_exch{"H192 PQ9"}="2H19 PQ9"; $resn_exch{"H193 PQ9"}="3H19 PQ9"; $resn_exch{"H201 PQ9"}="1H20 PQ9";
$resn_exch{"H202 PQ9"}="2H20 PQ9"; $resn_exch{"H211 PQ9"}="1H21 PQ9"; $resn_exch{"H212 PQ9"}="2H21 PQ9";
$resn_exch{"H241 PQ9"}="1H24 PQ9"; $resn_exch{"H242 PQ9"}="2H24 PQ9"; $resn_exch{"H243 PQ9"}="3H24 PQ9";
$resn_exch{"H251 PQ9"}="1H25 PQ9"; $resn_exch{"H252 PQ9"}="2H25 PQ9"; $resn_exch{"H261 PQ9"}="1H26 PQ9";
$resn_exch{"H262 PQ9"}="2H26 PQ9"; $resn_exch{"H291 PQ9"}="1H29 PQ9"; $resn_exch{"H292 PQ9"}="2H29 PQ9";
$resn_exch{"H293 PQ9"}="3H29 PQ9"; $resn_exch{"H301 PQ9"}="1H30 PQ9"; $resn_exch{"H302 PQ9"}="2H30 PQ9";
$resn_exch{"H311 PQ9"}="1H31 PQ9"; $resn_exch{"H312 PQ9"}="2H31 PQ9"; $resn_exch{"H341 PQ9"}="1H34 PQ9";
$resn_exch{"H342 PQ9"}="2H34 PQ9"; $resn_exch{"H343 PQ9"}="3H34 PQ9"; $resn_exch{"H351 PQ9"}="1H35 PQ9";
$resn_exch{"H352 PQ9"}="2H35 PQ9"; $resn_exch{"H361 PQ9"}="1H36 PQ9"; $resn_exch{"H362 PQ9"}="2H36 PQ9";
$resn_exch{"H391 PQ9"}="1H39 PQ9"; $resn_exch{"H392 PQ9"}="2H39 PQ9"; $resn_exch{"H393 PQ9"}="3H39 PQ9";
$resn_exch{"H401 PQ9"}="1H40 PQ9"; $resn_exch{"H402 PQ9"}="2H40 PQ9"; $resn_exch{"H411 PQ9"}="1H41 PQ9";
$resn_exch{"H412 PQ9"}="2H41 PQ9"; $resn_exch{"H441 PQ9"}="1H44 PQ9"; $resn_exch{"H442 PQ9"}="2H44 PQ9";
$resn_exch{"H443 PQ9"}="3H44 PQ9"; $resn_exch{"H451 PQ9"}="1H45 PQ9"; $resn_exch{"H452 PQ9"}="2H45 PQ9";
$resn_exch{"H453 PQ9"}="3H45 PQ9"; $resn_exch{"HN11 PQA"}="1HN1 PQA"; $resn_exch{"HN12 PQA"}="2HN1 PQA";
$resn_exch{"H231 PQA"}="1H23 PQA"; $resn_exch{"H232 PQA"}="2H23 PQA"; $resn_exch{"H241 PQA"}="1H24 PQA";
$resn_exch{"H242 PQA"}="2H24 PQA"; $resn_exch{"H261 PQA"}="1H26 PQA"; $resn_exch{"H262 PQA"}="2H26 PQA";
$resn_exch{"H271 PQA"}="1H27 PQA"; $resn_exch{"H272 PQA"}="2H27 PQA"; $resn_exch{"HN11 PQB"}="1HN1 PQB";
$resn_exch{"HN12 PQB"}="2HN1 PQB"; $resn_exch{"H221 PQB"}="1H22 PQB"; $resn_exch{"H222 PQB"}="2H22 PQB";
$resn_exch{"H261 PQB"}="1H26 PQB"; $resn_exch{"H262 PQB"}="2H26 PQB"; $resn_exch{"H681 PQE"}="1H68 PQE";
$resn_exch{"H682 PQE"}="2H68 PQE"; $resn_exch{"H521 PQE"}="1H52 PQE"; $resn_exch{"H522 PQE"}="2H52 PQE";
$resn_exch{"H551 PQE"}="1H55 PQE"; $resn_exch{"H552 PQE"}="2H55 PQE"; $resn_exch{"H581 PQE"}="1H58 PQE";
$resn_exch{"H582 PQE"}="2H58 PQE"; $resn_exch{"H611 PQE"}="1H61 PQE"; $resn_exch{"H612 PQE"}="2H61 PQE";
$resn_exch{"H621 PQE"}="1H62 PQE"; $resn_exch{"H622 PQE"}="2H62 PQE"; $resn_exch{"H631 PQE"}="1H63 PQE";
$resn_exch{"H632 PQE"}="2H63 PQE"; $resn_exch{"H661 PQE"}="1H66 PQE"; $resn_exch{"H662 PQE"}="2H66 PQE";
$resn_exch{"H671 PQE"}="1H67 PQE"; $resn_exch{"H672 PQE"}="2H67 PQE"; $resn_exch{"H651 PQE"}="1H65 PQE";
$resn_exch{"H652 PQE"}="2H65 PQE"; $resn_exch{"H641 PQE"}="1H64 PQE"; $resn_exch{"H642 PQE"}="2H64 PQE";
$resn_exch{"H601 PQE"}="1H60 PQE"; $resn_exch{"H602 PQE"}="2H60 PQE"; $resn_exch{"H591 PQE"}="1H59 PQE";
$resn_exch{"H592 PQE"}="2H59 PQE"; $resn_exch{"H571 PQE"}="1H57 PQE"; $resn_exch{"H572 PQE"}="2H57 PQE";
$resn_exch{"H561 PQE"}="1H56 PQE"; $resn_exch{"H562 PQE"}="2H56 PQE"; $resn_exch{"H541 PQE"}="1H54 PQE";
$resn_exch{"H542 PQE"}="2H54 PQE"; $resn_exch{"H531 PQE"}="1H53 PQE"; $resn_exch{"H532 PQE"}="2H53 PQE";
$resn_exch{"H511 PQE"}="1H51 PQE"; $resn_exch{"H512 PQE"}="2H51 PQE"; $resn_exch{"H501 PQE"}="1H50 PQE";
$resn_exch{"H502 PQE"}="2H50 PQE"; $resn_exch{"H491 PQE"}="1H49 PQE"; $resn_exch{"H492 PQE"}="2H49 PQE";
$resn_exch{"H481 PQE"}="1H48 PQE"; $resn_exch{"H482 PQE"}="2H48 PQE"; $resn_exch{"H471 PQE"}="1H47 PQE";
$resn_exch{"H472 PQE"}="2H47 PQE"; $resn_exch{"H461 PQE"}="1H46 PQE"; $resn_exch{"H462 PQE"}="2H46 PQE";
$resn_exch{"H451 PQE"}="1H45 PQE"; $resn_exch{"H452 PQE"}="2H45 PQE"; $resn_exch{"H441 PQE"}="1H44 PQE";
$resn_exch{"H442 PQE"}="2H44 PQE"; $resn_exch{"H431 PQE"}="1H43 PQE"; $resn_exch{"H432 PQE"}="2H43 PQE";
$resn_exch{"H421 PQE"}="1H42 PQE"; $resn_exch{"H422 PQE"}="2H42 PQE"; $resn_exch{"H411 PQE"}="1H41 PQE";
$resn_exch{"H412 PQE"}="2H41 PQE"; $resn_exch{"H401 PQE"}="1H40 PQE"; $resn_exch{"H402 PQE"}="2H40 PQE";
$resn_exch{"H403 PQE"}="3H40 PQE"; $resn_exch{"H2M1 PQN"}="1H2M PQN"; $resn_exch{"H2M2 PQN"}="2H2M PQN";
$resn_exch{"H2M3 PQN"}="3H2M PQN"; $resn_exch{"H111 PQN"}="1H11 PQN"; $resn_exch{"H112 PQN"}="2H11 PQN";
$resn_exch{"H141 PQN"}="1H14 PQN"; $resn_exch{"H142 PQN"}="2H14 PQN"; $resn_exch{"H143 PQN"}="3H14 PQN";
$resn_exch{"H151 PQN"}="1H15 PQN"; $resn_exch{"H152 PQN"}="2H15 PQN"; $resn_exch{"H161 PQN"}="1H16 PQN";
$resn_exch{"H162 PQN"}="2H16 PQN"; $resn_exch{"H171 PQN"}="1H17 PQN"; $resn_exch{"H172 PQN"}="2H17 PQN";
$resn_exch{"H191 PQN"}="1H19 PQN"; $resn_exch{"H192 PQN"}="2H19 PQN"; $resn_exch{"H193 PQN"}="3H19 PQN";
$resn_exch{"H201 PQN"}="1H20 PQN"; $resn_exch{"H202 PQN"}="2H20 PQN"; $resn_exch{"H211 PQN"}="1H21 PQN";
$resn_exch{"H212 PQN"}="2H21 PQN"; $resn_exch{"H221 PQN"}="1H22 PQN"; $resn_exch{"H222 PQN"}="2H22 PQN";
$resn_exch{"H241 PQN"}="1H24 PQN"; $resn_exch{"H242 PQN"}="2H24 PQN"; $resn_exch{"H243 PQN"}="3H24 PQN";
$resn_exch{"H251 PQN"}="1H25 PQN"; $resn_exch{"H252 PQN"}="2H25 PQN"; $resn_exch{"H261 PQN"}="1H26 PQN";
$resn_exch{"H262 PQN"}="2H26 PQN"; $resn_exch{"H271 PQN"}="1H27 PQN"; $resn_exch{"H272 PQN"}="2H27 PQN";
$resn_exch{"H291 PQN"}="1H29 PQN"; $resn_exch{"H292 PQN"}="2H29 PQN"; $resn_exch{"H293 PQN"}="3H29 PQN";
$resn_exch{"H301 PQN"}="1H30 PQN"; $resn_exch{"H302 PQN"}="2H30 PQN"; $resn_exch{"H303 PQN"}="3H30 PQN";
$resn_exch{"HOB2 PQQ"}="2HOB PQQ"; $resn_exch{"HOB7 PQQ"}="7HOB PQQ"; $resn_exch{"HOB9 PQQ"}="9HOB PQQ";
$resn_exch{" H2  PR0"}=" HN2 PR0"; $resn_exch{" HB2 PR0"}="1HB  PR0"; $resn_exch{" HB3 PR0"}="2HB  PR0";
$resn_exch{" H11 PR1"}="1H1  PR1"; $resn_exch{" H21 PR1"}="1H2  PR1"; $resn_exch{" H51 PR1"}="1H5  PR1";
$resn_exch{"H111 PR1"}="1H11 PR1"; $resn_exch{"H121 PR1"}="1H12 PR1"; $resn_exch{"H122 PR1"}="2H12 PR1";
$resn_exch{"H131 PR1"}="1H13 PR1"; $resn_exch{"H132 PR1"}="2H13 PR1"; $resn_exch{"H441 PR1"}="1H44 PR1";
$resn_exch{"H171 PR1"}="1H17 PR1"; $resn_exch{"H181 PR1"}="1H18 PR1"; $resn_exch{"H191 PR1"}="1H19 PR1";
$resn_exch{"H251 PR1"}="1H25 PR1"; $resn_exch{"H301 PR1"}="1H30 PR1"; $resn_exch{"H311 PR1"}="1H31 PR1";
$resn_exch{"H312 PR1"}="2H31 PR1"; $resn_exch{"H321 PR1"}="1H32 PR1"; $resn_exch{"H322 PR1"}="2H32 PR1";
$resn_exch{"H371 PR1"}="1H37 PR1"; $resn_exch{"H171 PR2"}="1H17 PR2"; $resn_exch{"H181 PR2"}="1H18 PR2";
$resn_exch{"H191 PR2"}="1H19 PR2"; $resn_exch{"H251 PR2"}="1H25 PR2"; $resn_exch{"H301 PR2"}="1H30 PR2";
$resn_exch{"H311 PR2"}="1H31 PR2"; $resn_exch{"H312 PR2"}="2H31 PR2"; $resn_exch{"H321 PR2"}="1H32 PR2";
$resn_exch{"H322 PR2"}="2H32 PR2"; $resn_exch{"H371 PR2"}="1H37 PR2"; $resn_exch{" H11 PR2"}="1H1  PR2";
$resn_exch{" H12 PR2"}="2H1  PR2"; $resn_exch{" H31 PR2"}="1H3  PR2"; $resn_exch{" H41 PR2"}="1H4  PR2";
$resn_exch{" H61 PR2"}="1H6  PR2"; $resn_exch{"H271 PR2"}="1H27 PR2"; $resn_exch{" H   PR3"}="1HN  PR3";
$resn_exch{" H2  PR3"}="2HN  PR3"; $resn_exch{" HC  PR3"}=" H   PR3"; $resn_exch{" HB2 PR3"}="1HB  PR3";
$resn_exch{" HB3 PR3"}="2HB  PR3"; $resn_exch{" HE2 PR3"}="1HE  PR3"; $resn_exch{" HE3 PR3"}="2HE  PR3";
$resn_exch{" HZ2 PR3"}="1HZ  PR3"; $resn_exch{" HZ3 PR3"}="2HZ  PR3"; $resn_exch{" HH1 PR3"}="1HH  PR3";
$resn_exch{" HH2 PR3"}="2HH  PR3"; $resn_exch{" HH3 PR3"}="3HH  PR3"; $resn_exch{"HOP2 PR5"}="2HOP PR5";
$resn_exch{" H5' PR5"}="1H5* PR5"; $resn_exch{"H5'' PR5"}="2H5* PR5"; $resn_exch{" H4' PR5"}=" H4* PR5";
$resn_exch{" H1' PR5"}=" H1* PR5"; $resn_exch{" H3' PR5"}=" H3* PR5"; $resn_exch{"HO3' PR5"}="H3T  PR5";
$resn_exch{" H2' PR5"}=" H2* PR5"; $resn_exch{"HO2' PR5"}="2HO* PR5"; $resn_exch{"HOP3 PR5"}="3HOP PR5";
$resn_exch{" H11 PRA"}="1H1  PRA"; $resn_exch{" H12 PRA"}="2H1  PRA"; $resn_exch{" H21 PRA"}="1H2  PRA";
$resn_exch{" H22 PRA"}="2H2  PRA"; $resn_exch{" H31 PRA"}="1H3  PRA"; $resn_exch{" H32 PRA"}="2H3  PRA";
$resn_exch{" HN1 PRA"}="1HN  PRA"; $resn_exch{" HN2 PRA"}="2HN  PRA"; $resn_exch{" HN3 PRA"}="3HN  PRA";
$resn_exch{" H51 PRB"}="1H5  PRB"; $resn_exch{" H52 PRB"}="2H5  PRB"; $resn_exch{"HO12 PRB"}="2HO1 PRB";
$resn_exch{"H161 PRB"}="1H16 PRB"; $resn_exch{"H162 PRB"}="2H16 PRB"; $resn_exch{"H163 PRB"}="3H16 PRB";
$resn_exch{"H171 PRB"}="1H17 PRB"; $resn_exch{"H172 PRB"}="2H17 PRB"; $resn_exch{"H173 PRB"}="3H17 PRB";
$resn_exch{"H181 PRB"}="1H18 PRB"; $resn_exch{"H182 PRB"}="2H18 PRB"; $resn_exch{"H183 PRB"}="3H18 PRB";
$resn_exch{"H191 PRB"}="1H19 PRB"; $resn_exch{"H192 PRB"}="2H19 PRB"; $resn_exch{"H193 PRB"}="3H19 PRB";
$resn_exch{"H201 PRB"}="1H20 PRB"; $resn_exch{"H202 PRB"}="2H20 PRB"; $resn_exch{"HO20 PRB"}="0HO2 PRB";
$resn_exch{"HA21 PRB"}="1HA2 PRB"; $resn_exch{"HA22 PRB"}="2HA2 PRB"; $resn_exch{"HA23 PRB"}="3HA2 PRB";
$resn_exch{"H11C PRC"}="CH11 PRC"; $resn_exch{"H12C PRC"}="CH12 PRC"; $resn_exch{"H13N PRC"}="NH13 PRC";
$resn_exch{"H15C PRC"}="CH15 PRC"; $resn_exch{"H17C PRC"}="CH17 PRC"; $resn_exch{"H18C PRC"}="CH18 PRC";
$resn_exch{"H201 PRC"}="1H20 PRC"; $resn_exch{"H202 PRC"}="2H20 PRC"; $resn_exch{"H203 PRC"}="3H20 PRC";
$resn_exch{"H21N PRC"}="NH21 PRC"; $resn_exch{"H25C PRC"}="CH25 PRC"; $resn_exch{"H26C PRC"}="CH26 PRC";
$resn_exch{"H27C PRC"}="CH27 PRC"; $resn_exch{"H29C PRC"}="CH29 PRC"; $resn_exch{" H7' PRD"}=" H7* PRD";
$resn_exch{"HC61 PRD"}="1HC6 PRD"; $resn_exch{"HC62 PRD"}="2HC6 PRD"; $resn_exch{"HC63 PRD"}="3HC6 PRD";
$resn_exch{" H5' PRD"}=" H5* PRD"; $resn_exch{"HN41 PRD"}="1HN4 PRD"; $resn_exch{"HN42 PRD"}="2HN4 PRD";
$resn_exch{"HN21 PRD"}="1HN2 PRD"; $resn_exch{"HN22 PRD"}="2HN2 PRD"; $resn_exch{"HC71 PRD"}="1HC7 PRD";
$resn_exch{"HC72 PRD"}="2HC7 PRD"; $resn_exch{"HC21 PRD"}="1HC2 PRD"; $resn_exch{"HC22 PRD"}="2HC2 PRD";
$resn_exch{"HC23 PRD"}="3HC2 PRD"; $resn_exch{"HC51 PRD"}="1HC5 PRD"; $resn_exch{"HC52 PRD"}="2HC5 PRD";
$resn_exch{"HC53 PRD"}="3HC5 PRD"; $resn_exch{"HO72 PRE"}="2HO7 PRE"; $resn_exch{" H81 PRE"}="1H8  PRE";
$resn_exch{" H82 PRE"}="2H8  PRE"; $resn_exch{"HO2' PRE"}="'HO2 PRE"; $resn_exch{" H91 PRF"}="1H9  PRF";
$resn_exch{"H101 PRF"}="1H10 PRF"; $resn_exch{"H102 PRF"}="2H10 PRF"; $resn_exch{"H111 PRF"}="1H11 PRF";
$resn_exch{"H112 PRF"}="2H11 PRF"; $resn_exch{" H81 PRF"}="1H8  PRF"; $resn_exch{"HN91 PRF"}="1HN9 PRF";
$resn_exch{"HN21 PRF"}="1HN2 PRF"; $resn_exch{"HN22 PRF"}="2HN2 PRF"; $resn_exch{" H1' PRH"}=" H1* PRH";
$resn_exch{" H2' PRH"}=" H2* PRH"; $resn_exch{"HO2' PRH"}="*HO2 PRH"; $resn_exch{" H3' PRH"}=" H3* PRH";
$resn_exch{"HO3' PRH"}="*HO3 PRH"; $resn_exch{" H4' PRH"}=" H4* PRH"; $resn_exch{"H5'1 PRH"}="1H5* PRH";
$resn_exch{"H5'2 PRH"}="2H5* PRH"; $resn_exch{"HO5' PRH"}="*HO5 PRH"; $resn_exch{"HCB1 PRI"}="1HCB PRI";
$resn_exch{"HCB2 PRI"}="2HCB PRI"; $resn_exch{"HCG1 PRI"}="1HCG PRI"; $resn_exch{"HCG2 PRI"}="2HCG PRI";
$resn_exch{"HO'T PRI"}="THO' PRI"; $resn_exch{"HCD1 PRI"}="1HCD PRI"; $resn_exch{"HCD2 PRI"}="2HCD PRI";
$resn_exch{" HCA PRI"}=" AHC PRI"; $resn_exch{"HNF1 PRL"}="1HNF PRL"; $resn_exch{"HNF2 PRL"}="2HNF PRL";
$resn_exch{"HNG1 PRL"}="1HNG PRL"; $resn_exch{"HNG2 PRL"}="2HNG PRL"; $resn_exch{"H361 PRM"}="1H36 PRM";
$resn_exch{"H362 PRM"}="2H36 PRM"; $resn_exch{"H371 PRM"}="1H37 PRM"; $resn_exch{"H372 PRM"}="2H37 PRM";
$resn_exch{"H411 PRM"}="1H41 PRM"; $resn_exch{"H412 PRM"}="2H41 PRM"; $resn_exch{"H421 PRM"}="1H42 PRM";
$resn_exch{"H422 PRM"}="2H42 PRM"; $resn_exch{"H431 PRM"}="1H43 PRM"; $resn_exch{"H432 PRM"}="2H43 PRM";
$resn_exch{"H441 PRM"}="1H44 PRM"; $resn_exch{"H442 PRM"}="2H44 PRM"; $resn_exch{"H451 PRM"}="1H45 PRM";
$resn_exch{"H452 PRM"}="2H45 PRM"; $resn_exch{"H453 PRM"}="3H45 PRM"; $resn_exch{"H461 PRM"}="1H46 PRM";
$resn_exch{"H462 PRM"}="2H46 PRM"; $resn_exch{"H463 PRM"}="3H46 PRM"; $resn_exch{"H471 PRM"}="1H47 PRM";
$resn_exch{"H472 PRM"}="2H47 PRM"; $resn_exch{"H481 PRM"}="1H48 PRM"; $resn_exch{"H482 PRM"}="2H48 PRM";
$resn_exch{"H483 PRM"}="3H48 PRM"; $resn_exch{" H1' PRN"}=" H1* PRN"; $resn_exch{" H2' PRN"}="1H2* PRN";
$resn_exch{"H2'' PRN"}="2H2* PRN"; $resn_exch{" H3' PRN"}=" H3* PRN"; $resn_exch{"HO3' PRN"}="HO3* PRN";
$resn_exch{" H4' PRN"}=" H4* PRN"; $resn_exch{" H5' PRN"}="1H5* PRN"; $resn_exch{"H5'' PRN"}="2H5* PRN";
$resn_exch{"HOP2 PRN"}="2HOP PRN"; $resn_exch{"HOP3 PRN"}="3HOP PRN"; $resn_exch{" H   PRO"}=" HT1 PRO";
$resn_exch{" HB2 PRO"}="1HB  PRO"; $resn_exch{" HB3 PRO"}="2HB  PRO"; $resn_exch{" HG2 PRO"}="1HG  PRO";
$resn_exch{" HG3 PRO"}="2HG  PRO"; $resn_exch{" HD2 PRO"}="1HD  PRO"; $resn_exch{" HD3 PRO"}="2HD  PRO";
$resn_exch{" H51 PRP"}="1H5  PRP"; $resn_exch{" H52 PRP"}="2H5  PRP"; $resn_exch{"HOP2 PRP"}="2HOP PRP";
$resn_exch{"HOP3 PRP"}="3HOP PRP"; $resn_exch{"HOA2 PRP"}="2HOA PRP"; $resn_exch{"HOB2 PRP"}="2HOB PRP";
$resn_exch{"HOB3 PRP"}="3HOB PRP"; $resn_exch{"HAJ1 PRQ"}="1HAJ PRQ"; $resn_exch{"HAJ2 PRQ"}="2HAJ PRQ";
$resn_exch{"HAA1 PRQ"}="1HAA PRQ"; $resn_exch{"HAA2 PRQ"}="2HAA PRQ"; $resn_exch{" H2  PRR"}=" HN2 PRR";
$resn_exch{" HC2 PRR"}=" H2  PRR"; $resn_exch{" H51 PRR"}="1H5  PRR"; $resn_exch{" H52 PRR"}="2H5  PRR";
$resn_exch{"H101 PRR"}="1H10 PRR"; $resn_exch{"H102 PRR"}="2H10 PRR"; $resn_exch{"H103 PRR"}="3H10 PRR";
$resn_exch{" H   PRS"}=" HN  PRS"; $resn_exch{" HD3 PRS"}="1HD  PRS"; $resn_exch{" HD2 PRS"}="2HD  PRS";
$resn_exch{" HB2 PRS"}="1HB  PRS"; $resn_exch{" HB3 PRS"}="2HB  PRS"; $resn_exch{"H5'1 PRT"}="1H5* PRT";
$resn_exch{"H5'2 PRT"}="2H5* PRT"; $resn_exch{" H4' PRT"}=" H4* PRT"; $resn_exch{" H3' PRT"}=" H3* PRT";
$resn_exch{" H2' PRT"}=" H2* PRT"; $resn_exch{" H1' PRT"}=" H1* PRT"; $resn_exch{"H151 PRT"}="1H15 PRT";
$resn_exch{"H152 PRT"}="2H15 PRT"; $resn_exch{" H2' PRX"}=" H2* PRX"; $resn_exch{" H1' PRX"}=" H1* PRX";
$resn_exch{" H4' PRX"}=" H4* PRX"; $resn_exch{"H5'1 PRX"}="1H5* PRX"; $resn_exch{"H5'2 PRX"}="2H5* PRX";
$resn_exch{"H1P1 PRX"}="1H1P PRX"; $resn_exch{"H1P2 PRX"}="2H1P PRX"; $resn_exch{"H2P1 PRX"}="1H2P PRX";
$resn_exch{"H2P2 PRX"}="2H2P PRX"; $resn_exch{"H3P1 PRX"}="1H3P PRX"; $resn_exch{"H3P2 PRX"}="2H3P PRX";
$resn_exch{"H3P3 PRX"}="3H3P PRX"; $resn_exch{" H3' PRX"}=" H3* PRX"; $resn_exch{"HN61 PRX"}="1HN6 PRX";
$resn_exch{"HN62 PRX"}="2HN6 PRX"; $resn_exch{"HC41 PRY"}="1HC4 PRY"; $resn_exch{"HC42 PRY"}="2HC4 PRY";
$resn_exch{"HN61 PRY"}="1HN6 PRY"; $resn_exch{"HN62 PRY"}="2HN6 PRY"; $resn_exch{"HC91 PRY"}="1HC9 PRY";
$resn_exch{"HC92 PRY"}="2HC9 PRY"; $resn_exch{"H101 PRY"}="1H10 PRY"; $resn_exch{"H102 PRY"}="2H10 PRY";
$resn_exch{"H103 PRY"}="3H10 PRY"; $resn_exch{"H212 PRZ"}="2H21 PRZ"; $resn_exch{"H211 PRZ"}="1H21 PRZ";
$resn_exch{"H233 PRZ"}="3H23 PRZ"; $resn_exch{"H232 PRZ"}="2H23 PRZ"; $resn_exch{"H231 PRZ"}="1H23 PRZ";
$resn_exch{"H243 PRZ"}="3H24 PRZ"; $resn_exch{"H242 PRZ"}="2H24 PRZ"; $resn_exch{"H241 PRZ"}="1H24 PRZ";
$resn_exch{"H313 PRZ"}="3H31 PRZ"; $resn_exch{"H312 PRZ"}="2H31 PRZ"; $resn_exch{"H311 PRZ"}="1H31 PRZ";
$resn_exch{" HB1 PS0"}="1HB  PS0"; $resn_exch{" HB2 PS0"}="2HB  PS0"; $resn_exch{"H481 PS2"}="1H48 PS2";
$resn_exch{"H482 PS2"}="2H48 PS2"; $resn_exch{"H483 PS2"}="3H48 PS2"; $resn_exch{"H471 PS2"}="1H47 PS2";
$resn_exch{"H472 PS2"}="2H47 PS2"; $resn_exch{"H461 PS2"}="1H46 PS2"; $resn_exch{"H462 PS2"}="2H46 PS2";
$resn_exch{"H451 PS2"}="1H45 PS2"; $resn_exch{"H452 PS2"}="2H45 PS2"; $resn_exch{"H441 PS2"}="1H44 PS2";
$resn_exch{"H442 PS2"}="2H44 PS2"; $resn_exch{"H431 PS2"}="1H43 PS2"; $resn_exch{"H432 PS2"}="2H43 PS2";
$resn_exch{"H421 PS2"}="1H42 PS2"; $resn_exch{"H422 PS2"}="2H42 PS2"; $resn_exch{"H411 PS2"}="1H41 PS2";
$resn_exch{"H412 PS2"}="2H41 PS2"; $resn_exch{"H381 PS2"}="1H38 PS2"; $resn_exch{"H382 PS2"}="2H38 PS2";
$resn_exch{"H371 PS2"}="1H37 PS2"; $resn_exch{"H372 PS2"}="2H37 PS2"; $resn_exch{"H361 PS2"}="1H36 PS2";
$resn_exch{"H362 PS2"}="2H36 PS2"; $resn_exch{"H351 PS2"}="1H35 PS2"; $resn_exch{"H352 PS2"}="2H35 PS2";
$resn_exch{"H341 PS2"}="1H34 PS2"; $resn_exch{"H342 PS2"}="2H34 PS2"; $resn_exch{"H331 PS2"}="1H33 PS2";
$resn_exch{"H332 PS2"}="2H33 PS2"; $resn_exch{"H321 PS2"}="1H32 PS2"; $resn_exch{"H322 PS2"}="2H32 PS2";
$resn_exch{" H11 PS2"}="1H1  PS2"; $resn_exch{" H12 PS2"}="2H1  PS2"; $resn_exch{" H41 PS2"}="1H4  PS2";
$resn_exch{" H42 PS2"}="2H4  PS2"; $resn_exch{"HN51 PS2"}="1HN5 PS2"; $resn_exch{"HN52 PS2"}="2HN5 PS2";
$resn_exch{" H31 PS2"}="1H3  PS2"; $resn_exch{" H32 PS2"}="2H3  PS2"; $resn_exch{"H121 PS2"}="1H12 PS2";
$resn_exch{"H122 PS2"}="2H12 PS2"; $resn_exch{"H131 PS2"}="1H13 PS2"; $resn_exch{"H132 PS2"}="2H13 PS2";
$resn_exch{"H141 PS2"}="1H14 PS2"; $resn_exch{"H142 PS2"}="2H14 PS2"; $resn_exch{"H151 PS2"}="1H15 PS2";
$resn_exch{"H152 PS2"}="2H15 PS2"; $resn_exch{"H161 PS2"}="1H16 PS2"; $resn_exch{"H162 PS2"}="2H16 PS2";
$resn_exch{"H171 PS2"}="1H17 PS2"; $resn_exch{"H172 PS2"}="2H17 PS2"; $resn_exch{"H181 PS2"}="1H18 PS2";
$resn_exch{"H182 PS2"}="2H18 PS2"; $resn_exch{"H191 PS2"}="1H19 PS2"; $resn_exch{"H192 PS2"}="2H19 PS2";
$resn_exch{"H201 PS2"}="1H20 PS2"; $resn_exch{"H202 PS2"}="2H20 PS2"; $resn_exch{"H211 PS2"}="1H21 PS2";
$resn_exch{"H212 PS2"}="2H21 PS2"; $resn_exch{"H221 PS2"}="1H22 PS2"; $resn_exch{"H222 PS2"}="2H22 PS2";
$resn_exch{"H231 PS2"}="1H23 PS2"; $resn_exch{"H232 PS2"}="2H23 PS2"; $resn_exch{"H241 PS2"}="1H24 PS2";
$resn_exch{"H242 PS2"}="2H24 PS2"; $resn_exch{"H251 PS2"}="1H25 PS2"; $resn_exch{"H252 PS2"}="2H25 PS2";
$resn_exch{"H261 PS2"}="1H26 PS2"; $resn_exch{"H262 PS2"}="2H26 PS2"; $resn_exch{"H263 PS2"}="3H26 PS2";
$resn_exch{" H   PSA"}="1HN  PSA"; $resn_exch{" H2  PSA"}="2HN  PSA"; $resn_exch{" HB2 PSA"}="1HB  PSA";
$resn_exch{" HB3 PSA"}="2HB  PSA"; $resn_exch{" HM1 PSA"}="1HM  PSA"; $resn_exch{" HM2 PSA"}="2HM  PSA";
$resn_exch{"HN11 PSB"}="1HN1 PSB"; $resn_exch{"HN12 PSB"}="2HN1 PSB"; $resn_exch{"H211 PSB"}="1H21 PSB";
$resn_exch{"H212 PSB"}="2H21 PSB"; $resn_exch{"H351 PSB"}="1H35 PSB"; $resn_exch{"H352 PSB"}="2H35 PSB";
$resn_exch{"H011 PSC"}="1H01 PSC"; $resn_exch{"H012 PSC"}="2H01 PSC"; $resn_exch{"H031 PSC"}="1H03 PSC";
$resn_exch{"H032 PSC"}="2H03 PSC"; $resn_exch{"H041 PSC"}="1H04 PSC"; $resn_exch{"H042 PSC"}="2H04 PSC";
$resn_exch{"H051 PSC"}="1H05 PSC"; $resn_exch{"H052 PSC"}="2H05 PSC"; $resn_exch{"H061 PSC"}="1H06 PSC";
$resn_exch{"H062 PSC"}="2H06 PSC"; $resn_exch{"H063 PSC"}="3H06 PSC"; $resn_exch{"H071 PSC"}="1H07 PSC";
$resn_exch{"H072 PSC"}="2H07 PSC"; $resn_exch{"H073 PSC"}="3H07 PSC"; $resn_exch{"H081 PSC"}="1H08 PSC";
$resn_exch{"H082 PSC"}="2H08 PSC"; $resn_exch{"H083 PSC"}="3H08 PSC"; $resn_exch{" H21 PSC"}="1H2  PSC";
$resn_exch{" H22 PSC"}="2H2  PSC"; $resn_exch{" H31 PSC"}="1H3  PSC"; $resn_exch{" H32 PSC"}="2H3  PSC";
$resn_exch{" H41 PSC"}="1H4  PSC"; $resn_exch{" H42 PSC"}="2H4  PSC"; $resn_exch{" H51 PSC"}="1H5  PSC";
$resn_exch{" H52 PSC"}="2H5  PSC"; $resn_exch{" H61 PSC"}="1H6  PSC"; $resn_exch{" H62 PSC"}="2H6  PSC";
$resn_exch{" H71 PSC"}="1H7  PSC"; $resn_exch{" H72 PSC"}="2H7  PSC"; $resn_exch{" H81 PSC"}="1H8  PSC";
$resn_exch{" H82 PSC"}="2H8  PSC"; $resn_exch{"H111 PSC"}="1H11 PSC"; $resn_exch{"H112 PSC"}="2H11 PSC";
$resn_exch{"H141 PSC"}="1H14 PSC"; $resn_exch{"H142 PSC"}="2H14 PSC"; $resn_exch{"H151 PSC"}="1H15 PSC";
$resn_exch{"H152 PSC"}="2H15 PSC"; $resn_exch{"H161 PSC"}="1H16 PSC"; $resn_exch{"H162 PSC"}="2H16 PSC";
$resn_exch{"H171 PSC"}="1H17 PSC"; $resn_exch{"H172 PSC"}="2H17 PSC"; $resn_exch{"H181 PSC"}="1H18 PSC";
$resn_exch{"H182 PSC"}="2H18 PSC"; $resn_exch{"H183 PSC"}="3H18 PSC"; $resn_exch{"H201 PSC"}="1H20 PSC";
$resn_exch{"H202 PSC"}="2H20 PSC"; $resn_exch{"H211 PSC"}="1H21 PSC"; $resn_exch{"H212 PSC"}="2H21 PSC";
$resn_exch{"H221 PSC"}="1H22 PSC"; $resn_exch{"H222 PSC"}="2H22 PSC"; $resn_exch{"H231 PSC"}="1H23 PSC";
$resn_exch{"H232 PSC"}="2H23 PSC"; $resn_exch{"H241 PSC"}="1H24 PSC"; $resn_exch{"H242 PSC"}="2H24 PSC";
$resn_exch{"H251 PSC"}="1H25 PSC"; $resn_exch{"H252 PSC"}="2H25 PSC"; $resn_exch{"H261 PSC"}="1H26 PSC";
$resn_exch{"H262 PSC"}="2H26 PSC"; $resn_exch{"H271 PSC"}="1H27 PSC"; $resn_exch{"H272 PSC"}="2H27 PSC";
$resn_exch{"H281 PSC"}="1H28 PSC"; $resn_exch{"H282 PSC"}="2H28 PSC"; $resn_exch{"H291 PSC"}="1H29 PSC";
$resn_exch{"H292 PSC"}="2H29 PSC"; $resn_exch{"H301 PSC"}="1H30 PSC"; $resn_exch{"H302 PSC"}="2H30 PSC";
$resn_exch{"H311 PSC"}="1H31 PSC"; $resn_exch{"H312 PSC"}="2H31 PSC"; $resn_exch{"H321 PSC"}="1H32 PSC";
$resn_exch{"H322 PSC"}="2H32 PSC"; $resn_exch{"H331 PSC"}="1H33 PSC"; $resn_exch{"H332 PSC"}="2H33 PSC";
$resn_exch{"H341 PSC"}="1H34 PSC"; $resn_exch{"H342 PSC"}="2H34 PSC"; $resn_exch{"H343 PSC"}="3H34 PSC";
$resn_exch{"HCB1 PSD"}="1HCB PSD"; $resn_exch{"HCB2 PSD"}="2HCB PSD"; $resn_exch{"HCG1 PSD"}="1HCG PSD";
$resn_exch{"HCG2 PSD"}="2HCG PSD"; $resn_exch{"HCD1 PSD"}="1HCD PSD"; $resn_exch{"HCD2 PSD"}="2HCD PSD";
$resn_exch{" HCA PSD"}=" AHC PSD"; $resn_exch{"HN01 PSD"}="1HN0 PSD"; $resn_exch{"HOP2 PSD"}="2HOP PSD";
$resn_exch{"H5'1 PSD"}="1H5' PSD"; $resn_exch{"H5'2 PSD"}="2H5' PSD"; $resn_exch{"HC'4 PSD"}="4HC' PSD";
$resn_exch{"HC'1 PSD"}="1HC' PSD"; $resn_exch{" HC2 PSD"}="2HC  PSD"; $resn_exch{"HN61 PSD"}="1HN6 PSD";
$resn_exch{"HN62 PSD"}="2HN6 PSD"; $resn_exch{" HC8 PSD"}="8HC  PSD"; $resn_exch{"HC'2 PSD"}="2HC' PSD";
$resn_exch{"HO'2 PSD"}="2HO' PSD"; $resn_exch{"HC'3 PSD"}="3HC' PSD"; $resn_exch{"HO'3 PSD"}="3HO' PSD";
$resn_exch{"HN21 PSE"}="1HN2 PSE"; $resn_exch{"HN22 PSE"}="2HN2 PSE"; $resn_exch{" H11 PSE"}="1H1  PSE";
$resn_exch{" H12 PSE"}="2H1  PSE"; $resn_exch{" H31 PSE"}="1H3  PSE"; $resn_exch{" H32 PSE"}="2H3  PSE";
$resn_exch{" H21 PSF"}="1H2  PSF"; $resn_exch{" H22 PSF"}="2H2  PSF"; $resn_exch{" H31 PSF"}="1H3  PSF";
$resn_exch{" H41 PSF"}="1H4  PSF"; $resn_exch{" H42 PSF"}="2H4  PSF"; $resn_exch{" H61 PSF"}="1H6  PSF";
$resn_exch{" H62 PSF"}="2H6  PSF"; $resn_exch{" H71 PSF"}="1H7  PSF"; $resn_exch{" H72 PSF"}="2H7  PSF";
$resn_exch{"H131 PSF"}="1H13 PSF"; $resn_exch{"H132 PSF"}="2H13 PSF"; $resn_exch{"H141 PSF"}="1H14 PSF";
$resn_exch{"H142 PSF"}="2H14 PSF"; $resn_exch{"H151 PSF"}="1H15 PSF"; $resn_exch{"H152 PSF"}="2H15 PSF";
$resn_exch{" H1  PSF"}="1H   PSF"; $resn_exch{" H2  PSF"}="2H   PSF"; $resn_exch{" HB1 PSF"}="1HB  PSF";
$resn_exch{" HB2 PSF"}="2HB  PSF"; $resn_exch{" H81 PSF"}="1H8  PSF"; $resn_exch{" H82 PSF"}="2H8  PSF";
$resn_exch{" H91 PSF"}="1H9  PSF"; $resn_exch{" H92 PSF"}="2H9  PSF"; $resn_exch{"H101 PSF"}="1H10 PSF";
$resn_exch{"H102 PSF"}="2H10 PSF"; $resn_exch{"H103 PSF"}="3H10 PSF"; $resn_exch{"H161 PSF"}="1H16 PSF";
$resn_exch{"H162 PSF"}="2H16 PSF"; $resn_exch{"H171 PSF"}="1H17 PSF"; $resn_exch{"H172 PSF"}="2H17 PSF";
$resn_exch{"H173 PSF"}="3H17 PSF"; $resn_exch{" H62 PSG"}="2H6  PSG"; $resn_exch{" H61 PSG"}="1H6  PSG";
$resn_exch{"H2O1 PSG"}="1H2O PSG"; $resn_exch{"H3O1 PSG"}="1H3O PSG"; $resn_exch{"H4O1 PSG"}="1H4O PSG";
$resn_exch{"H6O1 PSG"}="1H6O PSG"; $resn_exch{" HN1 PSH"}="1HN  PSH"; $resn_exch{" HN2 PSH"}="2HN  PSH";
$resn_exch{" HB1 PSH"}="1HB  PSH"; $resn_exch{" HB2 PSH"}="2HB  PSH"; $resn_exch{"HB11 PSI"}="1HB1 PSI";
$resn_exch{"HB12 PSI"}="2HB1 PSI"; $resn_exch{"HB13 PSI"}="3HB1 PSI"; $resn_exch{"HN41 PSI"}="1HN4 PSI";
$resn_exch{"HN42 PSI"}="2HN4 PSI"; $resn_exch{"HB31 PSI"}="1HB3 PSI"; $resn_exch{"HB32 PSI"}="2HB3 PSI";
$resn_exch{"HB33 PSI"}="3HB3 PSI"; $resn_exch{"HB21 PSI"}="1HB2 PSI"; $resn_exch{"HB22 PSI"}="2HB2 PSI";
$resn_exch{"H441 PSI"}="1H44 PSI"; $resn_exch{"H442 PSI"}="2H44 PSI"; $resn_exch{"HA41 PSI"}="1HA4 PSI";
$resn_exch{"HA42 PSI"}="2HA4 PSI"; $resn_exch{"HG61 PSI"}="1HG6 PSI"; $resn_exch{"HG62 PSI"}="2HG6 PSI";
$resn_exch{"HG63 PSI"}="3HG6 PSI"; $resn_exch{"HG51 PSI"}="1HG5 PSI"; $resn_exch{"HG52 PSI"}="2HG5 PSI";
$resn_exch{"HG53 PSI"}="3HG5 PSI"; $resn_exch{"HG81 PSI"}="1HG8 PSI"; $resn_exch{"HG82 PSI"}="2HG8 PSI";
$resn_exch{"HG83 PSI"}="3HG8 PSI"; $resn_exch{"HG71 PSI"}="1HG7 PSI"; $resn_exch{"HG72 PSI"}="2HG7 PSI";
$resn_exch{"HG73 PSI"}="3HG7 PSI"; $resn_exch{"H761 PSI"}="1H76 PSI"; $resn_exch{"H762 PSI"}="2H76 PSI";
$resn_exch{"H763 PSI"}="3H76 PSI"; $resn_exch{"HAA1 PSK"}="1HAA PSK"; $resn_exch{"HAA2 PSK"}="2HAA PSK";
$resn_exch{"HAA3 PSK"}="3HAA PSK"; $resn_exch{"HAJ1 PSK"}="1HAJ PSK"; $resn_exch{"HAJ2 PSK"}="2HAJ PSK";
$resn_exch{"HAB1 PSK"}="1HAB PSK"; $resn_exch{"HAB2 PSK"}="2HAB PSK"; $resn_exch{"HAB3 PSK"}="3HAB PSK";
$resn_exch{"HAL1 PSK"}="1HAL PSK"; $resn_exch{"HAL2 PSK"}="2HAL PSK"; $resn_exch{"H191 PSM"}="1H19 PSM";
$resn_exch{"H192 PSM"}="2H19 PSM"; $resn_exch{"H201 PSM"}="1H20 PSM"; $resn_exch{"H202 PSM"}="2H20 PSM";
$resn_exch{"H321 PSM"}="1H32 PSM"; $resn_exch{"H322 PSM"}="2H32 PSM"; $resn_exch{"H31A PSM"}="1H3  PSM";
$resn_exch{" H32 PSM"}="2H3  PSM"; $resn_exch{" H33 PSM"}="3H3  PSM"; $resn_exch{" H91 PSM"}="1H9  PSM";
$resn_exch{" H92 PSM"}="2H9  PSM"; $resn_exch{" H81 PSM"}="1H8  PSM"; $resn_exch{" H82 PSM"}="2H8  PSM";
$resn_exch{"HN21 PSM"}="1HN2 PSM"; $resn_exch{"HN22 PSM"}="2HN2 PSM"; $resn_exch{" H61 PSM"}="1H6  PSM";
$resn_exch{" H62 PSM"}="2H6  PSM"; $resn_exch{" H63 PSM"}="3H6  PSM"; $resn_exch{" H71 PSM"}="1H7  PSM";
$resn_exch{" H72 PSM"}="2H7  PSM"; $resn_exch{" H73 PSM"}="3H7  PSM"; $resn_exch{"H121 PSN"}="1H12 PSN";
$resn_exch{"H122 PSN"}="2H12 PSN"; $resn_exch{"H121 PSO"}="1H12 PSO"; $resn_exch{"H122 PSO"}="2H12 PSO";
$resn_exch{"H123 PSO"}="3H12 PSO"; $resn_exch{"H131 PSO"}="1H13 PSO"; $resn_exch{"H132 PSO"}="2H13 PSO";
$resn_exch{"H151 PSO"}="1H15 PSO"; $resn_exch{"H152 PSO"}="2H15 PSO"; $resn_exch{"H153 PSO"}="3H15 PSO";
$resn_exch{"H161 PSO"}="1H16 PSO"; $resn_exch{"H162 PSO"}="2H16 PSO"; $resn_exch{"H163 PSO"}="3H16 PSO";
$resn_exch{"H10A PSP"}="AH10 PSP"; $resn_exch{" HO1 PSQ"}="1HO  PSQ"; $resn_exch{" HN1 PSQ"}="1HN  PSQ";
$resn_exch{" HN2 PSQ"}="2HN  PSQ"; $resn_exch{"HN11 PSQ"}="1HN1 PSQ"; $resn_exch{" H11 PSQ"}="1H1  PSQ";
$resn_exch{" H21 PSQ"}="1H2  PSQ"; $resn_exch{" H22 PSQ"}="2H2  PSQ"; $resn_exch{" H31 PSQ"}="1H3  PSQ";
$resn_exch{" H32 PSQ"}="2H3  PSQ"; $resn_exch{" H41 PSQ"}="1H4  PSQ"; $resn_exch{" H42 PSQ"}="2H4  PSQ";
$resn_exch{"HN21 PSQ"}="1HN2 PSQ"; $resn_exch{"HN31 PSQ"}="1HN3 PSQ"; $resn_exch{"HN32 PSQ"}="2HN3 PSQ";
$resn_exch{"H231 PSR"}="1H23 PSR"; $resn_exch{"H261 PSR"}="1H26 PSR"; $resn_exch{"H281 PSR"}="1H28 PSR";
$resn_exch{"H282 PSR"}="2H28 PSR"; $resn_exch{"H301 PSR"}="1H30 PSR"; $resn_exch{"H302 PSR"}="2H30 PSR";
$resn_exch{"H303 PSR"}="3H30 PSR"; $resn_exch{"H311 PSR"}="1H31 PSR"; $resn_exch{"H312 PSR"}="2H31 PSR";
$resn_exch{"H313 PSR"}="3H31 PSR"; $resn_exch{"H321 PSR"}="1H32 PSR"; $resn_exch{"H331 PSR"}="1H33 PSR";
$resn_exch{"H361 PSR"}="1H36 PSR"; $resn_exch{"H371 PSR"}="1H37 PSR"; $resn_exch{"H372 PSR"}="2H37 PSR";
$resn_exch{"H381 PSR"}="1H38 PSR"; $resn_exch{"H382 PSR"}="2H38 PSR"; $resn_exch{"H411 PSR"}="1H41 PSR";
$resn_exch{"H421 PSR"}="1H42 PSR"; $resn_exch{"H422 PSR"}="2H42 PSR"; $resn_exch{"H431 PSR"}="1H43 PSR";
$resn_exch{"H432 PSR"}="2H43 PSR"; $resn_exch{"HC21 PSR"}="1HC2 PSR"; $resn_exch{"HC22 PSR"}="2HC2 PSR";
$resn_exch{"HC31 PSR"}="1HC3 PSR"; $resn_exch{"HC32 PSR"}="2HC3 PSR"; $resn_exch{"HC41 PSR"}="1HC4 PSR";
$resn_exch{"HC42 PSR"}="2HC4 PSR"; $resn_exch{"HC43 PSR"}="3HC4 PSR"; $resn_exch{" H11 PSS"}="1H1  PSS";
$resn_exch{" H12 PSS"}="2H1  PSS"; $resn_exch{" H21 PSS"}="1H2  PSS"; $resn_exch{" H22 PSS"}="2H2  PSS";
$resn_exch{" H23 PSS"}="3H2  PSS"; $resn_exch{" H'1 PSS"}="1H'  PSS"; $resn_exch{" H'2 PSS"}="2H'  PSS";
$resn_exch{"HOP1 PST"}="1HOP PST"; $resn_exch{"HOP2 PST"}="2HOP PST"; $resn_exch{" H5' PST"}="1H5* PST";
$resn_exch{"H5'' PST"}="2H5* PST"; $resn_exch{" H4' PST"}=" H4* PST"; $resn_exch{" H3' PST"}=" H3* PST";
$resn_exch{" H2' PST"}="1H2* PST"; $resn_exch{"H2'' PST"}="2H2* PST"; $resn_exch{" H1' PST"}=" H1* PST";
$resn_exch{" H71 PST"}="1H5M PST"; $resn_exch{" H72 PST"}="2H5M PST"; $resn_exch{" H73 PST"}="3H5M PST";
$resn_exch{" H1' PSU"}=" H1* PSU"; $resn_exch{" H2' PSU"}=" H2* PSU"; $resn_exch{"HO2' PSU"}="*HO2 PSU";
$resn_exch{" H3' PSU"}=" H3* PSU"; $resn_exch{" H4' PSU"}=" H4* PSU"; $resn_exch{"HO3' PSU"}="*HO3 PSU";
$resn_exch{" H5' PSU"}="1H5* PSU"; $resn_exch{"H5'' PSU"}="2H5* PSU"; $resn_exch{"HOP2 PSU"}="2HOP PSU";
$resn_exch{"HOP3 PSU"}="3HOP PSU"; $resn_exch{"HAA1 PSX"}="1HAA PSX"; $resn_exch{"HAA2 PSX"}="2HAA PSX";
$resn_exch{"HAA3 PSX"}="3HAA PSX"; $resn_exch{" H61 PSX"}="1H6  PSX"; $resn_exch{" H62 PSX"}="2H6  PSX";
$resn_exch{"HAB1 PSX"}="1HAB PSX"; $resn_exch{"HAB2 PSX"}="2HAB PSX"; $resn_exch{"HAB3 PSX"}="3HAB PSX";
$resn_exch{" H21 PSX"}="1H2  PSX"; $resn_exch{" H22 PSX"}="2H2  PSX"; $resn_exch{"H4A1 PSZ"}="1H4A PSZ";
$resn_exch{"H4A2 PSZ"}="2H4A PSZ"; $resn_exch{"H2A1 PSZ"}="1H2A PSZ"; $resn_exch{"H2A2 PSZ"}="2H2A PSZ";
$resn_exch{"H2A3 PSZ"}="3H2A PSZ"; $resn_exch{"H5A1 PSZ"}="1H5A PSZ"; $resn_exch{"H5A2 PSZ"}="2H5A PSZ";
$resn_exch{"H111 PT1"}="1H11 PT1"; $resn_exch{"H112 PT1"}="2H11 PT1"; $resn_exch{"H131 PT1"}="1H13 PT1";
$resn_exch{"H132 PT1"}="2H13 PT1"; $resn_exch{"H161 PT2"}="1H16 PT2"; $resn_exch{"H162 PT2"}="2H16 PT2";
$resn_exch{"H171 PT2"}="1H17 PT2"; $resn_exch{"H172 PT2"}="2H17 PT2"; $resn_exch{" H21 PT3"}="1H2  PT3";
$resn_exch{" H31 PT3"}="1H3  PT3"; $resn_exch{"H41N PT3"}="1NH4 PT3"; $resn_exch{"H411 PT3"}="1H41 PT3";
$resn_exch{"H412 PT3"}="2H41 PT3"; $resn_exch{"H421 PT3"}="1H42 PT3"; $resn_exch{"H422 PT3"}="2H42 PT3";
$resn_exch{"H431 PT3"}="1H43 PT3"; $resn_exch{"H432 PT3"}="2H43 PT3"; $resn_exch{"H433 PT3"}="3H43 PT3";
$resn_exch{" H   PTA"}="1HN  PTA"; $resn_exch{" H2  PTA"}="2HN  PTA"; $resn_exch{" HB2 PTA"}="1HB  PTA";
$resn_exch{" HB3 PTA"}="2HB  PTA"; $resn_exch{"HD11 PTA"}="1HD1 PTA"; $resn_exch{"HD12 PTA"}="2HD1 PTA";
$resn_exch{"HD13 PTA"}="3HD1 PTA"; $resn_exch{"HD21 PTA"}="1HD2 PTA"; $resn_exch{"HD22 PTA"}="2HD2 PTA";
$resn_exch{"HD23 PTA"}="3HD2 PTA"; $resn_exch{"HOP2 PTA"}="2HOP PTA"; $resn_exch{" HM1 PTA"}="1HM  PTA";
$resn_exch{" HM2 PTA"}="2HM  PTA"; $resn_exch{" HB1 PTB"}="1HB  PTB"; $resn_exch{" HB2 PTB"}="2HB  PTB";
$resn_exch{" HG1 PTB"}="1HG  PTB"; $resn_exch{" HG2 PTB"}="2HG  PTB"; $resn_exch{" HD1 PTB"}="1HD  PTB";
$resn_exch{" HD2 PTB"}="2HD  PTB"; $resn_exch{" H71 PTB"}="1H7  PTB"; $resn_exch{" H72 PTB"}="2H7  PTB";
$resn_exch{" H73 PTB"}="3H7  PTB"; $resn_exch{" H81 PTB"}="1H8  PTB"; $resn_exch{" H82 PTB"}="2H8  PTB";
$resn_exch{" H83 PTB"}="3H8  PTB"; $resn_exch{" H91 PTB"}="1H9  PTB"; $resn_exch{" H92 PTB"}="2H9  PTB";
$resn_exch{" H93 PTB"}="3H9  PTB"; $resn_exch{" H21 PTC"}="1H2  PTC"; $resn_exch{" H22 PTC"}="2H2  PTC";
$resn_exch{" H23 PTC"}="3H2  PTC"; $resn_exch{"HC21 PTD"}="1HC2 PTD"; $resn_exch{"HC22 PTD"}="2HC2 PTD";
$resn_exch{"HC31 PTD"}="1HC3 PTD"; $resn_exch{"HC32 PTD"}="2HC3 PTD"; $resn_exch{"HC41 PTD"}="1HC4 PTD";
$resn_exch{"HC42 PTD"}="2HC4 PTD"; $resn_exch{" H11 PTE"}="1H1  PTE"; $resn_exch{" H12 PTE"}="2H1  PTE";
$resn_exch{"H101 PTE"}="1H10 PTE"; $resn_exch{"H102 PTE"}="2H10 PTE"; $resn_exch{"H211 PTE"}="1H21 PTE";
$resn_exch{"H212 PTE"}="2H21 PTE"; $resn_exch{"H301 PTE"}="1H30 PTE"; $resn_exch{"H302 PTE"}="2H30 PTE";
$resn_exch{"HOP2 PTE"}="2HOP PTE"; $resn_exch{"HOP6 PTE"}="6HOP PTE"; $resn_exch{"HO11 PTE"}="1HO1 PTE";
$resn_exch{"HO12 PTE"}="2HO1 PTE"; $resn_exch{"HO21 PTE"}="1HO2 PTE"; $resn_exch{"HO22 PTE"}="2HO2 PTE";
$resn_exch{" H11 PTF"}="1H1  PTF"; $resn_exch{" H12 PTF"}="2H1  PTF"; $resn_exch{" H13 PTF"}="3H1  PTF";
$resn_exch{" H31 PTF"}="1H3  PTF"; $resn_exch{" H32 PTF"}="2H3  PTF"; $resn_exch{" H   PTH"}="1HN  PTH";
$resn_exch{" H2  PTH"}="2HN  PTH"; $resn_exch{" HB2 PTH"}="1HB  PTH"; $resn_exch{" HB3 PTH"}="2HB  PTH";
$resn_exch{" HF1 PTH"}="1HF  PTH"; $resn_exch{" HF2 PTH"}="2HF  PTH"; $resn_exch{"HO2P PTH"}="PHO2 PTH";
$resn_exch{"HO3P PTH"}="PHO3 PTH"; $resn_exch{"H3C1 PTI"}="1H3C PTI"; $resn_exch{"H3C2 PTI"}="2H3C PTI";
$resn_exch{"H6C1 PTI"}="1H6C PTI"; $resn_exch{"H6C2 PTI"}="2H6C PTI"; $resn_exch{"H9C1 PTI"}="1H9C PTI";
$resn_exch{"H9C2 PTI"}="2H9C PTI"; $resn_exch{"H121 PTI"}="1H12 PTI"; $resn_exch{"H122 PTI"}="2H12 PTI";
$resn_exch{"H151 PTI"}="1H15 PTI"; $resn_exch{"H152 PTI"}="2H15 PTI"; $resn_exch{"H181 PTI"}="1H18 PTI";
$resn_exch{"H182 PTI"}="2H18 PTI"; $resn_exch{"H211 PTI"}="1H21 PTI"; $resn_exch{"H212 PTI"}="2H21 PTI";
$resn_exch{"H381 PTI"}="1H38 PTI"; $resn_exch{"H382 PTI"}="2H38 PTI"; $resn_exch{"H411 PTI"}="1H41 PTI";
$resn_exch{"H412 PTI"}="2H41 PTI"; $resn_exch{" H21 PTL"}="1H2  PTL"; $resn_exch{" H22 PTL"}="2H2  PTL";
$resn_exch{" H31 PTL"}="1H3  PTL"; $resn_exch{" H32 PTL"}="2H3  PTL"; $resn_exch{" H41 PTL"}="1H4  PTL";
$resn_exch{" H42 PTL"}="2H4  PTL"; $resn_exch{" H51 PTL"}="1H5  PTL"; $resn_exch{" H52 PTL"}="2H5  PTL";
$resn_exch{" H53 PTL"}="3H5  PTL"; $resn_exch{" H2  PTM"}=" HN2 PTM"; $resn_exch{"HC31 PTM"}="1HC3 PTM";
$resn_exch{"HC32 PTM"}="2HC3 PTM"; $resn_exch{"HC33 PTM"}="3HC3 PTM"; $resn_exch{" HB2 PTM"}="1HB  PTM";
$resn_exch{" HB3 PTM"}="2HB  PTM"; $resn_exch{" H11 PTN"}="1H1  PTN"; $resn_exch{" H12 PTN"}="2H1  PTN";
$resn_exch{" H13 PTN"}="3H1  PTN"; $resn_exch{" H21 PTN"}="1H2  PTN"; $resn_exch{" H22 PTN"}="2H2  PTN";
$resn_exch{" H23 PTN"}="3H2  PTN"; $resn_exch{" H31 PTN"}="1H3  PTN"; $resn_exch{" H32 PTN"}="2H3  PTN";
$resn_exch{" H33 PTN"}="3H3  PTN"; $resn_exch{" H21 PTO"}="1H2  PTO"; $resn_exch{" H22 PTO"}="2H2  PTO";
$resn_exch{" H41 PTO"}="1H4  PTO"; $resn_exch{" H42 PTO"}="2H4  PTO"; $resn_exch{" H61 PTO"}="1H6  PTO";
$resn_exch{" H62 PTO"}="2H6  PTO"; $resn_exch{" H71 PTO"}="1H7  PTO"; $resn_exch{" H72 PTO"}="2H7  PTO";
$resn_exch{" H91 PTO"}="1H9  PTO"; $resn_exch{" H92 PTO"}="2H9  PTO"; $resn_exch{" H93 PTO"}="3H9  PTO";
$resn_exch{"H5M1 PTP"}="1H5M PTP"; $resn_exch{"H5M2 PTP"}="2H5M PTP"; $resn_exch{"H5M3 PTP"}="3H5M PTP";
$resn_exch{" H1' PTP"}=" H1* PTP"; $resn_exch{"H2'1 PTP"}="1H2* PTP"; $resn_exch{"H2'2 PTP"}="2H2* PTP";
$resn_exch{" H3' PTP"}=" H3* PTP"; $resn_exch{" H4' PTP"}=" H4* PTP"; $resn_exch{"H5'1 PTP"}="1H5* PTP";
$resn_exch{"H5'2 PTP"}="2H5* PTP"; $resn_exch{" H   PTR"}="1HN  PTR"; $resn_exch{" HN2 PTR"}="2HN  PTR";
$resn_exch{" HB2 PTR"}="1HB  PTR"; $resn_exch{" HB3 PTR"}="2HB  PTR"; $resn_exch{"HO2P PTR"}="PHO2 PTR";
$resn_exch{"HO3P PTR"}="PHO3 PTR"; $resn_exch{"HN31 PTS"}="1HN3 PTS"; $resn_exch{"HN32 PTS"}="2HN3 PTS";
$resn_exch{"HN41 PTS"}="1HN4 PTS"; $resn_exch{"HN42 PTS"}="2HN4 PTS"; $resn_exch{"H151 PTS"}="1H15 PTS";
$resn_exch{"H152 PTS"}="2H15 PTS"; $resn_exch{"H153 PTS"}="3H15 PTS"; $resn_exch{"H101 PTT"}="1H10 PTT";
$resn_exch{"H102 PTT"}="2H10 PTT"; $resn_exch{"H301 PTT"}="1H30 PTT"; $resn_exch{"H302 PTT"}="2H30 PTT";
$resn_exch{"HA11 PTT"}="1HA1 PTT"; $resn_exch{"HA12 PTT"}="2HA1 PTT"; $resn_exch{"HA13 PTT"}="3HA1 PTT";
$resn_exch{"HOP3 PTT"}="3HOP PTT"; $resn_exch{"HOP7 PTT"}="7HOP PTT"; $resn_exch{"HN11 PTU"}="1HN1 PTU";
$resn_exch{"HN12 PTU"}="2HN1 PTU"; $resn_exch{" H21 PTU"}="1H2  PTU"; $resn_exch{" H31 PTU"}="1H3  PTU";
$resn_exch{" H41 PTU"}="1H4  PTU"; $resn_exch{" H51 PTU"}="1H5  PTU"; $resn_exch{" H61 PTU"}="1H6  PTU";
$resn_exch{"H1'1 PTU"}="1H1' PTU"; $resn_exch{"H1'2 PTU"}="2H1' PTU"; $resn_exch{"H2'1 PTU"}="1H2' PTU";
$resn_exch{"H2'2 PTU"}="2H2' PTU"; $resn_exch{"H2'3 PTU"}="3H2' PTU"; $resn_exch{" H71 PTX"}="1H7  PTX";
$resn_exch{" H72 PTX"}="2H7  PTX"; $resn_exch{"H181 PTX"}="1H18 PTX"; $resn_exch{"H182 PTX"}="2H18 PTX";
$resn_exch{"H183 PTX"}="3H18 PTX"; $resn_exch{" H11 PTX"}="1H1  PTX"; $resn_exch{" H12 PTX"}="2H1  PTX";
$resn_exch{"HC11 PTY"}="1HC1 PTY"; $resn_exch{"HC12 PTY"}="2HC1 PTY"; $resn_exch{"HC21 PTY"}="1HC2 PTY";
$resn_exch{"HC22 PTY"}="2HC2 PTY"; $resn_exch{"HC31 PTY"}="1HC3 PTY"; $resn_exch{"HC32 PTY"}="2HC3 PTY";
$resn_exch{"HC51 PTY"}="1HC5 PTY"; $resn_exch{"HC52 PTY"}="2HC5 PTY"; $resn_exch{"H111 PTY"}="1H11 PTY";
$resn_exch{"H112 PTY"}="2H11 PTY"; $resn_exch{"H121 PTY"}="1H12 PTY"; $resn_exch{"H122 PTY"}="2H12 PTY";
$resn_exch{"H131 PTY"}="1H13 PTY"; $resn_exch{"H132 PTY"}="2H13 PTY"; $resn_exch{"H141 PTY"}="1H14 PTY";
$resn_exch{"H142 PTY"}="2H14 PTY"; $resn_exch{"H151 PTY"}="1H15 PTY"; $resn_exch{"H152 PTY"}="2H15 PTY";
$resn_exch{"H161 PTY"}="1H16 PTY"; $resn_exch{"H162 PTY"}="2H16 PTY"; $resn_exch{"H171 PTY"}="1H17 PTY";
$resn_exch{"H172 PTY"}="2H17 PTY"; $resn_exch{"H181 PTY"}="1H18 PTY"; $resn_exch{"H182 PTY"}="2H18 PTY";
$resn_exch{"H191 PTY"}="1H19 PTY"; $resn_exch{"H192 PTY"}="2H19 PTY"; $resn_exch{"H201 PTY"}="1H20 PTY";
$resn_exch{"H202 PTY"}="2H20 PTY"; $resn_exch{"H211 PTY"}="1H21 PTY"; $resn_exch{"H212 PTY"}="2H21 PTY";
$resn_exch{"H221 PTY"}="1H22 PTY"; $resn_exch{"H222 PTY"}="2H22 PTY"; $resn_exch{"H231 PTY"}="1H23 PTY";
$resn_exch{"H232 PTY"}="2H23 PTY"; $resn_exch{"H241 PTY"}="1H24 PTY"; $resn_exch{"H242 PTY"}="2H24 PTY";
$resn_exch{"H251 PTY"}="1H25 PTY"; $resn_exch{"H252 PTY"}="2H25 PTY"; $resn_exch{"H261 PTY"}="1H26 PTY";
$resn_exch{"H262 PTY"}="2H26 PTY"; $resn_exch{"H271 PTY"}="1H27 PTY"; $resn_exch{"H272 PTY"}="2H27 PTY";
$resn_exch{"H281 PTY"}="1H28 PTY"; $resn_exch{"H282 PTY"}="2H28 PTY"; $resn_exch{"H291 PTY"}="1H29 PTY";
$resn_exch{"H292 PTY"}="2H29 PTY"; $resn_exch{"H293 PTY"}="3H29 PTY"; $resn_exch{"H311 PTY"}="1H31 PTY";
$resn_exch{"H312 PTY"}="2H31 PTY"; $resn_exch{"H321 PTY"}="1H32 PTY"; $resn_exch{"H322 PTY"}="2H32 PTY";
$resn_exch{"H331 PTY"}="1H33 PTY"; $resn_exch{"H332 PTY"}="2H33 PTY"; $resn_exch{"H341 PTY"}="1H34 PTY";
$resn_exch{"H342 PTY"}="2H34 PTY"; $resn_exch{"H351 PTY"}="1H35 PTY"; $resn_exch{"H352 PTY"}="2H35 PTY";
$resn_exch{"H361 PTY"}="1H36 PTY"; $resn_exch{"H362 PTY"}="2H36 PTY"; $resn_exch{"H371 PTY"}="1H37 PTY";
$resn_exch{"H372 PTY"}="2H37 PTY"; $resn_exch{"H381 PTY"}="1H38 PTY"; $resn_exch{"H382 PTY"}="2H38 PTY";
$resn_exch{"H391 PTY"}="1H39 PTY"; $resn_exch{"H392 PTY"}="2H39 PTY"; $resn_exch{"H401 PTY"}="1H40 PTY";
$resn_exch{"H402 PTY"}="2H40 PTY"; $resn_exch{"H411 PTY"}="1H41 PTY"; $resn_exch{"H412 PTY"}="2H41 PTY";
$resn_exch{"H421 PTY"}="1H42 PTY"; $resn_exch{"H422 PTY"}="2H42 PTY"; $resn_exch{"H431 PTY"}="1H43 PTY";
$resn_exch{"H432 PTY"}="2H43 PTY"; $resn_exch{"H441 PTY"}="1H44 PTY"; $resn_exch{"H442 PTY"}="2H44 PTY";
$resn_exch{"H443 PTY"}="3H44 PTY"; $resn_exch{"H12O PTY"}="OH12 PTY"; $resn_exch{"HN11 PTY"}="1HN1 PTY";
$resn_exch{"HN12 PTY"}="2HN1 PTY"; $resn_exch{"HOP2  PU"}="2HOP  PU"; $resn_exch{"HOP3  PU"}="3HOP  PU";
$resn_exch{" HN1  PU"}="1HN   PU"; $resn_exch{" HB1  PU"}="1HB   PU"; $resn_exch{" HB2  PU"}="2HB   PU";
$resn_exch{" HM1  PU"}="1HM   PU"; $resn_exch{" HM2  PU"}="2HM   PU"; $resn_exch{" HM3  PU"}="3HM   PU";
$resn_exch{"HO5'  PU"}="H5T   PU"; $resn_exch{" H5'  PU"}="1H5*  PU"; $resn_exch{"H5''  PU"}="2H5*  PU";
$resn_exch{" H4'  PU"}=" H4*  PU"; $resn_exch{" H3'  PU"}=" H3*  PU"; $resn_exch{"HN'3  PU"}="3HN*  PU";
$resn_exch{" H2'  PU"}=" H2*  PU"; $resn_exch{"HO2'  PU"}="*HO2  PU"; $resn_exch{" H1'  PU"}=" H1*  PU";
$resn_exch{" H91  PU"}="1H9   PU"; $resn_exch{" H92  PU"}="2H9   PU"; $resn_exch{" H93  PU"}="3H9   PU";
$resn_exch{"H101  PU"}="1H10  PU"; $resn_exch{"H102  PU"}="2H10  PU"; $resn_exch{"H103  PU"}="3H10  PU";
$resn_exch{"H191 PU0"}="1H19 PU0"; $resn_exch{"H192 PU0"}="2H19 PU0"; $resn_exch{"H193 PU0"}="3H19 PU0";
$resn_exch{"H8C1 PU0"}="1H8C PU0"; $resn_exch{"H8C2 PU0"}="2H8C PU0"; $resn_exch{"H8C3 PU0"}="3H8C PU0";
$resn_exch{"H9C1 PU0"}="1H9C PU0"; $resn_exch{"H9C2 PU0"}="2H9C PU0"; $resn_exch{"H5N1 PU0"}="1H5N PU0";
$resn_exch{"H5N2 PU0"}="2H5N PU0"; $resn_exch{"H151 PU0"}="1H15 PU0"; $resn_exch{"H152 PU0"}="2H15 PU0";
$resn_exch{"H161 PU0"}="1H16 PU0"; $resn_exch{"H162 PU0"}="2H16 PU0"; $resn_exch{"H171 PU0"}="1H17 PU0";
$resn_exch{"H172 PU0"}="2H17 PU0"; $resn_exch{"H181 PU0"}="1H18 PU0"; $resn_exch{"H182 PU0"}="2H18 PU0";
$resn_exch{"H183 PU0"}="3H18 PU0"; $resn_exch{"H201 PU1"}="1H20 PU1"; $resn_exch{"H202 PU1"}="2H20 PU1";
$resn_exch{"H203 PU1"}="3H20 PU1"; $resn_exch{"H191 PU1"}="1H19 PU1"; $resn_exch{"H192 PU1"}="2H19 PU1";
$resn_exch{"H193 PU1"}="3H19 PU1"; $resn_exch{"H181 PU1"}="1H18 PU1"; $resn_exch{"H182 PU1"}="2H18 PU1";
$resn_exch{"H183 PU1"}="3H18 PU1"; $resn_exch{"H6C1 PU1"}="1H6C PU1"; $resn_exch{"H6C2 PU1"}="2H6C PU1";
$resn_exch{"H5N1 PU1"}="1H5N PU1"; $resn_exch{"H5N2 PU1"}="2H5N PU1"; $resn_exch{"H7C1 PU1"}="1H7C PU1";
$resn_exch{"H7C2 PU1"}="2H7C PU1"; $resn_exch{"H8C1 PU1"}="1H8C PU1"; $resn_exch{"H8C2 PU1"}="2H8C PU1";
$resn_exch{"H9C1 PU1"}="1H9C PU1"; $resn_exch{"H9C2 PU1"}="2H9C PU1"; $resn_exch{"H181 PU2"}="1H18 PU2";
$resn_exch{"H182 PU2"}="2H18 PU2"; $resn_exch{"H183 PU2"}="3H18 PU2"; $resn_exch{"H9C1 PU2"}="1H9C PU2";
$resn_exch{"H9C2 PU2"}="2H9C PU2"; $resn_exch{"H5N1 PU2"}="1H5N PU2"; $resn_exch{"H5N2 PU2"}="2H5N PU2";
$resn_exch{"H7C1 PU2"}="1H7C PU2"; $resn_exch{"H7C2 PU2"}="2H7C PU2"; $resn_exch{"H7C3 PU2"}="3H7C PU2";
$resn_exch{"H191 PU3"}="1H19 PU3"; $resn_exch{"H192 PU3"}="2H19 PU3"; $resn_exch{"H193 PU3"}="3H19 PU3";
$resn_exch{"H7C1 PU3"}="1H7C PU3"; $resn_exch{"H7C2 PU3"}="2H7C PU3"; $resn_exch{"H7C3 PU3"}="3H7C PU3";
$resn_exch{"H8C1 PU3"}="1H8C PU3"; $resn_exch{"H8C2 PU3"}="2H8C PU3"; $resn_exch{"H8C3 PU3"}="3H8C PU3";
$resn_exch{"H9C1 PU3"}="1H9C PU3"; $resn_exch{"H9C2 PU3"}="2H9C PU3"; $resn_exch{"H5N1 PU3"}="1H5N PU3";
$resn_exch{"H5N2 PU3"}="2H5N PU3"; $resn_exch{"H151 PU3"}="1H15 PU3"; $resn_exch{"H152 PU3"}="2H15 PU3";
$resn_exch{"H161 PU3"}="1H16 PU3"; $resn_exch{"H162 PU3"}="2H16 PU3"; $resn_exch{"H171 PU3"}="1H17 PU3";
$resn_exch{"H172 PU3"}="2H17 PU3"; $resn_exch{"H181 PU3"}="1H18 PU3"; $resn_exch{"H182 PU3"}="2H18 PU3";
$resn_exch{"H183 PU3"}="3H18 PU3"; $resn_exch{"H181 PU4"}="1H18 PU4"; $resn_exch{"H182 PU4"}="2H18 PU4";
$resn_exch{"H183 PU4"}="3H18 PU4"; $resn_exch{"H171 PU4"}="1H17 PU4"; $resn_exch{"H172 PU4"}="2H17 PU4";
$resn_exch{"H161 PU4"}="1H16 PU4"; $resn_exch{"H162 PU4"}="2H16 PU4"; $resn_exch{"H151 PU4"}="1H15 PU4";
$resn_exch{"H152 PU4"}="2H15 PU4"; $resn_exch{"H5N1 PU4"}="1H5N PU4"; $resn_exch{"H5N2 PU4"}="2H5N PU4";
$resn_exch{"H9C1 PU4"}="1H9C PU4"; $resn_exch{"H9C2 PU4"}="2H9C PU4"; $resn_exch{"H7C1 PU4"}="1H7C PU4";
$resn_exch{"H7C2 PU4"}="2H7C PU4"; $resn_exch{"H7C3 PU4"}="3H7C PU4"; $resn_exch{"H181 PU5"}="1H18 PU5";
$resn_exch{"H182 PU5"}="2H18 PU5"; $resn_exch{"H183 PU5"}="3H18 PU5"; $resn_exch{"H171 PU5"}="1H17 PU5";
$resn_exch{"H172 PU5"}="2H17 PU5"; $resn_exch{"H161 PU5"}="1H16 PU5"; $resn_exch{"H162 PU5"}="2H16 PU5";
$resn_exch{"H151 PU5"}="1H15 PU5"; $resn_exch{"H152 PU5"}="2H15 PU5"; $resn_exch{"H9C1 PU5"}="1H9C PU5";
$resn_exch{"H9C2 PU5"}="2H9C PU5"; $resn_exch{"H5N1 PU5"}="1H5N PU5"; $resn_exch{"H5N2 PU5"}="2H5N PU5";
$resn_exch{"H7C1 PU5"}="1H7C PU5"; $resn_exch{"H7C2 PU5"}="2H7C PU5"; $resn_exch{"H7C3 PU5"}="3H7C PU5";
$resn_exch{"H181 PU6"}="1H18 PU6"; $resn_exch{"H182 PU6"}="2H18 PU6"; $resn_exch{"H183 PU6"}="3H18 PU6";
$resn_exch{"H171 PU6"}="1H17 PU6"; $resn_exch{"H172 PU6"}="2H17 PU6"; $resn_exch{"H161 PU6"}="1H16 PU6";
$resn_exch{"H162 PU6"}="2H16 PU6"; $resn_exch{"H151 PU6"}="1H15 PU6"; $resn_exch{"H152 PU6"}="2H15 PU6";
$resn_exch{"H5N1 PU6"}="1H5N PU6"; $resn_exch{"H5N2 PU6"}="2H5N PU6"; $resn_exch{"H9C1 PU6"}="1H9C PU6";
$resn_exch{"H9C2 PU6"}="2H9C PU6"; $resn_exch{"H7C1 PU6"}="1H7C PU6"; $resn_exch{"H7C2 PU6"}="2H7C PU6";
$resn_exch{"H191 PU7"}="1H19 PU7"; $resn_exch{"H192 PU7"}="2H19 PU7"; $resn_exch{"H193 PU7"}="3H19 PU7";
$resn_exch{"H8C1 PU7"}="1H8C PU7"; $resn_exch{"H8C2 PU7"}="2H8C PU7"; $resn_exch{"H8C3 PU7"}="3H8C PU7";
$resn_exch{"H9C1 PU7"}="1H9C PU7"; $resn_exch{"H9C2 PU7"}="2H9C PU7"; $resn_exch{"H5N1 PU7"}="1H5N PU7";
$resn_exch{"H5N2 PU7"}="2H5N PU7"; $resn_exch{"H151 PU7"}="1H15 PU7"; $resn_exch{"H152 PU7"}="2H15 PU7";
$resn_exch{"H161 PU7"}="1H16 PU7"; $resn_exch{"H162 PU7"}="2H16 PU7"; $resn_exch{"H171 PU7"}="1H17 PU7";
$resn_exch{"H172 PU7"}="2H17 PU7"; $resn_exch{"H181 PU7"}="1H18 PU7"; $resn_exch{"H182 PU7"}="2H18 PU7";
$resn_exch{"H183 PU7"}="3H18 PU7"; $resn_exch{"H191 PU8"}="1H19 PU8"; $resn_exch{"H192 PU8"}="2H19 PU8";
$resn_exch{"H193 PU8"}="3H19 PU8"; $resn_exch{"H7C1 PU8"}="1H7C PU8"; $resn_exch{"H7C2 PU8"}="2H7C PU8";
$resn_exch{"H7C3 PU8"}="3H7C PU8"; $resn_exch{"H8C1 PU8"}="1H8C PU8"; $resn_exch{"H8C2 PU8"}="2H8C PU8";
$resn_exch{"H8C3 PU8"}="3H8C PU8"; $resn_exch{"H9C1 PU8"}="1H9C PU8"; $resn_exch{"H9C2 PU8"}="2H9C PU8";
$resn_exch{"H5N1 PU8"}="1H5N PU8"; $resn_exch{"H5N2 PU8"}="2H5N PU8"; $resn_exch{"H151 PU8"}="1H15 PU8";
$resn_exch{"H152 PU8"}="2H15 PU8"; $resn_exch{"H161 PU8"}="1H16 PU8"; $resn_exch{"H162 PU8"}="2H16 PU8";
$resn_exch{"H171 PU8"}="1H17 PU8"; $resn_exch{"H172 PU8"}="2H17 PU8"; $resn_exch{"H181 PU8"}="1H18 PU8";
$resn_exch{"H182 PU8"}="2H18 PU8"; $resn_exch{"H183 PU8"}="3H18 PU8"; $resn_exch{"H201 PU9"}="1H20 PU9";
$resn_exch{"H202 PU9"}="2H20 PU9"; $resn_exch{"H203 PU9"}="3H20 PU9"; $resn_exch{"H191 PU9"}="1H19 PU9";
$resn_exch{"H192 PU9"}="2H19 PU9"; $resn_exch{"H193 PU9"}="3H19 PU9"; $resn_exch{"H181 PU9"}="1H18 PU9";
$resn_exch{"H182 PU9"}="2H18 PU9"; $resn_exch{"H183 PU9"}="3H18 PU9"; $resn_exch{"H6C1 PU9"}="1H6C PU9";
$resn_exch{"H6C2 PU9"}="2H6C PU9"; $resn_exch{"H5N1 PU9"}="1H5N PU9"; $resn_exch{"H5N2 PU9"}="2H5N PU9";
$resn_exch{"H7C1 PU9"}="1H7C PU9"; $resn_exch{"H7C2 PU9"}="2H7C PU9"; $resn_exch{"H8C1 PU9"}="1H8C PU9";
$resn_exch{"H8C2 PU9"}="2H8C PU9"; $resn_exch{"H9C1 PU9"}="1H9C PU9"; $resn_exch{"H9C2 PU9"}="2H9C PU9";
$resn_exch{"H6A1 PUA"}="1H6A PUA"; $resn_exch{"H6A2 PUA"}="2H6A PUA"; $resn_exch{"H5'1 PUA"}="1H5* PUA";
$resn_exch{"H5'2 PUA"}="2H5* PUA"; $resn_exch{" H4' PUA"}=" H4* PUA"; $resn_exch{" H1' PUA"}=" H1* PUA";
$resn_exch{" H2' PUA"}=" H2* PUA"; $resn_exch{"HO2' PUA"}="HO2* PUA"; $resn_exch{" H3' PUA"}=" H3* PUA";
$resn_exch{" H51 PUA"}="1H5  PUA"; $resn_exch{" H52 PUA"}="2H5  PUA"; $resn_exch{" H21 PUA"}="1H2  PUA";
$resn_exch{" H22 PUA"}="2H2  PUA"; $resn_exch{"HHC1 PUB"}="1HHC PUB"; $resn_exch{"HHC2 PUB"}="2HHC PUB";
$resn_exch{"HMC1 PUB"}="1HMC PUB"; $resn_exch{"HMC2 PUB"}="2HMC PUB"; $resn_exch{"HMC3 PUB"}="3HMC PUB";
$resn_exch{"HAC1 PUB"}="1HAC PUB"; $resn_exch{"HAC2 PUB"}="2HAC PUB"; $resn_exch{"HBC1 PUB"}="1HBC PUB";
$resn_exch{"HBC2 PUB"}="2HBC PUB"; $resn_exch{"HOC2 PUB"}="2HOC PUB"; $resn_exch{"H1D1 PUB"}="1H1D PUB";
$resn_exch{"HMD1 PUB"}="1HMD PUB"; $resn_exch{"HMD2 PUB"}="2HMD PUB"; $resn_exch{"HMD3 PUB"}="3HMD PUB";
$resn_exch{"HBD1 PUB"}="1HBD PUB"; $resn_exch{"HBD2 PUB"}="2HBD PUB"; $resn_exch{"HBD3 PUB"}="3HBD PUB";
$resn_exch{"H4A1 PUB"}="1H4A PUB"; $resn_exch{"HMA1 PUB"}="1HMA PUB"; $resn_exch{"HMA2 PUB"}="2HMA PUB";
$resn_exch{"HMA3 PUB"}="3HMA PUB"; $resn_exch{"HBA1 PUB"}="1HBA PUB"; $resn_exch{"HBA2 PUB"}="2HBA PUB";
$resn_exch{"HBA3 PUB"}="3HBA PUB"; $resn_exch{"HHA1 PUB"}="1HHA PUB"; $resn_exch{"HHA2 PUB"}="2HHA PUB";
$resn_exch{"HAA1 PUB"}="1HAA PUB"; $resn_exch{"HAA2 PUB"}="2HAA PUB"; $resn_exch{"HAD1 PUB"}="1HAD PUB";
$resn_exch{"HAD2 PUB"}="2HAD PUB"; $resn_exch{"HHB1 PUB"}="1HHB PUB"; $resn_exch{"HMB1 PUB"}="1HMB PUB";
$resn_exch{"HMB2 PUB"}="2HMB PUB"; $resn_exch{"HMB3 PUB"}="3HMB PUB"; $resn_exch{"HAB1 PUB"}="1HAB PUB";
$resn_exch{"HAB2 PUB"}="2HAB PUB"; $resn_exch{"HBB1 PUB"}="1HBB PUB"; $resn_exch{"HBB2 PUB"}="2HBB PUB";
$resn_exch{"HOB2 PUB"}="2HOB PUB"; $resn_exch{"H101 PUG"}="1H10 PUG"; $resn_exch{"H102 PUG"}="2H10 PUG";
$resn_exch{"H141 PUG"}="1H14 PUG"; $resn_exch{"H142 PUG"}="2H14 PUG"; $resn_exch{"H201 PUL"}="1H20 PUL";
$resn_exch{"H202 PUL"}="2H20 PUL"; $resn_exch{"H431 PUL"}="1H43 PUL"; $resn_exch{"H432 PUL"}="2H43 PUL";
$resn_exch{"H433 PUL"}="3H43 PUL"; $resn_exch{"H421 PUL"}="1H42 PUL"; $resn_exch{"H422 PUL"}="2H42 PUL";
$resn_exch{"H423 PUL"}="3H42 PUL"; $resn_exch{"H411 PUL"}="1H41 PUL"; $resn_exch{"H412 PUL"}="2H41 PUL";
$resn_exch{"H413 PUL"}="3H41 PUL"; $resn_exch{"H2C1 PUL"}="1H2C PUL"; $resn_exch{"H2C2 PUL"}="2H2C PUL";
$resn_exch{"H441 PUL"}="1H44 PUL"; $resn_exch{"H442 PUL"}="2H44 PUL"; $resn_exch{"H443 PUL"}="3H44 PUL";
$resn_exch{"H451 PUL"}="1H45 PUL"; $resn_exch{"H452 PUL"}="2H45 PUL"; $resn_exch{"H453 PUL"}="3H45 PUL";
$resn_exch{"H461 PUL"}="1H46 PUL"; $resn_exch{"H462 PUL"}="2H46 PUL"; $resn_exch{"H463 PUL"}="3H46 PUL";
$resn_exch{"H471 PUL"}="1H47 PUL"; $resn_exch{"H472 PUL"}="2H47 PUL"; $resn_exch{"H473 PUL"}="3H47 PUL";
$resn_exch{"H401 PUL"}="1H40 PUL"; $resn_exch{"H402 PUL"}="2H40 PUL"; $resn_exch{"H403 PUL"}="3H40 PUL";
$resn_exch{"H341 PUL"}="1H34 PUL"; $resn_exch{"H342 PUL"}="2H34 PUL"; $resn_exch{"H343 PUL"}="3H34 PUL";
$resn_exch{" H1' PUR"}=" H1* PUR"; $resn_exch{" H2' PUR"}=" H2* PUR"; $resn_exch{"HO2' PUR"}="*HO2 PUR";
$resn_exch{" H3' PUR"}=" H3* PUR"; $resn_exch{"HO3' PUR"}="*HO3 PUR"; $resn_exch{" H4' PUR"}=" H4* PUR";
$resn_exch{"H5'1 PUR"}="1H5* PUR"; $resn_exch{"H5'2 PUR"}="2H5* PUR"; $resn_exch{"HO5' PUR"}="*HO5 PUR";
$resn_exch{"HN21 PUT"}="1HN2 PUT"; $resn_exch{"HN22 PUT"}="2HN2 PUT"; $resn_exch{" H41 PUT"}="1H4  PUT";
$resn_exch{" H42 PUT"}="2H4  PUT"; $resn_exch{" H31 PUT"}="1H3  PUT"; $resn_exch{" H32 PUT"}="2H3  PUT";
$resn_exch{" H21 PUT"}="1H2  PUT"; $resn_exch{" H22 PUT"}="2H2  PUT"; $resn_exch{" H11 PUT"}="1H1  PUT";
$resn_exch{" H12 PUT"}="2H1  PUT"; $resn_exch{"HN11 PUT"}="1HN1 PUT"; $resn_exch{"HN12 PUT"}="2HN1 PUT";
$resn_exch{"H181 PUX"}="1H18 PUX"; $resn_exch{"H182 PUX"}="2H18 PUX"; $resn_exch{"H183 PUX"}="3H18 PUX";
$resn_exch{"H171 PUX"}="1H17 PUX"; $resn_exch{"H172 PUX"}="2H17 PUX"; $resn_exch{"H161 PUX"}="1H16 PUX";
$resn_exch{"H162 PUX"}="2H16 PUX"; $resn_exch{"H151 PUX"}="1H15 PUX"; $resn_exch{"H152 PUX"}="2H15 PUX";
$resn_exch{"H5N1 PUX"}="1H5N PUX"; $resn_exch{"H5N2 PUX"}="2H5N PUX"; $resn_exch{"H9C1 PUX"}="1H9C PUX";
$resn_exch{"H9C2 PUX"}="2H9C PUX"; $resn_exch{"H7C1 PUX"}="1H7C PUX"; $resn_exch{"H7C2 PUX"}="2H7C PUX";
$resn_exch{" HN1 PUY"}="1HN  PUY"; $resn_exch{" HN2 PUY"}="2HN  PUY"; $resn_exch{" HB1 PUY"}="1HB  PUY";
$resn_exch{" HB2 PUY"}="2HB  PUY"; $resn_exch{"HMZ1 PUY"}="1HMZ PUY"; $resn_exch{"HMZ2 PUY"}="2HMZ PUY";
$resn_exch{"HMZ3 PUY"}="3HMZ PUY"; $resn_exch{" H5' PUY"}=" H5* PUY"; $resn_exch{"H5'1 PUY"}="1H5* PUY";
$resn_exch{"H5'' PUY"}="2H5* PUY"; $resn_exch{" H4' PUY"}=" H4* PUY"; $resn_exch{" H1' PUY"}=" H1* PUY";
$resn_exch{" H91 PUY"}="1H9  PUY"; $resn_exch{" H92 PUY"}="2H9  PUY"; $resn_exch{" H93 PUY"}="3H9  PUY";
$resn_exch{"H101 PUY"}="1H10 PUY"; $resn_exch{"H102 PUY"}="2H10 PUY"; $resn_exch{"H103 PUY"}="3H10 PUY";
$resn_exch{" H2' PUY"}=" H2* PUY"; $resn_exch{" H3' PUY"}=" H3* PUY"; $resn_exch{"H171 PUZ"}="1H17 PUZ";
$resn_exch{"H172 PUZ"}="2H17 PUZ"; $resn_exch{"H161 PUZ"}="1H16 PUZ"; $resn_exch{"H162 PUZ"}="2H16 PUZ";
$resn_exch{"H151 PUZ"}="1H15 PUZ"; $resn_exch{"H152 PUZ"}="2H15 PUZ"; $resn_exch{"H5N1 PUZ"}="1H5N PUZ";
$resn_exch{"H5N2 PUZ"}="2H5N PUZ"; $resn_exch{"H9C1 PUZ"}="1H9C PUZ"; $resn_exch{"H9C2 PUZ"}="2H9C PUZ";
$resn_exch{"H7C1 PUZ"}="1H7C PUZ"; $resn_exch{"H7C2 PUZ"}="2H7C PUZ"; $resn_exch{"H7C3 PUZ"}="3H7C PUZ";
$resn_exch{"H8C1 PUZ"}="1H8C PUZ"; $resn_exch{"H8C2 PUZ"}="2H8C PUZ"; $resn_exch{"H8C3 PUZ"}="3H8C PUZ";
$resn_exch{" H31 PVA"}="1H3  PVA"; $resn_exch{" H32 PVA"}="2H3  PVA"; $resn_exch{" H33 PVA"}="3H3  PVA";
$resn_exch{" HM1 PVA"}="1HM  PVA"; $resn_exch{" HM2 PVA"}="2HM  PVA"; $resn_exch{" HM3 PVA"}="3HM  PVA";
$resn_exch{" HN1 PVA"}="1HN  PVA"; $resn_exch{" HN2 PVA"}="2HN  PVA"; $resn_exch{"HOP2 PVA"}="2HOP PVA";
$resn_exch{"HOP3 PVA"}="3HOP PVA"; $resn_exch{"H161 PVB"}="1H16 PVB"; $resn_exch{"H162 PVB"}="2H16 PVB";
$resn_exch{"H163 PVB"}="3H16 PVB"; $resn_exch{"H141 PVB"}="1H14 PVB"; $resn_exch{"H142 PVB"}="2H14 PVB";
$resn_exch{"H143 PVB"}="3H14 PVB"; $resn_exch{"H151 PVB"}="1H15 PVB"; $resn_exch{"H152 PVB"}="2H15 PVB";
$resn_exch{"H111 PVB"}="1H11 PVB"; $resn_exch{"H112 PVB"}="2H11 PVB"; $resn_exch{"H113 PVB"}="3H11 PVB";
$resn_exch{"H101 PVB"}="1H10 PVB"; $resn_exch{"H102 PVB"}="2H10 PVB"; $resn_exch{"H103 PVB"}="3H10 PVB";
$resn_exch{"HP21 PVC"}="1HP2 PVC"; $resn_exch{"HP22 PVC"}="2HP2 PVC"; $resn_exch{"HP31 PVC"}="1HP3 PVC";
$resn_exch{"HP32 PVC"}="2HP3 PVC"; $resn_exch{"HP41 PVC"}="1HP4 PVC"; $resn_exch{"HP42 PVC"}="2HP4 PVC";
$resn_exch{"HP51 PVC"}="1HP5 PVC"; $resn_exch{"HP52 PVC"}="2HP5 PVC"; $resn_exch{"HP61 PVC"}="1HP6 PVC";
$resn_exch{"HP62 PVC"}="2HP6 PVC"; $resn_exch{"HP71 PVC"}="1HP7 PVC"; $resn_exch{"HP72 PVC"}="2HP7 PVC";
$resn_exch{"HP81 PVC"}="1HP8 PVC"; $resn_exch{"HP82 PVC"}="2HP8 PVC"; $resn_exch{"HP91 PVC"}="1HP9 PVC";
$resn_exch{"HP92 PVC"}="2HP9 PVC"; $resn_exch{"HPA1 PVC"}="1HPA PVC"; $resn_exch{"HPA2 PVC"}="2HPA PVC";
$resn_exch{"HPB1 PVC"}="1HPB PVC"; $resn_exch{"HPB2 PVC"}="2HPB PVC"; $resn_exch{"HPC1 PVC"}="1HPC PVC";
$resn_exch{"HPC2 PVC"}="2HPC PVC"; $resn_exch{"HPD1 PVC"}="1HPD PVC"; $resn_exch{"HPD2 PVC"}="2HPD PVC";
$resn_exch{"HC61 PVC"}="1HC6 PVC"; $resn_exch{"HC62 PVC"}="2HC6 PVC"; $resn_exch{"HPE1 PVC"}="1HPE PVC";
$resn_exch{"HPE2 PVC"}="2HPE PVC"; $resn_exch{"HPF1 PVC"}="1HPF PVC"; $resn_exch{"HPF2 PVC"}="2HPF PVC";
$resn_exch{"HPG1 PVC"}="1HPG PVC"; $resn_exch{"HPG2 PVC"}="2HPG PVC"; $resn_exch{"HPG3 PVC"}="3HPG PVC";
$resn_exch{"H121 PVD"}="1H12 PVD"; $resn_exch{"H122 PVD"}="2H12 PVD"; $resn_exch{"H131 PVD"}="1H13 PVD";
$resn_exch{"H132 PVD"}="2H13 PVD"; $resn_exch{"H201 PVD"}="1H20 PVD"; $resn_exch{"H202 PVD"}="2H20 PVD";
$resn_exch{"H211 PVD"}="1H21 PVD"; $resn_exch{"H212 PVD"}="2H21 PVD"; $resn_exch{"H291 PVD"}="1H29 PVD";
$resn_exch{"H292 PVD"}="2H29 PVD"; $resn_exch{"H351 PVD"}="1H35 PVD"; $resn_exch{"H352 PVD"}="2H35 PVD";
$resn_exch{"H361 PVD"}="1H36 PVD"; $resn_exch{"H362 PVD"}="2H36 PVD"; $resn_exch{"H371 PVD"}="1H37 PVD";
$resn_exch{"H372 PVD"}="2H37 PVD"; $resn_exch{"H401 PVD"}="1H40 PVD"; $resn_exch{"H402 PVD"}="2H40 PVD";
$resn_exch{"H461 PVD"}="1H46 PVD"; $resn_exch{"H462 PVD"}="2H46 PVD"; $resn_exch{"H531 PVD"}="1H53 PVD";
$resn_exch{"H532 PVD"}="2H53 PVD"; $resn_exch{"H541 PVD"}="1H54 PVD"; $resn_exch{"H542 PVD"}="2H54 PVD";
$resn_exch{"H551 PVD"}="1H55 PVD"; $resn_exch{"H552 PVD"}="2H55 PVD"; $resn_exch{"H651 PVD"}="1H65 PVD";
$resn_exch{"H652 PVD"}="2H65 PVD"; $resn_exch{"H661 PVD"}="1H66 PVD"; $resn_exch{"H662 PVD"}="2H66 PVD";
$resn_exch{"H671 PVD"}="1H67 PVD"; $resn_exch{"H672 PVD"}="2H67 PVD"; $resn_exch{"H681 PVD"}="1H68 PVD";
$resn_exch{"H682 PVD"}="2H68 PVD"; $resn_exch{"H741 PVD"}="1H74 PVD"; $resn_exch{"H742 PVD"}="2H74 PVD";
$resn_exch{"H751 PVD"}="1H75 PVD"; $resn_exch{"H752 PVD"}="2H75 PVD"; $resn_exch{"H761 PVD"}="1H76 PVD";
$resn_exch{"H762 PVD"}="2H76 PVD"; $resn_exch{"H871 PVD"}="1H87 PVD"; $resn_exch{"H872 PVD"}="2H87 PVD";
$resn_exch{"H873 PVD"}="3H87 PVD"; $resn_exch{"H941 PVD"}="1H94 PVD"; $resn_exch{"H942 PVD"}="2H94 PVD";
$resn_exch{"H943 PVD"}="3H94 PVD"; $resn_exch{" H2  PVH"}=" HN2 PVH"; $resn_exch{" HB2 PVH"}="1HB  PVH";
$resn_exch{" HB3 PVH"}="2HB  PVH"; $resn_exch{" HM1 PVH"}="1HM  PVH"; $resn_exch{" HM2 PVH"}="2HM  PVH";
$resn_exch{" HM3 PVH"}="3HM  PVH"; $resn_exch{" H33 PVL"}="3H3  PVL"; $resn_exch{" H11 PVL"}="1H1  PVL";
$resn_exch{" H12 PVL"}="2H1  PVL"; $resn_exch{" H13 PVL"}="3H1  PVL"; $resn_exch{"H321 PVN"}="1H32 PVN";
$resn_exch{"H322 PVN"}="2H32 PVN"; $resn_exch{"H211 PVN"}="1H21 PVN"; $resn_exch{"H212 PVN"}="2H21 PVN";
$resn_exch{"H213 PVN"}="3H21 PVN"; $resn_exch{"H5C1 PVN"}="1H5C PVN"; $resn_exch{"H5C2 PVN"}="2H5C PVN";
$resn_exch{"H411 PVN"}="1H41 PVN"; $resn_exch{"H412 PVN"}="2H41 PVN"; $resn_exch{"H413 PVN"}="3H41 PVN";
$resn_exch{"H101 PVN"}="1H10 PVN"; $resn_exch{"H511 PVN"}="1H51 PVN"; $resn_exch{"H512 PVN"}="2H51 PVN";
$resn_exch{"H521 PVN"}="1H52 PVN"; $resn_exch{"H522 PVN"}="2H52 PVN"; $resn_exch{"H611 PVN"}="1H61 PVN";
$resn_exch{"H612 PVN"}="2H61 PVN"; $resn_exch{"H621 PVN"}="1H62 PVN"; $resn_exch{"H622 PVN"}="2H62 PVN";
$resn_exch{"H151 PVN"}="1H15 PVN"; $resn_exch{"H711 PVN"}="1H71 PVN"; $resn_exch{"H712 PVN"}="2H71 PVN";
$resn_exch{"H713 PVN"}="3H71 PVN"; $resn_exch{"H811 PVN"}="1H81 PVN"; $resn_exch{"H812 PVN"}="2H81 PVN";
$resn_exch{"H813 PVN"}="3H81 PVN"; $resn_exch{"H911 PVN"}="1H91 PVN"; $resn_exch{"H912 PVN"}="2H91 PVN";
$resn_exch{"H921 PVN"}="1H92 PVN"; $resn_exch{"H922 PVN"}="2H92 PVN"; $resn_exch{"H923 PVN"}="3H92 PVN";
$resn_exch{"H323 PVN"}="3H32 PVN"; $resn_exch{"H151 PX1"}="1H15 PX1"; $resn_exch{"H152 PX1"}="2H15 PX1";
$resn_exch{"H153 PX1"}="3H15 PX1"; $resn_exch{"H161 PX1"}="1H16 PX1"; $resn_exch{"H162 PX1"}="2H16 PX1";
$resn_exch{"H163 PX1"}="3H16 PX1"; $resn_exch{"H7N1 PX1"}="1H7N PX1"; $resn_exch{"H7N2 PX1"}="2H7N PX1";
$resn_exch{"HN21 PXA"}="1HN2 PXA"; $resn_exch{"HN22 PXA"}="2HN2 PXA"; $resn_exch{" H'  PXA"}=" H*  PXA";
$resn_exch{"H151 PXA"}="1H15 PXA"; $resn_exch{"H152 PXA"}="2H15 PXA"; $resn_exch{"H153 PXA"}="3H15 PXA";
$resn_exch{"H161 PXA"}="1H16 PXA"; $resn_exch{"H162 PXA"}="2H16 PXA"; $resn_exch{"H163 PXA"}="3H16 PXA";
$resn_exch{"HN21 PXF"}="1HN2 PXF"; $resn_exch{"HN22 PXF"}="2HN2 PXF"; $resn_exch{" H'  PXF"}=" H*  PXF";
$resn_exch{"H151 PXF"}="1H15 PXF"; $resn_exch{"H152 PXF"}="2H15 PXF"; $resn_exch{"H153 PXF"}="3H15 PXF";
$resn_exch{"H161 PXF"}="1H16 PXF"; $resn_exch{"H162 PXF"}="2H16 PXF"; $resn_exch{"H163 PXF"}="3H16 PXF";
$resn_exch{"H2A1 PXG"}="1H2A PXG"; $resn_exch{"H2A2 PXG"}="2H2A PXG"; $resn_exch{"H2A3 PXG"}="3H2A PXG";
$resn_exch{"H5A1 PXG"}="1H5A PXG"; $resn_exch{"H5A2 PXG"}="2H5A PXG"; $resn_exch{"HOP1 PXG"}="1HOP PXG";
$resn_exch{"HOP2 PXG"}="2HOP PXG"; $resn_exch{"H4A1 PXG"}="1H4A PXG"; $resn_exch{"H4A2 PXG"}="2H4A PXG";
$resn_exch{"H1C1 PXI"}="1H1C PXI"; $resn_exch{"H1C2 PXI"}="2H1C PXI"; $resn_exch{"H1C3 PXI"}="3H1C PXI";
$resn_exch{"H2C1 PXI"}="1H2C PXI"; $resn_exch{"H2C2 PXI"}="2H2C PXI"; $resn_exch{"H6C1 PXI"}="1H6C PXI";
$resn_exch{"H6C2 PXI"}="2H6C PXI"; $resn_exch{"H6C3 PXI"}="3H6C PXI"; $resn_exch{"H101 PXI"}="1H10 PXI";
$resn_exch{"H102 PXI"}="2H10 PXI"; $resn_exch{"H103 PXI"}="3H10 PXI"; $resn_exch{"H111 PXI"}="1H11 PXI";
$resn_exch{"H112 PXI"}="2H11 PXI"; $resn_exch{"H131 PXI"}="1H13 PXI"; $resn_exch{"H132 PXI"}="2H13 PXI";
$resn_exch{"H133 PXI"}="3H13 PXI"; $resn_exch{"H141 PXI"}="1H14 PXI"; $resn_exch{"H142 PXI"}="2H14 PXI";
$resn_exch{"H143 PXI"}="3H14 PXI"; $resn_exch{"H171 PXI"}="1H17 PXI"; $resn_exch{"H172 PXI"}="2H17 PXI";
$resn_exch{"H173 PXI"}="3H17 PXI"; $resn_exch{"H181 PXI"}="1H18 PXI"; $resn_exch{"H182 PXI"}="2H18 PXI";
$resn_exch{"H201 PXI"}="1H20 PXI"; $resn_exch{"H202 PXI"}="2H20 PXI"; $resn_exch{"H203 PXI"}="3H20 PXI";
$resn_exch{"H251 PXI"}="1H25 PXI"; $resn_exch{"H252 PXI"}="2H25 PXI"; $resn_exch{"H253 PXI"}="3H25 PXI";
$resn_exch{"H5A1 PXL"}="1H5A PXL"; $resn_exch{"H5A2 PXL"}="2H5A PXL"; $resn_exch{"H2A1 PXL"}="1H2A PXL";
$resn_exch{"H2A2 PXL"}="2H2A PXL"; $resn_exch{"H2A3 PXL"}="3H2A PXL"; $resn_exch{"H2A1 PXM"}="1H2A PXM";
$resn_exch{"H2A2 PXM"}="2H2A PXM"; $resn_exch{"H2A3 PXM"}="3H2A PXM"; $resn_exch{"H5A1 PXM"}="1H5A PXM";
$resn_exch{"H5A2 PXM"}="2H5A PXM"; $resn_exch{"H4A1 PXM"}="1H4A PXM"; $resn_exch{"H4A2 PXM"}="2H4A PXM";
$resn_exch{"HN41 PXM"}="1HN4 PXM"; $resn_exch{"HN42 PXM"}="2HN4 PXM"; $resn_exch{"H2A1 PXP"}="1H2A PXP";
$resn_exch{"H2A2 PXP"}="2H2A PXP"; $resn_exch{"H2A3 PXP"}="3H2A PXP"; $resn_exch{"H4A1 PXP"}="1H4A PXP";
$resn_exch{"H4A2 PXP"}="2H4A PXP"; $resn_exch{"HOA4 PXP"}="4HOA PXP"; $resn_exch{"H5A1 PXP"}="1H5A PXP";
$resn_exch{"H5A2 PXP"}="2H5A PXP"; $resn_exch{"HOP2 PXP"}="2HOP PXP"; $resn_exch{"HOP3 PXP"}="3HOP PXP";
$resn_exch{"H131 PXT"}="1H13 PXT"; $resn_exch{"H132 PXT"}="2H13 PXT"; $resn_exch{"H171 PXT"}="1H17 PXT";
$resn_exch{"H172 PXT"}="2H17 PXT"; $resn_exch{"H421 PXT"}="1H42 PXT"; $resn_exch{"H422 PXT"}="2H42 PXT";
$resn_exch{"H423 PXT"}="3H42 PXT"; $resn_exch{" H91 PXT"}="1H9  PXT"; $resn_exch{" H92 PXT"}="2H9  PXT";
$resn_exch{" H81 PXT"}="1H8  PXT"; $resn_exch{" H82 PXT"}="2H8  PXT"; $resn_exch{" H61 PXT"}="1H6  PXT";
$resn_exch{" H62 PXT"}="2H6  PXT"; $resn_exch{" H51 PXT"}="1H5  PXT"; $resn_exch{" H52 PXT"}="2H5  PXT";
$resn_exch{" H41 PXT"}="1H4  PXT"; $resn_exch{" H42 PXT"}="2H4  PXT"; $resn_exch{"H411 PXT"}="1H41 PXT";
$resn_exch{"H412 PXT"}="2H41 PXT"; $resn_exch{"H413 PXT"}="3H41 PXT"; $resn_exch{"H431 PXT"}="1H43 PXT";
$resn_exch{"H432 PXT"}="2H43 PXT"; $resn_exch{"H433 PXT"}="3H43 PXT"; $resn_exch{"H191 PXT"}="1H19 PXT";
$resn_exch{"H192 PXT"}="2H19 PXT"; $resn_exch{"H201 PXT"}="1H20 PXT"; $resn_exch{"H202 PXT"}="2H20 PXT";
$resn_exch{"H441 PXT"}="1H44 PXT"; $resn_exch{"H442 PXT"}="2H44 PXT"; $resn_exch{"H443 PXT"}="3H44 PXT";
$resn_exch{"H241 PXT"}="1H24 PXT"; $resn_exch{"H242 PXT"}="2H24 PXT"; $resn_exch{"H231 PXT"}="1H23 PXT";
$resn_exch{"H232 PXT"}="2H23 PXT"; $resn_exch{"H261 PXT"}="1H26 PXT"; $resn_exch{"H262 PXT"}="2H26 PXT";
$resn_exch{"H451 PXT"}="1H45 PXT"; $resn_exch{"H452 PXT"}="2H45 PXT"; $resn_exch{"H453 PXT"}="3H45 PXT";
$resn_exch{"H461 PXT"}="1H46 PXT"; $resn_exch{"H462 PXT"}="2H46 PXT"; $resn_exch{"H463 PXT"}="3H46 PXT";
$resn_exch{"H341 PXT"}="1H34 PXT"; $resn_exch{"H342 PXT"}="2H34 PXT"; $resn_exch{"H401 PXT"}="1H40 PXT";
$resn_exch{"H402 PXT"}="2H40 PXT"; $resn_exch{"H391 PXT"}="1H39 PXT"; $resn_exch{"H392 PXT"}="2H39 PXT";
$resn_exch{"H471 PXT"}="1H47 PXT"; $resn_exch{"H472 PXT"}="2H47 PXT"; $resn_exch{"H473 PXT"}="3H47 PXT";
$resn_exch{"H1'1 PXY"}="1H1' PXY"; $resn_exch{"H1'2 PXY"}="2H1' PXY"; $resn_exch{"H1'3 PXY"}="3H1' PXY";
$resn_exch{"H4'1 PXY"}="1H4' PXY"; $resn_exch{"H4'2 PXY"}="2H4' PXY"; $resn_exch{"H4'3 PXY"}="3H4' PXY";
$resn_exch{"HN21 PXZ"}="1HN2 PXZ"; $resn_exch{"HN22 PXZ"}="2HN2 PXZ"; $resn_exch{" H'  PXZ"}=" H*  PXZ";
$resn_exch{"H151 PXZ"}="1H15 PXZ"; $resn_exch{"H152 PXZ"}="2H15 PXZ"; $resn_exch{"H153 PXZ"}="3H15 PXZ";
$resn_exch{"H161 PXZ"}="1H16 PXZ"; $resn_exch{"H162 PXZ"}="2H16 PXZ"; $resn_exch{"H163 PXZ"}="3H16 PXZ";
$resn_exch{" H1  PY2"}="1H   PY2"; $resn_exch{" H2A PY2"}="2H   PY2"; $resn_exch{"H181 PY3"}="1H18 PY3";
$resn_exch{"H182 PY3"}="2H18 PY3"; $resn_exch{"H191 PY3"}="1H19 PY3"; $resn_exch{"H192 PY3"}="2H19 PY3";
$resn_exch{"H221 PY3"}="1H22 PY3"; $resn_exch{"H222 PY3"}="2H22 PY3"; $resn_exch{"H331 PY3"}="1H33 PY3";
$resn_exch{"H332 PY3"}="2H33 PY3"; $resn_exch{"H333 PY3"}="3H33 PY3"; $resn_exch{"H341 PY3"}="1H34 PY3";
$resn_exch{"H342 PY3"}="2H34 PY3"; $resn_exch{"H343 PY3"}="3H34 PY3"; $resn_exch{"H401 PY3"}="1H40 PY3";
$resn_exch{"H402 PY3"}="2H40 PY3"; $resn_exch{"H441 PY3"}="1H44 PY3"; $resn_exch{"H442 PY3"}="2H44 PY3";
$resn_exch{"H451 PY3"}="1H45 PY3"; $resn_exch{"H452 PY3"}="2H45 PY3"; $resn_exch{"H453 PY3"}="3H45 PY3";
$resn_exch{"H471 PY3"}="1H47 PY3"; $resn_exch{"H472 PY3"}="2H47 PY3"; $resn_exch{"H473 PY3"}="3H47 PY3";
$resn_exch{"H2A1 PY4"}="1H2A PY4"; $resn_exch{"H2A2 PY4"}="2H2A PY4"; $resn_exch{"H2A3 PY4"}="3H2A PY4";
$resn_exch{"H4A1 PY4"}="1H4A PY4"; $resn_exch{"H4A2 PY4"}="2H4A PY4"; $resn_exch{"H5A1 PY4"}="1H5A PY4";
$resn_exch{"H5A2 PY4"}="2H5A PY4"; $resn_exch{"HOP2 PY4"}="2HOP PY4"; $resn_exch{"HOP3 PY4"}="3HOP PY4";
$resn_exch{" HB1 PY4"}="1HB  PY4"; $resn_exch{" HB2 PY4"}="2HB  PY4"; $resn_exch{"H2A1 PY5"}="1H2A PY5";
$resn_exch{"H2A2 PY5"}="2H2A PY5"; $resn_exch{"H2A3 PY5"}="3H2A PY5"; $resn_exch{"H4A1 PY5"}="1H4A PY5";
$resn_exch{"H4A2 PY5"}="2H4A PY5"; $resn_exch{"H5A1 PY5"}="1H5A PY5"; $resn_exch{"H5A2 PY5"}="2H5A PY5";
$resn_exch{"HOP2 PY5"}="2HOP PY5"; $resn_exch{"HOP3 PY5"}="3HOP PY5"; $resn_exch{" HB1 PY5"}="1HB  PY5";
$resn_exch{" HB2 PY5"}="2HB  PY5"; $resn_exch{"H2A1 PY6"}="1H2A PY6"; $resn_exch{"H2A2 PY6"}="2H2A PY6";
$resn_exch{"H2A3 PY6"}="3H2A PY6"; $resn_exch{"H4A1 PY6"}="1H4A PY6"; $resn_exch{"H4A2 PY6"}="2H4A PY6";
$resn_exch{"H5A1 PY6"}="1H5A PY6"; $resn_exch{"H5A2 PY6"}="2H5A PY6"; $resn_exch{"HOP2 PY6"}="2HOP PY6";
$resn_exch{"HOP3 PY6"}="3HOP PY6"; $resn_exch{" HB1 PY6"}="1HB  PY6"; $resn_exch{" HB2 PY6"}="2HB  PY6";
$resn_exch{" H71 PY7"}="1H7  PY7"; $resn_exch{" H72 PY7"}="2H7  PY7"; $resn_exch{"H8C1 PY8"}="1H8C PY8";
$resn_exch{"H8C2 PY8"}="2H8C PY8"; $resn_exch{"H9C1 PY8"}="1H9C PY8"; $resn_exch{"H9C2 PY8"}="2H9C PY8";
$resn_exch{"H101 PY8"}="1H10 PY8"; $resn_exch{"H102 PY8"}="2H10 PY8"; $resn_exch{" HC2 PYA"}=" H2  PYA";
$resn_exch{" HB3 PYA"}="1HB  PYA"; $resn_exch{" HB2 PYA"}="2HB  PYA"; $resn_exch{" H   PYA"}="1HN  PYA";
$resn_exch{" H2  PYA"}="2HN  PYA"; $resn_exch{" HD1 PYB"}="1HD  PYB"; $resn_exch{" HD2 PYB"}="2HD  PYB";
$resn_exch{" HD3 PYB"}="3HD  PYB"; $resn_exch{" H31 PYC"}="1H3  PYC"; $resn_exch{" H41 PYC"}="1H4  PYC";
$resn_exch{" H61 PYC"}="1H6  PYC"; $resn_exch{"HM21 PYD"}="1HM2 PYD"; $resn_exch{"HM22 PYD"}="2HM2 PYD";
$resn_exch{"HM23 PYD"}="3HM2 PYD"; $resn_exch{"HN41 PYD"}="1HN4 PYD"; $resn_exch{"HN42 PYD"}="2HN4 PYD";
$resn_exch{"HC71 PYD"}="1HC7 PYD"; $resn_exch{"HC72 PYD"}="2HC7 PYD"; $resn_exch{"HC73 PYD"}="3HC7 PYD";
$resn_exch{" H11 PYE"}="1H1  PYE"; $resn_exch{" H12 PYE"}="2H1  PYE"; $resn_exch{" H21 PYE"}="1H2  PYE";
$resn_exch{" H22 PYE"}="2H2  PYE"; $resn_exch{" H31 PYE"}="1H3  PYE"; $resn_exch{" H32 PYE"}="2H3  PYE";
$resn_exch{" H41 PYE"}="1H4  PYE"; $resn_exch{" H42 PYE"}="2H4  PYE"; $resn_exch{" H51 PYE"}="1H5  PYE";
$resn_exch{" H52 PYE"}="2H5  PYE"; $resn_exch{" H71 PYF"}="1H7  PYF"; $resn_exch{" H72 PYF"}="2H7  PYF";
$resn_exch{" HB1 PYH"}="1HB  PYH"; $resn_exch{" HB2 PYH"}="2HB  PYH"; $resn_exch{" HG1 PYH"}="1HG  PYH";
$resn_exch{" HG2 PYH"}="2HG  PYH"; $resn_exch{" HD1 PYH"}="1HD  PYH"; $resn_exch{" HD2 PYH"}="2HD  PYH";
$resn_exch{" HE1 PYH"}="1HE  PYH"; $resn_exch{" HE2 PYH"}="2HE  PYH"; $resn_exch{"HAJ1 PYH"}="1HAJ PYH";
$resn_exch{"HAJ2 PYH"}="2HAJ PYH"; $resn_exch{"HAA1 PYH"}="1HAA PYH"; $resn_exch{"HAA2 PYH"}="2HAA PYH";
$resn_exch{"HAA3 PYH"}="3HAA PYH"; $resn_exch{"HM21 PYI"}="1HM2 PYI"; $resn_exch{"HM22 PYI"}="2HM2 PYI";
$resn_exch{"HM23 PYI"}="3HM2 PYI"; $resn_exch{"H7A1 PYI"}="1H7A PYI"; $resn_exch{"H7A2 PYI"}="2H7A PYI";
$resn_exch{"HM61 PYI"}="1HM6 PYI"; $resn_exch{"HM62 PYI"}="2HM6 PYI"; $resn_exch{"HM71 PYI"}="1HM7 PYI";
$resn_exch{"HM72 PYI"}="2HM7 PYI"; $resn_exch{"HM41 PYI"}="1HM4 PYI"; $resn_exch{"HM42 PYI"}="2HM4 PYI";
$resn_exch{"HM43 PYI"}="3HM4 PYI"; $resn_exch{"H4A1 PYI"}="1H4A PYI"; $resn_exch{"H4A2 PYI"}="2H4A PYI";
$resn_exch{"HCB1 PYL"}="1HCB PYL"; $resn_exch{"HCB2 PYL"}="2HCB PYL"; $resn_exch{"HCX1 PYL"}="1HCX PYL";
$resn_exch{"HCX2 PYL"}="2HCX PYL"; $resn_exch{"HCX3 PYL"}="3HCX PYL"; $resn_exch{"HCD1 PYL"}="1HCD PYL";
$resn_exch{"HCD2 PYL"}="2HCD PYL"; $resn_exch{"HCE1 PYL"}="1HCE PYL"; $resn_exch{"HCE2 PYL"}="2HCE PYL";
$resn_exch{"HCZ1 PYL"}="1HCZ PYL"; $resn_exch{"HN12 PYM"}="2HN1 PYM"; $resn_exch{"HN11 PYM"}="1HN1 PYM";
$resn_exch{" H33 PYM"}="3H3  PYM"; $resn_exch{" H32 PYM"}="2H3  PYM"; $resn_exch{" H31 PYM"}="1H3  PYM";
$resn_exch{" H21 PYN"}="1H2  PYN"; $resn_exch{" H51 PYN"}="1H5  PYN"; $resn_exch{" H71 PYN"}="1H7  PYN";
$resn_exch{" H81 PYN"}="1H8  PYN"; $resn_exch{" H91 PYN"}="1H9  PYN"; $resn_exch{"H101 PYN"}="1H10 PYN";
$resn_exch{"H121 PYN"}="1H12 PYN"; $resn_exch{" HN1 PYN"}="1HN  PYN"; $resn_exch{"H151 PYN"}="1H15 PYN";
$resn_exch{"H152 PYN"}="2H15 PYN"; $resn_exch{"H161 PYN"}="1H16 PYN"; $resn_exch{"H162 PYN"}="2H16 PYN";
$resn_exch{"HN11 PYN"}="1HN1 PYN"; $resn_exch{"H181 PYN"}="1H18 PYN"; $resn_exch{"H182 PYN"}="2H18 PYN";
$resn_exch{"H191 PYN"}="1H19 PYN"; $resn_exch{"H192 PYN"}="2H19 PYN"; $resn_exch{"H211 PYN"}="1H21 PYN";
$resn_exch{"H212 PYN"}="2H21 PYN"; $resn_exch{"H231 PYN"}="1H23 PYN"; $resn_exch{"H232 PYN"}="2H23 PYN";
$resn_exch{"H241 PYN"}="1H24 PYN"; $resn_exch{"H242 PYN"}="2H24 PYN"; $resn_exch{"H251 PYN"}="1H25 PYN";
$resn_exch{"H252 PYN"}="2H25 PYN"; $resn_exch{"H261 PYN"}="1H26 PYN"; $resn_exch{"H262 PYN"}="2H26 PYN";
$resn_exch{"H281 PYN"}="1H28 PYN"; $resn_exch{"H282 PYN"}="2H28 PYN"; $resn_exch{"H291 PYN"}="1H29 PYN";
$resn_exch{"H292 PYN"}="2H29 PYN"; $resn_exch{"H301 PYN"}="1H30 PYN"; $resn_exch{"H302 PYN"}="2H30 PYN";
$resn_exch{"HOP3 PYO"}="3HOP PYO"; $resn_exch{"HOP1 PYO"}="1HOP PYO"; $resn_exch{"H5'' PYO"}="2H5* PYO";
$resn_exch{" H5' PYO"}="1H5* PYO"; $resn_exch{" H4' PYO"}=" H4* PYO"; $resn_exch{" H3' PYO"}=" H3* PYO";
$resn_exch{" H1' PYO"}=" H1* PYO"; $resn_exch{" HO3 PYO"}="3HO  PYO"; $resn_exch{" H2' PYO"}=" H2* PYO";
$resn_exch{" HO2 PYO"}="2HO  PYO"; $resn_exch{"HOP2 PYP"}="2HOP PYP"; $resn_exch{"H5'1 PYP"}="1H5* PYP";
$resn_exch{"H5'2 PYP"}="2H5* PYP"; $resn_exch{" H4' PYP"}=" H4* PYP"; $resn_exch{" H1' PYP"}=" H1* PYP";
$resn_exch{"H2'2 PYP"}="2H2* PYP"; $resn_exch{"H2'1 PYP"}="1H2* PYP"; $resn_exch{" H3' PYP"}=" H3* PYP";
$resn_exch{"HOP3 PYP"}="3HOP PYP"; $resn_exch{" H11 PYQ"}="1H1  PYQ"; $resn_exch{" H61 PYQ"}="1H6  PYQ";
$resn_exch{" H62 PYQ"}="2H6  PYQ"; $resn_exch{" H71 PYQ"}="1H7  PYQ"; $resn_exch{" H72 PYQ"}="2H7  PYQ";
$resn_exch{"H111 PYQ"}="1H11 PYQ"; $resn_exch{"H112 PYQ"}="2H11 PYQ"; $resn_exch{"H121 PYQ"}="1H12 PYQ";
$resn_exch{"H122 PYQ"}="2H12 PYQ"; $resn_exch{"H131 PYQ"}="1H13 PYQ"; $resn_exch{"H141 PYQ"}="1H14 PYQ";
$resn_exch{" H31 PYR"}="1H3  PYR"; $resn_exch{" H32 PYR"}="2H3  PYR"; $resn_exch{" H33 PYR"}="3H3  PYR";
$resn_exch{" HN1 PYT"}="1HN  PYT"; $resn_exch{" HN2 PYT"}="2HN  PYT"; $resn_exch{" HC1 PYT"}="1HC  PYT";
$resn_exch{" HC2 PYT"}="2HC  PYT"; $resn_exch{" HT1 PYT"}="1HT  PYT"; $resn_exch{" HT2 PYT"}="2HT  PYT";
$resn_exch{" H   PYX"}="1HN  PYX"; $resn_exch{" H2  PYX"}="2HN  PYX"; $resn_exch{" HB2 PYX"}="1HB  PYX";
$resn_exch{" HB3 PYX"}="2HB  PYX"; $resn_exch{" HE2 PYX"}="1HE  PYX"; $resn_exch{" HE3 PYX"}="2HE  PYX";
$resn_exch{" HC2 PYX"}=" H2  PYX"; $resn_exch{"HM61 PYX"}="1HM6 PYX"; $resn_exch{"HM62 PYX"}="2HM6 PYX";
$resn_exch{"HN61 PYX"}="1HN6 PYX"; $resn_exch{"HN62 PYX"}="2HN6 PYX"; $resn_exch{"HM41 PYX"}="1HM4 PYX";
$resn_exch{"HM42 PYX"}="2HM4 PYX"; $resn_exch{"HM43 PYX"}="3HM4 PYX"; $resn_exch{"HOP2 PYY"}="2HOP PYY";
$resn_exch{"H5'1 PYY"}="1H5* PYY"; $resn_exch{"H5'2 PYY"}="2H5* PYY"; $resn_exch{" H4' PYY"}=" H4* PYY";
$resn_exch{" H1' PYY"}=" H1* PYY"; $resn_exch{" H2' PYY"}=" H2* PYY"; $resn_exch{"HO'2 PYY"}="2HO* PYY";
$resn_exch{" H3' PYY"}=" H3* PYY"; $resn_exch{"HOP3 PYY"}="3HOP PYY"; $resn_exch{"H1C1 PZ1"}="1H1C PZ1";
$resn_exch{"H1C2 PZ1"}="2H1C PZ1"; $resn_exch{"H1C3 PZ1"}="3H1C PZ1"; $resn_exch{"H4C1 PZ1"}="1H4C PZ1";
$resn_exch{"H4C2 PZ1"}="2H4C PZ1"; $resn_exch{"H211 PZ1"}="1H21 PZ1"; $resn_exch{"H212 PZ1"}="2H21 PZ1";
$resn_exch{"H221 PZ1"}="1H22 PZ1"; $resn_exch{"H222 PZ1"}="2H22 PZ1"; $resn_exch{"H241 PZ1"}="1H24 PZ1";
$resn_exch{"H242 PZ1"}="2H24 PZ1"; $resn_exch{"H311 PZ1"}="1H31 PZ1"; $resn_exch{"H312 PZ1"}="2H31 PZ1";
$resn_exch{"H321 PZ1"}="1H32 PZ1"; $resn_exch{"H322 PZ1"}="2H32 PZ1"; $resn_exch{"H331 PZ1"}="1H33 PZ1";
$resn_exch{"H332 PZ1"}="2H33 PZ1"; $resn_exch{"H341 PZ1"}="1H34 PZ1"; $resn_exch{"H342 PZ1"}="2H34 PZ1";
$resn_exch{"H112 PZ1"}="12H1 PZ1"; $resn_exch{"H181 PZ1"}="1H18 PZ1"; $resn_exch{"H182 PZ1"}="2H18 PZ1";
$resn_exch{"H191 PZ1"}="1H19 PZ1"; $resn_exch{"H192 PZ1"}="2H19 PZ1"; $resn_exch{"HN11 PZM"}="1HN1 PZM";
$resn_exch{"HN12 PZM"}="2HN1 PZM"; $resn_exch{" HB1 PZM"}="1HB  PZM"; $resn_exch{" HB2 PZM"}="2HB  PZM";
$resn_exch{"HF11 PZM"}="1HF1 PZM"; $resn_exch{"HF12 PZM"}="2HF1 PZM"; $resn_exch{"HF13 PZM"}="3HF1 PZM";
$resn_exch{" H11 PZN"}="1H1  PZN"; $resn_exch{" H12 PZN"}="2H1  PZN"; $resn_exch{" H13 PZN"}="3H1  PZN";
$resn_exch{" H21 PZN"}="1H2  PZN"; $resn_exch{" H22 PZN"}="2H2  PZN"; $resn_exch{" H31 PZN"}="1H3  PZN";
$resn_exch{" H32 PZN"}="2H3  PZN"; $resn_exch{" H41 PZN"}="1H4  PZN"; $resn_exch{" H42 PZN"}="2H4  PZN";
$resn_exch{" H51 PZN"}="1H5  PZN"; $resn_exch{" H52 PZN"}="2H5  PZN"; $resn_exch{" H61 PZN"}="1H6  PZN";
$resn_exch{" H62 PZN"}="2H6  PZN"; $resn_exch{" H71 PZN"}="1H7  PZN"; $resn_exch{" H72 PZN"}="2H7  PZN";
$resn_exch{" H73 PZN"}="3H7  PZN"; $resn_exch{" H81 PZN"}="1H8  PZN"; $resn_exch{" H82 PZN"}="2H8  PZN";
$resn_exch{"H111 PZN"}="1H11 PZN"; $resn_exch{"H112 PZN"}="2H11 PZN"; $resn_exch{"H151 PZN"}="1H15 PZN";
$resn_exch{"H152 PZN"}="2H15 PZN"; $resn_exch{"H161 PZN"}="1H16 PZN"; $resn_exch{"H162 PZN"}="2H16 PZN";
$resn_exch{"H163 PZN"}="3H16 PZN"; $resn_exch{"H231 PZN"}="1H23 PZN"; $resn_exch{"H232 PZN"}="2H23 PZN";
$resn_exch{"H233 PZN"}="3H23 PZN"; $resn_exch{"H211 PZN"}="1H21 PZN"; $resn_exch{"H212 PZN"}="2H21 PZN";
$resn_exch{"H213 PZN"}="3H21 PZN"; $resn_exch{"H221 PZN"}="1H22 PZN"; $resn_exch{"H222 PZN"}="2H22 PZN";
$resn_exch{"H223 PZN"}="3H22 PZN"; $resn_exch{"H251 PZN"}="1H25 PZN"; $resn_exch{"H252 PZN"}="2H25 PZN";
$resn_exch{"H281 PZN"}="1H28 PZN"; $resn_exch{"H282 PZN"}="2H28 PZN"; $resn_exch{"H283 PZN"}="3H28 PZN";
$resn_exch{" H21 PZQ"}="1H2  PZQ"; $resn_exch{" H22 PZQ"}="2H2  PZQ"; $resn_exch{" H31 PZQ"}="1H3  PZQ";
$resn_exch{" H32 PZQ"}="2H3  PZQ"; $resn_exch{" H51 PZQ"}="1H5  PZQ"; $resn_exch{" H52 PZQ"}="2H5  PZQ";
$resn_exch{" H61 PZQ"}="1H6  PZQ"; $resn_exch{" H62 PZQ"}="2H6  PZQ"; $resn_exch{" H91 PZQ"}="1H9  PZQ";
$resn_exch{" H92 PZQ"}="2H9  PZQ"; $resn_exch{"H101 PZQ"}="1H10 PZQ"; $resn_exch{"H102 PZQ"}="2H10 PZQ";
$resn_exch{"H131 PZQ"}="1H13 PZQ"; $resn_exch{"H132 PZQ"}="2H13 PZQ"; $resn_exch{"H151 PZQ"}="1H15 PZQ";
$resn_exch{"H152 PZQ"}="2H15 PZQ"; $resn_exch{"H161 PZQ"}="1H16 PZQ"; $resn_exch{"H162 PZQ"}="2H16 PZQ";
$resn_exch{"H101 PZZ"}="1H10 PZZ"; $resn_exch{"H102 PZZ"}="2H10 PZZ"; $resn_exch{"H121 PZZ"}="1H12 PZZ";
$resn_exch{"H122 PZZ"}="2H12 PZZ"; $resn_exch{"H123 PZZ"}="3H12 PZZ"; $resn_exch{"H131 PZZ"}="1H13 PZZ";
$resn_exch{"H132 PZZ"}="2H13 PZZ"; $resn_exch{"H133 PZZ"}="3H13 PZZ"; $resn_exch{"H141 PZZ"}="1H14 PZZ";
$resn_exch{"H142 PZZ"}="2H14 PZZ"; $resn_exch{"H151 PZZ"}="1H15 PZZ"; $resn_exch{"H152 PZZ"}="2H15 PZZ";
$resn_exch{"H161 PZZ"}="1H16 PZZ"; $resn_exch{"H162 PZZ"}="2H16 PZZ"; $resn_exch{" H71 Q2Y"}="1H7  Q2Y";
$resn_exch{" H72 Q2Y"}="2H7  Q2Y"; $resn_exch{" H91 Q2Y"}="1H9  Q2Y"; $resn_exch{" H92 Q2Y"}="2H9  Q2Y";
$resn_exch{"H171 Q2Y"}="1H17 Q2Y"; $resn_exch{"H172 Q2Y"}="2H17 Q2Y"; $resn_exch{"H11A Q2Y"}="1H1  Q2Y";
$resn_exch{" H12 Q2Y"}="2H1  Q2Y"; $resn_exch{" H13 Q2Y"}="3H1  Q2Y"; $resn_exch{"H501 Q50"}="1H50 Q50";
$resn_exch{"H502 Q50"}="2H50 Q50"; $resn_exch{"H411 Q50"}="1H41 Q50"; $resn_exch{"H412 Q50"}="2H41 Q50";
$resn_exch{"H321 Q50"}="1H32 Q50"; $resn_exch{"H322 Q50"}="2H32 Q50"; $resn_exch{"H331 Q50"}="1H33 Q50";
$resn_exch{"H332 Q50"}="2H33 Q50"; $resn_exch{"H361 Q50"}="1H36 Q50"; $resn_exch{"H362 Q50"}="2H36 Q50";
$resn_exch{"H211 Q50"}="1H21 Q50"; $resn_exch{"H212 Q50"}="2H21 Q50"; $resn_exch{"H191 Q50"}="1H19 Q50";
$resn_exch{"H192 Q50"}="2H19 Q50"; $resn_exch{" H41 Q50"}="1H4  Q50"; $resn_exch{" H42 Q50"}="2H4  Q50";
$resn_exch{"H43A Q50"}="3H4  Q50"; $resn_exch{"H31A Q50"}="1H3  Q50"; $resn_exch{" H32 Q50"}="2H3  Q50";
$resn_exch{" H33 Q50"}="3H3  Q50"; $resn_exch{" H21 Q50"}="1H2  Q50"; $resn_exch{" H22 Q50"}="2H2  Q50";
$resn_exch{"H23A Q50"}="3H2  Q50"; $resn_exch{"H101 Q50"}="1H10 Q50"; $resn_exch{"H102 Q50"}="2H10 Q50";
$resn_exch{"H41A Q72"}="1H4  Q72"; $resn_exch{" H42 Q72"}="2H4  Q72"; $resn_exch{" H43 Q72"}="3H4  Q72";
$resn_exch{"H151 Q72"}="1H15 Q72"; $resn_exch{"H152 Q72"}="2H15 Q72"; $resn_exch{"H153 Q72"}="3H15 Q72";
$resn_exch{"H141 Q72"}="1H14 Q72"; $resn_exch{"H142 Q72"}="2H14 Q72"; $resn_exch{"H143 Q72"}="3H14 Q72";
$resn_exch{"H231 Q72"}="1H23 Q72"; $resn_exch{"H232 Q72"}="2H23 Q72"; $resn_exch{"H321 Q72"}="1H32 Q72";
$resn_exch{"H322 Q72"}="2H32 Q72"; $resn_exch{"H381 Q72"}="1H38 Q72"; $resn_exch{"H382 Q72"}="2H38 Q72";
$resn_exch{"H371 Q72"}="1H37 Q72"; $resn_exch{"H372 Q72"}="2H37 Q72"; $resn_exch{"H461 Q72"}="1H46 Q72";
$resn_exch{"H462 Q72"}="2H46 Q72"; $resn_exch{"H451 Q72"}="1H45 Q72"; $resn_exch{"H452 Q72"}="2H45 Q72";
$resn_exch{"H491 Q72"}="1H49 Q72"; $resn_exch{"H492 Q72"}="2H49 Q72"; $resn_exch{"H431 Q72"}="1H43 Q72";
$resn_exch{"H432 Q72"}="2H43 Q72"; $resn_exch{"H421 Q72"}="1H42 Q72"; $resn_exch{"H422 Q72"}="2H42 Q72";
$resn_exch{"H351 Q72"}="1H35 Q72"; $resn_exch{"H352 Q72"}="2H35 Q72"; $resn_exch{"H341 Q72"}="1H34 Q72";
$resn_exch{"H342 Q72"}="2H34 Q72"; $resn_exch{"H121 Q72"}="1H12 Q72"; $resn_exch{"H122 Q72"}="2H12 Q72";
$resn_exch{"H123 Q72"}="3H12 Q72"; $resn_exch{"H111 Q72"}="1H11 Q72"; $resn_exch{"H112 Q72"}="2H11 Q72";
$resn_exch{"H113 Q72"}="3H11 Q72"; $resn_exch{" H11 Q72"}="1H1  Q72"; $resn_exch{" H12 Q72"}="2H1  Q72";
$resn_exch{"H13A Q72"}="3H1  Q72"; $resn_exch{"H21A Q82"}="1H2  Q82"; $resn_exch{"H22A Q82"}="2H2  Q82";
$resn_exch{"H71A Q82"}="1H7  Q82"; $resn_exch{"H72A Q82"}="2H7  Q82"; $resn_exch{"H311 Q82"}="1H31 Q82";
$resn_exch{"H312 Q82"}="2H31 Q82"; $resn_exch{"H611 Q82"}="1H61 Q82"; $resn_exch{"H612 Q82"}="2H61 Q82";
$resn_exch{"H761 Q82"}="1H76 Q82"; $resn_exch{"H762 Q82"}="2H76 Q82"; $resn_exch{"H261 Q82"}="1H26 Q82";
$resn_exch{"H262 Q82"}="2H26 Q82"; $resn_exch{"H151 QFI"}="1H15 QFI"; $resn_exch{"H152 QFI"}="2H15 QFI";
$resn_exch{"H161 QFI"}="1H16 QFI"; $resn_exch{"H162 QFI"}="2H16 QFI"; $resn_exch{"H241 QFI"}="1H24 QFI";
$resn_exch{"H242 QFI"}="2H24 QFI"; $resn_exch{"H243 QFI"}="3H24 QFI"; $resn_exch{"H251 QFI"}="1H25 QFI";
$resn_exch{"H252 QFI"}="2H25 QFI"; $resn_exch{"H253 QFI"}="3H25 QFI"; $resn_exch{"H261 QFI"}="1H26 QFI";
$resn_exch{"H262 QFI"}="2H26 QFI"; $resn_exch{"H263 QFI"}="3H26 QFI"; $resn_exch{" H81 QFI"}="1H8  QFI";
$resn_exch{" H82 QFI"}="2H8  QFI"; $resn_exch{"H411 QFI"}="1H41 QFI"; $resn_exch{"H412 QFI"}="2H41 QFI";
$resn_exch{" H31 QFI"}="1H3  QFI"; $resn_exch{" H32 QFI"}="2H3  QFI"; $resn_exch{" H21 QFI"}="1H2  QFI";
$resn_exch{"H22A QFI"}="2H2  QFI"; $resn_exch{"H11A QFI"}="1H1  QFI"; $resn_exch{"H12A QFI"}="2H1  QFI";
$resn_exch{"H9C1 QGA"}="1H9C QGA"; $resn_exch{"H9C2 QGA"}="2H9C QGA"; $resn_exch{"H121 QGA"}="1H12 QGA";
$resn_exch{"H122 QGA"}="2H12 QGA"; $resn_exch{"H1C1 QGA"}="1H1C QGA"; $resn_exch{"H1C2 QGA"}="2H1C QGA";
$resn_exch{"H151 QGA"}="1H15 QGA"; $resn_exch{"H152 QGA"}="2H15 QGA"; $resn_exch{"H181 QGA"}="1H18 QGA";
$resn_exch{"H182 QGA"}="2H18 QGA"; $resn_exch{"H241 QGA"}="1H24 QGA"; $resn_exch{"H242 QGA"}="2H24 QGA";
$resn_exch{"H271 QGA"}="1H27 QGA"; $resn_exch{"H272 QGA"}="2H27 QGA"; $resn_exch{"H351 QGA"}="1H35 QGA";
$resn_exch{"H352 QGA"}="2H35 QGA"; $resn_exch{"H361 QGA"}="1H36 QGA"; $resn_exch{"H362 QGA"}="2H36 QGA";
$resn_exch{"H601 QGA"}="1H60 QGA"; $resn_exch{"H602 QGA"}="2H60 QGA"; $resn_exch{"H431 QGA"}="1H43 QGA";
$resn_exch{"H432 QGA"}="2H43 QGA"; $resn_exch{"H1N1 QGG"}="1H1N QGG"; $resn_exch{"H1N2 QGG"}="2H1N QGG";
$resn_exch{"H271 QGG"}="1H27 QGG"; $resn_exch{"H272 QGG"}="2H27 QGG"; $resn_exch{"H191 QGG"}="1H19 QGG";
$resn_exch{"H192 QGG"}="2H19 QGG"; $resn_exch{"H201 QGG"}="1H20 QGG"; $resn_exch{"H202 QGG"}="2H20 QGG";
$resn_exch{"H211 QGG"}="1H21 QGG"; $resn_exch{"H212 QGG"}="2H21 QGG"; $resn_exch{"H291 QIN"}="1H29 QIN";
$resn_exch{"H292 QIN"}="2H29 QIN"; $resn_exch{"H293 QIN"}="3H29 QIN"; $resn_exch{"H561 QIN"}="1H56 QIN";
$resn_exch{"H562 QIN"}="2H56 QIN"; $resn_exch{"H221 QIN"}="1H22 QIN"; $resn_exch{"H222 QIN"}="2H22 QIN";
$resn_exch{"H223 QIN"}="3H22 QIN"; $resn_exch{" H81 QIN"}="1H8  QIN"; $resn_exch{" H82 QIN"}="2H8  QIN";
$resn_exch{"HN11 QIN"}="1HN1 QIN"; $resn_exch{"HN12 QIN"}="2HN1 QIN"; $resn_exch{"H101 QIN"}="1H10 QIN";
$resn_exch{"H102 QIN"}="2H10 QIN"; $resn_exch{"H111 QIN"}="1H11 QIN"; $resn_exch{"H112 QIN"}="2H11 QIN";
$resn_exch{"H131 QIN"}="1H13 QIN"; $resn_exch{"H132 QIN"}="2H13 QIN"; $resn_exch{"H133 QIN"}="3H13 QIN";
$resn_exch{" H51 QIN"}="1H5  QIN"; $resn_exch{" H52 QIN"}="2H5  QIN"; $resn_exch{" H11 QIN"}="1H1  QIN";
$resn_exch{" H12 QIN"}="2H1  QIN"; $resn_exch{" H41 QIN"}="1H4  QIN"; $resn_exch{" H42 QIN"}="2H4  QIN";
$resn_exch{" H43 QIN"}="3H4  QIN"; $resn_exch{"H671 QKH"}="1H67 QKH"; $resn_exch{"H672 QKH"}="2H67 QKH";
$resn_exch{"H811 QKH"}="1H81 QKH"; $resn_exch{"H812 QKH"}="2H81 QKH"; $resn_exch{"H813 QKH"}="3H81 QKH";
$resn_exch{"H821 QKH"}="1H82 QKH"; $resn_exch{"H822 QKH"}="2H82 QKH"; $resn_exch{"H823 QKH"}="3H82 QKH";
$resn_exch{"H831 QKH"}="1H83 QKH"; $resn_exch{"H832 QKH"}="2H83 QKH"; $resn_exch{"H833 QKH"}="3H83 QKH";
$resn_exch{"H661 QKH"}="1H66 QKH"; $resn_exch{"H662 QKH"}="2H66 QKH"; $resn_exch{"H791 QKH"}="1H79 QKH";
$resn_exch{"H792 QKH"}="2H79 QKH"; $resn_exch{"H793 QKH"}="3H79 QKH"; $resn_exch{"H651 QKH"}="1H65 QKH";
$resn_exch{"H652 QKH"}="2H65 QKH"; $resn_exch{"H781 QKH"}="1H78 QKH"; $resn_exch{"H782 QKH"}="2H78 QKH";
$resn_exch{"H783 QKH"}="3H78 QKH"; $resn_exch{"H801 QKH"}="1H80 QKH"; $resn_exch{"H802 QKH"}="2H80 QKH";
$resn_exch{"H803 QKH"}="3H80 QKH"; $resn_exch{"H641 QKH"}="1H64 QKH"; $resn_exch{"H642 QKH"}="2H64 QKH";
$resn_exch{"H631 QKH"}="1H63 QKH"; $resn_exch{"H632 QKH"}="2H63 QKH"; $resn_exch{"H621 QKH"}="1H62 QKH";
$resn_exch{"H622 QKH"}="2H62 QKH"; $resn_exch{"H611 QKH"}="1H61 QKH"; $resn_exch{"H612 QKH"}="2H61 QKH";
$resn_exch{"H841 QKH"}="1H84 QKH"; $resn_exch{"H842 QKH"}="2H84 QKH"; $resn_exch{"H843 QKH"}="3H84 QKH";
$resn_exch{"HD11 QLG"}="1HD1 QLG"; $resn_exch{"HD12 QLG"}="2HD1 QLG"; $resn_exch{"HD13 QLG"}="3HD1 QLG";
$resn_exch{"HD21 QLG"}="1HD2 QLG"; $resn_exch{"HD22 QLG"}="2HD2 QLG"; $resn_exch{"HD23 QLG"}="3HD2 QLG";
$resn_exch{"HB11 QLG"}="1HB1 QLG"; $resn_exch{"HB12 QLG"}="2HB1 QLG"; $resn_exch{"HG11 QLG"}="1HG1 QLG";
$resn_exch{"HG12 QLG"}="2HG1 QLG"; $resn_exch{"HE11 QLG"}="1HE1 QLG"; $resn_exch{"HE12 QLG"}="2HE1 QLG";
$resn_exch{"HA31 QLG"}="1HA3 QLG"; $resn_exch{"HA32 QLG"}="2HA3 QLG"; $resn_exch{" HB1 QLP"}="1HB  QLP";
$resn_exch{" HB2 QLP"}="2HB  QLP"; $resn_exch{" HG1 QLP"}="1HG  QLP"; $resn_exch{" HG2 QLP"}="2HG  QLP";
$resn_exch{"HE21 QLP"}="1HE2 QLP"; $resn_exch{"HE22 QLP"}="2HE2 QLP"; $resn_exch{"H4A1 QLP"}="1H4A QLP";
$resn_exch{"H4A2 QLP"}="2H4A QLP"; $resn_exch{"H5A1 QLP"}="1H5A QLP"; $resn_exch{"H5A2 QLP"}="2H5A QLP";
$resn_exch{"H2A1 QLP"}="1H2A QLP"; $resn_exch{"H2A2 QLP"}="2H2A QLP"; $resn_exch{"H2A3 QLP"}="3H2A QLP";
$resn_exch{" H71 QMS"}="1H7  QMS"; $resn_exch{" H72 QMS"}="2H7  QMS"; $resn_exch{" H73 QMS"}="3H7  QMS";
$resn_exch{"HN11 QN1"}="1HN1 QN1"; $resn_exch{"HN12 QN1"}="2HN1 QN1"; $resn_exch{"H331 QN1"}="1H33 QN1";
$resn_exch{"H332 QN1"}="2H33 QN1"; $resn_exch{"H251 QN1"}="1H25 QN1"; $resn_exch{"H252 QN1"}="2H25 QN1";
$resn_exch{"H211 QN1"}="1H21 QN1"; $resn_exch{"H212 QN1"}="2H21 QN1"; $resn_exch{" H71 QN1"}="1H7  QN1";
$resn_exch{" H72 QN1"}="2H7  QN1"; $resn_exch{"HN21 QN1"}="1HN2 QN1"; $resn_exch{"HN2A QN1"}="2HN2 QN1";
$resn_exch{" H71 QN2"}="1H7  QN2"; $resn_exch{" H72 QN2"}="2H7  QN2"; $resn_exch{"H331 QN2"}="1H33 QN2";
$resn_exch{"H332 QN2"}="2H33 QN2"; $resn_exch{"H251 QN2"}="1H25 QN2"; $resn_exch{"H252 QN2"}="2H25 QN2";
$resn_exch{"H211 QN2"}="1H21 QN2"; $resn_exch{"H212 QN2"}="2H21 QN2"; $resn_exch{"H291 QNO"}="1H29 QNO";
$resn_exch{"H292 QNO"}="2H29 QNO"; $resn_exch{"H293 QNO"}="3H29 QNO"; $resn_exch{"H281 QNO"}="1H28 QNO";
$resn_exch{"H282 QNO"}="2H28 QNO"; $resn_exch{"H271 QNO"}="1H27 QNO"; $resn_exch{"H272 QNO"}="2H27 QNO";
$resn_exch{"H261 QNO"}="1H26 QNO"; $resn_exch{"H262 QNO"}="2H26 QNO"; $resn_exch{"H251 QNO"}="1H25 QNO";
$resn_exch{"H252 QNO"}="2H25 QNO"; $resn_exch{"H241 QNO"}="1H24 QNO"; $resn_exch{"H242 QNO"}="2H24 QNO";
$resn_exch{"H231 QNO"}="1H23 QNO"; $resn_exch{"H232 QNO"}="2H23 QNO"; $resn_exch{"H221 QNO"}="1H22 QNO";
$resn_exch{"H222 QNO"}="2H22 QNO"; $resn_exch{"H211 QNO"}="1H21 QNO"; $resn_exch{"H212 QNO"}="2H21 QNO";
$resn_exch{"HAW1 QPP"}="1HAW QPP"; $resn_exch{"HAW2 QPP"}="2HAW QPP"; $resn_exch{"HAW3 QPP"}="3HAW QPP";
$resn_exch{"H161 QQ2"}="1H16 QQ2"; $resn_exch{"H162 QQ2"}="2H16 QQ2"; $resn_exch{"H271 QQ2"}="1H27 QQ2";
$resn_exch{"H272 QQ2"}="2H27 QQ2"; $resn_exch{"H291 QQ2"}="1H29 QQ2"; $resn_exch{"H292 QQ2"}="2H29 QQ2";
$resn_exch{"H311 QQ2"}="1H31 QQ2"; $resn_exch{"H312 QQ2"}="2H31 QQ2"; $resn_exch{"H313 QQ2"}="3H31 QQ2";
$resn_exch{"H321 QQ2"}="1H32 QQ2"; $resn_exch{"H322 QQ2"}="2H32 QQ2"; $resn_exch{"H323 QQ2"}="3H32 QQ2";
$resn_exch{"H331 QQ2"}="1H33 QQ2"; $resn_exch{"H332 QQ2"}="2H33 QQ2"; $resn_exch{"H333 QQ2"}="3H33 QQ2";
$resn_exch{" HN1 QSI"}="1HN  QSI"; $resn_exch{" HN2 QSI"}="2HN  QSI"; $resn_exch{" HB1 QSI"}="1HB  QSI";
$resn_exch{" HB2 QSI"}="2HB  QSI"; $resn_exch{" HG1 QSI"}="1HG  QSI"; $resn_exch{" HG2 QSI"}="2HG  QSI";
$resn_exch{"HN21 QSI"}="1HN2 QSI"; $resn_exch{"HN22 QSI"}="2HN2 QSI"; $resn_exch{"H5'1 QSI"}="1H5' QSI";
$resn_exch{"H5'2 QSI"}="2H5' QSI"; $resn_exch{"HN61 QSI"}="1HN6 QSI"; $resn_exch{"HN62 QSI"}="2HN6 QSI";
$resn_exch{"HC71 QUA"}="1HC7 QUA"; $resn_exch{"HC72 QUA"}="2HC7 QUA"; $resn_exch{"H141 QUA"}="1H14 QUA";
$resn_exch{"H142 QUA"}="2H14 QUA"; $resn_exch{"H143 QUA"}="3H14 QUA"; $resn_exch{"H201 QUM"}="1H20 QUM";
$resn_exch{"H202 QUM"}="2H20 QUM"; $resn_exch{"H203 QUM"}="3H20 QUM"; $resn_exch{"H211 QUM"}="1H21 QUM";
$resn_exch{"H212 QUM"}="2H21 QUM"; $resn_exch{"H221 QUM"}="1H22 QUM"; $resn_exch{"H222 QUM"}="2H22 QUM";
$resn_exch{"H231 QUM"}="1H23 QUM"; $resn_exch{"H232 QUM"}="2H23 QUM"; $resn_exch{"H271 QUM"}="1H27 QUM";
$resn_exch{"H272 QUM"}="2H27 QUM"; $resn_exch{"H251 QUM"}="1H25 QUM"; $resn_exch{"H252 QUM"}="2H25 QUM";
$resn_exch{"H281 QUM"}="1H28 QUM"; $resn_exch{"H282 QUM"}="2H28 QUM"; $resn_exch{"H261 QUM"}="1H26 QUM";
$resn_exch{"H262 QUM"}="2H26 QUM"; $resn_exch{"H171 QUM"}="1H17 QUM"; $resn_exch{"H172 QUM"}="2H17 QUM";
$resn_exch{"H173 QUM"}="3H17 QUM"; $resn_exch{" H21 QUN"}="1H2  QUN"; $resn_exch{" H31 QUN"}="1H3  QUN";
$resn_exch{" H61 QUN"}="1H6  QUN"; $resn_exch{"H101 QUN"}="1H10 QUN"; $resn_exch{"H121 QUN"}="1H12 QUN";
$resn_exch{"H131 QUN"}="1H13 QUN"; $resn_exch{"H141 QUN"}="1H14 QUN"; $resn_exch{"H142 QUN"}="2H14 QUN";
$resn_exch{"H143 QUN"}="3H14 QUN"; $resn_exch{"HN21 QUN"}="1HN2 QUN"; $resn_exch{"H151 QUN"}="1H15 QUN";
$resn_exch{"H161 QUN"}="1H16 QUN"; $resn_exch{"H162 QUN"}="2H16 QUN"; $resn_exch{"H163 QUN"}="3H16 QUN";
$resn_exch{"H171 QUN"}="1H17 QUN"; $resn_exch{"H172 QUN"}="2H17 QUN"; $resn_exch{"H181 QUN"}="1H18 QUN";
$resn_exch{"H182 QUN"}="2H18 QUN"; $resn_exch{"H191 QUN"}="1H19 QUN"; $resn_exch{"H192 QUN"}="2H19 QUN";
$resn_exch{"H201 QUN"}="1H20 QUN"; $resn_exch{"H202 QUN"}="2H20 QUN"; $resn_exch{"H211 QUN"}="1H21 QUN";
$resn_exch{"H212 QUN"}="2H21 QUN"; $resn_exch{"H221 QUN"}="1H22 QUN"; $resn_exch{"H222 QUN"}="2H22 QUN";
$resn_exch{"H223 QUN"}="3H22 QUN"; $resn_exch{"H231 QUN"}="1H23 QUN"; $resn_exch{"H232 QUN"}="2H23 QUN";
$resn_exch{"H233 QUN"}="3H23 QUN"; $resn_exch{"HOP2 QUO"}="2HOP QUO"; $resn_exch{"HOP3 QUO"}="3HOP QUO";
$resn_exch{" H5' QUO"}="1H5* QUO"; $resn_exch{"H5'' QUO"}="2H5* QUO"; $resn_exch{" H4' QUO"}=" H4* QUO";
$resn_exch{" H1' QUO"}=" H1* QUO"; $resn_exch{"HN21 QUO"}="1HN2 QUO"; $resn_exch{"HN22 QUO"}="2HN2 QUO";
$resn_exch{"H101 QUO"}="1H10 QUO"; $resn_exch{"H102 QUO"}="2H10 QUO"; $resn_exch{"H151 QUO"}="1H15 QUO";
$resn_exch{"H152 QUO"}="2H15 QUO"; $resn_exch{"H161 QUO"}="1H16 QUO"; $resn_exch{"H162 QUO"}="2H16 QUO";
$resn_exch{" H2' QUO"}=" H2* QUO"; $resn_exch{"HO2' QUO"}="HO2* QUO"; $resn_exch{" H3' QUO"}=" H3* QUO";
$resn_exch{"HO3' QUO"}="HO3* QUO"; $resn_exch{"H031 QUS"}="1H03 QUS"; $resn_exch{"H032 QUS"}="2H03 QUS";
$resn_exch{"HP31 QUS"}="1HP3 QUS"; $resn_exch{"HP32 QUS"}="2HP3 QUS"; $resn_exch{"H6Q1 QV4"}="1H6Q QV4";
$resn_exch{"H6Q2 QV4"}="2H6Q QV4"; $resn_exch{"H6P1 QV4"}="1H6P QV4"; $resn_exch{"H6P2 QV4"}="2H6P QV4";
$resn_exch{"H6B1 QV4"}="1H6B QV4"; $resn_exch{"H6B2 QV4"}="2H6B QV4"; $resn_exch{"H6S1 QV4"}="1H6S QV4";
$resn_exch{"H6S2 QV4"}="2H6S QV4"; $resn_exch{"H6S3 QV4"}="3H6S QV4"; $resn_exch{"H7T1 QV4"}="1H7T QV4";
$resn_exch{"H7T2 QV4"}="2H7T QV4"; $resn_exch{"HOP2   R"}="2HOP   R"; $resn_exch{"HOP3   R"}="3HOP   R";
$resn_exch{" H5'   R"}="1H5*   R"; $resn_exch{"H5''   R"}="2H5*   R"; $resn_exch{" H4'   R"}=" H4*   R";
$resn_exch{" H3'   R"}=" H3*   R"; $resn_exch{"HO3'   R"}="*HO3   R"; $resn_exch{" H2'   R"}="1H2*   R";
$resn_exch{"H2''   R"}="2H2*   R"; $resn_exch{" H1'   R"}=" H1*   R"; $resn_exch{" HB1   R"}="1HB    R";
$resn_exch{" HB2   R"}="2HB    R"; $resn_exch{"H201 R01"}="1H20 R01"; $resn_exch{"H202 R01"}="2H20 R01";
$resn_exch{"H271 R01"}="1H27 R01"; $resn_exch{"H272 R01"}="2H27 R01"; $resn_exch{"H273 R01"}="3H27 R01";
$resn_exch{"H251 R01"}="1H25 R01"; $resn_exch{"H252 R01"}="2H25 R01"; $resn_exch{"H291 R01"}="1H29 R01";
$resn_exch{"H292 R01"}="2H29 R01"; $resn_exch{"H151 R02"}="1H15 R02"; $resn_exch{"H152 R02"}="2H15 R02";
$resn_exch{"H171 R02"}="1H17 R02"; $resn_exch{"H172 R02"}="2H17 R02"; $resn_exch{"H191 R02"}="1H19 R02";
$resn_exch{"H192 R02"}="2H19 R02"; $resn_exch{"H271 R02"}="1H27 R02"; $resn_exch{"H272 R02"}="2H27 R02";
$resn_exch{"H251 R02"}="1H25 R02"; $resn_exch{"H252 R02"}="2H25 R02"; $resn_exch{"H291 R02"}="1H29 R02";
$resn_exch{"H292 R02"}="2H29 R02"; $resn_exch{"H3F1 R02"}="1H3F R02"; $resn_exch{"H3F2 R02"}="2H3F R02";
$resn_exch{"H3A1 R03"}="1H3A R03"; $resn_exch{"H3A2 R03"}="2H3A R03"; $resn_exch{"H1A1 R03"}="1H1A R03";
$resn_exch{"H1A2 R03"}="2H1A R03"; $resn_exch{"H1B1 R03"}="1H1B R03"; $resn_exch{"H1B2 R03"}="2H1B R03";
$resn_exch{"H2B1 R03"}="1H2B R03"; $resn_exch{"H2B2 R03"}="2H2B R03"; $resn_exch{"H3B1 R03"}="1H3B R03";
$resn_exch{"H3B2 R03"}="2H3B R03"; $resn_exch{"H4B1 R03"}="1H4B R03"; $resn_exch{"H4B2 R03"}="2H4B R03";
$resn_exch{"H5B1 R03"}="1H5B R03"; $resn_exch{"H5B2 R03"}="2H5B R03"; $resn_exch{"H6B1 R03"}="1H6B R03";
$resn_exch{"H6B2 R03"}="2H6B R03"; $resn_exch{"H1E1 R03"}="1H1E R03"; $resn_exch{"H1E2 R03"}="2H1E R03";
$resn_exch{"H1E3 R03"}="3H1E R03"; $resn_exch{"H1F1 R03"}="1H1F R03"; $resn_exch{"H1F2 R03"}="2H1F R03";
$resn_exch{"H1F3 R03"}="3H1F R03"; $resn_exch{"H3A1 R04"}="1H3A R04"; $resn_exch{"H3A2 R04"}="2H3A R04";
$resn_exch{"H1A1 R04"}="1H1A R04"; $resn_exch{"H1A2 R04"}="2H1A R04"; $resn_exch{"H1B1 R04"}="1H1B R04";
$resn_exch{"H1B2 R04"}="2H1B R04"; $resn_exch{"H4B1 R04"}="1H4B R04"; $resn_exch{"H4B2 R04"}="2H4B R04";
$resn_exch{"H1E1 R04"}="1H1E R04"; $resn_exch{"H1E2 R04"}="2H1E R04"; $resn_exch{"H1E3 R04"}="3H1E R04";
$resn_exch{"H1F1 R04"}="1H1F R04"; $resn_exch{"H1F2 R04"}="2H1F R04"; $resn_exch{"H1F3 R04"}="3H1F R04";
$resn_exch{"HN21 R11"}="1HN2 R11"; $resn_exch{"HN22 R11"}="2HN2 R11"; $resn_exch{"H10C R11"}="CH10 R11";
$resn_exch{"H11C R11"}="CH11 R11"; $resn_exch{"H13C R11"}="CH13 R11"; $resn_exch{"H141 R11"}="1H14 R11";
$resn_exch{"H142 R11"}="2H14 R11"; $resn_exch{"H143 R11"}="3H14 R11"; $resn_exch{"HC91 R11"}="1HC9 R11";
$resn_exch{"HC92 R11"}="2HC9 R11"; $resn_exch{"H19C R11"}="CH19 R11"; $resn_exch{"H20C R11"}="CH20 R11";
$resn_exch{"H21C R11"}="CH21 R11"; $resn_exch{"H22C R11"}="CH22 R11"; $resn_exch{"H231 R11"}="1H23 R11";
$resn_exch{"H232 R11"}="2H23 R11"; $resn_exch{"H233 R11"}="3H23 R11"; $resn_exch{"H141 R12"}="1H14 R12";
$resn_exch{"H142 R12"}="2H14 R12"; $resn_exch{"H181 R12"}="1H18 R12"; $resn_exch{"H182 R12"}="2H18 R12";
$resn_exch{"H183 R12"}="3H18 R12"; $resn_exch{"H191 R12"}="1H19 R12"; $resn_exch{"H192 R12"}="2H19 R12";
$resn_exch{"H193 R12"}="3H19 R12"; $resn_exch{"H201 R12"}="1H20 R12"; $resn_exch{"H202 R12"}="2H20 R12";
$resn_exch{"H203 R12"}="3H20 R12"; $resn_exch{"H231 R12"}="1H23 R12"; $resn_exch{"H232 R12"}="2H23 R12";
$resn_exch{"H233 R12"}="3H23 R12"; $resn_exch{" H21 R13"}="1H2  R13"; $resn_exch{" H22 R13"}="2H2  R13";
$resn_exch{" H31 R13"}="1H3  R13"; $resn_exch{" H32 R13"}="2H3  R13"; $resn_exch{"H181 R13"}="1H18 R13";
$resn_exch{"H182 R13"}="2H18 R13"; $resn_exch{"H183 R13"}="3H18 R13"; $resn_exch{"H191 R13"}="1H19 R13";
$resn_exch{"H192 R13"}="2H19 R13"; $resn_exch{"H193 R13"}="3H19 R13"; $resn_exch{"H201 R13"}="1H20 R13";
$resn_exch{"H202 R13"}="2H20 R13"; $resn_exch{"H203 R13"}="3H20 R13"; $resn_exch{"H211 R13"}="1H21 R13";
$resn_exch{"H212 R13"}="2H21 R13"; $resn_exch{"H213 R13"}="3H21 R13"; $resn_exch{"H221 R13"}="1H22 R13";
$resn_exch{"H222 R13"}="2H22 R13"; $resn_exch{"H223 R13"}="3H22 R13"; $resn_exch{"H231 R13"}="1H23 R13";
$resn_exch{"H232 R13"}="2H23 R13"; $resn_exch{"H233 R13"}="3H23 R13"; $resn_exch{"H271 R16"}="1H27 R16";
$resn_exch{"H272 R16"}="2H27 R16"; $resn_exch{"H273 R16"}="3H27 R16"; $resn_exch{"H281 R16"}="1H28 R16";
$resn_exch{"H282 R16"}="2H28 R16"; $resn_exch{"H291 R16"}="1H29 R16"; $resn_exch{"H292 R16"}="2H29 R16";
$resn_exch{"H301 R16"}="1H30 R16"; $resn_exch{"H302 R16"}="2H30 R16"; $resn_exch{"H311 R16"}="1H31 R16";
$resn_exch{"H312 R16"}="2H31 R16"; $resn_exch{"H321 R16"}="1H32 R16"; $resn_exch{"H322 R16"}="2H32 R16";
$resn_exch{"H331 R16"}="1H33 R16"; $resn_exch{"H332 R16"}="2H33 R16"; $resn_exch{"H341 R16"}="1H34 R16";
$resn_exch{"H342 R16"}="2H34 R16"; $resn_exch{"H351 R16"}="1H35 R16"; $resn_exch{"H352 R16"}="2H35 R16";
$resn_exch{"H361 R16"}="1H36 R16"; $resn_exch{"H362 R16"}="2H36 R16"; $resn_exch{"H371 R16"}="1H37 R16";
$resn_exch{"H372 R16"}="2H37 R16"; $resn_exch{"H381 R16"}="1H38 R16"; $resn_exch{"H382 R16"}="2H38 R16";
$resn_exch{"H391 R16"}="1H39 R16"; $resn_exch{"H392 R16"}="2H39 R16"; $resn_exch{"H401 R16"}="1H40 R16";
$resn_exch{"H402 R16"}="2H40 R16"; $resn_exch{"H411 R16"}="1H41 R16"; $resn_exch{"H412 R16"}="2H41 R16";
$resn_exch{"H421 R16"}="1H42 R16"; $resn_exch{"H422 R16"}="2H42 R16"; $resn_exch{"H423 R16"}="3H42 R16";
$resn_exch{"H3A1 R17"}="1H3A R17"; $resn_exch{"H3A2 R17"}="2H3A R17"; $resn_exch{"H1A1 R17"}="1H1A R17";
$resn_exch{"H1A2 R17"}="2H1A R17"; $resn_exch{"H1B1 R17"}="1H1B R17"; $resn_exch{"H1B2 R17"}="2H1B R17";
$resn_exch{"H4B1 R17"}="1H4B R17"; $resn_exch{"H4B2 R17"}="2H4B R17"; $resn_exch{"H1E1 R17"}="1H1E R17";
$resn_exch{"H1E2 R17"}="2H1E R17"; $resn_exch{"H1E3 R17"}="3H1E R17"; $resn_exch{"H12A R18"}="2H1  R18";
$resn_exch{"H11A R18"}="1H1  R18"; $resn_exch{" H22 R18"}="2H2  R18"; $resn_exch{" H21 R18"}="1H2  R18";
$resn_exch{" H62 R18"}="2H6  R18"; $resn_exch{" H61 R18"}="1H6  R18"; $resn_exch{" H72 R18"}="2H7  R18";
$resn_exch{" H71 R18"}="1H7  R18"; $resn_exch{"H152 R18"}="2H15 R18"; $resn_exch{"H151 R18"}="1H15 R18";
$resn_exch{"H162 R18"}="2H16 R18"; $resn_exch{"H161 R18"}="1H16 R18"; $resn_exch{"H183 R18"}="3H18 R18";
$resn_exch{"H182 R18"}="2H18 R18"; $resn_exch{"H181 R18"}="1H18 R18"; $resn_exch{"H273 R18"}="3H27 R18";
$resn_exch{"H272 R18"}="2H27 R18"; $resn_exch{"H271 R18"}="1H27 R18"; $resn_exch{"H1A1 R19"}="1H1A R19";
$resn_exch{"H1A2 R19"}="2H1A R19"; $resn_exch{"H1E1 R19"}="1H1E R19"; $resn_exch{"H1E2 R19"}="2H1E R19";
$resn_exch{"H1E3 R19"}="3H1E R19"; $resn_exch{"H1B1 R19"}="1H1B R19"; $resn_exch{"H1B2 R19"}="2H1B R19";
$resn_exch{"H2B1 R19"}="1H2B R19"; $resn_exch{"H2B2 R19"}="2H2B R19"; $resn_exch{"H3B1 R19"}="1H3B R19";
$resn_exch{"H3B2 R19"}="2H3B R19"; $resn_exch{"H4B1 R19"}="1H4B R19"; $resn_exch{"H4B2 R19"}="2H4B R19";
$resn_exch{"H5B1 R19"}="1H5B R19"; $resn_exch{"H5B2 R19"}="2H5B R19"; $resn_exch{"H6B1 R19"}="1H6B R19";
$resn_exch{"H6B2 R19"}="2H6B R19"; $resn_exch{"H3A1 R19"}="1H3A R19"; $resn_exch{"H3A2 R19"}="2H3A R19";
$resn_exch{" H   R1A"}="1HN  R1A"; $resn_exch{" H2  R1A"}="2HN  R1A"; $resn_exch{" HB2 R1A"}="1HB  R1A";
$resn_exch{" HB3 R1A"}="2HB  R1A"; $resn_exch{" HE2 R1A"}="1HE  R1A"; $resn_exch{" HE3 R1A"}="2HE  R1A";
$resn_exch{" H91 R1A"}="1H9  R1A"; $resn_exch{" H92 R1A"}="2H9  R1A"; $resn_exch{" H93 R1A"}="3H9  R1A";
$resn_exch{" H81 R1A"}="1H8  R1A"; $resn_exch{" H82 R1A"}="2H8  R1A"; $resn_exch{" H83 R1A"}="3H8  R1A";
$resn_exch{" H71 R1A"}="1H7  R1A"; $resn_exch{" H72 R1A"}="2H7  R1A"; $resn_exch{" H73 R1A"}="3H7  R1A";
$resn_exch{" H61 R1A"}="1H6  R1A"; $resn_exch{" H62 R1A"}="2H6  R1A"; $resn_exch{" H63 R1A"}="3H6  R1A";
$resn_exch{" H81 R1B"}="1H8  R1B"; $resn_exch{" H82 R1B"}="2H8  R1B"; $resn_exch{" H83 R1B"}="3H8  R1B";
$resn_exch{" H91 R1B"}="1H9  R1B"; $resn_exch{" H92 R1B"}="2H9  R1B"; $resn_exch{" H93 R1B"}="3H9  R1B";
$resn_exch{" H71 R1B"}="1H7  R1B"; $resn_exch{" H72 R1B"}="2H7  R1B"; $resn_exch{" H73 R1B"}="3H7  R1B";
$resn_exch{" H61 R1B"}="1H6  R1B"; $resn_exch{" H62 R1B"}="2H6  R1B"; $resn_exch{" H63 R1B"}="3H6  R1B";
$resn_exch{"H101 R1B"}="1H10 R1B"; $resn_exch{"H102 R1B"}="2H10 R1B"; $resn_exch{"H103 R1B"}="3H10 R1B";
$resn_exch{" HE3 R1B"}="1HE  R1B"; $resn_exch{" HE2 R1B"}="2HE  R1B"; $resn_exch{" HB3 R1B"}="1HB  R1B";
$resn_exch{" HB2 R1B"}="2HB  R1B"; $resn_exch{" H   R1B"}="1HN  R1B"; $resn_exch{" H2  R1B"}="2HN  R1B";
$resn_exch{" H   R1F"}="1HN  R1F"; $resn_exch{" H2  R1F"}="2HN  R1F"; $resn_exch{" HB2 R1F"}="1HB  R1F";
$resn_exch{" HB3 R1F"}="2HB  R1F"; $resn_exch{" HE2 R1F"}="1HE  R1F"; $resn_exch{" HE3 R1F"}="2HE  R1F";
$resn_exch{" H91 R1F"}="1H9  R1F"; $resn_exch{" H92 R1F"}="2H9  R1F"; $resn_exch{" H93 R1F"}="3H9  R1F";
$resn_exch{" H81 R1F"}="1H8  R1F"; $resn_exch{" H82 R1F"}="2H8  R1F"; $resn_exch{" H83 R1F"}="3H8  R1F";
$resn_exch{" H61 R1F"}="1H6  R1F"; $resn_exch{" H62 R1F"}="2H6  R1F"; $resn_exch{" H63 R1F"}="3H6  R1F";
$resn_exch{" H71 R1F"}="1H7  R1F"; $resn_exch{" H72 R1F"}="2H7  R1F"; $resn_exch{" H73 R1F"}="3H7  R1F";
$resn_exch{"H5'1 R1P"}="1H5' R1P"; $resn_exch{"H5'2 R1P"}="2H5' R1P"; $resn_exch{"HO5' R1P"}="'HO5 R1P";
$resn_exch{"HO2' R1P"}="'HO2 R1P"; $resn_exch{"HO3' R1P"}="'HO3 R1P"; $resn_exch{"HOP2 R1P"}="2HOP R1P";
$resn_exch{"HOP3 R1P"}="3HOP R1P"; $resn_exch{" H61 R1T"}="1H6  R1T"; $resn_exch{" H62 R1T"}="2H6  R1T";
$resn_exch{" H71 R1T"}="1H7  R1T"; $resn_exch{" H72 R1T"}="2H7  R1T"; $resn_exch{"HM41 R1T"}="1HM4 R1T";
$resn_exch{"HM42 R1T"}="2HM4 R1T"; $resn_exch{"HM43 R1T"}="3HM4 R1T"; $resn_exch{"H7'1 R1T"}="1H7' R1T";
$resn_exch{"H7'2 R1T"}="2H7' R1T"; $resn_exch{"HM21 R1T"}="1HM2 R1T"; $resn_exch{"HM22 R1T"}="2HM2 R1T";
$resn_exch{"HM23 R1T"}="3HM2 R1T"; $resn_exch{"H4'1 R1T"}="1H4' R1T"; $resn_exch{"H4'2 R1T"}="2H4' R1T";
$resn_exch{"HP21 R1T"}="1HP2 R1T"; $resn_exch{"HP22 R1T"}="2HP2 R1T"; $resn_exch{"HP23 R1T"}="3HP2 R1T";
$resn_exch{"H121 R20"}="1H12 R20"; $resn_exch{"H122 R20"}="2H12 R20"; $resn_exch{"H123 R20"}="3H12 R20";
$resn_exch{" H2' R22"}=" H2* R22"; $resn_exch{" H4' R22"}=" H4* R22"; $resn_exch{" H6' R22"}=" H6* R22";
$resn_exch{"H8'1 R22"}="1H8* R22"; $resn_exch{"H8'2 R22"}="2H8* R22"; $resn_exch{"H8'3 R22"}="3H8* R22";
$resn_exch{"H9'1 R22"}="1H9* R22"; $resn_exch{"H9'2 R22"}="2H9* R22"; $resn_exch{"H9'3 R22"}="3H9* R22";
$resn_exch{" H71 R22"}="1H7  R22"; $resn_exch{" H72 R22"}="2H7  R22"; $resn_exch{" H73 R22"}="3H7  R22";
$resn_exch{"H101 R22"}="1H10 R22"; $resn_exch{"H102 R22"}="2H10 R22"; $resn_exch{"H121 R22"}="1H12 R22";
$resn_exch{"H122 R22"}="2H12 R22"; $resn_exch{"H4E1 R23"}="1H4E R23"; $resn_exch{"H4E2 R23"}="2H4E R23";
$resn_exch{"H5E1 R23"}="1H5E R23"; $resn_exch{"H5E2 R23"}="2H5E R23"; $resn_exch{"H6E1 R23"}="1H6E R23";
$resn_exch{"H6E2 R23"}="2H6E R23"; $resn_exch{"H2E1 R23"}="1H2E R23"; $resn_exch{"H2E2 R23"}="2H2E R23";
$resn_exch{"H3E1 R23"}="1H3E R23"; $resn_exch{"H3E2 R23"}="2H3E R23"; $resn_exch{"H1D1 R23"}="1H1D R23";
$resn_exch{"H1D2 R23"}="2H1D R23"; $resn_exch{"H3B1 R23"}="1H3B R23"; $resn_exch{"H3B2 R23"}="2H3B R23";
$resn_exch{"H2B1 R23"}="1H2B R23"; $resn_exch{"H2B2 R23"}="2H2B R23"; $resn_exch{"H6B1 R23"}="1H6B R23";
$resn_exch{"H6B2 R23"}="2H6B R23"; $resn_exch{"H5B1 R23"}="1H5B R23"; $resn_exch{"H5B2 R23"}="2H5B R23";
$resn_exch{"H1A1 R23"}="1H1A R23"; $resn_exch{"H1A2 R23"}="2H1A R23"; $resn_exch{"H1A3 R23"}="3H1A R23";
$resn_exch{"H101 R36"}="1H10 R36"; $resn_exch{"H131 R36"}="1H13 R36"; $resn_exch{"H141 R36"}="1H14 R36";
$resn_exch{" H31 R36"}="1H3  R36"; $resn_exch{" H32 R36"}="2H3  R36"; $resn_exch{" H33 R36"}="3H3  R36";
$resn_exch{" H21 R36"}="1H2  R36"; $resn_exch{" H22 R36"}="2H2  R36"; $resn_exch{" H23 R36"}="3H2  R36";
$resn_exch{"H261 R36"}="1H26 R36"; $resn_exch{"H271 R36"}="1H27 R36"; $resn_exch{"H281 R36"}="1H28 R36";
$resn_exch{" H11 R36"}="1H1  R36"; $resn_exch{" H12 R36"}="2H1  R36"; $resn_exch{"H301 R36"}="1H30 R36";
$resn_exch{"H341 R36"}="1H34 R36"; $resn_exch{"H342 R36"}="2H34 R36"; $resn_exch{"H161 R36"}="1H16 R36";
$resn_exch{"H151 R36"}="1H15 R36"; $resn_exch{"H14O R36"}="OH14 R36"; $resn_exch{"H121 R36"}="1H12 R36";
$resn_exch{"H122 R36"}="2H12 R36"; $resn_exch{" H41 R36"}="1H4  R36"; $resn_exch{" H51 R36"}="1H5  R36";
$resn_exch{" H52 R36"}="2H5  R36"; $resn_exch{" H61 R36"}="1H6  R36"; $resn_exch{"H111 R36"}="1H11 R36";
$resn_exch{"H112 R36"}="2H11 R36"; $resn_exch{"H113 R36"}="3H11 R36"; $resn_exch{" H71 R36"}="1H7  R36";
$resn_exch{" H72 R36"}="2H7  R36"; $resn_exch{" H73 R36"}="3H7  R36"; $resn_exch{"HN31 R36"}="1HN3 R36";
$resn_exch{"H171 R36"}="1H17 R36"; $resn_exch{"H181 R36"}="1H18 R36"; $resn_exch{"H182 R36"}="2H18 R36";
$resn_exch{"H201 R36"}="1H20 R36"; $resn_exch{"H211 R36"}="1H21 R36"; $resn_exch{"H221 R36"}="1H22 R36";
$resn_exch{"H231 R36"}="1H23 R36"; $resn_exch{"H241 R36"}="1H24 R36"; $resn_exch{"H101 R37"}="1H10 R37";
$resn_exch{"H131 R37"}="1H13 R37"; $resn_exch{"H141 R37"}="1H14 R37"; $resn_exch{" H31 R37"}="1H3  R37";
$resn_exch{" H32 R37"}="2H3  R37"; $resn_exch{" H33 R37"}="3H3  R37"; $resn_exch{" H21 R37"}="1H2  R37";
$resn_exch{" H22 R37"}="2H2  R37"; $resn_exch{" H23 R37"}="3H2  R37"; $resn_exch{"H261 R37"}="1H26 R37";
$resn_exch{"H271 R37"}="1H27 R37"; $resn_exch{"H291 R37"}="1H29 R37"; $resn_exch{" H11 R37"}="1H1  R37";
$resn_exch{" H12 R37"}="2H1  R37"; $resn_exch{"H301 R37"}="1H30 R37"; $resn_exch{"H341 R37"}="1H34 R37";
$resn_exch{"H342 R37"}="2H34 R37"; $resn_exch{"H161 R37"}="1H16 R37"; $resn_exch{"H151 R37"}="1H15 R37";
$resn_exch{"H14O R37"}="OH14 R37"; $resn_exch{"H121 R37"}="1H12 R37"; $resn_exch{"H122 R37"}="2H12 R37";
$resn_exch{" H41 R37"}="1H4  R37"; $resn_exch{" H51 R37"}="1H5  R37"; $resn_exch{" H52 R37"}="2H5  R37";
$resn_exch{" H61 R37"}="1H6  R37"; $resn_exch{"H111 R37"}="1H11 R37"; $resn_exch{"H112 R37"}="2H11 R37";
$resn_exch{"H113 R37"}="3H11 R37"; $resn_exch{" H71 R37"}="1H7  R37"; $resn_exch{" H72 R37"}="2H7  R37";
$resn_exch{" H73 R37"}="3H7  R37"; $resn_exch{"HN31 R37"}="1HN3 R37"; $resn_exch{"H171 R37"}="1H17 R37";
$resn_exch{"H181 R37"}="1H18 R37"; $resn_exch{"H182 R37"}="2H18 R37"; $resn_exch{"H201 R37"}="1H20 R37";
$resn_exch{"H211 R37"}="1H21 R37"; $resn_exch{"H221 R37"}="1H22 R37"; $resn_exch{"H231 R37"}="1H23 R37";
$resn_exch{"H241 R37"}="1H24 R37"; $resn_exch{"H3A1 R46"}="1H3A R46"; $resn_exch{"H3A2 R46"}="2H3A R46";
$resn_exch{"H1A1 R46"}="1H1A R46"; $resn_exch{"H1A2 R46"}="2H1A R46"; $resn_exch{"H1B1 R46"}="1H1B R46";
$resn_exch{"H1B2 R46"}="2H1B R46"; $resn_exch{"H2B1 R46"}="1H2B R46"; $resn_exch{"H2B2 R46"}="2H2B R46";
$resn_exch{"H3B1 R46"}="1H3B R46"; $resn_exch{"H3B2 R46"}="2H3B R46"; $resn_exch{"H4B1 R46"}="1H4B R46";
$resn_exch{"H4B2 R46"}="2H4B R46"; $resn_exch{"H5B1 R46"}="1H5B R46"; $resn_exch{"H5B2 R46"}="2H5B R46";
$resn_exch{"H6B1 R46"}="1H6B R46"; $resn_exch{"H6B2 R46"}="2H6B R46"; $resn_exch{"H1F1 R46"}="1H1F R46";
$resn_exch{"H1F2 R46"}="2H1F R46"; $resn_exch{"H1F3 R46"}="3H1F R46"; $resn_exch{"HM23 R4A"}="3HM2 R4A";
$resn_exch{"HM22 R4A"}="2HM2 R4A"; $resn_exch{"HM21 R4A"}="1HM2 R4A"; $resn_exch{"HM13 R4A"}="3HM1 R4A";
$resn_exch{"HM12 R4A"}="2HM1 R4A"; $resn_exch{"HM11 R4A"}="1HM1 R4A"; $resn_exch{"HL41 R4A"}="1HL4 R4A";
$resn_exch{"HL42 R4A"}="2HL4 R4A"; $resn_exch{"HL31 R4A"}="1HL3 R4A"; $resn_exch{"HL32 R4A"}="2HL3 R4A";
$resn_exch{"HL21 R4A"}="1HL2 R4A"; $resn_exch{"HL22 R4A"}="2HL2 R4A"; $resn_exch{"HL11 R4A"}="1HL1 R4A";
$resn_exch{"HL12 R4A"}="2HL1 R4A"; $resn_exch{"HAN1 R4A"}="1HAN R4A"; $resn_exch{"HCK1 R4A"}="1HCK R4A";
$resn_exch{"HCJ1 R4A"}="1HCJ R4A"; $resn_exch{"HCE1 R4A"}="1HCE R4A"; $resn_exch{"HCD1 R4A"}="1HCD R4A";
$resn_exch{"HCC1 R4A"}="1HCC R4A"; $resn_exch{"HCB1 R4A"}="1HCB R4A"; $resn_exch{"HBK1 R4A"}="1HBK R4A";
$resn_exch{"HBJ1 R4A"}="1HBJ R4A"; $resn_exch{"HBI1 R4A"}="1HBI R4A"; $resn_exch{"HBH1 R4A"}="1HBH R4A";
$resn_exch{"HBE1 R4A"}="1HBE R4A"; $resn_exch{"HBD1 R4A"}="1HBD R4A"; $resn_exch{"HBC1 R4A"}="1HBC R4A";
$resn_exch{"HBB1 R4A"}="1HBB R4A"; $resn_exch{"H321 R55"}="1H32 R55"; $resn_exch{"H322 R55"}="2H32 R55";
$resn_exch{"H341 R55"}="1H34 R55"; $resn_exch{"H342 R55"}="2H34 R55"; $resn_exch{"H351 R55"}="1H35 R55";
$resn_exch{"H352 R55"}="2H35 R55"; $resn_exch{"H361 R55"}="1H36 R55"; $resn_exch{"H362 R55"}="2H36 R55";
$resn_exch{"H921 R55"}="1H92 R55"; $resn_exch{"H922 R55"}="2H92 R55"; $resn_exch{"H931 R55"}="1H93 R55";
$resn_exch{"H932 R55"}="2H93 R55"; $resn_exch{"H971 R55"}="1H97 R55"; $resn_exch{"H972 R55"}="2H97 R55";
$resn_exch{"H973 R55"}="3H97 R55"; $resn_exch{"H981 R55"}="1H98 R55"; $resn_exch{"H982 R55"}="2H98 R55";
$resn_exch{"H983 R55"}="3H98 R55"; $resn_exch{"H951 R55"}="1H95 R55"; $resn_exch{"H952 R55"}="2H95 R55";
$resn_exch{"H961 R55"}="1H96 R55"; $resn_exch{"H962 R55"}="2H96 R55"; $resn_exch{"H121 R56"}="1H12 R56";
$resn_exch{"H122 R56"}="2H12 R56"; $resn_exch{"H271 R56"}="1H27 R56"; $resn_exch{"H272 R56"}="2H27 R56";
$resn_exch{"H301 R56"}="1H30 R56"; $resn_exch{"H302 R56"}="2H30 R56"; $resn_exch{"H341 R56"}="1H34 R56";
$resn_exch{"H342 R56"}="2H34 R56"; $resn_exch{"H431 R56"}="1H43 R56"; $resn_exch{"H432 R56"}="2H43 R56";
$resn_exch{"H461 R56"}="1H46 R56"; $resn_exch{"H462 R56"}="2H46 R56"; $resn_exch{"H491 R56"}="1H49 R56";
$resn_exch{"H492 R56"}="2H49 R56"; $resn_exch{"HN51 R56"}="1HN5 R56"; $resn_exch{"HN52 R56"}="2HN5 R56";
$resn_exch{"HM23 R5A"}="3HM2 R5A"; $resn_exch{"HM22 R5A"}="2HM2 R5A"; $resn_exch{"HM21 R5A"}="1HM2 R5A";
$resn_exch{"HM13 R5A"}="3HM1 R5A"; $resn_exch{"HM12 R5A"}="2HM1 R5A"; $resn_exch{"HM11 R5A"}="1HM1 R5A";
$resn_exch{"HL51 R5A"}="1HL5 R5A"; $resn_exch{"HL52 R5A"}="2HL5 R5A"; $resn_exch{"HL41 R5A"}="1HL4 R5A";
$resn_exch{"HL42 R5A"}="2HL4 R5A"; $resn_exch{"HL31 R5A"}="1HL3 R5A"; $resn_exch{"HL32 R5A"}="2HL3 R5A";
$resn_exch{"HL21 R5A"}="1HL2 R5A"; $resn_exch{"HL22 R5A"}="2HL2 R5A"; $resn_exch{"HL11 R5A"}="1HL1 R5A";
$resn_exch{"HL12 R5A"}="2HL1 R5A"; $resn_exch{"HCK1 R5A"}="1HCK R5A"; $resn_exch{"HCJ1 R5A"}="1HCJ R5A";
$resn_exch{"HCE1 R5A"}="1HCE R5A"; $resn_exch{"HCD1 R5A"}="1HCD R5A"; $resn_exch{"HCC1 R5A"}="1HCC R5A";
$resn_exch{"HCB1 R5A"}="1HCB R5A"; $resn_exch{"HBK1 R5A"}="1HBK R5A"; $resn_exch{"HBJ1 R5A"}="1HBJ R5A";
$resn_exch{"HBI1 R5A"}="1HBI R5A"; $resn_exch{"HBH1 R5A"}="1HBH R5A"; $resn_exch{"HBE1 R5A"}="1HBE R5A";
$resn_exch{"HBD1 R5A"}="1HBD R5A"; $resn_exch{"HBC1 R5A"}="1HBC R5A"; $resn_exch{"HBB1 R5A"}="1HBB R5A";
$resn_exch{"HM23 R5B"}="3HM2 R5B"; $resn_exch{"HM22 R5B"}="2HM2 R5B"; $resn_exch{"HM21 R5B"}="1HM2 R5B";
$resn_exch{"HM13 R5B"}="3HM1 R5B"; $resn_exch{"HM12 R5B"}="2HM1 R5B"; $resn_exch{"HM11 R5B"}="1HM1 R5B";
$resn_exch{"HL51 R5B"}="1HL5 R5B"; $resn_exch{"HL52 R5B"}="2HL5 R5B"; $resn_exch{"HL41 R5B"}="1HL4 R5B";
$resn_exch{"HL42 R5B"}="2HL4 R5B"; $resn_exch{"HL31 R5B"}="1HL3 R5B"; $resn_exch{"HL32 R5B"}="2HL3 R5B";
$resn_exch{"HL21 R5B"}="1HL2 R5B"; $resn_exch{"HL22 R5B"}="2HL2 R5B"; $resn_exch{"HL11 R5B"}="1HL1 R5B";
$resn_exch{"HL12 R5B"}="2HL1 R5B"; $resn_exch{"HCK1 R5B"}="1HCK R5B"; $resn_exch{"HCJ1 R5B"}="1HCJ R5B";
$resn_exch{"HCE1 R5B"}="1HCE R5B"; $resn_exch{"HCD1 R5B"}="1HCD R5B"; $resn_exch{"HCC1 R5B"}="1HCC R5B";
$resn_exch{"HCB1 R5B"}="1HCB R5B"; $resn_exch{"HBK1 R5B"}="1HBK R5B"; $resn_exch{"HBJ1 R5B"}="1HBJ R5B";
$resn_exch{"HBI1 R5B"}="1HBI R5B"; $resn_exch{"HBH1 R5B"}="1HBH R5B"; $resn_exch{"HBE1 R5B"}="1HBE R5B";
$resn_exch{"HBD1 R5B"}="1HBD R5B"; $resn_exch{"HBC1 R5B"}="1HBC R5B"; $resn_exch{"HBB1 R5B"}="1HBB R5B";
$resn_exch{"HOP2 R5P"}="2HOP R5P"; $resn_exch{"HOP3 R5P"}="3HOP R5P"; $resn_exch{"H301 R64"}="1H30 R64";
$resn_exch{"H302 R64"}="2H30 R64"; $resn_exch{"H303 R64"}="3H30 R64"; $resn_exch{"H111 R64"}="1H11 R64";
$resn_exch{"H112 R64"}="2H11 R64"; $resn_exch{"H113 R64"}="3H11 R64"; $resn_exch{"H131 R64"}="1H13 R64";
$resn_exch{"H132 R64"}="2H13 R64"; $resn_exch{"H141 R64"}="1H14 R64"; $resn_exch{"H142 R64"}="2H14 R64";
$resn_exch{"H151 R64"}="1H15 R64"; $resn_exch{"H152 R64"}="2H15 R64"; $resn_exch{"H171 R64"}="1H17 R64";
$resn_exch{"H172 R64"}="2H17 R64"; $resn_exch{"H321 R68"}="1H32 R68"; $resn_exch{"H322 R68"}="2H32 R68";
$resn_exch{"H341 R68"}="1H34 R68"; $resn_exch{"H342 R68"}="2H34 R68"; $resn_exch{"H351 R68"}="1H35 R68";
$resn_exch{"H352 R68"}="2H35 R68"; $resn_exch{"H361 R68"}="1H36 R68"; $resn_exch{"H362 R68"}="2H36 R68";
$resn_exch{"H971 R68"}="1H97 R68"; $resn_exch{"H972 R68"}="2H97 R68"; $resn_exch{"H973 R68"}="3H97 R68";
$resn_exch{"H981 R68"}="1H98 R68"; $resn_exch{"H982 R68"}="2H98 R68"; $resn_exch{"H983 R68"}="3H98 R68";
$resn_exch{"H931 R68"}="1H93 R68"; $resn_exch{"H932 R68"}="2H93 R68"; $resn_exch{"H941 R68"}="1H94 R68";
$resn_exch{"H942 R68"}="2H94 R68"; $resn_exch{"H951 R68"}="1H95 R68"; $resn_exch{"H952 R68"}="2H95 R68";
$resn_exch{"H321 R69"}="1H32 R69"; $resn_exch{"H322 R69"}="2H32 R69"; $resn_exch{"H341 R69"}="1H34 R69";
$resn_exch{"H342 R69"}="2H34 R69"; $resn_exch{"H351 R69"}="1H35 R69"; $resn_exch{"H352 R69"}="2H35 R69";
$resn_exch{"H361 R69"}="1H36 R69"; $resn_exch{"H362 R69"}="2H36 R69"; $resn_exch{"H981 R69"}="1H98 R69";
$resn_exch{"H982 R69"}="2H98 R69"; $resn_exch{"H983 R69"}="3H98 R69"; $resn_exch{"H991 R69"}="1H99 R69";
$resn_exch{"H992 R69"}="2H99 R69"; $resn_exch{"H993 R69"}="3H99 R69"; $resn_exch{"HM23 R6A"}="3HM2 R6A";
$resn_exch{"HM22 R6A"}="2HM2 R6A"; $resn_exch{"HM21 R6A"}="1HM2 R6A"; $resn_exch{"HM13 R6A"}="3HM1 R6A";
$resn_exch{"HM12 R6A"}="2HM1 R6A"; $resn_exch{"HM11 R6A"}="1HM1 R6A"; $resn_exch{"HL61 R6A"}="1HL6 R6A";
$resn_exch{"HL62 R6A"}="2HL6 R6A"; $resn_exch{"HL51 R6A"}="1HL5 R6A"; $resn_exch{"HL52 R6A"}="2HL5 R6A";
$resn_exch{"HL41 R6A"}="1HL4 R6A"; $resn_exch{"HL42 R6A"}="2HL4 R6A"; $resn_exch{"HL31 R6A"}="1HL3 R6A";
$resn_exch{"HL32 R6A"}="2HL3 R6A"; $resn_exch{"HCK1 R6A"}="1HCK R6A"; $resn_exch{"HCJ1 R6A"}="1HCJ R6A";
$resn_exch{"HCE1 R6A"}="1HCE R6A"; $resn_exch{"HCD1 R6A"}="1HCD R6A"; $resn_exch{"HCC1 R6A"}="1HCC R6A";
$resn_exch{"HCB1 R6A"}="1HCB R6A"; $resn_exch{"HBK1 R6A"}="1HBK R6A"; $resn_exch{"HL21 R6A"}="1HL2 R6A";
$resn_exch{"HL22 R6A"}="2HL2 R6A"; $resn_exch{"HBJ1 R6A"}="1HBJ R6A"; $resn_exch{"HBI1 R6A"}="1HBI R6A";
$resn_exch{"HL11 R6A"}="1HL1 R6A"; $resn_exch{"HL12 R6A"}="2HL1 R6A"; $resn_exch{"HBH1 R6A"}="1HBH R6A";
$resn_exch{"HBE1 R6A"}="1HBE R6A"; $resn_exch{"HBD1 R6A"}="1HBD R6A"; $resn_exch{"HBC1 R6A"}="1HBC R6A";
$resn_exch{"HBB1 R6A"}="1HBB R6A"; $resn_exch{"HAQ1 R6C"}="1HAQ R6C"; $resn_exch{"HAQ2 R6C"}="2HAQ R6C";
$resn_exch{"HAK1 R6C"}="1HAK R6C"; $resn_exch{"HAK2 R6C"}="2HAK R6C"; $resn_exch{"HAI1 R6C"}="1HAI R6C";
$resn_exch{"HAI2 R6C"}="2HAI R6C"; $resn_exch{"HAI3 R6C"}="3HAI R6C"; $resn_exch{"HAY1 R6C"}="1HAY R6C";
$resn_exch{"HAY2 R6C"}="2HAY R6C"; $resn_exch{"HAY3 R6C"}="3HAY R6C"; $resn_exch{"HBA1 R6C"}="1HBA R6C";
$resn_exch{"HBA2 R6C"}="2HBA R6C"; $resn_exch{"HBA3 R6C"}="3HBA R6C"; $resn_exch{"HAD1 R6C"}="1HAD R6C";
$resn_exch{"HAD2 R6C"}="2HAD R6C"; $resn_exch{" HC1 R6G"}="1HC  R6G"; $resn_exch{" HC2 R6G"}="2HC  R6G";
$resn_exch{"H281 R6G"}="1H28 R6G"; $resn_exch{"H282 R6G"}="2H28 R6G"; $resn_exch{"H311 R6G"}="1H31 R6G";
$resn_exch{"H312 R6G"}="2H31 R6G"; $resn_exch{"H291 R6G"}="1H29 R6G"; $resn_exch{"H292 R6G"}="2H29 R6G";
$resn_exch{"H211 R6G"}="1H21 R6G"; $resn_exch{"H212 R6G"}="2H21 R6G"; $resn_exch{"H213 R6G"}="3H21 R6G";
$resn_exch{"H201 R6G"}="1H20 R6G"; $resn_exch{"H202 R6G"}="2H20 R6G"; $resn_exch{"H203 R6G"}="3H20 R6G";
$resn_exch{"H241 R6G"}="1H24 R6G"; $resn_exch{"H242 R6G"}="2H24 R6G"; $resn_exch{"H251 R6G"}="1H25 R6G";
$resn_exch{"H252 R6G"}="2H25 R6G"; $resn_exch{"H261 R6G"}="1H26 R6G"; $resn_exch{"H262 R6G"}="2H26 R6G";
$resn_exch{"H271 R6G"}="1H27 R6G"; $resn_exch{"H272 R6G"}="2H27 R6G"; $resn_exch{"H301 R6G"}="1H30 R6G";
$resn_exch{"H302 R6G"}="2H30 R6G"; $resn_exch{"H303 R6G"}="3H30 R6G"; $resn_exch{"H321 R6G"}="1H32 R6G";
$resn_exch{"H322 R6G"}="2H32 R6G"; $resn_exch{"H323 R6G"}="3H32 R6G"; $resn_exch{" HO1 R6G"}="1HO  R6G";
$resn_exch{"H151 R71"}="1H15 R71"; $resn_exch{"H152 R71"}="2H15 R71"; $resn_exch{"H161 R71"}="1H16 R71";
$resn_exch{"H162 R71"}="2H16 R71"; $resn_exch{"H171 R71"}="1H17 R71"; $resn_exch{"H172 R71"}="2H17 R71";
$resn_exch{"H181 R71"}="1H18 R71"; $resn_exch{"H182 R71"}="2H18 R71"; $resn_exch{"H191 R71"}="1H19 R71";
$resn_exch{"H192 R71"}="2H19 R71"; $resn_exch{"H201 R71"}="1H20 R71"; $resn_exch{"H202 R71"}="2H20 R71";
$resn_exch{"H271 R71"}="1H27 R71"; $resn_exch{"H272 R71"}="2H27 R71"; $resn_exch{"H273 R71"}="3H27 R71";
$resn_exch{"H251 R71"}="1H25 R71"; $resn_exch{"H252 R71"}="2H25 R71"; $resn_exch{"H291 R71"}="1H29 R71";
$resn_exch{"H292 R71"}="2H29 R71"; $resn_exch{"H3A1 R79"}="1H3A R79"; $resn_exch{"H3A2 R79"}="2H3A R79";
$resn_exch{"H1A1 R79"}="1H1A R79"; $resn_exch{"H1A2 R79"}="2H1A R79"; $resn_exch{"H1B1 R79"}="1H1B R79";
$resn_exch{"H1B2 R79"}="2H1B R79"; $resn_exch{"H2B1 R79"}="1H2B R79"; $resn_exch{"H2B2 R79"}="2H2B R79";
$resn_exch{"H3B1 R79"}="1H3B R79"; $resn_exch{"H3B2 R79"}="2H3B R79"; $resn_exch{"H4B1 R79"}="1H4B R79";
$resn_exch{"H4B2 R79"}="2H4B R79"; $resn_exch{"H5B1 R79"}="1H5B R79"; $resn_exch{"H5B2 R79"}="2H5B R79";
$resn_exch{"H6B1 R79"}="1H6B R79"; $resn_exch{"H6B2 R79"}="2H6B R79"; $resn_exch{"H1F1 R79"}="1H1F R79";
$resn_exch{"H1F2 R79"}="2H1F R79"; $resn_exch{"H1F3 R79"}="3H1F R79"; $resn_exch{" H   R7A"}="1HN  R7A";
$resn_exch{" H2  R7A"}="2HN  R7A"; $resn_exch{" HB2 R7A"}="1HB  R7A"; $resn_exch{" HB3 R7A"}="2HB  R7A";
$resn_exch{" HE2 R7A"}="1HE  R7A"; $resn_exch{" HE3 R7A"}="2HE  R7A"; $resn_exch{" H91 R7A"}="1H9  R7A";
$resn_exch{" H92 R7A"}="2H9  R7A"; $resn_exch{" H93 R7A"}="3H9  R7A"; $resn_exch{" H81 R7A"}="1H8  R7A";
$resn_exch{" H82 R7A"}="2H8  R7A"; $resn_exch{" H83 R7A"}="3H8  R7A"; $resn_exch{" H61 R7A"}="1H6  R7A";
$resn_exch{" H62 R7A"}="2H6  R7A"; $resn_exch{" H63 R7A"}="3H6  R7A"; $resn_exch{" H71 R7A"}="1H7  R7A";
$resn_exch{" H72 R7A"}="2H7  R7A"; $resn_exch{" H73 R7A"}="3H7  R7A"; $resn_exch{"HM23 R7U"}="3HM2 R7U";
$resn_exch{"HM22 R7U"}="2HM2 R7U"; $resn_exch{"HM21 R7U"}="1HM2 R7U"; $resn_exch{"HM13 R7U"}="3HM1 R7U";
$resn_exch{"HM12 R7U"}="2HM1 R7U"; $resn_exch{"HM11 R7U"}="1HM1 R7U"; $resn_exch{"HL71 R7U"}="1HL7 R7U";
$resn_exch{"HL72 R7U"}="2HL7 R7U"; $resn_exch{"HL61 R7U"}="1HL6 R7U"; $resn_exch{"HL62 R7U"}="2HL6 R7U";
$resn_exch{"HL51 R7U"}="1HL5 R7U"; $resn_exch{"HL52 R7U"}="2HL5 R7U"; $resn_exch{"HL41 R7U"}="1HL4 R7U";
$resn_exch{"HL42 R7U"}="2HL4 R7U"; $resn_exch{"HCK1 R7U"}="1HCK R7U"; $resn_exch{"HCJ1 R7U"}="1HCJ R7U";
$resn_exch{"HCE1 R7U"}="1HCE R7U"; $resn_exch{"HCD1 R7U"}="1HCD R7U"; $resn_exch{"HCC1 R7U"}="1HCC R7U";
$resn_exch{"HCB1 R7U"}="1HCB R7U"; $resn_exch{"HBK1 R7U"}="1HBK R7U"; $resn_exch{"HL31 R7U"}="1HL3 R7U";
$resn_exch{"HL32 R7U"}="2HL3 R7U"; $resn_exch{"HBJ1 R7U"}="1HBJ R7U"; $resn_exch{"HBI1 R7U"}="1HBI R7U";
$resn_exch{"HL11 R7U"}="1HL1 R7U"; $resn_exch{"HL12 R7U"}="2HL1 R7U"; $resn_exch{"HL21 R7U"}="1HL2 R7U";
$resn_exch{"HL22 R7U"}="2HL2 R7U"; $resn_exch{"HBH1 R7U"}="1HBH R7U"; $resn_exch{"HBE1 R7U"}="1HBE R7U";
$resn_exch{"HBD1 R7U"}="1HBD R7U"; $resn_exch{"HBC1 R7U"}="1HBC R7U"; $resn_exch{"HBB1 R7U"}="1HBB R7U";
$resn_exch{"H481 R86"}="1H48 R86"; $resn_exch{"H482 R86"}="2H48 R86"; $resn_exch{"H483 R86"}="3H48 R86";
$resn_exch{"H381 R86"}="1H38 R86"; $resn_exch{"H382 R86"}="2H38 R86"; $resn_exch{"H371 R86"}="1H37 R86";
$resn_exch{"H372 R86"}="2H37 R86"; $resn_exch{"H321 R86"}="1H32 R86"; $resn_exch{"H322 R86"}="2H32 R86";
$resn_exch{"H261 R86"}="1H26 R86"; $resn_exch{"H262 R86"}="2H26 R86"; $resn_exch{"H311 R86"}="1H31 R86";
$resn_exch{"H312 R86"}="2H31 R86"; $resn_exch{"H313 R86"}="3H31 R86"; $resn_exch{"H301 R86"}="1H30 R86";
$resn_exch{"H302 R86"}="2H30 R86"; $resn_exch{"H303 R86"}="3H30 R86"; $resn_exch{"H231 R86"}="1H23 R86";
$resn_exch{"H232 R86"}="2H23 R86"; $resn_exch{"H221 R86"}="1H22 R86"; $resn_exch{"H222 R86"}="2H22 R86";
$resn_exch{"H171 R86"}="1H17 R86"; $resn_exch{"H172 R86"}="2H17 R86"; $resn_exch{"H173 R86"}="3H17 R86";
$resn_exch{"H151 R88"}="1H15 R88"; $resn_exch{"H152 R88"}="2H15 R88"; $resn_exch{"H201 R88"}="1H20 R88";
$resn_exch{"H202 R88"}="2H20 R88"; $resn_exch{"H271 R88"}="1H27 R88"; $resn_exch{"H272 R88"}="2H27 R88";
$resn_exch{"H273 R88"}="3H27 R88"; $resn_exch{"H261 R88"}="1H26 R88"; $resn_exch{"H262 R88"}="2H26 R88";
$resn_exch{"H3E1 R88"}="1H3E R88"; $resn_exch{"H3E2 R88"}="2H3E R88"; $resn_exch{"H921 R94"}="1H92 R94";
$resn_exch{"H922 R94"}="2H92 R94"; $resn_exch{"H931 R94"}="1H93 R94"; $resn_exch{"H932 R94"}="2H93 R94";
$resn_exch{"H941 R94"}="1H94 R94"; $resn_exch{"H942 R94"}="2H94 R94"; $resn_exch{"H951 R94"}="1H95 R94";
$resn_exch{"H952 R94"}="2H95 R94"; $resn_exch{"H961 R94"}="1H96 R94"; $resn_exch{"H962 R94"}="2H96 R94";
$resn_exch{"H361 R94"}="1H36 R94"; $resn_exch{"H362 R94"}="2H36 R94"; $resn_exch{"H351 R94"}="1H35 R94";
$resn_exch{"H352 R94"}="2H35 R94"; $resn_exch{"H341 R94"}="1H34 R94"; $resn_exch{"H342 R94"}="2H34 R94";
$resn_exch{"H321 R94"}="1H32 R94"; $resn_exch{"H322 R94"}="2H32 R94"; $resn_exch{"H321 R96"}="1H32 R96";
$resn_exch{"H322 R96"}="2H32 R96"; $resn_exch{"H341 R96"}="1H34 R96"; $resn_exch{"H342 R96"}="2H34 R96";
$resn_exch{"H351 R96"}="1H35 R96"; $resn_exch{"H352 R96"}="2H35 R96"; $resn_exch{"H361 R96"}="1H36 R96";
$resn_exch{"H362 R96"}="2H36 R96"; $resn_exch{"H921 R96"}="1H92 R96"; $resn_exch{"H922 R96"}="2H92 R96";
$resn_exch{"H931 R96"}="1H93 R96"; $resn_exch{"H932 R96"}="2H93 R96"; $resn_exch{"H941 R96"}="1H94 R96";
$resn_exch{"H942 R96"}="2H94 R96"; $resn_exch{"H971 R96"}="1H97 R96"; $resn_exch{"H972 R96"}="2H97 R96";
$resn_exch{"H973 R96"}="3H97 R96"; $resn_exch{"H981 R96"}="1H98 R96"; $resn_exch{"H982 R96"}="2H98 R96";
$resn_exch{"H983 R96"}="3H98 R96"; $resn_exch{"H961 R96"}="1H96 R96"; $resn_exch{"H962 R96"}="2H96 R96";
$resn_exch{" H21 R99"}="1H2  R99"; $resn_exch{" H22 R99"}="2H2  R99"; $resn_exch{" H31 R99"}="1H3  R99";
$resn_exch{"H101 R99"}="1H10 R99"; $resn_exch{"H121 R99"}="1H12 R99"; $resn_exch{"H122 R99"}="2H12 R99";
$resn_exch{"H131 R99"}="1H13 R99"; $resn_exch{"H132 R99"}="2H13 R99"; $resn_exch{"H151 R99"}="1H15 R99";
$resn_exch{"H161 R99"}="1H16 R99"; $resn_exch{"H171 R99"}="1H17 R99"; $resn_exch{"H181 R99"}="1H18 R99";
$resn_exch{"H191 R99"}="1H19 R99"; $resn_exch{"H211 R99"}="1H21 R99"; $resn_exch{"H212 R99"}="2H21 R99";
$resn_exch{"H221 R99"}="1H22 R99"; $resn_exch{"H222 R99"}="2H22 R99"; $resn_exch{"H231 R99"}="1H23 R99";
$resn_exch{"H251 R99"}="1H25 R99"; $resn_exch{"H252 R99"}="2H25 R99"; $resn_exch{"H253 R99"}="3H25 R99";
$resn_exch{"H241 R99"}="1H24 R99"; $resn_exch{"H242 R99"}="2H24 R99"; $resn_exch{"H243 R99"}="3H24 R99";
$resn_exch{"HM23 R9A"}="3HM2 R9A"; $resn_exch{"HM22 R9A"}="2HM2 R9A"; $resn_exch{"HM21 R9A"}="1HM2 R9A";
$resn_exch{"HM13 R9A"}="3HM1 R9A"; $resn_exch{"HM12 R9A"}="2HM1 R9A"; $resn_exch{"HM11 R9A"}="1HM1 R9A";
$resn_exch{"HL91 R9A"}="1HL9 R9A"; $resn_exch{"HL92 R9A"}="2HL9 R9A"; $resn_exch{"HL81 R9A"}="1HL8 R9A";
$resn_exch{"HL82 R9A"}="2HL8 R9A"; $resn_exch{"HL71 R9A"}="1HL7 R9A"; $resn_exch{"HL72 R9A"}="2HL7 R9A";
$resn_exch{"HL61 R9A"}="1HL6 R9A"; $resn_exch{"HL62 R9A"}="2HL6 R9A"; $resn_exch{"HCK1 R9A"}="1HCK R9A";
$resn_exch{"HCJ1 R9A"}="1HCJ R9A"; $resn_exch{"HCE1 R9A"}="1HCE R9A"; $resn_exch{"HCD1 R9A"}="1HCD R9A";
$resn_exch{"HCC1 R9A"}="1HCC R9A"; $resn_exch{"HCB1 R9A"}="1HCB R9A"; $resn_exch{"HBK1 R9A"}="1HBK R9A";
$resn_exch{"HL51 R9A"}="1HL5 R9A"; $resn_exch{"HL52 R9A"}="2HL5 R9A"; $resn_exch{"HBJ1 R9A"}="1HBJ R9A";
$resn_exch{"HBI1 R9A"}="1HBI R9A"; $resn_exch{"HL31 R9A"}="1HL3 R9A"; $resn_exch{"HL32 R9A"}="2HL3 R9A";
$resn_exch{"HL21 R9A"}="1HL2 R9A"; $resn_exch{"HL22 R9A"}="2HL2 R9A"; $resn_exch{"HL41 R9A"}="1HL4 R9A";
$resn_exch{"HL42 R9A"}="2HL4 R9A"; $resn_exch{"HL11 R9A"}="1HL1 R9A"; $resn_exch{"HL12 R9A"}="2HL1 R9A";
$resn_exch{"HBH1 R9A"}="1HBH R9A"; $resn_exch{"HBE1 R9A"}="1HBE R9A"; $resn_exch{"HBD1 R9A"}="1HBD R9A";
$resn_exch{"HBC1 R9A"}="1HBC R9A"; $resn_exch{"HBB1 R9A"}="1HBB R9A"; $resn_exch{"H141 RA2"}="1H14 RA2";
$resn_exch{"H142 RA2"}="2H14 RA2"; $resn_exch{"H151 RA2"}="1H15 RA2"; $resn_exch{"H152 RA2"}="2H15 RA2";
$resn_exch{"H153 RA2"}="3H15 RA2"; $resn_exch{"H211 RA2"}="1H21 RA2"; $resn_exch{"H212 RA2"}="2H21 RA2";
$resn_exch{"H231 RA2"}="1H23 RA2"; $resn_exch{"H232 RA2"}="2H23 RA2"; $resn_exch{"H233 RA2"}="3H23 RA2";
$resn_exch{"H301 RA2"}="1H30 RA2"; $resn_exch{"H302 RA2"}="2H30 RA2"; $resn_exch{"H311 RA2"}="1H31 RA2";
$resn_exch{"H312 RA2"}="2H31 RA2"; $resn_exch{"H381 RA2"}="1H38 RA2"; $resn_exch{"H282 RA2"}="2H28 RA2";
$resn_exch{"H421 RA2"}="1H42 RA2"; $resn_exch{"H422 RA2"}="2H42 RA2"; $resn_exch{"H221 RA4"}="1H22 RA4";
$resn_exch{"H222 RA4"}="2H22 RA4"; $resn_exch{"H201 RA4"}="1H20 RA4"; $resn_exch{"H202 RA4"}="2H20 RA4";
$resn_exch{"H203 RA4"}="3H20 RA4"; $resn_exch{"H191 RA4"}="1H19 RA4"; $resn_exch{"H192 RA4"}="2H19 RA4";
$resn_exch{"H193 RA4"}="3H19 RA4"; $resn_exch{"H151 RA4"}="1H15 RA4"; $resn_exch{"H152 RA4"}="2H15 RA4";
$resn_exch{"H141 RA4"}="1H14 RA4"; $resn_exch{"H142 RA4"}="2H14 RA4"; $resn_exch{"H131 RA4"}="1H13 RA4";
$resn_exch{"H132 RA4"}="2H13 RA4"; $resn_exch{" H81 RA4"}="1H8  RA4"; $resn_exch{" H82 RA4"}="2H8  RA4";
$resn_exch{" H71 RA4"}="1H7  RA4"; $resn_exch{" H72 RA4"}="2H7  RA4"; $resn_exch{" H61 RA4"}="1H6  RA4";
$resn_exch{" H62 RA4"}="2H6  RA4"; $resn_exch{" H51 RA4"}="1H5  RA4"; $resn_exch{" H52 RA4"}="2H5  RA4";
$resn_exch{"HN31 RA4"}="1HN3 RA4"; $resn_exch{"HN32 RA4"}="2HN3 RA4"; $resn_exch{"HN31 RA8"}="1HN3 RA8";
$resn_exch{"HN32 RA8"}="2HN3 RA8"; $resn_exch{" H61 RA8"}="1H6  RA8"; $resn_exch{" H62 RA8"}="2H6  RA8";
$resn_exch{" H71 RA8"}="1H7  RA8"; $resn_exch{" H72 RA8"}="2H7  RA8"; $resn_exch{" H81 RA8"}="1H8  RA8";
$resn_exch{" H82 RA8"}="2H8  RA8"; $resn_exch{" H91 RA8"}="1H9  RA8"; $resn_exch{" H92 RA8"}="2H9  RA8";
$resn_exch{"H271 RA8"}="1H27 RA8"; $resn_exch{"H272 RA8"}="2H27 RA8"; $resn_exch{"H201 RA8"}="1H20 RA8";
$resn_exch{"H202 RA8"}="2H20 RA8"; $resn_exch{"H231 RA8"}="1H23 RA8"; $resn_exch{"H232 RA8"}="2H23 RA8";
$resn_exch{"H233 RA8"}="3H23 RA8"; $resn_exch{"H221 RA8"}="1H22 RA8"; $resn_exch{"H222 RA8"}="2H22 RA8";
$resn_exch{"H223 RA8"}="3H22 RA8"; $resn_exch{"H171 RA8"}="1H17 RA8"; $resn_exch{"H172 RA8"}="2H17 RA8";
$resn_exch{"H131 RA8"}="1H13 RA8"; $resn_exch{"H132 RA8"}="2H13 RA8"; $resn_exch{"H141 RA8"}="1H14 RA8";
$resn_exch{"H142 RA8"}="2H14 RA8"; $resn_exch{"HC61 RAA"}="1HC6 RAA"; $resn_exch{"HC62 RAA"}="2HC6 RAA";
$resn_exch{"HC63 RAA"}="3HC6 RAA"; $resn_exch{"H5'1 RAB"}="1H5' RAB"; $resn_exch{"H5'2 RAB"}="2H5' RAB";
$resn_exch{"HN61 RAB"}="1HN6 RAB"; $resn_exch{"HN62 RAB"}="2HN6 RAB"; $resn_exch{"HC81 RAC"}="1HC8 RAC";
$resn_exch{"HC82 RAC"}="2HC8 RAC"; $resn_exch{"HC83 RAC"}="3HC8 RAC"; $resn_exch{"HC91 RAC"}="1HC9 RAC";
$resn_exch{"HC92 RAC"}="2HC9 RAC"; $resn_exch{"HC93 RAC"}="3HC9 RAC"; $resn_exch{"H111 RAC"}="1H11 RAC";
$resn_exch{"H112 RAC"}="2H11 RAC"; $resn_exch{"H13C RAC"}="CH13 RAC"; $resn_exch{"H14C RAC"}="CH14 RAC";
$resn_exch{"H16C RAC"}="CH16 RAC"; $resn_exch{"H17C RAC"}="CH17 RAC"; $resn_exch{" H31 RAD"}="1H3  RAD";
$resn_exch{" H32 RAD"}="2H3  RAD"; $resn_exch{" H41 RAD"}="1H4  RAD"; $resn_exch{" H42 RAD"}="2H4  RAD";
$resn_exch{" H51 RAD"}="1H5  RAD"; $resn_exch{" H52 RAD"}="2H5  RAD"; $resn_exch{" H61 RAD"}="1H6  RAD";
$resn_exch{" H62 RAD"}="2H6  RAD"; $resn_exch{"H111 RAD"}="1H11 RAD"; $resn_exch{"H112 RAD"}="2H11 RAD";
$resn_exch{"H121 RAD"}="1H12 RAD"; $resn_exch{"H122 RAD"}="2H12 RAD"; $resn_exch{"H141 RAD"}="1H14 RAD";
$resn_exch{"H142 RAD"}="2H14 RAD"; $resn_exch{"H231 RAD"}="1H23 RAD"; $resn_exch{"H232 RAD"}="2H23 RAD";
$resn_exch{"H321 RAD"}="1H32 RAD"; $resn_exch{"H322 RAD"}="2H32 RAD"; $resn_exch{"H351 RAD"}="1H35 RAD";
$resn_exch{"H352 RAD"}="2H35 RAD"; $resn_exch{"H371 RAD"}="1H37 RAD"; $resn_exch{"H372 RAD"}="2H37 RAD";
$resn_exch{"H401 RAD"}="1H40 RAD"; $resn_exch{"H402 RAD"}="2H40 RAD"; $resn_exch{"H411 RAD"}="1H41 RAD";
$resn_exch{"H412 RAD"}="2H41 RAD"; $resn_exch{"H421 RAD"}="1H42 RAD"; $resn_exch{"H422 RAD"}="2H42 RAD";
$resn_exch{"H423 RAD"}="3H42 RAD"; $resn_exch{"H431 RAD"}="1H43 RAD"; $resn_exch{"H432 RAD"}="2H43 RAD";
$resn_exch{"H433 RAD"}="3H43 RAD"; $resn_exch{"H441 RAD"}="1H44 RAD"; $resn_exch{"H442 RAD"}="2H44 RAD";
$resn_exch{"H443 RAD"}="3H44 RAD"; $resn_exch{"H451 RAD"}="1H45 RAD"; $resn_exch{"H452 RAD"}="2H45 RAD";
$resn_exch{"H453 RAD"}="3H45 RAD"; $resn_exch{"H461 RAD"}="1H46 RAD"; $resn_exch{"H462 RAD"}="2H46 RAD";
$resn_exch{"H463 RAD"}="3H46 RAD"; $resn_exch{"H471 RAD"}="1H47 RAD"; $resn_exch{"H472 RAD"}="2H47 RAD";
$resn_exch{"H473 RAD"}="3H47 RAD"; $resn_exch{"H481 RAD"}="1H48 RAD"; $resn_exch{"H482 RAD"}="2H48 RAD";
$resn_exch{"H483 RAD"}="3H48 RAD"; $resn_exch{"H491 RAD"}="1H49 RAD"; $resn_exch{"H492 RAD"}="2H49 RAD";
$resn_exch{"H501 RAD"}="1H50 RAD"; $resn_exch{"H502 RAD"}="2H50 RAD"; $resn_exch{"H503 RAD"}="3H50 RAD";
$resn_exch{"H511 RAD"}="1H51 RAD"; $resn_exch{"H512 RAD"}="2H51 RAD"; $resn_exch{"H513 RAD"}="3H51 RAD";
$resn_exch{"H521 RAD"}="1H52 RAD"; $resn_exch{"H522 RAD"}="2H52 RAD"; $resn_exch{"H523 RAD"}="3H52 RAD";
$resn_exch{"HC21 RAE"}="1HC2 RAE"; $resn_exch{"HC22 RAE"}="2HC2 RAE"; $resn_exch{"HC61 RAE"}="1HC6 RAE";
$resn_exch{"HC62 RAE"}="2HC6 RAE"; $resn_exch{"HC63 RAE"}="3HC6 RAE"; $resn_exch{"H261 RAG"}="1H26 RAG";
$resn_exch{"H262 RAG"}="2H26 RAG"; $resn_exch{"H263 RAG"}="3H26 RAG"; $resn_exch{"H201 RAG"}="1H20 RAG";
$resn_exch{"H202 RAG"}="2H20 RAG"; $resn_exch{"H191 RAG"}="1H19 RAG"; $resn_exch{"H192 RAG"}="2H19 RAG";
$resn_exch{"H181 RAG"}="1H18 RAG"; $resn_exch{"H182 RAG"}="2H18 RAG"; $resn_exch{"H311 RAG"}="1H31 RAG";
$resn_exch{"H312 RAG"}="2H31 RAG"; $resn_exch{"H313 RAG"}="3H31 RAG"; $resn_exch{"H121 RAG"}="1H12 RAG";
$resn_exch{"H122 RAG"}="2H12 RAG"; $resn_exch{"H123 RAG"}="3H12 RAG"; $resn_exch{"H561 RAG"}="1H56 RAG";
$resn_exch{"H562 RAG"}="2H56 RAG"; $resn_exch{"H481 RAG"}="1H48 RAG"; $resn_exch{"H482 RAG"}="2H48 RAG";
$resn_exch{"H391 RAG"}="1H39 RAG"; $resn_exch{"H392 RAG"}="2H39 RAG"; $resn_exch{"H141 RAI"}="1H14 RAI";
$resn_exch{"H142 RAI"}="2H14 RAI"; $resn_exch{"H171 RAI"}="1H17 RAI"; $resn_exch{"H172 RAI"}="2H17 RAI";
$resn_exch{"H181 RAI"}="1H18 RAI"; $resn_exch{"H182 RAI"}="2H18 RAI"; $resn_exch{"H241 RAI"}="1H24 RAI";
$resn_exch{"H242 RAI"}="2H24 RAI"; $resn_exch{"H281 RAI"}="1H28 RAI"; $resn_exch{"H282 RAI"}="2H28 RAI";
$resn_exch{"H281 RAJ"}="1H28 RAJ"; $resn_exch{"H282 RAJ"}="2H28 RAJ"; $resn_exch{"H283 RAJ"}="3H28 RAJ";
$resn_exch{"H291 RAJ"}="1H29 RAJ"; $resn_exch{"H292 RAJ"}="2H29 RAJ"; $resn_exch{"H293 RAJ"}="3H29 RAJ";
$resn_exch{"H271 RAJ"}="1H27 RAJ"; $resn_exch{"H272 RAJ"}="2H27 RAJ"; $resn_exch{"H261 RAJ"}="1H26 RAJ";
$resn_exch{"H262 RAJ"}="2H26 RAJ"; $resn_exch{"H251 RAJ"}="1H25 RAJ"; $resn_exch{"H252 RAJ"}="2H25 RAJ";
$resn_exch{"H161 RAJ"}="1H16 RAJ"; $resn_exch{"H162 RAJ"}="2H16 RAJ"; $resn_exch{"H163 RAJ"}="3H16 RAJ";
$resn_exch{"H241 RAL"}="1H24 RAL"; $resn_exch{"H242 RAL"}="2H24 RAL"; $resn_exch{"H251 RAL"}="1H25 RAL";
$resn_exch{"H252 RAL"}="2H25 RAL"; $resn_exch{"H271 RAL"}="1H27 RAL"; $resn_exch{"H272 RAL"}="2H27 RAL";
$resn_exch{"H281 RAL"}="1H28 RAL"; $resn_exch{"H282 RAL"}="2H28 RAL"; $resn_exch{"H291 RAL"}="1H29 RAL";
$resn_exch{"H292 RAL"}="2H29 RAL"; $resn_exch{"H301 RAL"}="1H30 RAL"; $resn_exch{"H302 RAL"}="2H30 RAL";
$resn_exch{"H311 RAL"}="1H31 RAL"; $resn_exch{"H312 RAL"}="2H31 RAL"; $resn_exch{" H61 RAM"}="1H6  RAM";
$resn_exch{" H62 RAM"}="2H6  RAM"; $resn_exch{" H63 RAM"}="3H6  RAM"; $resn_exch{" H81 RAN"}="1H8  RAN";
$resn_exch{" H82 RAN"}="2H8  RAN"; $resn_exch{" H83 RAN"}="3H8  RAN"; $resn_exch{" H61 RAN"}="1H6  RAN";
$resn_exch{" H62 RAN"}="2H6  RAN"; $resn_exch{"HC71 RAO"}="1HC7 RAO"; $resn_exch{"HC72 RAO"}="2HC7 RAO";
$resn_exch{"HC73 RAO"}="3HC7 RAO"; $resn_exch{"HC61 RAO"}="1HC6 RAO"; $resn_exch{"HC62 RAO"}="2HC6 RAO";
$resn_exch{"HC63 RAO"}="3HC6 RAO"; $resn_exch{"H31A RAP"}="1H3  RAP"; $resn_exch{" H32 RAP"}="2H3  RAP";
$resn_exch{" H41 RAP"}="1H4  RAP"; $resn_exch{" H42 RAP"}="2H4  RAP"; $resn_exch{" H51 RAP"}="1H5  RAP";
$resn_exch{" H52 RAP"}="2H5  RAP"; $resn_exch{" H61 RAP"}="1H6  RAP"; $resn_exch{" H62 RAP"}="2H6  RAP";
$resn_exch{"H121 RAP"}="1H12 RAP"; $resn_exch{"H122 RAP"}="2H12 RAP"; $resn_exch{"H131 RAP"}="1H13 RAP";
$resn_exch{"H132 RAP"}="2H13 RAP"; $resn_exch{"H151 RAP"}="1H15 RAP"; $resn_exch{"H152 RAP"}="2H15 RAP";
$resn_exch{"H241 RAP"}="1H24 RAP"; $resn_exch{"H242 RAP"}="2H24 RAP"; $resn_exch{"H331 RAP"}="1H33 RAP";
$resn_exch{"H332 RAP"}="2H33 RAP"; $resn_exch{"H361 RAP"}="1H36 RAP"; $resn_exch{"H362 RAP"}="2H36 RAP";
$resn_exch{"H381 RAP"}="1H38 RAP"; $resn_exch{"H382 RAP"}="2H38 RAP"; $resn_exch{"H411 RAP"}="1H41 RAP";
$resn_exch{"H412 RAP"}="2H41 RAP"; $resn_exch{"H421 RAP"}="1H42 RAP"; $resn_exch{"H422 RAP"}="2H42 RAP";
$resn_exch{"H431 RAP"}="1H43 RAP"; $resn_exch{"H432 RAP"}="2H43 RAP"; $resn_exch{"H433 RAP"}="3H43 RAP";
$resn_exch{"H441 RAP"}="1H44 RAP"; $resn_exch{"H442 RAP"}="2H44 RAP"; $resn_exch{"H443 RAP"}="3H44 RAP";
$resn_exch{"H451 RAP"}="1H45 RAP"; $resn_exch{"H452 RAP"}="2H45 RAP"; $resn_exch{"H453 RAP"}="3H45 RAP";
$resn_exch{"H461 RAP"}="1H46 RAP"; $resn_exch{"H462 RAP"}="2H46 RAP"; $resn_exch{"H463 RAP"}="3H46 RAP";
$resn_exch{"H471 RAP"}="1H47 RAP"; $resn_exch{"H472 RAP"}="2H47 RAP"; $resn_exch{"H473 RAP"}="3H47 RAP";
$resn_exch{"H481 RAP"}="1H48 RAP"; $resn_exch{"H482 RAP"}="2H48 RAP"; $resn_exch{"H483 RAP"}="3H48 RAP";
$resn_exch{"H491 RAP"}="1H49 RAP"; $resn_exch{"H492 RAP"}="2H49 RAP"; $resn_exch{"H493 RAP"}="3H49 RAP";
$resn_exch{"H501 RAP"}="1H50 RAP"; $resn_exch{"H502 RAP"}="2H50 RAP"; $resn_exch{"H503 RAP"}="3H50 RAP";
$resn_exch{"H511 RAP"}="1H51 RAP"; $resn_exch{"H512 RAP"}="2H51 RAP"; $resn_exch{"H513 RAP"}="3H51 RAP";
$resn_exch{"H521 RAP"}="1H52 RAP"; $resn_exch{"H522 RAP"}="2H52 RAP"; $resn_exch{"H523 RAP"}="3H52 RAP";
$resn_exch{"H111 RAS"}="1H11 RAS"; $resn_exch{"H112 RAS"}="2H11 RAS"; $resn_exch{" H11 RAS"}="1H1  RAS";
$resn_exch{" H12 RAS"}="2H1  RAS"; $resn_exch{" H21 RAS"}="1H2  RAS"; $resn_exch{" H22 RAS"}="2H2  RAS";
$resn_exch{"H021 RAZ"}="1H02 RAZ"; $resn_exch{"H022 RAZ"}="2H02 RAZ"; $resn_exch{"H031 RAZ"}="1H03 RAZ";
$resn_exch{"H032 RAZ"}="2H03 RAZ"; $resn_exch{"H071 RAZ"}="1H07 RAZ"; $resn_exch{"H072 RAZ"}="2H07 RAZ";
$resn_exch{"H101 RAZ"}="1H10 RAZ"; $resn_exch{"H102 RAZ"}="2H10 RAZ"; $resn_exch{"H111 RAZ"}="1H11 RAZ";
$resn_exch{"H112 RAZ"}="2H11 RAZ"; $resn_exch{"H171 RAZ"}="1H17 RAZ"; $resn_exch{"H172 RAZ"}="2H17 RAZ";
$resn_exch{"H173 RAZ"}="3H17 RAZ"; $resn_exch{"H181 RAZ"}="1H18 RAZ"; $resn_exch{"H182 RAZ"}="2H18 RAZ";
$resn_exch{"H183 RAZ"}="3H18 RAZ"; $resn_exch{"H191 RAZ"}="1H19 RAZ"; $resn_exch{"H192 RAZ"}="2H19 RAZ";
$resn_exch{"H193 RAZ"}="3H19 RAZ"; $resn_exch{"H321 RAZ"}="1H32 RAZ"; $resn_exch{"H322 RAZ"}="2H32 RAZ";
$resn_exch{"H323 RAZ"}="3H32 RAZ"; $resn_exch{"H201 RB1"}="1H20 RB1"; $resn_exch{"H202 RB1"}="2H20 RB1";
$resn_exch{"H203 RB1"}="3H20 RB1"; $resn_exch{"H211 RB1"}="1H21 RB1"; $resn_exch{"H212 RB1"}="2H21 RB1";
$resn_exch{"H213 RB1"}="3H21 RB1"; $resn_exch{"H221 RB1"}="1H22 RB1"; $resn_exch{"H222 RB1"}="2H22 RB1";
$resn_exch{"H223 RB1"}="3H22 RB1"; $resn_exch{"H101 RB1"}="1H10 RB1"; $resn_exch{"H102 RB1"}="2H10 RB1";
$resn_exch{"H111 RB1"}="1H11 RB1"; $resn_exch{"H112 RB1"}="2H11 RB1"; $resn_exch{"H111 RB2"}="1H11 RB2";
$resn_exch{"H112 RB2"}="2H11 RB2"; $resn_exch{"H113 RB2"}="3H11 RB2"; $resn_exch{" H41 RB2"}="1H4  RB2";
$resn_exch{" H42 RB2"}="2H4  RB2"; $resn_exch{" H21 RB2"}="1H2  RB2"; $resn_exch{" H22 RB2"}="2H2  RB2";
$resn_exch{" H11 RB2"}="1H1  RB2"; $resn_exch{"H12A RB2"}="2H1  RB2"; $resn_exch{"H2A1 RB3"}="1H2A RB3";
$resn_exch{"H2A2 RB3"}="2H2A RB3"; $resn_exch{"H4A1 RB3"}="1H4A RB3"; $resn_exch{"H4A2 RB3"}="2H4A RB3";
$resn_exch{"H4A3 RB3"}="3H4A RB3"; $resn_exch{"H2B1 RB3"}="1H2B RB3"; $resn_exch{"H2B2 RB3"}="2H2B RB3";
$resn_exch{"H4B1 RB3"}="1H4B RB3"; $resn_exch{"H4B2 RB3"}="2H4B RB3"; $resn_exch{"H4B3 RB3"}="3H4B RB3";
$resn_exch{"H2C1 RB3"}="1H2C RB3"; $resn_exch{"H2C2 RB3"}="2H2C RB3"; $resn_exch{"H4C1 RB3"}="1H4C RB3";
$resn_exch{"H4C2 RB3"}="2H4C RB3"; $resn_exch{"H4C3 RB3"}="3H4C RB3"; $resn_exch{" H11 RB3"}="1H1  RB3";
$resn_exch{" H12 RB3"}="2H1  RB3"; $resn_exch{" H13 RB3"}="3H1  RB3"; $resn_exch{"H111 RBC"}="1H11 RBC";
$resn_exch{"H112 RBC"}="2H11 RBC"; $resn_exch{"H121 RBC"}="1H12 RBC"; $resn_exch{"H122 RBC"}="2H12 RBC";
$resn_exch{" H71 RBC"}="1H7  RBC"; $resn_exch{" H72 RBC"}="2H7  RBC"; $resn_exch{"H211 RBC"}="1H21 RBC";
$resn_exch{"H212 RBC"}="2H21 RBC"; $resn_exch{"H411 RBC"}="1H41 RBC"; $resn_exch{"H412 RBC"}="2H41 RBC";
$resn_exch{"H413 RBC"}="3H41 RBC"; $resn_exch{"H1D1 RBC"}="1H1D RBC"; $resn_exch{"H1D2 RBC"}="2H1D RBC";
$resn_exch{" H41 RBE"}="1H4  RBE"; $resn_exch{" H42 RBE"}="2H4  RBE"; $resn_exch{" H61 RBE"}="1H6  RBE";
$resn_exch{" H62 RBE"}="2H6  RBE"; $resn_exch{" H71 RBE"}="1H7  RBE"; $resn_exch{" H72 RBE"}="2H7  RBE";
$resn_exch{"HN11 RBE"}="1HN1 RBE"; $resn_exch{"HN12 RBE"}="2HN1 RBE"; $resn_exch{"HC71 RBF"}="1HC7 RBF";
$resn_exch{"HC72 RBF"}="2HC7 RBF"; $resn_exch{"HC73 RBF"}="3HC7 RBF"; $resn_exch{"HC81 RBF"}="1HC8 RBF";
$resn_exch{"HC82 RBF"}="2HC8 RBF"; $resn_exch{"HC83 RBF"}="3HC8 RBF"; $resn_exch{"HC11 RBF"}="1HC1 RBF";
$resn_exch{"HC12 RBF"}="2HC1 RBF"; $resn_exch{"HC2' RBF"}="HC2* RBF"; $resn_exch{"HO2' RBF"}="HO2* RBF";
$resn_exch{"HC3' RBF"}="HC3* RBF"; $resn_exch{"HO3' RBF"}="HO3* RBF"; $resn_exch{"HC4' RBF"}="HC4* RBF";
$resn_exch{"HO4' RBF"}="HO4* RBF"; $resn_exch{"HC51 RBF"}="1HC5 RBF"; $resn_exch{"HC52 RBF"}="2HC5 RBF";
$resn_exch{"HO5' RBF"}="HO5* RBF"; $resn_exch{"H121 RBP"}="1H12 RBP"; $resn_exch{"H122 RBP"}="2H12 RBP";
$resn_exch{"H111 RBP"}="1H11 RBP"; $resn_exch{"H112 RBP"}="2H11 RBP"; $resn_exch{"H151 RBP"}="1H15 RBP";
$resn_exch{"H152 RBP"}="2H15 RBP"; $resn_exch{"H141 RBP"}="1H14 RBP"; $resn_exch{"H142 RBP"}="2H14 RBP";
$resn_exch{"H111 RBS"}="1H11 RBS"; $resn_exch{"H511 RBS"}="1H51 RBS"; $resn_exch{"H512 RBS"}="2H51 RBS";
$resn_exch{"H131 RBT"}="1H13 RBT"; $resn_exch{"H132 RBT"}="2H13 RBT"; $resn_exch{"H133 RBT"}="3H13 RBT";
$resn_exch{"H141 RBT"}="1H14 RBT"; $resn_exch{"H142 RBT"}="2H14 RBT"; $resn_exch{"H143 RBT"}="3H14 RBT";
$resn_exch{"H301 RBT"}="1H30 RBT"; $resn_exch{"H302 RBT"}="2H30 RBT"; $resn_exch{"H303 RBT"}="3H30 RBT";
$resn_exch{"H311 RBT"}="1H31 RBT"; $resn_exch{"H312 RBT"}="2H31 RBT"; $resn_exch{"H313 RBT"}="3H31 RBT";
$resn_exch{"H321 RBT"}="1H32 RBT"; $resn_exch{"H322 RBT"}="2H32 RBT"; $resn_exch{"H323 RBT"}="3H32 RBT";
$resn_exch{"H331 RBT"}="1H33 RBT"; $resn_exch{"H332 RBT"}="2H33 RBT"; $resn_exch{"H333 RBT"}="3H33 RBT";
$resn_exch{"H341 RBT"}="1H34 RBT"; $resn_exch{"H342 RBT"}="2H34 RBT"; $resn_exch{"H343 RBT"}="3H34 RBT";
$resn_exch{"H361 RBT"}="1H36 RBT"; $resn_exch{"H362 RBT"}="2H36 RBT"; $resn_exch{"H363 RBT"}="3H36 RBT";
$resn_exch{"H371 RBT"}="1H37 RBT"; $resn_exch{"H372 RBT"}="2H37 RBT"; $resn_exch{"H373 RBT"}="3H37 RBT";
$resn_exch{"H391 RBT"}="1H39 RBT"; $resn_exch{"H392 RBT"}="2H39 RBT"; $resn_exch{"H401 RBT"}="1H40 RBT";
$resn_exch{"H402 RBT"}="2H40 RBT"; $resn_exch{"H411 RBT"}="1H41 RBT"; $resn_exch{"H412 RBT"}="2H41 RBT";
$resn_exch{"H421 RBT"}="1H42 RBT"; $resn_exch{"H422 RBT"}="2H42 RBT"; $resn_exch{"H431 RBT"}="1H43 RBT";
$resn_exch{"H432 RBT"}="2H43 RBT"; $resn_exch{"H451 RBT"}="1H45 RBT"; $resn_exch{"H452 RBT"}="2H45 RBT";
$resn_exch{"H453 RBT"}="3H45 RBT"; $resn_exch{"H461 RBT"}="1H46 RBT"; $resn_exch{"H462 RBT"}="2H46 RBT";
$resn_exch{"H463 RBT"}="3H46 RBT"; $resn_exch{"H5'1 RBY"}="1H5* RBY"; $resn_exch{"H5'2 RBY"}="2H5* RBY";
$resn_exch{" H4' RBY"}=" H4* RBY"; $resn_exch{" H3' RBY"}=" H3* RBY"; $resn_exch{"HO3' RBY"}="HO3* RBY";
$resn_exch{" H2' RBY"}=" H2* RBY"; $resn_exch{"HO2' RBY"}="HO2* RBY"; $resn_exch{" H1' RBY"}=" H1* RBY";
$resn_exch{"HN61 RBY"}="1HN6 RBY"; $resn_exch{"HN62 RBY"}="2HN6 RBY"; $resn_exch{" HX1 RBY"}="1HX  RBY";
$resn_exch{" HX2 RBY"}="2HX  RBY"; $resn_exch{"HR51 RBY"}="1HR5 RBY"; $resn_exch{"HR52 RBY"}="2HR5 RBY";
$resn_exch{" H81 RBZ"}="1H8  RBZ"; $resn_exch{" H82 RBZ"}="2H8  RBZ"; $resn_exch{" H83 RBZ"}="3H8  RBZ";
$resn_exch{" H91 RBZ"}="1H9  RBZ"; $resn_exch{" H92 RBZ"}="2H9  RBZ"; $resn_exch{" H93 RBZ"}="3H9  RBZ";
$resn_exch{"HOP2 RBZ"}="2HOP RBZ"; $resn_exch{"HOP3 RBZ"}="3HOP RBZ"; $resn_exch{"H5'1 RBZ"}="1H5* RBZ";
$resn_exch{"H5'2 RBZ"}="2H5* RBZ"; $resn_exch{" H4' RBZ"}=" H4* RBZ"; $resn_exch{" H3' RBZ"}=" H3* RBZ";
$resn_exch{"HO'3 RBZ"}="3HO* RBZ"; $resn_exch{" H2' RBZ"}=" H2* RBZ"; $resn_exch{"HO'2 RBZ"}="2HO* RBZ";
$resn_exch{" H1' RBZ"}=" H1* RBZ"; $resn_exch{" H61 RC1"}="1H6  RC1"; $resn_exch{" H62 RC1"}="2H6  RC1";
$resn_exch{" H91 RC1"}="1H9  RC1"; $resn_exch{" H92 RC1"}="2H9  RC1"; $resn_exch{" H93 RC1"}="3H9  RC1";
$resn_exch{"HB21 RC7"}="1HB2 RC7"; $resn_exch{"HB22 RC7"}="2HB2 RC7"; $resn_exch{"HA31 RC7"}="1HA3 RC7";
$resn_exch{"HA32 RC7"}="2HA3 RC7"; $resn_exch{"HA11 RC7"}="1HA1 RC7"; $resn_exch{"HA12 RC7"}="2HA1 RC7";
$resn_exch{"HB11 RC7"}="1HB1 RC7"; $resn_exch{"HB12 RC7"}="2HB1 RC7"; $resn_exch{" H43 RCA"}="3H4  RCA";
$resn_exch{" H42 RCA"}="2H4  RCA"; $resn_exch{" H41 RCA"}="1H4  RCA"; $resn_exch{"H122 RCA"}="2H12 RCA";
$resn_exch{"H121 RCA"}="1H12 RCA"; $resn_exch{" H62 RCA"}="2H6  RCA"; $resn_exch{" H61 RCA"}="1H6  RCA";
$resn_exch{" H83 RCA"}="3H8  RCA"; $resn_exch{" H82 RCA"}="2H8  RCA"; $resn_exch{" H81 RCA"}="1H8  RCA";
$resn_exch{" H21 RCL"}="1H2  RCL"; $resn_exch{" H22 RCL"}="2H2  RCL"; $resn_exch{" H31 RCL"}="1H3  RCL";
$resn_exch{" H32 RCL"}="2H3  RCL"; $resn_exch{" H41 RCL"}="1H4  RCL"; $resn_exch{" H42 RCL"}="2H4  RCL";
$resn_exch{" H51 RCL"}="1H5  RCL"; $resn_exch{" H52 RCL"}="2H5  RCL"; $resn_exch{" H61 RCL"}="1H6  RCL";
$resn_exch{" H62 RCL"}="2H6  RCL"; $resn_exch{" H71 RCL"}="1H7  RCL"; $resn_exch{" H72 RCL"}="2H7  RCL";
$resn_exch{" H81 RCL"}="1H8  RCL"; $resn_exch{" H82 RCL"}="2H8  RCL"; $resn_exch{"H111 RCL"}="1H11 RCL";
$resn_exch{"H112 RCL"}="2H11 RCL"; $resn_exch{"H131 RCL"}="1H13 RCL"; $resn_exch{"H132 RCL"}="2H13 RCL";
$resn_exch{"H141 RCL"}="1H14 RCL"; $resn_exch{"H142 RCL"}="2H14 RCL"; $resn_exch{"H151 RCL"}="1H15 RCL";
$resn_exch{"H152 RCL"}="2H15 RCL"; $resn_exch{"H161 RCL"}="1H16 RCL"; $resn_exch{"H162 RCL"}="2H16 RCL";
$resn_exch{"H171 RCL"}="1H17 RCL"; $resn_exch{"H172 RCL"}="2H17 RCL"; $resn_exch{"H181 RCL"}="1H18 RCL";
$resn_exch{"H182 RCL"}="2H18 RCL"; $resn_exch{"H183 RCL"}="3H18 RCL"; $resn_exch{"H291 RCP"}="1H29 RCP";
$resn_exch{"H292 RCP"}="2H29 RCP"; $resn_exch{"H301 RCP"}="1H30 RCP"; $resn_exch{"H302 RCP"}="2H30 RCP";
$resn_exch{"H281 RCP"}="1H28 RCP"; $resn_exch{"H282 RCP"}="2H28 RCP"; $resn_exch{"H271 RCP"}="1H27 RCP";
$resn_exch{"H272 RCP"}="2H27 RCP"; $resn_exch{"H241 RCP"}="1H24 RCP"; $resn_exch{"H242 RCP"}="2H24 RCP";
$resn_exch{"H231 RCP"}="1H23 RCP"; $resn_exch{"H232 RCP"}="2H23 RCP"; $resn_exch{"H221 RCP"}="1H22 RCP";
$resn_exch{"H222 RCP"}="2H22 RCP"; $resn_exch{"H211 RCP"}="1H21 RCP"; $resn_exch{"H212 RCP"}="2H21 RCP";
$resn_exch{"H191 RCP"}="1H19 RCP"; $resn_exch{"H192 RCP"}="2H19 RCP"; $resn_exch{"H171 RCP"}="1H17 RCP";
$resn_exch{"H172 RCP"}="2H17 RCP"; $resn_exch{"H181 RCP"}="1H18 RCP"; $resn_exch{"H182 RCP"}="2H18 RCP";
$resn_exch{"H161 RCP"}="1H16 RCP"; $resn_exch{"H162 RCP"}="2H16 RCP"; $resn_exch{"H1C1 RCY"}="1H1C RCY";
$resn_exch{"H1C2 RCY"}="2H1C RCY"; $resn_exch{"H1C3 RCY"}="3H1C RCY"; $resn_exch{"H1V1 RCY"}="1H1V RCY";
$resn_exch{"H1V2 RCY"}="2H1V RCY"; $resn_exch{"H1V3 RCY"}="3H1V RCY"; $resn_exch{"H1Y1 RCY"}="1H1Y RCY";
$resn_exch{"H1Y2 RCY"}="2H1Y RCY"; $resn_exch{"H1Y3 RCY"}="3H1Y RCY"; $resn_exch{"H1Z1 RCY"}="1H1Z RCY";
$resn_exch{"H1Z2 RCY"}="2H1Z RCY"; $resn_exch{"H1Z3 RCY"}="3H1Z RCY"; $resn_exch{"H1M1 RCY"}="1H1M RCY";
$resn_exch{"H1M2 RCY"}="2H1M RCY"; $resn_exch{"H1L1 RCY"}="1H1L RCY"; $resn_exch{"H1L2 RCY"}="2H1L RCY";
$resn_exch{" HB1 RCY"}="1HB  RCY"; $resn_exch{" HB2 RCY"}="2HB  RCY"; $resn_exch{" HN1 RCY"}="1HN  RCY";
$resn_exch{" HN2 RCY"}="2HN  RCY"; $resn_exch{" H71 RDA"}="1H7  RDA"; $resn_exch{" H72 RDA"}="2H7  RDA";
$resn_exch{" H73 RDA"}="3H7  RDA"; $resn_exch{" H91 RDA"}="1H9  RDA"; $resn_exch{" H92 RDA"}="2H9  RDA";
$resn_exch{"H101 RDA"}="1H10 RDA"; $resn_exch{"H102 RDA"}="2H10 RDA"; $resn_exch{"H161 RDA"}="1H16 RDA";
$resn_exch{"H162 RDA"}="2H16 RDA"; $resn_exch{"H163 RDA"}="3H16 RDA"; $resn_exch{" H81 RDC"}="1H8  RDC";
$resn_exch{" H82 RDC"}="2H8  RDC"; $resn_exch{"H161 RDC"}="1H16 RDC"; $resn_exch{"H162 RDC"}="2H16 RDC";
$resn_exch{"H181 RDC"}="1H18 RDC"; $resn_exch{"H182 RDC"}="2H18 RDC"; $resn_exch{"H183 RDC"}="3H18 RDC";
$resn_exch{"HAC1 RDE"}="1HAC RDE"; $resn_exch{"HAC2 RDE"}="2HAC RDE"; $resn_exch{"HAC3 RDE"}="3HAC RDE";
$resn_exch{"HAA1 RDE"}="1HAA RDE"; $resn_exch{"HAA2 RDE"}="2HAA RDE"; $resn_exch{"HAA3 RDE"}="3HAA RDE";
$resn_exch{"HAJ1 RDE"}="1HAJ RDE"; $resn_exch{"HAB1 RDE"}="1HAB RDE"; $resn_exch{"HAB2 RDE"}="2HAB RDE";
$resn_exch{"HAB3 RDE"}="3HAB RDE"; $resn_exch{"HAM1 RDE"}="1HAM RDE"; $resn_exch{"HAM2 RDE"}="2HAM RDE";
$resn_exch{"HAL1 RDE"}="1HAL RDE"; $resn_exch{"HAL2 RDE"}="2HAL RDE"; $resn_exch{"HAK1 RDE"}="1HAK RDE";
$resn_exch{"HAI1 RDE"}="1HAI RDE"; $resn_exch{"HD11 RDF"}="1HD1 RDF"; $resn_exch{"HD12 RDF"}="2HD1 RDF";
$resn_exch{"HD13 RDF"}="3HD1 RDF"; $resn_exch{"HD21 RDF"}="1HD2 RDF"; $resn_exch{"HD22 RDF"}="2HD2 RDF";
$resn_exch{"HD23 RDF"}="3HD2 RDF"; $resn_exch{"HB01 RDF"}="1HB0 RDF"; $resn_exch{"HB02 RDF"}="2HB0 RDF";
$resn_exch{" H41 RDI"}="1H4  RDI"; $resn_exch{" H42 RDI"}="2H4  RDI"; $resn_exch{" H81 RDI"}="1H8  RDI";
$resn_exch{" H82 RDI"}="2H8  RDI"; $resn_exch{"H101 RDI"}="1H10 RDI"; $resn_exch{"H102 RDI"}="2H10 RDI";
$resn_exch{"H111 RDI"}="1H11 RDI"; $resn_exch{"H112 RDI"}="2H11 RDI"; $resn_exch{"H121 RDI"}="1H12 RDI";
$resn_exch{"H122 RDI"}="2H12 RDI"; $resn_exch{"H131 RDI"}="1H13 RDI"; $resn_exch{"H132 RDI"}="2H13 RDI";
$resn_exch{"H161 RDI"}="1H16 RDI"; $resn_exch{"H162 RDI"}="2H16 RDI"; $resn_exch{"H181 RDI"}="1H18 RDI";
$resn_exch{"H182 RDI"}="2H18 RDI"; $resn_exch{"H183 RDI"}="3H18 RDI"; $resn_exch{"H15O RDL"}="OH15 RDL";
$resn_exch{"H151 RDL"}="1H15 RDL"; $resn_exch{"H152 RDL"}="2H15 RDL"; $resn_exch{"H141 RDL"}="1H14 RDL";
$resn_exch{"H14O RDL"}="OH14 RDL"; $resn_exch{"H131 RDL"}="1H13 RDL"; $resn_exch{"H13O RDL"}="OH13 RDL";
$resn_exch{"H121 RDL"}="1H12 RDL"; $resn_exch{"H12O RDL"}="OH12 RDL"; $resn_exch{"H111 RDL"}="1H11 RDL";
$resn_exch{"H112 RDL"}="2H11 RDL"; $resn_exch{"HOP2 RDP"}="2HOP RDP"; $resn_exch{"HOP3 RDP"}="3HOP RDP";
$resn_exch{" H1' RDP"}=" H1* RDP"; $resn_exch{" H2' RDP"}=" H2* RDP"; $resn_exch{"HO2' RDP"}="*HO2 RDP";
$resn_exch{" H3' RDP"}=" H3* RDP"; $resn_exch{"HO3' RDP"}="*HO3 RDP"; $resn_exch{" H4' RDP"}=" H4* RDP";
$resn_exch{"H5'1 RDP"}="1H5* RDP"; $resn_exch{"H5'2 RDP"}="2H5* RDP"; $resn_exch{"HO5' RDP"}="*HO5 RDP";
$resn_exch{"H101 RDR"}="1H10 RDR"; $resn_exch{"H102 RDR"}="2H10 RDR"; $resn_exch{" H61 RDR"}="1H6  RDR";
$resn_exch{" H62 RDR"}="2H6  RDR"; $resn_exch{" H41 RDR"}="1H4  RDR"; $resn_exch{" H42 RDR"}="2H4  RDR";
$resn_exch{"H171 RDR"}="1H17 RDR"; $resn_exch{"H172 RDR"}="2H17 RDR"; $resn_exch{"H191 RDR"}="1H19 RDR";
$resn_exch{"H192 RDR"}="2H19 RDR"; $resn_exch{"H131 RDR"}="1H13 RDR"; $resn_exch{"H132 RDR"}="2H13 RDR";
$resn_exch{" H21 RE9"}="1H2  RE9"; $resn_exch{" H22 RE9"}="2H2  RE9"; $resn_exch{" H31 RE9"}="1H3  RE9";
$resn_exch{" H32 RE9"}="2H3  RE9"; $resn_exch{" H41 RE9"}="1H4  RE9"; $resn_exch{" H42 RE9"}="2H4  RE9";
$resn_exch{" H81 RE9"}="1H8  RE9"; $resn_exch{" H82 RE9"}="2H8  RE9"; $resn_exch{"H101 RE9"}="1H10 RE9";
$resn_exch{"H102 RE9"}="2H10 RE9"; $resn_exch{"H161 RE9"}="1H16 RE9"; $resn_exch{"H162 RE9"}="2H16 RE9";
$resn_exch{"H163 RE9"}="3H16 RE9"; $resn_exch{"H171 RE9"}="1H17 RE9"; $resn_exch{"H172 RE9"}="2H17 RE9";
$resn_exch{"H173 RE9"}="3H17 RE9"; $resn_exch{"H181 RE9"}="1H18 RE9"; $resn_exch{"H182 RE9"}="2H18 RE9";
$resn_exch{"H191 RE9"}="1H19 RE9"; $resn_exch{"H192 RE9"}="2H19 RE9"; $resn_exch{"H193 RE9"}="3H19 RE9";
$resn_exch{"H201 RE9"}="1H20 RE9"; $resn_exch{"H202 RE9"}="2H20 RE9"; $resn_exch{"H203 RE9"}="3H20 RE9";
$resn_exch{" H21 REA"}="1H2  REA"; $resn_exch{" H22 REA"}="2H2  REA"; $resn_exch{" H31 REA"}="1H3  REA";
$resn_exch{" H32 REA"}="2H3  REA"; $resn_exch{" H41 REA"}="1H4  REA"; $resn_exch{" H42 REA"}="2H4  REA";
$resn_exch{"H161 REA"}="1H16 REA"; $resn_exch{"H162 REA"}="2H16 REA"; $resn_exch{"H163 REA"}="3H16 REA";
$resn_exch{"H171 REA"}="1H17 REA"; $resn_exch{"H172 REA"}="2H17 REA"; $resn_exch{"H173 REA"}="3H17 REA";
$resn_exch{"H181 REA"}="1H18 REA"; $resn_exch{"H182 REA"}="2H18 REA"; $resn_exch{"H183 REA"}="3H18 REA";
$resn_exch{"H191 REA"}="1H19 REA"; $resn_exch{"H192 REA"}="2H19 REA"; $resn_exch{"H193 REA"}="3H19 REA";
$resn_exch{"H201 REA"}="1H20 REA"; $resn_exch{"H202 REA"}="2H20 REA"; $resn_exch{"H203 REA"}="3H20 REA";
$resn_exch{" H22 RED"}="2H2  RED"; $resn_exch{" H21 RED"}="1H2  RED"; $resn_exch{" H32 RED"}="2H3  RED";
$resn_exch{" H31 RED"}="1H3  RED"; $resn_exch{" H42 RED"}="2H4  RED"; $resn_exch{" H41 RED"}="1H4  RED";
$resn_exch{" H52 RED"}="2H5  RED"; $resn_exch{" H51 RED"}="1H5  RED"; $resn_exch{" H72 RED"}="2H7  RED";
$resn_exch{" H71 RED"}="1H7  RED"; $resn_exch{" H82 RED"}="2H8  RED"; $resn_exch{" H81 RED"}="1H8  RED";
$resn_exch{" H71 REP"}="1H7  REP"; $resn_exch{" H81 REP"}="1H8  REP"; $resn_exch{" H91 REP"}="1H9  REP";
$resn_exch{"H141 REP"}="1H14 REP"; $resn_exch{"H151 REP"}="1H15 REP"; $resn_exch{"H161 REP"}="1H16 REP";
$resn_exch{"H171 REP"}="1H17 REP"; $resn_exch{"H181 REP"}="1H18 REP"; $resn_exch{" H11 REQ"}="1H1  REQ";
$resn_exch{" H12 REQ"}="2H1  REQ"; $resn_exch{" H21 REQ"}="1H2  REQ"; $resn_exch{" H22 REQ"}="2H2  REQ";
$resn_exch{" H31 REQ"}="1H3  REQ"; $resn_exch{" H32 REQ"}="2H3  REQ"; $resn_exch{"H191 REQ"}="1H19 REQ";
$resn_exch{"H192 REQ"}="2H19 REQ"; $resn_exch{"H193 REQ"}="3H19 REQ"; $resn_exch{"H201 REQ"}="1H20 REQ";
$resn_exch{"H202 REQ"}="2H20 REQ"; $resn_exch{"H203 REQ"}="3H20 REQ"; $resn_exch{"H4C1 RES"}="1H4C RES";
$resn_exch{"H4C2 RES"}="2H4C RES"; $resn_exch{" H21 RET"}="1H2  RET"; $resn_exch{" H22 RET"}="2H2  RET";
$resn_exch{" H31 RET"}="1H3  RET"; $resn_exch{" H32 RET"}="2H3  RET"; $resn_exch{" H41 RET"}="1H4  RET";
$resn_exch{" H42 RET"}="2H4  RET"; $resn_exch{"H161 RET"}="1H16 RET"; $resn_exch{"H162 RET"}="2H16 RET";
$resn_exch{"H163 RET"}="3H16 RET"; $resn_exch{"H171 RET"}="1H17 RET"; $resn_exch{"H172 RET"}="2H17 RET";
$resn_exch{"H173 RET"}="3H17 RET"; $resn_exch{"H181 RET"}="1H18 RET"; $resn_exch{"H182 RET"}="2H18 RET";
$resn_exch{"H183 RET"}="3H18 RET"; $resn_exch{"H191 RET"}="1H19 RET"; $resn_exch{"H192 RET"}="2H19 RET";
$resn_exch{"H193 RET"}="3H19 RET"; $resn_exch{"H201 RET"}="1H20 RET"; $resn_exch{"H202 RET"}="2H20 RET";
$resn_exch{"H203 RET"}="3H20 RET"; $resn_exch{"H131 REW"}="1H13 REW"; $resn_exch{"H132 REW"}="2H13 REW";
$resn_exch{"H221 REW"}="1H22 REW"; $resn_exch{"H222 REW"}="2H22 REW"; $resn_exch{"H351 REW"}="1H35 REW";
$resn_exch{"H352 REW"}="2H35 REW"; $resn_exch{"H353 REW"}="3H35 REW"; $resn_exch{"H141 REW"}="1H14 REW";
$resn_exch{"H142 REW"}="2H14 REW"; $resn_exch{"H11A REW"}="1H1  REW"; $resn_exch{"H12A REW"}="2H1  REW";
$resn_exch{" H13 REW"}="3H1  REW"; $resn_exch{"HC11 REX"}="1HC1 REX"; $resn_exch{"HC12 REX"}="2HC1 REX";
$resn_exch{"HC21 REX"}="1HC2 REX"; $resn_exch{"HC22 REX"}="2HC2 REX"; $resn_exch{"HC31 REX"}="1HC3 REX";
$resn_exch{"HC32 REX"}="2HC3 REX"; $resn_exch{"HC41 REX"}="1HC4 REX"; $resn_exch{"HC42 REX"}="2HC4 REX";
$resn_exch{"HC81 REX"}="1HC8 REX"; $resn_exch{"HC82 REX"}="2HC8 REX"; $resn_exch{"HC83 REX"}="3HC8 REX";
$resn_exch{"H111 REX"}="1H11 REX"; $resn_exch{"H112 REX"}="2H11 REX"; $resn_exch{"H113 REX"}="3H11 REX";
$resn_exch{"H151 REX"}="1H15 REX"; $resn_exch{"H152 REX"}="2H15 REX"; $resn_exch{"HN41 REX"}="1HN4 REX";
$resn_exch{"HN42 REX"}="2HN4 REX"; $resn_exch{"HN43 REX"}="3HN4 REX"; $resn_exch{"HC11 REY"}="1HC1 REY";
$resn_exch{"HC12 REY"}="2HC1 REY"; $resn_exch{"HC21 REY"}="1HC2 REY"; $resn_exch{"HC22 REY"}="2HC2 REY";
$resn_exch{"HC31 REY"}="1HC3 REY"; $resn_exch{"HC32 REY"}="2HC3 REY"; $resn_exch{"HC41 REY"}="1HC4 REY";
$resn_exch{"HC42 REY"}="2HC4 REY"; $resn_exch{"HC81 REY"}="1HC8 REY"; $resn_exch{"HC82 REY"}="2HC8 REY";
$resn_exch{"HC83 REY"}="3HC8 REY"; $resn_exch{"H121 REY"}="1H12 REY"; $resn_exch{"H122 REY"}="2H12 REY";
$resn_exch{"HN41 REY"}="1HN4 REY"; $resn_exch{"HN42 REY"}="2HN4 REY"; $resn_exch{"HN43 REY"}="3HN4 REY";
$resn_exch{"HA11 REZ"}="1HA1 REZ"; $resn_exch{"HA12 REZ"}="2HA1 REZ"; $resn_exch{"HB11 REZ"}="1HB1 REZ";
$resn_exch{"HB12 REZ"}="2HB1 REZ"; $resn_exch{"HG11 REZ"}="1HG1 REZ"; $resn_exch{"HG12 REZ"}="2HG1 REZ";
$resn_exch{"HD11 REZ"}="1HD1 REZ"; $resn_exch{"HD12 REZ"}="2HD1 REZ"; $resn_exch{"HZ11 REZ"}="1HZ1 REZ";
$resn_exch{"HZ12 REZ"}="2HZ1 REZ"; $resn_exch{"HBC1 REZ"}="1HBC REZ"; $resn_exch{"HBC2 REZ"}="2HBC REZ";
$resn_exch{"HBC3 REZ"}="3HBC REZ"; $resn_exch{" H31 RFA"}="1H3  RFA"; $resn_exch{" H32 RFA"}="2H3  RFA";
$resn_exch{" H41 RFA"}="1H4  RFA"; $resn_exch{" H42 RFA"}="2H4  RFA"; $resn_exch{" H51 RFA"}="1H5  RFA";
$resn_exch{" H61 RFA"}="1H6  RFA"; $resn_exch{" H91 RFA"}="1H9  RFA"; $resn_exch{" H92 RFA"}="2H9  RFA";
$resn_exch{"H101 RFA"}="1H10 RFA"; $resn_exch{"H102 RFA"}="2H10 RFA"; $resn_exch{"H111 RFA"}="1H11 RFA";
$resn_exch{"H121 RFA"}="1H12 RFA"; $resn_exch{"H151 RFA"}="1H15 RFA"; $resn_exch{"H152 RFA"}="2H15 RFA";
$resn_exch{"H161 RFA"}="1H16 RFA"; $resn_exch{"H162 RFA"}="2H16 RFA"; $resn_exch{"H181 RFA"}="1H18 RFA";
$resn_exch{"H119 RFA"}="19H1 RFA"; $resn_exch{"H220 RFA"}="20H2 RFA"; $resn_exch{"H211 RFA"}="1H21 RFA";
$resn_exch{"H212 RFA"}="2H21 RFA"; $resn_exch{"H222 RFA"}="22H2 RFA"; $resn_exch{"H231 RFA"}="1H23 RFA";
$resn_exch{"H232 RFA"}="2H23 RFA"; $resn_exch{"H241 RFA"}="1H24 RFA"; $resn_exch{"H242 RFA"}="2H24 RFA";
$resn_exch{"H271 RFA"}="1H27 RFA"; $resn_exch{"H272 RFA"}="2H27 RFA"; $resn_exch{"H281 RFA"}="1H28 RFA";
$resn_exch{"H282 RFA"}="2H28 RFA"; $resn_exch{"H291 RFA"}="1H29 RFA"; $resn_exch{"H292 RFA"}="2H29 RFA";
$resn_exch{"H301 RFA"}="1H30 RFA"; $resn_exch{"H302 RFA"}="2H30 RFA"; $resn_exch{"H331 RFA"}="1H33 RFA";
$resn_exch{"H332 RFA"}="2H33 RFA"; $resn_exch{"H341 RFA"}="1H34 RFA"; $resn_exch{"H342 RFA"}="2H34 RFA";
$resn_exch{"H351 RFA"}="1H35 RFA"; $resn_exch{"H352 RFA"}="2H35 RFA"; $resn_exch{"H361 RFA"}="1H36 RFA";
$resn_exch{"H362 RFA"}="2H36 RFA"; $resn_exch{"H381 RFA"}="1H38 RFA"; $resn_exch{"H382 RFA"}="2H38 RFA";
$resn_exch{"H383 RFA"}="3H38 RFA"; $resn_exch{"H391 RFA"}="1H39 RFA"; $resn_exch{"H392 RFA"}="2H39 RFA";
$resn_exch{"H440 RFA"}="40H4 RFA"; $resn_exch{"H441 RFA"}="41H4 RFA"; $resn_exch{"H442 RFA"}="42H4 RFA";
$resn_exch{"H443 RFA"}="43H4 RFA"; $resn_exch{"H444 RFA"}="44H4 RFA"; $resn_exch{"H445 RFA"}="45H4 RFA";
$resn_exch{"H550 RFA"}="50H5 RFA"; $resn_exch{"H551 RFA"}="51H5 RFA"; $resn_exch{"H552 RFA"}="52H5 RFA";
$resn_exch{"H553 RFA"}="53H5 RFA"; $resn_exch{"H554 RFA"}="54H5 RFA"; $resn_exch{"H555 RFA"}="55H5 RFA";
$resn_exch{"H601 RFA"}="1H60 RFA"; $resn_exch{"H611 RFA"}="1H61 RFA"; $resn_exch{"H621 RFA"}="1H62 RFA";
$resn_exch{"H622 RFA"}="2H62 RFA"; $resn_exch{"H631 RFA"}="1H63 RFA"; $resn_exch{"H641 RFA"}="1H64 RFA";
$resn_exch{"H642 RFA"}="2H64 RFA"; $resn_exch{"H651 RFA"}="1H65 RFA"; $resn_exch{"H661 RFA"}="1H66 RFA";
$resn_exch{"H671 RFA"}="1H67 RFA"; $resn_exch{"H672 RFA"}="2H67 RFA"; $resn_exch{"H681 RFA"}="1H68 RFA";
$resn_exch{"H682 RFA"}="2H68 RFA"; $resn_exch{"H691 RFA"}="1H69 RFA"; $resn_exch{"H701 RFA"}="1H70 RFA";
$resn_exch{"H702 RFA"}="2H70 RFA"; $resn_exch{" H31 RFB"}="1H3  RFB"; $resn_exch{" H32 RFB"}="2H3  RFB";
$resn_exch{" H41 RFB"}="1H4  RFB"; $resn_exch{" H42 RFB"}="2H4  RFB"; $resn_exch{" H51 RFB"}="1H5  RFB";
$resn_exch{" H61 RFB"}="1H6  RFB"; $resn_exch{" H91 RFB"}="1H9  RFB"; $resn_exch{" H92 RFB"}="2H9  RFB";
$resn_exch{"H101 RFB"}="1H10 RFB"; $resn_exch{"H102 RFB"}="2H10 RFB"; $resn_exch{"H111 RFB"}="1H11 RFB";
$resn_exch{"H121 RFB"}="1H12 RFB"; $resn_exch{"H151 RFB"}="1H15 RFB"; $resn_exch{"H152 RFB"}="2H15 RFB";
$resn_exch{"H161 RFB"}="1H16 RFB"; $resn_exch{"H162 RFB"}="2H16 RFB"; $resn_exch{"H181 RFB"}="1H18 RFB";
$resn_exch{"H119 RFB"}="19H1 RFB"; $resn_exch{"H220 RFB"}="20H2 RFB"; $resn_exch{"H211 RFB"}="1H21 RFB";
$resn_exch{"H212 RFB"}="2H21 RFB"; $resn_exch{"H222 RFB"}="22H2 RFB"; $resn_exch{"H231 RFB"}="1H23 RFB";
$resn_exch{"H232 RFB"}="2H23 RFB"; $resn_exch{"H241 RFB"}="1H24 RFB"; $resn_exch{"H242 RFB"}="2H24 RFB";
$resn_exch{"H271 RFB"}="1H27 RFB"; $resn_exch{"H272 RFB"}="2H27 RFB"; $resn_exch{"H281 RFB"}="1H28 RFB";
$resn_exch{"H282 RFB"}="2H28 RFB"; $resn_exch{"H291 RFB"}="1H29 RFB"; $resn_exch{"H292 RFB"}="2H29 RFB";
$resn_exch{"H301 RFB"}="1H30 RFB"; $resn_exch{"H302 RFB"}="2H30 RFB"; $resn_exch{"H331 RFB"}="1H33 RFB";
$resn_exch{"H332 RFB"}="2H33 RFB"; $resn_exch{"H341 RFB"}="1H34 RFB"; $resn_exch{"H342 RFB"}="2H34 RFB";
$resn_exch{"H351 RFB"}="1H35 RFB"; $resn_exch{"H352 RFB"}="2H35 RFB"; $resn_exch{"H361 RFB"}="1H36 RFB";
$resn_exch{"H362 RFB"}="2H36 RFB"; $resn_exch{"H381 RFB"}="1H38 RFB"; $resn_exch{"H382 RFB"}="2H38 RFB";
$resn_exch{"H383 RFB"}="3H38 RFB"; $resn_exch{"H391 RFB"}="1H39 RFB"; $resn_exch{"H392 RFB"}="2H39 RFB";
$resn_exch{"H440 RFB"}="40H4 RFB"; $resn_exch{"H441 RFB"}="41H4 RFB"; $resn_exch{"H442 RFB"}="42H4 RFB";
$resn_exch{"H443 RFB"}="43H4 RFB"; $resn_exch{"H444 RFB"}="44H4 RFB"; $resn_exch{"H445 RFB"}="45H4 RFB";
$resn_exch{"H550 RFB"}="50H5 RFB"; $resn_exch{"H551 RFB"}="51H5 RFB"; $resn_exch{"H552 RFB"}="52H5 RFB";
$resn_exch{"H553 RFB"}="53H5 RFB"; $resn_exch{"H554 RFB"}="54H5 RFB"; $resn_exch{"H555 RFB"}="55H5 RFB";
$resn_exch{"H601 RFB"}="1H60 RFB"; $resn_exch{"H611 RFB"}="1H61 RFB"; $resn_exch{"H621 RFB"}="1H62 RFB";
$resn_exch{"H622 RFB"}="2H62 RFB"; $resn_exch{"H631 RFB"}="1H63 RFB"; $resn_exch{"H641 RFB"}="1H64 RFB";
$resn_exch{"H642 RFB"}="2H64 RFB"; $resn_exch{"H651 RFB"}="1H65 RFB"; $resn_exch{"H661 RFB"}="1H66 RFB";
$resn_exch{"H671 RFB"}="1H67 RFB"; $resn_exch{"H672 RFB"}="2H67 RFB"; $resn_exch{"H681 RFB"}="1H68 RFB";
$resn_exch{"H682 RFB"}="2H68 RFB"; $resn_exch{"H691 RFB"}="1H69 RFB"; $resn_exch{"H701 RFB"}="1H70 RFB";
$resn_exch{"H702 RFB"}="2H70 RFB"; $resn_exch{" H3' RFC"}=" H3* RFC"; $resn_exch{" H2' RFC"}=" H2* RFC";
$resn_exch{" H1' RFC"}=" H1* RFC"; $resn_exch{"HN61 RFC"}="1HN6 RFC"; $resn_exch{"HN62 RFC"}="2HN6 RFC";
$resn_exch{" H4' RFC"}=" H4* RFC"; $resn_exch{"H5'1 RFC"}="1H5* RFC"; $resn_exch{"H5'2 RFC"}="2H5* RFC";
$resn_exch{" H41 RFC"}="1H4  RFC"; $resn_exch{" H42 RFC"}="2H4  RFC"; $resn_exch{"H11X RFC"}="1H1  RFC";
$resn_exch{" H12 RFC"}="2H1  RFC"; $resn_exch{" H13 RFC"}="3H1  RFC"; $resn_exch{"H141 RFC"}="1H14 RFC";
$resn_exch{"H142 RFC"}="2H14 RFC"; $resn_exch{"H143 RFC"}="3H14 RFC"; $resn_exch{" H71 RFC"}="1H7  RFC";
$resn_exch{" H72 RFC"}="2H7  RFC"; $resn_exch{" H61 RFC"}="1H6  RFC"; $resn_exch{" H62 RFC"}="2H6  RFC";
$resn_exch{" H31 RFC"}="1H3  RFC"; $resn_exch{" H32 RFC"}="2H3  RFC"; $resn_exch{" H21 RFC"}="1H2  RFC";
$resn_exch{" H22 RFC"}="2H2  RFC"; $resn_exch{"H131 RFC"}="1H13 RFC"; $resn_exch{"H132 RFC"}="2H13 RFC";
$resn_exch{"H133 RFC"}="3H13 RFC"; $resn_exch{" H91 RFC"}="1H9  RFC"; $resn_exch{" H92 RFC"}="2H9  RFC";
$resn_exch{"H121 RFC"}="1H12 RFC"; $resn_exch{"H122 RFC"}="2H12 RFC"; $resn_exch{"H123 RFC"}="3H12 RFC";
$resn_exch{"H111 RFC"}="1H11 RFC"; $resn_exch{"H112 RFC"}="2H11 RFC"; $resn_exch{"H113 RFC"}="3H11 RFC";
$resn_exch{"H5'1 RFL"}="1H5' RFL"; $resn_exch{"H5'2 RFL"}="2H5' RFL"; $resn_exch{"HC21 RFL"}="1HC2 RFL";
$resn_exch{"HN61 RFL"}="1HN6 RFL"; $resn_exch{"HN62 RFL"}="2HN6 RFL"; $resn_exch{"HC81 RFL"}="1HC8 RFL";
$resn_exch{"H5R1 RFL"}="1H5R RFL"; $resn_exch{"H5R2 RFL"}="2H5R RFL"; $resn_exch{"H1R1 RFL"}="1H1R RFL";
$resn_exch{"H1R2 RFL"}="2H1R RFL"; $resn_exch{"HM11 RFL"}="1HM1 RFL"; $resn_exch{"HM12 RFL"}="2HM1 RFL";
$resn_exch{"HM13 RFL"}="3HM1 RFL"; $resn_exch{"HM21 RFL"}="1HM2 RFL"; $resn_exch{"HM22 RFL"}="2HM2 RFL";
$resn_exch{"HM23 RFL"}="3HM2 RFL"; $resn_exch{"H7M1 RFL"}="1H7M RFL"; $resn_exch{"H7M2 RFL"}="2H7M RFL";
$resn_exch{"H7M3 RFL"}="3H7M RFL"; $resn_exch{"H131 RFP"}="1H13 RFP"; $resn_exch{"H132 RFP"}="2H13 RFP";
$resn_exch{"H133 RFP"}="3H13 RFP"; $resn_exch{"H141 RFP"}="1H14 RFP"; $resn_exch{"H142 RFP"}="2H14 RFP";
$resn_exch{"H143 RFP"}="3H14 RFP"; $resn_exch{"H17C RFP"}="CH17 RFP"; $resn_exch{"H18C RFP"}="CH18 RFP";
$resn_exch{"H19C RFP"}="CH19 RFP"; $resn_exch{"H20C RFP"}="CH20 RFP"; $resn_exch{"H211 RFP"}="1H21 RFP";
$resn_exch{"H22C RFP"}="CH22 RFP"; $resn_exch{"H23C RFP"}="CH23 RFP"; $resn_exch{"H24C RFP"}="CH24 RFP";
$resn_exch{"H25C RFP"}="CH25 RFP"; $resn_exch{"H26C RFP"}="CH26 RFP"; $resn_exch{"H27C RFP"}="CH27 RFP";
$resn_exch{"H28C RFP"}="CH28 RFP"; $resn_exch{"H29C RFP"}="CH29 RFP"; $resn_exch{"H301 RFP"}="1H30 RFP";
$resn_exch{"H302 RFP"}="2H30 RFP"; $resn_exch{"H303 RFP"}="3H30 RFP"; $resn_exch{"H311 RFP"}="1H31 RFP";
$resn_exch{"H312 RFP"}="2H31 RFP"; $resn_exch{"H313 RFP"}="3H31 RFP"; $resn_exch{"H321 RFP"}="1H32 RFP";
$resn_exch{"H322 RFP"}="2H32 RFP"; $resn_exch{"H323 RFP"}="3H32 RFP"; $resn_exch{"H331 RFP"}="1H33 RFP";
$resn_exch{"H332 RFP"}="2H33 RFP"; $resn_exch{"H333 RFP"}="3H33 RFP"; $resn_exch{"H341 RFP"}="1H34 RFP";
$resn_exch{"H342 RFP"}="2H34 RFP"; $resn_exch{"H343 RFP"}="3H34 RFP"; $resn_exch{"H361 RFP"}="1H36 RFP";
$resn_exch{"H362 RFP"}="2H36 RFP"; $resn_exch{"H363 RFP"}="3H36 RFP"; $resn_exch{"H371 RFP"}="1H37 RFP";
$resn_exch{"H372 RFP"}="2H37 RFP"; $resn_exch{"H373 RFP"}="3H37 RFP"; $resn_exch{"H381 RFP"}="1H38 RFP";
$resn_exch{"H382 RFP"}="2H38 RFP"; $resn_exch{"H383 RFP"}="3H38 RFP"; $resn_exch{"H391 RFP"}="1H39 RFP";
$resn_exch{"H392 RFP"}="2H39 RFP"; $resn_exch{"H401 RFP"}="1H40 RFP"; $resn_exch{"H402 RFP"}="2H40 RFP";
$resn_exch{"H411 RFP"}="1H41 RFP"; $resn_exch{"H412 RFP"}="2H41 RFP"; $resn_exch{"H421 RFP"}="1H42 RFP";
$resn_exch{"H422 RFP"}="2H42 RFP"; $resn_exch{"H10O RFP"}="OH10 RFP"; $resn_exch{"H12O RFP"}="OH12 RFP";
$resn_exch{"H6'1 RG1"}="1H6' RG1"; $resn_exch{"H6'2 RG1"}="2H6' RG1"; $resn_exch{"HO2' RG1"}="'HO2 RG1";
$resn_exch{"HO3' RG1"}="'HO3 RG1"; $resn_exch{"HO4' RG1"}="'HO4 RG1"; $resn_exch{"HO6' RG1"}="'HO6 RG1";
$resn_exch{"HM11 RG1"}="1HM1 RG1"; $resn_exch{"HM12 RG1"}="2HM1 RG1"; $resn_exch{"HM13 RG1"}="3HM1 RG1";
$resn_exch{"HM21 RG1"}="1HM2 RG1"; $resn_exch{"HM22 RG1"}="2HM2 RG1"; $resn_exch{"HM23 RG1"}="3HM2 RG1";
$resn_exch{"H21A RG1"}="1H2  RG1"; $resn_exch{" H22 RG1"}="2H2  RG1"; $resn_exch{" H31 RG1"}="1H3  RG1";
$resn_exch{" H32 RG1"}="2H3  RG1"; $resn_exch{" H41 RG1"}="1H4  RG1"; $resn_exch{" H42 RG1"}="2H4  RG1";
$resn_exch{"HM31 RG1"}="1HM3 RG1"; $resn_exch{"HM32 RG1"}="2HM3 RG1"; $resn_exch{"HM33 RG1"}="3HM3 RG1";
$resn_exch{"HM41 RG1"}="1HM4 RG1"; $resn_exch{"HM42 RG1"}="2HM4 RG1"; $resn_exch{"HM43 RG1"}="3HM4 RG1";
$resn_exch{"HM51 RG1"}="1HM5 RG1"; $resn_exch{"HM52 RG1"}="2HM5 RG1"; $resn_exch{"HM53 RG1"}="3HM5 RG1";
$resn_exch{"HM61 RG1"}="1HM6 RG1"; $resn_exch{"HM62 RG1"}="2HM6 RG1"; $resn_exch{"HM63 RG1"}="3HM6 RG1";
$resn_exch{"HM71 RG1"}="1HM7 RG1"; $resn_exch{"HM72 RG1"}="2HM7 RG1"; $resn_exch{"HM73 RG1"}="3HM7 RG1";
$resn_exch{"HM81 RG1"}="1HM8 RG1"; $resn_exch{"HM82 RG1"}="2HM8 RG1"; $resn_exch{"HM83 RG1"}="3HM8 RG1";
$resn_exch{"HM91 RG1"}="1HM9 RG1"; $resn_exch{"HM92 RG1"}="2HM9 RG1"; $resn_exch{"HM93 RG1"}="3HM9 RG1";
$resn_exch{"HM01 RG1"}="1HM0 RG1"; $resn_exch{"HM02 RG1"}="2HM0 RG1"; $resn_exch{"HM03 RG1"}="3HM0 RG1";
$resn_exch{"H381 RGA"}="1H38 RGA"; $resn_exch{"H382 RGA"}="2H38 RGA"; $resn_exch{"H383 RGA"}="3H38 RGA";
$resn_exch{" H61 RGA"}="1H6  RGA"; $resn_exch{" H62 RGA"}="2H6  RGA"; $resn_exch{"H481 RGA"}="1H48 RGA";
$resn_exch{"H482 RGA"}="2H48 RGA"; $resn_exch{"H483 RGA"}="3H48 RGA"; $resn_exch{" H81 RGA"}="1H8  RGA";
$resn_exch{" H82 RGA"}="2H8  RGA"; $resn_exch{"H101 RGA"}="1H10 RGA"; $resn_exch{"H102 RGA"}="2H10 RGA";
$resn_exch{"H411 RGA"}="1H41 RGA"; $resn_exch{"H412 RGA"}="2H41 RGA"; $resn_exch{"H413 RGA"}="3H41 RGA";
$resn_exch{"H491 RGA"}="1H49 RGA"; $resn_exch{"H492 RGA"}="2H49 RGA"; $resn_exch{"H493 RGA"}="3H49 RGA";
$resn_exch{"H141 RGA"}="1H14 RGA"; $resn_exch{"H142 RGA"}="2H14 RGA"; $resn_exch{"H501 RGA"}="1H50 RGA";
$resn_exch{"H502 RGA"}="2H50 RGA"; $resn_exch{"H503 RGA"}="3H50 RGA"; $resn_exch{"H421 RGA"}="1H42 RGA";
$resn_exch{"H422 RGA"}="2H42 RGA"; $resn_exch{"H423 RGA"}="3H42 RGA"; $resn_exch{"H511 RGA"}="1H51 RGA";
$resn_exch{"H512 RGA"}="2H51 RGA"; $resn_exch{"H513 RGA"}="3H51 RGA"; $resn_exch{"H201 RGA"}="1H20 RGA";
$resn_exch{"H202 RGA"}="2H20 RGA"; $resn_exch{"H521 RGA"}="1H52 RGA"; $resn_exch{"H522 RGA"}="2H52 RGA";
$resn_exch{"H523 RGA"}="3H52 RGA"; $resn_exch{"H431 RGA"}="1H43 RGA"; $resn_exch{"H432 RGA"}="2H43 RGA";
$resn_exch{"H433 RGA"}="3H43 RGA"; $resn_exch{"H441 RGA"}="1H44 RGA"; $resn_exch{"H442 RGA"}="2H44 RGA";
$resn_exch{"H443 RGA"}="3H44 RGA"; $resn_exch{"H531 RGA"}="1H53 RGA"; $resn_exch{"H532 RGA"}="2H53 RGA";
$resn_exch{"H533 RGA"}="3H53 RGA"; $resn_exch{"H451 RGA"}="1H45 RGA"; $resn_exch{"H452 RGA"}="2H45 RGA";
$resn_exch{"H453 RGA"}="3H45 RGA"; $resn_exch{"H291 RGA"}="1H29 RGA"; $resn_exch{"H292 RGA"}="2H29 RGA";
$resn_exch{"H301 RGA"}="1H30 RGA"; $resn_exch{"H302 RGA"}="2H30 RGA"; $resn_exch{"H461 RGA"}="1H46 RGA";
$resn_exch{"H462 RGA"}="2H46 RGA"; $resn_exch{"H463 RGA"}="3H46 RGA"; $resn_exch{"H541 RGA"}="1H54 RGA";
$resn_exch{"H542 RGA"}="2H54 RGA"; $resn_exch{"H543 RGA"}="3H54 RGA"; $resn_exch{"H341 RGA"}="1H34 RGA";
$resn_exch{"H342 RGA"}="2H34 RGA"; $resn_exch{"H471 RGA"}="1H47 RGA"; $resn_exch{"H472 RGA"}="2H47 RGA";
$resn_exch{"H473 RGA"}="3H47 RGA"; $resn_exch{"H381 RGC"}="1H38 RGC"; $resn_exch{"H382 RGC"}="2H38 RGC";
$resn_exch{"H383 RGC"}="3H38 RGC"; $resn_exch{" H61 RGC"}="1H6  RGC"; $resn_exch{" H62 RGC"}="2H6  RGC";
$resn_exch{"H481 RGC"}="1H48 RGC"; $resn_exch{"H482 RGC"}="2H48 RGC"; $resn_exch{"H483 RGC"}="3H48 RGC";
$resn_exch{" H81 RGC"}="1H8  RGC"; $resn_exch{" H82 RGC"}="2H8  RGC"; $resn_exch{"H101 RGC"}="1H10 RGC";
$resn_exch{"H102 RGC"}="2H10 RGC"; $resn_exch{"H411 RGC"}="1H41 RGC"; $resn_exch{"H412 RGC"}="2H41 RGC";
$resn_exch{"H413 RGC"}="3H41 RGC"; $resn_exch{"H491 RGC"}="1H49 RGC"; $resn_exch{"H492 RGC"}="2H49 RGC";
$resn_exch{"H493 RGC"}="3H49 RGC"; $resn_exch{"H141 RGC"}="1H14 RGC"; $resn_exch{"H142 RGC"}="2H14 RGC";
$resn_exch{"H501 RGC"}="1H50 RGC"; $resn_exch{"H502 RGC"}="2H50 RGC"; $resn_exch{"H503 RGC"}="3H50 RGC";
$resn_exch{"H421 RGC"}="1H42 RGC"; $resn_exch{"H422 RGC"}="2H42 RGC"; $resn_exch{"H423 RGC"}="3H42 RGC";
$resn_exch{"H511 RGC"}="1H51 RGC"; $resn_exch{"H512 RGC"}="2H51 RGC"; $resn_exch{"H513 RGC"}="3H51 RGC";
$resn_exch{"H201 RGC"}="1H20 RGC"; $resn_exch{"H202 RGC"}="2H20 RGC"; $resn_exch{"H521 RGC"}="1H52 RGC";
$resn_exch{"H522 RGC"}="2H52 RGC"; $resn_exch{"H523 RGC"}="3H52 RGC"; $resn_exch{"H431 RGC"}="1H43 RGC";
$resn_exch{"H432 RGC"}="2H43 RGC"; $resn_exch{"H433 RGC"}="3H43 RGC"; $resn_exch{"H441 RGC"}="1H44 RGC";
$resn_exch{"H442 RGC"}="2H44 RGC"; $resn_exch{"H443 RGC"}="3H44 RGC"; $resn_exch{"H531 RGC"}="1H53 RGC";
$resn_exch{"H532 RGC"}="2H53 RGC"; $resn_exch{"H533 RGC"}="3H53 RGC"; $resn_exch{"H451 RGC"}="1H45 RGC";
$resn_exch{"H452 RGC"}="2H45 RGC"; $resn_exch{"H453 RGC"}="3H45 RGC"; $resn_exch{"H291 RGC"}="1H29 RGC";
$resn_exch{"H292 RGC"}="2H29 RGC"; $resn_exch{"H301 RGC"}="1H30 RGC"; $resn_exch{"H302 RGC"}="2H30 RGC";
$resn_exch{"H461 RGC"}="1H46 RGC"; $resn_exch{"H462 RGC"}="2H46 RGC"; $resn_exch{"H463 RGC"}="3H46 RGC";
$resn_exch{"H541 RGC"}="1H54 RGC"; $resn_exch{"H542 RGC"}="2H54 RGC"; $resn_exch{"H543 RGC"}="3H54 RGC";
$resn_exch{"H341 RGC"}="1H34 RGC"; $resn_exch{"H342 RGC"}="2H34 RGC"; $resn_exch{"H351 RGC"}="1H35 RGC";
$resn_exch{"H352 RGC"}="2H35 RGC"; $resn_exch{"H1C1 RGI"}="1H1C RGI"; $resn_exch{" HA2 RGI"}="2HA  RGI";
$resn_exch{" HB3 RGI"}="3HB  RGI"; $resn_exch{" HG1 RGP"}="1HG  RGP"; $resn_exch{" HG2 RGP"}="2HG  RGP";
$resn_exch{" HB1 RGP"}="1HB  RGP"; $resn_exch{" HB2 RGP"}="2HB  RGP"; $resn_exch{" HN1 RGP"}="1HN  RGP";
$resn_exch{" HN2 RGP"}="2HN  RGP"; $resn_exch{"HOB1 RGS"}="1HOB RGS"; $resn_exch{"HN21 RGS"}="1HN2 RGS";
$resn_exch{"HN22 RGS"}="2HN2 RGS"; $resn_exch{" H31 RGS"}="1H3  RGS"; $resn_exch{" H32 RGS"}="2H3  RGS";
$resn_exch{" H41 RGS"}="1H4  RGS"; $resn_exch{" H42 RGS"}="2H4  RGS"; $resn_exch{" H61 RGS"}="1H6  RGS";
$resn_exch{" H62 RGS"}="2H6  RGS"; $resn_exch{"HOB7 RGS"}="7HOB RGS"; $resn_exch{" HB1 RGS"}="1HB  RGS";
$resn_exch{" HB2 RGS"}="2HB  RGS"; $resn_exch{"HB'1 RGS"}="1HB* RGS"; $resn_exch{"HB'2 RGS"}="2HB* RGS";
$resn_exch{" HA' RGS"}=" HA* RGS"; $resn_exch{" HN' RGS"}=" HN* RGS"; $resn_exch{"H6'1 RGS"}="1H6* RGS";
$resn_exch{"H6'2 RGS"}="2H6* RGS"; $resn_exch{"HO7' RGS"}="*HO7 RGS"; $resn_exch{"HN4' RGS"}="*HN4 RGS";
$resn_exch{"H4'1 RGS"}="1H4* RGS"; $resn_exch{"H4'2 RGS"}="2H4* RGS"; $resn_exch{"H3'1 RGS"}="1H3* RGS";
$resn_exch{"H3'2 RGS"}="2H3* RGS"; $resn_exch{" H2' RGS"}=" H2* RGS"; $resn_exch{"H2'1 RGS"}="1H2* RGS";
$resn_exch{"H2'2 RGS"}="2H2* RGS"; $resn_exch{"HO1' RGS"}="*HO1 RGS"; $resn_exch{"HC71 RH1"}="1HC7 RH1";
$resn_exch{"HC72 RH1"}="2HC7 RH1"; $resn_exch{"HC81 RH1"}="1HC8 RH1"; $resn_exch{"HC82 RH1"}="2HC8 RH1";
$resn_exch{"HC91 RH1"}="1HC9 RH1"; $resn_exch{"HC92 RH1"}="2HC9 RH1"; $resn_exch{"H101 RH1"}="1H10 RH1";
$resn_exch{"H102 RH1"}="2H10 RH1"; $resn_exch{"H111 RH1"}="1H11 RH1"; $resn_exch{"H112 RH1"}="2H11 RH1";
$resn_exch{" HO3 RH1"}="3HO  RH1"; $resn_exch{"H121 RH1"}="1H12 RH1"; $resn_exch{"H122 RH1"}="2H12 RH1";
$resn_exch{"H123 RH1"}="3H12 RH1"; $resn_exch{" H61 RHA"}="1H6  RHA"; $resn_exch{" H62 RHA"}="2H6  RHA";
$resn_exch{" H63 RHA"}="3H6  RHA"; $resn_exch{"HOP2 RHA"}="2HOP RHA"; $resn_exch{"HOP3 RHA"}="3HOP RHA";
$resn_exch{"HC11 RHC"}="1HC1 RHC"; $resn_exch{"HO11 RHC"}="1HO1 RHC"; $resn_exch{"HC21 RHC"}="1HC2 RHC";
$resn_exch{"HO21 RHC"}="1HO2 RHC"; $resn_exch{"HC31 RHC"}="1HC3 RHC"; $resn_exch{"HO31 RHC"}="1HO3 RHC";
$resn_exch{"HC41 RHC"}="1HC4 RHC"; $resn_exch{"HCE1 RHC"}="1HCE RHC"; $resn_exch{"HCE2 RHC"}="2HCE RHC";
$resn_exch{" HN1 RHC"}="1HN  RHC"; $resn_exch{" HN2 RHC"}="2HN  RHC"; $resn_exch{"HCA1 RHC"}="1HCA RHC";
$resn_exch{"HCB1 RHC"}="1HCB RHC"; $resn_exch{"HCB2 RHC"}="2HCB RHC"; $resn_exch{"HCG1 RHC"}="1HCG RHC";
$resn_exch{"HCG2 RHC"}="2HCG RHC"; $resn_exch{"HOX1 RHC"}="1HOX RHC"; $resn_exch{"HN11 RHD"}="1HN1 RHD";
$resn_exch{"HN12 RHD"}="2HN1 RHD"; $resn_exch{"HN13 RHD"}="3HN1 RHD"; $resn_exch{"HN21 RHD"}="1HN2 RHD";
$resn_exch{"HN22 RHD"}="2HN2 RHD"; $resn_exch{"HN23 RHD"}="3HN2 RHD"; $resn_exch{"HN31 RHD"}="1HN3 RHD";
$resn_exch{"HN32 RHD"}="2HN3 RHD"; $resn_exch{"HN33 RHD"}="3HN3 RHD"; $resn_exch{"HN41 RHD"}="1HN4 RHD";
$resn_exch{"HN42 RHD"}="2HN4 RHD"; $resn_exch{"HN43 RHD"}="3HN4 RHD"; $resn_exch{"HN51 RHD"}="1HN5 RHD";
$resn_exch{"HN52 RHD"}="2HN5 RHD"; $resn_exch{"HN53 RHD"}="3HN5 RHD"; $resn_exch{"HN61 RHD"}="1HN6 RHD";
$resn_exch{"HN62 RHD"}="2HN6 RHD"; $resn_exch{"HN63 RHD"}="3HN6 RHD"; $resn_exch{" H21 RHM"}="1H2  RHM";
$resn_exch{" H22 RHM"}="2H2  RHM"; $resn_exch{" H23 RHM"}="3H2  RHM"; $resn_exch{"H111 RHM"}="1H11 RHM";
$resn_exch{"H112 RHM"}="2H11 RHM"; $resn_exch{"H113 RHM"}="3H11 RHM"; $resn_exch{" H91 RHM"}="1H9  RHM";
$resn_exch{" H92 RHM"}="2H9  RHM"; $resn_exch{" H41 RHM"}="1H4  RHM"; $resn_exch{" H42 RHM"}="2H4  RHM";
$resn_exch{" H61 RHM"}="1H6  RHM"; $resn_exch{" H62 RHM"}="2H6  RHM"; $resn_exch{" H71 RHM"}="1H7  RHM";
$resn_exch{" H72 RHM"}="2H7  RHM"; $resn_exch{"HC61 RHO"}="1HC6 RHO"; $resn_exch{"HC62 RHO"}="2HC6 RHO";
$resn_exch{"HC63 RHO"}="3HC6 RHO"; $resn_exch{"HC71 RHO"}="1HC7 RHO"; $resn_exch{"HC72 RHO"}="2HC7 RHO";
$resn_exch{"HC73 RHO"}="3HC7 RHO"; $resn_exch{"H10C RHO"}="CH10 RHO"; $resn_exch{"H12C RHO"}="CH12 RHO";
$resn_exch{"H131 RHO"}="1H13 RHO"; $resn_exch{"H132 RHO"}="2H13 RHO"; $resn_exch{"H151 RHO"}="1H15 RHO";
$resn_exch{"H152 RHO"}="2H15 RHO"; $resn_exch{"H153 RHO"}="3H15 RHO"; $resn_exch{"H161 RHO"}="1H16 RHO";
$resn_exch{"H162 RHO"}="2H16 RHO"; $resn_exch{"H163 RHO"}="3H16 RHO"; $resn_exch{"H19C RHO"}="CH19 RHO";
$resn_exch{"H21C RHO"}="CH21 RHO"; $resn_exch{"H22C RHO"}="CH22 RHO"; $resn_exch{"H25C RHO"}="CH25 RHO";
$resn_exch{"H26C RHO"}="CH26 RHO"; $resn_exch{"H111 RHP"}="1H11 RHP"; $resn_exch{"H112 RHP"}="2H11 RHP";
$resn_exch{" H11 RHP"}="1H1  RHP"; $resn_exch{" H12 RHP"}="2H1  RHP"; $resn_exch{" H21 RHP"}="1H2  RHP";
$resn_exch{" H22 RHP"}="2H2  RHP"; $resn_exch{" H31 RHQ"}="1H3  RHQ"; $resn_exch{" H61 RHQ"}="1H6  RHQ";
$resn_exch{"H101 RHQ"}="1H10 RHQ"; $resn_exch{"H131 RHQ"}="1H13 RHQ"; $resn_exch{"HN11 RHQ"}="1HN1 RHQ";
$resn_exch{"H151 RHQ"}="1H15 RHQ"; $resn_exch{"H161 RHQ"}="1H16 RHQ"; $resn_exch{"H171 RHQ"}="1H17 RHQ";
$resn_exch{"H181 RHQ"}="1H18 RHQ"; $resn_exch{"H201 RHQ"}="1H20 RHQ"; $resn_exch{"H202 RHQ"}="2H20 RHQ";
$resn_exch{"H203 RHQ"}="3H20 RHQ"; $resn_exch{"H211 RHQ"}="1H21 RHQ"; $resn_exch{"H212 RHQ"}="2H21 RHQ";
$resn_exch{"H213 RHQ"}="3H21 RHQ"; $resn_exch{"H221 RHQ"}="1H22 RHQ"; $resn_exch{"H222 RHQ"}="2H22 RHQ";
$resn_exch{"H231 RHQ"}="1H23 RHQ"; $resn_exch{"H232 RHQ"}="2H23 RHQ"; $resn_exch{"H233 RHQ"}="3H23 RHQ";
$resn_exch{"HN21 RHQ"}="1HN2 RHQ"; $resn_exch{"H241 RHQ"}="1H24 RHQ"; $resn_exch{"H242 RHQ"}="2H24 RHQ";
$resn_exch{"H251 RHQ"}="1H25 RHQ"; $resn_exch{"H252 RHQ"}="2H25 RHQ"; $resn_exch{"H253 RHQ"}="3H25 RHQ";
$resn_exch{"H281 RHQ"}="1H28 RHQ"; $resn_exch{"H282 RHQ"}="2H28 RHQ"; $resn_exch{"H291 RHQ"}="1H29 RHQ";
$resn_exch{"H292 RHQ"}="2H29 RHQ"; $resn_exch{"H293 RHQ"}="3H29 RHQ"; $resn_exch{"HN'1 RHS"}="1HN' RHS";
$resn_exch{"HN'2 RHS"}="2HN' RHS"; $resn_exch{"HT21 RHS"}="1HT2 RHS"; $resn_exch{"HT22 RHS"}="2HT2 RHS";
$resn_exch{"HT23 RHS"}="3HT2 RHS"; $resn_exch{"HT31 RHS"}="1HT3 RHS"; $resn_exch{"HT32 RHS"}="2HT3 RHS";
$resn_exch{"HT33 RHS"}="3HT3 RHS"; $resn_exch{"HT41 RHS"}="1HT4 RHS"; $resn_exch{"HT42 RHS"}="2HT4 RHS";
$resn_exch{"HT43 RHS"}="3HT4 RHS"; $resn_exch{"H1'1 RHS"}="1H1' RHS"; $resn_exch{"H1'2 RHS"}="2H1' RHS";
$resn_exch{" H41 RHS"}="1H4  RHS"; $resn_exch{" H42 RHS"}="2H4  RHS"; $resn_exch{" H51 RHS"}="1H5  RHS";
$resn_exch{" H52 RHS"}="2H5  RHS"; $resn_exch{" H53 RHS"}="3H5  RHS"; $resn_exch{" H21 RHS"}="1H2  RHS";
$resn_exch{" H22 RHS"}="2H2  RHS"; $resn_exch{" H11 RHS"}="1H1  RHS"; $resn_exch{" H12 RHS"}="2H1  RHS";
$resn_exch{" H13 RHS"}="3H1  RHS"; $resn_exch{"H121 RHX"}="1H12 RHX"; $resn_exch{"H122 RHX"}="2H12 RHX";
$resn_exch{" H81 RHX"}="1H8  RHX"; $resn_exch{" H82 RHX"}="2H8  RHX"; $resn_exch{"HOP2 RIA"}="2HOP RIA";
$resn_exch{"HOP3 RIA"}="3HOP RIA"; $resn_exch{" H5' RIA"}="1H5* RIA"; $resn_exch{"H5'' RIA"}="2H5* RIA";
$resn_exch{" H4A RIA"}=" H4* RIA"; $resn_exch{" H3A RIA"}=" H3* RIA"; $resn_exch{"HO3A RIA"}="*HO3 RIA";
$resn_exch{" H2A RIA"}=" H2* RIA"; $resn_exch{" H1A RIA"}=" H1* RIA"; $resn_exch{"HN61 RIA"}="1HN6 RIA";
$resn_exch{"HN62 RIA"}="2HN6 RIA"; $resn_exch{"HO2' RIA"}="'HO2 RIA"; $resn_exch{"HO3' RIA"}="'HO3 RIA";
$resn_exch{"H5'1 RIA"}="1H5' RIA"; $resn_exch{"H5'2 RIA"}="2H5' RIA"; $resn_exch{"HO'2 RIA"}="2HO' RIA";
$resn_exch{"HO'3 RIA"}="3HO' RIA"; $resn_exch{"HO5' RIB"}="*HO5 RIB"; $resn_exch{"H5'1 RIB"}="1H5* RIB";
$resn_exch{"H5'2 RIB"}="2H5* RIB"; $resn_exch{" H4' RIB"}=" H4* RIB"; $resn_exch{" H3' RIB"}=" H3* RIB";
$resn_exch{"HO3' RIB"}="*HO3 RIB"; $resn_exch{" H2' RIB"}=" H2* RIB"; $resn_exch{"HO2' RIB"}="*HO2 RIB";
$resn_exch{" H1' RIB"}=" H1* RIB"; $resn_exch{"HO1' RIB"}="*HO1 RIB"; $resn_exch{" H21 RIC"}="1H2  RIC";
$resn_exch{" H41 RIC"}="1H4  RIC"; $resn_exch{" H71 RIC"}="1H7  RIC"; $resn_exch{" H81 RIC"}="1H8  RIC";
$resn_exch{" H82 RIC"}="2H8  RIC"; $resn_exch{" H83 RIC"}="3H8  RIC"; $resn_exch{" H91 RIC"}="1H9  RIC";
$resn_exch{" H92 RIC"}="2H9  RIC"; $resn_exch{" H93 RIC"}="3H9  RIC"; $resn_exch{"HC51 RIC"}="1HC5 RIC";
$resn_exch{"HC52 RIC"}="2HC5 RIC"; $resn_exch{"H1Q1 RIC"}="1H1Q RIC"; $resn_exch{"HC51 RIF"}="1HC5 RIF";
$resn_exch{"HC52 RIF"}="2HC5 RIF"; $resn_exch{"HC53 RIF"}="3HC5 RIF"; $resn_exch{"HC61 RIF"}="1HC6 RIF";
$resn_exch{"HC62 RIF"}="2HC6 RIF"; $resn_exch{"HC63 RIF"}="3HC6 RIF"; $resn_exch{"H241 RIF"}="1H24 RIF";
$resn_exch{"H242 RIF"}="2H24 RIF"; $resn_exch{"H243 RIF"}="3H24 RIF"; $resn_exch{"H261 RIF"}="1H26 RIF";
$resn_exch{"H262 RIF"}="2H26 RIF"; $resn_exch{"H271 RIF"}="1H27 RIF"; $resn_exch{"H272 RIF"}="2H27 RIF";
$resn_exch{"H281 RIF"}="1H28 RIF"; $resn_exch{"H282 RIF"}="2H28 RIF"; $resn_exch{"H291 RIF"}="1H29 RIF";
$resn_exch{"H292 RIF"}="2H29 RIF"; $resn_exch{"H311 RIF"}="1H31 RIF"; $resn_exch{"H312 RIF"}="2H31 RIF";
$resn_exch{"H313 RIF"}="3H31 RIF"; $resn_exch{"H321 RIF"}="1H32 RIF"; $resn_exch{"H322 RIF"}="2H32 RIF";
$resn_exch{"H323 RIF"}="3H32 RIF"; $resn_exch{"H331 RIF"}="1H33 RIF"; $resn_exch{"H332 RIF"}="2H33 RIF";
$resn_exch{"H333 RIF"}="3H33 RIF"; $resn_exch{"H341 RIF"}="1H34 RIF"; $resn_exch{"H342 RIF"}="2H34 RIF";
$resn_exch{"H343 RIF"}="3H34 RIF"; $resn_exch{"H361 RIF"}="1H36 RIF"; $resn_exch{"H362 RIF"}="2H36 RIF";
$resn_exch{"H371 RIF"}="1H37 RIF"; $resn_exch{"H372 RIF"}="2H37 RIF"; $resn_exch{"H373 RIF"}="3H37 RIF";
$resn_exch{"H10O RIF"}="OH10 RIF"; $resn_exch{"H17O RIF"}="OH17 RIF"; $resn_exch{"H451 RIF"}="1H45 RIF";
$resn_exch{"H452 RIF"}="2H45 RIF"; $resn_exch{"H501 RIF"}="1H50 RIF"; $resn_exch{"H502 RIF"}="2H50 RIF";
$resn_exch{"H503 RIF"}="3H50 RIF"; $resn_exch{"H551 RIF"}="1H55 RIF"; $resn_exch{"H1B1 RIG"}="1H1B RIG";
$resn_exch{"H1B2 RIG"}="2H1B RIG"; $resn_exch{"H1B3 RIG"}="3H1B RIG"; $resn_exch{"HDC1 RIG"}="1HDC RIG";
$resn_exch{"HDC2 RIG"}="2HDC RIG"; $resn_exch{"HGC1 RIG"}="1HGC RIG"; $resn_exch{"HGC2 RIG"}="2HGC RIG";
$resn_exch{"HBC1 RIG"}="1HBC RIG"; $resn_exch{"HBC2 RIG"}="2HBC RIG"; $resn_exch{"HAR1 RIG"}="1HAR RIG";
$resn_exch{"HAR2 RIG"}="2HAR RIG"; $resn_exch{"HAQ1 RIG"}="1HAQ RIG"; $resn_exch{"HAQ2 RIG"}="2HAQ RIG";
$resn_exch{"HAP1 RIG"}="1HAP RIG"; $resn_exch{"HAP2 RIG"}="2HAP RIG"; $resn_exch{"HAW1 RIG"}="1HAW RIG";
$resn_exch{"HAW2 RIG"}="2HAW RIG"; $resn_exch{"HBD1 RIG"}="1HBD RIG"; $resn_exch{"HBD2 RIG"}="2HBD RIG";
$resn_exch{"HBQ1 RIG"}="1HBQ RIG"; $resn_exch{"HBQ2 RIG"}="2HBQ RIG"; $resn_exch{"HBR1 RIG"}="1HBR RIG";
$resn_exch{"HBR2 RIG"}="2HBR RIG"; $resn_exch{"HBN1 RIG"}="1HBN RIG"; $resn_exch{"HBN2 RIG"}="2HBN RIG";
$resn_exch{" H11 RIL"}="1H1  RIL"; $resn_exch{" H12 RIL"}="2H1  RIL"; $resn_exch{" H51 RIL"}="1H5  RIL";
$resn_exch{" H52 RIL"}="2H5  RIL"; $resn_exch{" H71 RIL"}="1H7  RIL"; $resn_exch{" H72 RIL"}="2H7  RIL";
$resn_exch{"H101 RIL"}="1H10 RIL"; $resn_exch{"H102 RIL"}="2H10 RIL"; $resn_exch{"H131 RIL"}="1H13 RIL";
$resn_exch{"H132 RIL"}="2H13 RIL"; $resn_exch{"H231 RIL"}="1H23 RIL"; $resn_exch{"H232 RIL"}="2H23 RIL";
$resn_exch{"H271 RIL"}="1H27 RIL"; $resn_exch{"H272 RIL"}="2H27 RIL"; $resn_exch{"H341 RIL"}="1H34 RIL";
$resn_exch{"H342 RIL"}="2H34 RIL"; $resn_exch{"H343 RIL"}="3H34 RIL"; $resn_exch{"H381 RIL"}="1H38 RIL";
$resn_exch{"H382 RIL"}="2H38 RIL"; $resn_exch{"H383 RIL"}="3H38 RIL"; $resn_exch{" HA1 RIM"}="1HA  RIM";
$resn_exch{" HA2 RIM"}="2HA  RIM"; $resn_exch{" HA3 RIM"}="3HA  RIM"; $resn_exch{"HNC1 RIM"}="1HNC RIM";
$resn_exch{"HNC2 RIM"}="2HNC RIM"; $resn_exch{"HE11 RIM"}="1HE1 RIM"; $resn_exch{"HE12 RIM"}="2HE1 RIM";
$resn_exch{"HE21 RIM"}="1HE2 RIM"; $resn_exch{"HE22 RIM"}="2HE2 RIM"; $resn_exch{"HE31 RIM"}="1HE3 RIM";
$resn_exch{"HE32 RIM"}="2HE3 RIM"; $resn_exch{"HG11 RIM"}="1HG1 RIM"; $resn_exch{"HG12 RIM"}="2HG1 RIM";
$resn_exch{"HG21 RIM"}="1HG2 RIM"; $resn_exch{"HG22 RIM"}="2HG2 RIM"; $resn_exch{"HG31 RIM"}="1HG3 RIM";
$resn_exch{"HG32 RIM"}="2HG3 RIM"; $resn_exch{" HB1 RIN"}="1HB  RIN"; $resn_exch{" HB2 RIN"}="2HB  RIN";
$resn_exch{" HG1 RIN"}="1HG  RIN"; $resn_exch{" HG2 RIN"}="2HG  RIN"; $resn_exch{" HD1 RIN"}="1HD  RIN";
$resn_exch{" HD2 RIN"}="2HD  RIN"; $resn_exch{" HE1 RIN"}="1HE  RIN"; $resn_exch{" HE2 RIN"}="2HE  RIN";
$resn_exch{" HZ1 RIN"}="1HZ  RIN"; $resn_exch{" HZ2 RIN"}="2HZ  RIN"; $resn_exch{" H51 RIN"}="1H5  RIN";
$resn_exch{" H52 RIN"}="2H5  RIN"; $resn_exch{" H41 RIN"}="1H4  RIN"; $resn_exch{" H42 RIN"}="2H4  RIN";
$resn_exch{" H31 RIN"}="1H3  RIN"; $resn_exch{" H32 RIN"}="2H3  RIN"; $resn_exch{" H21 RIN"}="1H2  RIN";
$resn_exch{" H22 RIN"}="2H2  RIN"; $resn_exch{" H11 RIN"}="1H1  RIN"; $resn_exch{" H12 RIN"}="2H1  RIN";
$resn_exch{" H51 RIP"}="1H5  RIP"; $resn_exch{" H52 RIP"}="2H5  RIP"; $resn_exch{"HC71 RIS"}="1HC7 RIS";
$resn_exch{"HC72 RIS"}="2HC7 RIS"; $resn_exch{" H61 RIT"}="1H6  RIT"; $resn_exch{"H62A RIT"}="2H6  RIT";
$resn_exch{"H141 RIT"}="1H14 RIT"; $resn_exch{"H142 RIT"}="2H14 RIT"; $resn_exch{"H261 RIT"}="1H26 RIT";
$resn_exch{"H262 RIT"}="2H26 RIT"; $resn_exch{"H441 RIT"}="1H44 RIT"; $resn_exch{"H442 RIT"}="2H44 RIT";
$resn_exch{"H641 RIT"}="1H64 RIT"; $resn_exch{"H642 RIT"}="2H64 RIT"; $resn_exch{"H643 RIT"}="3H64 RIT";
$resn_exch{"H681 RIT"}="1H68 RIT"; $resn_exch{"H682 RIT"}="2H68 RIT"; $resn_exch{"H683 RIT"}="3H68 RIT";
$resn_exch{"H751 RIT"}="1H75 RIT"; $resn_exch{"H752 RIT"}="2H75 RIT"; $resn_exch{"H861 RIT"}="1H86 RIT";
$resn_exch{"H862 RIT"}="2H86 RIT"; $resn_exch{"H863 RIT"}="3H86 RIT"; $resn_exch{"H901 RIT"}="1H90 RIT";
$resn_exch{"H902 RIT"}="2H90 RIT"; $resn_exch{"H903 RIT"}="3H90 RIT"; $resn_exch{"H951 RIT"}="1H95 RIT";
$resn_exch{"H952 RIT"}="2H95 RIT"; $resn_exch{"H953 RIT"}="3H95 RIT"; $resn_exch{" H21 RL2"}="1H2  RL2";
$resn_exch{" H22 RL2"}="2H2  RL2"; $resn_exch{" H31 RL2"}="1H3  RL2"; $resn_exch{"H101 RL2"}="1H10 RL2";
$resn_exch{"H121 RL2"}="1H12 RL2"; $resn_exch{"H122 RL2"}="2H12 RL2"; $resn_exch{"H131 RL2"}="1H13 RL2";
$resn_exch{"H132 RL2"}="2H13 RL2"; $resn_exch{"H151 RL2"}="1H15 RL2"; $resn_exch{"H161 RL2"}="1H16 RL2";
$resn_exch{"H171 RL2"}="1H17 RL2"; $resn_exch{"H181 RL2"}="1H18 RL2"; $resn_exch{"H191 RL2"}="1H19 RL2";
$resn_exch{"H211 RL2"}="1H21 RL2"; $resn_exch{"H212 RL2"}="2H21 RL2"; $resn_exch{"H221 RL2"}="1H22 RL2";
$resn_exch{"H222 RL2"}="2H22 RL2"; $resn_exch{"H231 RL2"}="1H23 RL2"; $resn_exch{"H251 RL2"}="1H25 RL2";
$resn_exch{"H252 RL2"}="2H25 RL2"; $resn_exch{"H253 RL2"}="3H25 RL2"; $resn_exch{"H241 RL2"}="1H24 RL2";
$resn_exch{"H242 RL2"}="2H24 RL2"; $resn_exch{"H243 RL2"}="3H24 RL2"; $resn_exch{"H19O RLP"}="OH19 RLP";
$resn_exch{"H171 RLP"}="1H17 RLP"; $resn_exch{"H172 RLP"}="2H17 RLP"; $resn_exch{"H161 RLP"}="1H16 RLP";
$resn_exch{"H162 RLP"}="2H16 RLP"; $resn_exch{"H111 RLP"}="1H11 RLP"; $resn_exch{"H112 RLP"}="2H11 RLP";
$resn_exch{"H121 RLP"}="1H12 RLP"; $resn_exch{"H12O RLP"}="OH12 RLP"; $resn_exch{"H131 RLP"}="1H13 RLP";
$resn_exch{"H13O RLP"}="OH13 RLP"; $resn_exch{"H141 RLP"}="1H14 RLP"; $resn_exch{"H14O RLP"}="OH14 RLP";
$resn_exch{"H151 RLP"}="1H15 RLP"; $resn_exch{"H152 RLP"}="2H15 RLP"; $resn_exch{"H15O RLP"}="OH15 RLP";
$resn_exch{"H101 RM1"}="1H10 RM1"; $resn_exch{"H102 RM1"}="2H10 RM1"; $resn_exch{"H103 RM1"}="3H10 RM1";
$resn_exch{"H2C1 RM1"}="1H2C RM1"; $resn_exch{"H2C2 RM1"}="2H2C RM1"; $resn_exch{"H1C1 RM1"}="1H1C RM1";
$resn_exch{"H1C2 RM1"}="2H1C RM1"; $resn_exch{"H131 RM2"}="1H13 RM2"; $resn_exch{"H132 RM2"}="2H13 RM2";
$resn_exch{"H1C1 RM2"}="1H1C RM2"; $resn_exch{"H1C2 RM2"}="2H1C RM2"; $resn_exch{"H2C1 RM2"}="1H2C RM2";
$resn_exch{"H2C2 RM2"}="2H2C RM2"; $resn_exch{"H111 RMA"}="1H11 RMA"; $resn_exch{"H112 RMA"}="2H11 RMA";
$resn_exch{"H101 RMA"}="1H10 RMA"; $resn_exch{"H102 RMA"}="2H10 RMA"; $resn_exch{"H103 RMA"}="3H10 RMA";
$resn_exch{" H11 RMA"}="1H1  RMA"; $resn_exch{" H12 RMA"}="2H1  RMA"; $resn_exch{" H21 RMA"}="1H2  RMA";
$resn_exch{" H22 RMA"}="2H2  RMA"; $resn_exch{" H21 RMB"}="1H2  RMB"; $resn_exch{" H41 RMB"}="1H4  RMB";
$resn_exch{" H61 RMB"}="1H6  RMB"; $resn_exch{" H71 RMB"}="1H7  RMB"; $resn_exch{" H81 RMB"}="1H8  RMB";
$resn_exch{" H82 RMB"}="2H8  RMB"; $resn_exch{" H83 RMB"}="3H8  RMB"; $resn_exch{"HOP2 RMB"}="2HOP RMB";
$resn_exch{"HOP3 RMB"}="3HOP RMB"; $resn_exch{"H5'1 RMB"}="1H5* RMB"; $resn_exch{"H5'2 RMB"}="2H5* RMB";
$resn_exch{" H4' RMB"}=" H4* RMB"; $resn_exch{" H3' RMB"}=" H3* RMB"; $resn_exch{"HO3' RMB"}="*HO3 RMB";
$resn_exch{" H2' RMB"}=" H2* RMB"; $resn_exch{"HO2' RMB"}="*HO2 RMB"; $resn_exch{" H1' RMB"}=" H1* RMB";
$resn_exch{"HAQ1 RMC"}="1HAQ RMC"; $resn_exch{"HAQ2 RMC"}="2HAQ RMC"; $resn_exch{"HAK1 RMC"}="1HAK RMC";
$resn_exch{"HAK2 RMC"}="2HAK RMC"; $resn_exch{"HAI1 RMC"}="1HAI RMC"; $resn_exch{"HAI2 RMC"}="2HAI RMC";
$resn_exch{"HAI3 RMC"}="3HAI RMC"; $resn_exch{"HAY1 RMC"}="1HAY RMC"; $resn_exch{"HAY2 RMC"}="2HAY RMC";
$resn_exch{"HAY3 RMC"}="3HAY RMC"; $resn_exch{"HBA1 RMC"}="1HBA RMC"; $resn_exch{"HBA2 RMC"}="2HBA RMC";
$resn_exch{"HBA3 RMC"}="3HBA RMC"; $resn_exch{"HAD1 RMC"}="1HAD RMC"; $resn_exch{"HAD2 RMC"}="2HAD RMC";
$resn_exch{"HBB1 RMC"}="1HBB RMC"; $resn_exch{"HBB2 RMC"}="2HBB RMC"; $resn_exch{"HBB3 RMC"}="3HBB RMC";
$resn_exch{"HMP1 RMP"}="1HMP RMP"; $resn_exch{"HMP2 RMP"}="2HMP RMP"; $resn_exch{"HMP3 RMP"}="3HMP RMP";
$resn_exch{" H5' RMP"}="1H5* RMP"; $resn_exch{"H5'' RMP"}="2H5* RMP"; $resn_exch{" H4' RMP"}=" H4* RMP";
$resn_exch{" H1' RMP"}=" H1* RMP"; $resn_exch{" H61 RMP"}="1H6  RMP"; $resn_exch{" H62 RMP"}="2H6  RMP";
$resn_exch{" H3' RMP"}=" H3* RMP"; $resn_exch{" H2' RMP"}="1H2* RMP"; $resn_exch{"H2'' RMP"}="2H2* RMP";
$resn_exch{"HO3' RMP"}="H3T  RMP"; $resn_exch{"HOP3 RMP"}="3HOP RMP"; $resn_exch{"H131 RNA"}="1H13 RNA";
$resn_exch{"H132 RNA"}="2H13 RNA"; $resn_exch{"H133 RNA"}="3H13 RNA"; $resn_exch{" H41 RNA"}="1H4  RNA";
$resn_exch{" H42 RNA"}="2H4  RNA"; $resn_exch{" H43 RNA"}="3H4  RNA"; $resn_exch{" HA1 RNG"}="1HA  RNG";
$resn_exch{" HA2 RNG"}="2HA  RNG"; $resn_exch{" HG1 RNG"}="1HG  RNG"; $resn_exch{" HG2 RNG"}="2HG  RNG";
$resn_exch{" HF1 RNG"}="1HF  RNG"; $resn_exch{" HF2 RNG"}="2HF  RNG"; $resn_exch{" HE1 RNG"}="1HE  RNG";
$resn_exch{" HE2 RNG"}="2HE  RNG"; $resn_exch{" HD1 RNG"}="1HD  RNG"; $resn_exch{" HD2 RNG"}="2HD  RNG";
$resn_exch{" HC1 RNG"}="1HC  RNG"; $resn_exch{" HC2 RNG"}="2HC  RNG"; $resn_exch{" HB1 RNG"}="1HB  RNG";
$resn_exch{" HB2 RNG"}="2HB  RNG"; $resn_exch{" H81 RNO"}="1H8  RNO"; $resn_exch{" H82 RNO"}="2H8  RNO";
$resn_exch{"H111 RNP"}="1H11 RNP"; $resn_exch{"H112 RNP"}="2H11 RNP"; $resn_exch{"H151 RNP"}="1H15 RNP";
$resn_exch{"H152 RNP"}="2H15 RNP"; $resn_exch{"H153 RNP"}="3H15 RNP"; $resn_exch{"H161 RNP"}="1H16 RNP";
$resn_exch{"H162 RNP"}="2H16 RNP"; $resn_exch{"H163 RNP"}="3H16 RNP"; $resn_exch{" H61 RNS"}="1H6  RNS";
$resn_exch{" H62 RNS"}="2H6  RNS"; $resn_exch{" H63 RNS"}="3H6  RNS"; $resn_exch{" H61 RNT"}="1H6  RNT";
$resn_exch{" H62 RNT"}="2H6  RNT"; $resn_exch{" H63 RNT"}="3H6  RNT"; $resn_exch{" H71 RO0"}="1H7  RO0";
$resn_exch{" H72 RO0"}="2H7  RO0"; $resn_exch{"H111 RO0"}="1H11 RO0"; $resn_exch{"H112 RO0"}="2H11 RO0";
$resn_exch{"H191 RO0"}="1H19 RO0"; $resn_exch{"H192 RO0"}="2H19 RO0"; $resn_exch{"H193 RO0"}="3H19 RO0";
$resn_exch{"H201 RO0"}="1H20 RO0"; $resn_exch{"H202 RO0"}="2H20 RO0"; $resn_exch{"H211 RO0"}="1H21 RO0";
$resn_exch{"H212 RO0"}="2H21 RO0"; $resn_exch{"H221 RO0"}="1H22 RO0"; $resn_exch{"H222 RO0"}="2H22 RO0";
$resn_exch{"H223 RO0"}="3H22 RO0"; $resn_exch{"H251 RO0"}="1H25 RO0"; $resn_exch{"H252 RO0"}="2H25 RO0";
$resn_exch{"H261 RO0"}="1H26 RO0"; $resn_exch{"H262 RO0"}="2H26 RO0"; $resn_exch{"H311 RO0"}="1H31 RO0";
$resn_exch{"H312 RO0"}="2H31 RO0"; $resn_exch{"H313 RO0"}="3H31 RO0"; $resn_exch{"H331 RO0"}="1H33 RO0";
$resn_exch{"H332 RO0"}="2H33 RO0"; $resn_exch{"H333 RO0"}="3H33 RO0"; $resn_exch{"H401 RO0"}="1H40 RO0";
$resn_exch{"H402 RO0"}="2H40 RO0"; $resn_exch{"H403 RO0"}="3H40 RO0"; $resn_exch{"H371 RO1"}="1H37 RO1";
$resn_exch{"H372 RO1"}="2H37 RO1"; $resn_exch{"H373 RO1"}="3H37 RO1"; $resn_exch{"H331 RO1"}="1H33 RO1";
$resn_exch{"H332 RO1"}="2H33 RO1"; $resn_exch{"H333 RO1"}="3H33 RO1"; $resn_exch{"H341 RO1"}="1H34 RO1";
$resn_exch{"H342 RO1"}="2H34 RO1"; $resn_exch{"H343 RO1"}="3H34 RO1"; $resn_exch{"H361 RO1"}="1H36 RO1";
$resn_exch{"H362 RO1"}="2H36 RO1"; $resn_exch{"H191 RO1"}="1H19 RO1"; $resn_exch{"H192 RO1"}="2H19 RO1";
$resn_exch{" H81 RO1"}="1H8  RO1"; $resn_exch{" H82 RO1"}="2H8  RO1"; $resn_exch{"H231 RO1"}="1H23 RO1";
$resn_exch{"H232 RO1"}="2H23 RO1"; $resn_exch{"H233 RO1"}="3H23 RO1"; $resn_exch{"H221 RO1"}="1H22 RO1";
$resn_exch{"H222 RO1"}="2H22 RO1"; $resn_exch{"H223 RO1"}="3H22 RO1"; $resn_exch{"H211 RO1"}="1H21 RO1";
$resn_exch{"H212 RO1"}="2H21 RO1"; $resn_exch{"H213 RO1"}="3H21 RO1"; $resn_exch{" H91 RO1"}="1H9  RO1";
$resn_exch{" H92 RO1"}="2H9  RO1"; $resn_exch{"H141 RO1"}="1H14 RO1"; $resn_exch{"H142 RO1"}="2H14 RO1";
$resn_exch{" H71 RO1"}="1H7  RO1"; $resn_exch{" H72 RO1"}="2H7  RO1"; $resn_exch{"H161 RO1"}="1H16 RO1";
$resn_exch{"H162 RO1"}="2H16 RO1"; $resn_exch{"H201 RO1"}="1H20 RO1"; $resn_exch{"H202 RO1"}="2H20 RO1";
$resn_exch{"H181 RO1"}="1H18 RO1"; $resn_exch{"H182 RO1"}="2H18 RO1"; $resn_exch{"H211 RO2"}="1H21 RO2";
$resn_exch{"H212 RO2"}="2H21 RO2"; $resn_exch{"H221 RO2"}="1H22 RO2"; $resn_exch{"H222 RO2"}="2H22 RO2";
$resn_exch{"H231 RO2"}="1H23 RO2"; $resn_exch{"H232 RO2"}="2H23 RO2"; $resn_exch{"H241 RO2"}="1H24 RO2";
$resn_exch{"H242 RO2"}="2H24 RO2"; $resn_exch{"H253 RO2"}="3H25 RO2"; $resn_exch{"H252 RO2"}="2H25 RO2";
$resn_exch{"H251 RO2"}="1H25 RO2"; $resn_exch{" H91 RO4"}="1H9  RO4"; $resn_exch{" H92 RO4"}="2H9  RO4";
$resn_exch{"H311 RO4"}="1H31 RO4"; $resn_exch{"H312 RO4"}="2H31 RO4"; $resn_exch{"H151 RO4"}="1H15 RO4";
$resn_exch{"H152 RO4"}="2H15 RO4"; $resn_exch{"H191 RO4"}="1H19 RO4"; $resn_exch{"H192 RO4"}="2H19 RO4";
$resn_exch{"H321 RO4"}="1H32 RO4"; $resn_exch{"H322 RO4"}="2H32 RO4"; $resn_exch{"H323 RO4"}="3H32 RO4";
$resn_exch{"H171 RO4"}="1H17 RO4"; $resn_exch{"H172 RO4"}="2H17 RO4"; $resn_exch{"H173 RO4"}="3H17 RO4";
$resn_exch{"H181 RO4"}="1H18 RO4"; $resn_exch{"H182 RO4"}="2H18 RO4"; $resn_exch{"H183 RO4"}="3H18 RO4";
$resn_exch{"H211 RO4"}="1H21 RO4"; $resn_exch{"H212 RO4"}="2H21 RO4"; $resn_exch{"H213 RO4"}="3H21 RO4";
$resn_exch{"H221 RO4"}="1H22 RO4"; $resn_exch{"H222 RO4"}="2H22 RO4"; $resn_exch{"H223 RO4"}="3H22 RO4";
$resn_exch{"H271 RO4"}="1H27 RO4"; $resn_exch{"H272 RO4"}="2H27 RO4"; $resn_exch{"H273 RO4"}="3H27 RO4";
$resn_exch{"H1'1 ROB"}="1H1' ROB"; $resn_exch{"H1'2 ROB"}="2H1' ROB"; $resn_exch{"HO2' ROB"}="'HO2 ROB";
$resn_exch{"HO3' ROB"}="'HO3 ROB"; $resn_exch{"H5'1 ROB"}="1H5' ROB"; $resn_exch{"H5'2 ROB"}="2H5' ROB";
$resn_exch{"HOP2 ROB"}="2HOP ROB"; $resn_exch{"HOP3 ROB"}="3HOP ROB"; $resn_exch{" H31 ROC"}="1H3  ROC";
$resn_exch{" H41 ROC"}="1H4  ROC"; $resn_exch{" H61 ROC"}="1H6  ROC"; $resn_exch{" H71 ROC"}="1H7  ROC";
$resn_exch{" H81 ROC"}="1H8  ROC"; $resn_exch{" H91 ROC"}="1H9  ROC"; $resn_exch{"HN21 ROC"}="1HN2 ROC";
$resn_exch{"H111 ROC"}="1H11 ROC"; $resn_exch{"H131 ROC"}="1H13 ROC"; $resn_exch{"H132 ROC"}="2H13 ROC";
$resn_exch{"HN31 ROC"}="1HN3 ROC"; $resn_exch{"HN32 ROC"}="2HN3 ROC"; $resn_exch{"H151 ROC"}="1H15 ROC";
$resn_exch{"HO41 ROC"}="1HO4 ROC"; $resn_exch{"H161 ROC"}="1H16 ROC"; $resn_exch{"HN41 ROC"}="1HN4 ROC";
$resn_exch{"H171 ROC"}="1H17 ROC"; $resn_exch{"H172 ROC"}="2H17 ROC"; $resn_exch{"H191 ROC"}="1H19 ROC";
$resn_exch{"H201 ROC"}="1H20 ROC"; $resn_exch{"H211 ROC"}="1H21 ROC"; $resn_exch{"H221 ROC"}="1H22 ROC";
$resn_exch{"H231 ROC"}="1H23 ROC"; $resn_exch{"H241 ROC"}="1H24 ROC"; $resn_exch{"H242 ROC"}="2H24 ROC";
$resn_exch{"H251 ROC"}="1H25 ROC"; $resn_exch{"H271 ROC"}="1H27 ROC"; $resn_exch{"H272 ROC"}="2H27 ROC";
$resn_exch{"H281 ROC"}="1H28 ROC"; $resn_exch{"H291 ROC"}="1H29 ROC"; $resn_exch{"H292 ROC"}="2H29 ROC";
$resn_exch{"H301 ROC"}="1H30 ROC"; $resn_exch{"H302 ROC"}="2H30 ROC"; $resn_exch{"H311 ROC"}="1H31 ROC";
$resn_exch{"H312 ROC"}="2H31 ROC"; $resn_exch{"H321 ROC"}="1H32 ROC"; $resn_exch{"H322 ROC"}="2H32 ROC";
$resn_exch{"H331 ROC"}="1H33 ROC"; $resn_exch{"H341 ROC"}="1H34 ROC"; $resn_exch{"H342 ROC"}="2H34 ROC";
$resn_exch{"HN61 ROC"}="1HN6 ROC"; $resn_exch{"H361 ROC"}="1H36 ROC"; $resn_exch{"H362 ROC"}="2H36 ROC";
$resn_exch{"H363 ROC"}="3H36 ROC"; $resn_exch{"H371 ROC"}="1H37 ROC"; $resn_exch{"H372 ROC"}="2H37 ROC";
$resn_exch{"H373 ROC"}="3H37 ROC"; $resn_exch{"H381 ROC"}="1H38 ROC"; $resn_exch{"H382 ROC"}="2H38 ROC";
$resn_exch{"H383 ROC"}="3H38 ROC"; $resn_exch{"H231 ROF"}="1H23 ROF"; $resn_exch{"H232 ROF"}="2H23 ROF";
$resn_exch{"H221 ROF"}="1H22 ROF"; $resn_exch{"H222 ROF"}="2H22 ROF"; $resn_exch{"H201 ROF"}="1H20 ROF";
$resn_exch{"H202 ROF"}="2H20 ROF"; $resn_exch{"HN21 ROI"}="1HN2 ROI"; $resn_exch{"HN22 ROI"}="2HN2 ROI";
$resn_exch{"H161 ROI"}="1H16 ROI"; $resn_exch{"H162 ROI"}="2H16 ROI"; $resn_exch{"H163 ROI"}="3H16 ROI";
$resn_exch{"H151 ROI"}="1H15 ROI"; $resn_exch{"H152 ROI"}="2H15 ROI"; $resn_exch{"H153 ROI"}="3H15 ROI";
$resn_exch{" H61 ROL"}="1H6  ROL"; $resn_exch{" H91 ROL"}="1H9  ROL"; $resn_exch{"H101 ROL"}="1H10 ROL";
$resn_exch{" H31 ROL"}="1H3  ROL"; $resn_exch{" H21 ROL"}="1H2  ROL"; $resn_exch{" H22 ROL"}="2H2  ROL";
$resn_exch{" H41 ROL"}="1H4  ROL"; $resn_exch{" H42 ROL"}="2H4  ROL"; $resn_exch{"H111 ROL"}="1H11 ROL";
$resn_exch{"H151 ROL"}="1H15 ROL"; $resn_exch{"H152 ROL"}="2H15 ROL"; $resn_exch{"H141 ROL"}="1H14 ROL";
$resn_exch{"H142 ROL"}="2H14 ROL"; $resn_exch{"H131 ROL"}="1H13 ROL"; $resn_exch{"H132 ROL"}="2H13 ROL";
$resn_exch{"H121 ROL"}="1H12 ROL"; $resn_exch{"H122 ROL"}="2H12 ROL"; $resn_exch{"H161 ROL"}="1H16 ROL";
$resn_exch{"H162 ROL"}="2H16 ROL"; $resn_exch{"H163 ROL"}="3H16 ROL"; $resn_exch{"H2'2 ROM"}="2H2' ROM";
$resn_exch{"H3'2 ROM"}="2H3' ROM"; $resn_exch{"H6B1 ROM"}="1H6B ROM"; $resn_exch{"H6B3 ROM"}="3H6B ROM";
$resn_exch{"H6B2 ROM"}="2H6B ROM"; $resn_exch{"H6A1 ROM"}="1H6A ROM"; $resn_exch{"H6A3 ROM"}="3H6A ROM";
$resn_exch{"H6A2 ROM"}="2H6A ROM"; $resn_exch{"H4B1 ROM"}="1H4B ROM"; $resn_exch{"H4B3 ROM"}="3H4B ROM";
$resn_exch{"H4B2 ROM"}="2H4B ROM"; $resn_exch{"H4A1 ROM"}="1H4A ROM"; $resn_exch{"H4A3 ROM"}="3H4A ROM";
$resn_exch{"H4A2 ROM"}="2H4A ROM"; $resn_exch{"H3A1 ROM"}="1H3A ROM"; $resn_exch{"H3A2 ROM"}="2H3A ROM";
$resn_exch{"H7A1 ROM"}="1H7A ROM"; $resn_exch{"H7A3 ROM"}="3H7A ROM"; $resn_exch{"H7A2 ROM"}="2H7A ROM";
$resn_exch{"H142 ROM"}="2H14 ROM"; $resn_exch{"H141 ROM"}="1H14 ROM"; $resn_exch{"H172 ROM"}="2H17 ROM";
$resn_exch{"H171 ROM"}="1H17 ROM"; $resn_exch{"H183 ROM"}="3H18 ROM"; $resn_exch{"H182 ROM"}="2H18 ROM";
$resn_exch{"H184 ROM"}="4H18 ROM"; $resn_exch{"H232 ROM"}="2H23 ROM"; $resn_exch{" HN1 RON"}="1HN  RON";
$resn_exch{" HN2 RON"}="2HN  RON"; $resn_exch{" HB1 RON"}="1HB  RON"; $resn_exch{" HB2 RON"}="2HB  RON";
$resn_exch{" HG1 RON"}="1HG  RON"; $resn_exch{" HG2 RON"}="2HG  RON"; $resn_exch{" HD1 RON"}="1HD  RON";
$resn_exch{" HD2 RON"}="2HD  RON"; $resn_exch{" HD3 RON"}="3HD  RON"; $resn_exch{"HCA1 ROP"}="1HCA ROP";
$resn_exch{"HCA2 ROP"}="2HCA ROP"; $resn_exch{"HCA3 ROP"}="3HCA ROP"; $resn_exch{"HCB1 ROP"}="1HCB ROP";
$resn_exch{"HCB2 ROP"}="2HCB ROP"; $resn_exch{"HND1 ROP"}="1HND ROP"; $resn_exch{"HND2 ROP"}="2HND ROP";
$resn_exch{"H201 ROS"}="1H20 ROS"; $resn_exch{"H202 ROS"}="2H20 ROS"; $resn_exch{"H203 ROS"}="3H20 ROS";
$resn_exch{"H211 ROS"}="1H21 ROS"; $resn_exch{"H212 ROS"}="2H21 ROS"; $resn_exch{"H213 ROS"}="3H21 ROS";
$resn_exch{"H221 ROS"}="1H22 ROS"; $resn_exch{"H222 ROS"}="2H22 ROS"; $resn_exch{"H223 ROS"}="3H22 ROS";
$resn_exch{"H231 ROS"}="1H23 ROS"; $resn_exch{"H232 ROS"}="2H23 ROS"; $resn_exch{"H233 ROS"}="3H23 ROS";
$resn_exch{" H71 ROX"}="1H7  ROX"; $resn_exch{" H72 ROX"}="2H7  ROX"; $resn_exch{"H341 ROX"}="1H34 ROX";
$resn_exch{"H342 ROX"}="2H34 ROX"; $resn_exch{"H343 ROX"}="3H34 ROX"; $resn_exch{"H331 ROX"}="1H33 ROX";
$resn_exch{"H332 ROX"}="2H33 ROX"; $resn_exch{"H333 ROX"}="3H33 ROX"; $resn_exch{"H351 ROX"}="1H35 ROX";
$resn_exch{"H352 ROX"}="2H35 ROX"; $resn_exch{"H353 ROX"}="3H35 ROX"; $resn_exch{"H361 ROX"}="1H36 ROX";
$resn_exch{"H362 ROX"}="2H36 ROX"; $resn_exch{"H301 ROX"}="1H30 ROX"; $resn_exch{"H302 ROX"}="2H30 ROX";
$resn_exch{"H303 ROX"}="3H30 ROX"; $resn_exch{"H321 ROX"}="1H32 ROX"; $resn_exch{"H322 ROX"}="2H32 ROX";
$resn_exch{"H323 ROX"}="3H32 ROX"; $resn_exch{"H251 ROX"}="1H25 ROX"; $resn_exch{"H252 ROX"}="2H25 ROX";
$resn_exch{"H271 ROX"}="1H27 ROX"; $resn_exch{"H272 ROX"}="2H27 ROX"; $resn_exch{"H273 ROX"}="3H27 ROX";
$resn_exch{"H281 ROX"}="1H28 ROX"; $resn_exch{"H282 ROX"}="2H28 ROX"; $resn_exch{"H283 ROX"}="3H28 ROX";
$resn_exch{"H151 ROX"}="1H15 ROX"; $resn_exch{"H152 ROX"}="2H15 ROX"; $resn_exch{"H201 ROX"}="1H20 ROX";
$resn_exch{"H202 ROX"}="2H20 ROX"; $resn_exch{"H203 ROX"}="3H20 ROX"; $resn_exch{"H291 ROX"}="1H29 ROX";
$resn_exch{"H292 ROX"}="2H29 ROX"; $resn_exch{"H293 ROX"}="3H29 ROX"; $resn_exch{"H211 ROX"}="1H21 ROX";
$resn_exch{"H212 ROX"}="2H21 ROX"; $resn_exch{"H213 ROX"}="3H21 ROX"; $resn_exch{"H371 ROX"}="1H37 ROX";
$resn_exch{"H372 ROX"}="2H37 ROX"; $resn_exch{"H373 ROX"}="3H37 ROX"; $resn_exch{"H311 ROX"}="1H31 ROX";
$resn_exch{"H312 ROX"}="2H31 ROX"; $resn_exch{"H313 ROX"}="3H31 ROX"; $resn_exch{"H191 ROX"}="1H19 ROX";
$resn_exch{"H192 ROX"}="2H19 ROX"; $resn_exch{"H193 ROX"}="3H19 ROX"; $resn_exch{"H381 ROX"}="1H38 ROX";
$resn_exch{"H382 ROX"}="2H38 ROX"; $resn_exch{"H391 ROX"}="1H39 ROX"; $resn_exch{"H392 ROX"}="2H39 ROX";
$resn_exch{"H401 ROX"}="1H40 ROX"; $resn_exch{"H402 ROX"}="2H40 ROX"; $resn_exch{"H411 ROX"}="1H41 ROX";
$resn_exch{"H412 ROX"}="2H41 ROX"; $resn_exch{"H413 ROX"}="3H41 ROX"; $resn_exch{"HOP2 RP1"}="2HOP RP1";
$resn_exch{"H5'1 RP1"}="1H5* RP1"; $resn_exch{"H5'2 RP1"}="2H5* RP1"; $resn_exch{" H4' RP1"}=" H4* RP1";
$resn_exch{" H3' RP1"}=" H3* RP1"; $resn_exch{" H2' RP1"}=" H2* RP1"; $resn_exch{"HO2' RP1"}="*HO2 RP1";
$resn_exch{" H1' RP1"}=" H1* RP1"; $resn_exch{"HN61 RP1"}="1HN6 RP1"; $resn_exch{"HN62 RP1"}="2HN6 RP1";
$resn_exch{"H121 RP4"}="1H12 RP4"; $resn_exch{"H122 RP4"}="2H12 RP4"; $resn_exch{"H5'1 RP5"}="1H5' RP5";
$resn_exch{"H5'2 RP5"}="2H5' RP5"; $resn_exch{"H6'1 RPA"}="1H6' RPA"; $resn_exch{"H6'2 RPA"}="2H6' RPA";
$resn_exch{"HO2' RPA"}="'HO2 RPA"; $resn_exch{"HO3' RPA"}="'HO3 RPA"; $resn_exch{"HO4' RPA"}="'HO4 RPA";
$resn_exch{"HO6' RPA"}="'HO6 RPA"; $resn_exch{"HM11 RPA"}="1HM1 RPA"; $resn_exch{"HM12 RPA"}="2HM1 RPA";
$resn_exch{"HM13 RPA"}="3HM1 RPA"; $resn_exch{"HM21 RPA"}="1HM2 RPA"; $resn_exch{"HM22 RPA"}="2HM2 RPA";
$resn_exch{"HM23 RPA"}="3HM2 RPA"; $resn_exch{"H21A RPA"}="1H2  RPA"; $resn_exch{" H22 RPA"}="2H2  RPA";
$resn_exch{" H31 RPA"}="1H3  RPA"; $resn_exch{" H32 RPA"}="2H3  RPA"; $resn_exch{" H41 RPA"}="1H4  RPA";
$resn_exch{" H42 RPA"}="2H4  RPA"; $resn_exch{"HM31 RPA"}="1HM3 RPA"; $resn_exch{"HM32 RPA"}="2HM3 RPA";
$resn_exch{"HM33 RPA"}="3HM3 RPA"; $resn_exch{"HM41 RPA"}="1HM4 RPA"; $resn_exch{"HM42 RPA"}="2HM4 RPA";
$resn_exch{"HM43 RPA"}="3HM4 RPA"; $resn_exch{"HM51 RPA"}="1HM5 RPA"; $resn_exch{"HM52 RPA"}="2HM5 RPA";
$resn_exch{"HM53 RPA"}="3HM5 RPA"; $resn_exch{"HM62 RPA"}="2HM6 RPA"; $resn_exch{"HM63 RPA"}="3HM6 RPA";
$resn_exch{"HM71 RPA"}="1HM7 RPA"; $resn_exch{"HM72 RPA"}="2HM7 RPA"; $resn_exch{"HM73 RPA"}="3HM7 RPA";
$resn_exch{"HM81 RPA"}="1HM8 RPA"; $resn_exch{"HM82 RPA"}="2HM8 RPA"; $resn_exch{"HM83 RPA"}="3HM8 RPA";
$resn_exch{"HM91 RPA"}="1HM9 RPA"; $resn_exch{"HM92 RPA"}="2HM9 RPA"; $resn_exch{"HM93 RPA"}="3HM9 RPA";
$resn_exch{"HM01 RPA"}="1HM0 RPA"; $resn_exch{"HM02 RPA"}="2HM0 RPA"; $resn_exch{"HM03 RPA"}="3HM0 RPA";
$resn_exch{"HM61 RPA"}="1HM6 RPA"; $resn_exch{"H5'1 RPD"}="1H5* RPD"; $resn_exch{"H5'2 RPD"}="2H5* RPD";
$resn_exch{"HO3' RPD"}="HO3* RPD"; $resn_exch{"HO2' RPD"}="HO2* RPD"; $resn_exch{" H51 RPD"}="1H5  RPD";
$resn_exch{" H52 RPD"}="2H5  RPD"; $resn_exch{" H61 RPD"}="1H6  RPD"; $resn_exch{" H62 RPD"}="2H6  RPD";
$resn_exch{" H71 RPD"}="1H7  RPD"; $resn_exch{" H72 RPD"}="2H7  RPD"; $resn_exch{" H81 RPD"}="1H8  RPD";
$resn_exch{"H111 RPD"}="1H11 RPD"; $resn_exch{"H112 RPD"}="2H11 RPD"; $resn_exch{"H1C1 RPF"}="1H1C RPF";
$resn_exch{"H1C2 RPF"}="2H1C RPF"; $resn_exch{"H1C3 RPF"}="3H1C RPF"; $resn_exch{"H4C1 RPF"}="1H4C RPF";
$resn_exch{"H4C2 RPF"}="2H4C RPF"; $resn_exch{"H211 RPF"}="1H21 RPF"; $resn_exch{"H212 RPF"}="2H21 RPF";
$resn_exch{"H221 RPF"}="1H22 RPF"; $resn_exch{"H222 RPF"}="2H22 RPF"; $resn_exch{"H241 RPF"}="1H24 RPF";
$resn_exch{"H242 RPF"}="2H24 RPF"; $resn_exch{"H311 RPF"}="1H31 RPF"; $resn_exch{"H312 RPF"}="2H31 RPF";
$resn_exch{"H321 RPF"}="1H32 RPF"; $resn_exch{"H322 RPF"}="2H32 RPF"; $resn_exch{"H331 RPF"}="1H33 RPF";
$resn_exch{"H332 RPF"}="2H33 RPF"; $resn_exch{"H341 RPF"}="1H34 RPF"; $resn_exch{"H342 RPF"}="2H34 RPF";
$resn_exch{"H351 RPF"}="1H35 RPF"; $resn_exch{"H352 RPF"}="2H35 RPF"; $resn_exch{"H361 RPF"}="1H36 RPF";
$resn_exch{"H362 RPF"}="2H36 RPF"; $resn_exch{" H91 RPH"}="1H9  RPH"; $resn_exch{" H92 RPH"}="2H9  RPH";
$resn_exch{"H111 RPH"}="1H11 RPH"; $resn_exch{"H112 RPH"}="2H11 RPH"; $resn_exch{"H121 RPH"}="1H12 RPH";
$resn_exch{"H122 RPH"}="2H12 RPH"; $resn_exch{"H141 RPH"}="1H14 RPH"; $resn_exch{"H142 RPH"}="2H14 RPH";
$resn_exch{"H5'1 RPL"}="1H5* RPL"; $resn_exch{"H5'2 RPL"}="2H5* RPL"; $resn_exch{"HO3' RPL"}="HO3* RPL";
$resn_exch{"HO2' RPL"}="HO2* RPL"; $resn_exch{" H51 RPL"}="1H5  RPL"; $resn_exch{" H52 RPL"}="2H5  RPL";
$resn_exch{" H61 RPL"}="1H6  RPL"; $resn_exch{" H62 RPL"}="2H6  RPL"; $resn_exch{" H71 RPL"}="1H7  RPL";
$resn_exch{" H72 RPL"}="2H7  RPL"; $resn_exch{" H81 RPL"}="1H8  RPL"; $resn_exch{"H111 RPL"}="1H11 RPL";
$resn_exch{"H112 RPL"}="2H11 RPL"; $resn_exch{"HC11 RPN"}="1HC1 RPN"; $resn_exch{"HC12 RPN"}="2HC1 RPN";
$resn_exch{"HN61 RPP"}="1HN6 RPP"; $resn_exch{"HN62 RPP"}="2HN6 RPP"; $resn_exch{" H1' RPP"}=" H1* RPP";
$resn_exch{" H2' RPP"}=" H2* RPP"; $resn_exch{" H3' RPP"}=" H3* RPP"; $resn_exch{" H4' RPP"}=" H4* RPP";
$resn_exch{"HO'2 RPP"}="2HO* RPP"; $resn_exch{"HO'3 RPP"}="3HO* RPP"; $resn_exch{"H5'1 RPP"}="1H5* RPP";
$resn_exch{"H5'2 RPP"}="2H5* RPP"; $resn_exch{"HO'5 RPP"}="5HO* RPP"; $resn_exch{" H11 RPR"}="1H1  RPR";
$resn_exch{" H31 RPR"}="1H3  RPR"; $resn_exch{" H41 RPR"}="1H4  RPR"; $resn_exch{" H51 RPR"}="1H5  RPR";
$resn_exch{" H71 RPR"}="1H7  RPR"; $resn_exch{" H91 RPR"}="1H9  RPR"; $resn_exch{"H101 RPR"}="1H10 RPR";
$resn_exch{"H121 RPR"}="1H12 RPR"; $resn_exch{"H141 RPR"}="1H14 RPR"; $resn_exch{"H151 RPR"}="1H15 RPR";
$resn_exch{"H161 RPR"}="1H16 RPR"; $resn_exch{"H162 RPR"}="2H16 RPR"; $resn_exch{"H163 RPR"}="3H16 RPR";
$resn_exch{"H171 RPR"}="1H17 RPR"; $resn_exch{"H201 RPR"}="1H20 RPR"; $resn_exch{"H202 RPR"}="2H20 RPR";
$resn_exch{"H211 RPR"}="1H21 RPR"; $resn_exch{"H212 RPR"}="2H21 RPR"; $resn_exch{"H221 RPR"}="1H22 RPR";
$resn_exch{"H222 RPR"}="2H22 RPR"; $resn_exch{"H231 RPR"}="1H23 RPR"; $resn_exch{"H251 RPR"}="1H25 RPR";
$resn_exch{"H261 RPR"}="1H26 RPR"; $resn_exch{"H271 RPR"}="1H27 RPR"; $resn_exch{"H301 RPR"}="1H30 RPR";
$resn_exch{"H302 RPR"}="2H30 RPR"; $resn_exch{"H311 RPR"}="1H31 RPR"; $resn_exch{"H341 RPR"}="1H34 RPR";
$resn_exch{"H342 RPR"}="2H34 RPR"; $resn_exch{"H343 RPR"}="3H34 RPR"; $resn_exch{"H131 RPT"}="1H13 RPT";
$resn_exch{"H132 RPT"}="2H13 RPT"; $resn_exch{"H133 RPT"}="3H13 RPT"; $resn_exch{"H141 RPT"}="1H14 RPT";
$resn_exch{"H142 RPT"}="2H14 RPT"; $resn_exch{"H143 RPT"}="3H14 RPT"; $resn_exch{"H301 RPT"}="1H30 RPT";
$resn_exch{"H302 RPT"}="2H30 RPT"; $resn_exch{"H303 RPT"}="3H30 RPT"; $resn_exch{"H311 RPT"}="1H31 RPT";
$resn_exch{"H312 RPT"}="2H31 RPT"; $resn_exch{"H313 RPT"}="3H31 RPT"; $resn_exch{"H321 RPT"}="1H32 RPT";
$resn_exch{"H322 RPT"}="2H32 RPT"; $resn_exch{"H323 RPT"}="3H32 RPT"; $resn_exch{"H331 RPT"}="1H33 RPT";
$resn_exch{"H332 RPT"}="2H33 RPT"; $resn_exch{"H333 RPT"}="3H33 RPT"; $resn_exch{"H341 RPT"}="1H34 RPT";
$resn_exch{"H342 RPT"}="2H34 RPT"; $resn_exch{"H343 RPT"}="3H34 RPT"; $resn_exch{"H361 RPT"}="1H36 RPT";
$resn_exch{"H362 RPT"}="2H36 RPT"; $resn_exch{"H363 RPT"}="3H36 RPT"; $resn_exch{"H371 RPT"}="1H37 RPT";
$resn_exch{"H372 RPT"}="2H37 RPT"; $resn_exch{"H373 RPT"}="3H37 RPT"; $resn_exch{"H391 RPT"}="1H39 RPT";
$resn_exch{"H392 RPT"}="2H39 RPT"; $resn_exch{"H401 RPT"}="1H40 RPT"; $resn_exch{"H402 RPT"}="2H40 RPT";
$resn_exch{"H411 RPT"}="1H41 RPT"; $resn_exch{"H412 RPT"}="2H41 RPT"; $resn_exch{"H421 RPT"}="1H42 RPT";
$resn_exch{"H422 RPT"}="2H42 RPT"; $resn_exch{"H441 RPT"}="1H44 RPT"; $resn_exch{"H442 RPT"}="2H44 RPT";
$resn_exch{"H451 RPT"}="1H45 RPT"; $resn_exch{"H452 RPT"}="2H45 RPT"; $resn_exch{"H461 RPT"}="1H46 RPT";
$resn_exch{"H462 RPT"}="2H46 RPT"; $resn_exch{"H471 RPT"}="1H47 RPT"; $resn_exch{"H472 RPT"}="2H47 RPT";
$resn_exch{"H31A RPX"}="1H3  RPX"; $resn_exch{" H32 RPX"}="2H3  RPX"; $resn_exch{"H41A RPX"}="1H4  RPX";
$resn_exch{"H42A RPX"}="2H4  RPX"; $resn_exch{" H51 RPX"}="1H5  RPX"; $resn_exch{" H52 RPX"}="2H5  RPX";
$resn_exch{" H61 RPX"}="1H6  RPX"; $resn_exch{" H62 RPX"}="2H6  RPX"; $resn_exch{"H121 RPX"}="1H12 RPX";
$resn_exch{"H122 RPX"}="2H12 RPX"; $resn_exch{"H131 RPX"}="1H13 RPX"; $resn_exch{"H132 RPX"}="2H13 RPX";
$resn_exch{"H151 RPX"}="1H15 RPX"; $resn_exch{"H152 RPX"}="2H15 RPX"; $resn_exch{"H241 RPX"}="1H24 RPX";
$resn_exch{"H242 RPX"}="2H24 RPX"; $resn_exch{"H301 RPX"}="1H30 RPX"; $resn_exch{"H302 RPX"}="2H30 RPX";
$resn_exch{"H331 RPX"}="1H33 RPX"; $resn_exch{"H332 RPX"}="2H33 RPX"; $resn_exch{"H361 RPX"}="1H36 RPX";
$resn_exch{"H362 RPX"}="2H36 RPX"; $resn_exch{"H431 RPX"}="1H43 RPX"; $resn_exch{"H432 RPX"}="2H43 RPX";
$resn_exch{"H433 RPX"}="3H43 RPX"; $resn_exch{"H441 RPX"}="1H44 RPX"; $resn_exch{"H442 RPX"}="2H44 RPX";
$resn_exch{"H443 RPX"}="3H44 RPX"; $resn_exch{"H451 RPX"}="1H45 RPX"; $resn_exch{"H452 RPX"}="2H45 RPX";
$resn_exch{"H453 RPX"}="3H45 RPX"; $resn_exch{"H461 RPX"}="1H46 RPX"; $resn_exch{"H462 RPX"}="2H46 RPX";
$resn_exch{"H463 RPX"}="3H46 RPX"; $resn_exch{"H471 RPX"}="1H47 RPX"; $resn_exch{"H472 RPX"}="2H47 RPX";
$resn_exch{"H473 RPX"}="3H47 RPX"; $resn_exch{"H481 RPX"}="1H48 RPX"; $resn_exch{"H482 RPX"}="2H48 RPX";
$resn_exch{"H483 RPX"}="3H48 RPX"; $resn_exch{"H491 RPX"}="1H49 RPX"; $resn_exch{"H492 RPX"}="2H49 RPX";
$resn_exch{"H493 RPX"}="3H49 RPX"; $resn_exch{"H501 RPX"}="1H50 RPX"; $resn_exch{"H502 RPX"}="2H50 RPX";
$resn_exch{"H511 RPX"}="1H51 RPX"; $resn_exch{"H512 RPX"}="2H51 RPX"; $resn_exch{"H513 RPX"}="3H51 RPX";
$resn_exch{"H521 RPX"}="1H52 RPX"; $resn_exch{"H522 RPX"}="2H52 RPX"; $resn_exch{"H523 RPX"}="3H52 RPX";
$resn_exch{"H531 RPX"}="1H53 RPX"; $resn_exch{"H532 RPX"}="2H53 RPX"; $resn_exch{"H533 RPX"}="3H53 RPX";
$resn_exch{" H51 RQ3"}="1H5  RQ3"; $resn_exch{" H52 RQ3"}="2H5  RQ3"; $resn_exch{" H53 RQ3"}="3H5  RQ3";
$resn_exch{" H61 RQ3"}="1H6  RQ3"; $resn_exch{" H62 RQ3"}="2H6  RQ3"; $resn_exch{" H63 RQ3"}="3H6  RQ3";
$resn_exch{" H91 RQ3"}="1H9  RQ3"; $resn_exch{"H101 RQ3"}="1H10 RQ3"; $resn_exch{"H121 RQ3"}="1H12 RQ3";
$resn_exch{"H131 RQ3"}="1H13 RQ3"; $resn_exch{"H141 RQ3"}="1H14 RQ3"; $resn_exch{"H142 RQ3"}="2H14 RQ3";
$resn_exch{"H171 RQ3"}="1H17 RQ3"; $resn_exch{"H191 RQ3"}="1H19 RQ3"; $resn_exch{"H211 RQ3"}="1H21 RQ3";
$resn_exch{"H221 RQ3"}="1H22 RQ3"; $resn_exch{"H241 RQ3"}="1H24 RQ3"; $resn_exch{"H242 RQ3"}="2H24 RQ3";
$resn_exch{"H243 RQ3"}="3H24 RQ3"; $resn_exch{"H251 RQ3"}="1H25 RQ3"; $resn_exch{"H252 RQ3"}="2H25 RQ3";
$resn_exch{"H253 RQ3"}="3H25 RQ3"; $resn_exch{"HL11 RR1"}="1HL1 RR1"; $resn_exch{"HL12 RR1"}="2HL1 RR1";
$resn_exch{"HL21 RR1"}="1HL2 RR1"; $resn_exch{"HL22 RR1"}="2HL2 RR1"; $resn_exch{"HAQ1 RRC"}="1HAQ RRC";
$resn_exch{"HAQ2 RRC"}="2HAQ RRC"; $resn_exch{"HAK1 RRC"}="1HAK RRC"; $resn_exch{"HAK2 RRC"}="2HAK RRC";
$resn_exch{"HAI1 RRC"}="1HAI RRC"; $resn_exch{"HAI2 RRC"}="2HAI RRC"; $resn_exch{"HAI3 RRC"}="3HAI RRC";
$resn_exch{"HAY1 RRC"}="1HAY RRC"; $resn_exch{"HAY2 RRC"}="2HAY RRC"; $resn_exch{"HAY3 RRC"}="3HAY RRC";
$resn_exch{"HBA1 RRC"}="1HBA RRC"; $resn_exch{"HBA2 RRC"}="2HBA RRC"; $resn_exch{"HBA3 RRC"}="3HBA RRC";
$resn_exch{"HAD1 RRC"}="1HAD RRC"; $resn_exch{"HAD2 RRC"}="2HAD RRC"; $resn_exch{"H301 RRP"}="1H30 RRP";
$resn_exch{"H302 RRP"}="2H30 RRP"; $resn_exch{"H211 RRP"}="1H21 RRP"; $resn_exch{"H212 RRP"}="2H21 RRP";
$resn_exch{" H41 RRP"}="1H4  RRP"; $resn_exch{" H42 RRP"}="2H4  RRP"; $resn_exch{" H71 RRP"}="1H7  RRP";
$resn_exch{" H72 RRP"}="2H7  RRP"; $resn_exch{" H21 RRP"}="1H2  RRP"; $resn_exch{"H22A RRP"}="2H2  RRP";
$resn_exch{"H131 RRR"}="1H13 RRR"; $resn_exch{"H132 RRR"}="2H13 RRR"; $resn_exch{"H181 RRR"}="1H18 RRR";
$resn_exch{"H182 RRR"}="2H18 RRR"; $resn_exch{"H201 RRR"}="1H20 RRR"; $resn_exch{"H202 RRR"}="2H20 RRR";
$resn_exch{"H211 RRR"}="1H21 RRR"; $resn_exch{"H212 RRR"}="2H21 RRR"; $resn_exch{" H21 RRS"}="1H2  RRS";
$resn_exch{"H22A RRS"}="2H2  RRS"; $resn_exch{" H71 RRS"}="1H7  RRS"; $resn_exch{" H72 RRS"}="2H7  RRS";
$resn_exch{" H81 RRS"}="1H8  RRS"; $resn_exch{" H82 RRS"}="2H8  RRS"; $resn_exch{" H91 RRS"}="1H9  RRS";
$resn_exch{" H92 RRS"}="2H9  RRS"; $resn_exch{"H101 RRS"}="1H10 RRS"; $resn_exch{"H102 RRS"}="2H10 RRS";
$resn_exch{"H111 RRS"}="1H11 RRS"; $resn_exch{"H112 RRS"}="2H11 RRS"; $resn_exch{"H121 RRS"}="1H12 RRS";
$resn_exch{"H122 RRS"}="2H12 RRS"; $resn_exch{"H131 RRS"}="1H13 RRS"; $resn_exch{"H132 RRS"}="2H13 RRS";
$resn_exch{"H151 RRS"}="1H15 RRS"; $resn_exch{"H152 RRS"}="2H15 RRS"; $resn_exch{"H153 RRS"}="3H15 RRS";
$resn_exch{"H161 RRS"}="1H16 RRS"; $resn_exch{"H162 RRS"}="2H16 RRS"; $resn_exch{"H163 RRS"}="3H16 RRS";
$resn_exch{"H171 RRS"}="1H17 RRS"; $resn_exch{"H172 RRS"}="2H17 RRS"; $resn_exch{"HB21 RRT"}="1HB2 RRT";
$resn_exch{"HB22 RRT"}="2HB2 RRT"; $resn_exch{"HB11 RRT"}="1HB1 RRT"; $resn_exch{"HB12 RRT"}="2HB1 RRT";
$resn_exch{"HA21 RRT"}="1HA2 RRT"; $resn_exch{"HA22 RRT"}="2HA2 RRT"; $resn_exch{"H251 RS1"}="1H25 RS1";
$resn_exch{"H252 RS1"}="2H25 RS1"; $resn_exch{"H291 RS1"}="1H29 RS1"; $resn_exch{"H292 RS1"}="2H29 RS1";
$resn_exch{"H301 RS1"}="1H30 RS1"; $resn_exch{"H302 RS1"}="2H30 RS1"; $resn_exch{"H321 RS1"}="1H32 RS1";
$resn_exch{"H322 RS1"}="2H32 RS1"; $resn_exch{"H331 RS1"}="1H33 RS1"; $resn_exch{"H332 RS1"}="2H33 RS1";
$resn_exch{"H271 RS2"}="1H27 RS2"; $resn_exch{"H272 RS2"}="2H27 RS2"; $resn_exch{"H361 RS2"}="1H36 RS2";
$resn_exch{"H362 RS2"}="2H36 RS2"; $resn_exch{"H371 RS2"}="1H37 RS2"; $resn_exch{"H372 RS2"}="2H37 RS2";
$resn_exch{"H391 RS2"}="1H39 RS2"; $resn_exch{"H392 RS2"}="2H39 RS2"; $resn_exch{"H401 RS2"}="1H40 RS2";
$resn_exch{"H402 RS2"}="2H40 RS2"; $resn_exch{" H21 RS7"}="1H2  RS7"; $resn_exch{" H31 RS7"}="1H3  RS7";
$resn_exch{"H121 RS7"}="1H12 RS7"; $resn_exch{"H122 RS7"}="2H12 RS7"; $resn_exch{"H131 RS7"}="1H13 RS7";
$resn_exch{"H132 RS7"}="2H13 RS7"; $resn_exch{"H141 RS7"}="1H14 RS7"; $resn_exch{"H142 RS7"}="2H14 RS7";
$resn_exch{"H151 RS7"}="1H15 RS7"; $resn_exch{"H152 RS7"}="2H15 RS7"; $resn_exch{"H161 RS7"}="1H16 RS7";
$resn_exch{"H162 RS7"}="2H16 RS7"; $resn_exch{"H171 RS7"}="1H17 RS7"; $resn_exch{"H172 RS7"}="2H17 RS7";
$resn_exch{"H173 RS7"}="3H17 RS7"; $resn_exch{"H111 RSA"}="1H11 RSA"; $resn_exch{"H112 RSA"}="2H11 RSA";
$resn_exch{" H11 RSA"}="1H1  RSA"; $resn_exch{" H12 RSA"}="2H1  RSA"; $resn_exch{" H21 RSA"}="1H2  RSA";
$resn_exch{" H22 RSA"}="2H2  RSA"; $resn_exch{" H42 RSD"}="2H4  RSD"; $resn_exch{"H332 RSD"}="2H33 RSD";
$resn_exch{"H472 RSD"}="2H47 RSD"; $resn_exch{"H392 RSD"}="2H39 RSD"; $resn_exch{"H31A RSD"}="1H3  RSD";
$resn_exch{"H32A RSD"}="2H3  RSD"; $resn_exch{" H33 RSD"}="3H3  RSD"; $resn_exch{" H41 RSD"}="1H4  RSD";
$resn_exch{" H61 RSD"}="1H6  RSD"; $resn_exch{" H62 RSD"}="2H6  RSD"; $resn_exch{" H63 RSD"}="3H6  RSD";
$resn_exch{" H81 RSD"}="1H8  RSD"; $resn_exch{" H82 RSD"}="2H8  RSD"; $resn_exch{" H83 RSD"}="3H8  RSD";
$resn_exch{"H112 RSD"}="2H11 RSD"; $resn_exch{"H111 RSD"}="1H11 RSD"; $resn_exch{"H131 RSD"}="1H13 RSD";
$resn_exch{"H132 RSD"}="2H13 RSD"; $resn_exch{"H133 RSD"}="3H13 RSD"; $resn_exch{"H151 RSD"}="1H15 RSD";
$resn_exch{"H152 RSD"}="2H15 RSD"; $resn_exch{"H153 RSD"}="3H15 RSD"; $resn_exch{"H119 RSD"}="19H1 RSD";
$resn_exch{"H221 RSD"}="21H2 RSD"; $resn_exch{"H228 RSD"}="28H2 RSD"; $resn_exch{"H301 RSD"}="1H30 RSD";
$resn_exch{"H331 RSD"}="1H33 RSD"; $resn_exch{"H341 RSD"}="1H34 RSD"; $resn_exch{"H342 RSD"}="2H34 RSD";
$resn_exch{"H343 RSD"}="3H34 RSD"; $resn_exch{"H371 RSD"}="1H37 RSD"; $resn_exch{"H372 RSD"}="2H37 RSD";
$resn_exch{"H381 RSD"}="1H38 RSD"; $resn_exch{"H382 RSD"}="2H38 RSD"; $resn_exch{"H383 RSD"}="3H38 RSD";
$resn_exch{"H391 RSD"}="1H39 RSD"; $resn_exch{"H411 RSD"}="1H41 RSD"; $resn_exch{"H412 RSD"}="2H41 RSD";
$resn_exch{"H413 RSD"}="3H41 RSD"; $resn_exch{"H421 RSD"}="1H42 RSD"; $resn_exch{"H422 RSD"}="2H42 RSD";
$resn_exch{"H423 RSD"}="3H42 RSD"; $resn_exch{"H431 RSD"}="1H43 RSD"; $resn_exch{"H432 RSD"}="2H43 RSD";
$resn_exch{"H433 RSD"}="3H43 RSD"; $resn_exch{"H461 RSD"}="1H46 RSD"; $resn_exch{"H462 RSD"}="2H46 RSD";
$resn_exch{"H463 RSD"}="3H46 RSD"; $resn_exch{"H471 RSD"}="1H47 RSD"; $resn_exch{"H491 RSD"}="1H49 RSD";
$resn_exch{"H492 RSD"}="2H49 RSD"; $resn_exch{"H493 RSD"}="3H49 RSD"; $resn_exch{"H511 RSD"}="1H51 RSD";
$resn_exch{"H512 RSD"}="2H51 RSD"; $resn_exch{"H513 RSD"}="3H51 RSD"; $resn_exch{"H2C1 RSH"}="1H2C RSH";
$resn_exch{"H3C1 RSH"}="1H3C RSH"; $resn_exch{"H3C2 RSH"}="2H3C RSH"; $resn_exch{"H6C1 RSH"}="1H6C RSH";
$resn_exch{"H6C2 RSH"}="2H6C RSH"; $resn_exch{"H131 RSO"}="1H13 RSO"; $resn_exch{"H132 RSO"}="2H13 RSO";
$resn_exch{" H21 RSS"}="1H2  RSS"; $resn_exch{"H22A RSS"}="2H2  RSS"; $resn_exch{" H71 RSS"}="1H7  RSS";
$resn_exch{" H72 RSS"}="2H7  RSS"; $resn_exch{" H81 RSS"}="1H8  RSS"; $resn_exch{" H82 RSS"}="2H8  RSS";
$resn_exch{" H91 RSS"}="1H9  RSS"; $resn_exch{" H92 RSS"}="2H9  RSS"; $resn_exch{"H101 RSS"}="1H10 RSS";
$resn_exch{"H102 RSS"}="2H10 RSS"; $resn_exch{"H111 RSS"}="1H11 RSS"; $resn_exch{"H112 RSS"}="2H11 RSS";
$resn_exch{"H121 RSS"}="1H12 RSS"; $resn_exch{"H122 RSS"}="2H12 RSS"; $resn_exch{"H131 RSS"}="1H13 RSS";
$resn_exch{"H132 RSS"}="2H13 RSS"; $resn_exch{"H151 RSS"}="1H15 RSS"; $resn_exch{"H152 RSS"}="2H15 RSS";
$resn_exch{"H153 RSS"}="3H15 RSS"; $resn_exch{"H161 RSS"}="1H16 RSS"; $resn_exch{"H162 RSS"}="2H16 RSS";
$resn_exch{"H163 RSS"}="3H16 RSS"; $resn_exch{"H171 RSS"}="1H17 RSS"; $resn_exch{"H172 RSS"}="2H17 RSS";
$resn_exch{"HOP3  RT"}="3HOP  RT"; $resn_exch{"HOP2  RT"}="2HOP  RT"; $resn_exch{" H5'  RT"}="1H5*  RT";
$resn_exch{"H5''  RT"}="2H5*  RT"; $resn_exch{" H4'  RT"}=" H4*  RT"; $resn_exch{" H3'  RT"}=" H3*  RT";
$resn_exch{"HO3'  RT"}="H3T   RT"; $resn_exch{" H2'  RT"}=" H2*  RT"; $resn_exch{"HO2'  RT"}="2HO*  RT";
$resn_exch{" H1'  RT"}=" H1*  RT"; $resn_exch{" H71  RT"}="1H5M  RT"; $resn_exch{" H72  RT"}="2H5M  RT";
$resn_exch{" H73  RT"}="3H5M  RT"; $resn_exch{" H71 RTA"}="1H7  RTA"; $resn_exch{" H72 RTA"}="2H7  RTA";
$resn_exch{" H81 RTA"}="1H8  RTA"; $resn_exch{" H82 RTA"}="2H8  RTA"; $resn_exch{" H91 RTA"}="1H9  RTA";
$resn_exch{"H101 RTA"}="1H10 RTA"; $resn_exch{"H111 RTA"}="11H1 RTA"; $resn_exch{"H112 RTA"}="12H1 RTA";
$resn_exch{"H131 RTA"}="1H13 RTA"; $resn_exch{"H132 RTA"}="2H13 RTA"; $resn_exch{"H141 RTA"}="1H14 RTA";
$resn_exch{"H142 RTA"}="2H14 RTA"; $resn_exch{"H151 RTA"}="1H15 RTA"; $resn_exch{"H152 RTA"}="2H15 RTA";
$resn_exch{"H116 RTA"}="16H1 RTA"; $resn_exch{"H171 RTA"}="1H17 RTA"; $resn_exch{"H172 RTA"}="2H17 RTA";
$resn_exch{"H181 RTA"}="1H18 RTA"; $resn_exch{"H182 RTA"}="2H18 RTA"; $resn_exch{"H191 RTA"}="1H19 RTA";
$resn_exch{"H192 RTA"}="2H19 RTA"; $resn_exch{"H201 RTA"}="1H20 RTA"; $resn_exch{"H202 RTA"}="2H20 RTA";
$resn_exch{"H221 RTA"}="21H2 RTA"; $resn_exch{"H222 RTA"}="22H2 RTA"; $resn_exch{"H231 RTA"}="1H23 RTA";
$resn_exch{"H232 RTA"}="2H23 RTA"; $resn_exch{"H241 RTA"}="1H24 RTA"; $resn_exch{"H242 RTA"}="2H24 RTA";
$resn_exch{"H251 RTA"}="1H25 RTA"; $resn_exch{"H252 RTA"}="2H25 RTA"; $resn_exch{"H261 RTA"}="1H26 RTA";
$resn_exch{"H262 RTA"}="2H26 RTA"; $resn_exch{"H227 RTA"}="27H2 RTA"; $resn_exch{"H281 RTA"}="1H28 RTA";
$resn_exch{"H282 RTA"}="2H28 RTA"; $resn_exch{"H291 RTA"}="1H29 RTA"; $resn_exch{"H292 RTA"}="2H29 RTA";
$resn_exch{"H301 RTA"}="1H30 RTA"; $resn_exch{"H302 RTA"}="2H30 RTA"; $resn_exch{"H311 RTA"}="1H31 RTA";
$resn_exch{"H312 RTA"}="2H31 RTA"; $resn_exch{" H71 RTB"}="1H7  RTB"; $resn_exch{" H81 RTB"}="1H8  RTB";
$resn_exch{" H91 RTB"}="1H9  RTB"; $resn_exch{"H131 RTB"}="1H13 RTB"; $resn_exch{"H141 RTB"}="1H14 RTB";
$resn_exch{"H151 RTB"}="1H15 RTB"; $resn_exch{"H171 RTB"}="1H17 RTB"; $resn_exch{"H181 RTB"}="1H18 RTB";
$resn_exch{"H191 RTB"}="1H19 RTB"; $resn_exch{"H231 RTB"}="1H23 RTB"; $resn_exch{"H241 RTB"}="1H24 RTB";
$resn_exch{"H251 RTB"}="1H25 RTB"; $resn_exch{"H261 RTB"}="1H26 RTB"; $resn_exch{"H281 RTB"}="1H28 RTB";
$resn_exch{"H291 RTB"}="1H29 RTB"; $resn_exch{"H301 RTB"}="1H30 RTB"; $resn_exch{"H311 RTB"}="1H31 RTB";
$resn_exch{"H321 RTB"}="1H32 RTB"; $resn_exch{"H331 RTB"}="1H33 RTB"; $resn_exch{" H21 RTL"}="1H2  RTL";
$resn_exch{" H22 RTL"}="2H2  RTL"; $resn_exch{" H31 RTL"}="1H3  RTL"; $resn_exch{" H32 RTL"}="2H3  RTL";
$resn_exch{" H41 RTL"}="1H4  RTL"; $resn_exch{" H42 RTL"}="2H4  RTL"; $resn_exch{"H151 RTL"}="1H15 RTL";
$resn_exch{"H152 RTL"}="2H15 RTL"; $resn_exch{"H161 RTL"}="1H16 RTL"; $resn_exch{"H162 RTL"}="2H16 RTL";
$resn_exch{"H163 RTL"}="3H16 RTL"; $resn_exch{"H171 RTL"}="1H17 RTL"; $resn_exch{"H172 RTL"}="2H17 RTL";
$resn_exch{"H173 RTL"}="3H17 RTL"; $resn_exch{"H181 RTL"}="1H18 RTL"; $resn_exch{"H182 RTL"}="2H18 RTL";
$resn_exch{"H183 RTL"}="3H18 RTL"; $resn_exch{"H191 RTL"}="1H19 RTL"; $resn_exch{"H192 RTL"}="2H19 RTL";
$resn_exch{"H193 RTL"}="3H19 RTL"; $resn_exch{"H201 RTL"}="1H20 RTL"; $resn_exch{"H202 RTL"}="2H20 RTL";
$resn_exch{"H203 RTL"}="3H20 RTL"; $resn_exch{"HOG2 RTP"}="2HOG RTP"; $resn_exch{"HOG3 RTP"}="3HOG RTP";
$resn_exch{"HOB2 RTP"}="2HOB RTP"; $resn_exch{"HOA2 RTP"}="2HOA RTP"; $resn_exch{"H5'1 RTP"}="1H5* RTP";
$resn_exch{"H5'2 RTP"}="2H5* RTP"; $resn_exch{" H4' RTP"}=" H4* RTP"; $resn_exch{" H1' RTP"}=" H1* RTP";
$resn_exch{"HN31 RTP"}="1HN3 RTP"; $resn_exch{"HN32 RTP"}="2HN3 RTP"; $resn_exch{" H2' RTP"}=" H2* RTP";
$resn_exch{"HO2' RTP"}="*HO2 RTP"; $resn_exch{" H3' RTP"}=" H3* RTP"; $resn_exch{"HO3' RTP"}="*HO3 RTP";
$resn_exch{"H301 RTR"}="1H30 RTR"; $resn_exch{"H302 RTR"}="2H30 RTR"; $resn_exch{"H211 RTR"}="1H21 RTR";
$resn_exch{"H212 RTR"}="2H21 RTR"; $resn_exch{"H121 RTR"}="1H12 RTR"; $resn_exch{"H122 RTR"}="2H12 RTR";
$resn_exch{"H131 RTR"}="1H13 RTR"; $resn_exch{"H132 RTR"}="2H13 RTR"; $resn_exch{"H171 RTR"}="1H17 RTR";
$resn_exch{"H172 RTR"}="2H17 RTR"; $resn_exch{"H301 RU6"}="1H30 RU6"; $resn_exch{"H302 RU6"}="2H30 RU6";
$resn_exch{"H303 RU6"}="3H30 RU6"; $resn_exch{"H311 RU6"}="1H31 RU6"; $resn_exch{"H312 RU6"}="2H31 RU6";
$resn_exch{"H313 RU6"}="3H31 RU6"; $resn_exch{"H251 RU6"}="1H25 RU6"; $resn_exch{"H252 RU6"}="2H25 RU6";
$resn_exch{"H271 RU6"}="1H27 RU6"; $resn_exch{"H272 RU6"}="2H27 RU6"; $resn_exch{"H273 RU6"}="3H27 RU6";
$resn_exch{"H281 RU6"}="1H28 RU6"; $resn_exch{"H282 RU6"}="2H28 RU6"; $resn_exch{"H283 RU6"}="3H28 RU6";
$resn_exch{"H291 RU6"}="1H29 RU6"; $resn_exch{"H292 RU6"}="2H29 RU6"; $resn_exch{"H293 RU6"}="3H29 RU6";
$resn_exch{"H321 RU6"}="1H32 RU6"; $resn_exch{"H322 RU6"}="2H32 RU6"; $resn_exch{"H323 RU6"}="3H32 RU6";
$resn_exch{" H71 RU6"}="1H7  RU6"; $resn_exch{" H72 RU6"}="2H7  RU6"; $resn_exch{"H331 RU6"}="1H33 RU6";
$resn_exch{"H332 RU6"}="2H33 RU6"; $resn_exch{"H333 RU6"}="3H33 RU6"; $resn_exch{"H351 RU6"}="1H35 RU6";
$resn_exch{"H352 RU6"}="2H35 RU6"; $resn_exch{"H353 RU6"}="3H35 RU6"; $resn_exch{"H381 RU6"}="1H38 RU6";
$resn_exch{"H382 RU6"}="2H38 RU6"; $resn_exch{"H383 RU6"}="3H38 RU6"; $resn_exch{"H401 RU6"}="1H40 RU6";
$resn_exch{"H402 RU6"}="2H40 RU6"; $resn_exch{"H411 RU6"}="1H41 RU6"; $resn_exch{"H412 RU6"}="2H41 RU6";
$resn_exch{"H421 RU6"}="1H42 RU6"; $resn_exch{"H422 RU6"}="2H42 RU6"; $resn_exch{"H431 RU6"}="1H43 RU6";
$resn_exch{"H432 RU6"}="2H43 RU6"; $resn_exch{"H341 RU6"}="1H34 RU6"; $resn_exch{"H342 RU6"}="2H34 RU6";
$resn_exch{"H343 RU6"}="3H34 RU6"; $resn_exch{"H361 RU6"}="1H36 RU6"; $resn_exch{"H362 RU6"}="2H36 RU6";
$resn_exch{"H371 RU6"}="1H37 RU6"; $resn_exch{"H372 RU6"}="2H37 RU6"; $resn_exch{"H373 RU6"}="3H37 RU6";
$resn_exch{"H151 RU6"}="1H15 RU6"; $resn_exch{"H152 RU6"}="2H15 RU6"; $resn_exch{"H201 RU6"}="1H20 RU6";
$resn_exch{"H202 RU6"}="2H20 RU6"; $resn_exch{"H203 RU6"}="3H20 RU6"; $resn_exch{"H191 RU6"}="1H19 RU6";
$resn_exch{"H192 RU6"}="2H19 RU6"; $resn_exch{"H193 RU6"}="3H19 RU6"; $resn_exch{"H211 RU6"}="1H21 RU6";
$resn_exch{"H212 RU6"}="2H21 RU6"; $resn_exch{"H213 RU6"}="3H21 RU6"; $resn_exch{"H101 RU7"}="1H10 RU7";
$resn_exch{"H102 RU7"}="2H10 RU7"; $resn_exch{"H103 RU7"}="3H10 RU7"; $resn_exch{" H91 RU7"}="1H9  RU7";
$resn_exch{" H92 RU7"}="2H9  RU7"; $resn_exch{" H93 RU7"}="3H9  RU7"; $resn_exch{" H71 RU7"}="1H7  RU7";
$resn_exch{" H72 RU7"}="2H7  RU7"; $resn_exch{" H73 RU7"}="3H7  RU7"; $resn_exch{"H381 RUA"}="1H38 RUA";
$resn_exch{"H382 RUA"}="2H38 RUA"; $resn_exch{"H383 RUA"}="3H38 RUA"; $resn_exch{"H181 RUA"}="1H18 RUA";
$resn_exch{"H119 RUA"}="19H1 RUA"; $resn_exch{"H151 RUA"}="1H15 RUA"; $resn_exch{"H241 RUA"}="1H24 RUA";
$resn_exch{"H231 RUA"}="1H23 RUA"; $resn_exch{"H220 RUA"}="20H2 RUA"; $resn_exch{"H211 RUA"}="1H21 RUA";
$resn_exch{"H222 RUA"}="22H2 RUA"; $resn_exch{"H391 RUA"}="1H39 RUA"; $resn_exch{"H392 RUA"}="2H39 RUA";
$resn_exch{"H393 RUA"}="3H39 RUA"; $resn_exch{" H31 RUA"}="1H3  RUA"; $resn_exch{" H41 RUA"}="1H4  RUA";
$resn_exch{" H51 RUA"}="1H5  RUA"; $resn_exch{" H77 RUA"}="7H7  RUA"; $resn_exch{" H61 RUA"}="1H6  RUA";
$resn_exch{" H88 RUA"}="8H8  RUA"; $resn_exch{" H91 RUA"}="1H9  RUA"; $resn_exch{"H101 RUA"}="1H10 RUA";
$resn_exch{"H111 RUA"}="1H11 RUA"; $resn_exch{"H121 RUA"}="1H12 RUA"; $resn_exch{"H271 RUA"}="1H27 RUA";
$resn_exch{"H281 RUA"}="1H28 RUA"; $resn_exch{"H291 RUA"}="1H29 RUA"; $resn_exch{"H331 RUA"}="31H3 RUA";
$resn_exch{"H301 RUA"}="1H30 RUA"; $resn_exch{"H332 RUA"}="32H3 RUA"; $resn_exch{"H33A RUA"}="1H33 RUA";
$resn_exch{"H341 RUA"}="1H34 RUA"; $resn_exch{"H351 RUA"}="1H35 RUA"; $resn_exch{"H361 RUA"}="1H36 RUA";
$resn_exch{" H11 RUB"}="1H1  RUB"; $resn_exch{" H12 RUB"}="2H1  RUB"; $resn_exch{" H51 RUB"}="1H5  RUB";
$resn_exch{" H52 RUB"}="2H5  RUB"; $resn_exch{"HOP2 RUB"}="2HOP RUB"; $resn_exch{"HOP3 RUB"}="3HOP RUB";
$resn_exch{"HOP5 RUB"}="5HOP RUB"; $resn_exch{"HOP6 RUB"}="6HOP RUB"; $resn_exch{" H2' RUN"}=" H2\" RUN";
$resn_exch{"H3'1 RUN"}="1H3\" RUN"; $resn_exch{"H3'2 RUN"}="2H3\" RUN"; $resn_exch{"H3'3 RUN"}="3H3\" RUN";
$resn_exch{"H4'1 RUN"}="1H4\" RUN"; $resn_exch{"H4'2 RUN"}="2H4\" RUN"; $resn_exch{"H4'3 RUN"}="3H4\" RUN";
$resn_exch{" H21 RUN"}="1H2  RUN"; $resn_exch{" H22 RUN"}="2H2  RUN"; $resn_exch{"H111 RUN"}="1H11 RUN";
$resn_exch{"H112 RUN"}="2H11 RUN"; $resn_exch{"H113 RUN"}="3H11 RUN"; $resn_exch{"H121 RUN"}="1H12 RUN";
$resn_exch{"H122 RUN"}="2H12 RUN"; $resn_exch{"H123 RUN"}="3H12 RUN"; $resn_exch{" HA1 RUN"}="1HA  RUN";
$resn_exch{" HA2 RUN"}="2HA  RUN"; $resn_exch{" HB1 RUN"}="1HB  RUN"; $resn_exch{" HB2 RUN"}="2HB  RUN";
$resn_exch{" HC1 RUN"}="1HC  RUN"; $resn_exch{" HC2 RUN"}="2HC  RUN"; $resn_exch{" H61 RUP"}="1H6  RUP";
$resn_exch{" H62 RUP"}="2H6  RUP"; $resn_exch{"H281 RUP"}="1H28 RUP"; $resn_exch{"H282 RUP"}="2H28 RUP";
$resn_exch{"H291 RUP"}="1H29 RUP"; $resn_exch{"H292 RUP"}="2H29 RUP"; $resn_exch{"H151 RUP"}="1H15 RUP";
$resn_exch{"H152 RUP"}="2H15 RUP"; $resn_exch{"H251 RUP"}="1H25 RUP"; $resn_exch{"H252 RUP"}="2H25 RUP";
$resn_exch{" H81 RUP"}="1H8  RUP"; $resn_exch{" H82 RUP"}="2H8  RUP"; $resn_exch{"H111 RUP"}="1H11 RUP";
$resn_exch{"H112 RUP"}="2H11 RUP"; $resn_exch{" H51 RUP"}="1H5  RUP"; $resn_exch{" H52 RUP"}="2H5  RUP";
$resn_exch{" H31 RUP"}="1H3  RUP"; $resn_exch{"H32A RUP"}="2H3  RUP"; $resn_exch{"H21A RUP"}="1H2  RUP";
$resn_exch{"H22A RUP"}="2H2  RUP"; $resn_exch{"H211 RUT"}="1H21 RUT"; $resn_exch{"H212 RUT"}="2H21 RUT";
$resn_exch{"H271 RUT"}="1H27 RUT"; $resn_exch{"H272 RUT"}="2H27 RUT"; $resn_exch{"H273 RUT"}="3H27 RUT";
$resn_exch{"HOP1 RVP"}="1HOP RVP"; $resn_exch{"HOP2 RVP"}="2HOP RVP"; $resn_exch{"H5'1 RVP"}="1H5* RVP";
$resn_exch{"H5'2 RVP"}="2H5* RVP"; $resn_exch{" H4' RVP"}=" H4* RVP"; $resn_exch{" H3' RVP"}=" H3* RVP";
$resn_exch{"HO'3 RVP"}="3HO* RVP"; $resn_exch{" H2' RVP"}=" H2* RVP"; $resn_exch{"HO'2 RVP"}="2HO* RVP";
$resn_exch{" H1' RVP"}=" H1* RVP"; $resn_exch{" HN1 RVP"}="1HN  RVP"; $resn_exch{" HN2 RVP"}="2HN  RVP";
$resn_exch{"H3C1 RWF"}="1H3C RWF"; $resn_exch{"H3C2 RWF"}="2H3C RWF"; $resn_exch{"H5C1 RWF"}="1H5C RWF";
$resn_exch{"H5C2 RWF"}="2H5C RWF"; $resn_exch{"H5C3 RWF"}="3H5C RWF"; $resn_exch{"H511 RWJ"}="1H51 RWJ";
$resn_exch{"H521 RWJ"}="1H52 RWJ"; $resn_exch{"H522 RWJ"}="2H52 RWJ"; $resn_exch{"H531 RWJ"}="1H53 RWJ";
$resn_exch{"H532 RWJ"}="2H53 RWJ"; $resn_exch{"H541 RWJ"}="1H54 RWJ"; $resn_exch{"H542 RWJ"}="2H54 RWJ";
$resn_exch{"H551 RWJ"}="1H55 RWJ"; $resn_exch{"H552 RWJ"}="2H55 RWJ"; $resn_exch{" H1N RWJ"}="1NH  RWJ";
$resn_exch{" HA1 RWJ"}="1HA  RWJ"; $resn_exch{" HB1 RWJ"}="1HB  RWJ"; $resn_exch{" HB2 RWJ"}="2HB  RWJ";
$resn_exch{" HG1 RWJ"}="1HG  RWJ"; $resn_exch{" HG2 RWJ"}="2HG  RWJ"; $resn_exch{" HD1 RWJ"}="1HD  RWJ";
$resn_exch{" HD2 RWJ"}="2HD  RWJ"; $resn_exch{" HE1 RWJ"}="1HE  RWJ"; $resn_exch{"HN11 RWJ"}="1HN1 RWJ";
$resn_exch{"HN21 RWJ"}="1HN2 RWJ"; $resn_exch{"HN22 RWJ"}="2HN2 RWJ"; $resn_exch{"H631 RWJ"}="1H63 RWJ";
$resn_exch{"H641 RWJ"}="1H64 RWJ"; $resn_exch{"H651 RWJ"}="1H65 RWJ"; $resn_exch{"H661 RWJ"}="1H66 RWJ";
$resn_exch{"H251 RX3"}="1H25 RX3"; $resn_exch{"H252 RX3"}="2H25 RX3"; $resn_exch{"H201 RX3"}="1H20 RX3";
$resn_exch{"H202 RX3"}="2H20 RX3"; $resn_exch{"H211 RX3"}="1H21 RX3"; $resn_exch{"H212 RX3"}="2H21 RX3";
$resn_exch{"H221 RX3"}="1H22 RX3"; $resn_exch{"H222 RX3"}="2H22 RX3"; $resn_exch{"H101 RX3"}="1H10 RX3";
$resn_exch{"H102 RX3"}="2H10 RX3"; $resn_exch{" H21 RX3"}="1H2  RX3"; $resn_exch{" H22 RX3"}="2H2  RX3";
$resn_exch{"HN11 RXP"}="1HN1 RXP"; $resn_exch{"HN22 RXP"}="2HN2 RXP"; $resn_exch{" H21 RXP"}="1H2  RXP";
$resn_exch{"H22A RXP"}="2H2  RXP"; $resn_exch{"H131 RXP"}="1H13 RXP"; $resn_exch{"H132 RXP"}="2H13 RXP";
$resn_exch{"H381 RXP"}="1H38 RXP"; $resn_exch{"H382 RXP"}="2H38 RXP"; $resn_exch{"H391 RXP"}="1H39 RXP";
$resn_exch{"H392 RXP"}="2H39 RXP"; $resn_exch{"H211 RXP"}="1H21 RXP"; $resn_exch{"H212 RXP"}="2H21 RXP";
$resn_exch{"H231 RXP"}="1H23 RXP"; $resn_exch{"H232 RXP"}="2H23 RXP"; $resn_exch{"H311 RXP"}="1H31 RXP";
$resn_exch{"H312 RXP"}="2H31 RXP"; $resn_exch{"H711 RYU"}="1H71 RYU"; $resn_exch{"H712 RYU"}="2H71 RYU";
$resn_exch{"H713 RYU"}="3H71 RYU"; $resn_exch{"H211 RYU"}="1H21 RYU"; $resn_exch{"H212 RYU"}="2H21 RYU";
$resn_exch{"H221 RYU"}="1H22 RYU"; $resn_exch{"H222 RYU"}="2H22 RYU"; $resn_exch{"H241 RYU"}="1H24 RYU";
$resn_exch{"H242 RYU"}="2H24 RYU"; $resn_exch{"H251 RYU"}="1H25 RYU"; $resn_exch{"H252 RYU"}="2H25 RYU";
$resn_exch{"H111 S01"}="1H11 S01"; $resn_exch{"H112 S01"}="2H11 S01"; $resn_exch{"H141 S01"}="1H14 S01";
$resn_exch{"H142 S01"}="2H14 S01"; $resn_exch{"H231 S01"}="1H23 S01"; $resn_exch{"H232 S01"}="2H23 S01";
$resn_exch{"H233 S01"}="3H23 S01"; $resn_exch{"H311 S01"}="1H31 S01"; $resn_exch{"H312 S01"}="2H31 S01";
$resn_exch{"H341 S01"}="1H34 S01"; $resn_exch{"H342 S01"}="2H34 S01"; $resn_exch{"H381 S01"}="1H38 S01";
$resn_exch{"H382 S01"}="2H38 S01"; $resn_exch{"H411 S01"}="1H41 S01"; $resn_exch{"H412 S01"}="2H41 S01";
$resn_exch{"H461 S01"}="1H46 S01"; $resn_exch{"H462 S01"}="2H46 S01"; $resn_exch{"HOP2 S02"}="2HOP S02";
$resn_exch{"HOP3 S02"}="3HOP S02"; $resn_exch{"H5'1 S02"}="1H5* S02"; $resn_exch{"H5'2 S02"}="2H5* S02";
$resn_exch{" HA1 S02"}="1HA  S02"; $resn_exch{" HA2 S02"}="2HA  S02"; $resn_exch{"H6A1 S02"}="1H6A S02";
$resn_exch{"H6A2 S02"}="2H6A S02"; $resn_exch{"H2A1 S02"}="1H2A S02"; $resn_exch{"H2A2 S02"}="2H2A S02";
$resn_exch{"H3A1 S02"}="1H3A S02"; $resn_exch{"H3A2 S02"}="2H3A S02"; $resn_exch{"H5A1 S02"}="1H5A S02";
$resn_exch{"H5A2 S02"}="2H5A S02"; $resn_exch{"H6B1 S02"}="1H6B S02"; $resn_exch{"H6B2 S02"}="2H6B S02";
$resn_exch{"H2B1 S02"}="1H2B S02"; $resn_exch{"H2B2 S02"}="2H2B S02"; $resn_exch{"H3B1 S02"}="1H3B S02";
$resn_exch{"H3B2 S02"}="2H3B S02"; $resn_exch{"H5B1 S02"}="1H5B S02"; $resn_exch{"H5B2 S02"}="2H5B S02";
$resn_exch{"HD'1 S02"}="1HD* S02"; $resn_exch{"HD'2 S02"}="2HD* S02"; $resn_exch{"H3'1 S02"}="1H3* S02";
$resn_exch{"H3'2 S02"}="2H3* S02"; $resn_exch{"HO3' S02"}="HO3* S02"; $resn_exch{"H331 S03"}="1H33 S03";
$resn_exch{"H332 S03"}="2H33 S03"; $resn_exch{"H171 S03"}="1H17 S03"; $resn_exch{"H172 S03"}="2H17 S03";
$resn_exch{"H173 S03"}="3H17 S03"; $resn_exch{"H241 S03"}="1H24 S03"; $resn_exch{"H242 S03"}="2H24 S03";
$resn_exch{"H271 S03"}="1H27 S03"; $resn_exch{"H272 S03"}="2H27 S03"; $resn_exch{"H281 S03"}="1H28 S03";
$resn_exch{"H282 S03"}="2H28 S03"; $resn_exch{"H311 S03"}="1H31 S03"; $resn_exch{"H312 S03"}="2H31 S03";
$resn_exch{"H301 S03"}="1H30 S03"; $resn_exch{"H302 S03"}="2H30 S03"; $resn_exch{" H21 S0H"}="1H2  S0H";
$resn_exch{" H22 S0H"}="2H2  S0H"; $resn_exch{" H11 S0H"}="1H1  S0H"; $resn_exch{" H12 S0H"}="2H1  S0H";
$resn_exch{"H13A S0H"}="3H1  S0H"; $resn_exch{"H121 S10"}="1H12 S10"; $resn_exch{"H122 S10"}="2H12 S10";
$resn_exch{"H131 S10"}="1H13 S10"; $resn_exch{"H132 S10"}="2H13 S10"; $resn_exch{"H133 S10"}="3H13 S10";
$resn_exch{"H8C1 S10"}="1H8C S10"; $resn_exch{"H8C2 S10"}="2H8C S10"; $resn_exch{"H7C1 S10"}="1H7C S10";
$resn_exch{"H7C2 S10"}="2H7C S10"; $resn_exch{"H6C1 S10"}="1H6C S10"; $resn_exch{"H6C2 S10"}="2H6C S10";
$resn_exch{"H2C1 S10"}="1H2C S10"; $resn_exch{"H2C2 S10"}="2H2C S10"; $resn_exch{"H1C1 S10"}="1H1C S10";
$resn_exch{"H1C2 S10"}="2H1C S10"; $resn_exch{"H1C3 S10"}="3H1C S10"; $resn_exch{"H131 S11"}="1H13 S11";
$resn_exch{"H132 S11"}="2H13 S11"; $resn_exch{"H133 S11"}="3H13 S11"; $resn_exch{"H141 S11"}="1H14 S11";
$resn_exch{"H142 S11"}="2H14 S11"; $resn_exch{"H143 S11"}="3H14 S11"; $resn_exch{"H151 S11"}="1H15 S11";
$resn_exch{"H152 S11"}="2H15 S11"; $resn_exch{" HA1 S11"}="1HA  S11"; $resn_exch{" HA2 S11"}="2HA  S11";
$resn_exch{"H241 S11"}="1H24 S11"; $resn_exch{"H242 S11"}="2H24 S11"; $resn_exch{"H301 S14"}="1H30 S14";
$resn_exch{"H302 S14"}="2H30 S14"; $resn_exch{"H351 S14"}="1H35 S14"; $resn_exch{"H352 S14"}="2H35 S14";
$resn_exch{"H353 S14"}="3H35 S14"; $resn_exch{"H391 S14"}="1H39 S14"; $resn_exch{"H392 S14"}="2H39 S14";
$resn_exch{"H393 S14"}="3H39 S14"; $resn_exch{"H431 S14"}="1H43 S14"; $resn_exch{"H432 S14"}="2H43 S14";
$resn_exch{"H451 S14"}="1H45 S14"; $resn_exch{"H452 S14"}="2H45 S14"; $resn_exch{"H461 S14"}="1H46 S14";
$resn_exch{"H462 S14"}="2H46 S14"; $resn_exch{"HBC1 S17"}="1HBC S17"; $resn_exch{"HBC2 S17"}="2HBC S17";
$resn_exch{"HAA1 S17"}="1HAA S17"; $resn_exch{"HAA2 S17"}="2HAA S17"; $resn_exch{"HAA3 S17"}="3HAA S17";
$resn_exch{"HAP1 S17"}="1HAP S17"; $resn_exch{"HAP2 S17"}="2HAP S17"; $resn_exch{"HAN1 S17"}="1HAN S17";
$resn_exch{"HAN2 S17"}="2HAN S17"; $resn_exch{"HAL1 S17"}="1HAL S17"; $resn_exch{"HAL2 S17"}="2HAL S17";
$resn_exch{"HAK1 S17"}="1HAK S17"; $resn_exch{"HAK2 S17"}="2HAK S17"; $resn_exch{"HAM1 S17"}="1HAM S17";
$resn_exch{"HAM2 S17"}="2HAM S17"; $resn_exch{"HAO1 S17"}="1HAO S17"; $resn_exch{"HAO2 S17"}="2HAO S17";
$resn_exch{"H131 S1A"}="1H13 S1A"; $resn_exch{"H132 S1A"}="2H13 S1A"; $resn_exch{"H141 S1A"}="1H14 S1A";
$resn_exch{"H142 S1A"}="2H14 S1A"; $resn_exch{"H151 S1A"}="1H15 S1A"; $resn_exch{"H152 S1A"}="2H15 S1A";
$resn_exch{"H161 S1A"}="1H16 S1A"; $resn_exch{"H162 S1A"}="2H16 S1A"; $resn_exch{"H191 S1A"}="1H19 S1A";
$resn_exch{"H192 S1A"}="2H19 S1A"; $resn_exch{"H193 S1A"}="3H19 S1A"; $resn_exch{"H201 S1A"}="1H20 S1A";
$resn_exch{"H202 S1A"}="2H20 S1A"; $resn_exch{"H203 S1A"}="3H20 S1A"; $resn_exch{"H211 S1A"}="1H21 S1A";
$resn_exch{"H212 S1A"}="2H21 S1A"; $resn_exch{"H213 S1A"}="3H21 S1A"; $resn_exch{"H221 S1A"}="1H22 S1A";
$resn_exch{"H222 S1A"}="2H22 S1A"; $resn_exch{"H223 S1A"}="3H22 S1A"; $resn_exch{"H231 S1A"}="1H23 S1A";
$resn_exch{"H232 S1A"}="2H23 S1A"; $resn_exch{"H233 S1A"}="3H23 S1A"; $resn_exch{"H241 S1A"}="1H24 S1A";
$resn_exch{"H242 S1A"}="2H24 S1A"; $resn_exch{"H243 S1A"}="3H24 S1A"; $resn_exch{" H2  S1H"}=" HN2 S1H";
$resn_exch{" HB2 S1H"}="1HB  S1H"; $resn_exch{" HB3 S1H"}="2HB  S1H"; $resn_exch{" H11 S1H"}="1H1  S1H";
$resn_exch{" H12 S1H"}="2H1  S1H"; $resn_exch{" H21 S1H"}="1H2  S1H"; $resn_exch{" H22 S1H"}="2H2  S1H";
$resn_exch{" H31 S1H"}="1H3  S1H"; $resn_exch{" H32 S1H"}="2H3  S1H"; $resn_exch{" H41 S1H"}="1H4  S1H";
$resn_exch{" H42 S1H"}="2H4  S1H"; $resn_exch{" H51 S1H"}="1H5  S1H"; $resn_exch{" H52 S1H"}="2H5  S1H";
$resn_exch{" H61 S1H"}="1H6  S1H"; $resn_exch{" H62 S1H"}="2H6  S1H"; $resn_exch{" H71 S1H"}="1H7  S1H";
$resn_exch{" H72 S1H"}="2H7  S1H"; $resn_exch{" H81 S1H"}="1H8  S1H"; $resn_exch{" H82 S1H"}="2H8  S1H";
$resn_exch{" H91 S1H"}="1H9  S1H"; $resn_exch{" H92 S1H"}="2H9  S1H"; $resn_exch{"H101 S1H"}="1H10 S1H";
$resn_exch{"H102 S1H"}="2H10 S1H"; $resn_exch{"H111 S1H"}="1H11 S1H"; $resn_exch{"H112 S1H"}="2H11 S1H";
$resn_exch{"H121 S1H"}="1H12 S1H"; $resn_exch{"H122 S1H"}="2H12 S1H"; $resn_exch{"H131 S1H"}="1H13 S1H";
$resn_exch{"H132 S1H"}="2H13 S1H"; $resn_exch{"H141 S1H"}="1H14 S1H"; $resn_exch{"H142 S1H"}="2H14 S1H";
$resn_exch{"H151 S1H"}="1H15 S1H"; $resn_exch{"H152 S1H"}="2H15 S1H"; $resn_exch{"H161 S1H"}="1H16 S1H";
$resn_exch{"H162 S1H"}="2H16 S1H"; $resn_exch{"H163 S1H"}="3H16 S1H"; $resn_exch{"H391 S1S"}="1H39 S1S";
$resn_exch{"H392 S1S"}="2H39 S1S"; $resn_exch{"H121 S1S"}="1H12 S1S"; $resn_exch{"H122 S1S"}="2H12 S1S";
$resn_exch{"H321 S1S"}="1H32 S1S"; $resn_exch{"H322 S1S"}="2H32 S1S"; $resn_exch{"H331 S1S"}="1H33 S1S";
$resn_exch{"H332 S1S"}="2H33 S1S"; $resn_exch{"H341 S1S"}="1H34 S1S"; $resn_exch{"H342 S1S"}="2H34 S1S";
$resn_exch{"H351 S1S"}="1H35 S1S"; $resn_exch{"H352 S1S"}="2H35 S1S"; $resn_exch{"H361 S1S"}="1H36 S1S";
$resn_exch{"H362 S1S"}="2H36 S1S"; $resn_exch{"H281 S1S"}="1H28 S1S"; $resn_exch{"H282 S1S"}="2H28 S1S";
$resn_exch{"HN11 S1S"}="1HN1 S1S"; $resn_exch{"HN12 S1S"}="2HN1 S1S"; $resn_exch{"H251 S1S"}="1H25 S1S";
$resn_exch{"H252 S1S"}="2H25 S1S"; $resn_exch{"H101 S1S"}="1H10 S1S"; $resn_exch{"H102 S1S"}="2H10 S1S";
$resn_exch{" H61 S1T"}="1H6  S1T"; $resn_exch{" H62 S1T"}="2H6  S1T"; $resn_exch{" H71 S1T"}="1H7  S1T";
$resn_exch{" H72 S1T"}="2H7  S1T"; $resn_exch{"HM41 S1T"}="1HM4 S1T"; $resn_exch{"HM42 S1T"}="2HM4 S1T";
$resn_exch{"HM43 S1T"}="3HM4 S1T"; $resn_exch{"H7'1 S1T"}="1H7' S1T"; $resn_exch{"H7'2 S1T"}="2H7' S1T";
$resn_exch{"HM21 S1T"}="1HM2 S1T"; $resn_exch{"HM22 S1T"}="2HM2 S1T"; $resn_exch{"HM23 S1T"}="3HM2 S1T";
$resn_exch{"H4'1 S1T"}="1H4' S1T"; $resn_exch{"H4'2 S1T"}="2H4' S1T"; $resn_exch{"HP21 S1T"}="1HP2 S1T";
$resn_exch{"HP22 S1T"}="2HP2 S1T"; $resn_exch{"HP23 S1T"}="3HP2 S1T"; $resn_exch{" H91 S27"}="1H9  S27";
$resn_exch{" H92 S27"}="2H9  S27"; $resn_exch{" H93 S27"}="3H9  S27"; $resn_exch{"H111 S27"}="1H11 S27";
$resn_exch{"H112 S27"}="2H11 S27"; $resn_exch{" H71 S27"}="1H7  S27"; $resn_exch{" H72 S27"}="2H7  S27";
$resn_exch{"H101 S27"}="1H10 S27"; $resn_exch{"H102 S27"}="2H10 S27"; $resn_exch{"H103 S27"}="3H10 S27";
$resn_exch{"H231 S27"}="1H23 S27"; $resn_exch{"H232 S27"}="2H23 S27"; $resn_exch{"H233 S27"}="3H23 S27";
$resn_exch{"H311 S2A"}="1H31 S2A"; $resn_exch{"H312 S2A"}="2H31 S2A"; $resn_exch{"H313 S2A"}="3H31 S2A";
$resn_exch{" HN1 S2A"}="1HN  S2A"; $resn_exch{" HN2 S2A"}="2HN  S2A"; $resn_exch{" HN3 S2A"}="3HN  S2A";
$resn_exch{" HM1 S2A"}="1HM  S2A"; $resn_exch{" HM2 S2A"}="2HM  S2A"; $resn_exch{" HM3 S2A"}="3HM  S2A";
$resn_exch{" HB1 S2C"}="1HB  S2C"; $resn_exch{" HB2 S2C"}="2HB  S2C"; $resn_exch{" HD1 S2C"}="1HD  S2C";
$resn_exch{" HD2 S2C"}="2HD  S2C"; $resn_exch{" HE1 S2C"}="1HE  S2C"; $resn_exch{" HE2 S2C"}="2HE  S2C";
$resn_exch{"H3C1 S2D"}="1H3C S2D"; $resn_exch{"H3C2 S2D"}="2H3C S2D"; $resn_exch{"H3C3 S2D"}="3H3C S2D";
$resn_exch{" H71 S2M"}="1H5M S2M"; $resn_exch{" H72 S2M"}="2H5M S2M"; $resn_exch{" H73 S2M"}="3H5M S2M";
$resn_exch{" H5' S2M"}="1H5* S2M"; $resn_exch{"H5'' S2M"}="2H5* S2M"; $resn_exch{"HOP2 S2M"}="2HOP S2M";
$resn_exch{" H4' S2M"}=" H4* S2M"; $resn_exch{" H1' S2M"}=" H1* S2M"; $resn_exch{" H2' S2M"}=" H2* S2M";
$resn_exch{" H3' S2M"}=" H3* S2M"; $resn_exch{"HO3' S2M"}="H3T  S2M"; $resn_exch{"HA'1 S2M"}="1HA* S2M";
$resn_exch{"HA'2 S2M"}="2HA* S2M"; $resn_exch{"HB'1 S2M"}="1HB* S2M"; $resn_exch{"HB'2 S2M"}="2HB* S2M";
$resn_exch{"HD'1 S2M"}="1HD* S2M"; $resn_exch{"HD'2 S2M"}="2HD* S2M"; $resn_exch{"HD'3 S2M"}="3HD* S2M";
$resn_exch{"HOP3 S2M"}="3HOP S2M"; $resn_exch{" H61 S3P"}="1H6  S3P"; $resn_exch{" H62 S3P"}="2H6  S3P";
$resn_exch{"HAB1 S48"}="1HAB S48"; $resn_exch{"HAB2 S48"}="2HAB S48"; $resn_exch{"HAB3 S48"}="3HAB S48";
$resn_exch{" HA1 S48"}="1HA  S48"; $resn_exch{" HA2 S48"}="2HA  S48"; $resn_exch{"HAA1 S48"}="1HAA S48";
$resn_exch{"HAA2 S48"}="2HAA S48"; $resn_exch{"HAA3 S48"}="3HAA S48"; $resn_exch{"HAV1 S48"}="1HAV S48";
$resn_exch{"HAV2 S48"}="2HAV S48"; $resn_exch{"HAS1 S48"}="1HAS S48"; $resn_exch{"HAS2 S48"}="2HAS S48";
$resn_exch{"HAU1 S48"}="1HAU S48"; $resn_exch{"HAU2 S48"}="2HAU S48"; $resn_exch{"HAC1 S48"}="1HAC S48";
$resn_exch{"HAC2 S48"}="2HAC S48"; $resn_exch{"HAC3 S48"}="3HAC S48"; $resn_exch{" H1' S4C"}=" H1* S4C";
$resn_exch{" H41 S4C"}="1H4  S4C"; $resn_exch{" H42 S4C"}="2H4  S4C"; $resn_exch{" H2' S4C"}=" H2* S4C";
$resn_exch{" H3' S4C"}=" H3* S4C"; $resn_exch{" H4' S4C"}=" H4* S4C"; $resn_exch{" H5' S4C"}="1H5* S4C";
$resn_exch{"H5'' S4C"}="2H5* S4C"; $resn_exch{" HA1 S4M"}="1HA  S4M"; $resn_exch{" HA2 S4M"}="2HA  S4M";
$resn_exch{" HN1 S4M"}="1HN  S4M"; $resn_exch{" HN2 S4M"}="2HN  S4M"; $resn_exch{" HB1 S4M"}="1HB  S4M";
$resn_exch{" HB2 S4M"}="2HB  S4M"; $resn_exch{" HG1 S4M"}="1HG  S4M"; $resn_exch{" HG2 S4M"}="2HG  S4M";
$resn_exch{" HE1 S4M"}="1HE  S4M"; $resn_exch{" HE2 S4M"}="2HE  S4M"; $resn_exch{" HE3 S4M"}="3HE  S4M";
$resn_exch{"H5'1 S4M"}="1H5* S4M"; $resn_exch{"H5'2 S4M"}="2H5* S4M"; $resn_exch{" H4' S4M"}=" H4* S4M";
$resn_exch{" H1' S4M"}=" H1* S4M"; $resn_exch{" H2' S4M"}=" H2* S4M"; $resn_exch{"HO2' S4M"}="HO2* S4M";
$resn_exch{" H3' S4M"}=" H3* S4M"; $resn_exch{"HO3' S4M"}="HO3* S4M"; $resn_exch{"HN61 S4M"}="1HN6 S4M";
$resn_exch{"HN62 S4M"}="2HN6 S4M"; $resn_exch{"HOP1 S4U"}="1HOP S4U"; $resn_exch{"HOP2 S4U"}="2HOP S4U";
$resn_exch{"H5'1 S4U"}="1H5' S4U"; $resn_exch{"H5'2 S4U"}="2H5' S4U"; $resn_exch{"HO2' S4U"}="'HO2 S4U";
$resn_exch{"HO3' S4U"}="'HO3 S4U"; $resn_exch{"H231 S55"}="1H23 S55"; $resn_exch{"H232 S55"}="2H23 S55";
$resn_exch{"H233 S55"}="3H23 S55"; $resn_exch{"H331 S55"}="1H33 S55"; $resn_exch{"H332 S55"}="2H33 S55";
$resn_exch{"H251 S55"}="1H25 S55"; $resn_exch{"H252 S55"}="2H25 S55"; $resn_exch{"H261 S55"}="1H26 S55";
$resn_exch{"H262 S55"}="2H26 S55"; $resn_exch{"H263 S55"}="3H26 S55"; $resn_exch{"H141 S55"}="1H14 S55";
$resn_exch{"H142 S55"}="2H14 S55"; $resn_exch{"H111 S55"}="1H11 S55"; $resn_exch{"H112 S55"}="2H11 S55";
$resn_exch{"H101 S55"}="1H10 S55"; $resn_exch{"H102 S55"}="2H10 S55"; $resn_exch{" H11 S57"}="1H1  S57";
$resn_exch{" H12 S57"}="2H1  S57"; $resn_exch{" H13 S57"}="3H1  S57"; $resn_exch{"H6B1 S57"}="1H6B S57";
$resn_exch{"H6B2 S57"}="2H6B S57"; $resn_exch{"H5B1 S57"}="1H5B S57"; $resn_exch{"H5B2 S57"}="2H5B S57";
$resn_exch{"H4B1 S57"}="1H4B S57"; $resn_exch{"H4B2 S57"}="2H4B S57"; $resn_exch{"H3B1 S57"}="1H3B S57";
$resn_exch{"H3B2 S57"}="2H3B S57"; $resn_exch{"H2B1 S57"}="1H2B S57"; $resn_exch{"H2B2 S57"}="2H2B S57";
$resn_exch{"H1B1 S57"}="1H1B S57"; $resn_exch{"H1B2 S57"}="2H1B S57"; $resn_exch{"HN31 S58"}="1HN3 S58";
$resn_exch{"HN32 S58"}="2HN3 S58"; $resn_exch{" H11 S5H"}="1H1  S5H"; $resn_exch{" H12 S5H"}="2H1  S5H";
$resn_exch{" H21 S5H"}="1H2  S5H"; $resn_exch{" H22 S5H"}="2H2  S5H"; $resn_exch{" H41 S5H"}="1H4  S5H";
$resn_exch{" H42 S5H"}="2H4  S5H"; $resn_exch{" H61 S5H"}="1H6  S5H"; $resn_exch{" H62 S5H"}="2H6  S5H";
$resn_exch{" H71 S5H"}="1H7  S5H"; $resn_exch{" H72 S5H"}="2H7  S5H"; $resn_exch{"H111 S5H"}="1H11 S5H";
$resn_exch{"H112 S5H"}="2H11 S5H"; $resn_exch{"H121 S5H"}="1H12 S5H"; $resn_exch{"H122 S5H"}="2H12 S5H";
$resn_exch{"H151 S5H"}="1H15 S5H"; $resn_exch{"H152 S5H"}="2H15 S5H"; $resn_exch{"H161 S5H"}="1H16 S5H";
$resn_exch{"H162 S5H"}="2H16 S5H"; $resn_exch{"H181 S5H"}="1H18 S5H"; $resn_exch{"H182 S5H"}="2H18 S5H";
$resn_exch{"H183 S5H"}="3H18 S5H"; $resn_exch{"H191 S5H"}="1H19 S5H"; $resn_exch{"H192 S5H"}="2H19 S5H";
$resn_exch{"H193 S5H"}="3H19 S5H"; $resn_exch{"H211 S5H"}="1H21 S5H"; $resn_exch{"H212 S5H"}="2H21 S5H";
$resn_exch{"H213 S5H"}="3H21 S5H"; $resn_exch{"HH21 S5H"}="1HH2 S5H"; $resn_exch{"HH22 S5H"}="2HH2 S5H";
$resn_exch{"HH31 S5H"}="1HH3 S5H"; $resn_exch{"HH32 S5H"}="2HH3 S5H"; $resn_exch{"HOH5 S5H"}="5HOH S5H";
$resn_exch{"HOP2 S6G"}="2HOP S6G"; $resn_exch{"HOP3 S6G"}="3HOP S6G"; $resn_exch{" H5' S6G"}="1H5* S6G";
$resn_exch{"H5'' S6G"}="2H5* S6G"; $resn_exch{" H4' S6G"}=" H4* S6G"; $resn_exch{" H3' S6G"}=" H3* S6G";
$resn_exch{"HO3' S6G"}="*HO3 S6G"; $resn_exch{" H2' S6G"}="1H2* S6G"; $resn_exch{"H2'' S6G"}="2H2* S6G";
$resn_exch{" H1' S6G"}=" H1* S6G"; $resn_exch{"HN21 S6G"}="1HN2 S6G"; $resn_exch{"HN22 S6G"}="2HN2 S6G";
$resn_exch{" H11 S6P"}="1H1  S6P"; $resn_exch{" H12 S6P"}="2H1  S6P"; $resn_exch{" H61 S6P"}="1H6  S6P";
$resn_exch{" H62 S6P"}="2H6  S6P"; $resn_exch{" H11 S70"}="1H1  S70"; $resn_exch{" H12 S70"}="2H1  S70";
$resn_exch{" H41 S70"}="1H4  S70"; $resn_exch{" H42 S70"}="2H4  S70"; $resn_exch{"HN11 S79"}="1HN1 S79";
$resn_exch{"HN12 S79"}="2HN1 S79"; $resn_exch{"H171 S79"}="1H17 S79"; $resn_exch{"H172 S79"}="2H17 S79";
$resn_exch{" H91 S80"}="1H9  S80"; $resn_exch{" H92 S80"}="2H9  S80"; $resn_exch{" H81 S80"}="1H8  S80";
$resn_exch{" H82 S80"}="2H8  S80"; $resn_exch{" H71 S80"}="1H7  S80"; $resn_exch{" H72 S80"}="2H7  S80";
$resn_exch{" H61 S80"}="1H6  S80"; $resn_exch{" H62 S80"}="2H6  S80"; $resn_exch{"H111 S80"}="1H11 S80";
$resn_exch{"H112 S80"}="2H11 S80"; $resn_exch{"H121 S80"}="1H12 S80"; $resn_exch{"H122 S80"}="2H12 S80";
$resn_exch{"H131 S80"}="1H13 S80"; $resn_exch{"H132 S80"}="2H13 S80"; $resn_exch{"H141 S80"}="1H14 S80";
$resn_exch{"H142 S80"}="2H14 S80"; $resn_exch{"H171 S80"}="1H17 S80"; $resn_exch{"H172 S80"}="2H17 S80";
$resn_exch{"H173 S80"}="3H17 S80"; $resn_exch{"H161 S80"}="1H16 S80"; $resn_exch{"H162 S80"}="2H16 S80";
$resn_exch{"H163 S80"}="3H16 S80"; $resn_exch{"H181 S80"}="1H18 S80"; $resn_exch{"H182 S80"}="2H18 S80";
$resn_exch{"H191 S80"}="1H19 S80"; $resn_exch{"H192 S80"}="2H19 S80"; $resn_exch{"H201 S80"}="1H20 S80";
$resn_exch{"H202 S80"}="2H20 S80"; $resn_exch{"H203 S80"}="3H20 S80"; $resn_exch{"H121 S9L"}="1H12 S9L";
$resn_exch{"H122 S9L"}="2H12 S9L"; $resn_exch{"H221 S9L"}="1H22 S9L"; $resn_exch{"H222 S9L"}="2H22 S9L";
$resn_exch{"H131 S9L"}="1H13 S9L"; $resn_exch{"H132 S9L"}="2H13 S9L"; $resn_exch{"H231 S9L"}="1H23 S9L";
$resn_exch{"H232 S9L"}="2H23 S9L"; $resn_exch{"H141 S9L"}="1H14 S9L"; $resn_exch{"H142 S9L"}="2H14 S9L";
$resn_exch{"H241 S9L"}="1H24 S9L"; $resn_exch{"H242 S9L"}="2H24 S9L"; $resn_exch{"HO3' S9L"}="HO3* S9L";
$resn_exch{"H11A SA1"}="1H1  SA1"; $resn_exch{"H12A SA1"}="2H1  SA1"; $resn_exch{" H41 SA1"}="1H4  SA1";
$resn_exch{" H42 SA1"}="2H4  SA1"; $resn_exch{" H43 SA1"}="3H4  SA1"; $resn_exch{"H141 SA1"}="1H14 SA1";
$resn_exch{"H142 SA1"}="2H14 SA1"; $resn_exch{"H151 SA1"}="1H15 SA1"; $resn_exch{"H152 SA1"}="2H15 SA1";
$resn_exch{"H161 SA1"}="1H16 SA1"; $resn_exch{"H162 SA1"}="2H16 SA1"; $resn_exch{"H201 SA1"}="1H20 SA1";
$resn_exch{"H202 SA1"}="2H20 SA1"; $resn_exch{"H191 SA2"}="1H19 SA2"; $resn_exch{"H192 SA2"}="2H19 SA2";
$resn_exch{"H181 SA2"}="1H18 SA2"; $resn_exch{"H182 SA2"}="2H18 SA2"; $resn_exch{"H171 SA2"}="1H17 SA2";
$resn_exch{"H172 SA2"}="2H17 SA2"; $resn_exch{"H141 SA2"}="1H14 SA2"; $resn_exch{"H142 SA2"}="2H14 SA2";
$resn_exch{"H211 SA2"}="1H21 SA2"; $resn_exch{"H212 SA2"}="2H21 SA2"; $resn_exch{"H213 SA2"}="3H21 SA2";
$resn_exch{" H52 SA2"}="2H5  SA2"; $resn_exch{" H62 SA2"}="2H6  SA2"; $resn_exch{"H361 SA3"}="1H36 SA3";
$resn_exch{"H362 SA3"}="2H36 SA3"; $resn_exch{" HN1 SA8"}="1HN  SA8"; $resn_exch{" HN2 SA8"}="2HN  SA8";
$resn_exch{" HB1 SA8"}="1HB  SA8"; $resn_exch{" HB2 SA8"}="2HB  SA8"; $resn_exch{" HG1 SA8"}="1HG  SA8";
$resn_exch{" HG2 SA8"}="2HG  SA8"; $resn_exch{" HE1 SA8"}="1HE  SA8"; $resn_exch{" HE2 SA8"}="2HE  SA8";
$resn_exch{" HE3 SA8"}="3HE  SA8"; $resn_exch{"H5'1 SA8"}="1H5* SA8"; $resn_exch{"H5'2 SA8"}="2H5* SA8";
$resn_exch{" H4' SA8"}=" H4* SA8"; $resn_exch{" H3' SA8"}=" H3* SA8"; $resn_exch{" H2' SA8"}=" H2* SA8";
$resn_exch{" H1' SA8"}=" H1* SA8"; $resn_exch{"HN61 SA8"}="1HN6 SA8"; $resn_exch{"HN62 SA8"}="2HN6 SA8";
$resn_exch{" H91 SAA"}="1H9  SAA"; $resn_exch{" H92 SAA"}="2H9  SAA"; $resn_exch{" H93 SAA"}="3H9  SAA";
$resn_exch{"H1'1 SAB"}="1H1' SAB"; $resn_exch{"H1'2 SAB"}="2H1' SAB"; $resn_exch{"H2'1 SAB"}="1H2' SAB";
$resn_exch{"H2'2 SAB"}="2H2' SAB"; $resn_exch{"H3'1 SAB"}="1H3' SAB"; $resn_exch{"H3'2 SAB"}="2H3' SAB";
$resn_exch{"H4'1 SAB"}="1H4' SAB"; $resn_exch{"H4'2 SAB"}="2H4' SAB"; $resn_exch{"HN41 SAB"}="1HN4 SAB";
$resn_exch{"HN42 SAB"}="2HN4 SAB"; $resn_exch{"HN31 SAB"}="1HN3 SAB"; $resn_exch{"HN32 SAB"}="2HN3 SAB";
$resn_exch{"H2A1 SAC"}="1H2A SAC"; $resn_exch{"H2A2 SAC"}="2H2A SAC"; $resn_exch{"H2A3 SAC"}="3H2A SAC";
$resn_exch{" H   SAC"}=" HN  SAC"; $resn_exch{" HB2 SAC"}="1HB  SAC"; $resn_exch{" HB3 SAC"}="2HB  SAC";
$resn_exch{" HG  SAC"}=" HOG SAC"; $resn_exch{"HOA2 SAD"}="2HOA SAD"; $resn_exch{"H51A SAD"}="AH51 SAD";
$resn_exch{"H52A SAD"}="AH52 SAD"; $resn_exch{" H4B SAD"}="AH4* SAD"; $resn_exch{" H3B SAD"}="AH3* SAD";
$resn_exch{"HO3A SAD"}="AHO3 SAD"; $resn_exch{" H2B SAD"}="AH2* SAD"; $resn_exch{"HO2A SAD"}="AHO2 SAD";
$resn_exch{" H1B SAD"}="AH1* SAD"; $resn_exch{" H8A SAD"}=" AH8 SAD"; $resn_exch{"H61A SAD"}="AH61 SAD";
$resn_exch{"H62A SAD"}="AH62 SAD"; $resn_exch{" H2A SAD"}=" AH2 SAD"; $resn_exch{" H31 SAD"}="1H3  SAD";
$resn_exch{" H32 SAD"}="2H3  SAD"; $resn_exch{"HON2 SAD"}="2HON SAD"; $resn_exch{"H51N SAD"}="NH51 SAD";
$resn_exch{"H52N SAD"}="NH52 SAD"; $resn_exch{" H4D SAD"}="NH4* SAD"; $resn_exch{" H3D SAD"}="NH3* SAD";
$resn_exch{"HO3N SAD"}="NHO3 SAD"; $resn_exch{" H2D SAD"}="NH2* SAD"; $resn_exch{"HO2N SAD"}="NHO2 SAD";
$resn_exch{" H1D SAD"}="NH1* SAD"; $resn_exch{"H61N SAD"}="NH61 SAD"; $resn_exch{"H62N SAD"}="NH62 SAD";
$resn_exch{"HOA2 SAE"}="2HOA SAE"; $resn_exch{"H51A SAE"}="AH51 SAE"; $resn_exch{"H52A SAE"}="AH52 SAE";
$resn_exch{" H4B SAE"}="AH4* SAE"; $resn_exch{" H3B SAE"}="AH3* SAE"; $resn_exch{"HO3A SAE"}="AHO3 SAE";
$resn_exch{" H2B SAE"}="AH2* SAE"; $resn_exch{"HO2A SAE"}="AHO2 SAE"; $resn_exch{" H1B SAE"}="AH1* SAE";
$resn_exch{" H8A SAE"}=" AH8 SAE"; $resn_exch{"H61A SAE"}="AH61 SAE"; $resn_exch{"H62A SAE"}="AH62 SAE";
$resn_exch{" H2A SAE"}=" AH2 SAE"; $resn_exch{"HON2 SAE"}="2HON SAE"; $resn_exch{"H51N SAE"}="NH51 SAE";
$resn_exch{"H52N SAE"}="NH52 SAE"; $resn_exch{" H4D SAE"}="NH4* SAE"; $resn_exch{" H3D SAE"}="NH3* SAE";
$resn_exch{"HO3N SAE"}="NHO3 SAE"; $resn_exch{" H2D SAE"}="NH2* SAE"; $resn_exch{"HO2N SAE"}="NHO2 SAE";
$resn_exch{" H1D SAE"}="NH1* SAE"; $resn_exch{"H61N SAE"}="NH61 SAE"; $resn_exch{"H62N SAE"}="NH62 SAE";
$resn_exch{"H7C1 SAF"}="1H7C SAF"; $resn_exch{"H7C2 SAF"}="2H7C SAF"; $resn_exch{"H7C3 SAF"}="3H7C SAF";
$resn_exch{"H8C1 SAF"}="1H8C SAF"; $resn_exch{"H8C2 SAF"}="2H8C SAF"; $resn_exch{"H8C3 SAF"}="3H8C SAF";
$resn_exch{"H101 SAF"}="1H10 SAF"; $resn_exch{"H102 SAF"}="2H10 SAF"; $resn_exch{"H103 SAF"}="3H10 SAF";
$resn_exch{"H7C1 SAG"}="1H7C SAG"; $resn_exch{"H7C2 SAG"}="2H7C SAG"; $resn_exch{"H181 SAG"}="1H18 SAG";
$resn_exch{"H182 SAG"}="2H18 SAG"; $resn_exch{"H183 SAG"}="3H18 SAG"; $resn_exch{"H211 SAG"}="1H21 SAG";
$resn_exch{"H212 SAG"}="2H21 SAG"; $resn_exch{" HN1 SAH"}="1HN  SAH"; $resn_exch{" HN2 SAH"}="2HN  SAH";
$resn_exch{" HB1 SAH"}="1HB  SAH"; $resn_exch{" HB2 SAH"}="2HB  SAH"; $resn_exch{" HG1 SAH"}="1HG  SAH";
$resn_exch{" HG2 SAH"}="2HG  SAH"; $resn_exch{"H5'1 SAH"}="1H5* SAH"; $resn_exch{"H5'2 SAH"}="2H5* SAH";
$resn_exch{" H4' SAH"}=" H4* SAH"; $resn_exch{" H3' SAH"}=" H3* SAH"; $resn_exch{"HO3' SAH"}="*HO3 SAH";
$resn_exch{" H2' SAH"}=" H2* SAH"; $resn_exch{"HO2' SAH"}="*HO2 SAH"; $resn_exch{" H1' SAH"}=" H1* SAH";
$resn_exch{"HN61 SAH"}="1HN6 SAH"; $resn_exch{"HN62 SAH"}="2HN6 SAH"; $resn_exch{" HN1 SAI"}="1HN  SAI";
$resn_exch{" HN2 SAI"}="2HN  SAI"; $resn_exch{" HB1 SAI"}="1HB  SAI"; $resn_exch{" HB2 SAI"}="2HB  SAI";
$resn_exch{" HG1 SAI"}="1HG  SAI"; $resn_exch{" HG2 SAI"}="2HG  SAI"; $resn_exch{"H5'1 SAI"}="1H5* SAI";
$resn_exch{"H5'2 SAI"}="2H5* SAI"; $resn_exch{" H4' SAI"}=" H4* SAI"; $resn_exch{" H3' SAI"}=" H3* SAI";
$resn_exch{"HO3' SAI"}="*HO3 SAI"; $resn_exch{" H2' SAI"}=" H2* SAI"; $resn_exch{"HO2' SAI"}="*HO2 SAI";
$resn_exch{" H1' SAI"}=" H1* SAI"; $resn_exch{"HN61 SAI"}="1HN6 SAI"; $resn_exch{"HN62 SAI"}="2HN6 SAI";
$resn_exch{"HO2' SAL"}="'HO2 SAL"; $resn_exch{" HN1 SAM"}="1HN  SAM"; $resn_exch{" HN2 SAM"}="2HN  SAM";
$resn_exch{" HB1 SAM"}="1HB  SAM"; $resn_exch{" HB2 SAM"}="2HB  SAM"; $resn_exch{" HG1 SAM"}="1HG  SAM";
$resn_exch{" HG2 SAM"}="2HG  SAM"; $resn_exch{" HE1 SAM"}="1HE  SAM"; $resn_exch{" HE2 SAM"}="2HE  SAM";
$resn_exch{" HE3 SAM"}="3HE  SAM"; $resn_exch{"H5'1 SAM"}="1H5* SAM"; $resn_exch{"H5'2 SAM"}="2H5* SAM";
$resn_exch{" H4' SAM"}=" H4* SAM"; $resn_exch{" H3' SAM"}=" H3* SAM"; $resn_exch{"HO3' SAM"}="*HO3 SAM";
$resn_exch{" H2' SAM"}=" H2* SAM"; $resn_exch{"HO2' SAM"}="*HO2 SAM"; $resn_exch{" H1' SAM"}=" H1* SAM";
$resn_exch{"HN61 SAM"}="1HN6 SAM"; $resn_exch{"HN62 SAM"}="2HN6 SAM"; $resn_exch{"HN11 SAN"}="1HN1 SAN";
$resn_exch{"HN12 SAN"}="2HN1 SAN"; $resn_exch{"HN21 SAN"}="1HN2 SAN"; $resn_exch{"HN22 SAN"}="2HN2 SAN";
$resn_exch{"HOG2 SAP"}="2HOG SAP"; $resn_exch{"HOG3 SAP"}="3HOG SAP"; $resn_exch{"HOB2 SAP"}="2HOB SAP";
$resn_exch{"HOA2 SAP"}="2HOA SAP"; $resn_exch{"H5'1 SAP"}="1H5* SAP"; $resn_exch{"H5'2 SAP"}="2H5* SAP";
$resn_exch{" H4' SAP"}=" H4* SAP"; $resn_exch{" H3' SAP"}=" H3* SAP"; $resn_exch{"HO3' SAP"}="*HO3 SAP";
$resn_exch{" H2' SAP"}=" H2* SAP"; $resn_exch{"HO2' SAP"}="*HO2 SAP"; $resn_exch{" H1' SAP"}=" H1* SAP";
$resn_exch{"HN61 SAP"}="1HN6 SAP"; $resn_exch{"HN62 SAP"}="2HN6 SAP"; $resn_exch{" H   SAR"}=" HN  SAR";
$resn_exch{" HA2 SAR"}="1HA  SAR"; $resn_exch{" HA3 SAR"}="2HA  SAR"; $resn_exch{" HN1 SAR"}="1HN  SAR";
$resn_exch{" HN2 SAR"}="2HN  SAR"; $resn_exch{" HN3 SAR"}="3HN  SAR"; $resn_exch{" H21 SAT"}="1H2  SAT";
$resn_exch{" H22 SAT"}="2H2  SAT"; $resn_exch{"H021 SAZ"}="1H02 SAZ"; $resn_exch{"H022 SAZ"}="2H02 SAZ";
$resn_exch{"H031 SAZ"}="1H03 SAZ"; $resn_exch{"H032 SAZ"}="2H03 SAZ"; $resn_exch{"H071 SAZ"}="1H07 SAZ";
$resn_exch{"H072 SAZ"}="2H07 SAZ"; $resn_exch{"H091 SAZ"}="1H09 SAZ"; $resn_exch{"H092 SAZ"}="2H09 SAZ";
$resn_exch{"H101 SAZ"}="1H10 SAZ"; $resn_exch{"H102 SAZ"}="2H10 SAZ"; $resn_exch{"H161 SAZ"}="1H16 SAZ";
$resn_exch{"H162 SAZ"}="2H16 SAZ"; $resn_exch{"H163 SAZ"}="3H16 SAZ"; $resn_exch{"H171 SAZ"}="1H17 SAZ";
$resn_exch{"H172 SAZ"}="2H17 SAZ"; $resn_exch{"H173 SAZ"}="3H17 SAZ"; $resn_exch{"H261 SAZ"}="1H26 SAZ";
$resn_exch{"H262 SAZ"}="2H26 SAZ"; $resn_exch{"H263 SAZ"}="3H26 SAZ"; $resn_exch{"H361 SAZ"}="1H36 SAZ";
$resn_exch{"H362 SAZ"}="2H36 SAZ"; $resn_exch{"H363 SAZ"}="3H36 SAZ"; $resn_exch{" H51 SB1"}="1H5  SB1";
$resn_exch{" H52 SB1"}="2H5  SB1"; $resn_exch{" H81 SB1"}="1H8  SB1"; $resn_exch{" H82 SB1"}="2H8  SB1";
$resn_exch{" H91 SB1"}="1H9  SB1"; $resn_exch{" H92 SB1"}="2H9  SB1"; $resn_exch{"H101 SB1"}="1H10 SB1";
$resn_exch{"H102 SB1"}="2H10 SB1"; $resn_exch{"H111 SB1"}="1H11 SB1"; $resn_exch{"H112 SB1"}="2H11 SB1";
$resn_exch{"H121 SB1"}="1H12 SB1"; $resn_exch{"H122 SB1"}="2H12 SB1"; $resn_exch{"H131 SB1"}="1H13 SB1";
$resn_exch{"H132 SB1"}="2H13 SB1"; $resn_exch{"H141 SB1"}="1H14 SB1"; $resn_exch{"H142 SB1"}="2H14 SB1";
$resn_exch{"H151 SB1"}="1H15 SB1"; $resn_exch{"H152 SB1"}="2H15 SB1"; $resn_exch{"H161 SB1"}="1H16 SB1";
$resn_exch{"H162 SB1"}="2H16 SB1"; $resn_exch{"H171 SB1"}="1H17 SB1"; $resn_exch{"H172 SB1"}="2H17 SB1";
$resn_exch{"H181 SB1"}="1H18 SB1"; $resn_exch{"H182 SB1"}="2H18 SB1"; $resn_exch{"H221 SB1"}="1H22 SB1";
$resn_exch{"H222 SB1"}="2H22 SB1"; $resn_exch{"H231 SB1"}="1H23 SB1"; $resn_exch{"H232 SB1"}="2H23 SB1";
$resn_exch{"H241 SB1"}="1H24 SB1"; $resn_exch{"H242 SB1"}="2H24 SB1"; $resn_exch{"H251 SB1"}="1H25 SB1";
$resn_exch{"H252 SB1"}="2H25 SB1"; $resn_exch{"H261 SB1"}="1H26 SB1"; $resn_exch{"H262 SB1"}="2H26 SB1";
$resn_exch{"H263 SB1"}="3H26 SB1"; $resn_exch{"H271 SB1"}="1H27 SB1"; $resn_exch{"H272 SB1"}="2H27 SB1";
$resn_exch{"H273 SB1"}="3H27 SB1"; $resn_exch{" H11 SB2"}="1H1  SB2"; $resn_exch{" H12 SB2"}="2H1  SB2";
$resn_exch{" H13 SB2"}="3H1  SB2"; $resn_exch{" H31 SB3"}="1H3  SB3"; $resn_exch{" H32 SB3"}="2H3  SB3";
$resn_exch{" H41 SB3"}="1H4  SB3"; $resn_exch{" H42 SB3"}="2H4  SB3"; $resn_exch{" H51 SB3"}="1H5  SB3";
$resn_exch{" H52 SB3"}="2H5  SB3"; $resn_exch{" H61 SB3"}="1H6  SB3"; $resn_exch{" H62 SB3"}="2H6  SB3";
$resn_exch{"H111 SB3"}="1H11 SB3"; $resn_exch{"H112 SB3"}="2H11 SB3"; $resn_exch{"H121 SB3"}="1H12 SB3";
$resn_exch{"H122 SB3"}="2H12 SB3"; $resn_exch{"H123 SB3"}="3H12 SB3"; $resn_exch{"H131 SB3"}="1H13 SB3";
$resn_exch{"H132 SB3"}="2H13 SB3"; $resn_exch{"H133 SB3"}="3H13 SB3"; $resn_exch{"H141 SB3"}="1H14 SB3";
$resn_exch{"H142 SB3"}="2H14 SB3"; $resn_exch{"H143 SB3"}="3H14 SB3"; $resn_exch{"H161 SB3"}="1H16 SB3";
$resn_exch{"H162 SB3"}="2H16 SB3"; $resn_exch{"H171 SB3"}="1H17 SB3"; $resn_exch{"H172 SB3"}="2H17 SB3";
$resn_exch{" H11 SB4"}="1H1  SB4"; $resn_exch{" H12 SB4"}="2H1  SB4"; $resn_exch{" H21 SB4"}="1H2  SB4";
$resn_exch{" H22 SB4"}="2H2  SB4"; $resn_exch{" H41 SB4"}="1H4  SB4"; $resn_exch{" H42 SB4"}="2H4  SB4";
$resn_exch{" H51 SB4"}="1H5  SB4"; $resn_exch{" H52 SB4"}="2H5  SB4"; $resn_exch{"HN71 SB4"}="1HN7 SB4";
$resn_exch{"HN72 SB4"}="2HN7 SB4"; $resn_exch{"HN71 SB5"}="1HN7 SB5"; $resn_exch{"HN72 SB5"}="2HN7 SB5";
$resn_exch{"HCD1 SB5"}="1HCD SB5"; $resn_exch{"HCD2 SB5"}="2HCD SB5"; $resn_exch{"HC21 SB5"}="1HC2 SB5";
$resn_exch{"HC22 SB5"}="2HC2 SB5"; $resn_exch{"HC31 SB5"}="1HC3 SB5"; $resn_exch{"HC32 SB5"}="2HC3 SB5";
$resn_exch{"HCD1 SB6"}="1HCD SB6"; $resn_exch{"HCD2 SB6"}="2HCD SB6"; $resn_exch{"HE21 SB6"}="1HE2 SB6";
$resn_exch{"HE22 SB6"}="2HE2 SB6"; $resn_exch{"HE31 SB6"}="1HE3 SB6"; $resn_exch{"HE32 SB6"}="2HE3 SB6";
$resn_exch{" H71 SB7"}="1H7  SB7"; $resn_exch{" H72 SB7"}="2H7  SB7"; $resn_exch{" H81 SB7"}="1H8  SB7";
$resn_exch{" H82 SB7"}="2H8  SB7"; $resn_exch{"H111 SB7"}="1H11 SB7"; $resn_exch{"H112 SB7"}="2H11 SB7";
$resn_exch{"H241 SB7"}="1H24 SB7"; $resn_exch{"H242 SB7"}="2H24 SB7"; $resn_exch{"H251 SB8"}="1H25 SB8";
$resn_exch{"H252 SB8"}="2H25 SB8"; $resn_exch{"H261 SB8"}="1H26 SB8"; $resn_exch{"H262 SB8"}="2H26 SB8";
$resn_exch{" H71 SB9"}="1H7  SB9"; $resn_exch{" H72 SB9"}="2H7  SB9"; $resn_exch{" H73 SB9"}="3H7  SB9";
$resn_exch{"H111 SB9"}="1H11 SB9"; $resn_exch{"H112 SB9"}="2H11 SB9"; $resn_exch{"H121 SB9"}="1H12 SB9";
$resn_exch{"H122 SB9"}="2H12 SB9"; $resn_exch{"H151 SB9"}="1H15 SB9"; $resn_exch{"H152 SB9"}="2H15 SB9";
$resn_exch{" H71 SBA"}="1H7  SBA"; $resn_exch{" H72 SBA"}="2H7  SBA"; $resn_exch{"H231 SBA"}="1H23 SBA";
$resn_exch{"H232 SBA"}="2H23 SBA"; $resn_exch{"H251 SBA"}="1H25 SBA"; $resn_exch{"H252 SBA"}="2H25 SBA";
$resn_exch{"H253 SBA"}="3H25 SBA"; $resn_exch{"H261 SBA"}="1H26 SBA"; $resn_exch{"H262 SBA"}="2H26 SBA";
$resn_exch{"H263 SBA"}="3H26 SBA"; $resn_exch{"H191 SBA"}="1H19 SBA"; $resn_exch{"H192 SBA"}="2H19 SBA";
$resn_exch{"H211 SBA"}="1H21 SBA"; $resn_exch{"H212 SBA"}="2H21 SBA"; $resn_exch{"H213 SBA"}="3H21 SBA";
$resn_exch{"H221 SBA"}="1H22 SBA"; $resn_exch{"H222 SBA"}="2H22 SBA"; $resn_exch{"H223 SBA"}="3H22 SBA";
$resn_exch{"H161 SBA"}="1H16 SBA"; $resn_exch{"H162 SBA"}="2H16 SBA"; $resn_exch{"H181 SBA"}="1H18 SBA";
$resn_exch{"H182 SBA"}="2H18 SBA"; $resn_exch{"H183 SBA"}="3H18 SBA"; $resn_exch{"H101 SBB"}="1H10 SBB";
$resn_exch{"H102 SBB"}="2H10 SBB"; $resn_exch{"H121 SBB"}="1H12 SBB"; $resn_exch{"H122 SBB"}="2H12 SBB";
$resn_exch{"H131 SBB"}="1H13 SBB"; $resn_exch{"H132 SBB"}="2H13 SBB"; $resn_exch{"H133 SBB"}="3H13 SBB";
$resn_exch{"H151 SBB"}="1H15 SBB"; $resn_exch{"H152 SBB"}="2H15 SBB"; $resn_exch{"H201 SBC"}="1H20 SBC";
$resn_exch{"H202 SBC"}="2H20 SBC"; $resn_exch{"H251 SBC"}="1H25 SBC"; $resn_exch{"H252 SBC"}="2H25 SBC";
$resn_exch{" H   SBD"}="1HN  SBD"; $resn_exch{" H2  SBD"}="2HN  SBD"; $resn_exch{" HB2 SBD"}="1HB  SBD";
$resn_exch{" HB3 SBD"}="2HB  SBD"; $resn_exch{"H111 SBD"}="1H11 SBD"; $resn_exch{"H112 SBD"}="2H11 SBD";
$resn_exch{" HC2 SBD"}=" H2  SBD"; $resn_exch{"H161 SBD"}="1H16 SBD"; $resn_exch{"H162 SBD"}="2H16 SBD";
$resn_exch{"H163 SBD"}="3H16 SBD"; $resn_exch{"HZ12 SBE"}="2HZ1 SBE"; $resn_exch{"HZ13 SBE"}="3HZ1 SBE";
$resn_exch{"HZ22 SBE"}="2HZ2 SBE"; $resn_exch{"HZ23 SBE"}="3HZ2 SBE"; $resn_exch{" HN1 SBG"}="1HN  SBG";
$resn_exch{" HN2 SBG"}="2HN  SBG"; $resn_exch{"HBC1 SBG"}="1HBC SBG"; $resn_exch{"HBC2 SBG"}="2HBC SBG";
$resn_exch{"H1C1 SBG"}="1H1C SBG"; $resn_exch{"H1C2 SBG"}="2H1C SBG"; $resn_exch{"H1C3 SBG"}="3H1C SBG";
$resn_exch{" H11 SBH"}="1H1  SBH"; $resn_exch{" H12 SBH"}="2H1  SBH"; $resn_exch{" H21 SBH"}="1H2  SBH";
$resn_exch{" H22 SBH"}="2H2  SBH"; $resn_exch{" H41 SBH"}="1H4  SBH"; $resn_exch{" H42 SBH"}="2H4  SBH";
$resn_exch{" H43 SBH"}="3H4  SBH"; $resn_exch{" H51 SBH"}="1H5  SBH"; $resn_exch{" H52 SBH"}="2H5  SBH";
$resn_exch{" H51 SBI"}="1H5  SBI"; $resn_exch{" H52 SBI"}="2H5  SBI"; $resn_exch{" H61 SBI"}="1H6  SBI";
$resn_exch{" H62 SBI"}="2H6  SBI"; $resn_exch{" H   SBL"}="1HN  SBL"; $resn_exch{" H2  SBL"}="2HN  SBL";
$resn_exch{" HB2 SBL"}="1HB  SBL"; $resn_exch{" HB3 SBL"}="2HB  SBL"; $resn_exch{"H111 SBL"}="1H11 SBL";
$resn_exch{"H112 SBL"}="2H11 SBL"; $resn_exch{" HC2 SBL"}=" H2  SBL"; $resn_exch{"H161 SBL"}="1H16 SBL";
$resn_exch{"H162 SBL"}="2H16 SBL"; $resn_exch{"H163 SBL"}="3H16 SBL"; $resn_exch{"HN11 SBN"}="1HN1 SBN";
$resn_exch{"HN12 SBN"}="2HN1 SBN"; $resn_exch{"H141 SBN"}="1H14 SBN"; $resn_exch{"H142 SBN"}="2H14 SBN";
$resn_exch{"H143 SBN"}="3H14 SBN"; $resn_exch{"H151 SBN"}="1H15 SBN"; $resn_exch{"H152 SBN"}="2H15 SBN";
$resn_exch{"H153 SBN"}="3H15 SBN"; $resn_exch{" H11 SBO"}="1H1  SBO"; $resn_exch{" H21 SBO"}="1H2  SBO";
$resn_exch{" H31 SBO"}="1H3  SBO"; $resn_exch{"HZ22 SBP"}="2HZ2 SBP"; $resn_exch{"HZ23 SBP"}="3HZ2 SBP";
$resn_exch{"HZ32 SBP"}="2HZ3 SBP"; $resn_exch{"HZ33 SBP"}="3HZ3 SBP"; $resn_exch{"HZ12 SBP"}="2HZ1 SBP";
$resn_exch{"HZ13 SBP"}="3HZ1 SBP"; $resn_exch{" HH2 SBP"}="2HH  SBP"; $resn_exch{" HH3 SBP"}="3HH  SBP";
$resn_exch{" HQ2 SBP"}="2HQ  SBP"; $resn_exch{" HQ3 SBP"}="3HQ  SBP"; $resn_exch{"HL11 SBP"}="1HL1 SBP";
$resn_exch{"HL12 SBP"}="2HL1 SBP"; $resn_exch{"HL21 SBP"}="1HL2 SBP"; $resn_exch{"HL22 SBP"}="2HL2 SBP";
$resn_exch{"HC11 SBR"}="1HC1 SBR"; $resn_exch{"HC12 SBR"}="2HC1 SBR"; $resn_exch{"HC31 SBR"}="1HC3 SBR";
$resn_exch{"HC32 SBR"}="2HC3 SBR"; $resn_exch{"HN61 SBR"}="1HN6 SBR"; $resn_exch{"HN62 SBR"}="2HN6 SBR";
$resn_exch{"H261 SBR"}="1H26 SBR"; $resn_exch{"H262 SBR"}="2H26 SBR"; $resn_exch{"H263 SBR"}="3H26 SBR";
$resn_exch{"HC11 SBS"}="1HC1 SBS"; $resn_exch{"HC12 SBS"}="2HC1 SBS"; $resn_exch{"HC31 SBS"}="1HC3 SBS";
$resn_exch{"HC32 SBS"}="2HC3 SBS"; $resn_exch{"HN61 SBS"}="1HN6 SBS"; $resn_exch{"HN62 SBS"}="2HN6 SBS";
$resn_exch{"H261 SBS"}="1H26 SBS"; $resn_exch{"H262 SBS"}="2H26 SBS"; $resn_exch{"H263 SBS"}="3H26 SBS";
$resn_exch{" H11 SBT"}="1H1  SBT"; $resn_exch{" H12 SBT"}="2H1  SBT"; $resn_exch{" H13 SBT"}="3H1  SBT";
$resn_exch{" H31 SBT"}="1H3  SBT"; $resn_exch{" H32 SBT"}="2H3  SBT"; $resn_exch{" H41 SBT"}="1H4  SBT";
$resn_exch{" H42 SBT"}="2H4  SBT"; $resn_exch{" H43 SBT"}="3H4  SBT"; $resn_exch{" H11 SBU"}="1H1  SBU";
$resn_exch{" H12 SBU"}="2H1  SBU"; $resn_exch{" H13 SBU"}="3H1  SBU"; $resn_exch{" H21 SBU"}="1H2  SBU";
$resn_exch{" H31 SBU"}="1H3  SBU"; $resn_exch{" H32 SBU"}="2H3  SBU"; $resn_exch{" H41 SBU"}="1H4  SBU";
$resn_exch{" H42 SBU"}="2H4  SBU"; $resn_exch{" H43 SBU"}="3H4  SBU"; $resn_exch{" H31 SBX"}="1H3  SBX";
$resn_exch{" H32 SBX"}="2H3  SBX"; $resn_exch{" H41 SBX"}="1H4  SBX"; $resn_exch{" H42 SBX"}="2H4  SBX";
$resn_exch{" H51 SBX"}="1H5  SBX"; $resn_exch{" H52 SBX"}="2H5  SBX"; $resn_exch{" H61 SBX"}="1H6  SBX";
$resn_exch{" H62 SBX"}="2H6  SBX"; $resn_exch{"H111 SBX"}="1H11 SBX"; $resn_exch{"H112 SBX"}="2H11 SBX";
$resn_exch{"H121 SBX"}="1H12 SBX"; $resn_exch{"H122 SBX"}="2H12 SBX"; $resn_exch{"H123 SBX"}="3H12 SBX";
$resn_exch{"H131 SBX"}="1H13 SBX"; $resn_exch{"H132 SBX"}="2H13 SBX"; $resn_exch{"H133 SBX"}="3H13 SBX";
$resn_exch{"H141 SBX"}="1H14 SBX"; $resn_exch{"H142 SBX"}="2H14 SBX"; $resn_exch{"H143 SBX"}="3H14 SBX";
$resn_exch{"H161 SBX"}="1H16 SBX"; $resn_exch{"H162 SBX"}="2H16 SBX"; $resn_exch{"H171 SBX"}="1H17 SBX";
$resn_exch{"H172 SBX"}="2H17 SBX"; $resn_exch{"H251 SBX"}="1H25 SBX"; $resn_exch{"H252 SBX"}="2H25 SBX";
$resn_exch{"H261 SBX"}="1H26 SBX"; $resn_exch{"H262 SBX"}="2H26 SBX"; $resn_exch{"H271 SBX"}="1H27 SBX";
$resn_exch{"H272 SBX"}="2H27 SBX"; $resn_exch{"H281 SBX"}="1H28 SBX"; $resn_exch{"H282 SBX"}="2H28 SBX";
$resn_exch{"H291 SBX"}="1H29 SBX"; $resn_exch{"H292 SBX"}="2H29 SBX"; $resn_exch{"HZ22 SBZ"}="2HZ2 SBZ";
$resn_exch{"HZ23 SBZ"}="3HZ2 SBZ"; $resn_exch{"HZ32 SBZ"}="2HZ3 SBZ"; $resn_exch{"HZ33 SBZ"}="3HZ3 SBZ";
$resn_exch{"HZ12 SBZ"}="2HZ1 SBZ"; $resn_exch{"HZ13 SBZ"}="3HZ1 SBZ"; $resn_exch{"HM12 SBZ"}="2HM1 SBZ";
$resn_exch{"HM13 SBZ"}="3HM1 SBZ"; $resn_exch{"HM22 SBZ"}="2HM2 SBZ"; $resn_exch{"HM23 SBZ"}="3HM2 SBZ";
$resn_exch{"HN41  SC"}="1HN4  SC"; $resn_exch{"HN42  SC"}="2HN4  SC"; $resn_exch{" H1'  SC"}=" H1*  SC";
$resn_exch{" H2'  SC"}="1H2*  SC"; $resn_exch{"H2''  SC"}="2H2*  SC"; $resn_exch{" H3'  SC"}=" H3*  SC";
$resn_exch{" H4'  SC"}=" H4*  SC"; $resn_exch{"HO3'  SC"}="*HO3  SC"; $resn_exch{" H5'  SC"}="1H5*  SC";
$resn_exch{"H5''  SC"}="2H5*  SC"; $resn_exch{"HOP1  SC"}="1HOP  SC"; $resn_exch{"HOP3  SC"}="3HOP  SC";
$resn_exch{" H61 SC1"}="1H6  SC1"; $resn_exch{" H62 SC1"}="2H6  SC1"; $resn_exch{" H91 SC1"}="1H9  SC1";
$resn_exch{" H92 SC1"}="2H9  SC1"; $resn_exch{" H93 SC1"}="3H9  SC1"; $resn_exch{"HBC1 SC2"}="1HBC SC2";
$resn_exch{"HBC2 SC2"}="2HBC SC2"; $resn_exch{"HMC1 SC2"}="1HMC SC2"; $resn_exch{"HMC2 SC2"}="2HMC SC2";
$resn_exch{"HMC3 SC2"}="3HMC SC2"; $resn_exch{"H111 SC3"}="1H11 SC3"; $resn_exch{"H112 SC3"}="2H11 SC3";
$resn_exch{"H121 SC3"}="1H12 SC3"; $resn_exch{"H122 SC3"}="2H12 SC3"; $resn_exch{"H131 SC3"}="1H13 SC3";
$resn_exch{"H132 SC3"}="2H13 SC3"; $resn_exch{"H141 SC3"}="1H14 SC3"; $resn_exch{"H142 SC3"}="2H14 SC3";
$resn_exch{"H151 SC3"}="1H15 SC3"; $resn_exch{"H152 SC3"}="2H15 SC3"; $resn_exch{"H161 SC3"}="1H16 SC3";
$resn_exch{"H162 SC3"}="2H16 SC3"; $resn_exch{"H241 SC3"}="1H24 SC3"; $resn_exch{"H242 SC3"}="2H24 SC3";
$resn_exch{"H243 SC3"}="3H24 SC3"; $resn_exch{"H261 SC3"}="1H26 SC3"; $resn_exch{"H262 SC3"}="2H26 SC3";
$resn_exch{"H263 SC3"}="3H26 SC3"; $resn_exch{" H71 SC4"}="1H7  SC4"; $resn_exch{" H72 SC4"}="2H7  SC4";
$resn_exch{"H211 SC4"}="1H21 SC4"; $resn_exch{"H212 SC4"}="2H21 SC4"; $resn_exch{"H213 SC4"}="3H21 SC4";
$resn_exch{"H141 SC4"}="1H14 SC4"; $resn_exch{"H142 SC4"}="2H14 SC4"; $resn_exch{" H71 SC5"}="1H7  SC5";
$resn_exch{" H72 SC5"}="2H7  SC5"; $resn_exch{" H73 SC5"}="3H7  SC5"; $resn_exch{" H91 SC5"}="1H9  SC5";
$resn_exch{" H92 SC5"}="2H9  SC5"; $resn_exch{"H121 SC5"}="1H12 SC5"; $resn_exch{"H122 SC5"}="2H12 SC5";
$resn_exch{"HN41 SC5"}="1HN4 SC5"; $resn_exch{"HN42 SC5"}="2HN4 SC5"; $resn_exch{" H81 SC6"}="1H8  SC6";
$resn_exch{" H82 SC6"}="2H8  SC6"; $resn_exch{" H91 SC6"}="1H9  SC6"; $resn_exch{" H92 SC6"}="2H9  SC6";
$resn_exch{"H101 SC6"}="1H10 SC6"; $resn_exch{"H102 SC6"}="2H10 SC6"; $resn_exch{"H121 SC6"}="1H12 SC6";
$resn_exch{"H122 SC6"}="2H12 SC6"; $resn_exch{"H301 SC6"}="1H30 SC6"; $resn_exch{"H302 SC6"}="2H30 SC6";
$resn_exch{"H303 SC6"}="3H30 SC6"; $resn_exch{"H171 SC6"}="1H17 SC6"; $resn_exch{"H172 SC6"}="2H17 SC6";
$resn_exch{"H181 SC6"}="1H18 SC6"; $resn_exch{"H182 SC6"}="2H18 SC6"; $resn_exch{"H201 SC6"}="1H20 SC6";
$resn_exch{"H202 SC6"}="2H20 SC6"; $resn_exch{"H371 SC6"}="1H37 SC6"; $resn_exch{"H372 SC6"}="2H37 SC6";
$resn_exch{"H373 SC6"}="3H37 SC6"; $resn_exch{" H81 SC7"}="1H8  SC7"; $resn_exch{" H82 SC7"}="2H8  SC7";
$resn_exch{" H91 SC7"}="1H9  SC7"; $resn_exch{" H92 SC7"}="2H9  SC7"; $resn_exch{"H101 SC7"}="1H10 SC7";
$resn_exch{"H102 SC7"}="2H10 SC7"; $resn_exch{"H111 SC7"}="1H11 SC7"; $resn_exch{"H112 SC7"}="2H11 SC7";
$resn_exch{"H113 SC7"}="3H11 SC7"; $resn_exch{"H121 SC7"}="1H12 SC7"; $resn_exch{"H122 SC7"}="2H12 SC7";
$resn_exch{"H131 SC7"}="1H13 SC7"; $resn_exch{"H132 SC7"}="2H13 SC7"; $resn_exch{"H133 SC7"}="3H13 SC7";
$resn_exch{"H141 SC7"}="1H14 SC7"; $resn_exch{"H142 SC7"}="2H14 SC7"; $resn_exch{"H143 SC7"}="3H14 SC7";
$resn_exch{"H181 SC7"}="1H18 SC7"; $resn_exch{"H182 SC7"}="2H18 SC7"; $resn_exch{"H261 SC7"}="1H26 SC7";
$resn_exch{"H262 SC7"}="2H26 SC7"; $resn_exch{"H281 SC7"}="1H28 SC7"; $resn_exch{"H282 SC7"}="2H28 SC7";
$resn_exch{"H291 SC7"}="1H29 SC7"; $resn_exch{"H292 SC7"}="2H29 SC7"; $resn_exch{"H301 SC7"}="1H30 SC7";
$resn_exch{"H302 SC7"}="2H30 SC7"; $resn_exch{"H311 SC7"}="1H31 SC7"; $resn_exch{"H312 SC7"}="2H31 SC7";
$resn_exch{"H313 SC7"}="3H31 SC7"; $resn_exch{"H121 SC8"}="1H12 SC8"; $resn_exch{"H122 SC8"}="2H12 SC8";
$resn_exch{"H111 SC9"}="1H11 SC9"; $resn_exch{"H112 SC9"}="2H11 SC9"; $resn_exch{"HN61 SCA"}="1HN6 SCA";
$resn_exch{"HN62 SCA"}="2HN6 SCA"; $resn_exch{"HO2' SCA"}="'HO2 SCA"; $resn_exch{"H5'1 SCA"}="1H5' SCA";
$resn_exch{"H5'2 SCA"}="2H5' SCA"; $resn_exch{"HPB1 SCA"}="1HPB SCA"; $resn_exch{"HPB2 SCA"}="2HPB SCA";
$resn_exch{" H91 SCA"}="1H9  SCA"; $resn_exch{" H92 SCA"}="2H9  SCA"; $resn_exch{" H93 SCA"}="3H9  SCA";
$resn_exch{" H81 SCA"}="1H8  SCA"; $resn_exch{" H82 SCA"}="2H8  SCA"; $resn_exch{" H83 SCA"}="3H8  SCA";
$resn_exch{" H51 SCA"}="1H5  SCA"; $resn_exch{" H52 SCA"}="2H5  SCA"; $resn_exch{" H41 SCA"}="1H4  SCA";
$resn_exch{" H42 SCA"}="2H4  SCA"; $resn_exch{" H21 SCA"}="1H2  SCA"; $resn_exch{" H22 SCA"}="2H2  SCA";
$resn_exch{" H11 SCA"}="1H1  SCA"; $resn_exch{" H12 SCA"}="2H1  SCA"; $resn_exch{"HS21 SCA"}="1HS2 SCA";
$resn_exch{"HS22 SCA"}="2HS2 SCA"; $resn_exch{"HS31 SCA"}="1HS3 SCA"; $resn_exch{"HS32 SCA"}="2HS3 SCA";
$resn_exch{" H11 SCB"}="1H1  SCB"; $resn_exch{"H12A SCB"}="2H1  SCB"; $resn_exch{"H13A SCB"}="3H1  SCB";
$resn_exch{"H21A SCB"}="1H2  SCB"; $resn_exch{"H22A SCB"}="2H2  SCB"; $resn_exch{" H31 SCB"}="1H3  SCB";
$resn_exch{" H32 SCB"}="2H3  SCB"; $resn_exch{" H11 SCC"}="1H1  SCC"; $resn_exch{" H12 SCC"}="2H1  SCC";
$resn_exch{" H21 SCC"}="1H2  SCC"; $resn_exch{" H22 SCC"}="2H2  SCC"; $resn_exch{" H23 SCC"}="3H2  SCC";
$resn_exch{"HN61 SCD"}="1HN6 SCD"; $resn_exch{"HN62 SCD"}="2HN6 SCD"; $resn_exch{"HO2' SCD"}="'HO2 SCD";
$resn_exch{"H5'1 SCD"}="1H5' SCD"; $resn_exch{"H5'2 SCD"}="2H5' SCD"; $resn_exch{"HPB1 SCD"}="1HPB SCD";
$resn_exch{"HPB2 SCD"}="2HPB SCD"; $resn_exch{"HP91 SCD"}="1HP9 SCD"; $resn_exch{"HP92 SCD"}="2HP9 SCD";
$resn_exch{"HP93 SCD"}="3HP9 SCD"; $resn_exch{"HP81 SCD"}="1HP8 SCD"; $resn_exch{"HP82 SCD"}="2HP8 SCD";
$resn_exch{"HP83 SCD"}="3HP8 SCD"; $resn_exch{"HOP3 SCD"}="3HOP SCD"; $resn_exch{"HNP2 SCD"}="2HNP SCD";
$resn_exch{"HP51 SCD"}="1HP5 SCD"; $resn_exch{"HP52 SCD"}="2HP5 SCD"; $resn_exch{"HP41 SCD"}="1HP4 SCD";
$resn_exch{"HP42 SCD"}="2HP4 SCD"; $resn_exch{"HNP1 SCD"}="1HNP SCD"; $resn_exch{"HP21 SCD"}="1HP2 SCD";
$resn_exch{"HP22 SCD"}="2HP2 SCD"; $resn_exch{"HP11 SCD"}="1HP1 SCD"; $resn_exch{"HP12 SCD"}="2HP1 SCD";
$resn_exch{"HPS1 SCD"}="1HPS SCD"; $resn_exch{"HPS2 SCD"}="2HPS SCD"; $resn_exch{"HS21 SCD"}="1HS2 SCD";
$resn_exch{"HS22 SCD"}="2HS2 SCD"; $resn_exch{"HS31 SCD"}="1HS3 SCD"; $resn_exch{"HS32 SCD"}="2HS3 SCD";
$resn_exch{"HOS5 SCD"}="5HOS SCD"; $resn_exch{"H111 SCE"}="1H11 SCE"; $resn_exch{"H112 SCE"}="2H11 SCE";
$resn_exch{"H181 SCE"}="1H18 SCE"; $resn_exch{"H182 SCE"}="2H18 SCE"; $resn_exch{"H183 SCE"}="3H18 SCE";
$resn_exch{"H121 SCF"}="1H12 SCF"; $resn_exch{"H122 SCF"}="2H12 SCF"; $resn_exch{" H91 SCG"}="1H9  SCG";
$resn_exch{" H92 SCG"}="2H9  SCG"; $resn_exch{" H81 SCG"}="1H8  SCG"; $resn_exch{" H82 SCG"}="2H8  SCG";
$resn_exch{"H151 SCG"}="1H15 SCG"; $resn_exch{"H152 SCG"}="2H15 SCG"; $resn_exch{"H153 SCG"}="3H15 SCG";
$resn_exch{"H231 SCG"}="1H23 SCG"; $resn_exch{"H232 SCG"}="2H23 SCG"; $resn_exch{" H2  SCH"}=" HN  SCH";
$resn_exch{" HB2 SCH"}="1HB  SCH"; $resn_exch{" HB3 SCH"}="2HB  SCH"; $resn_exch{" HE1 SCH"}="1HE  SCH";
$resn_exch{" HE2 SCH"}="2HE  SCH"; $resn_exch{" HE3 SCH"}="3HE  SCH"; $resn_exch{" HN1 SCI"}="1HN  SCI";
$resn_exch{" HN2 SCI"}="2HN  SCI"; $resn_exch{" HB1 SCI"}="1HB  SCI"; $resn_exch{" HB2 SCI"}="2HB  SCI";
$resn_exch{" HG1 SCI"}="1HG  SCI"; $resn_exch{" HG2 SCI"}="2HG  SCI"; $resn_exch{" HD1 SCI"}="1HD  SCI";
$resn_exch{" HD2 SCI"}="2HD  SCI"; $resn_exch{"HN11 SCI"}="1HN1 SCI"; $resn_exch{"HN12 SCI"}="2HN1 SCI";
$resn_exch{"H121 SCJ"}="1H12 SCJ"; $resn_exch{"H122 SCJ"}="2H12 SCJ"; $resn_exch{"H151 SCK"}="1H15 SCK";
$resn_exch{"H152 SCK"}="2H15 SCK"; $resn_exch{"H161 SCK"}="1H16 SCK"; $resn_exch{"H162 SCK"}="2H16 SCK";
$resn_exch{"H201 SCK"}="1H20 SCK"; $resn_exch{"H202 SCK"}="2H20 SCK"; $resn_exch{"H203 SCK"}="3H20 SCK";
$resn_exch{"H191 SCK"}="1H19 SCK"; $resn_exch{"H192 SCK"}="2H19 SCK"; $resn_exch{"H193 SCK"}="3H19 SCK";
$resn_exch{"H181 SCK"}="1H18 SCK"; $resn_exch{"H182 SCK"}="2H18 SCK"; $resn_exch{"H183 SCK"}="3H18 SCK";
$resn_exch{"H111 SCK"}="1H11 SCK"; $resn_exch{"H112 SCK"}="2H11 SCK"; $resn_exch{" H61 SCK"}="1H6  SCK";
$resn_exch{" H62 SCK"}="2H6  SCK"; $resn_exch{" H31 SCK"}="1H3  SCK"; $resn_exch{" H32 SCK"}="2H3  SCK";
$resn_exch{" H21 SCK"}="1H2  SCK"; $resn_exch{" H22 SCK"}="2H2  SCK"; $resn_exch{" H81 SCK"}="1H8  SCK";
$resn_exch{" H82 SCK"}="2H8  SCK"; $resn_exch{" H83 SCK"}="3H8  SCK"; $resn_exch{" H91 SCK"}="1H9  SCK";
$resn_exch{" H92 SCK"}="2H9  SCK"; $resn_exch{" H93 SCK"}="3H9  SCK"; $resn_exch{"H101 SCK"}="1H10 SCK";
$resn_exch{"H102 SCK"}="2H10 SCK"; $resn_exch{"H103 SCK"}="3H10 SCK"; $resn_exch{" H91 SCL"}="1H9  SCL";
$resn_exch{" H92 SCL"}="2H9  SCL"; $resn_exch{" H93 SCL"}="3H9  SCL"; $resn_exch{"H2M1 SCM"}="1H2M SCM";
$resn_exch{"H2M2 SCM"}="2H2M SCM"; $resn_exch{"H2M3 SCM"}="3H2M SCM"; $resn_exch{" H31 SCM"}="1H3  SCM";
$resn_exch{" H32 SCM"}="2H3  SCM"; $resn_exch{"H8M1 SCM"}="1H8M SCM"; $resn_exch{"H8M2 SCM"}="2H8M SCM";
$resn_exch{"H8M3 SCM"}="3H8M SCM"; $resn_exch{"H1M1 SCM"}="1H1M SCM"; $resn_exch{"H1M2 SCM"}="2H1M SCM";
$resn_exch{"H1M3 SCM"}="3H1M SCM"; $resn_exch{"HN61 SCO"}="1HN6 SCO"; $resn_exch{"HN62 SCO"}="2HN6 SCO";
$resn_exch{"HC2' SCO"}="*HC2 SCO"; $resn_exch{"HC51 SCO"}="1HC5 SCO"; $resn_exch{"HC52 SCO"}="2HC5 SCO";
$resn_exch{"H111 SCO"}="1H11 SCO"; $resn_exch{"H112 SCO"}="2H11 SCO"; $resn_exch{"H131 SCO"}="1H13 SCO";
$resn_exch{"H132 SCO"}="2H13 SCO"; $resn_exch{"H133 SCO"}="3H13 SCO"; $resn_exch{"H141 SCO"}="1H14 SCO";
$resn_exch{"H142 SCO"}="2H14 SCO"; $resn_exch{"H143 SCO"}="3H14 SCO"; $resn_exch{"H271 SCO"}="1H27 SCO";
$resn_exch{"H272 SCO"}="2H27 SCO"; $resn_exch{"H261 SCO"}="1H26 SCO"; $resn_exch{"H262 SCO"}="2H26 SCO";
$resn_exch{"H331 SCO"}="1H33 SCO"; $resn_exch{"H332 SCO"}="2H33 SCO"; $resn_exch{"H321 SCO"}="1H32 SCO";
$resn_exch{"H322 SCO"}="2H32 SCO"; $resn_exch{"H421 SCO"}="1H42 SCO"; $resn_exch{"H422 SCO"}="2H42 SCO";
$resn_exch{"H431 SCO"}="1H43 SCO"; $resn_exch{"H432 SCO"}="2H43 SCO"; $resn_exch{"H2A1 SCP"}="1H2A SCP";
$resn_exch{"H2A2 SCP"}="2H2A SCP"; $resn_exch{"H2A3 SCP"}="3H2A SCP"; $resn_exch{"H4A1 SCP"}="1H4A SCP";
$resn_exch{"H4A2 SCP"}="2H4A SCP"; $resn_exch{"H5A1 SCP"}="1H5A SCP"; $resn_exch{"H5A2 SCP"}="2H5A SCP";
$resn_exch{"HOP2 SCP"}="2HOP SCP"; $resn_exch{"HOP3 SCP"}="3HOP SCP"; $resn_exch{" HB1 SCP"}="1HB  SCP";
$resn_exch{" HB2 SCP"}="2HB  SCP"; $resn_exch{"H121 SCQ"}="1H12 SCQ"; $resn_exch{"H122 SCQ"}="2H12 SCQ";
$resn_exch{" H61 SCR"}="1H6  SCR"; $resn_exch{" H62 SCR"}="2H6  SCR"; $resn_exch{"H111 SCR"}="1H11 SCR";
$resn_exch{"H112 SCR"}="2H11 SCR"; $resn_exch{"H161 SCR"}="1H16 SCR"; $resn_exch{"H162 SCR"}="2H16 SCR";
$resn_exch{" H   SCS"}="1HN  SCS"; $resn_exch{" H2  SCS"}="2HN  SCS"; $resn_exch{" HC  SCS"}=" H   SCS";
$resn_exch{" HB2 SCS"}="1HB  SCS"; $resn_exch{" HB3 SCS"}="2HB  SCS"; $resn_exch{" HE1 SCS"}="1HE  SCS";
$resn_exch{" HE2 SCS"}="2HE  SCS"; $resn_exch{" HE3 SCS"}="3HE  SCS"; $resn_exch{" HZ2 SCS"}="1HZ  SCS";
$resn_exch{" HZ3 SCS"}="2HZ  SCS"; $resn_exch{"H141 SCT"}="1H14 SCT"; $resn_exch{"H142 SCT"}="2H14 SCT";
$resn_exch{"H121 SCT"}="1H12 SCT"; $resn_exch{"H122 SCT"}="2H12 SCT"; $resn_exch{"H151 SCT"}="1H15 SCT";
$resn_exch{"H152 SCT"}="2H15 SCT"; $resn_exch{"H3C1 SCT"}="1H3C SCT"; $resn_exch{"H3C2 SCT"}="2H3C SCT";
$resn_exch{"H3C3 SCT"}="3H3C SCT"; $resn_exch{"H111 SCU"}="1H11 SCU"; $resn_exch{"H112 SCU"}="2H11 SCU";
$resn_exch{" H61 SCU"}="1H6  SCU"; $resn_exch{" H62 SCU"}="2H6  SCU"; $resn_exch{"H151 SCU"}="1H15 SCU";
$resn_exch{"H152 SCU"}="2H15 SCU"; $resn_exch{"H161 SCU"}="1H16 SCU"; $resn_exch{"H162 SCU"}="2H16 SCU";
$resn_exch{"H201 SCU"}="1H20 SCU"; $resn_exch{"H202 SCU"}="2H20 SCU"; $resn_exch{"H203 SCU"}="3H20 SCU";
$resn_exch{"H191 SCU"}="1H19 SCU"; $resn_exch{"H192 SCU"}="2H19 SCU"; $resn_exch{"H193 SCU"}="3H19 SCU";
$resn_exch{"H181 SCU"}="1H18 SCU"; $resn_exch{"H182 SCU"}="2H18 SCU"; $resn_exch{"H183 SCU"}="3H18 SCU";
$resn_exch{" HC2 SCV"}="2HC  SCV"; $resn_exch{"HC31 SCV"}="1HC3 SCV"; $resn_exch{"HC32 SCV"}="2HC3 SCV";
$resn_exch{"HC41 SCV"}="1HC4 SCV"; $resn_exch{"HC42 SCV"}="2HC4 SCV"; $resn_exch{" HC7 SCV"}="7HC  SCV";
$resn_exch{"HC01 SCV"}="1HC0 SCV"; $resn_exch{"HC21 SCV"}="1HC2 SCV"; $resn_exch{"H131 SCV"}="1H13 SCV";
$resn_exch{"H132 SCV"}="2H13 SCV"; $resn_exch{"H133 SCV"}="3H13 SCV"; $resn_exch{"HC51 SCV"}="1HC5 SCV";
$resn_exch{"HC52 SCV"}="2HC5 SCV"; $resn_exch{"HN11 SCV"}="1HN1 SCV"; $resn_exch{"HN12 SCV"}="2HN1 SCV";
$resn_exch{" HO2 SCV"}="2HO  SCV"; $resn_exch{"H141 SCV"}="1H14 SCV"; $resn_exch{"H142 SCV"}="2H14 SCV";
$resn_exch{"H143 SCV"}="3H14 SCV"; $resn_exch{" HO6 SCV"}="6HO  SCV"; $resn_exch{" HN2 SCV"}="2HN  SCV";
$resn_exch{"H121 SCW"}="1H12 SCW"; $resn_exch{"H122 SCW"}="2H12 SCW"; $resn_exch{"H121 SCX"}="1H12 SCX";
$resn_exch{"H122 SCX"}="2H12 SCX"; $resn_exch{"H241 SCX"}="1H24 SCX"; $resn_exch{"H242 SCX"}="2H24 SCX";
$resn_exch{"H251 SCX"}="1H25 SCX"; $resn_exch{"H252 SCX"}="2H25 SCX"; $resn_exch{"H261 SCX"}="1H26 SCX";
$resn_exch{"H262 SCX"}="2H26 SCX"; $resn_exch{"H263 SCX"}="3H26 SCX"; $resn_exch{" H2  SCY"}=" HN2 SCY";
$resn_exch{" HB2 SCY"}="1HB  SCY"; $resn_exch{" HB3 SCY"}="2HB  SCY"; $resn_exch{" HE1 SCY"}="1HE  SCY";
$resn_exch{" HE2 SCY"}="2HE  SCY"; $resn_exch{" HE3 SCY"}="3HE  SCY"; $resn_exch{"H121 SCZ"}="1H12 SCZ";
$resn_exch{"H122 SCZ"}="2H12 SCZ"; $resn_exch{"H251 SCZ"}="1H25 SCZ"; $resn_exch{"H252 SCZ"}="2H25 SCZ";
$resn_exch{"H261 SCZ"}="1H26 SCZ"; $resn_exch{"H262 SCZ"}="2H26 SCZ"; $resn_exch{"HBE1 SD2"}="1HBE SD2";
$resn_exch{"HBE2 SD2"}="2HBE SD2"; $resn_exch{"HBE3 SD2"}="3HBE SD2"; $resn_exch{"HBC1 SD2"}="1HBC SD2";
$resn_exch{"HBC2 SD2"}="2HBC SD2"; $resn_exch{"HBB1 SD2"}="1HBB SD2"; $resn_exch{"HBB2 SD2"}="2HBB SD2";
$resn_exch{"HBF1 SD2"}="1HBF SD2"; $resn_exch{"HBF2 SD2"}="2HBF SD2"; $resn_exch{"HAU1 SD2"}="1HAU SD2";
$resn_exch{"HAU2 SD2"}="2HAU SD2"; $resn_exch{"HAV1 SD2"}="1HAV SD2"; $resn_exch{"HAV2 SD2"}="2HAV SD2";
$resn_exch{"HAW1 SD2"}="1HAW SD2"; $resn_exch{"HAW2 SD2"}="2HAW SD2"; $resn_exch{"HAD1 SD2"}="1HAD SD2";
$resn_exch{"HAD2 SD2"}="2HAD SD2"; $resn_exch{"HAI1 SD2"}="1HAI SD2"; $resn_exch{"HAI2 SD2"}="2HAI SD2";
$resn_exch{" H21 SD8"}="1H2  SD8"; $resn_exch{" H22 SD8"}="2H2  SD8"; $resn_exch{" H91 SD8"}="1H9  SD8";
$resn_exch{" H92 SD8"}="2H9  SD8"; $resn_exch{"H101 SD8"}="1H10 SD8"; $resn_exch{"H102 SD8"}="2H10 SD8";
$resn_exch{"H111 SD8"}="1H11 SD8"; $resn_exch{"H112 SD8"}="2H11 SD8"; $resn_exch{"H121 SD8"}="1H12 SD8";
$resn_exch{"H122 SD8"}="2H12 SD8"; $resn_exch{"H171 SD8"}="1H17 SD8"; $resn_exch{"H172 SD8"}="2H17 SD8";
$resn_exch{"H181 SD8"}="1H18 SD8"; $resn_exch{"H182 SD8"}="2H18 SD8"; $resn_exch{"H183 SD8"}="3H18 SD8";
$resn_exch{"H131 SDA"}="1H13 SDA"; $resn_exch{"H132 SDA"}="2H13 SDA"; $resn_exch{"H101 SDA"}="1H10 SDA";
$resn_exch{"H102 SDA"}="2H10 SDA"; $resn_exch{"HP21 SDC"}="1HP2 SDC"; $resn_exch{"HP22 SDC"}="2HP2 SDC";
$resn_exch{"H061 SDC"}="1H06 SDC"; $resn_exch{"H062 SDC"}="2H06 SDC"; $resn_exch{"H081 SDC"}="1H08 SDC";
$resn_exch{"H082 SDC"}="2H08 SDC"; $resn_exch{"H091 SDC"}="1H09 SDC"; $resn_exch{"H092 SDC"}="2H09 SDC";
$resn_exch{"H121 SDC"}="1H12 SDC"; $resn_exch{"H122 SDC"}="2H12 SDC"; $resn_exch{" H71 SDK"}="1H7  SDK";
$resn_exch{" H72 SDK"}="2H7  SDK"; $resn_exch{"H121 SDK"}="1H12 SDK"; $resn_exch{"H122 SDK"}="2H12 SDK";
$resn_exch{"H141 SDK"}="1H14 SDK"; $resn_exch{"H142 SDK"}="2H14 SDK"; $resn_exch{"H143 SDK"}="3H14 SDK";
$resn_exch{"H151 SDK"}="1H15 SDK"; $resn_exch{"H152 SDK"}="2H15 SDK"; $resn_exch{"H153 SDK"}="3H15 SDK";
$resn_exch{"H191 SDK"}="1H19 SDK"; $resn_exch{"H192 SDK"}="2H19 SDK"; $resn_exch{"H291 SDK"}="1H29 SDK";
$resn_exch{"H292 SDK"}="2H29 SDK"; $resn_exch{"H341 SDK"}="1H34 SDK"; $resn_exch{"H342 SDK"}="2H34 SDK";
$resn_exch{"H361 SDK"}="1H36 SDK"; $resn_exch{"H362 SDK"}="2H36 SDK"; $resn_exch{"H363 SDK"}="3H36 SDK";
$resn_exch{"H371 SDK"}="1H37 SDK"; $resn_exch{"H372 SDK"}="2H37 SDK"; $resn_exch{"H373 SDK"}="3H37 SDK";
$resn_exch{"H411 SDK"}="1H41 SDK"; $resn_exch{"H412 SDK"}="2H41 SDK"; $resn_exch{" H2  SDP"}=" HN2 SDP";
$resn_exch{" HB2 SDP"}="1HB  SDP"; $resn_exch{" HB3 SDP"}="2HB  SDP"; $resn_exch{"H311 SDP"}="1H31 SDP";
$resn_exch{"H312 SDP"}="2H31 SDP"; $resn_exch{"H321 SDP"}="1H32 SDP"; $resn_exch{"H322 SDP"}="2H32 SDP";
$resn_exch{"H323 SDP"}="3H32 SDP"; $resn_exch{"H411 SDP"}="1H41 SDP"; $resn_exch{"H412 SDP"}="2H41 SDP";
$resn_exch{"H421 SDP"}="1H42 SDP"; $resn_exch{"H422 SDP"}="2H42 SDP"; $resn_exch{"H423 SDP"}="3H42 SDP";
$resn_exch{"H1C1 SDS"}="1H1C SDS"; $resn_exch{"H1C2 SDS"}="2H1C SDS"; $resn_exch{"H2C1 SDS"}="1H2C SDS";
$resn_exch{"H2C2 SDS"}="2H2C SDS"; $resn_exch{"H2C3 SDS"}="3H2C SDS"; $resn_exch{"H3C1 SDS"}="1H3C SDS";
$resn_exch{"H3C2 SDS"}="2H3C SDS"; $resn_exch{"H4C1 SDS"}="1H4C SDS"; $resn_exch{"H4C2 SDS"}="2H4C SDS";
$resn_exch{"H5C1 SDS"}="1H5C SDS"; $resn_exch{"H5C2 SDS"}="2H5C SDS"; $resn_exch{"H6C1 SDS"}="1H6C SDS";
$resn_exch{"H6C2 SDS"}="2H6C SDS"; $resn_exch{"H7C1 SDS"}="1H7C SDS"; $resn_exch{"H7C2 SDS"}="2H7C SDS";
$resn_exch{"H8C1 SDS"}="1H8C SDS"; $resn_exch{"H8C2 SDS"}="2H8C SDS"; $resn_exch{"H9C1 SDS"}="1H9C SDS";
$resn_exch{"H9C2 SDS"}="2H9C SDS"; $resn_exch{"H101 SDS"}="1H10 SDS"; $resn_exch{"H102 SDS"}="2H10 SDS";
$resn_exch{"H111 SDS"}="1H11 SDS"; $resn_exch{"H112 SDS"}="2H11 SDS"; $resn_exch{"H121 SDS"}="1H12 SDS";
$resn_exch{"H122 SDS"}="2H12 SDS"; $resn_exch{" H71 SDZ"}="1H7  SDZ"; $resn_exch{" H72 SDZ"}="2H7  SDZ";
$resn_exch{" H81 SDZ"}="1H8  SDZ"; $resn_exch{" H82 SDZ"}="2H8  SDZ"; $resn_exch{" H91 SDZ"}="1H9  SDZ";
$resn_exch{" H92 SDZ"}="2H9  SDZ"; $resn_exch{"H101 SDZ"}="1H10 SDZ"; $resn_exch{"H102 SDZ"}="2H10 SDZ";
$resn_exch{"H111 SDZ"}="1H11 SDZ"; $resn_exch{"H112 SDZ"}="2H11 SDZ"; $resn_exch{"H131 SDZ"}="1H13 SDZ";
$resn_exch{"H132 SDZ"}="2H13 SDZ"; $resn_exch{"H211 SDZ"}="1H21 SDZ"; $resn_exch{"H212 SDZ"}="2H21 SDZ";
$resn_exch{"H221 SDZ"}="1H22 SDZ"; $resn_exch{"H222 SDZ"}="2H22 SDZ"; $resn_exch{"H231 SDZ"}="1H23 SDZ";
$resn_exch{"H232 SDZ"}="2H23 SDZ"; $resn_exch{"H241 SDZ"}="1H24 SDZ"; $resn_exch{"H242 SDZ"}="2H24 SDZ";
$resn_exch{"H251 SDZ"}="1H25 SDZ"; $resn_exch{"H252 SDZ"}="2H25 SDZ"; $resn_exch{" HN1 SEA"}="1HN  SEA";
$resn_exch{" HN2 SEA"}="2HN  SEA"; $resn_exch{" HA1 SEA"}="1HA  SEA"; $resn_exch{" HA2 SEA"}="2HA  SEA";
$resn_exch{" HB1 SEA"}="1HB  SEA"; $resn_exch{" HB2 SEA"}="2HB  SEA"; $resn_exch{" H2  SEB"}=" HN2 SEB";
$resn_exch{" HE3 SEB"}="1HE  SEB"; $resn_exch{" HE2 SEB"}="2HE  SEB"; $resn_exch{" HB3 SEB"}="1HB  SEB";
$resn_exch{" HB2 SEB"}="2HB  SEB"; $resn_exch{" H   SEC"}="1HN  SEC"; $resn_exch{" H2  SEC"}="2HN  SEC";
$resn_exch{" HB2 SEC"}="1HB  SEC"; $resn_exch{" HB3 SEC"}="2HB  SEC"; $resn_exch{" H   SEG"}="1HN  SEG";
$resn_exch{" H2  SEG"}="2HN  SEG"; $resn_exch{" HD  SEG"}=" HOD SEG"; $resn_exch{" HB3 SEG"}="1HB  SEG";
$resn_exch{" HB2 SEG"}="2HB  SEG"; $resn_exch{"H1C1 SEH"}="1H1C SEH"; $resn_exch{"H1C2 SEH"}="2H1C SEH";
$resn_exch{"H2C1 SEH"}="1H2C SEH"; $resn_exch{"H2C2 SEH"}="2H2C SEH"; $resn_exch{"H2C3 SEH"}="3H2C SEH";
$resn_exch{"HBC1 SEH"}="1HBC SEH"; $resn_exch{"HBC2 SEH"}="2HBC SEH"; $resn_exch{" H21 SEI"}="1H2  SEI";
$resn_exch{" H22 SEI"}="2H2  SEI"; $resn_exch{" H31 SEI"}="1H3  SEI"; $resn_exch{" H32 SEI"}="2H3  SEI";
$resn_exch{"H201 SEI"}="1H20 SEI"; $resn_exch{"H211 SEI"}="1H21 SEI"; $resn_exch{"H301 SEI"}="1H30 SEI";
$resn_exch{"H311 SEI"}="1H31 SEI"; $resn_exch{"H312 SEI"}="2H31 SEI"; $resn_exch{"H313 SEI"}="3H31 SEI";
$resn_exch{"H351 SEI"}="1H35 SEI"; $resn_exch{"H352 SEI"}="2H35 SEI"; $resn_exch{"H353 SEI"}="3H35 SEI";
$resn_exch{" H51 SEI"}="1H5  SEI"; $resn_exch{" H52 SEI"}="2H5  SEI"; $resn_exch{" H61 SEI"}="1H6  SEI";
$resn_exch{" H62 SEI"}="2H6  SEI"; $resn_exch{"H171 SEI"}="1H17 SEI"; $resn_exch{"H182 SEI"}="2H18 SEI";
$resn_exch{"H281 SEI"}="1H28 SEI"; $resn_exch{"H291 SEI"}="1H29 SEI"; $resn_exch{"H441 SEI"}="1H44 SEI";
$resn_exch{"H451 SEI"}="1H45 SEI"; $resn_exch{"H452 SEI"}="2H45 SEI"; $resn_exch{"H461 SEI"}="1H46 SEI";
$resn_exch{"H462 SEI"}="2H46 SEI"; $resn_exch{"H471 SEI"}="1H47 SEI"; $resn_exch{"H472 SEI"}="2H47 SEI";
$resn_exch{"H531 SEI"}="1H53 SEI"; $resn_exch{"H591 SEI"}="1H59 SEI"; $resn_exch{"H611 SEI"}="1H61 SEI";
$resn_exch{"H621 SEI"}="1H62 SEI"; $resn_exch{"H622 SEI"}="2H62 SEI"; $resn_exch{"H623 SEI"}="3H62 SEI";
$resn_exch{"H661 SEI"}="1H66 SEI"; $resn_exch{"H662 SEI"}="2H66 SEI"; $resn_exch{"H663 SEI"}="3H66 SEI";
$resn_exch{"HN11 SEL"}="1HN1 SEL"; $resn_exch{"HN12 SEL"}="2HN1 SEL"; $resn_exch{" HA1 SEL"}="1HA  SEL";
$resn_exch{" HB1 SEL"}="1HB  SEL"; $resn_exch{" HB2 SEL"}="2HB  SEL"; $resn_exch{"HB21 SEL"}="1HB2 SEL";
$resn_exch{"HB22 SEL"}="2HB2 SEL"; $resn_exch{"HO21 SEL"}="1HO2 SEL"; $resn_exch{" H11 SEM"}="1H1  SEM";
$resn_exch{" H12 SEM"}="2H1  SEM"; $resn_exch{" H13 SEM"}="3H1  SEM"; $resn_exch{"HN31 SEM"}="1HN3 SEM";
$resn_exch{"HN32 SEM"}="2HN3 SEM"; $resn_exch{" H41 SEM"}="1H4  SEM"; $resn_exch{" H42 SEM"}="2H4  SEM";
$resn_exch{" H'1 SEM"}="1H'  SEM"; $resn_exch{" H'2 SEM"}="2H'  SEM"; $resn_exch{" HO1 SEO"}="1HO  SEO";
$resn_exch{"HB11 SEO"}="1HB1 SEO"; $resn_exch{"HB12 SEO"}="2HB1 SEO"; $resn_exch{" HB1 SEO"}="1HB  SEO";
$resn_exch{" HB2 SEO"}="2HB  SEO"; $resn_exch{" H2  SEP"}="2HN  SEP"; $resn_exch{" HB2 SEP"}="1HB  SEP";
$resn_exch{" HB3 SEP"}="2HB  SEP"; $resn_exch{"HOP2 SEP"}="2HOP SEP"; $resn_exch{"HOP3 SEP"}="3HOP SEP";
$resn_exch{" H2  SER"}=" HN2 SER"; $resn_exch{" HB2 SER"}="1HB  SER"; $resn_exch{" HB3 SER"}="2HB  SER";
$resn_exch{" H11 SES"}="1H1  SES"; $resn_exch{" H12 SES"}="2H1  SES"; $resn_exch{" H13 SES"}="3H1  SES";
$resn_exch{" H21 SES"}="1H2  SES"; $resn_exch{" H22 SES"}="2H2  SES"; $resn_exch{" H31 SES"}="1H3  SES";
$resn_exch{" H32 SES"}="2H3  SES"; $resn_exch{" H41 SES"}="1H4  SES"; $resn_exch{" H42 SES"}="2H4  SES";
$resn_exch{"H111 SES"}="1H11 SES"; $resn_exch{"H112 SES"}="2H11 SES"; $resn_exch{"H113 SES"}="3H11 SES";
$resn_exch{"H121 SES"}="1H12 SES"; $resn_exch{"H122 SES"}="2H12 SES"; $resn_exch{"H131 SES"}="1H13 SES";
$resn_exch{"H132 SES"}="2H13 SES"; $resn_exch{"HN21 SES"}="1HN2 SES"; $resn_exch{"HN22 SES"}="2HN2 SES";
$resn_exch{" H   SET"}="1HN  SET"; $resn_exch{" H2  SET"}="2HN  SET"; $resn_exch{" HB2 SET"}="1HB  SET";
$resn_exch{" HB3 SET"}="2HB  SET"; $resn_exch{" HG  SET"}=" HOG SET"; $resn_exch{"HNT1 SET"}="1HNT SET";
$resn_exch{"HNT2 SET"}="2HNT SET"; $resn_exch{" H11 SEU"}="1H1  SEU"; $resn_exch{" H12 SEU"}="2H1  SEU";
$resn_exch{" H13 SEU"}="3H1  SEU"; $resn_exch{" H21 SEU"}="1H2  SEU"; $resn_exch{" H22 SEU"}="2H2  SEU";
$resn_exch{"HN61 SEU"}="1HN6 SEU"; $resn_exch{"HN62 SEU"}="2HN6 SEU"; $resn_exch{"HAQ1 SF1"}="1HAQ SF1";
$resn_exch{"HAQ2 SF1"}="2HAQ SF1"; $resn_exch{"HAA1 SF2"}="1HAA SF2"; $resn_exch{"HAA2 SF2"}="2HAA SF2";
$resn_exch{"HAA3 SF2"}="3HAA SF2"; $resn_exch{"HAQ1 SF2"}="1HAQ SF2"; $resn_exch{"HAQ2 SF2"}="2HAQ SF2";
$resn_exch{"H431 SFA"}="1H43 SFA"; $resn_exch{"H432 SFA"}="2H43 SFA"; $resn_exch{"H441 SFA"}="1H44 SFA";
$resn_exch{"H442 SFA"}="2H44 SFA"; $resn_exch{"H443 SFA"}="3H44 SFA"; $resn_exch{"H391 SFA"}="1H39 SFA";
$resn_exch{"H392 SFA"}="2H39 SFA"; $resn_exch{"H451 SFA"}="1H45 SFA"; $resn_exch{"H452 SFA"}="2H45 SFA";
$resn_exch{"H453 SFA"}="3H45 SFA"; $resn_exch{"H461 SFA"}="1H46 SFA"; $resn_exch{"H462 SFA"}="2H46 SFA";
$resn_exch{"H463 SFA"}="3H46 SFA"; $resn_exch{"H471 SFA"}="1H47 SFA"; $resn_exch{"H472 SFA"}="2H47 SFA";
$resn_exch{"H473 SFA"}="3H47 SFA"; $resn_exch{"H321 SFA"}="1H32 SFA"; $resn_exch{"H322 SFA"}="2H32 SFA";
$resn_exch{"H481 SFA"}="1H48 SFA"; $resn_exch{"H482 SFA"}="2H48 SFA"; $resn_exch{"H483 SFA"}="3H48 SFA";
$resn_exch{"H291 SFA"}="1H29 SFA"; $resn_exch{"H292 SFA"}="2H29 SFA"; $resn_exch{"H281 SFA"}="1H28 SFA";
$resn_exch{"H282 SFA"}="2H28 SFA"; $resn_exch{"H491 SFA"}="1H49 SFA"; $resn_exch{"H492 SFA"}="2H49 SFA";
$resn_exch{"H493 SFA"}="3H49 SFA"; $resn_exch{"H221 SFA"}="1H22 SFA"; $resn_exch{"H222 SFA"}="2H22 SFA";
$resn_exch{"H501 SFA"}="1H50 SFA"; $resn_exch{"H502 SFA"}="2H50 SFA"; $resn_exch{"H503 SFA"}="3H50 SFA";
$resn_exch{"H511 SFA"}="1H51 SFA"; $resn_exch{"H512 SFA"}="2H51 SFA"; $resn_exch{"H521 SFA"}="1H52 SFA";
$resn_exch{"H522 SFA"}="2H52 SFA"; $resn_exch{"H541 SFA"}="1H54 SFA"; $resn_exch{"H542 SFA"}="2H54 SFA";
$resn_exch{"H543 SFA"}="3H54 SFA"; $resn_exch{"H561 SFA"}="1H56 SFA"; $resn_exch{"H562 SFA"}="2H56 SFA";
$resn_exch{"H563 SFA"}="3H56 SFA"; $resn_exch{"H571 SFA"}="1H57 SFA"; $resn_exch{"H572 SFA"}="2H57 SFA";
$resn_exch{"H573 SFA"}="3H57 SFA"; $resn_exch{"H581 SFA"}="1H58 SFA"; $resn_exch{"H582 SFA"}="2H58 SFA";
$resn_exch{" H51 SFA"}="1H5  SFA"; $resn_exch{" H52 SFA"}="2H5  SFA"; $resn_exch{" H41 SFA"}="1H4  SFA";
$resn_exch{"H42A SFA"}="2H4  SFA"; $resn_exch{"H31A SFA"}="1H3  SFA"; $resn_exch{" H32 SFA"}="2H3  SFA";
$resn_exch{" H3' SFC"}=" H3* SFC"; $resn_exch{" H2' SFC"}=" H2* SFC"; $resn_exch{" H1' SFC"}=" H1* SFC";
$resn_exch{"HN61 SFC"}="1HN6 SFC"; $resn_exch{"HN62 SFC"}="2HN6 SFC"; $resn_exch{" H4' SFC"}=" H4* SFC";
$resn_exch{"H5'1 SFC"}="1H5* SFC"; $resn_exch{"H5'2 SFC"}="2H5* SFC"; $resn_exch{" H41 SFC"}="1H4  SFC";
$resn_exch{" H42 SFC"}="2H4  SFC"; $resn_exch{"H11X SFC"}="1H1  SFC"; $resn_exch{" H12 SFC"}="2H1  SFC";
$resn_exch{" H13 SFC"}="3H1  SFC"; $resn_exch{"H141 SFC"}="1H14 SFC"; $resn_exch{"H142 SFC"}="2H14 SFC";
$resn_exch{"H143 SFC"}="3H14 SFC"; $resn_exch{" H71 SFC"}="1H7  SFC"; $resn_exch{" H72 SFC"}="2H7  SFC";
$resn_exch{" H61 SFC"}="1H6  SFC"; $resn_exch{" H62 SFC"}="2H6  SFC"; $resn_exch{" H31 SFC"}="1H3  SFC";
$resn_exch{" H32 SFC"}="2H3  SFC"; $resn_exch{" H21 SFC"}="1H2  SFC"; $resn_exch{" H22 SFC"}="2H2  SFC";
$resn_exch{"H131 SFC"}="1H13 SFC"; $resn_exch{"H132 SFC"}="2H13 SFC"; $resn_exch{"H133 SFC"}="3H13 SFC";
$resn_exch{" H91 SFC"}="1H9  SFC"; $resn_exch{" H92 SFC"}="2H9  SFC"; $resn_exch{"H121 SFC"}="1H12 SFC";
$resn_exch{"H122 SFC"}="2H12 SFC"; $resn_exch{"H123 SFC"}="3H12 SFC"; $resn_exch{"H111 SFC"}="1H11 SFC";
$resn_exch{"H112 SFC"}="2H11 SFC"; $resn_exch{"H113 SFC"}="3H11 SFC"; $resn_exch{"H3F1 SFD"}="1H3F SFD";
$resn_exch{"H3F2 SFD"}="2H3F SFD"; $resn_exch{"H3F3 SFD"}="3H3F SFD"; $resn_exch{"HAF1 SFD"}="1HAF SFD";
$resn_exch{"HAF2 SFD"}="2HAF SFD"; $resn_exch{"HAF3 SFD"}="3HAF SFD"; $resn_exch{"H1R1 SFD"}="1H1R SFD";
$resn_exch{"H1R2 SFD"}="2H1R SFD"; $resn_exch{"H5R1 SFD"}="1H5R SFD"; $resn_exch{"H5R2 SFD"}="2H5R SFD";
$resn_exch{"H0R1 SFD"}="1H0R SFD"; $resn_exch{"H0R2 SFD"}="2H0R SFD"; $resn_exch{"HN91 SFD"}="1HN9 SFD";
$resn_exch{"HN92 SFD"}="2HN9 SFD"; $resn_exch{" HN1 SFG"}="1HN  SFG"; $resn_exch{" HN2 SFG"}="2HN  SFG";
$resn_exch{" HB1 SFG"}="1HB  SFG"; $resn_exch{" HB2 SFG"}="2HB  SFG"; $resn_exch{" HG1 SFG"}="1HG  SFG";
$resn_exch{" HG2 SFG"}="2HG  SFG"; $resn_exch{"HNE1 SFG"}="1HNE SFG"; $resn_exch{"HNE2 SFG"}="2HNE SFG";
$resn_exch{"H5'1 SFG"}="1H5* SFG"; $resn_exch{"H5'2 SFG"}="2H5* SFG"; $resn_exch{" H4' SFG"}=" H4* SFG";
$resn_exch{" H3' SFG"}=" H3* SFG"; $resn_exch{"HO3' SFG"}="*HO3 SFG"; $resn_exch{" H2' SFG"}=" H2* SFG";
$resn_exch{"HO2' SFG"}="*HO2 SFG"; $resn_exch{" H1' SFG"}=" H1* SFG"; $resn_exch{"HN61 SFG"}="1HN6 SFG";
$resn_exch{"HN62 SFG"}="2HN6 SFG"; $resn_exch{"H541 SFM"}="1H54 SFM"; $resn_exch{"H542 SFM"}="2H54 SFM";
$resn_exch{"H543 SFM"}="3H54 SFM"; $resn_exch{"H521 SFM"}="1H52 SFM"; $resn_exch{"H522 SFM"}="2H52 SFM";
$resn_exch{"H511 SFM"}="1H51 SFM"; $resn_exch{"H512 SFM"}="2H51 SFM"; $resn_exch{"H561 SFM"}="1H56 SFM";
$resn_exch{"H562 SFM"}="2H56 SFM"; $resn_exch{"H563 SFM"}="3H56 SFM"; $resn_exch{"H571 SFM"}="1H57 SFM";
$resn_exch{"H572 SFM"}="2H57 SFM"; $resn_exch{"H573 SFM"}="3H57 SFM"; $resn_exch{"H581 SFM"}="1H58 SFM";
$resn_exch{"H582 SFM"}="2H58 SFM"; $resn_exch{" H51 SFM"}="1H5  SFM"; $resn_exch{" H52 SFM"}="2H5  SFM";
$resn_exch{" H41 SFM"}="1H4  SFM"; $resn_exch{" H42 SFM"}="2H4  SFM"; $resn_exch{" H31 SFM"}="1H3  SFM";
$resn_exch{" H32 SFM"}="2H3  SFM"; $resn_exch{"H221 SFM"}="1H22 SFM"; $resn_exch{"H222 SFM"}="2H22 SFM";
$resn_exch{"H501 SFM"}="1H50 SFM"; $resn_exch{"H502 SFM"}="2H50 SFM"; $resn_exch{"H503 SFM"}="3H50 SFM";
$resn_exch{"H491 SFM"}="1H49 SFM"; $resn_exch{"H492 SFM"}="2H49 SFM"; $resn_exch{"H493 SFM"}="3H49 SFM";
$resn_exch{"H261 SFM"}="1H26 SFM"; $resn_exch{"H262 SFM"}="2H26 SFM"; $resn_exch{"H2'1 SFT"}="1H2' SFT";
$resn_exch{"H2'2 SFT"}="2H2' SFT"; $resn_exch{"H2'3 SFT"}="3H2' SFT"; $resn_exch{"H2A1 SFT"}="1H2A SFT";
$resn_exch{"H2A2 SFT"}="2H2A SFT"; $resn_exch{"H2B1 SFT"}="1H2B SFT"; $resn_exch{"H2B2 SFT"}="2H2B SFT";
$resn_exch{"H2C1 SFT"}="1H2C SFT"; $resn_exch{"H2C2 SFT"}="2H2C SFT"; $resn_exch{"H2D1 SFT"}="1H2D SFT";
$resn_exch{"H2D2 SFT"}="2H2D SFT"; $resn_exch{"H1'1 SFT"}="1H1' SFT"; $resn_exch{"H1'2 SFT"}="2H1' SFT";
$resn_exch{"H1A1 SFT"}="1H1A SFT"; $resn_exch{"H1A2 SFT"}="2H1A SFT"; $resn_exch{"H1B1 SFT"}="1H1B SFT";
$resn_exch{"H1B2 SFT"}="2H1B SFT"; $resn_exch{"H1C1 SFT"}="1H1C SFT"; $resn_exch{"H1C2 SFT"}="2H1C SFT";
$resn_exch{"H1D1 SFT"}="1H1D SFT"; $resn_exch{"H1D2 SFT"}="2H1D SFT"; $resn_exch{"H1E1 SFT"}="1H1E SFT";
$resn_exch{"H1E2 SFT"}="2H1E SFT"; $resn_exch{"H1F1 SFT"}="1H1F SFT"; $resn_exch{"H1F2 SFT"}="2H1F SFT";
$resn_exch{"H1G1 SFT"}="1H1G SFT"; $resn_exch{"H1G2 SFT"}="2H1G SFT"; $resn_exch{"H1H1 SFT"}="1H1H SFT";
$resn_exch{"H1H2 SFT"}="2H1H SFT"; $resn_exch{"H1I1 SFT"}="1H1I SFT"; $resn_exch{"H1I2 SFT"}="2H1I SFT";
$resn_exch{"H9'1 SFT"}="1H9' SFT"; $resn_exch{"H9'2 SFT"}="2H9' SFT"; $resn_exch{"H8'1 SFT"}="1H8' SFT";
$resn_exch{"H8'2 SFT"}="2H8' SFT"; $resn_exch{"H7'1 SFT"}="1H7' SFT"; $resn_exch{"H7'2 SFT"}="2H7' SFT";
$resn_exch{"H6'1 SFT"}="1H6' SFT"; $resn_exch{"H6'2 SFT"}="2H6' SFT"; $resn_exch{"H5'1 SFT"}="1H5' SFT";
$resn_exch{"H5'2 SFT"}="2H5' SFT"; $resn_exch{"H4'1 SFT"}="1H4' SFT"; $resn_exch{"H4'2 SFT"}="2H4' SFT";
$resn_exch{"H3'1 SFT"}="1H3' SFT"; $resn_exch{"H3'2 SFT"}="2H3' SFT"; $resn_exch{" HA1 SFT"}="1HA  SFT";
$resn_exch{" HB2 SFT"}="2HB  SFT"; $resn_exch{" H1' SFT"}=" H1* SFT"; $resn_exch{" H2B SFT"}=" H2* SFT";
$resn_exch{" H5' SFT"}=" H5* SFT"; $resn_exch{"H6A1 SFT"}="1H6* SFT"; $resn_exch{"H6A2 SFT"}="2H6* SFT";
$resn_exch{" H6' SFT"}=" H6* SFT"; $resn_exch{" H4' SFT"}=" H4* SFT"; $resn_exch{" H3' SFT"}=" H3* SFT";
$resn_exch{"H6C1 SFT"}="1H6C SFT"; $resn_exch{"H6C2 SFT"}="2H6C SFT"; $resn_exch{"H7C1 SFT"}="1H7C SFT";
$resn_exch{"H7C2 SFT"}="2H7C SFT"; $resn_exch{"H8C1 SFT"}="1H8C SFT"; $resn_exch{"H8C2 SFT"}="2H8C SFT";
$resn_exch{"H9C1 SFT"}="1H9C SFT"; $resn_exch{"H9C2 SFT"}="2H9C SFT"; $resn_exch{"H101 SFT"}="1H10 SFT";
$resn_exch{"H102 SFT"}="2H10 SFT"; $resn_exch{"H111 SFT"}="1H11 SFT"; $resn_exch{"H112 SFT"}="2H11 SFT";
$resn_exch{"H121 SFT"}="1H12 SFT"; $resn_exch{"H122 SFT"}="2H12 SFT"; $resn_exch{"H131 SFT"}="1H13 SFT";
$resn_exch{"H132 SFT"}="2H13 SFT"; $resn_exch{"H141 SFT"}="1H14 SFT"; $resn_exch{"H142 SFT"}="2H14 SFT";
$resn_exch{"H151 SFT"}="1H15 SFT"; $resn_exch{"H152 SFT"}="2H15 SFT"; $resn_exch{"H161 SFT"}="1H16 SFT";
$resn_exch{"H162 SFT"}="2H16 SFT"; $resn_exch{"H171 SFT"}="1H17 SFT"; $resn_exch{"H172 SFT"}="2H17 SFT";
$resn_exch{"H181 SFT"}="1H18 SFT"; $resn_exch{"H182 SFT"}="2H18 SFT"; $resn_exch{"H183 SFT"}="3H18 SFT";
$resn_exch{" H31 SG1"}="1H3  SG1"; $resn_exch{" H41 SG1"}="1H4  SG1"; $resn_exch{" H91 SG1"}="1H9  SG1";
$resn_exch{" H92 SG1"}="2H9  SG1"; $resn_exch{"H101 SG1"}="1H10 SG1"; $resn_exch{"H102 SG1"}="2H10 SG1";
$resn_exch{"H111 SG1"}="1H11 SG1"; $resn_exch{"H112 SG1"}="2H11 SG1"; $resn_exch{"H161 SG1"}="1H16 SG1";
$resn_exch{"H171 SG1"}="1H17 SG1"; $resn_exch{"H172 SG1"}="2H17 SG1"; $resn_exch{" H31 SG2"}="1H3  SG2";
$resn_exch{" H41 SG2"}="1H4  SG2"; $resn_exch{" H61 SG2"}="1H6  SG2"; $resn_exch{" H62 SG2"}="2H6  SG2";
$resn_exch{" H63 SG2"}="3H6  SG2"; $resn_exch{"H111 SG2"}="1H11 SG2"; $resn_exch{"H121 SG2"}="1H12 SG2";
$resn_exch{"H122 SG2"}="2H12 SG2"; $resn_exch{"H5C2 SG3"}="2H5C SG3"; $resn_exch{"H6C2 SG3"}="2H6C SG3";
$resn_exch{"H121 SG3"}="1H12 SG3"; $resn_exch{"H122 SG3"}="2H12 SG3"; $resn_exch{"H123 SG3"}="3H12 SG3";
$resn_exch{"H9C1 SG3"}="1H9C SG3"; $resn_exch{"H9C2 SG3"}="2H9C SG3"; $resn_exch{" H61 SGA"}="1H6  SGA";
$resn_exch{" H62 SGA"}="2H6  SGA"; $resn_exch{"HOS3 SGA"}="3HOS SGA"; $resn_exch{" HN1 SGB"}="1HN  SGB";
$resn_exch{" HN2 SGB"}="2HN  SGB"; $resn_exch{"HBC1 SGB"}="1HBC SGB"; $resn_exch{"HBC2 SGB"}="2HBC SGB";
$resn_exch{"H1C1 SGB"}="1H1C SGB"; $resn_exch{"H1C2 SGB"}="2H1C SGB"; $resn_exch{"H1C3 SGB"}="3H1C SGB";
$resn_exch{"H4C1 SGB"}="1H4C SGB"; $resn_exch{"H4C2 SGB"}="2H4C SGB"; $resn_exch{"H4C3 SGB"}="3H4C SGB";
$resn_exch{"H3C1 SGB"}="1H3C SGB"; $resn_exch{"H3C2 SGB"}="2H3C SGB"; $resn_exch{"H3C3 SGB"}="3H3C SGB";
$resn_exch{" H62 SGC"}="2H6  SGC"; $resn_exch{" H61 SGC"}="1H6  SGC"; $resn_exch{" HN1 SGI"}="1HN  SGI";
$resn_exch{" HN2 SGI"}="2HN  SGI"; $resn_exch{" HA1 SGI"}="1HA  SGI"; $resn_exch{" HA2 SGI"}="2HA  SGI";
$resn_exch{" HB1 SGI"}="1HB  SGI"; $resn_exch{" HB2 SGI"}="2HB  SGI"; $resn_exch{" H11 SGL"}="1H1  SGL";
$resn_exch{" H12 SGL"}="2H1  SGL"; $resn_exch{" H31 SGL"}="1H3  SGL"; $resn_exch{" H32 SGL"}="2H3  SGL";
$resn_exch{" H11 SGM"}="1H1  SGM"; $resn_exch{" H12 SGM"}="2H1  SGM"; $resn_exch{" H31 SGM"}="1H3  SGM";
$resn_exch{" H32 SGM"}="2H3  SGM"; $resn_exch{" H61 SGN"}="1H6  SGN"; $resn_exch{" H62 SGN"}="2H6  SGN";
$resn_exch{"HOS3 SGN"}="3HOS SGN"; $resn_exch{"HOS6 SGN"}="6HOS SGN"; $resn_exch{"HOP1 SGP"}="1HOP SGP";
$resn_exch{"HO5' SGP"}="*HO5 SGP"; $resn_exch{"H5'1 SGP"}="1H5* SGP"; $resn_exch{"H5'2 SGP"}="2H5* SGP";
$resn_exch{" H4' SGP"}=" H4* SGP"; $resn_exch{" H3' SGP"}=" H3* SGP"; $resn_exch{" H2' SGP"}=" H2* SGP";
$resn_exch{" H1' SGP"}=" H1* SGP"; $resn_exch{"HN21 SGP"}="1HN2 SGP"; $resn_exch{"HN22 SGP"}="2HN2 SGP";
$resn_exch{"HBC1 SGR"}="1HBC SGR"; $resn_exch{"HBC2 SGR"}="2HBC SGR"; $resn_exch{"H4N1 SGR"}="1H4N SGR";
$resn_exch{"H4N2 SGR"}="2H4N SGR"; $resn_exch{"H3C1 SGR"}="1H3C SGR"; $resn_exch{"H3C2 SGR"}="2H3C SGR";
$resn_exch{"H3C3 SGR"}="3H3C SGR"; $resn_exch{" HA1 SGX"}="1HA  SGX"; $resn_exch{" HB2 SGX"}="2HB  SGX";
$resn_exch{"H3N1 SGX"}="1H3N SGX"; $resn_exch{"H3N2 SGX"}="2H3N SGX"; $resn_exch{"H1C1 SGX"}="1H1C SGX";
$resn_exch{"H1C2 SGX"}="2H1C SGX"; $resn_exch{"H1C3 SGX"}="3H1C SGX"; $resn_exch{"H1C1 SH1"}="1H1C SH1";
$resn_exch{"H1C2 SH1"}="2H1C SH1"; $resn_exch{"H2C1 SH1"}="1H2C SH1"; $resn_exch{"H2C2 SH1"}="2H2C SH1";
$resn_exch{"H101 SH1"}="1H10 SH1"; $resn_exch{"H102 SH1"}="2H10 SH1"; $resn_exch{"H131 SH1"}="1H13 SH1";
$resn_exch{"H132 SH1"}="2H13 SH1"; $resn_exch{"H133 SH1"}="3H13 SH1"; $resn_exch{"H161 SH1"}="1H16 SH1";
$resn_exch{"H162 SH1"}="2H16 SH1"; $resn_exch{"H171 SH1"}="1H17 SH1"; $resn_exch{"H172 SH1"}="2H17 SH1";
$resn_exch{"H181 SH1"}="1H18 SH1"; $resn_exch{"H182 SH1"}="2H18 SH1"; $resn_exch{"H5N1 SH1"}="1H5N SH1";
$resn_exch{"H5N2 SH1"}="2H5N SH1"; $resn_exch{"H6N1 SH1"}="1H6N SH1"; $resn_exch{"H6N2 SH1"}="2H6N SH1";
$resn_exch{"H201 SH1"}="1H20 SH1"; $resn_exch{"H202 SH1"}="2H20 SH1"; $resn_exch{" H81 SH4"}="1H8  SH4";
$resn_exch{" H82 SH4"}="2H8  SH4"; $resn_exch{" H83 SH4"}="3H8  SH4"; $resn_exch{" H91 SH4"}="1H9  SH4";
$resn_exch{" H92 SH4"}="2H9  SH4"; $resn_exch{"H151 SH4"}="1H15 SH4"; $resn_exch{"H152 SH4"}="2H15 SH4";
$resn_exch{"H153 SH4"}="3H15 SH4"; $resn_exch{" H   SHC"}="1HN  SHC"; $resn_exch{" H2  SHC"}="2HN  SHC";
$resn_exch{" HB2 SHC"}="1HB  SHC"; $resn_exch{" HB3 SHC"}="2HB  SHC"; $resn_exch{" H11 SHC"}="1H1  SHC";
$resn_exch{" H12 SHC"}="2H1  SHC"; $resn_exch{" H21 SHC"}="1H2  SHC"; $resn_exch{" H22 SHC"}="2H2  SHC";
$resn_exch{" H31 SHC"}="1H3  SHC"; $resn_exch{" H32 SHC"}="2H3  SHC"; $resn_exch{" H41 SHC"}="1H4  SHC";
$resn_exch{" H42 SHC"}="2H4  SHC"; $resn_exch{" H51 SHC"}="1H5  SHC"; $resn_exch{" H52 SHC"}="2H5  SHC";
$resn_exch{" H61 SHC"}="1H6  SHC"; $resn_exch{" H62 SHC"}="2H6  SHC"; $resn_exch{" H63 SHC"}="3H6  SHC";
$resn_exch{" H51 SHF"}="1H5  SHF"; $resn_exch{" H52 SHF"}="2H5  SHF"; $resn_exch{" H53 SHF"}="3H5  SHF";
$resn_exch{" H31 SHF"}="1H3  SHF"; $resn_exch{" H32 SHF"}="2H3  SHF"; $resn_exch{" H21 SHF"}="1H2  SHF";
$resn_exch{" H22 SHF"}="2H2  SHF"; $resn_exch{" H21 SHH"}="1H2  SHH"; $resn_exch{" H22 SHH"}="2H2  SHH";
$resn_exch{" H31 SHH"}="1H3  SHH"; $resn_exch{" H32 SHH"}="2H3  SHH"; $resn_exch{" H41 SHH"}="1H4  SHH";
$resn_exch{" H42 SHH"}="2H4  SHH"; $resn_exch{" H51 SHH"}="1H5  SHH"; $resn_exch{" H52 SHH"}="2H5  SHH";
$resn_exch{" H61 SHH"}="1H6  SHH"; $resn_exch{" H62 SHH"}="2H6  SHH"; $resn_exch{" H71 SHH"}="1H7  SHH";
$resn_exch{" H72 SHH"}="2H7  SHH"; $resn_exch{"H101 SHH"}="1H10 SHH"; $resn_exch{"H111 SHH"}="1H11 SHH";
$resn_exch{"H121 SHH"}="1H12 SHH"; $resn_exch{"H131 SHH"}="1H13 SHH"; $resn_exch{"H141 SHH"}="1H14 SHH";
$resn_exch{"H411 SHI"}="1H41 SHI"; $resn_exch{"H412 SHI"}="2H41 SHI"; $resn_exch{"HN11 SHI"}="1HN1 SHI";
$resn_exch{"HN12 SHI"}="2HN1 SHI"; $resn_exch{"H11A SHM"}="1H1  SHM"; $resn_exch{" H21 SHM"}="1H2  SHM";
$resn_exch{"H12A SHM"}="2H1  SHM"; $resn_exch{" H22 SHM"}="2H2  SHM"; $resn_exch{" H13 SHM"}="3H1  SHM";
$resn_exch{" H23 SHM"}="3H2  SHM"; $resn_exch{" H14 SHM"}="4H1  SHM"; $resn_exch{" H24 SHM"}="4H2  SHM";
$resn_exch{" H15 SHM"}="5H1  SHM"; $resn_exch{" H25 SHM"}="5H2  SHM"; $resn_exch{"HAC1 SHO"}="1HAC SHO";
$resn_exch{"HAC2 SHO"}="2HAC SHO"; $resn_exch{"HBC1 SHO"}="1HBC SHO"; $resn_exch{"HBC2 SHO"}="2HBC SHO";
$resn_exch{"HGC1 SHO"}="1HGC SHO"; $resn_exch{"HGC2 SHO"}="2HGC SHO"; $resn_exch{"HDC1 SHO"}="1HDC SHO";
$resn_exch{"HDC2 SHO"}="2HDC SHO"; $resn_exch{" H   SHP"}="1HN  SHP"; $resn_exch{" H2  SHP"}="2HN  SHP";
$resn_exch{" H2B SHP"}=" H2* SHP"; $resn_exch{" H3B SHP"}=" H3* SHP"; $resn_exch{" H5B SHP"}=" H5* SHP";
$resn_exch{" H6' SHP"}=" H6* SHP"; $resn_exch{" HC2 SHP"}=" H2  SHP"; $resn_exch{" H61 SHP"}="1H6  SHP";
$resn_exch{" H62 SHP"}="2H6  SHP"; $resn_exch{"H6'1 SHP"}="1H6' SHP"; $resn_exch{"H6'2 SHP"}="2H6' SHP";
$resn_exch{"HO2' SHP"}="'HO2 SHP"; $resn_exch{"HO3' SHP"}="'HO3 SHP"; $resn_exch{"HO4' SHP"}="'HO4 SHP";
$resn_exch{"HO6' SHP"}="'HO6 SHP"; $resn_exch{" H12 SHR"}="2H1  SHR"; $resn_exch{" H11 SHR"}="1H1  SHR";
$resn_exch{" H92 SHR"}="2H9  SHR"; $resn_exch{" H91 SHR"}="1H9  SHR"; $resn_exch{" H82 SHR"}="2H8  SHR";
$resn_exch{" H81 SHR"}="1H8  SHR"; $resn_exch{" H72 SHR"}="2H7  SHR"; $resn_exch{" H71 SHR"}="1H7  SHR";
$resn_exch{" H62 SHR"}="2H6  SHR"; $resn_exch{" H61 SHR"}="1H6  SHR"; $resn_exch{" H32 SHR"}="2H3  SHR";
$resn_exch{" H31 SHR"}="1H3  SHR"; $resn_exch{" H22 SHR"}="2H2  SHR"; $resn_exch{" H21 SHR"}="1H2  SHR";
$resn_exch{"HO2S SHT"}="SHO2 SHT"; $resn_exch{"HCD1 SHT"}="1HCD SHT"; $resn_exch{"HCD2 SHT"}="2HCD SHT";
$resn_exch{"HCC1 SHT"}="1HCC SHT"; $resn_exch{"HCC2 SHT"}="2HCC SHT"; $resn_exch{"HK61 SHT"}="1HK6 SHT";
$resn_exch{"HK62 SHT"}="2HK6 SHT"; $resn_exch{"HK51 SHT"}="1HK5 SHT"; $resn_exch{"HK52 SHT"}="2HK5 SHT";
$resn_exch{"HK41 SHT"}="1HK4 SHT"; $resn_exch{"HK42 SHT"}="2HK4 SHT"; $resn_exch{"HK31 SHT"}="1HK3 SHT";
$resn_exch{"HK32 SHT"}="2HK3 SHT"; $resn_exch{"HC2K SHT"}="KHC2 SHT"; $resn_exch{"HC1K SHT"}="KHC1 SHT";
$resn_exch{" HCA SHT"}=" AHC SHT"; $resn_exch{" HCB SHT"}=" BHC SHT"; $resn_exch{"HG21 SHT"}="1HG2 SHT";
$resn_exch{"HG22 SHT"}="2HG2 SHT"; $resn_exch{"HG23 SHT"}="3HG2 SHT"; $resn_exch{"HOP2 SHT"}="2HOP SHT";
$resn_exch{"HOP8 SHT"}="8HOP SHT"; $resn_exch{"HC51 SHU"}="1HC5 SHU"; $resn_exch{"HC52 SHU"}="2HC5 SHU";
$resn_exch{" HC1 SHU"}="1HC  SHU"; $resn_exch{" HC2 SHU"}="2HC  SHU"; $resn_exch{" HC3 SHU"}="3HC  SHU";
$resn_exch{"HC31 SHU"}="1HC3 SHU"; $resn_exch{"HC32 SHU"}="2HC3 SHU"; $resn_exch{"HC21 SHU"}="1HC2 SHU";
$resn_exch{"HC22 SHU"}="2HC2 SHU"; $resn_exch{" H21 SHV"}="1H2  SHV"; $resn_exch{" H22 SHV"}="2H2  SHV";
$resn_exch{" H31 SHV"}="1H3  SHV"; $resn_exch{" H32 SHV"}="2H3  SHV"; $resn_exch{" H41 SHV"}="1H4  SHV";
$resn_exch{" H42 SHV"}="2H4  SHV"; $resn_exch{" H51 SHV"}="1H5  SHV"; $resn_exch{" H52 SHV"}="2H5  SHV";
$resn_exch{" H61 SHV"}="1H6  SHV"; $resn_exch{" H62 SHV"}="2H6  SHV"; $resn_exch{" H71 SHV"}="1H7  SHV";
$resn_exch{" H72 SHV"}="2H7  SHV"; $resn_exch{" H73 SHV"}="3H7  SHV"; $resn_exch{"H2N1 SHY"}="1H2N SHY";
$resn_exch{"H2N2 SHY"}="2H2N SHY"; $resn_exch{"H121 SHY"}="1H12 SHY"; $resn_exch{"H122 SHY"}="2H12 SHY";
$resn_exch{"H141 SHY"}="1H14 SHY"; $resn_exch{"H142 SHY"}="2H14 SHY"; $resn_exch{"H231 SHY"}="1H23 SHY";
$resn_exch{"H232 SHY"}="2H23 SHY"; $resn_exch{"H281 SHY"}="1H28 SHY"; $resn_exch{"H282 SHY"}="2H28 SHY";
$resn_exch{" H31 SI2"}="1H3  SI2"; $resn_exch{" H32 SI2"}="2H3  SI2"; $resn_exch{" H91 SI2"}="1H9  SI2";
$resn_exch{" H92 SI2"}="2H9  SI2"; $resn_exch{"H111 SI2"}="1H11 SI2"; $resn_exch{"H112 SI2"}="2H11 SI2";
$resn_exch{"H113 SI2"}="3H11 SI2"; $resn_exch{"HOB1 SI2"}="1HOB SI2"; $resn_exch{" H31 SIA"}="1H3  SIA";
$resn_exch{" H32 SIA"}="2H3  SIA"; $resn_exch{" H91 SIA"}="1H9  SIA"; $resn_exch{" H92 SIA"}="2H9  SIA";
$resn_exch{"H111 SIA"}="1H11 SIA"; $resn_exch{"H112 SIA"}="2H11 SIA"; $resn_exch{"H113 SIA"}="3H11 SIA";
$resn_exch{"HOB1 SIA"}="1HOB SIA"; $resn_exch{" H   SIB"}="1HN  SIB"; $resn_exch{" H2  SIB"}="2HN  SIB";
$resn_exch{" HB2 SIB"}="1HB  SIB"; $resn_exch{" HB3 SIB"}="2HB  SIB"; $resn_exch{" HG2 SIB"}="1HG  SIB";
$resn_exch{" HG3 SIB"}="2HG  SIB"; $resn_exch{"H5'1 SIB"}="1H5* SIB"; $resn_exch{"H5'2 SIB"}="2H5* SIB";
$resn_exch{" H4' SIB"}=" H4* SIB"; $resn_exch{" H3' SIB"}=" H3* SIB"; $resn_exch{"HO3' SIB"}="HO3* SIB";
$resn_exch{" H2' SIB"}=" H2* SIB"; $resn_exch{"HO2' SIB"}="HO2* SIB"; $resn_exch{" H1' SIB"}=" H1* SIB";
$resn_exch{" HC2 SIB"}=" H2  SIB"; $resn_exch{"HS01 SIE"}="1HS0 SIE"; $resn_exch{"HS02 SIE"}="2HS0 SIE";
$resn_exch{"HP11 SIE"}="1HP1 SIE"; $resn_exch{"HP12 SIE"}="2HP1 SIE"; $resn_exch{"HP21 SIE"}="1HP2 SIE";
$resn_exch{"HP22 SIE"}="2HP2 SIE"; $resn_exch{"HP31 SIE"}="1HP3 SIE"; $resn_exch{"HP32 SIE"}="2HP3 SIE";
$resn_exch{"HP41 SIE"}="1HP4 SIE"; $resn_exch{"HP42 SIE"}="2HP4 SIE"; $resn_exch{"HP51 SIE"}="1HP5 SIE";
$resn_exch{"HP52 SIE"}="2HP5 SIE"; $resn_exch{"HP61 SIE"}="1HP6 SIE"; $resn_exch{"HP62 SIE"}="2HP6 SIE";
$resn_exch{"HP71 SIE"}="1HP7 SIE"; $resn_exch{"HP72 SIE"}="2HP7 SIE"; $resn_exch{"HP81 SIE"}="1HP8 SIE";
$resn_exch{"HP82 SIE"}="2HP8 SIE"; $resn_exch{"HP91 SIE"}="1HP9 SIE"; $resn_exch{"HP92 SIE"}="2HP9 SIE";
$resn_exch{"HP01 SIE"}="1HP0 SIE"; $resn_exch{"HP02 SIE"}="2HP0 SIE"; $resn_exch{"HP03 SIE"}="3HP0 SIE";
$resn_exch{"H201 SIE"}="1H20 SIE"; $resn_exch{"H202 SIE"}="2H20 SIE"; $resn_exch{" H21 SIF"}="1H2  SIF";
$resn_exch{" H22 SIF"}="2H2  SIF"; $resn_exch{" H51 SIF"}="1H5  SIF"; $resn_exch{" H52 SIF"}="2H5  SIF";
$resn_exch{" H53 SIF"}="3H5  SIF"; $resn_exch{" H61 SIF"}="1H6  SIF"; $resn_exch{" H62 SIF"}="2H6  SIF";
$resn_exch{" H63 SIF"}="3H6  SIF"; $resn_exch{" H71 SIF"}="1H7  SIF"; $resn_exch{" H72 SIF"}="2H7  SIF";
$resn_exch{" H73 SIF"}="3H7  SIF"; $resn_exch{"H161 SIG"}="1H16 SIG"; $resn_exch{"H162 SIG"}="2H16 SIG";
$resn_exch{"H163 SIG"}="3H16 SIG"; $resn_exch{"H201 SIG"}="1H20 SIG"; $resn_exch{"H202 SIG"}="2H20 SIG";
$resn_exch{"H211 SIG"}="1H21 SIG"; $resn_exch{"H212 SIG"}="2H21 SIG"; $resn_exch{"H261 SIG"}="1H26 SIG";
$resn_exch{"H262 SIG"}="2H26 SIG"; $resn_exch{"H263 SIG"}="3H26 SIG"; $resn_exch{"H271 SIG"}="1H27 SIG";
$resn_exch{"H272 SIG"}="2H27 SIG"; $resn_exch{"H273 SIG"}="3H27 SIG"; $resn_exch{"H281 SIG"}="1H28 SIG";
$resn_exch{"H282 SIG"}="2H28 SIG"; $resn_exch{"H283 SIG"}="3H28 SIG"; $resn_exch{"H331 SIG"}="1H33 SIG";
$resn_exch{"H332 SIG"}="2H33 SIG"; $resn_exch{"H333 SIG"}="3H33 SIG"; $resn_exch{"H341 SIG"}="1H34 SIG";
$resn_exch{"H342 SIG"}="2H34 SIG"; $resn_exch{"H343 SIG"}="3H34 SIG"; $resn_exch{"H351 SIG"}="1H35 SIG";
$resn_exch{"H352 SIG"}="2H35 SIG"; $resn_exch{"H353 SIG"}="3H35 SIG"; $resn_exch{"H381 SIG"}="1H38 SIG";
$resn_exch{"H382 SIG"}="2H38 SIG"; $resn_exch{"H383 SIG"}="3H38 SIG"; $resn_exch{"H391 SIG"}="1H39 SIG";
$resn_exch{"H392 SIG"}="2H39 SIG"; $resn_exch{"H393 SIG"}="3H39 SIG"; $resn_exch{"H11A SIH"}="1H1  SIH";
$resn_exch{" H12 SIH"}="2H1  SIH"; $resn_exch{" H21 SIH"}="1H2  SIH"; $resn_exch{" H22 SIH"}="2H2  SIH";
$resn_exch{" H41 SIH"}="1H4  SIH"; $resn_exch{" H61 SIH"}="1H6  SIH"; $resn_exch{" H62 SIH"}="2H6  SIH";
$resn_exch{" H71 SIH"}="1H7  SIH"; $resn_exch{" H72 SIH"}="2H7  SIH"; $resn_exch{"H121 SIH"}="1H12 SIH";
$resn_exch{"H122 SIH"}="2H12 SIH"; $resn_exch{"H151 SIH"}="1H15 SIH"; $resn_exch{"H152 SIH"}="2H15 SIH";
$resn_exch{"H161 SIH"}="1H16 SIH"; $resn_exch{"H162 SIH"}="2H16 SIH"; $resn_exch{"H181 SIH"}="1H18 SIH";
$resn_exch{"H182 SIH"}="2H18 SIH"; $resn_exch{"H183 SIH"}="3H18 SIH"; $resn_exch{"H191 SIH"}="1H19 SIH";
$resn_exch{"H192 SIH"}="2H19 SIH"; $resn_exch{"H193 SIH"}="3H19 SIH"; $resn_exch{"H211 SIH"}="1H21 SIH";
$resn_exch{"H212 SIH"}="2H21 SIH"; $resn_exch{"H213 SIH"}="3H21 SIH"; $resn_exch{"HH21 SIH"}="1HH2 SIH";
$resn_exch{"HH22 SIH"}="2HH2 SIH"; $resn_exch{"HH31 SIH"}="1HH3 SIH"; $resn_exch{"HH32 SIH"}="2HH3 SIH";
$resn_exch{"HOH5 SIH"}="5HOH SIH"; $resn_exch{" H11 SIL"}="1H1  SIL"; $resn_exch{" H12 SIL"}="2H1  SIL";
$resn_exch{" H51 SIL"}="1H5  SIL"; $resn_exch{" H52 SIL"}="2H5  SIL"; $resn_exch{" H71 SIL"}="1H7  SIL";
$resn_exch{" H72 SIL"}="2H7  SIL"; $resn_exch{"H101 SIL"}="1H10 SIL"; $resn_exch{"H102 SIL"}="2H10 SIL";
$resn_exch{"H131 SIL"}="1H13 SIL"; $resn_exch{"H132 SIL"}="2H13 SIL"; $resn_exch{"H231 SIL"}="1H23 SIL";
$resn_exch{"H232 SIL"}="2H23 SIL"; $resn_exch{"H271 SIL"}="1H27 SIL"; $resn_exch{"H272 SIL"}="2H27 SIL";
$resn_exch{"H341 SIL"}="1H34 SIL"; $resn_exch{"H342 SIL"}="2H34 SIL"; $resn_exch{"H343 SIL"}="3H34 SIL";
$resn_exch{"H381 SIL"}="1H38 SIL"; $resn_exch{"H382 SIL"}="2H38 SIL"; $resn_exch{"H383 SIL"}="3H38 SIL";
$resn_exch{" H21 SIM"}="1H2  SIM"; $resn_exch{" H22 SIM"}="2H2  SIM"; $resn_exch{" H41 SIM"}="1H4  SIM";
$resn_exch{" H42 SIM"}="2H4  SIM"; $resn_exch{" H61 SIM"}="1H6  SIM"; $resn_exch{" H62 SIM"}="2H6  SIM";
$resn_exch{" H71 SIM"}="1H7  SIM"; $resn_exch{" H72 SIM"}="2H7  SIM"; $resn_exch{"H151 SIM"}="1H15 SIM";
$resn_exch{"H152 SIM"}="2H15 SIM"; $resn_exch{"H9A1 SIM"}="1H9A SIM"; $resn_exch{"H9A2 SIM"}="2H9A SIM";
$resn_exch{"H9A3 SIM"}="3H9A SIM"; $resn_exch{"H201 SIM"}="1H20 SIM"; $resn_exch{"H202 SIM"}="2H20 SIM";
$resn_exch{"H211 SIM"}="1H21 SIM"; $resn_exch{"H212 SIM"}="2H21 SIM"; $resn_exch{"H213 SIM"}="3H21 SIM";
$resn_exch{"H221 SIM"}="1H22 SIM"; $resn_exch{"H222 SIM"}="2H22 SIM"; $resn_exch{"H223 SIM"}="3H22 SIM";
$resn_exch{"H231 SIM"}="1H23 SIM"; $resn_exch{"H232 SIM"}="2H23 SIM"; $resn_exch{"H233 SIM"}="3H23 SIM";
$resn_exch{"H241 SIM"}="1H24 SIM"; $resn_exch{"H242 SIM"}="2H24 SIM"; $resn_exch{"H243 SIM"}="3H24 SIM";
$resn_exch{" H21 SIN"}="1H2  SIN"; $resn_exch{" H22 SIN"}="2H2  SIN"; $resn_exch{" H31 SIN"}="1H3  SIN";
$resn_exch{" H32 SIN"}="2H3  SIN"; $resn_exch{"H121 SK2"}="1H12 SK2"; $resn_exch{"H122 SK2"}="2H12 SK2";
$resn_exch{" H61 SK2"}="1H6  SK2"; $resn_exch{" H62 SK2"}="2H6  SK2"; $resn_exch{" H63 SK2"}="3H6  SK2";
$resn_exch{" H31 SK2"}="1H3  SK2"; $resn_exch{" H32 SK2"}="2H3  SK2"; $resn_exch{" H81 SK2"}="1H8  SK2";
$resn_exch{" H82 SK2"}="2H8  SK2"; $resn_exch{" H61 SK3"}="1H6  SK3"; $resn_exch{" H62 SK3"}="2H6  SK3";
$resn_exch{"H111 SKA"}="1H11 SKA"; $resn_exch{"H112 SKA"}="2H11 SKA"; $resn_exch{" H91 SKA"}="1H9  SKA";
$resn_exch{" H92 SKA"}="2H9  SKA"; $resn_exch{" H81 SKA"}="1H8  SKA"; $resn_exch{" H82 SKA"}="2H8  SKA";
$resn_exch{" H31 SKD"}="1H3  SKD"; $resn_exch{" H32 SKD"}="2H3  SKD"; $resn_exch{" H91 SKD"}="1H9  SKD";
$resn_exch{" H92 SKD"}="2H9  SKD"; $resn_exch{"H111 SKD"}="1H11 SKD"; $resn_exch{"H112 SKD"}="2H11 SKD";
$resn_exch{"H113 SKD"}="3H11 SKD"; $resn_exch{"HOB1 SKD"}="1HOB SKD"; $resn_exch{"HC11 SKF"}="1HC1 SKF";
$resn_exch{"HC12 SKF"}="2HC1 SKF"; $resn_exch{"HC21 SKF"}="1HC2 SKF"; $resn_exch{"HC22 SKF"}="2HC2 SKF";
$resn_exch{"HC31 SKF"}="1HC3 SKF"; $resn_exch{"HC32 SKF"}="2HC3 SKF"; $resn_exch{" H51 SKM"}="1H5  SKM";
$resn_exch{" H52 SKM"}="2H5  SKM"; $resn_exch{" H61 SKP"}="1H6  SKP"; $resn_exch{" H62 SKP"}="2H6  SKP";
$resn_exch{" H91 SKP"}="1H9  SKP"; $resn_exch{" H92 SKP"}="2H9  SKP"; $resn_exch{" H93 SKP"}="3H9  SKP";
$resn_exch{"H3C1 SL1"}="1H3C SL1"; $resn_exch{"H3C2 SL1"}="2H3C SL1"; $resn_exch{"H201 SL1"}="1H20 SL1";
$resn_exch{"H202 SL1"}="2H20 SL1"; $resn_exch{"H211 SL1"}="1H21 SL1"; $resn_exch{"H212 SL1"}="2H21 SL1";
$resn_exch{"H221 SL1"}="1H22 SL1"; $resn_exch{"H222 SL1"}="2H22 SL1"; $resn_exch{"H6N1 SL1"}="1H6N SL1";
$resn_exch{"H6N2 SL1"}="2H6N SL1"; $resn_exch{"H241 SL1"}="1H24 SL1"; $resn_exch{"H242 SL1"}="2H24 SL1";
$resn_exch{" H31 SLB"}="1H3  SLB"; $resn_exch{" H32 SLB"}="2H3  SLB"; $resn_exch{" H91 SLB"}="1H9  SLB";
$resn_exch{" H92 SLB"}="2H9  SLB"; $resn_exch{"H111 SLB"}="1H11 SLB"; $resn_exch{"H112 SLB"}="2H11 SLB";
$resn_exch{"H113 SLB"}="3H11 SLB"; $resn_exch{"HOB1 SLB"}="1HOB SLB"; $resn_exch{" HH1 SLE"}="1HH  SLE";
$resn_exch{" HH2 SLE"}="2HH  SLE"; $resn_exch{" H31 SLE"}="1H3  SLE"; $resn_exch{" H32 SLE"}="2H3  SLE";
$resn_exch{" HM1 SLE"}="1HM  SLE"; $resn_exch{" HM2 SLE"}="2HM  SLE"; $resn_exch{" HM3 SLE"}="3HM  SLE";
$resn_exch{" H51 SLE"}="1H5  SLE"; $resn_exch{" H52 SLE"}="2H5  SLE"; $resn_exch{" H53 SLE"}="3H5  SLE";
$resn_exch{"H351 SLF"}="1H35 SLF"; $resn_exch{"H352 SLF"}="2H35 SLF"; $resn_exch{"H353 SLF"}="3H35 SLF";
$resn_exch{"H341 SLF"}="1H34 SLF"; $resn_exch{"H342 SLF"}="2H34 SLF"; $resn_exch{"H331 SLF"}="1H33 SLF";
$resn_exch{"H332 SLF"}="2H33 SLF"; $resn_exch{"H321 SLF"}="1H32 SLF"; $resn_exch{"H322 SLF"}="2H32 SLF";
$resn_exch{"H311 SLF"}="1H31 SLF"; $resn_exch{"H312 SLF"}="2H31 SLF"; $resn_exch{"H301 SLF"}="1H30 SLF";
$resn_exch{"H302 SLF"}="2H30 SLF"; $resn_exch{"H291 SLF"}="1H29 SLF"; $resn_exch{"H292 SLF"}="2H29 SLF";
$resn_exch{"H281 SLF"}="1H28 SLF"; $resn_exch{"H282 SLF"}="2H28 SLF"; $resn_exch{"H271 SLF"}="1H27 SLF";
$resn_exch{"H272 SLF"}="2H27 SLF"; $resn_exch{"H261 SLF"}="1H26 SLF"; $resn_exch{"H262 SLF"}="2H26 SLF";
$resn_exch{"H251 SLF"}="1H25 SLF"; $resn_exch{"H252 SLF"}="2H25 SLF"; $resn_exch{"H241 SLF"}="1H24 SLF";
$resn_exch{"H242 SLF"}="2H24 SLF"; $resn_exch{"H231 SLF"}="1H23 SLF"; $resn_exch{"H232 SLF"}="2H23 SLF";
$resn_exch{"H221 SLF"}="1H22 SLF"; $resn_exch{"H222 SLF"}="2H22 SLF"; $resn_exch{"H211 SLF"}="1H21 SLF";
$resn_exch{"H212 SLF"}="2H21 SLF"; $resn_exch{"H201 SLF"}="1H20 SLF"; $resn_exch{"H202 SLF"}="2H20 SLF";
$resn_exch{"H191 SLF"}="1H19 SLF"; $resn_exch{"H192 SLF"}="2H19 SLF"; $resn_exch{" H1A SLF"}="1H   SLF";
$resn_exch{" H2A SLF"}="2H   SLF"; $resn_exch{"H411 SLF"}="1H41 SLF"; $resn_exch{"H412 SLF"}="2H41 SLF";
$resn_exch{" H51 SLF"}="1H5  SLF"; $resn_exch{" H52 SLF"}="2H5  SLF"; $resn_exch{" H61 SLF"}="1H6  SLF";
$resn_exch{" H62 SLF"}="2H6  SLF"; $resn_exch{" H71 SLF"}="1H7  SLF"; $resn_exch{" H72 SLF"}="2H7  SLF";
$resn_exch{" H81 SLF"}="1H8  SLF"; $resn_exch{" H82 SLF"}="2H8  SLF"; $resn_exch{" H91 SLF"}="1H9  SLF";
$resn_exch{" H92 SLF"}="2H9  SLF"; $resn_exch{"H101 SLF"}="1H10 SLF"; $resn_exch{"H102 SLF"}="2H10 SLF";
$resn_exch{"H111 SLF"}="1H11 SLF"; $resn_exch{"H112 SLF"}="2H11 SLF"; $resn_exch{"H121 SLF"}="1H12 SLF";
$resn_exch{"H122 SLF"}="2H12 SLF"; $resn_exch{"H131 SLF"}="1H13 SLF"; $resn_exch{"H132 SLF"}="2H13 SLF";
$resn_exch{"H141 SLF"}="1H14 SLF"; $resn_exch{"H142 SLF"}="2H14 SLF"; $resn_exch{"H151 SLF"}="1H15 SLF";
$resn_exch{"H152 SLF"}="2H15 SLF"; $resn_exch{"H161 SLF"}="1H16 SLF"; $resn_exch{"H162 SLF"}="2H16 SLF";
$resn_exch{"H171 SLF"}="1H17 SLF"; $resn_exch{"H172 SLF"}="2H17 SLF"; $resn_exch{"H173 SLF"}="3H17 SLF";
$resn_exch{"H181 SLT"}="1H18 SLT"; $resn_exch{"H182 SLT"}="2H18 SLT"; $resn_exch{"H183 SLT"}="3H18 SLT";
$resn_exch{"H161 SLT"}="1H16 SLT"; $resn_exch{"H162 SLT"}="2H16 SLT"; $resn_exch{"H101 SLT"}="1H10 SLT";
$resn_exch{"H102 SLT"}="2H10 SLT"; $resn_exch{" H61 SLT"}="1H6  SLT"; $resn_exch{" H62 SLT"}="2H6  SLT";
$resn_exch{"H6A1 SLT"}="1H6A SLT"; $resn_exch{"H6A2 SLT"}="2H6A SLT"; $resn_exch{"H211 SLU"}="1H21 SLU";
$resn_exch{"H212 SLU"}="2H21 SLU"; $resn_exch{"H381 SLU"}="1H38 SLU"; $resn_exch{"H382 SLU"}="2H38 SLU";
$resn_exch{" H   SLZ"}="1HN  SLZ"; $resn_exch{" H2  SLZ"}="2HN  SLZ"; $resn_exch{" HB2 SLZ"}="1HB  SLZ";
$resn_exch{" HB3 SLZ"}="2HB  SLZ"; $resn_exch{" HD2 SLZ"}="1HD  SLZ"; $resn_exch{" HD3 SLZ"}="2HD  SLZ";
$resn_exch{" HE2 SLZ"}="1HE  SLZ"; $resn_exch{" HE3 SLZ"}="2HE  SLZ"; $resn_exch{" HZ1 SLZ"}="1HZ  SLZ";
$resn_exch{" HZ2 SLZ"}="2HZ  SLZ"; $resn_exch{"H6N1 SM1"}="1H6N SM1"; $resn_exch{"H6N2 SM1"}="2H6N SM1";
$resn_exch{"H111 SM1"}="1H11 SM1"; $resn_exch{"H112 SM1"}="2H11 SM1"; $resn_exch{"H221 SM1"}="1H22 SM1";
$resn_exch{"H222 SM1"}="2H22 SM1"; $resn_exch{"H223 SM1"}="3H22 SM1"; $resn_exch{"H6C1 SM1"}="1H6C SM1";
$resn_exch{"H6C2 SM1"}="2H6C SM1"; $resn_exch{"H1C1 SM1"}="1H1C SM1"; $resn_exch{"H1C2 SM1"}="2H1C SM1";
$resn_exch{"HAG1 SM2"}="1HAG SM2"; $resn_exch{"HAG2 SM2"}="2HAG SM2"; $resn_exch{"HAG1 SM3"}="1HAG SM3";
$resn_exch{"HAG2 SM3"}="2HAG SM3"; $resn_exch{"HAG1 SM4"}="1HAG SM4"; $resn_exch{"HAG2 SM4"}="2HAG SM4";
$resn_exch{"H3M1 SMA"}="1H3M SMA"; $resn_exch{"H3M2 SMA"}="2H3M SMA"; $resn_exch{"H3M3 SMA"}="3H3M SMA";
$resn_exch{"H5M1 SMA"}="1H5M SMA"; $resn_exch{"H5M2 SMA"}="2H5M SMA"; $resn_exch{"H5M3 SMA"}="3H5M SMA";
$resn_exch{"H7M1 SMA"}="1H7M SMA"; $resn_exch{"H7M2 SMA"}="2H7M SMA"; $resn_exch{"H7M3 SMA"}="3H7M SMA";
$resn_exch{" H91 SMA"}="1H9  SMA"; $resn_exch{" H92 SMA"}="2H9  SMA"; $resn_exch{"H101 SMA"}="1H10 SMA";
$resn_exch{"H102 SMA"}="2H10 SMA"; $resn_exch{"H211 SMA"}="1H21 SMA"; $resn_exch{"H212 SMA"}="2H21 SMA";
$resn_exch{"H213 SMA"}="3H21 SMA"; $resn_exch{"H221 SMA"}="1H22 SMA"; $resn_exch{"H222 SMA"}="2H22 SMA";
$resn_exch{"H223 SMA"}="3H22 SMA"; $resn_exch{"H231 SMA"}="1H23 SMA"; $resn_exch{"H232 SMA"}="2H23 SMA";
$resn_exch{"H233 SMA"}="3H23 SMA"; $resn_exch{"H241 SMA"}="1H24 SMA"; $resn_exch{"H242 SMA"}="2H24 SMA";
$resn_exch{"H243 SMA"}="3H24 SMA"; $resn_exch{"H251 SMA"}="1H25 SMA"; $resn_exch{"H252 SMA"}="2H25 SMA";
$resn_exch{"H253 SMA"}="3H25 SMA"; $resn_exch{"H261 SMA"}="1H26 SMA"; $resn_exch{"H262 SMA"}="2H26 SMA";
$resn_exch{"H263 SMA"}="3H26 SMA"; $resn_exch{" H11 SMB"}="1H1  SMB"; $resn_exch{" H12 SMB"}="2H1  SMB";
$resn_exch{" H13 SMB"}="3H1  SMB"; $resn_exch{" H21 SMB"}="1H2  SMB"; $resn_exch{" H22 SMB"}="2H2  SMB";
$resn_exch{" H41 SMB"}="1H4  SMB"; $resn_exch{" H42 SMB"}="2H4  SMB"; $resn_exch{" H43 SMB"}="3H4  SMB";
$resn_exch{" H   SMC"}="1HN  SMC"; $resn_exch{" H2  SMC"}="2HN  SMC"; $resn_exch{" HB2 SMC"}="1HB  SMC";
$resn_exch{" HB3 SMC"}="2HB  SMC"; $resn_exch{"HCS1 SMC"}="1HCS SMC"; $resn_exch{"HCS2 SMC"}="2HCS SMC";
$resn_exch{"HCS3 SMC"}="3HCS SMC"; $resn_exch{"H161 SMD"}="1H16 SMD"; $resn_exch{"H162 SMD"}="2H16 SMD";
$resn_exch{"H271 SMD"}="1H27 SMD"; $resn_exch{"H272 SMD"}="2H27 SMD"; $resn_exch{"H273 SMD"}="3H27 SMD";
$resn_exch{"H261 SMD"}="1H26 SMD"; $resn_exch{"H262 SMD"}="2H26 SMD"; $resn_exch{" H   SME"}="1HN  SME";
$resn_exch{" H2  SME"}="2HN  SME"; $resn_exch{" HB2 SME"}="1HB  SME"; $resn_exch{" HB3 SME"}="2HB  SME";
$resn_exch{" HG2 SME"}="1HG  SME"; $resn_exch{" HG3 SME"}="2HG  SME"; $resn_exch{" HE1 SME"}="1HE  SME";
$resn_exch{" HE2 SME"}="2HE  SME"; $resn_exch{" HE3 SME"}="3HE  SME"; $resn_exch{" H2  SMF"}=" HN2 SMF";
$resn_exch{" HB2 SMF"}="1HB  SMF"; $resn_exch{" HB3 SMF"}="2HB  SMF"; $resn_exch{" HH2 SMF"}="1HH  SMF";
$resn_exch{" HH3 SMF"}="2HH  SMF"; $resn_exch{" H51 SMG"}="1H5  SMG"; $resn_exch{" H52 SMG"}="2H5  SMG";
$resn_exch{" H41 SMG"}="1H4  SMG"; $resn_exch{" H42 SMG"}="2H4  SMG"; $resn_exch{" HB1 SMG"}="1HB  SMG";
$resn_exch{" HB2 SMG"}="2HB  SMG"; $resn_exch{" HG1 SMG"}="1HG  SMG"; $resn_exch{" HG2 SMG"}="2HG  SMG";
$resn_exch{" HE1 SMG"}="1HE  SMG"; $resn_exch{" HE2 SMG"}="2HE  SMG"; $resn_exch{" HE3 SMG"}="3HE  SMG";
$resn_exch{"H321 SML"}="1H32 SML"; $resn_exch{"H322 SML"}="2H32 SML"; $resn_exch{"H323 SML"}="3H32 SML";
$resn_exch{"H371 SML"}="1H37 SML"; $resn_exch{"H381 SML"}="1H38 SML"; $resn_exch{"H391 SML"}="1H39 SML";
$resn_exch{"H401 SML"}="1H40 SML"; $resn_exch{"H411 SML"}="1H41 SML"; $resn_exch{" HN1 SMM"}="1HN  SMM";
$resn_exch{" HN2 SMM"}="2HN  SMM"; $resn_exch{" HA1 SMM"}="1HA  SMM"; $resn_exch{" HB1 SMM"}="1HB  SMM";
$resn_exch{" HB2 SMM"}="2HB  SMM"; $resn_exch{" HG1 SMM"}="1HG  SMM"; $resn_exch{" HG2 SMM"}="2HG  SMM";
$resn_exch{" HD1 SMM"}="1HD  SMM"; $resn_exch{" HE1 SMM"}="1HE  SMM"; $resn_exch{" HE2 SMM"}="2HE  SMM";
$resn_exch{" HE3 SMM"}="3HE  SMM"; $resn_exch{"H5'1 SMM"}="1H5* SMM"; $resn_exch{"H5'2 SMM"}="2H5* SMM";
$resn_exch{" H4' SMM"}=" H4* SMM"; $resn_exch{" H1' SMM"}=" H1* SMM"; $resn_exch{"HN61 SMM"}="1HN6 SMM";
$resn_exch{"HN62 SMM"}="2HN6 SMM"; $resn_exch{" H2' SMM"}=" H2* SMM"; $resn_exch{"HO2' SMM"}="*HO2 SMM";
$resn_exch{" H3' SMM"}=" H3* SMM"; $resn_exch{"HO3' SMM"}="*HO3 SMM"; $resn_exch{"HXT1 SMM"}="1HXT SMM";
$resn_exch{"HXT2 SMM"}="2HXT SMM"; $resn_exch{"HXT3 SMM"}="3HXT SMM"; $resn_exch{"HMP1 SMP"}="1HMP SMP";
$resn_exch{"HMP2 SMP"}="2HMP SMP"; $resn_exch{"HMP3 SMP"}="3HMP SMP"; $resn_exch{" H5' SMP"}="1H5* SMP";
$resn_exch{"H5'' SMP"}="2H5* SMP"; $resn_exch{" H4' SMP"}=" H4* SMP"; $resn_exch{" H1' SMP"}=" H1* SMP";
$resn_exch{" H61 SMP"}="1H6  SMP"; $resn_exch{" H62 SMP"}="2H6  SMP"; $resn_exch{" H3' SMP"}=" H3* SMP";
$resn_exch{" H2' SMP"}="1H2* SMP"; $resn_exch{"H2'' SMP"}="2H2* SMP"; $resn_exch{"HO3' SMP"}="H3T  SMP";
$resn_exch{"HOP3 SMP"}="3HOP SMP"; $resn_exch{"HN11 SMS"}="1HN1 SMS"; $resn_exch{"HN12 SMS"}="2HN1 SMS";
$resn_exch{"HC11 SMS"}="1HC1 SMS"; $resn_exch{"HC12 SMS"}="2HC1 SMS"; $resn_exch{"HC61 SMS"}="1HC6 SMS";
$resn_exch{"HC62 SMS"}="2HC6 SMS"; $resn_exch{"HC91 SMS"}="1HC9 SMS"; $resn_exch{"HC92 SMS"}="2HC9 SMS";
$resn_exch{"HC93 SMS"}="3HC9 SMS"; $resn_exch{"HC81 SMS"}="1HC8 SMS"; $resn_exch{"HC82 SMS"}="2HC8 SMS";
$resn_exch{"HC83 SMS"}="3HC8 SMS"; $resn_exch{"H5'2 SMT"}="2H5' SMT"; $resn_exch{"H5A1 SMT"}="1H5A SMT";
$resn_exch{"H5A2 SMT"}="2H5A SMT"; $resn_exch{" HA1 SMT"}="1HA  SMT"; $resn_exch{" HA2 SMT"}="2HA  SMT";
$resn_exch{" HB1 SMT"}="1HB  SMT"; $resn_exch{" HB2 SMT"}="2HB  SMT"; $resn_exch{" HD1 SMT"}="1HD  SMT";
$resn_exch{" HD2 SMT"}="2HD  SMT"; $resn_exch{" HD3 SMT"}="3HD  SMT"; $resn_exch{" HB1 SN0"}="1HB  SN0";
$resn_exch{" HB2 SN0"}="2HB  SN0"; $resn_exch{" HG1 SN0"}="1HG  SN0"; $resn_exch{" HG2 SN0"}="2HG  SN0";
$resn_exch{"HD1A SN0"}="1HD  SN0"; $resn_exch{" HD2 SN0"}="2HD  SN0"; $resn_exch{" HD3 SN0"}="3HD  SN0";
$resn_exch{" H21 SN0"}="1H2  SN0"; $resn_exch{" H22 SN0"}="2H2  SN0"; $resn_exch{" H31 SN0"}="1H3  SN0";
$resn_exch{" H32 SN0"}="2H3  SN0"; $resn_exch{"HC71 SN2"}="1HC7 SN2"; $resn_exch{"HC72 SN2"}="2HC7 SN2";
$resn_exch{"HC81 SN2"}="1HC8 SN2"; $resn_exch{"HC82 SN2"}="2HC8 SN2"; $resn_exch{"HC91 SN2"}="1HC9 SN2";
$resn_exch{"HC92 SN2"}="2HC9 SN2"; $resn_exch{"H101 SN2"}="1H10 SN2"; $resn_exch{"H102 SN2"}="2H10 SN2";
$resn_exch{"HN41 SN2"}="1HN4 SN2"; $resn_exch{"HN42 SN2"}="2HN4 SN2"; $resn_exch{" H21 SN3"}="1H2  SN3";
$resn_exch{" H22 SN3"}="2H2  SN3"; $resn_exch{" H51 SN3"}="1H5  SN3"; $resn_exch{" H52 SN3"}="2H5  SN3";
$resn_exch{"H101 SN3"}="1H10 SN3"; $resn_exch{"H102 SN3"}="2H10 SN3"; $resn_exch{"H131 SN3"}="1H13 SN3";
$resn_exch{"H132 SN3"}="2H13 SN3"; $resn_exch{"H141 SN3"}="1H14 SN3"; $resn_exch{"H142 SN3"}="2H14 SN3";
$resn_exch{"H161 SN3"}="1H16 SN3"; $resn_exch{"H162 SN3"}="2H16 SN3"; $resn_exch{"H181 SN3"}="1H18 SN3";
$resn_exch{"H182 SN3"}="2H18 SN3"; $resn_exch{"H192 SN3"}="2H19 SN3"; $resn_exch{"H221 SN3"}="1H22 SN3";
$resn_exch{"H222 SN3"}="2H22 SN3"; $resn_exch{"H291 SN3"}="1H29 SN3"; $resn_exch{"H292 SN3"}="2H29 SN3";
$resn_exch{"H293 SN3"}="3H29 SN3"; $resn_exch{"H301 SN3"}="1H30 SN3"; $resn_exch{"H302 SN3"}="2H30 SN3";
$resn_exch{"H303 SN3"}="3H30 SN3"; $resn_exch{"H361 SN3"}="1H36 SN3"; $resn_exch{"H362 SN3"}="2H36 SN3";
$resn_exch{"H363 SN3"}="3H36 SN3"; $resn_exch{"H371 SN3"}="1H37 SN3"; $resn_exch{"H372 SN3"}="2H37 SN3";
$resn_exch{"H341 SN6"}="1H34 SN6"; $resn_exch{"H342 SN6"}="2H34 SN6"; $resn_exch{"H343 SN6"}="3H34 SN6";
$resn_exch{"H351 SN6"}="1H35 SN6"; $resn_exch{"H352 SN6"}="2H35 SN6"; $resn_exch{"H353 SN6"}="3H35 SN6";
$resn_exch{"H341 SN7"}="1H34 SN7"; $resn_exch{"H342 SN7"}="2H34 SN7"; $resn_exch{"H343 SN7"}="3H34 SN7";
$resn_exch{"H351 SN7"}="1H35 SN7"; $resn_exch{"H352 SN7"}="2H35 SN7"; $resn_exch{"H353 SN7"}="3H35 SN7";
$resn_exch{"H361 SN7"}="1H36 SN7"; $resn_exch{"H362 SN7"}="2H36 SN7"; $resn_exch{"H371 SN7"}="1H37 SN7";
$resn_exch{"H372 SN7"}="2H37 SN7"; $resn_exch{"HAA1 SN8"}="1HAA SN8"; $resn_exch{"HAA2 SN8"}="2HAA SN8";
$resn_exch{"HAA3 SN8"}="3HAA SN8"; $resn_exch{"HAC1 SN8"}="1HAC SN8"; $resn_exch{"HAC2 SN8"}="2HAC SN8";
$resn_exch{"HAC3 SN8"}="3HAC SN8"; $resn_exch{"HAT1 SN8"}="1HAT SN8"; $resn_exch{"HAJ1 SN8"}="1HAJ SN8";
$resn_exch{"HAL1 SN8"}="1HAL SN8"; $resn_exch{"HAI1 SN8"}="1HAI SN8"; $resn_exch{"HAK1 SN8"}="1HAK SN8";
$resn_exch{"HAF1 SN8"}="1HAF SN8"; $resn_exch{"HAH1 SN8"}="1HAH SN8"; $resn_exch{"HAE1 SN8"}="1HAE SN8";
$resn_exch{"HAG1 SN8"}="1HAG SN8"; $resn_exch{"HAN1 SN8"}="1HAN SN8"; $resn_exch{"HAR1 SN8"}="1HAR SN8";
$resn_exch{"HAB1 SN8"}="1HAB SN8"; $resn_exch{"HAB2 SN8"}="2HAB SN8"; $resn_exch{"HAB3 SN8"}="3HAB SN8";
$resn_exch{"HAS1 SN8"}="1HAS SN8"; $resn_exch{"HAO1 SN8"}="1HAO SN8"; $resn_exch{"HAA1 SN9"}="1HAA SN9";
$resn_exch{"HAA2 SN9"}="2HAA SN9"; $resn_exch{"HAA3 SN9"}="3HAA SN9"; $resn_exch{"HAC1 SN9"}="1HAC SN9";
$resn_exch{"HAC2 SN9"}="2HAC SN9"; $resn_exch{"HAC3 SN9"}="3HAC SN9"; $resn_exch{"HAU1 SN9"}="1HAU SN9";
$resn_exch{"HAL1 SN9"}="1HAL SN9"; $resn_exch{"HAN1 SN9"}="1HAN SN9"; $resn_exch{"HAK1 SN9"}="1HAK SN9";
$resn_exch{"HAM1 SN9"}="1HAM SN9"; $resn_exch{"HAH1 SN9"}="1HAH SN9"; $resn_exch{"HAJ1 SN9"}="1HAJ SN9";
$resn_exch{"HAG1 SN9"}="1HAG SN9"; $resn_exch{"HAI1 SN9"}="1HAI SN9"; $resn_exch{"HAO1 SN9"}="1HAO SN9";
$resn_exch{"HAS1 SN9"}="1HAS SN9"; $resn_exch{"HAB1 SN9"}="1HAB SN9"; $resn_exch{"HAB2 SN9"}="2HAB SN9";
$resn_exch{"HAB3 SN9"}="3HAB SN9"; $resn_exch{"HAT1 SN9"}="1HAT SN9"; $resn_exch{"HAP1 SN9"}="1HAP SN9";
$resn_exch{"H141 SNA"}="1H14 SNA"; $resn_exch{"H142 SNA"}="2H14 SNA"; $resn_exch{"H201 SNA"}="1H20 SNA";
$resn_exch{"H202 SNA"}="2H20 SNA"; $resn_exch{"H211 SNA"}="1H21 SNA"; $resn_exch{"H212 SNA"}="2H21 SNA";
$resn_exch{"H221 SNA"}="1H22 SNA"; $resn_exch{"H222 SNA"}="2H22 SNA"; $resn_exch{"H231 SNA"}="1H23 SNA";
$resn_exch{"H232 SNA"}="2H23 SNA"; $resn_exch{"H241 SNA"}="1H24 SNA"; $resn_exch{"H242 SNA"}="2H24 SNA";
$resn_exch{"H271 SNA"}="1H27 SNA"; $resn_exch{"H272 SNA"}="2H27 SNA"; $resn_exch{"H461 SNA"}="1H46 SNA";
$resn_exch{"H462 SNA"}="2H46 SNA"; $resn_exch{"H531 SNA"}="1H53 SNA"; $resn_exch{"H532 SNA"}="2H53 SNA";
$resn_exch{"H571 SNA"}="1H57 SNA"; $resn_exch{"H572 SNA"}="2H57 SNA"; $resn_exch{" H   SNC"}="1HN  SNC";
$resn_exch{" H2  SNC"}="2HN  SNC"; $resn_exch{" HB2 SNC"}="1HB  SNC"; $resn_exch{" HB3 SNC"}="2HB  SNC";
$resn_exch{"HOA2 SND"}="2HOA SND"; $resn_exch{"H51A SND"}="AH51 SND"; $resn_exch{"H52A SND"}="AH52 SND";
$resn_exch{" H4B SND"}="AH4* SND"; $resn_exch{" H3B SND"}="AH3* SND"; $resn_exch{"HO3A SND"}="AHO3 SND";
$resn_exch{" H2B SND"}="AH2* SND"; $resn_exch{"HO2A SND"}="AHO2 SND"; $resn_exch{" H1B SND"}="AH1* SND";
$resn_exch{" H8A SND"}=" AH8 SND"; $resn_exch{"H61A SND"}="AH61 SND"; $resn_exch{"H62A SND"}="AH62 SND";
$resn_exch{" H2A SND"}=" AH2 SND"; $resn_exch{"H51N SND"}="NH51 SND"; $resn_exch{"H52N SND"}="NH52 SND";
$resn_exch{" H4D SND"}="NH4* SND"; $resn_exch{" H3D SND"}="NH3* SND"; $resn_exch{"HO3N SND"}="NHO3 SND";
$resn_exch{" H2D SND"}="NH2* SND"; $resn_exch{"HO2N SND"}="NHO2 SND"; $resn_exch{" H1D SND"}="NH1* SND";
$resn_exch{" H2N SND"}=" NH2 SND"; $resn_exch{"H71N SND"}="NH71 SND"; $resn_exch{"H72N SND"}="NH72 SND";
$resn_exch{" H4N SND"}=" NH4 SND"; $resn_exch{" H5N SND"}=" NH5 SND"; $resn_exch{" H6N SND"}=" NH6 SND";
$resn_exch{"H6C1 SNG"}="1H6C SNG"; $resn_exch{"H6C2 SNG"}="2H6C SNG"; $resn_exch{"H8C1 SNG"}="1H8C SNG";
$resn_exch{"H8C2 SNG"}="2H8C SNG"; $resn_exch{"H8C3 SNG"}="3H8C SNG"; $resn_exch{"HMC1 SNG"}="1HMC SNG";
$resn_exch{"HMC2 SNG"}="2HMC SNG"; $resn_exch{"HMC3 SNG"}="3HMC SNG"; $resn_exch{"H201 SNH"}="1H20 SNH";
$resn_exch{"H202 SNH"}="2H20 SNH"; $resn_exch{"H203 SNH"}="3H20 SNH"; $resn_exch{"H211 SNH"}="1H21 SNH";
$resn_exch{"H212 SNH"}="2H21 SNH"; $resn_exch{"H213 SNH"}="3H21 SNH"; $resn_exch{"H281 SNH"}="1H28 SNH";
$resn_exch{"H282 SNH"}="2H28 SNH"; $resn_exch{"H283 SNH"}="3H28 SNH"; $resn_exch{" H5' SNI"}=" H5* SNI";
$resn_exch{"H5'1 SNI"}="1H5* SNI"; $resn_exch{"H5'2 SNI"}="2H5* SNI"; $resn_exch{" H4' SNI"}=" H4* SNI";
$resn_exch{" H1' SNI"}=" H1* SNI"; $resn_exch{" H2' SNI"}=" H2* SNI"; $resn_exch{" H3' SNI"}=" H3* SNI";
$resn_exch{" H81 SNL"}="1H8  SNL"; $resn_exch{" H82 SNL"}="2H8  SNL"; $resn_exch{" H41 SNL"}="1H4  SNL";
$resn_exch{" H42 SNL"}="2H4  SNL"; $resn_exch{" H11 SNL"}="1H1  SNL"; $resn_exch{" H12 SNL"}="2H1  SNL";
$resn_exch{" H31 SNL"}="1H3  SNL"; $resn_exch{" H32 SNL"}="2H3  SNL"; $resn_exch{" H71 SNL"}="1H7  SNL";
$resn_exch{" H72 SNL"}="2H7  SNL"; $resn_exch{"H101 SNL"}="1H10 SNL"; $resn_exch{"H102 SNL"}="2H10 SNL";
$resn_exch{"H103 SNL"}="3H10 SNL"; $resn_exch{"H111 SNL"}="1H11 SNL"; $resn_exch{"H112 SNL"}="2H11 SNL";
$resn_exch{"H113 SNL"}="3H11 SNL"; $resn_exch{"H121 SNL"}="1H12 SNL"; $resn_exch{"H122 SNL"}="2H12 SNL";
$resn_exch{"H131 SNL"}="1H13 SNL"; $resn_exch{"H132 SNL"}="2H13 SNL"; $resn_exch{"H141 SNL"}="1H14 SNL";
$resn_exch{"H142 SNL"}="2H14 SNL"; $resn_exch{"H171 SNL"}="1H17 SNL"; $resn_exch{"H172 SNL"}="2H17 SNL";
$resn_exch{"H173 SNL"}="3H17 SNL"; $resn_exch{"H211 SNL"}="1H21 SNL"; $resn_exch{"H212 SNL"}="2H21 SNL";
$resn_exch{"HN31 SNN"}="1HN3 SNN"; $resn_exch{"HN32 SNN"}="2HN3 SNN"; $resn_exch{" H41 SNN"}="1H4  SNN";
$resn_exch{" H42 SNN"}="2H4  SNN"; $resn_exch{" H22 SNO"}="2H2  SNO"; $resn_exch{" H33 SNO"}="3H3  SNO";
$resn_exch{" H55 SNO"}="5H5  SNO"; $resn_exch{" H66 SNO"}="6H6  SNO"; $resn_exch{" H77 SNO"}="7H7  SNO";
$resn_exch{" H81 SNO"}="1H8  SNO"; $resn_exch{" H82 SNO"}="2H8  SNO"; $resn_exch{"H112 SNP"}="2H11 SNP";
$resn_exch{"H111 SNP"}="1H11 SNP"; $resn_exch{"H2O1 SNP"}="1H2O SNP"; $resn_exch{"H132 SNP"}="2H13 SNP";
$resn_exch{"H131 SNP"}="1H13 SNP"; $resn_exch{"H153 SNP"}="3H15 SNP"; $resn_exch{"H152 SNP"}="2H15 SNP";
$resn_exch{"H151 SNP"}="1H15 SNP"; $resn_exch{"H163 SNP"}="3H16 SNP"; $resn_exch{"H162 SNP"}="2H16 SNP";
$resn_exch{"H161 SNP"}="1H16 SNP"; $resn_exch{" H11 SNR"}="1H1  SNR"; $resn_exch{" H21 SNR"}="1H2  SNR";
$resn_exch{" H12 SNR"}="2H1  SNR"; $resn_exch{" H22 SNR"}="2H2  SNR"; $resn_exch{" H13 SNR"}="3H1  SNR";
$resn_exch{" H23 SNR"}="3H2  SNR"; $resn_exch{"H121 SO1"}="1H12 SO1"; $resn_exch{"H122 SO1"}="2H12 SO1";
$resn_exch{"H101 SO1"}="1H10 SO1"; $resn_exch{"H102 SO1"}="2H10 SO1"; $resn_exch{"H181 SO1"}="1H18 SO1";
$resn_exch{"H182 SO1"}="2H18 SO1"; $resn_exch{"H241 SO1"}="1H24 SO1"; $resn_exch{"H242 SO1"}="2H24 SO1";
$resn_exch{"H251 SO1"}="1H25 SO1"; $resn_exch{"H252 SO1"}="2H25 SO1"; $resn_exch{"H253 SO1"}="3H25 SO1";
$resn_exch{" H71 SO1"}="1H7  SO1"; $resn_exch{" H72 SO1"}="2H7  SO1"; $resn_exch{" H81 SO1"}="1H8  SO1";
$resn_exch{" H82 SO1"}="2H8  SO1"; $resn_exch{"H611 SO1"}="1H61 SO1"; $resn_exch{"H612 SO1"}="2H61 SO1";
$resn_exch{"H613 SO1"}="3H61 SO1"; $resn_exch{"H651 SO1"}="1H65 SO1"; $resn_exch{"H652 SO1"}="2H65 SO1";
$resn_exch{"H653 SO1"}="3H65 SO1"; $resn_exch{"H201 SO1"}="1H20 SO1"; $resn_exch{"H202 SO1"}="2H20 SO1";
$resn_exch{"H203 SO1"}="3H20 SO1"; $resn_exch{"H211 SO1"}="1H21 SO1"; $resn_exch{"H212 SO1"}="2H21 SO1";
$resn_exch{"H213 SO1"}="3H21 SO1"; $resn_exch{"HNB1 SOA"}="1HNB SOA"; $resn_exch{"HNB2 SOA"}="2HNB SOA";
$resn_exch{"H121 SOA"}="1H12 SOA"; $resn_exch{"H122 SOA"}="2H12 SOA"; $resn_exch{" H   SOC"}="1HN  SOC";
$resn_exch{" H2  SOC"}="2HN  SOC"; $resn_exch{" HB2 SOC"}="1HB  SOC"; $resn_exch{" HB3 SOC"}="2HB  SOC";
$resn_exch{" HE  SOC"}="HSE1 SOC"; $resn_exch{"H101 SOD"}="1H10 SOD"; $resn_exch{"H102 SOD"}="2H10 SOD";
$resn_exch{"H181 SOD"}="1H18 SOD"; $resn_exch{"H182 SOD"}="2H18 SOD"; $resn_exch{"H251 SOD"}="1H25 SOD";
$resn_exch{"H252 SOD"}="2H25 SOD"; $resn_exch{"H253 SOD"}="3H25 SOD"; $resn_exch{" H71 SOD"}="1H7  SOD";
$resn_exch{" H72 SOD"}="2H7  SOD"; $resn_exch{" H81 SOD"}="1H8  SOD"; $resn_exch{" H82 SOD"}="2H8  SOD";
$resn_exch{"H611 SOD"}="1H61 SOD"; $resn_exch{"H612 SOD"}="2H61 SOD"; $resn_exch{"H613 SOD"}="3H61 SOD";
$resn_exch{"H651 SOD"}="1H65 SOD"; $resn_exch{"H652 SOD"}="2H65 SOD"; $resn_exch{"H701 SOD"}="1H70 SOD";
$resn_exch{"H702 SOD"}="2H70 SOD"; $resn_exch{"H531 SOD"}="1H53 SOD"; $resn_exch{"H532 SOD"}="2H53 SOD";
$resn_exch{"H201 SOD"}="1H20 SOD"; $resn_exch{"H202 SOD"}="2H20 SOD"; $resn_exch{"H203 SOD"}="3H20 SOD";
$resn_exch{"H211 SOD"}="1H21 SOD"; $resn_exch{"H212 SOD"}="2H21 SOD"; $resn_exch{"H213 SOD"}="3H21 SOD";
$resn_exch{" H61 SOG"}="1H6  SOG"; $resn_exch{" H62 SOG"}="2H6  SOG"; $resn_exch{"H1'1 SOG"}="1H1' SOG";
$resn_exch{"H1'2 SOG"}="2H1' SOG"; $resn_exch{"H2'1 SOG"}="1H2' SOG"; $resn_exch{"H2'2 SOG"}="2H2' SOG";
$resn_exch{"H3'1 SOG"}="1H3' SOG"; $resn_exch{"H3'2 SOG"}="2H3' SOG"; $resn_exch{"H4'1 SOG"}="1H4' SOG";
$resn_exch{"H4'2 SOG"}="2H4' SOG"; $resn_exch{"H5'1 SOG"}="1H5' SOG"; $resn_exch{"H5'2 SOG"}="2H5' SOG";
$resn_exch{"H6'1 SOG"}="1H6' SOG"; $resn_exch{"H6'2 SOG"}="2H6' SOG"; $resn_exch{"H7'1 SOG"}="1H7' SOG";
$resn_exch{"H7'2 SOG"}="2H7' SOG"; $resn_exch{"H8'1 SOG"}="1H8' SOG"; $resn_exch{"H8'2 SOG"}="2H8' SOG";
$resn_exch{"H8'3 SOG"}="3H8' SOG"; $resn_exch{" H11 SOM"}="1H1  SOM"; $resn_exch{" H12 SOM"}="2H1  SOM";
$resn_exch{" H13 SOM"}="3H1  SOM"; $resn_exch{" HB1 SON"}="1HB  SON"; $resn_exch{" HB2 SON"}="2HB  SON";
$resn_exch{"H5'1 SON"}="1H5' SON"; $resn_exch{"H5'2 SON"}="2H5' SON"; $resn_exch{"HN61 SON"}="1HN6 SON";
$resn_exch{"HN62 SON"}="2HN6 SON"; $resn_exch{"HN61 SOP"}="1HN6 SOP"; $resn_exch{"HN62 SOP"}="2HN6 SOP";
$resn_exch{" H1' SOP"}=" H1* SOP"; $resn_exch{" H2' SOP"}=" H2* SOP"; $resn_exch{"HO2' SOP"}="HO2* SOP";
$resn_exch{" H3' SOP"}=" H3* SOP"; $resn_exch{" H4' SOP"}=" H4* SOP"; $resn_exch{"H5'1 SOP"}="1H5* SOP";
$resn_exch{"H5'2 SOP"}="2H5* SOP"; $resn_exch{"HP91 SOP"}="1HP9 SOP"; $resn_exch{"HP92 SOP"}="2HP9 SOP";
$resn_exch{"HPB1 SOP"}="1HPB SOP"; $resn_exch{"HPB2 SOP"}="2HPB SOP"; $resn_exch{"HPB3 SOP"}="3HPB SOP";
$resn_exch{"HPA1 SOP"}="1HPA SOP"; $resn_exch{"HPA2 SOP"}="2HPA SOP"; $resn_exch{"HPA3 SOP"}="3HPA SOP";
$resn_exch{"HP51 SOP"}="1HP5 SOP"; $resn_exch{"HP52 SOP"}="2HP5 SOP"; $resn_exch{"HP41 SOP"}="1HP4 SOP";
$resn_exch{"HP42 SOP"}="2HP4 SOP"; $resn_exch{"HP21 SOP"}="1HP2 SOP"; $resn_exch{"HP22 SOP"}="2HP2 SOP";
$resn_exch{"HP11 SOP"}="1HP1 SOP"; $resn_exch{"HP12 SOP"}="2HP1 SOP"; $resn_exch{"HA11 SOP"}="1HA1 SOP";
$resn_exch{"HA12 SOP"}="2HA1 SOP"; $resn_exch{"HA31 SOP"}="1HA3 SOP"; $resn_exch{"HA32 SOP"}="2HA3 SOP";
$resn_exch{"HA33 SOP"}="3HA3 SOP"; $resn_exch{" H11 SOR"}="1H1  SOR"; $resn_exch{" H12 SOR"}="2H1  SOR";
$resn_exch{" H61 SOR"}="1H6  SOR"; $resn_exch{" H62 SOR"}="2H6  SOR"; $resn_exch{" H21 SOT"}="1H2  SOT";
$resn_exch{" H22 SOT"}="2H2  SOT"; $resn_exch{" H31 SOT"}="1H3  SOT"; $resn_exch{" H32 SOT"}="2H3  SOT";
$resn_exch{" H51 SOT"}="1H5  SOT"; $resn_exch{" H52 SOT"}="2H5  SOT"; $resn_exch{" H61 SOT"}="1H6  SOT";
$resn_exch{" H62 SOT"}="2H6  SOT"; $resn_exch{"H101 SOX"}="1H10 SOX"; $resn_exch{"H102 SOX"}="2H10 SOX";
$resn_exch{"H103 SOX"}="3H10 SOX"; $resn_exch{"H9C1 SOX"}="1H9C SOX"; $resn_exch{"H9C2 SOX"}="2H9C SOX";
$resn_exch{"H9C3 SOX"}="3H9C SOX"; $resn_exch{"H171 SOX"}="1H17 SOX"; $resn_exch{"H172 SOX"}="2H17 SOX";
$resn_exch{" HN1 SOY"}="1HN  SOY"; $resn_exch{" HN2 SOY"}="2HN  SOY"; $resn_exch{"HBC1 SOY"}="1HBC SOY";
$resn_exch{"HBC2 SOY"}="2HBC SOY"; $resn_exch{"H7C1 SOY"}="1H7C SOY"; $resn_exch{"H7C2 SOY"}="2H7C SOY";
$resn_exch{"H7C3 SOY"}="3H7C SOY"; $resn_exch{"H8C1 SOY"}="1H8C SOY"; $resn_exch{"H8C2 SOY"}="2H8C SOY";
$resn_exch{"H8C3 SOY"}="3H8C SOY"; $resn_exch{"H191 SOY"}="1H19 SOY"; $resn_exch{"H192 SOY"}="2H19 SOY";
$resn_exch{"H193 SOY"}="3H19 SOY"; $resn_exch{"HOP2 SP1"}="2HOP SP1"; $resn_exch{"H5'1 SP1"}="1H5* SP1";
$resn_exch{"H5'2 SP1"}="2H5* SP1"; $resn_exch{" H4' SP1"}=" H4* SP1"; $resn_exch{" H3' SP1"}=" H3* SP1";
$resn_exch{" H2' SP1"}=" H2* SP1"; $resn_exch{"HO2' SP1"}="*HO2 SP1"; $resn_exch{" H1' SP1"}=" H1* SP1";
$resn_exch{"HN61 SP1"}="1HN6 SP1"; $resn_exch{"HN62 SP1"}="2HN6 SP1"; $resn_exch{"HMA1 SP2"}="1HMA SP2";
$resn_exch{"HMA2 SP2"}="2HMA SP2"; $resn_exch{"HMA3 SP2"}="3HMA SP2"; $resn_exch{"HM11 SP2"}="1HM1 SP2";
$resn_exch{"HM12 SP2"}="2HM1 SP2"; $resn_exch{"HM13 SP2"}="3HM1 SP2"; $resn_exch{"HM21 SP2"}="1HM2 SP2";
$resn_exch{"HM22 SP2"}="2HM2 SP2"; $resn_exch{"HM23 SP2"}="3HM2 SP2"; $resn_exch{"H21A SP2"}="1H2  SP2";
$resn_exch{" H22 SP2"}="2H2  SP2"; $resn_exch{" H31 SP2"}="1H3  SP2"; $resn_exch{" H32 SP2"}="2H3  SP2";
$resn_exch{" H41 SP2"}="1H4  SP2"; $resn_exch{" H42 SP2"}="2H4  SP2"; $resn_exch{"HM31 SP2"}="1HM3 SP2";
$resn_exch{"HM32 SP2"}="2HM3 SP2"; $resn_exch{"HM33 SP2"}="3HM3 SP2"; $resn_exch{"HM41 SP2"}="1HM4 SP2";
$resn_exch{"HM42 SP2"}="2HM4 SP2"; $resn_exch{"HM43 SP2"}="3HM4 SP2"; $resn_exch{"HM51 SP2"}="1HM5 SP2";
$resn_exch{"HM52 SP2"}="2HM5 SP2"; $resn_exch{"HM53 SP2"}="3HM5 SP2"; $resn_exch{"HM61 SP2"}="1HM6 SP2";
$resn_exch{"HM62 SP2"}="2HM6 SP2"; $resn_exch{"HM63 SP2"}="3HM6 SP2"; $resn_exch{"HM71 SP2"}="1HM7 SP2";
$resn_exch{"HM72 SP2"}="2HM7 SP2"; $resn_exch{"HM73 SP2"}="3HM7 SP2"; $resn_exch{"H231 SP2"}="1H23 SP2";
$resn_exch{"H232 SP2"}="2H23 SP2"; $resn_exch{"H241 SP2"}="1H24 SP2"; $resn_exch{"H242 SP2"}="2H24 SP2";
$resn_exch{"HM81 SP2"}="1HM8 SP2"; $resn_exch{"HM82 SP2"}="2HM8 SP2"; $resn_exch{"HM83 SP2"}="3HM8 SP2";
$resn_exch{"H271 SP2"}="1H27 SP2"; $resn_exch{"H272 SP2"}="2H27 SP2"; $resn_exch{"H281 SP2"}="1H28 SP2";
$resn_exch{"H282 SP2"}="2H28 SP2"; $resn_exch{"HMB1 SP2"}="1HMB SP2"; $resn_exch{"HMB2 SP2"}="2HMB SP2";
$resn_exch{"HMB3 SP2"}="3HMB SP2"; $resn_exch{"HM91 SP2"}="1HM9 SP2"; $resn_exch{"HM92 SP2"}="2HM9 SP2";
$resn_exch{"HM93 SP2"}="3HM9 SP2"; $resn_exch{" H41 SP6"}="1H4  SP6"; $resn_exch{" H42 SP6"}="2H4  SP6";
$resn_exch{" H61 SP6"}="1H6  SP6"; $resn_exch{"H111 SP6"}="1H11 SP6"; $resn_exch{"H112 SP6"}="2H11 SP6";
$resn_exch{" H61 SPA"}="1H6  SPA"; $resn_exch{" H62 SPA"}="2H6  SPA"; $resn_exch{"H10C SPB"}="CH10 SPB";
$resn_exch{"H11C SPB"}="CH11 SPB"; $resn_exch{"H13C SPB"}="CH13 SPB"; $resn_exch{"H14C SPB"}="CH14 SPB";
$resn_exch{"H15N SPB"}="NH15 SPB"; $resn_exch{"H171 SPB"}="1H17 SPB"; $resn_exch{"H172 SPB"}="2H17 SPB";
$resn_exch{"H181 SPB"}="1H18 SPB"; $resn_exch{"H182 SPB"}="2H18 SPB"; $resn_exch{"H191 SPB"}="1H19 SPB";
$resn_exch{"H192 SPB"}="2H19 SPB"; $resn_exch{"H23O SPB"}="OH23 SPB"; $resn_exch{"HC21 SPC"}="1HC2 SPC";
$resn_exch{"HC31 SPC"}="1HC3 SPC"; $resn_exch{"HC51 SPC"}="1HC5 SPC"; $resn_exch{"HC61 SPC"}="1HC6 SPC";
$resn_exch{"HC81 SPC"}="1HC8 SPC"; $resn_exch{"HC82 SPC"}="2HC8 SPC"; $resn_exch{"HC83 SPC"}="3HC8 SPC";
$resn_exch{"H151 SPC"}="1H15 SPC"; $resn_exch{"H161 SPC"}="1H16 SPC"; $resn_exch{"H162 SPC"}="2H16 SPC";
$resn_exch{"H171 SPC"}="1H17 SPC"; $resn_exch{"H181 SPC"}="1H18 SPC"; $resn_exch{"H182 SPC"}="2H18 SPC";
$resn_exch{"H231 SPC"}="1H23 SPC"; $resn_exch{"H251 SPC"}="1H25 SPC"; $resn_exch{"H252 SPC"}="2H25 SPC";
$resn_exch{"H253 SPC"}="3H25 SPC"; $resn_exch{"HN11 SPD"}="1HN1 SPD"; $resn_exch{"HN12 SPD"}="2HN1 SPD";
$resn_exch{" H21 SPD"}="1H2  SPD"; $resn_exch{" H22 SPD"}="2H2  SPD"; $resn_exch{" H31 SPD"}="1H3  SPD";
$resn_exch{" H32 SPD"}="2H3  SPD"; $resn_exch{" H41 SPD"}="1H4  SPD"; $resn_exch{" H42 SPD"}="2H4  SPD";
$resn_exch{" H51 SPD"}="1H5  SPD"; $resn_exch{" H52 SPD"}="2H5  SPD"; $resn_exch{" H71 SPD"}="1H7  SPD";
$resn_exch{" H72 SPD"}="2H7  SPD"; $resn_exch{" H81 SPD"}="1H8  SPD"; $resn_exch{" H82 SPD"}="2H8  SPD";
$resn_exch{" H91 SPD"}="1H9  SPD"; $resn_exch{" H92 SPD"}="2H9  SPD"; $resn_exch{"H101 SPD"}="1H10 SPD";
$resn_exch{"H102 SPD"}="2H10 SPD"; $resn_exch{"HN11 SPE"}="1HN1 SPE"; $resn_exch{"HN12 SPE"}="2HN1 SPE";
$resn_exch{" H21 SPE"}="1H2  SPE"; $resn_exch{" H22 SPE"}="2H2  SPE"; $resn_exch{" H31 SPE"}="1H3  SPE";
$resn_exch{" H32 SPE"}="2H3  SPE"; $resn_exch{" H41 SPE"}="1H4  SPE"; $resn_exch{" H42 SPE"}="2H4  SPE";
$resn_exch{" H61 SPE"}="1H6  SPE"; $resn_exch{" H62 SPE"}="2H6  SPE"; $resn_exch{" H71 SPE"}="1H7  SPE";
$resn_exch{" H72 SPE"}="2H7  SPE"; $resn_exch{" H81 SPE"}="1H8  SPE"; $resn_exch{" H82 SPE"}="2H8  SPE";
$resn_exch{"H101 SPE"}="1H10 SPE"; $resn_exch{"H102 SPE"}="2H10 SPE"; $resn_exch{"H111 SPE"}="1H11 SPE";
$resn_exch{"H112 SPE"}="2H11 SPE"; $resn_exch{"H121 SPE"}="1H12 SPE"; $resn_exch{"H122 SPE"}="2H12 SPE";
$resn_exch{"HN31 SPE"}="1HN3 SPE"; $resn_exch{"HN32 SPE"}="2HN3 SPE"; $resn_exch{"HN61 SPF"}="1HN6 SPF";
$resn_exch{"HN62 SPF"}="2HN6 SPF"; $resn_exch{" H1' SPF"}=" H1* SPF"; $resn_exch{" H2' SPF"}=" H2* SPF";
$resn_exch{" H3' SPF"}=" H3* SPF"; $resn_exch{" H4' SPF"}=" H4* SPF"; $resn_exch{"H5'1 SPF"}="1H5* SPF";
$resn_exch{"H5'2 SPF"}="2H5* SPF"; $resn_exch{" H11 SPF"}="1H1  SPF"; $resn_exch{"H12X SPF"}="2H1  SPF";
$resn_exch{" H13 SPF"}="3H1  SPF"; $resn_exch{"H141 SPF"}="1H14 SPF"; $resn_exch{"H142 SPF"}="2H14 SPF";
$resn_exch{"H143 SPF"}="3H14 SPF"; $resn_exch{" H71 SPF"}="1H7  SPF"; $resn_exch{" H72 SPF"}="2H7  SPF";
$resn_exch{" H61 SPF"}="1H6  SPF"; $resn_exch{" H62 SPF"}="2H6  SPF"; $resn_exch{" H31 SPF"}="1H3  SPF";
$resn_exch{" H32 SPF"}="2H3  SPF"; $resn_exch{" H21 SPF"}="1H2  SPF"; $resn_exch{" H22 SPF"}="2H2  SPF";
$resn_exch{"H111 SPF"}="1H11 SPF"; $resn_exch{"H112 SPF"}="2H11 SPF"; $resn_exch{"H113 SPF"}="3H11 SPF";
$resn_exch{"H131 SPF"}="1H13 SPF"; $resn_exch{"H132 SPF"}="2H13 SPF"; $resn_exch{"H133 SPF"}="3H13 SPF";
$resn_exch{"HOP2 SPG"}="2HOP SPG"; $resn_exch{"HOP3 SPG"}="3HOP SPG"; $resn_exch{"HOA2 SPG"}="2HOA SPG";
$resn_exch{"HOA3 SPG"}="3HOA SPG"; $resn_exch{"H5'1 SPG"}="1H5* SPG"; $resn_exch{"H5'2 SPG"}="2H5* SPG";
$resn_exch{" H4' SPG"}=" H4* SPG"; $resn_exch{" H3' SPG"}=" H3* SPG"; $resn_exch{"HO3' SPG"}="*HO3 SPG";
$resn_exch{" H2' SPG"}=" H2* SPG"; $resn_exch{"HO2' SPG"}="*HO2 SPG"; $resn_exch{" H1' SPG"}=" H1* SPG";
$resn_exch{" H11 SPH"}="1H1  SPH"; $resn_exch{" H12 SPH"}="2H1  SPH"; $resn_exch{"HN21 SPH"}="1HN2 SPH";
$resn_exch{"HN22 SPH"}="2HN2 SPH"; $resn_exch{" H61 SPH"}="1H6  SPH"; $resn_exch{" H62 SPH"}="2H6  SPH";
$resn_exch{" H71 SPH"}="1H7  SPH"; $resn_exch{" H72 SPH"}="2H7  SPH"; $resn_exch{" H81 SPH"}="1H8  SPH";
$resn_exch{" H82 SPH"}="2H8  SPH"; $resn_exch{" H91 SPH"}="1H9  SPH"; $resn_exch{" H92 SPH"}="2H9  SPH";
$resn_exch{"H101 SPH"}="1H10 SPH"; $resn_exch{"H102 SPH"}="2H10 SPH"; $resn_exch{"H111 SPH"}="1H11 SPH";
$resn_exch{"H112 SPH"}="2H11 SPH"; $resn_exch{"H121 SPH"}="1H12 SPH"; $resn_exch{"H122 SPH"}="2H12 SPH";
$resn_exch{"H131 SPH"}="1H13 SPH"; $resn_exch{"H132 SPH"}="2H13 SPH"; $resn_exch{"H141 SPH"}="1H14 SPH";
$resn_exch{"H142 SPH"}="2H14 SPH"; $resn_exch{"H151 SPH"}="1H15 SPH"; $resn_exch{"H152 SPH"}="2H15 SPH";
$resn_exch{"H161 SPH"}="1H16 SPH"; $resn_exch{"H162 SPH"}="2H16 SPH"; $resn_exch{"H171 SPH"}="1H17 SPH";
$resn_exch{"H172 SPH"}="2H17 SPH"; $resn_exch{"H181 SPH"}="1H18 SPH"; $resn_exch{"H182 SPH"}="2H18 SPH";
$resn_exch{"H183 SPH"}="3H18 SPH"; $resn_exch{"HC21 SPI"}="1HC2 SPI"; $resn_exch{"HC31 SPI"}="1HC3 SPI";
$resn_exch{"HC51 SPI"}="1HC5 SPI"; $resn_exch{"HC61 SPI"}="1HC6 SPI"; $resn_exch{"HC71 SPI"}="1HC7 SPI";
$resn_exch{"HC72 SPI"}="2HC7 SPI"; $resn_exch{"HC73 SPI"}="3HC7 SPI"; $resn_exch{"HC81 SPI"}="1HC8 SPI";
$resn_exch{"HC91 SPI"}="1HC9 SPI"; $resn_exch{"HC92 SPI"}="2HC9 SPI"; $resn_exch{"H101 SPI"}="1H10 SPI";
$resn_exch{"H102 SPI"}="2H10 SPI"; $resn_exch{"H111 SPI"}="1H11 SPI"; $resn_exch{"H112 SPI"}="2H11 SPI";
$resn_exch{"H131 SPI"}="1H13 SPI"; $resn_exch{"H132 SPI"}="2H13 SPI"; $resn_exch{"H151 SPI"}="1H15 SPI";
$resn_exch{"H161 SPI"}="1H16 SPI"; $resn_exch{"H171 SPI"}="1H17 SPI"; $resn_exch{"H181 SPI"}="1H18 SPI";
$resn_exch{"H191 SPI"}="1H19 SPI"; $resn_exch{"H101 SPK"}="1H10 SPK"; $resn_exch{"H102 SPK"}="2H10 SPK";
$resn_exch{"H111 SPK"}="1H11 SPK"; $resn_exch{"H112 SPK"}="2H11 SPK"; $resn_exch{"H121 SPK"}="1H12 SPK";
$resn_exch{"H122 SPK"}="2H12 SPK"; $resn_exch{"H131 SPK"}="1H13 SPK"; $resn_exch{"H132 SPK"}="2H13 SPK";
$resn_exch{"H141 SPK"}="1H14 SPK"; $resn_exch{"H142 SPK"}="2H14 SPK"; $resn_exch{"H143 SPK"}="3H14 SPK";
$resn_exch{" H11 SPL"}="1H1  SPL"; $resn_exch{" H12 SPL"}="2H1  SPL"; $resn_exch{" H61 SPL"}="1H6  SPL";
$resn_exch{" H62 SPL"}="2H6  SPL"; $resn_exch{" H71 SPL"}="1H7  SPL"; $resn_exch{" H72 SPL"}="2H7  SPL";
$resn_exch{" H81 SPL"}="1H8  SPL"; $resn_exch{" H82 SPL"}="2H8  SPL"; $resn_exch{" H91 SPL"}="1H9  SPL";
$resn_exch{" H92 SPL"}="2H9  SPL"; $resn_exch{"H101 SPL"}="1H10 SPL"; $resn_exch{"H102 SPL"}="2H10 SPL";
$resn_exch{"H111 SPL"}="1H11 SPL"; $resn_exch{"H112 SPL"}="2H11 SPL"; $resn_exch{"H121 SPL"}="1H12 SPL";
$resn_exch{"H122 SPL"}="2H12 SPL"; $resn_exch{"H131 SPL"}="1H13 SPL"; $resn_exch{"H132 SPL"}="2H13 SPL";
$resn_exch{"H141 SPL"}="1H14 SPL"; $resn_exch{"H142 SPL"}="2H14 SPL"; $resn_exch{"H151 SPL"}="1H15 SPL";
$resn_exch{"H152 SPL"}="2H15 SPL"; $resn_exch{"H161 SPL"}="1H16 SPL"; $resn_exch{"H162 SPL"}="2H16 SPL";
$resn_exch{"H171 SPL"}="1H17 SPL"; $resn_exch{"H172 SPL"}="2H17 SPL"; $resn_exch{"H181 SPL"}="1H18 SPL";
$resn_exch{"H182 SPL"}="2H18 SPL"; $resn_exch{"H183 SPL"}="3H18 SPL"; $resn_exch{"H201 SPL"}="1H20 SPL";
$resn_exch{"H202 SPL"}="2H20 SPL"; $resn_exch{"H211 SPL"}="1H21 SPL"; $resn_exch{"H212 SPL"}="2H21 SPL";
$resn_exch{"H221 SPL"}="1H22 SPL"; $resn_exch{"H222 SPL"}="2H22 SPL"; $resn_exch{"H231 SPL"}="1H23 SPL";
$resn_exch{"H232 SPL"}="2H23 SPL"; $resn_exch{"H241 SPL"}="1H24 SPL"; $resn_exch{"H242 SPL"}="2H24 SPL";
$resn_exch{"H251 SPL"}="1H25 SPL"; $resn_exch{"H252 SPL"}="2H25 SPL"; $resn_exch{"H261 SPL"}="1H26 SPL";
$resn_exch{"H262 SPL"}="2H26 SPL"; $resn_exch{"H263 SPL"}="3H26 SPL"; $resn_exch{"HN11 SPM"}="1HN1 SPM";
$resn_exch{"HN12 SPM"}="2HN1 SPM"; $resn_exch{" H21 SPM"}="1H2  SPM"; $resn_exch{" H22 SPM"}="2H2  SPM";
$resn_exch{" H31 SPM"}="1H3  SPM"; $resn_exch{" H32 SPM"}="2H3  SPM"; $resn_exch{" H41 SPM"}="1H4  SPM";
$resn_exch{" H42 SPM"}="2H4  SPM"; $resn_exch{" H61 SPM"}="1H6  SPM"; $resn_exch{" H62 SPM"}="2H6  SPM";
$resn_exch{" H71 SPM"}="1H7  SPM"; $resn_exch{" H72 SPM"}="2H7  SPM"; $resn_exch{" H81 SPM"}="1H8  SPM";
$resn_exch{" H82 SPM"}="2H8  SPM"; $resn_exch{" H91 SPM"}="1H9  SPM"; $resn_exch{" H92 SPM"}="2H9  SPM";
$resn_exch{"H111 SPM"}="1H11 SPM"; $resn_exch{"H112 SPM"}="2H11 SPM"; $resn_exch{"H121 SPM"}="1H12 SPM";
$resn_exch{"H122 SPM"}="2H12 SPM"; $resn_exch{"H131 SPM"}="1H13 SPM"; $resn_exch{"H132 SPM"}="2H13 SPM";
$resn_exch{"HN41 SPM"}="1HN4 SPM"; $resn_exch{"HN42 SPM"}="2HN4 SPM"; $resn_exch{"HMA1 SPN"}="1HMA SPN";
$resn_exch{"HMA2 SPN"}="2HMA SPN"; $resn_exch{"HMA3 SPN"}="3HMA SPN"; $resn_exch{"HM11 SPN"}="1HM1 SPN";
$resn_exch{"HM12 SPN"}="2HM1 SPN"; $resn_exch{"HM13 SPN"}="3HM1 SPN"; $resn_exch{"HM21 SPN"}="1HM2 SPN";
$resn_exch{"HM22 SPN"}="2HM2 SPN"; $resn_exch{"HM23 SPN"}="3HM2 SPN"; $resn_exch{" H31 SPN"}="1H3  SPN";
$resn_exch{" H32 SPN"}="2H3  SPN"; $resn_exch{"HM31 SPN"}="1HM3 SPN"; $resn_exch{"HM32 SPN"}="2HM3 SPN";
$resn_exch{"HM33 SPN"}="3HM3 SPN"; $resn_exch{" H61 SPN"}="1H6  SPN"; $resn_exch{" H62 SPN"}="2H6  SPN";
$resn_exch{" H71 SPN"}="1H7  SPN"; $resn_exch{" H72 SPN"}="2H7  SPN"; $resn_exch{"HM41 SPN"}="1HM4 SPN";
$resn_exch{"HM42 SPN"}="2HM4 SPN"; $resn_exch{"HM43 SPN"}="3HM4 SPN"; $resn_exch{"H101 SPN"}="1H10 SPN";
$resn_exch{"H102 SPN"}="2H10 SPN"; $resn_exch{"H111 SPN"}="1H11 SPN"; $resn_exch{"H112 SPN"}="2H11 SPN";
$resn_exch{"HM51 SPN"}="1HM5 SPN"; $resn_exch{"HM52 SPN"}="2HM5 SPN"; $resn_exch{"HM53 SPN"}="3HM5 SPN";
$resn_exch{"H141 SPN"}="1H14 SPN"; $resn_exch{"H142 SPN"}="2H14 SPN"; $resn_exch{"H151 SPN"}="1H15 SPN";
$resn_exch{"H152 SPN"}="2H15 SPN"; $resn_exch{"H161 SPN"}="1H16 SPN"; $resn_exch{"H162 SPN"}="2H16 SPN";
$resn_exch{"H171 SPN"}="1H17 SPN"; $resn_exch{"H172 SPN"}="2H17 SPN"; $resn_exch{"HM61 SPN"}="1HM6 SPN";
$resn_exch{"HM62 SPN"}="2HM6 SPN"; $resn_exch{"HM63 SPN"}="3HM6 SPN"; $resn_exch{"H201 SPN"}="1H20 SPN";
$resn_exch{"H202 SPN"}="2H20 SPN"; $resn_exch{"H211 SPN"}="1H21 SPN"; $resn_exch{"H212 SPN"}="2H21 SPN";
$resn_exch{"HM71 SPN"}="1HM7 SPN"; $resn_exch{"HM72 SPN"}="2HM7 SPN"; $resn_exch{"HM73 SPN"}="3HM7 SPN";
$resn_exch{"H231 SPN"}="1H23 SPN"; $resn_exch{"H232 SPN"}="2H23 SPN"; $resn_exch{"H241 SPN"}="1H24 SPN";
$resn_exch{"H242 SPN"}="2H24 SPN"; $resn_exch{"HM81 SPN"}="1HM8 SPN"; $resn_exch{"HM82 SPN"}="2HM8 SPN";
$resn_exch{"HM83 SPN"}="3HM8 SPN"; $resn_exch{"H271 SPN"}="1H27 SPN"; $resn_exch{"H272 SPN"}="2H27 SPN";
$resn_exch{"H281 SPN"}="1H28 SPN"; $resn_exch{"H282 SPN"}="2H28 SPN"; $resn_exch{"HM91 SPN"}="1HM9 SPN";
$resn_exch{"HM92 SPN"}="2HM9 SPN"; $resn_exch{"HM93 SPN"}="3HM9 SPN"; $resn_exch{"HMB1 SPN"}="1HMB SPN";
$resn_exch{"HMB2 SPN"}="2HMB SPN"; $resn_exch{"HMB3 SPN"}="3HMB SPN"; $resn_exch{"HM11 SPO"}="1HM1 SPO";
$resn_exch{"HM12 SPO"}="2HM1 SPO"; $resn_exch{"HM13 SPO"}="3HM1 SPO"; $resn_exch{"H21A SPO"}="1H2  SPO";
$resn_exch{"H22A SPO"}="2H2  SPO"; $resn_exch{" H23 SPO"}="3H2  SPO"; $resn_exch{" H31 SPO"}="1H3  SPO";
$resn_exch{"H32A SPO"}="2H3  SPO"; $resn_exch{" H33 SPO"}="3H3  SPO"; $resn_exch{" H41 SPO"}="1H4  SPO";
$resn_exch{" H42 SPO"}="2H4  SPO"; $resn_exch{" H81 SPO"}="1H8  SPO"; $resn_exch{" H82 SPO"}="2H8  SPO";
$resn_exch{" H83 SPO"}="3H8  SPO"; $resn_exch{"H131 SPO"}="1H13 SPO"; $resn_exch{"H132 SPO"}="2H13 SPO";
$resn_exch{"H133 SPO"}="3H13 SPO"; $resn_exch{"H181 SPO"}="1H18 SPO"; $resn_exch{"H182 SPO"}="2H18 SPO";
$resn_exch{"H183 SPO"}="3H18 SPO"; $resn_exch{"H241 SPO"}="1H24 SPO"; $resn_exch{"H242 SPO"}="2H24 SPO";
$resn_exch{"H243 SPO"}="3H24 SPO"; $resn_exch{"H291 SPO"}="1H29 SPO"; $resn_exch{"H292 SPO"}="2H29 SPO";
$resn_exch{"H293 SPO"}="3H29 SPO"; $resn_exch{"H301 SPO"}="1H30 SPO"; $resn_exch{"H302 SPO"}="2H30 SPO";
$resn_exch{"H311 SPO"}="1H31 SPO"; $resn_exch{"H312 SPO"}="2H31 SPO"; $resn_exch{"H341 SPO"}="1H34 SPO";
$resn_exch{"H342 SPO"}="2H34 SPO"; $resn_exch{"H343 SPO"}="3H34 SPO"; $resn_exch{"H351 SPO"}="1H35 SPO";
$resn_exch{"H352 SPO"}="2H35 SPO"; $resn_exch{"H361 SPO"}="1H36 SPO"; $resn_exch{"H362 SPO"}="2H36 SPO";
$resn_exch{"H391 SPO"}="1H39 SPO"; $resn_exch{"H392 SPO"}="2H39 SPO"; $resn_exch{"H393 SPO"}="3H39 SPO";
$resn_exch{"H401 SPO"}="1H40 SPO"; $resn_exch{"H402 SPO"}="2H40 SPO"; $resn_exch{"H403 SPO"}="3H40 SPO";
$resn_exch{" H81 SPP"}="1H8  SPP"; $resn_exch{" H82 SPP"}="2H8  SPP"; $resn_exch{" H91 SPP"}="1H9  SPP";
$resn_exch{" H92 SPP"}="2H9  SPP"; $resn_exch{"H111 SPP"}="1H11 SPP"; $resn_exch{"H112 SPP"}="2H11 SPP";
$resn_exch{"H121 SPP"}="1H12 SPP"; $resn_exch{"H122 SPP"}="2H12 SPP"; $resn_exch{" HB1 SPP"}="1HB  SPP";
$resn_exch{" HB2 SPP"}="2HB  SPP"; $resn_exch{" HB3 SPP"}="3HB  SPP"; $resn_exch{" HC1 SPP"}="1HC  SPP";
$resn_exch{" HC2 SPP"}="2HC  SPP"; $resn_exch{" HC3 SPP"}="3HC  SPP"; $resn_exch{" HE1 SPP"}="1HE  SPP";
$resn_exch{" HE2 SPP"}="2HE  SPP"; $resn_exch{" HE3 SPP"}="3HE  SPP"; $resn_exch{"H101 SPQ"}="1H10 SPQ";
$resn_exch{"H102 SPQ"}="2H10 SPQ"; $resn_exch{" H61 SPQ"}="1H6  SPQ"; $resn_exch{" H62 SPQ"}="2H6  SPQ";
$resn_exch{"H181 SPR"}="1H18 SPR"; $resn_exch{"H182 SPR"}="2H18 SPR"; $resn_exch{" H21 SPR"}="1H2  SPR";
$resn_exch{" H22 SPR"}="2H2  SPR"; $resn_exch{" H71 SPR"}="1H7  SPR"; $resn_exch{" H72 SPR"}="2H7  SPR";
$resn_exch{"H141 SPR"}="1H14 SPR"; $resn_exch{"H142 SPR"}="2H14 SPR"; $resn_exch{"H161 SPR"}="1H16 SPR";
$resn_exch{"H162 SPR"}="2H16 SPR"; $resn_exch{"H163 SPR"}="3H16 SPR"; $resn_exch{"H171 SPR"}="1H17 SPR";
$resn_exch{"H172 SPR"}="2H17 SPR"; $resn_exch{"H173 SPR"}="3H17 SPR"; $resn_exch{"H201 SPR"}="1H20 SPR";
$resn_exch{"H202 SPR"}="2H20 SPR"; $resn_exch{"H203 SPR"}="3H20 SPR"; $resn_exch{"H6A1 SPR"}="1H6A SPR";
$resn_exch{"H6A2 SPR"}="2H6A SPR"; $resn_exch{"H6A3 SPR"}="3H6A SPR"; $resn_exch{"H7A1 SPR"}="1H7A SPR";
$resn_exch{"H7A2 SPR"}="2H7A SPR"; $resn_exch{"H7A3 SPR"}="3H7A SPR"; $resn_exch{"H8A1 SPR"}="1H8A SPR";
$resn_exch{"H8A2 SPR"}="2H8A SPR"; $resn_exch{"H8A3 SPR"}="3H8A SPR"; $resn_exch{"H2B1 SPR"}="1H2B SPR";
$resn_exch{"H2B2 SPR"}="2H2B SPR"; $resn_exch{"H6B1 SPR"}="1H6B SPR"; $resn_exch{"H6B2 SPR"}="2H6B SPR";
$resn_exch{"H6B3 SPR"}="3H6B SPR"; $resn_exch{"H7B1 SPR"}="1H7B SPR"; $resn_exch{"H7B2 SPR"}="2H7B SPR";
$resn_exch{"H7B3 SPR"}="3H7B SPR"; $resn_exch{"H1C1 SPR"}="1H1C SPR"; $resn_exch{"H2C1 SPR"}="1H2C SPR";
$resn_exch{"H2C2 SPR"}="2H2C SPR"; $resn_exch{"H3C1 SPR"}="1H3C SPR"; $resn_exch{"H3C2 SPR"}="2H3C SPR";
$resn_exch{"H6C1 SPR"}="1H6C SPR"; $resn_exch{"H6C2 SPR"}="2H6C SPR"; $resn_exch{"H6C3 SPR"}="3H6C SPR";
$resn_exch{"H7C1 SPR"}="1H7C SPR"; $resn_exch{"H7C2 SPR"}="2H7C SPR"; $resn_exch{"H7C3 SPR"}="3H7C SPR";
$resn_exch{"H8C1 SPR"}="1H8C SPR"; $resn_exch{"H8C2 SPR"}="2H8C SPR"; $resn_exch{"H8C3 SPR"}="3H8C SPR";
$resn_exch{" H71 SPS"}="1H7  SPS"; $resn_exch{" H72 SPS"}="2H7  SPS"; $resn_exch{" H73 SPS"}="3H7  SPS";
$resn_exch{" H81 SPS"}="1H8  SPS"; $resn_exch{" H91 SPS"}="1H9  SPS"; $resn_exch{"H121 SPS"}="1H12 SPS";
$resn_exch{"H131 SPS"}="1H13 SPS"; $resn_exch{"H132 SPS"}="2H13 SPS"; $resn_exch{"H141 SPS"}="1H14 SPS";
$resn_exch{"H142 SPS"}="2H14 SPS"; $resn_exch{"H161 SPS"}="1H16 SPS"; $resn_exch{"H162 SPS"}="2H16 SPS";
$resn_exch{"H181 SPS"}="1H18 SPS"; $resn_exch{"H182 SPS"}="2H18 SPS"; $resn_exch{"H183 SPS"}="3H18 SPS";
$resn_exch{" H71 SPT"}="1H5M SPT"; $resn_exch{" H72 SPT"}="2H5M SPT"; $resn_exch{" H73 SPT"}="3H5M SPT";
$resn_exch{" H2' SPT"}="1H2* SPT"; $resn_exch{"H2'' SPT"}="2H2* SPT"; $resn_exch{" H5' SPT"}="1H5* SPT";
$resn_exch{"H5'' SPT"}="2H5* SPT"; $resn_exch{" H4' SPT"}=" H4* SPT"; $resn_exch{" H1' SPT"}=" H1* SPT";
$resn_exch{" H3' SPT"}=" H3* SPT"; $resn_exch{"HO3' SPT"}="H3T  SPT"; $resn_exch{"HOP3 SPT"}="3HOP SPT";
$resn_exch{"HOP2 SPT"}="2HOP SPT"; $resn_exch{"H2'O SPV"}="OH2' SPV"; $resn_exch{" H31 SPV"}="1H3  SPV";
$resn_exch{" H32 SPV"}="2H3  SPV"; $resn_exch{"H2SO SPV"}="OH2S SPV"; $resn_exch{" DXT SPW"}="1D10 SPW";
$resn_exch{" DXU SPW"}="2D10 SPW"; $resn_exch{" DXX SPW"}="1D14 SPW"; $resn_exch{" DXY SPW"}="2D14 SPW";
$resn_exch{" DXZ SPW"}="3D14 SPW"; $resn_exch{" H81 SPX"}="1H8  SPX"; $resn_exch{" H82 SPX"}="2H8  SPX";
$resn_exch{"H481 SPX"}="1H48 SPX"; $resn_exch{"H482 SPX"}="2H48 SPX"; $resn_exch{"H483 SPX"}="3H48 SPX";
$resn_exch{" H61 SPX"}="1H6  SPX"; $resn_exch{" H62 SPX"}="2H6  SPX"; $resn_exch{"H381 SPX"}="1H38 SPX";
$resn_exch{"H382 SPX"}="2H38 SPX"; $resn_exch{"H383 SPX"}="3H38 SPX"; $resn_exch{"H411 SPX"}="1H41 SPX";
$resn_exch{"H412 SPX"}="2H41 SPX"; $resn_exch{"H413 SPX"}="3H41 SPX"; $resn_exch{"H491 SPX"}="1H49 SPX";
$resn_exch{"H492 SPX"}="2H49 SPX"; $resn_exch{"H493 SPX"}="3H49 SPX"; $resn_exch{"H141 SPX"}="1H14 SPX";
$resn_exch{"H142 SPX"}="2H14 SPX"; $resn_exch{"H501 SPX"}="1H50 SPX"; $resn_exch{"H502 SPX"}="2H50 SPX";
$resn_exch{"H503 SPX"}="3H50 SPX"; $resn_exch{"H421 SPX"}="1H42 SPX"; $resn_exch{"H422 SPX"}="2H42 SPX";
$resn_exch{"H423 SPX"}="3H42 SPX"; $resn_exch{"H201 SPX"}="1H20 SPX"; $resn_exch{"H202 SPX"}="2H20 SPX";
$resn_exch{"H511 SPX"}="1H51 SPX"; $resn_exch{"H512 SPX"}="2H51 SPX"; $resn_exch{"H513 SPX"}="3H51 SPX";
$resn_exch{"H431 SPX"}="1H43 SPX"; $resn_exch{"H432 SPX"}="2H43 SPX"; $resn_exch{"H433 SPX"}="3H43 SPX";
$resn_exch{"H441 SPX"}="1H44 SPX"; $resn_exch{"H442 SPX"}="2H44 SPX"; $resn_exch{"H443 SPX"}="3H44 SPX";
$resn_exch{"H521 SPX"}="1H52 SPX"; $resn_exch{"H522 SPX"}="2H52 SPX"; $resn_exch{"H523 SPX"}="3H52 SPX";
$resn_exch{"H451 SPX"}="1H45 SPX"; $resn_exch{"H452 SPX"}="2H45 SPX"; $resn_exch{"H453 SPX"}="3H45 SPX";
$resn_exch{"H291 SPX"}="1H29 SPX"; $resn_exch{"H292 SPX"}="2H29 SPX"; $resn_exch{"H301 SPX"}="1H30 SPX";
$resn_exch{"H302 SPX"}="2H30 SPX"; $resn_exch{"H461 SPX"}="1H46 SPX"; $resn_exch{"H462 SPX"}="2H46 SPX";
$resn_exch{"H463 SPX"}="3H46 SPX"; $resn_exch{"H531 SPX"}="1H53 SPX"; $resn_exch{"H532 SPX"}="2H53 SPX";
$resn_exch{"H533 SPX"}="3H53 SPX"; $resn_exch{"H341 SPX"}="1H34 SPX"; $resn_exch{"H342 SPX"}="2H34 SPX";
$resn_exch{"H471 SPX"}="1H47 SPX"; $resn_exch{"H472 SPX"}="2H47 SPX"; $resn_exch{"H473 SPX"}="3H47 SPX";
$resn_exch{"H1C1 SQA"}="1H1C SQA"; $resn_exch{"H1C2 SQA"}="2H1C SQA"; $resn_exch{"H1C3 SQA"}="3H1C SQA";
$resn_exch{"H3C1 SQA"}="1H3C SQA"; $resn_exch{"H3C2 SQA"}="2H3C SQA"; $resn_exch{"H4C1 SQA"}="1H4C SQA";
$resn_exch{"H4C2 SQA"}="2H4C SQA"; $resn_exch{"H5C1 SQA"}="1H5C SQA"; $resn_exch{"H5C2 SQA"}="2H5C SQA";
$resn_exch{"H8C1 SQA"}="1H8C SQA"; $resn_exch{"H8C2 SQA"}="2H8C SQA"; $resn_exch{"H9C1 SQA"}="1H9C SQA";
$resn_exch{"H9C2 SQA"}="2H9C SQA"; $resn_exch{"H121 SQA"}="1H12 SQA"; $resn_exch{"H122 SQA"}="2H12 SQA";
$resn_exch{"H131 SQA"}="1H13 SQA"; $resn_exch{"H132 SQA"}="2H13 SQA"; $resn_exch{"H161 SQA"}="1H16 SQA";
$resn_exch{"H162 SQA"}="2H16 SQA"; $resn_exch{"H171 SQA"}="1H17 SQA"; $resn_exch{"H172 SQA"}="2H17 SQA";
$resn_exch{"H201 SQA"}="1H20 SQA"; $resn_exch{"H202 SQA"}="2H20 SQA"; $resn_exch{"H211 SQA"}="1H21 SQA";
$resn_exch{"H212 SQA"}="2H21 SQA"; $resn_exch{"H241 SQA"}="1H24 SQA"; $resn_exch{"H242 SQA"}="2H24 SQA";
$resn_exch{"H243 SQA"}="3H24 SQA"; $resn_exch{"H251 SQA"}="1H25 SQA"; $resn_exch{"H252 SQA"}="2H25 SQA";
$resn_exch{"H253 SQA"}="3H25 SQA"; $resn_exch{"H261 SQA"}="1H26 SQA"; $resn_exch{"H262 SQA"}="2H26 SQA";
$resn_exch{"H263 SQA"}="3H26 SQA"; $resn_exch{"H271 SQA"}="1H27 SQA"; $resn_exch{"H272 SQA"}="2H27 SQA";
$resn_exch{"H273 SQA"}="3H27 SQA"; $resn_exch{"H281 SQA"}="1H28 SQA"; $resn_exch{"H282 SQA"}="2H28 SQA";
$resn_exch{"H283 SQA"}="3H28 SQA"; $resn_exch{"H291 SQA"}="1H29 SQA"; $resn_exch{"H292 SQA"}="2H29 SQA";
$resn_exch{"H293 SQA"}="3H29 SQA"; $resn_exch{"H301 SQA"}="1H30 SQA"; $resn_exch{"H302 SQA"}="2H30 SQA";
$resn_exch{"H303 SQA"}="3H30 SQA"; $resn_exch{"H441 SQD"}="1H44 SQD"; $resn_exch{"H442 SQD"}="2H44 SQD";
$resn_exch{"H461 SQD"}="1H46 SQD"; $resn_exch{"H462 SQD"}="2H46 SQD"; $resn_exch{" H81 SQD"}="1H8  SQD";
$resn_exch{" H82 SQD"}="2H8  SQD"; $resn_exch{" H91 SQD"}="1H9  SQD"; $resn_exch{" H92 SQD"}="2H9  SQD";
$resn_exch{"H101 SQD"}="1H10 SQD"; $resn_exch{"H102 SQD"}="2H10 SQD"; $resn_exch{"H111 SQD"}="1H11 SQD";
$resn_exch{"H112 SQD"}="2H11 SQD"; $resn_exch{"H121 SQD"}="1H12 SQD"; $resn_exch{"H122 SQD"}="2H12 SQD";
$resn_exch{"H131 SQD"}="1H13 SQD"; $resn_exch{"H132 SQD"}="2H13 SQD"; $resn_exch{"H141 SQD"}="1H14 SQD";
$resn_exch{"H142 SQD"}="2H14 SQD"; $resn_exch{"H151 SQD"}="1H15 SQD"; $resn_exch{"H152 SQD"}="2H15 SQD";
$resn_exch{"H161 SQD"}="1H16 SQD"; $resn_exch{"H162 SQD"}="2H16 SQD"; $resn_exch{"H171 SQD"}="1H17 SQD";
$resn_exch{"H172 SQD"}="2H17 SQD"; $resn_exch{"H181 SQD"}="1H18 SQD"; $resn_exch{"H182 SQD"}="2H18 SQD";
$resn_exch{"H191 SQD"}="1H19 SQD"; $resn_exch{"H192 SQD"}="2H19 SQD"; $resn_exch{"H201 SQD"}="1H20 SQD";
$resn_exch{"H202 SQD"}="2H20 SQD"; $resn_exch{"H211 SQD"}="1H21 SQD"; $resn_exch{"H212 SQD"}="2H21 SQD";
$resn_exch{"H221 SQD"}="1H22 SQD"; $resn_exch{"H222 SQD"}="2H22 SQD"; $resn_exch{"H223 SQD"}="3H22 SQD";
$resn_exch{"H241 SQD"}="1H24 SQD"; $resn_exch{"H242 SQD"}="2H24 SQD"; $resn_exch{"H251 SQD"}="1H25 SQD";
$resn_exch{"H252 SQD"}="2H25 SQD"; $resn_exch{"H261 SQD"}="1H26 SQD"; $resn_exch{"H262 SQD"}="2H26 SQD";
$resn_exch{"H271 SQD"}="1H27 SQD"; $resn_exch{"H272 SQD"}="2H27 SQD"; $resn_exch{"H281 SQD"}="1H28 SQD";
$resn_exch{"H282 SQD"}="2H28 SQD"; $resn_exch{"H291 SQD"}="1H29 SQD"; $resn_exch{"H292 SQD"}="2H29 SQD";
$resn_exch{"H301 SQD"}="1H30 SQD"; $resn_exch{"H302 SQD"}="2H30 SQD"; $resn_exch{"H311 SQD"}="1H31 SQD";
$resn_exch{"H312 SQD"}="2H31 SQD"; $resn_exch{"H321 SQD"}="1H32 SQD"; $resn_exch{"H322 SQD"}="2H32 SQD";
$resn_exch{"H331 SQD"}="1H33 SQD"; $resn_exch{"H332 SQD"}="2H33 SQD"; $resn_exch{"H341 SQD"}="1H34 SQD";
$resn_exch{"H342 SQD"}="2H34 SQD"; $resn_exch{"H351 SQD"}="1H35 SQD"; $resn_exch{"H352 SQD"}="2H35 SQD";
$resn_exch{"H361 SQD"}="1H36 SQD"; $resn_exch{"H362 SQD"}="2H36 SQD"; $resn_exch{"H371 SQD"}="1H37 SQD";
$resn_exch{"H372 SQD"}="2H37 SQD"; $resn_exch{"H381 SQD"}="1H38 SQD"; $resn_exch{"H382 SQD"}="2H38 SQD";
$resn_exch{"H383 SQD"}="3H38 SQD"; $resn_exch{" H61 SQD"}="1H6  SQD"; $resn_exch{" H62 SQD"}="2H6  SQD";
$resn_exch{" H11 SQU"}="1H1  SQU"; $resn_exch{"H12A SQU"}="2H1  SQU"; $resn_exch{" H13 SQU"}="3H1  SQU";
$resn_exch{" H31 SQU"}="1H3  SQU"; $resn_exch{" H32 SQU"}="2H3  SQU"; $resn_exch{" H33 SQU"}="3H3  SQU";
$resn_exch{" H41 SQU"}="1H4  SQU"; $resn_exch{" H42 SQU"}="2H4  SQU"; $resn_exch{" H51 SQU"}="1H5  SQU";
$resn_exch{" H52 SQU"}="2H5  SQU"; $resn_exch{" H61 SQU"}="1H6  SQU"; $resn_exch{" H62 SQU"}="2H6  SQU";
$resn_exch{" H71 SQU"}="1H7  SQU"; $resn_exch{" H72 SQU"}="2H7  SQU"; $resn_exch{" H91 SQU"}="1H9  SQU";
$resn_exch{" H92 SQU"}="2H9  SQU"; $resn_exch{"H101 SQU"}="1H10 SQU"; $resn_exch{"H102 SQU"}="2H10 SQU";
$resn_exch{"H111 SQU"}="1H11 SQU"; $resn_exch{"H112 SQU"}="2H11 SQU"; $resn_exch{"H131 SQU"}="1H13 SQU";
$resn_exch{"H132 SQU"}="2H13 SQU"; $resn_exch{"H133 SQU"}="3H13 SQU"; $resn_exch{"H141 SQU"}="1H14 SQU";
$resn_exch{"H142 SQU"}="2H14 SQU"; $resn_exch{"H151 SQU"}="1H15 SQU"; $resn_exch{"H152 SQU"}="2H15 SQU";
$resn_exch{"H161 SQU"}="1H16 SQU"; $resn_exch{"H162 SQU"}="2H16 SQU"; $resn_exch{"H171 SQU"}="1H17 SQU";
$resn_exch{"H172 SQU"}="2H17 SQU"; $resn_exch{"H181 SQU"}="1H18 SQU"; $resn_exch{"H182 SQU"}="2H18 SQU";
$resn_exch{"H201 SQU"}="1H20 SQU"; $resn_exch{"H202 SQU"}="2H20 SQU"; $resn_exch{"H211 SQU"}="1H21 SQU";
$resn_exch{"H212 SQU"}="2H21 SQU"; $resn_exch{"H221 SQU"}="1H22 SQU"; $resn_exch{"H222 SQU"}="2H22 SQU";
$resn_exch{"H231 SQU"}="1H23 SQU"; $resn_exch{"H232 SQU"}="2H23 SQU"; $resn_exch{"H251 SQU"}="1H25 SQU";
$resn_exch{"H252 SQU"}="2H25 SQU"; $resn_exch{"H261 SQU"}="1H26 SQU"; $resn_exch{"H262 SQU"}="2H26 SQU";
$resn_exch{"H271 SQU"}="1H27 SQU"; $resn_exch{"H272 SQU"}="2H27 SQU"; $resn_exch{"H291 SQU"}="1H29 SQU";
$resn_exch{"H292 SQU"}="2H29 SQU"; $resn_exch{"H293 SQU"}="3H29 SQU"; $resn_exch{"H301 SQU"}="1H30 SQU";
$resn_exch{"H302 SQU"}="2H30 SQU"; $resn_exch{"H303 SQU"}="3H30 SQU"; $resn_exch{" HS1 SR1"}="1HS  SR1";
$resn_exch{" HS2 SR1"}="2HS  SR1"; $resn_exch{" HS3 SR1"}="3HS  SR1"; $resn_exch{"H5'1 SR1"}="1H5' SR1";
$resn_exch{"H5'2 SR1"}="2H5' SR1"; $resn_exch{"HO'1 SR1"}="1HO' SR1"; $resn_exch{"HO'2 SR1"}="2HO' SR1";
$resn_exch{"HO'3 SR1"}="3HO' SR1"; $resn_exch{"H111 SR2"}="1H11 SR2"; $resn_exch{"H511 SR2"}="1H51 SR2";
$resn_exch{"H611 SR2"}="1H61 SR2"; $resn_exch{"H612 SR2"}="2H61 SR2"; $resn_exch{"H651 SR2"}="1H65 SR2";
$resn_exch{"H652 SR2"}="2H65 SR2"; $resn_exch{"H653 SR2"}="3H65 SR2"; $resn_exch{"H671 SR2"}="1H67 SR2";
$resn_exch{"H672 SR2"}="2H67 SR2"; $resn_exch{"H673 SR2"}="3H67 SR2"; $resn_exch{"HOP1 SRA"}="1HOP SRA";
$resn_exch{"HOP3 SRA"}="3HOP SRA"; $resn_exch{" H5' SRA"}="1H5* SRA"; $resn_exch{"H5'' SRA"}="2H5* SRA";
$resn_exch{" H4' SRA"}=" H4* SRA"; $resn_exch{" H3' SRA"}=" H3* SRA"; $resn_exch{"HO3' SRA"}="*HO3 SRA";
$resn_exch{" H2' SRA"}="1H2* SRA"; $resn_exch{" H1' SRA"}=" H1* SRA"; $resn_exch{"HN61 SRA"}="1HN6 SRA";
$resn_exch{"HN62 SRA"}="2HN6 SRA"; $resn_exch{"HO2' SRA"}="*HO2 SRA"; $resn_exch{"H7'1 SRB"}="1H7' SRB";
$resn_exch{"H7'2 SRB"}="2H7' SRB"; $resn_exch{"H9'1 SRB"}="1H9' SRB"; $resn_exch{"H9'2 SRB"}="2H9' SRB";
$resn_exch{"H9'3 SRB"}="3H9' SRB"; $resn_exch{"HOB1 SRB"}="1HOB SRB"; $resn_exch{"HOB2 SRB"}="2HOB SRB";
$resn_exch{"HOB3 SRB"}="3HOB SRB"; $resn_exch{"H7'1 SRD"}="1H7' SRD"; $resn_exch{"H7'2 SRD"}="2H7' SRD";
$resn_exch{"H9'1 SRD"}="1H9' SRD"; $resn_exch{"H9'2 SRD"}="2H9' SRD"; $resn_exch{"H9'3 SRD"}="3H9' SRD";
$resn_exch{"HOB1 SRD"}="1HOB SRD"; $resn_exch{"HOB2 SRD"}="2HOB SRD"; $resn_exch{"HOB3 SRD"}="3HOB SRD";
$resn_exch{" HB1 SRI"}="1HB  SRI"; $resn_exch{" HB2 SRI"}="2HB  SRI"; $resn_exch{"H141 SRL"}="1H14 SRL";
$resn_exch{"H142 SRL"}="2H14 SRL"; $resn_exch{"H143 SRL"}="3H14 SRL"; $resn_exch{"H151 SRL"}="1H15 SRL";
$resn_exch{"H152 SRL"}="2H15 SRL"; $resn_exch{"H153 SRL"}="3H15 SRL"; $resn_exch{"H161 SRL"}="1H16 SRL";
$resn_exch{"H162 SRL"}="2H16 SRL"; $resn_exch{"H163 SRL"}="3H16 SRL"; $resn_exch{"H171 SRL"}="1H17 SRL";
$resn_exch{"H172 SRL"}="2H17 SRL"; $resn_exch{"H173 SRL"}="3H17 SRL"; $resn_exch{"H181 SRL"}="1H18 SRL";
$resn_exch{"H182 SRL"}="2H18 SRL"; $resn_exch{"H183 SRL"}="3H18 SRL"; $resn_exch{"H191 SRL"}="1H19 SRL";
$resn_exch{"H192 SRL"}="2H19 SRL"; $resn_exch{"H193 SRL"}="3H19 SRL"; $resn_exch{"H221 SRL"}="1H22 SRL";
$resn_exch{"H222 SRL"}="2H22 SRL"; $resn_exch{"H231 SRL"}="1H23 SRL"; $resn_exch{"H232 SRL"}="2H23 SRL";
$resn_exch{"H233 SRL"}="3H23 SRL"; $resn_exch{"H251 SRL"}="1H25 SRL"; $resn_exch{"H252 SRL"}="2H25 SRL";
$resn_exch{"H261 SRL"}="1H26 SRL"; $resn_exch{"H262 SRL"}="2H26 SRL"; $resn_exch{"H263 SRL"}="3H26 SRL";
$resn_exch{"H291 SRL"}="1H29 SRL"; $resn_exch{"H292 SRL"}="2H29 SRL"; $resn_exch{"H301 SRL"}="1H30 SRL";
$resn_exch{"H302 SRL"}="2H30 SRL"; $resn_exch{"H303 SRL"}="3H30 SRL"; $resn_exch{"H321 SRL"}="1H32 SRL";
$resn_exch{"H322 SRL"}="2H32 SRL"; $resn_exch{"H331 SRL"}="1H33 SRL"; $resn_exch{"H332 SRL"}="2H33 SRL";
$resn_exch{"H333 SRL"}="3H33 SRL"; $resn_exch{"HMA1 SRM"}="1HMA SRM"; $resn_exch{"HMA2 SRM"}="2HMA SRM";
$resn_exch{"HMA3 SRM"}="3HMA SRM"; $resn_exch{"HDA1 SRM"}="1HDA SRM"; $resn_exch{"HDA2 SRM"}="2HDA SRM";
$resn_exch{"HAA1 SRM"}="1HAA SRM"; $resn_exch{"HAA2 SRM"}="2HAA SRM"; $resn_exch{"HBA1 SRM"}="1HBA SRM";
$resn_exch{"HBA2 SRM"}="2HBA SRM"; $resn_exch{"HMB1 SRM"}="1HMB SRM"; $resn_exch{"HMB2 SRM"}="2HMB SRM";
$resn_exch{"HMB3 SRM"}="3HMB SRM"; $resn_exch{"HDB1 SRM"}="1HDB SRM"; $resn_exch{"HDB2 SRM"}="2HDB SRM";
$resn_exch{"HAB1 SRM"}="1HAB SRM"; $resn_exch{"HAB2 SRM"}="2HAB SRM"; $resn_exch{"HBB1 SRM"}="1HBB SRM";
$resn_exch{"HBB2 SRM"}="2HBB SRM"; $resn_exch{"HCD1 SRM"}="1HCD SRM"; $resn_exch{"HCD2 SRM"}="2HCD SRM";
$resn_exch{"HAC1 SRM"}="1HAC SRM"; $resn_exch{"HAC2 SRM"}="2HAC SRM"; $resn_exch{"HBC1 SRM"}="1HBC SRM";
$resn_exch{"HBC2 SRM"}="2HBC SRM"; $resn_exch{"HAD1 SRM"}="1HAD SRM"; $resn_exch{"HAD2 SRM"}="2HAD SRM";
$resn_exch{"HBD1 SRM"}="1HBD SRM"; $resn_exch{"HBD2 SRM"}="2HBD SRM"; $resn_exch{"HDD1 SRM"}="1HDD SRM";
$resn_exch{"HDD2 SRM"}="2HDD SRM"; $resn_exch{"H441 SRN"}="1H44 SRN"; $resn_exch{"H442 SRN"}="2H44 SRN";
$resn_exch{"H431 SRN"}="1H43 SRN"; $resn_exch{"H432 SRN"}="2H43 SRN"; $resn_exch{"H421 SRN"}="1H42 SRN";
$resn_exch{"H422 SRN"}="2H42 SRN"; $resn_exch{"H411 SRN"}="1H41 SRN"; $resn_exch{"H412 SRN"}="2H41 SRN";
$resn_exch{"H401 SRN"}="1H40 SRN"; $resn_exch{"H402 SRN"}="2H40 SRN"; $resn_exch{"H403 SRN"}="3H40 SRN";
$resn_exch{"H371 SRN"}="1H37 SRN"; $resn_exch{"H372 SRN"}="2H37 SRN"; $resn_exch{"H373 SRN"}="3H37 SRN";
$resn_exch{"H301 SRN"}="1H30 SRN"; $resn_exch{"H302 SRN"}="2H30 SRN"; $resn_exch{"H271 SRN"}="1H27 SRN";
$resn_exch{"H272 SRN"}="2H27 SRN"; $resn_exch{"H261 SRN"}="1H26 SRN"; $resn_exch{"H262 SRN"}="2H26 SRN";
$resn_exch{"H201 SRN"}="1H20 SRN"; $resn_exch{"H202 SRN"}="2H20 SRN"; $resn_exch{"H181 SRN"}="1H18 SRN";
$resn_exch{"H182 SRN"}="2H18 SRN"; $resn_exch{"H183 SRN"}="3H18 SRN"; $resn_exch{"H151 SRN"}="1H15 SRN";
$resn_exch{"H152 SRN"}="2H15 SRN"; $resn_exch{"H341 SRN"}="1H34 SRN"; $resn_exch{"H342 SRN"}="2H34 SRN";
$resn_exch{"H111 SRN"}="1H11 SRN"; $resn_exch{"H112 SRN"}="2H11 SRN"; $resn_exch{"H113 SRN"}="3H11 SRN";
$resn_exch{" HB1 SRO"}="1HB  SRO"; $resn_exch{" HB2 SRO"}="2HB  SRO"; $resn_exch{" HA1 SRO"}="1HA  SRO";
$resn_exch{" HA2 SRO"}="2HA  SRO"; $resn_exch{"HNZ1 SRO"}="1HNZ SRO"; $resn_exch{"HNZ2 SRO"}="2HNZ SRO";
$resn_exch{"H5'1 SRP"}="1H5* SRP"; $resn_exch{"H5'2 SRP"}="2H5* SRP"; $resn_exch{" H4' SRP"}=" H4* SRP";
$resn_exch{" H3' SRP"}=" H3* SRP"; $resn_exch{" H2' SRP"}=" H2* SRP"; $resn_exch{" H1' SRP"}=" H1* SRP";
$resn_exch{"HN61 SRP"}="1HN6 SRP"; $resn_exch{"HN62 SRP"}="2HN6 SRP"; $resn_exch{" HN1 SRP"}="1HN  SRP";
$resn_exch{" HN2 SRP"}="2HN  SRP"; $resn_exch{" HB1 SRP"}="1HB  SRP"; $resn_exch{" HB2 SRP"}="2HB  SRP";
$resn_exch{" H21 SRS"}="1H2  SRS"; $resn_exch{"H22A SRS"}="2H2  SRS"; $resn_exch{" H71 SRS"}="1H7  SRS";
$resn_exch{" H72 SRS"}="2H7  SRS"; $resn_exch{" H81 SRS"}="1H8  SRS"; $resn_exch{" H82 SRS"}="2H8  SRS";
$resn_exch{" H91 SRS"}="1H9  SRS"; $resn_exch{" H92 SRS"}="2H9  SRS"; $resn_exch{"H101 SRS"}="1H10 SRS";
$resn_exch{"H102 SRS"}="2H10 SRS"; $resn_exch{"H111 SRS"}="1H11 SRS"; $resn_exch{"H112 SRS"}="2H11 SRS";
$resn_exch{"H121 SRS"}="1H12 SRS"; $resn_exch{"H122 SRS"}="2H12 SRS"; $resn_exch{"H131 SRS"}="1H13 SRS";
$resn_exch{"H132 SRS"}="2H13 SRS"; $resn_exch{"H151 SRS"}="1H15 SRS"; $resn_exch{"H152 SRS"}="2H15 SRS";
$resn_exch{"H153 SRS"}="3H15 SRS"; $resn_exch{"H161 SRS"}="1H16 SRS"; $resn_exch{"H162 SRS"}="2H16 SRS";
$resn_exch{"H163 SRS"}="3H16 SRS"; $resn_exch{"H171 SRS"}="1H17 SRS"; $resn_exch{"H172 SRS"}="2H17 SRS";
$resn_exch{"H261 SRS"}="1H26 SRS"; $resn_exch{"H262 SRS"}="2H26 SRS"; $resn_exch{"H263 SRS"}="3H26 SRS";
$resn_exch{"HN11 SRY"}="1HN1 SRY"; $resn_exch{"HB11 SRY"}="1HB1 SRY"; $resn_exch{"HB12 SRY"}="2HB1 SRY";
$resn_exch{"HC11 SRY"}="1HC1 SRY"; $resn_exch{"HO21 SRY"}="1HO2 SRY"; $resn_exch{"HN31 SRY"}="1HN3 SRY";
$resn_exch{"HE11 SRY"}="1HE1 SRY"; $resn_exch{"HE12 SRY"}="2HE1 SRY"; $resn_exch{"HF11 SRY"}="1HF1 SRY";
$resn_exch{"HO51 SRY"}="1HO5 SRY"; $resn_exch{"HO61 SRY"}="1HO6 SRY"; $resn_exch{"HO32 SRY"}="2HO3 SRY";
$resn_exch{"HG21 SRY"}="1HG2 SRY"; $resn_exch{"HH21 SRY"}="1HH2 SRY"; $resn_exch{"HH22 SRY"}="2HH2 SRY";
$resn_exch{"HH23 SRY"}="3HH2 SRY"; $resn_exch{"HN23 SRY"}="3HN2 SRY"; $resn_exch{"HI31 SRY"}="1HI3 SRY";
$resn_exch{"HI32 SRY"}="2HI3 SRY"; $resn_exch{"HI33 SRY"}="3HI3 SRY"; $resn_exch{"HO33 SRY"}="3HO3 SRY";
$resn_exch{"HO43 SRY"}="3HO4 SRY"; $resn_exch{"H631 SRY"}="1H63 SRY"; $resn_exch{"H632 SRY"}="2H63 SRY";
$resn_exch{"HO63 SRY"}="3HO6 SRY"; $resn_exch{" H81 SS1"}="1H8  SS1"; $resn_exch{" H82 SS1"}="2H8  SS1";
$resn_exch{" H83 SS1"}="3H8  SS1"; $resn_exch{" H81 SS2"}="1H8  SS2"; $resn_exch{" H82 SS2"}="2H8  SS2";
$resn_exch{" H83 SS2"}="3H8  SS2"; $resn_exch{"H9C1 SS3"}="1H9C SS3"; $resn_exch{"H9C2 SS3"}="2H9C SS3";
$resn_exch{"H9C3 SS3"}="3H9C SS3"; $resn_exch{"H241 SS3"}="1H24 SS3"; $resn_exch{"H242 SS3"}="2H24 SS3";
$resn_exch{"H251 SS3"}="1H25 SS3"; $resn_exch{"H252 SS3"}="2H25 SS3"; $resn_exch{"H4N1 SS3"}="1H4N SS3";
$resn_exch{"H4N2 SS3"}="2H4N SS3"; $resn_exch{"H9C1 SS4"}="1H9C SS4"; $resn_exch{"H9C2 SS4"}="2H9C SS4";
$resn_exch{"H9C3 SS4"}="3H9C SS4"; $resn_exch{"H251 SS4"}="1H25 SS4"; $resn_exch{"H252 SS4"}="2H25 SS4";
$resn_exch{"H241 SS4"}="1H24 SS4"; $resn_exch{"H242 SS4"}="2H24 SS4"; $resn_exch{"H4N1 SS4"}="1H4N SS4";
$resn_exch{"H4N2 SS4"}="2H4N SS4"; $resn_exch{"H7C1 SS5"}="1H7C SS5"; $resn_exch{"H7C2 SS5"}="2H7C SS5";
$resn_exch{"H291 SS5"}="1H29 SS5"; $resn_exch{"H292 SS5"}="2H29 SS5"; $resn_exch{"H101 SS5"}="1H10 SS5";
$resn_exch{"H102 SS5"}="2H10 SS5"; $resn_exch{"H121 SS5"}="1H12 SS5"; $resn_exch{"H122 SS5"}="2H12 SS5";
$resn_exch{"H281 SS5"}="1H28 SS5"; $resn_exch{"H282 SS5"}="2H28 SS5"; $resn_exch{"H283 SS5"}="3H28 SS5";
$resn_exch{"H311 SS5"}="1H31 SS5"; $resn_exch{"H312 SS5"}="2H31 SS5"; $resn_exch{"H351 SS5"}="1H35 SS5";
$resn_exch{"H352 SS5"}="2H35 SS5"; $resn_exch{"H321 SS5"}="1H32 SS5"; $resn_exch{"H322 SS5"}="2H32 SS5";
$resn_exch{"H331 SS5"}="1H33 SS5"; $resn_exch{"H332 SS5"}="2H33 SS5"; $resn_exch{"H341 SS5"}="1H34 SS5";
$resn_exch{"H342 SS5"}="2H34 SS5"; $resn_exch{"HN11 SSA"}="1HN1 SSA"; $resn_exch{"HN12 SSA"}="2HN1 SSA";
$resn_exch{" HB1 SSA"}="1HB  SSA"; $resn_exch{" HB2 SSA"}="2HB  SSA"; $resn_exch{"H5'1 SSA"}="1H5* SSA";
$resn_exch{"H5'2 SSA"}="2H5* SSA"; $resn_exch{" H4' SSA"}=" H4* SSA"; $resn_exch{" H3' SSA"}=" H3* SSA";
$resn_exch{"HO3' SSA"}="*HO3 SSA"; $resn_exch{" H2' SSA"}=" H2* SSA"; $resn_exch{"HO2' SSA"}="*HO2 SSA";
$resn_exch{" H1' SSA"}=" H1* SSA"; $resn_exch{"HN61 SSA"}="1HN6 SSA"; $resn_exch{"HN62 SSA"}="2HN6 SSA";
$resn_exch{" H21 SSB"}="1H2  SSB"; $resn_exch{" H22 SSB"}="2H2  SSB"; $resn_exch{" H41 SSB"}="1H4  SSB";
$resn_exch{" H42 SSB"}="2H4  SSB"; $resn_exch{" H51 SSB"}="1H5  SSB"; $resn_exch{" H52 SSB"}="2H5  SSB";
$resn_exch{" H71 SSB"}="1H7  SSB"; $resn_exch{" H72 SSB"}="2H7  SSB"; $resn_exch{" H81 SSB"}="1H8  SSB";
$resn_exch{" H82 SSB"}="2H8  SSB"; $resn_exch{" H91 SSB"}="1H9  SSB"; $resn_exch{" H92 SSB"}="2H9  SSB";
$resn_exch{"H101 SSB"}="1H10 SSB"; $resn_exch{"H102 SSB"}="2H10 SSB"; $resn_exch{"H103 SSB"}="3H10 SSB";
$resn_exch{"HSS1 SSC"}="1HSS SSC"; $resn_exch{"HSS2 SSC"}="2HSS SSC"; $resn_exch{"H3S1 SSC"}="1H3S SSC";
$resn_exch{"H3S2 SSC"}="2H3S SSC"; $resn_exch{"H5S1 SSC"}="1H5S SSC"; $resn_exch{"H5S2 SSC"}="2H5S SSC";
$resn_exch{" H31 SSD"}="1H3  SSD"; $resn_exch{" H32 SSD"}="2H3  SSD"; $resn_exch{" H61 SSD"}="1H6  SSD";
$resn_exch{" H62 SSD"}="2H6  SSD"; $resn_exch{" H71 SSD"}="1H7  SSD"; $resn_exch{" H72 SSD"}="2H7  SSD";
$resn_exch{"H101 SSD"}="1H10 SSD"; $resn_exch{"H102 SSD"}="2H10 SSD"; $resn_exch{" H31 SSE"}="1H3  SSE";
$resn_exch{" H32 SSE"}="2H3  SSE"; $resn_exch{" H61 SSE"}="1H6  SSE"; $resn_exch{" H62 SSE"}="2H6  SSE";
$resn_exch{" H71 SSE"}="1H7  SSE"; $resn_exch{" H72 SSE"}="2H7  SSE"; $resn_exch{"H101 SSE"}="1H10 SSE";
$resn_exch{"H102 SSE"}="2H10 SSE"; $resn_exch{" H62 SSG"}="2H6  SSG"; $resn_exch{" H61 SSG"}="1H6  SSG";
$resn_exch{"H2C1 SSH"}="1H2C SSH"; $resn_exch{"H3C1 SSH"}="1H3C SSH"; $resn_exch{"H3C2 SSH"}="2H3C SSH";
$resn_exch{"H6C1 SSH"}="1H6C SSH"; $resn_exch{"H6C2 SSH"}="2H6C SSH"; $resn_exch{" H31 SSO"}="1H3  SSO";
$resn_exch{" H32 SSO"}="2H3  SSO"; $resn_exch{" H61 SSO"}="1H6  SSO"; $resn_exch{" H62 SSO"}="2H6  SSO";
$resn_exch{" H71 SSO"}="1H7  SSO"; $resn_exch{" H72 SSO"}="2H7  SSO"; $resn_exch{"H101 SSO"}="1H10 SSO";
$resn_exch{"H102 SSO"}="2H10 SSO"; $resn_exch{"HA11 SSP"}="1HA1 SSP"; $resn_exch{"HA12 SSP"}="2HA1 SSP";
$resn_exch{"HA13 SSP"}="3HA1 SSP"; $resn_exch{"HB11 SSP"}="1HB1 SSP"; $resn_exch{"HB12 SSP"}="2HB1 SSP";
$resn_exch{"HG11 SSP"}="1HG1 SSP"; $resn_exch{"HG12 SSP"}="2HG1 SSP"; $resn_exch{"HA21 SSP"}="1HA2 SSP";
$resn_exch{"HA22 SSP"}="2HA2 SSP"; $resn_exch{"HA23 SSP"}="3HA2 SSP"; $resn_exch{"HB21 SSP"}="1HB2 SSP";
$resn_exch{"HB22 SSP"}="2HB2 SSP"; $resn_exch{"HG21 SSP"}="1HG2 SSP"; $resn_exch{"HG22 SSP"}="2HG2 SSP";
$resn_exch{"H5'1 SSS"}="1H5' SSS"; $resn_exch{"H5'2 SSS"}="2H5' SSS"; $resn_exch{"H3N1 SSS"}="1H3N SSS";
$resn_exch{"H3N2 SSS"}="2H3N SSS"; $resn_exch{"H631 SSS"}="1H63 SSS"; $resn_exch{"H632 SSS"}="2H63 SSS";
$resn_exch{"HO2' SSU"}="2HO* SSU"; $resn_exch{" H2' SSU"}=" H2* SSU"; $resn_exch{" H1' SSU"}=" H1* SSU";
$resn_exch{" H4' SSU"}=" H4* SSU"; $resn_exch{" H5' SSU"}="1H5* SSU"; $resn_exch{"H5'' SSU"}="2H5* SSU";
$resn_exch{" H3' SSU"}=" H3* SSU"; $resn_exch{"HO3' SSU"}="H3T  SSU"; $resn_exch{" H6  SSU"}="6H   SSU";
$resn_exch{" H5  SSU"}="5H   SSU"; $resn_exch{" H3  SSU"}="3H   SSU"; $resn_exch{"HO2' ST1"}="'HO2 ST1";
$resn_exch{"HM41 ST1"}="1HM4 ST1"; $resn_exch{"HM42 ST1"}="2HM4 ST1"; $resn_exch{"HM43 ST1"}="3HM4 ST1";
$resn_exch{"HO2' ST2"}="'HO2 ST2"; $resn_exch{"HN31 ST2"}="1HN3 ST2"; $resn_exch{"HN32 ST2"}="2HN3 ST2";
$resn_exch{"HM41 ST2"}="1HM4 ST2"; $resn_exch{"HM42 ST2"}="2HM4 ST2"; $resn_exch{"HM43 ST2"}="3HM4 ST2";
$resn_exch{"HO2' ST3"}="'HO2 ST3"; $resn_exch{"HN31 ST3"}="1HN3 ST3"; $resn_exch{"HN32 ST3"}="2HN3 ST3";
$resn_exch{"HM41 ST3"}="1HM4 ST3"; $resn_exch{"HM42 ST3"}="2HM4 ST3"; $resn_exch{"HM43 ST3"}="3HM4 ST3";
$resn_exch{"HO2' ST4"}="'HO2 ST4"; $resn_exch{"HN11 ST4"}="1HN1 ST4"; $resn_exch{"HN12 ST4"}="2HN1 ST4";
$resn_exch{"HN21 ST4"}="1HN2 ST4"; $resn_exch{"HN22 ST4"}="2HN2 ST4"; $resn_exch{"HM41 ST4"}="1HM4 ST4";
$resn_exch{"HM42 ST4"}="2HM4 ST4"; $resn_exch{"HM43 ST4"}="3HM4 ST4"; $resn_exch{"HO2' ST5"}="'HO2 ST5";
$resn_exch{"HM31 ST5"}="1HM3 ST5"; $resn_exch{"HM32 ST5"}="2HM3 ST5"; $resn_exch{"HM41 ST5"}="1HM4 ST5";
$resn_exch{"HM42 ST5"}="2HM4 ST5"; $resn_exch{"HM43 ST5"}="3HM4 ST5"; $resn_exch{"HO2' ST6"}="'HO2 ST6";
$resn_exch{"HM31 ST6"}="1HM3 ST6"; $resn_exch{"HM32 ST6"}="2HM3 ST6"; $resn_exch{"HN31 ST6"}="1HN3 ST6";
$resn_exch{"HN32 ST6"}="2HN3 ST6"; $resn_exch{"HN33 ST6"}="3HN3 ST6"; $resn_exch{"HM41 ST6"}="1HM4 ST6";
$resn_exch{"HM42 ST6"}="2HM4 ST6"; $resn_exch{"HM43 ST6"}="3HM4 ST6"; $resn_exch{"H101 ST8"}="1H10 ST8";
$resn_exch{"H102 ST8"}="2H10 ST8"; $resn_exch{"H121 ST8"}="1H12 ST8"; $resn_exch{"H122 ST8"}="2H12 ST8";
$resn_exch{"H131 ST8"}="1H13 ST8"; $resn_exch{"H132 ST8"}="2H13 ST8"; $resn_exch{"H141 ST8"}="1H14 ST8";
$resn_exch{"H142 ST8"}="2H14 ST8"; $resn_exch{"H151 ST8"}="1H15 ST8"; $resn_exch{"H152 ST8"}="2H15 ST8";
$resn_exch{"H161 ST8"}="1H16 ST8"; $resn_exch{"H162 ST8"}="2H16 ST8"; $resn_exch{"H251 ST8"}="1H25 ST8";
$resn_exch{"H252 ST8"}="2H25 ST8"; $resn_exch{"H9N1 ST8"}="1H9N ST8"; $resn_exch{"H9N2 ST8"}="2H9N ST8";
$resn_exch{" HN1 STA"}="1HN  STA"; $resn_exch{" HN2 STA"}="2HN  STA"; $resn_exch{" HB1 STA"}="1HB  STA";
$resn_exch{" HB2 STA"}="2HB  STA"; $resn_exch{"HD11 STA"}="1HD1 STA"; $resn_exch{"HD12 STA"}="2HD1 STA";
$resn_exch{"HD13 STA"}="3HD1 STA"; $resn_exch{"HD21 STA"}="1HD2 STA"; $resn_exch{"HD22 STA"}="2HD2 STA";
$resn_exch{"HD23 STA"}="3HD2 STA"; $resn_exch{" HM1 STA"}="1HM  STA"; $resn_exch{" HM2 STA"}="2HM  STA";
$resn_exch{"H1'1 STB"}="1H1' STB"; $resn_exch{"H1'2 STB"}="2H1' STB"; $resn_exch{"H2'1 STB"}="1H2' STB";
$resn_exch{"H2'2 STB"}="2H2' STB"; $resn_exch{"H3'1 STB"}="1H3' STB"; $resn_exch{"H3'2 STB"}="2H3' STB";
$resn_exch{"H4'1 STB"}="1H4' STB"; $resn_exch{"H4'2 STB"}="2H4' STB"; $resn_exch{"H6'1 STB"}="1H6' STB";
$resn_exch{"H6'2 STB"}="2H6' STB"; $resn_exch{"HN31 STB"}="1HN3 STB"; $resn_exch{"HN32 STB"}="2HN3 STB";
$resn_exch{"HN11 STC"}="1HN1 STC"; $resn_exch{" H31 STC"}="1H3  STC"; $resn_exch{" H41 STC"}="1H4  STC";
$resn_exch{" H51 STC"}="1H5  STC"; $resn_exch{" H61 STC"}="1H6  STC"; $resn_exch{"H191 STC"}="1H19 STC";
$resn_exch{"H221 STC"}="1H22 STC"; $resn_exch{"H231 STC"}="1H23 STC"; $resn_exch{"H111 STD"}="1H11 STD";
$resn_exch{"H112 STD"}="2H11 STD"; $resn_exch{"H113 STD"}="3H11 STD"; $resn_exch{"H121 STD"}="1H12 STD";
$resn_exch{"H122 STD"}="2H12 STD"; $resn_exch{"H141 STD"}="1H14 STD"; $resn_exch{"H142 STD"}="2H14 STD";
$resn_exch{"H143 STD"}="3H14 STD"; $resn_exch{"H151 STD"}="1H15 STD"; $resn_exch{"H152 STD"}="2H15 STD";
$resn_exch{"H181 STD"}="1H18 STD"; $resn_exch{"H182 STD"}="2H18 STD"; $resn_exch{"H183 STD"}="3H18 STD";
$resn_exch{"H231 STD"}="1H23 STD"; $resn_exch{"H232 STD"}="2H23 STD"; $resn_exch{"H233 STD"}="3H23 STD";
$resn_exch{"H241 STD"}="1H24 STD"; $resn_exch{"H242 STD"}="2H24 STD"; $resn_exch{"H243 STD"}="3H24 STD";
$resn_exch{"H271 STD"}="1H27 STD"; $resn_exch{"H272 STD"}="2H27 STD"; $resn_exch{"H273 STD"}="3H27 STD";
$resn_exch{"H291 STD"}="1H29 STD"; $resn_exch{"H292 STD"}="2H29 STD"; $resn_exch{"H293 STD"}="3H29 STD";
$resn_exch{"H311 STD"}="1H31 STD"; $resn_exch{"H312 STD"}="2H31 STD"; $resn_exch{" H21 STE"}="1H2  STE";
$resn_exch{" H22 STE"}="2H2  STE"; $resn_exch{" H31 STE"}="1H3  STE"; $resn_exch{" H32 STE"}="2H3  STE";
$resn_exch{" H41 STE"}="1H4  STE"; $resn_exch{" H42 STE"}="2H4  STE"; $resn_exch{" H51 STE"}="1H5  STE";
$resn_exch{" H52 STE"}="2H5  STE"; $resn_exch{" H61 STE"}="1H6  STE"; $resn_exch{" H62 STE"}="2H6  STE";
$resn_exch{" H71 STE"}="1H7  STE"; $resn_exch{" H72 STE"}="2H7  STE"; $resn_exch{" H81 STE"}="1H8  STE";
$resn_exch{" H82 STE"}="2H8  STE"; $resn_exch{" H91 STE"}="1H9  STE"; $resn_exch{" H92 STE"}="2H9  STE";
$resn_exch{"H101 STE"}="1H10 STE"; $resn_exch{"H102 STE"}="2H10 STE"; $resn_exch{"H111 STE"}="1H11 STE";
$resn_exch{"H112 STE"}="2H11 STE"; $resn_exch{"H121 STE"}="1H12 STE"; $resn_exch{"H122 STE"}="2H12 STE";
$resn_exch{"H131 STE"}="1H13 STE"; $resn_exch{"H132 STE"}="2H13 STE"; $resn_exch{"H141 STE"}="1H14 STE";
$resn_exch{"H142 STE"}="2H14 STE"; $resn_exch{"H151 STE"}="1H15 STE"; $resn_exch{"H152 STE"}="2H15 STE";
$resn_exch{"H161 STE"}="1H16 STE"; $resn_exch{"H162 STE"}="2H16 STE"; $resn_exch{"H171 STE"}="1H17 STE";
$resn_exch{"H172 STE"}="2H17 STE"; $resn_exch{"H181 STE"}="1H18 STE"; $resn_exch{"H182 STE"}="2H18 STE";
$resn_exch{"H183 STE"}="3H18 STE"; $resn_exch{" H7  STF"}="7H   STF"; $resn_exch{" H19 STF"}="9H1  STF";
$resn_exch{" H6  STF"}="6H   STF"; $resn_exch{" H5  STF"}="5H   STF"; $resn_exch{" H4  STF"}="4H   STF";
$resn_exch{" H3  STF"}="3H   STF"; $resn_exch{" H2  STF"}="2H   STF"; $resn_exch{" H1  STF"}="1H   STF";
$resn_exch{" H61 STG"}="1H6  STG"; $resn_exch{" H62 STG"}="2H6  STG"; $resn_exch{" H71 STG"}="1H7  STG";
$resn_exch{" H72 STG"}="2H7  STG"; $resn_exch{"H111 STG"}="1H11 STG"; $resn_exch{"H112 STG"}="2H11 STG";
$resn_exch{"H121 STG"}="1H12 STG"; $resn_exch{"H122 STG"}="2H12 STG"; $resn_exch{"H151 STG"}="1H15 STG";
$resn_exch{"H152 STG"}="2H15 STG"; $resn_exch{"H181 STG"}="1H18 STG"; $resn_exch{"H182 STG"}="2H18 STG";
$resn_exch{"H183 STG"}="3H18 STG"; $resn_exch{"HN71 STH"}="1HN7 STH"; $resn_exch{"HN72 STH"}="2HN7 STH";
$resn_exch{" H11 STI"}="1H1  STI"; $resn_exch{" H61 STI"}="1H6  STI"; $resn_exch{" H41 STI"}="1H4  STI";
$resn_exch{" H21 STI"}="1H2  STI"; $resn_exch{"H121 STI"}="1H12 STI"; $resn_exch{"H111 STI"}="1H11 STI";
$resn_exch{"H131 STI"}="1H13 STI"; $resn_exch{"H181 STI"}="1H18 STI"; $resn_exch{"H171 STI"}="1H17 STI";
$resn_exch{"H151 STI"}="1H15 STI"; $resn_exch{"H211 STI"}="1H21 STI"; $resn_exch{"H251 STI"}="1H25 STI";
$resn_exch{"H261 STI"}="1H26 STI"; $resn_exch{"H281 STI"}="1H28 STI"; $resn_exch{"H291 STI"}="1H29 STI";
$resn_exch{"H461 STI"}="1H46 STI"; $resn_exch{"H462 STI"}="2H46 STI"; $resn_exch{"H531 STI"}="1H53 STI";
$resn_exch{"H532 STI"}="2H53 STI"; $resn_exch{"H521 STI"}="1H52 STI"; $resn_exch{"H522 STI"}="2H52 STI";
$resn_exch{"H541 STI"}="1H54 STI"; $resn_exch{"H542 STI"}="2H54 STI"; $resn_exch{"H543 STI"}="3H54 STI";
$resn_exch{"H501 STI"}="1H50 STI"; $resn_exch{"H502 STI"}="2H50 STI"; $resn_exch{"H491 STI"}="1H49 STI";
$resn_exch{"H492 STI"}="2H49 STI"; $resn_exch{"H201 STI"}="1H20 STI"; $resn_exch{"H202 STI"}="2H20 STI";
$resn_exch{"H203 STI"}="3H20 STI"; $resn_exch{"HD11 STN"}="1HD1 STN"; $resn_exch{"HD12 STN"}="2HD1 STN";
$resn_exch{"HD13 STN"}="3HD1 STN"; $resn_exch{"HCN1 STN"}="1HCN STN"; $resn_exch{"HCN2 STN"}="2HCN STN";
$resn_exch{"HCB1 STN"}="1HCB STN"; $resn_exch{"HCB2 STN"}="2HCB STN"; $resn_exch{"HD21 STN"}="1HD2 STN";
$resn_exch{"HD22 STN"}="2HD2 STN"; $resn_exch{"HD23 STN"}="3HD2 STN"; $resn_exch{"H201 STO"}="1H20 STO";
$resn_exch{"H202 STO"}="2H20 STO"; $resn_exch{"H271 STO"}="1H27 STO"; $resn_exch{"H272 STO"}="2H27 STO";
$resn_exch{"H321 STO"}="1H32 STO"; $resn_exch{"H322 STO"}="2H32 STO"; $resn_exch{"H323 STO"}="3H32 STO";
$resn_exch{"H341 STO"}="1H34 STO"; $resn_exch{"H342 STO"}="2H34 STO"; $resn_exch{"H343 STO"}="3H34 STO";
$resn_exch{"H351 STO"}="1H35 STO"; $resn_exch{"H352 STO"}="2H35 STO"; $resn_exch{"H353 STO"}="3H35 STO";
$resn_exch{"HOG2 STP"}="2HOG STP"; $resn_exch{"HOG3 STP"}="3HOG STP"; $resn_exch{"HOB2 STP"}="2HOB STP";
$resn_exch{"HOA2 STP"}="2HOA STP"; $resn_exch{"H5'1 STP"}="1H5* STP"; $resn_exch{"H5'2 STP"}="2H5* STP";
$resn_exch{" H4' STP"}=" H4* STP"; $resn_exch{" H3' STP"}=" H3* STP"; $resn_exch{"HO3' STP"}="*HO3 STP";
$resn_exch{"H2'1 STP"}="1H2* STP"; $resn_exch{"H2'2 STP"}="2H2* STP"; $resn_exch{" H1' STP"}=" H1* STP";
$resn_exch{"HN61 STP"}="1HN6 STP"; $resn_exch{"HN62 STP"}="2HN6 STP"; $resn_exch{" H11 STR"}="1H1  STR";
$resn_exch{" H12 STR"}="2H1  STR"; $resn_exch{" H21 STR"}="1H2  STR"; $resn_exch{" H22 STR"}="2H2  STR";
$resn_exch{" H61 STR"}="1H6  STR"; $resn_exch{" H62 STR"}="2H6  STR"; $resn_exch{" H71 STR"}="1H7  STR";
$resn_exch{" H72 STR"}="2H7  STR"; $resn_exch{"H111 STR"}="1H11 STR"; $resn_exch{"H112 STR"}="2H11 STR";
$resn_exch{"H121 STR"}="1H12 STR"; $resn_exch{"H122 STR"}="2H12 STR"; $resn_exch{"H151 STR"}="1H15 STR";
$resn_exch{"H152 STR"}="2H15 STR"; $resn_exch{"H161 STR"}="1H16 STR"; $resn_exch{"H162 STR"}="2H16 STR";
$resn_exch{"H181 STR"}="1H18 STR"; $resn_exch{"H182 STR"}="2H18 STR"; $resn_exch{"H183 STR"}="3H18 STR";
$resn_exch{"H191 STR"}="1H19 STR"; $resn_exch{"H192 STR"}="2H19 STR"; $resn_exch{"H193 STR"}="3H19 STR";
$resn_exch{"H211 STR"}="1H21 STR"; $resn_exch{"H212 STR"}="2H21 STR"; $resn_exch{"H213 STR"}="3H21 STR";
$resn_exch{" H21 STS"}="1H2  STS"; $resn_exch{" H22 STS"}="2H2  STS"; $resn_exch{" H31 STS"}="1H3  STS";
$resn_exch{" H32 STS"}="2H3  STS"; $resn_exch{"H241 STU"}="1H24 STU"; $resn_exch{"H242 STU"}="2H24 STU";
$resn_exch{"H261 STU"}="1H26 STU"; $resn_exch{"H262 STU"}="2H26 STU"; $resn_exch{"H263 STU"}="3H26 STU";
$resn_exch{" H91 STU"}="1H9  STU"; $resn_exch{" H92 STU"}="2H9  STU"; $resn_exch{"H271 STU"}="1H27 STU";
$resn_exch{"H272 STU"}="2H27 STU"; $resn_exch{"H273 STU"}="3H27 STU"; $resn_exch{"H281 STU"}="1H28 STU";
$resn_exch{"H282 STU"}="2H28 STU"; $resn_exch{"H283 STU"}="3H28 STU"; $resn_exch{" HA1 STX"}="1HA  STX";
$resn_exch{" HA2 STX"}="2HA  STX"; $resn_exch{" HD1 STX"}="1HD  STX"; $resn_exch{" HD2 STX"}="2HD  STX";
$resn_exch{" HD3 STX"}="3HD  STX"; $resn_exch{" H71 STX"}="1H7  STX"; $resn_exch{" H72 STX"}="2H7  STX";
$resn_exch{" H73 STX"}="3H7  STX"; $resn_exch{" H81 STX"}="1H8  STX"; $resn_exch{" H82 STX"}="2H8  STX";
$resn_exch{" H83 STX"}="3H8  STX"; $resn_exch{" H91 STX"}="1H9  STX"; $resn_exch{" H92 STX"}="2H9  STX";
$resn_exch{" H93 STX"}="3H9  STX"; $resn_exch{" HB1 STY"}="1HB  STY"; $resn_exch{" HB2 STY"}="2HB  STY";
$resn_exch{"H101 SU1"}="1H10 SU1"; $resn_exch{"HN1' SU1"}="'HN1 SU1"; $resn_exch{"H5'1 SU1"}="1H5' SU1";
$resn_exch{"H111 SU1"}="1H11 SU1"; $resn_exch{"H112 SU1"}="2H11 SU1"; $resn_exch{"H113 SU1"}="3H11 SU1";
$resn_exch{"H121 SU1"}="1H12 SU1"; $resn_exch{"H122 SU1"}="2H12 SU1"; $resn_exch{"H131 SU1"}="1H13 SU1";
$resn_exch{"H132 SU1"}="2H13 SU1"; $resn_exch{"H101 SU2"}="1H10 SU2"; $resn_exch{"H102 SU2"}="2H10 SU2";
$resn_exch{"H2'1 SU2"}="1H2* SU2"; $resn_exch{"H2'2 SU2"}="2H2* SU2"; $resn_exch{"H3'1 SU2"}="1H3* SU2";
$resn_exch{"H3'2 SU2"}="2H3* SU2"; $resn_exch{"H5'1 SU2"}="1H5* SU2"; $resn_exch{"H5'2 SU2"}="2H5* SU2";
$resn_exch{"H6'1 SU2"}="1H6* SU2"; $resn_exch{"H6'2 SU2"}="2H6* SU2"; $resn_exch{"H121 SU9"}="1H12 SU9";
$resn_exch{"H122 SU9"}="2H12 SU9"; $resn_exch{"H123 SU9"}="3H12 SU9"; $resn_exch{"HN11 SUA"}="1HN1 SUA";
$resn_exch{"HN12 SUA"}="2HN1 SUA"; $resn_exch{" H11 SUA"}="1H1  SUA"; $resn_exch{" H21 SUA"}="1H2  SUA";
$resn_exch{" H41 SUA"}="1H4  SUA"; $resn_exch{" H51 SUA"}="1H5  SUA"; $resn_exch{"HN21 SUA"}="1HN2 SUA";
$resn_exch{" H81 SUA"}="1H8  SUA"; $resn_exch{" H82 SUA"}="2H8  SUA"; $resn_exch{" H91 SUA"}="1H9  SUA";
$resn_exch{" H92 SUA"}="2H9  SUA"; $resn_exch{"H111 SUA"}="1H11 SUA"; $resn_exch{"H121 SUA"}="1H12 SUA";
$resn_exch{"H131 SUA"}="1H13 SUA"; $resn_exch{" H31 SUB"}="1H3  SUB"; $resn_exch{" H32 SUB"}="2H3  SUB";
$resn_exch{" H41 SUB"}="1H4  SUB"; $resn_exch{" H42 SUB"}="2H4  SUB"; $resn_exch{"H121 SUB"}="1H12 SUB";
$resn_exch{"H122 SUB"}="2H12 SUB"; $resn_exch{"H123 SUB"}="3H12 SUB"; $resn_exch{"H131 SUB"}="1H13 SUB";
$resn_exch{"H132 SUB"}="2H13 SUB"; $resn_exch{"H141 SUB"}="1H14 SUB"; $resn_exch{"H142 SUB"}="2H14 SUB";
$resn_exch{"H151 SUB"}="1H15 SUB"; $resn_exch{"H152 SUB"}="2H15 SUB"; $resn_exch{"H231 SUB"}="1H23 SUB";
$resn_exch{"H232 SUB"}="2H23 SUB"; $resn_exch{"H233 SUB"}="3H23 SUB"; $resn_exch{" H61 SUC"}="1H6  SUC";
$resn_exch{" H62 SUC"}="2H6  SUC"; $resn_exch{"H1'1 SUC"}="1H1' SUC"; $resn_exch{"H1'2 SUC"}="2H1' SUC";
$resn_exch{"H6'1 SUC"}="1H6' SUC"; $resn_exch{"H6'2 SUC"}="2H6' SUC"; $resn_exch{"HO1' SUC"}="'HO1 SUC";
$resn_exch{"HO3' SUC"}="'HO3 SUC"; $resn_exch{"HO4' SUC"}="'HO4 SUC"; $resn_exch{"HO6' SUC"}="'HO6 SUC";
$resn_exch{" H11 SUD"}="1H1  SUD"; $resn_exch{" H12 SUD"}="2H1  SUD"; $resn_exch{"HN41 SUD"}="1HN4 SUD";
$resn_exch{"HN42 SUD"}="2HN4 SUD"; $resn_exch{" H1' SUD"}=" H1* SUD"; $resn_exch{" H2' SUD"}=" H2* SUD";
$resn_exch{" H3' SUD"}=" H3* SUD"; $resn_exch{" H4' SUD"}=" H4* SUD"; $resn_exch{"H5'1 SUD"}="1H5* SUD";
$resn_exch{"H5'2 SUD"}="2H5* SUD"; $resn_exch{"H291 SUD"}="1H29 SUD"; $resn_exch{"H292 SUD"}="2H29 SUD";
$resn_exch{"H321 SUD"}="1H32 SUD"; $resn_exch{"H322 SUD"}="2H32 SUD"; $resn_exch{"H323 SUD"}="3H32 SUD";
$resn_exch{" H21 SUF"}="1H2  SUF"; $resn_exch{" H22 SUF"}="2H2  SUF"; $resn_exch{" H31 SUF"}="1H3  SUF";
$resn_exch{" H32 SUF"}="2H3  SUF"; $resn_exch{" H33 SUF"}="3H3  SUF"; $resn_exch{" H41 SUF"}="1H4  SUF";
$resn_exch{" H42 SUF"}="2H4  SUF"; $resn_exch{" H43 SUF"}="3H4  SUF"; $resn_exch{" HB1 SUG"}="1HB  SUG";
$resn_exch{" HB2 SUG"}="2HB  SUG"; $resn_exch{" HG1 SUG"}="1HG  SUG"; $resn_exch{" HG2 SUG"}="2HG  SUG";
$resn_exch{" HD1 SUG"}="1HD  SUG"; $resn_exch{"HD2A SUG"}="2HD  SUG"; $resn_exch{"HH21 SUG"}="1HH2 SUG";
$resn_exch{"HH22 SUG"}="2HH2 SUG"; $resn_exch{" HW1 SUG"}="1HW  SUG"; $resn_exch{" HW2 SUG"}="2HW  SUG";
$resn_exch{" HX1 SUG"}="1HX  SUG"; $resn_exch{" HX2 SUG"}="2HX  SUG"; $resn_exch{" HN1 SUJ"}="1HN  SUJ";
$resn_exch{" HN2 SUJ"}="2HN  SUJ"; $resn_exch{"HBC1 SUJ"}="1HBC SUJ"; $resn_exch{"HBC2 SUJ"}="2HBC SUJ";
$resn_exch{"HGC1 SUJ"}="1HGC SUJ"; $resn_exch{"HGC2 SUJ"}="2HGC SUJ"; $resn_exch{"H4C1 SUJ"}="1H4C SUJ";
$resn_exch{"H4C2 SUJ"}="2H4C SUJ"; $resn_exch{"H4C3 SUJ"}="3H4C SUJ"; $resn_exch{" H61 SUM"}="1H6  SUM";
$resn_exch{" H62 SUM"}="2H6  SUM"; $resn_exch{"H111 SUM"}="1H11 SUM"; $resn_exch{"H112 SUM"}="2H11 SUM";
$resn_exch{"H221 SUM"}="1H22 SUM"; $resn_exch{"H222 SUM"}="2H22 SUM"; $resn_exch{"H231 SUM"}="1H23 SUM";
$resn_exch{"H232 SUM"}="2H23 SUM"; $resn_exch{"H241 SUM"}="1H24 SUM"; $resn_exch{"H242 SUM"}="2H24 SUM";
$resn_exch{"H251 SUM"}="1H25 SUM"; $resn_exch{"H252 SUM"}="2H25 SUM"; $resn_exch{"H261 SUM"}="1H26 SUM";
$resn_exch{"H262 SUM"}="2H26 SUM"; $resn_exch{"H271 SUM"}="1H27 SUM"; $resn_exch{"H272 SUM"}="2H27 SUM";
$resn_exch{"H281 SUM"}="1H28 SUM"; $resn_exch{"H282 SUM"}="2H28 SUM"; $resn_exch{"H283 SUM"}="3H28 SUM";
$resn_exch{"HCM1 SUM"}="1HCM SUM"; $resn_exch{"HCM2 SUM"}="2HCM SUM"; $resn_exch{" HN1 SUN"}="1HN  SUN";
$resn_exch{" HN2 SUN"}="2HN  SUN"; $resn_exch{"HBC1 SUN"}="1HBC SUN"; $resn_exch{"HBC2 SUN"}="2HBC SUN";
$resn_exch{"H2C1 SUN"}="1H2C SUN"; $resn_exch{"H2C2 SUN"}="2H2C SUN"; $resn_exch{"H2C3 SUN"}="3H2C SUN";
$resn_exch{"H1C1 SUN"}="1H1C SUN"; $resn_exch{"H1C2 SUN"}="2H1C SUN"; $resn_exch{"H1C3 SUN"}="3H1C SUN";
$resn_exch{"H3C1 SUN"}="1H3C SUN"; $resn_exch{"H3C2 SUN"}="2H3C SUN"; $resn_exch{"H4C1 SUN"}="1H4C SUN";
$resn_exch{"H4C2 SUN"}="2H4C SUN"; $resn_exch{"H4C3 SUN"}="3H4C SUN"; $resn_exch{" HB1 SUO"}="1HB  SUO";
$resn_exch{" HB2 SUO"}="2HB  SUO"; $resn_exch{" HG1 SUO"}="1HG  SUO"; $resn_exch{" HG2 SUO"}="2HG  SUO";
$resn_exch{"HD1A SUO"}="1HD  SUO"; $resn_exch{" HD2 SUO"}="2HD  SUO"; $resn_exch{"HNE1 SUO"}="1HNE SUO";
$resn_exch{"HNE2 SUO"}="2HNE SUO"; $resn_exch{" HW1 SUO"}="1HW  SUO"; $resn_exch{" HW2 SUO"}="2HW  SUO";
$resn_exch{" HX1 SUO"}="1HX  SUO"; $resn_exch{" HX2 SUO"}="2HX  SUO"; $resn_exch{" H61 SUP"}="1H6  SUP";
$resn_exch{" H62 SUP"}="2H6  SUP"; $resn_exch{"H1'1 SUP"}="1H1' SUP"; $resn_exch{"H1'2 SUP"}="2H1' SUP";
$resn_exch{"H6'1 SUP"}="1H6' SUP"; $resn_exch{"H6'2 SUP"}="2H6' SUP"; $resn_exch{"HOP1 SUR"}="1HOP SUR";
$resn_exch{"HOP3 SUR"}="3HOP SUR"; $resn_exch{"H5'1 SUR"}="1H5' SUR"; $resn_exch{"H5'2 SUR"}="2H5' SUR";
$resn_exch{"HO21 SUW"}="1HO2 SUW"; $resn_exch{"HO22 SUW"}="2HO2 SUW"; $resn_exch{"HO23 SUW"}="3HO2 SUW";
$resn_exch{"HX31 SUW"}="1HX3 SUW"; $resn_exch{"HX32 SUW"}="2HX3 SUW"; $resn_exch{"H111 SUW"}="1H11 SUW";
$resn_exch{"H112 SUW"}="2H11 SUW"; $resn_exch{"H113 SUW"}="3H11 SUW"; $resn_exch{"HX91 SUW"}="1HX9 SUW";
$resn_exch{"HX92 SUW"}="2HX9 SUW"; $resn_exch{" H2  SVA"}=" HN2 SVA"; $resn_exch{" HB2 SVA"}="1HB  SVA";
$resn_exch{" HB3 SVA"}="2HB  SVA"; $resn_exch{"H271 SVR"}="1H27 SVR"; $resn_exch{"H272 SVR"}="2H27 SVR";
$resn_exch{"H273 SVR"}="3H27 SVR"; $resn_exch{"H591 SVR"}="1H59 SVR"; $resn_exch{"H592 SVR"}="2H59 SVR";
$resn_exch{"H593 SVR"}="3H59 SVR"; $resn_exch{"HBC1 SVV"}="1HBC SVV"; $resn_exch{" HA2 SVV"}="2HA  SVV";
$resn_exch{"H4N1 SVV"}="1H4N SVV"; $resn_exch{"H4N2 SVV"}="2H4N SVV"; $resn_exch{"HBC1 SVW"}="1HBC SVW";
$resn_exch{"HBC2 SVW"}="2HBC SVW"; $resn_exch{"H2N1 SVW"}="1H2N SVW"; $resn_exch{"H2N2 SVW"}="2H2N SVW";
$resn_exch{" HN1 SVX"}="1HN  SVX"; $resn_exch{" HN2 SVX"}="2HN  SVX"; $resn_exch{"HBC1 SVX"}="1HBC SVX";
$resn_exch{"HBC2 SVX"}="2HBC SVX"; $resn_exch{"H4C1 SVX"}="1H4C SVX"; $resn_exch{"H4C2 SVX"}="2H4C SVX";
$resn_exch{"H4C3 SVX"}="3H4C SVX"; $resn_exch{"H2C1 SVX"}="1H2C SVX"; $resn_exch{"H2C2 SVX"}="2H2C SVX";
$resn_exch{"H1C1 SVX"}="1H1C SVX"; $resn_exch{"H1C2 SVX"}="2H1C SVX"; $resn_exch{"H1C3 SVX"}="3H1C SVX";
$resn_exch{" HN1 SVY"}="1HN  SVY"; $resn_exch{" HN2 SVY"}="2HN  SVY"; $resn_exch{"HBC1 SVY"}="1HBC SVY";
$resn_exch{"HBC2 SVY"}="2HBC SVY"; $resn_exch{"H2C1 SVY"}="1H2C SVY"; $resn_exch{"H2C2 SVY"}="2H2C SVY";
$resn_exch{"H2C3 SVY"}="3H2C SVY"; $resn_exch{"H3C1 SVY"}="1H3C SVY"; $resn_exch{"H3C2 SVY"}="2H3C SVY";
$resn_exch{"H3C3 SVY"}="3H3C SVY"; $resn_exch{"H2'1 SVY"}="1H2' SVY"; $resn_exch{"H2'2 SVY"}="2H2' SVY";
$resn_exch{"H2'3 SVY"}="3H2' SVY"; $resn_exch{"H3'1 SVY"}="1H3' SVY"; $resn_exch{"H3'2 SVY"}="2H3' SVY";
$resn_exch{"H3'3 SVY"}="3H3' SVY"; $resn_exch{" HN1 SVZ"}="1HN  SVZ"; $resn_exch{" HN2 SVZ"}="2HN  SVZ";
$resn_exch{"HBC1 SVZ"}="1HBC SVZ"; $resn_exch{"HBC2 SVZ"}="2HBC SVZ"; $resn_exch{"H7C1 SVZ"}="1H7C SVZ";
$resn_exch{"H7C2 SVZ"}="2H7C SVZ"; $resn_exch{"H7C3 SVZ"}="3H7C SVZ"; $resn_exch{"H6C1 SVZ"}="1H6C SVZ";
$resn_exch{"H6C2 SVZ"}="2H6C SVZ"; $resn_exch{"H6C3 SVZ"}="3H6C SVZ"; $resn_exch{"HC51 SWA"}="1HC5 SWA";
$resn_exch{"HC52 SWA"}="2HC5 SWA"; $resn_exch{"HC61 SWA"}="1HC6 SWA"; $resn_exch{"HC62 SWA"}="2HC6 SWA";
$resn_exch{"HC21 SWA"}="1HC2 SWA"; $resn_exch{"HC22 SWA"}="2HC2 SWA"; $resn_exch{"HC91 SWA"}="1HC9 SWA";
$resn_exch{"HC92 SWA"}="2HC9 SWA"; $resn_exch{"H13O SWA"}="OH13 SWA"; $resn_exch{"H11O SWA"}="OH11 SWA";
$resn_exch{"H1C1 SWF"}="1H1C SWF"; $resn_exch{"H1C2 SWF"}="2H1C SWF"; $resn_exch{"H1C3 SWF"}="3H1C SWF";
$resn_exch{"H4C1 SWF"}="1H4C SWF"; $resn_exch{"H4C2 SWF"}="2H4C SWF"; $resn_exch{"H251 SWI"}="1H25 SWI";
$resn_exch{"H252 SWI"}="2H25 SWI"; $resn_exch{"H261 SWI"}="1H26 SWI"; $resn_exch{"H262 SWI"}="2H26 SWI";
$resn_exch{"H321 SWI"}="1H32 SWI"; $resn_exch{"H322 SWI"}="2H32 SWI"; $resn_exch{"H323 SWI"}="3H32 SWI";
$resn_exch{"H301 SWI"}="1H30 SWI"; $resn_exch{"H302 SWI"}="2H30 SWI"; $resn_exch{"H391 SWI"}="1H39 SWI";
$resn_exch{"H392 SWI"}="2H39 SWI"; $resn_exch{"H393 SWI"}="3H39 SWI"; $resn_exch{"H281 SWI"}="1H28 SWI";
$resn_exch{"H282 SWI"}="2H28 SWI"; $resn_exch{"H371 SWI"}="1H37 SWI"; $resn_exch{"H372 SWI"}="2H37 SWI";
$resn_exch{"H373 SWI"}="3H37 SWI"; $resn_exch{"H361 SWI"}="1H36 SWI"; $resn_exch{"H362 SWI"}="2H36 SWI";
$resn_exch{"H363 SWI"}="3H36 SWI"; $resn_exch{"H351 SWI"}="1H35 SWI"; $resn_exch{"H352 SWI"}="2H35 SWI";
$resn_exch{"H353 SWI"}="3H35 SWI"; $resn_exch{"H181 SWI"}="1H18 SWI"; $resn_exch{"H182 SWI"}="2H18 SWI";
$resn_exch{"H341 SWI"}="1H34 SWI"; $resn_exch{"H342 SWI"}="2H34 SWI"; $resn_exch{"H343 SWI"}="3H34 SWI";
$resn_exch{"H381 SWI"}="1H38 SWI"; $resn_exch{"H382 SWI"}="2H38 SWI"; $resn_exch{"H383 SWI"}="3H38 SWI";
$resn_exch{"H141 SWI"}="1H14 SWI"; $resn_exch{"H142 SWI"}="2H14 SWI"; $resn_exch{"H121 SWI"}="1H12 SWI";
$resn_exch{"H122 SWI"}="2H12 SWI"; $resn_exch{" H81 SWI"}="1H8  SWI"; $resn_exch{" H82 SWI"}="2H8  SWI";
$resn_exch{"H61A SWI"}="1H6  SWI"; $resn_exch{"H62A SWI"}="2H6  SWI"; $resn_exch{"H331 SWI"}="1H33 SWI";
$resn_exch{"H332 SWI"}="2H33 SWI"; $resn_exch{"H333 SWI"}="3H33 SWI"; $resn_exch{"H721 SWI"}="1H72 SWI";
$resn_exch{"H722 SWI"}="2H72 SWI"; $resn_exch{"H723 SWI"}="3H72 SWI"; $resn_exch{"H451 SWI"}="1H45 SWI";
$resn_exch{"H452 SWI"}="2H45 SWI"; $resn_exch{"HO12 SWI"}="2HO1 SWI"; $resn_exch{"H471 SWI"}="1H47 SWI";
$resn_exch{"H472 SWI"}="2H47 SWI"; $resn_exch{"H511 SWI"}="1H51 SWI"; $resn_exch{"H512 SWI"}="2H51 SWI";
$resn_exch{"H531 SWI"}="1H53 SWI"; $resn_exch{"H532 SWI"}="2H53 SWI"; $resn_exch{"H771 SWI"}="1H77 SWI";
$resn_exch{"H772 SWI"}="2H77 SWI"; $resn_exch{"H773 SWI"}="3H77 SWI"; $resn_exch{"H731 SWI"}="1H73 SWI";
$resn_exch{"H732 SWI"}="2H73 SWI"; $resn_exch{"H733 SWI"}="3H73 SWI"; $resn_exch{"HO15 SWI"}="5HO1 SWI";
$resn_exch{"H571 SWI"}="1H57 SWI"; $resn_exch{"H572 SWI"}="2H57 SWI"; $resn_exch{"HO16 SWI"}="6HO1 SWI";
$resn_exch{"H741 SWI"}="1H74 SWI"; $resn_exch{"H742 SWI"}="2H74 SWI"; $resn_exch{"H743 SWI"}="3H74 SWI";
$resn_exch{"H751 SWI"}="1H75 SWI"; $resn_exch{"H752 SWI"}="2H75 SWI"; $resn_exch{"H753 SWI"}="3H75 SWI";
$resn_exch{"HO18 SWI"}="8HO1 SWI"; $resn_exch{"H761 SWI"}="1H76 SWI"; $resn_exch{"H762 SWI"}="2H76 SWI";
$resn_exch{"H763 SWI"}="3H76 SWI"; $resn_exch{"H641 SWI"}="1H64 SWI"; $resn_exch{"H642 SWI"}="2H64 SWI";
$resn_exch{"H651 SWI"}="1H65 SWI"; $resn_exch{"H652 SWI"}="2H65 SWI"; $resn_exch{"H711 SWI"}="1H71 SWI";
$resn_exch{"H712 SWI"}="2H71 SWI"; $resn_exch{"H713 SWI"}="3H71 SWI"; $resn_exch{"H691 SWI"}="1H69 SWI";
$resn_exch{"H692 SWI"}="2H69 SWI"; $resn_exch{"H671 SWI"}="1H67 SWI"; $resn_exch{"H672 SWI"}="2H67 SWI";
$resn_exch{"H781 SWI"}="1H78 SWI"; $resn_exch{"H782 SWI"}="2H78 SWI"; $resn_exch{"H783 SWI"}="3H78 SWI";
$resn_exch{" HN1 SXE"}="1HN  SXE"; $resn_exch{" HN2 SXE"}="2HN  SXE"; $resn_exch{"HBC1 SXE"}="1HBC SXE";
$resn_exch{"HBC2 SXE"}="2HBC SXE"; $resn_exch{"H3C1 SXE"}="1H3C SXE"; $resn_exch{"H3C2 SXE"}="2H3C SXE";
$resn_exch{"H3C3 SXE"}="3H3C SXE"; $resn_exch{"H2C1 SXE"}="1H2C SXE"; $resn_exch{"H2C2 SXE"}="2H2C SXE";
$resn_exch{"H2C3 SXE"}="3H2C SXE"; $resn_exch{"H4C1 SXE"}="1H4C SXE"; $resn_exch{"H4C2 SXE"}="2H4C SXE";
$resn_exch{"H5C1 SXE"}="1H5C SXE"; $resn_exch{"H5C2 SXE"}="2H5C SXE"; $resn_exch{"H5C3 SXE"}="3H5C SXE";
$resn_exch{"H151 SXX"}="1H15 SXX"; $resn_exch{"H152 SXX"}="2H15 SXX"; $resn_exch{"H153 SXX"}="3H15 SXX";
$resn_exch{"H141 SXX"}="1H14 SXX"; $resn_exch{"H142 SXX"}="2H14 SXX"; $resn_exch{"H143 SXX"}="3H14 SXX";
$resn_exch{"HN11 SY1"}="1HN1 SY1"; $resn_exch{"HN12 SY1"}="2HN1 SY1"; $resn_exch{" H41 SY1"}="1H4  SY1";
$resn_exch{" H42 SY1"}="2H4  SY1"; $resn_exch{" H51 SY1"}="1H5  SY1"; $resn_exch{" H52 SY1"}="2H5  SY1";
$resn_exch{" H71 SY1"}="1H7  SY1"; $resn_exch{" H72 SY1"}="2H7  SY1"; $resn_exch{"H191 SY1"}="1H19 SY1";
$resn_exch{"H192 SY1"}="2H19 SY1"; $resn_exch{" H31 SY1"}="1H3  SY1"; $resn_exch{" H32 SY1"}="2H3  SY1";
$resn_exch{"H31A SYB"}="1H3  SYB"; $resn_exch{"H32A SYB"}="2H3  SYB"; $resn_exch{" H41 SYB"}="1H4  SYB";
$resn_exch{"H42A SYB"}="2H4  SYB"; $resn_exch{"H43A SYB"}="3H4  SYB"; $resn_exch{" H71 SYB"}="1H7  SYB";
$resn_exch{" H72 SYB"}="2H7  SYB"; $resn_exch{" H81 SYB"}="1H8  SYB"; $resn_exch{" H82 SYB"}="2H8  SYB";
$resn_exch{" H91 SYB"}="1H9  SYB"; $resn_exch{" H92 SYB"}="2H9  SYB"; $resn_exch{"H121 SYB"}="1H12 SYB";
$resn_exch{"H122 SYB"}="2H12 SYB"; $resn_exch{"H123 SYB"}="3H12 SYB"; $resn_exch{"H131 SYB"}="1H13 SYB";
$resn_exch{"H132 SYB"}="2H13 SYB"; $resn_exch{"H201 SYB"}="1H20 SYB"; $resn_exch{"H202 SYB"}="2H20 SYB";
$resn_exch{"H203 SYB"}="3H20 SYB"; $resn_exch{"H211 SYB"}="1H21 SYB"; $resn_exch{"H212 SYB"}="2H21 SYB";
$resn_exch{"H213 SYB"}="3H21 SYB"; $resn_exch{"H241 SYB"}="1H24 SYB"; $resn_exch{"H242 SYB"}="2H24 SYB";
$resn_exch{"H271 SYB"}="1H27 SYB"; $resn_exch{"H272 SYB"}="2H27 SYB"; $resn_exch{"H391 SYB"}="1H39 SYB";
$resn_exch{"H392 SYB"}="2H39 SYB"; $resn_exch{"H393 SYB"}="3H39 SYB"; $resn_exch{"H811 SYB"}="1H81 SYB";
$resn_exch{"H812 SYB"}="2H81 SYB"; $resn_exch{"H911 SYB"}="1H91 SYB"; $resn_exch{"H912 SYB"}="2H91 SYB";
$resn_exch{"H921 SYB"}="1H92 SYB"; $resn_exch{"H922 SYB"}="2H92 SYB"; $resn_exch{"H941 SYB"}="1H94 SYB";
$resn_exch{"H942 SYB"}="2H94 SYB"; $resn_exch{"H951 SYB"}="1H95 SYB"; $resn_exch{"H952 SYB"}="2H95 SYB";
$resn_exch{"H961 SYB"}="1H96 SYB"; $resn_exch{"H962 SYB"}="2H96 SYB"; $resn_exch{" HB1 SYM"}="1HB  SYM";
$resn_exch{" HB2 SYM"}="2HB  SYM"; $resn_exch{"HG21 SYM"}="1HG2 SYM"; $resn_exch{"HG22 SYM"}="2HG2 SYM";
$resn_exch{"HG23 SYM"}="3HG2 SYM"; $resn_exch{" HN1 SYM"}="1HN  SYM"; $resn_exch{" HN2 SYM"}="2HN  SYM";
$resn_exch{"H101 SYR"}="1H10 SYR"; $resn_exch{"H102 SYR"}="2H10 SYR"; $resn_exch{"H103 SYR"}="3H10 SYR";
$resn_exch{"H7C1 SYR"}="1H7C SYR"; $resn_exch{"H7C2 SYR"}="2H7C SYR"; $resn_exch{"H7C3 SYR"}="3H7C SYR";
$resn_exch{"H9C1 SYR"}="1H9C SYR"; $resn_exch{"H9C2 SYR"}="2H9C SYR"; $resn_exch{"H9C3 SYR"}="3H9C SYR";
$resn_exch{"HOP3   T"}="3HOP   T"; $resn_exch{"HOP2   T"}="2HOP   T"; $resn_exch{" H5'   T"}="1H5*   T";
$resn_exch{"H5''   T"}="2H5*   T"; $resn_exch{" H4'   T"}=" H4*   T"; $resn_exch{" H3'   T"}=" H3*   T";
$resn_exch{"HO3'   T"}="H3T    T"; $resn_exch{" H2'   T"}="1H2*   T"; $resn_exch{"H2''   T"}="2H2*   T";
$resn_exch{" H1'   T"}=" H1*   T"; $resn_exch{" H71   T"}="1H5M   T"; $resn_exch{" H72   T"}="2H5M   T";
$resn_exch{" H73   T"}="3H5M   T"; $resn_exch{"HC61 T10"}="1HC6 T10"; $resn_exch{"HC62 T10"}="2HC6 T10";
$resn_exch{"H101 T10"}="1H10 T10"; $resn_exch{"H102 T10"}="2H10 T10"; $resn_exch{"H211 T10"}="1H21 T10";
$resn_exch{"H212 T10"}="2H21 T10"; $resn_exch{"H321 T10"}="1H32 T10"; $resn_exch{"H322 T10"}="2H32 T10";
$resn_exch{" H71 T11"}="1H7  T11"; $resn_exch{" H72 T11"}="2H7  T11"; $resn_exch{"H351 T12"}="1H35 T12";
$resn_exch{"H352 T12"}="2H35 T12"; $resn_exch{"H353 T12"}="3H35 T12"; $resn_exch{"H311 T12"}="1H31 T12";
$resn_exch{"H312 T12"}="2H31 T12"; $resn_exch{"H313 T12"}="3H31 T12"; $resn_exch{"H271 T12"}="1H27 T12";
$resn_exch{"H272 T12"}="2H27 T12"; $resn_exch{"H273 T12"}="3H27 T12"; $resn_exch{"H391 T12"}="1H39 T12";
$resn_exch{"H392 T12"}="2H39 T12"; $resn_exch{"H401 T12"}="1H40 T12"; $resn_exch{"H402 T12"}="2H40 T12";
$resn_exch{"H431 T12"}="1H43 T12"; $resn_exch{"H432 T12"}="2H43 T12"; $resn_exch{" H21 T16"}="1H2  T16";
$resn_exch{"H22A T16"}="2H2  T16"; $resn_exch{" H31 T16"}="1H3  T16"; $resn_exch{" H32 T16"}="2H3  T16";
$resn_exch{"HO1A T16"}="AHO1 T16"; $resn_exch{"HO1B T16"}="BHO1 T16"; $resn_exch{"H7A1 T16"}="1H7A T16";
$resn_exch{"H7A2 T16"}="2H7A T16"; $resn_exch{"H7B1 T16"}="1H7B T16"; $resn_exch{"H7B2 T16"}="2H7B T16";
$resn_exch{"H7C1 T16"}="1H7C T16"; $resn_exch{"H7C2 T16"}="2H7C T16"; $resn_exch{"H111 T16"}="1H11 T16";
$resn_exch{"H112 T16"}="2H11 T16"; $resn_exch{"H141 T16"}="1H14 T16"; $resn_exch{"H142 T16"}="2H14 T16";
$resn_exch{"H151 T16"}="1H15 T16"; $resn_exch{"H152 T16"}="2H15 T16"; $resn_exch{"H161 T16"}="1H16 T16";
$resn_exch{"H162 T16"}="2H16 T16"; $resn_exch{"H11A T19"}="1H1  T19"; $resn_exch{" H12 T19"}="2H1  T19";
$resn_exch{" H13 T19"}="3H1  T19"; $resn_exch{" H21 T19"}="1H2  T19"; $resn_exch{"H22A T19"}="2H2  T19";
$resn_exch{"H23A T19"}="3H2  T19"; $resn_exch{"H151 T19"}="1H15 T19"; $resn_exch{"H152 T19"}="2H15 T19";
$resn_exch{" H11 T1A"}="1H1  T1A"; $resn_exch{" H12 T1A"}="2H1  T1A"; $resn_exch{" H21 T1A"}="1H2  T1A";
$resn_exch{" H22 T1A"}="2H2  T1A"; $resn_exch{" H23 T1A"}="3H2  T1A"; $resn_exch{" H31 T1A"}="1H3  T1A";
$resn_exch{" H32 T1A"}="2H3  T1A"; $resn_exch{" H41 T1A"}="1H4  T1A"; $resn_exch{" H42 T1A"}="2H4  T1A";
$resn_exch{" H43 T1A"}="3H4  T1A"; $resn_exch{" H51 T1A"}="1H5  T1A"; $resn_exch{" H52 T1A"}="2H5  T1A";
$resn_exch{" H61 T1A"}="1H6  T1A"; $resn_exch{" H62 T1A"}="2H6  T1A"; $resn_exch{" H63 T1A"}="3H6  T1A";
$resn_exch{" H71 T1A"}="1H7  T1A"; $resn_exch{" H72 T1A"}="2H7  T1A"; $resn_exch{" H81 T1A"}="1H8  T1A";
$resn_exch{" H82 T1A"}="2H8  T1A"; $resn_exch{" H83 T1A"}="3H8  T1A"; $resn_exch{"H11A T1D"}="1H1  T1D";
$resn_exch{"H8C1 T1D"}="1H8C T1D"; $resn_exch{"H8C2 T1D"}="2H8C T1D"; $resn_exch{"H8C3 T1D"}="3H8C T1D";
$resn_exch{"H171 T1P"}="1H17 T1P"; $resn_exch{"H172 T1P"}="2H17 T1P"; $resn_exch{"H161 T1P"}="1H16 T1P";
$resn_exch{"H162 T1P"}="2H16 T1P"; $resn_exch{"H151 T1P"}="1H15 T1P"; $resn_exch{"H152 T1P"}="2H15 T1P";
$resn_exch{"H101 T1P"}="1H10 T1P"; $resn_exch{"H102 T1P"}="2H10 T1P"; $resn_exch{"H141 T1P"}="1H14 T1P";
$resn_exch{"H142 T1P"}="2H14 T1P"; $resn_exch{"HOP2 T23"}="2HOP T23"; $resn_exch{"HOP3 T23"}="3HOP T23";
$resn_exch{"H5A1 T23"}="1H5A T23"; $resn_exch{"H5A2 T23"}="2H5A T23"; $resn_exch{"H5A3 T23"}="3H5A T23";
$resn_exch{" H2' T23"}=" H2* T23"; $resn_exch{"H2M1 T23"}="1H2M T23"; $resn_exch{"H2M2 T23"}="2H2M T23";
$resn_exch{"H2M3 T23"}="3H2M T23"; $resn_exch{" H5' T23"}="1H5* T23"; $resn_exch{"H5'' T23"}="2H5* T23";
$resn_exch{" H4' T23"}=" H4* T23"; $resn_exch{" H1' T23"}=" H1* T23"; $resn_exch{" H3' T23"}=" H3* T23";
$resn_exch{"H3M1 T23"}="1H3M T23"; $resn_exch{"H3M2 T23"}="2H3M T23"; $resn_exch{"H3M3 T23"}="3H3M T23";
$resn_exch{"HN11 T29"}="1HN1 T29"; $resn_exch{"HN12 T29"}="2HN1 T29"; $resn_exch{" H31 T29"}="1H3  T29";
$resn_exch{" H32 T29"}="2H3  T29"; $resn_exch{" H41 T29"}="1H4  T29"; $resn_exch{" H42 T29"}="2H4  T29";
$resn_exch{" H51 T29"}="1H5  T29"; $resn_exch{" H52 T29"}="2H5  T29"; $resn_exch{"HO1A T29"}="AHO1 T29";
$resn_exch{"HO1B T29"}="BHO1 T29"; $resn_exch{"H9A1 T29"}="1H9A T29"; $resn_exch{"H9A2 T29"}="2H9A T29";
$resn_exch{"H9B1 T29"}="1H9B T29"; $resn_exch{"H9B2 T29"}="2H9B T29"; $resn_exch{"H9C1 T29"}="1H9C T29";
$resn_exch{"H9C2 T29"}="2H9C T29"; $resn_exch{"H131 T29"}="1H13 T29"; $resn_exch{"H132 T29"}="2H13 T29";
$resn_exch{"H161 T29"}="1H16 T29"; $resn_exch{"H162 T29"}="2H16 T29"; $resn_exch{"H171 T29"}="1H17 T29";
$resn_exch{"H172 T29"}="2H17 T29"; $resn_exch{"H173 T29"}="3H17 T29"; $resn_exch{"H111 T2D"}="1H11 T2D";
$resn_exch{"H112 T2D"}="2H11 T2D"; $resn_exch{" H11 T2M"}="1H1  T2M"; $resn_exch{" H12 T2M"}="2H1  T2M";
$resn_exch{" H13 T2M"}="3H1  T2M"; $resn_exch{" H81 T2M"}="1H8  T2M"; $resn_exch{" H82 T2M"}="2H8  T2M";
$resn_exch{" H83 T2M"}="3H8  T2M"; $resn_exch{" H91 T2M"}="1H9  T2M"; $resn_exch{" H92 T2M"}="2H9  T2M";
$resn_exch{" H93 T2M"}="3H9  T2M"; $resn_exch{"H101 T2M"}="1H10 T2M"; $resn_exch{"H102 T2M"}="2H10 T2M";
$resn_exch{"H103 T2M"}="3H10 T2M"; $resn_exch{"H171 T2P"}="1H17 T2P"; $resn_exch{"H172 T2P"}="2H17 T2P";
$resn_exch{"H161 T2P"}="1H16 T2P"; $resn_exch{"H162 T2P"}="2H16 T2P"; $resn_exch{"H151 T2P"}="1H15 T2P";
$resn_exch{"H152 T2P"}="2H15 T2P"; $resn_exch{"H101 T2P"}="1H10 T2P"; $resn_exch{"H102 T2P"}="2H10 T2P";
$resn_exch{"H141 T2P"}="1H14 T2P"; $resn_exch{"H142 T2P"}="2H14 T2P"; $resn_exch{"HOP3 T2S"}="3HOP T2S";
$resn_exch{"HOP2 T2S"}="2HOP T2S"; $resn_exch{" H5' T2S"}="1H5* T2S"; $resn_exch{"H5'' T2S"}="2H5* T2S";
$resn_exch{" H4' T2S"}=" H4* T2S"; $resn_exch{" H3' T2S"}=" H3* T2S"; $resn_exch{"HO3' T2S"}="H3T  T2S";
$resn_exch{" H2' T2S"}="1H2* T2S"; $resn_exch{"H2'' T2S"}="2H2* T2S"; $resn_exch{" H1' T2S"}=" H1* T2S";
$resn_exch{" H71 T2S"}="1H5M T2S"; $resn_exch{" H72 T2S"}="2H5M T2S"; $resn_exch{" H73 T2S"}="3H5M T2S";
$resn_exch{"HOP2 T2T"}="2HOP T2T"; $resn_exch{" H5' T2T"}="1H5* T2T"; $resn_exch{"H5'' T2T"}="2H5* T2T";
$resn_exch{"HAB1 T2T"}="1HAB T2T"; $resn_exch{"HAB2 T2T"}="2HAB T2T"; $resn_exch{"HAB3 T2T"}="3HAB T2T";
$resn_exch{"H2X1 T2T"}="1H2X T2T"; $resn_exch{"H2X2 T2T"}="2H2X T2T"; $resn_exch{"HH21 T2T"}="1HH2 T2T";
$resn_exch{"HH22 T2T"}="2HH2 T2T"; $resn_exch{"H2P1 T2T"}="1H2P T2T"; $resn_exch{"H2P2 T2T"}="2H2P T2T";
$resn_exch{"H2P3 T2T"}="3H2P T2T"; $resn_exch{"H5X1 T2T"}="1H5X T2T"; $resn_exch{"H5X2 T2T"}="2H5X T2T";
$resn_exch{" H4' T2T"}=" H4* T2T"; $resn_exch{" H1' T2T"}=" H1* T2T"; $resn_exch{"H5A1 T2T"}="1H5A T2T";
$resn_exch{"H5A2 T2T"}="2H5A T2T"; $resn_exch{"H5A3 T2T"}="3H5A T2T"; $resn_exch{" H2' T2T"}=" H2* T2T";
$resn_exch{"HH31 T2T"}="1HH3 T2T"; $resn_exch{"HH32 T2T"}="2HH3 T2T"; $resn_exch{"HH33 T2T"}="3HH3 T2T";
$resn_exch{" H3' T2T"}=" H3* T2T"; $resn_exch{"HO3' T2T"}="H3T  T2T"; $resn_exch{"HOP3 T2T"}="3HOP T2T";
$resn_exch{"H131  T3"}="1C13  T3"; $resn_exch{"H132  T3"}="2C13  T3"; $resn_exch{"HN11  T3"}="1HN1  T3";
$resn_exch{"HN12  T3"}="2HN1  T3"; $resn_exch{"H5'1 T31"}="1H5* T31"; $resn_exch{"H5'2 T31"}="2H5* T31";
$resn_exch{" H4' T31"}=" H4* T31"; $resn_exch{" H3' T31"}=" H3* T31"; $resn_exch{" H2' T31"}=" H2* T31";
$resn_exch{" H1' T31"}=" H1* T31"; $resn_exch{"H5'1 T32"}="1H5* T32"; $resn_exch{"H5'2 T32"}="2H5* T32";
$resn_exch{" H4' T32"}=" H4* T32"; $resn_exch{" H3' T32"}=" H3* T32"; $resn_exch{"H2'1 T32"}="1H2* T32";
$resn_exch{"H2'2 T32"}="2H2* T32"; $resn_exch{" H1' T32"}=" H1* T32"; $resn_exch{"H5M1 T32"}="1H5M T32";
$resn_exch{"H5M2 T32"}="2H5M T32"; $resn_exch{"H5M3 T32"}="3H5M T32"; $resn_exch{" H'  T32"}=" H*  T32";
$resn_exch{"HM'1 T32"}="1HM* T32"; $resn_exch{"HM'2 T32"}="2HM* T32"; $resn_exch{"HM'3 T32"}="3HM* T32";
$resn_exch{" H71 T33"}="1H7  T33"; $resn_exch{" H72 T33"}="2H7  T33"; $resn_exch{"HN81 T33"}="1HN8 T33";
$resn_exch{"HN82 T33"}="2HN8 T33"; $resn_exch{"HO'4 T33"}="4HO' T33"; $resn_exch{"H5'1 T36"}="1H5* T36";
$resn_exch{"H5'2 T36"}="2H5* T36"; $resn_exch{" H4' T36"}=" H4* T36"; $resn_exch{" H3' T36"}=" H3* T36";
$resn_exch{"H2'1 T36"}="1H2* T36"; $resn_exch{"H2'2 T36"}="2H2* T36"; $resn_exch{" H1' T36"}=" H1* T36";
$resn_exch{"H5M1 T36"}="1H5M T36"; $resn_exch{"H5M2 T36"}="2H5M T36"; $resn_exch{"H5M3 T36"}="3H5M T36";
$resn_exch{"H5'1 T37"}="1H5* T37"; $resn_exch{"H5'2 T37"}="2H5* T37"; $resn_exch{" H4' T37"}=" H4* T37";
$resn_exch{" H3' T37"}=" H3* T37"; $resn_exch{"HN31 T37"}="1HN3 T37"; $resn_exch{"HN32 T37"}="2HN3 T37";
$resn_exch{"H2'1 T37"}="1H2* T37"; $resn_exch{"H2'2 T37"}="2H2* T37"; $resn_exch{" H1' T37"}=" H1* T37";
$resn_exch{"H5A1 T37"}="1H5A T37"; $resn_exch{"H5A2 T37"}="2H5A T37"; $resn_exch{"H5A3 T37"}="3H5A T37";
$resn_exch{"H5'1 T38"}="1H5* T38"; $resn_exch{"H5'2 T38"}="2H5* T38"; $resn_exch{" H4' T38"}=" H4* T38";
$resn_exch{" H3' T38"}=" H3* T38"; $resn_exch{" H2' T38"}=" H2* T38"; $resn_exch{" H1' T38"}=" H1* T38";
$resn_exch{"H5M1 T38"}="1H5M T38"; $resn_exch{"H5M2 T38"}="2H5M T38"; $resn_exch{"H5M3 T38"}="3H5M T38";
$resn_exch{"HA'1 T38"}="1HA* T38"; $resn_exch{"HA'2 T38"}="2HA* T38"; $resn_exch{"HB'1 T38"}="1HB* T38";
$resn_exch{"HB'2 T38"}="2HB* T38"; $resn_exch{"HD'1 T38"}="1HD* T38"; $resn_exch{"HD'2 T38"}="2HD* T38";
$resn_exch{"HD'3 T38"}="3HD* T38"; $resn_exch{"H5'1 T39"}="1H5* T39"; $resn_exch{"H5'2 T39"}="2H5* T39";
$resn_exch{" H4' T39"}=" H4* T39"; $resn_exch{" H3' T39"}=" H3* T39"; $resn_exch{" H2' T39"}=" H2* T39";
$resn_exch{" H1' T39"}=" H1* T39"; $resn_exch{"H5M1 T39"}="1H5M T39"; $resn_exch{"H5M2 T39"}="2H5M T39";
$resn_exch{"H5M3 T39"}="3H5M T39"; $resn_exch{"HA'1 T39"}="1HA* T39"; $resn_exch{"HA'2 T39"}="2HA* T39";
$resn_exch{"HD'1 T39"}="1HD* T39"; $resn_exch{"HD'2 T39"}="2HD* T39"; $resn_exch{"HD'3 T39"}="3HD* T39";
$resn_exch{"HB'1 T39"}="1HB* T39"; $resn_exch{"HB'2 T39"}="2HB* T39"; $resn_exch{" H11 T3A"}="1H1  T3A";
$resn_exch{" H12 T3A"}="2H1  T3A"; $resn_exch{" H21 T3A"}="1H2  T3A"; $resn_exch{" H22 T3A"}="2H2  T3A";
$resn_exch{" H31 T3A"}="1H3  T3A"; $resn_exch{" H32 T3A"}="2H3  T3A"; $resn_exch{" H51 T3A"}="1H5  T3A";
$resn_exch{" H52 T3A"}="2H5  T3A"; $resn_exch{" H61 T3A"}="1H6  T3A"; $resn_exch{" H62 T3A"}="2H6  T3A";
$resn_exch{" H71 T3A"}="1H7  T3A"; $resn_exch{" H72 T3A"}="2H7  T3A"; $resn_exch{"HAN1 T3O"}="1HAN T3O";
$resn_exch{"HAN2 T3O"}="2HAN T3O"; $resn_exch{"HAL1 T3O"}="1HAL T3O"; $resn_exch{"HAL2 T3O"}="2HAL T3O";
$resn_exch{"HAL3 T3O"}="3HAL T3O"; $resn_exch{"HAM1 T3O"}="1HAM T3O"; $resn_exch{"HAM2 T3O"}="2HAM T3O";
$resn_exch{"HAF1 T3O"}="1HAF T3O"; $resn_exch{"HAF2 T3O"}="2HAF T3O"; $resn_exch{"HAF3 T3O"}="3HAF T3O";
$resn_exch{"HAQ1 T3O"}="1HAQ T3O"; $resn_exch{"HAQ2 T3O"}="2HAQ T3O"; $resn_exch{"HAQ3 T3O"}="3HAQ T3O";
$resn_exch{"HOP2 T3P"}="2HOP T3P"; $resn_exch{"HOP3 T3P"}="3HOP T3P"; $resn_exch{" H5' T3P"}=" H5* T3P";
$resn_exch{"H5'1 T3P"}="1H5* T3P"; $resn_exch{"H5'' T3P"}="2H5* T3P"; $resn_exch{" H4' T3P"}=" H4* T3P";
$resn_exch{" H3' T3P"}=" H3* T3P"; $resn_exch{" H2' T3P"}="1H2* T3P"; $resn_exch{"H2'' T3P"}="2H2* T3P";
$resn_exch{" H1' T3P"}=" H1* T3P"; $resn_exch{" H51 T3P"}="1H5  T3P"; $resn_exch{" H52 T3P"}="2H5  T3P";
$resn_exch{" H53 T3P"}="3H5  T3P"; $resn_exch{"H5M1 T41"}="1H5M T41"; $resn_exch{"H5M2 T41"}="2H5M T41";
$resn_exch{"H5M3 T41"}="3H5M T41"; $resn_exch{" H1' T41"}=" H1* T41"; $resn_exch{"H2'1 T41"}="1H2* T41";
$resn_exch{" H3' T41"}=" H3* T41"; $resn_exch{" H4' T41"}=" H4* T41"; $resn_exch{"H5'1 T41"}="1H5* T41";
$resn_exch{"H5'2 T41"}="2H5* T41"; $resn_exch{" H11 T41"}="1H1  T41"; $resn_exch{" H12 T41"}="2H1  T41";
$resn_exch{" H13 T41"}="3H1  T41"; $resn_exch{"H11A T42"}="1H1  T42"; $resn_exch{"H12A T42"}="2H1  T42";
$resn_exch{" H13 T42"}="3H1  T42"; $resn_exch{" H21 T42"}="1H2  T42"; $resn_exch{"H22A T42"}="2H2  T42";
$resn_exch{" H31 T42"}="1H3  T42"; $resn_exch{"H32A T42"}="2H3  T42"; $resn_exch{" H41 T42"}="1H4  T42";
$resn_exch{" H42 T42"}="2H4  T42"; $resn_exch{"H421 T42"}="1H42 T42"; $resn_exch{"H422 T42"}="2H42 T42";
$resn_exch{"H431 T42"}="1H43 T42"; $resn_exch{"H432 T42"}="2H43 T42"; $resn_exch{"H451 T42"}="1H45 T42";
$resn_exch{"H452 T42"}="2H45 T42"; $resn_exch{"H461 T42"}="1H46 T42"; $resn_exch{"H462 T42"}="2H46 T42";
$resn_exch{" H71 T44"}="1H7  T44"; $resn_exch{" H72 T44"}="2H7  T44"; $resn_exch{"HN81 T44"}="1HN8 T44";
$resn_exch{"HN82 T44"}="2HN8 T44"; $resn_exch{"HO4' T44"}="'HO4 T44"; $resn_exch{"H4C1 T45"}="1H4C T45";
$resn_exch{"H4C2 T45"}="2H4C T45"; $resn_exch{"H4C3 T45"}="3H4C T45"; $resn_exch{"H5'1 T48"}="1H5* T48";
$resn_exch{"H5'2 T48"}="2H5* T48"; $resn_exch{" H4' T48"}=" H4* T48"; $resn_exch{" H3' T48"}=" H3* T48";
$resn_exch{"H2'1 T48"}="1H2* T48"; $resn_exch{"H2'2 T48"}="2H2* T48"; $resn_exch{" H1' T48"}=" H1* T48";
$resn_exch{"H5M1 T48"}="1H5M T48"; $resn_exch{"H5M2 T48"}="2H5M T48"; $resn_exch{"H5M3 T48"}="3H5M T48";
$resn_exch{"H5'1 T49"}="1H5* T49"; $resn_exch{"H5'2 T49"}="2H5* T49"; $resn_exch{" H4' T49"}=" H4* T49";
$resn_exch{" H3' T49"}=" H3* T49"; $resn_exch{"H2'1 T49"}="1H2* T49"; $resn_exch{"H2'2 T49"}="2H2* T49";
$resn_exch{" H1' T49"}=" H1* T49"; $resn_exch{"H5M1 T49"}="1H5M T49"; $resn_exch{"H5M2 T49"}="2H5M T49";
$resn_exch{"H5M3 T49"}="3H5M T49"; $resn_exch{" H21 T4A"}="1H2  T4A"; $resn_exch{" H61 T4A"}="1H6  T4A";
$resn_exch{" H71 T4A"}="1H7  T4A"; $resn_exch{" H72 T4A"}="2H7  T4A"; $resn_exch{"H6'1 T4A"}="1H6' T4A";
$resn_exch{"H2'1 T4A"}="1H2' T4A"; $resn_exch{"HO91 T4A"}="1HO9 T4A"; $resn_exch{"H4'O T4A"}="OH4' T4A";
$resn_exch{" H91 T4B"}="1H9  T4B"; $resn_exch{" H92 T4B"}="2H9  T4B"; $resn_exch{"H211 T4B"}="1H21 T4B";
$resn_exch{"H212 T4B"}="2H21 T4B"; $resn_exch{"H151 T4B"}="1H15 T4B"; $resn_exch{"H152 T4B"}="2H15 T4B";
$resn_exch{"H153 T4B"}="3H15 T4B"; $resn_exch{"H2A1 T4K"}="1H2A T4K"; $resn_exch{"H2A2 T4K"}="2H2A T4K";
$resn_exch{"H2A3 T4K"}="3H2A T4K"; $resn_exch{"H4A1 T4K"}="1H4A T4K"; $resn_exch{"H5B1 T4K"}="1H5B T4K";
$resn_exch{"H5B2 T4K"}="2H5B T4K"; $resn_exch{"H6G1 T4K"}="1H6G T4K"; $resn_exch{"H6G2 T4K"}="2H6G T4K";
$resn_exch{"H6G3 T4K"}="3H6G T4K"; $resn_exch{" H51 T4K"}="1H5  T4K"; $resn_exch{" H52 T4K"}="2H5  T4K";
$resn_exch{" H21 T4K"}="1H2  T4K"; $resn_exch{" H22 T4K"}="2H2  T4K"; $resn_exch{"H5A1 T4K"}="1H5A T4K";
$resn_exch{"H5A2 T4K"}="2H5A T4K"; $resn_exch{"H5A3 T4K"}="3H5A T4K"; $resn_exch{"H171 T4P"}="1H17 T4P";
$resn_exch{"H172 T4P"}="2H17 T4P"; $resn_exch{"H161 T4P"}="1H16 T4P"; $resn_exch{"H162 T4P"}="2H16 T4P";
$resn_exch{"H151 T4P"}="1H15 T4P"; $resn_exch{"H152 T4P"}="2H15 T4P"; $resn_exch{"H101 T4P"}="1H10 T4P";
$resn_exch{"H102 T4P"}="2H10 T4P"; $resn_exch{"H141 T4P"}="1H14 T4P"; $resn_exch{"H142 T4P"}="2H14 T4P";
$resn_exch{"HOP3 T4S"}="3HOP T4S"; $resn_exch{"HOP2 T4S"}="2HOP T4S"; $resn_exch{"H5A1 T4S"}="1H5A T4S";
$resn_exch{"H5A2 T4S"}="2H5A T4S"; $resn_exch{"H5A3 T4S"}="3H5A T4S"; $resn_exch{" H2' T4S"}="1H2* T4S";
$resn_exch{"H2'' T4S"}="2H2* T4S"; $resn_exch{" H5' T4S"}="1H5* T4S"; $resn_exch{"H5'' T4S"}="2H5* T4S";
$resn_exch{" H4' T4S"}=" H4* T4S"; $resn_exch{" H1' T4S"}=" H1* T4S"; $resn_exch{" H3' T4S"}=" H3* T4S";
$resn_exch{"HO3' T4S"}="H3T  T4S"; $resn_exch{"H4A1 T50"}="1H4A T50"; $resn_exch{"H4A2 T50"}="2H4A T50";
$resn_exch{"H4A3 T50"}="3H4A T50"; $resn_exch{"H101 T50"}="1H10 T50"; $resn_exch{"H102 T50"}="2H10 T50";
$resn_exch{"HB71 T5A"}="1HB7 T5A"; $resn_exch{"HB72 T5A"}="2HB7 T5A"; $resn_exch{"HB73 T5A"}="3HB7 T5A";
$resn_exch{"HB51 T5A"}="1HB5 T5A"; $resn_exch{"HB52 T5A"}="2HB5 T5A"; $resn_exch{"HB21 T5A"}="1HB2 T5A";
$resn_exch{"HB22 T5A"}="2HB2 T5A"; $resn_exch{"HB'3 T5A"}="3HB* T5A"; $resn_exch{"HO1A T5A"}="AHO1 T5A";
$resn_exch{"HO2B T5A"}="BHO2 T5A"; $resn_exch{"HO2C T5A"}="CHO2 T5A"; $resn_exch{"HO1D T5A"}="DHO1 T5A";
$resn_exch{"HO1E T5A"}="EHO1 T5A"; $resn_exch{"HA51 T5A"}="1HA5 T5A"; $resn_exch{"HA52 T5A"}="2HA5 T5A";
$resn_exch{"HA61 T5A"}="1HA6 T5A"; $resn_exch{"HA62 T5A"}="2HA6 T5A"; $resn_exch{"H131 T5C"}="1H13 T5C";
$resn_exch{"H132 T5C"}="2H13 T5C"; $resn_exch{"H121 T5C"}="1H12 T5C"; $resn_exch{"H122 T5C"}="2H12 T5C";
$resn_exch{"H111 T5C"}="1H11 T5C"; $resn_exch{"H112 T5C"}="2H11 T5C"; $resn_exch{"H113 T5C"}="3H11 T5C";
$resn_exch{"H101 T5C"}="1H10 T5C"; $resn_exch{"H102 T5C"}="2H10 T5C"; $resn_exch{"H103 T5C"}="3H10 T5C";
$resn_exch{"H341 T5C"}="1H34 T5C"; $resn_exch{"H342 T5C"}="2H34 T5C"; $resn_exch{"H343 T5C"}="3H34 T5C";
$resn_exch{"H191 T5C"}="1H19 T5C"; $resn_exch{"H201 T5C"}="1H20 T5C"; $resn_exch{"H311 T5C"}="1H31 T5C";
$resn_exch{"H312 T5C"}="2H31 T5C"; $resn_exch{"H313 T5C"}="3H31 T5C"; $resn_exch{"H301 T5C"}="1H30 T5C";
$resn_exch{"H302 T5C"}="2H30 T5C"; $resn_exch{"H303 T5C"}="3H30 T5C"; $resn_exch{"H321 T5C"}="1H32 T5C";
$resn_exch{"H322 T5C"}="2H32 T5C"; $resn_exch{"H331 T5C"}="1H33 T5C"; $resn_exch{"H332 T5C"}="2H33 T5C";
$resn_exch{"H171 T5P"}="1H17 T5P"; $resn_exch{"H172 T5P"}="2H17 T5P"; $resn_exch{"H161 T5P"}="1H16 T5P";
$resn_exch{"H162 T5P"}="2H16 T5P"; $resn_exch{"H151 T5P"}="1H15 T5P"; $resn_exch{"H152 T5P"}="2H15 T5P";
$resn_exch{"H101 T5P"}="1H10 T5P"; $resn_exch{"H102 T5P"}="2H10 T5P"; $resn_exch{"H141 T5P"}="1H14 T5P";
$resn_exch{"H142 T5P"}="2H14 T5P"; $resn_exch{" H71 T5X"}="1H7  T5X"; $resn_exch{" H72 T5X"}="2H7  T5X";
$resn_exch{" H61 T5X"}="1H6  T5X"; $resn_exch{" H62 T5X"}="2H6  T5X"; $resn_exch{"HX51 T5X"}="1HX5 T5X";
$resn_exch{"HX52 T5X"}="2HX5 T5X"; $resn_exch{"HX11 T5X"}="1HX1 T5X"; $resn_exch{"HX12 T5X"}="2HX1 T5X";
$resn_exch{"HM41 T5X"}="1HM4 T5X"; $resn_exch{"HM42 T5X"}="2HM4 T5X"; $resn_exch{"HM43 T5X"}="3HM4 T5X";
$resn_exch{"H7,1 T5X"}="1H7, T5X"; $resn_exch{"H7,2 T5X"}="2H7, T5X"; $resn_exch{"H4,1 T5X"}="1H4, T5X";
$resn_exch{"H4,2 T5X"}="2H4, T5X"; $resn_exch{"HM21 T5X"}="1HM2 T5X"; $resn_exch{"HM22 T5X"}="2HM2 T5X";
$resn_exch{"HM23 T5X"}="3HM2 T5X"; $resn_exch{"H8'1 T66"}="1H8' T66"; $resn_exch{"H8'2 T66"}="2H8' T66";
$resn_exch{"H5'1 T66"}="1H5' T66"; $resn_exch{"H3'1 T66"}="1H3' T66"; $resn_exch{"H3'2 T66"}="2H3' T66";
$resn_exch{"H2'1 T66"}="1H2' T66"; $resn_exch{"H2'2 T66"}="2H2' T66"; $resn_exch{"H1'1 T66"}="1H1' T66";
$resn_exch{"H1'2 T66"}="2H1' T66"; $resn_exch{"HM51 T66"}="1HM5 T66"; $resn_exch{"HM52 T66"}="2HM5 T66";
$resn_exch{"HM53 T66"}="3HM5 T66"; $resn_exch{" HB1 T66"}="1HB  T66"; $resn_exch{" HB2 T66"}="2HB  T66";
$resn_exch{" HG1 T66"}="1HG  T66"; $resn_exch{" HG2 T66"}="2HG  T66"; $resn_exch{" HD1 T66"}="1HD  T66";
$resn_exch{" HD2 T66"}="2HD  T66"; $resn_exch{" HE1 T66"}="1HE  T66"; $resn_exch{" HE2 T66"}="2HE  T66";
$resn_exch{" HZ1 T66"}="1HZ  T66"; $resn_exch{" HZ2 T66"}="2HZ  T66"; $resn_exch{"HOP2 T6A"}="2HOP T6A";
$resn_exch{"HOP3 T6A"}="3HOP T6A"; $resn_exch{" H5' T6A"}="1H5* T6A"; $resn_exch{"H5'' T6A"}="2H5* T6A";
$resn_exch{" H4' T6A"}=" H4* T6A"; $resn_exch{" H3' T6A"}=" H3* T6A"; $resn_exch{"HO3' T6A"}="H3T  T6A";
$resn_exch{" H2' T6A"}=" H2* T6A"; $resn_exch{"HO2' T6A"}="2HO* T6A"; $resn_exch{" H1' T6A"}=" H1* T6A";
$resn_exch{"H151 T6A"}="1H15 T6A"; $resn_exch{"H152 T6A"}="2H15 T6A"; $resn_exch{"H153 T6A"}="3H15 T6A";
$resn_exch{" H71 T6F"}="1H7  T6F"; $resn_exch{" H72 T6F"}="2H7  T6F"; $resn_exch{" H61 T6F"}="1H6  T6F";
$resn_exch{" H62 T6F"}="2H6  T6F"; $resn_exch{"HF11 T6F"}="1HF1 T6F"; $resn_exch{"HF12 T6F"}="2HF1 T6F";
$resn_exch{"HF61 T6F"}="1HF6 T6F"; $resn_exch{"HF62 T6F"}="2HF6 T6F"; $resn_exch{"HM41 T6F"}="1HM4 T6F";
$resn_exch{"HM42 T6F"}="2HM4 T6F"; $resn_exch{"HM43 T6F"}="3HM4 T6F"; $resn_exch{"H7,1 T6F"}="1H7, T6F";
$resn_exch{"H7,2 T6F"}="2H7, T6F"; $resn_exch{"H4,1 T6F"}="1H4, T6F"; $resn_exch{"H4,2 T6F"}="2H4, T6F";
$resn_exch{"HM21 T6F"}="1HM2 T6F"; $resn_exch{"HM22 T6F"}="2HM2 T6F"; $resn_exch{"HM23 T6F"}="3HM2 T6F";
$resn_exch{"HC61 T6P"}="1HC6 T6P"; $resn_exch{"HC62 T6P"}="2HC6 T6P"; $resn_exch{"H6P1 T6P"}="1H6P T6P";
$resn_exch{"H6P2 T6P"}="2H6P T6P"; $resn_exch{"HOP7 T6P"}="7HOP T6P"; $resn_exch{"HOP8 T6P"}="8HOP T6P";
$resn_exch{" H71 T76"}="1H7  T76"; $resn_exch{" H72 T76"}="2H7  T76"; $resn_exch{"H181 T76"}="1H18 T76";
$resn_exch{"H182 T76"}="2H18 T76"; $resn_exch{"H191 T76"}="1H19 T76"; $resn_exch{"H192 T76"}="2H19 T76";
$resn_exch{"H201 T76"}="1H20 T76"; $resn_exch{"H202 T76"}="2H20 T76"; $resn_exch{"H341 T76"}="1H34 T76";
$resn_exch{"H342 T76"}="2H34 T76"; $resn_exch{"H351 T76"}="1H35 T76"; $resn_exch{"H352 T76"}="2H35 T76";
$resn_exch{"H431 T76"}="1H43 T76"; $resn_exch{"H432 T76"}="2H43 T76"; $resn_exch{"H471 T76"}="1H47 T76";
$resn_exch{"H472 T76"}="2H47 T76"; $resn_exch{"H481 T76"}="1H48 T76"; $resn_exch{"H482 T76"}="2H48 T76";
$resn_exch{"H491 T76"}="1H49 T76"; $resn_exch{"H492 T76"}="2H49 T76"; $resn_exch{"H501 T76"}="1H50 T76";
$resn_exch{"H502 T76"}="2H50 T76"; $resn_exch{"H511 T76"}="1H51 T76"; $resn_exch{"H512 T76"}="2H51 T76";
$resn_exch{" H11 T80"}="1H1  T80"; $resn_exch{" H12 T80"}="2H1  T80"; $resn_exch{" H13 T80"}="3H1  T80";
$resn_exch{" H31 T80"}="1H3  T80"; $resn_exch{" H32 T80"}="2H3  T80"; $resn_exch{" H41 T80"}="1H4  T80";
$resn_exch{" H42 T80"}="2H4  T80"; $resn_exch{" H61 T80"}="1H6  T80"; $resn_exch{" H62 T80"}="2H6  T80";
$resn_exch{" H71 T80"}="1H7  T80"; $resn_exch{" H72 T80"}="2H7  T80"; $resn_exch{" H91 T80"}="1H9  T80";
$resn_exch{" H92 T80"}="2H9  T80"; $resn_exch{"H101 T80"}="1H10 T80"; $resn_exch{"H102 T80"}="2H10 T80";
$resn_exch{"H121 T80"}="1H12 T80"; $resn_exch{"H122 T80"}="2H12 T80"; $resn_exch{"H131 T80"}="1H13 T80";
$resn_exch{"H132 T80"}="2H13 T80"; $resn_exch{"H151 T80"}="1H15 T80"; $resn_exch{"H152 T80"}="2H15 T80";
$resn_exch{"H161 T80"}="1H16 T80"; $resn_exch{"H162 T80"}="2H16 T80"; $resn_exch{"H201 T80"}="1H20 T80";
$resn_exch{"H202 T80"}="2H20 T80"; $resn_exch{"H211 T80"}="1H21 T80"; $resn_exch{"H212 T80"}="2H21 T80";
$resn_exch{"H221 T80"}="1H22 T80"; $resn_exch{"H222 T80"}="2H22 T80"; $resn_exch{"H231 T80"}="1H23 T80";
$resn_exch{"H232 T80"}="2H23 T80"; $resn_exch{"H241 T80"}="1H24 T80"; $resn_exch{"H242 T80"}="2H24 T80";
$resn_exch{"H251 T80"}="1H25 T80"; $resn_exch{"H252 T80"}="2H25 T80"; $resn_exch{"H261 T80"}="1H26 T80";
$resn_exch{"H262 T80"}="2H26 T80"; $resn_exch{"H271 T80"}="1H27 T80"; $resn_exch{"H272 T80"}="2H27 T80";
$resn_exch{"H281 T80"}="1H28 T80"; $resn_exch{"H282 T80"}="2H28 T80"; $resn_exch{"H291 T80"}="1H29 T80";
$resn_exch{"H292 T80"}="2H29 T80"; $resn_exch{"H301 T80"}="1H30 T80"; $resn_exch{"H302 T80"}="2H30 T80";
$resn_exch{"H311 T80"}="1H31 T80"; $resn_exch{"H312 T80"}="2H31 T80"; $resn_exch{"H321 T80"}="1H32 T80";
$resn_exch{"H322 T80"}="2H32 T80"; $resn_exch{"H331 T80"}="1H33 T80"; $resn_exch{"H332 T80"}="2H33 T80";
$resn_exch{"H341 T80"}="1H34 T80"; $resn_exch{"H342 T80"}="2H34 T80"; $resn_exch{"H351 T80"}="1H35 T80";
$resn_exch{"H352 T80"}="2H35 T80"; $resn_exch{"H361 T80"}="1H36 T80"; $resn_exch{"H362 T80"}="2H36 T80";
$resn_exch{"H363 T80"}="3H36 T80"; $resn_exch{"HN21 T87"}="1HN2 T87"; $resn_exch{"HN22 T87"}="2HN2 T87";
$resn_exch{"H10C T87"}="CH10 T87"; $resn_exch{"H11C T87"}="CH11 T87"; $resn_exch{"H13C T87"}="CH13 T87";
$resn_exch{"H141 T87"}="1H14 T87"; $resn_exch{"H142 T87"}="2H14 T87"; $resn_exch{"H143 T87"}="3H14 T87";
$resn_exch{"HC91 T87"}="1HC9 T87"; $resn_exch{"HC92 T87"}="2HC9 T87"; $resn_exch{"H151 T87"}="1H15 T87";
$resn_exch{"H152 T87"}="2H15 T87"; $resn_exch{"H19C T87"}="CH19 T87"; $resn_exch{"H20C T87"}="CH20 T87";
$resn_exch{"H21C T87"}="CH21 T87"; $resn_exch{"H231 T87"}="1H23 T87"; $resn_exch{"H232 T87"}="2H23 T87";
$resn_exch{"H251 T87"}="1H25 T87"; $resn_exch{"H252 T87"}="2H25 T87"; $resn_exch{"H253 T87"}="3H25 T87";
$resn_exch{"H261 T87"}="1H26 T87"; $resn_exch{"H262 T87"}="2H26 T87"; $resn_exch{"H11A T98"}="1H1  T98";
$resn_exch{"H12A T98"}="2H1  T98"; $resn_exch{" H13 T98"}="3H1  T98"; $resn_exch{"H141 T98"}="1H14 T98";
$resn_exch{"H142 T98"}="2H14 T98"; $resn_exch{"H143 T98"}="3H14 T98"; $resn_exch{"H161 T98"}="1H16 T98";
$resn_exch{"H162 T98"}="2H16 T98"; $resn_exch{"H163 T98"}="3H16 T98"; $resn_exch{"H011 TA1"}="1H01 TA1";
$resn_exch{"H021 TA1"}="1H02 TA1"; $resn_exch{"H051 TA1"}="1H05 TA1"; $resn_exch{"H061 TA1"}="1H06 TA1";
$resn_exch{"HC71 TA1"}="1HC7 TA1"; $resn_exch{"H081 TA1"}="1H08 TA1"; $resn_exch{"H091 TA1"}="1H09 TA1";
$resn_exch{"H101 TA1"}="1H10 TA1"; $resn_exch{"H131 TA1"}="1H13 TA1"; $resn_exch{"H132 TA1"}="2H13 TA1";
$resn_exch{"H133 TA1"}="3H13 TA1"; $resn_exch{"H141 TA1"}="1H14 TA1"; $resn_exch{"H142 TA1"}="2H14 TA1";
$resn_exch{"H151 TA1"}="1H15 TA1"; $resn_exch{"H161 TA1"}="1H16 TA1"; $resn_exch{"H162 TA1"}="2H16 TA1";
$resn_exch{"H171 TA1"}="1H17 TA1"; $resn_exch{"H071 TA1"}="1H07 TA1"; $resn_exch{"H191 TA1"}="1H19 TA1";
$resn_exch{"H192 TA1"}="2H19 TA1"; $resn_exch{"H193 TA1"}="3H19 TA1"; $resn_exch{"H211 TA1"}="1H21 TA1";
$resn_exch{"H231 TA1"}="1H23 TA1"; $resn_exch{"H232 TA1"}="2H23 TA1"; $resn_exch{"H233 TA1"}="3H23 TA1";
$resn_exch{"H261 TA1"}="1H26 TA1"; $resn_exch{"H281 TA1"}="1H28 TA1"; $resn_exch{"HO11 TA1"}="1HO1 TA1";
$resn_exch{"H291 TA1"}="1H29 TA1"; $resn_exch{"HN01 TA1"}="1HN0 TA1"; $resn_exch{"H321 TA1"}="1H32 TA1";
$resn_exch{"H331 TA1"}="1H33 TA1"; $resn_exch{"H341 TA1"}="1H34 TA1"; $resn_exch{"H351 TA1"}="1H35 TA1";
$resn_exch{"H361 TA1"}="1H36 TA1"; $resn_exch{"H381 TA1"}="1H38 TA1"; $resn_exch{"H391 TA1"}="1H39 TA1";
$resn_exch{"H401 TA1"}="1H40 TA1"; $resn_exch{"H411 TA1"}="1H41 TA1"; $resn_exch{"H421 TA1"}="1H42 TA1";
$resn_exch{"H431 TA1"}="1H43 TA1"; $resn_exch{"H432 TA1"}="2H43 TA1"; $resn_exch{"H441 TA1"}="1H44 TA1";
$resn_exch{"H442 TA1"}="2H44 TA1"; $resn_exch{"H443 TA1"}="3H44 TA1"; $resn_exch{"H461 TA1"}="1H46 TA1";
$resn_exch{"H462 TA1"}="2H46 TA1"; $resn_exch{"H463 TA1"}="3H46 TA1"; $resn_exch{"H471 TA1"}="1H47 TA1";
$resn_exch{"H472 TA1"}="2H47 TA1"; $resn_exch{"H473 TA1"}="3H47 TA1"; $resn_exch{" H1  TA2"}="1H   TA2";
$resn_exch{" H2  TA2"}="2H   TA2"; $resn_exch{" HN1 TA2"}="1HN  TA2"; $resn_exch{" HN2 TA2"}="2HN  TA2";
$resn_exch{" H5' TA3"}="1H5* TA3"; $resn_exch{"H5'' TA3"}="2H5* TA3"; $resn_exch{" H4' TA3"}=" H4* TA3";
$resn_exch{" H3' TA3"}=" H3* TA3"; $resn_exch{"HO3' TA3"}="HO3* TA3"; $resn_exch{" H2' TA3"}="1H2* TA3";
$resn_exch{"H2'' TA3"}="2H2* TA3"; $resn_exch{" H1' TA3"}=" H1* TA3"; $resn_exch{" H41 TA3"}="1H4  TA3";
$resn_exch{" H42 TA3"}="2H4  TA3"; $resn_exch{" H71 TA3"}="1H5M TA3"; $resn_exch{" H72 TA3"}="2H5M TA3";
$resn_exch{" H73 TA3"}="3H5M TA3"; $resn_exch{" HB2 TA4"}="1HB  TA4"; $resn_exch{" HB3 TA4"}="2HB  TA4";
$resn_exch{"HD21 TA4"}="1HD2 TA4"; $resn_exch{"HD22 TA4"}="2HD2 TA4"; $resn_exch{"HD23 TA4"}="3HD2 TA4";
$resn_exch{"HD11 TA4"}="1HD1 TA4"; $resn_exch{"HD12 TA4"}="2HD1 TA4"; $resn_exch{"HD13 TA4"}="3HD1 TA4";
$resn_exch{"HT61 TA4"}="1HT6 TA4"; $resn_exch{"HT62 TA4"}="2HT6 TA4"; $resn_exch{" H2  TA4"}=" HN2 TA4";
$resn_exch{"HBC1 TA5"}="1HBC TA5"; $resn_exch{"HBC2 TA5"}="2HBC TA5"; $resn_exch{"H1C1 TA6"}="1H1C TA6";
$resn_exch{"H1C2 TA6"}="2H1C TA6"; $resn_exch{"H6C1 TA6"}="1H6C TA6"; $resn_exch{"H6C2 TA6"}="2H6C TA6";
$resn_exch{"HC11 TAA"}="1HC1 TAA"; $resn_exch{"HC12 TAA"}="2HC1 TAA"; $resn_exch{"H121 TAA"}="1H12 TAA";
$resn_exch{"H122 TAA"}="2H12 TAA"; $resn_exch{"H3'1 TAB"}="1H3' TAB"; $resn_exch{"H4'1 TAB"}="1H4' TAB";
$resn_exch{"H6'1 TAB"}="1H6' TAB"; $resn_exch{"H111 TAB"}="1H11 TAB"; $resn_exch{"H112 TAB"}="2H11 TAB";
$resn_exch{"HN21 TAB"}="1HN2 TAB"; $resn_exch{"HN22 TAB"}="2HN2 TAB"; $resn_exch{"HN11 TAB"}="1HN1 TAB";
$resn_exch{"HN12 TAB"}="2HN1 TAB"; $resn_exch{"H191 TAB"}="1H19 TAB"; $resn_exch{"H192 TAB"}="2H19 TAB";
$resn_exch{"H201 TAB"}="1H20 TAB"; $resn_exch{"H202 TAB"}="2H20 TAB"; $resn_exch{"H203 TAB"}="3H20 TAB";
$resn_exch{"H211 TAB"}="1H21 TAB"; $resn_exch{"H212 TAB"}="2H21 TAB"; $resn_exch{"H221 TAB"}="1H22 TAB";
$resn_exch{"H222 TAB"}="2H22 TAB"; $resn_exch{"H251 TAB"}="1H25 TAB"; $resn_exch{"H252 TAB"}="2H25 TAB";
$resn_exch{"H253 TAB"}="3H25 TAB"; $resn_exch{"H281 TAB"}="1H28 TAB"; $resn_exch{"H291 TAB"}="1H29 TAB";
$resn_exch{"H301 TAB"}="1H30 TAB"; $resn_exch{"H311 TAB"}="1H31 TAB"; $resn_exch{"H321 TAB"}="1H32 TAB";
$resn_exch{"HN21 TAC"}="1HN2 TAC"; $resn_exch{"HN22 TAC"}="2HN2 TAC"; $resn_exch{"H421 TAC"}="1H42 TAC";
$resn_exch{"H422 TAC"}="2H42 TAC"; $resn_exch{"H423 TAC"}="3H42 TAC"; $resn_exch{"H431 TAC"}="1H43 TAC";
$resn_exch{"H432 TAC"}="2H43 TAC"; $resn_exch{"H433 TAC"}="3H43 TAC"; $resn_exch{"H51A TAC"}="1H5  TAC";
$resn_exch{" H52 TAC"}="2H5  TAC"; $resn_exch{"H621 TAC"}="1H62 TAC"; $resn_exch{"H622 TAC"}="2H62 TAC";
$resn_exch{"H623 TAC"}="3H62 TAC"; $resn_exch{"HOA2 TAD"}="2HOA TAD"; $resn_exch{"H51A TAD"}="AH51 TAD";
$resn_exch{"H52A TAD"}="AH52 TAD"; $resn_exch{" H4B TAD"}="AH4* TAD"; $resn_exch{" H3B TAD"}="AH3* TAD";
$resn_exch{"HO3A TAD"}="AHO3 TAD"; $resn_exch{" H2B TAD"}="AH2* TAD"; $resn_exch{"HO2A TAD"}="AHO2 TAD";
$resn_exch{" H1B TAD"}="AH1* TAD"; $resn_exch{" H8A TAD"}=" AH8 TAD"; $resn_exch{"H61A TAD"}="AH61 TAD";
$resn_exch{"H62A TAD"}="AH62 TAD"; $resn_exch{" H2A TAD"}=" AH2 TAD"; $resn_exch{" H31 TAD"}="1H3  TAD";
$resn_exch{" H32 TAD"}="2H3  TAD"; $resn_exch{"HON2 TAD"}="2HON TAD"; $resn_exch{"H51N TAD"}="NH51 TAD";
$resn_exch{"H52N TAD"}="NH52 TAD"; $resn_exch{" H4D TAD"}="NH4* TAD"; $resn_exch{" H3D TAD"}="NH3* TAD";
$resn_exch{"HO3N TAD"}="NHO3 TAD"; $resn_exch{" H2D TAD"}="NH2* TAD"; $resn_exch{"HO2N TAD"}="NHO2 TAD";
$resn_exch{" H1D TAD"}="NH1* TAD"; $resn_exch{" H5N TAD"}=" NH5 TAD"; $resn_exch{"H61N TAD"}="NH61 TAD";
$resn_exch{"H62N TAD"}="NH62 TAD"; $resn_exch{"HOP1 TAF"}="1HOP TAF"; $resn_exch{"HOP2 TAF"}="2HOP TAF";
$resn_exch{" H71 TAF"}="1H5M TAF"; $resn_exch{" H72 TAF"}="2H5M TAF"; $resn_exch{" H73 TAF"}="3H5M TAF";
$resn_exch{" H2' TAF"}=" H2* TAF"; $resn_exch{" H5' TAF"}="1H5* TAF"; $resn_exch{"H5'' TAF"}="2H5* TAF";
$resn_exch{" H4' TAF"}=" H4* TAF"; $resn_exch{" H1' TAF"}=" H1* TAF"; $resn_exch{" H3' TAF"}=" H3* TAF";
$resn_exch{"HO'3 TAF"}="3HO* TAF"; $resn_exch{" H11 TAG"}="1H1  TAG"; $resn_exch{" H12 TAG"}="2H1  TAG";
$resn_exch{" H61 TAG"}="1H6  TAG"; $resn_exch{" H62 TAG"}="2H6  TAG"; $resn_exch{"H6'1 TAL"}="1H6' TAL";
$resn_exch{"H6'2 TAL"}="2H6' TAL"; $resn_exch{"H6'3 TAL"}="3H6' TAL"; $resn_exch{" H71 TAL"}="1H7  TAL";
$resn_exch{" H72 TAL"}="2H7  TAL"; $resn_exch{" H73 TAL"}="3H7  TAL"; $resn_exch{" H11 TAM"}="1H1  TAM";
$resn_exch{" H12 TAM"}="2H1  TAM"; $resn_exch{" H21 TAM"}="1H2  TAM"; $resn_exch{" H22 TAM"}="2H2  TAM";
$resn_exch{" H31 TAM"}="1H3  TAM"; $resn_exch{" H32 TAM"}="2H3  TAM"; $resn_exch{" H41 TAM"}="1H4  TAM";
$resn_exch{" H42 TAM"}="2H4  TAM"; $resn_exch{" H51 TAM"}="1H5  TAM"; $resn_exch{" H52 TAM"}="2H5  TAM";
$resn_exch{" H61 TAM"}="1H6  TAM"; $resn_exch{" H62 TAM"}="2H6  TAM"; $resn_exch{" HN1 TAM"}="1HN  TAM";
$resn_exch{" HN2 TAM"}="2HN  TAM"; $resn_exch{"H531 TAO"}="1H53 TAO"; $resn_exch{"H532 TAO"}="2H53 TAO";
$resn_exch{"H533 TAO"}="3H53 TAO"; $resn_exch{"H421 TAO"}="1H42 TAO"; $resn_exch{"H422 TAO"}="2H42 TAO";
$resn_exch{"H521 TAO"}="1H52 TAO"; $resn_exch{"H522 TAO"}="2H52 TAO"; $resn_exch{"H523 TAO"}="3H52 TAO";
$resn_exch{"H561 TAO"}="1H56 TAO"; $resn_exch{"H562 TAO"}="2H56 TAO"; $resn_exch{"H563 TAO"}="3H56 TAO";
$resn_exch{"H221 TAO"}="1H22 TAO"; $resn_exch{"H222 TAO"}="2H22 TAO"; $resn_exch{"H223 TAO"}="3H22 TAO";
$resn_exch{"H161 TAO"}="1H16 TAO"; $resn_exch{"H162 TAO"}="2H16 TAO"; $resn_exch{"H163 TAO"}="3H16 TAO";
$resn_exch{"H201 TAO"}="1H20 TAO"; $resn_exch{"H202 TAO"}="2H20 TAO"; $resn_exch{"H203 TAO"}="3H20 TAO";
$resn_exch{"H51A TAO"}="1H5  TAO"; $resn_exch{" H52 TAO"}="2H5  TAO"; $resn_exch{" H71 TAO"}="1H7  TAO";
$resn_exch{" H72 TAO"}="2H7  TAO"; $resn_exch{"H131 TAO"}="1H13 TAO"; $resn_exch{"H132 TAO"}="2H13 TAO";
$resn_exch{"H133 TAO"}="3H13 TAO"; $resn_exch{"H411 TAO"}="1H41 TAO"; $resn_exch{"H412 TAO"}="2H41 TAO";
$resn_exch{"H413 TAO"}="3H41 TAO"; $resn_exch{"H401 TAO"}="1H40 TAO"; $resn_exch{"H402 TAO"}="2H40 TAO";
$resn_exch{"H461 TAO"}="1H46 TAO"; $resn_exch{"H462 TAO"}="2H46 TAO"; $resn_exch{"H463 TAO"}="3H46 TAO";
$resn_exch{"H471 TAO"}="1H47 TAO"; $resn_exch{"H472 TAO"}="2H47 TAO"; $resn_exch{"H473 TAO"}="3H47 TAO";
$resn_exch{"H441 TAO"}="1H44 TAO"; $resn_exch{"H442 TAO"}="2H44 TAO"; $resn_exch{"H443 TAO"}="3H44 TAO";
$resn_exch{"H251 TAO"}="1H25 TAO"; $resn_exch{"H252 TAO"}="2H25 TAO"; $resn_exch{"H253 TAO"}="3H25 TAO";
$resn_exch{" H81 TAO"}="1H8  TAO"; $resn_exch{" H82 TAO"}="2H8  TAO"; $resn_exch{" H83 TAO"}="3H8  TAO";
$resn_exch{"H371 TAO"}="1H37 TAO"; $resn_exch{"H372 TAO"}="2H37 TAO"; $resn_exch{"H373 TAO"}="3H37 TAO";
$resn_exch{"HOA2 TAP"}="2HOA TAP"; $resn_exch{"H51A TAP"}="AH51 TAP"; $resn_exch{"H52A TAP"}="AH52 TAP";
$resn_exch{" H4B TAP"}="AH4* TAP"; $resn_exch{" H3B TAP"}="AH3* TAP"; $resn_exch{"HO3A TAP"}="AHO3 TAP";
$resn_exch{" H2B TAP"}="AH2* TAP"; $resn_exch{" H1B TAP"}="AH1* TAP"; $resn_exch{" H8A TAP"}=" AH8 TAP";
$resn_exch{"H61A TAP"}="AH61 TAP"; $resn_exch{"H62A TAP"}="AH62 TAP"; $resn_exch{" H2A TAP"}=" AH2 TAP";
$resn_exch{"H51N TAP"}="NH51 TAP"; $resn_exch{"H52N TAP"}="NH52 TAP"; $resn_exch{" H4D TAP"}="NH4* TAP";
$resn_exch{" H3D TAP"}="NH3* TAP"; $resn_exch{"HO3N TAP"}="NHO3 TAP"; $resn_exch{" H2D TAP"}="NH2* TAP";
$resn_exch{"HO2N TAP"}="NHO2 TAP"; $resn_exch{" H1D TAP"}="NH1* TAP"; $resn_exch{" H2N TAP"}=" NH2 TAP";
$resn_exch{"H71N TAP"}="NH71 TAP"; $resn_exch{"H72N TAP"}="NH72 TAP"; $resn_exch{" H4N TAP"}=" NH4 TAP";
$resn_exch{" H5N TAP"}=" NH5 TAP"; $resn_exch{" H6N TAP"}=" NH6 TAP"; $resn_exch{"HOP2 TAP"}="2HOP TAP";
$resn_exch{"HOP3 TAP"}="3HOP TAP"; $resn_exch{"H4N1 TAQ"}="1H4N TAQ"; $resn_exch{"H4N2 TAQ"}="2H4N TAQ";
$resn_exch{"H2N1 TAQ"}="1H2N TAQ"; $resn_exch{"H2N2 TAQ"}="2H2N TAQ"; $resn_exch{"HAA1 TAQ"}="1HAA TAQ";
$resn_exch{"HAA2 TAQ"}="2HAA TAQ"; $resn_exch{" H11 TAS"}="1H1  TAS"; $resn_exch{" H21 TAS"}="1H2  TAS";
$resn_exch{" H31 TAS"}="1H3  TAS"; $resn_exch{"H5'1 TAT"}="1H5* TAT"; $resn_exch{"H5'2 TAT"}="2H5* TAT";
$resn_exch{" H4' TAT"}=" H4* TAT"; $resn_exch{" H3' TAT"}=" H3* TAT"; $resn_exch{" H2' TAT"}=" H2* TAT";
$resn_exch{" H1' TAT"}=" H1* TAT"; $resn_exch{"H6N1 TAT"}="1H6N TAT"; $resn_exch{"H6N2 TAT"}="2H6N TAT";
$resn_exch{"H1C1 TAU"}="1H1C TAU"; $resn_exch{"H1C2 TAU"}="2H1C TAU"; $resn_exch{"H2C1 TAU"}="1H2C TAU";
$resn_exch{"H2C2 TAU"}="2H2C TAU"; $resn_exch{"H1N1 TAU"}="1H1N TAU"; $resn_exch{"H1N2 TAU"}="2H1N TAU";
$resn_exch{"H561 TAV"}="1H56 TAV"; $resn_exch{"H562 TAV"}="2H56 TAV"; $resn_exch{"H563 TAV"}="3H56 TAV";
$resn_exch{"H671 TAV"}="1H67 TAV"; $resn_exch{"H672 TAV"}="2H67 TAV"; $resn_exch{" HA1 TAX"}="1HA  TAX";
$resn_exch{" HA2 TAX"}="2HA  TAX"; $resn_exch{" HB1 TAX"}="1HB  TAX"; $resn_exch{" HB2 TAX"}="2HB  TAX";
$resn_exch{" HB3 TAX"}="3HB  TAX"; $resn_exch{"HA11 TAX"}="1HA1 TAX"; $resn_exch{"HA12 TAX"}="2HA1 TAX";
$resn_exch{"HB11 TAX"}="1HB1 TAX"; $resn_exch{"HB12 TAX"}="2HB1 TAX"; $resn_exch{"H211 TAX"}="1H21 TAX";
$resn_exch{"H212 TAX"}="2H21 TAX"; $resn_exch{"H213 TAX"}="3H21 TAX"; $resn_exch{"H221 TAX"}="1H22 TAX";
$resn_exch{"H222 TAX"}="2H22 TAX"; $resn_exch{"H223 TAX"}="3H22 TAX"; $resn_exch{"HC51 TAZ"}="1HC5 TAZ";
$resn_exch{"HC61 TAZ"}="1HC6 TAZ"; $resn_exch{"HC62 TAZ"}="2HC6 TAZ"; $resn_exch{"H201 TAZ"}="1H20 TAZ";
$resn_exch{"H202 TAZ"}="2H20 TAZ"; $resn_exch{"H203 TAZ"}="3H20 TAZ"; $resn_exch{"H141 TAZ"}="1H14 TAZ";
$resn_exch{"H142 TAZ"}="2H14 TAZ"; $resn_exch{"H191 TAZ"}="1H19 TAZ"; $resn_exch{"H181 TAZ"}="1H18 TAZ";
$resn_exch{" H41 TB1"}="1H4  TB1"; $resn_exch{" H42 TB1"}="2H4  TB1"; $resn_exch{" HN1 TB1"}="1HN  TB1";
$resn_exch{" HN2 TB1"}="2HN  TB1"; $resn_exch{" H41 TB9"}="1H4  TB9"; $resn_exch{" H42 TB9"}="2H4  TB9";
$resn_exch{" H71 TB9"}="1H7  TB9"; $resn_exch{" H72 TB9"}="2H7  TB9"; $resn_exch{"H111 TB9"}="1H11 TB9";
$resn_exch{"H112 TB9"}="2H11 TB9"; $resn_exch{"H113 TB9"}="3H11 TB9"; $resn_exch{"H121 TB9"}="1H12 TB9";
$resn_exch{"H122 TB9"}="2H12 TB9"; $resn_exch{"H151 TB9"}="1H15 TB9"; $resn_exch{"H152 TB9"}="2H15 TB9";
$resn_exch{"H153 TB9"}="3H15 TB9"; $resn_exch{"H161 TB9"}="1H16 TB9"; $resn_exch{"H162 TB9"}="2H16 TB9";
$resn_exch{"H163 TB9"}="3H16 TB9"; $resn_exch{"H111 TBA"}="1H11 TBA"; $resn_exch{"H112 TBA"}="2H11 TBA";
$resn_exch{"H121 TBA"}="1H12 TBA"; $resn_exch{"H122 TBA"}="2H12 TBA"; $resn_exch{"H211 TBA"}="1H21 TBA";
$resn_exch{"H212 TBA"}="2H21 TBA"; $resn_exch{"H221 TBA"}="1H22 TBA"; $resn_exch{"H222 TBA"}="2H22 TBA";
$resn_exch{"H311 TBA"}="1H31 TBA"; $resn_exch{"H312 TBA"}="2H31 TBA"; $resn_exch{"H321 TBA"}="1H32 TBA";
$resn_exch{"H322 TBA"}="2H32 TBA"; $resn_exch{"H411 TBA"}="1H41 TBA"; $resn_exch{"H412 TBA"}="2H41 TBA";
$resn_exch{"H421 TBA"}="1H42 TBA"; $resn_exch{"H422 TBA"}="2H42 TBA"; $resn_exch{"H131 TBA"}="1H13 TBA";
$resn_exch{"H132 TBA"}="2H13 TBA"; $resn_exch{"H141 TBA"}="1H14 TBA"; $resn_exch{"H142 TBA"}="2H14 TBA";
$resn_exch{"H143 TBA"}="3H14 TBA"; $resn_exch{"H231 TBA"}="1H23 TBA"; $resn_exch{"H232 TBA"}="2H23 TBA";
$resn_exch{"H241 TBA"}="1H24 TBA"; $resn_exch{"H242 TBA"}="2H24 TBA"; $resn_exch{"H243 TBA"}="3H24 TBA";
$resn_exch{"H331 TBA"}="1H33 TBA"; $resn_exch{"H332 TBA"}="2H33 TBA"; $resn_exch{"H341 TBA"}="1H34 TBA";
$resn_exch{"H342 TBA"}="2H34 TBA"; $resn_exch{"H343 TBA"}="3H34 TBA"; $resn_exch{"H431 TBA"}="1H43 TBA";
$resn_exch{"H432 TBA"}="2H43 TBA"; $resn_exch{"H441 TBA"}="1H44 TBA"; $resn_exch{"H442 TBA"}="2H44 TBA";
$resn_exch{"H443 TBA"}="3H44 TBA"; $resn_exch{"H101 TBC"}="1H10 TBC"; $resn_exch{"H102 TBC"}="2H10 TBC";
$resn_exch{" H72 TBC"}="2H7  TBC"; $resn_exch{" H71 TBC"}="1H7  TBC"; $resn_exch{" H81 TBC"}="1H8  TBC";
$resn_exch{" H82 TBC"}="2H8  TBC"; $resn_exch{"H1BO TBD"}="OH1B TBD"; $resn_exch{"H2BO TBD"}="OH2B TBD";
$resn_exch{"H2AO TBD"}="OH2A TBD"; $resn_exch{"H5'1 TBD"}="1H5* TBD"; $resn_exch{"H5'2 TBD"}="2H5* TBD";
$resn_exch{"H4'C TBD"}="CH4* TBD"; $resn_exch{"H1'C TBD"}="CH1* TBD"; $resn_exch{"HCA1 TBD"}="1HCA TBD";
$resn_exch{"HCA2 TBD"}="2HCA TBD"; $resn_exch{"HCA3 TBD"}="3HCA TBD"; $resn_exch{"H2'1 TBD"}="1H2* TBD";
$resn_exch{"H2'2 TBD"}="2H2* TBD"; $resn_exch{"H3'C TBD"}="CH3* TBD"; $resn_exch{"H3'O TBD"}="OH3* TBD";
$resn_exch{"H201 TBE"}="1H20 TBE"; $resn_exch{"H202 TBE"}="2H20 TBE"; $resn_exch{"H203 TBE"}="3H20 TBE";
$resn_exch{"H141 TBE"}="1H14 TBE"; $resn_exch{"H142 TBE"}="2H14 TBE"; $resn_exch{" H11 TBF"}="1H1  TBF";
$resn_exch{" H12 TBF"}="2H1  TBF"; $resn_exch{" H13 TBF"}="3H1  TBF"; $resn_exch{" H21 TBF"}="1H2  TBF";
$resn_exch{" H22 TBF"}="2H2  TBF"; $resn_exch{" H23 TBF"}="3H2  TBF"; $resn_exch{" H31 TBF"}="1H3  TBF";
$resn_exch{" H32 TBF"}="2H3  TBF"; $resn_exch{" H33 TBF"}="3H3  TBF"; $resn_exch{" H   TBG"}="1HN  TBG";
$resn_exch{" H2  TBG"}="2HN  TBG"; $resn_exch{"HG11 TBG"}="1H1  TBG"; $resn_exch{"HG12 TBG"}="2H1  TBG";
$resn_exch{"HG13 TBG"}="3H1  TBG"; $resn_exch{"HG21 TBG"}="1H2  TBG"; $resn_exch{"HG22 TBG"}="2H2  TBG";
$resn_exch{"HG23 TBG"}="3H2  TBG"; $resn_exch{"HG31 TBG"}="1H3  TBG"; $resn_exch{"HG32 TBG"}="2H3  TBG";
$resn_exch{"HG33 TBG"}="3H3  TBG"; $resn_exch{"H421 TBH"}="1H42 TBH"; $resn_exch{" H51 TBH"}="1H5  TBH";
$resn_exch{"H231 TBH"}="1H23 TBH"; $resn_exch{"H232 TBH"}="2H23 TBH"; $resn_exch{"H261 TBH"}="1H26 TBH";
$resn_exch{" H21 TBH"}="1H2  TBH"; $resn_exch{" H31 TBH"}="1H3  TBH"; $resn_exch{" H71 TBH"}="1H7  TBH";
$resn_exch{" H72 TBH"}="2H7  TBH"; $resn_exch{" H81 TBH"}="1H8  TBH"; $resn_exch{"H111 TBH"}="1H11 TBH";
$resn_exch{"H131 TBH"}="1H13 TBH"; $resn_exch{"H151 TBH"}="1H15 TBH"; $resn_exch{"H152 TBH"}="2H15 TBH";
$resn_exch{"H161 TBH"}="1H16 TBH"; $resn_exch{"H162 TBH"}="2H16 TBH"; $resn_exch{"H171 TBH"}="1H17 TBH";
$resn_exch{"H172 TBH"}="2H17 TBH"; $resn_exch{"H181 TBH"}="1H18 TBH"; $resn_exch{"H182 TBH"}="2H18 TBH";
$resn_exch{"H191 TBH"}="1H19 TBH"; $resn_exch{"H192 TBH"}="2H19 TBH"; $resn_exch{"H193 TBH"}="3H19 TBH";
$resn_exch{"H141 TBH"}="1H14 TBH"; $resn_exch{"H101 TBH"}="1H10 TBH"; $resn_exch{"H121 TBH"}="1H12 TBH";
$resn_exch{"H201 TBH"}="1H20 TBH"; $resn_exch{"H211 TBH"}="1H21 TBH"; $resn_exch{"H221 TBH"}="1H22 TBH";
$resn_exch{"H351 TBH"}="1H35 TBH"; $resn_exch{"H361 TBH"}="1H36 TBH"; $resn_exch{"H401 TBH"}="1H40 TBH";
$resn_exch{"H402 TBH"}="2H40 TBH"; $resn_exch{"H403 TBH"}="3H40 TBH"; $resn_exch{"H391 TBH"}="1H39 TBH";
$resn_exch{"H392 TBH"}="2H39 TBH"; $resn_exch{"H393 TBH"}="3H39 TBH"; $resn_exch{"H411 TBH"}="1H41 TBH";
$resn_exch{"H412 TBH"}="2H41 TBH"; $resn_exch{"H413 TBH"}="3H41 TBH"; $resn_exch{"H281 TBH"}="1H28 TBH";
$resn_exch{"H282 TBH"}="2H28 TBH"; $resn_exch{"H301 TBH"}="1H30 TBH"; $resn_exch{"H311 TBH"}="1H31 TBH";
$resn_exch{"H321 TBH"}="1H32 TBH"; $resn_exch{"H331 TBH"}="1H33 TBH"; $resn_exch{"H341 TBH"}="1H34 TBH";
$resn_exch{"H141 TBI"}="1H14 TBI"; $resn_exch{"H142 TBI"}="2H14 TBI"; $resn_exch{"H201 TBI"}="1H20 TBI";
$resn_exch{"H202 TBI"}="2H20 TBI"; $resn_exch{"H203 TBI"}="3H20 TBI"; $resn_exch{" H77 TBI"}="7H7  TBI";
$resn_exch{"HCT1 TBM"}="1HCT TBM"; $resn_exch{"HCT2 TBM"}="2HCT TBM"; $resn_exch{"HCT3 TBM"}="3HCT TBM";
$resn_exch{"HCN1 TBM"}="1HCN TBM"; $resn_exch{"HCN2 TBM"}="2HCN TBM"; $resn_exch{"HCN3 TBM"}="3HCN TBM";
$resn_exch{"HD11 TBM"}="1HD1 TBM"; $resn_exch{"HD12 TBM"}="2HD1 TBM"; $resn_exch{"HD13 TBM"}="3HD1 TBM";
$resn_exch{"HD22 TBM"}="1HD2 TBM"; $resn_exch{"HD23 TBM"}="2HD2 TBM"; $resn_exch{" HH1 TBM"}="1HH  TBM";
$resn_exch{" HH2 TBM"}="2HH  TBM"; $resn_exch{" HH3 TBM"}="3HH  TBM"; $resn_exch{"H5'1 TBN"}="1H5' TBN";
$resn_exch{"H5'2 TBN"}="2H5' TBN"; $resn_exch{"HN61 TBN"}="1HN6 TBN"; $resn_exch{"HN62 TBN"}="2HN6 TBN";
$resn_exch{" H41 TBO"}="1H4  TBO"; $resn_exch{" H42 TBO"}="2H4  TBO"; $resn_exch{" H71 TBO"}="1H7  TBO";
$resn_exch{" H72 TBO"}="2H7  TBO"; $resn_exch{"H111 TBO"}="1H11 TBO"; $resn_exch{"H112 TBO"}="2H11 TBO";
$resn_exch{"H113 TBO"}="3H11 TBO"; $resn_exch{"H121 TBO"}="1H12 TBO"; $resn_exch{"H122 TBO"}="2H12 TBO";
$resn_exch{"H151 TBO"}="1H15 TBO"; $resn_exch{"H152 TBO"}="2H15 TBO"; $resn_exch{"H153 TBO"}="3H15 TBO";
$resn_exch{"H161 TBO"}="1H16 TBO"; $resn_exch{"H162 TBO"}="2H16 TBO"; $resn_exch{"H163 TBO"}="3H16 TBO";
$resn_exch{"H4C1 TBT"}="1H4C TBT"; $resn_exch{"H4C2 TBT"}="2H4C TBT"; $resn_exch{" H11 TBU"}="1H1  TBU";
$resn_exch{" H12 TBU"}="2H1  TBU"; $resn_exch{" H13 TBU"}="3H1  TBU"; $resn_exch{" H21 TBU"}="1H2  TBU";
$resn_exch{" H22 TBU"}="2H2  TBU"; $resn_exch{" H23 TBU"}="3H2  TBU"; $resn_exch{" H31 TBU"}="1H3  TBU";
$resn_exch{" H32 TBU"}="2H3  TBU"; $resn_exch{" H33 TBU"}="3H3  TBU"; $resn_exch{"HN11 TBZ"}="1HN1 TBZ";
$resn_exch{"HN12 TBZ"}="2HN1 TBZ"; $resn_exch{"HN13 TBZ"}="3HN1 TBZ"; $resn_exch{" H21 TBZ"}="1H2  TBZ";
$resn_exch{" H22 TBZ"}="2H2  TBZ"; $resn_exch{"H31A TBZ"}="1H3  TBZ"; $resn_exch{" H32 TBZ"}="2H3  TBZ";
$resn_exch{" H41 TBZ"}="1H4  TBZ"; $resn_exch{" H42 TBZ"}="2H4  TBZ"; $resn_exch{"H321 TBZ"}="1H32 TBZ";
$resn_exch{"H322 TBZ"}="2H32 TBZ"; $resn_exch{"H323 TBZ"}="3H32 TBZ"; $resn_exch{" H5' TC1"}="1H5* TC1";
$resn_exch{"H5'' TC1"}="2H5* TC1"; $resn_exch{" H4' TC1"}=" H4* TC1"; $resn_exch{" H3' TC1"}=" H3* TC1";
$resn_exch{" H2' TC1"}="1H2* TC1"; $resn_exch{"H2'' TC1"}="2H2* TC1"; $resn_exch{" H1' TC1"}=" H1* TC1";
$resn_exch{" H14 TC1"}="4H1  TC1"; $resn_exch{" H5  TC1"}="5H   TC1"; $resn_exch{" H7  TC1"}="7H   TC1";
$resn_exch{" H8  TC1"}="8H   TC1"; $resn_exch{" H9  TC1"}="9H   TC1"; $resn_exch{" H10 TC1"}="0H1  TC1";
$resn_exch{"HO3' TC1"}="H3T  TC1"; $resn_exch{"HOP3 TC1"}="3HOP TC1"; $resn_exch{" H11 TC4"}="1H1  TC4";
$resn_exch{" H12 TC4"}="2H1  TC4"; $resn_exch{" H21 TC4"}="1H2  TC4"; $resn_exch{" H22 TC4"}="2H2  TC4";
$resn_exch{" H31 TC4"}="1H3  TC4"; $resn_exch{" H32 TC4"}="2H3  TC4"; $resn_exch{" H41 TC4"}="1H4  TC4";
$resn_exch{" H42 TC4"}="2H4  TC4"; $resn_exch{" H43 TC4"}="3H4  TC4"; $resn_exch{" H51 TC4"}="1H5  TC4";
$resn_exch{" H52 TC4"}="2H5  TC4"; $resn_exch{" H81 TC4"}="1H8  TC4"; $resn_exch{" H82 TC4"}="2H8  TC4";
$resn_exch{" H91 TC4"}="1H9  TC4"; $resn_exch{" H92 TC4"}="2H9  TC4"; $resn_exch{"H101 TC4"}="1H10 TC4";
$resn_exch{"H102 TC4"}="2H10 TC4"; $resn_exch{"H111 TC4"}="1H11 TC4"; $resn_exch{"H112 TC4"}="2H11 TC4";
$resn_exch{"H113 TC4"}="3H11 TC4"; $resn_exch{"H121 TC4"}="1H12 TC4"; $resn_exch{"H122 TC4"}="2H12 TC4";
$resn_exch{"H141 TC4"}="1H14 TC4"; $resn_exch{"H142 TC4"}="2H14 TC4"; $resn_exch{"H151 TC4"}="1H15 TC4";
$resn_exch{"H152 TC4"}="2H15 TC4"; $resn_exch{"H161 TC4"}="1H16 TC4"; $resn_exch{"H162 TC4"}="2H16 TC4";
$resn_exch{"H171 TC4"}="1H17 TC4"; $resn_exch{"H172 TC4"}="2H17 TC4"; $resn_exch{"H173 TC4"}="3H17 TC4";
$resn_exch{" H61 TCB"}="1H6  TCB"; $resn_exch{" H62 TCB"}="2H6  TCB"; $resn_exch{"H6'1 TCB"}="1H6' TCB";
$resn_exch{"H6'2 TCB"}="2H6' TCB"; $resn_exch{"HO1' TCB"}="'HO1 TCB"; $resn_exch{"HO2' TCB"}="'HO2 TCB";
$resn_exch{"HO3' TCB"}="'HO3 TCB"; $resn_exch{"HO6' TCB"}="'HO6 TCB"; $resn_exch{"H12C TCC"}="CH12 TCC";
$resn_exch{"H10C TCC"}="CH10 TCC"; $resn_exch{" H11 TCH"}="1H1  TCH"; $resn_exch{"H12A TCH"}="2H1  TCH";
$resn_exch{" H21 TCH"}="1H2  TCH"; $resn_exch{" H22 TCH"}="2H2  TCH"; $resn_exch{" H41 TCH"}="1H4  TCH";
$resn_exch{" H42 TCH"}="2H4  TCH"; $resn_exch{" H61 TCH"}="1H6  TCH"; $resn_exch{" H62 TCH"}="2H6  TCH";
$resn_exch{"H111 TCH"}="1H11 TCH"; $resn_exch{"H112 TCH"}="2H11 TCH"; $resn_exch{"H151 TCH"}="1H15 TCH";
$resn_exch{"H152 TCH"}="2H15 TCH"; $resn_exch{"H161 TCH"}="1H16 TCH"; $resn_exch{"H162 TCH"}="2H16 TCH";
$resn_exch{"H181 TCH"}="1H18 TCH"; $resn_exch{"H182 TCH"}="2H18 TCH"; $resn_exch{"H183 TCH"}="3H18 TCH";
$resn_exch{"H191 TCH"}="1H19 TCH"; $resn_exch{"H192 TCH"}="2H19 TCH"; $resn_exch{"H193 TCH"}="3H19 TCH";
$resn_exch{"H211 TCH"}="1H21 TCH"; $resn_exch{"H212 TCH"}="2H21 TCH"; $resn_exch{"H213 TCH"}="3H21 TCH";
$resn_exch{"H221 TCH"}="1H22 TCH"; $resn_exch{"H222 TCH"}="2H22 TCH"; $resn_exch{"H231 TCH"}="1H23 TCH";
$resn_exch{"H232 TCH"}="2H23 TCH"; $resn_exch{"H251 TCH"}="1H25 TCH"; $resn_exch{"H252 TCH"}="2H25 TCH";
$resn_exch{"H261 TCH"}="1H26 TCH"; $resn_exch{"H262 TCH"}="2H26 TCH"; $resn_exch{" H11 TCK"}="1H1  TCK";
$resn_exch{" H12 TCK"}="2H1  TCK"; $resn_exch{" H13 TCK"}="3H1  TCK"; $resn_exch{" H81 TCK"}="1H8  TCK";
$resn_exch{" H82 TCK"}="2H8  TCK"; $resn_exch{" H83 TCK"}="3H8  TCK"; $resn_exch{" HB1 TCK"}="1HB  TCK";
$resn_exch{" HB2 TCK"}="2HB  TCK"; $resn_exch{" HG1 TCK"}="1HG  TCK"; $resn_exch{" HG2 TCK"}="2HG  TCK";
$resn_exch{" HD1 TCK"}="1HD  TCK"; $resn_exch{" HD2 TCK"}="2HD  TCK"; $resn_exch{" HE1 TCK"}="1HE  TCK";
$resn_exch{" HE2 TCK"}="2HE  TCK"; $resn_exch{"HNZ1 TCK"}="1HNZ TCK"; $resn_exch{"HNZ2 TCK"}="2HNZ TCK";
$resn_exch{" H11 TCL"}="1H1  TCL"; $resn_exch{" H41 TCL"}="1H4  TCL"; $resn_exch{" H31 TCL"}="1H3  TCL";
$resn_exch{"H101 TCL"}="1H10 TCL"; $resn_exch{"H121 TCL"}="1H12 TCL"; $resn_exch{"H131 TCL"}="1H13 TCL";
$resn_exch{"H171 TCL"}="1H17 TCL"; $resn_exch{" H31 TCM"}="1H3  TCM"; $resn_exch{" H32 TCM"}="2H3  TCM";
$resn_exch{" H51 TCM"}="1H5  TCM"; $resn_exch{" H52 TCM"}="2H5  TCM"; $resn_exch{" H61 TCM"}="1H6  TCM";
$resn_exch{" H62 TCM"}="2H6  TCM"; $resn_exch{" H81 TCM"}="1H8  TCM"; $resn_exch{" H82 TCM"}="2H8  TCM";
$resn_exch{" H83 TCM"}="3H8  TCM"; $resn_exch{" H91 TCM"}="1H9  TCM"; $resn_exch{" H92 TCM"}="2H9  TCM";
$resn_exch{" H93 TCM"}="3H9  TCM"; $resn_exch{"H101 TCM"}="1H10 TCM"; $resn_exch{"H102 TCM"}="2H10 TCM";
$resn_exch{"H103 TCM"}="3H10 TCM"; $resn_exch{"HG21 TCO"}="1HG2 TCO"; $resn_exch{"HG22 TCO"}="2HG2 TCO";
$resn_exch{"HD21 TCO"}="1HD2 TCO"; $resn_exch{"HD22 TCO"}="2HD2 TCO"; $resn_exch{" HE1 TCO"}="1HE  TCO";
$resn_exch{" HE2 TCO"}="2HE  TCO"; $resn_exch{"HD11 TCO"}="1HD1 TCO"; $resn_exch{"HD12 TCO"}="2HD1 TCO";
$resn_exch{"HG11 TCO"}="1HG1 TCO"; $resn_exch{"HG12 TCO"}="2HG1 TCO"; $resn_exch{" H41 TCO"}="1H4  TCO";
$resn_exch{" H42 TCO"}="2H4  TCO"; $resn_exch{" H43 TCO"}="3H4  TCO"; $resn_exch{" H61 TCO"}="1H6  TCO";
$resn_exch{" H62 TCO"}="2H6  TCO"; $resn_exch{" H63 TCO"}="3H6  TCO"; $resn_exch{" H51 TCO"}="1H5  TCO";
$resn_exch{" H52 TCO"}="2H5  TCO"; $resn_exch{" H53 TCO"}="3H5  TCO"; $resn_exch{" HP1 TCP"}="1HP  TCP";
$resn_exch{" HP2 TCP"}="2HP  TCP"; $resn_exch{" HP3 TCP"}="3HP  TCP"; $resn_exch{" H5' TCP"}="1H5* TCP";
$resn_exch{"H5'' TCP"}="2H5* TCP"; $resn_exch{" H4' TCP"}=" H4* TCP"; $resn_exch{" H3' TCP"}=" H3* TCP";
$resn_exch{"HO3' TCP"}="*HO3 TCP"; $resn_exch{" H2' TCP"}="1H2* TCP"; $resn_exch{"H2'' TCP"}="2H2* TCP";
$resn_exch{" H1' TCP"}=" H1* TCP"; $resn_exch{" H71 TCP"}="1H5M TCP"; $resn_exch{" H72 TCP"}="2H5M TCP";
$resn_exch{" H73 TCP"}="3H5M TCP"; $resn_exch{"HT21 TCQ"}="1HT2 TCQ"; $resn_exch{"HT22 TCQ"}="2HT2 TCQ";
$resn_exch{" HB1 TCR"}="1HB  TCR"; $resn_exch{" HB2 TCR"}="2HB  TCR"; $resn_exch{" H91 TCR"}="1H9  TCR";
$resn_exch{" H92 TCR"}="2H9  TCR"; $resn_exch{"H16C TCT"}="CH16 TCT"; $resn_exch{"H15C TCT"}="CH15 TCT";
$resn_exch{"H13C TCT"}="CH13 TCT"; $resn_exch{"H10C TCT"}="CH10 TCT"; $resn_exch{"H222 TDA"}="2H22 TDA";
$resn_exch{"H221 TDA"}="1H22 TDA"; $resn_exch{"H232 TDA"}="2H23 TDA"; $resn_exch{"H231 TDA"}="1H23 TDA";
$resn_exch{"H242 TDA"}="2H24 TDA"; $resn_exch{"H241 TDA"}="1H24 TDA"; $resn_exch{"H252 TDA"}="2H25 TDA";
$resn_exch{"H251 TDA"}="1H25 TDA"; $resn_exch{"H262 TDA"}="2H26 TDA"; $resn_exch{"H261 TDA"}="1H26 TDA";
$resn_exch{"H272 TDA"}="2H27 TDA"; $resn_exch{"H271 TDA"}="1H27 TDA"; $resn_exch{"H282 TDA"}="2H28 TDA";
$resn_exch{"H281 TDA"}="1H28 TDA"; $resn_exch{"H292 TDA"}="2H29 TDA"; $resn_exch{"H291 TDA"}="1H29 TDA";
$resn_exch{"H302 TDA"}="2H30 TDA"; $resn_exch{"H301 TDA"}="1H30 TDA"; $resn_exch{"H312 TDA"}="2H31 TDA";
$resn_exch{"H311 TDA"}="1H31 TDA"; $resn_exch{"H322 TDA"}="2H32 TDA"; $resn_exch{"H321 TDA"}="1H32 TDA";
$resn_exch{"H333 TDA"}="3H33 TDA"; $resn_exch{"H332 TDA"}="2H33 TDA"; $resn_exch{"H331 TDA"}="1H33 TDA";
$resn_exch{" H81 TDB"}="1H8  TDB"; $resn_exch{" H82 TDB"}="2H8  TDB"; $resn_exch{" H83 TDB"}="3H8  TDB";
$resn_exch{"H151 TDB"}="1H15 TDB"; $resn_exch{"H152 TDB"}="2H15 TDB"; $resn_exch{"H161 TDB"}="1H16 TDB";
$resn_exch{"H162 TDB"}="2H16 TDB"; $resn_exch{"H171 TDB"}="1H17 TDB"; $resn_exch{"H172 TDB"}="2H17 TDB";
$resn_exch{"H173 TDB"}="3H17 TDB"; $resn_exch{"H621 TDC"}="1H62 TDC"; $resn_exch{"H622 TDC"}="2H62 TDC";
$resn_exch{"H623 TDC"}="3H62 TDC"; $resn_exch{"H5C1 TDC"}="1H5C TDC"; $resn_exch{"H5C2 TDC"}="2H5C TDC";
$resn_exch{"H431 TDC"}="1H43 TDC"; $resn_exch{"H432 TDC"}="2H43 TDC"; $resn_exch{"H433 TDC"}="3H43 TDC";
$resn_exch{"H421 TDC"}="1H42 TDC"; $resn_exch{"H422 TDC"}="2H42 TDC"; $resn_exch{"H423 TDC"}="3H42 TDC";
$resn_exch{"H211 TDC"}="1H21 TDC"; $resn_exch{"H212 TDC"}="2H21 TDC"; $resn_exch{" H41 TDE"}="1H4  TDE";
$resn_exch{" H42 TDE"}="2H4  TDE"; $resn_exch{" H51 TDE"}="1H5  TDE"; $resn_exch{" H52 TDE"}="2H5  TDE";
$resn_exch{" H71 TDE"}="1H7  TDE"; $resn_exch{" H72 TDE"}="2H7  TDE"; $resn_exch{"H251 TDE"}="1H25 TDE";
$resn_exch{"H252 TDE"}="2H25 TDE"; $resn_exch{"H253 TDE"}="3H25 TDE"; $resn_exch{"H211 TDE"}="1H21 TDE";
$resn_exch{"H212 TDE"}="2H21 TDE"; $resn_exch{"H213 TDE"}="3H21 TDE"; $resn_exch{"H201 TDE"}="1H20 TDE";
$resn_exch{"H202 TDE"}="2H20 TDE"; $resn_exch{"H203 TDE"}="3H20 TDE"; $resn_exch{"H191 TDE"}="1H19 TDE";
$resn_exch{"H192 TDE"}="2H19 TDE"; $resn_exch{"H193 TDE"}="3H19 TDE"; $resn_exch{" H61 TDG"}="1H6  TDG";
$resn_exch{" H62 TDG"}="2H6  TDG"; $resn_exch{"HO2' TDG"}="'HO2 TDG"; $resn_exch{"HO3' TDG"}="'HO3 TDG";
$resn_exch{"HO4' TDG"}="'HO4 TDG"; $resn_exch{"H6'1 TDG"}="1H6' TDG"; $resn_exch{"H6'2 TDG"}="2H6' TDG";
$resn_exch{"HO6' TDG"}="'HO6 TDG"; $resn_exch{"HS51 TDI"}="1HS5 TDI"; $resn_exch{"HS52 TDI"}="2HS5 TDI";
$resn_exch{"HS53 TDI"}="3HS5 TDI"; $resn_exch{"H5'1 TDI"}="1H5* TDI"; $resn_exch{"H5'2 TDI"}="2H5* TDI";
$resn_exch{" H4' TDI"}=" H4* TDI"; $resn_exch{"H1'1 TDI"}="1H1* TDI"; $resn_exch{"H1'2 TDI"}="2H1* TDI";
$resn_exch{"H101 TDI"}="1H10 TDI"; $resn_exch{"H102 TDI"}="2H10 TDI"; $resn_exch{"HN61 TDI"}="1HN6 TDI";
$resn_exch{"HN62 TDI"}="2HN6 TDI"; $resn_exch{"H2'1 TDI"}="1H2* TDI"; $resn_exch{"H2'2 TDI"}="2H2* TDI";
$resn_exch{" H3' TDI"}=" H3* TDI"; $resn_exch{" H71 TDK"}="1H7  TDK"; $resn_exch{" H72 TDK"}="2H7  TDK";
$resn_exch{" H61 TDK"}="1H6  TDK"; $resn_exch{" H62 TDK"}="2H6  TDK"; $resn_exch{"HM41 TDK"}="1HM4 TDK";
$resn_exch{"HM42 TDK"}="2HM4 TDK"; $resn_exch{"HM43 TDK"}="3HM4 TDK"; $resn_exch{"HMB1 TDK"}="1HMB TDK";
$resn_exch{"HMB2 TDK"}="2HMB TDK"; $resn_exch{"HMB3 TDK"}="3HMB TDK"; $resn_exch{"HMC1 TDK"}="1HMC TDK";
$resn_exch{"HMC2 TDK"}="2HMC TDK"; $resn_exch{"HMC3 TDK"}="3HMC TDK"; $resn_exch{"H7'1 TDK"}="1H7* TDK";
$resn_exch{"H7'2 TDK"}="2H7* TDK"; $resn_exch{" H6' TDK"}=" H6* TDK"; $resn_exch{"HM21 TDK"}="1HM2 TDK";
$resn_exch{"HM22 TDK"}="2HM2 TDK"; $resn_exch{"HM23 TDK"}="3HM2 TDK"; $resn_exch{"H4'1 TDK"}="1H4* TDK";
$resn_exch{"H4'2 TDK"}="2H4* TDK"; $resn_exch{" H71 TDL"}="1H7  TDL"; $resn_exch{" H72 TDL"}="2H7  TDL";
$resn_exch{" H61 TDL"}="1H6  TDL"; $resn_exch{" H62 TDL"}="2H6  TDL"; $resn_exch{"HM41 TDL"}="1HM4 TDL";
$resn_exch{"HM42 TDL"}="2HM4 TDL"; $resn_exch{"HM43 TDL"}="3HM4 TDL"; $resn_exch{"HLB1 TDL"}="1HLB TDL";
$resn_exch{"HLB2 TDL"}="2HLB TDL"; $resn_exch{"HLB3 TDL"}="3HLB TDL"; $resn_exch{"H7'1 TDL"}="1H7* TDL";
$resn_exch{"H7'2 TDL"}="2H7* TDL"; $resn_exch{" H6' TDL"}=" H6* TDL"; $resn_exch{"HM21 TDL"}="1HM2 TDL";
$resn_exch{"HM22 TDL"}="2HM2 TDL"; $resn_exch{"HM23 TDL"}="3HM2 TDL"; $resn_exch{"H4'1 TDL"}="1H4* TDL";
$resn_exch{"H4'2 TDL"}="2H4* TDL"; $resn_exch{" H71 TDM"}="1H7  TDM"; $resn_exch{" H72 TDM"}="2H7  TDM";
$resn_exch{" H61 TDM"}="1H6  TDM"; $resn_exch{" H62 TDM"}="2H6  TDM"; $resn_exch{"H291 TDM"}="1H29 TDM";
$resn_exch{"H292 TDM"}="2H29 TDM"; $resn_exch{"H293 TDM"}="3H29 TDM"; $resn_exch{"HM41 TDM"}="1HM4 TDM";
$resn_exch{"HM42 TDM"}="2HM4 TDM"; $resn_exch{"HM43 TDM"}="3HM4 TDM"; $resn_exch{"H7'1 TDM"}="1H7* TDM";
$resn_exch{"H7'2 TDM"}="2H7* TDM"; $resn_exch{"H4'1 TDM"}="1H4* TDM"; $resn_exch{"H4'2 TDM"}="2H4* TDM";
$resn_exch{"HM21 TDM"}="1HM2 TDM"; $resn_exch{"HM22 TDM"}="2HM2 TDM"; $resn_exch{"HM23 TDM"}="3HM2 TDM";
$resn_exch{" H6' TDM"}=" H6* TDM"; $resn_exch{"H3PO TDO"}="OH3P TDO"; $resn_exch{"H6C1 TDO"}="1H6C TDO";
$resn_exch{"H6C2 TDO"}="2H6C TDO"; $resn_exch{"H6C3 TDO"}="3H6C TDO"; $resn_exch{"H2PO TDO"}="OH2P TDO";
$resn_exch{"H5'1 TDO"}="1H5' TDO"; $resn_exch{"H5'2 TDO"}="2H5' TDO"; $resn_exch{"H2'1 TDO"}="1H2' TDO";
$resn_exch{"H2'2 TDO"}="2H2' TDO"; $resn_exch{"H5A1 TDO"}="1H5A TDO"; $resn_exch{"H5A2 TDO"}="2H5A TDO";
$resn_exch{"H5A3 TDO"}="3H5A TDO"; $resn_exch{"H2A1 TDP"}="1H2A TDP"; $resn_exch{"H2A2 TDP"}="2H2A TDP";
$resn_exch{"H2A3 TDP"}="3H2A TDP"; $resn_exch{"H4'1 TDP"}="1H4* TDP"; $resn_exch{"H4'2 TDP"}="2H4* TDP";
$resn_exch{" H6' TDP"}=" H6* TDP"; $resn_exch{"H351 TDP"}="1H35 TDP"; $resn_exch{"H352 TDP"}="2H35 TDP";
$resn_exch{"H4A1 TDP"}="1H4A TDP"; $resn_exch{"H4A2 TDP"}="2H4A TDP"; $resn_exch{"H4A3 TDP"}="3H4A TDP";
$resn_exch{"H5A1 TDP"}="1H5A TDP"; $resn_exch{"H5A2 TDP"}="2H5A TDP"; $resn_exch{"H5B1 TDP"}="1H5B TDP";
$resn_exch{"H5B2 TDP"}="2H5B TDP"; $resn_exch{"H5M1 TDR"}="1H5M TDR"; $resn_exch{"H5M2 TDR"}="2H5M TDR";
$resn_exch{"H5M3 TDR"}="3H5M TDR"; $resn_exch{"HBC1 TDS"}="1HBC TDS"; $resn_exch{"HBC2 TDS"}="2HBC TDS";
$resn_exch{"HBC3 TDS"}="3HBC TDS"; $resn_exch{"HBB1 TDS"}="1HBB TDS"; $resn_exch{"HBB2 TDS"}="2HBB TDS";
$resn_exch{"HBA1 TDS"}="1HBA TDS"; $resn_exch{"HBA2 TDS"}="2HBA TDS"; $resn_exch{"HAZ1 TDS"}="1HAZ TDS";
$resn_exch{"HAZ2 TDS"}="2HAZ TDS"; $resn_exch{"HAY1 TDS"}="1HAY TDS"; $resn_exch{"HAY2 TDS"}="2HAY TDS";
$resn_exch{"HAX1 TDS"}="1HAX TDS"; $resn_exch{"HAX2 TDS"}="2HAX TDS"; $resn_exch{"HAW1 TDS"}="1HAW TDS";
$resn_exch{"HAW2 TDS"}="2HAW TDS"; $resn_exch{"HAV1 TDS"}="1HAV TDS"; $resn_exch{"HAV2 TDS"}="2HAV TDS";
$resn_exch{"HAU1 TDS"}="1HAU TDS"; $resn_exch{"HAU2 TDS"}="2HAU TDS"; $resn_exch{"HAT1 TDS"}="1HAT TDS";
$resn_exch{"HAT2 TDS"}="2HAT TDS"; $resn_exch{"HAS1 TDS"}="1HAS TDS"; $resn_exch{"HAS2 TDS"}="2HAS TDS";
$resn_exch{"HAR1 TDS"}="1HAR TDS"; $resn_exch{"HAR2 TDS"}="2HAR TDS"; $resn_exch{"HAQ1 TDS"}="1HAQ TDS";
$resn_exch{"HAQ2 TDS"}="2HAQ TDS"; $resn_exch{"HAI1 TDS"}="1HAI TDS"; $resn_exch{"HAI2 TDS"}="2HAI TDS";
$resn_exch{"HAI3 TDS"}="3HAI TDS"; $resn_exch{"HAJ1 TDS"}="1HAJ TDS"; $resn_exch{"HAJ2 TDS"}="2HAJ TDS";
$resn_exch{"HAJ3 TDS"}="3HAJ TDS"; $resn_exch{"HAA1 TDS"}="1HAA TDS"; $resn_exch{"HAA2 TDS"}="2HAA TDS";
$resn_exch{"HAA3 TDS"}="3HAA TDS"; $resn_exch{"H5M1 TDT"}="1H5M TDT"; $resn_exch{"H5M2 TDT"}="2H5M TDT";
$resn_exch{"H7N1 TDT"}="1H7N TDT"; $resn_exch{"H7N2 TDT"}="2H7N TDT"; $resn_exch{"H5B1 TDT"}="1H5B TDT";
$resn_exch{"H5B2 TDT"}="2H5B TDT"; $resn_exch{"H6A1 TDT"}="1H6A TDT"; $resn_exch{"H6A2 TDT"}="2H6A TDT";
$resn_exch{" H21 TDT"}="1H2  TDT"; $resn_exch{" H22 TDT"}="2H2  TDT"; $resn_exch{" H31 TDT"}="1H3  TDT";
$resn_exch{" H32 TDT"}="2H3  TDT"; $resn_exch{" H41 TDT"}="1H4  TDT"; $resn_exch{" H42 TDT"}="2H4  TDT";
$resn_exch{" H51 TDT"}="1H5  TDT"; $resn_exch{" H52 TDT"}="2H5  TDT"; $resn_exch{" H61 TDT"}="1H6  TDT";
$resn_exch{" H62 TDT"}="2H6  TDT"; $resn_exch{" H71 TDT"}="1H7  TDT"; $resn_exch{" H72 TDT"}="2H7  TDT";
$resn_exch{"H5M1 TDX"}="1H5M TDX"; $resn_exch{"H5M2 TDX"}="2H5M TDX"; $resn_exch{"H5M3 TDX"}="3H5M TDX";
$resn_exch{" H1B TDX"}=" H1* TDX"; $resn_exch{"H2'1 TDX"}="1H2* TDX"; $resn_exch{"H2'2 TDX"}="2H2* TDX";
$resn_exch{" H3B TDX"}=" H3* TDX"; $resn_exch{"HO3' TDX"}="*HO3 TDX"; $resn_exch{" H4B TDX"}=" H4* TDX";
$resn_exch{"H5'1 TDX"}="1H5* TDX"; $resn_exch{"H5'2 TDX"}="2H5* TDX"; $resn_exch{" H5B TDX"}="1H5' TDX";
$resn_exch{" H5D TDX"}="2H5' TDX"; $resn_exch{"HO2' TDX"}="'HO2 TDX"; $resn_exch{"HO3A TDX"}="'HO3 TDX";
$resn_exch{"HO4' TDX"}="'HO4 TDX"; $resn_exch{"HAM1 TDZ"}="1HAM TDZ"; $resn_exch{"HAM2 TDZ"}="2HAM TDZ";
$resn_exch{"HAL1 TDZ"}="1HAL TDZ"; $resn_exch{"HAL2 TDZ"}="2HAL TDZ"; $resn_exch{"HAC1 TDZ"}="1HAC TDZ";
$resn_exch{"HAC2 TDZ"}="2HAC TDZ"; $resn_exch{"HAC3 TDZ"}="3HAC TDZ"; $resn_exch{"HAA1 TDZ"}="1HAA TDZ";
$resn_exch{"HAA2 TDZ"}="2HAA TDZ"; $resn_exch{"HAA3 TDZ"}="3HAA TDZ"; $resn_exch{"HAB1 TDZ"}="1HAB TDZ";
$resn_exch{"HAB2 TDZ"}="2HAB TDZ"; $resn_exch{"HAB3 TDZ"}="3HAB TDZ"; $resn_exch{"HAD1 TDZ"}="1HAD TDZ";
$resn_exch{"HAD2 TDZ"}="2HAD TDZ"; $resn_exch{"HAD3 TDZ"}="3HAD TDZ"; $resn_exch{"HAN1 TDZ"}="1HAN TDZ";
$resn_exch{"HAN2 TDZ"}="2HAN TDZ"; $resn_exch{"HAO1 TDZ"}="1HAO TDZ"; $resn_exch{"HAO2 TDZ"}="2HAO TDZ";
$resn_exch{"H111 TEA"}="1H11 TEA"; $resn_exch{"H112 TEA"}="2H11 TEA"; $resn_exch{"H121 TEA"}="1H12 TEA";
$resn_exch{"H122 TEA"}="2H12 TEA"; $resn_exch{"H123 TEA"}="3H12 TEA"; $resn_exch{"H211 TEA"}="1H21 TEA";
$resn_exch{"H212 TEA"}="2H21 TEA"; $resn_exch{"H221 TEA"}="1H22 TEA"; $resn_exch{"H222 TEA"}="2H22 TEA";
$resn_exch{"H223 TEA"}="3H22 TEA"; $resn_exch{"H311 TEA"}="1H31 TEA"; $resn_exch{"H312 TEA"}="2H31 TEA";
$resn_exch{"H321 TEA"}="1H32 TEA"; $resn_exch{"H322 TEA"}="2H32 TEA"; $resn_exch{"H323 TEA"}="3H32 TEA";
$resn_exch{" HN1 TEE"}="1HN  TEE"; $resn_exch{" HN2 TEE"}="2HN  TEE"; $resn_exch{"H151 TEI"}="1H15 TEI";
$resn_exch{"H152 TEI"}="2H15 TEI"; $resn_exch{"H171 TEI"}="1H17 TEI"; $resn_exch{"H172 TEI"}="2H17 TEI";
$resn_exch{"H173 TEI"}="3H17 TEI"; $resn_exch{"H181 TEI"}="1H18 TEI"; $resn_exch{"H182 TEI"}="2H18 TEI";
$resn_exch{"H183 TEI"}="3H18 TEI"; $resn_exch{"H191 TEI"}="1H19 TEI"; $resn_exch{"H192 TEI"}="2H19 TEI";
$resn_exch{"H193 TEI"}="3H19 TEI"; $resn_exch{" H11 TEL"}="1H1  TEL"; $resn_exch{" H12 TEL"}="2H1  TEL";
$resn_exch{" H13 TEL"}="3H1  TEL"; $resn_exch{" H81 TEL"}="1H8  TEL"; $resn_exch{" H82 TEL"}="2H8  TEL";
$resn_exch{"H111 TEL"}="1H11 TEL"; $resn_exch{"H112 TEL"}="2H11 TEL"; $resn_exch{"H121 TEL"}="1H12 TEL";
$resn_exch{"H122 TEL"}="2H12 TEL"; $resn_exch{"H123 TEL"}="3H12 TEL"; $resn_exch{"H141 TEL"}="1H14 TEL";
$resn_exch{"H142 TEL"}="2H14 TEL"; $resn_exch{"H143 TEL"}="3H14 TEL"; $resn_exch{"H171 TEL"}="1H17 TEL";
$resn_exch{"H172 TEL"}="2H17 TEL"; $resn_exch{"H221 TEL"}="1H22 TEL"; $resn_exch{"H222 TEL"}="2H22 TEL";
$resn_exch{"H231 TEL"}="1H23 TEL"; $resn_exch{"H232 TEL"}="2H23 TEL"; $resn_exch{"H233 TEL"}="3H23 TEL";
$resn_exch{"H241 TEL"}="1H24 TEL"; $resn_exch{"H242 TEL"}="2H24 TEL"; $resn_exch{"H251 TEL"}="1H25 TEL";
$resn_exch{"H252 TEL"}="2H25 TEL"; $resn_exch{"H253 TEL"}="3H25 TEL"; $resn_exch{"H271 TEL"}="1H27 TEL";
$resn_exch{"H272 TEL"}="2H27 TEL"; $resn_exch{"H331 TEL"}="1H33 TEL"; $resn_exch{"H332 TEL"}="2H33 TEL";
$resn_exch{"H333 TEL"}="3H33 TEL"; $resn_exch{"H351 TEL"}="1H35 TEL"; $resn_exch{"H352 TEL"}="2H35 TEL";
$resn_exch{"H353 TEL"}="3H35 TEL"; $resn_exch{"H381 TEL"}="1H38 TEL"; $resn_exch{"H382 TEL"}="2H38 TEL";
$resn_exch{"H383 TEL"}="3H38 TEL"; $resn_exch{"H541 TEL"}="1H54 TEL"; $resn_exch{"H542 TEL"}="2H54 TEL";
$resn_exch{"H551 TEL"}="1H55 TEL"; $resn_exch{"H552 TEL"}="2H55 TEL"; $resn_exch{"H553 TEL"}="3H55 TEL";
$resn_exch{"H571 TEL"}="1H57 TEL"; $resn_exch{"H572 TEL"}="2H57 TEL"; $resn_exch{"H573 TEL"}="3H57 TEL";
$resn_exch{"H581 TEL"}="1H58 TEL"; $resn_exch{"H582 TEL"}="2H58 TEL"; $resn_exch{"H583 TEL"}="3H58 TEL";
$resn_exch{" H31 TEM"}="1H3  TEM"; $resn_exch{" H32 TEM"}="2H3  TEM"; $resn_exch{" H91 TEM"}="1H9  TEM";
$resn_exch{" H92 TEM"}="2H9  TEM"; $resn_exch{"HOB4 TEO"}="4HOB TEO"; $resn_exch{" H11 TEP"}="1H1  TEP";
$resn_exch{" H12 TEP"}="2H1  TEP"; $resn_exch{" H13 TEP"}="3H1  TEP"; $resn_exch{" H31 TEP"}="1H3  TEP";
$resn_exch{" H32 TEP"}="2H3  TEP"; $resn_exch{" H33 TEP"}="3H3  TEP"; $resn_exch{"HN11 TER"}="1HN1 TER";
$resn_exch{"HN12 TER"}="2HN1 TER"; $resn_exch{" H21 TER"}="1H2  TER"; $resn_exch{" H22 TER"}="2H2  TER";
$resn_exch{" H31 TER"}="1H3  TER"; $resn_exch{" H32 TER"}="2H3  TER"; $resn_exch{" H41 TER"}="1H4  TER";
$resn_exch{" H42 TER"}="2H4  TER"; $resn_exch{" H61 TER"}="1H6  TER"; $resn_exch{" H62 TER"}="2H6  TER";
$resn_exch{" H71 TER"}="1H7  TER"; $resn_exch{" H72 TER"}="2H7  TER"; $resn_exch{" H81 TER"}="1H8  TER";
$resn_exch{" H82 TER"}="2H8  TER"; $resn_exch{"H101 TER"}="1H10 TER"; $resn_exch{"H102 TER"}="2H10 TER";
$resn_exch{"H111 TER"}="1H11 TER"; $resn_exch{"H112 TER"}="2H11 TER"; $resn_exch{"H121 TER"}="1H12 TER";
$resn_exch{"H122 TER"}="2H12 TER"; $resn_exch{"H131 TER"}="1H13 TER"; $resn_exch{"H132 TER"}="2H13 TER";
$resn_exch{"HN41 TER"}="1HN4 TER"; $resn_exch{"HN42 TER"}="2HN4 TER"; $resn_exch{" H11 TES"}="1H1  TES";
$resn_exch{" H12 TES"}="2H1  TES"; $resn_exch{" H21 TES"}="1H2  TES"; $resn_exch{" H22 TES"}="2H2  TES";
$resn_exch{" H61 TES"}="1H6  TES"; $resn_exch{" H62 TES"}="2H6  TES"; $resn_exch{" H71 TES"}="1H7  TES";
$resn_exch{" H72 TES"}="2H7  TES"; $resn_exch{"H111 TES"}="1H11 TES"; $resn_exch{"H112 TES"}="2H11 TES";
$resn_exch{"H121 TES"}="1H12 TES"; $resn_exch{"H122 TES"}="2H12 TES"; $resn_exch{"H151 TES"}="1H15 TES";
$resn_exch{"H152 TES"}="2H15 TES"; $resn_exch{"H161 TES"}="1H16 TES"; $resn_exch{"H162 TES"}="2H16 TES";
$resn_exch{"H181 TES"}="1H18 TES"; $resn_exch{"H182 TES"}="2H18 TES"; $resn_exch{"H183 TES"}="3H18 TES";
$resn_exch{"H191 TES"}="1H19 TES"; $resn_exch{"H192 TES"}="2H19 TES"; $resn_exch{"H193 TES"}="3H19 TES";
$resn_exch{"H121 TET"}="1H12 TET"; $resn_exch{"H122 TET"}="2H12 TET"; $resn_exch{"H171 TET"}="1H17 TET";
$resn_exch{"H172 TET"}="2H17 TET"; $resn_exch{" H81 TET"}="1H8  TET"; $resn_exch{" H82 TET"}="2H8  TET";
$resn_exch{" H83 TET"}="3H8  TET"; $resn_exch{"H191 TET"}="1H19 TET"; $resn_exch{"H192 TET"}="2H19 TET";
$resn_exch{"H3C1 TF1"}="1H3C TF1"; $resn_exch{"H3C2 TF1"}="2H3C TF1"; $resn_exch{"H5C1 TF1"}="1H5C TF1";
$resn_exch{"H5C2 TF1"}="2H5C TF1"; $resn_exch{"H6C1 TF1"}="1H6C TF1"; $resn_exch{"H6C2 TF1"}="2H6C TF1";
$resn_exch{"H6C3 TF1"}="3H6C TF1"; $resn_exch{"H9C1 TF1"}="1H9C TF1"; $resn_exch{"H9C2 TF1"}="2H9C TF1";
$resn_exch{"H9C3 TF1"}="3H9C TF1"; $resn_exch{"H101 TF1"}="1H10 TF1"; $resn_exch{"H102 TF1"}="2H10 TF1";
$resn_exch{"H103 TF1"}="3H10 TF1"; $resn_exch{"H7C1 TF2"}="1H7C TF2"; $resn_exch{"H7C2 TF2"}="2H7C TF2";
$resn_exch{"H9C1 TF2"}="1H9C TF2"; $resn_exch{"H9C2 TF2"}="2H9C TF2"; $resn_exch{"H9C3 TF2"}="3H9C TF2";
$resn_exch{"H121 TF2"}="1H12 TF2"; $resn_exch{"H122 TF2"}="2H12 TF2"; $resn_exch{"H123 TF2"}="3H12 TF2";
$resn_exch{"H191 TF2"}="1H19 TF2"; $resn_exch{"H192 TF2"}="2H19 TF2"; $resn_exch{"H193 TF2"}="3H19 TF2";
$resn_exch{"H6C1 TF3"}="1H6C TF3"; $resn_exch{"H6C2 TF3"}="2H6C TF3"; $resn_exch{"H9C1 TF3"}="1H9C TF3";
$resn_exch{"H9C2 TF3"}="2H9C TF3"; $resn_exch{"H161 TF3"}="1H16 TF3"; $resn_exch{"H162 TF3"}="2H16 TF3";
$resn_exch{"H181 TF3"}="1H18 TF3"; $resn_exch{"H182 TF3"}="2H18 TF3"; $resn_exch{"H191 TF3"}="1H19 TF3";
$resn_exch{"H192 TF3"}="2H19 TF3"; $resn_exch{"H193 TF3"}="3H19 TF3"; $resn_exch{"H201 TF3"}="1H20 TF3";
$resn_exch{"H202 TF3"}="2H20 TF3"; $resn_exch{"H203 TF3"}="3H20 TF3"; $resn_exch{"H2N1 TF3"}="1H2N TF3";
$resn_exch{"H2N2 TF3"}="2H2N TF3"; $resn_exch{"H171 TF5"}="1H17 TF5"; $resn_exch{"H172 TF5"}="2H17 TF5";
$resn_exch{"H161 TF5"}="1H16 TF5"; $resn_exch{"H162 TF5"}="2H16 TF5"; $resn_exch{" HB1 TFB"}="1HB  TFB";
$resn_exch{" HB2 TFB"}="2HB  TFB"; $resn_exch{" HG1 TFB"}="1HG  TFB"; $resn_exch{" HG2 TFB"}="2HG  TFB";
$resn_exch{" HD1 TFB"}="1HD  TFB"; $resn_exch{" HD2 TFB"}="2HD  TFB"; $resn_exch{" H11 TFC"}="1H1  TFC";
$resn_exch{" H12 TFC"}="2H1  TFC"; $resn_exch{" H13 TFC"}="3H1  TFC"; $resn_exch{" H21 TFC"}="1H2  TFC";
$resn_exch{" H22 TFC"}="2H2  TFC"; $resn_exch{" H31 TFC"}="1H3  TFC"; $resn_exch{" H32 TFC"}="2H3  TFC";
$resn_exch{" H41 TFC"}="1H4  TFC"; $resn_exch{" H42 TFC"}="2H4  TFC"; $resn_exch{" H51 TFC"}="1H5  TFC";
$resn_exch{" H52 TFC"}="2H5  TFC"; $resn_exch{" H61 TFC"}="1H6  TFC"; $resn_exch{" H62 TFC"}="2H6  TFC";
$resn_exch{" H71 TFC"}="1H7  TFC"; $resn_exch{" H72 TFC"}="2H7  TFC"; $resn_exch{" H81 TFC"}="1H8  TFC";
$resn_exch{" H82 TFC"}="2H8  TFC"; $resn_exch{" H91 TFC"}="1H9  TFC"; $resn_exch{" H92 TFC"}="2H9  TFC";
$resn_exch{"HOP2 TFE"}="2HOP TFE"; $resn_exch{" H5' TFE"}="1H5* TFE"; $resn_exch{"H5'' TFE"}="2H5* TFE";
$resn_exch{" H4' TFE"}=" H4* TFE"; $resn_exch{" H1' TFE"}=" H1* TFE"; $resn_exch{" H71 TFE"}="1H5M TFE";
$resn_exch{" H72 TFE"}="2H5M TFE"; $resn_exch{" H73 TFE"}="3H5M TFE"; $resn_exch{" H2' TFE"}=" H2* TFE";
$resn_exch{"HB'1 TFE"}="1HB' TFE"; $resn_exch{"HB'2 TFE"}="2HB' TFE"; $resn_exch{" H3' TFE"}=" H3* TFE";
$resn_exch{"HO3' TFE"}="H3T  TFE"; $resn_exch{"HOP3 TFE"}="3HOP TFE"; $resn_exch{" H51 TFI"}="1H5  TFI";
$resn_exch{" H52 TFI"}="2H5  TFI"; $resn_exch{" H53 TFI"}="3H5  TFI"; $resn_exch{" H61 TFI"}="1H6  TFI";
$resn_exch{" H62 TFI"}="2H6  TFI"; $resn_exch{" H63 TFI"}="3H6  TFI"; $resn_exch{" H81 TFI"}="1H8  TFI";
$resn_exch{" H82 TFI"}="2H8  TFI"; $resn_exch{"H191 TFI"}="1H19 TFI"; $resn_exch{"H192 TFI"}="2H19 TFI";
$resn_exch{"H193 TFI"}="3H19 TFI"; $resn_exch{"HN51 TFI"}="1HN5 TFI"; $resn_exch{"HN52 TFI"}="2HN5 TFI";
$resn_exch{" H51 TFK"}="1H5  TFK"; $resn_exch{" H52 TFK"}="2H5  TFK"; $resn_exch{" H53 TFK"}="3H5  TFK";
$resn_exch{" H61 TFK"}="1H6  TFK"; $resn_exch{" H62 TFK"}="2H6  TFK"; $resn_exch{" H63 TFK"}="3H6  TFK";
$resn_exch{" H81 TFK"}="1H8  TFK"; $resn_exch{" H82 TFK"}="2H8  TFK"; $resn_exch{"H201 TFK"}="1H20 TFK";
$resn_exch{"H202 TFK"}="2H20 TFK"; $resn_exch{"H203 TFK"}="3H20 TFK"; $resn_exch{"H151 TFL"}="1H15 TFL";
$resn_exch{"H152 TFL"}="2H15 TFL"; $resn_exch{"H153 TFL"}="3H15 TFL"; $resn_exch{"H161 TFL"}="1H16 TFL";
$resn_exch{"H162 TFL"}="2H16 TFL"; $resn_exch{"H163 TFL"}="3H16 TFL"; $resn_exch{"H171 TFL"}="1H17 TFL";
$resn_exch{"H172 TFL"}="2H17 TFL"; $resn_exch{"H173 TFL"}="3H17 TFL"; $resn_exch{"HN11 TFM"}="1HN1 TFM";
$resn_exch{"HN12 TFM"}="2HN1 TFM"; $resn_exch{" H21 TFM"}="1H2  TFM"; $resn_exch{" H31 TFM"}="1H3  TFM";
$resn_exch{" H51 TFM"}="1H5  TFM"; $resn_exch{" H61 TFM"}="1H6  TFM"; $resn_exch{"H1'1 TFM"}="1H1' TFM";
$resn_exch{"H1'2 TFM"}="2H1' TFM"; $resn_exch{"H2'1 TFM"}="1H2' TFM"; $resn_exch{"H2'2 TFM"}="2H2' TFM";
$resn_exch{"H2'3 TFM"}="3H2' TFM"; $resn_exch{"HOP2 TFO"}="2HOP TFO"; $resn_exch{"H9'1 TFO"}="1H9' TFO";
$resn_exch{"H9'2 TFO"}="2H9' TFO"; $resn_exch{"H8'1 TFO"}="1H8' TFO"; $resn_exch{"H8'2 TFO"}="2H8' TFO";
$resn_exch{"H8'3 TFO"}="3H8' TFO"; $resn_exch{"H6'1 TFO"}="1H6' TFO"; $resn_exch{"H6'2 TFO"}="2H6' TFO";
$resn_exch{"HN61 TFO"}="1HN6 TFO"; $resn_exch{"HN62 TFO"}="2HN6 TFO"; $resn_exch{"HOP3 TFO"}="3HOP TFO";
$resn_exch{"H131 TFP"}="1H13 TFP"; $resn_exch{"H132 TFP"}="2H13 TFP"; $resn_exch{"H141 TFP"}="1H14 TFP";
$resn_exch{"H142 TFP"}="2H14 TFP"; $resn_exch{"H151 TFP"}="1H15 TFP"; $resn_exch{"H152 TFP"}="2H15 TFP";
$resn_exch{"H161 TFP"}="1H16 TFP"; $resn_exch{"H162 TFP"}="2H16 TFP"; $resn_exch{"H171 TFP"}="1H17 TFP";
$resn_exch{"H172 TFP"}="2H17 TFP"; $resn_exch{"H181 TFP"}="1H18 TFP"; $resn_exch{"H182 TFP"}="2H18 TFP";
$resn_exch{"H191 TFP"}="1H19 TFP"; $resn_exch{"H192 TFP"}="2H19 TFP"; $resn_exch{"H201 TFP"}="1H20 TFP";
$resn_exch{"H202 TFP"}="2H20 TFP"; $resn_exch{"H203 TFP"}="3H20 TFP"; $resn_exch{"H131 TFQ"}="1H13 TFQ";
$resn_exch{"H132 TFQ"}="2H13 TFQ"; $resn_exch{" H71 TFQ"}="1H7  TFQ"; $resn_exch{" H72 TFQ"}="2H7  TFQ";
$resn_exch{"HN91 TFQ"}="1HN9 TFQ"; $resn_exch{"HN92 TFQ"}="2HN9 TFQ"; $resn_exch{"HOP2 TFT"}="2HOP TFT";
$resn_exch{" H71 TFT"}="1H5M TFT"; $resn_exch{" H72 TFT"}="2H5M TFT"; $resn_exch{" H73 TFT"}="3H5M TFT";
$resn_exch{"H2TC TFT"}="CH2T TFT"; $resn_exch{"H4T1 TFT"}="1H4T TFT"; $resn_exch{"H4T2 TFT"}="2H4T TFT";
$resn_exch{"H1TC TFT"}="CH1T TFT"; $resn_exch{"H3TC TFT"}="CH3T TFT"; $resn_exch{"HOP3 TFT"}="3HOP TFT";
$resn_exch{"H341 TG1"}="1H34 TG1"; $resn_exch{"H342 TG1"}="2H34 TG1"; $resn_exch{"H343 TG1"}="3H34 TG1";
$resn_exch{"HC91 TG1"}="1HC9 TG1"; $resn_exch{"HC92 TG1"}="2HC9 TG1"; $resn_exch{"H141 TG1"}="1H14 TG1";
$resn_exch{"H142 TG1"}="2H14 TG1"; $resn_exch{"H151 TG1"}="1H15 TG1"; $resn_exch{"H152 TG1"}="2H15 TG1";
$resn_exch{"H161 TG1"}="1H16 TG1"; $resn_exch{"H162 TG1"}="2H16 TG1"; $resn_exch{"H171 TG1"}="1H17 TG1";
$resn_exch{"H172 TG1"}="2H17 TG1"; $resn_exch{"H181 TG1"}="1H18 TG1"; $resn_exch{"H182 TG1"}="2H18 TG1";
$resn_exch{"H191 TG1"}="1H19 TG1"; $resn_exch{"H192 TG1"}="2H19 TG1"; $resn_exch{"H201 TG1"}="1H20 TG1";
$resn_exch{"H202 TG1"}="2H20 TG1"; $resn_exch{"H203 TG1"}="3H20 TG1"; $resn_exch{"H231 TG1"}="1H23 TG1";
$resn_exch{"H232 TG1"}="2H23 TG1"; $resn_exch{"H233 TG1"}="3H23 TG1"; $resn_exch{"H251 TG1"}="1H25 TG1";
$resn_exch{"H252 TG1"}="2H25 TG1"; $resn_exch{"H253 TG1"}="3H25 TG1"; $resn_exch{"H261 TG1"}="1H26 TG1";
$resn_exch{"H262 TG1"}="2H26 TG1"; $resn_exch{"H263 TG1"}="3H26 TG1"; $resn_exch{"H311 TG1"}="1H31 TG1";
$resn_exch{"H312 TG1"}="2H31 TG1"; $resn_exch{"H313 TG1"}="3H31 TG1"; $resn_exch{"H331 TG1"}="1H33 TG1";
$resn_exch{"H332 TG1"}="2H33 TG1"; $resn_exch{"H333 TG1"}="3H33 TG1"; $resn_exch{"H281 TG1"}="1H28 TG1";
$resn_exch{"H282 TG1"}="2H28 TG1"; $resn_exch{"H291 TG1"}="1H29 TG1"; $resn_exch{"H292 TG1"}="2H29 TG1";
$resn_exch{"H301 TG1"}="1H30 TG1"; $resn_exch{"H302 TG1"}="2H30 TG1"; $resn_exch{"H303 TG1"}="3H30 TG1";
$resn_exch{"H191 TGC"}="1H19 TGC"; $resn_exch{"H192 TGC"}="2H19 TGC"; $resn_exch{"HN61 TGC"}="1HN6 TGC";
$resn_exch{"HN62 TGC"}="2HN6 TGC"; $resn_exch{"H461 TGC"}="1H46 TGC"; $resn_exch{"H462 TGC"}="2H46 TGC";
$resn_exch{"H531 TGC"}="1H53 TGC"; $resn_exch{"H532 TGC"}="2H53 TGC"; $resn_exch{"H533 TGC"}="3H53 TGC";
$resn_exch{"H571 TGC"}="1H57 TGC"; $resn_exch{"H572 TGC"}="2H57 TGC"; $resn_exch{"H573 TGC"}="3H57 TGC";
$resn_exch{"H621 TGC"}="1H62 TGC"; $resn_exch{"H622 TGC"}="2H62 TGC"; $resn_exch{"H611 TGC"}="1H61 TGC";
$resn_exch{"H612 TGC"}="2H61 TGC"; $resn_exch{"H741 TGC"}="1H74 TGC"; $resn_exch{"H742 TGC"}="2H74 TGC";
$resn_exch{"H731 TGC"}="1H73 TGC"; $resn_exch{"H732 TGC"}="2H73 TGC"; $resn_exch{"H2'1 TGC"}="1H2' TGC";
$resn_exch{"H2'2 TGC"}="2H2' TGC"; $resn_exch{"H3'1 TGC"}="1H3' TGC"; $resn_exch{"H3'2 TGC"}="2H3' TGC";
$resn_exch{"HA21 TGF"}="1HA2 TGF"; $resn_exch{"HA22 TGF"}="2HA2 TGF"; $resn_exch{" H71 TGF"}="1H7  TGF";
$resn_exch{" H72 TGF"}="2H7  TGF"; $resn_exch{" H91 TGF"}="1H9  TGF"; $resn_exch{" H92 TGF"}="2H9  TGF";
$resn_exch{"HB11 TGF"}="1HB1 TGF"; $resn_exch{"HB12 TGF"}="2HB1 TGF"; $resn_exch{"HG11 TGF"}="1HG1 TGF";
$resn_exch{"HG12 TGF"}="2HG1 TGF"; $resn_exch{"HB21 TGF"}="1HB2 TGF"; $resn_exch{"HB22 TGF"}="2HB2 TGF";
$resn_exch{"HG21 TGF"}="1HG2 TGF"; $resn_exch{"HG22 TGF"}="2HG2 TGF"; $resn_exch{"HB31 TGF"}="1HB3 TGF";
$resn_exch{"HB32 TGF"}="2HB3 TGF"; $resn_exch{"HG31 TGF"}="1HG3 TGF"; $resn_exch{"HG32 TGF"}="2HG3 TGF";
$resn_exch{"H23E TGF"}="EH23 TGF"; $resn_exch{"HN11 TGG"}="1HN1 TGG"; $resn_exch{"HN12 TGG"}="2HN1 TGG";
$resn_exch{"HB11 TGG"}="1HB1 TGG"; $resn_exch{"HB12 TGG"}="2HB1 TGG"; $resn_exch{"HG11 TGG"}="1HG1 TGG";
$resn_exch{"HG12 TGG"}="2HG1 TGG"; $resn_exch{"HA31 TGG"}="1HA3 TGG"; $resn_exch{"HA32 TGG"}="2HA3 TGG";
$resn_exch{"HB21 TGG"}="1HB2 TGG"; $resn_exch{"HB22 TGG"}="2HB2 TGG"; $resn_exch{"HF21 TGG"}="1HF2 TGG";
$resn_exch{"HF22 TGG"}="2HF2 TGG"; $resn_exch{"HA21 TGL"}="1HA2 TGL"; $resn_exch{"HA22 TGL"}="2HA2 TGL";
$resn_exch{"HA31 TGL"}="1HA3 TGL"; $resn_exch{"HA32 TGL"}="2HA3 TGL"; $resn_exch{"HA41 TGL"}="1HA4 TGL";
$resn_exch{"HA42 TGL"}="2HA4 TGL"; $resn_exch{"HA51 TGL"}="1HA5 TGL"; $resn_exch{"HA52 TGL"}="2HA5 TGL";
$resn_exch{"HA61 TGL"}="1HA6 TGL"; $resn_exch{"HA62 TGL"}="2HA6 TGL"; $resn_exch{"HA71 TGL"}="1HA7 TGL";
$resn_exch{"HA72 TGL"}="2HA7 TGL"; $resn_exch{"HA81 TGL"}="1HA8 TGL"; $resn_exch{"HA82 TGL"}="2HA8 TGL";
$resn_exch{"HA91 TGL"}="1HA9 TGL"; $resn_exch{"HA92 TGL"}="2HA9 TGL"; $resn_exch{"HB21 TGL"}="1HB2 TGL";
$resn_exch{"HB22 TGL"}="2HB2 TGL"; $resn_exch{"HB31 TGL"}="1HB3 TGL"; $resn_exch{"HB32 TGL"}="2HB3 TGL";
$resn_exch{"HB41 TGL"}="1HB4 TGL"; $resn_exch{"HB42 TGL"}="2HB4 TGL"; $resn_exch{"HB51 TGL"}="1HB5 TGL";
$resn_exch{"HB52 TGL"}="2HB5 TGL"; $resn_exch{"HB61 TGL"}="1HB6 TGL"; $resn_exch{"HB62 TGL"}="2HB6 TGL";
$resn_exch{"HB71 TGL"}="1HB7 TGL"; $resn_exch{"HB72 TGL"}="2HB7 TGL"; $resn_exch{"HB81 TGL"}="1HB8 TGL";
$resn_exch{"HB82 TGL"}="2HB8 TGL"; $resn_exch{"HB91 TGL"}="1HB9 TGL"; $resn_exch{"HB92 TGL"}="2HB9 TGL";
$resn_exch{"H101 TGL"}="1H10 TGL"; $resn_exch{"H102 TGL"}="2H10 TGL"; $resn_exch{"H111 TGL"}="1H11 TGL";
$resn_exch{"H112 TGL"}="2H11 TGL"; $resn_exch{"H121 TGL"}="1H12 TGL"; $resn_exch{"H122 TGL"}="2H12 TGL";
$resn_exch{"H131 TGL"}="1H13 TGL"; $resn_exch{"H132 TGL"}="2H13 TGL"; $resn_exch{"H141 TGL"}="1H14 TGL";
$resn_exch{"H142 TGL"}="2H14 TGL"; $resn_exch{"HG11 TGL"}="1HG1 TGL"; $resn_exch{"HG12 TGL"}="2HG1 TGL";
$resn_exch{"HG31 TGL"}="1HG3 TGL"; $resn_exch{"HG32 TGL"}="2HG3 TGL"; $resn_exch{"HC21 TGL"}="1HC2 TGL";
$resn_exch{"HC22 TGL"}="2HC2 TGL"; $resn_exch{"HC31 TGL"}="1HC3 TGL"; $resn_exch{"HC32 TGL"}="2HC3 TGL";
$resn_exch{"HC41 TGL"}="1HC4 TGL"; $resn_exch{"HC42 TGL"}="2HC4 TGL"; $resn_exch{"HC51 TGL"}="1HC5 TGL";
$resn_exch{"HC52 TGL"}="2HC5 TGL"; $resn_exch{"HC61 TGL"}="1HC6 TGL"; $resn_exch{"HC62 TGL"}="2HC6 TGL";
$resn_exch{"HC71 TGL"}="1HC7 TGL"; $resn_exch{"HC72 TGL"}="2HC7 TGL"; $resn_exch{"HC81 TGL"}="1HC8 TGL";
$resn_exch{"HC82 TGL"}="2HC8 TGL"; $resn_exch{"HC91 TGL"}="1HC9 TGL"; $resn_exch{"HC92 TGL"}="2HC9 TGL";
$resn_exch{"H151 TGL"}="1H15 TGL"; $resn_exch{"H152 TGL"}="2H15 TGL"; $resn_exch{"H161 TGL"}="1H16 TGL";
$resn_exch{"H162 TGL"}="2H16 TGL"; $resn_exch{"H171 TGL"}="1H17 TGL"; $resn_exch{"H172 TGL"}="2H17 TGL";
$resn_exch{"H181 TGL"}="1H18 TGL"; $resn_exch{"H182 TGL"}="2H18 TGL"; $resn_exch{"H191 TGL"}="1H19 TGL";
$resn_exch{"H192 TGL"}="2H19 TGL"; $resn_exch{"H201 TGL"}="1H20 TGL"; $resn_exch{"H202 TGL"}="2H20 TGL";
$resn_exch{"H211 TGL"}="1H21 TGL"; $resn_exch{"H212 TGL"}="2H21 TGL"; $resn_exch{"H221 TGL"}="1H22 TGL";
$resn_exch{"H222 TGL"}="2H22 TGL"; $resn_exch{"H231 TGL"}="1H23 TGL"; $resn_exch{"H232 TGL"}="2H23 TGL";
$resn_exch{"H241 TGL"}="1H24 TGL"; $resn_exch{"H242 TGL"}="2H24 TGL"; $resn_exch{"H251 TGL"}="1H25 TGL";
$resn_exch{"H252 TGL"}="2H25 TGL"; $resn_exch{"H261 TGL"}="1H26 TGL"; $resn_exch{"H262 TGL"}="2H26 TGL";
$resn_exch{"H271 TGL"}="1H27 TGL"; $resn_exch{"H272 TGL"}="2H27 TGL"; $resn_exch{"H281 TGL"}="1H28 TGL";
$resn_exch{"H282 TGL"}="2H28 TGL"; $resn_exch{"H283 TGL"}="3H28 TGL"; $resn_exch{"H291 TGL"}="1H29 TGL";
$resn_exch{"H292 TGL"}="2H29 TGL"; $resn_exch{"H301 TGL"}="1H30 TGL"; $resn_exch{"H302 TGL"}="2H30 TGL";
$resn_exch{"H311 TGL"}="1H31 TGL"; $resn_exch{"H312 TGL"}="2H31 TGL"; $resn_exch{"H321 TGL"}="1H32 TGL";
$resn_exch{"H322 TGL"}="2H32 TGL"; $resn_exch{"H323 TGL"}="3H32 TGL"; $resn_exch{"H331 TGL"}="1H33 TGL";
$resn_exch{"H332 TGL"}="2H33 TGL"; $resn_exch{"H341 TGL"}="1H34 TGL"; $resn_exch{"H342 TGL"}="2H34 TGL";
$resn_exch{"H351 TGL"}="1H35 TGL"; $resn_exch{"H352 TGL"}="2H35 TGL"; $resn_exch{"H361 TGL"}="1H36 TGL";
$resn_exch{"H362 TGL"}="2H36 TGL"; $resn_exch{"H363 TGL"}="3H36 TGL"; $resn_exch{"H141 TGN"}="1H14 TGN";
$resn_exch{"H142 TGN"}="2H14 TGN"; $resn_exch{"H143 TGN"}="3H14 TGN"; $resn_exch{"H181 TGN"}="1H18 TGN";
$resn_exch{"H182 TGN"}="2H18 TGN"; $resn_exch{"H211 TGN"}="1H21 TGN"; $resn_exch{"H212 TGN"}="2H21 TGN";
$resn_exch{"H261 TGN"}="1H26 TGN"; $resn_exch{"H262 TGN"}="2H26 TGN"; $resn_exch{"H11A TGN"}="1H1  TGN";
$resn_exch{" H12 TGN"}="2H1  TGN"; $resn_exch{"H13A TGN"}="3H1  TGN"; $resn_exch{" H21 TGP"}="1H2  TGP";
$resn_exch{" H22 TGP"}="2H2  TGP"; $resn_exch{" H2' TGP"}="1H2* TGP"; $resn_exch{"H2'' TGP"}="2H2* TGP";
$resn_exch{" H5' TGP"}="1H5* TGP"; $resn_exch{"H5'' TGP"}="2H5* TGP"; $resn_exch{" H4' TGP"}=" H4* TGP";
$resn_exch{" H1' TGP"}=" H1* TGP"; $resn_exch{" H3' TGP"}=" H3* TGP"; $resn_exch{"HO3' TGP"}="H3T  TGP";
$resn_exch{"HOP3 TGP"}="3HOP TGP"; $resn_exch{"HOP2 TGP"}="2HOP TGP"; $resn_exch{" H61 TGT"}="1H6  TGT";
$resn_exch{" H62 TGT"}="2H6  TGT"; $resn_exch{"H111 TGT"}="1H11 TGT"; $resn_exch{"H112 TGT"}="2H11 TGT";
$resn_exch{"H113 TGT"}="3H11 TGT"; $resn_exch{"HN11 TGT"}="1HN1 TGT"; $resn_exch{"HN12 TGT"}="2HN1 TGT";
$resn_exch{"HN21 TGT"}="1HN2 TGT"; $resn_exch{"HN22 TGT"}="2HN2 TGT"; $resn_exch{" H61 TH1"}="1H6  TH1";
$resn_exch{" H62 TH1"}="2H6  TH1"; $resn_exch{"H141 TH1"}="1H14 TH1"; $resn_exch{"H142 TH1"}="2H14 TH1";
$resn_exch{"H143 TH1"}="3H14 TH1"; $resn_exch{"H1C1 TH2"}="1H1C TH2"; $resn_exch{"H1C2 TH2"}="2H1C TH2";
$resn_exch{"H2C1 TH2"}="1H2C TH2"; $resn_exch{"H2C2 TH2"}="2H2C TH2"; $resn_exch{"H6C1 TH2"}="1H6C TH2";
$resn_exch{"H6C2 TH2"}="2H6C TH2"; $resn_exch{"H7C1 TH2"}="1H7C TH2"; $resn_exch{"H7C2 TH2"}="2H7C TH2";
$resn_exch{"H111 TH2"}="1H11 TH2"; $resn_exch{"H112 TH2"}="2H11 TH2"; $resn_exch{"H121 TH2"}="1H12 TH2";
$resn_exch{"H122 TH2"}="2H12 TH2"; $resn_exch{"H151 TH2"}="1H15 TH2"; $resn_exch{"H152 TH2"}="2H15 TH2";
$resn_exch{"H161 TH2"}="1H16 TH2"; $resn_exch{"H162 TH2"}="2H16 TH2"; $resn_exch{"H181 TH2"}="1H18 TH2";
$resn_exch{"H182 TH2"}="2H18 TH2"; $resn_exch{"H183 TH2"}="3H18 TH2"; $resn_exch{"H211 TH2"}="1H21 TH2";
$resn_exch{"H212 TH2"}="2H21 TH2"; $resn_exch{"H221 TH2"}="1H22 TH2"; $resn_exch{"H222 TH2"}="2H22 TH2";
$resn_exch{"H191 TH2"}="1H19 TH2"; $resn_exch{"H192 TH2"}="2H19 TH2"; $resn_exch{"H193 TH2"}="3H19 TH2";
$resn_exch{"H1C1 TH3"}="1H1C TH3"; $resn_exch{"H1C2 TH3"}="2H1C TH3"; $resn_exch{"H4C1 TH3"}="1H4C TH3";
$resn_exch{"H4C2 TH3"}="2H4C TH3"; $resn_exch{"H5C1 TH3"}="1H5C TH3"; $resn_exch{"H5C2 TH3"}="2H5C TH3";
$resn_exch{"H101 TH3"}="1H10 TH3"; $resn_exch{"H102 TH3"}="2H10 TH3"; $resn_exch{"H111 TH3"}="1H11 TH3";
$resn_exch{"H112 TH3"}="2H11 TH3"; $resn_exch{"H12A TH3"}="1H12 TH3"; $resn_exch{"H12B TH3"}="2H12 TH3";
$resn_exch{"H13A TH3"}="1H13 TH3"; $resn_exch{"H13B TH3"}="2H13 TH3"; $resn_exch{"H14A TH3"}="1H14 TH3";
$resn_exch{"H14B TH3"}="2H14 TH3"; $resn_exch{"H151 TH3"}="1H15 TH3"; $resn_exch{"H152 TH3"}="2H15 TH3";
$resn_exch{"H161 TH3"}="1H16 TH3"; $resn_exch{"H162 TH3"}="2H16 TH3"; $resn_exch{"H171 TH3"}="1H17 TH3";
$resn_exch{"H172 TH3"}="2H17 TH3"; $resn_exch{"H181 TH3"}="1H18 TH3"; $resn_exch{"H182 TH3"}="2H18 TH3";
$resn_exch{"H191 TH3"}="1H19 TH3"; $resn_exch{"H192 TH3"}="2H19 TH3"; $resn_exch{"H201 TH3"}="1H20 TH3";
$resn_exch{"H202 TH3"}="2H20 TH3"; $resn_exch{"H211 TH3"}="1H21 TH3"; $resn_exch{"H212 TH3"}="2H21 TH3";
$resn_exch{"H111 THA"}="1H11 THA"; $resn_exch{"H112 THA"}="2H11 THA"; $resn_exch{"H121 THA"}="1H12 THA";
$resn_exch{"H122 THA"}="2H12 THA"; $resn_exch{"H131 THA"}="1H13 THA"; $resn_exch{"H132 THA"}="2H13 THA";
$resn_exch{"H141 THA"}="1H14 THA"; $resn_exch{"H142 THA"}="2H14 THA"; $resn_exch{"HN51 THA"}="1HN5 THA";
$resn_exch{"HN52 THA"}="2HN5 THA"; $resn_exch{"HN21 THB"}="1HN2 THB"; $resn_exch{"HN22 THB"}="2HN2 THB";
$resn_exch{" H71 THB"}="1H7  THB"; $resn_exch{" H72 THB"}="2H7  THB"; $resn_exch{"H111 THB"}="1H11 THB";
$resn_exch{"H112 THB"}="2H11 THB"; $resn_exch{"H113 THB"}="3H11 THB"; $resn_exch{" H   THC"}=" HN  THC";
$resn_exch{" HM1 THC"}="1HM  THC"; $resn_exch{" HM2 THC"}="2HM  THC"; $resn_exch{" HM3 THC"}="3HM  THC";
$resn_exch{"HG21 THC"}="1HG2 THC"; $resn_exch{"HG22 THC"}="2HG2 THC"; $resn_exch{"HG23 THC"}="3HG2 THC";
$resn_exch{"HM21 THD"}="1HM2 THD"; $resn_exch{"HM22 THD"}="2HM2 THD"; $resn_exch{"HM23 THD"}="3HM2 THD";
$resn_exch{"H4'1 THD"}="1H4' THD"; $resn_exch{"H4'2 THD"}="2H4' THD"; $resn_exch{"H7'1 THD"}="1H7' THD";
$resn_exch{"H7'2 THD"}="2H7' THD"; $resn_exch{"HM41 THD"}="1HM4 THD"; $resn_exch{"HM42 THD"}="2HM4 THD";
$resn_exch{"HM43 THD"}="3HM4 THD"; $resn_exch{"H6C1 THD"}="1H6C THD"; $resn_exch{"H6C2 THD"}="2H6C THD";
$resn_exch{"H7C1 THD"}="1H7C THD"; $resn_exch{"H7C2 THD"}="2H7C THD"; $resn_exch{"H9C1 THD"}="1H9C THD";
$resn_exch{"H9C2 THD"}="2H9C THD"; $resn_exch{" H11 THE"}="1H1  THE"; $resn_exch{" H12 THE"}="2H1  THE";
$resn_exch{"HN21 THF"}="1HN2 THF"; $resn_exch{"HN22 THF"}="2HN2 THF"; $resn_exch{" H91 THF"}="1H9  THF";
$resn_exch{" H92 THF"}="2H9  THF"; $resn_exch{" HB1 THF"}="1HB  THF"; $resn_exch{" HB2 THF"}="2HB  THF";
$resn_exch{" HG1 THF"}="1HG  THF"; $resn_exch{" HG2 THF"}="2HG  THF"; $resn_exch{"HOE2 THF"}="2HOE THF";
$resn_exch{"HCP1 THF"}="1HCP THF"; $resn_exch{"HCP2 THF"}="2HCP THF"; $resn_exch{"HC71 THG"}="1HC7 THG";
$resn_exch{"HC72 THG"}="2HC7 THG"; $resn_exch{"HC91 THG"}="1HC9 THG"; $resn_exch{"HC92 THG"}="2HC9 THG";
$resn_exch{"HC61 THG"}="1HC6 THG"; $resn_exch{"HCB1 THG"}="1HCB THG"; $resn_exch{"HCB2 THG"}="2HCB THG";
$resn_exch{"HCG1 THG"}="1HCG THG"; $resn_exch{"HCG2 THG"}="2HCG THG"; $resn_exch{"HN21 THG"}="1HN2 THG";
$resn_exch{"HN22 THG"}="2HN2 THG"; $resn_exch{" HG1 THH"}="1HG  THH"; $resn_exch{" HG2 THH"}="2HG  THH";
$resn_exch{" HB1 THH"}="1HB  THH"; $resn_exch{" HB2 THH"}="2HB  THH"; $resn_exch{" H91 THH"}="1H9  THH";
$resn_exch{" H92 THH"}="2H9  THH"; $resn_exch{" H71 THH"}="1H7  THH"; $resn_exch{" H72 THH"}="2H7  THH";
$resn_exch{"H111 THH"}="1H11 THH"; $resn_exch{"H112 THH"}="2H11 THH"; $resn_exch{"H113 THH"}="3H11 THH";
$resn_exch{"HA21 THH"}="1HA2 THH"; $resn_exch{"HA22 THH"}="2HA2 THH"; $resn_exch{" H21 THI"}="1H2  THI";
$resn_exch{" H22 THI"}="2H2  THI"; $resn_exch{" H31 THI"}="1H3  THI"; $resn_exch{" H32 THI"}="2H3  THI";
$resn_exch{"H21A THK"}="1H2  THK"; $resn_exch{"H22A THK"}="2H2  THK"; $resn_exch{" H31 THK"}="1H3  THK";
$resn_exch{" H32 THK"}="2H3  THK"; $resn_exch{" H51 THK"}="1H5  THK"; $resn_exch{" H52 THK"}="2H5  THK";
$resn_exch{" H61 THK"}="1H6  THK"; $resn_exch{" H62 THK"}="2H6  THK"; $resn_exch{" H71 THK"}="1H7  THK";
$resn_exch{" H72 THK"}="2H7  THK"; $resn_exch{" H81 THK"}="1H8  THK"; $resn_exch{" H82 THK"}="2H8  THK";
$resn_exch{" H91 THK"}="1H9  THK"; $resn_exch{" H92 THK"}="2H9  THK"; $resn_exch{"H521 THK"}="1H52 THK";
$resn_exch{"H522 THK"}="2H52 THK"; $resn_exch{"H551 THK"}="1H55 THK"; $resn_exch{"H552 THK"}="2H55 THK";
$resn_exch{" H71 THL"}="1H7  THL"; $resn_exch{" H72 THL"}="2H7  THL"; $resn_exch{" H91 THL"}="1H9  THL";
$resn_exch{" H92 THL"}="2H9  THL"; $resn_exch{"HA21 THL"}="1HA2 THL"; $resn_exch{"HA22 THL"}="2HA2 THL";
$resn_exch{" HB1 THL"}="1HB  THL"; $resn_exch{" HB2 THL"}="2HB  THL"; $resn_exch{" HG1 THL"}="1HG  THL";
$resn_exch{" HG2 THL"}="2HG  THL"; $resn_exch{"HO5' THM"}="*HO5 THM"; $resn_exch{"H5'1 THM"}="1H5* THM";
$resn_exch{"H5'2 THM"}="2H5* THM"; $resn_exch{" H4' THM"}=" H4* THM"; $resn_exch{" H3' THM"}=" H3* THM";
$resn_exch{"HO3' THM"}="*HO3 THM"; $resn_exch{"H2'1 THM"}="1H2* THM"; $resn_exch{"H2'2 THM"}="2H2* THM";
$resn_exch{" H1' THM"}=" H1* THM"; $resn_exch{"HM51 THM"}="1HM5 THM"; $resn_exch{"HM52 THM"}="2HM5 THM";
$resn_exch{"HM53 THM"}="3HM5 THM"; $resn_exch{" H21 THN"}="1H2  THN"; $resn_exch{" H22 THN"}="2H2  THN";
$resn_exch{"H3'1 THN"}="1H3' THN"; $resn_exch{"H3'2 THN"}="2H3' THN"; $resn_exch{"H131 THN"}="1H13 THN";
$resn_exch{"H132 THN"}="2H13 THN"; $resn_exch{" H   THO"}="1HN  THO"; $resn_exch{" H2  THO"}="2HN  THO";
$resn_exch{"HG21 THO"}="1HG2 THO"; $resn_exch{"HG22 THO"}="2HG2 THO"; $resn_exch{"HG23 THO"}="3HG2 THO";
$resn_exch{" HC1 THO"}="1H   THO"; $resn_exch{" HC2 THO"}="2H   THO"; $resn_exch{"HOP5 THP"}="5HOP THP";
$resn_exch{"HOP6 THP"}="6HOP THP"; $resn_exch{"HOP2 THP"}="2HOP THP"; $resn_exch{"HOP3 THP"}="3HOP THP";
$resn_exch{"H5'1 THP"}="1H5* THP"; $resn_exch{"H5'2 THP"}="2H5* THP"; $resn_exch{" H4' THP"}=" H4* THP";
$resn_exch{" H3' THP"}=" H3* THP"; $resn_exch{"H2'1 THP"}="1H2* THP"; $resn_exch{"H2'2 THP"}="2H2* THP";
$resn_exch{" H1' THP"}=" H1* THP"; $resn_exch{" H51 THP"}="1H5  THP"; $resn_exch{" H52 THP"}="2H5  THP";
$resn_exch{" H53 THP"}="3H5  THP"; $resn_exch{" H11 THQ"}="1H1  THQ"; $resn_exch{"HM31 THQ"}="1HM3 THQ";
$resn_exch{"HM32 THQ"}="2HM3 THQ"; $resn_exch{"HM33 THQ"}="3HM3 THQ"; $resn_exch{"HC41 THQ"}="1HC4 THQ";
$resn_exch{"HC42 THQ"}="2HC4 THQ"; $resn_exch{"HC51 THQ"}="1HC5 THQ"; $resn_exch{"HC52 THQ"}="2HC5 THQ";
$resn_exch{" H91 THQ"}="1H9  THQ"; $resn_exch{"H101 THQ"}="1H10 THQ"; $resn_exch{" H2  THR"}=" HN2 THR";
$resn_exch{"HG21 THR"}="1HG2 THR"; $resn_exch{"HG22 THR"}="2HG2 THR"; $resn_exch{"HG23 THR"}="3HG2 THR";
$resn_exch{"HSP2 THS"}="2HSP THS"; $resn_exch{"HSP3 THS"}="3HSP THS"; $resn_exch{"H5'1 THS"}="1H5* THS";
$resn_exch{"H5'2 THS"}="2H5* THS"; $resn_exch{" H4' THS"}=" H4* THS"; $resn_exch{" H3' THS"}=" H3* THS";
$resn_exch{"HO3' THS"}="*HO3 THS"; $resn_exch{"H2'1 THS"}="1H2* THS"; $resn_exch{"H2'2 THS"}="2H2* THS";
$resn_exch{" H1' THS"}=" H1* THS"; $resn_exch{"HM51 THS"}="1HM5 THS"; $resn_exch{"HM52 THS"}="2HM5 THS";
$resn_exch{"HM53 THS"}="3HM5 THS"; $resn_exch{"HC11 THT"}="1HC1 THT"; $resn_exch{"HC12 THT"}="2HC1 THT";
$resn_exch{"HC13 THT"}="3HC1 THT"; $resn_exch{"HN31 THT"}="1HN3 THT"; $resn_exch{"HC41 THT"}="1HC4 THT";
$resn_exch{"HC42 THT"}="2HC4 THT"; $resn_exch{"HC51 THT"}="1HC5 THT"; $resn_exch{"HC52 THT"}="2HC5 THT";
$resn_exch{"HC81 THT"}="1HC8 THT"; $resn_exch{"HC82 THT"}="2HC8 THT"; $resn_exch{"HC91 THT"}="1HC9 THT";
$resn_exch{"HC92 THT"}="2HC9 THT"; $resn_exch{"H101 THT"}="1H10 THT"; $resn_exch{"H102 THT"}="2H10 THT";
$resn_exch{"H111 THT"}="1H11 THT"; $resn_exch{"H112 THT"}="2H11 THT"; $resn_exch{"H121 THT"}="1H12 THT";
$resn_exch{"H122 THT"}="2H12 THT"; $resn_exch{"H131 THT"}="1H13 THT"; $resn_exch{"H132 THT"}="2H13 THT";
$resn_exch{"H141 THT"}="1H14 THT"; $resn_exch{"H142 THT"}="2H14 THT"; $resn_exch{"H151 THT"}="1H15 THT";
$resn_exch{"H152 THT"}="2H15 THT"; $resn_exch{"H161 THT"}="1H16 THT"; $resn_exch{"H162 THT"}="2H16 THT";
$resn_exch{"H171 THT"}="1H17 THT"; $resn_exch{"H172 THT"}="2H17 THT"; $resn_exch{"H181 THT"}="1H18 THT";
$resn_exch{"H182 THT"}="2H18 THT"; $resn_exch{"H191 THT"}="1H19 THT"; $resn_exch{"H192 THT"}="2H19 THT";
$resn_exch{"H201 THT"}="1H20 THT"; $resn_exch{"H202 THT"}="2H20 THT"; $resn_exch{"H211 THT"}="1H21 THT";
$resn_exch{"H212 THT"}="2H21 THT"; $resn_exch{"H221 THT"}="1H22 THT"; $resn_exch{"H222 THT"}="2H22 THT";
$resn_exch{"H223 THT"}="3H22 THT"; $resn_exch{" H51 THU"}="1H5  THU"; $resn_exch{" H52 THU"}="2H5  THU";
$resn_exch{" H61 THU"}="1H6  THU"; $resn_exch{" H62 THU"}="2H6  THU"; $resn_exch{" H1' THU"}=" H1* THU";
$resn_exch{"H2'1 THU"}="1H2* THU"; $resn_exch{"H2'2 THU"}="2H2* THU"; $resn_exch{" H3' THU"}=" H3* THU";
$resn_exch{"HO3' THU"}="*HO3 THU"; $resn_exch{" H4' THU"}=" H4* THU"; $resn_exch{"H5'1 THU"}="1H5* THU";
$resn_exch{"H5'2 THU"}="2H5* THU"; $resn_exch{"HO5' THU"}="*HO5 THU"; $resn_exch{"H5B1 THV"}="1H5B THV";
$resn_exch{"H5B2 THV"}="2H5B THV"; $resn_exch{"H5A1 THV"}="1H5A THV"; $resn_exch{"H5A2 THV"}="2H5A THV";
$resn_exch{"H4A1 THV"}="1H4A THV"; $resn_exch{"H4A2 THV"}="2H4A THV"; $resn_exch{"H4A3 THV"}="3H4A THV";
$resn_exch{"H101 THV"}="1H10 THV"; $resn_exch{"H102 THV"}="2H10 THV"; $resn_exch{"H103 THV"}="3H10 THV";
$resn_exch{"H111 THV"}="1H11 THV"; $resn_exch{"H112 THV"}="2H11 THV"; $resn_exch{"H113 THV"}="3H11 THV";
$resn_exch{"H351 THV"}="1H35 THV"; $resn_exch{"H352 THV"}="2H35 THV"; $resn_exch{" H6' THV"}=" H6* THV";
$resn_exch{"H2A1 THV"}="1H2A THV"; $resn_exch{"H2A2 THV"}="2H2A THV"; $resn_exch{"H2A3 THV"}="3H2A THV";
$resn_exch{"H4'1 THV"}="1H4* THV"; $resn_exch{"H4'2 THV"}="2H4* THV"; $resn_exch{"H5A1 THW"}="1H5A THW";
$resn_exch{"H5A2 THW"}="2H5A THW"; $resn_exch{"H5B1 THW"}="1H5B THW"; $resn_exch{"H5B2 THW"}="2H5B THW";
$resn_exch{"H4A1 THW"}="1H4A THW"; $resn_exch{"H4A2 THW"}="2H4A THW"; $resn_exch{"H4A3 THW"}="3H4A THW";
$resn_exch{"H351 THW"}="1H35 THW"; $resn_exch{"H352 THW"}="2H35 THW"; $resn_exch{" H6' THW"}=" H6* THW";
$resn_exch{"H2A1 THW"}="1H2A THW"; $resn_exch{"H2A2 THW"}="2H2A THW"; $resn_exch{"H2A3 THW"}="3H2A THW";
$resn_exch{"H4'1 THW"}="1H4* THW"; $resn_exch{"H4'2 THW"}="2H4* THW"; $resn_exch{" H1' THX"}=" H1* THX";
$resn_exch{"H101 THX"}="1H10 THX"; $resn_exch{"H102 THX"}="2H10 THX"; $resn_exch{"H111 THX"}="1H11 THX";
$resn_exch{"H112 THX"}="2H11 THX"; $resn_exch{"H121 THX"}="1H12 THX"; $resn_exch{"H122 THX"}="2H12 THX";
$resn_exch{"H2'1 THX"}="1H2* THX"; $resn_exch{"H2'2 THX"}="2H2* THX"; $resn_exch{"H221 THX"}="1H22 THX";
$resn_exch{"H222 THX"}="2H22 THX"; $resn_exch{"H231 THX"}="1H23 THX"; $resn_exch{"H232 THX"}="2H23 THX";
$resn_exch{" H3' THX"}=" H3* THX"; $resn_exch{"H331 THX"}="1H33 THX"; $resn_exch{"H332 THX"}="2H33 THX";
$resn_exch{"H341 THX"}="1H34 THX"; $resn_exch{"H342 THX"}="2H34 THX"; $resn_exch{" H4' THX"}=" H4* THX";
$resn_exch{"H441 THX"}="1H44 THX"; $resn_exch{"H442 THX"}="2H44 THX"; $resn_exch{"H451 THX"}="1H45 THX";
$resn_exch{"H452 THX"}="2H45 THX"; $resn_exch{"H5'1 THX"}="1H5* THX"; $resn_exch{"H5'2 THX"}="2H5* THX";
$resn_exch{"H5M1 THX"}="1H5M THX"; $resn_exch{"H5M2 THX"}="2H5M THX"; $resn_exch{"H5M3 THX"}="3H5M THX";
$resn_exch{" H71 THX"}="1H7  THX"; $resn_exch{" H72 THX"}="2H7  THX"; $resn_exch{" H81 THX"}="1H8  THX";
$resn_exch{" H82 THX"}="2H8  THX"; $resn_exch{" H91 THX"}="1H9  THX"; $resn_exch{" H92 THX"}="2H9  THX";
$resn_exch{"H151 THX"}="1H15 THX"; $resn_exch{"H152 THX"}="2H15 THX"; $resn_exch{"HOP2 THX"}="2HOP THX";
$resn_exch{"HO'3 THX"}="3HO* THX"; $resn_exch{"H5B1 THY"}="1H5B THY"; $resn_exch{"H5B2 THY"}="2H5B THY";
$resn_exch{"H5A1 THY"}="1H5A THY"; $resn_exch{"H5A2 THY"}="2H5A THY"; $resn_exch{"H4A1 THY"}="1H4A THY";
$resn_exch{"H4A2 THY"}="2H4A THY"; $resn_exch{"H4A3 THY"}="3H4A THY"; $resn_exch{"H111 THY"}="1H11 THY";
$resn_exch{"H112 THY"}="2H11 THY"; $resn_exch{"H113 THY"}="3H11 THY"; $resn_exch{"H101 THY"}="1H10 THY";
$resn_exch{"H102 THY"}="2H10 THY"; $resn_exch{"H121 THY"}="1H12 THY"; $resn_exch{"H122 THY"}="2H12 THY";
$resn_exch{"H123 THY"}="3H12 THY"; $resn_exch{"H351 THY"}="1H35 THY"; $resn_exch{"H352 THY"}="2H35 THY";
$resn_exch{" H6' THY"}=" H6* THY"; $resn_exch{"H2A1 THY"}="1H2A THY"; $resn_exch{"H2A2 THY"}="2H2A THY";
$resn_exch{"H2A3 THY"}="3H2A THY"; $resn_exch{"H4'1 THY"}="1H4* THY"; $resn_exch{"H4'2 THY"}="2H4* THY";
$resn_exch{"HB11 TI1"}="1HB1 TI1"; $resn_exch{"HB12 TI1"}="2HB1 TI1"; $resn_exch{"HG11 TI1"}="1HG1 TI1";
$resn_exch{"HG12 TI1"}="2HG1 TI1"; $resn_exch{"HD11 TI1"}="1HD1 TI1"; $resn_exch{"HD12 TI1"}="2HD1 TI1";
$resn_exch{"HE11 TI1"}="1HE1 TI1"; $resn_exch{"HE12 TI1"}="2HE1 TI1"; $resn_exch{"HF11 TI1"}="1HF1 TI1";
$resn_exch{"HF12 TI1"}="2HF1 TI1"; $resn_exch{"HF13 TI1"}="3HF1 TI1"; $resn_exch{"HB21 TI1"}="1HB2 TI1";
$resn_exch{"HB22 TI1"}="2HB2 TI1"; $resn_exch{"HB31 TI1"}="1HB3 TI1"; $resn_exch{"HB32 TI1"}="2HB3 TI1";
$resn_exch{"HB33 TI1"}="3HB3 TI1"; $resn_exch{"HB11 TI2"}="1HB1 TI2"; $resn_exch{"HB12 TI2"}="2HB1 TI2";
$resn_exch{"HB21 TI2"}="1HB2 TI2"; $resn_exch{"HB22 TI2"}="2HB2 TI2"; $resn_exch{"HB31 TI2"}="1HB3 TI2";
$resn_exch{"HB32 TI2"}="2HB3 TI2"; $resn_exch{"HB11 TI3"}="1HB1 TI3"; $resn_exch{"HB12 TI3"}="2HB1 TI3";
$resn_exch{"HA21 TI3"}="1HA2 TI3"; $resn_exch{"HA22 TI3"}="2HA2 TI3"; $resn_exch{"HB31 TI3"}="1HB3 TI3";
$resn_exch{"HB32 TI3"}="2HB3 TI3"; $resn_exch{"HG31 TI3"}="1HG3 TI3"; $resn_exch{"HG32 TI3"}="2HG3 TI3";
$resn_exch{"HN61 TIA"}="1HN6 TIA"; $resn_exch{"HN62 TIA"}="2HN6 TIA"; $resn_exch{" H1' TIA"}=" H1* TIA";
$resn_exch{" H2' TIA"}=" H2* TIA"; $resn_exch{"HO2' TIA"}="*HO2 TIA"; $resn_exch{" H3' TIA"}=" H3* TIA";
$resn_exch{"HO3' TIA"}="*HO3 TIA"; $resn_exch{" H4' TIA"}=" H4* TIA"; $resn_exch{"H5'1 TIA"}="1H5* TIA";
$resn_exch{"H5'2 TIA"}="2H5* TIA"; $resn_exch{"H5'3 TIA"}="3H5* TIA"; $resn_exch{" H21 TIB"}="1H2  TIB";
$resn_exch{" H22 TIB"}="2H2  TIB"; $resn_exch{" H51 TIB"}="1H5  TIB"; $resn_exch{" H52 TIB"}="2H5  TIB";
$resn_exch{"H141 TIB"}="1H14 TIB"; $resn_exch{"H142 TIB"}="2H14 TIB"; $resn_exch{"H143 TIB"}="3H14 TIB";
$resn_exch{"H151 TIB"}="1H15 TIB"; $resn_exch{"H152 TIB"}="2H15 TIB"; $resn_exch{"H181 TIB"}="1H18 TIB";
$resn_exch{"H182 TIB"}="2H18 TIB"; $resn_exch{"H183 TIB"}="3H18 TIB"; $resn_exch{"H191 TIB"}="1H19 TIB";
$resn_exch{"H192 TIB"}="2H19 TIB"; $resn_exch{"H193 TIB"}="3H19 TIB"; $resn_exch{" H   TIH"}="1HN  TIH";
$resn_exch{" H2  TIH"}="2HN  TIH"; $resn_exch{" HB2 TIH"}="1HB  TIH"; $resn_exch{" HB3 TIH"}="2HB  TIH";
$resn_exch{"HC11 TIN"}="1HC1 TIN"; $resn_exch{"HC12 TIN"}="2HC1 TIN"; $resn_exch{"HC21 TIN"}="1HC2 TIN";
$resn_exch{"HC22 TIN"}="2HC2 TIN"; $resn_exch{"HC41 TIN"}="1HC4 TIN"; $resn_exch{"HC42 TIN"}="2HC4 TIN";
$resn_exch{"HC51 TIN"}="1HC5 TIN"; $resn_exch{"HC52 TIN"}="2HC5 TIN"; $resn_exch{"HC61 TIN"}="1HC6 TIN";
$resn_exch{"HC62 TIN"}="2HC6 TIN"; $resn_exch{"H171 TIN"}="1H17 TIN"; $resn_exch{"H172 TIN"}="2H17 TIN";
$resn_exch{"H191 TIN"}="1H19 TIN"; $resn_exch{"H192 TIN"}="2H19 TIN"; $resn_exch{"H251 TIN"}="1H25 TIN";
$resn_exch{"H252 TIN"}="2H25 TIN"; $resn_exch{"H261 TIN"}="1H26 TIN"; $resn_exch{"H262 TIN"}="2H26 TIN";
$resn_exch{"H291 TIN"}="1H29 TIN"; $resn_exch{"H292 TIN"}="2H29 TIN"; $resn_exch{"H321 TIN"}="1H32 TIN";
$resn_exch{"H322 TIN"}="2H32 TIN"; $resn_exch{"H323 TIN"}="3H32 TIN"; $resn_exch{"H381 TIN"}="1H38 TIN";
$resn_exch{"H382 TIN"}="2H38 TIN"; $resn_exch{"H741 TIN"}="1H74 TIN"; $resn_exch{"H742 TIN"}="2H74 TIN";
$resn_exch{"H761 TIN"}="1H76 TIN"; $resn_exch{"H762 TIN"}="2H76 TIN"; $resn_exch{"H821 TIN"}="1H82 TIN";
$resn_exch{"H822 TIN"}="2H82 TIN"; $resn_exch{"H823 TIN"}="3H82 TIN"; $resn_exch{"HC71 TIN"}="1HC7 TIN";
$resn_exch{"HC72 TIN"}="2HC7 TIN"; $resn_exch{"HC73 TIN"}="3HC7 TIN"; $resn_exch{"HB11 TIO"}="1HB1 TIO";
$resn_exch{"HB12 TIO"}="2HB1 TIO"; $resn_exch{"HA11 TIO"}="1HA1 TIO"; $resn_exch{"HB21 TIO"}="1HB2 TIO";
$resn_exch{"HB22 TIO"}="2HB2 TIO"; $resn_exch{"HA21 TIO"}="1HA2 TIO"; $resn_exch{"HA22 TIO"}="2HA2 TIO";
$resn_exch{"HOH1 TIO"}="1HOH TIO"; $resn_exch{"H211 TIT"}="1H21 TIT"; $resn_exch{"H212 TIT"}="2H21 TIT";
$resn_exch{"H201 TIT"}="1H20 TIT"; $resn_exch{"H202 TIT"}="2H20 TIT"; $resn_exch{"H061 TIT"}="1H06 TIT";
$resn_exch{"H062 TIT"}="2H06 TIT"; $resn_exch{"H011 TIT"}="1H01 TIT"; $resn_exch{"H012 TIT"}="2H01 TIT";
$resn_exch{"H291 TIT"}="1H29 TIT"; $resn_exch{"H292 TIT"}="2H29 TIT"; $resn_exch{"H311 TIT"}="1H31 TIT";
$resn_exch{"H312 TIT"}="2H31 TIT"; $resn_exch{"H313 TIT"}="3H31 TIT"; $resn_exch{"H321 TIT"}="1H32 TIT";
$resn_exch{"H322 TIT"}="2H32 TIT"; $resn_exch{"H323 TIT"}="3H32 TIT"; $resn_exch{"H281 TIT"}="1H28 TIT";
$resn_exch{"H282 TIT"}="2H28 TIT"; $resn_exch{"H283 TIT"}="3H28 TIT"; $resn_exch{"H181 TIT"}="1H18 TIT";
$resn_exch{"H182 TIT"}="2H18 TIT"; $resn_exch{"H183 TIT"}="3H18 TIT"; $resn_exch{"H191 TIT"}="1H19 TIT";
$resn_exch{"H192 TIT"}="2H19 TIT"; $resn_exch{"H193 TIT"}="3H19 TIT"; $resn_exch{"HO2' TIZ"}="HO2* TIZ";
$resn_exch{" H2' TIZ"}=" H2* TIZ"; $resn_exch{" H3' TIZ"}=" H3* TIZ"; $resn_exch{"HO3' TIZ"}="HO3* TIZ";
$resn_exch{" H4' TIZ"}=" H4* TIZ"; $resn_exch{"H5'1 TIZ"}="1H5* TIZ"; $resn_exch{"H5'2 TIZ"}="2H5* TIZ";
$resn_exch{"HO5' TIZ"}="HO5* TIZ"; $resn_exch{" H1' TIZ"}=" H1* TIZ"; $resn_exch{"H1A1 TIZ"}="1H1A TIZ";
$resn_exch{"H1A2 TIZ"}="2H1A TIZ"; $resn_exch{"HXT1 TK4"}="1HXT TK4"; $resn_exch{"H221 TK4"}="1H22 TK4";
$resn_exch{"H222 TK4"}="2H22 TK4"; $resn_exch{"H223 TK4"}="3H22 TK4"; $resn_exch{"HXT2 TK4"}="2HXT TK4";
$resn_exch{" H1  TKL"}="1H   TKL"; $resn_exch{" H2  TKL"}="2H   TKL"; $resn_exch{" HB1 TKL"}="1HB  TKL";
$resn_exch{" HB2 TKL"}="2HB  TKL"; $resn_exch{"HD11 TKL"}="1HD1 TKL"; $resn_exch{"HD12 TKL"}="2HD1 TKL";
$resn_exch{"HD13 TKL"}="3HD1 TKL"; $resn_exch{"HD21 TKL"}="1HD2 TKL"; $resn_exch{"HD22 TKL"}="2HD2 TKL";
$resn_exch{"HD23 TKL"}="3HD2 TKL"; $resn_exch{" HN1 TKL"}="1HN  TKL"; $resn_exch{" HN2 TKL"}="2HN  TKL";
$resn_exch{" HI1 TKL"}="1HI  TKL"; $resn_exch{" HI2 TKL"}="2HI  TKL"; $resn_exch{" HJ1 TKL"}="1HJ  TKL";
$resn_exch{" HJ2 TKL"}="2HJ  TKL"; $resn_exch{" HK1 TKL"}="1HK  TKL"; $resn_exch{" HK2 TKL"}="2HK  TKL";
$resn_exch{"HNZ1 TKL"}="1HNZ TKL"; $resn_exch{"HNZ2 TKL"}="2HNZ TKL"; $resn_exch{"HN21 TL1"}="1HN2 TL1";
$resn_exch{"HN22 TL1"}="2HN2 TL1"; $resn_exch{" H81 TL1"}="1H8  TL1"; $resn_exch{" H82 TL1"}="2H8  TL1";
$resn_exch{"HN41 TL1"}="1HN4 TL1"; $resn_exch{"HN42 TL1"}="2HN4 TL1"; $resn_exch{"H181 TL1"}="1H18 TL1";
$resn_exch{"H182 TL1"}="2H18 TL1"; $resn_exch{"H101 TL2"}="1H10 TL2"; $resn_exch{"H102 TL2"}="2H10 TL2";
$resn_exch{"H171 TL2"}="1H17 TL2"; $resn_exch{"H172 TL2"}="2H17 TL2"; $resn_exch{"H231 TL2"}="1H23 TL2";
$resn_exch{"H232 TL2"}="2H23 TL2"; $resn_exch{"H271 TL2"}="1H27 TL2"; $resn_exch{"H272 TL2"}="2H27 TL2";
$resn_exch{"HN21 TL3"}="1HN2 TL3"; $resn_exch{"HN22 TL3"}="2HN2 TL3"; $resn_exch{" H71 TL3"}="1H7  TL3";
$resn_exch{" H72 TL3"}="2H7  TL3"; $resn_exch{"HN41 TL3"}="1HN4 TL3"; $resn_exch{"HN42 TL3"}="2HN4 TL3";
$resn_exch{"H171 TL3"}="1H17 TL3"; $resn_exch{"H172 TL3"}="2H17 TL3"; $resn_exch{"H101 TL4"}="1H10 TL4";
$resn_exch{"H102 TL4"}="2H10 TL4"; $resn_exch{"H221 TL4"}="1H22 TL4"; $resn_exch{"H222 TL4"}="2H22 TL4";
$resn_exch{"H241 TL4"}="1H24 TL4"; $resn_exch{"H242 TL4"}="2H24 TL4"; $resn_exch{"HN31 TL4"}="1HN3 TL4";
$resn_exch{"HN32 TL4"}="2HN3 TL4"; $resn_exch{" H91 TL5"}="1H9  TL5"; $resn_exch{" H92 TL5"}="2H9  TL5";
$resn_exch{" H93 TL5"}="3H9  TL5"; $resn_exch{"H101 TL5"}="1H10 TL5"; $resn_exch{"H102 TL5"}="2H10 TL5";
$resn_exch{"H103 TL5"}="3H10 TL5"; $resn_exch{"H111 TL5"}="1H11 TL5"; $resn_exch{"H112 TL5"}="2H11 TL5";
$resn_exch{"H113 TL5"}="3H11 TL5"; $resn_exch{"H121 TL5"}="1H12 TL5"; $resn_exch{"H122 TL5"}="2H12 TL5";
$resn_exch{"H123 TL5"}="3H12 TL5"; $resn_exch{"H101 TL6"}="1H10 TL6"; $resn_exch{"H102 TL6"}="2H10 TL6";
$resn_exch{"H103 TL6"}="3H10 TL6"; $resn_exch{" H91 TL6"}="1H9  TL6"; $resn_exch{" H92 TL6"}="2H9  TL6";
$resn_exch{" H93 TL6"}="3H9  TL6"; $resn_exch{" H81 TL6"}="1H8  TL6"; $resn_exch{" H82 TL6"}="2H8  TL6";
$resn_exch{"H5'1 TLB"}="1H5* TLB"; $resn_exch{"H5'2 TLB"}="2H5* TLB"; $resn_exch{" H3' TLB"}=" H3* TLB";
$resn_exch{" H2' TLB"}=" H2* TLB"; $resn_exch{" H1' TLB"}=" H1* TLB"; $resn_exch{"H5M1 TLB"}="1H5M TLB";
$resn_exch{"H5M2 TLB"}="2H5M TLB"; $resn_exch{"H9'1 TLB"}="1H9* TLB"; $resn_exch{"H9'2 TLB"}="2H9* TLB";
$resn_exch{"HOP2 TLC"}="2HOP TLC"; $resn_exch{"HOP3 TLC"}="3HOP TLC"; $resn_exch{" H5' TLC"}="1H5* TLC";
$resn_exch{"H5'' TLC"}="2H5* TLC"; $resn_exch{" H4' TLC"}=" H4* TLC"; $resn_exch{" H1' TLC"}=" H1* TLC";
$resn_exch{" H71 TLC"}="1H5M TLC"; $resn_exch{" H72 TLC"}="2H5M TLC"; $resn_exch{" H73 TLC"}="3H5M TLC";
$resn_exch{" H2' TLC"}=" H2* TLC"; $resn_exch{"H3L1 TLC"}="1H3L TLC"; $resn_exch{"H3L2 TLC"}="2H3L TLC";
$resn_exch{"H2L1 TLC"}="1H2L TLC"; $resn_exch{"H2L2 TLC"}="2H2L TLC"; $resn_exch{"HO3' TLC"}="H3T  TLC";
$resn_exch{"H131 TLD"}="1H13 TLD"; $resn_exch{"H132 TLD"}="2H13 TLD"; $resn_exch{"H133 TLD"}="3H13 TLD";
$resn_exch{" H51 TLM"}="1H5  TLM"; $resn_exch{" H71 TLM"}="1H7  TLM"; $resn_exch{" H81 TLM"}="1H8  TLM";
$resn_exch{" H82 TLM"}="2H8  TLM"; $resn_exch{" H91 TLM"}="1H9  TLM"; $resn_exch{" H92 TLM"}="2H9  TLM";
$resn_exch{" H93 TLM"}="3H9  TLM"; $resn_exch{"H101 TLM"}="1H10 TLM"; $resn_exch{"H102 TLM"}="2H10 TLM";
$resn_exch{"H103 TLM"}="3H10 TLM"; $resn_exch{"H111 TLM"}="1H11 TLM"; $resn_exch{"H112 TLM"}="2H11 TLM";
$resn_exch{"H113 TLM"}="3H11 TLM"; $resn_exch{"HOP2 TLN"}="2HOP TLN"; $resn_exch{"HOP3 TLN"}="3HOP TLN";
$resn_exch{" H5' TLN"}="1H5* TLN"; $resn_exch{"H5'' TLN"}="2H5* TLN"; $resn_exch{" H1' TLN"}=" H1* TLN";
$resn_exch{" H71 TLN"}="1H5M TLN"; $resn_exch{" H72 TLN"}="2H5M TLN"; $resn_exch{" H73 TLN"}="3H5M TLN";
$resn_exch{" H3' TLN"}=" H3* TLN"; $resn_exch{" H2' TLN"}="1H2* TLN"; $resn_exch{"HO3' TLN"}="H3T  TLN";
$resn_exch{"H6'1 TLN"}="1H6* TLN"; $resn_exch{"H6'2 TLN"}="2H6* TLN"; $resn_exch{" H21 TLX"}="1H2  TLX";
$resn_exch{" H22 TLX"}="2H2  TLX"; $resn_exch{" H31 TLX"}="1H3  TLX"; $resn_exch{" H32 TLX"}="2H3  TLX";
$resn_exch{" H41 TLX"}="1H4  TLX"; $resn_exch{" H42 TLX"}="2H4  TLX"; $resn_exch{" H61 TLX"}="1H6  TLX";
$resn_exch{" H62 TLX"}="2H6  TLX"; $resn_exch{" H71 TLX"}="1H7  TLX"; $resn_exch{" H72 TLX"}="2H7  TLX";
$resn_exch{" H81 TLX"}="1H8  TLX"; $resn_exch{" H82 TLX"}="2H8  TLX"; $resn_exch{"H1'1 TLX"}="1H1' TLX";
$resn_exch{"H1'2 TLX"}="2H1' TLX"; $resn_exch{"H3'1 TLX"}="1H3' TLX"; $resn_exch{"H3'2 TLX"}="2H3' TLX";
$resn_exch{"H5'1 TLX"}="1H5' TLX"; $resn_exch{"H5'2 TLX"}="2H5' TLX"; $resn_exch{"H5'3 TLX"}="3H5' TLX";
$resn_exch{"HM'1 TLX"}="1HM' TLX"; $resn_exch{"HM'2 TLX"}="2HM' TLX"; $resn_exch{"HM'3 TLX"}="3HM' TLX";
$resn_exch{"HN'1 TLX"}="1HN' TLX"; $resn_exch{"HN'2 TLX"}="2HN' TLX"; $resn_exch{"H5'1 TM2"}="1H5' TM2";
$resn_exch{"H5'2 TM2"}="2H5' TM2"; $resn_exch{"H7C1 TM2"}="1H7C TM2"; $resn_exch{"H7C2 TM2"}="2H7C TM2";
$resn_exch{"H9C1 TM2"}="1H9C TM2"; $resn_exch{"H9C2 TM2"}="2H9C TM2"; $resn_exch{"H111 TM2"}="1H11 TM2";
$resn_exch{"H112 TM2"}="2H11 TM2"; $resn_exch{"H611 TM5"}="1H61 TM5"; $resn_exch{"H612 TM5"}="2H61 TM5";
$resn_exch{"H621 TM5"}="1H62 TM5"; $resn_exch{"H622 TM5"}="2H62 TM5"; $resn_exch{"H631 TM5"}="1H63 TM5";
$resn_exch{"H632 TM5"}="2H63 TM5"; $resn_exch{"H641 TM5"}="1H64 TM5"; $resn_exch{"H642 TM5"}="2H64 TM5";
$resn_exch{"H651 TM5"}="1H65 TM5"; $resn_exch{"H652 TM5"}="2H65 TM5"; $resn_exch{"H621 TM6"}="1H62 TM6";
$resn_exch{"H622 TM6"}="2H62 TM6"; $resn_exch{"H631 TM6"}="1H63 TM6"; $resn_exch{"H632 TM6"}="2H63 TM6";
$resn_exch{"H641 TM6"}="1H64 TM6"; $resn_exch{"H642 TM6"}="2H64 TM6"; $resn_exch{"H651 TM6"}="1H65 TM6";
$resn_exch{"H652 TM6"}="2H65 TM6"; $resn_exch{"H661 TM6"}="1H66 TM6"; $resn_exch{"H662 TM6"}="2H66 TM6";
$resn_exch{"H671 TM6"}="1H67 TM6"; $resn_exch{"H672 TM6"}="2H67 TM6"; $resn_exch{" H11 TMA"}="1H1  TMA";
$resn_exch{" H12 TMA"}="2H1  TMA"; $resn_exch{" H13 TMA"}="3H1  TMA"; $resn_exch{" H21 TMA"}="1H2  TMA";
$resn_exch{" H22 TMA"}="2H2  TMA"; $resn_exch{" H23 TMA"}="3H2  TMA"; $resn_exch{" H31 TMA"}="1H3  TMA";
$resn_exch{" H32 TMA"}="2H3  TMA"; $resn_exch{" H33 TMA"}="3H3  TMA"; $resn_exch{" H41 TMA"}="1H4  TMA";
$resn_exch{" H42 TMA"}="2H4  TMA"; $resn_exch{" H43 TMA"}="3H4  TMA"; $resn_exch{"HCT1 TMB"}="1HCT TMB";
$resn_exch{"HCT2 TMB"}="2HCT TMB"; $resn_exch{"HCT3 TMB"}="3HCT TMB"; $resn_exch{"HCN1 TMB"}="1HCN TMB";
$resn_exch{"HCN2 TMB"}="2HCN TMB"; $resn_exch{"HCN3 TMB"}="3HCN TMB"; $resn_exch{"HD11 TMB"}="1HD1 TMB";
$resn_exch{"HD12 TMB"}="2HD1 TMB"; $resn_exch{"HD13 TMB"}="3HD1 TMB"; $resn_exch{"HD21 TMB"}="1HD2 TMB";
$resn_exch{"HD22 TMB"}="2HD2 TMB"; $resn_exch{" HH1 TMB"}="1HH  TMB"; $resn_exch{" HH2 TMB"}="2HH  TMB";
$resn_exch{" HH3 TMB"}="3HH  TMB"; $resn_exch{"H142 TMC"}="2H14 TMC"; $resn_exch{"H141 TMC"}="1H14 TMC";
$resn_exch{"H152 TMC"}="2H15 TMC"; $resn_exch{"H151 TMC"}="1H15 TMC"; $resn_exch{"H122 TMC"}="2H12 TMC";
$resn_exch{"H121 TMC"}="1H12 TMC"; $resn_exch{" H33 TMC"}="3H3  TMC"; $resn_exch{" H32 TMC"}="2H3  TMC";
$resn_exch{" H31 TMC"}="1H3  TMC"; $resn_exch{" H   TMD"}=" HN  TMD"; $resn_exch{" HN1 TMD"}="1HN  TMD";
$resn_exch{" HN2 TMD"}="2HN  TMD"; $resn_exch{" HN3 TMD"}="3HN  TMD"; $resn_exch{"HD13 TMD"}="1HD1 TMD";
$resn_exch{"HD12 TMD"}="2HD1 TMD"; $resn_exch{"HD21 TMD"}="1HD2 TMD"; $resn_exch{"HD22 TMD"}="2HD2 TMD";
$resn_exch{"HD23 TMD"}="3HD2 TMD"; $resn_exch{" HE2 TMD"}="1HE  TMD"; $resn_exch{" HE3 TMD"}="2HE  TMD";
$resn_exch{" HZ2 TMD"}="1HZ  TMD"; $resn_exch{" HZ3 TMD"}="2HZ  TMD"; $resn_exch{" HH1 TMD"}="1HH  TMD";
$resn_exch{" HH2 TMD"}="2HH  TMD"; $resn_exch{" HH3 TMD"}="3HH  TMD"; $resn_exch{" HG2 TMD"}=" HO2 TMD";
$resn_exch{" H11 TME"}="1H1  TME"; $resn_exch{" H12 TME"}="2H1  TME"; $resn_exch{" H13 TME"}="3H1  TME";
$resn_exch{" H21 TME"}="1H2  TME"; $resn_exch{" H22 TME"}="2H2  TME"; $resn_exch{" H31 TME"}="1H3  TME";
$resn_exch{" H32 TME"}="2H3  TME"; $resn_exch{" H33 TME"}="3H3  TME"; $resn_exch{"HN21 TMF"}="1HN2 TMF";
$resn_exch{"HN22 TMF"}="2HN2 TMF"; $resn_exch{" H91 TMF"}="1H9  TMF"; $resn_exch{" H92 TMF"}="2H9  TMF";
$resn_exch{" HB1 TMF"}="1HB  TMF"; $resn_exch{" HB2 TMF"}="2HB  TMF"; $resn_exch{" HG1 TMF"}="1HG  TMF";
$resn_exch{" HG2 TMF"}="2HG  TMF"; $resn_exch{"HOE2 TMF"}="2HOE TMF"; $resn_exch{"HCP1 TMF"}="1HCP TMF";
$resn_exch{"HCP2 TMF"}="2HCP TMF"; $resn_exch{" H61 TMH"}="1H6  TMH"; $resn_exch{" H62 TMH"}="2H6  TMH";
$resn_exch{" H41 TMH"}="1H4  TMH"; $resn_exch{" H42 TMH"}="2H4  TMH"; $resn_exch{" H81 TMH"}="1H8  TMH";
$resn_exch{" H82 TMH"}="2H8  TMH"; $resn_exch{" H83 TMH"}="3H8  TMH"; $resn_exch{" H91 TMH"}="1H9  TMH";
$resn_exch{" H92 TMH"}="2H9  TMH"; $resn_exch{" H93 TMH"}="3H9  TMH"; $resn_exch{"H101 TMH"}="1H10 TMH";
$resn_exch{"H102 TMH"}="2H10 TMH"; $resn_exch{"H103 TMH"}="3H10 TMH"; $resn_exch{"H11A TMJ"}="1H1  TMJ";
$resn_exch{"H12A TMJ"}="2H1  TMJ"; $resn_exch{" H21 TMJ"}="1H2  TMJ"; $resn_exch{" H22 TMJ"}="2H2  TMJ";
$resn_exch{" H31 TMJ"}="1H3  TMJ"; $resn_exch{" H32 TMJ"}="2H3  TMJ"; $resn_exch{"HM11 TML"}="1HM1 TML";
$resn_exch{"HM12 TML"}="2HM1 TML"; $resn_exch{"HM13 TML"}="3HM1 TML"; $resn_exch{"HM21 TML"}="1HM2 TML";
$resn_exch{"HM22 TML"}="2HM2 TML"; $resn_exch{"HM23 TML"}="3HM2 TML"; $resn_exch{"HM31 TML"}="1HM3 TML";
$resn_exch{"HM32 TML"}="2HM3 TML"; $resn_exch{"HM33 TML"}="3HM3 TML"; $resn_exch{" H11 TMN"}="1H1  TMN";
$resn_exch{" H12 TMN"}="2H1  TMN"; $resn_exch{" H21 TMN"}="1H2  TMN"; $resn_exch{" H22 TMN"}="2H2  TMN";
$resn_exch{" H31 TMN"}="1H3  TMN"; $resn_exch{" H32 TMN"}="2H3  TMN"; $resn_exch{" HN1 TMN"}="1HN  TMN";
$resn_exch{" HN2 TMN"}="2HN  TMN"; $resn_exch{"HOP2 TMP"}="2HOP TMP"; $resn_exch{"HOP3 TMP"}="3HOP TMP";
$resn_exch{"H5'1 TMP"}="1H5* TMP"; $resn_exch{"H5'2 TMP"}="2H5* TMP"; $resn_exch{" H4' TMP"}=" H4* TMP";
$resn_exch{" H3' TMP"}=" H3* TMP"; $resn_exch{"HO3' TMP"}="*HO3 TMP"; $resn_exch{"H2'1 TMP"}="1H2* TMP";
$resn_exch{"H2'2 TMP"}="2H2* TMP"; $resn_exch{" H1' TMP"}=" H1* TMP"; $resn_exch{" H51 TMP"}="1H5  TMP";
$resn_exch{" H52 TMP"}="2H5  TMP"; $resn_exch{" H53 TMP"}="3H5  TMP"; $resn_exch{" H91 TMQ"}="1H9  TMQ";
$resn_exch{" H92 TMQ"}="2H9  TMQ"; $resn_exch{"H171 TMQ"}="1H17 TMQ"; $resn_exch{"H172 TMQ"}="2H17 TMQ";
$resn_exch{"H173 TMQ"}="3H17 TMQ"; $resn_exch{"H211 TMQ"}="1H21 TMQ"; $resn_exch{"H212 TMQ"}="2H21 TMQ";
$resn_exch{"H213 TMQ"}="3H21 TMQ"; $resn_exch{"H221 TMQ"}="1H22 TMQ"; $resn_exch{"H222 TMQ"}="2H22 TMQ";
$resn_exch{"H223 TMQ"}="3H22 TMQ"; $resn_exch{"H231 TMQ"}="1H23 TMQ"; $resn_exch{"H232 TMQ"}="2H23 TMQ";
$resn_exch{"H233 TMQ"}="3H23 TMQ"; $resn_exch{"HN21 TMQ"}="1HN2 TMQ"; $resn_exch{"HN22 TMQ"}="2HN2 TMQ";
$resn_exch{"HN51 TMQ"}="1HN5 TMQ"; $resn_exch{"HN52 TMQ"}="2HN5 TMQ"; $resn_exch{" H21 TMR"}="1H2  TMR";
$resn_exch{" H22 TMR"}="2H2  TMR"; $resn_exch{" H61 TMR"}="1H6  TMR"; $resn_exch{" H62 TMR"}="2H6  TMR";
$resn_exch{" H63 TMR"}="3H6  TMR"; $resn_exch{" H71 TMR"}="1H7  TMR"; $resn_exch{" H72 TMR"}="2H7  TMR";
$resn_exch{" H73 TMR"}="3H7  TMR"; $resn_exch{" H61 TMT"}="1H6  TMT"; $resn_exch{" H62 TMT"}="2H6  TMT";
$resn_exch{" H63 TMT"}="3H6  TMT"; $resn_exch{" H71 TMT"}="1H7  TMT"; $resn_exch{" H72 TMT"}="2H7  TMT";
$resn_exch{" H73 TMT"}="3H7  TMT"; $resn_exch{" H81 TMT"}="1H8  TMT"; $resn_exch{" H82 TMT"}="2H8  TMT";
$resn_exch{" H83 TMT"}="3H8  TMT"; $resn_exch{"H131 TMU"}="1H13 TMU"; $resn_exch{"H132 TMU"}="2H13 TMU";
$resn_exch{"H211 TMU"}="1H21 TMU"; $resn_exch{"H212 TMU"}="2H21 TMU"; $resn_exch{"H213 TMU"}="3H21 TMU";
$resn_exch{" H61 TMZ"}="1H6  TMZ"; $resn_exch{" H62 TMZ"}="2H6  TMZ"; $resn_exch{" H63 TMZ"}="3H6  TMZ";
$resn_exch{" H71 TMZ"}="1H7  TMZ"; $resn_exch{" H72 TMZ"}="2H7  TMZ"; $resn_exch{" H73 TMZ"}="3H7  TMZ";
$resn_exch{" H81 TMZ"}="1H8  TMZ"; $resn_exch{" H82 TMZ"}="2H8  TMZ"; $resn_exch{" H83 TMZ"}="3H8  TMZ";
$resn_exch{"H121 TN3"}="1H12 TN3"; $resn_exch{"H122 TN3"}="2H12 TN3"; $resn_exch{"H123 TN3"}="3H12 TN3";
$resn_exch{" H51 TN4"}="1H5  TN4"; $resn_exch{" H52 TN4"}="2H5  TN4"; $resn_exch{" H61 TN4"}="1H6  TN4";
$resn_exch{" H62 TN4"}="2H6  TN4"; $resn_exch{"H111 TN4"}="1H11 TN4"; $resn_exch{"H112 TN4"}="2H11 TN4";
$resn_exch{"H311 TN4"}="1H31 TN4"; $resn_exch{"H312 TN4"}="2H31 TN4"; $resn_exch{"H313 TN4"}="3H31 TN4";
$resn_exch{"H231 TN4"}="1H23 TN4"; $resn_exch{"H232 TN4"}="2H23 TN4"; $resn_exch{"H2B1 TN4"}="1H2B TN4";
$resn_exch{"H2B2 TN4"}="2H2B TN4"; $resn_exch{"H2B3 TN4"}="3H2B TN4"; $resn_exch{"H2C1 TN4"}="1H2C TN4";
$resn_exch{"H2C2 TN4"}="2H2C TN4"; $resn_exch{"H2C3 TN4"}="3H2C TN4"; $resn_exch{"H2A1 TN4"}="1H2A TN4";
$resn_exch{"H2A2 TN4"}="2H2A TN4"; $resn_exch{"H2A3 TN4"}="3H2A TN4"; $resn_exch{"H441 TN4"}="1H44 TN4";
$resn_exch{"H442 TN4"}="2H44 TN4"; $resn_exch{"HN11 TN5"}="1HN1 TN5"; $resn_exch{"HN12 TN5"}="2HN1 TN5";
$resn_exch{" H   TNB"}="1HN  TNB"; $resn_exch{" H2  TNB"}="2HN  TNB"; $resn_exch{" HB2 TNB"}="1HB  TNB";
$resn_exch{" HB3 TNB"}="2HB  TNB"; $resn_exch{"H111 TNC"}="1H11 TNC"; $resn_exch{"H112 TNC"}="2H11 TNC";
$resn_exch{"HN21 TNC"}="1HN2 TNC"; $resn_exch{"HN22 TNC"}="2HN2 TNC"; $resn_exch{"H201 TNC"}="1H20 TNC";
$resn_exch{"H202 TNC"}="2H20 TNC"; $resn_exch{"H203 TNC"}="3H20 TNC"; $resn_exch{"H211 TNC"}="1H21 TNC";
$resn_exch{"H212 TNC"}="2H21 TNC"; $resn_exch{"H213 TNC"}="3H21 TNC"; $resn_exch{"H5'1 TND"}="1H5* TND";
$resn_exch{"H5'2 TND"}="2H5* TND"; $resn_exch{" H4' TND"}=" H4* TND"; $resn_exch{" H3' TND"}=" H3* TND";
$resn_exch{"HO3' TND"}="*HO3 TND"; $resn_exch{" H2' TND"}=" H2* TND"; $resn_exch{"HN2' TND"}="*HN2 TND";
$resn_exch{" H1' TND"}=" H1* TND"; $resn_exch{" H21 TND"}="1H2  TND"; $resn_exch{" H22 TND"}="2H2  TND";
$resn_exch{" H31 TND"}="1H3  TND"; $resn_exch{" H32 TND"}="2H3  TND"; $resn_exch{" H41 TND"}="1H4  TND";
$resn_exch{" H42 TND"}="2H4  TND"; $resn_exch{"H2M1 TND"}="1H2M TND"; $resn_exch{"H2M2 TND"}="2H2M TND";
$resn_exch{"H2M3 TND"}="3H2M TND"; $resn_exch{"H1M1 TND"}="1H1M TND"; $resn_exch{"H1M2 TND"}="2H1M TND";
$resn_exch{"H1M3 TND"}="3H1M TND"; $resn_exch{"HO5' TND"}="*HO5 TND"; $resn_exch{" H21 TNE"}="1H2  TNE";
$resn_exch{" H22 TNE"}="2H2  TNE"; $resn_exch{" H41 TNE"}="1H4  TNE"; $resn_exch{" H42 TNE"}="2H4  TNE";
$resn_exch{" H61 TNE"}="1H6  TNE"; $resn_exch{" H62 TNE"}="2H6  TNE"; $resn_exch{" H71 TNE"}="1H7  TNE";
$resn_exch{" H72 TNE"}="2H7  TNE"; $resn_exch{" H91 TNE"}="1H9  TNE"; $resn_exch{" H92 TNE"}="2H9  TNE";
$resn_exch{" H93 TNE"}="3H9  TNE"; $resn_exch{"H131 TNK"}="1H13 TNK"; $resn_exch{"H132 TNK"}="2H13 TNK";
$resn_exch{"H151 TNK"}="1H15 TNK"; $resn_exch{"H152 TNK"}="2H15 TNK"; $resn_exch{"H153 TNK"}="3H15 TNK";
$resn_exch{"H161 TNK"}="1H16 TNK"; $resn_exch{"H162 TNK"}="2H16 TNK"; $resn_exch{"H163 TNK"}="3H16 TNK";
$resn_exch{"H171 TNK"}="1H17 TNK"; $resn_exch{"H172 TNK"}="2H17 TNK"; $resn_exch{"H181 TNK"}="1H18 TNK";
$resn_exch{"H182 TNK"}="2H18 TNK"; $resn_exch{"H7C1 TNL"}="1H7C TNL"; $resn_exch{"H7C2 TNL"}="2H7C TNL";
$resn_exch{"H7C3 TNL"}="3H7C TNL"; $resn_exch{"H151 TNP"}="1H15 TNP"; $resn_exch{"H152 TNP"}="2H15 TNP";
$resn_exch{"H153 TNP"}="3H15 TNP"; $resn_exch{"H161 TNP"}="1H16 TNP"; $resn_exch{"H162 TNP"}="2H16 TNP";
$resn_exch{"H163 TNP"}="3H16 TNP"; $resn_exch{"H171 TNP"}="1H17 TNP"; $resn_exch{"H172 TNP"}="2H17 TNP";
$resn_exch{"H173 TNP"}="3H17 TNP"; $resn_exch{"H181 TNP"}="1H18 TNP"; $resn_exch{"H182 TNP"}="2H18 TNP";
$resn_exch{"H183 TNP"}="3H18 TNP"; $resn_exch{"H191 TNP"}="1H19 TNP"; $resn_exch{"H192 TNP"}="2H19 TNP";
$resn_exch{"H193 TNP"}="3H19 TNP"; $resn_exch{" H   TNR"}="1HN  TNR"; $resn_exch{" H2  TNR"}="2HN  TNR";
$resn_exch{" HB2 TNR"}="1HCB TNR"; $resn_exch{" HB3 TNR"}="2HCB TNR"; $resn_exch{"HC81 TNR"}="1HC8 TNR";
$resn_exch{"HC82 TNR"}="2HC8 TNR"; $resn_exch{"HC83 TNR"}="3HC8 TNR"; $resn_exch{"HC61 TNR"}="1HC6 TNR";
$resn_exch{"HC62 TNR"}="2HC6 TNR"; $resn_exch{"H191 TNS"}="1H19 TNS"; $resn_exch{"H192 TNS"}="2H19 TNS";
$resn_exch{"H181 TNS"}="1H18 TNS"; $resn_exch{"H182 TNS"}="2H18 TNS"; $resn_exch{"H171 TNS"}="1H17 TNS";
$resn_exch{"H172 TNS"}="2H17 TNS"; $resn_exch{"H161 TNS"}="1H16 TNS"; $resn_exch{"H162 TNS"}="2H16 TNS";
$resn_exch{"H231 TNS"}="1H23 TNS"; $resn_exch{"H232 TNS"}="2H23 TNS"; $resn_exch{"H233 TNS"}="3H23 TNS";
$resn_exch{" HN1 TNT"}="1HN  TNT"; $resn_exch{" HN2 TNT"}="2HN  TNT"; $resn_exch{" HN3 TNT"}="3HN  TNT";
$resn_exch{"HN'1 TNT"}="1HN' TNT"; $resn_exch{"HN'2 TNT"}="2HN' TNT"; $resn_exch{"HN'3 TNT"}="3HN' TNT";
$resn_exch{" HA1 TNT"}="1HA  TNT"; $resn_exch{" HA2 TNT"}="2HA  TNT"; $resn_exch{" HB1 TNT"}="1HB  TNT";
$resn_exch{" HB2 TNT"}="2HB  TNT"; $resn_exch{"HA'1 TNT"}="1HA' TNT"; $resn_exch{"HA'2 TNT"}="2HA' TNT";
$resn_exch{"H9'1 TNV"}="1H9' TNV"; $resn_exch{"H9'2 TNV"}="2H9' TNV"; $resn_exch{"H8'1 TNV"}="1H8' TNV";
$resn_exch{"H8'2 TNV"}="2H8' TNV"; $resn_exch{"H8'3 TNV"}="3H8' TNV"; $resn_exch{"H6'1 TNV"}="1H6' TNV";
$resn_exch{"H6'2 TNV"}="2H6' TNV"; $resn_exch{"HN61 TNV"}="1HN6 TNV"; $resn_exch{"HN62 TNV"}="2HN6 TNV";
$resn_exch{" H61 TOA"}="1H6  TOA"; $resn_exch{" H62 TOA"}="2H6  TOA"; $resn_exch{"HN31 TOA"}="1HN3 TOA";
$resn_exch{"HN32 TOA"}="2HN3 TOA"; $resn_exch{"HN33 TOA"}="3HN3 TOA"; $resn_exch{" H21 TOB"}="1H2  TOB";
$resn_exch{" H22 TOB"}="2H2  TOB"; $resn_exch{" H41 TOB"}="1H4  TOB"; $resn_exch{" H42 TOB"}="2H4  TOB";
$resn_exch{"HN11 TOB"}="1HN1 TOB"; $resn_exch{"HN12 TOB"}="2HN1 TOB"; $resn_exch{"HN13 TOB"}="3HN1 TOB";
$resn_exch{"HN31 TOB"}="1HN3 TOB"; $resn_exch{"HN32 TOB"}="2HN3 TOB"; $resn_exch{"HN33 TOB"}="3HN3 TOB";
$resn_exch{" H31 TOC"}="1H3  TOC"; $resn_exch{" H32 TOC"}="2H3  TOC"; $resn_exch{" H61 TOC"}="1H6  TOC";
$resn_exch{" H62 TOC"}="2H6  TOC"; $resn_exch{"HN21 TOC"}="1HN2 TOC"; $resn_exch{"HN22 TOC"}="2HN2 TOC";
$resn_exch{"HN23 TOC"}="3HN2 TOC"; $resn_exch{"HN61 TOC"}="1HN6 TOC"; $resn_exch{"HN62 TOC"}="2HN6 TOC";
$resn_exch{"HN63 TOC"}="3HN6 TOC"; $resn_exch{"HA'1 TOE"}="1HA* TOE"; $resn_exch{"HA'2 TOE"}="2HA* TOE";
$resn_exch{"HB'1 TOE"}="1HB* TOE"; $resn_exch{"HB'2 TOE"}="2HB* TOE"; $resn_exch{"HD'1 TOE"}="1HD* TOE";
$resn_exch{"HD'2 TOE"}="2HD* TOE"; $resn_exch{"HE'1 TOE"}="1HE* TOE"; $resn_exch{"HE'2 TOE"}="2HE* TOE";
$resn_exch{"HG'1 TOE"}="1HG* TOE"; $resn_exch{"HG'2 TOE"}="2HG* TOE"; $resn_exch{"HH'1 TOE"}="1HH* TOE";
$resn_exch{"HH'2 TOE"}="2HH* TOE"; $resn_exch{"HK'1 TOE"}="1HK* TOE"; $resn_exch{"HK'2 TOE"}="2HK* TOE";
$resn_exch{"HK'3 TOE"}="3HK* TOE"; $resn_exch{" H81 TOH"}="1H8  TOH"; $resn_exch{" H71 TOH"}="1H7  TOH";
$resn_exch{" H41 TOL"}="1H4  TOL"; $resn_exch{" H42 TOL"}="2H4  TOL"; $resn_exch{" H43 TOL"}="3H4  TOL";
$resn_exch{"H141 TOL"}="1H14 TOL"; $resn_exch{"H142 TOL"}="2H14 TOL"; $resn_exch{"H143 TOL"}="3H14 TOL";
$resn_exch{"H151 TOL"}="1H15 TOL"; $resn_exch{"H152 TOL"}="2H15 TOL"; $resn_exch{"H111 TOM"}="1H11 TOM";
$resn_exch{"H112 TOM"}="2H11 TOM"; $resn_exch{"H113 TOM"}="3H11 TOM"; $resn_exch{" H11 TON"}="1H1  TON";
$resn_exch{" H12 TON"}="2H1  TON"; $resn_exch{" H21 TON"}="1H2  TON"; $resn_exch{" H22 TON"}="2H2  TON";
$resn_exch{" H31 TON"}="1H3  TON"; $resn_exch{" H32 TON"}="2H3  TON"; $resn_exch{" H41 TON"}="1H4  TON";
$resn_exch{" H42 TON"}="2H4  TON"; $resn_exch{"H162 TON"}="2H16 TON"; $resn_exch{" H81 TON"}="1H8  TON";
$resn_exch{" H82 TON"}="2H8  TON"; $resn_exch{" H83 TON"}="3H8  TON"; $resn_exch{" H71 TON"}="1H7  TON";
$resn_exch{" H72 TON"}="2H7  TON"; $resn_exch{" H73 TON"}="3H7  TON"; $resn_exch{" H61 TON"}="1H6  TON";
$resn_exch{" H62 TON"}="2H6  TON"; $resn_exch{"H121 TON"}="1H12 TON"; $resn_exch{"H122 TON"}="2H12 TON";
$resn_exch{"H123 TON"}="3H12 TON"; $resn_exch{"H111 TON"}="1H11 TON"; $resn_exch{"H112 TON"}="2H11 TON";
$resn_exch{"H113 TON"}="3H11 TON"; $resn_exch{"H101 TON"}="1H10 TON"; $resn_exch{"H102 TON"}="2H10 TON";
$resn_exch{"H103 TON"}="3H10 TON"; $resn_exch{"HN41 TOP"}="1HN4 TOP"; $resn_exch{"HN42 TOP"}="2HN4 TOP";
$resn_exch{"HN71 TOP"}="1HN7 TOP"; $resn_exch{"HN72 TOP"}="2HN7 TOP"; $resn_exch{" H91 TOP"}="1H9  TOP";
$resn_exch{" H92 TOP"}="2H9  TOP"; $resn_exch{"H141 TOP"}="1H14 TOP"; $resn_exch{"H142 TOP"}="2H14 TOP";
$resn_exch{"H143 TOP"}="3H14 TOP"; $resn_exch{"H171 TOP"}="1H17 TOP"; $resn_exch{"H172 TOP"}="2H17 TOP";
$resn_exch{"H173 TOP"}="3H17 TOP"; $resn_exch{"H201 TOP"}="1H20 TOP"; $resn_exch{"H202 TOP"}="2H20 TOP";
$resn_exch{"H203 TOP"}="3H20 TOP"; $resn_exch{"HOS2 TOS"}="2HOS TOS"; $resn_exch{" H1  TOS"}="1H   TOS";
$resn_exch{" H2A TOS"}="2H   TOS"; $resn_exch{" H3A TOS"}="3H   TOS"; $resn_exch{" H61 TOT"}="1H6  TOT";
$resn_exch{" H62 TOT"}="2H6  TOT"; $resn_exch{" H63 TOT"}="3H6  TOT"; $resn_exch{"H221 TOT"}="1H22 TOT";
$resn_exch{"H222 TOT"}="2H22 TOT"; $resn_exch{"H231 TOT"}="1H23 TOT"; $resn_exch{"H232 TOT"}="2H23 TOT";
$resn_exch{"H241 TOT"}="1H24 TOT"; $resn_exch{"H242 TOT"}="2H24 TOT"; $resn_exch{"H261 TOT"}="1H26 TOT";
$resn_exch{"H262 TOT"}="2H26 TOT"; $resn_exch{"H271 TOT"}="1H27 TOT"; $resn_exch{"H272 TOT"}="2H27 TOT";
$resn_exch{"H281 TOT"}="1H28 TOT"; $resn_exch{"H282 TOT"}="2H28 TOT"; $resn_exch{"H283 TOT"}="3H28 TOT";
$resn_exch{"H291 TOT"}="1H29 TOT"; $resn_exch{"H292 TOT"}="2H29 TOT"; $resn_exch{"H293 TOT"}="3H29 TOT";
$resn_exch{"H361 TOT"}="1H36 TOT"; $resn_exch{"H362 TOT"}="2H36 TOT"; $resn_exch{"H363 TOT"}="3H36 TOT";
$resn_exch{"H521 TOT"}="1H52 TOT"; $resn_exch{"H522 TOT"}="2H52 TOT"; $resn_exch{"H531 TOT"}="1H53 TOT";
$resn_exch{"H532 TOT"}="2H53 TOT"; $resn_exch{"H541 TOT"}="1H54 TOT"; $resn_exch{"H542 TOT"}="2H54 TOT";
$resn_exch{"H561 TOT"}="1H56 TOT"; $resn_exch{"H562 TOT"}="2H56 TOT"; $resn_exch{"H581 TOT"}="1H58 TOT";
$resn_exch{"H582 TOT"}="2H58 TOT"; $resn_exch{"H583 TOT"}="3H58 TOT"; $resn_exch{"H591 TOT"}="1H59 TOT";
$resn_exch{"H592 TOT"}="2H59 TOT"; $resn_exch{"H593 TOT"}="3H59 TOT"; $resn_exch{"HN11 TOU"}="1HN1 TOU";
$resn_exch{"HN12 TOU"}="2HN1 TOU"; $resn_exch{"HN21 TOU"}="1HN2 TOU"; $resn_exch{"HN22 TOU"}="2HN2 TOU";
$resn_exch{" H   TOX"}="1HN  TOX"; $resn_exch{" H2  TOX"}="2HN  TOX"; $resn_exch{" HC1 TOX"}="1H   TOX";
$resn_exch{" HC2 TOX"}="2H   TOX"; $resn_exch{" HB2 TOX"}="1HB  TOX"; $resn_exch{" HB3 TOX"}="2HB  TOX";
$resn_exch{"HN21 TOY"}="1HN2 TOY"; $resn_exch{"HN22 TOY"}="2HN2 TOY"; $resn_exch{"H311 TOY"}="1H31 TOY";
$resn_exch{"H312 TOY"}="2H31 TOY"; $resn_exch{"H41O TOY"}="OH41 TOY"; $resn_exch{"H611 TOY"}="1H61 TOY";
$resn_exch{"H612 TOY"}="2H61 TOY"; $resn_exch{"HN61 TOY"}="1HN6 TOY"; $resn_exch{"HN62 TOY"}="2HN6 TOY";
$resn_exch{"HN11 TOY"}="1HN1 TOY"; $resn_exch{"HN12 TOY"}="2HN1 TOY"; $resn_exch{"H221 TOY"}="1H22 TOY";
$resn_exch{"H222 TOY"}="2H22 TOY"; $resn_exch{" HN1 TOY"}="1HN  TOY"; $resn_exch{" HN2 TOY"}="2HN  TOY";
$resn_exch{"H52O TOY"}="OH52 TOY"; $resn_exch{"H23O TOY"}="OH23 TOY"; $resn_exch{"HN31 TOY"}="1HN3 TOY";
$resn_exch{"HN32 TOY"}="2HN3 TOY"; $resn_exch{"H43O TOY"}="OH43 TOY"; $resn_exch{"H631 TOY"}="1H63 TOY";
$resn_exch{"H632 TOY"}="2H63 TOY"; $resn_exch{"H63O TOY"}="OH63 TOY"; $resn_exch{"H8'1 TP1"}="1H8' TP1";
$resn_exch{"H8'2 TP1"}="2H8' TP1"; $resn_exch{"H5'1 TP1"}="1H5' TP1"; $resn_exch{"H5'2 TP1"}="2H5' TP1";
$resn_exch{"H3'1 TP1"}="1H3' TP1"; $resn_exch{"H3'2 TP1"}="2H3' TP1"; $resn_exch{"H2'1 TP1"}="1H2' TP1";
$resn_exch{"H2'2 TP1"}="2H2' TP1"; $resn_exch{"H1'1 TP1"}="1H1' TP1"; $resn_exch{"H1'2 TP1"}="2H1' TP1";
$resn_exch{"H1'3 TP1"}="3H1' TP1"; $resn_exch{"HM51 TP1"}="1HM5 TP1"; $resn_exch{"HM52 TP1"}="2HM5 TP1";
$resn_exch{"HM53 TP1"}="3HM5 TP1"; $resn_exch{"HC21 TP2"}="1HC2 TP2"; $resn_exch{"HC22 TP2"}="2HC2 TP2";
$resn_exch{"HC31 TP2"}="1HC3 TP2"; $resn_exch{"HC32 TP2"}="2HC3 TP2"; $resn_exch{"HN41 TP2"}="1HN4 TP2";
$resn_exch{"HC61 TP2"}="1HC6 TP2"; $resn_exch{"H101 TP2"}="1H10 TP2"; $resn_exch{"H111 TP2"}="1H11 TP2";
$resn_exch{"H131 TP2"}="1H13 TP2"; $resn_exch{"H141 TP2"}="1H14 TP2"; $resn_exch{"H151 TP2"}="1H15 TP2";
$resn_exch{"H152 TP2"}="2H15 TP2"; $resn_exch{"H153 TP2"}="3H15 TP2"; $resn_exch{"H181 TP2"}="1H18 TP2";
$resn_exch{"H182 TP2"}="2H18 TP2"; $resn_exch{"H191 TP2"}="1H19 TP2"; $resn_exch{"H192 TP2"}="2H19 TP2";
$resn_exch{"H201 TP2"}="1H20 TP2"; $resn_exch{"H202 TP2"}="2H20 TP2"; $resn_exch{"HC11 TP3"}="1HC1 TP3";
$resn_exch{"HC12 TP3"}="2HC1 TP3"; $resn_exch{"HC21 TP3"}="1HC2 TP3"; $resn_exch{"HC22 TP3"}="2HC2 TP3";
$resn_exch{"HC41 TP3"}="1HC4 TP3"; $resn_exch{"HC51 TP3"}="1HC5 TP3"; $resn_exch{"HC52 TP3"}="2HC5 TP3";
$resn_exch{"HO81 TP3"}="1HO8 TP3"; $resn_exch{"H131 TP3"}="1H13 TP3"; $resn_exch{"H141 TP3"}="1H14 TP3";
$resn_exch{"H161 TP3"}="1H16 TP3"; $resn_exch{"H171 TP3"}="1H17 TP3"; $resn_exch{"H191 TP3"}="1H19 TP3";
$resn_exch{"H201 TP3"}="1H20 TP3"; $resn_exch{"H211 TP3"}="1H21 TP3"; $resn_exch{"H212 TP3"}="2H21 TP3";
$resn_exch{"H221 TP3"}="1H22 TP3"; $resn_exch{"H222 TP3"}="2H22 TP3"; $resn_exch{"H271 TP3"}="1H27 TP3";
$resn_exch{"H281 TP3"}="1H28 TP3"; $resn_exch{"HC11 TP4"}="1HC1 TP4"; $resn_exch{"HC12 TP4"}="2HC1 TP4";
$resn_exch{"HC21 TP4"}="1HC2 TP4"; $resn_exch{"HC22 TP4"}="2HC2 TP4"; $resn_exch{"HC41 TP4"}="1HC4 TP4";
$resn_exch{"HC51 TP4"}="1HC5 TP4"; $resn_exch{"HC52 TP4"}="2HC5 TP4"; $resn_exch{"HN81 TP4"}="1HN8 TP4";
$resn_exch{"H131 TP4"}="1H13 TP4"; $resn_exch{"H141 TP4"}="1H14 TP4"; $resn_exch{"H161 TP4"}="1H16 TP4";
$resn_exch{"H171 TP4"}="1H17 TP4"; $resn_exch{"H191 TP4"}="1H19 TP4"; $resn_exch{"H201 TP4"}="1H20 TP4";
$resn_exch{"H211 TP4"}="1H21 TP4"; $resn_exch{"H212 TP4"}="2H21 TP4"; $resn_exch{"H221 TP4"}="1H22 TP4";
$resn_exch{"H222 TP4"}="2H22 TP4"; $resn_exch{"H271 TP4"}="1H27 TP4"; $resn_exch{"H281 TP4"}="1H28 TP4";
$resn_exch{"H301 TP4"}="1H30 TP4"; $resn_exch{"H302 TP4"}="2H30 TP4"; $resn_exch{"H311 TP4"}="1H31 TP4";
$resn_exch{"H312 TP4"}="2H31 TP4"; $resn_exch{"H331 TP4"}="1H33 TP4"; $resn_exch{"HN21 TP5"}="1HN2 TP5";
$resn_exch{"HN22 TP5"}="2HN2 TP5"; $resn_exch{"H191 TP6"}="1H19 TP6"; $resn_exch{"H192 TP6"}="2H19 TP6";
$resn_exch{"H181 TP6"}="1H18 TP6"; $resn_exch{"H182 TP6"}="2H18 TP6"; $resn_exch{"H171 TP6"}="1H17 TP6";
$resn_exch{"H172 TP6"}="2H17 TP6"; $resn_exch{"H161 TP6"}="1H16 TP6"; $resn_exch{"H162 TP6"}="2H16 TP6";
$resn_exch{"H151 TP6"}="1H15 TP6"; $resn_exch{"H152 TP6"}="2H15 TP6"; $resn_exch{"H101 TP6"}="1H10 TP6";
$resn_exch{"H102 TP6"}="2H10 TP6"; $resn_exch{"H141 TP6"}="1H14 TP6"; $resn_exch{"H142 TP6"}="2H14 TP6";
$resn_exch{"HOP2 TP7"}="2HOP TP7"; $resn_exch{"HOP3 TP7"}="3HOP TP7"; $resn_exch{" HG1 TP7"}="1HG  TP7";
$resn_exch{" HG2 TP7"}="2HG  TP7"; $resn_exch{" HG3 TP7"}="3HG  TP7"; $resn_exch{" H21 TP7"}="1H2  TP7";
$resn_exch{" H22 TP7"}="2H2  TP7"; $resn_exch{" H31 TP7"}="1H3  TP7"; $resn_exch{" H32 TP7"}="2H3  TP7";
$resn_exch{" H41 TP7"}="1H4  TP7"; $resn_exch{" H42 TP7"}="2H4  TP7"; $resn_exch{" H51 TP7"}="1H5  TP7";
$resn_exch{" H52 TP7"}="2H5  TP7"; $resn_exch{" H61 TP7"}="1H6  TP7"; $resn_exch{" H62 TP7"}="2H6  TP7";
$resn_exch{" H71 TP7"}="1H7  TP7"; $resn_exch{" H72 TP7"}="2H7  TP7"; $resn_exch{"H101 TP8"}="1H10 TP8";
$resn_exch{"H102 TP8"}="2H10 TP8"; $resn_exch{" H61 TP8"}="1H6  TP8"; $resn_exch{" H62 TP8"}="2H6  TP8";
$resn_exch{" H71 TP8"}="1H7  TP8"; $resn_exch{" H72 TP8"}="2H7  TP8"; $resn_exch{"HM41 TP8"}="1HM4 TP8";
$resn_exch{"HM42 TP8"}="2HM4 TP8"; $resn_exch{"HM43 TP8"}="3HM4 TP8"; $resn_exch{"H7,1 TP8"}="1H7, TP8";
$resn_exch{"H7,2 TP8"}="2H7, TP8"; $resn_exch{"HM21 TP8"}="1HM2 TP8"; $resn_exch{"HM22 TP8"}="2HM2 TP8";
$resn_exch{"HM23 TP8"}="3HM2 TP8"; $resn_exch{" H71 TP9"}="1H7  TP9"; $resn_exch{" H72 TP9"}="2H7  TP9";
$resn_exch{" H61 TP9"}="1H6  TP9"; $resn_exch{" H62 TP9"}="2H6  TP9"; $resn_exch{"H7'1 TP9"}="1H7' TP9";
$resn_exch{"H7'2 TP9"}="2H7' TP9"; $resn_exch{"H4'1 TP9"}="1H4' TP9"; $resn_exch{"H4'2 TP9"}="2H4' TP9";
$resn_exch{"HM21 TP9"}="1HM2 TP9"; $resn_exch{"HM22 TP9"}="2HM2 TP9"; $resn_exch{"HM23 TP9"}="3HM2 TP9";
$resn_exch{"HM41 TP9"}="1HM4 TP9"; $resn_exch{"HM42 TP9"}="2HM4 TP9"; $resn_exch{"HM43 TP9"}="3HM4 TP9";
$resn_exch{" H31 TPA"}="1H3  TPA"; $resn_exch{" H32 TPA"}="2H3  TPA"; $resn_exch{" HN1 TPA"}="1HN  TPA";
$resn_exch{" HN2 TPA"}="2HN  TPA"; $resn_exch{" HN3 TPA"}="3HN  TPA"; $resn_exch{"H211 TPB"}="1H21 TPB";
$resn_exch{"H212 TPB"}="2H21 TPB"; $resn_exch{"H213 TPB"}="3H21 TPB"; $resn_exch{"H611 TPB"}="1H61 TPB";
$resn_exch{"H612 TPB"}="2H61 TPB"; $resn_exch{"H613 TPB"}="3H61 TPB"; $resn_exch{"H411 TPB"}="1H41 TPB";
$resn_exch{"H412 TPB"}="2H41 TPB"; $resn_exch{"H413 TPB"}="3H41 TPB"; $resn_exch{"HN41 TPC"}="1HN4 TPC";
$resn_exch{"HN42 TPC"}="2HN4 TPC"; $resn_exch{"H2'1 TPC"}="1H2' TPC"; $resn_exch{"H2'2 TPC"}="2H2' TPC";
$resn_exch{"H5'1 TPC"}="1H5' TPC"; $resn_exch{"H5'2 TPC"}="2H5' TPC"; $resn_exch{" H91 TPD"}="1H9  TPD";
$resn_exch{" H92 TPD"}="2H9  TPD"; $resn_exch{"HNL1 TPD"}="1HNL TPD"; $resn_exch{"HNL2 TPD"}="2HNL TPD";
$resn_exch{"HPA2 TPE"}="2HPA TPE"; $resn_exch{"HPB2 TPE"}="2HPB TPE"; $resn_exch{"H5'2 TPE"}="2H5* TPE";
$resn_exch{"H5'1 TPE"}="1H5* TPE"; $resn_exch{" H4' TPE"}=" H4* TPE"; $resn_exch{" H3' TPE"}=" H3* TPE";
$resn_exch{"HHR3 TPE"}="3HHR TPE"; $resn_exch{"HHR2 TPE"}="2HHR TPE"; $resn_exch{"HHR1 TPE"}="1HHR TPE";
$resn_exch{"H2'2 TPE"}="2H2* TPE"; $resn_exch{"H2'1 TPE"}="1H2* TPE"; $resn_exch{" H1' TPE"}=" H1* TPE";
$resn_exch{"H5M3 TPE"}="3H5M TPE"; $resn_exch{"H5M2 TPE"}="2H5M TPE"; $resn_exch{"H5M1 TPE"}="1H5M TPE";
$resn_exch{"HC21 TPF"}="1HC2 TPF"; $resn_exch{"HC22 TPF"}="2HC2 TPF"; $resn_exch{"HC51 TPF"}="1HC5 TPF";
$resn_exch{"HC52 TPF"}="2HC5 TPF"; $resn_exch{"HBZ1 TPG"}="1HBZ TPG"; $resn_exch{"HBZ2 TPG"}="2HBZ TPG";
$resn_exch{"HBZ3 TPG"}="3HBZ TPG"; $resn_exch{"HBW1 TPG"}="1HBW TPG"; $resn_exch{"HBW2 TPG"}="2HBW TPG";
$resn_exch{"HBW3 TPG"}="3HBW TPG"; $resn_exch{"HBC1 TPG"}="1HBC TPG"; $resn_exch{"HBC2 TPG"}="2HBC TPG";
$resn_exch{"HBC3 TPG"}="3HBC TPG"; $resn_exch{"HBI1 TPG"}="1HBI TPG"; $resn_exch{"HBI2 TPG"}="2HBI TPG";
$resn_exch{"HBT1 TPG"}="1HBT TPG"; $resn_exch{"HBT2 TPG"}="2HBT TPG"; $resn_exch{" H5' TPG"}="1H5* TPG";
$resn_exch{"H5'' TPG"}="2H5* TPG"; $resn_exch{" H4' TPG"}=" H4* TPG"; $resn_exch{" H3' TPG"}=" H3* TPG";
$resn_exch{" H2' TPG"}=" H2* TPG"; $resn_exch{" H1' TPG"}=" H1* TPG"; $resn_exch{"HN21 TPG"}="1HN2 TPG";
$resn_exch{"HN22 TPG"}="2HN2 TPG"; $resn_exch{" H   TPH"}="1NH  TPH"; $resn_exch{" H2  TPH"}="2NH  TPH";
$resn_exch{" HB2 TPH"}="1HB  TPH"; $resn_exch{" HB3 TPH"}="2HB  TPH"; $resn_exch{"HO11 TPI"}="1HO1 TPI";
$resn_exch{"HO21 TPI"}="1HO2 TPI"; $resn_exch{" H21 TPI"}="1H2  TPI"; $resn_exch{" H41 TPI"}="1H4  TPI";
$resn_exch{" H51 TPI"}="1H5  TPI"; $resn_exch{" H61 TPI"}="1H6  TPI"; $resn_exch{" H81 TPI"}="1H8  TPI";
$resn_exch{" H91 TPI"}="1H9  TPI"; $resn_exch{"HN11 TPI"}="1HN1 TPI"; $resn_exch{"H131 TPI"}="1H13 TPI";
$resn_exch{"HN21 TPI"}="1HN2 TPI"; $resn_exch{"HN22 TPI"}="2HN2 TPI"; $resn_exch{"H151 TPI"}="1H15 TPI";
$resn_exch{"H152 TPI"}="2H15 TPI"; $resn_exch{"H161 TPI"}="1H16 TPI"; $resn_exch{"H162 TPI"}="2H16 TPI";
$resn_exch{"HO71 TPI"}="1HO7 TPI"; $resn_exch{" H2  TPL"}=" HN2 TPL"; $resn_exch{" HB1 TPL"}="1HB  TPL";
$resn_exch{" HB2 TPL"}="2HB  TPL"; $resn_exch{" HC1 TPL"}="1HC  TPL"; $resn_exch{" HC2 TPL"}="2HC  TPL";
$resn_exch{"HC21 TPM"}="1HC2 TPM"; $resn_exch{"HC31 TPM"}="1HC3 TPM"; $resn_exch{"HC51 TPM"}="1HC5 TPM";
$resn_exch{"HC61 TPM"}="1HC6 TPM"; $resn_exch{"HC71 TPM"}="1HC7 TPM"; $resn_exch{"HC72 TPM"}="2HC7 TPM";
$resn_exch{"HN11 TPM"}="1HN1 TPM"; $resn_exch{"HN12 TPM"}="2HN1 TPM"; $resn_exch{"HN21 TPM"}="1HN2 TPM";
$resn_exch{"HN31 TPM"}="1HN3 TPM"; $resn_exch{"HC91 TPM"}="1HC9 TPM"; $resn_exch{"HC92 TPM"}="2HC9 TPM";
$resn_exch{"H101 TPM"}="1H10 TPM"; $resn_exch{"H102 TPM"}="2H10 TPM"; $resn_exch{"H111 TPM"}="1H11 TPM";
$resn_exch{"H112 TPM"}="2H11 TPM"; $resn_exch{"H121 TPM"}="1H12 TPM"; $resn_exch{"H122 TPM"}="2H12 TPM";
$resn_exch{"H8'1 TPN"}="1H8' TPN"; $resn_exch{"H8'2 TPN"}="2H8' TPN"; $resn_exch{"H5'1 TPN"}="1H5' TPN";
$resn_exch{"H5'2 TPN"}="2H5' TPN"; $resn_exch{"H3'1 TPN"}="1H3' TPN"; $resn_exch{"H3'2 TPN"}="2H3' TPN";
$resn_exch{"H2'1 TPN"}="1H2' TPN"; $resn_exch{"H2'2 TPN"}="2H2' TPN"; $resn_exch{"H1'1 TPN"}="1H1' TPN";
$resn_exch{"H1'2 TPN"}="2H1' TPN"; $resn_exch{"H1'3 TPN"}="3H1' TPN"; $resn_exch{"HM51 TPN"}="1HM5 TPN";
$resn_exch{"HM52 TPN"}="2HM5 TPN"; $resn_exch{"HM53 TPN"}="3HM5 TPN"; $resn_exch{" H2  TPO"}="2HN  TPO";
$resn_exch{"HG21 TPO"}="1HG2 TPO"; $resn_exch{"HG22 TPO"}="2HG2 TPO"; $resn_exch{"HG23 TPO"}="3HG2 TPO";
$resn_exch{"HOP2 TPO"}="2HOP TPO"; $resn_exch{"HOP3 TPO"}="3HOP TPO"; $resn_exch{" H2  TPQ"}="2HN  TPQ";
$resn_exch{" HB2 TPQ"}="1HB  TPQ"; $resn_exch{" HB3 TPQ"}="2HB  TPQ"; $resn_exch{"HC71 TPR"}="1HC7 TPR";
$resn_exch{"HC72 TPR"}="2HC7 TPR"; $resn_exch{"HC73 TPR"}="3HC7 TPR"; $resn_exch{"HCD1 TPR"}="1HCD TPR";
$resn_exch{"HCD2 TPR"}="2HCD TPR"; $resn_exch{"HCG1 TPR"}="1HCG TPR"; $resn_exch{"HCG2 TPR"}="2HCG TPR";
$resn_exch{"HCB1 TPR"}="1HCB TPR"; $resn_exch{"HCB2 TPR"}="2HCB TPR"; $resn_exch{" H51 TPV"}="1H5  TPV";
$resn_exch{" H52 TPV"}="2H5  TPV"; $resn_exch{"HO81 TPV"}="1HO8 TPV"; $resn_exch{" H91 TPV"}="1H9  TPV";
$resn_exch{" H92 TPV"}="2H9  TPV"; $resn_exch{"H101 TPV"}="1H10 TPV"; $resn_exch{"H102 TPV"}="2H10 TPV";
$resn_exch{"H111 TPV"}="1H11 TPV"; $resn_exch{"H112 TPV"}="2H11 TPV"; $resn_exch{"H113 TPV"}="3H11 TPV";
$resn_exch{"H121 TPV"}="1H12 TPV"; $resn_exch{"H122 TPV"}="2H12 TPV"; $resn_exch{"H131 TPV"}="1H13 TPV";
$resn_exch{"H132 TPV"}="2H13 TPV"; $resn_exch{"H151 TPV"}="1H15 TPV"; $resn_exch{"H161 TPV"}="1H16 TPV";
$resn_exch{"H171 TPV"}="1H17 TPV"; $resn_exch{"H181 TPV"}="1H18 TPV"; $resn_exch{"H191 TPV"}="1H19 TPV";
$resn_exch{"H201 TPV"}="1H20 TPV"; $resn_exch{"H211 TPV"}="1H21 TPV"; $resn_exch{"H212 TPV"}="2H21 TPV";
$resn_exch{"H221 TPV"}="1H22 TPV"; $resn_exch{"H222 TPV"}="2H22 TPV"; $resn_exch{"H223 TPV"}="3H22 TPV";
$resn_exch{"H241 TPV"}="1H24 TPV"; $resn_exch{"H251 TPV"}="1H25 TPV"; $resn_exch{"H261 TPV"}="1H26 TPV";
$resn_exch{"H281 TPV"}="1H28 TPV"; $resn_exch{"H291 TPV"}="1H29 TPV"; $resn_exch{"H351 TPV"}="1H35 TPV";
$resn_exch{"H371 TPV"}="1H37 TPV"; $resn_exch{"H381 TPV"}="1H38 TPV"; $resn_exch{"H2A1 TPW"}="1H2A TPW";
$resn_exch{"H2A2 TPW"}="2H2A TPW"; $resn_exch{"H2A3 TPW"}="3H2A TPW"; $resn_exch{"H351 TPW"}="1H35 TPW";
$resn_exch{"H352 TPW"}="2H35 TPW"; $resn_exch{"H5A1 TPW"}="1H5A TPW"; $resn_exch{"H5A2 TPW"}="2H5A TPW";
$resn_exch{"H4A1 TPW"}="1H4A TPW"; $resn_exch{"H4A2 TPW"}="2H4A TPW"; $resn_exch{"H4A3 TPW"}="3H4A TPW";
$resn_exch{"H5B1 TPW"}="1H5B TPW"; $resn_exch{"H5B2 TPW"}="2H5B TPW"; $resn_exch{" H6' TPW"}=" H6* TPW";
$resn_exch{"H4'1 TPW"}="1H4* TPW"; $resn_exch{"H4'2 TPW"}="2H4* TPW"; $resn_exch{" HD2 TPX"}="2HD  TPX";
$resn_exch{" HD1 TPX"}="1HD  TPX"; $resn_exch{" HE3 TPX"}="3HE  TPX"; $resn_exch{" HE2 TPX"}="2HE  TPX";
$resn_exch{" HE1 TPX"}="1HE  TPX"; $resn_exch{" HK2 TPX"}="2HK  TPX"; $resn_exch{" HK1 TPX"}="1HK  TPX";
$resn_exch{" HN1 TPX"}="1H_N TPX"; $resn_exch{" HJ3 TPX"}="3HJ  TPX"; $resn_exch{" HJ2 TPX"}="2HJ  TPX";
$resn_exch{" HJ1 TPX"}="1HJ  TPX"; $resn_exch{" HD2 TPY"}="2HD  TPY"; $resn_exch{" HD1 TPY"}="1HD  TPY";
$resn_exch{" HE3 TPY"}="3HE  TPY"; $resn_exch{" HE2 TPY"}="2HE  TPY"; $resn_exch{" HE1 TPY"}="1HE  TPY";
$resn_exch{" HN1 TPY"}="1H_N TPY"; $resn_exch{" HK2 TPY"}="2HK  TPY"; $resn_exch{" HK1 TPY"}="1HK  TPY";
$resn_exch{"HN71 TQ3"}="1HN7 TQ3"; $resn_exch{"HN72 TQ3"}="2HN7 TQ3"; $resn_exch{"H141 TQ3"}="1H14 TQ3";
$resn_exch{"H142 TQ3"}="2H14 TQ3"; $resn_exch{"HN71 TQ4"}="1HN7 TQ4"; $resn_exch{"HN72 TQ4"}="2HN7 TQ4";
$resn_exch{"H141 TQ4"}="1H14 TQ4"; $resn_exch{"H142 TQ4"}="2H14 TQ4"; $resn_exch{"H321 TQ4"}="1H32 TQ4";
$resn_exch{"H322 TQ4"}="2H32 TQ4"; $resn_exch{"H323 TQ4"}="3H32 TQ4"; $resn_exch{"HN71 TQ5"}="1HN7 TQ5";
$resn_exch{"HN72 TQ5"}="2HN7 TQ5"; $resn_exch{"H141 TQ5"}="1H14 TQ5"; $resn_exch{"H142 TQ5"}="2H14 TQ5";
$resn_exch{"H331 TQ5"}="1H33 TQ5"; $resn_exch{"H332 TQ5"}="2H33 TQ5"; $resn_exch{"H333 TQ5"}="3H33 TQ5";
$resn_exch{"H371 TQ5"}="1H37 TQ5"; $resn_exch{"H372 TQ5"}="2H37 TQ5"; $resn_exch{"H373 TQ5"}="3H37 TQ5";
$resn_exch{"H411 TQ5"}="1H41 TQ5"; $resn_exch{"H412 TQ5"}="2H41 TQ5"; $resn_exch{"H413 TQ5"}="3H41 TQ5";
$resn_exch{"H111 TQ6"}="1H11 TQ6"; $resn_exch{"H112 TQ6"}="2H11 TQ6"; $resn_exch{"H121 TQ6"}="1H12 TQ6";
$resn_exch{"H122 TQ6"}="2H12 TQ6"; $resn_exch{"H321 TQ6"}="1H32 TQ6"; $resn_exch{"H322 TQ6"}="2H32 TQ6";
$resn_exch{"H331 TQ6"}="1H33 TQ6"; $resn_exch{"H332 TQ6"}="2H33 TQ6"; $resn_exch{"H351 TQ6"}="1H35 TQ6";
$resn_exch{"H352 TQ6"}="2H35 TQ6"; $resn_exch{"H361 TQ6"}="1H36 TQ6"; $resn_exch{"H362 TQ6"}="2H36 TQ6";
$resn_exch{" H2' TQD"}=" H2* TQD"; $resn_exch{" H4D TQD"}=" H4* TQD"; $resn_exch{"H5'1 TQD"}="1H5* TQD";
$resn_exch{"H5'2 TQD"}="2H5* TQD"; $resn_exch{" H6' TQD"}=" H6* TQD"; $resn_exch{"H7'1 TQD"}="1H7* TQD";
$resn_exch{"H7'2 TQD"}="2H7* TQD"; $resn_exch{" H81 TQD"}="1H8  TQD"; $resn_exch{" H82 TQD"}="2H8  TQD";
$resn_exch{"H8'1 TQD"}="1H8* TQD"; $resn_exch{"H8'2 TQD"}="2H8* TQD"; $resn_exch{"H2'1 TQD"}="1H2' TQD";
$resn_exch{"H2'2 TQD"}="2H2' TQD"; $resn_exch{"H3B1 TQD"}="1H3B TQD"; $resn_exch{"H3B2 TQD"}="2H3B TQD";
$resn_exch{"H3B3 TQD"}="3H3B TQD"; $resn_exch{"H4B1 TQD"}="1H4B TQD"; $resn_exch{"H4B2 TQD"}="2H4B TQD";
$resn_exch{"H4B3 TQD"}="3H4B TQD"; $resn_exch{"H5B1 TQD"}="1H5B TQD"; $resn_exch{"H5B2 TQD"}="2H5B TQD";
$resn_exch{"H5B3 TQD"}="3H5B TQD"; $resn_exch{"H6'1 TQD"}="1H6' TQD"; $resn_exch{"H6'2 TQD"}="2H6' TQD";
$resn_exch{" H1D TQD"}=" H1* TQD"; $resn_exch{"H2A1 TQD"}="1H2* TQD"; $resn_exch{"H2A2 TQD"}="2H2* TQD";
$resn_exch{" H3D TQD"}=" H3* TQD"; $resn_exch{"H4'1 TQD"}="1H4* TQD"; $resn_exch{"H4'2 TQD"}="2H4* TQD";
$resn_exch{"H9'1 TQD"}="1H9* TQD"; $resn_exch{"H9'2 TQD"}="2H9* TQD"; $resn_exch{"H9'3 TQD"}="3H9* TQD";
$resn_exch{" H   TQQ"}="1HN  TQQ"; $resn_exch{" H2  TQQ"}="2HN  TQQ"; $resn_exch{" HB2 TQQ"}="1HB  TQQ";
$resn_exch{" HB3 TQQ"}="2HB  TQQ"; $resn_exch{" H4B TQT"}=" H4* TQT"; $resn_exch{"H7'1 TQT"}="1H7' TQT";
$resn_exch{"H7'2 TQT"}="2H7' TQT"; $resn_exch{" H8' TQT"}=" H8* TQT"; $resn_exch{"H9'1 TQT"}="1H9' TQT";
$resn_exch{"H9'2 TQT"}="2H9' TQT"; $resn_exch{"H'01 TQT"}="1H*0 TQT"; $resn_exch{"H'02 TQT"}="2H*0 TQT";
$resn_exch{" H'1 TQT"}=" H*1 TQT"; $resn_exch{"H'21 TQT"}="1H*2 TQT"; $resn_exch{"H'22 TQT"}="2H*2 TQT";
$resn_exch{"H'31 TQT"}="1H*3 TQT"; $resn_exch{"H'32 TQT"}="2H*3 TQT"; $resn_exch{"H'41 TQT"}="1H*4 TQT";
$resn_exch{"H'42 TQT"}="2H*4 TQT"; $resn_exch{"H'51 TQT"}="1H*5 TQT"; $resn_exch{"H'52 TQT"}="2H*5 TQT";
$resn_exch{" H'6 TQT"}=" H*6 TQT"; $resn_exch{"H'71 TQT"}="1H*7 TQT"; $resn_exch{"H'72 TQT"}="2H*7 TQT";
$resn_exch{"H2'1 TQT"}="1H2' TQT"; $resn_exch{"H2'2 TQT"}="2H2' TQT"; $resn_exch{"H4'1 TQT"}="1H4' TQT";
$resn_exch{"H4'2 TQT"}="2H4' TQT"; $resn_exch{"H5'1 TQT"}="1H5' TQT"; $resn_exch{"H5'2 TQT"}="2H5' TQT";
$resn_exch{"H8'1 TQT"}="1H8' TQT"; $resn_exch{"H8'2 TQT"}="2H8' TQT"; $resn_exch{" HG1 TRA"}="1HG  TRA";
$resn_exch{" HG2 TRA"}="2HG  TRA"; $resn_exch{" H41 TRB"}="1H4  TRB"; $resn_exch{" H42 TRB"}="2H4  TRB";
$resn_exch{" H71 TRB"}="1H7  TRB"; $resn_exch{" H72 TRB"}="2H7  TRB"; $resn_exch{"H111 TRB"}="1H11 TRB";
$resn_exch{"H112 TRB"}="2H11 TRB"; $resn_exch{"H113 TRB"}="3H11 TRB"; $resn_exch{"H121 TRB"}="1H12 TRB";
$resn_exch{"H122 TRB"}="2H12 TRB"; $resn_exch{"H151 TRB"}="1H15 TRB"; $resn_exch{"H152 TRB"}="2H15 TRB";
$resn_exch{"H153 TRB"}="3H15 TRB"; $resn_exch{"H161 TRB"}="1H16 TRB"; $resn_exch{"H162 TRB"}="2H16 TRB";
$resn_exch{"H163 TRB"}="3H16 TRB"; $resn_exch{" H21 TRC"}="1H2  TRC"; $resn_exch{" H22 TRC"}="2H2  TRC";
$resn_exch{" H41 TRC"}="1H4  TRC"; $resn_exch{" H42 TRC"}="2H4  TRC"; $resn_exch{" H11 TRD"}="1H1  TRD";
$resn_exch{" H12 TRD"}="2H1  TRD"; $resn_exch{" H13 TRD"}="3H1  TRD"; $resn_exch{" H21 TRD"}="1H2  TRD";
$resn_exch{" H22 TRD"}="2H2  TRD"; $resn_exch{" H31 TRD"}="1H3  TRD"; $resn_exch{" H32 TRD"}="2H3  TRD";
$resn_exch{" H41 TRD"}="1H4  TRD"; $resn_exch{" H42 TRD"}="2H4  TRD"; $resn_exch{" H51 TRD"}="1H5  TRD";
$resn_exch{" H52 TRD"}="2H5  TRD"; $resn_exch{" H61 TRD"}="1H6  TRD"; $resn_exch{" H62 TRD"}="2H6  TRD";
$resn_exch{" H71 TRD"}="1H7  TRD"; $resn_exch{" H72 TRD"}="2H7  TRD"; $resn_exch{" H81 TRD"}="1H8  TRD";
$resn_exch{" H82 TRD"}="2H8  TRD"; $resn_exch{" H91 TRD"}="1H9  TRD"; $resn_exch{" H92 TRD"}="2H9  TRD";
$resn_exch{"H101 TRD"}="1H10 TRD"; $resn_exch{"H102 TRD"}="2H10 TRD"; $resn_exch{"H111 TRD"}="1H11 TRD";
$resn_exch{"H112 TRD"}="2H11 TRD"; $resn_exch{"H121 TRD"}="1H12 TRD"; $resn_exch{"H122 TRD"}="2H12 TRD";
$resn_exch{"H131 TRD"}="1H13 TRD"; $resn_exch{"H132 TRD"}="2H13 TRD"; $resn_exch{"H133 TRD"}="3H13 TRD";
$resn_exch{"HC61 TRE"}="1HC6 TRE"; $resn_exch{"HC62 TRE"}="2HC6 TRE"; $resn_exch{"H6P1 TRE"}="1H6P TRE";
$resn_exch{"H6P2 TRE"}="2H6P TRE"; $resn_exch{" H   TRF"}="1HN  TRF"; $resn_exch{" H2  TRF"}="2HN  TRF";
$resn_exch{" HB2 TRF"}="1HB  TRF"; $resn_exch{" HB3 TRF"}="2HB  TRF"; $resn_exch{" HB1 TRG"}="1HB  TRG";
$resn_exch{" HB2 TRG"}="2HB  TRG"; $resn_exch{" HG1 TRG"}="1HG  TRG"; $resn_exch{" HG2 TRG"}="2HG  TRG";
$resn_exch{" HD1 TRG"}="1HD  TRG"; $resn_exch{" HD2 TRG"}="2HD  TRG"; $resn_exch{" HE1 TRG"}="1HE  TRG";
$resn_exch{" HE2 TRG"}="2HE  TRG"; $resn_exch{"HH11 TRG"}="1HH1 TRG"; $resn_exch{"HH12 TRG"}="2HH1 TRG";
$resn_exch{"HH13 TRG"}="3HH1 TRG"; $resn_exch{"HH21 TRG"}="1HH2 TRG"; $resn_exch{"HH22 TRG"}="2HH2 TRG";
$resn_exch{"HH23 TRG"}="3HH2 TRG"; $resn_exch{"H4PO TRH"}="OH4P TRH"; $resn_exch{"HC61 TRH"}="1HC6 TRH";
$resn_exch{"HC62 TRH"}="2HC6 TRH"; $resn_exch{"HC63 TRH"}="3HC6 TRH"; $resn_exch{"H1PO TRH"}="OH1P TRH";
$resn_exch{"H5'1 TRH"}="1H5' TRH"; $resn_exch{"H5'2 TRH"}="2H5' TRH"; $resn_exch{"H4'C TRH"}="CH4' TRH";
$resn_exch{"H3'C TRH"}="CH3' TRH"; $resn_exch{"H3'O TRH"}="OH3' TRH"; $resn_exch{"H2'1 TRH"}="1H2' TRH";
$resn_exch{"H2'2 TRH"}="2H2' TRH"; $resn_exch{"H1'C TRH"}="CH1' TRH"; $resn_exch{"H31N TRH"}="NH31 TRH";
$resn_exch{"HCA1 TRH"}="1HCA TRH"; $resn_exch{"HCA2 TRH"}="2HCA TRH"; $resn_exch{"HCA3 TRH"}="3HCA TRH";
$resn_exch{"H61C TRH"}="CH61 TRH"; $resn_exch{" HN1 TRJ"}="1HN  TRJ"; $resn_exch{" HN2 TRJ"}="2HN  TRJ";
$resn_exch{" HA1 TRJ"}="1HA  TRJ"; $resn_exch{" HA2 TRJ"}="2HA  TRJ"; $resn_exch{"HD11 TRJ"}="1HD1 TRJ";
$resn_exch{"HD21 TRJ"}="1HD2 TRJ"; $resn_exch{"HE21 TRJ"}="1HE2 TRJ"; $resn_exch{" HZ1 TRJ"}="1HZ  TRJ";
$resn_exch{" HB1 TRJ"}="1HB  TRJ"; $resn_exch{" HB2 TRJ"}="2HB  TRJ"; $resn_exch{"HN21 TRJ"}="1HN2 TRJ";
$resn_exch{"HN22 TRJ"}="2HN2 TRJ"; $resn_exch{"HC31 TRL"}="1HC3 TRL"; $resn_exch{"HC32 TRL"}="2HC3 TRL";
$resn_exch{"HC41 TRL"}="1HC4 TRL"; $resn_exch{"HC42 TRL"}="2HC4 TRL"; $resn_exch{"HC43 TRL"}="3HC4 TRL";
$resn_exch{"HC91 TRL"}="1HC9 TRL"; $resn_exch{"HC92 TRL"}="2HC9 TRL"; $resn_exch{"H121 TRL"}="1H12 TRL";
$resn_exch{"H122 TRL"}="2H12 TRL"; $resn_exch{"HC11 TRL"}="1HC1 TRL"; $resn_exch{"HC12 TRL"}="2HC1 TRL";
$resn_exch{"HC13 TRL"}="3HC1 TRL"; $resn_exch{"H131 TRL"}="1H13 TRL"; $resn_exch{"H132 TRL"}="2H13 TRL";
$resn_exch{"H133 TRL"}="3H13 TRL"; $resn_exch{"HC71 TRL"}="1HC7 TRL"; $resn_exch{"HC72 TRL"}="2HC7 TRL";
$resn_exch{"HC73 TRL"}="3HC7 TRL"; $resn_exch{"H211 TRL"}="1H21 TRL"; $resn_exch{"H212 TRL"}="2H21 TRL";
$resn_exch{"H213 TRL"}="3H21 TRL"; $resn_exch{"H261 TRL"}="1H26 TRL"; $resn_exch{"H262 TRL"}="2H26 TRL";
$resn_exch{"H263 TRL"}="3H26 TRL"; $resn_exch{"H251 TRL"}="1H25 TRL"; $resn_exch{"H252 TRL"}="2H25 TRL";
$resn_exch{"H253 TRL"}="3H25 TRL"; $resn_exch{"HE11 TRM"}="1HE1 TRM"; $resn_exch{"HZ21 TRM"}="1HZ2 TRM";
$resn_exch{"HH21 TRM"}="1HH2 TRM"; $resn_exch{"HZ31 TRM"}="1HZ3 TRM"; $resn_exch{"HE31 TRM"}="1HE3 TRM";
$resn_exch{" H   TRN"}="1HN  TRN"; $resn_exch{" H2  TRN"}="2HN  TRN"; $resn_exch{" HB2 TRN"}="1HB  TRN";
$resn_exch{" HB3 TRN"}="2HB  TRN"; $resn_exch{" HE1 TRN"}=" HNE TRN"; $resn_exch{" H2  TRO"}=" HN2 TRO";
$resn_exch{" HB2 TRO"}="1HB  TRO"; $resn_exch{" HB3 TRO"}="2HB  TRO"; $resn_exch{" HD1 TRO"}=" HOD TRO";
$resn_exch{" H2  TRP"}=" HN2 TRP"; $resn_exch{" HB2 TRP"}="1HB  TRP"; $resn_exch{" HB3 TRP"}="2HB  TRP";
$resn_exch{" H   TRQ"}="1HN  TRQ"; $resn_exch{" H2  TRQ"}="2HN  TRQ"; $resn_exch{" HB2 TRQ"}="1HB  TRQ";
$resn_exch{" HB3 TRQ"}="2HB  TRQ"; $resn_exch{" HE1 TRQ"}=" HNE TRQ"; $resn_exch{" H21 TRR"}="1H2  TRR";
$resn_exch{" H22 TRR"}="2H2  TRR"; $resn_exch{" H41 TRR"}="1H4  TRR"; $resn_exch{" H42 TRR"}="2H4  TRR";
$resn_exch{" H71 TRR"}="1H7  TRR"; $resn_exch{" H72 TRR"}="2H7  TRR"; $resn_exch{"H171 TRR"}="1H17 TRR";
$resn_exch{"H172 TRR"}="2H17 TRR"; $resn_exch{"H173 TRR"}="3H17 TRR"; $resn_exch{"H181 TRR"}="1H18 TRR";
$resn_exch{"H182 TRR"}="2H18 TRR"; $resn_exch{"H183 TRR"}="3H18 TRR"; $resn_exch{"H191 TRR"}="1H19 TRR";
$resn_exch{"H192 TRR"}="2H19 TRR"; $resn_exch{"H193 TRR"}="3H19 TRR"; $resn_exch{" H11 TRS"}="1H1  TRS";
$resn_exch{" H12 TRS"}="2H1  TRS"; $resn_exch{" H21 TRS"}="1H2  TRS"; $resn_exch{" H22 TRS"}="2H2  TRS";
$resn_exch{" H31 TRS"}="1H3  TRS"; $resn_exch{" H32 TRS"}="2H3  TRS"; $resn_exch{" HN1 TRS"}="1HN  TRS";
$resn_exch{" HN2 TRS"}="2HN  TRS"; $resn_exch{" HN3 TRS"}="3HN  TRS"; $resn_exch{"H251 TRT"}="1H25 TRT";
$resn_exch{"H252 TRT"}="2H25 TRT"; $resn_exch{"H253 TRT"}="3H25 TRT"; $resn_exch{"H231 TRT"}="1H23 TRT";
$resn_exch{"H232 TRT"}="2H23 TRT"; $resn_exch{"H221 TRT"}="1H22 TRT"; $resn_exch{"H222 TRT"}="2H22 TRT";
$resn_exch{"H201 TRT"}="1H20 TRT"; $resn_exch{"H202 TRT"}="2H20 TRT"; $resn_exch{"H191 TRT"}="1H19 TRT";
$resn_exch{"H192 TRT"}="2H19 TRT"; $resn_exch{"H171 TRT"}="1H17 TRT"; $resn_exch{"H172 TRT"}="2H17 TRT";
$resn_exch{"H161 TRT"}="1H16 TRT"; $resn_exch{"H162 TRT"}="2H16 TRT"; $resn_exch{"H8C1 TRT"}="1H8C TRT";
$resn_exch{"H8C2 TRT"}="2H8C TRT"; $resn_exch{"H8C3 TRT"}="3H8C TRT"; $resn_exch{"H7C1 TRT"}="1H7C TRT";
$resn_exch{"H7C2 TRT"}="2H7C TRT"; $resn_exch{"H7C3 TRT"}="3H7C TRT"; $resn_exch{"H5C1 TRT"}="1H5C TRT";
$resn_exch{"H5C2 TRT"}="2H5C TRT"; $resn_exch{"H2C1 TRT"}="1H2C TRT"; $resn_exch{"H2C2 TRT"}="2H2C TRT";
$resn_exch{"H2C3 TRT"}="3H2C TRT"; $resn_exch{"H4C1 TRT"}="1H4C TRT"; $resn_exch{"H4C2 TRT"}="2H4C TRT";
$resn_exch{"H4C3 TRT"}="3H4C TRT"; $resn_exch{"H3C1 TRT"}="1H3C TRT"; $resn_exch{"H3C2 TRT"}="2H3C TRT";
$resn_exch{"H3C3 TRT"}="3H3C TRT"; $resn_exch{"H151 TRU"}="1H15 TRU"; $resn_exch{"H152 TRU"}="2H15 TRU";
$resn_exch{" H2  TRW"}=" HN2 TRW"; $resn_exch{" HB2 TRW"}="1HB  TRW"; $resn_exch{" HB3 TRW"}="2HB  TRW";
$resn_exch{" H2  TRX"}=" HN2 TRX"; $resn_exch{" HB2 TRX"}="1HB  TRX"; $resn_exch{" HB3 TRX"}="2HB  TRX";
$resn_exch{"H121 TRZ"}="1H12 TRZ"; $resn_exch{"H122 TRZ"}="2H12 TRZ"; $resn_exch{"H123 TRZ"}="3H12 TRZ";
$resn_exch{"H141 TRZ"}="1H14 TRZ"; $resn_exch{"H142 TRZ"}="2H14 TRZ"; $resn_exch{"H143 TRZ"}="3H14 TRZ";
$resn_exch{"H151 TRZ"}="1H15 TRZ"; $resn_exch{"H152 TRZ"}="2H15 TRZ"; $resn_exch{"H153 TRZ"}="3H15 TRZ";
$resn_exch{"H161 TRZ"}="1H16 TRZ"; $resn_exch{"H162 TRZ"}="2H16 TRZ"; $resn_exch{"H163 TRZ"}="3H16 TRZ";
$resn_exch{"H171 TRZ"}="1H17 TRZ"; $resn_exch{"H172 TRZ"}="2H17 TRZ"; $resn_exch{"H173 TRZ"}="3H17 TRZ";
$resn_exch{" H71 TRZ"}="1H7  TRZ"; $resn_exch{" H72 TRZ"}="2H7  TRZ"; $resn_exch{" H73 TRZ"}="3H7  TRZ";
$resn_exch{" H81 TRZ"}="1H8  TRZ"; $resn_exch{" H82 TRZ"}="2H8  TRZ"; $resn_exch{"H181 TRZ"}="1H18 TRZ";
$resn_exch{"H182 TRZ"}="2H18 TRZ"; $resn_exch{"HOP1  TS"}="1HOP  TS"; $resn_exch{"HOP3  TS"}="3HOP  TS";
$resn_exch{" H5'  TS"}="1H5*  TS"; $resn_exch{"H5''  TS"}="2H5*  TS"; $resn_exch{" H4'  TS"}=" H4*  TS";
$resn_exch{" H3'  TS"}=" H3*  TS"; $resn_exch{"HO3'  TS"}="*HO3  TS"; $resn_exch{" H2'  TS"}="1H2*  TS";
$resn_exch{"H2''  TS"}="2H2*  TS"; $resn_exch{" H1'  TS"}=" H1*  TS"; $resn_exch{"HM51  TS"}="1HM5  TS";
$resn_exch{"HM52  TS"}="2HM5  TS"; $resn_exch{"HM53  TS"}="3HM5  TS"; $resn_exch{"H101 TS0"}="1H10 TS0";
$resn_exch{"H102 TS0"}="2H10 TS0"; $resn_exch{"H141 TS0"}="1H14 TS0"; $resn_exch{"H142 TS0"}="2H14 TS0";
$resn_exch{"H151 TS0"}="1H15 TS0"; $resn_exch{"H152 TS0"}="2H15 TS0"; $resn_exch{"H161 TS0"}="1H16 TS0";
$resn_exch{"H162 TS0"}="2H16 TS0"; $resn_exch{"H171 TS0"}="1H17 TS0"; $resn_exch{"H172 TS0"}="2H17 TS0";
$resn_exch{"H181 TS0"}="1H18 TS0"; $resn_exch{"H182 TS0"}="2H18 TS0"; $resn_exch{"H101 TS1"}="1H10 TS1";
$resn_exch{"H102 TS1"}="2H10 TS1"; $resn_exch{"H141 TS1"}="1H14 TS1"; $resn_exch{"H142 TS1"}="2H14 TS1";
$resn_exch{"H151 TS1"}="1H15 TS1"; $resn_exch{"H152 TS1"}="2H15 TS1"; $resn_exch{"H161 TS1"}="1H16 TS1";
$resn_exch{"H162 TS1"}="2H16 TS1"; $resn_exch{"H171 TS1"}="1H17 TS1"; $resn_exch{"H172 TS1"}="2H17 TS1";
$resn_exch{"H181 TS1"}="1H18 TS1"; $resn_exch{"H182 TS1"}="2H18 TS1"; $resn_exch{"HN11 TS2"}="1HN1 TS2";
$resn_exch{"HN12 TS2"}="2HN1 TS2"; $resn_exch{"HB11 TS2"}="1HB1 TS2"; $resn_exch{"HB12 TS2"}="2HB1 TS2";
$resn_exch{"HG11 TS2"}="1HG1 TS2"; $resn_exch{"HG12 TS2"}="2HG1 TS2"; $resn_exch{"HB21 TS2"}="1HB2 TS2";
$resn_exch{"HB22 TS2"}="2HB2 TS2"; $resn_exch{"HA31 TS2"}="1HA3 TS2"; $resn_exch{"HA32 TS2"}="2HA3 TS2";
$resn_exch{"HA41 TS2"}="1HA4 TS2"; $resn_exch{"HA42 TS2"}="2HA4 TS2"; $resn_exch{"HB51 TS2"}="1HB5 TS2";
$resn_exch{"HB52 TS2"}="2HB5 TS2"; $resn_exch{"HN61 TS2"}="1HN6 TS2"; $resn_exch{"HN62 TS2"}="2HN6 TS2";
$resn_exch{"HB61 TS2"}="1HB6 TS2"; $resn_exch{"HB62 TS2"}="2HB6 TS2"; $resn_exch{"HG61 TS2"}="1HG6 TS2";
$resn_exch{"HG62 TS2"}="2HG6 TS2"; $resn_exch{"HN1' TS2"}="'HN1 TS2"; $resn_exch{"H1'1 TS2"}="1H1' TS2";
$resn_exch{"H1'2 TS2"}="2H1' TS2"; $resn_exch{"H2'1 TS2"}="1H2' TS2"; $resn_exch{"H2'2 TS2"}="2H2' TS2";
$resn_exch{"HN2' TS2"}="'HN2 TS2"; $resn_exch{"H3'1 TS2"}="1H3' TS2"; $resn_exch{"H3'2 TS2"}="2H3' TS2";
$resn_exch{"H4'1 TS2"}="1H4' TS2"; $resn_exch{"H4'2 TS2"}="2H4' TS2"; $resn_exch{"H5'1 TS2"}="1H5' TS2";
$resn_exch{"H5'2 TS2"}="2H5' TS2"; $resn_exch{"H6'1 TS2"}="1H6' TS2"; $resn_exch{"H6'2 TS2"}="2H6' TS2";
$resn_exch{"H7'1 TS2"}="1H7' TS2"; $resn_exch{"H7'2 TS2"}="2H7' TS2"; $resn_exch{"HN3' TS2"}="'HN3 TS2";
$resn_exch{"HN11 TS3"}="1HN1 TS3"; $resn_exch{"HN12 TS3"}="2HN1 TS3"; $resn_exch{"HB11 TS3"}="1HB1 TS3";
$resn_exch{"HB12 TS3"}="2HB1 TS3"; $resn_exch{"HG11 TS3"}="1HG1 TS3"; $resn_exch{"HG12 TS3"}="2HG1 TS3";
$resn_exch{"HB21 TS3"}="1HB2 TS3"; $resn_exch{"HB22 TS3"}="2HB2 TS3"; $resn_exch{"HA31 TS3"}="1HA3 TS3";
$resn_exch{"HA32 TS3"}="2HA3 TS3"; $resn_exch{"HA41 TS3"}="1HA4 TS3"; $resn_exch{"HA42 TS3"}="2HA4 TS3";
$resn_exch{" HB1 TS3"}="1HB  TS3"; $resn_exch{" HB2 TS3"}="2HB  TS3"; $resn_exch{"HN61 TS3"}="1HN6 TS3";
$resn_exch{"HN62 TS3"}="2HN6 TS3"; $resn_exch{"HB61 TS3"}="1HB6 TS3"; $resn_exch{"HB62 TS3"}="2HB6 TS3";
$resn_exch{"HG61 TS3"}="1HG6 TS3"; $resn_exch{"HG62 TS3"}="2HG6 TS3"; $resn_exch{"HN1' TS3"}="'HN1 TS3";
$resn_exch{"H1'1 TS3"}="1H1' TS3"; $resn_exch{"H1'2 TS3"}="2H1' TS3"; $resn_exch{"H2'1 TS3"}="1H2' TS3";
$resn_exch{"H2'2 TS3"}="2H2' TS3"; $resn_exch{"HN2' TS3"}="'HN2 TS3"; $resn_exch{"H3'1 TS3"}="1H3' TS3";
$resn_exch{"H3'2 TS3"}="2H3' TS3"; $resn_exch{"H4'1 TS3"}="1H4' TS3"; $resn_exch{"H4'2 TS3"}="2H4' TS3";
$resn_exch{"H5'1 TS3"}="1H5' TS3"; $resn_exch{"H5'2 TS3"}="2H5' TS3"; $resn_exch{"H6'1 TS3"}="1H6' TS3";
$resn_exch{"H6'2 TS3"}="2H6' TS3"; $resn_exch{"H7'1 TS3"}="1H7' TS3"; $resn_exch{"H7'2 TS3"}="2H7' TS3";
$resn_exch{"HN3' TS3"}="'HN3 TS3"; $resn_exch{"HN11 TS4"}="1HN1 TS4"; $resn_exch{"HN12 TS4"}="2HN1 TS4";
$resn_exch{"HB11 TS4"}="1HB1 TS4"; $resn_exch{"HB12 TS4"}="2HB1 TS4"; $resn_exch{"HG11 TS4"}="1HG1 TS4";
$resn_exch{"HG12 TS4"}="2HG1 TS4"; $resn_exch{"HB21 TS4"}="1HB2 TS4"; $resn_exch{"HB22 TS4"}="2HB2 TS4";
$resn_exch{"HA31 TS4"}="1HA3 TS4"; $resn_exch{"HA32 TS4"}="2HA3 TS4"; $resn_exch{"HA41 TS4"}="1HA4 TS4";
$resn_exch{"HA42 TS4"}="2HA4 TS4"; $resn_exch{"HB51 TS4"}="1HB5 TS4"; $resn_exch{"HB52 TS4"}="2HB5 TS4";
$resn_exch{"HN61 TS4"}="1HN6 TS4"; $resn_exch{"HN62 TS4"}="2HN6 TS4"; $resn_exch{"HB61 TS4"}="1HB6 TS4";
$resn_exch{"HB62 TS4"}="2HB6 TS4"; $resn_exch{"HG61 TS4"}="1HG6 TS4"; $resn_exch{"HG62 TS4"}="2HG6 TS4";
$resn_exch{"HN1' TS4"}="'HN1 TS4"; $resn_exch{"H1'1 TS4"}="1H1' TS4"; $resn_exch{"H1'2 TS4"}="2H1' TS4";
$resn_exch{"H2'1 TS4"}="1H2' TS4"; $resn_exch{"H2'2 TS4"}="2H2' TS4"; $resn_exch{"HN2' TS4"}="'HN2 TS4";
$resn_exch{"H3'1 TS4"}="1H3' TS4"; $resn_exch{"H3'2 TS4"}="2H3' TS4"; $resn_exch{"H4'1 TS4"}="1H4' TS4";
$resn_exch{"H4'2 TS4"}="2H4' TS4"; $resn_exch{"H5'1 TS4"}="1H5' TS4"; $resn_exch{"H5'2 TS4"}="2H5' TS4";
$resn_exch{"H6'1 TS4"}="1H6' TS4"; $resn_exch{"H6'2 TS4"}="2H6' TS4"; $resn_exch{"H7'1 TS4"}="1H7' TS4";
$resn_exch{"H7'2 TS4"}="2H7' TS4"; $resn_exch{"HN31 TS4"}="1HN3 TS4"; $resn_exch{"HN32 TS4"}="2HN3 TS4";
$resn_exch{"H1T1 TS4"}="1H1T TS4"; $resn_exch{"H1T2 TS4"}="2H1T TS4"; $resn_exch{"H2T1 TS4"}="1H2T TS4";
$resn_exch{"H2T2 TS4"}="2H2T TS4"; $resn_exch{"H3T1 TS4"}="1H3T TS4"; $resn_exch{"H3T2 TS4"}="2H3T TS4";
$resn_exch{"H4T1 TS4"}="1H4T TS4"; $resn_exch{"H4T2 TS4"}="2H4T TS4"; $resn_exch{"H5T1 TS4"}="1H5T TS4";
$resn_exch{"H5T2 TS4"}="2H5T TS4"; $resn_exch{"H6T1 TS4"}="1H6T TS4"; $resn_exch{"H6T2 TS4"}="2H6T TS4";
$resn_exch{"H7T1 TS4"}="1H7T TS4"; $resn_exch{"H7T2 TS4"}="2H7T TS4"; $resn_exch{"HNT1 TS4"}="1HNT TS4";
$resn_exch{"HNT2 TS4"}="2HNT TS4"; $resn_exch{"HN11 TS5"}="1HN1 TS5"; $resn_exch{"HN12 TS5"}="2HN1 TS5";
$resn_exch{"HB11 TS5"}="1HB1 TS5"; $resn_exch{"HB12 TS5"}="2HB1 TS5"; $resn_exch{"HG11 TS5"}="1HG1 TS5";
$resn_exch{"HG12 TS5"}="2HG1 TS5"; $resn_exch{"HB21 TS5"}="1HB2 TS5"; $resn_exch{"HB22 TS5"}="2HB2 TS5";
$resn_exch{"HA31 TS5"}="1HA3 TS5"; $resn_exch{"HA32 TS5"}="2HA3 TS5"; $resn_exch{"HN1' TS5"}="'HN1 TS5";
$resn_exch{"H1'1 TS5"}="1H1' TS5"; $resn_exch{"H1'2 TS5"}="2H1' TS5"; $resn_exch{"H2'1 TS5"}="1H2' TS5";
$resn_exch{"H2'2 TS5"}="2H2' TS5"; $resn_exch{"HN2' TS5"}="'HN2 TS5"; $resn_exch{"H3'1 TS5"}="1H3' TS5";
$resn_exch{"H3'2 TS5"}="2H3' TS5"; $resn_exch{"H4'1 TS5"}="1H4' TS5"; $resn_exch{"H4'2 TS5"}="2H4' TS5";
$resn_exch{"H5'1 TS5"}="1H5' TS5"; $resn_exch{"H5'2 TS5"}="2H5' TS5"; $resn_exch{"H6'1 TS5"}="1H6' TS5";
$resn_exch{"H6'2 TS5"}="2H6' TS5"; $resn_exch{"H7'1 TS5"}="1H7' TS5"; $resn_exch{"H7'2 TS5"}="2H7' TS5";
$resn_exch{"HN31 TS5"}="1HN3 TS5"; $resn_exch{"HN32 TS5"}="2HN3 TS5"; $resn_exch{" H61 TSA"}="1H6  TSA";
$resn_exch{" H62 TSA"}="2H6  TSA"; $resn_exch{" H91 TSA"}="1H9  TSA"; $resn_exch{" H92 TSA"}="2H9  TSA";
$resn_exch{"HG21 TSB"}="1HG2 TSB"; $resn_exch{"HG22 TSB"}="2HG2 TSB"; $resn_exch{"HG23 TSB"}="3HG2 TSB";
$resn_exch{"HN11 TSB"}="1HN1 TSB"; $resn_exch{"HN12 TSB"}="2HN1 TSB"; $resn_exch{"H5'1 TSB"}="1H5* TSB";
$resn_exch{"H5'2 TSB"}="2H5* TSB"; $resn_exch{" H4' TSB"}=" H4* TSB"; $resn_exch{" H3' TSB"}=" H3* TSB";
$resn_exch{"HO3' TSB"}="*HO3 TSB"; $resn_exch{" H2' TSB"}=" H2* TSB"; $resn_exch{"HO2' TSB"}="*HO2 TSB";
$resn_exch{" H1' TSB"}=" H1* TSB"; $resn_exch{"HN61 TSB"}="1HN6 TSB"; $resn_exch{"HN62 TSB"}="2HN6 TSB";
$resn_exch{" HN1 TSC"}="1HN  TSC"; $resn_exch{" HN2 TSC"}="2HN  TSC"; $resn_exch{" HB1 TSC"}="1HB  TSC";
$resn_exch{" HB2 TSC"}="2HB  TSC"; $resn_exch{" D31 TSD"}="1D3  TSD"; $resn_exch{" D32 TSD"}="2D3  TSD";
$resn_exch{" D21 TSD"}="1D2  TSD"; $resn_exch{" D22 TSD"}="2D2  TSD"; $resn_exch{" H41 TSD"}="1H4  TSD";
$resn_exch{" H42 TSD"}="2H4  TSD"; $resn_exch{" H43 TSD"}="3H4  TSD"; $resn_exch{" H51 TSD"}="1H5  TSD";
$resn_exch{" H52 TSD"}="2H5  TSD"; $resn_exch{"H2P1 TSE"}="1H2P TSE"; $resn_exch{"H2P2 TSE"}="2H2P TSE";
$resn_exch{"HOP3 TSE"}="3HOP TSE"; $resn_exch{"HOP2 TSE"}="2HOP TSE"; $resn_exch{"H181 TSF"}="1H18 TSF";
$resn_exch{"H182 TSF"}="2H18 TSF"; $resn_exch{"H171 TSF"}="1H17 TSF"; $resn_exch{"H172 TSF"}="2H17 TSF";
$resn_exch{"H161 TSF"}="1H16 TSF"; $resn_exch{"H162 TSF"}="2H16 TSF"; $resn_exch{"H151 TSF"}="1H15 TSF";
$resn_exch{"H152 TSF"}="2H15 TSF"; $resn_exch{"H101 TSF"}="1H10 TSF"; $resn_exch{"H102 TSF"}="2H10 TSF";
$resn_exch{"H141 TSF"}="1H14 TSF"; $resn_exch{"H142 TSF"}="2H14 TSF"; $resn_exch{" HB1 TSH"}="1HB  TSH";
$resn_exch{" HB2 TSH"}="2HB  TSH"; $resn_exch{"HN11 TSI"}="1HN1 TSI"; $resn_exch{"HN12 TSI"}="2HN1 TSI";
$resn_exch{"HC1A TSI"}="1HC  TSI"; $resn_exch{"HC2A TSI"}="2HC  TSI"; $resn_exch{" HC3 TSI"}="3HC  TSI";
$resn_exch{" H81 TSI"}="1H_A TSI"; $resn_exch{" H82 TSI"}="2H_B TSI"; $resn_exch{" H83 TSI"}="3H_C TSI";
$resn_exch{"H141 TSL"}="1H14 TSL"; $resn_exch{"H142 TSL"}="2H14 TSL"; $resn_exch{"H143 TSL"}="3H14 TSL";
$resn_exch{"H201 TSL"}="1H20 TSL"; $resn_exch{"H202 TSL"}="2H20 TSL"; $resn_exch{"H203 TSL"}="3H20 TSL";
$resn_exch{" H11 TSM"}="1H1  TSM"; $resn_exch{" H12 TSM"}="2H1  TSM"; $resn_exch{" H13 TSM"}="3H1  TSM";
$resn_exch{" H21 TSM"}="1H2  TSM"; $resn_exch{" H22 TSM"}="2H2  TSM"; $resn_exch{" H23 TSM"}="3H2  TSM";
$resn_exch{" H31 TSM"}="1H3  TSM"; $resn_exch{" H32 TSM"}="2H3  TSM"; $resn_exch{" H33 TSM"}="3H3  TSM";
$resn_exch{" H21 TSN"}="1H2  TSN"; $resn_exch{" H31 TSN"}="1H3  TSN"; $resn_exch{" H51 TSN"}="1H5  TSN";
$resn_exch{" H61 TSN"}="1H6  TSN"; $resn_exch{"H111 TSN"}="1H11 TSN"; $resn_exch{"H121 TSN"}="1H12 TSN";
$resn_exch{"H141 TSN"}="1H14 TSN"; $resn_exch{"H142 TSN"}="2H14 TSN"; $resn_exch{"H143 TSN"}="3H14 TSN";
$resn_exch{"H151 TSN"}="1H15 TSN"; $resn_exch{"H152 TSN"}="2H15 TSN"; $resn_exch{"H153 TSN"}="3H15 TSN";
$resn_exch{"H171 TSN"}="1H17 TSN"; $resn_exch{"H172 TSN"}="2H17 TSN"; $resn_exch{"H173 TSN"}="3H17 TSN";
$resn_exch{"H161 TSN"}="1H16 TSN"; $resn_exch{"H162 TSN"}="2H16 TSN"; $resn_exch{"H163 TSN"}="3H16 TSN";
$resn_exch{"HOP2 TSP"}="2HOP TSP"; $resn_exch{"HOP3 TSP"}="3HOP TSP"; $resn_exch{" H5' TSP"}="1H5* TSP";
$resn_exch{"H5'' TSP"}="2H5* TSP"; $resn_exch{" H4' TSP"}=" H4* TSP"; $resn_exch{" H3' TSP"}=" H3* TSP";
$resn_exch{"HS3' TSP"}="*HS3 TSP"; $resn_exch{" H2' TSP"}="1H2* TSP"; $resn_exch{"H2'' TSP"}="2H2* TSP";
$resn_exch{" H1' TSP"}=" H1* TSP"; $resn_exch{" H71 TSP"}="1H5M TSP"; $resn_exch{" H72 TSP"}="2H5M TSP";
$resn_exch{" H73 TSP"}="3H5M TSP"; $resn_exch{" HN1 TSR"}="1HN  TSR"; $resn_exch{" HN2 TSR"}="2HN  TSR";
$resn_exch{" HB1 TSR"}="1HB  TSR"; $resn_exch{" HB2 TSR"}="2HB  TSR"; $resn_exch{"HN11 TSS"}="1HN1 TSS";
$resn_exch{"HN12 TSS"}="2HN1 TSS"; $resn_exch{" HA1 TSS"}="1HA  TSS"; $resn_exch{" HA2 TSS"}="2HA  TSS";
$resn_exch{" HB1 TSS"}="1HB  TSS"; $resn_exch{" HB2 TSS"}="2HB  TSS"; $resn_exch{" H11 TST"}="1H1  TST";
$resn_exch{"H12A TST"}="2H1  TST"; $resn_exch{" H21 TST"}="1H2  TST"; $resn_exch{" H22 TST"}="2H2  TST";
$resn_exch{" H41 TST"}="1H4  TST"; $resn_exch{" H42 TST"}="2H4  TST"; $resn_exch{" H71 TST"}="1H7  TST";
$resn_exch{" H72 TST"}="2H7  TST"; $resn_exch{"H101 TST"}="1H10 TST"; $resn_exch{"H102 TST"}="2H10 TST";
$resn_exch{"H103 TST"}="3H10 TST"; $resn_exch{"H171 TST"}="1H17 TST"; $resn_exch{"H172 TST"}="2H17 TST";
$resn_exch{"H173 TST"}="3H17 TST"; $resn_exch{"H181 TST"}="1H18 TST"; $resn_exch{"H182 TST"}="2H18 TST";
$resn_exch{"H183 TST"}="3H18 TST"; $resn_exch{" H71 TSU"}="1H7  TSU"; $resn_exch{" H72 TSU"}="2H7  TSU";
$resn_exch{" H73 TSU"}="3H7  TSU"; $resn_exch{"H311 TSX"}="1H31 TSX"; $resn_exch{"H312 TSX"}="2H31 TSX";
$resn_exch{"H331 TSX"}="1H33 TSX"; $resn_exch{"H332 TSX"}="2H33 TSX"; $resn_exch{"H411 TSX"}="1H41 TSX";
$resn_exch{"H412 TSX"}="2H41 TSX"; $resn_exch{"H421 TSX"}="1H42 TSX"; $resn_exch{"H422 TSX"}="2H42 TSX";
$resn_exch{"H431 TSX"}="1H43 TSX"; $resn_exch{"H432 TSX"}="2H43 TSX"; $resn_exch{"H441 TSX"}="1H44 TSX";
$resn_exch{"H442 TSX"}="2H44 TSX"; $resn_exch{"H443 TSX"}="3H44 TSX"; $resn_exch{" H91 TSX"}="1H9  TSX";
$resn_exch{"H141 TSX"}="1H14 TSX"; $resn_exch{"H142 TSX"}="2H14 TSX"; $resn_exch{" HN1 TSX"}="1HN  TSX";
$resn_exch{" H41 TSX"}="1H4  TSX"; $resn_exch{" H42 TSX"}="2H4  TSX"; $resn_exch{" H43 TSX"}="3H4  TSX";
$resn_exch{"HN11 TSZ"}="1HN1 TSZ"; $resn_exch{"HN12 TSZ"}="2HN1 TSZ"; $resn_exch{"HN31 TSZ"}="1HN3 TSZ";
$resn_exch{"HN32 TSZ"}="2HN3 TSZ"; $resn_exch{"H5'1  TT"}="1H5*  TT"; $resn_exch{"H5'2  TT"}="2H5*  TT";
$resn_exch{" H4'  TT"}=" H4*  TT"; $resn_exch{" H3'  TT"}=" H3*  TT"; $resn_exch{" H1'  TT"}=" H1*  TT";
$resn_exch{"H2'1  TT"}="1H2*  TT"; $resn_exch{"H2'2  TT"}="2H2*  TT"; $resn_exch{"H7C1  TT"}="1H7C  TT";
$resn_exch{"H7C2  TT"}="2H7C  TT"; $resn_exch{"H5A1  TT"}="1H5A  TT"; $resn_exch{"H5A2  TT"}="2H5A  TT";
$resn_exch{"H5A3  TT"}="3H5A  TT"; $resn_exch{"H5R1  TT"}="1H5R  TT"; $resn_exch{"H5R2  TT"}="2H5R  TT";
$resn_exch{"H2R1  TT"}="1H2R  TT"; $resn_exch{"H2R2  TT"}="2H2R  TT"; $resn_exch{"H5M1  TT"}="1H5M  TT";
$resn_exch{"H5M2  TT"}="2H5M  TT"; $resn_exch{"H5M3  TT"}="3H5M  TT"; $resn_exch{" H1  TTB"}="1H   TTB";
$resn_exch{" H2  TTB"}="2H   TTB"; $resn_exch{" H3A TTB"}="3H   TTB"; $resn_exch{"H181 TTB"}="1H18 TTB";
$resn_exch{"H182 TTB"}="2H18 TTB"; $resn_exch{"H183 TTB"}="3H18 TTB"; $resn_exch{"H191 TTB"}="1H19 TTB";
$resn_exch{"H192 TTB"}="2H19 TTB"; $resn_exch{"H193 TTB"}="3H19 TTB"; $resn_exch{"H201 TTB"}="1H20 TTB";
$resn_exch{"H202 TTB"}="2H20 TTB"; $resn_exch{"H211 TTB"}="1H21 TTB"; $resn_exch{"H212 TTB"}="2H21 TTB";
$resn_exch{"H231 TTB"}="1H23 TTB"; $resn_exch{"H232 TTB"}="2H23 TTB"; $resn_exch{"H233 TTB"}="3H23 TTB";
$resn_exch{"H241 TTB"}="1H24 TTB"; $resn_exch{"H242 TTB"}="2H24 TTB"; $resn_exch{"H243 TTB"}="3H24 TTB";
$resn_exch{"H251 TTC"}="1H25 TTC"; $resn_exch{"H252 TTC"}="2H25 TTC"; $resn_exch{"H311 TTC"}="1H31 TTC";
$resn_exch{"H312 TTC"}="2H31 TTC"; $resn_exch{"H313 TTC"}="3H31 TTC"; $resn_exch{"H241 TTC"}="1H24 TTC";
$resn_exch{"H171 TTC"}="1H17 TTC"; $resn_exch{"H191 TTC"}="1H19 TTC"; $resn_exch{"H192 TTC"}="2H19 TTC";
$resn_exch{"H111 TTC"}="1H11 TTC"; $resn_exch{"H112 TTC"}="2H11 TTC"; $resn_exch{" H71 TTC"}="1H7  TTC";
$resn_exch{" H41 TTC"}="1H4  TTC"; $resn_exch{" H31 TTC"}="1H3  TTC"; $resn_exch{"H271 TTC"}="1H27 TTC";
$resn_exch{"H272 TTC"}="2H27 TTC"; $resn_exch{"H291 TTC"}="1H29 TTC"; $resn_exch{"H292 TTC"}="2H29 TTC";
$resn_exch{"H293 TTC"}="3H29 TTC"; $resn_exch{"H301 TTC"}="1H30 TTC"; $resn_exch{"H302 TTC"}="2H30 TTC";
$resn_exch{"H303 TTC"}="3H30 TTC"; $resn_exch{"H261 TTC"}="1H26 TTC"; $resn_exch{"HOP2 TTD"}="2HOP TTD";
$resn_exch{"HOP3 TTD"}="3HOP TTD"; $resn_exch{" H5' TTD"}="1H5* TTD"; $resn_exch{"H5'' TTD"}="2H5* TTD";
$resn_exch{" H2' TTD"}="1H2* TTD"; $resn_exch{"H2'' TTD"}="2H2* TTD"; $resn_exch{" H1' TTD"}=" H1* TTD";
$resn_exch{"H5A1 TTD"}="1H5A TTD"; $resn_exch{"H5A2 TTD"}="2H5A TTD"; $resn_exch{"H5A3 TTD"}="3H5A TTD";
$resn_exch{"H5R1 TTD"}="1H5R TTD"; $resn_exch{"H5R2 TTD"}="2H5R TTD"; $resn_exch{"H2R1 TTD"}="1H2R TTD";
$resn_exch{"H2R2 TTD"}="2H2R TTD"; $resn_exch{" H71 TTD"}="1H5M TTD"; $resn_exch{" H72 TTD"}="2H5M TTD";
$resn_exch{" H73 TTD"}="3H5M TTD"; $resn_exch{" H4' TTD"}=" H4* TTD"; $resn_exch{"HO3' TTD"}="H3T  TTD";
$resn_exch{"H5M1 TTE"}="1H5M TTE"; $resn_exch{"H5M2 TTE"}="2H5M TTE"; $resn_exch{"H5M3 TTE"}="3H5M TTE";
$resn_exch{" H1' TTE"}=" H1* TTE"; $resn_exch{"H2'1 TTE"}="1H2* TTE"; $resn_exch{"H2'2 TTE"}="2H2* TTE";
$resn_exch{" H3' TTE"}=" H3* TTE"; $resn_exch{"HO'3 TTE"}="3HO* TTE"; $resn_exch{" H4' TTE"}=" H4* TTE";
$resn_exch{"H5'1 TTE"}="1H5* TTE"; $resn_exch{"H5'2 TTE"}="2H5* TTE"; $resn_exch{"HOA2 TTE"}="2HOA TTE";
$resn_exch{"HOB2 TTE"}="2HOB TTE"; $resn_exch{"HCC1 TTE"}="1HCC TTE"; $resn_exch{"HCC2 TTE"}="2HCC TTE";
$resn_exch{"HOG2 TTE"}="2HOG TTE"; $resn_exch{"HOG3 TTE"}="3HOG TTE"; $resn_exch{" H61 TTF"}="1H6  TTF";
$resn_exch{" H62 TTF"}="2H6  TTF"; $resn_exch{"H251 TTG"}="1H25 TTG"; $resn_exch{"H252 TTG"}="2H25 TTG";
$resn_exch{"H311 TTG"}="1H31 TTG"; $resn_exch{"H312 TTG"}="2H31 TTG"; $resn_exch{"H313 TTG"}="3H31 TTG";
$resn_exch{"H241 TTG"}="1H24 TTG"; $resn_exch{"H171 TTG"}="1H17 TTG"; $resn_exch{"H22O TTG"}="OH22 TTG";
$resn_exch{"H191 TTG"}="1H19 TTG"; $resn_exch{"H192 TTG"}="2H19 TTG"; $resn_exch{"H111 TTG"}="1H11 TTG";
$resn_exch{"H112 TTG"}="2H11 TTG"; $resn_exch{" H71 TTG"}="1H7  TTG"; $resn_exch{" H41 TTG"}="1H4  TTG";
$resn_exch{" H31 TTG"}="1H3  TTG"; $resn_exch{"H271 TTG"}="1H27 TTG"; $resn_exch{"H272 TTG"}="2H27 TTG";
$resn_exch{"H291 TTG"}="1H29 TTG"; $resn_exch{"H292 TTG"}="2H29 TTG"; $resn_exch{"H293 TTG"}="3H29 TTG";
$resn_exch{"H301 TTG"}="1H30 TTG"; $resn_exch{"H302 TTG"}="2H30 TTG"; $resn_exch{"H303 TTG"}="3H30 TTG";
$resn_exch{"H261 TTG"}="1H26 TTG"; $resn_exch{"H19O TTG"}="OH19 TTG"; $resn_exch{"H201 TTH"}="1H20 TTH";
$resn_exch{"H202 TTH"}="2H20 TTH"; $resn_exch{"H203 TTH"}="3H20 TTH"; $resn_exch{"H191 TTH"}="1H19 TTH";
$resn_exch{"H192 TTH"}="2H19 TTH"; $resn_exch{"H193 TTH"}="3H19 TTH"; $resn_exch{"H161 TTH"}="1H16 TTH";
$resn_exch{"H162 TTH"}="2H16 TTH"; $resn_exch{"H141 TTH"}="1H14 TTH"; $resn_exch{"H142 TTH"}="2H14 TTH";
$resn_exch{"H143 TTH"}="3H14 TTH"; $resn_exch{"H151 TTH"}="1H15 TTH"; $resn_exch{"H152 TTH"}="2H15 TTH";
$resn_exch{"H111 TTH"}="1H11 TTH"; $resn_exch{"H112 TTH"}="2H11 TTH"; $resn_exch{"HC91 TTH"}="1HC9 TTH";
$resn_exch{"HC92 TTH"}="2HC9 TTH"; $resn_exch{"H101 TTH"}="1H10 TTH"; $resn_exch{"H102 TTH"}="2H10 TTH";
$resn_exch{"H103 TTH"}="3H10 TTH"; $resn_exch{"HC61 TTH"}="1HC6 TTH"; $resn_exch{"HC62 TTH"}="2HC6 TTH";
$resn_exch{"HC51 TTH"}="1HC5 TTH"; $resn_exch{"HC52 TTH"}="2HC5 TTH"; $resn_exch{"HC41 TTH"}="1HC4 TTH";
$resn_exch{"HC42 TTH"}="2HC4 TTH"; $resn_exch{"HC43 TTH"}="3HC4 TTH"; $resn_exch{"HC11 TTH"}="1HC1 TTH";
$resn_exch{"HC12 TTH"}="2HC1 TTH"; $resn_exch{"HC13 TTH"}="3HC1 TTH"; $resn_exch{"HAG1 TTL"}="1HAG TTL";
$resn_exch{"HAG2 TTL"}="2HAG TTL"; $resn_exch{"HAH1 TTL"}="1HAH TTL"; $resn_exch{"HAH2 TTL"}="2HAH TTL";
$resn_exch{"HAA1 TTL"}="1HAA TTL"; $resn_exch{"HAA2 TTL"}="2HAA TTL"; $resn_exch{"HOP2 TTM"}="2HOP TTM";
$resn_exch{" H5' TTM"}="1H5* TTM"; $resn_exch{"H5'' TTM"}="2H5* TTM"; $resn_exch{" H4' TTM"}=" H4* TTM";
$resn_exch{" H1' TTM"}=" H1* TTM"; $resn_exch{" H71 TTM"}="1H5M TTM"; $resn_exch{" H72 TTM"}="2H5M TTM";
$resn_exch{" H73 TTM"}="3H5M TTM"; $resn_exch{" H2' TTM"}="1H2* TTM"; $resn_exch{"H2'' TTM"}="2H2* TTM";
$resn_exch{" H3' TTM"}=" H3* TTM"; $resn_exch{"HO3' TTM"}="H3T  TTM"; $resn_exch{"HOP3 TTM"}="3HOP TTM";
$resn_exch{"HOA2 TTP"}="2HOA TTP"; $resn_exch{"HOB2 TTP"}="2HOB TTP"; $resn_exch{"HOG2 TTP"}="2HOG TTP";
$resn_exch{"HOG3 TTP"}="3HOG TTP"; $resn_exch{"H5'1 TTP"}="1H5* TTP"; $resn_exch{"H5'2 TTP"}="2H5* TTP";
$resn_exch{" H4' TTP"}=" H4* TTP"; $resn_exch{" H3' TTP"}=" H3* TTP"; $resn_exch{"HO3' TTP"}="*HO3 TTP";
$resn_exch{"H2'1 TTP"}="1H2* TTP"; $resn_exch{"H2'2 TTP"}="2H2* TTP"; $resn_exch{" H1' TTP"}=" H1* TTP";
$resn_exch{"HM51 TTP"}="1HM5 TTP"; $resn_exch{"HM52 TTP"}="2HM5 TTP"; $resn_exch{"HM53 TTP"}="3HM5 TTP";
$resn_exch{" H   TTQ"}="1HN  TTQ"; $resn_exch{" H2  TTQ"}="2HN  TTQ"; $resn_exch{" HB2 TTQ"}="1HB  TTQ";
$resn_exch{" HB3 TTQ"}="2HB  TTQ"; $resn_exch{" HE1 TTQ"}="HNE1 TTQ"; $resn_exch{"HN21 TTQ"}="1HN2 TTQ";
$resn_exch{"HN22 TTQ"}="2HN2 TTQ"; $resn_exch{" HXT TTQ"}="HOXT TTQ"; $resn_exch{" H2  TTS"}=" HN2 TTS";
$resn_exch{" HB2 TTS"}="1HB  TTS"; $resn_exch{" HB3 TTS"}="2HB  TTS"; $resn_exch{" H3' TTS"}=" H3* TTS";
$resn_exch{" H2' TTS"}=" H2* TTS"; $resn_exch{" H6' TTS"}=" H6* TTS"; $resn_exch{" H5' TTS"}=" H5* TTS";
$resn_exch{" H21 TTS"}="1H2  TTS"; $resn_exch{" H22 TTS"}="2H2  TTS"; $resn_exch{" H11 TTS"}="1H1  TTS";
$resn_exch{" H12 TTS"}="2H1  TTS"; $resn_exch{" H71 TTX"}="1H7  TTX"; $resn_exch{" H72 TTX"}="2H7  TTX";
$resn_exch{" H73 TTX"}="3H7  TTX"; $resn_exch{"H161 TTX"}="1H16 TTX"; $resn_exch{"H221 TTX"}="1H22 TTX";
$resn_exch{"H211 TTX"}="1H21 TTX"; $resn_exch{"H201 TTX"}="1H20 TTX"; $resn_exch{"H191 TTX"}="1H19 TTX";
$resn_exch{"H181 TTX"}="1H18 TTX"; $resn_exch{" H51 TTX"}="1H5  TTX"; $resn_exch{" H41 TTX"}="1H4  TTX";
$resn_exch{" H42 TTX"}="2H4  TTX"; $resn_exch{" H21 TTX"}="1H2  TTX"; $resn_exch{" H11 TTX"}="1H1  TTX";
$resn_exch{" H12 TTX"}="2H1  TTX"; $resn_exch{" H13 TTX"}="3H1  TTX"; $resn_exch{" H31 TTX"}="1H3  TTX";
$resn_exch{" H32 TTX"}="2H3  TTX"; $resn_exch{" H33 TTX"}="3H3  TTX"; $resn_exch{"HN21 TTX"}="1HN2 TTX";
$resn_exch{" H91 TTX"}="1H9  TTX"; $resn_exch{"H101 TTX"}="1H10 TTX"; $resn_exch{"H102 TTX"}="2H10 TTX";
$resn_exch{"H103 TTX"}="3H10 TTX"; $resn_exch{"H111 TTX"}="1H11 TTX"; $resn_exch{"H112 TTX"}="2H11 TTX";
$resn_exch{"H113 TTX"}="3H11 TTX"; $resn_exch{"H131 TTX"}="1H13 TTX"; $resn_exch{"H132 TTX"}="2H13 TTX";
$resn_exch{"HN41 TTX"}="1HN4 TTX"; $resn_exch{"H7C1 TTZ"}="1H7C TTZ"; $resn_exch{"H7C2 TTZ"}="2H7C TTZ";
$resn_exch{"H6'1 TTZ"}="1H6' TTZ"; $resn_exch{"H6'2 TTZ"}="2H6' TTZ"; $resn_exch{" H11 TUD"}="1H1  TUD";
$resn_exch{" H12 TUD"}="2H1  TUD"; $resn_exch{" H21 TUD"}="1H2  TUD"; $resn_exch{" H22 TUD"}="2H2  TUD";
$resn_exch{" H41 TUD"}="1H4  TUD"; $resn_exch{" H42 TUD"}="2H4  TUD"; $resn_exch{" H61 TUD"}="1H6  TUD";
$resn_exch{" H62 TUD"}="2H6  TUD"; $resn_exch{"H111 TUD"}="1H11 TUD"; $resn_exch{"H112 TUD"}="2H11 TUD";
$resn_exch{"H121 TUD"}="1H12 TUD"; $resn_exch{"H122 TUD"}="2H12 TUD"; $resn_exch{"H151 TUD"}="1H15 TUD";
$resn_exch{"H152 TUD"}="2H15 TUD"; $resn_exch{"H161 TUD"}="1H16 TUD"; $resn_exch{"H162 TUD"}="2H16 TUD";
$resn_exch{"H181 TUD"}="1H18 TUD"; $resn_exch{"H182 TUD"}="2H18 TUD"; $resn_exch{"H183 TUD"}="3H18 TUD";
$resn_exch{"H191 TUD"}="1H19 TUD"; $resn_exch{"H192 TUD"}="2H19 TUD"; $resn_exch{"H193 TUD"}="3H19 TUD";
$resn_exch{"H211 TUD"}="1H21 TUD"; $resn_exch{"H212 TUD"}="2H21 TUD"; $resn_exch{"H213 TUD"}="3H21 TUD";
$resn_exch{"H221 TUD"}="1H22 TUD"; $resn_exch{"H222 TUD"}="2H22 TUD"; $resn_exch{"H231 TUD"}="1H23 TUD";
$resn_exch{"H232 TUD"}="2H23 TUD"; $resn_exch{"H251 TUD"}="1H25 TUD"; $resn_exch{"H252 TUD"}="2H25 TUD";
$resn_exch{"H261 TUD"}="1H26 TUD"; $resn_exch{"H262 TUD"}="2H26 TUD"; $resn_exch{" H81 TUI"}="1H8  TUI";
$resn_exch{" H82 TUI"}="2H8  TUI"; $resn_exch{"H171 TUI"}="1H17 TUI"; $resn_exch{"H172 TUI"}="2H17 TUI";
$resn_exch{" HAE TUO"}=" H1  TUO"; $resn_exch{" HAA TUO"}=" H2  TUO"; $resn_exch{" HAB TUO"}=" H3  TUO";
$resn_exch{" HAC TUO"}=" H4  TUO"; $resn_exch{" HAG TUO"}=" H5  TUO"; $resn_exch{" HAL TUO"}=" H6  TUO";
$resn_exch{"HNAI TUO"}="H7   TUO"; $resn_exch{"HNAA TUO"}="H8   TUO"; $resn_exch{" HAT TUO"}=" H9  TUO";
$resn_exch{" HAU TUO"}=" H10 TUO"; $resn_exch{"HNAR TUO"}="H11  TUO"; $resn_exch{"HNAW TUO"}="H12  TUO";
$resn_exch{"H1G1 TUX"}="1H1G TUX"; $resn_exch{"H1G2 TUX"}="2H1G TUX"; $resn_exch{"H2A2 TUX"}="2H2A TUX";
$resn_exch{"H2A3 TUX"}="3H2A TUX"; $resn_exch{"H3A3 TUX"}="3H3A TUX"; $resn_exch{"H3A2 TUX"}="2H3A TUX";
$resn_exch{"H4A2 TUX"}="2H4A TUX"; $resn_exch{"H4A3 TUX"}="3H4A TUX"; $resn_exch{"H5A3 TUX"}="3H5A TUX";
$resn_exch{"H5A2 TUX"}="2H5A TUX"; $resn_exch{"H6A2 TUX"}="2H6A TUX"; $resn_exch{"H6A3 TUX"}="3H6A TUX";
$resn_exch{"H7A3 TUX"}="3H7A TUX"; $resn_exch{"H7A2 TUX"}="2H7A TUX"; $resn_exch{"H8A2 TUX"}="2H8A TUX";
$resn_exch{"H8A3 TUX"}="3H8A TUX"; $resn_exch{"H9A3 TUX"}="3H9A TUX"; $resn_exch{"H9A2 TUX"}="2H9A TUX";
$resn_exch{"HAA2 TUX"}="2HAA TUX"; $resn_exch{"HAA3 TUX"}="3HAA TUX"; $resn_exch{"HBA3 TUX"}="3HBA TUX";
$resn_exch{"HBA2 TUX"}="2HBA TUX"; $resn_exch{"HCA2 TUX"}="2HCA TUX"; $resn_exch{"HCA3 TUX"}="3HCA TUX";
$resn_exch{"HDA3 TUX"}="3HDA TUX"; $resn_exch{"HDA2 TUX"}="2HDA TUX"; $resn_exch{"HEA1 TUX"}="1HEA TUX";
$resn_exch{"HEA2 TUX"}="2HEA TUX"; $resn_exch{"HEA3 TUX"}="3HEA TUX"; $resn_exch{"HZH2 TUX"}="2HZH TUX";
$resn_exch{"HZH3 TUX"}="3HZH TUX"; $resn_exch{"HNH1 TUX"}="1HNH TUX"; $resn_exch{"HOH1 TUX"}="1HOH TUX";
$resn_exch{"HOG4 TUX"}="4HOG TUX"; $resn_exch{"H6G2 TUX"}="2H6G TUX"; $resn_exch{"H6G3 TUX"}="3H6G TUX";
$resn_exch{"HOG6 TUX"}="6HOG TUX"; $resn_exch{"HAJ1 TWN"}="1HAJ TWN"; $resn_exch{"HAJ2 TWN"}="2HAJ TWN";
$resn_exch{"HAM1 TWN"}="1HAM TWN"; $resn_exch{"HAM2 TWN"}="2HAM TWN"; $resn_exch{"HAO1 TWN"}="1HAO TWN";
$resn_exch{"HAO2 TWN"}="2HAO TWN"; $resn_exch{"HAO3 TWN"}="3HAO TWN"; $resn_exch{"HAD1 TWN"}="1HAD TWN";
$resn_exch{"HAD2 TWN"}="2HAD TWN"; $resn_exch{"HAN1 TWN"}="1HAN TWN"; $resn_exch{"HAN2 TWN"}="2HAN TWN";
$resn_exch{"HAN3 TWN"}="3HAN TWN"; $resn_exch{"HAK1 TWN"}="1HAK TWN"; $resn_exch{"HAK2 TWN"}="2HAK TWN";
$resn_exch{"HAK3 TWN"}="3HAK TWN"; $resn_exch{"HAL1 TWN"}="1HAL TWN"; $resn_exch{"HAL2 TWN"}="2HAL TWN";
$resn_exch{"HAP1 TWN"}="1HAP TWN"; $resn_exch{"HAP2 TWN"}="2HAP TWN"; $resn_exch{"HAW1 TWN"}="1HAW TWN";
$resn_exch{"HAW2 TWN"}="2HAW TWN"; $resn_exch{"HAS1 TWN"}="1HAS TWN"; $resn_exch{"HAS2 TWN"}="2HAS TWN";
$resn_exch{"HAX1 TWN"}="1HAX TWN"; $resn_exch{"HAX2 TWN"}="2HAX TWN"; $resn_exch{"HAT1 TWN"}="1HAT TWN";
$resn_exch{"HAT2 TWN"}="2HAT TWN"; $resn_exch{"HAU1 TWN"}="1HAU TWN"; $resn_exch{"HAU2 TWN"}="2HAU TWN";
$resn_exch{"HAV1 TWN"}="1HAV TWN"; $resn_exch{"HAV2 TWN"}="2HAV TWN"; $resn_exch{"HAR1 TWN"}="1HAR TWN";
$resn_exch{"HAR2 TWN"}="2HAR TWN"; $resn_exch{"HAQ1 TWN"}="1HAQ TWN"; $resn_exch{"HAQ2 TWN"}="2HAQ TWN";
$resn_exch{"HAY1 TWN"}="1HAY TWN"; $resn_exch{"HAY2 TWN"}="2HAY TWN"; $resn_exch{"HAY3 TWN"}="3HAY TWN";
$resn_exch{"H1C1 TWT"}="1H1C TWT"; $resn_exch{"H1C2 TWT"}="2H1C TWT"; $resn_exch{"H1C3 TWT"}="3H1C TWT";
$resn_exch{"H2C1 TWT"}="1H2C TWT"; $resn_exch{"H2C2 TWT"}="2H2C TWT"; $resn_exch{"H3C1 TWT"}="1H3C TWT";
$resn_exch{"H3C2 TWT"}="2H3C TWT"; $resn_exch{"H4C1 TWT"}="1H4C TWT"; $resn_exch{"H4C2 TWT"}="2H4C TWT";
$resn_exch{"H5C1 TWT"}="1H5C TWT"; $resn_exch{"H5C2 TWT"}="2H5C TWT"; $resn_exch{"H6C1 TWT"}="1H6C TWT";
$resn_exch{"H6C2 TWT"}="2H6C TWT"; $resn_exch{"H7C1 TWT"}="1H7C TWT"; $resn_exch{"H7C2 TWT"}="2H7C TWT";
$resn_exch{"H8C1 TWT"}="1H8C TWT"; $resn_exch{"H8C2 TWT"}="2H8C TWT"; $resn_exch{"H9C1 TWT"}="1H9C TWT";
$resn_exch{"H9C2 TWT"}="2H9C TWT"; $resn_exch{"H101 TWT"}="1H10 TWT"; $resn_exch{"H102 TWT"}="2H10 TWT";
$resn_exch{"H111 TWT"}="1H11 TWT"; $resn_exch{"H112 TWT"}="2H11 TWT"; $resn_exch{"H121 TWT"}="1H12 TWT";
$resn_exch{"H122 TWT"}="2H12 TWT"; $resn_exch{"H131 TWT"}="1H13 TWT"; $resn_exch{"H132 TWT"}="2H13 TWT";
$resn_exch{"H141 TWT"}="1H14 TWT"; $resn_exch{"H142 TWT"}="2H14 TWT"; $resn_exch{"H151 TWT"}="1H15 TWT";
$resn_exch{"H152 TWT"}="2H15 TWT"; $resn_exch{"H161 TWT"}="1H16 TWT"; $resn_exch{"H162 TWT"}="2H16 TWT";
$resn_exch{"H171 TWT"}="1H17 TWT"; $resn_exch{"H172 TWT"}="2H17 TWT"; $resn_exch{"H181 TWT"}="1H18 TWT";
$resn_exch{"H182 TWT"}="2H18 TWT"; $resn_exch{"H191 TWT"}="1H19 TWT"; $resn_exch{"H192 TWT"}="2H19 TWT";
$resn_exch{"H201 TWT"}="1H20 TWT"; $resn_exch{"H202 TWT"}="2H20 TWT"; $resn_exch{"H211 TWT"}="1H21 TWT";
$resn_exch{"H212 TWT"}="2H21 TWT"; $resn_exch{"H221 TWT"}="1H22 TWT"; $resn_exch{"H222 TWT"}="2H22 TWT";
$resn_exch{"H223 TWT"}="3H22 TWT"; $resn_exch{" H41 TX4"}="1H4  TX4"; $resn_exch{" H42 TX4"}="2H4  TX4";
$resn_exch{" H21 TX5"}="1H2  TX5"; $resn_exch{" H22 TX5"}="2H2  TX5"; $resn_exch{" H41 TX5"}="1H4  TX5";
$resn_exch{" H42 TX5"}="2H4  TX5"; $resn_exch{" H91 TX5"}="1H9  TX5"; $resn_exch{" H92 TX5"}="2H9  TX5";
$resn_exch{"H111 TX5"}="1H11 TX5"; $resn_exch{"H112 TX5"}="2H11 TX5"; $resn_exch{"H121 TX5"}="1H12 TX5";
$resn_exch{"H122 TX5"}="2H12 TX5"; $resn_exch{"H151 TX5"}="1H15 TX5"; $resn_exch{"H152 TX5"}="2H15 TX5";
$resn_exch{"H161 TX5"}="1H16 TX5"; $resn_exch{"H162 TX5"}="2H16 TX5"; $resn_exch{"H181 TX5"}="1H18 TX5";
$resn_exch{"H182 TX5"}="2H18 TX5"; $resn_exch{"H183 TX5"}="3H18 TX5"; $resn_exch{"H191 TX5"}="1H19 TX5";
$resn_exch{"H192 TX5"}="2H19 TX5"; $resn_exch{"H211 TX5"}="1H21 TX5"; $resn_exch{"H212 TX5"}="2H21 TX5";
$resn_exch{"H213 TX5"}="3H21 TX5"; $resn_exch{"H221 TX5"}="1H22 TX5"; $resn_exch{"H222 TX5"}="2H22 TX5";
$resn_exch{"H271 TX5"}="1H27 TX5"; $resn_exch{"H272 TX5"}="2H27 TX5"; $resn_exch{"H273 TX5"}="3H27 TX5";
$resn_exch{"H261 TX5"}="1H26 TX5"; $resn_exch{"H262 TX5"}="2H26 TX5"; $resn_exch{"H263 TX5"}="3H26 TX5";
$resn_exch{"H7N1 TXD"}="1H7N TXD"; $resn_exch{"H7N2 TXD"}="2H7N TXD"; $resn_exch{"H2N1 TXD"}="1H2N TXD";
$resn_exch{"H2N2 TXD"}="2H2N TXD"; $resn_exch{"H6N1 TXD"}="1H6N TXD"; $resn_exch{"H6N2 TXD"}="2H6N TXD";
$resn_exch{"H1'A TXD"}="H1*  TXD"; $resn_exch{"H2'A TXD"}="H2*  TXD"; $resn_exch{"HO2A TXD"}="HO2* TXD";
$resn_exch{"H3'A TXD"}="H3*  TXD"; $resn_exch{"HO3A TXD"}="HO3* TXD"; $resn_exch{"H4'A TXD"}="H4*  TXD";
$resn_exch{"H5'1 TXD"}="1H5* TXD"; $resn_exch{"H5'2 TXD"}="2H5* TXD"; $resn_exch{"H5'A TXD"}="1H5' TXD";
$resn_exch{"H5'B TXD"}="2H5' TXD"; $resn_exch{"H6A1 TXD"}="1H6A TXD"; $resn_exch{"H6A2 TXD"}="2H6A TXD";
$resn_exch{"H131 TXF"}="1H13 TXF"; $resn_exch{"H132 TXF"}="2H13 TXF"; $resn_exch{"H141 TXF"}="1H14 TXF";
$resn_exch{"H142 TXF"}="2H14 TXF"; $resn_exch{"H151 TXF"}="1H15 TXF"; $resn_exch{"H152 TXF"}="2H15 TXF";
$resn_exch{" H31 TXF"}="1H3  TXF"; $resn_exch{" H32 TXF"}="2H3  TXF"; $resn_exch{"H21A TXF"}="1H2  TXF";
$resn_exch{"H22A TXF"}="2H2  TXF"; $resn_exch{"H311 TXF"}="1H31 TXF"; $resn_exch{"H312 TXF"}="2H31 TXF";
$resn_exch{"H313 TXF"}="3H31 TXF"; $resn_exch{"H301 TXF"}="1H30 TXF"; $resn_exch{"H302 TXF"}="2H30 TXF";
$resn_exch{"H303 TXF"}="3H30 TXF"; $resn_exch{" H61 TXL"}="1H6  TXL"; $resn_exch{" H62 TXL"}="2H6  TXL";
$resn_exch{"H141 TXL"}="1H14 TXL"; $resn_exch{"H142 TXL"}="2H14 TXL"; $resn_exch{"H161 TXL"}="1H16 TXL";
$resn_exch{"H162 TXL"}="2H16 TXL"; $resn_exch{"H163 TXL"}="3H16 TXL"; $resn_exch{"H171 TXL"}="1H17 TXL";
$resn_exch{"H172 TXL"}="2H17 TXL"; $resn_exch{"H173 TXL"}="3H17 TXL"; $resn_exch{"H181 TXL"}="1H18 TXL";
$resn_exch{"H182 TXL"}="2H18 TXL"; $resn_exch{"H183 TXL"}="3H18 TXL"; $resn_exch{"H191 TXL"}="1H19 TXL";
$resn_exch{"H192 TXL"}="2H19 TXL"; $resn_exch{"H193 TXL"}="3H19 TXL"; $resn_exch{"H201 TXL"}="1H20 TXL";
$resn_exch{"H202 TXL"}="2H20 TXL"; $resn_exch{"H321 TXL"}="1H32 TXL"; $resn_exch{"H322 TXL"}="2H32 TXL";
$resn_exch{"H323 TXL"}="3H32 TXL"; $resn_exch{"H331 TXL"}="1H33 TXL"; $resn_exch{"H332 TXL"}="2H33 TXL";
$resn_exch{"H333 TXL"}="3H33 TXL"; $resn_exch{"H341 TXL"}="1H34 TXL"; $resn_exch{"H342 TXL"}="2H34 TXL";
$resn_exch{"H343 TXL"}="3H34 TXL"; $resn_exch{"H431 TXL"}="1H43 TXL"; $resn_exch{"H432 TXL"}="2H43 TXL";
$resn_exch{"H433 TXL"}="3H43 TXL"; $resn_exch{"H5'1 TXP"}="1H5* TXP"; $resn_exch{"H5'2 TXP"}="2H5* TXP";
$resn_exch{"H4'A TXP"}="H4*  TXP"; $resn_exch{"H3'A TXP"}="H3*  TXP"; $resn_exch{"HO3A TXP"}="HO3* TXP";
$resn_exch{"H2'A TXP"}="H2*  TXP"; $resn_exch{"HO2' TXP"}="HO2* TXP"; $resn_exch{"H1'A TXP"}="H1*  TXP";
$resn_exch{"H6N1 TXP"}="1H6N TXP"; $resn_exch{"H6N2 TXP"}="2H6N TXP"; $resn_exch{"H7N1 TXP"}="1H7N TXP";
$resn_exch{"H7N2 TXP"}="2H7N TXP"; $resn_exch{"H2N1 TXP"}="1H2N TXP"; $resn_exch{"H2N2 TXP"}="2H2N TXP";
$resn_exch{"H5'A TXP"}="1H5' TXP"; $resn_exch{"H5'B TXP"}="2H5' TXP"; $resn_exch{"H6A1 TXP"}="1H6A TXP";
$resn_exch{"H6A2 TXP"}="2H6A TXP"; $resn_exch{" HN1 TY2"}="1HN  TY2"; $resn_exch{" HA2 TY2"}="2HA  TY2";
$resn_exch{" HA1 TY2"}="1HA  TY2"; $resn_exch{" HB2 TY2"}="2HB  TY2"; $resn_exch{"HE21 TY2"}="1HE2 TY2";
$resn_exch{"HE22 TY2"}="2HE2 TY2"; $resn_exch{" HN1 TY3"}="1HN  TY3"; $resn_exch{" HN2 TY3"}="2HN  TY3";
$resn_exch{"HBC1 TY3"}="1HBC TY3"; $resn_exch{"HBC2 TY3"}="2HBC TY3"; $resn_exch{"H5'1 TYA"}="1H5* TYA";
$resn_exch{"H5'2 TYA"}="2H5* TYA"; $resn_exch{" H4' TYA"}=" H4* TYA"; $resn_exch{" H3' TYA"}=" H3* TYA";
$resn_exch{"HO'3 TYA"}="3HO* TYA"; $resn_exch{" H2' TYA"}=" H2* TYA"; $resn_exch{"HO'2 TYA"}="2HO* TYA";
$resn_exch{" H1' TYA"}=" H1* TYA"; $resn_exch{" H61 TYA"}="1H6  TYA"; $resn_exch{" H62 TYA"}="2H6  TYA";
$resn_exch{" HN1 TYA"}="1HN  TYA"; $resn_exch{" HN2 TYA"}="2HN  TYA"; $resn_exch{" H1  TYA"}="1H   TYA";
$resn_exch{" H2A TYA"}="2H   TYA"; $resn_exch{" HB1 TYA"}="1HB  TYA"; $resn_exch{" HB2 TYA"}="2HB  TYA";
$resn_exch{" H2  TYB"}=" HN2 TYB"; $resn_exch{" HB2 TYB"}="1HB  TYB"; $resn_exch{" HB3 TYB"}="2HB  TYB";
$resn_exch{"HT21 TYC"}="1HT2 TYC"; $resn_exch{"HT22 TYC"}="2HT2 TYC"; $resn_exch{"HOA2 TYD"}="2HOA TYD";
$resn_exch{"HOB2 TYD"}="2HOB TYD"; $resn_exch{"HOB3 TYD"}="3HOB TYD"; $resn_exch{"H5'1 TYD"}="1H5* TYD";
$resn_exch{"H5'2 TYD"}="2H5* TYD"; $resn_exch{" H4' TYD"}=" H4* TYD"; $resn_exch{" H3' TYD"}=" H3* TYD";
$resn_exch{"HO3' TYD"}="*HO3 TYD"; $resn_exch{"H2'1 TYD"}="1H2* TYD"; $resn_exch{"H2'2 TYD"}="2H2* TYD";
$resn_exch{" H1' TYD"}=" H1* TYD"; $resn_exch{" H51 TYD"}="1H5  TYD"; $resn_exch{" H52 TYD"}="2H5  TYD";
$resn_exch{" H53 TYD"}="3H5  TYD"; $resn_exch{" H   TYI"}="1HN  TYI"; $resn_exch{" H2  TYI"}="2HN  TYI";
$resn_exch{" HB2 TYI"}="1HB  TYI"; $resn_exch{" HB3 TYI"}="2HB  TYI"; $resn_exch{" HH  TYI"}=" HOH TYI";
$resn_exch{"H191 TYK"}="1H19 TYK"; $resn_exch{"H192 TYK"}="2H19 TYK"; $resn_exch{" H21 TYK"}="1H2  TYK";
$resn_exch{" H22 TYK"}="2H2  TYK"; $resn_exch{" H71 TYK"}="1H7  TYK"; $resn_exch{" H72 TYK"}="2H7  TYK";
$resn_exch{"H161 TYK"}="1H16 TYK"; $resn_exch{"H162 TYK"}="2H16 TYK"; $resn_exch{"H171 TYK"}="1H17 TYK";
$resn_exch{"H172 TYK"}="2H17 TYK"; $resn_exch{"H173 TYK"}="3H17 TYK"; $resn_exch{"H181 TYK"}="1H18 TYK";
$resn_exch{"H182 TYK"}="2H18 TYK"; $resn_exch{"H183 TYK"}="3H18 TYK"; $resn_exch{"H211 TYK"}="1H21 TYK";
$resn_exch{"H212 TYK"}="2H21 TYK"; $resn_exch{"H213 TYK"}="3H21 TYK"; $resn_exch{"H221 TYK"}="1H22 TYK";
$resn_exch{"H222 TYK"}="2H22 TYK"; $resn_exch{"H223 TYK"}="3H22 TYK"; $resn_exch{"H231 TYK"}="1H23 TYK";
$resn_exch{"H232 TYK"}="2H23 TYK"; $resn_exch{"H6A1 TYK"}="1H6A TYK"; $resn_exch{"H6A2 TYK"}="2H6A TYK";
$resn_exch{"H6A3 TYK"}="3H6A TYK"; $resn_exch{"H7A1 TYK"}="1H7A TYK"; $resn_exch{"H7A2 TYK"}="2H7A TYK";
$resn_exch{"H7A3 TYK"}="3H7A TYK"; $resn_exch{"H8A1 TYK"}="1H8A TYK"; $resn_exch{"H8A2 TYK"}="2H8A TYK";
$resn_exch{"H8A3 TYK"}="3H8A TYK"; $resn_exch{"H2B1 TYK"}="1H2B TYK"; $resn_exch{"H2B2 TYK"}="2H2B TYK";
$resn_exch{"H6B1 TYK"}="1H6B TYK"; $resn_exch{"H6B2 TYK"}="2H6B TYK"; $resn_exch{"H6B3 TYK"}="3H6B TYK";
$resn_exch{"H7B1 TYK"}="1H7B TYK"; $resn_exch{"H7B2 TYK"}="2H7B TYK"; $resn_exch{"H7B3 TYK"}="3H7B TYK";
$resn_exch{"H6C1 TYK"}="1H6C TYK"; $resn_exch{"H6C2 TYK"}="2H6C TYK"; $resn_exch{"H6C3 TYK"}="3H6C TYK";
$resn_exch{"H7C1 TYK"}="1H7C TYK"; $resn_exch{"H7C2 TYK"}="2H7C TYK"; $resn_exch{"H7C3 TYK"}="3H7C TYK";
$resn_exch{"H8C1 TYK"}="1H8C TYK"; $resn_exch{"H8C2 TYK"}="2H8C TYK"; $resn_exch{"H8C3 TYK"}="3H8C TYK";
$resn_exch{" HM1 TYL"}="1HM  TYL"; $resn_exch{" HM2 TYL"}="2HM  TYL"; $resn_exch{" HM3 TYL"}="3HM  TYL";
$resn_exch{"H1PO TYM"}="OH1P TYM"; $resn_exch{"H5'1 TYM"}="1H5' TYM"; $resn_exch{"H5'2 TYM"}="2H5' TYM";
$resn_exch{"H4'C TYM"}="CH4' TYM"; $resn_exch{"H1'C TYM"}="CH1' TYM"; $resn_exch{"HN61 TYM"}="1HN6 TYM";
$resn_exch{"HN62 TYM"}="2HN6 TYM"; $resn_exch{"H2'C TYM"}="CH2' TYM"; $resn_exch{"H2'O TYM"}="OH2' TYM";
$resn_exch{"H3'C TYM"}="CH3' TYM"; $resn_exch{"H3'O TYM"}="OH3' TYM"; $resn_exch{"HH31 TYM"}="1HH3 TYM";
$resn_exch{"HH32 TYM"}="2HH3 TYM"; $resn_exch{"HCB1 TYM"}="1HCB TYM"; $resn_exch{"HCB2 TYM"}="2HCB TYM";
$resn_exch{"HE3C TYM"}="CHE3 TYM"; $resn_exch{"HD1C TYM"}="CHD1 TYM"; $resn_exch{"HE1N TYM"}="NHE1 TYM";
$resn_exch{"HZ2C TYM"}="CHZ2 TYM"; $resn_exch{"HZ3C TYM"}="CHZ3 TYM"; $resn_exch{"HH2C TYM"}="CHH2 TYM";
$resn_exch{" H   TYN"}="1HN  TYN"; $resn_exch{" H2  TYN"}="2HN  TYN"; $resn_exch{" HB2 TYN"}="1HB  TYN";
$resn_exch{" HB3 TYN"}="2HB  TYN"; $resn_exch{"HN31 TYN"}="1HN3 TYN"; $resn_exch{"HN32 TYN"}="2HN3 TYN";
$resn_exch{" HN1 TYO"}="1HN  TYO"; $resn_exch{" HN2 TYO"}="2HN  TYO"; $resn_exch{" HB1 TYO"}="1HB  TYO";
$resn_exch{" HB2 TYO"}="2HB  TYO"; $resn_exch{"HD21 TYO"}="1HD2 TYO"; $resn_exch{"HBC1 TYP"}="1HBC TYP";
$resn_exch{"HBC2 TYP"}="2HBC TYP"; $resn_exch{"HBP1 TYP"}="1HBP TYP"; $resn_exch{"HBP2 TYP"}="2HBP TYP";
$resn_exch{"HGP1 TYP"}="1HGP TYP"; $resn_exch{"HGP2 TYP"}="2HGP TYP"; $resn_exch{"HDP1 TYP"}="1HDP TYP";
$resn_exch{"HDP2 TYP"}="2HDP TYP"; $resn_exch{" H2  TYQ"}=" HN2 TYQ"; $resn_exch{" HB2 TYQ"}="1HB  TYQ";
$resn_exch{" HB3 TYQ"}="2HB  TYQ"; $resn_exch{" HZ  TYQ"}=" HOZ TYQ"; $resn_exch{" H2  TYR"}=" HN2 TYR";
$resn_exch{" HB2 TYR"}="1HB  TYR"; $resn_exch{" HB3 TYR"}="2HB  TYR"; $resn_exch{" H   TYS"}="1HN  TYS";
$resn_exch{" HN2 TYS"}="2HN  TYS"; $resn_exch{" HB2 TYS"}="1HB  TYS"; $resn_exch{" HB3 TYS"}="2HB  TYS";
$resn_exch{" H5' TYU"}=" H5* TYU"; $resn_exch{"H5'1 TYU"}="1H5* TYU"; $resn_exch{"H5'' TYU"}="2H5* TYU";
$resn_exch{" H4' TYU"}=" H4* TYU"; $resn_exch{" H1' TYU"}=" H1* TYU"; $resn_exch{" H61 TYU"}="1H6  TYU";
$resn_exch{" H62 TYU"}="2H6  TYU"; $resn_exch{" H51 TYU"}="1H5  TYU"; $resn_exch{" H52 TYU"}="2H5  TYU";
$resn_exch{" H2' TYU"}=" H2* TYU"; $resn_exch{" H3' TYU"}=" H3* TYU"; $resn_exch{" H31 TYV"}="1H3  TYV";
$resn_exch{" H32 TYV"}="2H3  TYV"; $resn_exch{" H61 TYV"}="1H6  TYV"; $resn_exch{" H62 TYV"}="2H6  TYV";
$resn_exch{" H63 TYV"}="3H6  TYV"; $resn_exch{"HN1A TYX"}="1HN  TYX"; $resn_exch{" HN2 TYX"}="2HN  TYX";
$resn_exch{" HB1 TYX"}="1HB  TYX"; $resn_exch{" HB2 TYX"}="2HB  TYX"; $resn_exch{" H81 TYX"}="1H8  TYX";
$resn_exch{" H82 TYX"}="2H8  TYX"; $resn_exch{" H2  TYY"}=" HN2 TYY"; $resn_exch{" HB2 TYY"}="1HB  TYY";
$resn_exch{" HB3 TYY"}="2HB  TYY"; $resn_exch{"H9C1 TYZ"}="1H9C TYZ"; $resn_exch{"H9C2 TYZ"}="2H9C TYZ";
$resn_exch{"H9C3 TYZ"}="3H9C TYZ"; $resn_exch{"HN31 TZ4"}="1HN3 TZ4"; $resn_exch{"HN32 TZ4"}="2HN3 TZ4";
$resn_exch{"HN11 TZ4"}="1HN1 TZ4"; $resn_exch{"HN12 TZ4"}="2HN1 TZ4"; $resn_exch{"H201 TZ4"}="1H20 TZ4";
$resn_exch{"H202 TZ4"}="2H20 TZ4"; $resn_exch{"H211 TZ4"}="1H21 TZ4"; $resn_exch{"H212 TZ4"}="2H21 TZ4";
$resn_exch{"H221 TZ4"}="1H22 TZ4"; $resn_exch{"H222 TZ4"}="2H22 TZ4"; $resn_exch{"H231 TZ4"}="1H23 TZ4";
$resn_exch{"H232 TZ4"}="2H23 TZ4"; $resn_exch{"H241 TZ4"}="1H24 TZ4"; $resn_exch{"H242 TZ4"}="2H24 TZ4";
$resn_exch{"H251 TZ4"}="1H25 TZ4"; $resn_exch{"H252 TZ4"}="2H25 TZ4"; $resn_exch{"H281 TZ4"}="1H28 TZ4";
$resn_exch{"H282 TZ4"}="2H28 TZ4"; $resn_exch{"H291 TZ4"}="1H29 TZ4"; $resn_exch{"H292 TZ4"}="2H29 TZ4";
$resn_exch{"H381 TZ4"}="1H38 TZ4"; $resn_exch{"H382 TZ4"}="2H38 TZ4"; $resn_exch{"H401 TZ4"}="1H40 TZ4";
$resn_exch{"H402 TZ4"}="2H40 TZ4"; $resn_exch{"H411 TZ4"}="1H41 TZ4"; $resn_exch{"H412 TZ4"}="2H41 TZ4";
$resn_exch{"H421 TZ4"}="1H42 TZ4"; $resn_exch{"H422 TZ4"}="2H42 TZ4"; $resn_exch{"HN21 TZ5"}="1HN2 TZ5";
$resn_exch{"HN22 TZ5"}="2HN2 TZ5"; $resn_exch{"HN11 TZ5"}="1HN1 TZ5"; $resn_exch{"HN12 TZ5"}="2HN1 TZ5";
$resn_exch{"H201 TZ5"}="1H20 TZ5"; $resn_exch{"H202 TZ5"}="2H20 TZ5"; $resn_exch{"H211 TZ5"}="1H21 TZ5";
$resn_exch{"H212 TZ5"}="2H21 TZ5"; $resn_exch{"H221 TZ5"}="1H22 TZ5"; $resn_exch{"H222 TZ5"}="2H22 TZ5";
$resn_exch{"H231 TZ5"}="1H23 TZ5"; $resn_exch{"H232 TZ5"}="2H23 TZ5"; $resn_exch{"H241 TZ5"}="1H24 TZ5";
$resn_exch{"H242 TZ5"}="2H24 TZ5"; $resn_exch{"H251 TZ5"}="1H25 TZ5"; $resn_exch{"H252 TZ5"}="2H25 TZ5";
$resn_exch{"H281 TZ5"}="1H28 TZ5"; $resn_exch{"H282 TZ5"}="2H28 TZ5"; $resn_exch{"H291 TZ5"}="1H29 TZ5";
$resn_exch{"H292 TZ5"}="2H29 TZ5"; $resn_exch{"H381 TZ5"}="1H38 TZ5"; $resn_exch{"H382 TZ5"}="2H38 TZ5";
$resn_exch{"H401 TZ5"}="1H40 TZ5"; $resn_exch{"H402 TZ5"}="2H40 TZ5"; $resn_exch{"H411 TZ5"}="1H41 TZ5";
$resn_exch{"H412 TZ5"}="2H41 TZ5"; $resn_exch{"H421 TZ5"}="1H42 TZ5"; $resn_exch{"H422 TZ5"}="2H42 TZ5";
$resn_exch{"HNH1 TZA"}="1HNH TZA"; $resn_exch{"HNH2 TZA"}="2HNH TZA"; $resn_exch{"HE22 TZA"}="2HE2 TZA";
$resn_exch{"HD22 TZA"}="2HD2 TZA"; $resn_exch{"HE11 TZA"}="1HE1 TZA"; $resn_exch{"HD11 TZA"}="1HD1 TZA";
$resn_exch{"HCX1 TZA"}="1HCX TZA"; $resn_exch{"HCX2 TZA"}="2HCX TZA"; $resn_exch{"HCB1 TZA"}="1HCB TZA";
$resn_exch{"HCB2 TZA"}="2HCB TZA"; $resn_exch{" HN1 TZA"}="1HN  TZA"; $resn_exch{" HN2 TZA"}="2HN  TZA";
$resn_exch{" HC1 TZB"}="1HC  TZB"; $resn_exch{" HC2 TZB"}="2HC  TZB"; $resn_exch{"HC3A TZB"}="3HC  TZB";
$resn_exch{" HN1 TZB"}="1HN  TZB"; $resn_exch{" HN2 TZB"}="2HN  TZB"; $resn_exch{"HC41 TZB"}="1HC4 TZB";
$resn_exch{"HC42 TZB"}="2HC4 TZB"; $resn_exch{"HN71 TZC"}="1HN7 TZC"; $resn_exch{"HN72 TZC"}="2HN7 TZC";
$resn_exch{"HN81 TZC"}="1HN8 TZC"; $resn_exch{"HN82 TZC"}="2HN8 TZC"; $resn_exch{"H2A1 TZD"}="1H2A TZD";
$resn_exch{"H2A2 TZD"}="2H2A TZD"; $resn_exch{"H2A3 TZD"}="3H2A TZD"; $resn_exch{"H4'1 TZD"}="1H4* TZD";
$resn_exch{"H4'2 TZD"}="2H4* TZD"; $resn_exch{" H6' TZD"}=" H6* TZD"; $resn_exch{"H351 TZD"}="1H35 TZD";
$resn_exch{"H352 TZD"}="2H35 TZD"; $resn_exch{"H4A1 TZD"}="1H4A TZD"; $resn_exch{"H4A2 TZD"}="2H4A TZD";
$resn_exch{"H4A3 TZD"}="3H4A TZD"; $resn_exch{"H5A1 TZD"}="1H5A TZD"; $resn_exch{"H5A2 TZD"}="2H5A TZD";
$resn_exch{"H5B1 TZD"}="1H5B TZD"; $resn_exch{"H5B2 TZD"}="2H5B TZD"; $resn_exch{"H11' TZE"}="H11* TZE";
$resn_exch{"H12' TZE"}="H12* TZE"; $resn_exch{"H21' TZE"}="H21* TZE"; $resn_exch{"H22' TZE"}="H22* TZE";
$resn_exch{" H71 TZL"}="1H7  TZL"; $resn_exch{" H72 TZL"}="2H7  TZL"; $resn_exch{" H81 TZL"}="1H8  TZL";
$resn_exch{" H82 TZL"}="2H8  TZL"; $resn_exch{" H83 TZL"}="3H8  TZL"; $resn_exch{" H91 TZL"}="1H9  TZL";
$resn_exch{" H92 TZL"}="2H9  TZL"; $resn_exch{" H93 TZL"}="3H9  TZL"; $resn_exch{"HC21 TZO"}="1HC2 TZO";
$resn_exch{"HC41 TZO"}="1HC4 TZO"; $resn_exch{" H21 TZP"}="1H2  TZP"; $resn_exch{"HM41 TZP"}="1HM4 TZP";
$resn_exch{"HM42 TZP"}="2HM4 TZP"; $resn_exch{"HM43 TZP"}="3HM4 TZP"; $resn_exch{" H61 TZP"}="1H6  TZP";
$resn_exch{" H62 TZP"}="2H6  TZP"; $resn_exch{" H71 TZP"}="1H7  TZP"; $resn_exch{" H72 TZP"}="2H7  TZP";
$resn_exch{"HO11 TZP"}="1HO1 TZP"; $resn_exch{"HO31 TZP"}="1HO3 TZP"; $resn_exch{"H241 TZY"}="1H24 TZY";
$resn_exch{"H242 TZY"}="2H24 TZY"; $resn_exch{"H243 TZY"}="3H24 TZY"; $resn_exch{"H231 TZY"}="1H23 TZY";
$resn_exch{"H232 TZY"}="2H23 TZY"; $resn_exch{"H233 TZY"}="3H23 TZY"; $resn_exch{" H1  TZZ"}="1H   TZZ";
$resn_exch{" H2  TZZ"}="2H   TZZ"; $resn_exch{" H3  TZZ"}="3H   TZZ"; $resn_exch{" H11 TZZ"}="1H1  TZZ";
$resn_exch{" H12 TZZ"}="2H1  TZZ"; $resn_exch{" H13 TZZ"}="3H1  TZZ"; $resn_exch{" H21 TZZ"}="1H2  TZZ";
$resn_exch{" H22 TZZ"}="2H2  TZZ"; $resn_exch{" H23 TZZ"}="3H2  TZZ"; $resn_exch{"HOP3   U"}="3HOP   U";
$resn_exch{"HOP2   U"}="2HOP   U"; $resn_exch{" H5'   U"}="1H5*   U"; $resn_exch{"H5''   U"}="2H5*   U";
$resn_exch{" H4'   U"}=" H4*   U"; $resn_exch{" H3'   U"}=" H3*   U"; $resn_exch{"HO3'   U"}="H3T    U";
$resn_exch{" H2'   U"}=" H2*   U"; $resn_exch{"HO2'   U"}="2HO*   U"; $resn_exch{" H1'   U"}=" H1*   U";
$resn_exch{"HG31 U01"}="1HG3 U01"; $resn_exch{"HG32 U01"}="2HG3 U01"; $resn_exch{"HG33 U01"}="3HG3 U01";
$resn_exch{"HG41 U01"}="1HG4 U01"; $resn_exch{"HG42 U01"}="2HG4 U01"; $resn_exch{"HG43 U01"}="3HG4 U01";
$resn_exch{"HZ81 U02"}="1HZ8 U02"; $resn_exch{"HZ82 U02"}="2HZ8 U02"; $resn_exch{"HZ83 U02"}="3HZ8 U02";
$resn_exch{"HZ71 U02"}="1HZ7 U02"; $resn_exch{"HZ72 U02"}="2HZ7 U02"; $resn_exch{"HZ73 U02"}="3HZ7 U02";
$resn_exch{"HZ61 U02"}="1HZ6 U02"; $resn_exch{"HZ62 U02"}="2HZ6 U02"; $resn_exch{"HZ63 U02"}="3HZ6 U02";
$resn_exch{"HNZ3 U02"}="3HNZ U02"; $resn_exch{"HZ31 U02"}="1HZ3 U02"; $resn_exch{"HZ32 U02"}="2HZ3 U02";
$resn_exch{"HZ21 U02"}="1HZ2 U02"; $resn_exch{"HZ22 U02"}="2HZ2 U02"; $resn_exch{"HNZ1 U02"}="1HNZ U02";
$resn_exch{"HD11 U02"}="1HD1 U02"; $resn_exch{"HD12 U02"}="2HD1 U02"; $resn_exch{"HD21 U02"}="1HD2 U02";
$resn_exch{"HD22 U02"}="2HD2 U02"; $resn_exch{"HD23 U02"}="3HD2 U02"; $resn_exch{" HE1 U02"}="1HE  U02";
$resn_exch{"HE2A U02"}="2HE  U02"; $resn_exch{"HG21 U03"}="1HG2 U03"; $resn_exch{"HG22 U03"}="2HG2 U03";
$resn_exch{"HG31 U03"}="1HG3 U03"; $resn_exch{"HG32 U03"}="2HG3 U03"; $resn_exch{"HG33 U03"}="3HG3 U03";
$resn_exch{"HE31 U03"}="1HE3 U03"; $resn_exch{"HE32 U03"}="2HE3 U03"; $resn_exch{"HE33 U03"}="3HE3 U03";
$resn_exch{"HB71 U04"}="1HB7 U04"; $resn_exch{"HB72 U04"}="2HB7 U04"; $resn_exch{"HB81 U04"}="1HB8 U04";
$resn_exch{"HB82 U04"}="2HB8 U04"; $resn_exch{"HB83 U04"}="3HB8 U04"; $resn_exch{"HG21 U04"}="1HG2 U04";
$resn_exch{"HG22 U04"}="2HG2 U04"; $resn_exch{"HG51 U04"}="1HG5 U04"; $resn_exch{"HG52 U04"}="2HG5 U04";
$resn_exch{"HG53 U04"}="3HG5 U04"; $resn_exch{"HG61 U04"}="1HG6 U04"; $resn_exch{"HG62 U04"}="2HG6 U04";
$resn_exch{"HG63 U04"}="3HG6 U04"; $resn_exch{"HG71 U04"}="1HG7 U04"; $resn_exch{"HG72 U04"}="2HG7 U04";
$resn_exch{"HG73 U04"}="3HG7 U04"; $resn_exch{"H121 U05"}="1H12 U05"; $resn_exch{"H122 U05"}="2H12 U05";
$resn_exch{"H123 U05"}="3H12 U05"; $resn_exch{"H141 U05"}="1H14 U05"; $resn_exch{"H142 U05"}="2H14 U05";
$resn_exch{"H151 U05"}="1H15 U05"; $resn_exch{"H152 U05"}="2H15 U05"; $resn_exch{"H153 U05"}="3H15 U05";
$resn_exch{"H11A U0E"}="1H1  U0E"; $resn_exch{" H12 U0E"}="2H1  U0E"; $resn_exch{"H13A U0E"}="3H1  U0E";
$resn_exch{" H41 U0E"}="1H4  U0E"; $resn_exch{" H42 U0E"}="2H4  U0E"; $resn_exch{" H61 U0E"}="1H6  U0E";
$resn_exch{" H62 U0E"}="2H6  U0E"; $resn_exch{" H71 U0E"}="1H7  U0E"; $resn_exch{" H72 U0E"}="2H7  U0E";
$resn_exch{" H81 U0E"}="1H8  U0E"; $resn_exch{" H82 U0E"}="2H8  U0E"; $resn_exch{" H91 U0E"}="1H9  U0E";
$resn_exch{" H92 U0E"}="2H9  U0E"; $resn_exch{"H101 U0E"}="1H10 U0E"; $resn_exch{"H102 U0E"}="2H10 U0E";
$resn_exch{"H121 U0E"}="1H12 U0E"; $resn_exch{"H122 U0E"}="2H12 U0E"; $resn_exch{"H151 U0E"}="1H15 U0E";
$resn_exch{"H152 U0E"}="2H15 U0E"; $resn_exch{"H153 U0E"}="3H15 U0E"; $resn_exch{"H161 U0E"}="1H16 U0E";
$resn_exch{"H162 U0E"}="2H16 U0E"; $resn_exch{"H163 U0E"}="3H16 U0E"; $resn_exch{"H191 U0E"}="1H19 U0E";
$resn_exch{"H192 U0E"}="2H19 U0E"; $resn_exch{"H201 U0E"}="1H20 U0E"; $resn_exch{"H202 U0E"}="2H20 U0E";
$resn_exch{"H251 U0E"}="1H25 U0E"; $resn_exch{"H252 U0E"}="2H25 U0E"; $resn_exch{"H261 U0E"}="1H26 U0E";
$resn_exch{"H262 U0E"}="2H26 U0E"; $resn_exch{"H271 U0E"}="1H27 U0E"; $resn_exch{"H272 U0E"}="2H27 U0E";
$resn_exch{"HN31 U0E"}="1HN3 U0E"; $resn_exch{"HN32 U0E"}="2HN3 U0E"; $resn_exch{"HN51 U0E"}="1HN5 U0E";
$resn_exch{"HN52 U0E"}="2HN5 U0E"; $resn_exch{"HN81 U0E"}="1HN8 U0E"; $resn_exch{"HN82 U0E"}="2HN8 U0E";
$resn_exch{"H1M1 U10"}="1H1M U10"; $resn_exch{"H1M2 U10"}="2H1M U10"; $resn_exch{"H1M3 U10"}="3H1M U10";
$resn_exch{"H3M1 U10"}="1H3M U10"; $resn_exch{"H3M2 U10"}="2H3M U10"; $resn_exch{"H3M3 U10"}="3H3M U10";
$resn_exch{"H4M1 U10"}="1H4M U10"; $resn_exch{"H4M2 U10"}="2H4M U10"; $resn_exch{"H4M3 U10"}="3H4M U10";
$resn_exch{" H71 U10"}="1H7  U10"; $resn_exch{" H72 U10"}="2H7  U10"; $resn_exch{"H101 U10"}="1H10 U10";
$resn_exch{"H102 U10"}="2H10 U10"; $resn_exch{"H103 U10"}="3H10 U10"; $resn_exch{"H111 U10"}="1H11 U10";
$resn_exch{"H112 U10"}="2H11 U10"; $resn_exch{"H121 U10"}="1H12 U10"; $resn_exch{"H122 U10"}="2H12 U10";
$resn_exch{"H151 U10"}="1H15 U10"; $resn_exch{"H152 U10"}="2H15 U10"; $resn_exch{"H153 U10"}="3H15 U10";
$resn_exch{"H161 U10"}="1H16 U10"; $resn_exch{"H162 U10"}="2H16 U10"; $resn_exch{"H171 U10"}="1H17 U10";
$resn_exch{"H172 U10"}="2H17 U10"; $resn_exch{"H201 U10"}="1H20 U10"; $resn_exch{"H202 U10"}="2H20 U10";
$resn_exch{"H203 U10"}="3H20 U10"; $resn_exch{"H211 U10"}="1H21 U10"; $resn_exch{"H212 U10"}="2H21 U10";
$resn_exch{"H221 U10"}="1H22 U10"; $resn_exch{"H222 U10"}="2H22 U10"; $resn_exch{"H251 U10"}="1H25 U10";
$resn_exch{"H252 U10"}="2H25 U10"; $resn_exch{"H253 U10"}="3H25 U10"; $resn_exch{"H261 U10"}="1H26 U10";
$resn_exch{"H262 U10"}="2H26 U10"; $resn_exch{"H271 U10"}="1H27 U10"; $resn_exch{"H272 U10"}="2H27 U10";
$resn_exch{"H301 U10"}="1H30 U10"; $resn_exch{"H302 U10"}="2H30 U10"; $resn_exch{"H303 U10"}="3H30 U10";
$resn_exch{"H311 U10"}="1H31 U10"; $resn_exch{"H312 U10"}="2H31 U10"; $resn_exch{"H321 U10"}="1H32 U10";
$resn_exch{"H322 U10"}="2H32 U10"; $resn_exch{"H351 U10"}="1H35 U10"; $resn_exch{"H352 U10"}="2H35 U10";
$resn_exch{"H353 U10"}="3H35 U10"; $resn_exch{"H361 U10"}="1H36 U10"; $resn_exch{"H362 U10"}="2H36 U10";
$resn_exch{"H371 U10"}="1H37 U10"; $resn_exch{"H372 U10"}="2H37 U10"; $resn_exch{"H401 U10"}="1H40 U10";
$resn_exch{"H402 U10"}="2H40 U10"; $resn_exch{"H403 U10"}="3H40 U10"; $resn_exch{"H411 U10"}="1H41 U10";
$resn_exch{"H412 U10"}="2H41 U10"; $resn_exch{"H421 U10"}="1H42 U10"; $resn_exch{"H422 U10"}="2H42 U10";
$resn_exch{"H451 U10"}="1H45 U10"; $resn_exch{"H452 U10"}="2H45 U10"; $resn_exch{"H453 U10"}="3H45 U10";
$resn_exch{"H461 U10"}="1H46 U10"; $resn_exch{"H462 U10"}="2H46 U10"; $resn_exch{"H471 U10"}="1H47 U10";
$resn_exch{"H472 U10"}="2H47 U10"; $resn_exch{"H501 U10"}="1H50 U10"; $resn_exch{"H502 U10"}="2H50 U10";
$resn_exch{"H503 U10"}="3H50 U10"; $resn_exch{"H511 U10"}="1H51 U10"; $resn_exch{"H512 U10"}="2H51 U10";
$resn_exch{"H521 U10"}="1H52 U10"; $resn_exch{"H522 U10"}="2H52 U10"; $resn_exch{"H551 U10"}="1H55 U10";
$resn_exch{"H552 U10"}="2H55 U10"; $resn_exch{"H553 U10"}="3H55 U10"; $resn_exch{"H561 U10"}="1H56 U10";
$resn_exch{"H562 U10"}="2H56 U10"; $resn_exch{"H563 U10"}="3H56 U10"; $resn_exch{"H241 U11"}="1H24 U11";
$resn_exch{"H242 U11"}="2H24 U11"; $resn_exch{"H371 U11"}="1H37 U11"; $resn_exch{"H372 U11"}="2H37 U11";
$resn_exch{"H373 U11"}="3H37 U11"; $resn_exch{"H431 U11"}="1H43 U11"; $resn_exch{"H432 U11"}="2H43 U11";
$resn_exch{"H621 U11"}="1H62 U11"; $resn_exch{"H622 U11"}="2H62 U11"; $resn_exch{"H623 U11"}="3H62 U11";
$resn_exch{"HN52 U12"}="2HN5 U12"; $resn_exch{"HN21 U12"}="1HN2 U12"; $resn_exch{"HN22 U12"}="2HN2 U12";
$resn_exch{"H161 U13"}="1H16 U13"; $resn_exch{"H162 U13"}="2H16 U13"; $resn_exch{"H182 U13"}="2H18 U13";
$resn_exch{"HN41 U14"}="1HN4 U14"; $resn_exch{"HN42 U14"}="2HN4 U14"; $resn_exch{"HN12 U14"}="2HN1 U14";
$resn_exch{"H441 U15"}="1H44 U15"; $resn_exch{"H442 U15"}="2H44 U15"; $resn_exch{"H443 U15"}="3H44 U15";
$resn_exch{"H391 U15"}="1H39 U15"; $resn_exch{"H392 U15"}="2H39 U15"; $resn_exch{"H531 U15"}="1H53 U15";
$resn_exch{"H532 U15"}="2H53 U15"; $resn_exch{"H533 U15"}="3H53 U15"; $resn_exch{"H511 U15"}="1H51 U15";
$resn_exch{"H512 U15"}="2H51 U15"; $resn_exch{"H513 U15"}="3H51 U15"; $resn_exch{"H281 U15"}="1H28 U15";
$resn_exch{"H282 U15"}="2H28 U15"; $resn_exch{"H283 U15"}="3H28 U15"; $resn_exch{"H151 U15"}="1H15 U15";
$resn_exch{"H152 U15"}="2H15 U15"; $resn_exch{" H91 U15"}="1H9  U15"; $resn_exch{" H92 U15"}="2H9  U15";
$resn_exch{" H93 U15"}="3H9  U15"; $resn_exch{"H141 U16"}="1H14 U16"; $resn_exch{"H142 U16"}="2H14 U16";
$resn_exch{"H143 U16"}="3H14 U16"; $resn_exch{"H151 U16"}="1H15 U16"; $resn_exch{"H152 U16"}="2H15 U16";
$resn_exch{"H153 U16"}="3H15 U16"; $resn_exch{"H241 U16"}="1H24 U16"; $resn_exch{"H242 U16"}="2H24 U16";
$resn_exch{"H371 U16"}="1H37 U16"; $resn_exch{"H372 U16"}="2H37 U16"; $resn_exch{"H373 U16"}="3H37 U16";
$resn_exch{"H451 U16"}="1H45 U16"; $resn_exch{"H452 U16"}="2H45 U16"; $resn_exch{"H521 U16"}="1H52 U16";
$resn_exch{"H522 U16"}="2H52 U16"; $resn_exch{"H523 U16"}="3H52 U16"; $resn_exch{"H531 U16"}="1H53 U16";
$resn_exch{"H532 U16"}="2H53 U16"; $resn_exch{"H533 U16"}="3H53 U16"; $resn_exch{"H621 U16"}="1H62 U16";
$resn_exch{"H622 U16"}="2H62 U16"; $resn_exch{"H623 U16"}="3H62 U16"; $resn_exch{" H31 U17"}="1H3  U17";
$resn_exch{" H32 U17"}="2H3  U17"; $resn_exch{" H61 U17"}="1H6  U17"; $resn_exch{" H62 U17"}="2H6  U17";
$resn_exch{" H21 U17"}="1H2  U17"; $resn_exch{" H22 U17"}="2H2  U17"; $resn_exch{" H51 U17"}="1H5  U17";
$resn_exch{" H52 U17"}="2H5  U17"; $resn_exch{" H53 U17"}="3H5  U17"; $resn_exch{"HN11 U17"}="1HN1 U17";
$resn_exch{"HN12 U17"}="2HN1 U17"; $resn_exch{"H131 U17"}="1H13 U17"; $resn_exch{"H132 U17"}="2H13 U17";
$resn_exch{"H151 U17"}="1H15 U17"; $resn_exch{"H152 U17"}="2H15 U17"; $resn_exch{"H181 U17"}="1H18 U17";
$resn_exch{"H182 U17"}="2H18 U17"; $resn_exch{"H183 U17"}="3H18 U17"; $resn_exch{"H191 U17"}="1H19 U17";
$resn_exch{"H192 U17"}="2H19 U17"; $resn_exch{"H193 U17"}="3H19 U17"; $resn_exch{"H201 U17"}="1H20 U17";
$resn_exch{"H202 U17"}="2H20 U17"; $resn_exch{"H203 U17"}="3H20 U17"; $resn_exch{"HA21 U18"}="1HA2 U18";
$resn_exch{"HA22 U18"}="2HA2 U18"; $resn_exch{"HA23 U18"}="3HA2 U18"; $resn_exch{"H111 U18"}="1H11 U18";
$resn_exch{"H112 U18"}="2H11 U18"; $resn_exch{"H191 U18"}="1H19 U18"; $resn_exch{"H192 U18"}="2H19 U18";
$resn_exch{" HB1 U18"}="1HB  U18"; $resn_exch{" HB2 U18"}="2HB  U18"; $resn_exch{" HG1 U18"}="1HG  U18";
$resn_exch{" HG2 U18"}="2HG  U18"; $resn_exch{"HN51 U19"}="1HN5 U19"; $resn_exch{"HN52 U19"}="2HN5 U19";
$resn_exch{"HN42 U19"}="2HN4 U19"; $resn_exch{" H51 U1K"}="1H5  U1K"; $resn_exch{" H52 U1K"}="2H5  U1K";
$resn_exch{"H131 U1K"}="1H13 U1K"; $resn_exch{"H132 U1K"}="2H13 U1K"; $resn_exch{"H133 U1K"}="3H13 U1K";
$resn_exch{"H141 U1K"}="1H14 U1K"; $resn_exch{"H142 U1K"}="2H14 U1K"; $resn_exch{"H143 U1K"}="3H14 U1K";
$resn_exch{"H151 U1K"}="1H15 U1K"; $resn_exch{"H152 U1K"}="2H15 U1K"; $resn_exch{"H153 U1K"}="3H15 U1K";
$resn_exch{"H161 U1K"}="1H16 U1K"; $resn_exch{"H162 U1K"}="2H16 U1K"; $resn_exch{"H163 U1K"}="3H16 U1K";
$resn_exch{" H71 U1N"}="1H7  U1N"; $resn_exch{" H72 U1N"}="2H7  U1N"; $resn_exch{" H81 U1N"}="1H8  U1N";
$resn_exch{" H82 U1N"}="2H8  U1N"; $resn_exch{"H111 U1N"}="1H11 U1N"; $resn_exch{"H112 U1N"}="2H11 U1N";
$resn_exch{"H113 U1N"}="3H11 U1N"; $resn_exch{"H131 U1N"}="1H13 U1N"; $resn_exch{"H132 U1N"}="2H13 U1N";
$resn_exch{"H241 U1N"}="1H24 U1N"; $resn_exch{"H242 U1N"}="2H24 U1N"; $resn_exch{" H61 U20"}="1H6  U20";
$resn_exch{" H62 U20"}="2H6  U20"; $resn_exch{"HCB1 U20"}="1HCB U20"; $resn_exch{"HCB2 U20"}="2HCB U20";
$resn_exch{"HCB3 U20"}="3HCB U20"; $resn_exch{"HCA1 U20"}="1HCA U20"; $resn_exch{"HCA2 U20"}="2HCA U20";
$resn_exch{"HBZ1 U20"}="1HBZ U20"; $resn_exch{"HBZ2 U20"}="2HBZ U20"; $resn_exch{"HBY1 U20"}="1HBY U20";
$resn_exch{"HBY2 U20"}="2HBY U20"; $resn_exch{"HBX1 U20"}="1HBX U20"; $resn_exch{"HBX2 U20"}="2HBX U20";
$resn_exch{"HBW1 U20"}="1HBW U20"; $resn_exch{"HBW2 U20"}="2HBW U20"; $resn_exch{"HBV1 U20"}="1HBV U20";
$resn_exch{"HBV2 U20"}="2HBV U20"; $resn_exch{"HBU1 U20"}="1HBU U20"; $resn_exch{"HBU2 U20"}="2HBU U20";
$resn_exch{"HBS1 U20"}="1HBS U20"; $resn_exch{"HBS2 U20"}="2HBS U20"; $resn_exch{"HBR1 U20"}="1HBR U20";
$resn_exch{"HBR2 U20"}="2HBR U20"; $resn_exch{"HBL1 U20"}="1HBL U20"; $resn_exch{"HBL2 U20"}="2HBL U20";
$resn_exch{"HBI1 U20"}="1HBI U20"; $resn_exch{"HBI2 U20"}="2HBI U20"; $resn_exch{"HAQ1 U20"}="1HAQ U20";
$resn_exch{"HAQ2 U20"}="2HAQ U20"; $resn_exch{"HAQ3 U20"}="3HAQ U20"; $resn_exch{"H5'1 U20"}="1H5' U20";
$resn_exch{"H5'2 U20"}="2H5' U20"; $resn_exch{" H61 U21"}="1H6  U21"; $resn_exch{" H62 U21"}="2H6  U21";
$resn_exch{"HBX1 U21"}="1HBX U21"; $resn_exch{"HBX2 U21"}="2HBX U21"; $resn_exch{"HBX3 U21"}="3HBX U21";
$resn_exch{"HBW1 U21"}="1HBW U21"; $resn_exch{"HBW2 U21"}="2HBW U21"; $resn_exch{"HBV1 U21"}="1HBV U21";
$resn_exch{"HBV2 U21"}="2HBV U21"; $resn_exch{"HBU1 U21"}="1HBU U21"; $resn_exch{"HBU2 U21"}="2HBU U21";
$resn_exch{"HBS1 U21"}="1HBS U21"; $resn_exch{"HBS2 U21"}="2HBS U21"; $resn_exch{"HBR1 U21"}="1HBR U21";
$resn_exch{"HBR2 U21"}="2HBR U21"; $resn_exch{"HBL1 U21"}="1HBL U21"; $resn_exch{"HBL2 U21"}="2HBL U21";
$resn_exch{"HBI1 U21"}="1HBI U21"; $resn_exch{"HBI2 U21"}="2HBI U21"; $resn_exch{"HAQ1 U21"}="1HAQ U21";
$resn_exch{"HAQ2 U21"}="2HAQ U21"; $resn_exch{"HAQ3 U21"}="3HAQ U21"; $resn_exch{"H5'1 U21"}="1H5' U21";
$resn_exch{"H5'2 U21"}="2H5' U21"; $resn_exch{"HOP3 U25"}="3HOP U25"; $resn_exch{"HOP2 U25"}="2HOP U25";
$resn_exch{" H5' U25"}="1H5* U25"; $resn_exch{"H5'' U25"}="2H5* U25"; $resn_exch{" H4' U25"}=" H4* U25";
$resn_exch{" H3' U25"}=" H3* U25"; $resn_exch{"HO'3 U25"}="3HO* U25"; $resn_exch{" H2' U25"}=" H2* U25";
$resn_exch{"HO2' U25"}="2HO* U25"; $resn_exch{" H1' U25"}=" H1* U25"; $resn_exch{"H5'1 U2F"}="1H5' U2F";
$resn_exch{"H5'2 U2F"}="2H5' U2F"; $resn_exch{"H6C1 U2F"}="1H6C U2F"; $resn_exch{"H6C2 U2F"}="2H6C U2F";
$resn_exch{"H2G1 U2G"}="1H2G U2G"; $resn_exch{"H2G2 U2G"}="2H2G U2G"; $resn_exch{"H1G' U2G"}="*H1G U2G";
$resn_exch{"H2G' U2G"}="*H2G U2G"; $resn_exch{"HOG2 U2G"}="2HOG U2G"; $resn_exch{"H3G' U2G"}="*H3G U2G";
$resn_exch{"HOG3 U2G"}="3HOG U2G"; $resn_exch{"H4G' U2G"}="*H4G U2G"; $resn_exch{"H5G1 U2G"}="1H5G U2G";
$resn_exch{"H5G2 U2G"}="2H5G U2G"; $resn_exch{"HOP2 U2G"}="2HOP U2G"; $resn_exch{"HOU5 U2G"}="5HOU U2G";
$resn_exch{"H5U1 U2G"}="1H5U U2G"; $resn_exch{"H5U2 U2G"}="2H5U U2G"; $resn_exch{"H4U' U2G"}="*H4U U2G";
$resn_exch{"H3U' U2G"}="*H3U U2G"; $resn_exch{"HOU3 U2G"}="3HOU U2G"; $resn_exch{"H2U' U2G"}="*H2U U2G";
$resn_exch{"H1U' U2G"}="*H1U U2G"; $resn_exch{"HOP3 U2L"}="3HOP U2L"; $resn_exch{"HOP2 U2L"}="2HOP U2L";
$resn_exch{" H5' U2L"}="1H5* U2L"; $resn_exch{"H5'' U2L"}="2H5* U2L"; $resn_exch{" H4' U2L"}=" H4* U2L";
$resn_exch{" H3' U2L"}=" H3* U2L"; $resn_exch{" H2' U2L"}=" H2* U2L"; $resn_exch{"HO2' U2L"}="2HO* U2L";
$resn_exch{" H1' U2L"}=" H1* U2L"; $resn_exch{"HA'1 U2L"}="1HA* U2L"; $resn_exch{"HA'2 U2L"}="2HA* U2L";
$resn_exch{"HB'1 U2L"}="1HB* U2L"; $resn_exch{"HB'2 U2L"}="2HB* U2L"; $resn_exch{"HD'1 U2L"}="1HD* U2L";
$resn_exch{"HD'2 U2L"}="2HD* U2L"; $resn_exch{"HD'3 U2L"}="3HD* U2L"; $resn_exch{" H71 U2L"}="1H7  U2L";
$resn_exch{" H72 U2L"}="2H7  U2L"; $resn_exch{" H73 U2L"}="3H7  U2L"; $resn_exch{" H5' U2N"}="1H5* U2N";
$resn_exch{"H5'' U2N"}="2H5* U2N"; $resn_exch{" H4' U2N"}=" H4* U2N"; $resn_exch{" H1' U2N"}=" H1* U2N";
$resn_exch{" H2' U2N"}=" H2* U2N"; $resn_exch{"H2'1 U2N"}="1H2* U2N"; $resn_exch{"H2'' U2N"}="2H2* U2N";
$resn_exch{" H3' U2N"}=" H3* U2N"; $resn_exch{"HO3' U2N"}="H3T  U2N"; $resn_exch{"H5'1 U2P"}="1H5' U2P";
$resn_exch{"H5'2 U2P"}="2H5' U2P"; $resn_exch{"H5'1 U31"}="1H5* U31"; $resn_exch{"H5'2 U31"}="2H5* U31";
$resn_exch{" H4' U31"}=" H4* U31"; $resn_exch{" H3' U31"}=" H3* U31"; $resn_exch{" H2' U31"}=" H2* U31";
$resn_exch{" H1' U31"}=" H1* U31"; $resn_exch{"HA'1 U31"}="1HA* U31"; $resn_exch{"HA'2 U31"}="2HA* U31";
$resn_exch{"HB'1 U31"}="1HB* U31"; $resn_exch{"HB'2 U31"}="2HB* U31"; $resn_exch{"HCC1 U31"}="1HCC U31";
$resn_exch{"HCC2 U31"}="2HCC U31"; $resn_exch{"HD'1 U31"}="1HD* U31"; $resn_exch{"HD'2 U31"}="2HD* U31";
$resn_exch{"H5C1 U32"}="1H5C U32"; $resn_exch{"H5C2 U32"}="2H5C U32"; $resn_exch{"H5C3 U32"}="3H5C U32";
$resn_exch{"H6C1 U32"}="1H6C U32"; $resn_exch{"H6C2 U32"}="2H6C U32"; $resn_exch{"H6C3 U32"}="3H6C U32";
$resn_exch{"H111 U32"}="1H11 U32"; $resn_exch{"H112 U32"}="2H11 U32"; $resn_exch{" H1' U33"}=" H1* U33";
$resn_exch{"H2'1 U33"}="1H2* U33"; $resn_exch{"H2'2 U33"}="2H2* U33"; $resn_exch{" H3' U33"}=" H3* U33";
$resn_exch{" H4' U33"}=" H4* U33"; $resn_exch{"H5'1 U33"}="1H5* U33"; $resn_exch{"H5'2 U33"}="2H5* U33";
$resn_exch{" H5' U33"}=" H5* U33"; $resn_exch{"H5'1 U34"}="1H5* U34"; $resn_exch{"H5'2 U34"}="2H5* U34";
$resn_exch{" H4' U34"}=" H4* U34"; $resn_exch{" H3' U34"}=" H3* U34"; $resn_exch{" H2' U34"}=" H2* U34";
$resn_exch{" H1' U34"}=" H1* U34"; $resn_exch{"H5'1 U36"}="1H5* U36"; $resn_exch{"H5'2 U36"}="2H5* U36";
$resn_exch{" H4' U36"}=" H4* U36"; $resn_exch{" H3' U36"}=" H3* U36"; $resn_exch{" H2' U36"}=" H2* U36";
$resn_exch{" H1' U36"}=" H1* U36"; $resn_exch{"HA'1 U36"}="1HA* U36"; $resn_exch{"HA'2 U36"}="2HA* U36";
$resn_exch{"HB'1 U36"}="1HB* U36"; $resn_exch{"HB'2 U36"}="2HB* U36"; $resn_exch{"HD'1 U36"}="1HD* U36";
$resn_exch{"HD'2 U36"}="2HD* U36"; $resn_exch{"HD'3 U36"}="3HD* U36"; $resn_exch{"H5'1 U37"}="1H5* U37";
$resn_exch{"H5'2 U37"}="2H5* U37"; $resn_exch{" H4' U37"}=" H4* U37"; $resn_exch{" H3' U37"}=" H3* U37";
$resn_exch{" H2' U37"}=" H2* U37"; $resn_exch{" H1' U37"}=" H1* U37"; $resn_exch{" H2' U3H"}=" H2* U3H";
$resn_exch{" H1' U3H"}=" H1* U3H"; $resn_exch{"H2'' U3H"}="*H2* U3H"; $resn_exch{" H3' U3H"}=" H3* U3H";
$resn_exch{"H3'2 U3H"}="2H3* U3H"; $resn_exch{" H4' U3H"}=" H4* U3H"; $resn_exch{"H5'1 U3H"}="1H5* U3H";
$resn_exch{"H5'2 U3H"}="2H5* U3H"; $resn_exch{"HOP1 U3P"}="1HOP U3P"; $resn_exch{"HOP3 U3P"}="3HOP U3P";
$resn_exch{"HO5' U3P"}="*HO5 U3P"; $resn_exch{"H5'1 U3P"}="1H5* U3P"; $resn_exch{"H5'2 U3P"}="2H5* U3P";
$resn_exch{" H4' U3P"}=" H4* U3P"; $resn_exch{" H3' U3P"}=" H3* U3P"; $resn_exch{" H2' U3P"}=" H2* U3P";
$resn_exch{"HO2' U3P"}="*HO2 U3P"; $resn_exch{" H1' U3P"}=" H1* U3P"; $resn_exch{" H41 U49"}="1H4  U49";
$resn_exch{" H61 U49"}="1H6  U49"; $resn_exch{" H71 U49"}="1H7  U49"; $resn_exch{" H81 U49"}="1H8  U49";
$resn_exch{"H101 U49"}="1H10 U49"; $resn_exch{"H102 U49"}="2H10 U49"; $resn_exch{"H121 U49"}="1H12 U49";
$resn_exch{"H131 U49"}="1H13 U49"; $resn_exch{"H151 U49"}="1H15 U49"; $resn_exch{"H152 U49"}="2H15 U49";
$resn_exch{"H171 U49"}="1H17 U49"; $resn_exch{"H211 U49"}="1H21 U49"; $resn_exch{"H221 U49"}="1H22 U49";
$resn_exch{"H222 U49"}="2H22 U49"; $resn_exch{"H231 U49"}="1H23 U49"; $resn_exch{"H271 U49"}="1H27 U49";
$resn_exch{"H291 U49"}="1H29 U49"; $resn_exch{"H292 U49"}="2H29 U49"; $resn_exch{"H311 U49"}="1H31 U49";
$resn_exch{"H321 U49"}="1H32 U49"; $resn_exch{"H541 U49"}="1H54 U49"; $resn_exch{"HAR1 U51"}="1HAR U51";
$resn_exch{"HAR2 U51"}="2HAR U51"; $resn_exch{"HAQ1 U51"}="1HAQ U51"; $resn_exch{"HAQ2 U51"}="2HAQ U51";
$resn_exch{"HAP1 U51"}="1HAP U51"; $resn_exch{"HAP2 U51"}="2HAP U51"; $resn_exch{"HAO1 U51"}="1HAO U51";
$resn_exch{"HAO2 U51"}="2HAO U51"; $resn_exch{"HAB1 U51"}="1HAB U51"; $resn_exch{"HAB2 U51"}="2HAB U51";
$resn_exch{"HAI1 U51"}="1HAI U51"; $resn_exch{"HAI2 U51"}="2HAI U51"; $resn_exch{"HAJ1 U51"}="1HAJ U51";
$resn_exch{"HAJ2 U51"}="2HAJ U51"; $resn_exch{"HAK1 U51"}="1HAK U51"; $resn_exch{"HAK2 U51"}="2HAK U51";
$resn_exch{"HAL1 U51"}="1HAL U51"; $resn_exch{"HAL2 U51"}="2HAL U51"; $resn_exch{"HAM1 U51"}="1HAM U51";
$resn_exch{"HAM2 U51"}="2HAM U51"; $resn_exch{"HAU1 U51"}="1HAU U51"; $resn_exch{"HAU2 U51"}="2HAU U51";
$resn_exch{"HAU3 U51"}="3HAU U51"; $resn_exch{"H171 U55"}="1H17 U55"; $resn_exch{"H172 U55"}="2H17 U55";
$resn_exch{"H131 U55"}="1H13 U55"; $resn_exch{"H161 U55"}="1H16 U55"; $resn_exch{"H181 U55"}="1H18 U55";
$resn_exch{"H061 U55"}="1H06 U55"; $resn_exch{"H051 U55"}="1H05 U55"; $resn_exch{"H021 U55"}="1H02 U55";
$resn_exch{"H031 U55"}="1H03 U55"; $resn_exch{"H101 U55"}="1H10 U55"; $resn_exch{"H102 U55"}="2H10 U55";
$resn_exch{" H1' U5P"}=" H1* U5P"; $resn_exch{" H2' U5P"}=" H2* U5P"; $resn_exch{"HO2' U5P"}="*HO2 U5P";
$resn_exch{" H3' U5P"}=" H3* U5P"; $resn_exch{" H4' U5P"}=" H4* U5P"; $resn_exch{"HO3' U5P"}="*HO3 U5P";
$resn_exch{"H5'1 U5P"}="1H5* U5P"; $resn_exch{"H5'2 U5P"}="2H5* U5P"; $resn_exch{"HOP2 U5P"}="2HOP U5P";
$resn_exch{"HOP3 U5P"}="3HOP U5P"; $resn_exch{" H31 U66"}="1H3  U66"; $resn_exch{" H41 U66"}="1H4  U66";
$resn_exch{" H51 U66"}="1H5  U66"; $resn_exch{" H61 U66"}="1H6  U66"; $resn_exch{" H91 U66"}="1H9  U66";
$resn_exch{" H92 U66"}="2H9  U66"; $resn_exch{"H101 U66"}="1H10 U66"; $resn_exch{"H111 U66"}="1H11 U66";
$resn_exch{"H121 U66"}="1H12 U66"; $resn_exch{"H141 U66"}="1H14 U66"; $resn_exch{"H142 U66"}="2H14 U66";
$resn_exch{"H151 U66"}="1H15 U66"; $resn_exch{"H161 U66"}="1H16 U66"; $resn_exch{"H117 U66"}="17H1 U66";
$resn_exch{"H181 U66"}="1H18 U66"; $resn_exch{"H182 U66"}="2H18 U66"; $resn_exch{"H191 U66"}="1H19 U66";
$resn_exch{"H192 U66"}="2H19 U66"; $resn_exch{"H201 U66"}="1H20 U66"; $resn_exch{"H202 U66"}="2H20 U66";
$resn_exch{"H231 U66"}="1H23 U66"; $resn_exch{"H232 U66"}="2H23 U66"; $resn_exch{"H241 U66"}="1H24 U66";
$resn_exch{"H261 U66"}="1H26 U66"; $resn_exch{"H262 U66"}="2H26 U66"; $resn_exch{"H291 U66"}="1H29 U66";
$resn_exch{"H301 U66"}="1H30 U66"; $resn_exch{"H8C1 U73"}="1H8C U73"; $resn_exch{"H8C2 U73"}="2H8C U73";
$resn_exch{"H8C3 U73"}="3H8C U73"; $resn_exch{"H7C1 U73"}="1H7C U73"; $resn_exch{"H7C2 U73"}="2H7C U73";
$resn_exch{"H7C3 U73"}="3H7C U73"; $resn_exch{"H121 U73"}="1H12 U73"; $resn_exch{"H122 U73"}="2H12 U73";
$resn_exch{" HB1 U89"}="1HB  U89"; $resn_exch{" HB2 U89"}="2HB  U89"; $resn_exch{" HG1 U89"}="1HG  U89";
$resn_exch{" HG2 U89"}="2HG  U89"; $resn_exch{" H11 U89"}="1H1  U89"; $resn_exch{" H12 U89"}="2H1  U89";
$resn_exch{" H21 U89"}="1H2  U89"; $resn_exch{" H22 U89"}="2H2  U89"; $resn_exch{" H31 U89"}="1H3  U89";
$resn_exch{" H32 U89"}="2H3  U89"; $resn_exch{" H41 U89"}="1H4  U89"; $resn_exch{" H42 U89"}="2H4  U89";
$resn_exch{" H71 U89"}="1H7  U89"; $resn_exch{" H72 U89"}="2H7  U89"; $resn_exch{" H81 U89"}="1H8  U89";
$resn_exch{" H82 U89"}="2H8  U89"; $resn_exch{"H111 U89"}="1H11 U89"; $resn_exch{"H112 U89"}="2H11 U89";
$resn_exch{"H121 U89"}="1H12 U89"; $resn_exch{"H122 U89"}="2H12 U89"; $resn_exch{"H131 U89"}="1H13 U89";
$resn_exch{"H132 U89"}="2H13 U89"; $resn_exch{"HN81 U89"}="1HN8 U89"; $resn_exch{"HN82 U89"}="2HN8 U89";
$resn_exch{"HN11 U89"}="1HN1 U89"; $resn_exch{"HN12 U89"}="2HN1 U89"; $resn_exch{"HOP2 U89"}="2HOP U89";
$resn_exch{"HOP3 U89"}="3HOP U89"; $resn_exch{" HC1 U8U"}="1HC  U8U"; $resn_exch{" HC2 U8U"}="2HC  U8U";
$resn_exch{" HN1 U8U"}="1HN  U8U"; $resn_exch{" HA1 U8U"}="1HA  U8U"; $resn_exch{" HA2 U8U"}="2HA  U8U";
$resn_exch{" HA3 U8U"}="3HA  U8U"; $resn_exch{" H1' U8U"}=" H1* U8U"; $resn_exch{"HO2' U8U"}="2HO* U8U";
$resn_exch{" H2' U8U"}=" H2* U8U"; $resn_exch{"HO3' U8U"}="H3T  U8U"; $resn_exch{" H3' U8U"}=" H3* U8U";
$resn_exch{" H4' U8U"}=" H4* U8U"; $resn_exch{" H5' U8U"}="1H5* U8U"; $resn_exch{"H5'' U8U"}="2H5* U8U";
$resn_exch{"HOP2 U8U"}="2HOP U8U"; $resn_exch{"HOP3 U8U"}="3HOP U8U"; $resn_exch{"H551 UA1"}="1H55 UA1";
$resn_exch{"H552 UA1"}="2H55 UA1"; $resn_exch{"H553 UA1"}="3H55 UA1"; $resn_exch{"H541 UA1"}="1H54 UA1";
$resn_exch{"H542 UA1"}="2H54 UA1"; $resn_exch{"H543 UA1"}="3H54 UA1"; $resn_exch{"H531 UA1"}="1H53 UA1";
$resn_exch{"H532 UA1"}="2H53 UA1"; $resn_exch{"H533 UA1"}="3H53 UA1"; $resn_exch{"H341 UA1"}="1H34 UA1";
$resn_exch{"H342 UA1"}="2H34 UA1"; $resn_exch{"H231 UA1"}="1H23 UA1"; $resn_exch{"H232 UA1"}="2H23 UA1";
$resn_exch{"H233 UA1"}="3H23 UA1"; $resn_exch{"H661 UA1"}="1H66 UA1"; $resn_exch{"H662 UA1"}="2H66 UA1";
$resn_exch{"HO5' UA2"}="HO5* UA2"; $resn_exch{"H5'1 UA2"}="1H5* UA2"; $resn_exch{"H5'2 UA2"}="2H5* UA2";
$resn_exch{" H4' UA2"}=" H4* UA2"; $resn_exch{"HN4' UA2"}="HN4* UA2"; $resn_exch{" H3' UA2"}=" H3* UA2";
$resn_exch{"HO3' UA2"}="HO3* UA2"; $resn_exch{" H2' UA2"}=" H2* UA2"; $resn_exch{"HO2' UA2"}="HO2* UA2";
$resn_exch{" H1' UA2"}=" H1* UA2"; $resn_exch{"HN61 UA2"}="1HN6 UA2"; $resn_exch{"HN62 UA2"}="2HN6 UA2";
$resn_exch{" H5' UA3"}=" H5* UA3"; $resn_exch{" H51 UA3"}="1H5  UA3"; $resn_exch{" H52 UA3"}="2H5  UA3";
$resn_exch{" H3' UA3"}=" H3* UA3"; $resn_exch{" H2' UA3"}=" H2* UA3"; $resn_exch{" H1' UA3"}=" H1* UA3";
$resn_exch{"H101 UA4"}="1H10 UA4"; $resn_exch{"H102 UA4"}="2H10 UA4"; $resn_exch{"H103 UA4"}="3H10 UA4";
$resn_exch{"H221 UA4"}="1H22 UA4"; $resn_exch{"H222 UA4"}="2H22 UA4"; $resn_exch{"H241 UA4"}="1H24 UA4";
$resn_exch{"H242 UA4"}="2H24 UA4"; $resn_exch{"H261 UA4"}="1H26 UA4"; $resn_exch{"H262 UA4"}="2H26 UA4";
$resn_exch{"H263 UA4"}="3H26 UA4"; $resn_exch{"H351 UA4"}="1H35 UA4"; $resn_exch{"H352 UA4"}="2H35 UA4";
$resn_exch{"H353 UA4"}="3H35 UA4"; $resn_exch{"H231 UA4"}="1H23 UA4"; $resn_exch{"H232 UA4"}="2H23 UA4";
$resn_exch{" H91 UA4"}="1H9  UA4"; $resn_exch{" H92 UA4"}="2H9  UA4"; $resn_exch{"HN71 UA4"}="1HN7 UA4";
$resn_exch{"HN72 UA4"}="2HN7 UA4"; $resn_exch{"HN81 UA4"}="1HN8 UA4"; $resn_exch{"HN82 UA4"}="2HN8 UA4";
$resn_exch{" H71 UA5"}="1H7  UA5"; $resn_exch{" H72 UA5"}="2H7  UA5"; $resn_exch{" H81 UA5"}="1H8  UA5";
$resn_exch{" H82 UA5"}="2H8  UA5"; $resn_exch{" H83 UA5"}="3H8  UA5"; $resn_exch{"H7C1 UAA"}="1H7C UAA";
$resn_exch{"H7C2 UAA"}="2H7C UAA"; $resn_exch{" H1B UAG"}=" H1* UAG"; $resn_exch{" H2B UAG"}=" H2* UAG";
$resn_exch{"HO'2 UAG"}="2HO* UAG"; $resn_exch{" H3B UAG"}=" H3* UAG"; $resn_exch{"HO'3 UAG"}="3HO* UAG";
$resn_exch{" H4B UAG"}=" H4* UAG"; $resn_exch{"H5'1 UAG"}="1H5* UAG"; $resn_exch{"H5'2 UAG"}="2H5* UAG";
$resn_exch{"HOA2 UAG"}="2HOA UAG"; $resn_exch{"HOB2 UAG"}="2HOB UAG"; $resn_exch{"HN2' UAG"}="'HN2 UAG";
$resn_exch{"H8'1 UAG"}="1H8' UAG"; $resn_exch{"H8'2 UAG"}="2H8' UAG"; $resn_exch{"H8'3 UAG"}="3H8' UAG";
$resn_exch{"HO4' UAG"}="'HO4 UAG"; $resn_exch{"H6'1 UAG"}="1H6' UAG"; $resn_exch{"H6'2 UAG"}="2H6' UAG";
$resn_exch{"HO6' UAG"}="'HO6 UAG"; $resn_exch{"H201 UAG"}="1H20 UAG"; $resn_exch{"H202 UAG"}="2H20 UAG";
$resn_exch{"H203 UAG"}="3H20 UAG"; $resn_exch{"H231 UAG"}="1H23 UAG"; $resn_exch{"H232 UAG"}="2H23 UAG";
$resn_exch{"H233 UAG"}="3H23 UAG"; $resn_exch{"H251 UAG"}="1H25 UAG"; $resn_exch{"H252 UAG"}="2H25 UAG";
$resn_exch{"H261 UAG"}="1H26 UAG"; $resn_exch{"H262 UAG"}="2H26 UAG"; $resn_exch{" H11 UAP"}="1H1  UAP";
$resn_exch{" H12 UAP"}="2H1  UAP"; $resn_exch{"HOS3 UAP"}="3HOS UAP"; $resn_exch{"HOP3 UAR"}="3HOP UAR";
$resn_exch{"HOP2 UAR"}="2HOP UAR"; $resn_exch{" H5' UAR"}="1H5* UAR"; $resn_exch{"H5'' UAR"}="2H5* UAR";
$resn_exch{" H4' UAR"}=" H4* UAR"; $resn_exch{" H3' UAR"}=" H3* UAR"; $resn_exch{"HO3' UAR"}="H3T  UAR";
$resn_exch{" H2' UAR"}=" H2* UAR"; $resn_exch{"HO2' UAR"}="2HO* UAR"; $resn_exch{" H1' UAR"}=" H1* UAR";
$resn_exch{" HN1 UBA"}="1HN  UBA"; $resn_exch{" HN2 UBA"}="2HN  UBA"; $resn_exch{" HB1 UBA"}="1HB  UBA";
$resn_exch{" HB2 UBA"}="2HB  UBA"; $resn_exch{" H61 UBA"}="1H6  UBA"; $resn_exch{" H62 UBA"}="2H6  UBA";
$resn_exch{" H63 UBA"}="3H6  UBA"; $resn_exch{"H171 UBA"}="1H17 UBA"; $resn_exch{"H172 UBA"}="2H17 UBA";
$resn_exch{"H5'1 UBB"}="1H5* UBB"; $resn_exch{"H5'2 UBB"}="2H5* UBB"; $resn_exch{" H4' UBB"}=" H4* UBB";
$resn_exch{" H1' UBB"}=" H1* UBB"; $resn_exch{"H2'1 UBB"}="1H2* UBB"; $resn_exch{"H2'2 UBB"}="2H2* UBB";
$resn_exch{"H3'1 UBB"}="1H3* UBB"; $resn_exch{"H3'2 UBB"}="2H3* UBB"; $resn_exch{"H171 UBC"}="1H17 UBC";
$resn_exch{"H172 UBC"}="2H17 UBC"; $resn_exch{" HB1 UBC"}="1HB  UBC"; $resn_exch{" HB2 UBC"}="2HB  UBC";
$resn_exch{" HN1 UBC"}="1HN  UBC"; $resn_exch{" HN2 UBC"}="2HN  UBC"; $resn_exch{" HA1 UC1"}="1HA  UC1";
$resn_exch{" HA2 UC1"}="2HA  UC1"; $resn_exch{" HA3 UC1"}="3HA  UC1"; $resn_exch{" HC1 UC1"}="1HC  UC1";
$resn_exch{" HC2 UC1"}="2HC  UC1"; $resn_exch{" HF1 UC1"}="1HF  UC1"; $resn_exch{" HF2 UC1"}="2HF  UC1";
$resn_exch{" HF3 UC1"}="3HF  UC1"; $resn_exch{" HG1 UC1"}="1HG  UC1"; $resn_exch{" HG2 UC1"}="2HG  UC1";
$resn_exch{" HG3 UC1"}="3HG  UC1"; $resn_exch{" HA1 UC2"}="1HA  UC2"; $resn_exch{" HA2 UC2"}="2HA  UC2";
$resn_exch{" HA3 UC2"}="3HA  UC2"; $resn_exch{" HF1 UC2"}="1HF  UC2"; $resn_exch{" HF2 UC2"}="2HF  UC2";
$resn_exch{" HF3 UC2"}="3HF  UC2"; $resn_exch{" HG1 UC2"}="1HG  UC2"; $resn_exch{" HG2 UC2"}="2HG  UC2";
$resn_exch{" HG3 UC2"}="3HG  UC2"; $resn_exch{" HH1 UC2"}="1HH  UC2"; $resn_exch{" HH2 UC2"}="2HH  UC2";
$resn_exch{" HH3 UC2"}="3HH  UC2"; $resn_exch{" HD1 UC3"}="1HD  UC3"; $resn_exch{" HD2 UC3"}="2HD  UC3";
$resn_exch{" HD3 UC3"}="3HD  UC3"; $resn_exch{" HE1 UC3"}="1HE  UC3"; $resn_exch{" HE2 UC3"}="2HE  UC3";
$resn_exch{" HE3 UC3"}="3HE  UC3"; $resn_exch{" HH1 UC3"}="1HH  UC3"; $resn_exch{" HH2 UC3"}="2HH  UC3";
$resn_exch{" HH3 UC3"}="3HH  UC3"; $resn_exch{" HI1 UC3"}="1HI  UC3"; $resn_exch{" HI2 UC3"}="2HI  UC3";
$resn_exch{" HI3 UC3"}="3HI  UC3"; $resn_exch{" H91 UC4"}="1H9  UC4"; $resn_exch{" H92 UC4"}="2H9  UC4";
$resn_exch{" H93 UC4"}="3H9  UC4"; $resn_exch{"H101 UC4"}="1H10 UC4"; $resn_exch{"H102 UC4"}="2H10 UC4";
$resn_exch{"H103 UC4"}="3H10 UC4"; $resn_exch{" HD1 UC4"}="1HD  UC4"; $resn_exch{" HD2 UC4"}="2HD  UC4";
$resn_exch{" HE1 UC4"}="1HE  UC4"; $resn_exch{" HE2 UC4"}="2HE  UC4"; $resn_exch{" HF1 UC4"}="1HF  UC4";
$resn_exch{" HF2 UC4"}="2HF  UC4"; $resn_exch{" HF3 UC4"}="3HF  UC4"; $resn_exch{" H1' UC5"}=" H1* UC5";
$resn_exch{"H2'1 UC5"}="1H2* UC5"; $resn_exch{"H2'2 UC5"}="2H2* UC5"; $resn_exch{" H3' UC5"}=" H3* UC5";
$resn_exch{"HO3' UC5"}="HO3* UC5"; $resn_exch{" H4' UC5"}=" H4* UC5"; $resn_exch{"H5'1 UC5"}="1H5* UC5";
$resn_exch{"H5'2 UC5"}="2H5* UC5"; $resn_exch{"H3A1 UC5"}="1H3A UC5"; $resn_exch{"H3A2 UC5"}="2H3A UC5";
$resn_exch{"H291 UCB"}="1H29 UCB"; $resn_exch{"H292 UCB"}="2H29 UCB"; $resn_exch{"H293 UCB"}="3H29 UCB";
$resn_exch{"HAA1 UCD"}="1HAA UCD"; $resn_exch{"HAA2 UCD"}="2HAA UCD"; $resn_exch{"HAA3 UCD"}="3HAA UCD";
$resn_exch{" H61 UCD"}="1H6  UCD"; $resn_exch{" H62 UCD"}="2H6  UCD"; $resn_exch{"H5'1 UCL"}="1H5* UCL";
$resn_exch{"H5'2 UCL"}="2H5* UCL"; $resn_exch{" H4' UCL"}=" H4* UCL"; $resn_exch{" H3' UCL"}=" H3* UCL";
$resn_exch{"HO3' UCL"}="HO3* UCL"; $resn_exch{"H2'1 UCL"}="1H2* UCL"; $resn_exch{"H2'2 UCL"}="2H2* UCL";
$resn_exch{" H1' UCL"}=" H1* UCL"; $resn_exch{"H031 UCM"}="1H03 UCM"; $resn_exch{"H032 UCM"}="2H03 UCM";
$resn_exch{"H041 UCM"}="1H04 UCM"; $resn_exch{"H042 UCM"}="2H04 UCM"; $resn_exch{"H241 UCN"}="1H24 UCN";
$resn_exch{"H242 UCN"}="2H24 UCN"; $resn_exch{"H281 UCN"}="1H28 UCN"; $resn_exch{"H282 UCN"}="2H28 UCN";
$resn_exch{"H283 UCN"}="3H28 UCN"; $resn_exch{"H271 UCN"}="1H27 UCN"; $resn_exch{"H272 UCN"}="2H27 UCN";
$resn_exch{"H273 UCN"}="3H27 UCN"; $resn_exch{"H261 UCN"}="1H26 UCN"; $resn_exch{"H262 UCN"}="2H26 UCN";
$resn_exch{"H263 UCN"}="3H26 UCN"; $resn_exch{" H1' UCP"}=" H1* UCP"; $resn_exch{"H2'1 UCP"}="1H2* UCP";
$resn_exch{"H2'2 UCP"}="2H2* UCP"; $resn_exch{" H3' UCP"}=" H3* UCP"; $resn_exch{"HO'3 UCP"}="3HO* UCP";
$resn_exch{" H4' UCP"}=" H4* UCP"; $resn_exch{"H5'1 UCP"}="1H5* UCP"; $resn_exch{"H5'2 UCP"}="2H5* UCP";
$resn_exch{"HOA2 UCP"}="2HOA UCP"; $resn_exch{"HOB2 UCP"}="2HOB UCP"; $resn_exch{"HCC1 UCP"}="1HCC UCP";
$resn_exch{"HCC2 UCP"}="2HCC UCP"; $resn_exch{"HOG2 UCP"}="2HOG UCP"; $resn_exch{"HOG3 UCP"}="3HOG UCP";
$resn_exch{"H6'1 UD1"}="1H6' UD1"; $resn_exch{"H6'2 UD1"}="2H6' UD1"; $resn_exch{"H8'1 UD1"}="1H8' UD1";
$resn_exch{"H8'2 UD1"}="2H8' UD1"; $resn_exch{"H8'3 UD1"}="3H8' UD1"; $resn_exch{"HN2' UD1"}="'HN2 UD1";
$resn_exch{"HO3' UD1"}="'HO3 UD1"; $resn_exch{"HO4' UD1"}="'HO4 UD1"; $resn_exch{"HO6' UD1"}="'HO6 UD1";
$resn_exch{" H1B UD1"}=" H1* UD1"; $resn_exch{" H2B UD1"}=" H2* UD1"; $resn_exch{"HO2' UD1"}="*HO2 UD1";
$resn_exch{" H3B UD1"}=" H3* UD1"; $resn_exch{" H4B UD1"}=" H4* UD1"; $resn_exch{"HO3A UD1"}="*HO3 UD1";
$resn_exch{"H5'1 UD1"}="1H5* UD1"; $resn_exch{"H5'2 UD1"}="2H5* UD1"; $resn_exch{"HOA2 UD1"}="2HOA UD1";
$resn_exch{"HOB2 UD1"}="2HOB UD1"; $resn_exch{"H6'1 UD2"}="1H6' UD2"; $resn_exch{"H6'2 UD2"}="2H6' UD2";
$resn_exch{"H8'1 UD2"}="1H8' UD2"; $resn_exch{"H8'2 UD2"}="2H8' UD2"; $resn_exch{"H8'3 UD2"}="3H8' UD2";
$resn_exch{" H1B UD2"}=" H1* UD2"; $resn_exch{" H2B UD2"}=" H2* UD2"; $resn_exch{"HO2' UD2"}="HO2* UD2";
$resn_exch{" H3B UD2"}=" H3* UD2"; $resn_exch{" H4B UD2"}=" H4* UD2"; $resn_exch{"HO3A UD2"}="HO3* UD2";
$resn_exch{"H5'1 UD2"}="1H5* UD2"; $resn_exch{"H5'2 UD2"}="2H5* UD2"; $resn_exch{"HOA2 UD2"}="HO1A UD2";
$resn_exch{"HOB2 UD2"}="HO1B UD2"; $resn_exch{" H5' UD5"}="1H5* UD5"; $resn_exch{"H5'' UD5"}="2H5* UD5";
$resn_exch{" H4' UD5"}=" H4* UD5"; $resn_exch{" H3' UD5"}=" H3* UD5"; $resn_exch{" H2' UD5"}=" H2* UD5";
$resn_exch{" H1' UD5"}=" H1* UD5"; $resn_exch{"H121 UDA"}="1H12 UDA"; $resn_exch{"H122 UDA"}="2H12 UDA";
$resn_exch{"H171 UDA"}="1H17 UDA"; $resn_exch{"H172 UDA"}="2H17 UDA"; $resn_exch{" H81 UDA"}="1H8  UDA";
$resn_exch{" H82 UDA"}="2H8  UDA"; $resn_exch{" H83 UDA"}="3H8  UDA"; $resn_exch{"H191 UDA"}="1H19 UDA";
$resn_exch{"H192 UDA"}="2H19 UDA"; $resn_exch{"H193 UDA"}="3H19 UDA"; $resn_exch{" H1' UDH"}=" H1* UDH";
$resn_exch{" H2' UDH"}=" H2* UDH"; $resn_exch{"HO2' UDH"}="*HO2 UDH"; $resn_exch{" H3' UDH"}=" H3* UDH";
$resn_exch{" H4' UDH"}=" H4* UDH"; $resn_exch{"HO3' UDH"}="*HO3 UDH"; $resn_exch{"H5'1 UDH"}="1H5* UDH";
$resn_exch{"H5'2 UDH"}="2H5* UDH"; $resn_exch{"HOA2 UDH"}="2HOA UDH"; $resn_exch{"HOB3 UDH"}="3HOB UDH";
$resn_exch{"H1'1 UDH"}="1H1' UDH"; $resn_exch{"H1'2 UDH"}="2H1' UDH"; $resn_exch{"H2'1 UDH"}="1H2' UDH";
$resn_exch{"H2'2 UDH"}="2H2' UDH"; $resn_exch{"H3'1 UDH"}="1H3' UDH"; $resn_exch{"H3'2 UDH"}="2H3' UDH";
$resn_exch{"H4'1 UDH"}="1H4' UDH"; $resn_exch{"H4'2 UDH"}="2H4' UDH"; $resn_exch{" H5B UDH"}="1H5' UDH";
$resn_exch{" H5D UDH"}="2H5' UDH"; $resn_exch{"H6'1 UDH"}="1H6' UDH"; $resn_exch{"H6'2 UDH"}="2H6' UDH";
$resn_exch{"HN'1 UDH"}="1HN' UDH"; $resn_exch{"HN'2 UDH"}="2HN' UDH"; $resn_exch{"H6'1 UDM"}="1H6' UDM";
$resn_exch{"H6'2 UDM"}="2H6' UDM"; $resn_exch{"H8'1 UDM"}="1H8' UDM"; $resn_exch{"H8'2 UDM"}="2H8' UDM";
$resn_exch{"H8'3 UDM"}="3H8' UDM"; $resn_exch{" HB1 UDM"}="1HB  UDM"; $resn_exch{" HB2 UDM"}="2HB  UDM";
$resn_exch{" H1D UDM"}=" H1* UDM"; $resn_exch{" H2D UDM"}=" H2* UDM"; $resn_exch{" H3D UDM"}=" H3* UDM";
$resn_exch{" H4D UDM"}=" H4* UDM"; $resn_exch{"H5'1 UDM"}="1H5* UDM"; $resn_exch{"H5'2 UDM"}="2H5* UDM";
$resn_exch{" H1' UDP"}=" H1* UDP"; $resn_exch{" H2' UDP"}=" H2* UDP"; $resn_exch{"HO2' UDP"}="*HO2 UDP";
$resn_exch{" H3' UDP"}=" H3* UDP"; $resn_exch{" H4' UDP"}=" H4* UDP"; $resn_exch{"HO3' UDP"}="*HO3 UDP";
$resn_exch{"H5'1 UDP"}="1H5* UDP"; $resn_exch{"H5'2 UDP"}="2H5* UDP"; $resn_exch{"HOA2 UDP"}="2HOA UDP";
$resn_exch{"HOB2 UDP"}="2HOB UDP"; $resn_exch{"HOB3 UDP"}="3HOB UDP"; $resn_exch{"H111 UDT"}="1H11 UDT";
$resn_exch{"H112 UDT"}="2H11 UDT"; $resn_exch{"H113 UDT"}="3H11 UDT"; $resn_exch{"H101 UDT"}="1H10 UDT";
$resn_exch{"H102 UDT"}="2H10 UDT"; $resn_exch{" H91 UDT"}="1H9  UDT"; $resn_exch{" H92 UDT"}="2H9  UDT";
$resn_exch{" H81 UDT"}="1H8  UDT"; $resn_exch{" H82 UDT"}="2H8  UDT"; $resn_exch{" H71 UDT"}="1H7  UDT";
$resn_exch{" H72 UDT"}="2H7  UDT"; $resn_exch{" H61 UDT"}="1H6  UDT"; $resn_exch{" H62 UDT"}="2H6  UDT";
$resn_exch{" H51 UDT"}="1H5  UDT"; $resn_exch{" H52 UDT"}="2H5  UDT"; $resn_exch{" H41 UDT"}="1H4  UDT";
$resn_exch{" H42 UDT"}="2H4  UDT"; $resn_exch{" H31 UDT"}="1H3  UDT"; $resn_exch{" H32 UDT"}="2H3  UDT";
$resn_exch{" H21 UDT"}="1H2  UDT"; $resn_exch{" H22 UDT"}="2H2  UDT"; $resn_exch{"H2'1 UDX"}="1H2' UDX";
$resn_exch{"HO'2 UDX"}="2HO' UDX"; $resn_exch{"H3'1 UDX"}="1H3' UDX"; $resn_exch{"HO'3 UDX"}="3HO' UDX";
$resn_exch{"H4'1 UDX"}="1H4' UDX"; $resn_exch{"HO'4 UDX"}="4HO' UDX"; $resn_exch{"H5'1 UDX"}="1H5' UDX";
$resn_exch{"H5'2 UDX"}="2H5' UDX"; $resn_exch{"HOB2 UDX"}="2HOB UDX"; $resn_exch{"HOA2 UDX"}="2HOA UDX";
$resn_exch{"H5A1 UDX"}="1H5* UDX"; $resn_exch{"H5A2 UDX"}="2H5* UDX"; $resn_exch{"H4A1 UDX"}="1H4* UDX";
$resn_exch{"H3A1 UDX"}="1H3* UDX"; $resn_exch{"HOA3 UDX"}="3HO* UDX"; $resn_exch{"H2A1 UDX"}="1H2* UDX";
$resn_exch{"HOC2 UDX"}="2HO* UDX"; $resn_exch{"H1'1 UDX"}="1H1* UDX"; $resn_exch{" H61 UDX"}="1H6  UDX";
$resn_exch{" H51 UDX"}="1H5  UDX"; $resn_exch{" H1D UFG"}=" H1* UFG"; $resn_exch{" H2D UFG"}=" H2* UFG";
$resn_exch{"HO2' UFG"}="*HO2 UFG"; $resn_exch{" H3D UFG"}=" H3* UFG"; $resn_exch{" H4D UFG"}=" H4* UFG";
$resn_exch{"HO3' UFG"}="*HO3 UFG"; $resn_exch{"H5'1 UFG"}="1H5* UFG"; $resn_exch{"H5'2 UFG"}="2H5* UFG";
$resn_exch{"HOA2 UFG"}="2HOA UFG"; $resn_exch{"HOB2 UFG"}="2HOB UFG"; $resn_exch{"H6'1 UFG"}="1H6' UFG";
$resn_exch{"H6'2 UFG"}="2H6' UFG"; $resn_exch{"HO2A UFG"}="'HO2 UFG"; $resn_exch{"HO3A UFG"}="'HO3 UFG";
$resn_exch{"HO6' UFG"}="'HO6 UFG"; $resn_exch{" H1D UFM"}=" H1* UFM"; $resn_exch{" H2D UFM"}=" H2* UFM";
$resn_exch{"HO2' UFM"}="*HO2 UFM"; $resn_exch{" H3D UFM"}=" H3* UFM"; $resn_exch{" H4D UFM"}=" H4* UFM";
$resn_exch{"HO3' UFM"}="*HO3 UFM"; $resn_exch{"H5'1 UFM"}="1H5* UFM"; $resn_exch{"H5'2 UFM"}="2H5* UFM";
$resn_exch{"HOA2 UFM"}="2HOA UFM"; $resn_exch{"HOB2 UFM"}="2HOB UFM"; $resn_exch{"H6'1 UFM"}="1H6' UFM";
$resn_exch{"H6'2 UFM"}="2H6' UFM"; $resn_exch{"HO2A UFM"}="'HO2 UFM"; $resn_exch{"HO3A UFM"}="'HO3 UFM";
$resn_exch{"HO4' UFM"}="'HO4 UFM"; $resn_exch{"HO6' UFM"}="'HO6 UFM"; $resn_exch{" H1' UFP"}=" H1* UFP";
$resn_exch{"H2'1 UFP"}="1H2* UFP"; $resn_exch{"H2'2 UFP"}="2H2* UFP"; $resn_exch{" H3' UFP"}=" H3* UFP";
$resn_exch{" H4' UFP"}=" H4* UFP"; $resn_exch{"HO3' UFP"}="*HO3 UFP"; $resn_exch{"H5'1 UFP"}="1H5* UFP";
$resn_exch{"H5'2 UFP"}="2H5* UFP"; $resn_exch{"HOP2 UFP"}="2HOP UFP"; $resn_exch{"HOP3 UFP"}="3HOP UFP";
$resn_exch{"H5'1 UFR"}="1H5* UFR"; $resn_exch{"H5'2 UFR"}="2H5* UFR"; $resn_exch{" H4' UFR"}=" H4* UFR";
$resn_exch{" H3' UFR"}=" H3* UFR"; $resn_exch{"HO3' UFR"}="HO3* UFR"; $resn_exch{"H2'1 UFR"}="1H2* UFR";
$resn_exch{"H2'2 UFR"}="2H2* UFR"; $resn_exch{" H1' UFR"}=" H1* UFR"; $resn_exch{"H1'1 UGA"}="1H1' UGA";
$resn_exch{"H2'1 UGA"}="1H2' UGA"; $resn_exch{"HO'2 UGA"}="2HO' UGA"; $resn_exch{"H3'1 UGA"}="1H3' UGA";
$resn_exch{"HO'3 UGA"}="3HO' UGA"; $resn_exch{"H4'1 UGA"}="1H4' UGA"; $resn_exch{"HO'4 UGA"}="4HO' UGA";
$resn_exch{"H5'1 UGA"}="1H5' UGA"; $resn_exch{"HO'6 UGA"}="6HO' UGA"; $resn_exch{"HOB2 UGA"}="2HOB UGA";
$resn_exch{"HOA2 UGA"}="2HOA UGA"; $resn_exch{"H5A1 UGA"}="1H5* UGA"; $resn_exch{"H5'2 UGA"}="2H5* UGA";
$resn_exch{"H4A1 UGA"}="1H4* UGA"; $resn_exch{"H3A1 UGA"}="1H3* UGA"; $resn_exch{"HOA3 UGA"}="3HO* UGA";
$resn_exch{"H2A1 UGA"}="1H2* UGA"; $resn_exch{"HOC2 UGA"}="2HO* UGA"; $resn_exch{"H1A1 UGA"}="1H1* UGA";
$resn_exch{" H61 UGA"}="1H6  UGA"; $resn_exch{" H51 UGA"}="1H5  UGA"; $resn_exch{" H81 UHD"}="1H8  UHD";
$resn_exch{" H82 UHD"}="2H8  UHD"; $resn_exch{" H91 UHD"}="1H9  UHD"; $resn_exch{" H92 UHD"}="2H9  UHD";
$resn_exch{"H101 UHD"}="1H10 UHD"; $resn_exch{"H102 UHD"}="2H10 UHD"; $resn_exch{"H111 UHD"}="1H11 UHD";
$resn_exch{"H112 UHD"}="2H11 UHD"; $resn_exch{"H121 UHD"}="1H12 UHD"; $resn_exch{"H122 UHD"}="2H12 UHD";
$resn_exch{"H131 UHD"}="1H13 UHD"; $resn_exch{"H132 UHD"}="2H13 UHD"; $resn_exch{"H141 UHD"}="1H14 UHD";
$resn_exch{"H142 UHD"}="2H14 UHD"; $resn_exch{"H151 UHD"}="1H15 UHD"; $resn_exch{"H152 UHD"}="2H15 UHD";
$resn_exch{"H161 UHD"}="1H16 UHD"; $resn_exch{"H162 UHD"}="2H16 UHD"; $resn_exch{"H171 UHD"}="1H17 UHD";
$resn_exch{"H172 UHD"}="2H17 UHD"; $resn_exch{"H181 UHD"}="1H18 UHD"; $resn_exch{"H182 UHD"}="2H18 UHD";
$resn_exch{"H183 UHD"}="3H18 UHD"; $resn_exch{"HN11 UI1"}="1HN1 UI1"; $resn_exch{"HN12 UI1"}="2HN1 UI1";
$resn_exch{"H21A UI1"}="1H2  UI1"; $resn_exch{" H22 UI1"}="2H2  UI1"; $resn_exch{"H291 UI1"}="1H29 UI1";
$resn_exch{"H292 UI1"}="2H29 UI1"; $resn_exch{"H301 UI2"}="1H30 UI2"; $resn_exch{"H302 UI2"}="2H30 UI2";
$resn_exch{"H181 UI3"}="1H18 UI3"; $resn_exch{"H182 UI3"}="2H18 UI3"; $resn_exch{"H201 UI3"}="1H20 UI3";
$resn_exch{"H202 UI3"}="2H20 UI3"; $resn_exch{"H203 UI3"}="3H20 UI3"; $resn_exch{"H371 UI3"}="1H37 UI3";
$resn_exch{"H372 UI3"}="2H37 UI3"; $resn_exch{"H373 UI3"}="3H37 UI3"; $resn_exch{"HN11 UIB"}="1HN1 UIB";
$resn_exch{"HN12 UIB"}="2HN1 UIB"; $resn_exch{"H131 UIB"}="1H13 UIB"; $resn_exch{"H132 UIB"}="2H13 UIB";
$resn_exch{"H133 UIB"}="3H13 UIB"; $resn_exch{"H141 UIB"}="1H14 UIB"; $resn_exch{"H142 UIB"}="2H14 UIB";
$resn_exch{"H143 UIB"}="3H14 UIB"; $resn_exch{"H121 UIB"}="1H12 UIB"; $resn_exch{"H122 UIB"}="2H12 UIB";
$resn_exch{"H123 UIB"}="3H12 UIB"; $resn_exch{"H231 UIB"}="1H23 UIB"; $resn_exch{"H232 UIB"}="2H23 UIB";
$resn_exch{"H281 UIB"}="1H28 UIB"; $resn_exch{"H282 UIB"}="2H28 UIB"; $resn_exch{"H181 UIB"}="1H18 UIB";
$resn_exch{"H182 UIB"}="2H18 UIB"; $resn_exch{"H411 UIB"}="1H41 UIB"; $resn_exch{"H412 UIB"}="2H41 UIB";
$resn_exch{"H413 UIB"}="3H41 UIB"; $resn_exch{"H121 UIC"}="1H12 UIC"; $resn_exch{"H122 UIC"}="2H12 UIC";
$resn_exch{"H141 UIC"}="1H14 UIC"; $resn_exch{"H142 UIC"}="2H14 UIC"; $resn_exch{"H161 UIC"}="1H16 UIC";
$resn_exch{"H162 UIC"}="2H16 UIC"; $resn_exch{" H21 UIC"}="1H2  UIC"; $resn_exch{"H22A UIC"}="2H2  UIC";
$resn_exch{"H231 UIC"}="1H23 UIC"; $resn_exch{"H232 UIC"}="2H23 UIC"; $resn_exch{"H321 UIC"}="1H32 UIC";
$resn_exch{"H322 UIC"}="2H32 UIC"; $resn_exch{"H331 UIC"}="1H33 UIC"; $resn_exch{"H332 UIC"}="2H33 UIC";
$resn_exch{"H351 UIC"}="1H35 UIC"; $resn_exch{"H352 UIC"}="2H35 UIC"; $resn_exch{"H353 UIC"}="3H35 UIC";
$resn_exch{"H361 UIC"}="1H36 UIC"; $resn_exch{"H362 UIC"}="2H36 UIC"; $resn_exch{"H363 UIC"}="3H36 UIC";
$resn_exch{"H31A UIC"}="1H3  UIC"; $resn_exch{" H32 UIC"}="2H3  UIC"; $resn_exch{"HG21 UIN"}="1HG2 UIN";
$resn_exch{"HG22 UIN"}="2HG2 UIN"; $resn_exch{"HG31 UIN"}="1HG3 UIN"; $resn_exch{"HG32 UIN"}="2HG3 UIN";
$resn_exch{"HG33 UIN"}="3HG3 UIN"; $resn_exch{"HD11 UIN"}="1HD1 UIN"; $resn_exch{"HD12 UIN"}="2HD1 UIN";
$resn_exch{"HD21 UIN"}="1HD2 UIN"; $resn_exch{"HD22 UIN"}="2HD2 UIN"; $resn_exch{"HD31 UIN"}="1HD3 UIN";
$resn_exch{"HD32 UIN"}="2HD3 UIN"; $resn_exch{"HD41 UIN"}="1HD4 UIN"; $resn_exch{"HD42 UIN"}="2HD4 UIN";
$resn_exch{"HD51 UIN"}="1HD5 UIN"; $resn_exch{"HD52 UIN"}="2HD5 UIN"; $resn_exch{"HD61 UIN"}="1HD6 UIN";
$resn_exch{"HD62 UIN"}="2HD6 UIN"; $resn_exch{"HN21 UIP"}="1HN2 UIP"; $resn_exch{"HN22 UIP"}="2HN2 UIP";
$resn_exch{"H121 UIP"}="1H12 UIP"; $resn_exch{"H122 UIP"}="2H12 UIP"; $resn_exch{"H131 UIP"}="1H13 UIP";
$resn_exch{"H132 UIP"}="2H13 UIP"; $resn_exch{"H141 UIP"}="1H14 UIP"; $resn_exch{"H142 UIP"}="2H14 UIP";
$resn_exch{"H231 UIP"}="1H23 UIP"; $resn_exch{"H232 UIP"}="2H23 UIP"; $resn_exch{"H281 UIP"}="1H28 UIP";
$resn_exch{"H282 UIP"}="2H28 UIP"; $resn_exch{"H181 UIP"}="1H18 UIP"; $resn_exch{"H182 UIP"}="2H18 UIP";
$resn_exch{"H421 UIP"}="1H42 UIP"; $resn_exch{"H422 UIP"}="2H42 UIP"; $resn_exch{"H423 UIP"}="3H42 UIP";
$resn_exch{"HN21 UIQ"}="1HN2 UIQ"; $resn_exch{"HN22 UIQ"}="2HN2 UIQ"; $resn_exch{"H121 UIQ"}="1H12 UIQ";
$resn_exch{"H122 UIQ"}="2H12 UIQ"; $resn_exch{"H131 UIQ"}="1H13 UIQ"; $resn_exch{"H132 UIQ"}="2H13 UIQ";
$resn_exch{"H141 UIQ"}="1H14 UIQ"; $resn_exch{"H142 UIQ"}="2H14 UIQ"; $resn_exch{"H231 UIQ"}="1H23 UIQ";
$resn_exch{"H232 UIQ"}="2H23 UIQ"; $resn_exch{"H281 UIQ"}="1H28 UIQ"; $resn_exch{"H282 UIQ"}="2H28 UIQ";
$resn_exch{"H411 UIQ"}="1H41 UIQ"; $resn_exch{"H412 UIQ"}="2H41 UIQ"; $resn_exch{"H413 UIQ"}="3H41 UIQ";
$resn_exch{"H421 UIQ"}="1H42 UIQ"; $resn_exch{"H422 UIQ"}="2H42 UIQ"; $resn_exch{"H423 UIQ"}="3H42 UIQ";
$resn_exch{"HN11 UIR"}="1HN1 UIR"; $resn_exch{"HN12 UIR"}="2HN1 UIR"; $resn_exch{"H121 UIR"}="1H12 UIR";
$resn_exch{"H122 UIR"}="2H12 UIR"; $resn_exch{"H131 UIR"}="1H13 UIR"; $resn_exch{"H132 UIR"}="2H13 UIR";
$resn_exch{"H141 UIR"}="1H14 UIR"; $resn_exch{"H142 UIR"}="2H14 UIR"; $resn_exch{"H421 UIR"}="1H42 UIR";
$resn_exch{"H422 UIR"}="2H42 UIR"; $resn_exch{"H411 UIR"}="1H41 UIR"; $resn_exch{"H412 UIR"}="2H41 UIR";
$resn_exch{"H231 UIR"}="1H23 UIR"; $resn_exch{"H232 UIR"}="2H23 UIR"; $resn_exch{"H281 UIR"}="1H28 UIR";
$resn_exch{"H282 UIR"}="2H28 UIR"; $resn_exch{"HN21 UIZ"}="1HN2 UIZ"; $resn_exch{"HN22 UIZ"}="2HN2 UIZ";
$resn_exch{"H231 UIZ"}="1H23 UIZ"; $resn_exch{"H232 UIZ"}="2H23 UIZ"; $resn_exch{"H281 UIZ"}="1H28 UIZ";
$resn_exch{"H282 UIZ"}="2H28 UIZ"; $resn_exch{"H141 UIZ"}="1H14 UIZ"; $resn_exch{"H142 UIZ"}="2H14 UIZ";
$resn_exch{"H131 UIZ"}="1H13 UIZ"; $resn_exch{"H132 UIZ"}="2H13 UIZ"; $resn_exch{"H121 UIZ"}="1H12 UIZ";
$resn_exch{"H122 UIZ"}="2H12 UIZ"; $resn_exch{"H11A UIZ"}="1H1  UIZ"; $resn_exch{" H12 UIZ"}="2H1  UIZ";
$resn_exch{" H1  UKC"}="1H   UKC"; $resn_exch{" H2A UKC"}="2H   UKC"; $resn_exch{" H3A UKC"}="3H   UKC";
$resn_exch{" H61 UKC"}="1H6  UKC"; $resn_exch{" H62 UKC"}="2H6  UKC"; $resn_exch{" H63 UKC"}="3H6  UKC";
$resn_exch{"H151 UKC"}="1H15 UKC"; $resn_exch{"H152 UKC"}="2H15 UKC"; $resn_exch{"H161 UKC"}="1H16 UKC";
$resn_exch{"H162 UKC"}="2H16 UKC"; $resn_exch{"H163 UKC"}="3H16 UKC"; $resn_exch{" HN1 UKP"}="1HN  UKP";
$resn_exch{" HA1 UKP"}="1HA  UKP"; $resn_exch{"HCB1 UKP"}="1HCB UKP"; $resn_exch{"HCB2 UKP"}="2HCB UKP";
$resn_exch{"HD11 UKP"}="1HD1 UKP"; $resn_exch{" HZ1 UKP"}="1HZ  UKP"; $resn_exch{"HE21 UKP"}="1HE2 UKP";
$resn_exch{"HD21 UKP"}="1HD2 UKP"; $resn_exch{"HX11 UKP"}="1HX1 UKP"; $resn_exch{"HX12 UKP"}="2HX1 UKP";
$resn_exch{"HX21 UKP"}="1HX2 UKP"; $resn_exch{" H81 UKP"}="1H8  UKP"; $resn_exch{"H101 UKP"}="1H10 UKP";
$resn_exch{"H121 UKP"}="1H12 UKP"; $resn_exch{"H131 UKP"}="1H13 UKP"; $resn_exch{"H132 UKP"}="2H13 UKP";
$resn_exch{"H133 UKP"}="3H13 UKP"; $resn_exch{"H141 UKP"}="1H14 UKP"; $resn_exch{"H142 UKP"}="2H14 UKP";
$resn_exch{"H143 UKP"}="3H14 UKP"; $resn_exch{"H151 UKP"}="1H15 UKP"; $resn_exch{"H161 UKP"}="1H16 UKP";
$resn_exch{"H162 UKP"}="2H16 UKP"; $resn_exch{"H163 UKP"}="3H16 UKP"; $resn_exch{"H171 UKP"}="1H17 UKP";
$resn_exch{"H172 UKP"}="2H17 UKP"; $resn_exch{"H173 UKP"}="3H17 UKP"; $resn_exch{"H181 UKP"}="1H18 UKP";
$resn_exch{"H191 UKP"}="1H19 UKP"; $resn_exch{"H192 UKP"}="2H19 UKP"; $resn_exch{"H193 UKP"}="3H19 UKP";
$resn_exch{"H201 UKP"}="1H20 UKP"; $resn_exch{"H202 UKP"}="2H20 UKP"; $resn_exch{"H203 UKP"}="3H20 UKP";
$resn_exch{" H11 UKP"}="1H1  UKP"; $resn_exch{" H12 UKP"}="2H1  UKP"; $resn_exch{" H21 UKP"}="1H2  UKP";
$resn_exch{" H22 UKP"}="2H2  UKP"; $resn_exch{" H41 UKP"}="1H4  UKP"; $resn_exch{" H42 UKP"}="2H4  UKP";
$resn_exch{" H51 UKP"}="1H5  UKP"; $resn_exch{" H52 UKP"}="2H5  UKP"; $resn_exch{"H221 UKP"}="1H22 UKP";
$resn_exch{"H222 UKP"}="2H22 UKP"; $resn_exch{"H231 UKP"}="1H23 UKP"; $resn_exch{"H232 UKP"}="2H23 UKP";
$resn_exch{"H241 UKP"}="1H24 UKP"; $resn_exch{"H242 UKP"}="2H24 UKP"; $resn_exch{" H61 ULA"}="1H6  ULA";
$resn_exch{" H62 ULA"}="2H6  ULA"; $resn_exch{" H51 ULA"}="1H5  ULA"; $resn_exch{" H52 ULA"}="2H5  ULA";
$resn_exch{" H41 ULA"}="1H4  ULA"; $resn_exch{" H42 ULA"}="2H4  ULA"; $resn_exch{" H21 ULA"}="1H2  ULA";
$resn_exch{"H22A ULA"}="2H2  ULA"; $resn_exch{" H81 ULA"}="1H8  ULA"; $resn_exch{" H82 ULA"}="2H8  ULA";
$resn_exch{"H361 ULA"}="1H36 ULA"; $resn_exch{"H362 ULA"}="2H36 ULA"; $resn_exch{"H363 ULA"}="3H36 ULA";
$resn_exch{"H211 ULA"}="1H21 ULA"; $resn_exch{"H212 ULA"}="2H21 ULA"; $resn_exch{"H461 ULA"}="1H46 ULA";
$resn_exch{"H462 ULA"}="2H46 ULA"; $resn_exch{"H463 ULA"}="3H46 ULA"; $resn_exch{"H371 ULA"}="1H37 ULA";
$resn_exch{"H372 ULA"}="2H37 ULA"; $resn_exch{"H373 ULA"}="3H37 ULA"; $resn_exch{"H251 ULA"}="1H25 ULA";
$resn_exch{"H252 ULA"}="2H25 ULA"; $resn_exch{"H451 ULA"}="1H45 ULA"; $resn_exch{"H452 ULA"}="2H45 ULA";
$resn_exch{"H453 ULA"}="3H45 ULA"; $resn_exch{"H381 ULA"}="1H38 ULA"; $resn_exch{"H382 ULA"}="2H38 ULA";
$resn_exch{"H383 ULA"}="3H38 ULA"; $resn_exch{"H281 ULA"}="1H28 ULA"; $resn_exch{"H282 ULA"}="2H28 ULA";
$resn_exch{"H291 ULA"}="1H29 ULA"; $resn_exch{"H292 ULA"}="2H29 ULA"; $resn_exch{"H391 ULA"}="1H39 ULA";
$resn_exch{"H392 ULA"}="2H39 ULA"; $resn_exch{"H393 ULA"}="3H39 ULA"; $resn_exch{"H401 ULA"}="1H40 ULA";
$resn_exch{"H402 ULA"}="2H40 ULA"; $resn_exch{"H403 ULA"}="3H40 ULA"; $resn_exch{"H421 ULA"}="1H42 ULA";
$resn_exch{"H422 ULA"}="2H42 ULA"; $resn_exch{"H423 ULA"}="3H42 ULA"; $resn_exch{"H411 ULA"}="1H41 ULA";
$resn_exch{"H412 ULA"}="2H41 ULA"; $resn_exch{"HO21 ULA"}="1HO2 ULA"; $resn_exch{"H441 ULA"}="1H44 ULA";
$resn_exch{"H442 ULA"}="2H44 ULA"; $resn_exch{"H443 ULA"}="3H44 ULA"; $resn_exch{" H1' UM3"}=" H1* UM3";
$resn_exch{"H2'1 UM3"}="1H2* UM3"; $resn_exch{"H2'2 UM3"}="2H2* UM3"; $resn_exch{" H3' UM3"}=" H3* UM3";
$resn_exch{" H4' UM3"}=" H4* UM3"; $resn_exch{"H5'1 UM3"}="1H5* UM3"; $resn_exch{"H5'2 UM3"}="2H5* UM3";
$resn_exch{"HO5' UM3"}="*HO5 UM3"; $resn_exch{"HOP2 UM3"}="2HOP UM3"; $resn_exch{"HOP3 UM3"}="3HOP UM3";
$resn_exch{" H1B UMA"}=" H1* UMA"; $resn_exch{" H2B UMA"}=" H2* UMA"; $resn_exch{"HO2' UMA"}="*HO2 UMA";
$resn_exch{" H3B UMA"}=" H3* UMA"; $resn_exch{" H4B UMA"}=" H4* UMA"; $resn_exch{"HO3' UMA"}="*HO3 UMA";
$resn_exch{"H5'1 UMA"}="1H5* UMA"; $resn_exch{"H5'2 UMA"}="2H5* UMA"; $resn_exch{"HOA2 UMA"}="2HOA UMA";
$resn_exch{"HOB2 UMA"}="2HOB UMA"; $resn_exch{"H8'1 UMA"}="1H8' UMA"; $resn_exch{"H8'2 UMA"}="2H8' UMA";
$resn_exch{"H8'3 UMA"}="3H8' UMA"; $resn_exch{"H6'1 UMA"}="1H6' UMA"; $resn_exch{"H6'2 UMA"}="2H6' UMA";
$resn_exch{"H201 UMA"}="1H20 UMA"; $resn_exch{"H202 UMA"}="2H20 UMA"; $resn_exch{"H203 UMA"}="3H20 UMA";
$resn_exch{"H231 UMA"}="1H23 UMA"; $resn_exch{"H232 UMA"}="2H23 UMA"; $resn_exch{"H233 UMA"}="3H23 UMA";
$resn_exch{" H5' UMF"}=" H5* UMF"; $resn_exch{"H5'1 UMF"}="1H5* UMF"; $resn_exch{"H5'2 UMF"}="2H5* UMF";
$resn_exch{" H4' UMF"}=" H4* UMF"; $resn_exch{" H3' UMF"}=" H3* UMF"; $resn_exch{" H2' UMF"}=" H2* UMF";
$resn_exch{" H1' UMF"}=" H1* UMF"; $resn_exch{"H101 UMG"}="1H10 UMG"; $resn_exch{"H102 UMG"}="2H10 UMG";
$resn_exch{"H103 UMG"}="3H10 UMG"; $resn_exch{"H161 UMG"}="1H16 UMG"; $resn_exch{"H162 UMG"}="2H16 UMG";
$resn_exch{"H181 UMG"}="1H18 UMG"; $resn_exch{"H182 UMG"}="2H18 UMG"; $resn_exch{"H183 UMG"}="3H18 UMG";
$resn_exch{"H241 UMG"}="1H24 UMG"; $resn_exch{"H242 UMG"}="2H24 UMG"; $resn_exch{"H261 UMG"}="1H26 UMG";
$resn_exch{"H262 UMG"}="2H26 UMG"; $resn_exch{"H263 UMG"}="3H26 UMG"; $resn_exch{"H321 UMG"}="1H32 UMG";
$resn_exch{"H322 UMG"}="2H32 UMG"; $resn_exch{"H341 UMG"}="1H34 UMG"; $resn_exch{"H342 UMG"}="2H34 UMG";
$resn_exch{"H343 UMG"}="3H34 UMG"; $resn_exch{" H1' UMP"}=" H1* UMP"; $resn_exch{" H2' UMP"}="1H2* UMP";
$resn_exch{"H2'' UMP"}="2H2* UMP"; $resn_exch{" H3' UMP"}=" H3* UMP"; $resn_exch{" H4' UMP"}=" H4* UMP";
$resn_exch{"HO3' UMP"}="*HO3 UMP"; $resn_exch{" H5' UMP"}="1H5* UMP"; $resn_exch{"H5'' UMP"}="2H5* UMP";
$resn_exch{"HOP2 UMP"}="2HOP UMP"; $resn_exch{"HOP3 UMP"}="3HOP UMP"; $resn_exch{" H21 UMQ"}="1H2  UMQ";
$resn_exch{" H41 UMQ"}="1H4  UMQ"; $resn_exch{" H51 UMQ"}="1H5  UMQ"; $resn_exch{" H61 UMQ"}="1H6  UMQ";
$resn_exch{" H62 UMQ"}="2H6  UMQ"; $resn_exch{"HO21 UMQ"}="1HO2 UMQ"; $resn_exch{"HO41 UMQ"}="1HO4 UMQ";
$resn_exch{" H11 UMQ"}="1H1  UMQ"; $resn_exch{" H31 UMQ"}="1H3  UMQ"; $resn_exch{"HO31 UMQ"}="1HO3 UMQ";
$resn_exch{"HO61 UMQ"}="1HO6 UMQ"; $resn_exch{"H1'1 UMQ"}="1H1' UMQ"; $resn_exch{"H2'1 UMQ"}="1H2' UMQ";
$resn_exch{"H3'1 UMQ"}="1H3' UMQ"; $resn_exch{"H4'1 UMQ"}="1H4' UMQ"; $resn_exch{"H5'1 UMQ"}="1H5' UMQ";
$resn_exch{"H6'1 UMQ"}="1H6' UMQ"; $resn_exch{"H6'2 UMQ"}="2H6' UMQ"; $resn_exch{"HO3' UMQ"}="'HO3 UMQ";
$resn_exch{"HO6' UMQ"}="'HO6 UMQ"; $resn_exch{"HO2' UMQ"}="'HO2 UMQ"; $resn_exch{" HA1 UMQ"}="1HA  UMQ";
$resn_exch{" HA2 UMQ"}="2HA  UMQ"; $resn_exch{" HB1 UMQ"}="1HB  UMQ"; $resn_exch{" HB2 UMQ"}="2HB  UMQ";
$resn_exch{" HC1 UMQ"}="1HC  UMQ"; $resn_exch{" HC2 UMQ"}="2HC  UMQ"; $resn_exch{" HD1 UMQ"}="1HD  UMQ";
$resn_exch{" HD2 UMQ"}="2HD  UMQ"; $resn_exch{" HF1 UMQ"}="1HF  UMQ"; $resn_exch{" HF2 UMQ"}="2HF  UMQ";
$resn_exch{" HG1 UMQ"}="1HG  UMQ"; $resn_exch{" HG2 UMQ"}="2HG  UMQ"; $resn_exch{" HH1 UMQ"}="1HH  UMQ";
$resn_exch{" HH2 UMQ"}="2HH  UMQ"; $resn_exch{" HI1 UMQ"}="1HI  UMQ"; $resn_exch{" HI2 UMQ"}="2HI  UMQ";
$resn_exch{" HJ1 UMQ"}="1HJ  UMQ"; $resn_exch{" HJ2 UMQ"}="2HJ  UMQ"; $resn_exch{" HK1 UMQ"}="1HK  UMQ";
$resn_exch{" HK2 UMQ"}="2HK  UMQ"; $resn_exch{" HL1 UMQ"}="1HL  UMQ"; $resn_exch{" HL2 UMQ"}="2HL  UMQ";
$resn_exch{" HL3 UMQ"}="3HL  UMQ"; $resn_exch{"HOP3 UMS"}="3HOP UMS"; $resn_exch{"HOP2 UMS"}="2HOP UMS";
$resn_exch{"H5'2 UMS"}="2H5' UMS"; $resn_exch{"HO3' UMS"}="H3T  UMS"; $resn_exch{"HA'2 UMS"}="2HA' UMS";
$resn_exch{"HA'3 UMS"}="3HA' UMS"; $resn_exch{"H1C1 UN1"}="1H1C UN1"; $resn_exch{"H1C2 UN1"}="2H1C UN1";
$resn_exch{"H5C1 UN1"}="1H5C UN1"; $resn_exch{"H5C2 UN1"}="2H5C UN1"; $resn_exch{"H6C1 UN1"}="1H6C UN1";
$resn_exch{"H6C2 UN1"}="2H6C UN1"; $resn_exch{"H4N1 UN1"}="1H4N UN1"; $resn_exch{"H4N2 UN1"}="2H4N UN1";
$resn_exch{" HN1 UN2"}="1HN  UN2"; $resn_exch{" HN2 UN2"}="2HN  UN2"; $resn_exch{"HBC1 UN2"}="1HBC UN2";
$resn_exch{"HBC2 UN2"}="2HBC UN2"; $resn_exch{"HGC1 UN2"}="1HGC UN2"; $resn_exch{" H41 UN3"}="1H4  UN3";
$resn_exch{" H42 UN3"}="2H4  UN3"; $resn_exch{" H43 UN3"}="3H4  UN3"; $resn_exch{" H4' UN4"}=" H4* UN4";
$resn_exch{"H3'1 UN4"}="1H3* UN4"; $resn_exch{"H3'2 UN4"}="2H3* UN4"; $resn_exch{"H2'1 UN4"}="1H2* UN4";
$resn_exch{"H2'2 UN4"}="2H2* UN4"; $resn_exch{"H2N1 UN4"}="1H2N UN4"; $resn_exch{"H2N2 UN4"}="2H2N UN4";
$resn_exch{"H601 UN4"}="1H60 UN4"; $resn_exch{"H602 UN4"}="2H60 UN4"; $resn_exch{"H201 UN5"}="1H20 UN5";
$resn_exch{"H202 UN5"}="2H20 UN5"; $resn_exch{"H203 UN5"}="3H20 UN5"; $resn_exch{"H111 UN5"}="1H11 UN5";
$resn_exch{"H112 UN5"}="2H11 UN5"; $resn_exch{" H71 UN5"}="1H7  UN5"; $resn_exch{" H72 UN5"}="2H7  UN5";
$resn_exch{"H121 UN5"}="1H12 UN5"; $resn_exch{"H122 UN5"}="2H12 UN5"; $resn_exch{"H131 UN5"}="1H13 UN5";
$resn_exch{"H132 UN5"}="2H13 UN5"; $resn_exch{" H71 UN6"}="1H7  UN6"; $resn_exch{" H72 UN6"}="2H7  UN6";
$resn_exch{" H91 UN6"}="1H9  UN6"; $resn_exch{" H92 UN6"}="2H9  UN6"; $resn_exch{"H101 UN6"}="1H10 UN6";
$resn_exch{"H102 UN6"}="2H10 UN6"; $resn_exch{" H71 UN7"}="1H7  UN7"; $resn_exch{" H72 UN7"}="2H7  UN7";
$resn_exch{"H191 UN7"}="1H19 UN7"; $resn_exch{"H192 UN7"}="2H19 UN7"; $resn_exch{"H281 UN7"}="1H28 UN7";
$resn_exch{"H282 UN7"}="2H28 UN7"; $resn_exch{"H283 UN7"}="3H28 UN7"; $resn_exch{"H221 UN7"}="1H22 UN7";
$resn_exch{"H222 UN7"}="2H22 UN7"; $resn_exch{"H251 UN7"}="1H25 UN7"; $resn_exch{"H252 UN7"}="2H25 UN7";
$resn_exch{" H41 UN8"}="1H4  UN8"; $resn_exch{" H42 UN8"}="2H4  UN8"; $resn_exch{" H43 UN8"}="3H4  UN8";
$resn_exch{"H161 UN8"}="1H16 UN8"; $resn_exch{"H162 UN8"}="2H16 UN8"; $resn_exch{"H591 UN8"}="1H59 UN8";
$resn_exch{"H592 UN8"}="2H59 UN8"; $resn_exch{"H101 UN8"}="1H10 UN8"; $resn_exch{"H102 UN8"}="2H10 UN8";
$resn_exch{"H111 UN8"}="1H11 UN8"; $resn_exch{"H112 UN8"}="2H11 UN8"; $resn_exch{"H121 UN8"}="1H12 UN8";
$resn_exch{"H122 UN8"}="2H12 UN8"; $resn_exch{"H123 UN8"}="3H12 UN8"; $resn_exch{"H171 UN8"}="1H17 UN8";
$resn_exch{"H172 UN8"}="2H17 UN8"; $resn_exch{"H151 UN9"}="1H15 UN9"; $resn_exch{"H152 UN9"}="2H15 UN9";
$resn_exch{"H113 UNA"}="3H11 UNA"; $resn_exch{"H112 UNA"}="2H11 UNA"; $resn_exch{"H111 UNA"}="1H11 UNA";
$resn_exch{"H102 UNA"}="2H10 UNA"; $resn_exch{"H101 UNA"}="1H10 UNA"; $resn_exch{" H92 UNA"}="2H9  UNA";
$resn_exch{" H91 UNA"}="1H9  UNA"; $resn_exch{" H82 UNA"}="2H8  UNA"; $resn_exch{" H81 UNA"}="1H8  UNA";
$resn_exch{" H72 UNA"}="2H7  UNA"; $resn_exch{" H71 UNA"}="1H7  UNA"; $resn_exch{" H62 UNA"}="2H6  UNA";
$resn_exch{" H61 UNA"}="1H6  UNA"; $resn_exch{" H52 UNA"}="2H5  UNA"; $resn_exch{" H51 UNA"}="1H5  UNA";
$resn_exch{" H42 UNA"}="2H4  UNA"; $resn_exch{" H41 UNA"}="1H4  UNA"; $resn_exch{" H32 UNA"}="2H3  UNA";
$resn_exch{" H31 UNA"}="1H3  UNA"; $resn_exch{" H22 UNA"}="2H2  UNA"; $resn_exch{" H21 UNA"}="1H2  UNA";
$resn_exch{"H221 UNB"}="1H22 UNB"; $resn_exch{"H222 UNB"}="2H22 UNB"; $resn_exch{"H231 UNB"}="1H23 UNB";
$resn_exch{"H232 UNB"}="2H23 UNB"; $resn_exch{"H233 UNB"}="3H23 UNB"; $resn_exch{"H271 UNB"}="1H27 UNB";
$resn_exch{"H272 UNB"}="2H27 UNB"; $resn_exch{"H351 UNB"}="1H35 UNB"; $resn_exch{"H352 UNB"}="2H35 UNB";
$resn_exch{"HN51 UNC"}="1HN5 UNC"; $resn_exch{"HN52 UNC"}="2HN5 UNC"; $resn_exch{" H11 UND"}="1H1  UND";
$resn_exch{" H12 UND"}="2H1  UND"; $resn_exch{" H13 UND"}="3H1  UND"; $resn_exch{" H21 UND"}="1H2  UND";
$resn_exch{" H22 UND"}="2H2  UND"; $resn_exch{" H31 UND"}="1H3  UND"; $resn_exch{" H32 UND"}="2H3  UND";
$resn_exch{" H41 UND"}="1H4  UND"; $resn_exch{" H42 UND"}="2H4  UND"; $resn_exch{" H51 UND"}="1H5  UND";
$resn_exch{" H52 UND"}="2H5  UND"; $resn_exch{" H61 UND"}="1H6  UND"; $resn_exch{" H62 UND"}="2H6  UND";
$resn_exch{" H71 UND"}="1H7  UND"; $resn_exch{" H72 UND"}="2H7  UND"; $resn_exch{" H81 UND"}="1H8  UND";
$resn_exch{" H82 UND"}="2H8  UND"; $resn_exch{" H91 UND"}="1H9  UND"; $resn_exch{" H92 UND"}="2H9  UND";
$resn_exch{"H101 UND"}="1H10 UND"; $resn_exch{"H102 UND"}="2H10 UND"; $resn_exch{"H111 UND"}="1H11 UND";
$resn_exch{"H112 UND"}="2H11 UND"; $resn_exch{"H113 UND"}="3H11 UND"; $resn_exch{"H4C1 UNG"}="1H4C UNG";
$resn_exch{" H41 UNH"}="1H4  UNH"; $resn_exch{"H42A UNH"}="2H4  UNH"; $resn_exch{" H61 UNH"}="1H6  UNH";
$resn_exch{" H62 UNH"}="2H6  UNH"; $resn_exch{" H63 UNH"}="3H6  UNH"; $resn_exch{" H71 UNH"}="1H7  UNH";
$resn_exch{" H72 UNH"}="2H7  UNH"; $resn_exch{" H73 UNH"}="3H7  UNH"; $resn_exch{"H111 UNH"}="1H11 UNH";
$resn_exch{"H112 UNH"}="2H11 UNH"; $resn_exch{"H121 UNH"}="1H12 UNH"; $resn_exch{"H122 UNH"}="2H12 UNH";
$resn_exch{"H131 UNH"}="1H13 UNH"; $resn_exch{"H132 UNH"}="2H13 UNH"; $resn_exch{"H141 UNH"}="1H14 UNH";
$resn_exch{"H142 UNH"}="2H14 UNH"; $resn_exch{"H151 UNH"}="1H15 UNH"; $resn_exch{"H152 UNH"}="2H15 UNH";
$resn_exch{"H201 UNH"}="1H20 UNH"; $resn_exch{"H202 UNH"}="2H20 UNH"; $resn_exch{"H221 UNH"}="1H22 UNH";
$resn_exch{"H222 UNH"}="2H22 UNH"; $resn_exch{"H223 UNH"}="3H22 UNH"; $resn_exch{"H231 UNH"}="1H23 UNH";
$resn_exch{"H232 UNH"}="2H23 UNH"; $resn_exch{"H233 UNH"}="3H23 UNH"; $resn_exch{"H281 UNH"}="1H28 UNH";
$resn_exch{"H282 UNH"}="2H28 UNH"; $resn_exch{"H291 UNH"}="1H29 UNH"; $resn_exch{"H292 UNH"}="2H29 UNH";
$resn_exch{"H301 UNH"}="1H30 UNH"; $resn_exch{"H302 UNH"}="2H30 UNH"; $resn_exch{"H303 UNH"}="3H30 UNH";
$resn_exch{"H351 UNH"}="1H35 UNH"; $resn_exch{"H352 UNH"}="2H35 UNH"; $resn_exch{"H491 UNH"}="1H49 UNH";
$resn_exch{"H492 UNH"}="2H49 UNH"; $resn_exch{"HG31 UNI"}="1HG3 UNI"; $resn_exch{"HG32 UNI"}="2HG3 UNI";
$resn_exch{"HG41 UNI"}="1HG4 UNI"; $resn_exch{"HG42 UNI"}="2HG4 UNI"; $resn_exch{"HD11 UNI"}="1HD1 UNI";
$resn_exch{"HD12 UNI"}="2HD1 UNI"; $resn_exch{"HD21 UNI"}="1HD2 UNI"; $resn_exch{"HD22 UNI"}="2HD2 UNI";
$resn_exch{"HD31 UNI"}="1HD3 UNI"; $resn_exch{"HD32 UNI"}="2HD3 UNI"; $resn_exch{"HD41 UNI"}="1HD4 UNI";
$resn_exch{"HD42 UNI"}="2HD4 UNI"; $resn_exch{"HD51 UNI"}="1HD5 UNI"; $resn_exch{"HD52 UNI"}="2HD5 UNI";
$resn_exch{"HD61 UNI"}="1HD6 UNI"; $resn_exch{"HD62 UNI"}="2HD6 UNI"; $resn_exch{" H   UNK"}=" HN  UNK";
$resn_exch{" H2  UNK"}="2HN  UNK"; $resn_exch{" HB1 UNK"}="1HB  UNK"; $resn_exch{" HB2 UNK"}="2HB  UNK";
$resn_exch{" HG1 UNK"}="1HG  UNK"; $resn_exch{" HG2 UNK"}="2HG  UNK"; $resn_exch{" HG3 UNK"}="3HG  UNK";
$resn_exch{"H311 UNN"}="1H31 UNN"; $resn_exch{"H312 UNN"}="2H31 UNN"; $resn_exch{"H313 UNN"}="3H31 UNN";
$resn_exch{" H1' UP1"}=" H1* UP1"; $resn_exch{" H2' UP1"}=" H2* UP1"; $resn_exch{" H3' UP1"}=" H3* UP1";
$resn_exch{" H4' UP1"}=" H4* UP1"; $resn_exch{"H5'1 UP1"}="1H5* UP1"; $resn_exch{"H5'2 UP1"}="2H5* UP1";
$resn_exch{" H61 UP1"}="1H6  UP1"; $resn_exch{" H62 UP1"}="2H6  UP1"; $resn_exch{" H8A UP5"}=" AH8 UP5";
$resn_exch{"H61A UP5"}="AH61 UP5"; $resn_exch{"H62A UP5"}="AH62 UP5"; $resn_exch{" H2A UP5"}=" AH2 UP5";
$resn_exch{"H51A UP5"}="AH51 UP5"; $resn_exch{"H52A UP5"}="AH52 UP5"; $resn_exch{" H4F UP5"}="AH4* UP5";
$resn_exch{" H3F UP5"}="AH3* UP5"; $resn_exch{"HO3A UP5"}="AHO3 UP5"; $resn_exch{" H2F UP5"}="AH2* UP5";
$resn_exch{"HO2A UP5"}="AHO2 UP5"; $resn_exch{" H1F UP5"}="AH1* UP5"; $resn_exch{"HOA2 UP5"}="2HOA UP5";
$resn_exch{"HOB2 UP5"}="2HOB UP5"; $resn_exch{"HOG2 UP5"}="2HOG UP5"; $resn_exch{"HOD2 UP5"}="2HOD UP5";
$resn_exch{"HOE2 UP5"}="2HOE UP5"; $resn_exch{"H51U UP5"}="UH51 UP5"; $resn_exch{"H52U UP5"}="UH52 UP5";
$resn_exch{" H4J UP5"}="UH4* UP5"; $resn_exch{" H3J UP5"}="UH3* UP5"; $resn_exch{"HO3' UP5"}="*HO3 UP5";
$resn_exch{" H2J UP5"}="UH2* UP5"; $resn_exch{"HO2' UP5"}="*HO2 UP5"; $resn_exch{" H1J UP5"}="UH1* UP5";
$resn_exch{" H3U UP5"}=" UH3 UP5"; $resn_exch{" H5U UP5"}=" UH5 UP5"; $resn_exch{" H6U UP5"}=" UH6 UP5";
$resn_exch{" H1' UP6"}=" H1* UP6"; $resn_exch{" H2' UP6"}=" H2* UP6"; $resn_exch{" H3' UP6"}=" H3* UP6";
$resn_exch{" H4' UP6"}=" H4* UP6"; $resn_exch{"HO2' UP6"}="*HO2 UP6"; $resn_exch{"HO3' UP6"}="*HO3 UP6";
$resn_exch{"H5'1 UP6"}="1H5* UP6"; $resn_exch{"H5'2 UP6"}="2H5* UP6"; $resn_exch{"HOP2 UP6"}="2HOP UP6";
$resn_exch{"HOP3 UP6"}="3HOP UP6"; $resn_exch{"H5U1 UPA"}="1H5U UPA"; $resn_exch{"H5U2 UPA"}="2H5U UPA";
$resn_exch{"HOU5 UPA"}="5HOU UPA"; $resn_exch{"H4U' UPA"}="*H4U UPA"; $resn_exch{"H3U' UPA"}="*H3U UPA";
$resn_exch{"HOU3 UPA"}="3HOU UPA"; $resn_exch{"H2U' UPA"}="*H2U UPA"; $resn_exch{"H1U' UPA"}="*H1U UPA";
$resn_exch{"HOP2 UPA"}="2HOP UPA"; $resn_exch{"H5A1 UPA"}="1H5A UPA"; $resn_exch{"H5A2 UPA"}="2H5A UPA";
$resn_exch{"H4A' UPA"}="*H4A UPA"; $resn_exch{"H3A' UPA"}="*H3A UPA"; $resn_exch{"HOA3 UPA"}="3HOA UPA";
$resn_exch{"H2A' UPA"}="*H2A UPA"; $resn_exch{"HOA2 UPA"}="2HOA UPA"; $resn_exch{"H1A' UPA"}="*H1A UPA";
$resn_exch{"H6A1 UPA"}="1H6A UPA"; $resn_exch{"H6A2 UPA"}="2H6A UPA"; $resn_exch{" H1D UPF"}=" H1* UPF";
$resn_exch{" H2D UPF"}=" H2* UPF"; $resn_exch{"HO2' UPF"}="*HO2 UPF"; $resn_exch{" H3D UPF"}=" H3* UPF";
$resn_exch{" H4D UPF"}=" H4* UPF"; $resn_exch{"HO3' UPF"}="*HO3 UPF"; $resn_exch{"H5'1 UPF"}="1H5* UPF";
$resn_exch{"H5'2 UPF"}="2H5* UPF"; $resn_exch{"HOA2 UPF"}="2HOA UPF"; $resn_exch{"HOB2 UPF"}="2HOB UPF";
$resn_exch{"H6'1 UPF"}="1H6' UPF"; $resn_exch{"H6'2 UPF"}="2H6' UPF"; $resn_exch{"HO3A UPF"}="'HO3 UPF";
$resn_exch{"HO4' UPF"}="'HO4 UPF"; $resn_exch{"HO6' UPF"}="'HO6 UPF"; $resn_exch{" H1C UPG"}=" H1* UPG";
$resn_exch{" H2C UPG"}=" H2* UPG"; $resn_exch{"HO2C UPG"}="*HO2 UPG"; $resn_exch{" H3C UPG"}=" H3* UPG";
$resn_exch{" H4C UPG"}=" H4* UPG"; $resn_exch{"HO3C UPG"}="*HO3 UPG"; $resn_exch{"H5C1 UPG"}="1H5* UPG";
$resn_exch{"H5C2 UPG"}="2H5* UPG"; $resn_exch{"HOA2 UPG"}="2HOA UPG"; $resn_exch{"HOB2 UPG"}="2HOB UPG";
$resn_exch{"H6'1 UPG"}="1H6' UPG"; $resn_exch{"H6'2 UPG"}="2H6' UPG"; $resn_exch{"HO2' UPG"}="'HO2 UPG";
$resn_exch{"HO3' UPG"}="'HO3 UPG"; $resn_exch{"HO4' UPG"}="'HO4 UPG"; $resn_exch{"HO6' UPG"}="'HO6 UPG";
$resn_exch{" H11 UPL"}="1H1  UPL"; $resn_exch{" H12 UPL"}="2H1  UPL"; $resn_exch{" H13 UPL"}="3H1  UPL";
$resn_exch{" H21 UPL"}="1H2  UPL"; $resn_exch{" H22 UPL"}="2H2  UPL"; $resn_exch{" H31 UPL"}="1H3  UPL";
$resn_exch{" H32 UPL"}="2H3  UPL"; $resn_exch{" H41 UPL"}="1H4  UPL"; $resn_exch{" H42 UPL"}="2H4  UPL";
$resn_exch{" H51 UPL"}="1H5  UPL"; $resn_exch{" H52 UPL"}="2H5  UPL"; $resn_exch{" H61 UPL"}="1H6  UPL";
$resn_exch{" H62 UPL"}="2H6  UPL"; $resn_exch{" H71 UPL"}="1H7  UPL"; $resn_exch{" H72 UPL"}="2H7  UPL";
$resn_exch{" H81 UPL"}="1H8  UPL"; $resn_exch{" H82 UPL"}="2H8  UPL"; $resn_exch{" H91 UPL"}="1H9  UPL";
$resn_exch{" H92 UPL"}="2H9  UPL"; $resn_exch{"H101 UPL"}="1H10 UPL"; $resn_exch{"H102 UPL"}="2H10 UPL";
$resn_exch{"H111 UPL"}="1H11 UPL"; $resn_exch{"H112 UPL"}="2H11 UPL"; $resn_exch{"H121 UPL"}="1H12 UPL";
$resn_exch{"H131 UPL"}="1H13 UPL"; $resn_exch{"H132 UPL"}="2H13 UPL"; $resn_exch{"H141 UPL"}="1H14 UPL";
$resn_exch{"H142 UPL"}="2H14 UPL"; $resn_exch{"H151 UPL"}="1H15 UPL"; $resn_exch{"H152 UPL"}="2H15 UPL";
$resn_exch{"H161 UPL"}="1H16 UPL"; $resn_exch{"H162 UPL"}="2H16 UPL"; $resn_exch{"H171 UPL"}="1H17 UPL";
$resn_exch{"H172 UPL"}="2H17 UPL"; $resn_exch{"H181 UPL"}="1H18 UPL"; $resn_exch{"H182 UPL"}="2H18 UPL";
$resn_exch{"H191 UPL"}="1H19 UPL"; $resn_exch{"H192 UPL"}="2H19 UPL"; $resn_exch{"H201 UPL"}="1H20 UPL";
$resn_exch{"H202 UPL"}="2H20 UPL"; $resn_exch{"H211 UPL"}="1H21 UPL"; $resn_exch{"H212 UPL"}="2H21 UPL";
$resn_exch{"H221 UPL"}="1H22 UPL"; $resn_exch{"H222 UPL"}="2H22 UPL"; $resn_exch{"H231 UPL"}="1H23 UPL";
$resn_exch{"H232 UPL"}="2H23 UPL"; $resn_exch{"H241 UPL"}="1H24 UPL"; $resn_exch{"H242 UPL"}="2H24 UPL";
$resn_exch{"H251 UPL"}="1H25 UPL"; $resn_exch{"H252 UPL"}="2H25 UPL"; $resn_exch{"H253 UPL"}="3H25 UPL";
$resn_exch{"H261 UPL"}="1H26 UPL"; $resn_exch{"H262 UPL"}="2H26 UPL"; $resn_exch{"H271 UPL"}="1H27 UPL";
$resn_exch{"H272 UPL"}="2H27 UPL"; $resn_exch{"H281 UPL"}="1H28 UPL"; $resn_exch{"H282 UPL"}="2H28 UPL";
$resn_exch{"H291 UPL"}="1H29 UPL"; $resn_exch{"H292 UPL"}="2H29 UPL"; $resn_exch{"H301 UPL"}="1H30 UPL";
$resn_exch{"H302 UPL"}="2H30 UPL"; $resn_exch{"H311 UPL"}="1H31 UPL"; $resn_exch{"H312 UPL"}="2H31 UPL";
$resn_exch{"H321 UPL"}="1H32 UPL"; $resn_exch{"H322 UPL"}="2H32 UPL"; $resn_exch{"H331 UPL"}="1H33 UPL";
$resn_exch{"H332 UPL"}="2H33 UPL"; $resn_exch{"H341 UPL"}="1H34 UPL"; $resn_exch{"H342 UPL"}="2H34 UPL";
$resn_exch{"H343 UPL"}="3H34 UPL"; $resn_exch{"HOA2 UPM"}="2HOA UPM"; $resn_exch{"HOB2 UPM"}="2HOB UPM";
$resn_exch{" H1' UPM"}=" H1* UPM"; $resn_exch{" H2' UPM"}=" H2* UPM"; $resn_exch{"HO'2 UPM"}="2HO* UPM";
$resn_exch{" H3' UPM"}=" H3* UPM"; $resn_exch{"HO'3 UPM"}="3HO* UPM"; $resn_exch{" H4' UPM"}=" H4* UPM";
$resn_exch{"H5'1 UPM"}="1H5* UPM"; $resn_exch{"H5'2 UPM"}="2H5* UPM"; $resn_exch{"H201 UPM"}="1H20 UPM";
$resn_exch{"H202 UPM"}="2H20 UPM"; $resn_exch{"H203 UPM"}="3H20 UPM"; $resn_exch{"HND2 UPM"}="2HND UPM";
$resn_exch{"H8D1 UPM"}="1H8D UPM"; $resn_exch{"H8D2 UPM"}="2H8D UPM"; $resn_exch{"HOD4 UPM"}="4HOD UPM";
$resn_exch{"H6D1 UPM"}="1H6D UPM"; $resn_exch{"H6D2 UPM"}="2H6D UPM"; $resn_exch{"HOD6 UPM"}="6HOD UPM";
$resn_exch{" H1' UPP"}=" H1* UPP"; $resn_exch{" H2B UPP"}=" H2* UPP"; $resn_exch{"HO2' UPP"}="*HO2 UPP";
$resn_exch{" H3B UPP"}=" H3* UPP"; $resn_exch{" H4B UPP"}=" H4* UPP"; $resn_exch{"HO3' UPP"}="*HO3 UPP";
$resn_exch{"H5'1 UPP"}="1H5* UPP"; $resn_exch{"H5'2 UPP"}="2H5* UPP"; $resn_exch{"HOA2 UPP"}="2HOA UPP";
$resn_exch{"HOB2 UPP"}="2HOB UPP"; $resn_exch{"H5'1 UPR"}="1H5* UPR"; $resn_exch{"H5'2 UPR"}="2H5* UPR";
$resn_exch{" H4' UPR"}=" H4* UPR"; $resn_exch{" H3' UPR"}=" H3* UPR"; $resn_exch{"H2'1 UPR"}="1H2* UPR";
$resn_exch{"H2'2 UPR"}="2H2* UPR"; $resn_exch{" H1' UPR"}=" H1* UPR"; $resn_exch{"H1'A UPU"}="H1*A UPU";
$resn_exch{"H2AC UPU"}="CH2A UPU"; $resn_exch{"H2AO UPU"}="OH2A UPU"; $resn_exch{"H3AC UPU"}="CH3A UPU";
$resn_exch{"H3AO UPU"}="OH3A UPU"; $resn_exch{"H4'A UPU"}="H4*A UPU"; $resn_exch{"H'A1 UPU"}="1H*A UPU";
$resn_exch{"H'A2 UPU"}="2H*A UPU"; $resn_exch{"H3BC UPU"}="CH3B UPU"; $resn_exch{"H4'B UPU"}="H4*B UPU";
$resn_exch{"H'B1 UPU"}="1H*B UPU"; $resn_exch{"H'B2 UPU"}="2H*B UPU"; $resn_exch{"H5'B UPU"}="H5*B UPU";
$resn_exch{"H2BC UPU"}="CH2B UPU"; $resn_exch{"H2BO UPU"}="OH2B UPU"; $resn_exch{"H1'B UPU"}="H1*B UPU";
$resn_exch{"HM51  UQ"}="1HM5  UQ"; $resn_exch{"HM52  UQ"}="2HM5  UQ"; $resn_exch{"HM53  UQ"}="3HM5  UQ";
$resn_exch{"HM31  UQ"}="1HM3  UQ"; $resn_exch{"HM32  UQ"}="2HM3  UQ"; $resn_exch{"HM33  UQ"}="3HM3  UQ";
$resn_exch{"HM21  UQ"}="1HM2  UQ"; $resn_exch{"HM22  UQ"}="2HM2  UQ"; $resn_exch{"HM23  UQ"}="3HM2  UQ";
$resn_exch{" H71  UQ"}="1H7   UQ"; $resn_exch{" H72  UQ"}="2H7   UQ"; $resn_exch{"H101  UQ"}="1H10  UQ";
$resn_exch{"H102  UQ"}="2H10  UQ"; $resn_exch{"H103  UQ"}="3H10  UQ"; $resn_exch{"H111  UQ"}="1H11  UQ";
$resn_exch{"H112  UQ"}="2H11  UQ"; $resn_exch{"H121  UQ"}="1H12  UQ"; $resn_exch{"H122  UQ"}="2H12  UQ";
$resn_exch{"H151  UQ"}="1H15  UQ"; $resn_exch{"H152  UQ"}="2H15  UQ"; $resn_exch{"H153  UQ"}="3H15  UQ";
$resn_exch{"H161  UQ"}="1H16  UQ"; $resn_exch{"H162  UQ"}="2H16  UQ"; $resn_exch{"H171  UQ"}="1H17  UQ";
$resn_exch{"H172  UQ"}="2H17  UQ"; $resn_exch{"H201  UQ"}="1H20  UQ"; $resn_exch{"H202  UQ"}="2H20  UQ";
$resn_exch{"H203  UQ"}="3H20  UQ"; $resn_exch{"H211  UQ"}="1H21  UQ"; $resn_exch{"H212  UQ"}="2H21  UQ";
$resn_exch{"H221  UQ"}="1H22  UQ"; $resn_exch{"H222  UQ"}="2H22  UQ"; $resn_exch{"H251  UQ"}="1H25  UQ";
$resn_exch{"H252  UQ"}="2H25  UQ"; $resn_exch{"H253  UQ"}="3H25  UQ"; $resn_exch{"H261  UQ"}="1H26  UQ";
$resn_exch{"H262  UQ"}="2H26  UQ"; $resn_exch{"H271  UQ"}="1H27  UQ"; $resn_exch{"H272  UQ"}="2H27  UQ";
$resn_exch{"H301  UQ"}="1H30  UQ"; $resn_exch{"H302  UQ"}="2H30  UQ"; $resn_exch{"H303  UQ"}="3H30  UQ";
$resn_exch{"H311  UQ"}="1H31  UQ"; $resn_exch{"H312  UQ"}="2H31  UQ"; $resn_exch{"H321  UQ"}="1H32  UQ";
$resn_exch{"H322  UQ"}="2H32  UQ"; $resn_exch{"H351  UQ"}="1H35  UQ"; $resn_exch{"H352  UQ"}="2H35  UQ";
$resn_exch{"H353  UQ"}="3H35  UQ"; $resn_exch{"H361  UQ"}="1H36  UQ"; $resn_exch{"H362  UQ"}="2H36  UQ";
$resn_exch{"H371  UQ"}="1H37  UQ"; $resn_exch{"H372  UQ"}="2H37  UQ"; $resn_exch{"H401  UQ"}="1H40  UQ";
$resn_exch{"H402  UQ"}="2H40  UQ"; $resn_exch{"H403  UQ"}="3H40  UQ"; $resn_exch{"H411  UQ"}="1H41  UQ";
$resn_exch{"H412  UQ"}="2H41  UQ"; $resn_exch{"H421  UQ"}="1H42  UQ"; $resn_exch{"H422  UQ"}="2H42  UQ";
$resn_exch{"H451  UQ"}="1H45  UQ"; $resn_exch{"H452  UQ"}="2H45  UQ"; $resn_exch{"H453  UQ"}="3H45  UQ";
$resn_exch{"H461  UQ"}="1H46  UQ"; $resn_exch{"H462  UQ"}="2H46  UQ"; $resn_exch{"H471  UQ"}="1H47  UQ";
$resn_exch{"H472  UQ"}="2H47  UQ"; $resn_exch{"H501  UQ"}="1H50  UQ"; $resn_exch{"H502  UQ"}="2H50  UQ";
$resn_exch{"H503  UQ"}="3H50  UQ"; $resn_exch{"H511  UQ"}="1H51  UQ"; $resn_exch{"H512  UQ"}="2H51  UQ";
$resn_exch{"H521  UQ"}="1H52  UQ"; $resn_exch{"H522  UQ"}="2H52  UQ"; $resn_exch{"H551  UQ"}="1H55  UQ";
$resn_exch{"H552  UQ"}="2H55  UQ"; $resn_exch{"H553  UQ"}="3H55  UQ"; $resn_exch{"H561  UQ"}="1H56  UQ";
$resn_exch{"H562  UQ"}="2H56  UQ"; $resn_exch{"H563  UQ"}="3H56  UQ"; $resn_exch{"HM21 UQ1"}="1HM2 UQ1";
$resn_exch{"HM22 UQ1"}="2HM2 UQ1"; $resn_exch{"HM23 UQ1"}="3HM2 UQ1"; $resn_exch{"HM31 UQ1"}="1HM3 UQ1";
$resn_exch{"HM32 UQ1"}="2HM3 UQ1"; $resn_exch{"HM33 UQ1"}="3HM3 UQ1"; $resn_exch{"HM51 UQ1"}="1HM5 UQ1";
$resn_exch{"HM52 UQ1"}="2HM5 UQ1"; $resn_exch{"HM53 UQ1"}="3HM5 UQ1"; $resn_exch{" H71 UQ1"}="1H7  UQ1";
$resn_exch{" H72 UQ1"}="2H7  UQ1"; $resn_exch{"H101 UQ1"}="1H10 UQ1"; $resn_exch{"H102 UQ1"}="2H10 UQ1";
$resn_exch{"H103 UQ1"}="3H10 UQ1"; $resn_exch{"H111 UQ1"}="1H11 UQ1"; $resn_exch{"H112 UQ1"}="2H11 UQ1";
$resn_exch{"H113 UQ1"}="3H11 UQ1"; $resn_exch{"H2M1 UQ2"}="1H2M UQ2"; $resn_exch{"H2M2 UQ2"}="2H2M UQ2";
$resn_exch{"H2M3 UQ2"}="3H2M UQ2"; $resn_exch{"H3M1 UQ2"}="1H3M UQ2"; $resn_exch{"H3M2 UQ2"}="2H3M UQ2";
$resn_exch{"H3M3 UQ2"}="3H3M UQ2"; $resn_exch{"H5M1 UQ2"}="1H5M UQ2"; $resn_exch{"H5M2 UQ2"}="2H5M UQ2";
$resn_exch{"H5M3 UQ2"}="3H5M UQ2"; $resn_exch{" H71 UQ2"}="1H7  UQ2"; $resn_exch{" H72 UQ2"}="2H7  UQ2";
$resn_exch{"H101 UQ2"}="1H10 UQ2"; $resn_exch{"H102 UQ2"}="2H10 UQ2"; $resn_exch{"H103 UQ2"}="3H10 UQ2";
$resn_exch{"H111 UQ2"}="1H11 UQ2"; $resn_exch{"H112 UQ2"}="2H11 UQ2"; $resn_exch{"H121 UQ2"}="1H12 UQ2";
$resn_exch{"H122 UQ2"}="2H12 UQ2"; $resn_exch{"H151 UQ2"}="1H15 UQ2"; $resn_exch{"H152 UQ2"}="2H15 UQ2";
$resn_exch{"H153 UQ2"}="3H15 UQ2"; $resn_exch{"H161 UQ2"}="1H16 UQ2"; $resn_exch{"H162 UQ2"}="2H16 UQ2";
$resn_exch{"H163 UQ2"}="3H16 UQ2"; $resn_exch{"H1M1 UQ5"}="1H1M UQ5"; $resn_exch{"H1M2 UQ5"}="2H1M UQ5";
$resn_exch{"H1M3 UQ5"}="3H1M UQ5"; $resn_exch{"H3M1 UQ5"}="1H3M UQ5"; $resn_exch{"H3M2 UQ5"}="2H3M UQ5";
$resn_exch{"H3M3 UQ5"}="3H3M UQ5"; $resn_exch{"H4M1 UQ5"}="1H4M UQ5"; $resn_exch{"H4M2 UQ5"}="2H4M UQ5";
$resn_exch{"H4M3 UQ5"}="3H4M UQ5"; $resn_exch{" H71 UQ5"}="1H7  UQ5"; $resn_exch{" H72 UQ5"}="2H7  UQ5";
$resn_exch{"H101 UQ5"}="1H10 UQ5"; $resn_exch{"H102 UQ5"}="2H10 UQ5"; $resn_exch{"H103 UQ5"}="3H10 UQ5";
$resn_exch{"H111 UQ5"}="1H11 UQ5"; $resn_exch{"H112 UQ5"}="2H11 UQ5"; $resn_exch{"H121 UQ5"}="1H12 UQ5";
$resn_exch{"H122 UQ5"}="2H12 UQ5"; $resn_exch{"H151 UQ5"}="1H15 UQ5"; $resn_exch{"H152 UQ5"}="2H15 UQ5";
$resn_exch{"H153 UQ5"}="3H15 UQ5"; $resn_exch{"H161 UQ5"}="1H16 UQ5"; $resn_exch{"H162 UQ5"}="2H16 UQ5";
$resn_exch{"H171 UQ5"}="1H17 UQ5"; $resn_exch{"H172 UQ5"}="2H17 UQ5"; $resn_exch{"H201 UQ5"}="1H20 UQ5";
$resn_exch{"H202 UQ5"}="2H20 UQ5"; $resn_exch{"H203 UQ5"}="3H20 UQ5"; $resn_exch{"H211 UQ5"}="1H21 UQ5";
$resn_exch{"H212 UQ5"}="2H21 UQ5"; $resn_exch{"H221 UQ5"}="1H22 UQ5"; $resn_exch{"H222 UQ5"}="2H22 UQ5";
$resn_exch{"H251 UQ5"}="1H25 UQ5"; $resn_exch{"H252 UQ5"}="2H25 UQ5"; $resn_exch{"H253 UQ5"}="3H25 UQ5";
$resn_exch{"H261 UQ5"}="1H26 UQ5"; $resn_exch{"H262 UQ5"}="2H26 UQ5"; $resn_exch{"H271 UQ5"}="1H27 UQ5";
$resn_exch{"H272 UQ5"}="2H27 UQ5"; $resn_exch{"H301 UQ5"}="1H30 UQ5"; $resn_exch{"H302 UQ5"}="2H30 UQ5";
$resn_exch{"H303 UQ5"}="3H30 UQ5"; $resn_exch{"H311 UQ5"}="1H31 UQ5"; $resn_exch{"H312 UQ5"}="2H31 UQ5";
$resn_exch{"H313 UQ5"}="3H31 UQ5"; $resn_exch{"H1M1 UQ6"}="1H1M UQ6"; $resn_exch{"H1M2 UQ6"}="2H1M UQ6";
$resn_exch{"H1M3 UQ6"}="3H1M UQ6"; $resn_exch{"H3M1 UQ6"}="1H3M UQ6"; $resn_exch{"H3M2 UQ6"}="2H3M UQ6";
$resn_exch{"H3M3 UQ6"}="3H3M UQ6"; $resn_exch{"H4M1 UQ6"}="1H4M UQ6"; $resn_exch{"H4M2 UQ6"}="2H4M UQ6";
$resn_exch{"H4M3 UQ6"}="3H4M UQ6"; $resn_exch{" H71 UQ6"}="1H7  UQ6"; $resn_exch{" H72 UQ6"}="2H7  UQ6";
$resn_exch{"H101 UQ6"}="1H10 UQ6"; $resn_exch{"H102 UQ6"}="2H10 UQ6"; $resn_exch{"H103 UQ6"}="3H10 UQ6";
$resn_exch{"H111 UQ6"}="1H11 UQ6"; $resn_exch{"H112 UQ6"}="2H11 UQ6"; $resn_exch{"H121 UQ6"}="1H12 UQ6";
$resn_exch{"H122 UQ6"}="2H12 UQ6"; $resn_exch{"H151 UQ6"}="1H15 UQ6"; $resn_exch{"H152 UQ6"}="2H15 UQ6";
$resn_exch{"H153 UQ6"}="3H15 UQ6"; $resn_exch{"H161 UQ6"}="1H16 UQ6"; $resn_exch{"H162 UQ6"}="2H16 UQ6";
$resn_exch{"H171 UQ6"}="1H17 UQ6"; $resn_exch{"H172 UQ6"}="2H17 UQ6"; $resn_exch{"H201 UQ6"}="1H20 UQ6";
$resn_exch{"H202 UQ6"}="2H20 UQ6"; $resn_exch{"H203 UQ6"}="3H20 UQ6"; $resn_exch{"H211 UQ6"}="1H21 UQ6";
$resn_exch{"H212 UQ6"}="2H21 UQ6"; $resn_exch{"H221 UQ6"}="1H22 UQ6"; $resn_exch{"H222 UQ6"}="2H22 UQ6";
$resn_exch{"H251 UQ6"}="1H25 UQ6"; $resn_exch{"H252 UQ6"}="2H25 UQ6"; $resn_exch{"H253 UQ6"}="3H25 UQ6";
$resn_exch{"H261 UQ6"}="1H26 UQ6"; $resn_exch{"H262 UQ6"}="2H26 UQ6"; $resn_exch{"H271 UQ6"}="1H27 UQ6";
$resn_exch{"H272 UQ6"}="2H27 UQ6"; $resn_exch{"H301 UQ6"}="1H30 UQ6"; $resn_exch{"H302 UQ6"}="2H30 UQ6";
$resn_exch{"H303 UQ6"}="3H30 UQ6"; $resn_exch{"H311 UQ6"}="1H31 UQ6"; $resn_exch{"H312 UQ6"}="2H31 UQ6";
$resn_exch{"H321 UQ6"}="1H32 UQ6"; $resn_exch{"H322 UQ6"}="2H32 UQ6"; $resn_exch{"H351 UQ6"}="1H35 UQ6";
$resn_exch{"H352 UQ6"}="2H35 UQ6"; $resn_exch{"H353 UQ6"}="3H35 UQ6"; $resn_exch{"H361 UQ6"}="1H36 UQ6";
$resn_exch{"H362 UQ6"}="2H36 UQ6"; $resn_exch{"H363 UQ6"}="3H36 UQ6"; $resn_exch{"HM21 UQ7"}="1HM2 UQ7";
$resn_exch{"HM22 UQ7"}="2HM2 UQ7"; $resn_exch{"HM23 UQ7"}="3HM2 UQ7"; $resn_exch{"HM31 UQ7"}="1HM3 UQ7";
$resn_exch{"HM32 UQ7"}="2HM3 UQ7"; $resn_exch{"HM33 UQ7"}="3HM3 UQ7"; $resn_exch{"HM51 UQ7"}="1HM5 UQ7";
$resn_exch{"HM52 UQ7"}="2HM5 UQ7"; $resn_exch{"HM53 UQ7"}="3HM5 UQ7"; $resn_exch{" H71 UQ7"}="1H7  UQ7";
$resn_exch{" H72 UQ7"}="2H7  UQ7"; $resn_exch{"H101 UQ7"}="1H10 UQ7"; $resn_exch{"H102 UQ7"}="2H10 UQ7";
$resn_exch{"H103 UQ7"}="3H10 UQ7"; $resn_exch{"H111 UQ7"}="1H11 UQ7"; $resn_exch{"H112 UQ7"}="2H11 UQ7";
$resn_exch{"H121 UQ7"}="1H12 UQ7"; $resn_exch{"H122 UQ7"}="2H12 UQ7"; $resn_exch{"H151 UQ7"}="1H15 UQ7";
$resn_exch{"H152 UQ7"}="2H15 UQ7"; $resn_exch{"H153 UQ7"}="3H15 UQ7"; $resn_exch{"H161 UQ7"}="1H16 UQ7";
$resn_exch{"H162 UQ7"}="2H16 UQ7"; $resn_exch{"H171 UQ7"}="1H17 UQ7"; $resn_exch{"H172 UQ7"}="2H17 UQ7";
$resn_exch{"H201 UQ7"}="1H20 UQ7"; $resn_exch{"H202 UQ7"}="2H20 UQ7"; $resn_exch{"H203 UQ7"}="3H20 UQ7";
$resn_exch{"H211 UQ7"}="1H21 UQ7"; $resn_exch{"H212 UQ7"}="2H21 UQ7"; $resn_exch{"H221 UQ7"}="1H22 UQ7";
$resn_exch{"H222 UQ7"}="2H22 UQ7"; $resn_exch{"H251 UQ7"}="1H25 UQ7"; $resn_exch{"H252 UQ7"}="2H25 UQ7";
$resn_exch{"H253 UQ7"}="3H25 UQ7"; $resn_exch{"H261 UQ7"}="1H26 UQ7"; $resn_exch{"H262 UQ7"}="2H26 UQ7";
$resn_exch{"H271 UQ7"}="1H27 UQ7"; $resn_exch{"H272 UQ7"}="2H27 UQ7"; $resn_exch{"H301 UQ7"}="1H30 UQ7";
$resn_exch{"H302 UQ7"}="2H30 UQ7"; $resn_exch{"H303 UQ7"}="3H30 UQ7"; $resn_exch{"H311 UQ7"}="1H31 UQ7";
$resn_exch{"H312 UQ7"}="2H31 UQ7"; $resn_exch{"H321 UQ7"}="1H32 UQ7"; $resn_exch{"H322 UQ7"}="2H32 UQ7";
$resn_exch{"H351 UQ7"}="1H35 UQ7"; $resn_exch{"H352 UQ7"}="2H35 UQ7"; $resn_exch{"H353 UQ7"}="3H35 UQ7";
$resn_exch{"H361 UQ7"}="1H36 UQ7"; $resn_exch{"H362 UQ7"}="2H36 UQ7"; $resn_exch{"H371 UQ7"}="1H37 UQ7";
$resn_exch{"H372 UQ7"}="2H37 UQ7"; $resn_exch{"H401 UQ7"}="1H40 UQ7"; $resn_exch{"H402 UQ7"}="2H40 UQ7";
$resn_exch{"H403 UQ7"}="3H40 UQ7"; $resn_exch{"H411 UQ7"}="1H41 UQ7"; $resn_exch{"H412 UQ7"}="2H41 UQ7";
$resn_exch{"H413 UQ7"}="3H41 UQ7"; $resn_exch{"H171 UR2"}="1H17 UR2"; $resn_exch{"H172 UR2"}="2H17 UR2";
$resn_exch{"H181 UR2"}="1H18 UR2"; $resn_exch{"H182 UR2"}="2H18 UR2"; $resn_exch{"H251 UR2"}="1H25 UR2";
$resn_exch{"H252 UR2"}="2H25 UR2"; $resn_exch{"H253 UR2"}="3H25 UR2"; $resn_exch{"H261 UR2"}="1H26 UR2";
$resn_exch{"H262 UR2"}="2H26 UR2"; $resn_exch{"H271 UR2"}="1H27 UR2"; $resn_exch{"H272 UR2"}="2H27 UR2";
$resn_exch{"H281 UR2"}="1H28 UR2"; $resn_exch{"H282 UR2"}="2H28 UR2"; $resn_exch{"H301 UR2"}="1H30 UR2";
$resn_exch{"H302 UR2"}="2H30 UR2"; $resn_exch{"H311 UR2"}="1H31 UR2"; $resn_exch{"H312 UR2"}="2H31 UR2";
$resn_exch{"H321 UR2"}="1H32 UR2"; $resn_exch{"H322 UR2"}="2H32 UR2"; $resn_exch{"H331 UR2"}="1H33 UR2";
$resn_exch{"H332 UR2"}="2H33 UR2"; $resn_exch{"H341 UR2"}="1H34 UR2"; $resn_exch{"H342 UR2"}="2H34 UR2";
$resn_exch{" H71 UR2"}="1H7  UR2"; $resn_exch{" H72 UR2"}="2H7  UR2"; $resn_exch{" H91 UR2"}="1H9  UR2";
$resn_exch{" H92 UR2"}="2H9  UR2"; $resn_exch{"HOP2 UR2"}="2HOP UR2"; $resn_exch{"HOP3 UR2"}="3HOP UR2";
$resn_exch{"HOP2 UR3"}="2HOP UR3"; $resn_exch{" H5' UR3"}="1H5* UR3"; $resn_exch{"H5'' UR3"}="2H5* UR3";
$resn_exch{" H4' UR3"}=" H4* UR3"; $resn_exch{" H1' UR3"}=" H1* UR3"; $resn_exch{"H3U1 UR3"}="1H3U UR3";
$resn_exch{"H3U2 UR3"}="2H3U UR3"; $resn_exch{"H3U3 UR3"}="3H3U UR3"; $resn_exch{" H2' UR3"}=" H2* UR3";
$resn_exch{"HO2' UR3"}="2HO* UR3"; $resn_exch{" H3' UR3"}=" H3* UR3"; $resn_exch{"HO3' UR3"}="H3T  UR3";
$resn_exch{"HOP3 UR3"}="3HOP UR3"; $resn_exch{"HO5' URD"}="*HO5 URD"; $resn_exch{" H5' URD"}="1H5* URD";
$resn_exch{"H5'' URD"}="2H5* URD"; $resn_exch{" H4' URD"}=" H4* URD"; $resn_exch{" H1' URD"}=" H1* URD";
$resn_exch{" H31 URD"}="1H3  URD"; $resn_exch{" H32 URD"}="2H3  URD"; $resn_exch{" H2' URD"}=" H2* URD";
$resn_exch{"HO2' URD"}="*HO2 URD"; $resn_exch{" H3' URD"}=" H3* URD"; $resn_exch{"HO3' URD"}="*HO3 URD";
$resn_exch{"HN11 URE"}="1HN1 URE"; $resn_exch{"HN12 URE"}="2HN1 URE"; $resn_exch{"HN21 URE"}="1HN2 URE";
$resn_exch{"HN22 URE"}="2HN2 URE"; $resn_exch{" H1' URI"}=" H1* URI"; $resn_exch{" H2' URI"}=" H2* URI";
$resn_exch{" H3' URI"}=" H3* URI"; $resn_exch{" H4' URI"}=" H4* URI"; $resn_exch{"HO2' URI"}="*HO2 URI";
$resn_exch{"HO3' URI"}="*HO3 URI"; $resn_exch{"H5'1 URI"}="1H5* URI"; $resn_exch{"H5'2 URI"}="2H5* URI";
$resn_exch{"HO5' URI"}="*HO5 URI"; $resn_exch{"HN51 URN"}="1HN5 URN"; $resn_exch{"HN52 URN"}="2HN5 URN";
$resn_exch{"HN61 URN"}="1HN6 URN"; $resn_exch{"HN62 URN"}="2HN6 URN"; $resn_exch{"HN31 URP"}="1HN3 URP";
$resn_exch{"HN32 URP"}="2HN3 URP"; $resn_exch{" H61 URP"}="1H6  URP"; $resn_exch{" H62 URP"}="2H6  URP";
$resn_exch{" H51 URP"}="1H5  URP"; $resn_exch{" H52 URP"}="2H5  URP"; $resn_exch{"H5'1 US1"}="1H5* US1";
$resn_exch{"H5'2 US1"}="2H5* US1"; $resn_exch{" H4' US1"}=" H4* US1"; $resn_exch{" H3' US1"}=" H3* US1";
$resn_exch{"H2'1 US1"}="1H2* US1"; $resn_exch{"H2'2 US1"}="2H2* US1"; $resn_exch{" H1' US1"}=" H1* US1";
$resn_exch{" H1D USQ"}=" H1* USQ"; $resn_exch{" H2D USQ"}=" H2* USQ"; $resn_exch{"HO2' USQ"}="*HO2 USQ";
$resn_exch{" H3D USQ"}=" H3* USQ"; $resn_exch{" H4D USQ"}=" H4* USQ"; $resn_exch{"HO3' USQ"}="*HO3 USQ";
$resn_exch{"H5'1 USQ"}="1H5* USQ"; $resn_exch{"H5'2 USQ"}="2H5* USQ"; $resn_exch{"HOA2 USQ"}="2HOA USQ";
$resn_exch{"HOB2 USQ"}="2HOB USQ"; $resn_exch{"H6'1 USQ"}="1H6' USQ"; $resn_exch{"H6'2 USQ"}="2H6' USQ";
$resn_exch{"HO2A USQ"}="'HO2 USQ"; $resn_exch{"HO3A USQ"}="'HO3 USQ"; $resn_exch{"HO4' USQ"}="'HO4 USQ";
$resn_exch{"H5'1 UTP"}="1H5* UTP"; $resn_exch{"H5'2 UTP"}="2H5* UTP"; $resn_exch{" H4' UTP"}=" H4* UTP";
$resn_exch{" H1' UTP"}=" H1* UTP"; $resn_exch{" H2' UTP"}=" H2* UTP"; $resn_exch{" H3' UTP"}=" H3* UTP";
$resn_exch{" H1' UVC"}=" H1* UVC"; $resn_exch{" H2' UVC"}=" H2* UVC"; $resn_exch{" H3' UVC"}=" H3* UVC";
$resn_exch{" H4' UVC"}=" H4* UVC"; $resn_exch{"H5'1 UVC"}="1H5* UVC"; $resn_exch{"H5'2 UVC"}="2H5* UVC";
$resn_exch{"HO5' UVC"}="*HO5 UVC"; $resn_exch{"HOV1 UVC"}="1HOV UVC"; $resn_exch{"HOV3 UVC"}="3HOV UVC";
$resn_exch{"H1M1 UVW"}="1H1M UVW"; $resn_exch{"H1M2 UVW"}="2H1M UVW"; $resn_exch{"H1M3 UVW"}="3H1M UVW";
$resn_exch{"HOP2 UVW"}="2HOP UVW"; $resn_exch{"HOP3 UVW"}="3HOP UVW"; $resn_exch{"H1C1 UZ9"}="1H1C UZ9";
$resn_exch{"H1C2 UZ9"}="2H1C UZ9"; $resn_exch{"H2C1 UZ9"}="1H2C UZ9"; $resn_exch{"H2C2 UZ9"}="2H2C UZ9";
$resn_exch{"H4C1 UZ9"}="1H4C UZ9"; $resn_exch{"H4C2 UZ9"}="2H4C UZ9"; $resn_exch{"H6C1 UZ9"}="1H6C UZ9";
$resn_exch{"H6C2 UZ9"}="2H6C UZ9"; $resn_exch{" HN1 UZ9"}="1HN  UZ9"; $resn_exch{" HN2 UZ9"}="2HN  UZ9";
$resn_exch{"HBC1 UZ9"}="1HBC UZ9"; $resn_exch{"HBC2 UZ9"}="2HBC UZ9"; $resn_exch{"HGC1 UZ9"}="1HGC UZ9";
$resn_exch{"HGC2 UZ9"}="2HGC UZ9"; $resn_exch{"HDC1 UZ9"}="1HDC UZ9"; $resn_exch{"HDC2 UZ9"}="2HDC UZ9";
$resn_exch{"HEC1 UZ9"}="1HEC UZ9"; $resn_exch{"HEC2 UZ9"}="2HEC UZ9"; $resn_exch{"H231 UZ9"}="1H23 UZ9";
$resn_exch{"H232 UZ9"}="2H23 UZ9"; $resn_exch{"H221 UZ9"}="1H22 UZ9"; $resn_exch{"H222 UZ9"}="2H22 UZ9";
$resn_exch{"H211 UZ9"}="1H21 UZ9"; $resn_exch{"H212 UZ9"}="2H21 UZ9"; $resn_exch{"H213 UZ9"}="3H21 UZ9";
$resn_exch{"H161 UZ9"}="1H16 UZ9"; $resn_exch{"H162 UZ9"}="2H16 UZ9"; $resn_exch{"H121 UZ9"}="1H12 UZ9";
$resn_exch{"H122 UZ9"}="2H12 UZ9"; $resn_exch{"H181 UZ9"}="1H18 UZ9"; $resn_exch{"H182 UZ9"}="2H18 UZ9";
$resn_exch{"H183 UZ9"}="3H18 UZ9"; $resn_exch{"H151 UZ9"}="1H15 UZ9"; $resn_exch{"H152 UZ9"}="2H15 UZ9";
$resn_exch{"H7C1 UZ9"}="1H7C UZ9"; $resn_exch{"H7C2 UZ9"}="2H7C UZ9"; $resn_exch{"H111 UZ9"}="1H11 UZ9";
$resn_exch{"H112 UZ9"}="2H11 UZ9"; $resn_exch{"H191 UZ9"}="1H19 UZ9"; $resn_exch{"H192 UZ9"}="2H19 UZ9";
$resn_exch{"H193 UZ9"}="3H19 UZ9"; $resn_exch{"HAA1 V12"}="1HAA V12"; $resn_exch{"HAA2 V12"}="2HAA V12";
$resn_exch{"HAN1 V12"}="1HAN V12"; $resn_exch{"HAN2 V12"}="2HAN V12"; $resn_exch{"HAL1 V12"}="1HAL V12";
$resn_exch{"HAL2 V12"}="2HAL V12"; $resn_exch{"H141 V15"}="1H14 V15"; $resn_exch{"H142 V15"}="2H14 V15";
$resn_exch{"H151 V15"}="1H15 V15"; $resn_exch{"H152 V15"}="2H15 V15"; $resn_exch{"H161 V15"}="1H16 V15";
$resn_exch{"H162 V15"}="2H16 V15"; $resn_exch{"HAA1 V25"}="1HAA V25"; $resn_exch{"HAA2 V25"}="2HAA V25";
$resn_exch{"HAA3 V25"}="3HAA V25"; $resn_exch{"HAL1 V25"}="1HAL V25"; $resn_exch{"HAL2 V25"}="2HAL V25";
$resn_exch{"HAB1 V25"}="1HAB V25"; $resn_exch{"HAB2 V25"}="2HAB V25"; $resn_exch{"HAB3 V25"}="3HAB V25";
$resn_exch{"HAD1 V25"}="1HAD V25"; $resn_exch{"HAD2 V25"}="2HAD V25"; $resn_exch{" H71 V35"}="1H7  V35";
$resn_exch{" H72 V35"}="2H7  V35"; $resn_exch{" H91 V35"}="1H9  V35"; $resn_exch{" H92 V35"}="2H9  V35";
$resn_exch{" H93 V35"}="3H9  V35"; $resn_exch{"HOB1 V35"}="1HOB V35"; $resn_exch{"HOB2 V35"}="2HOB V35";
$resn_exch{"HOB3 V35"}="3HOB V35"; $resn_exch{" H71 V36"}="1H7  V36"; $resn_exch{" H72 V36"}="2H7  V36";
$resn_exch{" H91 V36"}="1H9  V36"; $resn_exch{" H92 V36"}="2H9  V36"; $resn_exch{" H93 V36"}="3H9  V36";
$resn_exch{"HOB1 V36"}="1HOB V36"; $resn_exch{"HOB2 V36"}="2HOB V36"; $resn_exch{"HOB3 V36"}="3HOB V36";
$resn_exch{"HBI1 V5X"}="1HBI V5X"; $resn_exch{"HBI2 V5X"}="2HBI V5X"; $resn_exch{"HBI3 V5X"}="3HBI V5X";
$resn_exch{"HBP1 V5X"}="1HBP V5X"; $resn_exch{"HBP2 V5X"}="2HBP V5X"; $resn_exch{"HBP3 V5X"}="3HBP V5X";
$resn_exch{"HBH1 V5X"}="1HBH V5X"; $resn_exch{"HBH2 V5X"}="2HBH V5X"; $resn_exch{"HBC1 V5X"}="1HBC V5X";
$resn_exch{"HBC2 V5X"}="2HBC V5X"; $resn_exch{"HAV1 V5X"}="1HAV V5X"; $resn_exch{"HAV2 V5X"}="2HAV V5X";
$resn_exch{"HAW1 V5X"}="1HAW V5X"; $resn_exch{"HAW2 V5X"}="2HAW V5X"; $resn_exch{"HAX1 V5X"}="1HAX V5X";
$resn_exch{"HAX2 V5X"}="2HAX V5X"; $resn_exch{"HAY1 V5X"}="1HAY V5X"; $resn_exch{"HAY2 V5X"}="2HAY V5X";
$resn_exch{"HAQ1 V5X"}="1HAQ V5X"; $resn_exch{"HAQ2 V5X"}="2HAQ V5X"; $resn_exch{"HAP1 V5X"}="1HAP V5X";
$resn_exch{"HAP2 V5X"}="2HAP V5X"; $resn_exch{"H221  VA"}="1H22  VA"; $resn_exch{"H222  VA"}="2H22  VA";
$resn_exch{"H223  VA"}="3H22  VA"; $resn_exch{"H11A  VA"}="1H1   VA"; $resn_exch{"H12A  VA"}="2H1   VA";
$resn_exch{" H21  VA"}="1H2   VA"; $resn_exch{" H22  VA"}="2H2   VA"; $resn_exch{" H31  VA"}="1H3   VA";
$resn_exch{" H32  VA"}="2H3   VA"; $resn_exch{" H41  VA"}="1H4   VA"; $resn_exch{" H42  VA"}="2H4   VA";
$resn_exch{" H51  VA"}="1H5   VA"; $resn_exch{" H52  VA"}="2H5   VA"; $resn_exch{" H61  VA"}="1H6   VA";
$resn_exch{" H62  VA"}="2H6   VA"; $resn_exch{" H71  VA"}="1H7   VA"; $resn_exch{" H72  VA"}="2H7   VA";
$resn_exch{" H81  VA"}="1H8   VA"; $resn_exch{" H82  VA"}="2H8   VA"; $resn_exch{" H91  VA"}="1H9   VA";
$resn_exch{" H92  VA"}="2H9   VA"; $resn_exch{"H101  VA"}="1H10  VA"; $resn_exch{"H102  VA"}="2H10  VA";
$resn_exch{"H131  VA"}="1H13  VA"; $resn_exch{"H132  VA"}="2H13  VA"; $resn_exch{"H141  VA"}="1H14  VA";
$resn_exch{"H142  VA"}="2H14  VA"; $resn_exch{"H151  VA"}="1H15  VA"; $resn_exch{"H152  VA"}="2H15  VA";
$resn_exch{"H161  VA"}="1H16  VA"; $resn_exch{"H162  VA"}="2H16  VA"; $resn_exch{"H171  VA"}="1H17  VA";
$resn_exch{"H172  VA"}="2H17  VA"; $resn_exch{"H181  VA"}="1H18  VA"; $resn_exch{"H182  VA"}="2H18  VA";
$resn_exch{"H183  VA"}="3H18  VA"; $resn_exch{"H121 VA1"}="1H12 VA1"; $resn_exch{"H122 VA1"}="2H12 VA1";
$resn_exch{"HG11 VAA"}="1HG1 VAA"; $resn_exch{"HG12 VAA"}="2HG1 VAA"; $resn_exch{"HG13 VAA"}="3HG1 VAA";
$resn_exch{"HG21 VAA"}="1HG2 VAA"; $resn_exch{"HG22 VAA"}="2HG2 VAA"; $resn_exch{"HG23 VAA"}="3HG2 VAA";
$resn_exch{"HNS3 VAA"}="3HNS VAA"; $resn_exch{"HN'5 VAA"}="5HN* VAA"; $resn_exch{"H5'1 VAA"}="1H5* VAA";
$resn_exch{"H5'2 VAA"}="2H5* VAA"; $resn_exch{" H4' VAA"}=" H4* VAA"; $resn_exch{" H1' VAA"}=" H1* VAA";
$resn_exch{" H61 VAA"}="1H6  VAA"; $resn_exch{" H62 VAA"}="2H6  VAA"; $resn_exch{" H2' VAA"}=" H2* VAA";
$resn_exch{"HO'2 VAA"}="2HO* VAA"; $resn_exch{" H3' VAA"}=" H3* VAA"; $resn_exch{"HT'3 VAA"}="3HT* VAA";
$resn_exch{" H31 VAC"}="1H3  VAC"; $resn_exch{" H32 VAC"}="2H3  VAC"; $resn_exch{" H41 VAC"}="1H4  VAC";
$resn_exch{" H42 VAC"}="2H4  VAC"; $resn_exch{"H121 VAC"}="1H12 VAC"; $resn_exch{"H122 VAC"}="2H12 VAC";
$resn_exch{"HN19 VAC"}="9HN1 VAC"; $resn_exch{"HO22 VAC"}="2HO2 VAC"; $resn_exch{"H261 VAC"}="1H26 VAC";
$resn_exch{"H262 VAC"}="2H26 VAC"; $resn_exch{"H311 VAC"}="1H31 VAC"; $resn_exch{"H312 VAC"}="2H31 VAC";
$resn_exch{"H681 VAC"}="1H68 VAC"; $resn_exch{"H682 VAC"}="2H68 VAC"; $resn_exch{"HO70 VAC"}="0HO7 VAC";
$resn_exch{"HN86 VAC"}="6HN8 VAC"; $resn_exch{"HG11 VAD"}="1HG1 VAD"; $resn_exch{"HG12 VAD"}="2HG1 VAD";
$resn_exch{"HG13 VAD"}="3HG1 VAD"; $resn_exch{"HG21 VAD"}="1HG2 VAD"; $resn_exch{"HG22 VAD"}="2HG2 VAD";
$resn_exch{"HG23 VAD"}="3HG2 VAD"; $resn_exch{" HN1 VAF"}="1HN  VAF"; $resn_exch{" HN2 VAF"}="2HN  VAF";
$resn_exch{"HG11 VAF"}="1HG1 VAF"; $resn_exch{"HG12 VAF"}="2HG1 VAF"; $resn_exch{"HG13 VAF"}="3HG1 VAF";
$resn_exch{"HG21 VAF"}="1HG2 VAF"; $resn_exch{"HG22 VAF"}="2HG2 VAF"; $resn_exch{"HG23 VAF"}="3HG2 VAF";
$resn_exch{" HJ1 VAF"}="1HJ  VAF"; $resn_exch{" HJ2 VAF"}="2HJ  VAF"; $resn_exch{" HJ3 VAF"}="3HJ  VAF";
$resn_exch{"H11A VAG"}="1H1  VAG"; $resn_exch{" H12 VAG"}="2H1  VAG"; $resn_exch{" H13 VAG"}="3H1  VAG";
$resn_exch{"HN21 VAG"}="1HN2 VAG"; $resn_exch{"HN22 VAG"}="2HN2 VAG"; $resn_exch{"H61A VAG"}="1H6  VAG";
$resn_exch{"H62A VAG"}="2H6  VAG"; $resn_exch{" H81 VAG"}="1H8  VAG"; $resn_exch{" H82 VAG"}="2H8  VAG";
$resn_exch{" H83 VAG"}="3H8  VAG"; $resn_exch{" H91 VAG"}="1H9  VAG"; $resn_exch{" H92 VAG"}="2H9  VAG";
$resn_exch{" H93 VAG"}="3H9  VAG"; $resn_exch{"H281 VAG"}="1H28 VAG"; $resn_exch{"H282 VAG"}="2H28 VAG";
$resn_exch{"HN11 VAG"}="1HN1 VAG"; $resn_exch{"HN12 VAG"}="2HN1 VAG"; $resn_exch{"HN43 VAG"}="3HN4 VAG";
$resn_exch{" H2  VAL"}=" HN2 VAL"; $resn_exch{"HG11 VAL"}="1HG1 VAL"; $resn_exch{"HG12 VAL"}="2HG1 VAL";
$resn_exch{"HG13 VAL"}="3HG1 VAL"; $resn_exch{"HG21 VAL"}="1HG2 VAL"; $resn_exch{"HG22 VAL"}="2HG2 VAL";
$resn_exch{"HG23 VAL"}="3HG2 VAL"; $resn_exch{"H121 VAM"}="1H12 VAM"; $resn_exch{"H122 VAM"}="2H12 VAM";
$resn_exch{"HMC1 VAM"}="1HMC VAM"; $resn_exch{"HMC2 VAM"}="2HMC VAM"; $resn_exch{"HMC3 VAM"}="3HMC VAM";
$resn_exch{"H2C1 VAM"}="1H2C VAM"; $resn_exch{"H2C2 VAM"}="2H2C VAM"; $resn_exch{"H11A VAN"}="1H1  VAN";
$resn_exch{" H12 VAN"}="2H1  VAN"; $resn_exch{" H13 VAN"}="3H1  VAN"; $resn_exch{"HN21 VAN"}="1HN2 VAN";
$resn_exch{"HN22 VAN"}="2HN2 VAN"; $resn_exch{"H61A VAN"}="1H6  VAN"; $resn_exch{"H62A VAN"}="2H6  VAN";
$resn_exch{" H81 VAN"}="1H8  VAN"; $resn_exch{" H82 VAN"}="2H8  VAN"; $resn_exch{" H83 VAN"}="3H8  VAN";
$resn_exch{" H91 VAN"}="1H9  VAN"; $resn_exch{" H92 VAN"}="2H9  VAN"; $resn_exch{" H93 VAN"}="3H9  VAN";
$resn_exch{"H281 VAN"}="1H28 VAN"; $resn_exch{"H282 VAN"}="2H28 VAN"; $resn_exch{"HN11 VAN"}="1HN1 VAN";
$resn_exch{"HN12 VAN"}="2HN1 VAN"; $resn_exch{"HN43 VAN"}="3HN4 VAN"; $resn_exch{"HO3' VAN"}="'HO3 VAN";
$resn_exch{"HO4' VAN"}="'HO4 VAN"; $resn_exch{"H6'1 VAN"}="1H6' VAN"; $resn_exch{"H6'2 VAN"}="2H6' VAN";
$resn_exch{"HO6' VAN"}="'HO6 VAN"; $resn_exch{" H1B VAN"}=" H1\" VAN"; $resn_exch{"H2'1 VAN"}="1H2\" VAN";
$resn_exch{"H2'2 VAN"}="2H2\" VAN"; $resn_exch{"HN31 VAN"}="1HN3 VAN"; $resn_exch{"HN32 VAN"}="2HN3 VAN";
$resn_exch{"HN33 VAN"}="3HN3 VAN"; $resn_exch{"H3M1 VAN"}="1H3M VAN"; $resn_exch{"H3M2 VAN"}="2H3M VAN";
$resn_exch{"H3M3 VAN"}="3H3M VAN"; $resn_exch{" H4B VAN"}=" H4\" VAN"; $resn_exch{"HO4A VAN"}="\"HO4 VAN";
$resn_exch{" H5B VAN"}=" H5\" VAN"; $resn_exch{"H5M1 VAN"}="1H5M VAN"; $resn_exch{"H5M2 VAN"}="2H5M VAN";
$resn_exch{"H5M3 VAN"}="3H5M VAN"; $resn_exch{" H11 VAS"}="1H1  VAS"; $resn_exch{" H12 VAS"}="2H1  VAS";
$resn_exch{" H13 VAS"}="3H1  VAS"; $resn_exch{"H2'1 VAS"}="1H2' VAS"; $resn_exch{"H2'2 VAS"}="2H2' VAS";
$resn_exch{"H2'3 VAS"}="3H2' VAS"; $resn_exch{"H3'1 VAS"}="1H3' VAS"; $resn_exch{"H3'2 VAS"}="2H3' VAS";
$resn_exch{"H3'3 VAS"}="3H3' VAS"; $resn_exch{"H11A VAX"}="1H1  VAX"; $resn_exch{" H12 VAX"}="2H1  VAX";
$resn_exch{" H13 VAX"}="3H1  VAX"; $resn_exch{"HN21 VAX"}="1HN2 VAX"; $resn_exch{"HN22 VAX"}="2HN2 VAX";
$resn_exch{"H61A VAX"}="1H6  VAX"; $resn_exch{"H62A VAX"}="2H6  VAX"; $resn_exch{"H81A VAX"}="1H8  VAX";
$resn_exch{" H82 VAX"}="2H8  VAX"; $resn_exch{"H83A VAX"}="3H8  VAX"; $resn_exch{" H91 VAX"}="1H9  VAX";
$resn_exch{" H92 VAX"}="2H9  VAX"; $resn_exch{" H93 VAX"}="3H9  VAX"; $resn_exch{"H281 VAX"}="1H28 VAX";
$resn_exch{"H282 VAX"}="2H28 VAX"; $resn_exch{"HN11 VAX"}="1HN1 VAX"; $resn_exch{"HN12 VAX"}="2HN1 VAX";
$resn_exch{"H841 VAX"}="1H84 VAX"; $resn_exch{"H842 VAX"}="2H84 VAX"; $resn_exch{"H843 VAX"}="3H84 VAX";
$resn_exch{"HN81 VAX"}="1HN8 VAX"; $resn_exch{"HN82 VAX"}="2HN8 VAX"; $resn_exch{"HN83 VAX"}="3HN8 VAX";
$resn_exch{"H891 VAX"}="1H89 VAX"; $resn_exch{"H892 VAX"}="2H89 VAX"; $resn_exch{"H893 VAX"}="3H89 VAX";
$resn_exch{"H901 VAX"}="1H90 VAX"; $resn_exch{"H902 VAX"}="2H90 VAX"; $resn_exch{"HO3' VAX"}="'HO3 VAX";
$resn_exch{"HO4' VAX"}="'HO4 VAX"; $resn_exch{"H6'1 VAX"}="1H6' VAX"; $resn_exch{"H6'2 VAX"}="2H6' VAX";
$resn_exch{"HO6' VAX"}="'HO6 VAX"; $resn_exch{" H1B VAX"}=" H1\" VAX"; $resn_exch{"H2'1 VAX"}="1H2\" VAX";
$resn_exch{"H2'2 VAX"}="2H2\" VAX"; $resn_exch{"HN31 VAX"}="1HN3 VAX"; $resn_exch{"HN32 VAX"}="2HN3 VAX";
$resn_exch{"HN33 VAX"}="3HN3 VAX"; $resn_exch{"H3M1 VAX"}="1H3M VAX"; $resn_exch{"H3M2 VAX"}="2H3M VAX";
$resn_exch{"H3M3 VAX"}="3H3M VAX"; $resn_exch{" H4B VAX"}=" H4\" VAX"; $resn_exch{"HO4A VAX"}="\"HO4 VAX";
$resn_exch{" H5B VAX"}=" H5\" VAX"; $resn_exch{"H5M1 VAX"}="1H5M VAX"; $resn_exch{"H5M2 VAX"}="2H5M VAX";
$resn_exch{"H5M3 VAX"}="3H5M VAX"; $resn_exch{"H8C1 VBZ"}="1H8C VBZ"; $resn_exch{"H8C2 VBZ"}="2H8C VBZ";
$resn_exch{"H6C1 VBZ"}="1H6C VBZ"; $resn_exch{"H6C2 VBZ"}="2H6C VBZ"; $resn_exch{"H7C1 VBZ"}="1H7C VBZ";
$resn_exch{"H7C2 VBZ"}="2H7C VBZ"; $resn_exch{"H9C1 VBZ"}="1H9C VBZ"; $resn_exch{"H9C2 VBZ"}="2H9C VBZ";
$resn_exch{" H21 VCA"}="1H2  VCA"; $resn_exch{" H22 VCA"}="2H2  VCA"; $resn_exch{" H31 VCA"}="1H3  VCA";
$resn_exch{" H32 VCA"}="2H3  VCA"; $resn_exch{" H41 VCA"}="1H4  VCA"; $resn_exch{" H42 VCA"}="2H4  VCA";
$resn_exch{" H51 VCA"}="1H5  VCA"; $resn_exch{" H52 VCA"}="2H5  VCA"; $resn_exch{" H61 VCA"}="1H6  VCA";
$resn_exch{" H62 VCA"}="2H6  VCA"; $resn_exch{" H71 VCA"}="1H7  VCA"; $resn_exch{" H72 VCA"}="2H7  VCA";
$resn_exch{" H81 VCA"}="1H8  VCA"; $resn_exch{" H82 VCA"}="2H8  VCA"; $resn_exch{" H91 VCA"}="1H9  VCA";
$resn_exch{" H92 VCA"}="2H9  VCA"; $resn_exch{"H101 VCA"}="1H10 VCA"; $resn_exch{"H102 VCA"}="2H10 VCA";
$resn_exch{"H131 VCA"}="1H13 VCA"; $resn_exch{"H132 VCA"}="2H13 VCA"; $resn_exch{"H141 VCA"}="1H14 VCA";
$resn_exch{"H142 VCA"}="2H14 VCA"; $resn_exch{"H151 VCA"}="1H15 VCA"; $resn_exch{"H152 VCA"}="2H15 VCA";
$resn_exch{"H161 VCA"}="1H16 VCA"; $resn_exch{"H162 VCA"}="2H16 VCA"; $resn_exch{"H171 VCA"}="1H17 VCA";
$resn_exch{"H172 VCA"}="2H17 VCA"; $resn_exch{"H181 VCA"}="1H18 VCA"; $resn_exch{"H182 VCA"}="2H18 VCA";
$resn_exch{"H183 VCA"}="3H18 VCA"; $resn_exch{"H281 VD1"}="1H28 VD1"; $resn_exch{"H282 VD1"}="2H28 VD1";
$resn_exch{" H41 VD1"}="1H4  VD1"; $resn_exch{" H42 VD1"}="2H4  VD1"; $resn_exch{"H101 VD1"}="1H10 VD1";
$resn_exch{"H102 VD1"}="2H10 VD1"; $resn_exch{" H91 VD1"}="1H9  VD1"; $resn_exch{" H92 VD1"}="2H9  VD1";
$resn_exch{"H111 VD1"}="1H11 VD1"; $resn_exch{"H112 VD1"}="2H11 VD1"; $resn_exch{"H121 VD1"}="1H12 VD1";
$resn_exch{"H122 VD1"}="2H12 VD1"; $resn_exch{"H181 VD1"}="1H18 VD1"; $resn_exch{"H182 VD1"}="2H18 VD1";
$resn_exch{"H183 VD1"}="3H18 VD1"; $resn_exch{"H151 VD1"}="1H15 VD1"; $resn_exch{"H152 VD1"}="2H15 VD1";
$resn_exch{"H161 VD1"}="1H16 VD1"; $resn_exch{"H162 VD1"}="2H16 VD1"; $resn_exch{"H211 VD1"}="1H21 VD1";
$resn_exch{"H212 VD1"}="2H21 VD1"; $resn_exch{"H213 VD1"}="3H21 VD1"; $resn_exch{"H221 VD1"}="1H22 VD1";
$resn_exch{"H222 VD1"}="2H22 VD1"; $resn_exch{"H231 VD1"}="1H23 VD1"; $resn_exch{"H232 VD1"}="2H23 VD1";
$resn_exch{"H233 VD1"}="3H23 VD1"; $resn_exch{"H271 VD2"}="1H27 VD2"; $resn_exch{"H272 VD2"}="2H27 VD2";
$resn_exch{"H273 VD2"}="3H27 VD2"; $resn_exch{"H261 VD2"}="1H26 VD2"; $resn_exch{"H262 VD2"}="2H26 VD2";
$resn_exch{"H263 VD2"}="3H26 VD2"; $resn_exch{"H241 VD2"}="1H24 VD2"; $resn_exch{"H242 VD2"}="2H24 VD2";
$resn_exch{"H231 VD2"}="1H23 VD2"; $resn_exch{"H232 VD2"}="2H23 VD2"; $resn_exch{"H221 VD2"}="1H22 VD2";
$resn_exch{"H222 VD2"}="2H22 VD2"; $resn_exch{"H211 VD2"}="1H21 VD2"; $resn_exch{"H212 VD2"}="2H21 VD2";
$resn_exch{"H213 VD2"}="3H21 VD2"; $resn_exch{"H161 VD2"}="1H16 VD2"; $resn_exch{"H162 VD2"}="2H16 VD2";
$resn_exch{"H181 VD2"}="1H18 VD2"; $resn_exch{"H182 VD2"}="2H18 VD2"; $resn_exch{"H183 VD2"}="3H18 VD2";
$resn_exch{"H151 VD2"}="1H15 VD2"; $resn_exch{"H152 VD2"}="2H15 VD2"; $resn_exch{"H121 VD2"}="1H12 VD2";
$resn_exch{"H122 VD2"}="2H12 VD2"; $resn_exch{"H111 VD2"}="1H11 VD2"; $resn_exch{"H112 VD2"}="2H11 VD2";
$resn_exch{" H91 VD2"}="1H9  VD2"; $resn_exch{" H92 VD2"}="2H9  VD2"; $resn_exch{"H101 VD2"}="1H10 VD2";
$resn_exch{"H102 VD2"}="2H10 VD2"; $resn_exch{" H41 VD2"}="1H4  VD2"; $resn_exch{" H42 VD2"}="2H4  VD2";
$resn_exch{"H281 VD2"}="1H28 VD2"; $resn_exch{"H282 VD2"}="2H28 VD2"; $resn_exch{"H283 VD2"}="3H28 VD2";
$resn_exch{" H11 VD3"}="1H1  VD3"; $resn_exch{" H12 VD3"}="2H1  VD3"; $resn_exch{" H21 VD3"}="1H2  VD3";
$resn_exch{" H22 VD3"}="2H2  VD3"; $resn_exch{" H41 VD3"}="1H4  VD3"; $resn_exch{" H42 VD3"}="2H4  VD3";
$resn_exch{"H101 VD3"}="1H10 VD3"; $resn_exch{"H102 VD3"}="2H10 VD3"; $resn_exch{"H111 VD3"}="1H11 VD3";
$resn_exch{"H112 VD3"}="2H11 VD3"; $resn_exch{"H121 VD3"}="1H12 VD3"; $resn_exch{"H122 VD3"}="2H12 VD3";
$resn_exch{"H151 VD3"}="1H15 VD3"; $resn_exch{"H152 VD3"}="2H15 VD3"; $resn_exch{"H161 VD3"}="1H16 VD3";
$resn_exch{"H162 VD3"}="2H16 VD3"; $resn_exch{"H181 VD3"}="1H18 VD3"; $resn_exch{"H182 VD3"}="2H18 VD3";
$resn_exch{"H183 VD3"}="3H18 VD3"; $resn_exch{"H191 VD3"}="1H19 VD3"; $resn_exch{"H192 VD3"}="2H19 VD3";
$resn_exch{"H211 VD3"}="1H21 VD3"; $resn_exch{"H212 VD3"}="2H21 VD3"; $resn_exch{"H213 VD3"}="3H21 VD3";
$resn_exch{"H221 VD3"}="1H22 VD3"; $resn_exch{"H222 VD3"}="2H22 VD3"; $resn_exch{"H231 VD3"}="1H23 VD3";
$resn_exch{"H232 VD3"}="2H23 VD3"; $resn_exch{"H241 VD3"}="1H24 VD3"; $resn_exch{"H242 VD3"}="2H24 VD3";
$resn_exch{"H261 VD3"}="1H26 VD3"; $resn_exch{"H262 VD3"}="2H26 VD3"; $resn_exch{"H263 VD3"}="3H26 VD3";
$resn_exch{"H271 VD3"}="1H27 VD3"; $resn_exch{"H272 VD3"}="2H27 VD3"; $resn_exch{"H273 VD3"}="3H27 VD3";
$resn_exch{"H261 VD4"}="1H26 VD4"; $resn_exch{"H262 VD4"}="2H26 VD4"; $resn_exch{"H263 VD4"}="3H26 VD4";
$resn_exch{"H271 VD4"}="1H27 VD4"; $resn_exch{"H272 VD4"}="2H27 VD4"; $resn_exch{"H273 VD4"}="3H27 VD4";
$resn_exch{"H241 VD4"}="1H24 VD4"; $resn_exch{"H242 VD4"}="2H24 VD4"; $resn_exch{"H231 VD4"}="1H23 VD4";
$resn_exch{"H232 VD4"}="2H23 VD4"; $resn_exch{"H221 VD4"}="1H22 VD4"; $resn_exch{"H222 VD4"}="2H22 VD4";
$resn_exch{"H211 VD4"}="1H21 VD4"; $resn_exch{"H212 VD4"}="2H21 VD4"; $resn_exch{"H213 VD4"}="3H21 VD4";
$resn_exch{"H181 VD4"}="1H18 VD4"; $resn_exch{"H182 VD4"}="2H18 VD4"; $resn_exch{"H183 VD4"}="3H18 VD4";
$resn_exch{"H121 VD4"}="1H12 VD4"; $resn_exch{"H122 VD4"}="2H12 VD4"; $resn_exch{"H111 VD4"}="1H11 VD4";
$resn_exch{"H112 VD4"}="2H11 VD4"; $resn_exch{" H91 VD4"}="1H9  VD4"; $resn_exch{" H92 VD4"}="2H9  VD4";
$resn_exch{"H161 VD4"}="1H16 VD4"; $resn_exch{"H162 VD4"}="2H16 VD4"; $resn_exch{"H151 VD4"}="1H15 VD4";
$resn_exch{"H152 VD4"}="2H15 VD4"; $resn_exch{"H101 VD4"}="1H10 VD4"; $resn_exch{"H102 VD4"}="2H10 VD4";
$resn_exch{" H41 VD4"}="1H4  VD4"; $resn_exch{" H42 VD4"}="2H4  VD4"; $resn_exch{"H281 VD4"}="1H28 VD4";
$resn_exch{"H282 VD4"}="2H28 VD4"; $resn_exch{"H281 VD5"}="1H28 VD5"; $resn_exch{"H282 VD5"}="2H28 VD5";
$resn_exch{"H101 VD5"}="1H10 VD5"; $resn_exch{"H102 VD5"}="2H10 VD5"; $resn_exch{" H41 VD5"}="1H4  VD5";
$resn_exch{" H42 VD5"}="2H4  VD5"; $resn_exch{" H91 VD5"}="1H9  VD5"; $resn_exch{" H92 VD5"}="2H9  VD5";
$resn_exch{"H111 VD5"}="1H11 VD5"; $resn_exch{"H112 VD5"}="2H11 VD5"; $resn_exch{"H121 VD5"}="1H12 VD5";
$resn_exch{"H122 VD5"}="2H12 VD5"; $resn_exch{"H181 VD5"}="1H18 VD5"; $resn_exch{"H182 VD5"}="2H18 VD5";
$resn_exch{"H183 VD5"}="3H18 VD5"; $resn_exch{"H151 VD5"}="1H15 VD5"; $resn_exch{"H152 VD5"}="2H15 VD5";
$resn_exch{"H161 VD5"}="1H16 VD5"; $resn_exch{"H162 VD5"}="2H16 VD5"; $resn_exch{"H211 VD5"}="1H21 VD5";
$resn_exch{"H212 VD5"}="2H21 VD5"; $resn_exch{"H213 VD5"}="3H21 VD5"; $resn_exch{"H221 VD5"}="1H22 VD5";
$resn_exch{"H222 VD5"}="2H22 VD5"; $resn_exch{"H231 VD5"}="1H23 VD5"; $resn_exch{"H232 VD5"}="2H23 VD5";
$resn_exch{"H241 VD5"}="1H24 VD5"; $resn_exch{"H242 VD5"}="2H24 VD5"; $resn_exch{"H271 VD5"}="1H27 VD5";
$resn_exch{"H272 VD5"}="2H27 VD5"; $resn_exch{"H273 VD5"}="3H27 VD5"; $resn_exch{"H261 VD5"}="1H26 VD5";
$resn_exch{"H262 VD5"}="2H26 VD5"; $resn_exch{"H263 VD5"}="3H26 VD5"; $resn_exch{" HN1 VDL"}="1HN  VDL";
$resn_exch{" HN2 VDL"}="2HN  VDL"; $resn_exch{"HG11 VDL"}="1HG1 VDL"; $resn_exch{"HG12 VDL"}="2HG1 VDL";
$resn_exch{"HG13 VDL"}="3HG1 VDL"; $resn_exch{"HG21 VDL"}="1HG2 VDL"; $resn_exch{"HG22 VDL"}="2HG2 VDL";
$resn_exch{"H7C1 VDM"}="1H7C VDM"; $resn_exch{"H7C2 VDM"}="2H7C VDM"; $resn_exch{" H1' VDM"}=" H1* VDM";
$resn_exch{" H2' VDM"}=" H2* VDM"; $resn_exch{"H6'1 VDM"}="1H6* VDM"; $resn_exch{"H6'2 VDM"}="2H6* VDM";
$resn_exch{" H3' VDM"}=" H3* VDM"; $resn_exch{" H5' VDM"}=" H5* VDM"; $resn_exch{"H7'1 VDM"}="1H7* VDM";
$resn_exch{"H7'2 VDM"}="2H7* VDM"; $resn_exch{" H4' VDM"}=" H4* VDM"; $resn_exch{" H7' VDM"}=" H7* VDM";
$resn_exch{" H11 VDN"}="1H1  VDN"; $resn_exch{" H12 VDN"}="2H1  VDN"; $resn_exch{" H13 VDN"}="3H1  VDN";
$resn_exch{" H21 VDN"}="1H2  VDN"; $resn_exch{" H22 VDN"}="2H2  VDN"; $resn_exch{"H131 VDN"}="1H13 VDN";
$resn_exch{"H132 VDN"}="2H13 VDN"; $resn_exch{"H133 VDN"}="3H13 VDN"; $resn_exch{"H151 VDN"}="1H15 VDN";
$resn_exch{"H152 VDN"}="2H15 VDN"; $resn_exch{"H161 VDN"}="1H16 VDN"; $resn_exch{"H162 VDN"}="2H16 VDN";
$resn_exch{"H181 VDN"}="1H18 VDN"; $resn_exch{"H182 VDN"}="2H18 VDN"; $resn_exch{"H191 VDN"}="1H19 VDN";
$resn_exch{"H192 VDN"}="2H19 VDN"; $resn_exch{"H201 VDN"}="1H20 VDN"; $resn_exch{"H202 VDN"}="2H20 VDN";
$resn_exch{"H311 VDN"}="1H31 VDN"; $resn_exch{"H312 VDN"}="2H31 VDN"; $resn_exch{"H313 VDN"}="3H31 VDN";
$resn_exch{"H321 VDN"}="1H32 VDN"; $resn_exch{"H322 VDN"}="2H32 VDN"; $resn_exch{"H331 VDN"}="1H33 VDN";
$resn_exch{"H332 VDN"}="2H33 VDN"; $resn_exch{"H341 VDN"}="1H34 VDN"; $resn_exch{"H342 VDN"}="2H34 VDN";
$resn_exch{"H343 VDN"}="3H34 VDN"; $resn_exch{" H11 VDX"}="1H1  VDX"; $resn_exch{" H21 VDX"}="1H2  VDX";
$resn_exch{" H22 VDX"}="2H2  VDX"; $resn_exch{" H31 VDX"}="1H3  VDX"; $resn_exch{" H41 VDX"}="1H4  VDX";
$resn_exch{" H42 VDX"}="2H4  VDX"; $resn_exch{" H61 VDX"}="1H6  VDX"; $resn_exch{" H71 VDX"}="1H7  VDX";
$resn_exch{" H91 VDX"}="1H9  VDX"; $resn_exch{" H92 VDX"}="2H9  VDX"; $resn_exch{"H111 VDX"}="1H11 VDX";
$resn_exch{"H112 VDX"}="2H11 VDX"; $resn_exch{"H121 VDX"}="1H12 VDX"; $resn_exch{"H122 VDX"}="2H12 VDX";
$resn_exch{"H141 VDX"}="1H14 VDX"; $resn_exch{"H151 VDX"}="1H15 VDX"; $resn_exch{"H152 VDX"}="2H15 VDX";
$resn_exch{"H161 VDX"}="1H16 VDX"; $resn_exch{"H162 VDX"}="2H16 VDX"; $resn_exch{"H171 VDX"}="1H17 VDX";
$resn_exch{"H181 VDX"}="1H18 VDX"; $resn_exch{"H182 VDX"}="2H18 VDX"; $resn_exch{"H183 VDX"}="3H18 VDX";
$resn_exch{"H191 VDX"}="1H19 VDX"; $resn_exch{"H192 VDX"}="2H19 VDX"; $resn_exch{"H201 VDX"}="1H20 VDX";
$resn_exch{"H211 VDX"}="1H21 VDX"; $resn_exch{"H212 VDX"}="2H21 VDX"; $resn_exch{"H213 VDX"}="3H21 VDX";
$resn_exch{"H221 VDX"}="1H22 VDX"; $resn_exch{"H222 VDX"}="2H22 VDX"; $resn_exch{"H231 VDX"}="1H23 VDX";
$resn_exch{"H232 VDX"}="2H23 VDX"; $resn_exch{"H241 VDX"}="1H24 VDX"; $resn_exch{"H242 VDX"}="2H24 VDX";
$resn_exch{"H261 VDX"}="1H26 VDX"; $resn_exch{"H262 VDX"}="2H26 VDX"; $resn_exch{"H263 VDX"}="3H26 VDX";
$resn_exch{"H271 VDX"}="1H27 VDX"; $resn_exch{"H272 VDX"}="2H27 VDX"; $resn_exch{"H273 VDX"}="3H27 VDX";
$resn_exch{" H11 VDY"}="1H1  VDY"; $resn_exch{" H12 VDY"}="2H1  VDY"; $resn_exch{" H21 VDY"}="1H2  VDY";
$resn_exch{" H22 VDY"}="2H2  VDY"; $resn_exch{" H31 VDY"}="1H3  VDY"; $resn_exch{" H41 VDY"}="1H4  VDY";
$resn_exch{" H42 VDY"}="2H4  VDY"; $resn_exch{" H61 VDY"}="1H6  VDY"; $resn_exch{" H71 VDY"}="1H7  VDY";
$resn_exch{" H91 VDY"}="1H9  VDY"; $resn_exch{" H92 VDY"}="2H9  VDY"; $resn_exch{"H111 VDY"}="1H11 VDY";
$resn_exch{"H112 VDY"}="2H11 VDY"; $resn_exch{"H121 VDY"}="1H12 VDY"; $resn_exch{"H122 VDY"}="2H12 VDY";
$resn_exch{"H141 VDY"}="1H14 VDY"; $resn_exch{"H151 VDY"}="1H15 VDY"; $resn_exch{"H152 VDY"}="2H15 VDY";
$resn_exch{"H161 VDY"}="1H16 VDY"; $resn_exch{"H162 VDY"}="2H16 VDY"; $resn_exch{"H171 VDY"}="1H17 VDY";
$resn_exch{"H181 VDY"}="1H18 VDY"; $resn_exch{"H182 VDY"}="2H18 VDY"; $resn_exch{"H183 VDY"}="3H18 VDY";
$resn_exch{"H191 VDY"}="1H19 VDY"; $resn_exch{"H192 VDY"}="2H19 VDY"; $resn_exch{"H201 VDY"}="1H20 VDY";
$resn_exch{"H211 VDY"}="1H21 VDY"; $resn_exch{"H212 VDY"}="2H21 VDY"; $resn_exch{"H213 VDY"}="3H21 VDY";
$resn_exch{"H221 VDY"}="1H22 VDY"; $resn_exch{"H222 VDY"}="2H22 VDY"; $resn_exch{"H231 VDY"}="1H23 VDY";
$resn_exch{"H232 VDY"}="2H23 VDY"; $resn_exch{"H241 VDY"}="1H24 VDY"; $resn_exch{"H242 VDY"}="2H24 VDY";
$resn_exch{"H261 VDY"}="1H26 VDY"; $resn_exch{"H262 VDY"}="2H26 VDY"; $resn_exch{"H263 VDY"}="3H26 VDY";
$resn_exch{"H271 VDY"}="1H27 VDY"; $resn_exch{"H272 VDY"}="2H27 VDY"; $resn_exch{"H273 VDY"}="3H27 VDY";
$resn_exch{"H281 VDZ"}="1H28 VDZ"; $resn_exch{"H282 VDZ"}="2H28 VDZ"; $resn_exch{" H41 VDZ"}="1H4  VDZ";
$resn_exch{" H42 VDZ"}="2H4  VDZ"; $resn_exch{"H101 VDZ"}="1H10 VDZ"; $resn_exch{"H102 VDZ"}="2H10 VDZ";
$resn_exch{" H91 VDZ"}="1H9  VDZ"; $resn_exch{" H92 VDZ"}="2H9  VDZ"; $resn_exch{"H111 VDZ"}="1H11 VDZ";
$resn_exch{"H112 VDZ"}="2H11 VDZ"; $resn_exch{"H121 VDZ"}="1H12 VDZ"; $resn_exch{"H122 VDZ"}="2H12 VDZ";
$resn_exch{"H181 VDZ"}="1H18 VDZ"; $resn_exch{"H182 VDZ"}="2H18 VDZ"; $resn_exch{"H183 VDZ"}="3H18 VDZ";
$resn_exch{"H151 VDZ"}="1H15 VDZ"; $resn_exch{"H152 VDZ"}="2H15 VDZ"; $resn_exch{"H161 VDZ"}="1H16 VDZ";
$resn_exch{"H162 VDZ"}="2H16 VDZ"; $resn_exch{"H211 VDZ"}="1H21 VDZ"; $resn_exch{"H212 VDZ"}="2H21 VDZ";
$resn_exch{"H213 VDZ"}="3H21 VDZ"; $resn_exch{"H221 VDZ"}="1H22 VDZ"; $resn_exch{"H222 VDZ"}="2H22 VDZ";
$resn_exch{"H231 VDZ"}="1H23 VDZ"; $resn_exch{"H232 VDZ"}="2H23 VDZ"; $resn_exch{"H241 VDZ"}="1H24 VDZ";
$resn_exch{"H242 VDZ"}="2H24 VDZ"; $resn_exch{"H261 VDZ"}="1H26 VDZ"; $resn_exch{"H262 VDZ"}="2H26 VDZ";
$resn_exch{"H263 VDZ"}="3H26 VDZ"; $resn_exch{"H271 VDZ"}="1H27 VDZ"; $resn_exch{"H272 VDZ"}="2H27 VDZ";
$resn_exch{"H273 VDZ"}="3H27 VDZ"; $resn_exch{"HMB1 VER"}="1HMB VER"; $resn_exch{"HMB2 VER"}="2HMB VER";
$resn_exch{"HMB3 VER"}="3HMB VER"; $resn_exch{"HBB1 VER"}="1HBB VER"; $resn_exch{"HBB2 VER"}="2HBB VER";
$resn_exch{"HMC1 VER"}="1HMC VER"; $resn_exch{"HMC2 VER"}="2HMC VER"; $resn_exch{"HMC3 VER"}="3HMC VER";
$resn_exch{"HBC1 VER"}="1HBC VER"; $resn_exch{"HBC2 VER"}="2HBC VER"; $resn_exch{"HMD1 VER"}="1HMD VER";
$resn_exch{"HMD2 VER"}="2HMD VER"; $resn_exch{"HMD3 VER"}="3HMD VER"; $resn_exch{"HAD1 VER"}="1HAD VER";
$resn_exch{"HAD2 VER"}="2HAD VER"; $resn_exch{"HBD1 VER"}="1HBD VER"; $resn_exch{"HBD2 VER"}="2HBD VER";
$resn_exch{" H11 VER"}="1H1  VER"; $resn_exch{" H12 VER"}="2H1  VER"; $resn_exch{" H13 VER"}="3H1  VER";
$resn_exch{"H5C1 VG0"}="1H5C VG0"; $resn_exch{"H5C2 VG0"}="2H5C VG0"; $resn_exch{"H6C1 VG0"}="1H6C VG0";
$resn_exch{"H6C2 VG0"}="2H6C VG0"; $resn_exch{"H391 VG0"}="1H39 VG0"; $resn_exch{"H392 VG0"}="2H39 VG0";
$resn_exch{"H401 VG0"}="1H40 VG0"; $resn_exch{"H402 VG0"}="2H40 VG0"; $resn_exch{"H403 VG0"}="3H40 VG0";
$resn_exch{"H471 VG0"}="1H47 VG0"; $resn_exch{"H472 VG0"}="2H47 VG0"; $resn_exch{"H481 VG0"}="1H48 VG0";
$resn_exch{"H482 VG0"}="2H48 VG0"; $resn_exch{"H491 VG0"}="1H49 VG0"; $resn_exch{"H492 VG0"}="2H49 VG0";
$resn_exch{"H611 VG0"}="1H61 VG0"; $resn_exch{"H612 VG0"}="2H61 VG0"; $resn_exch{"H613 VG0"}="3H61 VG0";
$resn_exch{"H651 VG0"}="1H65 VG0"; $resn_exch{"H652 VG0"}="2H65 VG0"; $resn_exch{"H653 VG0"}="3H65 VG0";
$resn_exch{"H691 VG0"}="1H69 VG0"; $resn_exch{"H692 VG0"}="2H69 VG0"; $resn_exch{"H701 VG0"}="1H70 VG0";
$resn_exch{"H702 VG0"}="2H70 VG0"; $resn_exch{"H731 VG0"}="1H73 VG0"; $resn_exch{"H732 VG0"}="2H73 VG0";
$resn_exch{"H791 VG0"}="1H79 VG0"; $resn_exch{"H792 VG0"}="2H79 VG0"; $resn_exch{"H793 VG0"}="3H79 VG0";
$resn_exch{"H831 VG0"}="1H83 VG0"; $resn_exch{"H832 VG0"}="2H83 VG0"; $resn_exch{"H833 VG0"}="3H83 VG0";
$resn_exch{"HOP1 VG1"}="1HOP VG1"; $resn_exch{"HOP2 VG1"}="2HOP VG1"; $resn_exch{"H2C1 VG2"}="1H2C VG2";
$resn_exch{"H2C2 VG2"}="2H2C VG2"; $resn_exch{"H6C1 VG2"}="1H6C VG2"; $resn_exch{"H6C2 VG2"}="2H6C VG2";
$resn_exch{"H3C1 VG2"}="1H3C VG2"; $resn_exch{"H3C2 VG2"}="2H3C VG2"; $resn_exch{"H4C1 VG2"}="1H4C VG2";
$resn_exch{"H4C2 VG2"}="2H4C VG2"; $resn_exch{"H5C1 VG2"}="1H5C VG2"; $resn_exch{"H5C2 VG2"}="2H5C VG2";
$resn_exch{"H191 VG2"}="1H19 VG2"; $resn_exch{"H192 VG2"}="2H19 VG2"; $resn_exch{"H193 VG2"}="3H19 VG2";
$resn_exch{"H221 VG2"}="1H22 VG2"; $resn_exch{"H222 VG2"}="2H22 VG2"; $resn_exch{"H231 VG2"}="1H23 VG2";
$resn_exch{"H232 VG2"}="2H23 VG2"; $resn_exch{"H241 VG2"}="1H24 VG2"; $resn_exch{"H242 VG2"}="2H24 VG2";
$resn_exch{"H291 VG2"}="1H29 VG2"; $resn_exch{"H292 VG2"}="2H29 VG2"; $resn_exch{"H293 VG2"}="3H29 VG2";
$resn_exch{"H5C1 VG3"}="1H5C VG3"; $resn_exch{"H5C2 VG3"}="2H5C VG3"; $resn_exch{"H211 VG3"}="1H21 VG3";
$resn_exch{"H212 VG3"}="2H21 VG3"; $resn_exch{"H241 VG3"}="1H24 VG3"; $resn_exch{"H242 VG3"}="2H24 VG3";
$resn_exch{"H243 VG3"}="3H24 VG3"; $resn_exch{"H601 VG3"}="1H60 VG3"; $resn_exch{"H602 VG3"}="2H60 VG3";
$resn_exch{"H431 VG3"}="1H43 VG3"; $resn_exch{"H432 VG3"}="2H43 VG3"; $resn_exch{"H471 VG3"}="1H47 VG3";
$resn_exch{"H472 VG3"}="2H47 VG3"; $resn_exch{"H441 VG3"}="1H44 VG3"; $resn_exch{"H442 VG3"}="2H44 VG3";
$resn_exch{"H451 VG3"}="1H45 VG3"; $resn_exch{"H452 VG3"}="2H45 VG3"; $resn_exch{"H461 VG3"}="1H46 VG3";
$resn_exch{"H462 VG3"}="2H46 VG3"; $resn_exch{"H611 VG3"}="1H61 VG3"; $resn_exch{"H612 VG3"}="2H61 VG3";
$resn_exch{"H641 VG3"}="1H64 VG3"; $resn_exch{"H642 VG3"}="2H64 VG3"; $resn_exch{"H671 VG3"}="1H67 VG3";
$resn_exch{"H672 VG3"}="2H67 VG3"; $resn_exch{"H701 VG3"}="1H70 VG3"; $resn_exch{"H702 VG3"}="2H70 VG3";
$resn_exch{"H703 VG3"}="3H70 VG3"; $resn_exch{"H111 VG4"}="1H11 VG4"; $resn_exch{"H112 VG4"}="2H11 VG4";
$resn_exch{"H151 VG4"}="1H15 VG4"; $resn_exch{"H152 VG4"}="2H15 VG4"; $resn_exch{"H221 VG4"}="1H22 VG4";
$resn_exch{"H222 VG4"}="2H22 VG4"; $resn_exch{"H223 VG4"}="3H22 VG4"; $resn_exch{"H231 VG4"}="1H23 VG4";
$resn_exch{"H232 VG4"}="2H23 VG4"; $resn_exch{"H271 VG4"}="1H27 VG4"; $resn_exch{"H272 VG4"}="2H27 VG4";
$resn_exch{"H241 VG4"}="1H24 VG4"; $resn_exch{"H242 VG4"}="2H24 VG4"; $resn_exch{"H251 VG4"}="1H25 VG4";
$resn_exch{"H252 VG4"}="2H25 VG4"; $resn_exch{"H261 VG4"}="1H26 VG4"; $resn_exch{"H262 VG4"}="2H26 VG4";
$resn_exch{"H671 VG4"}="1H67 VG4"; $resn_exch{"H672 VG4"}="2H67 VG4"; $resn_exch{"H681 VG4"}="1H68 VG4";
$resn_exch{"H682 VG4"}="2H68 VG4"; $resn_exch{"H711 VG4"}="1H71 VG4"; $resn_exch{"H712 VG4"}="2H71 VG4";
$resn_exch{"H713 VG4"}="3H71 VG4"; $resn_exch{"H801 VG4"}="1H80 VG4"; $resn_exch{"H802 VG4"}="2H80 VG4";
$resn_exch{"H111 VG5"}="1H11 VG5"; $resn_exch{"H112 VG5"}="2H11 VG5"; $resn_exch{"H151 VG5"}="1H15 VG5";
$resn_exch{"H152 VG5"}="2H15 VG5"; $resn_exch{"H221 VG5"}="1H22 VG5"; $resn_exch{"H222 VG5"}="2H22 VG5";
$resn_exch{"H223 VG5"}="3H22 VG5"; $resn_exch{"H231 VG5"}="1H23 VG5"; $resn_exch{"H232 VG5"}="2H23 VG5";
$resn_exch{"H271 VG5"}="1H27 VG5"; $resn_exch{"H272 VG5"}="2H27 VG5"; $resn_exch{"H241 VG5"}="1H24 VG5";
$resn_exch{"H242 VG5"}="2H24 VG5"; $resn_exch{"H251 VG5"}="1H25 VG5"; $resn_exch{"H252 VG5"}="2H25 VG5";
$resn_exch{"H261 VG5"}="1H26 VG5"; $resn_exch{"H262 VG5"}="2H26 VG5"; $resn_exch{"H671 VG5"}="1H67 VG5";
$resn_exch{"H672 VG5"}="2H67 VG5"; $resn_exch{"H681 VG5"}="1H68 VG5"; $resn_exch{"H682 VG5"}="2H68 VG5";
$resn_exch{"H683 VG5"}="3H68 VG5"; $resn_exch{"H801 VG5"}="1H80 VG5"; $resn_exch{"H802 VG5"}="2H80 VG5";
$resn_exch{"H111 VG6"}="1H11 VG6"; $resn_exch{"H112 VG6"}="2H11 VG6"; $resn_exch{"H151 VG6"}="1H15 VG6";
$resn_exch{"H152 VG6"}="2H15 VG6"; $resn_exch{"H591 VG6"}="1H59 VG6"; $resn_exch{"H592 VG6"}="2H59 VG6";
$resn_exch{"H391 VG6"}="1H39 VG6"; $resn_exch{"H392 VG6"}="2H39 VG6"; $resn_exch{"H411 VG6"}="1H41 VG6";
$resn_exch{"H412 VG6"}="2H41 VG6"; $resn_exch{"H413 VG6"}="3H41 VG6"; $resn_exch{"H511 VG6"}="1H51 VG6";
$resn_exch{"H512 VG6"}="2H51 VG6"; $resn_exch{"H491 VG6"}="1H49 VG6"; $resn_exch{"H492 VG6"}="2H49 VG6";
$resn_exch{"H501 VG6"}="1H50 VG6"; $resn_exch{"H502 VG6"}="2H50 VG6"; $resn_exch{"H111 VG7"}="1H11 VG7";
$resn_exch{"H112 VG7"}="2H11 VG7"; $resn_exch{"H151 VG7"}="1H15 VG7"; $resn_exch{"H152 VG7"}="2H15 VG7";
$resn_exch{"H391 VG7"}="1H39 VG7"; $resn_exch{"H392 VG7"}="2H39 VG7"; $resn_exch{"H411 VG7"}="1H41 VG7";
$resn_exch{"H412 VG7"}="2H41 VG7"; $resn_exch{"H413 VG7"}="3H41 VG7"; $resn_exch{"H511 VG7"}="1H51 VG7";
$resn_exch{"H512 VG7"}="2H51 VG7"; $resn_exch{"H491 VG7"}="1H49 VG7"; $resn_exch{"H492 VG7"}="2H49 VG7";
$resn_exch{"H501 VG7"}="1H50 VG7"; $resn_exch{"H502 VG7"}="2H50 VG7"; $resn_exch{"H611 VG7"}="1H61 VG7";
$resn_exch{"H612 VG7"}="2H61 VG7"; $resn_exch{"H651 VG7"}="1H65 VG7"; $resn_exch{"H652 VG7"}="2H65 VG7";
$resn_exch{"H621 VG7"}="1H62 VG7"; $resn_exch{"H622 VG7"}="2H62 VG7"; $resn_exch{"H631 VG7"}="1H63 VG7";
$resn_exch{"H632 VG7"}="2H63 VG7"; $resn_exch{"H641 VG7"}="1H64 VG7"; $resn_exch{"H642 VG7"}="2H64 VG7";
$resn_exch{"H341 VIA"}="1H34 VIA"; $resn_exch{"H342 VIA"}="2H34 VIA"; $resn_exch{"H343 VIA"}="3H34 VIA";
$resn_exch{"H331 VIA"}="1H33 VIA"; $resn_exch{"H332 VIA"}="2H33 VIA"; $resn_exch{"H321 VIA"}="1H32 VIA";
$resn_exch{"H322 VIA"}="2H32 VIA"; $resn_exch{"H311 VIA"}="1H31 VIA"; $resn_exch{"H312 VIA"}="2H31 VIA";
$resn_exch{"H313 VIA"}="3H31 VIA"; $resn_exch{" H21 VIA"}="1H2  VIA"; $resn_exch{"H22A VIA"}="2H2  VIA";
$resn_exch{" H11 VIA"}="1H1  VIA"; $resn_exch{" H12 VIA"}="2H1  VIA"; $resn_exch{" H13 VIA"}="3H1  VIA";
$resn_exch{"H191 VIA"}="1H19 VIA"; $resn_exch{"H192 VIA"}="2H19 VIA"; $resn_exch{"H181 VIA"}="1H18 VIA";
$resn_exch{"H182 VIA"}="2H18 VIA"; $resn_exch{"H201 VIA"}="1H20 VIA"; $resn_exch{"H202 VIA"}="2H20 VIA";
$resn_exch{"H203 VIA"}="3H20 VIA"; $resn_exch{"H161 VIA"}="1H16 VIA"; $resn_exch{"H162 VIA"}="2H16 VIA";
$resn_exch{"H151 VIA"}="1H15 VIA"; $resn_exch{"H152 VIA"}="2H15 VIA"; $resn_exch{"HM21 VIB"}="1HM2 VIB";
$resn_exch{"HM22 VIB"}="2HM2 VIB"; $resn_exch{"HM23 VIB"}="3HM2 VIB"; $resn_exch{"H4A1 VIB"}="1H4A VIB";
$resn_exch{"H4A2 VIB"}="2H4A VIB"; $resn_exch{"HM41 VIB"}="1HM4 VIB"; $resn_exch{"HM42 VIB"}="2HM4 VIB";
$resn_exch{"HM43 VIB"}="3HM4 VIB"; $resn_exch{"HC71 VIB"}="1HC7 VIB"; $resn_exch{"HC72 VIB"}="2HC7 VIB";
$resn_exch{"HC61 VIB"}="1HC6 VIB"; $resn_exch{"HC62 VIB"}="2HC6 VIB"; $resn_exch{"H7A1 VIB"}="1H7A VIB";
$resn_exch{"H7A2 VIB"}="2H7A VIB"; $resn_exch{"H6AC VIB"}="CH6A VIB"; $resn_exch{"H1C1 VIG"}="1H1C VIG";
$resn_exch{"H1C2 VIG"}="2H1C VIG"; $resn_exch{"H2C1 VIG"}="1H2C VIG"; $resn_exch{"H2C2 VIG"}="2H2C VIG";
$resn_exch{"H4C1 VIG"}="1H4C VIG"; $resn_exch{"H4C2 VIG"}="2H4C VIG"; $resn_exch{"H5C1 VIG"}="1H5C VIG";
$resn_exch{"H5C2 VIG"}="2H5C VIG"; $resn_exch{"H5C3 VIG"}="3H5C VIG"; $resn_exch{"H3N1 VIG"}="1H3N VIG";
$resn_exch{"H3N2 VIG"}="2H3N VIG"; $resn_exch{" HB1 VIO"}="1HB  VIO"; $resn_exch{" HB2 VIO"}="2HB  VIO";
$resn_exch{" HG1 VIO"}="1HG  VIO"; $resn_exch{" HG2 VIO"}="2HG  VIO"; $resn_exch{" HD1 VIO"}="1HD  VIO";
$resn_exch{" HD2 VIO"}="2HD  VIO"; $resn_exch{"HH11 VIO"}="1HH1 VIO"; $resn_exch{"HH12 VIO"}="2HH1 VIO";
$resn_exch{"HH21 VIO"}="1HH2 VIO"; $resn_exch{"HH22 VIO"}="2HH2 VIO"; $resn_exch{" H11 VIO"}="1H1  VIO";
$resn_exch{" H12 VIO"}="2H1  VIO"; $resn_exch{" H21 VIO"}="1H2  VIO"; $resn_exch{" H22 VIO"}="2H2  VIO";
$resn_exch{" H23 VIO"}="3H2  VIO"; $resn_exch{"H131 VIR"}="1H13 VIR"; $resn_exch{"H132 VIR"}="2H13 VIR";
$resn_exch{"H161 VIR"}="1H16 VIR"; $resn_exch{"H162 VIR"}="2H16 VIR"; $resn_exch{"H211 VIR"}="1H21 VIR";
$resn_exch{"H212 VIR"}="2H21 VIR"; $resn_exch{"H213 VIR"}="3H21 VIR"; $resn_exch{"H241 VIR"}="1H24 VIR";
$resn_exch{"H242 VIR"}="2H24 VIR"; $resn_exch{"HC31 VIR"}="1HC3 VIR"; $resn_exch{"HC32 VIR"}="2HC3 VIR";
$resn_exch{"H311 VIR"}="1H31 VIR"; $resn_exch{"H312 VIR"}="2H31 VIR"; $resn_exch{"H313 VIR"}="3H31 VIR";
$resn_exch{"H341 VIR"}="1H34 VIR"; $resn_exch{"H342 VIR"}="2H34 VIR"; $resn_exch{"H343 VIR"}="3H34 VIR";
$resn_exch{"H351 VIR"}="1H35 VIR"; $resn_exch{"H352 VIR"}="2H35 VIR"; $resn_exch{"H353 VIR"}="3H35 VIR";
$resn_exch{"HC41 VIR"}="1HC4 VIR"; $resn_exch{"HC42 VIR"}="2HC4 VIR"; $resn_exch{"H18' VIR"}="'H18 VIR";
$resn_exch{"HC11 VIT"}="1HC1 VIT"; $resn_exch{"HC12 VIT"}="2HC1 VIT"; $resn_exch{"HC13 VIT"}="3HC1 VIT";
$resn_exch{"HC81 VIT"}="1HC8 VIT"; $resn_exch{"HC82 VIT"}="2HC8 VIT"; $resn_exch{"HC83 VIT"}="3HC8 VIT";
$resn_exch{"H101 VIT"}="1H10 VIT"; $resn_exch{"H102 VIT"}="2H10 VIT"; $resn_exch{"H111 VIT"}="1H11 VIT";
$resn_exch{"H112 VIT"}="2H11 VIT"; $resn_exch{"H121 VIT"}="1H12 VIT"; $resn_exch{"H122 VIT"}="2H12 VIT";
$resn_exch{"H123 VIT"}="3H12 VIT"; $resn_exch{"H131 VIT"}="1H13 VIT"; $resn_exch{"H132 VIT"}="2H13 VIT";
$resn_exch{"H133 VIT"}="3H13 VIT"; $resn_exch{"HO21 VIT"}="1HO2 VIT"; $resn_exch{"H141 VIT"}="1H14 VIT";
$resn_exch{"H142 VIT"}="2H14 VIT"; $resn_exch{"H143 VIT"}="3H14 VIT"; $resn_exch{"H151 VIT"}="1H15 VIT";
$resn_exch{"H152 VIT"}="2H15 VIT"; $resn_exch{"H161 VIT"}="1H16 VIT"; $resn_exch{"H162 VIT"}="2H16 VIT";
$resn_exch{"H171 VIT"}="1H17 VIT"; $resn_exch{"H172 VIT"}="2H17 VIT"; $resn_exch{"H191 VIT"}="1H19 VIT";
$resn_exch{"H192 VIT"}="2H19 VIT"; $resn_exch{"H193 VIT"}="3H19 VIT"; $resn_exch{"H201 VIT"}="1H20 VIT";
$resn_exch{"H202 VIT"}="2H20 VIT"; $resn_exch{"H211 VIT"}="1H21 VIT"; $resn_exch{"H212 VIT"}="2H21 VIT";
$resn_exch{"H221 VIT"}="1H22 VIT"; $resn_exch{"H222 VIT"}="2H22 VIT"; $resn_exch{"H241 VIT"}="1H24 VIT";
$resn_exch{"H242 VIT"}="2H24 VIT"; $resn_exch{"H251 VIT"}="1H25 VIT"; $resn_exch{"H252 VIT"}="2H25 VIT";
$resn_exch{"H261 VIT"}="1H26 VIT"; $resn_exch{"H262 VIT"}="2H26 VIT"; $resn_exch{"H281 VIT"}="1H28 VIT";
$resn_exch{"H282 VIT"}="2H28 VIT"; $resn_exch{"H283 VIT"}="3H28 VIT"; $resn_exch{"H291 VIT"}="1H29 VIT";
$resn_exch{"H292 VIT"}="2H29 VIT"; $resn_exch{"H293 VIT"}="3H29 VIT"; $resn_exch{"HO21 VIV"}="1HO2 VIV";
$resn_exch{"HC11 VIV"}="1HC1 VIV"; $resn_exch{"HC12 VIV"}="2HC1 VIV"; $resn_exch{"HC13 VIV"}="3HC1 VIV";
$resn_exch{"HC81 VIV"}="1HC8 VIV"; $resn_exch{"HC82 VIV"}="2HC8 VIV"; $resn_exch{"HC83 VIV"}="3HC8 VIV";
$resn_exch{"H101 VIV"}="1H10 VIV"; $resn_exch{"H102 VIV"}="2H10 VIV"; $resn_exch{"H111 VIV"}="1H11 VIV";
$resn_exch{"H112 VIV"}="2H11 VIV"; $resn_exch{"H121 VIV"}="1H12 VIV"; $resn_exch{"H122 VIV"}="2H12 VIV";
$resn_exch{"H123 VIV"}="3H12 VIV"; $resn_exch{"H131 VIV"}="1H13 VIV"; $resn_exch{"H132 VIV"}="2H13 VIV";
$resn_exch{"H133 VIV"}="3H13 VIV"; $resn_exch{"H141 VIV"}="1H14 VIV"; $resn_exch{"H142 VIV"}="2H14 VIV";
$resn_exch{"H143 VIV"}="3H14 VIV"; $resn_exch{"H151 VIV"}="1H15 VIV"; $resn_exch{"H152 VIV"}="2H15 VIV";
$resn_exch{"H161 VIV"}="1H16 VIV"; $resn_exch{"H162 VIV"}="2H16 VIV"; $resn_exch{"H171 VIV"}="1H17 VIV";
$resn_exch{"H172 VIV"}="2H17 VIV"; $resn_exch{"H191 VIV"}="1H19 VIV"; $resn_exch{"H192 VIV"}="2H19 VIV";
$resn_exch{"H193 VIV"}="3H19 VIV"; $resn_exch{"H201 VIV"}="1H20 VIV"; $resn_exch{"H202 VIV"}="2H20 VIV";
$resn_exch{"H211 VIV"}="1H21 VIV"; $resn_exch{"H212 VIV"}="2H21 VIV"; $resn_exch{"H221 VIV"}="1H22 VIV";
$resn_exch{"H222 VIV"}="2H22 VIV"; $resn_exch{"H241 VIV"}="1H24 VIV"; $resn_exch{"H242 VIV"}="2H24 VIV";
$resn_exch{"H251 VIV"}="1H25 VIV"; $resn_exch{"H252 VIV"}="2H25 VIV"; $resn_exch{"H261 VIV"}="1H26 VIV";
$resn_exch{"H262 VIV"}="2H26 VIV"; $resn_exch{"H281 VIV"}="1H28 VIV"; $resn_exch{"H282 VIV"}="2H28 VIV";
$resn_exch{"H283 VIV"}="3H28 VIV"; $resn_exch{"H291 VIV"}="1H29 VIV"; $resn_exch{"H292 VIV"}="2H29 VIV";
$resn_exch{"H293 VIV"}="3H29 VIV"; $resn_exch{"H2K1 VK3"}="1H2K VK3"; $resn_exch{"H6K1 VK3"}="1H6K VK3";
$resn_exch{"H7K1 VK3"}="1H7K VK3"; $resn_exch{"H8K1 VK3"}="1H8K VK3"; $resn_exch{"H9K1 VK3"}="1H9K VK3";
$resn_exch{"H111 VK3"}="1H11 VK3"; $resn_exch{"H112 VK3"}="2H11 VK3"; $resn_exch{"H113 VK3"}="3H11 VK3";
$resn_exch{"H711 VLB"}="1H71 VLB"; $resn_exch{"H712 VLB"}="2H71 VLB"; $resn_exch{"H713 VLB"}="3H71 VLB";
$resn_exch{"H701 VLB"}="1H70 VLB"; $resn_exch{"H702 VLB"}="2H70 VLB"; $resn_exch{"H551 VLB"}="1H55 VLB";
$resn_exch{"H552 VLB"}="2H55 VLB"; $resn_exch{"H531 VLB"}="1H53 VLB"; $resn_exch{"H532 VLB"}="2H53 VLB";
$resn_exch{"H511 VLB"}="1H51 VLB"; $resn_exch{"H512 VLB"}="2H51 VLB"; $resn_exch{"H691 VLB"}="1H69 VLB";
$resn_exch{"H692 VLB"}="2H69 VLB"; $resn_exch{"H571 VLB"}="1H57 VLB"; $resn_exch{"H572 VLB"}="2H57 VLB";
$resn_exch{"H581 VLB"}="1H58 VLB"; $resn_exch{"H582 VLB"}="2H58 VLB"; $resn_exch{"H761 VLB"}="1H76 VLB";
$resn_exch{"H762 VLB"}="2H76 VLB"; $resn_exch{"H763 VLB"}="3H76 VLB"; $resn_exch{"H111 VLB"}="1H11 VLB";
$resn_exch{"H112 VLB"}="2H11 VLB"; $resn_exch{"H101 VLB"}="1H10 VLB"; $resn_exch{"H102 VLB"}="2H10 VLB";
$resn_exch{" H81 VLB"}="1H8  VLB"; $resn_exch{" H82 VLB"}="2H8  VLB"; $resn_exch{"H201 VLB"}="1H20 VLB";
$resn_exch{"H202 VLB"}="2H20 VLB"; $resn_exch{"H211 VLB"}="1H21 VLB"; $resn_exch{"H212 VLB"}="2H21 VLB";
$resn_exch{"H213 VLB"}="3H21 VLB"; $resn_exch{"H301 VLB"}="1H30 VLB"; $resn_exch{"H302 VLB"}="2H30 VLB";
$resn_exch{"H303 VLB"}="3H30 VLB"; $resn_exch{"H261 VLB"}="1H26 VLB"; $resn_exch{"H262 VLB"}="2H26 VLB";
$resn_exch{"H263 VLB"}="3H26 VLB"; $resn_exch{"H221 VLB"}="1H22 VLB"; $resn_exch{"H222 VLB"}="2H22 VLB";
$resn_exch{"H223 VLB"}="3H22 VLB"; $resn_exch{"H331 VLB"}="1H33 VLB"; $resn_exch{"H332 VLB"}="2H33 VLB";
$resn_exch{"H333 VLB"}="3H33 VLB"; $resn_exch{" HN1 VLL"}="1HN  VLL"; $resn_exch{" HN2 VLL"}="2HN  VLL";
$resn_exch{"HG11 VLL"}="1HG1 VLL"; $resn_exch{"HG12 VLL"}="2HG1 VLL"; $resn_exch{"HG13 VLL"}="3HG1 VLL";
$resn_exch{"HG21 VLL"}="1HG2 VLL"; $resn_exch{"HG22 VLL"}="2HG2 VLL"; $resn_exch{" H   VLM"}="1HN  VLM";
$resn_exch{" H2  VLM"}="2HN  VLM"; $resn_exch{"HG11 VLM"}="1HG1 VLM"; $resn_exch{"HG12 VLM"}="2HG1 VLM";
$resn_exch{"HG13 VLM"}="3HG1 VLM"; $resn_exch{"HG21 VLM"}="1HG2 VLM"; $resn_exch{"HG22 VLM"}="2HG2 VLM";
$resn_exch{"HG23 VLM"}="3HG2 VLM"; $resn_exch{"HNT1 VLM"}="1HNT VLM"; $resn_exch{"HNT2 VLM"}="2HNT VLM";
$resn_exch{" HN1 VME"}="1HN  VME"; $resn_exch{" HN2 VME"}="2HN  VME"; $resn_exch{"HG11 VME"}="1HG1 VME";
$resn_exch{"HG12 VME"}="2HG1 VME"; $resn_exch{"HG13 VME"}="3HG1 VME"; $resn_exch{"HG21 VME"}="1HG2 VME";
$resn_exch{"HG22 VME"}="2HG2 VME"; $resn_exch{"HG23 VME"}="3HG2 VME"; $resn_exch{"HMC1 VME"}="1HMC VME";
$resn_exch{"HMC2 VME"}="2HMC VME"; $resn_exch{"HMC3 VME"}="3HMC VME"; $resn_exch{"HG21 VMS"}="1HG2 VMS";
$resn_exch{"HG22 VMS"}="2HG2 VMS"; $resn_exch{"HG23 VMS"}="3HG2 VMS"; $resn_exch{"HG11 VMS"}="1HG1 VMS";
$resn_exch{"HG12 VMS"}="2HG1 VMS"; $resn_exch{"HG13 VMS"}="3HG1 VMS"; $resn_exch{" H   VMS"}="1HN  VMS";
$resn_exch{" H2  VMS"}="2HN  VMS"; $resn_exch{"H5'1 VMS"}="1H5* VMS"; $resn_exch{"H5'2 VMS"}="2H5* VMS";
$resn_exch{" H4' VMS"}=" H4* VMS"; $resn_exch{" H3' VMS"}=" H3* VMS"; $resn_exch{"HO3' VMS"}="H2   VMS";
$resn_exch{" H2' VMS"}=" H2* VMS"; $resn_exch{" H1' VMS"}=" H1* VMS"; $resn_exch{"HN61 VMS"}="1HN6 VMS";
$resn_exch{"HN62 VMS"}="2HN6 VMS"; $resn_exch{" HV1 VNL"}="1HV  VNL"; $resn_exch{" HV2 VNL"}="2HV  VNL";
$resn_exch{" HV3 VNL"}="3HV  VNL"; $resn_exch{" HN1 VOL"}="1HN  VOL"; $resn_exch{" HN2 VOL"}="2HN  VOL";
$resn_exch{" H51 VPR"}="1H5  VPR"; $resn_exch{" H52 VPR"}="2H5  VPR"; $resn_exch{" H53 VPR"}="3H5  VPR";
$resn_exch{" H41 VPR"}="1H4  VPR"; $resn_exch{" H42 VPR"}="2H4  VPR"; $resn_exch{" H31 VPR"}="1H3  VPR";
$resn_exch{" H32 VPR"}="2H3  VPR"; $resn_exch{"H3A1 VPR"}="1H3A VPR"; $resn_exch{"H3A2 VPR"}="2H3A VPR";
$resn_exch{"H4A1 VPR"}="1H4A VPR"; $resn_exch{"H4A2 VPR"}="2H4A VPR"; $resn_exch{"H5A1 VPR"}="1H5A VPR";
$resn_exch{"H5A2 VPR"}="2H5A VPR"; $resn_exch{"H5A3 VPR"}="3H5A VPR"; $resn_exch{" HN1 VPR"}="1HN  VPR";
$resn_exch{" HN2 VPR"}="2HN  VPR"; $resn_exch{" H61 VR1"}="1H6  VR1"; $resn_exch{" H62 VR1"}="2H6  VR1";
$resn_exch{" H71 VR1"}="1H7  VR1"; $resn_exch{" H72 VR1"}="2H7  VR1"; $resn_exch{" H73 VR1"}="3H7  VR1";
$resn_exch{" H81 VR1"}="1H8  VR1"; $resn_exch{"H31A VRS"}="1H3  VRS"; $resn_exch{"H32A VRS"}="2H3  VRS";
$resn_exch{" H41 VRS"}="1H4  VRS"; $resn_exch{"H42A VRS"}="2H4  VRS"; $resn_exch{"H43A VRS"}="3H4  VRS";
$resn_exch{" H71 VRS"}="1H7  VRS"; $resn_exch{" H72 VRS"}="2H7  VRS"; $resn_exch{" H81 VRS"}="1H8  VRS";
$resn_exch{" H82 VRS"}="2H8  VRS"; $resn_exch{" H91 VRS"}="1H9  VRS"; $resn_exch{" H92 VRS"}="2H9  VRS";
$resn_exch{"H121 VRS"}="1H12 VRS"; $resn_exch{"H122 VRS"}="2H12 VRS"; $resn_exch{"H123 VRS"}="3H12 VRS";
$resn_exch{"H131 VRS"}="1H13 VRS"; $resn_exch{"H132 VRS"}="2H13 VRS"; $resn_exch{"H241 VRS"}="1H24 VRS";
$resn_exch{"H242 VRS"}="2H24 VRS"; $resn_exch{"H251 VRS"}="1H25 VRS"; $resn_exch{"H252 VRS"}="2H25 VRS";
$resn_exch{"H271 VRS"}="1H27 VRS"; $resn_exch{"H272 VRS"}="2H27 VRS"; $resn_exch{"H391 VRS"}="1H39 VRS";
$resn_exch{"H392 VRS"}="2H39 VRS"; $resn_exch{"H393 VRS"}="3H39 VRS"; $resn_exch{"H261 VRX"}="1H26 VRX";
$resn_exch{"H262 VRX"}="2H26 VRX"; $resn_exch{"H263 VRX"}="3H26 VRX"; $resn_exch{"H251 VRX"}="1H25 VRX";
$resn_exch{"H252 VRX"}="2H25 VRX"; $resn_exch{" H31 VS1"}="1H3  VS1"; $resn_exch{"H121 VS1"}="1H12 VS1";
$resn_exch{"H122 VS1"}="2H12 VS1"; $resn_exch{"H151 VS1"}="1H15 VS1"; $resn_exch{"H161 VS1"}="1H16 VS1";
$resn_exch{"H171 VS1"}="1H17 VS1"; $resn_exch{"H181 VS1"}="1H18 VS1"; $resn_exch{"H191 VS1"}="1H19 VS1";
$resn_exch{"H211 VS1"}="1H21 VS1"; $resn_exch{"H231 VS1"}="1H23 VS1"; $resn_exch{"H232 VS1"}="2H23 VS1";
$resn_exch{"H241 VS1"}="1H24 VS1"; $resn_exch{"H242 VS1"}="2H24 VS1"; $resn_exch{"H301 VS1"}="1H30 VS1";
$resn_exch{"H311 VS1"}="1H31 VS1"; $resn_exch{"H321 VS1"}="1H32 VS1"; $resn_exch{"H331 VS1"}="1H33 VS1";
$resn_exch{"H341 VS1"}="1H34 VS1"; $resn_exch{"H251 VS1"}="1H25 VS1"; $resn_exch{"H252 VS1"}="2H25 VS1";
$resn_exch{"H361 VS1"}="1H36 VS1"; $resn_exch{"H372 VS1"}="2H37 VS1"; $resn_exch{"H381 VS1"}="1H38 VS1";
$resn_exch{"H391 VS1"}="1H39 VS1"; $resn_exch{"H401 VS1"}="1H40 VS1"; $resn_exch{" H51 VS1"}="1H5  VS1";
$resn_exch{" H52 VS1"}="2H5  VS1"; $resn_exch{" H61 VS1"}="1H6  VS1"; $resn_exch{" H62 VS1"}="2H6  VS1";
$resn_exch{" H81 VS1"}="1H8  VS1"; $resn_exch{" H82 VS1"}="2H8  VS1"; $resn_exch{" H91 VS1"}="1H9  VS1";
$resn_exch{" H92 VS1"}="2H9  VS1"; $resn_exch{"H221 VS1"}="1H22 VS1"; $resn_exch{"H222 VS1"}="2H22 VS1";
$resn_exch{" H21 VS2"}="1H2  VS2"; $resn_exch{" H31 VS2"}="1H3  VS2"; $resn_exch{" H41 VS2"}="1H4  VS2";
$resn_exch{" H51 VS2"}="1H5  VS2"; $resn_exch{" H61 VS2"}="1H6  VS2"; $resn_exch{" H71 VS2"}="1H7  VS2";
$resn_exch{" H72 VS2"}="2H7  VS2"; $resn_exch{"HN11 VS2"}="1HN1 VS2"; $resn_exch{" H91 VS2"}="1H9  VS2";
$resn_exch{"H101 VS2"}="1H10 VS2"; $resn_exch{"H102 VS2"}="2H10 VS2"; $resn_exch{"H121 VS2"}="1H12 VS2";
$resn_exch{"H131 VS2"}="1H13 VS2"; $resn_exch{"H141 VS2"}="1H14 VS2"; $resn_exch{"H151 VS2"}="1H15 VS2";
$resn_exch{"H161 VS2"}="1H16 VS2"; $resn_exch{"HN21 VS2"}="1HN2 VS2"; $resn_exch{"H181 VS2"}="1H18 VS2";
$resn_exch{"H191 VS2"}="1H19 VS2"; $resn_exch{"H192 VS2"}="2H19 VS2"; $resn_exch{"H201 VS2"}="1H20 VS2";
$resn_exch{"H202 VS2"}="2H20 VS2"; $resn_exch{"H221 VS2"}="1H22 VS2"; $resn_exch{"H231 VS2"}="1H23 VS2";
$resn_exch{"H241 VS2"}="1H24 VS2"; $resn_exch{"H251 VS2"}="1H25 VS2"; $resn_exch{"H261 VS2"}="1H26 VS2";
$resn_exch{"H281 VS2"}="1H28 VS2"; $resn_exch{"H282 VS2"}="2H28 VS2"; $resn_exch{"H291 VS2"}="1H29 VS2";
$resn_exch{"H292 VS2"}="2H29 VS2"; $resn_exch{"H311 VS2"}="1H31 VS2"; $resn_exch{"H321 VS2"}="1H32 VS2";
$resn_exch{"H331 VS2"}="1H33 VS2"; $resn_exch{"H341 VS2"}="1H34 VS2"; $resn_exch{"H351 VS2"}="1H35 VS2";
$resn_exch{"H271 VS2"}="1H27 VS2"; $resn_exch{"H272 VS2"}="2H27 VS2"; $resn_exch{" H21 VS3"}="1H2  VS3";
$resn_exch{" H31 VS3"}="1H3  VS3"; $resn_exch{" H41 VS3"}="1H4  VS3"; $resn_exch{" H51 VS3"}="1H5  VS3";
$resn_exch{" H61 VS3"}="1H6  VS3"; $resn_exch{" H71 VS3"}="1H7  VS3"; $resn_exch{" H72 VS3"}="2H7  VS3";
$resn_exch{"HN11 VS3"}="1HN1 VS3"; $resn_exch{" H91 VS3"}="1H9  VS3"; $resn_exch{"H101 VS3"}="1H10 VS3";
$resn_exch{"H102 VS3"}="2H10 VS3"; $resn_exch{"H121 VS3"}="1H12 VS3"; $resn_exch{"H131 VS3"}="1H13 VS3";
$resn_exch{"H141 VS3"}="1H14 VS3"; $resn_exch{"H151 VS3"}="1H15 VS3"; $resn_exch{"H161 VS3"}="1H16 VS3";
$resn_exch{"HN21 VS3"}="1HN2 VS3"; $resn_exch{"H181 VS3"}="1H18 VS3"; $resn_exch{"H191 VS3"}="1H19 VS3";
$resn_exch{"H192 VS3"}="2H19 VS3"; $resn_exch{"H201 VS3"}="1H20 VS3"; $resn_exch{"H202 VS3"}="2H20 VS3";
$resn_exch{"H221 VS3"}="1H22 VS3"; $resn_exch{"H231 VS3"}="1H23 VS3"; $resn_exch{"H241 VS3"}="1H24 VS3";
$resn_exch{"H251 VS3"}="1H25 VS3"; $resn_exch{"H261 VS3"}="1H26 VS3"; $resn_exch{"H281 VS3"}="1H28 VS3";
$resn_exch{"H282 VS3"}="2H28 VS3"; $resn_exch{"H311 VS3"}="1H31 VS3"; $resn_exch{"H321 VS3"}="1H32 VS3";
$resn_exch{"H341 VS3"}="1H34 VS3"; $resn_exch{"H351 VS3"}="1H35 VS3"; $resn_exch{"H271 VS3"}="1H27 VS3";
$resn_exch{"H272 VS3"}="2H27 VS3"; $resn_exch{"H391 VS4"}="1H39 VS4"; $resn_exch{"H392 VS4"}="2H39 VS4";
$resn_exch{"H401 VS4"}="1H40 VS4"; $resn_exch{"H402 VS4"}="2H40 VS4"; $resn_exch{"H331 VS4"}="1H33 VS4";
$resn_exch{"H332 VS4"}="2H33 VS4"; $resn_exch{"H333 VS4"}="3H33 VS4"; $resn_exch{"H491 VS4"}="1H49 VS4";
$resn_exch{"H492 VS4"}="2H49 VS4"; $resn_exch{"H481 VS4"}="1H48 VS4"; $resn_exch{"H482 VS4"}="2H48 VS4";
$resn_exch{"HN11 VS4"}="1HN1 VS4"; $resn_exch{" H91 VS4"}="1H9  VS4"; $resn_exch{"H101 VS4"}="1H10 VS4";
$resn_exch{"H102 VS4"}="2H10 VS4"; $resn_exch{"H121 VS4"}="1H12 VS4"; $resn_exch{"H131 VS4"}="1H13 VS4";
$resn_exch{"H141 VS4"}="1H14 VS4"; $resn_exch{"H151 VS4"}="1H15 VS4"; $resn_exch{"H161 VS4"}="1H16 VS4";
$resn_exch{"HN21 VS4"}="1HN2 VS4"; $resn_exch{"H181 VS4"}="1H18 VS4"; $resn_exch{"H191 VS4"}="1H19 VS4";
$resn_exch{"H192 VS4"}="2H19 VS4"; $resn_exch{"H201 VS4"}="1H20 VS4"; $resn_exch{"H202 VS4"}="2H20 VS4";
$resn_exch{"H221 VS4"}="1H22 VS4"; $resn_exch{"H231 VS4"}="1H23 VS4"; $resn_exch{"H241 VS4"}="1H24 VS4";
$resn_exch{"H251 VS4"}="1H25 VS4"; $resn_exch{"H261 VS4"}="1H26 VS4"; $resn_exch{"H281 VS4"}="1H28 VS4";
$resn_exch{"H282 VS4"}="2H28 VS4"; $resn_exch{"HN51 VS4"}="1HN5 VS4"; $resn_exch{"H341 VS4"}="1H34 VS4";
$resn_exch{"H342 VS4"}="2H34 VS4"; $resn_exch{"H421 VS4"}="1H42 VS4"; $resn_exch{"H431 VS4"}="1H43 VS4";
$resn_exch{"H441 VS4"}="1H44 VS4"; $resn_exch{"H451 VS4"}="1H45 VS4"; $resn_exch{"H461 VS4"}="1H46 VS4";
$resn_exch{"H271 VS4"}="1H27 VS4"; $resn_exch{"H272 VS4"}="2H27 VS4"; $resn_exch{" H11 VSC"}="1H1  VSC";
$resn_exch{" H12 VSC"}="2H1  VSC"; $resn_exch{" H91 VSC"}="1H9  VSC"; $resn_exch{"H321 VSC"}="1H32 VSC";
$resn_exch{"H322 VSC"}="2H32 VSC"; $resn_exch{"H331 VSC"}="1H33 VSC"; $resn_exch{"H332 VSC"}="2H33 VSC";
$resn_exch{"H131 VSC"}="1H13 VSC"; $resn_exch{"H251 VSC"}="1H25 VSC"; $resn_exch{"H252 VSC"}="2H25 VSC";
$resn_exch{"H271 VSC"}="1H27 VSC"; $resn_exch{"H281 VSC"}="1H28 VSC"; $resn_exch{"H291 VSC"}="1H29 VSC";
$resn_exch{"H301 VSC"}="1H30 VSC"; $resn_exch{"H311 VSC"}="1H31 VSC"; $resn_exch{"H141 VSC"}="1H14 VSC";
$resn_exch{"H142 VSC"}="2H14 VSC"; $resn_exch{"H161 VSC"}="1H16 VSC"; $resn_exch{"H171 VSC"}="1H17 VSC";
$resn_exch{"H181 VSC"}="1H18 VSC"; $resn_exch{"H191 VSC"}="1H19 VSC"; $resn_exch{"H201 VSC"}="1H20 VSC";
$resn_exch{"H21A VSC"}="1H2  VSC"; $resn_exch{" H22 VSC"}="2H2  VSC"; $resn_exch{" H41 VSC"}="1H4  VSC";
$resn_exch{" H51 VSC"}="1H5  VSC"; $resn_exch{" H61 VSC"}="1H6  VSC"; $resn_exch{" H71 VSC"}="1H7  VSC";
$resn_exch{" H81 VSC"}="1H8  VSC"; $resn_exch{"H371 VSC"}="1H37 VSC"; $resn_exch{"H381 VSC"}="1H38 VSC";
$resn_exch{"H391 VSC"}="1H39 VSC"; $resn_exch{"H401 VSC"}="1H40 VSC"; $resn_exch{"H411 VSC"}="1H41 VSC";
$resn_exch{" H41 VSO"}="1H4  VSO"; $resn_exch{" H42 VSO"}="2H4  VSO"; $resn_exch{"H141 VTQ"}="1H14 VTQ";
$resn_exch{"H142 VTQ"}="2H14 VTQ"; $resn_exch{"H143 VTQ"}="3H14 VTQ"; $resn_exch{"H111 VTQ"}="1H11 VTQ";
$resn_exch{"H112 VTQ"}="2H11 VTQ"; $resn_exch{"H101 VTQ"}="1H10 VTQ"; $resn_exch{"H102 VTQ"}="2H10 VTQ";
$resn_exch{"H8C1 VTQ"}="1H8C VTQ"; $resn_exch{"H8C2 VTQ"}="2H8C VTQ"; $resn_exch{"H8C3 VTQ"}="3H8C VTQ";
$resn_exch{"H121 VTQ"}="1H12 VTQ"; $resn_exch{"H122 VTQ"}="2H12 VTQ"; $resn_exch{"H123 VTQ"}="3H12 VTQ";
$resn_exch{"H131 VTQ"}="1H13 VTQ"; $resn_exch{"H132 VTQ"}="2H13 VTQ"; $resn_exch{"H133 VTQ"}="3H13 VTQ";
$resn_exch{"H151 VTQ"}="1H15 VTQ"; $resn_exch{"H152 VTQ"}="2H15 VTQ"; $resn_exch{"H161 VTQ"}="1H16 VTQ";
$resn_exch{"H162 VTQ"}="2H16 VTQ"; $resn_exch{"H171 VTQ"}="1H17 VTQ"; $resn_exch{"H172 VTQ"}="2H17 VTQ";
$resn_exch{"H191 VTQ"}="1H19 VTQ"; $resn_exch{"H192 VTQ"}="2H19 VTQ"; $resn_exch{"H193 VTQ"}="3H19 VTQ";
$resn_exch{"H201 VTQ"}="1H20 VTQ"; $resn_exch{"H202 VTQ"}="2H20 VTQ"; $resn_exch{"H211 VTQ"}="1H21 VTQ";
$resn_exch{"H212 VTQ"}="2H21 VTQ"; $resn_exch{"H221 VTQ"}="1H22 VTQ"; $resn_exch{"H222 VTQ"}="2H22 VTQ";
$resn_exch{"H1C1 VTQ"}="1H1C VTQ"; $resn_exch{"H1C2 VTQ"}="2H1C VTQ"; $resn_exch{"H1C3 VTQ"}="3H1C VTQ";
$resn_exch{"H241 VTQ"}="1H24 VTQ"; $resn_exch{"H242 VTQ"}="2H24 VTQ"; $resn_exch{"H251 VTQ"}="1H25 VTQ";
$resn_exch{"H252 VTQ"}="2H25 VTQ"; $resn_exch{"H261 VTQ"}="1H26 VTQ"; $resn_exch{"H262 VTQ"}="2H26 VTQ";
$resn_exch{"H281 VTQ"}="1H28 VTQ"; $resn_exch{"H282 VTQ"}="2H28 VTQ"; $resn_exch{"H283 VTQ"}="3H28 VTQ";
$resn_exch{"H291 VTQ"}="1H29 VTQ"; $resn_exch{"H292 VTQ"}="2H29 VTQ"; $resn_exch{"H293 VTQ"}="3H29 VTQ";
$resn_exch{" H2   VX"}="2H    VX"; $resn_exch{" H3   VX"}="3H    VX"; $resn_exch{" H21  VX"}="1H2   VX";
$resn_exch{" H22  VX"}="2H2   VX"; $resn_exch{" H31  VX"}="1H3   VX"; $resn_exch{" H32  VX"}="2H3   VX";
$resn_exch{" H33  VX"}="3H3   VX"; $resn_exch{" H21 VX6"}="1H2  VX6"; $resn_exch{" H22 VX6"}="2H2  VX6";
$resn_exch{" H31 VX6"}="1H3  VX6"; $resn_exch{" H32 VX6"}="2H3  VX6"; $resn_exch{" H51 VX6"}="1H5  VX6";
$resn_exch{" H52 VX6"}="2H5  VX6"; $resn_exch{" H61 VX6"}="1H6  VX6"; $resn_exch{" H62 VX6"}="2H6  VX6";
$resn_exch{" H71 VX6"}="1H7  VX6"; $resn_exch{" H72 VX6"}="2H7  VX6"; $resn_exch{" H73 VX6"}="3H7  VX6";
$resn_exch{"H211 VX6"}="1H21 VX6"; $resn_exch{"H212 VX6"}="2H21 VX6"; $resn_exch{"H213 VX6"}="3H21 VX6";
$resn_exch{"H341 VX6"}="1H34 VX6"; $resn_exch{"H342 VX6"}="2H34 VX6"; $resn_exch{"H351 VX6"}="1H35 VX6";
$resn_exch{"H352 VX6"}="2H35 VX6"; $resn_exch{" H11 VXA"}="1H1  VXA"; $resn_exch{" H12 VXA"}="2H1  VXA";
$resn_exch{" H13 VXA"}="3H1  VXA"; $resn_exch{" H61 VXR"}="1H6  VXR"; $resn_exch{" H62 VXR"}="2H6  VXR";
$resn_exch{" H71 VXR"}="1H7  VXR"; $resn_exch{" H72 VXR"}="2H7  VXR"; $resn_exch{" H73 VXR"}="3H7  VXR";
$resn_exch{"HVC1 VXX"}="1HVC VXX"; $resn_exch{"HVC2 VXX"}="2HVC VXX"; $resn_exch{"HVC3 VXX"}="3HVC VXX";
$resn_exch{"H9C1 VXX"}="1H9C VXX"; $resn_exch{"H9C2 VXX"}="2H9C VXX"; $resn_exch{"H9C3 VXX"}="3H9C VXX";
$resn_exch{"H111 VZZ"}="1H11 VZZ"; $resn_exch{"H112 VZZ"}="2H11 VZZ"; $resn_exch{"H101 VZZ"}="1H10 VZZ";
$resn_exch{"H102 VZZ"}="2H10 VZZ"; $resn_exch{" H91 VZZ"}="1H9  VZZ"; $resn_exch{" H92 VZZ"}="2H9  VZZ";
$resn_exch{" H81 VZZ"}="1H8  VZZ"; $resn_exch{" H82 VZZ"}="2H8  VZZ"; $resn_exch{" H71 VZZ"}="1H7  VZZ";
$resn_exch{" H72 VZZ"}="2H7  VZZ"; $resn_exch{" H61 VZZ"}="1H6  VZZ"; $resn_exch{" H62 VZZ"}="2H6  VZZ";
$resn_exch{" H51 VZZ"}="1H5  VZZ"; $resn_exch{" H52 VZZ"}="2H5  VZZ"; $resn_exch{" H41 VZZ"}="1H4  VZZ";
$resn_exch{" H42 VZZ"}="2H4  VZZ"; $resn_exch{" H31 VZZ"}="1H3  VZZ"; $resn_exch{" H32 VZZ"}="2H3  VZZ";
$resn_exch{" H21 VZZ"}="1H2  VZZ"; $resn_exch{" H22 VZZ"}="2H2  VZZ"; $resn_exch{"HM31 W01"}="1HM3 W01";
$resn_exch{"HM32 W01"}="2HM3 W01"; $resn_exch{"HM33 W01"}="3HM3 W01"; $resn_exch{"H1C1 W01"}="1H1C W01";
$resn_exch{"H1C2 W01"}="2H1C W01"; $resn_exch{"H2C1 W01"}="1H2C W01"; $resn_exch{"H2C2 W01"}="2H2C W01";
$resn_exch{"H3C1 W01"}="1H3C W01"; $resn_exch{"H3C2 W01"}="2H3C W01"; $resn_exch{"HM21 W01"}="1HM2 W01";
$resn_exch{"HM22 W01"}="2HM2 W01"; $resn_exch{"HM23 W01"}="3HM2 W01"; $resn_exch{"HM61 W01"}="1HM6 W01";
$resn_exch{"HM62 W01"}="2HM6 W01"; $resn_exch{"HM63 W01"}="3HM6 W01"; $resn_exch{"HM41 W01"}="1HM4 W01";
$resn_exch{"HM42 W01"}="2HM4 W01"; $resn_exch{"HM43 W01"}="3HM4 W01"; $resn_exch{"HM33 W02"}="3HM3 W02";
$resn_exch{"HM32 W02"}="2HM3 W02"; $resn_exch{"HM31 W02"}="1HM3 W02"; $resn_exch{"H1C2 W02"}="2H1C W02";
$resn_exch{"H1C1 W02"}="1H1C W02"; $resn_exch{"H2C2 W02"}="2H2C W02"; $resn_exch{"H2C1 W02"}="1H2C W02";
$resn_exch{"H3C2 W02"}="2H3C W02"; $resn_exch{"H3C1 W02"}="1H3C W02"; $resn_exch{"HM23 W02"}="3HM2 W02";
$resn_exch{"HM22 W02"}="2HM2 W02"; $resn_exch{"HM21 W02"}="1HM2 W02"; $resn_exch{"HM63 W02"}="3HM6 W02";
$resn_exch{"HM62 W02"}="2HM6 W02"; $resn_exch{"HM61 W02"}="1HM6 W02"; $resn_exch{"HM43 W02"}="3HM4 W02";
$resn_exch{"HM42 W02"}="2HM4 W02"; $resn_exch{"HM41 W02"}="1HM4 W02"; $resn_exch{"HM33 W03"}="3HM3 W03";
$resn_exch{"HM32 W03"}="2HM3 W03"; $resn_exch{"HM31 W03"}="1HM3 W03"; $resn_exch{"H1C2 W03"}="2H1C W03";
$resn_exch{"H1C1 W03"}="1H1C W03"; $resn_exch{"H2C2 W03"}="2H2C W03"; $resn_exch{"H2C1 W03"}="1H2C W03";
$resn_exch{"H3C2 W03"}="2H3C W03"; $resn_exch{"H3C1 W03"}="1H3C W03"; $resn_exch{"HM23 W03"}="3HM2 W03";
$resn_exch{"HM22 W03"}="2HM2 W03"; $resn_exch{"HM21 W03"}="1HM2 W03"; $resn_exch{"HM63 W03"}="3HM6 W03";
$resn_exch{"HM62 W03"}="2HM6 W03"; $resn_exch{"HM61 W03"}="1HM6 W03"; $resn_exch{"HM43 W03"}="3HM4 W03";
$resn_exch{"HM42 W03"}="2HM4 W03"; $resn_exch{"HM41 W03"}="1HM4 W03"; $resn_exch{"H3C1 W05"}="1H3C W05";
$resn_exch{"H3C2 W05"}="2H3C W05"; $resn_exch{"H4C1 W05"}="1H4C W05"; $resn_exch{"H4C2 W05"}="2H4C W05";
$resn_exch{"H7C1 W05"}="1H7C W05"; $resn_exch{"H7C2 W05"}="2H7C W05"; $resn_exch{"H141 W05"}="1H14 W05";
$resn_exch{"H142 W05"}="2H14 W05"; $resn_exch{"H161 W05"}="1H16 W05"; $resn_exch{"H162 W05"}="2H16 W05";
$resn_exch{"H321 W05"}="1H32 W05"; $resn_exch{"H322 W05"}="2H32 W05"; $resn_exch{"H323 W05"}="3H32 W05";
$resn_exch{"HM31 W11"}="1HM3 W11"; $resn_exch{"HM32 W11"}="2HM3 W11"; $resn_exch{"HM33 W11"}="3HM3 W11";
$resn_exch{" H41 W11"}="1H4  W11"; $resn_exch{"H1C1 W11"}="1H1C W11"; $resn_exch{"H1C2 W11"}="2H1C W11";
$resn_exch{"H2C1 W11"}="1H2C W11"; $resn_exch{"H2C2 W11"}="2H2C W11"; $resn_exch{"H3C1 W11"}="1H3C W11";
$resn_exch{"H3C2 W11"}="2H3C W11"; $resn_exch{"HM21 W11"}="1HM2 W11"; $resn_exch{"HM22 W11"}="2HM2 W11";
$resn_exch{"HM23 W11"}="3HM2 W11"; $resn_exch{"HM61 W11"}="1HM6 W11"; $resn_exch{"HM62 W11"}="2HM6 W11";
$resn_exch{"HM63 W11"}="3HM6 W11"; $resn_exch{"H251 W2X"}="1H25 W2X"; $resn_exch{"H252 W2X"}="2H25 W2X";
$resn_exch{"H7C1 W2X"}="1H7C W2X"; $resn_exch{"H7C2 W2X"}="2H7C W2X"; $resn_exch{"H4C1 W2X"}="1H4C W2X";
$resn_exch{"H4C2 W2X"}="2H4C W2X"; $resn_exch{"H3C1 W2X"}="1H3C W2X"; $resn_exch{"H3C2 W2X"}="2H3C W2X";
$resn_exch{"H141 W2X"}="1H14 W2X"; $resn_exch{"H142 W2X"}="2H14 W2X"; $resn_exch{"H371 W2X"}="1H37 W2X";
$resn_exch{"H372 W2X"}="2H37 W2X"; $resn_exch{"H331 W2X"}="1H33 W2X"; $resn_exch{"H332 W2X"}="2H33 W2X";
$resn_exch{"H333 W2X"}="3H33 W2X"; $resn_exch{"H311 W33"}="1H31 W33"; $resn_exch{"H312 W33"}="2H31 W33";
$resn_exch{"H313 W33"}="3H31 W33"; $resn_exch{"H1C1 W33"}="1H1C W33"; $resn_exch{"H1C2 W33"}="2H1C W33";
$resn_exch{"H2C1 W33"}="1H2C W33"; $resn_exch{"H2C2 W33"}="2H2C W33"; $resn_exch{"H3C1 W33"}="1H3C W33";
$resn_exch{"H3C2 W33"}="2H3C W33"; $resn_exch{"H4C1 W33"}="1H4C W33"; $resn_exch{"H4C2 W33"}="2H4C W33";
$resn_exch{"H5C1 W33"}="1H5C W33"; $resn_exch{"H5C2 W33"}="2H5C W33"; $resn_exch{"H4A1 W33"}="1H4A W33";
$resn_exch{"H4A2 W33"}="2H4A W33"; $resn_exch{"H5A1 W33"}="1H5A W33"; $resn_exch{"H5A2 W33"}="2H5A W33";
$resn_exch{"H311 W35"}="1H31 W35"; $resn_exch{"H312 W35"}="2H31 W35"; $resn_exch{"H313 W35"}="3H31 W35";
$resn_exch{"H1C1 W35"}="1H1C W35"; $resn_exch{"H1C2 W35"}="2H1C W35"; $resn_exch{"H2C1 W35"}="1H2C W35";
$resn_exch{"H2C2 W35"}="2H2C W35"; $resn_exch{"H3C1 W35"}="1H3C W35"; $resn_exch{"H3C2 W35"}="2H3C W35";
$resn_exch{"H4C1 W35"}="1H4C W35"; $resn_exch{"H4C2 W35"}="2H4C W35"; $resn_exch{"H5C1 W35"}="1H5C W35";
$resn_exch{"H5C2 W35"}="2H5C W35"; $resn_exch{"H4A1 W35"}="1H4A W35"; $resn_exch{"H4A2 W35"}="2H4A W35";
$resn_exch{"H5A1 W35"}="1H5A W35"; $resn_exch{"H5A2 W35"}="2H5A W35"; $resn_exch{"H3A1 W37"}="1H3A W37";
$resn_exch{"H3A2 W37"}="2H3A W37"; $resn_exch{"H1A1 W37"}="1H1A W37"; $resn_exch{"H1A2 W37"}="2H1A W37";
$resn_exch{"H1B1 W37"}="1H1B W37"; $resn_exch{"H1B2 W37"}="2H1B W37"; $resn_exch{"H2B1 W37"}="1H2B W37";
$resn_exch{"H2B2 W37"}="2H2B W37"; $resn_exch{"H3B1 W37"}="1H3B W37"; $resn_exch{"H3B2 W37"}="2H3B W37";
$resn_exch{"H4B1 W37"}="1H4B W37"; $resn_exch{"H4B2 W37"}="2H4B W37"; $resn_exch{"H5B1 W37"}="1H5B W37";
$resn_exch{"H5B2 W37"}="2H5B W37"; $resn_exch{"H6B1 W37"}="1H6B W37"; $resn_exch{"H6B2 W37"}="2H6B W37";
$resn_exch{"H1F1 W37"}="1H1F W37"; $resn_exch{"H1F2 W37"}="2H1F W37"; $resn_exch{"H1F3 W37"}="3H1F W37";
$resn_exch{"HOD1 W42"}="1HOD W42"; $resn_exch{"H2D1 W42"}="1H2D W42"; $resn_exch{"H2D2 W42"}="2H2D W42";
$resn_exch{"H3D1 W42"}="1H3D W42"; $resn_exch{"H3D2 W42"}="2H3D W42"; $resn_exch{"H5D1 W42"}="1H5D W42";
$resn_exch{"H5D2 W42"}="2H5D W42"; $resn_exch{"H311 W42"}="1H31 W42"; $resn_exch{"H312 W42"}="2H31 W42";
$resn_exch{"H1C1 W42"}="1H1C W42"; $resn_exch{"H1C2 W42"}="2H1C W42"; $resn_exch{"H2C1 W42"}="1H2C W42";
$resn_exch{"H2C2 W42"}="2H2C W42"; $resn_exch{"H3C1 W42"}="1H3C W42"; $resn_exch{"H3C2 W42"}="2H3C W42";
$resn_exch{"H4C1 W42"}="1H4C W42"; $resn_exch{"H4C2 W42"}="2H4C W42"; $resn_exch{"H5C1 W42"}="1H5C W42";
$resn_exch{"H5C2 W42"}="2H5C W42"; $resn_exch{"H4A1 W42"}="1H4A W42"; $resn_exch{"H4A2 W42"}="2H4A W42";
$resn_exch{"H5A1 W42"}="1H5A W42"; $resn_exch{"H5A2 W42"}="2H5A W42"; $resn_exch{"H311 W43"}="1H31 W43";
$resn_exch{"H312 W43"}="2H31 W43"; $resn_exch{"H313 W43"}="3H31 W43"; $resn_exch{"H1C1 W43"}="1H1C W43";
$resn_exch{"H1C2 W43"}="2H1C W43"; $resn_exch{"H2C1 W43"}="1H2C W43"; $resn_exch{"H2C2 W43"}="2H2C W43";
$resn_exch{"H3C1 W43"}="1H3C W43"; $resn_exch{"H3C2 W43"}="2H3C W43"; $resn_exch{"H4C1 W43"}="1H4C W43";
$resn_exch{"H4C2 W43"}="2H4C W43"; $resn_exch{"H5C1 W43"}="1H5C W43"; $resn_exch{"H5C2 W43"}="2H5C W43";
$resn_exch{"H6C1 W43"}="1H6C W43"; $resn_exch{"H6C2 W43"}="2H6C W43"; $resn_exch{"H7C1 W43"}="1H7C W43";
$resn_exch{"H7C2 W43"}="2H7C W43"; $resn_exch{"HM11 W43"}="1HM1 W43"; $resn_exch{"HM12 W43"}="2HM1 W43";
$resn_exch{"HM13 W43"}="3HM1 W43"; $resn_exch{"H5A1 W43"}="1H5A W43"; $resn_exch{"H5A2 W43"}="2H5A W43";
$resn_exch{"H311 W54"}="1H31 W54"; $resn_exch{"H312 W54"}="2H31 W54"; $resn_exch{"H313 W54"}="3H31 W54";
$resn_exch{"H1C1 W54"}="1H1C W54"; $resn_exch{"H1C2 W54"}="2H1C W54"; $resn_exch{"H2C1 W54"}="1H2C W54";
$resn_exch{"H2C2 W54"}="2H2C W54"; $resn_exch{"H3C1 W54"}="1H3C W54"; $resn_exch{"H3C2 W54"}="2H3C W54";
$resn_exch{"H4C1 W54"}="1H4C W54"; $resn_exch{"H4C2 W54"}="2H4C W54"; $resn_exch{"H5C1 W54"}="1H5C W54";
$resn_exch{"H5C2 W54"}="2H5C W54"; $resn_exch{"H4A1 W54"}="1H4A W54"; $resn_exch{"H4A2 W54"}="2H4A W54";
$resn_exch{"H5A1 W54"}="1H5A W54"; $resn_exch{"H5A2 W54"}="2H5A W54"; $resn_exch{"H311 W56"}="1H31 W56";
$resn_exch{"H312 W56"}="2H31 W56"; $resn_exch{"H313 W56"}="3H31 W56"; $resn_exch{"H1C1 W56"}="1H1C W56";
$resn_exch{"H1C2 W56"}="2H1C W56"; $resn_exch{"H2C1 W56"}="1H2C W56"; $resn_exch{"H2C2 W56"}="2H2C W56";
$resn_exch{"H3C1 W56"}="1H3C W56"; $resn_exch{"H3C2 W56"}="2H3C W56"; $resn_exch{"H4C1 W56"}="1H4C W56";
$resn_exch{"H4C2 W56"}="2H4C W56"; $resn_exch{"H5C1 W56"}="1H5C W56"; $resn_exch{"H5C2 W56"}="2H5C W56";
$resn_exch{"HM11 W56"}="1HM1 W56"; $resn_exch{"HM12 W56"}="2HM1 W56"; $resn_exch{"HM13 W56"}="3HM1 W56";
$resn_exch{"H5A1 W56"}="1H5A W56"; $resn_exch{"H5A2 W56"}="2H5A W56"; $resn_exch{"H311 W59"}="1H31 W59";
$resn_exch{"H312 W59"}="2H31 W59"; $resn_exch{"H313 W59"}="3H31 W59"; $resn_exch{"H1C1 W59"}="1H1C W59";
$resn_exch{"H1C2 W59"}="2H1C W59"; $resn_exch{"H2C1 W59"}="1H2C W59"; $resn_exch{"H2C2 W59"}="2H2C W59";
$resn_exch{"H3C1 W59"}="1H3C W59"; $resn_exch{"H3C2 W59"}="2H3C W59"; $resn_exch{"H4C1 W59"}="1H4C W59";
$resn_exch{"H4C2 W59"}="2H4C W59"; $resn_exch{"H5C1 W59"}="1H5C W59"; $resn_exch{"H5C2 W59"}="2H5C W59";
$resn_exch{"H6C1 W59"}="1H6C W59"; $resn_exch{"H6C2 W59"}="2H6C W59"; $resn_exch{"H7C1 W59"}="1H7C W59";
$resn_exch{"H7C2 W59"}="2H7C W59"; $resn_exch{"HM11 W59"}="1HM1 W59"; $resn_exch{"HM12 W59"}="2HM1 W59";
$resn_exch{"HM21 W59"}="1HM2 W59"; $resn_exch{"HM22 W59"}="2HM2 W59"; $resn_exch{"HM23 W59"}="3HM2 W59";
$resn_exch{"H5A1 W59"}="1H5A W59"; $resn_exch{"H5A2 W59"}="2H5A W59"; $resn_exch{"H311 W71"}="1H31 W71";
$resn_exch{"H312 W71"}="2H31 W71"; $resn_exch{"H313 W71"}="3H31 W71"; $resn_exch{"H1C1 W71"}="1H1C W71";
$resn_exch{"H1C2 W71"}="2H1C W71"; $resn_exch{"H2C1 W71"}="1H2C W71"; $resn_exch{"H2C2 W71"}="2H2C W71";
$resn_exch{"H3C1 W71"}="1H3C W71"; $resn_exch{"H3C2 W71"}="2H3C W71"; $resn_exch{"H4C1 W71"}="1H4C W71";
$resn_exch{"H4C2 W71"}="2H4C W71"; $resn_exch{"H5C1 W71"}="1H5C W71"; $resn_exch{"H5C2 W71"}="2H5C W71";
$resn_exch{"H6C1 W71"}="1H6C W71"; $resn_exch{"H6C2 W71"}="2H6C W71"; $resn_exch{"H7C1 W71"}="1H7C W71";
$resn_exch{"H7C2 W71"}="2H7C W71"; $resn_exch{"H4A1 W71"}="1H4A W71"; $resn_exch{"H4A2 W71"}="2H4A W71";
$resn_exch{"H5A1 W71"}="1H5A W71"; $resn_exch{"H5A2 W71"}="2H5A W71"; $resn_exch{" H31 W72"}="1H3  W72";
$resn_exch{" H32 W72"}="2H3  W72"; $resn_exch{" H61 W72"}="1H6  W72"; $resn_exch{" H62 W72"}="2H6  W72";
$resn_exch{" H71 W72"}="1H7  W72"; $resn_exch{" H72 W72"}="2H7  W72"; $resn_exch{"H311 W84"}="1H31 W84";
$resn_exch{"H312 W84"}="2H31 W84"; $resn_exch{"H313 W84"}="3H31 W84"; $resn_exch{"H1C1 W84"}="1H1C W84";
$resn_exch{"H1C2 W84"}="2H1C W84"; $resn_exch{"H2C1 W84"}="1H2C W84"; $resn_exch{"H2C2 W84"}="2H2C W84";
$resn_exch{"H3C1 W84"}="1H3C W84"; $resn_exch{"H3C2 W84"}="2H3C W84"; $resn_exch{"H4C1 W84"}="1H4C W84";
$resn_exch{"H4C2 W84"}="2H4C W84"; $resn_exch{"H5C1 W84"}="1H5C W84"; $resn_exch{"H5C2 W84"}="2H5C W84";
$resn_exch{"H6C1 W84"}="1H6C W84"; $resn_exch{"H6C2 W84"}="2H6C W84"; $resn_exch{"H7C1 W84"}="1H7C W84";
$resn_exch{"H7C2 W84"}="2H7C W84"; $resn_exch{"HM11 W84"}="1HM1 W84"; $resn_exch{"HM12 W84"}="2HM1 W84";
$resn_exch{"HM13 W84"}="3HM1 W84"; $resn_exch{"H5A1 W84"}="1H5A W84"; $resn_exch{"H5A2 W84"}="2H5A W84";
$resn_exch{"H311 W8R"}="1H31 W8R"; $resn_exch{"H312 W8R"}="2H31 W8R"; $resn_exch{"H313 W8R"}="3H31 W8R";
$resn_exch{"H1C1 W8R"}="1H1C W8R"; $resn_exch{"H1C2 W8R"}="2H1C W8R"; $resn_exch{"H2C1 W8R"}="1H2C W8R";
$resn_exch{"H2C2 W8R"}="2H2C W8R"; $resn_exch{"H3C1 W8R"}="1H3C W8R"; $resn_exch{"H3C2 W8R"}="2H3C W8R";
$resn_exch{"H4C1 W8R"}="1H4C W8R"; $resn_exch{"H4C2 W8R"}="2H4C W8R"; $resn_exch{"H5C1 W8R"}="1H5C W8R";
$resn_exch{"H5C2 W8R"}="2H5C W8R"; $resn_exch{"H6C1 W8R"}="1H6C W8R"; $resn_exch{"H6C2 W8R"}="2H6C W8R";
$resn_exch{"H7C1 W8R"}="1H7C W8R"; $resn_exch{"H7C2 W8R"}="2H7C W8R"; $resn_exch{"HM11 W8R"}="1HM1 W8R";
$resn_exch{"HM12 W8R"}="2HM1 W8R"; $resn_exch{"HM13 W8R"}="3HM1 W8R"; $resn_exch{"H5A1 W8R"}="1H5A W8R";
$resn_exch{"H5A2 W8R"}="2H5A W8R"; $resn_exch{"H311 W91"}="1H31 W91"; $resn_exch{"H312 W91"}="2H31 W91";
$resn_exch{"H313 W91"}="3H31 W91"; $resn_exch{"H1C1 W91"}="1H1C W91"; $resn_exch{"H1C2 W91"}="2H1C W91";
$resn_exch{"H2C1 W91"}="1H2C W91"; $resn_exch{"H2C2 W91"}="2H2C W91"; $resn_exch{"H3C1 W91"}="1H3C W91";
$resn_exch{"H3C2 W91"}="2H3C W91"; $resn_exch{"H4A1 W91"}="1H4A W91"; $resn_exch{"H4A2 W91"}="2H4A W91";
$resn_exch{"H5A1 W91"}="1H5A W91"; $resn_exch{"H5A2 W91"}="2H5A W91"; $resn_exch{"H121 WAC"}="1H12 WAC";
$resn_exch{"H122 WAC"}="2H12 WAC"; $resn_exch{"H161 WAC"}="1H16 WAC"; $resn_exch{"H162 WAC"}="2H16 WAC";
$resn_exch{"H171 WAC"}="1H17 WAC"; $resn_exch{"H172 WAC"}="2H17 WAC"; $resn_exch{"H181 WAC"}="1H18 WAC";
$resn_exch{"H182 WAC"}="2H18 WAC"; $resn_exch{"H191 WAC"}="1H19 WAC"; $resn_exch{"H192 WAC"}="2H19 WAC";
$resn_exch{"H291 WAC"}="1H29 WAC"; $resn_exch{"H292 WAC"}="2H29 WAC"; $resn_exch{"H301 WAC"}="1H30 WAC";
$resn_exch{"H302 WAC"}="2H30 WAC"; $resn_exch{"H331 WAC"}="1H33 WAC"; $resn_exch{"H332 WAC"}="2H33 WAC";
$resn_exch{"H411 WAC"}="1H41 WAC"; $resn_exch{"H412 WAC"}="2H41 WAC"; $resn_exch{"H413 WAC"}="3H41 WAC";
$resn_exch{"H421 WAC"}="1H42 WAC"; $resn_exch{"H422 WAC"}="2H42 WAC"; $resn_exch{"H423 WAC"}="3H42 WAC";
$resn_exch{"H201 WAI"}="1H20 WAI"; $resn_exch{"H202 WAI"}="2H20 WAI"; $resn_exch{"H203 WAI"}="3H20 WAI";
$resn_exch{"H351 WAI"}="1H35 WAI"; $resn_exch{"H352 WAI"}="2H35 WAI"; $resn_exch{"H361 WAI"}="1H36 WAI";
$resn_exch{"H362 WAI"}="2H36 WAI"; $resn_exch{"H381 WAI"}="1H38 WAI"; $resn_exch{"H382 WAI"}="2H38 WAI";
$resn_exch{"H391 WAI"}="1H39 WAI"; $resn_exch{"H392 WAI"}="2H39 WAI"; $resn_exch{"H481 WAI"}="1H48 WAI";
$resn_exch{"H482 WAI"}="2H48 WAI"; $resn_exch{"H491 WAI"}="1H49 WAI"; $resn_exch{"H492 WAI"}="2H49 WAI";
$resn_exch{"H521 WAI"}="1H52 WAI"; $resn_exch{"H522 WAI"}="2H52 WAI"; $resn_exch{"H181 WBT"}="1H18 WBT";
$resn_exch{"H182 WBT"}="2H18 WBT"; $resn_exch{"H191 WBT"}="1H19 WBT"; $resn_exch{"H192 WBT"}="2H19 WBT";
$resn_exch{"H291 WBT"}="1H29 WBT"; $resn_exch{"H292 WBT"}="2H29 WBT"; $resn_exch{"H301 WBT"}="1H30 WBT";
$resn_exch{"H302 WBT"}="2H30 WBT"; $resn_exch{"H321 WBT"}="1H32 WBT"; $resn_exch{"H322 WBT"}="2H32 WBT";
$resn_exch{"H331 WBT"}="1H33 WBT"; $resn_exch{"H332 WBT"}="2H33 WBT"; $resn_exch{"H011 WBU"}="1H01 WBU";
$resn_exch{"H012 WBU"}="2H01 WBU"; $resn_exch{"HMC1 WIA"}="1HMC WIA"; $resn_exch{"HMC2 WIA"}="2HMC WIA";
$resn_exch{"HMC3 WIA"}="3HMC WIA"; $resn_exch{"H161 WR1"}="1H16 WR1"; $resn_exch{"H162 WR1"}="2H16 WR1";
$resn_exch{" H81 WR1"}="1H8  WR1"; $resn_exch{" H82 WR1"}="2H8  WR1"; $resn_exch{" H51 WR1"}="1H5  WR1";
$resn_exch{" H52 WR1"}="2H5  WR1"; $resn_exch{" H53 WR1"}="3H5  WR1"; $resn_exch{"H21A WR1"}="1H2  WR1";
$resn_exch{"H22A WR1"}="2H2  WR1"; $resn_exch{"H11A WR1"}="1H1  WR1"; $resn_exch{"H12A WR1"}="2H1  WR1";
$resn_exch{" H81 WRA"}="1H8  WRA"; $resn_exch{" H82 WRA"}="2H8  WRA"; $resn_exch{" H91 WRA"}="1H9  WRA";
$resn_exch{" H92 WRA"}="2H9  WRA"; $resn_exch{"H101 WRA"}="1H10 WRA"; $resn_exch{"H102 WRA"}="2H10 WRA";
$resn_exch{" H11 WRA"}="1H1  WRA"; $resn_exch{" H12 WRA"}="2H1  WRA"; $resn_exch{" H13 WRA"}="3H1  WRA";
$resn_exch{" H21 WRA"}="1H2  WRA"; $resn_exch{" H22 WRA"}="2H2  WRA"; $resn_exch{" H23 WRA"}="3H2  WRA";
$resn_exch{"HH11 WRA"}="1HH1 WRA"; $resn_exch{"HH12 WRA"}="2HH1 WRA"; $resn_exch{"HH21 WRA"}="1HH2 WRA";
$resn_exch{"HH22 WRA"}="2HH2 WRA"; $resn_exch{" H81 WRB"}="1H8  WRB"; $resn_exch{" H82 WRB"}="2H8  WRB";
$resn_exch{" H91 WRB"}="1H9  WRB"; $resn_exch{" H92 WRB"}="2H9  WRB"; $resn_exch{"H101 WRB"}="1H10 WRB";
$resn_exch{"H102 WRB"}="2H10 WRB"; $resn_exch{"H131 WRB"}="1H13 WRB"; $resn_exch{"H141 WRB"}="1H14 WRB";
$resn_exch{"H151 WRB"}="1H15 WRB"; $resn_exch{"H161 WRB"}="1H16 WRB"; $resn_exch{"HM11 WRB"}="1HM1 WRB";
$resn_exch{"HM12 WRB"}="2HM1 WRB"; $resn_exch{"HM13 WRB"}="3HM1 WRB"; $resn_exch{"HM21 WRB"}="1HM2 WRB";
$resn_exch{"HM22 WRB"}="2HM2 WRB"; $resn_exch{"HM23 WRB"}="3HM2 WRB"; $resn_exch{"HN11 WRB"}="1HN1 WRB";
$resn_exch{"HN12 WRB"}="2HN1 WRB"; $resn_exch{"HN21 WRB"}="1HN2 WRB"; $resn_exch{"HN22 WRB"}="2HN2 WRB";
$resn_exch{" HC2 WRR"}="2HC  WRR"; $resn_exch{"HC31 WRR"}="1HC3 WRR"; $resn_exch{"HC32 WRR"}="2HC3 WRR";
$resn_exch{" HC4 WRR"}="4HC  WRR"; $resn_exch{" HC1 WRR"}="1HC  WRR"; $resn_exch{" HA2 WRR"}="2HA  WRR";
$resn_exch{" HC3 WRR"}="3HC  WRR"; $resn_exch{" HC7 WRR"}="7HC  WRR"; $resn_exch{" HC8 WRR"}="8HC  WRR";
$resn_exch{" HC9 WRR"}="9HC  WRR"; $resn_exch{"HC01 WRR"}="1HC0 WRR"; $resn_exch{"HC21 WRR"}="1HC2 WRR";
$resn_exch{"HC51 WRR"}="1HC5 WRR"; $resn_exch{"HC61 WRR"}="1HC6 WRR"; $resn_exch{"HC71 WRR"}="1HC7 WRR";
$resn_exch{"HC81 WRR"}="1HC8 WRR"; $resn_exch{" HO3 WRR"}="3HO  WRR"; $resn_exch{" HO4 WRR"}="4HO  WRR";
$resn_exch{" HC2 WRS"}="2HC  WRS"; $resn_exch{"HC31 WRS"}="1HC3 WRS"; $resn_exch{"HC32 WRS"}="2HC3 WRS";
$resn_exch{" HC4 WRS"}="4HC  WRS"; $resn_exch{" HC1 WRS"}="1HC  WRS"; $resn_exch{" HA2 WRS"}="2HA  WRS";
$resn_exch{" HC3 WRS"}="3HC  WRS"; $resn_exch{" HC7 WRS"}="7HC  WRS"; $resn_exch{" HC8 WRS"}="8HC  WRS";
$resn_exch{" HC9 WRS"}="9HC  WRS"; $resn_exch{"HC01 WRS"}="1HC0 WRS"; $resn_exch{"HC21 WRS"}="1HC2 WRS";
$resn_exch{"HC51 WRS"}="1HC5 WRS"; $resn_exch{"HC61 WRS"}="1HC6 WRS"; $resn_exch{"HC71 WRS"}="1HC7 WRS";
$resn_exch{"HC81 WRS"}="1HC8 WRS"; $resn_exch{" HO3 WRS"}="3HO  WRS"; $resn_exch{" HO4 WRS"}="4HO  WRS";
$resn_exch{" HB1 WSA"}="1HB  WSA"; $resn_exch{" HB2 WSA"}="2HB  WSA"; $resn_exch{"HH31 WSA"}="1HH3 WSA";
$resn_exch{"HH32 WSA"}="2HH3 WSA"; $resn_exch{"H5'1 WSA"}="1H5' WSA"; $resn_exch{"H5'2 WSA"}="2H5' WSA";
$resn_exch{"HN61 WSA"}="1HN6 WSA"; $resn_exch{"HN62 WSA"}="2HN6 WSA"; $resn_exch{"H151 WST"}="1H15 WST";
$resn_exch{"H152 WST"}="2H15 WST"; $resn_exch{"H161 WST"}="1H16 WST"; $resn_exch{"H162 WST"}="2H16 WST";
$resn_exch{"H171 WST"}="1H17 WST"; $resn_exch{"H172 WST"}="2H17 WST"; $resn_exch{"H111 WW7"}="1H11 WW7";
$resn_exch{"H112 WW7"}="2H11 WW7"; $resn_exch{"H121 WW7"}="1H12 WW7"; $resn_exch{"H122 WW7"}="2H12 WW7";
$resn_exch{"H141 WW7"}="1H14 WW7"; $resn_exch{"H142 WW7"}="2H14 WW7"; $resn_exch{"H131 WW7"}="1H13 WW7";
$resn_exch{"H132 WW7"}="2H13 WW7"; $resn_exch{"H151 WW7"}="1H15 WW7"; $resn_exch{"H152 WW7"}="2H15 WW7";
$resn_exch{"H161 WW7"}="1H16 WW7"; $resn_exch{"H162 WW7"}="2H16 WW7"; $resn_exch{"HN21 WW7"}="1HN2 WW7";
$resn_exch{"HN22 WW7"}="2HN2 WW7"; $resn_exch{"H5B1 WWF"}="1H5B WWF"; $resn_exch{"H5B2 WWF"}="2H5B WWF";
$resn_exch{"H5A1 WWF"}="1H5A WWF"; $resn_exch{"H5A2 WWF"}="2H5A WWF"; $resn_exch{"H4A1 WWF"}="1H4A WWF";
$resn_exch{"H4A2 WWF"}="2H4A WWF"; $resn_exch{"H4A3 WWF"}="3H4A WWF"; $resn_exch{"H9C1 WWF"}="1H9C WWF";
$resn_exch{"H9C2 WWF"}="2H9C WWF"; $resn_exch{"H111 WWF"}="1H11 WWF"; $resn_exch{"H112 WWF"}="2H11 WWF";
$resn_exch{"H113 WWF"}="3H11 WWF"; $resn_exch{"H121 WWF"}="1H12 WWF"; $resn_exch{"H122 WWF"}="2H12 WWF";
$resn_exch{"H123 WWF"}="3H12 WWF"; $resn_exch{"H351 WWF"}="1H35 WWF"; $resn_exch{"H352 WWF"}="2H35 WWF";
$resn_exch{" H6' WWF"}=" H6* WWF"; $resn_exch{"H2A1 WWF"}="1H2A WWF"; $resn_exch{"H2A2 WWF"}="2H2A WWF";
$resn_exch{"H2A3 WWF"}="3H2A WWF"; $resn_exch{"H4'1 WWF"}="1H4* WWF"; $resn_exch{"H4'2 WWF"}="2H4* WWF";
$resn_exch{" H21 WY2"}="1H2  WY2"; $resn_exch{" H22 WY2"}="2H2  WY2"; $resn_exch{"H691 WY2"}="1H69 WY2";
$resn_exch{"H692 WY2"}="2H69 WY2"; $resn_exch{"H701 WY2"}="1H70 WY2"; $resn_exch{"H702 WY2"}="2H70 WY2";
$resn_exch{"H711 WY2"}="1H71 WY2"; $resn_exch{"H712 WY2"}="2H71 WY2"; $resn_exch{"HC21 WY4"}="1HC2 WY4";
$resn_exch{"HC22 WY4"}="2HC2 WY4"; $resn_exch{"H671 WY4"}="1H67 WY4"; $resn_exch{"H672 WY4"}="2H67 WY4";
$resn_exch{"H691 WY4"}="1H69 WY4"; $resn_exch{"H692 WY4"}="2H69 WY4"; $resn_exch{"H701 WY4"}="1H70 WY4";
$resn_exch{"H702 WY4"}="2H70 WY4"; $resn_exch{"HOP2   X"}="2HOP   X"; $resn_exch{"HOP3   X"}="3HOP   X";
$resn_exch{" H5'   X"}="1H5*   X"; $resn_exch{"H5''   X"}="2H5*   X"; $resn_exch{" H4'   X"}=" H4*   X";
$resn_exch{" H3'   X"}=" H3*   X"; $resn_exch{"HO3'   X"}="*HO3   X"; $resn_exch{" H2'   X"}="1H2*   X";
$resn_exch{"H2''   X"}="2H2*   X"; $resn_exch{" H1'   X"}=" H1*   X"; $resn_exch{" H21   X"}="1H2    X";
$resn_exch{" H22   X"}="2H2    X"; $resn_exch{"H2A1   X"}="1H2A   X"; $resn_exch{"H2A2   X"}="2H2A   X";
$resn_exch{" H31   X"}="1H3    X"; $resn_exch{" H32   X"}="2H3    X"; $resn_exch{" HM1   X"}="1HM    X";
$resn_exch{" HM2   X"}="2HM    X"; $resn_exch{" HM3   X"}="3HM    X"; $resn_exch{"H5A1 X04"}="1H5A X04";
$resn_exch{"H5A2 X04"}="2H5A X04"; $resn_exch{"HN21 X04"}="1HN2 X04"; $resn_exch{"HN22 X04"}="2HN2 X04";
$resn_exch{"H2A1 X04"}="1H2A X04"; $resn_exch{"H2A2 X04"}="2H2A X04"; $resn_exch{"H2A3 X04"}="3H2A X04";
$resn_exch{" H51 X1P"}="1H5  X1P"; $resn_exch{" H52 X1P"}="2H5  X1P"; $resn_exch{" H51 X2F"}="1H5  X2F";
$resn_exch{" H52 X2F"}="2H5  X2F"; $resn_exch{"HOP3 X4A"}="3HOP X4A"; $resn_exch{"HOP2 X4A"}="2HOP X4A";
$resn_exch{" H5' X4A"}="1H5* X4A"; $resn_exch{"H5'' X4A"}="2H5* X4A"; $resn_exch{" H1' X4A"}=" H1* X4A";
$resn_exch{" H3' X4A"}=" H3* X4A"; $resn_exch{" H2' X4A"}="1H2* X4A"; $resn_exch{"H2'' X4A"}="2H2* X4A";
$resn_exch{"HO3' X4A"}="H3T  X4A"; $resn_exch{"HC41 X7O"}="1HC4 X7O"; $resn_exch{"HC42 X7O"}="2HC4 X7O";
$resn_exch{"HC61 X7O"}="1HC6 X7O"; $resn_exch{"HC62 X7O"}="2HC6 X7O"; $resn_exch{"HC63 X7O"}="3HC6 X7O";
$resn_exch{"HN21 X7O"}="1HN2 X7O"; $resn_exch{"HN22 X7O"}="2HN2 X7O"; $resn_exch{"HN71 XAA"}="1HN7 XAA";
$resn_exch{"HN72 XAA"}="2HN7 XAA"; $resn_exch{" HC1 XAA"}="1HC  XAA"; $resn_exch{" HC2 XAA"}="2HC  XAA";
$resn_exch{" HC3 XAA"}="3HC  XAA"; $resn_exch{"HOP3 XAD"}="3HOP XAD"; $resn_exch{"HOP2 XAD"}="2HOP XAD";
$resn_exch{"H6'1 XAD"}="1H6' XAD"; $resn_exch{"H6'2 XAD"}="2H6' XAD"; $resn_exch{"H3'1 XAD"}="1H3' XAD";
$resn_exch{"H3'2 XAD"}="2H3' XAD"; $resn_exch{"H2'1 XAD"}="1H2' XAD"; $resn_exch{"H2'2 XAD"}="2H2' XAD";
$resn_exch{"HN61 XAD"}="1HN6 XAD"; $resn_exch{"HN62 XAD"}="2HN6 XAD"; $resn_exch{"H5'1 XAE"}="1H5' XAE";
$resn_exch{"H5'2 XAE"}="2H5' XAE"; $resn_exch{"H2'1 XAE"}="1H2' XAE"; $resn_exch{"H2'2 XAE"}="2H2' XAE";
$resn_exch{"HN61 XAE"}="1HN6 XAE"; $resn_exch{"HN62 XAE"}="2HN6 XAE"; $resn_exch{"HO3' XAE"}="H3T  XAE";
$resn_exch{"HOP2 XAL"}="2HOP XAL"; $resn_exch{" H5' XAL"}="1H5* XAL"; $resn_exch{"H5'' XAL"}="2H5* XAL";
$resn_exch{" H4' XAL"}=" H4* XAL"; $resn_exch{" H6' XAL"}=" H6* XAL"; $resn_exch{" H7' XAL"}=" H7* XAL";
$resn_exch{" H3' XAL"}=" H3* XAL"; $resn_exch{"HO3' XAL"}="H3T  XAL"; $resn_exch{" H2' XAL"}="1H2* XAL";
$resn_exch{"H2'' XAL"}="2H2* XAL"; $resn_exch{" H1' XAL"}=" H1* XAL"; $resn_exch{"HN61 XAL"}="1HN6 XAL";
$resn_exch{"HN62 XAL"}="2HN6 XAL"; $resn_exch{"HOP3 XAL"}="3HOP XAL"; $resn_exch{" HN1 XAO"}="1HN  XAO";
$resn_exch{" HN2 XAO"}="2HN  XAO"; $resn_exch{" H41 XAO"}="1H4  XAO"; $resn_exch{" H42 XAO"}="2H4  XAO";
$resn_exch{" H61 XAO"}="1H6  XAO"; $resn_exch{" H62 XAO"}="2H6  XAO"; $resn_exch{" H63 XAO"}="3H6  XAO";
$resn_exch{" H71 XAO"}="1H7  XAO"; $resn_exch{" H72 XAO"}="2H7  XAO"; $resn_exch{" H73 XAO"}="3H7  XAO";
$resn_exch{"H7'1 XAO"}="1H7' XAO"; $resn_exch{"H7'2 XAO"}="2H7' XAO"; $resn_exch{"H2'1 XAO"}="1H2' XAO";
$resn_exch{"H2'2 XAO"}="2H2' XAO"; $resn_exch{"H6'1 XAO"}="1H6' XAO"; $resn_exch{"H6'2 XAO"}="2H6' XAO";
$resn_exch{"H3'1 XAO"}="1H3' XAO"; $resn_exch{"H3'2 XAO"}="2H3' XAO"; $resn_exch{"H5'1 XAO"}="1H5' XAO";
$resn_exch{"H5'2 XAO"}="2H5' XAO"; $resn_exch{"H4'1 XAO"}="1H4' XAO"; $resn_exch{"H4'2 XAO"}="2H4' XAO";
$resn_exch{"HN11 XAP"}="1HN1 XAP"; $resn_exch{"HN12 XAP"}="2HN1 XAP"; $resn_exch{" H11 XAP"}="1H1  XAP";
$resn_exch{" H12 XAP"}="2H1  XAP"; $resn_exch{" H21 XAP"}="1H2  XAP"; $resn_exch{" H22 XAP"}="2H2  XAP";
$resn_exch{" H31 XAP"}="1H3  XAP"; $resn_exch{" H32 XAP"}="2H3  XAP"; $resn_exch{"HN21 XAP"}="1HN2 XAP";
$resn_exch{"HN22 XAP"}="2HN2 XAP"; $resn_exch{"HOP3 XAR"}="3HOP XAR"; $resn_exch{"HOP2 XAR"}="2HOP XAR";
$resn_exch{"H5'1 XAR"}="1H5* XAR"; $resn_exch{"H5'2 XAR"}="2H5* XAR"; $resn_exch{" H4' XAR"}=" H4* XAR";
$resn_exch{" H3' XAR"}=" H3* XAR"; $resn_exch{"H2'1 XAR"}="1H2* XAR"; $resn_exch{"H2'2 XAR"}="2H2* XAR";
$resn_exch{" H1' XAR"}=" H1* XAR"; $resn_exch{" H7' XAR"}=" H7* XAR"; $resn_exch{" H6' XAR"}=" H6* XAR";
$resn_exch{"HN61 XAR"}="1HN6 XAR"; $resn_exch{"HN62 XAR"}="2HN6 XAR"; $resn_exch{" H21 XAT"}="1H2  XAT";
$resn_exch{" H22 XAT"}="2H2  XAT"; $resn_exch{" H41 XAT"}="1H4  XAT"; $resn_exch{" H42 XAT"}="2H4  XAT";
$resn_exch{"H161 XAT"}="1H16 XAT"; $resn_exch{"H162 XAT"}="2H16 XAT"; $resn_exch{"H163 XAT"}="3H16 XAT";
$resn_exch{"H171 XAT"}="1H17 XAT"; $resn_exch{"H172 XAT"}="2H17 XAT"; $resn_exch{"H173 XAT"}="3H17 XAT";
$resn_exch{"H181 XAT"}="1H18 XAT"; $resn_exch{"H182 XAT"}="2H18 XAT"; $resn_exch{"H183 XAT"}="3H18 XAT";
$resn_exch{"H191 XAT"}="1H19 XAT"; $resn_exch{"H192 XAT"}="2H19 XAT"; $resn_exch{"H193 XAT"}="3H19 XAT";
$resn_exch{"H201 XAT"}="1H20 XAT"; $resn_exch{"H202 XAT"}="2H20 XAT"; $resn_exch{"H203 XAT"}="3H20 XAT";
$resn_exch{"H221 XAT"}="1H22 XAT"; $resn_exch{"H222 XAT"}="2H22 XAT"; $resn_exch{"H241 XAT"}="1H24 XAT";
$resn_exch{"H242 XAT"}="2H24 XAT"; $resn_exch{"H361 XAT"}="1H36 XAT"; $resn_exch{"H362 XAT"}="2H36 XAT";
$resn_exch{"H363 XAT"}="3H36 XAT"; $resn_exch{"H371 XAT"}="1H37 XAT"; $resn_exch{"H372 XAT"}="2H37 XAT";
$resn_exch{"H373 XAT"}="3H37 XAT"; $resn_exch{"H381 XAT"}="1H38 XAT"; $resn_exch{"H382 XAT"}="2H38 XAT";
$resn_exch{"H383 XAT"}="3H38 XAT"; $resn_exch{"H391 XAT"}="1H39 XAT"; $resn_exch{"H392 XAT"}="2H39 XAT";
$resn_exch{"H393 XAT"}="3H39 XAT"; $resn_exch{"H401 XAT"}="1H40 XAT"; $resn_exch{"H402 XAT"}="2H40 XAT";
$resn_exch{"H403 XAT"}="3H40 XAT"; $resn_exch{"HC31 XBB"}="1HC3 XBB"; $resn_exch{"HC32 XBB"}="2HC3 XBB";
$resn_exch{"HC61 XBB"}="1HC6 XBB"; $resn_exch{"HC62 XBB"}="2HC6 XBB"; $resn_exch{"HC41 XBB"}="1HC4 XBB";
$resn_exch{"HC42 XBB"}="2HC4 XBB"; $resn_exch{"HN71 XBB"}="1HN7 XBB"; $resn_exch{"HN72 XBB"}="2HN7 XBB";
$resn_exch{" H11 XBP"}="1H1  XBP"; $resn_exch{" H12 XBP"}="2H1  XBP"; $resn_exch{" H51 XBP"}="1H5  XBP";
$resn_exch{" H52 XBP"}="2H5  XBP"; $resn_exch{"HOP2 XBP"}="2HOP XBP"; $resn_exch{"HOP3 XBP"}="3HOP XBP";
$resn_exch{"HOP5 XBP"}="5HOP XBP"; $resn_exch{"HOP6 XBP"}="6HOP XBP"; $resn_exch{"H5C1 XC2"}="1H5C XC2";
$resn_exch{"H5C2 XC2"}="2H5C XC2"; $resn_exch{"H241 XC2"}="1H24 XC2"; $resn_exch{"H242 XC2"}="2H24 XC2";
$resn_exch{"H261 XC2"}="1H26 XC2"; $resn_exch{"H262 XC2"}="2H26 XC2"; $resn_exch{"H1C1 XC2"}="1H1C XC2";
$resn_exch{"H1C2 XC2"}="2H1C XC2"; $resn_exch{"H4C1 XC2"}="1H4C XC2"; $resn_exch{"H4C2 XC2"}="2H4C XC2";
$resn_exch{"H2C1 XC2"}="1H2C XC2"; $resn_exch{"H2C2 XC2"}="2H2C XC2"; $resn_exch{"H341 XC2"}="1H34 XC2";
$resn_exch{"H342 XC2"}="2H34 XC2"; $resn_exch{"H311 XC2"}="1H31 XC2"; $resn_exch{"H312 XC2"}="2H31 XC2";
$resn_exch{"H301 XC2"}="1H30 XC2"; $resn_exch{"H302 XC2"}="2H30 XC2"; $resn_exch{"HOP2 XCL"}="2HOP XCL";
$resn_exch{" H5' XCL"}="1H5* XCL"; $resn_exch{"H5'' XCL"}="2H5* XCL"; $resn_exch{" H4' XCL"}=" H4* XCL";
$resn_exch{" H6' XCL"}=" H6* XCL"; $resn_exch{" H7' XCL"}=" H7* XCL"; $resn_exch{" H3' XCL"}=" H3* XCL";
$resn_exch{"HO3' XCL"}="H3T  XCL"; $resn_exch{" H2' XCL"}="1H2* XCL"; $resn_exch{"H2'' XCL"}="2H2* XCL";
$resn_exch{" H1' XCL"}=" H1* XCL"; $resn_exch{"HN41 XCL"}="1HN4 XCL"; $resn_exch{"HN42 XCL"}="2HN4 XCL";
$resn_exch{"HOP3 XCL"}="3HOP XCL"; $resn_exch{"HN41 XCS"}="1HN4 XCS"; $resn_exch{"HN42 XCS"}="2HN4 XCS";
$resn_exch{"H5'1 XCS"}="1H5' XCS"; $resn_exch{"H5'2 XCS"}="2H5' XCS"; $resn_exch{"H7A1 XCS"}="1H7A XCS";
$resn_exch{"H7A2 XCS"}="2H7A XCS"; $resn_exch{"H7A3 XCS"}="3H7A XCS"; $resn_exch{"H2'1 XCS"}="1H2' XCS";
$resn_exch{"H2'2 XCS"}="2H2' XCS"; $resn_exch{"HO3' XCS"}="H3T  XCS"; $resn_exch{"HOP3 XCT"}="3HOP XCT";
$resn_exch{"HOP2 XCT"}="2HOP XCT"; $resn_exch{"H6'1 XCT"}="1H6' XCT"; $resn_exch{"H6'2 XCT"}="2H6' XCT";
$resn_exch{"H3'1 XCT"}="1H3' XCT"; $resn_exch{"H3'2 XCT"}="2H3' XCT"; $resn_exch{"H2'1 XCT"}="1H2' XCT";
$resn_exch{"H2'2 XCT"}="2H2' XCT"; $resn_exch{"HN41 XCT"}="1HN4 XCT"; $resn_exch{"HN42 XCT"}="2HN4 XCT";
$resn_exch{"HOP3 XCY"}="3HOP XCY"; $resn_exch{"HOP2 XCY"}="2HOP XCY"; $resn_exch{"HNZ1 XCY"}="1HNZ XCY";
$resn_exch{"HNZ2 XCY"}="2HNZ XCY"; $resn_exch{" HB1 XCY"}="1HB  XCY"; $resn_exch{" HB2 XCY"}="2HB  XCY";
$resn_exch{" HH1 XCY"}="1HH  XCY"; $resn_exch{" HH2 XCY"}="2HH  XCY"; $resn_exch{" H2' XCY"}="1H2* XCY";
$resn_exch{"H2'' XCY"}="2H2* XCY"; $resn_exch{" H5' XCY"}="1H5* XCY"; $resn_exch{"H5'' XCY"}="2H5* XCY";
$resn_exch{" H4' XCY"}=" H4* XCY"; $resn_exch{" H1' XCY"}=" H1* XCY"; $resn_exch{" H3' XCY"}=" H3* XCY";
$resn_exch{"HO3' XCY"}="H3T  XCY"; $resn_exch{"H4C1 XDL"}="1H4C XDL"; $resn_exch{"H4C2 XDL"}="2H4C XDL";
$resn_exch{"H5C1 XDL"}="1H5C XDL"; $resn_exch{"H5C2 XDL"}="2H5C XDL"; $resn_exch{"H6C1 XDL"}="1H6C XDL";
$resn_exch{"H6C2 XDL"}="2H6C XDL"; $resn_exch{" H11 XDN"}="1H1  XDN"; $resn_exch{" H12 XDN"}="2H1  XDN";
$resn_exch{" H51 XDN"}="1H5  XDN"; $resn_exch{" H52 XDN"}="2H5  XDN"; $resn_exch{" H11 XDP"}="1H1  XDP";
$resn_exch{" H12 XDP"}="2H1  XDP"; $resn_exch{" H51 XDP"}="1H5  XDP"; $resn_exch{" H52 XDP"}="2H5  XDP";
$resn_exch{"HO21 XDP"}="1HO2 XDP"; $resn_exch{"HO22 XDP"}="2HO2 XDP"; $resn_exch{"HOP2 XDP"}="2HOP XDP";
$resn_exch{"HOP3 XDP"}="3HOP XDP"; $resn_exch{"HOP5 XDP"}="5HOP XDP"; $resn_exch{"HOP6 XDP"}="6HOP XDP";
$resn_exch{"H11A XE4"}="1H1  XE4"; $resn_exch{" H12 XE4"}="2H1  XE4"; $resn_exch{" H31 XE4"}="1H3  XE4";
$resn_exch{" H32 XE4"}="2H3  XE4"; $resn_exch{"H101 XE4"}="1H10 XE4"; $resn_exch{"H102 XE4"}="2H10 XE4";
$resn_exch{"H131 XE4"}="1H13 XE4"; $resn_exch{"H132 XE4"}="2H13 XE4"; $resn_exch{"H141 XE4"}="1H14 XE4";
$resn_exch{"H142 XE4"}="2H14 XE4"; $resn_exch{"H151 XE4"}="1H15 XE4"; $resn_exch{"H152 XE4"}="2H15 XE4";
$resn_exch{"H161 XE4"}="1H16 XE4"; $resn_exch{"H162 XE4"}="2H16 XE4"; $resn_exch{"H171 XE4"}="1H17 XE4";
$resn_exch{"H172 XE4"}="2H17 XE4"; $resn_exch{"H241 XE4"}="1H24 XE4"; $resn_exch{"H242 XE4"}="2H24 XE4";
$resn_exch{"H243 XE4"}="3H24 XE4"; $resn_exch{"H251 XE4"}="1H25 XE4"; $resn_exch{"H252 XE4"}="2H25 XE4";
$resn_exch{"H253 XE4"}="3H25 XE4"; $resn_exch{"H261 XE4"}="1H26 XE4"; $resn_exch{"H262 XE4"}="2H26 XE4";
$resn_exch{"H263 XE4"}="3H26 XE4"; $resn_exch{"H1C1 XED"}="1H1C XED"; $resn_exch{"H1C2 XED"}="2H1C XED";
$resn_exch{"H2C1 XED"}="1H2C XED"; $resn_exch{"H2C2 XED"}="2H2C XED"; $resn_exch{"H5C1 XED"}="1H5C XED";
$resn_exch{"H5C2 XED"}="2H5C XED"; $resn_exch{"H6C1 XED"}="1H6C XED"; $resn_exch{"H6C2 XED"}="2H6C XED";
$resn_exch{"H7C1 XED"}="1H7C XED"; $resn_exch{"H7C2 XED"}="2H7C XED"; $resn_exch{"H7C3 XED"}="3H7C XED";
$resn_exch{"H191 XED"}="1H19 XED"; $resn_exch{"H192 XED"}="2H19 XED"; $resn_exch{"H201 XED"}="1H20 XED";
$resn_exch{"H202 XED"}="2H20 XED"; $resn_exch{"H203 XED"}="3H20 XED"; $resn_exch{"H5'1 XGA"}="1H5' XGA";
$resn_exch{"H5'2 XGA"}="2H5' XGA"; $resn_exch{"H2'1 XGA"}="1H2' XGA"; $resn_exch{"H2'2 XGA"}="2H2' XGA";
$resn_exch{"HN21 XGA"}="1HN2 XGA"; $resn_exch{"HN22 XGA"}="2HN2 XGA"; $resn_exch{"HO3' XGA"}="H3T  XGA";
$resn_exch{"HOP2 XGL"}="2HOP XGL"; $resn_exch{" H5' XGL"}="1H5* XGL"; $resn_exch{"H5'' XGL"}="2H5* XGL";
$resn_exch{" H4' XGL"}=" H4* XGL"; $resn_exch{" H6' XGL"}=" H6* XGL"; $resn_exch{" H7' XGL"}=" H7* XGL";
$resn_exch{" H3' XGL"}=" H3* XGL"; $resn_exch{"HO3' XGL"}="H3T  XGL"; $resn_exch{" H2' XGL"}="1H2* XGL";
$resn_exch{"H2'' XGL"}="2H2* XGL"; $resn_exch{" H1' XGL"}=" H1* XGL"; $resn_exch{"HN21 XGL"}="1HN2 XGL";
$resn_exch{"HN22 XGL"}="2HN2 XGL"; $resn_exch{"HOP3 XGL"}="3HOP XGL"; $resn_exch{"HOP3 XGU"}="3HOP XGU";
$resn_exch{"HOP2 XGU"}="2HOP XGU"; $resn_exch{"H6'1 XGU"}="1H6' XGU"; $resn_exch{"H6'2 XGU"}="2H6' XGU";
$resn_exch{"H3'1 XGU"}="1H3' XGU"; $resn_exch{"H3'2 XGU"}="2H3' XGU"; $resn_exch{"H2'1 XGU"}="1H2' XGU";
$resn_exch{"H2'2 XGU"}="2H2' XGU"; $resn_exch{"HN21 XGU"}="1HN2 XGU"; $resn_exch{"HN22 XGU"}="2HN2 XGU";
$resn_exch{" H21 XIF"}="1H2  XIF"; $resn_exch{" H22 XIF"}="2H2  XIF"; $resn_exch{" H51 XIF"}="1H5  XIF";
$resn_exch{" H52 XIF"}="2H5  XIF"; $resn_exch{" H61 XIF"}="1H6  XIF"; $resn_exch{" H62 XIF"}="2H6  XIF";
$resn_exch{"H101 XIH"}="1H10 XIH"; $resn_exch{"H102 XIH"}="2H10 XIH"; $resn_exch{"H151 XIH"}="1H15 XIH";
$resn_exch{"H152 XIH"}="2H15 XIH"; $resn_exch{"H171 XIH"}="1H17 XIH"; $resn_exch{"H172 XIH"}="2H17 XIH";
$resn_exch{" H1  XIH"}="1H   XIH"; $resn_exch{" H2  XIH"}="2H   XIH"; $resn_exch{" H3  XIH"}="3H   XIH";
$resn_exch{"H111 XIH"}="1H11 XIH"; $resn_exch{"H112 XIH"}="2H11 XIH"; $resn_exch{"H121 XIH"}="1H12 XIH";
$resn_exch{"H122 XIH"}="2H12 XIH"; $resn_exch{"H141 XIH"}="1H14 XIH"; $resn_exch{"H142 XIH"}="2H14 XIH";
$resn_exch{"H271 XIH"}="1H27 XIH"; $resn_exch{"H272 XIH"}="2H27 XIH"; $resn_exch{"H273 XIH"}="3H27 XIH";
$resn_exch{" H51 XIL"}="1H5  XIL"; $resn_exch{" H52 XIL"}="2H5  XIL"; $resn_exch{"H5'1 XIL"}="1H5' XIL";
$resn_exch{"H5'2 XIL"}="2H5' XIL"; $resn_exch{"H6'1 XIL"}="1H6' XIL"; $resn_exch{"H6'2 XIL"}="2H6' XIL";
$resn_exch{" H51 XIM"}="1H5  XIM"; $resn_exch{" H52 XIM"}="2H5  XIM"; $resn_exch{"H21A XK2"}="1H2  XK2";
$resn_exch{"H22A XK2"}="2H2  XK2"; $resn_exch{"H71A XK2"}="1H7  XK2"; $resn_exch{" H72 XK2"}="2H7  XK2";
$resn_exch{"H311 XK2"}="1H31 XK2"; $resn_exch{"H312 XK2"}="2H31 XK2"; $resn_exch{"H611 XK2"}="1H61 XK2";
$resn_exch{"H612 XK2"}="2H61 XK2"; $resn_exch{" HA1 XL1"}="1HA  XL1"; $resn_exch{" HA2 XL1"}="2HA  XL1";
$resn_exch{" HA3 XL1"}="3HA  XL1"; $resn_exch{" HB1 XL1"}="1HB  XL1"; $resn_exch{" HB2 XL1"}="2HB  XL1";
$resn_exch{"H261 XLC"}="1H26 XLC"; $resn_exch{"H262 XLC"}="2H26 XLC"; $resn_exch{"H291 XLC"}="1H29 XLC";
$resn_exch{"H292 XLC"}="2H29 XLC"; $resn_exch{"H301 XLC"}="1H30 XLC"; $resn_exch{"H302 XLC"}="2H30 XLC";
$resn_exch{"H331 XLC"}="1H33 XLC"; $resn_exch{"H332 XLC"}="2H33 XLC"; $resn_exch{"H341 XLC"}="1H34 XLC";
$resn_exch{"H342 XLC"}="2H34 XLC"; $resn_exch{"H351 XLC"}="1H35 XLC"; $resn_exch{"H352 XLC"}="2H35 XLC";
$resn_exch{"H353 XLC"}="3H35 XLC"; $resn_exch{"H141 XLD"}="1H14 XLD"; $resn_exch{"H161 XLD"}="1H16 XLD";
$resn_exch{"H181 XLD"}="1H18 XLD"; $resn_exch{"H182 XLD"}="2H18 XLD"; $resn_exch{"H183 XLD"}="3H18 XLD";
$resn_exch{"H191 XLD"}="1H19 XLD"; $resn_exch{"H241 XLD"}="1H24 XLD"; $resn_exch{"H281 XLD"}="1H28 XLD";
$resn_exch{"H282 XLD"}="2H28 XLD"; $resn_exch{"H311 XLD"}="1H31 XLD"; $resn_exch{"H312 XLD"}="2H31 XLD";
$resn_exch{"H313 XLD"}="3H31 XLD"; $resn_exch{"H341 XLD"}="1H34 XLD"; $resn_exch{"H342 XLD"}="2H34 XLD";
$resn_exch{"H351 XLD"}="1H35 XLD"; $resn_exch{"H352 XLD"}="2H35 XLD"; $resn_exch{"H5C1 XLM"}="1H5C XLM";
$resn_exch{"H5C2 XLM"}="2H5C XLM"; $resn_exch{"H6'1 XLM"}="1H6' XLM"; $resn_exch{"H6'2 XLM"}="2H6' XLM";
$resn_exch{"H7'1 XLM"}="1H7' XLM"; $resn_exch{"H7'2 XLM"}="2H7' XLM"; $resn_exch{"H7'3 XLM"}="3H7' XLM";
$resn_exch{" H51 XLS"}="1H5  XLS"; $resn_exch{" H52 XLS"}="2H5  XLS"; $resn_exch{"H171 XMA"}="1H17 XMA";
$resn_exch{"H172 XMA"}="2H17 XMA"; $resn_exch{"H201 XMA"}="1H20 XMA"; $resn_exch{"H202 XMA"}="2H20 XMA";
$resn_exch{"H221 XMA"}="1H22 XMA"; $resn_exch{"H222 XMA"}="2H22 XMA"; $resn_exch{"H223 XMA"}="3H22 XMA";
$resn_exch{"H231 XMA"}="1H23 XMA"; $resn_exch{"H232 XMA"}="2H23 XMA"; $resn_exch{"H251 XMA"}="1H25 XMA";
$resn_exch{"H252 XMA"}="2H25 XMA"; $resn_exch{"H301 XMA"}="1H30 XMA"; $resn_exch{"H302 XMA"}="2H30 XMA";
$resn_exch{"H321 XMA"}="1H32 XMA"; $resn_exch{"H322 XMA"}="2H32 XMA"; $resn_exch{"H351 XMA"}="1H35 XMA";
$resn_exch{"H352 XMA"}="2H35 XMA"; $resn_exch{"H381 XMA"}="1H38 XMA"; $resn_exch{"H382 XMA"}="2H38 XMA";
$resn_exch{" H41 XMB"}="1H4  XMB"; $resn_exch{" H42 XMB"}="2H4  XMB"; $resn_exch{" H51 XMB"}="1H5  XMB";
$resn_exch{" H52 XMB"}="2H5  XMB"; $resn_exch{" H91 XMB"}="1H9  XMB"; $resn_exch{" H92 XMB"}="2H9  XMB";
$resn_exch{"H101 XMB"}="1H10 XMB"; $resn_exch{"H102 XMB"}="2H10 XMB"; $resn_exch{"H261 XMB"}="1H26 XMB";
$resn_exch{"H262 XMB"}="2H26 XMB"; $resn_exch{"H291 XMB"}="1H29 XMB"; $resn_exch{"H292 XMB"}="2H29 XMB";
$resn_exch{"H341 XMB"}="1H34 XMB"; $resn_exch{"H342 XMB"}="2H34 XMB"; $resn_exch{"H361 XMB"}="1H36 XMB";
$resn_exch{"H362 XMB"}="2H36 XMB"; $resn_exch{"H381 XMB"}="1H38 XMB"; $resn_exch{"H382 XMB"}="2H38 XMB";
$resn_exch{"H411 XMB"}="1H41 XMB"; $resn_exch{"H412 XMB"}="2H41 XMB"; $resn_exch{"H161 XMC"}="1H16 XMC";
$resn_exch{"H162 XMC"}="2H16 XMC"; $resn_exch{"H221 XMC"}="1H22 XMC"; $resn_exch{"H222 XMC"}="2H22 XMC";
$resn_exch{"H231 XMC"}="1H23 XMC"; $resn_exch{"H232 XMC"}="2H23 XMC"; $resn_exch{"H251 XMC"}="1H25 XMC";
$resn_exch{"H252 XMC"}="2H25 XMC"; $resn_exch{"H261 XMC"}="1H26 XMC"; $resn_exch{"H262 XMC"}="2H26 XMC";
$resn_exch{"H271 XMC"}="1H27 XMC"; $resn_exch{"H272 XMC"}="2H27 XMC"; $resn_exch{"H301 XMC"}="1H30 XMC";
$resn_exch{"H302 XMC"}="2H30 XMC"; $resn_exch{"H351 XMC"}="1H35 XMC"; $resn_exch{"H352 XMC"}="2H35 XMC";
$resn_exch{"H371 XMC"}="1H37 XMC"; $resn_exch{"H372 XMC"}="2H37 XMC"; $resn_exch{"H391 XMC"}="1H39 XMC";
$resn_exch{"H392 XMC"}="2H39 XMC"; $resn_exch{"H421 XMC"}="1H42 XMC"; $resn_exch{"H422 XMC"}="2H42 XMC";
$resn_exch{"H171 XMD"}="1H17 XMD"; $resn_exch{"H172 XMD"}="2H17 XMD"; $resn_exch{"H181 XMD"}="1H18 XMD";
$resn_exch{"H182 XMD"}="2H18 XMD"; $resn_exch{"H183 XMD"}="3H18 XMD"; $resn_exch{"H201 XMD"}="1H20 XMD";
$resn_exch{"H202 XMD"}="2H20 XMD"; $resn_exch{"H241 XMD"}="1H24 XMD"; $resn_exch{"H242 XMD"}="2H24 XMD";
$resn_exch{"H271 XMD"}="1H27 XMD"; $resn_exch{"H272 XMD"}="2H27 XMD"; $resn_exch{"H321 XMD"}="1H32 XMD";
$resn_exch{"H322 XMD"}="2H32 XMD"; $resn_exch{"H341 XMD"}="1H34 XMD"; $resn_exch{"H342 XMD"}="2H34 XMD";
$resn_exch{"H361 XMD"}="1H36 XMD"; $resn_exch{"H362 XMD"}="2H36 XMD"; $resn_exch{"H391 XMD"}="1H39 XMD";
$resn_exch{"H392 XMD"}="2H39 XMD"; $resn_exch{"H161 XME"}="1H16 XME"; $resn_exch{"H162 XME"}="2H16 XME";
$resn_exch{"H181 XME"}="1H18 XME"; $resn_exch{"H182 XME"}="2H18 XME"; $resn_exch{"H191 XME"}="1H19 XME";
$resn_exch{"H192 XME"}="2H19 XME"; $resn_exch{"H221 XME"}="1H22 XME"; $resn_exch{"H222 XME"}="2H22 XME";
$resn_exch{"H271 XME"}="1H27 XME"; $resn_exch{"H272 XME"}="2H27 XME"; $resn_exch{"H291 XME"}="1H29 XME";
$resn_exch{"H292 XME"}="2H29 XME"; $resn_exch{"H341 XME"}="1H34 XME"; $resn_exch{"H342 XME"}="2H34 XME";
$resn_exch{"H371 XME"}="1H37 XME"; $resn_exch{"H372 XME"}="2H37 XME"; $resn_exch{"H141 XMF"}="1H14 XMF";
$resn_exch{"H142 XMF"}="2H14 XMF"; $resn_exch{"H151 XMF"}="1H15 XMF"; $resn_exch{"H152 XMF"}="2H15 XMF";
$resn_exch{"H191 XMF"}="1H19 XMF"; $resn_exch{"H192 XMF"}="2H19 XMF"; $resn_exch{"H221 XMF"}="1H22 XMF";
$resn_exch{"H222 XMF"}="2H22 XMF"; $resn_exch{"H271 XMF"}="1H27 XMF"; $resn_exch{"H272 XMF"}="2H27 XMF";
$resn_exch{"H291 XMF"}="1H29 XMF"; $resn_exch{"H292 XMF"}="2H29 XMF"; $resn_exch{"H311 XMF"}="1H31 XMF";
$resn_exch{"H312 XMF"}="2H31 XMF"; $resn_exch{"H331 XMF"}="1H33 XMF"; $resn_exch{"H332 XMF"}="2H33 XMF";
$resn_exch{"H361 XMF"}="1H36 XMF"; $resn_exch{"H362 XMF"}="2H36 XMF"; $resn_exch{"H11A XMG"}="1H1  XMG";
$resn_exch{" H12 XMG"}="2H1  XMG"; $resn_exch{" H41 XMG"}="1H4  XMG"; $resn_exch{" H42 XMG"}="2H4  XMG";
$resn_exch{" H51 XMG"}="1H5  XMG"; $resn_exch{" H52 XMG"}="2H5  XMG"; $resn_exch{" H81 XMG"}="1H8  XMG";
$resn_exch{" H82 XMG"}="2H8  XMG"; $resn_exch{"H291 XMG"}="1H29 XMG"; $resn_exch{"H292 XMG"}="2H29 XMG";
$resn_exch{"H311 XMG"}="1H31 XMG"; $resn_exch{"H312 XMG"}="2H31 XMG"; $resn_exch{"H331 XMG"}="1H33 XMG";
$resn_exch{"H332 XMG"}="2H33 XMG"; $resn_exch{"H361 XMG"}="1H36 XMG"; $resn_exch{"H362 XMG"}="2H36 XMG";
$resn_exch{" H11 XMH"}="1H1  XMH"; $resn_exch{"H12A XMH"}="2H1  XMH"; $resn_exch{" H61 XMH"}="1H6  XMH";
$resn_exch{" H62 XMH"}="2H6  XMH"; $resn_exch{" H81 XMH"}="1H8  XMH"; $resn_exch{" H82 XMH"}="2H8  XMH";
$resn_exch{"H101 XMH"}="1H10 XMH"; $resn_exch{"H102 XMH"}="2H10 XMH"; $resn_exch{"H131 XMH"}="1H13 XMH";
$resn_exch{"H132 XMH"}="2H13 XMH"; $resn_exch{"H171 XMH"}="1H17 XMH"; $resn_exch{"H172 XMH"}="2H17 XMH";
$resn_exch{"H191 XMH"}="1H19 XMH"; $resn_exch{"H192 XMH"}="2H19 XMH"; $resn_exch{"H201 XMH"}="1H20 XMH";
$resn_exch{"H202 XMH"}="2H20 XMH"; $resn_exch{"H361 XMH"}="1H36 XMH"; $resn_exch{"H362 XMH"}="2H36 XMH";
$resn_exch{"H371 XMH"}="1H37 XMH"; $resn_exch{"H372 XMH"}="2H37 XMH"; $resn_exch{"H373 XMH"}="3H37 XMH";
$resn_exch{" H51 XMI"}="1H5  XMI"; $resn_exch{" H52 XMI"}="2H5  XMI"; $resn_exch{" H71 XMI"}="1H7  XMI";
$resn_exch{" H72 XMI"}="2H7  XMI"; $resn_exch{" H81 XMI"}="1H8  XMI"; $resn_exch{" H82 XMI"}="2H8  XMI";
$resn_exch{"H111 XMI"}="1H11 XMI"; $resn_exch{"H112 XMI"}="2H11 XMI"; $resn_exch{"H161 XMI"}="1H16 XMI";
$resn_exch{"H162 XMI"}="2H16 XMI"; $resn_exch{"H181 XMI"}="1H18 XMI"; $resn_exch{"H182 XMI"}="2H18 XMI";
$resn_exch{"H201 XMI"}="1H20 XMI"; $resn_exch{"H202 XMI"}="2H20 XMI"; $resn_exch{"H231 XMI"}="1H23 XMI";
$resn_exch{"H232 XMI"}="2H23 XMI"; $resn_exch{"H101 XMJ"}="1H10 XMJ"; $resn_exch{"H102 XMJ"}="2H10 XMJ";
$resn_exch{"H131 XMJ"}="1H13 XMJ"; $resn_exch{"H132 XMJ"}="2H13 XMJ"; $resn_exch{"H141 XMJ"}="1H14 XMJ";
$resn_exch{"H142 XMJ"}="2H14 XMJ"; $resn_exch{"H201 XMJ"}="1H20 XMJ"; $resn_exch{"H202 XMJ"}="2H20 XMJ";
$resn_exch{"H251 XMJ"}="1H25 XMJ"; $resn_exch{"H252 XMJ"}="2H25 XMJ"; $resn_exch{"H271 XMJ"}="1H27 XMJ";
$resn_exch{"H272 XMJ"}="2H27 XMJ"; $resn_exch{"H291 XMJ"}="1H29 XMJ"; $resn_exch{"H292 XMJ"}="2H29 XMJ";
$resn_exch{"H321 XMJ"}="1H32 XMJ"; $resn_exch{"H322 XMJ"}="2H32 XMJ"; $resn_exch{" H11 XMK"}="1H1  XMK";
$resn_exch{"H12A XMK"}="2H1  XMK"; $resn_exch{" H61 XMK"}="1H6  XMK"; $resn_exch{" H62 XMK"}="2H6  XMK";
$resn_exch{" H81 XMK"}="1H8  XMK"; $resn_exch{" H82 XMK"}="2H8  XMK"; $resn_exch{"H101 XMK"}="1H10 XMK";
$resn_exch{"H102 XMK"}="2H10 XMK"; $resn_exch{"H131 XMK"}="1H13 XMK"; $resn_exch{"H132 XMK"}="2H13 XMK";
$resn_exch{"H211 XMK"}="1H21 XMK"; $resn_exch{"H212 XMK"}="2H21 XMK"; $resn_exch{"H221 XMK"}="1H22 XMK";
$resn_exch{"H222 XMK"}="2H22 XMK"; $resn_exch{"H251 XMK"}="1H25 XMK"; $resn_exch{"H252 XMK"}="2H25 XMK";
$resn_exch{"H261 XMK"}="1H26 XMK"; $resn_exch{"H262 XMK"}="2H26 XMK"; $resn_exch{" H61 XMM"}="1H6  XMM";
$resn_exch{" H62 XMM"}="2H6  XMM"; $resn_exch{"H11A XN1"}="1H1  XN1"; $resn_exch{" H12 XN1"}="2H1  XN1";
$resn_exch{" H51 XN1"}="1H5  XN1"; $resn_exch{" H52 XN1"}="2H5  XN1"; $resn_exch{" H53 XN1"}="3H5  XN1";
$resn_exch{" H61 XN1"}="1H6  XN1"; $resn_exch{" H62 XN1"}="2H6  XN1"; $resn_exch{" H63 XN1"}="3H6  XN1";
$resn_exch{" H71 XN1"}="1H7  XN1"; $resn_exch{" H72 XN1"}="2H7  XN1"; $resn_exch{" H73 XN1"}="3H7  XN1";
$resn_exch{" H81 XN1"}="1H8  XN1"; $resn_exch{" H82 XN1"}="2H8  XN1"; $resn_exch{" H91 XN1"}="1H9  XN1";
$resn_exch{" H92 XN1"}="2H9  XN1"; $resn_exch{"H101 XN1"}="1H10 XN1"; $resn_exch{"H102 XN1"}="2H10 XN1";
$resn_exch{"H121 XN1"}="1H12 XN1"; $resn_exch{"H122 XN1"}="2H12 XN1"; $resn_exch{"H141 XN1"}="1H14 XN1";
$resn_exch{"H241 XN1"}="1H24 XN1"; $resn_exch{"H242 XN1"}="2H24 XN1"; $resn_exch{"H311 XN1"}="1H31 XN1";
$resn_exch{"H312 XN1"}="2H31 XN1"; $resn_exch{"H371 XN1"}="1H37 XN1"; $resn_exch{"H372 XN1"}="2H37 XN1";
$resn_exch{"H373 XN1"}="3H37 XN1"; $resn_exch{"H11A XN2"}="1H1  XN2"; $resn_exch{" H12 XN2"}="2H1  XN2";
$resn_exch{" H51 XN2"}="1H5  XN2"; $resn_exch{" H52 XN2"}="2H5  XN2"; $resn_exch{" H53 XN2"}="3H5  XN2";
$resn_exch{" H61 XN2"}="1H6  XN2"; $resn_exch{" H62 XN2"}="2H6  XN2"; $resn_exch{" H63 XN2"}="3H6  XN2";
$resn_exch{" H71 XN2"}="1H7  XN2"; $resn_exch{" H72 XN2"}="2H7  XN2"; $resn_exch{" H73 XN2"}="3H7  XN2";
$resn_exch{" H81 XN2"}="1H8  XN2"; $resn_exch{" H82 XN2"}="2H8  XN2"; $resn_exch{" H91 XN2"}="1H9  XN2";
$resn_exch{" H92 XN2"}="2H9  XN2"; $resn_exch{"H101 XN2"}="1H10 XN2"; $resn_exch{"H102 XN2"}="2H10 XN2";
$resn_exch{"H121 XN2"}="1H12 XN2"; $resn_exch{"H122 XN2"}="2H12 XN2"; $resn_exch{"H141 XN2"}="1H14 XN2";
$resn_exch{"H241 XN2"}="1H24 XN2"; $resn_exch{"H242 XN2"}="2H24 XN2"; $resn_exch{"H311 XN2"}="1H31 XN2";
$resn_exch{"H312 XN2"}="2H31 XN2"; $resn_exch{"H381 XN2"}="1H38 XN2"; $resn_exch{"H382 XN2"}="2H38 XN2";
$resn_exch{"H391 XN2"}="1H39 XN2"; $resn_exch{"H392 XN2"}="2H39 XN2"; $resn_exch{"H393 XN2"}="3H39 XN2";
$resn_exch{"H11A XN3"}="1H1  XN3"; $resn_exch{" H12 XN3"}="2H1  XN3"; $resn_exch{" H51 XN3"}="1H5  XN3";
$resn_exch{" H52 XN3"}="2H5  XN3"; $resn_exch{" H53 XN3"}="3H5  XN3"; $resn_exch{" H61 XN3"}="1H6  XN3";
$resn_exch{" H62 XN3"}="2H6  XN3"; $resn_exch{" H63 XN3"}="3H6  XN3"; $resn_exch{" H71 XN3"}="1H7  XN3";
$resn_exch{" H72 XN3"}="2H7  XN3"; $resn_exch{" H73 XN3"}="3H7  XN3"; $resn_exch{" H81 XN3"}="1H8  XN3";
$resn_exch{" H82 XN3"}="2H8  XN3"; $resn_exch{" H91 XN3"}="1H9  XN3"; $resn_exch{" H92 XN3"}="2H9  XN3";
$resn_exch{"H101 XN3"}="1H10 XN3"; $resn_exch{"H102 XN3"}="2H10 XN3"; $resn_exch{"H121 XN3"}="1H12 XN3";
$resn_exch{"H122 XN3"}="2H12 XN3"; $resn_exch{"H141 XN3"}="1H14 XN3"; $resn_exch{"H142 XN3"}="2H14 XN3";
$resn_exch{"H241 XN3"}="1H24 XN3"; $resn_exch{"H242 XN3"}="2H24 XN3"; $resn_exch{"H311 XN3"}="1H31 XN3";
$resn_exch{"H312 XN3"}="2H31 XN3"; $resn_exch{"H381 XN3"}="1H38 XN3"; $resn_exch{"H382 XN3"}="2H38 XN3";
$resn_exch{" H21 XPE"}="1H2  XPE"; $resn_exch{" H22 XPE"}="2H2  XPE"; $resn_exch{"H31A XPE"}="1H3  XPE";
$resn_exch{" H32 XPE"}="2H3  XPE"; $resn_exch{" H51 XPE"}="1H5  XPE"; $resn_exch{" H52 XPE"}="2H5  XPE";
$resn_exch{" H61 XPE"}="1H6  XPE"; $resn_exch{" H62 XPE"}="2H6  XPE"; $resn_exch{" H81 XPE"}="1H8  XPE";
$resn_exch{" H82 XPE"}="2H8  XPE"; $resn_exch{" H91 XPE"}="1H9  XPE"; $resn_exch{" H92 XPE"}="2H9  XPE";
$resn_exch{"H111 XPE"}="1H11 XPE"; $resn_exch{"H112 XPE"}="2H11 XPE"; $resn_exch{"H121 XPE"}="1H12 XPE";
$resn_exch{"H122 XPE"}="2H12 XPE"; $resn_exch{"H141 XPE"}="1H14 XPE"; $resn_exch{"H142 XPE"}="2H14 XPE";
$resn_exch{"H151 XPE"}="1H15 XPE"; $resn_exch{"H152 XPE"}="2H15 XPE"; $resn_exch{"H171 XPE"}="1H17 XPE";
$resn_exch{"H172 XPE"}="2H17 XPE"; $resn_exch{"H181 XPE"}="1H18 XPE"; $resn_exch{"H182 XPE"}="2H18 XPE";
$resn_exch{"H201 XPE"}="1H20 XPE"; $resn_exch{"H202 XPE"}="2H20 XPE"; $resn_exch{"H211 XPE"}="1H21 XPE";
$resn_exch{"H212 XPE"}="2H21 XPE"; $resn_exch{"H231 XPE"}="1H23 XPE"; $resn_exch{"H232 XPE"}="2H23 XPE";
$resn_exch{"H241 XPE"}="1H24 XPE"; $resn_exch{"H242 XPE"}="2H24 XPE"; $resn_exch{"H261 XPE"}="1H26 XPE";
$resn_exch{"H262 XPE"}="2H26 XPE"; $resn_exch{"H271 XPE"}="1H27 XPE"; $resn_exch{"H272 XPE"}="2H27 XPE";
$resn_exch{"H291 XPE"}="1H29 XPE"; $resn_exch{"H292 XPE"}="2H29 XPE"; $resn_exch{"H301 XPE"}="1H30 XPE";
$resn_exch{"H302 XPE"}="2H30 XPE"; $resn_exch{" H91 XPP"}="1H9  XPP"; $resn_exch{" H92 XPP"}="2H9  XPP";
$resn_exch{" H93 XPP"}="3H9  XPP"; $resn_exch{"H151 XPP"}="1H15 XPP"; $resn_exch{"H152 XPP"}="2H15 XPP";
$resn_exch{"H153 XPP"}="3H15 XPP"; $resn_exch{"H211 XPP"}="1H21 XPP"; $resn_exch{"H212 XPP"}="2H21 XPP";
$resn_exch{"H213 XPP"}="3H21 XPP"; $resn_exch{"H221 XPP"}="1H22 XPP"; $resn_exch{"H222 XPP"}="2H22 XPP";
$resn_exch{"H223 XPP"}="3H22 XPP"; $resn_exch{"H201 XPP"}="1H20 XPP"; $resn_exch{"H202 XPP"}="2H20 XPP";
$resn_exch{"H181 XPP"}="1H18 XPP"; $resn_exch{"H182 XPP"}="2H18 XPP"; $resn_exch{"H191 XPP"}="1H19 XPP";
$resn_exch{"H192 XPP"}="2H19 XPP"; $resn_exch{" H31 XPP"}="1H3  XPP"; $resn_exch{"H32A XPP"}="2H3  XPP";
$resn_exch{" H33 XPP"}="3H3  XPP"; $resn_exch{"H281 XPP"}="1H28 XPP"; $resn_exch{"H282 XPP"}="2H28 XPP";
$resn_exch{"H283 XPP"}="3H28 XPP"; $resn_exch{"H301 XPP"}="1H30 XPP"; $resn_exch{"H302 XPP"}="2H30 XPP";
$resn_exch{"H291 XPP"}="1H29 XPP"; $resn_exch{"H292 XPP"}="2H29 XPP"; $resn_exch{"H461 XPX"}="1H46 XPX";
$resn_exch{"H462 XPX"}="2H46 XPX"; $resn_exch{"H463 XPX"}="3H46 XPX"; $resn_exch{"H451 XPX"}="1H45 XPX";
$resn_exch{"H452 XPX"}="2H45 XPX"; $resn_exch{"H441 XPX"}="1H44 XPX"; $resn_exch{"H442 XPX"}="2H44 XPX";
$resn_exch{"H431 XPX"}="1H43 XPX"; $resn_exch{"H432 XPX"}="2H43 XPX"; $resn_exch{"H421 XPX"}="1H42 XPX";
$resn_exch{"H422 XPX"}="2H42 XPX"; $resn_exch{"H411 XPX"}="1H41 XPX"; $resn_exch{"H412 XPX"}="2H41 XPX";
$resn_exch{"H401 XPX"}="1H40 XPX"; $resn_exch{"H402 XPX"}="2H40 XPX"; $resn_exch{"H391 XPX"}="1H39 XPX";
$resn_exch{"H392 XPX"}="2H39 XPX"; $resn_exch{"H381 XPX"}="1H38 XPX"; $resn_exch{"H382 XPX"}="2H38 XPX";
$resn_exch{"H371 XPX"}="1H37 XPX"; $resn_exch{"H372 XPX"}="2H37 XPX"; $resn_exch{"H361 XPX"}="1H36 XPX";
$resn_exch{"H362 XPX"}="2H36 XPX"; $resn_exch{"H351 XPX"}="1H35 XPX"; $resn_exch{"H352 XPX"}="2H35 XPX";
$resn_exch{"H341 XPX"}="1H34 XPX"; $resn_exch{"H342 XPX"}="2H34 XPX"; $resn_exch{"H331 XPX"}="1H33 XPX";
$resn_exch{"H332 XPX"}="2H33 XPX"; $resn_exch{"H321 XPX"}="1H32 XPX"; $resn_exch{"H322 XPX"}="2H32 XPX";
$resn_exch{"H301 XPX"}="1H30 XPX"; $resn_exch{"H302 XPX"}="2H30 XPX"; $resn_exch{"H151 XPX"}="1H15 XPX";
$resn_exch{"H152 XPX"}="2H15 XPX"; $resn_exch{"H161 XPX"}="1H16 XPX"; $resn_exch{"H162 XPX"}="2H16 XPX";
$resn_exch{"H171 XPX"}="1H17 XPX"; $resn_exch{"H172 XPX"}="2H17 XPX"; $resn_exch{"H181 XPX"}="1H18 XPX";
$resn_exch{"H182 XPX"}="2H18 XPX"; $resn_exch{"H191 XPX"}="1H19 XPX"; $resn_exch{"H192 XPX"}="2H19 XPX";
$resn_exch{"H201 XPX"}="1H20 XPX"; $resn_exch{"H202 XPX"}="2H20 XPX"; $resn_exch{"H211 XPX"}="1H21 XPX";
$resn_exch{"H212 XPX"}="2H21 XPX"; $resn_exch{"H221 XPX"}="1H22 XPX"; $resn_exch{"H222 XPX"}="2H22 XPX";
$resn_exch{"H231 XPX"}="1H23 XPX"; $resn_exch{"H232 XPX"}="2H23 XPX"; $resn_exch{"H241 XPX"}="1H24 XPX";
$resn_exch{"H242 XPX"}="2H24 XPX"; $resn_exch{"H251 XPX"}="1H25 XPX"; $resn_exch{"H252 XPX"}="2H25 XPX";
$resn_exch{"H261 XPX"}="1H26 XPX"; $resn_exch{"H262 XPX"}="2H26 XPX"; $resn_exch{"H271 XPX"}="1H27 XPX";
$resn_exch{"H272 XPX"}="2H27 XPX"; $resn_exch{"H281 XPX"}="1H28 XPX"; $resn_exch{"H282 XPX"}="2H28 XPX";
$resn_exch{"H291 XPX"}="1H29 XPX"; $resn_exch{"H292 XPX"}="2H29 XPX"; $resn_exch{"H293 XPX"}="3H29 XPX";
$resn_exch{"H121 XPX"}="1H12 XPX"; $resn_exch{"H122 XPX"}="2H12 XPX"; $resn_exch{"H111 XPX"}="1H11 XPX";
$resn_exch{"H112 XPX"}="2H11 XPX"; $resn_exch{"H51A XPX"}="1H5  XPX"; $resn_exch{"H52A XPX"}="2H5  XPX";
$resn_exch{" H61 XS2"}="1H6  XS2"; $resn_exch{" H62 XS2"}="2H6  XS2"; $resn_exch{" H63 XS2"}="3H6  XS2";
$resn_exch{" H51 XS2"}="1H5  XS2"; $resn_exch{" H52 XS2"}="2H5  XS2"; $resn_exch{"H111 XS2"}="1H11 XS2";
$resn_exch{"H112 XS2"}="2H11 XS2"; $resn_exch{"H171 XS2"}="1H17 XS2"; $resn_exch{"H172 XS2"}="2H17 XS2";
$resn_exch{"H231 XS2"}="1H23 XS2"; $resn_exch{"H232 XS2"}="2H23 XS2"; $resn_exch{"H291 XS2"}="1H29 XS2";
$resn_exch{"H292 XS2"}="2H29 XS2"; $resn_exch{" H61 XTG"}="1H6  XTG"; $resn_exch{" H62 XTG"}="2H6  XTG";
$resn_exch{"H111 XTG"}="1H11 XTG"; $resn_exch{"H112 XTG"}="2H11 XTG"; $resn_exch{"HOP3 XTH"}="3HOP XTH";
$resn_exch{"HOP2 XTH"}="2HOP XTH"; $resn_exch{"H6'1 XTH"}="1H6' XTH"; $resn_exch{"H6'2 XTH"}="2H6' XTH";
$resn_exch{"H3'1 XTH"}="1H3' XTH"; $resn_exch{"H3'2 XTH"}="2H3' XTH"; $resn_exch{"H2'1 XTH"}="1H2' XTH";
$resn_exch{"H2'2 XTH"}="2H2' XTH"; $resn_exch{" H71 XTH"}="1H5M XTH"; $resn_exch{" H72 XTH"}="2H5M XTH";
$resn_exch{" H73 XTH"}="3H5M XTH"; $resn_exch{"HOP2 XTL"}="2HOP XTL"; $resn_exch{" H5' XTL"}="1H5* XTL";
$resn_exch{"H5'' XTL"}="2H5* XTL"; $resn_exch{" H4' XTL"}=" H4* XTL"; $resn_exch{" H6' XTL"}=" H6* XTL";
$resn_exch{" H7' XTL"}=" H7* XTL"; $resn_exch{" H3' XTL"}=" H3* XTL"; $resn_exch{"HO3' XTL"}="H3T  XTL";
$resn_exch{" H2' XTL"}="1H2* XTL"; $resn_exch{"H2'' XTL"}="2H2* XTL"; $resn_exch{" H1' XTL"}=" H1* XTL";
$resn_exch{" H71 XTL"}="1H5M XTL"; $resn_exch{" H72 XTL"}="2H5M XTL"; $resn_exch{" H73 XTL"}="3H5M XTL";
$resn_exch{"HOP3 XTL"}="3HOP XTL"; $resn_exch{" H1' XTS"}=" H1* XTS"; $resn_exch{" H4' XTS"}=" H4* XTS";
$resn_exch{"H5'1 XTS"}="1H5* XTS"; $resn_exch{"H5'2 XTS"}="2H5* XTS"; $resn_exch{"HO5' XTS"}="HO5* XTS";
$resn_exch{" H3' XTS"}=" H3* XTS"; $resn_exch{"HO3' XTS"}="HO3* XTS"; $resn_exch{" H2' XTS"}=" H2* XTS";
$resn_exch{"HO2' XTS"}="HO2* XTS"; $resn_exch{"H5'1 XTY"}="1H5' XTY"; $resn_exch{"H5'2 XTY"}="2H5' XTY";
$resn_exch{"H2'1 XTY"}="1H2' XTY"; $resn_exch{"H2'2 XTY"}="2H2' XTY"; $resn_exch{"H7A1 XTY"}="1H7A XTY";
$resn_exch{"H7A2 XTY"}="2H7A XTY"; $resn_exch{"H7A3 XTY"}="3H7A XTY"; $resn_exch{"HO3' XTY"}="H3T  XTY";
$resn_exch{"HN21 XUG"}="1HN2 XUG"; $resn_exch{"HN22 XUG"}="2HN2 XUG"; $resn_exch{" H1' XUG"}=" H1* XUG";
$resn_exch{" H2' XUG"}=" H2* XUG"; $resn_exch{"HA'1 XUG"}="1HA* XUG"; $resn_exch{"HA'2 XUG"}="2HA* XUG";
$resn_exch{"HA'3 XUG"}="3HA* XUG"; $resn_exch{" H3' XUG"}=" H3* XUG"; $resn_exch{"HO3' XUG"}="H3T  XUG";
$resn_exch{" H4' XUG"}=" H4* XUG"; $resn_exch{" H5' XUG"}="1H5* XUG"; $resn_exch{"H5'' XUG"}="2H5* XUG";
$resn_exch{"HOP2 XUG"}="2HOP XUG"; $resn_exch{"HOP3 XUG"}="3HOP XUG"; $resn_exch{" H11 XUL"}="1H1  XUL";
$resn_exch{" H12 XUL"}="2H1  XUL"; $resn_exch{" H51 XUL"}="1H5  XUL"; $resn_exch{" H52 XUL"}="2H5  XUL";
$resn_exch{"HO41 XV6"}="1HO4 XV6"; $resn_exch{"HO51 XV6"}="1HO5 XV6"; $resn_exch{" H21 XV6"}="1H2  XV6";
$resn_exch{" H22 XV6"}="2H2  XV6"; $resn_exch{" H31 XV6"}="1H3  XV6"; $resn_exch{" H41 XV6"}="1H4  XV6";
$resn_exch{" H51 XV6"}="1H5  XV6"; $resn_exch{" H61 XV6"}="1H6  XV6"; $resn_exch{" H71 XV6"}="1H7  XV6";
$resn_exch{" H72 XV6"}="2H7  XV6"; $resn_exch{"H311 XV6"}="1H31 XV6"; $resn_exch{"H312 XV6"}="2H31 XV6";
$resn_exch{"H331 XV6"}="1H33 XV6"; $resn_exch{"H341 XV6"}="1H34 XV6"; $resn_exch{"H351 XV6"}="1H35 XV6";
$resn_exch{"H361 XV6"}="1H36 XV6"; $resn_exch{"H371 XV6"}="1H37 XV6"; $resn_exch{"H611 XV6"}="1H61 XV6";
$resn_exch{"H612 XV6"}="2H61 XV6"; $resn_exch{"H631 XV6"}="1H63 XV6"; $resn_exch{"H641 XV6"}="1H64 XV6";
$resn_exch{"H651 XV6"}="1H65 XV6"; $resn_exch{"H661 XV6"}="1H66 XV6"; $resn_exch{"H671 XV6"}="1H67 XV6";
$resn_exch{"H211 XV6"}="1H21 XV6"; $resn_exch{"H231 XV6"}="1H23 XV6"; $resn_exch{"H241 XV6"}="1H24 XV6";
$resn_exch{"H251 XV6"}="1H25 XV6"; $resn_exch{"H281 XV6"}="1H28 XV6"; $resn_exch{"H291 XV6"}="1H29 XV6";
$resn_exch{"H711 XV6"}="1H71 XV6"; $resn_exch{"H731 XV6"}="1H73 XV6"; $resn_exch{"H741 XV6"}="1H74 XV6";
$resn_exch{"H751 XV6"}="1H75 XV6"; $resn_exch{"H781 XV6"}="1H78 XV6"; $resn_exch{"H791 XV6"}="1H79 XV6";
$resn_exch{" H   XX1"}="1HN  XX1"; $resn_exch{" H2  XX1"}="2HN  XX1"; $resn_exch{" HB2 XX1"}="1HB  XX1";
$resn_exch{" HB3 XX1"}="2HB  XX1"; $resn_exch{" HG2 XX1"}="1HG  XX1"; $resn_exch{" HG3 XX1"}="2HG  XX1";
$resn_exch{" HD2 XX1"}="1HD  XX1"; $resn_exch{" HD3 XX1"}="2HD  XX1"; $resn_exch{" HE2 XX1"}="1HE  XX1";
$resn_exch{" HE3 XX1"}="2HE  XX1"; $resn_exch{" HZ  XX1"}=" HNZ XX1"; $resn_exch{" HC2 XX1"}=" H2  XX1";
$resn_exch{" H31 XX3"}="1H3  XX3"; $resn_exch{"H011 XX4"}="1H01 XX4"; $resn_exch{"H012 XX4"}="2H01 XX4";
$resn_exch{"H013 XX4"}="3H01 XX4"; $resn_exch{"H051 XX4"}="1H05 XX4"; $resn_exch{"H052 XX4"}="2H05 XX4";
$resn_exch{"H061 XX4"}="1H06 XX4"; $resn_exch{"H062 XX4"}="2H06 XX4"; $resn_exch{"H081 XX4"}="1H08 XX4";
$resn_exch{"H082 XX4"}="2H08 XX4"; $resn_exch{"H251 XX4"}="1H25 XX4"; $resn_exch{"H252 XX4"}="2H25 XX4";
$resn_exch{"H271 XX4"}="1H27 XX4"; $resn_exch{"H272 XX4"}="2H27 XX4"; $resn_exch{"H281 XX4"}="1H28 XX4";
$resn_exch{"H282 XX4"}="2H28 XX4"; $resn_exch{"H291 XX4"}="1H29 XX4"; $resn_exch{"H292 XX4"}="2H29 XX4";
$resn_exch{"H301 XX4"}="1H30 XX4"; $resn_exch{"H302 XX4"}="2H30 XX4"; $resn_exch{"H311 XX4"}="1H31 XX4";
$resn_exch{"H312 XX4"}="2H31 XX4"; $resn_exch{"HC51 XX5"}="1HC5 XX5"; $resn_exch{"HC52 XX5"}="2HC5 XX5";
$resn_exch{"HC71 XX5"}="1HC7 XX5"; $resn_exch{"HC72 XX5"}="2HC7 XX5"; $resn_exch{"HC73 XX5"}="3HC7 XX5";
$resn_exch{"HC81 XX5"}="1HC8 XX5"; $resn_exch{"HC82 XX5"}="2HC8 XX5"; $resn_exch{"HC83 XX5"}="3HC8 XX5";
$resn_exch{"H141 XX5"}="1H14 XX5"; $resn_exch{"H142 XX5"}="2H14 XX5"; $resn_exch{"H201 XX5"}="1H20 XX5";
$resn_exch{"H202 XX5"}="2H20 XX5"; $resn_exch{"HN11 XXX"}="1HN1 XXX"; $resn_exch{"HN12 XXX"}="2HN1 XXX";
$resn_exch{" H71 XXX"}="1H7  XXX"; $resn_exch{" H72 XXX"}="2H7  XXX"; $resn_exch{"HN21 XXX"}="1HN2 XXX";
$resn_exch{"HN22 XXX"}="2HN2 XXX"; $resn_exch{"H121 XXX"}="1H12 XXX"; $resn_exch{"H122 XXX"}="2H12 XXX";
$resn_exch{"HN31 XXX"}="1HN3 XXX"; $resn_exch{"HN32 XXX"}="2HN3 XXX"; $resn_exch{"HN41 XXX"}="1HN4 XXX";
$resn_exch{"HN42 XXX"}="2HN4 XXX"; $resn_exch{"HN11 XXY"}="1HN1 XXY"; $resn_exch{"HN12 XXY"}="2HN1 XXY";
$resn_exch{"HG11 XXY"}="1HG1 XXY"; $resn_exch{"HG12 XXY"}="2HG1 XXY"; $resn_exch{"HG13 XXY"}="3HG1 XXY";
$resn_exch{"HA31 XXY"}="1HA3 XXY"; $resn_exch{"HA32 XXY"}="2HA3 XXY"; $resn_exch{"HB21 XXY"}="1HB2 XXY";
$resn_exch{"HB22 XXY"}="2HB2 XXY"; $resn_exch{"H191 XY2"}="1H19 XY2"; $resn_exch{"H192 XY2"}="2H19 XY2";
$resn_exch{"H193 XY2"}="3H19 XY2"; $resn_exch{"H161 XY2"}="1H16 XY2"; $resn_exch{"H162 XY2"}="2H16 XY2";
$resn_exch{"H163 XY2"}="3H16 XY2"; $resn_exch{"H141 XY2"}="1H14 XY2"; $resn_exch{"H142 XY2"}="2H14 XY2";
$resn_exch{"H131 XY2"}="1H13 XY2"; $resn_exch{"H132 XY2"}="2H13 XY2"; $resn_exch{"H121 XY2"}="1H12 XY2";
$resn_exch{"H122 XY2"}="2H12 XY2"; $resn_exch{"H123 XY2"}="3H12 XY2"; $resn_exch{"H5'1 XYA"}="1H5' XYA";
$resn_exch{"H5'2 XYA"}="2H5' XYA"; $resn_exch{"HN61 XYA"}="1HN6 XYA"; $resn_exch{"HN62 XYA"}="2HN6 XYA";
$resn_exch{" H81 XYD"}="1H8  XYD"; $resn_exch{" H82 XYD"}="2H8  XYD"; $resn_exch{" H83 XYD"}="3H8  XYD";
$resn_exch{" H91 XYD"}="1H9  XYD"; $resn_exch{" H92 XYD"}="2H9  XYD"; $resn_exch{" H93 XYD"}="3H9  XYD";
$resn_exch{"HA31 XYG"}="1HA3 XYG"; $resn_exch{"HA32 XYG"}="2HA3 XYG"; $resn_exch{" HN2 XYG"}="2HN  XYG";
$resn_exch{"HB11 XYG"}="1HB1 XYG"; $resn_exch{"HB12 XYG"}="2HB1 XYG"; $resn_exch{"HB13 XYG"}="3HB1 XYG";
$resn_exch{" H21 XYH"}="1H2  XYH"; $resn_exch{" H31 XYH"}="1H3  XYH"; $resn_exch{" H41 XYH"}="1H4  XYH";
$resn_exch{" H11 XYL"}="1H1  XYL"; $resn_exch{" H12 XYL"}="2H1  XYL"; $resn_exch{" H51 XYL"}="1H5  XYL";
$resn_exch{" H52 XYL"}="2H5  XYL"; $resn_exch{"H5B1 XYP"}="1H5B XYP"; $resn_exch{"H5B2 XYP"}="2H5B XYP";
$resn_exch{"HH31 XYQ"}="1HH3 XYQ"; $resn_exch{"HH32 XYQ"}="2HH3 XYQ"; $resn_exch{"HH33 XYQ"}="3HH3 XYQ";
$resn_exch{"H5N1 XYQ"}="1H5N XYQ"; $resn_exch{"H5N2 XYQ"}="2H5N XYQ"; $resn_exch{"H5A1 XYQ"}="1H5A XYQ";
$resn_exch{"H5A2 XYQ"}="2H5A XYQ"; $resn_exch{"HN61 XYQ"}="1HN6 XYQ"; $resn_exch{"HN62 XYQ"}="2HN6 XYQ";
$resn_exch{" H51 XYS"}="1H5  XYS"; $resn_exch{" H52 XYS"}="2H5  XYS"; $resn_exch{" H51 XYZ"}="1H5  XYZ";
$resn_exch{" H52 XYZ"}="2H5  XYZ"; $resn_exch{"HOP2   Y"}="2HOP   Y"; $resn_exch{"HOP3   Y"}="3HOP   Y";
$resn_exch{" H5'   Y"}="1H5*   Y"; $resn_exch{"H5''   Y"}="2H5*   Y"; $resn_exch{" H4'   Y"}=" H4*   Y";
$resn_exch{" H3'   Y"}=" H3*   Y"; $resn_exch{"HO3'   Y"}="*HO3   Y"; $resn_exch{" H2'   Y"}="1H2*   Y";
$resn_exch{"H2''   Y"}="2H2*   Y"; $resn_exch{" H1'   Y"}=" H1*   Y"; $resn_exch{" HB1   Y"}="1HB    Y";
$resn_exch{" HB2   Y"}="2HB    Y"; $resn_exch{"H021 Y11"}="1H02 Y11"; $resn_exch{"H022 Y11"}="2H02 Y11";
$resn_exch{"H031 Y11"}="1H03 Y11"; $resn_exch{"H032 Y11"}="2H03 Y11"; $resn_exch{"H041 Y11"}="1H04 Y11";
$resn_exch{"H042 Y11"}="2H04 Y11"; $resn_exch{"H051 Y11"}="1H05 Y11"; $resn_exch{"H052 Y11"}="2H05 Y11";
$resn_exch{"H111 Y11"}="1H11 Y11"; $resn_exch{"H112 Y11"}="2H11 Y11"; $resn_exch{"H131 Y11"}="1H13 Y11";
$resn_exch{"H132 Y11"}="2H13 Y11"; $resn_exch{"H141 Y11"}="1H14 Y11"; $resn_exch{"H142 Y11"}="2H14 Y11";
$resn_exch{"H151 Y11"}="1H15 Y11"; $resn_exch{"H152 Y11"}="2H15 Y11"; $resn_exch{"H161 Y11"}="1H16 Y11";
$resn_exch{"H162 Y11"}="2H16 Y11"; $resn_exch{"H171 Y11"}="1H17 Y11"; $resn_exch{"H172 Y11"}="2H17 Y11";
$resn_exch{"H211 Y11"}="1H21 Y11"; $resn_exch{"H212 Y11"}="2H21 Y11"; $resn_exch{"H221 Y11"}="1H22 Y11";
$resn_exch{"H222 Y11"}="2H22 Y11"; $resn_exch{"H231 Y11"}="1H23 Y11"; $resn_exch{"H232 Y11"}="2H23 Y11";
$resn_exch{"H241 Y11"}="1H24 Y11"; $resn_exch{"H242 Y11"}="2H24 Y11"; $resn_exch{"H261 Y11"}="1H26 Y11";
$resn_exch{"H262 Y11"}="2H26 Y11"; $resn_exch{"H263 Y11"}="3H26 Y11"; $resn_exch{" H71 Y12"}="1H7  Y12";
$resn_exch{" H72 Y12"}="2H7  Y12"; $resn_exch{" H81 Y12"}="1H8  Y12"; $resn_exch{" H82 Y12"}="2H8  Y12";
$resn_exch{"H251 Y12"}="1H25 Y12"; $resn_exch{"H252 Y12"}="2H25 Y12"; $resn_exch{"H253 Y12"}="3H25 Y12";
$resn_exch{"H211 Y12"}="1H21 Y12"; $resn_exch{"H212 Y12"}="2H21 Y12"; $resn_exch{"H213 Y12"}="3H21 Y12";
$resn_exch{" H71 Y13"}="1H7  Y13"; $resn_exch{" H72 Y13"}="2H7  Y13"; $resn_exch{" H81 Y13"}="1H8  Y13";
$resn_exch{" H82 Y13"}="2H8  Y13"; $resn_exch{"H021 Y14"}="1H02 Y14"; $resn_exch{"H022 Y14"}="2H02 Y14";
$resn_exch{"H031 Y14"}="1H03 Y14"; $resn_exch{"H032 Y14"}="2H03 Y14"; $resn_exch{"H041 Y14"}="1H04 Y14";
$resn_exch{"H042 Y14"}="2H04 Y14"; $resn_exch{"H051 Y14"}="1H05 Y14"; $resn_exch{"H052 Y14"}="2H05 Y14";
$resn_exch{"H111 Y14"}="1H11 Y14"; $resn_exch{"H112 Y14"}="2H11 Y14"; $resn_exch{"H131 Y14"}="1H13 Y14";
$resn_exch{"H132 Y14"}="2H13 Y14"; $resn_exch{"H141 Y14"}="1H14 Y14"; $resn_exch{"H142 Y14"}="2H14 Y14";
$resn_exch{"H151 Y14"}="1H15 Y14"; $resn_exch{"H152 Y14"}="2H15 Y14"; $resn_exch{"H161 Y14"}="1H16 Y14";
$resn_exch{"H162 Y14"}="2H16 Y14"; $resn_exch{"H171 Y14"}="1H17 Y14"; $resn_exch{"H172 Y14"}="2H17 Y14";
$resn_exch{"H201 Y14"}="1H20 Y14"; $resn_exch{"H202 Y14"}="2H20 Y14"; $resn_exch{"H211 Y14"}="1H21 Y14";
$resn_exch{"H212 Y14"}="2H21 Y14"; $resn_exch{"H221 Y14"}="1H22 Y14"; $resn_exch{"H222 Y14"}="2H22 Y14";
$resn_exch{"H251 Y14"}="1H25 Y14"; $resn_exch{"H252 Y14"}="2H25 Y14"; $resn_exch{"H021 Y15"}="1H02 Y15";
$resn_exch{"H022 Y15"}="2H02 Y15"; $resn_exch{"H031 Y15"}="1H03 Y15"; $resn_exch{"H032 Y15"}="2H03 Y15";
$resn_exch{"H041 Y15"}="1H04 Y15"; $resn_exch{"H042 Y15"}="2H04 Y15"; $resn_exch{"H051 Y15"}="1H05 Y15";
$resn_exch{"H052 Y15"}="2H05 Y15"; $resn_exch{"H111 Y15"}="1H11 Y15"; $resn_exch{"H112 Y15"}="2H11 Y15";
$resn_exch{"H131 Y15"}="1H13 Y15"; $resn_exch{"H132 Y15"}="2H13 Y15"; $resn_exch{"H141 Y15"}="1H14 Y15";
$resn_exch{"H142 Y15"}="2H14 Y15"; $resn_exch{"H151 Y15"}="1H15 Y15"; $resn_exch{"H152 Y15"}="2H15 Y15";
$resn_exch{"H161 Y15"}="1H16 Y15"; $resn_exch{"H162 Y15"}="2H16 Y15"; $resn_exch{"H171 Y15"}="1H17 Y15";
$resn_exch{"H172 Y15"}="2H17 Y15"; $resn_exch{"H201 Y15"}="1H20 Y15"; $resn_exch{"H202 Y15"}="2H20 Y15";
$resn_exch{"H211 Y15"}="1H21 Y15"; $resn_exch{"H212 Y15"}="2H21 Y15"; $resn_exch{"H213 Y15"}="3H21 Y15";
$resn_exch{"H241 Y15"}="1H24 Y15"; $resn_exch{"H242 Y15"}="2H24 Y15"; $resn_exch{"H151 Y19"}="1H15 Y19";
$resn_exch{"H152 Y19"}="2H15 Y19"; $resn_exch{"H161 Y19"}="1H16 Y19"; $resn_exch{"H162 Y19"}="2H16 Y19";
$resn_exch{"H163 Y19"}="3H16 Y19"; $resn_exch{"H8C1 Y19"}="1H8C Y19"; $resn_exch{"H8C2 Y19"}="2H8C Y19";
$resn_exch{"H121 Y19"}="1H12 Y19"; $resn_exch{"H122 Y19"}="2H12 Y19"; $resn_exch{"H361 Y27"}="1H36 Y27";
$resn_exch{"H362 Y27"}="2H36 Y27"; $resn_exch{"H351 Y27"}="1H35 Y27"; $resn_exch{"H352 Y27"}="2H35 Y27";
$resn_exch{"H331 Y27"}="1H33 Y27"; $resn_exch{"H332 Y27"}="2H33 Y27"; $resn_exch{"H321 Y27"}="1H32 Y27";
$resn_exch{"H322 Y27"}="2H32 Y27"; $resn_exch{"H421 Y27"}="1H42 Y27"; $resn_exch{"H422 Y27"}="2H42 Y27";
$resn_exch{"H423 Y27"}="3H42 Y27"; $resn_exch{"H431 Y27"}="1H43 Y27"; $resn_exch{"H432 Y27"}="2H43 Y27";
$resn_exch{"H181  Y3"}="1H18  Y3"; $resn_exch{"H182  Y3"}="2H18  Y3"; $resn_exch{"H183  Y3"}="3H18  Y3";
$resn_exch{"H121 YBH"}="1H12 YBH"; $resn_exch{"H122 YBH"}="2H12 YBH"; $resn_exch{"H111 YBH"}="1H11 YBH";
$resn_exch{"H112 YBH"}="2H11 YBH"; $resn_exch{"H251 YBH"}="1H25 YBH"; $resn_exch{"H252 YBH"}="2H25 YBH";
$resn_exch{"H131 YBH"}="1H13 YBH"; $resn_exch{"H132 YBH"}="2H13 YBH"; $resn_exch{"H141 YBH"}="1H14 YBH";
$resn_exch{"H142 YBH"}="2H14 YBH"; $resn_exch{" H21 YBT"}="1H2  YBT"; $resn_exch{" H22 YBT"}="2H2  YBT";
$resn_exch{" H31 YBT"}="1H3  YBT"; $resn_exch{" H32 YBT"}="2H3  YBT"; $resn_exch{" H41 YBT"}="1H4  YBT";
$resn_exch{" H42 YBT"}="2H4  YBT"; $resn_exch{" H51 YBT"}="1H5  YBT"; $resn_exch{" H52 YBT"}="2H5  YBT";
$resn_exch{" H61 YBT"}="1H6  YBT"; $resn_exch{" H62 YBT"}="2H6  YBT"; $resn_exch{" H71 YBT"}="1H7  YBT";
$resn_exch{" H72 YBT"}="2H7  YBT"; $resn_exch{" H81 YBT"}="1H8  YBT"; $resn_exch{" H82 YBT"}="2H8  YBT";
$resn_exch{" H2  YCM"}=" HN2 YCM"; $resn_exch{" HB2 YCM"}="1HB  YCM"; $resn_exch{" HB3 YCM"}="2HB  YCM";
$resn_exch{" HD2 YCM"}="1HD  YCM"; $resn_exch{" HD3 YCM"}="2HD  YCM"; $resn_exch{"HZ21 YCM"}="1HZ  YCM";
$resn_exch{"HZ22 YCM"}="2HZ  YCM"; $resn_exch{"HN61 YE1"}="1HN6 YE1"; $resn_exch{"HN62 YE1"}="2HN6 YE1";
$resn_exch{" H1' YE1"}=" H1* YE1"; $resn_exch{" H2' YE1"}=" H2* YE1"; $resn_exch{"HO2' YE1"}="HO2* YE1";
$resn_exch{" H3' YE1"}=" H3* YE1"; $resn_exch{" H4' YE1"}=" H4* YE1"; $resn_exch{"H5'1 YE1"}="1H5* YE1";
$resn_exch{"H5'2 YE1"}="2H5* YE1"; $resn_exch{"H121 YE1"}="1H12 YE1"; $resn_exch{"H122 YE1"}="2H12 YE1";
$resn_exch{"H141 YE1"}="1H14 YE1"; $resn_exch{"H142 YE1"}="2H14 YE1"; $resn_exch{"H143 YE1"}="3H14 YE1";
$resn_exch{"H131 YE1"}="1H13 YE1"; $resn_exch{"H132 YE1"}="2H13 YE1"; $resn_exch{"H133 YE1"}="3H13 YE1";
$resn_exch{"HC71 YE1"}="1HC7 YE1"; $resn_exch{"HC72 YE1"}="2HC7 YE1"; $resn_exch{"HC61 YE1"}="1HC6 YE1";
$resn_exch{"HC62 YE1"}="2HC6 YE1"; $resn_exch{"HC31 YE1"}="1HC3 YE1"; $resn_exch{"HC32 YE1"}="2HC3 YE1";
$resn_exch{"HC21 YE1"}="1HC2 YE1"; $resn_exch{"HC22 YE1"}="2HC2 YE1"; $resn_exch{"HAC1 YE1"}="1HAC YE1";
$resn_exch{"HAC2 YE1"}="2HAC YE1"; $resn_exch{"HAP1 YE7"}="1HAP YE7"; $resn_exch{"HAP2 YE7"}="2HAP YE7";
$resn_exch{" H11 YF1"}="1H1  YF1"; $resn_exch{" H12 YF1"}="2H1  YF1"; $resn_exch{" H13 YF1"}="3H1  YF1";
$resn_exch{" H51 YF1"}="1H5  YF1"; $resn_exch{" H52 YF1"}="2H5  YF1"; $resn_exch{" H53 YF1"}="3H5  YF1";
$resn_exch{"HN21 YF1"}="1HN2 YF1"; $resn_exch{"HN22 YF1"}="2HN2 YF1"; $resn_exch{"HM21 YF3"}="1HM2 YF3";
$resn_exch{"HM22 YF3"}="2HM2 YF3"; $resn_exch{"HM23 YF3"}="3HM2 YF3"; $resn_exch{"H4'1 YF3"}="1H4' YF3";
$resn_exch{"H4'2 YF3"}="2H4' YF3"; $resn_exch{"H7'1 YF3"}="1H7' YF3"; $resn_exch{"H7'2 YF3"}="2H7' YF3";
$resn_exch{"HM41 YF3"}="1HM4 YF3"; $resn_exch{"HM42 YF3"}="2HM4 YF3"; $resn_exch{"HM43 YF3"}="3HM4 YF3";
$resn_exch{" H21 YF3"}="1H2  YF3"; $resn_exch{" H22 YF3"}="2H2  YF3"; $resn_exch{"HM21 YF4"}="1HM2 YF4";
$resn_exch{"HM22 YF4"}="2HM2 YF4"; $resn_exch{"HM23 YF4"}="3HM2 YF4"; $resn_exch{"H4'1 YF4"}="1H4' YF4";
$resn_exch{"H4'2 YF4"}="2H4' YF4"; $resn_exch{"H7'1 YF4"}="1H7' YF4"; $resn_exch{"H7'2 YF4"}="2H7' YF4";
$resn_exch{"HM41 YF4"}="1HM4 YF4"; $resn_exch{"HM42 YF4"}="2HM4 YF4"; $resn_exch{"HM43 YF4"}="3HM4 YF4";
$resn_exch{" H41 YF4"}="1H4  YF4"; $resn_exch{" H42 YF4"}="2H4  YF4"; $resn_exch{" H21 YF4"}="1H2  YF4";
$resn_exch{" H22 YF4"}="2H2  YF4"; $resn_exch{" H23 YF4"}="3H2  YF4"; $resn_exch{" H31  YG"}="1H3   YG";
$resn_exch{" H32  YG"}="2H3   YG"; $resn_exch{" H33  YG"}="3H3   YG"; $resn_exch{"H101  YG"}="1H10  YG";
$resn_exch{"H102  YG"}="2H10  YG"; $resn_exch{"H103  YG"}="3H10  YG"; $resn_exch{"H131  YG"}="1H13  YG";
$resn_exch{"H132  YG"}="2H13  YG"; $resn_exch{"H141  YG"}="1H14  YG"; $resn_exch{"H142  YG"}="2H14  YG";
$resn_exch{"H191  YG"}="1H19  YG"; $resn_exch{"H192  YG"}="2H19  YG"; $resn_exch{"H193  YG"}="3H19  YG";
$resn_exch{"H241  YG"}="1H24  YG"; $resn_exch{"H242  YG"}="2H24  YG"; $resn_exch{"H243  YG"}="3H24  YG";
$resn_exch{" H1'  YG"}=" H1*  YG"; $resn_exch{" H2'  YG"}=" H2*  YG"; $resn_exch{"HO2'  YG"}="*HO2  YG";
$resn_exch{" H3'  YG"}=" H3*  YG"; $resn_exch{"HO3'  YG"}="*HO3  YG"; $resn_exch{" H4'  YG"}=" H4*  YG";
$resn_exch{" H5'  YG"}="1H5*  YG"; $resn_exch{"H5''  YG"}="2H5*  YG"; $resn_exch{"HOP2  YG"}="2HOP  YG";
$resn_exch{"HOP3  YG"}="3HOP  YG"; $resn_exch{"H6C1 YLL"}="1H6C YLL"; $resn_exch{"H6C2 YLL"}="2H6C YLL";
$resn_exch{"HAA1 YLY"}="1HAA YLY"; $resn_exch{"HAA2 YLY"}="2HAA YLY"; $resn_exch{"HAA3 YLY"}="3HAA YLY";
$resn_exch{"HAP1 YLY"}="1HAP YLY"; $resn_exch{"HAP2 YLY"}="2HAP YLY"; $resn_exch{"HAJ1 YLY"}="1HAJ YLY";
$resn_exch{"HAJ2 YLY"}="2HAJ YLY"; $resn_exch{"HAO1 YLY"}="1HAO YLY"; $resn_exch{"HAO2 YLY"}="2HAO YLY";
$resn_exch{"HAM1 YLY"}="1HAM YLY"; $resn_exch{"HAM2 YLY"}="2HAM YLY"; $resn_exch{"HAN1 YLY"}="1HAN YLY";
$resn_exch{"HAN2 YLY"}="2HAN YLY"; $resn_exch{" HB1 YLY"}="1HB  YLY"; $resn_exch{" HB2 YLY"}="2HB  YLY";
$resn_exch{" HN1 YLY"}="1HN  YLY"; $resn_exch{" HN2 YLY"}="2HN  YLY"; $resn_exch{"H5'1 YLY"}="1H5* YLY";
$resn_exch{"H5'2 YLY"}="2H5* YLY"; $resn_exch{" H4' YLY"}=" H4* YLY"; $resn_exch{" H3' YLY"}=" H3* YLY";
$resn_exch{"HO3' YLY"}="HO3* YLY"; $resn_exch{" H2' YLY"}=" H2* YLY"; $resn_exch{"HO2' YLY"}="HO2* YLY";
$resn_exch{" H1' YLY"}=" H1* YLY"; $resn_exch{"HN61 YLY"}="1HN6 YLY"; $resn_exch{"HN62 YLY"}="2HN6 YLY";
$resn_exch{" H11 YMA"}="1H1  YMA"; $resn_exch{" H12 YMA"}="2H1  YMA"; $resn_exch{"H101 YMA"}="1H10 YMA";
$resn_exch{"H102 YMA"}="2H10 YMA"; $resn_exch{"H111 YMA"}="1H11 YMA"; $resn_exch{"H112 YMA"}="2H11 YMA";
$resn_exch{"H141 YMA"}="1H14 YMA"; $resn_exch{"H142 YMA"}="2H14 YMA"; $resn_exch{"H161 YMA"}="1H16 YMA";
$resn_exch{"H162 YMA"}="2H16 YMA"; $resn_exch{"H181 YMA"}="1H18 YMA"; $resn_exch{"H182 YMA"}="2H18 YMA";
$resn_exch{"H121 YMA"}="1H12 YMA"; $resn_exch{"H122 YMA"}="2H12 YMA"; $resn_exch{" H31 YMA"}="1H3  YMA";
$resn_exch{" H32 YMA"}="2H3  YMA"; $resn_exch{" H41 YMA"}="1H4  YMA"; $resn_exch{" H42 YMA"}="2H4  YMA";
$resn_exch{" H51 YMA"}="1H5  YMA"; $resn_exch{" H52 YMA"}="2H5  YMA"; $resn_exch{" H71 YMA"}="1H7  YMA";
$resn_exch{" H72 YMA"}="2H7  YMA"; $resn_exch{" H91 YMA"}="1H9  YMA"; $resn_exch{" H92 YMA"}="2H9  YMA";
$resn_exch{"H171 YMA"}="1H17 YMA"; $resn_exch{"H172 YMA"}="2H17 YMA"; $resn_exch{" H71 YML"}="1H7  YML";
$resn_exch{" H72 YML"}="2H7  YML"; $resn_exch{" H21 YML"}="1H2  YML"; $resn_exch{" H22 YML"}="2H2  YML";
$resn_exch{" H91 YML"}="1H9  YML"; $resn_exch{" H92 YML"}="2H9  YML"; $resn_exch{"HAC1 YML"}="1HAC YML";
$resn_exch{"HAC2 YML"}="2HAC YML"; $resn_exch{"HAC3 YML"}="3HAC YML"; $resn_exch{"HAB1 YML"}="1HAB YML";
$resn_exch{"HAB2 YML"}="2HAB YML"; $resn_exch{"HAB3 YML"}="3HAB YML"; $resn_exch{"HAA1 YML"}="1HAA YML";
$resn_exch{"HAA2 YML"}="2HAA YML"; $resn_exch{"HAA3 YML"}="3HAA YML"; $resn_exch{" HN1 YMP"}="1HN  YMP";
$resn_exch{" HN2 YMP"}="2HN  YMP"; $resn_exch{" HB1 YMP"}="1HB  YMP"; $resn_exch{" HB2 YMP"}="2HB  YMP";
$resn_exch{"H5'1 YMP"}="1H5* YMP"; $resn_exch{"H5'2 YMP"}="2H5* YMP"; $resn_exch{" H4' YMP"}=" H4* YMP";
$resn_exch{" H1' YMP"}=" H1* YMP"; $resn_exch{" H2' YMP"}=" H2* YMP"; $resn_exch{" H3' YMP"}=" H3* YMP";
$resn_exch{"HN61 YMP"}="1HN6 YMP"; $resn_exch{"HN62 YMP"}="2HN6 YMP"; $resn_exch{" H   YOF"}="1HN  YOF";
$resn_exch{" H2  YOF"}="2HN  YOF"; $resn_exch{" HB2 YOF"}="1HB  YOF"; $resn_exch{" HB3 YOF"}="2HB  YOF";
$resn_exch{" HH  YOF"}=" HO  YOF"; $resn_exch{"HBA1 YOK"}="1HBA YOK"; $resn_exch{"HBA2 YOK"}="2HBA YOK";
$resn_exch{"HAA1 YOK"}="1HAA YOK"; $resn_exch{"HAA2 YOK"}="2HAA YOK"; $resn_exch{"H221 YPA"}="1H22 YPA";
$resn_exch{"H222 YPA"}="2H22 YPA"; $resn_exch{"H231 YPA"}="1H23 YPA"; $resn_exch{"H232 YPA"}="2H23 YPA";
$resn_exch{"H391 YPA"}="1H39 YPA"; $resn_exch{"H392 YPA"}="2H39 YPA"; $resn_exch{"H491 YPA"}="1H49 YPA";
$resn_exch{"H492 YPA"}="2H49 YPA"; $resn_exch{"H501 YPA"}="1H50 YPA"; $resn_exch{"H502 YPA"}="2H50 YPA";
$resn_exch{"H503 YPA"}="3H50 YPA"; $resn_exch{"HOP3 YRR"}="3HOP YRR"; $resn_exch{"HC51 YRR"}="1HC5 YRR";
$resn_exch{"HC52 YRR"}="2HC5 YRR"; $resn_exch{"HO3' YRR"}="H3T  YRR"; $resn_exch{"HC21 YRR"}="1HC2 YRR";
$resn_exch{"HC22 YRR"}="2HC2 YRR"; $resn_exch{"HC11 YRR"}="1HC1 YRR"; $resn_exch{"HC12 YRR"}="2HC1 YRR";
$resn_exch{" HB1 YSA"}="1HB  YSA"; $resn_exch{" HB2 YSA"}="2HB  YSA"; $resn_exch{" HN1 YSA"}="1HN  YSA";
$resn_exch{" HN2 YSA"}="2HN  YSA"; $resn_exch{"H5'1 YSA"}="1H5* YSA"; $resn_exch{"H5'2 YSA"}="2H5* YSA";
$resn_exch{" H4' YSA"}=" H4* YSA"; $resn_exch{" H3' YSA"}=" H3* YSA"; $resn_exch{" H2' YSA"}=" H2* YSA";
$resn_exch{" H1' YSA"}=" H1* YSA"; $resn_exch{"HN61 YSA"}="1HN6 YSA"; $resn_exch{"HN62 YSA"}="2HN6 YSA";
$resn_exch{"H151 YSH"}="1H15 YSH"; $resn_exch{"H152 YSH"}="2H15 YSH"; $resn_exch{"H171 YSH"}="1H17 YSH";
$resn_exch{"H172 YSH"}="2H17 YSH"; $resn_exch{"H173 YSH"}="3H17 YSH"; $resn_exch{"H181 YSH"}="1H18 YSH";
$resn_exch{"H182 YSH"}="2H18 YSH"; $resn_exch{"H183 YSH"}="3H18 YSH"; $resn_exch{"H191 YSH"}="1H19 YSH";
$resn_exch{"H192 YSH"}="2H19 YSH"; $resn_exch{"H193 YSH"}="3H19 YSH"; $resn_exch{"HOP2 YYG"}="2HOP YYG";
$resn_exch{"HOP3 YYG"}="3HOP YYG"; $resn_exch{" H5' YYG"}="1H5* YYG"; $resn_exch{"H5'' YYG"}="2H5* YYG";
$resn_exch{" H4' YYG"}=" H4* YYG"; $resn_exch{" H3' YYG"}=" H3* YYG"; $resn_exch{"HO3' YYG"}="H3T  YYG";
$resn_exch{" H2' YYG"}=" H2* YYG"; $resn_exch{"HO2' YYG"}="2HO* YYG"; $resn_exch{" H1' YYG"}=" H1* YYG";
$resn_exch{" H31 YYG"}="1H3  YYG"; $resn_exch{" H32 YYG"}="2H3  YYG"; $resn_exch{" H33 YYG"}="3H3  YYG";
$resn_exch{"H101 YYG"}="1H10 YYG"; $resn_exch{"H102 YYG"}="2H10 YYG"; $resn_exch{"H103 YYG"}="3H10 YYG";
$resn_exch{"H131 YYG"}="1H13 YYG"; $resn_exch{"H132 YYG"}="2H13 YYG"; $resn_exch{"H141 YYG"}="1H14 YYG";
$resn_exch{"H142 YYG"}="2H14 YYG"; $resn_exch{"H191 YYG"}="1H19 YYG"; $resn_exch{"H192 YYG"}="2H19 YYG";
$resn_exch{"H193 YYG"}="3H19 YYG"; $resn_exch{"H241 YYG"}="1H24 YYG"; $resn_exch{"H242 YYG"}="2H24 YYG";
$resn_exch{"H243 YYG"}="3H24 YYG"; $resn_exch{"H5'1 YYY"}="1H5* YYY"; $resn_exch{"H5'2 YYY"}="2H5* YYY";
$resn_exch{" H4' YYY"}=" H4* YYY"; $resn_exch{" H3' YYY"}=" H3* YYY"; $resn_exch{" H1' YYY"}=" H1* YYY";
$resn_exch{"H2'1 YYY"}="1H2* YYY"; $resn_exch{"H2'2 YYY"}="2H2* YYY"; $resn_exch{"H4N1 YYY"}="1H4N YYY";
$resn_exch{"H4N2 YYY"}="2H4N YYY"; $resn_exch{" H31 YZ9"}="1H3  YZ9"; $resn_exch{" H32 YZ9"}="2H3  YZ9";
$resn_exch{" H33 YZ9"}="3H3  YZ9"; $resn_exch{"H111 YZ9"}="1H11 YZ9"; $resn_exch{"H112 YZ9"}="2H11 YZ9";
$resn_exch{"HOP2   Z"}="2HOP   Z"; $resn_exch{" H5'   Z"}="1H5*   Z"; $resn_exch{"H5''   Z"}="2H5*   Z";
$resn_exch{" H4'   Z"}=" H4*   Z"; $resn_exch{" H3'   Z"}=" H3*   Z"; $resn_exch{"HO3'   Z"}="H3T    Z";
$resn_exch{" H2'   Z"}="1H2*   Z"; $resn_exch{"H2''   Z"}="2H2*   Z"; $resn_exch{" H1'   Z"}=" H1*   Z";
$resn_exch{"HOP3   Z"}="3HOP   Z"; $resn_exch{"H111 Z34"}="1H11 Z34"; $resn_exch{"H121 Z34"}="1H12 Z34";
$resn_exch{"H122 Z34"}="2H12 Z34"; $resn_exch{"H301 Z34"}="1H30 Z34"; $resn_exch{"H302 Z34"}="2H30 Z34";
$resn_exch{"H303 Z34"}="3H30 Z34"; $resn_exch{"H281 Z34"}="1H28 Z34"; $resn_exch{"H282 Z34"}="2H28 Z34";
$resn_exch{"H291 Z34"}="1H29 Z34"; $resn_exch{"H292 Z34"}="2H29 Z34"; $resn_exch{"H321 Z34"}="1H32 Z34";
$resn_exch{"H322 Z34"}="2H32 Z34"; $resn_exch{"H361 Z34"}="1H36 Z34"; $resn_exch{"H362 Z34"}="2H36 Z34";
$resn_exch{"H363 Z34"}="3H36 Z34"; $resn_exch{"H7T1 Z5A"}="1H7T Z5A"; $resn_exch{"H7T2 Z5A"}="2H7T Z5A";
$resn_exch{"H7T3 Z5A"}="3H7T Z5A"; $resn_exch{"H5'1 Z5A"}="1H5' Z5A"; $resn_exch{"H5'2 Z5A"}="2H5' Z5A";
$resn_exch{"H2'1 Z5A"}="1H2' Z5A"; $resn_exch{"H2'2 Z5A"}="2H2' Z5A"; $resn_exch{"H5A1 Z5A"}="1H5* Z5A";
$resn_exch{"H5A2 Z5A"}="2H5* Z5A"; $resn_exch{" H4B Z5A"}=" H4* Z5A"; $resn_exch{" H3B Z5A"}=" H3* Z5A";
$resn_exch{"HO'3 Z5A"}="3HO* Z5A"; $resn_exch{" H2' Z5A"}=" H2* Z5A"; $resn_exch{"HO'2 Z5A"}="2HO* Z5A";
$resn_exch{" H1B Z5A"}=" H1* Z5A"; $resn_exch{"H6A1 Z5A"}="1H6A Z5A"; $resn_exch{"H6A2 Z5A"}="2H6A Z5A";
$resn_exch{" HN1 ZAB"}="1HN  ZAB"; $resn_exch{" HN2 ZAB"}="2HN  ZAB"; $resn_exch{" HA2 ZAB"}="2HA  ZAB";
$resn_exch{" HA3 ZAB"}="3HA  ZAB"; $resn_exch{" HM2 ZAB"}="2HM  ZAB"; $resn_exch{" HM3 ZAB"}="3HM  ZAB";
$resn_exch{"HOB1 ZAF"}="1HOB ZAF"; $resn_exch{"H2A1 ZAF"}="1H2A ZAF"; $resn_exch{"H2A2 ZAF"}="2H2A ZAF";
$resn_exch{"HOB3 ZAF"}="3HOB ZAF"; $resn_exch{"H4A1 ZAF"}="1H4A ZAF"; $resn_exch{"H4A2 ZAF"}="2H4A ZAF";
$resn_exch{"H4A3 ZAF"}="3H4A ZAF"; $resn_exch{"H7A1 ZAF"}="1H7A ZAF"; $resn_exch{"H7A2 ZAF"}="2H7A ZAF";
$resn_exch{"H7A3 ZAF"}="3H7A ZAF"; $resn_exch{"H111 ZAF"}="1H11 ZAF"; $resn_exch{"H112 ZAF"}="2H11 ZAF";
$resn_exch{"HP2' ZAF"}="*HP2 ZAF"; $resn_exch{"HP3' ZAF"}="*HP3 ZAF"; $resn_exch{"HP4' ZAF"}="*HP4 ZAF";
$resn_exch{"HP5' ZAF"}="*HP5 ZAF"; $resn_exch{"HP6' ZAF"}="*HP6 ZAF"; $resn_exch{"H4C1 ZAH"}="1H4C ZAH";
$resn_exch{"H4C2 ZAH"}="2H4C ZAH"; $resn_exch{"H5C1 ZAH"}="1H5C ZAH"; $resn_exch{"H5C2 ZAH"}="2H5C ZAH";
$resn_exch{"H6C1 ZAH"}="1H6C ZAH"; $resn_exch{"H6C2 ZAH"}="2H6C ZAH"; $resn_exch{"H111 ZAH"}="1H11 ZAH";
$resn_exch{"H112 ZAH"}="2H11 ZAH"; $resn_exch{"H113 ZAH"}="3H11 ZAH"; $resn_exch{"H181 ZAH"}="1H18 ZAH";
$resn_exch{"H182 ZAH"}="2H18 ZAH"; $resn_exch{"H131 ZAM"}="1H13 ZAM"; $resn_exch{"H132 ZAM"}="2H13 ZAM";
$resn_exch{"H301 ZAM"}="1H30 ZAM"; $resn_exch{"H302 ZAM"}="2H30 ZAM"; $resn_exch{"H331 ZAM"}="1H33 ZAM";
$resn_exch{"H332 ZAM"}="2H33 ZAM"; $resn_exch{"H333 ZAM"}="3H33 ZAM"; $resn_exch{"H101 ZAM"}="1H10 ZAM";
$resn_exch{"H102 ZAM"}="2H10 ZAM"; $resn_exch{"H171 ZAM"}="1H17 ZAM"; $resn_exch{"H172 ZAM"}="2H17 ZAM";
$resn_exch{"H173 ZAM"}="3H17 ZAM"; $resn_exch{"H231 ZAM"}="1H23 ZAM"; $resn_exch{"H232 ZAM"}="2H23 ZAM";
$resn_exch{"H233 ZAM"}="3H23 ZAM"; $resn_exch{"H431 ZAM"}="1H43 ZAM"; $resn_exch{"H432 ZAM"}="2H43 ZAM";
$resn_exch{"H433 ZAM"}="3H43 ZAM"; $resn_exch{"H101 ZAP"}="1H10 ZAP"; $resn_exch{"H102 ZAP"}="2H10 ZAP";
$resn_exch{"HN21 ZAP"}="1HN2 ZAP"; $resn_exch{"HN22 ZAP"}="2HN2 ZAP"; $resn_exch{"HOP2 ZAP"}="2HOP ZAP";
$resn_exch{"HOP3 ZAP"}="3HOP ZAP"; $resn_exch{"H191 ZAR"}="1H19 ZAR"; $resn_exch{"H192 ZAR"}="2H19 ZAR";
$resn_exch{"H193 ZAR"}="3H19 ZAR"; $resn_exch{"HAK1 ZD6"}="1HAK ZD6"; $resn_exch{"HAK2 ZD6"}="2HAK ZD6";
$resn_exch{"HAM1 ZD6"}="1HAM ZD6"; $resn_exch{"HAM2 ZD6"}="2HAM ZD6"; $resn_exch{"HAB1 ZD6"}="1HAB ZD6";
$resn_exch{"HAB2 ZD6"}="2HAB ZD6"; $resn_exch{"HAB3 ZD6"}="3HAB ZD6"; $resn_exch{"HAN1 ZD6"}="1HAN ZD6";
$resn_exch{"HAN2 ZD6"}="2HAN ZD6"; $resn_exch{"HAL1 ZD6"}="1HAL ZD6"; $resn_exch{"HAL2 ZD6"}="2HAL ZD6";
$resn_exch{"HAO1 ZD6"}="1HAO ZD6"; $resn_exch{"HAO2 ZD6"}="2HAO ZD6"; $resn_exch{"HAA1 ZD6"}="1HAA ZD6";
$resn_exch{"HAA2 ZD6"}="2HAA ZD6"; $resn_exch{"HAA3 ZD6"}="3HAA ZD6"; $resn_exch{"HAB1 ZDP"}="1HAB ZDP";
$resn_exch{"HAB2 ZDP"}="2HAB ZDP"; $resn_exch{"HAK1 ZDP"}="1HAK ZDP"; $resn_exch{"HAK2 ZDP"}="2HAK ZDP";
$resn_exch{"HAJ1 ZDP"}="1HAJ ZDP"; $resn_exch{"HAJ2 ZDP"}="2HAJ ZDP"; $resn_exch{"HAM1 ZDP"}="1HAM ZDP";
$resn_exch{"HAM2 ZDP"}="2HAM ZDP"; $resn_exch{"HAC1 ZDP"}="1HAC ZDP"; $resn_exch{"HAC2 ZDP"}="2HAC ZDP";
$resn_exch{"HAN1 ZDP"}="1HAN ZDP"; $resn_exch{"HAN2 ZDP"}="2HAN ZDP"; $resn_exch{"HAN3 ZDP"}="3HAN ZDP";
$resn_exch{"HAN1 ZDR"}="1HAN ZDR"; $resn_exch{"HAN2 ZDR"}="2HAN ZDR"; $resn_exch{"HAN3 ZDR"}="3HAN ZDR";
$resn_exch{"HAC1 ZDR"}="1HAC ZDR"; $resn_exch{"HAC2 ZDR"}="2HAC ZDR"; $resn_exch{"HAM1 ZDR"}="1HAM ZDR";
$resn_exch{"HAM2 ZDR"}="2HAM ZDR"; $resn_exch{"HAJ1 ZDR"}="1HAJ ZDR"; $resn_exch{"HAJ2 ZDR"}="2HAJ ZDR";
$resn_exch{"HAK1 ZDR"}="1HAK ZDR"; $resn_exch{"HAK2 ZDR"}="2HAK ZDR"; $resn_exch{"HAB1 ZDR"}="1HAB ZDR";
$resn_exch{"HAB2 ZDR"}="2HAB ZDR"; $resn_exch{"HOP2 ZDU"}="2HOP ZDU"; $resn_exch{"H5A1 ZDU"}="1H5A ZDU";
$resn_exch{"H5A2 ZDU"}="2H5A ZDU"; $resn_exch{"H5B1 ZDU"}="1H5B ZDU"; $resn_exch{"H5B2 ZDU"}="2H5B ZDU";
$resn_exch{"H5G1 ZDU"}="1H5G ZDU"; $resn_exch{"H5G2 ZDU"}="2H5G ZDU"; $resn_exch{"H5D1 ZDU"}="1H5D ZDU";
$resn_exch{"H5D2 ZDU"}="2H5D ZDU"; $resn_exch{" H2' ZDU"}="1H2* ZDU"; $resn_exch{"H2'' ZDU"}="2H2* ZDU";
$resn_exch{" H5' ZDU"}="1H5* ZDU"; $resn_exch{"H5'' ZDU"}="2H5* ZDU"; $resn_exch{" H4' ZDU"}=" H4* ZDU";
$resn_exch{" H1' ZDU"}=" H1* ZDU"; $resn_exch{" H3' ZDU"}=" H3* ZDU"; $resn_exch{"HO3' ZDU"}="H3T  ZDU";
$resn_exch{"HOP3 ZDU"}="3HOP ZDU"; $resn_exch{"H141 ZEA"}="1H14 ZEA"; $resn_exch{"H142 ZEA"}="2H14 ZEA";
$resn_exch{"H151 ZEA"}="1H15 ZEA"; $resn_exch{"H152 ZEA"}="2H15 ZEA"; $resn_exch{"H153 ZEA"}="3H15 ZEA";
$resn_exch{"H111 ZEA"}="1H11 ZEA"; $resn_exch{"H112 ZEA"}="2H11 ZEA"; $resn_exch{"H5'1 ZEB"}="1H5' ZEB";
$resn_exch{"H5'2 ZEB"}="2H5' ZEB"; $resn_exch{"H141 ZEC"}="1H14 ZEC"; $resn_exch{"H142 ZEC"}="2H14 ZEC";
$resn_exch{"HZ91 ZED"}="1HZ9 ZED"; $resn_exch{"HZ92 ZED"}="2HZ9 ZED"; $resn_exch{"H111 ZED"}="1H11 ZED";
$resn_exch{"H112 ZED"}="2H11 ZED"; $resn_exch{"H121 ZED"}="1H12 ZED"; $resn_exch{"H122 ZED"}="2H12 ZED";
$resn_exch{"H123 ZED"}="3H12 ZED"; $resn_exch{"H151 ZED"}="1H15 ZED"; $resn_exch{"H152 ZED"}="2H15 ZED";
$resn_exch{"HAA1 ZEM"}="1HAA ZEM"; $resn_exch{"HAA2 ZEM"}="2HAA ZEM"; $resn_exch{"HMA1 ZEM"}="1HMA ZEM";
$resn_exch{"HMA2 ZEM"}="2HMA ZEM"; $resn_exch{"HMA3 ZEM"}="3HMA ZEM"; $resn_exch{"HBA1 ZEM"}="1HBA ZEM";
$resn_exch{"HBA2 ZEM"}="2HBA ZEM"; $resn_exch{"HMB1 ZEM"}="1HMB ZEM"; $resn_exch{"HMB2 ZEM"}="2HMB ZEM";
$resn_exch{"HMB3 ZEM"}="3HMB ZEM"; $resn_exch{"HBB1 ZEM"}="1HBB ZEM"; $resn_exch{"HBB2 ZEM"}="2HBB ZEM";
$resn_exch{"HMC1 ZEM"}="1HMC ZEM"; $resn_exch{"HMC2 ZEM"}="2HMC ZEM"; $resn_exch{"HMC3 ZEM"}="3HMC ZEM";
$resn_exch{"HBC1 ZEM"}="1HBC ZEM"; $resn_exch{"HBC2 ZEM"}="2HBC ZEM"; $resn_exch{"HMD1 ZEM"}="1HMD ZEM";
$resn_exch{"HMD2 ZEM"}="2HMD ZEM"; $resn_exch{"HMD3 ZEM"}="3HMD ZEM"; $resn_exch{"HAD1 ZEM"}="1HAD ZEM";
$resn_exch{"HAD2 ZEM"}="2HAD ZEM"; $resn_exch{"HBD1 ZEM"}="1HBD ZEM"; $resn_exch{"HBD2 ZEM"}="2HBD ZEM";
$resn_exch{" H82 ZEN"}="2H8  ZEN"; $resn_exch{" H81 ZEN"}="1H8  ZEN"; $resn_exch{" H92 ZEN"}="2H9  ZEN";
$resn_exch{" H91 ZEN"}="1H9  ZEN"; $resn_exch{"H112 ZEN"}="2H11 ZEN"; $resn_exch{"H111 ZEN"}="1H11 ZEN";
$resn_exch{"H122 ZEN"}="2H12 ZEN"; $resn_exch{"H121 ZEN"}="1H12 ZEN"; $resn_exch{"H152 ZEN"}="2H15 ZEN";
$resn_exch{"H151 ZEN"}="1H15 ZEN"; $resn_exch{"H162 ZEN"}="2H16 ZEN"; $resn_exch{"H161 ZEN"}="1H16 ZEN";
$resn_exch{"H182 ZEN"}="2H18 ZEN"; $resn_exch{"H181 ZEN"}="1H18 ZEN"; $resn_exch{"H192 ZEN"}="2H19 ZEN";
$resn_exch{"H191 ZEN"}="1H19 ZEN"; $resn_exch{"H141 ZES"}="1H14 ZES"; $resn_exch{"H142 ZES"}="2H14 ZES";
$resn_exch{"H271 ZES"}="1H27 ZES"; $resn_exch{"H272 ZES"}="2H27 ZES"; $resn_exch{"H171 ZEX"}="1H17 ZEX";
$resn_exch{"H172 ZEX"}="2H17 ZEX"; $resn_exch{"H173 ZEX"}="3H17 ZEX"; $resn_exch{" H41 ZEX"}="1H4  ZEX";
$resn_exch{" H42 ZEX"}="2H4  ZEX"; $resn_exch{" H21 ZEX"}="1H2  ZEX"; $resn_exch{" H22 ZEX"}="2H2  ZEX";
$resn_exch{"H181 ZEX"}="1H18 ZEX"; $resn_exch{"H182 ZEX"}="2H18 ZEX"; $resn_exch{"H183 ZEX"}="3H18 ZEX";
$resn_exch{"H161 ZEX"}="1H16 ZEX"; $resn_exch{"H162 ZEX"}="2H16 ZEX"; $resn_exch{"H163 ZEX"}="3H16 ZEX";
$resn_exch{"H191 ZEX"}="1H19 ZEX"; $resn_exch{"H192 ZEX"}="2H19 ZEX"; $resn_exch{"H193 ZEX"}="3H19 ZEX";
$resn_exch{"H201 ZEX"}="1H20 ZEX"; $resn_exch{"H202 ZEX"}="2H20 ZEX"; $resn_exch{"H203 ZEX"}="3H20 ZEX";
$resn_exch{"H401 ZEX"}="1H40 ZEX"; $resn_exch{"H402 ZEX"}="2H40 ZEX"; $resn_exch{"H403 ZEX"}="3H40 ZEX";
$resn_exch{"H391 ZEX"}="1H39 ZEX"; $resn_exch{"H392 ZEX"}="2H39 ZEX"; $resn_exch{"H393 ZEX"}="3H39 ZEX";
$resn_exch{"H361 ZEX"}="1H36 ZEX"; $resn_exch{"H362 ZEX"}="2H36 ZEX"; $resn_exch{"H363 ZEX"}="3H36 ZEX";
$resn_exch{"H371 ZEX"}="1H37 ZEX"; $resn_exch{"H372 ZEX"}="2H37 ZEX"; $resn_exch{"H373 ZEX"}="3H37 ZEX";
$resn_exch{"H221 ZEX"}="1H22 ZEX"; $resn_exch{"H222 ZEX"}="2H22 ZEX"; $resn_exch{"H381 ZEX"}="1H38 ZEX";
$resn_exch{"H382 ZEX"}="2H38 ZEX"; $resn_exch{"H383 ZEX"}="3H38 ZEX"; $resn_exch{" H21 ZFB"}="1H2  ZFB";
$resn_exch{" H22 ZFB"}="2H2  ZFB"; $resn_exch{"H111 ZFB"}="1H11 ZFB"; $resn_exch{"H112 ZFB"}="2H11 ZFB";
$resn_exch{"H121 ZFB"}="1H12 ZFB"; $resn_exch{"H122 ZFB"}="2H12 ZFB"; $resn_exch{"HN21 ZHH"}="1HN2 ZHH";
$resn_exch{"HN22 ZHH"}="2HN2 ZHH"; $resn_exch{" HA1 ZHH"}="1HA  ZHH"; $resn_exch{" HA2 ZHH"}="2HA  ZHH";
$resn_exch{" HB1 ZHH"}="1HB  ZHH"; $resn_exch{" HB2 ZHH"}="2HB  ZHH"; $resn_exch{"HF11 ZHH"}="1HF1 ZHH";
$resn_exch{"HF12 ZHH"}="2HF1 ZHH"; $resn_exch{"HF13 ZHH"}="3HF1 ZHH"; $resn_exch{"HN21 ZHZ"}="1HN2 ZHZ";
$resn_exch{"HN22 ZHZ"}="2HN2 ZHZ"; $resn_exch{" HB1 ZHZ"}="1HB  ZHZ"; $resn_exch{" HB2 ZHZ"}="2HB  ZHZ";
$resn_exch{"HF11 ZHZ"}="1HF1 ZHZ"; $resn_exch{"HF12 ZHZ"}="2HF1 ZHZ"; $resn_exch{"HF13 ZHZ"}="3HF1 ZHZ";
$resn_exch{"HOA2 ZID"}="2HOA ZID"; $resn_exch{"H51A ZID"}="AH51 ZID"; $resn_exch{"H52A ZID"}="AH52 ZID";
$resn_exch{" H4B ZID"}="AH4* ZID"; $resn_exch{" H3B ZID"}="AH3* ZID"; $resn_exch{"HO3A ZID"}="AHO3 ZID";
$resn_exch{" H2B ZID"}="AH2* ZID"; $resn_exch{"HO2A ZID"}="AHO2 ZID"; $resn_exch{" H1B ZID"}="AH1* ZID";
$resn_exch{" H8A ZID"}=" AH8 ZID"; $resn_exch{"H61A ZID"}="AH61 ZID"; $resn_exch{"H62A ZID"}="AH62 ZID";
$resn_exch{" H2A ZID"}=" AH2 ZID"; $resn_exch{"H51N ZID"}="NH51 ZID"; $resn_exch{"H52N ZID"}="NH52 ZID";
$resn_exch{" H4D ZID"}="NH4* ZID"; $resn_exch{" H3D ZID"}="NH3* ZID"; $resn_exch{"HO3N ZID"}="NHO3 ZID";
$resn_exch{" H2D ZID"}="NH2* ZID"; $resn_exch{"HO2N ZID"}="NHO2 ZID"; $resn_exch{" H1D ZID"}="NH1* ZID";
$resn_exch{" H2N ZID"}=" NH2 ZID"; $resn_exch{"H71N ZID"}="NH71 ZID"; $resn_exch{"H72N ZID"}="NH72 ZID";
$resn_exch{" H5N ZID"}=" NH5 ZID"; $resn_exch{" H6N ZID"}=" NH6 ZID"; $resn_exch{" H2Z ZID"}=" ZH2 ZID";
$resn_exch{" H3Z ZID"}=" ZH3 ZID"; $resn_exch{" H5Z ZID"}=" ZH5 ZID"; $resn_exch{" H6Z ZID"}=" ZH6 ZID";
$resn_exch{"H7C1 ZIO"}="1H7C ZIO"; $resn_exch{"H7C2 ZIO"}="2H7C ZIO"; $resn_exch{"H341 ZIO"}="1H34 ZIO";
$resn_exch{"H342 ZIO"}="2H34 ZIO"; $resn_exch{"H343 ZIO"}="3H34 ZIO"; $resn_exch{"H331 ZIO"}="1H33 ZIO";
$resn_exch{"H332 ZIO"}="2H33 ZIO"; $resn_exch{"H351 ZIO"}="1H35 ZIO"; $resn_exch{"H352 ZIO"}="2H35 ZIO";
$resn_exch{"H353 ZIO"}="3H35 ZIO"; $resn_exch{"H361 ZIO"}="1H36 ZIO"; $resn_exch{"H362 ZIO"}="2H36 ZIO";
$resn_exch{"H363 ZIO"}="3H36 ZIO"; $resn_exch{"H301 ZIO"}="1H30 ZIO"; $resn_exch{"H302 ZIO"}="2H30 ZIO";
$resn_exch{"H303 ZIO"}="3H30 ZIO"; $resn_exch{"H321 ZIO"}="1H32 ZIO"; $resn_exch{"H322 ZIO"}="2H32 ZIO";
$resn_exch{"H323 ZIO"}="3H32 ZIO"; $resn_exch{"H251 ZIO"}="1H25 ZIO"; $resn_exch{"H252 ZIO"}="2H25 ZIO";
$resn_exch{"H271 ZIO"}="1H27 ZIO"; $resn_exch{"H272 ZIO"}="2H27 ZIO"; $resn_exch{"H273 ZIO"}="3H27 ZIO";
$resn_exch{"H281 ZIO"}="1H28 ZIO"; $resn_exch{"H282 ZIO"}="2H28 ZIO"; $resn_exch{"H283 ZIO"}="3H28 ZIO";
$resn_exch{"H151 ZIO"}="1H15 ZIO"; $resn_exch{"H152 ZIO"}="2H15 ZIO"; $resn_exch{"H201 ZIO"}="1H20 ZIO";
$resn_exch{"H202 ZIO"}="2H20 ZIO"; $resn_exch{"H203 ZIO"}="3H20 ZIO"; $resn_exch{"H291 ZIO"}="1H29 ZIO";
$resn_exch{"H292 ZIO"}="2H29 ZIO"; $resn_exch{"H293 ZIO"}="3H29 ZIO"; $resn_exch{"H211 ZIO"}="1H21 ZIO";
$resn_exch{"H212 ZIO"}="2H21 ZIO"; $resn_exch{"H213 ZIO"}="3H21 ZIO"; $resn_exch{"H311 ZIO"}="1H31 ZIO";
$resn_exch{"H312 ZIO"}="2H31 ZIO"; $resn_exch{"H313 ZIO"}="3H31 ZIO"; $resn_exch{"H141 ZIP"}="1H14 ZIP";
$resn_exch{"H142 ZIP"}="2H14 ZIP"; $resn_exch{"H143 ZIP"}="3H14 ZIP"; $resn_exch{"H151 ZIP"}="1H15 ZIP";
$resn_exch{"H152 ZIP"}="2H15 ZIP"; $resn_exch{"H153 ZIP"}="3H15 ZIP"; $resn_exch{"H111 ZIP"}="1H11 ZIP";
$resn_exch{"H112 ZIP"}="2H11 ZIP"; $resn_exch{" H71 ZIT"}="1H7  ZIT"; $resn_exch{" H72 ZIT"}="2H7  ZIT";
$resn_exch{" H91 ZIT"}="1H9  ZIT"; $resn_exch{" H92 ZIT"}="2H9  ZIT"; $resn_exch{"H151 ZIT"}="1H15 ZIT";
$resn_exch{"H152 ZIT"}="2H15 ZIT"; $resn_exch{"H161 ZIT"}="1H16 ZIT"; $resn_exch{"H162 ZIT"}="2H16 ZIT";
$resn_exch{"H163 ZIT"}="3H16 ZIT"; $resn_exch{"H171 ZIT"}="1H17 ZIT"; $resn_exch{"H172 ZIT"}="2H17 ZIT";
$resn_exch{"H173 ZIT"}="3H17 ZIT"; $resn_exch{"H181 ZIT"}="1H18 ZIT"; $resn_exch{"H182 ZIT"}="2H18 ZIT";
$resn_exch{"H183 ZIT"}="3H18 ZIT"; $resn_exch{"H191 ZIT"}="1H19 ZIT"; $resn_exch{"H192 ZIT"}="2H19 ZIT";
$resn_exch{"H193 ZIT"}="3H19 ZIT"; $resn_exch{"H201 ZIT"}="1H20 ZIT"; $resn_exch{"H202 ZIT"}="2H20 ZIT";
$resn_exch{"H203 ZIT"}="3H20 ZIT"; $resn_exch{"H211 ZIT"}="1H21 ZIT"; $resn_exch{"H212 ZIT"}="2H21 ZIT";
$resn_exch{"H213 ZIT"}="3H21 ZIT"; $resn_exch{"H221 ZIT"}="1H22 ZIT"; $resn_exch{"H222 ZIT"}="2H22 ZIT";
$resn_exch{"H223 ZIT"}="3H22 ZIT"; $resn_exch{"H231 ZIT"}="1H23 ZIT"; $resn_exch{"H232 ZIT"}="2H23 ZIT";
$resn_exch{"H233 ZIT"}="3H23 ZIT"; $resn_exch{"H1A1 ZIT"}="1H1A ZIT"; $resn_exch{"H2A1 ZIT"}="1H2A ZIT";
$resn_exch{"H3A1 ZIT"}="1H3A ZIT"; $resn_exch{"H4A1 ZIT"}="1H4A ZIT"; $resn_exch{"H4A2 ZIT"}="2H4A ZIT";
$resn_exch{"H5A1 ZIT"}="1H5A ZIT"; $resn_exch{"H6A1 ZIT"}="1C6A ZIT"; $resn_exch{"H6A2 ZIT"}="2C6A ZIT";
$resn_exch{"H6A3 ZIT"}="3C6A ZIT"; $resn_exch{"H7A1 ZIT"}="1H7A ZIT"; $resn_exch{"H7A2 ZIT"}="2H7A ZIT";
$resn_exch{"H7A3 ZIT"}="3H7A ZIT"; $resn_exch{"H8A1 ZIT"}="1H8A ZIT"; $resn_exch{"H8A2 ZIT"}="2H8A ZIT";
$resn_exch{"H8A3 ZIT"}="3H8A ZIT"; $resn_exch{"H1B1 ZIT"}="1H1B ZIT"; $resn_exch{"H2B1 ZIT"}="1H2B ZIT";
$resn_exch{"H2B2 ZIT"}="2H2B ZIT"; $resn_exch{"H4B1 ZIT"}="1H4B ZIT"; $resn_exch{"H5B1 ZIT"}="1H5B ZIT";
$resn_exch{"H6B1 ZIT"}="1C6B ZIT"; $resn_exch{"H6B2 ZIT"}="2C6B ZIT"; $resn_exch{"H6B3 ZIT"}="3C6B ZIT";
$resn_exch{"H7B1 ZIT"}="1H7B ZIT"; $resn_exch{"H7B2 ZIT"}="2H7B ZIT"; $resn_exch{"H7B3 ZIT"}="3H7B ZIT";
$resn_exch{"H8B1 ZIT"}="1H8B ZIT"; $resn_exch{"H8B2 ZIT"}="2H8B ZIT"; $resn_exch{"H8B3 ZIT"}="3H8B ZIT";
$resn_exch{"H2C1 ZK5"}="1H2C ZK5"; $resn_exch{"H2C2 ZK5"}="2H2C ZK5"; $resn_exch{"H6C1 ZK5"}="1H6C ZK5";
$resn_exch{"H6C2 ZK5"}="2H6C ZK5"; $resn_exch{"H7C1 ZK5"}="1H7C ZK5"; $resn_exch{"H7C2 ZK5"}="2H7C ZK5";
$resn_exch{"H1C1 ZK5"}="1H1C ZK5"; $resn_exch{"H1C2 ZK5"}="2H1C ZK5"; $resn_exch{"H121 ZK5"}="1H12 ZK5";
$resn_exch{"H122 ZK5"}="2H12 ZK5"; $resn_exch{"H151 ZK5"}="1H15 ZK5"; $resn_exch{"H152 ZK5"}="2H15 ZK5";
$resn_exch{"H161 ZK5"}="1H16 ZK5"; $resn_exch{"H162 ZK5"}="2H16 ZK5"; $resn_exch{"H181 ZK5"}="1H18 ZK5";
$resn_exch{"H182 ZK5"}="2H18 ZK5"; $resn_exch{"H183 ZK5"}="3H18 ZK5"; $resn_exch{"H191 ZK5"}="1H19 ZK5";
$resn_exch{"H192 ZK5"}="2H19 ZK5"; $resn_exch{"H193 ZK5"}="3H19 ZK5"; $resn_exch{"H211 ZK5"}="1H21 ZK5";
$resn_exch{"H212 ZK5"}="2H21 ZK5"; $resn_exch{"H111 ZMR"}="1H11 ZMR"; $resn_exch{"H112 ZMR"}="2H11 ZMR";
$resn_exch{"H113 ZMR"}="3H11 ZMR"; $resn_exch{" H91 ZMR"}="1H9  ZMR"; $resn_exch{" H92 ZMR"}="2H9  ZMR";
$resn_exch{"HN11 ZMR"}="1HN1 ZMR"; $resn_exch{"HN12 ZMR"}="2HN1 ZMR"; $resn_exch{"HN21 ZMR"}="1HN2 ZMR";
$resn_exch{"HN22 ZMR"}="2HN2 ZMR"; $resn_exch{"HO11 ZN3"}="1HO1 ZN3"; $resn_exch{"HO12 ZN3"}="2HO1 ZN3";
$resn_exch{"HMA1 ZNH"}="1HMA ZNH"; $resn_exch{"HMA2 ZNH"}="2HMA ZNH"; $resn_exch{"HMA3 ZNH"}="3HMA ZNH";
$resn_exch{"HAA1 ZNH"}="1HAA ZNH"; $resn_exch{"HAA2 ZNH"}="2HAA ZNH"; $resn_exch{"HBA1 ZNH"}="1HBA ZNH";
$resn_exch{"HBA2 ZNH"}="2HBA ZNH"; $resn_exch{"HMB1 ZNH"}="1HMB ZNH"; $resn_exch{"HMB2 ZNH"}="2HMB ZNH";
$resn_exch{"HMB3 ZNH"}="3HMB ZNH"; $resn_exch{"HBB1 ZNH"}="1HBB ZNH"; $resn_exch{"HBB2 ZNH"}="2HBB ZNH";
$resn_exch{"HMC1 ZNH"}="1HMC ZNH"; $resn_exch{"HMC2 ZNH"}="2HMC ZNH"; $resn_exch{"HMC3 ZNH"}="3HMC ZNH";
$resn_exch{"HBC1 ZNH"}="1HBC ZNH"; $resn_exch{"HBC2 ZNH"}="2HBC ZNH"; $resn_exch{"HMD1 ZNH"}="1HMD ZNH";
$resn_exch{"HMD2 ZNH"}="2HMD ZNH"; $resn_exch{"HMD3 ZNH"}="3HMD ZNH"; $resn_exch{"HAD1 ZNH"}="1HAD ZNH";
$resn_exch{"HAD2 ZNH"}="2HAD ZNH"; $resn_exch{"HBD1 ZNH"}="1HBD ZNH"; $resn_exch{"HBD2 ZNH"}="2HBD ZNH";
$resn_exch{"HO11 ZNO"}="1HO1 ZNO"; $resn_exch{"HO12 ZNO"}="2HO1 ZNO"; $resn_exch{"HO13 ZNO"}="3HO1 ZNO";
$resn_exch{"HO14 ZNO"}="4HO1 ZNO"; $resn_exch{"HO11 ZO3"}="1HO1 ZO3"; $resn_exch{"HO12 ZO3"}="2HO1 ZO3";
$resn_exch{"HO21 ZO3"}="1HO2 ZO3"; $resn_exch{"HO22 ZO3"}="2HO2 ZO3"; $resn_exch{"HO31 ZO3"}="1HO3 ZO3";
$resn_exch{"HO32 ZO3"}="2HO3 ZO3"; $resn_exch{" H71 ZOL"}="1H7  ZOL"; $resn_exch{" H72 ZOL"}="2H7  ZOL";
$resn_exch{"H281 ZPG"}="1H28 ZPG"; $resn_exch{"H282 ZPG"}="2H28 ZPG"; $resn_exch{"H283 ZPG"}="3H28 ZPG";
$resn_exch{"H241 ZPG"}="1H24 ZPG"; $resn_exch{"H242 ZPG"}="2H24 ZPG"; $resn_exch{"H261 ZPG"}="1H26 ZPG";
$resn_exch{"H262 ZPG"}="2H26 ZPG"; $resn_exch{"H263 ZPG"}="3H26 ZPG"; $resn_exch{"H211 ZPG"}="1H21 ZPG";
$resn_exch{"H212 ZPG"}="2H21 ZPG"; $resn_exch{"H251 ZPG"}="1H25 ZPG"; $resn_exch{"H252 ZPG"}="2H25 ZPG";
$resn_exch{"H253 ZPG"}="3H25 ZPG"; $resn_exch{"H181 ZPG"}="1H18 ZPG"; $resn_exch{"H182 ZPG"}="2H18 ZPG";
$resn_exch{"H161 ZPG"}="1H16 ZPG"; $resn_exch{"H162 ZPG"}="2H16 ZPG"; $resn_exch{"H163 ZPG"}="3H16 ZPG";
$resn_exch{"H141 ZPG"}="1H14 ZPG"; $resn_exch{"H142 ZPG"}="2H14 ZPG"; $resn_exch{"H121 ZPG"}="1H12 ZPG";
$resn_exch{"H122 ZPG"}="2H12 ZPG"; $resn_exch{"H123 ZPG"}="3H12 ZPG"; $resn_exch{"H101 ZPG"}="1H10 ZPG";
$resn_exch{"H102 ZPG"}="2H10 ZPG"; $resn_exch{" H81 ZPG"}="1H8  ZPG"; $resn_exch{" H82 ZPG"}="2H8  ZPG";
$resn_exch{" H83 ZPG"}="3H8  ZPG"; $resn_exch{" H61 ZPG"}="1H6  ZPG"; $resn_exch{" H62 ZPG"}="2H6  ZPG";
$resn_exch{" H41 ZPG"}="1H4  ZPG"; $resn_exch{" H42 ZPG"}="2H4  ZPG"; $resn_exch{" H31 ZPG"}="1H3  ZPG";
$resn_exch{" H32 ZPG"}="2H3  ZPG"; $resn_exch{" H33 ZPG"}="3H3  ZPG"; $resn_exch{"H141 ZPQ"}="1H14 ZPQ";
$resn_exch{"H142 ZPQ"}="2H14 ZPQ"; $resn_exch{"H143 ZPQ"}="3H14 ZPQ"; $resn_exch{"H151 ZPQ"}="1H15 ZPQ";
$resn_exch{"H152 ZPQ"}="2H15 ZPQ"; $resn_exch{"H171 ZPQ"}="1H17 ZPQ"; $resn_exch{"H172 ZPQ"}="2H17 ZPQ";
$resn_exch{"H173 ZPQ"}="3H17 ZPQ"; $resn_exch{"H221 ZPQ"}="1H22 ZPQ"; $resn_exch{"H222 ZPQ"}="2H22 ZPQ";
$resn_exch{"H231 ZPQ"}="1H23 ZPQ"; $resn_exch{"H232 ZPQ"}="2H23 ZPQ"; $resn_exch{"H233 ZPQ"}="3H23 ZPQ";
$resn_exch{"H331 ZPQ"}="1H33 ZPQ"; $resn_exch{"H332 ZPQ"}="2H33 ZPQ"; $resn_exch{"H351 ZPQ"}="1H35 ZPQ";
$resn_exch{"H352 ZPQ"}="2H35 ZPQ"; $resn_exch{"H353 ZPQ"}="3H35 ZPQ"; $resn_exch{"H361 ZPQ"}="1H36 ZPQ";
$resn_exch{"H362 ZPQ"}="2H36 ZPQ"; $resn_exch{"H363 ZPQ"}="3H36 ZPQ"; $resn_exch{"H371 ZPQ"}="1H37 ZPQ";
$resn_exch{"H372 ZPQ"}="2H37 ZPQ"; $resn_exch{"H391 ZPQ"}="1H39 ZPQ"; $resn_exch{"H392 ZPQ"}="2H39 ZPQ";
$resn_exch{"H393 ZPQ"}="3H39 ZPQ"; $resn_exch{"H431 ZPQ"}="1H43 ZPQ"; $resn_exch{"H432 ZPQ"}="2H43 ZPQ";
$resn_exch{"H433 ZPQ"}="3H43 ZPQ"; $resn_exch{"H441 ZPQ"}="1H44 ZPQ"; $resn_exch{"H442 ZPQ"}="2H44 ZPQ";
$resn_exch{"H443 ZPQ"}="3H44 ZPQ"; $resn_exch{"H511 ZPQ"}="1H51 ZPQ"; $resn_exch{"H512 ZPQ"}="2H51 ZPQ";
$resn_exch{"H531 ZPQ"}="1H53 ZPQ"; $resn_exch{"H532 ZPQ"}="2H53 ZPQ"; $resn_exch{"H533 ZPQ"}="3H53 ZPQ";
$resn_exch{"H541 ZPQ"}="1H54 ZPQ"; $resn_exch{"H542 ZPQ"}="2H54 ZPQ"; $resn_exch{"H543 ZPQ"}="3H54 ZPQ";
$resn_exch{" H11 ZPR"}="1H1  ZPR"; $resn_exch{" H31 ZPR"}="1H3  ZPR"; $resn_exch{" H41 ZPR"}="1H4  ZPR";
$resn_exch{" H42 ZPR"}="2H4  ZPR"; $resn_exch{" H51 ZPR"}="1H5  ZPR"; $resn_exch{" H52 ZPR"}="2H5  ZPR";
$resn_exch{" H61 ZPR"}="1H6  ZPR"; $resn_exch{" H62 ZPR"}="2H6  ZPR"; $resn_exch{"H101 ZPR"}="1H10 ZPR";
$resn_exch{"H111 ZPR"}="1H11 ZPR"; $resn_exch{"H112 ZPR"}="2H11 ZPR"; $resn_exch{"H121 ZPR"}="1H12 ZPR";
$resn_exch{"H122 ZPR"}="2H12 ZPR"; $resn_exch{"H131 ZPR"}="1H13 ZPR"; $resn_exch{"H132 ZPR"}="2H13 ZPR";
$resn_exch{"H181 ZPR"}="1H18 ZPR"; $resn_exch{"H182 ZPR"}="2H18 ZPR"; $resn_exch{"H201 ZPR"}="1H20 ZPR";
$resn_exch{"H211 ZPR"}="1H21 ZPR"; $resn_exch{"H221 ZPR"}="1H22 ZPR"; $resn_exch{"H231 ZPR"}="1H23 ZPR";
$resn_exch{"H241 ZPR"}="1H24 ZPR"; $resn_exch{" H21 ZRA"}="1H2  ZRA"; $resn_exch{" H22 ZRA"}="2H2  ZRA";
$resn_exch{"HB11 ZRA"}="1HB1 ZRA"; $resn_exch{"HB12 ZRA"}="2HB1 ZRA"; $resn_exch{" HG1 ZRA"}="1HG  ZRA";
$resn_exch{" HG2 ZRA"}="2HG  ZRA"; $resn_exch{" HD1 ZRA"}="1HD  ZRA"; $resn_exch{" HD2 ZRA"}="2HD  ZRA";
$resn_exch{"HB21 ZRA"}="1HB2 ZRA"; $resn_exch{"HB22 ZRA"}="2HB2 ZRA"; $resn_exch{"HB23 ZRA"}="3HB2 ZRA";
$resn_exch{" HM1 ZRA"}="1HM  ZRA"; $resn_exch{" HM2 ZRA"}="2HM  ZRA"; $resn_exch{" HM3 ZRA"}="3HM  ZRA";
$resn_exch{" H91 ZST"}="1H9  ZST"; $resn_exch{" H92 ZST"}="2H9  ZST"; $resn_exch{"H171 ZST"}="1H17 ZST";
$resn_exch{"H172 ZST"}="2H17 ZST"; $resn_exch{" H21 ZYA"}="1H2  ZYA"; $resn_exch{" H22 ZYA"}="2H2  ZYA";
$resn_exch{"HB11 ZYA"}="1HB1 ZYA"; $resn_exch{"HB12 ZYA"}="2HB1 ZYA"; $resn_exch{"HB21 ZYA"}="1HB2 ZYA";
$resn_exch{"HB22 ZYA"}="2HB2 ZYA"; $resn_exch{"HB23 ZYA"}="3HB2 ZYA"; $resn_exch{" HM1 ZYA"}="1HM  ZYA";
$resn_exch{" HM2 ZYA"}="2HM  ZYA"; $resn_exch{" HM3 ZYA"}="3HM  ZYA"; $resn_exch{"HN11 ZYX"}="1HN1 ZYX";
$resn_exch{"HN12 ZYX"}="2HN1 ZYX"; $resn_exch{" H71 ZYX"}="1H7  ZYX"; $resn_exch{" H72 ZYX"}="2H7  ZYX";
$resn_exch{" H81 ZYX"}="1H8  ZYX"; $resn_exch{" H82 ZYX"}="2H8  ZYX"; $resn_exch{"HN21 ZYX"}="1HN2 ZYX";
$resn_exch{"HN22 ZYX"}="2HN2 ZYX"; $resn_exch{"H6C1 ZZ1"}="1H6C ZZ1"; $resn_exch{"H6C2 ZZ1"}="2H6C ZZ1";
$resn_exch{"H6C3 ZZ1"}="3H6C ZZ1"; $resn_exch{"H7C1 ZZZ"}="1H7C ZZZ"; $resn_exch{"H7C2 ZZZ"}="2H7C ZZZ";
$resn_exch{"H2N1 ZZZ"}="1H2N ZZZ"; $resn_exch{"H2N2 ZZZ"}="2H2N ZZZ"; 
}
